#
# $Id: srvctl.tcl,v 1.9 1999/05/18 03:02:33 bill Exp $
#
# Copyright (c) 1998-1999, Zero-Knowledge Systems Inc. All Rights Reserved.
#

package provide fradmin 1.0


#
# Return a list of server names, based on the contents
# the Freedom etc/rc directory.
#
proc listServers {} {
  global g_fr_root
  
  set ls [file join $g_fr_root etc rc S*.*]
  
  set srvs {}
  foreach f [glob -nocomplain -- $ls] {
    lappend srvs [string trimleft [file extension $f] .]
  }
  return [lsort $srvs]
}


#
# Return a list of symbolic links, from /freedom/etc/rc,
# corresponding to the operation `op' and the list of
# server names.
#
proc getCtlLinks { op srvs } {
  global g_fr_root

  switch $op {
    start   { set let S }
    stop    { set let K }

    default { return "" }
  }

  set dir [file join $g_fr_root etc rc]
  
  set ls {}
  foreach s $srvs {
    lappend ls [glob -nocomplain -- [file join $dir ${let}*.${s}]]
  }

  return [lsort $ls]
}


#
# Return the value of `up' if the server is running, and
# the value of `down' if it is not.
#
proc serverStatus { server up down } {
  global g_fr_root g_log_file
  
  set pidfile [file join $g_fr_root var run ${server}.pid]
  if {[catch {exec fr_killpid --logfile $g_log_file --pidfile $pidfile --signal 0}]} {
    return $down
  } else {
    return $up
  }
}


#
# Present a list of the existing services.
#
# If `selRunning' is 1 those services that are currently
# running should be initially selected.  Otherwise, those
# that are not running should be selected.
#
# Return a list of the items selected, and the empty string
# if nothing was selected.
#
proc pickServer {text selRunning} {
  global whiptcl_canceled
  
  set srvs [listServers]
  if {[llength $srvs] == 0} {
    whiptcl_cmd --msgbox [fmt {
      There do not appear to be any Freedom servers installed.
    }] 15 50
    return ""
  }
  set sel {}
  if {$selRunning} {
    set up on ; set down off
  } else {
    set up off ; set down on
  }
  foreach p $srvs {
    lappend sel $p [serverStatus $p $up $down]
  }
  set nitems [llength $sel]
  set x [eval whiptcl_cmd --noitem --checklist [list [fmt $text]] 20 75 10 $sel]
  if {$whiptcl_canceled} {
    return ""
  }
  return $x
}


#
# Perform an operation (start/stop/restart) on a list of
# servers.
#
# If completely successful return the empty list, else
# return a list of the errors that occurred.
#
proc opRun {op srvs} {
  global errorCode g_pass
  
  set errs {}
#  cmdtrace on noeval
  foreach l [getCtlLinks $op $srvs] {
    frPipe passRd passWr fdRd {}
    puts $passWr $g_pass ; close $passWr
#    puts stderr "pre-exec $l"
    if {[catch {exec $l --passPhrase.file /dev/fd/$fdRd} msg]} {
      lappend errs $msg
    }
#    puts stderr "post-exec $l"
    catch {close $passRd}
  }
#  cmdtrace off
  return $errs
}


#
# Perform an operation on the list of servers.
#
# Restarting is a special case: the procedure is called
# recursively to first stop the selected servers, then
# called again to start the same servers. For what its
# worth, this allows us to use the correct ordering both
# when stopping and starting.
#
proc opServers {op} {
  global errorCode
  
  switch $op {
    start   { set selrun 0 ; set a started   ; set b starting }
    stop    { set selrun 1 ; set a stopped   ; set b stopping }
    restart { set selrun 1 ; set a restarted ; set b restarting }

    default { error "bad operation `$op'" }
  }

  set srvs [pickServer [fmt {
    Only those servers that are marked with an
    asterisk (*) will be $a.
  }] $selrun]

  switch $op {
    start -
    stop    {
      set errs [opRun $op $srvs]
    }

    restart {
      set errs [concat [opRun stop $srvs] [opRun start $srvs]]
    }
  }
  
  if {[llength $errs] > 0} {
    whiptcl_cmd --scrolltext --msgbox [fmt {
      One or more errors occurred while $b the server(s):

      [join $errs "\n"]
    }] 20 75
  }
}


# note: move to another file, or rename file.
proc frAdminRun {} {
  global g_canceled whiptcl_canceled

#  cmdtrace on noeval notruncate

  if { ! [passphraseGetExisting]} {
    return 0
  }

  set keep_going 1
  
  while { $keep_going } {
    set whiptcl_canceled 0
    set sel [list \
                 { Start Servers} x \
                 { Stop Servers} x \
                 { Restart Servers} x \
                 { Exit} x \
                ]
#                 { Change Passphrase} x
    set x [eval whiptcl_cmd --noitem --menu [list {
      Select one of the administrative options.
      
      Cancel will exit this program.
    }] 20 75 [llength $sel] $sel]
    if {$whiptcl_canceled} {
      return 1
    }
    switch $x {
      { Start Servers} {
        opServers start
      }
      
      { Stop Servers} {
        opServers stop
      }
      
      { Restart Servers} {
        opServers restart
      }
      
      { Exit} {
        set keep_going 0
      }
    }

#      { Change Passphrase} {
#        changePassphrase
#      }
      
  }
}
