#include "entity.h"

//
//
//
FrEntIdType Entity::toFrEntIdType(const id_type & id)
{
  DECL_FUNC("Entity::toFrEntIdType");
  FrEntIdType frid;

  switch( id ) {
  case TYPE_UNUSED_:       frid = FRENT_TYPE_UNUSED_;       break;
  case TYPE_AIP:           frid = FRENT_TYPE_AIP;           break;
  case TYPE_FMG:           frid = FRENT_TYPE_FMG;           break;
  case TYPE_MAIP:          frid = FRENT_TYPE_MAIP;          break;
  case TYPE_MASTER:        frid = FRENT_TYPE_MASTER;        break;
  case TYPE_NIDB:          frid = FRENT_TYPE_NIDB;          break;
  case TYPE_NYM:           frid = FRENT_TYPE_NYM;           break;
  case TYPE_NYM_SRV:       frid = FRENT_TYPE_NYM_SRV;       break;
  case TYPE_SER_SRV:       frid = FRENT_TYPE_SER_SRV;       break;
  case TYPE_TOK_SRV:       frid = FRENT_TYPE_TOK_SRV;       break;
  case TYPE_KEYQRY_SRV:    frid = FRENT_TYPE_KEYQRY_SRV;    break;
  case TYPE_KEYUPD_SRV:    frid = FRENT_TYPE_KEYUPD_SRV;    break;
  case TYPE_ANON:          frid = FRENT_TYPE_ANON;          break;
  case TYPE_NNTP_SRV:      frid = FRENT_TYPE_NNTP_SRV;      break;
  case TYPE_FREEDOM_YEAR:  frid = FRENT_TYPE_FREEDOM_YEAR;  break;
  case TYPE_FREEDOM_MONTH: frid = FRENT_TYPE_FREEDOM_MONTH; break;
  case TYPE_SOFTUPD:       frid = FRENT_TYPE_SOFTUPD;       break;
  case TYPE_LINK:          frid = FRENT_TYPE_LINK;          break;
  case TYPE_HOST:          frid = FRENT_TYPE_HOST;          break;
  case TYPE_NIQS:          frid = FRENT_TYPE_NIQS;          break;
  case TYPE_NISS:          frid = FRENT_TYPE_NISS;          break;
  case TYPE_POP:           frid = FRENT_TYPE_POP;           break;
  case TYPE_NMTA:          frid = FRENT_TYPE_NMTA;          break;
  case TYPE_IMEP:          frid = FRENT_TYPE_IMEP;          break;
  case TYPE_COREAIP:       frid = FRENT_TYPE_COREAIP;       break;
  case TYPE_ANONRAND:      frid = FRENT_TYPE_ANONRAND;      break;
  default:
    CZKS::Throw(ZKS_UAIPPROXY_MOD,
                ZKS_UAIPPROXY_ID_ERR,
                func,
                "Invalide id_type: %d", (int32_t)id);    
  }
  return frid;
}


//
//
//
Entity::id_type Entity::toIdType(const FrEntIdType & frid)
{
  DECL_FUNC("Entity::toIdType");
  id_type id;

  switch( frid ) {
  case FRENT_TYPE_UNUSED_:       id = TYPE_UNUSED_;       break;
  case FRENT_TYPE_AIP:           id = TYPE_AIP;           break;
  case FRENT_TYPE_FMG:           id = TYPE_FMG;           break;
  case FRENT_TYPE_MAIP:          id = TYPE_MAIP;          break;
  case FRENT_TYPE_MASTER:        id = TYPE_MASTER;        break;
  case FRENT_TYPE_NIDB:          id = TYPE_NIDB;          break;
  case FRENT_TYPE_NYM:           id = TYPE_NYM;           break;
  case FRENT_TYPE_NYM_SRV:       id = TYPE_NYM_SRV;       break;
  case FRENT_TYPE_SER_SRV:       id = TYPE_SER_SRV;       break;
  case FRENT_TYPE_TOK_SRV:       id = TYPE_TOK_SRV;       break;
  case FRENT_TYPE_KEYQRY_SRV:    id = TYPE_KEYQRY_SRV;    break;
  case FRENT_TYPE_KEYUPD_SRV:    id = TYPE_KEYUPD_SRV;    break;
  case FRENT_TYPE_ANON:          id = TYPE_ANON;          break;
  case FRENT_TYPE_NNTP_SRV:      id = TYPE_NNTP_SRV;      break;
  case FRENT_TYPE_FREEDOM_YEAR:  id = TYPE_FREEDOM_YEAR;  break;
  case FRENT_TYPE_FREEDOM_MONTH: id = TYPE_FREEDOM_MONTH; break;
  case FRENT_TYPE_SOFTUPD:       id = TYPE_SOFTUPD;       break;
  case FRENT_TYPE_LINK:          id = TYPE_LINK;          break;
  case FRENT_TYPE_HOST:          id = TYPE_HOST;          break;
  case FRENT_TYPE_NIQS:          id = TYPE_NIQS;          break;
  case FRENT_TYPE_NISS:          id = TYPE_NISS;          break;
  case FRENT_TYPE_POP:           id = TYPE_POP;           break;
  case FRENT_TYPE_NMTA:          id = TYPE_NMTA;          break;
  case FRENT_TYPE_IMEP:          id = TYPE_IMEP;          break;
  case FRENT_TYPE_COREAIP:       id = TYPE_COREAIP;       break;
  case FRENT_TYPE_ANONRAND:      id = TYPE_ANONRAND;      break;
  default:
    CZKS::Throw(ZKS_UAIPPROXY_MOD,
                ZKS_UAIPPROXY_ID_ERR,
                func,
                "Invalide FrEntIdType: %d", (int32_t)id);    
  }
  return id;
}


//
//
//
Entity::Entity()
{
  memset(&m_ent, 0, sizeof m_ent);
}


//
//
//
Entity::Entity(const std::string & entType, const std::string & entName)
{
  DECL_FUNC("Entity::Entity");
  CZKS::Exception  ex;

  if( !frEntSetFromStr2(ex.zkError(), &m_ent, entType.c_str(), entName.c_str()) ) {
    ex.Push(func, "frEntSetFromStr2 failed").Throw();    
  }
}


//
//
//
Entity::Entity(const Entity & ent)
{      
  DECL_FUNC("Entity::Entity(copy)");

  try {
    (*this) = ent;
  }
  catch( ... ) {
    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
Entity::~Entity() 
{
}


//
//
//
Entity & Entity::operator=(const Entity & ent)
{
  DECL_FUNC("Entity::operator=(copy)");
  CZKS::Exception  ex;

  if( this != &ent ) {
    if( !frEntCopy(ex.zkError(), &m_ent, const_cast<FrEntIdExp *>(&ent.m_ent)) ) {
      ex.Push(func, "frEntCopy failed").Throw();
    }
  }
  return *this;
}
    

//
//
//
bool Entity::isValidIdType(const id_type & type)
{
  DECL_FUNC("Entity::isValidIdType");
  CZKS::Exception  ex;
  int              valid = 0;

  if( !frEntTypeIsValid(ex.zkError(), toFrEntIdType(type), &valid) ) {
    ex.Push(func, "frEntTypeIsValid failed").Throw();
  }  
  return valid ? true : false;
}


//
//
//
bool Entity::isValid()
{
  DECL_FUNC("Entity::operator=(copy)");
  CZKS::Exception  ex;
  int              valid = 0;

  if( !frEntIsValid(ex.zkError(), &m_ent, &valid) ) {
    ex.Push(func, "frEntIsValid failed").Throw();
  }  
  return valid ? true : false;
}


//
//
//
bool Entity::mayRevokeKey(const Entity & victim)
{
  DECL_FUNC("Entity::mayRevokeKey");
  CZKS::Exception  ex;
  int              allow = 0;

  if( !frEntMayRevokeKey(ex.zkError(), 
                         &m_ent, 
                         const_cast<FrEntIdExp *>(&victim.m_ent), 
                         &allow) ) {
    ex.Push(func, "frEntMayRevokeKey failed").Throw();
  }  
  return allow ? true : false;
}


//
//
//
Entity::id_type Entity::getSigKeySignerIdType(id_type idt)
{
  DECL_FUNC("Entity::getSigKeySignerIdType");
  CZKS::Exception  ex;
  FrEntIdType      signer;

  if( !frEntGetSignerEntIdType(ex.zkError(), &signer, 0, toFrEntIdType(idt)) ) {
    ex.Push(func, "frEntGetSignerEntIdType failed").Throw();
  }  
  return toIdType(signer);
}


//
//
//
Entity::id_type Entity::getCryptKeySignerIdType(id_type idt)
{
  DECL_FUNC("Entity::getCryptKeySignerIdType");
  CZKS::Exception  ex;
  FrEntIdType      signer;

  if( !frEntGetSignerEntIdType(ex.zkError(), &signer, 0, toFrEntIdType(idt)) ) {
    ex.Push(func, "frEntGetSignerEntIdType failed").Throw();
  }  
  return toIdType(signer);
}


//
//
//
Entity Entity::getCryptKeySigningParent()
{
  DECL_FUNC("Entity::isCryptKeySigningParent");
  CZKS::Exception  ex;
  Entity           parent;

  if( !frEntIsKeySigningParent(ex.zkError(), &m_ent, 1, &parent.m_ent) ) {
    ex.Push(func, "frEntIsKeySigningParent failed").Throw();
  }  
  return parent;
}


//
//
//
Entity Entity::getSigKeySigningParent()
{
  DECL_FUNC("Entity::isSignKeySigningParent");
  CZKS::Exception  ex;
  Entity           parent;

  if( !frEntIsKeySigningParent(ex.zkError(), &m_ent, 0, &parent.m_ent) ) {
    ex.Push(func, "frEntIsKeySigningParent failed").Throw();
  }  
  return parent;
}


//
//
//
uint32_t Entity::rawEntSize() const
{
  return (uint32_t)FRENT_PORTABLE_SIZE;
}


//
//
//
uint32_t Entity::rawEntStore(void * buff, uint32_t len) const
{
  DECL_FUNC("Entity::rawEntStore");
  CZKS::Exception  ex;
 
  if( !frEntStore(ex.zkError(), 
                  const_cast<FrEntIdExp *>(&m_ent), 
                  buff, 
                  len) ) {
    ex.Push(func, "frEntStore failed").Throw();
  }
  return rawEntSize();
}


//
//
//
void Entity::rawEntLoad(const void * buff, uint32_t len)
{
  DECL_FUNC("Entity::rawEntLoad");
  CZKS::Exception  ex;

  if( !frEntLoad(ex.zkError(), 
                 const_cast<FrEntIdExp *>(&m_ent), 
                 const_cast<void *>(buff),
                 len) ) {
    ex.Push(func, "frEntLoad failed").Throw();
  }  
}


//
//
//
std::string Entity::getTypeAsString() const
{
  DECL_FUNC("Entity::rawEntLoad");
  CZKS::Exception  ex;
  const char      *str;

  if( !frEntGetTypeStr(ex.zkError(), 
                       const_cast<FrEntIdExp *>(&m_ent),
                       &str) ) {
    ex.Push(func, "frEntGetTypeStr failed").Throw();
  }  
  return std::string(str);
}

