#ifndef ENTITY_H
# define ENTITY_H


#include <zksexception.h>
#include "frEnt.h"
#include "aig.h"

/*
 * C++ declarations
 */
# ifdef __cplusplus



// ///////////////////////////////////////////////////////////////////////////
// Entity handler
//
class Entity {
  enum id_type {
    TYPE_UNUSED_,
    TYPE_AIP,
    TYPE_FMG,
    TYPE_MAIP,
    TYPE_MASTER,
    TYPE_NIDB,
    TYPE_NYM,
    TYPE_NYM_SRV,
    TYPE_SER_SRV,
    TYPE_TOK_SRV,
    TYPE_KEYQRY_SRV,
    TYPE_KEYUPD_SRV,
    TYPE_ANON,
    TYPE_NNTP_SRV,
    TYPE_FREEDOM_YEAR,
    TYPE_FREEDOM_MONTH,
    TYPE_SOFTUPD,
    TYPE_LINK,
    TYPE_HOST,
    TYPE_NIQS,
    TYPE_NISS,
    TYPE_POP,
    TYPE_NMTA,
    TYPE_IMEP,
    TYPE_COREAIP,
    TYPE_ANONRAND
  };

 public:  
  static FrEntIdType toFrEntIdType(const id_type & id);
  static id_type     toIdType(const FrEntIdType & frid);
  static bool        isValidIdType(const id_type & type);
  static id_type     getSigKeySignerIdType(id_type idt);
  static id_type     getCryptKeySignerIdType(id_type idt);

 public:
  Entity();
  Entity(const std::string & entType, const std::string & entName);
  Entity(const Entity & ent);
  virtual ~Entity() ;

  Entity & operator=(const Entity & ent);

  bool        isValid();

  bool        mayRevokeKey(const Entity & victim);

  // Signing entiry
  Entity      getCryptKeySigningParent();
  Entity      getSigKeySigningParent();

  // Raw entity representation  
  uint32_t    rawEntSize() const;
  uint32_t    rawEntStore(void * buff, uint32_t len) const;
  void        rawEntLoad(const void * buff, uint32_t len);

  // Type handling
  std::string getTypeAsString() const;

  const FrEntIdExp & fr_ent() const { return m_ent; }

 private:
  FrEntIdExp m_ent;
};



# endif /* __cplusplus */  

#endif /* ENTITY_H */
