/* evil, evil, evil */
#define CONFIG_X86_TSC

#include <linux/config.h>
#include <linux/module.h>
#include <linux/modversions.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/in.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/if_arp.h>
#include <linux/inetdevice.h>
#include <linux/smp_lock.h>
#include <linux/unistd.h>
#include <linux/init.h>
#include <linux/random.h>
#include <linux/netlink.h>
#include <linux/proc_fs.h>
#include <linux/file.h>

#include <net/ip.h>
#include <net/tcp.h>
#include <net/icmp.h>
#include <net/protocol.h>
#include <net/arp.h>
#include <net/checksum.h>

#include "kaipreq.h"
#include <linux/crypto.h>

/* 
 *      Kernel Accelerator for Freedom[R] Anonymous IP version 1.
 *
 * Copyright (C) 2000 Zero-Knowledge[R] Systems, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 * --
 *TODO
 * put smarter outgoing packet version specifier
 * bpf for specifying return traffic?  awesome.
 * fill in interface stat numbers
 * make error paths show up in netstats, document
 * add byte counters per nei/pr in proc printouts
 * really make multiple kaips work.
 * recv_datagram returns -512 on restart?
 * verify ALWAYS_DEFRAG does the right thing?
 * make sure client == ip address assumptions are sound
 * clean up logging of sensitive info (?)
 * rmmod races with ioctl?
 * turn nei->aci lookup into something more efficient than dual lists
 * sanitize all ip headers we send out
 * add ability to do anon forwarding
 * make sure all protocol switches are complete
 * look at usage patterns and struct member ordering
 * look through code for chris-evans-esque MAX/MIN thinkos
 * make sure key doesn't stay around in memory if we can
 * --
 *
 *  Based on examples in other code written by ANK, sct
 *  and Alan.  Many thanks to Alan, Andi, and Jamal for 
 *  putting up with my annoying questions.
 *
 *  The first pass locking of data structures for the kaip
 *  is pretty simple.  We have a Big Scary rwlock per kaip
 *  that protects nearly all of the lists that the fast
 *  path needs.  All these accesses are done by the threads
 *  or the ioctl() path, so we don't have to worry about
 *  interrupts.  The packet pending list is the only
 *  thing filled by the int, so it's lock is irq safe, but
 *  maybe we could use a lock-free producer/consumer setup
 *  there?
 *
 *  In general, the accessor functions for the data structures
 *  are called with the lock held because the caller tends
 *  to do groups of these at a time.  
 *  
 *  The codepaths like to get pointers to structures and then
 *  use them to do very slow crypto.  We use refcounting in
 *  the structures to avoid them being freed while they're
 *  still in use.  The structures get a reference for anything
 *  that cares about them being around: being in lists, other
 *  structures storing a pointer to them, code paths having
 *  them while doing work.  When incrementing the refcount
 *  for a struct, it is important to remember to do that
 *  while still in the critical section that covers the 
 *  struct being removed from the data structure that gives
 *  access to others.  
 *
 *  Its dangerous to get a reference, let go of the list lock,
 *  then grab the list lock and do list manipulation on the reference
 *  that spanned the lock dance.   someone could have messed with
 *  its place in the list while you were without lock.  try_link_dec
 *  introduces the crappy live field to deal with this.  The rest tend
 *  to only do list maniuplation on a ref while they're still holding
 *  the lock that got them the ref.
 *
 *  SMP can bite me.
 */

/*this should really be done in the makefile system... */
#ifdef FREEDOM_DEVELOPMENT_BUILD
#define KAIP_DEBUG
#warning "Compiling kaip in debug mode"
#endif

static int num_devs=1;

#ifdef KAIP_DEBUG

#define DP_CON  (1<<0)  /* connection mapping stuff */
#define DP_NEI  (1<<1)  /* neighbor lists */
#define DP_INV  (1<<2)  /* invalid packet errors */
#define DP_ACI  (1<<3)  /* aci management */
#define DP_FN   (1<<4)  /* random info to watch code paths */
#define DP_PKT  (1<<5)  /* display packet stuff */
#define DP_HEX  (1<<6)  /* hexdumps of any kind */
static int kaip_debug=DP_HEX;

#define DPRINTK(BITS, ARG...) {\
    if(kaip_debug & DP_##BITS) {\
        printk("kaip: " ARG);\
    }\
}
#define DEBUGDO(BITS,FOO) {\
    if(kaip_debug & DP_##BITS) {\
        FOO;\
    }\
}
#else
#define DPRINTK(arg...) do {} while (0)
#define DEBUGDO(arg...) do {} while (0)
#endif

#define MIN(a,b) ((a) < (b) ? (a) : (b))

/*
 * crutches for kernels < 2.3.mumble
 */
#ifndef list_item
#define list_item list_entry
#endif
#ifndef list_for_each
#define list_for_each(pos, head) \
        for (pos = (head)->next; pos != (head); pos = pos->next)
#endif
#ifndef list_add_tail
static __inline__ void list_add_tail(struct list_head *new, struct list_head *head)
{
    __list_add(new, head->prev, head);
}
#endif

typedef u16 aci_t;

#define RAND_PAD_BYTES 8

struct freedomhdr {
    u8 rand[RAND_PAD_BYTES];
    u8 vers;
    u8 type;
    u8 priority;
    u8 pad[3];
    u16 aci; /* Anonymous Circuit Identifier */
    /* 
     * these next two are covered by the 
     * telescope encryption, but are contiguous
     * with the rest of the header..
     */
    u8 payload_rand[RAND_PAD_BYTES];
    u16 payloadlen;
};

/*
 * clearly we need OPP.. other people's packet.
 */
enum {
    OP_RCV, 
    OP_XMIT
};
/*
 * I'm told by people who undestand queueing theory
 * that blindly increasing this will do Bad Things.
 * Please investigate things like congestive collapse
 * before upping this..
 */
#define OP_LIMIT    69

struct our_packet {
    struct list_head pending_item;
    int control;
    struct sk_buff *skb;
    struct device_container *dc;
};

#define MAX_PAYLOAD_LEN 252
#define MAC_BYTES 10
#define MAC_KEYLEN_BYTES 20
#define EIGHTS_BYTES 8
#define DORKY_ALIGN_PAD 2
#define MIN_SANE_PAYLOAD (sizeof(struct freedomhdr) + EIGHTS_BYTES + 1)

#define DIR_FORWARD 0
#define DIR_REVERSE 1
#define MAX_LINK_KEYS 2

static u8 eights[EIGHTS_BYTES] = {
    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08
};
static unsigned char zeros[NLMSG_ALIGNTO] = {0,};

/* 
 * Someone who will be sending us data on
 * the freedom port
 */
struct kaip_neighbor {

    atomic_t    refcount;
    /* list items uses to traverse the IP list
     * when resolving key decryption/NAT problems.
     */
    struct list_head ip_list;
    /* list to traverse on hash collision in the
     * ip/port/key MRU
     */
    struct list_head ip_port_list;
    /*
     * see aci_lookup()
     */
    struct list_head aci_client_list_head;
    struct list_head aci_server_list_head;

    struct cipher_implementation *cipher_funcs;

    struct list_head nei_list;
    /*
     * We statically allocate all of these now because we're
     * pretty much guaranteed to use them.  If we ever actually
     * have sparse usage we should have an array of pointers, etc
     */
    struct cipher_context link_from_ctx[MAX_LINK_KEYS], link_to_ctx[MAX_LINK_KEYS];

    atomic_t        failed_link_dec;

    struct sockaddr_in addr;
    u32                 id;
    u8                 active_to_key;
    volatile int        live;
};

/*
 * our view of the route that is passing
 * through us.  freedom tunnels are directional
 * beasts.. client_* and server_* are so named
 * for the forward direction when you're 
 * coming from the client and to the outside
 * world and ultimately the destination server.
 *
 * *client* -> client_aci -> *us* -> server_aci -> *next route node*
 *
 * 'in' is traffic coming in to us, 'out' is traffic we
 * send out.
 *
 * a NULL server_out specifies that we are
 * the exit node.
 */
struct partial_route {
    struct list_head                aci_client_list_item;
    struct list_head                aci_server_list_item;
    aci_t                           client_aci, server_aci;
    struct kaip_neighbor            *client_nei, *server_nei;
    struct cipher_implementation    *cipher_funcs;
    struct cipher_context           telescope_in_ctx, telescope_out_ctx;
    struct digest_implementation    *hmac_funcs;
    struct digest_context           hmac_ctx;
    struct list_head                pr_list;
    struct list_head                pr_ip_list_item;
    unsigned int                    padded_size;
    struct device_container         *dc;
    u32                             fake_addr;
    atomic_t                        refcount;
};

#define IP_HASH_BITS (13)
#define IP_HASH_ENTRIES (1<<IP_HASH_BITS)
#define IP_HASH(SADDR) ( ((SADDR)^((SADDR) >> IP_HASH_BITS)) & (IP_HASH_ENTRIES-1) )

#define IP_PORT_HASH(SADDR, SOURCE) IP_HASH((SADDR) ^ (SOURCE))

struct ip_container {
    struct list_head ip_list_item;
    u32 saddr;
    unsigned int count;
    struct list_head ip_list_head;
};

/* see proc_doit: */
struct  proc_guy {
    struct kaip_instance *kaip;
    int line_len; /* including /n */
    char *header;
    unsigned int header_len;
    struct list_head *list;
    int (*printer)(char *buffer, struct list_head *pos);
};

struct device_container {
    struct list_head dc_list_item;
    struct kaip_instance *kaip;
    struct net_device_stats stats;
    unsigned char name[9];
    struct device dev;
};

struct kaip_instance {

    struct socket *sock, *netlinksock;
    struct file *filp;

    int             num_pending;
    struct list_head pending_packets;
    spinlock_t pp_lock;

    /*
     * locks around all list accesses
     */
    rwlock_t list_lock;

    /* 
     * these lists record all the data
     * associated with the tunnel.  used
     * to tear it down.
     */
    struct list_head nei_list_head;
    struct list_head pr_list_head;

    struct kaip_stats kstats;
    struct list_head dc_list;

    /*
     * hashes used to find neighbors based on
     * ip or ip+port
     */
    struct list_head *ip_port_hash;
    struct list_head *ip_hash;
    struct list_head *pr_ip_hash;

    struct sockaddr_nl nladdr;

    unsigned int        max_nei_attempts;
    int                 last_nei_disc;
    int                 num_nei_disc;
    int                 max_nei_disc;

    volatile int        shutdown;
    atomic_t            num_threads;
    struct wait_queue * shutdown_wq;
    struct semaphore    shutdown_sema;


    struct proc_guy     pg_nei;
    struct proc_guy     pg_pr;

    struct kaip_instance *next;
};

/*
 * For now this always only has one item on the list..
 */
static struct kaip_instance *global_kaip_list = NULL;

/* --- */
static void leave_cloud(struct sk_buff *skb, struct kaip_instance *kaip, 
        struct freedomhdr *fh, struct partial_route *pr);
static int kaip_process_ingress(struct kaip_instance *kaip, 
        struct device_container *dc, struct sk_buff *skb);
static void kaip_decref_pr(struct partial_route *pr);
static void kaip_decref_nei(struct kaip_neighbor *nei);
static void nei_ip_port_remove(struct kaip_instance *kaip,
        struct kaip_neighbor *nei);
/* -- */

#ifdef KAIP_DEBUG
/*
 * Quick debugging hack to show IP packets.
 *
 * This shouldn't be blindly following ihl into 
 * headers, but its debugging so I'm lazy.. (boo)
 */
static void dump_ip(char *string, struct iphdr *iph)
{
    struct udphdr *uh;
    struct tcphdr *th;

    printk("%s: ip: %d.%d.%d.%d:%d.%d.%d.%d v: %d il: %d tl: %d cs: %d ",
            string, 
            NIPQUAD(iph->saddr),
            NIPQUAD(iph->daddr),
            iph->version, 
            iph->ihl,
            ntohs(iph->tot_len),
            ip_fast_csum((char *)iph, iph->ihl));

    th = (struct tcphdr *)((char *)iph + (iph->ihl << 2));
    uh = (struct udphdr *)th;

    switch(iph->protocol) {
        case IPPROTO_TCP:
            printk("tcp: %d:%d ",
                    ntohs(th->source), ntohs(th->dest));
            break;
        case IPPROTO_UDP:
            printk("udp: %d:%d ",
                    ntohs(uh->source), ntohs(uh->dest));
            break;
        default:
            printk("?proto ");
            break;
    }
    printk("\n");
}
static void hexdump(char *string, void *ptr, int len)
{
    unsigned char *c = ptr;
    int i;

    if(! (kaip_debug & DP_HEX))
        return;

    printk("%d bytes of '%s' from 0x%p\n",
            len, string, ptr);

    for(i = 0; i < len;) {
        printk("%02x",*(c++));
        i++;
        if( ! (i & 15)) {
            printk("\n");
        } else if( ! (i&1)) {
            printk(" ");
        }
    }
    if(len & 15) {
        printk("\n");
    }
}
#endif

/*
 * Our poor reimplementation of a routing table :)
 * Find the returning tunnel based on its fake address.
 * see, the kaip should really be moving towards having
 * proper tunnel devices for each freedom route so that
 * the kernel would do this for us and we'd just do
 * pr = dev->priv; waiting for 2.4 before going down 
 * that road..
 */
static struct partial_route * kaip_pr_ip_lookup(struct kaip_instance *kaip, u32 saddr)
{
    struct list_head *pos, *head = &(kaip->pr_ip_hash[IP_HASH(saddr)]);
    struct partial_route *ret = NULL;

    read_lock(&(kaip->list_lock));

    list_for_each(pos, head) {
        struct partial_route *pr = list_item(pos, struct partial_route, pr_ip_list_item);

        if(pr->fake_addr == saddr) {
            ret = pr;
            break;
        }
    }

    if(ret)
        atomic_inc(&(ret->refcount));

    read_unlock(&(kaip->list_lock));

    DPRINTK(NEI, "pr_ip_lookup returning 0x%p for %d.%d.%d.%d\n",ret, 
            NIPQUAD(saddr));

    return ret;
}
static void kaip_pr_ip_insert(struct kaip_instance *kaip, struct partial_route *pr)
{
    struct list_head *head = &(kaip->pr_ip_hash[IP_HASH(pr->fake_addr)]);

    list_add(&(pr->pr_ip_list_item), head);
}
static void kaip_pr_ip_remove(struct kaip_instance *kaip, struct partial_route *pr)
{
    list_del(&(pr->pr_ip_list_item));
}

/*
 * see if we have a neighbor associated with a given udp
 * source ip and port.  
 */
static struct kaip_neighbor * nei_ip_port_lookup(struct kaip_instance *kaip, 
        u32 saddr,u16 source)
{
    struct list_head *pos, *head = &(kaip->ip_port_hash[IP_PORT_HASH(saddr, source)]);
    struct kaip_neighbor *nei, *ret = NULL;

    list_for_each(pos, head) {
        nei = list_item(pos, struct kaip_neighbor, ip_port_list);

        if(nei->addr.sin_addr.s_addr == saddr && nei->addr.sin_port == source)  {
            ret = nei;
            break;
        }
    }

    DPRINTK(NEI, "ip_port_lookup returning 0x%p for %d.%d.%d.%d:%d from bucket 0x%p\n",ret, 
            NIPQUAD(saddr), ntohs(source), head);

    return ret;
}

/* 
 * we do some maintenance by id.  this is highly lame, but
 * shouldn't be performance critical.  It'll show up
 * in the profiles if it is :)
 */
static struct kaip_neighbor *nei_id_lookup(struct kaip_instance *kaip,
        u32 id)
{
    struct list_head *pos;
    struct kaip_neighbor *nei, *ret = NULL;

    list_for_each(pos, &(kaip->nei_list_head)) {
        nei = list_item(pos, struct kaip_neighbor, nei_list);

        if(nei->id < id) continue;
        if(nei->id > id) break;

        ret = nei;
        break;
    }

    DPRINTK(NEI, "nei_id_lookup ret nei 0x%p for id 0x%lx on kaip 0x%p\n", ret, id, kaip);

    return ret;
}

int nei_id_add(struct kaip_instance *kaip, struct kaip_neighbor *new_nei)
{
    struct list_head *pos;
    struct kaip_neighbor *nei;
    int ret = 1;

    DPRINTK(NEI, "adding nei 0x%p as id 0x%lx on kaip 0x%p\n", 
            new_nei, new_nei->id, kaip);

    list_for_each(pos, &(kaip->nei_list_head)) {
        nei = list_item(pos, struct kaip_neighbor, nei_list);

        if(new_nei->id < nei->id) {
            list_add(&(new_nei->nei_list), pos->prev);
            ret = 0;
            break;
        }
        if(new_nei->id == nei->id) {
            ret = -EBUSY;
            break;
        }
    }

    /* 
     * either no list or fell off the end, in either case we go
     * at the end of the list..
     */
    if(ret == 1) {
        list_add_tail(&(new_nei->nei_list), &(kaip->nei_list_head));
        ret = 0;
    }

    return ret;
}


/*
 * we got a valid link decryption for a neighbor from 
 * some source ip and udp port.  to avoid rampant dos
 * we require that source ip/port are constant for
 * a neighbor.  
 * 
 * The caller is responsible for locking around accesses to
 * these and to grabbing refcounts if they care about the reference
 * after they release the lock.
 *
 */
static void nei_ip_port_insert(struct kaip_instance *kaip, 
        struct kaip_neighbor *nei, u32 saddr, u16 port)
{
    struct list_head *head = &(kaip->ip_port_hash[IP_PORT_HASH(saddr, port)]);
    struct kaip_neighbor *old = nei_ip_port_lookup(kaip, saddr, port);

    if(old) {
        /* 
         * oh great, someone in our spot?
         * no kitty, thats my chicken-pot-pie!
         */
        nei_ip_port_remove(kaip, old);
    }

    /*
     * oh man, with all this idiotic 'neighbor key 
     * discovery' sh*t, it could happen that some joker
     * convinces us to be in the hash from more than
     * one place if they send from multiple ports with
     * the same link keys..
     */
    if(! list_empty(&(nei->ip_port_list)))
        list_del(&(nei->ip_port_list));

    list_add(&(nei->ip_port_list), head);

    DPRINTK(NEI, "ip_port_inserting nei 0x%p (%d.%d.%d.%d:%d) into bucket 0x%p\n", nei, 
            NIPQUAD(saddr), ntohs(port),head);
}

static void nei_ip_port_remove(struct kaip_instance *kaip,
        struct kaip_neighbor *nei)
{
    DPRINTK(NEI, "removeing nei 0x%p from ip_port on kaip 0x%p\n",nei, kaip);

    if(! list_empty(&(nei->ip_port_list)))
        list_del(&(nei->ip_port_list));

    INIT_LIST_HEAD(&(nei->ip_port_list));
}

/*
 * We need to get the list head of the list that
 * contains the neighbors who haven't established
 * a udp source mapping but who we have keys for
 */
static struct ip_container * nei_ip_lookup(struct kaip_instance *kaip,
        u32 saddr)
{
    struct list_head *pos, *head = &(kaip->ip_hash[IP_HASH(saddr)]);
    struct ip_container *ret = NULL;

    list_for_each(pos, head) {
        struct ip_container *buck;
        buck = list_item(pos, struct ip_container, ip_list_item);

        if(buck->saddr == saddr)  {
            ret = buck;
            break;
        }
    }

    DPRINTK(NEI, "ip_lookup returning 0x%p (cnt %d) for %d.%d.%d.%d\n",
            ret, ret ? ret->count : 0, NIPQUAD(saddr));

    return ret;
}

/*
 * adding a neighbor to the IP list.  If we don't find
 * a bucket for the IP, we have to add it.
 */
static int nei_ip_insert(struct kaip_instance *kaip, 
        struct kaip_neighbor *nei, u32 saddr)
{
    struct ip_container *buck;

    if( (buck = nei_ip_lookup(kaip, saddr)) != NULL ) {
        list_add(&(nei->ip_list), &(buck->ip_list_head));
        buck->count++;
        return 0;
    }

    /*
     * again with the allocation with locks
     * held :/
     */
    buck = kmalloc(sizeof(struct ip_container), GFP_ATOMIC);
    if(!buck) return -ENOMEM;

    buck->saddr = saddr;
    INIT_LIST_HEAD(&(buck->ip_list_head));
    list_add(&(nei->ip_list), &(buck->ip_list_head));
    list_add(&(buck->ip_list_item), &(kaip->ip_hash[IP_HASH(saddr)]));
    buck->count = 1;

    return 0;
}

static void nei_ip_remove(struct kaip_instance *kaip,
        struct kaip_neighbor *nei)
{
    struct ip_container *buck;

    if( (buck = nei_ip_lookup(kaip, nei->addr.sin_addr.s_addr)) == NULL )
        return; 

    if( ! list_empty(&(nei->ip_list)) ) {
        list_del(&(nei->ip_list));
        buck->count--;
        INIT_LIST_HEAD(&(nei->ip_list));
    }

    if(list_empty(&(buck->ip_list_head))) {
        list_del(&(buck->ip_list_item));
        kfree(buck);
    }
}


static int prepare_nl_iov(struct msghdr *msg,
        struct iovec *iov, int nent)
{
    int i, len = 0;

    /* fill in padding */
    for(i = 2; i < nent-1; i+=3) {
        struct nlmsghdr *nlh = iov[i-2].iov_base;
        /*
         * padd out the alignment overhead in
         * nlmsg_len with zeros by subtracting the
         * real payload len and nlmsg hdr..
         */
        iov[i].iov_len = nlh->nlmsg_len - 
            (iov[i-1].iov_len + iov[i-2].iov_len);
    }

    for(i = 0; i < nent; i++)
        len += iov[i].iov_len;

    msg->msg_iov = iov;
    msg->msg_iovlen = nent;
    msg->msg_control = NULL;
    msg->msg_controllen = 0;
    msg->msg_flags = 0;

    return len;
}

/*
 * free keyinfo shit in nei key ctx
 */
static void cleanup_keyinfo(struct kaip_neighbor *nei)
{
    int i;

    for(i = 0; i < MAX_LINK_KEYS; i++) {
        if(nei->link_to_ctx[i].keyinfo)
            kfree(nei->link_to_ctx[i].keyinfo);
        if(nei->link_from_ctx[i].keyinfo)
            kfree(nei->link_from_ctx[i].keyinfo);
    }
}       

/*
 * Don't ask.  see try_link_dec()
 */
static void rekey_and_notify(struct kaip_instance *kaip,
        struct kaip_neighbor *nei,
        struct kaip_neighbor *new_nei)
{
    unsigned char ids[2 * sizeof(nei->id)];
    struct nlmsghdr 
        idhdr   = {NLMSG_LENGTH(sizeof(ids)), 1, NLM_F_MULTI, current->pid, 0},
        donehdr = {NLMSG_LENGTH(0), NLMSG_DONE, 0, current->pid, 0};
    struct msghdr msg;
    struct iovec iov[4] = {
        {&idhdr, sizeof(idhdr)},
        {ids, sizeof(ids)},
        {zeros, 0},
        {&donehdr, sizeof(donehdr)}
    };
    int len;

    cleanup_keyinfo( nei);
   
    memcpy(nei->link_from_ctx, new_nei->link_from_ctx, sizeof(nei->link_from_ctx));
    memcpy(nei->link_to_ctx, new_nei->link_to_ctx, sizeof(nei->link_to_ctx));

    /* zero out new_nei key ctx since they belong to nei now */
    memset(new_nei->link_from_ctx, 0, sizeof(new_nei->link_from_ctx));
    memset(new_nei->link_to_ctx, 0, sizeof(new_nei->link_to_ctx));

    memcpy(ids, &(new_nei->id), sizeof(nei->id));
    memcpy(ids + sizeof(nei->id), &(nei->id), sizeof(nei->id));

    len = prepare_nl_iov(&msg, iov, sizeof(iov) / sizeof(iov[0]));

    msg.msg_name = &(kaip->nladdr);
    msg.msg_namelen = sizeof(kaip->nladdr);

    sock_sendmsg(kaip->netlinksock, &msg, len);
}

/*
 * wrappers to get scheduling and stats
 * collection
 */
typedef int (*cipher_func_t)(struct cipher_context *ctx, 
                                  const u8 *in, u8 *out, int size);

static void do_cipher(cipher_func_t fn, struct cipher_context *cx,
        const u8 *in, u8 *out, int size, 
        struct kaip_cycle_measurement *st)
{
    cycles_t start;

    /* 
     * make the box usable while we're working
     */
    if(current->need_resched)
        schedule();

    start = get_cycles();

    fn(cx, in, out, size);

    st->cycles += get_cycles() - start;
    st->num += size;
}

/*
 * returns 1 on success..
 */
static int do_link_dec(struct kaip_instance *kaip, struct kaip_neighbor *nei, int key, 
        unsigned char *from, unsigned char *to, unsigned int data_len)
{

    do_cipher(nei->cipher_funcs->decrypt,
            &(nei->link_from_ctx[key]),
            from, to, data_len, &kaip->kstats.link_dec);

    if(! memcmp(to + (data_len - (EIGHTS_BYTES)), eights, EIGHTS_BYTES)) 
        return 1;

    return 0;
}

/*
 * It burns, it burns!
 *
 * Our key negotiation is currently fully anonymous and
 * we have no in-the-clear client identifier in packets.
 * This sucks a whole lot.  Have I mentioned the sucking 
 * part?  Allow me to detail the sucking part.  It
 * sucks.
 *
 * 1) We don't know what UDP port a brand new client's data
 * packets will come from.  When we get a packet from an
 * unrecognized UDP port, we try and link decrypt it with
 * the pending keys until we find one that works.
 *
 * 2) We don't know what UDP port a client behind NAT 
 * will come from so we don't know what key to use to link 
 * decrypt the packet, and the source port may change with
 * multiple clients behind a NAT.  So we just punt and say
 * that the client has to send keep-alives to keep the NAT
 * mapping around so the source port won't change.
 *
 * 3) If a client negotiates a new key and reuses their 
 * UDP source port we won't know to use the new key.  On
 * link decryption failure, then, we try to decrypt the 
 * packet with the pending keys. 
 *
 * We rate limit the above to make some attempt of reducing
 * their effictiveness as denial of service attacks.  We also
 * require that the source IP address remain the same for 
 * a given client so that we can limit our pain.
 */
static struct kaip_neighbor * try_link_decrypt(struct kaip_instance *kaip, u32 saddr, 
        u16 source, unsigned char *data, unsigned int data_len)
{
    struct kaip_neighbor *nei, *failed_nei = NULL;
    struct ip_container *buck;
    struct list_head *pos;
    unsigned char *from = data, *data_copy = NULL;
    struct kaip_neighbor * ret = NULL, **try_list = NULL;
    unsigned int key, i, attempts;

    /* 
     * Extract the magical key byte from the end of the
     * packet, we're assuming the resulting len will
     * be ok for the cipher functions
     */
    data_len--;
    key = data[data_len];
    if(key >= MAX_LINK_KEYS) {
        DPRINTK(INV, "invalid key byte %d\n",key);
        return NULL;
    }

    read_lock(&(kaip->list_lock));

    nei = nei_ip_port_lookup(kaip, saddr, source);
    if(nei)
        atomic_inc(&(nei->refcount));

    buck = nei_ip_lookup(kaip, saddr);

    read_unlock(&(kaip->list_lock));

    if(buck != NULL) {
        /* we might need the packet again if link dec fails */
        if( (data_copy = kmalloc(data_len, GFP_ATOMIC)) == NULL) 
            return NULL;

        memcpy(data_copy, data, data_len);
        from = data_copy;
    }

    if(nei) {
        /* we have a cached neighbor, try it..*/
        if( do_link_dec(kaip, nei, key, from, data, data_len) ) {
            /* all right, we win. */
            ret = nei;
            goto free_out;
        }

        atomic_inc(&(nei->failed_link_dec));
        failed_nei = nei;
    }

    /* pending neighbors? */
    if(buck == NULL) 
        goto free_out;

    /*
     * copy the list so we don't do crypto with
     * the lock held
     */
    write_lock(&(kaip->list_lock));

    buck = nei_ip_lookup(kaip, saddr);
    if(buck == NULL) {
        write_unlock(&(kaip->list_lock));
        goto free_out;
    }

    /*
     * ok, we have pending neighbors, check rate limit
     */
    if(time_after(jiffies, kaip->last_nei_disc + HZ)) {
        kaip->last_nei_disc = jiffies;
        kaip->num_nei_disc = 0;
    }

    if(kaip->num_nei_disc >= kaip->max_nei_disc) {
        DPRINTK(INV, "max nei disc reached\n");
        write_unlock(&(kaip->list_lock));
        goto free_out;
    }

    kaip->num_nei_disc++;

    attempts = MIN(kaip->max_nei_attempts, buck->count);

    if( (try_list = kmalloc(sizeof(struct kaip_neighbor *) * attempts, GFP_ATOMIC)) == NULL) {
        write_unlock(&(kaip->list_lock));
        goto free_out;
    }

    i = 0;

    list_for_each(pos, &(buck->ip_list_head)) {
        nei = list_item(pos, struct kaip_neighbor, ip_list);

        atomic_inc(&(nei->refcount));
        try_list[i++] = nei;

        if(i == attempts)
            break;
    }

    write_unlock(&(kaip->list_lock));

    for(i = 0; i < attempts; i++) {
        nei = try_list[i];

        DPRINTK(INV, "walking list, trying nei 0x%p id 0x%lx ip %d.%d.%d.%d\n",
                nei, nei->id, NIPQUAD(nei->addr.sin_addr.s_addr));

        if( do_link_dec(kaip, nei, key, from, data, data_len) ) {

            write_lock(&(kaip->list_lock));

            /* 
             * ok, we have a neighbor with a pending key that this 
             * packet was sent with, record that and pull it out
             * of the pending list.
             */
            nei->addr.sin_port = source;
            nei_ip_remove(kaip, nei);

            /*
             * We're magically "detecting" a re-key here.  If we
             * get a link decryption from a pending key for a
             * packet that was sent from a port that another
             * neighbor was on, we assume (ass, you, me) that
             * this new neighbor on the port is really the old
             * neighbor with a new key.  Current client behaviour
             * requires is todo this hack if we want the client
             * to work.   FMGWAC.  So we move the new keys into
             * the old neighbor.  Rendering the old keys useless.
             * la la la.
             */
            if(failed_nei) {
                rekey_and_notify(kaip, failed_nei, nei);
                nei_ip_port_remove(kaip, nei);
                kaip_decref_nei(failed_nei);
                nei = failed_nei;
                failed_nei = NULL;
            }
            if(nei->live) {
                nei_ip_port_insert(kaip, nei, saddr, source);
                atomic_inc(&(nei->refcount));
                ret = nei;
            }

            write_unlock(&(kaip->list_lock));
            break;
        }
    }
    for(i = 0; i < attempts; i++) 
        kaip_decref_nei(try_list[i]);
        
free_out:
    if(failed_nei)
        kaip_decref_nei(failed_nei);
    if(data_copy) 
        kfree(data_copy);
    if(try_list)
        kfree(try_list);

    return ret;
}

/*
 * Return the route for an aci that came from a neighbor.
 * 
 * This is a little strange.  The current aci lookup
 * mechanism is laziness incarnate.  We have list items
 * in the partial_route that put it on a neighbor's
 * ACI lists.  Since packets from two neighbors point to
 * the same partial_route we want a partial_route to be
 * on two neighbor's aci lookup lists.  This means we need
 * to have 2 items in the partial_route, so we have these
 * two dorky lists.. 
 */
static struct partial_route * aci_lookup(struct kaip_neighbor *nei,
        aci_t aci)
{
    struct list_head *pos;

    DPRINTK(ACI, "aci_lookup for aci %d in nei 0x%p\n",ntohs(aci) , nei);

    /* XXX clearly this should be more efficient :) */
    list_for_each(pos, &(nei->aci_client_list_head)) {
        struct partial_route *pr = list_item(pos, struct partial_route, aci_client_list_item);

        DPRINTK(ACI, " trying cl pr 0x%p nei 0x%p aci %d\n",pr, pr->client_nei, ntohs(pr->client_aci));
        if(pr->client_nei == nei && pr->client_aci == aci) return pr;
    }
    list_for_each(pos, &(nei->aci_server_list_head)) {
        struct partial_route *pr = list_item(pos, struct partial_route, aci_server_list_item);

        DPRINTK(ACI, " trying server pr 0x%p nei 0x%p aci %d\n",pr, pr->server_nei, ntohs(pr->server_aci));
        if(pr->server_nei == nei && pr->server_aci == aci) return pr;
    }

    return NULL;
}

/*
 * Given a neighbor and ACI, return the 'direction' that
 * the packet is heading.  either from the client to the 
 * server (forward) or server to client (reverse).
 */
static inline int packet_direction(struct partial_route *pr, 
        struct kaip_neighbor *nei)
{
    if(pr->client_nei == nei) return DIR_FORWARD;
    /* aci_lookup guarantees that it had to have come
     * from one of our neighbors :)
     */
    return DIR_REVERSE;
}

/*
 * We do a copy here when sending the udp, but 
 * gets rid of our knowledge of the routing
 * crappies in the stack and makes it so we
 * can forward to aips on the same machine.
 */
static void sock_send_udp(struct socket *sock, struct sockaddr_in *sin,
        void *data, int len)
{
    struct msghdr msg;
    struct iovec iov[1] = {
        {data , len}
    };

    msg.msg_name = sin;
    msg.msg_namelen = sizeof(*sin);
    msg.msg_iov = iov;
    msg.msg_iovlen = sizeof(iov) / sizeof(iov[0]);
    msg.msg_control = NULL;
    msg.msg_controllen = 0;
    msg.msg_flags = 0;

    sock_sendmsg(sock, &msg, len);

    return;
}

/*
 * incremental checksum updating, yeah. 
 * m and mnew are new and old values of an
 * aligned 16bit region of a packet, in 
 * network byte order.
 */
static void csum_inc_u16(u16 *csum, u16 m, u16 mnew)
{
    u32 sum;

#if 0
    /* This is
     *   HC' = HC + m + ~m'  
     * from rfc1141, it has bad boundry
     * conditions from confusing -0
     * and +0, as rfc1624 explains.
     */
    sum = ntohs(m) + (~ntohs(mnew) & 0xffff);
    sum += ntohs(*csum);
#else
    /* This is the proper, fast
     *   HC' = HC - ~m - m'
     * from 1624 that behaves well 
     */
    sum = ntohs(*csum);
    sum -= ~ntohs(m) & 0xffff;
    sum -= ntohs(mnew);
#endif

    sum = (sum & 0xffff) + (sum >> 16);
    *csum = htons(sum);
}
/*
 * Again, the u32 arguments must be aligned in
 * the packet 
 */
static void csum_inc_u32(u16 *csum, u32 m, u32 mnew)
{
    csum_inc_u16(csum, (m>>16) & 0xffff, (mnew>>16) & 0xffff);
    csum_inc_u16(csum, m & 0xffff, mnew & 0xffff);
}

static void recalculate_full_csum(struct iphdr *iph) 
{
    struct udphdr *uh = NULL; /* kill gcc warning */
    struct tcphdr *th = NULL; /* ... */
    u16 space;

    iph->check = 0;
    iph->check = ip_fast_csum((unsigned char *)iph, iph->ihl);

    space = (ntohs(iph->tot_len) - (iph->ihl<<2));

    if((ntohs(iph->frag_off) & IP_OFFSET) != 0)
        return;

    switch(iph->protocol) {
        case IPPROTO_UDP:
            if(space < sizeof(struct udphdr)) 
                break;

             uh = (struct udphdr *)((char *)iph + (iph->ihl*4));
 
             uh->check = 0;
             uh->check = csum_tcpudp_magic(iph->saddr, iph->daddr, 
                            space, iph->protocol, 
                            csum_partial((unsigned char *)uh, space, 0));
             break;
         case IPPROTO_TCP:
            if(space < sizeof(struct tcphdr)) 
                break;

            th = (struct tcphdr *)((char *)iph + (iph->ihl*4));

            th->check = 0;
            th->check = csum_tcpudp_magic(iph->saddr, iph->daddr, 
                           space, iph->protocol, 
                           csum_partial((unsigned char *)th, space, 0)); 
            break;
        default:
            break;
    }
}

/*
 * we have already init-ed the hmac context by calling
 * update() which calculated the ipad based on the key.
 * we make a copy of the context and operate on it
 * so that we can reuse this calculated ipad.
 */
static void calculate_mac(struct kaip_instance *kaip,
        struct digest_implementation *di, 
        struct digest_context *cached_ctx,
        char *input, int inlen,
        char *output, int outlen)
{
    struct digest_context ctx;
    cycles_t start;
    unsigned char mac[20];

    memset(mac, 0, sizeof(mac));

    if(outlen > 20) {
        DPRINTK(INV, "mac output size %d too big\n", outlen);
        return;
    }

    /*
     * should be cheaper than the crypto work.. could slabify
     * if its not
     */
    if( (ctx.digest_info = kmalloc(di->working_size, GFP_KERNEL)) == NULL) {
        DPRINTK(FN, "couldn't alloc tmp hmac context\n");
        return;
    }

    ctx.di = di;

    memcpy(ctx.digest_info, cached_ctx->digest_info, di->working_size);

    if(current->need_resched)
        schedule();

    start = get_cycles();
    
    di->update(&ctx, input, inlen);
    di->close(&ctx, mac);

    memcpy(output, mac, outlen);

    kaip->kstats.mac.cycles += get_cycles() - start;
    kaip->kstats.mac.num += inlen;

    kfree(ctx.digest_info);
}

static int verify_mac(struct kaip_instance *kaip,struct freedomhdr *fh,
        struct partial_route *pr)
{
    unsigned char mac[MAC_BYTES];
    int err;
    /* XXX this is dumb, payloadlen should include the padding, no? */
    int fixedsz = RAND_PAD_BYTES + 2 + MAC_BYTES;
    int payloadlen = (pr->padded_size ? pr->padded_size : 
                      (((fixedsz + 
                         ntohs(fh->payloadlen) + 0x7) & ~0x7) - fixedsz)) +
                     RAND_PAD_BYTES + 2;

    DEBUGDO(PKT, (hexdump("calculate mac on pkt", fh->payload_rand, payloadlen)));
    calculate_mac(kaip, pr->hmac_funcs, 
            &(pr->hmac_ctx),
            fh->payload_rand, payloadlen,
            mac, MAC_BYTES);

    DEBUGDO(PKT, (hexdump("calculated mac ", mac, MAC_BYTES)));
    err = memcmp(mac, fh->payload_rand + payloadlen, MAC_BYTES);

    return err;

}

/*
 * If sendmsg() ever becomes zero copy this might be
 * exciting as dmaing from the kernel stack is questionable,
 * at best :)
 */
static void kaip_netlink_handoff(struct kaip_instance *kaip,
        struct kaip_neighbor *nei, struct iphdr *iph,
        struct freedomhdr *fh, int fhlen)
{
    struct nlmsghdr 
        idhdr   = {NLMSG_LENGTH(sizeof(nei->id)), 0, NLM_F_MULTI, current->pid, 0},
        IPhdr = {NLMSG_LENGTH(ntohs(iph->tot_len)), 0, NLM_F_MULTI, current->pid, 0},
        fhhdr = {NLMSG_LENGTH(fhlen), 0, NLM_F_MULTI, current->pid, 0},
        donehdr = {NLMSG_LENGTH(0), NLMSG_DONE, 0, current->pid, 0};
    struct msghdr msg;
    struct iovec iov[10] = {
        {&idhdr, sizeof(idhdr)},
        {&(nei->id), sizeof(nei->id)},
        {zeros, 0},
        {&IPhdr, sizeof(IPhdr)},
        {iph, ntohs(iph->tot_len)},
        {zeros, 0},
        {&fhhdr, sizeof(fhhdr)},
        {fh, fhlen},
        {zeros, 0},
        {&donehdr, sizeof(donehdr)}
    };
    int len;

    len = prepare_nl_iov(&msg, iov, sizeof(iov) / sizeof(iov[0]));

    msg.msg_name = &(kaip->nladdr);
    msg.msg_namelen = sizeof(kaip->nladdr);

    DPRINTK(INV, "handing off %d bytes to nl fam: %d pid: %d groups: 0x%x\n",
            len, kaip->nladdr.nl_family, kaip->nladdr.nl_pid, kaip->nladdr.nl_groups);

    /*
     * this works 'cause we're in a thread
     * with set_fs(KERNEL_DS), ignoring err
     * msg that would probably show that the 
     * user hasn't set nladdr.
     */
    sock_sendmsg(kaip->netlinksock, &msg, len);
}

/*
 * This is called for freedom packets that have 
 * been received.   This was originally written
 * to alter the skb in place to be handed back
 * to the stack, but that proved problematic.
 * Calling udp sendmsg makes our life easier.
 * should that copy ever prove life threatening 
 * (unlikely), this could moved back to sending
 * via skbs.
 */
static void kaip_rcv(struct sk_buff *skb, struct kaip_instance *kaip,
        struct iphdr *iph, struct udphdr *uh)
{
    struct freedomhdr *fh;
    struct kaip_neighbor *nei, *next_nei = NULL;
    struct partial_route *pr = NULL;

    unsigned int inner_len;
    unsigned int telescope_len;

    DPRINTK(FN, "kaip_rcv() s-l: %d ip-t_l: %d u-l: %d s->typ: %d s->dst: 0x%p\n",
            skb->len, ntohs(iph->tot_len), ntohs(uh->len),
            skb->pkt_type, skb->dst);

    DEBUGDO(PKT, (dump_ip("got packet", iph)) );

    /*
     * The stack probably guarantees this 'cause we just
     * grabbed it from skb_recv_datagram, but..
     */
    if(skb->len != ntohs(uh->len)) {
        DPRINTK(INV, "dropping, skb->len %d is bigger than uh->len %d\n",
                skb->len, ntohs(uh->len));
        return;
    }

    DEBUGDO(PKT, (hexdump("iph", iph ,ntohs(iph->tot_len))) );
    inner_len = ntohs(uh->len) - sizeof(struct udphdr);

    /* Don't even bother if we don't have enough to
     * decrypt an IP header
     */
    if(inner_len < MIN_SANE_PAYLOAD) {
        DPRINTK(INV, "dropping, inner_len %d not large enough for a freedom packet\n",inner_len);
        return;
    }

    /*
     * lame hack for our lame crypto
     * that wants a lame length that
     * is a lame multiple of lame
     * 8 bytes.
     */
    if((inner_len - 1) & 7) { /* don't forget key byte */
        DPRINTK(INV, "invalid inner length for crypto: %d\n",inner_len);
        return;
    }

    nei = try_link_decrypt(kaip, iph->saddr, uh->source, 
        (char *)uh + sizeof(struct udphdr), inner_len);

    if(nei == NULL) {
        /* couldn't find a neighbor */
        DPRINTK(INV, "dropping, couldn't link dec to a nei\n");
        return;
    }

    kaip->kstats.freedom_rx_packets++;
    kaip->kstats.freedom_rx_bytes += inner_len;

    fh = (struct freedomhdr *)((char *)uh + sizeof(struct udphdr));

    DEBUGDO(PKT, (hexdump("after link decryption", fh, inner_len)));
 
    if(((fh->vers != 1) && (fh->vers != 2)) || (fh->type != 1)) {
        /*
         * Its a valid packet, but we don't understand it,
         * so we hand it to the daemon to deal with.
         */
        kaip_netlink_handoff(kaip, nei, iph, fh, inner_len);
        goto out_decref;
    }

    telescope_len = inner_len - sizeof(struct freedomhdr) + RAND_PAD_BYTES + 2 - EIGHTS_BYTES - 1;

    /* find the partial route that packet is 
     * a part of */
    read_lock(&(kaip->list_lock));
    pr = aci_lookup(nei,fh->aci);
    if(pr)
        atomic_inc(&(pr->refcount));
    read_unlock(&(kaip->list_lock));
    
    if(pr == NULL) {
        /* we have a valid packet from a neighbor
         * we don't have a route too?  Odd.
         */
        DPRINTK(INV, "dropping, couldn't match aci %d on nei %p\n",fh->aci, nei);
        goto out_decref;
    }

    pr->dc->stats.rx_packets++;
    pr->dc->stats.rx_bytes += skb->len;

    switch(packet_direction(pr, nei)) {

        case DIR_FORWARD:
            
            DEBUGDO(FN, (hexdump("decrypt telescope key", pr->telescope_in_ctx.key, 16)));
            do_cipher(pr->cipher_funcs->decrypt,
                    &(pr->telescope_in_ctx),
                    fh->payload_rand, fh->payload_rand,telescope_len,
                    &(kaip->kstats.telescope));

            DEBUGDO(PKT, (hexdump("pkt after telescope decryption", fh, inner_len)));
 
            if(pr->server_nei == NULL) {

                if(ntohs(fh->payloadlen) > (telescope_len - RAND_PAD_BYTES - 2 - MAC_BYTES))  {
                    DPRINTK(INV, "dropping, fh payloadlen %d bigger than packet\n",
                            ntohs(fh->payloadlen));
                    goto out_decref;
                }

                if(verify_mac(kaip, fh,pr)) {
                    DPRINTK(INV, "dropping, invalid mac\n");
                    goto out_decref;
                }

                /* all right, we're heading for the
                 * outside world */
                leave_cloud(skb, kaip, fh, pr);

                goto out_decref;

            } 

            fh->aci = pr->server_aci;
            next_nei = pr->server_nei;

        break;

        case DIR_REVERSE:

             DEBUGDO(FN, (hexdump("encrypt telescope key", pr->telescope_out_ctx.key, 16)));
             do_cipher(pr->cipher_funcs->encrypt,
                    &(pr->telescope_out_ctx),
                    fh->payload_rand, fh->payload_rand, telescope_len,
                    &(kaip->kstats.telescope));

            fh->aci = pr->client_aci;
            next_nei = pr->client_nei;

        break;

    };

    if(!next_nei) 
        goto out_decref;

    get_random_bytes(fh->rand, RAND_PAD_BYTES);

    /* inner_len includes lekv, fyi */
    do_cipher(pr->cipher_funcs->encrypt,
            &(next_nei->link_to_ctx[next_nei->active_to_key]),
            (u8 *)fh, (u8 *)fh, inner_len - 1,
            &(kaip->kstats.link_enc));

    ((u8 *)fh)[inner_len - 1] = next_nei->active_to_key;

    DEBUGDO(PKT, ( dump_ip("sending next hop", iph)));
    DPRINTK(FN, "send pkt to address: %d.%d.%d.%d \n",
            NIPQUAD(next_nei->addr.sin_addr.s_addr));
    DEBUGDO(PKT, (hexdump("pkt after link encryption", fh, inner_len))); 

    kaip->kstats.freedom_tx_packets++;
    kaip->kstats.freedom_tx_bytes += inner_len;

    pr->dc->stats.tx_packets++;
    pr->dc->stats.tx_bytes += skb->len;

    sock_send_udp(kaip->sock, &next_nei->addr, fh, inner_len);

out_decref:
    if(pr) 
        kaip_decref_pr(pr);
    kaip_decref_nei(nei);
}

static u16 calc_mss(struct kaip_instance *kaip, 
        struct partial_route *pr)
{
    u16 mss;

    mss = pr->dc->dev.mtu;
    if(pr->padded_size) 
        mss = MIN(pr->padded_size - 2, mss);

    return mss - 40;
}

/*
 * code derived from option parsing in the kernel proper.
 *
 * 'space' is the length from the start of the 
 * tcp header of the packet until the end of valid
 * data for the packet.
 *
 * this really just sets the mss, though it used to
 * nuke options..
 */
#define NOP_OPT memset(ptr - 2, TCPOPT_NOP, opsize)
static void cleanse_tcp_options(struct tcphdr *th, u16 mss,
        u16 space)
{
    unsigned char *ptr;
    unsigned int length=(th->doff*4);

    if(length > space)
        return;

    length -= sizeof(struct tcphdr);

    ptr = (unsigned char *)(th + 1);

    while(length > 0) {
        unsigned int opcode=*ptr++;
        unsigned int opsize;

        switch (opcode) {
            case TCPOPT_EOL:
                /* padding is 0s */
                if(length > 1) 
                    memset(ptr, 0, length-1);
                return;
            case TCPOPT_NOP:        
                length--;
                continue;

            default:
                /*
                 * special case a partial grossness, this
                 * makes the rest of this default path
                 * safe
                 */
                if(length == 1) {
                    *(ptr - 1) = TCPOPT_EOL;
                    break;
                }

                opsize = *ptr++;
                if (opsize < 2) {
                    /* silly option whose length doesn't
                     * even include the kind/len bytes.
                     * nop it */
                    memset(ptr - 2, TCPOPT_NOP, 2);
                    length -= 2;
                    break;
                } 

                if (opsize > length) {
                    /* don't parse partial options  */
                    *(ptr - 2) = TCPOPT_EOL;
                    memset(ptr - 1, 0, length-1);
                    return;
                }

                switch(opcode) {
                    case TCPOPT_MSS:
                        if(opsize == TCPOLEN_MSS && th->syn) {
                            if(mss < ntohs(*(__u16 *)ptr))
                                *(__u16 *)ptr = htons(mss);
                        } else 
                            NOP_OPT;
                        break;
#if 0
    /*
     * We used to cleanse the tcp options for the client,
     * but we don't want this complexity.  We just let
     * things through, it'd be up to the clients to cleanse
     * if they decided they didn't want to let some options
     * through.
     */
                    case TCP_OPT_WINDOW:
                        if(opsize != TCPOLEN_WINDOW || !th->syn)
                            NOP_OPT;
                        break;
                    case TCPOPT_TIMESTAMP:
                        if(opsize != TCPOLEN_TIMESTAMP)
                            NOP_OPT;
                        break;
                    case TCPOPT_SACK_PERM:
                        if(opsize != TCPOLEN_SACK_PERM || !th->syn)
                            NOP_OPT;
                        break;
                    case TCPOPT_SACK:
                        if(     (opsize < (TCPOLEN_SACK_BASE + TCPOLEN_SACK_PERBLOCK)) ||
                                th->syn ||
                                ((opsize - TCPOLEN_SACK_BASE) % TCPOLEN_SACK_PERBLOCK) )
                            NOP_OPT;
                        break;
                    default:
                        NOP_OPT;
                        break;
#endif
                    default:
                        break;
                }
                ptr += opsize-2;
                length -= opsize;
                break;
        }
    }
}

static int check_iph_lengths(struct iphdr *iph, u16 max)
{
    return ((ntohs(iph->tot_len) > max) ||
        (ntohs(iph->tot_len) < 20) ||
        ((iph->ihl<<2) > ntohs(iph->tot_len)) ||
        ((iph->ihl<<2) < 20));
}

/*
 * This takes the packet after it has been link decrypted
 * and had the payload verified after the mac decryption.
 *
 * It rewrites the ip header source addressing to
 * fake addresses and returns it to the fw chain,
 * where it will presumably be forwarded and
 * masqueraded.
 *
 * caller is responsible for freeing the packet passed
 * in, and decrefing the pr
 */
static void leave_cloud(struct sk_buff *skb, struct kaip_instance *kaip, 
        struct freedomhdr *fh, struct partial_route *pr)
{
    struct iphdr *iph;
    u16 payloadlen = ntohs(fh->payloadlen) - DORKY_ALIGN_PAD;

    if(payloadlen < sizeof(struct iphdr)) {
        DPRINTK(INV, "dropping, payload len %d doesn't fit ip header\n",
                payloadlen);
        return;
    }

    iph = (struct iphdr *)((char *)fh + sizeof(struct freedomhdr) + DORKY_ALIGN_PAD);
    DEBUGDO(PKT,( dump_ip("inner packet", iph) ));

    if( check_iph_lengths(iph, payloadlen) ) {
        DPRINTK(INV, "dropping, iph lengths inconsistent\n");
        return;
    }
    /*
     * old clients don't send packets with
     * valid csums.  neato.
     */
#if 0
    if( ip_fast_csum((char *)iph, iph->ihl) ) {
        DPRINTK(5, "ip checksum invalid\n");
        return; 
    }
#endif

    kaip->kstats.clear_tx_packets++;
    kaip->kstats.clear_tx_bytes += payloadlen;

    /* 
     * set MSS for outgoing syns from the tunnel.  
     * Dirty as this is, dumb old clients need it
     * to avoid broken pmtu configurations 
     */
    if( ( (ntohs(iph->frag_off) & IP_OFFSET) == 0) && 
            (iph->protocol == IPPROTO_TCP) &&
            (payloadlen - (iph->ihl*4)) >= sizeof(struct tcphdr) &&
            (fh->vers == 1)) {
        struct tcphdr *th;
        th = (struct tcphdr *)((char *)iph + (iph->ihl*4));

        cleanse_tcp_options(th, calc_mss(kaip, pr), payloadlen - (iph->ihl*4));
    }

    skb->masquerade_state = iph->saddr;
    iph->saddr = pr->fake_addr;

    /*
     * have to recalculate the csum 'cause the client
     * helpfully nuked it
     */
    recalculate_full_csum(iph);

    DEBUGDO(PKT, (dump_ip("rewritten", iph)));

    /*
     * XXX This is sub-optimal.  We'd prefer to alter
     * the existing skb so that netif_rx() likes it,
     * but I haven't taken the time to do that yet.
     * Instead we just alloc a new one and pass it
     * to netif_rx().. 
     */
#if 0
    /* we'd want to be doing this if we were fixing up the
     * skb and processing it rather than just copying
     * data out of it into a newly allocated skb..
     */
    skb->nh.iph = iph;
    skb_pull(skb, ((char *)iph - (char *)skb->data));
    skb_trim(skb, payloadlen);
#endif
    {
        struct sk_buff *newskb;
        int len = ntohs(iph->tot_len);

        newskb = alloc_skb(len + 2, GFP_ATOMIC);
        if(!newskb) 
            return;

        skb_reserve(newskb, 2);
        memcpy( skb_put(newskb, len) , iph, len);

        newskb->dev = &(pr->dc->dev);
        newskb->mac.raw = newskb->data;
        newskb->pkt_type = PACKET_HOST;
        newskb->protocol = __constant_htons(ETH_P_IP);
        newskb->masquerade_state = skb->masquerade_state;

        netif_rx(newskb);
    }

    return;
}

/*
 * The caller is assumed to have ->lock()ed the cipher.  We 0 the
 * context if we error, so checking for ctx->keyinfo is the way 
 * to know to free things in error conditions..
 */
static int copy_init_ctx(struct cipher_implementation *cip, struct cipher_context *ctx, 
        unsigned char *data, int len)
{
    int ret = -EINVAL;

    memset(ctx, 0, sizeof(*ctx));

    if((len < 0) || (len > sizeof(ctx->key)))  {
        DPRINTK(FN, "key size %d greater than %d\n",len, sizeof(ctx->key));
        goto out;
    }

    ret = -EFAULT;
    if(copy_from_user(ctx->key, data, len)) 
        goto out;

    ret = -ENOMEM;
    if( (ctx->keyinfo = (u32 *) kmalloc(cip->key_schedule_size, GFP_KERNEL)) == NULL) 
        goto out;

    ctx->ci = cip;
    
    ret = -EINVAL;
    if(cip->set_key(ctx, (unsigned char *)ctx->key, len))
        goto out;

    ret = 0;
    memset(ctx->iv,0,sizeof(ctx->iv));
    
out:
    if(ret) {
        if(ctx->keyinfo) 
            kfree(ctx->keyinfo);
        memset(ctx, 0, sizeof(*ctx));
    }
     
    return ret;
}

/*
 * Dis Gust Ing
 */
static int setup_cipher(struct kaip_neighbor *nei, struct kaip_req_neighbor *req)
{
    struct cipher_implementation *cip;
    int ret = 0;
    int i;

    if(req->active_to_key >= MAX_LINK_KEYS)
        return -EINVAL;

    req->link_cipher_name[MAX_CIPHER_NAME-1] = '\0';
    if( (cip = find_cipher_by_name(req->link_cipher_name)) == NULL)
        return -EINVAL;

    cip->lock();

    for(i = 0; i < MAX_LINK_KEYS; i++) {
        ret = copy_init_ctx(cip, &(nei->link_from_ctx[i]), 
                req->link_from_key_data[i], req->link_from_key_len[i]);
        if(ret) 
            break;
        ret = copy_init_ctx(cip, &(nei->link_to_ctx[i]), 
                req->link_to_key_data[i], req->link_to_key_len[i]);
        if(ret) 
            break;
    }
    if(ret)   {
        cleanup_keyinfo( nei);
        cip->unlock();
        return ret;
    }

    nei->active_to_key = req->active_to_key;
    nei->cipher_funcs = cip;

    return 0;
}
/*
 * This silly function is at least twice as long as it
 * needs to be.  notice the dorky s/_from_/_to_/ :/
 *
 * it should probably only allow updating one key so
 * that the user can know what things suceeded..
 */
static int update_keys(struct kaip_instance *kaip, struct kaip_req_neighbor *req)
{
    struct kaip_neighbor *nei;
    struct cipher_context ctx_tmp;
    int i;
    int ret = 0;

    if(req->active_to_key >= MAX_LINK_KEYS)
        return -EINVAL;

    read_lock(&(kaip->list_lock));
    nei = nei_id_lookup(kaip, req->id);
    if(nei)
        atomic_inc(&(nei->refcount));
    read_unlock(&(kaip->list_lock));
    if(nei == NULL)
        return -ENOENT;

    memset(&ctx_tmp, 0, sizeof(ctx_tmp));

    /*
     * recalc the keys that have data
     */
    for(i = 0; i < MAX_LINK_KEYS; i++) {
        if(req->link_from_key_data[i]) {
            ret = copy_init_ctx(nei->cipher_funcs, &ctx_tmp,
                    req->link_from_key_data[i], req->link_from_key_len[i]);
            if(ret) 
                break;

            if(nei->link_from_ctx[i].keyinfo)
                kfree(nei->link_from_ctx[i].keyinfo);

            memcpy(&(nei->link_from_ctx[i]), &ctx_tmp, sizeof(struct cipher_context));
            memset(&(ctx_tmp), 0, sizeof(ctx_tmp));
        }
        if(req->link_to_key_data[i]) {
            ret = copy_init_ctx(nei->cipher_funcs, &ctx_tmp,
                    req->link_to_key_data[i], req->link_to_key_len[i]);
            if(ret) 
                break;

            if(nei->link_to_ctx[i].keyinfo)
                kfree(nei->link_to_ctx[i].keyinfo);

            memcpy(&(nei->link_to_ctx[i]), &ctx_tmp, sizeof(struct cipher_context));
            memset(&(ctx_tmp), 0, sizeof(ctx_tmp));
        }
    }

    if(ctx_tmp.keyinfo)
        kfree(ctx_tmp.keyinfo);

    kaip_decref_nei(nei);
    return ret;
}

static int kaip_add_neighbor(struct kaip_instance *kaip, struct kaip_req_neighbor *req) 
{
    struct kaip_neighbor *nei;
    int ret = 0;

    nei = kmalloc(sizeof(struct kaip_neighbor), GFP_KERNEL);
    if(!nei) 
        return -ENOMEM;
    memset(nei, 0, sizeof(*nei));

    nei->id = req->id;
    nei->addr = req->addr;
    atomic_set(&(nei->refcount),1);
    atomic_set(&(nei->failed_link_dec),0);
    INIT_LIST_HEAD(&(nei->aci_client_list_head));
    INIT_LIST_HEAD(&(nei->aci_server_list_head));
    nei->live = 1;
    /*
     * hack, we init the items so that list_empty
     * can be used to test if they're on a list
     * or not.
     */
    INIT_LIST_HEAD(&(nei->ip_list));
    INIT_LIST_HEAD(&(nei->ip_port_list));

    if( (ret = setup_cipher(nei, req)))
        goto out;

    write_lock(&(kaip->list_lock));

    if((nei_id_lookup(kaip, nei->id)) != NULL)  {
        ret = -EBUSY;
        goto out_unlock;
    }

    /*
     * record the new neighbor in the 'pending' list
     * so that we can later discover its udp source
     * port
     */
    if( (ret = nei_ip_insert(kaip, nei, nei->addr.sin_addr.s_addr)))
        goto out_unlock;

    if( (ret = nei_id_add(kaip, nei))) {
        nei_ip_remove(kaip, nei);
        goto out_unlock;
    }

out_unlock:
    write_unlock(&(kaip->list_lock));
out:
    if(ret) 
        kfree(nei);

    return ret;
}

static void kaip_decref_nei(struct kaip_neighbor *nei)
{

    atomic_dec(&(nei->refcount));
    if(atomic_read(&(nei->refcount)) > 0)
        return;

    cleanup_keyinfo( nei);

    if(nei->cipher_funcs) nei->cipher_funcs->unlock();

    kfree(nei);
}

/*
 * We have to be careful only to remove the nei from lists
 * that provide others with references.  We have to keep
 * data around that existing references will need.
 */
static int kaip_unlink_nei(struct kaip_instance *kaip, struct kaip_neighbor *nei)
{
    if(!list_empty(&(nei->aci_client_list_head)) || 
                !list_empty(&(nei->aci_server_list_head))) {
        return -EBUSY;
    }

    nei_ip_port_remove(kaip, nei);
    nei_ip_remove(kaip, nei);
    list_del(&(nei->nei_list));
    nei->live = 0;

    kaip_decref_nei(nei);

    return 0;
}

/* 
 * oh boy.. copied from 2.2.18
 * is this realy portable across 2.2 versions?
 */
#ifndef daemonize
void daemonize(void)
{
    struct fs_struct *fs;
    /*
     * If we were started as result of loading a module, close all of the
     * user space pages.  We don't need them, and if we didn't close them
     * they would be locked into memory.
     */

    exit_mm(current);
    current->session = 1;
    current->pgrp = 1;
        
    /* Become as one with the init task */
            
    exit_fs(current);       /* current->fs->count--; */
    fs = init_task.fs;
    current->fs = fs;
    atomic_inc(&fs->count);
}
#endif

void nuke_files(void) 
{
    int i;

    for (i = 0; i < current->files->max_fds; i++ ) {
        if (current->files->fd[i] > 0) 
            sys_close(i);
    }
}

/*
 * copied verbatum from core/datagram.c
 */
static inline void wait_for_packet(struct sock * sk)
{
    struct wait_queue wait = { current, NULL };

    add_wait_queue(sk->sleep, &wait);
    current->state = TASK_INTERRUPTIBLE;

    if (skb_peek(&sk->receive_queue) == NULL)
        schedule();

    current->state = TASK_RUNNING;
    remove_wait_queue(sk->sleep, &wait);
}

static void clear_packet_lists(struct kaip_instance *kaip)
{
    struct list_head *pos;
    struct our_packet *pkt;

    list_for_each(pos, &(kaip->pending_packets)) {
        pkt = list_item(pos, struct our_packet, pending_item);

        pos = pos->prev;
        list_del(&(pkt->pending_item));

        switch(pkt->control) {
            case OP_RCV:
                /* 
                 * XXX we just 'know' that skb_free_datagram
                 * actually doesn't use the first argument so
                 * we can free things without the sock.. eeeep.
                 */
                skb_free_datagram(NULL, pkt->skb);
            break;
            case OP_XMIT:
                dev_kfree_skb(pkt->skb);
            break;
        }
        kfree(pkt);
    }

    kaip->num_pending = 0;

}

/*
 * Lets see if we have some packets waiting
 * in the udp queue, pop them on the pending
 * list if so.
 *
 * called with the list lock held
 */
static int enqueue_udp(struct kaip_instance *kaip, struct socket *sock)
{
    int err = 0;
    struct sk_buff *skb;
    struct our_packet *pkt;

    skb = skb_recv_datagram(sock->sk, 0, 1, &err);
    DPRINTK(FN, "recv skb 0x%p err: %d\n",skb, err);
    switch(err) {
        case 0:
        case -ECONNREFUSED:
        case -EAGAIN:
        case -ETIMEDOUT:
        case -ERESTARTSYS:
        case -ENOBUFS:
            break;

        default:
            printk("kaip enqueue_udp skb_recv_datagram error! %d\n", err);
            return 0;
            break;
    }

    if(!skb)
        return 0;

    pkt = kmalloc(sizeof(*pkt), GFP_ATOMIC);
    if(!pkt) { 
        skb_free_datagram(sock->sk, skb);
        return 0;
    }

    memset(pkt, 0, sizeof(*pkt));

    pkt->control = OP_RCV;
    pkt->skb = skb;
    list_add_tail(&(pkt->pending_item), &(kaip->pending_packets));
    kaip->num_pending++;

    return 0;
}

/*
 * all worker threads run here.
 *
 * packets are fed into the pending packet list from 
 * the kaip device xmit routine and from grabbing packets
 * off of the udp queue.  we sleep on the socket sleep
 * queue, which implies that we can only have one sock
 * per kaip for now.the _xmit wakes the sleep queue, which 
 * is probably racey?
 */
static int kaipd(void *arg)
{
    struct kaip_instance *kaip = arg;
    struct socket *sock = kaip->sock;
    int done = 0;
    struct our_packet *pkt = NULL;
    unsigned long flags;
    struct list_head *pos;
    struct sk_buff *skb;
    cycles_t start;

    MOD_INC_USE_COUNT;

    lock_kernel();

    nuke_files();
    daemonize();
    /*
     * We want to do sendmsg() from 
     * kernel space
     */
    exit_mm(current);
    set_fs(KERNEL_DS);

    unlock_kernel();

    strcpy(current->comm, "kaipd");

    /* flush pending signals  */
    spin_lock_irq(&current->sigmask_lock);
    flush_signals(current);
    recalc_sigpending(current);
    /* block signals */
    sigfillset(&current->blocked);
    spin_unlock_irq(&current->sigmask_lock);

    DPRINTK(FN, "kaipd starting\n");

    atomic_inc(&(kaip->num_threads));

    while(! done) {

        if(kaip->shutdown) 
            break;

        spin_lock_irqsave(&(kaip->pp_lock), flags);


        if(kaip->num_pending < OP_LIMIT) {
            if(enqueue_udp(kaip, sock))
                done = 1 ;
        }

        /*
         * yank the next packet off the list
         */
        pkt = NULL;
        list_for_each(pos, &(kaip->pending_packets)) {
            pkt = list_item(pos, struct our_packet, pending_item);

            pos = pos->prev;
            list_del(&(pkt->pending_item));
            kaip->num_pending--;
            break;
        }

        spin_unlock_irqrestore(&(kaip->pp_lock), flags);

        if(!pkt) {
            wait_for_packet(sock->sk);
            continue;
        }

        skb = pkt->skb;


        /*
         * code paths further down are responsible for cleaning up
         * the skb :(
         */
        start = get_cycles();
        switch(pkt->control) {
            case OP_RCV:
                kaip->kstats.out_path.num++;

                kaip_rcv(skb, kaip, skb->nh.iph, skb->h.uh);
                skb_free_datagram(sock->sk, skb);

                kaip->kstats.out_path.cycles += get_cycles() - start;
                break;
            case OP_XMIT:
                kaip->kstats.in_path.num++;
                kaip_process_ingress(kaip, pkt->dc, skb);
                kaip->kstats.in_path.cycles += get_cycles() - start;
                break;
        }

        kfree(pkt);
        pkt = NULL;
    }

    atomic_dec(&(kaip->num_threads));
    wake_up(&(kaip->shutdown_wq));

    if(pkt)
        kfree(pkt);

    MOD_DEC_USE_COUNT;
    return 0;
}

/*
 * This is the only place to start threads.  It is called from the ioctl
 * path which won't be called until the device is up and has a kaip
 * struct allocated via _open();
 */
extern __inline__ struct socket *socki_lookup(struct inode *inode)
{
    return &inode->u.socket_i;
}
static int kaip_add_listen(struct kaip_instance *kaip, int fd)
{
    int err = 0;
    struct file *filp = NULL;
    struct socket *sock = NULL;

    if(kaip->sock) 
        return -EAGAIN;

    kaip->netlinksock = NULL;
    kaip->sock = NULL;
    kaip->filp = NULL;

    err = sock_create(PF_NETLINK,SOCK_DGRAM,NETLINK_USERSOCK,&(kaip->netlinksock));
    if(err) {
        DPRINTK(FN, "sock_create() failed: %d\n",err);
        goto out;
    }

    lock_kernel();
    filp = fget(fd);
    unlock_kernel();

    if(!filp) {
        err = -EINVAL;
        DPRINTK(FN, "fget() failed\n");
        goto out;
    }

    sock = socki_lookup(filp->f_dentry->d_inode); 

    if(sock == NULL) {
        err = -EINVAL;
        DPRINTK(FN, "socki_lookup() failed\n");
        goto out;
    }

    kaip->sock = sock;
    kaip->filp = filp;

    if (kernel_thread(kaipd, kaip, 0 /* share nothing with parent */) < 0)
        err = -EAGAIN;
out:

    if(err) {
        if(filp)
            fput(filp);
        if(kaip->netlinksock)
            sock_release(kaip->netlinksock);
    }

    return err;
}

static void cleanup_partial_keys(struct partial_route *pr)
{
    if(pr->telescope_in_ctx.keyinfo)  {
        kfree(pr->telescope_in_ctx.keyinfo);
        pr->telescope_in_ctx.keyinfo = NULL;
    }
    if(pr->telescope_out_ctx.keyinfo)  {
        kfree(pr->telescope_out_ctx.keyinfo); 
        pr->telescope_out_ctx.keyinfo = NULL;
    }
    if(pr->hmac_ctx.digest_info)  {
        kfree(pr->hmac_ctx.digest_info);
        pr->hmac_ctx.digest_info = NULL;
    }
    if(pr->cipher_funcs)  {
        pr->cipher_funcs->unlock();
        pr->cipher_funcs = NULL;
    }
    if(pr->hmac_funcs)  {
        pr->hmac_funcs->unlock();
        pr->hmac_funcs = NULL;
    }
}

static int kaip_setup_partial_keys(struct kaip_instance *kaip,
        struct partial_route *pr, struct kaip_req_route *troute)
{
    int ret = 0;

    troute->telescope_cipher_name[MAX_CIPHER_NAME-1] = '\0';
    if( (pr->cipher_funcs = find_cipher_by_name(troute->telescope_cipher_name)) == NULL)  {
        DPRINTK(FN, "coulnd't find telescope cipher %s\n",troute->telescope_cipher_name);
        return -EINVAL;
    }

    pr->cipher_funcs->lock();

    if( (ret = copy_init_ctx(pr->cipher_funcs, &(pr->telescope_in_ctx),
                    troute->telescope_in_key_data, troute->telescope_in_key_len)))
        goto out;
    if( (ret = copy_init_ctx(pr->cipher_funcs, &(pr->telescope_out_ctx),
                    troute->telescope_out_key_data, troute->telescope_out_key_len)))
        goto out;

    /* 
     * exit routes have to worry about
     * validating the mac
     */
    if(troute->exit_route) {
        unsigned char mac_key[MAC_KEYLEN_BYTES];
        struct digest_implementation *di;

        if(troute->mac_key_len != MAC_KEYLEN_BYTES) {
            DPRINTK(FN, "mac key wrong size: %d, should be %d\n",
                    troute->mac_key_len,MAC_KEYLEN_BYTES);
            ret = -EINVAL;
            goto out;
        }

        troute->mac_digest_name[MAX_CIPHER_NAME-1] = '\0';
        if( (di = find_digest_by_name(troute->mac_digest_name)) == NULL) {
            DPRINTK(FN, "coulnd't find mac digest %s\n",troute->mac_digest_name);
            ret = -EINVAL;
            goto out;
        }
        di->lock();

        pr->hmac_funcs = di;
        pr->hmac_ctx.di = di;

        if( (pr->hmac_ctx.digest_info = kmalloc(di->working_size, GFP_KERNEL)) == NULL) {
            ret = -ENOMEM;
            goto out;
        }

        if(copy_from_user(mac_key, troute->mac_key_data, MAC_KEYLEN_BYTES)) {
            ret = -EFAULT;
            goto out;
        }

        /*
         * Magic.  calculate the pad based on the key
         * material.. the digest_implementation api doesn't
         * have a set_key(), so overload the first update..
         */
        di->open(&(pr->hmac_ctx));
        di->update(&(pr->hmac_ctx), mac_key, MAC_KEYLEN_BYTES);
    }

out:
    if(ret)
        cleanup_partial_keys(pr);

    return ret;
}

/*
 * This is a rather nasty function.  First it does the 
 * slow buildup that doesn't require locking, then it tries
 * adds to the list at the very end when everything is ready
 * to go.  
 */
static int kaip_add_partial_route(struct kaip_instance *kaip,
        struct device_container *dc, struct kaip_req_route *troute)
{
    struct kaip_neighbor *client_nei, *server_nei = NULL;
    struct partial_route *pr = NULL;
    int ret = 0;

    /*
     * First allocate and initialize the 
     * new guy
     */
    if((pr = kmalloc(sizeof(struct partial_route), GFP_KERNEL)) == NULL)
        return -ENOMEM;
    memset(pr, 0, sizeof(*pr));

    pr->padded_size = troute->padded_size;
    pr->client_aci = troute->client_aci;
    pr->server_aci = troute->server_aci;
    pr->fake_addr = troute->fake_addr.sin_addr.s_addr;
    pr->dc = dc;
    atomic_set(&(pr->refcount),1);

    if(pr->padded_size &&
           (((pr->padded_size - DORKY_ALIGN_PAD) < 48) ||
            ((pr->padded_size - DORKY_ALIGN_PAD) > pr->dc->dev.mtu))) {
        ret = -EINVAL;
        goto cleanup_exit;
    }

    /*
     * Then do [slow] crypto setup
     */

    if( (ret = kaip_setup_partial_keys(kaip, pr, troute)) )
        goto cleanup_exit;

    /*
     * verify its neighbor info and get
     * refs on them
     */

    write_lock(&(kaip->list_lock));

    client_nei = nei_id_lookup(kaip, troute->client_neighbor_id);
    if(client_nei == NULL) 
        ret = -EINVAL;
    else {
        if(aci_lookup(client_nei, troute->client_aci) != NULL) 
            ret = -EINVAL;
    }
    if( ! troute->exit_route) {
        server_nei = nei_id_lookup(kaip, troute->server_neighbor_id);
        if(server_nei == NULL) 
            ret = -EINVAL;
        else {
            if(aci_lookup(server_nei, troute->server_aci) != NULL) 
                ret = -EINVAL;
        }
    }

    if(ret) {
        DPRINTK(FN, "cant find route neis: cl 0x%08lx 0x%p sv 0x%08lx 0x%p\n",
                (long)troute->client_neighbor_id, client_nei,
                (long)troute->server_neighbor_id, server_nei);
        goto unlock_exit;
    }

    /*
     * ok, its all good.  once its on these lists
     * its live..
     */

    pr->client_nei = client_nei;
    pr->server_nei = server_nei;

    atomic_inc(&(client_nei->refcount));
    list_add(&(pr->aci_client_list_item), &(client_nei->aci_client_list_head));

    kaip_pr_ip_insert(kaip, pr);

    if(pr->server_nei) {
        atomic_inc(&(server_nei->refcount));
        list_add(&(pr->aci_server_list_item), &(server_nei->aci_server_list_head));
    }

    list_add(&(pr->pr_list), &(kaip->pr_list_head));

unlock_exit:
    write_unlock(&(kaip->list_lock));

cleanup_exit:
    if(ret)  {
        cleanup_partial_keys(pr);
        kfree(pr);
    }

    return ret;
}

/*
 * be careful only to remove the client pointer
 * refcounts only when we're really destroying
 * the pr
 */
static void kaip_decref_pr(struct partial_route *pr)
{
    atomic_dec(&(pr->refcount));

    if(atomic_read(&(pr->refcount)) > 0)
        return;

    if(pr->server_nei != NULL)  
        kaip_decref_nei(pr->server_nei);

    kaip_decref_nei(pr->client_nei);

    cleanup_partial_keys(pr);
    kfree(pr);
}

/*
 * Same as unlink_nei, we must be careful what we tear down.
 * so that people with references don't get surprises.
 *
 * called with the lock held.
 */
static int kaip_unlink_pr(struct kaip_instance *kaip, struct partial_route *pr)
{
    if(pr->server_nei != NULL)  
        list_del(&(pr->aci_server_list_item));

    list_del(&(pr->aci_client_list_item));
    list_del(&(pr->pr_list));

    kaip_pr_ip_remove(kaip, pr);

    kaip_decref_pr(pr);

    return 0;
}

/*
 * This tears down the resources on an aip and
 * leaves it in a pristine state, ready to be
 * reconfigured.  It can be called multiple times
 * on a kaip, but isn't reentrant.  We use a semaphore
 * to protect against concurrency.  
 */
static int kaip_shutdown(struct kaip_instance *kaip)
{
    struct list_head *pos;
    int ret = 0;

    down(&(kaip->shutdown_sema));

    /*
     * Make sure our threads are dead 
     */
    if(atomic_read(&(kaip->num_threads)) > 0) {

        kaip->shutdown = 1;
        /*
         * kaip->sock can only exist if threads do,
         * and it won't be freed below until they
         * exit..
         */
        wake_up_interruptible(kaip->sock->sk->sleep);

        if( wait_event_interruptible(kaip->shutdown_wq, 
                    atomic_read(&(kaip->num_threads)) == 0) == -ERESTARTSYS ) {

            ret = -EBUSY;
            goto out;
        }
    }

    kaip->shutdown = 0;

    if(kaip->filp) {
        fput(kaip->filp);
        kaip->filp = NULL;
        kaip->sock = NULL;
        sock_release(kaip->netlinksock);
        clear_packet_lists(kaip);
    }

    /*
     * even though threads are dead,
     * we should protect against other
     * ioctl()s..
     */
    write_lock(&(kaip->list_lock));

    /*
     * Ordering of removal is important..
     */
    list_for_each(pos, &(kaip->pr_list_head)) {
        struct partial_route *pr = list_item(pos, struct partial_route, pr_list);
        pos = pos->prev;
        kaip_unlink_pr(kaip, pr);
    }
    list_for_each(pos, &(kaip->nei_list_head)) {
        struct kaip_neighbor *nei = list_item(pos, struct kaip_neighbor, nei_list);
        pos = pos->prev;
        kaip_unlink_nei(kaip, nei);
    }

    write_unlock(&(kaip->list_lock));

out:
    up(&(kaip->shutdown_sema));
    return ret;
}

/*
 * called with the lock held and doesn't return the
 * neis, so don't worry about refcounts on them.
 */
struct partial_route * kaip_pr_lookup(struct kaip_instance *kaip,
        struct kaip_req_route *troute)
{
    struct kaip_neighbor *nei;
    struct partial_route *prc, *prs, *ret = NULL;

    if(((nei = nei_id_lookup(kaip, troute->client_neighbor_id)) == NULL)  ||
        ((prc = aci_lookup(nei, troute->client_aci)) == NULL)) {
            goto out;
    } 

    /*
     * the incoming neighbor/aci is all we should need,
     * the rest is paranoia..
     */
    if( troute->exit_route ) {
        if(prc->server_nei != NULL)
            goto out;
        ret = prc;
    } else {
        if(((nei = nei_id_lookup(kaip, troute->server_neighbor_id)) == NULL)  ||
            ((prs = aci_lookup(nei, troute->server_aci)) == NULL)) {
                goto out;
        } 
        if (prs == prc) 
            ret = prs;
    }


out:
    return ret;
}

/*
 * all paths through this can be concurrent, but
 * I bet 2.2. doesn't allow that.
 */
static int kaip_ioctl(struct device *dev,  struct ifreq *ifr, int cmd)
{
    struct device_container *dc = dev->priv;
    struct kaip_instance *kaip = dc->kaip;
    struct kaip_request req_copy, *ureq, *req;
    struct kaip_stats kstats_copy;
    struct partial_route *pr;
    struct kaip_neighbor *nei;
    int ret = 0;


    MOD_INC_USE_COUNT; /* lame hack */

    if (copy_from_user(&(req_copy), ifr->ifr_data, sizeof(struct kaip_request))) {
        ret = -EFAULT;
        goto out;
    }

    req = &req_copy;
    ureq = (struct kaip_request *)ifr->ifr_data;

    DPRINTK(FN, "kaip_ioctl req: 0x%p cmd: %d\n",
            req, req->cmd);

    if (!capable(CAP_NET_ADMIN))  {
        ret = -EPERM;
        goto out;
    }

    switch (req->cmd) {
        case KAIP_ADDNEIGHBOR:
            ret = kaip_add_neighbor(kaip, &(req->u.tnei));
        break;

        case KAIP_REMOVENEIGHBOR:
            write_lock(&(kaip->list_lock));
            nei = nei_id_lookup(kaip, req->u.tnei.id);
            if(nei)
                ret = kaip_unlink_nei(kaip, nei);
            else 
                ret = -ENOENT;
            write_unlock(&(kaip->list_lock));
        break;

        case KAIP_UPDATENEIBORKEYS:
            ret = update_keys(kaip, &(req->u.tnei));
        break;

        case KAIP_ADDROUTE:
            ret = kaip_add_partial_route(kaip, dc, &(req->u.troute));
        break;

        case KAIP_REMOVEROUTE:
            write_lock(&(kaip->list_lock));
            if( (pr = kaip_pr_lookup(kaip, &(req->u.troute))) == NULL)
                ret = -EINVAL;
            else
                ret = kaip_unlink_pr(kaip, pr);
            write_unlock(&(kaip->list_lock));
        break;

        case KAIP_UDPBIND:
            ret = kaip_add_listen(kaip, req->u.fd);
        break;

        case KAIP_SETNETLINKADDR:
            write_lock(&(kaip->list_lock));
            memcpy(&(kaip->nladdr), &(req->u.nladdr), sizeof(kaip->nladdr));
            write_unlock(&(kaip->list_lock));
            ret = 0;
        break;

        case KAIP_SHUTDOWN:
            ret = kaip_shutdown(kaip);
        break;

        case KAIP_SETSTATS:
            write_lock(&(kaip->list_lock));
            memcpy(&(kaip->kstats), &(req->u.tstats), sizeof(struct kaip_stats));
            write_unlock(&(kaip->list_lock));
        break;

        case KAIP_GETSTATS:
            read_lock(&(kaip->list_lock));
            memcpy(&(kstats_copy),&(kaip->kstats), sizeof(struct kaip_stats));
            read_unlock(&(kaip->list_lock));
            if (copy_to_user(&(ureq->u.tstats), &(kstats_copy), sizeof(struct kaip_stats))) {
                ret = -EFAULT;
                goto out;
            }
        break;

        default:
            ret = -EINVAL;
        break;
    }

out:
    MOD_DEC_USE_COUNT;
    return ret;
}

/* 
 * stub that just queues the packet for 
 * worker threads
 */
static int kaip_dev_start_xmit(struct sk_buff *skb, struct device *dev)
{
    struct device_container *dc = dev->priv;
    struct kaip_instance *kaip = dc->kaip;
    unsigned long flags;
    struct our_packet *pkt;

    if(kaip == NULL || kaip->sock == NULL)
        return 0;

    DPRINTK(FN, "_xmit got a packet, throwing it on the lists\n");

    pkt = kmalloc(sizeof(*pkt), GFP_ATOMIC);
    if(!pkt) {
        dev_kfree_skb(skb);
        return 0;
    }

    pkt->skb = skb;
    pkt->control = OP_XMIT;
    pkt->dc = dc;

    spin_lock_irqsave(&(kaip->pp_lock), flags);

    if(kaip->num_pending < OP_LIMIT) {
        list_add_tail(&(pkt->pending_item), &(kaip->pending_packets));
        kaip->num_pending++;
        pkt = NULL;
    }

    spin_unlock_irqrestore(&(kaip->pp_lock), flags);

    /* failed, free */
    if(pkt) {
        dev_kfree_skb(pkt->skb);
        kfree(pkt);
    }

    /* 
     * wake-all is a non-problem.  We're only able
     * to wake all the threads if they're waiting.
     * They'll only be waiting if they're idle, in which
     * case we have cpu to burn.  Under heavy load they'll
     * all be off processing things so it doesn't matter
     * if we try to wake the entire queue, it'll be mostly
     * if not entirely empty.  It is probably racey with
     * regard to checking it and going to sleep, though..
     */
    wake_up_interruptible(kaip->sock->sk->sleep);

    return 0;
}

/*
 * The return path from the server to the clients.  Packets
 * arriving here have already passed through the masq layer
 * and been routed to our device which catches the fake address
 * ranges.  Being routed to the device should have fragmented
 * large packets down into ones that can fit in the tunnel, so
 * we don't have to do length checking here.  
 *
 * like _rcv, this was originally written to process skbs.
 */
static int kaip_process_ingress(struct kaip_instance *kaip, 
        struct device_container *dc, struct sk_buff *skb)
{
    struct net_device_stats *stats = &(dc->stats);
    struct freedomhdr *fh;
    struct iphdr *tiph, *iph_icmp = NULL;
    struct udphdr *uh;
    struct tcphdr *th;
    struct icmphdr *ih;
    struct partial_route *pr = NULL;
    unsigned int max_headroom, tailroom, 
        payload_len, payload_pad;
    unsigned char *tailstuff;

    u16 dorky_payload_size;

    DPRINTK(FN, "process_ingress got: skb len: %d proto: %d ms: 0x%x\n",
            skb->len, skb->protocol, skb->masquerade_state);

    if (skb->protocol != __constant_htons(ETH_P_IP))  {
        stats->tx_errors++;
        DPRINTK(INV, "proto %d isn't ETH_IP)IP ?\n", skb->protocol);
        goto out_free;
    } 

    tiph = skb->nh.iph;

    DEBUGDO(PKT, (dump_ip("process_ingress got packet",tiph)));

    if(check_iph_lengths(tiph, skb->len)) {
        DPRINTK(INV, "ip headers aren't kosher\n");
        goto out_free;
    }

    kaip->kstats.clear_rx_packets++;
    kaip->kstats.clear_rx_bytes += skb->len;

    if( (pr = kaip_pr_ip_lookup(kaip, tiph->daddr)) == NULL) {
        stats->tx_errors++;
        goto out_free;
    }

    if((ntohs(tiph->frag_off) & IP_OFFSET) == 0) {
        /* 
         * we're the first fragment, mess around with 
         * protocol headers 
         */

        u16 space = ntohs(tiph->tot_len) - (tiph->ihl*4);

        switch(tiph->protocol) {
            case IPPROTO_TCP: 
                if(sizeof(struct tcphdr) >= space) 
                    break;
                th = (struct tcphdr *)((char *)tiph + (tiph->ihl*4));

                /*
                 * Naughty, naughty.  but some of our software
                 * wants this to happen for now.  ahh, inertia.
                 */
                if (pr->padded_size) {
                    cleanse_tcp_options(th, calc_mss(kaip, pr), space);
		}
                /* gross, obviously needs to go. */
                recalculate_full_csum(tiph);

                break;
            case IPPROTO_UDP: 
                if(sizeof(struct udphdr) >= space) 
                    break;
                uh = (struct udphdr *)((char *)tiph + (tiph->ihl*4));

                csum_inc_u32(&(uh->check), tiph->daddr, skb->masquerade_state);

                break;
            case IPPROTO_ICMP: 
                if((sizeof(struct icmphdr)+sizeof(struct iphdr)) >= space) 
                    break;
                ih = (struct icmphdr *)((char *)tiph + (tiph->ihl*4));
                iph_icmp = (struct iphdr *)((char *)tiph + (tiph->ihl*4) + sizeof(struct icmphdr));
 
                iph_icmp->saddr = skb->masquerade_state;
                ih->checksum = 0;
                ih->checksum = ip_compute_csum((unsigned char *)ih, space);

                break;
            default:
                break;
        }
    }

    /*
     * calculate nutty 8byte + 4 alignment
     * padding for the ciphers..
     */
    payload_len = DORKY_ALIGN_PAD + ntohs(tiph->tot_len);
    payload_pad = ((((payload_len - 4) + 7) & ~7) +4) - payload_len;

    payload_len += payload_pad;

    if(payload_len < pr->padded_size) {
        payload_pad += pr->padded_size - payload_len;
        payload_len += pr->padded_size - payload_len;
    }

    tiph->daddr = skb->masquerade_state;

    tiph->check = 0;
    tiph->check = ip_fast_csum((unsigned char *)tiph, tiph->ihl);

    DEBUGDO(PKT, (dump_ip("tx rewrote",tiph)));

    max_headroom = ((sizeof(struct freedomhdr) + DORKY_ALIGN_PAD + 15)&~15);

    tailroom = payload_pad + MAC_BYTES + EIGHTS_BYTES + 1;
        
    /* 
     * see if we have room for tunnel headers in this skb, 
     * if not generate a new one.
     */
    DPRINTK(FN, "want %d headroom, have %d. want %d tailroom, have %d\n",
            max_headroom, skb_headroom(skb),
            tailroom, skb_tailroom(skb));

    if (skb_headroom(skb) < max_headroom || skb_tailroom(skb) < tailroom ||
            skb_cloned(skb) || skb_shared(skb)) {

        struct sk_buff *new_skb = alloc_skb(max_headroom + ntohs(tiph->tot_len) + tailroom, 
                GFP_ATOMIC);
        if (!new_skb) {
            stats->tx_dropped++;
            goto out_decref;
        }
        { /* nuts, no skb_realloc_tailroom */
            unsigned long offset;
            skb_reserve(new_skb, max_headroom);
            skb_put(new_skb, skb->len);
            offset = new_skb->data - skb->data;

            memcpy(new_skb->data, skb->data, skb->len);
            
            new_skb->priority = skb->priority;
            new_skb->protocol = skb->protocol;
            new_skb->dev = skb->dev;
            new_skb->dst = dst_clone(skb->dst); 

            memcpy(new_skb->cb, skb->cb, sizeof(skb->cb));
            new_skb->h.raw      =   skb->h.raw + offset;
            new_skb->nh.raw     =   skb->nh.raw + offset;
            new_skb->mac.raw    =   skb->mac.raw + offset;
            new_skb->used       =   skb->used;
            new_skb->pkt_type   =   skb->pkt_type;
            new_skb->stamp      =   skb->stamp;
            new_skb->security   =   skb->security;
#ifdef CONFIG_IP_FIREWALL
            new_skb->fwmark     =   skb->fwmark;
#endif
        }

        if (skb->sk)
            skb_set_owner_w(new_skb, skb->sk);
        dev_kfree_skb(skb);
        skb = new_skb;
        DPRINTK(FN, "created a new skb\n");
    }
    tiph = NULL; /* paranoia */

    skb_push(skb, sizeof(struct freedomhdr) + DORKY_ALIGN_PAD);

#if 0
    /* nuke options? */
    memset(&(IPCB(skb)->opt), 0, sizeof(IPCB(skb)->opt));
#endif
 
    tailstuff = skb_put(skb, tailroom);

    fh                      =       (struct freedomhdr  *)skb->data;
    fh->aci                 =       pr->client_aci;
    fh->vers                =       (pr->padded_size ? 1 : 2); /* XXX */
    fh->type                =       1;  /* data */
    fh->priority            =       1; /* ignored for now.. */
    fh->payloadlen          =       htons(payload_len);

    if (fh->vers == 1) {
       memset(fh->pad, 8 - sizeof(fh->pad)%8, sizeof(fh->pad));
    }
    else {
      /* for v2, priority field is part of the padding? */
       memset(&fh->priority, 8 - (sizeof(fh->pad)+sizeof(fh->priority))%8, 
                                 sizeof(fh->pad)+sizeof(fh->priority));
    }
    get_random_bytes(fh->rand, RAND_PAD_BYTES);
    get_random_bytes(fh->payload_rand, RAND_PAD_BYTES);

    /* 
     * DORKY payload size is the first two bytes of the payload 
     * must be set to the length of the payload (not including itself) 
     */
    dorky_payload_size = htons(payload_len - DORKY_ALIGN_PAD);
    memcpy( fh+1, &dorky_payload_size, DORKY_ALIGN_PAD);    

    if(payload_pad) {
        memset(tailstuff, 8 - (payload_len & 0x7), payload_pad);
        tailstuff += payload_pad;
    }
    
    calculate_mac(kaip, pr->hmac_funcs, 
            &(pr->hmac_ctx),
            fh->payload_rand, payload_len + RAND_PAD_BYTES + 2,
            tailstuff, MAC_BYTES);
    
    tailstuff += MAC_BYTES;
    memcpy(tailstuff, eights, EIGHTS_BYTES);
    tailstuff += EIGHTS_BYTES;
    *(tailstuff++) = pr->client_nei->active_to_key;

    DPRINTK(FN, "sending %d payload to aci %d on nei 0x%p\n",
            payload_len, ntohs(fh->aci), 
            pr->client_nei);

    DEBUGDO(PKT, (hexdump("pkt before link encryption", fh, (tailstuff-(unsigned char*)fh)-1)));
     /*
     * we're done with the tunneled packet, encrypt it.
     */
    DEBUGDO(FN, (hexdump("encrypt telescope key", pr->telescope_out_ctx.key, 16)));
    do_cipher(pr->cipher_funcs->encrypt,
            &(pr->telescope_out_ctx),
            (u8 *)fh->payload_rand, (u8 *)fh->payload_rand,
            ((tailstuff - fh->payload_rand) - EIGHTS_BYTES - 1),
            &(kaip->kstats.telescope));

    do_cipher(pr->cipher_funcs->encrypt,
            &(pr->client_nei->link_to_ctx[pr->client_nei->active_to_key]),
            (u8 *)fh,  (u8 *)fh,
            ((tailstuff - (unsigned char *)fh) - 1),
            &(kaip->kstats.link_enc));

    stats->tx_bytes += skb->len;
    stats->tx_packets++;
    kaip->kstats.freedom_tx_bytes += tailstuff - (unsigned char *)fh;
    kaip->kstats.freedom_tx_packets++;

    DPRINTK(FN, "send pkt to address: %d.%d.%d.%d \n",
            NIPQUAD(pr->client_nei->addr.sin_addr.s_addr));
 
    sock_send_udp(kaip->sock, &(pr->client_nei->addr), 
            fh, (tailstuff - (unsigned char *)fh));

out_decref:
    if(pr)
        kaip_decref_pr(pr);

out_free:
    dev_kfree_skb(skb);
    return 0;
}

/*
 * build us a new kaip.  This only happens
 * at device initializaiton via probe
 */
struct kaip_instance * kaip_new(void)
{
    int i;
    struct kaip_instance  *kaip;

    kaip = kmalloc(sizeof (struct kaip_instance), GFP_KERNEL);
    if(kaip == NULL) 
        return NULL;

    memset(kaip, 0, sizeof(*kaip));

    if( (kaip->ip_hash = kmalloc(sizeof(struct list_head)*IP_HASH_ENTRIES, GFP_KERNEL)) == NULL) {
        kfree(kaip);
        return NULL;
    }
    if( (kaip->ip_port_hash = kmalloc(sizeof(struct list_head)*IP_HASH_ENTRIES, GFP_KERNEL)) == NULL) {
        kfree(kaip->ip_hash);
        kfree(kaip);
        return NULL;
    }
    if( (kaip->pr_ip_hash = kmalloc(sizeof(struct list_head)*IP_HASH_ENTRIES, GFP_KERNEL)) == NULL) {
        kfree(kaip->ip_port_hash);
        kfree(kaip->ip_hash);
        kfree(kaip);
        return NULL;
    }
    for(i = 0; i < IP_HASH_ENTRIES; i++) {
        INIT_LIST_HEAD(&(kaip->ip_hash[i]));
        INIT_LIST_HEAD(&(kaip->ip_port_hash[i]));
        INIT_LIST_HEAD(&(kaip->pr_ip_hash[i]));
    }

    INIT_LIST_HEAD(&(kaip->pr_list_head));
    INIT_LIST_HEAD(&(kaip->nei_list_head));
    INIT_LIST_HEAD(&(kaip->dc_list));

    INIT_LIST_HEAD(&(kaip->pending_packets));
    spin_lock_init(&(kaip->pp_lock));
    kaip->list_lock = RW_LOCK_UNLOCKED;
    init_waitqueue(&(kaip->shutdown_wq));
    kaip->shutdown_sema = MUTEX;
    atomic_set(&(kaip->num_threads), 0);

    kaip->max_nei_disc = 100;
    kaip->max_nei_attempts = 100;

    return kaip;
}

#ifdef CONFIG_PROC_FS
/*
 * this function is as crazy as the proc filling api
 * requires.
 *
 * buffer: page allocated for us to scribble in.  the
 *  data returned to the user will be taken from here.
 * *start: address of the pointer that will tell the 
 *  caller where in buffer the data the user wants is.
 * ppos: offset in the entire /proc file that the user
 *  currently wants.
 * wanted: the amount of data the user wants.
 *
 * while going, 'curpos' is the offset in the entire
 * file where we currently are.  We only actually
 * start filling buffer when we get to a place in
 * the file that the user cares about.
 *
 * we take care to only sprintf when the user cares because
 * we're holding a lock while we do this.
 *
 * we're smart and know that we generate fixed size lines.
 * we only start writing to the buffer when the user cares.
 * This is unpredictable because we don't snapshot the
 * list between calls that are filling in a file from
 * the list.  The list could change mid read and the
 * output will look very weird indeed.  oh well.
 */

static int kaip_proc_doit(char *buffer, char **start, 
        off_t ppos, int wanted, int *eof, void *data)
{
    struct proc_guy *pg = data;
    struct list_head *pos;
    unsigned int len = 0;
    int curpos;

    if(pg == NULL || pg->kaip == NULL || 
            pg->list == NULL || pg->printer == NULL ||
            wanted <= 0)
        return 0;

    *start = buffer;

    if(ppos < pg->header_len) {
        unsigned int diff = MIN(pg->header_len, wanted);
        memcpy(buffer, pg->header + ppos, diff);
        len += diff;
        ppos += diff;
    }

    if(len >= wanted)
        goto out;

    curpos = pg->header_len;

    read_lock(&(pg->kaip->list_lock));

    list_for_each(pos, pg->list) {
        int copied;
        /* 
         * just walk the list until we get to the 
         * region the caller wants
         */
        if(ppos >= (curpos + pg->line_len))  {
            curpos += pg->line_len;
            continue;
        }
         /* the clever caller split a line */
        if(ppos > curpos) {
            *start = buffer + (ppos - curpos);
        }

        copied = pg->printer(buffer + len, pos);

        len += copied;

        /* pad to line len, -1 for \n */
        if((copied < pg->line_len-1)) {
            int diff = (pg->line_len-1) - copied;
            memset(buffer + len, ' ', diff);
            len += diff;
            copied += diff;
        }

        buffer[len++]= '\n';

        /* bail if we have enough */
        if(((buffer + len) - *start) >= wanted) 
            break;

        curpos += pg->line_len;
    }

    read_unlock(&(pg->kaip->list_lock));

    /* lameness */
    if(pos == pg->list)
        *eof = 1;
out:

    return MIN(((buffer + len) - *start), wanted);
}

#define PG_TEMPLATE(FOO, LINE_LEN, HEADER)       \
    struct proc_guy pg_tmpl_##FOO = {            \
        line_len:   (LINE_LEN),                  \
        header:     HEADER,                      \
        printer:    pg_print_##FOO               \
    }

static int pg_print_nei(char *buf, struct list_head *pos)
{
    struct kaip_neighbor *nei = list_item(pos, struct kaip_neighbor, nei_list);

    return sprintf(buf, "0x%08x %5d %7d %d.%d.%d.%d:%d", 
            nei->id, 
            atomic_read(&(nei->refcount)), 
            atomic_read(&(nei->failed_link_dec)), 
            NIPQUAD(nei->addr.sin_addr.s_addr),
            ntohs(nei->addr.sin_port));
}
PG_TEMPLATE( nei, (10+1 + 6 + 7 + 4+4+4+4+1+5 +1),
       "    nei id rfcnt  ldfail  udp source\n");

#define PRID(nei) ( nei ? nei->id : 0 )
static int pg_print_pr(char *buf, struct list_head *pos)
{
    struct partial_route *pr = list_item(pos, struct partial_route, pr_list);

    return sprintf(buf, "0x%08x 0x%04x 0x%08x 0x%04x %5d %5d %d.%d.%d.%d",
            PRID(pr->client_nei),
            ntohs(pr->client_aci), 
            PRID(pr->server_nei),
            ntohs(pr->server_aci),
            atomic_read(&(pr->refcount)),
            pr->padded_size,
            NIPQUAD(pr->fake_addr));
}
#undef PRID
PG_TEMPLATE( pr, (11 + 7 + 11 + 7 + 6 + 6 + 4+4+4+4+1+5 +1),
       " client id cl aci  server id sv aci rfcnt psize fk addr\n");

static unsigned char basedir[]="net/kaip";

unsigned int kaip_insert_proc(struct kaip_instance *kaip)
{
    unsigned char dir[128];
    struct proc_dir_entry *ent;
    int end;

    dir[0]= '\0';

    /*
     * This is pretty lame.  assuming that failure just
     * means that they already existed.
     */
    strcat(dir, basedir);
    create_proc_entry(dir, S_IFDIR|S_IXUSR|S_IRUSR, 0);

    end = strlen(dir);

    strcat(dir, "/neighbors");
    ent = create_proc_entry(dir, 0, 0);
    if(!ent) {
        DPRINTK(FN, "nei create_proc_entry died\n");
        return 1;
    }

    memcpy(&(kaip->pg_nei), &pg_tmpl_nei, sizeof(struct proc_guy));
    kaip->pg_nei.kaip = kaip;
    kaip->pg_nei.header_len = strlen(kaip->pg_nei.header);
    kaip->pg_nei.list = &(kaip->nei_list_head);

    ent->data = &(kaip->pg_nei);
    ent->read_proc = kaip_proc_doit;

    dir[end] = '\0';
    strcat(dir, "/partial_routes");
    ent = create_proc_entry(dir, 0, 0);
    if(!ent) {
        dir[end] = '\0';
        strcat(dir, "/neighbors");
        remove_proc_entry(dir,0);
        DPRINTK(FN, "pr create_proc_entry died\n");
        return 1;
    }

    memcpy(&(kaip->pg_pr), &pg_tmpl_pr, sizeof(struct proc_guy));
    kaip->pg_pr.kaip = kaip;
    kaip->pg_pr.header_len = strlen(kaip->pg_pr.header);
    kaip->pg_pr.list = &(kaip->pr_list_head);

    ent->data = &(kaip->pg_pr);
    ent->read_proc = kaip_proc_doit;

    dir[end] = '\0';
    
    return 0;
}

void kaip_remove_proc(struct kaip_instance *kaip)
{
    unsigned char dir[128];
    int end;

    dir[0]='\0';
    strcat(dir, basedir);

    end = strlen(dir);

    strcat(dir, "/neighbors");
    remove_proc_entry(dir,0);

    dir[end] = '\0';
    strcat(dir, "/partial_routes");
    remove_proc_entry(dir,0);

    dir[end] = '\0';
    remove_proc_entry(dir,0);
}

void kaip_nuke_proc(void)
{
    remove_proc_entry(basedir,0);
}

#else
#define kaip_insert_proc(a...) ({int _ret = 0; _ret;})
#define kaip_remove_proc(a...) ({int _ret = 0; _ret;})
#define kaip_nuke_proc(a...)   ({int _ret = 0; _ret;})
#endif

#define CRAP {\
/*    printk("uh oh.  " __FUNCTION__ "\n");*/ \
    return 0; \
}

static int kaip_dev_hard_header(struct sk_buff *skb, struct device *dev, 
                unsigned short type, void *daddr, void *saddr, unsigned len) {
    DPRINTK(FN, "kaip_dev_hard_header : s: 0x%p d: 0x%p len: %d type: %d\n",
            saddr, daddr, len, type);
    return 0;
}

static int kaip_dev_rebuild_header(struct sk_buff *skb)
    CRAP

static int kaip_dev_neigh_setup(struct device * dev, struct neigh_parms *p)
    CRAP

static struct net_device_stats *kaip_dev_get_stats(struct device *dev)
{
    struct device_container *dc = dev->priv;

    return &(dc->stats);
}

static int kaip_dev_close(struct device  *dev)
{
    MOD_DEC_USE_COUNT;

    return 0;
}
static int kaip_dev_open(struct device  *dev)
{
    MOD_INC_USE_COUNT;
    return 0;
}

void kaip_free_kaip(struct kaip_instance *kaip)
{
    struct list_head *pos;

    list_for_each(pos, &(kaip->dc_list)) {
        struct device_container *dc = list_item(pos, struct device_container, dc_list_item);

        unregister_netdev(&(dc->dev));
        kfree(dc);
    }

    kfree(kaip->ip_port_hash);
    kfree(kaip->ip_hash);
    kfree(kaip->pr_ip_hash);
    kfree(kaip);
}

__initfunc(int kaip_dev_probe(struct device *dev))
{
    DPRINTK(FN, "entering kaip_dev_probe for dev: 0x%p named: %s\n",
            dev, dev->name);

    dev->open               = kaip_dev_open;
    dev->stop               = kaip_dev_close;
    dev->hard_start_xmit    = kaip_dev_start_xmit;
    dev->set_multicast_list = NULL;
    dev->do_ioctl           = kaip_ioctl;

    /* init packet queues */
    dev_init_buffers(dev);

    /* Hmm. */
    dev->hard_header        = kaip_dev_hard_header;
    dev->rebuild_header     = kaip_dev_rebuild_header;
    dev->neigh_setup        = kaip_dev_neigh_setup;
    dev->get_stats          = kaip_dev_get_stats;

    dev->hard_header_len    = ((sizeof(struct freedomhdr)) + 15 ) & ~15;

    /* XXX This is wrong, and confuses tcpdump */
    dev->type               = ARPHRD_ETHER; /* initially */
    dev->set_mac_address    = NULL;
    dev->mtu                = 1500;

    dev->addr_len           = 0;
    dev->tx_queue_len       = 10;
    dev->flags              = 0;

    return 0;
}

#define TUNNEL_BANNER   "Kernel Anonymous IP Accelerator\n"

MODULE_PARM(kaip_debug, "i");
MODULE_PARM(num_devs, "i");

int init_module(void)
{
    int ret = 0;
    struct kaip_instance *kaip;

    kaip = kaip_new();
    if(!kaip) 
        return -ENOMEM;
    kaip->next = global_kaip_list;
    global_kaip_list = kaip;

    while(num_devs--) {
        struct device_container *dc;

        if( (dc = kmalloc(sizeof(struct device_container), GFP_KERNEL)) == NULL) {
            ret = -ENOMEM;
            break;
        }

        memset(dc, 0, sizeof(struct device_container));

        dc->dev.name = dc->name;
        dc->kaip = kaip;
        dc->dev.priv = dc;
        dc->dev.init = kaip_dev_probe;

        if (dev_alloc_name(&(dc->dev),"kaip%d") < 0) {
            ret = -ENFILE;
            kfree(dc);
            break;
        }

        if (register_netdev(&(dc->dev)) != 0) {
            kfree(dc);
            ret = -EIO;
            break;
        }

        list_add(&(dc->dc_list_item), &(kaip->dc_list));
    }

    if(!ret && kaip_insert_proc(kaip))  
        ret = -EAGAIN;

    if(ret) {
        kaip_free_kaip(kaip);
        return ret;
    }

    printk(TUNNEL_BANNER);  
    printk("KAIP initialised.\n");

    return 0;
}

void cleanup_module(void)
{
    struct kaip_instance *kaip;

    while(global_kaip_list) {
        kaip = global_kaip_list;
        global_kaip_list = kaip->next;

        /*
         * Do this for good measure to clean up resources.
         * The scary part of it blocking waiting for threads
         * to shutdown won't happen as the threads inc the mod
         * count while they're alive.
         */
        kaip_shutdown(kaip);

        kaip_remove_proc(kaip);
        kaip_free_kaip(kaip);

    }

    kaip_nuke_proc();
}
