#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "aip-crypto.h"

int urandom;
#define DEV_URANDOM "/dev/urandom"

#if !defined( __KERNEL__ )

int aip_crypto_init( void )
{
  size_t r;
  char buf;

  urandom = open( DEV_URANDOM, O_RDONLY );

  if ( urandom < 0 ) {
    return 0;
  }

  r = read( urandom, &buf, 1 );

  if ( r < 1 ) {
    return 0;
  }

  return 1;
}

int aip_crypto_random( void* buf, size_t len )
{
  size_t r;

  r = read( urandom, buf, len );

  if ( r < len ) {
    return 0;
  }
  return 1;
}

void aip_crypto_close( void )
{
  close( urandom );
}

#endif

void aip_crypto_setup( AIP_CRYPTO_CTX* ctx, byte key[ KEY_SIZE ] )
{
  BF_set_key( &ctx->key_schedule, KEY_SIZE, key );
  ctx->num = 0;

}

void aip_crypto_setup_iv( AIP_CRYPTO_CTX* ctx, byte iv[ IV_SIZE ] )
{
  memcpy( ctx->iv, iv, IV_SIZE );
  ctx->num = 0;
}

void aip_crypto_encrypt( AIP_CRYPTO_CTX* ctx, void* data, size_t len )
{
#if 0
  BF_cfb64_encrypt( data, data, len, 
		    &ctx->key_schedule, ctx->iv, &ctx->num, BF_ENCRYPT );
#else 
  BF_cbc_encrypt( data, data, len, 
		    &ctx->key_schedule, ctx->iv, BF_ENCRYPT );
#endif
}

void aip_crypto_decrypt( AIP_CRYPTO_CTX* ctx, void* data, size_t len )
{
#if 0
  BF_cfb64_encrypt( data, data, len, 
		    &ctx->key_schedule, ctx->iv, &ctx->num, BF_DECRYPT );
#else
  BF_cbc_encrypt( data, data, len, 
		    &ctx->key_schedule, ctx->iv, BF_DECRYPT );
#endif
}
