#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <arpa/inet.h>

//#include "libnet/libnet_test.h"
#include "libnet.h"
#include "aip-crypto.h"

#include <sys/socket.h>
#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>     /* the L2 protocols */
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>   /* The L2 protocols */
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

#include <sys/ioctl.h> 
#include <net/if.h>

/*
 * Bad unportable juju.
 */
#include <linux/types.h>

int do_hexdump = 0;

static void hexdump(const char *string, const void *ptr, int len)
{
    const unsigned char *c = ptr;
    int i;

    if(!do_hexdump)
        return;

    printf("%d bytes of '%s' from 0x%p\n",
            len, string, ptr);

    for(i = 0; i < len;) {
        printf("%02x",*(c++));
        i++;
        if( ! (i & 15)) {
            printf("\n");
        } else if( ! (i&1)) {
            printf(" ");
        }
    }
    if(len & 15) {
        printf("\n");
    }
}

#define RAND_PAD_BYTES 8
#define BUFFER_LEN 65536 /* give us lots of room */

struct freedomhdr {
    uint8_t rand[RAND_PAD_BYTES];
    uint8_t vers;
    uint8_t type;
    uint8_t priority;
    uint8_t pad[3];
    uint16_t aci; /* Anonymous Circuit Identifier */
    uint8_t payload_rand[RAND_PAD_BYTES];
    uint16_t payloadlen;
};


#define MAX_PAYLOAD_LEN 252
#define MAC_BYTES 10
#define EIGHTS_BYTES 8
#define MIN_SANE_PAYLOAD (sizeof(struct freedomhdr) + LIBNET_IP_H + \
                 MAC_BYTES + EIGHTS_BYTES + 1) /*   1 key selection byte */
#define TRAILER_LEN (MAC_BYTES + EIGHTS_BYTES + 1)
#define DORKY_PADDING 2

#define DIR_FORWARD 0
#define DIR_REVERSE 1

static unsigned char eights[EIGHTS_BYTES] = {
        0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08
};

/*
 * using global variables.  I am weak.
 */
AIP_CRYPTO_CTX linkctx, telectx;
int use_crypto = 0, aci = 8;
struct sockaddr_in list, firsthop, myaddr;

/* 
 * encapsulate a packet with a freedom header
 * and trailer.  We assume we have enough room
 * for the both in the buffer.  The payload is assumed to follow
 * the fixed size header at the start of the buffer
 * that we are passed..
 */
static int freedomize(void *data, int payload_len, int aci)
{
    unsigned char *buffer = data; 
    struct freedomhdr *fh = (struct freedomhdr *)buffer;
    int diff;

    fh->vers = 1;
    fh->type = 1;
    fh->priority = 1;
    fh->aci = htons(aci);
    /* 
     * account for interesting padding 
     * that really comes before the 
     * freedom header
     */
    payload_len += 2;

    fh->payloadlen = htons(payload_len);

    buffer += sizeof (struct freedomhdr) + payload_len;
#if 0
    if(payload_len > MAX_PAYLOAD_LEN) {
        printf("%d too big\n",payload_len);
        return 0;
    }
    if(payload_len < MAX_PAYLOAD_LEN) {
        unsigned int len = MAX_PAYLOAD_LEN - payload_len;
        memset(buffer, 8 - (len & 0x7), len);
        buffer += len;
    } 
#else
    /* XXX our current friggin' cipher requires
     * hard multiples of 64bit blocks, and our
     * friggin headers (bless their pointy little
     * heads) require that the data have 4 'extra'
     * bytes to make up for the non-multiple length
     * of the headers.  HA-ZA.
     *
     * christ is this more messy than it needs to be.
     */
    diff = ((((payload_len - 4) + 7) & ~7) +4) - payload_len;
    if(diff) {
        memset(buffer, 0, diff);
        buffer+=diff;
    }
#endif
    memset(buffer, 0x69, MAC_BYTES);
    buffer += MAC_BYTES;
    memcpy(buffer, eights, EIGHTS_BYTES);
    buffer += EIGHTS_BYTES;
    *(buffer++) = 1; /* key byte */

    return (char *)buffer - (char *)data;
}

static void handle_outgoing(int tapfd, int udpfd)
{
    struct iphdr *iph;
#if 0
    struct udphdr *uh;
    struct tcphdr *th;
#endif
    int len;
    struct freedomhdr *fh;
    char buffer[BUFFER_LEN];

    read(tapfd, buffer + 1024, BUFFER_LEN - 1024);
    iph = (struct iphdr *)(buffer + 1024 + 16);
    fh = (struct freedomhdr *)((char *)iph - (sizeof(struct freedomhdr) + 2));

    len = ntohs(iph->tot_len);

    if(iph->version != 4) 
        return;

#if 0
    switch(iph->protocol) {
        case IPPROTO_TCP:
            th = (struct tcphdr *)((char *)iph + (iph->ihl << 2));
            if(th->dest != port) return;
        break;
        case IPPROTO_UDP:
            uh = (struct udphdr *)((char *)iph + (iph->ihl << 2));
            if(uh->dest != port) return;
        break;
        default:
            return;
    }
#endif

    hexdump("ip", iph, len);

    len = freedomize(fh, len, aci);
    if(!len) return;

    hexdump( "prepared packet", fh, len);

    if(use_crypto) {
        /* telescope encrypt */
        aip_crypto_encrypt( &telectx, fh->payload_rand,
            len - sizeof(struct freedomhdr) + 2 + 8 - 8 - 1);

    hexdump( "teled packet", fh, len);
        /* link encrypt */

    aip_crypto_encrypt( &linkctx, fh, 
#if 1
                len - 1 );
#else
                sizeof(struct freedomhdr));
#endif
    hexdump( "linked packet", fh, len);

    }

    sendto(udpfd, fh, len,
            0, &firsthop, sizeof(firsthop));
}

static int handle_incoming(int fd, int outfd)
{
    unsigned char buffer[BUFFER_LEN];
    unsigned char mac[]={0,0,0,1,2,3};
    struct iphdr *iph, *icmp_iph;
    struct udphdr *uh;
    struct ethhdr *eth;
    struct freedomhdr *fh;
    int len;

    len = read(fd, buffer, BUFFER_LEN);

    if(len > 0)
        hexdump("incoming", buffer, len);

    fh = (struct freedomhdr *)buffer;
    iph = (struct iphdr *)((char *)(fh + 1) + 2);
    eth = (struct ethhdr *)((char *)iph - sizeof(struct ethhdr));

    aip_crypto_decrypt(&linkctx, fh, len - 1);
    aip_crypto_decrypt(&telectx, fh->payload_rand, len - sizeof(struct freedomhdr) + 2 + 8 - 8 - 1);

    eth->h_proto = htons(ETH_P_IP);
    memcpy(eth->h_dest, mac, sizeof(eth->h_dest));
    memset(eth->h_source, 0, sizeof(eth->h_source));
    iph->daddr = myaddr.sin_addr.s_addr;

    switch(iph->protocol) {
        case IPPROTO_TCP:
            libnet_do_checksum((unsigned char *)iph, IPPROTO_TCP, 
                    ntohs(iph->tot_len) - (iph->ihl<<2));
            break;
        case IPPROTO_UDP: 
            uh = (struct udphdr *)((char *)iph + (iph->ihl<<2));
            libnet_do_checksum((unsigned char *)iph, IPPROTO_UDP, ntohs(uh->len));
            break;

        case IPPROTO_ICMP: 
            icmp_iph = (struct iphdr *)((char *)iph + (iph->ihl<<2));
            icmp_iph->saddr = myaddr.sin_addr.s_addr;
            break;
    }

    libnet_do_checksum((unsigned char *)iph, IPPROTO_IP, iph->ihl<<2);

    write(outfd, (char *)eth - 2, sizeof(struct ethhdr) + ntohs(iph->tot_len) + 2);

    return 0;
}

static int fill_sockaddr(int fd, const char *dev, struct sockaddr_in *sin) 
{
    struct ifconf iffc, *ifc = &iffc;
    struct ifreq *ifr;

    ifc->ifc_buf = alloca(50 * sizeof(struct ifreq));
    ifr = ifc->ifc_req;

    if(ioctl(fd, SIOCGIFCONF, ifc)) {
        perror("SIOCGIFCONF");
        return 1;
    }

    for( ;  ifc->ifc_len > 0 ; ifr++, ifc->ifc_len -= sizeof(struct ifreq)) {
        if(strcmp(dev, ifr->ifr_name)) continue;

        memcpy(&(sin->sin_addr.s_addr), &(ifr->ifr_addr.sa_data[2]), 4);
        return 0;
    }

    return 1;

}

int main(int argc, char  **argv) {
    int c;
    int opt = 1;
    int udpfd, tapfd;
    unsigned char *buffer;
    unsigned char zero[8] = {0,0,0,0,0,0,0,0};
    unsigned char key[] = "1234567890123456";
    struct pollfd pollfd[2];

    while( ( c = getopt(argc, argv, "xa:cp:") ) > 0) {
        switch (c) {
            case 'x':
                do_hexdump = 1;
                break;
            case 'a':
                aci = atol(optarg);
                break;
            case 'c':
                use_crypto = 1;
                break;
        }
    }

    if(use_crypto) {
        if ( !aip_crypto_init() ) {
            fprintf( stderr, "failed to access /dev/urandom" );
            exit( 1 );
        }
        aip_crypto_setup( &telectx, key );
        aip_crypto_setup_iv( &telectx, zero);
        aip_crypto_setup( &linkctx, key );
        aip_crypto_setup_iv( &linkctx, zero);
    }

    udpfd = socket(AF_INET,SOCK_DGRAM, IPPROTO_UDP); 
    if(udpfd < 0) {
        perror("socket");
        exit(1);
    }

    tapfd = open("/dev/tap0", O_RDWR);
    if(tapfd < 0) {
        perror("open tap0");
        exit(1);
    }

    if(fill_sockaddr(udpfd, "tap0", &myaddr)) 
        exit(1);

    pollfd[0].fd = udpfd;
    pollfd[0].events = POLLIN;
    pollfd[1].fd = tapfd;
    pollfd[1].events = POLLIN;

    list.sin_family = AF_INET;
    list.sin_addr.s_addr = INADDR_ANY;
    list.sin_port = htons(22222);

    firsthop.sin_family = AF_INET;
    firsthop.sin_addr.s_addr = inet_addr("10.16.1.17");
    firsthop.sin_port = htons(11111);

    if(bind(udpfd, &list, sizeof(list)) < 0) {
        close(udpfd);
        perror("bind");
        exit(1);
    }

    setsockopt(udpfd,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt));

    optind = opterr = 0;

    buffer = malloc(MIN_SANE_PAYLOAD + MAX_PAYLOAD_LEN + LIBNET_UDP_H + BUFFER_LEN);
    if(!buffer) {
        close(udpfd);
        perror("malloc");
        exit(1);
    }

    while(1) {
        int numready;

        numready = poll(pollfd, 2, -1);
        if(numready < 1) continue;

        if(pollfd[0].revents & POLLIN)  {
            handle_incoming(udpfd, tapfd);
        }

        if(pollfd[1].revents & POLLIN) {
            handle_outgoing(tapfd, udpfd);
        }
    }

    close(udpfd);
    close(tapfd);

    return 0;
}
