#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/ioctl.h>
#include <net/if.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/types.h>
#include <linux/netlink.h>

#include "kaipreq.h"


/*
 *      Cheesy kaip ioctl() configuration app
 *
 * Copyright (C) 2000 Zero-Knowledge[R] Systems, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

/*
 * I use this app to configure the kaip while testing.
 * Its incredibly fragile, but gives you something
 * to start with.  I'll happily accept pathces that
 * make it suck less :)
 *
 *          - zab
 */

void route_usage(char *bleh)
{
    printf(
        " %s -i [int] -c r blowfish 1234567890123456 [1.2.3.4 hmac 1234567890123456780] 1 8 3 9\n"
        " \n"
        "  This adds a rote to the kaip.  first are encryption algorithm\n"
        "  and key to be used for the telescope layer.  If traffic exits this\n"
        "  route (it is the last hop), then following the telescope info will\n"
        "  be the address to assign traffic that leaves vai this route, the \n"
        "  digest name to use for the hmac, and its key. Then comes the \n"
        "  incoming neighbor number, the incoming aci on that neighbor, the\n"
        "  outgoing neighbor id, and outgoing aci for that neighbor.\"0 0\"\n"
        "  as the outgoing neighbor id/aci specify an exit hop.\n",bleh);
}
int main(int argc, char  **argv)
{
    int c;
    extern char *optarg;
    extern int optind , opterr , optopt;
    struct sockaddr_in udpaddr;

    int sock;
    struct ifreq ifr;
    struct kaip_request req;

    memset(&ifr, 0, sizeof(ifr));
    memset(&req, 0, sizeof(req));

    ifr.ifr_data = (char *)&req;

    sock = socket(PF_INET, AF_LOCAL, IPPROTO_IP);
    if(sock < 0) {
        perror("socket()");
        exit(1);
    }

    optind = opterr = 0;

    while( ( c = getopt(argc, argv, "c:i:p:a:") ) > 0) {
        switch (c) {
            case 'c':

                switch(optarg[0]) {
                    case 'c': case 'C':
                        printf("clearstats ");
                        req.cmd = KAIP_SETSTATS;
                        memset(&(req.u.tstats),0,sizeof(req.u.tstats));
                        break;

                    case 'g': case 'G':
                        printf("getstats ");
                        req.cmd = KAIP_GETSTATS;
                        break;

                /* ./kaipctl -i kaip0 -c u -a 10.16.1.17 -p 11111 */
                    case 'u': case 'U':
                        printf("udpbind ");
                        req.cmd = KAIP_UDPBIND;
                        break;

                    case 's': case 'S':
                        printf("shutdown ");
                        req.cmd = KAIP_SHUTDOWN;
                        break;

                        /* see below.. */
                    case 'r': case 'R':
                        printf("add route ");
                        req.cmd = KAIP_ADDROUTE;
                        break;
                    case 'a': case 'A':
                        printf("add neigh ");
                        req.cmd = KAIP_ADDNEIGHBOR;
                        break;
                }
                break;

            case 'a':
                udpaddr.sin_family = AF_INET;
                udpaddr.sin_addr.s_addr = inet_addr(optarg);
                printf("addr: %s ",inet_ntoa(udpaddr.sin_addr));
                break;
            case 'p':
                udpaddr.sin_family = AF_INET;
                udpaddr.sin_port = ntohs(atol(optarg));
                printf("port %d ",atol(optarg));
                break;
            case 'i':
                printf("Interface: %s ",optarg);
                strncpy(ifr.ifr_name, optarg, IFNAMSIZ);
                break;
            case ':':
                printf("\n %c missing a parameter\n",optopt);
                exit(1);
                break;
            case '?':
                printf("\nunknown option %c\n",optopt);
                exit(1);
                break;
            default:
                printf("\nreally broken\n");
                exit(1);
                break;
        }
    }
    printf("\n");

    if( ifr.ifr_name[0]=='\0') {
        printf("need interface\n");
        exit(1);
    }

    if(req.cmd == KAIP_ADDNEIGHBOR) {
        /* lazy */
        memcpy(&req.u.tnei.addr, &udpaddr, sizeof(struct sockaddr_in));
    }

    if(req.cmd == KAIP_UDPBIND) {
        int udpfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

        if(udpfd < 0) {
            perror("socket");
            exit(1);
        }

        if(bind(udpfd, &udpaddr, sizeof(udpaddr)) < 0) {
            perror("bind");
            exit(1);
        }
        req.u.fd = udpfd;
    }

    if(req.cmd == KAIP_ADDROUTE) {
        struct kaip_req_route *troute = &req.u.troute;
        int offset;

        if(argc < 11) {
            route_usage(argv[0]);
            exit(1);
        }

        offset = argc-1;

        troute->server_aci = htons(atol(argv[offset--]));
        troute->server_neighbor_id = atol(argv[offset--]);

        troute->client_aci = htons(atol(argv[offset--]));
        troute->client_neighbor_id = atol(argv[offset--]);

        if(troute->server_aci == 0) {

            if(argc != 14) {
                route_usage(argv[0]);
                exit(1);
            }

            troute->mac_key_data = argv[offset--];
            troute->mac_key_len = strlen(troute->mac_key_data);

            strncpy(troute->mac_digest_name, argv[offset--], MAX_CIPHER_NAME);

            troute->fake_addr.sin_addr.s_addr = inet_addr(argv[offset--]);

            troute->exit_route = 1;
        }

        troute->telescope_in_key_data = 
            troute->telescope_out_key_data = argv[offset--];
        troute->telescope_in_key_len = 
            troute->telescope_out_key_len = strlen(troute->telescope_in_key_data);

        strncpy(troute->telescope_cipher_name, argv[offset--], MAX_CIPHER_NAME);
    }

/* 
 * ./kaipctl -i kaip0 -c a -a 10.16.128.79 -p 22222 1 blowfish 1234567890123456 1234567890123456 
 *
 * This adds a neighbor (client or aip) to the kaip.  First is the key number
 * that is active, then the encryption algo to be used on the link encryption, then
 * the two keys.
 */
    if(req.cmd == KAIP_ADDNEIGHBOR) {
        struct kaip_req_neighbor *tnei = &req.u.tnei;

        tnei->id = atol(argv[argc-4]);
        strncpy(tnei->link_cipher_name, argv[argc-3], MAX_CIPHER_NAME);
        tnei->link_from_key_data[0] = 
        tnei->link_from_key_data[1] = argv[argc-2];
        tnei->link_to_key_data[0] =
        tnei->link_to_key_data[1] = argv[argc-1];
        tnei->link_from_key_len[0] =
        tnei->link_from_key_len[1] = 16;
        tnei->link_to_key_len[0] =
        tnei->link_to_key_len[1] = 16;
    }

    if( ioctl(sock, SIOCDEVPRIVATE, &ifr) < 0) {
        perror("ioctl()");
    }

    /*
     * provide a vmstat 1 like output for
     * the kaip..
     */
    if( req.cmd == KAIP_GETSTATS ) {
        struct kaip_stats * stats = &(req.u.tstats);
        struct kaip_stats old;
        int i = 0;
        printf(" CTL-C to break..\n\n");
        memset(&old,0,sizeof(old));
        for(;;i--) {
            if(i == 0) {
                printf( 
                        "fr rx p    fr rx b:"
                        "fr tx p    fr tx b|"
                        "cl rx p    cl rx b:"
                        "cl tx p    cl tx b|"
                        "lnkd "
                        "lnke "
                        "tele "
                        " mac:"
                        "ocpb "
                        "icpb:"
                        " icycpp "
                        " ocycpp\n"

                        );
                i = 25;
            }
            ioctl(sock, SIOCDEVPRIVATE, &ifr);
#define VAL(FOO) ( stats->FOO - old.FOO)
#define DIV(A,B) ( (A) / ( (B) ? (B) : 1))
/*#define PER(ME) ( VAL(ME.cycles) / ( VAL(ME.num) ? VAL(ME.num) : 1))*/
#define PER(ME) ( DIV(VAL(ME.cycles) , VAL(ME.num)) )
            printf(
                    "%7lld %10lld:" 
                    "%7lld %10lld|" 
                    "%7lld %10lld:" 
                    "%7lld %10lld|" 
                    "%4lld "
                    "%4lld "
                    "%4lld "
                    "%4lld:"
                    "%4lld "
                    "%4lld:"
                    "%7lld "
                    "%7lld\n",

                VAL(freedom_rx_packets), VAL(freedom_rx_bytes),
                VAL(freedom_tx_packets), VAL(freedom_tx_bytes),
                VAL(clear_rx_packets), VAL(clear_rx_bytes),
                VAL(clear_tx_packets), VAL(clear_tx_bytes),

/*              PER(link_dec_cycles,link_dec_bytes), 
                PER(link_enc_cycles,link_enc_bytes),
                PER(telescope_cycles,telescope_bytes),
                PER(in_path_cycles,in_path_packets),
                PER(out_path_cycles,out_path_packets));*/

                PER(link_dec),
                PER(link_enc),
                PER(telescope),
                PER(mac),

                DIV(VAL(out_path.cycles), VAL(freedom_rx_bytes)),
                DIV(VAL(in_path.cycles), VAL(clear_rx_bytes)),

                PER(in_path),
                PER(out_path));

            memcpy(&old, stats, sizeof(old));
            sleep(1);
        }
    }

    close(sock);

    return 0;

}
