#ifndef _cAipKernel

#define _cAipKernel

// class cAipKernel
//

#include "frEnt.h"

#include "czkIntervalTime.h"
#include "czkLock.h"
#include "cRouteCreatorInfo.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "cConfigProvider.h"

#include "cLinkManagerInfo.h"

#include "cStatsHarvestor.h"
#include "cStatsHarvestorPktData.h"
#include "czkSockDgram.h"


#define AIPKERNEL_MAX_KEY_SIZE  24   // longest key is 24 bytes

#define AIPKERNEL_TX_BUF_SIZE   (64*1024)  // Socket tx buffer size
#define AIPKERNEL_RX_BUF_SIZE   (64*1024)  // Socket rx buffer size


class cAipKernel : public cAipComponent, public cStatsHarvestorPullHandler {

private:

 //configuration items
 string       kaipIfName;    // KAIP interface name
// string       aipAddr;       // AIP IP address
 uint32_t     nKaipDevices;  // number of Kaip devices

 int32_t      netlinkSock;   // netlink socket fd 

 int32_t      ioctlSock;     // used to issue commands to the kernel through ioctl() calls
 bool         useProxyIoctl; // Flag whether ioctl() calls to kaip are performed by parent or self
 
 cStatsHarvestor* pStatsHarvestor;
 cStatsHarvestorPktData* akStats;
 cConfigProvider *pConfigProvider;

 czkSockDgram frSocket;

  // copy constructor (enabled)
  cAipKernel(const cAipKernel &) : cAipComponent(), cStatsHarvestorPullHandler() { assert(0); };

  // assignment operator (disabled)
  void operator=(const cAipKernel &) { assert(0); };

  // set KAIP configuration point
  void configKaip(struct kaip_request & req, uint32_t nKaip);

  // Dispatch KAIP ioctl via parent proxy
  void ioctlProxy(struct kaip_request &req, const string &ifName);

public:

  // constructor
  cAipKernel(bool privilegedUser = true);

  // destructor
  virtual ~cAipKernel();

  void shutdown();
  
  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);
  void cleanupPrivateData();

  // startup
  void startup();

  // create/destroy a socket object that receives the management packets from KAIP
  void createSocket(czkSocket ** socket);
  void destroySocket(czkSocket ** socket);
  
  // link manipulation
  void addLink(LinkId linkId, cLinkManagerInfo* linkinfo);
  void updateLink(LinkId linkId, cLinkManagerInfo* linkinfo);
  void removeLink(LinkId linkId);

  // route manipulation
  void addRoute(RouteId routeId, cRouteCreatorInfo* routeinfo);
  void updateRoute(RouteId routeId, cRouteCreatorInfo* routeinfo, 
                   cRouteCreatorInfo* oldRouteinfo);
  void removeRoute(RouteId routeId, cRouteCreatorInfo* routeinfo);
  
  // shutdown kaip devices
  void shutdownKaip();

  // stats manipulation
  void resetStats();

  // handle the stats PULL requests from StatsHarvestor
  void handlePull( const char*, class StatsDataBase*);

  czkSockDgram * getFrSocket();
};

#endif
