#ifndef _cAipKernelPacketList

#define _cAipKernelPacketList

// class cAipKernelPacketList
//
#include "cLinkManager.h"
#include "cLinkManagerInfo.h"

#define AIPKERNEL_BUFFER_SIZE  65536

class cAipKernelPacketList
{
public:
    class cAipKernelPacket
    {
    public:
        char             * pkt;      // reference to packet payload
        uint32_t           pktSize;  // packet size
        LinkId             linkId;   // link ID from where the packet came from
        ZKPRNetAddr        srcAddr;  // source address of packet
        cAipKernelPacket * nextPkt;  // reference to next packet in list

    private:
        // copy constructor (disabled)
        cAipKernelPacket(const cAipKernelPacket &) { assert(0); };

        // assignment operator (disabled)
        void operator=(const cAipKernelPacket &) { assert(0); };

    public:
        // constructor
        cAipKernelPacket() : pkt(NULL), pktSize(0), linkId(0), nextPkt(NULL) {};

        // destructor
        virtual ~cAipKernelPacket() {};
    };
    
private:
    cAipKernelPacket firstPkt;          // head of the packet list
    cAipKernelPacket * currentPkt;      // reference to current pkt
    char buffer[AIPKERNEL_BUFFER_SIZE]; // 64k buffer

    cLinkManager* pLinkManager;

private:
    // copy constructor (disabled)
    cAipKernelPacketList(const cAipKernelPacketList &) { assert(0); };

    // assignment operator (disabled)
    void operator=(const cAipKernelPacketList &) { assert(0); };

    // free the existing packet list (if any)
    void destroyList();
        
public:
    // constructor
    cAipKernelPacketList();

    // destructor
    virtual ~cAipKernelPacketList();

    void setLinkManager( cLinkManager* pLinkManager) { this->pLinkManager = pLinkManager; };

    // receive packets from KAIP
    void recv(czkSocket &socket);

    // get rhe following packet from the packet list
    void getNextPacket(LinkId *linkId, ZKPRNetAddr * addr, 
                       char ** pkt, int32_t * pktSize);
};

#endif // _cAipKernelPacketList
