//
// cAipKernelPacketList class implementation
// 
// 

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <linux/types.h>
#include <linux/netlink.h>

#include "aip-modules.h"
#include "cAipKernelPacketList.h"


#define REV_ID "@(#) $Id: cAipKernelPacketList.cpp,v 1.2 2001/04/18 17:36:21 alainc Exp $"
#include <zkrev.h>



cAipKernelPacketList::cAipKernelPacketList()
{
    currentPkt = NULL;
}

cAipKernelPacketList::~cAipKernelPacketList()
{
    destroyList();
}

void cAipKernelPacketList::recv(czkSocket & sock)
{
    const char * func = "cAipKernelPacketList::recv";
    const char * msg = NULL;

    try
    {
        msg = "Freeing the current list";

        destroyList();
        
        msg = "Receiving from netlink socket";

        int32_t nb;
        if((nb = sock.recv(buffer, AIPKERNEL_BUFFER_SIZE, 0, 
                           ZKPR_INTERVAL_NO_WAIT)) <= 0)
        {
            CZKS::Throw(ZKS_AIPKERNEL_MOD, ZKS_AIPKERNEL_SOCKET_ERR, 
                        func, "Couldn't recv on netlink socket rc=%d", nb);
        }

        msg = "Extracting packets from netlink structure";

        char * bufferPtr = buffer;
        cAipKernelPacket * pkt = &firstPkt;
        struct nlmsghdr * nlh = (struct nlmsghdr *)bufferPtr;
        
        while(nb > 0)
        {
	    switch (nlh->nlmsg_type) {

            case 0: //data packet
	    {
                // get the source link ID
                pkt->linkId = *(LinkId *)NLMSG_DATA(nlh);
		bufferPtr += nlh->nlmsg_len;
		nb -= nlh->nlmsg_len;

		// get the source address
		nlh = (struct nlmsghdr *)bufferPtr;
		struct ip * ipHdr = (struct ip *)NLMSG_DATA(nlh);
		struct udphdr * udpHdr = (struct udphdr *)((char *)ipHdr + 
							   (ipHdr->ip_hl * sizeof(uint32_t)));
		pkt->srcAddr.inet.family = AF_INET;
		pkt->srcAddr.inet.ip = ipHdr->ip_src.s_addr;
		pkt->srcAddr.inet.port = udpHdr->source;
		bufferPtr += nlh->nlmsg_len;
		nb -= nlh->nlmsg_len;
            
            // get the packet info
		nlh = (struct nlmsghdr *)bufferPtr;
		pkt->pkt = (char *)NLMSG_DATA(nlh);
		pkt->pktSize = nlh->nlmsg_len - NLMSG_SPACE(0);
		if ((nb - nlh->nlmsg_len) <= 0)
		  {
		    ZKS_ASSERT(ZKS_AIPKERNEL_MOD, (nb - nlh->nlmsg_len) == 0);
		    pkt->nextPkt = NULL;
		  }
		else
		  {
		    pkt->nextPkt = new cAipKernelPacket;
		    pkt = pkt->nextPkt;
		    pkt->nextPkt = NULL;
		  }

		bufferPtr += nlh->nlmsg_len;
		nb -= nlh->nlmsg_len;
	    }
	    break;

	    case 1: // link rekey message
	    {
	        struct rekey_struct { 
                    LinkId newLinkId;
		    LinkId oldLinkId;
	        } *rekey;
                rekey = (struct rekey_struct*)NLMSG_DATA(nlh);
		bufferPtr += nlh->nlmsg_len;
		nb -= nlh->nlmsg_len;
                ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, ZKS_AIPKERNEL_MOD, 
                               " handleLinkRekey new %d old %d ", 
                               rekey->newLinkId, rekey->oldLinkId);
                try { 
                    pLinkManager->handleLinkRekey( rekey->newLinkId, 
                                                   rekey->oldLinkId);
		}
                catch (CZKS::Exception &e) {
      CZKS::Push(e, func, "failure");
		  CZKS::LogException(e);
		}
	    }		
            break;

	    default:

                CZKS::Throw(ZKS_AIPKERNEL_MOD, ZKS_AIPKERNEL_SOCKET_ERR, 
                            func, "Unrecognized message type received from netlink socket");

	    }
     
            nlh = (struct nlmsghdr *)bufferPtr;

	    // Is this the last nlmsg?
	    if( nlh->nlmsg_type == NLMSG_DONE )
	    {
	      // This is a NLMSG_DONE net link msg, indicating the end of
	      // the vector. This will actually nb to 0, thereby exiting
	      // the loop.
	      bufferPtr += nlh->nlmsg_len;
	      nb -= nlh->nlmsg_len;
	    }
        }        
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void cAipKernelPacketList::getNextPacket(LinkId * linkId, ZKPRNetAddr * addr,
                                         char ** pkt, int32_t * pktSize)
{
    const char * func = "cAipKernelPacketList::getNextPacket";
    const char * msg = NULL;

    try
    {
        msg = "Retrieving next packet info";

        if (currentPkt == NULL)
        {
            if (firstPkt.pktSize != 0)
            {
                // This is the first call
                currentPkt = &firstPkt;
                *pkt = currentPkt->pkt;
                *pktSize = currentPkt->pktSize;
                *linkId = currentPkt->linkId;
                *addr = currentPkt->srcAddr;
            }
            else
            {
                // no available packet
                *pkt = NULL;
                *pktSize = 0;
                *linkId = 0;
            }
        }
        else
        {
            if (currentPkt->nextPkt == NULL)
            {
                // there is no more available packets
                *pkt = NULL;
                *pktSize = 0;
                *linkId = 0;
            }
            else
            {
                // retrieve next packet info
                currentPkt = currentPkt->nextPkt;
                *pkt = currentPkt->pkt;
                *pktSize = currentPkt->pktSize;
                *linkId = currentPkt->linkId;
                *addr = currentPkt->srcAddr;
            }
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void cAipKernelPacketList::destroyList()
{
    // delete dynamically allocated packets from packet list (first pkt is static)
    cAipKernelPacket * current = NULL;
    cAipKernelPacket * pkt = firstPkt.nextPkt;
    while(pkt != NULL)
    {
        current = pkt;
        pkt = current->nextPkt;
        delete current;
    }
    
    currentPkt = NULL;
    firstPkt.pkt = NULL;
    firstPkt.pktSize = 0;
    firstPkt.linkId = 0;
    firstPkt.nextPkt = NULL;
}
