#define REV_ID "@(#) $Id: aipKernelUnitTest.cpp,v 1.2 2001/04/18 17:36:23 alainc Exp $"
#include <zkrev.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <linux/types.h>
#include <linux/netlink.h>

#include "aip-modules.h"
#include "zksexception.h"
#include "zkslog.h"
#include "zkspath.h"
#include "frCrypt.h"
#include "czkNetAddr.h"
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "cAipComponentManager.h"
#include "cRouteCreatorInfo.h"
#include "cKeyProviderCacheEntryData.h"
#include "cAipKernelPacketList.h"
#include "cAipKernel.h"
#include "cNetInfoProvider.h"
#include "cNetInfoEntity.h"

namespace {
 
  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };
   
  const uint16_t UT_FREEDOM_PORT = 34768;
  const uint16_t UT_NETLINK_PORT = 54323;
  const size_t UT_LINKID_1 = 23948743;
  const size_t UT_LINKID_2 = 32987464;
  const size_t UT_LINKID_3 = 98743432;
  const char * UT_MSG_1 = "Once upon a time";
  const char * UT_MSG_2 = "there was a young girl";
  const char * UT_MSG_3 = "and seven little dwarves";

};

//
// Signal handler stuff
// 

class SigHandler {

private:
  czkCoreLoop* coreloop;

public:

  void setCoreLoop(czkCoreLoop* cl) { coreloop = cl; };

  void addSignal( int32_t signum) { coreloop->addSignal( signum); }
};

SigHandler GlobalSigHandler;

void sighandler( int32_t signum)
{
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_AIPKERNEL_MOD, " adding signal %d ", signum);
  GlobalSigHandler.addSignal( signum);
}


//
// Required class implementation for linking
// 

cRouteCreatorInfo::cRouteCreatorInfo() {}
cRouteCreatorInfo::~cRouteCreatorInfo() {}

GenericId::GenericId() {}

cKeyProviderCacheEntryData::cKeyProviderCacheEntryData() {}
cKeyProviderCacheEntryData::~cKeyProviderCacheEntryData() {}

FrAci cAciBin::getAci() { return 0; }
void cAciBin::returnAci(uint32_t) {}

void cLinkManager::handleLinkRekey(uint32_t, uint32_t) {}

cLinkManagerInfo::cLinkManagerInfo() {}
cLinkManagerInfo::~cLinkManagerInfo() {}

cRouteCreatePkt::~cRouteCreatePkt() {}

cNetInfoEntity::entListType tmpList;
const cNetInfoEntity cNetInfoEntity::NullEntity;
cNetInfoEntity tmp;
cNetInfoProvider::cNetInfoProvider() {}
cNetInfoProvider::~cNetInfoProvider() {}
bool cNetInfoProvider::validateConfig() { return true; }
bool cNetInfoProvider::initializePrivateData(czkCoreLoop*) { return true; }
void cNetInfoProvider::cleanupPrivateData() {}
void cNetInfoProvider::startup() { signalReady(); }
void cNetInfoProvider::shutdown() {}
void cNetInfoProvider::refresh() {}
bool cNetInfoProvider::connectNids(unsigned int) { return true; }
void cNetInfoProvider::prodNid(cNetInfoData*) {}
const cNetInfoEntity &cNetInfoProvider::getEntity(FrEntIdType, const char*)
{ return tmp;}
const cNetInfoEntity &cNetInfoProvider::getEntity(FrEntIdType, uint32_t)
{return tmp;}
cNetInfoEntity::entListType *cNetInfoProvider::getEntities(FrEntIdType, bool)
{ return &tmpList;}
cNetInfoEntity cNetInfoProvider::getNextKQS() {return tmp;}
cNetInfoEntity cNetInfoProvider::getNextNISS() {return tmp;}

cNetInfoEntity::entListType *cNetInfoProvider::getAIPs() { return &tmpList; }
cNetInfoEntity::cNetInfoEntity() {}
cNetInfoEntity::~cNetInfoEntity() {}
cNetInfoEntity::cNetInfoEntity(const FrNetInfoDatum) {}
cNetInfoEntity::cNetInfoEntity(const cNetInfoEntity&) {};
cNetInfoEntity&  cNetInfoEntity::operator=(const cNetInfoEntity&) {return tmp;};

// cout - used to return "ok" or "not ok" to be picked up by the test
// framework
// cerr - used for regular output
int main(int, char * argv[])
{

    const char         * func = "main";
    ZksErr               zkserr = NULL;
    ZksLogAgent          logAgent = NULL;
    CZKS::Exception      zksException;
    cAipComponentManager mgr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return UT_FATAL;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      return UT_FATAL;
     }

    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n",
              "AIP");
      return UT_FATAL;
    }

    try
    {
        try
        {

	  //print the number of tests being run, in this case it is
	  // hard coded to 2... 
	  // BUG - make it a variable amount
	  std::cout << "1..2" << endl;

            if (!frCryptInit(zksException.zkError(), 0)) {
                CZKS::Throw(zksException);
            }

            // Test #1:  receive 1 packet
            
            // format simulated netlink address
            czkNetAddr netAddr;
            ZKPRNetAddr netlinkAddr;
            char hostName[256];
            netlinkAddr.inet.family = ZKPR_AF_INET;
            assert(gethostname(hostName, 256) == 0);
            netAddr.getHostbyName(hostName, 256);
            netAddr.enumerateHostEnt(0, UT_NETLINK_PORT, &netlinkAddr);

            // create a UDP socket for reception of netlink stuff
            czkSockAcceptor netlinkAcceptSock;
            assert(netlinkAcceptSock.open() == ZKPR_SUCCESS);
            assert(netlinkAcceptSock.bind(netlinkAddr) == ZKPR_SUCCESS);
            assert(netlinkAcceptSock.listen(1) == ZKPR_SUCCESS);
            
            // create a UDP socket for transmission of netlink stuff (simulates KAIP)
            czkSockConnector kaipSock;
            assert(kaipSock.open() == ZKPR_SUCCESS);
            kaipSock.connect(netlinkAddr, ZKPR_INTERVAL_NO_WAIT);

            sleep(1);

            czkSocket * netlinkSock;
            ZKPRNetAddr addrFrom;
            assert(netlinkAcceptSock.accept(&netlinkSock, &addrFrom, 
                                            ZKPR_INTERVAL_NO_TIMEOUT) == ZKPR_SUCCESS);
            
            // format a netlink struct containing 1 packet
            char buffer[65536];
            int32_t bufSize;
            struct nlmsghdr * nlh = (struct nlmsghdr *)buffer;
            char * msg = (char *)NLMSG_DATA(nlh);
            *(LinkId *)msg = UT_LINKID_1;
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(LinkId));
            bufSize += nlh->nlmsg_len;

            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            memset(msg, 0x55, sizeof(struct ip) + sizeof(struct udphdr));
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(struct ip) + sizeof(struct udphdr));
            bufSize += nlh->nlmsg_len;
            
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            strcpy(msg, UT_MSG_1);
            nlh->nlmsg_len = NLMSG_LENGTH(strlen(UT_MSG_1) + 1);
            bufSize += nlh->nlmsg_len;
            
            // send the netlink struct
            assert(kaipSock.send(buffer, bufSize, 0, 
                                 ZKPR_INTERVAL_NO_TIMEOUT) == bufSize);

            sleep(1);
            
            // recv the packet list
            cAipKernelPacketList pktList;
            pktList.recv(*netlinkSock);
            
            // get first packet and validate
            LinkId linkId;
            ZKPRNetAddr srcAddr;
            char * pkt;
            int32_t pktSize;
            pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
            if ((linkId != UT_LINKID_1) || (srcAddr.inet.ip != 0x55555555) ||
                (srcAddr.inet.port != 0x5555) || (pkt == NULL) || 
                (pktSize != (int32_t)(strlen(UT_MSG_1) + 1)) ||
                (memcmp(pkt, UT_MSG_1, pktSize) != 0))
            {
		std::cout << "not ok" << endl;
            }
            else
            {
                // verify that no other packet in packet list
                pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
                if ((linkId != 0) || (pkt != NULL) || (pktSize != 0))
                {
		    std::cout << "not ok" << endl;
                }
                else
                {
		    std::cout << "ok" << endl;
                }
            }

            // Test #2:  receive 3 packets
            
            // format a netlink struct containing 3 packets
            // first
            bufSize = 0;
            nlh = (struct nlmsghdr *)buffer;
            msg = (char *)NLMSG_DATA(nlh);
            *(LinkId *)msg = UT_LINKID_1;
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(LinkId));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            memset(msg, 0x11, sizeof(struct ip) + sizeof(struct udphdr));
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(struct ip) + sizeof(struct udphdr));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            strcpy(msg, UT_MSG_1);
            nlh->nlmsg_len = NLMSG_LENGTH(strlen(UT_MSG_1) + 1);
            bufSize += nlh->nlmsg_len;

            // second
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            *(LinkId *)msg = UT_LINKID_2;
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(LinkId));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            memset(msg, 0x22, sizeof(struct ip) + sizeof(struct udphdr));
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(struct ip) + sizeof(struct udphdr));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            strcpy(msg, UT_MSG_2);
            nlh->nlmsg_len = NLMSG_LENGTH(strlen(UT_MSG_2) + 1);
            bufSize += nlh->nlmsg_len;

            // third
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            *(LinkId *)msg = UT_LINKID_3;
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(LinkId));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            memset(msg, 0x33, sizeof(struct ip) + sizeof(struct udphdr));
            nlh->nlmsg_len = NLMSG_LENGTH(sizeof(struct ip) + sizeof(struct udphdr));
            bufSize += nlh->nlmsg_len;
            nlh = (struct nlmsghdr *)(buffer+bufSize);
            msg = (char *)NLMSG_DATA(nlh);
            strcpy(msg, UT_MSG_3);
            nlh->nlmsg_len = NLMSG_LENGTH(strlen(UT_MSG_3) + 1);
            bufSize += nlh->nlmsg_len;

            // send the netlink struct
            assert(kaipSock.send(buffer, bufSize, 0, 
                                 ZKPR_INTERVAL_NO_TIMEOUT) == bufSize);

            sleep(1);
            
            // recv the packet list
            pktList.recv(*netlinkSock);
            
            // get first packet and validate
            pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
            if ((linkId != UT_LINKID_1) || (srcAddr.inet.ip != 0x11111111) ||
                (srcAddr.inet.port != 0x1111) || (pkt == NULL) || 
                (pktSize != (int32_t)(strlen(UT_MSG_1) + 1)) ||
                (memcmp(pkt, UT_MSG_1, pktSize) != 0))
            {
		std::cout << "not ok" << endl;
            }
            else
            {
                // get 2nd packet and validate
                pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
                if ((linkId != UT_LINKID_2) || (srcAddr.inet.ip != 0x22222222) ||
                    (srcAddr.inet.port != 0x2222) || (pkt == NULL) || 
                    (pktSize != (int32_t)(strlen(UT_MSG_2) + 1)) ||
                    (memcmp(pkt, UT_MSG_2, pktSize) != 0))
                {
		    std::cout << "not ok" << endl;
                }
                else
                {
                    // get 3rd packet and validate
                    pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
                    if ((linkId != UT_LINKID_3) || (srcAddr.inet.ip != 0x33333333) ||
                        (srcAddr.inet.port != 0x3333) || (pkt == NULL) || 
                        (pktSize != (int32_t)(strlen(UT_MSG_3) + 1)) ||
                        (memcmp(pkt, UT_MSG_3, pktSize) != 0))
                    {
			std::cout << "not ok" << endl;
                    }
                    else
                    {
                        // verify that no other packet in packet list
                        pktList.getNextPacket(&linkId, &srcAddr, &pkt, &pktSize);
                        if ((linkId != 0) || (pkt != NULL) || (pktSize != 0))
                        {
			    std::cout << "not ok" << endl;
                        }
                        else
                        {
                          std::cout << "ok" << endl;
                        }
                    }
                }
            }

            assert(kaipSock.close() == ZKPR_SUCCESS);
            assert(netlinkSock->close() == ZKPR_SUCCESS);
            assert(netlinkAcceptSock.close() == ZKPR_SUCCESS);
        }
        catch (...)
        {
            CZKS::Rethrow(func, "Unexpected problem occured\n");
        }
    }
    catch (CZKS::Exception &e)
    {
        CZKS::LogException(e);
        return UT_FATAL;
    }

    zksLogAgentFileFree(zkserr, &logAgent);
    zksErrFree(&zkserr);
    
    return UT_PASS;
}

