#ifndef _cAipComponentManager

#define _cAipComponentManager

#include <map>
#include <vector>

#include "czknspr.h"

#include "aip-modules.h"

#include "cAipComponent.h"
#include "czkCoreLoop.h"
#include "czkThread.h"

class cConfigProvider;

struct ltcomponent
{
  bool operator()(const char* s1, const char* s2) const
  {
    // NOTE that "strcmp" is safe here, dealing with only
    // our own defined component name strings 
    return strcmp(s1, s2) < 0;
  }
};

// cAipComponentManager is a class which will provide
//  access to each component object
//
class cAipComponentManager 
{
  private:

  static const int SleepTime=1;

  czkThread threader;

  // a map containing a pointer to all component objects 
  typedef  map <const char*, cAipComponent*, ltcomponent> ComponentMap;
  ComponentMap components;
  
  //copy constructor (disabled)
  cAipComponentManager( const cAipComponentManager&) { assert(0); };

  //assignment operator (disabled)
  cAipComponentManager& operator=( const cAipComponentManager&) { assert(0); };

  public:
  
  //constructor
  cAipComponentManager();

  //destructor
  virtual ~cAipComponentManager();

   // add a new component to the component map
  void add( const char* name, cAipComponent* component);

  // retrieve a pointer to a specific component object
  cAipComponent* get(const char* name);

  // run a component's "startup" method (in a thread or not)
  void startComponent(const char* name, bool dothread);

  // run a component's "startup" method (in a thread or not)
  void startComponent(cAipComponent* pComponent, bool dothread);

  // validate the config of each component
  void validateAllConfig();

  // initialize specific component
  void initComponent(cAipComponent* pComponent, czkCoreLoop* coreloop);

  // initialize each component
  void initAllComponents(czkCoreLoop* coreloop);

  // blocks until a component fails, returns pointer
  cAipComponent* getFailedComponent();

   // wait for all threads to complete
  void waitForCompletion();

  // kill all components
  void deleteAllComponents();

};

#endif
