#include "czkIntervalTime.h"
#include "cAipComponent.h"

#define REV_ID "@(#) $Id: cAipComponent.cpp,v 1.2 2001/04/18 17:36:27 alainc Exp $"
#include <zkrev.h>



class cAipComponentManager;
  
// cAipComponent is the base class of all component classes
//
// contains basic entities such as the coreloop object
// and a pointer to the ComponentManager object (through which
// the component can get a pointer to other components)
//
// each component must have a "startup" method that
// is run to setup and perform processing (via coreloop)
//
//  
 
void cAipComponent::init()   
{
  const char* func = "cAipComponent::init";

  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " in AipComponent init ");

  threadAlive = true;
  ready = false;
  privateDataInitialized = false;

  if (startupLock.open() != ZKPR_SUCCESS) {
    CZKS::Throw( ZKS_COMPONENTMANAGER_MOD, ZKS_COMPONENTMANAGER_OPENLOCK_ERR,
                 func, "Failed to initialize");  
  }
  if (startupCondition.open(startupLock) != ZKPR_SUCCESS) {
    startupLock.close();
    CZKS::Throw( ZKS_COMPONENTMANAGER_MOD, ZKS_COMPONENTMANAGER_OPENCOND_ERR,
                 func, "Failed to initialize");  
  }

  isInitialized = true;
}

void cAipComponent::cleanup()
{

  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " in AipComponent cleanup ");

  if (isInitialized)
  {
    startupCondition.close();
    startupLock.close();
    isInitialized = false;
  }
  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " exit AipComponent cleanup ");
}
  

void cAipComponent::setComponentManager( cAipComponentManager* mgr)
  {
    // store the pointer to AipComponentManager object
    pComponentManager = mgr;
  }

void cAipComponent::signalReady()
  {
    // call this when the component is initialized & ready to go
  
    startupLock.acquire();
    ready = true;
    startupCondition.broadcast();
    startupLock.release();
  }

bool cAipComponent::waitForReadySignal( ZKPRIntervalTime timeout)
  {
    // called by other threads to wait for this component to complete
    // its initialization sequence

    startupLock.acquire();
    if (!ready) startupCondition.wait( timeout);
    startupLock.release();
    return ready;
  }

  





