#include "cAipComponentManager.h"

#define REV_ID "@(#) $Id: cAipComponentManager.cpp,v 1.2 2001/04/18 17:36:27 alainc Exp $"
#include <zkrev.h>



   
//constructor
cAipComponentManager::cAipComponentManager()
{
}

//destructor
cAipComponentManager::~cAipComponentManager()
{
}

void cAipComponentManager::add( const char* name, cAipComponent* component)
{
    components[name] = component;
    component->setComponentManager(this);
}

// retrieve a pointer to a specific component object
cAipComponent* cAipComponentManager::get(const char* name)
{
  ComponentMap::iterator j = components.find( name);
  cAipComponent* p;
  if (j == components.end()) {
    p = NULL;
  }
  else {
    p = j->second;
  }
  return p;
}

// if dothread, spawn a thread to run a component's "startup" method
// if not, simply run a component's "startup" method
void cAipComponentManager::startComponent(const char* name, 
                                          bool dothread)
{
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, "Starting component %s", name);
  startComponent( components[name], dothread);
}

// if dothread, spawn a thread to run a component's "startup" method
// if not, simply run a component's "startup" method
void cAipComponentManager::startComponent(cAipComponent* component, 
                                          bool dothread)
{
  const char* func = "cAipComponentManager::startComponent";

  //component->init();
  //component->setComponentManager(this);
  //component->initializePrivateData( coreloop);
  if (dothread) {
    component->setThreadAlive(true);
    ZKPRThread* handle = threader.spawn( StartThread, 
                                         (void*)component,
                                         (ZKPRThreadScope)0, 
                                         (ZKPRThreadState)0, 
                                         (ZKPRThreadPriority)0, 0);
    if (!handle) {
      CZKS::Throw( ZKS_COMPONENTMANAGER_MOD,  
                   ZKS_COMPONENTMANAGER_CREATETHREAD_ERR,
                   func, "Failed to spawn component");
    }
  }
  else {
    component->startup();
  }
}

void cAipComponentManager::validateAllConfig()
{
   const char* func = "cAipComponentManager::validateAllConfig";

   cAipComponent* pComponent = NULL;
  
   ComponentMap::iterator i;

   //loop to validate the configuration of each component
   i = components.begin();
   while (i != components.end())
   {
       pComponent = i->second;  
       if (!pComponent->validateConfig()) {
	 CZKS::Throw(ZKS_COMPONENTMANAGER_MOD,  ZKS_COMPONENTMANAGER_VALIDATECONFIG_ERR,
                     func, "failed to validate config for component %s", i->first);
       }
       i++;
   }
}

void cAipComponentManager::initComponent(cAipComponent* pComponent, 
                                         czkCoreLoop* coreloop)
{

   if (!pComponent->isInit()) {
      pComponent->init();
      pComponent->initializePrivateData( coreloop);
   }

}

void cAipComponentManager::initAllComponents(czkCoreLoop* coreloop)
{

   cAipComponent* pComponent = NULL;
  
   ComponentMap::iterator i;

   //loop to initialize each component
   i = components.begin();
   while (i != components.end())
   {
       pComponent = i->second;  
       initComponent( pComponent, coreloop);
       i++;
   }
}

// return pointer to a failed component if a thread exits
cAipComponent* cAipComponentManager::getFailedComponent()
{   

  cAipComponent* pComponent = NULL;

  while (components.size() > 0 && pComponent == NULL)
  {
     for (ComponentMap::iterator i = components.begin(); 
          i != components.end(); i++)
     {
       cAipComponent* component = i->second;
       if (!component->isThreadAlive()) {
         pComponent = component;
         break;
       }
     }
     sleep(SleepTime);
  }
  return pComponent;
}

// wait for all threads to complete
void cAipComponentManager::waitForCompletion()
{   
  while (components.size() > 0)
  {
     for (ComponentMap::iterator i = components.begin(); 
          i != components.end(); i++)
     {
       cAipComponent* pComponent = i->second;
       if (!pComponent->isThreadAlive()) {
         pComponent->cleanup();
         delete pComponent;
         components.erase(i);
       }
     }
     sleep(SleepTime);
  }
}

// kill all components
void cAipComponentManager::deleteAllComponents()
{
     cAipComponent* pComponent = NULL;
     ComponentMap::iterator next;

     //loop to shutdown all components
     ComponentMap::iterator i = components.begin();
     while (i != components.end())
     {
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " shutdown component %s", 
                      i->first);
       pComponent = i->second;  

       try 
       {
         pComponent->shutdown();
       }

       catch (CZKS::Exception &e)
       {
         CZKS::Push(e, "cAipComponentManager::deleteAllComponents", "shutdown failure");
         CZKS::LogException( e);
       }

       i++;
     }

     //loop to cleanup all components
     i = components.begin();
     while (i != components.end())
     {
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " cleanupPrivateData component %s", 
                      i->first);
       pComponent = i->second;  

       try 
       {
         pComponent->cleanupPrivateData();
       }

       catch (CZKS::Exception &e)
       {
         CZKS::Push(e, "cAipComponentManager::deleteAllComponents", "cleanup private data failure");
         CZKS::LogException( e);
       }

       i++;
     }

     //loop to delete all components
     i = components.begin();
     while (i != components.end())
     {
       next = i;
       next++;
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " cleanup component %s", 
                      i->first);
       pComponent = i->second;  

       try 
       {
         pComponent->cleanup();
       }

       catch (CZKS::Exception &e)
       {
         CZKS::Push(e, "cAipComponentManager::deleteAllComponents", "cleanup component failure");
         CZKS::LogException( e);
       }

       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " deleting component %s", 
                      i->first);
       delete pComponent;
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " erasing component %s", 
                      i->first);
       components.erase(i);
       i = next;
     }
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " exit deleteAllComponents"); 
}

