#define REV_ID "@(#) $Id: testComponentManager.cpp,v 1.2 2001/04/18 17:36:28 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cComponentManager
//
// usage:      testComponentManager
//
//
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include "zkspath.h"
#include "frCrypt.h"
    
#include <iostream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkEventHandler.h"
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

namespace {

  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };

  const string test_not_ok = "not ok";
  const string test_ok = "ok";
  
};

class cFakeComponent : public cAipComponent
{
private:

  int32_t sleepTime;
 
  //copy constructor (disabled)
  cFakeComponent( const cFakeComponent&);

  //assignment operator (disabled)
  cFakeComponent& operator=( const cFakeComponent&);

public:

  //constructor
  cFakeComponent() : cAipComponent() {};

  bool validateConfig() { return true; };

  bool initializePrivateData(czkCoreLoop*) { return true;}; 
  
  void cleanupPrivateData() {}; 
  
  void startup() { signalReady(); sleep(sleepTime); }

  void shutdown() {};

  void setSleepTime( int32_t sleepTimein) { sleepTime = sleepTimein; }
};

int main(int, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    cout << "1..1" << endl;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return UT_UNINITIATED;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      return UT_UNINITIATED;
    } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",          
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "AIP:  can't allocate the log agent.\n");
      return UT_UNINITIATED;
    }
    
    try {
      
      try {
        
        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

        bool dothread = false;

        czkCoreLoop* coreloop = new czkCoreLoop;

        // start comp1
        cFakeComponent* pcomp1 = new cFakeComponent;
        mgr.add("Comp1", pcomp1);
        pcomp1->validateConfig();

        // start comp2
        cFakeComponent* pcomp2 = new cFakeComponent;
        mgr.add("Comp2", pcomp2);
        pcomp2->validateConfig();

        mgr.initAllComponents(coreloop);
        
        pcomp1->setSleepTime(5);
        mgr.startComponent("Comp1", dothread);

        pcomp2->setSleepTime(2);
        mgr.startComponent("Comp2", dothread);

        pcomp1->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);
        pcomp2->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);

	ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " got LinkManager ready signal ");

         if (!dothread) {
 
	   //int32_t status = coreloop->runEventLoop();
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " TEST Completed successfully");
		  cout << test_ok << endl;

        }
        else {

          cAipComponent* pComponent = mgr.getFailedComponent();
          while (pComponent)
          {
            if (pComponent == pcomp1) {
              ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " comp1 failed");
			  cout<<test_not_ok;
			  cout.flush();
			  return UT_FAIL;
			}
            else if (pComponent == pcomp2) {
              ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_COMPONENTMANAGER_MOD, " comp2 failed");
			  cout<<test_not_ok;
			  cout.flush();
			  return UT_FAIL;
			}
            pComponent->cleanup();
            mgr.startComponent( pComponent, dothread);
            pComponent = mgr.getFailedComponent();
          }

		  // wait for all threads to complete

		  // mgr.waitForCompletion();
        }
      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_COMPONENTMANAGER_MOD, "exiting main");
      }
      catch (...) {
		
        CZKS::Rethrow( ZKS_COMPONENTMANAGER_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {
	  
      CZKS::LogException( e);
	  cout<<test_not_ok;
	  cout.flush();
	  return UT_FAIL;
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();

	return UT_PASS;
}


