#ifndef _cConfigProvider
#define _cConfigProvider

#include <iostream>
#include <map>
#include <string>
#include "cAipComponent.h"
#include <zkscmdline.h>

namespace CZKS {
  class Ini;
}


class cConfigProvider : public cAipComponent
{
 private:
  CZKS::Ini     * m_iniConfigP;
  CZKS::CmdLine * m_cmdLineP;

  template<class T> 
    bool checkCommandLine(const std::string & opt,
                          T                 & val, 
                          uint32_t            idx=0) const
    {
      try
        {
          if( m_cmdLineP->isSet(opt) )
            {
              m_cmdLineP->valueOf(opt, val, idx);
              return true;
            }
        }
      catch( ... )
        {
        }
      return false;
    }

    

 public:

  explicit cConfigProvider(CZKS::CmdLine & cmdLine, CZKS::Ini & iniConfig);
  virtual ~cConfigProvider();
    
  bool validateConfig();
    
  bool initializePrivateData(czkCoreLoop *coreloopin);

  void cleanupPrivateData();
    
  void startup();
  void shutdown();
    
  // Parameter access methods
  bool setOpt_bool(const char *section, const char *varname, bool val);
  bool getOpt_bool(const char *section, const char *varname);
  bool getOpt_bool(const char *section, const char *varname, bool def);

  bool setOpt_int(const char *section, const char *varname, int32_t val);
  int32_t getOpt_int(const char *section, const char *varname);
  int32_t getOpt_int(const char *section, const char *varname, int32_t def);

  bool setOpt_float(const char *section, const char *varname, float val);
  float getOpt_float(const char *section, const char *varname);
  float getOpt_float(const char *section, const char *varname, float def);

  bool setOpt_string(const char *section, const char *varname,
                     const string val);
  string getOpt_string(const char *section, const char *varname);
  string getOpt_string(const char *section, const char *varname, string def);    
};


#endif
