// TODO: what happens if same parameter fed more then once to cmdline?

#include <zksini.h>
#include <zkscmdline.h>

#include "cConfigProvider.h"

#define REV_ID "@(#) $Id: cConfigProvider.cpp,v 1.2 2001/04/18 17:36:31 alainc Exp $"
#include <zkrev.h>




cConfigProvider::cConfigProvider(CZKS::CmdLine & cmdLine, CZKS::Ini & iniConfig)
    :m_iniConfigP(&iniConfig)
    ,m_cmdLineP(&cmdLine)
{
}


cConfigProvider::~cConfigProvider()
{

}


bool cConfigProvider::validateConfig()
{
    return true;
}




bool cConfigProvider::initializePrivateData(czkCoreLoop *coreloopin)
{
    const char *func = "cConfigProvider::initializePrivateData()";
    const char *msg = NULL;
    
    try
    {
	msg = "Coreloop handling";
	coreloop = coreloopin; 
        
	if (coreloop == NULL)
        {
	    coreloop = new czkCoreLoop;
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    
    return true; 
}

void cConfigProvider::cleanupPrivateData()
{
}

void cConfigProvider::startup()
{
    signalReady();
}


void cConfigProvider::shutdown()
{
    if (ready)
    {
	ready = false;
    }
}



bool cConfigProvider::setOpt_string(const char *section,
                                    const char *varname, const string val)
{
    const char *func = "cConfigProvider::setOpt_string";
    const char *msg = NULL;
    
    try
    {
	msg = "Calling Ini::set";
	m_iniConfigP->set(section, varname, val);
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    
    return true;
}


bool cConfigProvider::setOpt_int(const char *section, const char *varname, int32_t val)
{
    const char *func = "cConfigProvider::setOpt_int";
    const char *msg = NULL;
    
    try
    {
	msg = "Calling Ini::set";
	m_iniConfigP->set(section, varname, val);
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    
    return true;
}

bool cConfigProvider::setOpt_float(const char *section, const char *varname, float val)
{
    const char *func = "cConfigProvider::setOpt_float";
    const char *msg = NULL;
    
    try
    {
	msg = "Calling Ini::set";
	m_iniConfigP->set(section, varname, val);
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    
    return true;
}

bool cConfigProvider::setOpt_bool(const char *section, const char *varname, bool val)
{
    const char *func = "cConfigProvider::setOpt_bool";
    const char *msg = NULL;
    
    try
    {
	msg = "Calling Ini::set";
	m_iniConfigP->set(section, varname, val);
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    
    return true;
}


bool cConfigProvider::getOpt_bool(const char *section, const char *varname)
{
    bool val;
    const char *func = "cConfigProvider::getOpt_bool(char, char)";
    const char *msg = NULL;

    try
    {
	if( checkCommandLine(varname, val) )
        {
	    if( !m_iniConfigP->at(section, varname, val, false) )
            {
		CZKS::Throw(ZKS_CONFIGPROVIDER_MOD, ZKS_CONFIGPROVIDER_INVALIDPARAM_ERR,
			    func, msg, "Invalid parameter [%s,%s]", section, varname);
            }
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;            
}


bool cConfigProvider::getOpt_bool(const char *section, const char *varname, bool def)
{
    bool val;
    const char *func = "cConfigProvider::getOpt_bool(char, char, bool)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) ) 
        {
	    m_iniConfigP->at(section, varname, val, def);
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;            
}


int32_t cConfigProvider::getOpt_int(const char *section, const char *varname)
{
    int32_t val;
    const char *func = "cConfigProvider::getOpt_int(char, char)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) )
	{
	    if( !m_iniConfigP->at(section, varname, val, 0) )
	    {
		CZKS::Throw(ZKS_CONFIGPROVIDER_MOD, ZKS_CONFIGPROVIDER_INVALIDPARAM_ERR,
			    func, "Invalid parameter [%s, %s]", section, varname);
	    }
	}
    }
  
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;            
}


int32_t cConfigProvider::getOpt_int(const char *section, const char *varname, int32_t def)
{
    int32_t val;
    const char *func = "cConfigProvider::getOpt_int(char, char, int32_t)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) )
        {
	    m_iniConfigP->at(section, varname, val, def);
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;    
}


float cConfigProvider::getOpt_float(const char *section, const char *varname)
{
    float val;
    const char *func = "cConfigProvider::getOpt_float(char *, char *)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) )
        {
	    if( !m_iniConfigP->at(section, varname, val, 0.0F) )
            {
		CZKS::Throw(ZKS_CONFIGPROVIDER_MOD, ZKS_CONFIGPROVIDER_INVALIDPARAM_ERR,
			    func, "Invalid parameter [%s,%s]", section, varname);
            }
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;            
}


float cConfigProvider::getOpt_float(const char *section, const char *varname, float def)
{
    float val;
    const char *func = "cConfigProvider::getOpt_float(char, char, float)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) )
        {
	    m_iniConfigP->at(section, varname, val, def);
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;
}


string cConfigProvider::getOpt_string(const char *section, const char *varname)
{
    // TODO: not use an intermediate string by using string &
    string val;
    string empty;
    const char *func = "cConfigProvider::getOpt_string(char, char)";
    const char *msg = NULL;

    try
    {
	if( !checkCommandLine(varname, val) )
        {
	    if( !m_iniConfigP->at(section, varname, val, empty) )
            {
		CZKS::Throw(ZKS_CONFIGPROVIDER_MOD, ZKS_CONFIGPROVIDER_INVALIDPARAM_ERR,
			    func, "Invalid parameter [%s,%s]", section, varname);
            }
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;            
}


string cConfigProvider::getOpt_string(const char *section, const char *varname, string def)
{
    string val;
    const char *func = "cConfigProvider::getOpt_float(char, char, string)";
    const char *msg = NULL;
    
    try
    {
	if( !checkCommandLine(varname, val) )
        {
	    m_iniConfigP->at(section, varname, val, def);
        }
    }
    catch(...)
    {
	CZKS::Rethrow(func, msg);
    }
    return val;
}

