#define REV_ID "@(#) $Id: config_test.cpp,v 1.2 2001/04/18 17:36:32 alainc Exp $"
#include <zkrev.h>
#include <zkspath.h>
#include <iostream>
#include <fstream>
#include "cConfigProvider.h"
#include "aip-params.h"
#include <zksini.h>

namespace {

  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };
}

int main(int argc, const char **argv)
{

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
      return UT_FATAL;
  }

  try
    {

      CZKS::Ini     ini;
      CZKS::CmdLine cmdline;
      std::ifstream file("./test.ini");    
      // TODO: basic tests
      // TODO: test same var in different sections (should be last val)
      // TODO: test unexisting val, should throw an exception
      // TODO: test with wrong var type
      // TODO: test unexisting val with default, should not throw exception.
      // TODO: test setting a run-time var
      // TODO: test setting a run-time var that overwrites another var
      // load config;

      file >> ini;
      
      cConfigProvider cp(cmdline, ini);
      cp.initializePrivateData(NULL);
      //        cp.processCommandLine(paramsCount, AIP_PARAMS, argc, argv);
//      cp.processIniStream(file);

      std::cout << "1..3" << endl;
      //////////////////
      // BASIC TESTS
      
      std::string string_val = cp.getOpt_string( "UAIP", "some_string" );
      std::cout << "some_string: " << string_val << std::endl;

      int integer_val = cp.getOpt_int( "UAIP", "some_integer" );
      std::cout << "some_integer: " << integer_val << std::endl;

      int float_val = cp.getOpt_float( "UAIP", "some_float" );
      std::cout << "some_float: " << float_val << std::endl;

      bool true_val = cp.getOpt_bool( "UAIP", "some_true_bool" );
      std::cout << "some_true_bool: " << true_val << std::endl;

      bool false_val = cp.getOpt_bool( "UAIP", "some_false_bool" );
      std::cout << "some_false_bool: " << false_val << std::endl;

      ////////////////
      // Same var, two different sections

      std::string var1 = cp.getOpt_string( "UAIP", "someparam" );
      std::string var2 = cp.getOpt_string( "KAIP", "someparam" );

      std::cout << "someparam, UAIP: " << var1 << ", " << 
	  "someparam, KAIP: " << var2 << std::endl;


      ///////////////
      // Unexisting val, should throw an exception

      try
      {
	  cp.getOpt_string( "SFDDSFDS", "DSFDSFS" );
          std::cout << "not ok" << endl;
      }
      catch(CZKS::Exception ex)
      {
        std::cout << "ok" << " Yay!! It did throw an exception" << std::endl;
      }


      ///////////////
      // Test with wrong var type

      try
      {
	  int blah = cp.getOpt_int( "UAIP", "someparam" );
          std::cout << "not ok" << endl;
      }
      catch(CZKS::Exception ex)
      {
        std::cout << "ok" << " Yay!! Wrong var type was caught" << std::endl;
      }

        
      // TODO: test with wrong var type, but with a default provided.

      /////////////////
      // setting run-time var

      cp.setOpt_int( "my_runtime_section", "new_runtime_var", 24 );

      try
      {
	  int runtimevar = cp.getOpt_int( "my_runtime_section", "new_runtime_var" );
	  if( runtimevar == 24 )
            std::cout << "ok" << " Successfully set run-time var" << std::endl;
	  else
            std::cout << "not ok" << " Setting a run time var did not work"
                      << std::endl;
      }
      catch( CZKS::Exception ex )
      {
        std::cout << "not ok" << " Setting a run time var did not work"
                  << std::endl;
      }
      
      /////////////////
      // setting run-time var that overites another var

      cp.setOpt_string( "KAIP", "someparam", "new_value" );
      
      try
      {
	  string var1 = cp.getOpt_string( "UAIP", "someparam" );
	  string var2 = cp.getOpt_string( "KAIP", "someparam" );

	  std::cout << "someparam, UAIP: " << var1 << ", " << 
	      "someparam, KAIP: " << var2 << std::endl;

      }
      catch( CZKS::Exception ex )
      {
	  // TODO: hmmm.....
      }

      

    }

  catch( CZKS::Exception ex )
    {
      CZKS::LogException(ex);
      return UT_FATAL;
    }
  
  return UT_PASS;
}
