//
// class cDHNegotiator
//
//

#ifndef _cDHNegotiator
#define _cDHNegotiator

#include <map>

#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "cAipComponent.h"
#include "frEnt.h"
#include "cConfigProvider.h"
#include "cNetInfoProvider.h"
#include "cKeyProviderCacheEntryData.h"
#include "cDHNegotiatorResultHandler.h"
#include "cStatsHarvestorPktData.h"


#define DHNEGO_TX_BUF_SIZE  (64*1024)
#define DHNEGO_RX_BUF_SIZE  (64*1024)

#define DHNEGO_BLOWFISH_ALG "BLOWFISH"


#define DHNEGO_FORMAT_RH_KEY(type, version)   ((type << 16) | version)


// Forward definition of private classes
class cDHNegotiatorEventHandlerAccept;
class cDHNegotiatorEventHandlerRecv;
class cDHNegotiatorInfo;

// Forward definition of other components
class cKeyProvider;
class cPrivateKeys;


//"less than" function for the map definition
struct lessent
{
  bool operator()(const FrEntIdExp s1, const FrEntIdExp s2) const
  {
    bool rc = (memcmp(&s1, &s2, sizeof(s1)) < 0);
    return rc;
  }
};

class cDHNegotiator : public cAipComponent
{
private:
    typedef std::map<uint32_t, cDHNegotiatorResultHandler *> DHNegoRhMap;

#pragma pack(1)
    typedef struct
    {
        FrEntIdExp entity;
        uint8_t    lekv;
    }DHIdPkt;
#pragma pack()
    
public:
    class DHNegoPortMapElem
    {
    public:
        czkSockAcceptor sock;
        DHNegoRhMap     rhMap;

    public:
        // constructor
        DHNegoPortMapElem() {};

        // destructor
        virtual ~DHNegoPortMapElem() {};
        
    private:
        // copy constructor (disabled)
        DHNegoPortMapElem(const DHNegoPortMapElem &) { assert(0); };

        // asignment operator (disabled)
        void operator=(const DHNegoPortMapElem &) { assert(0); };
    };

private:
    typedef std::map<uint16_t, DHNegoPortMapElem *> DHNegoPortMap;
    typedef std::map<FrEntIdExp, cDHNegotiatorInfo *, lessent> DHNegoPerName;

    cDHNegotiatorEventHandlerAccept * initiatorFirstAcceptEh;
    cDHNegotiatorEventHandlerAccept * recipientFirstAcceptEh;
    cDHNegotiatorEventHandlerRecv   * recvEh;

    cKeyProvider              * keyProvider;     // reference to Key Provider
    cPrivateKeys              * privateKeys;     // reference to Private Keys
    cLinkManager              * linkManager;     // reference to Link Manager
    cConfigProvider           * configProvider;  // reference to Config Provider
    cNetInfoProvider          * netInfoProvider; // reference to NetInfo Provider

    czkCoreLoop               * coreloop;   // reference to core loop
#ifdef AIPMULTITHREADED    
    czkLock                    negoMapLock; // D-H negotiation mapping lock
    czkLock                    aipDHLock;   // Inter-AIP D-H negotiation mapping lock
#else
#define negoMapLock 0
#define aipDHLock   0
#endif
    DHNegoPortMap              negoMap;        // D-H negotiation mapping
    DHNegoPerName              aipNegoPerName; // D-H negotiations with peer AIPs
    
    string                     aipIp;          // AIP IP address(string format)
    FrCryptSymAlg              symAlg;         // Symetric key algorithm type
    FrEntIdExp                 aipEntityId;    // AIP entity ID

    //DHNegotiator stats
    cStatsHarvestorPktData* dhStats;  // stats "counted" by the DHNegotiator

public:
    // constructor
    cDHNegotiator();

    // destructor
    virtual ~cDHNegotiator();

    // validate configuration
    bool validateConfig();

    // initialize private data members
    bool initializePrivateData(czkCoreLoop* coreloopin);

    // cleanup private data
    void cleanupPrivateData();

    // startup
    void startup();

    // shutdown
    void shutdown();
    
    void registerHandler(cDHNegotiatorResultHandler * rh);
    void removeHandler(cDHNegotiatorResultHandler * rh);

  
    void requestNegotiation(FrEntIdExp & entity, ZKPRNetAddr & peerAddr,
                            cDHNegotiatorResultHandler * rh, uint8_t lekv);

    void accept(czkSockAcceptor & sock, DHNegoType type);
    void exception(czkSockAcceptor & sock, DHNegoType type);
    
    void connected(czkSocket & sock, cDHNegotiatorInfo & negoInfo);
    void recv(czkSocket & sock, cDHNegotiatorInfo & negoInfo);
    void exception(czkSocket & socke, cDHNegotiatorInfo & negoInfo);
    void timeout(cDHNegotiatorInfo & negoInfo);
    void handleKey(cDHNegotiatorInfo & negoInfo, 
                   cKeyProviderCacheEntryData & entryData);
            
private:
    // copy constructor and assignment operator (disabled)
    cDHNegotiator(const cDHNegotiator &) : cAipComponent() { assert(0); };
    void operator=(const cDHNegotiator &) { assert(0); };
  
    void initSocket(czkSockConnector & sock);
    void initSocket(czkSockAcceptor & sock, DHNegoType type, uint16_t port);
    void reopenSocket(czkSockAcceptor & sock, DHNegoType type);

    cDHNegotiatorResultHandler * findResultHandler(DHNegoType type, uint8_t vers,
                                                   uint16_t port);
    void addResultHandler(DHNegoType type, uint8_t vers, uint16_t port,
                          cDHNegotiatorResultHandler * rhin);
    void removeResultHandler(DHNegoType type, uint8_t vers, uint16_t port);

    cDHNegotiatorInfo * findNegoInfo(FrEntIdExp & entity);
    void addNegoInfo(FrEntIdExp & entity, cDHNegotiatorInfo & negoInfo);
    void removeNegoInfo(FrEntIdExp & entity);

    void recvEntityId(czkSocket & sock, cDHNegotiatorInfo & negoInfo);
    void recvFirstDH(czkSocket & sock, cDHNegotiatorInfo & negoInfo);
    void recvSecondDH(czkSocket & sock, cDHNegotiatorInfo & negoInfo);

    void initDH(FrPktDifHel * dhDesc, bool sigCheck, czkSocket & sock,
                FrCryptSymAlg alg, uint8_t lekv = 0);
    int32_t getBacklog();
    int32_t getExchangeTimeout();
};

#endif
