#include "cDHNegotiatorInfo.h"

#define REV_ID "@(#) $Id: cDHNegotiatorInfo.cpp,v 1.2 2001/04/18 17:36:38 alainc Exp $"
#include <zkrev.h>




cDHNegotiatorInfo::cDHNegotiatorInfo() : state(lastDHNegoState),
                                         lekv(0), port(0), rh(NULL), 
                                         socket(NULL), timerId(NULL), dhDesc(NULL), 
                                         dhIdPkt(NULL), rxBytes(0), 
                                         keyHandler(NULL), sigVersion(0)
                                         
                                          
{ 
    memset(&entity, 0, sizeof(entity));
    memset(&dhIdPkt, 0, sizeof(dhIdPkt)); 
}

cDHNegotiatorInfo::~cDHNegotiatorInfo()
{
}

cDHNegotiatorInfo::cDHNegotiatorInfo(const cDHNegotiatorInfo &)
{
    ZKS_ASSERT(ZKS_DHNEGOTIATOR_MOD, 0);
}

void cDHNegotiatorInfo::operator=(const cDHNegotiatorInfo &)
{
    ZKS_ASSERT(ZKS_DHNEGOTIATOR_MOD, 0);
}

void cDHNegotiatorInfo::cleanup(czkCoreLoop * coreloop, cKeyProvider* keyProvider)
{
    const char * func = "cDHNegotiatorInfo::cleanup";
    const char * msg = NULL;

    try
    {
      if (coreloop == NULL) {
	CZKS::Throw(ZKS_DHNEGOTIATOR_MOD,ZKS_DHNEGOTIATOR_BADPARAM_ERR, func,
		    "Attempting to cleanup NULL coreloop");

      }
      if (keyProvider == NULL) {
	CZKS::Throw(ZKS_DHNEGOTIATOR_MOD,ZKS_DHNEGOTIATOR_BADPARAM_ERR, func,
		    "Attempting to cleanup NULL keyProvider");
      }

        if (keyHandler != NULL) {
            keyProvider->cancelKeyRequest(entity.name, 
                                          entity.type,
                                          FRCRYPT_KEYTYPE_SIGNING, sigVersion,
                                          keyHandler);
            keyHandler = NULL;
	}
        if (timerId != NULL)
        {
            msg = "Cleaning up timer";
            timerId->cancelTimer();
            delete timerId;
            timerId = NULL;
        }

        if (socket != NULL)
        {
            msg = "Cleaning up socket";

            int32_t rc;
            if ( (rc = coreloop->removeHandler(socket)) == ZKPR_FAILURE)
            {
                CZKS::Throw(ZKS_DHNEGOTIATOR_MOD, 
                            ZKS_DHNEGOTIATOR_CORELOOP_ERR,
                            func, 
                            "Couldn't un-register socket from core loop rc=%d", rc);
            }
            socket->close();
            delete socket;
            socket = NULL;
        }

        if (dhDesc != NULL)
        {
            msg = "Cleaning up D-H descriptor";

            CZKS::Exception e;
            int32_t rc;
            if ((rc = frPktDifHelFree(e.zkError(), &dhDesc)) == ZKS_FAILURE)
            {
                CZKS::Throw(ZKS_DHNEGOTIATOR_MOD, 
                            ZKS_DHNEGOTIATOR_DIFHEL_ERR,
                            func, 
                            "Couldn't free D-H descriptor rc=%d", rc);
            }
            dhDesc = NULL;
        }

        if (dhIdPkt != NULL)
        {
            delete dhIdPkt;
            dhIdPkt = NULL;
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

