//
// cDHNegotiatorInfo class definition
//
#ifndef __CDHNEGOINFO_H
#define __CDHNEGOINFO_H

#include "czkEventHandler.h"
#include "frEnt.h"
#include "cDHNegotiatorResultHandler.h"
#include "cDHNegotiatorKeyHandler.h"
#include "cKeyProvider.h"

// D-H negotiation states
typedef enum
{
    initiatorFirstConnectionPending,
    initiatorFirstWaitSecondMsg,
    initiatorFirstWaitSecondSigKey,
    initiatorFirstWaitEntityId,
    initiatorFirstWaitThirdMsg,
    initiatorFirstWaitThirdSigKey,
    recipientFirstWaitSecondMsg,
    lastDHNegoState                 // This is not a valid state
} DHNegoState;


class cDHNegotiatorInfo : public czkEventHandlerData
{
friend class cDHNegotiator;

private:
    DHNegoState                  state;  // DH negotiation state
    FrEntIdExp                   entity; // remote entity
    uint8_t                      lekv;   // Link encryption key version 
    uint16_t                     port;   // Acceptor socket local port
    cDHNegotiatorResultHandler * rh;     // Reference to result handler
    czkSocket                  * socket; // Reference to DH negotiation socket
    ZKPRNetAddr                  peerAddr; // Peer address
    czkTimerProxy              * timerId;// Reference to timer ID
    FrPktDifHel                  dhDesc; // DH negotiation descriptor
    char *                       dhIdPkt;   // DH negotiation ID requested by remote
    uint32_t                     rxBytes;// Number of received bytes in entity
    cDHNegotiatorKeyHandler *    keyHandler; 
    uint32_t                     sigVersion;
 
public:
    // constructor
    cDHNegotiatorInfo();

    // copy constructor
    cDHNegotiatorInfo(const cDHNegotiatorInfo & other);

    // assignment operator
    void operator=(const cDHNegotiatorInfo & other);

    // destructor
    virtual ~cDHNegotiatorInfo();

    // resource liberator
    void cleanup(czkCoreLoop * coreloop, cKeyProvider* keyProvider);
};
    
#endif
