#define REV_ID

#include "cDHNegotiator.h"
#include "cLinkManager.h"
#include "cPrivateKeys.h"

#include "zksini.h"

#include "elgparms.h"

#include <fstream>

#include <sys/poll.h>
#include <sys/wait.h>

namespace {
  
  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };

  const time_t UT_TIMEOUT = 3000;  // In milliseconds
  const time_t UT_RUNTIME = 10;    // In seconds
  
  const uint16_t UT_FREEDOM_PORT = 34567;
  const uint16_t UT_CLIENT_PORT = 34567;
  const uint16_t UT_AIP_PORT = 51106;

  const unsigned int UT_BACKLOG = 10;
  const unsigned int UT_EXCHANGETIMEOUT = 3;
  const size_t UT_BUFSZ = 16;

  const string UT_IPADDR = "10.16.128.187";
  // These should really be const strings.
  
  const char * UT_UAIPNAME = "uaipName";
  const char * UT_SYMALG = "BLOWFISH";

  // This is defined in aip-params.h, but it still won't compile when I include
  // that file, so I am redefining it locally here.  I believe it is probably
  // outdated and should be removed, but I'll leave that up to someone else.

  const char * AIP_IP_ADDRESS_DH = "AIP_IPAddressDH";

  FrCryptPubKey privkey;
  FrCryptPubKey myPubSig;
  FrCryptPubKey peerPrivSig;
  FrCryptPubKey peerPubSig;

  const size_t DH_FULL_SECRET_SIZE = BITS_TO_BYTES(2048);
  const size_t DH_HALF_SECRET_SIZE = BITS_TO_BYTES(256);

#pragma pack(1)

  struct freedom_dh_exchange_packet{
    uint8_t vers;
    uint8_t type;
    uint16_t pktlen;
    uint8_t symAlg;
    uint8_t lekv;
    uint16_t port;
    int64_t absTime;
    char halfsec[DH_FULL_SECRET_SIZE];
    char ent[FRENT_PORTABLE_SIZE];
    char sig[FRCRYPT_SIG_SIZE];
  };
  
#pragma pack()

}

class cUnitTestResultHandler : public cDHNegotiatorResultHandler
{
public:
    bool         exchangeDone;
    FrEntIdExp   exchangeEntity;
    DHNegoResult exchangeResult;
    uint8_t      exchangeLekv;
    SymKey       exchangeEncrKey;
    SymKey       exchangeDecrKey;

    void reportResult(FrEntIdExp &, DHNegoResult, uint8_t, SymKey, SymKey,
                      ZKPRNetAddr *);
};

void cUnitTestResultHandler::reportResult(FrEntIdExp & entity, 
                                          DHNegoResult result, uint8_t lekv,
                                          SymKey encrKey, SymKey decrKey,
                                          ZKPRNetAddr * peerAddr)
{

  if (result != DHNegoPass)
    cerr << " DH Negotiation failed" << endl;
  memcpy(&exchangeEntity, &entity, sizeof(FrEntIdExp));
  exchangeResult = result;
  exchangeLekv = lekv;
  exchangeEncrKey = encrKey;
  exchangeDecrKey = decrKey;

  exchangeDone = true;
  char basebuf[UT_BUFSZ];
  memset(basebuf, 'a', UT_BUFSZ);

  unsigned char nullivec[FRCRYPT_IVEC_SIZE];
  memset(nullivec, 0, FRCRYPT_IVEC_SIZE);
  if (!frCryptSymEncrypt(0, encrKey, nullivec, basebuf, UT_BUFSZ))
    cerr << "Encryption failed" << endl;
  
  czkSocket udpsock;
  if (udpsock.openUDPSocket() != ZKPR_SUCCESS)
    cerr << "Error opening UDP socket" << endl;
  if (udpsock.sendto(basebuf, UT_BUFSZ, 0, peerAddr, PR_INTERVAL_NO_TIMEOUT) != ZKPR_SUCCESS)
    cerr << "Error sending on UDP socket" << endl;
  else
    cerr << "Packet sent" << endl;
}


// A whole lot of stubbed methods follow.  They have to be defined because
// their classes can't be instantiated without them (so I understand).
// I have compressed them because they're just taking up space.
// If this really upsets anyone, come see me and we'll discuss abuses of
// inheritance.
// Also, I have removed all the useless comments.
// (i.e. // main, right before the main function)

cKeyProviderCacheEntryData::cKeyProviderCacheEntryData() {}
cKeyProviderCacheEntryData::~cKeyProviderCacheEntryData() {}
FrCryptPubKey & cKeyProviderCacheEntryData::getPubKey() { return peerPubSig; }

cKeyProviderCacheEntryData::cKeyProviderCacheEntryData( 
                                const cKeyProviderCacheEntryData& entryData)
{
  qry = entryData.qry;
  state = entryData.state;
  expires = entryData.expires; 
  expTime = entryData.expTime;
  replyLen = 0;
  reply = NULL; 
  keyInfo = entryData.keyInfo;
}

bool validKey = true;

FrKeyQryState cKeyProviderCacheEntryData::getState()
{
  if (validKey)
    return FRKEYQRY_ST_VALID;
  return FRKEYQRY_ST_INVALID;
}

FrKeyQryQueryExp cKeyProviderCacheEntryData::getQry() {

  FrKeyQryQueryExp query;
  query.keyType = FRCRYPT_KEYTYPE_SIGNING;
  return query;
}

cKeyProvider::cKeyProvider() {}
cKeyProvider::~cKeyProvider() {}
void cKeyProvider::shutdown() {}
bool cKeyProvider::validateConfig() { return true; }
bool cKeyProvider::initializePrivateData(czkCoreLoop *) { return true; }
void cKeyProvider::cleanupPrivateData() {}
void cKeyProvider::startup() {}
void cKeyProvider::getKeyNoWait(char *, FrEntIdType, FrPubKeyType, uint32_t,
                                cKeyProviderCacheEntryData &) {}
void cKeyProvider::cancelKeyRequest (char*, FrEntIdType, FrPubKeyType,
                                     uint32_t, cKeyProviderEventHandler*) {}

bool holdKey = false;
cKeyProviderEventHandler * savedKeyHandler = NULL;
cKeyProviderCacheEntryData * savedEntryData = NULL;

void cKeyProvider::makeKeyRequest (char*, FrEntIdType, FrPubKeyType, uint32_t, 
                                   cKeyProviderEventHandler* kh,
                                   cKeyProviderCacheEntryData& entryData)
{
    if (!holdKey)
    {
        kh->handleEvent(entryData);
    }
    else
    {
        savedKeyHandler = kh;
        savedEntryData = &entryData;
    }
}

cPrivateKeys::cPrivateKeys() {}
cPrivateKeys::~cPrivateKeys() {}
bool cPrivateKeys::validateConfig() { return true; }
bool cPrivateKeys::initializePrivateData(czkCoreLoop *) { return true; }
void cPrivateKeys::cleanupPrivateData() {}
void cPrivateKeys::startup() {}
void cPrivateKeys::getSigKey(uint32_t, FrCryptPubKey* key)
{
  FrPubKeyType type = FRCRYPT_KEYTYPE_SIGNING;
  FrEntIdExp ent;
  ent.type = FRENT_TYPE_AIP;
  size_t keysize = 1024;
  unsigned int keyvers = 1;
  time_t currentTime;
  time(&currentTime);
  currentTime += 365 * 24 * 60 * 60;
  ZkClock keyexp = zkClockFromTime(currentTime);
  
  if (!frCryptPubKeyNew(0, &privkey))
    exit(UT_FATAL);
  if (!frCryptPubKeySetType(0, privkey, type))
    exit(UT_FATAL);
  if (!frCryptPubKeySetSize(0, privkey, keysize))
    exit(UT_FATAL);
  if (!frCryptPubKeySetVersion(0, privkey, keyvers))
    exit(UT_FATAL);
  if (!frCryptPubKeySetExpiration(0, privkey, keyexp))
    exit(UT_FATAL);
  
  // The functions wrapped inside frCryptPubKeySetEntity() are failing
  // (incorrectly) and the subsequent exception causes a core dump.
  // So this is the hack for now.
  
  FrEntIdExp tempent;
  memset(&tempent, 0, sizeof(tempent));
  tempent.type = ent.type;
  strncpy(tempent.name, ent.name, sizeof(tempent.name));
  if (!frCryptPubKeySetEntity(0, privkey, &tempent))
    exit(UT_FATAL);
  if (!frCryptSigMakeSig(0, privkey))
    exit(UT_FATAL);
  if (!frCryptSigSetRandomKeys(0, privkey))
    exit(UT_FATAL);

  *key = privkey;
}

// I have to set this here because cNetInfoEntity::getFreedomPort(), which I
// have to stub, is defined in cNetInfoEntity.h, which I include.  So I
// covertly set the port to what I want.  For the record, I don't think this
// affects anything, but I want this to be done for good.

cNetInfoEntity::cNetInfoEntity()
{ netAddr.inet.port = czkNetAddr::HtoNs(UT_FREEDOM_PORT); }
cNetInfoEntity::~cNetInfoEntity() {}
cNetInfoEntity unitTestNetInfo;

cNetInfoProvider::cNetInfoProvider() {}
cNetInfoProvider::~cNetInfoProvider() {}
bool cNetInfoProvider::validateConfig() { return true; }
bool cNetInfoProvider::initializePrivateData(czkCoreLoop*) { return true; } 
void cNetInfoProvider::cleanupPrivateData() {}
void cNetInfoProvider::startup() {}
void cNetInfoProvider::shutdown() {}
const class cNetInfoEntity & cNetInfoProvider::getEntity(FrEntIdType,
                                                         char const *)
{ return unitTestNetInfo; }

cLinkManagerInfo unitTestLinkInfo;
cLinkManagerInfo::cLinkManagerInfo() {}
cLinkManagerInfo::~cLinkManagerInfo() {}
cLinkManagerInfo * cLinkManager::findLink(FrEntIdExp &)
{ return (&unitTestLinkInfo); }

cLinkManager::cLinkManager() {}
cLinkManager::~cLinkManager() {}
bool cLinkManager::validateConfig() { return true; }
bool cLinkManager::initializePrivateData(czkCoreLoop *) { return true; }
void cLinkManager::cleanupPrivateData() {}
void cLinkManager::startup() {}
void cLinkManager::shutdown() {}

FrAci cAciBin::getAci() { return 0; }
void cAciBin::returnAci(unsigned int) {}

GenericIdBin::~GenericIdBin() {}
GenericIdBin::GenericIdBin(unsigned long, unsigned long) {}
GenericId::GenericId() {}

cConfigProvider::cConfigProvider(CZKS::CmdLine &, CZKS::Ini &) {}
cConfigProvider::~cConfigProvider() {}
bool cConfigProvider::validateConfig() { return true; }
bool cConfigProvider::initializePrivateData(czkCoreLoop *) { return true; }
void cConfigProvider::cleanupPrivateData() {}
void cConfigProvider::startup() {}
void cConfigProvider::shutdown() {}

int32_t cConfigProvider::getOpt_int(const char *section, 
                                    const char *varname)
{
    if (strcmp(section, UAIP_SECTION_NAME) == 0)
    {
        if (strcmp(varname, DH_CONNECTION_BACKLOG) == 0)
        {
            return(UT_BACKLOG);
        }
        else if (strcmp(varname, AIP_FREEDOM_PORT) == 0)
        {
            return(UT_FREEDOM_PORT);
        }
        else if (strcmp(varname, DH_EXCHANGE_TIMEOUT) == 0)
        {
            return(UT_EXCHANGETIMEOUT);
        }
        else
        {
            exit(UT_FATAL);
        }
    }
    else
    {
        exit(UT_FATAL);
    }
}

int32_t cConfigProvider::getOpt_int(const char *section, const char *varname,
                                    int32_t)
{ return getOpt_int(section, varname); }

string cConfigProvider::getOpt_string(const char *section, const char *varname,
                                      string)
{ return getOpt_string(section, varname); }

string cConfigProvider::getOpt_string(const char *section, const char *varname)
{
    czkNetAddr netAddr;

    if (strcmp(section, UAIP_SECTION_NAME) == 0)
    {
        if (strcmp(varname, AIP_ENTITY_NAME) == 0)
        {
            string uaipName = UT_UAIPNAME;
            return (uaipName);
        }
        else if (strcmp(varname, AIP_IP_ADDRESS_DH) == 0)
        {
            ZKPRNetAddr addr;
            char hostName[256];
            gethostname(hostName, 256);
            netAddr.getHostbyName(hostName, 256);
            netAddr.enumerateHostEnt(0, 1000, &addr);
            struct in_addr ipAddr;
            ipAddr.s_addr = addr.inet.ip;
            char * hostIp = inet_ntoa(ipAddr);
            string hostIpAddr = hostIp;
            return hostIpAddr;
        }
        else if (strcmp(varname, DH_SYM_ALG) == 0)
        {
            string algo = UT_SYMALG;
            return algo;
        }
        else
        {
            exit(UT_FATAL);
        }
    }
    else
    {
        exit(UT_FATAL);
    }
}

class DHUnitTest {
public:
  DHUnitTest();
  ~DHUnitTest();
  void Run(size_t numloops = 1);
private:
  ZkError err;
  struct sockaddr_in clientaddr, aipaddr;
  list<pid_t> pidlist;
  pid_t svrpid;
  BIGNUM * SendMyHalfSecret(int dh_fd, FrCryptSig myPrivSig = 0);
  BIGNUM * ReceiveTheirHalfSecret(int dh_fd);
  bool ValidateExchange(BIGNUM * firsthalf, BIGNUM * secondhalf, int udpsock);
  void StartDHServer();
  bool BadStartup(size_t numloops = 1);
  bool BadHandlers(size_t numloops = 1);
  bool Client(size_t numloops = 1);
  bool InitiatorAIP(size_t numloops = 1);
  bool RecipientAIP(size_t numloops = 1);
  bool Client2Attempts(size_t numloops = 1);
  bool ClientAbort(size_t numloops = 1);
  bool ClientSendAbort(size_t numloops = 1);
};

DHUnitTest::DHUnitTest() {

  zkErrorNew(&err);
  frCryptInit(err, 0);
  memset(&clientaddr, 0, sizeof(clientaddr));
  memset(&aipaddr, 0, sizeof(aipaddr));
  clientaddr.sin_addr.s_addr = inet_addr(UT_IPADDR.c_str());
  aipaddr.sin_addr.s_addr = inet_addr(UT_IPADDR.c_str());
  clientaddr.sin_port = htons(UT_CLIENT_PORT);
  aipaddr.sin_port = htons(UT_AIP_PORT);
}

DHUnitTest::~DHUnitTest() {
  frCryptFinal(err, 0);
  zkErrorFree(&err);
}

void DHUnitTest::Run(size_t numloops) {
  
  svrpid = fork();
  if (svrpid < 0)
    exit(UT_FATAL);
  if (svrpid == 0)
    StartDHServer();

  sleep(1);
  pid_t pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0) {
    if (Client(numloops)) {
      cerr << "Client Passed" << endl;
      exit(UT_PASS);
    }
    else {
      cerr << "Client Failed" << endl;
      exit(UT_FATAL);
    }
  }
  pidlist.push_back(pid);

  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0) {
    if (Client(numloops)) {
      cerr << "Client #2 Passed" << endl;
      exit(UT_PASS);
    }
    else {
      cerr << "Client #2 Failed" << endl;
      exit(UT_FATAL);
    }
  }
  pidlist.push_back(pid);

  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0) {
    if (ClientAbort(numloops)) {
      cerr << "Client Abort Passed" << endl;
      exit(UT_PASS);
    }
    else {
      cerr << "Client Abort Failed" << endl;
      exit(UT_FATAL);
    }
  }
  pidlist.push_back(pid);

  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0) {
    if (ClientSendAbort(numloops)) {
      cerr << "Client Send Abort Passed" << endl;
      exit(UT_PASS);
    }
    else {
      cerr << "Client Send Abort Failed" << endl;
      exit(UT_FATAL);
    }
  }
  pidlist.push_back(pid);

  /*
  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0) {
    if (Client2Attempts(numloops)) {
      cerr << "Client 2 Attempts Passed" << endl;
      exit(UT_PASS);
    }
    else {
      cerr << "Client 2 Attempts Failed" << endl;
      exit(UT_FATAL);
    }
  }
  pidlist.push_back(pid);

  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0)
    InitiatorAIP(numloops);
  pidlist.push_back(pid);

  pid = fork();
  if (pid < 0)
    exit(UT_FATAL);
  if (pid == 0)
    RecipientAIP(numloops);
  pidlist.push_back(pid);
  */
  struct timeval start, now;
  memset(&start, 0, sizeof(start));
  gettimeofday(&start, 0);
  for (;;) {

    for (size_t i = 0; i < pidlist.size(); i++) {
      pid_t nextpid = pidlist.front();
      pidlist.pop_front();
      pid_t ret_val = waitpid(nextpid, 0, WNOHANG);
      
      if (ret_val == 0)
        pidlist.push_back(nextpid);
      else
        cerr << "waitpid returned:  " << ret_val << endl;
    }
    if (pidlist.size() == 0)
      break;

    memset(&now, 0, sizeof(now));
    gettimeofday(&now, 0);
    if ((now.tv_sec - start.tv_sec) > UT_RUNTIME) {
      cerr << "Wait loop timed out" << endl;
      break;
    }

    usleep(10);
  }

  // Kill anyone left...
  for (size_t i = 0; i < pidlist.size(); i++) {
    pid_t nextpid = pidlist.front();
    pidlist.pop_front();
    kill(nextpid, SIGKILL);
    waitpid(nextpid, 0, WNOHANG);
  }
  kill(svrpid, SIGKILL);
  waitpid(svrpid, 0, WNOHANG);
}

BIGNUM * DHUnitTest::SendMyHalfSecret(int dh_fd, FrCryptSig myPrivSig) {

  struct freedom_dh_exchange_packet sendPkt;

  unsigned char hbuf[DH_HALF_SECRET_SIZE];    
  if (!zkRandGetData(err, reinterpret_cast<char*>(hbuf), DH_HALF_SECRET_SIZE)) {
    cerr << "zkRandGetData() failed" << endl;
    exit(UT_FATAL);
  }
  
  BIGNUM * firsthalf = BN_bin2bn(hbuf, DH_HALF_SECRET_SIZE, 0);
  BIGNUM * temphalfsec;
  BN_CTX * bn_ctx;
  
  if (!(bn_ctx = BN_CTX_new())) {
    cerr << "BN_CTX_new() failed" << endl;
    exit(UT_FATAL);
  }
  
  if (!(temphalfsec = BN_new())) {
    cerr << "BN_new() failed" << endl;
    exit(UT_FATAL);
  }
  
  if (!BN_mod_exp(temphalfsec, &frBn_ElG_g, firsthalf, &frBn_ElG_p_2048,
                  bn_ctx)) {
    cerr << "BN_mod_exp() failed" << endl;
    exit(UT_FATAL);
  }
  BN_CTX_free(bn_ctx);
  
  size_t offset = 0;
  if (static_cast<size_t>(BN_num_bytes(temphalfsec)) !=
      sizeof(sendPkt.halfsec)) {
    offset = sizeof(sendPkt.halfsec) - BN_num_bytes(temphalfsec);
    if (offset > sizeof(sendPkt.halfsec)) {
      cerr << "Offset is bad" << endl;
      exit(UT_FATAL);
    }
    memset(sendPkt.halfsec, 0, offset);
  }
      
  if (!BN_bn2bin(temphalfsec,
                 reinterpret_cast<unsigned char*>(sendPkt.halfsec + offset))) {
    cerr << "BN_bn2bin() failed" << endl;
    exit(UT_FATAL);
  }
  BN_clear_free(temphalfsec);

  // Initialize the sendPkt structure.
  
  sendPkt.vers = static_cast<uint8_t>(1);
  sendPkt.type = FRPKTTYPE_DIFHEL;
  sendPkt.pktlen = htons(sizeof(sendPkt));
  sendPkt.symAlg = FRCRYPT_SYMALG_BLOWFISH;
  sendPkt.lekv = 0;
  sendPkt.port = htons(UT_CLIENT_PORT);
  sendPkt.absTime = zkClockToSeconds(zkClockToNbo(zkClockNow()));

  // If a private signature key was passed in, use it to sign the DH Packet.

  if (myPrivSig) {
    
    if (!frCryptPubKeyGetEntity(err, myPrivSig,
                                reinterpret_cast<FrEntIdExp*>(&(sendPkt.ent)))) {
      cerr << "frCryptPubKeyGetEntity() failed" << endl;
      exit(UT_FATAL);
    }
    
    if (!frCryptSigSignData(err, myPrivSig, &sendPkt,
                            (sizeof(sendPkt) - sizeof(sendPkt.sig)),
                            &(sendPkt.sig), sizeof(sendPkt.sig))) {
      cerr << "frCryptSigSignData() failed" << endl;
      exit(UT_FATAL);
    }
  }
  else {

    // If there is no signature key, specify that the user is anonymous
    
    memset(sendPkt.ent, 0, sizeof(sendPkt.ent));
    sendPkt.ent[0] = FRENT_TYPE_ANON;
  }
  
  // A simple send(), fails if anything goes wrong.
  
  if (send(dh_fd, reinterpret_cast<const void*>(&sendPkt),
           sizeof(sendPkt), 0) != sizeof(sendPkt)) {
    cerr << "send() failed:  " << strerror(errno) << endl;
    exit(UT_FATAL);
  }
  return firsthalf;
}

BIGNUM * DHUnitTest::ReceiveTheirHalfSecret(int dh_fd) {

  struct freedom_dh_exchange_packet recvPkt;  
  memset(&recvPkt, 0, sizeof(recvPkt));
  
  struct pollfd dhfdset = {dh_fd, POLLIN, 0};
  
  int pollret = poll(&dhfdset, 1, UT_TIMEOUT);

  if (pollret < 0) {
    cerr << "poll() failed:  " << strerror(errno) << endl;
    exit(UT_FATAL);
  }

  else if (pollret == 0) {
    cerr << "poll() timed-out" << endl;
    exit(UT_FATAL);
  }

  else if (pollret != 1) {
    cerr << "poll() returned incorrect value" << endl;
    exit(UT_FATAL);
  }

  // A simple recv(), fails if anything goes wrong.
  int n = recv(dh_fd, &recvPkt, sizeof(recvPkt), 0);
  if (n == -1) {
    cerr << "recv() failed:  " << strerror(errno) << endl;
    exit(UT_FATAL);
  }

  if (n != sizeof(recvPkt)) {
    cerr << "recv() returned incorrect buffer size:  "<< n << endl;
    exit(UT_FATAL);
  }

  // Extract and verify the field values.

  if (recvPkt.vers != static_cast<uint8_t>(1)) {
    cerr << "Version is bad" << endl;
    exit(UT_FATAL);
  }

  if (recvPkt.type != FRPKTTYPE_DIFHEL) {
    cerr << "Packet type is bad" << endl;
    exit(UT_FATAL);
  }
    
  if (ntohs(recvPkt.pktlen) != sizeof(recvPkt)) {
    cerr << "Packet length is bad" << endl;
    exit(UT_FATAL);
  }
    
  // Technically, we are supposed to negotiate this.  But BLOWFISH is
  // ubiquitous currently, so reject any other type.

  if (recvPkt.symAlg != FRCRYPT_SYMALG_BLOWFISH) {
    cerr << "Symmetric algorithm is bad" << endl;
    exit(UT_FATAL);
  }
    
  // Note that it is not an error if this field is different from ours

  if (recvPkt.lekv != 0) {
    cerr << "lekv is bad" << endl;
    exit(UT_FATAL);
  }

  //  Calculate the time offset between our local time and the remote
  //  system's time.  The difference is relative to us, so if our time is
  //  ahead of the remote time, the offset will be negative.
  
  //  Let's worry about this later.
  //  ZkClock remTime = zkClockFromNbo(zkClockFromSeconds(recvPkt.absTime));
  //  ZkClock remTimeOffset = zkClockDiffSat(remTime, zkClockNow());
  //  if (remTimeOffset > MAX_TIME_OFFSET)
  //  throw TestModuleException("Remote time is too different from ours.");

  // Get the secondhalf to the full secret, the key is ours!
  
  return BN_bin2bn(reinterpret_cast<unsigned char*>(recvPkt.halfsec),
                   sizeof(recvPkt.halfsec), 0);  
}

bool DHUnitTest::ValidateExchange(BIGNUM * firsthalf, BIGNUM * secondhalf, int udpsock) {

  BIGNUM * fullsecret;
  BN_CTX * bn_ctx;
  unsigned char secBuf[BITS_TO_BYTES(2048)];

  if (!(bn_ctx = BN_CTX_new())) {
    cerr << "BN_CTX_new() failed" << endl;
    return false;
  }
  
  if (!(fullsecret = BN_new())) {
    cerr << "BN_new() failed" << endl;
    return false;
  }
  
  if (!BN_mod_exp(fullsecret, secondhalf, firsthalf, &frBn_ElG_p_2048, bn_ctx)) {
    cerr << "BN_mod_exp() failed" << endl;
    return false;
  }

  BN_CTX_free(bn_ctx);
  size_t offset = 0;

  if (static_cast<size_t>(BN_num_bytes(fullsecret)) != sizeof(secBuf)) {
    offset = sizeof(secBuf) - BN_num_bytes(fullsecret);
    if (offset > sizeof(secBuf)) {
      cerr << "offset is bad" << endl;
      return false;
    }
    memset(secBuf, 0, offset);
  }
      
  if (!BN_bn2bin(fullsecret, secBuf + offset)) {
    cerr << "BN_bn2bin() failed" << endl;
    return false;
  }
  BN_clear_free(firsthalf);
  BN_clear_free(secondhalf);
  BN_clear_free(fullsecret);
  
  //  The low-order bit is known, so we must not use it to generate a key.
  //  Just to be safe we'll use a strict subset of the buffer beginning at an
  //  offset of one.

  FrCryptSym symkey;

  if (!frCryptSymNew(err, &symkey)) {
    cerr << "frCryptSymNew() failed" << endl;
    return false;
  }
  if (!frCryptSymSetAlg(err, symkey, FRCRYPT_SYMALG_BLOWFISH)) {
    cerr << "frCryptSymSetAlg() failed" << endl;
    return false;
  }
  if (!frCryptSymKeyFromData(err, symkey, secBuf + 1, sizeof(secBuf) - 2)) {
    cerr << "frCryptSymKeyFromData() failed" << endl;
    return false;
  }

  struct pollfd mysock;
  memset(&mysock, 0, sizeof(mysock));
  mysock.fd = udpsock;
  mysock.events = POLLIN;
  int n = poll(&mysock, 1, UT_TIMEOUT);
  if (n == 0) {
    cerr << "poll() timed out" << endl;
    return false;
  }
  if (n == -1) {
    cerr << "poll() returned error:  " << strerror(errno) << endl;
    return false;
  }

  if (n != 1) {
    cerr << "poll() returned bad value" << endl;
    return false;
  }
  char recvbuf[UT_BUFSZ], basebuf[UT_BUFSZ];
  memset(basebuf, 'a', UT_BUFSZ);
  n = recv(udpsock, recvbuf, UT_BUFSZ, 0);
  if (n != static_cast<int>(UT_BUFSZ)) {
    cerr << "recv() returned bad value:  " << strerror(errno) << endl;
    return false;
  }

  unsigned char nullivec[FRCRYPT_IVEC_SIZE];
  memset(nullivec, 0, FRCRYPT_IVEC_SIZE);
  if (!frCryptSymDecrypt(err, symkey, nullivec, recvbuf, UT_BUFSZ)) {
    cerr << "frCryptSymDecrypt() failed" << endl;
    return false;
  }
  
  return !memcmp(recvbuf, basebuf, UT_BUFSZ);
}

void DHUnitTest::StartDHServer() {

  try {
    cAipComponentManager mgr;
    czkCoreLoop coreloop;

    CZKS::CmdLine  m_cmdline;
    CZKS::Ini      m_ini;
    std::ifstream file("./freedom.conf");    
    file >> m_ini;

    cConfigProvider config(m_cmdline, m_ini);
    cKeyProvider keys;
    cStatsHarvestor stats;
    cDHNegotiator dhn;
    
    mgr.add(ZKS_CONFIGPROVIDER_MOD, &config);
    mgr.add(ZKS_STATSHARVESTOR_MOD, &stats);
    mgr.add(ZKS_KEYPROVIDER_MOD, &keys);
    mgr.add(ZKS_DHNEGOTIATOR_MOD, &dhn);
    
    stats.validateConfig();
    keys.validateConfig();
    dhn.validateConfig();
    mgr.initAllComponents(&coreloop);
    
    mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, false);      
    mgr.startComponent(ZKS_KEYPROVIDER_MOD, false);
    mgr.startComponent(ZKS_STATSHARVESTOR_MOD, false);
    mgr.startComponent(ZKS_DHNEGOTIATOR_MOD, false);
    dhn.waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);

    cUnitTestResultHandler clihnd, aiphnd;
    clihnd.setVersion(1);
    aiphnd.setVersion(1);
    clihnd.setType(recipientFirstUnauthenticated);
    aiphnd.setType(initiatorFirstAuthenticated);
    clihnd.setPort(UT_CLIENT_PORT);
    aiphnd.setPort(UT_AIP_PORT);

    dhn.registerHandler(&clihnd);
    dhn.registerHandler(&aiphnd);

    coreloop.runEventLoop();
  }
  catch(exception & e) {
    cerr << "StartDHServer() threw exception:  " << e.what() << endl;
    exit(UT_FATAL);
  }
  catch(...) {
    cerr << "StartDHServer() threw exception, exiting" << endl;
    exit(UT_FATAL);
  }
}

bool DHUnitTest::Client(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
      cerr << "setsockopt() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&clientaddr),
                sizeof(clientaddr)) < 0) {
      cerr << "Failed to connect to:  "
           << inet_ntoa(clientaddr.sin_addr) << " on port:  "
           << ntohs(clientaddr.sin_port) << " with error:  "
           << strerror(errno) << endl;
      return false;
    }

    BIGNUM * fh = SendMyHalfSecret(dh_fd, 0);
    BIGNUM * sh = ReceiveTheirHalfSecret(dh_fd);

    struct sockaddr_in myaddr;
    socklen_t addrlen = sizeof(myaddr);
    if (getsockname(dh_fd, reinterpret_cast<struct sockaddr*>(&myaddr), &addrlen) != 0) {
      cerr << "getsockname() failed:  " << strerror(errno) << endl;
      return false;
    }
    int recvsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (recvsock < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }
    if (bind(recvsock, reinterpret_cast<const struct sockaddr*>(&myaddr), addrlen) != 0) {
      cerr << "bind() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (!ValidateExchange(fh, sh, recvsock)) {
      cerr << "Exchange invalid" << endl;
      return false;
    }
    if (close(dh_fd) != 0) {
      cerr << "close() failed:  " << strerror(errno) << endl;
      return false;
    }
  }
  return true;
}

bool DHUnitTest::InitiatorAIP(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
      exit(UT_FATAL);

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET,SO_REUSEADDR, &on, sizeof(on)) < 0) 
      exit(UT_FATAL);

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&aipaddr),
                sizeof(aipaddr)) < 0)
      exit(UT_FATAL);
    BIGNUM * fh = SendMyHalfSecret(dh_fd, 0);
    BIGNUM * sh = ReceiveTheirHalfSecret(dh_fd);
    if (!ValidateExchange(fh, sh, dh_fd))
      return false;
    if (close(dh_fd) != 0)
      exit(UT_FATAL);
  }
  return true;
}

bool DHUnitTest::RecipientAIP(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
      exit(UT_FATAL);

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET,SO_REUSEADDR, &on, sizeof(on)) < 0) 
      exit(UT_FATAL);

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&aipaddr),
                sizeof(aipaddr)) < 0)
      exit(UT_FATAL);
    BIGNUM * fh = SendMyHalfSecret(dh_fd, 0);
    BIGNUM * sh = ReceiveTheirHalfSecret(dh_fd);
    if (!ValidateExchange(fh, sh, dh_fd))
      return false;
    if (close(dh_fd) != 0)
      exit(UT_FATAL);
  }
  return true;
}

bool DHUnitTest::Client2Attempts(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET,SO_REUSEADDR, &on, sizeof(on)) < 0) {
      cerr << "setsockopt() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&clientaddr),
                sizeof(clientaddr)) < 0) {
      cerr << "connect() failed:  " << strerror(errno) << endl;
      return false;
    }
    cerr << "Before first send" << endl;
    BIGNUM * fh = SendMyHalfSecret(dh_fd, 0);
    cerr << "Before second send" << endl;
    usleep(10);
    BIGNUM * fh0 = SendMyHalfSecret(dh_fd, 0);
    cerr << "Before freeing BIGNUM" << endl;
    BN_clear_free(fh0);
    cerr << "Before receive" << endl;
    BIGNUM * sh = ReceiveTheirHalfSecret(dh_fd);

    struct sockaddr_in myaddr;
    socklen_t addrlen = sizeof(myaddr);
    if (getsockname(dh_fd, reinterpret_cast<struct sockaddr*>(&myaddr), &addrlen) != 0) {
      cerr << "getsockname() failed:  " << strerror(errno) << endl;
      return false;
    }
    int recvsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (recvsock < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }
    if (bind(recvsock, reinterpret_cast<const struct sockaddr*>(&myaddr), addrlen) != 0) {
      cerr << "bind() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (!ValidateExchange(fh, sh, recvsock)) {
      cerr << "Exchange invalid" << endl;
      return false;
    }
    if (close(dh_fd) != 0) {
      cerr << "close() failed:  " << strerror(errno) << endl;
      return false;
    }
  }
  return true;
}

bool DHUnitTest::ClientAbort(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET,SO_REUSEADDR, &on, sizeof(on)) < 0) {
      cerr << "setsockopt() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&clientaddr),
                sizeof(clientaddr)) < 0) {
      cerr << "connect() failed:  " << strerror(errno) << endl;
      return false;
    }
    if (close(dh_fd) != 0) {
      cerr << "close() failed:  " << strerror(errno) << endl;
      return false;
    }
  }
  return true;
}

bool DHUnitTest::ClientSendAbort(size_t numloops) {

  for (size_t i = 0; i < numloops; i++) {
    int dh_fd;
    if ((dh_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
      cerr << "socket() failed:  " << strerror(errno) << endl;
      return false;
    }

    int on = 1;
    if (setsockopt(dh_fd, SOL_SOCKET,SO_REUSEADDR, &on, sizeof(on)) < 0) {
      cerr << "setsockopt() failed:  " << strerror(errno) << endl;
      return false;
    }

    if (connect(dh_fd, reinterpret_cast<struct sockaddr*>(&clientaddr),
                sizeof(clientaddr)) < 0) {
      cerr << "connect() failed:  " << strerror(errno) << endl;
      return false;
    }

    BIGNUM * fh = SendMyHalfSecret(dh_fd, 0);
    BN_clear_free(fh);
    if (close(dh_fd) != 0) {
      cerr << "close() failed:  " << strerror(errno) << endl;
      return false;
    }
  }
  return true;
}

int main(int, char **) {

  DHUnitTest dh_ut;

  dh_ut.Run(1);

  return UT_PASS;
}
