#ifndef _cKeyProvider

#define _cKeyProvider

// class cKeyProvider 
//
// KeyProvider thread will instantiate this class and run startup() method
//
// other component threads will call one of makeKeyRequest, GetKeyNoWait or 
//   GetKeyWait to get keys.
//

#include "frKeyQry.h"
#include "frEnt.h"
#include "frCrypt.h"

#include "czkCoreLoop.h"
#include "czkNetAddr.h"
#include "czkSockCoDgram.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"
#include "cConfigProvider.h"

#include "aip-modules.h"

#include "cKeyProviderCacheEntryData.h"
#include "cKeyProviderEventHandler.h"

#include "cStatsHarvestorPktData.h"

#define KeyProviderSocketReceiveBufferSize 64*1024
#define KeyProviderSocketSendBufferSize    64*1024

class cKeyProvider : public cAipComponent {

  friend class cKeyProviderEventHandlerIO;

private:

  czkSockCoDgram socket;
  class cKeyProviderCache* keyCache;

  class cKeyProviderEventHandlerDataIO* IOEvent;

  class cKeyProviderEventHandlerDataIO* pruneTimeoutEvent;

  class cKeyProviderEventHandlerIO* IOEventHandler;
  class cKeyProviderEventHandlerIO* keyRequestTimeoutEventHandler;
  
  class czkTimerProxy* pruneTimerProxy; 

  class cConfigProvider* pConfigProvider; 
  class cStatsHarvestor* pStatsHarvestor; 

  //KeyProvider stats
  cStatsHarvestorPktData* kpStats;  // stats "counted" by the KeyProvider 

  //copy constructor (disabled)
  cKeyProvider( const cKeyProvider& c): cAipComponent(c) { assert(0); };

  //assignment operator (disabled)
  cKeyProvider& operator=( const cKeyProvider&) { assert(0); };

public:

  // constructor
  cKeyProvider();

  // destructor
  virtual ~cKeyProvider();

  // methods to get config points
  int32_t getPruneTime();
  int32_t getKeyRequestTimeoutVal();
  int32_t getMaxRetry();

  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);

  // cleanup private data members
  void cleanupPrivateData();
 
  // startup
  void startup();

  //shutdown (nothing to do for this component)
  void shutdown();

  // connect to the KQD server
  void connectKQD();

  // send qry packet to KQD
  void sendQry(FrKeyQryQueryExp& pqry);

  // make a Key Request, optionally attach event handler
  void makeKeyRequest ( char* entity, 
                        FrEntIdType entity_type, 
                        FrPubKeyType key_type, 
                        uint32_t key_version, 
	      	        cKeyProviderEventHandler* eventhandler,
                        cKeyProviderCacheEntryData& entryData);

  // cancel a key request (remove eventhandler)
  void cancelKeyRequest( char* entity, 
                         FrEntIdType entity_type, 
                         FrPubKeyType key_type, 
                         uint32_t key_version, 
	                 cKeyProviderEventHandler* eventhandler);

  // make a Key Request, optionally attach event handler  
  void makeKeyRequest ( FrKeyQryQueryExp pqry,
                        cKeyProviderEventHandler* eventhandler,
                        cKeyProviderCacheEntryData& entryData);

  // make a Key Request - return entry right away
  void getKeyNoWait( char* entity, 
                     FrEntIdType entity_type,
                     FrPubKeyType key_type, 
                     uint32_t key_version,
                     cKeyProviderCacheEntryData& entryData);

  // make a Key Request - return entry when available (or timeout expires)
  void getKeyWait( char* entity, 
                   FrEntIdType entity_type, 
                   FrPubKeyType key_type, 
                   uint32_t key_version, 
                   ZKPRIntervalTime timeout,
                   cKeyProviderCacheEntryData& entryData);
};

#endif
