#ifndef _cKeyProviderCacheEntryData

#define _cKeyProviderCacheEntryData

#include "frEnt.h"
#include "frCrypt.h"
#include "frKeyQry.h"
#include "zkClock.h"

#include "aip-modules.h"
#include "czknspr.h"

class cKeyProvider;
class cKeyProviderCache;

typedef struct cache_key_info CacheKeyInfo;

#ifndef KEYPROVIDERUNITTEST

#define MINUTE(n) ((n) * 60)
#define HOUR(n) (MINUTE(n) * 60)

#else

#warning "***** using bogus cache expiration times for testing *****"
#define MINUTE(n) (n)
#define HOUR(n) (n)

#endif

//
// class cKeyProviderCacheKeyInfo to encapsulate the keyInfo data
//
class cKeyProviderCacheKeyInfo { 

 public:

  CacheKeyInfo keyInfo; 
  uint32_t refcount;

  cKeyProviderCacheKeyInfo() 
  {
    refcount = 1;
    keyInfo.key = NULL;
  }

  ~cKeyProviderCacheKeyInfo()
  {
    CZKS::Exception e;
    if (keyInfo.key) {
      frCryptPubKeyFree(e.zkError(), &keyInfo.key);
    }
  }

  void incRefcount() { refcount++; }
  uint32_t decRefcount() 
  {

    refcount--;
    return refcount;
  }
     
};

//
// class cKeyProviderCacheEntryData encapsulate the data in a key cache entry
//
class cKeyProviderCacheEntryData { 

private:

  FrKeyQryQueryExp qry; // the query 
  FrKeyQryState state;  // state of the query
  uint32_t expires;     // expiration date of key (for convenience) 
  ZkClock expTime;      // cache entry expiration time 
  size_t replyLen;      // full length of data, when `reply' is in use. 
  void* reply;          // the reply packet
  cKeyProviderCacheKeyInfo* pkeyInfo;               //the key info

public:

  //constructor
  cKeyProviderCacheEntryData();

  //destructor
  virtual ~cKeyProviderCacheEntryData();

  //copy constructor
  cKeyProviderCacheEntryData( const cKeyProviderCacheEntryData& entryData);

  //assignment operator
  cKeyProviderCacheEntryData& operator=( const cKeyProviderCacheEntryData& entryData);

  //return the qry
  FrKeyQryQueryExp getQry();

  //set the qry
  void setQry(FrKeyQryQueryExp& newqry);

  //return the state
  FrKeyQryState getState();

  //set the state
  void setState(FrKeyQryState newstate);

  //return the public key
  FrCryptPubKey& getPubKey();

  //return the key info structure
  CacheKeyInfo& getKeyInfo();

  //set the key info structure
  void setKeyInfo(CacheKeyInfo& newkeyinfo);

  //set the reply packet 
  void setReply( void* newreply, size_t rl);

  //set the key's expiry time (not used yet!)
  void setExpires(int32_t expsec);

  //get the key's expiry date (not used yet!)
  int32_t getExpires();

  //set the key's expiry time from the cache
  void setExpTime();

  //return if the key has expired from the cache
  bool expired();

  //build a query for the parent key
  void formParentKeyRequest( FrKeyQryQueryExp&);

  //validate the key 
  void validate( cKeyProviderCacheEntryData parent);
}; 

#endif
