#ifndef _cKeyProviderCache

#define _cKeyProviderCache

#include <map>
#include <ext/hash_map>

#include "frEnt.h"
#include "frCrypt.h"
#include "frKeyQry.h"

#include "aip-modules.h"

#include "czkLock.h"
#include "cKeyProviderCacheEntry.h"

//"equal" function for the STL hash_map definition
struct eqqry
{
  bool operator()(const FrKeyQryQueryExp s1, const FrKeyQryQueryExp s2) const
  {
    bool rc = (memcmp(&s1, &s2, sizeof(s1)) == 0);
    return rc;
  }
};

//"hash" function for the STL hash_map definition
template <class T> struct qryhash : unary_function<T,size_t> {
 public:

  size_t operator()(const T& key) const; 
}; 

//hash_map definition: key is FrKeyQryQueryExp, 
//                     data is a pointer to cKeyProviderCacheEntry
//
typedef hash_map<const FrKeyQryQueryExp, cKeyProviderCacheEntry*, qryhash<FrKeyQryQueryExp>, eqqry> KeyCacheHashMap;

//
// Class cKeyProviderCache encapsulates the key cache 
// (which is a STL hash-map with 
//      key = (entity-name + entity-type + key-type + key-version)
//
class cKeyProviderCache { 

  friend class cKeyProviderScopeLock;

private:

   KeyCacheHashMap cache;

   czkLock hashlock;

  //copy constructor (disabled)
  cKeyProviderCache( const cKeyProviderCache&) { assert(0); };

  //assignment operator (disabled)
  cKeyProviderCache& operator=( const cKeyProviderCache&) { assert(0); };

public:

   //constructor  
   cKeyProviderCache() {};

   //destructor
   virtual ~cKeyProviderCache() { cleanup(); };

   // open the locks
   void init();

   // close the locks
   void cleanup();

   // add a new KeyCacheEntry (pointer) to the cache using key "qry"
   void add(FrKeyQryQueryExp qry, cKeyProviderCacheEntry* entry);

   // returns a KeyCacheEntry (pointer) belonging to key "qry"
   cKeyProviderCacheEntry* get(FrKeyQryQueryExp query);

   // deletes the KeyCacheEntry belonging to key "qry"
   void remove(FrKeyQryQueryExp query);

   // prune expired & invalid keys from the cache
   void prune();

   // add the master key to the cache
   void addMaster();
};


#endif
