#ifndef _cKeyProviderCacheEntry

#define _cKeyProviderCacheEntry

#include <list>

#include "czknspr.h"
#include "aip-modules.h"

#include "czkLock.h"
#include "czkTimerProxy.h"
#include "cKeyProviderCacheEntryData.h"
#include "cKeyProviderEventHandler.h"

//
// class cKeyProviderCacheEntry encapsulate one entry in the key cache
// (inherited from cKeyProviderCacheEntryData
//
// in addition to the key info,
// contains an STL list of eventhandlers to call when
//   the key is validated (or invalidated)
//
class cKeyProviderCacheEntry : public cKeyProviderCacheEntryData {

  friend class cKeyProviderScopeLock;

private:
  typedef list<cKeyProviderEventHandler*> KeyProviderEventHandlerList;

  //list of event handlers attached to this entry
  KeyProviderEventHandlerList ehlist;

  //pointer to the TimerProxy to cancel the timeout event
  czkTimerProxy* pKeyRequestTimerProxy;

  //lock object
  czkLock entryLock;
 
  //copy constructor (disabled)
  cKeyProviderCacheEntry( const cKeyProviderCacheEntry& d) 
  : cKeyProviderCacheEntryData(d) { assert(0); };

  //assignment operator (disabled)
  cKeyProviderCacheEntry& operator=( const cKeyProviderCacheEntry&) { assert(0); };

public:

  //constructor
  cKeyProviderCacheEntry() { pKeyRequestTimerProxy = NULL;  }

  //destructor
  virtual ~cKeyProviderCacheEntry() { };

  //initialize lock
  void init();

  //close lock
  void cleanup();

  //validate the signature of this key's parent
  void validateParentSig( cKeyProviderCacheEntryData parent);

  //invoke the attached event handlers
  void invokeEventHandlers();

  //add an event handler to the list
  void addEventHandler( cKeyProviderEventHandler* eh);

  //remove an event handler from the list
  void removeEventHandler( cKeyProviderEventHandler* eh);

  //return the next event handler from the list
  cKeyProviderEventHandler* popNextEventHandler();

  //set the timer proxy pointer
  void setKeyRequestTimerProxy( czkTimerProxy* tp);

  //cancel the timeout event
  void cancelKeyRequestTimeout(); 

  //delete the timer proxy
  void deleteKeyRequestTimerProxy(); 

};

#endif
