#include "cKeyProviderCacheEntryData.h"
#include "cKeyProviderEventHandlerParent.h"

#define REV_ID "@(#) $Id: cKeyProviderCacheEntryData.cpp,v 1.3 2001/06/07 15:44:15 frankf Exp $"
#include <zkrev.h>
#include "zkPackBuf.h"



//
// class cKeyProviderCacheEntryData encapsulate the data in a key cache entry
//

//constructor
cKeyProviderCacheEntryData::cKeyProviderCacheEntryData()
{
  pkeyInfo = new cKeyProviderCacheKeyInfo();
  reply = NULL;
};

  //destructor
cKeyProviderCacheEntryData::~cKeyProviderCacheEntryData() 
{ 
  CZKS::Exception e;

  if (reply) {
    delete [] reinterpret_cast<char *>(reply);
    reply = NULL; 
  };
  if (pkeyInfo && pkeyInfo->decRefcount() == 0) {
    delete pkeyInfo;
    pkeyInfo = NULL;
  }
}

//copy constructor
//  NOTE that the "reply" pkt is not copied - only the original object keeps this
//  for validation purposes
cKeyProviderCacheEntryData::cKeyProviderCacheEntryData( 
                                const cKeyProviderCacheEntryData& entryData)
{
  CZKS::Exception e;

  qry = entryData.qry;
  state = entryData.state;
  expires = entryData.expires; 
  expTime = entryData.expTime;
  replyLen = 0;
  reply = NULL; 

  pkeyInfo = entryData.pkeyInfo;
  if (pkeyInfo) {
    pkeyInfo->incRefcount();
  } 

}

//assignment operator
//  NOTE that the "reply" pkt is not copied - only the original object keeps this
//  for validation purposes
cKeyProviderCacheEntryData& cKeyProviderCacheEntryData::operator=( 
                                const cKeyProviderCacheEntryData& entryData)
{
  CZKS::Exception e;

  if (this != &entryData) {
    qry = entryData.qry;
    state = entryData.state;
    expires = entryData.expires; 
    expTime = entryData.expTime;
    replyLen = 0;
    if (reply) {
      delete [] reinterpret_cast<char *>(reply);
    }
    reply = NULL; 

    if (pkeyInfo && pkeyInfo->decRefcount() == 0) {
      delete pkeyInfo;
      pkeyInfo = NULL;
    }

    pkeyInfo = entryData.pkeyInfo;
    if (pkeyInfo) {
      pkeyInfo->incRefcount();
    } 
  }
  return *this;
}

//return the qry
FrKeyQryQueryExp cKeyProviderCacheEntryData::getQry()
{
   return qry;
}

//set the qry
void  cKeyProviderCacheEntryData::setQry(FrKeyQryQueryExp& newqry)
{
   qry = newqry;
}

//return the key query state
FrKeyQryState cKeyProviderCacheEntryData::getState()
{
   return state;
}

//set the key query state
void cKeyProviderCacheEntryData::setState(FrKeyQryState newstate)
{
   state = newstate;
}

//return the public key
FrCryptPubKey& cKeyProviderCacheEntryData::getPubKey()
{
   return pkeyInfo->keyInfo.key;
}

//return the keyinfo data
CacheKeyInfo& cKeyProviderCacheEntryData::getKeyInfo()
{
   return pkeyInfo->keyInfo;
}

//set the keyinfo data
void cKeyProviderCacheEntryData::setKeyInfo(CacheKeyInfo& newkeyinfo)
{
    if (pkeyInfo && pkeyInfo->decRefcount() == 0) {
      delete pkeyInfo;
      pkeyInfo = NULL;
    }

    pkeyInfo = new cKeyProviderCacheKeyInfo();
    pkeyInfo->keyInfo = newkeyinfo;
}
  
//set the reply packet
void cKeyProviderCacheEntryData::setReply(void* newreply,
                                          size_t rl)
{
  if ( reply) {
    delete [] reinterpret_cast<char *>(reply);
    reply = NULL;
  }
  replyLen = rl;
  reply = reinterpret_cast<void *>(new char[rl]);

  memcpy( reply, newreply, rl);
}

//set the key expiry time
void cKeyProviderCacheEntryData::setExpires(int32_t expsec)
{
   expires = expsec;
}

//return the key expiry time
int32_t cKeyProviderCacheEntryData::getExpires()
{
   return expires;
}

//set the time thatthe key expires from the cache
void cKeyProviderCacheEntryData::setExpTime()
{
   //see setCacheExpTime
   //set the expiry time based on the owner's entity type

static struct {
  FrEntIdType type;
  ZkClock exp;
} expTab[] = {
  { FRENT_TYPE_AIP,           ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_FMG,           ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_FREEDOM_MONTH, ZKCLOCK_INIT(HOUR(24)) },
  { FRENT_TYPE_FREEDOM_YEAR,  ZKCLOCK_INIT(HOUR(48)) },
  { FRENT_TYPE_KEYQRY_SRV,    ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_KEYUPD_SRV,    ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_MAIP,          ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_NIDB,          ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_NIQS,          ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_NISS,          ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_NNTP_SRV,      ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_NYM,           ZKCLOCK_INIT(HOUR(3)) },
  { FRENT_TYPE_NYM_SRV,       ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_SER_SRV,       ZKCLOCK_INIT(HOUR(6)) },
  { FRENT_TYPE_TOK_SRV,       ZKCLOCK_INIT(HOUR(6)) },
  
  { FRENT_TYPE_UNUSED_, ZKCLOCK_INIT(0) } /* must be last */
};

  ZkClock now = zkClockNow();
  if (state != FRKEYQRY_ST_VALID) {
    expTime = zkClockAdd(now, zkClockFromSeconds(MINUTE(5)));
  } else {
    FrEntIdType kot =  pkeyInfo->keyInfo.keyOwner.type;

    if (kot == FRENT_TYPE_MASTER) {
      expTime = zkClockMaxPos(); /* never expire from cache */
    } 
    else {
      int32_t i;
    
      expTime = zkClockAdd(now, zkClockFromSeconds(MINUTE(30))); /* default, if not found in table */

      for (i = 0; zkClockToSeconds(expTab[i].exp) != 0; i++) {
        if (kot == expTab[i].type) {
          expTime = zkClockAdd(now, expTab[i].exp);
          break;
        }
      }
    }
  }
}

//return if the key has expired from the cache
bool cKeyProviderCacheEntryData::expired()
{
  //return true if key has expired from cache
  ZkClock now = zkClockNow();
  bool exp = (zkClockCmp(expTime, now) < 0); 
  return exp;
}

//build a query for the parent key
void cKeyProviderCacheEntryData::formParentKeyRequest(FrKeyQryQueryExp& pqry)
{
   const char* func = "cKeyProviderCacheEntryData::formParentKeyRequest";

   CZKS::Exception zksException;

   // we need to get the parent key to validate this entry's sig

   void *auth = ZKPACKBUF_ADDR_OF(reply, FR_KEYQRY_REPLY_PKT_SIZEOF);

   uint32_t pkv; /* parent's key version */

   try {

      if (!frCryptSigBufGetVersion(zksException.zkError(), 
                           ZKPACKBUF_ADDR_OF(auth, FR_KEYQRY_AUTH_SIGNERSIG),
                                   FRCRYPT_SIG_SIZE, &pkv)) {
         CZKS::Throw(zksException);

      }
   
      // create the query for the parent key

      pqry.keyVersion = pkv;
      pqry.keyType = FRCRYPT_KEYTYPE_SIGNING;
      pqry.keyOwner.type = 
        (FrEntIdType)zkPackBufGetUint8(auth, FR_KEYQRY_AUTH_KEYSIGNERENTITY);
      memcpy(pqry.keyOwner.name,
             ZKPACKBUF_CONST_ADDR_OF(auth, FR_KEYQRY_AUTH_KEYSIGNERENTITY+1),
             sizeof(pqry.keyOwner.name));
   }
   catch (CZKS::Exception& e) {

     e.Rethrow( func, "Failure");

   } 
   catch (...) {

     CZKS::Rethrow( func, "Failure");

   } 

}

//validate the parent's signature
void cKeyProviderCacheEntryData::validate( cKeyProviderCacheEntryData parent)
{
  const char* func = "cKeyProviderCacheEntryData::validate";

  CZKS::Exception zksException;

  // set state invalid in case of error
  state = FRKEYQRY_ST_INVALID;

  try {

     // validate the signature in the current key reply 
     // using the incoming parent key data
     if (!valid_parent_sig(zksException.zkError(), &parent.getKeyInfo(), 
                           reply, replyLen)) {
        CZKS::Throw(zksException);
     }

     size_t keySz = replyLen - (FR_KEYQRY_REPLY_PKT_SIZEOF +
                                FR_KEYQRY_AUTH_SIZEOF);
     void *auth = 
        ZKPACKBUF_ADDR_OF(reply, FR_KEYQRY_REPLY_PKT_SIZEOF);
     void *keyData = ZKPACKBUF_ADDR_OF(reply, FR_KEYQRY_REPLY_PKT_SIZEOF +
                                      FR_KEYQRY_AUTH_SIZEOF);

  
     if (!key_from_buf(zksException.zkError(), &pkeyInfo->keyInfo.key, keyData, keySz)) {
        CZKS::Throw(zksException);
     }

     fr_EntFromPacked(&pkeyInfo->keyInfo.keyOwner, 
                      ZKPACKBUF_ADDR_OF(auth, FR_KEYQRY_AUTH_KEYOWNERENTITY));

     state = FRKEYQRY_ST_VALID;

     //the state must be VALID for this call to set the expiry time correctly
     setExpTime();

  }
  catch (CZKS::Exception& e) {

     e.Rethrow(func, "Failure");

  }

} 
