#include "cKeyProviderEventHandlerIO.h"
#include "cKeyProviderEventHandlerDataIO.h"
#include "cKeyProvider.h"
#include "czkIntervalTime.h"
#include "cKeyProviderScopeLock.h"
#include "zkPackBuf.h"

#include <strstream>
  
#define REV_ID "@(#) $Id: cKeyProviderEventHandlerIO.cpp,v 1.3 2001/06/07 15:44:15 frankf Exp $"
#include <zkrev.h>




//constructor
cKeyProviderEventHandlerIO::cKeyProviderEventHandlerIO() {}

//destructor
cKeyProviderEventHandlerIO::~cKeyProviderEventHandlerIO() {}

//set the pointer to the KeyProvider object
void cKeyProviderEventHandlerIO::setKeyProvider( cKeyProvider* kp)
{
  // save a pointer to the KeyProvider object

  ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, kp);

  pKeyProvider = kp;
}

//handle incoming packets from the KQD
void cKeyProviderEventHandlerIO::handleInput(
                                       czkSocket* sock, 
                                       czkEventHandlerData* data)
{
  // receiving a packey from the KQD

  (void) data;

  const char* func = "cKeyProviderEventHandlerIO::handleInput"; 

  CZKS::Exception zksException;

  ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, sock);

  char buffer[BUFSIZ];

  FrKeyQryQueryExp cacheQuery;

  try {

    // receive data
    int32_t nb = BUFSIZ;
    int32_t pktlen = sock->recv( buffer, nb, 0, PR_INTERVAL_NO_WAIT);
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " pktlen %d recvd from KQD",  pktlen);

    if (pktlen < 0) {
      CZKS::Throw( ZKS_KEYPROVIDER_MOD, 
                   ZKS_KEYPROVIDER_SOCKRECV_ERR,
                   func, "Error %d receiving data on socket", pktlen);
    }
 
    // check if the packet has enough data
    if (pktlen < (int32_t) FR_KEYQRY_REPLY_PKT_SIZEOF) {
      *pKeyProvider->kpStats->pktDropped      += 1;
      *pKeyProvider->kpStats->bytesDropped    += pktlen;
      CZKS::Throw( ZKS_KEYPROVIDER_MOD, 
                   ZKS_KEYPROVIDER_SOCKRECV_ERR,
                   func, "Received insufficient size packet size=%d", pktlen);
    }

    *pKeyProvider->kpStats->pktRcvd      += 1;
    *pKeyProvider->kpStats->bytesRcvd    += pktlen;

    //extract key info from buffer
    void *reply = (void *)buffer;

    // validate reply->respCode
   
    cacheQuery.keyVersion = zkPackBufGetUint8(reply, 
                                              FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                              FR_KEYQRY_QRY_KEYVERSION);
    cacheQuery.keyType = (FrPubKeyType)zkPackBufGetUint8(reply, 
                                                         FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                                         FR_KEYQRY_QRY_KEYTYPE);
    fr_EntFromPacked(&cacheQuery.keyOwner, 
                     ZKPACKBUF_ADDR_OF(reply, 
                                       FR_KEYQRY_REPLY_PKT_ORIGQUERY + FR_KEYQRY_QRY_KEYOWNER));
 
    // get entry from cache for this key
    INITLOCK( cachelock, pKeyProvider->keyCache);
    LOCK( cachelock);
    cKeyProviderCacheEntry* entry = pKeyProvider->keyCache->get( cacheQuery);
     
    if (entry) {
     
      INITLOCK( entrylock, entry);
      LOCK( entrylock);
      UNLOCK( cachelock);
   
      switch (zkPackBufGetUint8(reply, FR_KEYQRY_REPLY_PKT_RESPCODE)) {
        case FR_KEYQRY_CODE_BADQUERY:
        case FR_KEYQRY_CODE_SRVERROR:
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " badquery response from KQD");
          entry->setState( FRKEYQRY_ST_INVALID);
          entry->invokeEventHandlers();
          UNLOCK(entrylock);
        break;
    
        case FR_KEYQRY_CODE_FOUND:
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " CODE_FOUND from KQD");

          size_t keyDataSz;
          ZkClock newExp;
          void *keyDataPtr;


          keyDataPtr = ZKPACKBUF_ADDR_OF(reply, FR_KEYQRY_REPLY_PKT_SIZEOF + 
                                                FR_KEYQRY_AUTH_SIZEOF);
          keyDataSz = pktlen - (FR_KEYQRY_REPLY_PKT_SIZEOF + FR_KEYQRY_AUTH_SIZEOF);
    
          /*
           *  Reject keys whose version, type and owner entity do not match
           *  that in the original query field of the response.
           */

          {
            FrEntIdExp kown, qown;
            FrPubKeyType ktype;
            int32_t eq;
            uint32_t kvers;

            if (!frEntLoad(zksException.zkError(), &qown,
                           ZKPACKBUF_ADDR_OF(reply, FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                                    FR_KEYQRY_QRY_KEYOWNER),
                           FRENT_PORTABLE_SIZE)) {
	      CZKS::Throw(zksException);
	    }
            else if (!frCryptPubKeyBufGetVersion(zksException.zkError(), 
                                                 keyDataPtr, 
                                                 keyDataSz, &kvers)) {
	      CZKS::Throw(zksException);
	    }
            else if (!frCryptPubKeyBufGetType(zksException.zkError(), 
                                              keyDataPtr, 
                                              keyDataSz, &ktype)) {
	      CZKS::Throw(zksException);
	    }
            else if (!frCryptPubKeyBufGetEntity(zksException.zkError(),
                                                keyDataPtr, 
                                                keyDataSz, &kown)) {
	      CZKS::Throw(zksException);
            }
     
            /* bug: is checking for 0 okay? */
            else if ( ! ((zkPackBufGetUint8(reply, 
                                            FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                            FR_KEYQRY_QRY_KEYVERSION) == kvers || 
                          zkPackBufGetUint8(reply, FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                                   FR_KEYQRY_QRY_KEYVERSION) == 0) &&
                          zkPackBufGetUint8(reply, FR_KEYQRY_REPLY_PKT_ORIGQUERY + 
                                                   FR_KEYQRY_QRY_KEYTYPE)    == ktype)) {
               zksException.Throw( ZKS_KEYPROVIDER_MOD, 
                                   ZKS_KEYPROVIDER_PKTBADVERS_ERR,
                                   func, "KQD key response has bad key info");
            }
            else if (!frEntEqual(zksException.zkError(), &qown, &kown, &eq)) { 
	       CZKS::Throw(zksException);
            }
            else if (!eq) {
               zksException.Throw( ZKS_KEYPROVIDER_MOD, 
                                   ZKS_KEYPROVIDER_PKTBADOWNER_ERR,
                                   func, "KQD key response has bad owner");
            }
          }
  
          /*
           *  Grab the expiry date from the response.
           */
  
          if ( ! frCryptPubKeyBufGetExpiration(zksException.zkError(), 
                                               keyDataPtr, keyDataSz, 
                                               &newExp)) {
	     CZKS::Throw(zksException);
          
          }
          /*
           *  Reject keys that have already expired.
           */

          // only make this check if key is currently valid?
          {
            bool replacekey = true;
            if (entry->getState() == FRKEYQRY_ST_VALID) {
              /*
               *  Replace the existing entry only if the new response has a
               *  greater expiry date.
               */
    
              if (zkClockCmp(newExp, 
                             zkClockFromSeconds(entry->getExpires())) <= 0) {
                replacekey = false;
              }
            }

            if (replacekey) { 

              entry->setReply(reply, pktlen);
              entry->setExpires(zkClockToSeconds(newExp));
              entry->setState(FRKEYQRY_ST_AWAITING_VALIDATION); // wait for sig

              FrKeyQryQueryExp pqry;
              try { 
                entry->formParentKeyRequest(pqry);
                UNLOCK(entrylock);
              }
              catch (...) {
                UNLOCK(entrylock);
                CZKS::Rethrow( func, "formParentKeyRequest failed");
	      } 
          
              // instantiate an eventhandler to handle the incoming parent key
              cKeyProviderEventHandlerParent* peh = 
                   new cKeyProviderEventHandlerParent();

              peh->setDispose(true);

              // give the eventhandler the original child query
              peh->setKeyQry(cacheQuery, pKeyProvider->keyCache);

              // make the request for the parent key
              ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " submit request for parent");
              cKeyProviderCacheEntryData entryData;
              pKeyProvider->makeKeyRequest(pqry, peh, entryData);
            }   
            else {

              UNLOCK(entrylock);

	    }

	  }

          break;
    
        case FR_KEYQRY_CODE_NOTFOUND:

          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " CODE_NOT_FOUND from KQD");
          entry->setState(FRKEYQRY_ST_NOT_FOUND);
          entry->invokeEventHandlers();
          UNLOCK(entrylock);
          break;
      }
    }
    else {
 
      UNLOCK( cachelock);

      //log "unidentified data recvd from KQD?"
      zksLogMesg( ZKS_KEYPROVIDER_MOD, "unidentified data recvd from KQD");
 
    }

  }
  catch (CZKS::Exception &e) {

    CZKS::Push(e, func, "failure");

    CZKS::LogException( e);

    zksLogMesg( ZKS_KEYPROVIDER_MOD, "%s terminated abnormally", func);

  }
  catch (...) {

    CZKS::Rethrow(func,"failed");

  }

}

//handle socket errors
void cKeyProviderEventHandlerIO::handleException( czkSocket* sock, 
                                                  czkEventHandlerData* data)
{

   const char* func = "cKeyProviderEventHandlerIO::handleException";

   ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, sock);
   ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, data);

   ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " socket i/o error %d", PR_GetError());

   if (PR_GetError() != 0) {

     try {
       CZKS::Throw(ZKS_KEYPROVIDER_MOD, ZKS_KEYPROVIDER_IO_ERR, func, 
                   "socket i/o error %d", PR_GetError());
     }
     catch (CZKS::Exception &zksException) {

       CZKS::LogException( zksException);

     }
     catch (...) {

       CZKS::Rethrow(func,"failed");

     }
   }

   cKeyProviderEventHandlerDataIO* eventdata = 
     (cKeyProviderEventHandlerDataIO*) data;
   
   eventdata->pCoreLoop->removeHandler( sock);

   sock->close();

   pKeyProvider->connectKQD();

}

//handle timeout events
int32_t cKeyProviderEventHandlerIO::handleTimeout( czkEventHandlerData* data)
{
  const char* func = "cKeyProviderEventHandlerIO::handleTimeout";

  int32_t reschedule = 0;

  ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, data);

  cKeyProviderEventHandlerDataIO* timeoutEvent = 
     (cKeyProviderEventHandlerDataIO*)data;

  ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, timeoutEvent);

  try {

    if (timeoutEvent->isKeyRequestTimeout()) {

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " key request timed out");

      if (FRENT_TYPE_AIP == timeoutEvent->qry.keyOwner.type) 
	ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
		       " for key request version %d type %d entity-type %d name %s ",
		       timeoutEvent->qry.keyVersion, timeoutEvent->qry.keyType, 
		       timeoutEvent->qry.keyOwner.type,
		       timeoutEvent->qry.keyOwner.name);
      else
	ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
		       " for key request version %d type %d entity-type %d name",
		       timeoutEvent->qry.keyVersion, timeoutEvent->qry.keyType, 
		       timeoutEvent->qry.keyOwner.type);

      INITLOCK( cachelock, pKeyProvider->keyCache);
      LOCK( cachelock);

      FrKeyQryState status;

      // get the CacheEntry 
      cKeyProviderCacheEntry* entry = 
        pKeyProvider->keyCache->get( timeoutEvent->qry);
      if (entry) {

        INITLOCK( entrylock, entry);
        LOCK( entrylock);
        UNLOCK(cachelock);

        // get current status
        status = entry->getState(); 

        // if we're waiting for parent, just reschedule
        // NOTE: if the parent key requests fails, the
        // KeyProviderEventHandlerParent will take care it
        // (e..g marking child status to INVALID)
        if (status == FRKEYQRY_ST_AWAITING_VALIDATION) {

          reschedule = 1;

        }
        else if (status ==  FRKEYQRY_ST_AWAITING_RESPONSE) {

          // still waiting for response from KQD, check if we want to retry
 
          if (timeoutEvent->nretry < pKeyProvider->getMaxRetry()) {
            timeoutEvent->nretry++;

            // send request to KQD
            try {
              pKeyProvider->sendQry( timeoutEvent->qry);
	    }
            catch (CZKS::Exception& zksException) {
              CZKS::Push(zksException, func, "failure");
	      CZKS::LogException( zksException);
	    }              
          
            reschedule = 1;
          }
          else {

            // no more retries - time to kill the request
            zksLogMesg( ZKS_KEYPROVIDER_MOD, 
                        "Key request timed out: retries exhausted");

            entry->setState(FRKEYQRY_ST_INVALID);
            try {
	       // MUST delete the timerProxy first because
	       // invokeEventHandlers() will try to delete the timer and associated data	      
               entry->deleteKeyRequestTimerProxy();
               entry->invokeEventHandlers();
	    }
            catch (CZKS::Exception& zksException) {
              CZKS::Push(zksException, func, "failure");
	      CZKS::LogException( zksException);
	    }              
           
          }
        }

        UNLOCK(entrylock);

      }
      else {

        UNLOCK(cachelock);
 
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
                       "Key request timeout: cache entry not found");
      }

    }
    else if (timeoutEvent->isPruneCache()) {

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " prune cache timeout");

      // check to make sure pKeyProvider is set
      if (pKeyProvider) {
        // do the pruning
        INITLOCK(cachelock, pKeyProvider->keyCache);
        LOCK(cachelock);
        pKeyProvider->keyCache->prune();
        UNLOCK(cachelock); 

      }
      reschedule = 1;

    }

  }
  catch (CZKS::Exception &zksException) {

    CZKS::Push(zksException, func, "failure");
    CZKS::LogException( zksException);

    zksLogMesg( ZKS_KEYPROVIDER_MOD, "%s terminated abnormally", func);

    reschedule = 0;

  }
  catch (...) {

    CZKS::Rethrow(func,"failed");

  }

  if (reschedule == 0) {
    delete timeoutEvent;
  }

  return reschedule;
  
}
