#include "cKeyProviderEventHandlerParent.h"
#include "cKeyProviderScopeLock.h"

#define REV_ID "@(#) $Id: cKeyProviderEventHandlerParent.cpp,v 1.2 2001/04/18 17:36:47 alainc Exp $"
#include <zkrev.h>




// call this method to set the query and KeyCache pointer
void cKeyProviderEventHandlerParent::setKeyQry(FrKeyQryQueryExp qryin, 
                                               cKeyProviderCache* pk)
{
  ZKS_ASSERT(ZKS_KEYPROVIDER_MOD, pk);

  qry = qryin;
  pKeyCache = pk;
}

// provides the actual implemention of a key validation event
// receives a copy of the newly-validated parent KeyCache entry
// and uses this to validate the signature in the child's KeyCache entry
void cKeyProviderEventHandlerParent::handleEvent(
                                        cKeyProviderCacheEntryData entryData)
{

  const char* func = "cKeyProviderEventHandlerParent::handleEvent";

  try {
    
    if (FRENT_TYPE_AIP == qry.keyOwner.type) 
      ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
		     " in eventhandler parent version %d type %d entity-type %d name %s ",
		     qry.keyVersion, qry.keyType, 
		     qry.keyOwner.type, qry.keyOwner.name);
    else 
      ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
		     " in eventhandler parent version %d type %d entity-type %d",
		     qry.keyVersion, qry.keyType, 
		     qry.keyOwner.type);

    INITLOCK( cachelock, pKeyCache);
    LOCK( cachelock);
    cKeyProviderCacheEntry* child = pKeyCache->get(qry); 
    if (child) {

      INITLOCK( entrylock, child);
      LOCK( entrylock);
      UNLOCK( cachelock);
      try {

        if (entryData.getState() == FRKEYQRY_ST_VALID) {
          child->validateParentSig(entryData);
        }
        else {
          child->setState( FRKEYQRY_ST_INVALID);
          child->invokeEventHandlers();
        }
        UNLOCK( entrylock);

      }
      catch (...) {

        UNLOCK( entrylock);
	CZKS::Rethrow( func, "error validating parent signature in child");

      } 

    }
    else {

      UNLOCK( cachelock);

    }
  }
  catch (CZKS::Exception &zksException) {

    CZKS::Push(zksException, func, "failure");
    CZKS::LogException( zksException);

  }
  catch (...) {

    CZKS::Rethrow(func,"failed");

  }
    
}


