#ifndef _cKeyProviderScopeLock

#define _cKeyProviderScopeLock

#include "czkScopeLock.h"
#include "cKeyProviderCache.h"
#include "cKeyProviderCacheEntry.h"

// Class to enable "safe" handling of a czkLock object.
// It will automatically release an acquired lock if the object is deleted
// (e.g. goes out of scope)
//

#ifdef AIPMULTITHREADED

class cKeyProviderScopeLock : public czkScopeLock {

 private:

    //copy constructor (disabled)
    cKeyProviderScopeLock( const cKeyProviderScopeLock& c) 
    : czkScopeLock(c) { assert(0); };

    //assignment operator (disabled)
    cKeyProviderScopeLock& operator=( const cKeyProviderScopeLock&) { assert(0); };

 public:

    cKeyProviderScopeLock(cKeyProviderCache* p): czkScopeLock(&p->hashlock) {};
 
    cKeyProviderScopeLock(cKeyProviderCacheEntry* p): czkScopeLock(&p->entryLock) {};
 
    virtual ~cKeyProviderScopeLock() {};

};

#define INITLOCK(a,b)  cKeyProviderScopeLock a(b)
#define LOCK(a)        a.acquire()
#define UNLOCK(a)      a.release()

#else  //not aipmultithreaded

#define INITLOCK(a,b) 
#define LOCK(a)       
#define UNLOCK(a)      

#endif  //multithreaded

#endif


