#define REV_ID "@(#) $Id: testKeyProvider.cpp,v 1.2 2001/04/18 17:36:52 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cKeyProvider
//
// usage:      testKeyProvider --testtype testtype --repeat repeat
//                             --entityname entityname --entitytype entitytype
//                             --keytype keytype --keyversion keyversion
//
// where "testtype"   is the test to run (0-4)
//       "repeat"     is no. of times to repeat the test (default: 1)
//       "entityname" is the entity name to get the key for (e.g. host1)
//       "entitytype" is the entity type (e.g. AIP)
//       "keytype"    is the key type (S for signature, C for cryption)
//       "keyversion" is the key version number (0 for current)
//
// test 0: calls getKeyWait (can only work in multi-threaded mode)
//
// test 1: calls getKeyNoWait (i.e. doesn't wait for key request to complete)
//
// test 2: call makeKeyRequest with an eventhandler to detect success 
//
// test 3: request keys for entities "host1" through "host10"
//
// test 4: call makeKeyRequest
//         then delete eventhandler
//

     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkSignal.h>
    
#include <iostream>
#include <strstream> 
#include <fstream>
  
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

#include "cConfigProvider.h"
#include "cKeyProvider.h"
#include "cStatsHarvestor.h"

#include "cArgParser.h"


const string usage = "\nusage: \n\t--testtype [0..4] \n\t--entityname [name (eg. host1)] \n\t--entitytype [type, eg AIP] \n\t--keytype [S(ignature) or C(ryption)] \n\t--keyversion [version no (1 for current)] \n\t--repeat [num repeat]\n\n\twhere testtype: \n\t\t0: calls getKeyWait (can only work in multi-threaded mode)\n\t\t1: calls getKeyNoWait (doesn't wait for key request to complete)\n\t\t2: call makeKeyRequest with an eventhandler to detect success\n\t\t3: request keys for entities \"host1\" through \"host10\"\n\t\t4: call makeKeyRequest then delete eventhandler\n";

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

class Test : public czkEventHandlerData {

public:
  int32_t type;
  char*   entityname;
  char*   entitytype;
  char*   keytype;
  int32_t keyversion;
  cKeyProvider* pKeyProvider;
  int32_t repeat;
  int32_t done;
  czkCoreLoop* pCoreLoop;
};

class cmyKeyProviderEventHandler : public cKeyProviderEventHandler {

  void handleEvent( cKeyProviderCacheEntryData entryData) 
  {
    CacheKeyInfo keyInfo = entryData.getKeyInfo();
    FrKeyQryQueryExp qry = entryData.getQry();

    ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
          " cmyKeyProviderEventHandler got key state %d name %s key-type %d",
          entryData.getState(), keyInfo.keyOwner.name, qry.keyType);

    Test* pTest = (Test*)getData();
    if (pTest->done) {
      pTest->pCoreLoop->abortEventLoop();
    }
  }
};

void testkeyrequest( Test* pTest)
{
    cKeyProviderCacheEntryData entryData;
	//    int32_t rc;

    cKeyProvider* pKeyProvider = pTest->pKeyProvider;

    char entity[128];
    FrEntIdType entity_type;
    FrPubKeyType key_type;

    memset(entity, 0, 128);
    strcpy(entity, pTest->entityname);
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_KEYPROVIDER_MOD, "************* entity %s ", entity);
    if (strcmp( pTest->entitytype, "AIP") == 0) {
      entity_type = (FrEntIdType)FRENT_TYPE_AIP;
    }
    else if (strcmp( pTest->entitytype, "MONTH") == 0) {
      entity_type = (FrEntIdType)FRENT_TYPE_FREEDOM_MONTH;
    }

    if (strcmp( pTest->keytype, "S") == 0) {
      key_type = (FrPubKeyType)FRCRYPT_KEYTYPE_SIGNING; 
    }
    else {
      key_type = (FrPubKeyType)FRCRYPT_KEYTYPE_CRYPTION; 
    }

    uint32_t key_version = pTest->keyversion;

    pTest->repeat--;
    if (pTest->type == 0) {
      pKeyProvider->getKeyWait( entity,  entity_type, 
                                key_type, key_version, 
                                ZKPR_INTERVAL_NO_TIMEOUT, entryData);
    }
    else if (pTest->type == 1) {
      pKeyProvider->getKeyNoWait( entity,  entity_type, 
                                  key_type, key_version,
                                  entryData);      
    }
    else if (pTest->type == 2) {
      cmyKeyProviderEventHandler* eh = new cmyKeyProviderEventHandler();
      if (pTest->repeat == 0) {
        pTest->done = 1;
      }
      eh->setData(pTest);
      pKeyProvider->makeKeyRequest( entity,  entity_type, 
                                    key_type, key_version, 
                                    eh, entryData);
      
    }
    else if (pTest->type == 3) {
      cmyKeyProviderEventHandler* eh = new cmyKeyProviderEventHandler();
      eh->setData(pTest);
      pKeyProvider->makeKeyRequest( entity,  entity_type, 
                                    key_type, key_version, 
                                    eh, entryData);

      for (int32_t i = 1; i < 10; i++)
      {
        sprintf(entity, "host%d\0", i);
        entity_type = (FrEntIdType)FRENT_TYPE_AIP;
        key_type = (FrPubKeyType)FRCRYPT_KEYTYPE_SIGNING; 
        key_version = 1;
        eh = new cmyKeyProviderEventHandler();
        eh->setData(pTest);
        pKeyProvider->makeKeyRequest( entity,  entity_type, 
                                      key_type, key_version, 
                                      eh, entryData);
      }
    }
    else if (pTest->type == 4) {
      cmyKeyProviderEventHandler* eh = new cmyKeyProviderEventHandler();
      if (pTest->repeat == 0) {
        pTest->done = 1;
      }
      eh->setData(pTest);
      pKeyProvider->makeKeyRequest( entity,  entity_type, 
                                    key_type, key_version, 
                                    eh, entryData);

      pKeyProvider->cancelKeyRequest( entity,  entity_type, 
                                      key_type, key_version, 
                                      eh);

    }

    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " got key state %d", entryData.getState());
}

class cAipTestHandler : public czkEventHandler {

private: 

public:

  cAipTestHandler() {}

  void handleInput( czkSocket*, czkEventHandlerData*) {};
  void handleOutput( czkSocket*, czkEventHandlerData*) {};
  void handleException( czkSocket*, czkEventHandlerData*) {};
 
  int32_t handleTimeout( czkEventHandlerData* data) 
  { 
    Test* pTest = (Test*) data;      

    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " got type %d ", pTest->type);

    pTest->done = 0;

    try {
      testkeyrequest( pTest);
    }
    catch (CZKS::Exception &e) {
      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught evenbt ");
      CZKS::LogException(e);
    }
    catch (...) {
      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught ... ");
    }

    int32_t repeat = pTest->repeat;
    if (!repeat) {
      //      delete pTest;
    }
    return repeat;
  };

};

class SigHandler {

private:  
  czkCoreLoop* coreloop;

public:

  void setCoreLoop(czkCoreLoop* cl) { coreloop = cl; };

  void addSignal( int32_t signum) { coreloop->addSignal( signum); }
};
 
SigHandler GlobalSigHandler;

void sighandler( int32_t signum)
{
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " adding signal %d ", signum);
  GlobalSigHandler.addSignal( signum);
}

int main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    cArgParser ap( argc, argv);
    ap.populate();

    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " starting key provider test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

        czkCoreLoop* coreloop = NULL;
   
#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " running in single thread mode");
        bool dothread = false;
        coreloop = new czkCoreLoop();
#endif

        std::ifstream istrm("./testKeyProvider.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_ROUTECREATOR_MOD,
                      1,
                      "FunctionalAip",
                      "Failed opening file: ./testKeyProvider.conf");
        }
		
		
        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;
		
        // start ConfigProvider component thread
        cConfigProvider* pConfigProvider = new cConfigProvider(m_cmdline, m_ini);
        mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);
        pConfigProvider->validateConfig();
		
        // start StatsHarvestor component thread
        cStatsHarvestor* pStatsHarvestor = new cStatsHarvestor();
        mgr.add(ZKS_STATSHARVESTOR_MOD, pStatsHarvestor);
        pStatsHarvestor->validateConfig();

        // start KeyProvider component thread
        cKeyProvider* pKeyProvider = new cKeyProvider();
        mgr.add(ZKS_KEYPROVIDER_MOD, pKeyProvider);
        if (!pKeyProvider->validateConfig()) {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD,"ValidateConfig failed");
		  abort();
		}

		
        mgr.initAllComponents(coreloop);
		
        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_STATSHARVESTOR_MOD, dothread);
        mgr.startComponent(ZKS_KEYPROVIDER_MOD, dothread);
		
        pKeyProvider->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);
		
        if (dothread) {
          coreloop = new czkCoreLoop();
		}
		
        GlobalSigHandler.setCoreLoop( coreloop);
		
        Test* pTest;

        char* arg;
		
        int32_t type = -1;
		string temp = "--testtype";
        if (ap.get(temp.c_str(), &arg)) {
          type = atoi( arg);
        }
        else {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " no testtype ");
		  cerr<<usage;
		  return 0;
		}
		
        char* entityname;
		temp = "--entityname";
        if (ap.get(temp.c_str(), &entityname)) {
		  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " entityname %s ", entityname);
        }
        else {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " no entityname ");
          cerr<<usage; 
		  return 0;
		}
		
        char* entitytype;
		temp = "--entitytype";
        if (ap.get(temp.c_str(), &entitytype)) {
		  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " entitytype %s ", entitytype);
        }
        else {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " no entitytype ");
          cerr<<usage; 
		  return 0;
		}

        char* keytype;
		temp = "--keytype";
        if (ap.get(temp.c_str(), &keytype)) {
		  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " keytype %s ", keytype);
        }
        else {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " no keytype ");
		  cerr<<usage;
		  return 0;
		}
		
        int32_t keyversion = 0;
		temp = "--keyversion";
        if (ap.get(temp.c_str(), &arg)) {
		  keyversion = atoi( arg);
        }
		
        char* repeat = NULL;
		temp = "--repeat";
        if (ap.get(temp.c_str(), &repeat)) {
		  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " repeat %s ", repeat);
        }
		
        if (argc >= 3) {
		  
          if (type >= 1 && type <= 9) {
            pTest = new Test;
            pTest->type = type;
            pTest->entityname = entityname;
            pTest->entitytype = entitytype;
            pTest->keytype = keytype;
            pTest->keyversion = keyversion;
            pTest->pKeyProvider = pKeyProvider;
            pTest->repeat = atoi(repeat);
            pTest->done = 0;
            pTest->pCoreLoop = coreloop;
            try {
              testkeyrequest( pTest);
			}
            catch (CZKS::Exception &e) {
			  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught main ");
			  CZKS::LogException(e);
			}
            catch (...) {
			  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught ... in main ");
			}
			
            if (pTest->repeat) {
              cAipTestHandler* handle = new cAipTestHandler();
              czkIntervalTime ti;   
              coreloop->scheduleTimer( handle, pTest, ti.getSeconds(60));
			}
          }
        }     
        else {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, 
                         " usage: ./testKeyProvider test-num repeat");
          cerr<<usage;
		  return 0;
        }        
		
        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " exiting coreloop status %d ",status);
        }
        else {
		  
          pTest = new Test;
          pTest->type = 0;
          pTest->repeat = 0;
          pTest->done = 0;
          pTest->pCoreLoop = coreloop;
          pTest->pKeyProvider = pKeyProvider;
		  
          try {
			testkeyrequest( pTest);
		  }
          catch (CZKS::Exception &e) {
			ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught main ");
			CZKS::LogException(e);
		  }
          catch (...) {
			ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " caught ... in main ");
		  }
		  
          int32_t status = coreloop->runEventLoop();
          ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_KEYPROVIDER_MOD, " exiting coreloop status %d ",status);
		  // mgr.waitForCompletion();
          // wait for all threads to complete
		  
          //cAipComponent* pComponent = mgr.getFailedComponent();
          //while (pComponent)
          //{
          //  pComponent->cleanup();
          //  mgr.startComponent( pComponent, coreloop, dothread);
          //  pComponent = mgr.getFailedComponent();
          //}
        }
      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_KEYPROVIDER_MOD, "exiting main");
      }
      catch (...) {
		
        CZKS::Rethrow( ZKS_KEYPROVIDER_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {
	  
      CZKS::LogException( e);
    }    
	
    // clean up all component objects

    mgr.deleteAllComponents();
    mgr.waitForCompletion();
}


