#ifndef _cLinkMaintainer
#define _cLinkMaintainer

// class cLinkMaintainer 
//
#include "czkCoreLoop.h"
#include "aip-modules.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"
#include "cPktDispatcherPktHandler.h"
#include "cPktDispatcher.h"
#include "cLinkManagerInfo.h"

class cLinkMaintainer : public cAipComponent,
                        public czkEventHandler,
                        public cPktDispatcherPktHandler
{

private:

    // This inner class is meant to be passed as some client data
    // to the coreloop when registering for a timeout event.

    class cTimeoutData : public czkEventHandlerData
    {
    private:

        bool send;
        bool first; // only applies when the timeout is for a send event
        LinkId linkId;



    public:

        cTimeoutData(bool inSend, bool inFirst, LinkId &inLinkId)
        {
            this->send = inSend;
            this->first = inFirst;
            this->linkId = inLinkId;
        }
        
        ~cTimeoutData() {};

        void setSend(bool inSend) {
            this->send = inSend;
        }
        
        bool getSend() {
            return send;
        }

        void setFirst(bool inFirst) {
            this->first = inFirst;
        }
        
        bool isFirst() {
            return first;
        }

        void setLinkId(LinkId &inLinkId) {
            this->linkId = inLinkId;
        }
        
        LinkId& getLinkId() {
            return linkId;
        }
    };


    class cLinkMaintInfo
    {

    private:

      LinkId            linkId;
      cLinkManagerInfo  *pLinkManInfo;  
      czkTimerProxy     *sendTimerProxy;
      czkTimerProxy     *recvTimerProxy;


    public:

        cLinkMaintInfo()
        {
            pLinkManInfo = NULL;
            sendTimerProxy = NULL;
            recvTimerProxy = NULL;
        }


        
        virtual ~cLinkMaintInfo()
        {
            cTimeoutData *pTimeoutData = NULL;

	    if (sendTimerProxy != NULL) {

	      pTimeoutData = (cTimeoutData *) sendTimerProxy->cancelTimer();
	      if( pTimeoutData )
		{
		  delete pTimeoutData;
		}

	      delete sendTimerProxy;
	      sendTimerProxy = NULL;

	    }



	    if (recvTimerProxy != NULL) {

	      pTimeoutData = (cTimeoutData *) recvTimerProxy->cancelTimer();
	      if( pTimeoutData )
		{
		  delete pTimeoutData;
		}

	      delete recvTimerProxy;
	      recvTimerProxy = NULL;

	    }
        }

        LinkId& getLinkId() { return linkId; }
        void setLinkId( LinkId &inLinkId ) { this->linkId = inLinkId; }

        cLinkManagerInfo* getLinkManInfo() { return pLinkManInfo; }
        void setLinkManInfo(cLinkManagerInfo *inLinkManInfo) {
            this->pLinkManInfo = inLinkManInfo;
        }

        czkTimerProxy* getSendTimerProxy() { return sendTimerProxy; }
        void setSendTimerProxy( czkTimerProxy *inSendTimerProxy ) {
            this->sendTimerProxy = inSendTimerProxy;
        }
        void deleteSendTimerProxy() {
            delete sendTimerProxy;
            sendTimerProxy = NULL;
        }

        czkTimerProxy* getRecvTimerProxy() { return recvTimerProxy; }
        void setRecvTimerProxy( czkTimerProxy *inRecvTimerProxy ) {
            this->recvTimerProxy = inRecvTimerProxy;
        }
        void deleteRecvTimerProxy() {
            delete recvTimerProxy;
            recvTimerProxy = NULL;
        }

    };



private:
  //disallow the following functions to avoid shallow memory copy
  cLinkMaintainer & operator=(const cLinkMaintainer &) {
    assert(0);
  }
  cLinkMaintainer( const cLinkMaintainer& cLM):cAipComponent(cLM),czkEventHandler(cLM),cPktDispatcherPktHandler() { 
    assert(0); 
  }
  void operator&() {
    assert(0);
  }

    // TODO: LinkId or LinkId &?
    typedef map<LinkId, cLinkMaintInfo *> LinkTableDef;

    czkLock             tableLock_;
    LinkTableDef        linkTable;
    
    cPktDispatcher      *pPktDispatcher;
    cLinkManager        *pLinkManager;
    cConfigProvider     *pConfigProvider;

    PktDispatcherHandleId pktDispHandlerId;

    void removeLink(LinkId &linkId);

#ifdef AIPMULTITHREADED
#warning Multithreading has not been tested - proceed at your own risk
// TODO: when common lock class is ready, substitute it here
    class cScopeLock
    {
    public:
        cScopeLock(czkLock &lock) { this.lock = lock; this.lock.acquire(); }
        ~cScopeLock() { lock.release(); }
    private:
        czkLock lock;
        cScopeLock( cScopeLock &copy ) {};
        cScopeLock & operator=(const cScopeLock &) {};
        
    };
#define SCOPE_LOCK(x) cScopeLock scopeLock____(x)
#else
#define SCOPE_LOCK(x)
#endif


public:

    // constructor
    cLinkMaintainer();

    // destructor
    virtual ~cLinkMaintainer();

    // validate configuration
    bool validateConfig();

    // initialize private data members
    bool initializePrivateData(czkCoreLoop* coreloopin);

    // startup & shutdown
    void startup();
    void shutdown();
    
    // cleanup private data members
    void cleanupPrivateData();

    void registerLink(cLinkManagerInfo *linkManInfo);
    void removeLink(cLinkManagerInfo *linkManInfo);

    // czkEventHandler::handleTimeout (not virtual anymore)
    int32_t handleTimeout(czkEventHandlerData* userData);
    void handleInput(czkSocket* sock, czkEventHandlerData* userData);
    void handleOutput(czkSocket* sock, czkEventHandlerData* userData);
    void handleException(czkSocket* sock, czkEventHandlerData* userData);


    // inherited from cPktDispatcherPKtHandler
    void handlePkt(cLinkManagerInfo *linkInfo, uint8_t *pkt, size_t size);

#ifdef UNITTEST
    void printTable();
#endif

};


#endif


