#ifndef _cLinkManagerInfo

#define _cLinkManagerInfo

#include "frEnt.h"
#include "frCrypt.h"

#include "aip-modules.h"

#include "czkIntervalTime.h"
#include "czkTimerProxy.h"
#include "czkCoreLoop.h"
#include "GenericId.h"
#include "cAciBin.h"

class cLinkManager;
class cLinkManagerInfo;
class cLinkManagerEventHandler;

#define MAXLEKV 2

/* possible links states */
enum LinkState { pendingNegoLinkDown, pendingNegoLinkUp, linkDown, linkUp };

typedef FrCryptSym SymKey;

typedef uint32_t LinkId;

#ifdef __cplusplus

class cLinkManagerInfo {

  friend class cLinkManager;

protected:

  FrEntIdExp  entity;         //entity name
  GenericId   linkId;         //link id
  ZKPRNetAddr addr;           //IP address
  ZKPRNetAddr dhaddr;         //IP address (for DH)
  ZKPRNetAddr dhaddrFixed;    //IP address (for DH as from NetinfoProvider)
  LinkState   state;          //link state
  uint8_t     lekv;           //key version
  SymKey      encrKey[MAXLEKV];     //encryption key
  SymKey      decrKey[MAXLEKV];     //decryption key

  cAciBin*    aciBin; 

private:
  ZKPRIntervalTime DHRenegoTime;  //calculated DHRenegoTime

  czkTimerProxy* pExpiryTimerProxy;  //needed to cancel expiry timer
  czkTimerProxy* pRenegoTimerProxy;  //needed to cancel renego timer

  class cLinkManagerEventHandlerDataTimeout* pExpiryEvent;  //expiry data
  class cLinkManagerEventHandlerDataTimeout* pRenegoEvent;  //renego data

  //private methods
  //copy constructor (disabled)
  cLinkManagerInfo( const cLinkManagerInfo&) { assert(0); };

  //assignment operator (disabled)
  cLinkManagerInfo& operator=( const cLinkManagerInfo&) { assert(0); };

public:

  cLinkManagerInfo();
  virtual ~cLinkManagerInfo();

  //free any allocated SymKeys
  void freeSymKeys(); 

  //free one version of SymKeys
  void freeSymKeys(uint8_t lekv); 

  //init
  void init(GenericIdBin& linkIdBin);

  //init aciBin
  void initAciBin( FrEntIdExp aipEntity);

  //cleanup
  void cleanup(GenericIdBin& linkIdBin); 

  //cancel a pending Expiry Timer
  void cancelExpiryTimer();

  //cancel a pending Renego Timer
  void cancelRenegoTimer();

  //cancel all pending timers
  void cancelTimers();

  void scheduleExpiryTimer(czkCoreLoop* coreloop, cLinkManager* pLinkManager, 
                           cLinkManagerEventHandler* eh);
  void scheduleRenegoTimer(czkCoreLoop* coreloop, cLinkManager* pLinkManager, 
                           cLinkManagerEventHandler* eh);
  void scheduleRenegoTimer(czkCoreLoop* coreloop, cLinkManager* pLinkManager, 
                           cLinkManagerEventHandler* eh, ZKPRIntervalTime interval);

  FrEntIdExp&  getEntity()    { return entity; };
  LinkId       getLinkId()    { return linkId.getID(); };
  ZKPRNetAddr& getAddr()      { return addr; };    
  ZKPRNetAddr& getDHAddr()    { return dhaddr; };    
  LinkState&   getLinkState() { return state; };
  SymKey&      getEncrKey()   { return encrKey[lekv]; } ;
  SymKey&      getDecrKey()   { return decrKey[lekv]; } ;
  SymKey&      getEncrKey(uint8_t lv)   { return encrKey[lv]; } ;
  SymKey&      getDecrKey(uint8_t lv)   { return decrKey[lv]; } ;
  uint8_t      getLekv()      { return lekv; };
  uint8_t      getNextLekv();  
  
  void         setAddr(ZKPRNetAddr& addrin) 
  { addr = addrin; 
  };

  FrAci        getAci()       
  { ZKS_ASSERT(ZKS_LINKMANAGER_MOD, aciBin); return aciBin->getAci(); };

  void         returnAci(FrAci aci) 
  { ZKS_ASSERT(ZKS_LINKMANAGER_MOD, aciBin); aciBin->returnAci(aci); };

};

#endif /* __cplusplus */

#endif



