#include "cLinkManagerInfo.h"
#include "cLinkManager.h"
#include "cLinkManagerEventHandler.h"
#include "cLinkManagerEventHandlerDataTimeout.h"

#define REV_ID "@(#) $Id: cLinkManagerInfo.cpp,v 1.2 2001/04/18 17:37:02 alainc Exp $"
#include <zkrev.h>




//constructor
cLinkManagerInfo::cLinkManagerInfo()
{
  //linkId = GenericId::null();	
   
  state = linkDown;

  lekv = 0;

  DHRenegoTime = 0;

  pExpiryTimerProxy = NULL;
  pRenegoTimerProxy = NULL;

  pExpiryEvent = NULL;
  pRenegoEvent = NULL;

  for (uint8_t i = 0; i < MAXLEKV; i++)
  { 
     encrKey[i] = NULL;
     decrKey[i] = NULL;
  }

  aciBin = NULL;
}

//destructor
cLinkManagerInfo::~cLinkManagerInfo() 
{
   freeSymKeys();
   delete aciBin;
}

//free any allocated sym keys
void cLinkManagerInfo::freeSymKeys() 
{
   const char* func = "cLinkManagerInfo::freeSymKeys";

   try {
      for (uint8_t i = 0; i < MAXLEKV; i++)
      {
         freeSymKeys(i);
      }
   }
   catch (CZKS::Exception& e) {
      CZKS::Push(e, func, "failure");
      CZKS::LogException(e);
   }
}

//free any allocated sym keys
void cLinkManagerInfo::freeSymKeys(uint8_t lekv) 
{
   const char* func = "cLinkManagerInfo::freeSymKeys";

   ZKS_ASSERT(ZKS_LINKMANAGER_MOD,  lekv < MAXLEKV);

   CZKS::Exception e;
   
   if (encrKey[lekv]) {
     if (encrKey[lekv] == decrKey[lekv]) {
        decrKey[lekv] = NULL;
     }
     if (!frCryptSymFree( e.zkError(), &encrKey[lekv])) {
        e.Rethrow(func, "fail");
     }
     encrKey[lekv] = NULL;
   }
   if (decrKey[lekv]) {
     if (!frCryptSymFree( e.zkError(), &decrKey[lekv])) {
        e.Rethrow(func, "fail");
     }
     decrKey[lekv] = NULL;
   }
}

//initialize the linkId
void cLinkManagerInfo::init(GenericIdBin& linkIdBin) 
{
  linkId = linkIdBin.pop(ZKS_LINKMANAGER_MOD);
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " linkid %x ", linkId.getID());
}

//initialize the aciBin
void cLinkManagerInfo::initAciBin(FrEntIdExp aipEntity) 
{
  aciBin = new cAciBin( aipEntity, entity);
}

//cleanup the timers and return the linkID to the bin
void cLinkManagerInfo::cleanup(GenericIdBin& linkIdBin) 
{
  cancelTimers();
  linkIdBin.push(linkId);
}

//cancel the expiry timer & associated event record
void cLinkManagerInfo::cancelExpiryTimer()
{

   if (pExpiryTimerProxy != NULL) {
      pExpiryTimerProxy->cancelTimer();
      delete pExpiryTimerProxy;
      pExpiryTimerProxy = NULL;
   }

   if (pExpiryEvent != NULL) {
      delete pExpiryEvent;
      pExpiryEvent = NULL;
   }

}

//cancel the renego timer & associated event record
void cLinkManagerInfo::cancelRenegoTimer()
{

   if (pRenegoTimerProxy != NULL) {
      pRenegoTimerProxy->cancelTimer();
      delete pRenegoTimerProxy;
      pRenegoTimerProxy = NULL;
   }

   if (pRenegoEvent != NULL) {
      delete pRenegoEvent;
      pRenegoEvent = NULL;
   }

}

//cancel all timers
void cLinkManagerInfo::cancelTimers()
{

   cancelExpiryTimer();
   cancelRenegoTimer();

}

//schedule a new expiry timer
void cLinkManagerInfo::scheduleExpiryTimer(czkCoreLoop* coreloop, 
                                           cLinkManager* pLinkManager, 
                                           cLinkManagerEventHandler* eh)
{
   const char* func = "cLinkManagerInfo::scheduleExpiryTimer";

   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, coreloop);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, pLinkManager);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, eh);

   //cancel it if it already exists
   cancelExpiryTimer();

   if (FRENT_TYPE_AIP == entity.type )
     ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " schedule expiry timer %s ", 
		    entity.name);
   else
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " schedule expiry timer ");

   try {
      pExpiryEvent = new cLinkManagerEventHandlerDataTimeout;
      pExpiryEvent->setLinkExpiryTimeout();
      pExpiryEvent->linkinfo = this;
      pExpiryEvent->pLinkManager = pLinkManager;
      pExpiryTimerProxy =
         coreloop->scheduleTimer( eh, pExpiryEvent, 
                                  pLinkManager->getLinkExpiryTimeInterval());
   }
   catch (CZKS::Exception &e) {
      e.Rethrow( func, "Failed to schedule Link Expiry Timeout");
   }
   catch (...) {
     CZKS::Rethrow( func, "Failed to schedule Link Expiry Timeout");
   }
}

//schedule a new renego timer
void cLinkManagerInfo::scheduleRenegoTimer(czkCoreLoop* coreloop, 
                                           cLinkManager* pLinkManager, 
                                           cLinkManagerEventHandler* eh)
{
   const char* func = "cLinkManagerInfo::scheduleRenegoTimer";
   (void) func;

   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, coreloop);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, pLinkManager);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, eh);

   DHRenegoTime = pLinkManager->getDHRenegoTimeInterval( state, DHRenegoTime);
   scheduleRenegoTimer(coreloop, pLinkManager, eh, DHRenegoTime); 

}

//schedule a new renego timer
void cLinkManagerInfo::scheduleRenegoTimer(czkCoreLoop* coreloop, 
                                           cLinkManager* pLinkManager, 
                                           cLinkManagerEventHandler* eh,
                                           ZKPRIntervalTime interval)
{
   const char* func = "cLinkManagerInfo::scheduleRenegoTimer";

   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, coreloop);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, pLinkManager);
   ZKS_ASSERT(ZKS_LINKMANAGER_MOD, eh);

   //only do this for other AIPs
   if (entity.type == FRENT_TYPE_AIP) {

      //cancel it if it already exists
      cancelRenegoTimer();

      if (FRENT_TYPE_AIP == entity.type) 
	ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " schedule retry nego timer %s ", 
		       entity.name);
      else
	ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " schedule retry nego timer ");

      try {
	 pRenegoEvent = new cLinkManagerEventHandlerDataTimeout;
         pRenegoEvent->setRetryNegoTimeout();
         pRenegoEvent->linkinfo = this;
         pRenegoEvent->pLinkManager = pLinkManager;
         pRenegoTimerProxy =
            coreloop->scheduleTimer( eh, pRenegoEvent, interval);
      }
      catch (CZKS::Exception &e) {
         e.Rethrow( func, "Failed to schedule DH Renego Timeout");
      }
      catch (...) {
         CZKS::Rethrow( func, "Failed to schedule DH Renego Timeout");
      }

   }

}

uint8_t cLinkManagerInfo::getNextLekv()  
{ 
   uint8_t next = lekv+1;
   if (next >= MAXLEKV) {
      next = 0;
   }
   return next; 
}
