#define REV_ID "@(#) $Id: testLinkManager.cpp,v 1.2 2001/04/18 17:37:06 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cLinkManager
//
// usage:      testLinkManager
//
//
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkspath.h>
#include <zkSignal.h>
    
#include <iostream>
#include <fstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkEventHandler.h"
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

//class cRouteCreator;
//class cAipKernel;
//class cLinkMaintainer;
//class cNetInfoProvider;

#include "cRouteCreator.h"
#include "cAipKernel.h"
#include "cLinkMaintainer.h"
#include "cDHNegotiator.h"
#include "cNetInfoProvider.h"
#include "cStatsHarvestor.h"
#include "cLinkManager.h"
#include "cConfigProvider.h"

#include "GenericId.h"
#include "cAciBin.h"

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

class cFakeEventHandlerData: public czkEventHandlerData {

public:

  czkCoreLoop*  pCoreLoop;
  cLinkManager* pLinkManager;
  DHNegoResult  result;
  int           testtype;
  FrEntIdExp    entity;

};
  
class cFakeEventHandler: public czkEventHandler {

  void    handleInput( czkSocket*, czkEventHandlerData*) {};
  void    handleOutput( czkSocket*, czkEventHandlerData*) {};
  void    handleException( czkSocket*, czkEventHandlerData*) {};
  int32_t handleTimeout( czkEventHandlerData* userData);

};

int32_t cFakeEventHandler::handleTimeout( czkEventHandlerData* userData)
{
   const char* func = "cFakeEventHandler::handleTimeout";

   CZKS::Exception e;

   cFakeEventHandlerData* p = (cFakeEventHandlerData*) userData;
   cLinkManager* pLinkManager = p->pLinkManager;

   FrEntIdExp entity = p->entity;

   int n = (rand()/(double)RAND_MAX*3.);

   DHNegoResult result = p->result;
   uint8_t version;
   uint8_t encrAlg;
   SymKey  encrKey;
   uint8_t decrAlg;
   SymKey  decrKey;
   ZKPRNetAddr dhaddr;
  
   try {

      if (p->testtype == 0) {
         ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " faking DHNego entity %s ", 
                                             entity.name);

         if (result == DHNegoPass) {

            if (!frCryptSymNew( e.zkError(), &encrKey)) {
               CZKS::Throw(e);
            }
            if (!frCryptSymNew( e.zkError(), &decrKey)) {
               CZKS::Throw(e);
            }
	 }
         version = 0;
         pLinkManager->negoResult( entity, result, version, encrKey, 
                                   decrKey, &dhaddr);
         std::cout << "ok" << std::endl;
      }
      else  if (p->testtype == 1) {

         ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " faking reportstate entity %s ", 
                                             entity.name);

 	 cLinkManagerInfo* linkinfo = pLinkManager->getLink(entity);
         if (linkinfo) {
	   pLinkManager->reportLinkState( linkDown, linkinfo); 
	 }
         std::cout << "ok" << std::endl;

      }

      else  if (p->testtype == 2) {
 
	 p->pCoreLoop->abortEventLoop();

      }

   }
   catch (CZKS::Exception &e) {

      std::cout << "not ok" << std::endl;
      CZKS::Push(e, func, "failure");
      CZKS::LogException(e);
      exit(0);

   }
   return 0;
}


int32_t main(int32_t argc, char **argv)
{ 
    const char* func = "testLinkManager";

    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit()  ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }


    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " starting LinkManager test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " running in single thread mode");
        bool dothread = false;
#endif

        czkCoreLoop* coreloop = NULL;
   
        coreloop = new czkCoreLoop();

        std::ifstream istrm("./testLinkManager.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_LINKMANAGER_MOD,
                      1,
                      "testLinkManager",
                      "Failed opening file: ./testLinkManager.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

         // start ConfigProvider component thread
        cConfigProvider* pConfigProvider = new cConfigProvider(m_cmdline, m_ini);
        pConfigProvider->validateConfig();
        mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);

         // start StatsHarvestor component thread
        cStatsHarvestor* pStatsHarvestor = new cStatsHarvestor();
        pStatsHarvestor->validateConfig();
        mgr.add(ZKS_STATSHARVESTOR_MOD, pStatsHarvestor);

        // start cAipKernel
        cAipKernel* pAipKernel = new cAipKernel();
        pAipKernel->validateConfig();
        mgr.add(ZKS_AIPKERNEL_MOD, pAipKernel);

        // start RouteCreator component thread
        cRouteCreator* pRouteCreator = new cRouteCreator();
        pRouteCreator->validateConfig();
        mgr.add(ZKS_ROUTECREATOR_MOD, pRouteCreator);

        // start DHNegotiator component thread
        cDHNegotiator* pDHNegotiator = new cDHNegotiator();
        pDHNegotiator->validateConfig();
        mgr.add(ZKS_DHNEGOTIATOR_MOD, pDHNegotiator);

        // start LinkMaintainer component thread
        cLinkMaintainer* pLinkMaintainer = new cLinkMaintainer();
        pLinkMaintainer->validateConfig();
        mgr.add(ZKS_LINKMAINTAINER_MOD, pLinkMaintainer);

        // start NetInfoProvider component thread
        cNetInfoProvider* pNetInfoProvider = new cNetInfoProvider();
        pNetInfoProvider->validateConfig();
        mgr.add(ZKS_NETINFOPROVIDER_MOD, pNetInfoProvider);

        // start LinkManager component thread
        cLinkManager* pLinkManager = new cLinkManager();
        mgr.add(ZKS_LINKMANAGER_MOD, pLinkManager);
        if (!pLinkManager->validateConfig()) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD,"validateConfig failed");
          abort();
	}


        mgr.initAllComponents(coreloop);

        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_STATSHARVESTOR_MOD, dothread);
        mgr.startComponent(ZKS_AIPKERNEL_MOD, dothread);
        mgr.startComponent(ZKS_ROUTECREATOR_MOD, dothread);
        mgr.startComponent(ZKS_DHNEGOTIATOR_MOD, dothread);
        mgr.startComponent(ZKS_LINKMAINTAINER_MOD, dothread);
        mgr.startComponent(ZKS_NETINFOPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_LINKMANAGER_MOD, dothread);

        pLinkManager->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);

	ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " got LinkManager ready signal ");

        cFakeEventHandler eh;
        czkIntervalTime ti;
	czkTimerProxy* pTimerProxy;

        cFakeEventHandlerData data;
        data.pLinkManager = pLinkManager;
        data.pCoreLoop = coreloop;

        data.testtype = 0;

        //client pass
        data.result = DHNegoPass;

        memset(data.entity.name, 0, sizeof( data.entity.name));
        data.entity.type = FRENT_TYPE_NYM;
        sprintf( data.entity.name,"pc%d\0",0);

	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data, 
                                    ti.getSeconds(1));
        delete pTimerProxy;

        //client fail
        cFakeEventHandlerData data2;
        data2.pLinkManager = pLinkManager;
        data2.pCoreLoop = coreloop;
        data2.testtype = 0;
        data2.result = DHNegoFail;
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data2, 
                                    ti.getSeconds(2));
        delete pTimerProxy;

        //aip pass
        cFakeEventHandlerData data3;
        data3.pLinkManager = pLinkManager;
        data3.pCoreLoop = coreloop;
        data3.testtype = 0;
        data3.result = DHNegoPass;
        data3.entity.type = FRENT_TYPE_AIP;
        memset(data3.entity.name, 0, sizeof( data3.entity.name));
        sprintf( data3.entity.name,"host%d\0",0);
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data3, 
                                    ti.getSeconds(3));
        delete pTimerProxy;

        //aip fail
        cFakeEventHandlerData data4;
        data4.pLinkManager = pLinkManager;
        data4.pCoreLoop = coreloop;
        data4.testtype = 0;
        data4.result = DHNegoFail;
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data4, 
                                    ti.getSeconds(4));
        delete pTimerProxy;

        //client report broken link
        cFakeEventHandlerData data5;
        data5.pLinkManager = pLinkManager;
        data5.pCoreLoop = coreloop;
        data5.testtype = 1;
        memset(data5.entity.name, 0, sizeof( data5.entity.name));
        data5.entity.type = FRENT_TYPE_NYM;
        sprintf( data5.entity.name,"pc%d\0",0);
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data5, 
                                    ti.getSeconds(5));
        delete pTimerProxy;

        //aip report broken link
        cFakeEventHandlerData data6;
        data6.pLinkManager = pLinkManager;
        data6.pCoreLoop = coreloop;
        data6.testtype = 1;
        memset(data6.entity.name, 0, sizeof( data6.entity.name));
        data6.entity.type = FRENT_TYPE_AIP;
        sprintf( data6.entity.name,"host%d\0",0);
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data6, 
                                    ti.getSeconds(6));
        delete pTimerProxy;

        //end test - abort event loop
        cFakeEventHandlerData data7;
        data7.pLinkManager = pLinkManager;
        data7.pCoreLoop = coreloop;
        data7.testtype = 2;
	pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data7, 
                                    ti.getSeconds(7));
        delete pTimerProxy;

        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;
        }
        else {


          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;

          // wait for all threads to complete

          mgr.waitForCompletion();
        }
      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_LINKMANAGER_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow(ZKS_LINKMANAGER_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      CZKS::Push(e, func, "failure");
      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();

    exit(1);
}


