#if !defined(_cNetInfoEntity)

#define _cNetInfoEntity

#include <list>

#include "frEnt.h"
#include "frNetInfo.h"

#include "czkNetAddr.h"


// Class to encapsulate subset of FrNetInfoDatum

class cNetInfoEntity {
 public:
  typedef list<cNetInfoEntity> entListType;

  // Class variable to be initialized to "NULL" by default constructor.
  static const cNetInfoEntity NullEntity;

 private:
  FrEntIdExp  idExp;
  ZKPRNetAddr netAddr;
  uint16_t    dhPort;
  uint32_t    idNum;


 public:
  // Constructor
  cNetInfoEntity();

  // Destructor
  virtual ~cNetInfoEntity();

  // Constructor derived from FrNetInfoDatum
  cNetInfoEntity(const FrNetInfoDatum);

  // Constructor for testing
  cNetInfoEntity(FrEntIdExp entity, ZKPRNetAddr addr, uint16_t dhport)
    {idExp = entity; netAddr = addr; dhPort = dhport; idNum = 0;}
  
  // Copy constructor
  cNetInfoEntity(const cNetInfoEntity&);

  // Assignment operator
  cNetInfoEntity& operator=(const cNetInfoEntity&);


  // Test for null instance
  bool isNull(void) const;

  // Print some entity info to standard out
  void dump(void) const;


  /*
   *  Public interface -- accessor methods
   */

  // Return entity type
  FrEntIdType getType() const {return idExp.type;}

  // Return name of entity (n.b., pointer to internal memory)
  const char *getName() const {return const_cast<char *> (idExp.name);}

  // Return numeric entity Id.
  uint32_t    getId() const {return idNum;}

  // Return reference to entity Id expression
  const FrEntIdExp &getIdExp() const {return idExp;}

  // Return entity's Diffie-Hellman port
  uint16_t    getDhPort() const {return dhPort;}

  // Return entity's Freedom port
  uint16_t    getFreedomPort() const 
    {return czkNetAddr::NtoHs(netAddr.inet.port);}

  // Return address of entity (IP, port, address family)
  const ZKPRNetAddr &getAddr() const {return netAddr;}
};

#endif /* _cNetInfoEntity */
