#if !defined(_cNetInfoProvider)

#define _cNetInfoProvider

/*
 * class cNetInfoProvider 
 *
 * NetInfoProvider thread will instantiate this class and run startup() method
 */

#include <list>
#include <vector>

#include "czkCoreLoop.h"
#include "czkNetAddr.h"
#include "czkIntervalTime.h"
#include "czkSockNativeTCP.h"

#include "cAipComponent.h"
#include "cAipComponentManager.h"
#include "cKeyProvider.h"

#include "aip-modules.h"

#include "InfoObj.h"

#include "cNetInfoEntity.h"


class cNetInfoData;
class cNetInfoProviderEventHandlerIO;
class cNetInfoWatchdog;


class cNetInfoProvider: public cAipComponent
{

 private:
  enum    nipEnum {
    NIP_DATA_UNUSED = -1, // Placeholder -- 1 before min defined value
    NIP_DATA_ENTITY,      // Network entity information
    NIP_DATA_MAX          // Placeholder -- 1 past max defined value
  };

  czkSockNativeTCP socket;

  cNetInfoWatchdog*               ehWatchdog;

  cNetInfoProviderEventHandlerIO* eventHandler;

  P_InfoObjMain infoObj;         // Info Object (NIDS interface)

  // Various netinfo data objects
  vector<cNetInfoData*>        niData; 

  // Retained state for round-robin access to core servers
  cNetInfoEntity::entListType *KQSList;
  cNetInfoEntity::entListType *NISSList;


 private: 
  // Copy constructor (disabled)
  cNetInfoProvider(const cNetInfoProvider& c): cAipComponent(c) 
    { assert(0); };

  // Assignment operator (disabled)
  cNetInfoProvider& operator=(const cNetInfoProvider&) 
    { assert(0); };

  // Acquire initial network information
  bool populate(time_t timeout);


 public:
  // constructor
  cNetInfoProvider();

  // destructor
  virtual ~cNetInfoProvider();

  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);
 
  // Startup
  void startup();

  // Shutdown 
  void shutdown();

  // cleanup private data
  void cleanupPrivateData();

  // Connect to the local NIDS
  bool connectNids(uint32_t timeout = 0);

  // Sever connection to local NIDS
  void disconnectNids(void);

  // Return connection status
  bool isConnected() {return (ZK_FALSE != infoObj->connected);}

  // Get KeyProvider reference
  cKeyProvider *getKeyProvider() 
    {return static_cast<cKeyProvider *> (pComponentManager->get(ZKS_KEYPROVIDER_MOD));}

  // Get ConfigProvider reference
  cConfigProvider *getConfigProvider() 
    {return static_cast<cConfigProvider *> (pComponentManager->get(ZKS_CONFIGPROVIDER_MOD));}

  // Reload internal structures with new, validated data
  void refresh();

  // Induce a NetInfoData object to poll NIDS for an update
  void prodNid(cNetInfoData *nid);


  /*
   *  Query methods.
   *
   *  These implement the API used by peer components at runtime.
   *
   *  The getNext...() functions implement a round-robin list for
   *  selected entity types.
   *
   *  Note that the internal lists are populated before the signature 
   *  on new incoming data is verified.  They are reverted to their
   *  previous values if the signature verification fails.
   *
   *  Functions returning a single entity object return a cNetInfoEntity
   *  reference.  Multiple values are provided in a dynamically allocated
   *  STL list of cNetInfoEntity objects, of which the caller assumes
   *  responsiblility for disposal.
   *
   *  See the cNetInfoEntity interface for a description of the object
   *  accessor methods.
   */

  // Return entity of specified type and name
  const cNetInfoEntity &getEntity(FrEntIdType entType, const char *name);

  // Return entity of specified type and id.
  const cNetInfoEntity &getEntity(FrEntIdType entType, uint32_t entId);

  // Return all entities of specified type
  cNetInfoEntity::entListType *getEntities(FrEntIdType entType,
					   bool bShuffled = false);

  // Return the next KQS entity (round-robin)
  cNetInfoEntity getNextKQS(void);

  // Return the next NISS entity (round-robin)
  cNetInfoEntity getNextNISS(void);

  // Return a list of AIP entities.
  cNetInfoEntity::entListType *getAIPs(void);
};

#endif /* _cNetInfoProvider */
