#if !defined(_cNetInfoDataEntities)

#define _cNetInfoDataEntities

#include <map>

#include "frEnt.h"
#include "frNetInfo.h"

#include "cNetInfoData.h"  // Base class

#include "cNetInfoEntity.h"



/*
 *  Derived class for network entity information
 */


class cNetInfoDataEntities: public cNetInfoData 
{
 private:
  typedef multimap<FrEntIdType, cNetInfoEntity* > entityMapType;

  time_t         lastUpdateTime;    // Time & version of currentData
  uint32_t       cacheDataVersion;
  entityMapType *m_CurrentData;     // Current network entity information
  bool           m_bCurrentValid;   // True if current data validated
  entityMapType *m_PreviousData;    // Previous network entity information
  bool           m_bPreviousValid;
  
 private:
  // Copy constructor (disabled)
  cNetInfoDataEntities(const cNetInfoDataEntities& e): cNetInfoData()
    { (void) e; assert(0); };

  // Assignment operator (disabled)
  cNetInfoDataEntities& operator=(const cNetInfoDataEntities& e) 
    { (void) e; assert(0); };

  void dispatchKeyReq(const FrNetInfo ni);
  void update(const FrNetInfo ni);
  void deleteMap(entityMapType **toKill);


 public:
  cNetInfoDataEntities();
  ~cNetInfoDataEntities();

  void init(void);
  // Method to refresh data store
  void trigger(czkSocket &socket);               
  void verify(uint8_t command, time_t timeStamp,
	      uint8_t *buffer, int32_t buflen);
  void validate(bool isValid);


  /*
   *  Accessor methods for individual entities or groups of
   *  entities of similar type.
   */
  // Return entity of specified type and name
  const cNetInfoEntity &getEntity(FrEntIdType entType, const char *name);

  // Return entity of specified type and id.
  const cNetInfoEntity &getEntity(FrEntIdType entType, uint32_t entId);
  
  // Return all entities of specified type
  cNetInfoEntity::entListType* getEntities(FrEntIdType entType,
					   bool bShuffled = false);
};

#endif /* _cNetInfoDataEntities */
