#include <string.h>

#include "zkslog.h"
#include "zksexception.h"

#include "cNetInfoEntity.h"

#define REV_ID "@(#) $Id: cNetInfoEntity.cpp,v 1.2 2001/04/18 17:37:11 alainc Exp $"
#include <zkrev.h>




// Class variable -- static "empty" instance to be built 
// with default constructor.
const cNetInfoEntity cNetInfoEntity::NullEntity;


// Default constructor
cNetInfoEntity::cNetInfoEntity():
  dhPort(0),
  idNum(0)
{
  idExp.type = FRENT_TYPE_UNUSED_;
  memset((void *) idExp.name, 0, FRENT_NAME_SIZE);
  
  // Set netAddr struct to known state
  PR_InitializeNetAddr(PR_IpAddrAny, 0, &netAddr);
}


// Destructor
cNetInfoEntity::~cNetInfoEntity()
{
  // No dynamic resources need to be freed.
}


// Constructor derived from FrNetInfoDatum
cNetInfoEntity::cNetInfoEntity(
  const FrNetInfoDatum niDatum)
{
  DECL_FUNC("cNetInfoEntity::cNetInfoEntity");
  CZKS::Exception e;

  try {
    /*
     *  Retrieve bits of information from FrNetInfoDatum structure
     */

    if (!frNetInfoDatumGetId(e.zkError(), niDatum, &idNum)) {
      CZKS::Throw(e);
    }

    if (!frNetInfoDatumGetEntity(e.zkError(), niDatum, &idExp)) {
      CZKS::Throw(e);
    }

    if (!frNetInfoDatumGetDifHelPort(e.zkError(), niDatum, &dhPort)) {
      CZKS::Throw(e);
    }

    
    // Convert FrNetInfoDatum's sockaddr_in to ZKPRNetAddr.inet
    struct sockaddr_in sockAddr;
    if (!frNetInfoDatumGetSockaddr(e.zkError(), niDatum, &sockAddr)) {
      CZKS::Throw(e);
    }

    // Set address to known state, then override required numeric
    // address and port fields
    PR_InitializeNetAddr(PR_IpAddrAny, 0, &netAddr);
    netAddr.inet.family = AF_INET;
    netAddr.inet.ip   = sockAddr.sin_addr.s_addr;
    netAddr.inet.port = sockAddr.sin_port;
  }
  catch (...) {
    CZKS::Rethrow(func, "failed");
  }
}


// Copy constructor
cNetInfoEntity::cNetInfoEntity(
  const cNetInfoEntity& toCopy)
{
  idNum               = toCopy.idNum;
  idExp               = toCopy.idExp;

  dhPort              = toCopy.dhPort;

  netAddr.inet.family = toCopy.netAddr.inet.family;
  netAddr.inet.ip     = toCopy.netAddr.inet.ip;
  netAddr.inet.port   = toCopy.netAddr.inet.port;
}


// Assignment operator
cNetInfoEntity& cNetInfoEntity::operator=(
  const cNetInfoEntity& toEquate)
{
  // Beware self-assignment!
  if (this != &toEquate) {
    idNum               = toEquate.idNum;
    idExp               = toEquate.idExp;
    
    dhPort              = toEquate.dhPort;

    netAddr.inet.family = toEquate.netAddr.inet.family;
    netAddr.inet.ip     = toEquate.netAddr.inet.ip;
    netAddr.inet.port   = toEquate.netAddr.inet.port;
  }

  return *this;
}


bool cNetInfoEntity::isNull(void) const
{
  return (idExp.type == FRENT_TYPE_UNUSED_);
}



void cNetInfoEntity::dump(void) const
{
  cout << "--> Dumping an entity: " << endl;
 
 if (isNull()) {
    cout << " NULL!!!" << endl;
  }
  else {
    const char *pName = getName();
    const char *pTypeName = frEntTypeGetStr(getType());
    
    cout << " Name \"" << (pName? pName: "") << "\"" << endl;
    cout << " Type \"" << (pTypeName? pTypeName: "") << "\""  << endl;
    cout << " Id   " << getId() << endl;
  }
}
