#include "cNetInfoKeyHandler.h"
#include "cNetInfoData.h"

#define REV_ID "@(#) $Id: cNetInfoKeyHandler.cpp,v 1.2 2001/04/18 17:37:11 alainc Exp $"
#include <zkrev.h>




void cNetInfoKeyHandler::handleEvent(
  cKeyProviderCacheEntryData entryData)
{
  DECL_FUNC("cNetInfoKeyHandler::handleEvent");

  CZKS::Exception zksException;
  cNetInfoSigData* databuf = NULL; 

 
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " In \"%s\"", func);

  try {
    try {
      //
      // First check the key state
      //
      FrKeyQryState state = entryData.getState();
      if (state != FRKEYQRY_ST_VALID) {
	CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, ZKS_NETINFOPROVIDER_KEYNOTVALID_ERR,
		    func, "Key is not valid");
      }


      //
      // Get the key (n.b., same method for cryption and signing).
      //
      FrCryptSig key = entryData.getPubKey();

      // Get the original query so we know what type of key it is
      FrKeyQryQueryExp qry = entryData.getQry();
     
      if (qry.keyType != FRCRYPT_KEYTYPE_SIGNING) {
	// Throw exception for invalid key type
	CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		    ZKS_NETINFOPROVIDER_KEYNOTVALID_ERR,
		    func, "Invalid key type");
      }


      //
      //  If it's a SIG key, time to validate pkt recv'd from NIQS
      //
    
      // Get (a copy of) the data pkt.
      databuf = (cNetInfoSigData*) getData(); 

      // Validate the signature
      int32_t valid = 1;
#if defined(ZKS_NETINFOPROVIDER_TEST)
#warning Signature verification stubbed out for testing.
      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, 
		     "Signature verification stubbed out for testing."); 
      (void) key;
#else
      if (!frSecSigVerify(zksException.zkError(), key, 
			  const_cast<uint8_t *>(databuf->getBuf()), 
			  databuf->getBufSiz(), 
			  &valid)) {
      
	CZKS::Throw(zksException);  
      }
#endif

      if (!valid) {
	CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		    ZKS_NETINFOPROVIDER_INVALIDSIGNATURE_ERR,
		    func, "Failed to validate NIQS signature");
      }

      // Normal case
      nid->validate(true);
    }
    catch (CZKS::Exception &e) {
      CZKS::Push(e, func, "failure");
      CZKS::LogException(e);

      // Roll back bogus information
      nid->validate(false);  
    }

    delete databuf;
    databuf = NULL;
  }
  catch (CZKS::Exception &e2) {
    // Will catch this exception if cannot roll back to previous netinfo.
    
    if (NULL != databuf) {
      delete databuf;
      databuf = NULL;
    }
    
    e2.Rethrow(func, "Fatal error.");
  }
}
