#include "czkIntervalTime.h"


#include "cNetInfoProvider.h"
#include "cNetInfoProviderEventHandlerIO.h"
#include "cNetInfoData.h"

#define REV_ID "@(#) $Id: cNetInfoProviderEventHandlerIO.cpp,v 1.2 2001/04/18 17:37:12 alainc Exp $"
#include <zkrev.h>




  
// Constructor
cNetInfoProviderEventHandlerIO::cNetInfoProviderEventHandlerIO() {}


// Destructor
cNetInfoProviderEventHandlerIO::~cNetInfoProviderEventHandlerIO() {}


// Set the pointer to the NetInfoProvider object
void cNetInfoProviderEventHandlerIO::setNetInfoProvider(cNetInfoProvider *nip)
{
  // Save a pointer to the NetInfoProvider object

  ZKS_ASSERT(ZKS_NETINFOPROVIDER_MOD, nip);

  pNetInfoProvider = nip;
}


// Handle incoming data from the netinfod
void cNetInfoProviderEventHandlerIO::handleInput(
  czkSocket* sock, 
  czkEventHandlerData* data
)
{
  DECL_FUNC("cNetInfoProviderEventHandlerIO::handleInput");

  CZKS::Exception zksException;

  uint8_t  *buffer = NULL;

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " entering \"%s\"", func);

  ZKS_ASSERT(ZKS_NETINFOPROVIDER_MOD, sock);
  ZKS_ASSERT(ZKS_NETINFOPROVIDER_MOD, data);  // Require a valid netInfoData reference

  // receiving a packet from the netinfod
  cNetInfoData *nid = static_cast<cNetInfoData *> (data);
  
  try {
    /*
     *  Steal internals of InfoObjClientReceive()
     */

    // Receive command token
    uint8_t cmd;
    int32_t gotBytes = sock->recv((char *) &cmd, sizeof(cmd), 0, PR_INTERVAL_NO_WAIT);
    // ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " pktlen %d recvd from netinfod",  pktlen);

    if (gotBytes < 0) {
      CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		  ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
		  func, "Error receiving data on socket: %d.",
		  gotBytes);
    }
 
    // Check if the packet has enough data
    if (gotBytes != sizeof(cmd)) {
      CZKS::Throw( ZKS_NETINFOPROVIDER_MOD, 
                   ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
                   func, "Received packet too small: %d.", 
		   gotBytes);
    }


    // Receive NIDS timestamp
    time_t tStamp;
    gotBytes = sock->recv((char *) &tStamp, sizeof(tStamp), 0, PR_INTERVAL_NO_WAIT);
    // ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " pktlen %d recvd from netinfod",  pktlen);

    if (gotBytes < 0) {
      CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		  ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
		  func, 
		  "Error receiving data on socket: %d.",
		  gotBytes);
    }
 
    // Check if the packet has enough data
    if (gotBytes != sizeof(tStamp)) {
      CZKS::Throw( ZKS_NETINFOPROVIDER_MOD, 
                   ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
                   func, 
		   "Received packet too small: %d",
		   gotBytes);
    }


    // Receive NIDS response size
    uint32_t buflen;
    gotBytes = sock->recv((char *) &buflen, sizeof(buflen), 0, PR_INTERVAL_NO_WAIT);
    // ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " pktlen %d recvd from netinfod",  pktlen);

    if (gotBytes < 0) {
      CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		  ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
		  func, 
		  "Error receiving data on socket: %d",
		  gotBytes);
    }
 
    // Check if the packet has enough data
    if (gotBytes != sizeof(buflen)) {
      CZKS::Throw( ZKS_NETINFOPROVIDER_MOD, 
                   ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
                   func, 
		   "Received packet too small: %d.",
		   gotBytes);
    }


    // Receive NIDS response packet
    buffer = new uint8_t[buflen];
    gotBytes = sock->recv((char *) buffer, buflen, 0, PR_INTERVAL_NO_WAIT);
    // ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " pktlen %d recvd from netinfod",  pktlen);

    if (gotBytes < 0) {
      CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		  ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
		  func, 
		  "Error receiving data on socket: %d.",
		  gotBytes);
    }
 
    // Check if the packet has enough data
    if ((uint32_t) gotBytes != buflen) {
      CZKS::Throw( ZKS_NETINFOPROVIDER_MOD, 
                   ZKS_NETINFOPROVIDER_SOCKRECV_ERR,
                   func, 
		   "Received packet too small: %d.",
		   gotBytes);
    }


    // Check whether data is novel; if so, create (pending)
    // netInfoData object and launch asynchronous signature
    // key query for verification.
    nid->verify(cmd, tStamp, buffer, buflen);
    delete buffer;
  }
  catch (CZKS::Exception &e) {
    delete buffer;
    CZKS::Push(zksException, func, "failure");
    CZKS::LogException(e);
  }
  catch (...) {
    delete buffer;
    CZKS::Rethrow(func, "failed");
  }
}



// Handle socket errors
void cNetInfoProviderEventHandlerIO::handleException( 
  czkSocket* sock, 
  czkEventHandlerData* data
)
{
  DECL_FUNC("cNetInfoProviderEventHandlerIO::handleException");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " socket i/o error %d", PR_GetError());

  if (PR_GetError() != 0) {
    try {
       CZKS::Throw(ZKS_NETINFOPROVIDER_MOD, 
		   ZKS_NETINFOPROVIDER_IO_ERR, func, 
                   "Socket i/o error %d.",
		   PR_GetError());

     }
     catch (CZKS::Exception &zksException) {
       CZKS::Push(zksException, func, "failure");
       CZKS::LogException(zksException);
     }
     catch (...) {
       CZKS::Rethrow(func, " failed");
     }
   }

  // Cleanup broken connection.
  pNetInfoProvider->disconnectNids();

  (void) sock; (void) data;
}


// Handle timeout events
int32_t cNetInfoProviderEventHandlerIO::handleTimeout(
  czkEventHandlerData* data)
{
  DECL_FUNC("cNetInfoProviderEventHandlerIO::handleTimeout");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " entering \"%s\"", func);

  cNetInfoData *niData = static_cast<cNetInfoData *> (data);

  int32_t reschedule = 0;

  try {
    ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " probe NIDS timeout");

    // Send probe to NIDS (netinfo daemon) for new data of appropriate type
    pNetInfoProvider->prodNid(niData);

    reschedule = 1;
  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException(e);
  }

  return reschedule;  
}
