#define REV_ID "@(#) $Id: testNetInfoProvider.cpp,v 1.2 2001/04/18 17:37:16 alainc Exp $"
#include <zkrev.h>

/*
 *  Unit test program for AIP NetInfoProvider component
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "zks.h"
#include "zksini.h"
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
    
#include <iostream>
#include <fstream>
#include <strstream> 
  
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"

#include "aip-modules.h"
#include "aip-params.h"

#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "cConfigProvider.h"

#include "cNetInfoProvider.h"
#include "cNetInfoEntity.h"


class SigHandler {

private:  
  czkCoreLoop* coreloop;

public:

  void setCoreLoop(czkCoreLoop* cl) { coreloop = cl; };

  void addSignal(int signum) {coreloop->addSignal(signum); }
};
 
SigHandler GlobalSigHandler;

void sighandler(
  int signum)
{
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " adding signal %d ", signum);
  GlobalSigHandler.addSignal( signum);
}


class cTestData: public czkEventHandlerData {
private:
  cNetInfoProvider *myNip;

  void dumpEntList(const cNetInfoEntity::entListType *entL)
  {
    cout << "===> Dumping an entity LIST: " << endl;
    cNetInfoEntity::entListType::const_iterator j = entL->begin();
    while (j != entL->end()) {
      (j++)->dump();
    }
  }

public:
  void init(cNetInfoProvider *pNip) 
  {
    myNip = pNip;
  }

  void update(void) 
  {
    int i;

    for (i = 0; i < 10; ++i) {
      cNetInfoEntity aKQD = myNip->getNextKQS();
      aKQD.dump();
    }

    for (i = 0; i < 10; ++i) {
      cNetInfoEntity aNISS = myNip->getNextNISS();
      aNISS.dump();
    }

    cNetInfoEntity::entListType *allAips = myNip->getAIPs();
    dumpEntList(allAips);
    delete allAips;

    // Once more, in random order
    allAips = myNip->getEntities(FRENT_TYPE_AIP, true);
    dumpEntList(allAips);
    delete allAips;

    // Test getting a bogus entity
    cNetInfoEntity anEnt = myNip->getEntity(FRENT_TYPE_NISS, "bogus");
    cout << "Bogus entity is " << (anEnt.isNull()? "NULL": "valid") << endl;
    anEnt.dump();

    // Test printing null entity
    cNetInfoEntity::NullEntity.dump();
  }
};


class cNiPuller: public czkEventHandler {

public:
  void handleInput(czkSocket*, czkEventHandlerData*) {};
  void handleOutput(czkSocket*, czkEventHandlerData*) {};
  void handleException(czkSocket*, czkEventHandlerData*) {};

  int32_t handleTimeout(czkEventHandlerData* data) 
  {
    cTestData* mydata = (cTestData *) data;
    mydata->update();
    return 1;
  }
};


int main(int argc, char* argv[])
{
  ZksErr zkserr = NULL;
  ZksLogAgent logAgent = NULL;
  CZKS::Ini ini;
  CZKS::CmdLine cmdline;

  int exitCode;

  (void) argc;

  CZKS::Exception zksException;

  cAipComponentManager mgr;

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
    return 1;
  }

  if (!zksErrNew(&zkserr)) {
    fprintf(stderr,"zksErrNew failed");
    exit(1);
  } 
    
  if (!zksLogAgentFileNew(zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
    fprintf(stderr, "%s: can't allocate the log agent.\n", 
	    "AIP");
    exitCode = EXIT_FAILURE;
    exit(exitCode);
  }


  try {
    try {

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, "Starting NetInfoProvider test.");

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " running in single thread mode");
      bool dothread = false;

      czkCoreLoop* coreloop = NULL;
   
      coreloop = new czkCoreLoop();

      GlobalSigHandler.setCoreLoop(coreloop);
      
      cmdline += CZKS::CmdArg("config", 
			      CZKS::CmdArg::NONE, 1, 1, 
			      "Full path configuration file name.");

      //
      // Parse and validate input arguments
      //
      cmdline.parse(argc, const_cast<const char**>(argv)).validate();

      if (cmdline.isSet("config")) {
	std::string inifile;

	//
	// Retreive the config file name from the command line
	//
	cmdline.valueOf("config", inifile);

	//
	// Open the config file
	//
	std::ifstream istrm(inifile.c_str());

	if (!istrm.is_open()) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, "Couldn't open config file.");
	  assert(0);
	}

	//
	// Load Ini object with the configuration file
	//
	istrm >> ini;

	//
	// Close the config file
	//
	istrm.close();
      }
      else {
	ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, "No config specified.");
	exit(1);
      }

      // Start ConfigProvider component thread
      cConfigProvider* pCfg = new cConfigProvider(cmdline, ini);
      mgr.add(ZKS_CONFIGPROVIDER_MOD, pCfg);
      pCfg->validateConfig();


      // Impose short NIDS poll interval for testing
      pCfg->setOpt_int("AIP", NIP_POLL_INTERVAL, 15);

      // Start NetInfoProvider component thread
      cNetInfoProvider* pNIP = new cNetInfoProvider();
      mgr.add(ZKS_NETINFOPROVIDER_MOD, pNIP);
      pNIP->validateConfig();


      //pNIP->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);

      // Start KeyProvider component
      cKeyProvider* pKeyProvider = new cKeyProvider();
      mgr.add(ZKS_KEYPROVIDER_MOD, pKeyProvider);
      pKeyProvider->validateConfig();


      mgr.initAllComponents(coreloop);

      mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, dothread);
      mgr.startComponent(ZKS_NETINFOPROVIDER_MOD, dothread);
      mgr.startComponent(ZKS_KEYPROVIDER_MOD, false);

      // Set up test environment.
      // Use fake event handler classes to provide & request netinfo
      cTestData myData;
      myData.init(pNIP);

      cNiPuller eh;
      czkIntervalTime ti;
      coreloop->scheduleTimer(&eh, &myData, ti.getSeconds(27));

      /*
       *  Tests to do:
       *    - successful/unsuccessful connect to netinfod (not running/bad configure)
       *    - fast/slow updates from netinfod
       *    - fast/slow iterations through lists of netinfo data
       *    - mal-formed queries; queries for non-existent entities/types
       *    - singletons, multiples, MIPAS
       */


      if (!dothread) {
	int32_t status = coreloop->runEventLoop();
	ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " exiting coreloop status %d ",status);
      }
      else {
	try {
	  // Do something insightful here
	}
	catch (CZKS::Exception &e) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " caught main ");
	  CZKS::LogException(e);
	}
	catch (...) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_NETINFOPROVIDER_MOD, " caught ... in main ");
	}
         
	mgr.waitForCompletion();
	// wait for all threads to complete

	//cAipComponent* pComponent = mgr.getFailedComponent();
	//while (pComponent)
	//{
	//  pComponent->cleanup();
	//  mgr.startComponent( pComponent, coreloop, dothread);
	//  pComponent = mgr.getFailedComponent();
	//}
      }
    }
    catch (CZKS::Exception e) {
      e.Rethrow( ZKS_NETINFOPROVIDER_MOD, "exiting main");
    }
    catch (...) {

      CZKS::Rethrow( ZKS_NETINFOPROVIDER_MOD, "exiting main");

    }
  }
  catch (CZKS::Exception e) {

    CZKS::LogException( e);
  }    
       
  // clean up all component objects
  mgr.deleteAllComponents();
}
