//
// cPktDispatcher class definition
// 
// 
#ifndef CPKTDISPATCHER_H
#define CPKTDISPATCHER_H

#include "czkSockDgram.h"
#include "czkNetAddr.h"
#include "cAipComponent.h"
#include "cAipKernel.h"
#include "cLinkManager.h"
#include "cStatsHarvestor.h"
#include "cPktDispatcherPktHandler.h"



// packet handler registration ID
typedef uint32_t PktDispatcherHandleId;


// class cPktDispatcher
// 
// PktDispatcher thread will instantiate this class and run startup method
// 
// other component threads will call one of registerHandle, removeHandle or
// send
// 
class cPktDispatcher : public cAipComponent
{
public:
    // class cPktDispatcherEventHandler
    // 
    // Embedded class used only be cPktDispatcher.  It handles the socket
    // events.
    // 
    class cPktDispatcherEventHandler : public czkEventHandler
    {
    public:
        // constructor
        cPktDispatcherEventHandler() { pd = NULL; };

        // destructor
        virtual ~cPktDispatcherEventHandler() {};

        // handle incoming packets
        void handleInput(czkSocket * sock, czkEventHandlerData * data);

        // handle socket exception
        void handleException(czkSocket * sock, czkEventHandlerData * data);

        // output event handler (disabled)
        void handleOutput(czkSocket *, czkEventHandlerData *) { assert(0); };

        // timeout event handler (disabled)
        int32_t handleTimeout(czkEventHandlerData *) { assert(0); };

        void setPktDispatcher(cPktDispatcher *pdin);

    private:
        // copy constructor (disabled)
        cPktDispatcherEventHandler(const cPktDispatcherEventHandler &) : 
            czkEventHandler() { assert(0); };

        // assigment operator (disabled)
        void operator=(const cPktDispatcherEventHandler &) { assert(0); };

    private:
        cPktDispatcher * pd;    // reference to packet dispatcher
    };

    class cPktDispatcherReg
    {
    public:
        // constructor
        cPktDispatcherReg() { registered = false; handler = NULL; };

        // destructor
        virtual ~cPktDispatcherReg() {};

        // copy constructor
        cPktDispatcherReg(const cPktDispatcherReg & p) { *this = p; };

        // assignment operator
        void operator=(const cPktDispatcherReg & p) 
        { 
            registered = p.registered;
            handler = p.handler;
        };

    public:
        bool                       registered;
        cPktDispatcherPktHandler * handler;
    };
    
public:
    // constructor
    cPktDispatcher();

    // destructor
    virtual ~cPktDispatcher();

    // validate configuration
    bool validateConfig();

    // initialize private data members
    bool initializePrivateData(czkCoreLoop * loop);

    // cleanup private data
    void cleanupPrivateData();

    // startup
    void startup(void);

    // shutdown
    void shutdown();
        
    // register a handler for a management packet type
    PktDispatcherHandleId registerHandler(FrPktType type, cPktDispatcherPktHandler * ph);

    // un-register a handler for a packet type
    void removeHandler(PktDispatcherHandleId);

    // wrap and send packet over a link
    // 
    // The given pkt must contain enough
    void send(cLinkManagerInfo * linkInfo, uint8_t * pkt, size_t pktSize);

    // handle reception of a packet over a socket
    void recv(czkSocket * socket);

    // handle a socket exception
    void exception(void);

private:
    // copy constructor (disabled)
    cPktDispatcher(const cPktDispatcher &) : cAipComponent() { assert(0); };

    // assignment operator (disabled)
    void operator=(const cPktDispatcher &) { assert(0); };

    void initInputSocket();
    void initOutputSocket();
    void reopenInputSocket();
    
private:
    typedef std::vector<cPktDispatcherReg> RegTable;

    cAipKernel               * pAipKernel;// reference to Kernel API
    cLinkManager             * pLinkManager; // reference to link manager
    cStatsHarvestor          * pStatsHarvestor; // reference to stats harvestor
    czkCoreLoop              * coreLoop;  // reference to core loop
    czkSocket                * inputSock; // incoming freedom pkt socket
    czkSockDgram             * outputSock;// outgoing freedom pkt socket
    size_t                     outputSockMaxSendSize; // Socket output buffer size
    cPktDispatcherEventHandler eh;        // event handler
    RegTable                   regTable;  // registration table
#ifdef AIPMULTITHREADED    
    czkLock                    tableLock; // registration table lock
#else
#define tableLock 0
#endif

    //stats
    cStatsHarvestorPktData* pdStats;  // stats "counted" by the PktDispatcher 

public:
    uint32_t getTxStats()              { return pdStats->pktSent->Val(); };
    uint32_t getRxStats()              { return pdStats->pktRcvd->Val(); };
    uint32_t getDropNoLinkInfoStats()  { return pdStats->pktDropped->Val(); };
    uint32_t getDropUnknownTypeStats() { return pdStats->pktDropped->Val(); };
    uint32_t getDropNotRegTypeStats()  { return pdStats->pktDropped->Val(); };
};

# ifdef __cplusplus
extern "C" {
# endif

# ifdef __cplusplus
}
# endif

#endif /* CPKTDISPATCHER_H */
