#define REV_ID "@(#) $Id: pktDispatcherUnitTest.cpp,v 1.2 2001/04/18 17:37:22 alainc Exp $"
#include <zkrev.h>
#define PKTDISPATCHER_UNIT_TEST

#include <iostream>
#include <unistd.h>
#include <netdb.h>
#include <fstream>
#include "frCrypt.h"
#include "aip-modules.h"
#include "zksexception.h"
#include "zksini.h"
#include <zkspath.h>
#include "czkCoreLoop.h"
#include "czkSockDgram.h"
#include "czkNetAddr.h"
#include "cAipComponentManager.h"
#include "cAipKernelPacketList.h"
#include "cAipKernel.h"
#include "cConfigProvider.h"
#include "cKeepAlivePkt.h"
#include "cLinkManager.h"
#include "cPktDispatcher.h"

namespace {

  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };
  
  const uint16_t UT_FREEDOM_PORT = 34567;
  const uint16_t UT_RX_NETLINK_PORT = 43210;
  const uint16_t UT_NEIGHBOR_PORT = 43211;
  const size_t UT_PKT_SIZE = 2000;
  const size_t UT_LINK_ID = 263244;

  // This is defined in aip-params.h, but I redefine it here because I am lazy.
  // Feel free to fix it, but realize that just adding the #include didn't
  // work for me.

  const char * AIP_IP_ADDRESS = "AIP_IPAddress";

  const string TEST1_STR = " Test 1:  registerHandle - duplicate type registration";
  const string TEST2_STR = " Test 2:  removeHandle - remove registered type";
  const string TEST3_STR = " Test 3:  exception - proper execution";
  const string TEST4_STR = " Test 4:  send - valid transmission";
  const string TEST5_STR = " Test 5:  recv - type not registered";
  const string TEST6_STR = " Test 6:  recv - type registered";
  const string TEST7_STR = " Test 7:  recv - type un-registered";
};

class cUnitTestPktHandler : public cPktDispatcherPktHandler
{
public:
    void handlePkt(cLinkManagerInfo * link, uint8_t * pkt, size_t pktSize);
};

static bool receivedPkt = false;

void cUnitTestPktHandler::handlePkt(cLinkManagerInfo*, uint8_t*, size_t)
{ receivedPkt = true; }

void ZKPR_CALLBACK runCoreLoop(void* arg)
{
    czkCoreLoop* coreloop = (czkCoreLoop*)arg;

    int32_t status = coreloop->runEventLoop();
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_PACKETDISPATCHER_MOD, " exiting coreloop status %d ",status);
}

class SigHandler {

private:
  czkCoreLoop* coreloop;

public:

  void setCoreLoop(czkCoreLoop* cl) { coreloop = cl; };

  void addSignal( int32_t signum) { coreloop->addSignal( signum); }
};

SigHandler GlobalSigHandler;

void sighandler( int32_t signum)
{
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_PACKETDISPATCHER_MOD, " adding signal %d ", signum);
  GlobalSigHandler.addSignal( signum);
}

cAciBin::cAciBin() {}
unsigned int cAciBin::getAci() { return 0; }
void cAciBin::returnAci(unsigned int) {}

GenericId::GenericId() {}
GenericIdBin::~GenericIdBin() {}
GenericIdBin::GenericIdBin(long unsigned int = GenericId::MIN_ID, 
                           long unsigned int = GenericId::MAX_ID) {}

class cLinkManagerInfoUnitTest : public cLinkManagerInfo
{
public:
    void setEncrKey(uint8_t alg);
    void setAddr(ZKPRNetAddr * addrin);
    void setLekv(uint8_t lekvin);
};

static cLinkManagerInfoUnitTest utLinkInfo;

cLinkManagerInfo::cLinkManagerInfo()
{
    CZKS::Exception e;
    assert(frCryptSymNew(e.zkError(), &encrKey[0]));
    assert(frCryptSymNew(e.zkError(), &encrKey[1]));
}

cLinkManagerInfo::~cLinkManagerInfo()
{
    CZKS::Exception e;
    frCryptSymFree(e.zkError(), &encrKey[0]);
    frCryptSymFree(e.zkError(), &encrKey[1]);
}

void cLinkManagerInfoUnitTest::setEncrKey(uint8_t alg)
{
    CZKS::Exception e;
 
    assert(frCryptSymSetAlg(e.zkError(), encrKey[0], (FrCryptSymAlg)alg));
    assert(frCryptSymSetAlg(e.zkError(), encrKey[1], (FrCryptSymAlg)alg));
    char buf[24];
    uint32_t len;
    uint32_t i;

    if (alg == FRCRYPT_SYMALG_DES3)
    {
        len = 24;
    }
    else if (alg == FRCRYPT_SYMALG_BLOWFISH)
    {
        len = 16;
    }

    for (i = 0; i < len; i++)
    {
        buf[i] = i;
    }
    assert(frCryptSymKeyFromData(e.zkError(), encrKey[0], buf, len));
    assert(frCryptSymKeyFromData(e.zkError(), encrKey[1], buf, len));
}

void cLinkManagerInfoUnitTest::setAddr(ZKPRNetAddr * addrin)
{
    memcpy(&addr, addrin, sizeof(ZKPRNetAddr));
}

void cLinkManagerInfoUnitTest::setLekv(uint8_t lekvin)
{
    lekv = lekvin;
}

void cLinkManagerInfo::cancelExpiryTimer() {}
void cLinkManagerInfo::cancelRenegoTimer() {}
void cLinkManagerInfo::cancelTimers() {}
void cLinkManagerInfo::scheduleExpiryTimer(czkCoreLoop*, cLinkManager*,
                                           cLinkManagerEventHandler*) {}
void cLinkManagerInfo::scheduleRenegoTimer(czkCoreLoop*, cLinkManager*,
                                           cLinkManagerEventHandler*) {}


cLinkManager::cLinkManager() {}
cLinkManager::~cLinkManager() {}
void cLinkManager::shutdown() {}
bool cLinkManager::validateConfig() { return true; }
bool cLinkManager::initializePrivateData(czkCoreLoop*) { return true; }
void cLinkManager::cleanupPrivateData() {}
void cLinkManager::startup(void) { signalReady(); }
void cLinkManager::openlock() {}
cLinkManagerInfo * cLinkManager::getLink(LinkId) { return(&utLinkInfo); }
void cLinkManager::closelock() {}

cAipKernel::cAipKernel(bool) {}
cAipKernel::~cAipKernel() {}
bool cAipKernel::validateConfig() { return true; }
bool cAipKernel::initializePrivateData(czkCoreLoop*) { return true; }
void cAipKernel::cleanupPrivateData() {}
void cAipKernel::startup(void) { signalReady(); }
void cAipKernel::shutdown() {}
void cAipKernel::handlePull(const char*, StatsDataBase*) {}

void cAipKernel::createSocket(czkSocket ** sock)
{
    ZKPRNetAddr netlinkAddr;
    czkNetAddr  netAddr;
    char        hostName[256];
    
    netlinkAddr.inet.family = ZKPR_AF_INET;
    assert(gethostname(hostName, 256) == 0);
    netAddr.getHostbyName(hostName, 256);
    netAddr.enumerateHostEnt(0, UT_RX_NETLINK_PORT, &netlinkAddr);

    czkSockDgram * mysock = new czkSockDgram;
    assert(mysock->open() == ZKPR_SUCCESS);
    assert(mysock->bind(netlinkAddr) == ZKPR_SUCCESS);

    *sock = mysock;
}

void cAipKernel::destroySocket(czkSocket ** sock)
{
    (*sock)->close();
    delete *sock;
}

czkSockDgram * cAipKernel::getFrSocket()
{

    uint16_t freedomPort = UT_FREEDOM_PORT;
    ZKPRSocketOptionData opt;
    czkNetAddr netAddr;
    ZKPRNetAddr sockAddr;
    char hostName[256];
    gethostname(hostName, 256);
    netAddr.getHostbyName(hostName, 256);
    netAddr.enumerateHostEnt(0, freedomPort, &sockAddr);
    
    assert(frSocket.open() == ZKPR_SUCCESS);
    assert(frSocket.bind(sockAddr) == ZKPR_SUCCESS);
    opt.option = ZKPR_SockOpt_Nonblocking;
    opt.value.non_blocking = 1;
    assert(frSocket.setOption(&opt) == ZKPR_SUCCESS);
    opt.option = ZKPR_SockOpt_RecvBufferSize;
    opt.value.recv_buffer_size = AIPKERNEL_RX_BUF_SIZE;
    assert(frSocket.setOption(&opt) == ZKPR_SUCCESS);
    opt.option = ZKPR_SockOpt_SendBufferSize;
    opt.value.recv_buffer_size = AIPKERNEL_TX_BUF_SIZE;
    assert(frSocket.setOption(&opt) == ZKPR_SUCCESS);

    return(&frSocket);
}

cAipKernelPacketList::cAipKernelPacketList() {}
cAipKernelPacketList::~cAipKernelPacketList() {}

void cAipKernelPacketList::recv(czkSocket & sock)
{
    char pkt[65536];
    ZKPRNetAddr fromAddr;
    czkSockDgram * mysock = (czkSockDgram *)&sock;

    mysock->recvfrom(&pkt[0], 65536, 0, &fromAddr, ZKPR_INTERVAL_NO_WAIT);
}


cConfigProvider::cConfigProvider(CZKS::CmdLine &, CZKS::Ini &) {}
cConfigProvider::~cConfigProvider() {}
bool cConfigProvider::validateConfig() { return true; }
bool cConfigProvider::initializePrivateData(czkCoreLoop*) { return true; }
void cConfigProvider::cleanupPrivateData() {}
void cConfigProvider::startup() {}
void cConfigProvider::shutdown() {}

int32_t cConfigProvider::getOpt_int(const char *section,
                                    const char *varname)
{
    if (strcmp(section, UAIP_SECTION_NAME) == 0)
    {
        if (strcmp(varname, AIP_FREEDOM_PORT) == 0)
        {
            return(UT_FREEDOM_PORT);
        }
        else
        {
            assert(0);
        }
    }
    else
    {
        assert(0);
    }
}


int32_t cConfigProvider::getOpt_int(const char *section,
                                    const char *varname,
                                    int32_t)
{
    return getOpt_int(section, varname);
}


string cConfigProvider::getOpt_string(const char *section,
                                   const char *varname)
{
    czkNetAddr netAddr;

    if (strcmp(section, UAIP_SECTION_NAME) == 0)
    {
        if (strcmp(varname, AIP_IP_ADDRESS) == 0)
        {
            ZKPRNetAddr addr;
            char hostName[256];
            gethostname(hostName, 256);
            netAddr.getHostbyName(hostName, 256);
            netAddr.enumerateHostEnt(0, 1000, &addr);
            struct in_addr ipAddr;
            ipAddr.s_addr = addr.inet.ip;
            char * hostIp = inet_ntoa(ipAddr);
            string hostIpAddr = hostIp;
            return hostIpAddr;
        }
        else
        {
            assert(0);
            string bogus = "BOGUS";
            return bogus;
        }
    }
    else
    {
        assert(0);
        string bogus = "BOGUS";
        return bogus;
    }
}

string cConfigProvider::getOpt_string(const char *section,
                                   const char *varname,
                                   string)
{
    return getOpt_string(section, varname);
}






#define UT_SRC_ADDR 0x55555555
#define UT_SRC_PORT 0xAAAA

LinkId utLinkId;
ZKPRNetAddr utSrcAddr;
char * utPkt;
int32_t utPktSize;
bool utPktRead = false;

void cAipKernelPacketList::getNextPacket(LinkId * linkId, ZKPRNetAddr * srcAddr,
                                         char ** pkt, int32_t * pktSize)
{
    if (!utPktRead)
    {
        *linkId = utLinkId;
        *srcAddr = utSrcAddr;
        *pkt = utPkt;
        *pktSize = utPktSize;
        utPktRead = true;
    }
    else
    {
        *linkId = 0;
        *pkt = NULL;
        *pktSize = 0;
    }
}


int main(int, char * argv[])
{

    const char         * func = "main";
    cUnitTestPktHandler  eh;
    ZksErr               zkserr = NULL;
    ZksLogAgent          logAgent = NULL;
    CZKS::Exception      zksException;
    cAipComponentManager mgr;

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
      return UT_FATAL;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      return UT_FATAL;
     }

    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n",
              "AIP");
      return UT_FATAL;
    }

    try
    {
        try
        {
            if (!frCryptInit(zksException.zkError(), 0)) {
                CZKS::Throw(zksException);
            }

            bool dothread = false;

            czkCoreLoop * coreloop = new czkCoreLoop;
            GlobalSigHandler.setCoreLoop(coreloop);

            // Start ConfigProvider
            CZKS::CmdLine  m_cmdline;
            CZKS::Ini      m_ini;
            std::ifstream file("./freedom.conf");
            file >> m_ini;
            cConfigProvider * pConfigProvider =
                                  new cConfigProvider(m_cmdline, m_ini);
            mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);

            // Start StatsHarvestor
            cStatsHarvestor * pStatsHarvestor = new cStatsHarvestor;
            mgr.add(ZKS_STATSHARVESTOR_MOD, pStatsHarvestor);

            // Start Aip Kernel
            cAipKernel * ak = new cAipKernel;
            mgr.add(ZKS_AIPKERNEL_MOD, ak);

            // Start Packet Dispatcher
            cPktDispatcher * pktDisp = new cPktDispatcher();
            cPktDispatcher & pd = *pktDisp;
            mgr.add(ZKS_PACKETDISPATCHER_MOD, &pd);
            pd.validateConfig();

            // Start Link Manager
            cLinkManager * lm = new cLinkManager();
            mgr.add(ZKS_LINKMANAGER_MOD, lm);

            mgr.initAllComponents(coreloop);

            mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, false);
            mgr.startComponent(ZKS_STATSHARVESTOR_MOD, false);
            mgr.startComponent(ZKS_AIPKERNEL_MOD, false);
            ak->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);
            mgr.startComponent(ZKS_PACKETDISPATCHER_MOD, dothread);
            pd.waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);
            mgr.startComponent(ZKS_LINKMANAGER_MOD, false);
            lm->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);
            // # of tests to be run.
            std::cout << "1..7" << endl;

            // Test #1
            pd.registerHandler(FRPKTTYPE_AIP_ROUTE_CREATE, &eh);
            try
            {
                pd.registerHandler(FRPKTTYPE_AIP_ROUTE_CREATE, &eh);
                std::cout << "not ok";
            }
            catch (CZKS::Exception &e)
            {
                std::cout << "ok";
            }
            std::cout << TEST1_STR << endl;

            // Test #2
            try {
              pd.removeHandler(FRPKTTYPE_AIP_ROUTE_CREATE);
              pd.registerHandler(FRPKTTYPE_AIP_ROUTE_CREATE, &eh);
              std::cout << "ok";
            }
            catch (CZKS::Exception & e) {
              std::cout << "not ok";
            }
            std::cout << TEST2_STR << endl;

            // Test #3
            // I think I edited this properly, I'm assuming that
            // pd.exception() is not supposed to throw.  If this is incorrect,
            // then it really should be changed.
            try
            {
                pd.exception();
                std::cout << "ok";
            }            
            catch (CZKS::Exception &e)
            {
              CZKS::Push(e, func, "failure");
                CZKS::LogException(e);
                std::cout << "not ok";
            }
            std::cout << TEST3_STR << endl;

            // Test #4
            czkSockDgram     utSock;
            ZKPRNetAddr      sockAddr;
            czkNetAddr       netAddr;
            uint8_t        * txPkt;
            size_t           txPktSize;
            uint8_t          rxPkt[UT_PKT_SIZE];
            int32_t          rxPktSize;
            cKeepAlivePkt    ka;
            uint8_t          lekv = 1;
            char             hostName[256];
            
            // open unit test socket (to simulate peer)
            assert(utSock.open() == ZKPR_SUCCESS);
            sockAddr.inet.family = ZKPR_AF_INET;

            
            assert(gethostname(hostName, 256) == 0);
            netAddr.getHostbyName(hostName, 256);
            netAddr.enumerateHostEnt(0, UT_NEIGHBOR_PORT, &sockAddr);
            assert(utSock.bind(sockAddr) == ZKPR_SUCCESS);

            // build packet
            ka.create(FRPKT_DATA_VERSION_1, FRPKT_PRIORITY_DEFAULT);
            ka.getBuffer(&txPkt, &txPktSize);

            // initialize link info
            utLinkInfo.setEncrKey(FRCRYPT_SYMALG_BLOWFISH);
            utLinkInfo.setAddr(&sockAddr);
            utLinkInfo.setLekv(lekv);

            // send packet
            pd.send(&utLinkInfo, txPkt, txPktSize);

            // recv packet (simulated peer)
            rxPktSize = utSock.recvfrom((char *)rxPkt, UT_PKT_SIZE, 0, &sockAddr, 
                                        ZKPR_INTERVAL_NO_WAIT);
            if (rxPktSize != (int32_t)txPktSize)
            {
              std::cout << "not ok" << " recv'd wrong sized pkt";
            }
            else
            {
                bool validHeader;
                bool validTrailer;
                CZKS::Exception e;
                
                // Decrypt pkt
                SymKey encrKey = utLinkInfo.getEncrKey();
                assert(frPktLinkDecrypt(e.zkError(), encrKey, rxPkt, rxPktSize));

                // Verify header + trailer were properly formatted
                validHeader = false;
                uint32_t i;
                for (i = 0; i < 8; i++)
                {
                    if (rxPkt[i] != txPkt[i])
                    {
                        validHeader = true;
                        break;
                    }
                }
                if (!validHeader)
                {
                  std::cout << "not ok" << " header wasn't modified before tx";
                }
                else
                {
                    validTrailer = true;
                    for (i = (txPktSize - 9); i < (txPktSize - 1); i++)
                    {
                        if (rxPkt[i] != 0x08)
                        {
                            validTrailer = false;
                            break;
                        }
                    }
                    if ((!validTrailer) || (rxPkt[txPktSize - 1] != lekv))
                    {
                      std::cout << "not ok" << " trailer wasn't formatted properly";
                    }
                    else
                    {
                        std::cout << "ok";
                    }
                } 
            }
            
            ka.destroy();
            std::cout << TEST4_STR << endl;

            // Test #5

            // Start thread that runs core loop
            czkThread threader;
            threader.spawn(runCoreLoop, 
                           (void*)coreloop,
                           (ZKPRThreadScope)0, 
                           (ZKPRThreadState)0, 
                           (ZKPRThreadPriority)0, 0);
            
            // Create keep alive packet
            ka.create(FRPKT_DATA_VERSION_1, FRPKT_PRIORITY_DEFAULT);
            ka.getBuffer(&txPkt, &txPktSize);

            // Config socket addr
            sockAddr.inet.family = ZKPR_AF_INET;
            assert(gethostname(hostName, 256) == 0);
            netAddr.getHostbyName(hostName, 256);
            netAddr.enumerateHostEnt(0, UT_RX_NETLINK_PORT, &sockAddr);

            // Send packet to freedom port
            receivedPkt = false;
            utPktRead = false;
            utPkt = (char *)txPkt;
            utPktSize = txPktSize;
            utLinkId = UT_LINK_ID;
            utSrcAddr.inet.family = AF_INET;
            utSrcAddr.inet.ip = UT_SRC_ADDR;
            utSrcAddr.inet.port = UT_SRC_PORT;
            uint32_t stats = pd.getDropNotRegTypeStats();
            assert(utSock.sendto((char *)txPkt, txPktSize, 0, &sockAddr, 
                                        ZKPR_INTERVAL_NO_WAIT) == (int32_t)txPktSize);

            // Check that packet was received
            sleep(1);
            if (!receivedPkt)
            {
                uint32_t newStats = pd.getDropNotRegTypeStats();
                if (newStats != (stats + 1))
                {
                  std::cout << "not ok" << " packets received";
                }
                else
                {
                    std::cout << "ok";
                }
            }
            else
            {
              std::cout << "not ok" << " received pkt while type not registered";
            }
            std::cout << TEST5_STR << endl;

            // Test #6

            // Register to receive keep alive type
            pd.registerHandler(FRPKTTYPE_AIP_ALIVE, &eh);

            // Send packet to freedom port
            receivedPkt = false;
            utPktRead = false;
            utPkt = (char *)txPkt;
            utPktSize = txPktSize;
            utLinkId = UT_LINK_ID;
            utSrcAddr.inet.family = AF_INET;
            utSrcAddr.inet.ip = UT_SRC_ADDR;
            utSrcAddr.inet.port = UT_SRC_PORT;
            stats = pd.getDropNotRegTypeStats();
            assert(utSock.sendto((char *)txPkt, txPktSize, 0, &sockAddr, 
                                        ZKPR_INTERVAL_NO_WAIT) == (int32_t)txPktSize);

            // Check that packet was received
            sleep(1);
            if (!receivedPkt)
            {
                uint32_t newStats = pd.getDropNotRegTypeStats();
                if (newStats == (stats + 1))
                {
                  std::cout << "not ok" << " packet received, but not forwarded to packet handler";
                }
                else
                {
                  std::cout << "not ok" << " packet not received";
                }
            }
            else
            {
                std::cout << "ok";
            }
            std::cout << TEST6_STR << endl;
            // Test #7

            pd.removeHandler(FRPKTTYPE_AIP_ALIVE);

            // Send packet to freedom port
            receivedPkt = false;
            utPktRead = false;
            utPkt = (char *)txPkt;
            utPktSize = txPktSize;
            utLinkId = UT_LINK_ID;
            utSrcAddr.inet.family = AF_INET;
            utSrcAddr.inet.ip = UT_SRC_ADDR;
            utSrcAddr.inet.port = UT_SRC_PORT;
            stats = pd.getDropNotRegTypeStats();
            assert(utSock.sendto((char *)txPkt, txPktSize, 0, &sockAddr, 
                                        ZKPR_INTERVAL_NO_WAIT) == (int32_t)txPktSize);

            // Check that packet was received
            sleep(1);
            if (!receivedPkt)
            {
                uint32_t newStats = pd.getDropNotRegTypeStats();
                if (newStats != (stats + 1))
                {
                  std::cout << "not ok" << " packets received";
                }
                else
                {
                    std::cout << "ok";
                }
            }
            else
            {
              std::cout << "not ok" << " received pkt while type not registered";
            }
            std::cout << TEST7_STR << endl;

            ka.destroy();
            utSock.close();
            coreloop->abortEventLoop();
            sleep(1);
            mgr.deleteAllComponents();    
            delete(lm);
            delete(pktDisp);
            delete(coreloop);
        }
        catch (...)
        {
            CZKS::Rethrow(func, "Unexpected problem occured");
        }
    }
    catch (CZKS::Exception &e)
    {
        CZKS::LogException(e);
        return UT_FATAL;
    }

    zksLogAgentFileFree(zkserr, &logAgent);
    zksErrFree(&zkserr);
    
    return UT_PASS;
}

