#ifndef _cPrivateKeys

#define _cPrivateKeys

#include "frEnt.h"
#include "frSec.h"

#include "aip-modules.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "czkLock.h"
#include "czkCoreLoop.h"

#include "cAipComponent.h"

#include "cConfigProvider.h"

class cPrivateKeys : public cAipComponent {

// class to provide access to the AIP's private crypt & signature keys

private:

FrSecKeyChain keys;  // structure to store key info
size_t sigHandle;    // handle to the signature key
size_t cryptHandle;  // handle to the crypt key
czkLock loadLock;

class cConfigProvider* pConfigProvider;

//copy constructor (disabled)
cPrivateKeys( const cPrivateKeys& c): cAipComponent(c) { assert(0); };

//assignment operator (disabled)
cPrivateKeys& operator=( const cPrivateKeys&) { assert(0); };

public:

//constructor
cPrivateKeys();

//destructor
virtual ~cPrivateKeys();

//initialize lock
void init();

//cleanup lock
void cleanup();

//methods to get config point values
string getAipName();
FrEntIdExp getAipEntity();   
string getAipPrivCrypt();
string getAipPrivSig();
string getPrompt();
string getPassfile();

bool validateConfig();

bool initializePrivateData(czkCoreLoop* pCoreLoopin);

void cleanupPrivateData();

//startup method
void startup();

//shutdown method (nothing to do in this case)
void shutdown() {};

//validate the entity associated with a key
void validateEntity( FrCryptPubKey key);

//reload the private keys
void reload();

//return the crypt key
void getCryptKey( uint32_t version, FrCryptPubKey* key);

//return the signature key
void getSigKey( uint32_t version, FrCryptPubKey* key);

};

#endif
