#define REV_ID "@(#) $Id: testPrivateKeys.cpp,v 1.2 2001/04/18 17:37:28 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cPrivateKeys
//
// usage:      testPrivateKeys 
//
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkspath.h>
#include <zkscmdline.h>
#include <zkSignal.h>
    
#include <iostream>
#include <fstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "cConfigProvider.h"

#include "aip-modules.h"

#include "cPrivateKeys.h"

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

class cFakeEventHandlerData: public czkEventHandlerData {

public:

  cPrivateKeys* pPrivateKeys;

};
  
class cFakeEventHandler: public czkEventHandler {

  void    handleInput( czkSocket*, czkEventHandlerData*) {};
  void    handleOutput( czkSocket*, czkEventHandlerData*) {};
  void    handleException( czkSocket*, czkEventHandlerData*) {};
  int32_t handleTimeout( czkEventHandlerData* userData);

};

int32_t cFakeEventHandler::handleTimeout( czkEventHandlerData* userData)
{

   cFakeEventHandlerData* p = (cFakeEventHandlerData*) userData;
   cPrivateKeys* pPrivateKeys = p->pPrivateKeys;

   try {
      pPrivateKeys->reload();
      FrCryptPubKey key;
      pPrivateKeys->getCryptKey( 0, &key);
      pPrivateKeys->validateEntity( key); 
      pPrivateKeys->getSigKey( 0, &key);
      pPrivateKeys->validateEntity( key); 
      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, 
                     " Private keys reload and validated ");
   }
   catch (CZKS::Exception &e) {
      CZKS::LogException(e);
   }
   return 1;
}

int32_t main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    char buf[2];
    strcpy(buf,"a");
    uint32_t c0 = (uint32_t)buf[0];
    uint32_t c1 = (uint32_t)buf[1];
    cout << " buf " << hex << (uint32_t)buf[0] << " " << (uint32_t)buf[1] << endl;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, " starting PrivateKeys test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, " running in single thread mode");
        bool dothread = false;
#endif

        czkCoreLoop* coreloop = NULL;
   
        coreloop = new czkCoreLoop(); 
  
        std::ifstream istrm("./testPrivateKeys.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_ROUTECREATOR_MOD,
                      1,
                      "FunctionalAip",
                      "Failed opening file: ./test.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

        // start ConfigProvider component
        cConfigProvider* pConfigProvider = new
	  cConfigProvider(m_cmdline, m_ini);
        mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);
        pConfigProvider->validateConfig();


        // start PrivateKeys component
        cPrivateKeys* pPrivateKeys = new cPrivateKeys();
        mgr.add(ZKS_PRIVATEKEYS_MOD, pPrivateKeys);
        if (!pPrivateKeys->validateConfig()) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, " validateConfig failed");
	  abort();
	}


        mgr.initAllComponents(coreloop);

        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, false);
        mgr.startComponent(ZKS_PRIVATEKEYS_MOD, false);

        cFakeEventHandler eh;
        czkIntervalTime ti;

        cFakeEventHandlerData data;
        data.pPrivateKeys = pPrivateKeys;

	czkTimerProxy* pTimerProxy = 
           coreloop->scheduleTimer( &eh, &data, 
                                    ti.getSeconds(10));

        delete pTimerProxy;

        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;
        }
        else {


          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;

          // wait for all threads to complete

          mgr.waitForCompletion();
        }

      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_PRIVATEKEYS_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow(ZKS_PRIVATEKEYS_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();
}


