#ifndef _cRouteCreator
#define _cRouteCreator

#include <map>

#include "czkCoreLoop.h"
#include "aip-modules.h"
#include "frEnt.h"
#include "frPktFmt.h"
#include "czkIntervalTime.h"
#include "czkLock.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"
#include "cAipKernel.h"
#include "cPrivateKeys.h"
#include "cLinkManager.h"
#include "cLinkManagerInfo.h"
#include "cPktDispatcher.h"
#include "cPktDispatcherPktHandler.h"
#include "cKeyProviderEventHandler.h"
#include "cKeyProvider.h"
#include "cAciBin.h"

#include "cRouteCreatePkt.h"
#include "cRouteCreateAckPkt.h"
#include "cRouteDestroyPkt.h"
#include "cKeepAlivePkt.h"

#include "cRouteCreatorInfo.h"
#include "cRouteCreatorErr.h"

class cFakeNetAddress;
class cRouteCreatorAciMap;

class cRouteCreator : public cAipComponent,
                      public cPktDispatcherPktHandler
                      
{

private:

  //"less than" function for the FrAci map definition
  struct less_FrAci
  {
    bool operator()(const FrAci r1, 
                    const FrAci r2) const
    {
       if( r1 < r2 ) {
          return true;
       }
       else {
          return false;
       }
    }
  };

    //"less than" function for the LinkId map definition
    struct less_LinkId
    {
        bool operator()(const LinkId r1, 
                        const LinkId r2) const
        {
            if( r1 < r2 )
                return true;
            else
                return false;
        }
    };

    // Event classes
    
    class cRouteCreatorKeyHandlerData : public czkEventHandlerData
    {
    public:
        
        LinkId linkId;
        FrAci  aci;
        cRouteCreatePkt* rcPkt;
        
        cRouteCreatorKeyHandlerData( LinkId inLinkId, FrAci inAci, 
                                     cRouteCreatePkt* inRcPkt) 
        {
            this->linkId = inLinkId;
            this->aci    = inAci;
            this->rcPkt  = inRcPkt;
        }
        
        virtual ~cRouteCreatorKeyHandlerData() {};
        
     };
    
    class cRouteCreatorKeyHandler : public cKeyProviderEventHandler
    {
    public:
        
        cRouteCreator *pRouteCreator;
        
        cRouteCreatorKeyHandler( cRouteCreator *inRouteCreator )
        {
	    ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, inRouteCreator);
            this->pRouteCreator = inRouteCreator;
        }
        
        virtual ~cRouteCreatorKeyHandler() {};
        
        void handleEvent(cKeyProviderCacheEntryData entryData) 
        {
	    cRouteCreatorKeyHandlerData* pData = 
               (cRouteCreatorKeyHandlerData*)getData();

            ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pData);

            pRouteCreator->handleKeyChangedEvent(entryData, 
                                                 pData->linkId, pData->aci,
                                                 pData->rcPkt);

            delete pData;
        };
        
    };
    

    //class to track usage of Worm Aci & associated nym entity
    class cRouteCreatorWormAci {

    public:

      uint32_t           count;
      FrEntIdExp         nymEntity;
      bool               fakeAddrSet;
      bool               authRoute;
      uint8_t            pktVersion;
      ZKPRNetAddr        fakeAddr;
      cRouteCreatorInfo* pOldRouteInfo;

      cRouteCreatorWormAci() 
         { count = 0; fakeAddrSet = false;};
      cRouteCreatorWormAci(uint32_t countin) 
         { count = countin; fakeAddrSet = false;};
      cRouteCreatorWormAci(uint32_t countin, FrEntIdExp nym) 
         { count = countin; nymEntity = nym; fakeAddrSet = false; };
      void setFakeAddr( ZKPRNetAddr addr, bool auth, uint8_t version) 
	 { fakeAddr = addr; fakeAddrSet = true; 
           authRoute = auth; pktVersion = version; };
      virtual ~cRouteCreatorWormAci() {};
    };

    typedef map<LinkId, cRouteCreatorAciMap*, less_LinkId> RouteTableLinkIdDef;
    typedef map<FrAci,  cRouteCreatorWormAci, less_FrAci> WormAciTableDef;
      
    czkLock             routeTableLock;
    RouteTableLinkIdDef routeTableLinkId;

    GenericIdBin        routeIdBin; //id bin for routeId

    cPktDispatcher      *pPktDispatcher;
    cKeyProvider        *pKeyProvider;
    cAipKernel          *pAipKernel;
    cPrivateKeys        *pPrivateKeys;
    cLinkManager        *pLinkManager;
    cConfigProvider     *pConfigProvider;

    cAciBin             wormAciBin;   //bin to generate wormhole aci's
    cFakeNetAddress*    authFakeAddrBinV2; //bin to gen fake addr for auth routes
    cFakeNetAddress*    unauthFakeAddrBinV2; //bin to gen fake addr for unauth routes
    cFakeNetAddress*    authFakeAddrBinV1; //bin to gen fake addr for V1 auth routes
    cFakeNetAddress*    unauthFakeAddrBinV1; //bin to gen fake addr for V1 unauth routes
    WormAciTableDef     wormAciTable;  

    bool                refuseConnections; // if true, new routes cannot be created

    //event handler to handle DH renego timeouts 
    class cRouteCreatorTimeoutHandler* routeExpiryEH;

    //event handler to handler link expiry timeouts
    class cRouteCreatorTimeoutHandler* routeAckEH;
 
    //route counter
    uint32_t            nRoute;

    //private methods
    void sendRouteDestroy( cLinkManagerInfo* linkInfo,
                           uint8_t pktVersion, FrAci aci,
                           uint32_t errcode, FrEntIdExp entity, 
                           FrCryptSym symKey);

    void passRouteDestroy( cLinkManagerInfo* linkInfo,
                           cRouteDestroyPkt* pkt, 
                           FrAci nextHopAci, FrCryptSym symKey);

    void sendRouteCreateAck( cLinkManagerInfo* linkInfo,
                             uint8_t pktVersion, 
                             FrAci exitAci, FrAci nextHopAci,
                             FrCryptMacKey& macKey, FrCryptSym symKey);

    void passRouteCreateAck( cLinkManagerInfo* linkInfo,
                             cRouteCreateAckPkt* pkt, 
                             FrAci nextHopAci, FrCryptSym symKey);

    void sendRouteCreate( cLinkManagerInfo* linkInfo,
                          cRouteCreatePkt* pkt, FrAci nextHopAci);


    void requestNymKey(FrEntIdExp entity, LinkId linkId, FrAci aci, 
                       cRouteCreatePkt* rcPkt);

    void insertRouteTableEntry( LinkId linkId, FrAci aci, 
                                cRouteCreatorAciMap* pAciMap,
                                cRouteCreatorInfo* pNewRouteInfo);

    void deleteRouteTableEntry( cRouteCreatorInfo* pRouteInfo,
                                cRouteCreatorAciMap* pAciMap,
                                FrAci aci, FrAci nextAci,
                                LinkId nextLinkId);

    void handleRouteCreatePkt( cLinkManagerInfo* linkInfo,
                               cRouteCreatePkt* rcPkt, 
                               ZKPRNetAddr addrin,
                               bool routeExists,
                               LinkId linkId, FrAci aci,
                               cRouteCreatorAciMap* pAciMap);

    void handleRouteCreateAckPkt( cRouteCreateAckPkt* rcaPkt, 
                                  bool routeExists,
                                  cRouteCreatorInfo* pRouteInfo,
                                  RouteState curRouteState);

    void handleRouteDestroyPkt( cRouteDestroyPkt* rdPkt, 
                                bool routeExists,
                                FrAci aci,
                                cRouteCreatorAciMap* pAciMap,
                                cRouteCreatorInfo* pRouteInfo,
                                RouteState curRouteState);

    void handleRouteAckTimeout(cRouteCreatorInfo* pRouteInfo);

    void handleRouteExpiryTimeout(cRouteCreatorInfo* pRouteInfo);

    cRouteCreatorAciMap* findAciMap( LinkId linkId);

    cRouteCreatorInfo* newRouteInfo();

    bool validateWormAci( FrAci aci);

    void returnWormAci( FrAci aci);

    //copy constructor (disabled)
    cRouteCreator( const cRouteCreator& c) : cAipComponent(c), cPktDispatcherPktHandler() { assert(0); };

    //assignment operator (disabled)
    cRouteCreator& operator=( const cRouteCreator&) { assert(0); };

public:

    cRouteCreator();
    virtual ~cRouteCreator();

    void shutdown();

    //methods to get config point values
    string getAipName();
    FrEntIdExp getAipEntity();
    string getAuthNetAddrMaskV2();
    string getUnauthNetAddrMaskV2();
    string getAuthNetAddrMaskV1();
    string getUnauthNetAddrMaskV1();
    int32_t getMaxRoutes();
    int32_t getMaxRouteExpiryTime();
    int32_t getRouteAckTime();

    // cRouteCreator methods
    bool validateConfig();
    bool initializePrivateData(czkCoreLoop* coreloopin);
    void cleanupPrivateData();
    void startup();

    void reportBrokenLink( LinkId linkId );
    void handlePkt( cLinkManagerInfo *linkInfo, 
                    uint8_t *pPkt, size_t pktSize );

    void handleRouteTimeoutEvent(czkEventHandlerData *pData); 

    void handleKeyChangedEvent( cKeyProviderCacheEntryData &entryData,
                                LinkId linkId, FrAci aci, 
                                cRouteCreatePkt* rcPkt);

    //return the route expiry time
    ZKPRIntervalTime getRouteExpiryTimeInterval(uint32_t delta);

    //return the route Ack time
    ZKPRIntervalTime getRouteAckTimeInterval();

    //toggle the state of the refuseConnections value
    void toggleRefuseConnections() { refuseConnections = !refuseConnections; }

};

#endif
