#ifndef _cRouteCreatorInfo
#define _cRouteCreatorInfo

#include "aip-modules.h"
#include "frEnt.h"
#include "frPktFmt.h"

#include "cRouteCreatePkt.h"

#include "cLinkManagerInfo.h"
#include "GenericId.h"

typedef uint32_t RouteId;

enum RouteState { intHopPendingAck, intHopRouteUp, 
                  lastHopPendingKey, lastHopRouteUp };

enum RouteType { fwdRoute, bwdRoute, wormholeRoute, extendedRoute };

class cRouteCreator;
class cRouteCreatorTimeoutHandler;

class cRouteCreatorInfo
{
    friend class cRouteCreator;
    
    class cRouteCreatorIPinfo {
 
        friend class cRouteCreatorInfo;
    
        private:

        LinkId      linkId;
        FrAci       aci;
        ZKPRNetAddr addr; 
    };

    private:

    GenericId           routeId;       

    uint8_t             pktVersion;

    cRouteCreatorIPinfo fwdIPinfo;        

    cRouteCreatorIPinfo bwdIPinfo;        

    RouteState          routeState;

    uint32_t            inTime;

    uint32_t            expTime;

    uint32_t            flags;

    RouteType           routeType;

    ZKPRNetAddr         fakeAddr;

    cRouteCreatePkt*    rcPkt;

    czkTimerProxy* pRouteExpiryTimerProxy;  //needed to cancel expiry timer
    czkTimerProxy* pRouteAckTimerProxy;     //needed to cancel ack timer

    class cRouteCreatorTimeoutHandlerData* pRouteExpiryEvent;  //expiry data
    class cRouteCreatorTimeoutHandlerData* pRouteAckEvent;     //ack data

    //copy constructor (disabled)
    cRouteCreatorInfo( const cRouteCreatorInfo&) { assert(0); };

    //assignment operator (disabled)
    cRouteCreatorInfo& operator=( const cRouteCreatorInfo&) { assert(0); };

    public:

    cRouteCreatorInfo();
    virtual ~cRouteCreatorInfo();

    //init
    void init(GenericIdBin& linkIdBin);

    //cleanup
    void cleanup(GenericIdBin& linkIdBin); 

    //cancel a pending Route Expiry Timer
    void cancelRouteExpiryTimer();

    //cancel a pending Renego Timer
    void cancelRouteAckTimer();

    //cancel all pending timers
    void cancelTimers();

    void scheduleRouteExpiryTimer(czkCoreLoop* coreloop, 
                                  cRouteCreator* pRouteCreator, 
                                  cRouteCreatorTimeoutHandler* eh);

    void scheduleRouteAckTimer(czkCoreLoop* coreloop, 
                               cRouteCreator* pRouteCreator, 
                               cRouteCreatorTimeoutHandler* eh);

    RouteId      getRouteId()    { return routeId.getID(); };
    uint8_t      getPktVersion() { return pktVersion; };
    FrCryptMacKey& getMacKey()   { return rcPkt->getMacKey(); }

    LinkId       getFwdLinkId()  { return fwdIPinfo.linkId; };
    ZKPRNetAddr& getFwdAddr()    { return fwdIPinfo.addr; };    
    FrAci&       getFwdAci()     { return fwdIPinfo.aci; };    
    FrCryptSym&  getFwdCrypt()   { return rcPkt->getFwdCrypt(); }

    LinkId       getBwdLinkId()  { return bwdIPinfo.linkId; };
    ZKPRNetAddr& getBwdAddr()    { return bwdIPinfo.addr; };    
    FrAci&       getBwdAci()     { return bwdIPinfo.aci; };   
    FrCryptSym&  getBwdCrypt()   { return rcPkt->getBckCrypt(); }
 
    RouteState&  getRouteState() { return routeState; };
    uint32_t     getFlags()      { return flags; };
   
    ZKPRNetAddr& getFakeAddr()   { return fakeAddr; };    

    void setFwd(LinkId linkId, FrAci aci, ZKPRNetAddr addr);
    void setBwd(LinkId linkId, FrAci aci, ZKPRNetAddr addr);
    void swapBwdFwd(); 

    void setExpTime( uint32_t exptime, uint32_t routeAckTime);

    void setRcPkt( cRouteCreatePkt* pkt)
      { rcPkt = pkt; pkt->incRefcount(); }

    void deleteRcPkt( )
      { if (rcPkt->decRefcount() == 0) delete rcPkt; }

};


#endif
