#include "cFakeNetAddress.h"
#include "czkNetAddr.h"

#define NBITS 8

#define REV_ID "@(#) $Id: cFakeNetAddress.cpp,v 1.2 2001/04/18 17:37:32 alainc Exp $"
#include <zkrev.h>





//constructor
cFakeNetAddress::cFakeNetAddress()
{
  const char* func = "cFakeNetAddress::cFakeNetAddress"; (void) func;
 
  validated = false;

  netAddrMask = "";
  nBits = 0;
  netAddrBin = NULL;

}

//destructor
cFakeNetAddress::~cFakeNetAddress()
{
  cleanup();
}   

//parse netAddrMask
void cFakeNetAddress::parse(
  const char* netAddrMaskIn)
{
  DECL_FUNC("cFakeNetAddress::parse");
 

  try {
    if (NULL == netAddrMaskIn) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input netaddrmask reference.");
    }

    string temp = netAddrMaskIn;

    int32_t n = temp.length();
    int32_t start, stop;

    string separator = "/";
    start = temp.find_first_not_of(separator);
    if (start >= 0 && start < n) {
      stop = temp.find_first_of(separator, start);
      if ((stop < 0) || (stop > n)) stop = n;

      netAddrMask = temp.substr(start, stop - start);

      if (stop < n) {
	start = temp.find_first_not_of(separator, stop+1);
    
	if (start >= 0 && start < n) {
	  stop = temp.find_first_of(separator, start);
	  if ((stop < 0) || (stop > n)) stop = n;
	  nBits = atoi(temp.substr(start, stop - start).c_str());
	}
      }
    }

    if (netAddrMask == "" || nBits <= 0) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NETADDRMASKPARSE_ERR,
		  func, "Unable to parse netAddrMask");
    }

    n = netAddrMask.length();
    separator = ".";
    start = netAddrMask.find_first_of(separator);
    uint32_t count = 0;
    while (start >= 0 && start < n) 
    {
	count++;
	start = netAddrMask.find_first_of(separator, start+1);
    }
    while (count < 3)
    {
	netAddrMask.append(".0");
	count++;
    }

    czkNetAddr netAddr;
    if (netAddr.getNetAddr(netAddrMask.c_str(), &baseAddr) != ZKPR_SUCCESS) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NETADDRMASKPARSE_ERR,
		  func, "Unable to convert netAddrMask to IPaddr");
    } 
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }

  validated = true;
}


//init bin for FakeNetAddr allocatoin
void cFakeNetAddress::init()
{
  const char* func = "cFakeNetAddress::init"; (void) func;

  if (validated) {
     maxAddr = (1<<nBits)-1;
     nFree = maxAddr+1;
     nBytes = nFree/(NBITS*sizeof(uint8_t));
     netAddrBin = new uint8_t[nBytes];
     memset( netAddrBin, 0, nBytes);
  }

}

void cFakeNetAddress::cleanup()
{

  delete netAddrBin;
  netAddrBin = NULL;

  validated = false;

}

//get fake address
ZKPRNetAddr cFakeNetAddress::getFakeNetAddr()
{
  CZKS::Exception e;
  const char* func = " cFakeNetAddress::getFakeNetAddr";
  const size_t MAXTRY = 100;

  uint8_t inuse = 1;
  size_t ntry = 0;
  uint32_t addr;
  uint32_t bytpos;
  uint32_t bitpos;

  ZKPRNetAddr fakeNetAddr;

  if (nFree == 0) {
    CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NETADDRNONELEFT_ERR,
                func, "no free fakeNetAddr left");
  }

  // find unused addr (try one random value (in range)) 
  while (inuse && ntry < MAXTRY)
  {  
    if ( ! zkRandGetUInt32InRange(e.zkError(), 0, maxAddr, &addr)) {
      e.Rethrow( func, "error getting random addr");
    }
    else {
      bytpos = addr/NBITS;
      bitpos = addr - bytpos*NBITS;
      inuse = (netAddrBin[bytpos] & (1<<bitpos));
    }
  }

  // ok, if no good, find out how many free left
  //   generate a random number 'n' in that range and 
  //   then pick the 'n'th free one 

  if (inuse) {
    uint32_t nFind = 0;
    uint32_t n = 0;

    if (nFree > 1) {
      if ( ! zkRandGetUInt32InRange(e.zkError(), 1, nFree, &nFind)) {
         e.Rethrow( func, "error getting random number");
      }
    }
    else {
      nFind = 1;
    }
    n = 0;
    bytpos = 0;
    while (n < nFind && bytpos < nBytes)
    {
      if (netAddrBin[bytpos] != 0xFF) {
        bitpos = 0;
        while (n < nFind && (int)bitpos < NBITS)        
        {
          if (!(netAddrBin[bytpos] & (1<<bitpos))) {
            nFind++;
	  }
          if (n < nFind) {
            bitpos++;
	  }
	}
      }
      if (n < nFind) {
        bytpos++;
      }
    } 
    if (n == nFind) {
      inuse = (netAddrBin[bytpos] & (1<<bitpos));
    }
  }


  /* ooops! can't find a free one */
  if (inuse) {
    CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NETADDRNOTFREE_ERR,
                func, "Unable to find free fakeNetAddr");
  }

  /* if not inuse, then we've found a fakeNetAddr to return */
  /* mark it inuse, and decrement free counter */
  if (!inuse) {
    netAddrBin[bytpos] |= (1<<bitpos);
    nFree--;
    fakeNetAddr = baseAddr;

    czkNetAddr na;
    fakeNetAddr.inet.ip |= na.HtoNl(bytpos*NBITS + bitpos);
  }

  return fakeNetAddr;

}   

//return fakeNetAddr
void cFakeNetAddress::returnFakeNetAddr(
  ZKPRNetAddr addr)
{
  DECL_FUNC("cFakeNetAddress::returnFakeNetAddr");

  
  czkNetAddr na;
  uint32_t n = na.NtoHl(addr.inet.ip & ~baseAddr.inet.ip);

  uint32_t bytpos = n / NBITS;
  uint32_t bitpos = n - bytpos*NBITS;

  uint8_t inuse = (netAddrBin[bytpos] & (1<<bitpos));
  if (!inuse) {
    CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NETADDRALREADYFREE_ERR,
                func, "Unable to return fakeNetAddr");
  }

  netAddrBin[bytpos] &= ((1<<bitpos) ^ 0xff);
  nFree++;

}     
