#include "cRouteCreator.h"
#include "cRouteCreatorAciMap.h"
#include "cRouteCreatorScopeLock.h"
#include "cRouteCreatorTimeoutHandler.h"
#include "cFakeNetAddress.h"


#define REV_ID "@(#) $Id: cRouteCreator.cpp,v 1.4 2001/06/07 15:44:21 frankf Exp $"
#include <zkrev.h>



cRouteCreator::cRouteCreator()
{
  const char* func = "cRouteCreator::cRouteCreator"; (void) func;

    routeAckEH = NULL;
    routeExpiryEH = NULL;

    authFakeAddrBinV2   = new cFakeNetAddress;
    unauthFakeAddrBinV2 = new cFakeNetAddress;
    authFakeAddrBinV1   = new cFakeNetAddress;
    unauthFakeAddrBinV1 = new cFakeNetAddress;

    refuseConnections = false;
   
}


cRouteCreator::~cRouteCreator()
{
    delete authFakeAddrBinV2;
    delete unauthFakeAddrBinV2;
    delete authFakeAddrBinV1;
    delete unauthFakeAddrBinV1;

}


string cRouteCreator::getAipName()
{ 
  DECL_FUNC("cRouteCreator::getAipName");

  string tmpName;

  try {
    tmpName = pConfigProvider->getOpt_string(UAIP_SECTION_NAME, 
					     AIP_ENTITY_NAME);

    if ((0 >= tmpName.length()) || 
	(FRENT_NAME_SIZE <= tmpName.length())) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_RANGE_ERR,
		  func, "Bad Aip name length.");
    }
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }

  return tmpName;
}



FrEntIdExp cRouteCreator::getAipEntity()
{
  DECL_FUNC("cRouteCreator::getAipEntity");
  FrEntIdExp aipEntity;

  try {
    string aipName = getAipName();

    strncpy(aipEntity.name, aipName.c_str(), FRENT_NAME_SIZE);

    // Safety NULL terminate
    aipEntity.name[FRENT_NAME_SIZE - 1] = '\0';

    aipEntity.type = FRENT_TYPE_AIP;
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }

  return aipEntity;
} 


   
string cRouteCreator::getAuthNetAddrMaskV1()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           RC_AUTH_ROUTE_NET_ADDRESS_MASK_V1);
}

string cRouteCreator::getUnauthNetAddrMaskV1()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           RC_UNAUTH_ROUTE_NET_ADDRESS_MASK_V1);
}

string cRouteCreator::getAuthNetAddrMaskV2()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           RC_AUTH_ROUTE_NET_ADDRESS_MASK_V2);
}

string cRouteCreator::getUnauthNetAddrMaskV2()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           RC_UNAUTH_ROUTE_NET_ADDRESS_MASK_V2);
}

int32_t cRouteCreator::getMaxRoutes()
{ 
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        RC_MAX_ROUTES_ALLOWED);
}

int32_t cRouteCreator::getMaxRouteExpiryTime()
{
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        RC_MAX_ROUTE_EXPIRY_TIME);
}

int32_t cRouteCreator::getRouteAckTime()
{ 
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        RC_ROUTE_ACK_TIME);
}

bool cRouteCreator::validateConfig()
{
  DECL_FUNC("cRouteCreator::validateConfig");
  const char* errmsg = "Invalid config parameter %s";


  bool rc = false;

  try {
     pConfigProvider = (cConfigProvider *)
       pComponentManager->get(ZKS_CONFIGPROVIDER_MOD);

     ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pConfigProvider);

     string aipName = getAipName();

     string authNetAddrMaskV2 = getAuthNetAddrMaskV2(); 
     if (authNetAddrMaskV2.length() > 0) {
       authFakeAddrBinV2->parse(authNetAddrMaskV2.c_str());
     }

     string unauthNetAddrMaskV2 = getUnauthNetAddrMaskV2(); 
     if (unauthNetAddrMaskV2.length() > 0) {
       unauthFakeAddrBinV2->parse(unauthNetAddrMaskV2.c_str());
     }

     string authNetAddrMaskV1 = getAuthNetAddrMaskV1(); 
     if (authNetAddrMaskV1.length() > 0) {
       authFakeAddrBinV1->parse(authNetAddrMaskV1.c_str());
     }

     string unauthNetAddrMaskV1 = getUnauthNetAddrMaskV1(); 
     if (unauthNetAddrMaskV1.length() > 0) {
       unauthFakeAddrBinV1->parse(unauthNetAddrMaskV1.c_str());
     }

     int32_t maxRoutes = getMaxRoutes(); 
     if (maxRoutes <= 0) {
       CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_INVALIDCONFIG_ERR,
                    func, errmsg, RC_MAX_ROUTES_ALLOWED);
     } 

     int32_t maxRouteExpiryTime = getMaxRouteExpiryTime();
     if (maxRouteExpiryTime <= 0 || maxRouteExpiryTime > 18000) {
       CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_INVALIDCONFIG_ERR,
                    func, errmsg, RC_MAX_ROUTE_EXPIRY_TIME);
     } 

     int32_t routeAckTime = getRouteAckTime(); 
     if (routeAckTime <= 0 || routeAckTime > 18000) {
       CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_INVALIDCONFIG_ERR,
                    func, errmsg, RC_ROUTE_ACK_TIME);
     } 

     // OK if we reached here unscathed.
     rc = true;
  }
  catch (CZKS::Exception& e) {
     CZKS::Push(e, func, "failure");
     CZKS::LogException(e);
  }
  catch (...) {
    ::zksLogMesg(ZKS_ROUTECREATOR_MOD, "%s:  Unknown error.", func);
  }

  return rc; 
}


bool cRouteCreator::initializePrivateData(czkCoreLoop* coreloopin)
{
  DECL_FUNC("cRouteCreator::initializePrivateData");

  try {
    if (!privateDataInitialized) {

      if (routeTableLock.open() != ZKPR_SUCCESS) {
	CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_OPENLOCK_ERR,
		    func, "opening routeTableLock");
      }

      coreloop = coreloopin; 
      if (coreloop == NULL) {
	coreloop = new czkCoreLoop;
      }

      //
      // Assume that components will always initialize (or die trying)
      // in release.
      //
      pPktDispatcher = (cPktDispatcher *)
	pComponentManager->get(ZKS_PACKETDISPATCHER_MOD);
      ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pPktDispatcher); 
    
      pKeyProvider = (cKeyProvider *)
	pComponentManager->get(ZKS_KEYPROVIDER_MOD);
      ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pKeyProvider);

      pAipKernel = (cAipKernel *)
	pComponentManager->get(ZKS_AIPKERNEL_MOD);
      ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pAipKernel);

      pPrivateKeys = (cPrivateKeys *)
	pComponentManager->get(ZKS_PRIVATEKEYS_MOD);
      ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pPrivateKeys);

      pLinkManager = (cLinkManager *)
	pComponentManager->get(ZKS_LINKMANAGER_MOD);
      ZKS_ASSERT(ZKS_ROUTECREATOR_MOD, pLinkManager);

      //create timeout handler
      routeAckEH    = new cRouteCreatorTimeoutHandler(this);
      routeExpiryEH = new cRouteCreatorTimeoutHandler(this);

      nRoute = 0;

      authFakeAddrBinV2->init();
      unauthFakeAddrBinV2->init();

      authFakeAddrBinV1->init();
      unauthFakeAddrBinV1->init();

      privateDataInitialized = true;
    }
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }

  return true; 
}


void cRouteCreator::cleanupPrivateData()
{

    if (privateDataInitialized) {
        delete routeAckEH;
        delete routeExpiryEH;

        authFakeAddrBinV2->cleanup();
        unauthFakeAddrBinV2->cleanup();
        authFakeAddrBinV1->cleanup();
        unauthFakeAddrBinV1->cleanup();

        routeTableLock.close();

        privateDataInitialized = false;
    }
}


void cRouteCreator::startup()
{
    // We are interested in:
    //   - route create packets
    //   - route create ACK packets
    //   - route destroy packets
    pPktDispatcher->registerHandler(FRPKTTYPE_AIP_ROUTE_CREATE, this );
    pPktDispatcher->registerHandler(FRPKTTYPE_AIP_ROUTE_CRTACK, this );
    pPktDispatcher->registerHandler(FRPKTTYPE_AIP_ROUTE_DESTROY, this );            
    signalReady();
}


void cRouteCreator::shutdown()
{
    if (ready)
    {
        ready = false;
    }
}


void cRouteCreator::sendRouteDestroy( 
  cLinkManagerInfo  *linkInfo,
  uint8_t            pktVersion, 
  FrAci              aci,
  uint32_t           errcode, 
  FrEntIdExp         entity,
  FrCryptSym         symKey) 
{
  DECL_FUNC("cRouteCreator::sendRouteDestroy");


  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		   " sending route destroy to LinkId %d Aci %d errcode %d",
		   linkInfo->getLinkId(), aci, errcode); 

    cRouteDestroyPkt pkt;
    pkt.create(pktVersion, FRPKT_PRIORITY_DEFAULT, aci, errcode, entity);
    pkt.encrypt(symKey);

    uint8_t *buf;
    size_t   buflen;
    pkt.getBuffer(&buf, &buflen);
    pPktDispatcher->send(linkInfo, buf, buflen);
    pkt.destroy();
  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException( e);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed to send Route Destroy");
  }
}
 


void cRouteCreator::passRouteDestroy( 
  cLinkManagerInfo   *linkInfo,
  cRouteDestroyPkt   *pkt, 
  FrAci               nextHopAci, 
  FrCryptSym          symKey)
{
  DECL_FUNC("cRouteCreator::passRouteDestroy");

  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    if (NULL == pkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input packet reference.");
    }

    ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                   " pass route destroy to LinkId %d Aci %d",
                   linkInfo->getLinkId(), nextHopAci); 

    pkt->formatNextHop(nextHopAci);
    pkt->encrypt(symKey);

    uint8_t *buf;
    size_t   buflen;
    pkt->getBuffer( &buf, &buflen);
    pPktDispatcher->send(linkInfo, buf, buflen);
  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException( e);
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed to pass Route Destroy");
  }
}
 


void cRouteCreator::sendRouteCreateAck( 
  cLinkManagerInfo  *linkInfo,
  uint8_t            pktVersion, 
  FrAci              exitAci, 
  FrAci              nextHopAci,
  FrCryptMacKey     &macKey, 
  FrCryptSym         symKey)
{
  DECL_FUNC("cRouteCreator::sendRouteCreateAck");

  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                   " sending route create ack to LinkId %d Aci %d exitaci %d",
                   linkInfo->getLinkId(), nextHopAci, exitAci); 

    cRouteCreateAckPkt pkt;
    pkt.create(pktVersion, FRPKT_PRIORITY_DEFAULT, exitAci, nextHopAci,
	       macKey);
    pkt.encrypt(symKey);
          
    uint8_t *buf;
    size_t   buflen;
    pkt.getBuffer( &buf, &buflen);
    pPktDispatcher->send(linkInfo, buf, buflen);
    pkt.destroy();
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed to send RouteCreateAck");
  }
}
 


void cRouteCreator::passRouteCreateAck( 
  cLinkManagerInfo    *linkInfo,
  cRouteCreateAckPkt  *pkt, 
  FrAci                nextHopAci, 
  FrCryptSym           symKey)
{
  DECL_FUNC("cRouteCreator::passRouteCreateAck");

 
  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    if (NULL == pkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input packet reference.");
    }

    ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                   " passing route create ack to LinkId %d Aci %d",
                   linkInfo->getLinkId(), nextHopAci);

    pkt->formatBackwardHop(nextHopAci);
    pkt->encrypt(symKey);
      
    uint8_t *buf;
    size_t   buflen;
    pkt->getBuffer( &buf, &buflen);
    pPktDispatcher->send(linkInfo, buf, buflen);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed to pass RouteCreateAck");
  }
}


 
void cRouteCreator::sendRouteCreate( 
  cLinkManagerInfo  *linkInfo,
  cRouteCreatePkt   *pkt, 
  FrAci              nextHopAci)
{
  DECL_FUNC("cRouteCreator::sendRouteCreate");

  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    if (NULL == pkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input packet reference.");
    }

    ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		   " sending route create to LinkId %d Aci %d",
		   linkInfo->getLinkId(), nextHopAci); 

    pkt->formatForwardHop( nextHopAci);
    
    uint8_t *buf;
    size_t   buflen;
    pkt->getBuffer( &buf, &buflen);
    pPktDispatcher->send(linkInfo, buf, buflen);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed to send Route Create");
  }
}
 


void cRouteCreator::reportBrokenLink( LinkId linkId )
{
    const char* func = "cRouteCreator::reportBrokenLink";

    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                   " report broken LinkId %x",
                   linkId); 

    try {

       INITLOCK(tablelock, &routeTableLock);
       LOCK(tablelock);

       cLinkManagerInfo* linkInfo = pLinkManager->getLink( linkId);
       if (!linkInfo) {
          CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NOLINKINFO_ERR,
                       func, "bad link id");
       }
 
       // get routeTableLinkId entry with this LinkId
       RouteTableLinkIdDef::iterator i = routeTableLinkId.find(linkId);
    
       if( i != routeTableLinkId.end() )
       {

          FrEntIdExp entity =  linkInfo->getEntity();

          cRouteCreatorAciMap* pAciMap = i->second;
     
          pAciMap->iterateBegin();

          cRouteCreatorInfo* pRouteInfo;

          //for every route on this link
          while ((pRouteInfo = pAciMap->iterate())) 
          {
  	      pRouteInfo->cancelRouteAckTimer();
 	      pRouteInfo->cancelRouteExpiryTimer();
              try {
	         handleRouteExpiryTimeout(pRouteInfo);
	      }
              catch (CZKS::Exception &e) {
		 CZKS::Push(e, func, "failure");
		 CZKS::LogException(e);
	      }
	  }

          // delete the pAciMap object
          delete pAciMap;

          // erase the link id
          routeTableLinkId.erase(i);
       }

       UNLOCK(tablelock);

    }
    catch (...) {
       CZKS::Rethrow( func, "Failed");
    }
}


cRouteCreatorInfo* cRouteCreator::newRouteInfo()
{
   const char* func = " cRouteCreator::newRouteInfo";

   cRouteCreatorInfo *pNewRouteInfo = NULL;

   try {
      pNewRouteInfo = new cRouteCreatorInfo();
      pNewRouteInfo->init( routeIdBin);
   }
   catch (...) {
      CZKS::Rethrow( func, "Failed to create new Route Info object");
   }

   return pNewRouteInfo;
}



void cRouteCreator::insertRouteTableEntry( 
  LinkId                linkId, 
  FrAci                 aci, 
  cRouteCreatorAciMap  *pAciMap,
  cRouteCreatorInfo    *pNewRouteInfo)
{
  DECL_FUNC("cRouteCreator::insertRouteTableEntry");

  try {
    if (NULL == pNewRouteInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorInfo reference.");
    }

    ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		   " insert route table entry LinkId %d Aci %d",
		   linkId, aci); 

    if (pAciMap == NULL) {
      pAciMap = new cRouteCreatorAciMap;
      routeTableLinkId[linkId] = pAciMap;
    }

    pAciMap->insert(aci, pNewRouteInfo);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}



bool cRouteCreator::validateWormAci(FrAci aci)
{

  WormAciTableDef::iterator i = wormAciTable.find(aci);
  return (i != wormAciTable.end());

}

void cRouteCreator::returnWormAci( FrAci aci)
{
   const char* func = "cRouteCreator::returnWormAci";

   if (!validateWormAci( aci)) {
      CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
                   ZKS_ROUTECREATOR_UNKNOWNWORMACI_ERR,
                   func, 
                   "Unable to return Worm Aci");
   }

   // decrement the count (no. of routes using this aci)
   wormAciTable[aci].count--;

   //if the count is zero, return the aci
   if (wormAciTable[aci].count == 0) {

      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, " return worm aci %d ", aci);

      wormAciBin.returnAci( aci);

      // return the fake address if it was set
      if (wormAciTable[aci].fakeAddrSet) {

         if (wormAciTable[aci].authRoute) {
            if (wormAciTable[aci].pktVersion == FRPKT_DATA_VERSION_1) {
	       authFakeAddrBinV1->returnFakeNetAddr( wormAciTable[aci].fakeAddr);
	    }
	    else {
	       authFakeAddrBinV2->returnFakeNetAddr( wormAciTable[aci].fakeAddr);
	    }
         }
         else {
            if (wormAciTable[aci].pktVersion == FRPKT_DATA_VERSION_1) {
  	       unauthFakeAddrBinV1->returnFakeNetAddr( wormAciTable[aci].fakeAddr);
	    }
	    else {
  	       unauthFakeAddrBinV2->returnFakeNetAddr( wormAciTable[aci].fakeAddr);
	    }
         }

      }

   }
}



void cRouteCreator::deleteRouteTableEntry(
  cRouteCreatorInfo     *pRouteInfo,
  cRouteCreatorAciMap   *pAciMap,
  FrAci                  aci, 
  FrAci                  nextAci,
  LinkId                 nextLinkId)
{
  DECL_FUNC("cRouteCreator::deleteRouteTableEntry");


  try {
    if (NULL == pRouteInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorInfo reference.");
    }

    ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                   " delete route table entry Aci %d nextAci %d routetype %d ",
                   aci, nextAci, pRouteInfo->routeType); 

    if (pRouteInfo->routeType == wormholeRoute || 
	pRouteInfo->routeType == extendedRoute) {

      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		     " returning wormAci %d",
		     nextAci); 
      returnWormAci(nextAci);
    }
    else {
      if (pRouteInfo->routeType == fwdRoute) {
	// return aci to pool
	cLinkManagerInfo* nextLinkInfo = pLinkManager->getLink( nextLinkId);
	if (!nextLinkInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		       func, 
		       "Link does not exist to return ACI");
	}

	ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		       " return aci to pool for linkId %d Aci %d",
		       nextLinkId, nextAci); 

	nextLinkInfo->returnAci( nextAci);
      }
            
      // look for cRouteCreatorInfo in other dir
      cRouteCreatorAciMap* pOtherAciMap = findAciMap( nextLinkId);
      if (pOtherAciMap) 
	{
	  cRouteCreatorInfo* pOtherRouteInfo = pOtherAciMap->find( nextAci);
	  if (pOtherRouteInfo) 
            {

	      if (pOtherRouteInfo->routeType == fwdRoute) 
		{

  	          LinkId otherLinkId = pOtherRouteInfo->getFwdLinkId();
                  cLinkManagerInfo* otherLinkInfo = 
		    pLinkManager->getLink( otherLinkId);
                  if (!otherLinkInfo) {
		    CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
				 ZKS_ROUTECREATOR_NOLINKINFO_ERR,
				 func, 
				 "Link does not exist to return ACI");
                  }

                  ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
				 " return aci to pool for linkId %d Aci %d",
				 otherLinkId, aci); 

                  otherLinkInfo->returnAci( aci);
		}

	      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
			     " delete acimap entry Aci %d",
			     nextAci);
 
	      pOtherAciMap->erase(nextAci);

	      pOtherRouteInfo->cleanup(routeIdBin);
	      delete pOtherRouteInfo;
	    }
	}
    }

    // remove cRouteCreatorInfo from routeTableAci
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		   " delete acimap entry Aci %d",
		   aci);

    if (NULL == pAciMap) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorAciMap reference.");
    }

    pAciMap->erase(aci); 
            
    // destroy cRouteCreatorInfo
    pRouteInfo->cleanup(routeIdBin);
    delete pRouteInfo;

  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}


 
void cRouteCreator::requestNymKey(
  FrEntIdExp         entity, 
  LinkId             linkId, 
  FrAci              aci, 
  cRouteCreatePkt   *rcPkt)
{
  DECL_FUNC("cRouteCreator::requestNymKey");

  cRouteCreatorKeyHandlerData* ehd = NULL;
  try {
    //create the KeyHandlerData object
    ehd = new cRouteCreatorKeyHandlerData(linkId, aci, rcPkt);

    //set up the KeyHandler 
    cRouteCreatorKeyHandler* eh = new cRouteCreatorKeyHandler( this);
    eh->setData(ehd);

    //make the key request     
    cKeyProviderCacheEntryData entryData; //not used for anything
    pKeyProvider->makeKeyRequest(entity.name, entity.type,  
				 FRCRYPT_KEYTYPE_SIGNING, 0,  
				 eh, entryData);
  }
  catch (...) {
    delete ehd;
    CZKS::Rethrow(func, "Failed");
  }
}



cRouteCreatorAciMap* cRouteCreator::findAciMap( LinkId linkId)
{
    const char* func = "cRouteCreator::findAciMap";

    cRouteCreatorAciMap* pAciMap;
    try {
       pAciMap = NULL;

       RouteTableLinkIdDef::iterator i = routeTableLinkId.find(linkId);
       if (i != routeTableLinkId.end()) {

          pAciMap = i->second;
          ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                          " found Aci map for linkID %d",
                          linkId); 

       }
       else {
          ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
                          " did NOT find Aci map for linkID %d",
                          linkId); 
       }
    }
    catch (...) {
       CZKS::Rethrow( func, "Failed");
    }

    return pAciMap;

}



void cRouteCreator::handleRouteCreatePkt( 
  cLinkManagerInfo     *linkInfo,
  cRouteCreatePkt      *rcPkt, 
  ZKPRNetAddr           addrin,
  bool                  routeExists,
  LinkId                linkId, 
  FrAci                 aci,
  cRouteCreatorAciMap  *pAciMap)
{
  DECL_FUNC("cRouteCreator::handleRouteCreatePkt");
  
  CZKS::Exception e;

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 

  cRouteCreatorInfo *pNewRouteInfo = NULL;
  cRouteCreatorInfo *pOthRouteInfo = NULL;

  cRouteCreatePkt* rcPktSave = rcPkt;

  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    if (NULL == rcPkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatePkt reference.");
    }

    // pAciMap may legitimately be NULL


    uint8_t* pkt;
    size_t   pktSize;
    uint8_t  pktVersion;

    rcPkt->getBuffer(&pkt, &pktSize); 

    uint32_t version;
    if (frPktGetVersion(e.zkError(), pkt, pktSize, &version) == 
	ZKS_FAILURE) {
      CZKS::Throw( e);
    }
    pktVersion = version;

    uint32_t privVers = rcPkt->getCryptKeyVersion();
 
    //get AIP's private crypt key
    FrCryptPubKey cryptKey;
    pPrivateKeys->getCryptKey( privVers, &cryptKey);

    // decrypt packet
    bool lastHop = rcPkt->decrypt( cryptKey);

    FrEntIdExp nextHost = rcPkt->getNextEntity();

    if(!routeExists)
      {
	if (refuseConnections) {
	  sendRouteDestroy( linkInfo, 
			    pktVersion,
			    aci,
			    ZKS_ROUTEDESTROY_REFUSECONN_ERR, 
			    getAipEntity(), rcPkt->getFwdCrypt());
	}
	else if (nRoute >= (uint32_t)getMaxRoutes()) {
	  sendRouteDestroy( linkInfo, 
			    pktVersion,
			    aci,
			    ZKS_ROUTEDESTROY_MAXROUTES_ERR, 
			    getAipEntity(), rcPkt->getFwdCrypt());
	}
	else if (lastHop)
	  {
	    cFakeNetAddress* fakeAddrBin;

	    //authenticated route
	    if(nextHost.type == FRENT_TYPE_AIP)
	      {
		if (pktVersion == FRPKT_DATA_VERSION_1) {
		  fakeAddrBin = authFakeAddrBinV1;
		}
		else {
		  fakeAddrBin = authFakeAddrBinV2;
		} 
	      }
	    else {
	      if (pktVersion == FRPKT_DATA_VERSION_1) {
		fakeAddrBin = unauthFakeAddrBinV1;
	      }
	      else {
		fakeAddrBin = unauthFakeAddrBinV2;
	      } 
	    }

	    if (!fakeAddrBin->isValidated()) {
	      sendRouteDestroy( linkInfo, 
				pktVersion,
				aci,
				ZKS_ROUTEDESTROY_FAKEADDRNOTCONF_ERR,
				getAipEntity(), rcPkt->getFwdCrypt());

	      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			  func, "Fake Address not configured");
	    }  

	    // creating route

	    pNewRouteInfo = newRouteInfo();
	    pNewRouteInfo->setRcPkt(rcPkt);
	    rcPktSave = NULL;
	    pNewRouteInfo->setBwd( linkId, aci, addrin);
	    FrAci wormaci;
	    FrEntIdExp nymEntity = rcPkt->getNymEntity();

	    if (rcPkt->getFlags() & FRPKT_RTECRT_FL_SPEC_ACI) {

	      //route extension

	      // don't allow extension of old type of unauth route
	      if (nextHost.type == FRENT_TYPE_ANON) {
		sendRouteDestroy( linkInfo, 
				  pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_ROUTEEXTENDAUTH_ERR, 
				  getAipEntity(), 
				  rcPkt->getFwdCrypt());

		CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			    func, "Not allowed to extend unauthenticated route."); 
	      }

	      // get the specAci from the pkt & validate
	      wormaci = rcPkt->getSpecAci();
	      if (!validateWormAci( wormaci)) {
		sendRouteDestroy( linkInfo, 
				  pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_ROUTEEXTENDACI_ERR, 
				  getAipEntity(), 
				  rcPkt->getFwdCrypt());
		
		CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			    func, "Invalid specAci");
	      }
	      if (wormAciTable[wormaci].count == 0) {
		sendRouteDestroy( linkInfo, 
				  pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_ROUTEEXTENDACI_ERR, 
				  getAipEntity(), 
				  rcPkt->getFwdCrypt());

		CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			    func, "specAci does not exist.");
	      }

	      if(nextHost.type == FRENT_TYPE_AIP) {

		//auth route - check that nym is same as original
		if (memcmp(&wormAciTable[wormaci].nymEntity,
			   &nymEntity, sizeof( nymEntity)) != 0) {
		  sendRouteDestroy( linkInfo, 
				    pktVersion,
				    aci,
				    ZKS_ROUTEDESTROY_ROUTEEXTENDNYM_ERR, 
				    getAipEntity(), 
				    rcPkt->getFwdCrypt());

		  CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			      func, "Nym entity does not match.");
		}
	      }                            
	      else {

		// unauth route, check that last hop entity is same
		if (memcmp(&wormAciTable[wormaci].nymEntity,
			   &nextHost, sizeof( nextHost)) != 0) {
		  sendRouteDestroy( linkInfo, 
				    pktVersion,
				    aci,
				    ZKS_ROUTEDESTROY_ROUTEEXTENDUNAUTH_ERR, 
				    getAipEntity(),
				    rcPkt->getFwdCrypt());

		  CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR,
			      func, " nextHost entity does not match.");
		}
	      }                            

	      // increment the count of routes using this wormAci
	      // and retrieve fake address that was used
	      wormAciTable[wormaci].count++;
	      pNewRouteInfo->fakeAddr = wormAciTable[wormaci].fakeAddr;
 
	    }
	    else { 

	      // new route

	      try {
		wormaci = wormAciBin.getAci();
	      }
	      catch (CZKS::Exception& ex) {
		sendRouteDestroy( linkInfo, 
				  pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_OUTOFACI_ERR, 
				  getAipEntity(), 
				  rcPkt->getFwdCrypt());
		ex.Rethrow( func, 
			    "could not allocate wormhole ACI");
	      }
	      try {
		if (nextHost.type == FRENT_TYPE_AIP) {
		  wormAciTable[wormaci] = 
		    cRouteCreatorWormAci(1, nymEntity); 
		}
		else {
		  wormAciTable[wormaci] = 
		    cRouteCreatorWormAci(1, nextHost);
		}
		pNewRouteInfo->fakeAddr = 
		  fakeAddrBin->getFakeNetAddr();
		wormAciTable[wormaci].pOldRouteInfo = pNewRouteInfo;
	      }
	      catch (CZKS::Exception& ex) {
		sendRouteDestroy(linkInfo, 
				 pktVersion,
				 aci,
				 ZKS_ROUTEDESTROY_OUTOFFAKEADDR_ERR, 
				 getAipEntity(),
				 rcPkt->getFwdCrypt());
		returnWormAci( wormaci);
		ex.Rethrow( func, 
			    "Could not create fake address");
	      }

	      //set the fake address in the table
	      wormAciTable[wormaci].setFakeAddr( 
						pNewRouteInfo->fakeAddr, 
						(nextHost.type == FRENT_TYPE_AIP), pktVersion);
	    }

	    //almost done, set the remaining fields
	    pNewRouteInfo->setFwd( 0, wormaci, pNewRouteInfo->fakeAddr);
	    pNewRouteInfo->routeType = wormholeRoute;                  
	    pNewRouteInfo->pktVersion = pktVersion;                  
	    pNewRouteInfo->flags = rcPkt->getFlags();

	    //validate the expiry time
	    try {
	      pNewRouteInfo->setExpTime(rcPkt->getExpTime(), 
					getRouteAckTime());
	    }
	    catch (CZKS::Exception& ex)
	      {
		sendRouteDestroy( linkInfo, 
				  pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_BADEXPTIME_ERR, 
				  getAipEntity(), rcPkt->getFwdCrypt());
		returnWormAci( wormaci);
		ex.Rethrow(func, "Route destroyed");
	      }

	    // insert the routeInfo entry
	    insertRouteTableEntry( linkId, aci, pAciMap, 
				   pNewRouteInfo);
	    cRouteCreatorInfo* pSaveRouteInfo = pNewRouteInfo;
            pNewRouteInfo = NULL;

	    if (nextHost.type == FRENT_TYPE_AIP) {

	      // authenticated route - set state to pending
	      // and request key to validate nym signature
	      // final processing of route done in handleKeyChangedEvent
	      pSaveRouteInfo->routeState = lastHopPendingKey;

	      requestNymKey(nymEntity, linkId, aci, rcPkt);

	    }
	    else {

	      // unauth route --- ready to go

	      pSaveRouteInfo->scheduleRouteExpiryTimer(coreloop, 
						      this, 
						      routeExpiryEH);

	      if (rcPkt->getFlags() & FRPKT_RTECRT_FL_SPEC_ACI) {

		cRouteCreatorInfo* pOldRouteInfo = 
		  wormAciTable[wormaci].pOldRouteInfo;

		pAipKernel->updateRoute( pSaveRouteInfo->getRouteId(),
					 pSaveRouteInfo,
					 pOldRouteInfo);

		// kill the old route
		pOldRouteInfo->routeType = extendedRoute;
		handleRouteExpiryTimeout(pOldRouteInfo);

		wormAciTable[wormaci].pOldRouteInfo = pSaveRouteInfo;


	      }
	      else {

		pAipKernel->addRoute( pSaveRouteInfo->getRouteId(),
				      pSaveRouteInfo);

	      }

	      nRoute++;

	      sendRouteCreateAck( linkInfo,
				  pSaveRouteInfo->getPktVersion(), 
				  pSaveRouteInfo->getFwdAci(), 
				  aci,
				  rcPkt->getMacKey(), 
				  rcPkt->getFwdCrypt());

	      pSaveRouteInfo->routeState = lastHopRouteUp;
    
                    
	    }
                
	  }
	else // Intermediate hop
	  {
	    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, 
			   "looking for link for entity %s ",
			   nextHost.name);

	    cLinkManagerInfo* nextLinkInfo = 
	      pLinkManager->getLink( nextHost);

	    // check if the next AIP is up
	    if(nextLinkInfo && nextLinkInfo->getLinkState() == linkUp)
	      {
		// get the linkId of the next hop and generate a new ACI
		LinkId nextLinkId = nextLinkInfo->getLinkId();

		// get the ACI for the next hop
		FrAci nextAci;
		try {
		  nextAci = nextLinkInfo->getAci();
		}
		catch (CZKS::Exception& ex) {
		  sendRouteDestroy( linkInfo, 
				    pktVersion,
				    aci,
				    ZKS_ROUTEDESTROY_OUTOFACI_ERR, 
				    getAipEntity(), rcPkt->getFwdCrypt());
		  ex.Rethrow(func, "Route destroyed");
		}

		ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, 
			       "next link id %d aci %d ",
			       nextLinkId, nextAci);
		ZKPRNetAddr nextAddr = nextLinkInfo->getAddr();

		// create a routeInfo entry for the fwd direction
		pNewRouteInfo = newRouteInfo();
		pNewRouteInfo->setRcPkt(rcPkt);
		rcPktSave = NULL;
		pNewRouteInfo->setBwd( linkId, aci, addrin);
		pNewRouteInfo->setFwd( nextLinkId, nextAci, nextAddr);
		pNewRouteInfo->routeType = fwdRoute;
		pNewRouteInfo->pktVersion = pktVersion;                  
		pNewRouteInfo->flags = rcPkt->getFlags();

		// validate the expiry time
		try {
		  pNewRouteInfo->setExpTime(rcPkt->getExpTime(), 
					    getRouteAckTime());
		}
		catch (CZKS::Exception& ex)
		  {
		    sendRouteDestroy( linkInfo, 
				      pktVersion,
				      aci,
				      ZKS_ROUTEDESTROY_BADEXPTIME_ERR, 
				      getAipEntity(), rcPkt->getFwdCrypt());
		    nextLinkInfo->returnAci( nextAci);
		    ex.Rethrow(func, "Route destroyed");
		  }
                   
		// generate a routeInfo record in the bwd direction
		pOthRouteInfo = newRouteInfo();
		pOthRouteInfo->setRcPkt(rcPkt);
		pOthRouteInfo->setFwd( linkId, aci, addrin);
		pOthRouteInfo->setBwd( nextLinkId, nextAci, nextAddr);
		pOthRouteInfo->routeType = bwdRoute;
		pOthRouteInfo->pktVersion = pktVersion;                  
		pOthRouteInfo->flags = rcPkt->getFlags();
		pOthRouteInfo->setExpTime(rcPkt->getExpTime(), 
					  getRouteAckTime());
              
		// forward the RouteCreate pkt to the next hop
		sendRouteCreate( nextLinkInfo, rcPkt, nextAci);
                  
		pOthRouteInfo->scheduleRouteAckTimer(coreloop, this, 
						     routeAckEH);
                                   
		// this means we're waiting for a RouteCrtAck pkt
		pNewRouteInfo->routeState = intHopPendingAck;
		pOthRouteInfo->routeState = intHopPendingAck;
 
		//insert routeInfo records into route table

		insertRouteTableEntry( linkId, aci, pAciMap, pNewRouteInfo);
                pNewRouteInfo = NULL;

		cRouteCreatorAciMap* pNextAciMap = findAciMap( nextLinkId);
		insertRouteTableEntry( nextLinkId, nextAci, 
				       pNextAciMap, pOthRouteInfo);
		pOthRouteInfo = NULL;

	      }
	    else // linkDown
	      {
                sendRouteDestroy( linkInfo, pktVersion,
				  aci,
				  ZKS_ROUTEDESTROY_LINKDOWN_ERR, 
				  nextHost, rcPkt->getFwdCrypt());
                if (rcPktSave) { 
	           rcPktSave->detachPkt();
            	   delete rcPktSave;
                }
 
              }
	  }
      } // endif(!routeExists)
        
    else //if routeExists
      {
	CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		     ZKS_ROUTECREATOR_ROUTEALREADYEXISTS_ERR,
		     func, 
		     "recvd RouteCreate Pkt for an existing route");
      }        
  }
  catch (...) {
    try {
      if (pNewRouteInfo) {
	pNewRouteInfo->cleanup(routeIdBin);
	delete pNewRouteInfo;
      }
      if (pOthRouteInfo) {
	pOthRouteInfo->cleanup(routeIdBin);
	delete pOthRouteInfo;
      }
      if (rcPktSave) {
	rcPktSave->detachPkt();
	delete rcPktSave;
      }
    }
    catch (...) {
      // "Ignore" possible cleanup errors
    }


    //
    //  Filter out "routine" failed route creates
    //
    try {
      CZKS::Rethrow(func, "Failed");
    }
    catch (CZKS::Exception &e) {
      if (e.isErrCode(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_CREATE_ABORT_ERR)) {
	ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD,
		       "%s:  Error %hd:  \"%s\".", func, 
		       ZKS_ERRCODE_ERR(e.Code()), e.Message());
      }
      else {
	// Propagate a serious error (context already set)
	CZKS::Rethrow(func, "Failed.");
      }
    }
  }
}



void cRouteCreator::handleRouteCreateAckPkt( 
  cRouteCreateAckPkt  *rcaPkt, 
  bool                 routeExists,
  cRouteCreatorInfo   *pRouteInfo,
  RouteState           curRouteState)
{
  DECL_FUNC("cRouteCreator::handleRouteCreateAckPkt");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 

  try {
    if (NULL == rcaPkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input createAckPkt reference.");
    }

    if (NULL == pRouteInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorInfo reference.");
    }


    if( !routeExists || 
	curRouteState == lastHopPendingKey || 
	curRouteState == lastHopRouteUp )
      {
	// nothing.
      }
    else if( curRouteState == intHopPendingAck ||
	     curRouteState == intHopRouteUp )
      {
	if( curRouteState == intHopPendingAck)
	  {
	    // cancel ACK timeout
	    pRouteInfo->cancelRouteAckTimer();   

	    // add route in kernel
	    pRouteInfo->routeState = intHopRouteUp;

	    cRouteCreatorAciMap* pAciMap = 
	      findAciMap( pRouteInfo->getFwdLinkId());
	    if (!pAciMap) {
	      CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
			   ZKS_ROUTECREATOR_NOACIMAP_ERR,
			   func, 
                           "AciMap entry does not exist for RouteCrtAck Pkt");
	    }

	    cRouteCreatorInfo* pOthRouteInfo = 
	      pAciMap->find(pRouteInfo->getFwdAci()); 
	    if (!pOthRouteInfo) {
	      CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
			   ZKS_ROUTECREATOR_NOROUTEINFO_ERR,
			   func, 
                           "Route entry does not exist for RouteCrtAck Pkt");
	    }

	    pAipKernel->addRoute( pOthRouteInfo->getRouteId(),
				  pOthRouteInfo);
	    nRoute++;
	    pOthRouteInfo->routeState = intHopRouteUp;
	    // cancel ACK timeout
	    pOthRouteInfo->cancelRouteAckTimer();   

	  }

	// modify ACK and forward
	LinkId nextLinkId = pRouteInfo->getFwdLinkId();
	cLinkManagerInfo* nextLinkInfo = 
	  pLinkManager->getLink( nextLinkId);
	if (!nextLinkInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		       func, 
		       "Link does not exist to forward RouteCrtAck Pkt");
	}
	FrAci nextAci = pRouteInfo->getFwdAci();
	passRouteCreateAck( nextLinkInfo,
			    rcaPkt, 
			    nextAci, pRouteInfo->getFwdCrypt());

	// schedule expiry timeout
	pRouteInfo->scheduleRouteExpiryTimer(coreloop, this, 
					     routeExpiryEH);

      }
    rcaPkt->detachPkt();
    delete rcaPkt;
  }
  catch (...) {
    rcaPkt->detachPkt();
    delete rcaPkt;
    CZKS::Rethrow( func, "Failed");
  }
}



void cRouteCreator::handleRouteDestroyPkt( 
  cRouteDestroyPkt      *rdPkt, 
  bool                   routeExists,
  FrAci                  aci,
  cRouteCreatorAciMap   *pAciMap,
  cRouteCreatorInfo     *pRouteInfo,
  RouteState             curRouteState)
{
  DECL_FUNC("cRouteCreator::handleRouteDestroyPkt");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 


  try {
    if (NULL == rdPkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeDestroyPkt reference.");
    }


    if( !routeExists )
    {
	// nothing.
    }
    else if (NULL == pRouteInfo) 
    {
        CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		    func, "Null input routeCreatorInfo reference.");
    }
    else if( curRouteState == intHopPendingAck ||
	     curRouteState == intHopRouteUp )
    {

	cRouteCreatorAciMap* pOthAciMap = 
	  findAciMap( pRouteInfo->getFwdLinkId());
	if (!pOthAciMap) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOACIMAP_ERR,
		       func, 
		       "AciMap entry does not exist for RouteDestroy Pkt");
	}

	cRouteCreatorInfo* pOthRouteInfo = 
	  pOthAciMap->find(pRouteInfo->getFwdAci()); 
	if (!pOthRouteInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOROUTEINFO_ERR,
		       func, 
		       "Route entry does not exist for RouteDestroy Pkt");
	}

	if( curRouteState == intHopRouteUp )
	{
	    // cancel expiry timeout in core loop
	    pRouteInfo->cancelRouteExpiryTimer(); 
	    pOthRouteInfo->cancelRouteExpiryTimer(); 

	    // remove route from kernel
	    try {
	      if (pRouteInfo->routeType == fwdRoute) {
		pAipKernel->removeRoute( pRouteInfo->getRouteId(), 
					 pRouteInfo);
	      }
	      else {
		pAipKernel->removeRoute( pOthRouteInfo->getRouteId(), 
					 pOthRouteInfo);
	      }
	    }
	    catch (CZKS::Exception &e)
	      {
		CZKS::Push(e, func, "failure");
		CZKS::LogException(e);
	      }
            if (nRoute > 0) {
   	        nRoute--;
	    }

	}
	else 
	{
	    // cancel ack timeout
	    pRouteInfo->cancelRouteAckTimer(); 
	    pOthRouteInfo->cancelRouteAckTimer(); 
	}

	// modify route destroy pkt  and forward
	LinkId nextLinkId = pRouteInfo->getFwdLinkId();
	cLinkManagerInfo* nextLinkInfo = 
	  pLinkManager->getLink( nextLinkId);
	if (!nextLinkInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		       func, 
		       "Link does not exist to forward RouteDestroy Pkt");
	}

	FrAci nextAci = pRouteInfo->getFwdAci();
	passRouteDestroy( nextLinkInfo,
			  rdPkt, 
			  nextAci, pRouteInfo->getFwdCrypt());

	deleteRouteTableEntry( pRouteInfo, pAciMap,
			       aci, nextAci, nextLinkId);
               
    }
    else if( curRouteState == lastHopPendingKey ||
	     curRouteState == lastHopRouteUp )
    {
	if(  curRouteState == lastHopRouteUp )
	  {
	    // cancel expiry timeout
	    pRouteInfo->cancelRouteExpiryTimer(); 
	    // remove route in kernel
	    try {
	      pAipKernel->removeRoute( pRouteInfo->getRouteId(), 
				       pRouteInfo);
	    }
	    catch (CZKS::Exception &e)
	      {
		CZKS::Push(e, func, "failure");
		CZKS::LogException(e);
	      }
            if (nRoute > 0) {
   	       nRoute--;
	    }
	  }

	// return aci to pool
	FrAci nextAci = pRouteInfo->getFwdAci();

	ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		       " return wormAci %d to bin ",
		       nextAci); 
	returnWormAci( nextAci);

	// remove cRouteCreatorInfo from routeTableAci
	ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		       " delete Aci map entry %d ",
		       aci); 

	if (NULL == pAciMap) {
	  CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		      func, "Null input routeCreatorAciMap reference.");
	}

	pAciMap->erase(aci); 
            
	// destroy cRouteCreatorInfo
        pRouteInfo->cleanup(routeIdBin);
	delete pRouteInfo;
    }


    rdPkt->detachPkt();
    delete rdPkt;
  }
  catch (...) {
    try {
      if (NULL != rdPkt) {
	rdPkt->detachPkt();
	delete rdPkt;
      }
    }
    catch (...) {
      // Trap "extra" exceptions on cleanup.
    }

    CZKS::Rethrow( func, "Failed");
  }
}



void cRouteCreator::handlePkt( 
  cLinkManagerInfo *linkInfo, 
  uint8_t          *pPkt, 
  size_t            pktSize )
{
  DECL_FUNC("cRouteCreator::handlePkt");


  CZKS::Exception e;

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, " handlePkt size %d ", pktSize);

  ZKPRNetAddr addrin; // this should really be a parameter
    
  try {
    if (NULL == linkInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input linkinfo reference.");
    }

    if (NULL == pPkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input packet reference.");
    }

    // Validate management packet
    FrPktType       type;
    if (frPktGetType(e.zkError(), pPkt, pktSize, &type) == ZKS_FAILURE) {
      CZKS::Throw(e);
    }

    cRouteCreatePkt*    rcPkt  = NULL;
    cRouteCreateAckPkt* rcaPkt = NULL;
    cRouteDestroyPkt*   rdPkt  = NULL;

    FrAci aci; 
    if (type == FRPKTTYPE_AIP_ROUTE_CREATE)
    {
	rcPkt = new cRouteCreatePkt;
	rcPkt->attachPkt( pPkt, pktSize);
	aci = rcPkt->getAci();
    }
    else if (type == FRPKTTYPE_AIP_ROUTE_CRTACK)
    {
	rcaPkt = new cRouteCreateAckPkt;
	rcaPkt->attachPkt( pPkt, pktSize);   
	aci = rcaPkt->getAci();
    }
    else if (type == FRPKTTYPE_AIP_ROUTE_DESTROY)
    {
	rdPkt = new cRouteDestroyPkt;
	rdPkt->attachPkt( pPkt, pktSize);
	aci = rdPkt->getAci();
    }
    else 
    {
      CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		   ZKS_ROUTECREATOR_INVALIDPKTTYPE_ERR,
		   func, 
		   "Packet type unknown");
    }        

    INITLOCK(tablelock, &routeTableLock);
    LOCK(tablelock);
        
    LinkId linkId = linkInfo->getLinkId();

    cRouteCreatorAciMap* pAciMap = findAciMap( linkId);
 
    bool routeExists = false;
    RouteState curRouteState;

    cRouteCreatorInfo *pRouteInfo;

    if (pAciMap) {

      pRouteInfo = pAciMap->find(aci); 
      routeExists = (pRouteInfo != NULL);
      if (routeExists) {
	curRouteState = pRouteInfo->getRouteState();
      }

      ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		     " linkid %d aci %d routeExists %d curRouteState %d ",
		     linkId, aci, routeExists, curRouteState); 

    }

    // Now handle the packet   
    if (type == FRPKTTYPE_AIP_ROUTE_CREATE)
    {

	handleRouteCreatePkt( linkInfo, rcPkt, addrin, routeExists,
			      linkId, aci, pAciMap);

    } // end route create pkt
    
    else if (type == FRPKTTYPE_AIP_ROUTE_CRTACK)
    {
 
	handleRouteCreateAckPkt( rcaPkt, routeExists,
				 pRouteInfo, curRouteState);

    } // end route create ACK
    
    else if (type == FRPKTTYPE_AIP_ROUTE_DESTROY)
    {

	handleRouteDestroyPkt( rdPkt, routeExists, aci,
			       pAciMap, pRouteInfo, curRouteState);

    } // if route destroy
    
    UNLOCK(tablelock);
  }
  catch (CZKS::Exception &ex) {
    CZKS::Push(ex, func, "failure");
    CZKS::LogException(ex);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}



void cRouteCreator::handleRouteAckTimeout(
  cRouteCreatorInfo* pRouteInfo)
{
  DECL_FUNC("cRouteCreator::handleRouteAckTimeout");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 

  try {
    if (NULL == pRouteInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorInfo reference.");
    }

    if( pRouteInfo->routeState == intHopPendingAck )
      {
	// Send a route destroy packet forward and backwards
	LinkId fwdLinkId = pRouteInfo->getFwdLinkId();
	cLinkManagerInfo* fwdLinkInfo = pLinkManager->getLink( fwdLinkId);
	if (!fwdLinkInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		       func, 
		       "FwdLink does not exist to forward RouteDestroy Pkt");
	}
	sendRouteDestroy( fwdLinkInfo, pRouteInfo->getPktVersion(),
			  pRouteInfo->getFwdAci(),
			  ZKS_ROUTEDESTROY_ACKTIMEOUT_ERR, 
			  getAipEntity(), pRouteInfo->getFwdCrypt());

	LinkId bwdLinkId = pRouteInfo->getBwdLinkId();
	cLinkManagerInfo* bwdLinkInfo = pLinkManager->getLink( bwdLinkId);
	if (!bwdLinkInfo) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		       ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		       func, 
		       "BwdLink does not exist to forward RouteDestroy Pkt");
	}
	sendRouteDestroy( bwdLinkInfo, pRouteInfo->getPktVersion(),
			  pRouteInfo->getBwdAci(),
			  ZKS_ROUTEDESTROY_ACKTIMEOUT_ERR, 
			  getAipEntity(), pRouteInfo->getBwdCrypt());

	//delete the route table entries
	cRouteCreatorAciMap* pAciMap = findAciMap( bwdLinkId);
	if (!pAciMap) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NOACIMAP_ERR,
		       func, "failed to find aci map");
	}
	deleteRouteTableEntry( pRouteInfo, pAciMap,
			       pRouteInfo->getBwdAci(), 
			       pRouteInfo->getFwdAci(), fwdLinkId);
      }
    else
      {
	CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		     ZKS_ROUTECREATOR_INVALIDROUTESTATE_ERR,
		     func, 
		     "state not intHopPendingAck");
      }
  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException(e);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}



void cRouteCreator::handleRouteExpiryTimeout(
  cRouteCreatorInfo* pRouteInfo)
{
  DECL_FUNC("cRouteCreator::handleRouteExpiryTimeout");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 

  try {
    if (NULL == pRouteInfo) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatorInfo reference.");
    }

    // remove route in kernel
    if (pRouteInfo->routeType != extendedRoute) {
      try {
	pAipKernel->removeRoute( pRouteInfo->getRouteId(), pRouteInfo);
      }
      catch (CZKS::Exception &e)
	{
	  CZKS::Push(e, func, "failure");
	  CZKS::LogException(e);
	}
      if (nRoute > 0) {
         nRoute--;
      }
    }

    // make a route destroy packet
    LinkId fwdLinkId = pRouteInfo->getFwdLinkId();
    if( pRouteInfo->routeState == intHopRouteUp )
      {
	cLinkManagerInfo* fwdLinkInfo = pLinkManager->getLink( fwdLinkId);
	if (fwdLinkInfo) {
	  sendRouteDestroy( fwdLinkInfo, pRouteInfo->getPktVersion(),
			    pRouteInfo->getFwdAci(),
			    ZKS_ROUTEDESTROY_ROUTETIMEOUT_ERR, 
			    getAipEntity(), pRouteInfo->getFwdCrypt());
	}
      }

    // send packet in backward direction
    LinkId bwdLinkId = pRouteInfo->getBwdLinkId();
    cLinkManagerInfo* bwdLinkInfo = pLinkManager->getLink( bwdLinkId);
    if (bwdLinkInfo) {
      sendRouteDestroy( bwdLinkInfo, pRouteInfo->getPktVersion(),
			pRouteInfo->getBwdAci(),
			ZKS_ROUTEDESTROY_ROUTETIMEOUT_ERR, 
			getAipEntity(), pRouteInfo->getBwdCrypt());
    }

    //delete the route table entries
    cRouteCreatorAciMap* pAciMap = findAciMap( bwdLinkId);
    if (!pAciMap) {
      CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NOACIMAP_ERR,
		   func, "failed to find aci map");
    }

    deleteRouteTableEntry( pRouteInfo, pAciMap,
			   pRouteInfo->getBwdAci(), 
			   pRouteInfo->getFwdAci(), fwdLinkId);

  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException(e);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}


void cRouteCreator::handleRouteTimeoutEvent(
  czkEventHandlerData *pData)
{
  DECL_FUNC("cRouteCreator::handleRouteTimeoutEvent");

  try { 
    if (NULL == pData) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input eventHandlerData reference.");
    }

    cRouteCreatorTimeoutHandlerData* pTimeoutData = 
      (cRouteCreatorTimeoutHandlerData*) pData;
 
    cRouteCreatorInfo *pRouteInfo = pTimeoutData->pRouteCreatorInfo;
    
    INITLOCK(tablelock, &routeTableLock);
    LOCK(tablelock);
       
    if( pTimeoutData->isRouteAckTimeout() )
      {
	handleRouteAckTimeout(pRouteInfo);
      }
    else if( pTimeoutData->isRouteExpiryTimeout())
      {
	handleRouteExpiryTimeout(pRouteInfo);
      }

    UNLOCK(tablelock);
  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException(e);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}


void cRouteCreator::handleKeyChangedEvent(
  cKeyProviderCacheEntryData  &entryData, 
  LinkId                       linkId, 
  FrAci                        aci, 
  cRouteCreatePkt             *rcPkt)
{
  DECL_FUNC("cRouteCreator::handleKeyChangedEvent");

  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  ZKS_ROUTECREATOR_MOD,
		 " in %s ",
		 func); 

  try {
    if (NULL == rcPkt) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routeCreatePkt reference.");
    }


    INITLOCK(tablelock, &routeTableLock);
    LOCK(tablelock);

    cRouteCreatorAciMap* pAciMap = findAciMap( linkId);
 
    bool routeExists = false;
            
    cRouteCreatorInfo *pRouteInfo;

    if (pAciMap) {

      pRouteInfo = pAciMap->find(aci); 
      routeExists = (pRouteInfo != NULL);

    }
    
    cLinkManagerInfo* linkInfo = pLinkManager->getLink( linkId);
    if (!linkInfo) {
      CZKS::Throw( ZKS_ROUTECREATOR_MOD, 
		   ZKS_ROUTECREATOR_NOLINKINFO_ERR,
		   func, 
		   "Link does not exist");
    }

    FrKeyQryState state = entryData.getState();
    if (state == FRKEYQRY_ST_VALID) 
      {

	if( routeExists )
	  {

	    if( pRouteInfo->routeState == lastHopPendingKey )
	      {

		//validate the nym sig now that we have its key
                if( rcPkt->authenticate( entryData.getPubKey()) )
		  {
   		    RouteId routeId = pRouteInfo->getRouteId();
                    if (rcPkt->getFlags() & FRPKT_RTECRT_FL_SPEC_ACI) 
		      {
			FrAci wormaci = pRouteInfo->getFwdAci();
                        cRouteCreatorInfo* pOldRouteInfo = 
			  wormAciTable[wormaci].pOldRouteInfo;

                        // update route in kernel
                        pAipKernel->updateRoute( routeId,
                                                 pRouteInfo,
                                                 pOldRouteInfo);
                        // kill the old route
                        pOldRouteInfo->routeType = extendedRoute;
                        handleRouteExpiryTimeout(pOldRouteInfo);

                        wormAciTable[wormaci].pOldRouteInfo = pRouteInfo;

		      }
                    else
		      {
                        // Add route to kernel
		        pAipKernel->addRoute( routeId, pRouteInfo);
                        nRoute++;

		      }

                    sendRouteCreateAck( linkInfo,
                                        pRouteInfo->getPktVersion(), 
                                        pRouteInfo->getFwdAci(), 
                                        aci,
                                        rcPkt->getMacKey(),
                                        pRouteInfo->getFwdCrypt());

                    pRouteInfo->scheduleRouteExpiryTimer(coreloop, this, 
                                                         routeExpiryEH);

                    pRouteInfo->routeState = lastHopRouteUp;
 
		  }
                else
		  {
   		    ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD,
                                   "Invalid Nym Signature");
                   
                    // Invalid signature
      	            sendRouteDestroy( linkInfo, pRouteInfo->getPktVersion(),
                                      aci,
                                      ZKS_ROUTEDESTROY_INVALIDSIG_ERR, 
                                      getAipEntity(), pRouteInfo->getBwdCrypt());

                    // return aci to pool
                    FrAci nextAci = pRouteInfo->getFwdAci();
                    returnWormAci( nextAci);

                    // remove cRouteCreatorInfo from routeTableAci
                    pAciMap->erase(aci); 
            
                    // destroy cRouteCreatorInfo
                    pRouteInfo->cleanup(routeIdBin);
                    delete pRouteInfo;
		  }
	      } // if( lastHopPendingKey)
	  }
      } 
    else  // Key request not successful
      {
         
	if( routeExists )
          {
	    if( pRouteInfo->routeState == lastHopPendingKey )
	      {
      	        sendRouteDestroy( linkInfo, pRouteInfo->getPktVersion(),
                                  aci,
                                  ZKS_ROUTEDESTROY_KEYREQFAILED_ERR, 
                                  getAipEntity(), pRouteInfo->getBwdCrypt());
                    
                // return aci to pool
                FrAci nextAci = pRouteInfo->getFwdAci();
                returnWormAci( nextAci);

                // remove cRouteCreatorInfo from routeTableAci
                pAciMap->erase(aci); 
            
                // destroy cRouteCreatorInfo
                pRouteInfo->cleanup(routeIdBin);
                delete pRouteInfo;
	      }
          }
      }
    UNLOCK(tablelock);

  }
  catch (CZKS::Exception &e) {
    CZKS::Push(e, func, "failure");
    CZKS::LogException(e);
  }
  catch (...) {
    CZKS::Rethrow( func, "Failed");
  }
}


//return the route expiry time
ZKPRIntervalTime cRouteCreator::getRouteExpiryTimeInterval(uint32_t delta)
{
    czkIntervalTime ti;
    
    uint32_t d = delta;
    if (d > (uint32_t)getMaxRouteExpiryTime()) {
       d = getMaxRouteExpiryTime();
    }
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, " route expiry time %d ", d);

    return ti.getSeconds( d);
}

//return the route Ack time
ZKPRIntervalTime cRouteCreator::getRouteAckTimeInterval()
{
   czkIntervalTime ti;
   return ti.getSeconds( getRouteAckTime());
}
