#include "cRouteCreatorInfo.h"
#include "cRouteCreator.h"
#include "cRouteCreatorTimeoutHandler.h"

#define REV_ID "@(#) $Id: cRouteCreatorInfo.cpp,v 1.2 2001/04/18 17:37:32 alainc Exp $"
#include <zkrev.h>




//constructor
cRouteCreatorInfo::cRouteCreatorInfo()
{
   
  pktVersion = 0;

  routeState = intHopPendingAck;

  flags = 0;

  pRouteExpiryTimerProxy = NULL;
  pRouteExpiryEvent = NULL;

  pRouteAckTimerProxy = NULL;
  pRouteAckEvent = NULL;

  rcPkt = NULL;
}

//destructor
cRouteCreatorInfo::~cRouteCreatorInfo() 
{
}

//initialize the routeId
void cRouteCreatorInfo::init(GenericIdBin& routeIdBin) 
{
  routeId = routeIdBin.pop(ZKS_LINKMANAGER_MOD);
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_LINKMANAGER_MOD, " linkid %x ", routeId.getID());
}

//cleanup the timers and return the routeID to the bin
void cRouteCreatorInfo::cleanup(GenericIdBin& routeIdBin) 
{
   cancelTimers();
   routeIdBin.push(routeId);

   if (rcPkt) {
      if (rcPkt->decRefcount() == 0) {
	 rcPkt->detachPkt();
         delete rcPkt;
      }
   }
}

void cRouteCreatorInfo::setFwd(LinkId linkId, FrAci aci, ZKPRNetAddr addr) 
{
   fwdIPinfo.linkId = linkId;
   fwdIPinfo.aci    = aci;
   fwdIPinfo.addr   = addr;
}
  
void cRouteCreatorInfo::setBwd(LinkId linkId, FrAci aci, ZKPRNetAddr addr) 
{
   bwdIPinfo.linkId = linkId;
   bwdIPinfo.aci    = aci;
   bwdIPinfo.addr   = addr;
  
}

void cRouteCreatorInfo::swapBwdFwd() 
{
   cRouteCreatorIPinfo temp;  

   temp      = bwdIPinfo;
   bwdIPinfo = fwdIPinfo;
   fwdIPinfo = temp;
  
}

//cancel the expiry timer & associated event record
void cRouteCreatorInfo::cancelRouteExpiryTimer()
{

   if (pRouteExpiryTimerProxy != NULL) {
      pRouteExpiryTimerProxy->cancelTimer();
      delete pRouteExpiryTimerProxy;
      pRouteExpiryTimerProxy = NULL;
   }

   if (pRouteExpiryEvent != NULL) {
      delete pRouteExpiryEvent;
      pRouteExpiryEvent = NULL;
   }

}

//cancel the route ack timer & associated event record
void cRouteCreatorInfo::cancelRouteAckTimer()
{

   if (pRouteAckTimerProxy != NULL) {
      pRouteAckTimerProxy->cancelTimer();
      delete pRouteAckTimerProxy;
      pRouteAckTimerProxy = NULL;
   }

   if (pRouteAckEvent != NULL) {
      delete pRouteAckEvent;
      pRouteAckEvent = NULL;
   }

}

//cancel all timers
void cRouteCreatorInfo::cancelTimers()
{

   cancelRouteExpiryTimer();
   cancelRouteAckTimer();

}

//schedule a new expiry timer
void cRouteCreatorInfo::scheduleRouteExpiryTimer(
  czkCoreLoop* coreloop, 
  cRouteCreator* pRouteCreator, 
  cRouteCreatorTimeoutHandler* eh)
{
  DECL_FUNC("cRouteCreatorInfo::scheduleRouteExpiryTimer");

  try {
    if (NULL == coreloop) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input coreloop reference.");
    }

    if (NULL == pRouteCreator) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routecreator reference.");
    }

    if (NULL == eh) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input eventhandler reference.");
    }

    //cancel it if it already exists
    cancelRouteExpiryTimer();

    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, 
		   " schedule route expiry timer routeId %d ", getRouteId()); 
 
    try {
      pRouteExpiryEvent = new cRouteCreatorTimeoutHandlerData( this);
      pRouteExpiryEvent->setRouteExpiryTimeout();
      pRouteExpiryTimerProxy =
	coreloop->scheduleTimer( eh, pRouteExpiryEvent, 
				 pRouteCreator->getRouteExpiryTimeInterval(expTime - inTime));
    }
    catch (CZKS::Exception &e) {
      e.Rethrow(func, "Failed to schedule Route Expiry Timeout");
    }
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }
}


//schedule a new route ack timer
void cRouteCreatorInfo::scheduleRouteAckTimer(
  czkCoreLoop* coreloop, 
  cRouteCreator* pRouteCreator, 
  cRouteCreatorTimeoutHandler* eh)
{
  DECL_FUNC("cRouteCreatorInfo::scheduleRouteAckTimer");

  try {
    if (NULL == coreloop) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input coreloop reference.");
    }

    if (NULL == pRouteCreator) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input routecreator reference.");
    }

    if (NULL == eh) {
      CZKS::Throw(ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_NULLPTR_ERR,
		  func, "Null input eventhandler reference.");
    }


    //cancel it if it already exists
    cancelRouteAckTimer();

    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, " schedule route Ack timer %d ", 
		   getRouteId());

    try {
      pRouteAckEvent = new cRouteCreatorTimeoutHandlerData(this);
      pRouteAckEvent->setRouteAckTimeout();
      pRouteAckTimerProxy =
	coreloop->scheduleTimer( eh, pRouteAckEvent, 
				 pRouteCreator->getRouteAckTimeInterval());
    }
    catch (CZKS::Exception &e) {
      e.Rethrow( func, "Failed to schedule Route Ack Timeout");
    }
  }
  catch (...) {
    CZKS::Rethrow(func, "Failed.");
  }
}


void cRouteCreatorInfo::setExpTime( uint32_t exptime, uint32_t routeAckTime)
{
   const char* func = "cRouteCreatorInfo::setExpiryTime";

   try {
       struct timeval now;
       if (gettimeofday(&now, 0) == -1) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_TIMEOFDAY_ERR,
                       func, "gettimeofday failed");
       }

       if (exptime <= now.tv_sec + routeAckTime) {
	  CZKS::Throw( ZKS_ROUTECREATOR_MOD, ZKS_ROUTECREATOR_BADEXPTIME_ERR,
                       func, 
                       "Route pre-expired.  Client time may be out of sync");
       }

       inTime = now.tv_sec;
       expTime = exptime;

   }
   catch (...) {
      CZKS::Rethrow(func, "Failed to set Route Expiry Time");
   }

}
