#define REV_ID "@(#) $Id: testRouteCreator.cpp,v 1.2 2001/04/18 17:37:36 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cRouteCreator
//
// usage:      testRouteCreator testType nhops specAci
//
//             where typeType = 1 (route create pkt)
//                              2 (route create, then route ack)
//                              3 (route create, then route destroy)
//                              4 (route create, then call reportBrokenLink)
//        
//               and    nhops = 1 or 2 (no. of hops in generated route create)
//
//                      specAci = 0 (normal) or 1 (extend route)
//                                
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkspath.h>
#include <zkSignal.h>
    
#include <iostream>
#include <fstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkEventHandler.h"
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

#include "cAipKernel.h"
#include "cNetInfoProvider.h"
#include "cConfigProvider.h"
#include "cLinkManager.h"
#include "cKeyProvider.h"
#include "cPrivateKeys.h"
#include "cPktDispatcher.h"
#include "cRouteCreator.h"

#include "GenericId.h"
#include "cAciBin.h"

#include "global.h"

namespace {

  enum UnitTestCode {
    UT_PASS = 0,
    UT_FAIL = 1,
    UT_UNRESOLVED = 2,
    UT_NOTINUSE = 3,
    UT_UNSUPPORTED = 4,
    UT_UNTESTED = 5,
    UT_UNINITIATED = 6,
    UT_NORESULT = 7,
    UT_FATAL = 101,
    UT_INSPECT = 102
  };
  
  const size_t NUM_ROUTES_MAX = 5;

  const string TEST1_STR = " Test 1:  Create route";
  const string TEST2_STR = " Test 2:  Create then ack route";
  const string TEST3_STR = " Test 3:  Create then destroy route";
  const string TEST4_STR = " Test 4:  Create route then mark link down";
  const string TEST5_STR = " Test 5:  Create and destroy route, then mark link down";
};

#define CRYPT_ALG_0 FRCRYPT_SYMALG_BLOWFISH
#define CRYPT_ALG_1 FRCRYPT_SYMALG_BLOWFISH
#define CRYPT_ALG_2 FRCRYPT_SYMALG_BLOWFISH
#define UINT16_MAX      (65535U)

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

FrCryptSym sndBckCrypt[NUM_ROUTES_MAX], sndFwdCrypt[NUM_ROUTES_MAX];

void setupRouteCreatePkt(ZkError err, int nhops, int specAci, char** retbuf,
                         size_t *retsize, FrCryptCrypt* retkey,
                         FrCryptSig* nymSig, FrAci* aci);

int32_t main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return UT_FATAL;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      return UT_FATAL;
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", "AIP");
      return UT_FATAL;
    }    

     try {
   
      try {

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

        bool dothread = false;

        czkCoreLoop * coreloop = new czkCoreLoop();

        std::ifstream istrm("./testRouteCreator.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_ROUTECREATOR_MOD, 1, "testRouteCreator",
                      "Failed opening file: ./testRouteCreator.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

        // start cConfigProvider
        cConfigProvider* pConfigProvider = new cConfigProvider(m_cmdline,
                                                               m_ini);
        mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);

        // start cAipKernel
        cAipKernel* pAipKernel = new cAipKernel;
        mgr.add(ZKS_AIPKERNEL_MOD, pAipKernel);

        // start NetInfoProvider component thread
        cNetInfoProvider* pNetInfoProvider = new cNetInfoProvider;
        mgr.add(ZKS_NETINFOPROVIDER_MOD, pNetInfoProvider);

        // start LinkManager component thread
        cLinkManager* pLinkManager = new cLinkManager;
        mgr.add(ZKS_LINKMANAGER_MOD, pLinkManager);

        // start PrivateKeys component thread
        cPrivateKeys* pPrivateKeys = new cPrivateKeys;
        mgr.add(ZKS_PRIVATEKEYS_MOD, pPrivateKeys);

        // start KeyProvider component thread
        cKeyProvider* pKeyProvider = new cKeyProvider;
        mgr.add(ZKS_KEYPROVIDER_MOD, pKeyProvider);

        // start PktDispatcher component thread
        cPktDispatcher* pPktDispatcher = new cPktDispatcher;
        mgr.add(ZKS_PACKETDISPATCHER_MOD, pPktDispatcher);

        // start RouteCreator component thread
        cRouteCreator* pRouteCreator = new cRouteCreator;
        mgr.add(ZKS_ROUTECREATOR_MOD, pRouteCreator);
        if (!pRouteCreator->validateConfig()) {
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_ROUTECREATOR_MOD, " validateConfig failed");
          return UT_FATAL;
	}


        mgr.initAllComponents(coreloop);

        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_AIPKERNEL_MOD, dothread);
        mgr.startComponent(ZKS_NETINFOPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_LINKMANAGER_MOD, dothread);
        mgr.startComponent(ZKS_PRIVATEKEYS_MOD, dothread);
        mgr.startComponent(ZKS_KEYPROVIDER_MOD, dothread);
        mgr.startComponent(ZKS_PACKETDISPATCHER_MOD, dothread);
        mgr.startComponent(ZKS_ROUTECREATOR_MOD, dothread);

        pRouteCreator->waitForReadySignal(ZKPR_INTERVAL_NO_TIMEOUT);

        cRouteCreatePkt rc;
        uint8_t       * pkt;
        size_t          pktSize;
        CZKS::Exception e;

        FrCryptPubKey   cryptKey;
        FrCryptSig      nymSig;
        FrAci           aci;

        int specAci2 = 0;
        int testType = rand() % 5 + 1;
        int nhop = rand() % 2 + 1;
        int specAci = rand() % 2;

        // create route create packet
        setupRouteCreatePkt(e.zkError(), nhop, specAci, (char**)&pkt,
                            &pktSize, &cryptKey, &nymSig, &aci); 

        uint8_t* sPkt = new uint8_t[pktSize];
        memcpy(sPkt, pkt, pktSize);

        uint8_t* tPkt = new uint8_t[pktSize];
        memcpy(tPkt, pkt, pktSize);

	setGcryptKey(cryptKey);
        setGnymSig(nymSig);

        rc.attachPkt(tPkt, pktSize);
        rc.decrypt(cryptKey);
        FrEntIdExp aip = rc.getNextEntity();
        delete tPkt;

        sprintf(aip.name,"foo%d\0", 1);

        cLinkManagerInfo* linkInfo = pLinkManager->getLink( aip);
        pRouteCreator->handlePkt( linkInfo, pkt, pktSize );

        setupRouteCreatePkt(e.zkError(), nhop, specAci2, (char**)&pkt,
                            &pktSize, &cryptKey, &nymSig, &aci); 
	setGcryptKey(cryptKey);
        setGnymSig(nymSig);
        pRouteCreator->handlePkt( linkInfo, pkt, pktSize );

        if (testType == 1) {
            std::cout << "ok" << std::endl;
	}
        else if (testType == 2) {
            cRouteCreateAckPkt rca;
            FrAci           exitAci = 1;
            FrAci           nextHopAci = aci;
            CZKS::Exception e;
 
            FrCryptMacKey mk = 0;
            char mkData[FRCRYPT_MACKEY_SIZE];

            assert(frCryptMacKeyNew(e.zkError(), &mk));
            assert(zkRandGetData(e.zkError(), mkData, sizeof mkData));
            assert(frCryptMacKeyLoad(e.zkError(), mk, mkData, sizeof mkData));
 
            // create route create ack packet
            rca.create(FRPKT_DATA_VERSION_1, FRPKT_PRIORITY_DEFAULT,
                       exitAci, nextHopAci, mk);
            for (int i = 0; i < nhop; i++)
	      rca.encrypt(sndFwdCrypt[i]);
            
            rca.getBuffer(&pkt, &pktSize);
            pRouteCreator->handlePkt( linkInfo, pkt, pktSize );
            rca.destroy();
            std::cout << "ok" << std::endl;
	}
        else if (testType == 3  || testType == 5) {
            cRouteDestroyPkt   rd;
            FrAci           nextHopAci = aci;
            FrEntIdExp      entity;
            strcpy(entity.name, "mememe");
            entity.type = FRENT_TYPE_AIP;
            // create route create destroy packet
            rd.create(FRPKT_DATA_VERSION_1, FRPKT_PRIORITY_DEFAULT,
                      nextHopAci, 1, entity);
            for (int i = 0; i < nhop; i++)
	      rd.encrypt(sndFwdCrypt[i]);
            rd.getBuffer(&pkt, &pktSize);
            ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, " rd "," rdpktsize %d ", pktSize);
            pRouteCreator->handlePkt( linkInfo, pkt, pktSize );
            rd.destroy();
            if (testType == 5) {
   	       pRouteCreator->reportBrokenLink( linkInfo->getLinkId() );
	    }
            std::cout << "ok" << std::endl;
	}
        else if (testType == 4) {
	    pRouteCreator->reportBrokenLink( linkInfo->getLinkId() );
            std::cout << "ok" << std::endl;
	}
        //int32_t status = coreloop->runEventLoop();
        sleep(1);
        coreloop->abortEventLoop();
      }
      catch (CZKS::Exception e) {
        std::cout << "not ok" << std::endl;
        e.Rethrow( ZKS_ROUTECREATOR_MOD, "exiting main");
      }
      catch (...) {
        std::cout << "not ok" << std::endl;
        CZKS::Rethrow(ZKS_ROUTECREATOR_MOD, "exiting main");
      }
    }
    catch (CZKS::Exception e) {

      CZKS::LogException( e);
      return UT_FATAL;
    }
       
    // clean up all component objects

    mgr.deleteAllComponents();
    return UT_PASS;
}


static void setRouteN(ZkError err, struct fr_rte_set_exp *r, unsigned int n,
                      int nhops, int specAci)
{
  const char* type = "aip";
  char name[10];
  sprintf( name, "foo%d\0", n);

  r->rteAlg = CRYPT_ALG_0;
  
  assert(frCryptSymNew(err, &r->bckCrypt));
  assert(frCryptSymSetAlg(err, r->bckCrypt, CRYPT_ALG_1));
  
  assert(frCryptSymNew(err, &r->fwdCrypt));
  assert(frCryptSymSetAlg(err, r->fwdCrypt, CRYPT_ALG_2));
  
  assert(frCryptMacKeyNew(err, &r->macKey));
  frEntSetFromStr2(err, &r->hostEnt, type, name);
       
  struct timeval now;
  assert(gettimeofday(&now, 0) != -1);

  r->expTime = now.tv_sec + 5 + (n + 1) * (n + 2) + (n + 3);
  r->flags = n == nhops - 1 ? FRPKT_RTECRT_FL_LAST_RTE : 0;
  if (specAci) {
    r->flags |= FRPKT_RTECRT_FL_SPEC_ACI;
    r->specAci = 8888;
  } else {
    r->specAci = 0;
  }
}


void setupRouteCreatePkt(ZkError err, int nhops, int specAci, char** retbuf,
                         size_t *retsize, FrCryptCrypt* retkey,
                         FrCryptSig* retnymsig, FrAci *retaci)
{
  FrPktRC sPkt;
  FrCryptCrypt aipKey[NUM_ROUTES_MAX];
  FrCryptMacKey sndMacKey[NUM_ROUTES_MAX];
  FrCryptSig nymSig;
  FrCryptSym sym;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;
  int i;
  size_t maxsize;
  size_t pbsz;
  struct fr_nym_auth_exp na0;
  void *pbuf;

  static const char *nymName = "bob the destroyer";

  /*
   *  Setup.
   */
  
  assert(frCryptSymNew(err, &sym));
  assert(frCryptSymSetAlg(err, sym, alg));
  assert(frPktAipRteCrtGetMaxSize(err, 1, &maxsize));
  pbsz = frPktGetSize(maxsize);
  assert(pbuf = malloc(pbsz));

  /* Create public/private encryption keys for all AIPs along the route. */
  for (i = 0; i < nhops; i++) {
    assert(frCryptCryptNew(err, &aipKey[i]));
    assert(frCryptPubKeySetVersion(err, aipKey[i], 1 + i % 7));
    assert(frCryptCryptSetKeySize(err, aipKey[i], FRCRYPT_CRYPTKEYSIZE_1024));
    assert(frCryptCryptSetRandomKeys(err, aipKey[i]));
  }

  /*
   *  Create the packet.
   */

  assert(frPktAipRteCrtNew(err, &sPkt));
  assert(frPktAipRteCrtCreateInBuffer(err, sPkt, pbuf, pbsz));

  /* Check that we can set and get the ACI. */
  {
    FrAci aci0 = (rand() % UINT16_MAX), aci1 = 0;

    assert(frPktAipRteCrtSetAci(err, sPkt, aci0));
    assert(frPktAipRteCrtGetAci(err, sPkt, &aci1));
    assert(aci0 == aci1);
     *retaci  = aci0;
  }
  
  /* Append routes. */
  for (i = 0; i < nhops; i++) {
    struct fr_rte_set_exp r;

    memset(&r, 99, sizeof r);
    setRouteN(err, &r, i, nhops, specAci && i+1 == nhops);
    assert(frPktAipRteCrtAppendRouteExp(err, sPkt, aipKey[i], &r));

    /*
     *  Save the backward and forward cryption objects, and the MAC key, for
     *  later testing.
     */

    sndBckCrypt[i] = r.bckCrypt; r.bckCrypt = 0;
    sndFwdCrypt[i] = r.fwdCrypt; r.fwdCrypt = 0;
    sndMacKey[i] = r.macKey; r.macKey = 0;
  }

  /*
   *  bug: for now use the same key, containing both public and private parts,
   *  for signing and authentication.  We ought to put the two parts into
   *  separate objects.
   */

  assert(frCryptSigNew(err, &nymSig));
  assert(frCryptPubKeySetVersion(err, nymSig, 4)); 
  assert(frCryptSigSetKeySize(err, nymSig, FRCRYPT_SIGKEYSIZE_1024));
  assert(frCryptSigSetRandomKeys(err, nymSig));

  /* Append the nym, then sign. */
  assert(frEntSetFromStr2(err, &na0.nymID, "nym", nymName));
  na0.nymSig = nymSig;

  assert(frPktAipRteCrtAppendNymAuthExp(err, sPkt, &na0));
  assert(frPktAipRteCrtEncryptAllRoutes(err, sPkt));

  /* Trash the sending route creation object. */
  assert(frPktAipRteCrtFree(err, &sPkt));
  assert( ! sPkt);

  *retsize = pbsz + 9;
  *retbuf = new uint8_t[ *retsize];

  void* p = *retbuf;

  memcpy( &p[FRCRYPT_RAND_SIZE], pbuf, pbsz);

            // verify route create packet format
	    
  assert(frPktSetVersion(err, *retbuf, *retsize, 1)); 
  assert(frPktSetType(err, *retbuf, *retsize, 
                     FRPKTTYPE_AIP_ROUTE_CREATE));

  *retkey  = aipKey[0];
  *retnymsig = nymSig;
}

