#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <sys/capability.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <zks.h>
#include <zkIntTypes.h>


/*
 * Enable this switch if you want the user to keep heneritable capacity flag on
 *
 * #define KEEP_INHERITABLE 1
 *
 */
#define KEEP_INHERITABLE 1


/*
 * Enable this switch to anable capability dumping to STDOUT
 *
 * #define XDEBUG 1
 *
 */



#ifdef XDEBUG
# define DUMP_CAPABILITY() dump_cap()
#else
# define DUMP_CAPABILITY()
#endif


#define CAP_VALUE(val)    { val, #val }

#define REV_ID "@(#) $Id: resetcap.cpp,v 1.2 2001/04/18 17:37:40 alainc Exp $"
#include <zkrev.h>




typedef struct cap_value {
  cap_value_t  val;
  const char  *str;
}cap_value;



static void dump_cap()
{
  cap_value cv[] = {
    CAP_VALUE(CAP_SETGID),
    CAP_VALUE(CAP_SETUID),
    CAP_VALUE(CAP_NET_ADMIN),
    CAP_VALUE(CAP_CHOWN),
    CAP_VALUE(CAP_DAC_OVERRIDE),
    CAP_VALUE(CAP_DAC_READ_SEARCH),
    CAP_VALUE(CAP_FOWNER),
    CAP_VALUE(CAP_FSETID),
    CAP_VALUE(CAP_KILL),
    CAP_VALUE(CAP_SETPCAP),
    CAP_VALUE(CAP_LINUX_IMMUTABLE),
    CAP_VALUE(CAP_NET_BIND_SERVICE),
    CAP_VALUE(CAP_NET_BROADCAST),
    CAP_VALUE(CAP_NET_RAW),
    CAP_VALUE(CAP_IPC_LOCK),
    CAP_VALUE(CAP_IPC_OWNER),
    CAP_VALUE(CAP_SYS_MODULE),
    CAP_VALUE(CAP_SYS_RAWIO),
    CAP_VALUE(CAP_SYS_CHROOT),
    CAP_VALUE(CAP_SYS_PTRACE),
    CAP_VALUE(CAP_SYS_PACCT),
    CAP_VALUE(CAP_SYS_ADMIN),
    CAP_VALUE(CAP_SYS_BOOT),
    CAP_VALUE(CAP_SYS_NICE),
    CAP_VALUE(CAP_SYS_RESOURCE),
    CAP_VALUE(CAP_SYS_TIME),
    CAP_VALUE(CAP_SYS_TTY_CONFIG),
  };

  uint32_t limit = sizeof(cv)/sizeof(cv[0]);

  cap_t cap = NULL;

  fprintf(stderr, "Current (uid,gid)=(%d,%d) (euid,egid)=(%d,%d)\n", getuid(), getgid(), geteuid(), getegid());

  if( (cap = cap_init())  == NULL ) {
    fprintf (stderr, "scan_cap: cap_init failed\n");
    exit(1);      
  }  

  if( (cap = cap_get_proc()) == NULL ) {
    fprintf (stderr, "scan_cap: can't iniitalyse capability struture\n");
    exit(1);
  } 

  for( uint32_t i=0; i<limit; i++ ) {
    cap_flag_value_t value[3];

    if( cap_get_flag(cap, cv[i].val ,CAP_EFFECTIVE, &value[0]) == -1 ) {
      fprintf (stderr, "scan_cap: EFFECTIVE cap_get_flag failed for cap %s\n", cv[i].str);
      exit(1);      
    }
    if( cap_get_flag(cap, cv[i].val ,CAP_PERMITTED, &value[1]) == -1 ) {
      fprintf (stderr, "scan_cap: PERMITTED cap_get_flag failed for cap %s\n", cv[i].str);
      exit(1);      
    }
    if( cap_get_flag(cap, cv[i].val ,CAP_INHERITABLE, &value[2]) == -1 ) {
      fprintf (stderr, "scan_cap: INHERITABLE cap_get_flag failed for cap %s\n",  cv[i].str);
      exit(1);      
    }
    fprintf(stdout, "CAP %20s [%c%c%c]\n", cv[i].str,
            ((value[0] == CAP_SET) ? 'e' : ' '),
            ((value[1] == CAP_SET) ? 'p' : ' '),
            ((value[2] == CAP_SET) ? 'i' : ' '));
  }
  cap_free(cap);
  fflush(stdout);
}




uint32_t validate_superuser_cap()
{
  cap_value cv[] = { CAP_VALUE(CAP_SETGID),
                       CAP_VALUE(CAP_SETUID),
                       CAP_VALUE(CAP_CHOWN) };

  uint32_t limit = sizeof(cv)/sizeof(cv[0]);
  cap_t    cap   = NULL;
  uint32_t ok    = 1;

  if( (cap = cap_init())  == NULL ) {    
    fprintf (stdout, "scan_cap: cap_init failed\n");
    exit(1);
  }  

  if( (cap = cap_get_proc()) == NULL ) {
    fprintf (stdout, "validate_superuser_cap: can't iniitalyse capability struture\n");
    exit(1);
  } 

  for( uint32_t i=0; ok && i<limit; i++ ) {
    cap_flag_value_t value;

    if( cap_get_flag(cap, cv[i].val ,CAP_EFFECTIVE, &value) == -1 ) {
      exit(1);      
    }
    ok = (value == CAP_SET) ? ok : 0;
  }
  if( cap_free(cap) ) {
    fprintf (stdout, "validate_superuser_cap: cap_free failed\n");
    exit(1);
  }   
  return ok;
}



//
//
// Reset the processe's capabilities to the strict minimum required in order 
// to be able to talk to the kaip, and then swtich to 'user'.
//
// - This call exit by itself in case of failure
// - Superuser capability should be verified before calling this function
//
void resetUserCapability()
{  
  //
  // Added needeed capability
  //
  //
  cap_value_t allowedCapabilitiesValues[] =  { CAP_SETGID,
                                                 CAP_SETUID,
                                                 CAP_NET_ADMIN,
                                                 CAP_NET_BIND_SERVICE,
                                                 CAP_DAC_OVERRIDE};

  cap_value_t allCapabilitiesValues[] =      { CAP_SETGID,
                                                 CAP_SETUID,
                                                 CAP_NET_ADMIN,
                                                 CAP_CHOWN,
                                                 CAP_DAC_OVERRIDE,
                                                 CAP_DAC_READ_SEARCH,
                                                 CAP_FOWNER,
                                                 CAP_FSETID,
                                                 CAP_KILL,
                                                 CAP_LINUX_IMMUTABLE,
                                                 CAP_NET_BIND_SERVICE,
                                                 CAP_NET_BROADCAST,
                                                 CAP_NET_RAW,
                                                 CAP_IPC_LOCK,
                                                 CAP_IPC_OWNER,
                                                 CAP_SYS_MODULE,
                                                 CAP_SYS_RAWIO,
                                                 CAP_SYS_CHROOT,
                                                 CAP_SYS_PTRACE,
                                                 CAP_SYS_PACCT,
                                                 CAP_SYS_ADMIN,
                                                 CAP_SYS_BOOT,
                                                 CAP_SYS_NICE,
                                                 CAP_SYS_RESOURCE,
                                                 CAP_SYS_TIME,
                                                 CAP_SYS_TTY_CONFIG };

  cap_value_t toRemoveCapabilitiesValues[] = { CAP_SETGID,
                                                 CAP_SETUID };
  

  uint32_t allowedCapcount  = sizeof(allowedCapabilitiesValues)  / sizeof(allowedCapabilitiesValues[0]);
  uint32_t allCapcount      = sizeof(allCapabilitiesValues)      / sizeof(allCapabilitiesValues[0]);
  uint32_t toRemoveCapcount = sizeof(toRemoveCapabilitiesValues) / sizeof(toRemoveCapabilitiesValues[0]);

  cap_t           capabilities = NULL;
  uid_t           uid          = 0;
  uid_t           gid          = 0;

  DUMP_CAPABILITY();

  //
  // Initialyse capability structure
  //
  if( (capabilities = cap_init())  == NULL ) {
    fprintf (stderr, "resetUserCapability: can't initialyse capability struture\n");
    goto failed;
  }


#ifdef KEEP_INHERITABLE
  if( cap_set_flag(capabilities, CAP_INHERITABLE, allCapcount, allCapabilitiesValues, CAP_SET) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1b) failed (%s)\n", strerror(errno));
    goto failed;
  }
#else
  if( cap_set_flag(capabilities, CAP_INHERITABLE, allCapcount, allCapabilitiesValues, CAP_CLEAR) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1b) failed (%s)\n", strerror(errno));
    goto failed;
  }
#endif

  if( cap_set_flag(capabilities, CAP_INHERITABLE,   allowedCapcount, allowedCapabilitiesValues, CAP_SET) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1a) failed (%s)\n", strerror(errno));
    goto failed;
  }

  if( cap_set_flag(capabilities, CAP_PERMITTED,   allCapcount, allCapabilitiesValues, CAP_CLEAR) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1c) failed (%s)\n", strerror(errno));
    goto failed;
  }

  if( cap_set_flag(capabilities, CAP_PERMITTED,   allowedCapcount, allowedCapabilitiesValues, CAP_SET) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1d) failed (%s)\n", strerror(errno));
    goto failed;
  }

  if( cap_set_flag(capabilities, CAP_EFFECTIVE,   allCapcount, allCapabilitiesValues, CAP_CLEAR) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1e) failed (%s)\n", strerror(errno));
    goto failed;
  }
    
  if( cap_set_flag(capabilities, CAP_EFFECTIVE,   allowedCapcount, allowedCapabilitiesValues, CAP_SET) == -1 ) {
    fprintf (stderr, "Call to cap_set_flag(1f) failed (%s)\n", strerror(errno));
    goto failed;
  }

  //
  // Set up final running capabilities
  //
  if( cap_set_proc(capabilities) == -1 ) {
    fprintf (stderr, "Call to cap_set_proc(2) failed (%s)\n", strerror(errno));
    goto failed;
  }

  //
  // Remove unwanted capability
  //
  if( (cap_set_flag(capabilities, CAP_EFFECTIVE, toRemoveCapcount, toRemoveCapabilitiesValues, CAP_CLEAR) == -1) ) {
    fprintf (stderr, "Call to cap_set_flag(6a) failed (%s)\n", strerror(errno));
    goto failed;
  }
  if( (cap_set_flag(capabilities, CAP_PERMITTED, toRemoveCapcount, toRemoveCapabilitiesValues, CAP_CLEAR) == -1) ) {
    fprintf (stderr, "Call to cap_set_flag(6b) failed (%s)\n", strerror(errno));
    goto failed;
  }
#ifndef KEEP_INHERITABLE
  if( (cap_set_flag(capabilities, CAP_INHERITABLE, toRemoveCapcount, toRemoveCapabilitiesValues, CAP_CLEAR) == -1) ) {
    fprintf (stderr, "Call to cap_set_flag(6c) failed (%s)\n", strerror(errno));
    goto failed;
  }
#endif

  if( cap_set_proc(capabilities) == -1 ) {
    fprintf (stderr, "Call to cap_set_proc(6d) failed (%s)\n", strerror(errno));
    goto failed;
  }  

  DUMP_CAPABILITY();

  //
  // Free the capability set
  //
  if( cap_free(capabilities) == -1 ) {
    fprintf (stderr, "Call to cap_free failed (%s)\n", strerror(errno));
    goto failed;
  }

  return;

 failed:
  dump_cap();
  exit(1);
}

