#ifndef STATSDATA_H
# define STATSDATA_H

# include <algorithm>
# include <iostream>

# ifdef __cplusplus

#include "cStatsHarvestorPullHandler.h"

//base class to enable storage in a map
class StatsDataBase {
 private:

  cStatsHarvestorPullHandler* pPullHandler;

 public:

  StatsDataBase() { pPullHandler = NULL; }
  virtual ~StatsDataBase() {};

  void registerPullHandler( cStatsHarvestorPullHandler* ph)
      { pPullHandler = ph; }

  void cancelPullHandler() { pPullHandler = NULL; }

  cStatsHarvestorPullHandler* getPullHandler() { return pPullHandler; }

};

//template class to enable different types of stats data points
template <class T>
class StatsData: public StatsDataBase {
public:
  inline StatsData(const T & val);

  inline StatsData<T> & operator+=(const T & val);
  inline StatsData<T> & operator-=(const T & val);
  inline StatsData<T> & operator= (const T & val);
  inline T &            Val();  
  inline const T &      Val() const;  
  inline T &            Delta();  
  inline const T &      Delta() const;  
  inline T &            Max();  
  inline const T &      Max() const;  
  
  
private:
  T m_val;
  T m_delta;
  T m_temp;
  T m_max;
};
  

template <class T>
StatsData<T>::StatsData(const T & val)
  :m_val(val)
  ,m_delta(val) 
  ,m_temp(val) 
  ,m_max(val) 
{
}


template <class T>
StatsData<T> & StatsData<T>::operator+=(const T & val)
{  
  m_val += val;
  m_delta += val;
#ifdef WIN32
  m_max  = max(m_max, val);
#else
  m_max  = std::max(m_max, val);
#endif
  return *this;
}


template <class T>
StatsData<T> & StatsData<T>::operator-=(const T & val)
{
  m_val -= val;
  m_delta -= val;
#ifdef WIN32
  m_max  = max(m_max, - val);
#else
  m_max  = std::max(m_max, - val);
#endif
  return *this;
}


template <class T>
StatsData<T> & StatsData<T>::operator= (const T & val)
{
  m_val = val;
  m_delta = val;
  m_max  = std::max(m_max, val);
  return *this;
}


template <class T>
T & StatsData<T>::Val()
{
  return m_val;
}


template <class T>
const T & StatsData<T>::Val() const
{
  return m_val;
}

template <class T>
T & StatsData<T>::Delta()
{
  m_temp = m_delta;
  m_delta = (T)0;
  return m_temp;
}

template <class T>
const T & StatsData<T>::Delta() const
{
  m_temp = m_delta;
  m_delta = (T)0;
  return m_temp;
}


template <class T>
T & StatsData<T>::Max()
{
  return m_max;
}


template <class T>
const T & StatsData<T>::Max() const
{
  return m_max;
}

# endif /* __cplusplus */
#endif /* STATSDATA_H */
