#ifndef _cStatsHarvestor

#define _cStatsHarvestor

// class cStatsHarvestor
//

#include "czkIntervalTime.h"
#include "czkLock.h"
#include "czkTimerProxy.h"
#include "czkCoreLoop.h"
#include "czkSockCoDgram.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "frCrypt.h"
#include "frEnt.h"

#include "aip-modules.h"

#include "StatsData.h"
#include "aip-stats-parms.h"

class cNetinfoProvider;

struct ltstat
{
  bool operator()(const char* s1, const char* s2) const
  {
    //use of strcmp is ok
    return strcmp(s1, s2) < 0;
  }
};

class cStatsHarvestor : public cAipComponent {

  friend class cStatsHarvestorPktData;

private:

  // a map containing a pointer to all component objects 
  typedef  map <const char*,  StatsDataBase*, ltstat> StatsMap;
  StatsMap stats;
  
  //configuration items

  //private methods
  //copy constructor (disabled)
  cStatsHarvestor( const cStatsHarvestor& c): cAipComponent(c) { assert(0); };

  //assignment operator (disabled)
  cStatsHarvestor& operator=( const cStatsHarvestor&) { assert(0); };
 
  //return a stats point
  StatsDataBase* findStats(const char* name);

public:

  // constructor
  cStatsHarvestor();

  // destructor
  virtual ~cStatsHarvestor();

  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);

  // cleanup private data members
  void cleanupPrivateData();

  // startup
  void startup();

  // shutdown
  void shutdown() {};

  //register a stats point 
  void registerStats(const char* name, StatsDataBase* p);

  //register a stats point (with a pullHandler) 
  void registerStats(const char* name, StatsDataBase* p, 
                     cStatsHarvestorPullHandler* ph);

  //remove a stats point 
  void removeStats(const char* name);

  //return a const stats point
  const StatsDataBase* getStats(const char* name);
    
  //invoke any register pullHandlers
  void invokePullHandlers();

};

#endif
