#include "cStatsHarvestor.h"

#define REV_ID "@(#) $Id: cStatsHarvestor.cpp,v 1.2 2001/04/18 17:37:49 alainc Exp $"
#include <zkrev.h>




// constructor
cStatsHarvestor::cStatsHarvestor() {}

// destructor
cStatsHarvestor::~cStatsHarvestor() 
{
}
 
// validate configuration
bool cStatsHarvestor::validateConfig()
{
   return true;
}

// initialize private data members
bool cStatsHarvestor::initializePrivateData(czkCoreLoop* coreloopin) 
{ 
   (void)coreloopin;
   return true;
}

// cleanup private data members
void cStatsHarvestor::cleanupPrivateData() 
{ 
  // delete all register stat points
   StatsMap::iterator i = stats.begin();
   while (i != stats.end()) {   
      delete i->second;
      i++;
   }
   stats.clear();
}

// startup
void cStatsHarvestor::startup() 
{
   signalReady();
}

//register a new stats point
void cStatsHarvestor::registerStats(const char* name, StatsDataBase* p)
{ 
   const char* func = "cStatsHarvestor::registerStats";

   ZKS_ASSERT(ZKS_STATSHARVESTOR_MOD, p);

   StatsMap::iterator i = stats.find(name);
   if (i != stats.end()) {
      ostrstream os;
      os << "Stat point " << name << " already registered" << std::ends;
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, 
                   ZKS_STATSHARVESTOR_ALREADYREGISTERED_ERR,
                   func, os.str());
   }
   stats[name] = p;
      
}

//register a new stats point (with PullHandler
void cStatsHarvestor::registerStats(const char* name, StatsDataBase* p,
                                    cStatsHarvestorPullHandler* ph)
{ 
   const char* func = "cStatsHarvestor::registerStats";

   ZKS_ASSERT(ZKS_STATSHARVESTOR_MOD, p);
   ZKS_ASSERT(ZKS_STATSHARVESTOR_MOD, ph);

   StatsMap::iterator i = stats.find(name);
   if (i != stats.end()) {
      ostrstream os;
      os << "Stat point " << name << " already registered" << std::ends;
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, 
                   ZKS_STATSHARVESTOR_ALREADYREGISTERED_ERR,
                   func, os.str());
   }
   p->registerPullHandler( ph);
   stats[name] = p;
      
}

//remove an existing stats point
void cStatsHarvestor::removeStats(const char* name)
{ 
   const char* func = "cStatsHarvestor::removeStats";

   StatsMap::iterator i = stats.find(name);
   if (i == stats.end()) {
      ostrstream os;
      os << "Stat point " << name << " not registered" << std::ends;
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, 
                   ZKS_STATSHARVESTOR_NOTREGISTERED_ERR,
                   func, os.str());
   }
   delete i->second;
   stats.erase(i);
      
}

//return a const stats point 
const StatsDataBase* cStatsHarvestor::getStats(const char* name)
{ 
   const char* func = "cStatsHarvestor::getStats";

   StatsMap::iterator i = stats.find(name);
   if (i == stats.end()) {
      ostrstream os;
      os << "Stat point " << name << " not registered" << std::ends;
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, 
                   ZKS_STATSHARVESTOR_NOTREGISTERED_ERR,
                   func, os.str());
   }
   return i->second;
 
}

//return a stats point
StatsDataBase* cStatsHarvestor::findStats(const char* name)
{ 
   const char* func = "cStatsHarvestor::getStats";

   StatsMap::iterator i = stats.find(name);
   if (i == stats.end()) {
      ostrstream os;
      os << "Stat point " << name << " not registered" << std::ends;
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, 
                   ZKS_STATSHARVESTOR_NOTREGISTERED_ERR,
                   func, os.str());
   } 
   return i->second;
 
}

void cStatsHarvestor::invokePullHandlers()
{
   const char* func = "cStatsHarvestor::invokePullHandlers";

   StatsMap::iterator i = stats.begin();
   while (i != stats.end())
   {
      StatsDataBase* p = i->second;
      cStatsHarvestorPullHandler* ph = p->getPullHandler();
      if (ph) {
	 try {
 	    ph->handlePull( i->first, p);
	 }
         catch (CZKS::Exception &e) {
      CZKS::Push(e, func, "failure");
  	    CZKS::LogException(e);
	 }
      }
      i++;
   }
}
