#include "cStatsHarvestorPktData.h"
#include "cStatsHarvestor.h"

#define REV_ID "@(#) $Id: cStatsHarvestorPktData.cpp,v 1.2 2001/04/18 17:37:49 alainc Exp $"
#include <zkrev.h>

//constructor - init dynamic memory pointers to NULL

cStatsHarvestorPktData::cStatsHarvestorPktData() 
{

   pktRcvd      = NULL;
   pktSent      = NULL;
   pktDropped   = NULL;
   bytesRcvd    = NULL;
   bytesSent    = NULL;
   bytesDropped = NULL;

}

// initialize reference to registered stats
void cStatsHarvestorPktData::initRegisteredStats(const char* name1, 
                                                const char* name2, 
                                                const char* name3, 
                                                const char* name4, 
                                                const char* name5, 
                                                const char* name6 )
{
   pktRcvd      = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name1);
   pktSent      = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name2);
   pktDropped   = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name3);
   bytesRcvd    = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name4);
   bytesSent    = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name5);
   bytesDropped = (StatsData<uint64_t>*)pStatsHarvestor->findStats(name6);
}

//remove registered stats
void cStatsHarvestorPktData::removeRegisteredStats(const char* name1, 
                                                  const char* name2, 
                                                  const char* name3, 
                                                  const char* name4, 
                                                  const char* name5, 
                                                  const char* name6 )
{
   pStatsHarvestor->removeStats(name1);
   pStatsHarvestor->removeStats(name2);
   pStatsHarvestor->removeStats(name3);
   pStatsHarvestor->removeStats(name4);
   pStatsHarvestor->removeStats(name5);
   pStatsHarvestor->removeStats(name6);
}

// just create local stats points
void cStatsHarvestorPktData::initNewStats()
{
   const char* func = "cStatsHarvestorPktData::initNewStats";

   try {
      pktRcvd      = new StatsData<uint64_t>(0);
      pktSent      = new StatsData<uint64_t>(0);
      pktDropped   = new StatsData<uint64_t>(0);
      bytesRcvd    = new StatsData<uint64_t>(0);
      bytesSent    = new StatsData<uint64_t>(0);
      bytesDropped = new StatsData<uint64_t>(0);
   }
   catch (...) {
      deleteLocalStats();
      CZKS::Throw( ZKS_STATSHARVESTOR_MOD, ZKS_STATSHARVESTOR_INITNEWSTATS_ERR,
                   func, "failure");
   }

}

// delete local stats points
void cStatsHarvestorPktData::deleteLocalStats()
{

   delete pktRcvd;
   delete pktSent;
   delete pktDropped;
   delete bytesRcvd;
   delete bytesSent;
   delete bytesDropped;

}

// register stat points
void cStatsHarvestorPktData::registerStats(const char* name1, 
                                           const char* name2, 
                                           const char* name3, 
                                           const char* name4, 
                                           const char* name5, 
                                           const char* name6 )
{
   pStatsHarvestor->registerStats(name1, pktRcvd);
   pStatsHarvestor->registerStats(name2, pktSent);
   pStatsHarvestor->registerStats(name3, pktDropped);
   pStatsHarvestor->registerStats(name4, bytesRcvd);
   pStatsHarvestor->registerStats(name5, bytesSent);
   pStatsHarvestor->registerStats(name6, bytesDropped);
}

// register with a pull handler
void cStatsHarvestorPktData::registerStats(const char* name1, 
                                           const char* name2, 
                                           const char* name3, 
                                           const char* name4, 
                                           const char* name5, 
                                           const char* name6,
                                           cStatsHarvestorPullHandler* ph)
{
   pStatsHarvestor->registerStats(name1, pktRcvd,      ph);
   pStatsHarvestor->registerStats(name2, pktSent,      ph);
   pStatsHarvestor->registerStats(name3, pktDropped,   ph);
   pStatsHarvestor->registerStats(name4, bytesRcvd,    ph);
   pStatsHarvestor->registerStats(name5, bytesSent,    ph);
   pStatsHarvestor->registerStats(name6, bytesDropped, ph);
}

// assignment operator
cStatsHarvestorPktData& cStatsHarvestorPktData::operator=(const cStatsHarvestorPktData& val)
{
   *pktRcvd      = val.pktRcvd->Val();
   *pktSent      = val.pktSent->Val();
   *pktDropped   = val.pktDropped->Val();
   *bytesRcvd    = val.bytesRcvd->Val();
   *bytesSent    = val.bytesSent->Val();
   *bytesDropped = val.bytesDropped->Val();
   return *this;
}

// += operator
cStatsHarvestorPktData& cStatsHarvestorPktData::operator+=(const cStatsHarvestorPktData& val)
{  
   *pktRcvd      += val.pktRcvd->Val();
   *pktSent      += val.pktSent->Val();
   *pktDropped   += val.pktDropped->Val();
   *bytesRcvd    += val.bytesRcvd->Val();
   *bytesSent    += val.bytesSent->Val();
   *bytesDropped += val.bytesDropped->Val();
   return *this;
}
 
