#define REV_ID "@(#) $Id: testStatsHarvestor.cpp,v 1.2 2001/04/18 17:37:50 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cStatsHarvestor
//
// usage:      testStatsHarvestor 
//
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
    
#include <iostream>
#include <strstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

#include "cStatsHarvestor.h"
#include "StatsData.h"
#include "aip-stats-parms.h"
#include "cPktDispatcher.h"
#include "cPktDispatcherPktHandler.h"

class cFakePullHandler : public cStatsHarvestorPullHandler {

public:

  void handlePull( const char* name, StatsDataBase* d)
  {
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, " pull for stats %s ", name);
    StatsData<int32_t>* p = (StatsData<int32_t>*)d;
    *p += 100;
  }
};     

class cFakeEventHandlerTimeoutData: public czkEventHandlerData {

public:
 
        StatsData<int32_t>*  dataPull;
        StatsData<int32_t>*  data1;
        StatsData<int64_t>*  data2; 
        StatsData<uint32_t>* data3;
        StatsData<uint64_t>* data4;
        StatsData<float>*    data5; 
        StatsData<double>*   data6;

        cStatsHarvestor* pStatsHarvestor;
        czkCoreLoop* pCoreLoop;

        int count;
   
  void init(cStatsHarvestor* p, czkCoreLoop* pc) {

       pStatsHarvestor = p;
       pCoreLoop = pc;

       count = 0;

       //register a point 
       StatsData<double>*   dum = new StatsData<double>(0.);
       try {
           pStatsHarvestor->registerStats("x0",dum);
           ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, " point registered successfully");
           std::cout << "ok" << std::endl;
	}
        catch (CZKS::Exception &e) {
           std::cout << "not ok" << std::endl;
 	   CZKS::LogException(e);
           abort();
	}

        // this should NOT log a not registered error
        try {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "trying to get a point that is registered");
           const StatsDataBase* psdb = pStatsHarvestor->getStats("x0");
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "success");
           std::cout << "ok" << std::endl;
        }
        catch (CZKS::Exception &e) {
           std::cout << "not ok" << std::endl;
 	   CZKS::LogException(e);
           abort();
	}
 
         // this should log an already registered error
        try {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "trying to register a point already registered");
           pStatsHarvestor->registerStats("x0",dum);
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "registered a point already registered!!!!!");
           std::cout << "not ok" << std::endl;
           abort();
	}
        catch (CZKS::Exception &e) {
           std::cout << "ok" << std::endl;
	   CZKS::LogException(e);
	}

        // this should succeed
        try {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "remove a registered point");
           pStatsHarvestor->removeStats("x0");
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "success");
           std::cout << "ok" << std::endl;
	}
        catch (CZKS::Exception &e) {
           std::cout << "not ok" << std::endl;
	   CZKS::LogException(e);
           abort();
	}

        // this should log a not registered error
        try {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "trying to remove a point not registered");
           pStatsHarvestor->removeStats("x0");
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "removed a point not registered");
           std::cout << "not ok" << std::endl;
           abort();
	}
        catch (CZKS::Exception &e) {
           std::cout << "ok" << std::endl;
 	   CZKS::LogException(e);
	}

        // this should log a not registered error
        try {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "trying to get a point not registered");
           const StatsDataBase* psdb = pStatsHarvestor->getStats("x0");
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, 
                          "got a point not registered");
           std::cout << "not ok" << std::endl;
           abort();
        }
        catch (CZKS::Exception &e) {
           std::cout << "ok" << std::endl;
 	   CZKS::LogException(e);
	}

        dataPull = new StatsData<int32_t>(0);
        data1 = new StatsData<int32_t>(0);
        data2 = new StatsData<int64_t>(0);
        data3 = new StatsData<uint32_t>(0);
        data4 = new StatsData<uint64_t>(0);
        data5 = new StatsData<float>(0.);
        data6 = new StatsData<double>(0.);

        cFakePullHandler* ph = new cFakePullHandler();
    
        pStatsHarvestor->registerStats("dp",dataPull,ph);
        pStatsHarvestor->registerStats("d1",data1);
        pStatsHarvestor->registerStats("d2",data2);
        pStatsHarvestor->registerStats("d3",data3);
        pStatsHarvestor->registerStats("d4",data4);
        pStatsHarvestor->registerStats("d5",data5);
        pStatsHarvestor->registerStats("d6",data6);
 
  };

  void update() {

    pStatsHarvestor->invokePullHandlers();

    *data1 += 10.*rand()/RAND_MAX;
    *data2 += 20.*rand()/RAND_MAX;
    *data3 += 30.*rand()/RAND_MAX;
    *data4 += 40.*rand()/RAND_MAX;
    *data5 += (float)50.*rand()/RAND_MAX;
    *data6 += (double)60.*rand()/RAND_MAX;

    ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "*******************************");
    if (count == 0) {

      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta dp %d ",dataPull->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d1 %d ",data1->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d2 %d ",data2->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d3 %d ",data3->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d4 %d ",data4->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d5 %f ",data5->Delta());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "delta d6 %g ",data6->Delta());
    }
    else if (count == 1) {

      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs dp %d ",dataPull->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d1 %d ",data1->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d2 %d ",data2->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d3 %d ",data3->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d4 %d ",data4->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d5 %f ",data5->Val());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "abs d6 %g ",data6->Val());
    }
    else if (count == 2) {

      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max dp %d ",dataPull->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d1 %d ",data1->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d2 %d ",data2->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d3 %d ",data3->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d4 %d ",data4->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d5 %f ",data5->Max());
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, "max d6 %g ",data6->Max());
    }
    else {
      pCoreLoop->abortEventLoop();
    }

    std::cout << "ok" << std::endl;
    count++;

  };
};

class cFakeEventHandler : public czkEventHandler {

public:

  void handleInput( czkSocket*, czkEventHandlerData*) {};
  void handleOutput( czkSocket*, czkEventHandlerData*) {};
  void handleException( czkSocket*, czkEventHandlerData*) {};
 
  int32_t handleTimeout( czkEventHandlerData* data) 
  {
    cFakeEventHandlerTimeoutData* mydata = (cFakeEventHandlerTimeoutData*)data;
    mydata->update();
    return 1;
  }
};

int32_t main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, " starting StatsHarvestor test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSHARVESTOR_MOD, " running in single thread mode");
        bool dothread = false;
#endif

        czkCoreLoop* coreloop = NULL;
   
        coreloop = new czkCoreLoop(); 
  
        // start StatsHarvestor component
        cStatsHarvestor* pStatsHarvestor = new cStatsHarvestor();
        mgr.add(ZKS_STATSHARVESTOR_MOD, pStatsHarvestor);
        pStatsHarvestor->validateConfig();

 
        mgr.initAllComponents(coreloop);

        mgr.startComponent(ZKS_STATSHARVESTOR_MOD, false);

        cFakeEventHandlerTimeoutData data;
        data.init(pStatsHarvestor, coreloop);

        cFakeEventHandler eh;

        czkIntervalTime ti;
        coreloop->scheduleTimer( &eh, &data, ti.getSeconds(5));

        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;
        }
        else {


          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;

          // wait for all threads to complete

          mgr.waitForCompletion();
        }

      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_STATSHARVESTOR_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow( ZKS_STATSHARVESTOR_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();
}


