#ifndef _cStatsPackager

#define _cStatsPackager

// class cStatsPackager
//

#include "czkIntervalTime.h"
#include "czkLock.h"
#include "czkTimerProxy.h"
#include "czkCoreLoop.h"
#include "czkSockCoDgram.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "frCrypt.h"
#include "frEnt.h"
#include "frstats.h"

#include "aip-modules.h"

#include "cConfigProvider.h"
#include "cKeyProvider.h"

#include "cStatsHarvestorPktData.h"

class cNetInfoProvider;
class cStatsHarvestor;
class cPrivateKeys;

//this is the size (in bytes) of the AIP stats pkt that frstats sends
//to the NISS.  Ideally, the frstats API should return the size, but
//since this interface is supposed to be obsoleted in the near future,
//we will stick with this kludge for now.
#define STATS_PACKAGER_FRSTATS_XMIT_SIZE 816

class cStatsPackager : public cAipComponent {

  friend class cStatsPackagerEventHandlerIO;
  friend class cStatsPackagerKeyHandler;
  friend class cStatsPackagerKeyHandlerCryptData;

private:
  czkSockCoDgram socket;

  class cStatsPackagerEventHandlerIO* eventHandler;
  class cStatsPackagerKeyHandler* keyHandler;
  class cStatsPackagerKeyHandlerCryptData* keyHandlerCryptData;
  class czkTimerProxy* sendTimerProxy; 

  cNetInfoProvider* pNetInfoProvider;
  cKeyProvider*     pKeyProvider;
  cStatsHarvestor*  pStatsHarvestor;
  cPrivateKeys*     pPrivateKeys;
  cConfigProvider*  pConfigProvider;

  uint32_t reqCount;           //no of requests serviced so far this period
  uint32_t statSeq;            //seq. no per STAT packet
  time_t   launchTime;         //time of startup
  time_t   lastNiqsReloadTime; //see above 
  time_t   lastStatTime;       //last time STATS were sent to NISS

  FrStatsCtx statsCtx;             // Aip stats 

  //StatsPackager stats
  cStatsHarvestorPktData* spStats;  // stats "counted" by the StatsPackager 

  bool        NISSinitialized;
  FrEntIdExp  NISSentity;
  ZKPRNetAddr NISSaddr;

  //private methods
  //copy constructor (disabled)
  cStatsPackager( const cStatsPackager& c): cAipComponent(c) { assert(0); };

  //assignment operator (disabled)
  cStatsPackager& operator=( const cStatsPackager&) { assert(0); };
 
public:

  // constructor
  cStatsPackager();

  // destructor
  virtual ~cStatsPackager();

  //methods to get config point values
  string getAipName();
  FrEntIdExp getAipEntity();
  int32_t getStatsPort();
  int32_t getSendStatsTime();
  int32_t getMaxReq();
  int32_t getStatsHistory();
 
  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);

  // cleanup private data members
  void cleanupPrivateData();

  // startup
  void startup();

  // shutdown
  void shutdown();

  // update NISS info via NetInfoProvider
  void updateNISSinfo();

  // initialize the socket
  void initSocket();

};

#endif
