#include "cStatsPackagerEventHandlerIO.h"
#include "cStatsPackager.h"
#include "cStatsPackagerKeyHandler.h"
#include "cStatsPackagerKeyHandlerSigData.h"
  
#include "cNetInfoProvider.h"
#include "cNetInfoEntity.h"

#define REV_ID "@(#) $Id: cStatsPackagerEventHandlerIO.cpp,v 1.2 2001/04/18 17:37:54 alainc Exp $"
#include <zkrev.h>




//set the pointer to the StatsPackager object
void cStatsPackagerEventHandlerIO::setStatsPackager( cStatsPackager* sp)
{
  // save a pointer to the StatsPackager object

  ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, sp);

  pStatsPackager = sp;
}

//handle incoming packets from the NISS
void cStatsPackagerEventHandlerIO::handleInput(
                                       czkSocket* sock, 
                                       czkEventHandlerData* data)
{
  // receiving a packet from the NISS

  (void) data;

  ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, sock);

  const char* func = "cStatsPackagerEventHandlerIO::handleInput"; 

  CZKS::Exception zksException;

  char buffer[BUFSIZ];
  FrEntIdExp entity;
  uint32_t keyVers;

  int32_t pktlen;

  try {

    // receive data
    int32_t nb = BUFSIZ;
    pktlen = sock->recv( buffer, nb, 0, PR_INTERVAL_NO_WAIT);
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, " pktlen %d recvd from NISS",  pktlen);

    if (pktlen < 0) {
      CZKS::Throw( ZKS_STATSPACKAGER_MOD, 
                   ZKS_STATSPACKAGER_SOCKRECV_ERR,
                   func, "Error %d receiving data on socket ", pktlen);
    }
 
    if (!frStatPktGetSigInfo(zksException.zkError(), buffer, pktlen, 
                             &entity, &keyVers)) {
        CZKS::Throw( zksException);
    }

    /*
     * Make sure it is the entity that I'm supposed to be getting for 
     * these types of packets.
     */
    if (entity.type != FRENT_TYPE_NISS) {

      ostrstream os;
      os << "Got a STATS Update query from someone who isn't a NISS " << std::ends;
      CZKS::Throw( ZKS_STATSPACKAGER_MOD, 
                   ZKS_STATSPACKAGER_BADENTITY_ERR,
                   func, os.str());
    }
  
    //set up the KeyHandler data (contains the packet just recvd, 
    // so it can be decrypted0
    cStatsPackagerKeyHandlerSigData* ehd = new cStatsPackagerKeyHandlerSigData;
    memcpy( ehd->buffer, buffer, pktlen);
    ehd->pktlen = pktlen;

    //set up the KeyHandler 
    cStatsPackagerKeyHandler* eh = new cStatsPackagerKeyHandler;
    eh->init( pStatsPackager);
    eh->setData(ehd);

    //make the key request     
    cKeyProviderCacheEntryData entryData; //not used for anything
    pStatsPackager->pKeyProvider->makeKeyRequest( entity.name, entity.type,  
                                                  FRCRYPT_KEYTYPE_SIGNING, 0,  
			                          eh, entryData);

    *pStatsPackager->spStats->pktRcvd      += 1;
    *pStatsPackager->spStats->bytesRcvd    += pktlen;

  }
  catch (CZKS::Exception &e) {

    if (pktlen > 0) {
       *pStatsPackager->spStats->pktDropped     += 1;
       *pStatsPackager->spStats->bytesDropped   += pktlen;
    }

    CZKS::Push(e, func, "failure");
    CZKS::LogException( e);

    zksLogMesg( ZKS_STATSPACKAGER_MOD, "%s terminated abnormally", func);

  }
  catch (...) {

    if (pktlen > 0) {
       *pStatsPackager->spStats->pktDropped     += 1;
       *pStatsPackager->spStats->bytesDropped   += pktlen;
    }

    CZKS::Rethrow( func, "failed");

  }

}

//handle socket errors
void cStatsPackagerEventHandlerIO::handleException( czkSocket* sock, 
                                                  czkEventHandlerData* data)
{

   const char* func = "cStatsPackagerEventHandlerIO::handleException";

   (void) data;

   ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, sock);

   ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, " socket i/o error %d", PR_GetError());

   if (PR_GetError() != 0) {

     try {
       CZKS::Throw(ZKS_STATSPACKAGER_MOD, ZKS_STATSPACKAGER_IO_ERR, func, 
                   "socket i/o error %d ", PR_GetError());
     }
     catch (CZKS::Exception &zksException) {

       CZKS::Push(zksException, func, "failure");
       CZKS::LogException( zksException);

     }
     catch (...) {

       CZKS::Rethrow( func, "failed");

     }
   }

   pStatsPackager->coreloop->removeHandler( sock);

   sock->close();

   pStatsPackager->initSocket();

}

//should never be called
void cStatsPackagerEventHandlerIO::handleOutput( czkSocket*, 
                                                 czkEventHandlerData*)
{
  ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, 0);
}

//handle timeout events
int32_t cStatsPackagerEventHandlerIO::handleTimeout( czkEventHandlerData* data)
{
  const char* func = "cStatsPackagerEventHandlerIO::handleTimeout";

  (void) data;

  //it's time to send the stats 
  // making the CRYPT key request will trigger it all

  try {

    // update NISS info    
    pStatsPackager->updateNISSinfo();

    cKeyProviderCacheEntryData entryData;
    pStatsPackager->pKeyProvider->makeKeyRequest( 
                                    pStatsPackager->NISSentity.name, 
                                    pStatsPackager->NISSentity.type,  
                                    FRCRYPT_KEYTYPE_CRYPTION, 0,  
			            pStatsPackager->keyHandler, entryData);

    //reset the reqCount
    pStatsPackager->reqCount = 0;

  }
  catch (CZKS::Exception &zksException) {

      CZKS::Push(zksException, func, "failure");
    CZKS::LogException( zksException);

    zksLogMesg( ZKS_STATSPACKAGER_MOD, "%s terminated abnormally", func);

  }
  catch (...) {

    CZKS::Rethrow( func, "failed");

  }

  return 1;  // always reschedule
  
}
