#include "cStatsPackagerKeyHandler.h"
#include "cStatsPackager.h"
#include "cStatsHarvestor.h"
#include "cKeyProviderCacheEntryData.h"
#include "StatsData.h"
#include "aip-stats-parms.h"

#define REV_ID "@(#) $Id: cStatsPackagerKeyHandler.cpp,v 1.2 2001/04/18 17:37:54 alainc Exp $"
#include <zkrev.h>




//set the pointer to the StatsPackager object
void cStatsPackagerKeyHandler::init( cStatsPackager* sp)
{
  // save a pointer to the StatsPackager object

  ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, sp);

  pStatsPackager = sp;

}


// handle a key event
void cStatsPackagerKeyHandler::handleEvent(
                                        cKeyProviderCacheEntryData entryData)
{

  const char* func = "cStatsPackagerKeyHandler::handleEvent";

  CZKS::Exception zksException;

  void *buffer = 0;
  void *dbuf = 0;
  cStatsPackagerKeyHandlerSigData* databuf = NULL; 
 
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, 
                 " in handleEvent statsCtx %x ", pStatsPackager->statsCtx);

  try {

    // first check the key state
    FrKeyQryState state = entryData.getState();
    if (state != FRKEYQRY_ST_VALID) {
      CZKS::Throw(ZKS_STATSPACKAGER_MOD, ZKS_STATSPACKAGER_KEYNOTVALID_ERR,
                  func, "Key is not valid");
    }

    // ok, get the key    
    FrCryptPubKey key = entryData.getPubKey();

    // get the original query so we know what type of key it is
    FrKeyQryQueryExp qry = entryData.getQry();
     
    // if it's a SIG key, time to decrypt pkt recv'd from NISS 
    if (qry.keyType == FRCRYPT_KEYTYPE_SIGNING) {

       // get the data pkt to decrypt
       databuf = (cStatsPackagerKeyHandlerSigData*)getData(); 

       // validate the signature
       int32_t valid;
       if (!frSecSigVerify(zksException.zkError(), key, 
                           databuf->buffer, databuf->pktlen, &valid)) {
      
	 CZKS::Throw( zksException);  
       }

       if (!valid) {

	 CZKS::Throw( ZKS_STATSPACKAGER_MOD, 
                      ZKS_STATSPACKAGER_INVALIDSIGNATURE_ERR,
                      func, "Failed to validate NISS signature");

       }
      
       size_t dsz;
       if (!frSecSigRemove(zksException.zkError(), databuf->buffer, 
                           databuf->pktlen,
                           &dbuf, &dsz)) {

	  CZKS::Throw( zksException);

       }

       int32_t id;
       size_t bufsz;
       uint32_t ttl;
       uint32_t vers;
       FrPktType type;
       if (!frStatPktGetUpdInfo(zksException.zkError(), dbuf, dsz, &vers, 
                                &type, &ttl, &id,   
                                &buffer, &bufsz)) {

	  CZKS::Throw( zksException);

       }

       // check if there has not been too many requests
       if (pStatsPackager->reqCount < (uint32_t)pStatsPackager->getMaxReq()) {

          // update NISS info    
          pStatsPackager->updateNISSinfo();

          cKeyProviderCacheEntryData entryData;
          
          pStatsPackager->pKeyProvider->makeKeyRequest( 
                                        pStatsPackager->NISSentity.name, 
                                        pStatsPackager->NISSentity.type,  
                                        FRCRYPT_KEYTYPE_CRYPTION, 0,  
		    	                pStatsPackager->keyHandler, entryData);
          pStatsPackager->reqCount++;

       }
       else {

	 CZKS::Throw( ZKS_STATSPACKAGER_MOD, 
                      ZKS_STATSPACKAGER_TOOMANYREQUESTS_ERR,
                      func, "Exceeded max number of stat requests per period");
       }

    }

    // if it's the CRYPT key, time to make the donuts
    // e.g. get the stats from the StatsHarvestor,
    // package and send them to the NISS
    else if (qry.keyType == FRCRYPT_KEYTYPE_CRYPTION) {

       //tell components that registered "pull" stats to update their stats
       pStatsPackager->pStatsHarvestor->invokePullHandlers();

       // set the CRYPT key      
       if( !frStatsSetNISSCurrentCryptKey(zksException.zkError(), 
                                          pStatsPackager->statsCtx, key)) {
	  CZKS::Throw( zksException);
       }

       //set AIP stats message type
       if( !frStatsUpdSetType(zksException.zkError(), pStatsPackager->statsCtx,
                              FRSTAT_T_AIP)) {
	  CZKS::Throw( zksException);
       }

       cStatsPackagerKeyHandlerCryptData* statsbuf = 
          (cStatsPackagerKeyHandlerCryptData*)getData(); 

       *statsbuf->newClientsCur = statsbuf->newClients->Val();
       *statsbuf->clientsCur    = statsbuf->clients->Val();

       *statsbuf->dhCur = *statsbuf->dh;
       *statsbuf->akCur = *statsbuf->ak;
       *statsbuf->pdCur = *statsbuf->pd;
       *statsbuf->kpCur = *statsbuf->kp;
       *statsbuf->spCur = *statsbuf->sp;
         
       *statsbuf->totCur  = *statsbuf->dhCur;
       *statsbuf->totCur += *statsbuf->akCur;
       *statsbuf->totCur += *statsbuf->pdCur;
       *statsbuf->totCur += *statsbuf->kpCur;
       *statsbuf->totCur += *statsbuf->spCur;

       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " newclients %d ", 
                      statsbuf->newClientsCur->Val() -
		      statsbuf->newClientsLast->Val()); 

       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " clients %d ", 
                      statsbuf->clientsCur->Val() -
		      statsbuf->clientsLast->Val()); 

       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " pktsRcvd %d ", 
                      statsbuf->totCur->pktRcvd->Val() -
		      statsbuf->totLast->pktRcvd->Val());
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " pktsSent %d ", 
                      statsbuf->totCur->pktSent->Val() -
		      statsbuf->totLast->pktSent->Val());
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " pktsDropped %d ", 
                      statsbuf->totCur->pktDropped->Val() -
		      statsbuf->totLast->pktDropped->Val());
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " bytesRcvd %d ", 
                      statsbuf->totCur->bytesRcvd->Val() -
		      statsbuf->totLast->bytesRcvd->Val());
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " bytesSent %d ", 
                      statsbuf->totCur->bytesSent->Val() -
		      statsbuf->totLast->bytesSent->Val());
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD,
                      " bytesDropped %d ", 
                      statsbuf->totCur->bytesDropped->Val() -
		      statsbuf->totLast->bytesDropped->Val());

       //update lastStatTime
       struct timeval tv;
       if( gettimeofday(&tv, NULL) ) {
          CZKS::Throw(ZKS_STATSPACKAGER_MOD, 
                      ZKS_STATSPACKAGER_GETTIMEOFDAY_ERR,
                      func, "gettimeofday failed");
       }
       pStatsPackager->lastStatTime = tv.tv_sec;

       // package stats in NISS format
       if( !frStatsFillAipMsg(zksException.zkError(), 
                              pStatsPackager->statsCtx, 
                              pStatsPackager->statSeq, 
                              pStatsPackager->launchTime, 
                              pStatsPackager->lastNiqsReloadTime, 
                              pStatsPackager->lastStatTime, 
                              statsbuf->newClientsCur->Val() -
                                statsbuf->newClientsLast->Val(), 
                              statsbuf->clientsCur->Val(),  //this is NOT a delta
                              statsbuf->totCur->pktRcvd->Val() -
                                statsbuf->totLast->pktRcvd->Val(),
                              statsbuf->totCur->pktSent->Val() -
                                statsbuf->totLast->pktSent->Val(),
                              statsbuf->totCur->pktDropped->Val() -
                                statsbuf->totLast->pktDropped->Val(),
                              statsbuf->totCur->bytesRcvd->Val() -
                                statsbuf->totLast->bytesRcvd->Val(),
                              statsbuf->totCur->bytesSent->Val() -
                                statsbuf->totLast->bytesSent->Val(),
                              statsbuf->totCur->bytesDropped->Val() -
                                statsbuf->totLast->bytesDropped->Val()))
       {

	  CZKS::Throw( zksException);
       } 
   
       // send stats to NISS
       if( !frStatsSendMsg(zksException.zkError(), pStatsPackager->statsCtx)) {
	  CZKS::Throw( zksException);
       } 

       //only reset the stats if frStatsSendMsg was successful 
       *statsbuf->newClientsLast = *statsbuf->newClientsCur;
       *statsbuf->clientsLast    = *statsbuf->clientsCur;
 
       *statsbuf->dhLast = *statsbuf->dhCur;
       *statsbuf->akLast = *statsbuf->akCur;
       *statsbuf->pdLast = *statsbuf->pdCur;
       *statsbuf->kpLast = *statsbuf->kpCur;
       *statsbuf->spLast = *statsbuf->spCur;
         
       *statsbuf->totLast = *statsbuf->totCur;

       *pStatsPackager->spStats->pktSent   += 1;
       *pStatsPackager->spStats->bytesSent += STATS_PACKAGER_FRSTATS_XMIT_SIZE;

       ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK,  ZKS_STATSPACKAGER_MOD, 
                       "Stats sent to NISS successfully");

       pStatsPackager->statSeq++;

    }

    if (dbuf) {
       free(dbuf);
    } 
    if (buffer) {
       free(buffer);
    }
    delete databuf;

  }
  catch (CZKS::Exception &e) {

     if (dbuf) {
        free(dbuf);
     } 
     if (buffer) {
        free(buffer);
     }
     delete databuf;
     CZKS::Push(e, func, "failure");
     CZKS::LogException(e);

  }
  catch (...) {

     if (dbuf) {
        free(dbuf);
     } 
     if (buffer) {
        free(buffer);
     }
     delete databuf;
     CZKS::Rethrow(func, "failed");
    
  }

}


