#include "cStatsPackagerKeyHandlerCryptData.h"
#include "cStatsPackager.h"
#include "cStatsHarvestor.h"
#include "StatsData.h"
#include "aip-stats-parms.h"

#define REV_ID "@(#) $Id: cStatsPackagerKeyHandlerCryptData.cpp,v 1.2 2001/04/18 17:37:54 alainc Exp $"
#include <zkrev.h>


//constructor - init all dynamic memory pointers to NULL
cStatsPackagerKeyHandlerCryptData::cStatsPackagerKeyHandlerCryptData()
{
  newClientsLast = NULL;
  clientsLast    = NULL;

  newClientsCur = NULL;
  clientsCur    = NULL;

  dh     = NULL;
  dhLast = NULL;         
  dhCur  = NULL;          

  ak     = NULL;
  akLast = NULL;
  akCur  = NULL;

  pd     = NULL;
  pdLast = NULL;
  pdCur  = NULL;

  kp     = NULL;
  kpLast = NULL;
  kpCur  = NULL;

  sp     = NULL;
  spLast = NULL;
  spCur  = NULL;

  totLast = NULL;
  totCur  = NULL;

}


//initialize the statsdata objects
void cStatsPackagerKeyHandlerCryptData::init(cStatsPackager* pStatsPackager)
{
  const char* func = "cStatsPackagerKeyHandlerCryptData::init";

  // save a pointer to the StatsPackager object
  ZKS_ASSERT(ZKS_STATSPACKAGER_MOD, pStatsPackager);

  try {

     //RouteCreator Stats
     newClients = (const StatsData<uint32_t>*)
         pStatsPackager->pStatsHarvestor->getStats(LMnewClients); 
     clients    = (const StatsData<uint32_t>*)
         pStatsPackager->pStatsHarvestor->getStats(LMclients);

     newClientsLast = new StatsData<uint32_t>(0);
     clientsLast    = new StatsData<uint32_t>(0);

     newClientsCur = new StatsData<uint32_t>(0);
     clientsCur    = new StatsData<uint32_t>(0);

     //DHNegotiator stats
     dh = new cStatsHarvestorPktData;
     dh->setStatsHarvestor(pStatsPackager->pStatsHarvestor); 
     dh->initRegisteredStats(DHPktRcvd,   DHPktSent,   DHPktDropped, 
                             DHBytesRcvd, DHBytesSent, DHBytesDropped);

     dhLast = new cStatsHarvestorPktData;         
     dhLast->initNewStats();

     dhCur  = new cStatsHarvestorPktData;          
     dhCur->initNewStats();

     //AipKernel stats
     ak = new cStatsHarvestorPktData;
     ak->setStatsHarvestor(pStatsPackager->pStatsHarvestor); 
     ak->initRegisteredStats(AKPktRcvd,   AKPktSent,   AKPktDropped, 
                             AKBytesRcvd, AKBytesSent, AKBytesDropped);

     akLast = new cStatsHarvestorPktData;
     akLast->initNewStats();

     akCur  = new cStatsHarvestorPktData;
     akCur->initNewStats();

     //PktDispatcher stats
     pd = new cStatsHarvestorPktData;
     pd->setStatsHarvestor(pStatsPackager->pStatsHarvestor); 
     pd->initRegisteredStats(PDPktRcvd,   PDPktSent,   PDPktDropped, 
                             PDBytesRcvd, PDBytesSent, PDBytesDropped);

     pdLast = new cStatsHarvestorPktData;
     pdLast->initNewStats();

     pdCur  = new cStatsHarvestorPktData;
     pdCur->initNewStats();

     //KeyProvider stats
     kp = new cStatsHarvestorPktData;
     kp->setStatsHarvestor(pStatsPackager->pStatsHarvestor); 
     kp->initRegisteredStats(KPPktRcvd,   KPPktSent,   KPPktDropped, 
                             KPBytesRcvd, KPBytesSent, KPBytesDropped);

     kpLast = new cStatsHarvestorPktData;
     kpLast->initNewStats();

     kpCur  = new cStatsHarvestorPktData;
     kpCur->initNewStats();

     //StatsPackager stats
     sp = new cStatsHarvestorPktData;
     sp->setStatsHarvestor(pStatsPackager->pStatsHarvestor); 
     sp->initRegisteredStats(SPPktRcvd,   SPPktSent,   SPPktDropped, 
                             SPBytesRcvd, SPBytesSent, SPBytesDropped);

     spLast = new cStatsHarvestorPktData;
     spLast->initNewStats();

     spCur  = new cStatsHarvestorPktData;
     spCur->initNewStats();

     //totals
     totLast = new cStatsHarvestorPktData;
     totLast->initNewStats();

     totCur  = new cStatsHarvestorPktData;
     totCur->initNewStats();

  }
  catch (...) {

     cleanup();  
     CZKS::Throw( ZKS_STATSPACKAGER_MOD, ZKS_STATSPACKAGER_INIT_ERR,
                  func, "failure");

  }
 
}

//cleanup dynamically allocated memory
void cStatsPackagerKeyHandlerCryptData::cleanup()
{
  delete newClientsLast;
  delete clientsLast;

  delete newClientsCur;
  delete clientsCur;

  delete dh;
  if (dhLast) {
     dhLast->deleteLocalStats();
  }
  delete dhLast;         
  if (dhCur) {
     dhCur->deleteLocalStats();
  }
  delete dhCur;          

  delete ak;
  if (akLast) {
     akLast->deleteLocalStats();
  }
  delete akLast;
  if (akCur) {
     akCur->deleteLocalStats();
  }
  delete akCur;

  delete pd;
  if (pdLast) {
     pdLast->deleteLocalStats();
  }
  delete pdLast;
  if (pdCur) {
     pdCur->deleteLocalStats();
  }
  delete pdCur;

  delete kp;
  if (kpLast) {
     kpLast->deleteLocalStats();
  }
  delete kpLast;
  if (kpCur) {
     kpCur->deleteLocalStats();
  }
  delete kpCur;

  delete sp;
  if (spLast) {
     spLast->deleteLocalStats();
  }
  delete spLast;
  if (spCur) {
     spCur->deleteLocalStats();
  }
  delete spCur;

  if (totLast) {
     totLast->deleteLocalStats();
  }
  delete totLast;
  if (totCur) {
     totCur->deleteLocalStats();
  }
  delete totCur;

}



