#define REV_ID "@(#) $Id: testStatsPackager.cpp,v 1.2 2001/04/18 17:37:57 alainc Exp $"
#include <zkrev.h>
//
// unit test program for cStatsPackager
//
// usage:      testStatsPackager 
//
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkSignal.h>
    
#include <iostream>
#include <strstream>
#include <fstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "aip-modules.h"

#include "cPrivateKeys.h"
#include "cConfigProvider.h"
#include "cStatsPackager.h"
#include "cStatsHarvestor.h"
#include "cStatsHarvestorPktData.h"
#include "StatsData.h"
#include "aip-stats-parms.h"

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

class cFakeEventHandlerTimeoutData: public czkEventHandlerData {

public:
  int first;
  cStatsHarvestor* pStatsHarvestor;
  czkCoreLoop* pCoreLoop;
 
   //RouteCreator stats
   StatsData<uint32_t>* newClients; 
   StatsData<uint32_t>* clients;
  
   //DHNegotiator stats
   cStatsHarvestorPktData* dh;  

   //AipKernel stats
   cStatsHarvestorPktData* ak;  

   //PktDispatcher stats
   cStatsHarvestorPktData* pd;  

   //KeyProvider stats
   cStatsHarvestorPktData* kp;  

   //StatsPackager stats
   cStatsHarvestorPktData* sp;  
  
  void init(cStatsHarvestor* p, czkCoreLoop* pc) {

    pStatsHarvestor = p;
    pCoreLoop = pc;

    dh = new cStatsHarvestorPktData;
    dh->initNewStats();
    dh->setStatsHarvestor( pStatsHarvestor);
    dh->registerStats(DHPktRcvd,   DHPktSent,   DHPktDropped, 
                      DHBytesRcvd, DHBytesSent, DHBytesDropped);

    ak = new cStatsHarvestorPktData;
    ak->initNewStats();
    ak->setStatsHarvestor( pStatsHarvestor);
    ak->registerStats(AKPktRcvd,   AKPktSent,   AKPktDropped, 
                      AKBytesRcvd, AKBytesSent, AKBytesDropped);

    pd = new cStatsHarvestorPktData;
    pd->initNewStats();
    pd->setStatsHarvestor( pStatsHarvestor);
    pd->registerStats(PDPktRcvd,   PDPktSent,   PDPktDropped, 
                      PDBytesRcvd, PDBytesSent, PDBytesDropped);

    kp = new cStatsHarvestorPktData;
    kp->initNewStats();
    kp->setStatsHarvestor( pStatsHarvestor);
    kp->registerStats(KPPktRcvd,   KPPktSent,   KPPktDropped, 
                      KPBytesRcvd, KPBytesSent, KPBytesDropped);

 
        newClients = new StatsData<uint32_t>(1);
        clients = new StatsData<uint32_t>(2);

        pStatsHarvestor->registerStats(LMnewClients,newClients);
        pStatsHarvestor->registerStats(LMclients,clients);

	first = 0;

  };

  void update() {
    if (first == 0) {
       sp = new cStatsHarvestorPktData;
       //    sp->initNewStats();
       sp->setStatsHarvestor( pStatsHarvestor);
       sp->initRegisteredStats(SPPktRcvd,   SPPktSent,   SPPktDropped, 
                               SPBytesRcvd, SPBytesSent, SPBytesDropped);
    }

    *newClients += 10;
    *clients += 20;

    *dh->pktRcvd      += 30;
    *dh->pktSent      += 40;
    *dh->pktDropped   += 50;
    *dh->bytesRcvd    += 60;
    *dh->bytesSent    += 70;
    *dh->bytesDropped += 80;

    *ak->pktRcvd      += 30;
    *ak->pktSent      += 40;
    *ak->pktDropped   += 50;
    *ak->bytesRcvd    += 60;
    *ak->bytesSent    += 70;
    *ak->bytesDropped += 80;

    *pd->pktRcvd      += 30;
    *pd->pktSent      += 40;
    *pd->pktDropped   += 50;
    *pd->bytesRcvd    += 60;
    *pd->bytesSent    += 70;
    *pd->bytesDropped += 80;

    *kp->pktRcvd      += 30;
    *kp->pktSent      += 40;
    *kp->pktDropped   += 50;
    *kp->bytesRcvd    += 60;
    *kp->bytesSent    += 70;
    *kp->bytesDropped += 80;

    *sp->pktRcvd      += 30;
    *sp->pktSent      += 40;
    *sp->pktDropped   += 50;
    *sp->bytesRcvd    += 60;
    *sp->bytesSent    += 70;
    *sp->bytesDropped += 80;

    if (first > 10) {
      pCoreLoop->abortEventLoop();
    }

    first++;

    std::cout << "ok" << std::endl;

  };
};

class cFakeEventHandler : public czkEventHandler {

public:

  void handleInput( czkSocket*, czkEventHandlerData*) {};
  void handleOutput( czkSocket*, czkEventHandlerData*) {};
  void handleException( czkSocket*, czkEventHandlerData*) {};

  int32_t handleTimeout( czkEventHandlerData* data) 
  {
    cFakeEventHandlerTimeoutData* mydata = (cFakeEventHandlerTimeoutData*)data;
    mydata->update();
    return 1;
  }
};

int32_t main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, " starting StatsPackager test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, " running in single thread mode");
        bool dothread = false;
#endif

        czkCoreLoop* coreloop = NULL;
   
        coreloop = new czkCoreLoop(); 
  
        std::ifstream istrm("./testStatsPackager.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_STATSPACKAGER_MOD,
                      1,
                      "testStatsPackager",
                      "Failed opening file: ./testStatsPackager.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

        // start ConfigProvider component
        cConfigProvider* pConfigProvider = new
	  cConfigProvider(m_cmdline, m_ini);
        mgr.add(ZKS_CONFIGPROVIDER_MOD, pConfigProvider);
        pConfigProvider->validateConfig();

 
        // start PrivateKeys component
        cPrivateKeys* pPrivateKeys = new cPrivateKeys();
        mgr.add(ZKS_PRIVATEKEYS_MOD, pPrivateKeys);
        pPrivateKeys->validateConfig();


        // start KeyProvider component
        cKeyProvider* pKeyProvider = new cKeyProvider();
        mgr.add(ZKS_KEYPROVIDER_MOD, pKeyProvider);
        pKeyProvider->validateConfig();


        // start StatsHarvestor component
        cStatsHarvestor* pStatsHarvestor = new cStatsHarvestor();
        mgr.add(ZKS_STATSHARVESTOR_MOD, pStatsHarvestor);
        pStatsHarvestor->validateConfig();


        cFakeEventHandlerTimeoutData data;
        data.init(pStatsHarvestor, coreloop);

        // start StatsPackager component
        cStatsPackager* pStatsPackager = new cStatsPackager();
        mgr.add(ZKS_STATSPACKAGER_MOD, pStatsPackager);
        if (!pStatsPackager->validateConfig()) {
	   ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_STATSPACKAGER_MOD, "validateConfig failed");
           abort();
	}


        mgr.initAllComponents(coreloop);

        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, false);
        mgr.startComponent(ZKS_PRIVATEKEYS_MOD, false);
        mgr.startComponent(ZKS_KEYPROVIDER_MOD, false);
        mgr.startComponent(ZKS_STATSHARVESTOR_MOD, false);
        mgr.startComponent(ZKS_STATSPACKAGER_MOD, false);
   
        cFakeEventHandler eh;

        czkIntervalTime ti;
        coreloop->scheduleTimer( &eh, &data, ti.getSeconds(1));

        czkSockDgram socket;
        if (socket.open() != ZKPR_SUCCESS) {
           CZKS::Throw(ZKS_STATSPACKAGER_MOD, ZKS_STATSPACKAGER_SOCKET_ERR,
                       "testStatsPackager","opening socket");
        }
        czkNetAddr netaddr;
        ZKPRNetAddr addr;
        memset(&addr, 0 , sizeof(addr));
        addr.inet.family = ZKPR_AF_INET;
        netaddr.initNetAddr( ZKPR_IpAddrLoopback, 51113, &addr);
        char buf[1];  
        socket.sendto( buf, 1, 0, &addr, ZKPR_INTERVAL_NO_TIMEOUT); 

        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;
        }
        else {


          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;

          // wait for all threads to complete

          mgr.waitForCompletion();
        }

      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_STATSPACKAGER_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow( ZKS_STATSPACKAGER_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      std::cout << "not ok" << std::endl;
      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();
}


