#define REV_ID "@(#) $Id: cAipKernel_stub.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include "cAipKernel.h"
#include "cAipKernelStub.h"
#include "cAipKernelPacketList.h"
#include "czkSockDgram.h"
#include "czkNetAddr.h"

#include "cStatsHarvestorPktData.h"

#include "cLinkMonitorInfo.h"
#include "global.h"

czkSockNativeTCP* stream;
bool streamUp = false;

  // constructor
cAipKernel::cAipKernel(bool) {}

  // destructor
cAipKernel::~cAipKernel() {}

  // validate configuration
bool cAipKernel::validateConfig()
{ return true; }

  // initialize private data members
bool cAipKernel::initializePrivateData(czkCoreLoop* cl) 
{
  const char* func="cAipKernel::initializePrivateData"; 
  coreloop = cl; 

        cStatsHarvestor* pStatsHarvestor = 
            (cStatsHarvestor*)pComponentManager->get(ZKS_STATSHARVESTOR_MOD);
 //register AipKernel stats for now
        cStatsHarvestorPktData* akStats = new cStatsHarvestorPktData;
        akStats->initNewStats();
        akStats->setStatsHarvestor( pStatsHarvestor);
        akStats->registerStats(AKPktRcvd,   AKPktSent,   AKPktDropped, 
                               AKBytesRcvd, AKBytesSent, AKBytesDropped);

	ZKPRNetAddr sockAddr;
        int rc;
        czkNetAddr netAddr;

        sockAddr = getMyAipAddr(); 
	sockAddr.inet.port = htons(5555);

	if( frSocket.open() != ZKPR_SUCCESS )
	{
            CZKS::Throw(ZKS_AIPKERNEL_MOD, ZKS_AIPKERNEL_SOCKET_ERR, 
                        func, "Couldn't create freedom socket");
	}

	if( frSocket.bind(sockAddr) != ZKPR_SUCCESS )
	{
            CZKS::Throw(ZKS_PACKETDISPATCHER_MOD, ZKS_PKTDISPATCHER_SOCKET_ERR, 
                        func, "Couldn't bind UDP socket rc=%d", rc);
 	}

   return true; 
}
void cAipKernel::cleanupPrivateData() {} 

  // startup
void cAipKernel::startup() 
{

}
void cAipKernel::shutdown() {}

void cAipKernel::addLink(LinkId linkId, cLinkManagerInfo* info) 
{
  if (!streamUp) return;

  ZKPRIntervalTime timeout;
  czkIntervalTime interval;
  timeout = interval.getSeconds(0);

  LINKMONXFER xfer;
  xfer.action = monAddLink;
  xfer.entity = info->getEntity();
  ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, "kernel addlink "," entity %s dhaddr %x", 
                 xfer.entity.name, xfer.dhaddr); 
  xfer.linkId = linkId;
  xfer.dhaddr = info->getDHAddr();
  stream->send( (char*)&xfer, sizeof(xfer), 0, timeout);
}

void cAipKernel::updateLink(LinkId linkId, cLinkManagerInfo* info)
{
  if (!streamUp) return;

  ZKPRIntervalTime timeout;
  czkIntervalTime interval;
  timeout = interval.getSeconds(0);

  LINKMONXFER xfer;
  xfer.action = monUpdateLink;
  xfer.entity = info->getEntity();
  xfer.linkId = linkId;
  xfer.dhaddr = info->getDHAddr();
  stream->send( (char*)&xfer, sizeof(xfer), 0, timeout);
}

void cAipKernel::removeLink(LinkId linkId)
{
  if (!streamUp) return;

  ZKPRIntervalTime timeout;
  czkIntervalTime interval;
  timeout = interval.getSeconds(0);

  LINKMONXFER xfer;
  xfer.action = monRemoveLink;
  memset(&xfer.entity, 0, sizeof(xfer.entity));
  xfer.linkId = linkId;
  stream->send( (char*)&xfer, sizeof(xfer), 0, timeout);
}

void cAipKernel::addRoute(RouteId routeId, cRouteCreatorInfo* ) 
{
  LINKMONXFER xfer;
  xfer.action = monAddRoute;
  xfer.linkId = routeId;
  stream->send( (char*)&xfer, sizeof(xfer), 0, ZKPR_INTERVAL_NO_TIMEOUT);
}

void cAipKernel::updateRoute(RouteId routeId
, cRouteCreatorInfo* ,cRouteCreatorInfo* )
{
  LINKMONXFER xfer;
  xfer.action = monUpdateRoute;
  xfer.linkId = routeId;
  stream->send( (char*)&xfer, sizeof(xfer), 0, ZKPR_INTERVAL_NO_TIMEOUT);
}

void cAipKernel::removeRoute(RouteId routeId, cRouteCreatorInfo*  )
{
  LINKMONXFER xfer;
  xfer.action = monRemoveRoute;
  xfer.linkId = routeId;
  stream->send( (char*)&xfer, sizeof(xfer), 0, ZKPR_INTERVAL_NO_TIMEOUT);
}

void cAipKernel::createSocket(czkSocket ** socket)
{
  czkSockDgram* sock = new czkSockDgram;

  sock->open();

  ZKPRNetAddr addr = getMyAipAddr();
  sock->bind(addr);

  *socket = (czkSocket*)sock;
}
void cAipKernel::destroySocket(czkSocket** socket)
{
  (*socket)->close();
  delete (*socket);
}

cAipKernelPacketList::cAipKernelPacketList()
{
    currentPkt = NULL;
}

cAipKernelPacketList::~cAipKernelPacketList()
{
    destroyList();
}

void cAipKernelPacketList::recv(czkSocket & sock)
{
    const char * func = "cAipKernelPacketList::recv";
    const char * msg = NULL;

    CZKS::Exception e;

    try
    {
        msg = "Freeing the current list";

        destroyList();
        
        msg = "Receiving from netlink socket";

        ZKPRNetAddr addr;
        
        int32_t nb;
        if((nb = sock.recvfrom(buffer, AIPKERNEL_BUFFER_SIZE, 0, 
                               &addr, ZKPR_INTERVAL_NO_WAIT)) <= 0)
        {
            CZKS::Throw(ZKS_AIPKERNEL_MOD, ZKS_AIPKERNEL_SOCKET_ERR, 
                        func, "Couldn't recv on netlink socket rc=%d", nb);
        }

        msg = "Extracting packets from netlink structure";
 
        cAipKernelPacket * pkt = &firstPkt;

        czkNetAddr netaddr;
        LinkId* linkId = (LinkId*)&buffer[nb-sizeof(LinkId)];
        pkt->linkId = *linkId;

        FrPktType       type;
        if (frPktGetType(e.zkError(), buffer, nb, &type) == ZKS_FAILURE) {
           ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " cannot get pkt type ");
        }

        ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK, "FunctionalAip", 
                       " recvd data %d linkid %d type %d addr %x", 
                       nb, pkt->linkId, type, addr.inet.ip);

        pkt->pkt = (char *)buffer;
        pkt->pktSize = nb-sizeof(LinkId);
        pkt->srcAddr = addr;
        pkt->nextPkt = NULL;
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void cAipKernelPacketList::getNextPacket(LinkId * linkId, ZKPRNetAddr* addr,
                                         char ** pkt, 
                                         int32_t * pktSize)
{
    const char * func = "cAipKernelPacketList::getNextPacket";
    const char * msg = NULL;

    try
    {
        msg = "Retrieving next packet info";

        if (currentPkt == NULL)
        {
            if (firstPkt.pktSize != 0)
            {
                // This is the first call
                currentPkt = &firstPkt;
                *pkt = currentPkt->pkt;
                *pktSize = currentPkt->pktSize;
                *addr = currentPkt->srcAddr;
                *linkId = currentPkt->linkId;
            }
            else
            {
                // no available packet
                *pkt = NULL;
                *pktSize = 0;
                *linkId = 0;
            }
        }
        else
        {
            if (currentPkt->nextPkt == NULL)
            {
                // there is no more available packets
                *pkt = NULL;
                *pktSize = 0;
                *linkId = 0;
            }
            else
            {
                // retrieve next packet info
                currentPkt = currentPkt->nextPkt;
                *pkt = currentPkt->pkt;
                *pktSize = currentPkt->pktSize;
                *addr = currentPkt->srcAddr;
                *linkId = currentPkt->linkId;
            }
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void cAipKernelPacketList::destroyList()
{
    // delete dynamically allocated packets from packet list (first pkt is static)
    cAipKernelPacket * current = NULL;
    cAipKernelPacket * pkt = firstPkt.nextPkt;
    while(pkt != NULL)
    {
        current = pkt;
        pkt = current->nextPkt;
        delete current;
    }
    
    currentPkt = NULL;
    firstPkt.pkt = NULL;
    firstPkt.pktSize = 0;
    firstPkt.linkId = 0;
    firstPkt.nextPkt = NULL;
}

// connect the socket to a file (for Unix Domain sockets)
int32_t cAipKernelStub::connect(int32_t fd, char* path)
{
     int32_t rc;
 
     struct sockaddr_un  addrun;

     memset(&addrun,0,sizeof(addrun));

     addrun.sun_family = AF_LOCAL;
        
     strncpy( addrun.sun_path, path, sizeof(addrun.sun_path) - 1);

     int32_t status = ::connect( fd, 
                                 (struct sockaddr*)&addrun, sizeof (addrun));
     if (status) {
        rc = ZKPR_FAILURE;
     }
     else {
        rc = ZKPR_SUCCESS;      
     }
     return rc;
}

void cAipKernelStub::initSocket(czkCoreLoop* cl, cAipKernel* p)
{
  stream = new czkSockNativeTCP;

  CZKS::Exception e;

  pAipKernel = p;
  coreloop = cl;

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  
  
  //create UDS socket     
  int32_t fd;
  if (!zkSocketNew( e.zkError(), AF_LOCAL, SOCK_STREAM, IPPROTO_IP, &zksock)) {
     cout << " error creating zksocket " << endl;
     abort();
  } 
  else if (!zkSocketGetFD( e.zkError(), zksock, &fd)) {
     cout << " error getting fd zksocket " << endl;
     abort();
  }

  stream->open(fd); 

  ZKPRSocketOptionData opt;
  opt.option = ZKPR_SockOpt_Reuseaddr;
  opt.value.reuse_addr = ZKPR_TRUE;
  stream->setOption( &opt);
  opt.option = ZKPR_SockOpt_Nonblocking;
  opt.value.non_blocking = 1;
  stream->setOption( &opt);

  coreloop->registerHandler( this, NULL, (czkSocket*)stream, 
                             ZKPR_POLL_READ | ZKPR_POLL_EXCEPT);

  if (connect( fd, "/turf/frankf/UDSlinkmonitor") != ZKPR_SUCCESS) {
    cout << " uds connect failed " << endl;
    retryConnect((czkSocket*)stream);
  }
  else {
    streamUp = true;
  }

} 

void cAipKernelStub::handleInput( czkSocket* sock, czkEventHandlerData*)
{
  char buf[BUFSIZ];

  ZKPRIntervalTime timeout;
  czkIntervalTime interval;
  timeout = interval.getSeconds(0);

  int nb = sock->recv( buf, BUFSIZ, 0, timeout); 
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, "cAipKernelStub"," handleInput read %d ", nb);
}

void cAipKernelStub::retryConnect()
{
  retryConnect( stream);
}

void cAipKernelStub::retryConnect( czkSocket* sock)
{
  CZKS::Exception e;

  cout << " in  cAipKernelStub::retryConnect" << endl;

  streamUp = false;

  coreloop->removeHandler(sock);

  sock->close();
  if (!zkSocketFree( e.zkError(), &zksock)) {
     cout << " error creating zksocket " << endl;
     cout << flush;
     abort();
  } 

  czkIntervalTime interval;
  ZKPRIntervalTime timeout = interval.getSeconds(10);
  coreloop->scheduleTimer( this, NULL, timeout);

  delete sock;
}

void cAipKernelStub::handleOutput( czkSocket*, czkEventHandlerData*)
{
}

void cAipKernelStub::handleException( czkSocket* sock, 
                                        czkEventHandlerData*)
{
  CZKS::Exception e;
  
  retryConnect(sock);
}

int32_t cAipKernelStub::handleTimeout( czkEventHandlerData*)
{
  cout << " in  cAipKernelStub::handleTimeout" << endl;
  initSocket(coreloop, pAipKernel);
  return 0;
}

void cAipKernel::handlePull( const char*, StatsDataBase*)
{
} 
    
  
czkSockDgram * cAipKernel::getFrSocket() 
{ 
    return &frSocket; 
}
