#ifndef _cCommandParser

#define _cCommandParser

#include <string>
#include <list>

#define commandNetInfoList  "NetInfoList"
#define commandDHNegInit    "DHNegInit"
#define commandDHNegRespond "DHNegRespond"
#define commandRouteCreate  "RouteCreate"
#define commandRouteExtend  "RouteExtend"
#define commandRouteExtend  "RouteExtend"
#define commandRouteAck     "RouteAck"
#define commandRouteDestroy "RouteDestroy"
#define commandKeepAlive    "KeepAlive"
#define commandStop         "Stop"

#define qualifierEntityList     "EntityList"
#define qualifierFunctionalAip  "FunctionalAip"
#define qualifierEntityName     "EntityName"
#define qualifierClientEntityName "ClientEntityName"
#define qualifierClient         "Client"
#define qualifierDelay          "Delay"
#define qualifierAction         "Action"
#define qualifierRepeatInterval "RepeatInterval"
#define qualifierMaxWait        "MaxWait"
#define qualifierRepeat         "Repeat"
#define qualifierID             "ID"
#define qualifierNHops          "NHops"
#define qualifierHop1           "Hop1"
#define qualifierHop2           "Hop2"
#define qualifierHop3           "Hop3"
#define qualifierFixedAci       "FixedAci"
#define qualifierNymName        "NymName"
#define qualifierValidNymSig    "ValidNymSig"
#define qualifierAnonRand       "AnonRand"
#define qualifierAuthRoute      "AuthRoute"
#define qualifierRouteTTL       "RouteTTL"
#define qualifierVersion        "Version"
#define qualifierExitAci        "ExitAci"
#define qualifierNextAci        "NextAci"
#define qualifierValidateRoute  "ValidateRoute"
#define qualifierErrorCode      "ErrorCode"
#define qualifierMessage        "Message"
#define qualifierBad            "Bad"

#define valPASS     "PASS"
#define valFAIL     "FAIL"
#define valTIMEOUT  "TIMEOUT"

#define valTRUE     "TRUE"
#define valFALSE    "FALSE"

#define valGOOD     "GOOD"
#define valBAD      "BAD"

#define valACK      "ACK"
#define valNOACK    "NOACK"
#define valBADACK   "BADACK"

#define valAUTH     "AUTH"
#define valANON     "ANON"
#define valANONRAND "ANONRAND"

#define MaxQualifier 9
 
class cParser {

 private:
  
 public:

  void split 
   (string & text, string & separators, list<string> & words)
  {
    int n = text.length();
    int start, stop;

    start = text.find_first_not_of(separators);
    while ((start >= 0) && (start < n)) 
    {
      stop = text.find_first_of(separators, start);
      if ((stop < 0) || (stop > n)) stop = n;
      words.push_back(text.substr(start, stop - start));
      start = text.find_first_not_of(separators, stop+1);
    }
  }

};

typedef list<string> StringList;

class cQualifier {

 private:

  string qualifier;
  StringList vals;

 public:

  cQualifier( string in)
  {
    cParser p;
    string sep = "=";
    StringList temp;
    p.split( in, sep, temp);
    StringList::iterator i = temp.begin();
    if (i != temp.end()) {
      qualifier = *i;
    }
    i++;
    if (i != temp.end()) {
      sep = ",";
      p.split( *i, sep, vals);
    }  

  }

  string& getQualifier() { return qualifier; }

  StringList& getVals() { return vals; } 
        
};      

typedef list<cQualifier> QualifierList;

class cCommandParser {

 private:

  uint32_t commandTime;  
  string command;
  QualifierList qualifiers;

 public:

  cCommandParser(string commandString)
  {
    string sep = " ";
    StringList words;

    cParser p;
    p.split(commandString, sep, words);

    StringList::iterator i = words.begin();
    if (i != words.end()) {
      commandTime = atoi((*i).c_str());
      words.erase(i);
    }

    i = words.begin();
    if (i != words.end()) {
      command = *i;
      words.erase(i);
    }

    i = words.begin();
    while (i != words.end()) 
    {
      cQualifier* q = new cQualifier(*i);
      qualifiers.push_back( *q);
      i++;
    }
  
  }

  uint32_t getTime() { return commandTime; }
 
  string& getCommand() { return command; }

  QualifierList& getQualifiers() { return qualifiers; }  

};





#endif
