#define REV_ID "@(#) $Id: cDHNego.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include "cDHNego.h"
#include "global.h"

void cDHNego::init(czkCoreLoop* cl, cLinkMonitor* pl)
{
  CZKS::Exception e;

  coreloop = cl;
  pLinkMonitor = pl;

  char sendPriv[] = "/turf/frankf/sig/m.priv";
  if (!frCryptSigNewFromFile(e.zkError(), &peerPrivSig, sendPriv))
  { 
    CZKS::Throw(e);
  }
  
  char sendPub[] = "/turf/frankf/sig/m.pub";
  if (!frCryptSigNewFromFile(e.zkError(), &peerPubSig, sendPub))
  { 
    CZKS::Throw(e);
  }
  
  if (!frCryptSigNewFromFile(e.zkError(), &myPubSig,   sendPub))
  { 
    CZKS::Throw(e);
  }

  successInit = false;
  successResp = false;

  doInit = false;
  doResp = false;

}

void cDHNego::initSocket(ZKPRNetAddr peerAddr, FrEntIdExp entity, 
                         DHNegAction action)
{
  CZKS::Exception e;

  if (entity.type == FRENT_TYPE_AIP) {

     // Initialize simulated peer socket
     assert(peerListenSock.open() == ZKPR_SUCCESS);
     assert(peerListenSock.bind(peerAddr) == ZKPR_SUCCESS);
     assert(peerListenSock.listen(1) == ZKPR_SUCCESS);

     cDHNegoData* d = new cDHNegoData;
     d->sendEntity = entity;
     d->action = action;
     d->type = dhResp;
     coreloop->registerHandler( this, d, (czkSocket*)&peerListenSock, 
                                ZKPR_POLL_READ); 
  }
 
}

void cDHNego::closeSocket()
{
  coreloop->removeHandler( (czkSocket*)&peerListenSock); 
  peerListenSock.close();
}
   
//
// init DH exchange
// 
// 
void cDHNego::initDH(FrPktDifHel * dhDesc, bool sigGenerate, 
                         czkSocket & sock, uint8_t lekv)
{
    const char * func = "initDH";
    const char * msg = NULL;

    try
    {
        msg = "Initializing D-H descriptor";
        
        CZKS::Exception e;

        // Allocate D-H descriptor
        if (frPktDifHelNew(e.zkError(), dhDesc) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }

        // Indicate that remote entity signature must be verified
        if (frPktDifHelSetSigCheck(e.zkError(), *dhDesc, 1) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }

        // Indicate that we must sign our D-H message
        if (sigGenerate)
        {
            if (frPktDifHelSetSigGenerate(e.zkError(), *dhDesc, 1) == ZKS_FAILURE)
            {
                CZKS::Throw(e);
            }
        }
        else
        {
            if (frPktDifHelSetSigGenerate(e.zkError(), *dhDesc, 0) == ZKS_FAILURE)
            {
                CZKS::Throw(e);
            }
        }

        // Set socket descriptor
        int32_t fd = sock.getNativeFD();
        if (frPktDifHelSetFd(e.zkError(), *dhDesc, fd) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }

        // Set my signature key
        if (frPktDifHelSetMySigKey(e.zkError(), *dhDesc, peerPrivSig) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        
        // Set the Freedom packet port number
        if (frPktDifHelSetLocalPort(e.zkError(), *dhDesc, 10000) == 
            ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        
        // set encryption algorithm
        if (frPktDifHelSetSymAlg(e.zkError(), *dhDesc, FRCRYPT_SYMALG_BLOWFISH) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
       }
        
        // set lekv
        if (frPktDifHelSetLekv(e.zkError(), *dhDesc, lekv) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, msg);
    }
}

/*
bool cDHNego::validateExchange(cUnitTestResultHandler & rh, 
                                   FrEntIdExp * entity,
                      DHNegoResult dhResult, SymKey encrKey, SymKey decrKey, 
                      uint8_t lekv)
{
    CZKS::Exception e;
    
    // Verify that sym keys are functional
    uint16_t testBufSize = 256;
    uint8_t plainText[testBufSize];
    uint8_t testBuf[testBufSize];
    bool result = false;
    bool keysFunctional = false;
    char ivec[FRCRYPT_IVEC_SIZE];
            
    if (dhResult == DHNegoPass)
    {
        for (uint16_t i = 0; i < testBufSize; i++)
        {
            plainText[i] = (uint8_t)i;
        }
        memcpy(testBuf, plainText, testBufSize);
        memset(ivec, 0, sizeof(ivec));
        assert(frCryptSymEncrypt(e.zkError(), encrKey, ivec, testBuf, 
                                 testBufSize) != ZKS_FAILURE);
        memset(ivec, 0, sizeof(ivec));
        assert(frCryptSymDecrypt(e.zkError(), rh.exchangeDecrKey, ivec,
                                 testBuf, testBufSize) != ZKS_FAILURE);
        if (memcmp(testBuf, plainText, testBufSize) == 0)
        {
            // encryption-decryption successful
            memset(ivec, 0, sizeof(ivec));
            assert(frCryptSymEncrypt(e.zkError(), rh.exchangeEncrKey, ivec, 
                                             testBuf, testBufSize) != ZKS_FAILURE);
            memset(ivec, 0, sizeof(ivec));
            assert(frCryptSymDecrypt(e.zkError(), decrKey, ivec,
                                     testBuf, testBufSize) != ZKS_FAILURE);
            if (memcmp(testBuf, plainText, testBufSize) == 0)
            {
                // encryption-decryption successful
                keysFunctional = true;
            }
        }
        if (((entity == NULL) || (memcmp(entity, &(rh.exchangeEntity), sizeof(rh.exchangeEntity)) == 0)) &&
            (rh.exchangeLekv == lekv) &&
            (rh.exchangeResult == DHNegoPass) &&
            (keysFunctional))
        {
            result = true;
        }
    }
    else
    {
        if (((entity == NULL) || 
             (memcmp(entity, &(rh.exchangeEntity), sizeof(rh.exchangeEntity)) == 0)) &&
            (rh.exchangeResult == DHNegoFail))
        {
            result = true;
        }
    }
    
    return result;
}
*/

void cDHNego::sendDH(FrPktDifHel& peerDH)
{
  CZKS::Exception e;

  int32_t done;

  // send second DH message
  if (frPktDifHelSend(e.zkError(), peerDH, &done) != ZKS_FAILURE) {
     CZKS::Throw(e);
  }
  assert(done == 0);
  //while (done != 1)
  //{
     if (frPktDifHelSend(e.zkError(), peerDH, &done) != ZKS_FAILURE) {
       //	CZKS::Throw(e);
     }
     //}

  //ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "sent dh message"); 
}

int32_t cDHNego::recvDH(FrPktDifHel& peerDH)
{
  CZKS::Exception e;

  int32_t done;

  //ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "wait for dh message"); 

  if (frPktDifHelReceive(e.zkError(), peerDH, &done) != ZKS_FAILURE) {
     CZKS::Throw(e);
  }
  assert(done == 0);
  //while (done != 1)
  //{
     if (frPktDifHelReceive(e.zkError(), peerDH, &done) != ZKS_FAILURE) {
       //CZKS::Throw(e);
     }
     
  return done;
 
     //}

  //ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "got dh message"); 
}

void cDHNego::handleInput( czkSocket* sock, czkEventHandlerData* data)
{
  CZKS::Exception e;

  ZKPRNetAddr addr;
  char testBuf[BUFSIZ];

  cDHNegoData* d = (cDHNegoData*)data;

  if (sock == &peerListenSock) {

    // Await connection
    assert(peerListenSock.accept(&peerAcceptSock, &addr, 
                                 PR_INTERVAL_NO_TIMEOUT) == ZKPR_SUCCESS);
    d->state = waitEntity;
        
    coreloop->registerHandler( this, d, peerAcceptSock, 
                               ZKPR_POLL_READ); 
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "got connection sock %x", peerAcceptSock);

  }
  else {

    if (d->state == waitEntity) {
     
       FrEntIdExp functionalAip = getMyAipEntity();

       FrEntIdExp* myEntityId = (FrEntIdExp*)testBuf;
       //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "wait for entity sock %x", sock);
       int32_t nb = sock->recv((char *)(testBuf), 
                               BUFSIZ, 0, 
                               PR_INTERVAL_NO_WAIT);
       if (nb != sizeof(FrEntIdExp) + 1)
       {
          ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, 
                         "Test failed - received entity ID of invalid size %d",
                         nb);
       }
       else if (myEntityId->type != FRENT_TYPE_AIP)
       {
          ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, 
                         "Test failed - received entity ID of wrong type");
       }
       else if (doInit && 
                memcmp(&d->sendEntity, myEntityId, sizeof(FrEntIdExp)) > 0)
       {
          ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, 
                         "cancelling because peer %s has priority over %s", 
                         d->sendEntity.name, myEntityId->name);
          coreloop->removeHandler( sock);
          sock->close();
       } 
       else
       {
         
	  ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "got aip entity");

          // initialize DH exchange            
          initDH(&d->peerDH, true, *sock, testBuf[sizeof(FrEntIdExp)]);

          if (d->action == Pass) {
             sendDH(d->peerDH);
          }
          d->state = waitResponse;
          d->recvEntity = *myEntityId;
          d->lekv = (uint8_t)testBuf[sizeof(FrEntIdExp)];

       }
     }
     else {                  
       // receive first DH message
//  	if (sock == &peerConnectSock) {

//            if (d->action != Pass) {
//     	     goto dhabort;
//  	  }

//  	}

       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "recv dh sock %x", sock);
       if (!recvDH(d->peerDH)) return;
        
	if (sock == &peerConnectSock) {

          if (d->action == Pass) {
   	     sendDH(d->peerDH);
	  }

	}

       FrCryptSym peerSymKey; 
       assert(frCryptSymNew(e.zkError(), &peerSymKey) != ZKS_FAILURE);
       if (frPktDifHelGetKey(e.zkError(), d->peerDH, myPubSig, 
			     peerSymKey) == ZKS_FAILURE) {
	 ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, 
                        "frPktDifHelGetKey failed for entity %s", 
                        d->sendEntity.name);
       }
       else {
         FrCryptSym peerSymKeyDup; 
         if (frCryptSymCopy(e.zkError(), peerSymKey, 
                            &peerSymKeyDup) == ZKS_FAILURE)
  
         {
            CZKS::Throw(e);
         }

         //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," looking for entity %s ",d->sendEntity.name);
         cLinkMonitorInfo* info = pLinkMonitor->findLink( d->sendEntity);
         if (info) {
           ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                          "update sym key linkid %d lekv %d ",
                          info->linkId, d->lekv);

	   info->lekv = d->lekv;
           if (d->sendEntity.type != FRENT_TYPE_AIP) {
	     info->lekv = 1-info->lekv;
	   }
           info->encrKey[info->lekv] = peerSymKey;
           info->decrKey[info->lekv] = peerSymKeyDup;
           info->state   = monLinkUp;
         }
     
         if (d->type == dhInit) {
            successInit = true;
	 }
         else {
            successResp = true;
	 }
       }
       coreloop->removeHandler( sock);
       sock->close();

       //assert(frCryptSymFree(e.zkError(), &peerSymKey) != ZKS_FAILURE);
       assert(frPktDifHelFree(e.zkError(), &d->peerDH));
       //delete d;
      
    }
            
  }
}


void cDHNego::handleException( czkSocket* sock, czkEventHandlerData*)
{
  CZKS::Exception e;

  ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, " exception socket err %d %x", PR_GetError(), sock);

  if (sock == &peerConnectSock) { 
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, " connect failed");
  }
  else if (sock == &peerListenSock) { 
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, " listen failed");
  }
  else { 
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, " dh sock failed");
  }
  coreloop->removeHandler( sock); 
  sock->close();
  /*
  if (sock == &peerAcceptSock) { 
    assert(peerListenSock.open() == ZKPR_SUCCESS);
    assert(peerListenSock.bind(peerAddr) == ZKPR_SUCCESS);
    assert(peerListenSock.listen(1) == ZKPR_SUCCESS);
  }
  
  coreloop->registerHandler( this, NULL, (czkSocket*)&peerListenSock, 
                             ZKPR_POLL_READ); 
  */
}

void cDHNego::handleOutput( czkSocket* sock, czkEventHandlerData* data)
{
  const char* func = "cDHNego::handleOutput";

  CZKS::Exception e;

  typedef struct DHidstruct DHid;
#pragma pack(1)
  struct DHidstruct {
    FrEntIdExp entity;
    uint8_t lekv;
  };
#pragma pack()
  DHid dhid;

  if (data) {  

     cDHNegoData* d = (cDHNegoData*)data;
  
     if (d->state == tryConnect) {

       //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "try connect sock %x", sock);
        int32_t rc = peerConnectSock.connect(d->addr, PR_INTERVAL_NO_TIMEOUT);
        if (rc == ZKPR_SUCCESS) {
           if (d->sendEntity.type == FRENT_TYPE_AIP) {
              d->state = sendEntity;
	   }
           else {
             d->state = waitResponse;
             initDH(&d->peerDH, false, peerConnectSock, 0);
	   }  
        }
        else if (rc != ZKPR_IN_PROGRESS_ERROR) {
           ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  TESTFUNCTIONALAIP," error doing DH-connect %d",rc);
           coreloop->removeHandler( sock);
           sock->close();
        }

     } 
     if (d->state == sendEntity) {

        ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "send entity sock %x entity %s ", 
                       sock, d->sendEntity.name);
        dhid.entity = d->sendEntity;
        dhid.lekv = 0;
        assert(peerConnectSock.send((char*)&dhid, sizeof(dhid), 0,
                                PR_INTERVAL_NO_TIMEOUT) == sizeof(dhid));

        // initialize DH exchange
        initDH(&d->peerDH, true, peerConnectSock, 0);

        d->state = waitResponse;

     }
     if (d->state == waitResponse) {

       coreloop->removeHandler( (czkSocket*)&peerConnectSock);
       coreloop->registerHandler( this, d, (czkSocket*)&peerConnectSock, 
                                  ZKPR_POLL_READ); 

     }
  }

}

void cDHNego::connect(ZKPRNetAddr addr, FrEntIdExp entity, 
                      DHNegAction action)
{
   const char* func = "cDHNego::connect";

   successInit = false;

  //init the connect socket
  assert(peerConnectSock.open() == ZKPR_SUCCESS);

   // Peer connects 
   cDHNegoData* d = new cDHNegoData;
   d->sendEntity = entity;
   d->action = action;
   d->lekv = 0;
   d->type = dhInit;

   //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, "try connect sock %x", &peerConnectSock);
   int32_t rc = peerConnectSock.connect(addr, PR_INTERVAL_NO_TIMEOUT);
   if (rc == ZKPR_SUCCESS) {
     if (entity.type == FRENT_TYPE_AIP) {
        d->state = sendEntity;
     }
     else {
        d->state = waitResponse;
        initDH(&d->peerDH, false, peerConnectSock, 0);
     }
   }
   else if (rc == ZKPR_IN_PROGRESS_ERROR) {
     d->state = tryConnect;
     d->addr = addr;
   }
   else {
     ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK,  TESTFUNCTIONALAIP," error doing DH-connect %d",rc);
     peerConnectSock.close();
     return;
   }

   coreloop->registerHandler( this, d, (czkSocket*)&peerConnectSock, 
                              ZKPR_POLL_WRITE); 
}
