#define REV_ID "@(#) $Id: cKeyProvider_stub.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include "cKeyProvider.h"
#include "cStatsHarvestorPktData.h"

#include "global.h"
#include <strstream>

//constructor
cKeyProvider::cKeyProvider() {}
  
//destructor
cKeyProvider::~cKeyProvider() 
{
}

int32_t cKeyProvider::getPruneTime()
{
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        KP_CACHE_PRUNE_INTERVAL,  
                                        KP_CACHE_PRUNE_INTERVAL_DEF);
}

int32_t cKeyProvider::getKeyRequestTimeoutVal()
{
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        KP_KEY_REQUEST_TIMEOUT, 
                                        KP_KEY_REQUEST_TIMEOUT_DEF);

}

int32_t cKeyProvider::getMaxRetry()
{
    return pConfigProvider->getOpt_int( UAIP_SECTION_NAME, 
                                        KP_KEY_REQUEST_MAXRETRY, 
                                        KP_KEY_REQUEST_MAXRETRY_DEF);

}

// validate configuration items
bool cKeyProvider::validateConfig() 
{ 

  return true; 
}

// initialize private data elements
bool  cKeyProvider::initializePrivateData(czkCoreLoop* ) 
{ 
        cStatsHarvestor* pStatsHarvestor = 
            (cStatsHarvestor*)pComponentManager->get(ZKS_STATSHARVESTOR_MOD);
       
         cStatsHarvestorPktData* kpStats = new cStatsHarvestorPktData;
        kpStats->initNewStats();
        kpStats->setStatsHarvestor( pStatsHarvestor);
        kpStats->registerStats(KPPktRcvd,   KPPktSent,   KPPktDropped, 
                               KPBytesRcvd, KPBytesSent, KPBytesDropped);

  return true;
};
void cKeyProvider::cleanupPrivateData() {} 

// startup routine
void cKeyProvider::startup()
{
}

void cKeyProvider::shutdown()
{
}

// try to connect to the KQD
void cKeyProvider::connectKQD()
{
}

void cKeyProvider::sendQry(FrKeyQryQueryExp& )
{
}

// make a key request 
void cKeyProvider::makeKeyRequest( char* entity, 
                                   FrEntIdType entity_type, 
                                   FrPubKeyType key_type, 
                                   uint32_t key_version, 
			           cKeyProviderEventHandler* eventhandler,
                                   cKeyProviderCacheEntryData& entryData)
{
   FrKeyQryQueryExp pqry; /* key query */

   // build the key query 

   pqry.keyVersion = key_version;
   pqry.keyType = key_type;
   pqry.keyOwner.type = entity_type;
   memset(pqry.keyOwner.name, 0, sizeof( pqry.keyOwner.name));
   size_t n = strnlen(entity,sizeof(pqry.keyOwner.name)-1)+1;
   if (n > sizeof(pqry.keyOwner.name)) {
     n = sizeof(pqry.keyOwner.name);
   }
   strncpy(pqry.keyOwner.name,
           entity,
           n);

   // pass request to the "real" makeKeyRequest
   makeKeyRequest( pqry, eventhandler, entryData);
}

// cancel a key request (actually, simply removes the attached eventhandler)
void cKeyProvider::cancelKeyRequest( char* , 
                                     FrEntIdType , 
                                     FrPubKeyType , 
                                     uint32_t , 
			             cKeyProviderEventHandler* )
{
}

// make the key request
//
// NOTE that the default action is to delete the eventhandler object
// after it is invoked. To suppress this, call the eventhandler's
// "setDispose( false)" method (before calling this routine)
//
void cKeyProvider::makeKeyRequest ( FrKeyQryQueryExp qry,
                                    cKeyProviderEventHandler* eventhandler,
                                    cKeyProviderCacheEntryData& entryData)
{
    CZKS::Exception e;
    static FrCryptPubKey key;
    static bool loadsig = false;

    entryData.setState( FRKEYQRY_ST_VALID);
    entryData.setQry( qry);

    if (qry.keyType == FRCRYPT_KEYTYPE_SIGNING) {
      if (qry.keyOwner.type == FRENT_TYPE_AIP) {
       if (!loadsig) {   
	         char sendPub[] = "/turf/frankf/sig/m.pub";
         if (!frCryptSigNewFromFile(e.zkError(), &key, sendPub))
         { 
            CZKS::Throw(e);
         }
         loadsig = true;
       }
       entryData.setPubKey(key);
      }
      else {
        if (strcmp(qry.keyOwner.name, "notfound") == 0) {
	   entryData.setState(FRKEYQRY_ST_NOT_FOUND);
	}
        else {
           FrCryptSig* sigKey = getGnymSig();
           entryData.setPubKey(*sigKey);
	}
      }
    }
    else {
  
      FrCryptSig* cryptSig = getGnymSig();
    
      entryData.setPubKey(*cryptSig);
    }

    if (eventhandler) {
        eventhandler->handleEvent( entryData);
        if (eventhandler->getDispose()) {
          delete eventhandler;
          eventhandler = NULL;
	}
      }

  return;

}

// make a key request and return entry as is
void cKeyProvider::getKeyNoWait( char* , 
                                 FrEntIdType , 
                                 FrPubKeyType, 
                                 uint32_t ,
                                 cKeyProviderCacheEntryData& )
{
}

// make a key request and wait (at most "timeout" seconds) for response
void cKeyProvider::getKeyWait( char* , 
                               FrEntIdType , 
                               FrPubKeyType, 
                               uint32_t , 
                               ZKPRIntervalTime,
                               cKeyProviderCacheEntryData& )
{
}

