#define REV_ID "@(#) $Id: cLinkMonitor.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include "cLinkMonitor.h"
#include "czkNetAddr.h"
#include "czkFile.h"

#include "global.h"

//constructor
cLinkMonitor::cLinkMonitor() {}

//destructor
cLinkMonitor::~cLinkMonitor() 
{
   cLinkMonitorInfo* linkinfo;

   // delete all the linkinfo objects (reference the linkId table)
   LinkIdTable::iterator j = linkIdTable.begin(); 
   while (j != linkIdTable.end())
   {
      linkinfo = j->second;
      delete linkinfo;
      j++;
   }

}

//shhutdown links with other components
void cLinkMonitor::shutdown() 
{
}


//validate config parameters
bool cLinkMonitor::validateConfig()
{
   return true;
}

// initialize private data
bool cLinkMonitor::initializePrivateData( czkCoreLoop* pcoreloop)
{
   coreloop = pcoreloop;
   return true;
}

// start
void cLinkMonitor::startup()
{
}

// dump link name table
void cLinkMonitor::dumpLinkNameTable()
{
#ifdef LINKMANAGERUNITTEST
      cLinkMonitorInfo* linkinfo;

      LinkNameTable::iterator j;

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " *************************");
      ZKS_LOGDEBUG_0( ZKS_LINKMANAGER_MOD, "   Contents of Link Name Table ");

      j = linkNameTable.begin(); 
      while (j != linkNameTable.end())
      {

         linkinfo = j->second;
        
         ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " name %s type %d linkid %x state %d ",
                         linkinfo->entity.name, linkinfo->entity.type,
                         linkinfo->getLinkId(), linkinfo->state);
         j++;
      }

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " *************************");
#endif
}

// dump link id table
void cLinkMonitor::dumpLinkIdTable()
{
#ifdef LINKMANAGERUNITTEST
      cLinkMonitorInfo* linkinfo;

      LinkIdTable::iterator j;

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " *************************");
      ZKS_LOGDEBUG_0( ZKS_LINKMANAGER_MOD, "   Contents of Link Id Table ");

      j = linkIdTable.begin(); 
      while (j != linkIdTable.end())
      {

         linkinfo = j->second;
        
         ZKS_LOGDEBUG_4(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " name %s type %d linkid %x state %d ",
                         linkinfo->entity.name, linkinfo->entity.type,
                         linkinfo->getLinkId(), linkinfo->state);
         j++;
      }

      ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK,  ZKS_LINKMANAGER_MOD, " *************************");
#endif
}

// lookup entity in link name table and return linkinfo
cLinkMonitorInfo* cLinkMonitor::findLink( FrEntIdExp& entity)
{
   cLinkMonitorInfo* linkinfo = NULL;

   LinkNameTable::iterator i = linkNameTable.find( entity);
   if (i != linkNameTable.end()) {
      linkinfo = i->second;
   }
   return linkinfo;
}

// lookup linkId in link id table and return linkinfo
cLinkMonitorInfo* cLinkMonitor::findLink( LinkId linkId)
{
   cLinkMonitorInfo* linkinfo = NULL;

   cLinkMonitorInfo* linkinfofound = NULL;

   LinkNameTable::iterator i = linkNameTable.begin();
   while (linkinfofound == NULL && i != linkNameTable.end()) {
      linkinfo = i->second;
      if (linkinfo->linkId == linkId) {
        linkinfofound = linkinfo;
      }
      else {
        i++;
      }
   }
   return linkinfofound;
}

// lookup linkId in link id table and return linkinfo
cLinkMonitorInfo* cLinkMonitor::findLink( ZKPRNetAddr addr)
{
   cLinkMonitorInfo* linkinfo = NULL;
   cLinkMonitorInfo* linkinfofound = NULL;

   LinkNameTable::iterator i = linkNameTable.begin();
   while (linkinfofound == NULL && i != linkNameTable.end()) {
      linkinfo = i->second;
      if (memcmp(&linkinfo->addr, &addr, sizeof(addr)) == 0) {
        linkinfofound = linkinfo;
      }
      else {
        i++;
      }
   }
   return linkinfofound;
}

// lookup linkId in link id table and return linkinfo
cLinkMonitorInfo* cLinkMonitor::findLinkClient()
{
   cLinkMonitorInfo* linkinfo = NULL;
   cLinkMonitorInfo* linkinfofound = NULL;

   LinkNameTable::iterator i = linkNameTable.begin();
   while (linkinfofound == NULL && i != linkNameTable.end()) {
      linkinfo = i->second;
      if (linkinfo->entity.type == FRENT_TYPE_ANON) {
        linkinfofound = linkinfo;
      }
      else {
        i++;
      }
   }
   return linkinfofound;
}

// lookup linkId in link id table and delete the entry
void cLinkMonitor::removeLinkId( LinkId linkId)
{
   LinkIdTable::iterator i = linkIdTable.find( linkId);
   if (i != linkIdTable.end()) {
      linkIdTable.erase(i);
   }
}

int32_t cLinkMonitor::handleTimeout(czkEventHandlerData*)
{
  return 0;
} 

void cLinkMonitor::handleException(czkSocket* zkfd, 
                                        czkEventHandlerData*)
{ 
  czkIntervalTime interval;

  PRErrorCode status = PR_GetError();
  
  ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," exception %d", status);

  coreloop->removeHandler( zkfd);
  zkfd->close();

  acceptedsock = false;
}

void cLinkMonitor::handleInput(czkSocket* zkfd, 
                                    czkEventHandlerData*)
{
  char buf[BUFSIZ];

  czkIntervalTime interval;
  ZKPRIntervalTime timeout; 

  ZKPRNetAddr addrfrom;

  //cout << " in handle input " << endl;

  timeout = interval.getSeconds(0);

  if (zkfd == me) {
    if (zkfd->accept( &stream, &addrfrom, timeout) == 0) {
      cout << " get a connect " << endl;

      coreloop->registerHandler( this, NULL, stream, 
                                 ZKPR_POLL_READ  | ZKPR_POLL_EXCEPT);
    } 
    else {
      cout << " error " << PR_GetError() << endl;
    }
  }
  else {
    LINKMONXFER* xfer;
    int nb = zkfd->recv( buf, BUFSIZ, 0, timeout);
    ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP, " LINKMON READ XFER %d size %d ",
		   nb, sizeof(*xfer));
    if ((uint32_t)nb >= sizeof(*xfer)) {
      for (uint32_t i = 0; i < nb/sizeof(*xfer); i++)
      {
         xfer = (LINKMONXFER*)&buf[i*sizeof(*xfer)];
         if (xfer->action == monAddLink) {
           cLinkMonitorInfo* linkInfo = findLink( xfer->entity);
           if (linkInfo) { 
	      linkInfo->linkId = xfer->linkId;
              linkInfo->state = monLinkUp;	 
              ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"LINK ADDED for %s LINKID %x",
                                                xfer->entity.name, 
                                                linkInfo->linkId);
	   }
           else {
             cLinkMonitorInfo* linkInfo = findLinkClient();
             if (linkInfo) { 
	       linkInfo->linkId = xfer->linkId;
               linkInfo->state = monLinkUp;	 
               ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"LINK ADDED for CLIENT %s LINKID %x",
                                                 linkInfo->entity.name, 
                                                 linkInfo->linkId);
	     }
             else {
               ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," link mon cant find client");
	     }
	   }
	 }
         else if (xfer->action == monUpdateLink || 
                  xfer->action == monRemoveLink) {
           cLinkMonitorInfo* linkInfo = findLink( xfer->linkId);
           if (linkInfo) { 
	      linkInfo->linkId = xfer->linkId;
              if (xfer->action == monUpdateLink) {
                 linkInfo->state = monLinkUp;
                 ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"LINK UPDATED for LINKID %x",
                                                linkInfo->linkId);
	      }
              else {
                linkInfo->state = monLinkUp;
                ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"LINK REMOVED for LINKID %x",
                                                linkInfo->linkId);
	      }
	   }
           else if (xfer->action == monUpdateLink) {
             cLinkMonitorInfo* linkInfo = findLink( xfer->entity);
             if (linkInfo) { 
	       linkInfo->linkId = xfer->linkId;
               linkInfo->state = monLinkUp;	 
               ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"LINK ADDED for %s LINKID %x",
                                                xfer->entity.name, 
                                                linkInfo->linkId);
	     }
             else {
               ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," link mon cant find ent %s ",
                                                 xfer->entity.name);
	     }
	   }
           else {
              ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," link mon cant find id %x ",
                                                xfer->linkId);
	   }
	 }
         else if (xfer->action == monAddRoute) {
            ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"ROUTE ADDED (RouteId %x)",
                                              xfer->linkId);
         }
         else if (xfer->action == monUpdateRoute) {
            ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"ROUTE UPDATED (RouteId %x)",
                                              xfer->linkId);
         }
         else if (xfer->action == monRemoveRoute) {
            ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,"ROUTE REMOVED (RouteId %x)",
                                             xfer->linkId);	   
	 }
      }
    }
    else {
      ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," recv err %d", nb);
    }
  }
}

void cLinkMonitor::handleOutput(czkSocket*, 
                                czkEventHandlerData*)
{
}

void cLinkMonitor::init(czkCoreLoop* cl)
{
  coreloop = cl;

  CZKS::Exception e;

  czkFile file;
  file.deleteFile(UDSlinkmonitor);

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  

  ZkSocket s;
  int32_t fd;
  if (!zkSocketNew( e.zkError(), AF_LOCAL, SOCK_STREAM, IPPROTO_IP, &s)) {
     cout << " error creating zksocket " << endl;
     abort();
  } 
  else if (!zkSocketBind( e.zkError(), s, UDSlinkmonitor)) {
     cout << " error bindinging zksocket " << endl;
     abort();
  } 
  else if (!zkSocketGetFD( e.zkError(), s, &fd)) {
     cout << " error getting fd zksocket " << endl;
     abort();
  }

  me = new czkSockNativeTCP();
  me->open(fd); 

  me->listen( 1);

  acceptedsock = false;
  
  cl->registerHandler( this, NULL, (czkSocket*)me, 
                       ZKPR_POLL_READ  | ZKPR_POLL_EXCEPT);

} 

