#ifndef _cLinkMonitor

#define _cLinkMonitor

// class cLinkMonitor
//

#include "czkIntervalTime.h"
#include "czkLock.h"
#include "czkTimerProxy.h"
#include "czkCoreLoop.h"
#include "czkSockNativeTCP.h"
#include "cAipComponent.h"

#include "zksocket.h"
#include "frCrypt.h"
#include "frEnt.h"

#include "aip-modules.h"

#include "cLinkMonitorInfo.h"

#include "testFunctionalAip.h"

#define UDSlinkmonitor "/turf/frankf/UDSlinkmonitor"

//"less than" function for the LinkNameTable map definition
struct lesslinkname
{
  bool operator()(const FrEntIdExp s1, const FrEntIdExp s2) const
  {
    bool rc = (memcmp(&s1, &s2, sizeof(s1)) < 0);
    return rc;
  }
};

//"less than" function for the LinkIdTable map definition
struct lesslinkid
{
  bool operator()(const LinkId s1, const LinkId s2) const
  {
    bool rc = (s1 < s2);
    return rc;
  }
};

class cLinkMonitor : public cAipComponent, public czkEventHandler {

private:

  typedef map<FrEntIdExp, cLinkMonitorInfo*, lesslinkname> LinkNameTable;
  typedef map<LinkId,     cLinkMonitorInfo*, lesslinkid>   LinkIdTable;

  LinkNameTable linkNameTable;  //map of LinkMonitorInfo by entity
  LinkIdTable   linkIdTable;    //map of LinkMonitorInfo by linkId
 
  int32_t acceptedsock;
  czkCoreLoop* coreloop; 
  czkSockNativeTCP* me;
  czkSocket* stream;

  //private methods
 
  //copy constructor (disabled)
  cLinkMonitor( const cLinkMonitor& c): cAipComponent(c), czkEventHandler(c) { assert(0); };

  //assignment operator (disabled)
  cLinkMonitor& operator=( const cLinkMonitor&) { assert(0); };

  //remove an entry from the link id table
  void removeLinkId( LinkId linkId);

  //debug routines
  void dumpLinkNameTable();
  void dumpLinkIdTable();

public:

  // constructor
  cLinkMonitor();

  // destructor
  virtual ~cLinkMonitor();

  // validate configuration
  bool validateConfig();

  // initialize private data members
  bool initializePrivateData(czkCoreLoop* coreloopin);

  void cleanupPrivateData() {};

  // shutdown all links shared by other components
  void shutdown();

  // startup
  void startup();

   void handleInput( czkSocket*, czkEventHandlerData*);
   void handleOutput( czkSocket*, czkEventHandlerData*);
   void handleException( czkSocket*, czkEventHandlerData*);
   int32_t handleTimeout(czkEventHandlerData* user_data);

  void init( czkCoreLoop* cl);

  void addLink( FrEntIdExp entity, LinkId linkId, cLinkMonitorInfo* info)
  { 
     linkNameTable[entity] = info; 
     if (linkId > 0) {
       linkIdTable[linkId] = info; 
     }
  }
        
  //find a link name table entry
  cLinkMonitorInfo* findLink( FrEntIdExp& entity);

  //find a link id table entry
  cLinkMonitorInfo* findLink( LinkId linkId);

  cLinkMonitorInfo* findLink( ZKPRNetAddr addr);

  cLinkMonitorInfo* findLinkClient();
};

#endif
