#ifndef _cLinkMonitorInfo

#define _cLinkMonitorInfo

#include "frEnt.h"
#include "frCrypt.h"

#include "aip-modules.h"

#include "czkIntervalTime.h"
#include "czkTimerProxy.h"
#include "czkCoreLoop.h"

#include "cPktMon.h"

class cLinkMonitor;
class cLinkMonitorInfo;

enum MonLinkAction { monAddLink, monUpdateLink, monRemoveLink,
                     monAddRoute, monUpdateRoute, monRemoveRoute };

typedef uint32_t LinkId;

typedef struct linkmonxferstruct LINKMONXFER;
struct linkmonxferstruct {
  MonLinkAction action;
  FrEntIdExp    entity;         //entity name
  LinkId        linkId;         //link id
  ZKPRNetAddr   dhaddr;
};  

/* possible links states */
enum MonLinkState { monLinkDown, monLinkUp, monLinkUpdated };

typedef FrCryptSym SymKey;


#ifdef __cplusplus

class cLinkMonitorInfo {

  friend class cLinkMonitor;

public:

  FrEntIdExp  entity;         //entity name
  LinkId      linkId;         //link id
  ZKPRNetAddr addr;           //IP address
  ZKPRNetAddr dhaddr;         //IP address (for DH)
  MonLinkState   state;          //link state
  uint8_t     lekv;        //key version
  SymKey      encrKey[2];        //encryption key
  SymKey      decrKey[2];        //decryption key

  cPktMon*    pktMon;
  class cDHNego*    pDHNego;

private:

  //private methods
  //copy constructor (disabled)
  cLinkMonitorInfo( const cLinkMonitorInfo&) { assert(0); };

  //assignment operator (disabled)
  cLinkMonitorInfo& operator=( const cLinkMonitorInfo&) { assert(0); };

public:

  cLinkMonitorInfo();
  virtual ~cLinkMonitorInfo();

  //free any allocated SymKeys
  void freeSymKeys(); 

  FrEntIdExp&   getEntity()    { return entity; };
  LinkId        getLinkId()    { return linkId; };
  ZKPRNetAddr&  getAddr()      { return addr; };    
  MonLinkState& getMonLinkState() { return state; };
  SymKey&       getEncrKey()   { return encrKey[lekv]; } ;
  SymKey&       getDecrKey()   { return decrKey[lekv]; } ;
  uint8_t       getLekv()      { return lekv; };

};

#endif /* __cplusplus */

#endif
