#define REV_ID "@(#) $Id: cPktMon.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include <iostream>
#include "zksexception.h"
#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zkIntTypes.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include "frCrypt.h"
#include "frPktFmt.h"

#include "cPktMon.h"
#include "czkSockDgram.h"
#include "czkNetAddr.h"
#include "cKeepAlivePkt.h"
#include "cRouteCreatePkt.h"
#include "cRouteCreateAckPkt.h"
#include "cRouteDestroyPkt.h"
#include "cLinkMonitor.h"
#include "cRouteMonitor.h"

#include "cPrivateKeys.h"

#include "setupRouteCreate.h"

#include "global.h"

void cPktMon::initSocket(czkCoreLoop* cl, cLinkMonitor* pl, cRouteMonitor* pr,
                         ZKPRNetAddr addrin,
                         bool boundin)
{
  stream = new czkSockDgram;

  CZKS::Exception e;

  coreloop = cl;
  pLinkMonitor = pl;
  pRouteMonitor = pr;
  addr = addrin;

  stream->open();

  ZKPRSocketOptionData opt;
  opt.option = ZKPR_SockOpt_Reuseaddr;
  opt.value.reuse_addr = ZKPR_TRUE;
  stream->setOption( &opt);
  opt.option = ZKPR_SockOpt_Nonblocking;
  opt.value.non_blocking = 1;
  stream->setOption( &opt);

  bound = boundin;
  if (bound) {
    stream->bind(addr); 
  }

  coreloop->registerHandler( this, NULL, (czkSocket*)stream, 
                             ZKPR_POLL_READ | ZKPR_POLL_EXCEPT);

} 

void cPktMon::handlePkt( cLinkMonitorInfo* linkInfo, uint8_t* pPkt, 
                         size_t pktSize)
{
  CZKS::Exception e;

  FrPktType       type;
  if (frPktGetType(e.zkError(), pPkt, pktSize, &type) == ZKS_FAILURE) {
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " cannot get pkt type ");
    //CZKS::Throw(e);
  }

  cRouteCreatePkt*    rcPkt  = NULL;
  cRouteCreateAckPkt* rcaPkt = NULL;
  cRouteDestroyPkt*   rdPkt  = NULL;

  FrAci aci; 
  if (type == FRPKTTYPE_AIP_ROUTE_CREATE)
  {
     rcPkt = new cRouteCreatePkt;
     rcPkt->attachPkt( pPkt, pktSize);
     aci = rcPkt->getAci();

     uint32_t version;
     if (frPktGetVersion(e.zkError(), pPkt, pktSize, &version) == 
                         ZKS_FAILURE) {
	CZKS::Throw( e);
     }

     uint32_t privVers = rcPkt->getCryptKeyVersion();
 
     //get AIP's private crypt key
     cPrivateKeys* pPrivateKeys = getPrivateKeys();

     FrCryptPubKey cryptKey;
     pPrivateKeys->getCryptKey( privVers, &cryptKey);

     // decrypt packet
     bool lastHop = false;
     while (!lastHop)
     {
        lastHop = rcPkt->decrypt( cryptKey);
     }
     ZKS_LOGDEBUG_3(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " received RouteCreate for link %d aci %d lastHop %d", 
                    linkInfo->linkId, aci, lastHop);


        FrAci exitAci;
        if (rcPkt->getFlags() & FRPKT_RTECRT_FL_SPEC_ACI) {
 	   exitAci = rcPkt->getSpecAci();
	}
        else {
           exitAci = (rand() % 32767);
	}

     cRouteMonitorInfo* r = pRouteMonitor->findRouteNext(linkInfo->linkId);
     if (r && r->action == ACK) {

        cRouteCreateAckPkt pkt;
        pkt.create( version, FRPKT_PRIORITY_DEFAULT, exitAci, aci,
                    r->MacKey[r->nHops-1]);
        uint8_t* pAck;
        size_t ackLen;
        pkt.getBuffer(&pAck, &ackLen);
        ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    "sending RouteCreateAck for link %d aci %d", 
                    linkInfo->linkId, aci);
        sendPkt(pAck, ackLen, getMyAipAddr(), linkInfo->linkId);
        pkt.destroy();

     }
 
     rcPkt->detachPkt();
     delete rcPkt;

  }
  else if (type == FRPKTTYPE_AIP_ROUTE_CRTACK)
  {
     rcaPkt = new cRouteCreateAckPkt;
     rcaPkt->attachPkt( pPkt, pktSize);   
     aci = rcaPkt->getAci();
     ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " received RouteCreateACK for link %d aci %d", 
                    linkInfo->linkId, aci);

     cRouteMonitorInfo* r = pRouteMonitor->findRoute(linkInfo->linkId, aci);
     if (r) {
       r->gotAck = true;
       rcaPkt->decrypt( r->fwdSym[r->iHop]);
       r->exitAci = rcaPkt->getExitAci( r->MacKey[r->nHops-1]);
       ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                      " decoded exitAci %d", 
                      r->exitAci);
     }
     else {
       ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                      " not Route Info entry found");
     }
     rcaPkt->detachPkt();
     delete rcaPkt;
  }
  else if (type == FRPKTTYPE_AIP_ROUTE_DESTROY)
  {
     rdPkt = new cRouteDestroyPkt;
     rdPkt->attachPkt( pPkt, pktSize);
     aci = rdPkt->getAci();
     ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " received RouteDestroy for link %d aci %d", 
                    linkInfo->linkId, aci);

     cRouteMonitorInfo* r = pRouteMonitor->findRoute(linkInfo->linkId, aci);
     if (r) {
       r->gotDestroy = true;
       rdPkt->decrypt( r->fwdSym[0]);
     }
     else {
       ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                      " not Route Info entry found");
     }

     rdPkt->detachPkt();
     delete rdPkt;
  }
  else if (type == FRPKTTYPE_AIP_ALIVE)
  {
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                   " received keepalive for link %d", linkInfo->linkId);
  }
  else {
    ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," PktMon recv unknown type buflen %d", 
                   pktSize);
  }

}


void cPktMon::handleInput( czkSocket* sock, czkEventHandlerData*)
{
  CZKS::Exception e;

  char buf[BUFSIZ];

  ZKPRIntervalTime timeout;
  czkIntervalTime interval;
  timeout = interval.getSeconds(0);

  ZKPRNetAddr addrfrom;

  int nb = sock->recvfrom( buf, BUFSIZ, 0, &addrfrom, timeout); 

  uint8_t lekv = buf[nb-1];

  cLinkMonitorInfo* linkInfo = pLinkMonitor->findLink( addr);
  if (linkInfo) {
     if (frPktLinkDecrypt(e.zkError(), linkInfo->decrKey[lekv], 
                          buf, nb) == ZKS_FAILURE)
     {
        ZKS_LOGDEBUG_2(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                       " cannot decrypt pkt for link %d lekv %d ", 
                       linkInfo->linkId, lekv);
        //CZKS::Throw(e);
     }
     handlePkt( linkInfo, (uint8_t*)buf, nb);
  }
  else {
     ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP,
                    " cPktMon: cannot find linkid ");
  }

}

void cPktMon::handleOutput( czkSocket*, czkEventHandlerData*)
{
}

void cPktMon::handleException( czkSocket* sock, 
                                        czkEventHandlerData*)
{
  CZKS::Exception e;

  coreloop->removeHandler(sock);

  sock->close();
  delete sock;

  czkIntervalTime interval;
  ZKPRIntervalTime timeout = interval.getSeconds(10);
  coreloop->scheduleTimer( this, NULL, timeout);
}

int32_t cPktMon::handleTimeout( czkEventHandlerData* data)
{
  int32_t rs;

  if (data == NULL) {
     initSocket(coreloop,pLinkMonitor,pRouteMonitor,addr, bound);
     rs = 0;
  }
  else {

     cPktMonTimeoutData* d = (cPktMonTimeoutData*) data;
     sendKeepAlive( &d->addr, d->linkId, d->bad);
     rs = 1;
  }
  return rs;
}

void cPktMon::sendKeepAlive(ZKPRNetAddr* addr, LinkId linkId, bool bad)
{
  char sendbuf[BUFSIZ];

  cKeepAlivePkt ka;
  ka.create(FRPKT_DATA_VERSION_2, FRPKT_PRIORITY_DEFAULT);
  uint8_t* buf;
  size_t buflen;
  ka.getBuffer(&buf, &buflen);
  if (bad) buflen--;

  memcpy( sendbuf, buf, buflen);
  memcpy( &sendbuf[buflen], &linkId, sizeof(linkId));
  buflen += sizeof(linkId);

  //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," sending keepalive link %d ",linkId);
  czkIntervalTime interval;
  stream->sendto( sendbuf, buflen, 0, addr, interval.getSeconds(0));

  ka.destroy();

}

void cPktMon::sendPkt(uint8_t* pkt, size_t pktSize, 
                      ZKPRNetAddr addr, LinkId linkId)
{
  char sendbuf[BUFSIZ];

  size_t buflen = pktSize;
  memcpy( sendbuf, pkt, buflen);
  memcpy( &sendbuf[buflen], &linkId, sizeof(linkId));
  buflen += sizeof(linkId);

  //ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, TESTFUNCTIONALAIP," sending pkt link %d ",linkId);
  czkIntervalTime interval;
  stream->sendto( sendbuf, buflen, 0, &addr, interval.getSeconds(0));

}

void cPktMon::startKeepAlive(ZKPRNetAddr addr, LinkId linkId, 
                             uint32_t repeatInterval, bool bad)
{

  if (!pKeepAliveProxy) {

     czkIntervalTime interval;
  
     timeoutData.addr = addr;
     timeoutData.linkId = linkId;
     timeoutData.bad = bad;
     pKeepAliveProxy = 
        coreloop->scheduleTimer( this, &timeoutData, 
                                 interval.getSeconds(repeatInterval));

  }

}   

void cPktMon::cancelKeepAlive()
{
  if (pKeepAliveProxy) {
    pKeepAliveProxy->cancelTimer();
    delete pKeepAliveProxy;
  }
}
 
    
  






