#define REV_ID "@(#) $Id: cPrivateKeys_stub.cpp,v 1.2 2001/04/18 17:38:01 alainc Exp $"
#include <zkrev.h>
#include "cPrivateKeys.h"
#include "global.h"

//constructor
cPrivateKeys::cPrivateKeys() { keys = NULL; }

//destructor
cPrivateKeys::~cPrivateKeys() {

  CZKS::Exception zksException;

  if (keys) {
     if (!frSecKeyChainFree(zksException.zkError(), &keys)) {
        CZKS::Throw(zksException);
     }
  }
      
}

//initialize the lock
void cPrivateKeys::init()
{
#ifdef AIPMULTITHREADED
   const char* func = "cPrivateKeys::init";
  
#endif
}

//cleanup the lock
void cPrivateKeys::cleanup()
{
#ifdef AIPMULTITHREADED
#endif
}

string cPrivateKeys::getAipName()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           AIP_ENTITY_NAME);
}

FrEntIdExp cPrivateKeys::getAipEntity()
{
    FrEntIdExp aipEntity;
    string aipName = getAipName();
    strncpy( aipEntity.name, aipName.c_str(), sizeof(aipEntity.name));
    aipEntity.type = FRENT_TYPE_AIP;
    return aipEntity;
} 
   
string cPrivateKeys::getAipPrivCrypt()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           PK_CRYPT_DIRECTORY,
                                           PK_CRYPT_DIRECTORY_DEF);
}

string cPrivateKeys::getAipPrivSig()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           PK_SIG_DIRECTORY,
                                           PK_SIG_DIRECTORY_DEF);
}

string cPrivateKeys::getPrompt()
{
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           PK_PASSPHRASE_PROMPT,
                                           PK_PASSPHRASE_PROMPT_DEF);
}

string cPrivateKeys::getPassfile()
{ 
    return pConfigProvider->getOpt_string( UAIP_SECTION_NAME, 
                                           PK_PASSPHRASE_FILE);
}

//validate configuration parameters
bool cPrivateKeys::validateConfig()
{
  const char* func = "cPrivateKeys::validateConfig";

  const char* errmsg = "Invalid config parameter %s";

  pConfigProvider =
      (cConfigProvider *) pComponentManager->get(ZKS_CONFIGPROVIDER_MOD);

  bool rc = true;

  try {
     string aipName = getAipName();
     if (aipName.length() <= 0 || 
         aipName.length() >= FRENT_NAME_SIZE) {
       CZKS::Throw( ZKS_PRIVATEKEYS_MOD, ZKS_PRIVATEKEYS_INVALIDCONFIG_ERR,
                    func, errmsg, AIP_ENTITY_NAME);
     } 

     string aipPrivCrypt = getAipPrivCrypt();
     if (aipPrivCrypt.length() <= 0) {
       CZKS::Throw( ZKS_PRIVATEKEYS_MOD, ZKS_PRIVATEKEYS_INVALIDCONFIG_ERR,
                    func, errmsg, PK_CRYPT_DIRECTORY);
     } 

     string aipPrivSig = getAipPrivSig(); 
     if (aipPrivSig.length() <= 0) {
       CZKS::Throw( ZKS_PRIVATEKEYS_MOD, ZKS_PRIVATEKEYS_INVALIDCONFIG_ERR,
                    func, errmsg, PK_SIG_DIRECTORY);
     } 

     //
     // Get the passphrase access method: eighter via the prompt or a file
     //
     if( getPrompt().empty() ) {
       string passfile = getPassfile();
       if( passfile.empty() ) {
         CZKS::Throw( ZKS_PRIVATEKEYS_MOD, ZKS_PRIVATEKEYS_INVALIDCONFIG_ERR,
                      func, "failure: eighter %s or %s must be provide", 
                      PK_PASSPHRASE_PROMPT, PK_PASSPHRASE_FILE);
       } 
     }
  }
  catch (CZKS::Exception& e) {

     CZKS::LogException(e);
     rc = false;

  }
  return rc; 

}

//initialize private data elements
bool cPrivateKeys::initializePrivateData(czkCoreLoop* pCoreLoopin)
{
  const char* func = "cPrivateKeys::initializePrivateData";

  CZKS::Exception zksException;

  coreloop = pCoreLoopin;
  if (coreloop == NULL) {
     coreloop = new czkCoreLoop;
  }

  return true;
}
void cPrivateKeys::cleanupPrivateData()
{}
 
//startup method
void cPrivateKeys::startup() {

  const char* func = "cPrivateKeys::startup";

  CZKS::Exception zksException;

  try {
  
    // initialize
    if (!frSecKeyChainNew(zksException.zkError(), &keys)) {
      CZKS::Throw(zksException);
    }

    // load the key info from disk
    if (!frSecKeyChainAddSymKey(zksException.zkError(), keys, getPassfile().c_str(), 
                                getPrompt().c_str(), FRCRYPT_SYMALG_BLOWFISH)) {
       CZKS::Throw(zksException);
    }

    if (!frSecKeyChainRegisterDir(zksException.zkError(), keys, &sigHandle, 
                                  getAipPrivSig().c_str())) {
       CZKS::Throw(zksException);
    }
    
    if (!frSecKeyChainRegisterDir(zksException.zkError(), keys, &cryptHandle,
                                  getAipPrivCrypt().c_str())) {
       CZKS::Throw(zksException);
    }

    int32_t rc = frSecKeyChainReloadKeys(zksException.zkError(), keys);
    if (!rc) {
       CZKS::Throw(zksException);
    }

    //get and validate the keys
    FrCryptPubKey key;
    getCryptKey( 0, &key);
    validateEntity( key); 
    getSigKey( 1, &key);
    validateEntity( key); 

    ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_PRIVATEKEYS_MOD, " startup complete");

    signalReady();

  }
  catch (CZKS::Exception &e) {

    e.Rethrow( func, "cannot startup cPrivateKeys");

  }
  catch (...) {

    CZKS::Rethrow( func, "cannot startup cPrivateKeys");

  }
}

//validate entity in key against "aip" entity
void cPrivateKeys::validateEntity( FrCryptPubKey key)
{
  const char* func = "cPrivateKeys::validateEntity";
 
  CZKS::Exception zksException;
  
  FrEntIdExp keyEntity;
  FrEntIdExp aipEntity = getAipEntity();

  try {

     if (!frCryptPubKeyGetEntity(zksException.zkError(), key, &keyEntity)) {
        CZKS::Throw(zksException);
     }

     if (memcmp(&aipEntity, &keyEntity, sizeof(keyEntity)) != 0) {
        ostrstream os;
        os << "Entity [" << aipEntity.name << " " << aipEntity.type <<
	  "] in config file doesn't match key signature key entity [" <<
          keyEntity.name << " " << keyEntity.type << "]";
        CZKS::Throw( ZKS_PRIVATEKEYS_MOD, ZKS_PRIVATEKEYS_ENTITYMISMATCH_ERR, 
                     func, os.str());
     }

  }
  catch (CZKS::Exception &e) {

    e.Rethrow( func, "failed");

  }
  catch (...) {

    CZKS::Rethrow( func, "failed");

  }

}

//reload the private keys
void cPrivateKeys::reload()
{
  // reload the keys

  const char* func = "cPrivateKeys::reload";
 
  CZKS::Exception zksException;

  try {

     int32_t rc = frSecKeyChainReloadKeys(zksException.zkError(), keys);
  
     if (!rc) {
       CZKS::Throw( zksException);
     }
  }
  catch (CZKS::Exception& e) {

     e.Rethrow( func, "Failure");

  }
  catch (...) {

     CZKS::Rethrow( func, "Failure");

  }
}

//return the crypt key
void cPrivateKeys::getCryptKey( uint32_t version, FrCryptPubKey* key)
{ 
  // return the crypt key

  const char* func = "cPrivateKeys::getCryptKey";
 
  CZKS::Exception zksException;

  try {

     int32_t rc = frSecKeyChainGetVersion(zksException.zkError(), keys, 
                                          cryptHandle, version, key);
     if (!rc) {
        CZKS::Throw( zksException);
     }

  }
  catch (CZKS::Exception& e) {

     e.Rethrow( func, "Failure");

  }
  catch (...) {

     CZKS::Rethrow( func, "Failure");

  }

}  

//return the sig key
void cPrivateKeys::getSigKey( uint32_t version, FrCryptPubKey* key)
{ 
  // return the signature key
  
  const char* func = "cPrivateKeys::getSigKey";
 
  CZKS::Exception zksException;


  if (version == 1) {

     int32_t rc = frSecKeyChainGetVersion(zksException.zkError(), keys, 
                                          sigHandle, 0, key);

     if (!rc) {
        CZKS::Throw( zksException);
     }
  }
  else {
    char sendPriv[] = "/turf/frankf/sig/m.priv";
    assert(frCryptSigNewFromFile(zksException.zkError(), key,   sendPriv));
  }
}

