#ifndef _cRouteMonitor
#define _cRouteMonitor

#include <map>

#include "czkCoreLoop.h"
#include "aip-modules.h"
#include "frEnt.h"
#include "frPktFmt.h"
#include "czkIntervalTime.h"
#include "czkLock.h"

#include "cRouteCreatePkt.h"
#include "cRouteCreateAckPkt.h"
#include "cRouteDestroyPkt.h"
#include "cKeepAlivePkt.h"

#include "cRouteMonitorInfo.h"

typedef map<uint32_t, cRouteMonitorInfo*> RouteMonitorInfoMap; 

class cRouteMonitor 
                      
{

private:

    RouteMonitorInfoMap routeTable;

    //private methods

    //copy constructor (disabled)
    cRouteMonitor( const cRouteMonitor& ) { assert(0); };

    //assignment operator (disabled)
    cRouteMonitor& operator=( const cRouteMonitor&) { assert(0); };

public:

    cRouteMonitor() {};
    virtual ~cRouteMonitor() {};

    void insertRouteTableEntry( uint32_t routeId, 
                                cRouteMonitorInfo* pNewRouteInfo)
    {
      routeTable[routeId] = pNewRouteInfo;
    }

    void deleteRouteTableEntry( uint32_t routeId)
    {
      RouteMonitorInfoMap::iterator i = routeTable.find(routeId);
      if (i != routeTable.end()) {
	routeTable.erase(i);
      }
    }

    cRouteMonitorInfo* findRoute( uint32_t routeId)
    {
      cRouteMonitorInfo* p = NULL;
      RouteMonitorInfoMap::iterator i = routeTable.find(routeId);
      if (i != routeTable.end()) {
	p = i->second;
      }
      return p;
    }

    cRouteMonitorInfo* findRoute( uint32_t linkId, FrAci aci)
    {
      cRouteMonitorInfo* find = NULL;
      cRouteMonitorInfo* p = NULL;
      RouteMonitorInfoMap::iterator i = routeTable.begin();
      while (i != routeTable.end()) 
      {
	p = i->second;
        if (p->prevLinkId == linkId && p->prevAci == aci) {
          find = p;
          break;
        }
        i++;
      }
      return find;
    }

    cRouteMonitorInfo* findRouteNext( uint32_t linkId)
    {
      cRouteMonitorInfo* find = NULL;
      cRouteMonitorInfo* p = NULL;
      RouteMonitorInfoMap::iterator i = routeTable.begin();
      while (i != routeTable.end()) 
      {
	p = i->second;
        if (p->nextLinkId == linkId) {
          find = p;
          break;
        }
        i++;
      }
      return find;
    }
};

#endif
