#define REV_ID "@(#) $Id: functionalIntegrationAip.cpp,v 1.2 2001/04/18 17:38:02 alainc Exp $"
#include <zkrev.h>
//
// main program for functionalAip test module
//
//                                
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkSignal.h>
    
#include <iostream>
#include <fstream>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkEventHandler.h"
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"
#include "cAipComponent.h"
#include "cAipComponentManager.h"

#include "frCrypt.h"

#include "aip-modules.h"

#include "cAipKernelStub.h"
#include "cAipKernel.h"
#include "cConfigProvider.h"
#include "cDHNegotiator.h"
#include "cNetInfoProvider.h"
#include "cNetinfoProviderStub.h"
#include "cKeyProvider.h"
#include "cLinkManager.h"
#include "cLinkMaintainer.h"
#include "cPktDispatcher.h"
#include "cPrivateKeys.h"
#include "cRouteCreator.h"
#include "cStatsPackager.h"

#include "../../src/uaip.h"

#include "global.h"

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

int main(int32_t argc, char **argv)
{ 
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;

    (void)argc;

    CZKS::Exception zksException;

    cAipComponentManager mgr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     } 
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                             ZKS_LOGAGENT_FILEPERM_USER_READ   |
                             ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                             ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                             ZKS_LOGAGENT_FILEPERM_OTHER_READ)) {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    if( zkSignalIntr(SIGPIPE, SIG_IGN) == SIG_ERR ) {
      CZKS::Throw(ZKS_AIP_MOD,
                  ZKS_UAIP_SIGNAL_ERR,
                  "testFunctiona;Aip",
                  "zkSignalIntr failed");
    }

    try {
   
      try {

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_AIP_MOD, "starting functionalIntegrationAip test ");

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

        setMyAipAddr();

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_AIP_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_AIP_MOD, " running in single thread mode");
        bool dothread = false;
#endif

        czkCoreLoop* coreloop = NULL;
   
        coreloop = new czkCoreLoop();


        std::ifstream istrm("./test.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_ROUTECREATOR_MOD,
                      1,
                      "FunctionalAip",
                      "Failed opening file: ./test.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

        // start cConfigProvider
        cConfigProvider* pConfigProvider = new cConfigProvider(m_cmdline, m_ini);

        // create all the components
        cAipKernel*            pAipKernel       = new cAipKernel();
        cPrivateKeys*          pPrivateKeys     = new cPrivateKeys();
        cNetInfoProvider*      pNetInfoProvider = new cNetInfoProvider();
        cKeyProvider*          pKeyProvider     = new cKeyProvider();
        cStatsHarvestor*       pStatsHarvestor  = new cStatsHarvestor();
        cStatsPackager*        pStatsPackager   = new cStatsPackager();
        cLinkManager*          pLinkManager     = new cLinkManager();
        cLinkMaintainer*       pLinkMaintainer  = new cLinkMaintainer();
        cDHNegotiator*         pDHNegotiator    = new cDHNegotiator();
	cRouteCreator*         pRouteCreator    = new cRouteCreator();
        cPktDispatcher*        pPktDispatcher   = new cPktDispatcher();

        //add them to the component manager

        mgr.add(ZKS_CONFIGPROVIDER_MOD,      pConfigProvider);
        mgr.add(ZKS_STATSHARVESTOR_MOD,      pStatsHarvestor);
        mgr.add(ZKS_PRIVATEKEYS_MOD,         pPrivateKeys);
        mgr.add(ZKS_KEYPROVIDER_MOD,         pKeyProvider);
        mgr.add(ZKS_NETINFOPROVIDER_MOD,     pNetInfoProvider);
        mgr.add(ZKS_AIPKERNEL_MOD,           pAipKernel);
        mgr.add(ZKS_DHNEGOTIATOR_MOD,        pDHNegotiator);
        mgr.add(ZKS_PACKETDISPATCHER_MOD,    pPktDispatcher);
        mgr.add(ZKS_ROUTECREATOR_MOD,        pRouteCreator);
        mgr.add(ZKS_LINKMAINTAINER_MOD,      pLinkMaintainer);
        mgr.add(ZKS_LINKMANAGER_MOD,         pLinkManager);
        mgr.add(ZKS_STATSPACKAGER_MOD,       pStatsPackager);

        mgr.validateAllConfig();
        mgr.initAllComponents( coreloop);

        mgr.startComponent(ZKS_STATSHARVESTOR_MOD,     dothread);
        mgr.startComponent(ZKS_PRIVATEKEYS_MOD,        dothread);
        mgr.startComponent(ZKS_KEYPROVIDER_MOD,        dothread);
        mgr.startComponent(ZKS_NETINFOPROVIDER_MOD,    dothread);
        mgr.startComponent(ZKS_AIPKERNEL_MOD,          dothread);
        mgr.startComponent(ZKS_DHNEGOTIATOR_MOD,       dothread);
        mgr.startComponent(ZKS_PACKETDISPATCHER_MOD,   dothread);
        mgr.startComponent(ZKS_ROUTECREATOR_MOD,       dothread);
        mgr.startComponent(ZKS_LINKMAINTAINER_MOD,     dothread);
        mgr.startComponent(ZKS_LINKMANAGER_MOD,        dothread);
        mgr.startComponent(ZKS_STATSPACKAGER_MOD,      dothread);

        FrCryptSig nymSig;
        pPrivateKeys->getSigKey( 0, &nymSig);
        setGnymSig(nymSig);

        cAipKernelStub aipKernelStub;
        aipKernelStub.initSocket( coreloop, pAipKernel); 

        cNetinfoProviderStub netinfoProviderStub;
        netinfoProviderStub.initSocket( coreloop, pNetInfoProvider, 
                                        &aipKernelStub);

        if (!dothread) {
          int32_t status = coreloop->runEventLoop();
          cout << " coreloop status " << status << endl;
        }
        else {

          // wait for all threads to complete
          mgr.waitForCompletion();
        }
      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_AIP_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow(ZKS_AIP_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();
}


