#!/usr/bin/perl

use Getopt::Long;
use FileHandle;
use IPC::Open3;

# =====================================================
# Global variables
# =====================================================
my $funcIntAip = "$ENV{'ZK_ROOT'}/arch/linux-2.2-ix86/Debug/bin/functionalIntegrationAip";
my $testFuncIntAip = "$ENV{'ZK_ROOT'}/arch/linux-2.2-ix86/Debug/bin/testFunctionalIntegrationAip";
my $configFile = "$ENV{'ZK_ROOT'}/arch/linux-2.2-ix86/Debug/bin/test.conf";
#my @fileInput = ("in-1hop","in-1hop-3routes","in-1hop-3routes-unauth","in-1hop-destroy","in-1hop-dup,in-1hop-extend","in-1hop-extend-badnym","in-1hop-extend-invalid","in-1hop-repeat","in-1hop-unauth");
my @fileInput = ("in-1hop");
my $bufferTestFunc = "";
my $bufferFunc = ""; #not used yet
my @bufferGlobal = ();
my $bufferSize = 0;
my $bufferCursor = 0;
my $version = "version 1.0";
my $debug = 0;
my $cmdNetInfoList = "command NetInfoList";
my $cmdDHNegInit    = "command DHNegInit";
my $cmdDHNegRespond = "command DHNegRespond";
my $cmdRouteCreate  = "command RouteCreate";
my $cmdRouteExtend  = "command RouteExtend";
my $cmdRouteAck     = "command RouteAck";
my $cmdRouteDestroy = "command RouteDestroy";
my $cmdKeepAlive    = "command KeepAlive";
my %NetInfoList  = ();
my %DHNegInit    = ();
my %DHNegRespond = ();
my %RouteCreate  = ();
my %RouteExtend  = ();
my %RouteAck     = ();
my %RouteDestroy = ();
my %KeepAlive    = ();
my $fileBuffer   = "";
my $file         = "";
my $dir          = "";

# =====================================================
# Entry point
# =====================================================

main();

# =====================================================
# Main program
# =====================================================
sub main{
  my $loop = 0;
  my $answer = 1;
  
  getCommandLineConfig();
    
  if($fileBuffer){
    $bufferTestFunc = GetBufferFromFile($fileBuffer); 
    $answer = RunTest($bufferTestFunc);
    print "\n$answer\n";
  }
  else{
    if(!ValidateProg()){
      exit(0);
    }
    if(!ValidateFiles()){
      exit(0);
    }    
    while($fileInput[$loop]){
      $bufferTestFunc = RunProg($fileInput[$loop]);
      $answer = RunTest($bufferTestFunc);
      print "\n$answer\n";

      # reset variables
      %NetInfoList  = ();
      %DHNegInit    = ();
      %DHNegRespond = ();
      %RouteCreate  = ();
      %RouteExtend  = ();
      %RouteAck     = ();
      %RouteDestroy = ();
      %KeepAlive    = ();
      $bufferTestFunc = "";
      $bufferFunc = "";
      @bufferGlobal = ();
      $bufferSize = 0;
      $bufferCursor = 0;

      $loop++;
    }
  }    
}

# =====================================================
# function   : RunTest()
# Description: This function run the test.
#              1- Test the buffer, get the size and separate line
#              2- For each command used, 
#                 get command name and parameters   
#                 call the proper test case 
# Input : buffer (string)
# Output: String                
# =====================================================
sub RunTest($){
 my ($bufferTest)= @_; 
 my $cmd = "";
 my $bufferPosition = 0;
 my $RouteId = 0;

 if($bufferTest ne ""){
    $bufferSize = split(/\n/,$bufferTest);
    if(debug){print "Number of line in buffer = $bufferSize \n";}
    @bufferGlobal = split(/\n/,$bufferTest);
  }
  else{ 
    return "Error: buffer is empty! \n"; 
  }

  print "\nStart testing the buffer\n";
  print "=======================\n";

  while($bufferPosition < $bufferSize){
    $bufferPosition = GetLineNumWithCommand($bufferPosition);      
    if($debug){print "Cursor postion in buffer = $bufferPosition\n";}
    if(!$bufferPosition){last;}
    $cmd = GetCommandAndParameters($bufferPosition); 
    if($cmd eq $cmdNetInfoList){
      TestNetInfoList($bufferPosition);
      %NetInfoList = ();
    }
    elsif($cmd eq $cmdDHNegInit){
      TestDHNegInit($bufferPosition);   
      %DHNegInit = ();
    }
    elsif($cmd eq $cmdDHNegRespond){
      TestDHNegRespond($bufferPosition);
      %DHNegRespond = ();
    }
    elsif($cmd eq $cmdRouteCreate){
      TestRouteCreate($bufferPosition);
      %RouteCreate = ();
    }
    elsif($cmd eq $cmdRouteExtend){
      TestRouteExtend($bufferPosition);
      %RouteExtend = ();
    }
    elsif($cmd eq $cmdRouteAck){
      TestRouteAck($bufferPosition);
      %RouteAck = ();
    }
    elsif($cmd eq $cmdRouteDestroy){
      $RouteId = TestRouteDestroy($bufferPosition,$RouteId);
      %RouteDestroy = ();
    }
    elsif($cmd eq $cmdKeepAlive){
      TestKeepAlive($bufferPosition);
      %KeepAlive = ();
    }
    elsif($cmd eq ""){
      return "Error: Unavailable Command [$cmd] to test! \n";
    }
    $bufferPosition++;
  }
  
  if($debug){PrintAllError();}   

  return "RunTest Successfully!\n";
}


# ============================================================
# function   : PrintAllError()
# Description:1-Search for every lines with error and print it
#             2-Search for every lines with failed and print it
# Input : null
# Output: null
# ============================================================
sub PrintAllError(){  
  my $i = 0;
  my $error = 0;
  my $failed = 0;

  print "\nDUMP all lines with ERROR messages: \n";
  print "=================================== \n";

  for($i = 0; $i < $bufferSize; $i++){
    $_ = $bufferGlobal[$i];
    if(/ERROR/i){ 
      print "line $i:$_\n";
      $error++;
    } 
  }
  print "There's [$error] line(s) with ERROR messages!\n";
 
 print "\nDUMP all lines with fialed messages: \n";
  print "==================================== \n";
  for($i = 0; $i < $bufferSize; $i++){
    $_ = $bufferGlobal[$i];
    if(/failed/i){
      print "line $i:$_\n";
      $failed++;
    }
  }
  print "There's [$failed] line(s)with failed messages!\n";  
}


# =====================================================
# function   : TestNetInfoList
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestNetInfoList($){
  my ($pos)= @_;
  my ($FunctionalAip,@EntityList,$arraySz);
  my ($cont,@ok,$i);
  
  $FunctionalAip = $NetInfoList{'FunctionalAip'};
  @EntityList = ();
  $arraySz = 0; ;

  while($NetInfoList{'EntityList'}[$arraySz]){
   push (@EntityList,$NetInfoList{'EntityList'}[$arraySz]);
   $arraySz++;
  }

  print "-> NetInfoList FunctionalAip=$FunctionalAip EntityList=@EntityList \n";
  $i = 0;
  print "-> EXPECTING : processing $EntityList[$i]\n";

  for($i=1;$i<$arraySz;$i++){
    print "->             processing $EntityList[$i]\n";
  }
  
  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];

    for($i=0 ; $i < $arraySz;$i++){
      if(/processing $EntityList[$i]/i){
        $ok[$i] = 1;
      }
    }

    $cont = 1;

    for($i=0; ($i<$arraySz)&&(1==$cont); $i++){
      if(0 == $ok[$i]){
        $cont = 0;
      }
      if( (1==$ok[$arraySz-1]) && ($i==($arraySz-1)) ){
        print "-> $cmdNetInfoList pass \n";
        if($debug){print"line $pos:$_\n";}
        return 1;
      }
    }
    $pos++;
  }
  print "-> $cmdNetInfoList failed \n"; 
  return 0;        
}


# =====================================================
# function   : TestDHNegInit
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestDHNegInit($){
  my ($pos)= @_;
  my ($EntityName,$Delay,$Action,$RepeatInterval);
  my ($i,$cont);
  my @MBuf = ("EXPECTING","LINK","ADDED");
  my $MSz = 3;
  my $Message = "";

  $bon = 0;
  
  $EntityName = $DHNegInit{'EntityName'};
  $Delay = $DHNegInit{'Delay'};
  $Action = $DHNegInit{'Action'};
  $RepeatInterval = $DHNegInit{'RepeatInterval'};
  $Message = $DHNegInit{'Message'};

  print "-> DHNegInit EntityName=$EntityName Delay=$Delay Action=$Action RepeatInterval=$RepeatInterval \n";

  if($Message ne ""){
    @MBuf = ();
    $MSz = split(/-/,$Message);
    @MBuf = split(/-/,$Message);

  } else{
    push (@MBuf,$EntityName);
    $MSz++;
  }

  print "-> Message(s): @MBuf\n";

  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];

    $cont = 1;
    for($i=1 ;(1==$cont)&&($i<$MSz);$i++){
      if( (/$MBuf[0]/i) || (!/$MBuf[$i]/i)  ){
        $cont = 0;
      }
      if((1==$cont)&&($i==($MSz-1))){
        print "-> $cmdDHNegInit pass \n";
        if($debug){print"line $pos:$_\n";}
        return 1;
      }
    }
    $pos++;
  }  
  print "-> $cmdDHNegInit failed \n"; 
  return 0;      
}


# =====================================================
# function   : TestDHNegRespond
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestDHNegRespond($){
  my ($pos)= @_;
  my ($EntityName,$Delay,$MaxWait,$Action,$Repeat);  
  my ($i,$cont);
  my @MBuf = ("EXPECTING","LINK","ADDED");
  my $MSz = 3;
  my $Message = "";

  $EntityName = $DHNegRespond{'EntityName'}; 
  $Delay = $DHNegRespond{'Delay'};
  $MaxWait = $DHNegRespond{'MaxWait'};
  $Action = $DHNegRespond{'Action'};  
  $Repeat = $DHNegRespond{'Repeat'};
  $Message = $DHNegRespond{'Message'};

  print "-> DHNegRespond EntityName=$EntityName Delay=$Delay MaxWait=$MaxWait Action=$Action Repeat=$Repeat\n";
  
  if($Message ne ""){
    @MBuf = ();
    $MSz = split(/-/,$Message);
    @MBuf = split(/-/,$Message); 

  } else{
    push (@MBuf,$EntityName);
    $MSz++;
  }

  print "-> Message(s): @MBuf\n";

  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];

    $cont = 1;
    for($i=1 ;(1==$cont)&&($i<$MSz);$i++){
      if( (/$MBuf[0]/i) || (!/$MBuf[$i]/i)  ){
        $cont = 0;
      }
      if((1==$cont)&&($i==($MSz-1))){
        print "-> $cmdDHNegRespond pass \n";
        if($debug){print"line $pos:$_\n";}
        return 1;
      }
    }
    $pos++;
  }  
  print "-> $cmdDHNegRespond failed \n";  
  return 0;
}


# =====================================================
# function   : TestRouteCreate
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestRouteCreate($){
  my ($pos)= @_;
  my ($ID,$Nhops,@EntityList,$NymName,$ValidNymSig,$RouteTTL,$Action,$Version);
  my ($ClientEntityName,$Hop1,$Hop2,$Hop3,$AnonymousRoute);
  my ($arraySz,$ok1,$ok2,$aci,$link);
  my @MBuf = ("EXPECTING","received", "RouteCreateAck");
  my $MSz = 3;
  my $Message = "";

  $ClientEntityName = $RouteCreate{'ClientEntityName'};
  $Hop1 = $RouteCreate{'Hop1'};
  $Hop2 = $RouteCreate{'Hop2'};
  $Hop3 = $RouteCreate{'Hop3'};
  $AnonymousRoute = $RouteCreate{'AnonymousRoute'};
  $ID = $RouteCreate{'ID'};
  $Nhops = $RouteCreate{'Nhops'};
  $NymName = $RouteCreate{'NymName'};
  $ValidNymSig = $RouteCreate{'ValidNymSig'};
  $RouteTTL = $RouteCreate{'RouteTTL'};
  $Action = $RouteCreate{'Action'};
  $Version = $RouteCreate{'Version'};
  $arraySz = 0; 
  $Message = $RouteCreate{'Message'};

  while($RouteCreate{'EntityList'}[$arraySz]){
    push (@EntityList,$RouteCreate{'EntityList'}[$arraySz]);
    $arraySz++;
  }

  print "-> RouteCreate ID=$ID Nhops=$Nhops NymName=$NymName ValidNymSig=$ValidNymSig RouteTTL=$RouteTTL Action=$Action Version=$Version EntityList=@EntityList\n";
  print "               ClientEntityName=$ClientEntityName Hop1=$Hop1 Hop2=$Hop2 Hop3=$Hop3 AnonymousRoute=$AnonymousRoute \n";

  if($Message ne ""){
    @MBuf = ();
    $MSz = split(/-/,$Message);
    @MBuf = split(/-/,$Message); 
  } 

  print "-> Message(s): @MBuf\n";

  $ok1=0; $ok2=0;

  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];
   
    if((/adding route id $ID/i) && (!/failed/i) && (!/ERROR/i)){
      m/prevAci (\d+) prevLinkId (\d+)/i;
      $aci = $1;
      $link = $2;
      $ok1 = 1;
      $ok2 = 0;
    }

    $cont = 1;

    for($i=1 ; ($i < $MSz) && (1==$cont);$i++){
      if( (/$MBuf[0]/i) || (!/$MBuf[$i]/i) ){
        $cont = 0;
      }
      if((1==$cont) && ($i == ($MSz -1))){
         if(m/link $link aci $aci/i){
          $ok2 = 1;
         }
      }
    }
    if($ok1 && $ok2 ){
      print "-> $cmdRouteCreate pass \n";
      if($debug){print"line $pos:$_\n";}
      return 1;
    }
    $pos++;
  }
  print "-> $cmdRouteCreate failed \n";
  return 0;
}


# =====================================================
# function   : TestRouteExtend
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestRouteExtend($){
  my ($pos)= @_;
  my ($ID,$Nhops,@EntityList,$NymName,$ValidNymSig,$RouteTTL,$Action,$ExitAci);
  my ($ClientEntityName,$Hop1,$Hop2,$Hop3,$AnonymousRoute);  
  my ($arraySz,$ok1,$ok2,$aci,$link,$list,$str,$cont);
  my @MBuf = ("EXPECTING","received", "RouteCreateAck");
  my $MSz = 3;
  my $Message = "";

  $ClientEntityName = $RouteExtend{'ClientEntityName'};
  $Hop1 = $RouteExtend{'Hop1'};
  $Hop2 = $RouteExtend{'Hop2'};
  $Hop3 = $RouteExtend{'Hop3'};
  $AnonymousRoute = $RouteExtend{'AnonymousRoute'};
  $ID = $RouteExtend{'ID'};
  $Nhops = $RouteExtend{'Nhops'};
  $NymName = $RouteExtend{'NymName'};
  $ValidNymSig = $RouteExtend{'ValidNymSig'};
  $RouteTTL = $RouteExtend{'RouteTTL'};
  $Action = $RouteExtend{'Action'};
  $ExitAci = $RouteExtend{'ExitAci'};
  $arraySz = 0;
  $Message = $RouteExtend{'Message'};
  
  while($RouteExtend{'EntityList'}[$arraySz]){
    push (@EntityList,$RouteExtend{'EntityList'}[$arraySz]);
    $arraySz++;
  }

  print "-> RouteExtend ID=$ID Nhops=$Nhops NymName=$NymName ValidNymSig=$ValidNymSig RouteTTL=$RouteTTL Action=$Action ExitAci=$ExitAci EntityList=@EntityList\n";
  print "               ClientEntityName=$ClientEntityName Hop1=$Hop1 Hop2=$Hop2 Hop3=$Hop3 AnonymousRoute=$AnonymousRoute \n";

  if($Message ne ""){
    @MBuf = ();
    $MSz = split(/-/,$Message);
    @MBuf = split(/-/,$Message);
  }

  print "-> Message(s): @MBuf\n";

  $ok1=0; $ok2=0;
  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];
    
    if((/adding route id $ID/i) && (!/failed/i) && (!/ERROR/i)){
      m/prevAci (\d+) prevLinkId (\d+)/i;
      $aci = $1;
      $link = $2;
      $ok1 = 1;
      $ok2 = 0;
    }

    $cont = 1;

    for($i=1 ; ($i < $MSz) && (1==$cont);$i++){
      if( (/$MBuf[0]/i) || (!/$MBuf[$i]/i) ){
        $cont = 0;
      }
      if(($cont)&&($i == ($MSz -1))){
         if(m/link $link aci $aci/i){         
          $ok2 = 1;
         }
      }
    }
    if($ok1 && $ok2 ){
      print "-> $cmdRouteExtend pass \n";
      if($debug){print"line $pos:$_\n";}
      return 1;
    }
    $pos++;
  }
  print "-> $cmdRouteExtend failed \n";
  return 0;
}


# =====================================================
# function   : TestRouteAck
# description: Not yet implement 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestRouteAck($){
  my ($pos)= @_;
  my ($ID,$NextAci,$ExitAci,$ValidateRoute,$Version);
  
  $ID = $RouteAck{'ID'};
  $NextAci = $RouteAck{'NextAci'};
  $ExitAci = $RouteAck{'NextAci'};
  $ValidateRoute = $RouteAck{'ValidateRoute'};
  $Version = $RouteAck{'Version'};

  print "-> RouteAck ID=$ID NextAci=$NextAci ExitAci=$ExitAci ValidateRoute=$ValidateRoute Version=$Version\n";
 
  print "Not implement in the script yet!\n";

  return 0;
}


# =====================================================
# function   : TestRouteDestroy
# description: 
# Input : int (position in the buffer)
# Output: int (routeId)
# =====================================================
sub TestRouteDestroy($$){
  my ($pos,$RouteId)= @_;
  my ($ID,$NextAci,$ErrorCode,$EntityName);
  my ($i,$cont,$funcRouteId,$IdTmp,$ok);
  my @MBuf = ("EXPECTING","ROUTE","REMOVED");
  my $MSz = 3;
  my $Message = "";
 
  $ID = $RouteDestroy{'ID'};
  $NextAci = $RouteDestroy{'NextAci'};
  $ErrorCode = $RouteDestroy{'ErrorCode'};
  $EntityName = $RouteDestroy{'EntityName'};
  $Message = $RouteDestroy{'Message'};
 
  print "-> RouteDestroy ID=$ID NextAci=$NextAci ErrorCode=$ErrorCode EntityName=$EntityName \n"; 

  if($Message ne ""){
    @MBuf = ();
    $MSz = split(/-/,$Message);
    @MBuf = split(/-/,$Message);
  }

  print "-> Message(s): @MBuf\n";
    
  $ok = 1;
  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];   

    #search for route added and take the ID
    if((/ROUTE ADDED/i) && ($ok) && (!/failed/i) && (!/ERROR/i)){
      m/RouteId (\d+)/i;
      $funcRouteId = $1;
    }

    if($funcRouteId > $RouteId){ #Suppose the ID always Increment! (migth not be good)
      $cont = 1;
      $ok = 0;
    }
 
    for($i=1 ; (1==$cont) && ($i < $MSz);$i++){
      if( (/$MBuf[0]/i) || (!/$MBuf[$i]/i) ){       
        $cont = 0;
      }
      if((1==$cont) && ($i==($MSz-1))){
         m/RouteId (\d+)/i;
         $IdTmp = $1;
         if($IdTmp == $funcRouteId){
           print "-> $cmdRouteDestroy pass (RouteId = $IdTmp) \n";
           if($debug){print "-> Add RouteId=$funcRouteId <-> Remove RouteId=$IdTmp \n";}
           if($debug){print"line $pos:$_\n";}
           return $funcRouteId;          
         }
         else{
           $ok = 1;
         }
      }         
    }
    $pos++;
  }
  print "-> $cmdRouteDestroy failed (RouteId = $IdTmp) \n"; 
  if($debug){print "-> Add RouteId=$funcRouteId <-> Remove RouteId=$IdTmp \n";}
  return 0;
}


# =====================================================
# function   : TestKeepAlive
# description: 
# Input  : int (position in the buffer)
# Output : 0 (error) or 1 (sucess)
# =====================================================
sub TestKeepAlive($){
  my ($pos)= @_;
  my ($EntityName,$RepeatInterval,$Message);
  my ($link);
#  my @MBuf = ("EXPECTING","received", "keepalive");
#  my $MSz = 3;

  $EntityName = $KeepAlive{'EntityName'};
  $RepeatInterval = $KeepAlive{'RepeatInterval'};
#  $Message = $KeepAlive{'Message'};

  print "-> KeepAlive EntityName=$EntityName RepeatInterval=$RepeatInterval \n";
  
#  if($Message){
#    @MBuf = ();
#    $MSz = split(/-/,$Message);
#    @MBuf = split(/-/,$Message);
#  }
#   print "-> Message(s): @MBuf\n";

  $ok1=0; $ok2=0;
  while($pos < $bufferSize){
    $_ = $bufferGlobal[$pos];
    if((/LINK ADDED for $EntityName/i) && (!/failed/i) && (!/ERROR/i)){
      m/ LINKID (\d+)/i;   
      $link = $1;
      $ok1 = 1;
      $ok2 = 0;
    }
    if((/received keepalive/i) && (/link $link/i) &&  (!/failed/i) && (!/ERROR/i)){
      $ok2 = 1;
    }
    if($ok1 && $ok2 ){
      print "-> $cmdKeepAlive pass \n";
      if($debug){print"line $pos:$_\n";}
      return 1;
    }
    $pos++;
  }
  print "-> $cmdKeepAlive failed \n";
  return 0;
}


# ============================================================
# function   : GetLineNumWithCommand
# Description: Return the buffer line number which contained a 
#              command (any type of command).
# Input  : int (the current line (position) in the buffer)
# Output : int (the position (line) of the command)
# ==============================================================
sub GetLineNumWithCommand($){
  my ($i) = @_; 
  my ($bufferSz,@buffer);
  $bufferSz = $bufferSize;
  @buffer = @bufferGlobal;
 
  for($i; $i<=$bufferSz;$i++ ){   
    $_ = $buffer[$i]; 
    if(/$cmdNetInfoList/){
      print "\nValidating $cmdNetInfoList \n";
      print "=========================== \n";
      return $i;            
    }
    elsif(/$cmdDHNegInit/){
      print "\nValidating $cmdDHNegInit \n";
      print "========================= \n";
      return $i;     
    }
    elsif(/$cmdDHNegRespond/){
      print "\nValidating $cmdDHNegRespond \n";
      print "============================ \n";
      return $i;     
    }
    elsif(/$cmdRouteCreate/){
      print "\nValidating $cmdRouteCreate \n";
      print "=========================== \n";
      return $i;     
    }
    elsif(/$cmdRouteExtend/){
      print "\nValidating $cmdRouteExtend \n";
      print "=========================== \n";
      return $i;           
    }
    elsif(/$cmdRouteAck/){
      print "\nValidating $cmdRouteAck \n";
      print "======================== \n";
      return $i;     
    }
    elsif(/$cmdRouteDestroy/){
      print "\nValidating $cmdRouteDestroy \n";
      print "============================ \n";
      return $i;     
    }
    elsif(/$cmdKeepAlive/){
      print "\nValidating $cmdKeepAlive \n";
      print "========================= \n";
      return $i;     
    }
  }   
  return 0;    
}


# =====================================================
# function : GetCommandAndParameters
# Description : Get the parameters of a commands
# Input : int (position of the commands)
# Output : string ("" or command name)
# =====================================================
sub GetCommandAndParameters($){
  my ($i) = @_;
  my (@buffer,@tempArray);
  my ($ok, $ok1, $var, $val,$qual,$dump,$time);
 
  $ok = 1; $ok1 = 1; $val = ""; $cmd ="";
  $val = "val"; $qual = "qual"; $time = "time";
  @tempArray = ();
  @buffer = @bufferGlobal;

  for($i; $i<=$bufferSize;$i++ ){   

    $_ = $buffer[$i]; 
    $i++;
    if(/$cmdNetInfoList/){
      while($ok){          
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
          ($dump,$_) = split(/ /,$_,2);
          $dump =~ s/\s+//g;
        }
	$var = $_;
	$var =~ s/\s+//g;       
        if($dump eq $time){
   	  $NetInfoList{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
          $i++;
	}
        elsif($dump eq $qual){      
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
            if(($dump eq $val) && ($var eq "EntityList")){            
              push(@tempArray,$_);
            }       
	    elsif($var eq "EntityList"){
              $NetInfoList{$var} = \@tempArray;
              if($debug){ print "$var = @tempArray \n"; }
              $ok1=0;
            }
	    elsif($dump eq $val){
              s/\s+//g;
	      $NetInfoList{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdNetInfoList;
    }
    elsif(/$cmdDHNegInit/){
      while($ok){        
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
            ($dump,$_) = split(/ /,$_,2);
            $dump =~ s/\s+//g;
        }
	$var = $_;
	$var =~ s/\s+//g;       
        if($dump eq $time){
   	  $DHNegInit{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
	    if($dump eq $val){
              s/\s+//g;
	      $DHNegInit{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdDHNegInit;
    }
    elsif(/$cmdDHNegRespond/){
      while($ok){        
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
            ($dump,$_) = split(/ /,$_,2);
            $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;         
        if($dump eq $time){
   	  $DHNegRespond{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
	    if($dump eq $val){
              s/\s+//g;
	      $DHNegRespond{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdDHNegRespond;  
    }
    elsif(/$cmdRouteCreate/){
      while($ok){
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
          ($dump,$_) = split(/ /,$_,2);
          $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;         
        if($dump eq $time){
   	  $RouteCreate{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
            if(($dump eq $val) && ($var eq "EntityList")){            
              push(@tempArray,$_);
            }       
	    elsif($var eq "EntityList"){
              $RouteCreate{$var} = \@tempArray;
              if($debug){ print "$var = @tempArray \n"; }
              $ok1=0;
            }
	    elsif($dump eq $val){
              s/\s+//g;
	      $RouteCreate{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }   
      return $cmdRouteCreate;
    }
    elsif(/$cmdRouteExtend/){
      while($ok){
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
          ($dump,$_) = split(/ /,$_,2);
          $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;         
        if($dump eq $time){
   	  $RouteExtend{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
            if(($dump eq $val) && ($var eq "EntityList")){            
              push(@tempArray,$_);
            }       
	    elsif($var eq "EntityList"){
              $RouteExtend{$var} = \@tempArray;
              if($debug){ print "$var = @tempArray \n"; }
              $ok1=0;
            }
	    elsif($dump eq $val){
              s/\s+//g;
	      $RouteExtend{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }   
      return $cmdRouteExtend;
    }
    elsif(/$cmdRouteAck/){
      while($ok){
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
            ($dump,$_) = split(/ /,$_,2);
            $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;      
	if($dump eq $time){
   	  $RouteAck{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
	  $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
	    if($dump eq $val){
              s/\s+//g;
	      $RouteAck{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdRouteAck;
    }
    elsif(/$cmdRouteDestroy/){
       while($ok){     
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
          ($dump,$_) = split(/ /,$_,2);
          $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;      
	if($dump eq $time){
   	  $RouteDestroy{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
	    if($dump eq $val){
              s/\s+//g;
	      $RouteDestroy{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 =0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdRouteDestroy;
    }
    elsif(/$cmdKeepAlive/){
      while($ok){
        $_ = $buffer[$i];    
        $dump="";$var="";
        while(($dump eq "") && ($_ ne "")){
          ($dump,$_) = split(/ /,$_,2);
          $dump =~ s/\s+//g;
        }
        $var = $_;
        $var =~ s/\s+//g;      
	if($dump eq $time){
   	  $KeepAlive{$dump} = $var;
          if($debug){ print "$dump = $var \n"; }
	  $i++;
	}
        elsif($dump eq $qual){
          $ok1 = 1;
          @tempArray = ();
          while($ok1){
            $i++;
            $_ = $buffer[$i];
            $dump="";
            while(($dump eq "") && ($_ ne "")){
              ($dump,$_) = split(/ /,$_,2);
              $dump =~ s/\s+//g;       
            }
	    if($dump eq $val){
              s/\s+//g;
	      $KeepAlive{$var} = $_;
              if($debug){ print "$var = $_ \n"; }
            }
            else{
	      $ok1 = 0;
	    }  
          }        
	}
        else{
	  $ok = 0;
        }
      }  
      return $cmdKeepAlive;
    }
  }       
  return "";
}


# =============================================================
# function   : RunProg
# Description: Run the testFunctionalIntegrationAip and if 
#              success then run functionalIntegrationAip
# Input : file (the file input for testFunctionalIntegrationAip)
# Output: string (buffer or "" (no character))
# ==============================================================
sub RunProg($){
  my ($file) = @_;
  my ($FDIN,$FDOUT,$FDIN2,$FDOUT2);
  my ($output, $ct, $pidTestFunc, $pidFunc, $cmdTest, $cmd,$ok);
  my $buff = "";

  $output = ""; $ct = 0; $ok = 1; 
  $pidTestFunc = 0; $pidFunc = 0; 
  $cmdTest = $testFuncIntAip." ".$file;
  $cmd = $funcIntAip;
  $FDIN  = new FileHandle;
  $FDOUT = new FileHandle;  
  $FDIN2  = new FileHandle;
  $FDOUT2 = new FileHandle;

  $pidTestFunc = open3($FDOUT, $FDIN, $FDIN, "$cmdTest");
  print "Start [$cmdTest]\n";
  print "<**************************************** \n";
  
  $FDIN->autoflush();
  $FDOUT->autoflush();

  while($_ = $FDIN->getline()){   
    $output .= $_;       
    if($_ != /startup complete/i){                 
      $pidFunc = open3($FDOUT2, $FDIN2, $FDIN2, "$cmd");
      close($FDOUT2);
      close($FDIN2);
      print "\nStart [$cmd]\n";
      print "<**************************************** \n";
      if($pidFunc == 0){
      print "Error: Unable to start functionalIntegrationAip!\n";
      last;
      }
    }
    elsif( ($_ !~ /startup complete/i) && (2 == $ct) ){
      if($pidTestFunc != 0){`kill -9 $pidTestFunc`;}
      if($pidFunc != 0){`kill -9 $pidFunc`;}
      if($debug){ print "$_"; }          
      print "\nError: Unable to start testFunctionalIntegrationAip!\n"; 
      $ok = 0;
      last;
    }
    if($debug){    print "$_";    }
    $ct++;
  }
  print "\n"; 
  close($FDOUT);
  close($FDIN);

  if($pidTestFunc != 0){ `kill -9 $pidTestFunc`; }

  print "End [$cmdTest]\n";
  print "****************************************> \n";
   
  if($pidFunc != 0){ 
    `kill -9 $pidFunc`; 
     print "End [$cmd]\n";
     print "****************************************> \n";
  }
   
  if(1 == $ok){
    return $output;
  }else{
    return "";
  }    
}


# ===============================================================
# function   : ValidatePorg
# Description: Verify the executions files (programs) are present
# Input : null
# Output: 0 (error) or 1 (success)
# ===============================================================
sub ValidateProg{
  my $i = 0;

  print "\nValidating program files \n";
  print "========================= \n";

  if($debug){print "  validating [$funcIntAip]\n"}
  if(!(lstat $funcIntAip)){
    print "Error: file do not exist [$funcIntAip]!\n";
    if($debug){print "ZK_ROOT = $ENV{'ZK_ROOT'}\n"; }
    if(!$ENV{'ZK_ROOT'}){    
      print "ZK_ROOT migth be NULL, try to specify the working directory!\n";
    }   
    return 0;
  }
  if($debug){print "  validating [$testFuncIntAip]\n"}
  if(!(lstat $testFuncIntAip)){
    print "Error: file do not exist [$testFuncIntAip]!\n";
    return 0;
  } 

  #configuration file to run testFunctionalIntegrationAip and functionalIntegrationAip
  if($debug){print "  validating [$configFile]\n"}
  if(!(lstat $configFile)){
    print "Error: file do not exist [$configFile]!\n";
    return 0;
  }
  print "\n";
  return 1;
}


# =====================================================
# function   : ValidateFiles
# Description: Verify all the inputs files for testFunctionalInegrationAip
# Input : null
# Output: 0 (error) or 1 (success)
# =====================================================
sub ValidateFiles{
  my $i = 0;

  print "Validating input files for testFunctionalIntegrationAip\n";
  print "======================================================= \n";

  for( $i=0 ; $fileInput[$i]; $i++){
    if($debug){print "  validating [$fileInput[$i]]\n"}
    if(!(lstat $fileInput[$i])){
      print "Error: file do not exist [$fileInput[$i]]!\n";
      return 0;
    }
  }
  print "\n";
  return 1;
}


# =====================================================
# function   : getCommandLineConfig
# Description: Get the parameters
# =====================================================
sub getCommandLineConfig
{
  my $res;
  my %optctl  = ();

  my @helpString =
  (
   "\nThis script is to test the AIP functional integrety",
   "\nUsage: $0 [--version] [--help|-h] [--debug] [--load <file>]",
   "\t                             [--file <fileName>] [--dir </path/dirName>]",
   "\t--version        : version information.",
   "\t--help           : this help.",
   "\t--debug          : Debug mode.",
   "\t--load  <file>   : Load a buffer from a file.",
   "\t                   Note: testfunctionalIntegrationAip and",
   "\t                   functionalIntegrationAip will not be run",
   "\t--file  <name>   : Specify the input file to run with testFunctionalIntegrationAip.",
   "\t                   Note: the default list of input files won't be used",
   "\t--dir </path/dir>: Specify the working directory.",
   "\t                   Note: this script need to be in the same directory as",
   "\t                   functionalIntegrationAip, testFunctionalIntegrationAip",
   "\t                   and their configuration file.\n"
  );

  ## Get the Options
  ##
  $res = GetOptions(\%optctl,
                  "version",
                  "help",                  
                  "debug",
                  "load=s",
                  "file=s",
                  "dir=s",                  
                 ) || do {
                       print STDOUT "\n****Error in get options (getOpt)\n",
                         join("\n", @helpString),
                         "\n";
                       exit 1;
                      };

  ##
  ## print out the usage string
  ##
  if (defined($optctl{'version'})){
    print  STDOUT "$version\n";
    exit 1;
  }

  if (defined($optctl{'help'})){
    print  STDOUT join("\n", @helpString),"\n";
    exit 1;
  }

  if (defined($optctl{'debug'})){
    $debug = 1;
  } 

  if (defined($optctl{'load'})){    
    $fileBuffer= $optctl{'load'};
  } 

  if(defined($optctl{'file'})){
    @fileInput=();
    $file= $optctl{'file'};    
    push(@fileInput,$file);    
  }

  if(defined($optctl{'dir'})){    
    $dir= $optctl{'dir'};
    $funcIntAip     = $dir."/functionalIntegrationAip";
    $testFuncIntAip = $dir."/testFunctionalIntegrationAip";
    $configFile     = $dir."/test.conf"; 
  }

}


# =================================================================
# function   : GetBufferFromFile
# Description: Open a log file and put contents in a buffer
# Input : string (file name)
# Output: string (buffer)
# ==================================================================
sub GetBufferFromFile($){
  my ($file) = @_;
  my $myBuffer = "";
  if(open _F,"<$file") {
    while(<_F>){
      $myBuffer .= $_;
    } 
    close(_F);
  }
  else{
    die "\nError! Unable to open the config file [$file] !\n\n";
  }
  return $myBuffer;
}
