#define REV_ID "@(#) $Id: setupRouteCreate.cpp,v 1.2 2001/04/18 17:38:03 alainc Exp $"
#include <zkrev.h>
#include <iostream>
#include <fstream>
#include "zksexception.h"
#include <zks.h>
#include "zkserr.h"
#include "zkError.h"
#include "zkIntTypes.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>
#include <zksini.h>
#include <zkscmdline.h>
#include <zkSignal.h>

#include "frCrypt.h"
#include "frPktFmt.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"

#include "cKeepAlivePkt.h"
#include "cRouteCreatePkt.h"
#include "cRouteCreateAckPkt.h"
#include "cRouteDestroyPkt.h"

#include "cConfigProvider.h"
#include "cPrivateKeys.h"
#include "cAipComponentManager.h"
#include "cRouteCreate.h"

#define NUM_ROUTES_MAX 5
#if 1
#define CRYPT_ALG_0 FRCRYPT_SYMALG_BLOWFISH
#define CRYPT_ALG_1 FRCRYPT_SYMALG_BLOWFISH
#define CRYPT_ALG_2 FRCRYPT_SYMALG_BLOWFISH
#else
#define CRYPT_ALG_0 FRCRYPT_SYMALG_XOR
#define CRYPT_ALG_1 FRCRYPT_SYMALG_XOR
#define CRYPT_ALG_2 FRCRYPT_SYMALG_XOR
#endif
#define UINT16_MAX      (65535U)

cPrivateKeys* pPrivateKeys;
cAipComponentManager mgr;
FrCryptSig nymSig;
FrCryptMacKey sndMacKey[NUM_ROUTES_MAX];

FrCryptSym sndBckCrypt[NUM_ROUTES_MAX], sndFwdCrypt[NUM_ROUTES_MAX];

CZKS::Ini            m_ini;
CZKS::CmdLine        m_cmdline;   // command line parser

static void setRouteN(ZkError err, struct fr_rte_set_exp *r, unsigned int n, int nhops, FrEntIdExp entity, uint32_t ttl, bool specAci, int wormAci)
{
  r->rteAlg = CRYPT_ALG_0;
  
  assert(frCryptSymNew(err, &r->bckCrypt));
  assert(frCryptSymSetAlg(err, r->bckCrypt, CRYPT_ALG_1));
  
  assert(frCryptSymNew(err, &r->fwdCrypt));
  assert(frCryptSymSetAlg(err, r->fwdCrypt, CRYPT_ALG_2));
  
  assert(frCryptMacKeyNew(err, &r->macKey));
  r->hostEnt = entity;

  struct timeval now;
  gettimeofday(&now, 0);
  r->expTime = now.tv_sec+ttl;
  r->flags = n == nhops - 1 ? FRPKT_RTECRT_FL_LAST_RTE : 0;
  if (specAci) {
    r->flags |= FRPKT_RTECRT_FL_SPEC_ACI;
    r->specAci = wormAci;
  } else {
    r->specAci = 0;
  }
}

void setPrivateKeys(czkCoreLoop* coreloop)
{
        std::ifstream istrm("./test.conf");    
        if( !istrm.is_open() ) {
          CZKS::Throw(ZKS_ROUTECREATOR_MOD,
                      1,
                      "FunctionalAip",
                      "Failed opening file: ./test.conf");
        }


        //
        // Load Ini object with the configuration file
        //
        istrm >> m_ini;

        // start cConfigProvider
        cConfigProvider* pConfigProvider = new cConfigProvider(m_cmdline, m_ini);

  pPrivateKeys = new cPrivateKeys();
  mgr.add(ZKS_PRIVATEKEYS_MOD, pPrivateKeys);
  pPrivateKeys->validateConfig();
  
  mgr.initAllComponents(coreloop);

  mgr.startComponent(ZKS_PRIVATEKEYS_MOD, false);
}


cPrivateKeys* getPrivateKeys()
{
  return pPrivateKeys;
}

void sendNymSig(czkSocket* stream)
{
  char buffer[BUFSIZ];

  CZKS::Exception e;

    assert(frCryptSigNew(e.zkError(), &nymSig));
    assert(frCryptPubKeySetVersion(e.zkError(), nymSig, 4)); 
    assert(frCryptSigSetKeySize(e.zkError(), nymSig, FRCRYPT_SIGKEYSIZE_1024));
    assert(frCryptSigSetRandomKeys(e.zkError(), nymSig));

    size_t dsz = BUFSIZ-1;
  uint8_t command = 1;
  memcpy( buffer, &command, sizeof(command));  
  assert(frCryptSigStore(e.zkError(), nymSig, FRCRYPT_SIGPARMS_ALL, &buffer[1],                         &dsz));

  czkIntervalTime interval;
  stream->send( buffer, 1+dsz, 0, interval.getSeconds(0)); 

}

FrCryptMacKey& getMacKey(int i)
{
  return sndMacKey[i];
}

FrCryptSym& getFwdSym(int i)
{
  return sndFwdCrypt[i];
}

FrCryptSym& getBwdSym(int i)
{
  return sndBckCrypt[i];
}

void setupRouteCreatePkt(ZkError err, int nhops, 
                         FrEntIdExp* hop, bool fixedAci, FrAci aci, 
                         char* nymName, bool validNymSig, uint32_t ttl,
                         uint32_t version, bool specAci, int wormAci, 
                         RouteAuth auth, uint32_t anonRand,
                         uint8_t** retbuf, size_t *retsize, 
                         FrCryptCrypt* retkey, FrCryptSig* retnymsig,
                         FrAci* retAci)
{
  FrCryptCrypt aipKey[NUM_ROUTES_MAX];
  FrCryptSym sym;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;
  int i;
  size_t pbsz;
  struct fr_nym_auth_exp na0;
  uint8_t *pbuf;
  size_t maxsize;

  /*
   *  Setup.
   */
  
  assert(frCryptSymNew(err, &sym));
  assert(frCryptSymSetAlg(err, sym, alg));
  assert(frPktAipRteCrtGetMaxSize(err, 1, &maxsize));

  cRouteCreatePkt rc;
  rc.create( version, 1); 
  
  /* Create public/private encryption keys for all AIPs along the route. */
  for (i = 0; i < nhops; i++) {
    pPrivateKeys->getCryptKey( 0, &aipKey[i]);
    retkey[i]  = aipKey[i];
    //assert(frCryptCryptNew(err, &aipKey[i]));
    //assert(frCryptPubKeySetVersion(err, aipKey[i], 1 + i % 7));
    //assert(frCryptCryptSetKeySize(err, aipKey[i], FRCRYPT_CRYPTKEYSIZE_1024));
    //assert(frCryptCryptSetRandomKeys(err, aipKey[i]));
  }

  /*
   *  Create the packet.
   */

  FrAci aci0;
  if ( fixedAci) {
    aci0 = aci;
  }
  else {
    aci0 = (rand() % UINT16_MAX);
  }
  FrAci aci1 = 0;

  rc.formatForwardHop( aci0);
  aci1 = rc.getAci();
  assert(aci0 == aci1);
  
  /* Append routes. */
  for (i = 0; i < nhops; i++) {
    struct fr_rte_set_exp r;

    memset(&r, 99, sizeof r);
    FrEntIdExp ent;
    if (i < nhops-1) {
      ent = hop[i+1];
    }
    else {
      memset(ent.name, 0, sizeof(ent.name));
      if (auth == ANON) {
	ent.type = FRENT_TYPE_ANON;
      }
      else if (auth == ANONRAND) {
	ent.type = FRENT_TYPE_ANONRAND;
        memcpy( ent.name, &anonRand, sizeof(anonRand));
      }
      else {
        ent.type = FRENT_TYPE_AIP;
      }
    }
    setRouteN(err, &r, i, nhops, ent, ttl, specAci, wormAci);

    rc.appendRouteExp( aipKey[i], &r);

    /* 
     *  Save the backward and forward cryption objects, and the MAC key, for
     *  later testing.
     */

    sndBckCrypt[i] = r.bckCrypt; r.bckCrypt = 0;
    sndFwdCrypt[i] = r.fwdCrypt; r.fwdCrypt = 0;
    sndMacKey[i] = r.macKey; r.macKey = 0;
  }


  /*
   *  bug: for now use the same key, containing both public and private parts,
   *  for signing and authentication.  We ought to put the two parts into
   *  separate objects.
   */

  /* Append the nym, then sign. */
  assert(frEntSetFromStr2(err, &na0.nymID, "nym", nymName));
  if (validNymSig) {
    na0.nymSig = nymSig;
  }
  else {
    FrCryptSig nymSig2;
    assert(frCryptSigNew(err, &nymSig2));
    assert(frCryptPubKeySetVersion(err, nymSig2, 4)); 
    assert(frCryptSigSetKeySize(err, nymSig2, FRCRYPT_SIGKEYSIZE_1024));
    assert(frCryptSigSetRandomKeys(err, nymSig2));
    na0.nymSig = nymSig2;
  }
  rc.appendNymAuthExp( &na0);
 
  rc.getBuffer( &pbuf, &pbsz);

  *retsize = pbsz;
  *retbuf = (uint8_t*)malloc(pbsz);
  memcpy(*retbuf, pbuf, pbsz);

  
  *retnymsig = nymSig;
  *retAci = aci0;
}




