#define REV_ID "@(#) $Id: cDHNegotiatorStub.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "frEnt.h"
#include "frCrypt.h"
#include "zkPrdefs.h"
#include "czkSocket.h"
#include "czkEventHandlerData.h"
#include "cDHNegotiatorResultHandler.h"
#include "cDHNegotiator.h"
#include "commonUtils.h"


DHNegoResult dhResult = DHNegoPass;
uint8_t      dhKeyVersion = 0;


typedef std::map<DHNegoType, cDHNegotiatorResultHandler *> RHMap;
RHMap rhMap;

void doDH(DHInfo & dhInfo)
{
    // TODO: put "aip1" and "aip2" defines somewhere global
    char *encBuf, *decBuf;
    SymKey encrKey;
    SymKey decrKey;

    if( strcmp( dhInfo.aipInfo.entity.name, "aip1" ) == 0 ) {
      encBuf = keyAip1_enc;
      decBuf = keyAip1_dec;
    }
    else if( strcmp( dhInfo.aipInfo.entity.name, "aip2" ) == 0 ) {
      encBuf = keyAip2_enc;
      decBuf = keyAip2_dec;
    }
    else
      assert(0);

    // "enc" and "buf" being from the fake aips' point of view.
    makeKeyFromBuf(decBuf, &encrKey);
    makeKeyFromBuf(encBuf, &decrKey);
    //    generateSymKey(&encrKey, dhInfo.lekv);
    //    generateSymKey(&decrKey, dhInfo.lekv);
    
    (rhMap.find(initiatorFirstAuthenticated))->second->reportResult(
        dhInfo.aipInfo.entity,
        DHNegoPass,
        dhInfo.lekv,
        encrKey,
        decrKey,
        &dhInfo.aipInfo.addr);
}


class cDHEventData : public czkEventHandlerData
{
public:
    FrEntIdExp                   entity;
    ZKPRNetAddr                  addr;
    cDHNegotiatorResultHandler * rh;
    uint8_t                      lekv;
};

class cDHEventHandler : public czkEventHandler
{
    void handleInput(czkSocket *, czkEventHandlerData *) { assert(0); };
    void handleOutput(czkSocket *, czkEventHandlerData *) { assert(0); };
    void handleException(czkSocket *, czkEventHandlerData *) { assert(0); };
    int32_t handleTimeout(czkEventHandlerData *);
};

static cDHEventHandler testDHEh;

int32_t cDHEventHandler::handleTimeout(czkEventHandlerData * data)
{
    assert(data != NULL);
    cDHEventData * info = (cDHEventData *)data;
    
    // call result handler
    if (dhResult == DHNegoPass)
    {

      FrCryptSym encrKey;
      FrCryptSym decrKey;

      char *encBuf, *decBuf;

      if( strcmp( info->entity.name, "aip1" ) == 0 ) {
	encBuf = keyAip1_enc;
	decBuf = keyAip1_dec;
      }
      else if( strcmp( info->entity.name, "aip2" ) == 0 ) {
	encBuf = keyAip2_enc;
	decBuf = keyAip2_dec;
      }
      else
	assert(0);

      // "enc" and "buf" being from the fake aips' point of view.
      makeKeyFromBuf(decBuf, &encrKey);
      makeKeyFromBuf(encBuf, &decrKey);


	// generateSymKey(&encrKey, dhKeyVersion);
        // generateSymKey(&decrKey, dhKeyVersion);

      info->rh->reportResult(info->entity, DHNegoPass, info->lekv, 
			     encrKey, decrKey, &(info->addr));
    }
    else
    {
        info->rh->reportResult(info->entity, DHNegoFail, 0, NULL, NULL, NULL);
    }
    
    return 0;
}

cDHNegotiator::cDHNegotiator()
{
}

cDHNegotiator::~cDHNegotiator()
{
}

bool cDHNegotiator::validateConfig(cConfigProvider*)
{
    return true;
}

bool cDHNegotiator::initializePrivateData(czkCoreLoop *pCoreLoop)
{
  coreloop = pCoreLoop;
  return true;
}
 
void cDHNegotiator::startup()
{
}

void cDHNegotiator::shutdown()
{
}

void cDHNegotiator::registerHandler(cDHNegotiatorResultHandler * rhin)
{
    assert(rhin != NULL);

    DHNegoType type = rhin->getType();

    assert((type == initiatorFirstAuthenticated) || 
           (type == recipientFirstUnauthenticated));

    rhMap[type] = rhin;
}

void cDHNegotiator::removeHandler(cDHNegotiatorResultHandler * rhin)
{
    assert(rhin != NULL);

    DHNegoType type = rhin->getType();

    assert((type == initiatorFirstAuthenticated) || 
           (type == recipientFirstUnauthenticated));

    rhMap.erase(rhMap.find(type));
}

void cDHNegotiator::requestNegotiation(FrEntIdExp & entity, 
                                       ZKPRNetAddr & peerAddr,   
                                       cDHNegotiatorResultHandler * rh,
                                       uint8_t lekv)
{
    // Format timeout data
    cDHEventData * data = new cDHEventData;
    data->entity = entity;
    data->addr = peerAddr;
    data->rh = rh;
    data->lekv = lekv;

    // schedule timeout to send result
    czkIntervalTime  interval;
    ZKPRIntervalTime timeout = interval.getMilliseconds(250); // call rh 250 msec later
    coreloop->scheduleTimer(&testDHEh, data, timeout);
}
