#define REV_ID "@(#) $Id: cFakeAip.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
// blah
#include <assert.h>
#include "frPktWrap.h"
#include "cKeepAlivePkt.h"
#include "zksexception.h"
#include "commonUtils.h"
#include "cRouteCreatePkt.h"
#include "cKeyStore.h"
#include "cFakeAip.h"

// TODO: make a hash somewhere containing keys indexed by entity name

cFakeAip::cFakeAip(cKeyStore & keyStoreIn, 
                                 AipInfo & aipInfo)
{
    keyStore = &keyStoreIn;

    addr = aipInfo.addr;
    memcpy(&entity, &aipInfo.entity, sizeof(entity));
}

cFakeAip::~cFakeAip()
{
}

void cFakeAip::init()
{
    assert(sock.open() == ZKPR_SUCCESS);
    assert(sock.bind(addr) == ZKPR_SUCCESS);
}

void cFakeAip::sendKeepalive(AipInfo & aipInfo, uint8_t lekv)
{
  CZKS::Exception e;

  char *encBuf, *decBuf;

  if( strcmp( entity.name, "aip1" ) == 0 ) {
    encBuf = keyAip1_enc;
  }
  else if( strcmp( entity.name, "aip2" ) == 0 ) {
    encBuf = keyAip2_enc;
  }
  else
    assert(0);

      // "enc" and "buf" being from the fake aips' point of view.
  FrCryptSym symKey;
  makeKeyFromBuf(encBuf, &symKey);
    
  //  assert(lekv <= 1);
    
  // generate keepalive packet
  cKeepAlivePkt ka;
  ka.create(FRPKT_DATA_VERSION_1, FRPKT_PRIORITY_DEFAULT);
    
    // get link encryption key
  //  FrCryptSym symKey = keyStore->getSymKey(lekv);
    
  // link encrypt packet
  uint8_t * pkt;
  size_t pktSize;
  ka.getBuffer(&pkt, &pktSize);
  assert(frPktLinkEncrypt(e.zkError(), symKey, pkt, pktSize));
  assert(frPktSetLekv(e.zkError(), pkt, pktSize, lekv));

  // send keepalive packet
  assert(!sock.sendto((char *)pkt, (int32_t)pktSize, 0, &(aipInfo.addr), 
		      ZKPR_INTERVAL_NO_TIMEOUT) == ZKPR_SUCCESS);
                       
  // free keepalive packet
  ka.destroy();
}

void cFakeAip::buildRoute(cRouteInfo & routeInfo, uint8_t lekv)
{
    CZKS::Exception e;
    cRouteCreatePkt pkt;
    FrCryptPubKey pubKey;
    generatePubCryptKey(&pubKey);
    SymKey symKey;
    generateSymKey(&symKey, 0);
    FrCryptMacKey macKey;
    generateMacKey(&macKey);
    FrCryptSig privSigKey;
    generatePrivSigKey(&privSigKey);
    
    // build the route create packet
    pkt.create(routeInfo.version, FRPKT_PRIORITY_DEFAULT);

    pkt.setAci(routeInfo.aci);
    
    struct fr_rte_set_exp routeExp;
    uint32_t i;
    for (i = 0; i < routeInfo.nbNextHops; i++)
    {
        routeExp.rteAlg = FRCRYPT_SYMALG_BLOWFISH;
        routeExp.bckCrypt = symKey;
        routeExp.fwdCrypt = symKey;
        routeExp.hostEnt = routeInfo.hopInfo[i].entity;
        routeExp.specAci = 0;
        if (i == (uint32_t)(routeInfo.nbNextHops - 1))
        {
            struct timeval now;
            gettimeofday(&now, 0);
            routeExp.macKey = macKey;
            routeExp.expTime = now.tv_sec + routeInfo.timeout;
            routeExp.flags = FRPKT_RTECRT_FL_LAST_RTE;
        }
        else
        {
            routeExp.macKey = NULL;
            routeExp.expTime = 0;
            routeExp.flags = 0;
        }
        pkt.appendRouteExp(pubKey, &routeExp);
    }

    struct fr_nym_auth_exp nymAuthExp;
    nymAuthExp.nymID.type = FRENT_TYPE_ANON;
    nymAuthExp.nymSig = privSigKey;
    
    pkt.appendNymAuthExp(&nymAuthExp);

    // send the route create packet
    char * buf;
    size_t bufSize;
    pkt.getBuffer((uint8_t **)&buf, &bufSize);
    assert(frPktLinkEncrypt(e.zkError(), symKey, buf, bufSize));
    assert(frPktSetLekv(e.zkError(), buf, bufSize, lekv));
    
    assert(sock.sendto(buf, (int32_t)bufSize, 0, &(routeInfo.hopInfo[0].addr), 
                       ZKPR_INTERVAL_NO_TIMEOUT) == (int32_t)bufSize);
    pkt.destroy();
}

void cFakeAip::completeRoute(cRouteInfo & routeInfo, 
                                    FrAci * exitAci)
{
    cRouteCreatePkt pkt;
    FrCryptPubKey pubKey;
    generatePubCryptKey(&pubKey);
    SymKey symKey;
    generateSymKey(&symKey, 0);
    FrCryptMacKey macKey;
    generateMacKey(&macKey);
    FrCryptSig privSigKey;
    generatePrivSigKey(&privSigKey);
    
    // await reception of the route ACK packet
    char ackBuf[65536];
    ZKPRNetAddr peerAddr;
    int32_t ackBufSize = sock.recvfrom(ackBuf, 65536, 0, &peerAddr, 
                                       ZKPR_INTERVAL_NO_TIMEOUT);
    assert(ackBufSize > 0);

    // verify route ack pkt
    CZKS::Exception e;
    FrPktType type;
    assert(frPktGetType(e.zkError(), ackBuf, ackBufSize, &type));
    assert(type == FRPKTTYPE_AIP_ROUTE_CRTACK);

    FrPktAD ackPkt;
    assert(frPktAipDataNew(e.zkError(), &ackPkt));
    assert(frPktAipDataReceiveInBuffer(e.zkError(), ackPkt, ackBuf, ackBufSize));

    for (uint32_t i = 0; i < routeInfo.nbNextHops; i++)
    {
        assert(frPktAipDataDecryptPayload(e.zkError(), ackPkt, symKey));
    }

    char * payload;
    size_t payloadSize;
    assert(frPktAipDataGetPayload(e.zkError(), ackPkt, macKey, (void**)&payload, 
                                  &payloadSize));
    *exitAci = netAddr.ntohs(*(uint16_t *)payload);

    frCryptPubKeyFree(e.zkError(), &pubKey);
    frCryptSymFree(e.zkError(), &symKey);
    frCryptPubKeyFree(e.zkError(), &privSigKey);
}

void cFakeAip::sendDataPkt(cRouteInfo & routeInfo,
                                  char * pkt, size_t size, uint8_t lekv)
{
    CZKS::Exception e;
    
    FrCryptMacKey macKey;
    generateMacKey(&macKey);

    SymKey symKey;
    generateSymKey(&symKey, 0);
    
    char linkPkt[2048];
    FrPktAD freedomPkt;
    assert(frPktAipDataNew(e.zkError(), &freedomPkt));
    assert(frPktAipDataCreateInBuffer(e.zkError(), freedomPkt, linkPkt, 2048));
    assert(frPktAipDataSetVersion(e.zkError(), freedomPkt, FRPKT_DATA_VERSION_1));
    assert(frPktAipDataSetUseMac(e.zkError(), freedomPkt, 1));
    assert(frPktAipDataSetCheckSpec(e.zkError(), freedomPkt, 1));
    assert(frPktAipDataSetType(e.zkError(), freedomPkt));
    assert(frPktAipDataSetAci(e.zkError(), freedomPkt, routeInfo.aci));
    assert(frPktAipDataSetPayload(e.zkError(), freedomPkt, macKey, pkt, size));
    for (uint32_t i = 0; i < routeInfo.nbNextHops; i++)
    {
        assert(frPktAipDataEncryptPayload(e.zkError(), freedomPkt, symKey));
    }

    size_t txSize;
    assert(frPktAipDataGetCurrentSize(e.zkError(), freedomPkt, &txSize));
    txSize = frPktGetSize(txSize);

    SymKey linkKey;
    generateSymKey(&linkKey, lekv);
        
    assert(frPktLinkEncrypt(e.zkError(), linkKey, linkPkt, txSize));
    assert(frPktSetLekv(e.zkError(), linkPkt, txSize, lekv));

    assert(sock.sendto(linkPkt, (int32_t)txSize, 0, &(routeInfo.hopInfo[0].addr), 
                       ZKPR_INTERVAL_NO_TIMEOUT) == (int32_t)txSize);

    assert(frCryptSymFree(e.zkError(), &linkKey));
    assert(frCryptSymFree(e.zkError(), &symKey));
    assert(frCryptMacKeyFree(e.zkError(), &macKey));
    assert(frPktAipDataFree(e.zkError(), &freedomPkt));
}

