#define REV_ID "@(#) $Id: cInetServer.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "netinet/ip.h"
#include "netinet/udp.h"
#include "czkIntervalTime.h"
#include "cInetServer.h"

cInetServer::cInetServer()
{
    initialized = false;
}

cInetServer::~cInetServer()
{
    if (initialized)
    {
        sock.close();
    }
}

void cInetServer::init()
{
    assert(sock.open() == ZKPR_SUCCESS);

    ZKPRNetAddr addr;
    char hostName[256];
    assert(gethostname(hostName, 256) == 0);
    netAddr.getHostbyName(hostName, 256);
    netAddr.enumerateHostEnt(0, INETSERVER_PORT, &addr);
    assert(sock.bind(addr) == ZKPR_SUCCESS);

    initialized = true;
}

void cInetServer::buildPkt(char * pkt)
{
    char hostName[256];
    assert(gethostname(hostName, 256) == 0);
    netAddr.getHostbyName(hostName, 256);
    ZKPRNetAddr addr;
    netAddr.enumerateHostEnt(0, 643, &addr);


    struct iphdr * ipHeader = (struct iphdr *)&pkt[0];
    struct udphdr * udpHeader = (struct udphdr *)&pkt[sizeof(struct iphdr)];

    ipHeader->ihl = 5;
    ipHeader->version = IPVERSION;
    ipHeader->tos = 0;
    ipHeader->tot_len = INETSERVER_PKT_SIZE;
    ipHeader->id = 455;
    ipHeader->frag_off = 0;
    ipHeader->ttl = 15;
    ipHeader->protocol = 17;  //UDP
    ipHeader->check = 0;
    ipHeader->saddr = 0;
    ipHeader->daddr = addr.inet.ip;
    
    udpHeader->source = 17423;
    udpHeader->dest = netAddr.htons(INETSERVER_PORT);
    udpHeader->len = INETSERVER_PKT_PAYLOAD_SIZE - sizeof(struct iphdr);
    udpHeader->check = 0;

    // fill up payload
    uint32_t i;
    for (i = 0; i < INETSERVER_PKT_PAYLOAD_SIZE; i++)
    {
        pkt[sizeof(struct iphdr) + sizeof(struct udphdr) + i] = i % 256;
    }
    
    // calculate ip header checksum
    uint32_t checksum = 0;
    for (i = 0; i < (sizeof(struct iphdr) / sizeof(uint16_t)); i++)
    {
        checksum += (uint32_t)*(uint16_t *)&pkt[i*sizeof(uint16_t)];
    }

    uint32_t tmp = (checksum & 0x0000FFFF) + ((checksum & 0xFFFF0000) >> 16);
    ipHeader->check = ~((uint16_t)((tmp & 0x0000FFFF) + ((tmp & 0xFFFF0000) >> 16)));
}

bool cInetServer::recvAndVerifyPkt(uint16_t * sourcePort)
{
    bool result = true;
    
    // receive the data pkt
    char buf[65536];
    ZKPRNetAddr addr;
    int32_t nb;
    ZKPRIntervalTime timeout;
    czkIntervalTime interval;
    timeout = interval.getSeconds(2);
    nb = sock.recvfrom(buf, 65536, 0, &addr, timeout);
    if (nb != INETSERVER_PKT_PAYLOAD_SIZE)
    {
        result = false;
    }
    else
    {
        for (uint32_t i = 0; (i < INETSERVER_PKT_PAYLOAD_SIZE) && result; i++)
        {
            if (buf[i] != (char)(i % 256))
            {
                result = false;
            }
        }
    }

    *sourcePort = netAddr.ntohs(addr.inet.port);

    return (result);
}

