#define REV_ID "@(#) $Id: cKeyProviderStub.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "cKeyProvider.h"
#include "commonUtils.h"

//
// Key Provider Cache Entry Data stubs
// 
// 

cKeyProviderCacheEntryData::cKeyProviderCacheEntryData()
{
}

cKeyProviderCacheEntryData::~cKeyProviderCacheEntryData()
{
}

cKeyProviderCacheEntryData::cKeyProviderCacheEntryData( 
                                const cKeyProviderCacheEntryData& entryData)
{
  qry = entryData.qry;
  state = entryData.state;
  expires = entryData.expires; 
  expTime = entryData.expTime;
  replyLen = 0;
  reply = NULL; 
  keyInfo = entryData.keyInfo;
}

FrKeyQryState cKeyProviderCacheEntryData::getState()
{
    return FRKEYQRY_ST_VALID;
}

FrCryptPubKey& cKeyProviderCacheEntryData::getPubKey()
{
   return keyInfo.key;
}

void cKeyProviderCacheEntryData::setPubKey(FrCryptPubKey& newkey)
{
   CZKS::Exception e;

   if (!frCryptPubKeyDup(e.zkError(), &keyInfo.key, newkey)) {
      CZKS::Throw(e);
   }
}

void cKeyProviderCacheEntryData::setQry(FrKeyQryQueryExp& newqry)
{
   qry = newqry;
}

void cKeyProviderCacheEntryData::setState(FrKeyQryState newstate)
{
   state = newstate;
}



//
// Key Provider stubs
// 
// 

cKeyProvider::cKeyProvider()
{
}

cKeyProvider::~cKeyProvider()
{
}

bool cKeyProvider::validateConfig(cConfigProvider*)
{
    return true;
}

bool cKeyProvider::initializePrivateData(czkCoreLoop *pCoreloop)
{
  coreloop = pCoreloop;
  return true;
}
 
void cKeyProvider::startup()
{
}

void cKeyProvider::shutdown()
{
}


// make a key request 
void cKeyProvider::makeKeyRequest( char* entity, 
                                   FrEntIdType entity_type, 
                                   FrPubKeyType key_type, 
                                   uint32_t key_version, 
			           cKeyProviderEventHandler* eventhandler,
                                   cKeyProviderCacheEntryData& entryData)
{
   FrKeyQryQueryExp pqry; /* key query */

   // build the key query 

   pqry.keyVersion = key_version;
   pqry.keyType = key_type;
   pqry.keyOwner.type = entity_type;
   memset(pqry.keyOwner.name, 0, sizeof( pqry.keyOwner.name));
   size_t n = strnlen(entity,sizeof(pqry.keyOwner.name)-1)+1;
   if (n > sizeof(pqry.keyOwner.name)) {
     n = sizeof(pqry.keyOwner.name);
   }
   strncpy(pqry.keyOwner.name,
           entity,
           n);

   // pass request to the "real" makeKeyRequest
   makeKeyRequest( pqry, eventhandler, entryData);
}

// make the key request
//
// NOTE that the default action is to delete the eventhandler object
// after it is invoked. To suppress this, call the eventhandler's
// "setDispose( false)" method (before calling this routine)
//
void cKeyProvider::makeKeyRequest ( FrKeyQryQueryExp pqry,
                                    cKeyProviderEventHandler* eventhandler,
                                    cKeyProviderCacheEntryData& entryData)
{
  CZKS::Exception e;

  entryData.setQry(pqry);
  entryData.setState(FRKEYQRY_ST_VALID);

  if (pqry.keyType == FRCRYPT_KEYTYPE_CRYPTION)
  {
    CZKS::Exception e;
    
    FrCryptPubKey cryptKey;
    generatePubCryptKey(&cryptKey);
    entryData.setPubKey(cryptKey);
    assert(frCryptPubKeyFree(e.zkError(), &cryptKey));
  }
  else if (pqry.keyType == FRCRYPT_KEYTYPE_SIGNING)
  {
    CZKS::Exception e;
    
    FrCryptSig sigKey;
    generatePubSigKey(&sigKey);
    entryData.setPubKey(sigKey);
    assert(frCryptSigFree(e.zkError(), &sigKey));
  }
  else
  {
    assert(0);
  }
  
  if (eventhandler) 
  {
    eventhandler->handleEvent(entryData);
    if (eventhandler->getDispose()) 
    {
      delete eventhandler;
      eventhandler = NULL;
    }
  }
}
