#define REV_ID "@(#) $Id: commonUtils.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "zksexception.h"
#include "frCrypt.h"


char keyAip1_enc[] = { 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
		      0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F };

char keyAip1_dec[] = { 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
		      0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F };

char keyAip2_enc[] = { 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,
		      0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F };

char keyAip2_dec[] = { 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
		      0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F };


void rotateBufferLeft( char *buffer, int size, int n )
{
  // totally unoptimized
  int i, j;

  char *tmp = new char[n];

  for( i = 0 ; i < n ; i++ )
    tmp[i] = buffer[i];

  for( i = 0; i < size - n; i++ )
    buffer[i] = buffer[i + n];

  j = 0;
  for( i = size - n; i < size; i++, j++ )
    buffer[i] = tmp[j];

  delete [] tmp;
}


void makeKeyFromBuf(char *buf, FrCryptSym * symKey)
{
  // buf has to have at least 16 bytes
    const char * func = "makeKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing symmetric key";
        CZKS::Exception e;

        if (!frCryptSymNew(e.zkError(), symKey))
            CZKS::Throw(e);

        msg = "setting symmetric algorithm";
        if (!frCryptSymSetAlg(e.zkError(), *symKey, FRCRYPT_SYMALG_BLOWFISH))
            CZKS::Throw(e);

        msg = "setting symmetric key from data";

        if (!frCryptSymKeyFromData(e.zkError(), *symKey, buf, 16))
            CZKS::Throw(e);
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}



void generatePubCryptKey(FrCryptPubKey * pubKey)
{
    const char * func = "generatePubCryptKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing public crypto key";

        CZKS::Exception e;

        char file[] = "./m.pub";
        if (!frCryptPubKeyNewFromFile(e.zkError(), pubKey, file))
        {
            CZKS::Throw(e);
        }
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void generatePrivCryptKey(FrCryptPubKey * privKey)
{
    const char * func = "generatePrivCryptKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing private crypto key";

        CZKS::Exception e;

        char file[] = "./m.priv";
        if (!frCryptPubKeyNewFromFile(e.zkError(), privKey, file))
        {
            CZKS::Throw(e);
        }
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void generatePubSigKey(FrCryptSig * pubKey)
{
    const char * func = "generatePubSigKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing public signature key";

        CZKS::Exception e;

        char file[] = "./m.pub";
        if (!frCryptSigNewFromFile(e.zkError(), pubKey, file))
        {
            CZKS::Throw(e);
        }
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void generatePrivSigKey(FrCryptSig * privKey)
{
    const char * func = "generatePrivSigKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing private signature key";

        CZKS::Exception e;

        char file[] = "./m.priv";
        if (!frCryptSigNewFromFile(e.zkError(), privKey, file))
        {
            CZKS::Throw(e);
        }
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void generateSymKey(FrCryptSym * symKey, uint8_t keyNo)
{
    const char * func = "generateSymKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing symmetric key";

        CZKS::Exception e;

        if (!frCryptSymNew(e.zkError(), symKey))
        {
            CZKS::Throw(e);
        }

        msg = "setting symmetric algorithm";

        if (!frCryptSymSetAlg(e.zkError(), *symKey, FRCRYPT_SYMALG_BLOWFISH))
        {
            CZKS::Throw(e);
        }        

        msg = "setting symmetric key from data";

        uint32_t i;
        char buf[16];

        for(i = keyNo; i < ((uint32_t)keyNo + 16); i++)
        {
            buf[i] = i;
        }
        if (!frCryptSymKeyFromData(e.zkError(), *symKey, buf, 16))
        {
            CZKS::Throw(e);
        }
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

void generateMacKey(FrCryptMacKey * macKey)
{
    const char * func = "generateMacKey";
    const char * msg = NULL;

    try
    {
        msg = "initializing mac key";

        CZKS::Exception e;

        if (!frCryptMacKeyNew(e.zkError(), macKey))
        {
            CZKS::Throw(e);
        }

        msg = "setting mac key";

        char kbuf[FRCRYPT_MACKEY_SIZE];
        for (uint32_t i = 0; i < sizeof kbuf; i++) 
        { 
            kbuf[i] = i + 1; 
        }
        if(!frCryptMacKeyLoad(e.zkError(), *macKey, kbuf, sizeof kbuf))
        {
            CZKS::Throw(e);
        }        
    }
    catch(...)
    {
        CZKS::Rethrow(func, msg);
    }
}

