#ifndef __COMMONUTILS_H
#define __COMMONUTILS_H

#include "frEnt.h"
#include "frCrypt.h"
#include "zkPrdefs.h"
#include "cDHNegotiatorResultHandler.h"

#define FUNCTIONALAIP_ENTITY_NAME  "functionalAip"
#define FUNCTIONALAIP_FREEDOM_PORT 51101

#define COMMUNICATION_CENTER_PORT  53000

// request command definitions
#define NETINFO_REQUEST          1 // Functional aip requests to the netinfo
#define KEEPALIVECOUNT_REQUEST   2 // Request to report # of keepalive rcvd from an aip
#define AIPDOWN_REQUEST          3 // Request to report the link to a peer aip as down
#define SETDHRESULT_REQUEST      4 // Request to set the subsequent DH results
#define DODH_REQUEST             5 // Request to emulate peer initiated DH


void makeKeyFromBuf(char *buf, FrCryptSym * symKey);

extern char keyAip1_enc[];
extern char keyAip1_dec[];
extern char keyAip2_enc[];
extern char keyAip2_dec[];

// information describing the aip characteristics
typedef struct 
{
    FrEntIdExp entity;          // entity ID
    ZKPRNetAddr addr;           // entity ip address and freedom port number
} AipInfo;

#define MAX_AIP_INFO  10
typedef struct
{
    uint8_t             nbAipInfo;
    AipInfo  aipInfo[MAX_AIP_INFO];
} NetInfoList;

// information describing DH result
typedef struct
{
    AipInfo aipInfo;
    uint8_t            lekv;
} DHInfo;

// request format
typedef struct
{
    uint16_t cmd;
    union
    {
        AipInfo            aipInfo;
        DHInfo             dhInfo;
        DHNegoResult       dhResult;
    }u;
} Request;

// response format
typedef union
{
    NetInfoList    netinfoList;
    uint32_t       keepaliveCount;
} Response;


// prototypes
void generatePubCryptKey(FrCryptPubKey * pubKey);
void generatePrivCryptKey(FrCryptPubKey * privKey);
void generatePubSigKey(FrCryptSig * pubKey);
void generatePrivSigKey(FrCryptSig * privKey);
void generateSymKey(FrCryptSym * symKey, uint8_t keyNo);
void generateMacKey(FrCryptMacKey * macKey);


#endif __COMMONUTILS_H
