#define REV_ID "@(#) $Id: functionalAip.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "zks.h"
#include "zksini.h"
#include "zkError.h"
#include "zkserr.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include <zkspath.h>

#include "aip-modules.h"

#include "cAipComponentManager.h"
#include "cCommCenter.h"
#include "cAipKernel.h"
#include "cConfigProvider.h"
#include "cDHNegotiator.h"
#include "cKeyProvider.h"
#include "cLinkManager.h"
#include "cLinkMaintainer.h"
#include "cNetInfoProvider.h"
#include "cPktDispatcher.h"
#include "cPrivateKeys.h"
#include "cRouteCreator.h"
#include "cStatsHarvestor.h"
#include "cStatsPackager.h"

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <iostream>
#include <fstream>




cCommCenter commCenter;







class cCommCenterEventHandler: public czkEventHandler
{
private:
    Request request;
    uint32_t rxBytes;
public:
    cCommCenterEventHandler()
    {
        rxBytes = 0;
    }
    void handleInput(czkSocket * sock, czkEventHandlerData *)
    {
        int32_t nb;
        char * buf = (char *)&request + rxBytes;
        nb = sock->recv(buf, sizeof(request) - rxBytes, 0, ZKPR_INTERVAL_NO_TIMEOUT);
        assert(nb > 0);
        rxBytes += nb;
        if (rxBytes == sizeof(request))
        {
            commCenter.processRequest(request);
            rxBytes = 0;
        }        
    };
    void handleOutput(czkSocket *, czkEventHandlerData *)
    {
        assert(0);
    };
    void handleException(czkSocket *, czkEventHandlerData *)
    {
        assert(0);
    };
    int32_t handleTimeout(czkEventHandlerData *)
    {
        assert(0);
    };
};

int main(int32_t argc, char **argv)
{ 
    (void)argc;
    (void)argv;
    ZksErr zkserr = NULL;
    ZksLogAgent logAgent = NULL;
    int32_t exitCode;
    cAipComponentManager mgr;

    CZKS::CmdLine        m_cmdline;
    CZKS::Ini            m_ini;



    std::ifstream istrm("./freedom.conf");

    if( !istrm.is_open() ) {
      fprintf(stderr,"couldn't open freedom.conf\n");
    }
      
      //
      // Load Ini object with the configuration file
      //
      istrm >> m_ini;

      //
      // Close the config file
      //


    CZKS::Exception zksException;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      return 1;
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed\n");
      exit(1);
    }
    
    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                              ZKS_LOGAGENT_FILEPERM_USER_READ   |
                              ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                              ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                              ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                              ZKS_LOGAGENT_FILEPERM_OTHER_READ)
	)
    {
      fprintf(stderr, "%s: can't allocate the log agent.\n", 
              "AIP");
      exitCode = EXIT_FAILURE;
      exit(exitCode);
    }

    try {
   
      try {

        if (!frCryptInit(zksException.zkError(), 0)) {
          throw zksException;
        }

        czkCoreLoop* coreloop = NULL;

#ifdef AIPMULTITHREADED
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_AIP_MOD, " running in multithreaded mode ");
        bool dothread = true;
#else
        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, ZKS_AIP_MOD, " running in single thread mode");
        bool dothread = false;
        coreloop = new czkCoreLoop();
#endif

	printf( "Before netinfo" );

        // init communication center
        commCenter.init(false);
        commCenter.getNetinfo();

	printf( "After netinfo" );

        cCommCenterEventHandler eh;
        czkSocket * sock = commCenter.getCommSock();
        assert(coreloop->registerHandler(&eh, NULL, 
                                         sock, 
                                         ZKPR_POLL_READ) == ZKPR_SUCCESS);
   
        // create all the components
        cAipKernel*        pAipKernel        = new cAipKernel();
        cConfigProvider*   pConfigProvider   = new cConfigProvider(m_cmdline, m_ini);
        cPrivateKeys*      pPrivateKeys      = new cPrivateKeys();
        cNetInfoProvider*  pNetinfoProvider  = new cNetInfoProvider();
        cKeyProvider*      pKeyProvider      = new cKeyProvider();
        cStatsHarvestor*   pStatsHarvestor   = new cStatsHarvestor();
        cStatsPackager*    pStatsPackager    = new cStatsPackager();
        cLinkManager*      pLinkManager      = new cLinkManager();
        cLinkMaintainer*   pLinkMaintainer   = new cLinkMaintainer();
        cDHNegotiator*     pDHNegotiator     = new cDHNegotiator();
        cRouteCreator*     pRouteCreator     = new cRouteCreator();
        cPktDispatcher*    pPktDispatcher    = new cPktDispatcher();

        // Feed the config provider with the command line and
        // a .conf file
        

        //add them to the component manager
        mgr.add(ZKS_CONFIGPROVIDER_MOD,      pConfigProvider);
        mgr.add(ZKS_STATSHARVESTOR_MOD,      pStatsHarvestor);
        mgr.add(ZKS_PRIVATEKEYS_MOD,         pPrivateKeys);
        mgr.add(ZKS_KEYPROVIDER_MOD,         pKeyProvider);
        mgr.add(ZKS_NETINFOPROVIDER_MOD,     pNetinfoProvider);
        mgr.add(ZKS_AIPKERNEL_MOD,           pAipKernel);
        mgr.add(ZKS_LINKMAINTAINER_MOD,      pLinkMaintainer);
        mgr.add(ZKS_DHNEGOTIATOR_MOD,        pDHNegotiator);
        mgr.add(ZKS_ROUTECREATOR_MOD,        pRouteCreator);
        mgr.add(ZKS_PACKETDISPATCHER_MOD,    pPktDispatcher);
        mgr.add(ZKS_LINKMANAGER_MOD,         pLinkManager);
        mgr.add(ZKS_STATSPACKAGER_MOD,       pStatsPackager);

        //start the configProvider first
        mgr.startComponent(ZKS_CONFIGPROVIDER_MOD, coreloop, dothread);

        //validate all component configs
        pStatsHarvestor->validateConfig(pConfigProvider);
        pPrivateKeys->validateConfig(pConfigProvider);
        pKeyProvider->validateConfig(pConfigProvider);
        pNetinfoProvider->validateConfig(pConfigProvider);
        pAipKernel->validateConfig(pConfigProvider);
        pDHNegotiator->validateConfig(pConfigProvider);
        pLinkMaintainer->validateConfig(pConfigProvider);
        pRouteCreator->validateConfig(pConfigProvider);
        pPktDispatcher->validateConfig(pConfigProvider);
        pLinkManager->validateConfig(pConfigProvider);
        pStatsPackager->validateConfig(pConfigProvider);

        //start all components (order is important)
        mgr.startComponent(ZKS_STATSHARVESTOR_MOD,      coreloop, dothread);
        mgr.startComponent(ZKS_PRIVATEKEYS_MOD,         coreloop, dothread);
        mgr.startComponent(ZKS_KEYPROVIDER_MOD,         coreloop, dothread);
        mgr.startComponent(ZKS_NETINFOPROVIDER_MOD, coreloop, dothread);
        mgr.startComponent(ZKS_AIPKERNEL_MOD,           coreloop, dothread);
        mgr.startComponent(ZKS_DHNEGOTIATOR_MOD,        coreloop, dothread);
        mgr.startComponent(ZKS_PACKETDISPATCHER_MOD,    coreloop, dothread);
        mgr.startComponent(ZKS_ROUTECREATOR_MOD,        coreloop, dothread);
        mgr.startComponent(ZKS_LINKMAINTAINER_MOD,      coreloop, dothread);
        mgr.startComponent(ZKS_LINKMANAGER_MOD,         coreloop, dothread);
        mgr.startComponent(ZKS_STATSPACKAGER_MOD,       coreloop, dothread);

        if (!dothread) 
	    {
	        //single-threaded - there is only one coreloop 
            coreloop->runEventLoop();
        }
        else 
        {
            // wait for all threads to complete
            mgr.waitForCompletion();
        }
      }
      catch (CZKS::Exception e) {
        e.Rethrow( ZKS_AIP_MOD, "exiting main");
      }
      catch (...) {

        CZKS::Rethrow(ZKS_AIP_MOD, "exiting main");

      }
    }
    catch (CZKS::Exception e) {

      CZKS::LogException( e);
    }    
       
    // clean up all component objects

    mgr.deleteAllComponents();
 
    // cleanup the log agent

    // cleanup the log agent
    zksLogAgentFileFree(zkserr, &logAgent);
    zksErrFree(&zkserr);

    return 0;
}    
