#define REV_ID "@(#) $Id: testTool.cpp,v 1.2 2001/04/18 18:25:51 alainc Exp $"
#include <zkrev.h>
#include "zkserr.h"
#include "zkslog.h"
#include "zkslogagent.h"
#include "zksexception.h"
#include <zkspath.h>
#include "frCrypt.h"
#include "czkNetAddr.h"
#include "frPktWrap.h"
#include "commonUtils.h"
#include "cFakeAip.h"
#include "cNetInfo.h"
#include "cKeyStore.h"
#include "cCommCenter.h"
#include "cInetServer.h"


#define UAIPKAIPINTTESTTOOL_MOD "integrationTestTool"

#define AIP1_ENTITY_NAME  "aip1"
#define AIP1_FREEDOM_PORT 52101

#define AIP2_ENTITY_NAME  "aip2"
#define AIP2_FREEDOM_PORT 52102


// Misc stubs (never used, but must be defined)
uint32_t getKeepaliveCount(FrEntIdExp &)
{
    return (0);
}

void reportAipDown(FrEntIdExp &)
{
}

DHNegoResult dhResult = DHNegoPass;



//
// main
//
// 
int main(int, char * argv[])
{
    (void)argv; // to get rid of warning

    const char         * func = "main";
    ZksErr               zkserr = NULL;
    ZksLogAgent          logAgent = NULL;
    CZKS::Exception      zksException;
    czkNetAddr           netAddr;

    if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
        !::zksLogTermInit() ) {
      exit(1);
    }

    if( !zksErrNew(&zkserr) ) {
      fprintf(stderr,"zksErrNew failed");
      exit(1);
     }



    if (!zksLogAgentFileNew( zkserr, &logAgent, "./test.log",
                              ZKS_LOGAGENT_FILEPERM_USER_READ   |
                              ZKS_LOGAGENT_FILEPERM_USER_WRITE  |
                              ZKS_LOGAGENT_FILEPERM_GROUP_READ  |
                              ZKS_LOGAGENT_FILEPERM_GROUP_WRITE |
                              ZKS_LOGAGENT_FILEPERM_OTHER_READ))
    {
      fprintf(stderr, "%s: can't allocate the log agent.\n", "AIP");
      exit(1);
    }

    try
    {
    try
    {
        cCommCenter commCenter;
        commCenter.init(true);
        cNetInfo netinfo(&commCenter);

        if (!frCryptInit(zksException.zkError(), 0)) 
        {
            CZKS::Throw(zksException);
        }

        // initialize key store
        cKeyStore keyStore;
        keyStore.init();

        // get host name
        char hostName[256];
        assert(gethostname(hostName, 256) == 0);
        netAddr.getHostbyName(hostName, 256);

        // initialize AIP #1
        AipInfo aip1Info;
        aip1Info.entity.type = FRENT_TYPE_AIP;
        strncpy(aip1Info.entity.name, AIP1_ENTITY_NAME, sizeof(aip1Info.entity.name));
        netAddr.enumerateHostEnt(0, AIP1_FREEDOM_PORT, &aip1Info.addr);
        netinfo.add(aip1Info);
        cFakeAip aip1(keyStore, aip1Info);
	aip1.init();


        // initialize AIP #2
        AipInfo aip2Info;
        aip2Info.entity.type = FRENT_TYPE_AIP;
        strncpy(aip2Info.entity.name, AIP2_ENTITY_NAME, sizeof(aip2Info.entity.name));
        netAddr.enumerateHostEnt(0, AIP2_FREEDOM_PORT, &aip2Info.addr);
        netinfo.add(aip2Info);
        cFakeAip aip2(keyStore, aip2Info);
	aip2.init();

        // update netinfo with functional aip info
	AipInfo functionalAipInfo;
        functionalAipInfo.entity.type = FRENT_TYPE_AIP;
        strncpy(functionalAipInfo.entity.name, FUNCTIONALAIP_ENTITY_NAME, 
                sizeof(functionalAipInfo.entity.name));
        netAddr.enumerateHostEnt(0, FUNCTIONALAIP_FREEDOM_PORT, 
                                 &functionalAipInfo.addr);
        netinfo.add(functionalAipInfo);

        
        // wait for functional AIP to connect
        commCenter.awaitClient();

        // send the netinfo to functional aip
        commCenter.getRequest();

        // wait for functional aip to be initialized
        sleep(1);


        //
        // Link Creation Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, "***** Testing link creation");

        // send a keepalive to the functional aip from aip1
        aip1.sendKeepalive(functionalAipInfo, 0);
        
        // wait for functional aip to receive the keepalive
        sleep(1);
                
        // verify that the keepalive was received
        uint32_t aip1KaCount = commCenter.getKeepAliveCount(aip1Info);
        if (aip1KaCount > 1)
        {
            ZKS_LOGDEBUG_1(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Received %d keepalives instead of just 1", aip1KaCount);
        }
        else if (aip1KaCount < 1)
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Keepalive wasn't received");
        }
        else
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test PASSED");
        }

        
        //
        // Link Extension Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, "***** Testing link extension");

        uint32_t count;
        
        // order stubbed DHNegotiator of functional aip to emulate a 
        // link extension for aip1
        commCenter.doDH(aip1Info, 1);
        
        // send a keepalive to the functional aip from aip1 using new key
        aip1.sendKeepalive(functionalAipInfo, 1);

        // wait for functional aip to receive the keepalive
        sleep(1);
        
        // verify that the keepalive was received
        count = commCenter.getKeepAliveCount(aip1Info);
        if (count == aip1KaCount)
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Keepalive(1) was not received");
        }
        else 
        {
            assert(count == (aip1KaCount + 1));
            aip1KaCount++;
            
            // send a keepalive to the functional aip from aip1 using old key
            aip1.sendKeepalive(functionalAipInfo, 0);

            // wait for functional aip to receive the keepalive
            sleep(1);
        
            // verify that the keepalive was received
            count = commCenter.getKeepAliveCount(aip1Info);
            if (count == aip1KaCount)
            {
                ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                               "***** Test FAILED: Keepalive(0) was not received");
            }
            else 
            {
                assert(count == (aip1KaCount + 1));
                aip1KaCount++;

                ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                               "***** Test PASSED");
            }
        }



        //
        // Route Creation (exit hop) Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                       "***** Testing route creation (exit hop)");

        // create an aip1+functionalAip route
        cRouteInfo firstExitRoute;
        firstExitRoute.version = FRPKT_ROUTE_VERSION_1;
        firstExitRoute.hopInfo[0] = functionalAipInfo;
        firstExitRoute.nbNextHops = 1;
        firstExitRoute.timeout = 3600;
        firstExitRoute.aci = 1;
        FrAci firstExitRouteExitAci;
        aip1.buildRoute(firstExitRoute, 0);
        aip1.completeRoute(firstExitRoute, &firstExitRouteExitAci);

        // format a packet destined to server emulator 
        char pkt[65536];
        cInetServer inetServer;
        inetServer.init();
        inetServer.buildPkt(pkt);

        // send pkt through aip1+functionalAip route
        aip1.sendDataPkt(firstExitRoute, pkt, INETSERVER_PKT_SIZE, 0);
        
        // verify that server emulator received the packet
        uint16_t firstExitRoutePort;
        if (!inetServer.recvAndVerifyPkt(&firstExitRoutePort))
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Data packet was not received properly by the inet server");
        }
        else
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test PASSED");
        }


#if 0
        //
        // Route Creation (intermediate hop) Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                       "***** Testing route creation (intermediate hop)");

        // create an aip1+functionalAip+aip2 route
        cRouteInfo firstIntermediateRoute;
        firstExitRoute.version = FRPKT_ROUTE_VERSION_1;
        firstIntermediateRoute.hopInfo[0] = functionalAipInfo;
        firstIntermediateRoute.hopInfo[1] = aip2Info;
        firstIntermediateRoute.nbNextHops = 2;
        firstIntermediateRoute.timeout = 3600;
        firstIntermediateRoute.aci = 2;
        FrAci firstIntermediateRouteExitAci;
        int32_t firstIntermediateRouteId = aip1.buildRoute(firstIntermediateRoute, 0);

        // format a packet destined to exit hop aip2 
        aip2.buildPkt(pkt);

        // send pkt through aip1+functionalAip+aip2 route
        aip1.sendDataPkt(firstIntermediateRoute, pkt, AIP_PKT_SIZE, 0);

        // verify that aip2 received the packet
        if (!aip2.recvAndVerifyDataPkt())
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Data packet was not received properly by aip2");
        }
        else
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test PASSED");
        }


        //
        // Route Extension Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                       "***** Testing route creation (intermediate hop)");

        // extend an aip1+functionalAip route
        firstExitRoute.aci = 3;
        int32_t secondExitRouteId = aip1.extendRoute(firstExitRoute, 
                                                     firstExitRouteExitAci);

        // format a packet destined to server emulator 
        inetServer.buildPkt(pkt);

        // send pkt through new aip1+functionalAip route
        aip1.sendDataPkt(secondExitRouteId, pkt, INETSERVER_PKT_SIZE);

        // verify that server emulator received the packet
        if (!inetServer.recvAndVerifyPktFromExtendedRoute(firstExitRoutePort))
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Data packet was not received properly by the inet server");
        }
        else
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test PASSED");
        }


        //
        // Route Deletion Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                       "***** Testing route deletion");

        // destroy new aip1+functionalAip route
        aip1.destroyRoute(secondExitRouteId);

        // format a packet destined to server emulator 
        inetServer.buildPkt(pkt);

        // send pkt through new aip1+functionalAip route
        aip1.sendDataPkt(secondExitRouteId, pkt, INETSERVER_PKT_SIZE);

        // verify that server emulator didn't receive the packet
        if (inetServer.recvAndVerifyPktFromExtendedRoute(firstExitRoutePort))
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Data packet was received by the inet server");
        }
        else
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test PASSED");
        }
#endif


        //
        // Link Deletion Test
        // 

        ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, "***** Testing link deletion");

        // order stubbed Link Maintainer of functional aip to emulate a
        // link going down for aip1
        commCenter.setDHResult(DHNegoFail);
        commCenter.setAipDown(aip1Info);

        // send a keepalive to the functional aip from aip1 using key 0
        aip1.sendKeepalive(functionalAipInfo, 0);

        // wait for functional aip to receive the keepalive
        sleep(1);
        
        // verify that the keepalive was NOT received
        count = commCenter.getKeepAliveCount(aip1Info);
        if (count > aip1KaCount)
        {
            ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                           "***** Test FAILED: Keepalive (0) was received, link deletion didn't work");
        }
        else 
        {
            assert(count == aip1KaCount);
            
            // send a keepalive to the functional aip from aip1 using key 1
            aip1.sendKeepalive(functionalAipInfo, 1);

            // wait for functional aip to receive the keepalive
            sleep(1);
        
            // verify that the keepalive was NOT received
            count = commCenter.getKeepAliveCount(aip1Info);
            if (count > aip1KaCount)
            {
                ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                               "***** Test FAILED: Keepalive (1) was received, link deletion didn't work");
            }
            else
            {
                ZKS_LOGDEBUG_0(ZKS_LOG_ANY_MASK, UAIPKAIPINTTESTTOOL_MOD, 
                               "***** Test PASSED");
            }
        }
    }
    catch (...)
    {
        CZKS::Rethrow(func, "Unexpected problem occured");
    }
    }
    catch (CZKS::Exception &e)
    {
        CZKS::LogException(e);
    }

    zksLogAgentFileFree(zkserr, &logAgent);
    zksErrFree(&zkserr);

    return 0;
}
