/*
 * $Id: log.c,v 1.1 2000/02/03 03:42:29 philippe Exp $
 */

static const char * module_id_ = "$Id: log.c,v 1.1 2000/02/03 03:42:29 philippe Exp $";

/*
 * This file implements some thread safe wrappers around the logging functions
 * zkLogError() and zkLogWrite_va().  This allows the various threads to 
 * safely update the log file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "dispatcher.h"

/*
 *
 */

int
dispatcherLogError (
  Context * c ,
  ZkError err )
{
  int ret = 1;       /* Function return code */

  (void) module_id_; /* Gets rid of the compile warning */
  if (!c) {
    ret = 0;
  } else {
    pthread_mutex_lock(&(c->log_mutex));
    ret = zkLogError(c->log,err);
    pthread_mutex_unlock(&(c->log_mutex));
  }

  return ret;
}

/*
 * 
 */

int
dispatcherLogWrite_va (
  Context * c ,
  const char * fmt ,
  ...  )
{
#define BUF_SZ 512
  int ret = 1;              /* Function return code */
  va_list ap;               /* Variable parameter list */
  static char buf[BUF_SZ];  /* Output buffer (protected by the log_mutex) */
  int buf_sz = 0;           /* Number of bytes output to buffer */
 
  if (!c) {
    ret = 0;
  } else {
    pthread_mutex_lock(&(c->log_mutex));
    va_start(ap,fmt);
    buf_sz = vsnprintf(buf,BUF_SZ,fmt,ap);
    va_end(ap);
    if (buf_sz < 0) {
      ret = 0;
    } else {
      ret = zkLogWrite(c->log,buf);
    } 
    pthread_mutex_unlock(&(c->log_mutex));
  }

  return ret;
}

/*
 *
 */

int
dispatcherRotateLog (
  Context * c )
{
  int ret = 0;

  if (!c) {  
    ret = 0;
  } else {
    pthread_mutex_lock(&(c->log_mutex));
    if (c->flags & FL_DEBUG) {
      zkLogWrite(c->log,mesg_16_);
    }
    ret = zkLogReopenFile(c->log,c->log_file);
    pthread_mutex_unlock(&(c->log_mutex));
  }

  return ret;
}
