/*
 * $Id: alias.c,v 1.7 1999/11/03 18:49:43 roger Exp $
 */

static const char * module_id_ = "$Id: alias.c,v 1.7 1999/11/03 18:49:43 roger Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <ctype.h>
#include <zkError.h>
#include <zkAlloc.h>
#include "zkParms.h"
#include "alias.h"

#define FMG_ALIASLIST_SZ  17


/*
 * Internal prototypes
 */

DLLExport size_t FMGVariableHash ( ZkHash , void * , void * );


/*
 * This is taken from zkShell
 */

DLLExport size_t
FMGVariableHash (
  ZkHash h ,
  void * key ,
  void * userdata )
{
  char *p = key;
  size_t v = 0, i;

  h = NULL; /* Gets rid of compile time warnings */
  userdata = NULL;
  for (i = 0; p[i]; i++) {
    v = v * 33 + p[i];
  }
  return v;
}

DLLExport int 
FMGAliasNewFromConfig (
  ZkError err ,
  FMGAlias * a ,
  ZkConfig c )
{
  static const char * func = "FMGAliasNewFromConfig";
  char * pval = NULL;
  char * email = NULL;
  char * alias = NULL;

  (void) module_id_;
  zkErrorClear(err);
  if (a && c) {
    if (!FMGAliasNew(err,a)) {
      return 0;
    }
    if (zkConfigGet(err,c,S_SERVER_FMG,P_ALIAS,(const char **)&pval)) {
      while (pval) {
        alias = strtok(pval," \t\n\r");    
        email = strtok(NULL," \t\n\r");    
        if (!(alias && email)) {
          zkErrorSet(err,func,ZKERR,ZKMFMG,ZKEPARSE,
                     "Unable to parse alias");
          return 0;
        }
        if (!FMGAliasAdd(err,*a,alias,email)) {
          return 0;
        } 
        if (!zkConfigGetNext(err,c,&pval)) { 
          break;
        }
      }
    } 
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}


DLLExport int
FMGAliasNew (
  ZkError err ,
  FMGAlias * alias )
{
  static const char * func = "FMGAliasNew";
  zkErrorClear(err);
  if (alias) {
    if ((*alias = (FMGAlias) malloc(sizeof(struct T_FMG_Alias))) == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENOMEM,"malloc");
      return 0;
    }
    memset(*alias,0,sizeof(struct T_FMG_Alias));
    (*alias)->aliases = 0;
    if (!(zkHashNew(err,&((*alias)->a_list)) &&
          zkHashSetTableSize(err,(*alias)->a_list,FMG_ALIASLIST_SZ) &&
          zkHashSetHashFn(err,(*alias)->a_list,FMGVariableHash))) {
      return 0;
    }
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
FMGAliasFree (
  ZkError err ,
  FMGAlias * alias ) 
{
  static const char * func = "FMGAliasFree";
  zkErrorClear(err);
  if (alias) {
    if (*alias) {
      zkHashFree(NULL,&((*alias)->a_list));
      memset(*alias,0,sizeof(struct T_FMG_Alias));
      free(*alias);
      *alias = 0;
      return 1;
    }
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
FMGAliasAdd (
  ZkError err ,
  FMGAlias a ,
  const char * alias ,
  const char * email )
{
  static const char * func = "FMGAliasAdd";
  char * email_cp = NULL;
  char * alias_cp = NULL;
  int n = 0;

  zkErrorClear(err);
  if (alias &&  a && email) {
    if ((alias_cp = (char *) malloc(sizeof(char)*(strlen(alias)+1))) == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENOMEM,"malloc");
      return 0;
    } else {
      while (alias[n]) {
        alias_cp[n] = toupper(alias[n]);
        n++;
      }
      alias_cp[n] = (char) 0;
    }
    if ((email_cp = strdup(email)) == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENOMEM,"strdup");
      return 0;
    }
    if (!zkHashInsert(err,a->a_list,alias_cp,email_cp)) {
      return 0;
    }
    a->aliases += 1;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}

/*
 * This will convert the alias string to upper case
 */

DLLExport int
FMGAliasLookup (
  ZkError err ,
  FMGAlias a ,
  const char * in_alias ,
  const char ** email ,
  int * found )
{
  static const char * func = "FMGAliasLookup";
  int n = 0;
  char * alias = NULL;

  zkErrorClear(err);
  if (a && in_alias && email && found) {
    *found = 0;
    if (a->aliases == 0) {
      return 1;
    }
    if (!zkMemDup(err,(void**)&alias,in_alias,strlen(in_alias)+1)) {
      return 0;
    }
    while (alias[n]) {
      alias[n] = toupper(alias[n]);
      n++;
    }
    if (zkHashSearch(err,a->a_list,alias,(void **) email)) {
      *found = 1;
    }
    zkMemFree(NULL,(void**)&alias);
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}  
