/*
 *  $Id: fmg.c,v 1.41 2001/04/19 19:26:42 alainc Exp $
 */

static const char * const
module_id_ = "$Id: fmg.c,v 1.41 2001/04/19 19:26:42 alainc Exp $";

#include "zks.h"

#include "zkIntTypes.h"
#include <wait.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <ctype.h>
#include <sys/param.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkSecure.h"
#include "zkConfig.h"
#include "zkList.h"
#include "zkMIME.h"
#include "zkDNS.h"
#include "zkEmailUtil.h"
#include "frAmtpClient.h"
#include "frCrypt.h"
#include "frSec.h"
#include <signal.h>
#include "zkSignal.h"
#include "zkCStr.h"
#include "zkFile.h"
#include "zkParms.h"
#include "fmg.h"
#include "sm.h"
#include "zkAlloc.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifndef BYTES_PER_KILOBYTE
#  define BYTES_PER_KILOBYTE (1024)
#endif

#ifndef MAXPATHLEN /* In case sys/param.h does not have it */
#define MAXPATHLEN 256
#endif /* !MAXPATHLEN */

#define ZKMODULE ZKMFMG

/*
 * Configuration file parameters
 */

/*
 *  The following is a debugging parameter.  Keep it here for now; we can move
 *  it to zkParms.h if it turns out to be needed in the long term.
 */
#define PARM_ARCHIVE     "debug.archiveDir"

/*
 * Configuration file section names
 */

#if 0
#define SEC1            "FMG"
#endif


#define TOKVAL_YES      "yes"


static volatile sig_atomic_t reloadKeys = 0;
static volatile sig_atomic_t reloadClientInfo = 0;
static volatile sig_atomic_t running = 1;
static volatile sig_atomic_t sigchild = 0;

/*
 * Function prototypes
 */

static int FMGContextFree ( ZkError , FMGContext * );
static int FMGContextLoadConfig ( ZkError , FMGContext * );
static int FMGContextLoadParameters ( ZkError , FMGContext * , int , const char ** );
static int FMGContextLoad ( ZkError , FMGContext * , int , const char ** );
static int FMGContextLoadKeys ( ZkError , FMGContext * );
static int FMGNetInfoGetType( ZkError, FMGContext *, FrEntIdType,
                              struct sockaddr_in *, FrEntIdExp *);
static int FMGUsage( FMGContext *, FILE * );

#if 0
static int FMGProcessAddToRecipientList (
  ZkError ,             /* Return the error if any occured */
  FMGContext * ,        /* FMG filter application context */
  ZkList ,              /* The recipient list */
  char * );             /* A value string of addresses from a To: field */
#endif

/*
 * These functions implement some of the nym controlled spam avoidance features
 */

static int FMGContextCheckBlackholeList (
  ZkError ,
  FMGContext * context ,
  char * source ,
  int * drop );

static int FMGProcessMessage (
  ZkError ,            /* Return the error if any occured */
  FMGContext * ,       /* FMG filter application context */
  ZkEmail );           /* Email message to process */

static int FMGMessageSend (
  ZkError ,             /* Returns the error if any occured */
  FMGContext * ,        /* FMG filter application context */
  char * ,              /* Sender email address */
  char * ,              /* Name of the sending host */
  char * ,              /* Name of recipient (target nym) */
  char * ,              /* Email domain of recipient nym */
  char * ,              /* Message buffer */
  size_t ,              /* Size of message buffer */
  int * );              /* Set to one if the sender should be notified of a
                         * send failure 
                         */

static int FMGGetSendingHost (
  ZkError err ,
  char * rcvd_fld ,       /* Field value from an 822 "Received" field */
  char ** sending_host ,  /* Returns a pointer inside rcvd_fld */
  size_t * sz );           /* Returns the size of the sending host name */


static int FMGArchiveMessage (
  ZkError ,            /* Return the error */
  ZkEmail ,            /* Message to archive */
  const char * );               /* Archive directory */


/*
 *
 */

static int FMGUsage( FMGContext *context, FILE *stream)
{
  if (!context) {
    return 0;
  }

  if (stream == NULL) {
    stream = stderr;
  }

  fprintf(stream,
          VERSION_FMG_STR "\n\n"

          "Usage:  %s options...\n\n"

          O_HELP "\n\t"
                  "Displays this help screen\n\n"

          O_VERSION "\n\t"
                  "Displays the version\n\n"

          O_CONFIG_FILE " filename\n\t"
                  "Specifies the configuration file\n\n"

          "--" P_DEBUG " [yes|true|on|no|false|off]\n\t"
                  "Sends debugging information to stderr\n\n"

          "--" P_DAEMON " [yes|true|on|no|false|off]\n\t"
                  "Run the server in daemon mode (default is yes)\n\n"

          "--" P_LOGFILE " filename\n\t"
                  "Specifies the server log file\n\n"

          "--" P_PORT " port\n\t"
                  "Specifies this FMG's listening port\n\n"

          "--" P_PIDFILE " filename\n\t"
                  "Specifies a file for the process pid\n\n"

          "--" PARM_ARCHIVE " dir\n\t"
                  "Specifies the name of a directory where a copy of all incoming messages will be stored\n\n"

          "--" P_ALIAS_MSG " filename\n\t"
                  "Specifies the file containing the alias message\n\n"

          "--" P_BANNER_FILE " filename\n\t"
                  "Specifies the file containing the banner which will be inserted in each mail message\n\n"

          "--" P_KEYINFO_DIRECTORY_CRYPT " dir\n\t"
                  "Specifies the server's encrypted pivate cryption key directory\n\n"

          "--" P_KEYINFO_DIRECTORY_SIG " dir\n\t"
                  "Specifies the server's encrypted pivate signature key directory\n\n"

          "--" P_KEYQRY_RETRIES " nb\n\t"
                  "Specifies the number of retries when FMG queries its current public key from keyqry\n\n"

          "--" P_KEYQRY_TIMEOUT " timeout\n\t"
                  "Specifies the timeout when FMG queries its current public key from keyqry\n\n"

          "--" P_LISTEN_QUEUE_LENGTH " nb\n\t"
                  "Specifies the length of the pending queue of the listening socket\n\n"

          "--" P_MAIP_TIMEOUT " timeout\n\t"
                  "Specifies the timeout for the connection with MAIP\n\n"

          "--" P_MAX_MAIL_RECIPIENTS " nb\n\t"
                  "Specifies the maximum number of entities which can receive an email message (default=%d)\n\n"

          "--" P_MTA_PATH " path\n\t"
                  "Specifies the path and the name of the local mail program (default="DEFAULT_SENDMAIL")\n\n"

          "--" P_NIQS_SERVER " host_name\n\t"
                  "NIQS hostname\n\n"

          "--" P_NIQS_PORT " port\n\t"
                  "NIQS port\n\n"

          "--" P_NIQS_FREQUENCY " period\n\t"
                  "NIQS period between updates in sec (default=%d)\n\n"

          "--" P_NIQS_FULLUPDATE " period\n\t"
                  "NIQS period between complete updates in sec\n\n"

          "--" P_NIQS_MUTEXTEMPLATE " str_template\n\t"
                  "NIQS Mutex filename template\n\n"

          "--" P_NIQS_SHMTEMPLATE " str_template\n\t"
                  "NIQS Shared memory filename template\n\n"

          "--" P_NIQS_SHARED_MEM_SIZE " isize\n\t"
                  "NIQS Shared memory size in kBytes (default=%d)\n\n"

          "--" P_NIQS_TIMEOUT " timeout\n\t"
                  "NIQS timeout\n\n"

#ifdef FREEDOM_DEVELOPMENT_BUILD

          "--" P_NIQS_DUMP_NETINFO_ON_UPDATE " [yes|true|on|no|false|off]\n\t"
                  "Specifies if a dump of NetInfo should be done everytime it is updated\n\n"

#endif

          "--" P_NOT_FOUND_MSG " filename\n\t"
                  "Specifies the file which contains a message sent back by the FMG\n\t"
                  "filter when a mail comes in for a non-existent or disabled nym\n\n"

          "--" P_NYMSRV_TIMEOUT " timeout\n\t"
                  "Specifies the timeout when trying to communicate with the NYM server (default=%d)\n\n"

          "--" P_PASSPHRASE_FILE " filename\n\t"
                  "Passphrase file\n\n"

          "--" P_PASSPHRASE_PROMPT " prompt\n\t"
                  "Prompt for system passphrase\n\n"

          "--" P_SENDER_MAIL_ADDR " sender\n\t"
                  "Specifies the FROM: field when sending text email (default="DEFAULT_SENDER")\n\n"

          "Command line options override settings in the config file\n",
          context->progname,
          DEFAULT_MAX_RECIPIENTS,
          DEFAULT_NI_RELOAD_TIME,
          DEFAULT_NI_SHARED_MEM_SIZE,
          DEFAULT_NYMSERVTOUT);

  return 1;
}

/*
 * Signal handler
 */
static void sig_handler(int sig)
{
  switch (sig) {

  case SIGCHLD:
    sigchild = 1;
    break;

  case SIGTERM:
  case SIGINT:
    running = 0;
    break;

  case SIGUSR1:
    reloadKeys = 1;
    break;

  case SIGUSR2:
    reloadClientInfo = 1;
    break;

  default:
    fprintf(stderr, "Unhandled signal %d\n", sig);
    break;
  }
  return;
}

/*
 * This is a debugging function which will store a copy of all incoming 
 * messages.
 */

static int
FMGArchiveMessage (
  ZkError err ,
  ZkEmail message ,
  const char * archive )
{ 
  FILE * f = NULL;
  char pathname[MAXPATHLEN + 1];
  const char * func = "FMGArchiveMessage";
  int return_code = 1;

  (void) module_id_;
  zkErrorClear(err);
  if (message && archive) {
    snprintf(pathname,sizeof pathname,"%s/FMG.%d.munged.XXXXXX",archive,(int)getpid());
    if ( ! zkFileTempFp(err,pathname,&f)) {
      return 0;
    }
    if (!zkEmailDump(err,message,zkEmailWrite,f)) {
      return_code = 0;
    }
    fclose(f);
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMODULE,ZKENULLPTR,NULL);
  return 0;
}


/*
 * 
 */

static int
FMGContextLoadConfig (
  ZkError err,
  FMGContext * ctx )
{
  uint8_t  bVal;
  int32_t  i32Val;
  uint32_t ui32Val;
  uint16_t ui16Val;

  if (!zkConfigLoad(err,ctx->config,ctx->configfile)) {
    fprintf(stderr,"%s:  Unable to load \"%s\".\n",ctx->progname,
            ctx->configfile);
    return 0;
  }

  if (zkConfigGetParameter(err,ctx->config,TYPE_BOOLEAN,S_SERVER_FMG,P_DEBUG,&bVal) &&
      bVal) {
    ctx->flags |= FLAG_FMG_DEBUG;
  }
 
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_LOGFILE,&ctx->logfile);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_BANNER_FILE,&ctx->bannerfile);

  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_SERVER_FMG,P_MAX_MAIL_RECIPIENTS,&i32Val)) {
    ctx->max_recipients = i32Val;
  }

  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,PARM_ARCHIVE,&ctx->archive);
  
  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_CLIENT_MAIP,P_MAIP_TIMEOUT,&i32Val)) {
    ctx->maip.timeout = i32Val;
  }

  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_CLIENT_NYMSRV,P_NYMSRV_TIMEOUT,&i32Val)) {
    ctx->nymserv.timeout = i32Val;
  } 

  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_NOT_FOUND_MSG,&ctx->notfoundmsg_file);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_ALIAS_MSG,&ctx->aliasmsg_file);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_SENDER_MAIL_ADDR,&ctx->sender);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_MTA_PATH,&ctx->sendmail);

  /* Load parameters for the key query server */
  zkConfigGetParameter(err,ctx->config,TYPE_UINT32,S_CLIENT_KEYQRY,P_KEYQRY_TIMEOUT,&ctx->kqtimeout);

  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_CLIENT_KEYQRY,P_KEYQRY_RETRIES,&i32Val)) {
    ctx->kqretries = i32Val;
  }

  /* Load parameters for the private keys... */
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_KEYINFO_DIRECTORY_SIG,&ctx->privSigDir);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_KEYINFO_DIRECTORY_CRYPT,&ctx->privCryptDir);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_PASSPHRASE_PROMPT,&ctx->ppprompt);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_PASSPHRASE_FILE,&ctx->ppfile);

  if (zkConfigGetParameter(err,ctx->config,TYPE_UINT16,S_SERVER_FMG,P_PORT,&ui16Val)) {
    ctx->port = ui16Val;
  }

  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_SERVER_FMG,P_LISTEN_QUEUE_LENGTH,&i32Val)) {
    ctx->listenq = i32Val;
  }

  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_SERVER_FMG,P_PIDFILE,&ctx->pidfile);

  if (zkConfigGetParameter(err,ctx->config,TYPE_BOOLEAN,S_SERVER_FMG,P_DAEMON,&bVal)) {
    ctx->fork = bVal;
  }

  /* Get NIQS parameters */
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_CLIENT_NIQS,P_NIQS_SERVER,&ctx->niqsHost);

  if (zkConfigGetParameter(err,ctx->config,TYPE_UINT16,S_CLIENT_NIQS,P_NIQS_PORT,&ui16Val)) {
    ctx->niqsPort = ui16Val;
  }

  if (zkConfigGetParameter(err,ctx->config,TYPE_INT32,S_CLIENT_NIQS,P_NIQS_TIMEOUT,&i32Val)) {
    ctx->niqsTimeout = i32Val;
  }

  zkConfigGetParameter(err,ctx->config,TYPE_UINT32,S_CLIENT_NIQS,P_NIQS_FREQUENCY,&ctx->niReloadPeriod);

  if (zkConfigGetParameter(err,ctx->config,TYPE_UINT16,S_CLIENT_NIQS,P_NIQS_SHARED_MEM_SIZE,&ui16Val)) {
    ctx->niSharedMemSz = ui16Val * BYTES_PER_KILOBYTE;
  }

  if (zkConfigGetParameter(err,ctx->config,TYPE_UINT32,S_CLIENT_NIQS,P_NIQS_FULLUPDATE,&ui32Val)) {
    ctx->niqsFullUpdatePeriod = (time_t)ui32Val;
  }

  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_CLIENT_NIQS,P_NIQS_SHMTEMPLATE,&ctx->niShmTemplate);
  zkConfigGetParameter(err,ctx->config,TYPE_STRING,S_CLIENT_NIQS,P_NIQS_MUTEXTEMPLATE,&ctx->niMtxTemplate);

#ifdef FREEDOM_DEVELOPMENT_BUILD

  if (zkConfigGetParameter(err,ctx->config,TYPE_BOOLEAN,S_CLIENT_NIQS,P_NIQS_DUMP_NETINFO_ON_UPDATE ,&bVal)) {
    ctx->niDumpOnUpdate = bVal;
  }

#endif

  return 1;
}

/*
 * 
 */

static int
FMGContextLoadParameters (
  ZkError err ,
  FMGContext * context ,
  int argc,
  const char ** argv )
{
  static const char * func = "FMGContextLoadParameters";
  int opt = 1;
  int return_code = 1;

  if (!zkConfigSetCmdParms(err,context->config,argc,argv)) {
    zkLogWriteContext(context->log,err,func,"zkConfigSetCmdParms failed");
    fprintf(stderr,"%s:  zkConfigSetCmdParms failed.\n", context->progname);
    return 0;
  }

  /*
   * Find the configuration file first so that command line parameters can
   * overide these values.
   */
 
  for (opt = 1; opt < argc; opt++) {
    if (strcasecmp(O_CONFIG_FILE,argv[opt]) == 0) {
      if (++opt < argc) {
        context->configfile = argv[opt];
        if (!FMGContextLoadConfig(err,context)) {
          zkLogWriteContext(context->log,err,func,"FMGContextLoadConfig failed");
          fprintf(stderr,"%s:  Failed to load config file (check log).\n",
                  context->progname);
          return 0;
        }
        break;
      } else {
        fprintf(stderr,"%s:  You must specify a configuration file.\n",
                context->progname);
        return_code = 0;
      }
    } 
  }

  return return_code;
}


static int FMGContextLoadKeys(ZkError err, FMGContext *c)
{
  static const char *func = "FMGContextLoadKeys";

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "Context is NULL");
    return 0;
  }

  if (!c->kc) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKEDATABAD, "Key chain not init");
    return 0;
  }

  if (!c->ppprompt && !c->ppfile) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKEDATABAD, "Can't get keys");
    return 0;
  }

  /* Bug: Need a consistent way of setting algorithm */
  if (!frSecKeyChainAddSymKey(err, c->kc, c->ppfile, c->ppprompt, 
                              FRCRYPT_SYMALG_BLOWFISH)) {
    zkLogWriteContext(c->log,err,func,"frSecKeyChainAddSymKey failed");
    return 0;
  }

  if (c->privCryptDir) {
    if (!frSecKeyChainRegisterDir(err, c->kc, &c->cryptHandle, 
                                  c->privCryptDir)) {
      zkLogWriteContext(c->log,err,func,
                        "frSecKeyChainRegisterDir failed (crypt)");
      return 0;
    }
  }

  if (c->privSigDir) {
    if (!frSecKeyChainRegisterDir(err, c->kc, &c->sigHandle, c->privSigDir)) {
      zkLogWriteContext(c->log,err,func,
                        "frSecKeyChainRegisterDir failed (sig)");
      return 0;
    }
  }

  /* 
   * Make sure at least one was set to load the keys, otherwise, we have an
   * empty keychain
   */ 
  if (c->privCryptDir || c->privSigDir) {
    if (!frSecKeyChainReloadKeys(err, c->kc)) {
      zkLogWriteContext(c->log,err,func,"frSecKeyChainReloadKeys failed");
      return 0;
    }
  }

  return 1;
}

/*
 *
 */

static int
FMGContextVerify (
  ZkError err ,
  FMGContext * context )
{
  static const char * func = "FMGContextVerify";
  int return_code = 1;

  if (context) {
 
    /*
     * Turn on the log file
     */

    if (context->logfile) {
      if (!(zkLogSetFile(context->log,context->logfile) &&
            zkLogSetProgname(context->log,context->progname) &&
            zkLogEnable(context->log))) {
        fprintf(stderr,"%s:  Unable to log to \"%s\".\n",context->progname,
                context->logfile);
        return_code = 0;
      }
    }

    if (context->maip.timeout < 5) {
      fprintf(stderr,
              "%s:  You must have a MAIP timeout of at least 5 seconds.\n",
              context->progname);
      return_code = 0;
    }

    if (context->nymserv.timeout < 5) {
      fprintf(stderr,"%s:  You must have a nym server timeout of at least 5 seconds.\n",
              context->progname);
      return_code = 0;
    }

    if (context->sender == NULL) {
      fprintf(stderr,"%s:  You must specify an FMG sender address.\n",
              context->progname);
      return_code = 0;
    }
 
    if (context->sendmail == NULL) {
      fprintf(stderr,"%s:  You must specify a sendmail executable.\n",
              context->progname);
      return_code = 0;
    } else {
      if (!zkSecureCheckFile(err,context->sendmail,context->security)) {
        zkLogWriteContext(context->log,err,func,"MTA security check failed");
        fprintf(stderr,"%s:  \"%s\" security check failed.\n",
                context->progname,context->sendmail);
        return_code  = 0;
      }
    }

    /* 
     * Try to load the nym not found message
     */

    if (context->notfoundmsg_file) {
      if (!zkFileLoad(err,context->notfoundmsg_file,
                      &(context->notfoundmsg),&(context->nfmsz))) {
        zkLogWriteContext(context->log,err,func,"Can't load nym not found msg");
        fprintf(stderr,
                "%s:  Unable to load the nym not found message \"%s\".\n",
                context->progname,context->notfoundmsg_file);
        return_code = 0;
      }
    } else {
      fprintf(stderr,"%s:  You must specify a nym not found message.\n",
              context->progname);
      return_code = 0;
    }

    /*
     * Try to load the alias message
     */
    if (context->aliasmsg_file) {
      if (!zkFileLoad(err,context->aliasmsg_file,
                      &(context->aliasmsg),&(context->amsz))) {
        zkLogWriteContext(context->log,err,func,"Can't load alias message");
        fprintf(stderr,
                "%s:  Unable to load the alias message \"%s\".\n",
                context->progname,context->aliasmsg_file);
        return_code = 0;
      }
    }


    /* 
     * If privCryptDir is not set I won't use it.  Currently the fmg filter 
     * does not have to decrypt any publicly encrypted files.
     */ 
    if (!context->privSigDir) {
      fprintf(stderr, "%s: You must specify the priv sig key directory.\n",
              context->progname);
      return_code = 0;
    } else if (!context->ppprompt && !context->ppfile) {
      fprintf(stderr, "%s: Specify either the passphrase prompt or file.\n",
              context->progname);
      return_code = 0;
    }

    /*
     * Load the FMG keys
     */
    if (!FMGContextLoadKeys(err,context)) {
      zkLogWriteContext(context->log,err,func,"Can't initialize FMG keys");
      fprintf(stderr,"%s: Unable to initialize the FMG keys, check logfile.\n",
              context->progname);
      return_code = 0; 
    }

    if (context->port < 1024) {
      fprintf(stderr, "%s: FMG port is not usable %d.\n", context->progname,
              context->port);
      return_code = 0;
    }

    if (context->listenq < 1) {
      fprintf(stderr, "%s: Listen queue set at %d is too low.\n", 
              context->progname, context->listenq);
      return_code = 0;
    }

    if (context->kqtimeout < 1) {
      fprintf(stderr, "%s: Key query timeout [%d] is too low.\n", 
              context->progname, context->kqtimeout);
      return_code = 0;
    }

    if (context->kqretries < 0) {
      fprintf(stderr, "%s: Key query retry [%d] is negative.\n",
              context->progname, context->kqretries);      
      return_code = 0;
    }

    /* NIQS parameters */
    if (!context->niqsHost) {
      fprintf(stderr, "%s: NIQS host must be specified.\n", 
              context->progname);
      return_code = 0;
    }

    if (context->niqsTimeout < 1) {
      fprintf(stderr, "%s: NIQS timeout [%d] is too low.\n", 
              context->progname, context->niqsTimeout);
      return_code = 0;
    }

    if (context->niqsPort < 1024) {
      fprintf(stderr, "%s: NIQS port [%d] is too low.\n",
              context->progname, context->niqsPort);      
      return_code = 0;
    }

    /*
     * You can set the NIQS update period to be 0 in order to turn off the
     * feature.  If you do this, the FrNetInfo will always just do an update
     * when tou call the update functions.  If you set a period however,
     * the frNetInfo object will do a complete reload periodically, based
     * on the specified period.
     */

#ifndef FREEDOM_DEVELOPMENT_BUILD

    if (context->niqsFullUpdatePeriod && context->niqsFullUpdatePeriod < 60) {
      fprintf(stderr,"%s:  NIQS update period %d is too short.\n",
              context->progname,(int)context->niqsFullUpdatePeriod);
      return_code = 0;
    }

    if (context->niReloadPeriod < 60) {
      fprintf(stderr,"%s:  NIQS frequency [%d] is too short.\n",
              context->progname,context->niReloadPeriod);
      return_code = 0;
    }

    if (context->niSharedMemSz < 60) {
      fprintf(stderr,"%s:  NIQS shared memory size [%d] is too small.\n",
              context->progname,context->niSharedMemSz );
      return_code = 0;
    }

#endif

    if (!context->niShmTemplate) {
      fprintf(stderr,"%s:  NIQS shared memory template file not specified.\n",
              context->progname);
      return_code = 0;
    }

    if (!context->niMtxTemplate) {
      fprintf(stderr,"%s:  NIQS mutex template file not specified.\n",
              context->progname);
      return_code = 0;
    }

    /*
     * Verify the maximum number of message recipients
     */

    if (context->max_recipients < 1) {
      fprintf(stderr,"%s:  You must allow at least one message recipient.\n",
              context->progname);
      return_code = 0;
    }

    /*
     * Load the banner file if one is specified
     */
  
    if (context->bannerfile) {
      int size = 0;
      if (!zkFileLoad(err,context->bannerfile,&(context->banner),&size)) {
        zkLogWriteContext(context->log,err,func,"Can't load banner file");
        fprintf(stderr,"%s:  Unable to load banner file \"%s\".\n",
                context->progname,context->bannerfile);
        return_code = 0;
      } else {
        context->bsz = (size_t) size;
      }
    }

    /*
     * Build the alias list
     */

    if (!FMGAliasNewFromConfig(err,&(context->alias),context->config)) {
      fprintf(stderr,"%s:  Unable to initialize the alias list.\n",
              context->progname);
      return_code = 0;
    }

    return return_code;
  }
  return 0;
}
 

/*
 * 
 */

static int
FMGContextLoad (
  ZkError err ,
  FMGContext * context ,
  int argc ,
  const char ** argv )
{
  int opt = 1;

  zkErrorClear(err);
  if (context && argv) {

    memset(context,0,sizeof(FMGContext));

    /*
     * Get the program name.
     */

    context->progname = strrchr(argv[0],'/');
    if (context->progname) { 
      context->progname++;
    } else {
      context->progname = argv[0];
    }

    /*
     * Now check to see if we should be in debug mode
     */
  
    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp("--" P_DEBUG,argv[opt]) == 0) {
        ZkLog log = NULL;
        context->flags |= FLAG_FMG_DEBUG;
        zkErrorLog(err,&log);
        zkLogSetProgname(log,context->progname);
        zkLogSetStream(log,stderr);
        zkLogEnable(log);
        break;
      }
    }

    /*
     * Are we looking for some information instead of trying to run the
     * program?
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],O_HELP) == 0) {
        exit(FMGUsage(context, stderr) ? EXIT_SUCCESS : EXIT_FAILURE) ;
      } else if (strcasecmp(argv[opt],O_VERSION) == 0) {
        fprintf(stderr,"%s\n",VERSION_FMG_STR);
        exit(EXIT_SUCCESS);
      }
    }

    /*
     * First initialize the context structure and set default values
     */
 
    if (!(zkLogNew(&(context->log)) &&
          zkConfigNew(err,&(context->config)) &&
          zkEmailNew(err,&(context->email)))) {
      zkErrorSet(err,"FMGContextLoad",ZKERR,ZKMFMG,ZKERROR,
                 "Unable to create internal context data structures");
      return 0;
    }

    context->logfile = DEFAULT_LOGFILE;
    context->configfile = DEFAULT_CONFIGFILE;
    context->input = DEFAULT_INPUT;
    context->output = DEFAULT_OUTPUT;
    context->error = DEFAULT_ERROR;
    context->exit_code = EXIT_FAILURE;
    context->max_recipients = DEFAULT_MAX_RECIPIENTS;
    context->bannerfile = DEFAULT_BANNERFILE;
    context->banner = DEFAULT_BANNER;
    zkCStrlcpy(context->maip.maip,DEFAULT_MAIP,sizeof context->maip.maip);
    context->maip.port = DEFAULT_MAIPPORT;
    context->maip.timeout = DEFAULT_MAIPTOUT;
    zkCStrlcpy(context->nymserv.nymserver,DEFAULT_NYMSERV,sizeof context->nymserv.nymserver);
    context->nymserv.port = DEFAULT_NYMSERVPORT;
    context->nymserv.timeout = DEFAULT_NYMSERVTOUT;
    context->sender = DEFAULT_SENDER;
    context->sendmail = DEFAULT_SENDMAIL;
    context->security = DEFAULT_SECURITY;
    context->archive = DEFAULT_ARCHIVE;
    context->port = 0;
    context->listenq = 0;
    context->listenFd = -1;
    context->fork = 1;
    context->fptr = NULL;

    if (!frSecKeyChainNew(err, &context->kc)) {
      fprintf(stderr, "%s: Unable to create a new FrSecKeyChain.\n", 
              context->progname);
      return 0;
    }
    context->cryptHandle = 0;
    context->sigHandle = 0;
    context->ppprompt = NULL;
    context->ppfile = NULL;
    context->privCryptDir = NULL;
    context->privSigDir = NULL;

    if (!frKeyQryNew(err, &context->kq)) {
      fprintf(stderr, "%s: Unable to create a new FrKeyQry object.\n", 
              context->progname);
      return 0;
    }
    memset(&context->kh, 0, sizeof(FrKeyQryHost));
    context->kqtimeout = 0;
    context->kqretries = -1;

    context->niqsHost = NULL;
    context->niqsPort = 0;
    context->niqsTimeout = 0;
    context->niqsFullUpdatePeriod = 0;

    context->ni             = NULL;
    context->niCtx          = NULL;
    context->niPid          = 0;
    context->niReloadPeriod = DEFAULT_NI_RELOAD_TIME;
    context->niSharedMemSz  = DEFAULT_NI_SHARED_MEM_SIZE;
    context->niShmTemplate  = NULL;
    context->niMtxTemplate  = NULL;

#ifdef FREEDOM_DEVELOPMENT_BUILD
    context->niDumpOnUpdate = 0;
#endif

    if (context->banner) {
      context->bsz = (size_t) strlen(context->banner);
    } else {
      context->bsz = 0;
    }

    /*
     * Load the parameters and the config file
     */
 
    if (!FMGContextLoadParameters(err,context,argc,argv)) {
      fprintf(stderr,"%s:  Unable to load parameters and or configuration.\n",
              context->progname);
      return 0;
    }

    /*
     * Verify that the context is OK
     */
  
    if (!FMGContextVerify(err,context)) {
      fprintf(stderr,"%s:  Context verification failure.\n",context->progname);
      return 0;
    }

    return 1;
  }
  return 0;
}

/*
 * Checking the Vixie blackhole list to see if the mail comes from a suspected
 * spammer.  If the mail should be dropped, the drop parameter will be set
 * to a non-zero value.
 */

static int
FMGContextCheckBlackholeList (
  ZkError err ,
  FMGContext * context ,
  char * source ,
  int * drop )
{
#define VIXIE_RBL_DNS "rbl.maps.vix.com"
  static const char * func = "FMGContextCheckBlackholeList";
  char rbl_addr[256];         /* Used to build the hostname for RBL lookup */
  struct hostent * host_ent = NULL; /* for a DNS lookup of the sending host */
  char * addr = NULL;               /* For converting an address */
  char ** addr_ptr = NULL;          /* For going through the address list */
  int ret = 0;

  if (source && context && drop) {
    *drop = 0;

    /*
     * First, get the IP address of the host mentioned in the most recent
     * Received field.  If we fail to get this address, we'll drop the mail
     */

    host_ent = gethostbyname(source); /* audit: ok */
    if (host_ent == NULL) {
      zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKENOTFOUND,
                    "Failed to lookup host entry for \"%s\"",source);
      *drop = 1;
    
    /* See URL: http://maps.vix.com/rbl/usage.htlm
     *
     * Now create the lookup name string for each IP address and alias for
     * the machine in question.  Lookup each one and drop the mail if a 
     * record is returned for any of them.  
     * %%Bug:  Should I look at  the returned IP address inside the record
     *         before dropping the mail? (The address should always be
     *         127.0.0.2)
     */   

    } else {
      addr_ptr = host_ent->h_addr_list;
      while (*addr_ptr) {
        addr = *addr_ptr;
        if (snprintf(rbl_addr,256,"%u.%u.%u.%u." VIXIE_RBL_DNS,
                     (unsigned char) addr[3],
                     (unsigned char) addr[2],
                     (unsigned char) addr[1],
                     (unsigned char) addr[0]) < 0) {
          zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKEINCOMPLETE,
                        "Unable to create RBL name string for \"%s\"",source);
          *drop = 1;
          break;
        } else if ((host_ent = gethostbyname(rbl_addr)) != NULL) { /* audit: ok */
          zkLogWrite_va(context->log,
                        "RBL blocked message from \"%s\"",source);
          *drop = 1;
          break;
        }
        addr_ptr++;
      }
    }
    ret = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMODULE,ZKENULLPTR,NULL);
  }

  return ret;
}

/*
 *
 *
 */

static int
FMGContextFree (
  ZkError err ,
  FMGContext * context )
{
  int exit_code = 0;

  exit_code = context->exit_code;
  zkConfigFree(err,&(context->config));
  zkLogFree(&(context->log));

  if (context->fptr) {
    fclose(context->fptr);
  }

  if (context->email) {
    zkEmailFree(NULL,&(context->email));
  }
  if (context->config) {
    zkConfigFree(NULL,&(context->config));
  }
  if (context->kc) {
    (void) frSecKeyChainFree(err, &(context->kc));
  }
  if (context->kq) {
    (void) frKeyQryFree(err, &(context->kq));
  }
  if (context->input) {
    fclose(context->input);
    context->input = NULL;
  }
  if (context->output) {
    fclose(context->output);
    context->output = NULL;
  }
  if (context->error) {
    fclose(context->error);
    context->error = NULL;
  }

  if (context->ni) {
    (void) frNetInfoFree(err, &(context->ni));
  }

  if (context->niCtx) {
    (void) frNetInfoChildCtxFree(err, &context->niCtx);
  }

  memset(context,0,sizeof(FMGContext));
  exit(exit_code);
  return 1; /* We should never get here */
}
 

static int
FMGMessageSend (
  ZkError err ,
  FMGContext * context ,
  char * sender ,
  char * sending_host ,
  char * nym ,
  char * domain ,
  char * message ,
  size_t msz ,
  int * notify )
{
  static const char * func = "FMGMessageSend";
  int return_code = 0;
  ZkEmailHeader header = NULL;
  FrRemoteNym rnym = NULL;
  NymServerResponse resp_nym;
  MaipServerResponse resp_maip;
  char * message_copy = NULL;
  char * message_copy2 = NULL;
  char * message_copy3 = NULL;
  char * email_domain = NULL;
  uint32_t spam_ctl = 0;           /* Target nym's spam controls */
  size_t email_domain_sz = 0;
  size_t msz_copy = 0;
  size_t msz_copy2 = 0;
  size_t msz_copy3 = 0;
  FrCryptCrypt crypt = NULL; /* Nym public cryption key */
  FrCryptSig sig = NULL; /* Nym public signature key */
  

  FrCryptPubKey fmgMaipKey = NULL;
  FrCryptPubKey myCurrentSigKey = NULL;

  zkErrorClear(err);
  if (context && nym && message && domain && notify) {

    /* By default, always inform the sender if the message failed */

    *notify = 1;
 
    /*
     * Create the AMTP message header (this should look the same as what the
     * client will have to send out.
     */

    if (!zkEmailHeaderNew(err,&header)
        || !zkEmailHeaderAddFieldData(err,header,"From",sender)
        || !zkEmailHeaderAddFieldData(err,header,"Comments",FMG_COMMENT)
        || !zkEmailHeaderAddFieldData(err,header,"X-ZKS-Freedom-Comment",FMG_COMMENT)) {
      zkLogWriteContext(context->log,err,func,"Unable to create message header");
      goto finish;
    }

    /*
     * Make a connection to the nym server, retrieve the target nym and pull
     * out its public keys.  Encrypt the message and send it to the MAIP
     * The error logging here is a bit tricky since we dont want to 
     * lose the message from the nym server and the maip if something
     * went wrong.
     */

    if (!frRemoteNymGet(err,&rnym,nym,&(context->nymserv),&resp_nym)) {
      if (zkErrorCode(err)) {
        zkLogWriteContext(context->log,err,func,"Failed to retrieve nym");
      } else {
        zkLogWrite_va(context->log,"%s:%d response was %d '%s'",
                      context->nymserv.nymserver,context->nymserv.port,
                      resp_nym.code,resp_nym.message);
        zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKERROR,
                      "Unable to lookup nym \"%s\" from nymserver",nym);
      }
      goto finish;
    }
    if (!frRemoteNymGetParameter(err,rnym,NymEmailDomain,
                                 (const unsigned char **) &email_domain,
                                 &email_domain_sz)) {
      zkLogWrite_va(context->log,"Could not retrieve email domain for %s",nym);
      goto finish;
    }

    /*
     * Get the target nym's spam control flags.  This function call should
     * always succeed.
     */

    if (!frRemoteNymGetSpamControls(err,rnym,&spam_ctl)) {
      zkLogWriteContext(context->log,err,func,"Failed to retrieve spam controls");
      goto finish;
    }

    /*
     * Check the spam control flags
     */

    if (spam_ctl & NYM_SPAM_CTL_BLACKHOLE_LIST) {
      int eat_message = 0;       /* Should I drop the mail? */
      if (!FMGContextCheckBlackholeList(err,context,
                                        sending_host,&eat_message)) {
        zkLogWriteContext(context->log,err,func,"Couldn't check black hole list");
        goto finish;
      }
      if (eat_message) {
        zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKERROR,
                      "Blackhole list dropped message for %s",nym);
        goto finish;
      }
    }

    /*
     * Now check that the mail domain match the nym's email domain.
     */
   
    {
      /* used extra braces so I could declare variables */
      size_t tgt_domain_len = strlen(domain);
      size_t real_domain_len = strlen(email_domain);
      
      if  ((tgt_domain_len < real_domain_len)
           || (strcasecmp(email_domain, domain+tgt_domain_len-real_domain_len))) {
        zkLogWrite_va(context->log,
                      "Trying to send mail to %s@%s, email domain should be %s",
                      nym,domain,email_domain);
        zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKERROR,
                      "Sending to wrong email domain for %s",nym);
        goto finish;
      }
    }

    if (!frRemoteNymGetKeys(err,rnym,&sig,&crypt)) {
      zkLogWrite_va(context->log,"Could not retrieve keys for %s",nym);
      goto finish;
    }

    /*
     * Encrypt message here 
     * 1. Encrypt message using target nym's public encryption key
     * 2. Sign message using FMG private signature key
     * 3. Encrypt message again using the FMG maip public key
     *
     */
    message_copy = NULL;
    if (!frSecPubKeyEncryptData(err,crypt,FRCRYPT_SYMALG_BLOWFISH,message,msz,
                                (void **) &message_copy2,&msz_copy2)) {
      zkLogWriteContext(context->log,err,func,
                        "Failed to encrypt message with target nym key");
      goto finish;
    }

    /* Get my current signing key from my keyChain */
    if (!frSecKeyChainGetVersion(err, context->kc, context->sigHandle,
                                 0, &myCurrentSigKey)) {
      zkLogWriteContext(context->log,err,func,
                        "Failed to retrieve my private signature key");
      goto finish;
    }

    if (!frSecSignData(err, myCurrentSigKey ,message_copy2,msz_copy2,
                       (void **) &message_copy3,&msz_copy3)) {
      zkLogWriteContext(context->log,err,func,
                        "Failed to sign data with my private key");
      goto finish;
    }

    free(message_copy2);
    message_copy2 = NULL;

    {
      struct protoent *ppe = NULL;
      /* Create a socket for the key queries */
      if ((ppe = getprotobyname("udp")) == NULL) {
        zkLogWrite(context->log, "getprotobyname() failed for \"udp\"\n");
        goto finish;
      } else {
        if ((context->kh.fd = socket(AF_INET, SOCK_DGRAM, ppe->p_proto)) < 0) {
          zkLogWrite_va(context->log, "Error creating socket (%s).\n", 
                        strerror(errno));
          goto finish;
        }
      }
    }

    /* Get the current public encryption key for the FMG maip */
    if (!frKeyQryFetchPubKey(err, context->kq, 0, FRCRYPT_KEYTYPE_CRYPTION, 
                             context->fmgMaipEntity, context->kh, 
                             context->kqretries, context->kqtimeout,
                             &fmgMaipKey)) {
      zkLogWriteContext(context->log,err,func,"Failed to fetch key for %s (%d)",
                    context->fmgMaipEntity.name,context->fmgMaipEntity.type);
      goto finish;
    }                             

    close(context->kh.fd);
    context->kh.fd = -1;

    if (!frSecPubKeyEncryptData(err, fmgMaipKey, FRCRYPT_SYMALG_BLOWFISH,
                                message_copy3,msz_copy3,
                                (void **) &message_copy,&msz_copy)) {
      zkLogWriteContext(context->log,err,func,"Failed to encrypt message with FMG key");
      goto finish;
    }


    /*
     * Send encrypted message to the maip
     */

    if (!frAmtpSendMessageToNym(err,&(context->maip),&resp_maip,header,
                                message_copy,msz_copy,nym)) {
      
      /* 
       * _COMMAND_NOTEXECUTED is returned by the FMG maip if this sender
       * was blocked by the recipient nym.  In this case, we dont want to 
       * notify the sender.
       */

      if (resp_maip.code == _COMMAND_NOTEXECUTED) {
        *notify = 0;
      }

      /*
       * Log it!
       */

      if (zkErrorCode(err)) {
        zkLogWriteContext(context->log,err,func,"frAmtpSendMessageToNym failed");
      } else {
        zkLogWrite_va(context->log,"%s:%d response was %d '%s'",
                      context->maip.maip,context->maip.port,
                      resp_maip.code,resp_maip.message);
        zkErrorSet(err,func,ZKERR,ZKMODULE,ZKERROR,
                   "Unable to complete transaction with MAIP");
      } 
      goto finish;
    }

    /*
     * Message successfully sent!
     */

    return_code = 1;

finish:

    if (context->kh.fd > 0) {
      close(context->kh.fd);
      context->kh.fd = -1;
    }

    if (header) {
      zkEmailHeaderFree(NULL,&header);
    }
    if (rnym) {
      frRemoteNymFree(NULL,&rnym);
    }
    if (message_copy) {
      memset(message_copy,0,msz);
      free(message_copy);
    }
    if (message_copy2) {
      memset(message_copy2,0,msz_copy2);
      free(message_copy2);
    }
    if (message_copy3) {
      memset(message_copy3,0,msz_copy3);
      free(message_copy3);
    }
    if (sig) {
      frCryptSigFree(NULL,&sig);
    }
    if (crypt) {
      frCryptCryptFree(NULL,&crypt);
    } 
    if (rnym) {
      frRemoteNymFree(NULL,&rnym);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMODULE,ZKENULLPTR,NULL);
  return 0;
}

/*
 * This function will analyze the value of an RFC822 "Received" mail header
 * field in order to find the sending hostname of the message.  It will 
 * return a pointer into the field value string which indicates the beginning
 * of the hostname, as well as its size.
 *
 * Why is this not in libzkmime?  This function is so specific to this 
 * particular application that it is pretty much useless to put in the mime
 * library.  Feel free to disagree, but if you do and want to move it, you'll
 * have to make this a general parser for the Received field, something which
 * I dont have time to do.
 *   --philippe (philippe@zks.net)
 */

static int
FMGGetSendingHost (
  ZkError err ,
  char * rcvd_fld ,       /* Field value from an 822 "Received" field */
  char ** sending_host ,  /* Returns a pointer inside rcvd_fld */
  size_t * sz )           /* Returns the size of the sending host name */
{
  static const char * func = "FMGGetSendingHost";
  int ret = 0;      /* Function return code */
  size_t sh_sz = 0; /* Size of the sending host name */
  char * sh = NULL; /* Pointer to the start of the sending_host name in the 
                     * field value rcvd_fld
                     */

  if (rcvd_fld && sending_host && sz) {

    /*
     * Look for the first instance of "from" followed by a whitespace
     * character.  (see the grammar for the Received field in RFC822, page 17)
     */

    for (;;) {

      /* Look for the next "from*" substring */
      if ((sh = strstr(rcvd_fld,"from")) == NULL) {
        zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKENOTFOUND,
                     "Could not find sending host in Received field: \"%s\"",
                      rcvd_fld);
        break;
      }
    
      /* 
       * Move past the "from" substring to the next available character and
       * check to see if it is white space. If it is not, continue the loop.
       * (if "from" is the last 4 bytes of the string, sending_host will
       * point to "\000" and the strstr() call will return NULL, ending the 
       * loop.
       */

      sh = &(sh[4]);
      if (!isspace(*sh)) {
        rcvd_fld = sh;
        continue;
      } 

      /*
       * Move past all of the white space
       */

      while (isspace(*sh)) {
        sh++;
      }

      /*
       * Now compute the size of the sending host name
       */

      sh_sz = 0;      
      while (sh[sh_sz] && !isspace(sh[sh_sz])) {
        sh_sz += 1;
      }
      
      /*
       * We're done.  Return the sending_host pointer and the size of the
       * name.
       */

      ret = 1;
      *sending_host = sh;
      *sz = sh_sz;
      break;
    }
   
  } else {
    zkErrorSet(err,func,ZKERR,ZKMODULE,ZKENULLPTR,NULL);
  }
  return ret;
}

/*
 *
 */

static int
FMGProcessMessage (
  ZkError err ,
  FMGContext * context ,
  ZkEmail message )
{
#define SENDING_HOST_SZ 255
  static const char * func = "FMGProcessMessage";
  int return_code = 0;
  ZkEmailHeader header = NULL;
  ZkEmailField field = NULL;
  const unsigned char * value = NULL;
  char * buffer = NULL;
  size_t bsz = 0;
  ZkList recipient = NULL;
  char sending_host[SENDING_HOST_SZ + 1] = "";  /* Hostname of the machine which sent mail */
  char * from = NULL;
  char * addr = NULL;
  int recipients = 0;
  enum ZkMIMEFieldId field_id = UnknownField;
  char * received = NULL;  /* Value string of first received field */
  char * domain = NULL; /* Used to remove the domain name in the nym address */
  char * email = NULL; /* Aliased email address */
  int found = 0; /* Flag indicating if address is in the alias list */
  const char * src_subject = NULL; /* Subject line of original message */
  int notify = 1;  /* If the send fails, notify the message sender */

  zkErrorClear(err);
  if (context && message) {

    /* Clean up this string buffer */
    memset(sending_host,0,SENDING_HOST_SZ + 1);

    if (!(zkListNew(err,&recipient) &&
          zkListSetDataFreeFunction(err,recipient,zkListDataFree,NULL))) {
      return 0;
    }
  
    /*
     * Get and process contents of the To: and CC: fields, as well as the
     * FIRST Received: field (for blackhole spam control).
     */

    if (!zkEmailHeader(err,message,&header)) {
      goto finish;
    }
    if (!zkEmailHeaderFirstField(err,header,&field)) {
      goto finish;
    }
    do {
      if (!zkEmailFieldId(err,field,&field_id)) {
        goto finish;
      }
      switch (field_id) {
      case cc:
      case To:
        if (zkEmailFieldValue(err,field,&value)) {
          if (!zkEmailUtilAddToRecipientList(err,recipient,value)){
            goto finish;
          }
        }  
        break;
      case From:
        if (!zkEmailFieldValue(err,field,(const unsigned char **) &from)) {
          goto finish;
        } else {
          zkEmailUtilCleanAddress(err,from);
        }
        break;
      case Subject:
        if (!zkEmailFieldValue(err,field,
                               (const unsigned char **) &src_subject)) {
          src_subject = "(no subject)";
        }
        break;

        /*
         * Get the hostname which just sent us this message.  We'll use this
         * value if the recipient nym has specified that it want blackhole
         * spam control.
         */

      case Received:
        /* Get the first received field */
        if (sending_host[0] == (char) 0) {
          const char * DFLT_SENDER = "unknown_sending_host";
          char * sh = NULL;  /* To get a pointer inside the value string */
          size_t sh_sz = 0;  /* Size of the sending host name */
          if (!zkEmailFieldValue(err,field,
                                 (const unsigned char **) &received)) {
            zkLogWriteContext(context->log,err,func,"Failed to get 'received' field");
            break;
          }

          /*
           * Some of the Received fields sometimes contain a comment which 
           * identifies the sending host.  The parsing routines will ignore the
           * comment, so we need to stick in a default value so that we can
           * keep going.  This value will only be used by the spam control,
           * only when it is enabled for a particular nym.
           */

          if (!FMGGetSendingHost(err,received,&sh,&sh_sz)) {
            zkLogWriteContext(context->log,err,func,
                              "WARNING:  setting sending host to \"%s\"",
                              DFLT_SENDER);
            sh_sz = (size_t) strlen(DFLT_SENDER);
            if (!zkMemDup(err,(void **) &sh,DFLT_SENDER,sh_sz)) {  
              zkLogWriteContext(context->log,err,func,
                                "Failed to duplicate default sending host");
              break;
            }
#if 0
            break;
#endif
          }
          if (sh_sz > SENDING_HOST_SZ) {
            zkErrorSet(err,func,ZKERR,ZKMODULE,ZKERROR,
                       "sending hostname size is too long");
            goto finish;
          }
          strncpy(sending_host,sh,sh_sz);
          sending_host[sh_sz] = (char) 0;
        } 
        break;
      default:
        break;
      }
    } while (zkEmailHeaderNextField(err,header,&field));

    /*
     * Sanity check.  Each mail should have at least one "Received:" field
     * in its main header.
     */

    if (sending_host[0] == (char) 0) {
      zkErrorSet(err,func,ZKERR,ZKMODULE,ZKEDATABAD,
                 "Warning, message did not contain a \"Received\" field");
      goto finish;
    }
   
    zkListSize(err,recipient,&recipients);  
    if (recipients == 0) {  
      zkErrorSet(err,func,ZKERR,ZKMODULE,ZKEDATABAD,
                 "Message header did not contain recipients");
      goto finish;
    }
    if (from == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMODULE,ZKEDATABAD,
                 "Message did not contain a From: field");
      goto finish;
    }

    /*
     * Go through the recipient list and pull out all of the target nyms.  
     * ignore other email addresses, since they will have received the message
     * through the regular channels
     */

    {
      ZkList target_nyms = NULL;

      if (!zkEmailUtilSplitRecipientsByDomain(err,recipient,&target_nyms,
                                              context->ci.domain,
                                              FLAG_IGNORE_CASE)) {
        goto finish;
      }
      zkListFree(err,&recipient);
      recipient = target_nyms;
      target_nyms = NULL;
    }
  
    /*
     * Now convert the message to a buffer
     */

    if (!zkEmailBuffer(err,message,&buffer,&bsz)) {
      goto finish;
    }

    /*  
     * Send the message for each valid target 
     */

    zkListMoveCursor(err,recipient,LIST_HEAD);
    while (zkListGet(err,recipient,(void *) &addr,LIST_FORWARDS)) {
    
      /*
       * Remove the domain name from the nym address if required.
       * %%Note:  At some point, we might want to check the domain name 
       *          against the nym's email domain in the nym server.  This would
       *          require a set of rules however, since some domains would be
       *          "supersets" of others.  EG: if a user starts off with a trial
       *          nym, nym@trial.freedom.net, that domain is the only one to 
       *          which email for that nym can be sent.  When that nym is 
       *          upgraded to a real nym, email to nym@trial.freedom.net AND
       *          mail to nym@freedom.net should both work
       */

      if ((domain = strchr(addr,'@')) != NULL) {
        *domain++ = (char) 0;
      } else {
        zkErrorSet_va(err,func,ZKERR,ZKMODULE,ZKERROR,
                      "No email domain specified for target nym %s",addr); 
        zkLogWrite_va(context->log,"Missing email domain for target nym %s",
                      addr);
        goto finish;
      }
        

      /*
       * Check to see if this target is aliased.  If so send the message
       * via normal mail to the new address.  NOTE:  This is mostly a debugging
       * feature.  The normal way to redirect the mail would be to create
       * a proper nym. (This applies to things like fmg, abuse, postmaster, etc)
       */

      FMGAliasLookup(err,context->alias,addr,(const char **) &email,
                     &found);
      if (found) {
        if (!FMGSendTextMessageToInternetUser(err,context,email,
                                              "Redirected mail",buffer)) {
          zkLogWriteContext(context->log,err,func,
                            "FMGSendTextMessageToInternetUser failed");
        }
         
        /*
         * If we have a message for a aliases, send it
         */

        if ((context->aliasmsg) && (context->amsz)) {
          char new_subj[64];
          snprintf(new_subj,64,"RE: %s",src_subject);
          if (!FMGSendTextMessageToInternetUser(err,context,from,new_subj,
                                                context->aliasmsg)) {
            zkLogWriteContext(context->log,err,func,
                              "FMGSendTextMessageToInternetUser failed (alias)");
          }
        }

      } else if (!FMGMessageSend(err,context,from,sending_host,addr,domain,
                                 buffer,bsz,&notify)) {
        char subject[128];

        zkLogWriteContext(context->log,err,func,
                          "Unable to send message to \"%s\" from \"%s\"",
                          addr,from);
        /*
         * I failed to send the message VIA AMTP to the first MAIP, so
         * I'll try to notify the internet user
         */
        snprintf(subject,sizeof subject,"Unable to deliver message to nym \"%s\"",addr);
        
        if (notify) {
          if (!FMGSendTextMessageToInternetUser(err,context,from,subject,
                                                context->notfoundmsg)) {
            zkLogWriteContext(context->log,err,func,
                              "FMGSendTextMessageToInternetUser failed (delivery failure)");
          }
        }
      } else {
        zkLogWrite_va(context->log,"Message sent to %s from %s",addr,from);
      }
      if (++recipients >= context->max_recipients) {
        break;
      }
    }
    return_code = 1;

finish:
    if (context->archive) {
      if (!FMGArchiveMessage(err,message,context->archive)) {
        zkLogWriteContext(context->log,err,func,"FMGArchiveMessage failed");
      }
    }
    if (buffer) {
      free(buffer);
    }
    if (recipient) {
      zkListFree(NULL,&recipient);
    }

    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMODULE,ZKENULLPTR,NULL);
  return 0;
}

/**
 * Each message spawns off a child process to handle the incoming message.
 * This is the function that is called using the context we can get at
 * keys, ...
 */
static int FMGChild(ZkError err, FMGContext *context, int sock)
{
  const char *func = "FMGChild";
#define LINE_SZ 2048
  char line[LINE_SZ + 1];
  size_t line_sz = 0;
  FILE * f = NULL; /* For saving a copy of the original message */
  char pathname[MAXPATHLEN + 1];
  int partial_message = 0;
  
  if (!context) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "context is NULL");
    goto fail;
  }

  if (sock == -1) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKEPARMBAD, "sock descriptor is -1");
    goto fail;
  }

  /* Dup the socket descriptor into a regular stdin/stdout pair */
  dup2(sock, 0);
  dup2(sock, 1);
  close(sock);

  context->exit_code = EXIT_FAILURE; /* pessimistic */
  zkLogWrite(context->log,"Processing new mail message");

  /*
   * Get all of the input we can.... If we have set an archive directory,
   * log the entire message so we can compare what goes into the filter
   * with what comes out.
   */

  f = NULL;
  if (context->archive) {
    snprintf(pathname,sizeof pathname,"%s/FMG.%d.source.XXXXXX",context->archive,(int)getpid());
    if ( ! zkFileTempFp(err,pathname,&f)) {
      zkLogWriteContext(context->log,err,func,"Can't create source archive");
      f = NULL;
    } else {
      zkLogWrite_va(context->log,"Logging incoming message to \"%s\"",
                    pathname);
    }
  }

  while (!feof(context->input)) {
    line_sz = fread(line,1,LINE_SZ,context->input);
    if (line_sz < LINE_SZ) {
      if (ferror(context->input)) {
        zkErrorSet(err,"fread",ZKERR,ZKMODULE,ZKEREAD,
                      "Error reading from input stream");
        zkLogError(context->log,err);
        goto fail;
      } 
    }
    if (!zkEmailAddText(err,context->email,(unsigned char *)line,
                        line_sz)) {
      zkLogWriteContext(context->log,err,func,"Can't append data to mail record");
      goto fail;
    } else if (f) {
      fwrite(line,1,line_sz,f);
    }
  }

  /*
   * If we are logging, close the log file 
   */

  if (f) {
    fclose(f);
  }

  /*
   * Parse the mail and convert it to MIME
   */

  if (!zkEmailParse(err,context->email)) {
    zkLogWriteContext(context->log,err,func,"Can't parse message");
    goto fail;
  }

  /*
   * Is this a multipart/partial message?  If so, dont mess with it too
   * much, otherwise, the recipient's email reader will not be able to 
   * reassemble the message.
   */

  partial_message = zkEmailIsPartialMessage(err,context->email);

  if (!partial_message) {
    if (!zkEmailConvertToMIME(err,context->email)) {
      zkLogWriteContext(context->log,err,func,"Can't convert message to MIME");
      goto fail;
    }
  }

  /*
   * If the FMG has a banner specified, insert it into the mail  message
   */
  
  if (!partial_message && context->banner) {
    if (!zkEmailAddHtmlBlipvert(err,context->email, 1,
                                context->banner,context->bsz)) {
      zkLogWriteContext(context->log,err,func,"Can't add HTML blip-vert");
      goto fail;
    }
  }

  /*
   * Now lookup the target nym, encrypt the message, get the reply blocks
   * and forward the whole thing to a MAIP. (do additionaly processing as
   * required at this point)
   */
  if (!FMGProcessMessage(err,context,context->email)) {
    zkLogWriteContext(context->log,err,func,"Can't process message");
    goto fail;
  }
  context->exit_code = EXIT_SUCCESS;

 fail:  
  /*
   * all done.
   */
  exit(context->exit_code);
  return 0; /* to shut up insure */
}
  
/*
 * Create a socket and bind to the port found in the context.  Set up a
 * listen() on it.
 */
static int FMGBindPort(ZkError err, FMGContext *c)
{
  const char *func = "FMGBindPort";
  int one = 1;
  int s = -1;
  struct sockaddr_in myAddr;
  
  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "context is NULL");
    return 0;
  }

  if (c->port < 1024) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKEDATABAD,
                  "Port to bind to is too low %d", c->port);
    return 0;
  }

  if (c->listenq < 1) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKEDATABAD,
                  "Listen queue is too small %d", c->listenq);
    return 0;
  }

  if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKESOCKET, "socket(%s)",
                  strerror(errno));
    return 0;
  }

  if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(int)) 
      < 0) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKESOCKET, "setsockopt(%s)",
                  strerror(errno));
    close(s);
    return 0;
  }

  memset(&myAddr, 0, sizeof(struct sockaddr_in));
  myAddr.sin_family = AF_INET;
  myAddr.sin_port = htons(c->port);
  myAddr.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(s, (struct sockaddr *) &myAddr, sizeof(struct sockaddr_in)) < 0) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKESOCKET, "bind(%s)",
               strerror(errno));
    close(s);
    return 0;
  }

  if (listen(s, c->listenq) < 0) {
    zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKESOCKET, "listen(%s)",
               strerror(errno));
    close(s);
    return 0;
  }

  c->listenFd = s;
  return 1;
}


/*
 *
 */
static int startNiChild
(
  ZkError     err, 
  FMGContext *c, 
  pid_t      *pid,
  ZkBool      bRestart)
{
  const char *func = "startNiChild";
  void *buf = NULL;
  size_t bsz = 0;
  int ret = 0;

  if (!c || !pid) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param(s)");
    return 0;
  }


  if (!bRestart) {
    if (!frNetInfoChildCtxNew(err, &c->niCtx)) {
      goto nifail;
    }

    if (!frNetInfoChildCtxSetUpdateOffset(err, c->niCtx,
					  c->niqsFullUpdatePeriod)) {
      goto nifail;
    }

    if (!frNetInfoChildCtxSetNIQSParms(err, c->niCtx, c->niqsHost,
				       c->niqsPort, c->niqsTimeout)) {
      goto nifail;
    }

    if (!frNetInfoChildCtxSetPollParms(err, c->niCtx, c->niReloadPeriod, 
				       c->niReloadPeriod / 2)) {
      goto nifail;
    }
  }


  *pid = 0;

  if (!frNetInfoStore(err, c->ni, &buf, &bsz)) {
    goto nifail;
  }


  if (bRestart) {
    if (!frNetInfoChildRefresh(err, c->niCtx)) {
      goto nifail;
    }
  }
  else {
    if (!frNetInfoChildCreate(err, c->niCtx, 
			      c->niSharedMemSz, 
			      c->niShmTemplate,
			      c->niMtxTemplate)) 
      {
	goto nifail;
      }
  }

  if (!frNetInfoChildPid(err, c->niCtx, pid)) {
    goto nifail;
  }

  ret = 1;

 nifail:
  if (buf) {
    free(buf);
    buf = NULL;
  }
  return ret;
}


/*
 * Check if I received any signals and handle it if it is set.
 */
static int maybeHandleSignal(ZkError err, FMGContext *c)
{
  const char  *func = "maybeHandleSignal";
  pid_t        childPid;
  int          status;
  int          bOk = 1;

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param");
    return 0;
  }

  if (reloadKeys) {
       /* Signal caught, reload keys */
    if (!frSecKeyChainReloadKeys(err, c->kc)) {
      return 0;
    }
    reloadKeys = 0;
  }

  if (reloadClientInfo) { /* Signal caught, reload clientinfo */
    ZkipClient cli = NULL;

    /* Assume the sanity check was done properly in FMGNiqsInit */

    if (!frNiqsConnect(err, &cli, c->niqsHost, c->niqsPort, 
                      c->niqsTimeout)) {
      zkLogWrite_va(c->log, 
                    "Unable to connect to %s:%d to update clientinfo, will soon retry\n",
                    c->niqsHost, c->niqsPort);
      bOk = 0 ;
    } else {
      /*
       * Get the basic server clientinfo so that we can know what the freedom
       * product domain name is.
       */
      if (!frNiqsGetClientInfo(err,cli,c->kh,c->kq,
                               FREEDOM_SERVER_UNIX_CURRENT,&(c->ci))) {
        zkLogWriteContext(c->log,err,func,"frNiqsGetClientInfo failed");
        bOk = 0 ;
      } else {
        /*
         * The client info code will take care of default values, however, if the
         * NIQS version of the clientinfo has the domain explicitly set to an empty
         * string, the proper default will be used and this message will be put into
         * the log file.
         */
        reloadClientInfo = 0;

#ifdef FREEDOM_DEVELOPMENT_BUILD

        zkLogWrite_va(c->log,
                      "Clientinfo reloaded. New domain:<%.*s>\n",
                      (int)sizeof(c->ci.domain),
                      c->ci.domain);
#endif

        if (c->ci.domain[0] == (char) 0) {
          snprintf(c->ci.domain,CI_DOMAIN_SZ,"%s",FR_DFLT_FREEDOM_DOMAIN); 
          zkLogWrite_va(c->log,
                        "WARNING:  Unable to find freedom domain name, "
                        "using %s as default",c->ci.domain); 
        }
      }
      if (cli) {
        (void) frNiqsDisconnect(err, &cli);
      }
    }
  }

  if (sigchild) {
    /* 
     * I've found that I can really screw things up by creating a whole
     * bunch of connects and killing them all at once.  I seem to be missing
     * signals, so this loop makes sure to kill all exited children when a 
     * SIGCHILD is received.
     */
    while ((childPid = waitpid(-1,&status,WNOHANG)) > 0) {
      if (childPid == c->niPid) {
        /* oh-oh log this... */
        zkLogWrite(c->log, "Child process to update netinfo object failed");

        /* Warning: If the NetInfoChild process exits right
           after it starts, FMG might enter an endless loop here */
        if (!startNiChild(err, c, &c->niPid, ZK_TRUE)) {
          return 0;
        }
      }
    }
    sigchild = 0; /* Hope we have not receive another one in the (short) mean time */
  }

  return bOk;
}


/*
 * Try to get a serialized buffer of the NetInfo object.
 */
static int FMGUpdateNi(ZkError err, FMGContext *c)
{
  const char *func = "FMGUpdateNi";
  void *buf = NULL;
  size_t bsz;
  int ret = 0;
  int changed;
  FrNetInfo ni = NULL;

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param");
    return 0;
  }

  if (!frNetInfoChildCtxGetNiBuf(err, c->niCtx, &changed, &buf, &bsz)) {
    goto bfail;
  }

  if (changed) {
    struct sockaddr_in addr;

    if (c->ni) {
      ni = c->ni;
      c->ni = NULL;
    }

    if (!frNetInfoNew(err, &c->ni)) {
      goto bfail;
    }

    if (!frNetInfoLoad(err, c->ni, c->kq, buf, bsz)) {
      goto bfail;
    }

    if (!FMGNetInfoGetType(err, c, FRENT_TYPE_KEYQRY_SRV, &addr, NULL)) {
     goto bfail;
    }

    c->kh.host.sin_addr = addr.sin_addr; /* sin_family was previously assiged to AF_INET */
    c->kh.host.sin_port = addr.sin_port;

    if (!FMGNetInfoGetType(err, c, FRENT_TYPE_FMG, &addr, &c->fmgMaipEntity)) {
     goto bfail;
    }

    strncpy(c->maip.maip, inet_ntoa(addr.sin_addr), MSHOST_SZ);
    c->maip.port = ntohs(addr.sin_port);

    if (!FMGNetInfoGetType(err, c, FRENT_TYPE_NYM_SRV, &addr, NULL)) {
     goto bfail;
    }

    strncpy(c->nymserv.nymserver, inet_ntoa(addr.sin_addr), MSHOST_SZ);
    c->nymserv.port = ntohs(addr.sin_port);
  }

  (void) frNetInfoFree(err, &ni);
  ni = NULL;

  ret = 1;

#ifdef FREEDOM_DEVELOPMENT_BUILD

  if (c->niDumpOnUpdate) {
    frNetInfoDumpInLog(err,c->ni,FRENT_TYPE_KEYQRY_SRV,c->log);
    frNetInfoDumpInLog(err,c->ni,FRENT_TYPE_FMG,c->log);
    frNetInfoDumpInLog(err,c->ni,FRENT_TYPE_NYM_SRV,c->log);
  }

#endif

 bfail:
  if (buf) {
    free(buf);
    buf = NULL;
  }

  if (ni) {
    c->ni = ni;
    ni = NULL;
  }

  zkLogWrite_va(c->log, "NetInfo update %s\n", ret ? "succeeded" : "failed");
  return ret;
}

/*
 * Check if it's time to check if I need to update my NetInfo object.  If
 * I update it, reset all my other parameters for Freedom servers.  Update
 * the *last to the current time if I updated.
 */
static int maybeReloadNi(ZkError err, FMGContext *c, time_t *last)
{
  const char *func = "maybeReloadNi";
  time_t now = 0;

  if (!c || !last) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  time(&now);
  
  if ((unsigned long) now > c->niReloadPeriod + *last) {
    /* Update my objects and Freedom server pointers */
    if (!FMGUpdateNi(err, c)) {
      return 0;
    }
    *last = now;
  }

  return 1;
}

/**
 * FMG server thread which runs the accept() and fork() for each
 * message.
 */
static int FMGServer(ZkError err, FMGContext *c)
{
  const char *func = "FMGServer";
  int s = -1;
  struct sockaddr_in remote;
  int rLen;
  fd_set rset;
  int nsel = 0;
  struct timeval timeout;
  time_t last_ni = 0;
  time_t now = 0;

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "ctx var is NULL");
    return 0;
  }

  if (c->listenFd < 0) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKEPARMBAD, "Listen fd is -1");
    return 0;
  }

  if (!startNiChild(err, c, &c->niPid, ZK_FALSE)) {
    return 0;
  }

  /* Initial timeout value */
  timeout.tv_sec  = c->niReloadPeriod;
  timeout.tv_usec = 0;

  FD_ZERO(&rset);

  while (running) {
    time(&now);

    if (!maybeHandleSignal(err, c)) {
      zkLogWriteContext(c->log, err, func, "maybeHandleSignal failed");
      return 0 ;
    }

    if (!maybeReloadNi(err, c, &last_ni)) {
      zkLogWriteContext(c->log, err, func, "maybeReloadNi failed");
      return 0 ;
    }

    /* Update timeout value */
    if (now <= last_ni) {
      timeout.tv_sec  = last_ni + c->niReloadPeriod - now;
      timeout.tv_usec = 0;
    }

    FD_SET(c->listenFd, &rset);

    if ((nsel = zkSelectUpdateTimeout(err, c->listenFd + 1, &rset,
                                      NULL, NULL, &timeout)) < 0) {
      if (errno == EINTR) {
        /* Interrupted signal call */
        continue;
      } else {
        zkLogWrite_va(c->log, "%s: Unrecoverable error in select(%s)\n",
                      c->progname, strerror(errno));
        return 0;
      }
    } else if (nsel) {
      rLen = sizeof(struct sockaddr_in);
      if ((s = accept(c->listenFd, &remote, &rLen)) < 1) {
        if (errno == EINTR) {
          /* Interrupted signal call, this can't happen under Linux */
          continue;
        } else {
          zkLogWrite_va(c->log, "%s: Unrecoverable error in accept(%s)\n",
                        c->progname, strerror(errno));
          return 0;
        }
      }

      /* 
       * Bug: Check the remote address that connected to me, only allow
       * localhost.
       */         

      /* Fork a child to handle the message */
      switch (fork()) {
      case -1:
        zkErrorSet_va(err, func, ZKERR, ZKMODULE, ZKEFORK, "fork(%s) failed",
                      strerror(errno));
        return 0;

      case 0:                     /* child */
        (void)FMGChild(err, c, s);
        exit(EXIT_SUCCESS);
        break;

      default:                    /* parent */
        close(s);
        s = -1;
        break;
      }
    } else { /* nsel == 0 (timeout expired) */
      timeout.tv_sec  = c->niReloadPeriod;
      timeout.tv_usec = 0;
    }
  }
  /* SIGTERM or SIGINT handled */

  return 1;
}

static int FMGNetInfoGetType(ZkError err, FMGContext *c, FrEntIdType type,
                             struct sockaddr_in *addr, FrEntIdExp *ent)
{
  const char *func = "FMGNetInfoGetType";
  ZkList list = NULL;
  FrNetInfoDatum data = NULL;
  int found = 0;
  FrEntIdExp  myEntity;

  if (!c || !c->ni || !addr) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoRandomizeList(err, c->ni, type)) {
    return 0;
  }

  if (!frNetInfoGetListOf(err, c->ni, &list, type)) {
    return 0;
  }

  if (!list) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "Server type not found");
    return 0;
  }

  if (!zkListMoveCursor(err, list, LIST_HEAD)) {
    return 0;
  }

  /* We try to select a server marked as UP in NetInfo.
   * This is only a best guest.
   * If all server are marked as DOWN, we take the last one
   */
  while (!found && zkListGet(err, list, (void **)&data, LIST_FORWARDS)) {
    int  status;

    found = frNetInfoDatumGetStatus(err, data, &status) && status == 1;
  }

  if (!data                                                 || 
      (ent && !frNetInfoDatumGetEntity(err,data,&myEntity)) ||
      !frNetInfoDatumGetSockaddr(err, data, addr)) {
    return 0;
  }

  if (ent) {
    *ent = myEntity;
  }

#ifdef FREEDOM_DEVELOPMENT_BUILD
  {
    char       *str   = NULL;
    const int   szStr = 1024;

    if (ent) {
      if (zkMemAlloc(err,(void **)&str,szStr)) {
        frEntSnprint(err,str,szStr,&myEntity);
      }
    } else if (frNetInfoDatumGetEntity(err,data,&myEntity) &&
               zkMemAlloc(err,(void **)&str,szStr)) {
      frEntSnprint(err,str,szStr,&myEntity);
    }

    zkLogWrite_va(c->log,
                  "new server  IP: %s  port:%d - %.*s\n\t",
                  inet_ntoa( addr->sin_addr ),
                  addr->sin_port,
                  szStr,
                  str?str:"???");

    if (str) {
      zkMemFree(err,(void**)&str);
    }
  }
#endif

  return 1;
}

/*
 * Load up the initial parameters from the NIQS server and populate my
 * FrNetInfo object.  Get IP addr/ports for other Freedom servers that I
 * need to talk to (fmgmaip, keyqry, nymserv).
 */
static int FMGNiqsInit(ZkError err, FMGContext *ctx)
{
  const char *func = "FMGNiqsInit";
  int connected = 0;
  ZkipClient cli = NULL;
  int ret = 0;
  struct sockaddr_in addr;

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKENULLPTR, "NULL param");
    return 0;
  }

  /*
   * Check NIQS parameters
   * (The fourth condition makes sure NIQS address is not in IP dot notation)
   */

  if (!ctx->niqsHost                                    ||
      (ctx->niqsPort                            < 1024) ||
      (ctx->niqsTimeout                         <    1) ||
      (inet_aton(ctx->niqsHost, &addr.sin_addr) !=   0)) {
    zkErrorSet(err, func, ZKERR, ZKMODULE, ZKEPARMBAD, "Bad NIQS parameters");
    return 0;
  }

  /* Allocate space for the objects that will be used */
  if (!ctx->kq) {
    if (!frKeyQryNew(err, &ctx->kq)) {
      return 0;
    }
  }

  if (!ctx->ni) {
    if (!frNetInfoNew(err, &ctx->ni)) {
      return 0;
    }
  }

  while (!connected) {
    if (frNiqsConnect(err, &cli, ctx->niqsHost, ctx->niqsPort, 
                      ctx->niqsTimeout)) {
      connected = 1;
    } else {
      zkLogWrite_va(ctx->log, 
                    "Unable to connect to %s:%d, retry in 60 seconds\n",
                    ctx->niqsHost, ctx->niqsPort);
      sleep(60);
    }
  }

  if (!frNetInfoPopulate(err, ctx->ni, cli, ctx->kq)) {
    goto niqsdone;
  }

  /* Get the key query server */
  if (!FMGNetInfoGetType(err, ctx, FRENT_TYPE_KEYQRY_SRV, &addr, NULL)) {
    goto niqsdone;
  }
  ctx->kh.host.sin_family = AF_INET;
  ctx->kh.host.sin_addr = addr.sin_addr;
  ctx->kh.host.sin_port = addr.sin_port;

  /*
   * Get the basic server clientinfo so that we can know what the freedom
   * product domain name is.
   */

  if (!frNiqsGetClientInfo(err,cli,ctx->kh,ctx->kq,
                           FREEDOM_SERVER_UNIX_CURRENT,&(ctx->ci))) {
    zkLogWriteContext(ctx->log,err,func,"frNiqsGetClientInfo failed");
  }

  /*
   * The client info code will take care of default values, however, if the
   * NIQS version of the clientinfo has the domain explicitly set to an empty
   * string, the proper default will be used and this message will be put into
   * the log file.
   */

  if (ctx->ci.domain[0] == (char) 0) {
    snprintf(ctx->ci.domain,CI_DOMAIN_SZ,"%s",FR_DFLT_FREEDOM_DOMAIN); 
    zkLogWrite_va(ctx->log,
                  "WARNING:  Unable to find freedom domain name, "
                  "using %s as default",ctx->ci.domain); 
  }
  
  /* FMG Maip */
  if (!FMGNetInfoGetType(err, ctx, FRENT_TYPE_FMG, &addr, &ctx->fmgMaipEntity)) {
    zkLogWriteContext(ctx->log,err,func,"FMGNetInfoGetType failed (fmg-maip)");
    goto niqsdone;
  }
  strncpy(ctx->maip.maip, inet_ntoa(addr.sin_addr), MSHOST_SZ);
  ctx->maip.port = ntohs(addr.sin_port);
  
  /* Nym Server */
  if (!FMGNetInfoGetType(err, ctx, FRENT_TYPE_NYM_SRV, &addr, NULL)) {
    zkLogWriteContext(ctx->log,err,func,"FMGNetInfoGetType failed (nym server)");
    goto niqsdone;
  }
  strncpy(ctx->nymserv.nymserver, inet_ntoa(addr.sin_addr), NSHOST_SZ);
  ctx->nymserv.port = ntohs(addr.sin_port);

  ret = 1;
 niqsdone:   
  if (cli) {
    (void) frNiqsDisconnect(err, &cli);
  }

  return ret;
}

/*
 * main program
 */
int main(int argc, char **argv)
{
  static const char * func = "main";
  FMGContext context;
  ZkError err = NULL;

  zkErrorNew(&err);

  if (!frCryptInit(err, 0)) {
    fprintf(stderr, "%s: Unable to initialize crypto\n", argv[0]);
    goto finish;
  }

  if (!FMGContextLoad(err, &context, argc, (const char **)argv)) {
    fprintf(stderr, "%s: Unable to intialize context.\n", context.progname);
    goto finish;
  }

  /* Set up my listen socket */
  if (!FMGBindPort(err, &context)) {
    fprintf(stderr, "%s: Unable to bind to my port %d.\n", context.progname,
            context.port);
    goto finish;
  }

  if (context.fork) {           /* Put myself in the background */
    pid_t pid = 0;

    if ((pid = fork()) == 0) {
      if (context.pidfile) {
        if (!zkFileWritePid(err, context.pidfile, &context.fptr)) {
          fprintf(stderr,"%s:  Unable to write to \"%s\".\n",context.progname,
                  context.pidfile);
          exit(EXIT_FAILURE);
        }
      } 
    } else {
      fprintf(stderr,"%d\n",pid);
      exit(EXIT_SUCCESS);
    }
  } else {
    if (context.pidfile) {
      if (!zkFileWritePid(err, context.pidfile, &context.fptr)) {
        fprintf(stderr,"%s:  Unable to write to \"%s\".\n",context.progname,
                context.pidfile);
        exit(EXIT_FAILURE);
      }
    } 
  }

  /* Set up my signal handlers */
  zkSignal(SIGTERM, sig_handler);
  zkSignal(SIGINT, sig_handler);
  zkSignal(SIGUSR1, sig_handler);
  zkSignal(SIGUSR2, sig_handler);
  zkSignal(SIGCHLD, sig_handler);

  /* Get my initial parameters from the NIQS.  Need to get the IP addr/ports 
     for the FMG maip, key query server, and nym server */
  if (!FMGNiqsInit(err, &context)) {
    context.exit_code = EXIT_FAILURE;
    goto finish;
  }

  /* Run my server in the background and fork() off children as needed */
  if (!FMGServer(err, &context)) {
    context.exit_code = EXIT_FAILURE;
  } else {
    context.exit_code = EXIT_SUCCESS;
  }

finish:
  if (context.exit_code == EXIT_FAILURE) {
    zkLogWriteContext(context.log,err,func,
                      "Message processing failure");
    fprintf(stderr,"%s:  Failure.\n",context.progname);
  } else {
    zkLogWrite(context.log,"Message processing success");
  }

  FMGContextFree(err,&context);
  zkErrorFree(&err);
  exit(context.exit_code);
  return 0; /* to shut up insure */
}
