/*
 * $Id: fmg.h,v 1.16 2001/04/19 19:26:42 alainc Exp $
 */

#ifndef FMG_H_
#define FMG_H_


#include "zkError.h"
#include "zkLog.h"
#include "zkSecure.h"
#include "zkConfig.h"
#include "zkMIME.h"
#include "zkEmailUtil.h"
#include "frAmtpClient.h"
#include "frCrypt.h"
#include "frSec.h"
#include "frEnt.h"
#include "frNetworkDefaults.h"
#include "alias.h"
#include "frNym.h"
#include "frNymUtil.h"
#include "frNymClient.h"
#include "frNiqs.h"
#include "zkSecure.h"
#include "frKeyQry.h"
#include "frNiqs.h"
#include "frNetInfo.h"

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */


#ifdef FREEDOM_DEVELOPMENT_BUILD
#define VERSION_FMG_STR "Zero-Knowledge FMG server (development) ("__DATE__", "__TIME__")"
#else
#define VERSION_FMG_STR "Zero-Knowledge FMG server (production) ("__DATE__", "__TIME__")"
#endif

#define FMG_COMMENT "This message was processed by the Zero-Knowledge FMG"

/*
 * Each individual email address is allowed to be at most this size
 */

#define FMG_ADDRESS_SZ    128

/*
 * Behaviour flags for the gateway filter
 */

#define FLAG_FMG_DEBUG              0x1

/*
 * Default parameter values for the FMG filter context
 */

#define DEFAULT_LOGFILE    NULL
#define DEFAULT_CONFIGFILE NULL
#define DEFAULT_INPUT      stdin
#define DEFAULT_OUTPUT     stdout
#define DEFAULT_ERROR      stderr
#define DEFAULT_BANNER     NULL
#define DEFAULT_BANNERFILE NULL
#define DEFAULT_MAX_RECIPIENTS 25
#define DEFAULT_MAIP       ""
#define DEFAULT_MAIPPORT   0
#define DEFAULT_MAIPTOUT   30
#define DEFAULT_NYMSERV    ""
#define DEFAULT_NYMSERVPORT 0
#define DEFAULT_NYMSERVTOUT 30
#define DEFAULT_FMGCRYPTPUB NULL;
#define DEFAULT_SENDER     "fmg@"FR_DFLT_FREEDOM_DOMAIN
#define DEFAULT_SENDMAIL   "/usr/lib/sendmail"
#define DEFAULT_SECURITY   ZKSECURE_SAFE
#define DEFAULT_ARCHIVE    NULL
#define DEFAULT_NOTFOUNDMSG_FILE  NULL
#define DEFAULT_ALIASMSG_FILE NULL
#define DEFAULT_NI_RELOAD_TIME     (30 * 60)  /* Reload NetInfo every 30 min */
#define DEFAULT_NI_SHARED_MEM_SIZE (500)

/*
 * The context structure
 */

struct T_FMGContext {
  uint32_t flags;
  ZkConfig config;
  ZkLog log;
  ZkEmail email;
  int exit_code;
  ZkSecureParanoidLevel security; /* For the ZkSecure calls */
  const char * progname;
  const char * logfile;
  const char * configfile;
  MaipServer maip;            /* MAIP server connection */
  NymServer nymserv;          /* The nym server */
  const char * sender;        /* FMG filter sender email address */
  const char * sendmail;      /* Local mail send */
  const char * archive;       /* Archive directory */
  const char * notfoundmsg_file;     /* Nym not found mail message file */
  const char * aliasmsg_file;   /* Alias message file */
  const char * bannerfile;    /* The file which contains the banner */
  char * banner;              /* The FMG banner MIME part added to all mail */
  size_t bsz;                 /* Size of the banner string */
  int max_recipients;         /* Maximum number of entities which can receive
                               * an email message  
                               */
  FMGAlias alias;             /* The FMG alias list */
  char * notfoundmsg;         /* The nym not found message */
  size_t nfmsz;               /* Size of the nym not found message */
  char * aliasmsg;            /* Message sent when a mail is aliased */
  size_t amsz;                /* Size of the alias message */
  FILE * input;
  FILE * error;
  FILE * output;

  /* Private key stuff */
  FrSecKeyChain kc;             /* Key chain for all my private key sets */
  size_t cryptHandle;           /* Cryption handle */
  size_t sigHandle;             /* Signature handle */

  const char *ppprompt;         /* Passphrase prompt */
  const char *ppfile;           /* Passphrase file */
  const char *privCryptDir;     /* Private cryption key directory */
  const char *privSigDir;       /* Private signature key directory */
  
  /* Key query server params */
  FrKeyQry kq;                  /* Key query cache */
  FrKeyQryHost kh;              /* KeyQry Host addr and fd */
  uint32_t kqtimeout;           /* Timeout waiting for key query request */
  int kqretries;                /* Number of times to retry the key query */
  
  FrEntIdExp fmgMaipEntity;     /* FMG MAIP entity for key queries */

  unsigned short port;          /* My listening port */
  int listenFd;                 /* My listen descriptor */
  int listenq;                  /* Listen queue parameter */

  int fork;                     /* Whether to daemonize or not */

  const char     *niqsHost;         /* NIQS hostname */
  unsigned short  niqsPort;         /* NIQS port */
  int             niqsTimeout;      /* NIQS timeout */
  time_t          niqsFullUpdatePeriod; /* Force complete NIQS update every X sec */

  FrNetInfo         ni;            /* NetInfo object for holding Freedom info */
  FrNetInfoChildCtx niCtx;         /* NetInfo updater process */
  pid_t             niPid;         /* NetInfo updater process ID */
  uint32_t          niReloadPeriod;  /* Period for updating NetInfo (in sec.)*/
  size_t            niSharedMemSz; /* Buffer size for NetInfo shared memory */
  const char       *niShmTemplate; /* Template fileame for SHM */
  const char       *niMtxTemplate; /* Template filename for Mutex */

#ifdef FREEDOM_DEVELOPMENT_BUILD
  int               niDumpOnUpdate;
#endif

  const char *pidfile;          /* PID file */
  FILE *fptr;                   /* FILE Pointer to the PID open/locked file */
  FrNiqsClientInfo ci;          /* Client info data from the NIQS */
};

typedef struct T_FMGContext FMGContext;

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FMG_H_ */
