/*
 * $Id: sm.c,v 1.7 2001/04/19 19:26:42 alainc Exp $
 */

static const char * module_id_ = "$Id: sm.c,v 1.7 2001/04/19 19:26:42 alainc Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#include <errno.h>
#endif /* !WIN32 */
#include <string.h>
#include <zkError.h>
#include <zkMIME.h>
#include "fmg.h"
#include "sm.h"

DLLExport int
FMGSendTextMessageToInternetUser (
  ZkError err ,
  FMGContext * context ,
  const char * you ,
  const char * subject ,
  const char * text )
{
  static const char * func = "FMGSendTextMessageToInternetUser";
  ZkEmail message = NULL;
  ZkEmailHeader header = NULL;
  ZkEmailBody body = NULL;
  int return_code = 0;

  (void) module_id_;
  zkErrorClear(err);
  if (context && you && subject && text) {
    if (!(zkEmailNew(err,&message) &&
          zkEmailHeader(err,message,&header) &&
          zkEmailBody(err,message,&body) &&
          zkEmailHeaderAddFieldData(err,header,"From",context->sender) &&
          zkEmailHeaderAddFieldData(err,header,"To",you) &&
          zkEmailHeaderAddFieldData(err,header,"Subject",subject) &&
          zkEmailHeaderAddFieldData(err,header,"MIME-Version","1.0") &&
          zkEmailHeaderAddFieldData(err,header,"Content-Type","text/plain") &&
          zkEmailBodyAddText(err,body,text,strlen(text)) &&
          zkEmailBodyParse(err,body))) {
      zkLogWriteContext(context->log,err,func,"Failed to build message");
      goto finish; 
    }
    if (!FMGSendInternetMail(err,context,context->sendmail,you,message)) {
      zkLogWriteContext(context->log,err,func,"Failed to send message");
      goto finish;
    }
    return_code = 1; 

finish:
    if (message) {
      zkEmailFree(NULL,&message);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0;
}


DLLExport int
FMGSendInternetMail ( 
  ZkError err ,
  FMGContext * context ,
  const char * sendmail ,
  const char * to ,
  ZkEmail message )
{
  static const char * func = "FMGSendInternetMail";
  int return_code = 0;
  FILE * f = NULL;
  char * cmd = NULL;
  size_t cmdsz = 0;
  char * buffer = NULL;
  size_t bsz = 0;

  zkErrorClear(err);
  if (sendmail && to && message) {
    cmdsz = strlen(to) + strlen(sendmail) + 5;
    if ((cmd = (char *) malloc(sizeof(char) * cmdsz)) == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENOMEM,
                 "Failed to allocate memory for command string (malloc)");
      goto finish;
    } 

#warning "Bug #4172. This code will go away when the new mail system comes online"

/*
 * WARNING WARNING WARNING BUG WARNING
 *
 * ok, I'm using a popen here.  I know this is the very embodiment of all
 * that is dark and evil, and yes I am going to fix it.  but since your're
 * reading this, you could come over and ask me when I'm gonna take care
 * of it. --philippe
 */

    snprintf(cmd,cmdsz,"%s %s",sendmail,to);
    if ((f = (FILE *) popen(cmd,"wb")) == NULL) { /* audit: ok */
      zkErrorSet_va(err,func,ZKERR,ZKMFMG,ZKERROR,
                    "popen(\"%s\",\"rb\") returned with error %s", /* audit: ok */
                    cmd,strerror(errno));
      goto finish;
    }
    if (!zkEmailBuffer(err,message,&buffer,&bsz)) {
      zkLogWriteContext(context->log,err,func,"Failed to get message buffer");
      goto finish;
    }
    if (fwrite(buffer,bsz,1,f) != 1) {
      zkErrorSet(err,func,ZKERROR,ZKMFMG,ZKEINCOMPLETE,
                 "Message buffer was not completely sent");
      goto finish;
    }
    return_code = 1;
           
finish: /* Cleanup */
    if (f) {
      pclose(f);
    }
    if (buffer) {
      free(buffer);
    }
    if (cmd) {
      free(cmd);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFMG,ZKENULLPTR,NULL);
  return 0; 
}
