#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include "zkError.h"
#include "zkDNS.h"

#define OPT_FMGPORT "--fmgport"

int main(int argc, char **argv)
{
  int s = -1;
  struct protoent *ppe = NULL;
  struct hostent *phe = NULL;
  char *progname = NULL;
  ZkError err = NULL;
  const char *localhost = "localhost";
  int exit_code = EXIT_FAILURE;
#define LINE_SZ 2048
  char line[LINE_SZ + 1];
  int line_sz = 0;
  int nbytes = 0;
  struct sockaddr_in fmg;
  int i;
  unsigned short port = 0;
  int fail = 0;

  zkErrorNew(&err);

  if ((progname = strrchr(argv[0], '/')) == NULL) {
    progname = argv[0];
  } else {
    progname++;
  }

  /* Parse arguments */
  for (i = 1; i < argc; i++) {
    if (strcmp(OPT_FMGPORT, argv[i]) == 0) {
      if (i++ < argc) {
        port = (unsigned short) atoi(argv[i]);
      } else {
        fprintf(stderr, "%s: FMG port number must be specified.\n", progname);
        fail = 1;
      }
    } else {
      fprintf(stderr, "%s: Unknown argument \"%s\".\n", progname, argv[i]);
      fail = 1;
    }
  }

  if (fail) {
    goto fail;
  }

  if (port == 0) {
    fprintf(stderr, "%s: FMG port number must be specified.\n", progname);
    goto fail;
  }
  
  if ((ppe = getprotobyname("tcp")) == NULL) {
    fprintf(stderr, "%s: Can't find \"tcp\" in protocols.\n", progname);
    goto fail;
  }

  if ((s = socket(AF_INET, SOCK_STREAM, ppe->p_proto)) < 0) {
    fprintf(stderr, "%s: socket(%s) failed.\n", progname, strerror(errno));
    goto fail;
  }

  fmg.sin_family = AF_INET;
  if (!zkGetHostByName(err, localhost, &phe)) {
    fprintf(stderr, "%s: Couldn't resolve %s.\n", progname, localhost);
    goto fail;
  } else {                    /* Resolved */
    memcpy(&(fmg.sin_addr), phe->h_addr, phe->h_length);
  }
  fmg.sin_port = htons(port);

  if (connect(s, (struct sockaddr *) &fmg, sizeof(fmg)) < 0) {
    fprintf(stderr, "%s: Failed to connect to %s:%d.\n", progname, 
            inet_ntoa(fmg.sin_addr), ntohs(fmg.sin_port));
    goto fail;
  }

  while (!feof(stdin)) {
    line_sz = fread(line, 1, LINE_SZ, stdin);
    if (line_sz < LINE_SZ) {
      if (ferror(stdin)) {
        fprintf(stderr, "%s: Error reading from stdin.\n", progname);
        goto fail;
      }
    }
    if ((nbytes = write(s, line, line_sz)) < line_sz) {
      fprintf(stderr, "%s: Short write [%d out of %d] to socket.\n", 
              progname, nbytes, line_sz);
      goto fail;
    }
  }
  
  exit_code = EXIT_SUCCESS;
 fail:

  if (s > 0) {
    close(s);
  }

  if (err) {
    zkErrorFree(&err);
  }
  exit(exit_code);
  return 0; /* to shut up insure */
}
