/*
 ***************************************************************************
 * $Id: frpg_keydb.c,v 1.5 2001/04/19 19:26:29 alainc Exp $
 *
 * Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
 ***************************************************************************
 */


#include "zks.h"

#include <unistd.h>

#include <sys/types.h>
#include <sys/wait.h>

#include "zkParms.h"
#include "zkAlloc.h"
#include "zkCStr.h"
#include "zkPlugin.h"
#include "zkFile.h"
#include "zkConfig.h"

#include "zkPFork.h"

#include "frKeyQry.h"
#include "zkPluginImpl.h"

/* headers for this plugin */
#include "frpg_keydb.h"
#include "frpg_keydbInt.h"


#if 0
/*
 * This is a HUGE hack to get the dynamic library services offered by
 * Linux to load my shared objec (i.e., this plugin).  I don't know
 * what it's REALLY supposed to do ... but defining this functions
 * allows us to test the rest of the plugin API.
 *
 *  N.B.:  Noxious to aip2 due to conflicts with /usr/lib/sys/stat.h.
 *  Furtermore, no longer seems to be necessary (glibc?).
 *  
 */
int stat(void);
int stat(void)
{
    fprintf(stderr, "\n called the stubbed stat: THIS is a serious problem\n\n");
    return 0;
}
#endif


/* Nasty static vTable */

static const struct t_ZkPluginVersion_1 plugin_vtable = {
    destructor,
    dataFreeer,
    commandHandler
};

/* --------------------------------------------------------------------------
 * Entry-Points
 */

/* this function is our constructor, it is called first, and only once by any
   one using the plugin.  This will be added to plugin docs soon. */

DLLExport int
zkPluginGetInfo(
    ZK_OUT    ZkError          err,
    ZK_IN     uint32_t         desired_version,
    ZK_OUT    ZkPluginInfo     plugin_info )
{
  const char       func[]         = "zkPluginGetInfo ($Id: frpg_keydb.c,v 1.5 2001/04/19 19:26:29 alainc Exp $)";
  int              ret            = 1;   /* return value, 1 means success */
  frpg_keydbcxt *  cxt            = NULL;
  const char *     keydbName      = NULL;
  char *           keydbNameDup   = NULL; /* Database names for context */
  const char *     dkeydbName     = NULL;
  char *           dkeydbNameDup  = NULL;
  ZkConfig         conf           = NULL;

  if (desired_version != FRPG_KEYDB_API_VERSION) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKEVERSION, NULL);
    ret = 0;

  } else if (!(plugin_info && plugin_info->object_handle)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY ,ZKENULLPTR, NULL);
    ret = 0;

      /* Access the config file in order to get database names */

  } else if (!zkConfigNew(err, &conf)) {
    ret = 0;

  } else if (!zkConfigLoad(err, conf, plugin_info->conf_file_name)) {
    ret = 0;

  } else if (!zkConfigGet(err, conf, S_SERVER_KEYUPD, P_DATABASE, &keydbName)) {
    ret = 0;

  } else if (!zkConfigGet(err,
                          conf,
                          S_SERVER_KEYUPD,
                          P_DATABASE_DISABLED,
                          &dkeydbName)) {
    ret = 0;

    /* Database names */

  } else if (NULL == (keydbNameDup = zkCStrDup(keydbName))) {
    ret = 0;

  } else if (NULL == (dkeydbNameDup = zkCStrDup(dkeydbName))) {
    ret = 0;

    /* Allocate the context, and set parameters */

  } else if (!zkMemAlloc(err, (void **) &cxt, sizeof(frpg_keydbcxt))) {
    ret = 0;

  } else {
    (cxt->dispatch)[0].name = FRPG_KEYDB_CMD_DELETE;
    (cxt->dispatch)[0].func = frpgDeleteKey;
    (cxt->dispatch)[1].name = FRPG_KEYDB_CMD_RESTORE;
    (cxt->dispatch)[1].func = frpgRestoreKey;

    cxt->keydbName          = keydbNameDup;
    cxt->dkeydbName         = dkeydbNameDup;

    plugin_info->private_data = (void *) cxt;
    plugin_info->name         = FRPG_KEYDB_NAME;
    plugin_info->version      = FRPG_KEYDB_VERSION;
    plugin_info->api_version  = FRPG_KEYDB_API_VERSION;
    plugin_info->vtable       = &plugin_vtable;

  }

  /* Cleanup */
  /* cleanup stuff we don't need anymore, the config stuff */

  if (conf != NULL) {
    zkConfigFree(NULL, &conf);
  }

  /* On error clean up everything else.  There's no need to clean up the plugin
     info structure, since it won't be touched if there was an error */

  if (ret != 1) {

    if (keydbName != NULL) {
      zkCStrFree( &keydbNameDup);
    }

    if (dkeydbName != NULL) {
      zkCStrFree( &dkeydbNameDup);
    }

    if (cxt != NULL) {
      zkMemFree(NULL, (void **) &cxt);
    }
  }

  return ret;
}

/* --------------------------------------------------------------------------
 * Member Functions
 */

DLLExport int destructor(
    ZkError                    err,
    struct t_ZkPluginInfo *    plugin_info )
{
  const char       func[]   = "zkPluginGetInfo ($Id: frpg_keydb.c,v 1.5 2001/04/19 19:26:29 alainc Exp $)";
  int              ret      = 1;   /* return value, 1 means success */
  frpg_keydbcxt *  cxt      = NULL;

  /* close the databases, dellocate the context, and NULL out the plugin_info */

  if ((plugin_info == NULL) || (plugin_info->private_data == NULL)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY ,ZKENULLPTR, NULL);
    ret = 0;

  } else {

    cxt = (frpg_keydbcxt *) plugin_info->private_data;

    if (cxt->keydbName != NULL) {
      zkCStrFree( &cxt->keydbName);
    }

    if (cxt->dkeydbName != NULL) {
      zkCStrFree( &cxt->dkeydbName);
    }

    if (cxt != NULL) {
      zkMemFree(NULL, (void **) &cxt);
    }

    plugin_info->name         = NULL;
    plugin_info->version      = 0;
    plugin_info->api_version  = 0;
    plugin_info->vtable       = NULL;
    plugin_info->private_data = NULL;

  }

  return ret;
}
    
DLLExport int dataFreeer(
    ZkError                    err,
    void *                     plugin_data,
    void **                    data_to_free )
{
    (void)plugin_data; /* to silence compiler */

    return zkMemFree(err, data_to_free);
}

/*
 * The command handler dispatches commands to the appropriate functions.
 *
 */

DLLExport int commandHandler(
    ZkError                    err,
    void *                     plugin_data,
    const char *               param[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size )
{
  static const char *        func = "commandHandler ($Id: frpg_keydb.c,v 1.5 2001/04/19 19:26:29 alainc Exp $)";
  int                        ret  = 1; /* Return code, 1 is success */
  frpg_keydbcxt *            cxt = (frpg_keydbcxt *) plugin_data;

  if (    !(plugin_data && param && numparams && out_data && out_data_size)
       && (!in_data_size || in_data) ) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENULLPTR, "null input param");
    ret = 0;

  } else if (numparams < 2) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENUMPARAM, "not enough params");
    ret = 0;

  } else if ( strcasecmp(FRPG_KEYDB_NAME, param[0]) != 0) {
    zkErrorSet(err, func, ZKERR, ZKMPLUGIN, ZKEBADR, "service mismatch");
    ret = 0;

    /* Look for the command in the table and dispatch it. */

  } else {
    size_t i;

    for (i = 0; i < FRPG_KEYDB_NUMDISPATCH ; ++i) {
      if (strcasecmp((cxt->dispatch)[i].name, param[1]) == 0) {
        return (*((cxt->dispatch)[i].func))(err,
                                           plugin_data,
                                           param,
                                           numparams,
                                           in_data,
                                           in_data_size,
                                           out_data,
                                           out_data_size);
      }
    }

    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKEBADR, "command not found");
    ret = 0;
  }

  return ret;
}

/* --------------------------------------------------------------------------
 * Command Handlers
 */

/* Since these are only invoked by the command handler and it checks if all the
 * parameters are NULL, we don't do it in these functions.  However, we do
 * check that an extra parameter is present, containing the name of the nym.
 *
 */


/*
 *  This function deletes the nym keys, by moving them from the pubkey db to
 *  the pubkey-dis db.
 */

static int frpgDeleteKey(
    ZK_OUT    ZkError            err,
    ZK_IN     void *             plugin_data,
    ZK_IN     const char *       params[],
    ZK_IN     size_t             numparams,
    ZK_IN     const void *       in_data,
    ZK_IN     size_t             in_data_size,
    ZK_OUT    void **            out_data,
    ZK_OUT    size_t *           out_data_size )
{
  const char *     func       = "frpgDeleteKey";
  int              ret        = 1;  /* Return value, 1 is success */
  frpg_keydbcxt *  cxt        = NULL;

  (void)in_data;
  (void)in_data_size;
  (void)out_data;
  (void)out_data_size;

  /* Check input parameters (some already checked by commandhandler */

  if (NULL == (cxt = (frpg_keydbcxt *) plugin_data)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENULLPTR, NULL);
    ret = 0;
  }

  if ((ret) && (numparams < 3)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENUMPARAM, "not enough params");
    ret = 0;
  }

    /* do the transfer */

  if (ret) {
    ret = frpgTransferKeys(err,
                           params[2],
                           cxt->keydbName,    /* source */
                           cxt->dkeydbName);  /* destination */
  }

  return ret;
}

/*
 *  This function restores the nym keys, by moving them to the pubkey db from
 *  the pubkey-dis db.
 */

static int frpgRestoreKey(
    ZK_OUT    ZkError            err,
    ZK_IN     void *             plugin_data,
    ZK_IN     const char *       params[],
    ZK_IN     size_t             numparams,
    ZK_IN     const void *       in_data,
    ZK_IN     size_t             in_data_size,
    ZK_OUT    void **            out_data,
    ZK_OUT    size_t *           out_data_size )
{
  const char *     func       = "frpgRestoreKey";
  int              ret        = 1;  /* Return value, 1 is success */
  frpg_keydbcxt *  cxt        = NULL;

  (void)in_data;
  (void)in_data_size;
  (void)out_data;
  (void)out_data_size;

  /* Check input parameters (some already checked by commandhandler */

  if (NULL == (cxt = (frpg_keydbcxt *) plugin_data)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENULLPTR, NULL);
    ret = 0;
  }

  if ((ret) && (numparams < 3)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENUMPARAM, "not enough params");
    ret = 0;
  }

    /* do the transfer */

  if (ret) {
    ret = frpgTransferKeys(err,
                           params[2],
                           cxt->dkeydbName,  /* source */
                           cxt->keydbName);  /* destination */
  }

  return ret;
}

/*
 * This function dows the actual transferring of keys between the two databases
 *
 * It first checks that the destination doesn't already contain keys for the
 * given nym.  If it does, an error is returned, otherwise the keys in the
 * source are read, written to the destination, and finally deleted from the
 * source.
 *
 *
 */


static int frpgTransferKeys(
    ZkError       err,
    const char *  nymName,
    char *        sourceDbName,
    char *        destDbName)
{
  const char *     func       = "frpgTransferKeys";
  int              ret        = 1;  /* Return value, 1 is success */
  ZkDB             sourceDb   = NULL;
  ZkDB             destDb     = NULL;


  /* For holding queried key data */
  frNymKeys        nymKeys = NULL;

  /* check parameters */

  if ((nymName == NULL) || (sourceDbName == NULL) || (destDbName == NULL)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKENULLPTR, NULL);
    ret = 0;
  }

  if (ret) {
    ret = zkDB_Open(err, &sourceDb, sourceDbName);
  }

  if (ret) {
    ret = zkDB_Open(err, &destDb, destDbName);
  }

  /* check for keys in the destination */

  if (ret) {
    ret = fr_KeyQryDB_GetNymKeys(err, destDb, nymName, &nymKeys);
  }

  if ((ret) && (nymKeys != NULL)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKEPARMBAD, "dest contains keys");
    ret = 0;

    fr_KeyQryDB_FreeNymKeys(NULL, &nymKeys);
  }

  /* Transfer them -- read, write, expunge originals
   *
   * If the expunge fails we don't try and recover, because we at least have
   * one database that's ok, the destination.
   */

  if (ret) {
    ret = fr_KeyQryDB_GetNymKeys(err, sourceDb, nymName, &nymKeys);
  }

  if ((ret) && (nymKeys == NULL)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGKEY, ZKEPARMBAD, "no keys in source");
    ret = 0;
  }

  if (ret) {
    ret = fr_KeyQryDB_WriteNymKeys(err, destDb, nymKeys);
  }

  if (ret) {
    ret = fr_KeyQryDB_FreeNymKeys(NULL, &nymKeys);
  }

  if (ret) {
    ret = fr_KeyQryDB_ExpungeNymKeys(err, sourceDb, nymName);
  }

  /* Clean up */

  if (sourceDb != NULL) {
    zkDB_Close(NULL, &sourceDb);
  }
  if (destDb != NULL) {
    zkDB_Close(NULL, &destDb);
  }

  return ret;
}
