/*
 ***************************************************************************
 * $Id: frpg_keydbInt.h,v 1.2 2000/04/17 22:34:53 alext Exp $
 *
 * Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
 ***************************************************************************
 */

#ifndef FRPG_KEYDBINT_H
#define FRPG_KEYDBINT_H


#include "frpg_keydb.h"
#include "zkPluginImpl.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Internal stuff for use by the keydb plugin */

/* plugin information */

#define FRPG_KEYDB_VERSION          1
#define FRPG_KEYDB_API_VERSION      ZKPLUGIN_APIVERSION_1

/* The following describe where to look in the config file for the names of the
   databases to access. */

#define FRPG_KEYDB_CONF_KEYDB_SECTION    "KeyUpdateServer"
#define FRPG_KEYDB_CONF_KEYDB_PARAM      "database"
#define FRPG_KEYDB_CONF_DKEYDB_SECTION   "KeyUpdateServer"
#define FRPG_KEYDB_CONF_DKEYDB_PARAM     "database.disabled"

/* --------------------------------------------------------------------------
 * Plugin Member Functions Prototypes
 *
 * These follow typedefs for callbacks declared in zkPluginImpl.h
 * They are the functions exposed to fulfill this API.
 */

DLLExport int destructor(
    ZkError                    err,
    struct t_ZkPluginInfo *    plugin_info );
    
DLLExport int dataFreeer(
    ZkError                    err,
    void *                     plugin_data,
    void **                    data_to_free );

DLLExport int commandHandler(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

/* --------------------------------------------------------------------------
 * Command Handlers
 *
 * Various functions which perform the work of the plugin.  They are called by
 * the command handler, and share its prototype.  They do not need to be
 * exported.
 */

static int frpgDeleteKey(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

static int frpgRestoreKey(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

static int frpgTransferKeys(
    ZkError       err,
    const char *  nymName,
    char *        sourceDbName,
    char *        destDbName);

/* --------------------------------------------------------------------------
 * context for the plugin, including a dispatch table for mapping cmd ids to
 * functions.
 */

/* table of name/pointer elements for disaptching functions */

typedef struct { 
  const char * name; 
  ZkPluginCommandHandler_1 func; 
} pg_dispatch;

#define FRPG_KEYDB_NUMDISPATCH 2

/* the plugin context */

typedef struct frpg_keydbcxt_s {
  pg_dispatch  dispatch[FRPG_KEYDB_NUMDISPATCH];    /* plugin dispatch table */
  char *       keydbName;           /* The database for the keydb */
  char *       dkeydbName;          /* database of disabled keys */
} frpg_keydbcxt;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRPG_KEYDBINT_H */
