/*
 ***************************************************************************
 * $Id: frpg_nymdb.c,v 1.4 2001/04/19 19:26:29 alainc Exp $
 *
 * Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
 ***************************************************************************
 */


#include "zks.h"

#include <unistd.h>

#include <sys/types.h>
#include <sys/wait.h>

#include "zkParms.h"
#include "zkAlloc.h"
#include "zkCStr.h"
#include "zkPlugin.h"
#include "zkFile.h"
#include "zkConfig.h"

#include "zkPFork.h"

#include "frnymsdb.h"
#include "zkPluginImpl.h"

/* headers for this plugin */
#include "frpg_nymdb.h"
#include "frpg_nymdbInt.h"


#if 0
/*
 * This is a HUGE hack to get the dynamic library services offered by
 * Linux to load my shared objec (i.e., this plugin).  I don't know
 * what it's REALLY supposed to do ... but defining this functions
 * allows us to test the rest of the plugin API.
 *
 *  N.B.:  Noxious to aip2 due to conflicts with /usr/lib/sys/stat.h.
 *  Furtermore, no longer seems to be necessary (glibc?).
 *  
 */
int stat(void);
int stat(void)
{
    fprintf(stderr, "\n called the stubbed stat: THIS is a serious problem\n\n");
    return 0;
}
#endif


/* Nasty static vTable */

static const struct t_ZkPluginVersion_1 plugin_vtable = {
    destructor,
    dataFreeer,
    commandHandler
};

/* --------------------------------------------------------------------------
 * Entry-Points
 */

/* this function is our constructor, it is called first, and only once by any
   one using the plugin.  This will be added to plugin docs soon. */

DLLExport int
zkPluginGetInfo(
    ZK_OUT    ZkError          err,
    ZK_IN     uint32_t         desired_version,
    ZK_OUT    ZkPluginInfo     plugin_info )
{
  const char       func[]      = "zkPluginGetInfo ($Id: frpg_nymdb.c,v 1.4 2001/04/19 19:26:29 alainc Exp $)";
  int              ret         = 1;   /* return value, 1 means success */
  frpg_nymdbcxt *  cxt         = NULL;
  const char *     nsdbName    = NULL;
  char *           nsdbNameDup = NULL; /* copy of database name for context */
  const char *     dndbName    = NULL;
  char *           dndbNameDup = NULL; /* copy of database name for context */
  ZkConfig         conf        = NULL;

  if (desired_version != FRPG_NYMDB_API_VERSION) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKEVERSION, NULL);
    ret = 0;

  } else if (!(plugin_info && plugin_info->object_handle)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM ,ZKENULLPTR, NULL);
    ret = 0;

    /* Access the config file in order to get database names */

  } else if (!zkConfigNew(err, &conf)) {
    ret = 0;

  } else if (!zkConfigLoad(err, conf, plugin_info->conf_file_name)) {
    ret = 0;

  } else if (!zkConfigGet(err, conf, S_SERVER_NYM, P_DATABASE, &nsdbName)) {
    ret = 0;

  } else if (!zkConfigGet(err,
                          conf,
                          S_SERVER_NYM,
                          P_DATABASE_DISABLED,
                          &dndbName)) {
    ret = 0;

    /* Database names */

  } else if (NULL == (nsdbNameDup = zkCStrDup(nsdbName))) {
    ret = 0;

  } else if (NULL == (dndbNameDup = zkCStrDup(dndbName))) {
    ret = 0;

    /* Allocate the context, and set parameters */

  } else if (!zkMemAlloc(err, (void **) &cxt, sizeof(frpg_nymdbcxt))) {
    ret = 0;

  } else {
    (cxt->dispatch)[0].name = FRPG_NYMDB_CMD_NUKE;
    (cxt->dispatch)[0].func = frpgNukeNym;
    (cxt->dispatch)[1].name = FRPG_NYMDB_CMD_DELETE;
    (cxt->dispatch)[1].func = frpgDeleteNym;
    (cxt->dispatch)[2].name = FRPG_NYMDB_CMD_RESTORE;
    (cxt->dispatch)[2].func = frpgRestoreNym;

    cxt->nsdbName           = nsdbNameDup;
    cxt->dndbName           = dndbNameDup;

    plugin_info->private_data = (void *) cxt;
    plugin_info->name         = FRPG_NYMDB_NAME;
    plugin_info->version      = FRPG_NYMDB_VERSION;
    plugin_info->api_version  = FRPG_NYMDB_API_VERSION;
    plugin_info->vtable       = &plugin_vtable;

  }

  /* Cleanup */
  /* cleanup stuff we don't need anymore, the config stuff */

  if (conf != NULL) {
    zkConfigFree(NULL, &conf);
  }

  /* On error clean up everything else.  There's no need to clean up the plugin
     info structure, since it won't be touched if there was an error */

  if (ret != 1) {

    if (nsdbNameDup != NULL) {
      zkCStrFree( &nsdbNameDup);
    }

    if (dndbNameDup != NULL) {
      zkCStrFree( &dndbNameDup);
    }

    if (cxt != NULL) {
      zkMemFree(NULL, (void **) &cxt);
    }
  }

  return ret;
}

/* --------------------------------------------------------------------------
 * Member Functions
 */

DLLExport int destructor(
    ZkError                    err,
    struct t_ZkPluginInfo *    plugin_info )
{
  const char       func[]   = "zkPluginGetInfo ($Id: frpg_nymdb.c,v 1.4 2001/04/19 19:26:29 alainc Exp $)";
  int              ret      = 1;   /* return value, 1 means success */
  frpg_nymdbcxt *  cxt      = NULL;

  /* dellocate the context, and NULL out the plugin_info */

  if ((plugin_info == NULL) || (plugin_info->private_data == NULL)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM ,ZKENULLPTR, NULL);
    ret = 0;

  } else {

    cxt = (frpg_nymdbcxt *) plugin_info->private_data;

    if (cxt->nsdbName != NULL) {
      zkCStrFree(&cxt->nsdbName);
    }

    if (cxt->dndbName != NULL) {
      zkCStrFree(&cxt->dndbName);
    }

    if (cxt != NULL) {
      zkMemFree(NULL, (void **) &cxt);
    }

    plugin_info->name         = NULL;
    plugin_info->version      = 0;
    plugin_info->api_version  = 0;
    plugin_info->vtable       = NULL;
    plugin_info->private_data = NULL;

  }

  return ret;
}
    
DLLExport int dataFreeer(
    ZkError                    err,
    void *                     plugin_data,
    void **                    data_to_free )
{
    (void)plugin_data; /* to silence compiler */

    return zkMemFree(err, data_to_free);
}

/*
 * The command handler dispatches commands to the appropriate functions.
 *
 */

DLLExport int commandHandler(
    ZkError                    err,
    void *                     plugin_data,
    const char *               param[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size )
{
  static const char *        func = "commandHandler ($Id: frpg_nymdb.c,v 1.4 2001/04/19 19:26:29 alainc Exp $)";
  int                        ret  = 1; /* Return code, 1 is success */
  frpg_nymdbcxt *            cxt = (frpg_nymdbcxt *) plugin_data;

  if (    !(plugin_data && param && numparams && out_data && out_data_size)
       && (!in_data_size || in_data) ) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENULLPTR, "null input param");
    ret = 0;

  } else if (numparams < 2) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENUMPARAM, "not enough params");
    ret = 0;

  } else if ( strcasecmp(FRPG_NYMDB_NAME, param[0]) != 0) {
    zkErrorSet(err, func, ZKERR, ZKMPLUGIN, ZKEBADR, "service mismatch");
    ret = 0;

    /* Look for the command in the table and dispatch it. */

  } else {
    size_t i;

    for (i = 0; i < FRPG_NYMDB_NUMDISPATCH ; ++i) {
      if (strcasecmp((cxt->dispatch)[i].name, param[1]) == 0) {
        return (*((cxt->dispatch)[i].func))(err,
                                           plugin_data,
                                           param,
                                           numparams,
                                           in_data,
                                           in_data_size,
                                           out_data,
                                           out_data_size);
      }
    }

    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKEBADR, "command not found");
    ret = 0;
  }

  return ret;
}

/* --------------------------------------------------------------------------
 * Command Handlers
 */

/* Since these are only invoked by the command handler and it checks if all the
 * parameters are NULL, we don't do it in these functions.  However, we do
 * check that an extra parameter is present, for the nym name.
 *
 */


/* this function copies the nym to the disabled nym database (dndb), and
 * nukes the nym in the nsdb.  It removes the public keys from the nym
 * structure in the nsdb.  A nym nuked in this way can be restored by
 * copying it back into nsdb from the dndb (use frpgRestoreNym)
 *
 * Before doing this it verifies that the nym was not already nuked.
 *
 * If the function encounters an error it restores stuff to the way it was.
 * A back up of what was overwritten in the dndb is stored.
 *
 */

static int frpgNukeNym(
    ZK_OUT    ZkError            err,
    ZK_IN     void *             plugin_data,
    ZK_IN     const char *       params[],
    ZK_IN     size_t             numparams,
    ZK_IN     const void *       in_data,
    ZK_IN     size_t             in_data_size,
    ZK_OUT    void **            out_data,
    ZK_OUT    size_t *           out_data_size )
{
  const char *     func       = "frpgNukeNym";
  int              ret        = 1;  /* Return value, 1 is success */
  FrRemoteNym      nym        = NULL;
  FrRemoteNym      dndbBack   = NULL;
  frpg_nymdbcxt *  cxt        = NULL;
  int              dndbFound  = 0;
  int              nsdbFound  = 0;
  const unsigned char *           pData      = NULL;
  size_t           pSize      = 0;
  ZkDB             nsdb       = NULL;
  ZkDB             dndb       = NULL;

  (void)in_data;
  (void)in_data_size;
  (void)out_data;
  (void)out_data_size;

  if (NULL == (cxt = (frpg_nymdbcxt *) plugin_data)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENULLPTR, NULL);
    ret = 0;

  } else if (numparams < 3) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENUMPARAM, "not enough params");
    ret = 0;

  } else if (!zkDB_Open(err, &nsdb, cxt->nsdbName)) {
    ret = 0;

  } else if (!zkDB_Open(err, &dndb, cxt->dndbName)) {
    ret = 0;

    /* Get it from the nsdb */

  } else if (!frnymsdbGetNym(err, nsdb, params[2], &nym, &nsdbFound)) {
    ret = 0;

    /* It's an error not to find it */

  } else if (!nsdbFound) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENOTFOUND, "nym not found");
    ret = 0;

    /* Check that the nym isn't already nuked */

  } else if (frRemoteNymGetParameter(NULL, nym, NymNuked, &pData, &pSize)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENYMNUKED, "already nuked");
    ret = 0;

    /* back up info in dndb */

  } else if (!frnymsdbGetNym(err, dndb, params[2], &dndbBack,&dndbFound)) {
    ret = 0;

    /* write to the dndb */

  } else if (!frnymsdbAddOrReplaceNym(err,
                                      (dndbFound)
                                      ? FRNYMSDB_MODE_ADDORREPLACENYM_REPLACE
                                      : FRNYMSDB_MODE_ADDORREPLACENYM_ADD,
                                      dndb,
                                      params[2],
                                      nym)) {
    ret = 0;

    /* nuke it in the nsdb */

  } else if (!frnymsdbNukeNym(err, nsdb, params[2], nym)) {
    /* on error restore the dndb */
    if (dndbFound) {
      frnymsdbReplaceNym(err, dndb, params[2], dndbBack);
    } else {
      frnymsdbDeleteNym(err, dndb, params[2]);
    }      
    ret = 0;
  }

  /* Clean up, destroy local copy of nym */

  if (nym != NULL) {
    frRemoteNymFree(NULL, &nym);
  }

  if (dndbBack != NULL) {
    frRemoteNymFree(NULL, &dndbBack);
  }

  if (nsdb != NULL) {
    zkDB_Close(NULL, &nsdb);
  }

  if (dndb != NULL) {
    zkDB_Close(NULL, &dndb);
  }

  return ret;
}

/* this function copies the nym to the disabled nym database (dndb), and
 * deletes the nym from the nsdb.  It removes the public keys from the nsdb.  A
 * nym disabled in this way can be restored by copying it back into nsdb from
 * the dndb (use frpgRestoreNym)
 *
 * If the nym was already nuked, it verifies that there is already a back up
 * in the dndb, and doesn't overwrite it.
 *
 * If the function encounters an error it restores stuff to the way it was.
 * A back up of what was overwritten in the dndb is stored.
 * */

static int frpgDeleteNym(
    ZK_OUT    ZkError            err,
    ZK_IN     void *             plugin_data,
    ZK_IN     const char *       params[],
    ZK_IN     size_t             numparams,
    ZK_IN     const void *       in_data,
    ZK_IN     size_t             in_data_size,
    ZK_OUT    void **            out_data,
    ZK_OUT    size_t *           out_data_size )
{
  const char *     func       = "frpgDeleteNym";
  int              ret        = 1;  /* Return value, 1 is success */
  FrRemoteNym      nym        = NULL;
  FrRemoteNym      dndbBack   = NULL;
  frpg_nymdbcxt *  cxt        = NULL;
  int              dndbFound  = 0;
  int              nsdbFound  = 0;
  const unsigned char *           pData      = NULL;
  size_t           pSize      = 0;
  ZkDB             nsdb       = NULL;
  ZkDB             dndb       = NULL;

  (void)in_data;
  (void)in_data_size;
  (void)out_data;
  (void)out_data_size;

  if (NULL == (cxt = (frpg_nymdbcxt *) plugin_data)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENULLPTR, NULL);
    ret = 0;

  } else if (numparams < 3) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENUMPARAM, "not enough params");
    ret = 0;

  } else if (!zkDB_Open(err, &nsdb, cxt->nsdbName)) {
    ret = 0;

  } else if (!zkDB_Open(err, &dndb, cxt->dndbName)) {
    ret = 0;

    /* Get it from the nsdb */

  } else if (!frnymsdbGetNym(err, nsdb, params[2], &nym, &nsdbFound)) {
    ret = 0;

    /* It's an error not to find it */

  } else if (!nsdbFound) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENOTFOUND, "nym not found");
    ret = 0;

    /* backup info in dndb */

  } else if (!frnymsdbGetNym(err, dndb, params[2], &dndbBack, &dndbFound)) {
    ret = 0;

    /* If the nym is nuked, and not already backed up in the dndb then we
       have a problem, because we don't know where its real information is, so
       we gack.  If it wasn't already nuked we overwrite the dndb. */

  } else if (frRemoteNymGetParameter(NULL, nym, NymNuked, &pData, &pSize)) {

    if (!dndbFound) {
      zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENOTFOUND, "no back up found");
      ret = 0;
    }

    /* if it wasn't already nuked, write to the dndb */

  } else if (!frnymsdbAddOrReplaceNym(err,
                                      (dndbFound)
                                      ? FRNYMSDB_MODE_ADDORREPLACENYM_REPLACE
                                      : FRNYMSDB_MODE_ADDORREPLACENYM_ADD,
                                      dndb,
                                      params[2],
                                      nym)) {
    ret = 0;

  }

  /* If there's no error so far, delete the entry from the nsdb */

  if ((ret != 0) && (!frnymsdbDeleteNym(err, nsdb, params[2]))) {
    /* on error restore the dndb */
    if (dndbFound) {
      frnymsdbReplaceNym(err, dndb, params[2], dndbBack);
    } else {
      frnymsdbDeleteNym(err, dndb, params[2]);
    }      
    ret = 0;
  }

  /* Clean up, destroy local copy of nym */

  if (nym != NULL) {
    frRemoteNymFree(NULL, &nym);
  }

  if (dndbBack != NULL) {
    frRemoteNymFree(NULL, &dndbBack);
  }

  if (nsdb != NULL) {
    zkDB_Close(NULL, &nsdb);
  }

  if (dndb != NULL) {
    zkDB_Close(NULL, &dndb);
  }

  return ret;
}


/* this function copies a nym from the disabled nym database (dndb), back to
 * the nsdb, and deletes the copy from the dndb.
 *
 * Before doing this it verifies that the nym was already nuked, or deleted.
 *
 * If the function encounters an error it restores stuff to the way it was.
 * A back up of what was overwritten in the dndb is recovered.
 *
 */


static int frpgRestoreNym(
    ZK_OUT    ZkError            err,
    ZK_IN     void *             plugin_data,
    ZK_IN     const char *       params[],
    ZK_IN     size_t             numparams,
    ZK_IN     const void *       in_data,
    ZK_IN     size_t             in_data_size,
    ZK_OUT    void **            out_data,
    ZK_OUT    size_t *           out_data_size )
{
  const char *     func       = "frpgRestoreNym";
  int              ret        = 1;  /* Return value, 1 is success */
  FrRemoteNym      nym        = NULL;
  FrRemoteNym      nsdbBack   = NULL;
  frpg_nymdbcxt *  cxt        = NULL;
  int              dndbFound  = 0;
  int              nsdbFound  = 0;
  const unsigned char *           pData      = NULL;
  size_t           pSize      = 0;
  ZkDB             nsdb       = NULL;
  ZkDB             dndb       = NULL;

  (void)in_data;
  (void)in_data_size;
  (void)out_data;
  (void)out_data_size;

  if (NULL == (cxt = (frpg_nymdbcxt *) plugin_data)) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENULLPTR, NULL);
    ret = 0;

  } else if (numparams < 3) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENUMPARAM, "not enough params");
    ret = 0;

  } else if (!zkDB_Open(err, &nsdb, cxt->nsdbName)) {
    ret = 0;

  } else if (!zkDB_Open(err, &dndb, cxt->dndbName)) {
    ret = 0;

    /* Get it from the dndb */

  } else if (!frnymsdbGetNym(err, dndb, params[2], &nym, &dndbFound)) {
    ret = 0;

    /* It's an error not to find it */

  } else if (!dndbFound) {
    zkErrorSet(err, func, ZKERR, ZKMFRPGNYM, ZKENOTFOUND, "nym not found");
    ret = 0;

    /* backup info in nsdb */

  } else if (!frnymsdbGetNym(err, nsdb, params[2], &nsdbBack, &nsdbFound)) {
    ret = 0;

    /* Check that the nym was nuked in the nsdb */

  } else if (   nsdbFound
             && !frRemoteNymGetParameter(NULL,
                                         nsdbBack,
                                         NymNuked,
                                         &pData,
                                         &pSize)) {
    ret = 0;

    /* overwrite the nsdb */

  } else if (!frnymsdbAddOrReplaceNym(err,
                                      (nsdbFound)
                                      ? FRNYMSDB_MODE_ADDORREPLACENYM_REPLACE
                                      : FRNYMSDB_MODE_ADDORREPLACENYM_ADD,
                                      nsdb,
                                      params[2],
                                      nym)) {
    ret = 0;

    /* delete copy from dndb, restore nsdb on error  */

  } else if (!frnymsdbDeleteNym(err, dndb, params[2])) {
    ret = 0;

    /* on error, restore the nsdb */
    if (nsdbFound) {
      frnymsdbReplaceNym(err, nsdb, params[2], nsdbBack);
    } else {
      frnymsdbDeleteNym(err, nsdb, params[2]);
    }
  }

  /* Clean up, destroy local copy of nym */

  if (nym != NULL) {
    frRemoteNymFree(NULL, &nym);
  }

  if (nsdbBack != NULL) {
    frRemoteNymFree(NULL, &nsdbBack);
  }

  if (nsdb != NULL) {
    zkDB_Close(NULL, &nsdb);
  }

  if (dndb != NULL) {
    zkDB_Close(NULL, &dndb);
  }

  return ret;
}

