/*
 ***************************************************************************
 * $Id: frpg_nymdbInt.h,v 1.2 2000/04/17 22:34:53 alext Exp $
 *
 * Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
 ***************************************************************************
 */

#ifndef FRPG_NYMDBINT_H
#define FRPG_NYMDBINT_H

#include "frpg_nymdb.h"
#include "zkPluginImpl.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Internal stuff for use by the nymdb plugin */

/* plugin information */

#define FRPG_NYMDB_VERSION          1
#define FRPG_NYMDB_API_VERSION      ZKPLUGIN_APIVERSION_1

/* --------------------------------------------------------------------------
 * Plugin Member Functions Prototypes
 *
 * These follow typedefs for callbacks declared in zkPluginImpl.h
 * They are the functions exposed to fulfill this API.
 */

DLLExport int destructor(
    ZkError                    err,
    struct t_ZkPluginInfo *    plugin_info );
    
DLLExport int dataFreeer(
    ZkError                    err,
    void *                     plugin_data,
    void **                    data_to_free );

DLLExport int commandHandler(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

/* --------------------------------------------------------------------------
 * Command Handlers
 *
 * Various functions which perform the work of the plugin.  They are called by
 * the command handler, and share its prototype.  They do not need to be
 * exported.
 */

static int frpgNukeNym(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

static int frpgDeleteNym(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

static int frpgRestoreNym(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );



/* --------------------------------------------------------------------------
 * context for the plugin, including a dispatch table for mapping cmd ids to
 * functions.
 */

/* table of name/pointer elements for disaptching functions */

typedef struct { 
  const char * name; 
  ZkPluginCommandHandler_1 func; 
} pg_dispatch;

#define FRPG_NYMDB_NUMDISPATCH 3 

/* the plugin context */

typedef struct frpg_nymdbcxt_s {
  pg_dispatch  dispatch[FRPG_NYMDB_NUMDISPATCH];    /* plugin dispatch table */
  char *       nsdbName;           /* The database for the nsdb */
  char *       dndbName;           /* database of disabled nyms */
} frpg_nymdbcxt;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRPG_NYMDBINT_H */
