/*
 * $Id: frlodecmd.c,v 1.5 1999/11/12 18:45:30 alext Exp $
 */

static const char * module_id_ = "$Id: frlodecmd.c,v 1.5 1999/11/12 18:45:30 alext Exp $";

#include <zks.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* ! WIN32 */
#include <string.h>
#include "zkPlugin.h"
#include "zkauth.h"
#include "frlodectx.h"
#include "frlodecmd.h"
#include "zkCStr.h"
#include "zkAlloc.h"

#define MAX_PARMS    64

#define ZKAUDIT_RESULTCODE_SUCCESS     0x00000000
#define ZKAUDIT_RESULTCODE_FAIL        0xFFFFFFFF

/* --------------------------------------------------------------------------
 * Private Function Prototypes
 */

static int frLodeCmdGetParameters (
  ZK_INOUT ZkError err ,                  /* Return the error if one occured */
  ZK_IN    ZkipSession session ,          /* ZKIP session structure */
  ZK_INOUT const char * parm[MAX_PARMS] , /* Array to hold parameters */
  ZK_OUT   int * parameters );            /* Number of parameters */

static int frLodeCmdLookupService(
  ZK_INOUT ZkError err,                   /* Return the error if one occured */
  ZK_IN    FrLodeCtx c,                   /* Lode server context */
  ZK_IN    const char * name,             /* Name of the service to find */
  ZK_OUT   ZkPluginInfo * plugin );       /* holder for service plugin */

/* --------------------------------------------------------------------------
 * 
 */

static int
frLodeCmdGetParameters (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_INOUT const char * parm[MAX_PARMS] ,
  ZK_OUT   int * parameters )
{
  static const char * func = "frLodeCmdGetParameters";
  int ret = 0;       /* Function return code */
  int n = 0;         /* Loop through the parameters and get pointers to them */

  (void) module_id_; /* Gets rid of the compile warnings */
  if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Session is NULL");
  } else if (!parm) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"parameter array is NULL");
  } else if (!parameters) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"parameter count is NULL");
  } else if (!zkipSessionParameters(err,session,parameters)) {
    ret = 0;
  } else if (*parameters > MAX_PARMS) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKERROR,"Too many parameters");
  } else { 
    ret = 1;
    for (n = 0; n < *parameters; n++) {
      if (!zkipSessionParameter(err,session,n,
                                (const unsigned char **) &(parm[n]))) {
        ret = 0;
        break;
      }
    }
  }
  
  return ret;
}

/* --------------------------------------------------------------------------
 *
 */

static int
frLodeCmdLookupService(
  ZK_INOUT ZkError err,
  ZK_IN    FrLodeCtx c,
  ZK_IN    const char * name,
  ZK_OUT   ZkPluginInfo * plugin )
{
  static const char * func = "frLodeCmdLookupService";
  int res = 0;

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!name) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Service name is NULL");
  } else if (!plugin) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Plugin is NULL");
  } else {

    /*
     * it is not an error for the service not to be found.  People who
     * call this function must verify that they get back a valid plugin.
     */

    *plugin = NULL;
    if ( zkHashSearch(err,c->table,name,(void**)plugin)
         || (zkErrorCode(err) == ZKENOERROR) ) {
      res = 1;
    }
  }

  return (res);
}

/* --------------------------------------------------------------------------
 *
 */

DLLExport int
frLodeCmdInfo (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdInfo";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"session is NULL");
  } else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command parameters",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else {
    ret = _SESSION_CONTINUE;
    zkipSessionResponse(NULL,session,0,_COMMAND_NOTIMPLEMENTED,
                       "Info command is currently not implemented",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);

  }

  return ret;
}

/* --------------------------------------------------------------------------
 *
 */

DLLExport int
frLodeCmdLogin (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdLogin";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;
  const char * name = NULL;
  const char * passphrase = NULL;
  const char * ticket = NULL;
  const char * client = NULL;
  const char * transaction = NULL;
  size_t passphrase_sz = 0;

  /*
   * Validate the function's input params
   */

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"session is NULL");
  }
  
  /*
   * Get any ZKIP parameters which were included
   */
  
  else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "Error reading command parameters",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  }
  
  /*
   * Make sure there's a user name
   */
  else if (parameters < 1) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "User name not specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else {

    /*
     * Figure out what goes into the audit session
     */
    switch (parameters) {
    default:
    case 4:
      transaction = parm[3];
    case 3:
      client = parm[2];
    case 2:
      ticket = parm[1];
    case 1:
      name = parm[0];
    }

    /*
     * Make sure that the user hasn't already logged in
     */

    if (c->auth_session) {
      zkipSessionResponse(NULL,session,0,_SERVICE_UNAVAILABLE,
                          "Already authenticated.  You must log out first.",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,ZKAUDIT_RESULTCODE_FAIL,
                             "Already authenticated",NULL);
      ret = _SESSION_CONTINUE;
    }

    /*
     * Begin a new audit session
     */

    else if (!zkAuditSessionBegin(err,c->audit_log,name,ticket,client,transaction,
                                  &(c->audit_session))) {
      zkLogError(c->log,err);
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    }

    /*
     * Make sure that the command looks valid
     */
    else  if (parameters > 4) {
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Error reading command parameters",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      if (!zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,
                                  ZKAUDIT_RESULTCODE_FAIL,"Malformed command",NULL)) {
        zkLogError(c->log,err);
      }
      if (!zkAuditSessionEnd(err,&(c->audit_session))) {
        zkLogError(c->log,err);
        c->audit_session = NULL;
      }
    }

    /*
     * Check if the user is sending us a passphrase
     */

    else if (!zkipSessionGetDataSize(err,session,&passphrase_sz)) {
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Error reading passphrase length",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      if (!zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,
                                  ZKAUDIT_RESULTCODE_FAIL,"Missing passphrase",NULL)) {
        zkLogError(c->log,err);
      }
      if (!zkAuditSessionEnd(err,&(c->audit_session))) {
        zkLogError(c->log,err);
        c->audit_session = NULL;
      }
    }

    /*
     * Read the passphrase (if necessary)
     */

    else if (passphrase_sz
             && !zkipSessionReadData(err,session,
                                     (unsigned char**)&passphrase,
                                     &passphrase_sz)) {
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Error reading passphrase",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      if (!zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,
                                  ZKAUDIT_RESULTCODE_FAIL,"Malformed passphrase",NULL)) {
        zkLogError(c->log,err);
      }
      if (!zkAuditSessionEnd(err,&(c->audit_session))) {
        zkLogError(c->log,err);
        c->audit_session = NULL;
      }
    }

    /*
     * Attempt to authenticate the user
     */
    else if (!zkAuthLogin(err, c->auth_context,
                          name, (passphrase ? passphrase : ""),
                          &(c->auth_session))) {
      zkLogError(c->log,err);
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    }
  
    /*
     * Check if the authentication succeeded
     */
    else if (!c->auth_session) {
      zkipSessionResponse(NULL,session,0,_SERVICE_FORBIDDEN,
                          "Authentication failed",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      if (!zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,
                                  ZKAUDIT_RESULTCODE_FAIL,"Failed login attempt",NULL)) {
        zkLogError(c->log,err);
      }
      if (!zkAuditSessionEnd(err,&(c->audit_session))) {
        zkLogError(c->log,err);
        c->audit_session = NULL;
      }
    }
    
    /*
     * We're done ... lets get outa here
     */
    else {
      zkipSessionResponse(NULL,session,0,_SERVICE_READY,"Ok",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      if (!zkAuditSessionAddEntry(err,c->audit_session,"login",name,NULL,
                                  ZKAUDIT_RESULTCODE_SUCCESS,"user authenticated",NULL)) {
        zkLogError(c->log,err);
      }
      ret = _SESSION_CONTINUE;
    }
  }
    
  return ret;
}


/* --------------------------------------------------------------------------
 *
 */

DLLExport int
frLodeCmdLogout (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdLogout";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;

  /*
   * Validate the function's input params
   */

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"session is NULL");
  }
  
  /*
   * Get any ZKIP parameters which were included
   */
  
  else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "Error reading command parameters",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  }
  
  /*
   * Make sure that the command is well formed ... it takes no params
   */
  
  else if (parameters > 0) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "malformed command",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  }
  
  /*
   * Make sure we're dealing with an authenticated user
   */
  else if (!c->auth_session) {
    zkipSessionResponse(NULL,session,0,_SERVICE_READY,"You have not logged in!",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  }

  /*
   * End the users session
   */
  else {

    if (!zkAuthLogout(err, &(c->auth_session))) {
      zkLogError(c->log,err);
      c->auth_session = NULL;
    }

    if (!zkAuditSessionAddEntry(err,c->audit_session,"logout","logout",NULL,
                                ZKAUDIT_RESULTCODE_SUCCESS,"session concluded",NULL)) {
      zkLogError(c->log,err);
    }
    
    if (!zkAuditSessionEnd(err, &(c->audit_session))) {
      zkLogError(c->log,err);
      c->audit_session = NULL;
    }
    
    zkipSessionResponse(NULL,session,0,_SERVICE_READY,"Ok",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    
    ret = _SESSION_CONTINUE;
  }

  return ret;
}


/* --------------------------------------------------------------------------
 *
 */

DLLExport int
frLodeCmdPassphrase (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdPassphrase";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;
  char * passphrase;
  size_t passphrase_sz = 0;

  /*
   * Validate the function's input params
   */

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"session is NULL");
  }

  /*
   * Get any ZKIP parameters which were included
   */

  else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command parameters",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  }

  /*
   * Make sure that the command is well formed ... it takes no params
   */

  else if (parameters > 0) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "malformed command",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  }

  /*
   * Make sure the the user has already logged in
   */

  else if (!c->auth_session) {
    zkipSessionResponse(NULL,session,0,_SERVICE_READY,"You have not logged in!",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  }

  /*
   * Get the length of the new passphrase
   */

  else if (!zkipSessionGetDataSize(err,session,&passphrase_sz)) {

    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "Error reading passphrase length",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);

    if (!zkAuditSessionAddEntry(err,c->audit_session,"passphrase","update",NULL,
                                ZKAUDIT_RESULTCODE_FAIL,"Missing passphrase",NULL)) {
      zkLogError(c->log,err);
    }

    if (!zkAuditSessionEnd(err,&(c->audit_session))) {
      zkLogError(c->log,err);
      c->audit_session = NULL;
    }
  }

  /*
   * Read the passphrase
   */
  
  else if (!zkipSessionReadData(err,session,
                                (unsigned char**)&passphrase,
                                &passphrase_sz)) {

    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "Error reading passphrase",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);

    if (!zkAuditSessionAddEntry(err,c->audit_session,"passphrase","upate",NULL,
                                ZKAUDIT_RESULTCODE_FAIL,"Malformed passphrase",NULL)) {
      zkLogError(c->log,err);
    }
  }

  /*
   * Change the passphrase
   */
  
  else if (!zkAuthChangePassphrase(err,c->auth_session,passphrase)) {
    zkLogError(c->log,err);

    if (!zkAuditSessionAddEntry(err,c->audit_session,
                                "passphrase","update",NULL,
                                ZKAUDIT_RESULTCODE_FAIL,
                                "passphrase update failed",NULL)) {
      zkLogError(c->log,err);
    }

    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                        "Failed to update passphrase",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);

    ret = _SESSION_CONTINUE;
  }

  /*
   * Time to go home ...
   */

  else {

    if (!zkAuditSessionAddEntry(err,c->audit_session,
                                "passphrase","update",NULL,
                                ZKAUDIT_RESULTCODE_SUCCESS,
                                "passphrase modified",NULL)) {
      zkLogError(c->log,err);
    }

    zkipSessionResponse(NULL,session,0,_SERVICE_READY,"Ok",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);

    ret = _SESSION_CONTINUE;
  }

  return ret;
}


/* --------------------------------------------------------------------------
 *
 */

DLLExport int
frLodeCmdCheckAccess (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdCheckAccess";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;
  ZkBool allowed = ZK_FALSE;

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Session is NULL");
  } else if (!c->auth_session) {
    zkipSessionResponse(NULL,session,0,_SERVICE_UNAUTHORIZED,
                       "A valid user has not been specified.  Please login.",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command parameters",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (parameters < 1) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "The service has not been specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (parameters < 2) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "The request has not been specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (!zkAuthCheckAccess(err,c->auth_session,parm[0],parm[1], &allowed)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (!allowed) {
    zkipSessionResponse(NULL,session,0,_SERVICE_FORBIDDEN,
                       "You do not have access to this service",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else {
    zkipSessionResponse(NULL,session,0,_SERVICE_READY,
                       "You have access to this service",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  }

  return (ret);
}


/* --------------------------------------------------------------------------
 *
 * This function services a command to the lode server by calling the
 * appropriate plugin's command handler with the arguments.
 *
 * These commands are called with ZKIP command "service".  The first parameter
 * is a text string written to the audit log containing a reason for performing
 * the command.  The second parameter is the service which should perform the
 * command, this is the name of the appropriate plugin.  The third parameter is
 * the command that should be called in that plugin. The rest of the parameters
 * are used by the command.
 *
 * The first parameter, the reason, is not sent to the plugin, but is just used
 * for the audit log.
 *
 */

DLLExport int
frLodeCmdService (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdService";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;
  ZkPluginInfo plugin = NULL;
  unsigned char * in_data = NULL;
  size_t in_data_size = 0;
  unsigned char * out_data = NULL;
  size_t out_data_size = 0;
  ZkBool allowed = ZK_FALSE;

  /* parameters */

  const char * serviceName  = NULL;
  const char * commandName  = NULL;
  const char * reason       = NULL;

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Session is NULL");
  } else if (!c->auth_session) {
    /* should we log/audit this? */
    zkipSessionResponse(NULL,session,0,_SERVICE_UNAUTHORIZED,
                       "A valid user has not been specified.  Please login.",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command parameters",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (parameters < 1) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "The service has not been specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (parameters < 2) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "A reason has not been specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (parameters < 3) {
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "The request has not been specified",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
    /* Stupid hack to do something that's not a function call, grrr. */
  } else if (   !(reason      = parm[0])
             || !(serviceName = parm[1])
             || !(commandName = parm[2])) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"result from ZKIP is NULL");
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (!zkAuthCheckAccess(err, c->auth_session,
                                serviceName, commandName, &allowed)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (!allowed) {
    zkipSessionResponse(NULL,session,0,_SERVICE_FORBIDDEN,
                       "You do not have access to this service",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (!frLodeCmdLookupService(err,c,serviceName,&plugin)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,"Internal Error",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (!plugin) {
    zkipSessionResponse(NULL,session,0,_SERVICE_UNAVAILABLE,
                       "The requested service is unavailable",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    ret = _SESSION_CONTINUE;
  } else if (!zkipSessionGetDataSize(err,session,&in_data_size)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command data size",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else if (in_data_size
             && !zkipSessionReadData(err,session,&in_data,&in_data_size)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Could not read command data",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else {
    int result = 0;
    char * cmdStr = NULL;
    char * cmdStrTemp = NULL;
    int    parmLooper = 1;

    /* Don't give the plugin the reason, that's just for audit, and we don't
       want it to parse or try and use it. Also, this way I don't have to
       change plugin code as I add the reason */

    result = zkPluginCommand(err, plugin,
                             parm + 1, parameters - 1,
                             in_data,in_data_size,
                             (void**)&out_data,&out_data_size );
    /* Write any error to the log */
    if (!result) {
      zkLogError(c->log,err);
    }

    /* Write to the audit session.  
     *
     * Generate a string containing the issued command to write.
     *
     */

    cmdStr = zkCStrAppend_va("(", parm[parmLooper], ")", NULL);
    parmLooper++;

    for (; (parmLooper < parameters) && (cmdStr != NULL); parmLooper++) {
      cmdStrTemp = zkCStrAppend_va(cmdStr, " (", parm[parmLooper], ")", NULL);

      /* Don't care if it's NULL, 'cause we fail then */
      zkMemFree(NULL, (void **) &cmdStr);
      cmdStr = cmdStrTemp;
    }

    if (cmdStr == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENOMEM,"zkCStr* failed");
      zkLogError(c->log,err);
    }

    /* Always write to the audit log, cmdStr == NULL is ok */

    if (!zkAuditSessionAddEntry(err,
                                c->audit_session,
                                serviceName,
                                commandName,
                                reason,
                                (result) ? ZKAUDIT_RESULTCODE_SUCCESS
                                         : ZKAUDIT_RESULTCODE_FAIL,
                                cmdStr,
                                NULL)) {
      zkLogError(c->log,err);
    }

    /* Always send a zkip response */

    if (result) {
      zkipSessionResponse(NULL,session,0,_SERVICE_READY,"Ok",
                          out_data_size ? "XML" : "NODATA",
                          out_data_size ? ZKIP_ENCODING_8BIT
                                        : ZKIP_ENCODING_NONE,
                          out_data_size,out_data);
    } else {
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Failed to execute the requested command",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    }

    if (cmdStr != NULL) {
      zkMemFree(NULL, (void **) &cmdStr);
    }

    ret = _SESSION_CONTINUE;
  }

  if (out_data) {
    zkPluginMemFree(NULL,plugin,(void**)&out_data);
  }

  return ret;
}

#if 0

/* --------------------------------------------------------------------------
 * This is a command function template
 */

DLLExport int
frLodeCmdLogin (
  ZK_INOUT ZkError err ,
  ZK_IN    ZkipSession session ,
  ZK_IN    void * data )
{
  static const char * func = "frLodeCmdLogin";
  int ret = _SESSION_ERROR;
  const char * parm[MAX_PARMS];
  int parameters = 0;
  FrLodeCtx c = (FrLodeCtx) data;

  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"Context is NULL");
  } else if (!session) {
    zkErrorSet(err,func,ZKERR,ZKMLODE,ZKENULLPTR,"session is NULL");
  } else if (!frLodeCmdGetParameters(err,session,parm,&parameters)) {
    zkLogError(c->log,err);
    zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                       "Error reading command parameters",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
  } else {
    ret = _SESSION_CONTINUE;
    zkipSessionResponse(NULL,session,0,_COMMAND_NOTIMPLEMENTED,
                       "The login command is not yet implemented",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);

  }

  return ret;
}
#endif

