/*
 * $Id: frloded.c,v 1.3 1999/10/17 16:56:52 bill Exp $
 */

static const char * module_id_ = "$Id: frloded.c,v 1.3 1999/10/17 16:56:52 bill Exp $";

#include <zks.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include "frlodectx.h"
#include "frlodesrv.h"
#include "frlodecmd.h"

int 
main (
  int argc ,
  const char ** argv )
{
  FrLodeCtx ctx = NULL;             /* Application context structure */
  ZkError err = NULL;               /* Application error structure */
  int exit_code = EXIT_FAILURE;     /* Application return code */

  (void) module_id_;  /* Gets rid of the compile warning */
  zkErrorNew(&err);

  /*
   * Create a new application context for the server 
   */

  if (!frLodeCtxNew(err,&ctx)) {
    fprintf(stderr,"%s:  %s",argv[0],mesg_1_); 

  /* 
   * Initialize it with default values.  You can see these in the 
   * frlodectx.h file.
   */

  } else if (!frLodeCtxInit(err,ctx)) {
    fprintf(stderr,"%s:  %s",argv[0],mesg_2_);

  /*
   * Load context values from the specified configuration file and the
   * parameters.  When a parameter is specified in both places, the command
   * line value has precedence.
   */

  } else if (!frLodeCtxLoad(err,ctx,argc,argv)) {
    frLodeCtxMessage(err,ctx,mesg_3_);

  /*
   * Verify the context.  This will turn on logging and a few other things
   */

  } else if (!frLodeCtxVerify(err,ctx)) {
    zkLogError(ctx->log,err);
    frLodeCtxMessage(err,ctx,mesg_4_);

  /*
   * Initialize the server.  This will bind it to a port and load the 
   * ZKIP engine with the standard ZKIP commands.
   */

  } else if (!frLodeSrvInitServer(err,ctx)) {
    zkLogError(ctx->log,err);
    frLodeCtxMessage(err,ctx,mesg_6_);

  /*
   * If the daemon flag was specied (it is usually a default), this function
   * will fork and continue in daemon mode.  Regardless, it will create a pid
   * file with the server process's pid.
   */

  } else if (!frLodeSrvMakeDaemon(err,ctx)) {
    zkLogError(ctx->log,err);
    frLodeCtxMessage(err,ctx,mesg_9_);

  /*
   * Set the basic signal handlers for killing the server and cleaning up
   * children which have terminated
   */

  } else if (!frLodeSrvSetSignalHandlers(err,ctx)) {
    zkLogError(ctx->log,err);
    frLodeCtxMessage(err,ctx,mesg_10_);
  } else {

    /*
     * Enable debug mode if required
     */

    if (ctx->flags & FL_DEBUG) {
      ZkLog log = NULL;
      zkErrorLog(err,&log);
      if (ctx->file_log) {
        zkLogSetFile(log,ctx->file_log);
      } else {
        zkLogSetStream(log,stderr);
      }
      zkLogSetProgname(log,ctx->progname);
      zkLogEnable(log); 
      frLodeCtxMessage(err,ctx,mesg_5_);
    }

    /*
     * Add the ZKIP commands 
     */
   
    if (!(zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdLogin,"login",0) &&
          zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdLogout,"logout",0) &&
          zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdPassphrase,"passphrase",0) &&
          zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdCheckAccess,"checkaccess",0) &&
          zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdInfo,"info",0) &&
          zkipEngineAddCommand(err,ctx->engine,CMDID(ZKIP_PROTO_LODE,1,0),
                               frLodeCmdService,"service",0))) {
      zkLogError(ctx->log,err);
      frLodeCtxMessage(err,ctx,mesg_11_);

    /*
     * Run the server
     */

    } else if (!frLodeSrvEventLoop(err,ctx)) {
      zkLogError(ctx->log,err);
      frLodeCtxMessage(err,ctx,mesg_7_);

    /*
     * Server event loop has successfully terminated (usually as a result of
     * a SIGTERM or SIGINT).
     */

    } else {
      exit_code = EXIT_SUCCESS;
    }
  }

  /*
   * Cleanup
   */

  if (!frLodeCtxCleanup(err,ctx)) {
    frLodeCtxMessage(err,ctx,mesg_8_);
  }
  frLodeCtxFree(err,&ctx); 

  exit(exit_code);
  return 0; /* to shut up insure */
}
