/* $Id: frPnewsPost.c,v 1.10 2001/04/19 19:26:17 alainc Exp $ */

#include "zks.h"
#include "zkAlloc.h"
#include "zkCStr.h"
#include "frPnewsPost.h"
#include "frPnewsSpool.h"
#include "zkList.h"
#include "zkFile.h"
#include "zkPFork.h"
#include <sys/signal.h>
#include "zkSignal.h"
#include <unistd.h>
#include "zkAlloc.h"
#include "zkMemBuf.h"
#include "zkMIME.h"
#include "zkEmailUtil.h"
#include "zkFile.h"
#include "rfc0977.h"
#include <netdb.h> 
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "frMailAccount.h"

#define HEADER_ORGANIZATION "Organization"
#define HEADER_COMMENT "X-Comment"
#define HEADER_PATH "Path"

#define ERROR_MESSAGE "From: Freedom Mail2News Daemon <mail2news@%s>\015\012" \
                      "To: %s\015\012" \
                      "Subject: News Posting Failed!\015\012\015\012" \
                      "Your message could not be posted due to the following" \
                      " error.  Please verify\nthat the newsgroups you are" \
                      " posting to are supported by the Freedom Network.\n\n" \
                      "----- Error Message Follows -----------\n\n%s\n\n" \
                      "----- Undelivered Message Follows -----\n\n%s" \
                      "\015\012.\015\012"

#define CROSSPOST_LIMIT_EXCEEDED (-1)




/**
 * Massage the headers and add parameters to it (organization, xheader, ...).
 * Return the From: field for sendmail.
 */
static int massage_headers(ZkError err, FrPostNewsCtx *ctx, ZkEmail email,
                           const char **from, const unsigned char **ng,
                           const unsigned char **subject)
{
  const char *func = "massage_headers";
  ZkEmailHeader eheader;
  ZkEmailField efield;

  if (!ctx || !email || !from || !subject || !ng) {
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  /* Clean up the headers, make sure there is nothing incriminating */
  if (!zkEmailStripHeader(err, email, DONT_STRIP_NEWSGROUPS)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailStripHeader failed");
    return 0;
  }

  if (!zkEmailHeader(err, email, &eheader)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailHeader failed");
    return 0;
  }

  /* Parse the headers for specific headers */
  /* Check if there is a Newsgroups: field */
  if (!zkEmailHeaderGetField(err, eheader, Newsgroups, &efield)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailHeaderGetField failed (newsgroups)");
    return 0;
  }
  
  if (!zkEmailFieldValue(err, efield, ng)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailFieldValue failed (newsgroups)");
    return 0;
  }

  efield = NULL;

  /* Get the From field for use when posting using inews */
  if (!zkEmailHeaderGetField(err, eheader, From, &efield)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailHeaderGetField failed (from)");
    return 0;
  }  
  if (!zkEmailFieldValue(err, efield, (const unsigned char **) from)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailFieldValue failed (from)");
    return 0;
  }
  efield = NULL;

  /* Grab the Subject: line for logging purposes */
  if (!zkEmailHeaderGetField(err, eheader, Subject, &efield)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailHeaderGetField failed (subject)");
    return 0;
  }
  
  if (!zkEmailFieldValue(err, efield, subject)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailFieldValue failed (subject)");
    return 0;
  }

  efield = NULL;

  /* Bug: Strip the headers; the To: field */
  if (!zkEmailHeaderDeleteFieldById(err, eheader, To)) {
    zkLogWriteContext(ctx->log,err,func,
                      "zkEmailHeaderDeleteFieldById failed (to)");
    return 0;
  }

  if (!zkEmailHeaderDeleteFieldById(err, eheader, Organization)) {
    zkLogWriteContext(ctx->log,err,func,
                      "zkEmailHeaderDeleteFieldById failed (organization)");
    return 0;
  }

  /* Add my headers */
  if (ctx->path) {
    if (!zkEmailHeaderAddFieldData(err, eheader, HEADER_PATH, ctx->path)) {
      zkLogWriteContext(ctx->log,err,func,
                        "zkEmailHeaderAddFieldData failed (header path)");
      return 0;
    }
  }

  if (ctx->organization) {
    if (!zkEmailHeaderAddFieldData(err, eheader, HEADER_ORGANIZATION, 
                                   ctx->organization)) {
      zkLogWriteContext(ctx->log,err,func,
                        "zkEmailHeaderAddFieldData failed (header organization)");
      return 0;
    }
  }

  if (ctx->xheader) {
    if (!zkEmailHeaderAddFieldData(err, eheader, HEADER_COMMENT,
                                   ctx->xheader)) {
      zkLogWriteContext(ctx->log,err,func,
                        "zkEmailHeaderAddFieldData failed (header comment)");
      return 0;
    }
  }

  if (!zkEmailHeaderFormatForInews(err, eheader)) {
    zkLogWriteContext(ctx->log,err,func,
                      "zkEmailHeaderFormatForInews failed");
    return 0;
  }

  return 1;
}

/**
 * Append data to the membuf.
 */
static int emailDumpToMemBuf(ZkError err, void *data, 
                             const unsigned char *text, int size)
{
  ZkMemBuf membuf = (ZkMemBuf) data;

  if (!zkMemBufAppend(err, membuf, text, (size_t) size)) {
    return 0;
  }

  return 1;
}

/**
 * Given a ZkEmail object, dump the contents into a newly generated buffer.
 * Return it with the size.
 */
static int emaildump(ZkError err, FrPostNewsCtx *ctx, ZkEmail email,
                     char **outbuf, size_t *osz)
{
  const char *func = "emaildump";
  ZkMemBuf membuf = NULL;
  void *ptr = NULL;
  int ret = 0;
  char *tmp = NULL;
  
  if (!email || !outbuf || !osz) {
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENULLPTR, "NULL param(s)");
    return 0;
  }
  *outbuf = NULL;
  *osz = 0;

  if (!(zkMemBufNew(err, &membuf) &&
        zkEmailDump(err, email, emailDumpToMemBuf, membuf))) {
    zkLogError(ctx->log,err);
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENOMEM, 
               "Error dumping ZkEmail to buffer");
    goto clmembuf;
  }
  

  if (!(zkMemBufAppend(err, membuf, "\0", 1) &&
        zkMemBufGetData(err, membuf, &ptr, osz))) {
    zkLogError(ctx->log,err);
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENOMEM, 
               "Error retrieving email buffer");
    goto clmembuf;
  }

  if ((tmp = malloc(sizeof(char) * *osz)) == NULL) {
    zkLogError(ctx->log,err);
    zkErrorSet_va(err, func, ZKERR, ZKMPNEWS, ZKENOMEM, "malloc(%d) failed",
                  sizeof(char) * *osz);
    goto clmembuf;
  }

  memcpy(tmp, ptr, *osz);
  ptr = NULL;

  *outbuf = tmp;
  tmp = NULL;

  ret = 1;

 clmembuf:
  if (membuf) {
    (void) zkMemBufFree(NULL, &membuf);
  }

  return ret;
}

/* 
 * Return an error message to the sender of the original message.
 */
static int return_tosender(ZkError err, FrPostNewsCtx *ctx, 
                           const char *message, const char *from,
                           int rcode)
{
  const char *func = "return_tosender";
  const char *errmsg = NULL;
  int fd0 = -1;
  int fd1 = -1;
  pid_t pid;
  const char *argv0 = NULL;

  if (!ctx || !message || !from) {
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  zkLogWrite_va(ctx->log, "Failed to post news article, sending mail too %s",
                from);

  /* Choose error message based on return code from the server */
  switch (rcode) {
  case OK_NOPOST:           /* Connection was alright, but I can't post */
    errmsg = "NNTP server refusing articles posted\n";
    break;

  case CROSSPOST_LIMIT_EXCEEDED:
    errmsg = "This message exceeds the cross-post limit";
    break;

  case 0:                   /* Connection error */
  default:
    errmsg = "NNTP server failure, tried multiple times\n";
    break;
  }
   



 
  switch ((pid = zkPFork(err, &fd0, &fd1))) {
  case -1:

    exit(EXIT_FAILURE);
    return 0;
      
  case 0:
    zkLogWrite_va(ctx->log, " I should be dumping newspost bounce message in  %s 's account. Fix me.",
		  from);
    
    /*
    dup2(fd0, 0); close(fd0);
   
                
    if ((argv0 = strrchr(ctx->mta, '/')) != NULL) {
      argv0++;
    } else {
      argv0 = ctx->mta;
    }
      
    zkLogWrite_va(ctx->log, "dumping bounce in  %s 's account.",
		  from);
    {

      char *tmpDir, *newDir;
      char* tmp;
      int len;
      int fd;
      
      ZkError error = NULL;
      char* path = (char*) malloc(sizeof(char) * 1024);
      
            



      char* account = frMailAccount_GetPath( error,
					     "/export/popmail",
					     from,
					     path,
					     1024);   
     


      if(!account) {
	zkLogWrite_va(ctx->log,  "Error computing mailaccount path for the nym that sent the news item.");
	exit(100);
      }
      if(path == NULL){
	zkLogWrite_va(ctx->log,  "Out of memery");
	exit(100);
      }
      
                  

      if((fd =open("/freedom/qmail/control/mailmackey", O_RDONLY)) == -1) {
	
	zkLogWrite_va(ctx->log,  "Error opening mailmackey file");
	exit(100);
      }
      
      zkLogWrite_va(ctx->log,  "Diagnostic-info: got here.");
      if(fd != 3) {
	if(dup2(fd, 3) == -1) {
	  zkLogWrite_va(ctx->log,  "Error dupping mailmackey file descriptor to fd 3");
	  exit(100);
	}
	if(close(fd) == -1) {
	  zkLogWrite_va(ctx->log,  "Error closing mailmackey file descriptor");
	  exit(100);	  
	}
      }
      
      
      if(fcntl(3, F_SETFD, 0 ) == -1) {
	
	zkLogWrite_va(ctx->log,  "Error unsetting close on exec flag for fd 3");
	exit(100);
      }
      

      len = strlen(account);
       
      tmpDir = (char*) malloc(sizeof(char) * (len + 13));
      if(tmpDir == NULL) {
	zkLogWrite_va(ctx->log,  "Out of memery");
	exit(100);
      }
      
      strcpy(tmpDir, account); // audit: ok
      strcat(tmpDir, "/Maildir/tmp"); // audit: ok

      newDir = (char*) malloc(sizeof(char) * (len + 13));
      if(newDir == NULL){
	zkLogWrite_va(ctx->log,  "Out of memery");
	exit(100);
      }

      strcpy(newDir, account); // audit: ok
      strcat(newDir, "/Maildir/new"); // audit: ok

      tmp = strchr(from, '@');
      if(tmp!= NULL) { *tmp = '\0';}

      setenv("LOCAL", from, 1);

      zkLogWrite_va(ctx->log,  "Diagnostic-info: %s %s\n", tmpDir, newDir);

   
      execlp("imep-encrypt", "imep-encrypt", "safecat", tmpDir, newDir, 0); // audit: ok
      exit(EXIT_FAILURE);
    }
    */


  default:
    /* parent should write the bounce message to child on stdout  fix me.*/

    /*
    dup2(fd0, 0); close(fd0);
    dup2(fd1, 1); close(fd1);
      
    {
      char * buf = NULL;
      if (zkCStrSprintf( &buf,
			 ERROR_MESSAGE,
			 FR_DFLT_FREEDOM_DOMAIN,
			 from,
			 errmsg, 
			 message ) < 0) {
	zkLogWriteContext(ctx->log,NULL,func,"Failed to construct message");
	fprintf( stdout,ERROR_MESSAGE,FR_DFLT_FREEDOM_DOMAIN,
		 from,errmsg,message );
      } else {
	fwrite(buf,sizeof(char),strlen(buf),stdout);
	zkMemFree(NULL,(void**)&buf);
      }
    }
    
    fflush(stdout);
    close(1);

    exit(EXIT_SUCCESS);
    */
  }
    
  return 1;
}




/**
 * Post the article, if we fail, send mail back to the from address.
 */
static int postit(ZkError err, FrPostNewsCtx *ctx, const char *outbuf,
                  size_t osz, const char *from, const char *ng,
                  const char *subject)
{
  const char *func = "postit";
  int nfd = -1;                 /* socket descriptor */
  struct protoent *ppe = NULL;
  int ret = 1;
  int retry = 0;
  int rcode = 0;

  if (!ctx || !outbuf || !osz) {
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENULLPTR, "NULL param(s)");
    return 1;
  }

  /* Create a socket */
  if ((ppe = getprotobyname("tcp")) == NULL) {
    zkErrorSet_va(err, func, ZKERR, ZKMPNEWS, errno,
                  "getprotobyname(tcp) failed %s", strerror(errno));
    return 1;
  }

  retry = ctx->nntpretries + 1; /* Try at least once */

  while (retry) {
    if ((nfd = socket(AF_INET, SOCK_STREAM, ppe->p_proto)) == -1) {
      zkErrorSet_va(err, func, ZKERR, ZKMPNEWS, errno, 
                    "Failed to create a socket [%s]", strerror(errno));
      goto closenntp;
    }

    if (connect(nfd, (struct sockaddr *) &ctx->nntpaddr, 
                sizeof(ctx->nntpaddr)) < 0) {
      zkErrorSet_va(err, func, ZKERR, ZKMPNEWS, errno, 
                    "Unable to connect to NNTP server [%s]", strerror(errno));
      ret = 111; /* defer delivery*/
      goto closenntp;
    }

    if (!rfc0977_post(err, nfd, outbuf, osz, &rcode)) {
      retry = 1;                /* For the end of the loop */

      /* Send a mail message back to the from address based on the rcode */
      (void) return_tosender(err, ctx, outbuf, from, rcode);
      ret = 1;
      goto closenntp;
    }

    zkLogWrite_va(ctx->log, "Posting article \"%s\" to %s from %s successful",
                  subject ? subject : "(no subject)", 
                  ng ? ng : "(no newsgroup)",
                  from);

    ret = 0;
    retry = 1;

  closenntp:
    if (nfd > 0) {
      close(nfd);
      nfd = -1;
    } 
    retry--;
  }
  return ret;
}








/**
 * Parse the message in stdin and check the validity, then post it using inews.
 */
int parseit(ZkError err, FrPostNewsCtx *ctx, FILE *stream)
{
  const char *func = "parseit";
  ZkEmail email;
  ZkList grouplist = NULL;
  int ret = 1;
  char buf[BUFSIZ];
  const char *from = NULL;
  const unsigned char *subject = NULL;
  const unsigned char *ng = NULL;
  char *outbuf = NULL;
  size_t osz;
  size_t num_ngs;

  if (!ctx || !stream) {
    zkErrorSet(err, func, ZKERR, ZKMPNEWS, ZKENULLPTR, "NULL param(s)");
    return 1;
  }

  if (!zkEmailNew(err, &email)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailNew failed");
    return 1;
  }

  while (fgets(buf, sizeof(buf), stream) != NULL) {
    if (!zkEmailAddText(err, email, buf, strlen(buf))) {
      zkLogWriteContext(ctx->log,err,func,"zkEmailAddText failed");
      goto cleanup;
    }
  }
  
  if (!zkEmailParse(err, email)) {
    zkLogWriteContext(ctx->log,err,func,"zkEmailParse failed");
    goto cleanup;
  }

  /* Clean up and modify the headers */
  if (!massage_headers(err, ctx, email, &from, &ng, &subject)) {
    zkLogWriteContext(ctx->log,err,func,"massage_headers failed");
    goto cleanup;
  }

  if (!emaildump(err, ctx, email, &outbuf, &osz)) {
    zkLogWriteContext(ctx->log,err,func,"emaildump failed");
    goto cleanup;
  }

  /* Make sure that the cross post limit hasn't been exceeded */
  if (!zkListNew(err,&grouplist)
      || !zkEmailUtilAddToRecipientList(err,grouplist,ng)
      || !zkListSize(err,grouplist,&num_ngs)) {
    zkLogWriteContext(ctx->log,err,func,"Failed to get crosspost list");
    goto cleanup;
  }

  if (num_ngs > ctx->crosspost_limit) {
    zkLogWrite_va(ctx->log,
                  "Article \"%s\" from [%s] to [%s] exceeds crosspost limit",
                  (subject ? (char*)subject : "(no subject)"),from, ng);
    if (!return_tosender(err,ctx,outbuf,from,CROSSPOST_LIMIT_EXCEEDED)) {
      zkLogWriteContext(ctx->log,err,func,"return_tosender failed");
    }     
    
    ret = 0;
    goto cleanup;
  }

  /* Post it */
  ret = postit(err, ctx, outbuf, osz, from, ng, subject);


 cleanup:
  from = NULL;
  if (email) {
    (void) zkEmailFree(NULL, &email);
  }
  
  if (outbuf) {
    free(outbuf);
    outbuf = NULL;
  }

  if (grouplist) {
    (void) zkListFree(NULL,&grouplist);
  }
  return ret;
}









