/*
 *  $Id: rd_config.c,v 1.12 1999/03/31 23:18:47 bill Exp $
 */

#define _GNU_SOURCE 1 /* strcasecmp() */

#include "zks.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <frFilenames.h>
#include "rd_config.h"
#include "zkCStr.h"
#include "zkConfig.h"
#include "zkError.h"
#include "zkParms.h"


#define RD_DFL_AUDIODEV   "/dev/fraudio"
#define RD_DFL_AUDIOINPUT "line"
#define RD_DFL_LOGFILE    FREEDOM_LOGDIR "/frrandd"
#define RD_DFL_RANDFILE   FREEDOM_RUNDIR "/urandom"
#define RD_DFL_RUNDIR     FREEDOM_ROOTDIR


extern char *prog;

static const char cfSection[] = "RandomDataServer";

static int dflDaemon        = 1;
static int dflVerbose       = 0;
static char dflAudioDev[]   = RD_DFL_AUDIODEV;
static char dflAudioInput[] = RD_DFL_AUDIOINPUT;
static char dflLogFile[]    = RD_DFL_LOGFILE;
static char dflRandFile[]   = RD_DFL_RANDFILE; /* bug: should be under freedom root */
static char dflRunDir[]     = RD_DFL_RUNDIR;


/*
 *  bug: unfinished.
 */
static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " --config <file>"
          " [--help]"
          " [--daemon no|yes]"
          " [--verbose no|yes]"
          " [--rundir <dir>]"
          " [--logfile <file>]"
          " [--randfile <file>]"
          " [--audiodev <file>]"
          " [--audioinput line|mic]"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


static int cvtBool(const char *s, int *bool)
{
  if (strcasecmp(s, "1") == 0 ||
      strcasecmp(s, "true") == 0 ||
      strcasecmp(s, "yes") == 0) {
    *bool = 1;
    return 1;
  }

  if (strcasecmp(s, "0") == 0 ||
      strcasecmp(s, "false") == 0 ||
      strcasecmp(s, "no") == 0) {
    *bool = 0;
    return 1;
  }

  return 0;
}


static int cvtStr(const char *s_in, char **s_out)
{
  if (s_in) {
    return !! (*s_out = zkCStrDup(s_in));
  }

  return 0;
}


static int confCmdLine(struct randsrv_config *cf, int ac, char **av)
{
  /* Every option takes one parameter. */

  if ((ac + 1) % 2 != 0) {
    fprintf(stderr, "%s: all options require a single argument.\n", prog);
    return 0;
  }
  
  /* bug? assumes existence of av[1]? */
  
#define TRY(field, type, optstr)                                   \
  if (strcmp("--" optstr, av[0]) == 0) {                           \
    if (cvt ## type(av[1], &cf->field)) {                          \
      av++, --ac; continue;                                        \
    } else {                                                       \
      fprintf(stderr, "%s: error setting value for `%s' option.n", \
              prog, "--" optstr);                                  \
      return 0;                                                    \
    }                                                              \
  }

  /* bug? assumes existence of av[1]? */
  
#define IGNORE(optstr)                   \
  if (strcmp("--" optstr, av[0]) == 0) { \
      av++, --ac; continue;              \
  }

  while (av++, --ac) {
    TRY(daemon,     Bool, P_DAEMON);
    TRY(verbose,    Bool, P_VERBOSE);
    TRY(rundir,     Str,  P_RUNDIR);
    TRY(logfile,    Str,  P_LOGFILE);
    TRY(randfile,   Str,  P_RANDFILE);
    TRY(audiodev,   Str,  P_AUDIO_DEVICE);
    TRY(audioinput, Str,  P_AUDIO_INPUT);
    
    IGNORE(P_PASSPHRASE_FILE);
    IGNORE(P_PASSPHRASE_PROMPT);
    IGNORE(P_PASSPHRASE_FD);
  }

#undef TRY
#undef IGNORE

  return 1;
}


static int confFromFile(struct randsrv_config *cf, char *file)
{
  ZkConfig conf = 0;
  ZkError err = 0;
  const char *name, *value;
  int ret = 0;

  if ( ! zkErrorNew(&err)) {
    fprintf(stderr, "%s: confFromFile: can't create error object.\n", prog);
    return 0;
  }

  if ( ! (zkConfigNew(err, &conf) &&
          zkConfigLoad(err, conf, file))) {
    fprintf(stderr, "%s: confFromFile: can't load config file `%s'.\n",
            prog, file);
    goto done;
  }

  /* bug? assumes existence of av[1]? */
  
#define TRY(field, type, parmstr)                                            \
  if (strcmp(parmstr, name) == 0 &&                                          \
      ! cvt ## type(value, &cf->field)) {                                    \
    fprintf(stderr, "%s: error setting value for config file option `%s'\n", \
            prog, parmstr);                                                  \
    goto done;                                                               \
  }

#define IGNORE(parmstr)
  
  /*
   *  Even though we don't use keys, we still accept the passphrase arguments,
   *  to be consistent with other servers.  (Also, the fradmin script counts
   *  on this behaviour.)
   */
  
  while (zkConfigGetPair(err, conf, cfSection, &name, &value)) {
    TRY(daemon,     Bool, P_DAEMON);
    TRY(verbose,    Bool, P_VERBOSE);
    TRY(rundir,     Str,  P_RUNDIR);
    TRY(logfile,    Str,  P_LOGFILE);
    TRY(randfile,   Str,  P_RANDFILE);
    TRY(audiodev,   Str,  P_AUDIO_DEVICE);
    TRY(audioinput, Str,  P_AUDIO_INPUT);

    IGNORE(P_PASSPHRASE_FILE);
    IGNORE(P_PASSPHRASE_PROMPT);
    IGNORE(P_PASSPHRASE_FD);
  }

#undef TRY
#undef IGNORE

  if (zkErrorCode(err) != ZKENOTFOUND) {
    fprintf(stderr, "%s: confFromFile: error reading section `%s' of file `%s'.\n",
            prog, cfSection, file);
    goto done;
  }

  ret = 1;

 done:

  if (conf) { zkConfigFree(0, &conf); }
  if (err) { zkErrorFree(&err); }

  return ret;
}


static int defaultConfig(struct randsrv_config *conf)
{
  conf->daemon = dflDaemon;
  conf->verbose = dflVerbose;

  if ( ! (cvtStr(dflRunDir,     &conf->rundir)   &&
          cvtStr(dflLogFile,    &conf->logfile)  &&
          cvtStr(dflRandFile,   &conf->randfile) &&
          cvtStr(dflAudioDev,   &conf->audiodev) &&
          cvtStr(dflAudioInput, &conf->audioinput))) {
    return 0;
  }

  return 1;
}


DLLExport int getConfig(struct randsrv_config *conf, int ac, char **av)
{
  char *cfile = 0;
  int ret = 0, i;

  defaultConfig(conf);

  for (i = 1; i < ac; i++) {
    if (strcmp(av[i], "--config") == 0) {
      if (i < ac - 1) {
        cfile = av[i+1];
        break;
      } else {
        fprintf(stderr, "%s: --config option requires an argument.\n",
                prog);
        goto done;
      }
    } else if (strcmp(av[i], "--help") == 0) {
      usage(1);
    }
  }

  if ( ! cfile) {
    fprintf(stderr, "%s: --config option is required.\n",
            prog);
    goto done;
  }

  if ( ! confFromFile(conf, cfile)) {
    goto done;
  }
  
  if ( ! confCmdLine(conf, ac, av)) {
    usage(0);
  }

  ret = 1;

 done:

  return ret;
}
