/*
 *  $Id: rd_out.c,v 1.4 2000/04/25 16:38:04 ulf Exp $
 *  
 *  Write random data to any program requesting it.
 */

#define DEBUG_OUT 1

#include "zks.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <pthread.h>
#include <string.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include "openssl/rand.h"
#include "rd_config.h"
#include "rd_out.h"
#include "rd_thrarg.h"
#include "zkError.h"
#include "zkLog.h"


DLLExport void *write_random(void *thrarg)
{
  ZkLog log;
  int rfd;
  struct thrArg *arg = thrarg;

  pthread_mutex_lock(arg->mutSSLeay);
  (void)zkErrorLog(arg->err, &log);
  pthread_mutex_unlock(arg->mutSSLeay);

  /*
   *  bug? we need a reliable mechanism to ensure no other process is serving
   *  data over the pipe.  A separate lock file, with a write lock?
   */
  
  while ((rfd = open(arg->cf->randfile, O_WRONLY)) != -1) {
    unsigned char d[8];
        
#if DEBUG_OUT
    zkLogWrite(log, "opened randfile");
#endif

    do {
      pthread_mutex_lock(arg->mutSSLeay);
      RAND_bytes(d, sizeof d);
      pthread_mutex_unlock(arg->mutSSLeay);
    } while (write(rfd, &d, sizeof d) != -1);

    close(rfd);

#if DEBUG_OUT
    zkLogWrite(log, "closed randfile");
#endif
  }

#if DEBUG_OUT
  zkLogWrite(log, "randout done");
#endif

  pthread_mutex_lock(arg->mutZkError);
  zkErrorSet_va(arg->err, "write_random", ZKERR, ZKMRANDSRV, errno,
                "can't open `%s' for writing: %s",
                arg->cf->randfile, strerror(errno));
  zkLogError(log, arg->err);
  pthread_mutex_unlock(arg->mutZkError);

  pthread_mutex_lock(arg->mutFatal);
  arg->fatal = 1;
  pthread_cond_signal(arg->cndFatal);
  pthread_mutex_unlock(arg->mutFatal);

  return 0;
}
