/*
 *  dad: dump audio data.
 */

#include <sys/audioio.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stropts.h>
#include <unistd.h>


typedef signed short Sample;


static char *prog;


static int setupAudioDev(int fd, uint_t input)
{
  audio_info_t ai;
  
  if (ioctl(fd, AUDIO_GETINFO, &ai) == -1) {
    fprintf(stderr, "%s: error getting audio info: %s.\n",
            prog, strerror(errno));
    return 0;
  }

  ai.record.precision = 16;
  ai.record.encoding = AUDIO_ENCODING_LINEAR;
  ai.record.gain = AUDIO_MAX_GAIN;
  ai.record.port = input;

  ai.play.precision = 16;
  ai.play.encoding = AUDIO_ENCODING_LINEAR;
  ai.play.gain = AUDIO_MAX_GAIN;
  ai.play.port = input;

  if (ioctl(fd, AUDIO_SETINFO, &ai) == -1) {
    fprintf(stderr, "%s: error setting audio info: %s.\n",
            prog, strerror(errno));
    return 0;
  }

  return 1;
}


int main(int ac, char **av)
{
  char adev[] = "/dev/audio";
  int afd = -1;
  int i, ok = 0;
  size_t nbytes = 1024;
  size_t skip = 0;
  uint_t audiosrc = AUDIO_MICROPHONE;

  prog = av[0];

  while (av++, --ac) {
    if (strcmp(av[0], "--nbytes") == 0) {
      if (ac < 2) {
        fprintf(stderr, "%s: %s requires an argument.\n",
                prog, av[0]);
        goto done;
      }
      if (sscanf(av[1], "%u", &nbytes) != 1) {
        fprintf(stderr, "%s: invalid argument `%s' to %s.\n",
                prog, av[1], av[0]);
        goto done;
      }
      av++, --ac;
    } else if (strcmp(av[0], "--input") == 0) {
      if (ac < 2) {
        fprintf(stderr, "%s: %s requires an argument.\n",
                prog, av[0]);
        goto done;
      }
      if        (strcmp(av[1], "line") == 0) {
        audiosrc = AUDIO_LINE_IN;
      } else if (strcmp(av[1], "mic")  == 0) {
        audiosrc = AUDIO_MICROPHONE;
      } else {
        fprintf(stderr, "%s: argument to %s must be `line' or `mic'.\n",
                prog, av[0]);
        goto done;
      }
      av++, --ac;
    } else if (strcmp(av[0], "--skip") == 0) {
      if (ac < 2) {
        fprintf(stderr, "%s: %s requires an argument.\n",
                prog, av[0]);
        goto done;
      }
      if (sscanf(av[1], "%u", &skip) != 1) {
        fprintf(stderr, "%s: invalid argument `%s' to %s.\n",
                prog, av[1], av[0]);
        goto done;
      }
      av++, --ac;
    } else {
      fprintf(stderr, "%s: invalid argument `%s'.\n", prog, av[0]);
      goto done;
    }
  }

  if ((afd = open(adev, O_RDONLY)) == -1) {
    fprintf(stderr, "%s: error opening %s: %s\n",
            prog, adev, strerror(errno));
    goto done;
  }

  if ( ! setupAudioDev(afd, audiosrc)) {
    fprintf(stderr, "%s: error setting up audio device.\n", prog);
    goto done;
  }
  
  for (i = 0; i < skip; i++) {
    Sample x;
    
    if (read(afd, &x, 1) < 1) {
      fprintf(stderr, "%s: error reading from %s: %s.\n",
              prog, adev, strerror(errno));
      goto done;
    }
  }
  
  for (i = 0; i < nbytes; i++) {
    Sample x;
    
    if (read(afd, &x, 1) < 1) {
      fprintf(stderr, "%s: error reading from %s: %s.\n",
              prog, adev, strerror(errno));
      goto done;
    }

    printf("%ld\n", (long int)x);
  }

  ok = 1;
  
 done:
  
  if (afd != -1) { close(afd); }
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
