/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#include <string>
#include "client.h"
#include "frsimexception.h"
#include "freedomevent.h"
#include "entity.h"
#include "monitor.h"
#include "ppp.h"
#include "ethernet.h"
#include "niqs.h"
#include "aip.h"
#include "statemachinehandler.h"
#include "statemachine.h"
#include "frsimexception.h"


const std::string Client::EntityName = "CLIENT";


Entity & Client::init()
{
  // Set the default attribute  
  setAttribute(THREE_HOP | MODEM_CLIENT);
  return *this;
}


void Client::execute(FreedomEvent * aFreedomEventP)
{
  (void)aFreedomEventP;
	throw FrsimException("Client::execute(FreedomEvent *): incompleted body");
}


Client::sm_id Client::sm_factory(const StateMachine::name_type & name) 
{
  sm_id          id  = m_states.create(name);
  StateMachine & mac = m_states[id];
 
  if( name == "NIQS_TOPOLOGIE_UPDATE" ) {
    mac.add(StateMachine::initial, FreedomEvent::TCP_CONNECT,                        "STEP 1",  StateMachine::final);
    mac.add("STEP 1",              FreedomEvent::TCP_NIQS_QUERY_FULL_ENTITY_LIST,    "STEP 2",  "DISCONNECT");
    mac.add("STEP 2",              FreedomEvent::TCP_NIQS_QUERY_ENTITTDESCRIPTION,   "STEP 3",  "DISCONNECT");
    mac.add("DISCONNECT",          FreedomEvent::TCP_DISCONNECT,                     StateMachine::final);

  } else if( name == "INTERNET_SEND_PACKET" ) {
    mac.add(StateMachine::initial,    FreedomEvent::UDP_AIP_PACKET_SEND,  StateMachine::final);

  } else {
    throw FrsimException(std::string("Client::factory: invalid StateMachine, ") + name);
  }
  return id;
}



FreedomEvent * Client::request_new(Event::Type type, Time launchTime, Entity::Id destination, bool anonRoute)
{  
  unsigned nbhop = (anonRoute ? 0 : 1) + (matchAttribute(THREE_HOP) ? 2 : (matchAttribute(TWO_HOP) ? 1 : 0));
  Entity::Id fst_link;

  if( matchAttribute(MODEM_CLIENT) ) {
    fst_link = g_monitor.entity_get(Ppp::EntityName);

  } else if( matchAttribute(ETHERNET_CLIENT) ) {
    fst_link = g_monitor.entity_get(Ethernet::EntityName);
  }

  EntityIdList eth_link = g_monitor.entity_multipleget(Ppp::EntityName, nbhop);
  EntityIdList aips     = g_monitor.entity_multipleget(Aip::EntityName, nbhop);

  // Create the Event
  FreedomEvent * aEvent = new FreedomEvent(type, launchTime);

  // create the path
  aEvent->addEntity(id());

  aEvent->addEntity(fst_link);

  for( unsigned i=0; i<nbhop; i++ ) {  
    aEvent->addEntity(eth_link[i]);
    aEvent->addEntity(aips[i]);
  }

  aEvent->addEntity(destination);
  return aEvent;
}


