/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#include <iostream>
#include <string>
#include "clock.h"
#include "frsimexception.h"
#include "entity.h"
#include "event.h"
#include "packet.h"
#include "scheduler.h"
#include "frsimeventspecific.h"
#include "freedomevent.h"


GenericIdBin g_entityIdBin;


const std::string Entity::EntityName = "ENTITY";


Entity::Entity()
:m_meter()
,m_attribute(0)
{
	m_id = GenericId::null();	
}


Entity::Entity(const Entity & aEntity)
{
	m_id        = aEntity.m_id;
	m_meter     = aEntity.m_meter;
	m_attribute = aEntity.m_attribute;
}


Entity::~Entity() 
{
  g_entityIdBin.push(m_id);
}


//
// This call initialyzed the Entity, in must be call co mather what.
//
Entity & Entity::init()
{
  // Prevent over initialysation
  if( m_id == GenericId::null() ) {
    m_id = g_entityIdBin.pop(isA());
  }
  return *this;
}


void Entity::execute(Event * aEventP)
{ 
	throw FrsimException(std::string("Entity::execute: not implemented for ") + aEventP->isA());
}


Entity & Entity::operator=(const Entity & aEntity)
{
  if( this != &aEntity ) {
	  g_entityIdBin.push(m_id);
    m_id        = aEntity.m_id;
		m_meter     = aEntity.m_meter;
		m_attribute = aEntity.m_attribute;
  }
  return *this;
}


Metrics Entity::metrics() const
{
  return m_meter.throughput();
}


void Entity::cummulateMetrics(const Packet  & aPacket)
{  
  cummulateMetrics(Metrics(0, aPacket.size(), 0, 0));
}


void Entity::startStopConnection(const Time & when, const Time & elapse) const
{
  startConnection(when);
  stopConnection(when + elapse);
}


void  Entity::startConnection(const Time & when) const
{
  //
  // Setup concurrent connection start
  //
  g_scheduler.push(new FrsimEventSpecific(FrsimEventSpecific::ENTITY_ACCESS_INCREMENT,
                                          when,
                                          Time(0),
                                          id()));
}


void  Entity::stopConnection(const Time & when) const
{
  //
  // Setup concurrent connection end
  //
  g_scheduler.push(new FrsimEventSpecific(FrsimEventSpecific::ENTITY_ACCESS_DECREMENT,
                                          when,
                                          Time(0),
                                          id()));
}



void Entity::execute(FreedomEvent * aFreedomEventP)
{ 
  //
  // Cummulate the imcomming traffic
  //
  Entity::cummulateMetrics(aFreedomEventP->packet());

  //
  // Push the event back to the scheduler if we haven't reach the end of the path.
  //
  if( aFreedomEventP->last() != id() ) {
    // Increment the Event path to the next Entity    
    (aFreedomEventP)++;
    // Push back this event to the scheduler
    g_scheduler.push(aFreedomEventP);

  } else {
    delete aFreedomEventP;
  }
}



void Entity::execute(FrsimEventSpecific * aFrsimEventP)
{ 
  switch( aFrsimEventP->type() ) {

  case FrsimEventSpecific::ENTITY_ACCESS_INCREMENT:
    Entity::cummulateMetrics(Metrics(0,0,0,1));
    break;

  case FrsimEventSpecific::ENTITY_ACCESS_DECREMENT:
    Entity::cummulateMetrics(Metrics(0,0,0,-1));
    break;

  default:
    throw FrsimException("execute(FrsimEventSpecific *): not implemented");
  }
}
