/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

class Event;
class FreedomEvent;
class FrsimEventSpecific;
class Packet;

#ifndef ENTITY_H
#define ENTITY_H

#include <string>
#include "idbin.h"
#include "meter.h"
#include "metrics.h"

extern GenericIdBin g_entityIdBin;



class Entity {
public:
    typedef GenericId           Id;
    typedef unsigned long       attr_type;

public:
    static const std::string    EntityName;

public:
    Entity();
    Entity(const Entity & aEntity);

    virtual ~Entity();

    virtual Entity &            init();

    virtual void                execute(Event              * aEventP);
    virtual void                execute(FreedomEvent       * aFreedomEventP);
    virtual void                execute(FrsimEventSpecific * aFrsimEventP);

    virtual const std::string & isA() const { return EntityName; }

    inline       Id &           id();
    inline const Id &           id() const;

    Entity &                    operator= (const Entity & aEntity);

    Metrics                     metrics() const;

    inline const attr_type &    setAttribute(const attr_type   & aAttribute);
    inline const attr_type &    unsetAttribute(const attr_type & aAttribute);

    inline bool                 matchAttribute(const attr_type & include);
    inline bool                 matchAttribute(const attr_type & include, const attr_type & exclude);

protected:
    void                        startStopConnection(const Time & when, const Time & elapse) const;

    void                        startConnection(const Time & when) const;
    void                        stopConnection(const Time & when) const;

    inline void                 cummulateMetrics(const Metrics & aMetrics);
    void                        cummulateMetrics(const Packet  & aPacket);

protected:
    Id                          m_id;
    Meter                       m_meter;
    attr_type                   m_attribute;

private:

    /**
     * @link dependency
     * @stereotype ManagedBy
     */
  /*#  MetaEntity lnkMetaEntity; */
};


inline const Entity::attr_type & Entity::setAttribute(const attr_type & aAttribute)
{
  m_attribute |= aAttribute;
  return m_attribute;
}


inline const Entity::attr_type & Entity::unsetAttribute(const attr_type & aAttribute)
{
  m_attribute &= ~aAttribute;
  return m_attribute;
}


inline bool Entity::matchAttribute(const attr_type & include)
{
  return (m_attribute &= include) ? true : false;
}


inline bool Entity::matchAttribute(const attr_type & include, const attr_type & exclude)
{
  return ( matchAttribute(include) && !matchAttribute(exclude) ) ? true : false;
}


inline void Entity::cummulateMetrics(const Metrics & aMetrics)
{
  m_meter.add(aMetrics);
}


inline Entity::Id & Entity::id()
{ 
  return m_id; 
}


inline const Entity::Id & Entity::id() const
{ 
  return m_id; 
}


#endif   //ENTITY_H
