/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#include <algorithm>
#include "entitylist.h"
#include "frsimexception.h"
#include "entity.h"


EntityList::EntityList(const EntityList & aEntityList)
:m_list(aEntityList.m_list)
{
}


void EntityList::randomMultipleGet
(
 EntityList & aEntityList, 
 size_type    count,
 attr_type    include,
 attr_type    exclude)
{
  size_type  i;
  iterator   it;
  EntityList tmp_EntityList;
  // Check if the list contain enough Entity
	if( count >= size() ) {
    throw FrsimException("EntityList::randomMultipleGet: Not enough Entity");
  }

#ifndef WIN32
# warning "Should use the Randomizer to shuffle the list"
#endif

  // Shuffle the list
  std::random_shuffle(begin(), end());

  // Populate the return list
  for( i=(size_type)0, it=begin(); (i < count) && (it != end()); it++ ) {      
      if( (*it)->matchAttribute(include, exclude) ) {
        tmp_EntityList.add(*it);
        i++;
      }
  }
  if( i != count ) {
    throw FrsimException("EntityList::randomMultipleGet: Not enough valid Entity");
  }
  aEntityList += tmp_EntityList;
}


EntityList EntityList::operator+ (const EntityList & aEntityList) const
{
  return (EntityList(*this) += aEntityList);
}


EntityList & EntityList::operator+=(const EntityList & aEntityList)
{
  if( this != &aEntityList ) {
    std::copy(aEntityList.m_list.begin(), aEntityList.m_list.end(), m_list.end());
  }
  return *this;
}


Entity * EntityList::randomGet(attr_type include,
                               attr_type exclude)
{
#ifndef WIN32
# warning "Should use the Randomizer to shuffle the list"
#endif

	if( size() ) {
    // Shuffle the list
    std::random_shuffle(begin(), end());

    for( iterator it=begin(); it!=end(); it++ ) {      
      if( (*it)->matchAttribute(include, exclude) ) {
        return *it;
      }
    }
  }
  throw FrsimException("EntityList::randomGet: Not enough Entity");
  return NULL;
}


void EntityList::remove(const Entity * aEntityP)
{
  // Search for the Entity
  iterator it=std::find(begin(), end(), aEntityP);
  // if found
  if( it != m_list.end() ) {
      m_list.erase(it);
  }
}


void EntityList::add(Entity * aEntityP)
{
  m_list.push_back(aEntityP);
}


void EntityList::clear()
{
  while ( ! m_list.empty() ) {
    m_list.pop_back(); //bugbug: Should I use delete here
  }
}


Entity * EntityList::find(Entity::Id aId)
{
	for( iterator it=begin(); it!=end(); it++ ) {
		if( (*it)->id() == aId ) {
			return *it;
		}
	}
	return NULL;
}


Entity * EntityList::find(const size_type pos)
{
  if( pos < m_list.size() ) {
    return m_list[pos];
  }
  return NULL;
}

const Entity * EntityList::find(const size_type pos) const
{
  if( pos < m_list.size() ) {
    return m_list[pos];
  }
  return NULL;
}
