/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef ENTITYLIST_H
#define ENTITYLIST_H

#include <vector>
#include <algorithm>
#include "entity.h"


class EntityList {
 private:
  typedef std::vector<Entity * >            list_type;
 
 public:
  typedef Entity::attr_type                 attr_type;

  typedef list_type::value_type             value_type;
  typedef list_type::size_type              size_type;
  typedef list_type::iterator               iterator;
  typedef list_type::const_iterator         const_iterator;
  typedef list_type::reverse_iterator       reverse_iterator;
  typedef list_type::const_reverse_iterator const_reverse_iterator;

 public:
  EntityList() {}

  EntityList(const EntityList & aEntityList);

  ~EntityList() {}

  iterator       begin()         { return m_list.begin(); }
  iterator       end()           { return m_list.end(); }
  const_iterator begin()   const { return m_list.begin(); }
  const_iterator end()     const { return m_list.end(); }

  size_type      size()    const { return m_list.size(); }
  bool           empty()   const { return m_list.empty(); }
  void           reverse()       { std::reverse(begin(), end()); }

  Entity       * find(const size_type pos);
  const Entity * find(const size_type pos) const;

  EntityList   operator+  (const EntityList & aEntityList) const;
  EntityList & operator+= (const EntityList & aEntityList);

  Entity       * find(Entity::Id aId);
  void           add(Entity * aEntityP);
  void           remove(const Entity * aEntityP);

  Entity       * randomGet(attr_type include = 0, 
                           attr_type exclude = 0);

  void           randomMultipleGet(EntityList & aEntityList, 
                                   size_type count, 
                                   attr_type include = 0, 
                                   attr_type exclude = 0);

  void           clear();
  
private:
    /**
     * @supplierCardinality * 
     */
    list_type m_list;
};
#endif //ENTITYLIST_H
