/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#include <string>
#include "entity.h"
#include "freedomevent.h"
#include "frsimexception.h"
#include "clock.h"
#include "entity.h"


const std::string FreedomEvent::EventName = std::string("FREEDOMEVENT");


void FreedomEvent::addLatency(Time latency)
{
  *(static_cast<Event *>(this)) += latency;
}


const Entity::Id & FreedomEvent::first() const
{
  if( !m_path.size() ) {
    return GenericId::null();
  }
  return m_path[0];
}


const Entity::Id & FreedomEvent::last() const
{
  size_type size = m_path.size();

  if( !size ) {
    return GenericId::null();
  }
  return m_path[size];
}


const Entity::Id & FreedomEvent::current() const
{
  if( !m_path.size() ) {
    return GenericId::null();
  }
  return m_path[m_pos];
}


const Entity::Id &  FreedomEvent::operator++(int)
{
  return ++(*this);
}

const Entity::Id &  FreedomEvent::operator--(int)
{
  return --(*this);
}


const Entity::Id &  FreedomEvent::operator++()
{
  if( m_pos >= (m_path.size()-1) ) {
    throw FrsimException("FreedomEvent::operator++: overflow");
    return GenericId::null();
  }
  return m_path[++m_pos];
}       


const Entity::Id &  FreedomEvent::operator--()
{
  if (!m_pos) {
    throw FrsimException("FreedomEvent::operator--: underflow");
    return GenericId::null();
  }
  return m_path[--m_pos];
}


const Entity::Id & FreedomEvent::operator+(size_type increment)
{
  unsigned long pos = m_pos+increment;

  // Look for wrap around and boundary coundition
  if( (increment < (((unsigned long)-1) - pos)) && (pos < (m_path.size()-1)) ) {
    return m_path[pos];
  }
  return GenericId::null();
}


const Entity::Id & FreedomEvent::operator-(size_type decrement)
{
  // Look for  wrap around and boundary condition
  if( (decrement <= m_pos) && (m_pos-decrement < (m_path.size()-1)) ) {
    return m_path[m_pos-decrement];
  }
  return GenericId::null();
}


//
// Add an Entity to the Entity path.
//
void FreedomEvent::addEntity(const Entity::Id & aEntityId)
{
  m_path.add(aEntityId);
}


void FreedomEvent::reply(bool success)
{
  m_path.reverse();
  m_pos = 0;
  m_state = success ? SUCCESS : FAILURE;
  m_packet.clear();
}

bool FreedomEvent::isQuery() const
{
 return (m_state == QUERY) ? true : false;
}


bool FreedomEvent::isAnswer() const
{
 return ((m_state == SUCCESS) || (m_state == FAILURE));
}


void FreedomEvent::clear()
{
  m_pos = 0;
  m_path.clear();
  m_packet.clear();
  m_state = QUERY;
}

