/* Zero-Knowledge Systems Inc. - Freedom Simulator */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef FREQUENCY_H
#define FREQUENCY_H

#include <iostream>
#include <iomanip>
#include "stime.h"

class Frequency {
public:
	Frequency();
	explicit Frequency(unsigned long aAmount, const Time & aPeriod);

	double                frequence();
	unsigned long       & amount();
	Time                & period();
	const unsigned long & amount() const;
	const Time          & period() const;

  inline operator bool() const;

  friend std::ostream & operator<< (std::ostream & ostr, const Frequency & aFrequency);
  friend std::istream & operator>> (std::istream & istr,       Frequency & aFrequency);

private:
  unsigned long m_amount;
  Time          m_period;
};


inline Frequency::Frequency():m_amount(0), m_period(0)
{	 
}


inline Frequency::Frequency(unsigned long aAmount, const Time & aPeriod):m_amount(aAmount), m_period(aPeriod)
{
}


inline double Frequency::frequence()
{
	return m_period / m_amount;
}


inline unsigned long & Frequency::amount()
{
	return m_amount;
}


inline Time & Frequency::period()
{
    return m_period;
}


inline const unsigned long & Frequency::amount() const
{
	return m_amount;
}


inline const Time & Frequency::period() const
{
    return m_period;
}


inline Frequency::operator bool() const
{             
  return ( m_amount || m_period ) ? true : false;
}


inline std::ostream & operator<< (std::ostream & ostr, const Frequency & aFrequency)
{
  ostr << std::dec << aFrequency.m_amount
       << "/" 
       << aFrequency.m_period;

  return ostr;
}
    

inline std::istream & operator>> (std::istream & istr, Frequency & aFrequency)
{
  istr >> std::dec >> aFrequency.m_amount;
  istr.ignore(strlen("/"));
  istr >> aFrequency.m_period;

  return istr;
}

#endif     //FREQUENCY_H
