/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef FRSIMEVENTSPECIFIC_H
#define FRSIMEVENTSPECIFIC_H
#include <string>
#include "stime.h"
#include "frsimevent.h"
#include "entity.h"
#include "propertylist.h"


//
// Event sent to the Entities to manage there behavior
//
class FrsimEventSpecific: public FrsimEvent {
 public:
  enum Type {
    ENTITY_ACCESS_INCREMENT,   // Increment the concurrent access counter.
    ENTITY_ACCESS_DECREMENT,   // Decrement the concurrent access counter.
		ENTITY_DESTROY,            // Destroy a given Entity.
		ENTITY_STATE_MACHINE_RUN,  // Execute a state machine.
  };

 public:
  static const std::string EventName;

 public:
  FrsimEventSpecific(Type       type, 
                     Time       launchTime, 
                     Time       repeatDelta,
                     Entity::Id entityId);

  inline const std::string & isA()      const;
  inline const Entity::Id  & entityId() const;

  inline Property          & operator[] (const std::string & aTag);
  inline const Property    & operator[] (const std::string & aTag) const;


 private:
  Entity::Id    m_entityId;
  PropertyList  m_property;  // List of property that can be associate to a packet

};
 

inline FrsimEventSpecific::FrsimEventSpecific(Type       type, 
                                              Time       launchTime, 
                                              Time       repeatDelta,
                                              Entity::Id entityId)
:FrsimEvent(type, launchTime, repeatDelta)
,m_entityId(entityId) 
{
}


inline const std::string & FrsimEventSpecific::isA() const  
{
  return EventName; 
}


inline const Entity::Id  & FrsimEventSpecific::entityId() const 
{
  return m_entityId; 
}


inline Property & FrsimEventSpecific::operator[] (const std::string & aTag)
{
  return m_property[aTag];
}

inline const Property & FrsimEventSpecific::operator[] (const std::string & aTag) const
{
  return m_property[aTag];
}

#endif  //FRSIMEVENTSPECIFIC_H
