/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif


#include "idbin.h"
#include "frsimexception.h"


// ////////////////////////////////////////////////////////////////////////////
// GenericId
//

GenericId GenericId::m_null_id = GenericId(0,"");


GenericId::GenericId()
{
	*this = GenericId::null();
}


GenericId::GenericId(id_type id, name_type name)
:m_id(id), m_name(name)
{
}


GenericId::GenericId(const GenericId & aId)
{
	*this = aId;
}


GenericId & GenericId::operator=(const GenericId & aId)
{
  if( &aId != this ) {
    m_id   = aId.m_id;
    m_name = aId.m_name;
  }
  return *this;
}


const std::string & GenericId::name() const
{
	return m_name;
}


void GenericId::name(const std::string & name)
{
	m_name = name;
}


bool GenericId::operator==(id_type id) const
{
  return ( m_id == id ) ? true : false;
}


bool GenericId::operator==(const GenericId & aId) const
{
  return ( m_id == aId.m_id ) ? true : false;
}


bool GenericId::operator!=(const GenericId & aId) const
{
  return !(*this == aId);
}


bool GenericId::operator>(const GenericId & aId) const
{
  return ( m_id > aId.m_id ) ? true : false;
}


bool GenericId::operator>=(const GenericId & aId) const
{
  return ( m_id >= aId.m_id ) ? true : false;
}


bool GenericId::operator<(const GenericId & aId) const
{
  return ( m_id < aId.m_id ) ? true : false;
}


bool GenericId::operator<=(const GenericId & aId) const
{
  return ( m_id <= aId.m_id ) ? true : false;
}



//
// Rotate GenericId value
//
GenericId & GenericId::rotate()
{
  // Rotate th high bits of the value
  m_id = (((m_id >> 24) + 1UL) << 24) | (m_id & 0x00ffffffUL);
  return *this;
}


//
// Increase the GenericId value
//
GenericId & GenericId::operator++(int)
{
  // Check boundary condition
  if( !(m_id ^ 0x00ffffffUL) ) {
    throw FrsimException("GenericId::operator++(int): overflow");
  }
  m_id = ((m_id & 0x00ffffffUL) + 1UL) | (m_id & 0xff000000UL);
  return *this;
}


GenericId & GenericId::operator=(id_type id)
{
  if( !id ) {
    throw FrsimException("GenericId::operator=(): invalid id");
  }
  m_id = id;
  return *this;
}




// ////////////////////////////////////////////////////////////////////////////
// GenericIdBin 
//

GenericIdBin::GenericIdBin(GenericId::id_type min, GenericId::id_type max)
:m_freelist()
{
  m_current = min;    
  m_max     = max;
}


//
// Pop out an GenericId from the bin
//
GenericId GenericIdBin::pop(const std::string & name)
{
  if( m_freelist.size() == 0 ) {
		GenericId aId = m_current++;
    // Over check boundary condition
    if( aId == m_max ) {
      throw FrsimException("GenericIdBin::pop: overflow");
    }
		aId.name(name);
    return aId;
  }
  GenericId aId = m_freelist.back();
  aId.name(name);
  m_freelist.pop_back();
  return aId;
}


//
// Push back an GenericId to the bin
//
void GenericIdBin::push(const GenericId & aId)
{
  //
  // Rotate GenericId value than push it through the stack.
  // It would delayed the reusing of this GenericId.
  //
  if( aId != GenericId::null() ) {    
		m_freelist.push_front(GenericId(aId).rotate());
	}
  
}


