/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef IDBIN_H
#define IDBIN_H

#include <list>
#include <map>
#include <string>

class IdBin;

//
// GenericId
//
class GenericId {
  enum {
    MIN_ID = 0x00000001UL,
    MAX_ID = 0x01000000UL 
  };

  friend class GenericIdBin;

  typedef unsigned long id_type;
  typedef std::string   name_type;

 private:
  GenericId(id_type id, name_type name);

 public:
  GenericId();
  GenericId(const GenericId & aId);

  const std::string & name() const;
	void                name(const std::string & name);
	
  GenericId & operator=(const GenericId & aId);

  bool operator==(id_type id)            const;

  bool operator==(const GenericId & aId) const;
  bool operator!=(const GenericId & aId) const;
  bool operator> (const GenericId & aId) const;
  bool operator>=(const GenericId & aId) const;
  bool operator< (const GenericId & aId) const;
  bool operator<=(const GenericId & aId) const;

  inline static const GenericId & null();

  friend std::ostream & operator<< (std::ostream & ostr, const GenericId & aGenericId);
  friend std::istream & operator>> (std::istream & istr,       GenericId & aGenericId);

 private:
  GenericId & rotate();
  GenericId & operator++(int);

  GenericId & operator=(id_type id);

 private:
  id_type   m_id;
  name_type m_name;

 private:
  static GenericId m_null_id;
};


inline std::ostream & operator<< (std::ostream & ostr, const GenericId & aGenericId)
{
  ostr << "<";
  ostr << std::dec << aGenericId.m_id;
  ostr << "::";
  ostr << aGenericId.m_name;
  ostr << ">";

  return ostr;
}
    

inline std::istream & operator>> (std::istream & istr, GenericId & aGenericId)
{
  istr.ignore(strlen("<"));
  istr >> std::dec >> aGenericId.m_id;
  istr.ignore(strlen("::"));
  istr >> aGenericId.m_name;
  istr.ignore(strlen(">"));

  return istr;
}

inline const GenericId & GenericId::null()
{
  return GenericId::m_null_id;
}

//
// General Id Bin
//
class GenericIdBin {  
    typedef std::list<GenericId> id_fifo;

public:
    GenericIdBin(GenericId::id_type min = GenericId::MIN_ID, GenericId::id_type max = GenericId::MAX_ID);

    void      push(const GenericId & aId);
    GenericId pop(const std::string & name);
private:
    GenericId::id_type m_max;
    GenericId          m_current;
    id_fifo            m_freelist;
};



#endif       //IDBIN_H
