/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif


class Event;
//class Entity;
class FreedomEvent;
class FrsimEventGeneric;
class FrsimEventSpecific;

#ifndef METAENTITY_H
#define METAENTITY_H

#include <string>
#include <map>
#include <set>
#include "entity.h"
#include "entitylist.h"
#include "entityidlist.h"
#include "initinfolist.h"
#include "property.h"
#include "statusreport.h"


class MetaEntity {
  typedef std::map<const std::string, EntityList> map_type;
  typedef std::set<std::string>                   set_type;

 public:
	typedef EntityList::size_type                   size_type;
  typedef EntityList::attr_type                   attr_type;

  typedef set_type::const_iterator                const_iterator;

  inline  const_iterator  begin() const;
  inline  const_iterator  end()   const;
  
 public:
  MetaEntity() {}
  inline virtual         ~MetaEntity();

  MetaEntity &            init(const InitInfoList & aInitInfoList);
      
  virtual void            execute(FreedomEvent       * aFreedomEventP);
  virtual void            execute(FrsimEventGeneric  * aFrsimEventP);
  virtual void            execute(FrsimEventSpecific * aFrsimEventP);

  StatusReport            entity_status(const std::string aEntityName);
  void                    entity_schedule(const std::string aEntityName);
  void                    entity_create(const std::string aEntityName);
	size_type               entity_count(const std::string aEntityName);

  Entity::Id              entity_get(const std::string aEntityName,
                                     attr_type         include = 0, 
                                     attr_type         exclude = 0);

  EntityIdList            entity_multipleget(const std::string aEntityName,
                                             size_type         count,
                                             attr_type         include = 0, 
                                             attr_type         exclude = 0);
	
 protected:

  Property              & initinfo(const std::string aEntityName);
  const Property        & initinfo(const std::string aEntityName) const;

  virtual Entity        * entity_new(const std::string aEntityName);

	void                    entity_subscribe(const std::string entityName);
  void                    entity_init_all(const std::string aEntityName);
  void                    entity_delete_all(const std::string aEntityName);
	void                    entity_remove(Entity * aEntityP);

	inline void             entity_add(Entity * aEntityP);
  inline Entity         * entity_find(Entity::Id aId);

  inline EntityList     & entity_list(const std::string aEntityName);

 protected:  
  mutable map_type        m_entitymap;
  mutable InitInfoList    m_initinfolist;
  mutable set_type        m_subscriber;
};


inline MetaEntity::const_iterator MetaEntity::begin() const 
{
	return m_subscriber.begin(); 
}


inline MetaEntity::const_iterator MetaEntity::end() const
{
	return m_subscriber.end(); 
}


inline MetaEntity::~MetaEntity()
{ 
  //
  // Scan the subscribers list and cleanup Entity Instances
  //
  for( const_iterator it = begin(); it != end(); it++ ) {
    entity_delete_all(*it);
  }  
}


inline void MetaEntity::entity_add(Entity * aEntityP)
{
  entity_list(aEntityP->isA()).add(aEntityP);
}


inline Entity * MetaEntity::entity_find(Entity::Id aId)
{
  return entity_list(aId.name()).find(aId);
}


inline EntityList & MetaEntity::entity_list(const std::string aEntityName)
{
  return m_entitymap[aEntityName];
}

#endif     //METAENTITY_H
