#include "meter.h"


#ifndef WIN32
# warning "Should be implemented using a circular list of pair <Time,Metrics>"
#endif


void Meter::add(const Metrics & aMetrics) 
{
  Time curtime = g_clock.getTime();

  // Insert | Add a new Metrics to the priority_queue.
  m_vector[curtime] += aMetrics;
  // Remove obsoleted entry.
  if( m_vector.size() > static_cast<size_type>(m_eval_period.sec()) ) {
    Time min = curtime - m_eval_period;

    for( iterator it=begin(); it!=end(); it++ ) {
      if( (*it).first < min ) {
        erase(it);
        it=begin(); // Restart the scan has the iterator has been invalidated
      }
    }
  }
}



Metrics Meter::throughput() const 
{
  Time     curtime = g_clock.getTime();
  Metrics  cumul;
  unsigned long nb = 0;

  // Remove obsoleted entry.
  Time min = (curtime >= m_eval_period) ? curtime - m_eval_period: curtime;

  //
  // Should use accumulate but egcs is broken and 
  // crash compile when parsing the binary function.
  //

  for( const_iterator it=begin(); it!=end(); it++ ) {
    if( (*it).first > min ) {
      cumul += (*it).second;
      nb++;
    }
  }
  if( nb ) {
    cumul /= nb;
  }
  return cumul;
}


void Meter::clear()
{
  m_vector.clear();
}
