/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

class Metrics;

#ifndef METER_H
#define METER_H

#include <map>
#include <utility>
#include <functional>
#include <numeric>
#include "clock.h"
#include "metrics.h"


class Meter {
  typedef std::map<Time, Metrics>   vect_type;
  typedef vect_type::size_type      size_type;
  typedef vect_type::key_type       key_type;
#ifdef WIN32
  typedef vect_type::referent_type  data_type;
#else
	typedef vect_type::data_type      data_type;
#endif	
  typedef vect_type::value_type     value_type;
  typedef vect_type::key_compare    key_compare;
  typedef vect_type::value_compare  value_compare;
  typedef vect_type::iterator       iterator;
  typedef vect_type::const_iterator const_iterator;

 public:
  Meter(unsigned long eval_period = 10):m_eval_period((Time::time_type)eval_period) {}
  
  void add(const Metrics & aMetrics);
  void clear();

  Metrics throughput() const;

  
 private:
  vect_type      m_vector;
  Time           m_eval_period;
 private:
  iterator       begin()            { return m_vector.begin(); }
  iterator       end()              { return m_vector.end(); }
  const_iterator begin() const      { return m_vector.begin(); }
  const_iterator end() const        { return m_vector.end(); }
  void           erase(iterator it) { m_vector.erase(it); }
};


#endif //METER_H
