/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef METRICS_H
#define METRICS_H
#include "stime.h"

class Metrics {
  typedef long   metrics_type;
 public:
  inline Metrics();
  inline Metrics(metrics_type lookup, metrics_type throughput, metrics_type connections, metrics_type concurrent);
  inline Metrics(const Metrics & aMetrics);

  inline metrics_type & lookup()      { return m_lookup; }
  inline metrics_type & bytes()       { return m_throughput; }
  inline metrics_type & connections() { return m_connections; }
  inline metrics_type & concurrent()  { return m_concurrent; }

  inline const metrics_type & lookup() const      { return m_lookup; }
  inline const metrics_type & bytes()  const      { return m_throughput; }
  inline const metrics_type & connections() const { return m_connections; }
  inline const metrics_type & concurrent()  const { return m_concurrent; }

  inline Metrics & operator= (const Metrics & aMetrics);

  inline Metrics   operator+ (const Metrics & aMetrics) const;
  inline Metrics & operator+=(const Metrics & aMetrics);

  inline Metrics   operator/ (metrics_type denom) const;
  inline Metrics & operator/=(metrics_type denom);

 private:

  /**
   * @link dependency
   * @stereotype CollectedBy
   */
  /*#  Monitor lnkMonitor; */

 protected:
  metrics_type m_lookup;
  metrics_type m_throughput;
  metrics_type m_connections;
  metrics_type m_concurrent;
};


inline Metrics::Metrics():m_lookup(0),m_throughput(0),m_connections(0),m_concurrent(0)
{
}



inline Metrics::Metrics(metrics_type lookup, metrics_type throughput, metrics_type connections, metrics_type concurrent)
:m_lookup(lookup)
,m_throughput(throughput)
,m_connections(connections)
,m_concurrent(concurrent)
{  
}


inline Metrics::Metrics(const Metrics & aMetrics)
{
  *this = aMetrics;
}


inline Metrics & Metrics::operator=(const Metrics & aMetrics)
{
  if( this != &aMetrics ) {
    m_lookup      = aMetrics.m_lookup;
    m_throughput  = aMetrics.m_throughput;
    m_connections = aMetrics.m_connections;
    m_concurrent  = aMetrics.m_concurrent;
  }
  return *this;
}


inline Metrics & Metrics::operator+=(const Metrics & aMetrics) 
{
  m_lookup      += aMetrics.m_lookup;
  m_throughput  += aMetrics.m_throughput;
  m_connections += aMetrics.m_connections;
  m_concurrent  += aMetrics.m_concurrent;
  return *this;		 
}		  	  


inline Metrics Metrics::operator+(const Metrics & aMetrics) const 
{
  return (Metrics(*this) += aMetrics);
}		  	  


inline Metrics Metrics::operator/ (metrics_type denom) const
{
  Metrics result = *this;

  result.m_lookup      /= denom;
  result.m_throughput  /= denom;
  result.m_connections /= denom;
  result.m_concurrent  /= denom;
  return result;		 
}		  	  


inline Metrics & Metrics::operator/=(metrics_type denom)
{
  m_lookup      /= denom;
  m_throughput  /= denom;
  m_connections /= denom;
  m_concurrent  /= denom;
  return *this;
}		  	  

#endif  //METRICS_H
