/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

class MetaEntity;
class FreedomEvent;
class FrsimEventControl;
class FrsimEventGeneric;
class FrsimEventSpecific;
//class InitInfoList;


#ifndef MONITOR_H
#define MONITOR_H
#include <map>
#include "scheduler.h"
#include "metrics.h"
#include "metaentity.h"


class Monitor {
  typedef std::map < std::string, MetaEntity * > meta_map;
  typedef meta_map::iterator                     iterator;
  typedef meta_map::const_iterator               const_iterator;

 public:  
#ifdef WIN32
  typedef enum    { RUN    = 0x00000001UL, 
                    PAUSE  = 0x00000002UL, 
                    STOP   = 0x00000004UL,
                    UPDATE = 0x00000008UL
                  } state_type;  
#else  
  enum state_type { RUN    = 0x00000001UL, 
                    PAUSE  = 0x00000002UL, 
                    STOP   = 0x00000004UL,
                    UPDATE = 0x00000008UL
                  };
#endif

 public:
	typedef EntityList::size_type                  size_type;
  typedef EntityList::attr_type                  attr_type;

 public:
  Monitor() {};
  ~Monitor();

  Monitor & init(const InitInfoList & aInitInfoList);

  void run();
  void update();
  void stop();

  state_type state() const { return m_state; }

	unsigned long  entity_count(const std::string aEntityName);

  Entity::Id     entity_get(const std::string aEntityName,
                            attr_type         include = 0, 
                            attr_type         exclude = 0);

  EntityIdList   entity_multipleget(const std::string aEntityName,
                                    size_type         count,
                                    attr_type         include = 0, 
                                    attr_type         exclude = 0);  

 private:
  iterator       begin()       { return m_metamap.begin(); }
  iterator       end()         { return m_metamap.end(); }
  const_iterator begin() const { return m_metamap.begin(); }
  const_iterator end()   const { return m_metamap.end(); }

  void           execute(FreedomEvent       * aFreedomEventP);
	void           execute(FrsimEventControl  * aFrsimEventP);
  void           execute(FrsimEventGeneric  * aFrsimEventP);
  void           execute(FrsimEventSpecific * aFrsimEventP);

 private:
  meta_map   m_metamap;
  state_type m_state;

  /**
   * @link dependency
   * @stereotype ControledBy-
   */

  /*#  Supervisor lnkSupervisor; */
};

extern Monitor g_monitor;

#endif   //MONITOR_H
