/* Zero-Knowledge Systems Inc. - Freedom Simulator */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef PROPERTY_H
#define PROPERTY_H

#include <string>
#include <map>
#include "stime.h"
#include "frequency.h"
#include "timerange.h"
#include "idbin.h"




// ////////////////////////////////////////////////////////
//
// Init Info handler
//
// ////////////////////////////////////////////////////////

class Property {
#ifdef WIN32 // Crapy compiler: didn't recownyze inbeded declaration A::B::C 
 public:
#endif
  class Attr {
  public:
    Attr() {}

    Attr & operator=(const bool & val);
    Attr & operator=(const char & val);
    Attr & operator=(const short & val);
    Attr & operator=(const long & val);
    Attr & operator=(const int & val);
    Attr & operator=(const unsigned char & val);
    Attr & operator=(const unsigned short & val);
    Attr & operator=(const unsigned long & val);
    Attr & operator=(const unsigned int & val);
    Attr & operator=(const Frequency & val);
    Attr & operator=(const Time & val);
    Attr & operator=(const TimeRange & val);
    Attr & operator=(const GenericId & val);
    Attr & operator=(const std::string & val);

		bool           asBool() const;
		char           asChar() const;
		short          asShort() const;
		long           asLong() const;
		int            asInt() const;
		unsigned char  asUnsignedChar() const;
		unsigned short asUnsignedShort() const;
		unsigned long  asUnsignedLong() const;
		unsigned int   asUnsignedInt() const;
		Frequency      asFrequency() const;
		Time           asTime() const;
    GenericId      asGenericId() const;
		TimeRange      asTimeRange() const;
		std::string    asString() const;

  public:
    std::string value;
  };
  
  typedef std::map< std::string, Attr> map_type;
  typedef map_type::value_type      value_type;
  typedef map_type::iterator        iterator;
  typedef map_type::const_iterator  const_iterator;

 public:
  iterator       begin()       { return m_attr.begin(); }
  iterator       end()         { return m_attr.end();   }
  const_iterator begin() const { return m_attr.begin(); }
  const_iterator end()   const { return m_attr.end();   }

 public:
  inline Property &     set(const std::string & tag, const bool & value);
  inline Property &     set(const std::string & tag, const char & value);
  inline Property &     set(const std::string & tag, const short & value);
  inline Property &     set(const std::string & tag, const long & value);
  inline Property &     set(const std::string & tag, const int & value);
  inline Property &     set(const std::string & tag, const unsigned char & value);
  inline Property &     set(const std::string & tag, const unsigned short & value);
  inline Property &     set(const std::string & tag, const unsigned long & value);
  inline Property &     set(const std::string & tag, const unsigned int & value);
  inline Property &     set(const std::string & tag, const Frequency & value);
  inline Property &     set(const std::string & tag, const Time & value);
	inline Property &     set(const std::string & tag, const GenericId & value);
	inline Property &     set(const std::string & tag, const TimeRange & value);
  inline Property &     set(const std::string & tag, std::string & value);

  inline bool           asBool(const std::string & tag) const;
  inline char           asChar(const std::string & tag) const;
  inline short          asShort(const std::string & tag) const;
  inline long           asLong(const std::string & tag) const;
  inline int            asInt(const std::string & tag) const;
  inline unsigned char  asUnsignedChar(const std::string & tag) const;
  inline unsigned short asUnsignedShort(const std::string & tag) const;
  inline unsigned long  asUnsignedLong(const std::string & tag) const;
  inline unsigned int   asUnsignedInt(const std::string & tag) const;
  inline Frequency      asFrequency(const std::string & tag) const;
  inline Time           asTime(const std::string & tag) const;
	inline GenericId      asGenericId(const std::string & tag) const;
	inline TimeRange      asTimeRange(const std::string & tag) const;
  inline std::string    asString(const std::string & tag) const;

 private:
  map_type m_attr;
};


inline Property & Property::set(const std::string & tag, const bool & value)
{
  m_attr[tag] = value;
  return *this;
}


inline Property & Property::set(const std::string & tag, const char & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const short & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const long & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const int & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const unsigned char & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const unsigned short & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const unsigned long & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const unsigned int & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, std::string & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const Time & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const GenericId & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const TimeRange & value)
{
  m_attr[tag] = value;
  return *this;
}

inline Property & Property::set(const std::string & tag, const Frequency & value)
{
  m_attr[tag] = value;
  return *this;
}


inline bool Property::asBool(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asBool();
  }
  return static_cast<bool>(0);
}


inline char Property::asChar(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asChar();
  }
  return static_cast<char>(0);
}

inline short Property::asShort(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asShort();
  }
  return static_cast<short>(0);
}

inline long Property::asLong(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asLong();
  }
  return static_cast<long>(0);
}

inline int Property::asInt(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asInt();
  }
  return static_cast<int>(0);
}

inline unsigned char Property::asUnsignedChar(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asUnsignedChar();
  }
  return static_cast<unsigned char>(0);
}

inline unsigned short Property::asUnsignedShort(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asUnsignedShort();
  }
  return static_cast<unsigned short>(0);
}

inline unsigned long Property::asUnsignedLong(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asUnsignedLong();
  }
  return static_cast<unsigned long >(0);
}

inline unsigned int Property::asUnsignedInt(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asUnsignedInt();
  }
  return static_cast<unsigned int>(0);
}

inline Frequency Property::asFrequency(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asFrequency();
  }
  return Frequency();
}

inline Time Property::asTime(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asTime();
  }
  return Time();
}

inline GenericId Property::asGenericId(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asGenericId();
  }
  return GenericId();
}

inline TimeRange Property::asTimeRange(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asTimeRange();
  }
  return TimeRange();
}

inline std::string Property::asString(const std::string & tag) const
{
  const_iterator it = m_attr.find(tag);

  if( it != end() ) {
    return (*it).second.asString();
  }
  return std::string();
}



#endif     //PROPERTY_H
