/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#include <time.h>

#include "frequency.h"
#include "randomizer.h"
#include "timerange.h"
#include "stime.h"


#ifndef WIN32
# warning "Randomizer class should use Yarrow!"
#endif

Randomizer g_randomizer;

Randomizer::Randomizer()
{
	seed((unsigned long) ::clock());
}


void Randomizer::seed(unsigned long aSeed)
{
	::srand(aSeed);
}


unsigned long Randomizer::rand(unsigned long max)
{
	unsigned long rval = (unsigned long) ::rand();
  seed(rval * (unsigned long)::clock());
	return (unsigned long)((double)rval * ((double)max / (double)RAND_MAX));
}


Time Randomizer::rand(const Time & max)
{
	unsigned long sec  =  max.sec();
	unsigned long usec =  max.usec();

	sec  = ( sec ) ? rand(sec-1) : 0;
  usec = rand(Time::USEC_PER_SEC);

	return Time(sec, usec);
}


Time Randomizer::rand(const TimeRange & range)
{
	return range.min() + rand(range.delta());
}


Randomizer::list_type Randomizer::rand(const Time & start, const Frequency & freq)
{
	list_type list;

  if( freq.period()	) {
	  while( list.size() < list_type::size_type(freq.amount()) ) {
		  list.insert(rand(freq.period()) += start);
	  }
  }
	return list;
}


Randomizer::list_type Randomizer::rand(const Frequency & freq)
{
	return rand(Time(0), freq);
}
