/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef RANGE_H
#define RANGE_H

#include <iostream>


template <class T>
class Range
{
	public:
		inline Range();
	  inline Range(const T & from, const T & to);

		inline bool in(const T & value) const;
		inline bool out(const T & value) const;

		inline const T & min() const { return m_min; }
		inline const T & max() const { return m_max; }
		inline T &       min()       { return m_min; }
		inline T &       max()       { return m_max; }

		inline T         delta() const;

		inline           operator bool() const;

	  std::ostream & iostream_output (std::ostream & ostr) const;
		std::istream & iostream_input  (std::istream & istr);

	private:
		T m_min;
		T m_max;
};

template <class T>
inline Range<T>::Range()
:m_min(T())
,m_max(T())
{
}

template <class T>
inline Range<T>::Range(const T & from, const T & to)
{
	if( from < to ) {
		m_min = from;
		m_max = to;
	} else {
		m_max = from;
		m_min = to;
	}
}

template <class T>
inline bool Range<T>::in(const T & value) const
{
	return ( (m_min <= value) && (value <= m_max) ) ? true : false;
}

template <class T>
inline bool Range<T>::out(const T & value) const
{
	return ( (value < m_min) || (m_max < value) ) ? true : false;
}

template <class T>
inline T Range<T>::delta() const
{
	return m_max - m_min;
}


template <class T>
inline Range<T>::operator bool() const
{
	return( m_min || m_max ) ? true : false;
}


template <class T>
std::ostream & Range<T>::iostream_output(std::ostream & ostr) const
{
  ostr << "[";
	ostr << m_min;
	ostr << ", ";
	ostr << m_max;
	ostr << "]";
  return ostr;
}


template <class T>
std::istream & Range<T>::iostream_input(std::istream & istr)
{
	istr.ignore(strlen("["));
	istr >> m_min;
	istr.ignore(strlen(", "));
	istr >> m_max;
	istr.ignore(strlen("]"));
  return istr;
}

template <class T>
inline std::ostream & operator<< (std::ostream & ostr, const Range<T> & aRange)
{
  return aRange.iostream_output(ostr);
}


template <class T>
inline std::istream & operator>> (std::istream & istr, Range<T> & aRange)
{
  return aRange.iostream_input(istr);
}

#endif //RANGE_H
