/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )

#endif

#include "frsimexception.h"
#include "statemachine.h"


const StateMachine::state_type StateMachine::initial = std::string("INITIAL_STATE");
const StateMachine::state_type StateMachine::final   = std::string("FINAL_STATE");


StateMachine::StateMachine(const name_type & name)
{
  m_name    = name;
  m_current = initial;
  m_state_map[m_current];
}


//
// Reset the machine to it's initial state
//
StateMachine & StateMachine::reset()
{
  m_current = initial;
  return *this;
}


//
// Reset the machine to it's initial state
//
StateMachine & StateMachine::stop()
{
  m_current = final;
  return *this;
}


//
// Clear the StateMachine content
//
StateMachine & StateMachine::clear()
{
  m_state_map.clear();
  m_current = initial;
  m_state_map[m_current];
  return *this;
}


StateMachine & StateMachine::add(const state_type & aState, const event_type & aTrans, const state_type & aSuccess, const state_type & aFailure)
{
  m_state_map[aState].trans   = aTrans;
  m_state_map[aState].success = aSuccess;
  m_state_map[aState].failure = aFailure;
  return *this;
}


const StateMachine::state_type & StateMachine::operator=(const state_type & aState)
{
  if( m_state_map.find(aState) == m_state_map.end() ) {
    throw FrsimException(std::string("StateMachine::operator=: invalid state, ") + aState);
  }
  m_current = aState;
  return m_current;
}


const StateMachine::state_type & StateMachine::operator+= (bool success)
{
  state_type & next = success ? m_state_map[m_current].success : m_state_map[m_current].failure;

  if( m_state_map.find(next) == m_state_map.end() ) {
    throw FrsimException(std::string("StateMachine::operator=: invalid transition, ") 
                         + m_current + std::string(" -> ") + next);
  }
  m_current = next;
  return m_current;
}


const StateMachine::state_type & StateMachine::current() const
{
  return m_current;
}


const StateMachine::event_type & StateMachine::transition() const
{
  return m_state_map[m_current].trans;
}
