/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef STATEMACHINE_H
#define STATEMACHINE_H

#include <map>
#include <string>
#include "event.h"


class StateMachine {  
  typedef std::string  name_type;
  typedef Event::Type  event_type;
  typedef std::string  state_type;

  typedef struct {
    event_type trans;
    state_type success;
    state_type failure;
  } trans_type;

  typedef map<state_type, trans_type>  state_map;
 
 public:
  static const state_type initial;
  static const state_type final;

 public:
  StateMachine(const name_type & name = "");

  StateMachine     & reset();
  StateMachine     & stop();

  StateMachine     & clear();
  
  StateMachine     & add(const state_type & aState, const Event::Type & aTrans, const state_type & aSuccess, const state_type & aFailure = final);

  const state_type & operator+= (bool success);

  const event_type & transition() const;
  const state_type & current() const;

  const state_type & operator= (const state_type & aState);

 private: 
  mutable std::string m_name;
  mutable state_map   m_state_map;
  mutable state_type  m_current;
};



#endif //STATEMACHINE_H
