/* Zeroknowledge Systems Inc. - Freedom Simulation  */

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif

#ifndef STATUSREPORT_H
#define STATUSREPORT_H
#include <string>
#include "metrics.h"
#include "stime.h"
#include "clock.h"

class StatusReport {
public:
	inline StatusReport();
	inline StatusReport(unsigned long instance, const Metrics & metrics);
	inline StatusReport(const StatusReport & aStatusReport);

  inline StatusReport & operator=(const StatusReport & aStatusReport);

	inline Time                & time();
  inline unsigned long       & instance();
	inline Metrics             & metrics();
	inline const Time          & time() const;
  inline const unsigned long & instance() const;
	inline const Metrics       & metrics() const;

private:        
	Time          m_time;
  unsigned long m_instance;
  Metrics       m_metrics;
};


inline StatusReport & StatusReport::operator=(const StatusReport & aStatusReport)
{
  if( this != &aStatusReport ) {
    m_time       = aStatusReport.m_time;
    m_instance   = aStatusReport.m_instance;
    m_metrics    = aStatusReport.m_metrics;
  }
  return *this;
}


inline StatusReport::StatusReport()
:m_time(g_clock.getTime())
,m_instance(0)
,m_metrics()
{
}


inline StatusReport::StatusReport(unsigned long instance, const Metrics & metrics)
:m_time(g_clock.getTime())
,m_instance(instance)
,m_metrics(metrics)  
{
}


inline StatusReport::StatusReport(const StatusReport & aStatusReport)
{
  *this = aStatusReport;
}


inline Time & StatusReport::time()
{
  return m_time; 
}


inline unsigned long & StatusReport::instance()
{
  return m_instance;
}


inline Metrics & StatusReport::metrics()
{
  return m_metrics; 
}


inline const Time & StatusReport::time() const
{
  return m_time; 
}


inline const unsigned long & StatusReport::instance() const
{
  return m_instance;
}


inline const Metrics & StatusReport::metrics() const
{
  return m_metrics; 
}



#endif    //STATUSREPORT_H
