#ifndef HARVESTER_APP_H
# define HARVESTER_APP_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif


# ifdef __cplusplus  

#  include "harvester.h"
#  include "keys.h"

#  include "zks.h" 
#  include "zkIntTypes.h"
#  include "zkscmdline.h"
#  include "zksini.h"

#  include "frKeyQry.h"

#  include <string> 


// /////////////////////////////////////////////////////////////////////////////
// Application context handler
//
class Application {
  Application(const Application &);
  Application & operator=(const Application &);  

 public:
  enum State { 
    ACCEPT = 0,  // Accept state or wait to be on accept mode
    HOLD,        // Hold state or wait to be on hold
    STOP};       // Stop state or wait to be on stop

 public:
  Application()
    :m_debug(false)
    ,m_verbose(false)
    ,m_daemon(false)
    ,m_passprompt("")
    ,m_passfile("")
    ,m_priv_keys(0)
    ,m_crypt_version(0)
    ,m_sig_version(0)
    {
    }

  virtual ~Application()
    {
      delete m_priv_keys;
    }

  void init_appname(int32_t argc, const char ** argv);
  void init_cmdline(int32_t argc, const char ** argv);
  void init_ini();
  void init_logfile();
  void init_debug_status();
  void init_verbose_status();
  void init_passphrase();
  void init_daemon_status();
  void init_period();
  void init_priv_keys();

  void init_public_key();
  void init_dest();

  void init_sighandler();

  void lock_pid();
  void unlock_pid();

  void run();

  // Attribute access
  inline const std::string & name()              const { return m_appname; } 
  inline const CZKS::Ini   & ini()               const { return m_ini; } 
  inline cKeys             * priv_keys()         const { return m_priv_keys; } 
  inline uint16_t            crypt_key_version() const { return m_crypt_version; }
  inline uint16_t            sig_key_version()   const { return m_sig_version; }

  inline bool                debug()             const { return m_debug; }
  inline bool                verbose()           const { return m_verbose; }

 protected:
  std::string   m_appname;            // Name of the application.
  CZKS::Ini     m_ini;                // Configuration file handler.
  CZKS::CmdLine m_cmdline;            // Command line handler.
  std::string   m_logfile;            // Log file name.
  std::string   m_inifile;            // Configuration file name.
  bool          m_debug;              // true if running in debug mode.  (enable console logging)
  bool          m_verbose;            // true if running in verbose mode.(enable extra messages)
  bool          m_daemon;             // true if running as a daemon.    (enable extra messages)
  std::string   m_passprompt;         // Passphrase prompt
  std::string   m_passfile;           // Passphrase file name
  cKeys       * m_priv_keys;          // Private keys handler
  uint16_t      m_crypt_version;      // Version of the current crypto key
  uint16_t      m_sig_version;        // Version of the current signature key

  std::string   m_entity_name;
  std::string   m_entity_type;

  uint16_t      m_period;

  FILE *        m_pidFptr;            // Pointer to locked PID file


  // member needed to get the public encryption key from the key server
  // FIXME: should go in the function init_public_key
  std::string   m_kqdHostname;        // default host name for the kqd server
  uint16_t      m_kqdPort;            // default port for the kqd server
  FrKeyQry      m_public_kc;          // public key cache
  unsigned int  m_keyVers;            // key version used to sign packet
  FrEntIdExp    m_entid;              // entity id of the server to which send data
  FrKeyQryHost  m_kqHost;             // key query server host object
  int           m_kqRetries;          // number of key query retries
  int           m_kqTimeout;          // timeout for key query


  FrCryptPubKey m_cryptionKey;        // the public encryption key

  struct sockaddr_in m_server;        // the destination server
};


# endif /* __cplusplus */

#endif /* HARVESTER_APP_H */
