#define REV_ID "@(#) $Id: collect.cpp,v 1.2 2001/05/16 18:08:31 build Exp $"
#include "zkrev.h"

#include "zks.h"

#include "collect.h"
#include "harvester.h"

#include "zksexception.h"
#include "zkslogwrap.h"

#include "zkIntTypes.h"

#include "frCrypt.h"
#include "frSec.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <list>

#include <netdb.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <errno.h>

namespace ICSPhase1Data {

    //
    // parses /proc/net/dev to extract bytes/packets that have been sent/received
    //
    void getEth0( uint32_t &bytesIn,
                  uint32_t &bytesOut,
                  uint32_t &pktsIn,
                  uint32_t &pktsOut) {

      DECL_FUNC("ICSPhase1Data::getEth0");
      
      std::list<std::string> list;
      std::ifstream file("/proc/net/dev");
      std::string s;
      std::string::size_type pos;

      std::string tmp;

      bool found = false;

      // look for the line starting with "eth0:"
      while( ! found ) {
          std::getline(file, s);
          if( file.eof() ) {
              CZKS::Throw(ZKS_HARVESTER_MOD,
                          ZKS_HARVESTER_DATAFILE_ERR,
                          func,
                          "Didn't find token 'eth0:' in /proc/net/dev");
          }
          if( (pos = s.find("eth0:")) != std::string::npos ) {
              found = true;
          }
      }

      std::istringstream strm(s.substr(pos+strlen("eth0:")));

      strm >> bytesIn >> pktsIn >> tmp >> tmp >> tmp >> tmp >> tmp >> tmp >> bytesOut >> pktsOut;
    }

    //
    // parses /proc/net/dev to extract bytes/packets that have been sent/received
    //
    void getHostUptime( uint32_t &hostUptime) {

        //DECL_FUNC("ICSPhase1Data::getHostUptime");
      
        std::list<std::string> list;
        std::ifstream file("/proc/uptime");

        uint32_t uptime;

        file >> uptime;

        hostUptime = uptime;
    }



    //
    // collects the data, forms a message with it and sends it on the net
    //
    void GenerateMessage(std::string entityName,
                         FrCryptSig signingKey,
                         FrCryptCrypt cryptionKey,
                         struct sockaddr_in server) {

        DECL_FUNC("ICSPhase1Data::GenerateMessage");

        CZKS::Exception e;

        uint32_t bytesIn, bytesOut, pktsIn, pktsOut;
        uint32_t hostUptime;

        FrCryptSymAlg symalg = FRCRYPT_SYMALG_BLOWFISH;

        struct protoent *ppe = NULL;
        int fd;
        int bytes;

        std::ostringstream  oString;

        char  *dataBuf   = NULL;
        size_t dataBsz   = 0;

        void  *signedBuf = NULL;
        size_t signedBsz = 0;

        void  *cryptBuf  = NULL;
        size_t cryptBsz  = 0;

        try {

            //
            // get data
            //
            getEth0(bytesIn, bytesOut, pktsIn, pktsOut);
            getHostUptime(hostUptime);
            
            //
            // format it
            //
            oString << "hostname="   << entityName;
            oString << " timestamp=" << time(NULL);
            oString << " hostuptime=" << hostUptime;
            oString << " eth0_bytes_i=" << bytesIn;
            oString << " eth0_bytes_o=" << bytesOut;
            oString << " ports_bytes_i=0";  // hardcoded to 0
            oString << " ports_bytes_o=0";  // hardcoded to 0

            dataBsz = strlen( oString.str().c_str() ) + 1;
            dataBuf = new char[dataBsz];
            strcpy( dataBuf, oString.str().c_str() );


            CZKS::Log::debug(CZKS::Log::DATA | CZKS::Log::TRACE, 
                             ZKS_HARVESTER_MOD,
                             "message [%s]", dataBuf);

            //
            // send the formated information
            //
            // sign
            if( ! frSecSignData(e.zkError(), signingKey,
                                (void *)dataBuf, dataBsz, &signedBuf, &signedBsz) ) {
                CZKS::Throw(ZKS_HARVESTER_MOD,
                            ZKS_HARVESTER_FRSEC_ERR,
                            func,
                            "Failed at signing data");
            }

            // encrypt
            if( ! frSecPubKeyEncryptData(e.zkError(), cryptionKey, symalg,
                                         signedBuf, signedBsz, &cryptBuf, &cryptBsz)) {
                CZKS::Throw(ZKS_HARVESTER_MOD,
                            ZKS_HARVESTER_FRSEC_ERR,
                            func,
                            "Failed at encrypting data");
            }

            // open a socket
            ppe = getprotobyname("udp");
            if((fd = socket(AF_INET, SOCK_DGRAM, ppe->p_proto)) < 0) {
                CZKS::Throw(ZKS_HARVESTER_MOD,
                            ZKS_HARVESTER_SOCKET_ERR,
                            func,
                            "Failed creating socket");
            }

            // send
            if( ( bytes = sendto(fd, cryptBuf, (int)cryptBsz, 0,
                                 (struct sockaddr *)&server, sizeof(struct sockaddr_in))) < 0) {
                CZKS::Throw(ZKS_HARVESTER_MOD,
                            ZKS_HARVESTER_SENDTO_ERR,
                            func,
                            "Failed at sending data to network: %d %s", bytes, strerror(errno));
            }
            if(cryptBsz != (size_t) bytes) {
                CZKS::Throw(ZKS_HARVESTER_MOD,
                            ZKS_HARVESTER_SENDTO_ERR,
                            func,
                            "Sending incomplete data to network");
            }

            // close the socket
            close(fd);

            // free buffers
            if(dataBuf)   { delete [] dataBuf; }
            if(signedBuf) { free(signedBuf); signedBuf = NULL; }
            if(cryptBuf)  { free(cryptBuf);  cryptBuf  = NULL; }


        } catch ( ... ) {

            // free buffers
            if(signedBuf) { free(signedBuf); signedBuf = NULL; }
            if(cryptBuf)  { free(cryptBuf);  cryptBuf  = NULL; }

            CZKS::Rethrow(func, "failure");    
        }
    }

} // namespace ICSPhase1Data
