/*
 *  $Id: IcsContext.h,v 1.3 2001/05/16 20:35:41 build Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Freedom code.
 *
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2001
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 *
 * Contributors:
 *   Dan Kha Pham danp@zeroknowledge.com
 */


#ifndef __ICSCONTEXT_H__
#define __ICSCONTEXT_H__

#include <string.h>
#include <sys/time.h>
#include <fstream>
#include "zkscmdline.h"
#include "fr_keychain.h"
#include "fr_keyqry.h"
#include "fr_keyqry_fetchkey.h"
#include "fr_sig.h"
#include "zkslogwrap.h"

// Default configuration parameters
#define DEFAULT_ICSPORT        51201
#define DEFAULT_SELECTTIMEOUT  2
#define DEFAULT_DAEMONFLAG     0
#define DEFAULT_DEBUGFLAG      0
#define DEFAULT_PASSPHRASEFILE ""
#define DEFAULT_PPHRASEPROMPT  ""
#define DEFAULT_CONFIGFILE     "/usr/freedom/etc/freedom-"ICS_APPNAME".conf"
#define DEFAULT_LOGFILE        "/usr/freedom/var/log/"ICS_APPNAME".log"
#define DEFAULT_DATAFILE       "/usr/freedom/var/log/"ICS_APPNAME".data"
#define DEFAULT_PIDFILE        "/usr/freedom/var/run/"ICS_APPNAME".pid"
#define DEFAULT_KEYSDIR        "/usr/freedom/privkey/NISSD"
#define DEFAULT_KQDHOSTNAME    "kqd.freedom.net"
#define DEFAULT_KQDTIMEOUT      5
#define DEFAULT_KQDRETRIES      10
#define DEFAULT_KQDPORT         51109

// Command line specifiers for configuration parameters
#define ICS_PORT_CMDLINESTR       "port"
#define ICS_PASSPHR_CMDLINESTR    "passphrase.file"
#define ICS_PASSPHRPR_CMDLINESTR  "passphrase.prompt"
#define ICS_CONFIG_CMDLINESTR     "config.file"
#define ICS_LOG_CMDLINESTR        "log.file"
#define ICS_DATA_CMDLINESTR       "data.file"
#define ICS_PID_CMDLINESTR        "pid.file"
#define ICS_KEYSDIR_CMDLINESTR    "keys.dir"
#define ICS_KQDHOST_CMDLINESTR    "kqd.hostname"
#define ICS_KQDTIMEOUT_CMDLINESTR "kqd.timeout"
#define ICS_KQDRETRIES_CMDLINESTR "kqd.retries"
#define ICS_KQDPORT_CMDLINESTR    "kqd.port"


class IcsContext
{

public:
  IcsContext();
  virtual ~IcsContext();

  int init( int argc, const char **argv,
            const char *pIcsAppName,
            const char *pIcsAppDescr,
            const char *pIcsAppVersion );

  void showContext();
  void lockPidFile();
  int verify();

  unsigned short getPort() { return m_Port; }
  int getSocket() { return m_Socket; }
  int getSelectTimeout() { return m_SelectTimeout; }
  int getDaemonFlag() { return m_DaemonFlag; }
  int getDebugFlag() { return m_DebugFlag; }
  std::string getPassphraseFile() { return m_PassphraseFile; }
  std::string getPassphrasePrompt() { return m_PassphrasePrompt; }
  std::string getLogFile() { return m_LogFile; }
  std::string getDataFile() { return m_DataFile; }
  std::string getPidFile() { return m_PidFile; }
  std::string getKeysDirectory() { return m_KeysDirectory; }
  std::string getKqdHostName() { return m_KqdHostName; }
  FrKeyQry getKeyQueryCache() { return m_KeyQueryCache; }
  FrKeyQryHost getKeyQueryHost() { return m_KeyQueryHost; }
  unsigned short getKqdTimeout() { return m_KqdTimeout; }
  unsigned short getKqdRetries() { return m_KqdRetries; }
  int getSignatureKeyHandle() { return m_SignatureKeyHandle; }
  int getEncryptionKeyHandle() { return m_EncryptionKeyHandle; }
  std::string getEncryptionKeyDir() { return m_EncryptionKeyDir; }
  std::string getSignatureKeyDir() { return m_SignatureKeyDir; }
  FrSecKeyChain getKeyChain() { return m_KeyChain; }

  fd_set         m_ReadFdSet;
  std::ofstream  m_DataFileStream;

private:
  unsigned short m_Port;
  int            m_Socket;
  int            m_SelectTimeout;
  int            m_DaemonFlag;
  int            m_DebugFlag;
  std::string    m_PassphraseFile;
  std::string    m_PassphrasePrompt;
  std::string    m_LogFile;
  std::string    m_DataFile;
  std::string    m_PidFile;
  FILE          *m_pPidFile;
  std::string    m_KeysDirectory;

  std::string    m_KqdHostName;
  unsigned long  m_KqdAddress;
  FrKeyQry       m_KeyQueryCache;
  FrKeyQryHost   m_KeyQueryHost;
  unsigned short m_KqdTimeout;
  unsigned short m_KqdRetries;
  unsigned short m_KqdPort;

  CZKS::CmdLine  m_Cmdline;

  int            m_SignatureKeyHandle;
  int            m_EncryptionKeyHandle;
  std::string    m_EncryptionKeyDir;
  std::string    m_SignatureKeyDir;
  FrSecKeyChain  m_KeyChain;

  CZKS::Exception m_Exception;

  int bindUdpPort( unsigned short udpPort, int *pSocket );
  void initKeyChain();

};


#endif   // ifndef __ICSCONTEXT_H__
