

#include <iostream>
#include <sys/types.h>
#include <sys/socket.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>

#include <arpa/inet.h>



class IcsClientSimulator
{

public:
  IcsClientSimulator();
  ~IcsClientSimulator() {}
  int sendMessage( std::string icsHostName,
                   unsigned short icsPort, 
                   std::string message );

private:
  int                m_Socket;
  struct sockaddr_in m_Server;
  struct sockaddr_in m_Client;
  struct protoent *  m_pProtocol;

};


IcsClientSimulator::IcsClientSimulator()
{
  m_Socket = -1;
  m_pProtocol = NULL;(struct protoent *)getprotobyname( "udp" );
  memset( &m_Server, 0, sizeof(m_Server) );
  memset( &m_Client, 0, sizeof(m_Client) );

  if( (m_pProtocol=(struct protoent *)getprotobyname("udp"))==NULL )
    std::cout << "Error: protocol data structure is NULL." << std::endl;
  else if( (m_Socket=socket(AF_INET,SOCK_DGRAM,m_pProtocol->p_proto)) < 0 )
    std::cout << "Error creating socket: " << strerror(errno) << std::endl;
}

int IcsClientSimulator::sendMessage( std::string ip,
                                     unsigned short icsPort, 
                                     std::string message )
{
  if( ip=="" )
  {
    std::cout << "Invalid hostname '" << ip.c_str() << "'" << std::endl;
    return -1;
  }
  if( icsPort<1024 )
  {
    std::cout << "Invalid port number: " << icsPort << std::endl;
    return -1;
  }
  if( message=="" )
    std::cout << "Warning: empty message." << std::endl;

  size_t datasz = strlen( (const char*)message.c_str() ) + 1;

  unsigned long ip_addr = inet_addr( ip.c_str() );

  m_Server.sin_addr.s_addr = ip_addr;
  m_Server.sin_family = AF_INET;
  m_Server.sin_port = htons( icsPort );

  m_Client.sin_addr.s_addr = htonl( INADDR_ANY );
  m_Client.sin_port = htons( 0 );
  m_Client.sin_family = AF_INET;

  if( bind(m_Socket,(struct sockaddr *)&m_Client,sizeof(struct sockaddr_in)) < 0 )
  {
    std::cout << "Error binding client socket: " << strerror(errno) << std::endl;
    return -1;
  }

  return sendto(m_Socket,(const void *)message.c_str(),(int)datasz,0,
                (struct sockaddr *)&m_Server,sizeof(struct sockaddr_in));

}


int main( int argc, char **argv)
{

  IcsClientSimulator icsclient;
  int bytes = 0;

  if( argc==3 )
  {
    std::cout << "Sending [" << argv[1] << "] to " << argv[2] << \
               ":" << 51201 << std::endl;
    bytes = icsclient.sendMessage( argv[2], 51201, argv[1] );
  }
  else if( argc==4 )
  {
    std::cout << "Sending [" << argv[1] << "] to " << argv[2] << \
               ":" << argv[3] << std::endl;
    bytes = icsclient.sendMessage( argv[2], (unsigned short)atoi(argv[3]), argv[1] );
  }
  else
  {
    std::cout << "Usage: icsclient <message> <ip> [port]" << std::endl;
    return -1;
  }

  std::cout << "Number of bytes sent: " << bytes << std::endl;

  return 0;
}
