/*
 *  $Id: kc_config.c,v 1.7 1999/03/31 23:18:43 bill Exp $
 */

#define _GNU_SOURCE /* strcasecmp() */

#include "zks.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <frFilenames.h>
#include "frNetworkDefaults.h"
#include "kc_config.h"
#include "zkCStr.h"
#include "zkConfig.h"
#include "zkError.h"
#include "zkParms.h"


extern char *prog;


static const char cfSection[] = S_DAEMON_KEYCHANGE;


#if 0
static int argExists(const char *arg, int ac, char **av)
{
  while (av++, --ac) {
    if (strcmp(arg, av[0]) == 0) {
      return 1;
    }
  }

  return 0;
}
#endif


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s \n"
          "         [--" P_DAEMON " {yes|no}]\n"
          "         [--" P_ONCE " {yes|no}]\n"
          "         [--" P_RETRY_SLEEP " <seconds>]\n"
          "         [--" P_PROXY_CONFIG_FILE " <path>]\n"
          "         [--" P_RUNDIR " <path>]\n"
          "         [--" P_LOCKFILE_CRYPT " <path>]\n"
          "         [--" P_LOGFILE " <path>]\n"
          "         [--" P_PIDFILE " <path>]\n"
          "         [--" P_RANDFILE " <path>]\n"
          "         [--" P_VERBOSE " {yes|no}]\n"
          "         [--" P_KEYUPD_HOSTNAME " <host-name>]\n"
          "         [--" P_KEYUPD_PORT " <port-number>]\n"
          "         [--" P_KEYQRY_TIMEOUT " <seconds>]\n",
          prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


static void configInit(struct frkeychg_config *cf)
{
  cf->daemon = 1;
  cf->once = 0;
  cf->naptime = 600; /* ten minutes */
  cf->proxyconfig = FREEDOM_CONFDIR "/freedom.conf";
  cf->rundir = FREEDOM_ROOTDIR;
  cf->lockfile = FREEDOM_CRYPT_LOCK_FILE;
  cf->logfile = FREEDOM_LOGDIR "/frkeychgd";
  cf->pidfile = FREEDOM_RUNDIR "/frkeychgd";
  cf->randsrc = FREEDOM_RUNDIR "/urandom";
  cf->srvhost = FR_DFLT_KEYQRY_SERVER_HOST;
  cf->srvport = FR_DFLT_FMGMAIP_SERVER_PORT;
  cf->srvtimeout = 30; /* seconds */
  cf->verbose = 0;
  
  cf->systemSymKey = 0;
}


static int cvtBool(const char *s, int *bool)
{
  if (strcasecmp(s, "1") == 0 ||
      strcasecmp(s, "true") == 0 ||
      strcasecmp(s, "yes") == 0) {
    *bool = 1;
    return 1;
  }

  if (strcasecmp(s, "0") == 0 ||
      strcasecmp(s, "false") == 0 ||
      strcasecmp(s, "no") == 0) {
    *bool = 0;
    return 1;
  }

  return 0;
}


#if 0
static int cvtInt(const char *s, int *n)
{
  char *end;
  long l;
  
  errno = 0;
  l = strtol(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }

  *n = l;
  return 1;
}
#endif


static int cvtUInt(const char *s, unsigned int *n)
{
  char *end;
  unsigned long l;
  
  errno = 0;
  l = strtoul(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }

  *n = l;
  return 1;
}


#if 0
static int cvtShort(const char *s, short *n)
{
  char *end;
  long l;
  
  errno = 0;
  l = strtol(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }

  *n = l;
  return 1;
}
#endif


static int cvtUShort(const char *s, unsigned short *n)
{
  char *end;
  unsigned long l;
  
  errno = 0;
  l = strtoul(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }

  *n = l;
  return 1;
}


/*
 *  Harbison & Steele claims that time_t can be any arithmetic type.  We'll
 *  assume unsigned long.
 */
static int cvtTime_t(const char *s, time_t *n)
{
  char *end;
  unsigned long l;
  
  errno = 0;
  l = strtoul(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }

  *n = l;
  return 1;
}


static int cvtStr(const char *s_in, const char **s_out)
{
  if (s_in) {
    return !! (*s_out = zkCStrDup(s_in));
  }

  return 0;
}


static int confCmdLine(struct frkeychg_config *cf, int ac, char **av)
{
  /* Every option takes one parameter. */

  if ((ac + 1) % 2 != 0) {
    fprintf(stderr, "%s: all options require a single argument.\n", prog);
    return 0;
  }
  
#define TRY(field, type, optstr)                                   \
  if (strcmp("--" optstr, av[0]) == 0) {                           \
    if (cvt ## type(av[1], &cf->field)) {                          \
      av++, --ac; continue;                                        \
    } else {                                                       \
      fprintf(stderr, "%s: error setting value for `%s' option.n", \
              prog, "--" optstr);                                  \
      return 0;                                                    \
    }                                                              \
  }

  while (av++, --ac) {
    TRY(daemon,      Bool,   P_DAEMON);
    TRY(once,        Bool,   P_ONCE);
    TRY(naptime,     Time_t, P_RETRY_SLEEP);
    TRY(proxyconfig, Str,    P_PROXY_CONFIG_FILE);
    TRY(rundir,      Str,    P_RUNDIR);
    TRY(lockfile,    Str,    P_LOCKFILE_CRYPT);
    TRY(logfile,     Str,    P_LOGFILE);
    TRY(pidfile,     Str,    P_PIDFILE);
    TRY(randsrc,     Str,    P_RANDFILE);
    TRY(verbose,     Bool,   P_VERBOSE);
    TRY(srvhost,     Str,    P_KEYUPD_HOSTNAME);
    TRY(srvport,     UShort, P_KEYUPD_PORT);
    TRY(srvtimeout,  UInt,   P_KEYQRY_TIMEOUT);
  }

#undef TRY

  return 1;
}


static int confFromFile(struct frkeychg_config *cf, char *file)
{
  ZkConfig conf = 0;
  ZkError err = 0;
  const char *name, *value;
  int ret = 0;

  if ( ! zkErrorNew(&err)) {
    fprintf(stderr, "%s: confFromFile: can't create error object.\n", prog);
    return 0;
  }

  if ( ! (zkConfigNew(err, &conf) &&
          zkConfigLoad(err, conf, file))) {
    fprintf(stderr, "%s: confFromFile: can't load config file `%s'.\n",
            prog, file);
    goto done;
  }

#define TRY(field, type, parmstr)                                            \
  if (strcmp(parmstr, name) == 0 &&                                          \
      ! cvt ## type(value, &cf->field)) {                                    \
    fprintf(stderr, "%s: error setting value for config file option `%s'\n", \
            prog, parmstr);                                                  \
    goto done;                                                               \
  }

  while (zkConfigGetPair(err, conf, cfSection, &name, &value)) {
    TRY(daemon,      Bool,   P_DAEMON);
    TRY(once,        Bool,   P_ONCE);
    TRY(naptime,     Time_t, P_RETRY_SLEEP);
    TRY(proxyconfig, Str,    P_PROXY_CONFIG_FILE);
    TRY(rundir,      Str,    P_RUNDIR);
    TRY(lockfile,    Str,    P_LOCKFILE_CRYPT);
    TRY(logfile,     Str,    P_LOGFILE);
    TRY(pidfile,     Str,    P_PIDFILE);
    TRY(randsrc,     Str,    P_RANDFILE);
    TRY(verbose,     Bool,   P_VERBOSE);
  }

  while (zkConfigGetPair(err, conf, S_CLIENT_KEYUPD, &name, &value)) {
    TRY(srvhost,     Str,    P_KEYUPD_HOSTNAME);
    TRY(srvport,     UShort, P_KEYUPD_PORT);
    TRY(srvtimeout,  UInt,   P_KEYQRY_TIMEOUT);
  }

#undef TRY

  if (zkErrorCode(err) != ZKENOTFOUND) {
    fprintf(stderr, "%s: confFromFile: error reading section `%s' of file `%s'.\n",
            prog, cfSection, file);
    goto done;
  }

  ret = 1;

 done:

  if (conf) { zkConfigFree(0, &conf); }
  if (err) { zkErrorFree(&err); }

  return ret;
}


DLLExport int getConfig(struct frkeychg_config *conf, int ac, char **av)
{
  char *cfile = 0;
  int i;

#if 0
  if ( ! configDefault(conf)) {
    goto fail;
  }
#else
  configInit(conf);
#endif

  for (i = 1; i < ac; i++) {
    if (strcasecmp(av[i], "--config") == 0) {
      if (i < ac - 1) {
        cfile = av[i+1];
        break;
      } else {
        fprintf(stderr, "%s: --config option requires an argument.\n",
                prog);
        goto fail;
      }
    } else if (strcasecmp(av[i], "-h") == 0 ||
               strcasecmp(av[i], "-help") == 0 ||
               strcasecmp(av[i], "--help") == 0) {
      usage(1);
    }
  }

  if ( ! cfile) {
    fprintf(stderr, "%s: --config option is required.\n",
            prog);
    goto fail;
  }

  if ( ! (confFromFile(conf, cfile) &&
          confCmdLine(conf, ac, av))) {
    goto fail;
  }

  return 1;

 fail:

  return 0;
}
