/*
 *  $Id: kc_config.h,v 1.5 1999/04/01 05:02:00 bill Exp $
 */

#ifndef KC_CONFIG_H_
#define KC_CONFIG_H_


#include "zks.h"

#include <sys/types.h>
#include "frCrypt.h"
#include "frKeyQry.h"


/*
 *  The information we need from our config file.
 */
struct frkeychg_config {
  int daemon; /* 0: don't run as daemon; 1: run as daemon */
  int once; /* 0: process config file periodically; 1: once only */
  time_t naptime; /* time to sleep between checking keys */
  const char *proxyconfig; /* config file of those programs for whom we change keys */
  const char *rundir; /* cd to this directory when becoming a daemon */
  const char *lockfile; /* lock file for changing encrypted data */
  const char *logfile; /* the log file */
  const char *pidfile; /* process ID file */
  const char *randsrc; /* device from which to read random data */
  const char *srvhost; /* key update server host name */
  unsigned short srvport; /* port number at host */
  unsigned int srvtimeout;
  int verbose; /* 0: remain quiet; 1: print debugging information */

  /*
   *  The following values are not set from the configuration file, but they
   *  need to be passed around alot, and we just need a convenient location in
   *  which to store them.
   */

  FrCryptSym systemSymKey; /* unlocks all encrypted keys */
};


/*
 *  The information we need from the config file of the program whose key
 *  we're changing.
 */
struct proxy_config {
  const char *chgSigKeyDir; /* directory containing the key that signs key changes */
  const char *privKeyDir; /* directory containing cryption and signature directories */
  unsigned int keyLifetime; /* lifetime of key in seconds */
  unsigned int window;
  int force; /* force an update, ignoring the window */
  const char *pidFile; /* file containing the process ID to signal */
  int signal; /* the signal to send to the process */
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int getConfig(struct frkeychg_config *conf, int ac, char **av);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* KC_CONFIG_H_ */
