/*
 *  $Id: fr_dbresignmany.c,v 1.12 2000/03/24 00:40:26 alext Exp $
 *  
 *  Re-sign all entries in the database whose signing parent matches the owner
 *  of the supplied private signing key.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frCrypt.h"
#include "frFilenames.h"
#include "frKeyQry.h"
#include "frSec.h"
#include "zkError.h"
#include "zkFile.h"
#include "zkLog.h"
#include "zkDB.h"


#define LN_LEN_MAX 130


static const char *prog;
static const char *dflPubkeyDbCfgFileName = "/freedom/etc/db/pubkeydb.remote.conf";
static const char *dflLogFile = "/dev/tty";


static char *tail(char *path)
{
  char *p = strrchr(path, '/'); /*
   *  bug: Windows
   */
  return p ? p+1 : path;
}


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " --signerPrivFile <path>"
          " [-h | -help | --help]"
          " [--pubkeyDbCfgFile <string>]"
          " [--passInfo <path>]"
          " [--symAlg <string>]"
          "\n",
          prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  FILE *pinfoFp = 0;
  FrCryptSig sig = 0;
  FrCryptSym sym = 0;
  FrCryptSymAlg symAlg;
  FrSecPassInfo pinfo = 0;
  ZkDB kdb = 0;
  ZkError err = 0;
  ZkLog log;
  char *pass = 0;
  const char *algStr = "blowfish";
  const char *pubkeyDbCfgFileName = dflPubkeyDbCfgFileName;
  const char *logfile = dflLogFile;
  const char *passFile = "/dev/tty";
  const char *pinfoFile = FREEDOM_PASSINFO_FILE;
  const char *prompt = "Enter passphrase to decrypt signing key: ";
  const char *sigPrivFile = 0;
  int ok = 0;
  size_t pisz, plen;
  void *pimem = 0;


  printf("\nThis utility is broken, see bug 2761\n\n");
  exit(EXIT_FAILURE);


  prog = tail(av[0]);
  
  while (av++, --ac) {
    if (strcasecmp("--pubkeyDbCfgFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      pubkeyDbCfgFileName = av[0];
    } else if (strcasecmp("-h",        av[0]) == 0 ||
               strcasecmp("-help",     av[0]) == 0 ||
               strcasecmp("--help",    av[0]) == 0) {
      usage(1);
    } else if (strcasecmp(av[0], "--passInfo") == 0) {
      if ((av++, --ac) < 1) { usage(0); }
      pinfoFile = av[0];
    } else if (strcasecmp("--signerPrivFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      sigPrivFile = av[0];
    } else if (strcasecmp(av[0], "--symAlg") == 0) {
      if ((av++, --ac) < 1) {
        fprintf(stderr, "%s: %s requires an argument.\n", prog, "--symAlg");
        usage(0);
      }
      algStr = av[0];
    } else {
      break;
    }
  }
  
  if ( ! zkErrorNew(&err)) {
    fprintf(stderr, "%s: can't allocate error object -- exiting.\n", prog);
    goto done;
  }

  if ( ! (zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logfile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: can't set log object -- exiting.\n", prog);
    goto done;
  }

  if ( ! sigPrivFile) {
    fprintf(stderr, "%s: required argument %s missing.\n",
            prog, "--signerPrivFile");
    usage(0);
  }

  if ( ! frCryptSymNameToAlg(err, algStr, &symAlg)) {
    usage(0);
  }

  if ( ! frCryptInit(err, 0)) {
    fprintf(stderr, "%s: error initializing crypto library.\n", prog);
    goto done;
  }

  if ( ! (pinfoFp = fopen(pinfoFile, "rb"))) {
    zkLogWrite_va(log, "error %d opening `%s': %s",
                  errno, pinfoFile, strerror(errno));
    goto done;
  }

  if ( ! (frSecPassReadFromFile(err, passFile, prompt, 0, &pass, &plen) &&
          frSecPassInfoNew(err, &pinfo) &&
          zkFileReadFp(err, pinfoFp, 1024, &pimem, &pisz) &&
          frSecPassInfoSetFromBuf(err, pinfo, pimem, pisz) &&
          frCryptSymNew(err, &sym) &&
          frCryptSymSetAlg(err, sym, symAlg) &&
          frSecPassVerifySetSymKey(err, pass, sym, pinfo))) {
    goto done;
  }
  
  memset(pass, 0, plen); free(pass); pass = 0;

  if (!zkDB_Open(err, &kdb, pubkeyDbCfgFileName)) {
    goto done;
  }

  if ( ! (frSecPubKeyNewFromFile(err, &sig, sym, sigPrivFile) &&
          frCryptPubKeySetSharedParms(err, sig) &&
          frKeyUpdReSignChildren(err, kdb, sig))) {
    goto done;
  }

  ok = 1;

 done:

  if (pinfoFp) { fclose(pinfoFp); }
  if (sig) { frCryptPubKeyFree(err, &sig); }
  if (sym) { frCryptSymFree(err, &sym); }
  if (pinfo) { frSecPassInfoFree(err, &pinfo); }
  if (kdb) { zkDBClose(err, &kdb); }
  if (pass) { memset(pass, 0, plen); free(pass); }
  if (pimem) { free(pimem); }
  frCryptFinal(err, 0);
  if (err) { zkErrorFree(&err); }

  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
  return 0; /* to shut up insure */
}
