/*
 *  $Id: tpdatum.c,v 1.3 1999/05/04 22:07:33 bill Exp $
 *  
 *  Program to print a public key database datum.
 *  
 *  Mainly for debugging.
 *  
 *  bug: code taken from tpkdb.
 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "zkError.h"
#include "zkFile.h"
#include "zkLog.h"
#include "zkDB.h"
#include "frKeyQry.h"


static char *prog;

const size_t dflDataSz = 4 * 1024;
const size_t dflKeySz = 256;


static char *tail(char *path)
{
  char *p = strrchr(path, '/');
  return p ? p+1 : path;
}


static void usage(void)
{
  fprintf(stderr, "Usage: %s <datum-file>\n", prog);
  exit(EXIT_FAILURE);
}


int main(int ac, char **av)
{
  FILE *fp = 0;
  ZkError err = 0;
  ZkLog log = 0;
  const char *logfile = "/dev/tty";
  int ok = 0;
  size_t dsz;
  void *d = 0;

  prog = tail(av[0]);

  while (av++, --ac) {
    if (strcasecmp(av[0], "--logfile") == 0) {
      if (av++, --ac < 1) { usage(); }
      logfile = av[0];
    } else {
      break;
    }
  }

  if (ac != 1) {
    usage();
  }

  if ( ! zkErrorNew(&err)) {
    zkLogWrite(log, "can't create error object -- exiting.");
    fprintf(stderr, "%s: can't create error object -- exiting.\n", prog);
    goto done;
  }

  if ( ! (zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logfile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: can't create log object -- exiting.\n", prog);
    goto done;
  }
  
  if ( ! (fp = fopen(av[0], "rb"))) {
    zkErrorSet_va(err, "main", ZKERR, ZKMKEYUTILS, errno,
                  "can't open `%s': %s", av[0], strerror(errno));
    goto done;
  }
  
  if ( ! (zkFileReadFp(err, fp, 0, &d, &dsz) &&
          (printf("\nDatum:\n\n"), 1) &&
          fr_KeyQryDbDatumPrint(err, stdout, d, dsz))) {
    zkLogWrite(log, "can't print datum -- exiting.");
    goto done;
  }

  ok = 1;

 done:

  if (fp) { fclose(fp); }
  if (d) { free(d); }
  if (err) { zkErrorFree(&err); }

  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
