/*
 *  $Id: tsigpid.c,v 1.2 1999/03/12 05:11:16 bill Exp $
 *  
 *  This program helps test frkeychgd (and fr_killpid).
 *  
 *  Write our process ID to a file specified on the command line, then print a
 *  message everytime we receive a signal.
 */

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include "zkFile.h"


static char *prog;

int sigs[] = {
  SIGHUP,
  SIGTERM,
  SIGUSR1,
  SIGUSR2,
  0
};


static void sighand(int n)
{
  fprintf(stderr, "%s: caught signal %d\n", prog, n);
}


int main(int ac, char **av)
{
  FILE *fp, *pfp;
  const char *pidfile = 0;
  int closefile = 0;
  pid_t pid;
  size_t i;
  struct sigaction act;

  if ((prog = strrchr(av[0], '/'))) {
    prog += 1;
  } else {
    prog = av[0];
  }

  switch (ac) {
  case 2:
    pidfile = av[1];
    break;

  case 3:
    if (strcmp(av[1], "--close") == 0) {
      closefile = 1;
    } else {
      fprintf(stderr, "Usage: %s [--close] <pid-file>\n", prog);
      exit(EXIT_FAILURE);
    }
    pidfile = av[2];
    break;

  default:
    fprintf(stderr, "Usage: %s [--close] <pid-file>\n", prog);
    exit(EXIT_FAILURE);
  }
  
  act.sa_handler = sighand;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;

  for (i = 0; sigs[i]; i++) {
    if (sigaction(sigs[i], &act, 0) == -1) {
      fprintf(stderr, "%s: sigaction(): %s\n", prog, strerror(errno));
      exit(EXIT_FAILURE);
    }
  }

#if 1

  if ( ! zkFileWritePid(0, pidfile, &pfp)) {
    fprintf(stderr, "%s: error opening `%s': %s.\n",
            prog, pidfile, strerror(errno));
    exit(EXIT_FAILURE);
  }
  
  if (closefile) {
    fclose(pfp);
  }

#else
  
  pid = getpid();

  if ( ! (fp = fopen(av[1], "w"))) {
    fprintf(stderr, "%s: fopen(%s, \"w\"): %s\n",
            prog, av[1], strerror(errno));
    exit(EXIT_SUCCESS);
  }
  if (fprintf(fp, "%ld\n", (long)pid) == -1) {
    fprintf(stderr, "%s: error writing pid to `%s': %s\n",
            prog, av[1], strerror(errno));
    exit(EXIT_SUCCESS);
  }
  (void)fclose(fp);

#endif

  while (1) {
    sleep(100);
  }
  
  exit(EXIT_SUCCESS);
}
