#include <stdio.h>
#include <stdlib.h>
#include <zkError.h>
#include <fr_keyqry_db.h>
#include <frEnt.h>

static void printUsage(const char *progname) 
{
  fprintf(stderr, 
          "\n  Usage: %s <keyDbConfigFile> <entityType> <entityName> \n\n"
          "  This program is used to delete all keys associated with the given entity from the key database.\n\n"
          "  Where: \n" 
          "    <keyDbConfigFile>  Indicates the pub key DB config file.\n"
          "    <entityType>       Entity type (aip, niqs, nym, nymsrv, ...)\n"
          "    <entityName>       Name of the entity that should have its keys removed.\n\n",
          progname);
}

int main(int argc, const char *argv[])
{
  ZkError err = NULL;
  ZkDB keyDB = NULL;
  const char *dbConfigFile = NULL,
             *keyOwner = NULL;
  FrEntIdType entType;
  int count = 0;
  int ret = EXIT_FAILURE;
 
  if (argc != 4) {
    printUsage(argv[0]);
    goto done;
  }

  if (!zkErrorNew(&err)) {
    fprintf(stderr, "ERROR: Failed to allocate ZkError object...\n");
    goto done;
  }

  dbConfigFile = argv[1];
  entType = frEntTypeFromStr(argv[2]);
  keyOwner = argv[3];

  if (entType == FRENT_TYPE_UNUSED_) {
    fprintf(stderr, "ERROR: Invalid entity type (%s)\n", argv[2]);
    goto done;
  }

  if (!zkDB_Open(err, &keyDB, dbConfigFile) ||
      !fr_KeyQryDB_ExpungeKeys(err, keyDB, entType, keyOwner, &count)) {
      fprintf(stderr, "ERROR: %s\n", zkErrorDetail(err));
    goto done;
  }

  if (count == 0) {
    fprintf(stderr, "ERROR: Keys not found for entity %s\n", keyOwner);
    goto done;
  }
    
  fprintf(stdout, "%d key(s) successfully deleted for entity %s\n", 
          count, 
          keyOwner);
  ret = EXIT_SUCCESS;

 done:
  if (ret) { 
    zkErrorFree(&err);
  }

  if (keyDB) {
    zkDBClose(NULL, &keyDB);
  }

  return ret;
}
