/*
 *  $Id: fr_kqd_dbck.c,v 1.2 2000/02/29 16:35:59 alext Exp $
 *  
 *  Check the consistency of a public key database.
 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include "frCrypt.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkParms.h"


static const char *prog;


static void usage(int ok) {
  fprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--" P_LOGFILE " <path> ]"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


static const char *tail(const char *path)
{
  const char *p = strrchr(path, '/');
  return p ? p+1 : path;
}


int main(int ac, char **av)
{
  ZkError err = 0;
  ZkLog log;
  const char *logfile = "/dev/tty";
  int ok = 0;

  prog = tail(av[0]);

  while (av++, --ac) {
    if (strcasecmp(av[0], "-h")     == 0 ||
        strcasecmp(av[0], "-help")  == 0 ||
        strcasecmp(av[0], "--help") == 0) {
      usage(1);
    } else if (strcasecmp(av[0], "--" P_LOGFILE) == 0) {
      if (av++, --ac < 1) {
        fprintf(stderr, "%s: %s requires an argument.\n", prog, "--" P_LOGFILE);
        goto done;
      }
      logfile = av[0];
    } else {
      fprintf(stderr, "%s: invalid argument `%s' -- exiting.\n", prog, av[0]);
      usage(0);
    }
  }
  
  if ( ! zkErrorNew(&err)) {
    fprintf(stderr, "%s: can't create error object -- exiting.\n", prog);
    goto done;
  }

  if ( ! (zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logfile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: can't set log object -- exiting.\n", prog);
    goto done;
  }
  
  if ( ! frCryptInit(err, 0)) {
    zkLogError(log, err);
    zkLogWrite(log, "can't initialize crypto library -- exiting.");
    fprintf(stderr, "%s: can't initialize crypto library -- exiting.\n", prog);
    goto done;
  }

  zkErrorSet(err, "main", ZKINFO, 0 /* bug */, 0, "begin check");
  zkErrorSet(err, "main", ZKINFO, 0 /* bug */, 0, "end check");

  ok = 1;

 done:
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
