/* $Id: laip.h,v 1.22 2001/04/19 19:24:03 alainc Exp $ */
#ifndef LAIP_H_
#define LAIP_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "zks.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkList.h"
#include "zkConfig.h"
#include "frCrypt.h"
#include "frSec.h"
#include "frEnt.h"
#include "frKeyQry.h"
#include "frAips.h"
#include "frLocalAip.h"
#include "frNetInfo.h"
#include "frNetRoute.h"
#include "laipAciMapping.h"
#include "laipRoute.h"
#include "zkshimlib.h"
#include "zkFragCache.h"

#define FREEDOMKEYDIR ".freedom/keys"
#define SIGDIR "sig"
#define CRYPTDIR "crypt"

  typedef
  struct {
    const char *progname;       /* Program name */
    const char *logfile;        /* Log file */
    const char *configfile;     /* Configuration file */
    const char *homedir;        /* Home directory */
    char *keydir;               /* Key directory */
    const char *ppprompt;       /* Passphrase prompt */
    const char *ppfile;         /* Passphrase filename */
    const char *ppinfo;         /* Passphrase info filename */
    const char *pidfile;        /* PID file */
    
    ZkConfig config;            /* ZkConfig object, used for freeing */

    unsigned short cmdPort;     /* TCP port to listen to for commands */
    int cmdOpen;                /* Command port is connected / refuse access */
    int cmdFd;                  /* Command socket descriptor */
    int cmdListenFd;            /* Command socket descriptor */
    int listenq;                /* listen() queue value */
    const char *cmdWelcome;     /* Welcome message on the descriptor */
    const char *cmdRefused;     /* Welcome message on the descriptor */

  
    struct {
      struct {
        unsigned long version;/* Current data packet's protocol version */    
      } protocol;
    } data;

    struct {
      struct {
        unsigned long version;/* Current route create packet's protocol version */    
      } protocol;
    } route;

    unsigned long myIp;         /* For the shim packets */

    /* Key query */
    FrKeyQry kq;                /* Key query cache */
    FrKeyQryHost kh;            /* Key host IP/port and socket descriptor */
    uint32_t kqTime;            /* Timeout for key queries */
    int kqRetry;                /* Number of retries for key queries */

    FrSecKeyChain kc;           /* Key chain */

    FrLocalAipCtx la;           /* LocalAip object */
    uint32_t maxRoutes;         /* Max routes */

    LaipRoute routes;           /* Routing information application wise */
    LaipAciMap aciMap;          /* Mapping info */
    ZkACI currentAci;           /* Last ACI created */

    unsigned short anonPort;    /* Anonymous packet UDP port */
    int anonUdpFd;              /* Anonymous socket descriptor */
    int anonTcpFd;              /* Diffie Hellman socket descriptor */

    ZkShim shim;                /* Shim rules ... */
    int shimFd;                 /* Shim socket descriptor */
    const char *shimdev;        /* Shim device */

    FrNetInfo ni;               /* Network information */
    FrNetRoute nr;              /* Network routing object */
    const char *niqsHost;       /* NIQS hostname */
    unsigned short niqsPort;    /* NIQS port */
    int niqsTimeout;            /* NIQS timeout */
  
    /* Flags */
    int daemon;                 /* Fork off and listen in the background */
    int debug;                  /* Debug flag */    

    /* Events ... */
    ZkList eList;               /* Event List */

    /* My nyms... */
    ZkList nymList;             /* Nym information */
    char *currentNym;           /* Current used nym (NULL if anon) */

    int nhops;                  /* Number of hops to use for the new route */

    /* Fragmentation stuff */
    ZkFragCache fragCache;
    
    FILE *fptr;                 /* FILE * for PID file */
  } LaipCtx;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* LAIP_H_ */
