/* $Id: laipEvents.h,v 1.4 1999/05/21 20:04:13 kent Exp $ */
#ifndef LAIPEVENTS_H_
#define LAIPEVENTS_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "laip.h"
#include "zkList.h"

  typedef struct laip_event_shimrule_exp {
    struct sockaddr_in addr;
    ZkACI aci;
  } Laip_Event_Shimrule_Exp;

  typedef struct laip_event *LaipEvent;

  /* Values in () is the data that needs/is stored in the event list */
  typedef enum {
    LAIP_EV_NONE,               /* NULL event */
    LAIP_EV_DELETED,            /* Handled event, marked for deletion */
    LAIP_EV_ROUTE_CREATE_START, /* ROUTE CREATE needed/requested */
    LAIP_EV_DIFFIE_HELLMAN,     /* Diffie Hellman needed (first hop, aci,
                                   FrAipsDifHel) */
    LAIP_EV_KEYQUERY_REQUEST,   /* KEY requested (FrKeyQryQueryExp, aci) */
    LAIP_EV_ROUTE_CREATE_KEYQUERY, /* ROUTE CREATE keys requested (aci) */
    LAIP_EV_ROUTE_CREATE_PENDING, /* ROUTE CREATE sent (aci) */
    LAIP_EV_ROUTE_CREATE_ACKED, /* ROUTE CREATE ACKED (aci) */
    LAIP_EV_TOGGLE_DEBUG,       /* Toggle debug variable */
    LAIP_EV_ADD_SHIMRULE,       /* Add SHIM rule (shim rule, port only) */
    LAIP_EV_ADD_SHIMRULE2,      /* Add SHIM rule (shim rule, ip and port) */
    LAIP_EV_DEL_SHIMRULE,       /* Delete SHIM rule (shim rule, port only) */
    LAIP_EV_DEL_SHIMRULE2,      /* Delete SHIM rule (shim rule, ip and port) */
    LAIP_EV_SET_CURRENT_NYM,    /* Current nym name (nym name or NULL) */
    LAIP_EV_DESTROY_ROUTE,      /* Destroy ROUTE (aci) */
    LAIP_EV_DESTROY_ALLROUTES,  /* Destroy all ROUTES */
    LAIP_EV_QUIT,               /* Shutdown the program */
    LAIP_EV_LISTAIPS,           /* List all available AIPS */
    LAIP_EV_RR_CLEAR,           /* Clear my routing rules */
    LAIP_EV_RR_NHOPS,           /* Set the number of hops for this route */
    LAIP_EV_RR_FH,              /* Set first hop */
    LAIP_EV_RR_EXIT,            /* Set exit funnel */
    LAIP_EV_RR_HOP,             /* Set specific hop */
    LAIP_EV_RR_MUSTUSE,         /* Must use this hop */
    LAIP_EV_RR_DONTUSE,         /* Dont use this hop */
    LAIP_EV_MAX                 /* Place holder */
  } LaipEventType;

  extern int laipEventNew(ZkError, LaipEvent *);

  extern int laipEventFree(ZkError, LaipEvent *);

  extern int laipEventAdd(ZkError, ZkList, LaipEventType, void *);

  extern int laipEventGet(ZkError, ZkList, LaipEventType *, void **);
  
  extern int laipEventUpdate(ZkError, ZkList, LaipEventType, void *);

  extern int laipEventCleanList(ZkError, ZkList);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* LAIPEVENTS_H_ */
