/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRAIPS_H_
#define FRAIPS_H_

#ifndef _ZKERROR_H
#include "zkError.h"
#endif

#ifndef _ZKLOG_H
#include "zkLog.h"
#endif

#ifndef ZKNETINETIN_H_
#include "zkNetinetIn.h"
#endif

#ifndef ZKSYSTIME_H_
#include "zkSysTime.h"
#endif

#ifndef ZKSYSTYPES_H_
#include "zkSysTypes.h"
#endif

#ifndef _ZKQUEUE_H
#include "zkQueue.h"
#endif

#ifndef FR_AIPS_DH_H_
#include "fr_aips_dh.h"
#endif /* FR_AIPS_DH_H_ */

#ifndef _ZKINTTYPES_H
#include "zkIntTypes.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define FRAIPS_SYMKEY_INITIAL 0

  /* Socket stuff */
  extern int zkSocketsSetSockaddrIn(ZkError, const char *, const char *, 
                                    const char *, struct sockaddr_in *);
  extern int zkSocketsSetSockaddrIn2(ZkError, const char *, unsigned short,
                                     struct sockaddr_in *);
  extern int zkSocketsFdSetopt(ZkError, int, int);

  /* ACI stuff */
  /** 
   * ACI type.
   * 
   * ZKACIMAX must be set to the maximum available value.
   * This should be set based on the type of ZkACI.
   */ 
# define ZKACIMAX UINT16_MAX
  typedef unsigned int ZkACI;

  typedef struct T_ZkACIContext *ZkACIContext;
    
  int zkACIContextNewInternal(ZkError, ZkACIContext *, int);
  int zkACIContextNew(ZkError, ZkACIContext *);
  DLLExport int zkACIContextNewByAddress(ZkError, ZkACIContext *aci,
                               struct sockaddr_in, struct sockaddr_in);
  int zkACIContextNewByEntity(ZkError , ZkACIContext *,
                              FrEntIdExp, FrEntIdExp);

#if !defined(ZK_KERNEL_MAC_OS8)
  int zkACIGetAciInternal(ZkError, ZkACIContext, int, ZkACI *);  
  int zkACIGetAci(ZkError, ZkACIContext, struct sockaddr_in,
                  struct sockaddr_in, ZkACI *);
  DLLExport int zkACIGetAciOneRange(ZkError, ZkACIContext, ZkACI *);
#endif /* !defined(ZK_KERNEL_MAC_OS8) */
    
  DLLExport int zkACIReturnAci(ZkError, ZkACIContext, ZkACI);

  DLLExport int zkACIContextFree(ZkError, ZkACIContext *);


  /* Select stuff */
#if !defined(ZK_KERNEL)
  DLLExport extern int frSelectUpdateTimeout(ZkError, int, fd_set *, fd_set *, 
                                             fd_set *, struct timeval *);
#endif	/* !defined(ZK_KERNEL) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRAIPS_H */
