/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkString.h"
#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkSysTypes.h"
#include "zkNetinetIn.h"
#include "zkTypes.h"
#include "frAips.h"

static const int NBITS = 8;

static const int RANGELO = 0;
static const int RANGEHI = 1;
static const int RANGEBOTH = 2;


struct T_ZkACIbitmap {
 unsigned char *bitmap;
};

typedef struct T_ZkACIbitmap *ZkACIbitmap;

struct T_ZkACIContext {
  int nfreeLowAci;
  int nfreeHiAci;
  struct T_ZkACIbitmap *lowAciBitmap;
  struct T_ZkACIbitmap *hiAciBitmap;
};

int main(void) 
{
  ZkError err;
  ZkACIContext aciContext;

  int RangeMin=0;
  int RangeMax=65535;

  ZkACI aci;
  int i;
  uint8_t c[65536];

  if( !zkErrorNew(&err) ) {             /* ZkError Object */
    fprintf(stderr, "can't allocate the error handler.\n");
    abort();
  }

  if (!frCryptInit(err, 0)) {
    fprintf(stderr, "Unable to initialize crypto.\n");
    abort();
  }

  memset( c, 0, sizeof(c));

  //zkACIContextNew(err, &aciContext);
  zkACIContextNew(err, &aciContext);

  for (i = RangeMin; i <= RangeMax; i++)
  {
    //zkACIGetAci(err, aciContext, me, next, 
    //            &aci);
    if (!zkACIGetAciOneRange(err, aciContext, &aci)) {
      printf(" error getting aci i= %d\n",i);
      abort();
    }
    if (c[aci] == 1) printf(" aci already taken %d\n",aci);
    c[aci] = 1;
    printf(" aci %d \n", aci);
  }

  for (i = RangeMin; i <= RangeMax; i++)
  {
     if (c[i] == 0) printf(" aci not taken %d\n",i);
  }

   for (i = 0; i < (RangeMax-RangeMin+1)/NBITS; i++)
  {
    //printf(" bitmap i %x \n",aciContext->lowAciBitmap->bitmap[i]);
  }

  printf(" nfreeLowAci %d\n",aciContext->nfreeLowAci);

  printf(" nfreeHiAci %d\n",aciContext->nfreeHiAci);

  for (i = RangeMin; i <= RangeMax; i++)
  {
    if (!zkACIReturnAci(err, aciContext, i)) {
      printf(" error returning aci %d\n ", i);
      abort();
    }
  }

  for (i = 0; i < (RangeMax-RangeMin+1)/NBITS; i++)
  {
    //printf(" bitmap i %x \n",aciContext->lowAciBitmap->bitmap[i]);
  }

  printf(" nfreeLowAci %d\n",aciContext->nfreeLowAci);
  printf(" nfreeHiAci %d\n",aciContext->nfreeHiAci);

  zkACIContextFree(err, &aciContext);

  return (EXIT_SUCCESS);
}

