/*
 * $Id: frAmtpClient.h,v 1.13 2001/04/19 19:59:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */
#ifndef FRAMTPCLIENT_H_
#define FRAMTPCLIENT_H_

#ifndef ZKLIST_H_
#include "zkList.h"
#endif /* ZKLIST_H_ */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef ZKMIME_H_
#include "zkMIME.h"
#endif

#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "zkNetinetIn.h"


/*
 * alttext is an alternate message that will be sent with the encrypted
 * message in the case that the encrypted message is downloaded to an MUA
 * without going through freedom.  This alternate message will then be
 * displayed by a MIME complient MUA.  Only servers can specify this
 * alternate text.
 */

#define MSHOST_SZ 64
struct T_MaipServer {
  char maip[MSHOST_SZ + 1]; /* Name of the remote nym server host */
  unsigned short port;           /* Port on which the nymserver is running */
  struct sockaddr_in addr;       /* Alternate way of specifying the maip.
                                  * if the port is 0 or the maip string is
                                  * empty, this sockaddr_in will be used 
                                  * instead.
                                  */
  int timeout;                   /* Timeout for trying to communicate */
  size_t alttext_sz;             /* Size of the alternate text */
  char * alttext;                /* Alternate text for the message.  
                                  * This is for the servers only.  If you 
                                  * are a client application using these
                                  * functions, you MUST set this pointer to NULL
                                  * (if you set a string here, the server will
                                  * just ignore it anyways...)
                                  */
};

typedef struct T_MaipServer MaipServer;

#define MSRMESG_SZ 256
struct T_MaipServerResponse {
  char message[MSRMESG_SZ + 1]; /* The message string */
  int code;  /* The 3 digit code.  If 0, then no response came from server */
};

typedef struct T_MaipServerResponse MaipServerResponse;


DLLExport int amtpHeaderDump (
  ZkError ,
  void *  ,
  const unsigned char * ,
  int );

DLLExport int frAmtpSendMessageToNym (
  ZkError ,
  MaipServer * ,
  MaipServerResponse * ,
  ZkEmailHeader ,
  const char *  ,
  size_t ,
  const char * );

DLLExport int frAmtpSendMessageToReplyBlock (
  ZkError ,
  MaipServer * ,
  MaipServerResponse * ,
  ZkEmailHeader ,
  const char * ,
  size_t ,
  void * ,
  size_t  );

DLLExport int frAmtpSendMessageToInternetUsersOrMail2NewsGateway (
  ZkError err ,
  MaipServer * maip ,
  MaipServerResponse * msresp ,
  ZkEmailHeader header ,
  const char * mesg ,
  size_t mesg_sz ,
  ZkList iusers );


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRAMTPCLIENT_H_ */
