#include "btoken.h"
#include <stdlib.h>
#include <assert.h>

#if 1 
unsigned char *TOKENTYPE = "Type 1";
#else
unsigned char TOKENTYPE[1] = "\1";
#endif

int main(void)
{
  frBTParam p;
  frBTKey sec;
  frBTPubKey pub;

  frBTClient c;
  frBTServer s;

  unsigned char *f;
  int flen;

  unsigned char *a0, *c0, *r0, *token;
  int a0len, c0len, r0len, tokenlen;

  /* set public parameters */
  assert(frBTParamZk(0, &p));
  assert(BN_is_prime(p.p, 0, 0, 0, 0));
  assert(BN_is_prime(p.q, 0, 0, 0, 0));

  printf("Blind token protocol example using a %d bit prime\n\n", BN_num_bits(p.p));

  printf("Public parameters\np = ");
  BN_print_fp(stdout, p.p);
  printf("\nq = ");
  BN_print_fp(stdout, p.q);
  printf("\ng = ");
  BN_print_fp(stdout, p.g);
  printf("\n\n");

  /* generate key pair */
  assert(frBTKeyGen(0, &sec, &p));
  assert(frBTKeyPrecompute(0, &sec));
  assert(frBTPubKeyInit(0, &pub));
  assert(frBTPubKeySet(0, &pub, &sec));

  printf("Server key:\ny_1 = ");
  BN_print_fp(stdout, sec.y1);
  printf("\nx_0 = ");
  BN_print_fp(stdout, sec.x0);
  printf("\ng_1 = ");
  BN_print_fp(stdout, pub.g1);
  printf("\nh_0 = ");
  BN_print_fp(stdout, pub.h0);

  printf("\n\nToken type: %s\n\nPrecomputed values:\nd = ", TOKENTYPE);
  if (sizeof TOKENTYPE == 1 && TOKENTYPE[0] < NUMTOKENTYPES && sec.d[TOKENTYPE[0]-1])
    BN_print_fp(stdout, sec.d[TOKENTYPE[0]-1]);
  else
    printf("n/a");
  printf("\nh0h := h_0 * g1_exp_x1 = ");
  if (TOKENTYPE[0] < NUMTOKENTYPES && sec.h0h[TOKENTYPE[0]-1])
    BN_print_fp(stdout, sec.h0h[TOKENTYPE[0]-1]);
  else
    printf("n/a");
  printf("\n\n");

  /* C1 */
  assert(frBTClientInit(0, &c, &pub, "btoken v0.0"));

  /* if the TOKENTYPE is sent by the server, this is done as part of step C2 */
  assert(frBTClientPrecompute(0, &c, TOKENTYPE, sizeof TOKENTYPE));

  printf("C1:");
  printf("\na_2 = ");
  BN_print_fp(stdout, c.a2);
  printf("\na_3 = ");
  BN_print_fp(stdout, c.a3);
  printf("\nh   = ");
  BN_print_fp(stdout, c.h);
  printf("\nh0h  = ");
  BN_print_fp(stdout, c.h0h);
  printf("\na_1 = ");
  BN_print_fp(stdout, c.a1);
  printf("\n\n");


  /* S1 */
  assert(frBTServerInit(0, &s, &sec, "btoken v0.0"));
  assert(frBTServerPrecompute(0, &s, &a0, &a0len));

  printf("S1:\nw_0 = ");
  BN_print_fp(stdout, s.w0);
  printf("\na_0 = ");
  BN_print_fp(stdout, s.a0);
  printf("\n\n");

  /* M1: client sends serial number (AC) */

  /* server: if AC is not unspent, goto M4 */

  /* M2: server sends a0 and TOKENTYPE */

  /* C2 */
  assert(frBTClientChallenge(0, &c, a0, a0len, &c0, &c0len));
  free(a0);


  printf("C2:\na_prime_0= ");
  BN_print_fp(stdout, c.a0p);
  printf("C2:\nc_prime_0= ");
  BN_print_fp(stdout, c.c0p);
  printf("\nc_0 = ");
  BN_print_fp(stdout, c.c0);
  printf("\n\n");

  /* C3 (store to disk) */
  
  assert(frBTClientSave(0, &c, &f, &flen));

#if 1 /* test save/load */  
  assert(frBTClientLoad(0, &c, f));
#endif

  /* M3: client sends c0 */

  /* S2 */
  assert(frBTServerResponse(0, &s, c0, c0len, TOKENTYPE, sizeof TOKENTYPE, &r0, &r0len));
  free(c0);

  printf("S2:\nr_0 = ");
  BN_print_fp(stdout, s.r0);
  printf("\n\n");


  /* 
   * Must make sur to call frBTServerStore before we set 
   * entry in databse.
   */
 /* frBTServerStore(&s, mem, size); */ 

 /* store r_0 in database,
     check-and-set AC unspent -> spent-not-committed */
 

  /* YYYY */

  /* M4: server sends r0 */

  /* C4 */
  assert(frBTClientVerify(0, &c, r0, r0len));
  free(r0);
  printf("C3:\nverification passed\nr_prime_0 = ");
  BN_print_fp(stdout, c.r0p);
  printf("\n\n");

  /* M5: client sends commit */

  /* S3: (set AC to spent) */
  printf("\nS3: Server sets DBTS[AC] spend (not implemented here).\n\n");

  /* C5: */
  /* put back assert XXX */
  /*  assert(frBTClientToken(0, &c, TOKENTYPE, sizeof TOKENTYPE, &token, &tokenlen));
  {
    printf("Error in frBTClientToken \n\n");
  }*/

  frBTClientToken(0, &c, TOKENTYPE, sizeof TOKENTYPE, &token, &tokenlen);


  printf("S4:\n");

  /* version string valid?
     token type valid?
     server key currently valid?
     r0p,c0p,hp still unspent? => store r0p,c0p,hp,nil
  */


  assert(frBTServerToken(0, &s, token, tokenlen));


  free(token);

  /* store r0p,c0p,hp,nym */
  /* issue nym certificate */

  printf("Server verification ok\n");

  return 0;
}
