#include "btoken.h"
#include <stdlib.h>
#include <assert.h>
#include <time.h>

#define TOKENTYPE 1
#define ITERATIONS 100
#define TICKS 450000000.0

/************************/
/* Timing code          */
typedef union ctrval {
  struct {
    unsigned int l;  /* least significant word */
    unsigned int h;  /* most significant word */
  } w32;
  unsigned long long w64;
} ctrval;

/* Should take 11 cycles at user-level. */
static inline void
counter (union ctrval *v)
{
  __asm __volatile ("rdtsc"
                    : "=a" (v->w32.l), "=d" (v->w32.h) :);
}

ctrval start, end;

void start_timer( void ) {
  counter( &start );
}


/* 
 * return the difference from the time this 
 * funciton is called and the time when 
 * start_timer was called
 */
unsigned long long stop_timer( void /*const char* note*/ )
{
  unsigned long val;
  counter( &end );
  val = end.w64 - start.w64;
  /*  fprintf( stderr, "%s took %ld clock ticks\n", note, val ); */
  return val;
}			




int main(void)
{
  frBTParam p;
  frBTKey sec;
  frBTPubKey pub;

  frBTClient c;
  frBTServer s;

  FILE *f;

  unsigned char *a0, *c0, *r0, *token;
  int a0len, c0len, r0len, tokenlen;

  unsigned long long c_time = 0, c_best = -1, s_time = 0, s_best =-1, s_verify = 0, s_verify_best = -1;
  unsigned int i;

  /* ***** test some shit ****** */
   BN_CTX* ctx = BN_CTX_new();
  BIGNUM* xx = BN_new();
  BIGNUM* rr = BN_new();
  unsigned long long val = 0, best = -1;  
/* *************************** */


  if (! (TOKENTYPE > 0)) {printf("error\n");}

  /* set public parameters */
  if (! (frBTParamZk(0, &p))) {printf("error\n");}
  if (! (BN_is_prime(p.p, 0, 0, 0, 0))) {printf("error\n");}
  if (! (BN_is_prime(p.q, 0, 0, 0, 0))) {printf("error\n");}

  /* generate key pair */
  if (! (frBTKeyGen(0, &sec, &p))) {printf("error\n");}
  if (! (frBTKeyPrecompute(0, &sec))) {printf("error\n");}
  if (! (frBTPubKeyInit(0, &pub))) {printf("error\n");}
  if (! (frBTPubKeySet(0, &pub, &sec))) {printf("error\n");}

  /* ******** test time for 1 mod exp 160 bit  ***** */
  
  /* get a 160 bit exponent x */
  BN_rand(xx, 160, 1, 0); 
  /* BN_hex2bn( &xx, "F"); */

  for (i=0; i<ITERATIONS; i++) {
    counter( &start );
    BN_mod_exp( rr, pub.param->g, xx, pub.param->p, ctx );
    counter( &end );
    val = end.w64 - start.w64;
    if (val < best) { best = val;}
  }

  printf("=======================================\n");
 printf("%d mod exp (160 bit random exponent, small base, 1024 bit modulus) took: %lld ticks\n", 1, best );

  /* ***************************** */


  for (i = 0; i < ITERATIONS; i++) {

    s_time = 0;
    c_time = 0;
    /*
    start_timer();
    c_time += stop_timer();
    */
    
    /* C1 */
    start_timer(); 
    if (! (frBTClientInit(0, &c, &pub, "btoken v0.0"))) {printf("error\n");}
    if (! (frBTClientPrecompute(0, &c, TOKENTYPE))) {printf("error\n");}
    c_time += stop_timer();
 
    /*    fprintf(stderr, "C1: %lld\n", c_time); */

    /* S1 */
    start_timer();
    if (! (frBTServerInit(0, &s, &sec, "btoken v0.0"))) {printf("error\n");}
    if (! (frBTServerPrecompute(0, &s, &a0, &a0len))) {printf("error\n");}
    s_time += stop_timer();    

    /* M1: client sends serial number (AC) */
    
    /* server: if AC is not unspent, goto M4 */
    
    /* M2: server sends a0 */

    /* C2 */
    start_timer();
    if (! (frBTClientChallenge(0, &c, a0, a0len, &c0, &c0len))) {printf("error\n");}
    free(a0);
    c_time += stop_timer();

    /* C3 (store to disk) */
    start_timer();
    if (! ((f = fopen("tempfile", "wb")) != NULL)) {printf("error\n");}
    if (! (frBTClientSave(0, &c, f))) {printf("error\n");}
    fclose(f);
    c_time += stop_timer();


#if 0 /* test save/load */  
    if (! ((f = fopen("tempfile", "rb")) != NULL)) {printf("error\n");}
    if (! (frBTClientLoad(0, &c, f))) {printf("error\n");}
    fclose(f);
#endif

    /* M3: client sends c0 */
    
    /* S2 */
    start_timer();
    if (! (frBTServerResponse(0, &s, c0, c0len, TOKENTYPE, &r0, &r0len))) {printf("error\n");}
    free(c0);
    s_time += stop_timer();
    
    /* 
     * Must make sur to call frBTServerStore before we set 
     * entry in databse.
     */
    /* frBTServerStore(&s, mem, size); */ 
    
    /* store r_0 in database,
       check-and-set AC unspent -> spent-not-committed */
  
    
    /* YYYY */
    
    /* M4: server sends r0 */
    

    /* C4 */
    start_timer();
    if (! (frBTClientVerify(0, &c, r0, r0len))) {printf("error\n");}
    free(r0);
    c_time += stop_timer();

    /* M5: client sends commit */
    
    /* S3: (set AC to spent) */
    

    /* C5: */
    start_timer();
    /* put back if (!  XXX */
    /*  if (! (frBTClientToken(0, &c, TOKENTYPE, &token, &tokenlen)))
	{
	printf("Error in frBTClientToken \n\n");
	}*/
    
    frBTClientToken(0, &c, TOKENTYPE, &token, &tokenlen);
    c_time += stop_timer();


    /* S4 */
    start_timer();
    /* version string valid?
       token type valid?
       server key currently valid?
       r0p,c0p,hp still unspent? => store r0p,c0p,hp,nil
    */
    if (! (frBTServerToken(0, &s, token, tokenlen))) {printf("error\n");}
    
    free(token);
    
    /* store r0p,c0p,hp,nym */
    /* issue nym certificate */
    s_verify = stop_timer();
    s_time += s_verify;

    /*    s_time += stop_timer();    */
 
    if (s_verify < s_verify_best) {
      s_verify_best = s_verify;
    }   

    if (s_time < s_best) {
      s_best = s_time;
    }
    if (c_time < c_best) {
      c_best = c_time;
    }

  } /* endif iteration */



  /* print out timings */
  printf("Client: Minimum clock ticks (out of %d tries) for 1 run of the protocol: %lld \n", ITERATIONS,
          c_best);
  printf("Server: Minimum clock ticks (out of %d tries) for 1 run of the protocol: %lld \n", ITERATIONS,
          s_best);
  printf("Note: Server time inludes time for verification in spend protocol.\n");
  printf("Server Verify: clock ticks for 1 run of the protocol: %lld \n", s_verify_best);
  printf("=========================================\n");

  return 0;
}
