/*
 *  $Id: fr_pk.h,v 1.21 2001/04/19 19:59:57 alainc Exp $
 */

#ifndef FRPK_H_
#define FRPK_H_

/*
 *
 *
 *                       General public key routines.
 *
 *
 */

#include "zks.h"

#include "zkStdio.h"
#include "frEnt.h"
#include "zkError.h"
#include "zkIntTypes.h"
#include "zkClock.h"
#include "fr_gencrypt.h"
#include "openssl/bn.h"


typedef struct fr_crypt_pubkey *FrCryptPubKey;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern FRCRYPT_API int frCryptPubKeyNew(ZkError err, FrCryptPubKey *key);
#if !defined(ZK_KERNEL)
  extern FRCRYPT_API int frCryptPubKeyNewFromFp(ZkError err, FrCryptPubKey *key, FILE *fp);
  extern FRCRYPT_API int frCryptPubKeyNewFromFile(ZkError err, FrCryptPubKey *key, const char *file);
#endif
  extern FRCRYPT_API int frCryptPubKeyDup(ZkError err, FrCryptPubKey *copy, FrCryptPubKey key);
  extern FRCRYPT_API int frCryptPubKeyFree(ZkError err, FrCryptPubKey *key);

  extern FRCRYPT_API int frCryptPubKeyGetVersion(ZkError err, FrCryptPubKey key, unsigned int *vers);
  extern FRCRYPT_API int frCryptPubKeySetVersion(ZkError err, FrCryptPubKey key, unsigned int vers);
  extern FRCRYPT_API unsigned int frCryptPubKeyMaxVersion(void);
  extern FRCRYPT_API unsigned int frCryptPubKeyMinVersion(void);
  extern FRCRYPT_API int frCryptPubKeyNextVersion(ZkError err, FrCryptPubKey key, unsigned int *vers);

  extern FRCRYPT_API int frCryptPubKeyGetType(ZkError err, FrCryptPubKey key, FrPubKeyType *type);
  extern FRCRYPT_API int frCryptPubKeySetType(ZkError err, FrCryptPubKey key, FrPubKeyType type);

  extern FRCRYPT_API int frCryptPubKeyGetEntity(ZkError err, FrCryptPubKey key, FrEntIdExp *ent);
  extern FRCRYPT_API int frCryptPubKeySetEntity(ZkError err, FrCryptPubKey key, FrEntIdExp *ent);

  extern FRCRYPT_API int frCryptPubKeyGetSize(ZkError err, FrCryptPubKey key, unsigned int *nbits);
  extern FRCRYPT_API int frCryptPubKeySetSize(ZkError err, FrCryptPubKey key, unsigned int nbits);

  extern FRCRYPT_API int frCryptPubKeyGetExpiration(ZkError err, FrCryptPubKey key, ZkClock *exp);
  extern FRCRYPT_API int frCryptPubKeySetExpiration(ZkError err, FrCryptPubKey key, ZkClock exp);

  extern FRCRYPT_API int frCryptPubKeyGetParm(ZkError err, FrCryptPubKey key, size_t n, BIGNUM **bn, int *willFree, int *willSave);
  extern FRCRYPT_API int frCryptPubKeySetParm(ZkError err, FrCryptPubKey key, size_t n, BIGNUM *bn, int freeIt, int saveIt);

  extern FRCRYPT_API int frCryptPubKeyGetFreeMask(ZkError err, FrCryptPubKey key, uint8_t *mask);
  extern FRCRYPT_API int frCryptPubKeySetFreeMask(ZkError err, FrCryptPubKey key, uint8_t mask);
  extern FRCRYPT_API int frCryptPubKeyGetSaveMask(ZkError err, FrCryptPubKey key, uint8_t *mask);
  extern FRCRYPT_API int frCryptPubKeySetSaveMask(ZkError err, FrCryptPubKey key, uint8_t mask);

  extern FRCRYPT_API int frCryptPubKeyStorageSize(ZkError err, FrCryptPubKey key, size_t *n);

  extern FRCRYPT_API int frCryptPubKeyReadFromBuf(ZkError err, FrCryptPubKey key, char *b, size_t *bsz);
  extern FRCRYPT_API int frCryptPubKeyWriteToBuf(ZkError err, FrCryptPubKey key, char *b, size_t *bsz);
  extern FRCRYPT_API int frCryptPubKeyWriteToMem(ZkError err, FrCryptPubKey key, void **mem, size_t *msz);

  extern FRCRYPT_API int frCryptPubKeyBufGetVersion(ZkError err, void *kbuf, size_t ksz, unsigned int *vers);
  extern FRCRYPT_API int frCryptPubKeyBufGetType(ZkError err, void *kbuf, size_t ksz, FrPubKeyType *type);
  extern FRCRYPT_API int frCryptPubKeyBufGetExpiration(ZkError err, void *kbuf, size_t ksz, ZkClock *exp);
  extern FRCRYPT_API int frCryptPubKeyBufGetEntity(ZkError err, void *kbuf, size_t ksz, FrEntIdExp *ent);
  
  extern FRCRYPT_API int frCryptPubKeyEqual(ZkError err, FrCryptPubKey k0, FrCryptPubKey k1, int *equal);
  extern FRCRYPT_API int frCryptPubKeyMaskEqual(ZkError err, FrCryptPubKey k0, FrCryptPubKey k1, uint8_t mask, int *equal);


  extern FRCRYPT_API const char *frCryptPubKeyTypeStr(FrPubKeyType type);
  extern FRCRYPT_API const char *frCryptPubKeyAlgStr(FrCryptPubAlg alg);
#if !defined(ZK_KERNEL)
  extern FRCRYPT_API int frCryptPubKeyPrint(ZkError err, FrCryptPubKey key, FILE *fp);
#endif
  extern FRCRYPT_API char *frCryptPubKeyBufIdSnprint(ZkError err, void *k, size_t ksz, char *s, size_t ssz);
#if !defined(ZK_KERNEL)
  extern FRCRYPT_API int frCryptPubKeyShortPrint(ZkError err, FrCryptPubKey key, int indent, FILE *fp);
  extern FRCRYPT_API int frCryptPubKeyToCString(ZkError err, FrCryptPubKey key, char *aname, uint8_t mask, char **str);
  extern FRCRYPT_API int frCryptPubKeyPrintAsC(ZkError err, FrCryptPubKey key, uint8_t mask, char *name, FILE *fp);
#endif

#if !defined(ZK_KERNEL)
  extern FRCRYPT_API int frCryptPubKeySetSharedParms(ZkError err, FrCryptPubKey key);
  extern FRCRYPT_API int frCryptPubKeyPublicValidate(ZkError err, FrCryptPubKey key);
  extern FRCRYPT_API int frCryptPubKeyBufPublicValidate(ZkError err, void *buf, size_t bsz);
#endif
  extern FRCRYPT_API int frCryptPubKeySetBlinding(ZkError err, FrCryptPubKey key, int boolean);

  extern FRCRYPT_API int frCryptPubKeySetCanCreateRoutes(ZkError err, FrCryptPubKey key, int canCreateRoutes);
  extern FRCRYPT_API int frCryptPubKeyGetCanCreateRoutes(ZkError err, FrCryptPubKey key, int * canCreateRoutes);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRPK_H_ */
