/*
 *  $Id: fr_rand.h,v 1.13 2001/04/19 19:59:57 alainc Exp $
 */

#ifndef FR_RAND_H_
#define FR_RAND_H_

#include "openssl/crypto.h"
#ifndef HAVE_YARROW
# define HAVE_YARROW 1
#endif
#if OPENSSL_VERSION_NUMBER < 0x0090581fL
# undef HAVE_YARROW
#endif

#ifdef linux
# undef HAVE_YARROW
# define USE_DEVRANDOM
#endif

/*
 *
 *
 *                           Randomness routines.
 *
 *
 */

#include "zks.h"
#include "zkError.h"
#include "fr_gencrypt.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
  
#if !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY)
  FRCRYPT_API extern int frCryptAddEntropy(ZkError err, void *data, size_t dsz);
#endif	/* !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY) */
#if !defined(ZK_KERNEL_MAC_OS8)
  FRCRYPT_API int frCryptNewSource(ZkError err, unsigned *source);
  FRCRYPT_API int frCryptAddEntropyFromSource(ZkError err, int source_id,
					    void *data, size_t dsz,
					    unsigned int bits);
  /* *enough is set to 1 if the PRNG has enough entropy */
  FRCRYPT_API int frCryptGetEntropyCount(ZkError err, int *enough,
                                       int *num_sources, unsigned *source_id,
                                       size_t *entropy_bits,
                                       size_t *entropy_max);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */
#if !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY)
  FRCRYPT_API extern int frCryptSetRandomData(ZkError err, void *data, size_t sz);
#endif	/* !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY) */
#if !defined(ZK_KERNEL_MAC_OS8)
  FRCRYPT_API extern int frCryptRandomUInt32InRange(ZkError err, 
					         uint32_t min, uint32_t max,
					         uint32_t * rn);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */
  FRCRYPT_API void frRand_Debug(void); /* to be called in test programs only */
  
  /* caller asserts that /dev/urandom has entropy and it is not
     necessary to read from /dev/random in blocking mode */
  FRCRYPT_API void frRand_DontBlock(void);

  /*
   * Internal Functions */
  
  extern int fr_CryptRandInit(ZkError err, const char *entropyfile);
  extern int fr_CryptRandFinal(ZkError err);
  
#if defined(macintosh)

  extern pascal short frCryptLibInit(const struct CFragSystem7InitBlock *initBlock);
  extern pascal short frCryptAppInit(const struct CFragSystem7InitBlock *initBlock);
  extern pascal short ZksCryptAppInit(const struct CFragSystem7InitBlock *initBlock);
  extern pascal void  frCryptLibTerm(void);
  extern pascal void  frCryptAppTerm(void);
  extern pascal void  ZksCryptAppTerm(void);
  
#endif	/* defined(macintosh) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_RAND_H_ */
