/*
 *  $Id: fr_sym.h,v 1.7 2001/04/19 19:59:57 alainc Exp $
 */

#ifndef FR_SYM_H_
#define FR_SYM_H_

/*
 *  
 *  
 *                           Symmetric Cryption.
 *  
 *  
 */
  
#include "zks.h"
#include "zkError.h"
#include "zkvheap.h"


/*
 *  Symmetric keys should be generated with this many bytes of seed material.
 */
#define FRCRYPT_SYMKEY_SEED_SIZE 24


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  FRCRYPT_API extern int frCryptSymNameToAlg(ZkError err, const char *name, FrCryptSymAlg *alg);

  FRCRYPT_API extern int frCryptSymDecrypt(ZkError err, FrCryptSym sym, void *ivec, void *data, size_t len);
  FRCRYPT_API extern int frCryptSymDecryptToBuf(ZkError err, FrCryptSym sym, void *ivec, void *data, void *output, size_t len);
  FRCRYPT_API extern int frCryptSymEncrypt(ZkError err, FrCryptSym sym, void *ivec, void *data, size_t len);
  FRCRYPT_API extern int frCryptSymEncryptToBuf(ZkError err, FrCryptSym sym, void *ivec, void *data, void *output, size_t len);
  FRCRYPT_API extern int frCryptSymFree(ZkError err, FrCryptSym *sym);
  FRCRYPT_API extern int frCryptSymGetAlg(ZkError err, FrCryptSym sym, FrCryptSymAlg *alg);
  FRCRYPT_API extern int frCryptSymKeyFromData(ZkError err, FrCryptSym sym, void *data, size_t len);
  FRCRYPT_API extern int frCryptSymLoad(ZkError err, FrCryptSym sym, void *data, size_t len);
  FRCRYPT_API extern int frCryptSymNew(ZkError err, FrCryptSym *sym);

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
  FRCRYPT_API extern int frCryptSymVHeapNew(ZkError err, FrCryptSym *sym, ZkVHeap vheap);
  FRCRYPT_API extern int frCryptSymVHeapFree(ZkError err, FrCryptSym *sym, ZkVHeap vheap);
#endif

  FRCRYPT_API extern int frCryptSymPortableSize(ZkError err, FrCryptSym sym, size_t *sz);
  FRCRYPT_API extern int frCryptSymCopy(ZkError err, FrCryptSym key, FrCryptSym *copy);
  FRCRYPT_API extern int frCryptSymGetKey(ZkError err, FrCryptSym sym, void *kbuf, size_t kbsz);
  FRCRYPT_API extern int frCryptSymGetKeySize(ZkError err, FrCryptSym sym, size_t *sz);
  FRCRYPT_API extern int frCryptSymSetKeySize(ZkError err, FrCryptSym sym, size_t sz);
#if !defined(ZK_KERNEL_MAC_OS8)
  FRCRYPT_API extern int frCryptSymRandomKey(ZkError err, FrCryptSym sym);
#endif
  FRCRYPT_API extern int frCryptSymSetAlg(ZkError err, FrCryptSym sym, FrCryptSymAlg alg);
  FRCRYPT_API extern int frCryptSymSizeMultiple(ZkError err, FrCryptSym sym, size_t *sz);
  FRCRYPT_API extern int frCryptSymStore(ZkError err, FrCryptSym sym, void *data, size_t *len);
  FRCRYPT_API extern int frCryptSymDebugDump(ZkError err, FrCryptSym sym, char *buffer, uint16_t buffersz);

  FRCRYPT_API int frCryptSymGetSerialiseSize(ZkError err, const FrCryptSym sym, size_t* size);
  FRCRYPT_API int frCryptSymSerialise(ZkError err, const FrCryptSym sym, void* buff, size_t buffsize, size_t* offset);
  FRCRYPT_API int frCryptSymDeserialise(ZkError err, FrCryptSym sym, const void* buff, size_t buffsize, size_t* offset);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_SYM_H_ */
