/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRDESX_H_
#define FRDESX_H_

#include "zks.h"
#include "openssl/des.h"
#include "fr_gencrypt.h"
#include "fr_sym.h"
#include "zkError.h"


/* bug: this code is currently just a placeholder */


/** The default size, in bytes, of the Desx key. */
#define DESX_DFL_KEY_SIZE 8 /* bug? 16 */


/*
 *  Once the keys are generated, only the key schedules are needed for
 *  cryption, but we keep the original keys around in case we need to load or
 *  store them.
 */
struct desx {
  des_cblock key;
  des_cblock inw;
  des_cblock outw;
  des_key_schedule ks;
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if !defined(ZK_KERNEL_MAC_OS8)
  int fr_des_random_key(ZkError err, des_cblock *ret);
#endif
  extern int fr_DesxInit(ZkError err, FrCryptSym sym);
  extern int fr_DesxGetKey(ZkError err, FrCryptSym sym, void *kbuf, size_t kbsz);
  extern int fr_DesxGetKeySize(ZkError err, FrCryptSym sym, size_t *sz);
  extern int fr_DesxSetKeySize(ZkError err, FrCryptSym sym, size_t sz);
#if !defined(ZK_KERNEL_MAC_OS8)
  extern int fr_DesxRandomKey(ZkError err, FrCryptSym sym);
#endif
  extern int fr_DesxDecrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_DesxDecryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);
  extern int fr_DesxEncrypt(ZkError err, FrCryptSym sym, void *iv, void *data, size_t sz);
  extern int fr_DesxEncryptToBuf(ZkError err, FrCryptSym sym, void *iv, void *data, void *output, size_t sz);

  extern int fr_DesxLoad(ZkError err, FrCryptSym sym, void *data, size_t sz);
  extern int fr_DesxStore(ZkError err, FrCryptSym sym, void *data, size_t *sz);

  extern int fr_DesxGetSerialiseSize(ZkError err, const FrCryptSym sym, size_t* size);
  extern int fr_DesxSerialise(ZkError err, const FrCryptSym sym, void* buff, size_t buffsize, size_t* offset);
  extern int fr_DesxDeserialise(ZkError err, FrCryptSym sym, const void* buff, size_t buffsize, size_t* offset);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRDESX_H_ */
