/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef SIG_H_
#define SIG_H_


#include "fr_pk.h"


/*
 *  The BIGNUM pointers are for naming convenience only.  The actually point
 *  to bignums that are the "property" of the FrCryptPubKey object.  They will
 *  be freed with that object.
 */

struct fr_crypt_sig {
#if 1
  FrCryptPubKey pk;
  BIGNUM *p; /* large prime number */
  BIGNUM *q; /* a 160 bit prime factor of p-1 */
  BIGNUM *g; /* a `generator' */
  BIGNUM *privKey; /* private key */
  BIGNUM *pubKey; /* public key */
#else
  uint8_t vers; /* key version */
  FrCryptSigKeySize keySize;
  BIGNUM *p; /* large prime number */
  BIGNUM *q; /* a 160 bit prime factor of p-1 */
  BIGNUM *g; /* a `generator' */
  BIGNUM *privKey; /* private key */
  BIGNUM *pubKey; /* public key */
#endif
};


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern int fr_CryptSigInit(ZkError err);
  extern int fr_CryptSigFinal(ZkError err);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* SIG_H_ */
