/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Determine an upper bound on the amount of space used by the ASN.1
 *  representation of various objects.
 *  
 *  Code taken from SSLeay.
 */


#include "zkStdio.h"
#include <assert.h>
#include "zkStdlib.h"
#include "openssl/bn.h"
#include "openssl/dsa.h"
#include "openssl/asn1.h"
#include "zkError.h"
#include "frCrypt.h"



static int my_DSA_size(void)
	{
	int ret,i;
	ASN1_INTEGER bs;
	unsigned char buf[4];

#if 1
  i = 160; /* assume a maximum of 160 bits. */
#else
	i=BN_num_bits(r->q);
#endif
	bs.length=(i+7)/8;
	bs.data=buf;
	bs.type=V_ASN1_INTEGER;
	/* If the top bit is set the asn1 encoding is 1 larger. */
	buf[0]=0xff;	

	i=i2d_ASN1_INTEGER(&bs,NULL);
	i+=i; /* r and s */
	ret=ASN1_object_size(1,i,V_ASN1_SEQUENCE);
	return(ret);
	}


int main(void)
{
  ZkError err;
	
  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  
  zkPrintf("DSA signature takes %d bytes.\n", my_DSA_size());

  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));
  
  return (EXIT_SUCCESS);
}
