/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Convert a BIGNUM to the corresponding C static declaration.
 */

#include "zkStdio.h"
#include "zkStdlib.h"
#include "frCrypt.h"
#include "zkError.h"
#include "elgparms.h"
#include "dsaparms.h"


#define A(x) #x, &x


static void prtBn(const char *bnName, BIGNUM *bn)
{
  int i;
  
  zkPrintf("static BN_ULONG %s_digits[] = {", bnName);
  for (i = 0; i < bn->top; i++) {
    if (i % 6 == 0) { zkPrintf("\n  "); }
    zkPrintf("%10luU%s", bn->d[i], i == bn->top - 1 ? "\n" : ", ");
  }
  zkPrintf("};\n\n");
  
  zkPrintf("BIGNUM %s = {\n", bnName);
  zkPrintf("  %s_digits,\n", bnName);
  zkPrintf("  %d,\n", bn->top);
  zkPrintf("  sizeof %s_digits / sizeof %s_digits[0],\n", bnName, bnName);
  zkPrintf("  0\n");
  zkPrintf("};\n\n");
}


int main(void)
{
#if 1 
  /* ElGamal */

  prtBn(A(frBn_ElG_p_1024));
  prtBn(A(frBn_ElG_p_1536));
  prtBn(A(frBn_ElG_p_2048));
  prtBn(A(frBn_ElG_p_3072));
  prtBn(A(frBn_ElG_p_4096));
  prtBn(A(frBn_ElG_g));

#else

  /* DSA */

  prtBn(A(frBn_Dsa_p_1024));
  prtBn(A(frBn_Dsa_q_1024));
  prtBn(A(frBn_Dsa_g_1024));
  
  prtBn(A(frBn_Dsa_p_1536));
  prtBn(A(frBn_Dsa_q_1536));
  prtBn(A(frBn_Dsa_g_1536));
  
  prtBn(A(frBn_Dsa_p_2048));
  prtBn(A(frBn_Dsa_q_2048));
  prtBn(A(frBn_Dsa_g_2048));
  
  prtBn(A(frBn_Dsa_p_4096));
  prtBn(A(frBn_Dsa_q_4096));
  prtBn(A(frBn_Dsa_g_4096));

#endif

  return (EXIT_SUCCESS);
}
