/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Storage and retieval of DSA public/private keys.
 */

#include "zkSysTime.h"
#include <assert.h>

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "openssl/evp.h"
#include "zkShell.h"
#include "frCrypt.h"



#define BSZ 128


static void pbytes(FILE *fp, unsigned char *buf, size_t sz)
{
  size_t i;
  
  for (i = 0; i < sz; i++) {
    zkFprintf(fp, " %03u", (unsigned int)buf[i]);
    if ((i + 1) % 16 == 0) fputs("\n", fp);
  }
  fputs("\n", fp);
}


int main(int ac, char **av)
{
  DSA *dsa = 0;
  unsigned char privBuf[512], pubBuf[512];
  int nbits = 1024;
  int sz = 0;
  size_t i;
  unsigned char *p;
  unsigned char seed[SHA_DIGEST_LENGTH];
  const char* progname;
  ZkError err;
  
  progname = zkShellGetProgramName(&ac, &av);

  zkPrintf("dsaparms...\n");

  if (ac == 2) {
    if (sscanf(av[1], "%u", &nbits) != 1) {
      zkFprintf(stderr, "%s: bad args.\n", progname);
      exit(EXIT_FAILURE);
    }
  }

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  
  /* Initialize the seed. */

  zkPrintf("Initializing the seed...\n");

  zkPrintf("initializing the seed...\n");
  
  for (i = 0; i < sizeof seed; i++) { seed[i] = sizeof seed - 1 - i; }

  /* Create a signing key pair. */
    
  zkPrintf("Creating a signing key pair...\n");

  assert(dsa = DSA_generate_parameters(nbits, seed, sizeof seed, 0, 0, 0, 0));
  assert(DSA_generate_key(dsa));

  zkPrintf("Generated a %d bit public/private DSA key pair.\n", nbits);

  /* Store the private key in a buffer. */

  p = privBuf;
  sz = i2d_DSAPrivateKey(dsa, &p);
  zkPrintf("Private key required %d bytes of storage (p changed by %lu).\n",
         sz, (unsigned long)(p - &privBuf[0]));
  pbytes(stdout, privBuf, sz);

  /* Store the public key in a buffer. */

  p = pubBuf;
  sz = i2d_DSAPublicKey(dsa, &p);
  zkPrintf("Public key required %d bytes of storage (p changed by %lu).\n",
         sz, (unsigned long)(p - &pubBuf[0]));
  pbytes(stdout, pubBuf, sz);

  /* Clean up. */

  DSA_free(dsa);
  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));

  return (EXIT_SUCCESS);
}
