/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Test the ElGamal shared secret agreement code.
 */

#include <assert.h>
#include <limits.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkString.h"


int main(void)
{
  FrCryptCrypt key;
  ZkError err;
  char ps[FRCRYPT_SECRET_SIZE];
  char privsec[FRCRYPT_SECRET_SIZE], pubsec[FRCRYPT_SECRET_SIZE];
  size_t pssz = sizeof ps;
  size_t privsz = sizeof privsec, pubsz = sizeof pubsec;

  zkPrintf("Starting elg...\n");

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  zkRand_Debug();
  assert(frCryptCryptNew(err, &key));
  
  assert(frCryptCryptSetKeySize(err, key, FRCRYPT_CRYPTKEYSIZE_1024));
  assert(frCryptCryptSetRandomKeys(err, key));

#if 1
   assert(frCryptPubKeySetBlinding(err, key, 0));
#endif


  assert(frCryptCryptSetPreSecret(err, key, ps, pssz, pubsec, pubsz));
  assert(frCryptCryptSecretFromPrivKey(err, key, ps, pssz, privsec, privsz));

  assert(privsz == pubsz);
  assert(memcmp(privsec, pubsec, pubsz) == 0);

  assert(frCryptCryptFree(err, &key));
  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));

  zkPrintf(".\n");

  return (EXIT_SUCCESS);
}
