/*
 *  $Id: elgstld.c,v 1.7 2001/04/19 20:00:02 alainc Exp $
 *
 *  Test the storing and loading of cryption keys.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkTime.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkString.h"


static void checkStLd(ZkError err, FrCryptCryptKeySize cryptSz, int mask, int stOkay)
{
  FrCryptCrypt crypt0, crypt1;
  char parms0[2048], parms1[2048];
  size_t psz0, psz1;

  zkRand_Debug();
  assert(frCryptCryptNew(err, &crypt0));
  assert(frCryptCryptSetKeySize(err, crypt0, cryptSz));
  assert(frCryptCryptSetRandomKeys(err, crypt0));
  
  /* Store all parameters. */

  psz0 = sizeof parms0;
  if (stOkay) {
    assert(frCryptCryptStore(err, crypt0, mask, parms0, &psz0));
  } else {
    /* Bad parameters: store should fail */
    assert( ! frCryptCryptStore(err, crypt0, mask, parms0, &psz0));
    return;
  }
  zkPrintf("Stored complete key in %lu bytes.\n", (unsigned long)psz0);

  memcpy(parms1, parms0, psz0);
  psz1 = sizeof parms1;
  assert(frCryptCryptNewFromData(err, &crypt1, parms1, &psz1));
  assert(psz1 == psz0);

  /* Verify that the cryption objects are the same. */

  {
    int equal = 0;
    
    assert(frCryptCryptEqual(err, crypt0, crypt1, &equal));
    assert(equal);
  }

  /* Clean up. */
  
  assert(frCryptCryptFree(err, &crypt0));
  assert(frCryptCryptFree(err, &crypt1));
}



int main(void)
{
  FrCryptCryptKeySize ks[] = {
    FRCRYPT_CRYPTKEYSIZE_1024,
    FRCRYPT_CRYPTKEYSIZE_1536,
    FRCRYPT_CRYPTKEYSIZE_2048,
    FRCRYPT_CRYPTKEYSIZE_3072,
    FRCRYPT_CRYPTKEYSIZE_4096,
    (FrCryptCryptKeySize) 10000
  };
  ZkError err;
  int badMask[] = {
    0,
    10000
  };
  int goodMask[] = {
    FRCRYPT_CRYPTPARMS_ALL,
    FRCRYPT_CRYPTPARMS_PUBPRIV,
#if 0
    FRCRYPT_CRYPTPARMS_PUB,
    FRCRYPT_CRYPTPARMS_PRIV,
#endif
    10000
  };
  int i, j;

  srand(time(0));

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));

  for (i = 0; ks[i] < 10000; i++) {
    for (j = 0; goodMask[j] < 10000; j++) {
      checkStLd(err, ks[i], goodMask[j], 1);
    }
  }
  
  for (i = 0; ks[i] < 10000; i++) {
    for (j = 0; goodMask[j] < 10000; j++) {
      checkStLd(err, ks[i], badMask[j], 0);
    }
  }
  
  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));
  
  return (EXIT_SUCCESS);
}
