/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <assert.h>
#include "zks.h"
#include "frCrypt.h"
#include "zkShell.h"

int main( int argc, char * argv[] )
{
  uint32_t n, min, max, t,i;
  ZkError err;
  struct timeval tv;
  const char* progname;
  
  progname = zkShellGetProgramName(&argc, &argv);
  
  zkPrintf("Starting rntest...\n");

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  zkRand_Debug();
  
  if (argc == 1) {
    zkGettimeofday(&tv, NULL);
    srand( tv.tv_usec ^ tv.tv_sec );
    min = 0;
    max = rand();
    t = rand();
    if (t & 0x1) { max *= 2; t = rand() & 0x1; max |= t; }
  }
  if (argc > 1) {
    min = 0;
    max = strtoul( argv[1], NULL, 10 );
  }
  if (argc > 2) {
    min = max;
    max = strtoul( argv[2], NULL, 10 );
  }

  /*  zkFprintf(stderr, "range = (0..%lu)\n", max); */

  for( i = 0; i < 100 ; i++){
    if (!frCryptRandomUInt32InRange(err, min, max, &n)) { fprintf(stderr,"error"); }
    zkPrintf("range (%lu..%lu) = %lu\n", min, max, n);
  }
  
  frCryptFinal(err, NULL);
  zkErrorFree(&err);
  zkPrintf(".\n");
  
  return (0);
}
