/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* 
 * Decrypt the begining of an encrypted buffer, check some condition on plaintext and then decrypt the rest 
 * only if the condition is met.
 * Usefull when testing different keys to decrypt an incomming packet.
 * Here, we decrypt once with a good key, once with a wrong key
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frCrypt.h"

#define GOOD_SIZE  64
#define MIN 8
#define MASK 'T'
#define POS 0

static const char plaintext[] = "Therefore it is necessary \
to learn how not to be good, and to use this knowledge and \
not use it, according to the necessity of the cause. -- Machiavelli";


static int conditionalDecrypt(ZkError err, FrCryptSym sym, void *ivec, void *data, void *output, size_t len)
{
  unsigned char *d;
  unsigned char *o;
  
  d = (unsigned char *)data;
  o = (unsigned char *)output;
  
  assert(frCryptSymDecryptToBuf(err, sym, ivec, d, o, MIN));
    if ( o[POS] != MASK ) {
    printf("--- No match\n");
    return 0;
  }
  assert(frCryptSymDecryptToBuf(err, sym, ivec, d+MIN, o+MIN, len - MIN));
  printf("--- Match\n");
  return 1;
}


int main(int ac, char **av) {
#ifndef FREEDOM_PRODUCTION_BUILD
  FrCryptSym sym,sym2;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_UNUSED_;  
  
  ZkError err = 0;
  const char * algStr = "blowfish";
  char ivec[FRCRYPT_IVEC_SIZE];
  char buf[GOOD_SIZE + 1];
  char output[GOOD_SIZE + 1];
  size_t mult, sz;
  
  assert(zkErrorNew(&err));
  
  assert(frCryptSymPortableSize(err, sym, &sz));
  assert(frCryptSymSizeMultiple(err, sym, &mult));
  assert( !(GOOD_SIZE % mult) );

  if (ac == 2) {
    algStr = av[1];
  }
  assert(frCryptSymNameToAlg(err, algStr, &alg));

  /* the key we're going to encrypt with */
  assert(frCryptSymNew(err, &sym)); 
  assert(frCryptSymSetAlg(err, sym, alg));
  assert(frCryptSymRandomKey(err, sym));

  /* another random key */
  assert(frCryptSymNew(err, &sym2)); 
  assert(frCryptSymSetAlg(err, sym2, alg));
  assert(frCryptSymRandomKey(err, sym2));
  
  switch (alg) {
  case FRCRYPT_SYMALG_NONE: 
    printf("Using no alg!!!\n"); 
    break;
  case FRCRYPT_SYMALG_XOR:
    printf("Using xor\n"); 
    break;
  case FRCRYPT_SYMALG_DES3:
    printf("Using des3\n"); 
    break;
  case FRCRYPT_SYMALG_DESX:
    printf("Using desx\n"); 
    break;
  case FRCRYPT_SYMALG_BLOWFISH:
    printf("Using blowfish\n"); 
    break;
  default:
    printf("Default!!!\n"); 
    break;
  }
  
  
  memcpy(buf, plaintext, GOOD_SIZE); buf[GOOD_SIZE] = '\0';

  /* encrypt buf with sym */
  memset(ivec, 0, sizeof ivec);  
  assert(frCryptSymEncrypt(err, sym, ivec, buf, GOOD_SIZE));

  
  /* decrypt buf with sym (ok) */
  memset(ivec, 0, sizeof ivec); /* reset ivec */
  printf("Decrypting with good key");
  assert(conditionalDecrypt(err, sym, ivec, buf, output, GOOD_SIZE));
  output[GOOD_SIZE] = '\0';
  assert(!strncmp(output,plaintext,GOOD_SIZE));
  
  /* decrypt buf with sym (ok), for a second time, should't change anything */
  memset(ivec, 0, sizeof ivec); /* reset ivec */
  printf("Decrypting with good key");
  assert(conditionalDecrypt(err, sym, ivec, buf, output, GOOD_SIZE));
  output[GOOD_SIZE] = '\0';
  assert(!strncmp(output,plaintext,GOOD_SIZE));
  

  /* decrypt buf with sym2 (not ok) */
  memset(ivec, 0, sizeof ivec); /* reset ivec */
  printf("Decrypting with wrong key");
  assert(!conditionalDecrypt(err, sym2, ivec, buf, output, GOOD_SIZE));
  
  assert(frCryptSymFree(err, &sym));
  assert( ! sym);
#endif  
  exit(EXIT_SUCCESS);
  return 0; /* to shut up insure */
}

