/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frCrypt.h"

#define GOOD_SIZE  64

static const char plaintext[] = "Therefore it is necessary \
to learn how not to be good, and to use this knowledge and \
not use it, according to the necessity of the cause. -- Machiavelli";


int main(int ac, char **av) {
#ifndef FREEDOM_PRODUCTION_BUILD
  FrCryptSym sym;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;  
  
  ZkError err = 0;
  char * algStr = "blowfish"; /* default */
  
  char ivec[FRCRYPT_IVEC_SIZE];
  char buf[GOOD_SIZE + 1];
  char buf2[GOOD_SIZE + 1];
  char output[GOOD_SIZE + 1];
  size_t mult, sz;
  
  assert(zkErrorNew(&err));
  
  assert(frCryptSymPortableSize(err, sym, &sz));
  assert(frCryptSymSizeMultiple(err, sym, &mult));
  assert( !(GOOD_SIZE % mult) );

  assert(frCryptSymNew(err, &sym)); 
  
  if (ac == 2) {
    algStr = av[1];
  }
  else {
    printf("Using default, try \"test2buf <algo>\" where algo= blowfish | des3 | desx | xor\n");
  }
  
  assert(frCryptSymNameToAlg(err, algStr, &alg));
  assert(frCryptSymSetAlg(err, sym, alg));
  assert(frCryptSymRandomKey(err, sym));
  
  switch (alg) {
  case FRCRYPT_SYMALG_NONE: 
    printf("Using no alg!!!\n"); 
    break;
  case FRCRYPT_SYMALG_XOR:
    printf("Using xor\n"); 
    break;
  case FRCRYPT_SYMALG_DES3:
    printf("Using des3\n"); 
    break;
  case FRCRYPT_SYMALG_DESX:
    printf("Using desx\n"); 
    break;
  case FRCRYPT_SYMALG_BLOWFISH:
    printf("Using blowfish\n"); 
    break;
  default:
    printf("Default!!!\n"); 
    break;
  }
  

  memcpy(buf, plaintext, GOOD_SIZE); 
  buf[GOOD_SIZE] = '\0';
  buf2[GOOD_SIZE] = '\0';
  output[GOOD_SIZE] = '\0';


  /* encrypt buf in buf2 with sym */
  memset(ivec, 0, sizeof ivec);  
  assert(frCryptSymEncryptToBuf(err, sym, ivec, buf, buf2, GOOD_SIZE));
  
  /* buf untouched ? */
  assert(!memcmp(buf,plaintext,GOOD_SIZE));
  
  /* encrypt buf on itself with sym, should be the same as buf2 */
  memset(ivec, 0, sizeof ivec);  
  assert(frCryptSymEncrypt(err, sym, ivec, buf, GOOD_SIZE));
  assert(!memcmp(buf,buf2,GOOD_SIZE));
    

  /* decrypt buf on output with sym */
  memset(ivec, 0, sizeof ivec); /* reset ivec */
  assert(frCryptSymDecryptToBuf(err, sym, ivec, buf, output, GOOD_SIZE));
  printf("output decrypted: '%s'\n",output);

  /* buf untouched ? */
  assert(!memcmp(buf,buf2,GOOD_SIZE));

  /* decrypt buf on itself with sym */
  memset(ivec, 0, sizeof ivec); /* reset ivec */
  assert(frCryptSymDecrypt(err, sym, ivec, buf, GOOD_SIZE));
  printf("buf decrypted: '%s'\n",buf);

  /* decryptions are the same, and good? */
  assert(!memcmp(buf,output,GOOD_SIZE));
  assert(!memcmp(buf,plaintext,GOOD_SIZE));

  assert(frCryptSymFree(err, &sym));
  assert( ! sym);

#endif
  exit(EXIT_SUCCESS);
  return 0; /* to shut up insure */
}
