/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "frCrypt.h"
#include "zkShell.h"


static FrCryptSymAlg strToSymAlg(char *algStr)
{
  FrCryptSymAlg alg = FRCRYPT_SYMALG_UNUSED_;
  
  if        (strcasecmp(algStr, "null") == 0) {
    alg = FRCRYPT_SYMALG_NONE;
  } else if (strcasecmp(algStr, "blowfish") == 0) {
    alg = FRCRYPT_SYMALG_BLOWFISH;
  } else if (strcasecmp(algStr, "des3") == 0) {
    alg = FRCRYPT_SYMALG_DES3;
  } else if (strcasecmp(algStr, "desx") == 0) {
    alg = FRCRYPT_SYMALG_DESX;
  } else if (strcasecmp(algStr, "xor") == 0) {
    alg = FRCRYPT_SYMALG_XOR;
  } else if (strcasecmp(algStr, "idem") == 0) {
    alg = FRCRYPT_SYMALG_NONE;
  }

  return alg;
}


int main(int ac, char **av)
{
  FrCryptSym sym0, sym1, sym2;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_XOR;
  ZkError err;
  char d[16];
  size_t i;
  const char* progname;
  
  zkPrintf("Starting testKeyFromData...\n");
  
  progname = zkShellGetProgramName(&ac, &av);

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  assert(frCryptSymNew(err, &sym0));
  assert(frCryptSymNew(err, &sym1));
  assert(frCryptSymNew(err, &sym2));

  if (ac == 2) {
    alg = strToSymAlg(av[1]);
  }

  assert(frCryptSymSetAlg(err, sym0, alg));
  assert(frCryptSymSetAlg(err, sym1, alg));
  assert(frCryptSymSetAlg(err, sym2, alg));

  /*
   *  Initialize the key seed data.
   */
  
  for (i = 0; i < sizeof d; i++) {
    d[i] = i;
  }

  /*
   *  Test too little data.
   */
  
  assert( ! frCryptSymKeyFromData(err, sym0, d, 0));
  
  /*
   *  Make sure multiple calls with the same data yield the same keys.
   */

  assert(frCryptSymKeyFromData(err, sym0, d, sizeof d));
  assert(frCryptSymKeyFromData(err, sym1, d, sizeof d));
  assert(frCryptSymKeyFromData(err, sym2, d, sizeof d));

#ifndef macintosh
#pragma message(Reminder "for now use the debugger to check that they're the same")
#endif

  /*
   *  Clean up.
   */
  
  assert(frCryptSymFree(err, &sym0));
  assert( ! sym0);
  assert(frCryptSymFree(err, &sym1));
  assert( ! sym1);
  assert(frCryptSymFree(err, &sym2));
  assert( ! sym2);

  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));
  assert( ! err);
  
  zkPrintf(".\n");

  return (EXIT_SUCCESS);
}
