/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Test message authentication codes.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "frCrypt.h"
#include "zkShell.h"


int main(int argc, char** argv)
{
  ZkError err;
  FrCryptMacKey mkey;
  char kbuf0[FRCRYPT_MACKEY_SIZE], kbuf1[FRCRYPT_MACKEY_SIZE];
  char data[317]; /* make it a strange size */
  char mac[FRCRYPT_MAC_SIZE];
  int valid = 0;
  size_t i;
  
  (void) argc;
  (void) argv;
  
  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  assert(frCryptMacKeyNew(err, &mkey));

  /*
   *  Load and store.
   */

  /* Should all fail: uninit. */
  memset(mkey, 0, sizeof mkey);
  assert( ! frCryptMacKeyStore(err, mkey, kbuf0, sizeof kbuf0));
  assert( ! frCryptMacIntoBuffer(err, mkey, data, sizeof data, mac, sizeof mac));
  assert( ! frCryptMacVerifyFromBuffer(err, mkey, data, sizeof data, mac, sizeof mac, &valid));

  for (i = 0; i < sizeof kbuf0; i++) { kbuf0[i] = i + 1; }
  assert(frCryptMacKeyLoad(err, mkey, kbuf0, sizeof kbuf0));
  memset(kbuf1, 0, sizeof kbuf1);
  assert(frCryptMacKeyStore(err, mkey, kbuf1, sizeof kbuf1));
  assert(memcmp(kbuf0, kbuf1, sizeof kbuf0) == 0);

  /*
   *  MAC generation and verification.
   */

  assert(frCryptMacIntoBuffer(err, mkey, data, sizeof data, mac, sizeof mac));
  assert(frCryptMacVerifyFromBuffer(err, mkey, data, sizeof data, mac, sizeof mac, &valid));
  assert(valid == 1);

  /* Invalid MAC. */
  assert(frCryptMacIntoBuffer(err, mkey, data, sizeof data, mac, sizeof mac));
  mac[sizeof mac - 2]++;
  assert(frCryptMacVerifyFromBuffer(err, mkey, data, sizeof data, mac, sizeof mac, &valid));
  assert(valid == 0);

  /* Different data. */
  assert(frCryptMacIntoBuffer(err, mkey, data, sizeof data, mac, sizeof mac));
  data[sizeof data / 2]++;
  assert(frCryptMacVerifyFromBuffer(err, mkey, data, sizeof data, mac, sizeof mac, &valid));
  assert(valid == 0);

  /* Too small a MAC buffer. */
  assert( ! frCryptMacIntoBuffer(err, mkey, data, sizeof data, mac, 5));

  /*
   *  Clean up.
   */
  
  assert(frCryptMacKeyFree(err, &mkey));
  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));

  zkPrintf(".\n");
  return (EXIT_SUCCESS);
}
