/*
 *  $Id: timecrypt.c,v 1.6 2001/04/19 20:00:04 alainc Exp $
 *  
 *  Time encryption.
 */

#ifdef NDEBUG
#undef NDEBUG
#endif

#include "zks.h"

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkTime.h"
#include "frCrypt.h"
#include "zkShell.h"

static char seedfile[] = "seedfile";

static const char *prog;


int main(int ac, char **av)
{
  FrCryptCrypt key;
  ZkError err;
  char data[500];
  char preSec[FRCRYPT_SECRET_SIZE];
  char sec[FRCRYPT_SECRET_SIZE];
  size_t dcount, ecount, i;
  size_t prvsz = 1024;

  time_t seed = time(0), globalstart, globalstop;
  clock_t start, stop;
  
  prog = zkShellGetProgramName(&ac, &av);
  
  if (ac != 3 && ac != 4) {
    zkFprintf(stderr, "Usage: %s <encr-count> <decr-count>\n", prog);
    exit(EXIT_FAILURE);
  }

  ecount = atoi(av[1]);
  dcount = atoi(av[2]);
  if (ac == 4) {
    prvsz = strtoul(av[3], 0, 10);
  }

  time(&globalstart);

  /*
   *  Set up.
   */

#if 0
  srand(123456);
#else
  zkPrintf("Using seed = %lu.\n", (unsigned long)seed);
  srand(seed);
#endif
  
  for (i = 0; i < sizeof data; i++) { data[i] = rand(); }

  assert(frCryptInit(0, seedfile));
  zkRand_Debug();

  /*
   *  New.
   */
  
  assert(zkErrorNew(&err));
  assert(frCryptCryptNew(err, &key));
  assert(frCryptPubKeySetVersion(err, key, 1 + rand() % 7));

  assert(frCryptCryptNew(err, &key));
  assert(frCryptPubKeySetVersion(err, key, 1 + rand() % 7));
  assert(frCryptCryptSetKeySize(err, key, FRCRYPT_CRYPTKEYSIZE_1024));
  assert(frCryptCryptSetRandomKeyWithPrivSize(err, key, prvsz));

  if (ecount > 0) {
    /*
     *  Encrypt.
     */

    assert(zkRandGetData(err, sec, sizeof sec));

    start = clock();

    for (i = 0; i < ecount; i++) {
      assert(frCryptCryptSetPreSecret(err, key, preSec, sizeof preSec, sec, sizeof sec));
    }

    stop = clock();
  
    zkPrintf("Encryption rate: %.3f encrs/sec.\n",
           CLOCKS_PER_SEC * ecount / (double)(stop - start));

    zkPrintf("Start [%lu]\n", start);
    zkPrintf("Stop [%lu]\n", stop);
  }
  
  if (dcount > 0) {
    /*
     *  Decrypt.
     */

    assert(frCryptCryptSetPreSecret(err, key, preSec, sizeof preSec, sec, sizeof sec));

    start = clock();

    for (i = 0; i < dcount; i++) {
      assert(frCryptCryptSecretFromPrivKey(err, key, preSec, sizeof preSec, sec, sizeof sec));
    }

    stop = clock();
  
    zkPrintf("Decryption rate: %.3f vrfys/sec.\n",
           CLOCKS_PER_SEC * dcount / (double)(stop - start));

    zkPrintf("Start [%lu]\n", start);
    zkPrintf("Stop [%lu]\n", stop);
  }
  
  /*
   *  Clean up.
   */
  
  assert(frCryptCryptFree(err, &key));
  assert(frCryptFinal(err, 0));
  assert(zkErrorFree(&err));
  assert(err == 0);

  time(&globalstop);

  zkPrintf("Global time [%lu]\n", globalstop - globalstart);
  
  return EXIT_SUCCESS;
}
