/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <sys/types.h>
#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkLog.h"

#include "openssl/rand.h"
#include "zkShell.h"


static char seedfile[] = "seedfile";

int main(int ac, char **av)
{
  ZkError e;
  ZkLog log;
  const char *prog;

  prog = zkShellGetProgramName(&ac, &av);

  if ( ! (zkErrorNew(&e) &&
          zkErrorLog(e, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, "/dev/tty") &&
          zkLogEnable(log))) {
    zkFprintf(stderr, "%s: error setting log object -- exiting.\n", prog);
    exit(EXIT_FAILURE);
  }

  assert(frCryptInit(e, seedfile));
  assert(frCryptInit(e, seedfile));

  /* Change the entropy pool. */

  {
    pid_t p = zkGetpid();
    RAND_seed((unsigned char *)&p, sizeof p);
  }

  assert(frCryptFinal(e, seedfile));
  assert(frCryptFinal(e, seedfile));

  /*
   *  Calling Final more times than Init should produce an error.
   */

  zkFprintf(stderr, "%s: expect an error about calling frCryptFinal() too many times.\n",
          prog);
  assert( ! frCryptFinal(e, seedfile));

  assert(zkErrorFree(&e));
  
  return (EXIT_SUCCESS);
}
