/*
 *  $Id: tprmast.c,v 1.3 2001/04/19 20:00:05 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Hack to print the master key.
 *  
 *  bug: based on tprkey.
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkString.h"
#include "zkShell.h"

static char seedfile[] = "seedfile";

static const char *prog;


static void usage(int ok)
{
  zkFprintf(stderr, "Usage: %s [--help] [--logfile]\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  ZkError err;
  ZkLog log;
  const char *logfile = "/dev/tty";
  int ok = 0;

  prog = zkShellGetProgramName(&ac, &av);

  zkErrorNew(&err);

  while (av++, --ac) {
    if (strcasecmp(av[0], "-h") == 0 &&
        strcasecmp(av[0], "-help") == 0 &&
        strcasecmp(av[0], "--help") == 0) {
      usage(1);
    } if (strcasecmp(av[0], "--logfile") == 0) {
      if (av++, --ac < 1) { usage(0); }
      logfile = av[0];
    } else {
      usage(0);
    }
  }

  if ( ! (zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logfile) &&
          zkLogEnable(log))) {
    zkFprintf(stderr, "%s: can't set log object -- exiting.\n", prog);
    goto done;
  }
  
  if ( ! frCryptInit(err, seedfile)) {
    zkLogError(log, err);
    zkFprintf(stderr, "%s: can't initialize crypto library -- exiting.\n", prog);
    goto done;
  }

  if ( ! frCryptPubKeyPrint(err, frCryptSigZkMaster, stdout)) {
    goto done;
  }

  ok = 1;

 done:

  if (err) { zkErrorFree(&err); }

  return (ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
