/* Basic test program that tests the serialise/deserialise functions in frcrypt. */
 
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "frCrypt.h"

int main(void)
{
	static const FrCryptSymAlg	kAlgs[] = {
		FRCRYPT_SYMALG_NONE,
		FRCRYPT_SYMALG_XOR,
		FRCRYPT_SYMALG_BLOWFISH,
		FRCRYPT_SYMALG_DES3,
		FRCRYPT_SYMALG_DESX,
	};
	
	ZkError	err;
	int		i;
	
	assert(zkErrorNew(&err));
	assert(frCryptInit(err, NULL));
	
	for (i = 0; i < sizeof(kAlgs) / sizeof(kAlgs[0]); i++)
	{
		char		srcplaintext[96], dstplaintext[96], ciphertext[96];
		char		serbuff[100];
		char		ivec[FRCRYPT_IVEC_SIZE];
		FrCryptSym	srcsym, dstsym;
		size_t		sersize, srcserused, dstserused;
		
		assert(zkRandGetData(err, srcplaintext, sizeof(srcplaintext)));
		
		assert(frCryptSymNew(err, &srcsym));
		assert(frCryptSymSetAlg(err, srcsym, kAlgs[i]));
		if (kAlgs[i] == FRCRYPT_SYMALG_BLOWFISH)
		{
			uint32_t	keysize;
			
			assert(zkRandGetUInt32InRange(err, 16, 56, &keysize));
			assert(frCryptSymSetKeySize(err, srcsym, keysize));
		}
		assert(frCryptSymRandomKey(err, srcsym));
		
		memset(ivec, 0, sizeof(ivec));
		assert(frCryptSymEncryptToBuf(err, srcsym, ivec, srcplaintext, ciphertext, sizeof(srcplaintext)));
		
		assert(frCryptSymGetSerialiseSize(err, srcsym, &sersize));
		srcserused = 0;
		assert(frCryptSymSerialise(err, srcsym, serbuff, sersize, &srcserused));
		assert(sersize == srcserused);
		
		assert(frCryptSymNew(err, &dstsym));
		dstserused = 0;
		assert(frCryptSymDeserialise(err, dstsym, serbuff, sersize, &dstserused));
		assert(sersize == dstserused);
		
		memset(ivec, 0, sizeof(ivec));
		assert(frCryptSymDecryptToBuf(err, dstsym, ivec, ciphertext, dstplaintext, sizeof(dstplaintext)));
		assert(memcmp(srcplaintext, dstplaintext, sizeof(srcplaintext)) == 0);
	}
	
	return 0;
}
