/*
 *  $Id: tsha1.c,v 1.5 2001/04/19 20:00:05 alainc Exp $
 *  
 *  Test program initially written to confirm that a single SHA-1 update over
 *  a chunk of data is equivalent to a sequence of updates over smaller chunks
 *  of the same data.
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkShell.h"
#include "zkString.h"

static char seedfile[] = "seedfile";

/*
 *  Ensure the size is a multiple of 1024 bytes.
 */
static char d[10 * 1024];


int main(int ac, char **av)
{
  FrCryptHash hash = 0;
  ZkError err = 0;
  ZkLog log;
  const char *prog;
  char h0[FRCRYPT_HASH_SIZE], h1[FRCRYPT_HASH_SIZE];
  const char *logfile = "/dev/tty";
  int ok = 0;
  size_t i;

  prog = zkShellGetProgramName(&ac, &av);

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logfile) &&
          zkLogEnable(log) &&
          frCryptInit(err, 0))) {
    zkFprintf(stderr, "%s: error initializing -- exiting.\n", prog);

    goto done;
  }
  zkRand_Debug();

  for (i = 0; i < sizeof d; i++) { d[i] = i; }

  /*
   *  Do one hash over the entire data.
   */

  if ( ! frCryptHashGenerate(err, d, sizeof d, h0, sizeof h0)) {
    goto done;
  }

  /*
   *  Do the hash in 10 steps.
   */

  if ( ! frCryptHashNew(err, &hash)) {
    goto done;
  }

  for (i = 0; i < sizeof d; i += 1024) {
    if ( ! frCryptHashUpdate(err, hash, d + i, 1024)) {
      goto done;
    }
  }

  if ( ! frCryptHashGetHash(err, hash, h1, sizeof h1)) {
    goto done;
  }
          
  if (memcmp(h0, h1, sizeof h0) == 0) {
    zkFprintf(stderr, "%s: the hashes agree.\n", prog);
  } else {
    zkFprintf(stderr, "%s: the hashes differ.\n", prog);
  }

  ok = 1;

 done:
  
  frCryptFinal(err, 0);
  if (err) { zkErrorFree(&err); }

  if (ok)
    zkPrintf("OK.\n");
  else
    zkPrintf("Failed.\n");
  return (ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
