/*
 *  $Id: tssig.c,v 1.6 2001/04/19 20:00:05 alainc Exp $
 * 
 *  
 *  Test our DSA parameter generation code with SSLeay's high level signature
 *  code.
 */

#include "zkSysTime.h"
#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkUnistd.h"
#include "openssl/evp.h"
#include "frCrypt.h"


#define BSZ 128


static void pbytes(FILE *fp, unsigned char *buf, size_t sz)
{
  size_t i;
  
  for (i = 0; i < sz; i++) {
    zkFprintf(fp, " %03u", buf[i]);
    if ((i + 1) % 16 == 0) fputs("\n", fp);
  }
  fputs("\n", fp);
}


int main(void)
{
  DSA *dsa = 0;
  EVP_MD_CTX signCtx;
  EVP_MD_CTX vrfyCtx;
  EVP_PKEY *pkey = 0;
  ZkError err;
  size_t i;
  unsigned char buf0[BSZ], buf1[BSZ], smd[BSZ];
  unsigned int sz = 0;

  zkPrintf("Starting tssig...\n");

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, NULL));
  zkRand_Debug();

  /* Initialize the buffer to be signed. */
  
  for (i = 0; i < sizeof buf0; i++) { buf0[i] = i; }
  memcpy(buf1, buf0, sizeof buf1);

  /* Create the keys. */
  
  assert(fr_CryptSigRandomDsaNew(err, &dsa));

  assert(pkey = EVP_PKEY_new());
  EVP_PKEY_assign_DSA(pkey, dsa);

  /* Sign the message. */
  
  EVP_SignInit(&signCtx, EVP_dss1());
  EVP_SignUpdate(&signCtx, buf0, sizeof buf0);
  assert(EVP_SignFinal(&signCtx, smd, &sz, pkey));
  assert(sz <= sizeof buf0);

  /* Verify that buf0 remains unchanged. */

  assert(memcmp(buf0, buf1, sizeof buf0) == 0);

  /* Verify the signature. */

  EVP_VerifyInit(&vrfyCtx, EVP_dss1());
  EVP_VerifyUpdate(&vrfyCtx, buf1, sizeof buf1);
  assert(EVP_VerifyFinal(&vrfyCtx, smd, sz, pkey));

  /* Verify that buf1 remains unchanged. */

  assert(memcmp(buf1, buf0, sizeof buf1) == 0);

  /* Clean up. */
  
  EVP_PKEY_free(pkey); /* This also frees dsa. */
  assert(frCryptFinal(err, NULL));
  assert(zkErrorFree(&err));
  
  zkPrintf(".\n");

  return (EXIT_SUCCESS);
}
