/*
 *  $Id: tsymcopy.c,v 1.2 2001/04/19 20:00:05 alainc Exp $
 *  
 *  Test frCryptSymCopy().
 */

#ifdef NDEBUG
#undef NDEBUG
#endif

#include "zks.h"

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frCrypt.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkrand.h"


static char *prog;


static char *tail(char *path)
{
  char *p = strrchr(path, '/'); /* bug: Windows */
  return p ? p+1 : path;
}


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--logFile <path>]"
          "\n", prog);
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


static void test0(ZkError err, ZkLog log)
{
  FrCryptSym sym0, sym1;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;
  char iv0[FRCRYPT_IVEC_SIZE];
  char iv1[FRCRYPT_IVEC_SIZE];
  size_t iters = 20, sz = 100 * 8, i;
  void *copy, *orig;

  (void)log;

  assert(copy = malloc(sz));
  assert(orig = malloc(sz));

  assert(frCryptSymNew(err, &sym0));
  assert(frCryptSymSetAlg(err, sym0, alg));
  assert(frCryptSymRandomKey(err, sym0));
  assert(frCryptSymCopy(err, sym0, &sym1));

  for (i = 0; i < iters; i++) {
    assert(zkRandGetData(err, orig, sizeof orig));
    assert(zkRandGetData(err, iv0, sizeof iv0));
    memcpy(iv1, iv0, sizeof iv1);
    assert(frCryptSymEncryptToBuf(err, sym0, iv0, orig, copy, sz));
    assert(frCryptSymDecrypt(err, sym1, iv1, copy, sz));
    assert(memcmp(orig, copy, sz) == 0);
  }

  free(copy);
  free(orig);
}


int main(int ac, char **av)
{
  ZkError err;
  ZkLog log;
  const char *logFile = "/dev/tty";
  int ok = 0;

  prog = tail(av[0]);

  while (av++, --ac) {
    if (strcasecmp("-h",     av[0]) == 0 ||
        strcasecmp("-help",  av[0]) == 0 ||
        strcasecmp("--help", av[0]) == 0) {
      usage(1);
    } else if (strcasecmp("--logFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      logFile = av[0];
    } else {
      fprintf(stderr, "%s: invalid option `%s'.\n", prog, av[0]);
      usage(0);
    }
  }
  
  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logFile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: error initializing logging -- exiting.\n", prog);
    goto done;
  }

  test0(err, log);

  printf("%s: all tests passed.\n", prog);
  ok = 1;

 done:
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
  return ok ? EXIT_SUCCESS : EXIT_FAILURE; /* keep insure happy */
}

