/*
 *  $Id: tzkm.c,v 1.9 2001/04/19 20:00:05 alainc Exp $
 *  
 *  Test the Zero Knowledge master signing key and software update key.
 *  
 *  Usage: tzkm <priv-key>
 *  
 *  Load the private key from the file <priv-key>, generate some random data,
 *  then sign and verify it.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkTime.h"
#include "frCrypt.h"
#include "zkError.h"
#include "zkShell.h"
#include "zkString.h"

static char seedfile[] = "seedfile";

int main(int ac, char **av)
{
  FILE *fp;
  FrCryptSig priv, pub;
  ZkError e;
  char buf[2048], data[2048], sigbuf[FRCRYPT_SIG_SIZE];
  int i;
  size_t n, s = sizeof sigbuf;
  const char* progname;

  progname = zkShellGetProgramName(&ac, &av);
  
  if (ac != 3) {
    zkFprintf(stderr, "Usage: %s [--master | --supd] <priv-key>\n", progname);
    exit(EXIT_FAILURE);
  }

  assert(zkErrorNew(&e));
  assert(frCryptInit(e, seedfile));
  zkRand_Debug();
  if (strcmp(av[1], "--master") == 0) {
    assert(pub = frCryptSigZkMaster);
  } else if (strcmp(av[1], "--supd") == 0) {
    assert(pub = frCryptSigZkSupd);
  } else {
    zkFprintf(stderr, "Usage: %s [--master | --supd] <priv-key>\n", progname);
    exit(EXIT_FAILURE);
  }

  if ( ! (fp = zkFopen(av[2], "rb"))) {
    perror(av[2]);
    exit(EXIT_FAILURE);
  }

  if ((n = zkFread(buf, 1, sizeof buf, fp)) < 1) {
    perror("fread");
    exit(EXIT_FAILURE);
  }

  (void)zkFclose(fp);

  assert(frCryptSigNewFromData(e, &priv, buf, &n));

  srand(time(0));

  for (i = 0; i < 10; i++) {
    int valid = 0;
    size_t sz = 1 + rand() % (sizeof data - 1);

    zkPrintf("Round %d of %d.\r", i + 1, 10); zkFflush(stdout);

    assert(zkRandGetData(e, data, sz));
    assert(frCryptSigSignData(e, priv, data, sz, sigbuf, s));
    assert(frCryptSigVerifyData(e, pub, data, sz, sigbuf, sizeof sigbuf, &valid));
    assert(valid);
  }

  zkPrintf("\n");

  assert(frCryptSigFree(e, &priv));
  /* Don't free pub; it's a constant. */
  assert(frCryptFinal(e, 0));
  assert(zkErrorFree(&e));

  return (EXIT_SUCCESS);
}
