/*
 * Freedom Nidb Objects 
 * $Id: fr_dbo_nidb_obj.h,v 1.12 2001/04/19 20:00:19 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 *
 */


#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "zkNetinetIn.h"
#include "zkArpaInet.h"

#include "zkIntTypes.h"
#include "zkDBO.h"
#include "zkDBODefs.h"
#include "zkDBOTypes.h"
#include "frEnt.h"
#include "frDBOTypeNidbObj.h"


#ifndef FRDBO_INTTYPE_NIDB_OBJ_H_
#define FRDBO_INTTYPE_NIDB_OBJ_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /**
   * The structure used to represent a network service.
   * Never refernce directly
   */
  struct FrDBOTypeNidbObjServerEntity_T {
    struct in_addr sin_addr;	/* The IP address of this AIP */
    uint32_t port_nbo;          /* The port number of this service */
    uint32_t status;		/* The latest state */
    uint32_t rating;		/* The latest rating */
    char region[16];
    char servoperator[64];
  };
  typedef struct FrDBOTypeNidbObjServerEntity_T FrDBOTypeNidbObjServerEntity;
  
  /**
   * Special case for the Aip
   */
  struct FrDBOTypeNidbObjAipEntity_T {
    struct in_addr sin_addr;	/* The IP address of this AIP */
    uint32_t dh_port_nbo;	/* The DH Port */
    uint32_t fr_port_nbo;	/* The freedom (UDP) port */
    uint32_t num_neighbours;	/* The number of neighbours this AIP has */
    uint32_t status;		/* The latest state */
    uint32_t rating;		/* The latest rating */
    uint32_t connections;	/* The latest rating */
    char region[FRNIQSOBJ_REGION_MAX];
    char servoperator[FRNIQSOBJ_OPERATOR_MAX];
    struct FrDBOTypeNidbAipNeighbour_T {
      FrEntIdent rem_aip;	/* The remote AIP entity identifier */
      /* FrEntIdent lnk;          The entity identifier of this link */
    } neighbours[1];		/* The list of neighbours and links */
  };
  typedef struct FrNidbObjAipEntity_T FrNidbObjAipEntity;


  /* The union part of a structure sent back as a result a status command */
  struct FrDBOTypeNidbObjStatusResp_T {
    uint32_t num_resp;
    struct FrDBOTypeNidbStatusRespEntry_T {
      FrEntIdent ent_ident;	/* The remote AIP entity identifier */
      uint32_t status;
      uint32_t rating;
      uint32_t connections;
    } response[1];
  };
  typedef struct FrDBOTypeNidbObjStatusResp_T FrDBOTypeNidbObjStatusResp;

  /* The List part of a structure in response to a MxQuery*/
  struct FrDBOTypeNidbObjListResp_T {
    uint32_t num_NidbObj;
    struct FrDBOTypeNidbListRespEntry_T { 
      void * nidb_obj;
    } entry[1];
  };
  typedef struct FrDBOTypeNidbObjListResp_T FrDBOTypeNidbObjListResp;
  
  union FrDBOTypeNidbObjDatum_U {
    struct FrDBOTypeNidbObjAipEntity_T aip;
    struct FrDBOTypeNidbObjServerEntity_T server;
    struct FrDBOTypeNidbObjStatusResp_T status; 
    struct FrDBOTypeNidbObjListResp_T list;
  };

  typedef union FrDBOTypeNidbObjDatum_U *FrDBOTypeNidbObjDatum;

  struct FrDBOTypeNidbObj_T_v4 {
    uint32_t regID;
    uint32_t _ds_size;        /* The overall size of this datastructure
                                   Private internal variable */
    time_t   last_update_t;   /* The last time this information was updated
                               * This time is context sensitive, depending on
                               * what last_update we're talking about.
                               */
    uint32_t ttl_secs;        /* The amount of time this response is valid for */
    FrEntIdent ent_ident;
    FrNidbObjType type;
    union FrDBOTypeNidbObjDatum_U s;
  };

  struct FrDBOTypeNidbObj_T {
    uint32_t regID;
    uint32_t _ds_size;        /* The overall size of this datastructure
                                   Private internal variable */
    time_t   last_update_t;   /* The last time this information was updated
                               * This time is context sensitive, depending on
                               * what last_update we're talking about.
                               */
    uint32_t ttl_secs;        /* The amount of time this response is valid for */
    FrEntIdent ent_ident;
    FrEntIdExp  entity;       /* New in Version 5 */
    uint32_t type;            /* Really an FrNidbObjType, but because it's an enum
                                 it can't be relied upon to be 32 bits in size. */
    union FrDBOTypeNidbObjDatum_U s;
  };
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
