/*
 * NSDB aip DBO Driver routines
 * $Id: frDBOTypeNtdbTopo.c,v 1.8 2001/04/19 20:00:20 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>

 *
 */

#include "zks.h"

#if 0
#include <assert.h>
#include "zkStdlib.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkDefs.h"
#include "zkAlloc.h"
#include "zkDump.h"
#include "zkDebug.h"
#include "frNIDB.h"
#include "frDBOTypes.h"
#if 0
#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNDB

#define DBODS_REGID ZKDBO_REG_FRNTDBTOPO
#define DBODS_T     T_FrDBOTypeNtdbNeighbours


typedef struct DBODS_T *DBODS;

static int DSnew (ZkError err, uint32_t version, void **ds, size_t *dsz)
{
  static const char *funcname = "DSnew";
  /* Special handing of DS */
  size_t num = *dsz;

  DBODS d;

  if (!dsz || !ds) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers passed to function");
    return 0;
  }
  if (num == 0) num = 1;
  
  *dsz = sizeof(struct DBODS_T) + (sizeof (uint32_t) * (num-1));
  zkMALLOC (d, DBODS, *dsz);

  d->regID = DBODS_REGID;
  *ds = (void *)d;
  return 1;
}

static int DSfree (ZkError err, void **ds, size_t *dsz)
{
  static const char *funcname = "DSfree";
  
  if (!dsz || !ds || !*ds) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers passed to function");
    return 0;
  }
  
  memset (*ds, 0, *dsz);
  zkFree(*ds);
  *ds = NULL;
  *dsz = 0;
  return 1;
}

/**
 * Passing a null pointer to *pobj will create the buffer you need.
 * On failure, you'll have to take care of cleaning it up.
 */
static int DSpack (ZkError err, void *ds, size_t dsz, void **pds, size_t *psz)
{
  static const char *funcname = "DSpack";
  DBODS d = NULL;
  size_t pos = 0;
  size_t i = 0, min_size = 0;
  
  if ( !ds || !dsz || !pds || !psz ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers or values passed to function");
    return 0;
  }
  d = ds;

  /* Now we allocate additional space for us */
  if (!zkDBOPackedReqSpace(err, d->regID, &min_size))
    return 0;

  *psz = min_size + ((d->num_neighbours - 1) * sizeof(uint32_t));

  zkDebug ("Adjusted packing size is now %d", *psz);

  if (!zkDBOPackedReqSpace(err, d->regID, &min_size))
    return 0;
  
  if (!zkDBOPackedInit(err, d->regID, pds, psz, &pos))
    return 0;

#define DBO_PACK(x) \
  if (!zkDBOPackField(err, d->regID, #x, (void *) & d->x, \
		      sizeof(d->x), pds, psz, &pos)) { \
    return 0; \
  }

#define DBO_PACK2(w, x) \
  if (!zkDBOPackField(err, d->regID, #w, (void *) & d->x, \
		      sizeof(d->x), pds, psz, &pos)) { \
    return 0; \
  }
  DBO_PACK(regID);
  DBO_PACK(node_entid);
  DBO_PACK(num_neighbours);
  for (i = 0; i < d->num_neighbours; i++) {
    DBO_PACK2(neighbour, neighbour[i]);
  }
  return 1;
}

/**
 * Passing a null pointer to *ds will create the buffer you need.
 * On failure, you'll have to take care of cleaning it up.
 */
static int DSunpack(ZkError err, void **ds, size_t *dsz, void *pds, size_t psz)
{
  static const char *funcname = "DSunpack";
  size_t s = 0, pos=0, i = 0;
  int regID = 0;
  void *pd = NULL;
  struct DBODS_T *d = NULL;
  
  if ( !ds || !dsz || !pds || !psz) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
		  "Null pointers or values passed to function");
    return 0;
  }

  if (!zkDBOUnpackedInit(err, &regID, ds, dsz, pds, psz, &pos))
    return 0;

  d = *ds;

#define DBO_UNPACK(x) \
  s = sizeof(d->x); \
  pd = &(d->x); \
  if (!zkDBOUnpackField(err, regID, #x, &pd, \
		 &s, pds, psz, &pos)) \
    return 0;

#define DBO_UNPACK2(w,x) \
  s = sizeof(d->x); \
  pd = &(d->x); \
  if (!zkDBOUnpackField(err, regID, #w, &pd, \
		 &s, pds, psz, &pos)) \
    return 0;

  DBO_UNPACK(regID);
  DBO_UNPACK(node_entid);
  DBO_UNPACK(num_neighbours);
  if (d->num_neighbours > 1) {
    size_t osz = *dsz;
    /* Need to reallocate space */
    *dsz += ( d->num_neighbours - 1 ) * sizeof (d->node_entid);
    if (! zkMemRealloc (err, ds, *dsz)) {
      zkFree(*ds);
      return 0;
    }
    zkDebug("Realloced space to %d bytes", *dsz);
    d = *ds; /* Just in case */
    memset(((char *)*ds)+osz, 0, *dsz-osz);
  }
  for (i = 0; i < d->num_neighbours; i++) {
    DBO_UNPACK2(neighbour, neighbour[i]);
  }
  
  return 1;
}

#ifndef DSdump
static int DSdump (ZkError err, void *ds)
{
  static const char *funcname = "DSdump";
  DBODS d;
  size_t i; 
  
  if ( !ds ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
	      		  "Null pointers or values passed to function");
    return 0;
  }
  d = (DBODS) ds;
  zkDebug("**Dumping Registered Datastructure for Topology Configuraion ***");
#define DBO_DUMP(x)  zkDebug("%-16s: %lu", #x, (unsigned long) d->x);
#define DBO_DUMP2(w, x)  zkDebug("%-16s: %lu", #w, (unsigned long) d->x);
  
  DBO_DUMP(regID);
  DBO_DUMP(node_entid);
  DBO_DUMP(num_neighbours);
  for (i = 0; i < d->num_neighbours; i++) {
    DBO_DUMP2(neighbour, neighbour[i]);
  }

  return 1;
}

static int DStest (ZkError err)
{
  DBODS d1 = NULL;
  DBODS d2 = NULL;
  void *pds = NULL , *ds2_p = NULL;
  size_t dsz1 = 0 , dsz2 = 0 , pdsz = 0 ;
  int retcode = 0;

  dsz1 = sizeof(struct DBODS_T);
  dsz1 = 3 ; /* Neighbours */ 
  zkDBONew(err, DBODS_REGID, (void **) &d1, &dsz1);
  zkDBONew(err, DBODS_REGID, (void **) &d2, &dsz2); /*Let's see it adjust itself*/

  /*
   * Do any special initialization here
   */
  d1->node_entid = 16;
  d1->num_neighbours = 3;
  d1->neighbour[0] = 18;
  d1->neighbour[1] = 20;
  d1->neighbour[2] = 22;

  /*
   * The rest should be the same
   */
  DSdump(err, d1);
  	 
  zkDebug("Packing datastructure");
  if (!zkDBOPack (err, d1, dsz1, &pds, &pdsz))
    goto fail;

  /* dsz2 = sizeof(struct DBODS_T); */
  ds2_p = (void *) d2;

  zkDebug("Unpacking datastructure");
  if (!zkDBOUnpack (err, &ds2_p, &dsz2, pds, pdsz))
    goto fail;

  /* Because a realloc might happen, let's track it.. */
  d2 = (DBODS) ds2_p;
     

  if (dsz1 != dsz2) {
    zkDebug("Data structure sizes have changed (%u -> %u)", dsz1, dsz2);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d1,dsz1,zkDumpToStream,stderr);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d2,dsz2,zkDumpToStream,stderr);
    goto fail;

  }

  if (memcmp(d1, d2, dsz1) != 0) {
    zkDebug("Data structure improperly copied");
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d1,dsz1,zkDumpToStream,stderr);
    if (zkDebugIsOn())
      zkDumpBuffer(err,(void *)d2,dsz2,zkDumpToStream,stderr);
    goto fail;
  }

  retcode = 1;
  
 fail:
  if (retcode) {
    zkDebug("Self test passed");
  } else {
    zkDebug("Self test FAILED");
  }
  FREEIT (d1);
  FREEIT (d2);
  FREEIT (pds);
  return retcode;
  
}

#else
#define DSdump NULL
#define DStest NULL
#endif


DLLExport
int frDBODriverInitNtdbTopo(ZkError err)
{
  ZkDBODesc dboDesc;
  ZkDBODriver dboDrv;
  size_t sz, psz;

#define NUMFIELDS 4

  if (!zkDBODescNew(err, NUMFIELDS, &dboDesc))
    return 0;

  /* Bug: Fix the userdefs */
  /* Bug: Problem with recording sizes of BUF and STR */
  if (!zkDBODescSetField(err, dboDesc, 0, "regID", FT_UINT32, &sz, &psz)
    ||!zkDBODescSetField(err, dboDesc, 1, "node_entid", FT_UINT32, &sz, &psz)
    ||!zkDBODescSetField(err, dboDesc, 2, "num_neighbours", FT_UINT32, &sz, &psz)
    ||!zkDBODescSetField(err, dboDesc, 3, "neighbour", FT_UINT32,&sz, &psz)
      )
    return 0;

  if (!zkDBODriverNew (err, &dboDrv))
    return 0;

  if (!zkDBODriverInit (err, DBODS_REGID, dboDrv,
			DSnew, DSfree, DSpack, DSunpack,
			NULL, NULL, DSdump, DStest))
    return 0;

  if (!zkDBODescRegister(err, dboDesc, dboDrv))
    return 0;

# ifndef NDEBUG
  if (!zkDBODescDump(err, dboDesc))
    return 0;
# endif  
  return 1;
}
#endif
#endif
