/*
 * Database Object Functions
 *
 * DBO Driver handler routines
 *
 * $Id: zkDBODriver.c,v 1.8 2001/04/19 20:00:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>

 *
 */
#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkError.h"
#include "zkDebug.h"
#include "zkDBO.h"
#include "zkDBOTypes.h"
#include "zkDBOpriv.h"

#ifdef ZKMOD
#undef ZKMOD
#endif
#define ZKMOD ZKMDB

DLLExport
int zkDBODriverNew(ZkError err, ZkDBODriver *dbd)
{
  static const char *funcname = "zkDBODriverNew";

  if (!dbd) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
		    "ZkDBODriver is an invalid pointer");
      return 0;
  }

  zkMALLOC(*dbd, ZkDBODriver, sizeof (struct ZkDBODriver_T));
  
  return 1;
}

DLLExport
int zkDBODriverFree(ZkError err, ZkDBODriver *dbd)
{
  (void) err;
  zkFree(*dbd);
  *dbd = NULL;
  return 1;
}

DLLExport
int zkDBODriverInit(ZkError err,
		    int regID,
		    ZkDBODriver          dbd,
		    ZkDBODriverAlloc_f   new,
		    ZkDBODriverDealloc_f freeit,
		    ZkDBODriverPack_f    pack,
		    ZkDBODriverUnpack_f  unpack,
		    ZkDBODriverPkSz_f    packedsize,
		    ZkDBODriverUnPkSz_f  unpackedsize,
		    ZkDBODriverDump_f    dump,
		    ZkDBODriverTest_f    test)
		    
{
  static const char *funcname = "zkDBODriverInit";
  
  if (!dbd) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
		    "invalid DBODriver pointer");
      return 0;
  }
  
  /* Bug: need to check to see if this regID already exists */
  dbd->regID        = regID; 
  dbd->new          = new;
  dbd->free         = freeit;
  dbd->pack         = pack;
  dbd->unpack       = unpack;
  dbd->packedsize   = packedsize;
  dbd->unpackedsize = unpackedsize;
  dbd->dump         = dump;
  dbd->test         = test;
  return 1;
}
