/**
 * DBO Utilities
 *
 * Pack and unpack and Ip Address. It's assumed that the
 * address is in Network Byte Order
 *
 * $Id: zkDBOTypeIpAddr.c,v 1.5 2001/04/19 20:00:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#include "zks.h"


#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkArpaInet.h"
#include "zkString.h"
#include "zkNetinetIn.h"

#include "zkDebug.h"
#include "zkDump.h"
#include "zkDBO.h"
#include "zkDBOTypes.h"


#ifdef ZKMOD
#undef ZKMOD
#endif
#define ZKMOD ZKMDB

#define ZK_PACKEDSIZE 4
DLLExport
int zkDBOPackIpAddr (ZkError err, const char *funcname, void *data, size_t ds,
	      void **packed, size_t *psize) 
{
  static const char *callee = "zkDBOPackIpAddr";
  uint32_t hbo = 0;

  if (!funcname) funcname = callee;

  if (!data || !packed || !psize ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Data objects have null pointers");
    return 0;
  }

  hbo = ntohl( * (uint32_t *) data);

  return (zkDBOPackUint32 (err, funcname, &hbo, ds, packed, psize));
  
}

DLLExport
int zkDBOUnpackIpAddr (ZkError err, const char *funcname, void **data, size_t *ds,
	      void *packed, size_t *psize) 
{
  static const char *callee = "zkDBOUnpackIpAddr";
  uint32_t hbo = 0;
  
  if (!funcname) funcname = callee;

  if ( !packed || !psize || !data || !ds ) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Data objects have null pointers");
    return 0;
  }

  if ( zkDBOUnpackUint32 (err, funcname, data, ds, packed, psize)) {
    hbo = ** (uint32_t **) data;
    ** (uint32_t **) data = htonl(hbo);
  }


  return 1;
}

#ifndef NDEBUG

DLLExport
int zkDBOTestIpAddr (ZkError err, char *ipaddr)
{
  static const char *funcname = "zkDBOTestIpAddr";
  unsigned char *dbuf = NULL;
  unsigned char sbuf[16] = {0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00};

  struct in_addr sin_addr;
  uint32_t recovered, *pu, nbo;
  unsigned char *p;
  size_t s = 16, ps = 4;

  dbuf = sbuf;
  zkDebug ("%s: testing %s", funcname, ipaddr);

  nbo = zkInet_addr(ipaddr);
  
  if (!zkDBOPackIpAddr (err, funcname, &nbo, sizeof(nbo),
		    (void **) &dbuf, &s))
    return 0;

  zkDebug ("%s: (static buf) packed value of size %lu)", funcname, (unsigned long) s);
  zkDumpBuffer(err, dbuf, s, zkDumpToStream, stderr);  

  pu = &recovered;
  
  if (!zkDBOUnpackUint32 (err, funcname, (void **) &pu, &ps,
	      (void *)dbuf, &s))
    return 0;

  p = (unsigned char *) &recovered;
  sin_addr.s_addr = recovered;
  zkDebug ("%s: (static buf) original IP=%s - final IP=%s",
	   funcname, ipaddr, zkInet_ntoa(sin_addr));
  return 1;
}

#endif
