/**
 * DBO Utilities
 *
 * Pack and unpack a string buffer
 * INCOMPLETE
 *
 * $Id: zkDBOTypeStr.c,v 1.6 2001/04/19 20:00:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkDebug.h"
#include "zkDump.h"
#include "zkDBO.h"
#include "zkDBOTypes.h"

/*
      +----+---------+
      |SIZE| String  |
      +----+---------+

      size is sizof(strfer)+sizeof(size);
      Eventually, it will be variable length
*/

#ifdef ZKMOD
#undef ZKMOD
#endif
#define ZKMOD ZKMDB

DLLExport
int zkDBOPackStr (ZkError err, const char *funcname, void *data, size_t ds,
	      void **packed, size_t *psize) 
{
  static const char *callee = "zkDBOPackStr";
  uint32_t obj ;
  unsigned char *p;
    
  if (!funcname) funcname = callee;
  if (!data) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Data objects have null pointers");
    return 0;
  }

  if (!ds) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Original data has 0 length");
    return 0;
  }

  if (!*packed) {
    zkMALLOC (p, unsigned char *, ds + sizeof(ds));
  } else {
    if (*psize < (ds + sizeof(uint32_t))) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
           "Insufficient space (%d) allocated for packed data object (%d)",
		    *psize, ds);
      return 0;
    }
    p = (unsigned char *)*packed;
  }
  /* Copy the size of the strfer, followed by the strfer */
  obj = ds;
  
  p[0] = (unsigned char) ((obj & 0xff000000) >> 24);
  p[1] = (unsigned char) ((obj & 0xff0000) >> 16);
  p[2] = (unsigned char) ((obj & 0xff00) >> 8);
  p[3] = (unsigned char) (obj & 0xff);
  
  memcpy(&p[4], data, ds);
  
  *packed = (void *) p;
  *psize = ds + sizeof(ds);

  return 1;
}

DLLExport
int zkDBOUnpackStr (ZkError err, const char *funcname, void **data, size_t *ds,
	      void *packed, size_t *psize) 
{
  static const char *callee = "zkDBOUnpackStr";
  uint32_t size;
  unsigned char *str, *obj;
  unsigned char *p ;

  (void) psize;
  
  
  if (!funcname) funcname = callee;

  if (!packed) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Data objects have null pointers");
    return 0;
  }

  p = (unsigned char *)packed;
  
  size = p[0]*0x1000000 + p[1]*0x10000 + p[2]*0x100 + p[3];

  *psize = size;

  obj = &p[4];
  
  if (!*data) {
    zkMALLOC (str, unsigned char *, size);
    zkDebug("malloc'd %lu bytes", (unsigned long)size);
    *data = (void *)str;
  } else {
    if (*ds < size) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
	     "Insufficient space [%d] allocated for unpacked data object(%lu)",
		    *ds, (unsigned long) size);
      return 0;
    }
    str = (unsigned char *)*data;
  }

  memcpy(str, obj, size);
  *ds = size;

  return 1;
}

#ifndef NDEBUG

DLLExport
int zkDBOTestStr (ZkError err, char *str, size_t len)
{
  static const char *funcname = "zkDBOTestStr";
  unsigned char *dstr = NULL;
  unsigned char *rstr = NULL;
  size_t dlen, rlen;

  zkDebug ("%s: testing TypeStr", funcname);
  
  if (!zkDBOPackStr (err, funcname, str, len,
		    (void **) &dstr, &dlen))
    return 0;

  zkDebug ("%s: (dynstr) packed contents of %d bytes", funcname, dlen);
  zkDumpBuffer(err, dstr, dlen, zkDumpToStream, stderr);  

  if (!zkDBOUnpackStr (err, funcname, (void **) &rstr, &rlen,
	      (void *)dstr, &len))
    return 0;

  zkDebug ("%s: (dynstr) unpacked value using %d bytes (returned %d bytes)",
	   funcname,  (int) rlen, (int) len);
  zkDumpBuffer(err, rstr, rlen, zkDumpToStream, stderr);  
  return 1;
}

#endif
