/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

/*
 * Handle incoming AIP updates
 *
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkUnistd.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkFcntl.h"
#include "zkSysTypes.h"
#include "zkSysStat.h"
#include <assert.h>

#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkBase64.h"
#include "zkDBO.h"
#include "frDBOTypes.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

int main(void)
{

  ZkError err;
  ZkLog log;

  zkDebugOn();	

  assert(zkErrorNew(&err));
  assert(zkErrorLog (err, &log));
  assert(zkLogSetStream (log, stdout));
  assert(zkLogEnable (log));

  if ( ! frDBODriverInitAll(err)) {
    exit (EXIT_FAILURE);
  }

  if ( ! zkDBOTestAll(err)) {
    exit (EXIT_FAILURE);
  }

  if ( ! frDBODriverClearAll(err)) {
    exit( EXIT_FAILURE);
  }

  /* This is suppose to fail */
  if (  frDBODriverClearAll(err)) {
    exit( EXIT_FAILURE);
  }
  
  zkErrorFree(&err);

  return EXIT_SUCCESS;

}
