/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRKEY_H_
#define FRKEY_H_

#ifndef FRCRYPT_H_
#include <frCrypt.h>
#endif /* FRCRYPT_H_ */

#ifndef FRPKTFMT_H_
#include <frPktFmt.h>
#endif /* FRPKTFMT_H_ */

#include <zkError.h>

#ifndef _ZKIP_H
#include <ZKIP.h>
#endif /* _ZKIP_H */

#ifndef ZKIPCLIENT_H_ 
#include <zkipClient.h>
#endif /* ZKIPCLIENT_H_ */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define KSHOST_SZ 64
struct T_FrKeyServer {
 char keyserver[KSHOST_SZ + 1]; /* Name of the remote key server host */
 uint16_t port;                 /* Port on which the key update server runs */
 int timeout;                   /* Timeout for connection */
};

typedef struct T_FrKeyServer FrKeyServer;

#define KSRMESG_SZ 256
struct T_FrKeyServerResponse {
  char message[KSRMESG_SZ + 1]; /* Key update server response message str */
  int code;                     /* 3 digit code, if 0, not returned by server */
};

typedef struct T_FrKeyServerResponse FrKeyServerResponse;

/*
 * Possible key update commands
 */

typedef enum {
  FRKEY_CMD_CREATE ,
  FRKEY_CMD_UPDATE ,
  FRKEY_CMD_REMOVE ,
  FRKEY_CMD_UNUSED 
} FrKeyUpdateCommand;



DLLExport int frKeyCreate (
  ZkError ,                         /* Returns the error if any occured */
  ZkipClient ,
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptPubKey );                  /* Public key to create/update */

DLLExport int frKeyUpdate (
  ZkError ,                         /* Returns the error if any occured */
  ZkipClient ,
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptPubKey );                  /* Public key to create/update */

DLLExport int frKeyCreateSig (
  ZkError ,                         /* Returns the error if any occured */
  FrKeyServer * ,                   /* Target key update server host */
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptSig key );

DLLExport int frKeyCreateCrypt (
  ZkError ,                         /* Returns the error if any occured */
  FrKeyServer * ,                   /* Target key update server host */
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptCrypt key );

DLLExport int frKeyUpdateSig (
  ZkError ,                         /* Returns the error if any occured */
  FrKeyServer * ,                   /* Target key update server host */
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptSig key );

DLLExport int frKeyUpdateCrypt (
  ZkError ,                         /* Returns the error if any occured */
  FrKeyServer * ,                   /* Target key update server host */
  FrKeyServerResponse * ,           /* Returns the server /error response */
  FrCryptSig ,                      /* To sign the key data */
  FrCryptCrypt key );

DLLExport int frKeyServerCommand (
  ZkError err ,
  ZkipClient client,
  int command ,
  FrKeyServerResponse * ksresp ,
  FrCryptSig sig,
  FrCryptPubKey pub );

DLLExport int frKeyServerCommandCore (
  ZkError err ,
  ZkipClient client ,
  FrKeyUpdateCommand command ,
  FrKeyServerResponse * ksresp ,
  char * keyreq ,
  size_t keyreqsz );

DLLExport int frKeyServerConnect (
  ZkError err ,
  FrKeyServer * keyserv ,
  ZkipClient * client );

DLLExport int frKeyServerDisconnect (
  ZkError err ,
  ZkipClient * client );





#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRKEY_H_ */
