/*
 *  ($Id: fr_keyqry.h,v 1.16 2001/05/04 19:50:23 bill Exp $)
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_KEYQRY_H_
#define FR_KEYQRY_H_


#include "zks.h"
#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "frCrypt.h"
#include "frEnt.h"
#include "zkError.h"

#include "fr_keyqry_wire.h" /* bug: for hack below */


/*
 *  Limit the number of version for which we may ask: 1 upto the defined
 *  value.  A key query having a version of 0 is treated as a request for the
 *  current key, whatever its actual version.
 */
#define FR_KEYQRY_KEY_VERSION_MAX 8

/*
 *  Error response codes.
 *  
 *  Zero is unused and should be considered an error.
 */

#define FR_KEYQRY_CODE_BADQUERY 1 /* original query was malformed */
#define FR_KEYQRY_CODE_FOUND    2 /* key is contained in response */
#define FR_KEYQRY_CODE_NOTFOUND 3 /* key was not found */
#define FR_KEYQRY_CODE_SRVERROR 4 /* an error occurred in the server */


/*
 *  Information about the key stored in the cache.
 */
struct cache_key_info {
  FrCryptPubKey key; /* the key itself */
  FrEntIdExp keyOwner;
};

typedef struct fr_keyqry *FrKeyQry;

/*
 *  Fill this in to query for a key.
 */
typedef struct {
  unsigned int keyVersion;
  FrPubKeyType keyType;
  FrEntIdExp keyOwner;
} FrKeyQryQueryExp;

/*
 *  These values indicate the state of a query for a given key.
 */
typedef enum {
  FRKEYQRY_ST_INVALID, /* key is invalid (e.g. bad parent signature) */
  FRKEYQRY_ST_VALID, /* have key, and it's valid */
  FRKEYQRY_ST_NOT_FOUND, /* server couldn't find key */
  FRKEYQRY_ST_AWAITING_RESPONSE, /* sent query, waiting for response */
  FRKEYQRY_ST_AWAITING_VALIDATION /* have key, but not yet validated */
} FrKeyQryState;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int frKeyQryNew(ZkError err, FrKeyQry *kq);
  extern DLLExport int frKeyQryFree(ZkError err, FrKeyQry *kq);

  extern DLLExport int frKeyQryCacheSetMaxEntries(ZkError err, FrKeyQry kq, size_t nelts);
  extern DLLExport int frKeyQryCacheRead(ZkError err, FrKeyQry kq, FrCryptSym sym, FILE *fp);
  extern DLLExport int frKeyQryCacheGetNumWouldWrite(ZkError err, FrKeyQry kq, size_t *nw);
  extern DLLExport int frKeyQryCacheWrite(ZkError err, FrKeyQry kq, FrCryptSym sym, FILE *fp);
  extern DLLExport int frKeyQryCachePrint(ZkError err, FrKeyQry kq, FILE *fp);

  extern DLLExport int frKeyQryCacheClear(ZkError err, FrKeyQry kq);
  extern DLLExport int frKeyQryCachePrune(ZkError err, FrKeyQry kq);

  extern DLLExport int frKeyQryFindKey(ZkError err, FrKeyQry kq, int fd, struct sockaddr *addr, FrKeyQryQueryExp *qry, FrKeyQryState *state, FrCryptPubKey *key);
  extern DLLExport int frKeyQryCacheFindKey(ZkError err, FrKeyQry kq, FrKeyQryQueryExp *qry, FrKeyQryState *state, FrCryptPubKey *key);
  extern DLLExport int frKeyQryRecvKey(ZkError err, FrKeyQry kq, int fd, struct sockaddr *addr, void *pkt, size_t pktLen);

  extern DLLExport int frKeyQryValidateCache(ZkError err, FrKeyQry kq, int fd, struct sockaddr *addr);

  extern DLLExport int frKeyQryRecvQuery(ZkError err, void *pkt, size_t psz, void **q, size_t *qsz);
  extern DLLExport int frKeyQryMakeResp(ZkError err, int code, void *k, size_t ksz, void *d, size_t dsz, void *pkt, size_t *psz);

  extern DLLExport int fr_KeyQryDbFromOwnerKey(ZkError err, int curr, void *q, size_t *qsz, void *d, size_t *dsz, FrCryptPubKey pk);
  extern DLLExport int fr_KeyQryDbDatumFromKey(ZkError err, void *d, size_t *dsz, FrCryptPubKey key);
  extern DLLExport int fr_KeyQryDbQueryFromKey(ZkError err, int curr, void *q, size_t *qsz, FrCryptPubKey key);
  extern DLLExport int fr_KeyQryGetKeyData(ZkError err, void *d, size_t dsz, void **keydata, size_t *ksz);
  extern DLLExport int fr_KeyQrySignDbDatum(ZkError err, FrCryptSig sig, void *d, size_t dsz);
  extern DLLExport int fr_KeyQryVerifyDbDatum(ZkError err, FrCryptSig sig, void *d, size_t dsz);

  extern DLLExport int fr_KeyQryDbKeyPrint(ZkError err, FILE *fp, void *key, size_t ksz);
  extern DLLExport int fr_KeyQryDbDatumPrint(ZkError err, FILE *fp, void *datum, size_t dsz);

  extern DLLExport int valid_parent_sig(ZkError err, 
                                        struct cache_key_info *pki, 
                                        void /* struct fr_keyqry_reply_pkt */ *reply, 
                                        size_t replySz);

  extern DLLExport int key_from_buf(ZkError err, FrCryptPubKey *pk, 
                                    void *buf, size_t bsz);

  
  extern DLLExport int makeQueryPkt(ZkError err, void /* struct fr_keyqry_qry_pkt */ *pkt, FrKeyQryQueryExp *qry);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_KEYQRY_H_ */
