/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_KEYQRY_WIRE_H_
#define FR_KEYQRY_WIRE_H_

/*
 *  This should be included only by libraries that need to know the wire
 *  format of key queries.
 */

#include "frCrypt.h"
#include "frPktFmt.h"
#include "zkIntTypes.h"


#if 0	/* NOTE: Replacing packed structs with calls to zkPackBuf */
#pragma pack(1)
#endif

/*
 *  This structure is the wire format of a `struct fr_keyqry_query'.  It is
 *  also the database key.
 */

/* Bill: keyType is of type FrPubKeyType */
/* Bill: keyOwner is actually an FrEntIdExp.  
   Use functions frEntSetFromStr2 and frEntStore to write it */

#define FR_KEYQRY_MAXKEYVERSION 1

#if 0
struct fr_keyqry_qry {
  uint8_t keyVersion;
  uint8_t keyType;
  char keyOwner[FRENT_PORTABLE_SIZE];
};
#endif

enum {
	FR_KEYQRY_QRY_KEYVERSION	= 0,
	FR_KEYQRY_QRY_KEYTYPE		= FR_KEYQRY_QRY_KEYVERSION + sizeof(uint8_t),
	FR_KEYQRY_QRY_KEYOWNER		= FR_KEYQRY_QRY_KEYTYPE    + sizeof(uint8_t),
	FR_KEYQRY_QRY_SIZEOF		= FR_KEYQRY_QRY_KEYOWNER   + FRENT_PORTABLE_SIZE
};

#define FR_KEYQRY_QRY_PKT_SPEC_SIZE	4

#if 0
struct fr_keyqry_qry_pkt {
  /* Standard packet header. */
  
  char rand[FRCRYPT_RAND_SIZE]; /* currently unused */
  uint8_t vers;
  uint8_t type;
  char spec[FR_KEYQRY_QRY_PKT_SPEC_SIZE]; /* currently unused */

  char /* struct fr_keyqry_qry */ query[FR_KEYQRY_QRY_SIZEOF];
};
#endif

enum {
	FR_KEYQRY_QRY_PKT_RAND		= 0,
	FR_KEYQRY_QRY_PKT_VERS		= FR_KEYQRY_QRY_PKT_RAND  + FRCRYPT_RAND_SIZE,
	FR_KEYQRY_QRY_PKT_TYPE		= FR_KEYQRY_QRY_PKT_VERS  + sizeof(uint8_t),
	FR_KEYQRY_QRY_PKT_SPEC		= FR_KEYQRY_QRY_PKT_TYPE  + sizeof(uint8_t),
	FR_KEYQRY_QRY_PKT_QUERY		= FR_KEYQRY_QRY_PKT_SPEC  + FR_KEYQRY_QRY_PKT_SPEC_SIZE,
	FR_KEYQRY_QRY_PKT_SIZEOF	= FR_KEYQRY_QRY_PKT_QUERY + FR_KEYQRY_QRY_SIZEOF
};

#if 0
struct fr_keyqry_auth {
  char signerSig[FRCRYPT_SIG_SIZE];
  /* bug: signed length used to be here; now in signature buffer */
  char keyOwnerEntity[FRENT_PORTABLE_SIZE];
  char keySignerEntity[FRENT_PORTABLE_SIZE];
};
#endif

enum {
	FR_KEYQRY_AUTH_SIGNERSIG		= 0,
	FR_KEYQRY_AUTH_KEYOWNERENTITY	= FR_KEYQRY_AUTH_SIGNERSIG       + FRCRYPT_SIG_SIZE,
	FR_KEYQRY_AUTH_KEYSIGNERENTITY	= FR_KEYQRY_AUTH_KEYOWNERENTITY  + FRENT_PORTABLE_SIZE,
	FR_KEYQRY_AUTH_SIZEOF			= FR_KEYQRY_AUTH_KEYSIGNERENTITY + FRENT_PORTABLE_SIZE
};

/*
 *  A reply from the key query server will be the following structure,
 *  possibly followed by an fr_keyqry_auth structure (above) and key data.
 */

#define FR_KEYQRY_REPLY_PKT_SPEC_SIZE	4

#if 0
struct fr_keyqry_reply_pkt {
  /* Standard packet header. */
  
  char rand[FRCRYPT_RAND_SIZE]; /* currently unused */
  uint8_t vers;
  uint8_t type;
  char spec[FR_KEYQRY_REPLY_PKT_SPEC_SIZE]; /* currently unused */

  /*
   *  bug: we probably ought to sign responses that claim a given query was
   *  invalid, or that a key was not found.  Such replies may cause changes in
   *  our local cache & we don't want just anyone to be able to do that.
   *  Responses that contain a key need not be signed, as the client will
   *  verify the signature over the key data.
   */

  uint8_t respCode;

  char /* struct fr_keyqry_qry */ origQuery[FR_KEYQRY_QRY_SIZEOF];
};
#endif

enum {
	FR_KEYQRY_REPLY_PKT_RAND		= 0,
	FR_KEYQRY_REPLY_PKT_VERS		= FR_KEYQRY_REPLY_PKT_RAND      + FRCRYPT_RAND_SIZE,
	FR_KEYQRY_REPLY_PKT_TYPE		= FR_KEYQRY_REPLY_PKT_VERS      + sizeof(uint8_t),
	FR_KEYQRY_REPLY_PKT_SPEC		= FR_KEYQRY_REPLY_PKT_TYPE      + sizeof(uint8_t),
	FR_KEYQRY_REPLY_PKT_RESPCODE	= FR_KEYQRY_REPLY_PKT_SPEC      + FR_KEYQRY_REPLY_PKT_SPEC_SIZE,
	FR_KEYQRY_REPLY_PKT_ORIGQUERY	= FR_KEYQRY_REPLY_PKT_RESPCODE  + sizeof(uint8_t),
	FR_KEYQRY_REPLY_PKT_SIZEOF		= FR_KEYQRY_REPLY_PKT_ORIGQUERY + FR_KEYQRY_QRY_SIZEOF
};

#if 0
#pragma pack()
#endif


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_KEYQRY_WIRE_H_ */
