/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_KEYUPD_H_
#define FR_KEYUPD_H_

#include "zks.h"
#include "frCrypt.h"
#if !defined(ZK_CLIENT)
#include "zkDB.h"
#endif
#include "frEnt.h"
#include "zkError.h"
#include "zkLog.h"
#include "fr_keyqry_wire.h" /* bug: for hack below */


/*
 *  If this is set in the `flags' field of the fr_ku_update_pk structure, the
 *  the operation also applies to the current key.  That is, the current key
 *  will become the key contained within the submitted request.
 */
#define FRKU_FLAG_CURRENT_KEY 0x01


typedef enum {
  FRKEYUPD_OP_UNUSED_,
  FRKEYUPD_OP_CREATE,
  FRKEYUPD_OP_REMOVE,
  FRKEYUPD_OP_UPDATE
} FrKeyUpdOp;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int frKeyUpdSignUpdate(ZkError err, FrCryptSig signerPriv, void *data, size_t dsz);
  extern DLLExport int frKeyUpdGenUpdate(ZkError err, FrCryptPubKey key, unsigned int flags, void *data, size_t *dsz);
  extern DLLExport int frKeyUpdGenRemove(ZkError err, FrCryptSig removerPriv, unsigned int flags, FrKeyQryQueryExp *victim, void *data, size_t *dsz);
  extern DLLExport size_t frKeyUpdMinDbDatumSize(void);
  extern DLLExport size_t frKeyUpdMinDbQuerySize(void);
  extern DLLExport int frKeyUpdFromDatum(ZkError err, void *dbuf, size_t dsz, unsigned int flags, void **ubuf, size_t *usz);
  extern DLLExport int frKeyUpdGetKeyParms(ZkError err, void *upd, size_t usz, void **kbuf, size_t *ksz);
  extern DLLExport int frKeyUpdCopyKeyParms(ZkError err, void *upd, size_t usz, void *data, size_t *dsz);
  extern DLLExport int frKeyQryLogQueryPkt(ZkError err, ZkLog log, void *pkt, size_t psz);
  extern DLLExport int frKeyQryLogRespPkt(ZkError err, ZkLog log, void *pkt, size_t psz);
  extern DLLExport int frKeyUpdShortPrint(ZkError err, FILE *fp, int indent, void *data, size_t dsz);
  extern DLLExport int frKeyUpdDbDatumPrint(ZkError err, FILE *fp, void *datum, size_t dsz);
  extern DLLExport int fr_KeyQryDbDatumKeyShortPrint(ZkError err, FILE *fp, void *datum, size_t dsz);
  extern DLLExport int frKeyUpdDbKeyPrint(ZkError err, FILE *fp, void *key, size_t ksz);
  extern DLLExport int frKeyUpdWindozeSux(ZkError err);

  extern DLLExport char *frKeyUpdQuerySnprint(ZkError err, void *q, char *s, size_t ssz);

  /* bug: hack! */
#if !defined(ZK_CLIENT)
  extern DLLExport int fr_KeyUpdRemove(ZkError err, ZkDB kdb, void *q);
#endif

#if !defined(ZK_CLIENT)
  extern DLLExport int frKeyUpdRemove(ZkError err, ZkDB kdb, void *buf, size_t bsz);
  /* bug: following function is a hack for testing. */
  extern DLLExport int fr_KeyUpdDelete(ZkError err, ZkDB kdb, void *q);
  extern int frKeyUpdSubmit(ZkError err, ZkLog log, ZkDB kdb, FrKeyUpdOp op, int allowAny, void *buf, size_t bsz);
  extern DLLExport int frKeyUpdReSignChildren(ZkError err, ZkDB kdb, FrCryptPubKey sig);
  extern DLLExport int frKeyUpdReSignMatching(ZkError err, ZkDB kdb, FrKeyQryQueryExp *qry, FrCryptPubKey sig);
#endif

DLLExport int frKeyUpdGetKey ( ZK_INOUT ZkError err , ZK_IN void * b ,
                               ZK_IN size_t bsz , ZK_OUT FrCryptPubKey * pk );


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_KEYUPD_H_ */
