/* $Id: localAipStructs.h,v 1.19 2001/04/19 20:25:41 alainc Exp $ */
#include "frLocalAip.h"

typedef
struct T_RouterInfo {
  struct sockaddr_in host;      /* Host IP address and port */
  FrCryptSym forwSymKey;        /* Forward symmetric key */
  FrCryptSym backSymKey;        /* Backward symmetric key */
  FrEntIdExp aipEntId;          /* AIPs Entity ID (for key lookups) */
  uint32_t aipId;               /* AIP ID number */
} *RouterInfo;

#if !defined(ZK_KERNEL)
struct T_DH {
  FrPktDifHel dh;               /* Diffie-Hellman Key Exchange */
  int send;                     /* Send state, Send = 1, Recv = 0 */
};
#endif

#if !defined(ZK_KERNEL)
/* 
 * modified Feb. 2000, Frank Foster
 * - move aciCtx from LocalAipCtx to FirstHop object
 */ 
typedef
struct T_FirstHops {
  FrCryptSym symKey[2];
  struct sockaddr_in host;
  int valid;
  size_t used;
  ZkACIContext aciCtx;          /* ACIs available for use */
} *FirstHops;
#endif

typedef 
struct T_RouterTableEntry {
  ZkACI aciIn;                  /* Incoming ACI */
  FrCryptMacKey macKey;         /* MAC for incoming/outgoing data */
  FrCryptSym symKey[2];         /* Link Encryption key to first router */
  int current;                  /* Current version of the symKey */
  uint32_t expiry;              /* Expiry time (actually, seconds until expiration) */
  int valid;                    /* Valid bit */
  ZkACI aciOut;                 /* Outgoing ACI */

  unsigned int prio;            /* Priority of the packets */

                                /* Router List */
  size_t numRouters;
  size_t maxRouters;
  RouterInfo router;            /* Array of routes */
#if !defined(ZK_KERNEL)
  struct T_DH dh;               /* Diffie-Hellman stuff */

  FrCryptSig fhSigKey;          /* First Hop's Public Signing Key */

  struct fr_nym_auth_exp na;    /* Nym Authentication information */
#endif
  ZkACI aciExit;                /* The ACI chosen for the wormhole at the 
                                   exit funnel */
} *RouterTableEntry;

typedef
struct T_RouterTable {
  size_t curRoute;              /* Number of used slots */
  size_t maxRoutes;             /* Maximum size of table */
  RouterTableEntry route;       /* Routes */
#if !defined(ZK_KERNEL)
  FirstHops fh;                 /* First hop addr and symKey */
#endif
} *RouterTable;

struct T_FrLocalAipCtx {
  struct sockaddr_in me;        /* My IP Address & Port */

  struct {                      /* Data handling */
#if !defined(ZK_KERNEL)
    struct {
      FrPktAD   handler;        /* Packet handler */
      size_t    max_size;       /* Maximum possible data packet size */
    } packet;
#endif

    struct {
      unsigned long version;    /* Current data packet's protocol version */    
    } protocol;
  } data;

  struct {                      /* Route handling */
    RouterTable context;        /* Router Table Context */

#if !defined(ZK_KERNEL)
    struct {
      FrPktRC   handler;        /* Packet handler */
      size_t    max_size;       /* Maximum possible rte create packet size */
    } packet;                   
#endif

    struct {
      unsigned long version;    /* Current route create packet's protocol version */    
    } protocol;
  } route;

#if !defined(ZK_KERNEL)
                                /**
                                 * Pointer to key query cache.  Do not
                                 * initialize or free (read-only)
                                 */ 
  FrKeyQry kq;
  FrNetInfo ni;                 /* 
                                 * Network information object. Do not
                                 * initialize or free (read-only)
                                 */  
#endif

  int32_t freedomDelta;         /* Delta Time to the Freedom network time, in seconds */
};
