/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#define DEBUG_SHIM 0
#define _GNU_SOURCE
#include "zkStdio.h"
#include "zkUnistd.h"
#include "zkStdlib.h"
#ifdef ZK_SOLARIS
#include <assert.h>
#include "zkString.h"
#include "zkSysSocket.h"
#include <poll.h>
#include "zkNetinetIn.h"
#include "zkArpaInet.h"
#include "zkNetdb.h"
#include <sys/stropts.h>
#include "zkFcntl.h"
#include "zkSysTime.h"
#include "frCrypt.h"
#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkPacket.h"
#include "zkLog.h"
#include "zkError.h"
#include "zkQueue.h"
#include "frCrypt.h"
#include "frPktFmt.h"
#include "frAips.h"
#include "zkshimlib.h"
#include "frLocalAip.h"
#include "frNDB.h"
#include "test_support.h"

#define FRNETCONFNAME "/home/kent/work/zk/etc/ndb/v5253/frnetconf.ini"
#define ZKSHIM_DEVNAME		"/dev/zkshim"

int main(int argc, char **argv)
{
  /* Local AIP Context */
  FrLocalAipCtx laip;
  ZkError err;
  ZkACI aci;
  char data[BUFSIZ];
  FILE *fptr = NULL;
  struct sockaddr_in firstHop, newHop, recvHost;
  struct pollfd pfds[2];
  size_t i;
  int exitcode;
  int n;
  int nbytes, p;
  char udpbuf[FRPKT_BUF_MAX_SIZE], ipbuf[FRPKT_BUF_MAX_SIZE];
  FrPktType type;
  unsigned int vers;
  int loopcnt = 0;
  int count = 0;
  int waiting = 0;
  ZkACI aciCreate = 0;
  ZkACI aciOut = 0;
  struct timeval now, timeout;

  char rteAlg[15];
  FrCryptSymAlg algLE, algRC;
  int myPort = -1;
  size_t nhops;

  int udpFd = -1, shimFd = -1;
  int sendFd = -1;
  int done = 0;
  char nymName[64];
  int fromLen = sizeof(recvHost);

  FrKeyQry kq;
  struct sockaddr_in keyHost;

#if 0
  char aipNames[][64] = {"lincoln", "gehenna", "mizu", "dogbert",
                         "zimmerman", "jefferson", 0};
#endif
  char aipNames[][64] = {"dogbert", 0};
                         
  int keyFd = -1;

  if (argc != 2) {
    zkFprintf(stderr, "Usage: %s data-directory\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  /* Assuming argv[1] is the data directory */

  zkSnprintf(data, BUFSIZ, "%s/client.data", argv[1]);
  if ((fptr = zkFopen(data, "r")) == NULL) {
    zkFprintf(stderr, "Unable to open data file %s\n", data);
    exit(EXIT_FAILURE);
  }

  if (fscanf(fptr, "%d %s %s", &myPort, rteAlg, nymName) == EOF) {
    zkFprintf(stderr, "Error reading from client data\n");
    exit(EXIT_FAILURE);
  }
  zkFclose(fptr);

  if (!setAlg(rteAlg, &algRC)) {
    zkFprintf(stderr, "Bad parameter for Route Create Pkt algorithm [%s]\n", rteAlg);
    exit(EXIT_FAILURE);
  }  
  assert(zkErrorNew(&err));
  assert(frCryptInit(err, 0));
  { 
	FrNDB ndb;
        time_t timeout;

	assert(frNDBOpen(err, &ndb, FRNETCONFNAME, &timeout));
  }

  assert(frLocalAipCtxNew(err, &laip, 50, 6, myPort));

  assert(frKeyQryNew(err, &kq));
  assert(frLocalAipCtxSetKeyCache(err, laip, kq));
  assert(bindUDPSocket(err, 60050, &keyFd));
  assert(setSockaddrIn(err, "caesar", "60002", "udp", &keyHost));

 redo:
  assert(keyPrefetch(err, kq, keyFd, keyHost, aipNames, 
                     FRCRYPT_KEYTYPE_SIGNING));
  assert(keyPrefetch(err, kq, keyFd, keyHost, aipNames, 
                     FRCRYPT_KEYTYPE_CRYPTION));

  assert(frLocalAipTest(err, laip, 3));
  assert(bindUDPSocket(err, myPort, &udpFd));
  /*  assert(bindTCPSocket(err, myPort + 1, &listenFd));*/

  /* #define ZKSHIM_DEVNAME "/dev/null" */
  if ((shimFd = open(ZKSHIM_DEVNAME, O_RDWR)) < 0) {
    perror(ZKSHIM_DEVNAME);
    zkClose(shimFd);
    zkClose(udpFd);
    exit(EXIT_FAILURE);
  }

#if 1
  /* read mode, message non-discard mode */
  if(ioctl(shimFd, I_SRDOPT, RMSGN) < 0) {
    perror("ioctl I_SRDOPT");
    zkClose(shimFd);
    zkClose(udpFd);
    exit(EXIT_FAILURE);
  }
#endif

  pfds[0].fd = udpFd;
  pfds[0].events = POLLIN;
  
  pfds[1].fd = shimFd;
  pfds[1].events = POLLIN;

 create:                        /* When getting a DESTROY, I'll come here */
  /* ZkError, FrLocalAipCtx, expiry, num hops, first hop sockaddr_in, aci */
  assert(frLocalAipStartNewRoute(err, laip, nymName, 3600, &nhops, &firstHop,
                                 &aci));
  zkFprintf(stderr, "First Hop %s:%d\n", inet_ntoa(firstHop.sin_addr), 
          ntohs(firstHop.sin_port));

  {
    char names[5][64];
    size_t num = 5;
    
    assert(frLocalAipGetAipNames(err, laip, aci, names, &num));
    zkFprintf(stderr, "Number of AIPS %d\n", num);
    
    for (i = 0; i < num; i++) {
      zkFprintf(stderr, "AIP name = %s\n", names[i]);
    }
  }

  assert(frLAReadFirstHopPubSigKey(err, laip, aci));

  {
    FrCryptSig sig = NULL;

    assert(readNymSigKey(err, &sig, nymName, strlen(nymName), argv[1], 
                         strlen(argv[1])));
    
    assert(frLocalAipSetRouteNym(err, laip, aci, nymName, sig));
    assert(frCryptSigFree(err, &sig));
  }

  /* KEY Exchange */ 
  assert(connectTcp(err, firstHop, myPort, &sendFd));

  assert(frLocalAipInitDH(err, laip, aci, sendFd, FRCRYPT_SYMALG_BLOWFISH,
                          myPort));

  done = 0;
  do {
    assert(frLocalAipKeyExchangeDH(err, laip, aci, &done));
  } while (!done);

  assert(frLocalAipGetDHSymKey(err, laip, aci));
  /* Key Exchange */
  zkClose(sendFd);

#if 0
  for (i = 0; i < nhops; i++) {
    assert(frLocalAipGetHostPubKey(err, laip, i, aci, argv[1], filenames[i], BUFSIZ));
  }    
#endif

  /* Set up a Route Create pkt */
  type = FRPKTTYPE_AIP_DATA;
  do {
    n = FRPKT_BUF_MAX_SIZE;
    assert(frLocalAipMakeCreatePkt(err, laip, udpbuf, &n, aci, 0, algRC));
                                   
    assert(frLocalAipLinkEncrypt(err, laip, aci, FRPKTTYPE_AIP_ROUTE_CREATE,
                                 udpbuf, n));
    zkFprintf(stderr, "Attempting to send a ROUTE CREATE pkt\n");
    nbytes = sendto(udpFd, udpbuf, n, 0, (struct sockaddr *) &firstHop,
                    sizeof(firstHop));
    {
      fd_set rset;
      struct timeval timer = { 25, 0}; /* Arbitrary timer of 15 secs */

      FD_ZERO(&rset);
      FD_SET(udpFd, &rset);
      if (frSelectUpdateTimeout(err, ZK_FD_MAXFD1(udpFd), &rset, 0, 0, &timer)) {
        nbytes = recv(udpFd, udpbuf, FRPKT_BUF_MAX_SIZE, 0);
        if (!frLocalAipLinkDecrypt(err, laip, udpbuf, nbytes, firstHop,
                                   &type, &vers, &aci)) {
          continue;
        }
      }
    }
  } while (type != FRPKTTYPE_AIP_ROUTE_CRTACK);
  zkFprintf(stderr, "CREATE Packet ACKed!!!!!!!!!!!!!!!!!!!!!!\n");

  assert(frLocalAipReadCreateAckPkt(err, laip, aci, udpbuf, nbytes, 0));
  aciOut = aci;

  goto redo;

#ifndef INFTIM
#define INFTIM -1
#endif
  while(1) {

    if (count++ > 2000) {
      /* Testing for quitting and memory leaks */
      break;
    }

    p = poll(pfds, 2, INFTIM);
    if(p < 0) {
      perror("poll");
      exitcode = 4;
      break;
    }

    /* data from socket available */
    if ((pfds[0].revents&POLLIN) != 0) {
      n = recvfrom(udpFd, udpbuf, FRPKT_BUF_MAX_SIZE, 0, 
                   (struct sockaddr *)&recvHost, &fromLen);
      if(n < 0) {
        perror("recvfrom");
        break;
      }
      /* zkFprintf(stderr, "received %d bytes from %s on UDP socket\n", n,
         inet_ntoa(hop1sin.sin_addr));
         zkFprintf(stderr, "UDP Packet Returned:\n");
         hex_print(udpbuf, n); */

      if (frLocalAipLinkDecrypt(err, laip, udpbuf, n,
                                recvHost, &type, &vers, &aci)) {
        if (type == FRPKTTYPE_AIP_DATA) {
          i = FRPKT_BUF_MAX_SIZE;
          if (frLocalAipReadDataPkt(err, laip, aci, udpbuf, n,
                                    ipbuf, &i)) {
#if 0
            zkFprintf(stderr, " Decrypted %d bytes from UDP packet\n", i);
            zkFprintf(stderr, " Content: \n");
            hex_print(ipbuf, i);
            zkIPHeaderDump((ZkIPHeader *)ipbuf, stderr); 
            zkFprintf(stderr, " Checksum test please: \n");
#endif

            zkIPHeaderSetDestinationAddress ((ZkIPHeader *)ipbuf,
                                           inet_addr("192.168.1.7"));
            zkTCPHeaderChecksum((ZkTCPHeader *) ipbuf);
#if 0
            zkIPHeaderDump((ZkIPHeader *)ipbuf, stderr);
            zkTCPHeaderDump((ZkIPHeader *)ipbuf, stderr);
#endif

#if DEBUG_SHIM
            zkFprintf(stderr, " <<<---- Writing %d to the shim\n", i);
#endif
            n = zkWrite(shimFd, ipbuf, i);
            if( n < 0) {
              perror("write");
              break;
            }
          }
        } else if (type == FRPKTTYPE_AIP_ROUTE_DESTROY) {
          zkFprintf(stderr, "Route Destroyed\n");
          assert(frLocalAipDeleteRoute(err, laip, aci));
          if (aci == aciOut) {
            goto create;
          }
        } else if (type == FRPKTTYPE_AIP_ROUTE_CRTACK) {
          zkFprintf(stderr, "Link Decrypted - ACI is %d", aci);
          zkFprintf(stderr, " - type is FRPKTTYPE_AIP_ROUTE_CRTACK\n");
          assert(frLocalAipReadCreateAckPkt(err, laip, aci, udpbuf, 
                                            nbytes, 1));
          zkFprintf(stderr, "Updated route\n");
          aci = aciCreate;
          waiting = 0;
          loopcnt = 0;
          aciOut = aci;
          firstHop = newHop;
        } else {
          zkFprintf(stderr, "Unhandled packet format\n");
        }
      } else {
        zkFprintf(stderr, "Link Decryption failed from %s:%d\n", 
                inet_ntoa(recvHost.sin_addr), ntohs(recvHost.sin_port));
      }
    }

    /* data from shim available */
    if ((pfds[1].revents&POLLIN) != 0) { 
      n = zkRead(shimFd, ipbuf, sizeof(ipbuf));
      if(n < 0) {
        perror("read");
        break;
      }

#if DEBUG_SHIM
      zkFprintf(stderr, "received %d bytes from shim\n", n);
#endif
#if 0
      zkFprintf(stderr, "IP Packet Returned:\n");
      hex_print(ipbuf, n); 
#endif
      i = sizeof(ipbuf);
#if 0
      zkIPHeaderDump((ZkIPHeader *)ipbuf, stderr);
      zkTCPHeaderChecksum((ZkTCPHeader *) ipbuf);
      zkIPHeaderDump((ZkIPHeader *)ipbuf, stderr);
      zkFprintf(stderr, " Checksum test please: \n");
#endif

      zkIPHeaderSetSourceAddress ((ZkIPHeader *)ipbuf, 0);

#if 0
      zkIPHeaderDump((ZkIPHeader *)ipbuf, stderr);
      zkTCPHeaderDump((ZkIPHeader *)ipbuf, stderr);
#endif
      i = FRPKT_BUF_MAX_SIZE;
      if (frLocalAipMakeDataPkt(err, laip, aciOut, ipbuf, n, udpbuf, &i)) {
#if 0
        zkFprintf(stderr, " Generated %d bytes for Data packet\n", i);
        zkFprintf(stderr, " Content: \n");
        hex_print(udpbuf, i);
#endif
        /* Now send the UDP packet via the cloud */
	
#if 0
        zkFprintf(stderr, "Sending --->> ");
#endif
        assert(frLocalAipLinkEncrypt(err, laip, aciOut, FRPKTTYPE_AIP_DATA, 
                                     udpbuf, i));
        nbytes = sendto(udpFd, udpbuf, i, 0, (struct sockaddr *) &firstHop,
                        sizeof(firstHop));
      }
    }
    if ((loopcnt++ > 1000) || waiting) {

      if (waiting) {
        if (gettimeofday(&now, 0) == -1) {
          zkFprintf(stderr, "gettimeofday() failed\n");
        }
        if (now.tv_sec < timeout.tv_sec + 30) {
          continue;
        } else {
          frLocalAipDeleteRoute(err, laip, aciCreate);
        }
      }

      zkFprintf(stderr, "Changing route\n");
      /* ZkError, FrLocalAipCtx, expiry, num hops, hop sockaddr_in, aci */
      assert(frLocalAipNewRouteReuseExit(err, laip, 3600, &nhops, &newHop, 
                                         &aciCreate, aci));
      zkFprintf(stderr, "First Hop %s:%d\n", inet_ntoa(newHop.sin_addr), 
              ntohs(newHop.sin_port));

      {
        FrCryptSig sig = NULL;
        
        assert(readNymSigKey(err, &sig, nymName, strlen(nymName), argv[1], 
                             strlen(argv[1])));
    
        assert(frLocalAipSetRouteNym(err, laip, aciCreate, nymName, sig));
        assert(frCryptSigFree(err, &sig));
      }
      assert(frLAReadFirstHopPubSigKey(err, laip, aciCreate));

      /* KEY Exchange */ 
      assert(connectTcp(err, newHop, myPort, &sendFd));

      assert(frLocalAipInitDH(err, laip, aciCreate, sendFd, 
                              FRCRYPT_SYMALG_BLOWFISH, myPort));
      done = 0;
      do {
        assert(frLocalAipKeyExchangeDH(err, laip, aciCreate, &done));
      } while (!done);

      assert(frLocalAipGetDHSymKey(err, laip, aciCreate));
      /* Key Exchange */
      zkClose(sendFd);

#if 0
      for (i = 0; i < nhops; i++) {
        assert(frLocalAipGetHostPubKey(err, laip, i, aciCreate, argv[1], 
                                       filenames[i], BUFSIZ));
      }    
#endif

      /* Set up a Route Create pkt */
      type = FRPKTTYPE_AIP_DATA;
      n = FRPKT_BUF_MAX_SIZE;
      assert(frLocalAipMakeCreatePkt(err, laip, udpbuf, &n, aciCreate, 1,
                                     algRC));
      
      assert(frLocalAipLinkEncrypt(err, laip, aciCreate,
                                   FRPKTTYPE_AIP_ROUTE_CREATE, udpbuf, n));
      zkFprintf(stderr, "Attempting to send a ROUTE CREATE pkt\n");
      nbytes = sendto(udpFd, udpbuf, n, 0, (struct sockaddr *) &newHop,
                      sizeof(newHop));
      waiting = 1;
      if (gettimeofday(&timeout, 0) == -1) {
        zkFprintf(stderr, "gettimeofday() failed");
      }
    }
  }
  zkClose(shimFd);
  zkClose(udpFd);
  assert(frLocalAipCtxFree(err, &laip));
  assert(frKeyQryFree(err, &kq));
  assert(zkErrorFree(&err));

  return 0;
}
#else
int main(int argc, char **argv)
{
  zkFprintf(stderr, "This program does nothing on non-SOLARIS platforms\n");
  exit(EXIT_SUCCESS);
}
#endif
