/*
 * $Id: frlode.h,v 1.5 2001/04/19 20:26:02 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRLODE_H_
#define FRLODE_H_

#include "zkList.h"
#include <sys/socket.h>

#ifndef _ZKIP_H
#include "ZKIP.h"
#endif /* _ZKIP_H */

#ifndef ZKIPCLIENT_H_
#include "zkipClient.h"
#endif /* ZKIPCLIENT_H_ */

#include "zkTypes.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


DLLExport int
frLodeConnect(
    ZK_INOUT ZkError      err,     /* Returns the error if any occured */
    ZK_IN    const char * host,    /* Lode Server host name */
    ZK_IN    uint16_t     port,    /* Lode server port number */
    ZK_IN    uint16_t     timeout, /* Lode server connection timeout */
    ZK_OUT   ZkipClient * client   /* Allocates and returns a new ZKIP client struct */
);

DLLExport int
frLodeDisconnect(
    ZK_INOUT ZkError      err,    /* Returns the error if any occured */
    ZK_IN    ZkipClient * client  /* The ZKIP client to disconnect (ptr reset to NULL)*/
);

DLLExport int
frLodeLogin(
    ZK_INOUT ZkError      err,       /* Returns the error if any occured */
    ZK_IN    ZkipClient   client,    /* The ZKIP client structure connected to Lode */
    ZK_IN    const char * username,  /* The user name */
    ZK_IN    const char * passData,  /* pointer to passphrase data */
    ZK_IN    size_t       passSize,  /* size of passphrase data */
    ZK_OUT   void **      rtnData,   /* Allocates and returns a pointer to the data */
    ZK_OUT   size_t *     rtnSize    /* Returns the size of the data */
);

DLLExport int
frLodeLogout(
    ZK_INOUT ZkError    err,      /* Returns the error if any occured */
    ZK_IN    ZkipClient client,   /* The ZKIP client structure connected to Lode */
    ZK_OUT   void **    rtnData,  /* Allocates and returns a pointer to the data */
    ZK_OUT   size_t *   rtnSize   /* Returns the size of the data */
);

DLLExport int
frLodeChangePassphrase(
    ZK_INOUT ZkError      err,       /* Returns the error if any occured */
    ZK_IN    ZkipClient   client,    /* The ZKIP client structure connected to Lode */
    ZK_IN    const char * passData,  /* pointer to new passphrase data */
    ZK_IN    size_t       passSize,  /* size of new passphrase data */
    ZK_OUT   void **      rtnData,   /* Allocates and returns a pointer to the data */
    ZK_OUT   size_t *     rtnSize    /* Returns the size of the data */
);

DLLExport int
frLodeService(
    ZK_INOUT ZkError       err,        /* Returns the error if any occured */
    ZK_IN    ZkipClient    client,     /* The ZKIP client structure connected to Lode */
    ZK_IN    const char *  reason,     /* Reason for command */
    ZK_IN    const char *  service,    /* The service name */
    ZK_IN    const char *  operation,  /* The operation name */
    ZK_IN    int           paramSize,  /* number of extra parameters */
    ZK_IN    const char ** paramVect,  /* extra parameters for the specified operation */
    ZK_IN    size_t        extraSize,  /* size of authentication data */
    ZK_IN    const char *  extraData,  /* pointer to extra data */
    ZK_OUT   void **       rtnData,    /* Allocates and returns a pointer to the data */
    ZK_OUT   size_t *      rtnSize     /* Returns the size of the data */
);

DLLExport int
frLodeCheckAccess(
    ZK_INOUT ZkError       err,        /* Returns the error if any occured */
    ZK_IN    ZkipClient    client,     /* The ZKIP client structure connected to Lode */
    ZK_IN    const char *  service,    /* The service name */
    ZK_IN    const char *  operation,  /* The operation name */
    ZK_IN    int           paramSize,  /* number of extra parameters */
    ZK_IN    const char ** paramVect,  /* extra parameters for the specified operation */
    ZK_IN    size_t        extraSize,  /* size of authentication data */
    ZK_IN    const char *  extraData,  /* pointer to extra data */
    ZK_OUT   void **       rtnData,    /* Allocates and returns a pointer to the data */
    ZK_OUT   size_t *      rtnSize,    /* Returns the size of the data */
    ZK_OUT   ZkBool *      hasAccess   /* Returns true if the access is granted */
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRLODE_H_ */
 
