/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

# include "frMailAccount.h"
# include "zkStdio.h"
# include "zkString.h"


static void print_error( const char * const prog, ZkError error )
{
    char message[1024];

    zkErrorMessage(error, message, sizeof(message));

    fprintf( stderr, "%s: %s\n", prog, message );
}

int main(int argc, const char * argv[])
{
    char         path[1024];
    ZkError      error  = NULL;
    const char * prog   = strrchr(argv[0], '/');
    int          result = -1;

    prog = (prog == NULL) ? argv[0] : (prog + 1);    

    if (argc != 3)
    {
        fprintf( stderr,
                 "%s: root directory and user name must be only arguments " \
                 "on command line\n",
                 prog );
    }
    else if (!zkErrorNew(&error))
    {
        fprintf( stderr,
                 "%s: failed to create error object",
                 prog );
    }
    else if (!frMailAccount_GetPath( error,
                                     argv[1],
                                     argv[2],
                                     path,
                                     sizeof(path)))
    {
        print_error(prog, error);
    }
    else
    {
        fprintf( stdout, "%s%c", path, '\0');
        result = 0;
    }

    if (error != NULL)
    {
        zkErrorFree(&error);
    }

    if (result != 0)
    {
        fprintf( stdout, "/dev/null");
    }

    return (result);
}
