/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

# include "frMailAccount.h"
# include "zkStdlib.h"
# include "zkStdio.h"
# include "zkUnistd.h"
# include "zkString.h"

int main(int argc, const char * argv[])
{
    ZkError       error       = NULL;
    ZkBool        has_tagline = ZK_TRUE;
    const char *  prog        = strrchr(argv[0], '/');
    char *        dir         = getcwd(NULL,0);

    prog = (prog == NULL) ? argv[0] : prog + 1;

    if (argc != 1)
    {
        fprintf(stderr, "%s: I don't take any parameters!\n", prog);
    }
    else if (dir == NULL)
    {
        fprintf(stderr, "%s: can't get current working directory!\n", prog);
    }
    else if (!zkErrorNew(&error))
    {
        fprintf(stderr, "%s: can't allocate error object!\n", prog);
    }
    else if (!frMailAccount_HasTagline( error, dir, &has_tagline ))
    {
        char message[1024];
        zkErrorMessage(error, message, sizeof(message));
        fprintf(stderr, "%s: %s\n", prog, message);
    }
    else
    {
        fprintf(stdout, "%s\n", has_tagline ? "on" : "off");
    }

    /*
     * Cleanup
     */

    if (dir != NULL)
    {
        free(dir);
    }

    if (error != NULL)
    {
        zkErrorFree(&error);
    }

    return (0);
}
