/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

# include "frMailAccount.h"
# include <stdio.h>
# include <stdlib.h>
# include <readline/readline.h>

int main(int argc, char * argv[])
{
    ZkBool  is_valid = ZK_FALSE;
    ZkError error   = NULL;
    char *  string  = 0;

    if (!zkErrorNew(&error))
    {
        fprintf(stderr, "%s: cannot allocate error object\n", argv[0]);
        goto cleanup;
    }

    string = readline("Name: ");
    if (string == NULL)
    {
        fprintf(stderr, "%s: failed to read name\n", argv[0]);
        goto cleanup;
    }

    if (!frMailAccount_NameIsValid(error, string, &is_valid))
    {
        char message[1024];
        zkErrorMessage(error, message, sizeof(message)-1);
        fprintf(stderr, "%s: %s\n", argv[0], message);
        goto cleanup;
    }

    if (argc == 1)
    {
        fprintf( stdout, "%svalid\n", is_valid ? "" : "not " );
    }

cleanup:

    if (string != NULL)
    {
        free(string);
    }

    if (error)
    {
        zkErrorFree(&error);
    }
    
    return (is_valid);
}
