// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <stdio.h>
#include <pwd.h>
#include <errno.h>
#include <unistd.h>
#include <grp.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <string>
#include <vector>
#include <iostream>
#include <strstream>
#include <fstream>

#include "entry.hh"

bool Entry::exists(const string &str) {
  struct stat st;
  return (stat(str.c_str(), &st) == 0); 
}

string Entry::pathcat(const string &root, const string &newelem) {
  return (root + "/") + newelem;
}

void Entry::fail(const char *msg, const string &specifics) {
  cerr << "-ERR " << msg << " " << specifics << endl;
  _exit(100);
}

Dir::Dir(const char *ch, int m = 0700) : name_(ch), mode_(m) {}

void Dir::eraseFrom(const string &root) {
  string newpath = pathcat(root, name_);
  for (vector<Entry *>::iterator i = sub.begin(); i != sub.end(); ++i) {
    (*i)->eraseFrom(newpath.c_str());
  }
  if (exists(newpath)) 
    if (rmdir(newpath.c_str()) != 0)
      fail("removing directory", newpath);
}

void Dir::createIn(const string &root) {
  string newroot = pathcat(root, name_);
  if (mkdir(newroot.c_str(), mode_) != 0) fail("making directory", newroot);
  for (vector<Entry *>::iterator i = sub.begin(); i != sub.end(); ++i) {
    (*i)->createIn(newroot);
  }
}

Dir *Dir::add(Entry *e) {
  sub.push_back(e);
  return this;
}

bool Dir::move(const string &srcdir, const string &target) {
  string path = pathcat(srcdir.c_str(), name_);
  if (rename(path.c_str(), target.c_str()) != 0)
    return exists(target);
  return true;
}


CopyOfFile::CopyOfFile(const char *from, const char *to) : from_(from), to_(to) {}  

void CopyOfFile::eraseFrom(const string & root) {
  string newpath = pathcat(root, to_);
  if (exists(newpath)) unlink(newpath.c_str());
}

void CopyOfFile::createIn(const string &root) {
  ifstream from(from_.c_str());
  if (!from) fail("creating ofile", from_);
  string newpath = pathcat(root, to_);
  ofstream to(newpath.c_str());
  if (!to) fail("creating ifile", to_) ;
  char c;
  while(from.get(c)) to.put(c);     
}

