/*
 * $Id$
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 

 * Original author: Philippe Lachlan McLean 2 November 2000
 * Inspired by genpasswd.
 * Dumps the contents of a base64 encoded mailcert to stdout.
 * Copyright (c) 2000 Zero-Knowledge Systems Inc.
 */

#include <string>
#include <iostream>
#include <stdio.h>
#include <pwd.h>
#include <errno.h>
#include <unistd.h>
#include <grp.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>

#include <zkTypes.h>
#include <zkIntTypes.h>
#include <zkErrdef.h>
#include <zkError.h>
#include <frMailAuth.h>

#include <zkBase64.h>
#include "zkCStr.h"

#include "auth-common.hh"

using namespace std;

static ZkError        err_; // this is a straight-line program 
extern int            errno;
extern char       **  environ;  // "const" clashes with unistd.h

static void dumpCert(FrMailAuth cert)
{
  FrEntIdExp entity;
  size_t size = 0;
  uint16_t version = 0;
  ZkClock expiration;
  uint8_t nym_type = 0;
  uint32_t mail_limit = 0;
  uint32_t volume_limit = 0;
  uint32_t size_limit = 0;
  uint32_t cross_limit = 0;

  frMailAuthGetEnt(err_, cert, &entity);
  frMailAuthGetEntSize(err_, cert, &size);
  frMailAuthGetExpiration(err_, cert, &expiration);
  frMailAuthGetVersion(err_, cert, &version);
  frMailAuthGetNymType(err_, cert, &nym_type);
  frMailAuthGetMailLimit(err_, cert, &mail_limit);
  frMailAuthGetVolLimit(err_, cert, &volume_limit);
  frMailAuthGetSizeLimit(err_, cert, &size_limit);
  frMailAuthGetCrossLimit(err_, cert, &cross_limit);
    
  printf("entity.type=%d entity.name=%s size=%d version=%d expiration=%lld nym-type=%d mail-limit=%d volume-limit=%d size-limit=%d crosspost-limit=%d\n",
	 entity.type,
	 entity.name,
	 size,
	 version,
	 zkClockToSeconds(expiration),
	 nym_type,
	 mail_limit,
	 volume_limit,
	 size_limit,
	 cross_limit);
}

int main(int argc, char **argv)
{
  FrMailAuth cert = NULL;

  if (argc != 2)
    {
      fprintf(stderr, "usage: %s <base64 encoded mailcert>\n", argv[0]);
      exit(-1);
    }

  string pass(argv[1]);

  unBase64(err_, pass);

  zkErrorNew (&err_);    

  frMailAuthNewFromBuf(err_, &cert, const_cast<char *>(pass.c_str()), pass.size()); 
     
  dumpCert(cert);
}
 
