// $Id: genpasswd.cpp,v 1.5 2000/11/23 15:13:02 marcp Exp $
// 
// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

//

// Generates the mail certificate for a nym
//


#include <zkTypes.h>
#include <zkIntTypes.h>
#include <zkErrdef.h>
#include <fstream>
#include <string>
#include <iostream>
#include <stdlib.h>
#include "zkCStr.h"

#include "auth-common.hh"

#include <unistd.h>

#define _GNU_SOURCE
#include <getopt.h>

static ZkError err_; // this is a straight-line program 

#define ensure(x) if(!x) {cerr << "died in library call" << endl; exit(1);}

static void usage()
{
      cerr << "usage: genpasswd <nymname> [--mail-limit n]" << endl 
           << "                           [--crosspost-limit n]" << endl
           << "                           [--volume-limit n]" << endl
           << "                           [--size-limit n]" << endl
           << "                           [--nym-type n]" << endl
           << "                           [--expiration n]" << endl
           << "                           [--version n]" << endl
           << "                           [--mailmackeyfile filename]" << endl
	   << endl
           << "       where n is an integer. For mail-limit, volume-limit, " << endl
           << "       and size-limit, n is kilobytes. For expiration, n is seconds." << endl
           << endl;

      _exit(1);
}

int main(int argc, char **argv) {
  
  FrMailAuth cert;
  FrCryptMacKey mkey;


  FrEntIdExp ent;

  char* qmailCtlDir = (char*) malloc(sizeof(char) * (strlen(qmailCtlDirDefault)+1));
  zkCStrlcpy(qmailCtlDir, qmailCtlDirDefault, strlen(qmailCtlDirDefault)+1);

  string mailMacKeyFile(string(qmailCtlDir) + "/mailmackey");  

  zkErrorNew (&err_);

  int yearInSeconds = 3600 * 24 * 365;
  int mailLimit = 250; // 250 a day
  int volLimit = 1024;
  int sizeLimit = 3072;
  int crossPostLimit = 5;
  int nymType = 1;
  char *buf = 0;
  size_t bsz = 0;
  ZkClock exp = zkClockAdd( zkClockNow(), zkClockFromSeconds(yearInSeconds) );
  int version = 1;

  for (;;)
    {
      int c;
      int option_index = 0;
      static struct option long_options[] =
      {
        {"mail-limit", 1, 0, 0},
        {"crosspost-limit", 1, 0, 0},
        {"volume-limit", 1, 0, 0},
        {"size-limit", 1, 0, 0},
        {"nym-type", 1, 0, 0},
        {"expiration", 1, 0, 0},
        {"version", 1, 0, 0},
        {"mailmackeyfile", 1, 0, 0},
        {0, 0, 0, 0}
      };

      c = getopt_long (argc, argv, "",
                       long_options, &option_index);
      if (c == -1)
        {
          break;
        }

      switch (c)
        {
        case 0:
          {
            const char *name = long_options[option_index].name;
            if (strcmp(name, "mail-limit") == 0) 
              {
                mailLimit = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "crosspost-limit") == 0)
              {
                crossPostLimit = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "volume-limit") == 0)     
              {
                volLimit = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "size-limit") == 0)       
              {
                sizeLimit = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "nym-type") == 0)         
              {
                nymType = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "expiration") == 0)       
              {
                exp = zkClockFromSeconds(strtoull(optarg, 0, 0));
              }
            else if (strcmp(name, "version") == 0)          
              {
                version = strtoul(optarg, 0, 0);
              }
            else if (strcmp(name, "mailmackeyfile") == 0)          
              {
		mailMacKeyFile = optarg;
              }
          }
          break;

        default:
          {
	    usage();
          }
        }
    }

  if (optind != argc - 1)
    { 
      usage();
    }

  string nym(argv[optind]);

  loadMacKey(err_, mkey, mailMacKeyFile.c_str());
  nym2Entity(err_, nym,ent);

  /* generate the mail certificate */
  ensure(frMailAuthNew(err_, &cert));  dieOnError(err_, "frMailAuthNew");

  /* set certificate parameters */
  ensure(frMailAuthSetEnt(err_, cert, &ent)); dieOnError(err_, "frMailAuthSetEnt");
  ensure(frMailAuthSetVersion(err_, cert, version)); dieOnError(err_, "frMailAuthSetVersion");
  ensure(frMailAuthSetExpiration(err_, cert, exp)); dieOnError(err_, "frMailAuthSetExpiration");
  ensure(frMailAuthSetNymType(err_, cert, nymType)); dieOnError(err_, "frMailAuthSetNymType");
  ensure(frMailAuthSetMailLimit(err_, cert, mailLimit)); dieOnError(err_, "frMailAuthSetMailLimit");
  ensure(frMailAuthSetVolLimit(err_, cert, volLimit)); dieOnError(err_, "frMailAuthSetVolLimit");
  ensure(frMailAuthSetSizeLimit(err_, cert, sizeLimit)); dieOnError(err_, "frMailAuthSetSizeLimit");
  ensure(frMailAuthSetCrossLimit(err_, cert, crossPostLimit)); dieOnError(err_, "frMailAuthSetCrossLimit");

  /* create the mail cert */
  ensure(frMailAuthMac(err_, cert, mkey)); dieOnError(err_, "frMailAuthMac");
  ensure(frMailAuthToBuf(err_, cert, (void **)(&buf), &bsz)); dieOnError(err_, "frMailAuthToBuf");

  string pass(buf,bsz);
  Base64(err_, pass);
  cout << pass << endl;

  if (buf) { free(buf); }
  ensure(frMailAuthFree(err_, &cert));
  ensure(frCryptMacKeyFree(err_, &mkey));
  ensure(zkErrorFree(&err_));
  
  return 0; /* to shut up insure */
}



