// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <pwd.h>
#include <sys/types.h>
#include <fstream>
#include <iostream>

#include "auth-common.hh"
#include "entry.hh"

// this is a drop-in replacement for qmail-getpw, which yields the uname,
// location and uid/gid of a user in our mail system, as required by
// qmail-lspawn. in our case, since we generate user accounts "on the fly", all
// we need to do is check that ensure-user has already created the directory for
// us, and return the appropriate user,uid,gid,location,dash,ext tuple.

int main (int argc, char **argv) {

  if (argc != 2) {
    cerr << "usage: qmail-genpw <username>" << endl;
    exit(100);
  }
  string nym(*++argv);


  if (nym.rfind("@") != string::npos) {
    nym = nym.substr(0,nym.rfind("@"));
  }

  ZkError err_; 
  zkErrorNew (&err_);

  string uname("alias"); // all mailboxes owned by 1 uid 
  string mailroot(slurpConfigFile("mailroot")); // based around a single mount point
  string home = calculateUserDir(err_,mailroot,nym);
  if (!Entry::exists(home)) _exit(100);

 
  struct passwd *pw = getpwnam(uname.c_str());
  if (!pw) {
    _exit(100);
  }

  cout << uname;
  cout.put('\0');
  cout << pw->pw_uid;
  cout.put('\0');
  cout << pw->pw_gid;
  cout.put('\0');
  cout << home;
  cout.put('\0');
  // "dash"
  cout.put('\0');
  // "ext"
  cout.put('\0');
  return 0;
}
