/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *   
 *   Gives an example on how to create and verify a mail certificate
 *
 */



#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frMailAuth.h"


int main() {
  
  ZkError err;
  
  FrMailAuth cert, cert2;
  FrCryptMacKey mkey;
  FrEntIdExp ent;
  ZkClock clk, clk2;
  
  char macKeyBuf[FRCRYPT_MACKEY_SIZE];
  
  int i = 0;
  char *buf;
  size_t bsz;
  
  int ok = 0;

  assert(zkErrorNew(&err));

  
  /* generate the owner entity */
  frEntSetFromStr2(err, &ent, "nym", "Alice");
  
  /* generate a mac key */
  assert(frCryptMacKeyNew(err, &mkey));
  memset(mkey, 0, sizeof mkey);
  for (i = 0; i < sizeof macKeyBuf; i++) { macKeyBuf[i] = i + 1; }
  assert(frCryptMacKeyLoad(err, mkey, macKeyBuf, sizeof macKeyBuf));
 

  /* 
   *   Creation
   */
  
  clk = zkClockNow();
  clk2 = zkClockFromSeconds(3600);
  clk = zkClockAdd(clk,clk2);
  
  /* generate the mail certificate */
  assert(frMailAuthNew(err, &cert));  
  assert(frMailAuthSetEnt(err,cert,&ent));
  assert(frMailAuthSetVersion(err,cert,1));
  assert(frMailAuthSetExpiration(err,cert,clk));
  assert(frMailAuthSetNymType(err,cert,1));
  assert(frMailAuthSetMailLimit(err,cert,1234));
  assert(frMailAuthSetVolLimit(err,cert,2345));
  assert(frMailAuthSetSizeLimit(err,cert,3456));
  assert(frMailAuthSetCrossLimit(err,cert,56));
  assert(frMailAuthMac(err, cert, mkey));

  assert(frMailAuthToBuf(err, cert, (void **)&buf, &bsz));


  /* 
   *   Verification
   */

  assert(frMailAuthNewFromBuf(err, &cert2, buf, bsz));
  assert(frMailAuthCheckOwnerAndMac(err, cert2, &ent, mkey, &ok));
  assert(ok);
  assert(frMailAuthCheckExpiration(err, cert2, zkClockNow(), &ok));
  assert(ok);

  /*
   *  Clean up.
   */
  assert(frMailAuthFree(err, &cert));
  assert(frMailAuthFree(err, &cert2));
  assert(frCryptMacKeyFree(err, &mkey));
  assert(zkErrorFree(&err));

  exit(EXIT_SUCCESS);
  return 0; /* to shut up insure */
}



