/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *
 *         Test the mail authentication library
 *
 */


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "frMailAuth.h"


int main() {
  
  ZkError err;
  
  FrMailAuth cert1,cert2;
  FrCryptMacKey mkey;
  FrEntIdExp ent1;
  ZkClock clk;

  char macKeyBuf[FRCRYPT_MACKEY_SIZE];
  
  int i = 0;
  int ok = 0;
  char *buf1, *buf2;
  size_t bsz1, bsz2;
  uint16_t vers2;

  assert(zkErrorNew(&err));
  
  /* generate a mac key */
  assert(frCryptMacKeyNew(err, &mkey));
  memset(mkey, 0, sizeof mkey);
  for (i = 0; i < sizeof macKeyBuf; i++) { macKeyBuf[i] = i + 1; }
  assert(frCryptMacKeyLoad(err, mkey, macKeyBuf, sizeof macKeyBuf));
  
  assert(frMailAuthNew(err, &cert1));

  /* set the owner entity */
  frEntSetFromStr2(err, &ent1, "nym", "Bozo");

  clk = zkClockAdd( zkClockNow(), zkClockFromSeconds(3600) ); 
  
  /* set fields */
  assert(frMailAuthSetEnt(err,cert1,&ent1));
  assert(frMailAuthSetVersion(err,cert1,1));
  assert(frMailAuthSetExpiration(err,cert1,clk));
  assert(frMailAuthSetNymType(err,cert1,1));
  assert(frMailAuthSetMailLimit(err,cert1,200));
  assert(frMailAuthSetVolLimit(err,cert1,2000));
  assert(frMailAuthSetSizeLimit(err,cert1,500));
  assert(frMailAuthSetCrossLimit(err,cert1,25));
  
  assert(frMailAuthToBuf(err, cert1, (void **)&buf1, &bsz1));
  
  /* new cert from previous one */
  assert(frMailAuthNewFromBuf(err, &cert2, buf1, bsz1));
  assert(frMailAuthGetVersion(err,cert2,&vers2));
  assert(vers2 == 1);

  assert(frMailAuthToBuf(err, cert2, (void **)&buf2, &bsz2));  
  
  
  /* buf 1 & 2 should be identical */
  assert(bsz1 == bsz2);
  assert( ! memcmp(buf1, buf2, bsz1));
  
  assert(frMailAuthMac(err, cert2, mkey));
  
  assert(frMailAuthCheckMac(err, cert2, mkey, &ok));
  assert(ok);
  assert(frMailAuthCheckOwner(err, cert2, &ent1, &ok));
  assert(ok);
  assert(frMailAuthCheckOwnerAndMac(err, cert2, &ent1, mkey, &ok));
  assert(ok);
  assert(frMailAuthCheckExpiration(err, cert2, zkClockNow(), &ok));
  assert(ok);
  
  
  /* if expired */
  clk = zkClockAdd(clk,clk);
  assert(frMailAuthCheckExpiration(err, cert2, clk, &ok));
  assert(!ok);

  /* if cert changed */
  assert(frMailAuthSetVersion(err,cert2,2));
  assert(frMailAuthCheckMac(err, cert2, mkey, &ok));
  assert(!ok);
  
  /*
   *  Clean up.
   */
  assert(frMailAuthFree(err, &cert1));
  assert(frMailAuthFree(err, &cert2));
  assert(frCryptMacKeyFree(err, &mkey));
  assert(zkErrorFree(&err));
  if (buf1) {free(buf1);}
  if (buf2) {free(buf2);}

  exit(EXIT_SUCCESS);
  return 0; /* to shut up insure */
}



