/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

# ifndef frMailBlockList_server_h_INCLUDED
# define frMailBlockList_server_h_INCLUDED

# include "zkError.h"
# include "zks.h"

# define FRMAIL_BLOCK_SENDER           (0x00000001)
# define FRMAIL_BLOCK_RECIPIENT        (FRMAIL_BLOCK_SENDER << 1)
# define FRMAIL_BLOCK_COMBINATIONS     (FRMAIL_BLOCK_SENDER << 2)

# ifdef __cplusplus
extern "C" {
# endif

typedef struct T_frMailBlockList * FrMailBlockList;

/*
 * The functions that most people will be interested in
 */

DLLExport int frMailBlockList_Open(
    ZK_INOUT ZkError            error,
    ZK_OUT   FrMailBlockList *  blocklist,
    ZK_IN    const char * const path );

DLLExport int frMailBlockList_OpenFD(
    ZK_INOUT ZkError           error,
    ZK_OUT   FrMailBlockList * blocklist,
    ZK_IN    int               descriptor );

DLLExport int frMailBlockList_Close(
    ZK_INOUT ZkError            error,
    ZK_OUT   FrMailBlockList *  blocklist );

DLLExport int frMailBlockList_Check(
    ZK_INOUT ZkError             error,
    ZK_IN    FrMailBlockList     blocklist,
    ZK_IN    const char * const  sender_address,
    ZK_IN    const char * const  recipient_address,
    ZK_IN    uint32_t            flags,
    ZK_OUT   ZkBool *            is_found );

/*
 * If you want to do specific checks that aren't covered by the ones
 * offered by frMailBlockList_Check,  you can also use these lookup
 * functions directly.  These assume that the sender and recipients
 * given comprise exactly the key you want to use when searching.
 * They do no special handling of email addresses or domains.
 */

DLLExport int frMailBlockList_LookupSender(
    ZK_INOUT ZkError            error,
    ZK_IN   FrMailBlockList     blocklist,
    ZK_IN   const char * const  sender,
    ZK_OUT  ZkBool *            is_found );

DLLExport int frMailBlockList_LookupRecipient(
    ZK_INOUT ZkError            error,
    ZK_IN   FrMailBlockList     blocklist,
    ZK_IN   const char * const  recipient,
    ZK_OUT  ZkBool *            is_found );

DLLExport int frMailBlockList_LookupSenderRecipient(
    ZK_INOUT ZkError            error,
    ZK_IN   FrMailBlockList     blocklist,
    ZK_IN   const char * const  sender,
    ZK_IN   const char * const  recipient,
    ZK_OUT  ZkBool *            is_found );

# ifdef __cplusplus
}
# endif

# endif /* frMailBlockList_server_h_INCLUDED */
