// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is Freedom code.
// 
// The Initial Developer of the Original Code is Zero-Knowledge Systems,
// Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
// Zero-Knowledge Systems, Inc.  All Rights Reserved.
// 
// Contributor(s):

#include <unistd.h>
#include <stdlib.h>
#include <string>
#include <fstream>
#include <iostream>
#include <istream>

#include <zkTypes.h>
#include <zkIntTypes.h>
#include <zkErrdef.h>
#include <zkError.h>

#include "frmailmsg.h"
#include "frMailAuth.h"

using namespace std;
// this reads a recipient privkey on fd 3, a signing mac key on fd 4, a decoded
// auth header on fd 5, and a decoded message body on stdin, decrypts the
// message with the privkey, and verifies the signature with the signing mac key
// (returns 0 if the signature is ok)
//
// if anything goes wrong, it will return nonzero

ZkError err_; 

#define ensure(x) do { if (! x) { char buf[1024]; zkErrorMessage(err_,buf,1024); \
cerr << "failure " #x " " << endl << buf << endl; exit(100); }} while (0)

int main() {

  zkErrorNew (&err_);
  
  ensure(frCryptInit(err_, 0));
  zkRand_Debug();

  // load private key from fd 3
  istream pk(new std::filebuf(3,"privkey",ios_base::in));
  char c;  string privkey;  
  while(pk.get(c)) privkey += c;    
  size_t pk_sz = privkey.size();
  FrCryptCrypt renc = 0; 
  ensure(frCryptPubKeyNew(err_, &renc));
  ensure(frCryptPubKeyReadFromBuf(err_, renc, const_cast<char *>(privkey.c_str()), &pk_sz));
  ensure(frCryptPubKeySetSharedParms(err_,renc));

  // load shared secret from fd 4
  istream sk(new filebuf(4,"sharedsecret",ios_base::in));
  string secret;
  while(sk.get(c)) secret += c;

  //work out nym
  const char * local_ch = getenv("LOCAL"); 
  if(local_ch == NULL) { cerr << "must specify environment variable LOCAL" << endl;
  _exit(100);
  }
  FrEntIdExp nym;
  ensure(frEntSetFromStr2(err_, &nym, "NYM", local_ch));
  
  // derive mac key
  FrCryptMacKey mac = 0; 
  ensure(frCryptMacKeyNew(err_, &mac));
  ensure(frGenSharedNymMailMacKey(err_, const_cast<char *>(secret.c_str()), secret.size(), &nym, mac));

  // load extracted and decoded header from fd 5
  istream hd(new filebuf(5,"header",ios_base::in));
  string hdr;
  while(hd.get(c)) hdr += c;
    
  // load extracted and decoded message body from stdin
  string msg(""); while(cin.get(c)) msg += c;  
  
  FrMailMsg mm = 0;
  void *omsg; size_t omsz;
  int valid = 1;

  ensure(frMailMsgNew(err_, &mm));  
  ensure(frMailMsgSetFromHeader(err_, mm, renc, const_cast<char *>(hdr.c_str()), hdr.size()));
  ensure(frMailMsgMacVerify(err_, mm, mac, &valid));
  ensure(frMailMsgDecrypt(err_, mm, const_cast<char *>(msg.c_str()), msg.size(), &omsg, &omsz));
  cout << string((char *)(omsg), omsz);
  return ! valid;
}
