
use strict;
use Test;
BEGIN { plan tests => 3 }

=head1 fmg-encrypt round trip test

test sequence for the freedom mail gateway. proceeds by generating a
pair of test keys in F</tmp/public> and F</tmp/private>, then feeding
a test email into C<fmg-encrypt> and piping the results through
C<normalize-mail>, and saving to a file. the file is then fed into
C<reformail> to extract the auth header, C<reformime> to extract the
body and C<fmg-decrypt> to verify that the message can be decrypted
using the key, and verified using the FMG\'s mac. we then run C<cmp>
on the decypted email and the original file to ensure that they are
identical

=head2 requirements

maildrop (http://www.flounder.net/~mrsam/maildrop/)

metamail (http://bmrc.berkeley.edu/~trey/emacs/metamail.html)

some entropy (you need to sit here wiggling the mouse)

=cut


sub squirt {
  my ($dat, $file) = @_;
  return 0 unless defined(open(F, ">$file"));
  return 0 unless (print F $dat) ;
  return 0 unless close(F);
  return 1;
}

my @tmpfiles = ();

my $pubkey = "/tmp/public"; push @tmpfiles, $pubkey;
my $privkey = "/tmp/private"; push @tmpfiles, $privkey;
my $sender = 'bob@zorkmid.net';
my $local = 'alice';
my $recipient = "$local\@yoyodyne.com";
$ENV{'LOCAL'} = $local;
$ENV{'RECIPIENT'} = $recipient;
$ENV{'SENDER'} = $sender;

   
my $email = <<END_TEST_EMAIL;
To: $recipient
From: $sender

hello there, this is a test of the emergency cryptogram service. if this were a 
real encrypted email, it would contain something either secretive, sensitive,
scandalous, or at least substantive. while we all want to use powerful crypto
for our daily conversations, it is probably a bit overkill for test messages, 
unless they are actually testing the functionality of the cryptosystem itself.

-scary unknown nym
END_TEST_EMAIL

my $email_file = '/tmp/test-email'; push @tmpfiles, $email_file;
my $mac = 'some-random-mac-key-definitely-longer-than-20-characters'; 
my $mac_file = '/tmp/mac-file'; push @tmpfiles, $mac_file;
my $encrypted = '/tmp/encrypted'; push @tmpfiles, $encrypted;
my $decrypted = '/tmp/decrypted'; push @tmpfiles, $decrypted;

print STDERR "encrypting mail with $pubkey, signing with FMG mac key\n";
ok(
   (0 == system(
		"fr_newkey --type c --size 1024 --version 1 " .
		"--expires never --owner nym:$recipient " .
		"--private $privkey --public $pubkey" )) &&
   squirt($email, $email_file) &&
   squirt($mac, $mac_file) &&
   (0 == system("fmg-encrypt $pubkey 3<$mac_file 0<$email_file | normalize-mail > $encrypted")));

print STDERR "decrypting mail with $privkey and verifying FMG's mac\n";
my $hdr = 'X-Freedom-Message-Auth:';
ok( 0 == system("fmg-decrypt 3<$privkey 4<$mac_file " .
		" 5< <(cat $encrypted | reformail -x'$hdr' | mimencode -u) " .
		" 0< <(cat $encrypted | reformime -s 1.1 -e | mimencode -u) " .
		" 1>$decrypted " ));

print STDERR "comparing original mail to encrypted mail\n";
ok( 0 == system("cmp $email_file $decrypted"));


print STDERR "cleaning up\n";
for my $file (@tmpfiles) {
#  unlink($file);
}
