/*
 *   Sign an enveloppe header and verify the signature.
 *
 *   $Id: tfrenvsig.c,v 1.3 2000/11/23 15:13:08 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include <assert.h>
#include <stdio.h>
#include <errno.h>
#include "zks.h"
#include "frmailmsg.h"
#include "frCrypt.h"
#include "frEnt.h"

/*#define NDEBUG*/

/*
 *  Generate a new, random key.
 *  
 *  bug: stolen from kgdemo.c.
 */
static int newKey(ZkError err, FrEntIdExp *owner, FrPubKeyType kt, int kv, int ks, ZkClock ke, FrCryptPubKey *newkey)
{
  FrCryptPubKey pk = 0;
  int ret = 0;

  /*
   *  Create a new key object, and set various required parameters.
   */
  
  if ( ! (frCryptPubKeyNew(err, &pk) &&
          frCryptPubKeySetType(err, pk, kt) &&
          frCryptPubKeySetSize(err, pk, ks) &&
          frCryptPubKeySetVersion(err, pk, kv) &&
          frCryptPubKeySetExpiration(err, pk, ke) &&
          frCryptPubKeySetEntity(err, pk, owner))) {
    goto done;
  }

  /*
   *  Generate the bignum parameters.
   */
  
  switch (kt) {
  case FRCRYPT_KEYTYPE_CRYPTION:
    if ( ! (frCryptCryptMakeCrypt(err, pk) &&
            frCryptCryptSetRandomKeys(err, pk))) {
      /* log error */
      goto done;
    }
    break;
    
  case FRCRYPT_KEYTYPE_SIGNING:
    if ( ! (frCryptSigMakeSig(err, pk) &&
            frCryptSigSetRandomKeys(err, pk))) {
      /* log error */
      goto done;
    }
    break;

  default:
    /* log error */
    goto done;
  }

  ret = 1;
  *newkey = pk; pk = 0; /* caller now owns the key */

 done:

  if (pk) { frCryptPubKeyFree(err, &pk); }

  return ret;
}


int main() {
  const char* env_sender1 = "alice@freedom.net";
  const char* env_sender2 = "bob@freedom.net";
  const char* env_recip1 =  "foo@bar.com";
  const char* env_recip2 =  "foo@bar.net";

  const char *prog = "tfrenvsig";

  FrCryptSig sigKey1 = 0, sigKey2 = 0;
  FrEntIdExp owner;
  ZkError err = 0;
  ZkClock ke;
  int ok1 = 0, ok2 =0, ok3 = 0, ok4 = 0, ok5 = 0, ret = 0;
  char sig[ FRMAIL_ENV_SIG_SIZE ];

  if ( ! frCryptInit(err, 0)) {
    goto done;
  }

  fr_EntSetRandom(&owner);
  owner.type = FRENT_TYPE_NYM;
  ke.rep = time(0);
  if ( !newKey(err, &owner, FRCRYPT_KEYTYPE_SIGNING, 3, 1024, ke, &sigKey1) ||
       !newKey(err, &owner, FRCRYPT_KEYTYPE_SIGNING, 3, 1024, ke, &sigKey2) ) {
    goto done;
  }

  
  /* sign the enveloppe addresses */

  if ( !frMailEnvelopeAddrSign( err, sigKey1, env_sender1, env_recip1, sig ) ) {
    goto done;
  }
  fprintf(stdout, "%s: successfully signed envelope addresses.\n", prog);
  fprintf(stdout, "sig: '%s'\n", sig);

  /* verify the enveloppe address */

  if ( 
      /* should succeed */
      !frMailEnvelopeAddrSigVerify( err, sigKey1, env_sender1, env_recip1, sig, &ok1 ) ||
      /* wrong key */
      !frMailEnvelopeAddrSigVerify( err, sigKey2, env_sender1, env_recip1, sig, &ok2 ) ||
      /* wrong sender */
      !frMailEnvelopeAddrSigVerify( err, sigKey1, env_sender2, env_recip1, sig, &ok3 ) ||
      /* wrong recipient */
      !frMailEnvelopeAddrSigVerify( err, sigKey1, env_sender1, env_recip2, sig, &ok4 ) ||
      /* wrong enveloppe */
      !frMailEnvelopeAddrSigVerify( err, sigKey1, env_sender2, env_recip2, sig, &ok5 ) ) {
    fprintf( stderr, "error verifying signatures\n" );
    goto done;
  }

  if (ok1 && !(ok2 || ok3 || ok4 || ok5)) {
    fprintf( stdout, "signature verification: ok\n" );
  }
  else {
    fprintf( stdout, "signature verification: ERROR\n" );
  }
  
  ret = 1;

 done:

  if (sigKey1) { frCryptSigFree(err, &sigKey1); }
  if (sigKey2) { frCryptSigFree(err, &sigKey2); }
  if (err) { zkErrorFree(&err); }

  if (ret == 0) {fprintf( stdout, "%s: something went wrong\n", prog );}
  return ret;
}
