#ifndef FRMAIP_RBPRIV_H_
#define FRMAIP_RBPRIV_H_

/*
 *  $Id: frmaip_repblk.h,v 1.11 2001/04/19 20:28:12 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "frCrypt.h"
#include "frPktFmt.h"


#define FRMAIP_EMAIL_LEN  128 /* max. len of email address, incl. nul term. */
#define FRMAIP_HOPS_MAX     5 /* max. number of MAIP hosts */
#define FRMAIP_RBNAME_LEN  64 /* max. len of this reply block's name */


typedef struct frmaip_rbpriv *FrMaipRbPriv;
typedef struct frmaip_rbpub_activation_req FrMaipRbActivationReq;

/*
 *  Fill in this structure, then call frMaipRbPrivSetNextHop() to create one
 *  hop in a route.  Although the cryption key in `maipPub' is owned by the
 *  caller it must not be freed until after the call to
 *  frMaipRbPrivSetNextHop().
 */
struct frmaip_rbpriv_set_hop {
  FrCryptCrypt encrPubKey;      /* the MAIP's public cryption key */
  FrCryptSymAlg encrSymAlg;
  FrCryptSymAlg replySymAlg;
  FrEntIdExp maip;
  uint16_t port;                /* This value will be ignored and zeroed */
  uint32_t latency;
};

struct frmaip_rbpub_hop {
  FrCryptSym replySym;
  FrCryptMacKey replyMacKey;
  FrEntIdExp maip;
  uint16_t port;                /* This value will be ignored and zeroed */
  uint32_t latency;
  char email[FRMAIP_EMAIL_LEN];
};

struct frmaip_rbpub_activation_req {
  uint8_t type;
  uint8_t version;
  uint8_t pad1_[2]; /* padding - must be zero */
  uint8_t timestamp[17];
  uint8_t pad2_[3]; /* padding - must be zero */
  uint8_t hash[FRCRYPT_HASH_SIZE];
};

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport int frMaipRbPrivNew(ZkError err, FrMaipRbPriv *repblk);
  extern DLLExport int frMaipRbPrivFree(ZkError err, FrMaipRbPriv *repblk);
  
  extern DLLExport int frMaipRbPrivGetEmail(ZkError err, FrMaipRbPriv repblk, const char **email);
  extern DLLExport int frMaipRbPrivSetEmail(ZkError err, FrMaipRbPriv repblk, const char *email);
  
  extern DLLExport int frMaipRbPrivGetName(ZkError err, FrMaipRbPriv repblk, const char **name);
  extern DLLExport int frMaipRbPrivSetName(ZkError err, FrMaipRbPriv repblk, const char *name);
  
  extern DLLExport int frMaipRbPrivGetNym(ZkError err, FrMaipRbPriv repblk, const FrEntIdExp **nym);
  extern DLLExport int frMaipRbPrivSetNym(ZkError err, FrMaipRbPriv repblk, FrEntIdExp *nym);
  
  extern DLLExport int frMaipRbPrivGetUserData(ZkError err, FrMaipRbPriv repblk, void **ud);
  extern DLLExport int frMaipRbPrivSetUserData(ZkError err, FrMaipRbPriv repblk, void *ud);

  extern DLLExport int frMaipRbPrivGetHop(ZkError err, FrMaipRbPriv repblk, unsigned int n, int *lastHop, struct frmaip_rbpriv_set_hop *h);
  extern DLLExport int frMaipRbPrivSetNextHop(ZkError err, FrMaipRbPriv repblk, int isFirst, int isLast, struct frmaip_rbpriv_set_hop *hop);

  extern DLLExport int frMaipRbPrivReadFromBuf(ZkError err, FrMaipRbPriv repblk, void *rbuf, size_t rsz);
  extern DLLExport int frMaipRbPrivWriteToBuf(ZkError err, FrMaipRbPriv repblk, void **rbuf, size_t *rsz);

  extern DLLExport int frMaipRbPrivDecrypt(ZkError err, FrMaipRbPriv repblk, void *encr, size_t esz, void **clr, size_t *csz);
  extern DLLExport int frMaipRbPubHopEncrypt(ZkError err, struct frmaip_rbpub_hop *nextHop, int firstHop, void *clr, size_t csz, void **encr, size_t *esz);

  extern DLLExport int frMaipRbPrivMakePublic(ZkError err, FrMaipRbPriv repblk, void **rbpub, size_t *rsz);
  extern DLLExport int frMaipRbPubGetHopCryptKeyVersion(ZkError err, void *rbpub, size_t rsz, unsigned int *vers);
  extern DLLExport int frMaipRbPubGetNextHop(ZkError err, void *rbpub, size_t rsz, FrCryptCrypt maipPriv, int firstHop, int *lastHop, struct frmaip_rbpub_hop *nextHop);
  extern DLLExport int frMaipRbPubClearHop(ZkError err, struct frmaip_rbpub_hop *hop);

  extern DLLExport int frMaipRbPubIsActive( ZK_INOUT   ZkError        err,
                                            ZK_IN      void *         rbpub,
                                            ZK_IN      size_t         rsz,
                                            ZK_OUT     unsigned int * is_active );

  extern DLLExport int frMaipRbPubActivate( ZK_INOUT   ZkError        err,
                                            ZK_IN      void *         rbpub,
                                            ZK_IN      size_t         rsz );

  extern DLLExport int frMaipRbPubDeactivate( ZK_INOUT   ZkError        err,
                                              ZK_IN      void *         rbpub,
                                              ZK_IN      size_t         rsz );
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRMAIP_RBPRIV_H_ */
