#!/bin/sh
#
# $Id: thin.tcl,v 1.1 2000/03/24 22:52:26 adamb Exp $
#
#\
exec tclsh "$0" "$@"
#
# Generate test input for thdrsig.
#

set tprog [file join $env(ZK_FREEDOM_ROOT) libs frmaip arch $env(ZK_ARCH) thdrsig]

set from "From:"
set to   "To:"
set ng   "Newsgroups:"
set date "Date:"
set cc   "Cc:"
set subj "Subject:"

proc globals {} {
  uplevel global from to ng date cc subj
}


#
#
#                                  Signing
#
#

#
# Valid
#

# Minimum valid headers.
proc sv0 {f} {
  globals
  puts $f "$from foo@freedom.net"
  puts $f "$to bar@baz.com"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}

# Minimum valid headers; Newgroups replaces To.
proc sv1 {f} {
  globals
  puts $f "$from foo@freedom.net"
  puts $f "$ng comp.lang.c,alt.mcdonalds"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}

# Minimum valid headers.
proc sv2 {f} {
  globals
  puts $f "$from \"Fred Foo\" <foo@freedom.net>"
  puts $f "$to bar@baz.com"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}

# Valid headers, including subject.
proc sv3 {f} {
  globals
  puts $f "$subj Re: future primitives"
  puts $f "$from \"Fred Foo\" <foo@freedom.net>"
  puts $f "$to bar@baz.com"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}

# Valid headers, different order.
proc sv4 {f} {
  globals
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
  puts $f "$to bar@baz.com"
  puts $f "$subj Re: future primitives"
  puts $f "$from \"Fred Foo\" <foo@freedom.net>"
}

# Valid headers, split lines.
proc sv5 {f} {
  globals
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
  puts $f "$to bar@baz.com"
  puts $f "$subj Re: future primitives"
  puts $f "\t\ta.k.a. delerium"
  puts $f "$from \"Fred Foo\""
  puts $f " <foo@freedom.net>"
}

#
# Invalid.
#

# Invalid headers: missing From.
proc si0 {f} {
  globals
  puts $f "$to bar@baz.com"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}

# Invalid headers: missing To.
proc si1 {f} {
  globals
  puts $f "$from foo@freedom.net"
  puts $f "$date Thu, 23 Mar 2000 23:11:06 -0500"
}


#
#
#                                Verification
#
#

proc vv0 {f} {
}



set tmp [file join / tmp thin]
set svdir [file join $tmp sv]
set sidir [file join $tmp si]
set vvdir [file join $tmp vv]
set vidir [file join $tmp vi]

set log [file join / tmp thin.log]
set null [file join / dev null]

file mkdir $svdir $sidir $vvdir $vidir

# Test valid headers.

proc mkmsg {file hproc} {
  set f [open $file w]
  fconfigure $f -translation crlf
  $hproc $f
  puts $f ""
  puts $f "...message body..."
  close $f
}

puts "rm -f $log"
puts "echo \"***** sign valid headers *****\""
foreach p [info procs sv*] {
  set if [file join $svdir $p]
  set of [file join $vvdir $p] ; # check the output later
  mkmsg $if $p
  puts "$tprog --logfile $log --sign --infile $if --outfile $of"
}

puts "echo \"***** sign invalid headers *****\""
foreach p [info procs si*] {
  set if [file join $sidir $p]
  set of $null
  mkmsg $if $p
  puts "$tprog --logfile $log --sign --infile $if --outfile $of"
}

puts "echo \"***** verify valid headers *****\""
puts "for f in $vvdir/* ; do"
puts "  $tprog --logfile $log --verify --infile \$f"
puts "done"

#
# bug: we want to attempt verification against originally valid
#      headers that have been damaged in some way.
#
#puts "echo \"***** verify invalid headers *****\""
#puts "for f in $vidir/* ; do"
#puts "  $tprog --logfile $log --verify --infile \$f"
#puts "done"
