/* $Id: frNCDB.h,v 1.17 2001/04/19 20:28:36 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRNCDB_H_
#define FRNCDB_H_
#include <netinet/in.h>
#include <zkHash.h>
#include <frNDB.h>
#include <frPktFmt.h>
#include <zkMemPool.h>

#define FRPKT_NUM_ROUTES_MAX 5
#define FR_NCDB_EXPIRED_TTL 600 /* 10 minutes */

/**
 * The currently supported AIP protocols we
 * understand; this is a bitmask
 */
#define FR_AIP_SUPPORTED_PROTOCOLS 0x0001 

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  typedef struct T_FrAIPinfo *FrAIPinfo;
  typedef struct T_FrRoutes *FrRoutes;
  typedef struct T_FrLinks *FrLinks;

  struct T_FrAIPinfo {
    int      id;      /* Starts at 0 */
    char     entity[ZK_NDBNAME_SIZE]; /* Redundant? */
    char     name[ZK_NDBNAME_SIZE];
    char     ip[ZK_NDBNAME_SIZE];
    uint16_t tcpport; /* TCP Port (NBO?) */
    uint16_t udpport; /* UDP port =  TCP Port + 1 (NBO?) */
    int     isexit;  /* Do we have a wormhole available */
    int     isup;    /* Is it up? */
    char    region[ZK_NDBNAME_SIZE];
    int     rating;
    int     protocol;  /* Actually, a bit field */
    int     isusable;  /*
			* Can we use this AIP with our client?
			* Base this decision on the protocol
			*/
      uint16_t numconn; /* Number of neighbors */
  
    /* For efficiency */ 
    struct sockaddr_in saddr; /* in NBO */
  };

  struct T_FrRoutes {
    int id;
    int links;  /* Set internally */
    int available; /* If route is available */
    int aip[FRPKT_NUM_ROUTES_MAX+1];
  };

  struct T_FrLinks {
    int id;
    int aip[2];
    /* Bug: Change this to:
          int aip1;
	  int aip2; */
    int cost;
    /* Internal */
    int cnt;  
  };

#if 0
  struct T_FrDnsCache {
    char     name[ZK_NDBNAME_SIZE];
    struct in_addr ip_addr;
  };
#endif
  typedef struct T_FrDnsCache *FrDnsCache;
  

  struct T_FrNCDB {
    char *revision;       /* The revision string */
    char filename[ZK_NDBNAME_SIZE*2];
    char *updated;
    char *expiration;
    // char *signature;
    char *url;
    uint32_t ttl;
    char *timeout;        /* When should the timeout occur */
    uint16_t numaips;
    uint16_t numroutes;
    uint16_t availroutes; /* Number of routes that are currently available */
    uint16_t numlinks;

    int overrideRoute;    /* Global Override - overrides all overrides */
    int sequencedRoutes;  /* Global Override - overrides all overrides */
    int disableRoutes;    /* Return no routes */
    int currSequenceRoute;
    int randomized;       /* Have we been seeded? */
    int lastSelectedRoute;
    
  
    FrAIPinfo aipinfo;    /* Pointer to an arrary of AIPs */
    FrRoutes  routes;     /* Pointer to an arrary of routes */
    FrLinks    links;     /* Pointer to an arrary of links */
    struct T_winclient {
      char *revision;
      char *updated;
      char *infoURL;
      char *upgradeURL;
      char *downloadURL;
      char *expireClient;
      uint32_t urgency;   /* Refer to FRNDB_URGFL_* for the flags */
    } winclient;
    /* This is going to have to change */
    struct T_servers {
      uint16_t  aipudpport;
      uint16_t  aiptcpport;
      uint16_t  clientudptcpport;
      char     *nymhost;
      char     *nymip;
      uint16_t  nymport;
      int       nymtimeout;
      char     *tokenhost;
      char     *tokenip;
      uint16_t  tokenport;
      int       tokentimeout;
      char     *newsgrphost;
      char     *newsgrpip;
      uint16_t  newsgrpport;
      int       newsgrptimeout;
      char     *dnshost;
      char     *dnsip;
      uint16_t  dnsport;
      int       dnstimeout;
      char     *nidbhost;
      char     *nidbip;
      uint16_t  nidbport;
      int       nidbtimeout;
      char     *keyupdhost;
      char     *keyupdip;
      uint16_t  keyupdport;
      int       keyupdtimeout;
      char     *keyqryhost;
      char     *keyqryip;
      uint16_t  keyqryport;
      int       keyqrytimeout;
      char     *fmgmaiphost;
      char     *fmgmaipip;
      uint16_t  fmgmaipport;
      int       fmgmaiptimeout;
      uint16_t  maipport;
      int       maiptimeout;
      char     *trialserialURL;
    } servers;
    ZkConfig config;
    ZkMemPool mpool;
#if 0    
    FrDnsCache dns[10]; /*
			 * BUG: THIS HAS TO CHANGE
			 * Quick fix for Mario (45 min job)
			 */
#endif    
    FrNCDB update; /* Pointer to the next update */
  };

  extern DLLExport int frNCDBInit (ZkError , FrNCDB *,
				   const char *,
				   time_t *);
  extern DLLExport int frNCDBClose (ZkError, FrNCDB *);

  extern DLLExport int frNCDBSetOverrideRoute (ZkError, FrNCDB,
					       int);
  extern DLLExport int frNCDBResetOverrideRoute (ZkError err, FrNCDB nc);
  extern DLLExport int frNCDBSetSequencedRoute (ZkError err, FrNCDB nc);
  extern DLLExport int frNCDBResetSequencedRoute (ZkError err, FrNCDB nc);
  extern DLLExport int frNCDBGetMaxNumRoutes (ZkError err, FrNCDB nc, int *routes);

  extern DLLExport int frNCDBGetAllAIPInfo (ZkError, FrNDB,
				  char * /*version*/,
				  FrNDBAipList * /* aiplist */,
				  int * /* number of aips in list */ );
  extern DLLExport int frNCDBFreeAIPInfo (ZkError, FrNDBAipList *);

  
#ifndef NDEBUG  
  extern char *saddrSprintf (struct sockaddr_in *host);
  extern int frNCDBDumpClientAipList (FrNDB, FrNDBAipList);
#endif  

  extern DLLExport int frNCDBIsOverrideRouteSet (ZkError err, FrNCDB nc);
  extern DLLExport int frNCDBGetOverrideRoute (ZkError err,
					       FrNCDB nc, int *override);
  extern DLLExport int frNCDBIsGetRoutesInSequenceSet (ZkError err, FrNCDB nc);
  extern DLLExport int frNCDBGetNextRouteInSequence (ZkError err, FrNCDB nc,
						     int *nxtRoute);
  extern DLLExport int frNCDBGetRandomRoute (ZkError err, FrNCDB nc,
					     int *route);

#ifndef NDEBUG
  extern DLLExport int frNCDBDump (FrNCDB);
  extern int frNCDBAIPinfoDump(FrAIPinfo);
  extern int frNCDBLinkInfoDump(FrLinks );
#endif

  extern int frNCDBRoutesDenied (ZkError, FrNCDB);

  extern int frNCDBNew (ZkError, FrNCDB *);
  extern int frNCDBLoad(ZkError, FrNCDB, time_t *);

  extern int frNCDBGetAipInfoFromId (ZkError, FrNCDB, int, FrAIPinfo *);
  extern int frNCDBGetLinkInfoFromId (ZkError, FrNCDB, int, FrLinks *);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#include <frNCDBqry.h>

#endif /* FRNCDB_H_ */

