/*
 * frNCDBqry.h
 * $Id: frNCDBqry.h,v 1.9 2001/04/19 20:28:36 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 * 
 */

#ifndef FRCNDBQRY_H_
#define FRCNDBQRY_H_

#include <time.h>
#include <zks.h>
#include <zkList.h>
#include <zkIntTypes.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  struct T_FrNDBHostPort {
    const char *host; /* Copy this if you need it */
    uint16_t   port;
    int        timeout;
  };

  typedef struct T_FrNDBHostPort *FrNDBHostPort;

  DLLExport int frNCDBGetNymServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetTokenServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetNewsGroupServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetDNSServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetNIDBServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetKeyQueryServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetKeyUpdateServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBGetFMGServers (ZkError, FrNDB, ZkList *);
  DLLExport int frNCDBFreeHostPortList (ZkError, ZkList *);
  DLLExport int frNCDBGetMAIPPortAndTimeout (ZkError, FrNDB,
					     uint16_t *,  /* Port */
					     uint16_t *); /* Timeout */
  DLLExport int frNCDBGetClientUdpTcpPort (ZkError, FrNDB, uint16_t *port);

  DLLExport int frNCDBGetAIPsNameUdpTcpPort (ZkError, FrNDB,
					    const char *,/* hostname */
					    uint16_t *,  /* udpport */
					    uint16_t *); /* tcpport */
  DLLExport int frNCDBGetAIPsIPAddrUdpTcpPort (ZkError, FrNDB,
					    const char *,/* ipaddr */
					    uint16_t *,  /* udpport */
					    uint16_t *); /* tcpport */
  DLLExport int frNCDBGetAIPsInAddrUdpTcpPort (ZkError, FrNDB,
					    struct in_addr, /* ipaddr */
					    uint16_t *,     /* udpport */
					    uint16_t *);    /* tcpport */
  
  DLLExport int frNCDBGetRevision (ZkError, FrNDB, const char **);

  /* Get the TimeToLive for this Configuration */
  DLLExport int frNCDBGetTTL (ZkError, FrNDB, uint32_t *);
  /* Get the Time when the config will expire */
  DLLExport int frNCDBGetLocalTimeExpired (ZkError, FrNDB, time_t *);
  
  /* Get the Time when the config file was updated */
  DLLExport int frNCDBGetLocalTimeUpdated (ZkError, FrNDB, time_t *);

  /* Get the URL where the fetch the config files */
  DLLExport int frNCDBGetNIDBConfigURL (ZkError, FrNDB, const char **);

  /*
   * Access to the Windows Client Update NCDB Information
   */
  /** The currently available revision on our servers */
  DLLExport int frNCDBGetWinClientCurrRevision (ZkError, FrNDB, const char **);

  /** The URL where info for this client can be found */
  DLLExport int frNCDBGetWinClientCurrInfoURL (ZkError, FrNDB, const char **);

  /** The URL to get the upgrade */
  DLLExport int frNCDBGetWinClientCurrUpgradeURL (ZkError, FrNDB, const char **);

  /** The URL to download the complete client */
  DLLExport int frNCDBGetWinClientCurrDownloadURL (ZkError, FrNDB,
						   const char **);
  /** How important it is to get this client.
   *   Note that the Urgency flags can be found in frNDB.h in the
   *   FRNDB_URGFL_* flags.
   */
  DLLExport int frNCDBGetWinClientCurrUrgency (ZkError, FrNDB,
					       uint32_t *); /* Urgency */
  /** Get the time when the client was updated */
  DLLExport int frNCDBGetWinClientLocalTimeUpdated (ZkError, FrNDB, time_t *);

  /** The URL to get trial serial numbers */
  DLLExport int frNCDBGetTrialSerialNumberURL (ZkError, FrNDB,
					    const char **);

  /** Get the list of all neighbours of an AIP */
  DLLExport int frNCDBGetAllAipNeighboursForAIPId (ZkError,  FrNCDB,
						   int,    /* AIPid */
						   int **, /* neighbourlist */
						   int *); /* numneighbours */

  /**
   * Given a hostname in the NCDB, get the struct in_addr for it
   * INADDR_NONE is hostname is not found
   */
  DLLExport int frNCDBGetIPfromHostname (ZkError err, FrNCDB ncdb,
			     const char *hostname, struct in_addr *ip_addr);
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRCNDBQRY_H_ */
