/*
 * frNDB.h
 * $Id: frNDB.h,v 1.27 2001/04/19 20:28:36 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 * 
 */

#ifndef FRNDB_H_
#define FRNDB_H_

#include <arpa/inet.h>
#include <time.h>
#include <zks.h>
#include <zkList.h>
#include <zkIntTypes.h>
#include <zkConfig.h>
#include <zkIntTypes.h>
#include <frEnt.h>

#define ZK_NDBNAME_SIZE FRENT_NAME_SIZE

#define FRNDB_URGFL_NONE         0
#define FRNDB_URGFL_BUG_FIX      1
#define FRNDB_URGFL_NEW_FEATURE  2
#define FRNDB_URGFL_SECURITY_FIX 4

#define FRNDB_LINK_IS_DOWN  255

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  typedef struct T_FrNDB *FrNDB;
  typedef struct T_FrNCDB *FrNCDB;
  typedef struct T_FrNNP *FrNNP;
  /* typedef struct T_FrNSDB *FrNSDB; */

  /* Only a single instance of this object should exist */
  /* Will need to mutex lock this thing, I think */
  struct T_FrNDB {
    int initialized;
    int refcount;
    FrNCDB ncdb;       /* Network Configuration (List of AIPs and Routes) */
    /* struct T_FrNSDB *nsdb; */ /* Nym status database */
    /* struct T_FrNKDB *nkdb; */ /* Network Key */
    /* struct T_FrNRDB *nrdb; */ /* Network Routing (AIP Status) */
    ZkList nnplist;
  };

  /**
   * Structure returned in a list when requesting a list of
   * AIPs
   */
  struct T_FrNDBAipList {
    int id;
    char entity[ZK_NDBNAME_SIZE];    /* Bug: make this dynamic */
    char hostname[ZK_NDBNAME_SIZE];    /* Bug: make this dynamic */
    char region[ZK_NDBNAME_SIZE];   /* Bug: make this dynamic */
    struct sockaddr_in saddr;
    int isup;
    int isexit;
    int isusable;
    int rating;
    FrEntIdExp aip_entity;  
    FrEntIdExp maip_entity; 
  };

  typedef struct T_FrNDBAipList *FrNDBAipList;

  /*
   * Open access to the Network Database
   *
   * Returns a pointer to the NDB (always the same pointer)
   * Configfile must be NULL if previously set by frNDBOpen or frNDBEnv
   */
  DLLExport extern int frNDBOpen (ZkError, FrNDB *,
				  const char *, /* Configfile */
				  time_t *      /* timeout */);
  DLLExport extern int frNDBClose (ZkError, FrNDB *);

  DLLExport extern int frNCDBUpdate (ZkError, FrNCDB *ncdb,
				     const char *, /* newfile */
				     time_t *);    /* timeout */
  extern int frNDBDump (FrNDB);
  
  /*
   * Sets the DBEnvironment; ie: the config file location
   * Will initialize the NDB structures
   */
  DLLExport extern int frNDBEnv (ZkError, const char *, /* Config filename */
				 time_t *               /* timeout */);
  /*
   * Return a pointer to the NDB (always the same)
   */
  DLLExport extern int frNDBGetPtrToNDB (ZkError, FrNDB *);

  /* 
   * Routing information
   */
  DLLExport extern int frNRDBGetAnonRouteToExit (ZkError, FrNDB, int * );

  DLLExport extern int frNRDBGetNumLinksForAnonRouteId (ZkError, FrNDB,
							int   /* routeID */,
							int * /* links */);

  DLLExport extern int frNCDBGetNumLinksForRouteId (ZkError, FrNDB,
						    int   /* routeid */,
						    int * /* links */ );
  
  DLLExport extern int frNCDBGetAipXinRouteId (ZkError, FrNDB,
					       int   /* routeid */,
					       int   /* hop */,
					       int * /* aip */);

  /**
   * Return a list of aip names
   * Please do a free(aipnamelist) when you're done.
   */
  DLLExport int frNCDBGetAllAipNames (ZkError, FrNDB, 
				      const char *** /* aipnamelist */,
				      int * );          /* numaips */


  DLLExport extern int frNCDBGetAllAipNamesForRouteId (ZkError,
					  FrNDB,
  					  int,   /* RouteID */
				          char ***,  /* ***aipnamelist - the list of aips */
				          int *);  /* *numaips - Number of aips */

  DLLExport extern int frNCDBFreeAipNames (ZkError, FrNDB,
					   char ***  /* ***aipnamelist */);

  /* Warning: This call will be removed. Do no use */
  DLLExport extern int frNRDBGetAipXinRouteId (ZkError, FrNDB,
					       int   /* route */,
					       int   /* hop */,
					       int * /* aip*/);


  /*
   * AIP Information
   */
  DLLExport extern int frNNPGetAllAIPInfo (ZkError, FrNNP,
			char *          /* version number
			       		   use preallocated buffer
			                   of ZK_NDBNAME_SIZE bytes */,
			FrNDBAipList * /* newly allocated list of aips */,
 			int *          /* number of aips in list*/);

  DLLExport extern int frNNPFreeAIPInfo (ZkError,
		              FrNDBAipList * /* frees this allocation */);
  
  DLLExport extern int frNCDBGetEntityForAipID (ZkError, FrNDB,
					       int aip, const char **entity);
  DLLExport extern int frNCDBGetAipIDFromHostname (ZkError, FrNDB,
						   const char *, int *);
  DLLExport extern int frNCDBGetNameForAipID (ZkError, FrNDB,
					      int aip, const char **name);
  DLLExport extern int frNCDBGetIPStrForAipID (ZkError, FrNDB,
					       int aip, const char **ipstr);
  DLLExport extern int frNCDBGetIPAddrForAipID (ZkError, FrNDB,
						int aip,
						struct in_addr *);
  DLLExport extern int frNCDBGetTCPPortForAipID (ZkError, FrNDB,
						 int aip, uint16_t *tcpport);
  DLLExport extern int frNCDBGetUDPPortForAipID (ZkError, FrNDB,
						 int aip, uint16_t *udpport);
  DLLExport extern int frNCDBGetUDPsockaddr_inForAipID (ZkError, FrNDB,
							int /* aip */,
							struct sockaddr_in *);
  /*
   * Special Routing behaviours
   */
  DLLExport extern int frNRDBSetOverrideRoute (ZkError, FrNDB,
					       int /* override */);
  DLLExport extern int frNRDBResetOverrideRoute (ZkError, FrNDB);
  DLLExport extern int frNRDBSetSequencedRoute (ZkError, FrNDB);
  DLLExport extern int frNRDBResetSequencedRoute (ZkError, FrNDB);

  /*
   * NYM RELATED CALLS
   */
  DLLExport extern int frNNPNew (ZkError, FrNNP *, 
				  const char *   /* nym */,
				  uint32_t       /* nym id */);
  DLLExport extern int frNNPFree (ZkError, FrNNP *);
  DLLExport extern int frNNPDump (ZkError, FrNNP);

  DLLExport extern int frNNPGetNym (ZkError, FrNNP, const char **);

  /* Return NULL pointer if not nnp is available */
  DLLExport extern int frNNPFromNymGetNNP (ZkError,
					   const char * /* nym */,
					   FrNNP *);


  /*
   * Get the list of available routeID's for thus Nym, based on
   * active Rules.
   * Returns a. calloc'ed  array of int, representing the routeID's
   *         b. the size of the previous array
   */
  DLLExport extern int frNNPGetAvailRoutes (ZkError, FrNNP,
					    int **, /* Array of routeID's */
					    int *); /* Number in list */

  /* Free the list allocated by frNNPGetAvailRoutes() */
  DLLExport extern int frNNPFreeRouteList (ZkError, FrNNP,
					   int **); /* Array of routeID's to free */

  DLLExport extern int frNNPGetAnonRouteToExit (ZkError,
						FrNNP,
						int * /* anonRoute */);

  DLLExport extern int frNNPRuleDelete (ZkError, FrNNP,
					int   /* ruleId */,
					int * /* routes matched */);

  DLLExport extern int frNNPClearAllRules (ZkError, FrNNP,
					   int *);  /* routes available */

  DLLExport extern int frNNPRuleAddStartAIP (ZkError, FrNNP,
					     int   /* start aip */,
					     int * /* ruleId for this rule */,
					     int * /* routes matched */);

  DLLExport extern int frNNPRuleAddExitAIP (ZkError, FrNNP,
					    int   /* exit aip */,
					    int * /* ruleId for this rule */,
					    int * /* routes matched */);

  DLLExport extern int frNNPRuleAddAvoidAIP (ZkError, FrNNP,
					  int   /* avoid this aip */,
					  int * /* ruleId for this rule */,
					  int * /* routes matched */);

  /* No ruleId returned for this - this is probably a bug */
  DLLExport extern int frNNPRuleSetNumHops (ZkError, FrNNP,
					   int   /* Number of Hops to use */,       
					   int * /* routes matched */);


# define ZK_BUFSIZE 1024

#if 0
extern char *strdup(const char *);
extern int strncasecmp(const char * ,const char *, int );
extern int snprintf (char *str, size_t n, const char *format, ... );
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNDB_H_ */
