/*
 * $Id: frNCDB.c,v 1.41 2001/04/19 20:28:37 alainc Exp $
 *
 * The Network Configuration Data Module
 * Luc Boulianne
 *
 */
/*
 * Bug: Site Owner? Wait for now
 * Bug: Timeout values?
 * Bug: Entity Names 
 * Bug: Port Numbers from IP/Name 
 */

#include "zks.h"

#include <stdio.h>
#include <stdarg.h>
#include <netdb.h>
#include <sys/time.h>
#include <sys/socket.h>
#ifndef WIN32
#include <unistd.h>
#define __EXTENSIONS__
#include <string.h>
#undef __EXTENSIONS__
#endif
#include <limits.h>
#include <frNDB.h>
#include <frNCDB.h>
#include <frNRDB.h>
#include <frCrypt.h>
#include <zkDebug.h>
#include <zkError.h>
#include <zkList.h>
#include <zkDateTime.h>
#include "zkCStr.h"
#include <zkFile.h>
#include <frEnt.h>
#include "frNetworkDefaults.h"
#include "zkConfFile.h"
#include "frNdbCvt.h"


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMDB

#define INI_DONE -1
#define INI_INT  0
#define INI_UINT 1
#define INI_BOOL 2
#define INI_CHAR 3
#define INI_UINT16 4

#define ZKREQ 0
#define ZKOPT 1


struct T_IniSect {
  int16_t  id;
  int16_t  type;
  const char *token;
  uint16_t  count;      /* 1  means there will be only one entry */
			/* 0  means there are more than one      */
};
typedef struct T_IniSect *IniSect;

struct T_Ini {
  const char *name;
  IniSect *desc;
};

typedef struct T_IniSections *IniSections;

#define T_AIP_ID	0
#define T_AIP_ENTITY	1
#define T_AIP_NAME	2
#define T_AIP_IP	3
#define T_AIP_TCPPORT   4
#define T_AIP_UDPPORT   5
#define T_AIP_ISEXIT    6
#define T_AIP_ISUP      7
#define T_AIP_REGION    8
#define T_AIP_RATING    9
#define T_AIP_PROTO     10
#define T_AIP_DONE      -1

#define T_RT_ID         0
#define T_RT_AIP        1
#define T_RT_DONE       -1

#define T_LINK_ID       0
#define T_LINK_AIP      1
#define T_LINK_COST     2
#define T_LINK_DONE     -1

/* ----------------- [ END of New zkConf Stuff ] ------- */

static struct T_IniSect AipIniProto[] = {
  {T_AIP_ID, INI_INT, "id", 1},
  {T_AIP_ENTITY, INI_CHAR, "entity", 1},
  {T_AIP_NAME, INI_CHAR, "hostname", 1},
  {T_AIP_IP, INI_CHAR, "ip", 1},
  {T_AIP_TCPPORT, INI_UINT16, "tcpport", 1},
  {T_AIP_UDPPORT, INI_UINT16, "udpport", 1},
  {T_AIP_ISEXIT, INI_BOOL, "isexit", 1},
  {T_AIP_ISUP, INI_BOOL, "isup", 1},
  {T_AIP_REGION, INI_CHAR, "region", 1},
  {T_AIP_RATING, INI_INT, "rating", 1},
  {T_AIP_PROTO, INI_INT, "protocol", 1},
  {T_AIP_DONE, INI_DONE, "", 0}
};

struct T_IniSect RouteIniProto[] = {
  {T_RT_ID, INI_INT, "id", 1},
  {T_RT_AIP, INI_INT, "aip", 0},
  {T_RT_DONE, INI_DONE, "", 0}
};

struct T_IniSect LinksIniProto[] = {
  {T_LINK_ID, INI_INT, "id", 1},
  {T_LINK_AIP, INI_INT, "aip", 0},
  {T_LINK_COST, INI_INT, "cost", 1},
  {T_LINK_DONE, INI_DONE, "", 0}
};

struct T_NameList {
  char *list[FRPKT_NUM_ROUTES_MAX];
  int num;
  char buf[ZK_NDBNAME_SIZE * FRPKT_NUM_ROUTES_MAX];
};

typedef struct T_NameList *NameList;

extern int RouteInfoDump (FrRoutes);

int frNCDBNew (ZkError err, FrNCDB *ncdb) {
  static const char *funcname = "frNCDBNew";
	
  if (ncdb) {
    zkMALLOC(*ncdb, FrNCDB, (sizeof(struct T_FrNCDB)));

    /* Initialize necessary context values */
    *((*ncdb)->filename)='\0';
    (*ncdb)->revision=NULL;
    (*ncdb)->expiration=NULL;
    (*ncdb)->ttl=0;
    (*ncdb)->timeout=NULL;

    (*ncdb)->winclient.revision=NULL;
    (*ncdb)->winclient.updated=NULL;
    (*ncdb)->winclient.infoURL=NULL;
    (*ncdb)->winclient.upgradeURL=NULL;
    (*ncdb)->winclient.downloadURL=NULL;
    (*ncdb)->winclient.urgency=0;
    (*ncdb)->winclient.expireClient=NULL;

    (*ncdb)->servers.aipudpport=0;
    (*ncdb)->servers.aiptcpport=0;
    (*ncdb)->servers.clientudptcpport=0;
    (*ncdb)->servers.nymhost=NULL;
    (*ncdb)->servers.nymport=0;
    (*ncdb)->servers.nymtimeout=0;
    (*ncdb)->servers.tokenhost=NULL;
    (*ncdb)->servers.tokenport=0;
    (*ncdb)->servers.tokentimeout=0;
    (*ncdb)->servers.newsgrphost=NULL;
    (*ncdb)->servers.newsgrpport=0;
    (*ncdb)->servers.newsgrptimeout=0;
    (*ncdb)->servers.dnshost=NULL;
    (*ncdb)->servers.dnsport=0;
    (*ncdb)->servers.dnstimeout=0;
    (*ncdb)->servers.nidbhost=NULL;
    (*ncdb)->servers.nidbport=0;
    (*ncdb)->servers.nidbtimeout=0;
    (*ncdb)->servers.keyupdhost=NULL;
    (*ncdb)->servers.keyupdport=0;
    (*ncdb)->servers.keyupdtimeout=0;
    (*ncdb)->servers.keyqryhost=NULL;
    (*ncdb)->servers.keyqryport=0;
    (*ncdb)->servers.keyqrytimeout=0;
    (*ncdb)->servers.fmgmaiphost=NULL;
    (*ncdb)->servers.fmgmaipport=0;
    (*ncdb)->servers.fmgmaiptimeout=0;
    (*ncdb)->servers.maipport=0;
    (*ncdb)->servers.maiptimeout=0;

    (*ncdb)->config=NULL;
    (*ncdb)->numaips=0;
    (*ncdb)->numroutes=0;    
    (*ncdb)->availroutes=0;     
    (*ncdb)->numlinks=0;    
    (*ncdb)->overrideRoute=-1;
    (*ncdb)->sequencedRoutes=0;
    (*ncdb)->currSequenceRoute=0;
    (*ncdb)->disableRoutes=0;   /* Used when client is expired */
    (*ncdb)->randomized=0;
    (*ncdb)->aipinfo=NULL;
    (*ncdb)->routes=NULL;
    (*ncdb)->links=NULL;
    if (!zkMemPoolNew(err,&(*ncdb)->mpool)) {
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
    return 0;
  }
  return 1;
}

static int frNCDBFree (ZkError err, FrNCDB *ncdb)
{
  static const char *funcname = "frNCDBFree";

  if (ncdb) {
    if (*ncdb) {
      if (!zkMemPoolFree(err, &(*ncdb)->mpool))
	return 0;
      if ((*ncdb)->config) {
	if (!zkConfigFree(err,&((*ncdb)->config)))
	  zkDebug_va("zkConfigFree: %s", zkErrorDetail(err));
      }
#define FREEPTR(x) if (!(x)) free(x);
      FREEPTR((*ncdb)->aipinfo);
      FREEPTR((*ncdb)->routes);
      FREEPTR((*ncdb)->links);
#undef FREEPTR
    }
    free(*ncdb);
    *ncdb= NULL;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed; nothing to free");
    return 0;
  }
  return 1;    
}

#ifndef NDEBUG

int frNCDBAIPinfoDump (FrAIPinfo ai)
{
  const size_t  szp = 1024;
  char *p = (char *) NEW(szp);
  int   i = 0;
  
  snprintf (p,szp,"AIP [%2d] %-13s %-21s %20s:%5hu/%-5hu \n\t{p=0x%04x} {%2d} %-20s ",
	   ai->id, ai->entity, ai->name, ai->ip, ai->tcpport, ai->udpport,
	   ai->protocol, (int) ai->rating, ai->region);
  
  if (ai->isexit) {
    strncat (p, "exit ", 6);
  } else {
    strncat (p, "---- ", 6);

  }
  if (ai->isup) {
    strncat (p, "up ", 4);
  } else {
    strncat (p, "-- ", 4);
  }
  if (ai->isusable ) {
    strncat (p, "usable\n\t", 8);
  } else {
    strncat (p, "------\n\t", 8);
  }
  i = strlen (p); 
  snprintf(p+i, szp - i, " sin_addr=%-15s", inet_ntoa(ai->saddr.sin_addr));
  i = strlen (p); 
  snprintf (p+i, szp - i, " conn=%d", ai->numconn);
  zkDebug_va ("%s", p);
  free(p);
  return 1;
}

int RouteInfoDump (FrRoutes r)
{

  char *p = (char *) NEW(ZK_BUFSIZE+1);
  int pos = 0;
  int len = 0;
  int i = 0; 
  
  len = snprintf (p+pos, ZK_BUFSIZE-pos, "RTE [%2d] {%dh}: ", r->id, r->links);
  if (len >= 0) pos += len;
  else goto full;

  for (i = 0; i < r->links; i++) {
    len = snprintf (p+pos, ZK_BUFSIZE-pos, " ->%2d", r->aip[i]);
    if (len >= 0) pos += len;
    else goto full;
  }

  if (r->available ) {
    strncat (p, " avail", 6);
  } else {
    strncat (p, " -----", 6);
  }
  
 full:
  zkDebug_va ("%s", p);
  free (p);
  return 1;
}

int frNCDBLinkInfoDump (FrLinks h)
{
  char *p = (char *) NEW(ZK_BUFSIZE+1);
  int pos = 0;
  int len = 0;
  int i = 0; 
  
  len = snprintf (p+pos, ZK_BUFSIZE-pos, "LINK [%2d]: ", h->id);
  if (len >= 0) pos += len;
  else goto full;
  
  for (i = 0; i < 2; i++) {
    len = snprintf (p+pos, ZK_BUFSIZE-pos, " ->%2d", h->aip[i]);
    if (len >= 0) pos += len;
    else goto full;
  }

  len = snprintf (p+pos, ZK_BUFSIZE-pos, " {cost=%d}", h->cost);
  if (len >= 0) pos += len;
  
  
 full:
  zkDebug_va ("%s", p);
  free (p);
  return 1;
}

int frNCDBDump (FrNCDB ncdb)
{
  FrAIPinfo ai;
  FrRoutes rt;
  FrLinks lnk;
    
  zkDebug_va("Dumping NCDB: ");
	
  if (ncdb) {
    zkDebug_va ("[Network Configuration rev = %s]", ncdb->revision);
    zkDebug_va ("[INI File = %s]", ncdb->filename);
    zkDebug_va ("[Updated = %s]", ncdb->updated);
    zkDebug_va ("[Expirat = %s]", ncdb->expiration);
    zkDebug_va ("[URL     = %s]", ncdb->url);
    zkDebug_va ("[TTL (sec)= %lu]", (unsigned long) ncdb->ttl);
    zkDebug_va ("[Timeout = %s]", ncdb->timeout);
    //    zkDebug_va ("[Signtr  = %s]", ncdb->signature);

    zkDebug_va ("Num Aips = %2d; Num Rts  = %2d/%2d; Num Links = %2d",
	     ncdb->numaips, ncdb->availroutes, ncdb->numroutes, ncdb->numlinks);
    zkDebug_va ("{Override = %d}", ncdb->overrideRoute);
    zkDebug_va ("{SeqRoute = %d}", ncdb->sequencedRoutes);
    zkDebug_va ("{CurrSeqRt= %d}", ncdb->currSequenceRoute);
    zkDebug_va ("{Randomized?= %d}", ncdb->randomized);
    zkDebug_va ("{lastRoute= %d}", ncdb->lastSelectedRoute);

    if (ncdb->aipinfo) {
      for (ai = ncdb->aipinfo; ai && ai->id >= 0; ai++) {
	frNCDBAIPinfoDump (ai);
      }
    } else {
      zkDebug_va ("***No aipinfo");
    }
    if (ncdb->routes) {
      for (rt = ncdb->routes; rt && rt->id >= 0; rt++) {
	 RouteInfoDump (rt);
      }
    } else {
      zkDebug_va ("***No routes");
    }
    if (ncdb->links) {
      for (lnk = ncdb->links; lnk && lnk->id >= 0; lnk++) {
	 frNCDBLinkInfoDump (lnk);
      }
    } else {
      zkDebug_va ("***No links");
    }
  }
  return 1;
}

#endif /* NDEBUG */

static int getIniIndexEntry (struct T_IniSect *inisect, const char *token, int *id)
{
  int cnt = -1;

  while (inisect && inisect->token) {
    cnt ++;
    if (strncasecmp(token, inisect->token, strlen(token)+1) == 0) {
      /* this just changed, could be a bug */
      *id = inisect->id;
      break;
    }
    inisect++;
  }
  if (cnt < 0)
    return 0;
  return 1;
}
  
static int AIPStoreData(ZkError err, FrAIPinfo aipinfo,
			const char *token, void *ptr)
{
  const char *funcname = "AIPStoreData";
  int id = -2;
  int x = 0;
  /*  struct in_addr *inp; */
  
  if (!getIniIndexEntry (&AipIniProto[0], token, &id)) {
    return 0;
  }
  if (aipinfo && ptr) {
    switch (id) {
    case T_AIP_ID:
      aipinfo->id = *(int *)ptr;
      break;
    case T_AIP_ENTITY:
      /* BUG: Find a better way - some sort of mem pool */
      strncpy(aipinfo->entity,(char *)ptr, ZK_NDBNAME_SIZE);  
      break;
    case T_AIP_NAME:
      strncpy(aipinfo->name,(char *)ptr, ZK_NDBNAME_SIZE);
#if 0      
      /* Do the following to seed DNS local caches, at least under
       *  WIN32
       */
      {
	struct hostent *phe = NULL;

	if (!(phe = gethostbyname(aipinfo->name))) { /* audit: ok */
	  zkDebug_va("Problem with getting the aipinfo hostname");
	} else {
	  zkDebug_va("Seeded DNS entry for %s", aipinfo->name);
	}
      }
#endif      
      break;
    case T_AIP_IP:
      strncpy(aipinfo->ip,(char *)ptr, ZK_NDBNAME_SIZE);  
      aipinfo->saddr.sin_family = AF_INET;
      aipinfo->saddr.sin_port = 0;
      if (!inet_aton(aipinfo->ip, &(aipinfo->saddr.sin_addr))) {
	zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		      "couldn't covert ip address [%s]", aipinfo->ip);
	return 0;
      }
      break;
    case T_AIP_TCPPORT:
      x = *(int *)ptr;
      aipinfo->tcpport = x;
      break;
    case T_AIP_UDPPORT:
      x = *(int *)ptr;
      aipinfo->udpport = x;
      break;
    case T_AIP_PROTO:
      x = *(int *)ptr;
      aipinfo->protocol = x;
      aipinfo->isusable = x & FR_AIP_SUPPORTED_PROTOCOLS;
      break;
    case T_AIP_ISEXIT:
      aipinfo->isexit = *(int *)ptr;
      break;
    case T_AIP_ISUP:
      aipinfo->isup = *(int *)ptr;
      break;
    case T_AIP_REGION:
      strncpy(aipinfo->region, (char *)ptr, ZK_NDBNAME_SIZE);  
      break;
    case T_AIP_RATING:
      aipinfo->rating = *(int *)ptr;
      break;
    case T_AIP_DONE:
      break;
    default:
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKERROR,
		    "Couldn't determine structure element %s, %d", token, id);
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null pointer");
    return 0;
  }
  return 1;
}

static int RouteStoreData(ZkError err, FrRoutes routes,
			  const char *token, void *ptr)
{
  const char *funcname = "RouteStoreData";
  int id = -2;
  
  if (!getIniIndexEntry (&RouteIniProto[0], token, &id)) {
    return 0;
  }
  if (routes && ptr) {
    switch (id) {
    case T_RT_ID:
      routes->id = *(int *)ptr;
      break;
    case T_RT_AIP:
      routes->aip[routes->links++] = *(int *)ptr; 
      break;
    case T_RT_DONE:
      break;
    default:
      zkDebug_va("Couldn't determine structure element %d", id);
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Couldn't determine structure element");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null pointer");
    return 0;
  }
  return 1;
}

static int LinkStoreData(ZkError err, FrLinks lnk,
			 const char *token, void *ptr)
{
  static const char *funcname = "LinkStoreData";
  int id = -2;
  
  if (!getIniIndexEntry (&LinksIniProto[0], token, &id)) {
    return 0;
  }
  if (lnk && ptr) {
    switch (id) {
    case T_LINK_ID:
      lnk->id = *(int *)ptr;
      break;
    case T_LINK_AIP:
      lnk->aip[lnk->cnt++] = *(int *)ptr; 
      break;
    case T_LINK_COST:
      lnk->cost = *(int *)ptr; 
      break;
    case T_LINK_DONE:
      break;
    default:
      zkDebug_va("Couldn't determine structure element %d", id);
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Couldn't determine structure element");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null pointer");
    return 0;
  }
  return 1;
}

static int frNCDBLoadAipEntry(ZkError err, FrNCDB ncdb,
			      const char *section)
{
  static const char *funcname = "frNCDBLoadAipEntry";
  static int True = 1;
  static int False = 0;
  
  IniSect proto = NULL;
  FrAIPinfo aipinfo = NULL; /* The AIP we're interested in... */
  const char *data = NULL;
  int id = 0;

  /* First we get the id, to put it in the proper AIP array location */
  if (!zkConfigGet(err, ncdb->config, section, "id", &data)) {
    return 0;
  }
  /* We know this has to be a int */
  if (sscanf (data, "%d", &id) != 1) { /* audit: ok */
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Couldn't convert uint");
    return 0;
  }

  /* Now we set the proper AipTable index */
  aipinfo = &(ncdb->aipinfo[id]);
  aipinfo->id = id;
  
  /* Good. Now we load in the rest */
  /* We do this by inspecting all of the fields in the
     section prototype array; start at the top */
  proto=&AipIniProto[0];

  while (proto && proto++ && proto->type != INI_DONE) {
    if (strncasecmp(proto->token, "id", 3) == 0) {
      /* We've already done the ID, skip it */
      proto++;
      continue;
    }

    /* Let's find this guy; use a void pointer to handle the data */
    if (zkConfigGet(err, ncdb->config, section, proto->token, &data)) {
      void *ptr = NULL;
      int val = 0;
      /* extract the data according to the type */
      switch (proto->type) {
      case INI_INT:
      case INI_UINT16:
	ptr = (void *) & val;
	if (sscanf (data, "%d", (int *)ptr) != 1) { /* audit: ok */
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert uint");
	  return 0;
	}       
	break;
      case INI_CHAR:
	if (!zkMemPoolStrndup (err, ncdb->mpool, data, strlen(data)+1, (char **) &ptr)) {
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert string in MemPoolStrndup");
	  return 0;
	}
	break;
      case INI_BOOL:
	ptr = (void *) &val;
	if (strncasecmp ("true", data, 5) == 0) {
	  (void) memcpy(ptr, &True, sizeof(int));
	} else if (strncasecmp ("false", data, 6) == 0) {
	  (void) memcpy(ptr, &False, sizeof(int));
	} else {
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert bool");
	  return 0;
	}
	break;
      default:
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't determine conversion type");
	return 0;
      }
      /* Integrate this data into the AIP table */
      if (!AIPStoreData(err, aipinfo, proto->token, ptr)) {
	zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't store AIP data");
	return 0;
      }
    }
  }
  /* Seed the DNS Cache at this point */
  return 1;
}

static int frNCDBLoadRouteEntry(ZkError err, FrNCDB ncdb,
				const char *section)
{
  static const char *funcname = "frNCDBLoadRouteEntry";
  
  IniSect proto = NULL;
  FrRoutes routes = NULL; /* The AIP we're interested in... */
  const char *data = NULL;
  int id = 0;
  int val = 0;
  int getfirst = 1;
  int retcode = 1;

  /* First we get the id, to put it in the proper AIP array location */
  if (!zkConfigGet(err, ncdb->config, section, "id", &data)) {
    return 0;
  }
  /* We know this has to be a int */
  if (sscanf (data, "%d", &id) != 1) { /* audit: ok */
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Couldn't convert uint");
    return 0;
  }

  /* Now we set the proper AipTable index */
  routes = &(ncdb->routes[id]);
  routes->id = id;
  
  /* Good. Now we load in the rest */
  /* We do this by inspecting all of the fields in the
     section prototype array; start at the top */
  proto=&RouteIniProto[0];
  if (proto) proto++;

  while (proto && proto->type != INI_DONE) {
    if (strncasecmp(proto->token, "id", 3) == 0) {
      /* We've already done the ID, skip it */
      proto++;
      continue;
    }

    /* Let's find this guy; use a void pointer to handle the data */
    if (getfirst){ 
      retcode = zkConfigGet(err, ncdb->config, section, proto->token, &data);
    } else {
      retcode = zkConfigGetNext (err, ncdb->config, &data);
    }
    if (retcode) {
      /* extract the data according to the type */
      switch (proto->type) {
      case INI_INT:
	if (sscanf (data, "%d", &val) != 1) { /* audit: ok */
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert uint");
	  return 0;
	}       
	break;
      default:
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't determine conversion type");
	return 0;
      }
      /* Integrate this data into the Route table */
      if (!RouteStoreData(err, routes, proto->token, (void *)&val)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't store Route data");
	return 0; 
      }
      if (proto->count) proto++;
      else getfirst = 0;
    } else {
      proto++;
      getfirst = 1;
    }
  }
  return 1;
}

static int frNCDBLoadLinkEntry(ZkError err, FrNCDB ncdb,
			       const char *section)
{
  static const char *funcname = "frNCDBLoadLinkEntry";
  
  IniSect proto = NULL;
  FrLinks lnks = NULL; /* The LINKS we're interested in... */
  const char *data = NULL;
  int id = 0;
  int val= 0;
  int getfirst = 1;
  int retcode = 1;

  /* First we get the id, to put it in the proper AIP array location */
  if (!zkConfigGet(err, ncdb->config, section, "id", &data)) {
    return 0;
  }
  /* We know this has to be a int */
  if (sscanf (data, "%d", &id) != 1) { /* audit: ok */
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Couldn't convert uint");
    return 0;
  }

  /* Now we set the proper AipTable index */
  lnks = &(ncdb->links[id]);
  lnks->id = id;
  
  /* Good. Now we load in the rest */
  /* We do this by inspecting all of the fields in the
     section prototype array; start at the top */
  proto=&LinksIniProto[0];
  if (proto) proto++;

  while (proto && proto->type != INI_DONE) {
    if (strncasecmp(proto->token, "id", 3) == 0) {
      /* We've already done the ID, skip it */
      proto++;
      continue;
    }

    /* Let's find this guy; use a void pointer to handle the data */
    if (getfirst){ 
      retcode = zkConfigGet(err, ncdb->config, section, proto->token, &data);
    } else {
      retcode = zkConfigGetNext (err, ncdb->config, &data);
      getfirst = 1;
    }
    if (retcode) {
      /* extract the data according to the type */
      switch (proto->type) {
      case INI_INT:
	if (sscanf (data, "%d", &val) != 1) { /* audit: ok */
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert uint");
	  return 0;
	}       
	break;
      default:
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't determine conversion type");
	return 0;
      }
      /* Integrate this data into the Route table */
      if (!LinkStoreData(err, lnks, proto->token, (void *)&val)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't store Route data");
	return 0; 
      }
      if (proto->count) proto++;
      else getfirst = 0;
    } else {
      proto++;
      getfirst = 1;
    }
  }
  return 1;
}

static int frNCDBLoadFromBuffer(ZkError err, FrNCDB ncdb, char *buffer,
				size_t bufsize)
{
  static const char *funcname = "frNCDBLoadFromBuffer";
  
  /* --------------- [ START of New zkConf Stuff ] ------- */
  struct T_ZkConfSectionProto genSectProto[] = {
    {"Revision",	cvtStr, ZKREQ, &ncdb->revision,	NULL},
    {"Updated",		cvtStr, ZKREQ, &ncdb->updated,	NULL},
    {"Expiration",	cvtZkTime, ZKREQ, &ncdb->expiration, NULL},
    {"By",		cvtStr, ZKOPT, NULL,		NULL},
    {"CVS",		cvtStr, ZKOPT, NULL,		NULL},
    {"TTL", 		cvtTTL, ZKREQ, &ncdb->ttl, 	NULL},
    {"DownloadURL",	cvtStr, ZKREQ, &ncdb->url, 	NULL},
    {"", NULL, ZKOPT, NULL, NULL}
  };

#define STR2(x) #x  
#define STR(x)  STR2(x)

  struct T_ZkConfSectionProto serverSectProto[] = {
    /* Bug: This has to change */
    {"AipUDPPort",	cvtPort, ZKREQ, &ncdb->servers.aipudpport,
     STR(FR_DFLT_AIP_UDP_PORT)},
    {"AipTCPPort",	cvtPort, ZKREQ, &ncdb->servers.aiptcpport,
     STR(FR_DFLT_AIP_TCP_PORT)},
    {"ClientUdpTcpPort",cvtPort, ZKREQ, &ncdb->servers.clientudptcpport,
     STR(FR_DFLT_CLIENT_UDP_TCP_PORT)},
    {"NymHost",	        cvtStr,	 ZKREQ, &ncdb->servers.nymhost,
     FR_DFLT_NYM_SERVER_HOST},
    {"NymIP",	        cvtStr,	 ZKREQ, &ncdb->servers.nymip,
     NULL},
    {"NymPort",		cvtPort, ZKREQ, &ncdb->servers.nymport,
     STR(FR_DFLT_NYM_SERVER_PORT)},
    {"NymTimeout",	cvtInt,  ZKREQ, &ncdb->servers.nymtimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"TokenHost",	cvtStr,	 ZKREQ, &ncdb->servers.tokenhost,
     FR_DFLT_TOKEN_SERVER_HOST},
    {"TokenIP",	cvtStr,	 ZKREQ, &ncdb->servers.tokenip,
     NULL},
    {"TokenPort",	cvtPort, ZKREQ, &ncdb->servers.tokenport,
     STR(FR_DFLT_TOKEN_SERVER_PORT)},
    {"TokenTimeout",	cvtInt,  ZKREQ, &ncdb->servers.tokentimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"NewsgrpHost",	cvtStr,	 ZKREQ, &ncdb->servers.newsgrphost,
     FR_DFLT_NEWSGRP_SERVER_HOST},
    {"NewsgrpIP",	cvtStr,	 ZKREQ, &ncdb->servers.newsgrpip,
     NULL},
    {"NewsgrpPort",	cvtPort, ZKREQ, &ncdb->servers.newsgrpport,
     STR(FR_DFLT_NEWSGRP_SERVER_PORT)},
    {"NewsgrpTimeout",	cvtInt,  ZKREQ, &ncdb->servers.newsgrptimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"DnsHost",		cvtStr,	 ZKREQ, &ncdb->servers.dnshost,
     FR_DFLT_DNS_SERVER_HOST},
    {"DnsIP",		cvtStr,	 ZKREQ, &ncdb->servers.dnsip,
     NULL},
    {"DnsPort",		cvtPort, ZKREQ, &ncdb->servers.dnsport,
     STR(FR_DFLT_DNS_SERVER_PORT)},
    {"DnsTimeout",		cvtInt,  ZKREQ, &ncdb->servers.dnstimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"NidbHost",	cvtStr,	 ZKREQ, &ncdb->servers.nidbhost,
     FR_DFLT_NIDB_SERVER_HOST},
    {"NidbIP",	cvtStr,	 ZKREQ, &ncdb->servers.nidbip,
     NULL},
    {"NidbPort",	cvtPort, ZKREQ, &ncdb->servers.nidbport,
     STR(FR_DFLT_NIDB_SERVER_PORT)},
    {"NidbTimeout",	cvtInt,  ZKREQ, &ncdb->servers.nidbtimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"KeyupdHost",	cvtStr,	 ZKREQ, &ncdb->servers.keyupdhost,
     FR_DFLT_KEYUPD_SERVER_HOST},
    {"KeyupdIP",	cvtStr,	 ZKREQ, &ncdb->servers.keyupdip,
     NULL},
    {"KeyupdPort",	cvtPort, ZKREQ, &ncdb->servers.keyupdport,
     STR(FR_DFLT_KEYUPD_SERVER_PORT)},
    {"KeyupdTimeout",	cvtInt,  ZKREQ, &ncdb->servers.keyupdtimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"KeyqryHost",	cvtStr,	 ZKREQ, &ncdb->servers.keyqryhost,
     FR_DFLT_KEYQRY_SERVER_HOST},
    {"KeyqryIP",	cvtStr,	 ZKREQ, &ncdb->servers.keyqryip,
     NULL},
    {"KeyqryPort",	cvtPort, ZKREQ, &ncdb->servers.keyqryport,
     STR(FR_DFLT_KEYQRY_SERVER_PORT)},
    {"KeyqryTimeout",	cvtInt,  ZKREQ, &ncdb->servers.keyqrytimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"FmgmaipHost",	cvtStr,	 ZKREQ, &ncdb->servers.fmgmaiphost,
     FR_DFLT_FMGMAIP_SERVER_HOST},
    {"FmgmaipIP",	cvtStr,	 ZKREQ, &ncdb->servers.fmgmaipip,
     NULL},
    {"FmgmaipPort",	cvtPort, ZKREQ, &ncdb->servers.fmgmaipport,
     STR(FR_DFLT_FMGMAIP_SERVER_PORT)},
    {"FmgmaipTimeout",	cvtInt,  ZKREQ, &ncdb->servers.fmgmaiptimeout,
     STR(FR_DFLT_SERVER_PORT_TIMEOUT)},
    {"MaipPort",	cvtPort, ZKREQ, &ncdb->servers.maipport,
     STR(FR_DFLT_MAIP_PORT)},
    {"MaipTimeout",	cvtInt,  ZKREQ, &ncdb->servers.maiptimeout,
     STR(FR_MAIP_PORT_TIMEOUT)},
    {"TrialSerialNumberURL", cvtStr, ZKOPT, &ncdb->servers.trialserialURL,
     FR_DFLT_TRIAL_SERNO_URL},
    {"", NULL, ZKOPT, NULL, NULL}
  };
#undef STR
#undef STR2
  
  struct T_ZkConfSectionProto windowsClientSectProto[] = {
    {"Revision",	cvtStr,  ZKREQ, &ncdb->winclient.revision,	NULL},
    {"Updated", 	cvtStr,  ZKREQ, &ncdb->winclient.updated,	NULL},
    {"InfoURL", 	cvtStr,  ZKREQ, &ncdb->winclient.infoURL, 	NULL},
    {"upgradeURL", 	cvtStr,  ZKREQ, &ncdb->winclient.upgradeURL, 	NULL},
    {"DownloadURL", 	cvtStr,  ZKREQ, &ncdb->winclient.downloadURL,	NULL},
    {"Urgency", 	cvtUrgency, ZKOPT, &ncdb->winclient.urgency, 	NULL},
    {"ExpireClient",    cvtStr,  ZKOPT, &ncdb->winclient.expireClient, 	NULL},
    {"", NULL, ZKOPT, NULL, NULL}
  };

  struct T_ZkConfSectionProto signatureSectProto[] = {
    /*    {"Signature", 	cvtStr, ZKREQ, &ncdb->signature, NULL}, */
    {"", NULL, ZKOPT, NULL, NULL}
  };
  
  struct T_ZkConfProto conf_proto[] = {
    {"Signature",	signatureSectProto},
    {"General",		genSectProto},
    {"WindowsClient", 	windowsClientSectProto},
    {"Servers", 	serverSectProto},
    {"", NULL}  
  };

  size_t sections = 0;
  size_t i = 0; /* Loop index */
  size_t numaips=0;
  size_t numroutes=0;
  size_t numlinks=0;
  size_t numdunno=0;
  
  /* Check the required sections along with the signature */
  /* Bug: Going to have to change this call soon */
  if (!zkConfLoadFromBuffer (err, buffer, bufsize, conf_proto, ncdb->mpool)) {
    if (zkErrorCode(err) == ZKENOTFOUND) {
      zkDebug_va ("Config file error: [%s]", zkErrorDetail(err));
    } else {
      zkDebug_va ("ConfLoadFile failed");
    }
    return 0;
  }
  
  if (!zkConfigNew(err,&(ncdb->config))) {
    /* Bug: cleanup */
    return 0;
  }
  
  if (!zkConfigLoadFromBuffer(err, ncdb->config, buffer, bufsize)) {
    /* Bug: cleanup */
    return 0;
  }
  if (!zkConfigSections(err,ncdb->config,&sections)) {
    /* Bug: cleanup */
    return 0;
  }
  for (i = 0; i < sections; i++) {
    const char *name;
    if (zkConfigSection(err,ncdb->config, i, &name)) {
      if (!strncasecmp ("signature", name, 10)) {
	/* Ignore */
      } else if (!strncasecmp ("general", name, 8)) {
	/* Ignore */
      } else if (!strncasecmp ("servers", name, 8)) {
	/* Ignore */
      } else if (!strncasecmp ("windowsclient", name, 14)) {
	/* Ignore */
      } else if (!strncasecmp ("aip", name, 3)) {
	numaips += 1;
      } else if (!strncasecmp ("route", name, 5)) {
	numroutes += 1;
      } else if (!strncasecmp ("link", name, 3)) {
	numlinks += 1;
      } else
	numdunno += 1;
    } else {
      return 0;
    }
  }

  /* Allocate the datastructures */
  if (numaips) {
    if (!(ncdb->aipinfo =
	  (FrAIPinfo) calloc (numaips+1, sizeof(struct T_FrAIPinfo)))){
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for aipinfo");
      return 0;
    }
    if (!zkMemPoolRegister(err, ncdb->mpool, (void *) ncdb->aipinfo,
			   sizeof(struct T_FrAIPinfo) * (numaips+1)))
      return 0;
    ncdb->aipinfo[numaips].id = -1;
    ncdb->numaips = numaips;
  } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "No aips found in INI file");
      return 0;
  }
  if (numroutes) {
    if (!(ncdb->routes = (FrRoutes) calloc (numroutes+1, sizeof (struct T_FrRoutes)))) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for route table");
      return 0;
    }
    if (!zkMemPoolRegister(err, ncdb->mpool, (void *) ncdb->routes,
			   sizeof(struct T_FrRoutes) * (numroutes+1)))
      return 0;
    ncdb->routes[numroutes].id = -1;
    ncdb->numroutes = numroutes;
  } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "No routes found in INI file");
      return 0;
  }
  if (numlinks) {
    if (!(ncdb->links = (FrLinks) calloc (numlinks+1, sizeof (struct T_FrLinks)))) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for link table");
      return 0;
    }
    if (!zkMemPoolRegister(err, ncdb->mpool, (void *) ncdb->links,
			   sizeof(struct T_FrLinks) * (numlinks+1)))
      return 0;
    ncdb->links[numlinks].id = -1;
    ncdb->numlinks = numlinks;
  } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "No links found in INI file");
      return 0;
  }

  /* Now load the datastructures from the config file */
  for (i = 0; i < sections; i++) {
    const char *name;
    if (zkConfigSection(err,ncdb->config,i, &name)) {
      if (!strncasecmp ("signature", name, 10)) {
	/* Do nothing for now */
      } else if (!strncasecmp ("servers", name, 8)) {
	/* Ignore */
      } else if (!strncasecmp ("general", name, 8)) {
	/* Do nothing for now */
      } else if (!strncasecmp ("windowsclient", name, 14)) {
	/* Do nothing for now */
      } else if (!strncasecmp ("aip", name, 3)) {
	frNCDBLoadAipEntry(err, ncdb, name);
      } else if (!strncasecmp ("route", name, 5)) {
	frNCDBLoadRouteEntry(err, ncdb, name);
      } else if (!strncasecmp ("link", name, 3)) {
	frNCDBLoadLinkEntry(err, ncdb, name);
      } else {
            zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Unknown section to load");
	    return 0;
      }
    } else {
      return 0;
    }
  }
  if (!zkConfigFree(err,&(ncdb->config))){
     return 0; /* Bug: improper cleanup */
  }
  /* Bug: Close the config file... */
  return 1;
}

/** 
 * Upon a return of 0, the error code should be checked for ZKEDEFUNCT
 * which indicates that the client has expired.
 * It is IMPERATIVE to do a 
 *
 *	if (ZkErrorCode(err) == ZKEDEFUNCT) {
 *		Advise the user that his client has expired for good
 *              No routes will be available.
 *	}
 */
static int determineExpireAndTTL (ZkError err, FrNCDB ncdb, time_t *timeout)
{
  static const char *funcname = "determineExpireAndTTL";
  
  ZkDateTime currdt, expdt;
  int delta;
  time_t tout;
  char *buf1;

  if (!ncdb) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
    return 0;
  }
  
  zkMALLOC (buf1, char *, ZKDATETIMESTR_SZ);
  
  if (!zkDateTimeSetCurrent(err, &currdt, ZKTIME_FL_UTC)) {
    free (buf1);
    return 0;
  }

  if (!zkDateTimeParse (err, &expdt, ncdb->expiration)) {
    zkDebug_va("Can't parse the internal time!!! [%s]", ncdb->expiration);
    return 0;
  }

  /* Is our file expired */
  if (!zkDateTimeCompare(err, &currdt, &expdt , &delta))
    return 0;
  if (delta >= 0) {
    zkDebug_va ("Network configuration file has expired!!");
    tout = 0;
    if (!zkDateTimeStore (err, &currdt, buf1, ZKDATETIMESTR_SZ)) {
      free(buf1);
      return 0;
    }
    zkDebug_va ("Timeout will be set to [now] [%s]", buf1);
  } else {
    zkDebug_va ("Network configuration file still valid");
    /* Figure out when to next call us */
    /* Bug: Consider this timout a bit more closely */
    tout = ncdb->ttl;
    if (!zkDateTimeAdd (err, &currdt, ncdb->ttl, ZKDATETIME_SECONDS)) {
      free(buf1);
      return 0;
    }
    if (!zkDateTimeStore (err, &currdt, buf1, ZKDATETIMESTR_SZ)) {
      free(buf1);
      return 0;
    }
  }
#ifndef NDEBUG  
  ncdb->timeout = strdup ("300");
  ncdb->ttl = 300;
  zkDebug_va("** ** ** DEBUG TIMEOUT SET TO 5 MINUTES");
#else  
  ncdb->timeout = buf1;

  if (tout == 0) 
    ncdb->ttl = FR_NCDB_EXPIRED_TTL;
#endif  

  if (!zkMemPoolRegister(err, ncdb->mpool, (void *) buf1, ZKDATETIMESTR_SZ))
    return 0;
  if (timeout) {
    *timeout = tout;
  } else {
    zkErrorSet(err, funcname, ZKWARN, ZKMOD,
	       ZKENULLPTR, "Client will not be informed of timeout");
    zkDebug_va ("WARNING: Client will not be informed of NDB timeouts");
  }

#if (!defined (WIN32) && !defined(NDEBUG)) || defined(WIN32)
  {
    int major, minor;
    int minmajor, minminor;

    /* Let's check to see if this client is too old to run on our network */
    if (sscanf (FR_WINDOWSCLIENT_VERSION, "%d.%d", &major, &minor)) { /* audit: ok */
      if (ncdb->winclient.expireClient) {
	if (sscanf (ncdb->winclient.expireClient, "%d.%d",&minmajor,&minminor)) { /* audit: ok */
	  if ((major < minmajor) || (major == minmajor && minor <= minminor)) {
	    /* This client will not be allowed to connect to our network */
	    zkDebug_va ("Client (ver %s) is too old to use the Freedom network.",
			FR_WINDOWSCLIENT_VERSION);
	    zkErrorSet_va(err, funcname, ZKWARN, ZKMOD, ZKEDEFUNCT,
		  "Client (ver %s) is too old to use the Freedom network.",
			  FR_WINDOWSCLIENT_VERSION);
	    ncdb->disableRoutes = 1;
	    return 1;
	  }
	}
      }
    } else {
      zkErrorSet(err, funcname, ZKWARN, ZKMOD,
	       ZKEPARSE, "Could not extract the client version");
    }
  }
#endif  
  return 1;
}

static int frNCDBcomputeAvailRoutes (ZkError err, FrNCDB ncdb)
{
  static const char *funcname = "frNCDBncomputeAvailRoutes";

  /* Visit each route and check to see if there is a problem
   * with any AIP in it
   */
  FrRoutes rt;
  FrAIPinfo ai;
  int i;

  
  if (ncdb && ncdb->routes) {
    ncdb->availroutes = 0;
    ai = ncdb->aipinfo;
    for (rt = ncdb->routes; rt && rt->id >= 0; rt++) {
      rt->available = 1; /* Assume OK */
      for (i = 0; i < rt->links; i++) {
	if ((!ai[rt->aip[i]].isup) || (!ai[rt->aip[i]].isusable)) {
	  rt->available = 0;
	  break;
	}
      }
      if (rt->available) {
	ncdb->availroutes++;
	zkDebug_va("Route [%2d] is available", rt->id);
      }
      else
	zkDebug_va("Route [%2d] is NOT available", rt->id);
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
    return 0;
  }
  return 1;
}

    
  
/** It is IMPERATIVE to do a 
 *
 *	if (ZkErrorCode(err) == ZKEDEFUNCT) {
 *		Advise the user that his client has expired for good
 *              No routes will be available.
 *	}
 */
int frNCDBLoad(ZkError err, FrNCDB ncdb, time_t *timeout)
{
  char *buffer;
  size_t size;

  /* First, get the file */
  if (!zkFileLoad (err, ncdb->filename, &buffer, &size)) {
    return(0);
  }
  if (!frNCDBLoadFromBuffer (err, ncdb, buffer, size)) {
    free(buffer);
    return 0;
  };
  free(buffer);

  /* Next, Do a sanity check. */
  if (!determineExpireAndTTL (err, ncdb, timeout))
    return 0;

  if (!frNCDBcomputeAvailRoutes(err, ncdb)) {
    return 0;
  }
  
  return 1;
}

static int frNCDBUnLoad(ZkError err, FrNCDB *ncdb)
{
  if(!frNCDBFree(err, ncdb))
    return 0;
  return (1);
}

/* DO NOT CALL DIRECTLY  - use frNDBOpen or frNDBEnv instead */

/** It is IMPERATIVE to do a 
 *
 *	if (ZkErrorCode(err) == ZKEDEFUNCT) {
 *		Advise the user that his client has expired for good
 *              No routes will be available.
 *	}
 */
DLLExport int frNCDBInit (ZkError err, FrNCDB *ncdb, const char *configfile,
			  time_t *timeout)
{
  static const char *funcname = "frNCDBInit";
  static int instances = 0;

  if (instances) {
    zkErrorSet(err, funcname, ZKWARN, ZKMOD,
	       ZKERROR,
      "Only once instance of the Network Configuration Database can exist.");
  } else {
    instances = 1;
    if (!frNCDBNew(err, ncdb))
      return 0;
  }
  if (configfile) {
    if (*((*ncdb)->filename) == '\0') {
      strncpy((*ncdb)->filename, configfile, ZK_NDBNAME_SIZE*2);  
      zkDebug_va("Filename %s", (*ncdb)->filename);
      if (!frNCDBLoad (err, *ncdb, timeout)) {
	return (0);
      }
    } else {
      zkErrorSet(err, funcname, ZKWARN, ZKMOD,
		 ZKERROR,
		 "Attempt to change configuration file setting ignored.");
    }
  }
  return 1;
}

DLLExport int frNCDBClose (ZkError err, FrNCDB *ncdb)
{
  return (frNCDBUnLoad(err, ncdb));
}

int frNCDBGetAipInfoFromId (ZkError err, FrNCDB ncdb, int aip, FrAIPinfo *ai)
{
  static const char *funcname = "frNCDBGetAipInfoFromId";
  if ( ! ncdb || !ai) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB or ai");
    return 0;
  }

  if (aip < 0 || aip >= ncdb->numaips) {
    zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	       ZKEPARMRANGE, "aip ID [%d] is out of range", aip);
    return 0;
  }

  /*
   * Bug: this should become the only place where we get the AIPinfo
   * structure.
   */

  *ai = &ncdb->aipinfo[aip];
  return 1;
}

int frNCDBGetLinkInfoFromId (ZkError err, FrNCDB ncdb, int linkId, FrLinks *lnk)
{
  static const char *funcname = "frNCDBGetLinkInfoFromId";

  if ( ! ncdb || !lnk) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB or ai");
    return 0;
  }

  if (linkId < 0 || linkId >= ncdb->numlinks) {
    zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	       ZKEPARMRANGE, "Link ID [%d] is out of range", linkId);
    return 0;
  }

  /*
   * Bug: this should become the only place where we get the AIPinfo
   * structure.
   */

  *lnk = &ncdb->links[linkId];
  return 1;
}

/*
 * BUG: This feature is probably a security problem
 * Make sure that the code ignores these overrides in
 * the final distribution
 */
DLLExport
int frNCDBSetOverrideRoute (ZkError err, FrNCDB nc, int override)
{
  static const char *funcname = "frNCDBSetOverrideRoute";

  if (nc) {
    if (override >= 0 && override <= nc->numroutes) {
      nc->overrideRoute = override;
    } else {
      zkErrorSet(err, funcname, ZKWARN, ZKMOD,
		 ZKERROR, "requested override is not within range");
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBResetOverrideRoute (ZkError err, FrNCDB nc)
{
  static const char *funcname = "frNCDBResetOverrideRoute";

  if (nc) {
    nc->overrideRoute = -1;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNCDBGetOverrideRoute (ZkError err, FrNCDB nc, int *override)
{
  static const char *funcname = "frNCDBGetOverrideRoute";

  if (nc && override) {
      *override = nc->overrideRoute;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNCDBIsOverrideRouteSet (ZkError err, FrNCDB nc)
{
  static const char *funcname = "frNCDBIsOverrideRouteSet";

  if (nc) {
    if (nc->overrideRoute >= 0) {
      return 1;
    } else {
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
}

/*
 * BUG: This feature is probably a security problem
 * Make sure that the code ignores these overrides in
 * the final distribution
 */
DLLExport
int frNCDBSetSequencedRoute (ZkError err, FrNCDB nc)
{
  static const char *funcname = "frNCDBSetSequencedRoute";

  if (nc) {
    /* Bug: probably have to make sure nothing else is on */
    nc->sequencedRoutes = 1;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBResetSequencedRoute (ZkError err, FrNCDB nc)
{
  static const char *funcname = "frNCDBResetSequencedRoute";

  if (nc) {
    /* Bug: probably have to make sure nothing else is on */
    nc->sequencedRoutes = 0;
    nc->currSequenceRoute = 0;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBIsGetRoutesInSequenceSet (ZkError err, FrNCDB nc)
{
  static const char *funcname = "frNCDBIsGetRoutesInSequenceSet";

  if (nc) {
    return (nc->sequencedRoutes = 0);
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
}
  
DLLExport
int frNCDBGetNextRouteInSequence (ZkError err, FrNCDB nc, int *nxtRoute)
{
  static const char *funcname = "frNCDBGetNextRouteInSequence";
  if (nc) {
    if (nc->sequencedRoutes) {
      *nxtRoute = nc->currSequenceRoute;
      if (++nc->currSequenceRoute >= nc->numroutes) {
	nc->currSequenceRoute = 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKWARN, ZKMOD,
		 ZKERROR, "Sequenced routes is not set");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}
  

DLLExport
int frNCDBGetMaxNumRoutes (ZkError err, FrNCDB nc, int *routes)
{
  static const char *funcname = "frNCDBGetMaxNumRoutes";
  if (nc) {
    *routes = nc->numroutes;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Network pointer");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetRandomRoute (ZkError err, FrNCDB ncdb, int *route)
{  
  static const char *funcname = "frNCDBGetRandomRoute";
  uint32_t randata;
  int done = 0;
  
  if (ncdb) {
    if (ncdb->availroutes == 0) {
      zkErrorSet (err, funcname, ZKWARN, ZKMOD, ZKEEMPTY,
		  "No routes are available");
      *route = -1;
      return 1;
    }
    while (!done) {
      if (!zkRandGetData(err, &randata, sizeof(uint32_t))) {
	return 0; /* bug */
      }
      /* bug: quick hack here - need to check this out */
      *route = abs(randata%(ncdb->numroutes));
      if (ncdb->routes[*route].available) {
	zkDebug_va("Route %d  -  [randata=%lu  numroutes=%u]",
		   *route, (long unsigned int)randata, ncdb->numroutes);
	done = 1;
      } else {
	zkDebug_va("Route %d  is not available for use", *route);
      }
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Network pointer");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetNumLinksForRouteId (ZkError err, FrNDB ndb, int rid, int *nlinks)
{
  static const char *funcname = "frNCDBGetNumLinksforRouteId";

  if (ndb && ndb->ncdb && nlinks) {
    if (rid >= 0 && rid <= ndb->ncdb->numroutes) {
      *nlinks = ndb->ncdb->routes[rid].links;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Route ID is out of range");
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetAipXinRouteId (ZkError err, FrNDB ndb, int rid, int hop, int *aip)
{
  static const char *funcname = "frNRDBGetAipXinRouteId";

  if (ndb && ndb->ncdb && aip) {
    if (rid >= 0 && rid <= ndb->ncdb->numroutes) {
      if (hop >= 0 && hop < ndb->ncdb->routes[rid].links) {
	*aip = ndb->ncdb->routes[rid].aip[hop];
      } else {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Route ID is out of range");
	return 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Route ID is out of range");
      return 0;
      
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}


/**
 * Return a list of aip names
 * Please do a free(aipnamelist) when you're done.
 */
DLLExport
int frNCDBGetAllAipNames (ZkError err, FrNDB ndb, 
			  const char ***aipnamelist, int *numaips)
{
  static const char *funcname = "frNCDBGetAllAipNames";
  FrAIPinfo ai;
  const char **list;

  /* A few sanity checks */
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!(ndb->ncdb && aipnamelist && numaips)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Null parameter");
    return 0;
  }

  if (!ndb->ncdb->aipinfo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No Aipinfo available");
    return 0;
  }

  if (!(list =  (const char **) calloc (ndb->ncdb->numaips,
				 sizeof(char *)))){
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for aipname");
      return 0;
  }

  *aipnamelist = list;
    
  for (ai = ndb->ncdb->aipinfo; ai && ai->id >= 0; ai++, list++) {
    *list = ai->name;
  }
  *numaips = ndb->ncdb->numaips;

  return 1;
}

/* Bug: change this to a dynamic list of pointers to free */
/* Bug: will only be able to free the last namelist used */
/* Will need to make this a hash of char* to NameList Table */
/* and then look up the char* and free the corresponding NameList */
static NameList lastNameList = NULL;

DLLExport
int frNCDBGetAllAipNamesForRouteId (ZkError err, FrNDB ndb, int rid,
				    char ***aipnamelist, int *numaips)
{
  static const char *funcname = "frNCDBGetAllAipNamesForRouteId";
  int i, hop;
  char *p;
  NameList namelist;

  /* A few sanity checks */
  if (!(ndb && ndb->ncdb && aipnamelist && numaips)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENULLPTR, "Null parameter");
    return 0;
  }
  if (!(rid >= 0 && rid <= ndb->ncdb->numroutes)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Route ID is out of range");
    return 0;
  }

  zkMALLOC(namelist, NameList, (sizeof(struct T_NameList)));
  lastNameList = namelist; /* Use this to free things up */
  namelist->num = ndb->ncdb->routes[rid].links;
  p = namelist->buf;

  for (i = 0 ; i < namelist->num; i++) {
    hop = ndb->ncdb->routes[rid].aip[i];
    namelist->list[i] = p;
    namelist->list[i+1] = NULL; /* Precaution */
    strncpy(p, ndb->ncdb->
	    aipinfo[hop].name, ZK_NDBNAME_SIZE);
    p = strchr(p, '\0') + 1; /* Move the pointer ahead */
  }
  *numaips = namelist->num;
  *aipnamelist = namelist->list;

  return 1;
}

DLLExport
int frNCDBFreeAipNames (ZkError err, FrNDB ndb, char ***aipnamelist)
{
  static const char *funcname = "frNCDBFreeAipNames";

  if (!ndb) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENULLPTR, "Null parameter");
    return 0;
  }
  if (!lastNameList) {
    free(lastNameList);
    lastNameList = NULL;
    *aipnamelist = NULL;
  }
  return 1;
}

DLLExport
int frNCDBGetEntityForAipID (ZkError err, FrNDB ndb, int aip, const char **entity)
{
  static const char *funcname = "frNCDBGetEntityForAipID";

  if (ndb && ndb->ncdb && entity) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      *entity = ndb->ncdb->aipinfo[aip].entity;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/**
 * Get the AipID from the hostname
 * @return        0 on system error (pointers, missing datastructures etc)
 *                1 otherwise
 * @oaran ndb     NDB can be NULL
 * @param aipid   0 to numaips if the hostname was found among the aips
 *               -1            if no aips were found
 *                             ZKWARN is set with the detail describing the prob
 */
DLLExport
int frNCDBGetAipIDFromHostname (ZkError err, FrNDB ndb, const char *hostname,
				int *aip)
{
  static const char *funcname = "frNCDBGetAipIDFromHostname";
  FrAIPinfo ai;
  int found = 0, i;
  struct in_addr ip_addr;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb) && !ndb->ncdb) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NCDB");
    return 0;
  }

  if (!frNCDBGetIPfromHostname (err, ndb->ncdb, hostname, &ip_addr))
    return 0;
  
  if (ndb->ncdb->aipinfo) {
    ai = ndb->ncdb->aipinfo;
    for (i = 0; i < ndb->ncdb->numaips ; i++, ai++) {
      if (memcmp (&ip_addr, &(ai->saddr.sin_addr), sizeof(struct in_addr)) == 0) {
	*aip = i;
	found = 1;
	break;
      }
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKEDATABAD, "No AIP Information NCDB");
    return 0;
  }
  if (!found) {
    zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	       ZKENOTFOUND, "No aip found matching hostname[%s]", hostname);
    *aip = -1;
  }
  return 1;
}

DLLExport
int frNCDBGetNameForAipID (ZkError err, FrNDB ndb, int aip, const char **name)
{
  static const char *funcname = "frNCDBGetNameForAipID";

  if (ndb && ndb->ncdb && name) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      *name = ndb->ncdb->aipinfo[aip].name;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetIPStrForAipID (ZkError err, FrNDB ndb, int aip, const char **ipstr)
{
  static const char *funcname = "frNCDBGetIPStrForAipID";

  if (ndb && ndb->ncdb && ipstr) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      *ipstr = ndb->ncdb->aipinfo[aip].ip;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetIPAddrForAipID (ZkError err, FrNDB ndb, int aip, struct in_addr *ipaddr)
{
  static const char *funcname = "frNCDBGetIPAddrForAipID";

  if (ndb && ndb->ncdb && ipaddr) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      if (!inet_aton(ndb->ncdb->aipinfo[aip].ip, ipaddr)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKEDATABAD, "couldn't covert address");
	return 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetTCPPortForAipID (ZkError err, FrNDB ndb, int aip, uint16_t *tcpport)
{
  static const char *funcname = "frNCDBGetTCPPortForAipID";

  if (ndb && ndb->ncdb && tcpport) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      *tcpport = ndb->ncdb->aipinfo[aip].tcpport;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetUDPPortForAipID (ZkError err, FrNDB ndb, int aip, uint16_t *udpport)
{
  static const char *funcname = "frNCDBGetUDPPortForAipID";

  if (ndb && ndb->ncdb && udpport) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      *udpport = ndb->ncdb->aipinfo[aip].udpport;
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetUDPsockaddr_inForAipID (ZkError err, FrNDB ndb, int aip,
				     struct sockaddr_in *saddr)
{
  static const char *funcname = "frNCDBGetUDPsockaddr_inForAipID";

  if (ndb && ndb->ncdb && saddr) {
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      saddr->sin_family = AF_INET;
      saddr->sin_port = htons (ndb->ncdb->aipinfo[aip].udpport);
      if (!inet_aton(ndb->ncdb->aipinfo[aip].ip, &saddr->sin_addr)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKEDATABAD, "couldn't covert address");
	return 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

#ifndef NDEBUG
char *saddrSprintf (struct sockaddr_in *host)
{
  char *p = (char *)NEW(64);

  if (host) { 
    snprintf (p, 64, "%s:%d", inet_ntoa(host->sin_addr), (int) htons (host->sin_port));
  } else {
    snprintf (p, 64, "(NULL)");
  }
  return (p);
}


int frNCDBDumpClientAipList (FrNDB ndb, FrNDBAipList ai)
{
  FrNDBAipList aiptr;
  int i;
  char *p;

  zkDebug_va ("[Network AIP List]");

  if (ai) {
      aiptr = ai;
      zkDebug_va ("Dumping %d aips", ndb->ncdb->numaips);
      for (i = ndb->ncdb->numaips; i > 0 ; i--, aiptr++) {
	p = saddrSprintf(&aiptr->saddr);
	/* Bug: missing frent routines */
	zkDebug_va ("AIP [%2d] %-13s %20s {%2d} %4s %4s %6s %s",
		    aiptr->id,
		    aiptr->entity, p, aiptr->rating,
		    aiptr->isexit?"exit":"----", 
		    aiptr->isup?" up ":"----",
		    aiptr->isusable?"usable":"------",
		    aiptr->region);
	free(p);
      }
  } else {
    zkDebug_va ("***No aipinfo");
  }
  return 1;
}
#endif  

DLLExport
int frNCDBGetAllAIPInfo (ZkError err, FrNDB ndb, char *version, FrNDBAipList *aiplist, int *numaips)
{
  static const char *funcname = "frNCDBGetAllAIPInfo";
  
  FrAIPinfo ai;
  FrNDBAipList cai;
  int i;
  /* ZZZZ */  
  if (ndb && aiplist && numaips) {
    *numaips = 0;
    if (version) {
      strncpy(version, ndb->ncdb->revision, ZK_NDBNAME_SIZE);
    }
    if (!(*aiplist = (FrNDBAipList)
	  calloc (ndb->ncdb->numaips+1,sizeof(struct T_FrNDBAipList)))) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for AIPList");
      return 0;
    }
    
    if (ndb->ncdb->aipinfo) {
      cai = *aiplist;
      ai = ndb->ncdb->aipinfo;
      *numaips = ndb->ncdb->numaips;
      for (i = ndb->ncdb->numaips; i > 0 ; i--, ai++, cai++) {
	cai->id = ai->id;
	cai->isexit = ai->isexit;
	cai->isup = ai->isup;
	cai->rating = ai->rating;
	if (!frNCDBGetUDPsockaddr_inForAipID (err, ndb, ai->id, &ai->saddr)) {
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKENOMEM, "Couldn't convert aip saddr info");
	  return 0;
	}
	cai->isusable = ai->isusable;
	
	memcpy(&cai->saddr, &ai->saddr, sizeof(ai->saddr));
	strncpy(cai->entity, ai->entity, ZK_NDBNAME_SIZE);
	strncpy(cai->region, ai->region, ZK_NDBNAME_SIZE);
	/*
	 * Bug: HACK
	 * Assuming that the maip and aip names are the same
	 * Assuming that an maip is always present
	 */
	cai->aip_entity.type=FRENT_TYPE_AIP;
	cai->maip_entity.type=FRENT_TYPE_MAIP;
	strncpy(cai->aip_entity.name, ai->entity, ZK_NDBNAME_SIZE);
	strncpy(cai->maip_entity.name, ai->entity, ZK_NDBNAME_SIZE);
      }
    } else {
      zkDebug_va ("***No aipinfo");
    }
  }
  return 1;
}


DLLExport
int frNCDBFreeAIPInfo (ZkError err, FrNDBAipList *aiplist)
{
  static const char *funcname = "frNCDBFreeAIPInfo";

  if (aiplist && *aiplist) {
    free (*aiplist);
    aiplist = NULL;
    return 1;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENOMEM, "Couldn't allocate memory for AIPList");
      return 0;
  }
}

/**
 * frNCDBUpdate gets called periodically from the client.
 * It is IMPERATIVE to do a 
 *
 *	if (ZkErrorCode(err) == ZKEDEFUNCT) {
 *		Advise the user that his client has expired for good
 *              No routes will be available.
 *	}
 */
DLLExport int frNCDBUpdate (ZkError err, FrNCDB *ncdb,
			    const char *newfile, time_t *timeout)
{
  static const char *funcname = "frNCDBUpdate";
  FrNCDB ncdbtmp=NULL, tmp=NULL;
  FrNDB  ndbtmp=NULL;
  (void) funcname;
  
  if (! timeout) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  

  if (ncdb)
    ncdbtmp = *ncdb;
  
  if ( ! ncdb && ! frNDBGetPtrToNDB (err, &ndbtmp) && !ndbtmp->ncdb) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NCDB");
    return 0;
  }

  if (ndbtmp)
    ncdbtmp = ndbtmp->ncdb;


  if (!ncdbtmp) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NCDB");
    return 0;
  }

  if (!frNCDBNew(err, &(ncdbtmp->update)))
    return 0;

  /* bug: temporary */
  zkCStrlcpy(ncdbtmp->update->filename, newfile, sizeof ncdbtmp->update->filename);
  if (!frNCDBLoad (err, ncdbtmp->update, timeout)) {
    return 0;
  }
  if (zkErrorCode(err) == ZKEDEFUNCT) {
    /* Need to advise the customer of this */
    zkDebug_va ("NCDBLoad advises us that the client is defunct");
  }
  /* Switch over */
  zkDebug_va ("Switching over to the new network routers");
  /* Bug: probably need some sort of exclusive lock at some point */
  tmp = ncdbtmp;
  ncdbtmp = (ncdbtmp)->update;
  frNCDBFree(NULL, &tmp);     /* Need to propagate the ZKEDEFUNCT */

  /* Now clean up pointers ... */
  if (ncdb) {
    *ncdb = ncdbtmp;
  } else {
    ndbtmp->ncdb = ncdbtmp;
  }
  
  return 1;
}

/**
 * Returns true or false
 */
int frNCDBRoutesDenied (ZkError err, FrNCDB ncdb)
{
  static const char *funcname = "frNCDBRoutesDenied";

  if (ncdb->disableRoutes) {
    zkErrorSet_va(err, funcname, ZKWARN, ZKMOD, ZKEDEFUNCT,
	  "Client is too old to use the Freedom network. Please upgrade");
    return 1;
  }
  return 0;
}
