/*
 * $Id: frNCDBqry.c,v 1.12 2001/04/19 20:28:37 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 * The Network Configuration Data Module
 * Luc Boulianne
 *
 */
#include "zks.h"

#ifndef WIN32
#include <unistd.h>
#include <netinet/in.h>
#define __EXTENSIONS__
#include <string.h>
#undef __EXTENSIONS__
#endif
#include <frNDB.h>
#include <frNCDB.h>
#include <frNCDBqry.h>
#include <zkDebug.h>
#include <zkError.h>
#include <zkDateTime.h>
#include <zkList.h>
#include <zkTypes.h>


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMDB


static int hostPortEntryFree (ZkError err, ZkList list, void *entry, void *none) 
{ 
  (void)err; /* remove compiler warnings */ 
  (void)list; 
  (void)none; 

  free(entry); 
  return 1; 
}

DLLExport
int frNCDBFreeHostPortList (ZkError err, ZkList *hplist)
{
  static const char *funcname = "frNCDBFreeHostPortList";

  if (hplist && *hplist) {
    if (!zkListFree (err, hplist))
	  return 0;
    hplist = NULL;
    return 1;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENOMEM, "Couldn't free memory for HostPortList");
      return 0;
  }
}

static int getHostPortList (ZkError err, ZkList *hplist)
{
  static const char *funcname = "getHostPortList";

  if (hplist) {
    if (!*hplist) {
      /* Allocate the list if I'm passed a NULL */
      if (!zkListNew(err, hplist))
	return 0;
      if (!zkListSetDataFreeFunction (err, *hplist, hostPortEntryFree, NULL)) { 
	return 0; 
      }
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Null Pointer passed");
    return 0;
  }
  return 1;
}

/* BUG: change all ndb to ncdb */
   
DLLExport
int frNCDBGetNymServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetNymServers";

  FrNDBHostPort hp;

  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.nymhost;
  hp->port = ndb->ncdb->servers.nymport;
  hp->timeout = ndb->ncdb->servers.nymtimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetTokenServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetTokenServers";
  
  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.tokenhost;
  hp->port = ndb->ncdb->servers.tokenport;
  hp->timeout = ndb->ncdb->servers.tokentimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetNewsGroupServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetNewsGroupServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.newsgrphost;
  hp->port = ndb->ncdb->servers.newsgrpport;
  hp->timeout = ndb->ncdb->servers.newsgrptimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetDNSServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetDNSServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.dnshost;
  hp->port = ndb->ncdb->servers.dnsport;
  hp->timeout = ndb->ncdb->servers.dnstimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetNIDBServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetNIDBServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.nidbhost;
  hp->port = ndb->ncdb->servers.nidbport;
  hp->timeout = ndb->ncdb->servers.nidbtimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetKeyQueryServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetKeyQueryServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.keyqryhost;
  hp->port = ndb->ncdb->servers.keyqryport;
  hp->timeout = ndb->ncdb->servers.keyqrytimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetKeyUpdateServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetKeyUpdateServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.keyupdhost;
  hp->port = ndb->ncdb->servers.keyupdport;
  hp->timeout = ndb->ncdb->servers.keyupdtimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }
  return 1;
}

DLLExport
int frNCDBGetFMGServers (ZkError err, FrNDB ndb, ZkList *hostportlist)
{
  static const char *funcname = "frNCDBGetFMGServers";

  FrNDBHostPort hp;
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  /* Checks for null in here */
  if (!getHostPortList (err, hostportlist))
      return 0;

  zkMALLOC(hp, FrNDBHostPort, sizeof(struct T_FrNDBHostPort));
  
  hp->host = ndb->ncdb->servers.fmgmaiphost;
  hp->port = ndb->ncdb->servers.fmgmaipport;
  hp->timeout = ndb->ncdb->servers.fmgmaiptimeout;
    
  if (!zkListInsert(err, *hostportlist, hp, LIST_TAIL)) {
    return 0;
  }

  return 1;
}

DLLExport
int frNCDBGetMAIPPortAndTimeout (ZkError err, FrNDB ndb,
				 uint16_t *port,
				 uint16_t *timeout)
{
  static const char *funcname = "frNCDBGetMAIPPort";

  if ( ! port) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  
  *port = ndb->ncdb->servers.maipport;
  /* Bug: fix this warning */
  *timeout = (uint16_t) ndb->ncdb->servers.maiptimeout;

  return 1;
}

DLLExport
int frNCDBGetClientUdpTcpPort (ZkError err, FrNDB ndb, uint16_t *port)
{
  static const char *funcname = "frNCDBGetClientUdpTcpPort";
  
  if ( ! port) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  
  *port = ndb->ncdb->servers.clientudptcpport;

  return 1;
}

DLLExport
int frNCDBGetRevision (ZkError err, FrNDB ndb, const char **revision)
{
  static const char *funcname = "frNCDBGetRevision";
  
  if (! revision) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *revision = ndb->ncdb->revision;
  return 1;
}

DLLExport
int frNCDBGetTTL (ZkError err, FrNDB ndb, uint32_t *ttl)
{
  static const char *funcname = "frNCDBGetTTL";

  if (! ttl) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *ttl = ndb->ncdb->ttl;
  return 1;
}

DLLExport
int frNCDBGetNIDBConfigURL (ZkError err, FrNDB ndb, const char **url)
{
  static const char *funcname = "frNCDBGetNIDBConfigURL";

  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  
  *url = ndb->ncdb->url;

  return 1;
}

/* Bug: I'm getting UTC time here */
DLLExport
int frNCDBGetLocalTimeExpired (ZkError err, FrNDB ndb, time_t *expiration)
{
  static const char *funcname = "frNCDBGetLocalTimeExpired";
  ZkDateTime dt;

  if (! expiration) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!zkDateTimeParse (err, &dt, ndb->ncdb->expiration)) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEPARSE, 
	       "Can't parse the internal time!!! [%s]", ndb->ncdb->expiration);
    return 0;
  }

  return (zkDateTimeMkTime (err, &dt, ZKTIME_FL_LOCAL, expiration));

}

/* Bug: I'm getting UTC time here */
DLLExport
int frNCDBGetLocalTimeUpdated (ZkError err, FrNDB ndb, time_t *updated)
{
  static const char *funcname = "frNCDBGetLocalTimeUpdated";
  ZkDateTime dt;

  if (! updated) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!zkDateTimeParse (err, &dt, ndb->ncdb->updated)) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEPARSE,
	       "Can't parse the internal time!!! [%s]", ndb->ncdb->updated);
    return 0;
  }

  return (zkDateTimeMkTime (err, &dt, ZKTIME_FL_LOCAL, updated));

}

DLLExport
int frNCDBGetWinClientCurrRevision (ZkError err, FrNDB ndb, const char **revision)
{
  static const char *funcname = "frNCDBGetWinClientCurrRevision";

  if (!revision) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *revision = ndb->ncdb->winclient.revision;
  return 1;
}

DLLExport
int frNCDBGetWinClientCurrInfoURL (ZkError err, FrNDB ndb, const char **infoURL)
{
  static const char *funcname = "frNCDBGetWinClientCurrInfoURL";

  if  (!infoURL) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *infoURL = ndb->ncdb->winclient.infoURL;
  return 1;
}

DLLExport
int frNCDBGetWinClientCurrUpgradeURL (ZkError err, FrNDB ndb, const char **upgradeURL)
{
  static const char *funcname = "frNCDBGetWinClientCurrUpgradeURL";

  if (!upgradeURL) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *upgradeURL = ndb->ncdb->winclient.upgradeURL;
  return 1;
}

DLLExport
int frNCDBGetWinClientCurrDownloadURL (ZkError err, FrNDB ndb, const char **downloadURL)
{
  static const char *funcname = "frNCDBGetWinClientCurrDownloadURL";

  if (!downloadURL) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *downloadURL = ndb->ncdb->winclient.downloadURL;
  return 1;
}

DLLExport
int frNCDBGetTrialSerialNumberURL (ZkError err, FrNDB ndb,
					    const char **URL)
{
  static const char *funcname = "frNCDBGetTrialSerialNumberURL";

  if (! URL) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *URL = ndb->ncdb->servers.trialserialURL;
  return 1;
}

DLLExport
int frNCDBGetWinClientCurrUrgency (ZkError err, FrNDB ndb, uint32_t *urgency)
{
  static const char *funcname = "frNCDBGetClientTCPPort";

  if (! urgency) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  *urgency = ndb->ncdb->winclient.urgency;
  return 1;
}

/* Bug: I'm getting UTC time here */
DLLExport
int frNCDBGetWinClientLocalTimeUpdated (ZkError err, FrNDB ndb, time_t *updated)
{
  static const char *funcname = "frNCDBGetWinClientUpdated";
  ZkDateTime dt;
  
  if (! updated) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }
  
  if (!zkDateTimeParse (err, &dt, ndb->ncdb->winclient.updated)) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEPARSE,
		  "Can't parse the internal time!!! [%s]",
		  ndb->ncdb->winclient.updated);
    return 0;
  }

  return (zkDateTimeMkTime (err, &dt, ZKTIME_FL_LOCAL, updated));
}

/**
 * Get the TCP/UDP port for a given AIP hostname
 *
 * ports will be 0 if not found. Check ZkErrorCode
 * for exact error.
 *
 * NDB can be NULL
 */
DLLExport
int frNCDBGetAIPsNameUdpTcpPort (ZkError err, FrNDB ndb,
				 const char * hostname,
			         uint16_t * udpport,
				 uint16_t * tcpport )
{
  static const char *funcname = "frNCDBGetAIPsNameUdpTcpPort";
  FrAIPinfo ai;
  
  if (! udpport || ! tcpport) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  

  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  /* A few sanity checks */
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!(ndb->ncdb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "No NCDB available");
    return 0;
  }

  if (!ndb->ncdb->aipinfo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No Aipinfo available");
    return 0;
  }

  for (ai = ndb->ncdb->aipinfo; ai && ai->id >= 0; ai++) {
    if (strcasecmp (ai->name, hostname) == 0) {
      *udpport = ai->udpport;
      *tcpport = ai->tcpport;
      return 1;
    }
  }
  zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	     ZKENOTFOUND, "Aip [%s] not found", hostname);
  *udpport = 0;
  *tcpport = 0;
  return 1;
}

  
/**
 * Get the TCP/UDP port for a given AIP hostname
 *
 * ports will be 0 if not found. Check ZkErrorCode
 * for exact error.
 *
 * NDB can be NULL
 */
DLLExport
int frNCDBGetAIPsIPAddrUdpTcpPort (ZkError err, FrNDB ndb,
				   const char * ipaddr,
				   uint16_t * udpport, 
				   uint16_t * tcpport)
{
  static const char *funcname = "frNCDBGetAIPsIPAddrUdpTcpPort";
  FrAIPinfo ai;
  
  if (! udpport || ! tcpport) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  

  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  /* A few sanity checks */
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!(ndb->ncdb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "No NCDB available");
    return 0;
  }

  if (!ndb->ncdb->aipinfo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No Aipinfo available");
    return 0;
  }

  for (ai = ndb->ncdb->aipinfo; ai && ai->id >= 0; ai++) {
    if (strcasecmp (ai->ip, ipaddr) == 0) {
      *udpport = ai->udpport;
      *tcpport = ai->tcpport;
      return 1;
    }
  }
  zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	     ZKENOTFOUND, "Aip IPADDR [%s] not found", ipaddr);
  *udpport = 0;
  *tcpport = 0;
  return 1;
}


/**
 * Get the TCP/UDP port for a given AIP hostname
 *
 * ports will be 0 if not found. Check ZkErrorCode
 * for exact error.
 *
 * NDB can be NULL
 */
DLLExport
int frNCDBGetAIPsInAddrUdpTcpPort (ZkError err, FrNDB ndb,
				   struct in_addr ip_addr,
				   uint16_t * udpport,
				   uint16_t * tcpport)
{
  static const char *funcname = "frNCDBGetAIPsInAddrUdpTcpPort";
  FrAIPinfo ai;
  
  if (! udpport || ! tcpport) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  

  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  
  /* A few sanity checks */
  if ( ! ndb && ! frNDBGetPtrToNDB (err, &ndb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Unable to get pointer to NDB");
    return 0;
  }

  if (!(ndb->ncdb)) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "No NCDB available");
    return 0;
  }

  if (!ndb->ncdb->aipinfo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No Aipinfo available");
    return 0;
  }

  for (ai = ndb->ncdb->aipinfo; ai && ai->id >= 0; ai++) {
    if (ai->saddr.sin_addr.s_addr == ip_addr.s_addr) {
      *udpport = ai->udpport;
      *tcpport = ai->tcpport;
      return 1;
    }
  }
  zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	     ZKENOTFOUND, "Aip s_addr [0x%x] not found",
		ip_addr.s_addr);
  *udpport = 0;
  *tcpport = 0;
  return 1;
}

/**
 * Given an AIP, get a list of all it's neighbours
 *
 * NCDB can be NULL
 * You must free the aipidlist when you're done
 */

DLLExport
int frNCDBGetAllAipNeighboursForAIPId (ZkError err, FrNCDB ncdb, int aipid,
				    int **aipidlist, int *numaips)
{
  static const char *funcname = "frNCDBGetAllAipNeighboursForAIPId";
  FrNDB ndb;
  int *neigh, *nlist;
  int i, j, cnt = 0;
  FrLinks links;
  
  /* A few sanity checks */
  if ( ! ncdb ) {
    if (! frNDBGetPtrToNDB (err, &ndb)) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENULLPTR, "Unable to get pointer to NDB");
      return 0;
    } else {
      ncdb = ndb->ncdb;
    }
  }
  
  if (! ncdb || ! aipidlist || ! numaips) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Null parameter");
    return 0;
  }

   if ((aipid < 0 ) || aipid >= ncdb->numaips) {
    zkErrorSet_va(err, funcname, ZKERR, ZKMOD,
	       ZKEPARMRANGE, "aipid [%d] is out of range ", aipid);
    return 0;
  }

  neigh = (int *) calloc (ncdb->numaips+1, sizeof(int));
  memset (neigh, 0, (ncdb->numaips+1)*sizeof(int)); /* Paranoia */

  for (i = 0, links = ncdb->links;
       i < ncdb->numlinks;
       i++, links++) {
    if (links->aip[0] == aipid) {
      if (++neigh[links->aip[1]] == 1) {
	/* zkDebug_va(" ?? %d ", links->aip[1]); */
	/* count thefirst instance */
	cnt++;
      }
    }      
    if (links->aip[1] == aipid) {
      /* zkDebug_va(" ?? %d ", links->aip[0]); */
      if (++neigh[links->aip[0]] == 1) {
	/* count thefirst instance */
	cnt++;
      }
    }      
  }
  /* zkDebug_va("We have %d neighbours", cnt); */

  if (cnt) {
    nlist = (int *) calloc (cnt+1, sizeof(int));
    for (i = 0, j = 0 ; i < ncdb->numaips+1 ; i++) {
      if (neigh[i] > 0) {
	/* zkDebug_va(" --> %d", i); */
	nlist[j++] = i;
      }
    }
    *aipidlist = nlist;
  } else {
    *aipidlist = NULL;
  }
  *numaips = cnt;
  free(neigh);
  return 1;
}


/**
 * Get the IP address for any hostname we've encountered
 * in our configuration
 * NDB can be NULL
 */
DLLExport
int frNCDBGetIPfromHostname (ZkError err, FrNCDB ncdb,
			     const char *hostname, struct in_addr *ip_addr)
{
  static const char *funcname = "frNCDBGetIPfromHostname";
  FrNDB ndb; 
  FrAIPinfo ai;
  
  if (! hostname || ! ip_addr) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "Invalid parameter passed");
    return 0;
  }  

  /* A few sanity checks */
  if ( ! ncdb ) {
    if (! frNDBGetPtrToNDB (err, &ndb)) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENULLPTR, "Unable to get pointer to NDB");
      return 0;
    } else {
      ncdb = ndb->ncdb;
    }
  }

  if (!ncdb) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "No NCDB available");
    return 0;
  }

  if (!ncdb->aipinfo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No Aipinfo available");
    return 0;
  }

  for (ai = ncdb->aipinfo; ai && ai->id >= 0; ai++) {
    if (strcasecmp(hostname, ai->name) == 0) {
      /* we've found this */
      ip_addr->s_addr = ai->saddr.sin_addr.s_addr;
      return 1;
    }
  }

  /* Now the messy one */
  /* Don't give me hell for this one. Mario wanted it in 45 minutes */
  /* Bug: fix this code */
  if (strcasecmp (ncdb->servers.nymhost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.nymip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.tokenhost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.tokenip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.newsgrphost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.newsgrpip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.dnshost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.dnsip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.nidbhost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.nidbip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.keyupdhost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.keyupdip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.keyqryhost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.keyqryip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
  if (strcasecmp (ncdb->servers.fmgmaiphost, hostname) == 0) {
    if (!inet_aton(ncdb->servers.fmgmaipip, ip_addr)) {
      zkErrorSet_va(err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		    "couldn't covert ip address [%s]", ai->ip);
      return 0;
    }
    return 1;
  }
    
  ip_addr->s_addr = INADDR_NONE;
  zkErrorSet_va(err, funcname, ZKWARN, ZKMOD,
	     ZKENOTFOUND, "Hostname [%s] not found",
		hostname);
  return 1;
}
