
/*
 * $Id: frNDB.c,v 1.15 2001/04/19 20:28:37 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <stdlib.h>
#include <stdio.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <zkError.h>
#include <zkList.h>
#include <zkDebug.h>
#include <frNCDB.h>
#include <frNDB.h>
#include <frNNP.h>


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNDB

extern int frNDBInit (ZkError, FrNDB *);

/* Since only one of these can exist, let's create it now */

static struct T_FrNDB NDB;

int frNDBInit (ZkError err, FrNDB *ndb)
{
  /* static const char *funcname = "FrNDBInit"; */

  FrNDB myndb = &NDB;
  
  if (!frNCDBInit(err, &myndb->ncdb, NULL, NULL)) {
    return 0;
  }
  /*
    (*myndb)->nsdb = NULL;
    (*nyndb)->nkdb = NULL;
    (*myndb)->nrdb = NULL;
  */
  if (!zkListNew(err, &(myndb->nnplist))) {
    return 0;
  }
  if (ndb) {
    *ndb = &NDB;
  }
  myndb->initialized = 1;
  myndb->refcount = 0;
  return 1;
}

DLLExport int frNDBEnv (ZkError err, const char *configfile, time_t *timeout)
{

  static const char *funcname = "FrNDBEnv";
  FrNDB ndb;
  if (NDB.initialized) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "NDB already initialized. Can't change env.");
      return 1;
  }
  if (!frNDBInit(err, &ndb))
      return 0;
  if (!frNCDBInit(err, &ndb->ncdb, configfile, timeout)) {
    return 0;
  }
  return 1;
}

DLLExport int frNDBGetPtrToNDB (ZkError err,  FrNDB *ndb)
{
  static const char *funcname = "frNDBGetNDBPtr";

  if (!ndb) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "Null pointer provided");
      return 0;
  }
  if (!NDB.initialized) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "NDB is not initialized. Pointer unavailable.");
      return 0;
  }
  *ndb = &NDB;
  return 1;
}

int frNDBRegisterNNP (ZkError err,  FrNDB ndb, FrNNP nnp)
{
  static const char *funcname = "frNDBRegisterNNP";

  if (!ndb || !nnp) {
    zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "Null pointer provided");
      return 0;
  }
  if (!NDB.initialized) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "NDB is not initialized. Pointer unavailable.");
      return 0;
  }
  zkListInsert(err, ndb->nnplist, nnp, LIST_HEAD);
  return 1;
}

int frNDBUnRegisterNNP (ZkError err,  FrNDB ndb, FrNNP nnp)
{
  FrNNP nnp2;
  int found = 0;
  static const char *funcname = "frNDBUnRegisterNNP";

  if (!ndb || !nnp) {
    zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "Null pointer provided");
      return 0;
  }
  if (!NDB.initialized) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "NDB is not initialized. Pointer unavailable.");
      return 0;

  }

  if (!zkListMoveCursor (err, ndb->nnplist, LIST_HEAD))
    return 0;
  
  do {
    if (zkListGet(err, ndb->nnplist, (void **)&nnp2, LIST_STAY)) {
      if (nnp2 == nnp) {
	found = 1;
	if (!zkListDelete(err, ndb->nnplist, LIST_STAY))
	  return 0;
      }
    } else {
      found = -1;
    }
  } while (!found && zkListMoveCursor (err, ndb->nnplist, LIST_FORWARDS));
  
  return 1;
}

DLLExport int frNDBOpen (ZkError err, FrNDB *ndb, const char *configfile,
			 time_t *timeout)
{
  static const char *funcname = "FrNDBOpen";
	
  /* Bug: Probably have a possible race condition here */
  if (!NDB.initialized) {
    if (!ndb) {
      zkErrorSet(err, funcname, ZKERR, ZKMNDB,
		 ZKENULLPTR, "NULL Pointer provided.");
      return 0;
    }
    if (!frNDBInit(err, ndb))
      return 0;
    if ((configfile == NULL) || timeout == NULL) {
      zkErrorSet(err, funcname, ZKWARN, ZKMNDB,
		 ZKENULLPTR, "Call sequence error. Call frNDBOpen before calling this");
      return 0;
    }
    if (!frNCDBInit(err, &((*ndb)->ncdb), configfile, timeout)) 
      return 0;
    
    /*
      (*ndb)->nsdb = NULL;
      (*ndb)->nkdb = NULL;
      (*ndb)->nrdb = NULL;
    */
  } else {
    *ndb = &NDB;
  }
  (*ndb)->refcount ++;
  if (zkErrorCode(err) == ZKEDEFUNCT) {
    zkDebug_va ("NCDBInit advises us that the client is defunct");
  }
  return 1;
}

DLLExport int frNDBClose (ZkError err, FrNDB *ndb)
{
  static const char *funcname = "frNDBClose";

  if (ndb) {
    if ((*ndb)->refcount == 1) {
      if (!frNCDBClose(err, &((*ndb)->ncdb)))
	zkErrorSet(err, funcname, ZKWARN, ZKMOD,
		   ZKERROR,
		   "Problem closing NCDB");
      if (!(*ndb)->nnplist) {
	 zkListFree(err, &((*ndb)->nnplist));
      }
      *ndb= NULL;
    } else {
      (*ndb)->refcount --;
      zkErrorSet_va(err, funcname, ZKWARN, ZKMOD, ZKEBUSY,
		   "NDB not closed. %d open reference%s remaining",
		    (*ndb)->refcount,
		    (*ndb)->refcount == 1 ? "" : "s");
    }
  }
  return 1;    
}


#ifndef NDEBUG
int frNDBDump (FrNDB network)
{
  
  zkDebug_va ("[N E T W O R K  D A T A B A S E]");

  if (network) {
    if (network->ncdb) {
      if (network->initialized) {
	zkDebug_va ("NDB is initialized");
	zkDebug_va ("%d open reference%s to it",
		    network->refcount,
		    network->refcount == 1 ? "" : "s");
	frNCDBDump(network->ncdb);
      }
    } else {
      zkDebug_va ("No Network configuration available");
    }
  }

  return 1;
}
#endif
