/*
 * $Id: frNRDB.c,v 1.11 2001/04/19 20:28:37 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <sys/socket.h>
#include <zkError.h>
#include <zkDebug.h> 
#include <frNDB.h>
#include <frNCDB.h>
#include <frNRDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMDB


static const char vcrev[] = "$Revision: 1.11 $"; /* Maintained by CVS */
DLLExport void NetworkRDB_Version(char* buf, size_t bufsz, int* maj, int* min){
    sscanf (vcrev,"%*s %d.%d", maj, min); /* audit: ok */
    snprintf(buf,bufsz,"Build [%s], Version [%d,%d]", __TIMESTAMP__, *maj, *min);
}

/*
 * Override
 */
DLLExport
int frNRDBSetOverrideRoute (ZkError err, FrNDB n, int override)
{
  static const char *funcname = "frNRDBSetOverrideRoute";
  if (n) {
    return (frNCDBSetOverrideRoute (err, n->ncdb, override));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;

}

DLLExport
int frNRDBResetOverrideRoute (ZkError err, FrNDB n)
{
  static const char *funcname = "frNRDBSetOverrideRoute";
  if (n) {
    return (frNCDBResetOverrideRoute (err, n->ncdb));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;

}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBGetOverrideRoute (ZkError err, FrNDB n, int *override)
{
  static const char *funcname = "frNRDBGetOverrideRoute";
  if (n) {
    return (frNCDBGetOverrideRoute (err, n->ncdb, override));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

#if 0
/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBGetLastSelectedRouteUDPSockaddr_in (ZkError err, FrNDB ndb,
					   struct sockaddr_in *saddr)
{
  static const char *funcname = "frNRDBGetLastSelectedRouteSockaddr_in";
  int aip;

  if (ndb && ndb->ncdb && saddr) {
    aip = ndb->ncdb->lastSelectedRoute;
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      saddr->sin_family = AF_INET;
      saddr->sin_port = htons (ndb->ncdb->aipinfo[aip].udpport);
      if (!inet_aton(ndb->ncdb->aipinfo[aip].ip, &saddr->sin_addr)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKEDATABAD, "couldn't covert address");
	return 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBGetLastSelectedRouteTCPSockaddr_in (ZkError err, FrNDB ndb,
					   struct sockaddr_in *saddr)
{
  static const char *funcname = "frNRDBGetLastSelectedRouteSockaddr_in";
  int aip;

  if (ndb && ndb->ncdb && saddr) {
    aip = ndb->ncdb->lastSelectedRoute;
    if (aip >= 0 && aip <= ndb->ncdb->numaips) {
      saddr->sin_family = AF_INET;
      saddr->sin_port = htons (ndb->ncdb->aipinfo[aip].tcpport);
      if (!inet_aton(ndb->ncdb->aipinfo[aip].ip, &saddr->sin_addr)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKEDATABAD, "couldn't covert address");
	return 0;
      }
    } else {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "AIP is out of range");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}
#endif /* 0 */

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBIsOverrideRouteSet (ZkError err, FrNDB n) 
{
  static const char *funcname = "frNRDBIsOverrideRouteSet";
  
  if (n) {
    return (frNCDBIsOverrideRouteSet (err, n->ncdb));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/*
 * Sequenced
 */
DLLExport
int frNRDBResetSequencedRoute (ZkError err, FrNDB n)
{
  static const char *funcname = "frNRDBResetSequencedRoute";
  if (n) {
    return (frNCDBResetSequencedRoute (err, n->ncdb));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;

}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBSetSequencedRoute (ZkError err, FrNDB n)
{
  static const char *funcname = "frNRDBSetSequencedRoute";
  
  if (n) {
    return (frNCDBSetSequencedRoute (err, n->ncdb));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBIsGetRoutesInSequenceSet(ZkError err, FrNDB n) 
{
  static const char *funcname = "frNRDBIsGetRoutesInSequenceSet";
  
  if (n) {
    return (frNCDBIsGetRoutesInSequenceSet (err, n->ncdb));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/* Make sure the returned override is greater than 0 */
DLLExport
int frNRDBGetNextRouteInSequence (ZkError err, FrNDB n, int *override)
{
  static const char *funcname = "frNRDBGetNextRouteInSequence";
  if (n) {
    return (frNCDBGetNextRouteInSequence (err, n->ncdb, override));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

DLLExport
int frNRDBGetRandomRoute (ZkError err, FrNDB n, int *route)
{
  static const char *funcname = "frNRDBGetRandomRoute";
  if (n) {
    return (frNCDBGetRandomRoute (err, n->ncdb, route));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null parameter");
    return 0;
  }
  return 1;
}

/**
 * This is the central routine to get a route.
 */

DLLExport
int frNRDBGetAnonRouteToExit (ZkError err, FrNDB ndb, int *anonRoute)
{
  static const char *funcname = "frNRDBGetAnonRouteToExit";
  
  zkErrorClear (err);
  if (ndb && ndb->ncdb) {
    /* Check to see if routes have been denied */
    if (frNCDBRoutesDenied(err, ndb->ncdb)) {
      zkDebug_va ("WARNING: NCDB has probably expired");
      *anonRoute = -1;
      return 0;
    }
    /* Return override route if set */
    if (frNRDBIsOverrideRouteSet (err, ndb)) {
      if (!frNRDBGetOverrideRoute (err, ndb, anonRoute)) 
	return 0;
    } else if (frNRDBIsGetRoutesInSequenceSet (err, ndb)) {
      if (!frNRDBGetNextRouteInSequence (err, ndb, anonRoute))
	return 0;
    } else if (!frNRDBGetRandomRoute (err, ndb, anonRoute)) {
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter");
    return 0;
  }
  ndb->ncdb->lastSelectedRoute = *anonRoute;
  return 1;
}
    
DLLExport
int frNRDBGetNumLinksForAnonRouteId (ZkError err, FrNDB ndb, int route, int *links)
{
  static const char *funcname = "frNRDBGetNumLinksForAnonRouteId";

  if (ndb) {
    return (frNCDBGetNumLinksForRouteId (err, ndb, route, links));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter for ndb");
    return 0;
  }
}

DLLExport
int frNRDBGetAipXinRouteId (ZkError err, FrNDB ndb, int route, int hop, int *aip)
{
  static const char *funcname = "frNRDBGetAipXinRouteId";
    
  zkErrorClear(err);

  if (ndb) {
    return (frNCDBGetAipXinRouteId (err, ndb, route, hop, aip));
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Null parameter for ndb");
    return 0;
  }
}

#if 0
DLLExport int frAnonRouteSetNumLinks (ZkError err, FrAnonRoute route, int links, int *newlinks)
{

  static const char *funcname = "FrAnonRouteSetNumLinks";
  int retcode = 1;
    
  zkErrorClear(err);

  if (links > FRMAXLINKS) { /* bug: maxlinks */
    zkErrorSet(err, funcname, ZKERR, ZKMANONROUTE,
	       ZKENULLPTR, "Link number requested is greater than MAXLINKS");
    retcode = 0;
  } else {
    if (route) {
      route->links = *newlinks = DEFNUMLINKS;   /* bug: hardcoded */
    } else {
      zkErrorSet(err, funcname , ZKERR, ZKMANONROUTE,
		 ZKENULLPTR, "NULL Pointer passed for route");
      retcode = 0;
    }
  }
  return retcode;
}
#endif

#if 0
     
DLLExport extern int frAnonRouteSetExitAIP (ZkError err, FrAnonRoute route, FrAnonRouteAIP *);
DLLExport extern int frAnonRouteSetStartAIP (ZkError err, FrAnonRoute route, FrAnonRouteAIP *);
DLLExport extern int frAnonRouteSetLinkX(ZkError, FrAnonRoute, int link, struct sockaddr_in *);
DLLExport extern int frAnonRouteGetAvailRoute(ZkError, FrAnonRoute, int *link);
DLLExport extern int frAnonRouteSetRoute(ZkError, FrAnonRoute, int *link);
DLLExport extern int frAnonRouteGetLinkX(ZkError, FrAnonRoute, int link , struct sockaddr_in *);

#endif
