/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#define _XOPEN_SOURCE 1 /* strcasecmp() */
#define _XOPEN_SOURCE_EXTENDED 1 /* strcasecmp() */

#include "zks.h"

#include <errno.h>
#include <string.h>
#include <zkCStr.h>
#include <zkDebug.h>
#include <zkMemPool.h>
#include <zkIntTypes.h>
#include <zkIntLimits.h>
#include <zkDateTime.h>
#include <frNDB.h>
#include <frNCDB.h>
#include "frNdbCvt.h"
#include "zkConfFile.h"

int cvtTTL(const char *s, void *n, ZkMemPool mpool)
{
  unsigned long l;
  (void) mpool;

#if 0  
  if (sscanf(s, "%lu day", &l)) { /* audit: ok */
    zkDebug_va ("days [%s], %lu", s, l);
    l *= 60*60*24;
  } else {
    zkDebug_va ("ERROR");
    return 0;
  }
#endif
  
  if (sscanf(s, "%lu hr", &l)) { /* audit: ok */
    l *= 60*60;
    zkDebug_va ("Expecting hrs");
  }

#if 0
  else if (sscanf(s, "%lu min", &l)) { /* audit: ok */
    l *= 60;
    zkDebug_va ("minutes");
  } else if (sscanf(s, "%lu sec", &l)) { /* audit: ok */
    /* Nothing to do */
  } else if (sscanf(s, "%lu", &l)) { /* audit: ok */
    /* Nothing to do */
  }
#endif
  
  *(unsigned long *)n = l;
  return 1;
}


int cvtZkTime(const char *s, void *n, ZkMemPool mpool)
{
  ZkDateTime zkdt;
  (void) mpool;

  if (!zkDateTimeParse(NULL, &zkdt, s)) {
    zkDebug_va("Can't parse the date string %s", s);
    *(char **)n = NULL;
    return 0;
  }
  return(cvtStr(s, n, mpool));

}

static char **tokenize(const char *line, const char *delimiters)
{
  char *orig, *buf = strdup (line);
  char **list;
  char **p, *c;
  
  orig = buf;
  p = list = calloc (sizeof (char *), 32); /* bug: hardwired 32 */

  if (line != NULL)  {
    c = strtok (buf, delimiters);
    while (c != NULL) {
      *p = strdup (c);
      zkDebug_va (" ---> tokenized [%s]", *p); 
      p++;
      c = strtok (NULL, delimiters);
    }
  }
  *p = NULL;
  free (orig);
  return (list);
}
 
int cvtUrgency(const char *s_in, void *flags, ZkMemPool mpool)
{
  char **list, **orig;
  unsigned int f;
  (void) mpool;

  orig = list = tokenize (s_in, "| ");
  f = FRNDB_URGFL_NONE;
  while (*list != NULL) {
    if (strncasecmp(*list, "BUG_FIX", 8) == 0)
      f |= FRNDB_URGFL_BUG_FIX;
    else if (strncasecmp(*list, "NEW_FEATURE", 8) == 0)
      f |= FRNDB_URGFL_NEW_FEATURE;
    else if (strncasecmp(*list, "SECURITY_FIX", 8) == 0)
      f |= FRNDB_URGFL_SECURITY_FIX;
    else
      zkDebug_va("Unknown flag [%s]", *list);
    free(*list);
    list++;
  }
  free(orig);
  *(unsigned int *) flags = f;
  return 1;
}

int cvtPort(const char *s, void *n, ZkMemPool mpool)
{
  char *end;
  long l;
  (void) mpool;
  
  errno = 0;
  l = strtol(s, &end, 10);
  if (end == s || *end != '\0' || errno == ERANGE) {
    return 0;
  }
  if (l < 0 || ((uint32_t) l > (uint32_t) UINT16_MAX)) {
	return 0;
  }
  *(uint16_t *)n = (uint32_t) l;
  return 1;
}
