/* $Id: frconf.c,v 1.3 2001/04/19 20:28:38 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <netinet/in.h>
#include <assert.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkDebug.h>
#include <zkMemPool.h>
#include "../src/frNdbCvt.h"
#include "../src/zkConfFile.h"

int main(int ac, char **av)
{

  ZkError err;
  ZkLog	log;
  char *rev;
  char *upd;
  char *by;
  char *url;
  unsigned long ttl;
  unsigned int fl;
  
  struct T_ZkConfSectionProto GenSectProto[] = {
    {"Revision", cvtStr, 0, &rev, NULL},
    {"Updated", cvtStr, 0, &upd, NULL},
    {"By", cvtStr, 1, &by, NULL},
    {"CVS", cvtStr, 1,  NULL, NULL},
    {"TTL", cvtTTL, 0,  &ttl, "10"},
    {"", NULL, 0, NULL, NULL}
  };

  struct T_ZkConfSectionProto WindowsClientSectProto[] = {
    {"Revision", cvtStr, 0, NULL, NULL},
    {"Updated", cvtStr, 0, NULL, NULL},
    {"InfoURL", cvtStr, 0, NULL, NULL},
    {"upgradeURL", cvtStr, 0, &url, NULL},
    {"DownloadURL", cvtStr, 0, NULL, NULL},
    {"Urgency", cvtUrgency, 0, &fl, NULL},
    {"", NULL, 0, NULL, NULL}
  };

  struct T_ZkConfSectionProto SignatureSectProto[] = {
    {"Signature", cvtStr, 0, &rev, NULL},
    {"", NULL, 0, NULL, NULL}
  };
  
  struct T_ZkConfProto Section[] = {
    {"Signature", SignatureSectProto},
    {"General", GenSectProto},
    {"WindowsClient", WindowsClientSectProto},
    {"", NULL}  
  };
  
#ifdef WIN32
  WSADATA	stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  assert(0 == WSAStartup(wVersionRequired, &stWSAData));
#endif

  zkErrorNew(&err);
  zkErrorLog (err, &log);
  zkLogSetStream (log, stdout);
  zkLogEnable (log);

#ifndef WIN32
  zkLogWrite (log, "Testing config library...");
#endif
  zkDebug_va("TESTING NNP");
  if (!zkConfLoadFile (err, "test.ini", Section, NULL)) {
    if (zkErrorCode(err) == ZKENOTFOUND) {
      zkDebug_va ("Config file error: [%s]", zkErrorDetail(err));
    } else {
      zkDebug_va ("ConfLoadFile failed");
      return 0;
    }
  }
  zkDebug_va("Loaded [%s=%s=%s=%lu=%u]", rev, upd, by, ttl, fl);
  assert(zkErrorFree(&err));

#ifdef WIN32
  WSACleanup();
#endif
  return 0;
}
