 /* $Id: frnnp.c,v 1.11 2001/04/19 20:28:38 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <netinet/in.h>
#include <assert.h>
#include <time.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkList.h>
#include <zkDebug.h>
#include <frNDB.h> 
#include <frNCDB.h> 
#include <frNCDBqry.h> 
#include <frNRDB.h> 
#include <frNNP.h>

extern int printhostports (const char *, ZkList);

int printhostports (const char *msg, ZkList list)
{
  FrNDBHostPort hp;

  zkListMoveCursor (NULL, list, LIST_HEAD);
  while(zkListGet(NULL, list, (void **)&hp , LIST_FORWARDS)) {
    zkDebug_va("%s %20s:%u", msg, hp->host, (unsigned short) hp->port);
  }
  return 1;
}

   

int main(int ac, char **av)
{

  ZkError err;
  ZkLog	log;
  FrNDB	ndb;
  FrNNP nnp;
  FrNDBAipList aiplist;
  ZkList list;
  
  int ruleId;
  int numAvailRoutes = 0;
  int anonRoute;
  int numaips;
  uint16_t port;
  uint32_t mytime;
  time_t timeout;
  char version[ZK_NDBNAME_SIZE];
  char *str;
  const char *cstr;
  
#ifdef WIN32
  WSADATA	stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  assert(0 == WSAStartup(wVersionRequired, &stWSAData));
#endif

  zkErrorNew(&err);
  zkErrorLog (err, &log);
  zkLogSetStream (log, stdout);
  zkLogEnable (log);

#ifndef WIN32
  zkLogWrite (log, "Freedom Network Configuration Database version 1 Program Starting...");
#endif

  /* Starting the test */
  
  //assert(frNDBEnv(err, "/home/luc/work/zk.new/etc/ndb/v5051/frnetconf.ini")); 
  //assert(frNDBEnv(err, "/home/etc/ndb/v5253/frnetconf.ini", &timeout));
  assert(frNDBEnv(err, "/home/luc/work/zk/etc/ndb/v5253/frnetconf.ini", &timeout));
  zkDebug_va("Timeout this config file in %u seconds", (unsigned int) timeout);
  assert(frNDBOpen(err, &ndb, NULL, NULL));
  assert(frNDBDump(ndb));

  assert(frNCDBGetAllAIPInfo (err, ndb, version, &aiplist, &numaips));
  assert(frNCDBDumpClientAipList (ndb, aiplist));


  zkDebug_va("TESTING NNP");

  assert(frNNPNew(err, &nnp, "BoyWonder", 69));
  /*  assert(frNNPDump(err, nnp)); */
#if 0  
  assert(frNNPRuleAddExitAIP(err, nnp, 3, &ruleId, &numAvailRoutes));
  assert(frNNPRuleAddAvoidAIP(err, nnp, 0, &ruleId, &numAvailRoutes));
#endif  
  assert(frNNPRuleAddStartAIP(err, nnp, 6, &ruleId, &numAvailRoutes));
  assert(frNNPRuleAddStartAIP(err, nnp, 5, &ruleId, &numAvailRoutes));
  assert(frNNPDump(err, nnp));
  zkDebug_va("DELETING------------\n");
  assert(frNNPRuleDelete(err, nnp, ruleId, &numAvailRoutes));
  assert(frNNPDump(err, nnp));
  zkDebug_va("DONE-----------\n");
  assert(frNNPGetAnonRouteToExit(err, nnp, &anonRoute));
  zkDebug_va("Random Anonroute = %d", anonRoute);
  assert(frNNPGetAnonRouteToExit(err, nnp, &anonRoute));
  zkDebug_va("Random Anonroute = %d", anonRoute);
  assert(frNNPGetAnonRouteToExit(err, nnp, &anonRoute));
  zkDebug_va("Random Anonroute = %d", anonRoute);
  assert(frNNPClearAllRules(err, nnp, &numAvailRoutes));
  zkDebug_va("All routes cleared. Number of avail routes = %d", numAvailRoutes);
  assert(frNCDBFreeAIPInfo (err, &aiplist));

  zkDebug_va("\n\n------------------NCDB QUERY------------\n\n");
  list = NULL;

  assert(frNCDBGetLocalTimeExpired (err, NULL, &timeout));
  zkDebug_va("Config File will expire on: %s", ctime(&timeout));

  assert(frNCDBGetLocalTimeUpdated (err, NULL, &timeout));
  zkDebug_va("Config File was updated on: %s", ctime(&timeout));

  assert(frNCDBGetNymServers (err, NULL, &list));
  printhostports ("Nymservers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetTokenServers (err, NULL, &list));
  printhostports ("TokenServer", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetNewsGroupServers (err, NULL, &list));
  printhostports ("Newsgroup servers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetDNSServers (err, NULL, &list));
  printhostports ("DNSServers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetNIDBServers (err, NULL, &list));
  printhostports ("NIDB Servers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetKeyQueryServers (err, NULL, &list));
  printhostports ("KeyQuery Servers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetKeyUpdateServers (err, NULL, &list));
  printhostports ("KeyUpdate servers", list);
  assert(frNCDBFreeHostPortList (err, &list));

  assert(frNCDBGetFMGServers (err, NULL, &list));
  printhostports ("FMG", list);
  assert(frNCDBFreeHostPortList (err, &list));
  {
    unsigned int timeout;
    assert(frNCDBGetMAIPPortAndTimeout (err, NULL, &port, &timeout));
    zkDebug_va("MAIP Port: %hu %u", (unsigned short)port, timeout);
  }

  assert(frNCDBGetTrialSerialNumberURL (err, NULL, &cstr));
  zkDebug_va("Trial Serial Number URL: %s", cstr);

  assert(frNCDBGetClientUdpTcpPort (err, NULL, &port));
  zkDebug_va("Client Udp/Tcp Port: %hu", (unsigned short)port);
  
  assert(frNCDBGetWinClientCurrRevision (err, NULL, &cstr));
  zkDebug_va("Windows Current Version: %s", cstr);
  assert(frNCDBGetWinClientCurrInfoURL (err, NULL, &cstr));
  zkDebug_va("Windows Current InfoURL: %s", cstr);
  assert(frNCDBGetWinClientCurrUpgradeURL (err, NULL, &cstr));
  zkDebug_va("Windows Current UpgradeURL: %s", cstr);
  assert(frNCDBGetWinClientCurrDownloadURL (err, NULL, &cstr));
  zkDebug_va("Windows Current DownloadURL: %s", cstr);
  assert(frNCDBGetWinClientCurrUrgency (err, NULL, &mytime));
  zkDebug_va("Urgency: %lu", (unsigned long) mytime);
  assert(frNCDBGetRevision (err, NULL, &cstr));
  zkDebug_va("NDB Revision: %s", cstr);
  assert(frNCDBGetTTL (err, NULL, &mytime));
  zkDebug_va("TTL: %lu", (unsigned long) mytime);
  assert(frNCDBGetNIDBConfigURL (err, NULL, &cstr));
  zkDebug_va("NIDB Config URL: %s", cstr);
  assert(frNCDBGetWinClientLocalTimeUpdated (err, NULL, &timeout));
  zkDebug_va("Config File will expire on: %s", ctime(&timeout));
  
  
  zkDebug_va("\n\n------------------NCDB UPDATES------------\n\n");
  /*  */
  assert(frNCDBUpdate (err, &(ndb->ncdb), "/home/luc/work/zk/etc/ndb/v5253/frnetconf2.ini", &timeout));
  zkDebug_va("Timeout this config file in %u seconds", (unsigned int) timeout);
  assert(frNCDBDump (ndb->ncdb));
  assert(frNNPFree(err, &nnp));
  assert(frNDBClose (err, &ndb));
  assert(zkErrorFree(&err));

#ifdef WIN32
  WSACleanup();
#endif
  return 0;
}
     
