 /* $Id: frtest.c,v 1.15 2001/04/19 20:28:38 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <netinet/in.h>
#include <time.h>
#include <assert.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkDebug.h>
#include <frNDB.h> 
#include <frNCDB.h> 
#include <frNRDB.h> 

void myaddr;

int main(int ac, char **av)
{

  ZkError err;
  ZkLog	log;
  FrNDB	ndb;
  FrNNP nnp;
  FrNDBAipList aiplist;
  int links, aip;
  int ruleId;
  int anonRoute = 0;
  int numAvailRoutes = 0;
  int numaips = 0;
  time_t timeout;
  const char *alias;
  const char *name;
  const char *ipstr;
  const char **charlist, **cp;
  char version[64];
  struct in_addr ipaddr;
  struct sockaddr_in saddr;
  uint16_t tcpport;
  uint16_t udpport;
  const char *namelist[] = {
    "akira.zks.net",
    "akira.zks.net",
    "caesar.zks.net",
    "dns.ops.freedom.net",
    "dogbert.zks.net",
    "download.ops.freedom.net",
    "fmgmaip.ops.freedom.net",
    "gehenna.zks.net",
    "jefferson.zks.net",
    "keyqrys.ops.freedom.net",
    "keyupds.ops.freedom.net",
    "lincoln.zks.net",
    "mizu.zks.net",
    "newsgrps.ops.freedom.net",
    "nidb.ops.freedom.net",
    "nyms.ops.freedom.net",
    "tokens.ops.freedom.net",
    "www.zks.net",
    "zimmerman.zks.net",
    ""
  };
  const char **nl;
  
#ifdef WIN32
  WSADATA	stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  assert(0 == WSAStartup(wVersionRequired, &stWSAData));
#endif

  zkErrorNew(&err);
  zkErrorLog (err, &log);
  zkLogSetStream (log, stdout);
  zkLogEnable (log);

#ifndef WIN32
  zkLogWrite (log, "Freedom Network Configuration Database version 1 Program Starting...");
#endif

  /* Starting the test */
  
  assert(frNDBEnv(err, "/home/luc/work/zk/etc/ndb/v5253/frnetconf.ini", &timeout));
  assert(frNDBOpen(err, &ndb, NULL, NULL));
  assert(frNDBDump(ndb));
  zkDebug_va("Override route test");
  assert(frNRDBSetOverrideRoute(err, ndb, 2));
  if (!frNRDBGetAnonRouteToExit (err, ndb, &anonRoute)) 
    if (zkErrorCode(err) == ZKEDEFUNCT) {
      zkDebug_va("Client is no longer valid. No routes available.");
    } else {
      zkDebug_va("Error with Get Anon Route");
    }
  zkDebug_va("Override Anonroute 2 = %d", anonRoute);
  assert(frNRDBSetOverrideRoute(err, ndb, 3));
  if (!frNRDBGetAnonRouteToExit (err, ndb, &anonRoute)) 
    if (zkErrorCode(err) == ZKEDEFUNCT) {
      zkDebug_va("Client is no longer valid. No routes available.");
    } else {
      zkDebug_va("Error with Get Anon Route");
    }
  else
    zkDebug_va("Random Anonroute = %d", anonRoute);
  zkDebug_va("Override Anonroute 3 = %d", anonRoute);
  assert(frNRDBGetNumLinksForAnonRouteId (err, ndb, anonRoute, &links));
  zkDebug_va("Number of links for Route ID %d = %d", anonRoute, links);
  assert (frNRDBGetAipXinRouteId (err, ndb, anonRoute,links-1, &aip));
  zkDebug_va("Hop %d of Route_ID %d = %d", links-1, anonRoute, aip);
  assert(frNCDBGetEntityForAipID (err, ndb, aip, &alias));
  assert(frNCDBGetNameForAipID (err, ndb, aip, &name));
  assert(frNCDBGetIPStrForAipID (err, ndb, aip, &ipstr));
  assert(frNCDBGetIPAddrForAipID (err, ndb, aip, &ipaddr));
  assert(frNCDBGetTCPPortForAipID (err, ndb, aip, &tcpport));
  assert(frNCDBGetUDPPortForAipID (err, ndb, aip, &udpport));
  fprintf (stderr, "Aip %d:\talias: %s\n\tname: %s\n\tipstr: %s\n\taddr: %s\n\ttcpport: %d\n\tudpport: %d\n",
	  aip, alias, name, ipstr, inet_ntoa(ipaddr), (int) tcpport, (int) udpport);
  assert(frNCDBGetUDPsockaddr_inForAipID (err, ndb, aip, &saddr));
  fprintf(stderr, "AIP id [%d] belongs to %s\n", aip,
#ifndef NDEBUG
          saddrSprintf(&saddr)
#else
          "Undefined in this build"
#endif
          );
  assert(frNRDBResetOverrideRoute(err, ndb));
  if (!frNRDBGetAnonRouteToExit (err, ndb, &anonRoute)) 
    if (zkErrorCode(err) == ZKEDEFUNCT) {
      zkDebug_va("Client is no longer valid. No routes available.");
    } else {
      zkDebug_va("Error with Get Anon Route");
    }
  else
    zkDebug_va("Random Anonroute = %d", anonRoute);
  if (!frNRDBGetAnonRouteToExit (err, ndb, &anonRoute)) 
    if (zkErrorCode(err) == ZKEDEFUNCT) {
      zkDebug_va("Client is no longer valid. No routes available.");
    } else {
      zkDebug_va("Error with Get Anon Route");
    }
  else
    zkDebug_va("Random Anonroute = %d", anonRoute);
  if (!frNRDBGetAnonRouteToExit (err, ndb, &anonRoute)) 
    if (zkErrorCode(err) == ZKEDEFUNCT) {
      zkDebug_va("Client is no longer valid. No routes available.");
    } else {
      zkDebug_va("Error with Get Anon Route");
    }
  else
    zkDebug_va("Random Anonroute = %d", anonRoute);

  assert(frNCDBGetAllAIPInfo (err, ndb, version, &aiplist, &numaips));
  assert(frNCDBDumpClientAipList (ndb, aiplist));
  numaips = 50;
  assert(frNCDBGetAllAipNames (err, ndb, &charlist, &numaips));
  cp = charlist;
  while (numaips) {
    zkDebug_va("Aip %2d - %s\n", numaips --, *cp);
    cp++;
  }

  zkDebug_va("New routines");
  assert (frNCDBGetAIPsInAddrUdpTcpPort (err, NULL,
				   saddr.sin_addr,
				   &udpport,
				   &tcpport));
  zkDebug_va("frNCDBGetAIPsInAddrUdpTcpPort: saddr=%s [%d/%d]",
#ifndef NDEBUG
	     saddrSprintf(&saddr),
#else
             "Undefined in this build",
#endif
             udpport, tcpport);
  assert (frNCDBGetAIPsIPAddrUdpTcpPort (err, NULL,
				   "192.168.1.8",
				   &udpport,
				   &tcpport));
  zkDebug_va("frNCDBGetAIPsIPAddrUdpTcpPort: IPaddr=%s [%d/%d]",
	     "192.168.1.8", udpport, tcpport);
  assert (frNCDBGetAIPsNameUdpTcpPort (err, NULL,
				       "akira.zks.net",
				       &udpport,
				       &tcpport));
  zkDebug_va("frNCDBGetAIPsInAddrUdpTcpPort: saddr=%s [%d/%d]",
	     "akira.zks.net", udpport, tcpport);
  free(charlist);

  {
    int *aipidlist;
    int numaips;
    int i, j;
    for (i=1; i < 8; i++){
      zkDebug_va("AIPid %d", i);
      assert(frNCDBGetAllAipNeighboursForAIPId (err,NULL, i,
						&aipidlist,&numaips));
      for (j = 0; j<numaips; j++) {
	zkDebug_va(" --> %d", aipidlist[j]);
      }
      free (aipidlist);
    }
  }
  for (nl = namelist; **nl ; nl++) {
    assert (frNCDBGetIPfromHostname (err, NULL, *nl, &ipaddr));
    zkDebug_va("frNCDBGetIPfromHostname: host=%-15s ipaddr=0x%x/%s",
	       *nl, ipaddr.s_addr, inet_ntoa(ipaddr));
  }
  
  assert(frNCDBFreeAIPInfo (err, &aiplist));
  zkDebug_va("DONE-----------\n");
  assert(zkErrorFree(&err));

#ifdef WIN32
  WSACleanup();
#endif
  return 0;
}
     
