/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#if !defined(_InfoObj_H)
#define _InfoObj_H

#include "zks.h"

#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkDefs.h"

#include "zkError.h"

#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <pthread.h>
#include "zkSysTypes.h"
#include <sys/wait.h>
#include "zkSysSocket.h"
#include <sys/wait.h>
#include <signal.h>
#include "zkSignal.h"

#include "zkCStr.h"
#include "zksocket.h"
 
#include "frNetInfo.h"
#include "CacheObj.h"



typedef struct InfoObjBufStruc T_InfoObjBuf;
struct InfoObjBufStruc {
  char*    buffer;
  size_t   size;
  time_t   time;
};

typedef struct NiqsInfoStruc T_NiqsInfo;
struct NiqsInfoStruc { 
  const char*    niqshost;
  unsigned short niqsport;
  uint32_t       timeout;
};

typedef struct InfoObjStruc *P_InfoObj;
struct InfoObjStruc {
  P_CacheObj   cache;           /* pointer to CacheObj */
  P_InfoObj    next;            /* pointer to next InfoObj in list */
  FrEntIdExp   clientdata;      /* client data for topology */
  pid_t        reloadChildPid;  /* pid of DataReload child */  
  time_t       reloadtime;      /* time to wait between reloads */
};

typedef struct InfoObjClientStruc *P_InfoObjClient;
struct InfoObjClientStruc {
  ZkSocket        c;           /* client socket */
  int             cfd;         /* client socket file descriptor */
  P_InfoObjClient next;        /* pointer to next InfoObjClient */
}; 

typedef struct InfoObjMainStruc *P_InfoObjMain;
struct InfoObjMainStruc {
  ZkSocket        s;          /* server socket */
  int             sfd;        /* server socket file descriptor */
  fd_set          fdset;      /* fdset for select call */
  int             maxfd;      /* maxfd for select call */
  ZkBool          connected;  /* ZK_TRUE if connected */
  ZkBool          server;     /* ZK_TRUE if setup as server */
  P_InfoObj       InfoObjList; /* pointer to list of InfoObj */
  T_NiqsInfo      niqsInfo;    /* NIQS info */
  P_InfoObjClient InfoObjClientList;  /* pointer to list of client connections */
  char*           InfoObjSocketName;  /* name of Unix Domain socket */
};

#define INFOOBJSOCKETNAME "InfoObjSocket"

#define INFO_OBJ_TOPOLOGY "topology"
#define INFO_OBJ_ACL      "acl"
#define INFO_OBJ_NETINFO  "netinfo"

enum { 
       InfoObjCmdNone         = 0,     
       InfoObjCmdDisconnect      ,     
       InfoObjCmdSendTopologyData,
       InfoObjCmdUpdateTopology  ,
       InfoObjCmdGetTopology     ,
       InfoObjCmdUpdateACL       ,
       InfoObjCmdGetACL          ,
       InfoObjCmdUpdateNetInfo   ,
       InfoObjCmdGetNetInfo 
};

typedef struct InfoClientCtx *P_InfoClientCtx;
struct InfoClientCtx {
   ZkError     err;
   P_InfoObj   infoobj;
   char        command;
   ZkSocket    c;
   T_NiqsInfo  *niqsInfo;   
};

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

int InfoObjMainNew( ZkError, P_InfoObjMain*, ZkBool, const char*);
int InfoObjAdd(ZkError, P_InfoObjMain, const char *,  const char *);
int InfoObjFindByName(ZkError, P_InfoObj, const char*, P_InfoObj*);
int InfoObjFindByCommand(ZkError, P_InfoObjMain, char, P_InfoObj*);
int InfoObjFree(ZkError, P_InfoObjMain*);
int InfoObjFDset( ZkError, ZkSocket, int *, fd_set*);
int InfoObjSetMaxFD(ZkError, P_InfoObjMain);


/* InfoObj Server public API */
int  InfoObjServerSignalHandler(void);
int  InfoObjServerSetNiqsInfo(ZkError, P_InfoObjMain, const char*, unsigned short, 
                              uint32_t);
int  InfoObjServerDataUpdateChild(ZkError, P_InfoObjMain, const char *, time_t);
int  InfoObjServerWaitForConnect(ZkError, const char*, P_InfoObjMain);


/* InfoObj Client public API */
int InfoObjClientConnect(ZkError, P_InfoObjMain);
int InfoObjClientDisconnect(ZkError, P_InfoObjMain);
int InfoObjClientConnected(ZkError, P_InfoObjMain, ZkBool*);
int InfoObjClientSendCommand(ZkError, P_InfoObjMain, void*);
int InfoObjClientSendData(ZkError, P_InfoObjMain, void*, size_t);
int InfoObjClientReceive(ZkError, P_InfoObjMain, void*, void**, int*, time_t*);
int InfoObjClientReadCache(ZkError, P_InfoObjMain, char, void**, int*, time_t*);
int InfoObjClientGetFD(ZkError, P_InfoObjMain, int *);

int InfoObjGetNetInfo(ZkError, const char*, uint16_t, uint32_t, char**, int*);
int InfoObjCreateNetInfo(ZkError, FrNetInfo*, FrKeyQry, char*, int);
int InfoObjGetTopology(ZkError, const char*, unsigned short, uint32_t, 
		       FrEntIdExp*, char**, int*);
int InfoObjGetACL(ZkError, const char*, unsigned short, uint32_t, char**, int*);


#ifdef __cplusplus
} // extern "C"
#endif /* __cplusplus */

#endif /* _InfoObj_H */
