/*
 * $Id: tni.c,v 1.10 2000/11/23 15:13:11 marcp Exp $ 
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 * Test FrNetInfo object
 */
#include "zkIntTypes.h"
#include "zkHash.h"
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h> 
#include "zkError.h"
#include "frEnt.h"
#include "zkList.h"
#include "frNetInfo.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

char prog[256] = "";

typedef
struct aip_data {
  uint32_t id;
  FrEntIdExp entid;
  int status;
  int rating;
  const char *host;
  unsigned short port;
} AipData;

typedef
struct maip_data {
  uint32_t id;
  FrEntIdExp entid;
  int status;
  int rating;
  const char *host;
  unsigned short port;
} MaipData;

#define MAX_AIPS 10
static AipData aip[MAX_AIPS] = {
  {1, {FRENT_TYPE_AIP, "aip1"}, 1, 1, "192.168.1.1", 51101},
  {2, {FRENT_TYPE_AIP, "aip2"}, 0, 5, "192.168.1.10", 51101},
  {3, {FRENT_TYPE_AIP, "aip3"}, 1, 1, "192.168.1.9", 51101},
  {4, {FRENT_TYPE_AIP, "aip4"}, 1, 2, "192.168.1.8", 51101},
  {5, {FRENT_TYPE_AIP, "aip5"}, 0, 1, "192.168.1.7", 51101},
  {6, {FRENT_TYPE_AIP, "aip6"}, 1, 4, "192.168.1.6", 51101},
  {7, {FRENT_TYPE_AIP, "aip7"}, 1, 8, "192.168.1.5", 51101},
  {8, {FRENT_TYPE_AIP, "aip8"}, 1, 1, "192.168.1.4", 51101},
  {9, {FRENT_TYPE_AIP, "aip9"}, 0, 2, "192.168.1.3", 51101},
  {10, {FRENT_TYPE_AIP, "aip0"}, 1, 10, "192.168.1.2", 51101},
};

#define MAX_MAIPS 5
static MaipData maip[MAX_MAIPS] = {
  {11, {FRENT_TYPE_MAIP, "maip1"}, 1, 1, "192.168.1.1", 51102},
  {12, {FRENT_TYPE_MAIP, "maip2"}, 0, 5, "192.168.1.2", 51102},
  {13, {FRENT_TYPE_MAIP, "maip3"}, 1, 1, "192.168.1.3", 51102},
  {14, {FRENT_TYPE_MAIP, "maip4"}, 1, 2, "192.168.1.4", 51102},
  {15, {FRENT_TYPE_MAIP, "maip5"}, 0, 1, "192.168.1.5", 51102},
};

/**
 * Insert AIPS
 */
static int insert_aips(ZkError err, FrNetInfo ni)
{
  const char *func = "insert_aips";
  FrNetInfoDatum data = NULL;
  int i;
  struct sockaddr_in addr;

  if (!ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param");
    return 0;
  }

  for (i = 0; i < MAX_AIPS; i++) {
    if (!frNetInfoDatumNew(err, &data)) {
      return 0;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sin_addr.s_addr = inet_addr(aip[i].host);
    addr.sin_port = htons(aip[i].port);
    
    if (!frNetInfoDatumSetSockaddr(err, data, addr)) {
      return 0;
    }

    if (!frNetInfoDatumSetEntity(err, data, aip[i].entid)) {
      return 0;
    }

    if (!frNetInfoDatumSetId(err, data, aip[i].id)) {
      return 0;
    }

    if (!frNetInfoDatumSetStatus(err, data, aip[i].status)) {
      return 0;
    }
    
    if (!frNetInfoDatumSetRating(err, data, aip[i].rating)) {
      return 0;
    }

    if (!frNetInfoInsert(err, ni, data)) {
      return 0;
    }
    data = NULL;
  }
  
  return 1;
}

static int insert_maips(ZkError err, FrNetInfo ni)
{
  const char *func = "insert_maips";
  FrNetInfoDatum data = NULL;
  int i;
  struct sockaddr_in addr;

  if (!ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param");
    return 0;
  }

  for (i = 0; i < MAX_MAIPS; i++) {
    if (!frNetInfoDatumNew(err, &data)) {
      return 0;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sin_addr.s_addr = inet_addr(maip[i].host);
    addr.sin_port = htons(maip[i].port);
    
    if (!frNetInfoDatumSetSockaddr(err, data, addr)) {
      return 0;
    }

    if (!frNetInfoDatumSetEntity(err, data, maip[i].entid)) {
      return 0;
    }

    if (!frNetInfoDatumSetId(err, data, maip[i].id)) {
      return 0;
    }

    if (!frNetInfoDatumSetStatus(err, data, maip[i].status)) {
      return 0;
    }
    
    if (!frNetInfoDatumSetRating(err, data, maip[i].rating)) {
      return 0;
    }

    if (!frNetInfoInsert(err, ni, data)) {
      return 0;
    }
    data = NULL;
  }
  
  return 1;
}

/**
 * Print the data.
 */
static int print_datum(ZkError err, FrNetInfoDatum data)
{
  const char *func = "print_datum";
  AipData aipdata;

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoDatumGetId(err, data, &aipdata.id)) {
    return 0;
  }
  if (!frNetInfoDatumGetEntity(err, data, &aipdata.entid)) {
    return 0;
  }
  if (!frNetInfoDatumGetStatus(err, data, &aipdata.status)) {
    return 0;
  }
  if (!frNetInfoDatumGetRating(err, data, &aipdata.rating)) {
    return 0;
  }
  
  fprintf(stderr, "%5d %2d:%20s %s %d\n", aipdata.id, 
          aipdata.entid.type, aipdata.entid.name, 
          aipdata.status ? "up" : "down", aipdata.rating);
  return 1;
}

/**
 * Enumerate the different type lists.
 */  
static int enum_types(ZkError err, FrNetInfo ni)
{
  const char *func = "enum_types";
  ZkList l = NULL;
  FrEntIdType i;
  size_t sz, j;
  FrNetInfoDatum data = NULL;

  if (!ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr(s)");
    return 0;
  }

  for (i = 1; i < FRENT_TYPE_MAX - 1; i++) {
    if (!frNetInfoGetListOf(err, ni, &l, i)) {
      return 0;
    }

    if (l) {
      fprintf(stderr, "\n--> Type %d:\n", i);

      if (!zkListMoveCursor(err, l, LIST_HEAD)) {
        return 0;
      }

      if (!zkListSize(err, l, &sz)) {
        return 0;
      }

      for (j = 0; j < sz; j++) {
        if (!zkListGet(err, l, (void *) &data, LIST_STAY)) {
          return 0;
        }

        print_datum(err, data);

        if (!zkListMoveCursor(err, l, LIST_FORWARDS)) {
          return 0;
        }
      }
    } else {
      fprintf(stderr, "\n--> Type %d is empty\n", i);
    }
  }

  return 1;
}

/**
 * Randomly search for an ID and print it and then delete the entire entity.
 */
static int random_search(ZkError err, FrNetInfo ni)
{
  const char *func = "random_search";
  int id;
  FrNetInfoDatum data = NULL;

  if (!ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  id = (int) (16.0 * rand() / RAND_MAX);

  fprintf(stderr, "Searching for %d\n", id);

  if (!frNetInfoSearchById(err, ni, id, &data)) {
    return 0;
  }

  print_datum(err, data);

  fprintf(stderr, "Deleting %d\n", id);
  if (!frNetInfoDeleteById(err, ni, id)) {
    return 0;
  }

  return 1;
}

/*
 *
 */
int main(int argc, char **argv)
{
  ZkError err = NULL;
  ZkLog log = NULL;
  FrNetInfo ni = NULL;
  int exit_code = EXIT_FAILURE;

  (void) argc;
  (void) argv;
   
  srand(time(NULL));

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, "/dev/tty") &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: error setting log object -- exiting.\n", prog);
  }

  fprintf (stderr, "frNetInfoNew\n");
  if (!frNetInfoNew(err, &ni)) {
    goto done;
  }

  fprintf (stderr, "insert_aips\n");
  if (!insert_aips(err, ni)) {
    goto done;
  }

  if (!insert_aips(err, ni)) {
    goto done;
  }

  fprintf (stderr, "enum_types\n");
  if (!enum_types(err, ni)) {
    goto done;
  }

  fprintf (stderr, "insert_maips\n");
  if (!insert_maips(err, ni)) {
    goto done;
  }

  fprintf (stderr, "enum_types\n");
  if (!enum_types(err, ni)) {
    goto done;
  }

  fprintf (stderr, "random_search\n");
  if (!random_search(err, ni)) {
    goto done;
  }
  if (!random_search(err, ni)) {
    goto done;
  }
  fprintf (stderr, "random_search\n");
  if (!random_search(err, ni)) {
    goto done;
  }
  if (!random_search(err, ni)) {
    goto done;
  }

  fprintf (stderr, "enum_types\n");
  if (!enum_types(err, ni)) {
    goto done;
  }
  exit_code = EXIT_SUCCESS;

 done:
  fprintf (stderr, "Done\n");
  if (ni) {
    (void) frNetInfoFree(err, &ni);
  }

  if (err) {
    (void) zkErrorFree(&err);
  }
  exit(exit_code);
}
