/* $Id: tnichild.c,v 1.8 2001/04/19 20:28:52 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "frNetInfo.h"
#include "frEnt.h"
#include <assert.h>

#if !defined(ZK_CLIENT)

static int print_datum(ZkError err, FrNetInfoDatum data)
{
  const char *func = "print_datum";
  uint32_t id;
  FrEntIdExp entid;
  int status;
  uint32_t rating;

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoDatumGetId(err, data, &id)) {
    return 0;
  }
  if (!frNetInfoDatumGetEntity(err, data, &entid)) {
    return 0;
  }
  if (!frNetInfoDatumGetStatus(err, data, &status)) {
    return 0;
  }
  if (!frNetInfoDatumGetRating(err, data, &rating)) {
    return 0;
  }
  
  fprintf(stderr, "%5d %2d:%20s %s %d\n", id, entid.type, entid.name, 
          status ? "up" : "down", rating);
  return 1;
}

static int enum_types(ZkError err, FrNetInfo ni)
{
  const char *func = "enum_types";
  ZkList l = NULL;
  FrEntIdType i;
  size_t sz, j;
  FrNetInfoDatum data = NULL;

  if (!ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR, "NULL ptr(s)");
    return 0;
  }

  for (i = 1; i < FRENT_TYPE_MAX - 1; i++) {
    if (!frNetInfoGetListOf(err, ni, &l, i)) {
      return 0;
    }

    if (l) {
      fprintf(stderr, "\n--> Type %d:\n", i);

      if (!zkListMoveCursor(err, l, LIST_HEAD)) {
        return 0;
      }

      if (!zkListSize(err, l, &sz)) {
        return 0;
      }

      for (j = 0; j < sz; j++) {
        if (!zkListGet(err, l, (void *) &data, LIST_STAY)) {
          return 0;
        }

        print_datum(err, data);

        if (!zkListMoveCursor(err, l, LIST_FORWARDS)) {
          return 0;
        }
      }
    } else {
      fprintf(stderr, "\n--> Type %d is empty\n", i);
    }
  }

  return 1;
}

#endif	/* !defined(ZK_CLIENT) */


int main(int argc, char **argv)
{
#if !defined(ZK_CLIENT)

  ZkError err = NULL;
  FrNetInfoChildCtx niupdate;
  void *buf = NULL;
  FrNetInfo ni = NULL;
  int changed = 0;
  size_t bsz = 0;
  FrKeyQry kq = 0;

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, 0));
  assert(frNetInfoNew(err, &ni));
  assert(frNetInfoChildCtxNew(err, &niupdate));
  assert(frKeyQryNew(err, &kq));

  assert(frNetInfoChildCtxSetNIQSParms(err, niupdate, "niqs.freedom.net",
                                       51107, 60));
         
  assert(frNetInfoChildCtxSetPollParms(err, niupdate, 60, 30));

  assert(frNetInfoChildCreate(err, niupdate, NETINFO_CHILD_MEMSIZE, "/freedom/var/run/freedom-niqs%010x.shm", "/freedom/var/run/freedom-niqs%010x.mtx"));

  while (1) {
    assert(frNetInfoChildCtxGetNiBuf(err, niupdate, &changed, &buf, &bsz));

    if (changed) {
      fprintf(stderr, "NetInfo changed to size %d", bsz);
      if (ni) {
        assert(frNetInfoFree(err, &ni));
        assert(frNetInfoNew(err, &ni));
      }
      assert(frNetInfoLoad(err, ni, kq, buf, bsz));

      if (buf) {
        zkFree(buf);
        buf = NULL;
      }

      enum_types(err, ni);

    } else {

      fprintf(stderr, "Sleeping for 60 secs\n");
      sleep(60);
    }
  }

  if (buf) { zkFree(buf); buf = NULL; }
  if (niupdate) { assert(frNetInfoChildCtxFree(err, &niupdate)); }
  if (ni) { assert(frNetInfoFree(err, &ni)); }
  if (err) { assert(zkErrorFree(&err)); }

#else	/* !defined(ZK_CLIENT) */

  /* silence compiler */
  (void) argc;
  (void) argv;
  
  return (EXIT_SUCCESS);
  
#endif	/* defined(ZK_CLIENT) */
}
