/*
 *  $Id: tpop.c,v 1.7 2000/11/23 15:13:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 *  Test the population of a netinfo object.
 */

#include "zks.h"

#include <netinet/in.h>
#include <sys/socket.h> 
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "zkDNS.h"
#include "frEnt.h"
#include "frNetInfo.h"
#include "frNetworkDefaults.h"
#include "frNiqs.h"
#include "zkError.h"
#include "zkHash.h"
#include "zkIntTypes.h"
#include "zkList.h"


static const char *prog;
static const uint16_t sto = 30; /* timeout talking to server */


static void usage(int ok)
{
  fprintf(stderr, "Usage: %s <host> <port>\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  FrKeyQry kq = 0;
  FrNetInfo ni = 0;
  ZkError err = 0;
  ZkipClient cli = 0;
  ZkLog log = 0;
  const char *host;
  int ok = 0;
  int port;
  struct hostent * phe;
  struct in_addr sin_addr;
  uint32_t eid;
  
  prog = av[0];
  
  if (ac != 3) {
    usage(0);
  }

  host = av[1]; port = atoi(av[2]);

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, "/dev/tty") &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: error setting log object -- exiting.\n", prog);
    goto done;
  }

  if ( ! (frCryptInit(err, 0) &&
          frNetInfoNew(err, &ni) &&
          frKeyQryNew(err, &kq))) {
    goto done;
  }

  if ( ! (frNiqsConnect(err, &cli, host, port, sto) &&
          frNetInfoPopulate(err, ni, cli, kq))) {
    goto done;
  }

  if (zkGetHostByName (err, host, &phe)) {
    memcpy(&sin_addr, phe->h_addr, phe->h_length);
    if ( frNetInfoEntIdentFromIPandPort (err, ni, sin_addr, port, & eid) )
      fprintf (stderr, "%s: Located NIQS entity identifier -> %u\n", prog, eid);
  }

  ok = 1;

 done:

  if (kq) { frKeyQryFree(err, &kq); }
  if (ni) { frNetInfoFree(err, &ni); }
  if (cli) { frNiqsDisconnect(err, &cli); }
  if (err) { zkErrorFree(&err); }

  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
