/*
 *  $Id: tupdsince.c,v 1.4 2000/11/23 15:13:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test updating a netinfo object from a particular time.
 */

#include "zks.h"

#include <netinet/in.h>
#include <sys/socket.h> 
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "frEnt.h"
#include "frNetInfo.h"
#include "frNetworkDefaults.h"
#include "frNiqs.h"
#include "zkError.h"
#include "zkHash.h"
#include "zkIntTypes.h"
#include "zkList.h"


static const char *prog;
static const uint16_t sto = 30; /* timeout talking to server */


static void usage(int ok)
{
  fprintf(stderr, "Usage: %s <host> <port> [<since>]\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  FrKeyQry kq = 0;
  FrNetInfo ni = 0;
  ZkError err = 0;
  ZkipClient cli = 0;
  ZkLog log = 0;
  const char *host;
  int ok = 0;
  int port;
  time_t since = 0;

  prog = av[0];
  
  if (ac < 3 || ac > 4) {
    usage(0);
  }

  host = av[1]; port = atoi(av[2]);
  if (ac == 4) {
    since = strtoul(av[3], 0, 10);
  }

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, "/dev/tty") &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: error setting log object -- exiting.\n", prog);
  }

  if ( ! (frCryptInit(err, 0) &&
          frNetInfoNew(err, &ni) &&
          frKeyQryNew(err, &kq))) {
    goto done;
  }

  if ( ! (frNiqsConnect(err, &cli, host, port, sto))) {
    goto done;
  }

  if ( ! frNetInfoUpdateSince(err, ni, cli, kq, since)) {
    goto done;
  }

  if ( ! frNetInfoDoStatusUpdateSince(err, ni, cli, kq, since)) {
    goto done;
  }

  ok = 1;

 done:

  if (kq) { frKeyQryFree(err, &kq); }
  if (ni) { frNetInfoFree(err, &ni); }
  if (cli) { frNiqsDisconnect(err, &cli); }
  if (err) { zkErrorFree(&err); }

  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
