/* $Id: frNetRoute.h,v 1.8 2000/03/14 17:12:16 thierry Exp $ */
#ifndef FR_NETROUTE_H_
#define FR_NETROUTE_H_

#include "zkError.h"
#include "frNetInfo.h"
#include "frPktFmt.h"

/**
 * FrNetRoute object which provides Freedom network routing.
 */
typedef struct T_FrNetRoute *FrNetRoute;

typedef enum {
  FRNETROUTE_RULE_NONE = 0,
  FRNETROUTE_RULE_RANDOM = 1,     /* Random route */
  FRNETROUTE_RULE_SPECIFY = 2,    /* Specify all hops */
  FRNETROUTE_RULE_FIRSTHOP = 4,   /* Specify the first hop */
  FRNETROUTE_RULE_AVOID = 8,      /* Avoid this hop */
  FRNETROUTE_RULE_MUSTUSE = 16,   /* Must use this hop */
  FRNETROUTE_RULE_EXITFUNNEL = 32,/* Must use this exit funnel */
  FRNETROUTE_RULE_WITHDEAD = 64   /* consider dead hops */
} FrNetRouteRuleType;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /** 
   * Create a new FrNetRoute object.
   */
  DLLExport extern
  int frNetRouteNew(ZkError err,
                    FrNetRoute *nr);

  /** 
   * Free a new FrNetRoute object.
   */
  DLLExport extern
  int frNetRouteFree(ZkError err, 
                     FrNetRoute *nr);

  /** 
   * Associate a FrNetInfo object to the network routing object.  Data
   * from the FrNetInfo object is used (list of AIPS) to get information
   * about the current state of the Freedom network.  This is a read-only
   * copy for this object.  No changes will be made to it.
   */
  DLLExport extern
  int frNetRouteSetNetInfo(ZkError err, 
                           FrNetRoute nr,
                           const FrNetInfo ni);

  /**
   * Set a rule for the network routing.  Multiple rules can be set at the
   * same time, but certain ones will take precedence.  This still needs to 
   * be worked out.  The id is the AIP ID number that can be retrieved
   * from the FrNetInfo object.
   */
  DLLExport extern
  int frNetRouteAddRule(ZkError err, 
                        FrNetRoute nr, 
                        FrNetRouteRuleType rule, 
                        uint32_t id);

  /**
   * After rules are added, apply those rules for routes with nhops number of
   * hops.  The number of routes that satisfy the rules will be returned in
   * nroutes.  If nroutes is 0, there are no routes that satisfy the 
   * conditions.
   */
  DLLExport extern
  int frNetRouteRuleApply(ZkError err, 
                          FrNetRoute nr, 
                          size_t nhops,
                          size_t *nroutes);

  /**
   * Clear all the rules previously set in this object.
   */
  DLLExport extern
  int frNetRouteRuleClearAll(ZkError err, 
                             FrNetRoute nr);
  
  /**
   * When routes are created using frNetRouteRuleApply() get route numbered
   * x.  The route is stored in the route array (which is at most
   * FRPKT_NUM_ROUTES_MAX).  The number of hops is returned in nhops.
   */
  DLLExport extern
  int frNetRouteGetRouteX(ZkError err, 
                          FrNetRoute nr, 
                          const size_t x,
                          uint32_t route[FRPKT_NUM_ROUTES_MAX],
                          size_t *nhops);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_NETROUTE_H_ */


