/*
 * $Id: netroute.c,v 1.8 2001/04/19 20:29:00 alainc Exp $
 *
 * Functions for manipulating the FrNetRoute object.
 */

#include "zks.h"
#include "netroute.h"
#include "zkStdlib.h"

/**
 * Create a new networking object.
 */
DLLExport int frNetRouteNew(ZkError err, FrNetRoute *nr)
{
  const char *func = "frNetRouteNew";

  if (!nr) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if ((*nr = zkMalloc(sizeof(struct T_FrNetRoute))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKENOMEM, "malloc() failure");
    return 0;
  }

  (*nr)->routes = NULL;
  (*nr)->nroutes = 0;
  (*nr)->rules = 0;

  (*nr)->excludes = NULL;
  (*nr)->nexcludes = 0;
  (*nr)->includes = NULL;
  (*nr)->nincludes = 0;
  (*nr)->fh = 0;
  (*nr)->lh = 0;

  (*nr)->ni = NULL;
  (*nr)->graph = NULL;

  return 1;
}

/**
 * Free the network routing object.
 */
DLLExport int frNetRouteFree(ZkError err, FrNetRoute *nr)
{
  const char *func = "frNetRouteFree";

  if (!nr || !*nr) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKENULLPTR, "NULL param");
    return 0;
  }

  if ((*nr)->excludes) {
    zkFree((*nr)->excludes);
    (*nr)->excludes = NULL;
  }

  if ((*nr)->includes) {
    zkFree((*nr)->excludes);
    (*nr)->excludes = NULL;
  }

  if ((*nr)->routes) {
    zkFree((*nr)->routes);
    (*nr)->routes = NULL;
  }
  (*nr)->ni = NULL;
  if ((*nr)->graph) {
    (void) zkGraphFree(err, &((*nr)->graph));
    (*nr)->graph = NULL;
  }

  zkFree(*nr);
  *nr = NULL;
  
  return 1;
}

/**
 * Seed the network routing object with the current view of the Freedom 
 * network.  The FrNetInfo object is treated as a read-only object.
 */
DLLExport int frNetRouteSetNetInfo(ZkError err, FrNetRoute nr, 
                                   const FrNetInfo ni)
{
  const char *func = "frNetRouteSetNetInfo";

  if (!nr || !ni) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  nr->ni = ni;

  return 1;
}

/**
 * Returns an array of nhops of a route made from previous calls to the
 * rules.
 */
DLLExport int frNetRouteGetRouteX(ZkError err, FrNetRoute nr, const size_t x,
                                  uint32_t route[FRPKT_NUM_ROUTES_MAX],
                                  size_t *nhops)
{
  const char *func = "frNetRouteGetRouteX";
  size_t i;

  if (!nr || !nhops) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (x >= nr->nroutes) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKEPARMBAD, "No such route");
    return 0;
  }

  if (nr->routes[x].nhops > FRPKT_NUM_ROUTES_MAX) {
    zkErrorSet(err, func, ZKERR, ZKMFRNETROUTE, ZKEPARMBAD, "Too many hops");
    return 0;
  }
  *nhops = nr->routes[x].nhops;

  for (i = 0; i < *nhops; i++) {
    route[i] = nr->routes[x].router[i];
  }

  return 1;
}
