/*
 * $Id: tnr.c,v 1.15 2001/04/19 20:29:01 alainc Exp $
 */
#include "zkStdio.h"
#include "zkStdlib.h"
#include <assert.h>
#include "frNetRoute.h"
#include "frNetInfo.h"

typedef
struct nb_data {
  size_t nb;
  uint32_t nbid[5];
} NbData;

typedef 
struct aip_data {
  uint32_t id;
  FrEntIdExp entid;
  int status;
  int rating;
  struct sockaddr_in saddr;
  NbData nb;
} AipData;

char *prog = NULL;

static int seed_ni(ZkError err, FrNetInfo ni)
{
#ifdef macintosh 
#define INIT_FAMILY sizeof(struct sockaddr_in), AF_INET
#define INIT_ADDR {0}
#else 
#define INIT_FAMILY (AF_INET)
#define INIT_ADDR {{{0}}}
#endif
  AipData aip[] = {
    /* Since the sockaddr is used as an index, it's sufficient to change
       port numbers in order to provide a uniq key */
    {1, {FRENT_TYPE_AIP, "aip1"}, 1, 1, { INIT_FAMILY,1,INIT_ADDR}, {1, {2}}},
    {2, {FRENT_TYPE_AIP, "aip2"}, 1, 5, { INIT_FAMILY,2,INIT_ADDR}, {3, {1, 3, 5}}},
    {3, {FRENT_TYPE_AIP, "aip3"}, 1, 1, { INIT_FAMILY,3,INIT_ADDR}, {4, {2, 4, 10, 9}}},
    {4, {FRENT_TYPE_AIP, "aip4"}, 1, 2, { INIT_FAMILY,4,INIT_ADDR}, {3, {3, 5, 9}}},
    {5, {FRENT_TYPE_AIP, "aip5"}, 1, 1, { INIT_FAMILY,5,INIT_ADDR}, {4, {2, 4, 6, 10}}},
    {6, {FRENT_TYPE_AIP, "aip6"}, 1, 4, { INIT_FAMILY,6,INIT_ADDR}, {2, {5, 8}}},
    {7, {FRENT_TYPE_AIP, "aip7"}, 1, 8, { INIT_FAMILY,7,INIT_ADDR}, {2, {8, 9}}},
    {8, {FRENT_TYPE_AIP, "aip8"}, 1, 1, { INIT_FAMILY,8,INIT_ADDR}, {2, {6, 7}}},
    {9, {FRENT_TYPE_AIP, "aip9"}, 1, 2, { INIT_FAMILY,9,INIT_ADDR}, {3, {3, 4, 7}}},
    {10, {FRENT_TYPE_AIP, "aip0"}, 1, 10, { INIT_FAMILY,10,INIT_ADDR}, {2, {3, 5}}},
    {11, {FRENT_TYPE_AIP, "aip0"}, 1, 10, { INIT_FAMILY,11,INIT_ADDR}, {11, {0}}}
  };
  size_t i, j;
  FrNetInfoDatum data = NULL;

  if (!ni) {
    return 0;
  }

  for (i = 0; i < 10; i++) {
    if (!frNetInfoDatumNew(err, &data)) {
      return 0;
    }

    if (!frNetInfoDatumSetEntity(err, data, aip[i].entid)) {
      return 0;
    }

    if (!frNetInfoDatumSetId(err, data, aip[i].id)) {
      return 0;
    }

    if (!frNetInfoDatumSetStatus(err, data, aip[i].status)) {
      return 0;
    }
    
    if (!frNetInfoDatumSetRating(err, data, aip[i].rating)) {
      return 0;
    }

    if (!frNetInfoDatumSetSockaddr(err, data, aip[i].saddr)) {
      return 0;
    }

    for (j = 0; j < aip[i].nb.nb; j++) {
      if (!frNetInfoDatumAddNb(err, data, aip[i].nb.nbid[j])) {
        return 0;
      }
    }

    if (!frNetInfoInsert(err, ni, data)) {
      return 0;
    }
    
    data = NULL;
  }
  return 1;
}

int main(void)
{
  ZkError err = NULL;
  ZkLog log = NULL;
  FrNetInfo ni = NULL;
  FrNetRoute nr = NULL;
  size_t nroutes;
  uint32_t routers[FRPKT_NUM_ROUTES_MAX];
  size_t i, j;
  size_t nhops;

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, "output.txt") &&
          zkLogEnable(log))) {
    zkFprintf(stderr, "%s: error setting log object -- exiting.\n", prog);
  }

  assert(frNetInfoNew(err, &ni));
  assert(frNetRouteNew(err, &nr));
  assert(seed_ni(err, ni));
  assert(seed_ni(err, ni));

  assert(frNetRouteSetNetInfo(err, nr, ni));

  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_FIRSTHOP, 10));
  assert(frNetRouteRuleApply(err, nr, 4, &nroutes));
  
  zkFprintf(stderr, "There are %d 4 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }
  assert(frNetRouteRuleClearAll(err, nr));

  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_EXITFUNNEL, 8));
  assert(frNetRouteRuleApply(err, nr, 3, &nroutes));
  
  zkFprintf(stderr, "There are %d 3 hop routes\n", nroutes);

  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));

  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_EXITFUNNEL, 8));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_FIRSTHOP, 3));
  assert(frNetRouteRuleApply(err, nr, 4, &nroutes));
  zkFprintf(stderr, "There are %d 4 hop routes\n", nroutes);

  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 1));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 2));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 5));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 10));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 3));
  assert(frNetRouteRuleApply(err, nr, 5, &nroutes));

  zkFprintf(stderr, "There are %d 5 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));

  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 1));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 2));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 7));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 9));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_SPECIFY, 3));
  assert(frNetRouteRuleApply(err, nr, 5, &nroutes));
  assert(!nroutes);

  zkFprintf(stderr, "There are %d 5 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_FIRSTHOP, 4));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_AVOID, 5));
  assert(frNetRouteRuleApply(err, nr, 3, &nroutes));
  zkFprintf(stderr, "There are %d 3 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_FIRSTHOP, 4));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_AVOID, 5));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_AVOID, 7));
  assert(frNetRouteRuleApply(err, nr, 3, &nroutes));
  zkFprintf(stderr, "There are %d 3 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_FIRSTHOP, 4));
  assert(frNetRouteAddRule(err, nr, FRNETROUTE_RULE_MUSTUSE, 3));
  assert(frNetRouteRuleApply(err, nr, 3, &nroutes));
  zkFprintf(stderr, "There are %d 3 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  assert(frNetRouteRuleClearAll(err, nr));
  assert(frNetRouteRuleApply(err, nr, 1, &nroutes));
  zkFprintf(stderr, "There are %d 3 hop routes\n", nroutes);
  for (i = 0; i < nroutes; i++) {
    zkFprintf(stderr, "Route Number %d\n", i);
    assert(frNetRouteGetRouteX(err, nr, i, routers, &nhops));
    for (j = 0; j < nhops; j++) {
      zkFprintf(stderr, "-> %d ", routers[j]);
    }
    zkFprintf(stderr, "\n");
  }

  /* Clean up all my mess */
  if (nr) {
    assert(frNetRouteFree(err, &nr));
  }

  if (ni) {
    assert(frNetInfoFree(err, &ni));
  }

  if (err) {
    assert(zkErrorFree(&err));
  }

  return(EXIT_SUCCESS);
}
