/*
 * Freedom Network Information Database Object Library
 * $Id: frNidbObj.h,v 1.19 2001/04/19 20:29:15 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 *
 */

#ifndef FRNIDB_OBJ_H_
#define FRNIDB_OBJ_H_

#include "zks.h"


#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "zkNetinetIn.h"
#include "zkArpaInet.h"

#include "zkError.h"
#include "zkIntTypes.h"
#include "frEnt.h"
#include "frDBOTypes.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


  /**
   * Get the entity identification structure
   */
  DLLExport extern int frNidbObjGetEntity (ZkError err,
                                           FrNidbObj obj,
                                           FrEntIdExp *entity);
  
  /**
   * Get a entity identifier 
   */
  DLLExport extern int frNidbObjGetEntIdent (ZkError err,
                                             FrNidbObj obj,
                                             FrEntIdent *ent_ident);
  
  /**
   * Get a entity type
   */
  DLLExport extern int frNidbObjGetType (ZkError err,
                                         FrNidbObj obj,
                                         FrNidbObjType *type);

  /**
   * Get a time this object's info was updated
   */
  DLLExport extern int frNidbObjGetLastUpdateTime (ZkError err,
                                         FrNidbObj obj,
                                         time_t *last_update_t);

  /**
   * Get the time to live (in seconds) this response is valid for
   */
  DLLExport extern int frNidbObjGetTTLSeconds (ZkError err,
                                         FrNidbObj obj,
                                         uint32_t *ttl_seconds);

  /**
   * Get a the sizeof the structure 
   */
  DLLExport extern int frNidbObjGetSizeof (ZkError err,
                                           FrNidbObj obj,
                                           size_t *osz);

  /**
   * Get the server paramaters.
   */
  DLLExport extern int frNidbObjGetServerParams (ZkError err, FrNidbObj obj,
                         struct in_addr *sin_addr, uint16_t *port,
                         uint32_t *status, uint32_t *rating, const char **regioncode,
                         char ** region,
                         const char **servoperator);

  /**
   * Get the AIP Parameters.
   */
  DLLExport extern int frNidbObjGetAipParams (ZkError err, FrNidbObj obj, 
			 struct in_addr *sin_addr, uint16_t *dh_port,
			 uint16_t *main_port, uint32_t *status,
                         uint32_t *rating, const char **region_code,
                         char **region, const char **servoperator, size_t *num_n);

  /**
   * Get the i'th neighbour from this object list. First element is at 0.
   * NULL if none are available
   */
  DLLExport extern int frNidbObjGetAipNeighbour (ZkError err, FrNidbObj obj,
			    size_t idx,
			    FrEntIdent *remote_aip_eid);


  /**
   * Get the Status Parameters.
   */
  DLLExport extern
  int frNidbObjGetStatusParams (ZkError err, FrNidbObj obj, size_t *num_resp);

  /**
   * Get the i'th response from this object list. First element is at 0.
   */
  DLLExport
  int frNidbObjGetStatusRespAtIdx (ZkError err, FrNidbObj obj,
                                   size_t idx,
                                   FrEntIdent *ent_ident,
                                   uint32_t *status,
                                   uint32_t *rating);

  /*
   * Unpack an Nidb Object
   *
   * Caller must free the data when done
   */
  DLLExport extern int frNidbObjUnpack (ZkError err, void *buf,
                                        size_t bsz, FrNidbObj *obj);



  DLLExport extern int frNidbObjNew (ZkError err, FrNidbObj *obj);
  
  DLLExport extern int frNidbObjNewSpecifyVersion (ZkError err, FrNidbObj *obj, 
                                                   uint32_t version);
  
  DLLExport extern int frNidbObjFree (ZkError err, FrNidbObj *obj);
       
  /**
   * Get a time this object's info was updated
   */
  DLLExport extern int frNidbObjSetLastUpdateTime (ZkError err,
                                         FrNidbObj obj,
                                         time_t last_update_t);

  /**
   * Get the time to live (in seconds) this response is valid for
   */
  DLLExport extern int frNidbObjSetTTLSeconds (ZkError err,
                                         FrNidbObj obj,
                                         uint32_t ttl_seconds);
  /**
   * Set the entity identification structure
   */
  DLLExport extern int frNidbObjSetEntity (ZkError err,
                                           FrNidbObj obj,
                                           FrEntIdExp *entity);
  
  /**
   * Set a entity identifier
   */
  DLLExport extern int frNidbObjSetEntIdent (ZkError err, FrNidbObj obj, FrEntIdent ent_ident);

  /**
   * Set a object type
   */
  DLLExport extern int frNidbObjSetType (ZkError err, FrNidbObj obj, FrNidbObjType type);

  /**
   * Set the server paramaters.
   */
  DLLExport extern int frNidbObjSetServerParams (ZkError err, FrNidbObj obj,
			 struct in_addr sin_addr, uint16_t port,
                         uint32_t status, uint32_t rating,
                         const char * region, const char *servoperator);
  /**
   * Set the AIP Parameters.
   */
  DLLExport extern int frNidbObjSetAipParams (ZkError err, FrNidbObj *obj, 
			 struct in_addr sin_addr, uint16_t dh_port,
			 uint16_t fr_port, uint32_t status, uint32_t rating,
                         const char *region, const char *servoperator, size_t num_n);

  /**
   * Set the i'th neighbour from this object list. First element is at 0.
   * NULL if none are available
   */
  DLLExport extern int frNidbObjSetAipNeighbour (ZkError err, FrNidbObj obj,
                                size_t idx,
                                FrEntIdent remote_aip_eid);
       
  /**
   * Get the Status Parameters.
   */
  DLLExport extern
  int frNidbObjSetStatusParams (ZkError err, FrNidbObj *obj, size_t num_resp);

  /**
   * Set the i'th response from this object list. First element is at 0.
   */
  DLLExport
  int frNidbObjSetStatusRespAtIdx (ZkError err, FrNidbObj obj,
                                   size_t idx,
                                   FrEntIdent ent_ident,
                                   uint32_t status,
                                   uint32_t rating );

  /**
   * Get the QueryList parameters
   */
  DLLExport
  int frNidbObjGetListParams (ZkError err, FrNidbObj obj, 
			      size_t *num_resp);

  /**
   * Set the QueryList parameters
   */
  DLLExport
  int frNidbObjSetListParams (ZkError err, FrNidbObj *obj, 
			      size_t num_resp);
  
  
  /**
   * Set the i'th response from this object list. First element is at 0.
   */
  DLLExport
  int frNidbObjSetListRespAtIdx (ZkError err, FrNidbObj obj,
                                 size_t idx, 
				 FrNidbObj nidbObj );

  /**
   * Get the i'th response from this object list. First element is at 0.
   */
  DLLExport
  int frNidbObjGetListRespAtIdx (ZkError err, FrNidbObj obj,
				 size_t idx,
				 FrNidbObj *nidbObj);

  /**
   * Free the children NidbObj and the container List object
   */
  DLLExport
  int frNidbObjListFree (ZkError err, FrNidbObj *obj);
  
  
  /**
   * Description: frNidbVerifyFip10CountryRegionAbbreviation takes
   *              a Fip10 country region code in parameter and verifies
   *              if it exists in the FIPS10 list. (*pExists) will be set
   *              to 1 if the abbreviation exists in the table. Only 'abbrev_sz'
   *              caracters will be checked.
   *
   * Parameters: - [ZkError] error: current ZK error object
   *             - [const char *] abbrev: Abbreviation string to be verified
   *             - [size_t] abbrev_sz: number of caracters in abbrev (should be 2 or 4)
   *             - [int *] pExists: Pointer to the exist flag
   */
  DLLExport
  int frNidbVerifyFip10CountryRegionAbbreviation (
    ZK_INOUT ZkError err,
    ZK_IN    const char * abbrev,
    ZK_IN    size_t abbrev_sz,
    ZK_OUT   int *pExists );
  
  /**
   * Finds the representation of the region in the
   * FIPS10 list. Countryregion points to a static
   * buffer that will be overwritten at the next call; copy
   * it if you need it.
   */
  DLLExport
  extern int frNidbLocateFip10CountryRegion (
    ZK_INOUT ZkError err,
    ZK_IN    const char * abbrev,
    ZK_OUT   char **countryregion);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNIDB_OBJ_H_ */
