/*
 * Freedom Network Information Database Query Response Library
 * $Id: frNidbQryResp.h,v 1.9 2001/04/19 20:29:16 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 *
 */

#ifndef FRNIDB_QRYRESP_H_
#define FRNIDB_QRYRESP_H_

#include "zks.h"
#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "zkNetinetIn.h"
#include "zkArpaInet.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkDump.h"
#include "frEnt.h"
#include "frDBOTypes.h"

typedef enum {
  NTQP_NOOP_,
  NTQP_LIST_ENTITIES,
  NTQP_GET_TOPOLOGY,
  NTQP_GET_DESCRIPTION,
  NTQP_GET_STATE,
  NTPQ_UNDEF_
} FrNidbQueryType;

#define FRNIDB_QUERY_VERSION 1

typedef struct FrNidbQuery_T *FrNidbQuery;
typedef struct FrNidbResponse_T *FrNidbResponse;
typedef struct FrNidbQryResp_T *FrNidbQryResp;
/**
 * The responses that one gets from the Niqs (Nidb)
 */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  DLLExport extern const char *frNidbQRGetQueryTypeStr (ZkError, FrNidbQueryType qtype);
  DLLExport extern int frNidbQRNew (ZkError, FrNidbQryResp *qr);
  DLLExport extern int frNidbQRFree (ZkError, FrNidbQryResp *qr);
  DLLExport extern int frNidbQRSetQuery (ZkError, FrNidbQryResp qr, FrNidbQueryType qtype);
  DLLExport extern int frNidbQRSetQueryAddEnt (ZkError err, FrNidbQryResp qr,
				     FrEntIdType enttype);
  DLLExport extern int frNidbQRSetQueryFilter (ZkError, FrNidbQryResp qr, char *filter);
  DLLExport extern int frNidbQRPrintQuery (ZkError, FrNidbQryResp qr,
				 ZkDumpCallback callback, void * cdata );
  
  DLLExport extern int frNidbQRSetRespRetcode (ZkError err, FrNidbQryResp qr,
				     uint32_t retcode);
  DLLExport extern int frNidbQRSetRespTTL (ZkError err, FrNidbQryResp qr, uint32_t ttl);
  DLLExport extern int frNidbQRSetRespNumAns (ZkError err, FrNidbQryResp qr, uint32_t num);
  DLLExport extern int frNidbQRSetRespAnswers (ZkError err, FrNidbQryResp *qr,
				     char *answer, size_t asz);
  DLLExport extern int frNidbQRGetRespNumAns (ZkError err, FrNidbQryResp qr, uint32_t *num);
  DLLExport extern int frNidbQRGetRespAnswers (ZkError err, FrNidbQryResp qr,
                                     char **answer, size_t *asz);

  DLLExport extern int frNidbQRespListNew (ZkError err, FrDBOTypeNidbQRespList *rl,
                                 size_t num_resps);
  DLLExport extern int frNidbQRespListFree (ZkError err, FrDBOTypeNidbQRespList *rl);
  DLLExport extern int frNidbQRespListSetNum (ZkError err, FrDBOTypeNidbQRespList *rl,
                                    uint32_t num);
  DLLExport extern int frNidbQRespListSetEntIdentAtIdx (ZkError err, 
                                              FrDBOTypeNidbQRespList rl,
                                              FrDBOTypeEntityID eid,
                                              size_t idx);
  DLLExport extern int frNidbQRespListGetNum (ZkError err, FrDBOTypeNidbQRespList rl,
                                    uint32_t *num);
  DLLExport extern int frNidbQRespListGetEntIdentAtIdx (ZkError err,
                                              FrDBOTypeNidbQRespList rl,
                                              uint32_t *eid, size_t idx);




#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNIDB_QRYRESP_H_ */
