/*
 * Freedom Network Information Database
 *
 * Query Service
 * List Command
 *
 * $Id: fr_nidb_list.c,v 1.11 2001/04/19 20:29:16 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkCStr.h"
#include "zkAlloc.h"
#include "frNidbQryResp.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

/**
 * Set up a new list command
 */
DLLExport
int frNidbQRespListNew (ZkError err, FrDBOTypeNidbQRespList *rl, size_t numelems)
{
  static const char *funcname = "frNidbQRespListNew";
  size_t rsz = 0, i = 0;

  if ( !rl ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  rsz = numelems; /* To signal the DBO to allocate the proper amount of
                     space */
  if (! zkDBONew(err, ZKDBO_REG_FRNIDB_QRESPLIST, (void **) rl, &rsz))
    return 0;

  (*rl)->num_answers = numelems;
  (*rl)->dboSize = rsz;
  
  for (i = 0; i < numelems; i++) {
    (*rl)->answer[i].entid = 0;
  }
  
  return 1;
}

DLLExport
int frNidbQRespListFree (ZkError err, FrDBOTypeNidbQRespList *rl)
{
  static const char *funcname = "frNidbQRespListFree";

  if ( !rl || !*rl ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  zkFree(*rl);
  *rl = NULL;
  
  return 1;
}  

#if 0
int frNidbQRespListSetNum (ZkError err, FrDBOTypeNidbQRespList *rl, uint32_t num)
{
  static const char *funcname = "frNidbQRespListSetNum";
  size_t i;

  if ( !rl || !*rl ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if ((*rl)->num) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKERROR,"Unable to handle initialized list response");
    return 0;
  }
  
  (*rl)->num = num;
  (*rl)->size = sizeof(uint32_t)*(num+2);

  if (!zkMemRealloc(err, (void **) rl, (*rl)->size))
    return 0;

  for (i = 0; i < num; i++) {
    (*rl)->ent_ident[i] = 0;
  }

  return 1;
}  

#endif

DLLExport
int frNidbQRespListSetEntIdentAtIdx (ZkError err, FrDBOTypeNidbQRespList rl,
				    FrDBOTypeEntityID eid, size_t idx)
{
  static const char *funcname = "frNidbQRespListSetEntIdentAtIdx";

  if ( !rl ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if (eid == 0) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKEPARMRANGE,
	       "Entity of 0 is invalid");
    return 0;
  }    

  if (idx >= rl->num_answers ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKEPARMRANGE,
	       "Requested index is out of range");
    return 0;
  }

  /* rl->answer[idx].entid = eid; - removed due to warning */
  zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKERROR,
	       "Disabled function. This should not be called.");

  return 0;
}  

DLLExport
int frNidbQRespListGetNum (ZkError err, FrDBOTypeNidbQRespList rl, uint32_t *num)
{
  static const char *funcname = "frNidbQRespListGetNum";

  if ( !rl || !num ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }
  
  *num = rl->num_answers;

  return 1;
}  

DLLExport
int frNidbQRespListGetEntIdentAtIdx (ZkError err, FrDBOTypeNidbQRespList rl,
				    uint32_t *eid, size_t idx)
{
  static const char *funcname = "frNidbQRespListGetEntIdentAtIdx";

  if ( !rl || !eid ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if (idx >= rl->num_answers ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKEPARMRANGE,
	       "Requested index is out of range");
    return 0;
  }

  *eid = rl->answer[idx].entid;

  return 1;
}  
