/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

/*
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include <assert.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkBase64.h"
#include "frDBOFuncs.h"
#include "frNidbObj.h"
#include "zkString.h"
#include "zkCStr.h"


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

static int testNidbObjList( ZkError err )
{
  int retcode = 0;

  FrNidbObj srcList = NULL, src = NULL;
  FrNidbObj dstList = NULL; 
  void *buf = NULL;
  size_t src_sz = 0, buf_sz = 0;

  FrEntIdExp entity;
  FrEntIdent ent_ident;
  FrNidbObjType type;
  uint32_t ttl_seconds;
  time_t last_update;
  struct in_addr sin_addr;
  uint16_t dh_port;
  uint16_t fr_port;
  uint32_t status;
  uint32_t rating;
  const char *region;
  const char *servoperator;
  size_t num_n;
  FrEntIdent neigh1;
  FrEntIdent neigh2;
  size_t num_resp;

  if ( !frNidbObjNew(err, &srcList)
       || !frNidbObjNew(err, &src) ) {
    zkDebug( "Unable to instantiate source objects" );
    goto failed;
  }

  entity.type = FRENT_TYPE_UNUSED_;
  zkCStrlcpy(entity.name, "*", sizeof entity.name); 
  ent_ident = 1;
  type = FRNIDBOBJ_LIST_RESP;
  ttl_seconds = 3600;
  last_update = time(NULL);
  num_resp = 3;

  if ( !frNidbObjSetEntity( err, srcList, &entity )
       || !frNidbObjSetEntIdent( err, srcList, ent_ident )
       || !frNidbObjSetType( err, srcList, type )
       || !frNidbObjSetTTLSeconds( err, srcList, ttl_seconds )
       || !frNidbObjSetLastUpdateTime( err, srcList, last_update )
       || !frNidbObjSetListParams( err, &srcList, num_resp )
       ) {
    zkDebug( "Unable to set List params");
    goto failed;
  }

  entity.type = FRENT_TYPE_AIP;
  zkCStrlcpy(entity.name, "Aip7",sizeof entity.name); 
  ent_ident = 7;
  type = FRNIDBOBJ_AIP;
  ttl_seconds = 3600;
  last_update = time(NULL);
  inet_aton("10.16.128.31", &sin_addr );
  dh_port = 51000;
  fr_port = 51100;
  status = 1;
  rating = 100;
  region = "CA10";
  servoperator = "ZKS";
  num_n = 2;
  neigh1 = 8;
  neigh2 = 6;
  
  if ( !frNidbObjSetEntity( err, src, &entity )
       || !frNidbObjSetEntIdent( err, src, ent_ident )
       || !frNidbObjSetType( err, src, type )
       || !frNidbObjSetTTLSeconds( err, src, ttl_seconds )
       || !frNidbObjSetLastUpdateTime( err, src, last_update )
       || !frNidbObjSetAipParams( err, &src, sin_addr, dh_port, fr_port, 
				  status, rating, region, servoperator, num_n )
       || !frNidbObjSetAipNeighbour( err, src, 0, neigh1)
       || !frNidbObjSetAipNeighbour( err, src, 1, neigh2)
       ) {
    zkDebug( "Unable to set src 1" );
    goto failed;
  }

  if( !frNidbObjSetListRespAtIdx( err, srcList, 0, src ) ) {
    zkDebug("Unable to assign child 1");
    goto failed;
  }
  
  entity.type = FRENT_TYPE_AIP;
  zkCStrlcpy(entity.name, "Aip8", sizeof entity.name); 
  ent_ident = 8;
  type = FRNIDBOBJ_AIP;
  ttl_seconds = 3600;
  last_update = time(NULL);
  inet_aton("10.16.128.32", &sin_addr );
  dh_port = 51000;
  fr_port = 51100;
  status = 1;
  rating = 100;
  region = "CA10";
  servoperator = "ZKS";
  num_n = 2;
  neigh1 = 7;
  neigh2 = 6;
  
  if ( !frNidbObjSetEntity( err, src, &entity )
       || !frNidbObjSetEntIdent( err, src, ent_ident )
       || !frNidbObjSetType( err, src, type )
       || !frNidbObjSetTTLSeconds( err, src, ttl_seconds )
       || !frNidbObjSetLastUpdateTime( err, src, last_update )
       || !frNidbObjSetAipParams( err, &src, sin_addr, dh_port, fr_port, 
				  status, rating, region, servoperator, num_n )
       || !frNidbObjSetAipNeighbour( err, src, 0, neigh1)
       || !frNidbObjSetAipNeighbour( err, src, 1, neigh2)
       ) {
    zkDebug( "Unable to set src 2" );
    goto failed;
  }

  if( !frNidbObjSetListRespAtIdx( err, srcList, 1, src ) ) {
    zkDebug("Unable to assign child 2");
    goto failed;
  }

  entity.type = FRENT_TYPE_AIP;
  zkCStrlcpy(entity.name, "Aip9", sizeof entity.name); 
  ent_ident = 9;
  type = FRNIDBOBJ_SERVER;
  ttl_seconds = 3600;
  last_update = time(NULL);
  inet_aton("10.16.128.32", &sin_addr );
  fr_port = 51000;
  status = 1;
  rating = 100;
  region = "CA10";
  servoperator = "ZKS";
  
  if ( !frNidbObjSetEntity( err, src, &entity )
       || !frNidbObjSetEntIdent( err, src, ent_ident )
       || !frNidbObjSetType( err, src, type )
       || !frNidbObjSetTTLSeconds( err, src, ttl_seconds )
       || !frNidbObjSetLastUpdateTime( err, src, last_update )
       || !frNidbObjSetServerParams( err, src, sin_addr, fr_port, 
				  status, rating, region, servoperator )
       ) {
    zkDebug( "Unable to set src 3" );
    goto failed;
  }

  if( !frNidbObjSetListRespAtIdx( err, srcList, 2, src ) ) {
    zkDebug("Unable to assign child 2");
    goto failed;
  }
  
  frNidbObjGetSizeof( err, srcList, &src_sz );
  if( !zkDBOPack( err, srcList, src_sz, &buf, &buf_sz ) ) {
    zkDebug( "Error packing data");
    goto failed;
  }

  if( !frNidbObjUnpack( err, buf, buf_sz, &dstList ) ) {
    zkDebug( "Error unpacking data" );
    goto failed;
  }

  retcode = 1;
 failed:
  if( retcode ) {
    zkDebug("NidbObjList test passed");
  } else {
    zkDebug("NidbObjList test FAILED");
  }
  return retcode;;
}

int main(void)
{

  ZkError err;
  ZkLog log;

  FrNidbObj obj1 = NULL;
  FrNidbObj obj2 = NULL;
  FrNidbObjType type;
  FrEntIdExp entity;
  const char *servoperator = NULL;
  char *region = NULL;
  const char *rcode = NULL;
  size_t idx = 0;
  size_t num = 0;
  struct in_addr sin_addr;
  uint16_t port = 0, p1 = 0, p2 = 0;
  uint32_t eid = 0, status=0, rating=0;

  zkDebugOn();	

  assert(zkErrorNew(&err));
  assert(zkErrorLog (err, &log));
  assert(zkLogSetStream (log, stdout));
  assert(zkLogEnable (log));

  assert(frDBODriverInitAll(err));
  assert(frNidbObjNew (err, &obj1));
  entity.type = FRENT_TYPE_AIP;
  zkCStrlcpy(entity.name, "My Aip", sizeof entity.name);
  assert(frNidbObjSetEntity(err, obj1, &entity));
  assert(frNidbObjSetEntIdent (err, obj1, 101));
  assert(frNidbObjSetType (err, obj1, FRNIDBOBJ_SERVER));
  zkInet_aton("192.168.1.30", &sin_addr);
  assert(frNidbObjSetServerParams (err, obj1, sin_addr, 2020, 1, 3, "CA10", "ZKS"));

  assert(frNidbObjNew (err, &obj2));
  entity.type = FRENT_TYPE_FMG;
  zkCStrlcpy(entity.name, "My Fmg", sizeof entity.name);
  assert(frNidbObjSetEntity(err, obj2, &entity));
  assert(frNidbObjSetEntIdent (err, obj2, 102));
  assert(frNidbObjSetType (err, obj2, FRNIDBOBJ_AIP));
  zkInet_aton("192.168.1.25", &sin_addr);
  assert(frNidbObjSetAipParams (err, &obj2, sin_addr, 51101, 51102, 1, 4, "CA10", "ZKS", 4));
  assert(frNidbObjSetAipNeighbour (err, obj2, 0, 10));
  assert(frNidbObjSetAipNeighbour (err, obj2, 1, 20));
  assert(frNidbObjSetAipNeighbour (err, obj2, 2, 30));
  assert(frNidbObjSetAipNeighbour (err, obj2, 3, 40));

  assert(frNidbObjGetEntity (err, obj1, &entity));
  assert(frNidbObjGetEntIdent (err, obj1, &eid));
  assert(frNidbObjGetType (err, obj1, &type));
  assert(frNidbObjGetServerParams (err, obj1, &sin_addr, &port, &status, &rating,
                                   &rcode,
                                   &region, &servoperator));
  assert(frNidbObjFree (err, &obj1));

  zkDebug("type=%u name=%s eid=%u type=%u ip=%s:%hu s/r=%u/%u",
          entity.type, entity.name,
          eid, type, zkInet_ntoa(sin_addr), port, status, rating );
  zkDebug("    operator=%s region=%s/%s", servoperator, rcode, region);
  zkFree(region);
  assert(frNidbObjGetEntity (err, obj2, &entity));
  assert(frNidbObjGetEntIdent (err, obj2, &eid));
  assert(frNidbObjGetType (err, obj2, &type));
  assert(frNidbObjGetAipParams (err, obj2, &sin_addr, &p1, &p2,
                                &status, &rating, &rcode, &region,
                                &servoperator, &num));
  zkDebug("type=%u name=%s eid=%u type=%u ip=%s:%hu:%hu s/r=%u/%u neigh=%u",
          entity.type, entity.name,
          eid, type, zkInet_ntoa(sin_addr), p1, p2,
          status, rating, num);
  zkDebug("    operator=%s region=%s/%s", servoperator, rcode, region);
  zkFree(region);
  for (idx = 0 ; idx < num; idx ++) {
    assert(frNidbObjGetAipNeighbour (err, obj2, idx, &eid));
    zkDebug("eid=%u", eid);
  } 
  assert(frNidbObjFree (err, &obj2));

  assert( testNidbObjList( err ) );

  return EXIT_SUCCESS;

}
