
/*
 * $Id: frNiqsKeyQry.c,v 1.11 2001/04/19 20:29:26 alainc Exp $
 */

static const char * module_id_ = "$Id: frNiqsKeyQry.c,v 1.11 2001/04/19 20:29:26 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"
#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "frEnt.h"
#include "ZKIP.h"
#include "zkipClient.h"
#include "zkList.h"
#include "frNiqs.h"
#include "frSec.h"
#include "frNidbObj.h"
#include "frDBOTypes.h"
#include "zkNetinetIn.h"
#include "zkArpaInet.h"



/*
 * This only returns the FIRST key query host.  For now, we're only supposed to
 * have 1.  This function will strip the signatures from received buffers and
 * assume them to be correct (if you're calling this function then you dont
 * know where to get public keys anyways ;-)
 */

DLLExport int
frNiqsGetKeyQryAddr (
  ZkError err ,
  ZkipClient client ,
  struct sockaddr * addr )
{
  static const char * func = "frNiqsGetKeyQryAddr";
  int return_code = 0;
  void * buf = NULL;    /* Response buffer pointer */
  size_t sz = 0;        /* Size of the buffer */
  ZkList list = NULL;   /* List of key query hosts */
  struct sockaddr_in * addr_in = (struct sockaddr_in *) addr;

  FrEntIdType entid = FRENT_TYPE_KEYQRY_SRV;

  void *raw_buf = NULL;
  size_t raw_bsz = 0;

  (void) module_id_; /* Get rid of compile warning */


  if (client && addr) {
    memset(addr,0,sizeof(struct sockaddr_in));

    if(frNiqsMxQuery (err, client, FRNIQ_ATTR_DESCRIPTION, 0, 
                      ZKIP_ENCODING_BZIP2,
                      &entid, 1, &raw_buf, &raw_bsz) &&
       frSecSigRemoveInSitu(err,raw_buf,&raw_bsz)    &&
       frNiqsUncompress(err, (const unsigned char *)raw_buf, (unsigned char **)&buf, &sz)){


        FrNidbObj obj = NULL;
        FrNidbObjType type;

        FrNidbObj listObj;

        struct in_addr sin_addr;
        uint16_t port;

        /* The following are not used */
        uint32_t status, rating;
        const char *region_code = NULL;
        char *region = NULL;
        const char *servop = NULL;


        if ( frNidbObjUnpack (err, buf, sz, &obj) ) {
          if ( frNidbObjGetType (err, obj, &type) ) {
            if ( type != FRNIDBOBJ_LIST_RESP ) {
              zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKERROR,
                            "Improper reponse type (%u) from server", type);

            } else if ( ! frNidbObjGetListRespAtIdx(err,obj, 0, &listObj)) {
                zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKERROR,
                            "Can not position pointer in list a position 0");
            } else if ( frNidbObjGetServerParams(err, listObj, &sin_addr, &port,
                                                 &status, &rating, &region_code,
                                                 &region, &servop)) {
              /*
               * Convert the buffer to a description structure and fill the 
               * sockaddr_in structure.
               */
              
              addr_in->sin_family = AF_INET;
              addr_in->sin_port = htons(port);
              addr_in->sin_addr = sin_addr;
              
              return_code = 1;
            }
          }
        }
        if (region) {
          zkFree(region);
        }
        if (obj) {
          frNidbObjListFree(err, &obj);
        }
    }
    /*
     * Cleanup 
     */
    if (list) {
      zkListFree(err,&list);
    }
    if (buf) {
      zkFree(buf);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}


/*
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
**
** PATCH PATCH PATCH
** 
** from here to end of file
**
** this function has bee nre-introduce to allow the NEW client to talk with the
** OLD NIQS in case something goes wrong in the migration and a roll back in
** needed.
**
** LBe 2000.11.28
**------------------------------------------------------------------------------
**------------------------------------------------------------------------------
*/


/*
 * This only returns the FIRST key query host.  For now, we're only supposed to
 * have 1.  This function will strip the signatures from received buffers and
 * assume them to be correct (if you're calling this function then you dont
 * know where to get public keys anyways ;-)
 */

DLLExport int
OLD_frNiqsGetKeyQryAddr (
  ZkError err ,
  ZkipClient client ,
  struct sockaddr * addr )
{
  static const char * func = "OLD_frNiqsGetKeyQryAddr";
  int return_code = 0;
  void * buf = NULL;    /* Response buffer pointer */
  size_t sz = 0;        /* Size of the buffer */
  ZkList list = NULL;   /* List of key query hosts */
  struct sockaddr_in * addr_in = (struct sockaddr_in *) addr;
  FrNiqsEntity * list_ent = NULL;
  FrEntIdType entid = FRENT_TYPE_KEYQRY_SRV;

  (void) module_id_; /* Get rid of compile warning */

  if (client && addr) {
    memset(addr,0,sizeof(struct sockaddr_in));
    if (frNiqsList(err,client,0,&entid,1,&buf,&sz) &&
        frSecSigRemoveInSitu(err,buf,&sz) &&
        frNiqsListFromBuffer(err,buf,sz,&list) &&
        zkListMoveCursor(err,list,LIST_HEAD) &&
        zkListGet(err,list,(void **)&list_ent,LIST_STAY) &&
        list_ent) {
      zkFree(buf);
      buf = NULL; /* To avoid problems if frNiqsQuery fails */
      if (frNiqsQuery(err,client,FRNIQ_ATTR_DESCRIPTION,&(list_ent->entity),
                      &buf,&sz) &&
          frSecSigRemoveInSitu(err,buf,&sz)) {
        /*
         * Convert the buffer into a usuable structure
         * Use the Nidb Object API to extract the information
         * from it.
         */
        FrNidbObj obj = NULL;
        FrNidbObjType type;
        struct in_addr sin_addr;
        uint16_t port;
        /* The following are not used */
        uint32_t status, rating;
        const char *region_code = NULL;
        char *region = NULL;
        const char *servop = NULL;

        if ( frNidbObjUnpack (err, buf, sz, &obj) ) {
          if ( frNidbObjGetType (err, obj, &type) ) {
            if ( type != FRNIDBOBJ_SERVER ) {
              zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKERROR,
                            "Improper reponse type (%u) from server", type);
            } else if ( frNidbObjGetServerParams(err, obj, &sin_addr, &port,
                                                 &status, &rating, &region_code,
                                                 &region, &servop)) {
              /*
               * Convert the buffer to a description structure and fill the 
               * sockaddr_in structure.
               */
              
              addr_in->sin_family = AF_INET;
              addr_in->sin_port = htons(port);
              addr_in->sin_addr = sin_addr;
              
              return_code = 1;
            }
          }
        }
        if (region) {
          zkFree(region);
        }
        if (obj) {
          zkFree(obj);
        }
      }
    }

    /*
     * Cleanup 
     */
    if (list) {
      zkListFree(err,&list);
    }
    if (buf) {
      zkFree(buf);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
