/*
 * $Id: frNiqsTokenInfo.c,v 1.10 2001/04/19 20:29:26 alainc Exp $
 */

static const char * module_id_ = "$Id: frNiqsTokenInfo.c,v 1.10 2001/04/19 20:29:26 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"
#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "frEnt.h"
#include "ZKIP.h"
#include "zkipClient.h"
#include "zkList.h"
#include "frNiqs.h"
#include "frNiqsTokenInfo.h"
#include "zkConfig.h"
#include "frCrypt.h"
#include "frSec.h"
#include "frNidbObj.h"
#include "frDBOTypes.h"
#include "zkNetinetIn.h"
#include "zkArpaInet.h"


/*
 *
 */

DLLExport int
frNiqsGetTokenInfo (
  ZkError err, 
  ZkipClient client ,
  FrKeyQryHost kh ,
  FrKeyQry kq ,
  FrTokenType * tt )
{
  static const char * func = "frNiqsGetTokenInfo";
  int return_code = 0;
  FrEntIdExp signer;
  FrCryptPubKey sigkey = NULL;
  unsigned int kv = 0; /* Key version */
  void * buf = NULL; /* The token type description buffer */
  size_t sz = 0; /* Size of the buffer */
  int valid = 0;

#if defined(WIN32)
# pragma message(Reminder "Should not hard code the timeout and retry values")
#elif !defined(macintosh)
# warning "Should not hard code the timeout and retry values"
#endif

  int retries = 5;
  int timeout = 3;

  (void) module_id_; /* Get rid of compile warning */

  if (client && kq && tt) {
    if (frNiqsTokenInfo(err,client,&buf,&sz) &&
        frSecSigGetEntity(err,buf,sz,&signer) &&
        frSecSigGetVersion(err,buf,sz,&kv) &&
        (signer.type == FRENT_TYPE_NIQS) &&
        frKeyQryFetchPubKey(err,kq,kv,FRCRYPT_KEYTYPE_SIGNING,
                            signer,kh,retries,timeout,&sigkey) &&
        frSecSigVerify(err,sigkey,buf,sz,&valid) &&
        (valid) &&
        frSecSigRemoveInSitu(err,buf,&sz) &&
        frTokenTypeFromBinary(err,tt,buf,sz)) {
      return_code = 1;
    }

    /*
     * Cleanup
     */

    if (buf) {
      zkFree(buf);
    }

  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}
