/*
 * $Id: frNym.c,v 1.40 2001/04/19 20:29:37 alainc Exp $
 */

static const char * module_id_ = "$Id: frNym.c,v 1.40 2001/04/19 20:29:37 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkCtype.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkBase64.h"
#include "zkDateTime.h"
#include "zkAlloc.h"
#include "frCrypt.h"
#include "frNym.h"
#include "frNymUtil.h"
#include "frToken.h"
#include "frTokenType.h"

struct T_NymParm {
  enum FrRemoteNymParameter parmid;
  uint32_t flags;
  size_t length;
  unsigned char * data;
  struct T_NymParm * link;
};

typedef struct T_NymParm * NymParm;

struct T_FrRemoteNym {
  FrCryptSig prvsig;  /* For signing the nym binary data */
  int parameters;
  NymParm parameter;
  NymParm last_lookup;
};



static const struct T_NymParmData {
  uint32_t flags;
  enum FrRemoteNymParameter id;
  const char * name;
} NymParmData[] = {
  { (FL_DATA_ASCII), NymVersion , "VERSION" },
  { (FL_NYMSERVER | FL_DATA_ASCII|FL_PRIVATE), NymToken , "TOKEN" },
/*
 * Reply blocks used to be (and should probably have stayed)
 * FL_PRIVATE
 */

  { (FL_DATA_BINARY ), NymReplyBlock , "REPLYBLOCK" },
  { (FL_NYMSERVER | FL_DATA_ASCII), NymName , "NAME" },
  { (FL_DATA_UTF8), NymLongName , "LONGNAME" },
  { (FL_DATA_BINARY), NymEncryptionKey , "ENCRYPTION-KEY" },
  { (FL_DATA_BINARY), NymSigningKey , "SIGNING-KEY" },
  { (FL_DATA_BINARY), NymPhoto , "PHOTO" },
  { (FL_DATA_UTF8 | FL_PRIVATE), NymCapability , "CAPABILITY" },
  { (FL_DATA_BINARY), NymSound , "SOUND" },
  { (FL_DATA_UTF8), NymNote , "NOTE" },  /* Do not use this anymore */
  { (FL_DATA_BINARY), NymSpamControl, "SPAM-CONTROL" },
  { (FL_CLIENT_ONLY|FL_DATA_BINARY), NymPrivateNote , "PRIVATE-NOTE" },
  { (FL_DATA_BINARY), NymPublicNote , "PUBLIC-NOTE" },
  { (FL_DATA_ASCII), NymLanguage , "LANGUAGE" },
  { (FL_DATA_UTF8), NymExtension , "EXTENSION" },
  { (FL_DATA_UTF8), NymDescription , "DESCRIPTION" },
  { (FL_DATA_BINARY|FL_LIST), NymBlockList, "BLOCKLIST" },
  { (FL_DATA_BINARY | FL_CLIENT_ONLY) , NymPrivateReplyBlock, "PRIVATE-REPLY-BLOCK" },
  { (FL_CLIENT_ONLY | FL_DATA_ASCII) , NymPassphrase , "PASSPHRASE" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY|FL_LIST) , NymScanWord , "SCANWORD" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY|FL_LIST) , NymFriends , "FRIENDS" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymPreferences, "PREFERENCES" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymPrivateEmailAddr , "PRIVATE-EMAIL" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymRules , "RULES" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymNetworkPrefs, "NETWORK-PREFS"},
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymPorts , "PORTS" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymTokenCache , "TOKEN-CACHE" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymPrivateEncryptionKey, "PRIVATE-ENCRYPTION-KEY" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymPrivateSigningKey, "PRIVATE-SIGNING-KEY" },
  { (FL_CLIENT_ONLY | FL_DATA_BINARY) , NymSecuritySettings, "SECURITY-SETTINGS" },

  { (FL_NYMSERVER|FL_DATA_ASCII | FL_PRIVATE), NymTimeStamp ,"DTSTAMP" },
  { (FL_NYMSERVER|FL_DATA_ASCII|FL_PRIVATE), NymLastModified, "LAST-MODIFIED" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_ASCII|FL_PRIVATE),NymExpiry,"EXPIRY"},
  { (FL_NYMSERVER|FL_DATA_ASCII|FL_PRIVATE),NymSmooshed,"SMOOSHED"},
  { (FL_NYMSERVER|FL_DATA_ASCII|FL_PRIVATE), NymDisabled, "DISABLED" },
  { (FL_NYMSERVER|FL_DATA_ASCII|FL_PRIVATE), NymNuked, "NUKED" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_BINARY),NymType,"TYPE"},
  { (FL_NYMSERVER|FL_DATA_BINARY|FL_PRIVATE), NymSignature, "SIGNATURE" },
  { (FL_NYMSERVER|FL_TOKEN|FL_PRIVATE), NymRelease, "RELEASE" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_ASCII) , NymAccessLevel, "ACCESS-LEVEL" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_ASCII), NymEmailDomain , "EMAIL-DOMAIN" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_ASCII), NymCrossPostLimit , "CROSSPOST-LIMIT" },
  { (FL_NYMSERVER|FL_TOKEN|FL_DATA_ASCII), NymEmailLimit , "EMAIL-LIMIT" },
  { (FL_PRIVATE), NymUnknown, "UNKNOWN" }
};


#define NYMPARMLABELS (sizeof(NymParmData) / sizeof(struct T_NymParmData))

/**
 *  This function merges some of the components of one nym structure into
 *  another.  The destination nym is first stripped so that the  identified
 *  parameters from the source nym replace matching ones in the destination.
 *  WARNING:  do not call this function outside the nym server, if you call
 *  it in any other context, well, dont come crying to me.
 *  -Philippe
 */

DLLExport int
frRemoteNymReplaceParameters (
  ZkError err ,
  FrRemoteNym src ,
  FrRemoteNym dst ,
  uint32_t flags )
{
  NymParm parm = NULL;
  const char * func = "frRemoteNymReplaceParameters";

  (void) module_id_; /* Gets rid of compile time warning */
  zkErrorClear(err);
  if (src && dst) {
    if (!frRemoteNymStrip(err,dst,flags)) {
      return 0;
    }
    parm = src->parameter;
    while (parm) {
      if (parm->flags & flags) {
        if (!frRemoteNymAddParameter(err,dst,parm->parmid,parm->data,
                                     parm->length)) { 
          return 0;
        }
      }
      parm = parm->link;
    }  
    return 1;
  }  
  zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
} 

/*
 * Internal function prototypes
 */ 

static const char * remoteNymParameterToken ( enum FrRemoteNymParameter );
enum FrRemoteNymParameter remoteNymParameterId ( char * );

static const char * 
remoteNymParameterToken (
  enum FrRemoteNymParameter id )
{
  int i = 0; /* Loop index */

  for (i = 0; i < (int) NYMPARMLABELS; i++) {
    if (id == NymParmData[i].id) {
      return NymParmData[i].name;
    }
  }
  return "UNKNOWN";
}

enum FrRemoteNymParameter
remoteNymParameterId (
  char * token )
{
  int i = 0;
  
  for (i = 0; i < (int) NYMPARMLABELS; i++) {
    if (NymParmData[i].name[0] == toupper(token[0])) {
      if (strcasecmp(NymParmData[i].name,token) == 0) {
        return NymParmData[i].id;
      }
    }
  }
  return NymUnknown;
}

/*
 *
 */

DLLExport int
frRemoteNymParameterToToken (
  ZkError err ,
  enum FrRemoteNymParameter id ,
  const char ** token )
{
  static const char * func = "frRemoteNymParameterToToken";
  int return_code = 0;
  int i = 0;

  zkErrorClear(err);
  if (token) {
    *token = NULL;
    for (i = 0; i < (int) NYMPARMLABELS; i++) {
      if (id == NymParmData[i].id) {
        *token = NymParmData[i].name;
        return_code = 1;
        break;
      }
    }
    if (token == NULL) {
      zkErrorSet_va(err,func,ZKERR,ZKMRNYM,ZKENOTFOUND,
                    "Nym parameter 0x%X not registered",id);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

DLLExport int
frRemoteNymTokenToParameter (
  ZkError err ,
  const char * token ,
  enum FrRemoteNymParameter * id ) 
{
  static const char * func = "frRemoteNymTokenToParameter";
  int return_code = 0;
  int i = 0;

  if (token && id) {
    for (i = 0; i < (int) NYMPARMLABELS; i++) {
      if (NymParmData[i].name[0] == toupper(token[0])) {
        if (strcasecmp(NymParmData[i].name,token) == 0) {
          *id = NymParmData[i].id;
          return_code = 1;
          break;
        }
      }
    }
    if (*id == NymUnknown) {
      zkErrorSet_va(err,func,ZKERR,ZKMRNYM,ZKENOTFOUND,
                    "Nym parameter %s not registered",token);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport uint32_t
frRemoteNymParameterFlags (
  enum FrRemoteNymParameter id )
{
  int i = 0; /* Loop index */

  for (i = 0; i < (int) NYMPARMLABELS; i++) {
    if (id == NymParmData[i].id) {
      return NymParmData[i].flags;
    }
  }
  return 0;
}
  
/**
 *  Create a new structure for a Remote Nym.  All new remote nym structures
 *  will automatically be assigned the current nym format version (1.0)
 * 
 *  @param err Returns the error if one occured
 *  @param nym Returns the new remote nym structure
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 *  @author Philippe Boucher
 */

DLLExport int 
frRemoteNymNew (
  ZkError err ,
  FrRemoteNym * nym )
{
  zkErrorClear(err);
  if (nym == NULL) {
    zkErrorSet(err,"frRemoteNymNew",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }
  *nym = (FrRemoteNym) zkMalloc(sizeof(struct T_FrRemoteNym));
  if ((*nym) == NULL) {
    zkErrorSet(err,"frRemoteNymNew",ZKERR,ZKMRNYM,ZKENOMEM,NULL);
    return 0;
  }
  memset(*nym,0,sizeof(struct T_FrRemoteNym));
  (*nym)->prvsig = NULL;
  (*nym)->parameters = 0;
  (*nym)->parameter = NULL;
  (*nym)->last_lookup = NULL;
  if (!frRemoteNymAddParameter(err,*nym,NymVersion,
                               (const unsigned char *) "1.0",strlen("1.0"))) {
    return 0;
  }
  return 1;
}

/**
 *  Destroy the remote nym structure.  All associated bits of memory will be
 *  set to 0 before being freed.  If you have set a privatesignature key for
 *  the nym, that will be destroyed as well.  If you wish to protect it, you
 *  must set the pointer to NULL using 
 *  frRemoteNymSetPrivateSignatureKey(err,nym,NULL).
 *
 *  @param err Returns the error if one occured
 *  @param nym The Remote nym to destroy (the pointer is reset to NULL)
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int 
frRemoteNymFree (
  ZkError err ,
  FrRemoteNym * nym )
{
  NymParm link = NULL;
  zkErrorClear(err);
  if (nym) {
    if (*nym) {
      if ((*nym)->prvsig) {
        if (!frCryptSigFree(err,&((*nym)->prvsig))) {
          return 0;
        }
      }
      if ((*nym)->parameter) {
        link = (*nym)->parameter;
        while (link) {
          (*nym)->parameter = link->link;
          if (link->data) {
            memset(link->data,0,link->length);
            zkFree(link->data);
          }
          memset(link,0,sizeof(struct T_NymParm)); 
          zkFree(link);
          link = (*nym)->parameter; 
        }
      }
      memset(*nym,0,sizeof(struct T_FrRemoteNym));
      zkFree(*nym);
      *nym = NULL;
      return 1;
    }
  }
  zkErrorSet(err,"frRemoteNymFree",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
}

/**
 * Set the nym's private signature key.  This is used to digitally sign the
 * binary data sent to the nym server.  This key never actually leaves the 
 * client.
 */

DLLExport int
frRemoteNymSetPrivateSignatureKey (
  ZkError err ,
  FrRemoteNym nym ,
  FrCryptSig prvsig )
{
  zkErrorClear(err);
  if (nym) {
    nym->prvsig = prvsig;
    return 1;
  }
  zkErrorSet(err,"frRemoteNymSetPrivateSignatureKey",ZKERR,ZKMRNYM,ZKENULLPTR,
             NULL);
  return 0;
}

DLLExport int
frRemoteNymPrivateSignatureKey (
  ZkError err, 
  FrRemoteNym nym ,
  FrCryptSig * prvsig )
{
  zkErrorClear(err);
  if (nym && prvsig) {
    *prvsig = nym->prvsig;
    return 1;
  }
  zkErrorSet(err,"frRemoteNymPrivateSignatureKey",ZKERR,ZKMRNYM,ZKENULLPTR,
             NULL);
  return 0;
}

/**
 *  Remove fields from the remote nym which are unrecognized or only the
 *  nym server is allowed to add.  You do not need to call this function
 *  if you are client. 
 */

DLLExport int
frRemoteNymStrip (
  ZkError err ,
  FrRemoteNym nym ,
  uint32_t flags )
{
  NymParm parm = NULL;
  NymParm * previous = NULL;

  zkErrorClear(err);
  if (nym == NULL) {
    zkErrorSet(err,"frRemoteNymStrip",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }
  parm = nym->parameter;
  previous = &(nym->parameter);
  while (parm) {
    if ((parm->flags & flags) || (parm->parmid == NymUnknown)) {
      *previous = parm->link;
      parm->link = NULL;
      memset(parm->data,0,parm->length);
      zkFree(parm->data);
      memset(parm,0,sizeof(struct T_NymParm));
      zkFree(parm);
      parm = *previous;
      nym->parameters -= 1;
    } else {
      previous = &(parm->link);
      parm = parm->link;
    }
  }
  return 1;
}

DLLExport int
frRemoteNymDeleteParameter (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parmid )
{
  static const char * func = "frRemoteNymDeleteParameter";
  int return_code = 0;
  NymParm parm = NULL;
  NymParm * previous = NULL;

  zkErrorClear(err);
  if (nym) {
    parm = nym->parameter;
    previous = &(nym->parameter);
    while (parm) {
      if (parm->parmid == parmid) {
        *previous = parm->link;
        parm->link = NULL;
        memset(parm->data,0,parm->length);
        zkFree(parm->data);
        memset(parm,0,sizeof(struct T_NymParm));
        zkFree(parm);
        parm = *previous;
        nym->parameters -= 1;
      } else {
        previous = &(parm->link);
        parm = parm->link;
      }
    }
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"Null parameter");
  }
  return return_code;
}

/*
 *
 */

DLLExport int
frRemoteNymDeleteParameterPtr (
  ZkError err ,
  FrRemoteNym nym ,
  void ** ptr )
{
  static const char * func = "frRemoteNymDeleteParameterPtr";
  int return_code = 0;
  NymParm parm = NULL;
  NymParm * previous = NULL;

  zkErrorClear(err);
  if (nym && ptr) {
    parm = nym->parameter;
    previous = &(nym->parameter);
    while (parm) {
      if (parm->data == (unsigned char *) (*ptr)) {
        *previous = parm->link;
        parm->link = NULL;
        memset(parm->data,0,parm->length);
        zkFree(parm->data);
        memset(parm,0,sizeof(struct T_NymParm));
        zkFree(parm);
        parm = *previous;
        nym->parameters -= 1;
        *ptr = NULL;
        return_code = 1;
        break;
      } else {
        previous = &(parm->link);
        parm = parm->link;
      }
    }
    if (return_code == 0) {
      zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENOTFOUND,
                 "Data pointer is not contained in FrRemoteNym");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

/** 
 *  Create a RemoteNym structure from a binary datablock.  These blocks are
 *  returned by the nym servers.  They are the on the wire format of a 
 *  remote nym.
 * 
 *  @param err Returns the error if one occured
 *  @param nym Returns the new nym, built from the binary data block
 *  @param wire The binary data which makes up the nym
 *  @param size The number of bytes in the block.
 *  @param verify A signature key used to verify the nym (if set to NULL,
                  no verification will be done)
 *  @return A non-zero value on success, a 0 on failure
 *  @version 1.0
 *  @author Philippe Boucher
 */

DLLExport int
frRemoteNymFromBinary (
  ZkError err ,
  FrRemoteNym * nym ,
  FrCryptSig verify ,
  const unsigned char * wire ,
  size_t size )
{
  enum FrRemoteNymParameter parmid; /* ID code of a parameter */
  size_t offset = 0; /* Used to examine the wire format binary buffer */
  size_t psize = 0;  /* Size of a parameter */
  unsigned int version = 0; /* Binary format version (should be 1) */
  unsigned int parameters = 0; /* Number of parameters in the nym */
  unsigned long tsize = 0; /* Total size of the binary nym */
  const unsigned char * ptr = NULL; /* Moves withing the wire data */

  zkErrorClear(err);
  if ((nym == NULL) || (wire == NULL)) {
    zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }

  /*
   * Do we have enough bytes to get a header?
   */

  if (size < 8) {
    zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKEINCOMPLETE,
               "Incomplete binary nym header");
    return 0;
  }
  
  /*
   * Create the new nym structure.  I dont use the frRemoteNymNew() function
   * because all of the nym contents come straight from the binary block.
   * the frRemoteNymNew() function adds some parameters to the nym which
   * will already be present in the binary block.
   */
  
  *nym = (FrRemoteNym) zkMalloc(sizeof(struct T_FrRemoteNym));
  if ((*nym) == NULL) {
    zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKENOMEM,NULL);
    return 0;
  }
  memset(*nym,0,sizeof(struct T_FrRemoteNym));
  (*nym)->parameter = NULL;
  (*nym)->parameters = 0;
  (*nym)->last_lookup = NULL;
  
  /*
   * Get the header information
   */

  tsize = (wire[0]*0x1000000) + (wire[1]*0x10000) + (wire[2]*0x100) + wire[3];
  version = (wire[4]*0x100) + wire[5];
  parameters = (wire[6]*0x100) + wire[7];

  if (tsize != size) {
    zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKEINCOMPLETE,
               "Buffer size and header size do not match");
    return 0;
  }

  /*
   * Add the nym parameters
   */

  offset = 8;
  ptr = wire;
  while (parameters--)  {
    if (offset >= (tsize - 4)) {
      zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKEINCOMPLETE,
                 "Last parameter header is incomplete");
      frRemoteNymFree(NULL,nym);
      return 0;
    }
    parmid = (enum FrRemoteNymParameter) ((ptr[offset]*0x100) + ptr[offset + 1]);
    psize = (ptr[offset + 2]*0x100) + ptr[offset + 3];
    offset += 4;
    if ((offset + psize) > tsize) {
      zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKEINCOMPLETE,
                 "Not enough data in the binary block to complete the nym");
      frRemoteNymFree(NULL,nym);
      return 0;
    }
    if (!frRemoteNymAddParameter(err,*nym,parmid,&(ptr[offset]),psize)) {
      return 0;
    }
    offset += psize; 
  }

  /*
   * Verify the nym....
   */

  if (verify) {
    const unsigned char * signature = NULL;
    size_t siglen = 0;
    int valid = 0;

    if (!frRemoteNymGetParameter(err,*nym,NymSignature,&signature,&siglen)) {
      zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKENOTFOUND,
                 "Binary nym data did not contain a signature");
      frRemoteNymFree(NULL,nym);
      return 0;
    }
    if (!frCryptSigVerifyData(err,verify,(void *)wire,size - FRCRYPT_SIG_SIZE,
                              (void *) signature,siglen,&valid)) {
      frRemoteNymFree(NULL,nym);
      return 0;
    } 
    if (!valid) {
      zkErrorSet(err,"frRemoteNymFromBinary",ZKERR,ZKMRNYM,ZKESIGCK,
                 "Signature verification failed");
      frRemoteNymFree(NULL,nym);
      return 0;
    }
  }

  return 1;
}


/**
 *  Create the on the wire binary representation of the nym from the 
 *  FrRemoteNym structure.
 * 
 *  @param err Returns the error if any occured
 *  @param nym The nym to convert to binary form
 *  @param wire Allocates and returns a binary representation for the nym
 *  @param size Returns the number of bytes in the binary representation
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 *
 * release the allocated memory with frRemoteNymBinaryFree
 */

DLLExport int
frRemoteNymToBinary (
  ZkError err ,
  FrRemoteNym nym ,
  unsigned char ** wire ,
  size_t * size )
{
  unsigned int i = 0; /* Loop index */
  NymParm nymparm = NULL;
  unsigned char * ptr = NULL;
  int parmcount = 0;

  zkErrorClear(err);
  if ((wire == NULL) || (nym == NULL) || (size == NULL)) {
    zkErrorSet(err,"frRemoteNymToBinary",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }

  /* 
   * This is 4 bytes for size of the nym, 2 bytes for version code,  and,
   * 2 bytes for the number of parameters in the nym.  Then we loop through
   * the nym contents to find out how big the rest of it has to be.
   * Each parameter entry consists of two bytes for the parameter ID, 2 bytes
   * for the parameter size and X bytes for the parameter value.
   * All integers are in network byte order.
   */
  
  *size = 8;
  if (nym->prvsig) {
    *size += FRCRYPT_SIG_SIZE + 4;
  }
  nymparm = nym->parameter;
  while (nymparm) {
    parmcount += 1; 
    *size += (4 + nymparm->length);
    nymparm = nymparm->link;
  }

  /*
   * Make the binary buffer (add an extra byte for a NUL terminator, however
   * this does not count as part of the data)
   */

  if (!zkMemAlloc(err, (void **) wire, *size + 1)) {
    return 0;
  }
  ptr = *wire;

  /*
   * Store the size of the nym
   */

  ptr[0] = (*size & 0xFF000000) >> 24;
  ptr[1] = (*size & 0xFF0000) >> 16;
  ptr[2] = (*size & 0xFF00) >> 8;
  ptr[3] = (*size & 0xFF);

  /*
   * Store the version number of the nym
   */

  ptr[4] = 0;
  ptr[5] = 1;

  /*
   * Store the number of parameters
   */

  if (nym->prvsig) {
    ptr[6] = ((parmcount + 1) & 0xFF00) >> 8;
    ptr[7] = ((parmcount + 1) & 0xFF);
  } else { 
    ptr[6] = (parmcount & 0xFF00) >> 8;
    ptr[7] = (parmcount & 0xFF);
  }

  /*
   * Now loop through and add all of the nym parameters
   */

  ptr = &(ptr[8]);
  nymparm = nym->parameter;
  while (nymparm) {
    ptr[0] = nymparm->parmid / 0x100;
    ptr[1] = nymparm->parmid % 0x100;
    ptr[2] = (nymparm->length & 0xFF00) >> 8;
    ptr[3] = (nymparm->length & 0xFF);
    ptr = &(ptr[4]);
    for (i = 0; i < nymparm->length; i++) {
      *ptr++ = nymparm->data[i];
    }
    nymparm = nymparm->link;
  }

  /*
   * If the nym has a private signature key specified, sign the data
   */

  if (nym->prvsig) {
    ptr[0] = NymSignature / 0x100;
    ptr[1] = NymSignature % 0x100;
    ptr[2] = FRCRYPT_SIG_SIZE / 0x100;
    ptr[3] = FRCRYPT_SIG_SIZE % 0x100;
    if (!frCryptSigSignData(err,nym->prvsig,*wire,*size - FRCRYPT_SIG_SIZE,
                            &((*wire)[*size - FRCRYPT_SIG_SIZE]),
                            FRCRYPT_SIG_SIZE)) {
      zkMemFree(NULL, (void **) wire);
      *size = 0;
      return 0;
    }
    ptr += 4 + FRCRYPT_SIG_SIZE;
  }
  *ptr = 0;

  return 1;
}

/* This function is called to release the memory allocated by 
 * frRemoteNymToBinary.
 */

DLLExport int
frRemoteNymBinaryFree (
  ZkError err ,
  unsigned char ** wire)
{
  return zkMemFree(err, (void **) wire);
}


DLLExport int 
frRemoteNymTemplateDump (
  ZkError err ,
  FrRemoteNym nym ,
  const char * template ,
  ZkDumpCallback callback ,
  void * cdata )
{
#define PARAMETER_SZ 64
  const unsigned char * parmdata = NULL;
  size_t parmdatasz = 0;
  char * base64 = NULL;
  int base64sz = 0;
  enum FrRemoteNymParameter parmid;
  enum FrRemoteNymParameter last_parmid = NymUnknown;
  int width = 0; /* Width of display for binary output */
  int ndx = 0;
  int n = 0;
  int a = 0; /* Loop index for stripping whitespace at start of nymparm name */
  int lolipop = 1;
  char parameter[PARAMETER_SZ + 1];
  const char * start = NULL;
  char * display_type = NULL;
  int display = FL_DATA_ASCII;

  zkErrorClear(err);
  if (nym && template && callback) {
    start = template;
    while (lolipop) {
      switch (start[ndx]) {
      case (char) 0:
        lolipop = 0;
        break;
      case '\\':
        if (ndx > 0) {
          if (!callback(err,cdata,start,ndx)) {
            return 0;
          }
        }
        start = &(start[ndx + 1]);
        ndx = 0;
        if (*start == (char) 0) {
          lolipop = 0;
        }
        ndx++;
        break;
      case '$':
        if (ndx > 0) {
          if (!callback(err,cdata,start,ndx)) {
            return 0;
          }
        }
        n = 0;
        ndx++;
        while (start[ndx + n]) {
          if (start[ndx + n] == ';') {
            ndx++;
            break;
          }
          parameter[n] = start[ndx + n];
          if (++n >= PARAMETER_SZ) {
            break;
          }
        }
        start = &(start[ndx + n]);

        /* 
         * How do we want the parameter displayed
         */

        parameter[n] = (char) 0;
        display = FL_DATA_ASCII;
        if ((display_type = strrchr(parameter,':')) != NULL) {
          n = display_type - parameter;
          display_type++;
          switch (*display_type) {
          case 'b':
          case 'B':
            display = FL_DATA_BINARY;
            display_type++;
            width = 0;
            while (isdigit(*display_type)) {
              width = width * 10 + *display_type++ - '0';
            }
            break;
          }
        }

        /*
         * Clean out the whitespace from the parameter name 
         */

        while (isspace(parameter[--n]))
        	;
        parameter[n + 1] = (char) 0;
        a = 0; while (isspace(parameter[a])) a++;

        /*
         * Replace the parameter name with its value in the template text
         */

        parmid = remoteNymParameterId(&(parameter[a]));
        if (parmid != last_parmid) {
          frRemoteNymResetParameterSearch(NULL,nym);
        }
        last_parmid = parmid; 

        if (frRemoteNymGetParameter(err,nym,parmid,&parmdata,&parmdatasz)) {
          if (display == FL_DATA_BINARY) {
            base64 = (char *) zkMalloc((parmdatasz * 4) / 3 + 5);
            if (base64 == NULL) {
              zkErrorSet(err,"frRemoteNymTemplateDump",ZKERR,ZKMRNYM,ZKENOMEM,
                         "malloc");
              return 0;
            }
            if (!zkConvertToBase64(err,parmdata,
                                   (unsigned char *) base64,parmdatasz)) {
              zkFree(base64);
              return 0;
            }
            base64sz = strlen(base64);
            if (width == 0) { 
              if (!callback(err,cdata,(const char *) base64,base64sz)) {
                zkFree(base64);
                return 0;
              }
            } else {
              for (a = 0; a < base64sz; a += width) {
                if (!callback(err,cdata,(const char *)&(base64[a]),
                              (base64sz-a)<width?(base64sz-a):width)) {
                  zkFree(base64);
                  return 0;
                }
                if (!callback(err,cdata," \015\012",3)) {
                  zkFree(base64);
                  return 0;
                }
              }
            }
            zkFree(base64);
          } else if (!callback(err,cdata,(const char *) parmdata,parmdatasz)) {
            return 0;
          }
        }
        ndx = 0;
        if (*start == (char) 0) {
          lolipop = 0;
        }
        break;
      default:
        ndx++;
        break;
      }
    }
    if (ndx > 0) {
      if (!callback(err,cdata,start,ndx)) {
        return 0;
      }
    }
    return 1;
  }
  zkErrorSet(err,"frRemoteNymCallbackDump",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
#undef PARAMETER_SZ
}


/**
 *  Output the remote nym to the specified callback.  This function will create
 *  a text version of the nym somewhat resembling a VCARD. This is mostly for
 *  debugging purposes, but can be used to spit out the nym for a finger 
 *  server.  The callback mechanism is the same as the one used for
 *  the ZkEmail... functions.
 * 
 *  @param err Returns the error if one occured
 *  @param nym The nym to output
 *  @param callback The dump callback to use
 *  @param cdata The extra data pointer used by the callback.
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 *  @author Philippe Boucher
 */

DLLExport int 
frRemoteNymVDump (
  ZkError err ,
  FrRemoteNym nym ,
  ZkDumpCallback callback ,
  void * cdata )
{
  uint32_t flags;  /* The remote nym parameter flags */
  NymParm parm = NULL;
  const  char * str = NULL;
  char * base64 = NULL;

#define DUMP_TO_CALLBACK(_str) \
  if ((str = (const char *) (_str)) == NULL) \
    str = (const char *) ""; \
  if (!callback(err,cdata,str,strlen((const char *) str))) return 0; 

  zkErrorClear(err);
  if ((callback == NULL) || (nym == NULL)) {
    zkErrorSet(err,"frRemoteNymVDump",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }
  parm = nym->parameter;
  DUMP_TO_CALLBACK("BEGIN:NYM\015\012");
  while (parm) {
    DUMP_TO_CALLBACK(remoteNymParameterToken(parm->parmid));
    DUMP_TO_CALLBACK(":");
    flags = frRemoteNymParameterFlags(parm->parmid);
    if (flags & FL_DATA_BINARY) {
      base64 = (char *) zkMalloc((parm->length * 4) / 3 + 5);
      if (base64 == NULL) {
        zkErrorSet(err,"frRemoteNymVDump",ZKERR,ZKMRNYM,ZKENOMEM,"malloc");
        return 0;
      }
      if (!zkConvertToBase64(err,(unsigned char *) parm->data,
                             (unsigned char *) base64,parm->length)) {
        zkFree(base64);
        return 0;
      }
      if (!callback(err,cdata,(const char *) base64,strlen(base64))) {
        zkFree(base64);
        return 0;
      }
      zkFree(base64);
    } else if (!callback(err,cdata,(const char *) parm->data,
                         parm->length)) {
      return 0;
    }
    DUMP_TO_CALLBACK("\015\012");  
    parm = parm->link;
  }
  DUMP_TO_CALLBACK("END:NYM\015\012");
  return 1;
}

/**
 *  A different output format, more suitable for displaying the nym as
 *  text.
 * 
 *  @param err Returns the error if one occured
 *  @param nym The nym to output
 *  @param callback The dump callback to use
 *  @param cdata The extra data pointer used by the callback.
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 *  @author Philippe Boucher
 */

DLLExport int 
frRemoteNymDump2 (
  ZkError err ,
  FrRemoteNym nym ,
  ZkDumpCallback callback ,
  void * cdata )
{
#define MESG_SZ 80
  uint32_t flags;  /* The remote nym parameter flags */
  NymParm parm = NULL;
  const char * str = NULL;
  char mesg[MESG_SZ] = "";

#define DUMP_TO_CALLBACK(_str) \
  if ((str = (const char *) (_str)) == NULL) \
    str = (const char *) ""; \
  if (!callback(err,cdata,str,strlen((const char *) str))) return 0; \

  zkErrorClear(err);
  if ((callback == NULL) || (nym == NULL)) {
    zkErrorSet(err,"frRemoteNymVDump",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }
  parm = nym->parameter;
  while (parm) {
    zkSnprintf(mesg,sizeof mesg,"Parameter: \"%s\" (ID:0x%04X) %d bytes\015\012",
            remoteNymParameterToken(parm->parmid),parm->parmid,parm->length);
    DUMP_TO_CALLBACK(mesg);
    flags = frRemoteNymParameterFlags(parm->parmid);
    if (flags & FL_DATA_BINARY) {
      if (!zkDumpBuffer(err,parm->data,parm->length,callback,cdata)) {
        return 0;
      }
    } else if (!callback(err,cdata,(const char *) parm->data,
                         parm->length)) {
      return 0;
    }
    DUMP_TO_CALLBACK("\015\012");  
    parm = parm->link;
  }
  return 1;
#undef MESG_SZ
}

/**
 *  Add a parameter to the remote nym.  The list of available parameters is
 *  in the frNym.h file.
 * 
 *  @param err Returns the error if one occured
 *  @param nym The remote nym to which to add the parameter
 *  @param parmid The parameter ID of the parameter you wish to add
 *  @param data The value of the nym parameter
 *  @param length The number of bytes in the parameter value
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int 
frRemoteNymAddParameter (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parmid ,
  const unsigned char * data ,
  size_t length )
{
  return frRemoteNymAddParameterPtr(err,nym,parmid,data,length,NULL);
}

/* 
 * Remove instances of the provided parameter, then add it
 *
 */

DLLExport int 
frRemoteNymReplaceParameter (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parmid ,
  const unsigned char * data ,
  size_t length )
{
  return    frRemoteNymDeleteParameter(err, nym, parmid)
         && frRemoteNymAddParameterPtr(err, nym, parmid, data, length, NULL);
}

/**
 *  Add a parameter to the remote nym.  The list of available parameters is
 *  in the frNym.h file.  This function will also return the pointer to the
 *  data inserted into the frNym.
 * 
 *  @param err Returns the error if one occured
 *  @param nym The remote nym to which to add the parameter
 *  @param parmid The parameter ID of the parameter you wish to add
 *  @param data The value of the nym parameter
 *  @param length The number of bytes in the parameter value
 *  @param ptr Returns a pointer to the data
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int 
frRemoteNymAddParameterPtr (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parmid ,
  const unsigned char * data ,
  size_t length ,
  void ** ptr)
{
  static const char * func = "frRemoteNymAddParameterPtr";
  NymParm nymparm = NULL;

  zkErrorClear(err);
  if (ptr) {
    *ptr = NULL;
  }
  if ((nym == NULL) || (data == NULL)) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }
  if (length < 1) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKEPARMBAD,
               "Length of the parameter value must be at least 1 bytes");
    return 0;
  }
  nymparm = (NymParm) zkMalloc(sizeof(struct T_NymParm));
  if (nymparm == NULL) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENOMEM,NULL);
    return 0;
  }
  memset(nymparm,0,sizeof(struct T_NymParm));
  nymparm->data = (unsigned char *) zkMalloc(length + 1);
  if (nymparm->data == NULL) {
    zkFree(nymparm);
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENOMEM,NULL);
    return 0;
  }
  memcpy(nymparm->data,data,length);
  nymparm->data[length] = (char) 0;
  nymparm->length = length;
  nymparm->parmid = parmid;
  nymparm->link = nym->parameter;
  nymparm->flags = frRemoteNymParameterFlags(parmid);
  nym->parameter = nymparm;
  nym->parameters += 1;
  if (ptr) {
    *ptr = nymparm->data;
  }
  return 1;
}

/*
 * Determine how many instances of a certain parameter are present in a nym
 *
 */

DLLExport int 
frRemoteNymCountParameter (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parm ,
  uint32_t * count )
{
  const char * const func = "frRemoteNymCountParameter";
  NymParm traverser = NULL;

  zkErrorClear(err);
  if ((nym == NULL) || (count == NULL)) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"NULL pointer");
    return 0;
  }

  traverser = nym->parameter;
  *count = 0;

  while (traverser) {
    if (traverser->parmid == parm) {
      (*count)++;
    } 
    traverser = traverser->link;
  }
  return 1;
}


/**
 *  Reset the nym so that you can again search for the first instance of a 
 *  particular parameter ID. 
 * 
 *  @param err Returns the error if one occured
 *  @param nym The Remote nym to reset
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */ 

DLLExport int
frRemoteNymResetParameterSearch (
  ZkError err ,
  FrRemoteNym nym )
{
  zkErrorClear(err);
  if (nym == NULL) {
    zkErrorSet(err,"zkRemoteNymResetParameterSearch",ZKERR,ZKMRNYM,ZKENULLPTR,
               NULL);
    return 0;
  }
  nym->last_lookup = NULL;
  return 1;
}

/**
 *  Retrieve a particular parameter from the nym.  If there is more than one
 *  of the desired parameter, you can make successive calls using the same
 *  parameter ID.  The function will fail with a ZKENOTFOUND error once there
 *  are no more parameters of that type to find.  You can reset the search
 *  system to find the first parameter by specifying a different parameter ID
 *  or calling the function frRemoteNymResetParameterSearch()
 *
 *  @param err Returns the error if one occured
 *  @param nym The Remote nym to search
 *  @param parm The parameter ID of the parameter you wish to find.
 *  @param data Returns a pointer to the parameter value (this is a pointer to
 *              a string inside the RemoteNym structure, not a copy
 *  @param length The number of characters in the parameter value
 */

DLLExport int 
frRemoteNymGetParameter (
  ZkError err ,
  FrRemoteNym nym ,
  enum FrRemoteNymParameter parm ,
  const unsigned char ** data ,
  size_t * length )
{
  NymParm lastlookup = NULL;
  NymParm start = NULL;

  zkErrorClear(err);
  if ((nym == NULL) || (data == NULL) || (length == NULL)) {
    zkErrorSet(err,"frRemoteNymGetParameter",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
    return 0;
  }

  lastlookup = nym->last_lookup;
  start = nym->parameter;
  if (lastlookup != NULL) {
    if (lastlookup->parmid == parm) {
      start = lastlookup->link;
    }
  }
  while (start) {
    if (start->parmid == parm) {
      *data = start->data;
      *length = start->length;
      nym->last_lookup = start;
      return 1;
    } 
    start = start->link;
  }
  zkErrorSet(err,"frRemoteNymGetParameter",ZKERR,ZKMRNYM,ZKENOTFOUND,
             "Parameter not found");
  return 0;
}

/**
 *  Basic callback for dumping the RemoteNym to a stdout, stderr, or other
 *  FILE * stream.
 * 
 *  @param err Returns the error if one occured
 *  @param cdata The stream FILE * to which to output the data buffer
 *  @param data The data buffer
 *  @param size The number of bytes in the buffer
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
frRemoteNymWrite (
  ZkError err ,
  void * cdata ,
  const unsigned char * data ,
  int size )
{
  zkErrorClear(err);
  if (cdata && data) {
    zkFwrite(data,size,1,(FILE *)cdata);
    zkFflush((FILE *)cdata);
  }
  return 1;
}




/*
 * Sanity check functions.
 * These functions will analyze a nym struture to see if all of the 
 * required bits are present and in the correct format
 */

DLLExport int
frRemoteNymVerifyName (
  ZkError err ,
  FrRemoteNym nym )
{
  static const char isok[0x101] = 
  /* 0               1               2               3                */
  /* 0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF */
    "                                             -. 0123456789      "
    " ABCDEFGHIJKLMNOPQRSTUVWXYZ    _ abcdefghijklmnopqrstuvwxyz     "     
    "                                                                "
    "                                                                " ;
  const unsigned char * name;
  size_t name_sz = 0;
  int i = 0;

  zkErrorClear(err);
  if (nym) {
    if (!frRemoteNymResetParameterSearch(err,nym)) {
      return 0;
    }
    if (!frRemoteNymGetParameter(err,nym,NymName,&name,&name_sz)) {
      zkErrorSetDetail(err,"Missing nym name");
      return 0;
    }
    for (i = 0; i < (int) name_sz; i++) {
      if (isok[name[i]] == ' ') {
        zkErrorSet(err,"frRemoteNymVerifyName",ZKERR,ZKMRNYM,ZKEDATABAD,
                   "Nym name contains illegal character");
        return 0;
      }
    } 
    return 1;
  }
  zkErrorSet(err,"frRemoteNymVerifyName",ZKERR,ZKMRNYM,ZKENULLPTR,NULL); 
  return 0;
}

DLLExport int
frRemoteNymVerifyPublicKeys (
  ZkError err ,
  FrRemoteNym nym )
{
  int return_code = 0;
 
  zkErrorClear(err);
  if (nym) {
    FrCryptCrypt encpub = NULL;
    FrCryptSig sigpub = NULL;
    if (!frRemoteNymResetParameterSearch(err,nym)) {
      return 0;
    }
    return_code = frRemoteNymGetKeys(err,nym,&sigpub,&encpub);
    if (sigpub) {
      frCryptSigFree(NULL,&sigpub);
    }
    if (encpub) {
      frCryptCryptFree(NULL,&encpub);
    }
    return return_code;
  }
  zkErrorSet(err,"frRemoteNymVerifyPublicKeys",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
}


/*
 * This function is used to see if a token of a particular description can
 * be applied to the specified nym.  Note that in this case, you can pass
 * a NULL pointer for the nym (it means that the nym just doesnt exist yet,
 * in which case, the token description must have the FL_TOKEN_CREATE flag
 * set in order to be allowable)
 *
 * We now have to juggle multiple nym types.  Don't allow upgrades at the
 * moment.
 */

DLLExport int
frRemoteNymAllowToken (
  ZK_INOUT ZkError err ,
  ZK_IN    FrRemoteNym nym ,
  ZK_IN    const FrTokenTypeDescription * descr ,
  ZK_OUT   int * allow )
{
  static const char * func = "frRemoteNymAllowToken";
  int ret = 0;                  /* Function return code */

  if (!descr) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"Token description is NULL");
  } else if (!allow) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"Token allowed flag is NULL");
  } else {
    *allow = 0;   /* By default, do not allow application of the token */
    ret = 1;

    /*
     * If we dont have a nym, the token can only be applied if it has the
     * FL_TOKEN_CREATE flag set.
     */

    if (!nym) {
      if (descr->flags & FL_TOKEN_CREATE) {
        *allow = 1;
      } else {
        zkErrorSet(err,func,ZKWARN,ZKMRNYM,ZKERROR,
                   "This token type does not permit nym creation");  
      }

    /*
     * Adding token to existing nym.
     */

    } else {
      FrNymType nymType = FRNYM_TYPE_INIT;
      int secs = 0;                 /* Number of days left for the nym */
      int days = 0;                 /* Number of days left for the nym */


      if (   !frRemoteNymSecondsRemaining(err, nym, &secs)
          || !frRemoteNymGetType(err, nym, &nymType)) { 
        ret = 0; 
      } else {

        days = secs / 86400; /* Divide by number of seconds in 1 day */

        /* Token must be allow ugrades */
        if (!(descr->flags & FL_TOKEN_UPGRADE)) {
          zkErrorSet(err,func,ZKWARN,ZKMRNYM,ZKERROR,
                     "This type of token cannot be used to extend a nym");  

        /* Max Life */
        } else if ((days + descr->duration) > descr->maxlife) {
          zkErrorSet(err,func,ZKWARN,ZKMRNYM,ZKERROR,
                     "This token would extend the nym duration past "
                     "the token maxlife");  

        /* No nym type changes right now */
        } else if (nymType != FRNYM_TYPE_FROM_TOKEN_FLAGS(descr->flags)) {
          zkErrorSet(err,func,ZKWARN,ZKMRNYM,ZKERROR, "Can't mix up nym types");
        } else {
          *allow = 1;
        }
      }
    }
  }

  return ret;
}

/*
 * This function will return the number of seconds a nym has left to live until
 * it hits its expiry date.
 */

DLLExport int
frRemoteNymSecondsRemaining (
  ZK_INOUT ZkError err ,
  ZK_IN    FrRemoteNym nym ,
  ZK_OUT   int * seconds )
{
  static const char * func = "frRemoteNymDaysRemaining";
  unsigned char * dtstr = NULL; /* For reading the expiry date time stamp */
  size_t dtstr_sz = 0;          /* Size of the expiry date time stamp */
  ZkDateTime dt;                /* Parsed version of the expiry date time */
  time_t expiry = 0;            /* time_t version of the expiry date time */
  int ret = 0;

  if (!nym) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"nym pointer is NULL");
  } else if (!seconds) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"seconds pointer is NULL");
  } else {
    *seconds = 0;
    if (!frRemoteNymGetParameter(err,nym,NymExpiry,
                                 (const unsigned char **) &dtstr,&dtstr_sz)) {
      ret = 0;
    } else if (!zkDateTimeParse(err,&dt,(char *)dtstr)) {
      ret = 0;
    } else if (!zkDateTimeMkTime(err,&dt,ZKTIME_FL_LOCAL,&expiry)) {
      ret = 0;
    } else {
      *seconds = (int) expiry - (int) time(NULL);
      if (*seconds < 0) {
        *seconds = 0;
      }
      ret = 1; 
    }
  }

  return ret;
}


/**
 * 
 * NOTE:  this function MUST be kept up to date with the FrTokenTypeDescription
 * structure which is described in frTokenType.h
 */


DLLExport int
frRemoteNymAddTokenDescription (
  ZkError err ,
  FrRemoteNym nym ,
  FrTokenTypeDescription * descr )
{
  const char * const func = "frRemoteNymAddTokenDescription";

#define TSTR_SZ 64 /* Temporary string size */
  unsigned char tstr[TSTR_SZ + 1];
  unsigned char * dtstr = NULL;
  size_t dtstr_sz = 0;
  ZkDateTime dt;

  FrNymType nymType = FRNYM_TYPE_INIT;
  unsigned char nymTypeBuff[sizeof(FrNymType)];

  zkErrorClear(err);
  if (nym && descr) {

    /* 
     * Get the current expiry date of the nym.  This will first be the
     * expiry parameter if there is one, or the dtstamp parameter if there
     * is one.  If neither of these exist, the current time is used.
     * At the end of this block, the variable dt holds the current expiry 
     * time in UTC format.
     */

    if (!frRemoteNymResetParameterSearch(err,nym)) {
      return 0;
    }
    if (!frRemoteNymGetParameter(err,nym,NymExpiry,
                                 (const unsigned char **) &dtstr,&dtstr_sz)) {
      if (!frRemoteNymGetParameter(err,nym,NymTimeStamp,
                                   (const unsigned char **) &dtstr,&dtstr_sz)) {
        /*
         * We dont have an expiry date, nor a birth date.  Add the birth
         * date as being the current time.
         */
        if (!(zkDateTimeSetCurrent(err,&dt,ZKTIME_FL_UTC) &&
              zkDateTimeStore(err,&dt,(char *)tstr,TSTR_SZ) &&
              frRemoteNymAddParameter(err,nym,NymTimeStamp,tstr,
                                      strlen((char *)tstr)))){
          return 0;
        }
      } else if (!zkDateTimeParse(err,&dt,(char *)dtstr)) {
        return 0;
      }
    } else if (!zkDateTimeParse(err,&dt,(char *)dtstr)) {
      return 0;
    }

    /*
     * Clear out the parameters which are supposed to be controlled by the
     * token description
     */

    if (!frRemoteNymStrip(err,nym,FL_TOKEN)) {
      return 0;
    }
    
    /* 
     * Add the email domain  
     */
   
    if (!frRemoteNymAddParameter(err,nym,NymEmailDomain,(const unsigned char *) descr->email_domain,
                                (size_t) strlen((char *)descr->email_domain))) {
      return 0;
    }

    /*
     * Add the email limit
     */
  
    zkSnprintf((char *)tstr,sizeof tstr,"%d",descr->email_limit);
    if (!frRemoteNymAddParameter(err,nym,NymEmailLimit,tstr,
                                 (size_t) strlen((char *)tstr))) {
      return 0;
    }

    /*
     * Add the crosspost limit 
     */

    zkSnprintf((char *)tstr,TSTR_SZ,"%d",descr->crosspost_limit);
    if (!frRemoteNymAddParameter(err,nym,NymCrossPostLimit,tstr,
                                 (size_t) strlen((char *)tstr))) {
      return 0;
    }
    
    /*
     * Add the access level 
     */

    zkSnprintf((char *)tstr,sizeof tstr,"%d",descr->access_level);
    tstr[TSTR_SZ] = (char) 0;
    if (!frRemoteNymAddParameter(err,nym,NymAccessLevel,tstr,strlen((char *)tstr))) {
      return 0;
    }
 
    /*
     * Add the expiry date.  If the token specifies a duration overide, the
     * new expiry date will be the current date plus the token duration.  This
     * essentially has no effect on a new nym, only an an existing nym.  
     */

    if (descr->flags & FL_TOKEN_DURATION_OVERIDE) {
      if (!zkDateTimeSetCurrent(err,&dt,ZKTIME_FL_UTC)) {
        return 0;
      }

      /*
       * If the nym's expiry date has passed, we also reset the date to the 
       * current date so that the token is not applied to time already spent 
       * in the past.
       * %%Bug:  The zkDateTimeMkTime() function should use a UTC time flag
       */
    
    } else {
      time_t expiry = 0;
      if (!zkDateTimeMkTime(err,&dt,ZKTIME_FL_LOCAL,&expiry)) {
        return 0;
      }
      if (expiry < time(NULL)) {
        if (!zkDateTimeSetCurrent(err,&dt,ZKTIME_FL_UTC)) {
          return 0;
        }
      }
    }

    if (!(zkDateTimeAdd(err,&dt,descr->duration,ZKDATETIME_DAYS) &&
          zkDateTimeStore(err,&dt,(char *)tstr,TSTR_SZ) &&
          frRemoteNymAddParameter(err,nym,NymExpiry,tstr,
                                  strlen((char *)tstr)))) {
      return 0;
    }

    /*
     * Add the nym release date
     */

    if (!(zkDateTimeAdd(err,&dt,descr->reserved,ZKDATETIME_DAYS) &&
          zkDateTimeStore(err,&dt,(char *)tstr,TSTR_SZ) &&
          frRemoteNymAddParameter(err,nym,NymRelease,tstr,strlen((char *)tstr)))) {
      return 0;
    }

    /*
     * Add the type parameter.
     */

    nymType = FRNYM_TYPE_FROM_TOKEN_FLAGS(descr->flags);
    
    /* Someone got the brilliant idea of passing the address of nymType directly 
    to frRemoteNymAddParameter, which means that it was stored in host byte order, 
    *not* network byte order.  On Windows & Linux host byte order is little-endian.  
    So now we have to make sure nymType is *always* written in little-endian byte 
    order. */
    
    nymTypeBuff[0] = (unsigned char) (nymType >>  0);
    nymTypeBuff[1] = (unsigned char) (nymType >>  8);
    nymTypeBuff[2] = (unsigned char) (nymType >> 16);
    nymTypeBuff[3] = (unsigned char) (nymType >> 24);
    
    if (!frRemoteNymAddParameter(err,nym,NymType,nymTypeBuff,
                                 sizeof(nymTypeBuff))) {
      zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKERROR,"Could not add NymType param");
      return 0;
    }
    /*
     * All done!
     */

    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int
frRemoteNymVerify (
  ZkError err ,
  FrRemoteNym nym )
{
  zkErrorClear(err);

  if (nym) {
    if (!frRemoteNymVerifyName(err,nym)) {
      return 0;
    }
    if (!frRemoteNymVerifyPublicKeys(err,nym)) {
      return 0;
    }
    return 1;
  }
  zkErrorSet(err,"frRemoteNymVerify",ZKERR,ZKMRNYM,ZKENULLPTR,NULL);
  return 0;
}

/*
 * Get the nym type
 *
 */

DLLExport int 
frRemoteNymGetType (
  ZkError err ,
  FrRemoteNym nym ,
  FrNymType * nymType )
{
  const char * const func = "frRemoteNymGetType";
  int ret = 0;

  const unsigned char * data = NULL;
  size_t length = 0;

  zkErrorClear(err);
  if ((nym == NULL) || (nymType == NULL)) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"NULL pointer");

  } else if (!frRemoteNymResetParameterSearch(NULL, nym)) {

  } else if (!frRemoteNymGetParameter(err, nym, NymType, &data, &length)) {

    /* If there was no such parameter it defaults to nym type one */
    if (ZKENOTFOUND == zkErrorCode(err)) {
      *nymType = FRNYM_TYPE_ONE;
      ret = 1;
      zkErrorClear(err);
    }

  } else if (length != sizeof(FrNymType)) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKEPARMBAD,"NymType data wrong length");

  } else {
    /* Convert nym type from little endian. */
    *nymType = (FrNymType) ((((FrNymType) data[0]) <<  0) | 
    					    (((FrNymType) data[1]) <<  8) | 
    					    (((FrNymType) data[2]) << 16) | 
    					    (((FrNymType) data[3]) << 24));
    ret = 1;

    if (*nymType > FRNYM_TYPE_MAX) {
      zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKEPARMBAD,"NymType is too big");
      ret = 0;
    }
  }

  return ret;
}

/*
 * Get nym capabilities, from nym type
 *
 */

DLLExport int 
frRemoteNymGetCapabilityFromType (
  ZkError err ,
  FrNymType nymType ,
  FrNymCapability * nymCapability )
{
  const char * const func = "frRemoteNymGetCapabilityFromType";
  int ret = 0;

  FrNymCapabilityTable capTable = FRNYM_CAPABILITYTABLE_DEFAULT;

  zkErrorClear(err);
  if (nymType > FRNYM_TYPE_MAX) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKEPARMBAD,"NymType is wrong");
    ret = 0;
  }

  else if (nymCapability == NULL) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"NULL pointer");
    ret = 0;

  } else {
    *nymCapability = capTable[nymType];
    ret = 1;
  }

  return ret;
}

/*
 * Get the nym capabilities, dervied from the nym type
 *
 */

DLLExport int 
frRemoteNymGetCapability (
  ZkError err ,
  FrRemoteNym nym ,
  FrNymCapability * nymCapability )
{
  const char * const func = "frRemoteNymGetCapability";
  int ret = 1;

  FrNymType nymType = FRNYM_TYPE_INIT;

  zkErrorClear(err);
  if ((nym == NULL) || (nymCapability == NULL)) {
    zkErrorSet(err,func,ZKERR,ZKMRNYM,ZKENULLPTR,"NULL pointer");
    ret = 0;

  } else if (!frRemoteNymGetType(err, nym, &nymType)) {
    ret = 0;
  } else if (!frRemoteNymGetCapabilityFromType(err, nymType, nymCapability)) {
    ret = 0;
  }

  return ret;
}
