/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "frnymsdb.h"

#include <stdio.h>
#include <unistd.h>
#include <getopt.h>

#include <zkDB.h>

/* 
 * This test code generates a large nym database
 *
 * It takes the following parameters:
 *
 * nymsdb-gen seeddb seednym targetdb firstnym lastnym
 *
 * It will open the seeddb, read the seednym from it and close it.
 *
 * It will then open the targetdb, and write nyms to the targetdb, with names
 * like "autonym12345" and data copied from seed nym
 *
 */

#define USAGE printf("Usage:\n  %s seeddb seednym targetdb firstnym lastnym\n", argv[0])

int main(int argc, char * const argv[])
{
  ZkBool              cont                  = ZK_TRUE;
  ZkError             err                   = NULL;

  const char * const  seedDBName            = argv[1];
  const char * const  seedNymName           = argv[2];
  const char * const  targetDBName          = argv[3];
  uint32_t            firstNym              = (uint32_t) atoi(argv[4]);
  uint32_t            lastNym               = (uint32_t) atoi(argv[5]);

  ZkDB                seedDB                = NULL;
  ZkDB                targetDB              = NULL;

  FrRemoteNym         nymBody               = NULL;
  int                 found                 = 0;
  char                newNymName[NYMNAME_SZ] = {0};

  if (argc < 6) {
    USAGE;
    exit(EXIT_FAILURE);
  }

  cont = cont && zkErrorNew(&err);

  cont = cont && zkDB_Open(err, &seedDB, seedDBName);

  cont = cont && frnymsdbGetNym(err, seedDB, seedNymName, &nymBody, &found);

  found || (cont = ZK_FALSE)
        || printf("Couldn't find nym\n");

  cont || printf("SeedDB access failed: %s\n", zkErrorDetail(err));

  (seedDB == NULL) || zkDB_Close(err, &seedDB)
                   || printf("SeedDB close failed: %s\n", zkErrorDetail(err));


  cont = cont && zkDB_Open(err, &targetDB, targetDBName);

  for (; cont && (firstNym <= lastNym); firstNym++) {

    snprintf(newNymName, sizeof newNymName, "autonym%d", firstNym);

    cont && printf("Trying: %s ", newNymName);

    cont = cont && frRemoteNymReplaceParameter(err,
                                               nymBody,
                                               NymName, 
                                               newNymName,
                                               strlen(newNymName));

    cont = cont && frnymsdbAddNym(err, targetDB, newNymName, nymBody);

    cont ? printf("succeeded\n") : printf("failed\n");
  }

  cont || printf("TargetDB access failed: %x %s\n",
                 (int) zkErrorCode(err),
                 zkErrorDetail(err));

  (NULL == nymBody)  || frRemoteNymFree(err, &nymBody)
                     || printf("frRemoteNymFree() failed: %s\n",
                               zkErrorDetail(err));

  (NULL == targetDB) || zkDB_Close(err, &targetDB)
                     || printf("TargetDB close failed: %s\n",
                               zkErrorDetail(err));
  
  exit((cont) ? EXIT_SUCCESS : EXIT_FAILURE);
}

