/*
 *  $Id: fr_nymauth.h,v 1.5 2001/04/19 20:30:02 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_NYMAUTH_H_
#define FR_NYMAUTH_H_


/*
 *
 *
 *  Nym Authentication.
 *
 *
 */


#include "frEnt.h"


/*
 *  The nym authentication field appears after the last route.
 *  
 *  The serialNum field is a random value, generated by the client, that the
 *  exit funnel uses to prevent replay attacks over the lifetime of the route.
 *  That is, upon receiving a route creation request for a given nym, the exit
 *  funnel should check for the existence of the serial number, rejecting the
 *  request if it is found.
 *  
 *  The nymSig field is used to sign the last route, including the final
 *  shared secret (not the pre-secret), as well as the serial number and nym
 *  ID of the nym authentication field.
 *  
 *  The hash field contains a SHA-1 hash calculated over the last route, and
 *  part of the nym authentication section.
 *  
 *  The hashsig field is the nym's signature over the preceding hash.
 */
  
struct fr_nym_auth_exp {
  char serialNum[8];
  FrEntIdExp nymID;
  unsigned int sigKeyVersion;
  FrCryptSig nymSig;
  char hash[FRCRYPT_HASH_SIZE];
  char hashsig[FRCRYPT_SIG_SIZE];
};

#endif /* FR_NYMAUTH_H_ */
