/*
 *  $Id: fr_rcpkt.h,v 1.10 2001/04/19 20:30:02 alainc Exp $
 */

#ifndef FR_RCPKTH_
#define FR_RCPKTH_


/*
 *
 *
 *  Routines manipulating route information.
 *
 *
 */

/*
 *  A route creation packet may contain up to this many routes.
 */

#define FRPKT_NUM_ROUTES_MAX 5

/*
 *  The bits in the flags field are interpeted as follows:
 *  
 *  bit 0: last route -- 0 intermediate hop; 1 this is the last hop.
 *  
 *  bit 1: exit ACI -- 0 ignore the exit ACI; 1 use given ACI at exit funnel.
 */
#define FRPKT_RTECRT_FL_LAST_RTE 0x01
#define FRPKT_RTECRT_FL_SPEC_ACI 0x02


/*
 *  bug: for expediency we expose (hence the trailing _exp) two structures
 *  containing all the information we need to pass back and forth.  Given
 *  more time we probably ought to hide it.
 */

#if 0
/* bug: make consistent with rte.h. */
typedef enum {
  FRHOSTADDR_UNUSED_,
  FRHOSTADDR_IPV4
} FrHostAddrType;

#define ADDR_LEN	16
typedef char FrHostAddr[ADDR_LEN];
typedef uint16_t FrHostPort;
#endif
typedef uint32_t FrTime;	/* seconds since 1/1/1970, UTC */

/*
 *  Use this structure when generating route creation packets.
 */
  
struct fr_rte_set_exp {
  FrCryptSymAlg rteAlg; /* to encrypt trailing part of packet route field */
  FrCryptSym bckCrypt;
  FrCryptSym fwdCrypt;
  FrCryptMacKey macKey;
  FrEntIdExp hostEnt; /* identifies this hop's host */
  FrTime expTime;
  uint8_t flags;
  FrAci specAci;
};
  
/*
 *  Use this structure when extracting routes from route creation packets.
 */
  
struct fr_rte_get_exp {
  FrCryptSym bckCrypt;
  FrCryptSym fwdCrypt;
#if 0
  unsigned int cryptKeyVers;
#endif
  FrCryptMacKey macKey;
  FrEntIdExp hostEnt;
  FrTime expTime;
  uint8_t flags;
  FrAci specAci;
};
  

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /*
   *
   *
   *  Routines for AIP route creation packets.
   *
   *
   */
  
  typedef struct fr_pkt_rte_crt *FrPktRC;

#if !defined(ZK_KERNEL)
  DLLExport void fr_PktAipStructInfo(void);
#endif	/* !defined(ZK_KERNEL) */

  DLLExport size_t frPktAipRteCrtBufferSize(void);
  DLLExport size_t frPktAipRteCrtXmitSize(void);

  DLLExport int frPktAipRteCrtNew(ZkError err, FrPktRC *pkt);
  DLLExport int frPktAipRteCrtNewNoBuffer(ZkError err, FrPktRC *pkt);
  DLLExport int frPktAipRteCrtNewUsingBuffer(ZkError err, FrPktRC *pkt, void *data, size_t len);
  DLLExport int frPktAipRteCrtFree(ZkError err, FrPktRC *pkt);
  DLLExport int frPktAipRteCrtReset(ZkError err, FrPktRC pkt);

  DLLExport int frPktAipRteCrtSetAci(ZkError err, FrPktRC pkt, FrAci aci);
  DLLExport int frPktAipRteCrtGetAci(ZkError err, FrPktRC pkt, FrAci *aci);

#if !defined(ZK_KERNEL_MAC_OS8)
  DLLExport int frPktAipRteCrtAppendRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPubKey, struct fr_rte_set_exp *r);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */
  DLLExport int frPktAipRteCrtGetCryptKeyVersion(ZkError err, FrPktRC pkt, unsigned int *vers);
#if !defined(ZK_KERNEL_MAC_OS8)
  DLLExport int frPktAipRteCrtGetNextRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt privKey, struct fr_rte_get_exp *r);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */

  DLLExport int frPktAipRteCrtGetBuffer(ZkError err, FrPktRC pkt, void **data, size_t *len);
  DLLExport int frPktAipRteCrtSetBuffer(ZkError err, FrPktRC pkt, void *data, size_t len);

#if !defined(ZK_KERNEL_MAC_OS8)
  DLLExport int frPktAipRteCrtEncryptAllRoutes(ZkError err, FrPktRC pkt);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */

  DLLExport void fr_PktAipStructInfo(void);
  DLLExport int frPktAipRteCrtGetMaxSize(ZkError err, unsigned int vers, size_t *sz);
  DLLExport int frPktAipRteCrtNew(ZkError err, FrPktRC *pkt);
  DLLExport int frPktAipRteCrtFree(ZkError err, FrPktRC *pkt);
  DLLExport int frPktAipRteCrtReset(ZkError err, FrPktRC pkt);
  DLLExport int frPktAipRteCrtCreateInBuffer(ZkError err, FrPktRC pkt, void *buf, size_t bsz);
  DLLExport int frPktAipRteCrtReceiveInBuffer(ZkError err, FrPktRC pkt, void *buf, size_t bsz);
  DLLExport int frPktAipRteCrtSetAci(ZkError err, FrPktRC pkt, FrAci aci);
  DLLExport int frPktAipRteCrtGetAci(ZkError err, FrPktRC pkt, FrAci *aci);
  DLLExport int frPktAipRteCrtAppendRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPubKey, struct fr_rte_set_exp *r);
  DLLExport int frPktAipRteCrtGetCryptKeyVersion(ZkError err, FrPktRC pkt, unsigned int *vers);
  DLLExport int frPktAipRteCrtGetNextRouteExp(ZkError err, FrPktRC pkt, FrCryptCrypt aipPrivKey, struct fr_rte_get_exp *r);
#if !defined(ZK_KERNEL)
  DLLExport int frPktAipRteCrtAppendNymAuthExp(ZkError err, FrPktRC pkt, struct fr_nym_auth_exp *nymAuth);
  DLLExport int frPktAipRteCrtGetNymAuthExp(ZkError err, FrPktRC pkt, struct fr_nym_auth_exp *nymAuth);
  DLLExport int frPktAipRteCrtAuthenticateNym(ZkError err, struct fr_nym_auth_exp *nymAuth, int *isAuth);
#endif	/* !defined(ZK_KERNEL) */
  DLLExport int frPktAipRteCrtGetBuffer(ZkError err, FrPktRC pkt, void **data, size_t *len);
  DLLExport int frPktAipRteCrtEncryptAllRoutes(ZkError err, FrPktRC pkt);
  DLLExport int frPktAipRteCrtGetCurrentSize(ZkError err, FrPktRC pkt, size_t *sz);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FR_RCPKTH_ */
