/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef RTE_H_
#define RTE_H_

/*
 *  The following offsets are relative to the beginning of a route subfield.
 *  
 *  bug: each route field would be a good candidate for a checksum.
 */

#define KEY_SEED_MAT_OFFSET  0
#define KEY_SEED_MAT_SIZE   16

#define RTE_KEY_ALG_OFFSET  (KEY_SEED_MAT_OFFSET + KEY_SEED_MAT_SIZE)
#define RTE_KEY_ALG_SIZE    1

#define RTE_PAD_OFFSET      (RTE_KEY_ALG_OFFSET + RTE_KEY_ALG_SIZE)
#define RTE_PAD_SIZE        7

#define BCK_KEY_ALG_OFFSET  (RTE_PAD_OFFSET + RTE_PAD_SIZE)
#define BCK_KEY_ALG_SIZE    1

#define FWD_KEY_ALG_OFFSET  (BCK_KEY_ALG_OFFSET + BCK_KEY_ALG_SIZE)
#define FWD_KEY_ALG_SIZE    1

#define HOST_TYPE_OFFSET    (FWD_KEY_ALG_OFFSET + FWD_KEY_ALG_SIZE)
#define HOST_TYPE_SIZE      1

#define HOST_ADDR_OFFSET    (HOST_TYPE_OFFSET + HOST_TYPE_SIZE)
#define HOST_ADDR_SIZE      16

#define HOST_PORT_OFFSET    (HOST_ADDR_OFFSET + HOST_ADDR_SIZE)
#define HOST_PORT_SIZE      2

#define EXP_TIME_OFFSET     (HOST_PORT_OFFSET + HOST_PORT_SIZE)
#define EXP_TIME_SIZE       4

#define RTE_SIZE (EXP_TIME_OFFSET + EXP_TIME_SIZE - KEY_SEED_MAT_OFFSET)

/* Sanity check on the size */
#if RTE_SIZE != 49
#error "mistake in calculating the route size"
#endif

/* Sanity check on part to be symmetrically encrypted. */
#if (RTE_SIZE - RTE_PAD_OFFSET) % FRCRYPT_SIZE_MULT
#error "mistake in calculating padding"
#endif


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  typedef struct fr_rte *FrRte;

  extern int frRteNew(ZkError err, FrRte *rte);
  extern int frRteFree(ZkError err, FrRte *rte);

  extern int frRteGetBackSym(ZkError err, FrRte rte, FrCryptSym *bck);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* RTE_H_ */
