#
# Common procs for manipulating wire format public
# key requests and responses.
#


set g_db_root [file join [glob ~bill] tmp ndb]

array set g_keyType {
  1 cryption
  2 signing
}

array set g_entType {
  1 aip
  2 maip
  3 master
  4 ndb
  5 nym
  6 nymsrv
}

#
# Expand the query.
#
# Return a 5 element list containing the version, type,
# entity type, entity name, all parameters flag and the
# current version flag.
#
# Return the empty list on an error.
#
proc expQuery { q } {
  global g_entType g_keyType

  set res {}

  # debugging
  binary scan $q "c*" foo
  puts stderr "expQuery: foo is $foo"

  if {[binary scan $q "cccccA64c" \
           pktVers pktType \
           vers type entType entName \
           flags] != 7} {
    puts stderr "Didn't find 7 elements in query."
  } else {
    lappend res [format "%d" $vers]
    set x [format "%d" $type]
    if {[catch { lappend res $g_keyType($x) }]} {
      puts stderr "bad public key type $x"
      return {}
    }
    lappend res [expr $flags & 0x01] ; # all params
    lappend res [expr $flags & 0x02] ; # current version
    set x [format "%d" $entType]
    if {[catch { lappend res $g_entType($x) }]} {
      puts stderr "bad entity type $x"
      return {}
    }
    lappend res $entName
  }

  return $res
}


proc prtDbQry {q} {
  global g_keyType g_entType tcl_traceExec
  
#  set tcl_traceExec 2

  set qlen [string length $q]
  binary scan $q "cccA64" vers type entType entName
  set x [format "%d %d %d %s" $vers $type $entType $entName]
  return $x
}


proc prtNetQry {q} {
  global g_keyType g_entType tcl_traceExec
  
#  set tcl_traceExec 2

  set qlen [string length $q]
  binary scan $q "cccccA64" pktVers pktType vers type entType entName
  set x [format "%d %d %d %s" $vers $type $entType $entName]
  return $x
}


#
# Create the path to the file containing the response to
# the specified query.
#
# The file may not exist, in which case that entry is not
# in our database.
#
# Return the empty string on an error.
#
proc mkQueryPath { q } {
  global g_db_root

  set qlist [expQuery $q]
  if {[llength $qlist] == 0} {
    return ""
  }

  foreach {vers type allparms currvers entType entName} $qlist {
    break;
  }

  # Turn white space to underscores in the file name.
  regsub -all -- " |\t" $entName "_" pfx

  return [file join $g_db_root $type $entType [format "%s.%d" $pfx $vers]]
}
