#ifndef __RCACKPKT_H
#define __RCACKPKT_H

#include "zkserr.h"
#include "frPktFmt.h"
#include "frPktWrap.h"

class cRouteCreateAckPkt
{
private:
    FrPktAD  ackCtx;    // route create ack context 
    uint8_t  *pktbuffer;    //pkt buffer
    size_t   pktbufferlen;
    ZkBool   freePkt;   // indicates whether the pkt buffer must be freed  
                        // before destruction of the object 

    //copy constructor (disabled)
    cRouteCreateAckPkt( const cRouteCreateAckPkt&) { assert(0); };

    //assignment operator (disabled)
    cRouteCreateAckPkt& operator=( const cRouteCreateAckPkt&) { assert(0); };

public:
    cRouteCreateAckPkt();        // constructor 
    ~cRouteCreateAckPkt();       // destructor 

    // **********************************************************************
    // *
    // * This member generates a new Route Create ACK packet.  After a call of 
    // * this member, the packet should be considered in the same state as
    // * after a call to attachPkt, i.e. formatBackwardHop must be called
    // * before the packet is relayed to the next hop.
    // * 
    // * NOTE: After a call to this member, make sure destroy is called
    // *       before the object is destroyed.
     
    void create
    (
        uint32_t      version,    // packet version 
        uint8_t       prio,       // packet priority 
        FrAci         exitAci,    // exit hop ACI 
        FrAci         netHopAci,  // next hop ACI 
        FrCryptMacKey macKey      // MAC key 
    );

    // **********************************************************************
    // *
    // * This member is called to release the resources allocated during
    // * the previous call to create.
    // * 
    // * NOTE: This MUST be called before the object is destroyed if create
    // *       was used.
     
    void destroy();
    
    // **********************************************************************
    // *
    // * This member attaches the given packet to the cRCAckPkt object.
    // *
    // * The given packet must be a full link packet, including the link 
    // * header and trailer.
    // *
     
    void attachPkt(
        uint8_t * pkt,         // buffer containing RC pkt 
        size_t    pktSize      // pkt size 
    );

    // **********************************************************************
    // *
    // * This member detaches the Route Create ACK packet from the 
    // * cRouteCreateAckPkt object 
    // *
    // * This should be used to make sure that the reference to the original
    // * buffer is deleted.
    // * 
     
    void detachPkt();
    
    // **********************************************************************
    // *
    // * This member modifies the Route Create ACK packet to an appropriate 
    // * format for it to be forwarded to the next hop (backward or Internet-
    // * to-client direction).
    // * 
    // * A packet must currently be attached for this member to work,
    // * i.e. previous call to create or attachPkt was made.
    // * 
     
    void formatBackwardHop
    (
        FrAci  aci     // ACI for next hop 
    );

    // **********************************************************************
    // *
    // * This member returns the ACI in the attached pkt
    // * 
    // * A packet must currently be attached for this member to work.
    // * 
     
    FrAci getAci();

    // **********************************************************************
    // *
    // * This member returns a reference to the Route Create ACK packet buffer
    // * and its size.
    // * 
    // * A packet must currently be attached for this member to work,
    // * i.e. previous call to create or attachPkt was made.
    // * 
     
    FrAci getExitAci(FrCryptMacKey macKey);

    void decrypt(FrCryptSym cryptKey);
    void encrypt(FrCryptSym cryptKey);

    void getBuffer
    (
        uint8_t ** buf,    // out: pointer to buffer 
        size_t   * bufSize // out: buffer size 
    );
};

#endif // __RCACKPKT_H 
