#undef NDEBUG
#include <assert.h>
#include "zksexception.h"
#include "cKeepAlivePkt.h"


cKeepAlivePkt::cKeepAlivePkt()
{
    kaCtx = NULL;
    pktbuffer = NULL;
    pktbufferlen = 0;
    freePkt = ZK_FALSE;
}

cKeepAlivePkt::~cKeepAlivePkt()
{
   if (freePkt) {
      destroy();

   }
}

void cKeepAlivePkt::create(uint32_t version, uint8_t prio)
{
    const char    * func = "cKeepAlivePkt::create";
    CZKS::Exception e;

    /* Validate params */
    assert((version == FRPKT_ROUTE_VERSION_1) || (version == FRPKT_ROUTE_VERSION_2));
    assert(kaCtx == NULL);
    assert(!freePkt);
            
    try
    {
        uint8_t       * pkt = NULL;
        size_t          pktSize = 0;
        uint8_t         fake;
        size_t          maxsize;

        if (frPktAipDataGetMaxSize(e.zkError(), version, &maxsize) == 
            ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }

        pktbufferlen = frPktGetSize( maxsize)+1;
        pktbuffer = new uint8_t[pktbufferlen]; 

        // Allocate Keep Alive context
        if (frPktAipDataNew(e.zkError(), &kaCtx) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktSetVersion(e.zkError(), pktbuffer, pktbufferlen, 
                                 version) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        else if (frPktSetType(e.zkError(), pktbuffer, pktbufferlen, 
                              FRPKTTYPE_AIP_ALIVE) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktGetEmbeddedPacket(e.zkError(), 
                                        pktbuffer, pktbufferlen, 
                                        (void**)&pkt, &pktSize) == ZKS_FAILURE) 
        {
            CZKS::Throw(e);
        }
        else if (frPktAipDataCreateInBuffer(e.zkError(), kaCtx, pkt, 
                                            pktSize) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktAipDataSetUseMac(e.zkError(), kaCtx, ZK_FALSE) == 
                 ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        // NOTE: We have to give a fake buffer (even if length is 0) because
        // the function doesn't accept NULL
        else if (frPktAipDataSetPayload(e.zkError(), kaCtx, NULL, 
                                        &fake, 0) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        else if (frPktSetPriority(e.zkError(), pktbuffer, pktbufferlen, 
                                  prio) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        freePkt = ZK_TRUE;
    }
    catch (...)
    {
        delete pktbuffer;
        pktbuffer = NULL;
   
        if (kaCtx != NULL)
        {
            frPktAipDataFree(e.zkError(), &kaCtx);
            kaCtx = NULL;
        }
        
        CZKS::Rethrow(func, "Failed");
    }
}

void cKeepAlivePkt::destroy(void)
{
    const char * func = "cKeepAlivePkt::destroy";

    assert(kaCtx != NULL);
    assert(freePkt);
    
    try
    {
        CZKS::Exception e;
        
        delete pktbuffer;
        pktbuffer = NULL;
 
        if (frPktAipDataFree(e.zkError(), &kaCtx) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        freePkt = ZK_FALSE;
        kaCtx = NULL;
    }
    
    catch (CZKS::Exception &e)
    {
        CZKS::Rethrow(func, "Failed");
    }
}

void cKeepAlivePkt::getBuffer
(
    uint8_t ** buf,    // out: pointer to buffer
    size_t   * bufSize // out: buffer size
)
{
    const char * func = "cKeepAlivePkt::getBuffer";

    size_t pktsize;

    // Validate params
    assert(buf != NULL);
    assert(bufSize != NULL);

    // A packet MUST currently be attached
    assert(kaCtx != NULL);

    try
    {
        CZKS::Exception e;
        
        /* Get size of data packet. */
        if (frPktAipDataGetCurrentSize(e.zkError(), kaCtx, 
                                       &pktsize) == ZKS_FAILURE)
        {
            CZKS::Throw(e);
        }
        *bufSize = frPktGetSize(pktsize);
        *buf = (uint8_t*)pktbuffer;
 
        // frPktAipData library allocates an extra byte on creation,
        // don't report it
        //*bufSize -= 1; 
    }
    catch (CZKS::Exception &e)
    {
        CZKS::Rethrow(func, "Failed");
    }
}
