/*
 *  $Id: buf_v0.c,v 1.5 2001/04/19 20:30:59 alainc Exp $
 *  
 *  WARNING: this is the first, deprecated version of the encrypted buffer
 *  code.  It is being kept only while conversion to the new format is
 *  necessary.
 *  
 *  Simple decryption of data read from a file and encryption of data written
 *  to a memory buffer.
 *  
 *  The buffer will have a small header prepended to it, and the portion
 *  containing the data will be a multiple of 8 bytes in length.
 *  
 *  bug: add information to the header, such as version, so we can change the
 *  format of the buffer (e.g. the header).
 *  
 *  bug: add a hash over the data; put it in the header.
 */


#include "zks.h"

#include "zkArpaInet.h"
#include "zkErrno.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "fr_buf_v0.h"
#include "frCrypt.h"
#include "hdr_v0.h"
#include "zkAlloc.h"
#include "zkError.h"
#include "zkPackBuf.h"


/*
 *  Decrypt the contents of `src', allocating memory for the result, then
 *  setting `*dst' to that buffer.  The size of the buffer will be returned
 *  via `*dsz'.
 */
DLLExport int fr_SecDecryptFromBuf_v0(ZkError err, void *src, size_t ssz, FrCryptSym sym, void **dst, size_t *dsz)
{
  int ret = 0;
  size_t psz; /* plaintext size */
  void *buf;

  zkErrorClear(err);

  if ( ! src) {
    zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "src is null");
    return 0;
  }
  if ( ! sym) {
    zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "sym is null");
    return 0;
  }
  if ( ! dst) {
    zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dst is null");
    return 0;
  }
  if ( ! dsz) {
    zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dsz is null");
    return 0;
  }

  if (ssz < BUF_HDR_V0_SIZEOF) {
    /* Invalid buffer size */
    zkErrorSet_va(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "buffer too short (%lu < %lu)",
                  (unsigned long)ssz, (unsigned long)BUF_HDR_V0_SIZEOF);
    goto done;
  }

  if ((ssz - BUF_HDR_V0_SIZEOF) % FRCRYPT_SIZE_MULT != 0) {
    /* Invalid buffer size. */
    zkErrorSet_va(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "buffer size (%lu) not a multiple of %lu",
                  (unsigned long)(ssz - BUF_HDR_V0_SIZEOF),
                  (unsigned long)FRCRYPT_SIZE_MULT);
    goto done;
  }
  
  /*
   *  Copy the encrypted buffer to a zkMalloc()ed chunk of memory.  Decrypt it
   *  there, then later resize the memory to hold just the actual data.
   */
  
  /* bug: locked memory alloc. */
  if ( ! (buf = zkMalloc(ssz))) {
    zkErrorSet_va(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't malloc %lu bytes for data: %s",
                  (unsigned long)ssz, zkStrerror(errno));
    goto done;
  }

  memcpy(buf, src, ssz);

  /* Decrypt the buffer following the IV (which wasn't encrypted). */

  if ( ! frCryptSymDecrypt(err, sym, ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_IV), 
                           ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_DSZ), ssz - FRCRYPT_IVEC_SIZE)) {
    goto done;
  }

  psz = zkPackBufGetUint32Nbo(buf, BUF_HDR_V0_DSZ); /* amount of `actual' data in buffer */

  if (psz > ssz - BUF_HDR_V0_SIZEOF) {
    /* Inconsistent header. */
    zkErrorSet_va(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "inconsistent header (%lu > %lu)",
                  (unsigned long)psz, (unsigned long)(ssz - BUF_HDR_V0_SIZEOF));
    goto done;
  }

  if ( ! fr_CryptCheckPad(ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_PAD), BUF_HDR_V0_PAD_SIZE)) {
    zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
               "invalid padding in header");
    goto done;
  }

  if (psz < ssz - BUF_HDR_V0_SIZEOF) {
    size_t s = ssz - BUF_HDR_V0_SIZEOF - psz;
    if ( ! fr_CryptCheckPad((char *)buf + BUF_HDR_V0_SIZEOF + psz, s)) {
      zkErrorSet(err, "fr_SecDecryptFromBuf_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                 "invalid trailing padding");
      goto done;
    }
  }

  /* Reallocate space for the decrypted data. */
  

  memset(buf, 0, BUF_HDR_V0_SIZEOF);
  memmove(buf, (char *)buf + BUF_HDR_V0_SIZEOF, psz);
  if ( ! zkMemRealloc(err, &buf, psz)) {

    goto done;
  }
  
  *dst = buf; buf = 0; /* the caller now owns the buffer */
  *dsz = psz;
  
  ret = 1;

 done:

  if (buf) { memset(buf, 0, ssz); zkFree(buf); }

  return ret;
}


/*
 *  Encrypt the contents of `src', allocating memory for the result, then
 *  setting `*dst' to that buffer.  The size of the buffer will be returned
 *  via `*dsz'.
 */
DLLExport int fr_SecEncryptToBuf_v0(ZkError err, void *src, size_t ssz, FrCryptSym sym, void **dst, size_t *dsz)
{
  char iv[FRCRYPT_IVEC_SIZE];
  int ret = 0;
  size_t bsz; /* size of zkMalloc()ed buffer */
  void *buf;

  zkErrorClear(err);

  if ( ! src) {
    zkErrorSet(err, "fr_SecEncryptToBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "src is null");
    return 0;
  }
  if ( ! sym) {
    zkErrorSet(err, "fr_SecEncryptToBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "sym is null");
    return 0;
  }
  if ( ! dst) {
    zkErrorSet(err, "fr_SecEncryptToBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dst is null");
    return 0;
  }
  if ( ! dsz) {
    zkErrorSet(err, "fr_SecEncryptToBuf_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dsz is null");
    return 0;
  }

  bsz = FRCRYPT_SIZE_CEIL(ssz + BUF_HDR_V0_SIZEOF);
  /* bug: should allocate locked memory */
  if ( ! (buf = zkMalloc(bsz))) {
    zkErrorSet_va(err, "fr_SecEncryptToBuf_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't malloc %lu bytes for data: %s",
                  (unsigned long)bsz, zkStrerror(errno));
    goto done;
  }

  /* Fill in the header. */
  
  if ( ! zkRandGetData(err, ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_IV), FRCRYPT_IVEC_SIZE)) {
    goto done;
  }
  memcpy(iv, ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_IV), sizeof iv);
  zkPackBufSetUint32Nbo(buf, BUF_HDR_V0_DSZ, ssz);
  fr_CryptSetPad(ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_PAD), BUF_HDR_V0_PAD_SIZE);

  /* Copy the plaintext. */

  memcpy((char *)buf + BUF_HDR_V0_SIZEOF, src, ssz);
  
  /* Set any trailing padding to the appropriate value. */

  if (bsz > ssz + BUF_HDR_V0_SIZEOF) {
    size_t s = BUF_HDR_V0_SIZEOF + ssz;
    fr_CryptSetPad((char *)buf + s, bsz - s);
  }

  /*
   *  Encrypt the entire buffer following the IV.  The IV in the header
   *  retains its original value; our copy in `iv' is changed.
   */

  if ( ! frCryptSymEncrypt(err, sym, iv, ZKPACKBUF_ADDR_OF(buf, BUF_HDR_V0_DSZ), bsz - FRCRYPT_IVEC_SIZE)) {
    goto done;
  }

  *dst = buf; buf = 0; /* the caller now owns the buffer */
  *dsz = bsz;

  ret = 1;

 done:

  if (buf) { memset(buf, 0, bsz); zkFree(buf); }

  return ret;
}
