/*
 *  $Id: file_v0.c,v 1.4 2001/04/19 20:30:59 alainc Exp $
 *  
 *  WARNING: this is the first, deprecated version of the encrypted buffer
 *  code.  It is being kept only while conversion to the new format is
 *  necessary.
 *  
 *  Simple decryption of data read from a file and encryption of data written
 *  to a file.
 *  
 *  The file will have a small header prepended to it, and the portion
 *  containing the data will be a multiple of 8 bytes in length.
 *  
 *  bug: add information to the header, such as version, so we can change the
 *  format of the file (e.g. the header).
 *  
 *  bug: add a hash over the data; put it in the header.
 */

#define _BSD_SOURCE 1 /* S_ISLNK(), fstat() */
#if 0
#define _POSIX_SOURCE 1 /* fileno */
#endif

#include "zks.h"

#include "zkArpaInet.h"
#include "zkSysStat.h"
#include "zkUnistd.h"
#include "zkErrno.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "frCrypt.h"
#include "fr_file_v0.h"
#include "hdr_v0.h"
#include "zkError.h"
#include "zkFile.h"
#include "zkIntTypes.h"
#include "zkLog.h"
#include "zkPackBuf.h"



static int secMemCheck(void *d, size_t dsz)
{
  unsigned char *p = d;
  size_t i;

  for (i = 0; i < dsz; i++) {
    if (p[i] != dsz) {
      return 0;
    }
  }

  return 1;
}


static void secMemFill(void *d, size_t dsz)
{
  memset(d, dsz, dsz);
}


DLLExport int fr_SecDecryptFromFp_v0(ZkError err, FILE *fp, FrCryptSym key, void **data, size_t *dataSz)
{
  char *d = 0;
  char iv[FRCRYPT_IVEC_SIZE];
  int ret = 0;
  size_t dsz = 0; /* # bytes of encrypted data, including padding, but not the header */
  size_t fsz; /* size of the file */
  char fh[BUF_HDR_V0_SIZEOF];

  zkErrorClear(err);

  if ( ! fp) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "file is null");
    return 0;
  }
  if ( ! key) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "key is null");
    return 0;
  }
  if ( ! data) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "data is null");
    return 0;
  }
  if ( ! dataSz) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dataSz is null");
    return 0;
  }

  if ( ! (zkFileIsSecure(err, zkFileno(fp)) &&
          zkFileLengthFp(err, fp, &fsz))) {
    goto done;
  }

  if (fsz < BUF_HDR_V0_SIZEOF) {
    /* Invalid file size */
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "file too short (%lu < %lu)",
                  (unsigned long)fsz, (unsigned long)BUF_HDR_V0_SIZEOF);
    goto done;
  }

  dsz = fsz - BUF_HDR_V0_SIZEOF;

  if (dsz % FRCRYPT_SIZE_MULT != 0) {
    /* Invalid file size. */
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "data size (%lu) not a multiple of %lu",
                  (unsigned long)dsz, (unsigned long)FRCRYPT_SIZE_MULT);
    goto done;
  }
  
  /* Assume the file won't change size on us... */

  if (zkFread(&fh, 1, BUF_HDR_V0_SIZEOF, fp) != BUF_HDR_V0_SIZEOF) {
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "error reading header (%lu bytes) from file: %s",
                  (unsigned long)BUF_HDR_V0_SIZEOF, zkStrerror(errno));
    goto done;
  }

  /* Decrypt that part of the header following the IV. */

  memcpy(iv, ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_IV), sizeof iv);
  if ( ! frCryptSymDecrypt(err, key, iv, ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_DSZ), BUF_HDR_V0_SIZEOF - FRCRYPT_IVEC_SIZE)) {
    goto done;
  }

  if ( ! secMemCheck(ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_PAD), BUF_HDR_V0_PAD_SIZE)) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
               "invalid padding in file");
    goto done;
  }
  
  zkPackBufSetUint32(fh, BUF_HDR_V0_DSZ, zkPackBufGetUint32Nbo(fh, BUF_HDR_V0_DSZ));

  if (zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ) > dsz) {
    /* Inconsistent header. */
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, ZKEDATABAD,
                  "inconsistent header (%lu > %lu)",
                  (unsigned long)zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ), (unsigned long)dsz);
    goto done;
  }

  if ( ! (d = zkMalloc(dsz))) {
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't malloc %lu bytes for data: %s",
                  (unsigned long)dsz, zkStrerror(errno));
    goto done;
  }

  if (zkFread(d, 1, dsz, fp) != dsz) {
    zkErrorSet_va(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't read %lu bytes of data from file: %s",
                  (unsigned long)dsz, zkStrerror(errno));
    goto done;
  }

  if ( ! frCryptSymDecrypt(err, key, iv, d, dsz)) {
    goto done;
  }

  if (zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ) < dsz && ! secMemCheck(d + zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ), dsz - zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ))) {
    zkErrorSet(err, "fr_SecDecryptFromFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
               "invalid trailing padding in file");
    goto done;
  }

  *data = d; d = 0;
  *dataSz = zkPackBufGetUint32(fh, BUF_HDR_V0_DSZ);
  
  ret = 1;

 done:

  if (d) {
    memset(d, 0, dsz);
    zkFree(d);
  }

  return ret;
}


DLLExport int fr_SecDecryptFromFile_v0(ZkError err, const char *file, FrCryptSym key, void **data, size_t *dataSz)
{
  FILE *fp = 0;
  ZkLog log;
  int ret = 0;

  zkErrorClear(err);
  zkErrorLog(err, &log);

  if ( ! file) {
    zkErrorSet(err, "fr_SecDecryptFromFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "file is null");
    return 0;
  }
  if ( ! key) {
    zkErrorSet(err, "fr_SecDecryptFromFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "key is null");
    return 0;
  }
  if ( ! data) {
    zkErrorSet(err, "fr_SecDecryptFromFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "data is null");
    return 0;
  }
  if ( ! dataSz) {
    zkErrorSet(err, "fr_SecDecryptFromFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "dataSz is null");
    return 0;
  }

  if ( ! (fp = zkFopen(file, "rb"))) {
    zkErrorSet_va(err, "fr_SecDecryptFromFile_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't open `%s' for reading: %s", file, zkStrerror(errno));
    goto done;
  }

  if ( ! (ret = fr_SecDecryptFromFp_v0(err, fp, key, data, dataSz))) {
    zkLogWrite_va(log, "can't decrypt from `%s'", file);
  }

 done:

  if (fp) { zkFclose(fp); }

  return ret;
}


DLLExport int fr_SecEncryptToFp_v0(ZkError err, FILE *fp, FrCryptSym key, void *data, size_t dataSz)
{
  char *d = 0;
  char iv[FRCRYPT_IVEC_SIZE];
  int ret = 0;
  size_t dsz;
  char fh[BUF_HDR_V0_SIZEOF];

  zkErrorClear(err);

  if ( ! fp) {
    zkErrorSet(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "fp is null");
    return 0;
  }
  if ( ! key) {
    zkErrorSet(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "key is null");
    return 0;
  }
  if ( ! data) {
    zkErrorSet(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "data is null");
    return 0;
  }
  if (dataSz == 0) {
    zkErrorSet(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, ZKEPARMBAD,
               "dataSz is 0");
    return 0;
  }

  dsz = FRCRYPT_SIZE_CEIL(dataSz); /* amount to allocate and encrypt */

  /* Set up the file header. */

  if ( ! zkRandGetData(err, ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_IV), FRCRYPT_IVEC_SIZE)) {
    goto done;
  }
  memcpy(iv, ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_IV), sizeof iv);
  zkPackBufSetUint32Nbo(fh, BUF_HDR_V0_DSZ, dataSz);
  secMemFill(ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_PAD), BUF_HDR_V0_PAD_SIZE);

  if ( ! (d = zkMalloc(dsz))) {
    zkErrorSet_va(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't malloc %lu bytes for data: %s",
                  (unsigned long)dsz, zkStrerror(errno));
    goto done;
  }

  /* Encrypt that part of the header following the IV. */

  if ( ! frCryptSymEncrypt(err, key, iv,
                           ZKPACKBUF_ADDR_OF(fh, BUF_HDR_V0_DSZ), BUF_HDR_V0_SIZEOF - FRCRYPT_IVEC_SIZE)) {
    goto done;
  }

  memcpy(d, data, dataSz);
  if (dataSz < dsz) {
    secMemFill(d + dataSz, dsz - dataSz);
  }

  /* Continue encrypting the actual data to be stored. */

  if ( ! frCryptSymEncrypt(err, key, iv, d, dsz)) {
    goto done;
  }

  if ( ! (zkFwrite(&fh, 1, BUF_HDR_V0_SIZEOF, fp) == BUF_HDR_V0_SIZEOF &&
          zkFwrite(d, 1, dsz, fp) == dsz)) {
    zkErrorSet_va(err, "fr_SecEncryptToFp_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "error writing to file: %s", zkStrerror(errno));
    goto done;
  }

  ret = 1;

 done:

  if (d) {
    memset(d, 0, dsz);
    zkFree(d);
  }

  return ret;
}


/*
 *  Store an encrypted copy of the contents of `data' in `file'.  Any existing
 *  contents of `file' are overwritten.
 */
DLLExport int fr_SecEncryptToFile_v0(ZkError err, const char *file, FrCryptSym key, void *data, size_t dataSz)
{
  FILE *fp = 0;
  ZkLog log;
  int ret = 0;
  mode_t omask;

  zkErrorClear(err);
  zkErrorLog(err, &log);

  if ( ! file) {
    zkErrorSet(err, "fr_SecEncryptToFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "file is null");
    return 0;
  }
  if ( ! key) {
    zkErrorSet(err, "fr_SecEncryptToFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "key is null");
    return 0;
  }
  if ( ! data) {
    zkErrorSet(err, "fr_SecEncryptToFile_v0", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "data is null");
    return 0;
  }
  if (dataSz == 0) {
    zkErrorSet(err, "fr_SecEncryptToFile_v0", ZKERR, ZKMFRSEC, ZKEPARMBAD,
               "dataSz is 0");
    return 0;
  }

  omask = zkUmask(S_IRWXG | S_IRWXO);

  /* Open the file, then write everything. */

  if ( ! (fp = zkFopen(file, "wb"))) {
    zkErrorSet_va(err, "fr_SecEncryptToFile_v0", ZKERR, ZKMFRSEC, (unsigned short) errno,
                  "can't open `%s' for writing: %s", file, zkStrerror(errno));
    goto done;
  }

  /* bug: set permissions, check links, etc. */
  zkFprintf(stderr, "**** set permissions, check links, etc. ****\n");

  if ( ! (ret = fr_SecEncryptToFp_v0(err, fp, key, data, dataSz))) {
    zkLogWrite_va(log, "can't decrypt from `%s'", file);
  }

  ret = 1;

 done:

  zkUmask(omask);
  if (fp && zkFclose(fp) == EOF) {
    zkLogWrite_va(log, "error closing `%s': %s", file, zkStrerror(errno));
    ret = 0;
  }

  return ret;
}
