/*
 *  $Id: rand.c,v 1.4 2001/04/19 20:31:01 alainc Exp $
 */

#include "zks.h"

#include "zkErrno.h"
#include <limits.h>
#include "zkStdio.h"
#include "zkString.h"
#include "frsec_rand.h"
#include "frCrypt.h"
#include "zkError.h"


DLLExport int frSecSeedFromRandomSource(ZkError err, char *path, size_t nbits)
{
  FILE *fp = 0;
  int c, ret = 0;
  size_t n, nbytes = nbits / CHAR_BIT + 1;
  
  zkErrorClear(err);

  if ( ! path) {
    zkErrorSet(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, ZKENULLPTR,
               "path is null");
    return 0;
  }
  if (nbits == 0) {
    zkErrorSet(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, ZKEPARMBAD,
               "nbits is 0");
    return 0;
  }
  
  if ( ! (fp = zkFopen(path, "rb"))) {
    zkErrorSet_va(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, errno,
                  "can't open `%s' for reading: %s", path, zkStrerror(errno));
    goto done;
  }

  for (n = 0; (c = zkFgetc(fp)) != EOF && n < nbytes; n++) {
    unsigned char uc = c;
    (void)zkRandEntropyAdd(err, &uc, sizeof uc);
  }

  if (c == EOF) {
    zkErrorSet_va(err, "frSecSeedFromRandomSource", ZKERR, ZKMFRSEC, errno,
                  "error reading from `%s': %s", path, zkStrerror(errno));
    goto done;
  }
  
  ret = 1;

 done:

  if (fp) { zkFclose(fp); }

  return ret;
}
