/*
 *   Force the user to type inputs that will be sent to the PRNG.
 *
 *   $Id: fr_seed.c,v 1.3 2001/04/19 20:31:01 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 */

#ifdef WIN32
#error "There's nothing here for you!"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h> /* for getopt */

#include "frSec.h"
#include "frCrypt.h"
#include "zkError.h"

#define DEBUG 0

static void usage(void)
{
  fprintf( stderr, "usage: fr_seed [-b N]\n" );
  fprintf( stderr, "\n" );
  fprintf( stderr, "\t-b The program will ask for an estimated N bits of entropy to the user.\n"
                   "\t   The default value is the maximum value that the randomness device can hold.\n" );
}

int main(int argc, char **argv)
{
  ZkError err = 0;
  char *prog = argv[0];
  int ok = 0;
  int maxbits, nbits;
  int opt;

  if ( ! zkErrorNew(&err)) {
    fprintf(stderr, "%s: can't allocate error object -- exiting.\n", prog);
    goto done;
  }
  
  /* to be as conservative as possible, we require the maximum amount
   * of entropy from the user.
   *
   * bug: may be to long. If using /dev/random, it took about 30 sec. to
   *      fill the pool.
   */
  if ( ! frSecMaxEntropy(err, &maxbits)) {
    fprintf(stderr, "%s: can't get maximum entropy value -- exiting.\n", prog);
    goto done;
  }
  nbits = maxbits;
  
#if DEBUG
  fprintf(stdout, "Max entropy value: %d\n", maxbits);
#endif
  
  /* read the arguments */
  while ( ( opt = getopt( argc, argv, "b:" ) ) >= 0 ) {
    switch( opt ) {
    case 'b': nbits = atoi( optarg ); break;
    default: usage(); goto done;
    }
  }
  
  if (nbits <= 0) {
    fprintf(stderr, "%s: Number of entropy bits must be positive.\n", prog);
    goto done;
  }
  else if (nbits > maxbits) {
    /* no need to ask for more */
    nbits = maxbits;
  }

#if DEBUG
  fprintf(stdout, "nbits: %d\n", nbits);
#endif
  
  /* hook the randomness input to the right device */
  if ( ! frCryptInit(err, 0)) {
    fprintf(stderr, "%s: can't initialize crypto routines.\n", prog);
    goto done;
  }
  
  printf("Random bits are needed to seed the pseudo random number generator.\n"
	 "We'll collect %d estimated bits of entropy.\n", nbits);
  
  if ( ! frSecSeedFromUser(err, stdout, nbits)) {
    fprintf(stderr, "%s: error extracting random data from user input.\n", prog);
    goto done;
  }
    
  ok = 1;

 done:
    exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
