/**
 * $Id: tkc.c,v 1.7 2001/04/19 20:31:03 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 * 
 * Testing keychain.c functions.
 */
#include "zkStdio.h"
#include <assert.h>
#include <signal.h>
#include "zkUnistd.h"
#include "frSec.h"
#include "zkSignal.h"

#if !defined(ZK_CLIENT)

static volatile sig_atomic_t reload = 0;

static void sig_handler(int sig)
{
  switch (sig) {
  case SIGUSR1:
    reload = 1;
    break;

  default:
    zkFprintf(stderr, "Doing nothing\n");
  }
}
	
#endif	/* defined(ZK_CLIENT) */

int main(int argc, char **argv)
{
#if !defined(ZK_CLIENT)

  ZkError err = NULL;
  const char *cryptDir = "/tmp/crypt";
  const char *sigDir = "/tmp/sig";
  const char *prompt = "Enter system passphrase: ";
  const char *passfile = NULL;
  size_t sigHandle;
  size_t cryptHandle;
  
  int fd = -1;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;
  FrSecKeyChain kc = NULL;
  FrCryptPubKey key = NULL;

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, 0));

  /* Test 1 - only signature keys */
  assert(frSecKeyChainNew(err, &kc));
  assert(frSecKeyChainAddSymKey(err, kc, passfile, prompt, alg));
  assert(frSecKeyChainRegisterDir(err, kc, &sigHandle, sigDir));

  zkSignal(SIGUSR1, sig_handler);
  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainGetVersion(err, kc, sigHandle, 0, &key));
  assert(key);
  assert(frSecKeyChainGetVersion(err, kc, sigHandle, 1, &key));
  assert(key);
  assert(!frSecKeyChainGetVersion(err, kc, sigHandle + 1, 0, &key));

  zkFprintf(stderr, "Sleeping...send SIGUSR1 to %d\n", zkGetpid());
  while (!reload) ;

  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainFree(err, &kc));

  /* Test 2 - only cryption keys */
  kc = NULL;
  assert(frSecKeyChainNew(err, &kc));
  assert(frSecKeyChainAddSymKey(err, kc, passfile, prompt, alg));
  assert(frSecKeyChainRegisterDir(err, kc, &cryptHandle, cryptDir));

  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainGetVersion(err, kc, cryptHandle, 0, &key));
  assert(key);
  assert(frSecKeyChainGetVersion(err, kc, cryptHandle, 1, &key));
  assert(key);
  assert(!frSecKeyChainGetVersion(err, kc, cryptHandle + 1, 0, &key));

  zkFprintf(stderr, "Sleeping...send SIGUSR1 to %d\n", zkGetpid());
  reload = 0;
  while (!reload) ;

  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainFree(err, &kc));

  /* Test 3 - both cryption and signature keys */
  kc = NULL;
  assert(frSecKeyChainNew(err, &kc));
  assert(frSecKeyChainAddSymKey(err, kc, passfile, prompt, alg));
  assert(frSecKeyChainRegisterDir(err, kc, &sigHandle, sigDir));
  assert(frSecKeyChainRegisterDir(err, kc, &cryptHandle, cryptDir));

  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainGetVersion(err, kc, cryptHandle, 0, &key));
  assert(key);
  assert(frSecKeyChainGetVersion(err, kc, sigHandle, 1, &key));
  assert(key);
  assert(frSecKeyChainGetVersion(err, kc, sigHandle, 0, &key));
  assert(key);
  assert(frSecKeyChainGetVersion(err, kc, cryptHandle, 1, &key));
  assert(key);

  zkFprintf(stderr, "Sleeping...send SIGUSR1 to %d\n", zkGetpid());
  reload = 0;
  while (!reload) ;

  assert(frSecKeyChainReloadKeys(err, kc));

  assert(frSecKeyChainFree(err, &kc));

#else	/* !defined(ZK_CLIENT) */

  (void) argc;
  (void) argv;
	
#endif	/* defined(ZK_CLIENT) */

  zkPrintf(".\n");

  return (EXIT_SUCCESS);
}
