/*
 *  $Id: tks.c,v 1.6 2001/04/19 20:31:03 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test the encrypted key storage routines.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "fr_keystore.h"
#include "zkShell.h"


static const char *prog;


#if !defined(ZK_CLIENT)

static void t(ZkError err, const char *file)
{
  FrCryptCrypt crp0, crp1;
  FrCryptSig sig0, sig1;
  FrCryptSym sym;
  int equal;

  assert(frCryptSymNew(err, &sym));
  assert(frCryptSymSetAlg(err, sym, FRCRYPT_SYMALG_BLOWFISH));
  assert(frCryptSymRandomKey(err, sym));

  /* Cryption */

  assert(frCryptCryptNew(err, &crp0));
  assert(frCryptPubKeySetVersion(err, crp0, 5));
  assert(frCryptCryptSetKeySize(err, crp0, FRCRYPT_CRYPTKEYSIZE_1024));
  assert(frCryptCryptSetRandomKeys(err, crp0));

  assert(frSecCryptPrivWriteToFile(err, file, sym, crp0));
  assert(frSecCryptNewFromFile(err, file, sym, &crp1));

  assert(frCryptCryptEqual(err, crp0, crp1, &equal));
  assert(equal);
  
  assert(frCryptCryptFree(err, &crp0));
  assert(frCryptCryptFree(err, &crp1));

  /* Signature */

  assert(frCryptSigNew(err, &sig0));
  assert(frCryptPubKeySetVersion(err, sig0, 5));
  assert(frCryptSigSetKeySize(err, sig0, FRCRYPT_SIGKEYSIZE_1024));
  assert(frCryptSigSetRandomKeys(err, sig0));

  assert(frSecSigPrivWriteToFile(err, file, sym, sig0));
  assert(frSecSigNewFromFile(err, file, sym, &sig1));

  assert(frCryptSigEqual(err, sig0, sig1, &equal));
  assert(equal);
  
  assert(frCryptSigFree(err, &sig0));
  assert(frCryptSigFree(err, &sig1));


  assert(frCryptSymFree(err, &sym));
}

#endif	/* !defined(ZK_CLIENT) */


static void usage(void)
{
  zkFprintf(stderr, "Usage: %s <temp-file>\n", prog);
  exit(EXIT_FAILURE);
}


int main(int ac, char **av)
{
#if !defined(ZK_CLIENT)

  ZkError err;
  
  prog = zkShellGetProgramName(&ac, &av);

  if (ac != 2) {
    usage();
  }

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, 0));
  while (1) {
    t(err, av[1]);
  }
  assert(zkErrorFree(&err));

#else	/* !defined(ZK_CLIENT) */

  (void) ac;
  (void) av;
	
#endif	/* defined(ZK_CLIENT) */
  
  return (EXIT_SUCCESS);
}
