/*
 *  ($Id: tpass.c,v 1.8 2001/04/19 20:31:03 alainc Exp $)
 *  
 *  Test creating an encryption key from a user supplied passphrase.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "frSec.h"
#include "zkShell.h"


int main(int ac, char **av)
{
  FrCryptSym sd, se; /* decryption and encryption keys */
  FrCryptSymAlg a = FRCRYPT_SYMALG_DESX;
  ZkError e;
  char emsg[512], iv[FRCRYPT_IVEC_SIZE];
  char odata[1024], ddata[1024], edata[1024];
  char prompt0[] = "Enter passphrase: ";
  char prompt1[] = "Enter previous passphrase: ";
  const char* prog;

  prog = zkShellGetProgramName(&ac, &av);

  assert(zkErrorNew(&e));
  assert(frCryptInit(e, 0));
  
  /* Get the passphrase. */

  if ( ! frSecNewSymFromPass(e, &se, a, 0, prompt0, 1)) {
    assert(zkErrorMessage(e, emsg, sizeof emsg));
    zkFprintf(stderr, "%s: %s\n", prog, emsg);
    exit(EXIT_FAILURE);
  }

  /* Generate original, random data. */

  assert(zkRandGetData(e, odata, sizeof odata));

  /* Encrypt it. */

  memset(iv, 0, sizeof iv);
  memcpy(edata, odata, sizeof edata);
  assert(frCryptSymEncrypt(e, se, iv, edata, sizeof edata));

  /* Get the passphrase again. */

  if ( ! frSecNewSymFromPass(e, &sd, a, 0, prompt1, 1)) {
    assert(zkErrorMessage(e, emsg, sizeof emsg));
    zkFprintf(stderr, "%s: %s\n", prog, emsg);
    exit(EXIT_FAILURE);
  }

  /* Decrypt it. */

  memset(iv, 0, sizeof iv);
  memcpy(ddata, edata, sizeof ddata);
  assert(frCryptSymDecrypt(e, sd, iv, ddata, sizeof ddata));

  /* Compare with the original data. */

  assert(memcmp(ddata, odata, sizeof ddata) == 0);

  assert(frCryptSymFree(e, &sd));
  assert(frCryptSymFree(e, &se));
  assert(zkErrorFree(&e));

  return (EXIT_SUCCESS);
}
