/*
 *  $Id: tsig_v2.c,v 1.3 2001/04/19 20:31:03 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test signature header routines.
 */

#include "zks.h"

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkTime.h"
#include "frSec.h"
#include "frsec_sig.h"
#include "zkPackBuf.h"
#include "zkString.h"

/* structure copied from frsec_sig_v2.c */

#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)

struct frsec_sig_pk {
  uint8_t vers;
  uint8_t type;
  char sig[FRCRYPT_SIG_SIZE];
  char signer[FRENT_PORTABLE_SIZE];
  uint32_t dataLen;
  uint32_t extLen;
};

#pragma pack()
#endif

enum {
	FRSEC_SIG_PK_V2_VERS	= 0,
	FRSEC_SIG_PK_V2_TYPE	= FRSEC_SIG_PK_V2_VERS    + sizeof(uint8_t),
	FRSEC_SIG_PK_V2_SIG		= FRSEC_SIG_PK_V2_TYPE    + sizeof(uint8_t),
	FRSEC_SIG_PK_V2_SIGNER	= FRSEC_SIG_PK_V2_SIG     + FRCRYPT_SIG_SIZE,
	FRSEC_SIG_PK_V2_DATALEN	= FRSEC_SIG_PK_V2_SIGNER  + FRENT_PORTABLE_SIZE,
	FRSEC_SIG_PK_V2_EXTLEN	= FRSEC_SIG_PK_V2_DATALEN + sizeof(uint32_t),
	FRSEC_SIG_PK_V2_SIZEOF	= FRSEC_SIG_PK_V2_EXTLEN  + sizeof(uint32_t)
};


/*
 *  Generate a new, random key.
 *  
 *  bug: stolen from kgdemo.c.
 */
static int newKey(ZkError err, FrEntIdExp *owner, FrPubKeyType kt, int kv, int ks, ZkClock ke, FrCryptPubKey *newkey)
{
  FrCryptPubKey pk = 0;
  int ret = 0;

  /*
   *  Create a new key object, and set various required parameters.
   */
  
  if ( ! (frCryptPubKeyNew(err, &pk) &&
          frCryptPubKeySetType(err, pk, kt) &&
          frCryptPubKeySetSize(err, pk, ks) &&
          frCryptPubKeySetVersion(err, pk, kv) &&
          frCryptPubKeySetExpiration(err, pk, ke) &&
          frCryptPubKeySetEntity(err, pk, owner))) {
    goto done;
  }

  /*
   *  Generate the bignum parameters.
   */
  
  switch (kt) {
  case FRCRYPT_KEYTYPE_CRYPTION:
    if ( ! (frCryptCryptMakeCrypt(err, pk) &&
            frCryptCryptSetRandomKeys(err, pk))) {
      /* log error */
      goto done;
    }
    break;
    
  case FRCRYPT_KEYTYPE_SIGNING:
    if ( ! (frCryptSigMakeSig(err, pk) &&
            frCryptSigSetRandomKeys(err, pk))) {
      /* log error */
      goto done;
    }
    break;

  default:
    /* log error */
    goto done;
  }

  ret = 1;
  *newkey = pk; pk = 0; /* caller now owns the key */

 done:

  if (pk) { frCryptPubKeyFree(err, &pk); }

  return ret;
}


static void t(ZkError err, FrCryptSig sigkey, void *data, size_t dsz, uint32_t len)
{
  int valid;
  size_t i;
  size_t newdsz, ssz;
  void *newdata, *sigbuf;
  uint8_t *test;
  uint8_t junk = 0;
  uint8_t *ptr;
  uint32_t extLen;

  for (i = 0; i < dsz; i++) { *((char *)data + i) = rand(); }

  /* Test signing with v2 extension and verification. */

  assert(frSecSignData_v2(err, sigkey, data, dsz, &sigbuf, &ssz, len));

  /* check extension data copies across */

  extLen = zkPackBufGetUint32Nbo(sigbuf, FRSEC_SIG_PK_V2_EXTLEN);
  assert( extLen == len );
  test = (uint8_t*) zkMalloc( extLen );

  /* magic string used as test extension data */

  for (i = 0; i < extLen; i++, junk++) { test[i] = junk; }
  assert(memcmp( test, ZKPACKBUF_CONST_ADDR_OF(sigbuf, FRSEC_SIG_PK_V2_SIZEOF), 
		 extLen ) == 0);
  assert(frSecSigVerify(err, sigkey, sigbuf, ssz, &valid));
  assert(valid == 1);

  if (extLen > 0) {
    /* check extension data is verified by modifying it and checking
     * signature fails
     */
    ptr = (uint8_t*)ZKPACKBUF_CONST_ADDR_OF(sigbuf, FRSEC_SIG_PK_V2_SIZEOF);
    *ptr = '\1';			/* change extension data */
    assert(frSecSigVerify(err, sigkey, sigbuf, ssz, &valid));
    assert(valid == 0);
    *ptr = '\0';			/* change extension data back */
  } else {
    zkPackBufSetUint32Nbo(sigbuf, FRSEC_SIG_PK_V2_EXTLEN, 1);
    			   /* can't modify the data because
				   there isn't any, so modify
				   extLen which should also
				   be part of the sig */
    assert(frSecSigVerify(err, sigkey, sigbuf, ssz, &valid));
    assert(valid == 0);
    zkPackBufSetUint32Nbo(sigbuf, FRSEC_SIG_PK_V2_EXTLEN, 0);	/* change extLen back */
  }

  /* test extraction of signer entity. */

  {
    int equal;
    FrEntIdExp kent, sent;
    
    assert(frCryptPubKeyGetEntity(err, sigkey, &kent));
    assert(frSecSigGetEntity(err, sigbuf, ssz, &sent));
    assert(frEntEqual(err, &kent, &sent, &equal));
    assert(equal == 1);
  }

  /* Test removal of signature header and extension data. */

  assert(frSecSigRemove(err, sigbuf, ssz, &newdata, &newdsz));
  assert(newdsz == dsz);
  assert(memcmp(newdata, data, newdsz) == 0);

  /* Test removal of signature header and extension data, in place. */

  assert(frSecSigRemoveInSitu(err, sigbuf, &ssz));
  assert(ssz == dsz);
  assert(memcmp(sigbuf, data, ssz) == 0);

  zkFree(sigbuf); zkFree(newdata); zkFree(test);
}


int main(void)
{
  FrCryptSig sigkey;
  FrEntIdExp owner;
  ZkError err;
  char data[1024];
  size_t i;

  srand(time(0));

  assert(zkErrorNew(&err));
  assert(frCryptInit(err, 0));

  for (i = 0; i < 10; i++) {
    size_t dsz = rand() % sizeof data;
    
    fr_EntSetRandom(&owner);
    assert(newKey(err, &owner, FRCRYPT_KEYTYPE_SIGNING, 3, 1024, zkClockFromSeconds(123456789U), &sigkey));
    /* test v2 extension */
    t(err, sigkey, data, dsz, 256); /* multiple of 8 sized extension */
    t(err, sigkey, data, dsz, 123); /* odd sized extension */
    t(err, sigkey, data, dsz, 0);  /* 0 sized extension */
    assert(frCryptSigFree(err, &sigkey));
  }

  assert(frCryptFinal(err, 0));
  assert(zkErrorFree(&err));
  
  zkPrintf(".\n");
  
  return (EXIT_SUCCESS);
}
