/*
 *  $Id: tstretch.c,v 1.6 2001/04/19 20:31:03 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Gives the number of iterations in the stretch algorith that has been
 *  executed for a certain amount (given as an argument) of seconds.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkTime.h"
#include "frSec.h"
#include "zkShell.h"

static const char *prog;

static void usage(int ok)
{
  zkFprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--password <string>]"
          " [--logiterations <int>]"
          "\n",
          prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  int ok = 0;
  ZkError err;
  ZkLog log;
  FrCryptSym s = 0;
  FrCryptSymAlg alg = FRCRYPT_SYMALG_BLOWFISH;
  char* pw = 0;
  char* pwLong = 0;
  size_t szPwLong = 0;
  int lgiter = 0;
  clock_t start, finish;

  prog = zkShellGetProgramName(&ac, &av);

  assert(zkErrorNew(&err));

  if ( ! frCryptInit(err, 0)) {
    zkFprintf(stderr, "%s: can't initialize crypto library -- exiting.\n", prog);
    goto done;
  }

  while (av++, --ac) {
    if (strcmp("-h",      av[0]) == 0 ||
        strcmp("-help",   av[0]) == 0 ||
	strcmp("--help",  av[0]) == 0    ) {
      usage(1);
    }
    else if (strcmp("--password", av[0]) == 0) {
      if (av++, --ac < 1) {usage(0); }
      pw = av[0];
    }
    else if (strcmp("--logiterations", av[0]) == 0) {
      if (av++, --ac < 1) {usage(0); }
      lgiter = atoi(av[0]); /* bug: atoi() does not return over/underflow errors. */
   }
  }
  if ( ! pw) {
    zkFprintf(stderr, "%s: required argument %s is missing.\n", prog, "--password");
    usage(0);
  }
  if (lgiter <= 0) { 
    zkFprintf(stderr, "%s: required argument %s is missing or invalid (should be > 1)", prog, "--logiterations");
    usage(0);
  }

  if ( ! (zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetStream(log, stderr) &&
          zkLogEnable(log))) {
    zkFprintf(stderr, "%s: can't set log object -- exiting.\n", prog);
    goto done;
  }

  
  start = clock();

  if ( ! frSecStretchDataToMem(err, lgiter, pw, strlen(pw), (void *)&pwLong, &szPwLong)){
    goto done; 
  };
  
  finish = clock();

  zkFprintf(stderr, "Total time in sec: %f\n", ((double) (finish - start))/CLOCKS_PER_SEC);

  zkFprintf(stderr, "\nNow testing call to frSecNewSymFromStretchedPassPhrase...\n");

  /* testing call to frSecNewSymFromStretchedPassPhrase */
  assert(frSecNewSymFromStretchedPassPhrase(err, &s, alg, lgiter, (char *)pw));


  zkFprintf(stderr, "Return from call was o.k.\n");


  ok = 1;

 done:

  if ( pwLong ) zkFree(pwLong);
  
  return (ok ? EXIT_SUCCESS : EXIT_FAILURE);

}
