/*
 * $Id: frStatUpd.h,v 1.14 2000/11/23 15:13:25 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRSTATUPD_H_ 
#define FRSTATUPD_H_

#include <zks.h>
#include <zkIntTypes.h>
#include <frEnt.h>
#include <zk_fr_stat_defs.h>

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

#define FRSTAT_DEFAULT_FLAGS    0
#define FRSTAT_DEFAULT_HISTORY  1
#define FRSTAT_DEFAULT_SIGNATURE  0x0f0f0000
#define FRSTAT_DEFAULT_VERSION    0x01
#define FRSTAT_SIGNATURE_MASK     0xffff0000
#define FRSTAT_DEFAULT_SIGVERS  (FRSTAT_DEFAULT_SIGNATURE | FRSTAT_DEFAULT_VERSION)
  
typedef char FrStatUpdReleaseString[FRSTATUPD_RELEASE_STRING_SZ];

typedef enum {
  FRSTATUPD_BYTES_L_SENT = 0 ,
  FRSTATUPD_BYTES_L_RCVD = 1 , /* Includes dropped data */
  FRSTATUPD_BYTES_L_DROP = 2 ,
  FRSTATUPD_BYTES_H_SENT = 3 ,
  FRSTATUPD_BYTES_H_RCVD = 4 , /* Includes dropped data */
  FRSTATUPD_BYTES_H_DROP = 5 ,

  FRSTATUPD_PKTS_L_SENT = 6 ,
  FRSTATUPD_PKTS_L_RCVD = 7 ,
  FRSTATUPD_PKTS_L_DROP = 8 ,
  FRSTATUPD_PKTS_H_SENT = 9 ,
  FRSTATUPD_PKTS_H_RCVD = 10 ,
  FRSTATUPD_PKTS_H_DROP = 11 ,

  FRSTATUPD_CONNECTIONS = 12 ,
  FRSTATUPD_LAUNCHTIME = 13 ,
  FRSTATUPD_SEQNO = 14 ,
  FRSTATUPD_LNKSTAT = 15 ,
  FRSTATUPD_NIQS_RELOAD = 16 ,
  FRSTATUPD_LAST_STATUS = 17 ,
  FRSTATUPD_CURR_CONNECTS = 18 ,
  FRSTATUPD_CURR_PORTS = 19 ,
  FRSTATUPD_CONSTANT = 20 ,
  FRSTATUPD_AIP_NEIGHB = 21 ,
  FRSTATUPD_CURRTIME = 22 ,
  FRSTATUPD_TOTAL_PARMS
} FrStatUpdParameter;

typedef uint32_t FrStatUpdParm;

typedef enum {
  FRSTAT_T_UNUSED = 0,
  FRSTAT_T_AIP = 1,
  FRSTAT_T_MAIP = 2,
  FRSTAT_T_LINK = 3,
  FRSTAT_T_WORMHOLE = 4,
  FRSTAT_T_TOTAL_TYPES
} FrStatUpdTypes;
  
typedef struct T_FrStatUpd  * FrStatUpd;

DLLExport int frStatUpdNew (
  ZkError err ,
  FrStatUpd * fsu );

DLLExport int frStatUpdFree (
  ZkError err ,
  FrStatUpd * fsu );

/*
 * Get the packets signature and version
 */
DLLExport int frStatUpdGetSigVers (
  ZkError err ,
  FrStatUpd fsu ,
  uint32_t *sig,
  uint32_t *vers);

DLLExport int frStatUpdGetHistorySize (
  ZkError err ,
  FrStatUpd fsu ,
  size_t *history );

DLLExport const char *frStatUdpGetTypeString (
   ZkError err,
   FrStatUpdParameter p);


DLLExport const char * frStatUdpGetParameterString (
  ZkError err,
  FrStatUpdParameter p);
  
DLLExport int frStatUpdClearHistory (
  ZkError err ,
  FrStatUpd fsu );

DLLExport int frStatUpdSetType (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdTypes type );

DLLExport int frStatUpdSetNbEntity (
  ZkError err ,
  FrStatUpd fsu ,
  FrEntIdExp * nbentity );

DLLExport int frStatUpdSetEntity (
  ZkError err ,
  FrStatUpd fsu ,
  FrEntIdExp * entity );

DLLExport int frStatUpdSetEntity2 (
  ZkError err ,
  FrStatUpd fsu ,
  const char * name ,
  FrEntIdType type );

DLLExport int frStatUpdSetNbEntity2 (
  ZkError err ,
  FrStatUpd fsu ,
  const char * name ,
  FrEntIdType type );

DLLExport int frStatUpdEntity (
  ZkError err ,
  FrStatUpd fsu ,
  const FrEntIdExp ** entity );

DLLExport int frStatUpdType (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdTypes *type );

DLLExport int frStatUpdNbEntity (
  ZkError err ,
  FrStatUpd fsu ,
  const FrEntIdExp ** nbentity );

/**
 * Set the release string to the stat update object.
 */
DLLExport int frStatUpdSetReleaseString(ZkError err, FrStatUpd fsu, 
                                        const char *relString, size_t sz);
  

/**                                         
 * Get the release string.
 */
DLLExport int frStatUpdGetReleaseString(ZkError err, FrStatUpd fsu, 
                                        char **relString, size_t *sz);


DLLExport int frStatUpdHistory (
  ZkError err ,
  FrStatUpd  fsu ,
  size_t * h);

DLLExport int frStatUpdIncreaseParameter (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdParameter p ,
  int32_t v );

DLLExport int frStatUpdGetParameter (
  ZkError err ,                 /* Returns the error if one occured */
  FrStatUpd fsu ,               /* Stat structure */
  FrStatUpdParameter parm ,     /* Requested parameter */
  size_t h ,                    /* How far back (0 = most recent) */
  FrStatUpdParm * val ,         /* Returns the value */
  time_t * t);                  /* Returns absolute time when recorded */

DLLExport int frStatUpdSetParameter (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdParameter parm ,
  FrStatUpdParm v);

DLLExport int frStatUpdSetHistory (
  ZkError err ,
  FrStatUpd fsu ,
  size_t h);

DLLExport int frStatUpdToBuffer (
  ZkError err ,
  FrStatUpd fsu ,
  void ** b ,
  size_t *sz );

DLLExport int frStatUpdFromBuffer (
  ZkError err ,
  FrStatUpd * fsu ,
  void * b ,
  size_t sz);

DLLExport int frStatUpdUpdate (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpd upd );


/*
 * This is a debugging function
 */

DLLExport int frStatUpdDump (
  ZkError err ,
  FrStatUpd fsu ,
  int maxhistory , /* set to -1 to get all entries, 0 for the latest */
  FILE * stream );

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FRSTATUPD_H_ */
