/*
 * $Id: frstats.h,v 1.17 2000/11/23 15:13:25 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRSTATS_H_ 
#define FRSTATS_H_

#include "zks.h"
#include "zkIntTypes.h"
#include "frEnt.h"
#include "frStatUpd.h"
#include "frStatPkt.h"
#include "frStatClient.h"

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

  typedef struct T_FrStatsCtx *FrStatsCtx;
  
  extern int frStatsNew(
    ZkError err,
    FrStatsCtx *ctx);

  extern int frStatsNewWithNoSU(
    ZkError err,
    FrStatsCtx *ctx);

  extern int frStatsFree(
    ZkError err,
    FrStatsCtx *ctx);

  extern int frStatsIsServer(
    ZkError err, 
    FrStatsCtx ctx, 
    const char *hname, 
    unsigned short port, 
    int *same);

  extern int frStatsUpdateServer(
    ZkError err, 
    FrStatsCtx ctx, 
    const char *hname,
    unsigned short port);

  /* Client Side Administrative details */
  extern int frStatsSetServer(
    ZkError,
    FrStatsCtx,
    const char *hostname, 
    unsigned short port);

  extern int frStatsGetServer(
    ZkError err, 
    FrStatsCtx ctx, 
    int *pSock,
    struct sockaddr *pSockAddr,
    socklen_t *pAddrSize);

  extern int frStatsSetMyEntity(
    ZkError err, 
    FrStatsCtx ctx, 
    FrEntIdExp myEntity);

  extern int frStatsSetMyInfo(
    ZkError err,
    FrStatsCtx ctx,
    FrEntIdExp myEntity,
    const char *relString,
    size_t relstr_sz,
    FrCryptPubKey mySigKey);

  extern int frStatsSetMyCurrentKey(
    ZkError,
    FrStatsCtx,
    FrCryptPubKey myCurrentKey);

  int frStatsUpdSetType (
    ZkError err ,
    FrStatsCtx ctx, 
    FrStatUpdTypes type );

  extern int frStatsSetNISSCurrentCryptKey(
    ZkError err,
    FrStatsCtx ctx, 
    FrCryptPubKey nissPubKey);

  extern int frStatsSetHistory(
    ZkError,
    FrStatsCtx,
    size_t);

  /* Fill status updates with the various updates we have */
  extern int frStatsFillAipMsg(
    ZkError err, 
    FrStatsCtx ctx, 
    uint32_t sequence,
    time_t launchtime,
    time_t last_niqs_reload_time, 
    time_t last_status_time,
    uint32_t new_client_connections,
    uint32_t curr_connected_clients,
    UINT64_T client_pkts_rx,
    UINT64_T client_pkts_tx,
    UINT64_T client_pkts_dropped,
    UINT64_T client_bytes_rx,
    UINT64_T client_bytes_tx,
    UINT64_T client_bytes_dropped
    );

  extern int frStatsFillWormholeMsg(
    ZkError err,
    FrStatsCtx ctx,
    uint32_t sequence,
    time_t launchtime,
    time_t last_status_time,
    uint32_t maxin_thrput,
    uint32_t new_streams,
    uint32_t current_active_streams,
    uint32_t new_ports,
    uint32_t curr_used_ports,
    UINT64_T new_bytes_rx,
    UINT64_T new_bytes_tx);

  extern int frStatsFillLinkMsg (
    ZkError err,
    FrStatsCtx ctx, 
    uint32_t sequence,
    ZkBool is_up,
    FrEntIdExp *nb_entity,
    time_t launch_time,   /* When the link was brought up */
    time_t last_change_time, /* When a paramter change occurred */
    time_t last_status_time, /* The last update */
    UINT64_T new_pkts_rx,
    UINT64_T new_pkts_tx,
    UINT64_T new_pkts_drop,
    UINT64_T new_bytes_rx,
    UINT64_T new_bytes_tx,
    UINT64_T new_bytes_drop);

  extern int frStatsReadLinkMsg(
    ZkError err,
    FrStatsCtx ctx, 
    time_t *reptime,
    uint32_t *seqno,
    ZkBool *is_up,
    const FrEntIdExp **nb_entity,
    time_t *launchtime,   /* When the link was brought up */
    time_t *last_change_time, /* When a paramter change occurred */
    time_t *last_status_time, /* The last update */
    UINT64_T *new_pkts_rx,
    UINT64_T *new_pkts_tx,
    UINT64_T *new_pkts_drop,
    UINT64_T *new_bytes_rx,
    UINT64_T *new_bytes_tx,
    UINT64_T *new_bytes_drop);

  extern int frStatsFillMaipMsg(
    ZkError err, 
    FrStatsCtx ctx, 
    uint32_t sequence,
    time_t launchtime,
    time_t last_status_time,
    uint32_t new_msgs_sent, 
    uint32_t new_msgs_rcvd,
    uint32_t new_msgs_dumped,
    uint32_t num_queued_msgs,
    UINT64_T bytes_rx, 
    UINT64_T bytes_tx);
  

  /* Send the update to the NISS */
  extern int frStatsSendMsg(
    ZkError err,
    FrStatsCtx ctx);

  /* Build an NISS update message */
  extern int frStatsBuildMsg(
    ZkError err,
    FrStatsCtx ctx,
    void **pkt,
    size_t *pktSize);

  /* SERVER SIDE FUNCTIONS */

  /*
   * Associates an incoming frStatUpd structure to and
   * frStatsCtx. Do this before attempting to read the
   * packet
   */
  extern int frStatsSetUpdate(
    ZK_INOUT ZkError err, 
    ZK_INOUT FrStatsCtx ctx,
    ZK_IN    FrStatUpd fsu);

  extern int frStatsClearUpdate(
    ZK_INOUT ZkError err, 
    ZK_INOUT FrStatsCtx ctx);
  
  extern int frStatsReadAipMsg(
    ZkError err, 
    FrStatsCtx ctx, 
    time_t *reptime,
    uint32_t *seqno,
    time_t *launchtime,
    time_t *last_niqs_reload_time, 
    time_t *last_status_time,
    uint32_t *new_client_connections,
    uint32_t *curr_connected_clients,
    UINT64_T *client_pkts_rx,
    UINT64_T *client_pkts_tx,
    UINT64_T *client_pkts_dropped,
    UINT64_T *client_bytes_rx,
    UINT64_T *client_bytes_tx,
    UINT64_T *client_bytes_dropped
    );

  extern int frStatsReadWormholeMsg(
    ZkError err, 
    FrStatsCtx ctx, 
    time_t *reptime,
    uint32_t *seqno,
    time_t *launchtime,
    time_t *last_status_time,
    uint32_t *maxin_thrput,
    uint32_t *new_streams,
    uint32_t *current_active_streams,
    uint32_t *new_ports,
    uint32_t *curr_used_ports, 
    UINT64_T *new_bytes_rx,
    UINT64_T *new_bytes_tx);
  
  int frStatsReadMaipMsg(
    ZkError err, 
    FrStatsCtx ctx,
    time_t *reptime,
    uint32_t *seqno,
    time_t *launchtime,
    time_t *last_status_time,
    uint32_t *new_msgs_sent, 
    uint32_t *new_msgs_rcvd,
    uint32_t *new_msgs_dumped,
    uint32_t *num_queued_msgs,
    UINT64_T *bytes_rx, 
    UINT64_T *bytes_tx);

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FRSTATS_H_ */
