/*
 *  $Id: frStatBucket.c,v 1.10 2000/11/23 15:13:26 marcp Exp $
 */


static const char * module_id_ = "$Id: frStatBucket.c,v 1.10 2000/11/23 15:13:26 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <zkError.h>
#include <zkConfig.h>
#include <zkLog.h>
#include <zkHash.h>
#include <zkList.h>
#include <frEnt.h>
#include <frKeyQry.h>
#include <frStatUpd.h>
#include <frStatBucket.h>


#define ME_INDEX_SZ    32 
#define ME_LINE_SZ     256

#define SM_TABLE_SZ    93

#define FL_NEWENTRY         0x1
#define FL_UPDATED          0x2

struct T_FrStatBucketEntry {
  uint32_t flags;                      /* Entry status flags */
  uint32_t updates;                    /* Number of times this entry updated */
  FrStatUpd fsu;                       /* Statistics for this entry */
  struct T_FrStatBucketEntry * link;   /* Next entry in list */
};

typedef struct T_FrStatBucketEntry FrStatBucketEntry;


struct T_FrStatBucket {
  uint32_t flags;
  size_t entries;
  size_t history;                       /* History size for new statupds */
  struct T_FrStatBucketEntry * me;      /* Head... */
  struct T_FrStatBucketEntry * tail; 
  struct T_FrStatBucketEntry * next;    /* Get the next entry in list */
  ZkHash hash;
#if 0
  FrKeyQry kq;                          /* Optional key query cache */
  FrKeyQryHost kh;                      /* Optional key query host */
  unsigned short kqtimeout;             /* Optional key query timeout */
  int kqretries;                        /* Optional key query retries */
#endif
};


#define DEFAULT_HISTORY   FRSTAT_DEFAULT_HISTORY

static size_t frStatBucketHash ( ZkHash , void * , void * );
static int frStatBucketKeyFree ( ZkHash , void * , void * );

/* Takes a NUL terminated string */

static size_t
frStatBucketHash (
  ZkHash h ,
  void * key ,
  void * userdata )
{
  FrEntIdExp * flubber = (FrEntIdExp *) key;
  char *p = NULL;
  size_t v = 0; 
  size_t i = 0;
  (void) module_id_;

  (void) h;  /* Gets rid of compile time warnings */
  (void) userdata;
  p = flubber->name;
  v = (size_t) flubber->type; 
  for (i = 0; (p[i] && (i < FRENT_NAME_SIZE)); i++) { 
    v = v * 33 + p[i];
  }
  return v;
}

static int
frStatBucketKeyFree (
  ZkHash h ,
  void * data ,
  void * userdata )
{
  (void) userdata; /* Gets rid of compile time warnings */
  if (h && data) {
    free(data);
    return 1;
  }
  return 0;
}

/*
 *
 */

DLLExport int
frStatBucketGetReset (
  ZkError err ,
  FrStatBucket b )
{
  static const char * func = "frStatBucketGetReset";
  int return_code = 0;
 
  zkErrorClear(err);
  if (b) {
    b->next = b->me;
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/**
 *  Return the next entry in the StatBucket.  If you specify the version
 *  parameter, the version number (ie number of times this entry has
 *  received an update) will be returned.  If you dont care, just set it
 *  to NULL.
 */

DLLExport int
frStatBucketGetStatUpd (
  ZkError err ,
  FrStatBucket b ,
  FrStatUpd * fsu , 
  uint32_t * update )
{
  static const char * func = "frStatBucketGetStatUpd";
  FrStatBucketEntry * entry = NULL;
  int return_code = 0;

  zkErrorClear(err);
  if (b && fsu) {
    entry = b->next;
    if (entry != NULL) {
      if (entry->fsu) {
        *fsu = entry->fsu; 
        b->next = entry->link; 
        return_code = 1;
        if (update) {
          *update = entry->updates;
        }
      } else {
        zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                   "FrStatBucket is missing an internal FrStatUpd");
      }
    } else {
      zkErrorSet(err,func,ZKINFO,ZKMFRSTAT,ZKENOTFOUND,
                 "No entries in FrStatBucket to return");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */
#if 0
DLLExport int
frStatBucketSetKeyQueryParameters (
  ZkError err ,
  FrStatBucket b ,
  FrKeyQry kq ,
  FrKeyQryHost kh ,
  unsigned short kqtimeout ,
  int kqretries )
{
  static const char * func = "frStatBucketSetPublicKeyCache";
  int return_code = 0;

  zkErrorClear(err);
  if (b && kq) {
    if (kh.host.sin_port == 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "FrKqyQryHost port is not set");
    } else if (kh.host.sin_addr.s_addr == 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "FrKqyQryHost host address is set to 0");
    } else if (kqtimeout == 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "Key query timeout is set to 0");
    } else if (kqretries < 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "Key query retries is set less that 0");
    } else { 
      b->kq = kq;
      b->kh = kh;
      b->kqtimeout = kqtimeout;
      b->kqretries = kqretries;
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

#endif
/*
 * 
 */

DLLExport int
frStatBucketNew (
  ZkError err ,
  FrStatBucket * sm )
{
  const char * func = "frStatBucketInit";
  int return_code = 0;

  zkErrorClear(err);
  if (sm) {
    *sm = (FrStatBucket) malloc(sizeof(struct T_FrStatBucket));
    if (*sm == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,"malloc");
      goto finish;
    }
    memset(*sm,0,sizeof(struct T_FrStatBucket));
    if (!(zkHashNew(err,&((*sm)->hash)) &&
          zkHashSetUserData(err,(*sm)->hash,NULL) &&
          zkHashSetHashFn(err,(*sm)->hash,frStatBucketHash) &&
          zkHashSetKeyFreeFn(err,(*sm)->hash,frStatBucketKeyFree) &&
          zkHashSetTableSize(err,(*sm)->hash,SM_TABLE_SZ))) {
      goto finish;
    }        
    (*sm)->flags = 0;
    (*sm)->entries = 0;
    (*sm)->me = NULL;
    (*sm)->tail = NULL;
    (*sm)->history = DEFAULT_HISTORY;
    (*sm)->next = NULL;
#if 0
    (*sm)->kq = NULL;
#endif
    return_code = 1;
finish:
    if (return_code == 0) {
      if (*sm) {
        free(*sm);
        *sm = NULL;
      }
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

DLLExport int 
frStatBucketSetHistory (
  ZkError err ,
  FrStatBucket b ,
  size_t h )
{
  static const char * func = "frStatBucketSetHistory";
  int return_code = 0;

  zkErrorClear(err);
  if (b) {
    b->history = h;
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

DLLExport int
frStatBucketHistory (
  ZkError err ,
  FrStatBucket b ,
  size_t * h )
{
  static const char * func = "frStatBucketHistory";
  int return_code = 0;

  zkErrorClear(err);
  if (b && h) {
    *h = b->history;
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

DLLExport int
frStatBucketClearFlags (
  ZkError err ,
  FrStatBucket sm ,
  uint32_t flags )
{
  const char * func = "frStatBucketClearFlags";
  FrStatBucketEntry * me = NULL;
  
  zkErrorClear(err);
  if (sm) {
    me = sm->me;
    while (me) {
      me->flags &= ~flags;
      me = me->link;
    }
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

#if 0
DLLExport int
frStatBucketCheckUpdateBufferSignature (
  ZkError err ,
  FrStatBucket b ,
  void * buf ,
  size_t bsz ,
  FrEntIdExp * entity )
{
  static const char * func = "frStatBucketCheckUpdateSignature";
  FrCryptSig sigkey = NULL;
  int return_code = 0;
  int version = 0;
  int valid = 0;
 
  zkErrorClear(err);
  if (b && buf) {
    if (!(frSecSigGetEntity(err,buf,bsz,entity) &&
          frSecSigGetVersion(err,buf,bsz,&version))) {
      /* fail */
    } else if ((b->kh.fd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
      zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKERROR,"socket: %s", 
                    strerror(errno));
    } else if (!frKeyQryFetchPubKey(err,b->kq,version,FRCRYPT_KEYTYPE_SIGNING, 
                                    *entity,b->kh,b->kqretries,b->kqtimeout,
                                    &sigkey)) {
      /* fail, could not get the key... */
    } else if (!frSecSigVerify(err,sigkey,buf,bsz,&valid)) { 
      /* Fail here too... */
    } else if (valid) {
      return_code = 1;
    } else {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKERROR,
                 "Signature check on update buffer failed");
    }
    close(b->kh.fd);
    b->kh.fd = -1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}
#endif

DLLExport int
frStatBucketAddUpdateBuffer (
  ZkError err ,
  FrStatBucket b ,
  void * buf ,
  size_t * bsz )
{
  static const char * func = "frStatBucketAddUpdateBuffer";
  int return_code = 0;
  int chicken = 1;   /* Set to 0 if buffer is not properly signed */
  FrStatUpd fsu = NULL;
  FrEntIdExp * fsu_entity = NULL;

  zkErrorClear(err);
  if (b && buf && bsz) {

    /*
     * If we specified a key cache, verify the signature, remove it from the
     * buffer, check that the signing entity and the entity contained in the
     * data buffer match and if all that is ok, proceed.
     */   
#if 0
    if (b->kq) {
      if (!frStatBucketCheckUpdateBufferSignature(err,b,buf,*bsz,&entity)) {
        chicken = 0;
      }
    } else if (!frSecSigGetEntity(err,buf,*bsz,&entity)) {
      chicken = 0;
    }
#endif   
    /*
     * Remove the signature, reconstruct the Stat update and add it to the 
     * buffer
     */

    if (chicken) {  
#if 0
      if (frSecSigRemoveInSitu(err,buf,bsz) &&
          frStatUpdFromBuffer(err,&fsu,buf,*bsz) &&
          frStatUpdEntity(err,fsu,(const FrEntIdExp **) &fsu_entity) &&
          frEntEqual(err,fsu_entity,&entity,&equals)) {
#endif
      if (frStatUpdFromBuffer(err,&fsu,buf,*bsz) &&
          frStatUpdEntity(err,fsu,(const FrEntIdExp **) &fsu_entity)) {
	if (frStatBucketAddUpdate(err,b,fsu)) {
	  return_code = 1;
	}
      }
    }

    if (fsu) {
      frStatUpdFree(NULL,&fsu);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

DLLExport int
frStatBucketGetStatUpdByEntity (
  ZkError err ,
  FrStatBucket b ,
  FrEntIdExp * entity ,
  FrStatUpd * fsu , 
  uint32_t * update )
{
  static const char * func = "frStatBucketGetByEntity";
  FrStatBucketEntry * me = NULL;
  int return_code = 0;
  
  zkErrorClear(err);
  if (b && fsu && entity) {
    if (zkHashSearch(err,b->hash,(void *) entity,(void **) &me)) {
      *fsu = me->fsu;
      if (update) {
        *update = me->updates;
      }
      return_code = 1;
    } 
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

/*
 * 
 *
 */

DLLExport int
frStatBucketAddUpdate (
  ZkError err ,
  FrStatBucket sm ,
  FrStatUpd fsu  )
{
  const char * func = "frStatBucketAddEntry";
  FrEntIdExp * entityPtr = NULL;
  FrStatBucketEntry * me = NULL;
  FrEntIdExp * fsu_entity = NULL;

  zkErrorClear(err);
  if (sm && fsu) {
    if (!frStatUpdEntity(err,fsu,(const FrEntIdExp **)&fsu_entity)) {
      return 0;
    }
    if (!zkHashSearch(err,sm->hash,(void *) fsu_entity,(void **) &me)) {
      me = (FrStatBucketEntry *) malloc(sizeof(struct T_FrStatBucketEntry));
      if (me == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,
                   "malloc (FrStatBucketEntry)");
        return 0;
      }
      entityPtr = (FrEntIdExp *) malloc(sizeof(FrEntIdExp));
      if (entityPtr == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,"malloc (entity key)");
        return 0;
      } else if (!frEntCopy(err,entityPtr,fsu_entity)) {
        return 0;
      }
      memset(me,0,sizeof(struct T_FrStatBucketEntry));
      me->updates = 0;
      if (!(frStatUpdNew(err,&(me->fsu)) &&
            frStatUpdSetHistory(err,me->fsu,sm->history) &&
            frStatUpdSetEntity(err,me->fsu,fsu_entity))) {
        return 0;
      }

      /* Mark the FrStatUpd as a new entry and append it to the FrStatBucket
       * entry list.  Also insert it into the Bucket's hash table so it
       * can be quickly retrieved.
       */
      
      me->flags |= FL_NEWENTRY;
      if (sm->tail) {
        (sm->tail)->link = me;  
      } else {
        sm->next = me;
        sm->me = me;
      }
      sm->tail = me;
      if (!zkHashInsert(err,sm->hash,entityPtr,me)) {
        return 0;
      }
    }

    /* Merge the FrStatUpd with the one contained in me */

    if (!frStatUpdUpdate(err,me->fsu,fsu)) {
      return 0;
    }
    me->updates++;

    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 *
 */

DLLExport int
frStatBucketDump (
  ZkError err ,
  FrStatBucket sm ,
  FILE * f ,
  uint32_t flags )
{
#define FLSTR_SZ 3
  const char * func = "frStatBucketDump";
  FrStatBucketEntry * me = NULL;
  char flstr[FLSTR_SZ + 1];
  
  (void) flags;
  if (sm && f) {
    me = sm->me;
    while (me) {
      memset(flstr,' ',FLSTR_SZ);
      flstr[FLSTR_SZ] = (char) 0;
      if (me->flags & FL_NEWENTRY) {
        flstr[0] = 'N';
      } 
      if (me->flags & FL_UPDATED) {
        flstr[1] = 'U';
      }
      fprintf(f,"%s(%5d) \n",flstr,me->updates);
      frStatUpdDump(err,me->fsu,-1,f);
      me = me->link;
    }
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}
