/*
 * $Id: frStatClient.c,v 1.15 2001/04/19 20:31:14 alainc Exp $
 */

static const char * module_id_ = "$Id: frStatClient.c,v 1.15 2001/04/19 20:31:14 alainc Exp $";


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <zkError.h>
#include <zkDNS.h>
#include <frCrypt.h>
#include <frSec.h>
#include <frStatUpd.h>
#include <frStatClient.h>
#include "frStatPkt.h"
#include "frNetworkDefaults.h"

#define HOSTNAME_SZ 128

#define DEFAULT_HOST    FR_DFLT_NISS_SERVER_HOST
#define DEFAULT_PORT    FR_DFLT_NISS_AIP_SERVER_PORT

struct T_FrStatClient {
  uint32_t flags;
  uint32_t version;
  int s;                       /* UDP socket connection to stat server */
  char host[HOSTNAME_SZ + 1];  /* Name of UDP host name */
  unsigned short port;         /* UDP stat server port number */
  FrCryptPubKey crypt;         /* Buffer cryption key */
  FrCryptSig sig;              /* Buffer signature key */
  struct sockaddr_in server;   /* Target server address struct */
  struct sockaddr_in client;   /* client address struct */
};


DLLExport int
frStatClientNew (
  ZkError err ,
  FrStatClient * t )
{
  static const char * func = "frStatClientNew";
  int return_code = 0;
  (void) module_id_;
  
  if (t) {
    *t = (FrStatClient) malloc(sizeof(struct T_FrStatClient));
    if (*t == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,"malloc");
    } else {
      memset(*t,0,sizeof(struct T_FrStatClient));
      (*t)->s = -1;
      (*t)->flags = FRSTAT_DEFAULT_FLAGS;
      (*t)->version = FRSTAT_DEFAULT_SIGVERS;
      strncpy((*t)->host,DEFAULT_HOST,HOSTNAME_SZ);
      (*t)->port = DEFAULT_PORT;
      (*t)->sig = NULL;
      return_code = 1; 
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

DLLExport int
frStatClientFree (
  ZkError err ,
  FrStatClient * t )
{
  static const char * func = "frStatClientFree";
  int return_code = 0;

  zkErrorClear(err);
  if (t) {
    if (*t) {
      if (-1 < (*t)->s) {
        frStatClientDisconnect(err,*t);
      }
      memset(*t,0,sizeof(struct T_FrStatClient));
      free(*t);
      *t = NULL;
      return_code = 1;
    } else {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */

DLLExport int
frStatClientSetHostname (
  ZkError err ,
  FrStatClient t,
  const char * host )
{
  static const char * func = "frStatClientSetHostname";
  int return_code = 0;

  zkErrorClear(err);
  if (t && host) {
    strncpy(t->host,host,HOSTNAME_SZ);
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frStatClientGetSocket (
  ZkError err,
  FrStatClient t,
  int *pSock,
  struct sockaddr *pSockAddr,
  socklen_t *pAddrSize )
{
  static const char * func = "frStatClientSetHostname";
  int rc = 1;

  zkErrorClear(err);
  if (t && pSock && pSockAddr && pAddrSize && ((*pAddrSize) >= sizeof(struct sockaddr_in)) ) 
  {
    *pSock = t->s;
    memcpy(pSockAddr, &(t->server), sizeof(struct sockaddr_in));
    *pAddrSize = sizeof(struct sockaddr_in);
  } 
  else 
  {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    rc = 0;
  }
  return rc;
}

DLLExport int
frStatClientIsServer (
  ZkError err,
  FrStatClient t,
  const char * host,
  unsigned short port,
  int * same )
{
  static const char * func = "frStatClientIsServer";
  int rc = 1;

  zkErrorClear(err);

  if (t && host && same) {
    if ( (strncmp(t->host, host, HOSTNAME_SZ) == 0) &&
         (t->port == port) ) { 
      *same = ZK_TRUE;
    }
    else {
      *same = ZK_FALSE;
    } 
  } 
  else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    rc = 0;
  }

  return rc;
}

DLLExport int
frStatClientSetSignatureKey (
  ZkError err ,
  FrStatClient t, 
  FrCryptSig sig )
{
  static const char * func = "frStatClientSetSignatureKey";
  int return_code = 0;

  zkErrorClear(err);
  if (t && sig) {
    t->sig = sig;
    return_code = 1;
  } else {  
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frStatClientSetNISSCryptKey (
  ZkError err ,
  FrStatClient t, 
  FrCryptPubKey nissPubKey )
{
  static const char * func = "frStatClientSetNISSCryptKey";
  int return_code = 0;

  zkErrorClear(err);
  if (t && nissPubKey) {
    t->crypt = nissPubKey;
    return_code = 1;
  } else {  
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */
  
DLLExport int
frStatClientSetPort (
  ZkError err ,
  FrStatClient t,
  unsigned short port )
{
  static const char * func = "frStatClientSetPort";
  int return_code = 0;

  zkErrorClear(err);
  if (t) {
    t->port = port;
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}
 

/*
 *
 */

DLLExport int
frStatClientConnect (
  ZkError err ,
  FrStatClient t )
{
  static const char * func = "frStatClientConnect";
  int return_code = 0;
  unsigned long ip_addr = 0;

  zkErrorClear(err);
  if (t) {
    if (-1 < t->s) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKERROR,
                 "Server socket is already connected");
    } else {
      ip_addr = inet_addr(t->host);
      if (ip_addr == 0xFFFFFFFF) {
        struct hostent * host = NULL;
        if (zkGetHostByName(err,t->host,&host)) {
          memcpy((void *)&ip_addr,host->h_addr_list[0],host->h_length); 
        } 
      }
      if (ip_addr != 0xFFFFFFFF) {
        t->server.sin_addr.s_addr = ip_addr;
        t->server.sin_family = AF_INET;
        t->server.sin_port = htons(t->port); 
        t->client.sin_addr.s_addr = htonl(INADDR_ANY);
        t->client.sin_port = htons(0);
        t->client.sin_family = AF_INET;
        if ((t->s = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
          zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKESOCKET,
                        "Error creating socket: %s",strerror(errno));
        } else if (bind(t->s,(struct sockaddr *) &(t->client),
                 sizeof(struct sockaddr_in)) < 0) {
          zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKESOCKET,
                        "Error binding client socket: %s",strerror(errno));
        } else {
          return_code = 1;
        }
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */

DLLExport int
frStatClientDisconnect (
  ZkError err ,
  FrStatClient t )
{
  static const char * func = "frStatClientDisconnect";
  int return_code = 0;

  zkErrorClear(err);
  if (t) {
    if (-1 < t->s) {
      close(t->s);
      t->s = -1;
      memset(&(t->client),0,sizeof(struct sockaddr_in));
      memset(&(t->server),0,sizeof(struct sockaddr_in));
      return_code = 1;
    } else {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKESOCKET,"Not currently connected");
    } 
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Returns ZK_TRUE if the client is currently connected,otherwise ZK_FALSE.
 */

DLLExport int
frStatIsClientConnected (
  ZkError err ,
  FrStatClient t )
{
  static const char * func = "frStatIsClientConnected";
  int return_code = ZK_FALSE;

  zkErrorClear(err);

  if (t) {
    return_code = (-1 < t->s) ? ZK_TRUE : ZK_FALSE;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Submit an FrStatUpd structure to the statistics collecting server via 
 * UDP.  This function will modify the FrStatUpd's entity to be the 
 * same as the one in the signature key used to sign the update.  This 
 * function returns immediatly.
 *
 * @param err Returns the error if one occured
 * @param t The FrStatClient structure which denotes the statistics
 *          collection server.
 * @param fsu An FrStatUpd structure containing various bits of information
 * @version 1.0
 * @author Philippe Boucher
 * @return 1 on success, 0 on failure
 *
 */
  
DLLExport int
frStatClientSubmitUpdate (
  ZkError err ,
  FrStatClient t ,
  FrStatUpd fsu )
{
  FrCryptSymAlg symalg = FRCRYPT_SYMALG_BLOWFISH;
  FrEntIdExp entity;
  int bytes = 0;
  int return_code = 0;
  size_t bsz = 0;
  size_t psz = 0;
  size_t bsz_encr = 0;
  size_t bsz_signed = 0;
  static const char * func = "frStatClientSubmitUpdate";
  void *buf = NULL;
  void *packed = NULL;
  void *buf_signed = NULL;
  void *buf_encr = NULL;
  
  zkErrorClear(err);

  if ( ! t) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "t is null");
    goto done;
  }
  if ( ! fsu) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "fsu is null");
    goto done;
  }
  
  /*
   *  We need signature and encryption keys.
   */

  if ( ! t->sig) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD,
               "missing signature key");
    goto done;
  }
  if ( ! t->crypt) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD,
               "missing encryption key");
    goto done;
  }

  /*
   *  We also need to be connected to the server.
   */
  
  if (t->s < 0) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR,
               "not connected to server");
    goto done;
  }
  
  /*
   *  First we signe the buffer with our signature key (typically AIP or
   *  MAIP), then we encrypt it with the NISS encryption key.
   */
  
  if ( ! (frCryptPubKeyGetEntity(err, t->sig, &entity) &&
          frStatUpdSetEntity(err, fsu, &entity) &&
          frStatUpdToBuffer(err, fsu, &buf, &bsz) &&
          frStatPktUpdate(err, 1, 0, 0, buf, bsz, &packed, &psz) &&
          frSecSignData(err, t->sig, packed, psz, &buf_signed, &bsz_signed) &&
          frSecPubKeyEncryptData(err, t->crypt, symalg, buf_signed, bsz_signed,
                                 &buf_encr, &bsz_encr))) {
    goto done;
  }

  /* *(unsigned char *)(buf_signed+7) = 45;  */
  
  bytes = sendto(t->s, buf_encr, (int)bsz_encr, 0,
                 (struct sockaddr *)&(t->server),
                 sizeof(struct sockaddr_in));
  
  if (bytes < 0) {
    zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKERROR,
                  "Error writing stat update to network: %s",
                  strerror(errno)); 
  } else if (bsz_encr != (size_t) bytes) {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKERROR,
               "Stat update buffer was not completely written to network");
  }
  
  return_code = 1;
  
 done:

  if (buf) { memset(buf, 0, bsz); free(buf); }
  if (packed) { memset(packed, 0, psz); free(packed); }
  if (buf_signed) { memset(buf_signed, 0, bsz_signed); free(buf_signed); }
  if (buf_encr) { free(buf_encr); }
  
  return return_code;
}
