/* $Id: frStatPkt.c,v 1.7 2000/11/23 15:13:26 marcp Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include "frPktFmt.h"
#include "frSec.h"
#include "frCrypt.h"
#include "frStatPkt.h"

#define VERS_SIZE               1
#define TYPE_SIZE               1
#define TTL_SIZE                4
#define ID_SIZE                 2


#define DEBUG_STAT_CRC 0


#if DEBUG_STAT_CRC
/*
 *  bug: this code taken from crc.c in frpktfmt/src.
 */

/*
 *  CRC 0102010
 */
static unsigned short crc_table[256] = {
  0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
  0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
  0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
  0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
  0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
  0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
  0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
  0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
  0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
  0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
  0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
  0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
  0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
  0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
  0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
  0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
  0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
  0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
  0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
  0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
  0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
  0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
  0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
  0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
  0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
  0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
  0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
  0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
  0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
  0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
  0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
  0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
};


static unsigned short cksum(unsigned char *s, int n)
{
  unsigned int crc;

  for (crc = 0; n > 0; n--) {
    crc = crc_table[(crc ^ *s++) & 0xff] ^ (crc >> 8);
  }

  return crc;
}
#endif


/**
 * Make a NIDB_UPD_xxx packet.
 */
int frStatPktUpdateReq(ZkError err, FrPktType type, unsigned int vers,
                       uint32_t ttl, int id, FrCryptPubKey sig, void **buf, 
                       size_t *bsz)
{
  const char *func = "frStatPktUpdateReq";
  unsigned char updReq[VERS_SIZE + TYPE_SIZE + TTL_SIZE + ID_SIZE];

  zkErrorClear(err);
  if (!sig || !buf || !bsz) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  switch (type) {
  case FRPKTTYPE_NIDB_UPD_REQ:
  case FRPKTTYPE_NIDB_UPD_REGTOP:
  case FRPKTTYPE_NIDB_UPD_URGTOP:

    updReq[0] = vers;
    updReq[1] = type;
    updReq[2] = (unsigned char) ((ttl & 0xFF000000) / 0x1000000);
    updReq[3] = (unsigned char) ((ttl & 0x00FF0000) / 0x10000);;
    updReq[4] = (unsigned char) ((ttl & 0x0000FF00) / 0x100);
    updReq[5] = (unsigned char) (ttl & 0x000000FF);
    updReq[6] = (unsigned char) (id & 0xFF00) / 0x100;
    updReq[7] = (unsigned char) (id & 0xFF);
  
    if (!frSecSignData(err, sig, updReq, sizeof(updReq), buf, bsz)) {
      return 0;
    }
    break;

  default:
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEPARMBAD, 
                  "Type %d is not valid", type);
    return 0;
  }

  return 1;
}

/* 
 *  Make a FRPKTTYPE_NIDB_UPDATE using the supplied information.  Prepend the
 *  header information (vers, type, id, ttl), then return it to the calling
 *  program.  Up to the calling program to free it.
 */ 
int frStatPktUpdate(ZkError err, unsigned int vers, uint32_t ttl, int id,
                    char *data, size_t dsz, void **buf, size_t *bsz)
{
  char *hdrData = NULL;
  const char *func = "frStatPktUpdate";
  int ret = 0;
  size_t hsz;
  unsigned char updReq[VERS_SIZE + TYPE_SIZE + TTL_SIZE + ID_SIZE];

  /* Serialyze data to send to the wire */
  updReq[0] = vers;
  updReq[1] = FRPKTTYPE_NIDB_UPDATE;
  updReq[2] = (unsigned char) ((ttl & 0xFF000000) / 0x1000000);
  updReq[3] = (unsigned char) ((ttl & 0x00FF0000) / 0x10000);;
  updReq[4] = (unsigned char) ((ttl & 0x0000FF00) / 0x100);
  updReq[5] = (unsigned char) (ttl & 0x000000FF);
  updReq[6] = (unsigned char) (id & 0xFF00) / 0x100;
  updReq[7] = (unsigned char) (id & 0xFF);

  hsz = VERS_SIZE + TYPE_SIZE + TTL_SIZE + ID_SIZE;
  
  if ((hdrData = malloc(hsz * sizeof(char) + dsz * sizeof(char))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENOMEM, "malloc() failed");
    goto done;
  }

  memcpy(hdrData, updReq, hsz);
  memcpy(&hdrData[hsz], data, dsz);

#if DEBUG_STAT_CRC
  fprintf(stderr, "*** %s: full packet: %lu bytes, crc = %hu.\n",
          func, (unsigned long)hsz + dsz, cksum(hdrData, hsz + dsz));
  fprintf(stderr, "*** %s: stats data: %lu bytes, crc = %hu.\n",
          func, (unsigned long)dsz, cksum(hdrData + hsz, dsz));
#endif

  *buf = hdrData; hdrData = 0; /* the caller now owns it */
  *bsz = hsz + dsz;

  ret = 1;

 done:
  
  if (hdrData) { memset(hdrData, 0, hsz + dsz); free(hdrData); }

  return ret;
}

/**
 * Given a packet, try to get the Entity and key version of the signature in
 * the packet.
 */
int frStatPktGetSigInfo(ZkError err, void *data, size_t dsz, 
                        FrEntIdExp *entity, unsigned int *keyVers)
{
  const char *func = "frStatPktGetSigInfo";

  zkErrorClear(err);

  if (!data || !entity || !keyVers) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frSecSigGetEntity(err, data, dsz, entity)) {
    return 0;
  }

  if (!frSecSigGetVersion(err, data, dsz, keyVers)) {
    return 0;
  }

  return 1;
}

/*
 *  Extract the information out of the packet.  Buffer is allocated but is
 *  expected to be freed by the calling program.
 */
int frStatPktGetUpdInfo(ZkError err, void *data, size_t dsz,
                        unsigned int *vers, FrPktType *type, uint32_t *ttl,
                        int *id, void **buffer, size_t *bufsz)
{
  const char *func = "frStatPktGetUpdInfo";
  size_t bsz = dsz;
  size_t hsz = VERS_SIZE + TYPE_SIZE + TTL_SIZE + ID_SIZE;
  unsigned char *buf = NULL;

  zkErrorClear(err);
  if (!data || !vers || !type || !ttl || !id) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }
  
#if DEBUG_STAT_CRC
  fprintf(stderr, "*** %s: full packet: %lu bytes, crc = %hu.\n",
          func, (unsigned long)dsz, cksum(data, dsz));
  fprintf(stderr, "*** %s: stats data: %lu bytes, crc = %hu.\n",
          func, (unsigned long)dsz - hsz,
          cksum((char *)data + hsz, dsz - hsz));
#endif

  if (bsz < hsz) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEINVAL, 
                  "data is too small (%d < %d)", bsz, hsz);
    return 0;
  }
  buf = (unsigned char *) data;
  
  *vers = buf[0];
  *type = buf[1];
  *ttl = (buf[2] * 0x1000000) + (buf[3] * 0x10000) + (buf[4] * 0x100) + buf[5];
  *id = buf[6] * 0x100 + buf[7]; 

  if (*vers != 1) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEINVAL, 
                  "Vers [%d] is invalid", *vers);
    return 0;
  }

  if ((*type != FRPKTTYPE_NIDB_UPD_REQ) && (*type != FRPKTTYPE_NIDB_UPD_URGTOP)
      && (*type != FRPKTTYPE_NIDB_UPD_REGTOP) && 
      (*type != FRPKTTYPE_NIDB_UPDATE)) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEINVAL, 
                  "Type [%d] is invalid", *type);
    return 0;
  }

  if (bsz > hsz) {
    if ((*buffer = malloc(bsz - hsz)) == NULL) {
      zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENOMEM, "malloc() failed");
      return 0;
    }
    memcpy(*buffer, &buf[8], bsz - hsz);
    *bufsz = bsz - hsz;
  } else {
    *buffer = NULL;
    *bufsz = 0;
  }

  return 1;
}
