
/*
 * $Id: frStatUpd.c,v 1.25 2001/04/19 20:31:14 alainc Exp $
 */

static const char * module_id_ = "$Id: frStatUpd.c,v 1.25 2001/04/19 20:31:14 alainc Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <time.h>
#include <errno.h>
#include <zkError.h>
#include "zkCStr.h"
#include "zkFile.h"
#include <frStatUpd.h>
#include <frEnt.h>


/* For Packing */
#define INT32_T_BUF_SIZE        4
#define INT16_T_BUF_SIZE        2
#define BOOLEAN_BUF_SIZE        1
#define FRSTAT_PARAMETER_SIZE   2
#define FRSTAT_PARM_VSIZE       INT32_T_BUF_SIZE*2

struct T_FrStatUpdParmVal {
  time_t t;
  FrStatUpdParm v;   
};

typedef struct T_FrStatUpdParmVal FrStatUpdParmVal;
                                        
struct T_FrStatUpd {
  uint32_t sigvers;                      /* Signature and version of this pkt */
  uint32_t flags;                        /* Flags - Philippe's */
  FrEntIdExp entity;                     /* Owning entity */
  FrStatUpdTypes type;                   /* The type of update */
  FrEntIdExp nbentity;                   /* Neighbour entity, when needed */
                                         /* Only valid if type = FRSTAT_T_LINK */
  FrStatUpdReleaseString relString;      /* The reporting entities release str */
  size_t history;                        /* History size */
  size_t tail[FRSTATUPD_TOTAL_PARMS];    /* Tail of each */
  size_t set[FRSTATUPD_TOTAL_PARMS];     /* Current of each */
  FrStatUpdParmVal *pv[FRSTATUPD_TOTAL_PARMS];  /* param values */
};

/*
 * Print the string representation of the StatUdpParameter
 */
DLLExport const char *
frStatUdpGetParameterString (
   ZkError err,
   FrStatUpdParameter p)
{
  static char nbuf[60]; /* not reentrant */
  (void) err;

#define casestr(x) case x: return #x
  switch(p) {
    casestr(FRSTATUPD_BYTES_L_SENT);
    casestr(FRSTATUPD_BYTES_L_RCVD);
    casestr(FRSTATUPD_BYTES_L_DROP);
    casestr(FRSTATUPD_PKTS_L_SENT);
    casestr(FRSTATUPD_PKTS_L_RCVD);
    casestr(FRSTATUPD_PKTS_L_DROP);

    casestr(FRSTATUPD_BYTES_H_SENT);
    casestr(FRSTATUPD_BYTES_H_RCVD);
    casestr(FRSTATUPD_BYTES_H_DROP);
    casestr(FRSTATUPD_PKTS_H_SENT);
    casestr(FRSTATUPD_PKTS_H_RCVD);
    casestr(FRSTATUPD_PKTS_H_DROP);
    
    casestr(FRSTATUPD_CONNECTIONS);
    casestr(FRSTATUPD_LAUNCHTIME);
    casestr(FRSTATUPD_SEQNO);
    casestr(FRSTATUPD_LNKSTAT);
    casestr(FRSTATUPD_NIQS_RELOAD);
    casestr(FRSTATUPD_LAST_STATUS);
    casestr(FRSTATUPD_CURR_CONNECTS);
    casestr(FRSTATUPD_CURR_PORTS);
    casestr(FRSTATUPD_CONSTANT);
    casestr(FRSTATUPD_AIP_NEIGHB);
    casestr(FRSTATUPD_CURRTIME);
  default:
    if (p == FRSTATUPD_TOTAL_PARMS) {
      snprintf(nbuf, sizeof nbuf, "<Type Overflow 0x%x>", (int)p);
    } else {
      snprintf(nbuf, sizeof nbuf, "<unknown paramter 0x%x>", (int)p);
    }
    return (const char *)nbuf;
  }
} 

/*
 * Print the string representation of the StatUdpParameter
 */
DLLExport const char *
frStatUdpGetTypeString (
   ZkError err,
   FrStatUpdParameter p)
{
  static char nbuf[60]; /* not reentrant */
  (void) err;

#define casestr(x) case x: return #x
  switch(p) {
    casestr(FRSTAT_T_UNUSED);
    casestr(FRSTAT_T_AIP);
    casestr(FRSTAT_T_MAIP);
    casestr(FRSTAT_T_LINK);
    casestr(FRSTAT_T_WORMHOLE);
  default:
    if (p == FRSTAT_T_TOTAL_TYPES) {
      snprintf(nbuf, sizeof nbuf, "<Type Overflow 0x%x>", (int)p);
    } else {
      snprintf(nbuf, sizeof nbuf, "<unknown paramter 0x%x>", (int)p);
    }
    return (const char *)nbuf;
  }
} 

DLLExport int
frStatUpdDump (
  ZkError err ,
  FrStatUpd fsu ,
  int maxhistory,
  FILE * stream )
{
  static const char * func = NULL;
  int return_code = 0;
  FrStatUpdParameter n = 0;
  size_t h = 0;
  size_t mh = 0;
  time_t t = 0;
  FrStatUpdParm parm;

  (void) module_id_;

  zkErrorClear(err);
  if (fsu && stream) {

    if (maxhistory < 0) {
      mh = fsu->history;
    } else if (maxhistory > (int) fsu->history) {
      mh = fsu->history;
    } else {
      mh = (size_t)maxhistory;
    }
      
    fprintf(stream,"Signature:  0x%x\n",fsu->sigvers & FRSTAT_SIGNATURE_MASK);
    fprintf(stream,"Version  :  0x%x\n",fsu->sigvers & ~FRSTAT_SIGNATURE_MASK);
    fprintf(stream,"Owner    :  %s (%d)\n",fsu->entity.name,fsu->entity.type);
    fprintf(stream,"Type     :   %s (%d)\n", frStatUdpGetTypeString (err,fsu->type),
            fsu->type);
    if (fsu->type == FRSTAT_T_LINK) {
      fprintf(stream,"Neighbour:  %s (%d)\n",fsu->nbentity.name,fsu->nbentity.type);
    }
    fprintf(stream,"SWRelease: %s\n",fsu->relString);
    fprintf(stream,"History  : %d\n",fsu->history);

    for (n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) {
      for (h = 0; h < mh+1; h++) {
        if (frStatUpdGetParameter(err,fsu,n,h,&parm,&t)) {
          switch (n) {
          case FRSTATUPD_BYTES_L_SENT:
          case FRSTATUPD_BYTES_L_RCVD:
          case FRSTATUPD_BYTES_L_DROP:
          case FRSTATUPD_PKTS_L_SENT:
          case FRSTATUPD_PKTS_L_RCVD:
          case FRSTATUPD_PKTS_L_DROP:

          case FRSTATUPD_BYTES_H_SENT:
          case FRSTATUPD_BYTES_H_RCVD:
          case FRSTATUPD_BYTES_H_DROP:
          case FRSTATUPD_PKTS_H_SENT:
          case FRSTATUPD_PKTS_H_RCVD:
          case FRSTATUPD_PKTS_H_DROP:

          case FRSTATUPD_CONNECTIONS:
          case FRSTATUPD_SEQNO:
          case FRSTATUPD_LNKSTAT:
          case FRSTATUPD_CURR_CONNECTS:
          case FRSTATUPD_CURR_PORTS:
          case FRSTATUPD_CONSTANT:
          case FRSTATUPD_CURRTIME:
          case FRSTATUPD_AIP_NEIGHB:
            fprintf(stream, "Parameter %23s[%02d] was %u at %08lX %s",
                    frStatUdpGetParameterString (err,n),
                    h, parm, t, ctime(&t));
            break;
          case FRSTATUPD_LAUNCHTIME:
          case FRSTATUPD_NIQS_RELOAD:
          case FRSTATUPD_LAST_STATUS: {
            /* BUG: not threadsafe */
            char *s = ctime((time_t *)&parm);
            s[strlen(s)-2]='\0';
            fprintf(stream, "Parameter %23s[%02d] was %s[%08u] at %08lX %s",
                    frStatUdpGetParameterString (err,n),
                    h, s, parm, t, ctime(&t));
            break;
          }
          default:
            fprintf(stream, "Unknown parameter type %d\n", n);
            return_code = 0;
            break;
          }
        }
      }
    }
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,"Null parameters");
  }
  return return_code;
}


DLLExport int
frStatUpdGetHistorySize (
  ZkError err ,
  FrStatUpd fsu ,
  size_t *history )
{
  static const char *funcname = "frStatUpdGetHistorySize";

  zkErrorClear(err);
  if (!fsu || !history) {
    zkErrorSet(err,funcname,ZKERR,ZKMFRSTAT,ZKENULLPTR,
	       "Null Pointer passed");
    return 0;
  }
  *history = fsu->history;
  return 1;
}

/*
 * 
 */
  
DLLExport int frStatUpdSetReleaseString(ZkError err, FrStatUpd fsu, 
                                        const char *relString, size_t sz)
{
  const char *func = "frStatUpdSetReleaseString";

  zkErrorClear(err);
  if (!fsu || !relString) {
    zkErrorSet(err, func, ZKERR, ZKMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (sz > FRSTATUPD_RELEASE_STRING_SZ) {
    zkErrorSet_va(err, func, ZKWARN, ZKMAIP, ZKEPARMBAD, 
		  "sz %d is too long, truncating", sz);
  }

  memset(fsu->relString, 0, sizeof(FrStatUpdReleaseString));
  strncpy(fsu->relString, relString, FRSTATUPD_RELEASE_STRING_SZ-1);

  return 1; 
}

DLLExport int frStatUpdGetReleaseString(ZkError err, FrStatUpd fsu, 
                                        char **relString, size_t *sz)
{
  const char *func = "frStatUpdGetReleaseString";
  char *p = NULL;

  zkErrorClear(err);
  if (!fsu || !relString || !sz) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  *sz = strlen(fsu->relString);
  if ((p = malloc((*sz + 1) * sizeof(char))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENOMEM, "NO memory");
    return 0;
  }

  zkCStrlcpy(p, fsu->relString, *sz + 1);
  p[*sz] = '\0';

  *relString = p;

  return 1;
}

DLLExport int
frStatUpdGetSigVers (
  ZkError err ,
  FrStatUpd fsu ,
  uint32_t *sig,
  uint32_t *vers)
{
  static const char *func = "frStatUpdGetSigVers";
  zkErrorClear(err);
  
  if (!fsu || !sig || !vers) {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,
	       "Null Pointer(s) passed");
    return 0;
  }
  *sig  = fsu->sigvers & FRSTAT_SIGNATURE_MASK;
  *vers = fsu->sigvers & ~FRSTAT_SIGNATURE_MASK;
  return 1;
}

DLLExport int
frStatUpdSetType (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdTypes type )
{
  static const char * func = "frStatUpdSetEntity";
  int return_code = 0;

  zkErrorClear(err); 
  if (fsu) {
    fsu->type = type;
     return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}


DLLExport int
frStatUpdSetEntity (
  ZkError err ,
  FrStatUpd fsu ,
  FrEntIdExp * entity )
{
  static const char * func = "frStatUpdSetEntity";
  int return_code = 0;

  zkErrorClear(err); 
  if (fsu && entity) {
    if (frEntCopy(err,&(fsu->entity),entity)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frStatUpdSetNbEntity (
  ZkError err ,
  FrStatUpd fsu ,
  FrEntIdExp * nbentity )
{
  static const char *func = "frStatUpdSetNbEntity";
  int return_code = 0;

  zkErrorClear(err); 
  if (fsu && nbentity) {
    if (frEntCopy(err,&(fsu->nbentity),nbentity)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */

DLLExport int
frStatUpdEntity (
  ZkError err ,
  FrStatUpd fsu ,
  const FrEntIdExp ** entity )
{
  static const char * func = "frStatUpdEntity";
  int return_code = 0;
 
  zkErrorClear(err);
  if (fsu && entity) {
    *entity =  &(fsu->entity);
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}


DLLExport int
frStatUpdType (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdTypes *type )
{
  static const char * func = "frStatUpdSetEntity";
  int return_code = 0;

  zkErrorClear(err); 
  if (fsu && type) {
    *type = fsu->type;
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frStatUpdNbEntity (
  ZkError err ,
  FrStatUpd fsu ,
  const FrEntIdExp ** nbentity )
{
  static const char *func = "frStatUpdNbEntity";
  int return_code = 0;
 
  zkErrorClear(err);
  if (fsu && nbentity) {
    *nbentity =  &(fsu->nbentity);
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * 
 */

DLLExport int
frStatUpdGetParameter (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdParameter parm ,
  size_t h ,
  FrStatUpdParm *val,
  time_t * t)
{
  static const char * func = "frStatUpdGetParameter";
  FrStatUpdParmVal * pv = NULL;
  int return_code = 0;
  int ndx = 0;

  zkErrorClear(err);
  if (!fsu || !(val || t)) {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    return 0;
  }
  pv = fsu->pv[parm];
  if (parm >= FRSTATUPD_TOTAL_PARMS) { 
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
	       "Parameter out of range");
    /*
    ** Doen't make any sens, h doesn't represent the history size at all
    */
  } else if (h >= fsu->history) {
    zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
		  "Statistics history extends only to %d entries",
		  fsu->history);
  } else if (!(pv && fsu->set[parm])) {
    zkErrorSet_va(err,func,ZKINFO,ZKMFRSTAT,ZKENOTFOUND,
    "Parameter [%d] is not set", parm); 
  } else {   
    ndx = (fsu->tail[parm]+(fsu->history-1-h))%fsu->history;
    if (pv[ndx].t) {
      if (val) {
	*val = pv[ndx].v;
      }
      if (t) {
	*t = pv[ndx].t;
      }
      return_code = 1;
    } else {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOTFOUND,
		 "Parameter not set for this time");
    }
  }
  return return_code;
}

DLLExport int
frStatUpdIncreaseParameter (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdParameter p ,
  int32_t v )
{
  static const char * func = "frStatUpdIncreaseParameter";
  int return_code = 0;
  FrStatUpdParm v_old;
  
  zkErrorClear(err);
  if (fsu) {

    if ((p != FRSTATUPD_AIP_NEIGHB)) {
      /* Only makes sense for other parameters */
      if (frStatUpdGetParameter(err,fsu,p,0,&v_old,NULL)) {
        v_old += v;
        if (frStatUpdSetParameter(err,fsu,p,v_old)) {
          return_code = 1;
        }
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}




DLLExport int
frStatUpdHistory (
  ZkError err ,
  FrStatUpd  fsu ,
  size_t * h)
{
  static const char * func = "frStatUpdHistory";

  zkErrorClear(err);
  if (fsu && h) {
    *h = fsu->history;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */


DLLExport int
frStatUpdNew (
  ZkError err ,
  FrStatUpd * fsu )
{
  static const char * func = "frStatUpdNew";
  int n = 0;

  zkErrorClear(err);
  if (fsu) {
    *fsu = (FrStatUpd) malloc(sizeof(struct T_FrStatUpd));
    if (*fsu == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,"malloc");
      return 0;
    }
    memset(*fsu,0,sizeof(struct T_FrStatUpd));
    (*fsu)->flags = FRSTAT_DEFAULT_FLAGS;
    (*fsu)->sigvers = FRSTAT_DEFAULT_SIGVERS;
    (*fsu)->type = FRSTAT_T_UNUSED;
    (*fsu)->history = FRSTAT_DEFAULT_HISTORY;
    (*fsu)->entity.type = FRENT_TYPE_UNUSED_;
    (*fsu)->nbentity.type = FRENT_TYPE_UNUSED_;
    for (n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) {
      (*fsu)->tail[n] = 0;
      (*fsu)->set[n] = 0;
      (*fsu)->pv[n] = NULL;
    }
    if (!frStatUpdSetHistory(err,*fsu,FRSTAT_DEFAULT_HISTORY)) {
      return 0;
    }
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

DLLExport int
frStatUpdFree (
  ZkError err ,
  FrStatUpd * fsu )
{
  static const char * func = "frStatUpdFree";
  int n = 0;

  zkErrorClear(err);
  if (fsu) {
    if (*fsu) {
      for(n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) {
        if ((*fsu)->pv[n]) {
          free((*fsu)->pv[n]);
        }
      }
      memset(*fsu,0,sizeof(struct T_FrStatUpd));
      free(*fsu);
      *fsu = NULL;
      return 1;
    }
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

DLLExport int
frStatUpdSetParameter (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpdParameter parm ,
  FrStatUpdParm v)
{
  static const char * func = "frStatUpdParameter";
  FrStatUpdParmVal * pv = NULL;

  zkErrorClear(err);
  if (fsu) {
    if ((pv = (FrStatUpdParmVal *) fsu->pv[parm]) == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "FrStatUpd structure does not have space for parameter");
      return 0;
    }
    switch (parm) {
    case FRSTATUPD_BYTES_L_SENT:
    case FRSTATUPD_BYTES_L_RCVD:
    case FRSTATUPD_BYTES_L_DROP:
    case FRSTATUPD_PKTS_L_SENT:
    case FRSTATUPD_PKTS_L_RCVD:
    case FRSTATUPD_PKTS_L_DROP:
      
    case FRSTATUPD_BYTES_H_SENT:
    case FRSTATUPD_BYTES_H_RCVD:
    case FRSTATUPD_BYTES_H_DROP:
    case FRSTATUPD_PKTS_H_SENT:
    case FRSTATUPD_PKTS_H_RCVD:
    case FRSTATUPD_PKTS_H_DROP:

    case FRSTATUPD_CONNECTIONS:
    case FRSTATUPD_LAUNCHTIME:
    case FRSTATUPD_SEQNO:
    case FRSTATUPD_LNKSTAT:
    case FRSTATUPD_NIQS_RELOAD:
    case FRSTATUPD_LAST_STATUS:
    case FRSTATUPD_CURR_CONNECTS:
    case FRSTATUPD_CURR_PORTS:
    case FRSTATUPD_CONSTANT:
    case FRSTATUPD_AIP_NEIGHB:
    case FRSTATUPD_CURRTIME:
      pv[fsu->tail[parm]].v = v;
      break;

    default:
      zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEPARMBAD, 
                    "Unknown parameter type [%d]", parm);
      return 0;
      break;
    }

    time(&(pv[fsu->tail[parm]].t));
    fsu->tail[parm] = (fsu->tail[parm] + 1) % fsu->history;
    fsu->set[parm] = 1;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

DLLExport int
frStatUpdSetHistory (
  ZkError err ,
  FrStatUpd fsu ,
  size_t h)
{
  static const char * func = "frStatUpdHistory";
  int n = 0;

  zkErrorClear(err);
  if (fsu) {
    fsu->history = 0;
    for (n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) {
      fsu->tail[n] = 0;
      fsu->set[n] = 0;
      if (fsu->pv[n]) {
        free(fsu->pv[n]);
      }
      fsu->pv[n] = (FrStatUpdParmVal *) malloc(sizeof(FrStatUpdParmVal) * h);
      if (fsu->pv[n] == NULL) {
        zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENODATA,"malloc");
        return 0;
      }
      memset(fsu->pv[n],0,sizeof(FrStatUpdParmVal) * h);
    }
    fsu->history = h;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

DLLExport int
frStatUpdClearHistory (
  ZkError err ,
  FrStatUpd fsu ) 
{
  static const char * func = "frStatUpdClearHistory";
  int return_code = 0;

  zkErrorClear(err);
  if (fsu) {
    if (frStatUpdSetHistory(err,fsu,fsu->history)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}


static int frStatUpdGenericToBuffer(ZkError err, FrStatUpd fsu, int n,
                                    FrStatUpdParmVal *pv, unsigned char *buf, 
                                    size_t *bptr)
{
  const char *func = "frStatUpdGenericToBuffer";
  size_t m;
  int32_t t, v;
  size_t base;

  zkErrorClear(err);
  if (!fsu || !pv || !buf || !bptr) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  base = *bptr;
  for (m = 0; m < fsu->history; m++) {
    t = pv[(fsu->tail[n] + m) % fsu->history].t; 
    v = pv[(fsu->tail[n] + m) % fsu->history].v; 
    buf[base++] = (unsigned char) ((t & 0xFF000000) / 0x1000000);
    buf[base++] = (unsigned char) ((t & 0x00FF0000) / 0x10000);
    buf[base++] = (unsigned char) ((t & 0x0000FF00) / 0x100);
    buf[base++] = (unsigned char) (t & 0x000000FF);
    buf[base++] = (unsigned char) ((v & 0xFF000000) / 0x1000000);
    buf[base++] = (unsigned char) ((v & 0x00FF0000) / 0x10000);
    buf[base++] = (unsigned char) ((v & 0x0000FF00) / 0x100);
    buf[base++] = (unsigned char) (v & 0x000000FF);
  }
  
  *bptr = base;
  return 1;
}

/*
 *
 */

DLLExport int
frStatUpdToBuffer (
  ZkError err ,
  FrStatUpd fsu ,
  void ** b ,
  size_t *sz )
{
  static const char * func = "frStatUpdToBuffer";
  int n = 0;
  unsigned char * buf = NULL;
  int base = 0;
  size_t bsz = 0;
  FrStatUpdParmVal * pv = NULL;
  size_t words = 0, shorts = 0, extra = 0;
  size_t ent_buf_sz = FRENT_PORTABLE_SIZE;

  /* Counters for the different types of parameters */
  int vcnt = 0;

  zkErrorClear(err);
  if (fsu && b && sz) {
    for (n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) {
      if (fsu->set[n]) {
        switch (n) {
        case FRSTATUPD_BYTES_L_SENT:
        case FRSTATUPD_BYTES_L_RCVD:
        case FRSTATUPD_BYTES_L_DROP:
        case FRSTATUPD_PKTS_L_SENT:
        case FRSTATUPD_PKTS_L_RCVD:
        case FRSTATUPD_PKTS_L_DROP:
          
        case FRSTATUPD_BYTES_H_SENT:
        case FRSTATUPD_BYTES_H_RCVD:
        case FRSTATUPD_BYTES_H_DROP:
        case FRSTATUPD_PKTS_H_SENT:
        case FRSTATUPD_PKTS_H_RCVD:
        case FRSTATUPD_PKTS_H_DROP:

        case FRSTATUPD_CONNECTIONS:
        case FRSTATUPD_LAUNCHTIME:
        case FRSTATUPD_SEQNO:
        case FRSTATUPD_LNKSTAT:
        case FRSTATUPD_NIQS_RELOAD:
        case FRSTATUPD_LAST_STATUS:
        case FRSTATUPD_CURR_CONNECTS:
        case FRSTATUPD_CURR_PORTS:
        case FRSTATUPD_CONSTANT:
        case FRSTATUPD_AIP_NEIGHB:
        case FRSTATUPD_CURRTIME:
          vcnt++;
          break;

        default:
          zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD, 
                        "Unknown type %d", n);
          return 0;
          break;
        }
      }
    }

    /* bsz[4],sigvers[4],owner[FZ],type[2],
     * (neighbour[FZ]{0,1},relstr,history[2],count[2],
     * {parms,val,t}+
     */

    /* This is oh so error prone. Following previous work. DBO's are so much
       nicer */
    bsz = INT32_T_BUF_SIZE * 2  /* bsz, sigvers */
      + FRENT_PORTABLE_SIZE     /* entity */
      + INT16_T_BUF_SIZE        /* type */
      + ((fsu->type == FRSTAT_T_LINK)?FRENT_PORTABLE_SIZE:0) /* link->neighbour*/
      + FRSTATUPD_RELEASE_STRING_SZ /* Release string */
      + INT16_T_BUF_SIZE        /* history */
      + INT16_T_BUF_SIZE;        /* v count w/o history */

    bsz += FRSTAT_PARAMETER_SIZE*vcnt + FRSTAT_PARM_VSIZE*vcnt*fsu->history;
      
    buf = (unsigned char *) malloc(bsz + 1);
    
    if (buf == NULL) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENOMEM,"malloc");
      return 0;
    }

    memset (buf,0,bsz);

    /* Store the statistics header */
#   define COMPUTE(w,s,e) (INT32_T_BUF_SIZE*words+INT16_T_BUF_SIZE*shorts+extra)

    /* bsize */
    words = 0;
    shorts = 0;
    extra = 0;
    base = COMPUTE(words,shorts,extra);

    buf[base+0] = (bsz & 0xFF000000) / 0x1000000;
    buf[base+1] = (bsz & 0xFF0000) / 0x10000;
    buf[base+2] = (bsz & 0xFF00) / 0x100;
    buf[base+3] = (bsz & 0xFF);
    words ++; base = COMPUTE(words,shorts,extra);


    /* signature */
    buf[base+0] = (fsu->sigvers & 0xFF000000) / 0x1000000;
    buf[base+1] = (fsu->sigvers & 0xFF0000) / 0x10000;
    buf[base+2] = (fsu->sigvers & 0xFF00) / 0x100;
    buf[base+3] = (fsu->sigvers & 0xFF);
    words++; base = COMPUTE(words,shorts,extra);

    /* owner */

    /* Store the owning entity */
    if (!frEntStore(err,&(fsu->entity),&(buf[base]),ent_buf_sz)) {
      return 0;
    }
    extra += FRENT_PORTABLE_SIZE;
    base = (4*words+2*shorts+extra);

    /* Type */
    buf[base+0] = (fsu->type & 0xFF00) / 0x100;
    buf[base+1] = (fsu->type & 0xFF);
    shorts++; base = COMPUTE(words,shorts,extra);

    if (fsu->type == FRSTAT_T_LINK) {
      if (!frEntStore(err,&(fsu->nbentity),&(buf[base]),ent_buf_sz)) {
        return 0;
      }
      extra += FRENT_PORTABLE_SIZE;
      base = (4*words+2*shorts+extra);
    }
        
    memcpy(&buf[base], fsu->relString, FRSTATUPD_RELEASE_STRING_SZ);
    extra += FRSTATUPD_RELEASE_STRING_SZ;
    base = (4*words+2*shorts+extra);

    buf[base+0] = (vcnt & 0xFF00) / 0x100;
    buf[base+1] = (vcnt & 0xFF);
    shorts++; base = COMPUTE(words,shorts,extra);

    buf[base+0] = (fsu->history & 0xFF00) /0x100;
    buf[base+1] = (fsu->history & 0xFF);
    shorts++; base = COMPUTE(words,shorts,extra);

    
    for (n = 0; n < FRSTATUPD_TOTAL_PARMS; n++) if (fsu->set[n]) {
      buf[base++] = (n & 0xFF00) / 0x100;
      buf[base++] = (n & 0xFF);
      pv = fsu->pv[n];
      
      switch (n) {
      case FRSTATUPD_BYTES_L_SENT:
      case FRSTATUPD_BYTES_L_RCVD:
      case FRSTATUPD_BYTES_L_DROP:
      case FRSTATUPD_PKTS_L_SENT:
      case FRSTATUPD_PKTS_L_RCVD:
      case FRSTATUPD_PKTS_L_DROP:
        
      case FRSTATUPD_BYTES_H_SENT:
      case FRSTATUPD_BYTES_H_RCVD:
      case FRSTATUPD_BYTES_H_DROP:
      case FRSTATUPD_PKTS_H_SENT:
      case FRSTATUPD_PKTS_H_RCVD:
      case FRSTATUPD_PKTS_H_DROP:
        
      case FRSTATUPD_CONNECTIONS:
      case FRSTATUPD_LAUNCHTIME:
      case FRSTATUPD_SEQNO:
      case FRSTATUPD_LNKSTAT:
      case FRSTATUPD_NIQS_RELOAD:
      case FRSTATUPD_LAST_STATUS:
      case FRSTATUPD_CURR_CONNECTS:
      case FRSTATUPD_CURR_PORTS:
      case FRSTATUPD_CONSTANT:
      case FRSTATUPD_AIP_NEIGHB:
      case FRSTATUPD_CURRTIME:
        if (!frStatUpdGenericToBuffer(err, fsu, n, pv, buf, &base)) {
          return 0;
        }
        break;

      default:
        zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD, 
                      "Unknown type %d", n);
        return 0;
        break;
      }
    }

    *b = buf;
    *sz = base;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}


static int frStatUpdGenericFromBuffer(ZkError err, FrStatUpd *fsu, 
                                      FrStatUpdParmVal *pv, unsigned char *buf,
                                      size_t *bptr)
{
  const char *func = "frStatUpdGenericFromBuffer";
  size_t m;
  size_t base;
  
  if (!fsu || !*fsu || !pv || !bptr) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  base = *bptr;
  for (m = 0; m < (*fsu)->history; m++) {
    pv[m].t = (buf[base + 0] * 0x1000000) + (buf[base + 1] * 0x10000) +
      (buf[base + 2] * 0x100) + buf[base + 3];
    pv[m].v = (buf[base + 4] * 0x1000000) + (buf[base + 5] * 0x10000) +
      (buf[base + 6] * 0x100) + buf[base + 7];
    base += 8;
  }   
  *bptr = base;

  return 1;
}

/*
 *
 */
DLLExport int frStatUpdFromBuffer(ZkError err, FrStatUpd *fsu, void *b, 
                                  size_t sz)
{
  const char *func = "frStatUpdFromBuffer";
  int ret = 0;
  size_t bsz = 0;
  size_t history = 0;
  size_t count = 0;
  uint32_t sigvers = 0;
  size_t n = 0;
  int base = 0;
  FrStatUpdParmVal * pv = NULL;
  FrStatUpdParameter parm;
  size_t words = 0, shorts = 0, extra = 0;
  unsigned char * buf = NULL;
  size_t ent_buf_sz = FRENT_PORTABLE_SIZE;

  if (!fsu || !b) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frStatUpdNew(err, fsu)) {
    return 0;
  }

#   define COMPUTE(w,s,e) (INT32_T_BUF_SIZE*words+INT16_T_BUF_SIZE*shorts+extra)

  buf = (unsigned char *) b;
  bsz = buf[base+0]*0x1000000 + buf[base+1]*0x10000
    + buf[base+2]*0x100 + buf[base+3];
  words ++; base = COMPUTE(words,shorts,extra);

  if (bsz != sz) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD, 
                  "Buffer size mismatch (got %d/internal size %d)", sz, bsz);
    goto clean;
  }

  sigvers = buf[base+0]*0x1000000 + buf[base+1]*0x10000
    + buf[base+2]*0x100 + buf[base+3];
  words ++; base = COMPUTE(words,shorts,extra);

  /* Actually, this signature is more like a magic number */
  if ((sigvers & FRSTAT_DEFAULT_SIGNATURE) != FRSTAT_DEFAULT_SIGNATURE) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD,
               "Invalid signature on data block (got 0x%08x / expected 0x%08x)",
                  sigvers, FRSTAT_DEFAULT_SIGVERS);
    goto clean;
  }
  (*fsu)->sigvers = sigvers;
  
  if (!frEntLoad(err,&((*fsu)->entity),&(buf[base]),ent_buf_sz)) {
    return 0;
  }
  extra += FRENT_PORTABLE_SIZE; base = COMPUTE(words,shorts,extra);
  
  (*fsu)->type = buf[base+0] * 0x100 + buf[base+1];
  shorts ++; base = COMPUTE(words,shorts,extra);

  if ((*fsu)->type == FRSTAT_T_LINK) {
    if (!frEntLoad(err,&((*fsu)->nbentity),&(buf[base]),ent_buf_sz)) {
      return 0;
    }
    extra += FRENT_PORTABLE_SIZE;
    base = (4*words+2*shorts+extra);
  }

  memcpy((*fsu)->relString, &buf[base], FRSTATUPD_RELEASE_STRING_SZ);
  extra += FRSTATUPD_RELEASE_STRING_SZ;
  base = (4*words+2*shorts+extra);

  count = buf[base+0] * 0x100 + buf[base+1];
  shorts ++; base = COMPUTE(words,shorts,extra);

  if (count > FRSTATUPD_TOTAL_PARMS) {
    zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD,
                  "Too many statistics parameters [%d] in buffer", count);
    goto clean;
  }

  history = buf[base+0] * 0x100 + buf[base+1];
  shorts ++; base = COMPUTE(words,shorts,extra);


  if (!frStatUpdSetHistory(err, *fsu, history)) {
    goto clean;
  }

  for (n = 0; n < count; n++) {
    parm = buf[base] * 0x100 + buf[base + 1];
    (*fsu)->set[parm] = 1;
    (*fsu)->tail[parm] = 0;
    pv = (*fsu)->pv[parm];
    base += 2;
    
    switch (parm) {
      /* Take the rest of the action based on the parameter type */
    case FRSTATUPD_BYTES_L_SENT:
    case FRSTATUPD_BYTES_L_RCVD:
    case FRSTATUPD_BYTES_L_DROP:
    case FRSTATUPD_PKTS_L_SENT:
    case FRSTATUPD_PKTS_L_RCVD:
    case FRSTATUPD_PKTS_L_DROP:
      
    case FRSTATUPD_BYTES_H_SENT:
    case FRSTATUPD_BYTES_H_RCVD:
    case FRSTATUPD_BYTES_H_DROP:
    case FRSTATUPD_PKTS_H_SENT:
    case FRSTATUPD_PKTS_H_RCVD:
    case FRSTATUPD_PKTS_H_DROP:
      
    case FRSTATUPD_CONNECTIONS:
    case FRSTATUPD_LAUNCHTIME:
    case FRSTATUPD_SEQNO:
    case FRSTATUPD_LNKSTAT:
    case FRSTATUPD_NIQS_RELOAD:
    case FRSTATUPD_LAST_STATUS:
    case FRSTATUPD_CURR_CONNECTS:
    case FRSTATUPD_CURR_PORTS:
    case FRSTATUPD_CONSTANT:
    case FRSTATUPD_AIP_NEIGHB:
    case FRSTATUPD_CURRTIME:
      if (!frStatUpdGenericFromBuffer(err, fsu, pv, buf, &base)) {
        goto clean;
      }
      break;

    default:
      zkErrorSet_va(err, func, ZKERR, ZKMFRSTAT, ZKEDATABAD, 
                    "Unknown type %d", n);
      goto clean;
      break;
    }
  }
  ret = 1;

 clean:
  if (!ret) {
    (void) frStatUpdFree(NULL, fsu);
  }
  return ret;
}

/*
 * The assumption here is that the contents of both FrStatUpd structures
 * are in proper time order. (if they were created using the library functions
 * this should always be the case)
 */

DLLExport int
frStatUpdUpdate (
  ZkError err ,
  FrStatUpd fsu ,
  FrStatUpd upd )
{
  static const char * func = "frStatUpdUpdate";
  FrStatUpdParmVal * pv = NULL;
  FrStatUpdParmVal * fsu_pv = NULL;
  int return_code = 0;
  size_t n = 0;
  FrStatUpdParameter p = 0;
  time_t fsu_t = 0;
  time_t upd_t = 0;
  int ndx = 0;
  
  zkErrorClear(err);
  if (fsu && upd) {
    if (fsu != upd) {
      return_code = 1;
      for (p = 0; p < FRSTATUPD_TOTAL_PARMS; p++) if (upd->set[p]) {
        /* Get most recent time code for parm in FrStatUpd to be updated */
        if (!frStatUpdGetParameter(err,fsu,p,0,NULL,&fsu_t)) {
          fsu_t = 0;
        }

    
        /* Get the value arrays for both FrStatUpd structures.  Make sure
         * they are not NULL, although this is really likely overkill
         */
        if ((pv = upd->pv[p]) == NULL) {
          zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,
                     "FrStatUpd source data structure is corrupt");
          return_code = 0;
          break;
        }
        if ((fsu_pv = fsu->pv[p]) == NULL) {
          zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,
                     "FrStatUpd target data structure is corrupt");
          return_code = 0;
          break;
        }
  
        /* Only add the values to the fsu if the time is equal to or later
         * than the fsu's most recent entry for that parameter.  We dont
         * want to use the frStatUpdSetParameter() function here because we
         * want to explicitly set the time.
         */
        fsu->set[p] = 1;
        for (n = 0; n < upd->history; n++) {
          ndx = (upd->tail[p] + n) % upd->history;
          upd_t = pv[ndx].t;
          if (upd_t >= fsu_t) {
            memcpy(&fsu_pv[fsu->tail[p]].v, &pv[ndx].v, 
                   sizeof(FrStatUpdParm));
            fsu_pv[fsu->tail[p]].t = upd_t;
            fsu->tail[p] = (fsu->tail[p] + 1) % fsu->history;
            fsu_t = upd_t;
          }
        }
      }
    } else {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEPARMBAD,
                 "You cannot update an FrStatUpd with itself");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}
    

DLLExport int
frStatUpdSetEntity2 (
  ZkError err ,
  FrStatUpd fsu ,
  const char * name ,
  FrEntIdType type )
{
  static const char * func = "frStatUpdSetEntity2";
  int return_code = 0;
  FrEntIdExp entity;

  zkErrorClear(err);
  if (fsu && name) {
    memset(&entity,0,sizeof(FrEntIdExp));
    entity.type = type;
    strncpy(entity.name,name,FRENT_NAME_SIZE);
    if (frStatUpdSetEntity(err,fsu,&entity)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

DLLExport int
frStatUpdSetNbEntity2 (
  ZkError err ,
  FrStatUpd fsu ,
  const char * name ,
  FrEntIdType type )
{
  static const char *func = "frStatUpdSetNbEntity2";
  int return_code = 0;
  FrEntIdExp entity;

  zkErrorClear(err);
  if (fsu && name) {
    memset(&entity,0,sizeof(FrEntIdExp));
    entity.type = type;
    strncpy(entity.name,name,FRENT_NAME_SIZE);
    if (frStatUpdSetNbEntity(err,fsu,&entity)) {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

#if 0
DLLExport int
frStatUpd (
  ZkError err ,
  FrStatUpd fsu )
{
  static const char * func = "frStatUpd";
  int return_code = 0;

  zkErrorClear(err);
  if (fsu) {

    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}


static int frStatUpdAipStatusFromBuffer(ZkError err, FrStatUpd *fsu, 
                                        FrStatUpdParmVal *pv,
                                        unsigned char *buf,
                                        size_t *bptr)
{
  const char *func = "frStatUpdGenericFromBuffer";
  size_t m;
  size_t base;
  
  if (!fsu || !*fsu || !pv || !bptr) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  base = *bptr;
  for (m = 0; m < (*fsu)->history; m++) {
    pv[m].t = (buf[base + 0] * 0x1000000) + (buf[base + 1] * 0x10000) +
      (buf[base + 2] * 0x100) + buf[base + 3];
    base += 4;

    if (!frEntLoad(err, &(pv[m].v), &buf[base], 
                   FRENT_PORTABLE_SIZE)) {
      return 0;
    }
    base += FRENT_PORTABLE_SIZE;

    pv[m].v = buf[base++];
  }   
  *bptr = base;

  return 1;
}

static int frStatUpdAipStatsFromBuffer(ZkError err, FrStatUpd *fsu, 
                                       FrStatUpdParmVal *pv, 
                                       unsigned char *buf, size_t *bptr)
{
  const char *func = "frStatUpdGenericFromBuffer";
  size_t m;
  size_t base;
  
  if (!fsu || !*fsu || !pv || !bptr) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  base = *bptr;
  for (m = 0; m < (*fsu)->history; m++) {
    pv[m].t = (buf[base + 0] * 0x1000000) + (buf[base + 1] * 0x10000) +
      (buf[base + 2] * 0x100) + buf[base + 3];
    base += 4;

    if (!frEntLoad(err, &(pv[m].v), &buf[base], 
                   FRENT_PORTABLE_SIZE)) {
      return 0;
    }
    base += FRENT_PORTABLE_SIZE;

    pv[m].v.aip.sent = 
      (buf[base + 0] * 0x1000000) + (buf[base + 1] * 0x10000) +
      (buf[base + 2] * 0x100) + buf[base + 3];
    base += 4;

    pv[m].v.aip.rcvd =
      (buf[base + 0] * 0x1000000) + (buf[base + 1] * 0x10000) +
      (buf[base + 2] * 0x100) + buf[base + 3];
    base += 4;
  }   
  *bptr = base;

  return 1;
}


DLLExport int
frStatUpdFromBuffer (
  ZkError err ,
  FrStatUpd * fsu ,
  void * b ,
  size_t sz)
{
  static const char * func = "frStatUpdFromBuffer";
  int return_code = 0;
  unsigned char * buf = (unsigned char *) b;
  FrStatUpdParameter parm;
  size_t bsz = 0;
  size_t history = 0;
  size_t count = 0;
  size_t n = 0;
  size_t m = 0;
  int base = 0;
  FrStatUpdParmVal * pv = NULL;
  size_t ent_buf_sz = 0;

  zkErrorClear(err);
  if (fsu && b) {
    if (!frStatUpdNew(err,fsu)) {
      return 0;
    }

    bsz = buf[0] * 0x100 + buf[1]; 
    count = buf[2] * 0x100 + buf[3];
    history = buf[4] * 0x100 + buf[5];

    if (count > FRSTATUPD_PARMS) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "Too many statistics parameters in buffer");
      goto finish;
    }
    if (bsz != sz) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,"Buffer size mismatch");
      goto finish;
    }
    if (!frStatUpdSetHistory(err,*fsu,history)) {
      goto finish;
    }
    ent_buf_sz = buf[6] * 0x100 + buf[7];
    if (buf[8 + ent_buf_sz] != (char) 0) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                 "Owning entity buffer is not NUL terminated");
      goto finish;
    }
    if (!frEntLoad(err,&((*fsu)->entity),&(buf[8]),ent_buf_sz)) {
      goto finish;
    }

    base = 8 + ent_buf_sz + 1;
    for (n = 0; n < count; n++) {
      parm = buf[base] * 0x100 + buf[base + 1];
      (*fsu)->set[parm] = 1;
      (*fsu)->tail[parm] = 0;
      pv = (*fsu)->pv[parm];
      base += 2;
      for (m = 0; m < history; m++) {
        if (buf[base + 8] != (char) 0) {
          zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                        "Parameter %d value component %d is corrupted",
                        parm,m);
          goto finish;
        }
        pv[m].t = (buf[base+0]*0x1000000) + (buf[base+1]*0x10000) +
                  (buf[base+2]*0x100) + buf[base+3];
        pv[m].v.v = (buf[base+4]*0x1000000) + (buf[base+5]*0x10000) +
                  (buf[base+6]*0x100) + buf[base+7];
        base+= 9;
      }   
      if (buf[base] != (char) 0) {
        zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKEDATABAD,
                      "Parameter %d datablock is not NUL terminated",parm);
        goto finish;
      } 
      base++;
    }

    return_code = 1;

finish:
    if (return_code == 0) {
      frStatUpdFree(NULL,fsu);
    }

    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  return 0;
}
#endif
