/*
 * Encapuslation of freedom stats
 *
 * $Id: frstats.c,v 1.24 2001/04/19 20:31:15 alainc Exp $ 
 *
 * Luc Boulianne <luc@zeroknowledge.com>
 */

static const char * module_id_ = "$Id: frstats.c,v 1.24 2001/04/19 20:31:15 alainc Exp $";


#include "zks.h"
#include "zkIntTypes.h"
#include <netinet/in.h>
#include <sys/socket.h>
#include "frStatClient.h"
#include "frStatUpd.h"
#include "frstats.h"
#include <stdlib.h>
#include <time.h>
#include "zkDebug.h"


#define UNINT64_L32_MASK 0xffffffffULL
#define UNINT64_H32_MASK 0xffffffff00000000ULL
#define UINT64_B33_MASK  0x100000000ULL

#define DUMP32(x)  zkDebug("%-20s: %d", #x, (uint32_t) x);
#define DUMP64(x)  zkDebug("%-20s: %Lu", #x, (UINT64_T) x);
#define DUMP_TIME(x)  zkDebug("%-20s: [0x%08lx] %s", #x, x, ctime(&(x)));
#define DUMP_ENT(x) zkDebug("%-20s: %s [%s]", #x, x->name, frEntTypeGetStr(x->type));

/*
 * This whole was of doing things is an absolute hack. I have three
 * days left. It will need to be replaced by something more flexible.
 * 
 */
 
struct T_FrStatsCtx {
  FrStatUpd su;                 /* Stat update object */
  FrStatClient sc;              /* Stat client object - only used on client*/
  int bNewWithNoSU;             /* FLAG: Is Ctx allocated with frStatsNewWithNoSU ? (SR) */

#if LB_TRASH
  /* Unused and will never likely be used */
  time_t last_stats_xmit;
  uint32_t seqno;
#endif
};


int frStatsNew(ZkError err, FrStatsCtx *ctx)
{
  const char *func = "frStatsNew";

  zkDebug("Enter %s", func);

  zkErrorClear(err);
  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "FrStatsCtx is NULL");
    return 0;
  }

  if ((*ctx = malloc(sizeof(struct T_FrStatsCtx))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENOMEM, "malloc() failed");
    return 0;
  }
  (*ctx)->su = NULL;
  (*ctx)->sc = NULL;
  (*ctx)->bNewWithNoSU = ZK_FALSE;

  if (!frStatUpdNew(err, &((*ctx)->su))) {
    return 0;
  }
    
  if (!frStatClientNew(err, &((*ctx)->sc))) {
    return 0;
  }

  return 1;
}

int frStatsNewWithNoSU(ZkError err, FrStatsCtx *ctx)
{
  const char *func = "frStatsNewWithNoSU";

  (void) module_id_;

  zkDebug("Enter %s", func);

  zkErrorClear(err);
  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "FrStatsCtx is NULL");
    return 0;
  }

  if ((*ctx = malloc(sizeof(struct T_FrStatsCtx))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENOMEM, "malloc() failed");
    return 0;
  }
  (*ctx)->su = NULL;
  (*ctx)->sc = NULL;
  (*ctx)->bNewWithNoSU = ZK_TRUE;

  if (!frStatClientNew(err, &((*ctx)->sc))) {
    return 0;
  }

  return 1;
}

int frStatsFree(ZkError err, FrStatsCtx *ctx)
{
  const char *func = "frStatsFree";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  zkErrorClear(err);
  if (!ctx || !*ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "context is NULL");
    return 0;
  }

  if ((*ctx)->sc) {
    /* Make sure that the client is currently connected, to avoid the 
        WARNING message produces by frStatClientDisconnect (SR) */
    if (frStatIsClientConnected(err, (*ctx)->sc)) {
      (void) frStatClientDisconnect(err, (*ctx)->sc);
    }
    (void) frStatClientFree(err, &(*ctx)->sc);
  }

  if ((!(*ctx)->bNewWithNoSU) && (*ctx)->su) {
    (void) frStatUpdFree(err, &(*ctx)->su);
  }

  free(*ctx);
  *ctx = NULL;
  
  return 1;
}

/*  
 * Determine if the current stats server is the same as the one 
 * corresponding to hname and port.  Result returned in same
 */
int frStatsIsServer(ZkError err, FrStatsCtx ctx, const char *hname, 
                    unsigned short port, int *same)
{
  const char *func = "frStatsUpdateServer";
  
  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatClientIsServer(err, ctx->sc, hname, port, same)) {
    return 0;
  }

  return 1;
}

/*  
 * Disconnect from current stats server and connect to the one
 * given by hname and port
 */
int frStatsUpdateServer(ZkError err, FrStatsCtx ctx, const char *hname,
                        unsigned short port)
{
  const char *func = "frStatsUpdateServer";


  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatClientDisconnect(err, ctx->sc)) {
    return 0;
  }

  if (!frStatClientSetHostname(err, ctx->sc, hname)) {
    return 0;
  }

  if (!frStatClientSetPort(err, ctx->sc, port)) {
    return 0;
  }

  if (!frStatClientConnect(err, ctx->sc)) {
    return 0;
  }

  return 1;
}


int frStatsSetServer(ZkError err, FrStatsCtx ctx, const char *hname, 
                     unsigned short port)
{
  const char *func = "frStatsSetServer";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatClientSetHostname(err, ctx->sc, hname)) {
    return 0;
  }

  if (!frStatClientSetPort(err, ctx->sc, port)) {
    return 0;
  }

  if (!frStatClientConnect(err, ctx->sc)) {
    return 0;
  }

  return 1;
}


int frStatsGetServer(ZkError err, FrStatsCtx ctx, int *pSock,
                     struct sockaddr *pSockAddr, socklen_t *pAddrSize)
{
  const char *func = "frStatsGetServer";
  int rc = 1;


  zkErrorClear(err);
  if (!ctx || !ctx->sc || !pSock || !pSockAddr || !pAddrSize) 
  {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "given NULL parameter");
    rc = 0;
  }
  else if ( !frStatClientGetSocket(err, ctx->sc, pSock, pSockAddr, pAddrSize) ) 
  {
    rc = 0;
  }

  return rc;
}

int frStatsSetMyEntity(ZkError err, FrStatsCtx ctx, FrEntIdExp myEntity)
{
  const char *func = "frStatsSetMyEntity";

  zkErrorClear(err);
  if (!ctx || !ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatUpdSetEntity(err, ctx->su, &myEntity)) {
    return 0;
  }

  return 1;
}

int frStatsSetMyInfo(ZkError err, FrStatsCtx ctx, FrEntIdExp myEntity,
                     const char *relString, size_t sz,
                     FrCryptPubKey mySigKey)
{
  const char *func = "frStatsSetMyInfo";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc || ! mySigKey) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatUpdSetEntity(err, ctx->su, &myEntity)) {
    return 0;
  }

  if (!frStatUpdSetReleaseString(err, ctx->su, relString, sz)) {
    return 0;
  }
  
  if (!frStatClientSetSignatureKey(err, ctx->sc, mySigKey)) {
    return 0;
  }

  return 1;
}

/**
 * Wrapper for frStatUpdSetType (note: no trailing `s' on Stat)
 * allows us to change context
 */
int frStatsUpdSetType (
  ZkError err ,
  FrStatsCtx ctx, 
  FrStatUpdTypes type )
{
  return (frStatUpdSetType (err, ctx->su, type));
}


int frStatsSetNISSCurrentCryptKey(ZkError err, FrStatsCtx ctx, 
                                 FrCryptPubKey nissPubKey)
{
  static const char *func = "frStatsSetNISSCurrentCryptKey";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc || !nissPubKey) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatClientSetNISSCryptKey(err, ctx->sc, nissPubKey)) {
    return 0;
  }

  return 1;
}

int frStatsSetHistory(ZkError err, FrStatsCtx ctx, size_t history)
{
  const char *func = "frStatsSetHistory";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  zkErrorClear(err);
  if (!ctx || !ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatUpdSetHistory(err, ctx->su, history)) {
    return 0;
  }

  return 1;
}

int frStatsSetUpdate(
    ZK_INOUT ZkError err, 
    ZK_INOUT FrStatsCtx ctx,
    ZK_IN    FrStatUpd fsu)
{
  static const char *func = "frStatsSetUpdate";
  int retcode = 0;
  
  if (!ctx || !fsu) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is non-NULL:"
               " did you forget to free the previous one? ");
  } else {
    ctx->su = fsu;
    retcode = 1;
  }
  return retcode;
}


int frStatsClearUpdate(
    ZK_INOUT ZkError err, 
    ZK_INOUT FrStatsCtx ctx)
{
  static const char *func = "frStatsClearUpdate";
  int retcode = 0;
  
  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else {
    ctx->su = 0;
    retcode = 1;
  }
  return retcode;
}
  
  
  
/*
 * Mapping: 
 *  FRSTATUPD_LAUNCHTIME = launch time
 *  FRSTATUPD_SEQNO = sequence number
 *  FRSTATUPD_NIQS_RELOAD = last time we reloaded from the NIQS
 *  FRSTATUPD_LAST_STATUS = last time we sent a stats packet
 *  FRSTATUPD_CONNECTIONS = number of client connections in the last interval
 *  FRSTATUPD_CURR_CONNECTS = number of currently connected clients
 *  FRSTATUPD_BYTES_SENT = 1K bytes (client) sent
 *  FRSTATUPD_BYTES_RCVD = 1K bytes (client) rcvd
 *  FRSTATUPD_BYTES_H_DROP = packets dropped
 */

int frStatsFillAipMsg(
                      ZkError err, 
                      FrStatsCtx ctx, 
                      uint32_t seqno,
                      time_t launchtime,
                      time_t last_niqs_reload_time, 
                      time_t last_status_time,
                      uint32_t new_client_connections,
                      uint32_t curr_connected_clients,
                      UINT64_T client_pkts_rx,
                      UINT64_T client_pkts_tx,
                      UINT64_T client_pkts_dropped,
                      UINT64_T client_bytes_rx,
                      UINT64_T client_bytes_tx,
                      UINT64_T client_bytes_dropped
                      )
{
  static const char *func = "frStatsFillAipMsg";
  int retcode = 0;
  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;
  uint32_t bytes_h_xx = 0, bytes_l_xx = 0;

  uint32_t pkts_h_rx = 0, pkts_l_rx = 0;
  uint32_t pkts_h_tx = 0, pkts_l_tx = 0;
  uint32_t pkts_h_xx = 0, pkts_l_xx = 0;

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif

#ifndef NDEBUG  
#if FRSTAT_FAKE_UPDATES
    zkErrorSet(err, func, ZKWARN, ZKMFRSTAT, ZKEDATABAD,
               "Substituting all stats values with fake data");

    new_client_connections = 100;
    curr_connected_clients = 110;
    client_bytes_rx = 1000*1024;
    client_bytes_tx = 100*1024;
    client_bytes_dropped = 10*1024;
    client_pkts_rx = 10000;
    client_pkts_tx = 10;
    client_pkts_dropped = 1;
#endif
#endif    
  

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURRTIME, time(NULL)) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_SEQNO, seqno) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, launchtime ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, last_status_time ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_NIQS_RELOAD, last_niqs_reload_time ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, new_client_connections ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, curr_connected_clients )) {
    
    bytes_l_rx = client_bytes_rx & UNINT64_L32_MASK;
    bytes_h_rx = (client_bytes_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_tx = client_bytes_tx & UNINT64_L32_MASK;
    bytes_h_tx = (client_bytes_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_xx = client_bytes_dropped & UNINT64_L32_MASK;
    bytes_h_xx = (client_bytes_dropped & UNINT64_H32_MASK) / UINT64_B33_MASK;

    pkts_l_rx = client_pkts_rx & UNINT64_L32_MASK;
    pkts_h_rx = (client_pkts_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    pkts_l_tx = client_pkts_tx & UNINT64_L32_MASK;
    pkts_h_tx = (client_pkts_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    pkts_l_xx = client_pkts_dropped & UNINT64_L32_MASK;
    pkts_h_xx = (client_pkts_dropped & UNINT64_H32_MASK) / UINT64_B33_MASK;
      
    if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, bytes_l_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, bytes_h_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, bytes_l_tx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, bytes_h_tx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_DROP, bytes_l_xx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_DROP, bytes_h_xx) &&

        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, pkts_l_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_RCVD, pkts_h_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, pkts_l_tx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_SENT, pkts_h_tx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, pkts_l_xx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_DROP, pkts_h_xx)) {
      
      retcode = 1;
      zkDebug ("AIP: Outgoing");
      DUMP32(seqno);
      DUMP_TIME(launchtime);
      DUMP_TIME(last_niqs_reload_time);
      DUMP_TIME(last_status_time);
      DUMP32(new_client_connections);
      DUMP32(curr_connected_clients);
      DUMP64(client_bytes_rx);
      DUMP64(client_bytes_tx);
      DUMP64(client_bytes_dropped);
      DUMP64(client_pkts_rx);
      DUMP64(client_pkts_tx);
      DUMP64(client_pkts_dropped);
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be set.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be set.");
  }
    
  return retcode;
}

int frStatsReadAipMsg(
                      ZkError err, 
                      FrStatsCtx ctx, 
                      time_t *reptime,
                      uint32_t *seqno,
                      time_t *launchtime,
                      time_t *last_niqs_reload_time, 
                      time_t *last_status_time,
                      uint32_t *new_client_connections,
                      uint32_t *curr_connected_clients,
                      UINT64_T *client_pkts_rx,
                      UINT64_T *client_pkts_tx,
                      UINT64_T *client_pkts_dropped,
                      UINT64_T *client_bytes_rx,
                      UINT64_T *client_bytes_tx,
                      UINT64_T *client_bytes_dropped
                      )
{
  static const char *func = "frStatsReadAipMsg";
  int retcode = 0;
  time_t t = 0;
  size_t h = 0; /* sizeof(uint32_t); */

  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;
  uint32_t bytes_h_xx = 0, bytes_l_xx = 0;
  
  uint32_t pkts_h_rx = 0, pkts_l_rx = 0;
  uint32_t pkts_h_tx = 0, pkts_l_tx = 0;
  uint32_t pkts_h_xx = 0, pkts_l_xx = 0;


#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  if (!ctx || !reptime || !seqno || !launchtime || !last_niqs_reload_time ||
      !last_status_time || !new_client_connections || !curr_connected_clients ||
      !client_bytes_rx || !client_bytes_tx || !client_bytes_dropped ||
      !client_pkts_rx || !client_pkts_tx || !client_pkts_dropped) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx or other param is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURRTIME, h, (uint32_t *)reptime, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_SEQNO, h, seqno, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, h,  (uint32_t *)launchtime , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, h, (uint32_t *)last_status_time , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_NIQS_RELOAD, h, (uint32_t *)last_niqs_reload_time , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, h, new_client_connections , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, h, curr_connected_clients , &t)) {
    
    if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, h, &bytes_l_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, h, &bytes_h_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, h, &bytes_l_tx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, h, &bytes_h_tx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_DROP, h, &bytes_l_xx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_DROP, h, &bytes_h_xx, &t) &&

        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, h, &pkts_l_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_RCVD, h, &pkts_h_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, h, &pkts_l_tx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_SENT, h, &pkts_h_tx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, h, &pkts_l_xx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_DROP, h, &pkts_h_xx, &t) ) {

      *client_bytes_rx = bytes_l_rx + bytes_h_rx * UINT64_B33_MASK;
      *client_bytes_tx = bytes_l_tx + bytes_h_tx * UINT64_B33_MASK;
      *client_bytes_dropped = bytes_l_xx + bytes_h_xx * UINT64_B33_MASK;

      *client_pkts_rx = pkts_l_rx + pkts_h_rx * UINT64_B33_MASK;
      *client_pkts_tx = pkts_l_tx + pkts_h_tx * UINT64_B33_MASK;
      *client_pkts_dropped = pkts_l_xx + pkts_h_xx * UINT64_B33_MASK;

      zkDebug ("AIP: Incoming");
      DUMP_TIME(*reptime);
      DUMP32(*seqno);
      DUMP_TIME(*launchtime);
      DUMP_TIME(*last_niqs_reload_time);
      DUMP_TIME(*last_status_time);
      DUMP32(*new_client_connections);
      DUMP32(*curr_connected_clients);
      DUMP64(*client_bytes_rx);
      DUMP64(*client_bytes_tx);
      DUMP64(*client_bytes_dropped);
      DUMP64(*client_pkts_rx);
      DUMP64(*client_pkts_tx);
      DUMP64(*client_pkts_dropped);
      retcode = 1;
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be read.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be read.");
  }
  return retcode;
}

int frStatsSetMyCurrentKey(ZkError err, FrStatsCtx as, 
                           FrCryptPubKey myCurrentKey)
{
  static const char *func = "frStatsSetMyCurrentKey";

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif



  zkErrorClear(err);
  if (!as || !as->su || !as->sc || !myCurrentKey) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  if (!frStatClientSetSignatureKey(err, as->sc, myCurrentKey)) {
    return 0;
  }

  return 1;
}

/*
 * Mapping: 
 *  FRSTATUPD_LAUNCHTIME = launch time
 *  FRSTATUPD_SEQNO = sequence number
 *  FRSTATUPD_CONSTANT = maximum throughput
 *  FRSTATUPD_CONNECTIONS = number streams
 *  FRSTATUPD_CURR_CONNECTS = number active streams
 *  FRSTATUPD_CURR_PORTS = number used ports
 *  FRSTATUPD_BYTES_SENT = 1K bytes (client) sent
 *  FRSTATUPD_BYTES_RCVD = 1K bytes (client) rcvd
 */

int frStatsFillWormholeMsg(
                           ZkError err, 
                           FrStatsCtx ctx, 
                           uint32_t seqno,
                           time_t launchtime,
                           time_t last_status_time,
                           uint32_t maxin_thrput,
                           uint32_t new_streams,
                           uint32_t current_active_streams,
                           uint32_t new_ports,
                           uint32_t curr_used_ports, 
                           UINT64_T new_bytes_rx,
                           UINT64_T new_bytes_tx)
{
  static const char *func = "frStatsFillWormhole";
  int retcode = 0;
  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;

#ifndef NDEBUG
  zkDebug("Enter %s", func);
#endif

#ifndef NDEBUG  
#if FRSTAT_FAKE_UPDATES
    zkErrorSet(err, func, ZKWARN, ZKMFRSTAT, ZKEDATABAD,
               "Substituting all stats values with fake data");

    maxin_thrput = 100*1024;
    new_streams = 1000;
    current_active_streams = 100;
    new_ports = 1000;
    curr_used_ports = 100;
    new_bytes_rx = 10000*1024;
    new_bytes_tx = 100000*1024;
#endif
#endif    


  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURRTIME, time(NULL)) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_SEQNO, seqno) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, launchtime ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, last_status_time ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CONSTANT, maxin_thrput ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, new_streams ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, current_active_streams ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LNKSTAT, new_ports ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURR_PORTS, curr_used_ports )) {
    
    bytes_l_rx = new_bytes_rx & UNINT64_L32_MASK;
    bytes_h_rx = (new_bytes_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_tx = new_bytes_tx & UNINT64_L32_MASK;
    bytes_h_tx = (new_bytes_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    
    if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, bytes_l_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, bytes_h_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, bytes_l_tx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, bytes_h_tx)) {
      
      retcode = 1;
      zkDebug ("WORMHOLE: Outgoing");
      DUMP32(seqno);
      DUMP_TIME(launchtime);
      DUMP_TIME(last_status_time);
      DUMP32(maxin_thrput);
      DUMP32(new_streams);
      DUMP32(current_active_streams);
      DUMP32(new_ports);
      DUMP32(curr_used_ports);
      DUMP64(new_bytes_rx);
      DUMP64(new_bytes_tx);
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be set.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be set.");
  } 
  return retcode;
}


int frStatsReadWormholeMsg(
                           ZkError err, 
                           FrStatsCtx ctx, 
                           time_t *reptime,
                           uint32_t *seqno,
                           time_t *launchtime,
                           time_t *last_status_time,
                           uint32_t *maxin_thrput,
                           uint32_t *new_streams,
                           uint32_t *current_active_streams,
                           uint32_t *new_ports,
                           uint32_t *curr_used_ports, 
                           UINT64_T *new_bytes_rx,
                           UINT64_T *new_bytes_tx)
{
  static const char *func = "frStatsReadWormholeMsg";
  int retcode = 0;
  time_t t = 0;
  size_t h = 0; /* sizeof(uint32_t);*/
  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif



  if (!ctx || !reptime || !seqno || !launchtime || !last_status_time || !maxin_thrput ||
      !new_streams || !current_active_streams || !new_ports || !curr_used_ports ||
      !new_bytes_rx || !new_bytes_tx ) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURRTIME, h, (uint32_t *)reptime, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_SEQNO, h, seqno, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, h, (uint32_t *)launchtime, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, h, (uint32_t *)last_status_time, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CONSTANT, h, maxin_thrput, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, h, new_streams, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, h, current_active_streams, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LNKSTAT, h, new_ports, &t ) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURR_PORTS, h, curr_used_ports, &t )) {
    
    if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, h, &bytes_l_rx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, h, &bytes_h_rx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, h, &bytes_l_tx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, h, &bytes_h_tx, &t)) {
      
      *new_bytes_rx = bytes_l_rx + bytes_h_rx * UINT64_B33_MASK;
      *new_bytes_tx = bytes_l_tx + bytes_h_tx * UINT64_B33_MASK;
      retcode = 1;

      zkDebug ("WORMHOLE: Incoming");
      DUMP_TIME(*reptime);
      DUMP32(*seqno);
      DUMP_TIME(*launchtime);
      DUMP_TIME(*last_status_time);
      DUMP32(*maxin_thrput);
      DUMP32(*new_streams);
      DUMP32(*current_active_streams);
      DUMP32(*new_ports);
      DUMP32(*curr_used_ports);
      DUMP64(*new_bytes_rx);
      DUMP64(*new_bytes_tx);
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be read.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be read.");
  }
  return retcode;
}

/*
 * Mapping: 
 *  FRSTATUPD_LAUNCHTIME = launch time
 *  FRSTATUPD_SEQNO = sequence number
 *  FRSTATUPD_AIP_NEIGHB = neighbour information (entity name).
 *  FRSTATUPD_NIQS_RELOAD = last time we reloaded from the NIQS
 *  FRSTATUPD_LAST_STATUS = last time we sent a stats packet
 *  FRSTATUPD_AIP_NBSTATUS = are we up (1) or down (0)
 *  FRSTATUPD_PKTS_RCVD = Messages rcvd
 *  FRSTATUPD_PKTS_SENT = Messages sent
 *  FRSTATUPD_PKTS_DROP = packets dropped
 *  FRSTATUPD_BYTES_SENT = 1K bytes (client) sent
 *  FRSTATUPD_BYTES_RCVD = 1K bytes (client) rcvd
 */

int frStatsFillLinkMsg(
                       ZkError err,
                       FrStatsCtx ctx, 
                       uint32_t seqno,
                       ZkBool up_flag,
                       FrEntIdExp *nb_entity,
                       time_t launchtime,   
                       time_t last_change_time,
                       time_t last_status_time, 
                       UINT64_T new_pkts_rx,
                       UINT64_T new_pkts_tx,
                       UINT64_T new_pkts_drop,
                       UINT64_T new_bytes_rx,
                       UINT64_T new_bytes_tx,
                       UINT64_T new_bytes_drop)
{     
  static const char *func = "frStatsFillLink";
  int retcode = 0;

  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;
  uint32_t bytes_h_xx = 0, bytes_l_xx = 0;

  uint32_t pkts_h_rx = 0, pkts_l_rx = 0;
  uint32_t pkts_h_tx = 0, pkts_l_tx = 0;
  uint32_t pkts_h_xx = 0, pkts_l_xx = 0;

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif

#ifndef NDEBUG  
#if FRSTAT_FAKE_UPDATES
    zkErrorSet(err, func, ZKWARN, ZKMFRSTAT, ZKEDATABAD,
               "Substituting all stats values with fake data");

    up_flag = 1;
    new_pkts_rx = 1000;
    new_pkts_tx = 1010;
    new_pkts_drop = 110;
    new_bytes_rx = 1000*1024;
    new_bytes_tx = 1010*1024;
    new_bytes_drop = 10*1024;
    
#endif
#endif    

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdSetNbEntity (err, ctx->su, nb_entity) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURRTIME, time(NULL)) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_SEQNO, seqno) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, launchtime ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, last_status_time ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_NIQS_RELOAD, last_change_time )) {
    
    bytes_l_rx = new_bytes_rx & UNINT64_L32_MASK;
    bytes_h_rx = (new_bytes_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_tx = new_bytes_tx & UNINT64_L32_MASK;
    bytes_h_tx = (new_bytes_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_xx = new_bytes_drop & UNINT64_L32_MASK;
    bytes_h_xx = (new_bytes_drop & UNINT64_H32_MASK) / UINT64_B33_MASK;

    pkts_l_rx = new_pkts_rx & UNINT64_L32_MASK;
    pkts_h_rx = (new_pkts_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    pkts_l_tx = new_pkts_tx & UNINT64_L32_MASK;
    pkts_h_tx = (new_pkts_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    pkts_l_xx = new_pkts_drop & UNINT64_L32_MASK;
    pkts_h_xx = (new_pkts_drop & UNINT64_H32_MASK) / UINT64_B33_MASK;

    if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, bytes_l_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, bytes_h_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, bytes_l_tx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, bytes_h_tx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_DROP, bytes_l_xx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_DROP, bytes_h_xx) &&
      
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, pkts_l_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_RCVD, pkts_h_rx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, pkts_l_tx ) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_SENT, pkts_h_tx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, pkts_l_xx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_H_DROP, pkts_h_xx) ) {

      if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LNKSTAT, up_flag )) {
        retcode = 1;
        DUMP32(seqno);
        DUMP32(up_flag);
        DUMP_ENT(nb_entity);
        DUMP_TIME(launchtime);
        DUMP_TIME(last_change_time);
        DUMP_TIME(last_status_time);
        DUMP64(new_pkts_rx);
        DUMP64(new_pkts_tx);
        DUMP64(new_pkts_drop);
        DUMP64(new_bytes_rx);
        DUMP64(new_bytes_tx);
        DUMP64(new_bytes_drop);
      } else {
        zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group C could not be set.");
      }
    } else {
      zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be set.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be set.");
  }
  return retcode;
}

int frStatsReadLinkMsg(
                       ZkError err,
                       FrStatsCtx ctx, 
                       time_t *reptime,
                       uint32_t *seqno,
                       ZkBool *up_flag,
                       const FrEntIdExp **nb_entity,
                       time_t *launchtime,  
                       time_t *last_change_time, 
                       time_t *last_status_time, 
                       UINT64_T *new_pkts_rx,
                       UINT64_T *new_pkts_tx,
                       UINT64_T *new_pkts_drop,
                       UINT64_T *new_bytes_rx,
                       UINT64_T *new_bytes_tx,
                       UINT64_T *new_bytes_drop)
{     
  static const char *func = "frStatsFillLink";
  int retcode = 0;

  time_t t = 0;
  size_t h = 0; /* sizeof(uint32_t); */
  
  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;
  uint32_t bytes_h_xx = 0, bytes_l_xx = 0;

  uint32_t pkts_h_rx = 0, pkts_l_rx = 0;
  uint32_t pkts_h_tx = 0, pkts_l_tx = 0;
  uint32_t pkts_h_xx = 0, pkts_l_xx = 0;


#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif


  if (!ctx || !reptime || !seqno || !nb_entity || !launchtime || !last_change_time ||
      !last_status_time || !new_pkts_rx || !new_pkts_tx || !new_pkts_drop ||
      !new_bytes_rx || !new_bytes_tx || !new_bytes_drop) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdNbEntity (err, ctx->su, nb_entity) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURRTIME, h, (uint32_t *)reptime, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_SEQNO, h, seqno, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, h, (uint32_t *)launchtime , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, h, (uint32_t *)last_status_time , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_NIQS_RELOAD, h, (uint32_t *)last_change_time , &t)) {

    if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, h, &bytes_l_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, h, &bytes_h_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, h, &bytes_l_tx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, h, &bytes_h_tx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_DROP, h, &bytes_l_xx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_DROP, h, &bytes_h_xx, &t) &&
        
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, h, &pkts_l_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_RCVD, h, &pkts_h_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, h, &pkts_l_tx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_SENT, h, &pkts_h_tx, &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, h, &pkts_l_xx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_H_DROP, h, &pkts_h_xx, &t) ) {
    
      *new_bytes_rx = bytes_l_rx + bytes_h_rx * UINT64_B33_MASK;
      *new_bytes_tx = bytes_l_tx + bytes_h_tx * UINT64_B33_MASK;
      *new_bytes_drop = bytes_l_xx + bytes_h_xx * UINT64_B33_MASK;

      *new_pkts_rx = pkts_l_rx + pkts_h_rx * UINT64_B33_MASK;
      *new_pkts_tx = pkts_l_tx + pkts_h_tx * UINT64_B33_MASK;
      *new_pkts_drop = pkts_l_xx + pkts_h_xx * UINT64_B33_MASK;

      if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LNKSTAT, h, (uint32_t *)up_flag, &t) ){ 
        
        DUMP_TIME(*reptime);
        DUMP32(*seqno);
        DUMP_ENT((*nb_entity));
        DUMP_TIME(*launchtime);
        DUMP_TIME(*last_change_time);
        DUMP_TIME(*last_status_time);
        DUMP64(*new_pkts_rx);
        DUMP64(*new_pkts_tx);
        DUMP64(*new_pkts_drop);
        DUMP64(*new_bytes_rx);
        DUMP64(*new_bytes_tx);
        DUMP64(*new_bytes_drop);
        DUMP32(*up_flag);

        retcode = 1;
      } else {
        zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group C could not be read.");
      }
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be read.");
    }
  } else {
      zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be read.");
  }

  return retcode;
}

int frStatsFillMaipMsg(
                       ZkError err, 
                       FrStatsCtx ctx, 
                       uint32_t seqno,
                       time_t launchtime,
                       time_t last_status_time,
                       uint32_t new_msgs_sent, 
                       uint32_t new_msgs_rcvd,
                       uint32_t new_msgs_dumped,
                       uint32_t num_queued_msgs,
                       UINT64_T bytes_rx, 
                       UINT64_T bytes_tx)
{
  static const char *func = "frStatsFillMaipMsg";
  int retcode = 0;

  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif

#ifndef NDEBUG  
#if FRSTAT_FAKE_UPDATES
    zkErrorSet(err, func, ZKWARN, ZKMFRSTAT, ZKEDATABAD,
               "Substituting all stats values with fake data");

    new_msgs_sent = 500;
    new_msgs_rcvd = 1500;
    new_msgs_dumped = 50;
    num_queued_msgs = 2500;
    bytes_rx = 500000*1024;
    bytes_tx = 50050*1024;
#endif
#endif    

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURRTIME, time(NULL)) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_SEQNO, seqno) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, launchtime ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, last_status_time ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, new_msgs_rcvd ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, new_msgs_sent ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, new_msgs_dumped ) &&
             frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, num_queued_msgs )) {
    
    bytes_l_rx = bytes_rx & UNINT64_L32_MASK;
    bytes_h_rx = (bytes_rx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    bytes_l_tx = bytes_tx & UNINT64_L32_MASK;
    bytes_h_tx = (bytes_tx & UNINT64_H32_MASK) / UINT64_B33_MASK;
    
    if (frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, bytes_l_rx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, bytes_h_rx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, bytes_l_tx) &&
        frStatUpdSetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, bytes_h_tx)) {
      DUMP32(seqno);
      DUMP_TIME(launchtime);
      DUMP_TIME(last_status_time);
      DUMP32(new_msgs_sent);
      DUMP32(new_msgs_rcvd);
      DUMP32(new_msgs_dumped);
      DUMP32(num_queued_msgs);
      DUMP64(bytes_rx);
      DUMP64(bytes_tx);
      retcode = 1;
    } else {
      zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be set.");
    }
  } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be set.");
  }
  return retcode;
}

int frStatsReadMaipMsg(
                       ZkError err, 
                       FrStatsCtx ctx,
                       time_t *reptime,
                       uint32_t *seqno,
                       time_t *launchtime,
                       time_t *last_status_time,
                       uint32_t *new_msgs_sent, 
                       uint32_t *new_msgs_rcvd,
                       uint32_t *new_msgs_dumped,
                       uint32_t *num_queued_msgs,
                       UINT64_T *bytes_rx, 
                       UINT64_T *bytes_tx)
{
  static const char *func = "frStatsReadMaipMsg";
  int retcode = 0;

  time_t t = 0;
  size_t h = 0; /* sizeof(uint32_t); */
  
  uint32_t bytes_h_rx = 0, bytes_l_rx = 0;
  uint32_t bytes_h_tx = 0, bytes_l_tx = 0;

#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif

  if (!ctx || !reptime || !seqno || !launchtime || !last_status_time || !new_msgs_sent
      || !new_msgs_rcvd || !new_msgs_dumped || !num_queued_msgs || !bytes_rx ||
      !bytes_tx ) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->cu is NULL");
  } else if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURRTIME, h, (uint32_t *)reptime, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_SEQNO, h, seqno, &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, h, (uint32_t *)launchtime , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_LAST_STATUS, h, (uint32_t *)last_status_time , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_RCVD, h, new_msgs_rcvd , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_SENT, h, new_msgs_sent , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_PKTS_L_DROP, h, new_msgs_dumped , &t) &&
             frStatUpdGetParameter(err, ctx->su, FRSTATUPD_CURR_CONNECTS, h, num_queued_msgs , &t)) {
    
    
    if (frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_RCVD, h, &bytes_l_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_RCVD, h, &bytes_h_rx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_L_SENT, h, &bytes_l_tx , &t) &&
        frStatUpdGetParameter(err, ctx->su, FRSTATUPD_BYTES_H_SENT, h, &bytes_h_tx, &t)) {
      
      *bytes_rx = bytes_l_rx + bytes_h_rx * UINT64_B33_MASK;
      *bytes_tx = bytes_l_tx + bytes_h_tx * UINT64_B33_MASK;

      DUMP_TIME(*reptime);
      DUMP32(*seqno);
      DUMP_TIME(*launchtime);
      DUMP_TIME(*last_status_time);
      DUMP32(*new_msgs_sent);
      DUMP32(*new_msgs_rcvd);
      DUMP32(*new_msgs_dumped);
      DUMP32(*num_queued_msgs);
      DUMP64(*bytes_rx);
      DUMP64(*bytes_tx);
      retcode = 1;
    } else {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group B could not be read.");
    }
  } else {
      zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKERROR, "Some variables from Group A (and all Group B) could not be read.");
  }
  return retcode;
}


int frStatsSendMsg(ZkError err, FrStatsCtx ctx)
{
  static const char *func = "frStatsSendMsg";
  
#ifdef XDEBUG
  zkDebug("Enter %s", func);
#endif

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
  } else if (!ctx->su) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is NULL");
  } else if (!frStatClientSubmitUpdate(err, ctx->sc, ctx->su)) {
    return 0;
  } else {
    return 1;
  }
  return 0;
}


int frStatsBuildMsg 
(
  ZkError    err ,
  FrStatsCtx ctx,
  void     **pkt,
  size_t    *pktSize
)
{
  int       rc = 1;
  void     *buf = NULL;
  size_t    bsz = 0;
  void     *packed = NULL;
  size_t    psz = 0;
  static const char * func = "frStatClientSubmitUpdate";


  zkErrorClear(err);

  if ( !ctx || !pkt || !pktSize )
  {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "given null parameter");
    rc = 0;
  }
  else if ( ctx->su == NULL )
  {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx->su is null");
    rc = 0;
  }

  /*
   *  Build stats packet and encapsulate it in an NISS msg
   */

  else if ( ! (frStatUpdToBuffer(err, ctx->su, &buf, &bsz) &&
               frStatPktUpdate(err, 1, 0, 0, buf, bsz, &packed, &psz)) )
  {
    rc = 0;
  }
  else
  {
    *pkt = packed;
    *pktSize = psz;
  }

  /* Clean up */
  if (buf) 
  { 
    memset(buf, 0, bsz); 
    free(buf); 
  }

  return rc;
}

#if 0
/**
 * Set the update time, but don't send it in
 */
static int frStatsUptimeNoUpdate(ZkError err, FrStatsCtx ctx, 
                                 time_t launchTime)
{
  const char *func = "frStatsUptime";
  FrStatUpdParmUnionExp v;

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  v.v = (uint32_t) time(NULL) - launchTime;
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, v)) {
    return 0;
  }

  return 1;
}

int frStatsUptime(ZkError err, FrStatsCtx ctx, time_t launchTime)
{
  const char *func = "frStatsUptime";
  FrStatUpdParmUnionExp v;

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  v.v = (uint32_t) time(NULL) - launchTime;
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_LAUNCHTIME, v)) {
    return 0;
  }

  if (!frStatClientSubmitUpdate(err, ctx->sc, ctx->su)) {
    return 0;
  }

  return 1;
}

int frStatsNewUpdate(ZkError err, FrStatsCtx ctx, FrEntIdExp entity, 
                     uint32_t sent, uint32_t rcvd, time_t launchTime,
                     uint32_t conn)
{
  const char *func = "frStatsNewUpdate";
  FrStatUpdParmUnionExp v;

  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  memcpy(&v.aip.entity, &entity, sizeof(FrEntIdExp));
  v.aip.rcvd = rcvd;
  v.aip.sent = sent;
  
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_AIP_NBSTATS, v)) {
    return 0;
  }

  if (!frStatsUptimeNoUpdate(err, ctx, launchTime)) {
    return 0;
  }

  if (!frStatsWormConnectionsNoUpdate(err, ctx, conn)) {
    return 0;
  }

  if (!frStatClientSubmitUpdate(err, ctx->sc, ctx->su)) {
    return 0;
  }

  return 1;
}

/**
 * Send an update that a remote server hctx gone up or down
 */
int frStatsNbUpdate(ZkError err, FrStatsCtx ctx, FrEntIdExp entity, int up)
{
  const char *func = "frStatsNbUpdate";
  FrStatUpdParmUnionExp v;

  zkErrorClear(err);
  if (!ctx || !ctx->su || !ctx->sc) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  memcpy(&v.aipStatus.entity, &entity, sizeof(FrEntIdExp));
  v.aipStatus.status = up;
  
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_AIP_NBSTATUS, v)) {
    return 0;
  }

  if (!frStatClientSubmitUpdate(err, ctx->sc, ctx->su)) {
    return 0;
  }

  return 1;
}


/**
 * Wormhole active connections
 */
int frStatsWormConnections(ZkError err, FrStatsCtx ctx, uint32_t conn)
{
  const char *func = "frStatsWormConnections";
  FrStatUpdParmUnionExp v;

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  v.v = conn;
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, v)) {
    return 0;
  }

  if (!frStatClientSubmitUpdate(err, ctx->sc, ctx->su)) {
    return 0;
  }

  return 1;
}

/**
 * Wormhole active connections, don't send it in.
 */
int frStatsWormConnectionsNoUpdate(ZkError err, FrStatsCtx ctx, uint32_t conn)
{
  const char *func = "frStatsWormConnections";
  FrStatUpdParmUnionExp v;

  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMFRSTAT, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  v.v = conn;
  if (!frStatUpdSetParameter(err, ctx->su, FRSTATUPD_CONNECTIONS, v)) {
    return 0;
  }

  return 1;
}
#endif
