/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <zkIntTypes.h>
#include <zkDump.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <zkError.h>
#include <time.h>
#include <zkLog.h>
#include <frStatUpd.h>
#include <frStatClient.h>

#define OPT_DEBUG "--debug"
#define OPT_ENTITY "--entity"
#define OPT_BADSIG "--badsig"
#define OPT_MYSIG "--mysig"

#define FL_DEBUG 0x1
#define FL_BADSIG 0x2

#define SLEEP(x) 

int
main (
  int argc ,
  char ** argv )
{
  ZkError err  = NULL;
  ZkLog log = NULL;
  FrStatUpd fsu = NULL;
  FrStatUpd fsu2 = NULL;
  FrStatClient statd = NULL;
  void * b = NULL;
  size_t bsz = 0;
  uint32_t flags = 0;
  int opt = 1;
  FrCryptSig sig = NULL;
  char sigfile[256] = "";
  char * entity = NULL;
  /*   const char * sigdir = "/home/luc/lib/keys/priv/ndb.%s.s.1024.1"; */
  const char * sigdir = "/home/luc/lib/keys/privkey/NIDB/sig/1";

  zkErrorNew(&err);
  frCryptInit(err,NULL);

  while (opt < argc) {
    if (strcasecmp(argv[opt],OPT_DEBUG) == 0) {
      flags |= FL_DEBUG;
      zkErrorLog(err,&log);
      zkLogSetStream(log,stderr);
      zkLogEnable(log);
    } else if (strcasecmp(argv[opt],OPT_BADSIG) == 0) {
      flags |= FL_BADSIG;
    } else if (strcasecmp(argv[opt],OPT_MYSIG) == 0) {
      sigdir = "/home/luc/lib/keys/priv/nym.%s.s.1024.1";
    } else if (strcasecmp(argv[opt],OPT_ENTITY) == 0) {
      if (++opt < argc) {
        entity = argv[opt];
        snprintf(sigfile,256,sigdir,entity);
        if (!frCryptSigNewFromFile(err, &sig,sigfile)) {
          fprintf(stderr,"BLahhhh..%s.\n", zkErrorDetail(err));
          exit(0);
        }
      } else {
        fprintf(stderr,"Missing argument.\n");
        exit(0);
      }
    }
    opt++;
  }




 /* 
  * initialize the connection to the stat server
  */

  if (!frStatClientNew(err,&statd)) {
    fprintf(stderr,"frStatClientNew:  failed\n");
  }

  if (!frStatClientSetHostname(err,statd,"akira")) {
    fprintf(stderr,"frStatClientSetHostname:  failed\n");
  }
  if (!frStatClientSetPort(err,statd,52000)) {
    fprintf(stderr,"frStatClientSetPort:  failed\n");
  }
  if (!frStatClientConnect(err,statd)) {
    fprintf(stderr,"frStatClientConnect:  failed\n");
  }
  if (!frStatClientSetSignatureKey(err,statd,sig)) {
    fprintf(stderr,"frStatClientSetSignatureKey: failed\n");
  }

  if (!frStatUpdNew(err,&fsu)) {
    fprintf(stderr,"frStatUpdNew: failed\n");
  }

  if (!frStatUpdSetEntity2(err, fsu, "gehenna", FRENT_TYPE_AIP)) {
    fprintf(stderr,"frStatUpdSetEntity2:  failed\n");
  }

  if (!frStatUpdSetType(err,fsu,FRSTAT_T_LINK)) {
    fprintf(stderr,"frStatUpdNew: failed\n");
  }

  if (!frStatUpdSetNbEntity2(err, fsu, "akira", FRENT_TYPE_AIP)) {
    fprintf(stderr,"frStatUpdSetEntity2:  failed\n");
  }
 
  if (!frStatUpdSetHistory(err,fsu,7)) {
    fprintf(stderr,"frStatUpdSetHistory: failed\n");
  } 

  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,10)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_DROP,12)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_CONNECTIONS,1)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  if (!(frStatUpdIncreaseParameter(err,fsu,FRSTATUPD_CONNECTIONS,1) &&
        frStatUpdIncreaseParameter(err,fsu,FRSTATUPD_CONNECTIONS,1) &&
        frStatUpdIncreaseParameter(err,fsu,FRSTATUPD_CONNECTIONS,1) &&
        frStatUpdIncreaseParameter(err,fsu,FRSTATUPD_CONNECTIONS,1) &&
        frStatUpdIncreaseParameter(err,fsu,FRSTATUPD_CONNECTIONS,1))) {
    fprintf(stderr,"frStatUpdIncreaseParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,13)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_PKTS_H_RCVD,15)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,17)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_PKTS_H_RCVD,32)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }
  SLEEP(1);
  if (!frStatUpdSetParameter(err,fsu,FRSTATUPD_BYTES_L_SENT,33)) {
    fprintf(stderr,"frStatUpdSetParameter: failed\n");
  }

  if (!frStatUpdToBuffer(err,fsu,&b,&bsz)) {
    fprintf(stderr,"frStatUpdToBuffer:  failed\n");
  }
  
  fprintf(stdout,"\n\nBuffer (sz=%d)...\n", bsz);
  zkDumpBuffer(err, b
               , bsz, zkDumpToStream, stdout);  

  if (!frStatUpdFromBuffer(err,&fsu2,b,bsz)) {
    fprintf(stderr,"frStatUpdFromBuffer:  failed\n");
  }

  fprintf(stdout,"\n\nThe original...\n");
  if (!frStatUpdDump(err,fsu,-1,stdout)) {
    fprintf(stderr,"frStatUpdDump: failed on original buffer\n");
  }
  fprintf(stdout,"\n\nThe copy...\n");
  if (!frStatUpdDump(err,fsu2,-1,stdout)) {
    fprintf(stderr,"frStatUpdDump: failed on copy of buffer\n");
  }

  fprintf(stdout,"\nSubmitting....\n");
  if (!frStatClientSubmitUpdate(err,statd,fsu)) {
    fprintf(stderr,"frStatClientSubmitUpdate:  failed\n");
  }
  
  if (!frStatUpdFree(err,&fsu)) {
    fprintf(stderr,"frStatUpdFree:  failed on original\n");
  }
  if (!frStatUpdFree(err,&fsu2)) {
    fprintf(stderr,"frStatUpdFree:  failed on copy\n");
  }
  if (!frStatClientFree(err,&statd)) {
    fprintf(stderr,"frStatClientFree:  failed\n");
  }
  exit(0);
}
