/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/* Unit test for additional functions */
/* Executable is stand-alone */

#include <zkError.h>
#include <frStatUpd.h>
#include <frStatClient.h>
#include <frstats.h>
#include "zkCStr.h"

#define SERVER_NAME  "10.16.128.20"
#define SERVER_ADDR  0x0A108014
#define SERVER_PORT  50000

struct T_FrStatUpdParmVal {
  time_t t;
  FrStatUpdParm v;
};

typedef struct T_FrStatUpdParmVal FrStatUpdParmVal;

struct T_FrStatUpd {
  uint32_t sigvers;                      /* Signature and version of this pkt */  uint32_t flags;                        /* Flags - Philippe's */
  FrEntIdExp entity;                     /* Owning entity */
  FrStatUpdTypes type;                   /* The type of update */
  FrEntIdExp nbentity;                   /* Neighbour entity, when needed */
                                         /* Only valid if type = FRSTAT_T_LINK */
  FrStatUpdReleaseString relString;      /* The reporting entities release str */
  size_t history;                        /* History size */
  size_t tail[FRSTATUPD_TOTAL_PARMS];    /* Tail of each */
  size_t set[FRSTATUPD_TOTAL_PARMS];     /* Current of each */
  FrStatUpdParmVal *pv[FRSTATUPD_TOTAL_PARMS];  /* param values */
};

struct T_FrStatsCtx {
  FrStatUpd su;                 /* Stat update object */
  FrStatClient sc;              /* Stat client object - only used on client*/
#if LB_TRASH
  /* Unused and will never likely be used */
  time_t last_stats_xmit;
  uint32_t seqno;
#endif
};

int
main (
  int argc ,
  char ** argv )
{
  ZkError            err = NULL;
  int                rc = 1;
  FrStatsCtx         ctx = NULL;
  int                sock = 0;
  struct sockaddr_in sockAddr;
  socklen_t          addrSize = sizeof(struct sockaddr_in);
  socklen_t          wrongSize = sizeof(struct sockaddr_in)/2;
  FrEntIdExp         entity;
  char              *pPkt = NULL;
  size_t             size = 0;


  /* Shut up compiler */
  (void)argc;
  (void)argv;

  zkErrorNew(&err);

  memset(&sockAddr, 0, sizeof(struct sockaddr_in));

  if ( !frStatsNew(err, &ctx) )
  {
    fprintf(stderr, "frStatsNew failed\n");
    rc = 0;
  }
  else if ( !frStatsSetServer(err, ctx, SERVER_NAME, (unsigned short)SERVER_PORT) )
  {
    fprintf(stderr, "frStatsSetServer failed\n");
    rc = 0;
  }

  /******************************/
  /* Test frStatClientGetSocket */
  /******************************/

  else if ( frStatClientGetSocket(err, NULL, &sock, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatClientGetSocket doesn't handle erroneous t param properly\n");
    rc = 0;
  }
  else if ( frStatClientGetSocket(err, ctx->sc, NULL, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatClientGetSocket doesn't handle erroneous sock param properly\n");
    rc = 0;
  }
  else if ( frStatClientGetSocket(err, ctx->sc, &sock, (struct sockaddr *)NULL, &addrSize) )
  {
    fprintf(stderr, "frStatClientGetSocket doesn't handle erroneous sockAddr param properly\n");
    rc = 0;
  }
  else if ( frStatClientGetSocket(err, ctx->sc, &sock, (struct sockaddr *)&sockAddr, NULL) )
  {
    fprintf(stderr, "frStatClientGetSocket doesn't handle erroneous addrSize param properly\n");
    rc = 0;
  }
  else if ( frStatClientGetSocket(err, ctx->sc, &sock, (struct sockaddr *)&sockAddr, &wrongSize) )
  {
    fprintf(stderr, "frStatClientGetSocket doesn't handle small addrSize param properly\n");
    rc = 0;
  }
  else if ( !frStatClientGetSocket(err, ctx->sc, &sock, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatClientGetSocket failed\n");
    rc = 0;
  }
  else if ( (sock == 0) || 
            (sockAddr.sin_addr.s_addr != htonl(SERVER_ADDR)) || 
            (sockAddr.sin_port == 0) ||
            (addrSize != sizeof(struct sockaddr_in)) )
  {
    fprintf(stderr, "frStatClientGetSocket returned wrong data: \nsockFd = %d\naddr = 0x%x\nport = %d\nsize = %d\n",
            sock, sockAddr.sin_addr.s_addr, sockAddr.sin_port, addrSize);
    rc = 0;
  }

  /*************************/
  /* Test frStatsGetServer */
  /*************************/

  sock = 0;
  memset(&sockAddr, 0, sizeof(struct sockaddr_in));
  addrSize = sizeof(struct sockaddr_in);
  wrongSize = sizeof(struct sockaddr_in)/2;

  if ( frStatsGetServer(err, NULL, &sock, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatsGetServer doesn't handle erroneous ctx param properly\n");
    rc = 0;
  }
  else if ( frStatsGetServer(err, ctx, NULL, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatsGetServer doesn't handle erroneous sock param properly\n");
    rc = 0;
  }
  else if ( frStatsGetServer(err, ctx, &sock, (struct sockaddr *)NULL, &addrSize) )
  {
    fprintf(stderr, "frStatsGetServer doesn't handle erroneous sockAddr param properly\n");
    rc = 0;
  }
  else if ( frStatsGetServer(err, ctx, &sock, (struct sockaddr *)&sockAddr, NULL) )
  {
    fprintf(stderr, "frStatsGetServer doesn't handle erroneous addrSize param properly\n");
    rc = 0;
  }
  else if ( frStatsGetServer(err, ctx, &sock, (struct sockaddr *)&sockAddr, &wrongSize) )
  {
    fprintf(stderr, "frStatsGetServer doesn't handle small addrSize param properly\n");
    rc = 0;
  }
  else if ( !frStatsGetServer(err, ctx, &sock, (struct sockaddr *)&sockAddr, &addrSize) )
  {
    fprintf(stderr, "frStatsGetServer failed\n");
    rc = 0;
  } 
  else if ( (sock == 0) || 
            (sockAddr.sin_addr.s_addr != htonl(SERVER_ADDR)) || 
            (sockAddr.sin_port == 0) ||
            (addrSize != sizeof(struct sockaddr_in)) )
  {
    fprintf(stderr, "frStatsGetServer returned wrong data: \nsockFd = %d\naddr = 0x%x\nport = %d\nsize = %d\n",
            sock, sockAddr.sin_addr.s_addr, sockAddr.sin_port, addrSize);
    rc = 0;
  }

  /************************/
  /* Test frStatsBuildMsg */
  /************************/

  entity.type = FRENT_TYPE_NISS;
  zkCStrlcpy(entity.name, "niss", sizeof entity.name);

  if ( frStatsBuildMsg(err, NULL, (void **)&pPkt, &size) )
  {
    fprintf(stderr, "frStatUpdSetEntity doesn't handle erroneous ctx param properly\n");
    rc = 0;
  }
  else if ( frStatsBuildMsg(err, ctx, (void **)NULL, &size) )
  {
    fprintf(stderr, "frStatUpdSetEntity doesn't handle erroneous pkt param properly\n");
    rc = 0;
  }
  else if ( frStatsBuildMsg(err, ctx, (void **)&pPkt, NULL) )
  {
    fprintf(stderr, "frStatUpdSetEntity doesn't handle erroneous size param properly\n");
    rc = 0;
  }
  else if ( !frStatUpdSetEntity(err, ctx->su, &entity) )
  {
    fprintf(stderr, "frStatUpdSetEntity failed\n");
    rc = 0;
  }
  else if ( !frStatsBuildMsg(err, ctx, (void **)&pPkt, &size) )
  {
    fprintf(stderr, "frStatsBuildMsg failed\n");
    rc = 0;
  }
  else if ( (pPkt == NULL) || (size == 0) )
  {
    fprintf(stderr, "frStatsBuildMsg produced bad output\n");
    rc = 0;
  }
  else
  {
    size_t i;

    fprintf(stderr, "\nfrStatsBuildMsg built this message:\n");
    for (i = 0; i < size; i++)
    {
      if ( !(i % 16) )
      {
        fprintf(stderr, "\n");
      }

      fprintf(stderr, "%2x ", (unsigned char)pPkt[i]);
    }

    fprintf(stderr, "\n");
  }

  /* Clean up */
  if ( pPkt != NULL )
  {
    free(pPkt);
  }

  if ( err != NULL)
  {
    zkErrorFree(&err);
  }

  if ( ctx != NULL )
  {
    if ( !frStatsFree(err, &ctx) )
    {
      fprintf(stderr, "frStatsFree failed\n");
      rc = 0;
    }
  }

  if (rc)
  {
    fprintf(stderr, "\nUnit Test 2 passed\n");
  }

  return 0;
}

