/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkConfig.h>
#include <frKeyQry.h>
#include <zkDNS.h>
#include <frStatUpd.h>
#include <frStatBucket.h>

/*
 * WARNING: This test program does not work properly.
 * It's not worth my time to fix it now.
 * Problems: key signature handling 
 * Improper handling if header packet
 * Luc - Oct 3, 1999
 */
#warning Test program does not work properly

/*
 * Parameters
 */

#define P_DEBUG       "Debug"
#define P_CONFIG      "Config"
#define P_LOGFILE     "LogFile"
#define P_PORT        "Port"
#define P_KQHOST      "KQHost"
#define P_KQPORT      "KQPort"
#define P_KQTIMEOUT   "KQTimeOut"
#define P_KQRETRIES   "KQRetries"
#define P_CHECKKEYS   "CheckKeys"
#define P_OUTFILE     "Output"

#define SECTION "ZKSTATD"

/*
 * Flags 
 */

#define FL_DEBUG            0x1       /* Debug mode enabled */
#define FL_CHECKKEYS        0x2       /* Check keys on incoming buffers */


struct T_ZkStatContext {
  uint32_t flags;
  ZkError err;
  ZkLog log;
  ZkConfig config;
  const char * progname;
  const char * config_file;
  const char * log_file;
  const char * out_file;
  int exit_code;
  char * kqhost;
  unsigned short kqport;
  unsigned short kqretries;
  unsigned short kqtimeout;
  unsigned short port;
  FrKeyQry kq;
  FrKeyQryHost kh;
  FILE * std_err;
  FILE * std_out;
  FILE * std_in;
  struct sockaddr_in server;          /* Server address struct */
  int sock_fd;                        /* Server socket descriptor */
};

#define DEFAULT_CONFIG_FILE            NULL
#define DEFAULT_LOG_FILE               NULL
#define DEFAULT_PORT                   52000
#define DEFAULT_STD_IN                 stdin
#define DEFAULT_STD_OUT                stdout
#define DEFAULT_STD_ERR                stderr
#define DEFAULT_OUT_FILE               NULL

#define DEFAULT_KQPORT                 60002
#define DEFAULT_KQRETRIES              3
#define DEFAULT_KQTIMEOUT              30
#define DEFAULT_KQHOST                 NULL

typedef struct T_ZkStatContext ZkStatContext;

static int zkStatContextInit ( ZkError , ZkStatContext * , int , char ** );
static int zkStatContextLoad ( ZkError , ZkStatContext * );
static int zkStatContextVerify ( ZkError , ZkStatContext * );
static int zkStatBucketDump ( ZkError , ZkStatContext * , FrStatBucket );

static int
zkStatContextInit (
  ZkError err ,
  ZkStatContext * c ,
  int argc ,
  char ** argv)
{
  static const char * func = "zkStatContextInit";
  char cf_opt[32];
  int return_code = 0;
  int opt = 1;

  zkErrorClear(err);
  if (c && argv) {
    memset(c,0,sizeof(struct T_ZkStatContext));
    if (!(zkErrorNew(&(c->err)) &&
          zkLogNew(&(c->log)) &&
          zkConfigNew(err,&(c->config)))) {
      zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKERROR,
                 "Failed to create internal context data structures");
    } else {
      if (zkConfigSetCmdParms(err,c->config,argc,(const char **) argv)) {

        /* Initialize default context parameters */ 
        if ((c->progname = strrchr(argv[0],'/')) != NULL) {
          c->progname++;
        } else {
          c->progname = argv[0];
        }
        c->flags = FRSTAT_DEFAULT_FLAGS;
        c->config_file = DEFAULT_CONFIG_FILE;
        c->log_file = DEFAULT_LOG_FILE;
        c->port = DEFAULT_PORT;
        c->std_err = DEFAULT_STD_ERR;
        c->std_out = DEFAULT_STD_OUT;
        c->std_in = DEFAULT_STD_IN;
        c->exit_code = EXIT_FAILURE;
        c->kqhost = DEFAULT_KQHOST;
        c->kqretries = DEFAULT_KQRETRIES;
        c->kqtimeout = DEFAULT_KQTIMEOUT;
        c->kqport = DEFAULT_KQPORT;
        c->kq = NULL;
        c->out_file = DEFAULT_OUT_FILE;
        memset(&(c->kh),0,sizeof(FrKeyQryHost));
 
        /* See if we have a log file to load... */
        snprintf(cf_opt,32,"--%s",P_CONFIG);
        while (opt < argc) {
          if (strcasecmp(argv[opt],cf_opt) == 0) {
            if (++opt < argc) {
              c->config_file = argv[opt];
            } else {
              fprintf(c->std_err,
                      "%s:  You must specify a configuration file.\n",
                      c->progname);
            }
          }
          opt++;
        }
  
        /* If it is specified, load the config file */
        if (c->config_file) {
          if (zkConfigLoad(err,c->config,c->config_file)) {
            return_code = 1;
          }
        } else {
          return_code = 1;
        }
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}

static int
zkStatContextLoad (
  ZkError err ,
  ZkStatContext * c )
{
  static const char * func = "zkStatContextLoad";
  int return_code = 0;
  uint8_t bool = 0; 
  uint16_t u16 = 0;
  char * str = NULL;
  const char * sec = SECTION;

  zkErrorClear(err);
  if (c) {
  
    /* Run in debug mode? */

    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,sec,P_DEBUG,&bool)) {
      if (bool) {
        c->flags |= FL_DEBUG;
      }
    }
  
    /* Check keys on the incoming stat update buffers? */
    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,sec,
                             P_CHECKKEYS,&bool)) {
      if (bool) {
        c->flags |= FL_CHECKKEYS;
      }
    }
   
    /* Load the logfile name */
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,sec,P_LOGFILE,&str)) {
      c->log_file = str;
    }

    /* Optional output file */
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,sec,P_OUTFILE,&str)) {
      c->out_file = str;
    }
  
    /* Get the port number */
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,sec,P_PORT,&u16)) {
      c->port = u16;
    }

    /*
     * Get the key query server parameters port,host,timeout and retries
     */
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,sec,P_KQPORT,&u16)) {
      c->kqport = u16;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,sec,P_KQHOST,&str)) {
      c->kqhost = str;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,sec,P_KQRETRIES,&u16)) {
      c->kqretries = u16;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,sec,P_KQTIMEOUT,&u16)) {
      c->kqtimeout = u16;
    }

    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
  }
  return return_code;
}


static int
zkStatContextVerify (
  ZkError err ,
  ZkStatContext * c )
{
  static const char * func = "zkStatContextVerify";
  int return_code = 1;

  zkErrorClear(err);
  if (c) {
 
    /* Enable the debug mode if required */
    if (c->flags & FL_DEBUG) {
      ZkLog log = NULL;
      zkErrorLog(c->err,&log);
      zkLogSetProgname(log,c->progname);
      zkLogSetStream(log,c->std_err);
      zkLogEnable(log);
    }

    /* Startup the logfile */
    if (c->log_file) {
      if (!(zkLogSetFile(c->log,c->log_file) &&
            zkLogSetProgname(c->log,c->progname) &&
            zkLogEnable(c->log))) {
        return_code = 0;
        fprintf(c->std_err,"%s:  Unable to access log file \"%s\".\n",
                c->progname,c->log_file);
      }
    }

    if (c->port != 0) { 
      c->server.sin_addr.s_addr = htonl(INADDR_ANY);
      c->server.sin_port = htons(c->port);
      c->server.sin_family = AF_INET;
    } else {  
      fprintf(c->std_err,"%s:  You must specify a server port.\n",c->progname);
      return_code = 0;
    }

    if (c->out_file) {
      FILE * f = NULL;
      
      if ((f = fopen(c->out_file,"w")) == NULL) {
        fprintf(c->std_err,"%s:  Unable to access \"%s\".\n",c->progname,
                c->out_file);
        zkErrorSet_va(err,func,ZKERR,ZKMFRSTAT,ZKEFILEOPEN,
                      "Unable to access \"%s\":  %s",c->out_file,
                      strerror(errno));
        return_code = 0;
      } else {
        c->std_out = f;
      }
    }

    /*
     * Set up key query parameters 
     */
   
    if (c->flags & FL_CHECKKEYS) {
      if (c->kqhost == NULL) {
        return_code = 0;
        fprintf(c->std_err,"%s:  You must specify a key query host.\n",
                c->progname);
      } else if (c->kqretries == 0) {
        return_code = 0;
        fprintf(c->std_err,"%s:  You must specify a key query retry count.\n",
                c->progname);
      } else if (c->kqtimeout == 0) {
        return_code = 0;
        fprintf(c->std_err,"%s:  You must specify a key query timeout.\n",
                c->progname);
      } else if (c->kqport == 0) { 
        return_code = 0;
        fprintf(c->std_err,"%s:  You must specify a key query port number.\n",
                c->progname);
      } else if (!frKeyQryNew(err,&(c->kq))) {
        fprintf(c->std_err,"%s:  Failed to create key cache.\n",c->progname);
        return_code = 0; 
      } else {
        struct hostent * phe = NULL;
        c->kh.fd = -1;
        c->kh.host.sin_family = AF_INET;
        c->kh.host.sin_port = htons(c->kqport);
        if (!zkGetHostByName(err,c->kqhost,&phe)) {
          fprintf(c->std_err,"%s:  failed to lookup %s.\n",c->progname,
                  c->kqhost);
          return_code = 0;
        } else {
          memcpy(&(c->kh.host.sin_addr),phe->h_addr,phe->h_length);
        }
      }
    }    

  } else {
    zkErrorSet(err,func,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    return_code = 0;
  }
  return return_code;
}


static int
zkStatBucketDump (
  ZkError err ,
  ZkStatContext * c ,
  FrStatBucket bucket )
{
#define TYPE_STR_BUF_SZ 10
  static const char * func = "zkStatBucketDump";
  FrStatUpd fsu = NULL;
  int return_code = 0;
  FrEntIdExp * entity = NULL;
  FrStatUpdParameter p = 0;
  FrStatUpdParm v;
  time_t t = 0;
  uint32_t updates = 0;
  const char * label = NULL;
  const char * s = NULL;

  (void) func;

  zkErrorClear(err);
  if (c && bucket) {
    rewind(c->std_out);
    return_code = 1;
    fprintf(c->std_out,"         Entity/Type Update ");
    for (p = 0; p < FRSTATUPD_TOTAL_PARMS; p++) {
      if (!(label = frStatUdpGetParameterString (err,p)))
        fprintf(c->std_out,"%10s",label);
    }
  } 
  fprintf(c->std_out,"\n");
  frStatBucketGetReset(err,bucket);
  while (frStatBucketGetStatUpd(err,bucket,&fsu,&updates)) {
    if (!frStatUpdEntity(err,fsu,(const FrEntIdExp **) &entity)) {
      return_code = 0;
      break;
    }
    frEntPrint(err, c->std_out, entity);
    fprintf(c->std_out,"%6d ",updates);
    for (p = 0; p < FRSTATUPD_TOTAL_PARMS; p++) {
      if (frStatUpdGetParameter(err,fsu,p,0,&v,&t)) {
        if (!(s = frStatUdpGetParameterString (err,p)))
          fprintf(c->std_out,"%s: %10d",s, v);
        break;
      } else { 
        fprintf(c->std_out,"%10s","---");
      }
    }
    fprintf(c->std_out,"\n");
  }
  fflush(c->std_out);
  return return_code;
}

int
main (
  int argc ,
  char ** argv )
{
#define BUFFER_SZ   4096
  ZkStatContext context;
  ZkError err = NULL;
  struct sockaddr_in remote;
  int remote_sz = 0;
  char buf[BUFFER_SZ];
  int buf_sz = 0; 
  FrStatBucket bucket = NULL;

  zkErrorNew(&err);
  frCryptInit(err,NULL);
  frStatBucketNew(err,&bucket);
  frStatBucketSetHistory(err,bucket,20);


  if (!(zkStatContextInit(err,&context,argc,argv) &&
        zkStatContextLoad(context.err,&context) &&
        zkStatContextVerify(context.err,&context))) {
    fprintf(context.std_err,"%s:  Unable to initialize context.\n",
            context.progname);
    /*
     * If you dont want the FrStatBucket to check the signature on the
     * update buffer, do not call frStatBucketSetKeyQueryParameters().
     * In that case, the StatBucket will just check that the signature
     * headers are ok and that the entity names in the buffer match.
     * You can always check the signature yourself if you want.  Look at
     * the frStatBucketCheckUpdateBufferSignature() function to see how.
     */

  } else if ((context.flags & FL_CHECKKEYS) /*  && 
             !frStatBucketSetKeyQueryParameters(context.err,bucket,context.kq,
                                                context.kh,context.kqtimeout,
                                                context.kqretries)*/ ) {
    zkLogError(context.log,context.err);
    fprintf(context.std_err,
            "%s:  Unable to set key query paremeters in FrStatBucket.\n",
            context.progname);
  } else {
    /* First, initialize the network connection  */
    if ((context.sock_fd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
      zkErrorSet_va(context.err,"socket",ZKERR,ZKMFRSTAT,ZKESOCKET,
                    "Unable to create DGRAM socket:  %s",strerror(errno));
      zkLogError(context.log,context.err);
      fprintf(context.std_err,"%s:  Unable to create server socket:  %s.\n",
              context.progname,strerror(errno));
    } else if (bind(context.sock_fd,(struct sockaddr *)&(context.server),
                    sizeof(struct sockaddr_in)) < 0) { 
      zkErrorSet_va(context.err,"bind",ZKERR,ZKMFRSTAT,ZKESOCKET,
                    "Unable to bind to server address structure:  %s",
                    strerror(errno));
      zkLogError(context.log,context.err);
      fprintf(context.std_err,"%s:  Unable to bind to port %d:  %s.\n",
              context.progname,context.port,strerror(errno));
    } else {
      zkLogWrite_va(context.log,"Server listening on port %d",context.port);
      for (;;) {
        remote_sz = sizeof(struct sockaddr_in);
        buf_sz = recvfrom(context.sock_fd,buf,BUFFER_SZ,0,
                          (struct sockaddr *)&remote,&remote_sz);
        if (buf_sz > 0) {
          void *buf2 = NULL;
          size_t bsz2 = 0;
          uint32_t vers = 0;
          FrPktType type = 0;
          uint32_t ttl = 0;
          int id = 0;

          if ( frStatPktGetUpdInfo(err, buf, buf_sz, NULL, &vers,
                                   &type, &ttl, &id,
                                   &buf2, &bsz2) ) {
            if (frStatBucketAddUpdateBuffer(context.err,bucket,buf2,
                                            (size_t *)&bsz2)) {
              zkStatBucketDump(err,&context,bucket);
            } else { 
              zkLogError(context.log,context.err);
            }
          } else { 
            zkLogWrite_va(context.log, "Problem parsing Update packet: %s",
                        zkErrorDetail(err));
          }
        } else {
          zkLogWrite_va(context.log,"Error reading UDP packet: %s",
                        strerror(errno));
        }
      }
    }
  }
 

  if (context.exit_code == EXIT_FAILURE) {
    fprintf(context.std_err,"%s:  Failure.\n",context.progname);
  }
  exit(context.exit_code);
}

