/*
 * $Id: frTokenClient.h,v 1.8 2001/04/19 20:31:27 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */
#ifndef FRTOKENCLIENT_H_
#define FRTOKENCLIENT_H_

#ifndef _ZKIP_H
#include "ZKIP.h"
#endif /* _ZKIP_H */

#ifndef ZKIPCLIENT_H_
#include "zkipClient.h"
#endif /* ZKIPCLIENT_H_ */

#ifndef FRSERIAL_H_
#include "frSerial.h"
#endif /* FRSERIAL_H_ */

#ifndef FRNYM_H_
#include "frNym.h"
#endif /* FRNYM_H_ */

#ifndef FRNYMCLIENT_H_
#include "frNymClient.h"
#endif /* FRNYMCLIENT_H_ */

#include "frKeyQry.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Exactly the same idea as the NymServer structure
 */

#define TSHOST_SZ 64
struct T_TokenServer {
  char tokenserver[TSHOST_SZ + 1];   /* Name of the token server */
  unsigned short port;               /* Port number of the token server */
  int timeout;                       /* Timeout for connecting to server */
  FrCryptSig sigkey;                 /* Token server's public key */
};

typedef struct T_TokenServer TokenServer;

/*
 * Response codes are described in ZKIP.h.  The message string is for
 * human consumption only.
 */

#define TSRMESG_SZ 256
struct T_TokenServerResponse {
  char message[TSRMESG_SZ+ 1];   /* Message string returned by server */
  int code;                   /* 3 digit code returned by token server */
};

typedef struct T_TokenServerResponse TokenServerResponse;

/* Obsolete- please use frSerialNumberRedeemConnected() corey */
DLLExport int frSerialNumberRedeem (
  ZkError ,                    /* Return the error if any occured */
  FrSerialNumber ,             /* The serial number structure to redeem */
  TokenServer * ,              /* Token server you wish to use */
  TokenServerResponse * ,      /* Server response structure */
  char *** ,                   /* Returns an array of tokens */
  size_t * );                  /* Number of tokens in the array */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frSerialNumberRedeemConnected (
  ZkError ,                    /* Return the error if any occured */
  FrSerialNumber ,             /* The serial number structure to redeem */
  ZkipClient ,                 /* A ZkipClient that is already connected to a token server */
  FrCryptSig , 			       /* The sig key of the token server that the ZkipClient is connected to*/
  TokenServerResponse * ,      /* Server response structure */
  char *** ,                   /* Returns an array of tokens */
  size_t * );                  /* Number of tokens in the array */

/*
 * NOTE NOTE NOTE
 * Although this function returns a token type description (which is why it
 * is in this library) it connects to the nym server which maintains this
 * information, so you must specify a  NymServer structure instead of a
 * TokenServer structure.
 */

/* Obsolete- this function is only called by a test program 
 * (freedom\libs\frtoken\test\showtok.c)
 */
DLLExport int frTokenLoadTypeDescriptions (
  ZkError ,                  /* Return the error if one occured */
  FrTokenType * ,            /* Returns a token type description structure */
  NymServer * ,              /* target NYM server */
  NymServerResponse * );     /* NYM server response */


/*
 *
 */

/* Obsolete- please use frSerialNumberGetCapabilityConnected() corey */
DLLExport int frSerialNumberGetCapability (
  ZK_INOUT ZkError err ,                    /* Return the error if required */
  ZK_IN    FrKeyQryHost kh ,                /* Key query host */
  ZK_INOUT FrKeyQry kq ,                    /* Key query cache */
  ZK_IN    FrSerialNumber sn ,              /* Serial number structure */
  ZK_IN    TokenServer * tokenserver ,      /* Token server to use */
  ZK_OUT   TokenServerResponse * tsresp ,   /* Returns the response */
  ZK_OUT   char * cap ,                     /* array to hold capability code
                                             * (you need to allocate this 
                                             *  yourself)
                                             */
  ZK_INOUT size_t * cap_sz );               /* Set to the size of your array,
                                             * the function resets it to the 
                                             * size of the capability code
                                             * string.
                                             */

/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frSerialNumberGetCapabilityConnected (
  ZK_INOUT ZkError err ,                    /* Return the error if required */
  ZK_IN    FrKeyQryHost kh ,                /* Key query host */
  ZK_INOUT FrKeyQry kq ,                    /* Key query cache */
  ZK_IN    FrSerialNumber sn ,              /* Serial number structure */
  ZK_INOUT ZkipClient client,               /* A ZkipClient that is already 
                                             * connected to a token server 
											 */
  ZK_OUT   TokenServerResponse * tsresp ,   /* Returns the response */
  ZK_OUT   char * cap ,                     /* array to hold capability code
                                             * (you need to allocate this 
                                             *  yourself)
                                             */
  ZK_INOUT size_t * cap_sz );               /* Set to the size of your array,
                                             * the function resets it to the 
                                             * size of the capability code
                                             * string.
                                             */

/*
 *
 */

/* Obsolete- please use frSerialNumberCapabilityConnected() corey */
DLLExport int frSerialNumberCapability (
  ZK_INOUT ZkError err ,                   /* Returns the error if required */
  ZK_IN    FrSerialNumber sn ,             /* Serial number structure */
  ZK_IN    TokenServer * tokenserver ,     /* Token server to use */
  ZK_OUT   TokenServerResponse * tsresp ,  /* Token server response */
  ZK_OUT   char ** cap ,                   /* Returns signed capability code */
  ZK_INOUT size_t * cap_sz );              /* Returns size of signed capability
                                            * code 
                                            */
/* The ZkipClient must be disconnected and free'd be the caller */
DLLExport int frSerialNumberCapabilityConnected (
  ZK_INOUT ZkError err ,                   /* Returns the error if required */
  ZK_IN    FrSerialNumber sn ,             /* Serial number structure */
  ZK_INOUT ZkipClient client,               /* A ZkipClient that is already 
                                             * connected to a token server 
											 */
  ZK_OUT   TokenServerResponse * tsresp ,  /* Token server response */
  ZK_OUT   char ** cap ,                   /* Returns signed capability code */
  ZK_INOUT size_t * cap_sz );              /* Returns size of signed capability
                                            * code 
                                            */



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRTOKENCLIENT_H_ */
