/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "frSerial.h"
#include "frCrypt.h"
#include "zkCStr.h"

int 
main (
  int argc ,
  char ** argv )
{
  const char * digit = "AB123456789CDEFGHJKLMNPQRSTUVWXYZ";
  const char * progname = NULL;
  char sn[32];
  int opt = 1;
  int i  = 0;
  int fail = 0;
  ZkError err = NULL;
  ZkLog log = NULL;
  FrSerialNumber SN = NULL;
  int rotate = 0;
  int debug = 0;

  if ((progname = strrchr(argv[0],'/')) == NULL) {
    progname = argv[0];
  } else {
    progname++;
  }

  zkErrorNew(&err);
  frCryptInit(err,0);

  while (opt < argc) {
    if (strcmp(argv[opt],"--sn") == 0) {
      if (++opt < argc) {
        zkCStrlcpy(sn,argv[opt], sizeof sn);
      } else {
        zkFprintf(stderr,"%s:  You must provide a serial number\n",progname);
        fail = 1;
      } 
    } else if (strcmp(argv[opt],"--debug") == 0) {
      debug = 1;
    } else if (strcmp(argv[opt],"--rotate") == 0) {
      rotate = 1;
    } else {
      zkFprintf(stderr,"%s:  Unrecognized option %s\n",progname,argv[opt]);
      fail = 1;
    }
    opt++;
  }

  if (debug) {
    zkErrorLog(err,&log);
    zkLogSetStream(log,stderr);
    zkLogSetProgname(log,progname);
    zkLogEnable(log);
  }

  frSerialNumberNew(err,&SN);

  if (sn == NULL) {
    zkFprintf(stderr,"%s:  You must specify a serial number using --sn\n",progname);
    fail = 1;
  }

  if (fail) {
    zkFprintf(stderr,"%s:  Failed\n\n",progname);
    exit(EXIT_FAILURE);
  }

  if (rotate) {
    for (i = 0; i <= 0x03ff; i++) {
      sn[12] = digit[i / 32];
      sn[13] = digit[i % 32];
      sn[14] = (char) 0;
      if (frSerialNumberParse(err,SN,sn,strlen(sn))) {
        zkFprintf(stderr,"%s:  Matching serial number \"%s\"\n",progname,sn); 
      }
    }
  } else {
    if (!frSerialNumberParse(err,SN,sn,strlen(sn))) {
      exit(EXIT_FAILURE);
    }
    zkFprintf(stderr,"%s:  Serial number checksum is correct\n",progname);
  }
  return(EXIT_SUCCESS);
}

  
