/*
 * $Id: parse_test.c,v 1.3 2001/04/19 20:31:30 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkTime.h"
#include "zkError.h"
#include "zkLog.h"
#include "frToken.h"
#include "frCapability.h"
#include "zkShell.h"
#include "zkString.h"

int
main (
  int argc ,
  char ** argv )
{
  size_t size = 0;
  size_t token_count = 0;
  unsigned short token_type = 0;
  unsigned short sn_type = 0;
  time_t activate = 0;
  ZkError err = NULL;
  ZkLog log = NULL;

  zkShellGetProgramName(&argc, &argv);

  if (argc != 2) {
    zkFprintf(stderr,"Usage:  %s capability_code\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  size = (size_t) strlen(argv[1]);

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if (frCapabilityCodeParse(err,argv[1],size,
                            &token_count,&token_type,&sn_type,&activate)) {
    zkFprintf(stdout,"[A] tokens %d token_type %d sn_type %d activate %d %s",
            token_count,token_type,sn_type,(int) activate,ctime(&activate));
  } 
  token_count = 0;
  if (frCapabilityCodeParse(err,argv[1],size,
                            NULL,&token_type,&sn_type,&activate)) {
    zkFprintf(stdout,"[B] tokens %d token_type %d sn_type %d activate %d %s",
            token_count,token_type,sn_type,(int) activate,ctime(&activate));
  } 
  token_type = 0;
  if (frCapabilityCodeParse(err,argv[1],size,
                            &token_count,NULL,&sn_type,&activate)) {
    zkFprintf(stdout,"[C] tokens %d token_type %d sn_type %d activate %d %s",
            token_count,token_type,sn_type,(int) activate,ctime(&activate));
  } 
  sn_type = 0;
  if (frCapabilityCodeParse(err,argv[1],size,
                            &token_count,&token_type,NULL,&activate)) {
    zkFprintf(stdout,"[D] tokens %d token_type %d sn_type %d activate %d %s",
            token_count,token_type,sn_type,(int) activate,ctime(&activate));
  } 
  activate = zkClockToTime(zkClockFromSeconds(0));
  if (frCapabilityCodeParse(err,argv[1],size,
                            &token_count,&token_type,&sn_type,NULL)) {
    zkFprintf(stdout,"[E] tokens %d token_type %d sn_type %d activate %d %s",
            token_count,token_type,sn_type,(int) activate,ctime(&activate));
  } 

  return(EXIT_SUCCESS);
} 
 
  


