/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkDump.h"
#include "frToken.h"
#include "zkShell.h"
int 
main (
  int argc ,
  char ** argv )
{
#define FILE_SZ 256
#define BINARY_SZ 512
  char * sigfile = NULL;
  char * progname = NULL;
  int fail = 0;
  FrCryptSig sigpub = NULL;
  FrCryptSig sigprv = NULL;
  ZkError err = NULL;
  ZkLog log = NULL;
  int opt = 1;
  char sigfilepub[FILE_SZ];
  char sigfileprv[FILE_SZ];
  unsigned char binary[BINARY_SZ];
  FrToken token = NULL;
  FrToken token2 = NULL;
  size_t binsz;
  FrEntIdExp entity;

  zkShellGetProgramName(&argc, &argv);

  if ((progname = strrchr(argv[0],'/')) != NULL) {
    progname++;
  } else {
    progname = argv[0];
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetProgname(log,progname);
  if (!frCryptInit(err,0)) {
    zkFprintf(stderr,"%s:  Could not initialize frcrypt library.\n",progname);
    return(EXIT_FAILURE);
  }

  while (opt < argc) {
    if (strcmp(argv[opt],"--sigfile") == 0) {
      if (++opt < argc) {
        sigfile = argv[opt];
      } else {
        zkFprintf(stderr,"%s:  Missing signature file\n",progname);
        fail = 1;
      }
    } else if (strcmp(argv[opt],"--debug") == 0) {
      zkLogSetStream(log,stderr);
      zkLogEnable(log);
    } else {
      zkFprintf(stderr,"%s:  Unrecognized option %s\n",progname,argv[opt]);
      fail = 1;
    } 
    opt++;
  }

  if (sigfile == NULL) {
    zkFprintf(stderr,"%s:  You must specify a signature with --sigfile\n",progname);
    fail = 1;
  }

  if (fail) {
    zkFprintf(stderr,"%s:  Program terminating\n",progname);
    return(EXIT_FAILURE);
  }

  zkSnprintf(sigfilepub,sizeof sigfilepub,"%s.sig.pub",sigfile);
  zkSnprintf(sigfileprv,sizeof sigfileprv,"%s.sig",sigfile);
  if (!frCryptSigNewFromFile(err,&sigpub,sigfilepub)) {
    return(EXIT_FAILURE);
  }
  if (!frCryptSigNewFromFile(err,&sigprv,sigfileprv)) {
    return(EXIT_FAILURE);
  }

  if (!frTokenNew(err,&token)) {
    return(EXIT_FAILURE);
  }
  entity.type = FRENT_TYPE_TOK_SRV;
  strcpy(entity.name, "SPECIAL TOKEN SERVER");
  if (!frTokenSet(err,token,100,&entity)) {
    return(EXIT_FAILURE);
  }
  binsz = BINARY_SZ;
  if (!frTokenStoreToBinary(err,token,binary,&binsz,sigprv)) {
    return(EXIT_FAILURE);
  } 

  zkDumpBuffer(err,binary,binsz,zkDumpToStream,stdout);
  binary[0]=12;

  if (!frTokenNewFromBinary(err,&token2,binary,binsz,sigpub)) {
    return(EXIT_FAILURE);
  }
  {
    unsigned short typeid;
    if (!frTokenTypeId(err,token2,&typeid)) {
      exit(EXIT_FAILURE);
    }
    zkPrintf("TYPE ID is %d \n",typeid);
  } 


  zkFprintf(stderr,"%s:  Success!!!\n",progname);
  return(EXIT_SUCCESS);
}
