/*
 * $Id: toktype.c,v 1.3 2001/04/19 20:31:30 alainc Exp $
 */

static const char * module_id_ = "$Id: toktype.c,v 1.3 2001/04/19 20:31:30 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "frSerial.h"
#include "frToken.h"
#include "frTokenType.h"
#include "zkShell.h"

int 
main ( 
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  FrTokenType tt = NULL;
  FrTokenType tt2 = NULL;
  char * buffer = NULL;
  size_t size = 0;
  const FrTokenTypeDescription * descr = NULL; 

  (void) module_id_;

  zkShellGetProgramName(&argc, &argv);

  if (argc != 3) {
    zkFprintf(stderr,"Usage:  %s typeidfile typeid\n\n",argv[0]);
    return(EXIT_FAILURE);
  }


  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if (!frTokenTypeNewFromFile(err,&tt,argv[1])) {
    zkFprintf(stderr,"frTokenTypeNewFromFile: failed\n");
  }

  if (!frTokenTypeToBinary(err,tt,(void **) &buffer,&size)) {
    zkFprintf(stderr,"frTokenTypeToBinary: failed\n");
  }

  if (!frTokenTypeFree(err,&tt)) {
    zkFprintf(stderr,"frTokenTypeFree:  failed\n");
  }

  if (!frTokenTypeFromBinary(err,&tt2,buffer,size)) {
    zkFprintf(stderr,"frTokenTypeFromBinary:  failed\n");
  }
 
  if (!frTokenTypeLookup(err,tt2,(unsigned short) atoi(argv[2]),&descr)) {
    zkFprintf(stderr,"frTokenTypeLookup:  failed\n");
  } else {
    zkPrintf("Type ID: %d\n"
           "Duration: %d days\n"
           "Reserved: %d days\n"
           "Email Domain: %s\n"
           "Access Level: %d\n",
           descr->type_ID,
           descr->duration,
           descr->reserved,
           descr->email_domain,
           descr->access_level);
    zkPrintf("Maximum life:  %d\n",descr->maxlife);
    zkPrintf("Flags:  %s %s %s\n",
         (descr->flags & FL_TOKEN_CREATE)?FL_TOKEN_CREATE_STR:"",
         (descr->flags & FL_TOKEN_UPGRADE)?FL_TOKEN_UPGRADE_STR:"",
         (descr->flags & FL_TOKEN_DURATION_OVERIDE)?FL_TOKEN_DURATION_OVERIDE_STR:"");
  }

  zkFree(buffer);
  frTokenTypeFree(err,&tt2);

  return(0);
}  
  
