/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef __TS_FILESYS_H__
#define __TS_FILESYS_H__

/**
 * Routines to implement a shared-memory mini-filesystem (to store queued
 * packets of different sizes)
 * @author Ian Goldberg
 */

typedef unsigned int TS_fs_index;

int TS_fs_new(unsigned char *fs, unsigned int fragsize,
    unsigned int numfrags);

TS_fs_index TS_fs_insert(unsigned char *fs, unsigned char *buf,
    unsigned int len);

TS_fs_index TS_fs_insert_read(unsigned char *fs, int fd,
    unsigned int len);

int TS_fs_delete(unsigned char *fs, TS_fs_index ind);

/* Set *bufp to a pointer to a buffer containing the data indexed by ind.
   If this returns negative, there was an error.  If 0, all is OK, and
   *bufp points directly into the fs memory.  If 1, *bufp points to a
   newly-allocated buffer that must be free'd with TS_fs_freebuf().
   *lenp is set to the length of the data. */
int TS_fs_getbuf(unsigned char *fs, TS_fs_index ind, unsigned char **bufp,
    unsigned int *lenp);

/**
 * Free a buffer allocated by TS_fs_getbuf(), when that routine returned 1.
 * @param bufp     A <EM>pointer</EM> to the buffer to free.  *bufp will be
 *                 set to NULL.
 * @param lenp     A <EM>pointer</EN> to the length of the buffer.  *lenp
 *                 will be set to 0.
 */
void TS_fs_freebuf(unsigned char **bufp, unsigned int *lenp);

void TS_fs_destroy(unsigned char *fs);

#endif /* ! __TS_FILESYS_H__ */
