/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <stdlib.h>
#include "filesys.h"

int main(int argc, char **argv)
{
    int totsize = argc > 1 ? atoi(argv[1]) : 100;
    int fragsize = 11;
    int ret;
    char cmd[1024];
    unsigned char *fs = malloc(totsize);
    if (!fs) {
	fprintf(stderr, "Out of mem!\n");
	exit(1);
    }
    
    ret = TS_fs_new(fs, fragsize, totsize / (fragsize+8));
    if (ret < 0) {
	fprintf(stderr, "Unable to create fs\n");
	exit(1);
    }

    while(fgets(cmd, sizeof cmd, stdin)) {
	TS_fs_index ind;
	char *c;
	switch(cmd[0]) {
	case 'i':
	    /* Insert */
	    c = cmd+1;
	    while(*c == ' ' || *c == '\t') ++c;
	    ind = TS_fs_insert(fs, (unsigned char *)c, strlen(c)+1);
	    printf("ind = %u\n", ind);
	    break;
	case 'd':
	    /* Delete */
	    if (sscanf(cmd+1, "%u", &ind) < 1) {
		fprintf(stderr, "Bad arg\n");
	    } else {
		ret = TS_fs_delete(fs, ind);
		printf("ret = %d\n", ret);
	    }
	    break;
	case 'g':
	    /* Get */
	    if (sscanf(cmd+1, "%u", &ind) < 1) {
		fprintf(stderr, "Bad arg\n");
	    } else {
		unsigned char *buf;
		unsigned int len;
		ret = TS_fs_getbuf(fs, ind, &buf, &len);
		printf("ret = %d\n", ret);
		if (ret >= 0) {
		    printf("buf = %s\n", buf);
		}
		if (ret == 1) {
		    TS_fs_freebuf(&buf, &len);
		}
	    }
	    break;
	default:
	    fprintf(stderr, "Bad cmd\n");
	    break;
	}
    }
    return 0;
}
