/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <stdlib.h>
#include "frPktFmt.h"
#include "filesys.h"
#include "prioqueue.h"

int main(int argc, char **argv)
{
    int fragsize = 11;
    int totsize = argc > 1 ? atoi(argv[1]) : 500;
    int ret;
    int numnodes;
    char cmd[1024];
    unsigned char *fs, *pq, *buf = malloc(totsize);
    if (!buf) {
	fprintf(stderr, "Out of mem!\n");
	exit(1);
    }
    
    /* The filesystem is ((numnodes+20)*(fragsize+8)) bytes, the prioqueue is
       (numnodes*8) bytes, so the total is (numnodes+20)*(fragsize+16)-160
       bytes.  pq occupies bytes 0..numnodes*8-1, fs occupies bytes
       numnodes*8..(numnodes+20)*(fragsize+16)-161.  Node that numnodes here
       _includes_ the header node 0 that can't be used for data. */
    
    numnodes = (totsize+160) / (fragsize+16);
    numnodes -= 20;

    if (numnodes < 1) {
	printf("Not enough buffer space allocated!\n");
	exit(1);
    }

    pq = buf;
    ret = TS_pq_new(pq, numnodes);
    if (ret < 0) {
	fprintf(stderr, "Unable to create pq\n");
	exit(1);
    }
    fs = pq + numnodes * 8;
    ret = TS_fs_new(fs, fragsize, numnodes+20);
    if (ret < 0) {
	fprintf(stderr, "Unable to create fs\n");
	exit(1);
    }

    printf("%d nodes created.\n", numnodes - 1);

    while(fgets(cmd, sizeof cmd, stdin)) {
	TS_fs_index ind;
	char *c;
	unsigned int prio;
	int plen;

	switch(cmd[0]) {
	case 'i':
	    /* Insert */
	    if (sscanf(cmd+1, "%u%n", &prio, &plen) < 1) {
		fprintf(stderr, "Bad arg\n");
	    } else {
		c = cmd+1+plen;
		while(*c == ' ' || *c == '\t') ++c;
		ind = TS_fs_insert(fs, (unsigned char *)c, strlen(c)+1);
		if (ind == 0) {
		    fprintf(stderr, "FS full\n");
		} else {
		    TS_fs_index todel;
		    ret = TS_pq_insert(pq, prio, ind, &todel);
		    if (todel) {
			TS_fs_delete(fs, todel);
		    }
		    if (ret < 0) {
			TS_fs_delete(fs, ind);
		    }
		    printf("ret = %d\n", ret);
		}
	    }
	    break;
	case 'g':
	    /* Getmin */
	    ind = TS_pq_delmin(pq);
	    if (ind == 0) {
		fprintf(stderr, "Queue empty\n");
	    } else {
		unsigned char *lbuf;
		unsigned int len;
		ret = TS_fs_getbuf(fs, ind, &lbuf, &len);
		if (ret >= 0) {
		    printf("buf = %s\n", lbuf);
		}
		if (ret == 1) {
		    TS_fs_freebuf(&lbuf, &len);
		}
		ret = TS_fs_delete(fs, ind);
		printf("ret = %d\n", ret);
	    }
	    break;
	default:
	    fprintf(stderr, "Bad cmd\n");
	    break;
	}
    }
    return 0;
}
