/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "frPktFmt.h"
#include "trafficshaper.h"

int main(int argc, char **argv)
{
    struct hostent *h;
    struct sockaddr_in them;
    int s, ret;
    TrafficShaperParams params = { 1000000, 0, 0 };
    TrafficShaperHandle tsh = NULL;
    unsigned char buf[1024];

    if (argc != 3) {
	fprintf(stderr, "Usage: %s host port\n", argv[0]);
	exit(1);
    }

    /* Look up the host */
    h = gethostbyname(argv[1]);
    if (h == NULL) {
	fprintf(stderr, "Host %s not found.\n", argv[1]);
	exit(2);
    }
    them.sin_family = PF_INET;
    them.sin_port = htons(atoi(argv[2]));
    memmove(&them.sin_addr.s_addr, h->h_addr, 4);

    /* Set up a socket */
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0) {
	perror("socket");
	exit(2);
    }

    /*
    ret = connect(s, (struct sockaddr *)&them, sizeof(them));
    if (ret < 0) {
	perror("connect");
	exit(2);
    }
    */

    tsh = TS_new(s, them, 11, 500, &params);
    if (!tsh) {
	fprintf(stderr, "TS_new failed\n");
	exit(2);
    }
    while(gets((char *)buf, sizeof buf, stdin)) {
	unsigned int prio;
	int plen;

	ret = sscanf((char *)buf, "%u%n", &prio, &plen);
	if (ret < 1) {
	    fprintf(stderr,
		"Bad format.  Must start with a priority number.\n");
	    continue;
	}
	if (prio == 0) {
	    /* Change params */
	    ret = sscanf((char *)buf+plen, "%lu %u %u", &params.usec_delay,
		&params.num_packets, &params.fillersize);
	    if (ret < 3) {
		fprintf(stderr, "Bad format.\n");
	    } else {
		ret = TS_params(tsh, &params);
		printf("ret = %d\n", ret);
	    }
	} else {
	    ret = TS_send(tsh, prio, buf+plen, strlen((char *)buf+plen));
	    printf("ret = %d\n", ret);
	}
    }
    TS_destroy(&tsh, ZK_TRUE);

    return 0;
}
