/*
 * $Id: frWorm.h,v 1.10 2001/04/19 20:32:04 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRWORM_H_
#define	FRWORM_H_

#ifndef _ZKINTTYPES_H
#include "zkIntTypes.h"
#endif /* _ZKINTTYPES_H */

#ifndef LIBPKFMT_H_
#include "frPktFmt.h"
#endif /* LIBPKTFMT_H_ */

#include "zkPackBuf.h"

#include <sys/un.h>          

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define	WORMCMD_VER_1    0x1
#define	WORMCMD_VER_CUR  WORMCMD_VER_1      /* Current version code */

#define	WORMCMD_CMD_SEND     0x1            /* Send data to ethernet */
#define	WORMCMD_CMD_NEWMAP   0x2            /* Create an ACI->tcp port map */
#define	WORMCMD_CMD_DELMAP   0x3            /* Destroy an ACI->tcp port map */
#define	WORMCMD_CMD_MSG      0x4            /* Send (error) msg to client */
#define	WORMCMD_CMD_KILL     0x5            /* Kill the wormhole */
#define WORMCMD_CMD_CONNECT  0x6            /* Connect access library */
#define	WORMCMD_CMD_GETSTAT  0x7            /* Get the wormhole status */
#define	WORMCMD_CMD_STAT     0x8            /* Sends wormhole status */
#define WORMCMD_CMD_BLOCKPORT 0x9           /* Block a port for an access lvl */
#define WORMCMD_CMD_OPENPORT 0xA            /* Open port for an access lvl */
#define WORMCMD_CMD_SENDSIGKEY 0xB          /* Send a signature key to the wormhole */
#define WORMCMD_CMD_SENDNISSCRYPTKEY 0xC    /* Send the NISS crypt key the wormhole */
#define WORMCMD_CMD_SENDNISSINFO 0xD        /* Send the NISS server information to the wormhole */

#define	WORMCMD_MSG_EACIFULL       0x1      /* No more available connections */
#define	WORMCMD_MSG_EACIPRESENT    0x2      /* ACI is already active */
#define	WORMCMD_MSG_EADDACIFAILED  0x3      /* Operation to add ACI failed */
#define WORMCMD_MSG_EDELACIFAILED  0x4      /* Operation to delete ACI failed */
#define	WORMCMD_MSG_EACINOTFOUND   0x5      /* Could not find requested ACI */
#define WORMCMD_MSG_TERMINATING    0x6      /* Wormhole has terminated */

/**
 * Returns a ZkIPHeader pointer which points into the FrWormholeCommand buffer
 * to the packet data.
 */ 

#define PACKETDATA(_x) ((void /* ZkIPHeader */ *)ZKPACKBUF_ADDR_OF((_x), ZKIPHEADER_SIZEOF))

#pragma pack(1)
struct T_FrWormholeCommand {
  uint32_t length;                   /* bytes in complete packet (inc. hdr) */
  unsigned char command;             /* Wormhole command code (see above) */
  unsigned char version;             /* Command version (should be 1) */
  unsigned char access_level;        /* Access level for these commands:
                                      * WORMCMD_CMD_SEND
                                      * WORMCMD_CMD_BLOCKPORT
                                      * WORMCMD_CMD_OPENPORT
                                      * must be between 0 and 32
                                      */
  unsigned char reserved[9];         /* For future use */
  FrAci ACI;                         /* ACI to which command / pkt refers */
};
#pragma pack()

typedef struct T_FrWormholeCommand FrWormholeCommand;

struct T_FrWormholeConnection {
  struct sockaddr_un wormhole;       /* Unix domain sock addr of wormhole */
  struct sockaddr_un me;             /* Unix domain sock addr of client */
  int timeout;                       /* Write timeout in seconds */
  int fd;                            /* Client socket */
};

typedef struct T_FrWormholeConnection FrWormholeConnection;

struct T_FrWormholeStatus {
  uint32_t sent;                     /* Number of pkts sent since startup */
  uint32_t rcvd;                     /* Number of pkts received since startup */
};

typedef struct T_FrWormholeStatus FrWormholeStatus;

int frWormholeWriteCommand (
  ZkError ,                           /* Used to return the error (if any) */
  FrWormholeConnection * ,            /* The connection to use */
  FrWormholeCommand * );              /* Pointer to the wormhole command */

int frWormholeReadCommand (
  ZkError ,                           /* Used to return the error (if any) */
  FrWormholeConnection * ,            /* Wormhole conn. from which to read */
  FrWormholeCommand * ,               /* Pointer to buffer to hold command */
  size_t * );                         /* Size of the command */

int frWormholeConnect (
  ZkError ,                           /* Used to return the error (if any) */
  FrWormholeConnection * ,            /* Stores the wormhole connection */
  const char * ,                      /* Unix domain sock filename of client */
  int );                              /* Connection timeout in seconds */

int frWormholeTerminate (
  ZkError ,                           /* Used to return the error (if any) */
  FrWormholeConnection * );           /* Wormhole connection to terminate */

DLLExport int frWormholeOpenPort (
  ZkError ,                           /* Return the error if any occured */
  FrWormholeConnection * ,            /* THe wormhole connection */
  uint16_t  ,                         /* The access level */
  uint16_t  ,                         /* Range of ports to open, low */
  uint16_t  ,                         /* ... high */
  int protocol);                      /* Protocol for the port */

DLLExport int frWormholeClosePort (
  ZkError ,                           /* Return the error if any occured */
  FrWormholeConnection * ,            /* THe wormhole connection */
  uint16_t  ,                         /* The access level */
  uint16_t  ,                         /* Range of ports to close, low */
  uint16_t  ,                         /* ... high */
  int protocol);                      /* Protocol for the port */

  

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRWORM_H_ */
