/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
/*
#include <arpa/inet.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <netinet/in.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
*/
#include <zkConfig.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkPacket.h>
#include <frWorm.h>

#define __PREAMBLE_SZ    256
#define	__DATAGRAM_SZ    100000

int
main (
  int argc ,
  char ** argv )
{
  FrWormholeStatus * status = NULL;
  char datablock[__PREAMBLE_SZ + __DATAGRAM_SZ];
  FrWormholeCommand * wormcmd = NULL;
  FrWormholeConnection c;
  size_t dlen;
  void /* ZkIPHeader */ * iph = NULL;
  ZkError err = NULL;
  ZkLog log = NULL;
#if 0
  int i = 0;  /* loop index */
#endif

  if (argc != 2) {
    fprintf(stderr,"usage: %s uds\n\n",argv[0]);
    exit(0);
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  frWormholeConnect(err,&c,argv[1],10);

#if 0
  for (i = 0; i < 1000; i++) {
    wormcmd = (FrWormholeCommand *) &(datablock[__PREAMBLE_SZ]);
    wormcmd->version = WORMCMD_VER_CUR;
    wormcmd->command = WORMCMD_CMD_NEWMAP;
    wormcmd->ACI = (FrAci) i;
    wormcmd->length = sizeof(FrWormholeCommand);
    frWormholeWriteCommand(err,&c,wormcmd); 
  }
#endif
  
  sleep(4);
  wormcmd = (FrWormholeCommand *) &(datablock[__PREAMBLE_SZ]);
  wormcmd->version = WORMCMD_VER_CUR;
  wormcmd->command = WORMCMD_CMD_GETSTAT;
  wormcmd->ACI = (FrAci) 0;
  wormcmd->length = sizeof(FrWormholeCommand);
   
  if (!frWormholeWriteCommand(err,&c,wormcmd)) {
    printf("frWormholeWriteCommand failed\n");
  } else {
    dlen = __DATAGRAM_SZ;
    if (!frWormholeReadCommand(err,&c,wormcmd,&dlen)) {
      printf("frWormholeReadCommand failed\n");
    } else {
      status = (FrWormholeStatus *) &(wormcmd[1]);
      printf("STATUS send %d rcvd %d\n",status->sent,status->rcvd);
    }
  }
  
  if (!frWormholeTerminate(err,&c)) {
    printf("Termination failed\n");
  }
  exit(0); 
  
  for (;;) {
    wormcmd = (FrWormholeCommand *) &(datablock[__PREAMBLE_SZ]);
    dlen = __DATAGRAM_SZ;
    frWormholeReadCommand(NULL,&c,wormcmd,&dlen);
    switch(wormcmd->command) {
    case WORMCMD_CMD_SEND:
      iph = PACKETDATA(wormcmd);
      printf("IPH version %d\n",zkIPHeaderGetVersion(iph));
      wormcmd = (FrWormholeCommand *) &(datablock[__PREAMBLE_SZ]);
      wormcmd->version = WORMCMD_VER_CUR;
      wormcmd->command = WORMCMD_CMD_NEWMAP;
      wormcmd->ACI = 0xDEAD;
      wormcmd->length = sizeof(FrWormholeCommand);
      frWormholeWriteCommand(NULL,&c,wormcmd); 
      break;
    case WORMCMD_CMD_CONNECT:
      break;
    case WORMCMD_CMD_NEWMAP:
      break;
    case WORMCMD_CMD_DELMAP:
      break;
    case WORMCMD_CMD_KILL:
      printf("received a kill command\n");
      break;
    case WORMCMD_CMD_MSG:
      printf( "Received message %d\n",  *((uint32_t *) &(wormcmd[1])));
/*
    case WORMCMD_CMD_RESTART:
    case WORMCMD_CMD_BYE:
*/
    default:
     printf("Unknown command %d\n",wormcmd->command);
      break;
    }
  }

  exit(0);
}
