/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#ifdef ZK_LINUX
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#if defined(__linux__)
#include <linux/if_ether.h>
#endif
#include <sys/un.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkPacket.h>
#include <frWorm.h>
#include "zkCStr.h"

#if 0
#define KWIKKILL
#endif

#define	__TEST_MODE

#define	WORM_OK		0
#define	WORM_ERROR	1

#define	__MAP_MX	128
#define	__LINE_SZ	128

struct P_MirrorContext {
  struct in_addr real_host;
  struct in_addr virtual_host;
  char ether[6];
  char debug;
  char verbose;
  int read_sock;
  int write_sock;
  int arp_sock;
  struct in_addr mapping[__MAP_MX][2];
  FrWormholeConnection fwc;
  int maps;
  ZkError err;
};

typedef struct P_MirrorContext MirrorContext;

#define	__DATAGRAM_SZ	65535
#define	__PREAMBLE_SZ     256 

/*
 * General stuff
 */


/*
 * Data structures used to keep track of outgoing connections so that packets
 * can be sent back to their origin.
 */


/*
 * Computing checksums
 *  
 * This code was snarfed from "Internetworking with TCP/IP, volume II" by
 * Comer, page 70.  It should work on any platform.  
 * In order to compute the checksum on an ICMP echo/reply message, you must
 * include the ICMP header AND the 56 data bytes which follow it.
 */

unsigned short
in_checksum (
  unsigned short * data ,
  int data_len )
{
  unsigned long checksum = 0;
  int i = 0;

  checksum = 0;
  for (i = 0; i < data_len; i++) {
    checksum += *data++;
  }
  checksum = (checksum >> 16) + (checksum & 0xFFFF);
  checksum += (checksum >> 16);
  return ~checksum;
}

unsigned short
tcp_checksum (
  struct iphdr * iph ,
  struct tcphdr * tcph )
{
  unsigned long checksum = 0;
  int i = 0;
  int len = 0;
  unsigned short * sptr = NULL;
  int data_len = 0;
  unsigned short * data = NULL;

  tcph->check = 0;
  sptr = (unsigned short *) &(iph->saddr);
  checksum = sptr[0] + sptr[1];
  sptr = (unsigned short *) &(iph->daddr);
  checksum += sptr[0] + sptr[1];
  len = ntohs(iph->tot_len) - (iph->ihl * 4);
  checksum += htons(6) + htons(len);
  
  if (len % 2) {
    ((char *)tcph)[len] = 0;
    len += 1;
  }

  len >>= 1;
  sptr = (unsigned short *) tcph; 
  for (i = 0; i < len; i++) {
    checksum += *sptr++;
  }
  checksum = (checksum >> 16) + (checksum &0xFFFF);
  checksum += (checksum >> 16);
  return ~checksum;
} 

int
dump_arp_header (
  FILE * stream ,
  struct arphdr * arph )
{
  unsigned char * data1 = NULL;
  unsigned char  * data = NULL;
  data1 = (unsigned char *) arph;
  data = &(data1[sizeof(struct arphdr)]);
  fprintf(stream,"hardware address format  %d\n",arph->ar_hrd );
  fprintf(stream,"protocol address format  %d\n",arph->ar_pro );
  fprintf(stream,"hardware address length  %d\n",arph->ar_hln );
  fprintf(stream,"protocol address length  %d\n",arph->ar_pln );
  fprintf(stream,"ARP command %d\n",ntohs(arph->ar_op));
  fprintf(stream,"sender hardware %02X:%02X:%02X:%02X:%02X:%02X\n",
          data[0],data[1],data[2],data[3],data[4],data[5]);
  fprintf(stream,"sender protocol address %d.%d.%d.%d\n",
          data[6],data[7],data[8],data[9]);
  fprintf(stream,"sender hardware %02X:%02X:%02X:%02X:%02X:%02X\n",
          data[10],data[11],data[12],data[13],data[14],data[15]);
  fprintf(stream,"sender protocol address %d.%d.%d.%d\n",
          data[16],data[17],data[18],data[19]);
  return;
}

int
mirror_arp_response (
  MirrorContext * context ,
  struct arphdr * arph,
  struct arphdr ** response )
{
  char * source = (char * )arph;
  static char buffer[128];
  *response = (struct arphdr *) buffer;
  (*response)->ar_hrd = htons(ARPHRD_ETHER);
  (*response)->ar_pro = htons(0x0800);
  (*response)->ar_hln = 6;
  (*response)->ar_pln = 4;
  (*response)->ar_op = 2;
  memcpy(&(buffer[8]),&(context->ether),6);
  memcpy(&(buffer[14]),&((context->virtual_host).s_addr),4);
  memcpy(&(buffer[18]),&(source[8]),6);
  memcpy(&(buffer[24]),&(source[14]),4);
  return WORM_OK;
} 
 
int
dump_data (
  FILE * stream ,
  unsigned char * data ,
  int data_len )
{
  int i = 0;
  int j = 0;

  for (i = 0; i < data_len; i+= 16) {
    fprintf(stream,"0x%04X  ",i);
    for (j = 0; j < 16; j++) {
      if ((i + j) >= data_len) {
        while (j < 16) {
          fprintf(stream,"   ");
          j++;
        }
        break;
      }
      fprintf(stream,"%02X ",data[i + j]);
    }
    for (j = 0; j < 16; j++) {
      if ((i + j) >= data_len) {
        break;
      }
      if (isprint(data[i + j])) {
        fprintf(stream,"%c",data[i + j]);
      } else {
        fprintf(stream,".");
      }
    }
    fprintf(stream,"\n");
  }
  return WORM_OK;
}

int
dump_ip_header (
  FILE * stream ,
  struct iphdr * iph )
{
  u_int16_t temp;
  struct in_addr addr;
  fprintf(stream,"IP packet header:\n");
  addr.s_addr = iph->saddr;
  fprintf(stream,"  SOURCE:  %s\n",inet_ntoa(addr));
  addr.s_addr = iph->daddr;
  fprintf(stream,"  DEST:    %s\n",inet_ntoa(addr));
  fprintf(stream,"  CHECKSUM:  %d\n",iph->check);
  return WORM_OK;
}

int
dump_icmp_header (
  FILE * stream ,
  struct icmphdr * icmph )
{
  u_int16_t temp;
  fprintf(stream,"  ICMP packet header:\n");
  fprintf(stream,"    TYPE:  %d\n",icmph->type);
  fprintf(stream,"    CODE:  %d\n",icmph->code);
  fprintf(stream,"    CKSM:  %d\n",icmph->checksum);
  if ((icmph->type == 8) || (icmph->type == 0)) {
    fprintf(stream,"    IDNT:  %d\n",(icmph->un).echo.id);
    fprintf(stream,"    SEQN:  %d\n",(icmph->un).echo.sequence);
  }
  return WORM_OK;
}

int
dump_tcp_codebits (
  FILE * stream , 
  struct tcphdr * tcph )
{
  fprintf(stream,"    TCP code bits:  ");
  if (tcph->fin) {
    fprintf(stream,"FIN ");
  }
  if (tcph->syn) {
    fprintf(stream,"SYN ");
  }
  if (tcph->rst) {
    fprintf(stream,"RST ");
  }
  if (tcph->psh) {
    fprintf(stream,"PSH ");
  }
  if (tcph->ack) {
    fprintf(stream,"ACK ");
  }
  if (tcph->urg) {
    fprintf(stream,"URG ");
  }
  fprintf(stream,"\n");
  return WORM_OK;
}

int
dump_tcp_header (
  FILE * stream ,
  struct tcphdr * tcph )
{
  fprintf(stream,"  TCP packet header:\n");
  dump_tcp_codebits(stream,tcph);
  fprintf(stream,"    SOURCE PORT:  %d\n",ntohs(tcph->source));
  fprintf(stream,"    DEST PORT:    %d\n",ntohs(tcph->dest));
  fprintf(stream,"    CHECKSUM:     %d\n",tcph->check);
  return WORM_OK;
}

/*
 *
 *
 */

int
mirror_load_map (
  MirrorContext * context ,
  char * filename )
{
  FILE * f = NULL;
  char buffer[__LINE_SZ];
  char * tok1 = NULL;
  char * tok2 = NULL;
  struct in_addr from;
  struct in_addr to;

  f = fopen(filename,"r");
  if (f == NULL) {
    perror(filename);
    return WORM_ERROR;
  }
  while (!feof(f)) {
    buffer[0] = (char) 0x0;
    fgets(buffer,__LINE_SZ,f);
    if (buffer[0] == (char) 0x0) {
      continue;
    }
    tok1 = strtok(buffer," \t\n");
    if (tok1 == NULL) {
      continue;
    }
    inet_aton(tok1,&from);
    tok2 = strtok(NULL," \t\n");
    if (tok2 == NULL) {
      continue;
    }
    inet_aton(tok2,&to);
    context->mapping[context->maps][0] = from;
    context->mapping[context->maps][1] = to;
    context->mapping[context->maps + 1][0] = to;
    context->mapping[context->maps + 1][1] = from;
    context->maps += 2;
    if (context->maps == __MAP_MX) {
      break;
    }
  } 
  fclose(f);
  return WORM_OK;
}   


int
dump_mapping (
  FILE * stream ,
  MirrorContext * context )
{
  int i = 0;
  struct in_addr from;
  struct in_addr to;
  char addr1[64];
  char addr2[64];
  
  for (i = 0; i < context->maps; i++) {
    zkCStrlcpy(addr1,inet_ntoa(context->mapping[i][0]),sizeof addr1);
    zkCStrlcpy(addr2,inet_ntoa(context->mapping[i][1]),sizeof addr2);
    fprintf(stream,"%s <-> %s\n",addr1,addr2);
  }
  return WORM_OK;
}



/*
 * %%Name:
 *   int mirror_init(context,virtual_host)
 * %%Parameters:
 *   MirrorContext * context
 *   struct in_addr virtual_host
 * %%Description:
 *   The WormholeContext keeps track of the sockets which are used to read
 *   and write to the ethernet, as well as some default addresses and ports
 *   which are required by the wormhole.  Most of the wormhole functions 
 *   require this context, so it should be created first.  You can easily
 *   create the virtual host's in_addr structure from a string representation
 *   of the address (eg "205.236.83.10") using the system function inet_aton(3)
 * %%Notes:
 *   wormhole_init() currently only sets up the sockets which are used to 
 *   read and write IP packets on the network.  Eventually, the function should
 *   also send out an ARP broadcast which advertises the existance and 
 *   address of the virtual host.
 * %%Return Value:
 *   A non-zero value is returned on error
 */

  
int
mirror_init (
  MirrorContext * context ,
  struct in_addr real_host ,
  struct in_addr virtual_host )
{
  int one = 1;			/* Used to turn on a boolean socket option */
  struct sockaddr_in udpaddr;	/* Connect to the other wormhole */
  int i = 0;			/* Loop index */

  memset(context,0,sizeof(MirrorContext));
  context->debug = 1;
  context->verbose = 1;

  context->real_host = real_host;
  context->virtual_host = virtual_host;

  context->maps = 0;

  /*
   * Create the socket to read traffic directly off the ethernet.
   */

  context->read_sock = socket(AF_INET,SOCK_PACKET,htons(ETH_P_IP));
  if (context->read_sock < 0) {
    if (context->debug) {
      perror("socket (ETH)");
    }
    goto fail;
  }

  /*
   * Create the ARP socket
   */

  context->arp_sock = socket(AF_INET,SOCK_PACKET,htons(ETH_P_ARP));
  if (context->arp_sock < 0) {
    if (context->debug) {
      perror("socket (ARP)");
    }
    goto fail;
  }

  /*
   * Create the socket used to write to the ethernet layer.  Notice that this
   * is not exactly the same type of thing as the reading socket.
   */

  context->write_sock = socket(AF_INET,SOCK_RAW,IPPROTO_RAW);
  if (context->write_sock < 0) {
    if (context->debug) {
      perror("socket (RAW)");
    }
    goto fail;
  }

  /*
   * The IP_HDRINCL functions allows us to play with the IP packet header
   * when we write datagrams back out to the network.
   */

  if (setsockopt(context->write_sock,
                 IPPROTO_IP,
                 IP_HDRINCL,
                 &one,
                 sizeof(one)) < 0) {
    if (context->debug) {
      perror("setsockopt");
    }
    goto fail;
  }

  /*
   * Connect to the wormhole...
   */

  if (!frWormholeConnect(context->err,&(context->fwc),"/tmp/wormhole.test",10)) {
    fprintf(stderr,"Could not connect to wormhole\n");
    goto fail;
  }
  
  
  return WORM_OK;
fail:
  return WORM_ERROR;
}

int
mirror_new_connection (
  MirrorContext * context ,
  FrAci aci ) 
{
  char * buffer[512];
  FrWormholeCommand * wormcmd;
#define NYMNAME "Uncle Goober"

  wormcmd = buffer;
  wormcmd->version = WORMCMD_VER_CUR;
  wormcmd->length = sizeof(FrWormholeCommand) + strlen(NYMNAME);
  wormcmd->command = WORMCMD_CMD_NEWMAP;
  wormcmd->ACI = aci;
  strncpy(&(wormcmd[1]),NYMNAME,strlen(NYMNAME));
fprintf(stderr,"Adding new aci %d\n",wormcmd->ACI);
#if 0
  frWormholeWriteCommand(context->err,context->fwc.fd,&(context->fwc.wormhole),&wormcmd);
#endif
fprintf(stderr,"About to call WriteCommand\n");
  if (frWormholeWriteCommand(context->err,&(context->fwc),wormcmd)) {
    fprintf(stderr,"WORMCMD_CMD_NEWMAP successful\n");
  } else {
    fprintf(stderr,"WORMCMD_CMD_NEWMAP failed\n");
  }
  return WORM_OK;
}

int
mirror_read_arp_packet (
  MirrorContext * context ,
  struct arphdr ** arph ,
  unsigned char ** data ,
  int * data_len ) 
{
  struct ethhdr * ethh = NULL;
  static unsigned char datagram[__DATAGRAM_SZ];
  struct sockaddr_in remote;
  int offset = 0;
  int remote_len = sizeof(struct sockaddr_in);
  int datagram_len = 0;
  u_int32_t * adrreq = NULL;
  struct arphdr * response = NULL;
  struct sockaddr_in destination;

  

  /*
   * Pick up a copy of an incoming datagram and check the destination address.
   * If it is not destined for the virtual host identified in the context 
   * structure, ignore the packet and keep looping, otherwise, strip the
   * ethernet header and return pointers to the IP header and the data following
   * it.
   */

  datagram_len = recvfrom(context->arp_sock,
                          datagram,
                          __DATAGRAM_SZ,
                          0,
                          (struct sockaddr *) &remote,
                          &remote_len);
  if (datagram_len < 0) {
    return WORM_ERROR;
  }
  offset = 0;
  ethh = (struct ethhdr *) &(datagram[offset]);
  offset += sizeof(struct ethhdr);
  *arph = (struct arphdr *) &(datagram[offset]);

  adrreq = (u_int32_t *) &(datagram[offset  + 24]);
  { unsigned char * as = (unsigned char *) adrreq;
  printf("--------------------    %d.%d.%d.%d\n",as[0],as[1],as[2],as[3]);
    as = (unsigned char *) &((context->virtual_host).s_addr);
  printf("-----------VHOST----    %d.%d.%d.%d\n",as[0],as[1],as[2],as[3]);
  }
  if (*adrreq == (context->virtual_host).s_addr) {
    int bytes = 0;
    printf("SOMEONE WANTS MY VIRTUAL ADDRESS!!!!!!!!!!!!!!\n");
    mirror_arp_response(context,*arph,&response);

    memset(&destination,0,sizeof(struct sockaddr_in));
    destination.sin_family = AF_INET;
    destination.sin_addr.s_addr = htonl(*((u_int32_t *) &(datagram[offset  + 14])));
    printf("DESTINATION ADDRESS OF ARP PACKET 0x%08X\n",destination.sin_addr.s_addr);
    destination.sin_port = 0;
  
    bytes = sendto(context->arp_sock,
           response,
           28,
           0,
           (struct sockaddr *) &destination,
           sizeof(struct sockaddr_in));
    printf("           bytes sent..... %d\n",bytes);
/*
    dump_arp_header(stderr,response);
*/
  }
/*
  if ((*iph)->daddr != (context->virtual_host).s_addr) {
    return WORM_ERROR;
  }
  offset += (*iph)->ihl * 4;
  *data = (unsigned char *) &(datagram[offset]);
  *data_len = ntohs((*iph)->tot_len) - ((*iph)->ihl * 4);
  dump_arp_header(stderr,*arph);
*/
  return WORM_OK;
}


int
mirror_read_wormhole (
  MirrorContext * context ,
  struct iphdr ** iph ,
  FrAci * aci ,
  unsigned char ** data ,
  int * data_len ) 
{
  struct ethhdr * ethh = NULL;
  struct tcphdr * tcph = NULL;
  static unsigned char datagram[__PREAMBLE_SZ + __DATAGRAM_SZ];
  struct sockaddr_in remote;
  int offset = 0;
  int remote_len = sizeof(struct sockaddr_in);
  int datagram_len = 0;
  FrWormholeCommand * wormcmd = NULL;

  wormcmd = (FrWormholeCommand *) &(datagram[__PREAMBLE_SZ]);
  datagram_len = __DATAGRAM_SZ;
#if 0
  if (!frWormholeReadCommand(context->err,context->fwc.fd,wormcmd,&datagram_len)) {
#endif
  if (!frWormholeReadCommand(context->err,&(context->fwc),wormcmd,&datagram_len)) {
    fprintf(stderr,"frWormholeReadCommand:  failed\n");
  }
  offset = __PREAMBLE_SZ + sizeof(FrWormholeCommand);
  if (wormcmd->command == WORMCMD_CMD_STAT) {
    FrWormholeStatus * status = NULL;
    status = (FrWormholeStatus *) &(wormcmd[1]);
    printf("STATUS:  SENT %d RCVD %d\n",status->sent,status->rcvd);
    return WORM_ERROR;
  } else if (wormcmd->command == WORMCMD_CMD_MSG) {
    uint32_t * mesgcode = NULL;
    mesgcode = (uint32_t *) &(wormcmd[1]);
    printf("Received message id code %d\n",*mesgcode);
    if (*mesgcode == WORMCMD_MSG_TERMINATING) {
      unlink(context->fwc.me.sun_path);
      close(context->fwc.fd);
      exit(0);
    } else if (*mesgcode > 10) {
      ((char *)mesgcode)[wormcmd->length] = (char) 0;
      printf("MESSAGE STRING \"%s\"\n",(char *)mesgcode);
    }
    return WORM_ERROR;
  } else if (wormcmd->command != WORMCMD_CMD_SEND) {
printf("Unrecognized command %d\n",wormcmd->command);
    return WORM_ERROR;
  }
  *aci = wormcmd->ACI;
  *iph = &(wormcmd[1]);
  *data = &(((char *)(*iph))[(*iph)->ihl * 4]);
  *data_len = ntohs((*iph)->tot_len) - ((*iph)->ihl * 4);
  return WORM_OK;
}


int
mirror_write_wormhole (
  MirrorContext * context ,
  struct iphdr * iph ,
  FrAci aci ,
  char * data ,
  int data_len )
{
  int bytes = 0;
  unsigned char datagram[__PREAMBLE_SZ + __DATAGRAM_SZ];
  unsigned char *mydata = NULL;
  FrWormholeCommand * wormcmd = NULL;

  wormcmd = (FrWormholeCommand *)datagram;
  wormcmd->version = WORMCMD_VER_CUR;
  wormcmd->command = WORMCMD_CMD_SEND;
  wormcmd->length = sizeof(FrWormholeCommand) + ntohs(iph->tot_len);
  wormcmd->ACI = aci;
  mydata = (unsigned char *)&(wormcmd[1]);
  memcpy(mydata,iph,wormcmd->length);
#if 0
  if (!frWormholeWriteCommand(context->err,context->fwc.fd,&(context->fwc.wormhole),wormcmd)) {
#endif
fprintf(stderr,"About to call WriteCommand\n");
  if (!frWormholeWriteCommand(context->err,&(context->fwc),wormcmd)) {
    fprintf(stderr,"frWormholeWriteCommand:  failed\n");
  }
fprintf(stderr,"WriteCommand returned\n");
  return WORM_OK;
}

/*
 * %%Name:
 *   int wormhole_read_ip_packet(context,iph,data,data_len)
 * %%Parameters:
 *   WormholeContext * context
 *   struct iphdr ** iph
 *   unsigned char ** data
 *   int * data_len
 * %%Description:
 *   This function is used to read a packet destined for the wormhole off
 *   the network.  It will block until such a packet is encountered and will
 *   not react to packets which are destined for the real host.  On return,
 *   pointers to the IP header and following data are sent back.  Note that
 *   these will be contiguous in memory.  For now, the function uses a 
 *   static buffer area to hold the information.  You must process the packet
 *   before calling this function again.  This isnt really the best way to
 *   do this for a system that will have a high load.  There should be a 
 *   packet queue instead which is continuously updated by the wormhole and
 *   this function should then pop off a packet from the top of the queue.
 * %%Return Value:
 *   A non-zero value is returned on error or if the packet seen was not 
 *   destined for the virtual host.
 */

int
wormhole_read_ip_packet (
  MirrorContext * context ,
  struct iphdr ** iph ,
  unsigned char ** data ,
  int * data_len ) 
{
  struct ethhdr * ethh = NULL;
  struct tcphdr * tcph = NULL;
  static unsigned char datagram[__PREAMBLE_SZ + __DATAGRAM_SZ];
  struct sockaddr_in remote;
  int offset = 0;
  int remote_len = sizeof(struct sockaddr_in);
  int datagram_len = 0;
  FrWormholeCommand * wormcmd = NULL;
  

  /*
   * Pick up a copy of an incoming datagram and check the destination address.
   * If it is not destined for the virtual host identified in the context 
   * structure, ignore the packet and keep looping, otherwise, strip the
   * ethernet header and return pointers to the IP header and the data following
   * it.
   */

#if 0
  wormcmd = (FrWormholeCommand *) &(datagram[__PREAMBLE_SZ]);
  frWormholeReadCommand(context->err,context->fwc.fd,wormcmd,&datagram_len);
  offset = __PREAMBLE_SZ + sizeof(FrWormholeCommand);
#endif

  datagram_len = recvfrom(context->read_sock,
                          datagram,
                          __DATAGRAM_SZ,
                          0,
                          (struct sockaddr *) &remote,
                          &remote_len);
  if (datagram_len < 0) {
printf("recvfrom failed\n");
    return WORM_ERROR;
  }
  offset = 0;
  ethh = (struct ethhdr *) &(datagram[offset]);
  if (ethh->h_proto == ETH_P_ARP) {
    printf("ARP request!\n");
  }
  offset += sizeof(struct ethhdr);

  *iph = (struct iphdr *) &(datagram[offset]);

  if ((*iph)->daddr != (context->virtual_host).s_addr) {
    return WORM_ERROR;
  }
  offset += (*iph)->ihl * 4;
  *data = (unsigned char *) &(datagram[offset]);
  *data_len = ntohs((*iph)->tot_len) - ((*iph)->ihl * 4);

  return WORM_OK;
}

/*
 * %%Name:
 *   int wormhole_write_ip_packet(context,iph,data,data_len)
 * %%Parameters:
 *   WormholeContext * context
 *   struct iphdr * iph
 *   char * data
 *   int data_len
 * %%Description:
 *   This function is used to write a new datagram out to the network layer
 *   although you must pass a pointer to the IP header structure and another
 *   to the data for the packet, this function assumes that the data area 
 *   immediatly follows the IP packet header in memory.  ie:
 *                  _________________________________________
 *                 | IP Header | Data... (data_len bytes)    | 
 *                  -----------------------------------------
 * %%Return Value:
 *   A non-zero value is returned on error.
 */

int
mirror_write_ip_packet (
  MirrorContext * context ,
  struct iphdr * iph ,
  char * data ,
  int data_len )
{
  int bytes = 0;
  unsigned char datagram[__PREAMBLE_SZ + __DATAGRAM_SZ];
  FrWormholeCommand * wormcmd = NULL;
  struct sockaddr_in destination;

  /*
   * First, set our destination address.  This has to be extracted from the
   * IP packet header which was passed to this function.  Note that you must
   * still pass the packet header to the sendto() function.
   */
#if 0
printf("Setting mss\n");
  zkIPHeaderStripOptions((ZkIPHeader *)iph);
/*
  zkTCPHeaderSetMSS((ZkTCPHeader *) iph,12);
  zkTCPHeaderStripOptions((ZkTCPHeader *)iph);
  zkTCPHeaderChecksum((ZkTCPHeader *) iph);
*/
#endif
printf("Writing %d bytes to network\n", ntohs(((ZkIPHeader *)iph)->TotalLength));
  memset(&destination,0,sizeof(struct sockaddr_in));
  destination.sin_family = AF_INET;
  destination.sin_addr.s_addr = iph->daddr;
  destination.sin_port = 0;
  
  /*
   * Now compute a new checksum for the IP packet header to ensure propert
   * delivery.
   */

  iph->check = in_checksum((unsigned short *) iph,iph->ihl * 2);

  /*
   * Put our packet on the network.
   */

  bytes = sendto(context->write_sock,
                 iph,
                 data_len + (iph->ihl) * 4,
                 0,
                 (struct sockaddr *) &destination,
                 sizeof(struct sockaddr_in));

  if (bytes < 0) {
    if (context->debug) {
      fprintf(stderr,"ERRNO %d  ",errno);
      perror("sendto");
    }
    goto fail;
  }
 
  return WORM_OK;
fail:
  return WORM_ERROR;
}

/*
 * This function runs the virtual host wormhole.
 */

int
mirror_run (
  MirrorContext * context)
{
  struct ethhdr * ethh = NULL;	/* Ethernet packet header structure */
  struct iphdr * iph = NULL;	/* IP packet header structure */
  struct arphdr * arph = NULL;	/* ARP packet header structure */
  struct tcphdr * tcph = NULL;	/* TCP packet header structure */
  unsigned char * data = NULL;
  int data_len = 0;		/* Number of bytes read of the wire */
  int err = WORM_OK;
  int i = 0;			/* Loop index */
  u_int32_t tmp;		/* For swapping addresses */
  int acis[0x100];
  int count = 0;

  memset(acis,0,sizeof(int) * 0x100);

 
  /*
   * Initialize the return address array.  This is used to map an incoming
   * TCP port to a source address...
   */

  /*
   * This loop is used to scan the network... we'll look for IP packets which
   * are destined for the address which was passed as one of the arguments to
   * this function. (This IP address is the virtual host which we mapped to 
   * the computer running this server using ARP)
   */

  for (;;) {
    struct in_addr addr_t;
    fd_set readfds;
    int n = 0;

    /*
     * Wait for incoming data... this can come either from the wormhole itself
     * or from the UDP connection which is used to send packets back and forth
     */

    FD_ZERO(&readfds);
    n = context->fwc.fd;
    if (context->read_sock > n) {
      n = context->read_sock;
    }
    n += 1;
    FD_SET(context->fwc.fd,&readfds);
    FD_SET(context->read_sock,&readfds);
    select(n,&readfds,NULL,NULL,NULL);

    if (FD_ISSET(context->read_sock,&readfds)) {

      /* 
       * Packets entering the wormhole must be grabbed and spit out through the
       * UDP connection.  No munging is done at this point (In the production
       * version of the wormhole, munging will be done on the packet data)
       */

      err = wormhole_read_ip_packet(context,&iph,&data,&data_len);    
      if (err == WORM_OK) {
        FrAci machine = 0;

#ifdef KWIKKILL
      count += 1;
      if ((count % 100) == 0 ) {
        ZkLog log = NULL;
   
#if 0
        FrWormholeCommand wormstat;

        wormstat.length = sizeof(FrWormholeCommand);
        wormstat.version = WORMCMD_VER_CUR;
        wormstat.command = WORMCMD_CMD_KILL;
        wormstat.ACI = 0;
        frWormholeWriteCommand(context->err,context->fwc.fd,&(context->fwc.wormhole),&wormstat);
#endif
     
        if (frWormholeTerminate(context->err,&(context->fwc))) {
          printf("Successfully terminated the wormhole\n");
        } else {
          printf("Terminate wormhole failed\n");
        }
/*
        zkErrorFree(&err);
*/
        exit(0);
      }
#endif /* KWIKKILL */
      /*
       * At this point, I'm allowed to screw around with the IP header, but not
       * the contained data.  In the case of TCP the ports are required to be
       * intact so that the exit wormhole can munge them (technically this
       * could be partially done here)
       */
printf("Read %d bytes in a packet from the network\n",ntohs(((ZkIPHeader *)iph)->TotalLength));

       machine = (FrAci) ((char *)&(iph->saddr))[3];
       if (acis[(unsigned char) machine] == 0) {
         acis[(unsigned char)machine] = 1;
         mirror_new_connection(context,(FrAci) machine);
       }
       iph->daddr = iph->saddr;
       for (i = 0; i < context->maps; i++) {
         if (iph->saddr == context->mapping[i][0].s_addr) {
           iph->daddr = context->mapping[i][1].s_addr;
           break;
         }
       }
       iph->saddr = context->virtual_host.s_addr;
      
/*
       if (iph->protocol == 6) {
         tcph = (struct tcphdr *) data;
         tcph->check = tcp_checksum(iph,tcph);
       }
*/
fprintf(stderr,"About to write to wormhole\n");
       err = mirror_write_wormhole(context,iph,machine,data,data_len);
fprintf(stderr,"done write to wormhole\n");
/*
       err = mirror_write_ip_packet(context,iph,data,data_len);
 */
      }
    } else if (FD_ISSET(context->fwc.fd,&readfds)) {
      FrAci aci;
      unsigned char destaddr[4];
      destaddr[0] = 192;
      destaddr[1] = 168;
      destaddr[2] = 1;
      if (mirror_read_wormhole(context,&iph,&aci,&data,&data_len) != WORM_OK) {
      }  else {
        destaddr[3] = (unsigned char) aci; 
        zkIPHeaderSetSourceAddress((ZkIPHeader *)iph,(__int32)context->virtual_host.s_addr);
        zkIPHeaderSetDestinationAddress((ZkIPHeader *)iph,*((__int32 *)destaddr));
        zkTCPHeaderChecksum((ZkTCPHeader *) iph);
#if 0
zkIPHeaderDump((ZkIPHeader *) iph,stdout);
zkTCPHeaderDump((ZkTCPHeader *)iph,stdout);
#endif
       err = mirror_write_ip_packet(context,iph,data,data_len);
      
      }
    } else if (FD_ISSET(context->arp_sock,&readfds)) {
#if 0
      /*
       * ARP stuff is not required for this test program 
       */
      mirror_read_arp_packet(context,&arph,&data,&data_len);
#endif
    } else {
      if (context->debug) {
        fprintf(stderr,"wormhole_run:  nothing written to wire\n");
      }
    }
  } 
     

fail:
  return WORM_ERROR;
}

int
main (
  int argc ,
  char ** argv )
{
  MirrorContext context;
  struct in_addr address;
  struct in_addr real;
  int err = WORM_OK;
  ZkLog log = NULL;
  char myEther[6] = { 0x00, 0x10, 0x4b, 0x26, 0x30, 0x10 };

  if (argc != 4) {
    fprintf(stderr,"usage: %s real_host virtual_host mapfile\n\n",argv[0]);
    return 0;
  }

  zkErrorNew(&context.err);
  zkErrorLog(context.err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);
  inet_aton(argv[2],&address); 
  inet_aton(argv[1],&real);
  err = mirror_init(&context,real,address);
  if (err != WORM_OK) {
    return err;
  } 
  memcpy(&(context.ether),myEther,6);
  mirror_load_map(&context,argv[3]);
  dump_mapping(stderr,&context);
  err = mirror_run(&context);
 
  return err;
}
 
#else /* ZK_LINUX */

int
main (
  int argc ,
  char ** argv )
{
  fprintf (stderr, "IP MIRROR ONLY RUNS ON LINUX\n");
  exit (1);
}

#endif /* ZK_LINUX */
