/* 
 * $Id: zkNetInfoHash.h,v 1.2 2001/04/19 20:32:29 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZK_NETINFO_HASH_H_
#define ZK_NETINFO_HASH_H_

#include "zks.h"
#include "zkError.h"
#include "zkHash.h"
#include "zkNetInfo.h"

typedef enum E_ZkNetInfoKeyType { 
  ZKNETINFO_KEY_NONE, 
  ZKNETINFO_KEY_ID,             /* Key equality on the ID */
  ZKNETINFO_KEY_SADDR,          /* Key equality on the ip and port numbers */
  ZKNETINFO_KEY_MAX             /* 
                                 * Place holder.  Do NOT delete from last 
                                 * position in this enumerated type.
                                 */ 
} ZkNetInfoKeyType;

typedef struct T_ZkNetInfoKey * ZkNetInfoKey;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  DLLExport extern
  int zkNetInfoHashNew( ZK_INOUT ZkError err,
			ZK_OUT   ZkHash * h
			);

  DLLExport extern
  int zkNetInfoHashFree( ZK_INOUT ZkError err,
			 ZK_INOUT ZkHash * h );

  DLLExport extern
  int zkNetInfoHashKeyNew( ZK_INOUT ZkError err,
			   ZK_INOUT ZkNetInfoKey *key );

  DLLExport extern 
  int zkNetInfoHashKeyFree( ZK_INOUT ZkError err,
			    ZK_INOUT ZkNetInfoKey *key );

  DLLExport extern
  int zkNetInfoHashKeyMake( ZK_INOUT ZkError err,
			    ZK_IN    const ZkNetInfoKeyType type,
			    ZK_IN    const uint32_t *id,
			    ZK_IN    const struct sockaddr_in *saddr,
			    ZK_OUT   ZkNetInfoKey *key );

  DLLExport extern 
  int zkNetInfoHashInsert( ZK_INOUT ZkError err,          /* Error object */
			   ZK_IN    ZkHash h,             /* Hash Table */
			   ZK_IN    ZkNetInfoDatum data); /* Hash table datum*/

  DLLExport extern
  int zkNetInfoHashDelete( ZK_INOUT ZkError err,
			   ZK_INOUT ZkHash h, 
			   ZK_IN    ZkNetInfoDatum data);

  DLLExport extern
  int zkNetInfoHashFind( ZK_INOUT ZkError err,
			 ZK_IN    ZkHash h,
			 ZK_IN    ZkNetInfoKey key,
			 ZK_OUT   ZkNetInfoDatum *data );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_NETINFO_HASH_H_ */
