/*
 * $Id: t_zkNetInfo.c,v 1.2 2001/04/19 20:32:32 alainc Exp $ 
 *
 * Test zkNetInfo object
 */
#include "zkIntTypes.h"
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h> 
#include "zkError.h"
#include "frEnt.h"
#include "zkList.h"
#include "zkNetInfo.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "zkLog.h"
#include "zkParms.h"
#include "zkFile.h"
#include "zkAlloc.h"
#include "zkCStr.h"
#include "sys/stat.h"



/*
 * Test harness stuff
 */
#define NUMTEST     7
#define PASS        0
#define FAIL        1
#define UNRESOLVED  2
#define NOTINUSE    3
#define UNSUPPORTED 4
#define UNTESTED    5
#define UNINITIATED 6
#define NORESULT    7
#define FATAL       101
#define INSPECT     102


/* Display strings */
#define mesg_1 "%s:  Missing argument for parameter:  %s.\n"
#define mesg_2 "%s:  Unrecognized command line parameter:  %s.\n"
#define mesg_3 "%s:  Error in command line, try --"P_HELP".\n"

#define mesg_help "\n\nUsage: %s options\n\n" \
"Where options can be one or more of the following:\n\n" \
"  --"P_DEBUG"                     Enable debugging mode.\n" \
"  --"P_HELP"                      Show this screen.\n" \
"  --"P_LOGFILE" filename          Send ouput to file \"filename\" (%s).\n" \
"  --"P_DEBUGFILE" filename        Original data for comparisons.\n" \
"  --"P_VERBOSE" level             From 0 to 100\n" \
"\nValues in parentheses are default values if the option is not specified.\n"\
"\n"


/*
 * Test data 
 */
char prog[256] = "";

#define MAX_AIPS 10
#define MAX_SERVERS 11


typedef
struct aip_data {
    uint32_t id;
    FrEntIdExp entid;
    const char *host;
    unsigned short port;
    unsigned short dhport;
    int rating;
    int status;
    const char * region_code;
    const char * region;
    const char * operator;
    uint32_t neighbours;
    uint32_t neighbourId[MAX_AIPS];
} AipData;

static AipData aip[] = {
    { 48, {FRENT_TYPE_AIP, "defcon"},          
         "207.173.052.002", 51101, 51102, 
         1, 128, 
         "US53", "Washington, United States", "DEF CON", 
         5, { 96, 192, 240, 550, 35, 000, 000, 000, 000, 000}
    },
    { 96, {FRENT_TYPE_AIP, "rsc03-tx"},        
         "209.163.236.144", 51101, 51102,
         1, 128, 
         "US48", "Texas, United States", "Zero-Knowledge Systems",
         5, { 48, 51, 99, 163, 195, 000, 000, 000, 000, 000}
    },
    { 92, {FRENT_TYPE_AIP, "nobigbrother"},
         "206.221.233.026", 51101, 51102, 
         1, 128, 
         "US17", "Illinois, United States", "Kirkland" ,
         5, { 48, 240, 550, 35, 51, 000, 000, 000, 000, 000}
    },
    { 40, {FRENT_TYPE_AIP, "dgn03-va"},
         "209.207.174.161", 51101, 51102,
          1, 128, 
          "US51", "Virginia, United States", "Zero-Knowledge systems" ,
          5, { 48, 192, 99, 163, 195, 000, 000, 000, 000, 000}
    },
    { 50, {FRENT_TYPE_AIP, "carrier1-fra-01"},
         "212.004.195.106", 51101, 51102, 
         1, 128, 
         "GM", "Germany", "Carrier1"  ,
         5, { 48, 192, 35, 51, 99, 000, 000, 000, 000, 000}
    },
    { 35, {FRENT_TYPE_AIP, "core-zks-01"},
         "207.105.115.218", 51101, 51102,
         1, 128, 
         "CA104536N07324W", "Quebec, Canada", "Zero-Knowledge Systems"  ,
         5, { 48, 192, 550, 51, 99, 000, 000, 000, 000, 000}
    },
    { 51, {FRENT_TYPE_AIP, "rsc02-tx"},
         "209.163.236.148", 51101, 51102,
         1, 128, 
         "US48", "Texas, United States", "Zero-Knowledge Systems"  ,
         5, { 96, 192, 550, 35, 99, 000, 000, 000, 000, 000}
    },
    { 99, {FRENT_TYPE_AIP, "carrier1-lon-01"},
         "212.004.198.138", 51101, 51102,
         1, 128, 
         "UK17", "Greater London, United Kingdom", "Carrier1"  ,
         5, { 96, 240, 550, 35, 51, 000, 000, 000, 000, 000}
    },
    { 63, {FRENT_TYPE_AIP, "aldigital"},
         "213.129.065.012", 51101, 51102,
         1, 128, 
         "UK17", "Greater London, United Kingdom", "A.L. Digital Ltd"  ,
         3, { 96, 240, 195, 000, 000, 000, 000, 000, 000, 000}
    },
    {195,{FRENT_TYPE_AIP, "exo01-ca"},
         "216.032.190.225", 51101, 51102, 
         1, 128, 
         "US06", "California, United States", "Zero-Knowledge Systems"  ,
         3, { 96, 240, 163, 000, 000, 000, 000, 000, 000, 000}
    },
};

static AipData server[] = {
    { 24, {FRENT_TYPE_FMG, "FMG"}, 
      "207.107.115.67", 51110, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 28, {FRENT_TYPE_NYM_SRV, "NymServer"},
      "207.107.115.205", 51104, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 29, {FRENT_TYPE_NYM_SRV, "NymServer1"},
      "207.107.115.206", 51104, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 30, {FRENT_TYPE_TOK_SRV, "TokenServer"},
      "207.107.115.207", 51103, 0,
      1, 128,
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 25, {FRENT_TYPE_KEYQRY_SRV, "KeyQryServer"},
      "207.107.115.204", 51109, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },    
    { 23, {FRENT_TYPE_KEYUPD_SRV, "KeyUpdServer"}, 
      "207.107.115.65",  51108, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 22, {FRENT_TYPE_NNTP_SRV, "NewGroupsServer"},
      "207.107.115.67", 51105, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 26, {FRENT_TYPE_NIQS, "NIQS"}, 
      "207.107.115.201", 51107, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 31, {FRENT_TYPE_NIQS, "NIQS1"},
      "207.107.115.202", 51107, 0,
      1, 128, 
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 27, {FRENT_TYPE_NISS, "NISS"},
      "207.107.115.201", 51113, 0,
      1, 128,
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    { 33, {FRENT_TYPE_POP, "pop1"}, 
      "207.107.115.230", 51114, 0,
      1, 128,
      "CDN2", "Quebec, Canada", "Zero-Knowledge Systems",
      0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
};





/*
 * Configuration data
 */

struct T_PROG_CONF {
    const char * progName;
    uint8_t      debug;
    const char * debugFile;

    FILE       * outputFile;
    uint8_t      verbosity;
};

typedef struct T_PROG_CONF *ProgConf;









/*
 *  Usage printing function
 */

static void usage( ZkError err, ProgConf conf )
{

    if( conf != NULL ) {
        (void) zkFprintf( err, conf->outputFile, mesg_help , 
                          conf->progName,
                          "stderr" );
    }

    exit(PASS);
}







/**
 * Insert AIPS
 */
static int insert_aips(ZkError err, ZkNetInfo ni)
{
    const char *func = "insert_aips";
    ZkNetInfoDatum data = NULL;
    int i;
    uint32_t idx;
    struct sockaddr_in addr;
    
    if (!ni) {
        zkErrorSet(err, func, ZKERR, ZKMNETINFO, ZKENULLPTR, "NULL param");
        return 0;
    }
    
    for (i = 0; i < MAX_AIPS; i++) {
        if (!zkNetInfoDatumNew(err, &data)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetId(err, data, aip[i].id)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetEntity(err, data, aip[i].entid)) {
            return 0;
        }
        
        addr.sin_family = AF_INET;
        addr.sin_port = htons(aip[i].port);
        addr.sin_addr.s_addr = inet_addr(aip[i].host);
        
        if (!zkNetInfoDatumSetSockaddr(err, data, addr)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetDifHelPort(err, data, aip[i].dhport)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetRating(err, data, aip[i].rating)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetStatus(err, data, aip[i].status)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetRegionCode(err, data, aip[i].region_code)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetRegion(err, data, aip[i].region)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetOperator(err, data, aip[i].operator)) {
            return 0;
        }
        
        for( idx = 0; idx < aip[i].neighbours; idx++ ) {
            if( !zkNetInfoDatumAddNb(err, data, aip[i].neighbourId[idx])) {
                return 0;
            }
        }
        if (!zkNetInfoInsert(err, ni, data)) {
            return 0;
        }
        data = NULL;
    }
    
    return 1;
}









/* 
 * Insert server entities
 */
static int insert_servers(ZkError err, ZkNetInfo ni)
{
    const char *func = "insert_servers";
    ZkNetInfoDatum data = NULL;
    int i;
    struct sockaddr_in addr;
    
    if (!ni) {
        zkErrorSet(err, func, ZKERR, ZKMNETINFO, ZKENULLPTR, "NULL param");
        return 0;
    }
    
    for (i = 0; i < MAX_SERVERS; i++) {
        if (!zkNetInfoDatumNew(err, &data)) {
            return 0;
        }
        
        addr.sin_family = AF_INET;
        addr.sin_port = htons(server[i].port);
        addr.sin_addr.s_addr = inet_addr(server[i].host);
        
        if (!zkNetInfoDatumSetId(err, data, server[i].id)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetEntity(err, data, server[i].entid)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetSockaddr(err, data, addr)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetRating(err, data, server[i].rating)) {
            return 0;
        }
        
        if (!zkNetInfoDatumSetStatus(err, data, server[i].status)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetRegionCode(err, data, server[i].region_code)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetRegion(err, data, server[i].region)) {
            return 0;
        }
        
        if( !zkNetInfoDatumSetOperator(err, data, server[i].operator)) {
            return 0;
        }
        
        if (!zkNetInfoInsert(err, ni, data)) {
            return 0;
        }
        
        data = NULL;
    }
    
    return 1;
}









/**
 * aipDataSprintf
 */
static int aipDataSprintf( ZkError err, 
                           ProgConf conf, 
                           AipData *aipdata, 
                           struct sockaddr_in * addr,
                           char **str )
{
    static const char * func = "aipDataSprintf";
    char *aipStr = NULL;
    char *tmpStr = NULL;
    size_t idx;

    if( !conf || !str || !aipdata ) {
        zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR,"NULL param(s)");
        return 0;
    }
    zkCStrSprintf( &aipStr, "%5.5d %9.9s:%-64.64s\n", 
                   aipdata->id,
                   frEntTypeGetStr(aipdata->entid.type),
                   aipdata->entid.name);
    

    tmpStr = aipStr;
    zkCStrSprintf( &aipStr, "%s    %15.15s:%5.5d",
                   tmpStr,
                   inet_ntoa(addr->sin_addr), 
                   ntohs(addr->sin_port));
    zkCStrFree( &tmpStr );


    if( aipdata->entid.type == FRENT_TYPE_AIP ) {
        tmpStr = aipStr;
        zkCStrSprintf( &aipStr, "%s:%5.5d ", 
                       tmpStr,
                       aipdata->dhport);
        zkCStrFree( &tmpStr );
    }
    else {
        tmpStr = aipStr;
        zkCStrSprintf( &aipStr, "%s       ", 
                       tmpStr);
        zkCStrFree( &tmpStr );
    }


    tmpStr = aipStr;
    zkCStrSprintf( &aipStr, "%s%2.2s %3.3d",
                   tmpStr,
                   aipdata->status!=0?"Up":"Dn", 
                   aipdata->rating);
    zkCStrFree( &tmpStr );


    tmpStr = aipStr;
    zkCStrSprintf( &aipStr, "%s %-15.15s\n    %-64.64s\n    %-64.64s\n",
                   tmpStr,
                   aipdata->region_code, 
                   aipdata->region, 
                   aipdata->operator);
    zkCStrFree( &tmpStr );


    if( aipdata->entid.type == FRENT_TYPE_AIP ) {
        tmpStr = aipStr;
        zkCStrSprintf( &aipStr, "%s    ", 
                       tmpStr );
        zkCStrFree( &tmpStr );

        if( aipdata->neighbours == 0 ) {
            tmpStr = aipStr;
            zkCStrSprintf( &aipStr, "%sNo neighbours",
                           tmpStr );
            zkCStrFree( &tmpStr );
        } else {
            for( idx = 0; idx < aipdata->neighbours; idx++ ) {
                tmpStr = aipStr;
                zkCStrSprintf( &aipStr, "%s[%5.5d] ", 
                               tmpStr, 
                               aipdata->neighbourId[idx]);
                zkCStrFree( &tmpStr );
            }
        }
        tmpStr = aipStr;
        zkCStrSprintf( &aipStr, "%s\n", 
                       tmpStr);
        zkCStrFree( &tmpStr );
    }
    
    *str = aipStr;

    return 1;
}









/**
 * Prints the data to standard error.
 */
static int datumSprintf( ZkError err, 
                         ProgConf conf, 
                         ZkNetInfoDatum data,
                         char **str )
{
    const char *func = "print_datum";
    AipData aipdata;
    struct sockaddr_in addr;
    size_t idx;
    char *datumStr = NULL;
    
    if (!data || !conf || !str) {
        zkErrorSet(err, func, ZKERR, ZKMFRNETINFO, ZKENULLPTR,"NULL param(s)");
        return 0;
    }
    
    if (!zkNetInfoDatumGetId(err, data, &aipdata.id)) {
        return 0;
    }
    if (!zkNetInfoDatumGetEntity(err, data, &aipdata.entid)) {
        return 0;
    }
    if (!zkNetInfoDatumGetSockaddr(err, data, &addr)) {
        return 0;
    }
    if (!zkNetInfoDatumGetStatus(err, data, &aipdata.status)) {
        return 0;
    }
    if (!zkNetInfoDatumGetRating(err, data, &aipdata.rating)) {
        return 0;
    }
    if (!zkNetInfoDatumGetRegionCode(err, data, &aipdata.region_code)) {
        return 0;
    }
    if (!zkNetInfoDatumGetRegion(err, data, &aipdata.region)) {
        return 0;
    }
    if (!zkNetInfoDatumGetOperator(err, data, &aipdata.operator)) {
        return 0;
    }

    if( aipdata.entid.type == FRENT_TYPE_AIP ) {
        if (!zkNetInfoDatumGetDifHelPort( err, data, &aipdata.dhport)) {
            return 0;
        }
        if (!zkNetInfoDatumGetNumNb(err, data, &aipdata.neighbours)) {
            return 0;
        }
        for( idx = 0; idx < aipdata.neighbours; idx++ ) {
            if( !zkNetInfoDatumGetNbX(err, data, idx, 
                                      &aipdata.neighbourId[idx])) {
                return 0;
            }
        }
    }

    if( !aipDataSprintf( err, conf, &aipdata, &addr, &datumStr ) ) {
        return 0;
    }

    *str = datumStr;
    
  return 1;
}









/**
 * Enumerate the content of all entity type lists.
 */  
static int netInfoSprintf( ZkError err,
                           ProgConf conf,
                           ZkNetInfo ni,
                           char **str)
{
  const char *func = "enum_types";
  ZkNetInfoListIter iter = NULL;
  FrEntIdType i;
  size_t sz, j;
  ZkNetInfoDatum data = NULL;
  char * datumStr = NULL;
  char * netInfoStr = NULL;
  char * tmpStr = NULL;

  if (!ni || !conf) {
    zkErrorSet(err, func, ZKERR, ZKMNETINFO, ZKENULLPTR, "NULL ptr(s)");
    return 0;
  }

  for (i = 1; i < FRENT_TYPE_MAX - 1; i++) {
      if (!zkNetInfoGetListIterOf(err, ni, i, &iter)) {
          return 0;
      }
      
      if (iter) {
          if( conf->verbosity > 1 ) {
              (void) zkFprintf( err,conf->outputFile, "\n--> Type %s:\n", 
                                frEntTypeGetStr(i));
          }
          
          if( !zkNetInfoListIterMove(err, iter, LIST_HEAD)) {
              return 0;
          }
          
          if (!zkNetInfoListIterSize(err, iter, &sz)) {
              return 0;
          }
          
          for (j = 0; j < sz; j++) {
              if (!zkNetInfoListIterGet(err, iter, (void *) &data)) {
                  return 0;
              }
              
              datumSprintf(err, conf, data, &datumStr);
              if( conf->verbosity > 0 ) {
                  (void) zkFprintf( err, conf->outputFile, datumStr );
              }
              tmpStr = netInfoStr;
              if( netInfoStr == NULL ) {
                  zkCStrSprintf( &netInfoStr, "%s", datumStr );
              } else {
                  zkCStrSprintf( &netInfoStr, "%s%s", tmpStr, datumStr );
              }
              zkCStrFree( &datumStr );
              zkCStrFree( &tmpStr );

              
              if (!zkNetInfoListIterMove(err, iter, LIST_FORWARDS)) {
                  return 0;
              }
          }
      } else {
          if( conf->verbosity > 1 ) {
              (void) zkFprintf( err, conf->outputFile, 
                                "\n--> Type %s is empty\n", 
                                frEntTypeGetStr(i));
          }
      }
      
      if( !zkNetInfoListIterFree( err, &iter ) ) {
          return 0;
      }
  }

  *str = netInfoStr;;
  
  return 1;
}







/*
 * Prints the test data in a string buffer
 */
static int testDataSprintf( ZkError err, ProgConf conf, char **str )
{
    const char * fn = "testDataSprintf";
    size_t idx = 0;
    char *aipStr = NULL;
    char *serverStr = NULL;
    char *tmpStr = NULL;
    char *testStr = NULL;
    struct sockaddr_in addr;

    if( !conf || !str ) {
        zkErrorSet(err, fn, ZKERR, ZKMNETINFO, ZKENULLPTR, "NULL param");
        return 0;
    }

    for( idx = 0; idx < MAX_AIPS; idx++ ){
        addr.sin_family = AF_INET;
        addr.sin_port = htons(aip[idx].port);
        addr.sin_addr.s_addr = inet_addr(aip[idx].host);
        aipDataSprintf( err, conf, &aip[idx], &addr, &aipStr );
        tmpStr = testStr;
        if( testStr == NULL ) {
            zkCStrSprintf( &testStr, "%s", aipStr );
        } else {
            zkCStrSprintf( &testStr, "%s%s", tmpStr, aipStr );
        }
        zkCStrFree( &tmpStr );
        zkCStrFree( &aipStr );
    }
    for( idx = 0; idx < MAX_SERVERS; idx++ ) {
        addr.sin_family = AF_INET;
        addr.sin_port = htons(server[idx].port);
        addr.sin_addr.s_addr = inet_addr(server[idx].host);
        aipDataSprintf( err, conf, &server[idx], &addr, &serverStr );
        tmpStr = testStr;
        if( testStr == NULL ) {
            zkCStrSprintf( &testStr, "%s", serverStr );
        } else {
            zkCStrSprintf( &testStr, "%s%s", tmpStr, serverStr );
        }
        zkCStrFree( &tmpStr );
        zkCStrFree( &serverStr );
    }
    
    *str = testStr;

    return 1;
}







/*
 *
 */
int main(int argc, char **argv)
{
    ZkError err = NULL;
    ZkLog log = NULL;
    ZkNetInfo ni = NULL;
    int exit_code = FAIL;
    ProgConf conf = NULL;
    char * netInfoStr = NULL;
    char * testDataStr = NULL;

    /* Initialize Test Harness */
    fprintf( stdout,"1...%d\n", NUMTEST );
    /* Initialize Error reporting */
    if ( !zkErrorNew(&err) ) {
        fprintf(stderr, "%s: error setting log object -- exiting.\n", argv[0]);
        goto done;
    }

    /*
     * Initialize program configuration
     */
    if( !zkMemAlloc( err, (void**)&conf, sizeof(struct T_PROG_CONF)) ) {
        zkFprintf( err, stderr, 
                "%s: error allocating config structure -- exiting.\n",
                argv[0] );
        goto done;
    }
    memset( conf, 0, sizeof(struct T_PROG_CONF) );

    /*
     * Set the default values in case options are not specified
     */
    conf->progName = argv[0];
    conf->outputFile = stderr;
    conf->verbosity = 2;

    {
        int opt = 1;   /* Scan index for command line options in argv array */
        int fail = 0;  /* Set to 1 if an error is detected during 
                        * scan of cmd line 
                        */
        
        while( opt < argc ) {
            if(!strcasecmp(argv[opt],"--"P_DEBUG)){
                conf->debug = 1;
            } else if(!strcasecmp(argv[opt],"--"P_HELP)) {
                usage( err, conf );
            } else if(!strcasecmp(argv[opt],"--"P_LOGFILE)){
                if(++opt < argc) {
                    conf->outputFile = fopen( argv[opt], "w" );
                    if(conf->outputFile == NULL ) {
                        zkFprintf( err, stderr, "Unable to open file %s\n",
                                 argv[opt]);                        
                        fail = 1;
                    }                        
                } else {
                    zkFprintf( err, stderr, mesg_1,conf->progName,argv[opt-1]);
                    fail = 1;
                }
            } else if(!strcasecmp(argv[opt],"--"P_VERBOSE)) {
                if( ++opt < argc ) {
                    conf->verbosity = atoi(argv[opt]);
                } else {
                    zkFprintf( err, stderr, mesg_1,conf->progName,argv[opt-1]);
                    fail = 1;
                }
            } else if(!strcasecmp(argv[opt],"--"P_DEBUGFILE)) {
                if( ++opt < argc ) {
                    conf->debugFile = argv[opt];
                } else {
                    zkFprintf( err, stderr, mesg_1,conf->progName,argv[opt-1]);
                    fail = 1;
                }
            } else {
                zkFprintf(err, stderr,mesg_2,conf->progName,argv[opt]);
                fail = 1;
            }
            opt++;
        }
        if (fail) {    
            zkFprintf(err, stderr,mesg_3,conf->progName);
            zkFprintf(err, stderr,"\n");
            exit(UNINITIATED);
        }
    }
    
    if( conf->debug ) {
        if ( ! (zkErrorLog(err, &log) &&
                zkLogSetProgname(log, conf->progName) &&
                zkLogSetFile(log, "/dev/tty") &&
                zkLogEnable(log))) {
            (void) zkFprintf( err, stderr, 
                              "%s: error setting log object -- exiting.\n", 
                              conf->progName);
            exit_code = UNINITIATED;
            goto done;
        }
    }

    /* Create a NetInfo Object */
    if( conf->verbosity > 1 ) zkFprintf( err, conf->outputFile, "zkNetInfoNew\n");
    if (!zkNetInfoNew(err, &ni)) {
        fprintf( stdout,"not ok 1\n");
        goto done;
    }
    zkFprintf( err, stdout, "ok 1\n" );

    /* Insert data in object */
    if( conf->verbosity > 1 ) zkFprintf( err, conf->outputFile, "insert_aips\n");
    if (!insert_aips(err, ni)) {
        zkFprintf( err, stdout,"not ok 2\n");
        goto done;
    }
    zkFprintf( err, stdout, "ok 2\n" );
    
    /* Add the servers */
    if( conf->verbosity > 1 ) zkFprintf( err, conf->outputFile, "insert_servers\n");
    if (!insert_servers(err, ni)) {
        zkFprintf( err, stdout,"not ok 3\n");
        goto done;
    }
    zkFprintf( err, stdout, "ok 3\n" );
    
    /* Insert a second time to check for re-insertion behaviour */
    if (!insert_aips(err, ni)) {
        zkFprintf( err, stdout,"not ok 4\n");
        goto done;
    }
    zkFprintf( err, stdout, "ok 4\n" );

    /* Print the content of the data */
    if( !testDataSprintf( err, conf, &testDataStr ) ) {
        zkFprintf( err, stdout, "not ok 5\n" );
        goto done;
    }
    if( conf->debugFile ) {
        zkFileSave( err, 
                    conf->debugFile,
                    testDataStr, 
                    strlen(testDataStr), 
                    O_CREAT|O_TRUNC,
                    S_IRUSR|S_IWUSR);
    }

    /* Print the current content of each list */
    if( conf->verbosity > 1 ) zkFprintf( err, conf->outputFile, "netInfoSprintf\n");
    if (!netInfoSprintf(err, conf, ni, &netInfoStr)) {
        zkFprintf( err, stdout, "not ok 6\n");
        goto done;
    }
    zkFprintf( err, stdout, "ok 6\n" );

    /* Compare both outputs */
    if( strcmp( testDataStr, netInfoStr ) ) {
        zkFprintf( err, stdout, "not ok 7\n");
        zkCStrFree( &testDataStr );
        zkCStrFree( &netInfoStr );
        goto done;
    }
    zkFprintf( err, stdout, "ok 7\n" );

    zkCStrFree( &testDataStr );
    zkCStrFree( &netInfoStr );
    
    exit_code = PASS;
    
 done:
    if( conf->verbosity > 1 ) zkFprintf(err, conf->outputFile, "Done\n");
    zkFprintf( err, stdout, "END\n" );

    if( conf->outputFile ) fclose( conf->outputFile );
    if (ni) {
        (void) zkNetInfoFree(err, &ni);
    }
    
    if (err) {
        (void) zkErrorFree(&err);
    }
    exit(exit_code);
}
