
#ifndef LIB_ZKNETMGMT_H
#define LIB_ZKNETMGMT_H


#include "zkserr.h"


#ifdef __cplusplus

#include <map>
#include <set>
#include <string>

#include <iostream>
#include <sstream>

/*
** Error hadling and ModuleId section 
*/
# define ZKS_NETMGMT_MOD                "ZkNetMgmt"

# define ZKS_NETMGMT_KEYWORD_ERR       ((uint16_t) 0x0001)
# define ZKS_NETMGMT_FILE_IO_ERR       ((uint16_t) 0x0002)


/*
** table of valid keyword strings
*/
const int TABLE_MAX = 2;
const std::string kwTable[TABLE_MAX] = {
    "key1",
    "key2"};


/*
**------------------------------------------------------------------------------
** ZkNetMgmt class
**
**------------------------------------------------------------------------------
*/
class ZkNetMgmt
{
    typedef std::set<std::string>   KeywordSet;
    typedef std::map<std::string, std::string>  KeywordMap;
  
 private:

    // retrieving a value from kw_map
    bool getValue(const std::string & keyword, std::string & value) const { 

        KeywordMap::const_iterator it = m_kwMap.find(keyword);

        if( it == m_kwMap.end() ) {
            return false;
        }

        value = (*it).second;
        return true;
    }

 public:

    ZkNetMgmt();
    ~ZkNetMgmt() {};
    
    //
    // initialisation of the object
    //   - specify the name of the export file
    //   - specify the file descriptor to which write the notify message
    //
    void Init(const std::string filename /* , FIXME: add notify param here */ );

    // used to submit information to export
    void AddExportInfo(const std::string key, const std::string value);
    void AddExportInfo(const std::string key, const double      value);
    void AddExportInfo(const std::string key, const int         value);

    // request to export
    void Export();

    // request to notify
    void Notify(const std::string notifyString);


 private:
    KeywordSet m_kwSet;
    KeywordMap m_kwMap;

    std::string m_exportFilename;

};


extern "C" {
#endif // __cplusplus

/*
**------------------------------------------------------------------------------
** C funcitons to access ZkNetMgmt object
**
**------------------------------------------------------------------------------
*/

typedef struct T_zkNetMgmtObj *ZkNetMgmtObj;


DLLExport int
zkNetMgmtNew(ZksErr err, ZkNetMgmtObj *obj);

DLLExport int
zkNetMgmtFree(ZksErr err, ZkNetMgmtObj obj);


DLLExport int
zkNetMgmtAddExportInfoStr(ZksErr err, ZkNetMgmtObj obj, char *keyStr, char *infoStr);

DLLExport int
zkNetMgmtAddExportInfoDouble(ZksErr err, ZkNetMgmtObj obj, char *keyStr, double infoDouble);

DLLExport int
zkNetMgmtAddExportInfoInt(ZksErr err, ZkNetMgmtObj obj, char *keyStr, int infoInt);


DLLExport int
zkNetMgmtExport(ZksErr err, ZkNetMgmtObj obj);

DLLExport int
zkNetMgmtNotify(ZksErr err, ZkNetMgmtObj obj,  char *notifyStr);

#ifdef __cplusplus
    }
#endif

#endif // LIB_ZKNETMGMT_H
