#include "zkNetMgmt.h"

#include "zks.h"
#include "zkserr.h"
#include "zksexception.h"

#include <map>
#include <set>
#include <string>

#include <iostream>
#include <sstream>

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


// ******
// ******  Not sure if will use the following classes
// ******

//
//
//
class KeywordSet {

    typedef std::set<std::string> kw_set;

    static kw_set m_set;

public:

    KeywordSet() {};
    ~KeywordSet() {};

    bool operator[](const std::string & keyword) { 
        return ( m_set.find(keyword) != m_set.end() ) ? true : false;
    }
    
    void add(const std::string & keyword) {
        m_set.insert(keyword);
        
    }
};

//
//
//
class KeywordMap {

    typedef std::map<std::string, std::string> kw_map;

protected:

    kw_map m_map;

public:

    KeywordMap() {};
  
    bool getValue(const std::string & keyword, std::string & value) const { 
        kw_map::const_iterator it = m_map.find(keyword);
        
        if( it == m_map.end() ) return false;
        value = (*it).second;
        return true;
    }

    void setValue(const std::string & keyword, const std::string & value) {
        m_map[keyword] = value;
    }

};

// ******
// ******  Not sure if will use the preceeding classes
// ******

//
//==============================================================================
//==============================================================================
//==============================================================================
//


//
// ZkNetMgmt constructor
//
// - builds the set of valid keywords
//
ZkNetMgmt::ZkNetMgmt() {

    // initializing the set with valid keywords
    int i = 0;
    for(i=0; i < TABLE_MAX ;i++){
        // m_kwSet.add(kwTable[i]);
        m_kwSet.insert(kwTable[i]);
    }
}


//
//
//
void
ZkNetMgmt::Init(const std::string filename /* , FIXME: add notify param here */ ) {

    m_exportFilename = filename;
}


//
//
//
void
ZkNetMgmt::AddExportInfo(const std::string key, const std::string value) {

    DECL_FUNC("ZkNetMgmt::AddExportInfo");

    try {

        // keyword validation
        CZKS::ThrowIf(m_kwSet.find(key) == m_kwSet.end(),
                      ZKS_NETMGMT_MOD,
                      ZKS_NETMGMT_KEYWORD_ERR,
                      func,
                      "Invalid keyword");

        m_kwMap[key] = value;

    }
    catch ( ... ) {
        CZKS::Rethrow(func, "failure");
    }
}

//
//
//
void
ZkNetMgmt::AddExportInfo(const std::string key, const int value) {

    DECL_FUNC("ZkNetMgmt::AddExportInfo");

    std::ostringstream  oString;

    try {

        // keyword validation
        CZKS::ThrowIf(m_kwSet.find(key) == m_kwSet.end(),
                      ZKS_NETMGMT_MOD,
                      ZKS_NETMGMT_KEYWORD_ERR,
                      func,
                      "Invalid keyword");

        oString << value;
        m_kwMap[key] = oString.str();

    }
    catch ( ... ) {
        CZKS::Rethrow(func, "failure");
    }
}

//
//
//
void
ZkNetMgmt::AddExportInfo(const std::string key, const double value) {

    DECL_FUNC("ZkNetMgmt::AddExportInfo");

    std::ostringstream  oString;
    try {

        // keyword validation
        CZKS::ThrowIf(m_kwSet.find(key) == m_kwSet.end(),
                      ZKS_NETMGMT_MOD,
                      ZKS_NETMGMT_KEYWORD_ERR,
                      func,
                      "Invalid keyword");

        oString << value;
        m_kwMap[key] = oString.str();

    }
    catch ( ... ) {
        CZKS::Rethrow(func, "failure");
    }

}

//
//
//
void
ZkNetMgmt::Export() {

    DECL_FUNC("ZkNetMgmt::Export");

    int fd = -1;
    std::string tempFile;

    KeywordMap::const_iterator it;
    int length;

    try {

        // 
        // create a unique filename and open a file to that name
        //
        std::ostringstream osTemplate;
        osTemplate << m_exportFilename << ".XXXXXX";
        tempFile = osTemplate.str();

	CZKS::ThrowIf(-1 == (fd = mkstemp(const_cast<char*>(tempFile.c_str()))), 
		      ZKS_NETMGMT_MOD,
		      ZKS_NETMGMT_FILE_IO_ERR,
		      func,
		      "call to mkstemp() failed)");

        // FIXME: must add a time stamp at the beginning of the file

        //
        // write map content to temp file
        //
        it = m_kwMap.begin();
        while(it != m_kwMap.end()) {

            std::ostringstream outString;

	    // build the "key = value" string
	    outString << (*it).first << " = " << (*it).second << "\n";

	    CZKS::ThrowIf(-1 == (length = write(fd, (outString.str()).c_str(), strlen((outString.str()).c_str()))), 
			  ZKS_NETMGMT_MOD,
			  ZKS_NETMGMT_FILE_IO_ERR,
			  func,
			  "call to write() failed");

            it++;

	}

	CZKS::ThrowIf(-1 == close(fd), 
		      ZKS_NETMGMT_MOD,
		      ZKS_NETMGMT_FILE_IO_ERR,
		      func,
		      "call to close() failed");
	
        // rename it to final name
        CZKS::ThrowIf(-1 == rename(tempFile.c_str(), m_exportFilename.c_str()), 
                      ZKS_NETMGMT_MOD,
                      ZKS_NETMGMT_FILE_IO_ERR,
                      func,
                      "call to rename() failed");
    }
    catch ( ... ) {
        CZKS::Rethrow(func, "failure");
    }
}

