/*
 * $Id: zkNiqs.h,v 1.2 2001/04/19 20:32:42 alainc Exp $
 */

#ifndef ZKNIQS_H_
#define ZKNIQS_H_

#ifndef WIN32
#include <sys/socket.h>
#else
#include <winsock2.h>
#endif

#include "zks.h"
#include "zkError.h"
#include "zkipClient.h"

#include "frEnt.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
typedef struct T_ZkNiqsCDV * ZkNiqsCDV;

typedef enum {
    ZKNIQ_ATTR_NULL        = 0x0,
    ZKNIQ_ATTR_TOPOLOGY    = 0x1,
    ZKNIQ_ATTR_DESCRIPTION = 0x2,
    ZKNIQ_ATTR_LIST        = 0x3,
    ZKNIQ_ATTR_STATUS      = 0x4,
    ZKNIQ_ATTR_UNKNOWN     = 0x5
} ZkNiqAttr;

DLLExport extern
int zkNiqsCDVNew( ZK_INOUT ZkError   err,
                  ZK_OUT   ZkNiqsCDV * newCDV );
DLLExport extern 
int zkNiqsCDVFree(
		  ZK_INOUT ZkError err,
		  ZK_OUT   ZkNiqsCDV * oldCDV );
DLLExport extern
int zkNiqsCDVNewCopy( ZK_INOUT ZkError         err,
		      ZK_IN    const ZkNiqsCDV   orgCDV,
		      ZK_OUT ZkNiqsCDV         * cpyCDV );
DLLExport extern 
int zkNiqsCDVCopy( ZK_INOUT ZkError err,
		   ZK_IN    const ZkNiqsCDV orgCDV,
		   ZK_OUT   ZkNiqsCDV       cpyCDV );
DLLExport extern 
int zkNiqsCDVIsEqual( ZK_INOUT ZkError         err,
		      ZK_IN    const ZkNiqsCDV   lCDV,
		      ZK_IN    const ZkNiqsCDV   rCDV,
		      ZK_OUT   int           * cmp );
DLLExport extern
int zkNiqsCDVImportFromBuffer( ZK_INOUT ZkError err,
			       ZK_INOUT ZkNiqsCDV cdv,
			       ZK_IN    const void     * inBuf,
			       ZK_IN    const uint32_t   inBufSz,
			       ZK_OUT   uint32_t       * usedInBufSz );
DLLExport extern
int zkNiqsCDVSizeOfExportBuffer( ZK_INOUT ZkError           err, 
			     ZK_IN    const ZkNiqsCDV   cdv,
			     ZK_OUT   uint32_t        * size );
DLLExport extern
int zkNiqsCDVExportToBuffer( ZK_INOUT ZkError           err, 
			     ZK_IN    const ZkNiqsCDV   cdv,
			     ZK_OUT   void            * outBuf,
			     ZK_IN    uint32_t          outBufSz,
			     ZK_OUT   uint32_t        * usedOutBufSz );

DLLExport extern int
zkNiqsApiPostConnect( ZK_INOUT ZkError                     err,
			 ZK_INOUT ZkipClient                  client,
			 ZK_INOUT const struct ZkipCommandResponse * response,
			 ZK_INOUT const char                * extra_data
		      );
DLLExport extern int
zkNiqsApiPreDisconnect( ZK_INOUT ZkError     err,
			   ZK_IN    ZkipClient  client
			);
DLLExport extern int
zkNiqsApiGetClientInfo ( ZK_INOUT ZkError            err,
			    ZK_INOUT ZkipClient         client,
			    ZK_IN    const      char *  clientid,
			    ZK_OUT   void            ** data,
			 ZK_OUT   size_t          *  sz );
DLLExport extern int
zkNiqsApiGetCacheDataVersion( ZK_INOUT ZkError       err,
				 ZK_INOUT ZkipClient    client,
				 ZK_OUT   void       ** data,
				 ZK_OUT   size_t     *  sz 
			      );
DLLExport extern int
zkNiqsApiGetTokenInfo( ZK_INOUT ZkError       err,
			  ZK_INOUT ZkipClient    client,
			  ZK_OUT   void       ** data,
			  ZK_OUT   size_t     *  sz
		       );
DLLExport extern
int zkNiqsApiQuery( ZK_INOUT ZkError err,/* Returns the error if one occured */
                    ZK_INOUT ZkipClient client, /* The zkip client structure 
                                                   connected to the NIQS */
                    ZK_IN    ZkNiqAttr attr,   /* The requested attribute */
                    ZK_IN    FrEntIdType *  type,   /* (array) Type of entity 
                                                  for which we want to query */
                    ZK_IN    size_t   types,  /* the number of entity 
                                                 in the array */
                    ZK_OUT   void     ** data,   /* Returns a buffer 
                                                    containing the list */
                    ZK_OUT   size_t    * sz      /* size of the buffer */
                    );

DLLExport extern
int zkNiqsAttributeToToken( ZK_INOUT ZkError err,
                            ZK_IN    ZkNiqAttr attr,
                            ZK_OUT   const char ** token 
                            );

DLLExport extern
int zkNiqsUncompress( ZK_INOUT ZkError err, 
                      ZK_IN    const char *compressed_buf,
		      ZK_OUT   char **raw_buf, 
                      ZK_OUT   size_t *raw_sz
                      );
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKNIQS_H_ */
