/*
 * $Id: zkNiqsApi.c,v 1.2 2001/04/19 20:32:46 alainc Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>

#include "zks.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkAlloc.h"
#include "zkipClient.h"
#include "zkNiqs.h"
#include "frEnt.h"
#include "zkCStr.h"
#include "ZKIP.h"

    #define CMD_SZ 512

/* Forward declaration of module local functions */
static int
zkNiqsApiSendCommand(
			ZK_INOUT ZkError       err,
			ZK_INOUT ZkipClient    client,
			ZK_IN    const char *  cmd,
			ZK_OUT   void       ** data,
			ZK_OUT   size_t     *  sz );






DLLExport int
zkNiqsApiPostConnect( ZK_INOUT ZkError                     err,
			 ZK_INOUT ZkipClient                  client,
			 ZK_INOUT const struct ZkipCommandResponse * response,
			 ZK_INOUT const char                * extra_data
			 )
{
    static const char * fn = "zkNiqsApiPostConnectValidation";

    if( !client || !response ) {
	zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
	return 0;
    }

    if( !zkipClientIsConnected(err, client) ) {
	zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKEBADSTATE, "Not connected to NIQS");
	return 0;
    }

    /* Actually, there is nothing to validate in the welcome banner */
    /* and there is no connection state to exchange with the server */
    /* at this time */
    client = client;
    response = response;
    extra_data = extra_data;

    return 1;
}

DLLExport int
zkNiqsApiPreDisconnect( ZK_INOUT ZkError     err,
                        ZK_IN    ZkipClient  client
                        )
{
    static const char * fn = "zkNiqsPreDisconnect";
    static char quit_cmd[] = "0 0.1.0 quit nodata binary 0\015\012";
    struct ZkipCommandResponse * response = NULL;
    char * extra_data = NULL;

    if( !client ) {
	zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
	return 0;
    }

    if( !zkipClientIsConnected( err, client ) ){
	zkErrorSet(err,fn,ZKWARN,ZKMNIQS,ZKEBADSTATE,"Not connected to NIQS");
	return 0;
    }

    if( !zkipClientWrite(err,client,quit_cmd,strlen(quit_cmd)) ) {
	return 0; /* zkError must have been set by zkipClientWrite */
    }
		
    if( !zkipClientReadResponse(err,client,&response,&extra_data) ) {
	return 0; /* zkError must have been set by zkipClientReadResponse */
    }

    return 1;
}


static int
zkNiqsApiSendCommand(
			ZK_INOUT ZkError       err,
			ZK_INOUT ZkipClient    client,
			ZK_IN    const char *  cmd,
			ZK_OUT   void       ** data,
			ZK_OUT   size_t     *  sz )
{
  static const char * fn = "zkNiqsApiSendCommand";
  struct ZkipCommandResponse * response = NULL;
  char * extra_data = NULL;


  if (!client || !cmd || !data || !sz ) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
      return 0;      
  }

  if( !zkipClientIsConnected(err,client) ) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKEBADSTATE,
		 "ZkipClient is not connected");
      return 0;
  }

  if( !zkipClientWrite(err,client,cmd,strlen(cmd)) ) {
      return 0;/* ZkError should have been set by zkipClientWrite */
  }

  if( !zkipClientReadResponse(err,client,&response,&extra_data) ) {
      return 0;/* ZkError should have been set by zkipClientReadResponse */
  }

  if( !response ) {
      zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEPROTO,
		    "Query returned no response cmd=>[%s]",cmd); 
      return 0;
  }

  if (response->response != _SERVICE_READY) {
      zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEPROTO,
		    "Query failed:  [%d] [%s] cmd=>[%s]",
		    response->response,response->message,cmd); 
  }

  *sz = (size_t) response->datalen;

  if ((extra_data) && (*sz)) {

      if( !zkMemAlloc( err, data, (*sz) + 1) ) {
	  *data = NULL;
	  *sz = 0;
	  return 0; /* ZkError must have been set by zkMemAlloc */
      }
      memcpy(*data,extra_data,*sz);
      ((char *)*data)[*sz] = (char) 0;
  }

  /* 
   * NOTE: response and extra_data are contained in the ZkipClient structure
   *       and do not need to be freed at this point.
   *       ZkipClient takes care of this.
   */
  
  return 1;
}

DLLExport int
zkNiqsApiGetClientInfo ( ZK_INOUT ZkError            err,
                         ZK_INOUT ZkipClient         client,
                         ZK_IN    const      char *  clientid,
                         ZK_OUT   void            ** data,
                         ZK_OUT   size_t          *  sz ) 
{
  static const char * fn = "zkNiqsApiGetClientInfo";
  char cmd[CMD_SZ] = "";


  if (!client || !clientid || !data || !sz) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
      return 0;
  }

  if (snprintf(cmd,CMD_SZ,"1 7.1.0 clientinfo nodata binary 0 %s\015\012",
                 clientid) < 0) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKEPARSE,
                 "Unable to formulate 7.1.0 clientinfo command");
      return 0;
  } 
  
  if( !zkNiqsApiSendCommand(err,client,cmd,data,sz) ) {
      return 0; /* zkError must have been set by zkNiqsApiSendCommand */
  }

  return 1;
}

DLLExport int
zkNiqsApiGetCacheDataVersion( ZK_INOUT ZkError       err,
				 ZK_INOUT ZkipClient    client,
				 ZK_OUT   void       ** data,
				 ZK_OUT   size_t     *  sz 
				 )
{
  static const char * fn = "zkNiqsApiGetCacheDataVersion";
  char cmd[CMD_SZ] = "";

  if ( !client || !data || !sz) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }

  if(snprintf(cmd,CMD_SZ,"1 7.1.0 cacheversion nodata binary 0\015\012") < 0) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKERROR,
                 "Unable to formulate 7.1.0 cacheversion command");
      return 0;
  } 
  
  if ( !zkNiqsApiSendCommand(err,client,cmd,data,sz) ) {
      return 0; /* zkError must have been set by zkNiqsApiSendCommand */
  }

  return 1;
}

DLLExport int
zkNiqsApiGetTokenInfo( ZK_INOUT ZkError       err,
			  ZK_INOUT ZkipClient    client,
			  ZK_OUT   void       ** data,
			  ZK_OUT   size_t     *  sz
			  )
{
  static const char * fn = "zkNiqsApiGetTokenInfo";
  char cmd[CMD_SZ];
  

  if (!client || !data || !sz) {
    zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
    return 0;;
  }

  if (snprintf(cmd,CMD_SZ,"1 7.1.0 tokeninfo nodata binary 0\015\012") < 0) {
      zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKERROR,
                 "Unable to formulate 7.1.0 clientinfo command");
      return 0;
  }

  if( !zkNiqsApiSendCommand(err,client,cmd,data,sz) ) {
      return 0; /* ZkError must have been set by zkNiqsApiSendCommand */
  }

  return 1;
}

DLLExport int
zkNiqsApiQuery( ZK_INOUT ZkError err,    /* Returns the error if one occured */
                ZK_INOUT ZkipClient client, /* The zkip client structure 
                                                       connected to the NIQS */
                ZK_IN    ZkNiqAttr attr,   /* The attribute of interest */
                ZK_IN    FrEntIdType *  type,   /* (array) Type of entity for 
                                                      which we want to query */
                ZK_IN    size_t   types,/* the number of entity in the array */
                ZK_OUT   void     ** data,   /* Returns a buffer containing 
                                                                    the list */
                ZK_OUT   size_t    * sz      /* size of the output buffer */
                )
{

    static const char * fn = "zkNiqsApiQuery";
    const char * attribute = NULL;

    int bytes = 0;  /* Number of bytes written to the cmd buffer */
    int base = 0;  /* Next write position in the cmd buffer */
    size_t n = 0; /* Loop index */
    const char * ent_str = "";  /* Holds the entity type ID string token */
    const char * encoding_str = "BZIP2";

    char cmd[CMD_SZ + 2] = "";
 
    if ( ! client || ! attr || ! type || ! data || ! sz) {
        zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
	return 0;
    }


    *data = NULL;
    *sz = 0;
    if( !zkNiqsAttributeToToken(err,attr,&attribute)) {
	zkErrorSet_va(err,fn,ZKERR,ZKMNIQS,ZKEPARMBAD,
		      "Requested attribute %d is invalid", (int) attr);
	return 0;
    }

    if( (bytes = snprintf(cmd,CMD_SZ,"0 7.1.0 mxquery nodata binary 0 %s %d %s", attribute, /* since */0, encoding_str)) < 0) {
	zkErrorSet(err,fn,ZKERR,ZKMNIQS, ZKERROR, 
		   "Unable to formulate 7.1.0 mxquery command");
	return 0;
    }
    
    base = bytes;
            
    for (n = 0; n < types; n++) {
	/*
	 * Get the entity type string token
	 */
	
	if ((ent_str = frEntTypeGetStr(type[n])) == NULL) {
	    zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKEPARMBAD,
		       "Unable to identify requested entity type");
	    return 0;
	}
                    
	/*
	 * And add it as a parameter in the list command buffer
	 */
                    
	if ((bytes = snprintf(&(cmd[base]),CMD_SZ - base," %s",ent_str)) < 0) {
	    zkErrorSet(err,fn,ZKERR,ZKMNIQS,ZKERROR,
		       "Unable to formulate ZKIP 7.1.0 mxquery command");
	    return 0;
	} 
	
	base += bytes;

        zkCStrlcat(cmd,"\015\012", CMD_SZ);        
    }
        
    /*
     * Only send the command if we succeeded in creating the cmd buffer
     */
    if ( !zkNiqsApiSendCommand(err,client,cmd,data,sz)) {
	return 0; /* ZkError must have been set by zkNiqsApiSendCommand */
    }
            
    return 1;
}
