#ifndef LIB_ZKTKSRV_API_H
# define LIB_ZKTKSRV_API_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# include "zktkconst.h"

# include "zks.h" 
# include "zkIntTypes.h"
# include "zkserr.h"


/*
** Module name
*/

# define ZKS_ZKTKSRV_API_MOD                             "ZkTkSrv_API"


/*
** API errors
*/

# define ZKS_ZKTKSRV_API_BAD_SERIAL_ERR                         0x0001
# define ZKS_ZKTKSRV_API_BAD_PASSWORD_ERR                       0x0002
# define ZKS_ZKTKSRV_API_BAD_HOST_ERR                           0x0003
# define ZKS_ZKTKSRV_API_CONNECT_ERR                            0x0004
# define ZKS_ZKTKSRV_API_SERIAL_NOT_ACTIVE_ERR                  0x0005
# define ZKS_ZKTKSRV_API_UNKNOWN_PACKET_ERR                     0x0006
# define ZKS_ZKTKSRV_API_REQUEST_FORMAT_ERR                     0x0007
# define ZKS_ZKTKSRV_API_PKT_COMMAND_ERR                        0x0008
# define ZKS_ZKTKSRV_API_PKT_FORMAT_ERR                         0x0009
# define ZKS_ZKTKSRV_API_PKT_MAC_ERR                            0x000a
# define ZKS_ZKTKSRV_API_PKT_VERSION_ERR                        0x000b
# define ZKS_ZKTKSRV_API_SERVER_BUSY_ERR                        0x000c
# define ZKS_ZKTKSRV_API_SERVER_STATE_ERR                       0x000d



# ifdef __cplusplus  
extern "C" {
# endif 


  /*
  ** Server's Connection Context
  */

  typedef struct S_ZkTkSrvCtx     * ZkTkSrv;


  /*
  ** Client API calls
  */

  ZkBool tkSrvClientConnect         (ZksErr       err, 
                                     ZkTkSrv    * srvP,
                                     const char * srv, 
                                     uint16_t     port,
                                     uint16_t     retry);

  ZkBool tkSrvClientDisconnect      (ZksErr       err,
                                     ZkTkSrv    * srvP);


  ZkBool tkSrvClientTokenRequest    (ZksErr       err, 
                                     ZkTkSrv      srv,
                                     char         serial[ZKS_ZKTKSRV_SERIAL_LENGTH+1],
                                     char         password[ZKS_ZKTKSRV_SERIAL_PASSWORD_LENGTH+1],
                                     const struct timeval * timeoutP);

  ZkBool tkSrvClientTokenResponse   (ZksErr       err, 
                                     ZkTkSrv      srv,
                                     uint16_t   * countP, 
                                     FrToken    * token[ZKS_ZKTKSRV_TOKEN_MAX],
                                     FrCryptSig   sig,
                                     const struct timeval * timeoutP);


  /*
  ** Admin API calls
  **
  ** The following calls are only expose through a Unix domain socket (local to the server)
  **
  */

#  if defined( ZK_LINUX )

  ZkBool tkSrvAdminConnect                 (ZksErr       err, 
                                            ZkTkSrv    * srvP,
                                            const char * socketfile);

  ZkBool tkSrvAdminDisconnect              (ZksErr       err,
                                            ZkTkSrv    * srvP);


  ZkBool tkSrvAdminServerStopRequest       (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);

  ZkBool tkSrvAdminServerStopResponse      (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);


  ZkBool tkSrvAdminServerHoldRequest       (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);

  ZkBool tkSrvAdminServerHoldResponse      (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);


  ZkBool tkSrvAdminServerReleaseRequest    (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);

  ZkBool tkSrvAdminServerReleaseResponse   (ZksErr       err, 
                                            ZkTkSrv      srv,
                                            const struct timeval * timeoutP);

#  endif /* ZK_LINUX */


# ifdef __cplusplus  
}
# endif 

#endif /* LIB_ZKTKSRV_API_H */
