/*
 *	zktkndo.h		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zktknbo.h,v 1.3 2001/04/23 20:26:04 alainc Exp $
 *	
 *	Network byte ordering function for buffers.
 *
 *  This file contain functions use to pack and unpack
 *  char, short, long and string's objects over memory
 *  buffers.
 */

#ifndef LIB_ZKTKSRV_NBO_H
# define LIB_ZKTKSRV_NBO_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus  

#  include <netinet/in.h>
#  include <string>


//
// Network to host byte ordering method
//
// : The following method return the unpacked
//   value.
// 
inline char unpackc(const char * buff, size_t offset)
{
  return *const_cast<char *>(&buff[offset]);
}


inline short unpacks(const char * buff, size_t offset)
{
  return ntohs(*reinterpret_cast<const short *>(&buff[offset]));
}


inline long unpackl(const char * buff, size_t offset)
{
  return ntohl(*reinterpret_cast<const long *>(&buff[offset]));
}


inline const char * unpack_buff(const char * buff, size_t offset)
{
  return &buff[offset];
}


inline std::string unpack_string(size_t dsz, const char * buff, size_t offset)
{
  return std::string(&buff[offset], dsz);
}



//
// Host to network byte ordering method
//
// : The following method return the unpacked
//   value.
//
inline size_t packc(const char & c, char * buff, size_t offset = 0)
{
  buff[offset] = c;
  return 1;
}


inline size_t packs(const short & c, char * buff, size_t offset = 0)
{
  short v = c;
  
  *reinterpret_cast<short *>(&buff[offset]) = htons(v);
  return 2;
}


inline size_t packl(const short & c, char * buff, size_t offset = 0)
{
  long v = c;
  
  *reinterpret_cast<long *>(&buff[offset]) = htonl(v);
  return 4;
}


inline size_t pack_buff(const char * data, size_t dsz, char * buff, size_t offset)
{
  ::memcpy(&buff[offset], data, dsz);
  return dsz;
}


inline size_t pack_string(const std::string data, char * buff, size_t offset)
{
  ::memcpy(&buff[offset], data.c_str(), data.size());
  return data.size();
}


# endif /*__cplusplus */

#endif /* LIB_ZKTKSRV_NBO_H */
