#ifndef LIB_ZKTKSRV_PROTOCOL_H
# define LIB_ZKTKSRV_PROTOCOL_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus

#  include "zktkconst.h"

#  include "zks.h"
#  include "zkIntTypes.h"
#  include "zkserr.h"

#  include <string>


//
// Call use to received a packet header
//
void tkRcvPacketHeader              (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     uint16_t             & cmd,
                                     uint32_t             & paylen);



//
// Call use to received an answer packet payload
//
void tkRcvAnswerPacketPayload       (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     uint32_t             & errcode);



//
// Call use to received an identification packet payload
//
void tkRcvIdentifyPacketPayload     (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     std::string          & id,
                                     uint16_t             & isz);


//
// Call use to received a redirection packet payload
//
void tkRcvRedirectPacketPayload     (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     uint32_t             & ip,
                                     uint16_t             & port);


//
// Call use to received a server stop packet payload
//
void tkRcvServerStopPacketPayload   (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to received a server hold packet payload
//
void tkRcvServerHoldPacketPayload   (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to received a server release packet payload
//
void tkRcvServerReleasePacketPayload(int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to received a serial number add packet payload
//
void tkRcvSerialAddPacketPayload    (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     std::string          & serial);


//
// Call use to received a serial number reniew packet payload
//
void tkRcvSerialReniewPacketPayload (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     std::string          & serial);


//
// Call use to received a token request packet payload
//
void tkRcvTokenRequestPacketPayload (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     std::string          & serial,
                                     std::string          & password);


//
// Call use to received a token response packet payload
//
void tkRcvTokenResponsePacketPayload(int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     uint16_t             & count,
                                     const char           * token[ZKS_ZKTKSRV_TOKEN_MAX]);


//
// Call use to send an answer packet
//
void tkSndAnswerPacket              (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const uint32_t       & error);



//
// Call use to send an identification packet
//
void tkSndIdentifyPacket            (int                    fd,
                                     const char           * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const std::string    & id,
                                     const uint16_t       & isz);

//
// Call use to send a redirection packet
//
void tkSndRedirectPacket            (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const uint32_t       & ip,
                                     const uint16_t       & port);


//
// Call use to send a server stop packet
//
void tkSndServerStopPacket          (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to send a server hold packet
//
void tkSndServerHoldPacket          (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to send a server release packet
//
void tkSndServerReleasePacket       (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP);


//
// Call use to send a serial number add packet
//
void tkSndSerialAddPacket           (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const std::string    & serial,
                                     const std::string    & cksum,
                                     const std::string    & capacity);


//
// Call use to send a serial number reniew packet
//
void tkSndSerialReniewPacket        (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const std::string    & serial);


//
// Call use to send a token request packet
//
void tkSndTokenRequestPacket        (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const std::string    & serial,
                                     const std::string    & password);


//
// Call use to send a token response packet
//
void tkSndTokenResponsePacket       (int                    fd,
                                     char                 * buffer,
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     const uint16_t       & count,
                                     const char           * token[ZKS_ZKTKSRV_TOKEN_MAX]);



# endif /* __cplusplus  */

#endif /* LIB_ZKTKSRV_PROTOCOL_H */
