#define REV_ID "@(#) $Id: admin.cpp,v 1.3 2001/04/23 22:38:58 alainc Exp $"
#include "zkrev.h"

#if defined( ZK_LINUX )

# include "zktksrv.h"
# include "srvctx.h"

# include "zkserr.h"
# include "zksexception.h"

# include <sys/types.h>
# include <sys/socket.h>
# include <netdb.h>
# include <netinet/in.h>
# include <sys/un.h>
# include <arpa/inet.h>
# include <unistd.h>

//
// Compatibility macros
//
# ifndef AF_LOCAL
#  define	AF_LOCAL AF_UNIX
# endif

# ifndef PF_LOCAL
#  define	PF_LOCAL PF_UNIX
# endif


//
// Establish a connection w/ the token server
//
extern "C" ZkBool tkSrvAdminConnect(ZksErr       err, 
                                    ZkTkSrv    * srvP,
                                    const char * socketfile)
{
  DECL_FUNC("tkSrvAdminConnect");

  struct sockaddr_un srvaddr;
  ZkBool             success = ZK_TRUE;
  ZkTkSrv            ctx     = NULL;
  int                fd      = -1;

  try { 
    try {
      if( !srvP ) {
        /* Validate inval */
        success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                            func, "server context reference is null");

      } else {

        if( !socketfile ) {
          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                              func, "local socket file isn't provided");

          /* Allocate context handler */
        } else if( !(ctx = (ZkTkSrv) calloc(1, sizeof(struct S_ZkTkSrvCtx))) ) {

          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_CALLOC_ERR,
                              func, "context allocation failure");

          /* Get the socket */
        } else if( (fd  = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1 ) {

          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_SOCKET_ERR,
                              func, "invalid address length");


          /* setup the address of the server */
        } else {

          srvaddr.sun_family = AF_LOCAL;
          strncpy(srvaddr.sun_path, socketfile, sizeof(srvaddr.sun_path));
          srvaddr.sun_path[sizeof(srvaddr.sun_path)-1] = 0;

          /* Try to establish the connection w/ the server */
          if( connect(fd, (struct sockaddr *)&srvaddr, (int)sizeof(struct sockaddr_un)) == -1 ) {

            success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_SOCKET_ERR,
                                func, "connect failed");
          }
        }

        if( !success ) {
          /* Cleanup if failure */
          if( ctx )       free(ctx);
          if( fd != -1 )  close(fd);

        } else {

          /* Success: fill output params */
          (*srvP)        = ctx;
          (*srvP)->fd    = fd;
          (*srvP)->local = ZK_FALSE;
        }
      }

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Disconnect of the token server
//
extern "C" ZkBool tkSrvAdminDisconnect(ZksErr       err, 
                                       ZkTkSrv    * srvP)
{
  DECL_FUNC("tkSrvAdminDisconnect");
  ZkBool success = ZK_TRUE;

  try {
    try {

      if( !srvP ) {
        success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                            func, "server context reference is null");

      } else {
        if( (*srvP)->fd != -1 ) { 
          close((*srvP)->fd); 
          (*srvP)->fd = -1; 
        }

        free(*srvP); 
        (*srvP) = NULL;
      } 

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}


//
// Send a Stop request to the server
//
extern "C" ZkBool tkSrvAdminServerStopRequest(ZksErr       err, 
                                              ZkTkSrv      srv,
                                              const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerStopRequest");
  ZkBool success = ZK_TRUE;
  
  try {
    try {
      //
      // Send the request
      //
      tkSndServerStopPacket(srv->fd,
                            srv->txbuff,
                            sizeof(srv->txbuff),
                            timeoutP);
    }  catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}


//
// Receive the acknolege of the Stop request
//
extern "C" ZkBool tkSrvAdminServerStopResponse(ZksErr       err, 
                                               ZkTkSrv      srv,
                                               const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerStopResponse");
  ZkBool success = ZK_TRUE;
  
  try {
    try {
      uint16_t     cmd;
      uint32_t     paylen;
      uint32_t     apierr;

      //
      // Read the response header
      //
      tkRcvPacketHeader(srv->fd,
                        srv->txbuff,
                        sizeof(srv->txbuff),
                        timeoutP,
                        cmd,
                        paylen);

      switch( cmd ) 
        {
        case TkPacket::CMD_ANSWER:
          tkRcvAnswerPacketPayload(srv->fd,
                                   srv->txbuff,
                                   sizeof(srv->txbuff),
                                   timeoutP,
                                   apierr);
        
          if( apierr ) {
            success = zksErrCodeSet(err,
                                    apierr,
                                    func,
                                    "Remote server return an error");
          }
          break;

        default:
          success = zksErrSet(err, 
                              ZKS_ZKTKSRV_API_MOD, 
                              ZKS_ZKTKSRV_API_UNKNOWN_PACKET_ERR,
                              func, "invalid address length");
        }

    }  catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Send a Hold request to the server
//
extern "C" ZkBool tkSrvAdminServerHoldRequest(ZksErr       err, 
                                              ZkTkSrv      srv,
                                              const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerHoldRequest");
  ZkBool success = ZK_TRUE;
  
  try {
    try {
      //
      // Send the request
      //
      tkSndServerHoldPacket(srv->fd,
                            srv->txbuff,
                            sizeof(srv->txbuff),
                            timeoutP);
    }  catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}


//
// Receive the acknolege of the Hold request
//
extern "C" ZkBool tkSrvAdminServerHoldResponse(ZksErr       err, 
                                               ZkTkSrv      srv,
                                               const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerHoldResponse");
  ZkBool success = ZK_TRUE;
  
  try {
    try {

      uint16_t     cmd;
      uint32_t     paylen;
      uint32_t     apierr;

      //
      // Read the response header
      //
      tkRcvPacketHeader(srv->fd,
                        srv->txbuff,
                        sizeof(srv->txbuff),
                        timeoutP,
                        cmd,
                        paylen);

      switch( cmd ) 
        {
        case TkPacket::CMD_ANSWER:
          tkRcvAnswerPacketPayload(srv->fd,
                                   srv->txbuff,
                                   sizeof(srv->txbuff),
                                   timeoutP,
                                   apierr);
        
          if( apierr ) {
            success = zksErrCodeSet(err,
                                    apierr,
                                    func,
                                    "Remote server return an error");
          }
          break;

        default:
          success = zksErrSet(err, 
                              ZKS_ZKTKSRV_API_MOD, 
                              ZKS_ZKTKSRV_API_UNKNOWN_PACKET_ERR,
                              func, "invalid address length");
        }

    }  catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Send a Release request to the server
//
extern "C" ZkBool tkSrvAdminServerReleaseRequest(ZksErr       err, 
                                                 ZkTkSrv      srv,
                                                 const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerReleaseRequest");
  ZkBool success = ZK_TRUE;
  
  try {
    try {
      //
      // Send the request
      //
      tkSndServerReleasePacket(srv->fd,
                               srv->txbuff,
                               sizeof(srv->txbuff),
                               timeoutP);
    }  catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Receive the acknolege of the Release request
//
extern "C" ZkBool tkSrvAdminServerReleaseResponse(ZksErr       err, 
                                                  ZkTkSrv      srv,
                                                  const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvAdminServerReleaseResponse");
  ZkBool success = ZK_TRUE;
  
  try {
    try {

      uint16_t     cmd;
      uint32_t     paylen;
      uint32_t     apierr;

      //
      // Read the response header
      //
      tkRcvPacketHeader(srv->fd,
                        srv->txbuff,
                        sizeof(srv->txbuff),
                        timeoutP,
                        cmd,
                        paylen);

      switch( cmd ) 
        {
        case TkPacket::CMD_ANSWER:
          tkRcvAnswerPacketPayload(srv->fd,
                                   srv->txbuff,
                                   sizeof(srv->txbuff),
                                   timeoutP,
                                   apierr);
        
          if( apierr ) {
            success = zksErrCodeSet(err,
                                    apierr,
                                    func,
                                    "Remote server return an error");
          }
          break;

        default:
          success = zksErrSet(err, 
                              ZKS_ZKTKSRV_API_MOD, 
                              ZKS_ZKTKSRV_API_UNKNOWN_PACKET_ERR,
                              func, "invalid address length");
        }

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception & e ) {
    ::zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}




#endif /* ZK_LINUX */
