#define REV_ID "@(#) $Id: client.cpp,v 1.3 2001/04/23 22:38:58 alainc Exp $"
#include "zkrev.h"

#include "zktksrv.h"
#include "srvctx.h"

#include "zkserr.h"
#include "zksexception.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>




namespace {

  //
  // Return the ip of a remote node
  //
  // REM: This DO NOT WORK WITH IPv6
  //
  ZkBool hostAddrByName(ZksErr err, uint32_t * ipP, const char * host)
  {
    DECL_FUNC("{anonymous}::hostAddrByName");

    struct hostent * hp;
    ZkBool           success = ZK_TRUE;

    // lookup for the node host description.
    if( (hp = ::gethostbyname(host)) == NULL ) {
      success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_GETHOSTBYNAME_ERR,
                          func, "gethostbyname failed");

      // make sure it is an IPv4 address
    } else if( hp->h_addrtype != AF_INET ) {
      success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_IPVERSION_ERR,
                          func, "invalid address familly");

      // validate the address length
    } else if( hp->h_length != 4 ) {
      success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_IPVERSION_ERR,
                          func, "invalid address length");
    } else if( ipP ) { 
      *ipP = ::ntohl(((struct in_addr *)hp->h_addr)->s_addr);     
    }

    return success;
  }

} // namespace {anonymous} 


//
// Establish a connection w/ the token server
//
extern "C" ZkBool tkSrvClientConnect(ZksErr       err, 
                                     ZkTkSrv    * srvP,
                                     const char * server, 
                                     uint16_t     port, 
                                     uint16_t     retry)
{
  DECL_FUNC("tkSrvClientConnect");

  struct protoent  * ppe     = NULL;
  struct sockaddr_in srvaddr;
  ZkBool             success = ZK_TRUE;
  uint32_t           ip      = 0;
  ZkTkSrv            ctx     = NULL;
  int                fd      = -1;

  try { 
    try {
      if( !srvP ) {
        /* Validate inval */
        success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                            func, "server context reference is null");

      } else {
        if( !server ) {
          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                              func, "server (host||ip) isn't provided");

          /* Allocate context handler */
        } else if( !(ctx = (ZkTkSrv) calloc(1, sizeof(struct S_ZkTkSrvCtx))) ) {

          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_CALLOC_ERR,
                              func, "context allocation failure");
          /* Get protocol */
        } else if( (ppe = getprotobyname("tcp")) == NULL ) {

          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_GETPROTOBYNAME_ERR,
                              func, "invalid address length");

          /* Get the socket */
        } else if( (fd  = socket(AF_INET, SOCK_STREAM, ppe->p_proto)) == -1 ) {

          success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_SOCKET_ERR,
                              func, "invalid address length");


          /* setup the address of the server */
        } else if( (success = hostAddrByName(err, &ip, server)) ) {

          srvaddr.sin_family      = AF_INET;
          srvaddr.sin_port        = htons(port);
          srvaddr.sin_addr.s_addr = htonl(ip);

          /* Try to establish the connection w/ the server */
          if( connect(fd, (struct sockaddr *)&srvaddr, (int)sizeof(struct sockaddr_in)) == -1 ) {

            zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_SOCKET_ERR,
                      func, "connect failed");
          }

          /* Retry if didn't succeed */
          while( (fd == -1) && retry-- ) { 
            connect(fd, (struct sockaddr *)&srvaddr, (int)sizeof(struct sockaddr_in)); 
          }

          if( fd == -1 ) { success = ZK_FALSE; }
        }

        if( !success ) {

          /* Cleanup if failure */
          if( ctx )       free(ctx);
          if( fd != -1 )  close(fd);

        } else {

          /* Success: fill output params */
          (*srvP)        = ctx;
          (*srvP)->fd    = fd;
          (*srvP)->local = ZK_FALSE;
        }
      }

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Disconnect of the token server
//
extern "C" ZkBool tkSrvClientDisconnect(ZksErr       err, 
                                        ZkTkSrv    * srvP)
{
  DECL_FUNC("tkSrvClientDisconnect");
  ZkBool success = ZK_TRUE;

  try {
    try {

      if( !srvP ) {
        success = zksErrSet(err, ZKS_LIB_ZKTKSRV_MOD, ZKS_LIB_ZKTKSRV_EINVAL_ERR,
                            func, "server context reference is null");

      } else {
        if( (*srvP)->fd != -1 ) { 
          close((*srvP)->fd); 
          (*srvP)->fd = -1; 
        }

        free(*srvP); 
        (*srvP) = NULL;
      } 

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}





//
// Client request for tokens agains a serial number
//
extern "C" ZkBool tkSrvClientTokenRequest(ZksErr                 err, 
                                          ZkTkSrv                srv,
                                          char                   serial[ZKS_ZKTKSRV_SERIAL_LENGTH+1],
                                          char                   password[ZKS_ZKTKSRV_SERIAL_PASSWORD_LENGTH+1],
                                          const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvClientTokenRequest");
  ZkBool success = ZK_FALSE;

  try { 
    try {
      //
      // Send the request to the server
      //
      tkSndTokenRequestPacket(srv->fd,
                              srv->txbuff,
                              sizeof(srv->txbuff),
                              timeoutP,
                              serial,
                              password);

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



//
// Client reception for tokens request.  
//
// : Each token signature is verify using 'sig', the public signature key of the
//   token server.
// 
// : This function allocate the tokens. The invoker MUST FREE the tokens afterwise
//   using 'frTokenFree'
//
extern "C" ZkBool tkSrvClientTokenResponse(ZksErr       err, 
                                           ZkTkSrv      srv,
                                           uint16_t   * countP, 
                                           FrToken    * token[ZKS_ZKTKSRV_TOKEN_MAX],
                                           FrCryptSig   sig,
                                           const struct timeval * timeoutP)
{
  DECL_FUNC("tkSrvClientTokenResponse");
  ZkBool success = ZK_FALSE;

  try { 
    try {
      uint16_t     cmd;
      uint32_t     paylen;
      uint32_t     apierr;
      const char * binToken[ZKS_ZKTKSRV_TOKEN_MAX];

      // Preclean the response
      ::memset(token, 0, sizeof(token));

      //
      // Read the response header
      //
      tkRcvPacketHeader(srv->fd,
                        srv->txbuff,
                        sizeof(srv->txbuff),
                        timeoutP,
                        cmd,
                        paylen);


      //
      // Receive  the associate payload
      //
      switch( cmd ) 
        {
        case TkPacket::CMD_TOKEN_RESPONSE:          
          tkRcvTokenResponsePacketPayload(srv->fd,
                                          srv->txbuff,
                                          sizeof(srv->txbuff),
                                          timeoutP,
                                          *countP,
                                          binToken);
          
                     
          // Allocate each tokens and stream it
          for( uint16_t c=0; success && c<*countP; c++ ) {
            success = frTokenNewFromBinary(zkError(err), 
                                           token[c],
                                           const_cast<unsigned char *>(reinterpret_cast<const unsigned char *>(binToken[c])),
                                           ZKS_ZKTKSRV_TOKEN_LENGTH,
                                           sig);
          }

        case TkPacket::CMD_ANSWER:          
          tkRcvAnswerPacketPayload(srv->fd,
                                   srv->txbuff,
                                   sizeof(srv->txbuff),
                                   timeoutP,
                                   apierr);

          if( apierr ) {
            success = zksErrCodeSet(err,
                                    apierr,
                                    func,
                                    "Remote server return an error");
          }
          break;

        default:
          success = zksErrSet(err, 
                              ZKS_ZKTKSRV_API_MOD, 
                              ZKS_ZKTKSRV_API_UNKNOWN_PACKET_ERR,
                              func, "invalid address length");
        }

      if( !success ) {
        for( uint16_t c=0; c<ZKS_ZKTKSRV_TOKEN_MAX && token[c]; c++ ) {
          frTokenFree(0, token[c]);
          token[c] = 0;
        }
      }

    } catch( ... ) {
      CZKS::Rethrow(func, "failure");
    }
  } catch( CZKS::Exception &e ) {
    zksErrCtxClone(err, e.zksErr());
    success = ZK_FALSE;
  }

  return success;
}



