#define REV_ID "@(#) $Id: protocol.cpp,v 1.3 2001/04/23 22:38:58 alainc Exp $"
#include "zkrev.h"

#include "zktksrv.h"

#include "zks.h" 
#include "zkIntTypes.h"
#include "zksexception.h"


//
//
//
void tkRcvPacketHeader(int                    fd, 
                       const char           * buffer, 
                       uint32_t               bsz,
                       const struct timeval * timeoutP,
                       uint16_t             & cmd,
                       uint32_t             & paylen)
{
  DECL_FUNC("tkRcvPacketHeader");

  TkPktHeader hdr(const_cast<char *>(buffer), bsz);

  try {    
    hdr.rcvHeader(fd, timeoutP);

    cmd    = hdr.cmd();
    paylen = hdr.paylen();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvAnswerPacketPayload(int                    fd, 
                              const char           * buffer, 
                              uint32_t               bsz,
                              const struct timeval * timeoutP,
                              uint32_t             & error)
{
  DECL_FUNC("tkRcvAnswerPacketPayload");

  AnswerPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    error = pkt.errcode();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}



//
//
//
void tkRcvIdentifyPacketPayload(int                    fd, 
                                const char           * buffer, 
                                uint32_t               bsz,
                                const struct timeval * timeoutP,
                                std::string          & id,
                                uint16_t             & isz)
{
  DECL_FUNC("tkRcvIdentifyPacketPayload");
  
  IdentifyPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    isz = pkt.id_length();
    id  = pkt.id();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}



//
//
//
void tkRcvRedirectPacketPayload(int                    fd, 
                                const char           * buffer, 
                                uint32_t               bsz,
                                const struct timeval * timeoutP,
                                uint32_t             & ip, 
                                uint16_t             & port)
{
  DECL_FUNC("tkRcvRedirectPacketPayload");

  RedirectPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    ip   = pkt.redirectIP();
    port = pkt.redirectPort();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvServerStopPacketPayload(int                    fd, 
                                  const char           * buffer, 
                                  uint32_t               bsz,
                                  const struct timeval * timeoutP)
{
  DECL_FUNC("tkRcvServerStopPacketPayload");

  ServerStopPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvServerHoldPacketPayload(int                    fd, 
                                  const char           * buffer, 
                                  uint32_t               bsz,
                                  const struct timeval * timeoutP)
{
  DECL_FUNC("tkRcvServerHoldPacketPayload");

  ServerHoldPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvServerReleasePacketPayload(int                    fd, 
                                     const char           * buffer, 
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP)
{
  DECL_FUNC("tkRcvServerReleasePacketPayload");

  ServerReleasePacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvSerialAddPacketPayload(int                    fd, 
                                 const char           * buffer, 
                                 uint32_t               bsz,
                                 const struct timeval * timeoutP,
                                 std::string          & serial,
                                 std::string          & cksum,
                                 std::string          & capacity)
{
  DECL_FUNC("tkRcvSerialAddPacketPayload");

  SerialAddPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    serial   = pkt.serial();
    cksum    = pkt.cksum();
    capacity = pkt.capacity();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}



//
//
//
void tkRcvSerialReniewPacketPayload(int                    fd, 
                                    const char           * buffer, 
                                    uint32_t               bsz,
                                    const struct timeval * timeoutP,
                                    std::string          & serial)
{
  DECL_FUNC("tkRcvSerialReniewPacketPayload");

  SerialReniewPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    serial = pkt.serial();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvTokenRequestPacketPayload(int                    fd, 
                                    const char           * buffer, 
                                    uint32_t               bsz,
                                    const struct timeval * timeoutP,
                                    std::string          & serial, 
                                    std::string          & password)
{
  DECL_FUNC("tkRcvTokenRequestPacketPayload");

  TokenRequestPacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    serial   = pkt.serial();
    password = pkt.password();

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkRcvTokenResponsePacketPayload(int                    fd, 
                                     const char           * buffer, 
                                     uint32_t               bsz,
                                     const struct timeval * timeoutP,
                                     uint16_t             & count,
                                     uint32_t             & size,
                                     const char           * token[ZKS_ZKTKSRV_TOKEN_MAX])

{
  DECL_FUNC("tkRcvTokenResponsePacketPayload");

  TokenResponsePacket pkt(const_cast<char *>(buffer), bsz);

  try {
    pkt.rcvPayload(fd, timeoutP);

    count = pkt.tokenCount();
    size  = pkt.tokenSize();

    if( count > ZKS_ZKTKSRV_TOKEN_MAX ) {
      CZKS::Throw(ZKS_ZKTKSRV_API_MOD, 
                  ZKS_ZKTKSRV_API_PKT_FORMAT_ERR,
                  func, "token count %u exceed maximum %u", count, ZKS_ZKTKSRV_TOKEN_MAX);
    }

    for( uint16_t c=0; c<count && c<ZKS_ZKTKSRV_TOKEN_MAX; c++ ) { 
      token[c] = pkt.token(c);
    }

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndAnswerPacket(int                    fd, 
                       char                 * buffer, 
                       uint32_t               bsz,
                       const struct timeval * timeoutP,
                       const uint32_t       & errcode)
{
  DECL_FUNC("tkSndAnswerPacket");

  AnswerPacket pkt(buffer, bsz);

  try {
    pkt.errcode(errcode);
    
    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}



//
//
//
void tkSndIdentifyPacket(int                    fd, 
                         const char           * buffer, 
                         uint32_t               bsz,
                         const struct timeval * timeoutP,
                         const std::string    & id,
                         const uint16_t       & isz)
{
  DECL_FUNC("tkSndIdentifyPacket");
  
  IdentifyPacket pkt(const_cast<char *>(buffer), bsz);

  try {

    pkt.id_length(isz);
    pkt.id(id);

    pkt.rcvPayload(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndRedirectPacket(int                    fd, 
                         char                 * buffer, 
                         uint32_t               bsz,
                         const struct timeval * timeoutP,
                         const uint32_t       & ip, 
                         const uint16_t       & port)
{
  DECL_FUNC("tkSndRedirectPacket");

  RedirectPacket pkt(buffer, bsz);

  try {
    pkt.redirectIP(ip);
    pkt.redirectPort(port);

    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndServerStopPacket(int                    fd, 
                           char                 * buffer, 
                           uint32_t               bsz,
                           const struct timeval * timeoutP)
{
  DECL_FUNC("tkSndServerStopPacket");

  ServerStopPacket pkt(buffer, bsz);

  try {
    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndServerHoldPacket(int                    fd, 
                           char                 * buffer, 
                           uint32_t               bsz,
                           const struct timeval * timeoutP)
{
  DECL_FUNC("tkSndServerHoldPacket");

  ServerHoldPacket pkt(buffer, bsz);

  try {
    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndServerReleasePacket(int                    fd, 
                              char                 * buffer, 
                              uint32_t               bsz,
                              const struct timeval * timeoutP)
{
  DECL_FUNC("tkSndServerReleasePacket");

  ServerReleasePacket pkt(buffer, bsz);

  try {
    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndSerialAddPacket(int                    fd,
                          char                 * buffer,
                          uint32_t               bsz,
                          const struct timeval * timeoutP,
                          const std::string    & serial,
                          const std::string    & cksum,
                          const std::string    & capacity)
{
  DECL_FUNC("tkSndSerialAddPacket");

  SerialAddPacket pkt(buffer, bsz);

  try {
    pkt.serial(serial);
    pkt.cksum(cksum);
    pkt.capacity(capacity);

    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }

}


//
//
//
void tkSndSerialReniewPacket(int                    fd, 
                             char                 * buffer, 
                             uint32_t               bsz,
                             const struct timeval * timeoutP,
                             const std::string    & serial)
{
  DECL_FUNC("tkSndSerialReniewPacket");

  SerialAddPacket pkt(buffer, bsz);

  try {
    pkt.serial(serial);

    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}


//
//
//
void tkSndTokenRequestPacket(int                    fd, 
                             char                 * buffer, 
                             uint32_t               bsz,
                             const struct timeval * timeoutP,
                             const std::string    & serial, 
                             const std::string    & password)
{
  DECL_FUNC("tkSndTokenRequestPacket");

  TokenRequestPacket pkt(buffer, bsz);

  try {
    pkt.serial(serial);
    pkt.password(password);

    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}



//
//
//
void tkSndTokenResponsePacket(int                    fd, 
                              char                 * buffer, 
                              uint32_t               bsz,
                              const struct timeval * timeoutP,
                              const uint16_t       & count,
                              const uint32_t       & size,
                              const char           * token[ZKS_ZKTKSRV_TOKEN_MAX])
{
  DECL_FUNC("tkSndTokenResponsePacket");

  TokenResponsePacket pkt(buffer, bsz);

  try {
    pkt.tokenCount(count);
    pkt.tokenSize(size);

    for( uint16_t c=0; c<count && c<ZKS_ZKTKSRV_TOKEN_MAX; c++ ) { 
      pkt.token(c, token[c]);
    }

    pkt.snd(fd, timeoutP);

  } catch( CZKS::Exception & e ) {

    CZKS::Rethrow(func, "failure");
  }
}

