#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <zkConfig.h>
#include <zkError.h>
#include <zkConfig.h>
#include <zkIntTypes.h>
#include <zkMIME.h>
#include <errno.h>
#include <zkAlloc.h>
#include "m2n.h"

typedef struct {
  char *buffer;
  size_t sz;
} FrM2NEmailBufferCtx;

/* Swiped from libzk/zkFile.c */
int alloc(ZkError err, void **p, size_t n)
{
  if (*p) {
    return zkMemRealloc(err, p, n);
  } else if ( ! (*p = malloc(n))) {
    zkErrorSet_va(err, "alloc", ZKERR, ZKMFILE, errno,
                  "can't allocate %lu bytes: %s",
                  (unsigned long)n, strerror(errno));
    return 0;
  }

  return 1;
}

/**
 * Sane dumping of the email message to a buffer.  The ctx consists of a
 * char * buffer, size_t position indicator, size_t total number of bytes.
 * 
 * @param err
 * @param ctx                   Email buffer context
 * @param data                  Data
 * @param dlen                  Length of data
 */
int frNGDumpEmailToBuffer(ZkError err, void *ctx, 
                           const unsigned char *data, int dlen)
{
  FrM2NEmailBufferCtx *e = (FrM2NEmailBufferCtx *) ctx;

  if (!alloc(err, (void *) &e->buffer, (e->sz + dlen) * sizeof(char))) {
    return 0;
  }

  strncpy(&e->buffer[e->sz], data, dlen);

  e->sz += dlen;
  
  return 1;
}

int frNGEmailToBuffer(ZkError err, char **outbuf, size_t *sz, ZkEmail email)
{
  const char *func = "frNGEmailTobuffer";
  FrM2NEmailBufferCtx e;
  int ret = 1;

  if (!outbuf) {
    zkErrorSet(err, func, ZKERR, ZKMM2N, ZKENULLPTR, "outbuf is NULL");
    return 0;
  }

  if (!sz) {
    zkErrorSet(err, func, ZKERR, ZKMM2N, ZKENULLPTR, "sz is NULL");
    return 0;
  }

  if (!email) {
    zkErrorSet(err, func, ZKERR, ZKMM2N, ZKENULLPTR, "ZkEmail is NULL");
    return 0;
  }

  e.buffer = NULL;
  e.sz = 0;

  *outbuf = NULL;

  if (!zkEmailDump(err, email, frNGDumpEmailToBuffer, &e)) {
    goto cleanup;
  }

  if ((*outbuf = (char *) malloc(sizeof(char) * (e.sz + 1))) == NULL) {
    ret = 0;
    goto cleanup;
  }

  memcpy(*outbuf, e.buffer, e.sz);
  (*outbuf)[e.sz] = '\0';
  *sz = e.sz;

 cleanup:
  if (e.buffer) {
    free(e.buffer);
    e.buffer = NULL;
  }
  return ret;  
}
