/*
 * $Id: amtp.c,v 1.59 2000/05/11 19:14:35 serge Exp $
 */

static const char * module_id_ = "$Id: amtp.c,v 1.59 2000/05/11 19:14:35 serge Exp $";

#include "zks.h"
#include <netinet/in.h>
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <netdb.h>
#include <ctype.h>
#include "zkCStr.h"
#include "zkMIME.h"
#include "zkEmailHeader.h"
#include "zkMemBuf.h"
#include "zkList.h"
#include "zkEmailUtil.h"
#include "frNym.h"
#include "frCrypt.h"
#include "frSec.h"
#include "frNymClient.h"
#include "frNymUtil.h"
#include "maip.h"
#include "statdb.h"
#include "amtp.h"
#include "zkAlloc.h"

/*
 *
 */

#define AMTP_TARGET_REPLYBLOCK 1
#define AMTP_TARGET_NYM 2
#define AMTP_TARGET_INTERNET 3

/*
 * Internal function prototypes
 */

int amtpSend ( ZkError , FrMaipContext * , FrMaipSessionContext * );
static int amtpDeleteTrailingNuls ( ZkError , char * , size_t * );

#ifdef ZK_FMG

/*
 * These functions are only used when the maip is running in FMG mode
 */

static int amtpCheckSignature( ZkError , FrMaipContext *, 
                               FrMaipSessionContext * , void *, size_t );
static int amtpCheckBlocklist ( ZkError , FrRemoteNym , const char * , int * );
int amtpTransmissionAllowed ( ZkError , FrMaipContext * , FrEntIdExp * , 
                              int , int * );
static int strtolower ( 
  ZK_INOUT char * str );

static int amtpCheckNymBlockDatabase (
  ZK_INOUT ZkError err ,
  ZK_IN    FrMaipContext * c,
  ZK_IN    const char * nym ,
  ZK_IN    const char * target ,
  ZK_IN    int target_type ,
  ZK_OUT   int * ok );

#endif /* ZK_FMG */

#ifdef ZK_FMG

/*
 * Simple utility function to convert a string to lower case.  This is used
 * in the amtpCheckNymBlockDatabase() only.  This code is only used by the
 * FMG-MAIP.
 */

static int
strtolower (
  ZK_INOUT char * str )
{
  if (str) {
    while ((*str = tolower(*str)) != (char) 0) {
      str++;
    }
    return 1;
  } 
  return 0;
}

#endif /* ZK_FMG */

/*
 *  This function will remove the trailing NUL characters which me be present
 *  in a large section of text.  These NULs may be added by the windows 
 *  client on some occasions, so this acts as a sanity check.
 */

static int
amtpDeleteTrailingNuls (
  ZkError err ,
  char * d ,
  size_t * sz )
{
  static const char * func = "amtpDeleteTrailingNuls";
  int ret = 1;

  (void)module_id_;

  if (!d) {
    zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,"Data buffer pointer is NULL");
    ret = 0;
  } else if (!sz) {
    zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,"Data size pointer is NULL");
    ret = 0;
  } else if (*sz > 0) {
    while (d[*sz - 1] == '\0') {
      *sz -= 1;
      if (*sz < 1) {
        break;
      }
    }
  }
  return ret;
}


/*
 *
 * 
 */

#ifdef ZK_FMG
int
amtpTransmissionAllowed (
  ZkError err ,
  FrMaipContext * c ,
  FrEntIdExp * sender ,
  int recipients ,
  int * ok )
{
#define SENDER_C_SZ 128

#if 0
   /* Testing only - every 20 minutes */
#  warning "mail statistics database reset period is in testing mode!"
#  define FMGMAIP_MAIP_STAT_EXPIRY_PERIOD (20 * 60)
#else
   /* every 24 hours */
#  define FMGMAIP_MAIP_STAT_EXPIRY_PERIOD (24 * 60 * 60)
#endif 

  const char * func = "amtpTransmissionAllowed";
  MAIPStatRec r;
  int found = 0;
  time_t now = time(NULL);
  int ret = 0; /* return status */

  zkErrorClear(err);
  if (ok && c && sender) {
    *ok = 0;
   
    /*
     * The FMG entities can send as much mail as they want
     */

    if (sender->type == FRENT_TYPE_FMG) {
      zkLogWrite(c->log,"Processing message from FMG filter");
      *ok = 1;
      return 1;
    }

    /*
     * The nym server entities can send as much mail as they want
     */

    if (sender->type == FRENT_TYPE_NYM_SRV) {
      zkLogWrite(c->log,"Processing message from nym server");
      *ok = 1;
      return 1;
    }

    /*
     * Retrieve stat data for this sender
     */

    /* first open the db for it */
    if(c->statdbname == NULL) {
	zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,"The stat db name is null0");
	return 0;
    }

    /* we need to put the database in the context as it is passed to the */
    /* fectching and storing functions later */
    if (!zkDB_Open(err,&(c->statdb),c->statdbname)) {
        zkLogWriteContext(c->log,err,func,
                          "Failed to open stat db");
        if (!zkipSessionResponse(err,c->session,0,
                                 _SERVICE_CLOSING,
				 "Unable to open mail statistics database",
				 "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
	    zkLogWriteContext(c->log,err,func,
			      "Failed to notify client that stat db open failed");
	    return 0;
        }
    }
    

    memset(&r,0,sizeof(MAIPStatRec));
    if (!MAIPStatRecFetch(err,c,sender->name,&r,&found)) {
	/* close database */
	goto closeDB;
    }

    /* If we dont have a current stat entry for this sender, query the
     * nym server to get a description of the sender nym and create
     * A stat entry for it.
     */

    if (!found || (now > r.expire)) {
      FrRemoteNym nym = NULL;
      NymServer nymserver;
      NymServerResponse nsresp;
      char * l = NULL;
      size_t lsz = 0;

      /*
       * Bug: use the nymserver signature key to verify the data
       * returned from the nymserver
       */
      strncpy(nymserver.nymserver,c->nshost,NSHOST_SZ);
      nymserver.port = c->nsport;
      nymserver.timeout = c->nstimeout;
      nymserver.sigkey = NULL;
      if (!frRemoteNymGet(err,&nym,sender->name,&nymserver,&nsresp)) {
        zkLogWriteContext(c->log,err,func,
                          "Unable to retrieve sender nym information!");
        zkErrorSet_va(err,func,ZKERR,ZKMMAIP,ZKENOTFOUND,
                      "Sender nym \"%s\" not found",sender->name);
        goto closeDB;
      }
      frRemoteNymResetParameterSearch(err,nym);
      if (!frRemoteNymGetParameter(err,nym,NymEmailLimit,
                                   (const unsigned char **)&l,&lsz)) {
        zkLogWriteContext(c->log,err,func,
                          "Couldn't retrieve nym e-mail limit");
        frRemoteNymFree(NULL,&nym);
        goto closeDB;
      } 

      if (c->flags & FLAG_DEBUG) {
        zkLogWrite_va( c->log,
                       "INFO: E-mail limit for %s has been reset",
                       sender->name );
      }

      r.limit = atoi(l);
      r.sent = 0;
      r.expire = now + FMGMAIP_MAIP_STAT_EXPIRY_PERIOD;
      frRemoteNymFree(NULL,&nym);
    }

    /*
     * Check to see if we've reached the quota for that sender and update
     * the stat record in the database 
     */

    if ((r.sent + recipients) <= r.limit) {
      int stored = 0;
      r.sent += recipients;
      if (!MAIPStatRecStore(err,c,sender->name,&r,&stored)) {
        zkLogWriteContext(c->log,err,func,
                          "Unable to write stat record for %s",
                          sender->name);
        goto closeDB;
      }
      if (!stored) {
        zkLogWrite_va( c->log,
                       "Unable to write stat record for %s",
                       sender->name );
      }
      *ok = 1;
    }
    
    ret = 1;  
  } else {
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  }
  
 closeDB:
  if(c->statdb) {
      if (!zkDBClose(err,&(c->statdb))) {
	  zkLogWriteContext(c->log,err,func,
			    "Failed to close stat db");
	  ret = 0;
      }
  }
  return ret;
}

/*
 * 
 *
 */

static int
amtpCheckBlocklist (
  ZkError err ,
  FrRemoteNym target ,
  const char * sender , 
  int * blocked )
{
  const char * func = "amtpCheckBlocklist";
  ZkList blocklist = NULL;
  char * addr = NULL;

  zkErrorClear(err);
  if (sender && target && blocked) {
    * blocked = 0;
    frRemoteNymResetParameterSearch(err,target);
    if (!frRemoteNymGetBlockList(err,target,&blocklist)) {
      return 1;
    }
    zkListMoveCursor(err,blocklist,LIST_HEAD);
    while (zkListGet(err,blocklist, (void *) &addr,LIST_FORWARDS)) {
      if (strcasecmp(addr,sender) == 0) { 
        *blocked = 1;
        break;
      }
    }
    zkListFree(err,&blocklist);
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

static int
amtpCheckNymBlockDatabase (
  ZK_INOUT ZkError err ,
  ZK_IN    FrMaipContext * c ,
  ZK_IN    const char * nym ,
  ZK_IN    const char * target ,
  ZK_IN    int target_type ,
  ZK_OUT   int * ok )
{
#define KEY_DATA_SZ 256
  static const char * func = "amtpCheckNymBlockDatabase";
  char key_data[KEY_DATA_SZ];       /* Key data to search for */
  int ret = 0;                      /* Function return code */
  ZkDB db;                          /* The nymblock database */
  ZkDBT key = NULL;                 /* Search key */
  ZkDBT data = NULL;                /* Record (should always be empty) */
  int key_data_sz = 0;              /* Used size of the key_data */
  int found = 0;                    /* Set to non-zero if we find a record */
  int found2 = 0;                   /* Set to non-zero if we find a record */
  char * domain = NULL;             /* Pointer to start of address domain */
  
  if (ok) {
    *ok = 0;
  }
  if (!(nym && target && ok && c->nymblockdbname)) {
      zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,"NULL parameter pointer");
  } else {
      
      /*
       * Create the database, database key and data objects
       */
    if (!zkDB_Open(err,&db,
		   c->nymblockdbname)) {
	zkLogWriteContext(c->log,err,func,
			  "Failed to open nym block db");
	ret = 0;
    }
    else if (!(zkDBTNew(err,&key) &&
	       zkDBTNew(err,&data))) {
	zkLogWriteContext(c->log,err,func,"Failed to setup database access");
	ret = 0;
	/*
	 * Do searches on the keys
	 */
	
    } else { 
      switch (target_type) {

      /*
       * In the case of a nym to nym message, we just look up the key.  If
       * the key cannot be found, then there is no request to block this
       * transaction and we let it through.
       */

      case AMTP_TARGET_NYM:
        if ((key_data_sz = snprintf(key_data,KEY_DATA_SZ,"%s NYM %s",
                                    nym,target)) < 0) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Unable to create nym blocked address search key (nym)");
        } else if (!strtolower(key_data)) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Failed to convert key data to lower case (nym)");
        } else if (!zkDBSetKeyOrData(err,key,key_data,key_data_sz)) {
          zkLogWriteContext(c->log,err,func,"zkDBSetKeyOrData failed (nym)");
          ret = 0;
        } else if (!zkDBFetch(err,db,key,data,&found)) {
          zkLogWriteContext(c->log,err,func,"zkDBFetch failed (nym)");
          ret = 0;
        } else {
          ret = 1;
          if (found) { 
            *ok = 0;
          } else {
            *ok = 1;
          }
        }
        break;

      /*
       * In the case of an internet address, we look first for the address
       * itself, then we look for the domain (this allows us to block an
       * entire domain for a particular nym)
       */

      case AMTP_TARGET_INTERNET:

        domain = strchr(target,'@');
        key_data_sz = snprintf(key_data,KEY_DATA_SZ,"%s EMAIL %s",nym,target);

        if (key_data_sz < 0) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Unable to create nym blocked address search key (internet)");
        } else if (!strtolower(key_data)) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Failed to convert key data to lower case (internet)");
        } else if (!zkDBSetKeyOrData(err,key,key_data,key_data_sz)) {
          zkLogWriteContext(c->log,err,func,"zkDBSetKeyOrData failed (internet)");
          ret = 0;
        } else if (!zkDBFetch(err,db,key,data,&found)) {
          zkLogWriteContext(c->log,err,func,"zkDBFetch failed (internet)");
          ret = 0;
        } else if (domain == NULL) {
          ret = 1;
          *ok = !found;
        } else if ((key_data_sz = snprintf(key_data,KEY_DATA_SZ,"%s DOMAIN %s",
                                           nym,++domain)) < 0) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Unable to create nym blocked domain search key");
        } else if (!strtolower(key_data)) {
          zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,
                     "Failed to convert key data to lower case (domain)");
        } else if (!zkDBSetKeyOrData(err,key,key_data,key_data_sz)) {
          zkLogWriteContext(c->log,err,func,"zkDBSetKeyOrData failed (domain)");
          ret = 0;
        } else if (!zkDBFetch(err,db,key,data,&found2)) {
          zkLogWriteContext(c->log,err,func,"zkDBFetch failed (domain)");
          ret = 0;
        } else {
          ret = 1;
          if (found || found2) {
            *ok = 0;
          } else {
            *ok = 1; 
          }
        }
        break;

      default:
        zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKERROR,"Unrecognized target type");
        break;
      }
    }
  }

  /*
   * Cleanup 
   */


  if(db) {
      if (!zkDBClose(err,&db)) {
	  zkLogWriteContext(c->log,err,func,
			    "Failed to close nym block db");
	  ret = 0;
      }   
      db = NULL;
  }
  
  if (key) {
    zkDBTFree(NULL,&key);
  }
  if (data) {
    zkDBTFree(NULL,&data);
  }
 
  return ret;
}

#endif /* ZK_FMG */


/*
 * Specify an AMTP header
 */

int
amtpCommandHeader (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  static const char * func = "amtpCommandHeader";
  FrMaipSessionContext * sctx = (FrMaipSessionContext *) data;
  int return_code = _SESSION_CONTINUE;
  unsigned char * header_text = NULL;
  size_t header_text_sz = 0;
  int parameters = 0;


  zkErrorClear(err);
  if (session && sctx) {


    /*
     * Just make sure that the client is not trying to pass us parameters
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 0) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unexpected parameters (this command expects none)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 
   
    /*
     * Did we already get the AMTP message header? if so, we dont want
     * one again...
     */

    if (sctx->flags & FLAG_SCTX_HEADER_SET) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "The header has already been set","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * We can create the header.
     */

    if (!(zkEmailNew(err,&(sctx->email)) &&
          zkEmailHeader(err,sctx->email,&(sctx->header)))) {
      zkLogWriteContext(frMaipContext->log,err,func,"Can't create e-mail header");
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Unable to create internal data structure","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    }
    sctx->flags |= FLAG_SCTX_HEADER_SET;

    /*
     * Ok, now read the message header from the client....
     */
 
    if (!zkipSessionReadData(err,session,&header_text,&header_text_sz)) {
      zkLogWriteContext(frMaipContext->log,err,func,"Can't read header data");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Incorrect data block",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * Make sure that there are no trailing NULs in the header data.
     */

    if (!amtpDeleteTrailingNuls(err,header_text,&header_text_sz)) { 
      zkLogWriteContext(frMaipContext->log,err,func,"amtpDeleteTrailingNuls failed");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Internal MAIP error",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * And parse it.... (we add the extra CRLF CRLF pair to make sure that the
     * header is properly terminated as specified by RFC822 (this is the format
     * which is parsed by the zkEmail functions()).
     */
   
    if (!(zkEmailAddText(err,sctx->email,header_text,header_text_sz) &&
          zkEmailAddText(err,sctx->email,"\015\012\015\012",4) &&
          zkEmailParse(err,sctx->email))) {
      zkLogWriteContext(frMaipContext->log,err,func,"Failed to parse header data");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to parse header data","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    }

    frMaipStatusIncrementHeaderRx(err, sctx->status, header_text_sz);

    zkipSessionResponse(err,session,0,_SERVICE_READY,
                        "Header acknowledged, please send me the message",
                        "NODATA",ZKIP_ENCODING_8BIT,0,NULL);
    return_code = _SESSION_CONTINUE;
 
finish:
    if (return_code == _SESSION_ERROR) {
      if (sctx->email) {
        zkEmailFree(NULL,&(sctx->email));
      } 
      sctx->header = NULL;
      sctx->flags &= !FLAG_SCTX_HEADER_SET;
    }

    if (header_text) {
      free(header_text);
      header_text = NULL;
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}

#ifdef ZK_FMG
static int amtpCheckSignature(ZkError err, FrMaipContext *maip, FrMaipSessionContext * sctx , void *data, 
                              size_t dsz)
{
  const char *func = "amtpCheckSignature";
  FrCryptSig sigKey = NULL;
  int valid;
#if 0
  FrEntIdExp entity;
#endif
  int vers;

  if (!maip) {
    zkErrorSet(err, func, ZKERR, ZKMFRMAIP, ZKENULLPTR, "context is NULL");
    return 0;
  }

  if (!data) {
    zkErrorSet(err, func, ZKERR, ZKMFRMAIP, ZKENULLPTR, "data is NULL");
    return 0;
  }

  if (!frSecSigGetEntity(err, data, dsz, &(sctx->sender))) {
    zkLogWriteContext(maip->log,err,func,"frSegSigGetEntity failed");
    return 0;
  }
#if 0
  strncpy(sctx->sender,entity.name,NYMNAME_SZ);
  sctx->sender[NYMNAME_SZ] = (char) 0;
#endif

  if (!frSecSigGetVersion(err, data, dsz, &vers)) {
    zkLogWriteContext(maip->log,err,func,"frSecSigGetVersion failed");
    return 0;
  }


  if (maip->kh.fd == -1)
  {
    struct protoent *ppe;
    
    ppe = getprotobyname("udp");

    if ((maip->kh.fd = socket(AF_INET, SOCK_DGRAM, ppe->p_proto)) == -1) {
         
      zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, "socket() failed");
      return 0;
    }
  }

  if (maip->flags & FLAG_DEBUG) {
    zkLogWrite(maip->log, "DEBUG: Attempting to query for a signature key");
    zkLogWrite_va(maip->log, "DEBUG: Entity name %s / entity type %d",
                  sctx->sender.name, sctx->sender.type);
  }

  /* MIPAS for key query host */
  if (!frMaipNetInfoGetType(err, maip->ni, FRENT_TYPE_KEYQRY_SRV,
			    &maip->khhost, &maip->khport, NULL)) {
      zkLogWriteContext(maip->log,err,func,
			"frMaipNetInfoGetType failed (key query server)");
  }
  
  if (inet_aton(maip->khhost,&(maip->kh.host.sin_addr)) == 0) {
      zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, "Can't convert hostname");
  }
  
  maip->kh.host.sin_port = htons(maip->khport);
  
  free(maip->khhost);
  maip->khhost = NULL;
  /* end MIPAS */
  
  /* Query for this entity type */
  if (!frKeyQryFetchPubKey(err, maip->kq, vers, FRCRYPT_KEYTYPE_SIGNING,
                           sctx->sender, maip->kh, maip->kqretries, maip->kqtimeout,
                           &sigKey)) {
    zkLogWriteContext(maip->log,err,func,"frKeyQryFetchPubKey failed");
    close(maip->kh.fd);
    maip->kh.fd = -1;
    return 0;
  }

  close(maip->kh.fd);
  maip->kh.fd = -1;

  if (maip->flags & FLAG_DEBUG) {
    zkLogWrite(maip->log, "DEBUG: Fetch key returned, checking signature now");
  }

  if (!frSecSigVerify(err, sigKey, data, dsz, &valid)) {
    zkLogWriteContext(maip->log,err,func,"frSecSigVerify failed");
    return 0;
  }

  if (!valid) {
    if (maip->flags & FLAG_DEBUG) {
      zkLogWrite(maip->log, "DEBUG: Invalid signature");
    }
    zkErrorSet(err, func, ZKERR, ZKMFRMAIP, ZKEDATABAD, 
               "Signature on the data is not valid");
    return 0;
  }

  if (maip->flags & FLAG_DEBUG) {
    zkLogWrite(maip->log, "DEBUG: Valid signature found");
  }
  return 1;
}
#endif

int
amtpCommandMessage (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  static const char * func = "amtpCommandMessage";
  FrMaipSessionContext * sctx = (FrMaipSessionContext *) data;
  unsigned char * message_text = NULL;
  size_t message_text_sz = 0;
#ifdef ZK_FMG
  unsigned char * clear = NULL;   /* Nym encrypted message buffer */
  size_t clear_sz = 0;
#endif
  int parameters = 0;
  int return_code = _SESSION_CONTINUE;

  zkErrorClear(err);
  data = NULL;
  if (session) {
 
    /*
     * Just make sure that the client is not trying to pass us parameters
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext(frMaipContext->log, err,func,
                        "Failed to retrieve command parameters");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 0) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unexpected parameters (this command expects none)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * Did we already get the AMTP message? if so, we dont want
     * one again...
     */

    if (sctx->flags & FLAG_SCTX_MESSAGE_SET) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "The message has already been set","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * The extra_data pointer must be released by whoever called the
     * zkipSessionReadData() function.
     */

    if (!zkipSessionReadData(err,session,&message_text,&message_text_sz)) {
      zkLogWriteContext(frMaipContext->log, err, func,
                        "Failed to read message data");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Incorrect data block",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    frMaipStatusIncrementMessageRx(err, sctx->status, message_text_sz);

#ifdef ZK_FMG
    /*
     * If we're running as the FMG maip, we have to decrypt the message 
     * using the FMG maip private cryption key.  We then sign the message
     * to prove that it went through the FMG maip
     */

    if (frMaipContext->flags & FLAG_FMG) {
      FrCryptPubKey myCryptKey = NULL;
      FrCryptPubKey mySigKey = NULL;
      size_t privVers = 0;

      if (!frSecPubKeyGetCryptKeyVersion(err, message_text, message_text_sz,
                                         &privVers)) {
        zkLogWriteContext(frMaipContext->log, err,func,
                          "Couldn't get key version from data.\n");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Couldn't get key version from message data",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      if (!frSecKeyChainGetVersion(err, frMaipContext->keys, 
                                   frMaipContext->cryptHandle, privVers,
                                   &myCryptKey)) {
        zkLogWriteContext(frMaipContext->log, err,func,
                          "Couldn't get key version from key chain.\n");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Could get private key version from key chain",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      if (!frSecPubKeyDecryptData(err,myCryptKey,message_text,
                                 message_text_sz,(void **) &clear,&clear_sz)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Could not remove FMG encryption");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Could not remove FMG encryption",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      if (message_text) {
        memset(message_text,0,message_text_sz);
        free(message_text);
        message_text = NULL;
        message_text_sz = 0;
      }

      /*
       * Check the signature on the text we just received.  If it fails
       * we can let the the client know.
       */
      if (!amtpCheckSignature(err, frMaipContext, sctx ,clear, clear_sz)) {
        zkLogWriteContext(frMaipContext->log, err,func,
                          "Signature validation failed");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Signature validation failed",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * If the sender is a nym server, then the message is a reply block
       * activation token.  The checksum of the reply block we want to use
       * is prepended to the signed message body.
       */

      if (sctx->sender.type == FRENT_TYPE_NYM_SRV) {
        FrEntIdExp sender;
        int same = 0;
        /*
         * Remember the sender
         */
        if (!frEntCopy(err,&sender,&sctx->sender)) {
          zkLogWriteContext(frMaipContext->log, err,func,
                            "Entity ID copy failed");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Entity ID copy failed",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
        /*
         * Get rid of the signature
         */
        if (!frSecSigRemoveInSitu(err,clear,&clear_sz)) {
          zkLogWriteContext(frMaipContext->log, err,func,
                            "Signature removal failed");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Signature removal failed",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
        /*
         * Make sure we've got enough data to include the rb hash
         */
        if (clear_sz < FRCRYPT_HASH_SIZE) {
          zkLogWrite(frMaipContext->log, "target reply block is missing");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "target reply block is missing",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
        /*
         *  Remember the reply block to use
         */
        sctx->tgt_rb_hash_sz = FRCRYPT_HASH_SIZE;
        if (!zkMemDup(err,&sctx->tgt_rb_hash,clear,sctx->tgt_rb_hash_sz)) {
          zkLogWriteContext(frMaipContext->log, err,func,
                            "Reply block hash allocation failed");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Internal error",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        /*
         * Get rid of the reply block hash to get to the message body.  Move
         * the message body to the front of the buffer, and clear out the
         * data at the end of the buffer.
         */
        clear_sz -= sctx->tgt_rb_hash_sz;
        memmove( clear,
                 ((char*)clear) + sctx->tgt_rb_hash_sz,
                 clear_sz );
        memset( ((char*)clear) + clear_sz, 0, sctx->tgt_rb_hash_sz);

        /*
         * Check the message signature
         */
        if (!amtpCheckSignature(err, frMaipContext, sctx ,clear, clear_sz)) {
          zkLogWriteContext(frMaipContext->log, err,func,
                            "Signature validation failed");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Signature validation failed",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        /*
         * compare the entitity identifiers for the signers of the outer
         * and inner message bodies
         */
        if (!frEntEqual(err,&sender,&sctx->sender,&same)) {
          zkLogWriteContext(frMaipContext->log, err, func,
                            "Error validating entity match");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Error validation entity match",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        /*
         * Make sure the same entity signed both bodies
         */
        if (!same) {
          zkLogWrite(frMaipContext->log, "Signing entity mismatch!");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Signing entity mismatch",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
      }

      /* Get my signing key */
      if (!frSecKeyChainGetVersion(err, frMaipContext->keys, 
                                   frMaipContext->sigHandle, 0,
                                   &mySigKey)) {
        zkLogWriteContext(frMaipContext->log, err,func,
                          "Couldn't get key version from key chain.\n");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Could not obtain FMG-Maip key version from key chain",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }
      
      if (!frSecSignData(err,mySigKey,clear,clear_sz,
                                  (void **)&message_text,&message_text_sz)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Could not sign message with FMG key");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Could sign using FMG-Maip signature key",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }
    } 
#endif /* ZK_FMG */
    
    sctx->flags |= FLAG_SCTX_MESSAGE_SET;
    sctx->message = message_text;
    sctx->message_sz = message_text_sz;
    message_text = NULL;
    message_text_sz = 0;

    zkipSessionResponse(err,session,0,_SERVICE_READY,
                        "Message acknowledged, please specify the target in order to send",
                        "NODATA",ZKIP_ENCODING_8BIT,0,NULL);

    return_code = _SESSION_CONTINUE;

finish:

    if (message_text) {
      memset(message_text,0,message_text_sz);
      free(message_text);
      message_text = NULL;
      message_text_sz = 0;
    }

#ifdef ZK_FMG
    if (clear) {
      memset(clear,0,clear_sz);
      free(clear);
      clear = NULL; 
      clear_sz = 0;
    }
#endif

    return return_code;
  }
  zkErrorSet(err,"amtpCommandMessage",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}

int 
amtpCommandAlttext (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  static const char * func = "amtpCommandAlttext";
  int return_code = _SESSION_CONTINUE; 
  int parameters = 0;
  FrMaipSessionContext * sctx = (FrMaipSessionContext *) data;

  zkErrorClear(err);
  if (session && sctx) {

    /*
     * Just make sure that the client is not trying to pass us parameters
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext(frMaipContext->log,err,func,
                        "Can't retrieve paramater count");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 0) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unexpected parameters (this command expects none)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * Did we already get the alternate text? if so, we dont want
     * one again...
     */

    if (sctx->flags & FLAG_SCTX_ALTTEXT_SET) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "The alternate text has already been set","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * Only non-FMG maips are allowed to accept the alternate text.  This
     * will prevent clients from sending the alternate text themselves even
     * if they try....
     */

#ifdef ZK_FMG
    if (frMaipContext->flags & FLAG_FMG) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "FMG maip cannot accept alternate text","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }
#endif

    /*
     * The extra_data pointer must be released by whoever called the
     * zkipSessionReadData() function.
     */

    if (!zkipSessionReadData(err,session,&(sctx->alttext),&(sctx->alttext_sz))){
      zkLogWriteContext(frMaipContext->log,err,func,"Can't read alt-text data");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Incorrect data block",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 
    sctx->flags |= FLAG_SCTX_ALTTEXT_SET;

    /*
     * Cleanup any trailing NULs in the alternate text data.  This should
     * NEVER happen, but since we have this capability, we may as well use it
     */

    if (!amtpDeleteTrailingNuls(err,sctx->alttext,&(sctx->alttext_sz))) { 
      zkLogWriteContext(frMaipContext->log,err,func,
                        "amtpDeleteTrailingNuls failed");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Internal MAIP error",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    frMaipStatusIncrementAlttextRx(err, sctx->status, sctx->alttext_sz);

    zkipSessionResponse(err,session,0,_SERVICE_READY,
                        "Alternate text acknowledged",
                        "NODATA",ZKIP_ENCODING_8BIT,0,NULL);
    return _SESSION_CONTINUE;

finish:
    return return_code;
  }
  zkErrorSet(err,"amtpCommandAlttext",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}

#ifdef ZK_FMG
static int emailDumpToMemBuf(ZkError err, void *data, 
                             const unsigned char *text, int size)
{
  ZkMemBuf membuf = (ZkMemBuf) data;

  if (!zkMemBufAppend(err, membuf, text, (size_t) size)) {
    return 0;
  }

  return 1;
}


static int maipAppendTagLine(ZkError err, FrMaipSessionContext *sctx, 
                             const char *mesg, size_t msz, char **tagged,
                             size_t *tsz, const char *rcpts)
{
  const char *      func       = "maipAppendTagLine";
  ZkEmail           email      = NULL;
  int               ret        = 0;
  ZkList            iuser      = NULL;
  const char *      iuser_addr = NULL;
  ZkMemBuf          membuf     = NULL;
  char *            tmp        = NULL;
  ZkEmailHeader     eheader    = NULL;
  size_t            sz         = 0;

  if (!sctx || !mesg || !tagged || !tsz || !rcpts) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  /* Make sure that we actually have tagline data to add */
  if (!sctx->tagline.text.data && !sctx->tagline.html.data) {
    /* Nothing to add, skip this function */
    ret = 1;
    goto tagdone;
  }

  /* Check if mail recipient is mail2news.  If it is, nothing to be done. */
  if (!(zkListNew(err,&iuser) &&
        zkListSetDataFreeFunction(err,iuser,zkListDataFree,NULL) &&
        zkEmailUtilAddToRecipientList(err,iuser,rcpts) &&
        zkListMoveCursor(err,iuser,LIST_HEAD))) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't set up recipient list");    
    goto tagdone;
  }

  while (zkListGet(err,iuser,(void **) &iuser_addr,LIST_FORWARDS)) {
    if (strcmp(iuser_addr, "mail2news") == 0) {
      ret = 1;
      goto tagdone;
    }
  }

  if (!zkEmailNew(err, &email)) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't create email object");    
    goto tagdone;
  }

  if (!(zkMemBufNew(err, &membuf) &&
        zkEmailHeaderDump(err, sctx->header, emailDumpToMemBuf, membuf) &&
        zkMemBufGetData(err, membuf, (void **) &tmp, &sz))) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't get dump of mail header");    
    goto tagdone;
  }

  /*
   * Add the header to the new mail object
   */
  if (!zkEmailAddText(err, email, tmp, sz)) {
    zkLogWriteContext(frMaipContext->log,err,func,
                      "Can't add header text to mail object");    
    goto tagdone;
  }

  /*
   * We don't need the buffer used by the header any more
   */
  (void) zkMemBufFree(NULL, &membuf);
  tmp = NULL;

  /*
   * Add the original message body to the new mail object
   */
  if (!zkEmailAddText(err, email, mesg, msz)) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't add text to mail object");    
    goto tagdone;
  }

  /*
   * Turn the new e-mail into something we can work with
   */
  if (!zkEmailParse(err, email)) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't parse mail");    
    goto tagdone;
  }

  if (!zkEmailConvertToMIME(err, email)) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't convert mail to MIME");    
    goto tagdone;
  }

  /* At this point the message is in MIME format */
  if (!zkEmailUtilAppendBlipvert(err,email,
                                 sctx->tagline.text.data,
                                 sctx->tagline.text.size,
                                 sctx->tagline.html.data,
                                 sctx->tagline.html.size)) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't append blip-vert");    
    goto tagdone;
  }

  /* Replace the header in the session context structure */
  if (!(zkEmailHeader(err, email, &eheader) &&
        zkEmailHeaderFree(err, &sctx->header) &&
        zkEmailHeaderDup(err, eheader, &sctx->header))) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't replace header object");    
    goto tagdone;
  }

  /* Dump out the tagged message */
  if (!(zkMemBufNew(err, &membuf) &&
        zkEmailDump(err, email, emailDumpToMemBuf, membuf) &&
        zkMemBufAppend(err, membuf, (const void *) "\0", 1) &&
        zkMemBufGetData(err, membuf, (void **) &tmp, tsz))) {
    zkLogWriteContext(frMaipContext->log,err,func,"Can't dump tagged message");    
    goto tagdone;
  }

  /* Return the new message */
  {
    char *s_ptr = NULL;

    if ((s_ptr = strstr(tmp, "\015\012\015\012")) == NULL) {
      zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "Not 822 header");
      goto tagdone;
    }

    if ((*tagged = zkCStrDup(&s_ptr[4])) == NULL) {
      zkErrorSet_va(err, func, ZKERR, ZKMMAIP, ZKENOMEM, "Can't dup message");
      goto tagdone;
    }
    *tsz = strlen(*tagged);
  }

  ret = 1;
 tagdone:

  if (email) {
    (void) zkEmailFree(NULL, &email);
  }

  if (iuser) {
    (void) zkListFree(NULL, &iuser);
  }

  if (membuf) {
    (void) zkMemBufFree(NULL, &membuf);
  }

  return ret;
}
#endif

int
amtpCommandTarget (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
#define MSGID_SZ  48
#define BLOCK_MSG_SZ 128   /* For when a nym has been blocked by us. */
#define QUEUE_MSG_SZ 256   /* As the target command progresses, it figures out
                            * the names/email addresses of the sender and 
                            * recipients.  This variable is used to create the
                            * queuing log message so that it can be written
                            * when the message is actually queued.
                            */
#ifdef ZK_FMG
  unsigned char msgid[MSGID_SZ];
  char * encr_msgid = NULL; /* Processed version of the message ID */
  char * s_ptr = NULL; /* Temporary string pointer for cleaning strings */
  int msgs = 0; /* Count of message successfully sent */
  int rb_count = 0; /* The target nym reply block count */
  int recipients = 0; /* How many addresses will be receiving the message */
  int ok = 0; /* Is it ok for the FMG to send message */
  unsigned char * rb_text = NULL; /* Reply block text from the nym */
  size_t rb_text_sz = 0; /* Length of the reply block text */
  char queue_msg[QUEUE_MSG_SZ]; /* As the target command progresses, it 
                                 * figures out the names/email addresses of 
                                 * the sender and recipients.  This variable 
                                 * is used to create the queuing log message 
                                 * so that it can be written when the message 
                                 * is actually queued.
                                 */
  char * headerText = NULL;
  size_t headerText_sz = 0;
#endif /* ZK_FMG */

  static const char * func = "amtpCommandTarget";
  FrMaipSessionContext * sctx = (FrMaipSessionContext *) data;
  int parameters = 0;
  int target = 0;
  const unsigned char * pval = NULL;
  unsigned char * target_text = NULL;
  size_t target_text_sz = 0;
  ZkList iuser = NULL; /* List of internet email addresses */
  FrCryptCrypt nym_pkcrypt = NULL; /* Target nym public cryption key */
  FrCryptSig nym_pksig = NULL; /* Target nym public signature key */
  int return_code = _SESSION_CONTINUE;


  zkErrorClear(err);
  if (session && sctx) {
    /*
     * Did we specify a header and message first?  If not, don't continue...
     */

    if (!((sctx->flags & FLAG_SCTX_HEADER_SET) &&
          (sctx->flags & FLAG_SCTX_MESSAGE_SET))) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                         "You must specify a header and message first","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    /*
     * Get the parameter, it will identify the target type
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext(frMaipContext->log,err,func,
                        "Can't retrieve parameter count");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 1) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                    "Expected parameter is the target type (REPLYBLOCK, INTERNET or NYM)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } else if (!zkipSessionParameter(err,session,0,&pval)) {
      zkLogWriteContext(frMaipContext->log,err,func,"Can't retrieve target type");    
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to retrieve target type parameter",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    }
    if (strcasecmp((const char *)pval,"REPLYBLOCK") == 0) {
      target = AMTP_TARGET_REPLYBLOCK;
    } else if (strcasecmp((const char *)pval,"NYM") == 0) {
      target = AMTP_TARGET_NYM;
    } else if (strcasecmp((const char *)pval,"INTERNET") == 0) {
      target = AMTP_TARGET_INTERNET;
    } else {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unrecognized target type (should be REPLYBLOCK, NYM, or INTERNET)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    /*
     * Get the target text
     */

    if (!zkipSessionReadData(err,session,&target_text,&target_text_sz)) {
      zkLogWriteContext(frMaipContext->log,err,func,"Can't retrieve target data");    
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Incorrect data block",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    frMaipStatusIncrementReplyblockRx(err, sctx->status, target_text_sz);

    if (target == AMTP_TARGET_NYM) {
#ifndef ZK_FMG
      /*
       * Only the FMG can resolve a nym to a reply block and forward the mail
       */

      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                       "Cannot query reply block (I am not a primary gateway)",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
#else ZK_FMG
      FrRemoteNym nym = NULL;
      NymServer nymserver;
      NymServerResponse nsresp;
      unsigned const char * tgt_domain = NULL;
      const unsigned char * real_domain = NULL;
      size_t real_domain_len = 0;
      size_t tgt_domain_len = 0;
      

      /*
       * Set the alternate text
       */

      sctx->alttext = frMaipContext->alttext;
      sctx->alttext_sz = frMaipContext->alttext_sz;

      /* */

      if (frMaipContext->nshost == NULL) {
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                         "No nym server specified (I am not a primary gateway)",
                         "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /* 
	 Get the nymserver information here instead so we can pass it to
	 amtpTransmissionAllowed with MIPAS capability,
         store it in the context
      */
      if (!frMaipNetInfoGetType(err, frMaipContext->ni, FRENT_TYPE_NYM_SRV,
				&frMaipContext->nshost, &frMaipContext->nsport, NULL)) {
	  zkLogWriteContext(frMaipContext->log,err,func,
			    "frMaipNetInfoGetType failed (nym server)");
	  return 0;
      }

      nymserver.timeout = frMaipContext->nstimeout;
      nymserver.sigkey = NULL;


      /*
       * Clean up the nym name.  If it is formatted as an email address,
       * save the mail domain for later validation, and chop off the
       * @domain part)
       */

      s_ptr = strchr(target_text,'@');
      if (s_ptr != NULL) {
        *s_ptr = (char) 0;
        tgt_domain = s_ptr + 1;
      }  

      /*
       * Check to see if we're allowed to send to the nym (did the sender
       * exceed its limit?)
       */

      if (!amtpTransmissionAllowed(err,frMaipContext,&(sctx->sender),1,&ok)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "amtpTransmissionAllowed failed");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to determine if sender is allowed "
                            "to transmit message",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } else if (!ok) {
        zkLogWrite_va(frMaipContext->log,"Daily quota exceeded for %s",
                      sctx->sender.name);
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Daily email limit exceeded",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * Is this transaction in the nym -> address blocked database?
       */

      if (!amtpCheckNymBlockDatabase(err,frMaipContext,
                                     (const char *) &(sctx->sender.name),
                                     (const char *) target_text,
                                     AMTP_TARGET_NYM,&ok)){
        zkLogWriteContext(frMaipContext->log,err,func,
                          "amtpCheckNymBlockDatabase failed");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to determine if this mail transaction is"
                            " allowed",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } else if (!ok) {
        char block_msg[BLOCK_MSG_SZ];
        zkLogWrite(frMaipContext->log,"Message blocked");
        snprintf(block_msg,BLOCK_MSG_SZ,
                 "You are not allowed to send to the nym %s",
                 (char *) target_text);
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,block_msg,
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /* 
       * Set the target nym server
       */
    
      if (!frRemoteNymGet(err,&nym,(char *)target_text,&nymserver,&nsresp)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to retrieve target nym data");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Failed to retrieve nym data","NODATA",
                            ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }


      /*
       * Now check that the e-mail domain ascribed to the target nym actually matches the
       * nym's real email domain.  We will consider it a match if the domain is either a
       * perfect match of is more specialized then the real domain.  For instance, if the
       * target nym is foo@beta.freedom.net and foo has been upgraded to a non-beta nym
       * and is now really foo@freedom.net ... this mail should go through.  But, if the
       * user is sending to foo@freedom.net but foo's real addresss is foo@beta.freedom.net
       * then the e-mail should not go through.  Thus, we check to see if the target mail
       * domain ends with the real mail domain.
       */

      if (tgt_domain != NULL) {
        tgt_domain_len = strlen(tgt_domain);
        
        if (!frRemoteNymGetParameter(err,nym,NymEmailDomain,&real_domain,&real_domain_len)) {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Can't retrieve email domain");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Failed to retrieve recipient nym's email domain","NODATA",
                              ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        if  ((tgt_domain_len < real_domain_len)
             || (strcasecmp(real_domain,tgt_domain+tgt_domain_len-real_domain_len))) {
          zkLogWrite(frMaipContext->log, "e-mail domains are mismatched");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Incorrect e-mail domain for recipient nym","NODATA",
                              ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
      }
      
      /*
       * If the nym has a blocklist, make sure the sender is not in it.
       * If the sender of the message is a nym, then it is represented as
       * nym@freedom.net, otherwise, whatever was in the From: field of
       * the AMTP Header command will be used as the sender email string.
       */

      {
#define SENDER_EMAIL_SZ 128
        int blocked = 0;
        char sender_email_buf[SENDER_EMAIL_SZ];
        char * sender_email = NULL;
        ZkEmailField field = NULL;
        int is_internet = 0;

        if ( (sctx->sender.type == FRENT_TYPE_NYM)
             || (sctx->sender.type == FRENT_TYPE_NYM_SRV)) {
          sender_email = sctx->sender.name;
        } else {
          zkEmailHeaderGetField(err,sctx->header,From,&field);
          zkEmailFieldValue(err,field,(const unsigned char **)&sender_email);
          snprintf(sender_email_buf,SENDER_EMAIL_SZ,"%s",sender_email);
          zkEmailUtilCleanAddress(err,sender_email_buf);
          sender_email = sender_email_buf;
          is_internet = 1;
        }      
        if (!amtpCheckBlocklist(err,nym,sender_email,&blocked)) {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Failed to check block list - assuming blocked");
          blocked = 1;
        }
        if (blocked) {
          zkLogWrite(frMaipContext->log,
                     "Mail blocked by recipient");
          zkipSessionResponse(err,session,0,_COMMAND_NOTEXECUTED,
                              "Recipient nym has refused message","NODATA",
                              ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }
        snprintf(queue_msg,QUEUE_MSG_SZ,
                 "QUEUEING message from %s to nym",
                 (is_internet ? "internet" : sender_email));
#undef SENDER_EMAIL_SZ
      }


      /* 
       * Get the nym keys...
       */

      frRemoteNymResetParameterSearch(err,nym);
      nym_pksig = NULL;
      nym_pkcrypt = NULL;
      if (!frRemoteNymGetKeys(err,nym,&nym_pksig,&nym_pkcrypt)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Target nym has no public keys!");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Target nym does not have public keys","NODATA",
                            ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * Make a clean message header for the nym.  We dont want anything
       * going out...
       */
 
      zkEmailHeaderFree(err,&(sctx->header));
      if (!(zkEmailHeaderNew(err,&(sctx->header)) &&
            zkEmailHeaderAddFieldData(err,sctx->header,"From",
                                      "fmg@freedom.net") &&
            zkEmailHeaderAddFieldData(err,sctx->header,"Subject",
                                      "Freedom Encrypted Message") &&
            zkEmailHeaderAddFieldData(err,sctx->header,"X-ZKS-Freedom-Comment",
                               "This message was processed by the ZKS FMG"))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Can't create clean message header");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to construct message header","NODATA",
                             ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } 

      /*
       * Queue a copy of the message for each reply block
       */
      
      frRemoteNymResetParameterSearch(err,nym);
      msgs = 0;
      rb_count = 0;
      frMaipGenerateMessageID(err,msgid,MSGID_SZ - 1);
   
      while (frRemoteNymGetParameter(err,nym,NymReplyBlock,
                                     (const unsigned char **)
                                     &rb_text,&rb_text_sz)) {
        FrCryptPubKey mySigKey;

        /*
         * If the sending entity is a nym server than we're sending a reply
         * block activation token, and the nym server has provided a hash
         * of the target reply block which we're supposed to use.  Check if
         * this reply block is the right one.  If not ... move on to the
         * next reply block.
         */
        if (sctx->sender.type == FRENT_TYPE_NYM_SRV) {
          int match = 0;
          if ( !frCryptHashVerify(err,rb_text,rb_text_sz,
                                  sctx->tgt_rb_hash,sctx->tgt_rb_hash_sz,
                                  &match)) {
            zkLogWriteContext(frMaipContext->log, err, func,
                              "Failed to calculate reply block hash");
            continue;
          } else if (!match) {
            continue;
          }
        }

        /*
         * The sender is not a nym server.  We only want to use active reply
         * blocks to send this message.  Check if this reply block is active.
         * If it isn't ... move on to the next reply block.
         */
        else {
          int is_active = 0;
          if ( !frMaipRbPubIsActive(err,rb_text,rb_text_sz, &is_active) ) {
            zkLogWriteContext(frMaipContext->log, err,func,
                              "Couldn't check reply block activation status");
            continue;
          } else if ( !is_active ) {
            if (frMaipContext->flags & FLAG_DEBUG) {
              zkLogWrite(frMaipContext->log, 
                         "Skipping inactive reply block");
            }
            continue;
          }
        }

        rb_count++;
        
        /*
         * Generate a suitable message ID before queueing the message
         */
        if (!frSecKeyChainGetVersion(err, frMaipContext->keys, 
                                     frMaipContext->sigHandle, 0,
                                     &mySigKey)) {
          zkLogWriteContext(frMaipContext->log,err,func,
                     "Couldn't get key version from key chain");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Unable to get my key version","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        if (!(frMaipProcessMessageID(err,msgid,MSGID_SZ - 1,&encr_msgid,
                                      nym_pkcrypt,mySigKey) &&
              zkEmailHeaderAddFieldData(err,sctx->header,
                                        "X-ZKS-Freedom-MessageID",encr_msgid))){
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Failed to add freedom message id");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Unable to assign a message ID","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        } 
        sctx->target = rb_text;
        sctx->target_sz = rb_text_sz;

        /*
         * This could be a bit misleading.  We report success if at least 
         * one copy of the message got out.  There should be a better warning 
         * provided to the client if one of the amtpSend() calls failed.
         */

        if (!amtpSend(err,frMaipContext,sctx)) {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Failed to queue message");
          frMaipStatusIncrementDropped(err, sctx->status);
        } else {
          frMaipStatusIncrementRcvd(err, sctx->status);
          frMaipStatusIncrementQueued(err, sctx->status);
          msgs++;
        }
        zkEmailHeaderDeleteFieldById(err,sctx->header,X_ZKS_Freedom_MessageID);
      }
      if (rb_count < 1) {
        /*
         * BUG?
         * ----
         *
         *     Mail sent to a recipient who has no active reply blocks is simply
         *     discarded!
         *
         *     A better, but more costly solution would involve redistributing
         *     mail sent to an active-reply-block deficient nym over the system
         *     so that it eventually makes its way back to the FMG-Maip for a
         *     retry.
         *
         *     Oh well ... we'll do that later, I guess.
         */
        if (frMaipContext->flags & FLAG_DEBUG) {
          zkLogWrite(frMaipContext->log,
                     "Target nym does not have active replyblocks");
        }
      }
      if (msgs < 1) {
        zkLogWrite(frMaipContext->log,
                   "Unable to deliver message");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Failed to queue message, invalid or missing "
                            "replyblocks",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * Log the queueing message
       */

      zkLogWrite(frMaipContext->log,queue_msg);

      if (sctx->message) {
        free(sctx->message);
        sctx->message = NULL;
      }
#endif /* ZK_FMG */
    } else if (target == AMTP_TARGET_REPLYBLOCK) {

      /*
       * The FMG maip is not allowed to accept a reply block
       */

#ifdef ZK_FMG
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                       "Cannot accept a reply block (I am an FMG)",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
#else /* !ZK_FMG */
      /*
       * If the target is a reply block then we just pass the text that 
       * we received
       */

      sctx->target = target_text;
      sctx->target_sz = target_text_sz;
      if (!amtpSend(err,frMaipContext,sctx)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "amtpSend failed to queue message");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to queue message","NODATA",
                            ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } 

      frMaipStatusIncrementRcvd(err, sctx->status);
      frMaipStatusIncrementQueued(err, sctx->status);

      if (sctx->message) {
        free(sctx->message);
        sctx->message = NULL;
      }

#endif /* !ZK_FMG */
    } else if (target == AMTP_TARGET_INTERNET) {
      /*
       * Only the FMG MAIP can accept an Internet email address as a
       * target.  Standard maips will queue the message and will obtain
       * an email address when the open up the reply block.
       */
#ifndef ZK_FMG
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                       "Cannot accept an Internet email address"
                       " (I am a normal MAIP)",
                       "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
#else /* ZK_FMG */
      ZkEmail email = NULL;
      ZkEmailHeader header = NULL;
      
      /* Set the tag line(s) */
      sctx->tagline.text.data = frMaipContext->tagline.text.data;
      sctx->tagline.text.size = frMaipContext->tagline.text.size;
      sctx->tagline.html.data = frMaipContext->tagline.html.data;
      sctx->tagline.html.size = frMaipContext->tagline.html.size;

      /*
       * Set up an internal email object and its header so we can split the
       * message text
       */

      if (!(zkEmailNew(err,&email) &&
            zkEmailHeader(err,email,&header))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to create new header for internet mail");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Internal data structure error",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * First we need to remove the signatures from the message (they're
       * useless to internet users) There should be only 2 signatures.
       */

      if (!frSecSigRemoveInSitu(err,sctx->message,&(sctx->message_sz))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to remove sender signature");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to remove sender signature",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      if (!frSecSigRemoveInSitu(err,sctx->message,&(sctx->message_sz))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to remove FMG signature");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to remove FMG signature",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      /*
       * Remove trailing NULs from the message
       */

      if (!amtpDeleteTrailingNuls(err,sctx->message,&(sctx->message_sz))) { 
        zkLogWriteContext(frMaipContext->log,err,func,
                          "amtpDeleteTrailingNuls failed");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Internal MAIP error",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } 

      /*
       * Now we have what should be a clear message to send to the 'net
       * so pull the header out and add it to the header we got with the
       * AMTP 2.1.0 header command.  This allows us to process the mail in
       * the usual way.
       */

      headerText = sctx->message;

      if ((s_ptr = strstr(sctx->message,"\015\012\015\012")) == NULL) {
#if 0
      /* Note.  I'm not sure that I want to croak at this point.  If we
       * have a valid target, we could still try to deliver the mail even
       * though it does not seem to contain an 822 header
       */
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Email message contained no header",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
#endif
      } else {
       
        /*
         * Warning:  This whole thing is a fine example of "bad programming"
         *           I'm  essentially losing the pointer to the beginning of
         *           the original message text here.  This will cause 
         *           problems on cleanup
         */
        *s_ptr = (char) 0;
        headerText_sz = strlen(headerText);
        sctx->message = &(s_ptr[4]);
        sctx->message_sz = sctx->message_sz - headerText_sz - 4;

        /*
         * Make sure the header text itself does not have trailing NULs
         */

        if (!amtpDeleteTrailingNuls(err,headerText,&headerText_sz)) { 
          zkLogWriteContext(frMaipContext->log,err,func,
                            "failed to remove trailing nulls from header");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Internal MAIP error",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        } 

        if (!(zkEmailAddText(err,email,headerText,headerText_sz) &&
              zkEmailAddText(err,email,"\015\012\015\012",4) &&
              zkEmailHeaderMerge(err,sctx->header,header) &&
              zkEmailHeaderAddFieldData(err,sctx->header,
                                        "Comments",
                                        "This message was processed by the " \
                                        "Freedom Mail Gateway"))) {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Failed to complete internet mail header");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Could not reconstruct original message header",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        } 
      }

      /*
       * At this point we no longer need the extra email object, so nuke it.
       * This operation also gets rid of the associated email header.
       */

      zkEmailFree(NULL,&email);
      header = NULL;

      /* Add the tagline text */
      {
        char *newmsg = NULL;
        size_t newsz = 0;

        if (!maipAppendTagLine(err, sctx, sctx->message, sctx->message_sz, 
                            &newmsg, &newsz, target_text)) {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "Failed to append tag line");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Internal data structure error",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;
        }

        if (newmsg) {
          sctx->message = (unsigned char *) newmsg;
          sctx->message_sz = newsz;
          newsz = 0;
          newmsg = NULL;

          /*
           * headerText points to the beginning of the old sctx->message
           * buffer.  We've got to free it so that (1) we don't leak this
           * memory and (2) so that the cleanup code in this function knows
           * NOT to free the new memory that we've allocated for sctx->message
           * too soon.
           */
          free(headerText);
          headerText = NULL;
        }
      }

      /*
       * Now that we've merged the message headers together, we can create
       * a recipient list.
       */

      if (!(zkListNew(err,&iuser) &&
            zkListSetDataFreeFunction(err,iuser,zkListDataFree,NULL) &&
            zkEmailUtilAddToRecipientList(err,iuser,target_text))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to create recipient list");
        goto finish;
      }
      
      /*
       * Get the number of recipients and see if we are allowed to send
       * to that many.
       */
      zkListSize(err,iuser,&recipients);
      if (!amtpTransmissionAllowed(err,frMaipContext,&(sctx->sender),
                                   recipients,&ok)) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "amtpTransmissionAllowed failed (internet)");
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to determine if sender is allowed "
                            "to transmit message",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } else if (!ok) {
        zkLogWrite_va(frMaipContext->log,"Daily quota exceeded for %s",
                      sctx->sender.name);
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Daily email limit exceeded",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }

      {
        char * target_addr = NULL;
        zkListMoveCursor(err,iuser,LIST_HEAD);
        while (zkListGet(err,iuser,(void **) &target_addr,LIST_FORWARDS)) {
          if (!amtpCheckNymBlockDatabase(err, frMaipContext,
					 (const char *) &(sctx->sender.name), 
                                         (const char *) target_addr,
                                         AMTP_TARGET_INTERNET,&ok)){
            zkLogWriteContext(frMaipContext->log,err,func,
                              "amtpCheckNymBlockDatabase failed (internet)");
            zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                                "Unable to determine if this mail "
                                "transaction is allowed",
                                "NODATA",ZKIP_ENCODING_NONE,0,NULL);
            goto finish;
          } else if (!ok) {
            char block_msg[BLOCK_MSG_SZ];
            zkLogWrite(frMaipContext->log,"Message blocked by recipient");
            snprintf(block_msg,BLOCK_MSG_SZ,
                     "You are not allowed to send to %s",target_addr);
            zkipSessionResponse(err,session,0,_COMMAND_ERROR,block_msg,
                                "NODATA",ZKIP_ENCODING_NONE,0,NULL);
            goto finish;
          }
        }
      }
     

      zkListMoveCursor(err,iuser,LIST_HEAD);
      sctx->target = NULL;
      sctx->target_sz = 0;

      /* Try to add a message ID.  we can use the same one for all inet users */
      msgs = 0;

      /*
       * Bug:  Adding the message ID right now causes a blank space to be
       *       inserted into the header.  Since we dont really need to do this
       *       in the first place (the message ID is meaningless since we dont
       *       keep a list of them) this bug really isnt all that important
       *       --philippe
       */
#if 0
      if (!(frMaipGenerateMessageID(err,msgid,MSGID_SZ) &&
            zkEmailHeaderAddFieldData(err,sctx->header,
                                      "X-ZKS-Freedom-MessageID",msgid))) {
        zkLogWriteContext(frMaipContext->log,err,func,
                          "Failed to add message id (internet)");
/*
 * Bug: this should not fail , but seems to at times...
 * FIX ME
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to assign a message ID","NODATA",
                             ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
 */
      } 
#else
#warning FMG-MAIP currently does not add a zks message ID to internet mail
#endif

      while (zkListGet(err,iuser,(void **) &(sctx->iuser_addr),LIST_FORWARDS)) {
        sctx->iuser_addr_sz = (size_t) strlen(sctx->iuser_addr);
        if (amtpSend(err,frMaipContext,sctx)) {
          zkLogWrite_va(frMaipContext->log,
                        "QUEUEING message from %s to internet",
                        sctx->sender.name);
          msgs++;
          frMaipStatusIncrementRcvd(err, sctx->status);
          frMaipStatusIncrementQueued(err, sctx->status);
        } else {
          zkLogWriteContext(frMaipContext->log,err,func,
                            "FAILED to queue message");
          frMaipStatusIncrementDropped(err, sctx->status);
        }
      }
      if (msgs < 1) {
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Failed to queue messages",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }
#endif /* ZK_FMG */
    } else {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unrecognized target type (should be "
                          "REPLYBLOCK, NYM, or INTERNET)",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }
  

    /*
     *  If we get here, then we've successfully queue some mail (not
     *  nescessarily all of it though, error reporting in this case must
     *  be improved)
     */

    zkipSessionResponse(err,session,0,_SERVICE_READY,
                        "Message(s) succesfully queued","NODATA",
                        ZKIP_ENCODING_NONE,0,NULL);
finish:
    
    if (!return_code) {
#ifdef ZK_FMG
      if (msgs < 1) {           /* Counted above so don't count again */
#endif
        frMaipStatusIncrementDropped(err, sctx->status);
#ifdef ZK_FMG
      }
#endif
    }

# ifdef ZK_FMG
    if (headerText) {
      /*
       * We should get here under the following conditions:
       *
       * 1. an error occurred while creating an internet bound message
       * 2. we did not append a tagline (because none is specified, or
       *    because the message is a news posting)
       *
       * If headerText is non-null then it points to the beginning
       * of what user to be sctx->message.  sctx->message points to
       * an offset into the buffer pointed to by headerText.  We have
       * to make sure that we clean up this mess because if this
       * setup makes it out of this function then the higher level
       * cleanup code will try to free the memory pointed to by
       * sctx->message.  Not a good thing.
       *
       * We can either free the memory here and make sure that the
       * later cleanup code doesn't try to free it, or we can reset
       * the message pointer to the beginning of the allocated buffer.
       */

      free(headerText);
      headerText = NULL;
      sctx->message = NULL; /* won't be freed later */
    }
# endif
    if (nym_pksig) {
      frCryptSigFree(NULL,&nym_pksig);
    }
    if (nym_pkcrypt) {
      frCryptCryptFree(NULL,&nym_pkcrypt);
    }
    if (iuser) {
      zkListFree(NULL,&iuser);
    }
    return return_code;
  }
  zkErrorSet(err,"amtpCommandTarget",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}


int
amtpSend (
  ZkError err ,
  FrMaipContext * context ,
  FrMaipSessionContext * sctx )
{
  static const char * func = "amtpSend";
  int return_code = 0;
  FrMaipMesgBlk msg;
 
  zkErrorClear(err);
  if (context && sctx) {

    memset(&msg,0,sizeof(FrMaipMesgBlk));

    /*
     * Put the header, message and target into a structure which can be
     * processed by Kent's frMaip functions...
     */

    msg.header = sctx->header;
    msg.target = sctx->target;
    msg.tsz = sctx->target_sz;
    msg.message = sctx->message;
    msg.msz = sctx->message_sz; 
    /* Dont specify the alternate text if size or string dont make sense */
    /* Recall that alltext is optional */
    if (sctx->alttext && (sctx->alttext_sz > 0)) {
      msg.alttext = sctx->alttext;
      msg.asz = sctx->alttext_sz;
    } else {
      msg.alttext = NULL;
      msg.asz = 0;
    }

    if (msg.target == NULL) {  /* We want to use an email address instead */
      if (sctx->iuser_addr_sz >= FRMAIP_EMAIL_LEN) {
        zkErrorSet_va(err,"amtpSend",ZKERR,ZKMMAIP,ZKERANGE,
                      "Buffer is to small to pass email address \"%s\"",
                      sctx->iuser_addr);
        return 0;
      }  
      msg.esz = sctx->iuser_addr_sz;
      msg.isLast = 1;
      strncpy(msg.nextHop.email,sctx->iuser_addr,msg.esz);
      msg.nextHop.email[msg.esz] = (char) 0;
    } else {
      FrCryptPubKey myCryptKey = NULL;
      size_t privVers = 0;

      if (!frMaipRbPubGetHopCryptKeyVersion(err, msg.target, msg.tsz,
                                            &privVers)) {
        zkLogWriteContext(context->log,err,func,
                          "Couldn't extract key version from replyblock.\n");
        return 0;
      }

      if (!frSecKeyChainGetVersion(err, context->keys, context->cryptHandle,
                                   privVers, &myCryptKey)) {
        zkLogWriteContext(context->log,err,func,
                          "Couldn't get key version from key chain.\n");
        return 0;
      }

      if (!frMaipProcessIncoming(err, context->mspoolctx, &msg, myCryptKey, 
                                 context->flags & FLAG_FMG)) {
        zkLogWriteContext(context->log,err,func,
                          "frMaipProcessIncoming: failed");
        return 0;
      }
    } 
    if (frMaipSpoolSaveMesg(err,context->spooldir,&msg)) {
      return_code = 1;
    } else {
      zkLogWriteContext(context->log,err,func,
                        "frMaipSpoolSaveMesg: failed");
      return_code = 0;
    }

    if (msg.message != (char *) sctx->message) {
      /* This was overwritten in frMaipProcessIncoming .. we need to
         free it*/
      free(msg.message);
      msg.message = NULL;
    }
    
    if (msg.headerTxt) {
      free(msg.headerTxt);
      msg.headerTxt = NULL;
    }

    return return_code;
  }
  zkErrorSet(err,"amtpSend",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}
