/*
 * $Id: main.c,v 1.1 2000/01/06 21:22:44 roger Exp $
 */

#include "zks.h"

#include <netinet/in.h>
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include "zkError.h"
#include "zkAlloc.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkFile.h"
#include "zkBase64.h"
#include "ZKIP.h"
#include "zkipEngine.h"
#include "zkipSession.h"
#include "zkipClient.h"
#include "frNetInfo.h"
#include "maip.h"
#include "maipstat.h"
#include "amtp.h"
#include "zkParms.h"
#include <netdb.h>
#include "zkDNS.h"
#include <sys/socket.h>
#include <arpa/inet.h>
#include "frNiqs.h"
#include "zkDebug.h"

/*
 *
 */

int
main (
  int argc ,
  char ** argv )
{
  FrMaipContext context; 

  /*   
   * Assign the global variable
   */

  frMaipContext = &context;
  
  /*
   * Initialize the context
   */

  if (!frMaipContextInit(NULL,&context)) { 
    fprintf(stderr,"Unable to initialize application context.\n");
    exit(EXIT_FAILURE);
  }

  /*
   * Load all parameters from the command line and configuration file.  Value
   * on the command line have precedence over what is contained in the config
   * file.
   */

  if (!frMaipContextLoad(context.err,&context,argc,argv)) {
    fprintf(stderr,
            "%s:  Error while loading arguments and parameters, terminating\n",
             context.progname);
    exit(context.exit_code);
  }

  /*
   * Do the context sanity checks
   */

  if (!frMaipContextVerify(context.err,&context)) {
    fprintf(stderr,
            "%s:  Error verifying arguments and parameters, terminating\n",
            context.progname); 
    exit(context.exit_code);
  }

  /* Set up my key chain before I fork */
  if (!frSecKeyChainNew(context.err, &context.keys)) {
    fprintf(stderr, "%s: Error getting a new key chain.\n", context.progname);
    exit(EXIT_FAILURE);
  }

  if (!frSecKeyChainAddSymKey(context.err, context.keys, context.passfile,
                              context.prompt, FRCRYPT_SYMALG_BLOWFISH)) {
    fprintf(stderr, "%s: Error reading passphrase.\n", context.progname);
    exit(EXIT_FAILURE);
  }

  if (context.flags & FLAG_DONTFORK) {
    if (context.pidfile) {
      if (!zkFileWritePid(context.err, context.pidfile, &context.fptr)) {
        fprintf(stderr,"%s:  Unable to write to \"%s\".\n",context.progname,
                context.pidfile);
        exit(EXIT_FAILURE);
      }
    }
  } else {
    pid_t pid = 0;
    if ((pid = fork()) == 0) {
      if (context.pidfile) {
        if (!zkFileWritePid(context.err, context.pidfile, &context.fptr)) {
          fprintf(stderr,"%s:  Unable to write to \"%s\".\n",context.progname,
                  context.pidfile);
          exit(EXIT_FAILURE);
        }
      }
    } else {
      fprintf(stderr,"%d\n",pid);
      exit(EXIT_SUCCESS);
    }
  }

  /* 
   * Now that I have most of the parameters, get the rest of the Freedom 
   * network parameters from the NIQS server.
   */
  if (!frMaipNetInfoInit(context.err, &context)) {
    fprintf(stderr,
            "%s: Error setting up Freedom network parameters, terminating\n",
            context.progname); 
    exit(context.exit_code);
  }

  if (!frMaipSetSpoolParms(context.err, &context)) {
    fprintf(stderr,
            "%s:  Error setting up spool parameters, terminating\n",
            context.progname); 
    exit(context.exit_code);
  }

  if (!frSecKeyChainRegisterDir(context.err, context.keys, &context.sigHandle, 
                                context.sigDir)) {
    fprintf(stderr, "%s: Error setting signature key directory [%s].\n",
            context.progname, context.sigDir);
    exit(EXIT_FAILURE);
  }
  if (!frSecKeyChainRegisterDir(context.err, context.keys, 
                                &context.cryptHandle,
                                context.cryptDir)) {
    fprintf(stderr, "%s: Error setting cryption key directory [%s].\n",
            context.progname, context.cryptDir);
    exit(EXIT_FAILURE);
  }

  if (!frSecKeyChainReloadKeys(context.err, context.keys)) {
    fprintf(stderr, "%s: Couldn't load all my private keys.\n",
            context.progname);
    exit(EXIT_FAILURE);
  }

  if (!maipSetUpStat(context.err, &context)) {
    fprintf(stderr, "%s: Error setting up stat parameters.\n", 
            context.progname);
    exit(context.exit_code);
  }

  /* 
   * Now that we have all the parameters, split off into two processes
   */
  if ((context.queue_pid = fork()) == 0) {
    if (!frMaipAMTPQueue(context.err,&context)) {
      fprintf(stderr,"%s:  Failed to run AMTP queue.\n",context.progname);
      exit(EXIT_FAILURE);
    }
    exit(EXIT_SUCCESS);
  } else { 
    if (!frMaipAMTPServer(context.err,&context)) {
      fprintf(stderr,"%s:  Failed to run AMTP server.\n",context.progname);
    } 
  }

  kill(context.queue_pid, SIGTERM);

  (void) frMaipCleanContext(context.err, &context);

  exit(context.exit_code);
  return 0; /* to shut up insure */
}
