/*
 * $Id: maip.c,v 1.78 2001/04/19 19:23:35 alainc Exp $
 */

#include "zks.h"

#include <netinet/in.h>
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include "zkError.h"
#include "zkAlloc.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkFile.h"
#include "zkBase64.h"
#include "ZKIP.h"
#include "zkipEngine.h"
#include "zkipSession.h"
#include "zkipClient.h"
#include "frNetInfo.h"
#include "maip.h"
#include "maipstat.h"
#include "amtp.h"
#include "zkParms.h"
#include <netdb.h>
#include "zkDNS.h"
#include <sys/socket.h>
#include <arpa/inet.h>
#include "frNiqs.h"
#include "zkDebug.h"
#include "zkCStr.h"


static const char * const
module_id_ = "$Id: maip.c,v 1.78 2001/04/19 19:23:35 alainc Exp $";

/*
 * Internal function prototypes
 */

static int frMaipUsage ( FrMaipContext * , FILE * );
static int frMaipCredits ( FILE * );

/*
 * Stuff 
 */

# define DEFAULT_NI_SHARED_MEM_SIZE (500)
# define MINIMUM_NI_SHARED_MEM_SIZE (500)
# define BYTES_PER_KILOBYTE  (1024)

/*
 *
 */

static int
frMaipUsage (
  FrMaipContext * context ,
  FILE * stream )
{
  if (!context) {
    return 0;
  }
  if (stream == NULL) {
    stream = stderr;
  }
  fprintf(stream,"%s\n\nUsage:  %s options...\n"
          "\n%s\n\tSends debugging information to stderr\n"
          "\n%s config_file\n\tSpecifies the configuration file\n"
          "\n%s\n\tDisplays this help screen\n"
          "\n%s port_number\n\tThe MAIP listening port (default is %d)\n"
          "\n%s\n\tDisplay the version string of the server\n"
          "\n%s pid_file\n\tSpecifies a file for the process pid\n"
          "\n%s unique_name\n\tSpecifies a unique name for the server\n"
          "\n%s integer\n\tMaximum amount of data per AMTP command\n"
          "\n%s verbose_level\n\tVerboseness of log files (default is %d)\n"
          "\n%s duration\n\tMax connection duration in secs (default is %d)\n"
          "\n%s connections\n\tMax num of simultaneous conn (default is %d)\n"
          "\n%s yes | no\n\tRun the server in daemon mode (default is yes)\n"
          "\n%s log_file\n\tSpecifies the server log file\n"
#ifdef ZK_FMG
          "\n%s seconds\n\tNym server timeout (default is %d, FMG option)\n"
#endif
          "\n%s spooldir\n\tSpecify the spool directory (default %s)\n"
          "\n%s maxclients\n\tClients to run in parallel (default %d)\n"
          "\n%s queuetime\n\tQueue run interval [in secs] (default %d)\n"
          "\n%s mxmsgperloop\n\tMaximum number of processed messages per queue run (default %d)\n"
          "\n%s mta_path\n\tPath to Mail Transfer Agent (default %s)\n"
          "\n%s mta_argument\n\tOption to pass to MTA (multiple instances)\n"
#ifdef ZK_FMG
          "\n%s file\n\tAlternate message text file (FMG only)\n"
          "\n%s file\n\tTagline text file (FMG only)\n"
          "\n%s file\n\tTagline HTML file (FMG only)\n"
          "\n%s keyquerytimeout\n\tTimeout for key lookup (FMG only)\n"
          "\n%s keyqueryretries\n\tNum of retries for key lookups (FMG only)\n"
#endif
          "\n%s dbname\n\tMail usage statistics database [default %s]\n"
          "\n%s dbname\n\tNym to blocked email address database [default %s]\n"
          "\n%s cryption dir\n\tStorage directory of private crypto keys\n"
          "\n%s signature dir\n\tStorage directory of private sig keys\n"
          "\n%s passphrase_prompt\n\tPrompt for system passphrase\n"
          "\n%s passphrase_file\n\tPassphrase file\n"
          "\n%s niqshostname\n\tNIQS hostname\n"
          "\n%s niqsport\n\tNIQS port\n"
          "\n%s niqstimeout\n\tNIQS timeout\n"
          "\n%s niqsfrequency\n\tNIQS frequency of updates in secs\n"
          "\n%s buffersize\n\tNIQS shared memory buffer size (in kilobytes)\n"
          "\n%s sharedmemorytemplate\n\tNIQS shared memory filename template\n"
          "\n%s mutexttemplate\n\tNIQS mutex filename template\n"
          "\nCommand line options override settings in the config file\n",
          VERSION_AMTP_STR,context->progname,
          OPT_DEBUG,
          OPT_CONFIG,
          OPT_HELP,
          OPT_PORT,DEFAULT_PORT,
          OPT_VERSION,
          OPT_PIDFILE,
          OPT_NAME,
          OPT_MAXDATA,
          OPT_VERBOSE,DEFAULT_VERBOSE,
          OPT_MAXCONNTIME,DEFAULT_MXCONNTIME,
          OPT_MAXCONN,DEFAULT_MXCONN,
          "--"P_DAEMON,
          OPT_LOGFILE,
#ifdef ZK_FMG
          OPT_NYMSERVTOUT,DEFAULT_NSTIMEOUT,
#endif
          OPT_SPOOLDIR, DEFAULT_SPOOLDIR,
          OPT_MAXCLIENTS, DEFAULT_MAXCLIENTS,
          OPT_QUEUETIME, DEFAULT_QUEUETIME,
          OPT_MAXMSGPLOOP, DEFAULT_MAXMSGPLOOP,
          OPT_MTAPATH, DEFAULT_MTA,
          OPT_MTAOPT,
#ifdef ZK_FMG
          OPT_ALTTEXT,
          OPT_TAGLINE_FILE_TEXT,
          OPT_TAGLINE_FILE_HTML,
          OPT_KEYQRYTOUT,
          OPT_KEYQRYRETR,
#endif
          OPT_STATDB, DEFAULT_STATDB,
          OPT_NYMBLOCKDB, DEFAULT_NYMBLOCKDB,
          OPT_CRYPTDIR,
          OPT_SIGDIR,
          OPT_PPPROMPT,
          OPT_PPFILE,
          OPT_NIQS_SERVER,
          OPT_NIQS_PORT,
          OPT_NIQS_TIMEOUT,
          OPT_NIQS_FREQUENCY,
          OPT_NIQS_SHARED_MEM_SIZE,
          OPT_NIQS_SHMTEMPLATE,
          OPT_NIQS_MUTEXTEMPLATE);
  return 1;
}

static int
frMaipCredits (
  FILE * stream )
{
  unsigned char * credits = NULL;
  unsigned char * credit = NULL;
  size_t size = 0;
#include "mlist.h"

  (void)module_id_; /* To silence compiler warning about
                     * module_id_ not being used.
                     */

  credits = (unsigned char *)strdup(CREDIT_STR);
  credit = credits;
  while (*credit) *credit++ &= 0x7F;
  size = strlen(credits);
  if ((credit = (unsigned char *) malloc(size)) == NULL) {
    return 0;
  }
  if (zkConvertFromBase64(NULL,credits,credit,&size)) {
    fprintf(stream,"%s\n",credit);
  }
  free(credits);
  free(credit);
  return 1;
}

/*
 * 
 */

int
frMaipContextInit (
  ZkError err ,
  FrMaipContext * context )
{
  zkErrorClear(err);
  if (context) {
    memset(context,0,sizeof(FrMaipContext));  

    /*
     * Create required datastructures for the application context
     */

    if (!zkErrorNew(&(context->err))) {
      return 0;
    }
    if (!zkLogNew(&(context->log))) {
      return 0;
    }
    if (!zkConfigNew(context->err,&(context->config))) {
      return 0;
    }
    context->status = NULL;
  
    context->queue_pid = 0;

    /*
     * These are initially set to NULL because only the listening part of
     * the maip requires them.  They will be allocated after the fork()
     */

    context->session = NULL;
    context->engine = NULL;

    context->flags = 0;
    context->exit_code = EXIT_FAILURE;
    context->progname = DEFAULT_PROGNAME;
    context->configfile = DEFAULT_CONFIGFILE;
    context->pidfile = DEFAULT_PIDFILE;
    context->port = (unsigned short) DEFAULT_PORT;
    context->name = DEFAULT_NAME;
    context->maxdata = DEFAULT_MAXDATA;
    context->verbose = DEFAULT_VERBOSE;
    context->mxconn = DEFAULT_MXCONN;
    context->mxconntime = DEFAULT_MXCONNTIME;

    context->maillifetime = DEFAULT_MAIL_LIFETIIME;

#ifdef ZK_FMG
    context->nshost = DEFAULT_NSHOST;
    context->nsport = DEFAULT_NSPORT;
    context->nstimeout = DEFAULT_NSTIMEOUT;
    context->alttextfile = DEFAULT_ALTTEXTFILE;
    context->alttext = NULL;

    context->tagline.text.file = DEFAULT_TAGLINE_FILE_TEXT;
    context->tagline.text.data = NULL;
    context->tagline.text.size = 0;

    context->tagline.html.file = DEFAULT_TAGLINE_FILE_HTML;
    context->tagline.html.data = NULL;
    context->tagline.html.size = 0;

    context->statdb = NULL;
    context->nymblockdb = NULL;
#endif

    context->kqtimeout = DEFAULT_KEYQRYTOUT;
    context->kqretries = DEFAULT_KEYQRYRETR;
    context->kh.fd = -1;        /* Not initialized */
    memset(&context->kh.host, 0, sizeof(context->kh.host));
    context->kh.host.sin_family = AF_INET;

    context->khport = (unsigned short) DEFAULT_KEYQRYPORT;
    context->khhost = DEFAULT_KEYQRYHOST;
    context->kq = NULL;

    context->spooldir = DEFAULT_SPOOLDIR;
    context->maxclients = DEFAULT_MAXCLIENTS;
    context->queuetime = DEFAULT_QUEUETIME;
    context->mxmsgperloop = DEFAULT_MAXMSGPLOOP;
    context->mtaPath = DEFAULT_MTA;
    context->mtaOpts = NULL;
    context->statdbname = DEFAULT_STATDB;
    context->nymblockdbname = DEFAULT_NYMBLOCKDB;

    context->statTime = 5;
    context->statHistory = 0;
    context->statServer = NULL;
    context->statPort = 0;

    context->status = NULL;

    context->cryptDir = NULL;
    context->sigDir = NULL;
    context->keys = NULL;

    /* Arbitrary handles numbers, just need them to be associated */
    context->cryptHandle = 0;
    context->sigHandle = 0;

    context->prompt = NULL;
    context->passfile = NULL;

    context->fptr = NULL;

    context->mspoolctx = NULL;
    context->ni = NULL;

    context->niCtx = NULL;
    context->reloadNiTime = DEFAULT_NI_FREQUENCY;

    context->niqsUpdateOffset = 0;
    context->niqsHost = NULL;
    context->niqsPort = 0;
    context->ni_shared_mem_sz = DEFAULT_NI_SHARED_MEM_SIZE * BYTES_PER_KILOBYTE;
    context->shmtemplate = NULL;
    context->mutextemplate = NULL;

    return 1;
  }
  zkErrorSet(err,"frMaipContextInit",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 * 
 */

int
frMaipContextLoadConfig (
  ZkError err ,
  FrMaipContext * context )
{
  int return_code = 1;
  const char * pval = NULL;
  const char * sec = SEC1;

  zkErrorClear(err);
  if (context) {
    if (context->flags & FLAG_FMG) {
      sec = SECFMG;
    }
    if (context->configfile == NULL) {
      zkErrorSet(err,"frMaipContextLoadConfig",ZKERR,ZKMMAIP,ZKENULLPTR,
                 "Configuration file name is set to NULL");
      return 0;
    } 

    if (!zkConfigLoad(err,context->config,context->configfile)) {
      return 0;
    }

    if (zkConfigGet(err,context->config,sec,P_PASSPHRASE_PROMPT,&pval)) {
      context->prompt = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_PASSPHRASE_FILE,&pval)) {
      context->passfile = pval;
    }
   
    if (zkConfigGet(err,context->config,sec,P_PORT,&pval)) {
      context->port = (unsigned short) atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec,P_PIDFILE,&pval)) {
      context->pidfile = pval;
    }
   
    if (zkConfigGet(err,context->config,sec,P_SERVER_NAME,&pval)) {
      context->name = pval;
    }
   
    if (zkConfigGet(err,context->config,sec,P_MAX_DATA,&pval)) {
      context->maxdata = atoi(pval);
    }
  
    if (zkConfigGet(err,context->config,sec,P_VERBOSE,&pval)) {
      context->verbose = (unsigned int) atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec,P_MAX_CONNECTIONS,&pval)) {
      context->mxconn = (int) atoi(pval);
    }

    /* Statistics parameters */
    if (zkConfigGet(err, context->config, S_CLIENT_STAT, P_STAT_FREQUENCY, 
                     &pval)) {
      context->statTime = strtoul(pval, (char **) NULL, 10);
    }
    if (zkConfigGet(err, context->config, S_CLIENT_STAT, P_STAT_HISTORY,
                    &pval)) {
      context->statHistory = strtoul(pval, (char **) NULL, 10);
    }

    if (zkConfigGet(err,context->config,sec,P_MAX_CONNECTION_TIME,&pval)) {
      context->mxconntime = (int) atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec,P_SPOOL_MAIL_LIFETIME,&pval)) {
      context->maillifetime = strtoul(pval, (char **) NULL, 10);
    }

    if (zkConfigGet(err,context->config,sec,P_DAEMON,&pval)) {
      if (strcasecmp(PARMVAL_NO,pval) == 0) {
        context->flags |= FLAG_DONTFORK;
      }
    }

    if (zkConfigGet(err,context->config,sec,P_DEBUG,&pval)) {
      if (strcasecmp(PARMVAL_YES,pval) == 0) {
        context->flags |= FLAG_DEBUG;
      }
    }

    if (zkConfigGet(err,context->config,sec,P_RECIPIENT_COUNT_DB,&pval)) {
      context->statdbname = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_NYM_BLOCK_DB,&pval)) {
      context->nymblockdbname = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_LOGFILE,&pval)) {
      context->logfile = pval;
    }
   
    if (zkConfigGet(err,context->config,sec,P_RUN_AS_FMG,&pval)) {
      if (strcasecmp(PARMVAL_YES,pval) == 0) {
        context->flags |= FLAG_FMG;
      }
    }

#ifdef ZK_FMG
    if (zkConfigGet(err,context->config,S_CLIENT_NYMSRV,P_NYMSRV_TIMEOUT,&pval)) {
      context->nstimeout = (unsigned short) atoi(pval);
    }
#endif

    if (zkConfigGet(err,context->config,sec,P_SPOOL_DIR,&pval)) {
      context->spooldir = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_MAX_SPOOL_PROCS,&pval)) {
      context->maxclients = atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec,P_SPOOL_SLEEP_TIME,&pval)) {
      context->queuetime = atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec, P_SPOOL_MXMSGPLOOP,&pval)) {
      context->mxmsgperloop = atoi(pval);
    }

    if (zkConfigGet(err,context->config,sec,P_KEYINFO_DIRECTORY_CRYPT,&pval)) {
      context->cryptDir = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_KEYINFO_DIRECTORY_SIG,&pval)) {
      context->sigDir = pval;
    }

    if (zkConfigGet(err, context->config, sec, P_MTA_PATH, &pval)) {
      context->mtaPath = pval;
    }

#ifdef ZK_FMG   
    if (zkConfigGet(err,context->config,sec,P_ALTERNATE_TEXT,&pval)) {
      context->alttextfile = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_TAGLINE_FILE_TEXT,&pval)) {
      context->tagline.text.file = pval;
    }

    if (zkConfigGet(err,context->config,sec,P_TAGLINE_FILE_HTML,&pval)) {
      context->tagline.html.file = pval;
    }
#endif

    /* Process the MTA options (if any) */
    if (zkConfigGet(err, context->config, sec, P_MTA_OPTIONS, &pval)) {
      /* Create a new ZkList and set the free function to NULL */
      if (context->mtaOpts == NULL) {
        if (!zkListNew(err, &context->mtaOpts) ||
            !zkListSetDataFreeFunction(err, context->mtaOpts, NULL, NULL)) {
          zkErrorSet(err, "frMaipContextLoadConfig", ZKERR, ZKMMAIP, ZKERROR,
                     "Failed to create a new ZkList");
          return 0;
        }
      }

      do {
        if (!zkListInsert(err, context->mtaOpts, pval, LIST_TAIL)) {
          zkErrorSet(err, "frMaipContextLoadConfig", ZKERR, ZKMMAIP, ZKERROR,
                     "Failed to insert into ZkList");
          return 0;
        }
      } while (zkConfigGetNext(err, context->config, &pval));
    }

    /* Get the key query parameters */
    if (zkConfigGet(err, context->config, S_CLIENT_KEYQRY, P_KEYQRY_TIMEOUT, &pval)) {
      context->kqtimeout = (unsigned short) atoi(pval);
    }
    if (zkConfigGet(err, context->config, S_CLIENT_KEYQRY, P_KEYQRY_RETRIES, &pval)) {
      context->kqretries = atoi(pval);
    }

    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_SERVER,
                    &pval)) {
      context->niqsHost = pval;
    }
    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_PORT, &pval)) {
      context->niqsPort = (unsigned short) atoi(pval);
    }

    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_TIMEOUT,
                    &pval)) {
      context->niqsTimeout = (unsigned short) atoi(pval);
    }
    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_FREQUENCY,
                    &pval)) {
      context->reloadNiTime = (unsigned short) atoi(pval);
    }

    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_SHARED_MEM_SIZE,
                    &pval)) {
      context->ni_shared_mem_sz = (unsigned short) atoi(pval) * BYTES_PER_KILOBYTE;
    }

    if (context->ni_shared_mem_sz < (MINIMUM_NI_SHARED_MEM_SIZE * BYTES_PER_KILOBYTE)) {
      zkErrorSet_va(err, "frMaipContextLoadConfig", ZKERR, ZKMMAIP, ZKERROR,
                    "%s is below the minimum value of %d",
                    P_NIQS_SHARED_MEM_SIZE,
                    MINIMUM_NI_SHARED_MEM_SIZE);
      return_code = 0;
    }
    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_SHMTEMPLATE,
                    &pval)) {
      context->shmtemplate = pval;
    }

    if (zkConfigGet(err, context->config, S_CLIENT_NIQS, P_NIQS_MUTEXTEMPLATE,
                    &pval)) {
      context->mutextemplate = pval;
    }

    /*
     * This is to force a complete reload of the NIQS information in the
     * FrNetInfo object 
     */
    if (zkConfigGet(err,context->config, S_CLIENT_NIQS,P_NIQS_FULLUPDATE,
                    &pval)) {
      context->niqsUpdateOffset = (time_t) atoi(pval);
    }

    return return_code;
  }
  zkErrorSet(err,"frMaipContextLoadConfig",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 * 
 */

int
frMaipContextLoad (
  ZkError err ,
  FrMaipContext * context ,
  int argc ,
  char ** argv )
{
  int return_code = 1;
  int opt = 1;

  zkErrorClear(err);
  if (context && argv) {
  
    /* 
     * Set the program name
     */

    if ((context->progname = strrchr(argv[0],'/')) == NULL) {
      context->progname = argv[0];
    } else {
      context->progname++;
    }

#ifdef ZK_FMG
    context->flags |= FLAG_FMG;
#else
    context->flags &= ~FLAG_FMG;
#endif

    /*
     * Are we looking for some information instead of trying to run the
     * program?
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_HELP) == 0) {
        if (frMaipUsage(context,stderr)) {
          exit(EXIT_SUCCESS);
        } else {
          exit(EXIT_FAILURE);
        }
      } else if (strcasecmp(argv[opt],OPT_VERSION) == 0) {
        fprintf(stderr,"%s\n",VERSION_AMTP_STR);
        exit(EXIT_SUCCESS);
      } else if (strcasecmp(argv[opt],OPT_CREDITS) == 0) {
        frMaipCredits(stderr);
        exit(EXIT_SUCCESS);
      }
    }

    /*
     * Do we have to turn on debug mode? or are we running as the FMG?
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_DEBUG) == 0) {
        context->flags |= FLAG_DEBUG;  
        break;
      }
    }

    /*
     * Find and load the configuration file 
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_CONFIG) == 0) {
        if (++opt < argc) {
          context->configfile = argv[opt]; 
        } else {
          zkErrorSet_va(err,"frMaipContextLoad",ZKERR,ZKMMAIP,ZKERROR,
                        "%s option missing parameter.",OPT_CONFIG);  
          fprintf(stderr,"%s:  You must specify a configuration file.\n",
                  context->progname);
          return 0;
        }
        break;
      }
    } 
    if (context->configfile) {
      if (!frMaipContextLoadConfig(err,context)) {
        return 0;
      }
    }

    /*
     * Process command line parameters.  These overide options that are
     * present in the configuration file if there is some sort of conflict.
     */

#ifndef WIN32
#warning Need to check command line arguments for stat client values 
#endif

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],"--"P_DAEMON) == 0) {
        if (++opt < argc) {
          if ((strcasecmp(PARMVAL_NO,argv[opt]) == 0) ||
              (strcasecmp("0",argv[opt]) == 0)) {
            context->flags |= FLAG_DONTFORK;
          }
        } else {
          fprintf(stderr,
                  "%s:  The --"P_DAEMON" option must be set to yes or no.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PORT) == 0) {
        if (++opt < argc) {
          context->port = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s:  You must specify a port number.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PIDFILE) == 0) {
        if (++opt < argc) {
          context->pidfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a pid file name.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_NAME) == 0) {
        if (++opt < argc) {
          context->name = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a server name.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXDATA) == 0) {
        if (++opt < argc) {
          context->maxdata = atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s:  You must specify a maximum amount of data.\n",
                  context->progname);
          return_code = 0;
        } 
      } else if (strcasecmp(argv[opt],OPT_STATDB) == 0) {
        if (++opt < argc) {
          context->statdbname = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a mail usage stat database.\n",
                  context->progname);
          return_code = 0;
        }

      } else if (strcasecmp(argv[opt],OPT_NYMBLOCKDB) == 0) {
        if (++opt < argc) {
          context->nymblockdbname = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a nym -> address blocking database.\n",
                  context->progname);
          return_code = 0;
        }
    
      } else if (strcasecmp(argv[opt],OPT_ALTTEXT) == 0) {
        if (++opt < argc) {
          context->alttextfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify an alternate text file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_TAGLINE_FILE_TEXT) == 0) {
        if (++opt < argc) {
          context->tagline.text.file = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a tagline text file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_TAGLINE_FILE_HTML) == 0) {
        if (++opt < argc) {
          context->tagline.html.file = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a tagline HTML file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_LOGFILE) == 0) {
        if (++opt < argc) {
          context->logfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a logfile.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_VERBOSE) == 0) {
        if (++opt < argc) {
          context->verbose = (unsigned int) atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s: You must specify a verbose logging level.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_NYMSERVTOUT) == 0) {
        if (++opt < argc) {
          context->nstimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                 "%s:  You must specify a nym server connection timeout.\n",
                 context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCONN) == 0) {
        if (++opt < argc) {
          context->mxconn = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of connections.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCONNTIME) == 0) {
        if (++opt < argc) {
          context->mxconntime = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum connection time (in seconds).\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_SPOOLDIR) == 0) {
        if (++opt < argc) {
          context->spooldir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify the spool directory.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCLIENTS) == 0) {
        if (++opt < argc) {
          context->maxclients = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of clients.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_QUEUETIME) == 0) {
        if (++opt < argc) {
          context->queuetime = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a queue run interval (in seconds).\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXMSGPLOOP) == 0) {
        if (++opt < argc) {
          context->mxmsgperloop = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of processed messages per queue run.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_CRYPTDIR) == 0) {
        if (++opt < argc) {
          context->cryptDir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a crypt filename.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_SIGDIR) == 0) {
        if (++opt < argc) {
          context->sigDir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a signature directory.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PPPROMPT) == 0) {
        if (++opt < argc) {
          context->prompt = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a system passphrase prompt.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PPFILE) == 0) {
        if (++opt < argc) {
          context->passfile = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a passphrase file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_MTAPATH) == 0) {
        if (++opt < argc) {
          context->mtaPath = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a MTA pathname\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_MTAOPT) == 0) {
        if (++opt < argc) {
          /* Create a new ZkList and set the free function to NULL */
          if (context->mtaOpts == NULL) {
            if (!zkListNew(err, &context->mtaOpts) ||
                !zkListSetDataFreeFunction(err, context->mtaOpts, NULL, 
                                           NULL)) {
              zkErrorSet(err, "frMaipContextLoadConfig", ZKERR, ZKMMAIP, 
                         ZKERROR, "Failed to create a new ZkList");
              return 0;
            }
          }
          
          if (!zkListInsert(err, context->mtaOpts, argv[opt], LIST_TAIL)) {
            zkErrorSet(err, "frMaipContextLoadConfig", ZKERR, ZKMMAIP, 
                       ZKERROR, "Failed to insert into ZkList");
            return 0;
          }
        } else {
          fprintf(stderr, "%s: You need to specify an option for the MTA\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_KEYQRYTOUT) == 0) {
        if (++opt < argc) {
          context->kqtimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a key query timeout\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_KEYQRYRETR) == 0) {
        if (++opt < argc) {
          context->kqretries = atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify number of key query retries\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_SERVER) == 0) {
        if (++opt < argc) {
          context->niqsHost = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS hostname\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_PORT) == 0) {
        if (++opt < argc) {
          context->niqsPort = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS port\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_TIMEOUT) == 0) {
        if (++opt < argc) {
          context->niqsTimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS timeout\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_FREQUENCY) == 0) {
        if (++opt < argc) {
          context->reloadNiTime = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS frequency\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_SHMTEMPLATE) == 0) {
        if (++opt < argc) {
          context->shmtemplate = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS shared mem template\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_MUTEXTEMPLATE) == 0) {
        if (++opt < argc) {
          context->mutextemplate = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS mutex template\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_CONFIG) == 0) {
        opt++;
      } else if (!(strcasecmp(argv[opt],OPT_VERSION) &&
                   strcasecmp(argv[opt],OPT_DEBUG) &&
                   strcasecmp(argv[opt],OPT_FMG) &&
                   strcasecmp(argv[opt],OPT_CREDITS) &&
                   strcasecmp(argv[opt],OPT_HELP))) {
        /* Do diddly */
      } else {
        fprintf(stderr,"%s:  Unrecognized option \"%s\".\n",
                context->progname,argv[opt]);
        return_code = 0;
      }
    }

    return return_code;
  }
  zkErrorSet(err,"frMaipContextArguments",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 */

int
frMaipContextVerify (
  ZkError err ,
  FrMaipContext * context )
{
  int return_code = 1;

  zkErrorClear(err);
  if (context) {

    if (context->flags & FLAG_DEBUG) {
      ZkLog log = NULL;
      zkErrorLog(context->err,&log);
      zkLogSetStream(log,stderr);
      zkLogEnable(log);
      zkDebugOn();
    }
  
    if (context->name == NULL) {
      context->name = context->progname;
    }

    if (!frCryptInit(context->err,NULL)) {
      return_code = 0;
    }

    /*
     * Try to open up the logfile
     */

    if (context->logfile) {
      if (!zkLogSetFile(context->log,context->logfile)) {
        fprintf(stderr,"%s:  Unable to open logfile \"%s\".\n", 
                context->progname,context->logfile);
        return_code = 0;
      }
      zkLogSetProgname(context->log,context->name);
      zkLogEnable(context->log);
    }

    /*
     * Check all of our parameters
     */

    if (!context->passfile && !context->prompt) {
      fprintf(stderr, 
              "%s: Either passphrase file or prompt must be specified.\n", 
              context->progname);
      return_code = 0;
    }  

    if (context->port == 0) {
      fprintf(stderr,"%s:  AMTP port must be non-zero.\n",context->progname);
      return_code = 0;
    }

    if (context->maxdata < 0) {
      fprintf(stderr,"%s:  \"%s\" parameter must be a non-negative integer.\n",
              context->progname,P_MAX_DATA);
      return_code = 0;
    } 

    if (context->mxconn < 1) {
      fprintf(stderr,
              "%s:  \"%s\" must be set to allow at least 1 connection.\n", 
              context->progname,P_MAX_CONNECTIONS);
      return_code = 0;
    }

    if (context->statdbname == NULL) {
      fprintf(stderr,"%s:  You must specify a mail usage stats database.\n",
              context->progname);
      return_code = 0;
    }

    if (context->nymblockdbname == NULL) {
      fprintf(stderr,
              "%s:  You must specify a blocked nym->email address database.\n",
              context->progname);
      return_code = 0;
    }
  
    if (context->mxconntime < 0) {
      fprintf(stderr,
              "%s:  \"%s\" must be a non-negative integer in seconds (0 turns off the check).\n",
              context->progname,P_MAX_CONNECTION_TIME);
      return_code = 0;
    }     

    if (context->statTime < 5) {
      fprintf(stderr, "%s: Stat time [%d] is too low.\n", context->progname,
              context->statTime);
      return_code = 0;
    }

    if (context->niqsHost == NULL) {
      fprintf(stderr, "%s: NIQS server hostname not specified.\n",
              context->progname);
      return_code = 0;
    }

    if (context->niqsPort < 1024) {
      fprintf(stderr, "%s: NIQS server port [%d] invalid.\n", 
              context->progname, context->niqsPort);
      return_code = 0;
    }

    if (context->niqsTimeout < 1) {
      fprintf(stderr, "%s: NIQS server timeout [%d] is too short.\n", 
              context->progname, context->niqsTimeout);
      return_code = 0;
    }

    /*
     * You can set the niqs update offset to be 0 in order to turn off the
     * feature.  If you do this, the FrNetInfo will always just do an update
     * when you call the update functions.  If you set an offset however,
     * the frNetInfo object will do a complete reload periodically, based 
     * on the specified offset
     */

    if ((context->niqsUpdateOffset < 60) && (context->niqsUpdateOffset != 0)) {
      fprintf(stderr,"%s:  NIQS update offset %d is too short.\n",
              context->progname,(int) context->niqsUpdateOffset);
      return_code = 0;
    }

    if (context->reloadNiTime < 60) {
      fprintf(stderr, "%s: NIQS server frequency [%d] is too short.\n", 
              context->progname, context->reloadNiTime);
      return_code = 0;
    }

    if (!context->shmtemplate) {
      fprintf(stderr, "%s: NIQS shared memory template file not specified.\n", 
              context->progname);
      return_code = 0;
    }

    if (!context->mutextemplate) {
      fprintf(stderr, "%s: NIQS mutex template file not specified.\n", 
              context->progname);
      return_code = 0;
    }

#ifdef ZK_FMG
    context->flags |= FLAG_FMG;
    if (context->flags & FLAG_FMG) {
   
      /* 
       * When running as an FMG, certain parameters must be specified
       */
      if (context->nstimeout < 1) {
        fprintf(stderr,"%s:  You must specify at least 1 second for the nym server connection timeout.\n",
                context->progname);
        return_code = 0;
      }

      /*
       * The alternate text file is OPTIONAL
       */

      if (context->alttextfile) {
        if (!zkFileLoad(err,context->alttextfile,&(context->alttext),
                        &(context->alttext_sz))) {
          fprintf(stderr,"%s:  Unable to load alternate text file \"%s\".\n",
                  context->progname,context->alttextfile);
          return_code = 0;
        }
      } else {
        context->alttext = NULL;
        context->alttext_sz = 0;
      }

      if (context->tagline.text.file) {
        if (!zkFileLoad(err,context->tagline.text.file,&(context->tagline.text.data),
                        &(context->tagline.text.size))) {
          fprintf(stderr,"%s:  Unable to load tagline text file \"%s\".\n",
                  context->progname,context->tagline.text.file);
          return_code = 0;
        }
      } else {
        context->tagline.text.data = NULL;
        context->tagline.text.size = 0;
      }

      if (context->tagline.html.file) {
        if (!zkFileLoad(err,context->tagline.html.file,&(context->tagline.html.data),
                        &(context->tagline.html.size))) {
          fprintf(stderr,"%s:  Unable to load tagline HTML file \"%s\".\n",
                  context->progname,context->tagline.html.file);
          return_code = 0;
        }
      } else {
        context->tagline.html.data = NULL;
        context->tagline.html.size = 0;
      }

    } /* End of FMG sanity checks */
#else
    /* Can't run in FMG mode, so make sure none of those parameters
     * have any value.
     */
    if (context->flags & FLAG_FMG) {
      fprintf(stderr, "FMG option is only available to ZKS internal hosts.\n");
      return_code = 0;
    }

    context->flags &= ~FLAG_FMG;
    context->nshost = NULL;
    context->nsport = 0;
    context->nstimeout = 0;
    context->alttextfile = NULL;
    context->alttext = NULL;
    context->alttext_sz = 0;
    context->tagline.text.data = NULL;
    context->tagline.text.size = 0;
    context->tagline.html.data = NULL;
    context->tagline.html.size = 0;
#endif

    /* Need the key query parameters for initial NIQS query */
    if (context->kqtimeout < 1) {
      fprintf(stderr, "%s: Timeout for a key query is too short\n",
              context->progname);
      return_code = 0;
    }

    if (context->kqretries < 0) {
      fprintf(stderr, "%s: Key query retries value is negative [%d]\n",
              context->progname, context->kqretries);
      return_code = 0;
    }


    if (!frKeyQryNew(err, &(context->kq))) {
      return 0;
    }

    if (context->maxclients < 1) {
      fprintf(stderr,"%s:  \"%s\" parameter must be greater than 1.\n",
              context->progname,P_MAX_SPOOL_PROCS);
      return_code = 0;
    }

    if (context->queuetime < 1) {
      fprintf(stderr,"%s:  \"%s\" parameter must be greater than 1.\n",
              context->progname,P_SPOOL_SLEEP_TIME);
      return_code = 0;
    }

    if (context->mxmsgperloop < 1) {
      fprintf(stderr,"%s:  \"%s\" parameter must be greater than 1.\n",
              context->progname,P_SPOOL_MXMSGPLOOP);
      return_code = 0;
    }
    /* Bug? Check if spool directory exists and is writable by me only */

    /*
     * finish up some datastructures which are based on the input parameters
     */

    context->server.sin_family = AF_INET;
    context->server.sin_port = htons(context->port);
    context->server.sin_addr.s_addr = htonl(INADDR_ANY);

    if (!context->cryptDir || !context->sigDir) {
      fprintf(stderr, "%s: Both signature/cryption key"
              "directories need to be specified.\n", context->progname);
      return_code = 0;
    }

    /* Bug: Need the existance of the MTA Path and what are valid options */

    return return_code;
  }
  zkErrorSet(err,"frMaipContextVerify",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/**
 * Set up the FrMaipSpoolCtx object.
 */
int frMaipSetSpoolParms(ZkError err, FrMaipContext * context)
{
  const char *func = "frMaipSetSpoolParms";

  if (!context) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frMaipSpoolCtxNew(err, &context->mspoolctx)) {
    zkLogWriteContext(context->log,err,func,"frMaipSpoolCtxNew failed");
    return 0;
  }
  
  if (!frMaipSpoolCtxSetNetInfo(err, context->mspoolctx, context->ni)) {
    zkLogWriteContext(context->log,err,func,"frMaipSpoolCtxSetNetInfo failed");
    return 0;
  }
  
  return 1;
}    

int
frMaipMain (
  struct sockaddr_in remote,
  void * data )
{
#define HOSTNAME_SZ 128
  static const char * func = "frMaipMain";
  FrMaipContext * context = (FrMaipContext *) data;
  struct hostent * host = NULL;
  char hostname[HOSTNAME_SZ + 1];
  int return_code = 0;
  FrMaipSessionContext sctx; 
  uint32_t active = 0;

  if (context == NULL) {
    return 0;
  }
  if ((host = gethostbyaddr((char *)&(remote.sin_addr.s_addr),4,AF_INET))) { /* audit: ok */
    strncpy(hostname,host->h_name,HOSTNAME_SZ);
    hostname[HOSTNAME_SZ] = (char) 0;
  } else {
    unsigned char * ptr = (unsigned char *) &(remote.sin_addr.s_addr);
    snprintf(hostname,sizeof hostname,"%u.%u.%u.%u",ptr[0],ptr[1],ptr[2],ptr[3]); 
  }

  /*
   * Initialize the session context contents to 0.  This holds all of the 
   * bits of information required to forward an email via AMTP
   */

  memset(&sctx,0,sizeof(FrMaipSessionContext));
  sctx.status = context->status;

  if (!(zkipSessionNew(context->err,&(context->session)) &&
        zkipSessionSet(context->err,context->session,context->engine,
                       context->log,0,1,(void *)&sctx) &&
        zkipSessionSetMaxData(context->err,context->session,context->maxdata) &&
        zkipSessionSetMaxConnTime(context->err,
                                  context->session,
                                  context->mxconntime) &&
        zkipSessionSetVerbose(context->err,context->session,context->verbose) &&
        zkipSessionSetHostname(context->err,context->session,hostname) &&
        zkipSessionTerminateOnError(context->err,context->session))) {
    zkLogWriteContext(context->log,context->err,func,
                      "Failed to set up ZKIP session");
    goto finish;
  }

  if (!frMaipStatusGetActive(context->err, context->status, &active)) {
    zkLogWriteContext(context->log,context->err,func,
                      "frMaipStatusGetActive failed");
    goto finish;
  }

  if (context->flags & FLAG_REFUSEACCESS) {
    if (!zkipSessionResponse(context->err,context->session,0,_SERVICE_CLOSING,
                             "Server temporarily out of service","NODATA",
                             ZKIP_ENCODING_BINARY,0,NULL)) {
      zkLogWriteContext(context->log,context->err,func,
                        "Failed to return \"out of service\" message");
    } 
    if (context->verbose >= VERBOSE_SESSION_LOGGING) {
      zkLogWrite_va(context->log,"Refusing connection from %s.",hostname);
    }
  } else if (active > (uint32_t) context->mxconn) {
    if (!zkipSessionResponse(context->err,context->session,0,_SERVICE_CLOSING,
                             "Too many simultaneous connections","NODATA",
                             ZKIP_ENCODING_BINARY,0,NULL)) {
      zkLogWriteContext(context->log,context->err,func,
                        "Failed to return \"too many connections\" message");
    } 
    if (context->verbose >= VERBOSE_SESSION_LOGGING) {
      zkLogWrite_va(context->log,"Refusing connection from %s.",hostname);
    }
  } else {
#ifdef ZK_FMG
    if (context->flags & FLAG_FMG) {

      /*
       * Try to open the two fmg-maip databases.  This must succeed for
       * the fmg-maip to be allowed to process a mail message
       */

      if (!zkDB_Open(context->err,&(context->statdb),context->statdbname)) {
        zkLogWriteContext(context->log,context->err,func,
                          "Failed to open stat db");
        if (!zkipSessionResponse(context->err,context->session,0,
                                 _SERVICE_CLOSING,
                                "Unable to open mail statistics database",
                                "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
          zkLogWriteContext(context->log,context->err,func,
                            "Failed to notify client that stat db open failed");
        }
      } else if (!zkDB_Open(context->err,&(context->nymblockdb),
                           context->nymblockdbname)) {
        zkLogWriteContext(context->log,context->err,func,
                          "Failed to open nym block db");
        if (!zkipSessionResponse(context->err,context->session,0,
                                 _SERVICE_CLOSING,
                                "Unable to open blocked email address database",
                                "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
          zkLogWriteContext(context->log,context->err,func,
                            "Failed to notify client that mail block db open failed");
        }

      /*
       * Run the server event loop 
       */

      } else if (!zkipSessionRun(context->err,context->session)) {
        zkLogWriteContext(context->log,context->err,func,
                          "zkipSessionRun failed");
      } else {
        return_code = 1;
      }

      /*
       * Close the databases
       */

      if (!zkDBClose(context->err,&(context->statdb))) {
        zkLogWriteContext(context->log,context->err,func,
                          "Failed to close stat db");
      }
      if (!zkDBClose(context->err,&(context->nymblockdb))) {
        zkLogWriteContext(context->log,context->err,func,
                   "Failed to close nym block db");
      }

    } 
#else /* This is not an FMG-maip */

    if (!zkipSessionRun(context->err,context->session)) {
      zkLogWriteContext(context->log,context->err,func,
                        "zkipSessionRun failed");
    } else {
      return_code = 1;
    }

#endif

  }
  
finish:
  
  /*
   * Clean up some of the data structures.
   */

  if (context->session) {
    zkipSessionFree(NULL,&(context->session));
  }
#if 0
  if (sctx.header) {
    zkEmailHeaderFree(NULL,&(sctx.header));
  }
  if ( sctx.email) {
    (void) zkEmailFree(NULL, &(sctx.email));
  }
#endif

  if (sctx.message) {
    memset(sctx.message,0,sctx.message_sz);
    zkMemFree(NULL,(void**)&(sctx.message));
  }

#if 0
  if (sctx.alttext) {
    memset(sctx.alttext,0,sctx.alttext_sz);
    zkMemFree(NULL,(void**)&(sctx.alttext));
  }
#endif
  
  if (sctx.target) {
    memset(sctx.target,0,sctx.target_sz);
    zkMemFree(NULL,(void**)&(sctx.target));
  }

  if (sctx.tgt_rb_hash) {
    zkMemFree(NULL,&(sctx.tgt_rb_hash));
  }

  sctx.status = 0;

  memset(&sctx,0,sizeof(FrMaipSessionContext));

  return return_code;
}

/*
 *  Process number 1, the incoming AMTP server daemon
 */

int
frMaipAMTPServer (
  ZkError err ,
  FrMaipContext * context )
{
  static const char * func = "frMaipAMTPServer";
  zkErrorClear(err);
  if (context) {
    if (!zkipEngineNew(err,&(context->engine))) {
      zkLogWriteContext(context->log,err,func,
                        "zkipEngineNew failed");
      return 0;
    }

    /* 
     * Add some basic ZKIP commands and the AMTP commands to the server's
     * ZKIP engine
     */

    if (!(zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_ZKIP,1,0),
                               zkipCallbackQuit,"quit",0) &&
          zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_ZKIP,1,0),
                               zkipCallbackList,"list",0) &&
          zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_AMTP,1,0),
                                     amtpCommandHeader,"header",0) &&
          zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_AMTP,1,0),
                               amtpCommandMessage,"message",0) &&
          zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_AMTP,1,0),
                               amtpCommandAlttext,"alttext",0) &&
          zkipEngineAddCommand(err,context->engine,CMDID(ZKIP_PROTO_AMTP,1,0),
                               amtpCommandTarget,"target",0))) {
      zkLogWriteContext(context->log,err,func,
                 "zkipEngineAddCommand failed");
      return 0;
    }

    /*
     * Add the ZKIP-AMTP welcome banner 
     */

    if (!zkipEngineSetBanner(err,context->engine,VERSION_AMTP_STR)) {
      zkLogWriteContext(context->log,err,func,
                        "zkipEngineSetBanner failed");
      return 0;
    }

    /*
     * Write to log file that we're finally ready to accept ZKIP-AMTP 
     * connections and start the server listen loop.
     */

    zkLogWrite_va(context->log,"Accepting AMTP client connections on port %d.",
                  context->port);
    if (!RunAsStreamServer(err,context->log,&(context->server),
                           frMaipMain,(void *)context)) {
      zkLogWriteContext(context->log,err,func,
                        "RunAsStreamServer failed");
      context->exit_code = EXIT_FAILURE;
    } else {
      context->exit_code = EXIT_SUCCESS;
    }
    return 1;
  }
  zkErrorSet(err,"frMaipAMTPServer",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 *  Process number 2, the AMTP outgoing queue daemon
 */

int
frMaipAMTPQueue (
  ZkError err ,
  FrMaipContext * context )
{
  zkErrorClear(err);
  if (context) {

    zkLogWrite_va(context->log,"Queue running daemon using spool [%s]", 
                  context->spooldir);
    RunQueueClient(err, context->log, maipQueueLoop, (void *) context);
    return 1;
  }
  zkErrorSet(err,"frMaipAMTPQueue",ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/**
 * Set up the frstat objects.
 */
int maipSetUpStat(ZkError err, FrMaipContext *context)
{
  const char *func = "maipSetUpStat";
  FrCryptPubKey key = NULL;
  uint32_t nqueued;

  if (!context) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frMaipStatusNew(err, &(context->status))) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipStatusNew failed");
    return 0;
  }

  if (!frMaipStatusInit(err, context->status, "/freedom/var/run/stats-%d.shm", 
                        "/freedom/var/run/stats-%d.mtx")) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipStatusInit failed");
    return 0;
  }

  if (!frMaipStatusSetStatParms(err, context->status, context->statServer, 
                                context->statPort, context->statHistory)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipStatusSetStatParms failed");
    return 0;
  }

  if (!frSecKeyChainGetVersion(err, context->keys, context->sigHandle, 0,
                               &key)) {
    zkLogWriteContext(context->log,err,func,
                      "frSecKeyChainGetVersion failed");
    return 0;
  }

  if (!frMaipStatusSetCurrentSigKey(err, context->status, key, 
                                    VERSION_AMTP_STR)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipStatusSetCurrentSigKey failed");
    return 0;
  }

  /* Find out how many messages is in the queue right now when I start up */
  if (!frMaipSpoolGetQueued(err, context->spooldir, &nqueued)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipSpoolGetQueued failed");
    return 0;
  }

  if (!frMaipStatusSetQueued(err, context->status, nqueued)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipSpoolSetQueued failed");
    return 0;
  }

  return 1;
}

int frMaipNetInfoGetType(ZkError err, FrNetInfo ni, FrEntIdType type,
                                char **hname, unsigned short *port,
                                FrEntIdExp *entity)
{
  const char *func = "frMaipNetInfoGetType";
  ZkList list = NULL;
  FrNetInfoDatum data = NULL;
  char *p = NULL;
  struct sockaddr_in addr;

  if (!ni || !hname || !port) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoGetListOf(err, ni, &list, type)) {
    zkLogWriteContext(frMaipContext->log,err,func,
                      "frMaipStatusNew failed");
    return 0;
  }

  if (!list) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "Server type not found");
    return 0;
  }

  if (!zkListMoveCursor(err, list, LIST_HEAD)) {
    zkLogWriteContext(frMaipContext->log,err,func,
                      "zkListMoveCursor failed (NetInfo list)");
    return 0;
  }
  
  if (!zkListGet(err, list, (void *) &data, LIST_STAY)) {
    zkLogWriteContext(frMaipContext->log,err,func,
                      "zkListGet failed (NetInfo list)");
    return 0;
  }
  
  if (!frNetInfoDatumGetSockaddr(err, data, &addr)) {
    zkLogWriteContext(frMaipContext->log,err,func,
                      "frNetInfoDatumGetSockaddr failed");
    return 0;
  }

  *port = ntohs(addr.sin_port);
  
  if ((p = inet_ntoa(addr.sin_addr)) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, 
               "Can't convert sockaddr to dotted quad");
    return 0;
  }

  /* Make space for the hostname */
  if ((*hname = malloc(sizeof(char) * (strlen(p) + 1))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENOMEM, "malloc() failed");
    return 0;
  }
  zkCStrlcpy(*hname, p, strlen(p) + 1);

  if (entity) {
    if (!frNetInfoDatumGetEntity(err, data, entity)) {
      zkLogWriteContext(frMaipContext->log,err,func,
                        "frNetInfoDatumGetEntity failed");
      return 0;
    }
  }

  return 1;
}    

/**
 * Initialize my FrNetInfo object and populate it.  Get enough info
 * out of it so that I can continue.  In this case, I need the key query
 * server hostname/port, nymserver hostname/port, and the stat server 
 * hostname/port.
 */
int frMaipNetInfoInit(ZkError err, FrMaipContext *context)
{
  const char *func = "frMaipNetInfoInit";
  ZkipClient cli = NULL;
  int connected = 0;

  if (!context) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL ptr");
    return 0;
  }

  if (!frNetInfoNew(err, &context->ni)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoNew failed");
    return 0;
  }

  if (!frNetInfoChildCtxNew(err, &context->niCtx)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoChileCtxNew failed");
    return 0;
  }

  if (!frNetInfoChildCtxSetUpdateOffset(err,context->niCtx,
                                        context->niqsUpdateOffset)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoChildCtxSetUpdateOffset failed");
    return 0;
  }

  if (!frNetInfoChildCtxSetNIQSParms(err, context->niCtx, context->niqsHost,
                                     context->niqsPort,
                                     context->niqsTimeout)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoChildCtxSetNIQSParms failed");
    return 0;
  }

  if (!frNetInfoChildCtxSetPollParms(err, context->niCtx,
                                     context->reloadNiTime,
                                     context->reloadNiTime / 2)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoChildCtxSetPollParm failed");
    return 0;
  }

  while (!connected) {
    if (frNiqsConnect(err, &cli, context->niqsHost, context->niqsPort,
                       context->niqsTimeout)) {
      connected = 1;
    } else {
      zkLogWrite_va(context->log, 
                    "Unable to connect to %s:%d, retry in 60 seconds\n",
                    context->niqsHost, context->niqsPort);
      sleep(60);
    }
  }

  if (!frNetInfoPopulate(err, context->ni, cli, context->kq)) {
    zkLogWriteContext(context->log,err,func,
                      "frNetInfoPopulate failed");
    return 0;
  }

  (void) frNiqsDisconnect(err, &cli);

#ifdef ZK_FMG
  if (!frMaipNetInfoGetType(err, context->ni, FRENT_TYPE_NYM_SRV,
                            &context->nshost, &context->nsport, NULL)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipNetInfoGetType failed (nym server)");
    return 0;
  }

#endif

# if 0
  /*
   * This function used to set up a file descriptor for the UDP socket
   * connection to the key query server.  Each child would then over-
   * write the file descriptor and create its own UDP socket.  The
   * parent would continue to use this connection when it periodically
   * queried for the NISS key.
   *
   * Under Linux, it is a bad idea to keep a long lived UDP socket
   * around like this.  According to Philippe, if at some point the
   * connection becomes invalid, nasty things can happen.  Instead,
   * the parent now creates and destroys a new UDP socket when it
   * is needed.
   *
   */
  {
    struct protoent *ppe;
    
    ppe = getprotobyname("udp");

    if ((context->kh.fd = socket(AF_INET, SOCK_DGRAM, ppe->p_proto)) == -1) {
      zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, "socket() failed");
      return 0;
    }
  }
# endif

  if (!frMaipNetInfoGetType(err, context->ni, FRENT_TYPE_KEYQRY_SRV,
                            &context->khhost, &context->khport, NULL)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipNetInfoGetType failed (key query server)");
    return 0;
  }

/*
 *
 * inet_addr returns "-1" on error which actually translates to 255.255.255.255
 * which is in fact a valid address, so there is no way to detect an error when
 * using the inet_addr() function.  This is why this code has been commented
 * out 
 *  if ((context->kh.host.sin_addr.s_addr = inet_addr(context->khhost)) == -1) {
 *    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, "Can't convert hostname");
 *    return 0;
 *  }
 */

  if (inet_aton(context->khhost,&(context->kh.host.sin_addr)) == 0) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, "Can't convert hostname");
    return 0;
  }

  context->kh.host.sin_port = htons(context->khport);

  free(context->khhost);
  context->khhost = NULL;

  if (!frMaipNetInfoGetType(err, context->ni, FRENT_TYPE_NISS, 
                            &context->statServer, &context->statPort,
                            &context->statEntity)) {
    zkLogWriteContext(context->log,err,func,
                      "frMaipNetInfoGetType failed (NISS)");
    return 0;
  }

  return 1;
}

/**
 * Clean up my memory
 */
int frMaipCleanContext(ZkError err, FrMaipContext *c)
{
  const char *func = "frMaipCleanContext";

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param");
    return 0;
  }
  
  if (c->config) { zkConfigFree(err, &c->config); }
  if (c->statServer) { free(c->statServer); c->statServer = 0; }
  if (c->status) { frMaipStatusFree(err, &c->status); }
  if (c->nshost) { free(c->nshost); c->nshost = NULL; }
  if (c->alttext) { free(c->alttext); c->alttext = NULL; }
  if (c->khhost) { free(c->khhost); c->khhost = NULL; }
  if (c->kq) { frKeyQryFree(err, &c->kq); }
  if (c->keys) { frSecKeyChainFree(err, &c->keys); }
  if (c->mspoolctx) { frMaipSpoolCtxFree(err, &c->mspoolctx); }
  if (c->ni) { frNetInfoFree(err, &c->ni); }
  if (c->niCtx) { frNetInfoChildCtxFree(err, &c->niCtx); }
  if (c->fptr) { fclose(c->fptr); c->fptr = NULL; }
 
  zkErrorClear(err);

  return 1;
}
