#ifndef MAIP_H_
#define MAIP_H_

#include <netinet/in.h>

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */

#ifndef _ZKIP_H
#include <ZKIP.h>
#endif /* _ZKIP_H */

#ifndef _ZKIPENGINE_H
#include <zkipEngine.h>
#endif /* _ZKIPENGINE_H */

#ifndef FRCRYPT_H_
#include <frCrypt.h>
#endif /* FRCRYPT_H_ */

#ifndef FRKEYQRY_H_
#include <frKeyQry.h>
#endif /* FRKEYQRY_H_ */

#ifndef FRSEC_H_
#include <frSec.h>
#endif /* FRSEC_H_ */

#ifndef FRKEY_H_
#include <frKeyUpdate.h>
#endif /* FRKEY_H_ */

#ifndef ZKMIME_H_
#include <zkMIME.h>
#endif /* ZKMIME_H_ */

#ifndef ZKLIST_H_
#include <zkList.h>
#endif /* ZKLIST_H_ */

#ifndef ZKDB_H_
#include <zkDB.h>
#endif /* ZKDB_H_ */

#ifndef FRNYM_H_
#include <frNym.h>
#endif /* FRNYM_H_ */

#include "frNetInfo.h"
#include "frMaip.h"

#include "frStatUpd.h"
#include "frStatClient.h"

#include "frNetworkDefaults.h"
#include "frFilenames.h"
#include "maipstat.h"

#include <sys/types.h>
#include <sys/socket.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef FREEDOM_DEVELOPMENT_BUILD
#define VERSION_AMTP_STR "Zero-Knowledge AMTP server (development) ("__DATE__", "__TIME__")"
#else
#define VERSION_AMTP_STR "Zero-Knowledge AMTP server (production) ("__DATE__", "__TIME__")"
#endif

/*
 * Command line parameters 
 */

#define OPT_DEBUG       "--" P_DEBUG         /* Turn on debugging mode */
#define OPT_CONFIG      "--" P_CONFIG
#define OPT_HELP        O_HELP
#define OPT_PORT        "--" P_PORT
#define OPT_VERSION     O_VERSION
#define OPT_PIDFILE     "--" P_PIDFILE
#define OPT_NAME        "--name"
#define OPT_MAXDATA     "--" P_MAX_DATA
#define OPT_VERBOSE     "--" P_VERBOSE
#define OPT_MAXCONNTIME "--" P_MAX_CONNECTION_TIME
#define OPT_MAXCONN     "--" P_MAX_CONNECTIONS
#define OPT_LOGFILE     "--" P_LOGFILE
#define OPT_CREDITS     "--credits"

#if 0
#define OPT_NYMSERVHOST "--" P_NYMSRV_HOSTNAME
#define OPT_NYMSERVPORT "--" P_NYMSRV_PORT
#endif

#define OPT_NYMSERVTOUT "--" P_NYMSRV_TIMEOUT
#define OPT_SPOOLDIR    "--" P_SPOOL_DIR
#define OPT_MAXCLIENTS  "--" P_MAX_SPOOL_PROCS
#define OPT_FMG         "--" P_RUN_AS_FMG
#define OPT_QUEUETIME   "--" P_SPOOL_SLEEP_TIME
#define OPT_MAXMSGPLOOP "--" P_SPOOL_MXMSGPLOOP
#define OPT_SIGDIR      "--" P_KEYINFO_DIRECTORY_SIG
#define OPT_CRYPTDIR    "--" P_KEYINFO_DIRECTORY_CRYPT
#define OPT_MTAPATH     "--" P_MTA_PATH
#define OPT_MTAOPT      "--" P_MTA_OPTIONS
#define OPT_ALTTEXT     "--" P_ALTERNATE_TEXT

#define OPT_TAGLINE_FILE_TEXT "--" P_TAGLINE_FILE_TEXT
#define OPT_TAGLINE_FILE_HTML "--" P_TAGLINE_FILE_HTML

#if 0
#define OPT_KEYQRYHOST  "--" P_KEYQRY_HOSTNAME
#define OPT_KEYQRYPORT  "--" P_KEYQRY_PORT
#endif

#define OPT_KEYQRYTOUT  "--" P_KEYQRY_TIMEOUT
#define OPT_KEYQRYRETR  "--" P_KEYQRY_RETRIES
#define OPT_STATDB      "--statdb"
#define OPT_NYMBLOCKDB  "--nymblockdb"
#define OPT_STATDIR     "--statdirectory"
#define OPT_STATTIME    "--stattime"
#define OPT_PPPROMPT    "--" P_PASSPHRASE_PROMPT
#define OPT_PPFILE      "--" P_PASSPHRASE_FILE

#define OPT_NIQS_SERVER "--" P_NIQS_SERVER
#define OPT_NIQS_PORT   "--" P_NIQS_PORT
#define OPT_NIQS_TIMEOUT "--" P_NIQS_TIMEOUT
#define OPT_NIQS_FREQUENCY "--" P_NIQS_FREQUENCY
#define OPT_NIQS_MUTEXTEMPLATE "--" P_NIQS_MUTEXTEMPLATE
#define OPT_NIQS_SHMTEMPLATE "--" P_NIQS_SHMTEMPLATE
#define OPT_NIQS_SHARED_MEM_SIZE "--" P_NIQS_SHARED_MEM_SIZE

/*
 * Configuration file section labels
 */

#define SEC1  "MAIP"
#define SECFMG "FMG-MAIP"

#define PARMVAL_YES      "yes"
#define PARMVAL_NO       "no"

/*
 * Some context default values
 */

#define DEFAULT_CONFIGFILE    NULL          /* Default configuration file */
#define DEFAULT_PROGNAME      "maip"
#define DEFAULT_PORT          FR_DFLT_MAIP_PORT /* Default AMTP port */
#define DEFAULT_PIDFILE       NULL          /* Stores a PID on disk */
#define DEFAULT_NAME          NULL          /* Optional name for server */
#define DEFAULT_LOGFILE       NULL          /* Default logfile */
#define DEFAULT_MAXDATA       0             /* Maximum data / command 0 = off */
#define DEFAULT_VERBOSE       0             /* Default verbose logging mode */
#define DEFAULT_MXCONN        50            /* Max number of simult. connects */
#define DEFAULT_MXCONNTIME    600           /* Maximum connection duration */
#define DEFAULT_NSHOST        NULL          /* Default Nym server host */
#define DEFAULT_NSPORT        0             /* Default Nym server port */
#define DEFAULT_NSTIMEOUT     30            /* Default nym server timeout */
#define DEFAULT_SPOOLDIR      FREEDOM_SPOOLDIR"/maip" /* Default spool directory */
#define DEFAULT_MAXCLIENTS    1             /* Default number of clients */
#define DEFAULT_QUEUETIME     30            /* Queue run interval (secs) */
#define DEFAULT_MAXMSGPLOOP   5             /* Maximum number of processed 
                                               messages per queue run */
#define DEFAULT_ALTTEXTFILE   NULL
#define DEFAULT_MTAOPT_LIST   NULL
#define DEFAULT_TAGLINE_FILE_TEXT  NULL
#define DEFAULT_TAGLINE_FILE_HTML  NULL

#define DEFAULT_MTA           "/usr/lib/sendmail"
#define DEFAULT_KEYQRYHOST    NULL          /* Key server hostname */
#define DEFAULT_STATDB        "fmgstat"     /* Mail usage stats database */
#define DEFAULT_NYMBLOCKDB    "nymblock"    /* nym -> addr block database */
#define DEFAULT_KEYQRYPORT    0             /* Port number for key server */
#define DEFAULT_KEYQRYTOUT    0             /* Key lookup timeout */
#define DEFAULT_KEYQRYRETR    0             /* Key lookup timeout */
#define DEFAULT_STATDIR       NULL          /* Holds the stat files */
#define DEFAULT_STATTIME      60            /* Refresh every 60 seconds */
#define DEFAULT_MAIL_LIFETIIME (7*24*60)    /* maximum retry elapse time (in minute) */
#define DEFAULT_NI_FREQUENCY  (30 * 60)     /* Reload NetInfo every 30 min */

/*
 * Context behaviour flags (store in the flags variable)
 */

#define FLAG_DEBUG    0x1     /* Debug mode on */
#define FLAG_REFUSEACCESS 0x2 /* Refuse client connections */
#define FLAG_DONTFORK     0x4 /* Dont immediatly fork */
#define FLAG_FMG          0x8 /* MAIP runs as an FMG */

struct T_FrMaipTagLineInfo {
  const char * file;
  char *       data;
  size_t       size;
};

struct T_FrMaipContext {
  uint32_t flags;             /* Application behaviour flags */
  int exit_code;              /* Application exit code on termination */
  ZkError err;                /* Common error structure for application */
  ZkLog log;                  /* Application log file */
  ZkConfig config;            /* Application configuration */
  ZkipSession session;        /* ZkipSession */
  ZkipEngine engine;

  struct sockaddr_in server;  /* The server address structure */
  pid_t queue_pid;            /* Process which handles outgoing queue */
  const char * progname;      /* Application program name (argv[0]) */
  const char * name;          /* A maybe unique name for the server */
  const char * configfile;    /* Name of the configuration file */
  const char * pidfile;       /* Name of the PID file */
  const char * logfile;       /* Name of the logfile */
  const char * spooldir;      /* Spool directory */
  const char * statdbname;    /* Name of the mail usage stats DB */
  const char * nymblockdbname; /* Name of the nym ->addr mail blocking db */

  /* mailing retry parameters */
  uint32_t  maillifetime;     /* Maximum life time of a mail message */

  /* Statistics */
  uint32_t statTime;            /* Time to look at the statistics */
  size_t statHistory;           /* How many old entries to keep */
  char *statServer;             /* Hostname of the stat server */
  unsigned short statPort;      /* Stat server port */
  FrEntIdExp statEntity;        /* Stat Entity */
  
  FrMaipStatus status;          /* Status object */

  /* This is only required for the FMG maip to keep track of the recipient
   * counts
   */
#ifdef ZK_FMG  
  ZkDB statdb;                /* Mail usage stats DB */
  ZkDB nymblockdb;            /* nym -> addr Mail blocking database */
#endif /* ZK_FMG */
  unsigned short port;        /* Port on which MAIP will listen */

  char * nshost;              /* Nym server hostname */
  unsigned short nsport;      /* Nym server port */
  unsigned short nstimeout;   /* Nym server connection timeout (seconds) */
  const char * alttextfile;   /* Alternate text file name */
  char * alttext;             /* Alternate text for message to nym */
  size_t alttext_sz;          /* Size of the alternate text message */

  /* Tag Line Information */

  struct {
    struct T_FrMaipTagLineInfo html;
    struct T_FrMaipTagLineInfo text;
  } tagline;

  /* Key query stuff */
  FrKeyQryHost kh;            /* KeyQry Host addr and fd */
  char *khhost;
  unsigned short khport;
  unsigned short kqtimeout;   /* Timeout for fetching a key */
  int kqretries;              /* Timeout for fetching a key */
  FrKeyQry kq;                /* Public key cache */

  int maxdata;                /* Maximum amount of data per ZKIP command */
  unsigned int verbose;       /* Verbose logging level (see ZKIP) */
  int mxconn;                 /* Maximum number of simultaneous connections */
  int mxconntime;             /* Maximum duration of a connection */
  size_t maxclients;          /* Maximum clients to run spool */
  int queuetime;              /* Queue run interval */
  size_t mxmsgperloop;        /* Maximum number of processed messages per loop run */
  const char *mtaPath;        /* Path to MailTransferAgent (sendmail/qmail) */
  ZkList mtaOpts;             /* List of Options to pass to MTA */

  /* Private key stuff */
  const char *cryptDir;       /* Filename of the crypt key file */
  const char *sigDir;         /* Filename of the sig key file */
  FrSecKeyChain keys;         /* Key chain */
  size_t cryptHandle;         /* Handles for the key chain */
  size_t sigHandle;

  const char *prompt;         /* Prompt for the system passphrase */
  const char *passfile;       /* File from which to read passphrase */
  
  FILE *fptr;                 /* FILE pointer to the PID locked file */

  FrMaipSpoolCtx mspoolctx;   /* Context for spooling of MAIP mail */
  FrNetInfo ni;               /* Network information database */

  FrNetInfoChildCtx niCtx;      /* NetInfo child updater */
  uint32_t reloadNiTime;        /* Frequency */
  size_t ni_shared_mem_sz;      /* Buffer size for NetInfo shared memory */

  const char *shmtemplate;      /* Template filename for shared memory */
  const char *mutextemplate;    /* Template filename for shared memory */

  const char *niqsHost;         /* NIQS hostname */
  unsigned short niqsPort;      /* NIQS port */
  unsigned short niqsTimeout;   /* NIQS timeout */
  
  time_t niqsUpdateOffset;      /* Force complete NIQS update every X seconds */
};

typedef struct T_FrMaipContext FrMaipContext;

/*
 * This structure is passed to each of the AMTP commands.  It will store
 * pointers to the AMTP message header, the message body and some status
 * flags.
 */

#define FLAG_SCTX_HEADER_SET    0x1  /* AMTP Header has been set */
#define FLAG_SCTX_MESSAGE_SET   0x2  /* Message body has been set */
#define FLAG_SCTX_TARGET_SET    0x4  /* Recipient target has been set */
#define FLAG_SCTX_ALTTEXT_SET   0x8  /* Alternate message text has been set */

/*
 * There has to be a ZkEmail structure in there because the parsers that
 * build up the email object intertwine the body and header.
 *
 * NOTE:  The alternate text is optional.  The command does not have to be
 *        given and so the alttext pointer can be NULL.  Note that the 
 *        alttext command can be issued at anytime prior to the target command
 *
 *        The tag line is also optional.
 */

struct T_FrMaipSessionContext {
  uint32_t flags;
  ZkEmail email;
  ZkEmailHeader header;
  unsigned char * message;
  size_t message_sz;
  unsigned char * alttext;    /* Alternate message text (for dest. mailbox) */
  size_t alttext_sz;          /* Size of the alternate text */
  unsigned char * target;
  size_t target_sz;
  char * iuser_addr;          /* Email address instead of target */
  size_t iuser_addr_sz;       /* Size of the email address */
  FrEntIdExp sender;          /* Sender entity (according to mesg signature) */
  void * tgt_rb_hash;
  size_t tgt_rb_hash_sz;
  struct {
    struct T_FrMaipTagLineInfo html;
    struct T_FrMaipTagLineInfo text;
  } tagline;
  FrMaipStatus status;
};
  

typedef struct T_FrMaipSessionContext FrMaipSessionContext;
  




/*
 * Ok, its a global variable (*cringe*) but it is needed for the signal 
 * handlers so they have some clue as to what is going on 
 */

extern FrMaipContext * frMaipContext;


/*
 * Connection callback
 */

typedef int (*ZkServerMain)( struct sockaddr_in , void *);
typedef int (*ZkQueueLoop)(void *);

int RunAsStreamServer (
  ZkError ,                        /* Returns the error if one occured */
  ZkLog ,                          /* Main server log object */
  struct sockaddr_in * ,           /* Server address */
  ZkServerMain ,                   /* Connection callback */
  void * );                        /* Callback context structure */

extern
int frMaipAMTPQueue (
  ZkError err ,
  FrMaipContext * context );

int frMaipMain (
  struct sockaddr_in ,             /* Remote client address */
  void * );                        /* Key server context structure pointer */

extern int RunQueueClient(ZkError, ZkLog, ZkQueueLoop, void *);
extern int maipQueueLoop(void *);

/*
 * Internal function prototypes
 */

int frMaipContextInit( ZkError , FrMaipContext * );
int frMaipContextLoad( ZkError , FrMaipContext * , int , char ** );
int frMaipContextLoadConfig ( ZkError , FrMaipContext * );
int frMaipContextVerify ( ZkError , FrMaipContext * );
int frMaipAMTPServer ( ZkError , FrMaipContext * );
int frMaipNetInfoInit(ZkError err, FrMaipContext *context);
int frMaipSetSpoolParms(ZkError err, FrMaipContext * context);
int maipSetUpStat(ZkError err, FrMaipContext *context);
int frMaipCleanContext(ZkError err, FrMaipContext *c);
int frMaipNetInfoGetType(ZkError err, FrNetInfo ni, FrEntIdType type,
                         char **hname, unsigned short *port,
                         FrEntIdExp *entity);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* MAIP_H_ */
