/*
 * $Id: statdb.c,v 1.11 2000/04/05 18:24:24 danp Exp $
 * 
 * A set of functions for fetching and storing stat usage per nym in 
 * the FMG maip.
 */

static const char * module_id_ = "$Id: statdb.c,v 1.11 2000/04/05 18:24:24 danp Exp $";

#include "zks.h"

#include <netinet/in.h>
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <ctype.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <frNym.h>
#include "maip.h"
#include "statdb.h"


int
MAIPStatRecStore (
  ZkError err ,
  FrMaipContext * c ,
  char * nym ,
  MAIPStatRec * r ,
  int * stored )
{
  const char * func = "MAIPStatRecStore";
#ifdef ZK_FMG
  int return_code = 0;
  char buf[MAIPSTATREC_SZ];
  size_t sz = MAIPSTATREC_SZ; 
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int exists = 0;
  int rc = 0;
#else 
  (void) c;
  (void) nym;
  (void) r;
  (void) stored;
#endif /* !ZK_FMG */

  (void)module_id_; /* To silence compiler warning about
                     * module_id_ not being used.
                     */

  zkErrorClear(err);
#ifdef ZK_FMG
  if (c && nym && r && stored) {
    *stored = 0;
    if (!(zkDBTNew(err,&key) &&
          zkDBTNew(err,&data) &&
          zkDBSetKeyOrData(err,key,nym,strlen(nym)))) {
      goto finish;
    }
    if (!MAIPStatRecToBinary(err,r,buf,&sz)) {
      goto finish;
    }
    if (!zkDBSetKeyOrData(err,data,buf,sz)) {
      goto finish;
    }

    rc = zkDBReplace( err, c->statdb, key, data );
    if( !rc ) {
      goto finish;
    }
    *stored = 1;
    return_code = 1; 
finish:
    if (key) {
      zkDBTFree(NULL,&key);
    }
    if (data) {
      zkDBTFree(NULL,&data);
    }
    return return_code;
  }
#endif /* ZK_FMG */
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 * 
 *
 */

int
MAIPStatRecFetch (
  ZkError err ,
  FrMaipContext * c ,
  char * nym ,
  MAIPStatRec * r ,
  int * found )
{
  const char * func = "MAIPStatRecFetch";
#ifdef ZK_FMG
  int return_code = 0;
  char * buf =  NULL;
  size_t sz = 0; 
  ZkDBT key = NULL;
  ZkDBT data = NULL;
#else
  (void) c;
  (void) nym;
  (void) r;
  (void) found;
#endif /* !ZK_FMG */

  zkErrorClear(err);
#ifdef ZK_FMG
  if (c && nym && r && found) {
    *found = 0;
    if (!(zkDBTNew(err,&key) &&
          zkDBTNew(err,&data) &&
          zkDBSetKeyOrData(err,key,nym,strlen(nym)))) {
      goto finish;
    }
    if (!zkDBFetch(err,c->statdb,key,data,found)) {
      goto finish;
    } 
    if (!(*found)) {
      zkErrorSet_va(err,func,ZKWARN,ZKMMAIP,ZKENOTFOUND,
                    "Could not find record for %s",nym);
      /* Note although, we didnt find the record, nothing is actually
       * wrong, so we'll return a 1 in this case 
       */
      zkLogWrite_va(c->log,"stat rec for %s not found in database",nym);
      return_code = 1;
      goto finish;
    }

    if (!zkDBGetKeyOrData(err,data,(void **)&buf,&sz)) {
      goto finish;
    }
    if (!MAIPStatRecFromBinary(err,r,buf,sz)) {
      goto finish;
    }
    *found = 1;
    return_code = 1;

finish:
    if (key) {
      zkDBTFree(NULL,&key);
    }
    if (data) {
      zkDBTFree(NULL,&data);
    }
    return return_code;
  }
#endif /* ZK_FMG */
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
} 

/*
 *
 *
 */

int
MAIPStatRecToBinary (
  ZkError err ,
  MAIPStatRec * r,
  void * raw_buf ,
  size_t * sz )
{
  const char * func = "MAIPStatRecToBinary";
#ifdef ZK_FMG
  int return_code = 0;
#else
  (void) r;
  (void) raw_buf;
  (void) sz;
#endif /* !ZK_FMG */

  zkErrorClear(err);
#ifdef ZK_FMG
  if (r && raw_buf && sz) {
    if (*sz < 8) {
      zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKEINCOMPLETE,"Buffer is too short");
    } else {
      unsigned char * buf = (unsigned char *) raw_buf;

      buf[0] = (unsigned char) ((r->limit >> 8) & 0xFF);
      buf[1] = (unsigned char) (r->limit & 0xFF);

      buf[2] = (unsigned char) ((r->sent >> 8) & 0xFF);
      buf[3] = (unsigned char) (r->sent & 0xFF);

      buf[4] = (unsigned char) ((r->expire >> 24) & 0xFF);
      buf[5] = (unsigned char) ((r->expire >> 16) & 0xFF);
      buf[6] = (unsigned char) ((r->expire >>  8) & 0xFF);
      buf[7] = (unsigned char) (r->expire & 0xFF);
      *sz = 8;
      return_code = 1;
    }
    return return_code;
  }
#endif /* ZK_FMG */
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/*
 *
 *
 */

int
MAIPStatRecFromBinary (
  ZkError err ,
  MAIPStatRec * r,
  const void * raw_buf,
  size_t sz )
{
  const char * const func = "MAIPStatRecFromBinary";
#ifdef ZK_FMG
  int return_code = 0;
#else 
  (void) r;
  (void) raw_buf;
  (void) sz;
#endif /* !ZK_FMG */

  zkErrorClear(err);
#ifdef ZK_FMG
  if (r && raw_buf) {
    if (sz < 8) {
      zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKEINCOMPLETE,"Buffer is too short");
    } else {
      const unsigned char * buf = (const unsigned char *) raw_buf;
      r->limit  = (buf[0] <<  8) | buf[1];
      r->sent   = (buf[2] <<  8) | buf[3];
      r->expire = (buf[4] << 24) | (buf[5] << 16) | (buf[6] << 8) | buf[7];
      return_code = 1;
    }
    return return_code;
  }
#endif /* ZK_FMG */
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

