/*
 * $Id: fr_fmgstatcheck.c,v 1.2 2000/02/29 16:36:21 alext Exp $
 */

# define ZK_FMG 1

#include "zks.h"
#include <netinet/in.h>
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <netdb.h>
#include <ctype.h>
#include "zkList.h"
#include "frNym.h"
#include "frNymClient.h"
#include "frNymUtil.h"
#include "../src/maip.h"
#include "../src/statdb.h"
#include "zkAlloc.h"
#include "zkConfig.h"
#include "frNetInfo.h"
#include "zkParms.h"
#include <netdb.h>
#include "zkDNS.h"
#include <sys/socket.h>
#include <arpa/inet.h>
#include "frNiqs.h"
#include "zkDebug.h"

enum OUTPUT_FORMAT {
  FORMAT_DEFAULT,
  FORMAT_XML,
};

# define P_FORMAT_XML "formatXML"
# define P_SENDER_NAME "senderName"
# define P_SENDER_TYPE "senderType"

# define OPT_FORMAT_XML "--" P_FORMAT_XML
# define OPT_SENDER_NAME "--" P_SENDER_NAME
# define OPT_SENDER_TYPE "--" P_SENDER_TYPE

/* 
 * Function Prototypes
 */

static int show_usage(
  FrMaipContext * context,
  FILE * out_stream );

static int load_context(
  ZkError         err,
  FrMaipContext * context,
  FrEntIdExp *    sender,
  int *           format_id,
  int             argc,
  const char *    argv[] );

static int get_sender_stats(
  ZkError            err,
  FrMaipContext *    context,
  FrEntIdExp *       sender,
  size_t *           num_sent,
  size_t *           limit );

int dump_output(
  ZkError               err,
  const FrMaipContext * context,
  FILE *                outstream,
  const FrEntIdExp *    sender,
  int                   format_id,
  size_t                num_sent,
  size_t                limit );


/**
 *
 */
static int show_usage(
  FrMaipContext * context,
  FILE *          out_stream )
{
  if (!context) {
    return 0;
  }
  if (out_stream == NULL) {
    out_stream = stderr;
  }
  fprintf(out_stream,"%s\n\nUsage:  %s options...\n"
          "\n%s\n\tSends debugging information to stderr\n"
          "\n%s config_file\n\tSpecifies the configuration file\n"
          "\n%s\n\tDisplays this help screen\n"
          "\n%s port_number\n\tThe MAIP listening port (default is %d)\n"
          "\n%s\n\tDisplay the version string of the server\n"
          "\n%s pid_file\n\tSpecifies a file for the process pid\n"
          "\n%s unique_name\n\tSpecifies a unique name for the server\n"
          "\n%s integer\n\tMaximum amount of data per AMTP command\n"
          "\n%s verbose_level\n\tVerboseness of log files (default is %d)\n"
          "\n%s duration\n\tMax connection duration in secs (default is %d)\n"
          "\n%s connections\n\tMax num of simultaneous conn (default is %d)\n"
          "\n%s yes | no\n\tRun the server in daemon mode (default is yes)\n"
          "\n%s log_file\n\tSpecifies the server log file\n"
#ifdef ZK_FMG
          "\n%s seconds\n\tNym server timeout (default is %d, FMG option)\n"
#endif
          "\n%s spooldir\n\tSpecify the spool directory (default %s)\n"
          "\n%s maxclients\n\tClients to run in parallel (default %d)\n"
          "\n%s queuetime\n\tQueue run interval [in secs] (default %d)\n"
          "\n%s mxmsgperloop\n\tMaximum number of processed messages per queue run (default %d)\n"
          "\n%s mta_path\n\tPath to Mail Transfer Agent (default %s)\n"
          "\n%s mta_argument\n\tOption to pass to MTA (multiple instances)\n"
#ifdef ZK_FMG
          "\n%s file\n\tAlternate message text file (FMG only)\n"
          "\n%s file\n\tTagline text file (FMG only)\n"
          "\n%s file\n\tTagline HTML file (FMG only)\n"
          "\n%s keyquerytimeout\n\tTimeout for key lookup (FMG only)\n"
          "\n%s keyqueryretries\n\tNum of retries for key lookups (FMG only)\n"
#endif
          "\n%s dbname\n\tMail usage statistics database [default %s]\n"
          "\n%s dbname\n\tNym to blocked email address database [default %s]\n"
          "\n%s cryption dir\n\tStorage directory of private crypto keys\n"
          "\n%s signature dir\n\tStorage directory of private sig keys\n"
          "\n%s passphrase_prompt\n\tPrompt for system passphrase\n"
          "\n%s passphrase_file\n\tPassphrase file\n"
          "\n%s niqshostname\n\tNIQS hostname\n"
          "\n%s niqsport\n\tNIQS port\n"
          "\n%s niqstimeout\n\tNIQS timeout\n"
          "\n%s niqsfrequency\n\tNIQS frequency of updates in secs\n"
          "\n%s buffersize\n\tNIQS shared memory buffer size (in kilobytes)\n"
          "\n%s sharedmemorytemplate\n\tNIQS shared memory filename template\n"
          "\n%s mutexttemplate\n\tNIQS mutex filename template\n"
          "\n%s\n\tprints the output in XML format\n"
          "\n%s name\n\tthe name of the entity you want to check\n"
          "\n%s type\n\tthe type of the entity you want to check\n"
          "\nCommand line options override settings in the config file\n",
          VERSION_AMTP_STR,context->progname,
          OPT_DEBUG,
          OPT_CONFIG,
          OPT_HELP,
          OPT_PORT,DEFAULT_PORT,
          OPT_VERSION,
          OPT_PIDFILE,
          OPT_NAME,
          OPT_MAXDATA,
          OPT_VERBOSE,DEFAULT_VERBOSE,
          OPT_MAXCONNTIME,DEFAULT_MXCONNTIME,
          OPT_MAXCONN,DEFAULT_MXCONN,
          "--"P_DAEMON,
          OPT_LOGFILE,
#ifdef ZK_FMG
          OPT_NYMSERVTOUT,DEFAULT_NSTIMEOUT,
#endif
          OPT_SPOOLDIR, DEFAULT_SPOOLDIR,
          OPT_MAXCLIENTS, DEFAULT_MAXCLIENTS,
          OPT_QUEUETIME, DEFAULT_QUEUETIME,
          OPT_MAXMSGPLOOP, DEFAULT_MAXMSGPLOOP,
          OPT_MTAPATH, DEFAULT_MTA,
          OPT_MTAOPT,
#ifdef ZK_FMG
          OPT_ALTTEXT,
          OPT_TAGLINE_FILE_TEXT,
          OPT_TAGLINE_FILE_HTML,
          OPT_KEYQRYTOUT,
          OPT_KEYQRYRETR,
#endif
          OPT_STATDB, DEFAULT_STATDB,
          OPT_NYMBLOCKDB, DEFAULT_NYMBLOCKDB,
          OPT_CRYPTDIR,
          OPT_SIGDIR,
          OPT_PPPROMPT,
          OPT_PPFILE,
          OPT_NIQS_SERVER,
          OPT_NIQS_PORT,
          OPT_NIQS_TIMEOUT,
          OPT_NIQS_FREQUENCY,
          OPT_NIQS_SHARED_MEM_SIZE,
          OPT_NIQS_SHMTEMPLATE,
          OPT_NIQS_MUTEXTEMPLATE,
          OPT_FORMAT_XML,
          OPT_SENDER_NAME,
          OPT_SENDER_TYPE);
  return 1;
}
/**
 * Sets up all of the context we need to check the sending stats
 */

static int load_context(
  ZkError         err,
  FrMaipContext * context,
  FrEntIdExp *    sender,
  int *           format_id,
  int             argc,
  const char *    argv[] )
{
  static const char * func = "context_load";
  int return_code = 1;
  int opt = 1;

  if (context && argv && sender) {
  
    memset(sender, 0, sizeof(*sender));

    /* 
     * Set the program name
     */

    if ((context->progname = strrchr(argv[0],'/')) == NULL) {
      context->progname = argv[0];
    } else {
      context->progname++;
    }

#ifdef ZK_FMG
    context->flags |= FLAG_FMG;
#else
    context->flags &= ~FLAG_FMG;
#endif

    /*
     * Are we looking for some information instead of trying to run the
     * program?
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_HELP) == 0) {
        if (show_usage(context,stderr)) {
          exit(EXIT_SUCCESS);
        } else {
          exit(EXIT_FAILURE);
        }
      } else if (strcasecmp(argv[opt],OPT_VERSION) == 0) {
        fprintf(stderr,"%s\n",VERSION_AMTP_STR);
        exit(EXIT_SUCCESS);
      }
    }

    /*
     * Do we have to turn on debug mode? or are we running as the FMG?
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_DEBUG) == 0) {
        context->flags |= FLAG_DEBUG;  
        break;
      }
    }

    /*
     * Find and load the configuration file 
     */

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],OPT_CONFIG) == 0) {
        if (++opt < argc) {
          context->configfile = argv[opt]; 
        } else {
          zkErrorSet_va(err,func,ZKERR,ZKMMAIP,ZKERROR,
                        "%s option missing parameter.",OPT_CONFIG);  
          fprintf(stderr,"%s:  You must specify a configuration file.\n",
                  context->progname);
          return 0;
        }
        break;
      }
    } 
    if (context->configfile) {
      if (!frMaipContextLoadConfig(err,context)) {
        return 0;
      }
    }

    /*
     * Process command line parameters.  These overide options that are
     * present in the configuration file if there is some sort of conflict.
     */

#ifndef WIN32
#warning Need to check command line arguments for stat client values 
#endif

    for (opt = 1; opt < argc; opt++) {
      if (strcasecmp(argv[opt],"--"P_DAEMON) == 0) {
        if (++opt < argc) {
          if ((strcasecmp(PARMVAL_NO,argv[opt]) == 0) ||
              (strcasecmp("0",argv[opt]) == 0)) {
            context->flags |= FLAG_DONTFORK;
          }
        } else {
          fprintf(stderr,
                  "%s:  The --"P_DAEMON" option must be set to yes or no.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PORT) == 0) {
        if (++opt < argc) {
          context->port = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s:  You must specify a port number.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PIDFILE) == 0) {
        if (++opt < argc) {
          context->pidfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a pid file name.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_NAME) == 0) {
        if (++opt < argc) {
          context->name = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a server name.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXDATA) == 0) {
        if (++opt < argc) {
          context->maxdata = atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s:  You must specify a maximum amount of data.\n",
                  context->progname);
          return_code = 0;
        } 
      } else if (strcasecmp(argv[opt],OPT_STATDB) == 0) {
        if (++opt < argc) {
          context->statdbname = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a mail usage stat database.\n",
                  context->progname);
          return_code = 0;
        }

      } else if (strcasecmp(argv[opt],OPT_NYMBLOCKDB) == 0) {
        if (++opt < argc) {
          context->nymblockdbname = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a nym -> address blocking database.\n",
                  context->progname);
          return_code = 0;
        }
    
      } else if (strcasecmp(argv[opt],OPT_ALTTEXT) == 0) {
        if (++opt < argc) {
          context->alttextfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify an alternate text file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_TAGLINE_FILE_TEXT) == 0) {
        if (++opt < argc) {
          context->tagline.text.file = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a tagline text file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_TAGLINE_FILE_HTML) == 0) {
        if (++opt < argc) {
          context->tagline.html.file = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a tagline HTML file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_LOGFILE) == 0) {
        if (++opt < argc) {
          context->logfile = argv[opt];
        } else {
          fprintf(stderr,"%s:  You must specify a logfile.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_VERBOSE) == 0) {
        if (++opt < argc) {
          context->verbose = (unsigned int) atoi(argv[opt]);
        } else {
          fprintf(stderr,"%s: You must specify a verbose logging level.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_NYMSERVTOUT) == 0) {
        if (++opt < argc) {
          context->nstimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                 "%s:  You must specify a nym server connection timeout.\n",
                 context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCONN) == 0) {
        if (++opt < argc) {
          context->mxconn = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of connections.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCONNTIME) == 0) {
        if (++opt < argc) {
          context->mxconntime = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum connection time (in seconds).\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_SPOOLDIR) == 0) {
        if (++opt < argc) {
          context->spooldir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify the spool directory.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXCLIENTS) == 0) {
        if (++opt < argc) {
          context->maxclients = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of clients.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_QUEUETIME) == 0) {
        if (++opt < argc) {
          context->queuetime = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a queue run interval (in seconds).\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_MAXMSGPLOOP) == 0) {
        if (++opt < argc) {
          context->mxmsgperloop = (int) atoi(argv[opt]);
        } else {
          fprintf(stderr,
                  "%s:  You must specify a maximum number of processed messages per queue run.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_CRYPTDIR) == 0) {
        if (++opt < argc) {
          context->cryptDir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a crypt filename.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_SIGDIR) == 0) {
        if (++opt < argc) {
          context->sigDir = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a signature directory.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PPPROMPT) == 0) {
        if (++opt < argc) {
          context->prompt = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a system passphrase prompt.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_PPFILE) == 0) {
        if (++opt < argc) {
          context->passfile = argv[opt];
        } else {
          fprintf(stderr,
                  "%s:  You must specify a passphrase file.\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_MTAPATH) == 0) {
        if (++opt < argc) {
          context->mtaPath = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a MTA pathname\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_MTAOPT) == 0) {
        if (++opt < argc) {
          /* Create a new ZkList and set the free function to NULL */
          if (context->mtaOpts == NULL) {
            if (!zkListNew(err, &context->mtaOpts) ||
                !zkListSetDataFreeFunction(err, context->mtaOpts, NULL, 
                                           NULL)) {
              zkErrorSet(err, func, ZKERR, ZKMMAIP, 
                         ZKERROR, "Failed to create a new ZkList");
              return 0;
            }
          }
          
          if (!zkListInsert(err, context->mtaOpts, argv[opt], LIST_TAIL)) {
            zkErrorSet(err, func, ZKERR, ZKMMAIP, 
                       ZKERROR, "Failed to insert into ZkList");
            return 0;
          }
        } else {
          fprintf(stderr, "%s: You need to specify an option for the MTA\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_KEYQRYTOUT) == 0) {
        if (++opt < argc) {
          context->kqtimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a key query timeout\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_KEYQRYRETR) == 0) {
        if (++opt < argc) {
          context->kqretries = atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify number of key query retries\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_SERVER) == 0) {
        if (++opt < argc) {
          context->niqsHost = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS hostname\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_PORT) == 0) {
        if (++opt < argc) {
          context->niqsPort = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS port\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_TIMEOUT) == 0) {
        if (++opt < argc) {
          context->niqsTimeout = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS timeout\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_FREQUENCY) == 0) {
        if (++opt < argc) {
          context->reloadNiTime = (unsigned short) atoi(argv[opt]);
        } else {
          fprintf(stderr, "%s: You must specify a NIQS frequency\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_SHMTEMPLATE) == 0) {
        if (++opt < argc) {
          context->shmtemplate = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS shared mem template\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_NIQS_MUTEXTEMPLATE) == 0) {
        if (++opt < argc) {
          context->mutextemplate = argv[opt];
        } else {
          fprintf(stderr, "%s: You must specify a NIQS mutex template\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt],OPT_CONFIG) == 0) {
        opt++;
      } else if (!(strcasecmp(argv[opt],OPT_VERSION) &&
                   strcasecmp(argv[opt],OPT_DEBUG) &&
                   strcasecmp(argv[opt],OPT_FMG) &&
                   strcasecmp(argv[opt],OPT_HELP))) {
        /* Do diddly */
      } else if (strcasecmp(argv[opt],OPT_FORMAT_XML) == 0) {
        *format_id = FORMAT_XML;
      } else if (strcasecmp(argv[opt], OPT_SENDER_NAME) == 0) {
        if (++opt < argc) {
          strncpy(sender->name, argv[opt], sizeof(sender->name)-1);
        } else {
          fprintf(stderr, "%s: You must specify a sender name\n",
                  context->progname);
          return_code = 0;
        }
      } else if (strcasecmp(argv[opt], OPT_SENDER_TYPE) == 0) {
        if (++opt < argc) {
          sender->type = frEntTypeFromStr(argv[opt]);
          if (sender->type == FRENT_TYPE_UNUSED_) {
            fprintf(stderr, "%s: The entity type \"%s\" is invalid\n",
                    context->progname, argv[opt]);
            return_code = 0;
          }
        } else {
          fprintf(stderr, "%s: You must specify a sender type\n",
                  context->progname);
          return_code = 0;
        }
      } else {
        fprintf(stderr,"%s:  Unrecognized option \"%s\".\n",
                context->progname,argv[opt]);
        return_code = 0;
      }
    }

    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  return 0;
}

/**
 * Retrieves the number of messages sent by a nym and the nym's current mail
 * limit, as they will be seen by the fmg-maip when the nym next attempts to
 * send a message.
 */

static int get_sender_stats (
  ZkError            err,
  FrMaipContext *    context,
  FrEntIdExp * sender,
  size_t *           num_sent,
  size_t *           limit )
{
# define SENDER_C_SZ (128)
  const char * func = "GetSenderStats";
  MAIPStatRec r;
  int found = 0;

  zkErrorClear(err);
  if (!(context && sender && num_sent && limit)) {
    zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
    
  } else {

    *num_sent = 0;
    *limit = 0;
   
    /*
     * The FMG and NymServer entities can send as much mail as they want
     */

    if ( (sender->type == FRENT_TYPE_FMG) 
         || (sender->type == FRENT_TYPE_NYM_SRV)) {
      *limit = ~((size_t)0);
      return 1;
    }

    /*
     * Retrieve stat data for this sender
     */

    memset(&r,0,sizeof(MAIPStatRec));
    if (!MAIPStatRecFetch(err,context,sender->name,&r,&found)) {
      return 0;
    }

    if (found) {

      /*
       * If we find a record, use it directly
       */

      *num_sent = r.sent;
      *limit = r.limit;
    }
    else {

# if (0)

      /*
       * If we don't have a current stat entry for this sender, query the
       * nym server to get a description of the sender nym.
       */

      FrRemoteNym nym = NULL;
      NymServer nymserver;
      NymServerResponse nsresp;
      char * l = NULL;
      size_t lsz = 0;

      /*
       * Bug: use the nymserver signature key to verify the data
       * returned from the nymserver
       */

      strncpy(nymserver.nymserver,frMaipContext->nshost,NSHOST_SZ);
      nymserver.port = frMaipContext->nsport;
      nymserver.timeout = frMaipContext->nstimeout;
      nymserver.sigkey = NULL;
      if (!frRemoteNymGet(err,&nym,sender->name,&nymserver,&nsresp)) {
        zkErrorSet_va(err,func,ZKERR,ZKMMAIP,ZKENOTFOUND,
                      "Sender nym \"%s\" not found",sender->name);
        return 0;
      }
      frRemoteNymResetParameterSearch(NULL,nym);
      if (!frRemoteNymGetParameter(err,nym,NymEmailLimit,
                                   (const unsigned char **)&l,&lsz)) {
        zkErrorSet_va(err,func,ZKERR,ZKMMAIP,ZKERROR,
                      "Couldn't retrieve nym e-mail limit for %s",
                      sender->name);
        frRemoteNymFree(NULL,&nym);
        return 0;
      } 
      *limit = (size_t) atoi(l);
      frRemoteNymFree(NULL,&nym);
# endif
    }
    return 1; 
  }
  return 0;
}


/**
 * Dumps a report of the given stats to the given output stream.
 */

int dump_output(
  ZkError               err,
  const FrMaipContext * context,
  FILE *                outstream,
  const FrEntIdExp *    sender,
  int                   format_id,
  size_t                num_sent,
  size_t                limit )
{
  static const char * const func = "dump_output";
  int result = 0;
  const char * format_str = "%s has sent %u messages with a limit of %u\n";
  zkErrorClear(err);
  if (!(context && outstream)) {
    zkErrorSet(err,func,ZKERR,ZKMMAIP,ZKENULLPTR,NULL);
  } else {

    if (format_id == FORMAT_XML) {
      format_str = ( "<sender>"          \
                     "<name>%s</name>"   \
                     "<sent>%u</sent>"   \
                     "<limit>%u</limit>" \
                     "</sender>" );
    }

    fprintf(outstream, format_str, sender->name, num_sent, limit);
    result = 1;
  }

  return (result);
}


int main(int argc, const char * argv[])
{
  FrMaipContext  context;   
  FrEntIdExp     sender;
  int            exit_code = 1;
  int            format_id = FORMAT_DEFAULT;
  size_t         sent = 0;
  size_t         limit = 0;

  frMaipContext = &context;

  if ( frMaipContextInit(NULL,&context)
       && load_context(context.err, &context, &sender, &format_id, argc, argv)
       && zkDB_Open(context.err,&(context.statdb),context.statdbname)
       && get_sender_stats(context.err, &context, &sender, &sent, &limit)
       && dump_output(context.err,&context,stdout,&sender,format_id,sent,limit) ) {
    exit_code = 0;
  }

  if (context.statdb) {
    zkDBClose(NULL,&(context.statdb));
  }

  frMaipCleanContext(NULL,&context);

  return (exit_code);
}
