/*
 * $Id: frPsrCache.h,v 1.4 2001/04/19 19:23:12 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FR_PSR_CACHE_H_
#define FR_PSR_CACHE_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct T_FrPsrCache * FrPsrCache;

/*
 *
 */
typedef enum
{
  EPSRStampTtl,
  EPSRStampValue,
  EPSRStampLast     /* Must be the last one */
} EPSRStampType ;

typedef struct
{
  EPSRStampType  type ;
  union
  {
    unsigned short  ttl ; /* EPSRStampTtl (store only) */
    struct
    {
       size_t        len ;
       const void   *ptr ;
    }               val ; /* EPSRStampValue */
  } u ;
} FrPSRStamp ;


DLLExport int frPsrCacheNew (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache * qc );

DLLExport int frPsrCacheFree (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache * qc );
   
DLLExport int frPsrCacheSetName (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc ,
  ZK_IN    const char * name );

DLLExport int frPsrCacheInit (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc );

DLLExport int frPsrCacheTerminate (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc );

DLLExport int frPsrCacheLookup (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc ,
  ZK_IN    const void * qry ,         /* Query buffer */
  ZK_IN    size_t qry_sz ,            /* Query buffer size */
  ZK_OUT   void ** rsp ,              /* Response buffer (NULL if norhing found) */
  ZK_OUT   size_t * rsp_sz ,          /* Size of response buffer (0 is rsp == NULL) */
  ZK_OUT   int * in_cache ,           /* Set to non-zero if in cache */
  ZK_IN    const FrPSRStamp *stamp ); /* Check value */

DLLExport int frPsrCacheStore (  
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc ,
  ZK_IN    const void * qry ,         /* Query buffer */
  ZK_IN    size_t qry_sz ,            /* Query buffer size */
  ZK_OUT   void * rsp ,               /* Response buffer */
  ZK_OUT   size_t rsp_sz ,            /* Size of response buffer */
  ZK_IN    const FrPSRStamp *stamp ); /* Check value */

DLLExport int frPsrCacheRemove (
  ZK_INOUT ZkError err ,
  ZK_INOUT FrPsrCache qc ,
  ZK_INOUT const void * qry ,
  ZK_IN    size_t qry_sz ,      /* Query buffer size */
  ZK_OUT   int * found );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ! FR_PSR_CACHE_H_ */
