/**
 * zkAA.h
 *
 * Persistent Associative Arrays
 * $Id: zkAA.h,v 1.11 2000/11/24 15:04:09 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 */ 

#ifndef ZKAA_H
#define ZKAA_H

#include <zkError.h>
#include <zkDB.h> 
#include <zkIntTypes.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


  typedef struct T_ZkAA *ZkAA;

  extern int zkAANew(ZkError, const char *, ZkAA *);
  extern int zkAAFree(ZkError, ZkAA *);
  extern int zkAAGet(ZkError, ZkAA, const void *, size_t, void **, size_t *);
  extern int zkAAFind(ZkError, ZkAA, const void *, size_t, int *);
  extern int zkAAPut(ZkError, ZkAA, const void *, size_t, const void *, size_t );
  extern int zkAADelete(ZkError err, ZkAA aa, const void *key, size_t ks);
  extern int zkAAGetNextReset(ZkError, ZkAA);
  extern int zkAAGetNext(ZkError, ZkAA,
			 void **, size_t *,  
			 void **, size_t *, int *found);


  extern int zkAA_CursorOpen (ZkError err, ZkAA aa);
  extern int zkAA_CursorClose(ZkError err, ZkAA aa);
  extern int zkAA_CursorAbort  (ZkError err, ZkAA aa);
  
  extern int zkAA_CursorGetNext (ZkError err, ZkAA aa,
                          void **key, size_t *ks, void **data, size_t *ds, int *found);
  
  extern int zkAA_CursorDeleteCurrent  (ZkError err, ZkAA aa);
  
  
  extern int zkAA_CursorUpdateCurrent  (ZkError err, ZkAA aa, const void *key, size_t ks,
                                 const void *data, size_t ds);  
#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* ZKAA_H */
