/*
 * $Id: zkDB_priv.h,v 1.4 2001/04/19 19:23:12 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKDB_priv_h
#define ZKDB_priv_h

#ifdef ZKMOD
#undef ZKMOD
#endif

#define ZKMOD ZKMDB
#include <frFilenames.h>
#include <time.h>
#include "zkRemotePartition.h"
#include "zkLocalPartition.h"
#include "zkConfig.h"
#include <time.h>
/* definitions   */
#define  ZKDB_MAIN_SESSION "main"

#define  P_PARTINIONS       "partitions"
#define  P_ISOLATED         "isolated_partitions"
#define  P_IS_REMOTE        "IsRemote"
#define  P_MAX_LIMIT        "MaxLimit"
#define  P_MIN_LIMIT        "MinLimit"
#define  P_IS_HASHED        "IsHashed"
#define  P_CONNECTION_TIME  "ConnectionTimeOut"
#define  P_AUTHENTICATION   "AuthKey"
#define  P_PARTITION_TYPE   "PartitionType"
#define  P_READONLY         "ReadOnly"   
#define  P_TRANSACTMODE     "TransactionMode"
#define  P_SINGLEUSERMODE    "SingleUserMode"
#define  P_LOCK_ATTEMPTS    "MaxLockAttempts"
#define  P_DEFAULT_HOME     "DefaultHomeDir"
#define  P_DEFAULT_LOGDIR   "DefaultTransactLogDir"  
#define  P_RECOVER          "Recover"
#define  P_ADDRESS_PATH     "AddressPath"
#define  P_STARTPOS         "LimitStartPos"
#define  P_PARTITION_STATUS "Status"

#define  PARTITION_SEPARATOR ","

 
#define  MAX_STARTPOS   64    /* start position for the key limit. Can't be more the key size and 64 */   
/* char * host; */


/*  statistic structures and defines. They are not used yet in usial production environment. And only for testing purpose */
/*  allow to collect different timing information                                                                         */
#ifndef  NDEBUG   /* use only in debug version */ 

/* different type of the transactions */
#define  ZK_TRANS_TYPES 5                      /* READ, UPDATE, DELETE transactions */
#define  READ_TRANS     0
#define  UPDATE_TRANS   1
#define  DELETE_TRANS   2
#define  OPEN_TRANS     3
#define  CLOSE_TRANS    4

#define TIME_STAT            1024
#define ZKS_TIME() clock()
#define ZKS_TIME_TYPE clock_t
#define TIME_FACTOR CLOCKS_PER_SEC

/* statistic structure, contains all statistical information */
   
struct T_ZkStatistics {
 long          transact [ZK_TRANS_TYPES];
 ZKS_TIME_TYPE time[ZK_TRANS_TYPES];  /* times of the transaction spending
                                                 in deadlock mode */ 
};

typedef struct T_ZkStatistics * zkStatistics;  
/* statistical macros  */

#define  ZK_STAT_TRANS_TIME(db,started,type)   (db->stat.time[type]+=ZKS_TIME()-started, db->stat.transact[type]++ )

#define  outputFunc zkDebug  /* can use any printf kind function to output stat */

#define  SUM_TRANS(db, vl) (vl[READ_TRANS] + vl[UPDATE_TRANS] + vl[DELETE_TRANS])
#define  OUTPUT_TRANS(db,vt,name)    outputFunc ("\n %s :  FETCH %ld, UPDATE  %ld, DELETE  %ld, TOTAL %ld ", name,\
                                                                                                   vt[READ_TRANS], \
                                                                                                   vt[UPDATE_TRANS], \
                                                                                                   vt[DELETE_TRANS], \
                                                                                                   SUM_TRANS(db, vt))

#define  OUTPUT_TRANS_TIME(db,vp,name)   outputFunc("\n %s :  FETCH %ld unit, UPDATE  %ld unit, DELETE  %ld unit, TOTAL %ld unit.", name, \
                                                                                                       vp[READ_TRANS] / TIME_FACTOR, \
                                                                                                       vp[UPDATE_TRANS] / TIME_FACTOR, \
                                                                                                       vp[DELETE_TRANS] / TIME_FACTOR, \
                                                                                                       (SUM_TRANS(db, vp)) / TIME_FACTOR)

#define OUTPUT_OPENCLOSE(db,vk,name)     outputFunc("\n %s :  OPEN  %ld, CLOSE   %ld, ", name,\
                                                                                          vk[OPEN_TRANS], \
                                                                                          vk[CLOSE_TRANS])

#define OUTPUT_OPENCLOSE_TIME(db,vd,name)  outputFunc("\n %s :  OPEN  %ld, CLOSE   %ld, ", name,\
                                                                     vd[OPEN_TRANS] / TIME_FACTOR, \
                                                                     vd[CLOSE_TRANS] / TIME_FACTOR)


#define PRINT_STAT_TIME(db)    OUTPUT_TRANS_TIME(db,db->stat.time,    "TRANSACTIONS TIME:")
#define PRINT_STAT(db)         OUTPUT_TRANS(db,db->stat.transact,    "TRANSACTIONS:")   
#define PRINT_OPEN_CLOSE_STAT(db)         OUTPUT_OPENCLOSE(db,db->stat.transact,    "DB OPEN/CLOSE:")  
#define PRINT_OPEN_CLOSE_TIMESTAT(db)         OUTPUT_OPENCLOSE_TIME(db,db->stat.time,    "DB OPEN/CLOSE TIME:")


#else  /* for release version define empty macroses for statistical information */

#define ZKS_TIME_TYPE clock_t /* keep the same */
#define ZK_STAT_TRANS_TIME(db,started,type)
#define ZKS_TIME() 0         /* always return 0  */
#define SUM_TRANS(db, vl) 
#define OUTPUT_TRANS(db,vl,name)  
#define OUTPUT_TRANSTIME(db,vl,name)  
#define PRINT_STAT(db)
#define PRINT_STAT_TIME(db) 
#define PRINT_OPEN_CLOSE_STAT(db)
#define PRINT_OPEN_CLOSE_TIMESTAT(db) 

#endif  /* end of statistic  */


/*  defines used as default parameters for database environment, overwriting Berkley DB settings   */ 

#define MAX_NUMB_LOCKTRIES 20

#define ZKDBHOME FREEDOM_DBDIR
#define ZKDBLOCK FREEDOM_LOCKDIR
#define ZKDBLOG  FREEDOM_LOGDIR

#define ZKDB_DEFAULT_KEY_TABLE_SIZE 10000   
#define ZKDB_MAX_LOCKS  1024

#define  DEFAULT_CACHE_SIZE  1048576   /* 1 MgByte   Memory cache size                             */               
#define  DEFAULT_LOG_SIZE    10485760  /* 10 MGBytes  Log file size                                */
#define  DEFAULT_HASH_SIZE   262144    /* 256 K        hash table size                             */
#define  DEFAULT_PAGE_SIZE   0         /* use system   default page size                           */
#define  DEFAULT_CHECKPOINT_TIME  5    /* minimum time between checkpoints                         */
#define  DEFAULT_CHECKPOINT_SIZE  32   /* minimum number kbytes for checkpoint                     */ 
#define  ZKPR_LOCKS 10
#define  ZK_DB_DEFAULT_CONNECTION_TIME 30  /* 1 min */


#define  MAX_NUMBER_OF_PARTITIONS 256
#define  MAX_NUMBER_OF_TRANSACTIONS 256 
#define  PARTITION_SHIFT          8

struct T_ZkPartitionLimit { 
  /* string limit */
  char * StringLimit;
  int  LimitLength;
  /* range limit */ 
  int  rangeLimit; /* Used only when hashing is used.   */ 
} ; 

typedef struct T_ZkPartitionLimit  * ZkPartitionLimit;
#define PARTITION_DISABLED  0  /* partition disabled. It is used only for the purpose to get data for the partition restructuring */
#define PARTITION_ENABLED   1  /* default value */
#define ENABLED_STR  "ENABLED" 
#define DISABLED_STR "DISABLED"




struct T_ZkPartition {
  char * name;
  int    IsRemote;
  int    IsHashed;
  
  struct T_ZkPartitionLimit  minLimit;
  struct T_ZkPartitionLimit  maxLimit; 
  unsigned int    StartLimitPosition;  /* start limit comparison starting from this position */ 
 
 union { 
   struct T_ZkLocalPartition local;
   struct T_ZkRemotePartition remote;
 }  partition;
  int status;      /* partition status. Can take several values:
                       PARTITION_ENABLED,
                       PARTITION_DISABLED
                       
                   */
  int initialized;                      /* Structure has been initialized */

};

typedef struct T_ZkPartition * ZkPartition;

/** Stuff that needs to be remembered for a Database */
struct T_ZkDB {
 
  char *name;		            	/* The name of the database */
  ZkConfig config;                      /* database system configuration  */
  uint32_t flags;	               	/* Main creation flags */
  struct  T_ZkPartition  *  partitions;  /* array of partitions */
  int partition_number;                 /* number of partitions */ 
  int current_partition;                /* used for the scan operations */

#ifndef NDEBUG 
    struct T_ZkStatistics   stat;         /* statistical information */
#endif


};

int zkPartitions_LocatePartition(ZkError err, ZkDB  db,  ZkDBT key, ZkPartition * partition);
int   ZkPartitions_GetCurrentPartition(ZkError err,ZkDB  db,ZkPartition *  partition);

#endif /* ZKDB_priv_h */
