/*
 * $Id: zkLocalPartition.h,v 1.10 2001/04/19 19:23:12 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKLocalPartition_h
#define  ZKLocalPartition_h
#ifdef ZKMOD
#undef ZKMOD
#endif

#define ZKMOD ZKMDB
#include <frFilenames.h>
#include <time.h>
#include "zkLock.h"

#define ZKDB_DEFAULT_KEY_TABLE_SIZE 10000
#define ZKDB_MAX_LOCKS  1024
#define ZKPR_LOCKS 10



/* support macros */
#define ZK_FREE_PARTITION_ENV(partition) if((partition) && (partition)->dbenv){ free((partition)->dbenv); (partition)->dbenv = NULL;}
#define IS_ENV_NOT_STARTED(db) (!(partition) || !((partition)->dbenv))

/* this stuff should be somewhere in  other standard header file */

#define MAX_PROC_LENGTH 20
#define MAX_FILE_NAME   80
#define FORMAT_ADDITION ".*"
#define ERROR_SET_PREFIX(db,prefix)  (db->dbenv->db_errpfx=prefix)  



struct T_ZkLocalPartition {
  int initialized;
  const char * name;
  char * homedir;               /* The base directory of the db */
  char * logdir;                /* log directory */
  DBTYPE type;                  /* The type of the database */
  int    mode;                  /* Main permission flags */
  DB_ENV *dbenv;                /* The DB env pointer */
  uint32_t envflags;            /* Subsystem flags */
  DB_INFO *dbinfo;              /* Database internal defaults */
  DB *dbp;                      /* Pointer to the database */
  DBC *dbc;                     /* Cursor for the database */
  int db_reset;                 /* Reset the cursor to the start */
  int lockingEnabled;           /* Flag to tell us that we have locking on */
  uint32_t locker;              /* The unique ID for this database */
  DB_LOCK lock[ZKPR_LOCKS];     /* lock pointer */
  long locksHeld;               /* Number of locks currently held */
  int max_lock_tries;           /* hoe many times we have to try to receive lock */
  
  int singlemode;               /* specified that database access in the singlemode; */
  int transmode;                /* specified that transactions used for the database access */
  /* Transaction structure */
  int transactionsEnabled;
  int flags;
  DB_TXNMGR * txnmngr;
  DB_LOCKTAB * locktb;
  DB_TXN  *    transaction;
  int          readonly;        /* database in read only mode */
  DBT dblock;                   /* Lock for the whole database */
  /* database options */
  size_t    db_cachesize;       /* suggested cache size                      */
  size_t    db_pagesize;        /* sugested page size                        */
  size_t    mp_size;            /* sugested memory pool size                 */
  size_t    lg_max;             /* maximum log file size                     */
  size_t    hash_size;          /* suggested hash table size                 */
  size_t    min_checkpointtime; /* min checkpoint time in minutes            */
  size_t    min_checkpointsize; /* min checkpoint size in KB                 */
  int       flash_log;          /* flash transaction  log                    */
  int       flash_log_time;     /* timeout between flashing of the  log file */
  int       transactionmode;    /* enable or disable transactions            */
  int       lockmode;           /* activate locking                          */
  int       keylock;            /* enable or disable key locking             */
  int       waitOnLock;         /* defines behaviour in case when  the lock of the object, should it wait until object will be free or return error */
  int       checkpoints_active; /* do checkpoint operations or not */       
  int       startrecovery;      /* always start recovery operation when opening database */
  int       tx_max;             /* maximum number of concurrent transactions */
  int       checklock;          /* enable check lock. Used to minimize lock stagnation problem  */
  DB_TXN  * cursor_transaction;
  ZkLock    dbFileLock;  
};

typedef struct T_ZkLocalPartition * ZkLocalPartition; 



/* exported functions */
DLLExport 
int zkLocalPartition_Open(  ZK_OUT ZkError err, 
                            ZK_OUT ZkLocalPartition partition); 

DLLExport 
int zkLocalPartition_Close( ZK_OUT      ZkError err,
                            ZK_INOUT    ZkLocalPartition partition);

DLLExport 
int zkLocalPartition_Fetch (ZK_OUT  ZkError err, 
                            ZK_IN   ZkLocalPartition partition,
                            ZK_IN   ZkDBT key,
                            ZK_OUT  ZkDBT data,
                            ZK_OUT  int *found);

/* This function must be removed. It is not used any more and i keps only to avoid breaking the build */
DLLExport
int zkLocalPartition_ResetGet (ZK_INOUT ZkError err,
                               ZK_INOUT  ZkLocalPartition partition);
                               
DLLExport  
int zkLocalPartition_Update  (  ZK_INOUT    ZkError err,
                                ZK_IN       ZkLocalPartition partition,
                                ZK_IN       ZkDBT key,
                                ZK_IN       ZkDBT data,
                                ZK_OUT      int *exists,
                                ZK_IN       int overwrite);

DLLExport
int zkLocalPartition_Delete (ZK_OUT ZkError err,
                             ZK_IN  ZkLocalPartition partition,
                             ZK_IN  ZkDBT key,
                             ZK_OUT int *notfound);

/* This function must be removed. It is not used any more and i keps only to avoid breaking the build */
DLLExport
int zkLocalPartition_GetNext (ZK_INOUT ZkError err,
                              ZK_IN ZkLocalPartition partition,
                              ZK_OUT  ZkDBT key,
                              ZK_OUT ZkDBT data,
                              int *found);

DLLExport int zkLocalPartition_CheckpointsActivate(ZK_OUT ZkError err, ZK_IN ZkLocalPartition part, int  activ_stat);

DLLExport int zkLocalPartition_Checkpoint(ZK_OUT ZkError err, ZK_IN ZkLocalPartition part);


DLLExport 
int zkLocalPartition_CursorOpen (   ZK_OUT      ZkError err,
                                    ZK_INOUT    ZkLocalPartition parition);

DLLExport 
int zkLocalPartition_CursorClose (  ZK_OUT      ZkError err,
                                    ZK_INOUT    ZkLocalPartition partition);

DLLExport 
int zkLocalPartition_CursorAbort (  ZK_OUT      ZkError err,
                                    ZK_INOUT    ZkLocalPartition partition);
                                                
DLLExport 
int zkLocalPartition_CursorGetNext (ZK_OUT  ZkError err,
                                    ZK_IN   ZkLocalPartition partition, 
                                    ZK_OUT  ZkDBT key, 
                                    ZK_OUT  ZkDBT data, 
                                    ZK_IN   int *found);
                                                
DLLExport 
int zkLocalPartition_CursorDeleteCurrent (  ZK_OUT  ZkError err, 
                                            ZK_IN   ZkLocalPartition partition);
                                            
DLLExport 
int zkLocalPartition_CursorUpdateCurrent (  ZK_OUT  ZkError err, 
                                            ZK_IN   ZkLocalPartition partition, 
                                            ZK_IN   ZkDBT key, 
                                            ZK_IN   ZkDBT data); 

#endif /* ZKLocalPartition */
