/*
 * $Id: zkRemotePartition.h,v 1.7 2000/11/24 15:04:09 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef  ZKRemotePartition_h
#define  ZKRemotePartition_h
#ifdef ZKMOD
#undef ZKMOD
#endif

#define ZKMOD ZKMDB
#include <frFilenames.h>
#include <time.h>
#include <zkcp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <zkError.h>
#include <zkAlloc.h>
#include <zkMemPool.h>
#include <zkDebug.h>
#include <zkcp.h>
#include <zkcpValue_Binary.h>
#include <zkcpValue_Boolean.h>
#include <zkcpValue_String.h>
#include <zkcpValue_U32.h>
#include <zksocket.h>
#include <zkDB.h>
#include <frNetworkDefaults.h>

#define    ZKDB_FETCH_COMMAND                     1
#define    ZKDB_DELETE_COMMAND                    2
#define    ZKDB_UPDATE_COMMAND                    3
#define    ZKDB_CURSOR_OPEN_COMMAND               4
#define    ZKDB_CURSOR_CLOSE_COMMAND              5
#define    ZKDB_CURSOR_ABORT_COMMAND              6
#define    ZKDB_CURSOR_GETNEXT_COMMAND            7
#define    ZKDB_CURSOR_UPDATE_CURRENT_COMMAND     8
#define    ZKDB_CURSOR_DELETE_CURRENT_COMMAND     9   
#define    ZKDB_TRANSACTION_BEGIN_COMMAND         10
#define    ZKDB_TRANSACTION_COMMIT_COMMAND        11
#define    ZKDB_TRANSACTION_ABORT_COMMAND         12
#define    ZKDB_TRANSACTION_PREPARECOMMIT_COMMAND 13
#define    ZKDB_RESPONSE_COMMAND                  19
#define    ZKDB_CONNECT_COMMAND                   20
#define    ZKDB_DISCONNECT_COMMAND                21

#define    ZKDB_LAST_COMMAND_NUMBER ZKDB_DISCONNECT_COMMAND

typedef enum {

     ZKDB_UNDEFINED_PARAMMETER = -1,
     ZKDB_ERROR    =   1,
     ZKDB_RESULT  ,
     ZKDB_DBT_KEY,
     ZKDB_DBT_DATA  ,
     ZKDB_FOUND ,
     ZKDB_DBNAME,
     ZKDB_EXISTS,
     ZKDB_OVERWRITE
}   
     ZkcpParammeterId;



struct T_ZkRemotePartition {
  char * addrPath;
  char * name;                       /* remote partition name. Has to equal to the database name */ 
  char * AuthKey;                    /* Authentication Key */
  int connected;
  ZkSocket socket;
  ZkcpConnection connection; 
  ZkcpContext    connContext;      /* connection context */
  int connTime;
  void *dataBuffer,        /* Buffers for receiving keys and datas */
       *keyBuffer;         
  size_t dataBufferSize, 
         keyBufferSize;
};

typedef struct T_ZkRemotePartition  *  ZkRemotePartition;

int zkRemotePartition_Connect (ZK_OUT ZkError err,
                                ZK_INOUT  ZkRemotePartition partition);

int zkRemotePartition_Disconnect ( ZK_OUT ZkError err,
                                             ZK_INOUT ZkRemotePartition  partition);
int zkDBTCreateFromValue (ZkError err, ZkcpValue  value, ZkDBT * dbt);
int zkRemotePartition_Fetch (ZkError err, ZkRemotePartition  partition, ZkDBT key, ZkDBT data, int *found);
int zkRemotePartition_Delete  (ZkError err, ZkRemotePartition  partition, ZkDBT key ,  int *found);
int zkRemotePartition_Update  (ZkError err, ZkRemotePartition  partition, ZkDBT key , ZkDBT data, int *  exists,  int overwrite);
int zkRemotePartition_StartTransaction (ZkError err, ZkRemotePartition partition);
int zkRemotePartition_PrepareCommitTransaction (ZkError err, ZkRemotePartition partition);
int zkRemotePartition_CommitTransaction (ZkError err, ZkRemotePartition partition);
int zkRemotePartition_AbortTransaction (ZkError err, ZkRemotePartition partition);
int zkRemotePartition_Verify   (ZK_INOUT ZkError err,
                               ZK_IN ZkRemotePartition parition);


/* Cursor operations */
DLLExport int zkRemotePartition_CursorOpen(ZK_OUT ZkError           err,
                                           ZK_OUT ZkRemotePartition partition);

DLLExport int zkRemotePartition_CursorGetNext(ZK_OUT ZkError           err,
                                              ZK_IN  ZkRemotePartition partition,
                                              ZK_OUT ZkDBT             key,
                                              ZK_OUT ZkDBT             data,
                                              ZK_OUT int               *found);

DLLExport int zkRemotePartition_CursorDeleteCurrent(ZK_OUT ZkError           err, 
                                                    ZK_IN  ZkRemotePartition partition);

DLLExport int zkRemotePartition_CursorUpdateCurrent(ZK_OUT ZkError           err,
                                                    ZK_IN  ZkRemotePartition partition,
                                                    ZK_IN  ZkDBT             key,
                                                    ZK_IN  ZkDBT             data);

DLLExport int zkRemotePartition_CursorClose(ZK_OUT ZkError           err,
                                            ZK_IN  ZkRemotePartition partition);

DLLExport int zkRemotePartition_CursorAbort(ZK_OUT ZkError           err,
                                            ZK_IN  ZkRemotePartition partition);

                                                                     
#endif /* ZKRemotePartition */
