/*
 * $Id: zkAA.c,v 1.13 2000/11/24 15:04:10 marcp Exp $
 */

static const char * module_id_ = "$Id: zkAA.c,v 1.13 2000/11/24 15:04:10 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zkError.h>
#include <zkDB.h>
#include <zkAA.h>

/**
 * Persisent Associative arrayss, based on zkDB
 *
 */
#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNDB

struct T_ZkAA {
  ZkDB db;
  ZkDBT key;
  ZkDBT data;
};

  

int zkAANew(ZkError err, const char *dbname, ZkAA *aa)
{
  static const char *funcname = "zkAANew";

  (void) module_id_;
  zkMALLOC (*aa, ZkAA, sizeof(struct T_ZkAA));
  
  if (!zkDB_Open (err, &((*aa)->db), dbname))
    return 0;

  if (!zkDBTNew (err, &(*aa)->key))
    return 0;

  if (!zkDBTNew (err, &(*aa)->data))
    return 0;

  return 1;
  
}
    
		 
int zkAAFree(ZkError err, ZkAA *aa)
{

  if (!zkDBClose(err, &(*aa)->db))
    return 0;
  if (!zkDBTFree(err, &(*aa)->key))
    return 0;
  if (!zkDBTFree(err, &(*aa)->data))
    return 0;
  return 1;
}


/**
 * If the data is not found, ds will be set to 0
 */
int zkAAGet(ZkError err, ZkAA aa,
	    const void *key, size_t ks, void **data, size_t *ds)
{
  static const char *funcname = "zkAAGet";
  int found = 0;

  if (!aa || !key || !data) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
      return 0;
  }
  /* Warning OK here */
  if (!zkDBSetKeyOrData(err, aa->key,  key, ks))
    return 0;
  if (!zkDBFetch (err, aa->db, aa->key, aa->data, &found)) 
    return 0;
  if (!found) {
    *data = NULL;
    *ds = 0;
    return 1;
  }
  if (!zkDBGetKeyOrData(err, aa->data, data, ds))
    return 0;
  return 1;
}

/**
 * Delete the Associative Array Entry
 */
int zkAADelete(ZkError err, ZkAA aa,
	    const void *key, size_t ks)
{
  static const char *funcname = "zkAADelete";
  int notfound = 0;

  if (!aa || !key) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
      return 0;
  }

  if (!zkDBSetKeyOrData(err, aa->key, key, ks))
    return 0;

  if (!zkDBDelete (err, aa->db, aa->key, &notfound)) 
    return 0;

  if (notfound) {
    return 1;
  }
  return 1;
}

/**
 * If the data is not found, ds will be set to 0
 */
int zkAAFind(ZkError err, ZkAA aa, const void *key, size_t ks, int *found)
{
  static const char *funcname = "zkAAExists";

  if (!aa || !key || !found) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
      return 0;
  }
  /* Warning OK here */
  if (!zkDBSetKeyOrData(err, aa->key, key, ks))
    return 0;
  if (!zkDBFetch (err, aa->db, aa->key, aa->data, found)) 
    return 0;
  if (!found) {
    *found = 0;
    return 1;
  }
  *found = 1;
  return 1;
}

int zkAAPut(ZkError err, ZkAA aa, const void *key, size_t ks,
	    const void *data, size_t ds)
{
  static const char *funcname = "zkAAPut";

  if (!aa || !key || !data) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
      return 0;
  }
  /* Warnings OK here */
  aa->key->data = key;
  aa->key->size = ks;
  aa->data->data = data;
  aa->data->size = ds;
  
  if (!zkDBPut (err, aa->db, aa->key, aa->data)) 
    return 0;
  return 1;
}





int zkAA_CursorOpen (ZkError err, ZkAA aa)
{
  static const char *funcname = "zkAA_CursorOpen";

  if ((!aa) || (!aa->db)) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  return (zkDB_CursorOpen (err, aa->db));
}


int zkAA_CursorClose  (ZkError err, ZkAA aa)
{
  static const char *funcname = "zkAA_CursorClose";

  if ((!aa) || (!aa->db)) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  return (zkDB_CursorClose (err, aa->db));
}



int zkAA_CursorAbort  (ZkError err, ZkAA aa)
{
  static const char *funcname = "zkAA_CursorAbort";

  if ((!aa) || (!aa->db)) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  return (zkDB_CursorAbort (err, aa->db));
}




int zkAA_CursorGetNext (ZkError err, ZkAA aa,
		void **key, size_t *ks, void **data, size_t *ds, int *found)

{
  static const char *funcname = "zkAA_CursorGetNext";

  if ((!aa) || (!aa->db)) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  if (!zkDB_CursorGetNext(err, aa->db, aa->key, aa->data, found)) {
    *key = NULL;
    *data = NULL;
    *ks = 0;
    *ds = 0;
    return 0;
  }
  if (!zkDBGetKeyOrData(err, aa->data, data, ds))
    return 0;
  if (!zkDBGetKeyOrData(err, aa->key, key, ks))
    return 0;
  return 1;
}




int zkAA_CursorDeleteCurrent  (ZkError err, ZkAA aa)

{
  static const char *funcname = "zkAA_CursorDeleteCurrent";

  if ((!aa) || (!aa->db)) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  return (zkDB_CursorDeleteCurrent (err, aa->db));

}



int zkAA_CursorUpdateCurrent  (ZkError err, ZkAA aa, const void *key, size_t ks,
	    const void *data, size_t ds)
{
  static const char *funcname = "zkAA_CursorUpdateCurrent";

  if (!aa || !key || !data) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
      return 0;
  }
  /* Warnings OK here */
  aa->key->data = key;
  aa->key->size = ks;
  aa->data->data = data;
  aa->data->size = ds;

  return (zkDB_CursorUpdateCurrent (err, aa->db, aa->key, aa->data));

}




