/*
 * $Id: zkDBT.c,v 1.11 2000/11/24 15:04:10 marcp Exp $
 */

static const char * module_id_ = "$Id: zkDBT.c,v 1.11 2000/11/24 15:04:10 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <zkError.h>
#include <zkAlloc.h>
#include <zkMemPool.h>
#include <zkDB.h>
#include <zkDebug.h>
#include <db.h>
#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNDB


/**
 * Creates the opaque type for a database key/data pair
 *
 * @param err                   Error return object
 * @param dbt			A database key/data pair thing
 * @return                      Non-zero on success, 0 on failure
 * @version                     1.0
 * @author                      Luc Boulianne
 */
DLLExport
int zkDBTNew (ZkError err, ZkDBT *dbt)
{
  static const char *funcname = "zkDBTNew";
  
  (void) module_id_;
  if (!dbt) {
    zkErrorSet(err, funcname, ZKERR, ZKMANONROUTE,
	       ZKENULLPTR, "NULL Pointer passed");
    return 0;
  }
  zkMALLOC(*dbt, ZkDBT, sizeof(DBT));
  (*dbt)->data = NULL;
  (*dbt)->size = 0;
  (*dbt)->ulen = 0;
  (*dbt)->dlen = 0;
  (*dbt)->doff = 0;
  (*dbt)->flags = 0;
  
  /* DB_DBT_MALLOC; / * bug: using internal static structure */
  return 1;
}


DLLExport int zkDBTCreate (ZkError err, const char * buffer, int size, ZkDBT *dbt)
{
   static const char *funcname = "zkDBTCreate";
/*   char * buff = NULL; */

   (void)funcname;

   if (!zkDBTNew(err, dbt)) {
     return 0;
   }  

   /* allocate memory */
/*   if(!zkMemAlloc(err, (void **) &buff , size)) { */
/*       zkErrorSet(err, funcname, ZKERR, ZKMANONROUTE, */
/*                  ZKERROR, "Memory allocation problem"); */
/*       return 0; */
/*     }      */
   (*dbt)->data= buffer;
   /* memcpy (buff, buffer, size);*/
   (*dbt)->size = size;
   /*(*dbt)->ulen = size; */ /* need to specify to control memory freeing */

  return 1;
}  
     


/**
 * Frees the opaque type for a database key/data pair
 *
 * @param err                   Error return object
 * @param dbt			A database key/data pair thing
 * @return                      Non-zero on success, 0 on failure
 * @version                     1.0
 * @author                      Luc Boulianne
 */

DLLExport
int zkDBTFree (ZkError err, ZkDBT *dbt)
{
  static const char *funcname = "zkDBTFree";
  int retcode = 1;

  if (dbt) {
    if (*dbt) {
      if ((*dbt)->data) {
        /*   if ((*dbt)->ulen > 0)  */ 
             /*  free((*dbt)->data);*/
         }
        free (*dbt);
      *dbt=NULL;
    } else {
      zkErrorSet(err,funcname,ZKINFO,ZKMDB,ZKENOMEM, "Pointer contains nothing to free"); 
      retcode = 0;
    }
  } else {
    zkErrorSet(err,funcname,ZKERR,ZKMDB,ZKENOMEM, "Pointer content is NULL"); 
    retcode = 0;
  }
  return retcode;
}

/*   zkDBSetKeyOrData      set DBT structure      
*
* err                       Error return object
* dbt                       return data or key object
* data                      data or key buffer 
* size                      size of buffer in bytes                 
* return                    1 - success, 0  - failure 
* @version                  1.0
* @author                    Luc Boulianne
*/

DLLExport
int zkDBSetKeyOrData(ZkError err, ZkDBT dbt, void *data, uint32_t size)
{
  static const char *funcname = "zkDBSetKeyOrData";
  
  if (dbt) {
    dbt->data = data;
    dbt->size = size;
  } else {
    zkErrorSet(err,funcname,ZKERR,ZKMDB,ZKENULLPTR,"No pointer to data object");
    return 0;
  }
  return 1;
}
 
/*  zkDBGetKeyOrData        function. Used get buffer from the DBT structure.   
*
* err                       Error return object
* dbt                       data or key object 
* data                      return buffer where data located
* size                      return size of the buffer
* return                    1 success, 0 - failure  
* @version                  1.0
* @author                   Luc Boulianne
*/

DLLExport
int zkDBGetKeyOrData(ZK_INOUT ZkError err, ZK_IN ZkDBT dbt, ZK_OUT void **data, ZK_OUT uint32_t *size)
{
  static const char *funcname = "zkDBGetKeyOrData";
   
  if (dbt || data  || * data || size) {
    *data = dbt->data;
    *size = dbt->size;
    return 1;
  } else {
    zkErrorSet(err,funcname,ZKERR,ZKMDB,ZKENULLPTR,"Invalid parameter"); 
    return 0;
  }
}


