/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#define _POSIX_SOURCE 1
#include <signal.h>
#include <zkSignal.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkDebug.h>
#include <zkDB.h> 
#include <time.h>

#define  DEFAULT_RECORDS 1000
#define  DEFAULTFILE "dict"
#define  DEFAULTSLEEP 0


#define KEY_LENGTH 64
#define MAX_RECORD_SIZE 400
#define RECORD_SIZE 400



/* allow to execute folowing steps :

   1.   fill database with the random data  and exsesise different locking algorithms  */
 extern char *optarg;
 extern int errno, optind;

 
typedef enum OPER {

 OP_IMPORT,
 OP_EXPORT,
 OP_DELETE,
 OP_OPENCLOSE ,
 OP_SCAN
} OP_TYPE;  
  

static void usage()
{
	(void)fprintf(stderr,
	    "usage: db_trans [-d databasename] \n [-h homedir] \n [-l logdir] \n ");
        (void)fprintf(stderr,"[-o operation] ");
        (void)fprintf(stderr,
	    "where operation is : \n i-import data , \n e - export data \n");

        (void)fprintf(stderr,"\n[-f fork on N processes]  \n[-n number of records] \n [-s sleeptime] \n [-f filename]\n ");

        exit(1);
}





static void fillKey (char * keyname, int vl)
{
   sprintf (keyname, "KEY N: %d ", vl+1 ); /* audit: ok */
  
}



static int read_fromfile (FILE * filen, int recno, char ** rec, int seek)
{
 int len;
 char  * buffer;
 /* find rec */

 buffer = malloc(MAX_RECORD_SIZE+1);
 if (seek) {
  fseek (filen, 0L , SEEK_SET);    
  while (!feof(filen) || --recno > 0 ) fgets (buffer, MAX_RECORD_SIZE, filen);
 }
 if (feof(filen)) { free(buffer); return 0;}  
 fgets (buffer, MAX_RECORD_SIZE, filen);
 len = strlen(buffer);
 if (*rec) free(*rec);       
  (*rec) =  buffer;
 
  return 1;

}



static int write_tofile (FILE * fl,  char *  rec)
{
  fprintf (fl, "%s\n", rec);
  return 1;
}


static    int delete_record(ZkError err, ZkDB db , int nkey)
{
 
  ZkDBT key = NULL;   
  int found = 0;
  char * keyname;
  int return_code = 0;
         
   keyname = malloc (KEY_LENGTH);
   fillKey (keyname,nkey); 
  
   if (!(zkDBTNew(err,&key) &&
          zkDBSetKeyOrData(err,key,keyname, strlen(keyname)))) {
      goto done;
    }
      
 
     return_code = zkDBDelete(err,db,key, &found);
 
done:
    if (key) {
      zkDBTFree(NULL,&key);
    }

    free(keyname);
    return return_code;

}
     



static int  update_record(ZkError err, ZkDB db,int  nkey, char * rec)
{
  ZkDBT key = NULL;   
  ZkDBT data = NULL;
  char * keyname;
  int return_code = 0;
         
   keyname = malloc (KEY_LENGTH);
   fillKey (keyname,nkey); 
  
   if (!(zkDBTNew(err,&key) &&
          zkDBSetKeyOrData(err,key,keyname, strlen(keyname)) &&
          zkDBTNew(err,&data) && zkDBSetKeyOrData(err,data,rec,strlen(rec)))) {
      goto done;
    }
      
    /*
     * Try to stick it into the spent token database.  If this fails, we
     * cant lock it.
     */       
     return_code = zkDBReplace(err,db,key,data);
   
done:
    if (key) {
      zkDBTFree(NULL,&key);
    }
    if (data) {
      zkDBTFree(NULL,&data);
    }
    free(keyname);
    return return_code;
}
  


static int getnext_record(ZkError err, ZkDB db ,  char ** rec)
 {
  ZkDBT db_key = NULL;
  ZkDBT db_data = NULL;
  char * keyname=NULL;
  int notfound = 0;
  int bufflen;
  int ret = 0;
 
 
  keyname = (char *) malloc (KEY_LENGTH);
 
 
   if (!zkDBTNew(err,&db_key)) {
      printf("Internal error\n");
      goto clean;
    }
    if (!zkDBTNew(err,&db_data)) {
        printf("Internal error\n");
      goto clean;
    }

  if (!zkDB_CursorGetNext(err, db,   db_key,  db_data,  &notfound)) {
       printf ("db_getnext: data not found. ");
       goto clean;
    }
    if (!notfound) {
       goto clean;
    }
    if (!zkDBGetKeyOrData(err,db_data, (void **) rec, &bufflen)) {
        goto clean;
     }
     (*rec)[bufflen-1]= '\0';
    
  
ret = 1;
 
    
  clean:
 
    if (db_data) {
      zkDBTFree(err,&db_data);
    }

     if (db_key) {
      zkDBTFree(err,&db_key);
    }
    if (keyname) free(keyname);
  return (notfound);
}




static int find_record(ZkError err, ZkDB db , int nkey, char ** rec)
 {
  ZkDBT db_key = NULL;   
  ZkDBT db_data = NULL;
  char * keyname=NULL;
  int found = 0;
  int bufflen;
  int ret = 0;
  
  
  keyname = (char *) malloc (KEY_LENGTH);
  
  
   if (!zkDBTNew(err,&db_key)) {
      printf("Internal error\n");
      goto clean;
    }
    if (!zkDBTNew(err,&db_data)) {
        printf("Internal error\n");
      goto clean;     
    }

     
   fillKey (keyname,nkey); 
    
   if (!zkDBSetKeyOrData(err,db_key,keyname,strlen(keyname) )) {
      printf("Internal error\n");
      goto clean;     
     }
    
  if (!zkDBFetch(err, db,   db_key,  db_data,  &found)) {
       printf ("db_fetch: data not found. ");
       goto clean;     
    }
    if (!found) {  
       goto clean;     
    } 
    if (!zkDBGetKeyOrData(err,db_data, (void **) rec, &bufflen)) {
        goto clean;     
     }
     (*rec)[bufflen-1]= '\0';
     ret = 1;
      
 clean:
  
    if (db_data) {
      zkDBTFree(err,&db_data);
    }

     if (db_key) {
      zkDBTFree(err,&db_key);
    }
    if (keyname) free(keyname);
  return (found);
}




static int  insert_record (ZkError err, ZkDB db,int  key, char * rec)
{
   ZkDBT db_key = NULL;   
   ZkDBT db_data = NULL;
   char * keyname = NULL;
  int exists;
  
  int return_code = 0;
  
   keyname = malloc (KEY_LENGTH);
   fillKey (keyname,key); 
  
   if (!(zkDBTNew(err,&db_key) &&
          zkDBSetKeyOrData(err,db_key,keyname, strlen(keyname)) &&
          zkDBTNew(err,&db_data) &&
          zkDBSetKeyOrData(err,db_data,rec,strlen(rec)))) {
      goto done;
    }
        
    /*
     * Try to stick it into the spent token database.  If this fails, we
     * cant lock it.
     */

    if (zkDBStore(err,db,db_key,db_data,&exists)) {
      if (!exists) {
        return_code = 1;
      } else {
        zkErrorSet(err,"insert_record",ZKERR,ZKIP,ZKERROR,
                   "Token is already in the database");
      
      }
    } 
     
    /*
     * Cleanup
     */
done:
    if (db_key) {
      zkDBTFree(NULL,&db_key);
    }
    if (db_data) {
      zkDBTFree(NULL,&db_data);
    }
    free(keyname);
    return return_code;
}




static int  populate_rec (int key, char ** rec)
{
   int vl = key % 128 ;
   
   if (vl < 32) vl = 32;
   (*rec) = malloc (RECORD_SIZE+1);
   
   memset (*rec, vl, RECORD_SIZE); 
   memcpy (*rec, "RECORD N:", 9);
   (*rec)[RECORD_SIZE-1] = '\0';
   return 1; 
}


int main(int argc, char **argv)
{
  ZkDB  db = NULL;
  ZkError err;
  ZkLog log;
  FILE * wrkfile = NULL;
  int enabledtrans = 1;
  double rnd;
  int  ch, key, i;
  int nforks = 0;
  const char * database, * home = NULL;
  const char * logdir = NULL;
  OP_TYPE  op = OP_IMPORT; 
  int nrecords= DEFAULT_RECORDS;
  int found =0;
  const char * filename = (char *) DEFAULTFILE;
  long sleeptime = DEFAULTSLEEP;
  int rndvl = 0;
  char * rec =  NULL;
  char * bf;
  int pid=0;
  time_t started;  
  time_t ended;
  int ret; 
  (void)argc;

  
   

   database = "test1";
   
     
   while ((ch = getopt(argc, argv, "d:h:l:o:n:r:p:s:f:t:")) != EOF)
       	switch (ch) {
        case 'p':
               if ((i = atoi(optarg)) <= 0) {
                   usage(); return 1;
               }
               else
                   nforks =i;
               break;
        case 'd':
                database = optarg;
                break;
       	case 'h':
      		home = optarg;
       		break;
        case 'l':
                logdir = optarg;
                break; 
       	case 'o':
          switch (optarg[0]) {
          case 'i': op = OP_IMPORT;   break;
          case 'e': op = OP_EXPORT;   break;
          case 'd': op = OP_DELETE;   break;
          case 'o': op = OP_OPENCLOSE ; break;  
          case 's': op = OP_SCAN;      break;  
        } break;
        case 'n':  if ((i = atoi(optarg)) <= 0)
				usage();
                   else   nrecords = i;
                   break;        
        case 'f':  filename = optarg;            
                   break;
        case 's':
           if ((i = atoi(optarg)) <= 0)
				usage();
                   else  sleeptime  = i;
               break;
        case 't':
                 i = atoi(optarg);
		  enabledtrans = i;	
               break;
          
        case 'r':
           if ((i = atoi(optarg)) <= 0)
	     		rndvl = 1;
           else   rndvl   =  i;
               break;
  
        default:
          break;
        }   

  
  started = time(NULL);
 
  printf ("\nStart %s process. ", (op==OP_IMPORT)? "import": "export");
  printf ("\nWorking file : %s.", filename);
  printf ("\nDatabase  : %s.", database);
  printf ("\nProceed order : %s.",(rndvl)? "random":"sequential");
  printf ("\nRecords number  : %d.",nrecords);
  printf ("\nTime delay  : %ld.",sleeptime);
  printf ("\nNumber of forks :%d.",  nforks);
  printf ("\nTransactions are %s", (enabledtrans)? "enabled":"disabled");
  if (logdir) printf ("\nLog directory   : %s.",logdir);
  if( home) printf ("\nHome directory : %s.", home);
  printf ("\nStarted %s ", ctime(&started));  
    
  assert(zkErrorNew(&err));
  assert(zkErrorLog (err, &log));
  assert(zkLogSetStream (log, stdout));
  assert(zkLogEnable (log));
 
  zkDebugOn();
 
  if (op ==  OP_OPENCLOSE) {
    clock_t opTime[2], start;
    long unsigned int opCount = 0;

    opTime[0] = opTime[1] = 0;
    
    while(nrecords --) {
      start = clock();
      ret = zkDB_Open(err, &db, database);
      opTime[0] += (clock() - start);
      if (!ret) return (0);

      start = clock();
      ret = zkDBClose(err, &db);
      opTime[1] += (clock()-start);
      if (!ret) return (0);
      opCount++;

      if ((opCount % 500 == 0) ||
	  !nrecords) {
	if (opTime[0] && opTime[1]) {
	  printf ("-------------------------------------------------------------------\n");
	  printf ("Operations: %lu  -  Open: %ld sec.  -  Close: %ld seconds\n",
		  opCount, opTime[0]/CLOCKS_PER_SEC, opTime[1]/CLOCKS_PER_SEC);
	}
      }
    }      
    printf ("-------------------------------------------------------------------\n");
    
    goto Done;   
  }
  else if (!(ret = zkDB_Open(err, &db, database))) {
      zkLogWriteContext(log, err, "zkDB_Open", "Open failed ");
      goto Done;
  }

  if (op == OP_SCAN) {
     zkDB_CursorOpen (err, db);

   }
  wrkfile = fopen (filename, (op==OP_IMPORT)? "r":"a");

  if (!wrkfile) {
    printf ("Could not execute %s: error of the opening file %s \n",
           (op== OP_IMPORT)? "import" : "export", filename );
           return (0);
  }        

  if (!pid) while(nforks-- >  0 )   {  /* start fork processes */
   if ((pid = fork()) <  0) { 
     printf ("Not able to startup process %d. Error %d : %s ", i, errno, strerror(errno));   
     exit (EXIT_SUCCESS);  
   }
    
  }
  
  if (rndvl > 1)
    srandom (rndvl);      
                
  for (i=0 ; i < nrecords; i++) {
     if (i%500 == 7) { 
         zkDB_PrintStat (err, db);
         printf ("\n---------------------------------------------------------------------------------"); 
     }  
    if  (rndvl)
      {  rnd = random();
      
         key = ((rnd / RAND_MAX) *nrecords);

      }   
    else key = i;
    if (i%20 == 3)   
    printf ("\rProceed record %d from %d", i+1, nrecords);

    switch (op) {
     case OP_IMPORT:   
       found = find_record(err, db , key, &bf);  
      /*      if (!read_fromfile( wrkfile, key, &rec, rndvl)) 
      */                        populate_rec (key, &rec);
        
       if(found)   update_record(err, db, key, rec); 
       else        insert_record (err, db, key, rec);
       break;       
     case OP_EXPORT:
        if (find_record (err, db, key, &bf))
                     write_tofile ( wrkfile, bf);  /*
        else {
              populate_rec (key, &rec);
              write_tofile ( wrkfile, rec); 
              }*/
        break;  
    case OP_DELETE:
             delete_record(err, db,key);
 
          break; 
    case OP_SCAN: 
          if( getnext_record( err, db ,  &bf)) 
                      write_tofile (wrkfile, bf);

        break; 
     default: 
            printf ("insupported operation\n ");   
    }  
   

    if(rec) {
     free(rec);
     rec = NULL;
    }
  }


 zkDBClose(err, &db);

  printf ("\rProceed record %d from %d\n", i, nrecords);
Done:
  ended = time(NULL);
  printf ("\nFinished  : %s ", ctime(&ended)); 
  printf ("\nExecution time:  %ld sec\n",  difftime( ended, started)+(ended > started)? 0 : 1);
  if (rec) free(rec);   
  if (wrkfile) fclose( wrkfile);
  zkErrorFree(&err);
  exit (EXIT_SUCCESS);  
  
}
