/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include <stdio.h>
#include <assert.h>
#if 0
/* can't find this file anywhere in the tree */
#include <zk_string.h>
#endif
#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkDebug.h>


int main(int argc, char **argv)
{

  ZkDB  db = NULL;
  ZkDBT key, keylist;
  ZkError err;
  ZkLog log;
  ZkMemPool mpool;
  size_t numkeys, listsize;
  size_t i;

  (void)argc;

  assert(zkErrorNew(&err));
  assert(zkErrorLog (err, &log));
  assert(zkLogSetStream (log, stdout));
  assert(zkLogEnable (log));
  
  assert(zkMemPoolNew (err, &mpool));

  assert(zkDB_Open(err, &db, "ndb-test3"));

  //  assert(zkDBDump(err, db));

  zkDebug_va ("sorting ....");
  assert(zkDBGetAllKeysSorted (err, db, mpool, &keylist,
			  &numkeys, &listsize));

  for (key = keylist, i = 0 ; (i < numkeys) && key  ; i++, key++) {
    printf ("%-20s %d\n", (char *)key->data, i);
  }
  zkDebug_va ("done ....");
  
  
  /* Usefull stuff ends here */  
  assert(zkMemPoolFree (err, &mpool));
  free (keylist);
  assert(zkDBClose(err, &db));
  assert(zkErrorFree(&err));
  return (0);
}

  
