/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#define _POSIX_SOURCE 1
#include <signal.h>
#include <time.h>
#include <db.h>

#define DB_MODE         0600
#define SIZE_KEY        24


typedef struct T_test_scan_rec {
  time_t tmTimeStamp;
  int    nData;
} st_test_scan_rec;

void  DBReportError(const char * errprefix, char * buffer ) {
  printf ("ERROR : %s \n",  buffer);
}


static void  SetKey (DBT * key,  int ind) {
  key->data =(void *)  malloc (SIZE_KEY);
  key->size = SIZE_KEY;
  snprintf (key->data, SIZE_KEY, "KEY === %d", ind );

}


static int DBTNew ( DBT **dbt)
{

  (*dbt) = (DBT *) malloc (sizeof(DBT));
  (*dbt)->data = NULL;
  (*dbt)->size = 0;
  (*dbt)->ulen = 0;
  (*dbt)->dlen = 0;
  (*dbt)->doff = 0;
  (*dbt)->flags = 0;

  return 1;
}


static int DBTFree (DBT **dbt)
{
  if (dbt) {
    if (*dbt) {
      free (*dbt);
      *dbt=NULL;
    }
  }
  return 1;
}

static void usage (void) {

  printf ("db test: \n");
  printf("          -n number_of_records\n");
  printf("          -p number of processes\n");
  printf("         -d database name\n");
  printf("         -h home\n");
  printf("         -l logdir   \n");

}

int main(int argc, char **argv)
{

  int pid = 0;
  int nProcType = 0;
  DB_ENV * dbenv = NULL;
  DB_INFO * inf;
  DB *dbp = NULL;                      /* Pointer to the database */
  DBT * key;
  DBT * data;
  DBC * cursScan = NULL;
  char * config[2];
  char * logdir = "log";
  char * home = "TESTDIR";
  char * dbname="db1";
  int nforks =  3;
  int i, errno = 0;
  int nCursOption = 0;
  int nrecords = 1000000;
  DB_TXN  * transaction;
  char ch, buf[1024];
  time_t now = 0;
  time_t expire = 0;
  st_test_scan_rec   stData;
  st_test_scan_rec * pstData = NULL;
  int nRecKey;
  int nRnd = 0;
  int nDiffTime = 0;
  int bScanCleanedRecord = 0;
  
  system("rm -rf TESTDIR; mkdir -p TESTDIR/log");  /* audit: ok */
  
  /* read command line  parammeters  */

   while ((ch = getopt(argc, argv, "d:h:l:o:n:r:p:s:f:t:")) != EOF)
               switch (ch) {
       case 'p':
               if ((i = atoi(optarg)) <= 0) {
                   usage(); return 1;
               }
               else
                   nforks =i;
               break;
        case 'd':
                dbname = optarg;
               break;
               case 'h':
               home = optarg;
                       break;
        case 'l':
                logdir = optarg;
                break;
        case 'n':
          if ((i = atoi(optarg)) <= 0) {
            usage(); return 1;
          }
          else
           nrecords  =i;
          break;


        default:
	  usage(); return 1;
          break;
        }

   printf ("\nDatabase  : %s.\n", dbname);
   printf ("\nRecords number  : %d.\n",nrecords);
   printf ("\nNumber of forks :%d.\n",  nforks);
   printf ("\nLog directory   : %s.\n",logdir);
   printf ("Home directory : %s.\n", home);

  /* We lauch the child sub-process there is nforks process 
     writing info in the database, and 1 process scanning the
     database. */

   if (!pid) {
   /* the ReadWrite subprocesses */
     while (nforks-- > 1) {
       pid = fork();
       if (pid < 0) {
	 printf ("Not able to startup process %d. Error %d : %s ", i, errno, strerror(errno));
	 exit(1);
       } else if (pid == 0) {
	 break; /* This is a child, so open the database */
       } else {
        /* this is a parent, keep looping */
       }
     }

     if (pid) {
      /* The Scan subprocess */
       nProcType = 1;
       pid = fork();
       if (pid < 0) {
	 printf ("Not able to startup process %d. Error %d : %s ", i, errno, strerror(errno));
	 exit(1);
       }
     }
   }
  
   if (pid == 0)
     pid = getpid();
   else {
     int status;
     while ((pid = wait(&status)) != -1)
       printf("\n%u exited: %d\n", pid, status);
     printf("parent exiting: %s\n", strerror(errno));
     exit (0);
   }

  /* Open the database */

  dbenv = (DB_ENV * ) malloc ( sizeof(DB_ENV));
  memset (dbenv, 0, sizeof(DB_ENV));
  dbenv->tx_max = 250;
  dbenv->mp_size = 1048576; /* 1 MByte */

  dbenv->db_errcall= DBReportError;
  snprintf(buf, sizeof buf, "DB_LOG_DIR %s", logdir);
  config[0] = buf;
  config[1] = NULL;
  dbenv->db_errpfx =  "TEST";

  /* We initialize the database environment with NO transactions and NO LOG ! */
  if ((errno = db_appinit(home, NULL , dbenv,
	DB_CREATE |  DB_INIT_LOCK | DB_INIT_MPOOL )) != 0)
    return 0;

  inf = (DB_INFO * ) malloc ( sizeof(DB_INFO));
  memset (inf, 0, sizeof(DB_INFO));
  inf->db_lorder=4321;

  if ((errno = db_open(dbname, DB_HASH,DB_CREATE,DB_MODE,
		       dbenv, inf, &dbp)) != 0) {
    return 0;
  }

  /* Different processing depending on the process type */
  if (nProcType == 0) {
    fprintf(stderr, "Starting RW subprocess PID:%u - NRecords = %u\n", pid,nrecords);
    /* Read-Write sub-process */
    srandom(pid);
    while (1) {
      // Give some oxygen to the kernel !
      usleep(1000);
      /* We compute a key number */
      nRnd = random();
      nRecKey = nRnd - ((nRnd/100000)*100000);
      //fprintf(stderr, "RW PID:%u - index = %u - Reading = %u\n", pid,i,nRecKey);
      /* We allocate and build a record and its key */
      DBTNew(&key);
      DBTNew(&data);
      SetKey (key, nRecKey);

      
      /* Do we already have this record in the database ? */
      errno = dbp->get(dbp, NULL , key, data, 0);
      //fprintf(stderr, "RW PID:%u - Reading Key:%u\n", pid, nRecKey);
      if (errno == DB_NOTFOUND) {
  	/* This record does not exists...Try to add it */
        time(&stData.tmTimeStamp);
  	stData.nData = 0;
  	data->data = &stData;
  	data->size  = sizeof(st_test_scan_rec);
  	errno = dbp->put(dbp, NULL , key, data, DB_NOOVERWRITE);
  	//fprintf(stderr, "RW PID:%u - Writing Key:%u\n", pid, nRecKey);
      } else {
	fprintf(stderr, "RW PID:%u - Found Key:%u\n", pid, nRecKey);
      }

       
      free(key->data);
      //free(data->data);
      DBTFree(&key);
      DBTFree(&data);
    }
    
  } else {/* Scan sub-process*/
    fprintf(stderr, "Starting SCAN subprocess PID:%u\n", pid);
    while(1) {
      bScanCleanedRecord = 0;
      /* we create a cursor that will continuously scan the database */
      if ((errno = dbp->cursor(dbp, NULL, &cursScan, 0)) != 0) {
	fprintf(stderr, "Test scan: Scan cursor: %s\n", strerror(errno));
	exit (1);
      }
    
      /* We allocate and build a record and its key */
      DBTNew(&key);
      DBTNew(&data);
      time(&now);

      /* Scan the database continuously and retrieve the current record time */
 
      // Give some oxygen to the kernel !
      //usleep(10000);
      sleep(3);
      fprintf(stderr, "--------- SCAN : starting scan...\n");
      
      while ((errno = cursScan->c_get(cursScan, key, data, DB_NEXT)) == 0) {
	pstData = (st_test_scan_rec *) data->data;
	nDiffTime = difftime(now,pstData->tmTimeStamp);
	if (nDiffTime>=10){
	  /* This record is older than 10 Sec, we have to delete it ! */
	  if ((errno = cursScan->c_del(cursScan,0)) != 0)
	    fprintf(stderr, "Test scan : get: %s\n", strerror(errno));
	  bScanCleanedRecord++;
	}     
      }
      fprintf(stderr, "--------- SCAN : Cleaned %u Record(s)\n", bScanCleanedRecord);
      if (errno != DB_NOTFOUND)
	fprintf(stderr, "Test scan : get: %s\n", strerror(errno));
      (void)cursScan->c_close(cursScan);
      DBTFree(&key);
      DBTFree(&data);
    }    
  }
 
  /* Close database, clean environment and exit */

  if (dbp) {
    if (dbp->close) {
      if((errno = (dbp->close( dbp, 0))) != 0) {
       return (0);
      }
    }
  }
  
  if (db_appexit(dbenv)) {
    return (0);
  }
  free(dbenv);
  free(inf);
  return 0;

}
