static const char * revision_id_ = "$Revision: 1.14 $";

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <libgen.h>
#include <sys/stat.h>

#endif
#include "ndbutil.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMDBUTIL

#define DEFAULT_PROGNAME "ndbutil"
#define DEFAULT_PIDFILE  DEFAULT_PROGNAME ".pid" 
#define VERSION_SCANF_STR "Zero-Knowledge " DEFAULT_PROGNAME \
              " V%s ("__DATE__", "__TIME__")"
#ifdef MIN
# undef MIN
#endif
#define MIN(a,b)  ((a) < (b))? (a):(b)    /* old MIN macros should be very cafeful with it */
#define MAX_NUMBER_ATTEMPTS 20
#define  DEFAULT_DB_TYPE DB_HASH
void usage(void);

DB_ENV * dbInitRecover(ZkError err,  char * home, char * log, u_int32_t  flags);
ZkDatabaseCtx  DBCTX_ADD (ZkError err, char * name,   ZkDBUtilCtx  ctx );
void  zkDBReportError(ZK_IN const char * errprefix, ZK_IN const  char * buffer );
DB_ENV * dbInit(ZkError err, char * home, char *  logdir);

ZkDBUtilCtx  GlobalCtx= NULL;
const char * Progname;
int   interrupted;
int   frozen;

static void signalHandler ( int sig );
void SignalHandlingInit(ZkDBUtilCtx  ctx);

static int CheckAndCreateDir(ZkError err, char * directory, int * direxists, int * dircreated, int prompt);
     
static ZkError DBErr; /* static Error object referense. Used for the Database Error Call Back */ 

/* stolen from Berkley DB defines */
#define DEFAULT_CONFIG_FILE  "ndbutil.config"
#define        DB_FTYPE_BTREE          -1      
#define        DB_FTYPE_HASH           -2

int __bam_pgin __P((db_pgno_t, void *, DBT *));
int __bam_pgout __P((db_pgno_t, void *, DBT *));
int __ham_pgin __P((db_pgno_t, void *, DBT *));
int __ham_pgout __P((db_pgno_t, void *, DBT *));


static void nBanner (ZkError err, ZkDBUtilCtx ctx)
{ 
   char s[20] = "\0";
   
  if (sscanf(revision_id_, "$" "Revision: %19s$", s)) /* audit: ok */
  s[strlen(s)-1] = '\0';
  
  zkLogWrite_va (ctx->log,  VERSION_SCANF_STR , s );
  zkLogWrite_va(ctx->log,  "Build release: [%s]", zkLibGetBuildRelease(err));

}
static  ZkDatabaseCtx GetNextContextDB (ZkDBUtilCtx  ctx )
{
  ctx->curdb= ctx->curdb->nextDB; 
  return ctx->curdb;
}

static  ZkDatabaseCtx GetFirstContextDB (ZkDBUtilCtx  ctx )
{
  ctx->curdb= ctx->dbs; 
  return ctx->curdb;
}

    

static ZkDatabaseCtx  AddDatabaseToTheContext (ZkError err, char * name,   ZkDBUtilCtx  ctx )
{
  ZkDatabaseCtx db = NULL;
   
   if( ! zkMemCalloc(err, (void**) &db, sizeof(struct T_ZkDatabaseCtx),1)) {
    zkErrorSet(err, "AddDatabaseToTheContext", ZKERR, ZKMOD, ZKENULLPTR, "Failed to allocate memory \n");
     return NULL; 
  }
   
   if(ctx->dbs == NULL) {
     ctx->dbs = db;
   }  
   if(ctx->curdb) {
     ctx->curdb->nextDB = db;
   }  
   ctx->curdb = db;
   db->name = name;
   db-> dbHomeDir = NULL;
   db-> dbLogDir = NULL;
   db-> fatalrecovery = 0;
   db-> nextDB = NULL;
   db-> dbenv = NULL;
   db->dbFileLock = NULL;
    
  return db;
}

/*  zkDBReportError         call-back function. It is called by the Berkeley DB core in case of the error.   
* errprefix                 Context in which error happend. Ussially function name. 
* buffer                    memory buffer containing error description.
* return                    void sindise
* @version                  1.0
* @author                   Igor Stolbikov
*/
 void  zkDBReportError(ZK_IN const  char * errprefix, ZK_IN const  char * buffer )
{  
  /* need to filter recovering messages and error messages */
  if (errno != 0)  {
    zkErrorSet(DBErr,errprefix, ZKERR,ZKMOD,ZKERROR,buffer);
  }
  else if ( GlobalCtx->debug ) {
     zkLogWrite( GlobalCtx->log  ,buffer);    
 } 
         
}     

static int DBUtilCtxInit (ZkError err,  ZkDBUtilCtx * ctx)
{

  const char *func = "DBUtilCtxInit";

  if( ! zkMemAlloc(err, (void**) ctx, sizeof(struct T_DBUtilCtx))) {
      zkErrorSet(err, func ,  ZKERR,ZKMOD, errno,  "can't allocate context");
      return 0;
  } else {
   (*ctx)->config     = NULL;
   (*ctx)->log        = NULL;
   (*ctx)->fork        = 0;
   (*ctx)->configfile = NULL;
   (*ctx)->debug      = 0;
   (*ctx)->dbs        = NULL;
   (*ctx)->configfile = NULL;
   (*ctx)->curdb = NULL;
   return 1;

  }
}
/**
 * Load the context variable from the configuration file (if any) and the
 * command line arguments (if any).
 */
static int DBUtilCtxLoad(ZkError err,  ZkDBUtilCtx  ctx, const char * session,  int argc, 
                          const char **argv)
{
  const char *func = "DBUtilLoad";
  char *cf_opt = NULL;
  char * dblist= NULL, * dbs = NULL;
  char * dbname;
  char * op =  NULL;
  int ret =  0;
  int opt;
  char * dfltHomeDir = NULL, * dfltLogDir= NULL;
  char * local_home = NULL, * local_log = NULL;  
  ZkDatabaseCtx db = NULL; 
 
  
  if (!ctx || !ctx) {
     zkErrorSet(err, func, ZKERR, ZKMOD, ZKENULLPTR, "NULL param(s)");
     return 0;
  }

 
  /* First load up a configuration file */
  if (!zkCStrSprintf(&cf_opt, "--%s", P_CONFIG)) {
   zkErrorSet_va(err, func, ZKERR, ZKMOD, ZKENULLPTR, "Failed to create context option: %s\n",
	    zkErrorDetail(err));
    return 0;
  }

  for (opt = 1; opt < argc; opt++) {
    if (strcasecmp(argv[opt], cf_opt) == 0) {
      if (++opt < argc) {
        ctx->configfile = argv[opt];
        break;
      } else {
        fprintf(stderr, "%s: Configuration file is missing.\n", Progname);
        goto noconfig;
      }
    }
  }

  if (!zkConfigNew(err, &ctx->config)) {
    fprintf(stderr, "%s: Failed to create new context: %s\n", Progname, 
	    zkErrorDetail(err));
    goto noconfig;
  }

  if (ctx->configfile) {
    if (!zkConfigLoad(err, ctx->config, ctx->configfile)) {
    fprintf(stderr, "%s: Failed to load config file: %s\n", Progname,
	    zkErrorDetail(err));
      goto noconfig;
    }
  }

  if (!zkConfigSetCmdParms(err, ctx->config, argc, argv)) {
    fprintf(stderr, "%s: Failed to set command line params: %s\n", Progname,
	    zkErrorDetail(err));
    goto noconfig;
  }
  
  /* Get my parameters that I need */
 
  zkConfigGetParameter(err, ctx->config, TYPE_BOOLEAN, session , P_DEBUG,         &ctx->debug);
  zkConfigGetParameter(err, ctx->config, TYPE_STRING,  session , P_PARTITIONS,     &dbs);
  zkConfigGetParameter(err, ctx->config, TYPE_STRING,  session,  P_LOGFILE,       &ctx->logfile);
  zkConfigGetParameter(err, ctx->config, TYPE_STRING,  session,  P_OPERATION,      &op);
  /* read default home and transactional log directory */
 zkConfigGetParameter(err,  ctx-> config,  TYPE_STRING ,  session ,  P_DEFAULT_HOME ,    &dfltHomeDir); 
 zkConfigGetParameter(err,  ctx-> config,  TYPE_STRING ,  session ,  P_DEFAULT_LOGDIR ,  &dfltLogDir); 
  ctx->prompt = ZK_TRUE; 
  /* prompt for the operation confirmation */
  zkConfigGetParameter(err, ctx->config, TYPE_BOOLEAN, session , P_PROMPT,
                       &ctx->prompt);
 
  /* check for the name of the operation = is it recover , deadlock, checkpoint  */
  
  ctx->operation = RECOVER_OP;
  if (op == NULL) {
    ;   /* use default recovery operation */ 
  }  
  else if(!strncasecmp(op,P_RECOVER, strlen(P_RECOVER) )) {
    ctx->operation = RECOVER_OP;       
  }
  else if(!strncasecmp(op,P_VERIFY_DIRS, strlen( P_VERIFY_DIRS) )) {
    ctx->operation = VERIFY_DIRS; 
  }

  dblist = dbs;  /* need to have temporary list. */ 
  dbname  =   strsep (&dblist, PARTITION_SEPARATOR );
  while (dbname && dbname[0] != '\0') {
    db =  AddDatabaseToTheContext  (err,  dbname,    ctx );
    if( !db ) { 
      return 0; 
    }
  

    zkConfigGetParameter(err, ctx->config, TYPE_STRING,   dbname ,  P_LOGDIR,                        &local_log);
    zkConfigGetParameter(err, ctx->config, TYPE_STRING,   dbname ,  P_HOME,                          &local_home);

    if (local_home && *local_home != '\0') {
      db->dbHomeDir = strdup (local_home);
    } else {
      /* Make the db become DBHOME/dbname/dbname */
      if (!zkCStrSprintf(&db->dbHomeDir, "%s/%s", dfltHomeDir , dbname)) {
        fprintf(stderr, "%s: Failed to define homedir: %s\n", Progname, 
	    zkErrorDetail(err));
        return (0);
      }
    }

    if (local_log && *local_log != '\0') {
       db->dbLogDir = strdup (local_log);
    } else if (!zkCStrSprintf(&db->dbLogDir, "%s/%s",  dfltLogDir , dbname)) { {  
      /* Make the log dir became DBHOME/dbname/Log */
        fprintf(stderr, "%s: Failed to define logdir: %s\n", Progname, 
	    zkErrorDetail(err));
        return (0);
     
      }
    }

    
    zkConfigGetParameter(err, ctx->config, TYPE_BOOLEAN,  dbname ,  P_FATALRECOVERY  ,           &db->fatalrecovery);
 
    dbname  = strsep (&dblist, PARTITION_SEPARATOR );   
  }
  
  ret = 1;
  
 noconfig:
  free(cf_opt);
  return ret;
}

/**
 * Verify the parameters of the context variable.
 */
static int DBUtilCtxVerify(ZkError err, ZkDBUtilCtx  c)
{
  const char *func = "DBUtilVerify";
  int ret = 1;
  
  
  zkErrorClear(err);
  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMOD, ZKENULLPTR, "ctx is NULL");
    return 0;
  }

  
  if (c->debug) {
    zkErrorLog(err,&c->log); 
    zkLogSetProgname(c->log,Progname); 
    zkLogSetStream(c->log, stderr);
    zkLogEnable(c->log);
    zkDebugOn();
  } else {
    zkDebugOff();
    if (c->logfile) {
      if (!zkErrorLog(err,&c->log)) {
       zkErrorSet(err,func,ZKERR,ZKMOD,ZKERROR, "Unable to set error logfile\n");
	       
	return 0;
      }
      if (!zkLogSetFile(c->log, c->logfile)) {
        zkErrorSet_va(err,func,ZKERR,ZKMOD,ZKERROR, "Unable to open logfile \"%s\".\n",  c->logfile);
        ret = 0;
      }
      zkLogEnable(c->log);
    } else {
      zkLogDisable(c->log);
    }
  }

  if (!c->dbs)
    {
     zkErrorSet(err,func,ZKERR,ZKMOD,ZKERROR, "No database defined");
     ret = 0;
    }
  

  return ret;
}


static int DBUtilCtxFree(ZkError err, ZkDBUtilCtx  ctx)
{
  const char *func = "DBUtilCtxFree";
  ZkDatabaseCtx db = NULL, curdb = NULL; 
  if (!ctx) {
    zkErrorSet(err, func, ZKERR, ZKMOD, ZKENULLPTR, "context is NULL");
    return 0;
  }
  if (ctx->config) {
    zkConfigFree(err, &ctx->config);
  }

  ctx->config = NULL;
  ctx->log = NULL;
  ctx->configfile = NULL;
  ctx->debug = 0;
  db =  GetFirstContextDB(ctx);  
  while (db) {
    curdb = db;
    db = GetNextContextDB(ctx);
    free(curdb);
  } 
  ctx->dbs = NULL;   
  
  return 1;
}
    

int
main(int argc, const char **  argv)
 {
   ZkError err = NULL;
   ZkDBUtilCtx  ctx;        
   ZkDatabaseCtx db = NULL; 
   char  * log = NULL, * home = NULL;
   char msg[100];
   int lockGranted = ZK_FALSE;
   const char * funcname = "main";  
   int found, ret_code = EXIT_SUCCESS, created;
   /*  (void) module_id_;*/

   zkErrorNew(&err);
   DBErr = err; /* initialized global error code variable for call-back functions */
   Progname =  DEFAULT_PROGNAME;
   interrupted = 0;
   frozen = 0;
   /* clean up message buffer */
   
   memset(msg, 0, sizeof(msg)); 
   
   if (!  DBUtilCtxInit (err, &ctx)) {
     zkErrorMessage(err, msg, sizeof(msg)); 
     fprintf(stderr, "%s: Failed to initialize context variable. %s \n", Progname, msg);
     ret_code  = EXIT_FAILURE;
     goto cleanup;
   }
   
   if (! DBUtilCtxLoad (err, ctx,  NDBUTIL_SESSION , argc, argv)) {
     zkErrorMessage(err, msg, sizeof(msg)); 
     fprintf(stderr, "%s: Failed to load some context variables. %s \n", Progname, msg);
     ret_code = EXIT_FAILURE;
     goto cleanup;
   }
   
   if (! DBUtilCtxVerify(err, ctx)) {
     zkErrorMessage(err, msg, sizeof(msg)); 
     fprintf(stderr, "%s: Param(s) in context variable are bad. %s\n", 
             Progname, msg);
     ret_code = EXIT_FAILURE;
     goto cleanup;
   }
  
 
   nBanner(err, ctx);
   SignalHandlingInit(ctx);
   GlobalCtx = ctx;     /* need to keep global reference to the context for the database call-back functions */  

   if (ctx->operation == RECOVER_OP ) {
     db =  GetFirstContextDB(ctx);  
     while (db) {
       if(ctx->debug) { 
         zkLogWrite_va(ctx->log,"Database check and recover. Db: %s ", db->name);    
       }
       /* have to provide a copy of the  log variable because it will be free by db_appexit */     
       log  = strdup(db->dbLogDir);
       
       /*
        * We try to aquire an exclusive lock on the database file. If we cant have it, then we abort the
        * database initialization and recovery
        */
       if (!zkLockNew(err,&db->dbFileLock,db->dbHomeDir,NULL)) {
         zkLogWriteContext (ctx->log, err, funcname, "Cant open partition: Can not create file lock.");
	 ret_code = EXIT_FAILURE;
	 goto cleanup;
       }

       if (!zkLockRequestExclNoWait(err,db->dbFileLock,&lockGranted) || !lockGranted) {
         zkLogWriteContext (ctx->log, err, funcname, "Cant open partition: it is already in use.");
	 ret_code = EXIT_FAILURE;
	 goto cleanup;
       }
       /*
	* Initialize the database environment for recovery
	*/

       db->dbenv = dbInitRecover(err, db->dbHomeDir, log , (db->fatalrecovery)?   DB_RECOVER_FATAL : 0 );
       if(db->dbenv)  {
         db_appexit(db-> dbenv);
         db->dbenv = NULL;
       }
       else {
	 zkLockDelete(err,&db->dbFileLock);
         ret_code = EXIT_FAILURE;
         goto cleanup;
       }

       zkLockDelete(err,&db->dbFileLock);

       db = GetNextContextDB(ctx);
    
     } 
   }  else if (ctx->operation ==  VERIFY_DIRS) {
     db =  GetFirstContextDB(ctx);  
     while (db) {
       
       fprintf(stderr, "Database directory checks. Db: %s \n", db->name);    

       home = strdup( db->dbHomeDir);
       ret_code = CheckAndCreateDir(err, home, &found, &created, ctx->prompt);

       free(home);
       if (!ret_code) {
         goto cleanup;
       }
       
       if (found) {
         fprintf(stderr,"Home directory %s exists\n", db->dbHomeDir);    
       }
       else if (created)  {
         fprintf(stderr,"Created home directory %s\n", db->dbHomeDir);    
       }

       if  (db->dbLogDir[0] == '/') {
         log  = strdup(db->dbLogDir);
       }  
       else if (!zkCStrSprintf(&log, "%s/%s" , db->dbHomeDir,  db->dbLogDir)) {
         zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKERROR,strerror(errno));
         return (0);
       }
       ret_code = CheckAndCreateDir(err, log, &found, &created, ctx->prompt); 

       if (!ret_code) {
         free(log);
         goto cleanup;
       }  
       if (found) {
         fprintf(stderr, "Log directory %s exists\n", log);    
       }
       else if (created) {
         fprintf(stderr,"Created log directory %s\n", log);    
       }
       
       free(log);
       db = GetNextContextDB(ctx);
     }
    
 }  
      
  
 cleanup:
   if (ctx->log) { 
     zkLogWrite_va (ctx->log, "Terminating %s.", Progname);
   }
   if (!ret_code && msg[0]) {
     zkErrorMessage(err, msg, sizeof(msg)); 
     fprintf(stderr,msg);
   }          
   
   if ( ctx->fptr)  {
     fclose (ctx->fptr);
     unlink (ctx->pidfile);
   }

 
   (void) DBUtilCtxFree (err, ctx);
       
   if (err) {
     zkErrorFree(&err);
   }

   return ret_code;
}



DB_ENV * dbInitRecover(ZkError err,  char * home, char * log, u_int32_t  flags)
{
DB_ENV *dbenv;
u_int32_t local_flags;
const char * funcname = "dbInitRecover";  


 if( ! zkMemCalloc(err, (void**) &dbenv, sizeof(DB_ENV),1)) {
  zkErrorSet(err, funcname, ZKERR, ZKMOD, ZKENULLPTR, "Failed to allocate memory  for database structures. \n");
  return NULL; 
  }
  dbenv->db_errcall = zkDBReportError;   
  dbenv->db_errpfx  = Progname;
  dbenv->db_log_dir = log;
	/* Initialize environment for pathnames only. */
  local_flags = DB_CREATE | DB_INIT_LOG |
     DB_INIT_MPOOL | DB_INIT_LOCK | DB_INIT_TXN | DB_USE_ENVIRON;
  
  if (flags & DB_RECOVER_FATAL)
    local_flags |= DB_RECOVER_FATAL;
  else
    local_flags |= DB_RECOVER;
      
  if ((errno = db_appinit(home, NULL, dbenv, local_flags)) != 0)	{
   if (zkErrorCode (err) == 0) {
     /* Call-back function did not set error!  Need to report error */
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,errno,"Database initialization failed: %s", strerror(errno));
   } 
   free(dbenv);
    dbenv = NULL;
  }

  return (dbenv);
}
/*
 * SignalHandlingInit --
 *     Managing interuption.
 */
void
SignalHandlingInit(ZkDBUtilCtx  ctx)
{
  if (ctx->debug) return;
    zkSignalIntr(SIGINT,signalHandler);
    zkSignalIntr(SIGTERM,signalHandler);
#ifdef SIGHUP
    zkSignalIntr(SIGHUP,signalHandler);
#endif
    zkSignalIntr(SIGURG,signalHandler);        

     
}

void
usage()
{
	(void)fprintf(stderr, "ndbutil. Database proceed utility.\n");
	exit(1);
}


/*
 * db_init --
 *	Initialize the environment.
 */
DB_ENV *
dbInit(ZkError err, char * home, char * logdir)
{
  DB_ENV *dbenv;
  const char * func = "dbInit";  
  
  if ((dbenv = (DB_ENV *)calloc(sizeof(DB_ENV), 1)) == NULL) {
      zkErrorSet(err, func, ZKERR, ZKMOD, ZKERROR,"memory allocation problem.");
	  return NULL;
  }

  dbenv->db_log_dir = strdup(logdir);
  dbenv->db_errcall= zkDBReportError;   
  dbenv->db_errpfx = Progname;
  
  if ((errno = db_appinit(home, NULL, dbenv,
                          DB_INIT_LOG | DB_INIT_TXN | DB_INIT_MPOOL | DB_USE_ENVIRON)) != 0) {
   if (zkErrorCode (err) == 0) {
     /* Call-back function did not set error!  Need to report error */
    zkErrorSet_va(err,func,ZKERR,ZKMOD,errno,"Database initialization failed: %s", strerror(errno)); 
    }
   return NULL;
 }
  
  if (memp_register(dbenv->mp_info,
                    DB_FTYPE_BTREE, __bam_pgin, __bam_pgout) ||
      memp_register(dbenv->mp_info,
                    DB_FTYPE_HASH, __ham_pgin, __ham_pgout)) {
      (void)db_appexit(dbenv);
      zkErrorSet(err, func, ZKERR, ZKMOD, ZKERROR,"db_appinit:failed to register access method functions");
	  return NULL;
  }
  
  return (dbenv);
}


static int CheckAndCreateDir(ZkError err, char * directory, int * direxists,int *dircreated,   int prompt)
{
  DIR * dr;
  int exst = 0;
  char * drname = directory;
  char * sdir; 
  const char * func = "CheckAndCreateDir";  
  int ret_code = ZK_TRUE;
  int mode = S_IRWXG + S_IRWXU;
  int responded = ZK_FALSE;
  int crdir = ZK_FALSE;
  unsigned char ch;
  
  *direxists = ZK_FALSE;
  *dircreated = ZK_FALSE;
  dr = opendir(drname);
  if (dr != NULL) {
    *direxists = ZK_TRUE;
    closedir (dr);
    ret_code = ZK_TRUE;
    goto done; 
  }
 
  if (errno != ENOENT) {
    zkErrorSet_va(err, func, ZKERR, ZKMOD, ZKERROR,"Can not open directory %s:%s\n", drname, strerror(errno));
     ret_code = ZK_FALSE;
    goto done;
  }

  /* Verify if user want to have this directory */
   if (prompt) {
    fprintf (stderr, "\nDirectory %s did not exists. Do you want to create it ? (Y/N)\n", drname);
    responded = ZK_FALSE;
    while (!responded) {
      switch(ch= getchar()) {
      case 'Y': case 'y':
        responded = ZK_TRUE;
        break;  
      case 'N': case 'n':
        goto done;   
      case 13:case '\n':  /* Just eat it */
        break;
      default:
        if (ch < 32) {
          responded = ZK_TRUE;
        }
        break;
      } 
    }  
  }
  /* check sub directories */
  sdir = dirname(strdup(drname));
  ret_code = CheckAndCreateDir(err, sdir , &exst, &crdir, ZK_FALSE);
  free(sdir);
  
  if (!ret_code) {
    goto done;
  }  else   if (mkdir (drname, mode) ==  0 ) {
    ret_code = ZK_TRUE;
    *dircreated = ZK_TRUE;
    goto done;
  }
  else {
    ret_code = ZK_FALSE;
    zkErrorSet_va(err, func, ZKERR, ZKMOD, ZKERROR,"Can not open directory %s:%s\n", drname, strerror(errno));
    goto done;
  }
              
done:
  
  return ret_code;
     
}  


/*
* Generic signal handler.  Sets a variable (sigatomic_t) and returns.  This
* signal handler does no other work (and should not).
*/ 
 static void
signalHandler (
  int sig )
{
  switch (sig) {
  case SIGURG:
    frozen     = 1;
    break;
  case SIGHUP:
  case SIGTERM:
  case SIGINT:
  default:                      /* All other signals should terminate */
    interrupted = SIGINT;
    break;
  }
  return;
}
