/*
 * Freedom Network Information Database
 *
 * $Id: frNIDB.h,v 1.13 2000/11/24 15:03:08 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */
#ifndef FRNIDB_H_
#define FRNIDB_H_

#include <zkError.h>
#include <zkLog.h>
#include <zkConfig.h>
#include <frCrypt.h>
#include <frNDB.h>
#include <frNCDB.h>
#include <fr_entid.h>
#include <zkDateTime.h>
#include <zkDB.h>
#include <zkAA.h>

/*#include <sys/types.h>*/
/*#include <sys/socket.h>*/
/*#include <netinet/in.h>*/

#define FLAG_DEBUG        0x1 /* Debug mode on */
#define FLAG_REFUSEACCESS 0x2 /* Refuse client connections */
#define FLAG_DONTFORK     0x4 /* Dont immediatly fork */
#define FLAG_OVERWRITE    0x10 /* Allow files to be overwritten */
#define FLAG_DELETE       0x40 /* Delete files as they are sent */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  struct T_FrNIDBContext {
    int ready;            /* Are we ready for processing */
    
    /* Error/Logging */
    ZkError err;         
    ZkLog log;           
    ZkConfig config;     
    FILE *logFP;
    ZkMemPool mpool;

    /* Configuration Parameters */
    char * progname;      /* Application program name (argv[0]) */
    char * name;          /* A maybe unique name for the server */
    char * spooldir;      /* Spool directory */
    char * database;      /* database name */
    char * keyfile;       /* target remote filename (for client) */
    char * logfile;       /* transaction log file */
    char * outfile;       /* An output file */
    char * sigFile;       /* Filename of the sig key file */

    /* Reference  Data */
    FrCryptSig myPrivSigKey; /* My private crypto key */
    FrNCDB ncdb;

    /* Database Pointers */
    ZkDB db;               
    ZkAA aipInfoDB;        
    ZkAA linkInfoDB;       
    ZkAA genInfoDB;        
    ZkAA wincInfoDB;       
    ZkAA serverInfoDB;
    ZkAA aipUpdateDB;
    ZkAA linkUpdateDB;
    ZkAA entityIdDB;
    ZkAA idEntityDB;

    /* Extra data */
    int flags;
  };

  typedef struct T_FrNIDBContext *FrNIDBContext;

  struct T_FrNIDBAipUpdateInfo {
    short      reserved;
    int        aipid;
    char *     entity;
    int        serialnumber;
    ZkDateTime dategen;
    char *     hostname;
    char *     ipaddr;
    ZkDateTime datercvd;
    /* Set internally */
    long       reportSerialNumber;
    long       accumPktsSent;
    long       accumPktsRcvd;
    long       accumUptime;
    ZkDateTime lastReportDate;
    long       lastPktsSent;
    long       lastPktsRcvd;
    long       lastUptime;
  };
  typedef struct T_FrNIDBAipUpdateInfo *FrNIDBAipUpdateInfo;

  struct T_FrNIDBLinkUpdateInfo {
    int        linkid;
    char *     remoteEntity;
    char *     reportingAipEntity;  
    int        aipserialnumber;
    ZkDateTime dategen;
    int        cost;  /* Bug: do this */
    int        isUp;
    int        pktsSent;
    int        pktsRcvd;
    ZkDateTime datercvd;
  };
  typedef struct T_FrNIDBLinkUpdateInfo *FrNIDBLinkUpdateInfo;

  struct T_FrNIDBAipDataPkt {
    FrNIDBAipUpdateInfo aipinfo;
    FrNIDBLinkUpdateInfo linkinfo;
    int numlinks;
    ZkConfig config;
    ZkMemPool mpool;
  };

  typedef struct T_FrNIDBAipDataPkt *FrNIDBAipDataPkt;

  /*
   * Structure to hold the servers
   */
  struct T_FrServerInfo {
    FrEntIdType entid;
    char    *entname; /* Bug: temporary - remove this */
    char    *host;
    char    *ip;
    uint16_t port;
    int      timeout;
  };
  typedef struct T_FrServerInfo *FrServerInfo;

  /*
   * Structure to hold the Entity Id mappings
   */
  struct T_FrEntityId {
    FrEntIdType etype;
    long    id;
    char    *entity;
  };
  typedef struct T_FrEntityId *FrEntityId;

  struct T_FrDBOTypeEntId {
    uint32_t regID; /* MUST BE THE FIRST */
    FrEntIdExp ent;
  };
  typedef struct T_FrDBOTypeEntId *FrDBOTypeEntId;

  struct T_FrDBOTypeAipStatus {
    uint32_t regID; /* MUST BE THE FIRST */
    uint32_t status;
  };
  
  typedef struct T_FrDBOTypeAipStatus *FrDBOTypeAipStatus;

  /* Context */
  extern int frNIDBContextNew (ZkError, FrNIDBContext);
  extern int frNIDBContextInit (ZkError, FrNIDBContext);
  extern int frNIDBContextFree (ZkError, FrNIDBContext);
  extern int nidbConfigSave (FrNIDBContext, const char * /* configfile */);
  extern int frNIDBGenECommReport (FrNIDBContext nidb);

  /** Load the database from the config file */
  extern int nidbUpdateDatabaseFromConfig (FrNIDBContext);

  /** Dealing with Updates from the AIPs */
  extern int FrNIDBAipDataPktNew (ZkError, FrNIDBAipDataPkt *);
  extern int FrNIDBAipDataPktFree (ZkError, FrNIDBAipDataPkt *);
  extern int frNIDBAipDataPktStore (FrNIDBContext, FrNIDBAipDataPkt);
  extern int frNIDBAipDump(ZkError, FrNIDBAipDataPkt);
  extern int frNIDBAipDataPktLog(FrNIDBContext, FrNIDBAipDataPkt);
  extern int frNIDBProcessAipUpdatePktFromFile (FrNIDBContext, char *);
  extern int frNIDBAipDataPktLoad(ZkError err, FrNIDBAipDataPkt, char *);
  extern int frNIDBLoadAIPUpdateBuffer(ZkError, FrNIDBAipDataPkt, char *,size_t);
  extern int frNIDBAipDataPktConsolidate(FrNIDBContext, FrNIDBAipDataPkt);


  /* Entity/Id Requests */
  extern int nidbRegisterEntityIDinDB (FrNIDBContext, char *, long, FrEntIdType);
  extern int nidbGetEntityIDfromDBwithEntity(FrNIDBContext, char *,
					     short, FrEntityId);
  extern int nidbGetEntityIDFromDBwithID(FrNIDBContext, long, short, FrEntityId);
  extern int nidbPutEntityIDinDB(FrNIDBContext, FrEntityId);
  extern int nidbGetIDFromEntity(FrNIDBContext, char *, short, long *);
  extern int nidbGetEntityFromID(FrNIDBContext, long, short, char **);

  /* Info Requests */
  extern int nidbIsAipIDValid (FrNIDBContext, int);
  extern int nidbPutAipInfoToDB(FrNIDBContext, FrAIPinfo);
  extern int nidbPutLinkInfoToDB(FrNIDBContext, FrLinks);
  extern int nidbGetAipInfoFromDB(FrNIDBContext, long, FrAIPinfo);
  extern int nidbGetLinkInfoFromDB(FrNIDBContext, long, FrLinks);
  extern int nidbPutServerInfoToDB(FrNIDBContext, FrServerInfo);
  extern int nidbPutAipUpdateInDB(FrNIDBContext, FrNIDBAipUpdateInfo);
  extern int nidbPutLinkUpdateInDB(FrNIDBContext, FrNIDBLinkUpdateInfo);

  /* AIP API */
  extern int nidbAipGetID (FrNIDBContext, int, int * /*aipid*/);
  extern int nidbAipGetEntity (FrNIDBContext, int, char **);
  extern int nidbAipGetName (FrNIDBContext, int, char **);
  extern int nidbAipGetIP (FrNIDBContext, int, struct in_addr *);
  extern int nidbAipGetTCPport (FrNIDBContext, int, ushort *);
  extern int nidbAipGetUDPport (FrNIDBContext, int, ushort *);
  extern int nidbAipGetIsExitFlag (FrNIDBContext, int, int *);
  extern int nidbAipGetIsUpFlag (FrNIDBContext, int, int *);
  extern int nidbAipGetRegion (FrNIDBContext, int, char **);
  extern int nidbAipGetRating (FrNIDBContext, int, int *);
  extern int nidbAipGetProtocol (FrNIDBContext, int, uint *);

  extern int nidbAipSetID (FrNIDBContext, int, int /* aipid */, int);
  extern int nidbAipSetEntity (FrNIDBContext, int, char *, int);
  extern int nidbAipSetName (FrNIDBContext, int, char *, int);
  extern int nidbAipSetIP (FrNIDBContext, int, struct in_addr, int);
  extern int nidbAipSetTCPport (FrNIDBContext, int, ushort, int);
  extern int nidbAipSetUDPport (FrNIDBContext, int, ushort, int);
  extern int nidbAipSetIsExitFlag (FrNIDBContext, int, int, int);
  extern int nidbAipSetIsUpFlag (FrNIDBContext, int, int, int);
  extern int nidbAipSetRegion (FrNIDBContext, int, char *, int);
  extern int nidbAipSetRating (FrNIDBContext, int, int, int);
  extern int nidbAipSetProtocol (FrNIDBContext, int, uint, int);

  extern int nidbLinkGetID   (FrNIDBContext, int, int *);
  extern int nidbLinkGetAips (FrNIDBContext, int, int *, int *);
  extern int nidbLinkGetCost (FrNIDBContext, int, int *);
  extern int nidbLinkSetID   (FrNIDBContext, int, int, int);
  extern int nidbLinkSetAips (FrNIDBContext, int, int, int, int);
  extern int nidbLinkSetCost (FrNIDBContext, int, int, int);

  extern int nidbGenGetRevision (FrNIDBContext, char **);
  extern int nidbGenGetRevisionDate (FrNIDBContext, char **);
  extern int nidbGenGetExpirationDate (FrNIDBContext, char **);
  extern int nidbGenGetTTL (FrNIDBContext, uint32_t *);
  extern int nidbGenGetDowloadURL (FrNIDBContext, char **);
  extern int nidbGenGetNumAips (FrNIDBContext, int *);
  extern int nidbGenGetNumLinks (FrNIDBContext, int *);
  extern int nidbGenSetRevision (FrNIDBContext, char *);
  extern int nidbGenSetRevisionDate (FrNIDBContext, char *);
  extern int nidbGenSetExpirationDate (FrNIDBContext, char *);
  extern int nidbGenSetTTL (FrNIDBContext, uint32_t);
  extern int nidbGenSetNumAips (FrNIDBContext, int);
  extern int nidbGenSetNumLinks (FrNIDBContext, int);
  extern int nidbGenSetDowloadURL (FrNIDBContext, char *);

  extern int nidbWCGetRevision (FrNIDBContext , char **);
  extern int nidbWCGetRevisionDate (FrNIDBContext , char **);
  extern int nidbWCGetInfoURL (FrNIDBContext , char **);
  extern int nidbWCGetUpgradeURL (FrNIDBContext , char **);
  extern int nidbWCGetdownloadURL (FrNIDBContext , char **);
  extern int nidbWCGetExpireClientDate (FrNIDBContext , char **);
  extern int nidbWCGetUrgency (FrNIDBContext , uint32_t *);
  extern int nidbWCSetRevision (FrNIDBContext , char *);
  extern int nidbWCSetRevisionDate (FrNIDBContext , char *);
  extern int nidbWCSetInfoURL (FrNIDBContext , char *);
  extern int nidbWCSetUpgradeURL (FrNIDBContext , char *);
  extern int nidbWCSetdownloadURL (FrNIDBContext , char *);
  extern int nidbWCSetExpireClientDate (FrNIDBContext , char *);
  extern int nidbWCSetUrgency (FrNIDBContext , uint32_t );

  extern int nidbServersGetNymServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetTokenServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetNewsgrpServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetDnsServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetNidbServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetKeyupdServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetKeyqryServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetFmgmaipServer (FrNIDBContext , FrServerInfo);
  extern int nidbServersGetAipudpport (FrNIDBContext , uint16_t *);
  extern int nidbServersGetAiptcpport (FrNIDBContext , uint16_t *);
  extern int nidbServersGetMaiptimeout (FrNIDBContext , uint16_t *);
  extern int nidbServersGetMaipport (FrNIDBContext , uint16_t *);
  extern int nidbServersGetClientudptcpport (FrNIDBContext , uint16_t *);
  extern int nidbServersGetTrialserialURL (FrNIDBContext , char **);
  extern int nidbServersSetNymServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetTokenServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetNewsgrpServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetDnsServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetNidbServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetKeyupdServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetKeyqryServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetFmgmaipServer (FrNIDBContext , FrServerInfo );
  extern int nidbServersSetAipudpport (FrNIDBContext , uint16_t );
  extern int nidbServersSetAiptcpport (FrNIDBContext , uint16_t );
  extern int nidbServersSetMaiptimeout (FrNIDBContext , uint32_t );
  extern int nidbServersSetMaipport (FrNIDBContext , uint16_t );
  extern int nidbServersSetClientudptcpport (FrNIDBContext , uint16_t );
  extern int nidbServersSetTrialserialURL (FrNIDBContext , char * );

  /*
   * Local DBO definitions
   */
  extern int frDBODriverInitEntId(ZkError);
  extern int frDBODriverInitAipStatus(ZkError);

  
#ifndef NDEBUG
  extern int frNIDBAipDataPktDump(ZkError, FrNIDBAipDataPkt);
#endif
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FRNIDB_H */
