/*
 * NIDB core routines 
 *
 * $Id: fr_nidb.h,v 1.44 2001/04/19 19:22:54 alainc Exp $
 * 
 * Luc Boulianne <luc@zks.net>
 * 
 */

#ifndef FR_NIDB_H_
#define FR_NIDB_H_
#include "zks.h"
#include "zkTypes.h"

#include "zkDump.h"
#include "frDBOTypes.h"
#include "frNidbObj.h"
#include "fr_nidbctx.h"

struct link_entname {
  uint32_t node1;
  uint32_t node2;
};

typedef uint32_t FrNidbEntityID_t;
typedef FrNidbEntityID_t *FrNidbEntityID;

extern int frNidbGetEntityIdentifier (
  ZkError err,
  NidbContext ,
  FrDBOTypeEntityID entid,
  uint32_t *eid,
  int *found);

extern int frNsdbFetch (
  ZkError err,
  NidbContext c,
  FrDBOTypeNsdbAip state,
  int *found);

extern int frNsdbReplace (
  ZkError err,
  NidbContext c,
  FrDBOTypeNsdbAip state,
  int status_changed,
  int *replaced);

extern int frNsdbFetchCDV (
  ZkError err,
  NidbContext c,
  FrDBOTypeNsdbCDV cdv,
  int *found);

extern int frNsdbReplaceCDV (
  ZkError err,
  NidbContext c,
  FrDBOTypeNsdbCDV cdv,
  int *replaced);

extern int frNtdbReplaceLink (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbLink lnk,
  ZK_OUT int *replaced);

extern int frNtdbReplaceHost (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbHost host,
  ZK_OUT int *replaced);

extern int frNtdbFetch (
  ZkError err,
  NidbContext c,
  FrDBOTypeNtdbAip topo,
  int *found);

extern int frNtdbReplace (
  ZkError err,
  NidbContext c,
  FrDBOTypeNtdbAip topo,
  int *replaced);

extern int frNidbCanonLinkEntname (
  ZkError err,
  NidbContext c,
  struct link_entname *entname);

extern int frNidbGetLinkEntityName (
  ZkError err,
  NidbContext c,
  const char *node1,
  const char *node2,
  char **link_entname);

extern int frNidbGetLinkEntityFromEntityID2 (
  ZkError err,
  NidbContext c,
  FrDBOTypeEntityID entid1,
  FrDBOTypeEntityID entid2,
  uint32_t *eid,
  int *found);


extern int frNidbGetLinkEntityAndIDFromEntityID2 (
  ZkError err,
  NidbContext c,
  FrDBOTypeEntityID entid1,
  FrDBOTypeEntityID entid2,
  FrDBOTypeEntityID *entid, /* For the link */
  uint32_t *eid,
  int *found);

extern int frNidbGetLinkEntityAndIDFromEntityIdent2 (
  ZkError err,
  NidbContext c,
  FrEntIdent eid1,
  FrEntIdent eid2,
  FrDBOTypeEntityID *link_eidbo, /* For the link */
  uint32_t *link_eid,
  int *found);

extern int frNidbGetLinkEntityFromEntities (
  ZK_INOUT ZkError err,
  ZK_IN  NidbContext c,
  ZK_IN  FrEntIdExp *ent1,
  ZK_IN  FrEntIdExp *ent2,
  ZK_OUT FrEntIdExp **lnk);

extern int frNtdbGetEntityDBOUsingEntityIdentAndTypeStr (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN uint32_t eid,
  ZK_IN const char *type,
  ZK_OUT void **dbo,  /* Caller is responsible to free this */
  ZK_OUT size_t *dsz,
  ZK_OUT int *found);

extern int frNtdbGetEntityDBOUsingEntityIdentAndType (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN uint32_t eid,
  ZK_IN FrEntIdType type,
  ZK_OUT void **dbo,  /* Caller is responsible to free this */
  ZK_OUT size_t *dsz,
  ZK_OUT int *found);

extern int frNtdbGetEntityDBOsUsingEntityIdent (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN uint32_t eid,
  ZK_OUT FrDBOTypeEntityID *ent_dbo,
  ZK_OUT void **dbo,
  ZK_OUT size_t *dsz,
  ZK_OUT int *found);

extern int frNidbGetEntitiesOfType (
  ZkError err,
  NidbContext c,
  FrEntIdType type,
  FrNidbEntityID *ent_list,
  size_t *num_elems);

extern int frNidbGetEntitiesInTypesetUpdatedSince (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t typeset,
  FrDBOTypeEntityID *ent_ident_list,
  size_t *num_elems);

extern int frNidbGetEntitiesInTypeset (
  ZkError err,
  NidbContext c,
  uint32_t typeset,
  FrDBOTypeEntityID *ent_list,
  size_t *num_elems);

extern int frNidbGetAllEntityIDs (
  ZkError err,
  NidbContext c,
  FrNidbEntityID *ent_list,
  size_t *num_elems);

extern int frNidbEntityGetEidFromTypeAndName (
  ZkError err,
  NidbContext c,
  const char *type,
  const char *name,
  FrDBOTypeEntityID *entdbo,
  int *found);

extern int frNidbEntityGetEntIdentFromTypeAndName (
  ZkError err,
  NidbContext c,
  const char *type,
  const char *name,
  FrEntIdent *ent_ident,
  int *found);

extern int frNidbGetEntDboFromEntity (
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  FrDBOTypeEntityID *ent_dbo,
  size_t *dsz,
  int *found);

extern int frNidbAipEntityGetIpAndPort(
 ZkError err,
 NidbContext c,
 char *type,
 char *name,
 uint32_t *ipaddr,
 uint16_t *port,
 int *found);

extern int frNidbEidFetchEDS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  FrDBOTypeEntityID * entid,
  int *found);

extern int frNadbArchive (
  ZkError err,
  NidbContext c,
  FrDBOTypeNsdbAip state,
  char *entname,
  int sent,
  int /* recv */,
  int *archived);

extern int frNtdbDeleteNode (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbNode node,
  ZK_OUT int *deleted);

extern int frNtdbDeleteLink (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbLink lnk,
  ZK_OUT int *deleted);

extern int frNtdbDeleteAip (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbAip aip,
  ZK_OUT int *deleted);

extern int frNtdbDeleteHost (
  ZK_INOUT ZkError err,
  ZK_IN NidbContext c,
  ZK_IN FrDBOTypeNtdbHost host,
  ZK_OUT int *deleted);

extern int frNtdbNeighbourAdd(
  ZkError err,
  NidbContext c,
  uint32_t eid1,
  uint32_t eid2,
  int *ok);

extern int frNtdbNeighbourRemove(
  ZkError err,
  NidbContext c,
  uint32_t eid1,
  uint32_t eid2,
  int *ok);

extern int frNtdbGetLocalTopology(
  ZkError err ,
  NidbContext c,
  FrDBOTypeEntityID entid,
  void ** response,
  size_t * rsz,
  int *found);

extern int frNtdbGetCompleteLocalTopology(
  ZkError err ,
  NidbContext c,
  FrDBOTypeEntityID entid,
  void **response,
  size_t * rsz,
  int *found);

extern int frNidbGetStatusUpdateSince (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t mask,
  uint32_t resp_vers,
  FrNidbObj *resp,
  size_t *osz,
  int *found);

extern int frNtdbQueryListAllEntityIDs (
  ZkError err,
  NidbContext c,
  void **response,
  size_t * rsz,
  int *found);

extern int frNidbEidFetchDS (
  ZkError err,
  ZkDB db,
  uint32_t eid_num,
  void **ds,
  size_t *dsz,
  int *found);

#if 0
extern int frNidbEidReplaceDS (
  ZkError err,
  ZkDB db,
  uint32_t eid_num,
  void *ds,
  size_t dsz,
  int *replaced);
#endif

extern int frNtdbGetHostDS (
  ZkError err,
  NidbContext c,
  const char *hostentityname,
  FrDBOTypeNtdbHost *host_entname,
  FrDBOTypeEntityID *entDbo,
  int *found);

extern int frNtdbEntityGetIpAddr(
 ZkError err,
 NidbContext c,
 const char *type,
 const char *name,
 uint32_t *ipaddr,
 int *found);

int frNidbGetDescObjFromTypeEntityID (
  ZkError err,
  NidbContext c,
  FrDBOTypeEntityID ent_dbo,
  uint32_t resp_vers,
  FrNidbObj *resp,
  size_t *rsz,
  int *found);

extern int frNidbGetDescObjForEntity (
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  uint32_t resp_vers,
  FrNidbObj *resp,
  size_t *rsz,
  int *found);

int frNidbGetDescObjListFromTypeEntityIDList(
  ZkError err,
  NidbContext c,
  FrDBOTypeEntityID ent_ident_list,
  size_t num_entities,
  uint32_t resp_vers,
  FrNidbObj *resp,
  size_t *rsz,
  int *found );

extern int frNtdbGetAllEntityDBOsUsingEntityIdent (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  FrDBOTypeEntityID *ent_dbo,
  void **dbo,
  size_t *dsz,
  int *found);

extern int frNidbFindEntIdentUsingEntity(
  ZkError err,
  NidbContext c,
  const FrEntIdExp *entity,
  FrEntIdent *ent_ident,
  int *found);

extern int frNidbFindEntityIdDboUsingEntity(
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  FrDBOTypeEntityID *entid,
  int *found);

extern int frNidbGetStatusForEntIdent (
  ZkError err,
  NidbContext c,
  FrEntIdent ent_ident,
  uint32_t resp_vers,
  void **resp,
  size_t *rsz,
  int *found);

extern int frNidbSetEntityLastUpdateTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  int *found);

extern int frNidbGetEntityLastUpdateTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  time_t *last_update,
  int *found);

int frNidbDeleteEntityLastUpdateTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  int *found);

extern int frNidbGetEntityAnyUpdateTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  time_t *last_update,
  time_t *status_update,
  int *found);

extern int frNidbSetEntityStatusChangeTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  int *found);

extern int frNidbGetEntityStatusChangeTS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  time_t *status_change,
  int *found);

extern int frNsdbGetEntIdentStatusAndRating (
  ZkError err,
  NidbContext c,
  FrEntIdent eid,
  uint32_t *status,
  uint32_t *rating,
  int *found);

extern int frNsdbFetchDSforEID (
  ZkError err,
  NidbContext c,
  FrEntIdent eid,
  FrDBOTypeNsdbAip *state,
  int *found);

  
extern int frNidbDumpStateBuf (
  ZkError err, 
  FrDBOTypeNsdbAip state,
  void * idx,
  char ** buf);

extern int frNidbDumpState (
  ZkError err, 
  FrDBOTypeNsdbAip state,
  ZkDumpCallback callback,
  void * cdata,
  void * userdata);

extern int frNidbGetStatusConnections (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t entity_mask,
  FrNidbObj *resp,
  size_t *rsz,
  int *found);


/*
**------------------------------------------------------------------------------
** The following functions are used to overload the FrDBOTypeNsdbAip object
** with data from various types of entity:
**     - AIP
**     - LINK
**     - MAIP
**
** These should be used until the frDBO library is upgraded to use the new
** more generic object
**
** ( LBe - 1999.10.19 )
**------------------------------------------------------------------------------
*/

extern int
frNsdbSetStatusForAip(
    ZkError err,
    FrDBOTypeNsdbAip state,

    time_t    reptime,
    time_t    launchtime,
    time_t    last_status_time,
    time_t    last_niqs_reload_time,
    uint32_t  new_client_connections,
    uint32_t  curr_connected_clients,
    UINT64_T  client_bytes_rx,
    UINT64_T  client_bytes_tx,
    UINT64_T  client_bytes_dropped,
    UINT64_T  client_pkts_rx,
    UINT64_T  client_pkts_tx,
    UINT64_T  client_pkts_dropped,
    UINT64_T  total_bytes_in,
    UINT64_T  total_bytes_out,
    UINT64_T  total_bytes_dropped,
    time_t    time_period,
    char *    release
);

extern int
frNsdbGetStatusForAip(
    ZkError err,
    FrDBOTypeNsdbAip state,

    time_t *   reptime,
    time_t *   launchtime,
    time_t *   last_status_time,
    time_t *   last_niqs_reload_time,
    uint32_t * new_client_connections,
    uint32_t * curr_connected_clients,
    UINT64_T * client_bytes_rx,
    UINT64_T * client_bytes_tx,
    UINT64_T * client_bytes_dropped,
    UINT64_T * client_pkts_rx,
    UINT64_T * client_pkts_tx,
    UINT64_T * client_pkts_dropped,
    UINT64_T * total_bytes_in,
    UINT64_T * total_bytes_out,
    UINT64_T * total_bytes_dropped,
    time_t   * time_period,
    char **    release
);

extern int
frNsdbSetStatusForWormhole(
    ZkError err,
    FrDBOTypeNsdbAip state,

    uint32_t  maxin_thrput,
    uint32_t  new_streams,
    uint32_t  current_active_streams,
    uint32_t  new_ports,
    uint32_t  curr_used_ports, 
    UINT64_T  new_bytes_rx,
    UINT64_T  new_bytes_tx
);

extern int
frNsdbGetStatusForWormhole(
    ZkError err,
    FrDBOTypeNsdbAip state,
    uint32_t  * maxin_thrput,
    uint32_t  * new_streams,
    uint32_t  * current_active_streams,
    uint32_t  * new_ports,
    uint32_t  * curr_used_ports, 
    UINT64_T  * new_bytes_rx,
    UINT64_T  * new_bytes_tx
);

extern int
frNsdbSetStatusForLink_lowEnd(
    ZkError err,
    FrDBOTypeNsdbAip state,
    const FrEntIdExp  *nb_entity,
    ZkBool            up_flag,
    time_t            launchtime,
    time_t            last_change_time,
    time_t            last_status_time,
    UINT64_T          new_pkts_rx,
    UINT64_T          new_pkts_tx,
    UINT64_T          new_pkts_drop,
    UINT64_T          new_bytes_rx,
    UINT64_T          new_bytes_tx,
    UINT64_T          new_bytes_drop
);

extern int
frNsdbGetStatusForLink_lowEnd(
    ZkError err,
    FrDBOTypeNsdbAip state,
    const FrEntIdExp ** nb_entity,
    ZkBool           * up_flag,
    time_t           * launchtime,
    time_t           * last_change_time,
    time_t           * last_status_time,
    UINT64_T         * new_pkts_rx,
    UINT64_T         * new_pkts_tx,
    UINT64_T         * new_pkts_drop,
    UINT64_T         * new_bytes_rx,
    UINT64_T         * new_bytes_tx,
    UINT64_T         * new_bytes_drop
);

extern int
frNsdbGetUpStatusForLink(
    ZkError err,
    FrDBOTypeNsdbAip  state,
    ZkBool          * up_flag);

extern int
frNsdbSetUpStatusForLink(
    ZkError err,
    FrDBOTypeNsdbAip  state,
    ZkBool            up_flag);

extern int
frNsdbSetStatusForLink_highEnd(
    ZkError err,
    FrDBOTypeNsdbAip state,
    const FrEntIdExp  *nb_entity,
    ZkBool            up_flag,
    time_t            launchtime,
    time_t            last_change_time,
    time_t            last_status_time,
    UINT64_T          new_pkts_rx,
    UINT64_T          new_pkts_tx,
    UINT64_T          new_pkts_drop,
    UINT64_T          new_bytes_rx,
    UINT64_T          new_bytes_tx,
    UINT64_T          new_bytes_drop
);

extern int
frNsdbGetStatusForLink_highEnd(
    ZkError err,
    FrDBOTypeNsdbAip state,
    const FrEntIdExp ** nb_entity,
    ZkBool           * up_flag,
    time_t           * launchtime,
    time_t           * last_change_time,
    time_t           * last_status_time,
    UINT64_T         * new_pkts_rx,
    UINT64_T         * new_pkts_tx,
    UINT64_T         * new_pkts_drop,
    UINT64_T         * new_bytes_rx,
    UINT64_T         * new_bytes_tx,
    UINT64_T         * new_bytes_drop
);

extern int
frNsdbSetStatusForMaip(
    ZkError err,
    FrDBOTypeNsdbAip state,

    time_t    reptime,
    time_t    launchtime,
    time_t    last_status_time,
    uint32_t  new_msgs_sent, 
    uint32_t  new_msgs_rcvd,
    uint32_t  new_msgs_dumped,
    uint32_t  num_queued_msgs,
    UINT64_T  bytes_rx, 
    UINT64_T  bytes_tx,
    UINT64_T  total_bytes_rx, 
    UINT64_T  total_bytes_tx,
    time_t    time_period,
    char *    release
);

extern int
frNsdbGetStatusForMaip(
    ZkError err,
    FrDBOTypeNsdbAip state,

    time_t    * reptime,
    time_t    * launchtime,
    time_t    * last_status_time,
    uint32_t  * new_msgs_sent, 
    uint32_t  * new_msgs_rcvd,
    uint32_t  * new_msgs_dumped,
    uint32_t  * num_queued_msgs,
    UINT64_T  * bytes_rx, 
    UINT64_T  * bytes_tx,
    UINT64_T  * total_bytes_rx, 
    UINT64_T  * total_bytes_tx,
    time_t    * time_period,
    char     ** release
);

#endif
