/*
 * $Id: fr_nidbctx.h,v 1.11 2000/11/24 15:03:08 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 *
 */

#ifndef NIDBCTX_H_
#define NIDBCTX_H_

#include <stdio.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkConfig.h>
#include <zkAA.h>
#include <zkParms.h>
#include <mysql/mysql.h>

/*
 * Default context values
 */

#define DEFAULT_TOPODB          "/freedom/etc/db/nidb-topology.local.conf"
#define DEFAULT_STATEDB         "/freedom/etc/db/nidb-state.local.conf"
#define DEFAULT_ENTDB           "/freedom/etc/db/nidb-entity.local.conf"
#define DEFAULT_NEIBRDB         "/freedom/etc/db/nidb-neighbour.local.conf"
#define DEFAULT_ENTIDDB         "/freedom/etc/db/nidb-entityid.local.conf"
#define DEFAULT_ENTUPDDB        "/freedom/etc/db/nidb-entityupdate.local.conf"
#define DEFAULT_HOSTDB          "/freedom/etc/db/nidb-host.local.conf"
#define DEFAULT_EVRECDB         "/freedom/etc/db/nidb-nissevrecdb.local.conf"
#define DEFAULT_ARCHIVE_SQLHOST "akira.zks.net"
#define DEFAULT_ARCHIVE_SQLUSER "luc"
#define DEFAULT_ARCHIVE_SQLPSWD ""
#define DEFAULT_ARCHIVE_SQLDB   "test"

struct T_NidbContext {
  ZkError err;
  ZkLog log;
  
  const char * topodb;		/* Topology Database */
  const char * statedb;		/* State Database */
  const char * entdb;		/* Entity Database */
  const char * entupddb;	/* Entity Last Update Time Database */
  const char * entiddb;		/* Entity Identifier Database */
  const char * hostdb;		/* Hostname/Ipaddr Database */
  const char * neibrdb;		/* Neighbour Database */
  const char * archive_sqlhost;
  const char * archive_sqluser;
  const char * archive_sqlpswd;
  const char * archive_sqldb;

  ZkDB topoDB;
  ZkDB stateDB;
  ZkDB neibrDB;
  ZkDB hostDB;
  ZkAA entDB;
  ZkAA entupdDB;
  ZkAA entidDB;
  MYSQL *archiveDB;

  
};

typedef struct T_NidbContext *NidbContext;

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


extern int nidbContextInit (ZkError , NidbContext);
extern int nidbContextDump (ZkError , NidbContext, FILE * );
extern int nidbContextLoad (ZkError , ZkLog , ZkConfig , NidbContext );
extern int nidbContextVerify (ZkError ,  NidbContext );
extern int nidbContextCleanup (ZkError , NidbContext );

#endif /* NIDBCTX_H_ */
