#if 0
/*
  *  Prepare a report for the Commerce Server
  *
  *  $Id: commrep.c,v 1.5 2000/11/24 15:03:08 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
  *  Luc Boulianne <luc@zks.net>

  */

#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkFile.h>
#include <frNDB.h>
#include <frNCDB.h>
#include <frSec.h>

#include <frNIDB.h>


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

static int usage(FrNIDBContext c)
{
  fprintf (stderr,
	   "Usage: %s \
\t--database=databasefile\n\
\t\t--key=private-keyfile\n\
\t\t--outfile=outfile\n\
", c->progname);
  exit(EXIT_FAILURE);
}

static int doNIDBSrvArgs(int argc, char **argv, FrNIDBContext ctx)
{
  int c;
  int option_index = 0;
  struct option long_options[] =
  {
    {"help", no_argument, 0, 'h'},
    {"database", required_argument, 0, 'd'},
    {"key", required_argument, 0, 'k'},
    {"outfile", required_argument, 0, 'o'},
    {0, 0, 0, 0}
  };

  ctx->progname = argv[0];

  while (1) {
    c = getopt_long (argc, argv, "hb:d:k:o:",
		     long_options, &option_index);
    if (c == -1)
      break;
    
    switch (c) {
#ifndef NDEBUG      
    case 0:
      fprintf (stderr, "option %s", long_options[option_index].name);
      if (optarg)
	fprintf (stderr, " with arg %s", optarg);
      fprintf (stderr, "\n");
      break;
#endif

    case 'd':
      ctx->database = strdup(optarg);
      break;

    case 'k':
      ctx->keyfile = strdup(optarg);
      break;

    case 'o':
      ctx->outfile = strdup(optarg);
      break;

    case 'h':
      usage(ctx);
      break;
      
    case '?':
      fprintf (stderr, "Unknown arg [%c]\n", c);
      usage (ctx);
      break;
      
    default:
      fprintf (stderr, "getopt returned character code 0%o[%c]\n", c, c);
    }
  }
  
  if (optind != argc) {
    fprintf (stderr, "Unknown argument: ");
    while (optind < argc) 
      fprintf (stderr, "[%s] ", argv[optind++]);
    fprintf (stderr, "\n\n");
    usage(ctx);
  }
  return 1;
}

int main(int argc, char **argv)
{
  ZkError err;
  struct T_FrNIDBContext nidb;
  FrNDB	ndb;

  if (!zkErrorNew(&err)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }
  if (!frNIDBContextNew (err, &nidb)) {
    exit(EXIT_FAILURE);
  }

  if (!doNIDBSrvArgs (argc, argv, &nidb)) {
    exit (EXIT_FAILURE);
  }

  if (!frNIDBContextInit (err, &nidb)) {
    exit (EXIT_FAILURE);
  }

  nidb.ready = 1;

  if (!frNIDBGenECommReport (&nidb)) {
    exit (EXIT_FAILURE);
  }

  /* Bug: clean up the clean up */
  free(nidb.database);
  free(nidb.keyfile);
  free(nidb.spooldir);
  frCryptSigFree(err, &nidb.myPrivSigKey);
  frNDBClose(err, &ndb);
  zkDBClose(err, &nidb.db);
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}
#endif
