/*
 * $Id: frNidbReport.h,v 1.12 2000/11/24 15:03:09 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef FRNIDBREPORT_H_
#define FRNIDBREPORT_H_

#include <zks.h>

#define CMD_REPORT_USAGE \
"report ent-type ent-name"

#define CMD_REPORT_DESCR \
"Used to get info from database entries"

#define CMD_REPORT_HELP \
"this command queries information about the specified entity\n" \
"the content of the answer is:\n" \
"            TBD\n"

#include "zkIntTypes.h"

#include "fr_nidb.h"

typedef struct T_frRepEntityInfo *    FrRepEntityInfo;
typedef struct T_frRepEntityStat *    FrRepEntityStat;
typedef struct T_frRepFreedomInfo *   FrRepFreedomInfo;
typedef struct T_frRepMaipinfo *      FrRepMaipInfo;
typedef struct T_frRepNIC *           FrRepNicInfo;
typedef struct T_frRepStatusOfLink *  FrRepStatusOfLink;
typedef struct T_frRepAipinfo *       FrRepAipInfo;
typedef struct T_frRepNetworkInfo *   FrRepNetworkInfo;
typedef struct T_frRepServopInfo *    FrRepServopInfo;


/**
 * Basic structs present in all Requests
 */
struct T_frRepEntityInfo{
  char* hostname;
  char* ip;
  char* company;
  char* city;
  char* state;
  char* country;
};

struct T_frRepEntityStat{
    char*         uptime;
    char*         rating;
    char*         status;
    char*         availability;
    char*         cpuLoad;
    char*         freeDiskSpace;
    char*         freeMemorySpace;
    FrRepNicInfo  nicInfo;
};

struct T_frRepFreedomInfo{
    char* entity;
    char* release;
    char* referenceDate;
    char* launchDate;
    char* lastUpdateDate;
    char* lastStateDate;
    char* lastNiqsReloadDate;
    char* reportDate;
    char* timePeriod;
};

/**
 * Service specific structs
 */
struct T_frRepMaipinfo{
    char* nbMaipMsgRx;
    char* nbMaipMsgTx;
    char* nbMaipMsgDropped;
    char* nbMaipMsgQueued;
    char* nbMaipTotalBytesRx;
    char* nbMaipTotalBytesTx;
    char* maipBytesRx;
    char* maipBytesTx;
    char* reqstPeak;
    char* reqstAvg;
    char* reqstCurr;
};

struct T_frRepNIC{
    char* nbBytesIn;
    char* nbBytesOut;
    char* nbPktsIn;
    char* nbPktsOut;
};

struct T_frRepStatusOfLink{
   FrEntIdExp  remote_aip; /* ident of neighbour */
  /**
   * Bytes info
   */
    char* curThroughputBytesIn;
    char* curThroughputBytesOut;
    char* confThroughputBytesIn;
    char* confThroughputBytesOut;
    char* droppedBytesIn;
    char* droppedBytesOut;
  /**
   * Packets info
   */
    char* curThroughputPktsIn;
    char* curThroughputPktsOut;
    char* confThroughputPktsIn;
    char* confThroughputPktsOut;
    char* droppedPktsIn;
    char* droppedPktsOut;
    char* nbLostPktsIn;
    char* nbLostPktsOut;
  /**
   * misc
   */
    char* avgQueueLength;
    char* coverTrafficFlag;
    char* lastParamChangeForLink;
};

struct T_frRepAipinfo{
    char* nbTotalClientConnections;
    char* nbCurrentClientConnections;
 
    char* nbTotalActiveStreams;
    char* nbCurrentActiveStreams;

    char* nbTotalPortUsed;
    char* nbCurrentPortInUse;

    char* clientBytesRx;
    char* clientBytesTx;
    char* clientBytesDropped;
    char* clientPktsRx;
    char* clientPktsTx;
    char* clientPktsDropped;

    char* wormholeCurrentThroughPut;
    char* wormholeBytesRx;
    char* wormholeBytesTx;

    char* nbTotalBytesRx;
    char* nbTotalBytesTx;
    char* nbTotalBytesDropped;

    size_t i_nbNeighbours;
    size_t i_currSize;
    struct T_frRepStatusOfLink Links[1]; 
};

/**
 * Summary specific structs
 */
struct T_frRepNetworkInfo {
    char* nbServersDown;
    char* nbServersUp;
    char* nbNyms;
    char* nbConnectedClients;
    char* nbActiveStreams;
    char* currentNetworkKbps;
};

struct T_frRepServopInfo {
    char*    release;
    char*    serverStatus;
    char*    uptime;
    char* totalTrafficBytes;
    char* totalTrafficPkts;
};


#endif  /* FRNIDBREPORT_H_ */
