 /*
  *  Manage the Network Information Database
  *
  *  Load the Netconf file into the database
  *  $Id: nidb.c,v 1.11 2001/04/19 19:22:54 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
  *  Luc Boulianne <luc@zks.net>
  *
  */

#if 0
#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkFile.h>
#include <frNDB.h>
#include <frNCDB.h>
#include <frSec.h>
#include "zkCStr.h"

#include <frNIDB.h>


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

static int usage(FrNIDBContext c)
{
  fprintf (stderr,
	   "Usage: %s --key=private-keyfile --database=databasefile \n\
--netconf=filename\n",
	   c->progname);
  exit(EXIT_FAILURE);
}


static int doNIDBSrvArgs(int argc, char **argv, FrNIDBContext ctx,
			 char **netconf)
{
  int c;
  int option_index = 0;
  struct option long_options[] =
  {
    {"database", required_argument, 0, 'd'},
    {"help", no_argument, 0, 'h'},
    {"key", required_argument, 0, 'k'},
    {"netconf", required_argument, 0, 'n'},
    {0, 0, 0, 0}
  };

  ctx->progname = argv[0];

  while (1) {
    c = getopt_long (argc, argv, "hk:s:",
		     long_options, &option_index);
    if (c == -1)
      break;
    
    switch (c) {
#ifndef NDEBUG      
    case 0:
      printf ("option %s", long_options[option_index].name);
      if (optarg)
	printf (" with arg %s", optarg);
      printf ("\n");
      break;
#endif

    case 'd':
      ctx->database = strdup(optarg);
      break;

    case 'k':
      ctx->keyfile = strdup(optarg);
      break;

    case 'n':
      *netconf = strdup(optarg);
      break;
      
    case 'h':
      usage(ctx);
      break;
      
    case '?':
      printf ("Unknown arg");
      usage (ctx);
      break;
      
    default:
      printf ("getopt returned character code 0%o[%c]\n", c, c);
    }
  }
  
  if (optind != argc) {
    printf ("Unknown argument: ");
    while (optind < argc) 
      printf ("[%s] ", argv[optind++]);
    printf ("\n");
    usage(ctx);
  }
  return 1;
}

#if 0
int nidbUpdateFromDatabase (ZkError err, FrNIDBContext ctx, FrNDB ndb)
{
  FrAIPinfo ai;
  FrLinks lnk;
  /*
   * Scan the existing entries,
   * Update the AIPs and Links
   * Detect changed AIPS and Links
   * Write the config file out
   */
  
  if (ncdb->aipinfo) {
    for (ai = ncdb->aipinfo; ai && ai->id >= 0; ai++) {
	
    }
  } else {
    zkDebug_va ("***No aipinfo");
  }
  if (ncdb->links) {
    for (lnk = ncdb->links; lnk && lnk->id >= 0; lnk++) {
      frNCDBLinkInfoDump (lnk);
    }
  } else {
    zkDebug_va ("***No links");
  }
  return 1;
}
#endif

int main(int argc, char **argv)
{
  ZkError err;
  FrNIDBContext nidb;
  FrNDB	ndb;
  char *netconf;
  

  if (!zkErrorNew(&err)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }

  /* BUG! */
  if (!doNIDBSrvArgs (argc, argv, nidb, &netconf)) {
    exit (EXIT_FAILURE);
  }

  if (!frNIDBContextNew (err, &nidb, argc, argv)) {
    exit(EXIT_FAILURE);
  }

  if (!frNIDBContextInit (err, nidb)) {
    exit (EXIT_FAILURE);
  }

  if (!frNCDBNew(err, &(nidb->ncdb)))
    return 0;

  zkCStrlcpy(nidb->ncdb->filename, netconf, sizeof nidb->ncdb->filename);
  
  if (!frNCDBLoad (err, nidb->ncdb, NULL)) {
    return 0;
  }

  nidb->ready = 1;

#ifndef NDEBUG  
  frNCDBDump(nidb->ncdb);
#endif

  if (!nidbUpdateDatabaseFromConfig (nidb))
    return 0;

  /* Don't know where to put this yet. Making a note */
  if (!nidbConfigSave (nidb, "x"))
    return 0;
  
  free(nidb->database);
  free(nidb->keyfile);
  free(nidb->spooldir);
  free(netconf);
  frCryptSigFree(err, &nidb->myPrivSigKey);
  frNDBClose(err, &ndb);
  zkDBClose(err, &nidb->db);
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}
#endif
