/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#if 0
/*
 * Handle incoming AIP updates
 *
 */

#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkDump.h>
#include <zkBase64.h>
#include <zkDBO.h>
#include <zkDBODefs.h>
#include <zkFile.h>
#include <frSec.h>
#include <frNIDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


static int usage(FrNIDBContext c)
{
  fprintf (stderr,
	   "Usage: %s \
\t--database=databasefile\n\
\t\t--key=private-keyfile\n\
\t\t--logfile=logfile\n\
\t\t--spooldir=filename\n\
", c->progname);
  exit(EXIT_FAILURE);
}

static int doNIDBSrvArgs(int argc, char **argv, FrNIDBContext ctx)
{
  int c;
  int option_index = 0;
  struct option long_options[] =
  {
    {"help", no_argument, 0, 'h'},
    {"database", required_argument, 0, 'd'},
    {"key", required_argument, 0, 'k'},
    {"logfile", required_argument, 0, 'l'},
    {"spooldir", required_argument, 0, 's'},
    {0, 0, 0, 0}
  };

  ctx->progname = argv[0];

  while (1) {
    c = getopt_long (argc, argv, "hk:s:",
		     long_options, &option_index);
    if (c == -1)
      break;
    
    switch (c) {
#ifndef NDEBUG      
    case 0:
      printf ("option %s", long_options[option_index].name);
      if (optarg)
	printf (" with arg %s", optarg);
      printf ("\n");
      break;
#endif

    case 'd':
      ctx->database = strdup(optarg);
      break;

    case 'k':
      ctx->keyfile = strdup(optarg);
      break;

    case 'l':
      ctx->logfile = strdup(optarg);
      break;

    case 's':
      ctx->spooldir= strdup(optarg);
      break;

    case 'h':
      usage(ctx);
      break;
      
    case '?':
      printf ("Unknown arg");
      usage (ctx);
      break;
      
    default:
      printf ("getopt returned character code 0%o[%c]\n", c, c);
    }
  }
  
  if (optind != argc) {
    printf ("Unknown argument: ");
    while (optind < argc) 
      printf ("[%s] ", argv[optind++]);
    printf ("\n");
    usage(ctx);
  }
  return 1;
}


static int nidbDumpUpdate(ZkError err, FrNIDBContext nidb)
{
  void *pebuf = NULL;
  void *pabuf = NULL;
  int retcod, found;
  FrDBOTypeEntId dboEnt = NULL;
  FrDBOTypeAipStatus aipStat = NULL;
  size_t esz = 0, pasz, pesz;
  size_t asz = 0;
  
  if (!zkDBONew(err, ZKDBO_REG_FRENTID, (void **) &dboEnt, &esz))
    return 0;

  if (!zkDBONew(err, ZKDBO_REG_FRAIPSTATUS, (void **) &aipStat, &asz))
    return 0;

  if (!zkAA_CursorOpen(err, nidb->aipUpdateDB))
    return 0;
    
  while ((retcod = zkAA_CursorGetNext(err, nidb->aipUpdateDB,
			       (void **) &pebuf, &pesz,
			       (void **) &pabuf, &pasz, &found)) && found) {

    zkDebug("from DB: Key (entity)");
    zkDumpBuffer(err,pebuf,pesz,zkDumpToStream,stderr);
    zkDebug("from DB: data");
    zkDumpBuffer(err,pabuf,pasz,zkDumpToStream,stderr);

    if (!zkDBOUnpack (err, (void **)&dboEnt, &esz, pebuf, pesz)) {
      zkAA_CursorAbort(err, nidb->aipUpdateDB);
      return 0;
    }

    if (!zkDBOUnpack (err, (void **)&aipStat, &asz, pabuf, pasz)) {
      zkAA_CursorAbort(err, nidb->aipUpdateDB);
      return 0;
    }

    zkDebug("Dumping Entity");
    if (!zkDBODump (err, dboEnt, pebuf, pesz)) {
      zkAA_CursorAbort(err, nidb->aipUpdateDB);
      return 0;
    }
    
    zkDebug("Dumping Aipstat");
    if (!zkDBODump (err, aipStat, pabuf, pasz)) {
      zkAA_CursorAbort(err, nidb->aipUpdateDB);
      return 0;
    }

  }

  if (!retcod) {
    zkAA_CursorAbort(err, nidb->aipUpdateDB);
    return 0;
  }

  if (!zkAA_CursorClose(err, nidb->aipUpdateDB))
    return 0;

  free (pabuf);
  free (pabuf);
  
  return 1;
}


int main(int argc, char **argv)
{
  ZkError err;
  struct T_FrNIDBContext context;
  FILE *testfp;
  

  if (!zkErrorNew(&err)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }

  zkDebugOn();
  
  if (!frNIDBContextNew (err, &context)) {
    exit(EXIT_FAILURE);
  }

  if (!doNIDBSrvArgs (argc, argv, &context)) {
    exit (EXIT_FAILURE);
  }
  
  if (!frNIDBContextInit (err, &context)) {
    exit (EXIT_FAILURE);
  }

  /* Test Stuff */
  if (!(testfp = fopen("/tmp/xfer/aipsim", "r")))
    exit (EXIT_FAILURE);
  
  if (!frDBODriverInitEntId(err)) {
    exit (EXIT_FAILURE);
  }

  if (!frDBODriverInitAipStatus(err)) {
    exit (EXIT_FAILURE);
  }

  if (!nidbDumpUpdate(err, &context)) 
      exit(EXIT_SUCCESS);
  
  if (!frNIDBContextFree (err, &context)) {
    exit (EXIT_FAILURE);
  }
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}

#endif
