/*
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

/*
 * Handle incoming AIP updates
 *
 */
#if 0
#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkDBO.h>
#include <zkDBODefs.h>
#include <zkFile.h>
#include <frSec.h>
#include <frNIDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


static int usage(FrNIDBContext c)
{
  fprintf (stderr,
	   "Usage: %s \
\t--database=databasefile\n\
\t\t--key=private-keyfile\n\
\t\t--logfile=logfile\n\
\t\t--spooldir=filename\n\
", c->progname);
  exit(EXIT_FAILURE);
}

static int doNIDBSrvArgs(int argc, char **argv, FrNIDBContext ctx)
{
  int c;
  int option_index = 0;
  struct option long_options[] =
  {
    {"help", no_argument, 0, 'h'},
    {"database", required_argument, 0, 'd'},
    {"key", required_argument, 0, 'k'},
    {"logfile", required_argument, 0, 'l'},
    {"spooldir", required_argument, 0, 's'},
    {0, 0, 0, 0}
  };

  ctx->progname = argv[0];

  while (1) {
    c = getopt_long (argc, argv, "hk:s:",
		     long_options, &option_index);
    if (c == -1)
      break;
    
    switch (c) {
#ifndef NDEBUG      
    case 0:
      printf ("option %s", long_options[option_index].name);
      if (optarg)
	printf (" with arg %s", optarg);
      printf ("\n");
      break;
#endif

    case 'd':
      ctx->database = strdup(optarg);
      break;

    case 'k':
      ctx->keyfile = strdup(optarg);
      break;

    case 'l':
      ctx->logfile = strdup(optarg);
      break;

    case 's':
      ctx->spooldir= strdup(optarg);
      break;

    case 'h':
      usage(ctx);
      break;
      
    case '?':
      printf ("Unknown arg");
      usage (ctx);
      break;
      
    default:
      printf ("getopt returned character code 0%o[%c]\n", c, c);
    }
  }
  
  if (optind != argc) {
    printf ("Unknown argument: ");
    while (optind < argc) 
      printf ("[%s] ", argv[optind++]);
    printf ("\n");
    usage(ctx);
  }
  return 1;
}


static int nidbProcessUpdate(ZkError err, FrNIDBContext nidb, char * buf)
{
  char ename[64];
  void *pbuf = NULL;
  void *pASbuf = NULL;
  int  status;
  FrDBOTypeEntId dboEnt = NULL;
  FrDBOTypeAipStatus aipStat = NULL;
  size_t esz = 0, psz;
  size_t asz = 0;
  
  sscanf (buf, "%63s %d\n", ename, &status); /* audit: ok */

  if (!zkDBONew(err, ZKDBO_REG_FRENTID, (void **) &dboEnt, &esz))
    return 0;

  if (!zkDBONew(err, ZKDBO_REG_FRAIPSTATUS, (void **) &aipStat, &asz))
    return 0;

  if (!frEntSetFromStr2(err, &dboEnt->ent, "aip", ename))
    return 0;

  if (!zkDBOPack (err, dboEnt, esz, &pbuf, &psz))
    return 0;

  aipStat->status = status;
  
  if (!zkDBOPack (err, aipStat, asz, &pASbuf, &asz))
    return 0;

  if (!zkDBODump (err, dboEnt, pbuf, psz))
    return 0;

  if (!zkDBODump (err, aipStat, pASbuf, asz))
    return 0;

  if (!zkAAPut(nidb->err, nidb->aipUpdateDB,
	       pbuf, psz,
	       pASbuf, asz)) 
    return (0);

    
  free (pbuf);
  free (pASbuf);
  
  return 1;
}


int main(int argc, char **argv)
{
  ZkError err;
  struct T_FrNIDBContext context;
  int running = 1;
  FILE *testfp;
  char buf[256];
  

  if (!zkErrorNew(&err)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }

  if (!frNIDBContextNew (err, &context)) {
    exit(EXIT_FAILURE);
  }

  if (!doNIDBSrvArgs (argc, argv, &context)) {
    exit (EXIT_FAILURE);
  }
  
  if (!frNIDBContextInit (err, &context)) {
    exit (EXIT_FAILURE);
  }

  /* Test Stuff */
  if (!(testfp = fopen("/tmp/xfer/aipsim", "r")))
    exit (EXIT_FAILURE);
  
  if (!frDBODriverInitEntId(err)) {
    exit (EXIT_FAILURE);
  }

  if (!frDBODriverInitAipStatus(err)) {
    exit (EXIT_FAILURE);
  }

  while (fgets(buf, 256, testfp) != NULL) {
    if (!nidbProcessUpdate(err, &context, buf)) 
      exit(EXIT_SUCCESS);
    sleep (10);
  }
  
  if (!frNIDBContextFree (err, &context)) {
    exit (EXIT_FAILURE);
  }
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}

#endif
