
#if 0
/*
 * $Id: nidbtool-commands.c,v 1.5 2000/11/24 15:03:09 marcp Exp $
 */

static const char * module_id_ = "$Id: nidbtool-commands.c,v 1.5 2000/11/24 15:03:09 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkShell.h>
#include <zkDB.h>
#include "nidbtool.h"
#include "nidbtool-commands.h"

DLLExport int
cmdShowContext (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{ 
  NidbToolContext * context = (NidbToolContext *) userdata;

  nidbToolContextDump(err,context,stdout);

  return ZKSHELL_SESSION_CONTINUE;
}


#define CMD_TOK_REDEEM "redeem"
#define CMD_TOK_LIST   "list"
#define CMD_TOK_SAVE   "save"
#define CMD_TOK_DELETE "delete"
#define CMD_TOK_DESCRIPTION "description"

static int
cmdTokenRedeem (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  const char * sn )
{
  const char * func = "cmdTokenRedeem";
  FrSerialNumber frsn = NULL;
  TokenServerResponse tsresp;
  char ** tok = NULL;
  size_t toks = 0;
  size_t n = 0;
  int return_code = 0;
  int sn_sz = 0;
  unsigned int ndx = 0;
  char * demo_sn = NULL;

  zkErrorClear(err);
  if (c && sn && shell) {

    if ((strcasecmp(sn,DEMO_SERIAL_NUMBER) == 0) && c->demo_url) {
      size_t demo_sz = 0;
      if (!zkURLGetBody(err,c->demo_url,&demo_sn,&demo_sz)) {
        zkShellWrite_va(err,shell,"Unable to get serial number from \"%s\".\n",
                        c->demo_url);
        goto finish;
      }
      sn = demo_sn;
      sn_sz = (int) demo_sz;
    } else {
      sn_sz = strlen(sn);
    }

    if (!(frSerialNumberNew(err,&frsn) &&
          frSerialNumberParse(err,frsn,sn,sn_sz))) {
      zkShellWrite_va(err,shell,"Invalid serial number\n");
      goto finish;
    }
    if (!frSerialNumberRedeem(err,frsn,&(c->tokserv),&tsresp,&tok,&toks)) {
      if (tsresp.code != 0) {
        zkShellWrite_va(err,shell,"Token server response was \"%d %s\"\n",
                        tsresp.code,tsresp.message);
      } 
      zkShellWrite_va(err,shell,"Unable to get tokens\n");
      goto finish;
    }

    if (tok) {
      for (n = 0; n < toks; n++) {
        if (tok[n]) {
          if (!tokenCacheAddToken(err,c->tk,tok[n],&ndx)) {
            zkShellWrite_va(err,shell,"Unable to add token %d\n",n);
          } else {
            free(tok[n]);
            tok[n] = NULL;
          }
        }
      }
    } 
    return_code = 1;  

finish:
    if (tok) {
      free(tok);
    }
    if (frsn) {
      frSerialNumberFree(err,&frsn);
    }
    if (demo_sn) {
      free(demo_sn);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
cmdTokenDelete (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  unsigned int ndx )
{
  const char * func = "cmdTokenDelete";
  int return_code = 0;

  zkErrorClear(err);
  if (shell && c) {
    if (!tokenCacheDeleteToken(err,c->tk,ndx)) {
      zkShellWrite_va(err,shell,"Unable to delete token %d.\n",ndx);
    } else {
      zkShellWrite_va(err,shell,"Token %d deleted.\n",ndx);
      return_code = 1;
    } 
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
cmdTokenDescr (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  unsigned int ndx )
{
  const char * func = "cmdTokenDescription";
  const FrTokenTypeDescription * descr = NULL;
  int return_code = 0;
  FrToken frToken = NULL;
  const char * token = NULL;
 
  zkErrorClear(err);
  if (shell && c) {
    if (!tokenCacheGetToken(err,c->tk,ndx,&token)) {
      zkShellWrite_va(err,shell,"Token %d not found.\n",ndx);
    } else if (token) {
      if (frTokenNewFromBase64(err,&frToken,token,NULL)) {
        unsigned short typeid = 0;
        frTokenTypeId(err,frToken,&typeid);
        if (frTokenTypeLookup(err,c->tt,typeid,&descr)) {
          zkShellWrite_va(err,shell,
                          "Label        = %s\n"
                          "TypeID       = %d\n"
                          "Access-Level = %d\n"
                          "Duration     = %d days\n"
                          "Email-Limit  = %d mails/day\n"
                          "Email-Domain = %s\n\n",
                          descr->label,typeid,descr->access_level,
                          descr->duration,descr->email_limit,
                          descr->email_domain);
          return_code = 1;
        } else {
          zkShellWrite_va(err,shell,"Description not available.\n");
        }
      }
    }
    if (frToken) {
      frTokenFree(NULL,&frToken);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
} 
  

static int
cmdTokenList (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ) 
{
  const char * func = "cmdTokenList";
  const char * token = NULL;
  const FrTokenTypeDescription * descr = NULL;
  size_t toks = 0;
  size_t n = 0;
  int return_code = 0;
  FrToken frToken = NULL;


  zkErrorClear(err);
  if (shell && c) {
    if (!tokenCacheTokens(err,c->tk,&toks)) {
      goto finish;
    }
    for (n = 0; n < toks; n++) {
      if (!tokenCacheGetToken(err,c->tk,(unsigned int) n,&token)) {
        zkShellWrite_va(err,shell,"%3d { error }\n",n);
      } else if (token) {
        if (frTokenNewFromBase64(err,&frToken,token,NULL)) {
          unsigned short typeid = 0;
          frTokenTypeId(err,frToken,&typeid);
          if (!frTokenTypeLookup(err,c->tt,typeid,&descr)) {
            zkShellWrite_va(err,shell,"%3d %d\n",n,typeid);
          } else {
            zkShellWrite_va(err,shell,"%3d %s\n",n,descr->label);
          }
        }
        if (frToken) {
          frTokenFree(err,&frToken);
        }
      } else {
        zkShellWrite_va(err,shell,"%3d { unassigned }\n",n);
      }
    }

    return_code = 1;
finish:
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
cmdTokenSave (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c )
{
  const char * func = "cmdTokenSave";
  char pathname[MAXPATHLEN];
  int return_code = 0;

  zkErrorClear(err);
  if (shell && c) {
    snprintf(pathname,sizeof pathname,"%s/%s",c->root,c->tokencache);
    if (!tokenCacheSaveToFile(err,c->tk,pathname)) {
      zkShellWrite_va(err,shell,"Unable to save token cache to \"%s\".\n",
                      pathname);
    } else {
      return_code = 1;
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}



DLLExport int
cmdToken (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{
  int return_code = ZKSHELL_SESSION_CONTINUE;
  NymToolContext * context = (NymToolContext *) userdata;
  int rc = 0;

  zkErrorClear(err);
  
  if (tokens < 2) {
    zkShellWrite_va(err,shell,"Insufficient number of tokens\n"); 
    goto finish;
  }
  if (strcasecmp(token[1],CMD_TOK_REDEEM) == 0) {
    if (tokens == 3) {
      rc = cmdTokenRedeem(err,shell,context,token[2]); 
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a serial number or \"demo99\".\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_LIST) == 0) {
    if (tokens == 2) {
      rc = cmdTokenList(err,shell,context);
    } else {
      zkShellWrite_va(err,shell,"Error:  incorrect number of parameters.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_SAVE) == 0) {
    if (tokens == 2) {
      rc = cmdTokenSave(err,shell,context);
    } else {
      zkShellWrite_va(err,shell,"Error:  incorrect number of parameters.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_DELETE) == 0) {
    if (tokens == 3) {
      rc = cmdTokenDelete(err,shell,context,atoi(token[2]));
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a token number index.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_DESCRIPTION) == 0) {
    if (tokens == 3) {
      rc = cmdTokenDescr(err,shell,context,atoi(token[2]));
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a token number index.\n");
    }
  } else {
    zkShellWrite_va(err,shell,"unrecognized command \"%s\".\n",token[1]);
  }

finish:
  return return_code;
}
#endif  
