/*
 *
 * $Id: nidbtool.c,v 1.28 2001/04/19 19:22:54 alainc Exp $
 *
 * Support for availability
 * 17: entity set maip 
 *
 */
static const char * module_id_ = "$Id: nidbtool.c,v 1.28 2001/04/19 19:22:54 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>
 */

static const char * revision_id_ = "$Revision: 1.28 $";

#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkShell.h"
#include "zkDebug.h"
#include "zkDB.h"
#include "zkCStr.h"
#include "zkLib.h"
#include "frDBOTypes.h"
#include "fr_rand.h"
#include "frCrypt.h"
#include "frNetworkDefaults.h"

#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"
#include "nidbtoolCmdReport.h"
#include "nidbtoolCmdCache.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDBTOOL

#ifndef MAXPATHLEN
#define MAXPATHLEN 256
#endif /* !MAXPATHLEN */

#define DEFAULT_PROGNAME "nidbtool"
#define VERSION_SCANF_STR "Zero-Knowledge " DEFAULT_PROGNAME \
              " V%s ("__DATE__", "__TIME__")"

const int NidbBuild = 6;
 
/*
 * Internal function prototypes
 */

static int nidbToolContextInit ( ZkError , NidbToolContext * , int , char ** );
static int nidbToolContextLoad ( ZkError , NidbToolContext * );
static int nidbToolContextVerify ( ZkError , NidbToolContext * );
static int nidbToolContextCleanup ( ZkError , NidbToolContext * );

static int
nidbToolContextInit (
  ZkError err ,
  NidbToolContext * c ,
  int argc ,
  char ** argv )
{
  const char * funcname = "NidbToolContextInit";

  zkErrorClear(err);
  if (c && argv && (argc != 0)) {
    memset(c,0,sizeof(NidbToolContext));

    if ((c->progname = strrchr(argv[0],'/')) == NULL) {
      c->progname = argv[0];
    } else {
      c->progname++;
    }

    if (!(zkErrorNew(&(c->err)) &&
          zkLogNew(&(c->log)))) {
      zkErrorSet(err,funcname,ZKERR,ZKMNIDBTOOL,ZKERROR,
                 "Unable to create internal data structure");
      return 0;
    }

    if (!(zkConfigNew(err,&(c->config)) &&
          zkConfigSetCmdParms(err,c->config,argc, (const char **) argv))) {
      return 0;
    }


    c->flags = DEFAULT_FLAGS;
    c->config_file = DEFAULT_CONFIG_FILE;
    c->logfile = DEFAULT_LOG_FILE;
    {
      char t[64] = "\0";
      if (sscanf(revision_id_, "$" "Revision: %63s $", t)) /* audit: ok */
	t[strlen(t)] = '\0';
      /* Warning refers to const poisoning */
      zkCStrSprintf(&(c->prompt), DEFAULT_PROMPT, t, NidbBuild);
    }
    c->nissdHost = FR_DFLT_NISS_SERVER_HOST;
#   define STR(x) #x    
    c->nissdCmdPort = STR(FR_DFLT_NISS_SERVER_PORT);
    c->nissdCmdFd = -1;
      
    zkMALLOC (c->nidb, NidbContext, sizeof(struct T_NidbContext));
    if ( ! nidbContextInit (err, c->nidb) )
      return 0;

    return 1;
  }
  zkErrorSet(err,funcname,ZKERR,ZKMNIDBTOOL,ZKENULLPTR,NULL);
  return 0;
}

DLLExport int 
nidbToolContextDump (
  ZkError err ,
  NidbToolContext * c ,
  FILE * stream )
{
  const char * func = "nidbToolContextDump";

  zkErrorClear(err);
  if (c && stream) {
    fprintf(stream,"[ %s ]\n", S_TOOL_NIDBTOOL);
    if (c->flags & FL_DEBUG) fprintf(stream,"%s = yes\n",P_DEBUG);
    if (c->flags & FL_XML) fprintf(stream,"%s = yes\n",P_XML);
    if (c->config_file) fprintf(stream,"%s = %s\n",P_CONFIG,c->config_file);
    if (c->logfile) fprintf(stream,"%s = %s\n",P_LOGFILE,c->logfile);
    if (c->nissdHost) fprintf(stream,"%s = %s\n", P_NISS_SERVER,c->nissdHost);
    if (c->nissdCmdPort) fprintf(stream,"%s = %s\n", P_NISS_CMDPORT,
				 c->nissdCmdPort);
    fprintf(stream,"\n");
    nidbContextDump(err, c->nidb, stream);
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,NULL);
  return 0;
}


static int
nidbToolContextLoad (
  ZkError err ,
  NidbToolContext * c )
{
  const char * func = "nidbToolContextLoad";
  char * str = NULL;
  uint8_t bool = 0;
  //  uint16_t u16 = 0;

#define _SEC S_TOOL_NIDBTOOL
  
  zkErrorClear(err);
  if (c) {

    /*
     * First, get the configuration file and load it if it is specified 
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,_SEC,P_CONFIG,&str)) {
      c->config_file = str;
      if (!zkConfigLoad(err,c->config,c->config_file)) {
        return 0;
      }
    }
 
    /*
     * Do we want to run in debug mode?
     */
    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,_SEC,P_DEBUG,&bool)) {
      if (bool) {
        ZkLog log = NULL; 
        c->flags |= FL_DEBUG; 
        zkErrorLog(c->err,&log); 
        zkLogSetStream(log,stderr); 
        zkLogSetProgname(log,c->progname); 
        zkLogEnable(log);
	zkDebugOn();
        zkLogWrite_va(c->log,"Debugging enabled");
        zkLogWrite_va(log,"Debugging enabled");
      }
    }
    /*
    ** do we want output in XML format?
    */
    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,_SEC,P_XML,&bool)) {
      if (bool) {
        c->flags |= FL_XML; 

        /* must remove zkShell prompt */
        c->prompt = "";
      }
    }

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,_SEC,P_LOGFILE,&str)) {
      c->logfile = str;
    }

    if ( ! nidbContextLoad (err, c->log,  c->config, c->nidb) )
      return 0;

     return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMNIDBTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
nidbToolContextVerify (
  ZkError err ,
  NidbToolContext * c )
{
  const char * func = "nidbToolContextVerify";
  // char pathname[MAXPATHLEN];

  zkErrorClear(err);
  if (c) {
    if (c->logfile == NULL) {
      fprintf(stderr,"%s:  You must specify the \"%s\" parameter.\n",
              c->progname,P_LOGFILE);
      goto finish;
    }

    /* Always start the log file */
    if (!(c->flags & FL_DEBUG) && !zkErrorLog(c->err,&c->log)) {
      fprintf(stderr, "%s: Unable to set error logfile\n",
	      c->progname);
      return 0;
    }
    if (!(c->flags & FL_DEBUG) && !zkLogSetFile(c->log, c->logfile)) {
      fprintf(stderr,
	      "%s: Unable to open logfile \"%s\". Unable to run without one.\n",
	      c->progname,  c->logfile);
	  return (0);
    }
    zkLogEnable(c->log);

    if ( ! nidbContextVerify (err, c->nidb) )
      goto finish;

    return 1;
finish:
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKERROR,"Context verification failed");
    return 0;
  }
  zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,NULL);
  return 0;
}


static int
nidbToolContextCleanup (
  ZkError err ,
  NidbToolContext * c )
{
  const char * func = "nidbToolContextCleanup";
  int return_code = 1;

  zkErrorClear(err);
  if (c) {
    if (c->config) {
      zkConfigFree(err,&(c->config));
    }

    if ( ! nidbContextCleanup (err, c->nidb) )
      return 0;

    if ((c->flags & FL_DEBUG) && (c->log)) {
      zkLogFree(&(c->log));
    }
    if (c->err) {
      zkErrorFree(&(c->err));
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,NULL);
  return 0;
} 

int main (int argc, char ** argv)
{

  NidbToolContext context;
  ZkError err = NULL;
  int exit_code = EXIT_FAILURE;
  char * progname = NULL;
  ZkShell shell = NULL;
  
  (void) module_id_;

  if ((progname = strrchr(argv[0],'/')) == NULL) {
    progname = argv[0];
  } else {
    progname++;
  }

  zkErrorNew(&err);
  frCryptInit(err,NULL);

  
  if ( ! nidbToolContextInit(err,&context,argc,argv)) {
    fprintf(stderr,"%s:  Unable to initialize application context.\n",progname);
    goto finish;
  }
  
  if ( ! nidbToolContextLoad(context.err,&context)) {
    fprintf(stderr,"%s:  Unable to load application context.\n",progname);
    goto finish;
  }

  if ( ! nidbToolContextVerify(context.err,&context)) {
    fprintf(stderr,"%s:  Unable to verify application context.\n",progname);
    goto finish;
  }
  {
    char s[32] = "\0";
    if (sscanf(revision_id_, "$" "Revision: %31s $", s)) /* audit: ok */
      s[strlen(s)] = '\0';
    zkLogWrite_va (context.log, "Starting: " VERSION_SCANF_STR, s);

    zkLogWrite_va(context.log,"Build release: [%s]", zkLibGetBuildRelease(err));
  }

  if ( ! (zkShellNew(err,&shell) &&
	  zkShellSetContext(err,shell,&context) &&
	  zkShellSetPrompt(err,shell,context.prompt))) {
    fprintf(stderr,"%s:  Unable to initialize the application shell.\n",
            progname);
    goto finish;
  }
  if ( ! zkShellAddCommand(err,shell,cmdShowContext,"context",
	   "Display the current application context",
	   "This command gives a listing of the current context in a zkConfig"
	   "file format.",
       ZKSHELL_PARMS_NONE)) {
    goto finish;
  }
  if (!zkShellAddCommand(err,shell,cmdEntity,"entity",
                         CMD_ENTITY_USAGE,CMD_ENTITY_HELP, 
			 ZKSHELL_PARMS_10 |ZKSHELL_PARMS_9 | ZKSHELL_PARMS_8 |
                         ZKSHELL_PARMS_7 | ZKSHELL_PARMS_6 |
			 ZKSHELL_PARMS_5 | ZKSHELL_PARMS_4 |
                         ZKSHELL_PARMS_3 | ZKSHELL_PARMS_2 | ZKSHELL_PARMS_1 ) ||
      !zkShellAddCommand(err,shell,cmdReport,"report",
                         CMD_REPORT_USAGE,CMD_REPORT_HELP, 
			 ZKSHELL_PARMS_2) ||
      !zkShellAddCommand(err,shell,cmdCache,"cache",
                         CMD_CACHE_USAGE,CMD_CACHE_HELP,
                         ZKSHELL_PARMS_2 | ZKSHELL_PARMS_1) ) {
    goto finish;
  }

  if (zkShellRun(context.err,shell)) {
    exit_code = EXIT_SUCCESS;
  }

finish:
  if (exit_code == EXIT_FAILURE) {
    fprintf(stderr,"%s:  Application failure.\n",progname);
  }
 
  nidbToolContextCleanup(err,&context);
  exit(exit_code);
  return 0; /* to shut up insure */
}
