/*
 * Entity Command processor for the Add command
 *
 * $Id: nidbtoolCEAdd.c,v 1.21 2001/04/19 19:22:54 alainc Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCEAdd.c,v 1.21 2001/04/19 19:22:54 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"


int cmdEntityAddAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostentityname,   /* The host entity name this aip resides on */
  const char *aip_port,
  const char *dh_port,
  const char *dh_timeout,
  const char *dh_negotiation,
  const char *protocol_vers
  )
{
  static const char *funcname = "cmdEntityAddAip";
  FrDBOTypeNtdbHost host = NULL;
  FrDBOTypeNtdbAip aip = NULL;
  FrDBOTypeEntityID aip_entdbo = NULL;
  FrDBOTypeEntityID hostent_dbo = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int exists = 0, retcode = 0, newflag = 0;
  uint32_t eid = 0;
  void *p_aip = NULL;
  size_t psz = 0, p_aipsz = 0;
  int found = 0;
  (void) funcname;
  (void) module_id_;

  /* Start by checking if we can get the host record */
  if (! frNtdbGetHostDS (err, c->nidb, hostentityname, &host,
			 &hostent_dbo, &found)) {
    zkShellWrite_va(err,shell,
		    "Error: problem locating host entity [%s]\n",
		    hostentityname);
    goto finish;
  }

  if ( ! found ) {
    zkShellWrite_va(err,shell,
    "Error: Could not find host enitity [%s] in the database\n",
		    hostentityname);
    goto finish;
  }

  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &aip_entdbo, &psz))
    goto finish;

  if (!frEntSetFromStr2(err, &aip_entdbo->ent, type, entname)) 
    goto finish;

  if (!createEntityIdent (err, shell, c, aip_entdbo, &eid, &newflag)) 
    goto finish;

  if (!newflag) {
    zkShellWrite_va(err,shell,
		    "Error: Aip [%s] is already in the database\n", entname);
    goto finish;
  }

  /* Now we take care of the aip entity */
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Aip entity identifier (ie: key) is [%lu]\n", eid);

  
  psz = 0;
  if (!zkDBONew(err, ZKDBO_REG_FRNTDBAIP, (void **) &aip, &psz))
    goto finish;

  aip->aip_entid      = aip_entdbo->entid;
  aip->aip_port       = atoi(aip_port);
  aip->dh_port        = atoi(dh_port);
  aip->dh_timeout     = atoi(dh_timeout);
  aip->dh_negotiation = atoi(dh_negotiation);
  aip->protocol_vers  = atoi(protocol_vers);
  aip->host_entid     = hostent_dbo->entid;

  if (! zkDBOPack(err, aip, sizeof(struct T_FrDBOTypeNtdbAip), &p_aip, &p_aipsz))
    goto finish;

  if (!zkDBTNew (err, &key) || !zkDBTNew(err, &data))
    goto finish;
		 
  if (!zkDBSetKeyOrData (err, key, &aip_entdbo->entid, sizeof(aip_entdbo->entid)) ||
      !zkDBSetKeyOrData (err, data, p_aip, p_aipsz))
    goto finish;

  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }
  
  if (!zkDBStore (err, c->nidb->topoDB, key, data, &exists)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem storing entity. Not stored.\n");
    goto finish;
  }
  if (exists) {
    zkShellWrite_va(err,shell,"Error: Entity already exists. This should not happen.\n");
    goto finish;
  } else {
    zkShellWrite_va(err,shell,"Entity stored.\n");
    retcode = 1;
  }

  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, aip_entdbo->entid, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  }
  
 finish:
  if (hostent_dbo) free (hostent_dbo);
  if (aip_entdbo) free (aip_entdbo);
  if (key) free (key);
  if (data) free (data);
  if (aip) free (aip);
  if (host) free (host);
  return retcode;
}

int cmdEntityAddLink (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node1,
  int node1IsAip,
  const char *node2,
  int node2IsAip,
  const char *ts_delay,
  const char *ts_pkts,
  const char *ct_on_flag)
{
  static const char *funcname = "cmdEntityAddLink";

  FrDBOTypeNtdbLink lnk = NULL;
  FrDBOTypeEntityID entid = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int retcode = 0, newflag = 0, found = 0, ok = 0;
  uint32_t eid1 = 0;
  uint32_t eid2 = 0;
  uint32_t eid3 = 0;
  void *p_lnk = NULL;
  size_t psz = 0, p_lnksz = 0;
  struct link_entname entname;
  char buf[80];

  (void) funcname;


#warning Consolidation code code exists for this routine
  /* Just don't want to convert it with 12 hours to go for the B3 freeze */
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &psz))
    goto finish;
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNTDBLINK, (void **) &lnk, &psz))
    goto finish;
  
  /* Now we take care of the Node #1 */
  if( node1IsAip )
  {
    if ( ! frEntSetFromStr2(err, &entid->ent, "aip", node1 )) 
      goto finish;
  }
  else /* is coreaip */
  {
    if ( ! frEntSetFromStr2(err, &entid->ent, "coreaip", node1 )) 
      goto finish;
  }

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid1, &found)) 
    goto finish;

  if ( ! found) {
    zkShellWrite_va(err,shell,
		    "Error: Node [%s] is not in the database\n", node1);
    goto finish;
  }

  lnk->node1_entid = eid1;
  entname.node1 = eid1;
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Node #1 entity identifier (ie: key) is [%lu]\n", eid1);


  /* Now we take care of the Node #2 */
  if( node2IsAip )
  {
    if ( ! frEntSetFromStr2(err, &entid->ent, "aip", node2 )) 
      goto finish;
  }
  else /* is coreaip */
  {
    if ( ! frEntSetFromStr2(err, &entid->ent, "coreaip", node2 )) 
      goto finish;
  }

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid2, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Node [%s] is not in the database\n", node2);
    goto finish;
  }

  lnk->node2_entid = eid2;
  entname.node2 = eid2;
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Node #2 entity identifier (ie: key) is [%lu]\n", eid2);


  /* Now we take care of the link entity object */

  if (eid1 == eid2) {
    zkShellWrite_va(err,shell,
		    "Error: Link has identical nodes [%s - %s]\n", node1, node2);
    goto finish;
  }    

  if ( ! canonLinkEntname (err, shell, c, &entname))
    goto finish;

  if (snprintf (buf, 63, "_LINK:%lu,%lu", ( unsigned long )entname.node1, ( unsigned long )entname.node2) < 0) {
    zkShellWrite_va(err,shell,
		    "Error: Internal buffer overflow creating link name [%lu,%lu]",
		    entname.node1, entname.node2);
    return 0;
  }
  if ( ! frEntSetFromStr2(err, &entid->ent, type, buf)) 
    goto finish;

  if ( ! createEntityIdent(err, shell, c, entid, &eid3, &newflag)) 
    goto finish;

  if ( ! newflag) {
    zkShellWrite_va(err,shell,
		    "[%s] already exists in the database \n",
		    buf);
    goto finish;
  }

#warning Better command link parameter checking required.
#warning Change this when the ts-params routine has been tested.
  lnk->ts_delay =  atoi(ts_delay);
  lnk->ts_pkts =  atoi(ts_pkts);
  if (strcasecmp (ct_on_flag, "on") == 0) {
    lnk->ct_on_flag = 1;
  } else if (strcasecmp (ct_on_flag, "off") == 0) {
    lnk->ct_on_flag = 0;
  } else {
    zkShellWrite_va(err,shell,
		    "Error: [%s] is not an option for the cover traffic flag \n",
		    ct_on_flag);
    goto finish;
  }

  if (! zkDBOPack(err, lnk, sizeof(struct T_FrDBOTypeNtdbLink), &p_lnk, &p_lnksz))
    goto finish;

  if (!zkDBTNew (err, &key) || !zkDBTNew(err, &data))
    goto finish;
		 
  if (!zkDBSetKeyOrData (err, key, &entid->entid, sizeof(entid->entid)) ||
      !zkDBSetKeyOrData (err, data, p_lnk, p_lnksz))
    goto finish;

  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }
  
  if (!zkDBReplace (err, c->nidb->topoDB, key, data)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem storing entity. Not stored.\n");
    goto finish;
  }

  /* Indicate a topo change for the link */
  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, entid->entid, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  } 

#if 0
  if (exists) {
    zkShellWrite_va(err,shell,"Error: Entity already exists. This should not happen.\n");
    goto finish;
  } else {
    zkShellWrite_va(err,shell,"Entity stored.\n");
  }
#endif  

  zkShellWrite_va(err,shell,"Entity stored.\n");

  /* Now record these as neighbours */
  if (!frNtdbNeighbourAdd(err, c->nidb, eid1, eid2, &ok)) {
    zkShellWrite_va(err,shell,
		    "Error: Problem recording in the neighbour database.\n\
Database is likely inconsistent\n(err=%s)\n", zkErrorDetail(err));
    /* rollback */
    goto finish;
  }
  if (! ok) {
    zkShellWrite_va(err,shell,
		    "Error: Couldn't record neighbours in database.\n"
                    "       Database is likely inconsistent\n");
    /* bug: rollback */
  }     
  retcode = 1;

  /* Now we need to record that there has been a topo change for the nodes */
  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, eid1, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  } 
  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, eid2, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  } 


  
 finish:
  if (entid) free (entid);
  if (key) zkDBTFree (err, &key);
  if (data) zkDBTFree (err, &data);
  if (lnk) free (lnk);
  return retcode;
}

/**
 * Adds an entry in the Entity Node database
 */
int cmdEntityAddNode (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *nodename,
  const char *hostentityname,
  const char *port
  )
{
  static const char *funcname = "cmdEntityAddNode";

  FrDBOTypeNtdbHost host = NULL;
  FrDBOTypeNtdbNode node = NULL;
  FrDBOTypeEntityID node_entdbo = NULL;
  FrDBOTypeEntityID hostent_dbo = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int retcode = 0, newflag = 0, found = 0;
  uint32_t eid = 0;
  size_t psz = 0;
  size_t pnsz = 0;
  void *pnode = NULL;

  (void) funcname;

  /* Start by checking if we can get the host record */
  if (! frNtdbGetHostDS (err, c->nidb, hostentityname, &host,
			 &hostent_dbo, &found)) {
    zkShellWrite_va(err,shell,
		    "Error: problem locating host entity [%s]\n",
		    hostentityname);
    goto finish;
  }

  if ( ! found ) {
    zkShellWrite_va(err,shell,
    "Error: Could not find host entity [%s] in the database\n",
		    hostentityname);
    goto finish;
  }

  /* Setup */
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &node_entdbo, &psz))
    goto finish;

  if ( ! frEntSetFromStr2(err, &node_entdbo->ent, type, nodename)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, node_entdbo, &eid, &found))
    goto finish;

  if ( found ) {
    zkShellWrite_va(err,shell,
		    "Error: [%s/%s] already exists in  the database \n",
		    type, nodename);
    goto finish;
  }

  if ( ! createEntityIdent(err, shell, c, node_entdbo, &eid, &newflag)) 
    goto finish;

  if ( ! newflag) {
    zkShellWrite_va(err,shell,
     "Error: [%s] already exists in the database. This should not be.\n",
		    node_entdbo->ent.name);
  }

  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNTDB_NODE, (void **) &node, &psz))
    goto finish;

  node->node_entid = node_entdbo->entid;
  node->node_port  = atoi(port);
  node->num_hosts  = 1;
  node->host_entid = hostent_dbo->entid;

  if (! zkDBOPack(err, node, sizeof(struct T_FrDBOTypeNtdbNode), &pnode, &pnsz))
    goto finish;
  
  if (!zkDBTNew (err, &key) || !zkDBTNew(err, &data))
    goto finish;
  
  if (!zkDBSetKeyOrData (err, key, &node_entdbo->entid, sizeof(node_entdbo->entid)) ||
      !zkDBSetKeyOrData (err, data, pnode, pnsz))
    goto finish;

  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }
  
  if (!zkDBStore (err, c->nidb->topoDB, key, data, &found)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem storing entity. Not stored.\n");
    goto finish;
  } else if( found ) {
    zkShellWrite_va( err, shell, "Entity not stored. The key already exists.\n" );
    goto finish;
  } else {
      zkShellWrite_va(err,shell,"Entity stored.\n");
      retcode = 1;
  }

  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, node_entdbo->entid, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  } 
  
 finish:
  if (node_entdbo) free (node_entdbo);
  if (hostent_dbo) free (hostent_dbo);
  if (key) zkDBTFree (err, &key);
  if (data) zkDBTFree (err, &data);
  if (host) free (host);
  if (node) free (node);
  return retcode;
}

int cmdEntityAddHost (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostname,
  const char *ip_addr,
  const char *region,
  const char *servoperator,
  const char *policy
  )
{
  static const char *funcname = "cmdEntityAddHost";
  FrDBOTypeNtdbHost host = NULL;
  FrDBOTypeEntityID ent_dbo = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int exists = 0, retcode = 0, newflag = 0, found = 0;
  uint32_t eid = 0;
  void *phost = NULL;
  size_t psz = 0, phsz = 0;
  char *p;
  (void) funcname;

  if ( !c || !type || !entname || !hostname || !ip_addr || !region ||
       !servoperator || !policy ) {
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);
    goto finish;
  }
  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &ent_dbo, &psz))
    goto finish;

  if (!frEntSetFromStr2(err, &ent_dbo->ent, type, entname)) 
    goto finish;

  if (!createEntityIdent (err, shell, c, ent_dbo, &eid, &newflag)) 
    goto finish;

  if (!newflag) {
    zkShellWrite_va(err,shell,
		    "Error: Host [%s] is already in the database\n", entname);
    goto finish;
  }

  /* Now we take care of the host entity */
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Host entity identifier (ie: key) is [%lu]\n", eid);

  psz = 0;
  if (!zkDBONew(err, ZKDBO_REG_FRNTDB_HOST, (void **) &host, &psz))
    goto finish;

#warning Consolidate this Host Info 990819 (luc)
  host->host_entid   = ent_dbo->entid;
  host->policy = atoi(policy);
  host->num_ipaddrs = 1;
  host->ipaddr_nbo  = inet_addr(ip_addr);
  strncpy (host->hostname, hostname, 64);
  strncpy (host->region, region, 64);
  strncpy (host->servoperator, servoperator, 64);

  /* Cleanup the underscores */
  
  for (p = host->servoperator ; *p != '\0' ; p++ ) {
    if (*p == '_') {
      *p = ' ';
    }
  }
    
  fprintf (stderr, "\n[%s]\n", host->servoperator);
                    
  if (! zkDBOPack(err, host, sizeof(struct T_FrDBOTypeNtdbHost), &phost, &phsz))
    goto finish;

  if (!zkDBTNew (err, &key) || !zkDBTNew(err, &data))
    goto finish;
		 
  if (!zkDBSetKeyOrData (err, key, &ent_dbo->entid, sizeof(ent_dbo->entid)) ||
      !zkDBSetKeyOrData (err, data, phost, phsz))
    goto finish;

  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }
  
  if (!zkDBStore (err, c->nidb->hostDB, key, data, &exists)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem storing entity. Not stored.\n");
    goto finish;
  }
  if (exists) {
    zkShellWrite_va(err,shell,"Error: Entity already exists.\n");
    goto finish;
  } else {
    zkShellWrite_va(err,shell,"Entity stored.\n");
    retcode = 1;
  }

  if ( ! frNidbSetEntityLastUpdateTS ( err, c->nidb, ent_dbo->entid, &found) ) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with last_update_TS: %s\n", zkErrorDetail(err));
    goto finish;
  } else if ( ! found ) {
    zkShellWrite_va(NULL, shell,
                    "Note: New last_update_TS set: \n", zkErrorDetail(err));
  } 
  
 finish:
  if (ent_dbo) free (ent_dbo);
  if (key) free (key);
  if (data) free (data);
  if (host) free (host);
  return retcode;
}
