/*
 * Entity Command processor for the Add command
 *
 * $Id: nidbtoolCEList.c,v 1.6 2000/11/24 15:03:09 marcp Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCEList.c,v 1.6 2000/11/24 15:03:09 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"

#define FREEIT(x) if (x) { free (x); x = NULL; }

int cmdEntityListAll (ZkError err, ZkShell shell, NidbToolContext *c)
{
  static const char *funcname = "cmdEntityListAll";
  FrNidbEntityID ent_list = NULL, ep = NULL;
  size_t num = 0, i = 0;
  
  (void) funcname;
  (void) module_id_;

  if ( ! frNidbGetAllEntityIDs (err, c->nidb, &ent_list, &num))
    return 0;

  for (i = 0, ep = ent_list; i < num; ep++, i++) {
    zkShellWrite_va(err, shell, "EntityID: %u\n", *ep);
  }
  return 1;
}


/**
 * Using the same interface that the remote client uses, get a list
 * of all User defined entities.
 */
int cmdEntityListUser (ZkError err, ZkShell shell, NidbToolContext *c)
{
  static const char *funcname = "cmdEntityListUser";
  FrDBOTypeEntityID rlist = NULL;
  FrDBOTypeEntityID eidp = NULL;
  size_t i = 0, rsz = 0;
  int found = 0;

  (void) funcname;

  if ( ! frNtdbQueryListAllEntityIDs (err, c->nidb, (void **) &rlist, &rsz,
                                     &found))
    return 0;

  if ( ! found ) {
    zkShellWrite_va(err, shell, "Error: Nothing found in the NIDB\n");
    return 0;
  }
  
  zkShellWrite_va(err, shell, "NIDB matches: %u\n", rsz);

  for (i = 0, eidp = rlist; i < rsz; eidp++, i++) {
    zkShellWrite_va(err, shell, " [%2u] %s/%d \n", (*eidp).entid,
                    (*eidp).ent.name, (*eidp).ent.type);
  }
  FREEIT(rlist);
  return 1;
}
