/*
 * Entity Command processor for the Set command
 *
 * $Id: nidbtoolCESet.c,v 1.10 2001/04/19 19:22:54 alainc Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCESet.c,v 1.10 2001/04/19 19:22:54 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"

static int nidbtoolGetStateUsingEntity (
  ZkError err ,
  NidbContext c,
  FrDBOTypeEntityID eid_dbo,
  FrDBOTypeNsdbAip *state,
  size_t *ssz,
  int *found)
{
  static const char *funcname = "frNsdbGetStateUsingEntity";
  int return_code = 0;
  uint32_t eid = 0;
  size_t psz = 0;

  zkErrorClear(err);
  if (! c || !state || !eid_dbo) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
  }

  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNSDBAIP, (void **) state, ssz))
    goto finish;

  /* Get the entity identifier (ie: the key) for this entity */
  if ( ! frNidbGetEntityIdentifier (err, c, eid_dbo, &eid, found)) {
    goto finish;
  }
  
  if ( ! *found ) {
    zkLogWrite_va(c->log, "Failed to find entity %s/%d in entity database",
		  eid_dbo->ent.name, eid_dbo->ent.type);
    return_code = 1;
    goto finish ; 
  }
  zkDebug ("Entity:  %s (%d)\n",eid_dbo->ent.name, eid_dbo->ent.type);
  
  (*state)->entid = eid;
  
  if ( ! frNsdbFetch (err, c, *state, found)) {
    zkLogWrite_va(c->log,
		  "Error trying to fetch entity %s/%d state from state database",
		  eid_dbo->ent.name, eid_dbo->ent.type);
    goto finish;
  }
  
  if ( ! *found ) {
    zkLogWrite_va(c->log, "Failed to find entity %s/%d state in state database",
		  eid_dbo->ent.name, eid_dbo->ent.type);
  }
  return_code = 1;
  
 finish:
  return return_code;

}

int cmdEntitySetAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *avail_flag)
{
  static const char *funcname = "cmdEntitySetAip";
  FrDBOTypeEntityID ent_dbo = NULL;
  FrDBOTypeNsdbAip state = NULL;
  size_t esz = 0, ssz = 0;
  int found = 0, replaced = 0, retcode = 0;
  int status_changed = 1;

  /* Used to verify that the aip does exist in the database */
  int isFound = 0;
  FrDBOTypeNtdbAip  aip = NULL;
  FrDBOTypeEntityID entDBO = NULL;
  FrEntIdent eid = 0;

  (void) funcname;
  (void) module_id_;

  if ( !c || !type || !entname || !avail_flag ) {
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);
  } else if ( ((strcasecmp(type,"aip")!=0) && (strcasecmp(type,"coreaip")!=0)) ||
             ((strcasecmp (avail_flag, "available" ) != 0) &&
              (strcasecmp (avail_flag, "unavailable" ) != 0))) {
    zkShellWrite_va(err,shell,
                    "Usage: entity set [aip|coreaip] aip-name [available|unavailable]\n");
  } else if( !cmdEntityGetAipObjs(err,shell,c,type,entname,&aip,&entDBO,&eid,&isFound) || !isFound ) {
    zkShellWrite_va(err,shell,
                    "AIP Entity '%s' was not found in the database.\n", entname );
  } else {
      
    if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &ent_dbo, &esz)) {
      zkShellWrite_va(err,shell,
                      "Error: Internal memory allocation.\n");
      
    } else  if (! frEntSetFromStr2(err, &ent_dbo->ent, type, entname)) {
      zkShellWrite_va(err,shell,
                      "Error: Internal memory copy.\n");
    } else if (! nidbtoolGetStateUsingEntity (err, c->nidb, ent_dbo,
                                              &state, &ssz, &found)) {
      zkShellWrite_va(err,shell,
                      "Error: Error getting the state of this entity.\n");
      retcode = 0;
    } /* No else here */
    
    if ( ! found ) {
      zkShellWrite_va(err,shell,
                      "Entity not in state database - New entry.\n");
    }
    
    /* bug: need to be a bit better here */
    if (strcasecmp (avail_flag, "available")  == 0) {
      state->unavailable_flag = 0;
    } else {
      state->unavailable_flag = 1;
    }
    
    if ( ! frNsdbReplace (err, c->nidb, state, status_changed, &replaced )) {
      retcode = 0;
    } else if ( ! replaced ) {
      zkShellWrite_va(err,shell,
                      "Entity not in state database - New entry.\n");
    }
    retcode = 1;
  }
  
  FREEIT(ent_dbo);
  FREEIT(state);
  FREEIT(aip);
  FREEIT(entDBO);
  return retcode;
}

int cmdEntitySetMaip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *avail_flag)
{
  static const char *funcname = "cmdEntitySetMaip";
  FrDBOTypeEntityID ent_dbo = NULL;
  FrDBOTypeNsdbAip state = NULL;
  size_t esz = 0, ssz = 0;
  int found = 0, replaced = 0, retcode = 0;
  int status_changed = 0;
  
  (void) funcname;
  (void) module_id_;

  if ( !c || !type || !entname || !avail_flag ) {
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);
  } else if ( (strcasecmp (type, "maip" ) != 0) ||
             ((strcasecmp (avail_flag, "available" ) != 0) &&
              (strcasecmp (avail_flag, "unavailable" ) != 0))) {
    zkShellWrite_va(err,shell,
                    "Usage: entity set aip aip-name [available|unavailable\n");
  } else {
      
    if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &ent_dbo, &esz)) {
      zkShellWrite_va(err,shell,
                      "Error: Internal memory allocation.\n");
      
    } else  if (! frEntSetFromStr2(err, &ent_dbo->ent, type, entname)) {
      zkShellWrite_va(err,shell,
                      "Error: Internal memory copy.\n");

    } else if (! nidbtoolGetStateUsingEntity (err, c->nidb, ent_dbo,
                                              &state, &ssz, &found)) {
      zkShellWrite_va(err,shell,
                      "Error: Error getting the state of this entity.\n");
      retcode = 0;
    } /* No else here */
    
    if ( ! found ) {
      zkShellWrite_va(err,shell,
                      "Entity not in state database - New entry.\n");
    }
    
    /* bug: need to be a bit better here */
    if (strcasecmp (avail_flag, "available")  == 0) {
      if (state->unavailable_flag == 1) {
        status_changed = 1;
        zkDebug ("Status is changing");
      }
      state->unavailable_flag = 0;
      state->up_flag = 1;
      state->entity_rating = 1;
      zkShellWrite_va(err,shell,
                      "WARNING: up_flag forced UP. rating forced to 1.\n");
    } else {
      if (state->unavailable_flag != 1) {
        status_changed = 1;
        zkDebug ("Status is changing");
      }
      state->unavailable_flag = 1;
      state->up_flag = 0;
      state->entity_rating = 0;
      zkShellWrite_va(err,shell,
                      "WARNING: up_flag forced DOWN. rating forced to 0.\n");
    }

    if ( ! frNsdbReplace (err, c->nidb, state, status_changed, &replaced )) {
      retcode = 0;
    } else if ( ! replaced ) {
      zkShellWrite_va(err,shell,
                      "Entity not in state database - New entry.\n");
    }
    retcode = 1;
  }
  
  FREEIT(ent_dbo);
  FREEIT(state);
  return retcode;
}


int cmdEntitySetLink (
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *node1,
  ZK_IN const char *node2,
  ZK_IN const char *ts_delay,
  ZK_IN const char *ts_pkts,
  ZK_IN const char *ct_on_flag)
{
  static const char *funcname = "cmdEntitySetAip";
  FrDBOTypeNtdbLink lnk = NULL, oldlnk = NULL;
  FrDBOTypeEntityID ent_dbo = NULL;
  FrEntIdent l_eid = 0;
  size_t lsz = 0;
  int found = 0, retcode = 0;
  
  if ( !c || !type || !node1 || !node2 || !ts_delay ||
       !ts_pkts || !ct_on_flag ) {
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);

  } else if ( (strcasecmp (type, "link" ) != 0)) {
    zkShellWrite_va(err,shell,
                    "Usage: entity set link aip-name1 aip-name2 ts-delay "
                    "ts_pkts [on|off]\n");

  } else if ( ! zkDBONew(err, ZKDBO_REG_FRNTDBLINK, (void **) &lnk, &lsz)) {
    zkShellWrite_va(NULL,shell,
                    "Error: creating LINK DBO [%s]\n", zkErrorDetail(err));

  } else if ( ! cmdEntityGetLinkObjs (err, shell, c, type, node1, node2,
                                      &oldlnk, &ent_dbo, &l_eid, &found) ) {
    zkShellWrite_va(NULL,shell,
                    "Error: fetching LINK DBOs [%s]\n", zkErrorDetail(err));

  } else if ( ! found) {
    zkShellWrite_va (err, shell, "Error: Link not in database\n");

    
  } else if ( ! cmdEntityParseLinkParams (err, shell, c, ts_delay,
                                         ts_pkts, ct_on_flag, lnk) ) {
    zkShellWrite_va(NULL, shell, "Error: with parameters\n", zkErrorDetail(err));


  } else {
    /* All paramters check, previous records found. Check to see if
       and update is required */
    found = 0;
    if ((oldlnk->ts_delay == lnk->ts_delay) &&
        (oldlnk->ts_pkts == lnk->ts_pkts) &&
        (oldlnk->ct_on_flag == lnk->ct_on_flag)) {
      zkShellWrite_va(err, shell, "No changes to the link parameters\n");
      retcode = 1;
      
    } else {
      /* Bug: Should be using a method to do this */
#warning Bug: Method required to duplicate the object      
      lnk->link_entid = l_eid;
      lnk->node1_entid = oldlnk->node1_entid;
      lnk->node2_entid = oldlnk->node2_entid;
      lnk->spare1 = oldlnk->spare1;
      lnk->spare2 = oldlnk->spare2;
      if (! frNtdbReplaceLink (err, c->nidb, lnk, &found)) {
        zkShellWrite_va(NULL, shell, "Error: Getting record for link [%s]\n", 
                        zkErrorDetail(err));
      } else if (! found ) {
        zkShellWrite_va(err, shell,
                "Error: Unable to find old record to replace link %s to %s\n",
                       node1, node2);
      } else {
        retcode = 1;
      }
    }
  }
  FREEIT(ent_dbo);
  FREEIT(oldlnk);
  FREEIT(lnk);
  return retcode;
}

/* entity add host zksbeta5 zksbeta5.aip.freedom.net 209.5.124.69 CA10 ZKS 1 */

#warning Fix for B4
/*#ifdef NOT_READY_YET*/
/* This routine needs work. If the IP addess changes, then
   a lot of stuff has to happen. Next Release */
int cmdEntitySetHost (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostname,
  const char *ip_addr,
  const char *region,
  const char *servoperator,
  const char *policy )
{
  static const char *funcname = "cmdEntitySetHost";
  FrDBOTypeNtdbHost host = NULL;
  FrDBOTypeEntityID ent_dbo = NULL;
  FrEntIdent eid = 0;
  /* size_t esz = 0, ssz = 0;*/
  int found = 0, replaced = 0, retcode = 0;
  /* int status_changed = 1; */
  
  (void) funcname;
  (void) module_id_;

  if ( !c || !type || !entname ){
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);
  } else{
    if ( policy || hostname ) {
      if ( policy )
        zkShellWrite_va(err,shell,
                        "Warning: cannot change the policy for now\n");
      if ( hostname )
        zkShellWrite_va(err,shell,
                        "Warning: cannot change the hostname for now\n");
    } else {
/*
      if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &ent_dbo, &esz)) {
        zkShellWrite_va(err,shell,
                        "Error: Internal memory allocation.\n");
      
      } else  if (! frEntSetFromStr2(err, &ent_dbo->ent, type, entname)) {
        zkShellWrite_va(err,shell,
                        "Error: Internal memory copy.\n");


      } else
*/
      if ( !cmdEntityGetHostObjs(err, shell, c, type, entname,
                                        & host, & ent_dbo, & eid, & found) ) {
        zkShellWrite_va(NULL,shell,
                        "Error: fetching Host DBOs [%s]\n", zkErrorDetail(err));

      } else if ( ! found) {
      
        zkShellWrite_va (err, shell, "Error: Host not in database\n");

      } else {
        char *p = NULL;
#warning Consolidate this Host Info 990819 (luc)

        /* get a copy of the information */
      
/* original code here
      host->host_entid   = ent_dbo->entid;
      host->policy = atoi(policy);
      host->num_ipaddrs = 1;
      host->ipaddr_nbo  = inet_addr(ip_addr);
*/
      
      /* put the new region if there */
        if( region )
          strncpy (host->region, region, 64);

      /* put the new servoperator if there */
        if(servoperator)
          strncpy (host->servoperator, servoperator, 64);

      /* put the new ip address in there */
        if( ip_addr )
          host->ipaddr_nbo  = inet_addr(ip_addr);

      /* not for now 
       strncpy (host->hostname, hostname, 64);
      */
      
      /* Cleanup the underscores */
      
        for (p = host->servoperator ; *p != '\0' ; p++ ) {
          if (*p == '_') {
            *p = ' ';
          }
        }
        zkShellWrite_va (err, shell, "servoperator [%s]\n", host->servoperator);
        if ( ! frNtdbReplaceHost (err, c->nidb, host, &replaced)) {
          zkShellWrite_va(NULL,shell,
                          "Error: Replacing Host DBO [%s]\n", zkErrorDetail(err));
        } else if ( ! replaced ) {
          zkShellWrite_va(err,shell,
                          "Entity not in host database - New entry.\n");
        }
        retcode = 1;
      }
    }
  }
  
  FREEIT(ent_dbo);
  FREEIT(host);
/*  FREEIT(state); */
  return retcode;
}
/*#endif*/
