/*
 * Entity Command processor for the show command
 *
 * $Id: nidbtoolCEShow.c,v 1.8 2000/11/24 15:03:09 marcp Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCEShow.c,v 1.8 2000/11/24 15:03:09 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"

int printAipDBO (
  ZkError err,
  ZkShell shell,
  FrDBOTypeNtdbAip aip,
  const char *type,
  const char *ent)
{
  (void) module_id_;

  if ( ! aip) {
    return 0;
  }

  zkShellWrite_va(err,shell,
		  "entity %s %s %u %u %u %u %u %u %u\n",
		  type,
 		  ent,
                  aip->aip_entid,
                  aip->host_entid,
		  aip->aip_port,
		  aip->dh_port,
		  aip->dh_timeout,
		  aip->dh_negotiation,
		  aip->protocol_vers);
  return 1;
}

static int printHostDBO (
    ZkError            err,
    ZkShell            shell,
    FrDBOTypeNtdbHost  host,
    const char *       type,
    const char *       ent
)
{
    struct in_addr hostAddr;

    (void) module_id_;
    
    if ( ! host) {
	return 0;
    }

    hostAddr.s_addr = host->ipaddr_nbo;

    zkShellWrite_va(err,shell,
		    "entity %s %s %u %u %s %s %s %s\n",
		    type,
		    ent,
		    host->host_entid,
		    host->num_ipaddrs,
		    inet_ntoa(hostAddr),
		    host->hostname,
		    host->region,
		    host->servoperator);
    return 1;
}

static int printNodeDBO (
  ZkError err,
  ZkShell shell,
  FrDBOTypeNtdbNode node,
  const char *type,
  const char *ent)
{
  (void) module_id_;

  if ( ! node) {
    return 0;
  }

  zkShellWrite_va(err,shell,
		  "entity  %s %s %u %u %u\n",
                  type,
		  ent,
                  node->node_entid,
                  node->host_entid,
		  node->node_port);
  return 1;
}

int cmdEntityShowLink (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *node1,
  const char *node2)
{
  FrDBOTypeEntityID entid = NULL;
  FrDBOTypeNtdbLink lnk = NULL;
  char *link_entname = NULL;
  uint32_t eid_l = 0, psz = 0;
  int found = 0, retcode = 0;
  
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &psz))
    goto finish;
  psz = 0;

  if (!getLinkEntityName (err, shell, c, node1, node2, &link_entname))
    goto finish;

  if ( ! frEntSetFromStr2(err, &entid->ent, "link", link_entname)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid_l, &found))
    goto finish;

  if (! found) {
    zkShellWrite_va(err,shell,
		    "Error: No link exists.\n");
    return 1;
    goto finish;
  }
  
  /* Bug: Slight hack. I've found that the record exists. For now, I'll
     just print it and do a full check later */

  zkShellWrite_va(err, shell, "Link confirmed.\n", node1, node2);
  zkShellWrite_va(err, shell, "entity link %s %s\n", node1, node2);
  
  retcode = 1;
  
 finish:
  if (link_entname) free (link_entname);
  if (entid) free (entid);
  if (lnk) free (lnk);
  return retcode;
}

int
cmdEntityShowAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *aipname)
{
  static const char *funcname = "cmdEntityShowAip";
  FrDBOTypeNtdbAip aip = NULL;
  FrDBOTypeEntityID entid = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  uint32_t eid = 0;
  size_t aipsz = 0, esz = 0;
  int found = 0, retcode = 0;

  (void) funcname;
  
  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &esz))
    goto finish;

  if (!frEntSetFromStr2(err, &entid->ent, type, aipname)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Aip [%s] is not in the database\n", aipname);
    retcode = 1 ;
    goto finish;
  }

  if ( ! zkDBTNew (err, &key) || ! zkDBTNew(err, &data))
    goto finish;

  if ( ! zkDBSetKeyOrData (err, key, &eid, sizeof(eid)))
    goto finish;
  
  
  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }

  if ( ! zkDBFetch (err, c->nidb->topoDB, key, data, &found)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem retreiving entity.\n");
    goto finish;
  }
  if (!found) {
    zkShellWrite_va(err,shell, "Aip [%s] not found.\n", aipname);
    retcode = 1;
    goto finish;
  }
  
  if (! zkDBOUnpack(err, (void **) &aip, &aipsz, data->data, data->size)) {
    zkShellWrite_va(err,shell, "Stored data for [%s] is corrupted.\n", aipname);
    goto finish;
  }

  zkShellWrite_va(err,shell, "Data retrieved for [%s].\n", aipname);
  printAipDBO(err, shell, aip, type, aipname);
  retcode = 1;
  
 finish:
  if (entid) free (entid);
  if (aip) free (aip);
  if (key) free (key);
  if (data) free (data);
  return retcode;
}

int
cmdEntityShowNeighbour (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node)
{
  static const char *funcname = "cmdEntityShow";
  FrDBOTypeEntityID entid = NULL;
  FrDBOTypeNtdbDetailedNeighbours dNbrs = NULL; /* The response */
  uint32_t eid = 0;
  void * resp = NULL;
  size_t esz = 0, nsz = 0, rsz = 0, i;
  int found = 0, retcode = 0;

  (void) funcname;
  
  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &esz))
    goto finish;

  if (!frEntSetFromStr2(err, &entid->ent, type, node)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Aip [%s] is not in the database\n", node);
    retcode = 1 ;
    goto finish;
  }

  if ( ! frNtdbGetCompleteLocalTopology(err, c->nidb, entid, (void **)&resp,
					&rsz, &found) || !found)
    goto finish;

  if ( ! zkDBOUnpack (err, (void **)&dNbrs, &nsz, resp, rsz))
    goto finish;
  
  zkShellWrite_va (err, shell, "Neigbour list: \n");
  for (i = 0; i < dNbrs->num_neighbours; i++) {
    zkShellWrite_va (err, shell, " #%02d - [%s]\n", i,
		     dNbrs->neighbour[i].ent.name);
  }
  retcode = 1;
  
 finish:
  if (dNbrs) free (dNbrs);
  if (resp) free (resp);
  return retcode;
}


int
cmdEntityShowHost(
    ZkError           err,
    ZkShell           shell,
    NidbToolContext * c,
    const char *      type,
    const char *      hostname
)
{
    static const char *funcname = "cmdEntityShowHost";
    FrDBOTypeNtdbHost host = NULL;
    FrDBOTypeEntityID entid = NULL;
    ZkDBT key = NULL;
    ZkDBT data = NULL;
    uint32_t eid = 0;
    size_t hostsz = 0, esz = 0;
    int found = 0, retcode = 0;

    (void) funcname;


    if (! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &esz)  ||
	! frEntSetFromStr2(err, &entid->ent, type, hostname)           ||
	! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid, &found)  ) {
	/* error */
    }
    else if ( ! found ) {
	zkShellWrite_va(err,shell,
			"Error: Host [%s] is not in the database\n", hostname);
	retcode = 1 ;
    }
    else if ( ! zkDBTNew (err, &key) || ! zkDBTNew(err, &data) ||
	      ! zkDBSetKeyOrData (err, key, &eid, sizeof(eid))) {
	/* error */
    } else {

	/* debug info */
	if (zkDebugIsOn()) {
	    zkDebug("Value of key:");
	    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
	}

	/* get entry from host DB */
	if ( ! zkDBFetch (err, c->nidb->hostDB, key, data, &found)) {
	    zkShellWrite_va(err,shell,
			    "Error: Serious problem retreiving entity.\n");
	}
	else if (!found) {
	    zkShellWrite_va(err,shell, "Host [%s] not found.\n", hostname);
	    retcode = 1;
	}
	else if (! zkDBOUnpack(err, (void **) &host, &hostsz, data->data, data->size)) {
	    zkShellWrite_va(err,shell, "Stored data for [%s] is corrupted.\n", hostname);
	} else { 
	    zkShellWrite_va(err,shell, "Data retrieved for [%s].\n", hostname);
	    printHostDBO(err, shell, host, type, hostname);
	    retcode = 1;
	}
    }

    /* memory management */
    if (entid) free (entid);
    if (host) free (host);
    if (key) free (key);
    if (data) free (data);

    return retcode;
}


int
cmdEntityShowNode (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *nodename)
{
  static const char *funcname = "cmdEntityShowNode";
  FrDBOTypeNtdbNode node = NULL;
  FrDBOTypeEntityID entid = NULL;
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  uint32_t eid = 0;
  size_t nodesz = 0, esz = 0;
  int found = 0, retcode = 0;

  (void) funcname;
  
  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &esz))
    goto finish;

  if (!frEntSetFromStr2(err, &entid->ent, type, nodename)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Node [%s] is not in the database\n", nodename);
    retcode = 1 ;
    goto finish;
  }

  if ( ! zkDBTNew (err, &key) || ! zkDBTNew(err, &data))
    goto finish;

  if ( ! zkDBSetKeyOrData (err, key, &eid, sizeof(eid)))
    goto finish;
  
  
  if (zkDebugIsOn()) {
    zkDebug("Value of key:");
    zkDumpBuffer(err,key->data,key->size,zkDumpToStream,stderr);
  }

  if ( ! zkDBFetch (err, c->nidb->topoDB, key, data, &found)) {
    zkShellWrite_va(err,shell,
		    "Error: Serious problem retreiving entity.\n");
    goto finish;
  }
  if (!found) {
    zkShellWrite_va(err,shell, "Node [%s] not found.\n", nodename);
    retcode = 1;
    goto finish;
  }
  
  if (! zkDBOUnpack(err, (void **) &node, &nodesz, data->data, data->size)) {
    zkShellWrite_va(err,shell, "Stored data for [%s] is corrupted.\n", nodename);
    goto finish;
  }

  zkShellWrite_va(err,shell, "Data retrieved for [%s].\n", nodename);
  printNodeDBO(err, shell, node, type, nodename);
  retcode = 1;
  
 finish:
  if (entid) free (entid);
  if (node) free (node);
  if (key) free (key);
  if (data) free (data);
  return retcode;
}

