/*
 * Entity Command processor for the state command
 *
 * $Id: nidbtoolCEState.c,v 1.6 2000/11/24 15:03:09 marcp Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCEState.c,v 1.6 2000/11/24 15:03:09 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"
  

int
cmdEntityState (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node)
{
  static const char *funcname = "cmdEntityShow";
  FrDBOTypeNsdbAip state = NULL;
  FrDBOTypeEntityID entid = NULL;
  uint32_t eid = 0;
  size_t esz = 0, ssz = 0;
  int found = 0, retcode = 0;
  char *q = NULL;
  
  (void) funcname;
  (void) module_id_;
  
  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &esz))
    goto finish;

  if (!frEntSetFromStr2(err, &entid->ent, type, node)) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Entity [%s] is not in the database\n", node);
    retcode = 1 ;
    goto finish;
  }

  if (!zkDBONew(err, ZKDBO_REG_FRNSDBAIP, (void **) &state, &ssz))
    goto finish;

  state->entid = eid;

  if ( ! frNsdbFetch (err, c->nidb, state, &found) ) {
    zkShellWrite_va(err,shell,
		    "Error: [%s] ", zkErrorDetail(err));
    zkShellWrite_va(err,shell,
		    "while fetching [%s] \n", node);
    goto finish;
    retcode = 1;
  }    

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: No state found for entity [%s] \n", node);
    retcode = 1;
    goto finish;
  }

  if ( ! frNidbDumpStateBuf (err, state, NULL, & q)) {
    zkShellWrite_va(err,shell,
		    "Error: Unable to dump state for node %s: %s \n",
                    node, zkErrorDetail(err));
    retcode = 1;
    goto finish;
  }
     
  zkShellWrite_va (err, shell, q);;
  retcode = 1;
  
 finish:
  if (q) free (q);
  if (entid) free (entid);
  if (state) free (state);
  return retcode;
}
