/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/************************************************************************

   File:		nidbtoolCEUpdate.c

   Author(s):		Thierry Baron

   Created:		21 Oct 1999

   Last Revision:	$Date: 2001/04/19 19:22:54 $

   Description:	

   $Revision: 1.3 $

   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
   Please see the copyright notice included in this distribution 
   for full details.

 ***********************************************************************/

static const char * module_id_ = "$Id: nidbtoolCEUpdate.c,v 1.3 2001/04/19 19:22:54 alainc Exp $";

/**********************
     Include Files
 **********************/
#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>

#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"

#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"

/**********************
    Main Functions
 **********************/

int
cmdEntityUpdate(
    ZkError           err,
    ZkShell           shell,
    NidbToolContext * context,
    const char *      type,
    const char *      entname,
    const char *      whichField,
    const char *      newVal
){
  static const char *funcname = "cmdEntityUpdate";
  int retcode = 0;
  int rc = 0;
  (void) funcname;
  (void) module_id_;

  if(!context || !type || !entname || !whichField || !newVal || strcmp(type, "host")){
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);    
  } else {
    retcode = 1;
    if(!strcmp( whichField,"servoperator")){
      rc = cmdEntitySetHost(err, shell, context, type, entname, NULL, NULL, NULL, newVal, NULL);
      zkShellWrite_va(err,shell, " new servoperator will be [%s]\n", newVal);
    }else if(!strcmp( whichField, "region")){
      rc = cmdEntitySetHost(err, shell, context, type, entname, NULL, NULL, newVal, NULL, NULL);
      zkShellWrite_va(err,shell, " new region will be [%s]\n", newVal);
    } else if( !strcmp( whichField, "ip") ){
      rc = cmdEntitySetHost(err, shell, context, type, entname, NULL, newVal, NULL, NULL, NULL );
      zkShellWrite_va(err,shell, " new ip will be [%s]\n", newVal);
    } else {
      zkShellWrite_va(err,shell,
		    "Error: Invalid field parameter passed to %s.\n", funcname);    
      retcode = 0;
    }
  }
  return retcode;
}

/***************************************************************************************

   $Log: nidbtoolCEUpdate.c,v $
   Revision 1.3  2001/04/19 19:22:54  alainc
   Merge from INT_FREEZE_MAC2_1_INT_20010417 to main trunk

   Revision 1.2.2.1  2001/01/25 01:03:16  paull
   bringing in changes between tags BASE_SCARECROW_BETA2_0 and INT_SCARECROW_DEV_20010124_MAC2_1_INT to the mac2_1_int branch.

   Revision 1.1.28.2  2001/01/17 17:29:26  danp
   Bug 8076: extra features to the entity update command in nidbtool
   The cmdEntityUpdate() function used to support 2 fields for an update: region and servoperator. It has been modified to support "ip" as well. Code Reviewed by Louis Beaumier on January 17th, 2001.

   Revision 1.1.28.1  2000/10/11 13:19:51  phil

   License header additions in preparation for the source release.

   Revision 1.1  1999/10/22 23:28:05  thierry
   add region server operator support


*/
