/*
 * Entity Command processor
 *
 * $Id: nidbtoolCmdEntity.c,v 1.39 2001/04/19 19:22:54 alainc Exp $
 * 
 */

static const char * module_id_ = "$Id: nidbtoolCmdEntity.c,v 1.39 2001/04/19 19:22:54 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>
 */


#include <zks.h>

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <errno.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkShell.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"
#include "nidbtool.h"
#include "nidbtoolCmdEntity.h"
#include "zkCStr.h"
#include "nidbtoolCmdCache.h"
#include "frNidbObj.h"

/* Private function used for reformating and validating the region string (HOST DB) */
static int reformatRegionString( ZkError error, ZkShell shell, char *userString, int userStringSize );

/* Private function used for reformating a link end name */
static int reformatLinkEnd( ZkError error, ZkShell shell, char *linkEnd, int *pIsAip );

DLLExport int
cmdShowContext (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{
  NidbToolContext * context = (NidbToolContext *) userdata;

  (void) shell;
  (void) token;
  (void) tokens;

  nidbToolContextDump(err,context,stdout);

  return ZKSHELL_SESSION_CONTINUE;
}

#define CMD_ENT_ADD       "add"
#define CMD_ENT_DELETE    "delete"
#define CMD_ENT_DESCRIBE  "desc"
#define CMD_ENT_SET       "set"
#define CMD_ENT_SHOW      "show"
#define CMD_ENT_STATE     "state"
#define CMD_ENT_LIST      "list"
#define CMD_ENT_NEIGH     "neighbour"
#define CMD_ENT_UPDATE    "update"

#define CDV_NO_INCR_OPTION "--cdv_noincr"

int canonLinkEntname (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  struct link_entname *entname)
{
  static const char *funcname = "canonLinkEntname";
  uint32_t x;
  (void) err;
  (void) shell;
  (void) c;

  if ( !entname ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
    
  if (entname->node1 > entname->node2) {
    x = entname->node2;
    entname->node2 = entname->node1;
    entname->node1 = x;
  }
  return 1;
}


#if 0
/*
 * If found returns the entity ID of an entity
 * and sets the found flag
 */
static int getEntityIdent (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  FrDBOTypeEntityID entid,
  uint32_t *eid,
  int *found)
{
  static const char *funcname = "getEntityIdent";
  void *p = NULL;
  size_t psz = 0;
  FrDBOTypeEntityID t_e;
  (void) shell;
  (void) funcname;
  
  *eid = 0;
  *found = 0;

  /* Is the Entity already there? */
  if (!zkAAGet(err, c->nidb->entDB, &entid->ent, sizeof(FrEntIdExp),
	       &p, &psz))
    return 0;

  if (zkDebugIsOn() && psz) {
    zkDebug("Returned entity:");
    zkDumpBuffer(err,p,psz,zkDumpToStream,stderr);
  }

  if (psz) {
    t_e =  (FrDBOTypeEntityID) p;
    entid->entid = t_e->entid;
    *eid = t_e->entid;
    *found = 1;
    return 1;
  }
  
  return 1;  
}    
#endif

#if 0
static int printLinkDBO (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  FrDBOTypeNtdbLink lnk)
{
  (void) c;
  if ( !lnk ) {
    return 0;
  }
  zkShellWrite_va(err,shell, "entity link aip %d aip %d\n",
		  lnk->node1_entid,
		  lnk->node2_entid);
  return 1;
}
#endif

/*
 * Returns a valid Entity identifier, filling the entid type
 * and values
 * Also returns a newflag if this was a new entity
 */
int createEntityIdent (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  FrDBOTypeEntityID entid,
  uint32_t *eid,
  int *newflag)
{
  static const char *funcname = "getEntityIdent";
  void *p = NULL;
  size_t psz = 0;
  int found = 0;
  uint32_t CURRENT_ENTID_TAG = 0;
  (void) funcname;
  (void) shell;

  if ( !err || !c || !eid || !newflag) {
        zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
        return 0;
  }
  
  *eid = 0;
  *newflag = 1;

  if (!frNidbGetEntityIdentifier(err, c->nidb, entid, eid, &found))
    return 0;

  if (found) {
    *newflag = 0;
    return 1;
  }
      
  /* No Entity exists */
  /* Get the next available Entity-number */
  if (!zkAAGet(err, c->nidb->entidDB, &CURRENT_ENTID_TAG, sizeof(CURRENT_ENTID_TAG),
	       &p, &psz))
    return 0;

  /* Did we find it? */
  if (!p) {
    /* This is a start up situation */
    /* First number is 1 */
    *eid = 1;
  } else {
    *eid = *(uint32_t*)p;
    (*eid)++; /* For clarity */
  }
  p = eid;
  psz = sizeof(uint32_t);
  
  /* Store this number back */
  if (!zkAAPut(err, c->nidb->entidDB, &CURRENT_ENTID_TAG, sizeof(CURRENT_ENTID_TAG),
	       p, psz))
    return 0;

  /* Store the entity in the ent and entid DBs */
  entid->entid = *eid;
  p = entid;
  psz = sizeof(struct T_FrDBOTypeEntityID);

  /* Bug: Need to abstract this to ensure that the proper
     byte ordering is made */
  if (zkDebugIsOn()) {
    zkDebug("Saving entity [%lu]:", (unsigned long) *eid);
    zkDumpBuffer(err,p,psz,zkDumpToStream,stderr);
  }

  if (!zkAAPut(err, c->nidb->entDB, &entid->ent, sizeof(entid->ent),
	       p, psz))
    return 0;

  if (!zkAAPut(err, c->nidb->entidDB, &entid->entid, sizeof(entid->entid),
	       p, psz))
    return 0;

  return 1;
  
}

/*
 * Strikes the entity information from the two entity databases
 */
int deleteEntityIdent (
  ZK_INOUT ZkError err,
  ZK_IN  NidbContext c,
  ZK_IN  FrDBOTypeEntityID in_entdbo,
  ZK_IN  uint32_t in_eid)
{
  static const char *funcname = "deleteEntityIdent";
  int retcode = 0;

  if (!err || !c ||  !in_entdbo) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
    retcode = 0;

#if 0

    /* shouldn't need to do this, as these records should exist */
  } else if ( ! frNidbGetEntityIdentifier (err, c->nidb, in_entdbo, in_eid, &found)) {
    retcode = 0;

  } else if ( ! found) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENOTFOUND, "Unable to find requested entid [%u]", in_eid);
#endif

  } else if ( in_eid != in_entdbo->entid ) {
    zkErrorSet_va (err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "inconsistent data [eid=%u] != [entdbo->eid=%u]",
               in_eid, in_entdbo->entid);
    
  } else if ( ! zkAADelete (err, c->entDB, &in_entdbo->ent, sizeof(in_entdbo->ent)) ) {
    retcode = 0;

  } else if ( ! zkAADelete (err, c->entidDB, &in_entdbo->entid, sizeof(in_entdbo->entid)) ) {
    retcode = 0;

  } else {

    if (zkDebugIsOn()) {
      zkDebug("Deleted entity records for [%u]:", in_eid);
    }
    retcode = 1;
  }
  return retcode;
  
}

int
getLinkEntityName (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *node1,
  const char *node2,
  char **link_entname)
{
  static const char *funcname = "getLinkEntityName";
  FrDBOTypeNtdbLink lnk = NULL;
  FrDBOTypeEntityID entid = NULL;
  int retcode = 0, found = 0;
  uint32_t eid1 = 0;
  uint32_t eid2 = 0;
  size_t psz = 0;
  struct link_entname entname;
  char *buf = NULL;
  (void) funcname;

  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &entid, &psz))
    goto finish;
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNTDBLINK, (void **) &lnk, &psz))
    goto finish;
  
  /* Now we take care of the Node #1 */
  if ( ! frEntSetFromStr2(err, &entid->ent, "aip" , node1 )) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid1, &found)) 
    goto finish;

  if ( ! found) {
    zkShellWrite_va(err,shell,
		    "Error: Node [%s] is not in the database\n", node1);
    retcode = 1 ;
    goto finish;
  }

  lnk->node1_entid = eid1;
  entname.node1 = eid1;
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Node #1 entity identifier (ie: key) is [%lu]\n", eid1);

  /* Now we take care of the Node #2 */
  if ( ! frEntSetFromStr2(err, &entid->ent, "aip", node2 )) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c->nidb, entid, &eid2, &found)) 
    goto finish;

  if ( ! found ) {
    zkShellWrite_va(err,shell,
		    "Error: Node [%s] is not in the database\n", node2);
    retcode = 1 ;
    goto finish;
  }

  lnk->node2_entid = eid2;
  entname.node2 = eid2;
  if (zkDebugIsOn())
    zkShellWrite_va(err,shell,
		    "[*] Node #2 entity identifier (ie: key) is [%lu]\n", eid2);

  /* Now we take care of the link entity object */
  if (eid1 == eid2) {
    zkShellWrite_va(err,shell,
	    "Warning: Link has identical nodes [%s - %s]\n", node1, node2);
  }    

  if ( ! canonLinkEntname (err, shell, c, &entname))
    goto finish;

  buf = NEW(64);
  
  if (snprintf (buf, 63, "_LINK:%lu,%lu", (unsigned long)entname.node1, (unsigned long)entname.node2) < 0) {
    zkShellWrite_va(err,shell,
	    "Error: Internal buffer overflow creating link name [%lu,%lu]",
	    entname.node1, entname.node2);
    retcode =  0;
    free(buf);
    goto finish;
  }

  *link_entname = buf;
  retcode = 1;
  
 finish:
  if (entid) free (entid);
  if (lnk) free (lnk);
  
  return retcode;
}

int cmdEntityParseLinkParams (
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *ts_delay,
  ZK_IN const char *ts_pkts,
  ZK_IN const char *ct_on_flag,
  ZK_INOUT FrDBOTypeNtdbLink lnk)
{
  static const char *funcname = "cmdEntityParseLinkParams";
  char *ch = NULL;
  int retcode = 0;
  
   if ( !err || !c || !shell || !ts_delay || !ts_pkts || !ct_on_flag || !lnk) {
    zkShellWrite_va(err,shell,
		    "Error: Invalid parameters passed to %s.\n", funcname);
  } else { 
    lnk->ts_delay = strtol(ts_delay, &ch, 10);
    if (*ch != '\0') {
      zkShellWrite_va(err, shell,
                      "Error: error converting TS_DELAY [%s]\n", ts_delay) ;
    } else if (errno == ERANGE) {
      zkShellWrite_va(err, shell,
                      "Error: TS_DELAY is invalid [%s]\n", ts_delay);
    } else {
      ch = NULL;
      lnk->ts_pkts = strtol(ts_pkts, &ch, 10);
      
      if (*ch != '\0') {
        zkShellWrite_va(err, shell,
                        "Error: error converting TS_PKTS [%s]\n", ts_delay) ;
      } else if (errno == ERANGE) {
        zkShellWrite_va(err, shell,
                        "Error: TS_PKTS is invalid [%s]\n", ts_delay);
#ifdef FREEDOM_DEVELOPMENT_BUILD
      } else if ((lnk->ts_pkts <= 0) ||
                 (lnk->ts_delay < FR_TS_MIN_DELAY)) {
        zkShellWrite_va(err, shell,
                        "Error: %s or %s is out no in valid range\n",
                        ts_delay, ts_pkts);
#else
      } else if ((lnk->ts_pkts <= 0) ||
                 (lnk->ts_pkts > FR_TS_MAX_PPS) ||
                 (lnk->ts_delay < FR_TS_MIN_DELAY) ||
                 (lnk->ts_delay > FR_TS_MAX_DELAY)) {
        zkShellWrite_va(err, shell,
                        "Error: %s or %s is out no in valid range\n",
                        ts_delay, ts_pkts);
#endif
      } else if (strcasecmp (ct_on_flag, "on") == 0) {
        lnk->ct_on_flag = 1;
        retcode = 1;
      } else if (strcasecmp (ct_on_flag, "off") == 0) {
        lnk->ct_on_flag = 0;
        retcode = 1;
      } else {
        zkShellWrite_va(err,shell,
                        "Error: [%s] is not an option for the cover traffic flag \n",
                        ct_on_flag);
      }
    }
  }
  return retcode;
}

int cmdEntityGetLinkObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *node1,
  ZK_IN const char *node2,
  ZK_OUT FrDBOTypeNtdbLink *lnk,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *l_eid,
  ZK_OUT int *found)
{
  static const char *funcname = "cmdEntityGetLinkObjs";
  char *lnkname_p = NULL;
  size_t lsz = 0, psz = 0;
  int retcode = 0;

  if ( !c || !type || !node1 || !type || !node1 || !node2 || !lnk || !ent_dbo
       || !l_eid ) {
    zkErrorSet_va(err, funcname,  ZKERR, ZKMFRENT, ZKENULLPTR,
               "Invalid parameters passed to %s.\n", funcname);
    
  } else if ( (strcasecmp (type, "link" ) != 0)) {
    zkShellWrite_va(NULL, shell, "Error: %s\n", zkErrorDetail(err));
    
  } else if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) ent_dbo, &psz)) {
    zkShellWrite_va (NULL, shell,
                    "Error: creating DBO [%s]\n", zkErrorDetail(err));
    
  } else if ( ! frNidbGetLinkEntityName (err, c->nidb, node1, node2, &lnkname_p)){
    zkShellWrite_va (err, shell,
                    "Error: Unable to get the link name from %s and %s\n", node1, node2);

  } else if ( lnkname_p == NULL ) {
    zkShellWrite_va (err, shell,
                    "Error: One of the node names is invalid [%s] [%s]\n", node1, node2);

  } else if ( ! frEntSetFromStr2(err, &((*ent_dbo)->ent), type, lnkname_p)) {
    zkShellWrite_va(NULL, shell,
                     "Error: trouble with frEntSetFromStr2 [%s]\n",
                     zkErrorDetail(err));
    
  } else if ( ! frNidbGetEntityIdentifier (err, c->nidb, *ent_dbo, l_eid, found)) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with getting identifier [%s]\n",
                    zkErrorDetail(err));
    
  } else if ( ! *found) {
    zkShellWrite_va(err, shell,
                    "Error: link indentifier is not in the database  [%s]\n",
                    (*ent_dbo)->ent.name);
    retcode = 1;
    /* all parameters check out      */
    /* Now fetch the existing record */
#warning Bug: I noticed that the oldlnk is not correct
#warning Somewhere I am not setting the link_entid structure element when I add it to the DB
  } else if ( ! frNtdbGetEntityDBOUsingEntityIdentAndType (err, c->nidb, *l_eid,
                                                           FRENT_TYPE_LINK,
                                                           (void **) lnk,
                                                           &lsz, found)) {
    zkShellWrite_va(NULL, shell,
               "Error: trouble getting link object from database [%s]\n",
               zkErrorDetail(err));
  } else if ( ! *found) {
    zkShellWrite_va(err, shell,
               "Error: link record is not in the database  [%s]\n",
               (*ent_dbo)->ent.name);
  } else {
    retcode = 1;
  }
  if (retcode == 0) {
    FREEIT(*lnk);
    FREEIT(*ent_dbo);
    *l_eid = 0;
  }
  FREEIT(lnkname_p);
  return retcode;
}
            
int cmdEntityGetNodeObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *nodename,
  ZK_OUT FrDBOTypeNtdbNode *node,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found)
{
  static const char *funcname = "cmdEntityGetNodeObjs";
  size_t nsz = 0, esz = 0;
  int retcode = 0;

  if ( !err || !shell || !c || !type || !nodename || !node ||
       !ent_dbo || !eid || !found) {
    zkErrorSet_va(err, funcname,  ZKERR, ZKMFRENT, ZKENULLPTR,
               "Invalid parameters passed to %s.\n", funcname);

  } else if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) ent_dbo, &esz)) {
    zkShellWrite_va(NULL,shell,
                    "Error: creating DBO [%s]\n", zkErrorDetail(err));

  } else if ( ! frEntSetFromStr2(err, &((*ent_dbo)->ent), type, nodename)) {
    zkShellWrite_va(NULL, shell,
                     "Error: trouble with frEntSetFromStr2 [%s]\n",
                     zkErrorDetail(err));
    
  } else if ( ! frNidbGetEntityIdentifier (err, c->nidb, *ent_dbo, eid, found)) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with getting identifier [%s]\n",
                    zkErrorDetail(err));

  } else if ( ! found ) {
      zkShellWrite_va(err,shell,
                      "Error: Node [%s] is not in the database\n", nodename);
      retcode = 1 ;
  } else if ( ! frNtdbGetEntityDBOUsingEntityIdentAndType (err, c->nidb, *eid,
                                                           (*ent_dbo)->ent.type,
                                                           (void **) node,
                                                           &nsz, found)) {
    zkShellWrite_va(NULL, shell,
               "Error: trouble getting node object from database [%s]\n",
               zkErrorDetail(err));
  } else if ( ! *found) {
    zkShellWrite_va(err, shell,
               "Error: node record is not in the database  [%s]\n",
               (*ent_dbo)->ent.name);
  } else {
    retcode = 1;
  }
  if (retcode == 0) {
    FREEIT(*node);
    FREEIT(*ent_dbo);
    *eid = 0;
  }
  return retcode;
}

int cmdEntityGetAipObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *aipname,
  ZK_OUT FrDBOTypeNtdbAip *aip,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found)
{
  static const char *funcname = "cmdEntityGetAipObjs";
  size_t nsz = 0, esz = 0;
  int retcode = 0;

  if ( !err || !shell || !c || !type || !aipname || !aip ||
       !ent_dbo || !eid || !found) {
    zkErrorSet_va(err, funcname,  ZKERR, ZKMFRENT, ZKENULLPTR,
               "Invalid parameters passed to %s.\n", funcname);

  } else if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) ent_dbo, &esz)) {
    zkShellWrite_va(NULL,shell,
                    "Error: creating DBO [%s]\n", zkErrorDetail(err));

  } else if ( ! frEntSetFromStr2(err, &((*ent_dbo)->ent), type, aipname)) {
    zkShellWrite_va(NULL, shell,
                     "Error: trouble with frEntSetFromStr2 [%s]\n",
                     zkErrorDetail(err));
    
  } else if ( ! frNidbGetEntityIdentifier (err, c->nidb, *ent_dbo, eid, found)) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with getting identifier [%s]\n",
                    zkErrorDetail(err));

  } else if ( ! found ) {
      zkShellWrite_va(err,shell,
                      "Error: Aip [%s] is not in the database\n", aipname);
      retcode = 1 ;
  } else if ( ! frNtdbGetEntityDBOUsingEntityIdentAndType (err, c->nidb, *eid,
                                                           (*ent_dbo)->ent.type,
                                                           (void **) aip,
                                                           &nsz, found)) {
    zkShellWrite_va(NULL, shell,
               "Error: trouble getting aip object from database [%s]\n",
               zkErrorDetail(err));
  } else if ( ! *found) {
    zkShellWrite_va(err, shell,
               "Error: aip record is not in the database  [%s]\n",
               (*ent_dbo)->ent.name);
  } else {
    retcode = 1;
  }
  if (retcode == 0) {
    FREEIT(*aip);
    FREEIT(*ent_dbo);
    *eid = 0;
  }
  return retcode;
}

int cmdEntityGetHostObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *hostentname,
  ZK_OUT FrDBOTypeNtdbHost *host,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found)
{
  static const char *funcname = "cmdEntityGetHostObjs";
  size_t nsz = 0, esz = 0;
  int retcode = 0;

  if ( !err || !shell || !c || !type || !hostentname || !host ||
       !ent_dbo || !eid || !found) {
    zkErrorSet_va(err, funcname,  ZKERR, ZKMFRENT, ZKENULLPTR,
               "Invalid parameters passed to %s.\n", funcname);
  } else if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) ent_dbo, &esz)) {
    zkShellWrite_va(NULL,shell,
                    "Error: creating DBO [%s]\n", zkErrorDetail(err));

  } else if ( ! frEntSetFromStr2(err, &((*ent_dbo)->ent), type, hostentname)) {
    zkShellWrite_va(NULL, shell,
                     "Error: trouble with frEntSetFromStr2 [%s]\n",
                     zkErrorDetail(err));
    
  } else if ( ! frNidbGetEntityIdentifier (err, c->nidb, *ent_dbo, eid, found)) {
    zkShellWrite_va(NULL, shell,
                    "Error: trouble with getting identifier [%s]\n",
                    zkErrorDetail(err));

  } else if ( ! found ) {
      zkShellWrite_va(err,shell,
                      "Error: Host [%s] is not in the database\n", hostentname);
      retcode = 1 ;
  } else if ( ! frNtdbGetEntityDBOUsingEntityIdentAndType (err, c->nidb, *eid,
                                                           (*ent_dbo)->ent.type,
                                                           (void **) host,
                                                           &nsz, found)) {
    zkShellWrite_va(NULL, shell,
               "Error: trouble getting host object from database [%s]\n",
               zkErrorDetail(err));
  } else if ( ! *found) {
    zkShellWrite_va(err, shell,
               "Error: host record is not in the database  [%s]\n",
               (*ent_dbo)->ent.name);
  } else {
    retcode = 1;
  }
  if (retcode == 0) {
    FREEIT(*host);
    FREEIT(*ent_dbo);
    *eid = 0;
  }
  return retcode;
}

int cmdEntity (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{
  int return_code = ZKSHELL_SESSION_CONTINUE;
  NidbToolContext * context = (NidbToolContext *) userdata;
  int rc = 0, cdv_incr_flg = 1;
  (void) module_id_;

  zkErrorClear(err);
  
  if (tokens < 2) {
    zkShellWrite_va(err,shell,"Insufficient number of tokens\n"); 
    goto finish;
  }

  /* SHOW */
  if (strcasecmp(token[1],CMD_ENT_SHOW) == 0) {
    if (tokens == 4) {
      if( (strcasecmp( token[2] , "aip" )==0)||(strcasecmp( token[2] , "coreaip" ))==0 ) {
        rc = cmdEntityShowAip (err,shell,context,token[2],token[3]);
      } else if (strcasecmp(token[2],"host") == 0) {
        rc = cmdEntityShowHost (err,shell,context,token[2],token[3]);
      } else {
        rc = cmdEntityShowNode (err,shell,context,token[2],token[3]);
      }
    } else if (tokens == 5) {
      if (strcasecmp(token[2],CMD_ENT_NEIGH) == 0) {
	rc = cmdEntityShowNeighbour (err,shell,context,token[3],token[4]);
      } else {
	rc = cmdEntityShowLink (err,shell,context,token[3],token[4]);
      }
    } else {
      zkShellWrite_va(err,shell,
		      "Syntax: show aip entname \n\tshow link node1 node1\n");
    }
  } else if (strcasecmp(token[1],CMD_ENT_SET) == 0) {
    rc = 0;
    
    if( strcasecmp(token[tokens-1],CDV_NO_INCR_OPTION)==0 ) {
      cdv_incr_flg = 0;
      tokens--; /* Ignore last argument and continue */
    } else {
      cdv_incr_flg = 1;
    }

    if (tokens == 5) {
      /* entity set aip zksbeta1 available */
      zkLogWrite_va(context->log, "Command: %s %s %s %s",
		    token[1], token[2], token[3], token[4]);
      if( (strcasecmp(token[2],"aip")==0)||(strcasecmp(token[2],"coreaip")==0) ){
        rc = cmdEntitySetAip (err,shell,context,token[2],token[3],token[4]);

      } else if ( strcasecmp (token[2], "maip") == 0) {
        rc = cmdEntitySetMaip (err,shell,context,token[2],token[3],token[4]);
      } else {
        zkShellWrite_va(err,shell,
                        "Syntax:" \
                        "    set aip aip-ent-name < available | unavailable > [%s]\n" \
                        "    set maip maip-ent-name < available | unavailable > [%s]\n",
                        CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION);
      }

    } else if (tokens == 8 ) {
      /* set link aip1 aip2 ts-delay ts-pkts ct-flag */
      zkLogWrite_va(context->log, "Command: %s %s %s %s %s %s %s",
		    token[1], token[2], token[3], token[4],
                    token[5], token[6], token[7] ); 
      rc = cmdEntitySetLink (err,shell,context,token[2],token[3],token[4],
                             token[5], token[6], token[7] ); 

    } else {
      zkShellWrite_va(err,shell,
                      "Syntax:" \
                      "    set aip aip-ent-name < available | unavailable > [%s]\n" \
                      "    set maip maip-ent-name < available | unavailable > [%s]\n" \
                      "    set link aip1 aip2 ts-delay ts-pkts ct-flag [%s]\n",
                      CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION );
    }

    if( rc && cdv_incr_flg )  /* Increment CDV if success */
    {
      if( !cmdCacheIncrement(err,shell,context) )
        zkShellWrite_va(err,shell,"Error while trying to increment CDV.\n");
      else
        zkShellWrite_va(err,shell,"CDV Incremented.\n");
    }
    

    /* DELETE */
  } else if (strcasecmp(token[1],CMD_ENT_DELETE) == 0) {
    rc = 0;
    
    if( strcasecmp(token[tokens-1],CDV_NO_INCR_OPTION)==0 ) {
      cdv_incr_flg = 0;
      tokens--; /* Ignore last argument and continue */
    } else {
      cdv_incr_flg = 1;
    }

    if (tokens == 4) {
      zkLogWrite_va(context->log, "Command: %s %s %s",
		    token[1], token[2], token[3]); 
      rc = cmdEntityDelete(err,shell,context,token[2],token[3],NULL); 
    } else if (tokens == 5) {
      zkLogWrite_va(context->log, "Command: %s %s %s %s",
		    token[1], token[2], token[3], token[4]); 
      rc = cmdEntityDelete(err,shell,context,token[2],token[3],token[4]); 
    } else {
      zkShellWrite_va(err,shell,
                      "Syntax:" \
                      "\tentity delete aip aip-entity-name [%s]\n"
                      "\tentity delete host host-entity-name [%s]\n"
                      "\tentity delete entity entity-name [%s]\n"
                      "\tentity delete link aip-entity-name1 aip-entity-name2 [%s]\n",
                      CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION, CDV_NO_INCR_OPTION );
    }

    if( rc && cdv_incr_flg )  /* Increment CDV if success */
    {
      if( !cmdCacheIncrement(err,shell,context) )
        zkShellWrite_va(err,shell,"Error while trying to increment CDV.\n");
      else
        zkShellWrite_va(err,shell,"CDV Incremented.\n");
    }
      
    /* DESC */
  } else if (strcasecmp(token[1],CMD_ENT_DESCRIBE) == 0) {
    if (tokens == 3) {
      zkLogWrite_va(context->log, "Command: %s %s",
                    
		    token[1], token[2]); 
      rc = cmdEntityDescribe(err,shell,context,token[2]); 
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify the entity type and name.\n");
    }

    /* STATE */
  } else if (strcasecmp(token[1],CMD_ENT_STATE) == 0) {
    if (tokens == 4) {
      rc = cmdEntityState(err,shell,context,token[2],token[3]); 
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify the entity type and name.\n");
    }

    /* ADD */
  } else if (strcasecmp(token[1],CMD_ENT_ADD) == 0) {
    rc = 0;
    
    if( strcasecmp(token[tokens-1],CDV_NO_INCR_OPTION)==0 ) {
      cdv_incr_flg = 0;
      tokens--; /* Ignore last argument and continue */
    } else {
      cdv_incr_flg = 1;
    }

    if (tokens == 10) {
      /* add-AIP */
      zkLogWrite_va(context->log,
		    "Command: %s %s %s %s %s %s %s %s %s",
		    token[1], token[2], token[3], token[4],
		    token[5], token[6], token[7], token[8], token[9]);

      if( (strcasecmp(token[2],"aip")==0)||(strcasecmp(token[2],"coreaip")==0) ){
        rc = cmdEntityAddAip(err,shell,context, token[2], token[3],token[4],
                             token[5],token[6],token[7], token[8], token[9]); 
      } else {
        zkShellWrite_va(err,shell,
              "Error: Entity %s is unknown.\n",token[2]);
      }

    } else if (tokens == 9) {
      /* add-HOST  */
      zkLogWrite_va(context->log,
		    "Command: %s %s %s %s %s %s %s %s",
		    token[1], token[2], token[3], token[4],
		    token[5], token[6], token[7], token[8]);

      if (strcasecmp(token[2], "host") == 0) {
        if( reformatRegionString(err,shell,token[6],strlen(token[6])) ) {
          rc = cmdEntityAddHost(err,shell,context,token[2],token[3],token[4],
			token[5],token[6], token[7], token[8]);
	}
      } else {
        zkShellWrite_va(err,shell,
              "Error: Entity %s is unknown.\n",token[2]);
      }
        

    } else if (tokens == 6) {
      /* add-ENTITY */
      zkLogWrite_va(context->log, "Command: %s %s %s %s %s",
		    token[1], token[2], token[3], token[4],
		    token[5]); 
      rc = cmdEntityAddNode(err,shell,context,token[2],token[3],token[4],
			token[5]);

    } else if (tokens == 8) {
      /* add-LINK */
      int node1IsAip = 0;
      int node2IsAip = 0;
      zkLogWrite_va(context->log, "Command: %s %s %s %s %s %s %s",
		    token[1], token[2], token[3], token[4],
		    token[5], token[6], token[7]); 
		    
      if( reformatLinkEnd(err,shell,token[3],&node1IsAip) )
      {
        if( reformatLinkEnd(err,shell,token[4],&node2IsAip) )
	{
          rc = cmdEntityAddLink(err,shell,context,token[2],token[3], node1IsAip, token[4],
	  		  node2IsAip, token[5],token[6], token[7]);
	}
      }

    } else {
      zkShellWrite_va(err,shell,
      "You must specify the entity type and name and all parameters for that type.\n");
    }
    
    if( rc && cdv_incr_flg )  /* Increment CDV if success */
    {
      if( !cmdCacheIncrement(err,shell,context) )
        zkShellWrite_va(err,shell,"Error while trying to increment CDV.\n");
      else
        zkShellWrite_va(err,shell,"CDV Incremented.\n");
    }

    /* LIST */
  } else if (strcasecmp(token[1],CMD_ENT_LIST) == 0) {
    if (tokens == 2) {
      rc = cmdEntityListUser(err,shell,context);
    } else {
      zkShellWrite_va(err,shell,
                      "No parameters for this command.\n");
    }
  } else if (strcasecmp(token[1],CMD_ENT_UPDATE) == 0) {
    rc = 0;
    
    if( strcasecmp(token[tokens-1],CDV_NO_INCR_OPTION)==0 ) {
      cdv_incr_flg = 0;
      tokens--; /* Ignore last argument and continue */
    } else {
      cdv_incr_flg = 1;
    }

    if (tokens == 6) {
      if( strcmp(token[4],"region") || reformatRegionString(err,shell,token[5],strlen(token[5])) ) {
        zkShellWrite_va(err, shell, "Command: %s %s %s %s %s\n",
                        token[1], token[2], token[3], token[4], token[5]);
        rc = cmdEntityUpdate(err, shell, context, token[2], token[3], token[4], token[5]);
      }
    }else {
      zkShellWrite_va(err,shell,
         "you must specify entity type, entity name, parameter to update, new value.\n");
    }

    if( rc && cdv_incr_flg )  /* Increment CDV if success */
    {
      if( !cmdCacheIncrement(err,shell,context) )
        zkShellWrite_va(err,shell,"Error while trying to increment CDV.\n");
      else
        zkShellWrite_va(err,shell,"CDV Incremented.\n");
    }

  } else {
    zkShellWrite_va(err,shell,"unrecognized command \"%s\".\n",token[1]);
  }

finish:
  return return_code;
}


/* -----------------------------------------------------------------------------
 * Description: reformatRegionString() takes a User formated region string,
 *              reformats it into the database format, and validates the 
 *              format/values in the resulting string which contains:
 *              - the region code (4 or 2 characters)
 *              - the latitude (5 characters)
 *              - the longitude (6 characters)
 *
 *              Ex. "CA10:3025N/12030E" would be converted to "CA103025N12030E".
 *                           ^                                        ^
 *                      USER format                            DATABASE format
 *
 *                  "CA:3025N/12030E" would be converted to "CA3025N12030E".
 *                           ^                                        ^
 *                      USER format                            DATABASE format
 *
 *              The User formated region string gets reformated only if it is
 *              valid.
 *
 * Parameters: - [ZkError] error: current ZK error object
 *             - [ZkShell] shell: current ZK shell
 *             - [char *] userString: null terminated string containing the region
 *                                    string in the User Format. This string is
 *                                    normally reformated to the Database Format.
 *             - [int] userStringSize: the size of the userString size.
 *
 * Return values:
 *     0 if the format and/or values are incorrect:
 *          - the parameter (region string) is a NULL string, or
 *          - the parameter length (USER format) is NOT 17 NOR 15, or
 *          - the region code is not valid, or
 *          - the user format does not have the separator characters (: and /)
 *            at the right positions, or
 *          - the 9th (or 7th) character (DATABASE format) is not 'N', 'n', 'S' nor 's', or
 *          - the 15th (or 13th) character (DATABASE format) is not 'E', 'e', 'W' nor 'w', or
 *          - the longitude and/or latitude contains invalid character(s), or
 *          - the longitude and/or latitude is out of range (see range below).
 *     1 otherwise (check valid format below)
 * 
 * USER FORMAT: RRRR:ddmmD/dddmmD
 * RRRR is the region code (there may be 2 caracters only). Ex.: "CA10" or "CA"
 * ddmmD is latitude. Ex.: "3025N" or "3025S" (D can either be 'N' or 'S')
 * dddmmD is longitude. Ex.: "12030E" or "12030W" (D can either be 'E' or 'W')
 *
 * DATABASE FORMAT: RRRRddmmDdddmmD
 * RRRR is the region code (there may be 2 caracters only). Ex.: "CA10" or "CA"
 * ddmmD is latitude. Ex.: "3025N" or "3025S" (D can either be 'N' or 'S')
 * dddmmD is longitude. Ex.: "12030E" or "12030W" (D can either be 'E' or 'W')
 *
 * RANGE OF VALUES
 * Latitude : 0<dd<90, 0<mm<59, D=[N|S] (mm=00 if dd=90)
 * Longitude: 0<ddd<180, 0<mm<59, D=[E|W] (mm=00 if ddd=180)
 *
 * ----------------------------------------------------------------------------- */
static int reformatRegionString( ZkError error, ZkShell shell, char *userString, int userStringSize )
{
  static const char *funcname = "regStringIsValid";
  long latiDegrees, latiMinutes, longDegrees, longMinutes;
  char buffer[4];
  char nidbFormatString[16]; /* max 15 characters + \0 */
  char *pLastChar;
  int userStringLength, offset;
  
  char regCode[5];
  int regCode_sz = 0;
  int exists = 0;

  if( userString==NULL ) /* Check for NULL parameter */
  {
    zkShellWrite_va( error, shell,
      "%s: Internal error: NULL parameter passed for region string.\n", funcname );
    return 0; /* User string not valid: NULL parameter. */
  }
  
  userStringLength = strlen(userString);
  
  if( userStringLength!=17 && userStringLength!=15 ) /* Check for parameter's length */
  {
    zkShellWrite_va( error, shell, 
      "Error: '%s' is not 17 nor 15 characters long, so the user format is not appropriate.\n", userString );
    return 0; /* User string not valid: invalid string length. */
  }

  offset = 17 - userStringLength; /* This offset is either 2 or 0 */
  regCode_sz = 4 - offset; /* This length is either 2 or 4 */
  
  strncpy( regCode, userString, regCode_sz );
  regCode[regCode_sz] = '\0';
  
  if( !frNidbVerifyFip10CountryRegionAbbreviation(error,regCode,regCode_sz,&exists) )
  {
    zkShellWrite_va( error, shell,
      "%s: Internal error: frNidbVerifyFip10CountryRegionAbbreviation failed.", funcname );
    return 0; /* Internal error. */
  }
  else if( !exists )
  {
    zkShellWrite_va( error, shell, 
      "Error: '%s' is not a valid region code.\n", regCode );
    return 0; /* User string not valid: invalid region code. */
  }
  
  /* Checking for the USER format of the region string */
  if( userString[4-offset]!=':' || userString[10-offset]!='/' )
  {
    zkShellWrite_va( error, shell, 
      "Error: '%s' is not in the correct user format. Character %d is not ':' or character %d is not '/'.\n", userString, 4-offset+1, 10-offset+1 );
    return 0; /* User string not valid: Separator caracter not in the right position. */
  }

  /* Convert user format to Nidb format: ignore 5th (or 3rd) and 11th (or 9th) characters */
  strncpy( nidbFormatString, userString, 4-offset );
  nidbFormatString[4-offset] = '\0';
  strncat( nidbFormatString, &userString[5-offset], 5 );
  zkCStrlcat( nidbFormatString, &userString[11-offset], sizeof(nidbFormatString) );

  /* Convert to upper case the directions of the latitude/longitude */
  nidbFormatString[8-offset] = toupper( nidbFormatString[8-offset] );
  nidbFormatString[14-offset] = toupper( nidbFormatString[14-offset] );
  
  /* Check for latitude direction (North or South) */
  if( nidbFormatString[8-offset]!='N' && nidbFormatString[8-offset]!='S' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The latitude direction (caracter %d of '%s') is not appropriate, must be either 'N', 'n', 'S' or 's'.\n", 10-offset, userString );
    return 0; /* Invalid latitude direction. */
  }
  
  /* Check for longitude direction (East or West) */
  if( nidbFormatString[14-offset]!='E' && nidbFormatString[14-offset]!='W' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The longitude direction (caracter %d of '%s') is not appropriate, must be either 'E', 'e', 'W' or 'w'.\n", 17-offset, userString );
    return 0; /* Invalid longitude direction. */
  }

  /*
   * Extract and test values of latitude and longitude
   */

  strncpy( buffer, &nidbFormatString[4-offset], 2 ); /* Latitude degrees */
  buffer[2] = '\0';
  latiDegrees = strtol( buffer, &pLastChar, 10 );
  if( *pLastChar!='\0' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The latitude degree value extracted from '%s' contains an invalid character.", buffer );
    return 0; /* Invalid latitude degree value. */
  }

  strncpy( buffer, &nidbFormatString[6-offset], 2 ); /* Latitude minutes */
  buffer[2] = '\0';
  latiMinutes = strtol( buffer, &pLastChar, 10 );
  if( *pLastChar!='\0' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The latitude minute value extracted from '%s' contains an invalid character.", buffer );
    return 0; /* Invalid latitude minute value. */
  }

  strncpy( buffer, &nidbFormatString[9-offset], 3 ); /* Longitude degrees */
  buffer[3] = '\0';
  longDegrees = strtol( buffer, &pLastChar, 10 );
  if( *pLastChar!='\0' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The longitude degree value extracted from '%s' contains an invalid character.", buffer );
    return 0; /* Invalid longitude degree value. */
  }

  strncpy( buffer, &nidbFormatString[12-offset], 2 ); /* Longitude minutes */
  buffer[2] = '\0';
  longMinutes = strtol( buffer, &pLastChar, 10 );
  if( *pLastChar!='\0' )
  {
    zkShellWrite_va( error, shell, 
      "Error: The longitude minute value extracted from '%s' contains an invalid character.", buffer );
    return 0; /* Invalid longitude minute value. */
  }

  /* Check for value of latitude degrees */
  if( latiDegrees<0 || latiDegrees>90 )
  { 
    zkShellWrite_va( error, shell, 
      "Error: The value of latitude degrees '%d' extracted from '%s' is out of range, must be between 0 and 90.\n", latiDegrees, userString );
    return 0; /* Latitude degrees out of range. */
  }
  
  /* Check for value of latitude minutes */
  if( latiMinutes<0 || latiMinutes>59 )
  { 
    zkShellWrite_va( error, shell, 
      "Error: The value of latitude minutes '%d' extracted from '%s' is out of range, must be between 0 and 59.\n", latiMinutes, userString );
    return 0; /* Latitude minutes out of range. */
  }
  
  /* If the latitude is 90 degrees, the minutes need to be zero */
  if( latiDegrees==90 && latiMinutes!=0 )
  {
    zkShellWrite_va( error, shell, 
      "Error: The value of latitude minutes '%d' extracted from '%s' is out of range for a latitude of 90 degrees.\n", latiMinutes, userString );
    return 0; /* Latitude minutes out of range for 90 degrees. */
  }
  
  /* Check for value of longitude degrees */
  if( longDegrees<0 || longDegrees>180 )
  { 
    zkShellWrite_va( error, shell, 
      "Error: The value of longitude degrees '%d' extracted from '%s' is out of range, must be between 0 and 180.\n", longDegrees, userString );
    return 0; /* Longitude degrees out of range. */
  }
  
  /* Check for value of longitude minutes */
  if( longMinutes<0 || longMinutes>59 )
  { 
    zkShellWrite_va( error, shell, 
      "Error: The value of longitude minutes '%d' extracted from '%s' is out of range, must be between 0 and 59.\n", longMinutes, userString );
    return 0; /* Longitude minutes out of range. */
  }
  
  /* If the longitude is 180 degrees, the minutes need to be zero */
  if( longDegrees==180 && longMinutes!=0 )
  {
    zkShellWrite_va( error, shell, 
      "Error: The value of longitude minutes '%d' extracted from '%s' is out of range for a longitude of 180 degrees.\n", longMinutes, userString );
    return 0; /* Longitude minutes out of range for 180 degrees. */
  }

  zkCStrlcpy( userString, nidbFormatString, userStringSize-1 ); /* Else, the reformatted string is valid */

  return 1;
}




/* -----------------------------------------------------------------------------
 * Description: reformatLinkEnd() takes a name (string) of a link end in 
 *              parameter. If the link end starts with either "coreaip/" or
 *              "aip/", these substrings are removed. In the case of "coreaip/",
 *              the isAip flag is set to 0, any other cases, the isAip flag is
 *              set to 1.
 *
 * Parameters: - [ZkError] error: current ZK error object
 *             - [ZkShell] shell: current ZK shell
 *             - [char *] linkEnd: null terminated string containing the name
 *                                 of the link end. This is reformated if no
 *                                 error occurs.
 *             - [int *] pIsAip: a pointer to the "IS AIP" flag.
 *
 * Return values:
 *     0 if an internal error has occured:
 *          - linkEnd is a NULL pointer
 *          - pIsAip is a NULL pointer
 *     1 otherwise
 *
 * Examples:
 *  "aip/HOST1" would be reformated to "HOST1", and the isAip flag would be 1
 *  "coreaip/HOST1" would be reformated to "HOST1" and the isAip flag would be 0
 *  "HOST1" would remain unchanged and isAip flag would be 1
 *  "somethingelse/HOST1" would remain unchanged and isAip flag would be 1
 *
 * ----------------------------------------------------------------------------- */
static int reformatLinkEnd( ZkError error, ZkShell shell, char *linkEnd, int *pIsAip )
{
  static const char *funcname = "reformatLinkEnd";
  char buffer[128];
  int linkEndSize;
  
  if( (linkEnd==NULL) || (pIsAip==NULL) ) /* Check for NULL parameter */
  {
    zkShellWrite_va( error, shell,
      "%s: Internal error: NULL parameter passed to function.\n", funcname );
    return 0; /* Error: NULL parameter. */
  }
  linkEndSize = strlen( (const char *)linkEnd );
  
  if( strncasecmp((const char *)linkEnd,"coreaip/",8 )==0 ) /* Is a coreaip */
  {
    zkCStrlcpy( buffer, &linkEnd[8], sizeof buffer );
    strncpy( linkEnd, buffer, linkEndSize-8 );
    linkEnd[linkEndSize-8] = '\0';
    *pIsAip = 0; /* This entity is not an AIP */
  }
  else if( strncasecmp((const char *)linkEnd,"aip/",4 )==0 ) /* Is a aip */
  {
    zkCStrlcpy( buffer, &linkEnd[4], sizeof buffer );
    strncpy( linkEnd, buffer, linkEndSize-4 );
    linkEnd[linkEndSize-4] = '\0';
    *pIsAip = 1; /* This entity is an AIP */
  }
  else
  {
    *pIsAip = 1; /* This entity is, by default, an AIP (for backward compatibility) */
  }
  
  return 1; /* Success */
}
