/*
 * $Id: nidbtoolCmdEntity.h,v 1.26 2001/04/19 19:22:54 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef CMDENTITY_H_
#define CMDENTITY_H_

#include <zks.h>

DLLExport int cmdShowContext (
  ZkError ,              /* Return the error if any occured */
  ZkShell ,              /* A ZK command shell structure */
  char ** ,              /* The list of tokens on the input line */
  size_t ,               /* The number of tokens */
  void * );              /* Typecast this to NymToolContext * */

#define CMD_ENTITY_USAGE \
"entity ( list | show | add | delete | update | state | desc ) ..."

#define CMD_ENTITY_DESCR \
"Used to manipulate database entries"

#define CMD_ENTITY_HELP \
"The entity command has various subcommands used to manipulate the\n" \
"topology database:\n" \
"* list\n" \
"     list\n" \
"* delete\n" \
"     delete aip aip-entity-name [--cdv_noincr]\n" \
"     delete host host-entity-name [--cdv_noincr]\n" \
"     delete entity entity-name [--cdv_noincr]\n" \
"     delete link aip-entity-name1 aip-entity-name2 [--cdv_noincr]\n" \
"* desc ent-ident\n" \
"     desc ent-ident\n" \
"* state\n" \
"    state ent-type ent-name\n" \
"    - displays an entity's state\n" \
"* set\n" \
"    set aip aip-ent-name < available | unavailable > [--cdv_noincr]\n" \
"    set maip aip-ent-name < available | unavailable > [--cdv_noincr]\n" \
"    set link aip1 aip2 ts-delay ts-pkts ct-flag [--cdv_noincr]\n" \
"           where: ts-delay   is the number of micro seconds between salvos\n" \
"                  ts-pkts    is the number packets per salvo\n" \
"                  ct-flag    cover traffic is on or off\n" \
"* show\n" \
"    show ent-type ent-name\n" \
"    show link ent-name1 ent-name2\n" \
"    show neighbour ent-type ent-name\n" \
"* update\n" \
"    update ent-type ent-name var-to-update newval [--cdv_noincr]\n" \
"           where: ent-type is only host\n" \
"                  var-to-update is servoperator, region or ip (for now)\n" \
"* add\n" \
"    add host host-entity-name hostname ipaddr region servoperator policy [--cdv_noincr]\n"\
"           where: region is the region code \n" \
"                  <region code>: <latitude>/<longitude>\n" \
"                  ex: <region code>: \"CA10\" or \"CA\"\n" \
"                      <latitude>:    \"6045N\"\n" \
"                      <longitude>:   \"12025W\"\n"\
"    add entity-type entity-name host-entity-name port [--cdv_noincr]\n"\
"           where: entity-type is one of\n"\
"                  fmg, maip, nidb, nymsrv, sersrv, toksrv, keyqry, keyupd\n" \
"    add aip aip-name host-entity-name aip-port dh-port dh-timeout \n" \
"           dh-negotation protocol-version [--cdv_noincr]\n" \
"    add link link-name ent-name ent-name ts-delay ts-pkts ct-flag [--cdv_noincr]\n" \
"           where: ts-delay   is the number of micro seconds between salvos\n" \
"                  ts-pkts    is the number packets per salvo\n" \
"                  ct-flag    cover traffic is on or off\n"

#include "fr_nidb.h"

#define FR_TS_MAX_PPS    20
#define FR_TS_MIN_DELAY  1000
#define FR_TS_MAX_DELAY 50000

extern int cmdEntity (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata );

extern int cmdEntitySetAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node,
  const char *avail_flag);

int cmdEntitySetMaip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *avail_flag);

extern int cmdEntitySetLink (
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *node1,
  ZK_IN const char *node2,
  ZK_IN const char *ts_delay,
  ZK_IN const char *ts_pkts,
  ZK_IN const char *ct_on_flag);

extern int cmdEntityShowLink (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *node1,
  const char *node2);

extern int cmdEntityShowNode (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node);

extern int cmdEntityShowHost (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *hostname);

extern int cmdEntityShowAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *aipname);

extern int cmdEntityShowNeighbour (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node);

extern int getLinkEntityName (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *node1,
  const char *node2,
  char **link_entname);


extern int cmdEntityParseLinkParams (
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *ts_delay,
  ZK_IN const char *ts_pkts,
  ZK_IN const char *ct_on_flag,
  ZK_OUT FrDBOTypeNtdbLink lnk);

extern int cmdEntityGetLinkObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *node1,
  ZK_IN const char *node2,
  ZK_OUT FrDBOTypeNtdbLink *lnk,
  ZK_OUT FrDBOTypeEntityID *ent_dbo,
  ZK_OUT FrEntIdent *l_eid,
  ZK_OUT int *found);

extern int cmdEntityGetNodeObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *nodename,
  ZK_OUT FrDBOTypeNtdbNode *node,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found);

extern int cmdEntityGetAipObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *aipname,
  ZK_OUT FrDBOTypeNtdbAip *aip,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found);

extern int cmdEntityGetHostObjs(
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *hostname,
  ZK_OUT FrDBOTypeNtdbHost *host,     /* Free space when you're done */
  ZK_OUT FrDBOTypeEntityID *ent_dbo, /* Free space when you're done */
  ZK_OUT FrEntIdent *eid,
  ZK_OUT int *found);

extern int createEntityIdent (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  FrDBOTypeEntityID entid,
  uint32_t *eid,
  int *newflag);

extern int deleteEntityIdent (
  ZK_INOUT ZkError err,
  ZK_IN  NidbContext c,
  ZK_IN  FrDBOTypeEntityID in_entdbo,
  ZK_IN  uint32_t in_eid);

extern int cmdEntityListAll (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c);

extern int cmdEntityListUser (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c);

extern int printAipDBO (
  ZkError err,
  ZkShell shell,
  FrDBOTypeNtdbAip aip,
  const char *type,
  const char *ent);

extern int cmdEntityState (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node);

extern int cmdEntityDelete (
  ZK_INOUT ZkError err,
  ZK_IN ZkShell shell,
  ZK_IN NidbToolContext *c,
  ZK_IN const char *type,
  ZK_IN const char *node1,
  ZK_IN const char *node2);

extern int cmdEntityAddAip (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostentityname,
  const char *aip_port,
  const char *dh_port,
  const char *dh_timeout,
  const char *dh_negotiation,
  const char *protocol_vers
  );

extern int cmdEntityAddLink (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node1,
  int node1IsAip,
  const char *node2,
  int node2IsAip,
  const char *ts_delay,
  const char *ts_pkts,
  const char *ct_on_flag);

extern int canonLinkEntname (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  struct link_entname *entname);

extern int cmdEntityAddNode (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *node,
  const char *hostentityname,
  const char *port
  );

extern int cmdEntityAddHost (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostname,
  const char *ip_addr,
  const char *region,
  const char *servoperator,
  const char *policy
  );

extern int cmdEntityDescribe (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  char *entidstr);

extern int cmdEntityUpdate (
    ZkError           err,
    ZkShell           shell,
    NidbToolContext * c,
    const char *      type,
    const char *      entname,
    const char *      whichField,
    const char *      newVal);

int cmdEntitySetHost (
  ZkError err,
  ZkShell shell,
  NidbToolContext *c,
  const char *type,
  const char *entname,
  const char *hostname,
  const char *ip_addr,
  const char *region,
  const char *servoperator,
  const char *policy );
                            
#endif 
