/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#if 0
/*
 * Handle incoming AIP updates with SQL
 *
 */
#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <mysql/mysql.h>

#include <zkError.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkFile.h>
#include <frFilenames.h>
#include <frSec.h>

#include <frNIDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


static int scanNIDBdir (ZkError err, MYSQL *mysql)
{
  static const char *funcname = "scanNIDBdir";
  struct stat sbuf;    /* For identifying the file/directory */
  DIR * dir = NULL;
  int dirlen = 0; /* Length of the source directory name */
  struct dirent * dirent = NULL; 
  char pathname[MAXPATHLEN + 1];
  char spooldir[MAXPATHLEN + 1] = FREEDOM_STATSDIR;
  char * data = NULL; /* Data buffer read from file */
  size_t dsz = 0; /* Size of the data buffer */
  int retcode = 0;

  if (stat(spooldir, &sbuf) != 0) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMXFER,ZKERROR,
		  "Could not stat \"%s\"",spooldir);
    return 0;
  }
  if (sbuf.st_mode & S_IFDIR) { /* Local is a directory */

    /*
     * Transfer the contents of the directory.  In this case, the local
     * filename will be used for the remote filename as well.
     */

    if ((dir = opendir(spooldir)) == NULL) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMXFER,ZKERROR,
		    "Unable to open directory \"%s\"",spooldir);
      return 0;
    }
    dirlen = strlen(spooldir);
    while ((dirent = readdir(dir)) != NULL) {
      if ((dirlen + strlen(dirent->d_name)) >= MAXPATHLEN) {
	zkDebug("Filename too long for \"%s\"", dirent->d_name);
	continue;
      }
      snprintf(pathname,sizeof pathname,"%s/%s",spooldir,dirent->d_name);
      if ((stat(pathname,&sbuf) == 0) && (sbuf.st_mode & S_IFREG)) {
#ifdef LUCXX	
	if (!frSQLProcessAipUpdatePktFromFile (err, mysql, pathname)) {
            zkDebug ("Problem processing");
	} else {
#ifndef NDEBUG	  
	  if (unlink(pathname) != 0) {
	    zkDebug("Unable to unlink \"%s\"",pathname);
	  }
#endif	  
	}
#endif /* LUCXX */	
  	if (data) {
	  free(data);
	  data = NULL;
	}
	dsz = 0;
      }
    }
    closedir(dir);
    retcode = 1;
  } else {
    zkErrorSet_va(err,funcname,ZKERR,ZKMXFER,ZKERROR,
		  "[%s] is no longer available a directory" ,spooldir);
    return 0;
  }
  return 1;
}

static void exiterr(MYSQL *mysql, int exitcode)
{
  fprintf( stderr, "%s\n", mysql_error(mysql) );
  exit( exitcode );
}

int main(int argc, char **argv)
{
  ZkError err;
  struct T_FrNIDBContext context;
  int running = 1;

  MYSQL mysql_d, *mysql = &mysql_d;

  if (!zkErrorNew(&err)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }
  
#if 0

  if (!(mysql_connect(mysql,"localhost","luc",""))) 
    exiterr(mysql, 1);

  if (mysql_select_db(mysql,"test"))
    exiterr(mysql, 2);
  if (mysql_query(mysql,"SELECT * FROM aips"))
    exiterr(mysql, 3);

  if (!(res = mysql_store_result(mysql)))
    exiterr(4);

  while((row = mysql_fetch_row(res))) {
    for (i=0 ; i < mysql_num_fields(res); i++) 
      printf("%s\n",row[i]);
  }

  mysql_free_result(res);
  while (running) {
    scanNIDBdir (err, &mysql);
    sleep (10);
  }
  
  mysql_close(&mysql);
#endif
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}

#endif
