#!/usr/bin/perl -w
# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is Freedom code.
# 
# The Initial Developer of the Original Code is Zero-Knowledge Systems,
# Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
# Zero-Knowledge Systems, Inc.  All Rights Reserved.
# 
# Contributor(s):

#
#************************************************************************
#
#   File:		frgetcustomersupport.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		22 Oct 1999
#
#   Last Revision:	$Date: 2000/11/24 15:03:10 $
#
#   Description:	
#
#   $Revision: 1.16 $
#
#
#   $Id: frgetcustomersupport.pl,v 1.16 2000/11/24 15:03:10 marcp Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use FileHandle;
use IPC::Open2;
use Getopt::Long;
use Time::Local;

my $version = '$Id: frgetcustomersupport.pl,v 1.16 2000/11/24 15:03:10 marcp Exp $';
my $revision= '$Revision: 1.16 $';
my $progname= 'frgetcustomersupport.pl';

sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}
##
## need to convert a time stamp in
## 
my %DAY   = ('Sun', 0, 'Mon', 1, 'Tue', 2,'Wed',3, 'Thu',4,'Fri',5, 'Sat', 6);
my %MONTH = ('Jan', 0, 'Feb', 1, 'Mar', 2, 'Apr', 3, 'May', 4, 'Jun', 5, 
            'Jul', 6, 'Aug', 7, 'Sep', 8, 'Oct', 9, 'Nov', 10, 'Dec', 11);
##
## argument parsing and checking
##
STDOUT->autoflush();
STDERR->autoflush();

my %optctl  = ();
my($res);

my @helpString = 
    (
     "\n\n",
     "Usage $progname",
     "\t--help                  : this help",
     "\t--debug                 : debug mode on",
     "\t--networkSumm           : network summary",
     "\t--customSupport         : customer support",
     "\t--servopreliability     : server operator reliability",	
     "\t--servoptrafficsumm     : servop traffic summary",
     "\t--servoptotaltraffic    : server operator total summary"
     );

## Get the Options
##

$res = GetOptions(\%optctl, 
		  "help",
		  "debug",
		  "networksumm",
		  "customsupport",
		  "servopreliability",
		  "servoptrafficsumm",
		  "servoptotaltraffic",
		 ) || die "Error in GetOpt";

## checks if arguments
##
if(  scalar keys %optctl <= 0){
    ExitOnError("***No options provided",  
                @helpString
                );
}

## print out the usage string
##
if (defined($optctl{'help'})){
    print join("\n", @helpString),"\n";
    exit 1;
}

## print out the version
##
if(defined($optctl{'version'})){
    require 'digestFmg.pl';
    require 'digestSngen.pl';
    print STDOUT 
        "$progname\n\t$revision\n",
        "\t$version\n";
    exit 1;
}

if((! defined($optctl{'networksumm'})) && (! defined($optctl{'customsupport'})) 
  && (! defined($optctl{'servopreliability'})) && (! defined($optctl{'servoptrafficsumm'})) && (! defined($optctl{'servoptotaltraffic'})) ){
    ExitOnError("***No good options provided",  
                @helpString
                );
}

my($debug) = 0;
if(defined($optctl{'debug'})){
  $debug = 1;
}
  
##
## Types defined in fr_entid.h
## 
##
my @entTypes = (
  "FRENT_TYPE_UNUSED_",   
  "FRENT_TYPE_AIP",                 # AIP servers */
  "FRENT_TYPE_FMG",               # Freedom mail gateway */
  "FRENT_TYPE_MAIP",              # anonymous mail server */
  "FRENT_TYPE_MASTER",            
  "FRENT_TYPE_NIDB",              # NIDB - NISS and NIQS */
  "FRENT_TYPE_NYM",               # Nym users */
  "FRENT_TYPE_NYM_SRV",           # Nym servers */
  "FRENT_TYPE_SER_SRV",           
  "FRENT_TYPE_TOK_SRV",           # Token servers */
  "FRENT_TYPE_KEYQRY_SRV",        # Key query server */
  "FRENT_TYPE_KEYUPD_SRV",        # Key update server */
  "FRENT_TYPE_ANON",              # Anonymous users */
  "FRENT_TYPE_NNTP_SRV",          # NNTP server */
  "FRENT_TYPE_FREEDOM_YEAR",      # Freedom high-level key */
  "FRENT_TYPE_FREEDOM_MONTH",     # Freedom high-level key */
  "FRENT_TYPE_SOFTUPD",           # Freedom software update key */
  "FRENT_TYPE_LINK",              # Keep track of stuff in the link DB */
  "FRENT_TYPE_HOST",              # Keep track of stuff in the host DB */
  "FRENT_TYPE_NIQS",              # Network information query server */
  "FRENT_TYPE_NISS",              # Network information stats server */
  "FRENT_TYPE_MAX"                # This is a placeholder for the last type.  Do
);

## variables
##

my $nidbtool = "nidbtool";
my $NIDBTOOL = "/usr/freedom/bin/nidbtool";
checkFindExecutable($NIDBTOOL);
my $prompt = "XoX>\n";
my $chompPrompt = $prompt; chomp($chompPrompt);

##
## Checks executables
##
sub checkFindExecutable {
    my($whichExec) = @_;
    my $res = `which $whichExec`;
    if($res =~ /no $whichExec in/ && ! -x "$whichExec"){
        print STDERR "***Error: $whichExec is missing\n";
        exit 0;
    }
}

##
## Send info and get the information
##
sub cmdSendGet {
  my(@cmd) = @_;
  my(@tmp)=();

  my($strToSend) = join(" ", @cmd);
  $strToSend .= "\n";
  if($cmd[0] !~ /^prompt/){
    $strToSend .= "\n";
  }
  
  my(@answer)=();
  print FDOUT "$strToSend";
  my $line = <FDIN>;
  while ($line = <FDIN>){
    chomp($line);
    last if($line =~ /^$chompPrompt/);
    if($debug == 2){chomp($strToSend); chomp($strToSend); print "cmdSendGet [", $strToSend,"] : ->$line\n";}
    push @answer, $line;
  }

  ## process @answer to get rid of breakLines in times fields

  return @answer;
}

##
## Send info and get the information
##
sub getTagLine {
  my ($tag, $array) = @_;
  my @tmpRes = grep /<$tag>.*/, @$array;
  if($#tmpRes){
    if($debug){print "getTagLine: NO LINE\n";}
    return "";
  }else{
    if($debug == 2){print "getTagLine: $tmpRes[0]\n";}
    return $tmpRes[0];
  }
}

##
## Send info and get the information
##
sub getTagLines {
  my ($tag, $array) = @_;
  my @tmpRes = grep /<$tag>.*/, @$array;
  if($debug == 2){print join("\n", @tmpRes),"\n";}
  return @tmpRes;
}

##
## Send info and get the information
##
sub getTagValueInLine {
  my ($tag, $line) = @_;

  $line =~ /\s*<$tag>\s*(.*)\s*<\/$tag>/;
  my $val = $1;
  if($debug == 2){print "getTagValueInLine: [$tag] [$line] -> $val\n";}
  return $val;
}

##
## Send info and get the information
##
sub checkNumber {
  my ($num) = @_;
  if(defined($num)){
  if($num < 0){
    print "***Error: Negative number wrap around possible\n";
  }
}else{
    $num = 0;
}
  return $num;
}


##________________________________________________________________________________________
##
##
## Main stuff
##
##
my $line;
my $pid = open2( \*FDIN, \*FDOUT, "$NIDBTOOL");
FDIN->autoflush();
FDOUT->autoflush();
my(@res)=();

##
##change the prompt for easy input
## 
@res = cmdSendGet("prompt $prompt");

##
## get list of entity
##
@res = cmdSendGet("entity list");

##
## get a lsit for aips and one for maips
##
my($entity);
shift(@res); ## discards number of entity
my @aipList = ();
my @maipList = ();
foreach $entity (@res){
  $entity =~ /\[\s*([0-9]+)\]\s+(.*)\/([0-9]+)/;
  my ($entid, $entname, $service) = ($1, $2, $3);
  if($debug){printf("%16s %16s %16s", $entid, $entname, $service);}
  if($entTypes[$service] =~ /^FRENT_TYPE_AIP$/){
    if($debug){print " -> aip service";}
    push @aipList, $entname;
  }
  if($entTypes[$service] =~ /^FRENT_TYPE_MAIP$/){
    if($debug){print " -> maip service";}
    push @maipList, $entname;
  }
  if($debug){print "\n";}
}

if (defined($optctl{'customsupport'})){
  print "<customSupport>\n";
  report("aip", \@aipList);
  report("maip", \@maipList);
  print "</customSupport>\n";
}

if ( defined($optctl{'networksumm'})) {
  summaryStatus(\@aipList, \@maipList);
}

if( defined($optctl{'servopreliability'})){
   servopReliabReport(\@aipList, \@maipList);
}

if( defined($optctl{'servoptrafficsumm'})){
  servopTrafficSumm(\@aipList, \@maipList);
}

if( defined($optctl{'servoptotaltraffic'})){
  servopTotalTraffic(\@aipList, \@maipList);
}

sub servopTotalTraffic {
  my ($aipList, $maipList) = @_;

  ## declare used varaibles
  my($aip, $nbTotalBytesRx, $nbTotalBytesTx, $nbBytes, $lastUpdateStamp, $nb, $hasMaip);
  my(@tmp)=();

  ## loop for the aip
  print "<servopTotalTraffic>\n";
  foreach $aip (@$aipList){

    ## clear value
    $nbTotalBytesRx    = 0;
    $nbTotalBytesTx    = 0;

    ## verify that aip has state
    @res = ();
    @res = cmdSendGet("entity state aip $aip");
    if(scalar @res == 1){
      print "<errorMsg>***Error: aip $aip has no state ->$res[0]</errorMsg>\n\n";
      next;
    }else{
      ## get aip information for trafficSummary
      @res = ();
      @res = cmdSendGet("report aip $aip");
      ## a bit of processing to get one tag per line
      ##
      my($tmp) = join("", @res);
      $tmp =~ s/></>\n</gi;
      @res = split("\n", $tmp);

      ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ##
      ## get lastStateDate for the aip
      ## 
      $line = getTagLine("lastStateDate", \@res);
      if($debug){print "servopTotalTraffic: [$line]\n";}
      $lastUpdateStamp = getTagValueInLine("lastStateDate", $line);
      
      ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ##
      ## get nbTotalBytesRx and  nbTotalBytesTx for the aip
      ## do we need nbTotalBytesDropped
      $line = getTagLine("nbTotalBytesRx", \@res);
      if($debug){print "servopTotalTraffic: [$line]\n";}
      $nbBytes = getTagValueInLine("nbTotalBytesRx", $line);
      checkNumber($nbBytes);
      if($debug){print "servopTotalTraffic: aip Rx $nbBytes\n";}
      $nbTotalBytesRx += $nbBytes;

      $line = getTagLine("nbTotalBytesTx", \@res);
      if($debug){print "servopTotalTraffic: [$line]\n";}
      $nbBytes = getTagValueInLine("nbTotalBytesTx", $line);
      checkNumber($nbBytes);
      if($debug){print "servopTotalTraffic: aip Tx $nbBytes\n";}
      $nbTotalBytesTx += $nbBytes;

      ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ##
      ## get nbTotalBytesRx and  nbTotalBytesTx for the maip
      ## do we need nbTotalBytesDropped
      ## need to check fist if maip in list
       ## check if the aip has also a maip running
      @tmp = ();
      @tmp = grep /^$aip$/, @$maipList;
      $nb = scalar @tmp;
      if( $nb == 0){
	$hasMaip = 0;
      }elsif ($nb == 1){
	if($debug){print "servopTotalTraffic: $aip has maip\n";}
	
	## get maip information for trafficSummary
	@res = ();
	@res = cmdSendGet("report maip $aip");
	## a bit of processing to get one tag per line
	##
	my($tmp) = join("", @res);
	$tmp =~ s/></>\n</gi;
	@res = split("\n", $tmp);
	
	$line = getTagLine("nbTotalBytesRx", \@res);
	if($debug){print "servopTotalTraffic: [$line]\n";}
	$nbBytes = getTagValueInLine("nbTotalBytesRx", $line);
	checkNumber($nbBytes);
	if($debug){print "servopTotalTraffic: maip Rx $nbBytes\n";}
	$nbTotalBytesRx += $nbBytes;

	$line = getTagLine("nbTotalBytesTx", \@res);
	if($debug){print "servopTotalTraffic: [$line]\n";}
	$nbBytes = getTagValueInLine("nbTotalBytesTx", $line);
	checkNumber($nbBytes);
	if($debug){print "servopTotalTraffic: maip Tx $nbBytes\n";}
	$nbTotalBytesTx += $nbBytes;
	
	$hasMaip = 1;
      }else {
	print "<errorMsg>**Error: $aip duplicate name in maiplist $nb</errorMsg>\n\n";
	exit;
      }
      
      print
	"<servopTotalRep>\n",
	"<entityName> $aip </entityName>\n",
	"<lastStateDate>$lastUpdateStamp</lastStateDate>\n",
	"<totalTrafficRx>$nbTotalBytesRx</totalTrafficRx>\n",
	"<totalTrafficTx>$nbTotalBytesTx</totalTrafficTx>\n",
	"</servopTotalRep>\n\n";
	
    }
  }
    print "</servopTotalTraffic>\n";
}

##
## servop Reliability Report
##
sub servopReliabReport{
  my ($aipList, $maipList) = @_;

  my($totalPkts) = 0;
  my($nbTotalPktsDropped) = 0;
  my($nbTotalPktsIn) = 0;
  my($aip, $percentDownTime);
  my($downTime,$refTime);
  my(@linkDroppedPkts) = ();
  my(@linkInPkts) = ();
  my(@tmp)=();
  my($day, $month, $mday, $hours, $min, $sec, $year);
  my $mytimelocal;
  print "<servopReliabilityReport>\n";
  foreach $aip (@$aipList){

    ## clear value
    $nbTotalPktsDropped = 0;
    $percentDownTime    = 0;
    $nbTotalPktsIn      = 0;
    $mytimelocal        = 0;

    ## verify that aip has state
    @res = ();
    @res = cmdSendGet("entity state aip $aip");
    if(scalar @res == 1){
      print "<errorMsg>***Error: aip $aip has no state ->$res[0]</errorMsg>\n\n";
      next;
    }else{
      ## need to get here downtime and the refferrence date
      ## use previous data coming from @res (entity state)
      @tmp = grep /^\s*Downtime:/, @res;
      $line = pop @tmp;
      chomp($line);
      $line =~ /^\s*Downtime:\s*([0-9]+)\s*sec/;
      $downTime = $1;

      ##
      ## ref Date Tue Oct 26 09:38:21 1999
      @tmp = grep /^\s*Reference Date:/, @res;
      $line = pop @tmp;
      chomp($line);
      if($line =~/^\s*Reference Date:\s*(\w+)\s+(\w+)\s+(\w+)\s+([0-9]+):([0-9]+):([0-9]+)\s+([0-9]+)/){
	($day, $month, $mday, $hours, $min, $sec, $year) = ($1, $2, $3, $4, $5, $6, $7, $8);
	if($debug){print "($day, $month, $mday, $hours, $min, $sec, $year)\n";}
	$mytimelocal = timegm($sec, $min, $hours, $mday, $MONTH{$month}, $year);
      }else{
	$line =~ s/\s*//;
	print "<errorMsg>$aip has no referrence time ($line)</errorMsg>\n";
      }
      if($mytimelocal){
	$percentDownTime = $downTime / (time() - $mytimelocal);
      }else{
	$percentDownTime = "N/A";
      }
      ## get aip information
      @res = ();
      @res = cmdSendGet("report aip $aip");

      ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ##
      ## get dropped Pkts
      $line = getTagLine("clientPktsDropped", \@res);
      if($debug){print "[$line]\n";}
      $totalPkts = getTagValueInLine("clientPktsDropped", $line);
      checkNumber($totalPkts);
      $nbTotalPktsDropped += $totalPkts;

      ## get all links dropped info
      ## count bytes Out
      @linkDroppedPkts=();
      @linkDroppedPkts = getTagLines("droppedPktsIn", \@res);
      foreach $line (@linkDroppedPkts){
	$totalPkts = getTagValueInLine("droppedPktsIn", $line);
	checkNumber($totalPkts);
	$nbTotalPktsDropped += $totalPkts;
	if($debug){print "[$line]\n";}
      }

      ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      ##
      ## get In Pkts
      ##
      $line = getTagLine("clientPktsRx", \@res);
      if($debug){print "[$line]\n";}
      $totalPkts = getTagValueInLine("clientPktsRx", $line);
      checkNumber($totalPkts);
      $nbTotalPktsIn += $totalPkts;

      ## get all links dropped info
      ## count bytes Out
      @linkInPkts=();
      @linkInPkts = getTagLines("currThroughputPktsIn", \@res);
      foreach $line (@linkInPkts){
	$totalPkts = getTagValueInLine("currThroughputPktsIn", $line);
	checkNumber($totalPkts);
	$nbTotalPktsIn += $totalPkts;
	if($debug){print "[$line]\n";}
      }
    }

    my ($errorMsg) = "";
    my ($percentDropped)= 0;
    if($debug){
      print "$nbTotalPktsDropped / $nbTotalPktsIn\n";
    }
    if(! $nbTotalPktsIn){
      if($nbTotalPktsDropped ){
	$errorMsg = "<errorMsg>***Error: dropped Pkts $nbTotalPktsDropped greater than In Pkts $nbTotalPktsIn </errorMsg>\n";
      }else{
	$percentDropped = 0;
      }
    }else{
      if ($nbTotalPktsIn < $nbTotalPktsDropped ){
	$errorMsg = "<errorMsg>***Error: dropped Pkts $nbTotalPktsDropped greater than in Pkts $nbTotalPktsIn </errorMsg>\n";
      }else{
	$percentDropped = $nbTotalPktsDropped / $nbTotalPktsIn;
      }
    }

    ## printout the report
    ##
    print 
      "<reliabilityRep>\n",
      "<entityName>$aip</entityName>\n",
      "<percentDownTime>$percentDownTime</percentDownTime>\n";
    if($errorMsg){
      print $errorMsg;
    }else{
      print "<droppedPkts> ", $percentDropped, "</droppedPkts>\n";
    }
    print "</reliabilityRep>\n\n";
  }
  print "</servopReliabilityReport>\n";
  return;
}

##
## traffic summary report for all servop
##
sub servopTrafficSumm {
  my($aipList, $maipList) = @_;

  my(@tmp) = ();
  my($hasMaip) = 0;
  my($aip,$nb);
  my($nbTotalBytesRx) = 0;
  my($nbTotalBytesTx) = 0;
  my($line);
  my($totalBytes) = 0;
  my(@linkThroughput)=();

  my $aipTotalKbsRx=0;
  my $aipTotalKbsTx=0;
  my $maipTotalKbsRx=0;
  my $maipTotalKbsTx=0;
  my $aipTimePeriod=0;
  my $maipTimePeriod=0;
  my $nbTotalBytesLinkIn=0;
  my $nbTotalBytesLinkOut=0;
  my $currentTotalKbpsRx = 0;
  my $currentTotalKbpsTx = 0;

  print "<servopTrafficSumm>\n";
  foreach $aip (@$aipList){
    ## clear value
    $nbTotalBytesRx      = 0;
    $nbTotalBytesTx      = 0;
    $nbTotalBytesLinkIn  = 0;
    $nbTotalBytesLinkOut = 0;
    $aipTotalKbsRx=0;
    $aipTotalKbsTx=0;
    $maipTotalKbsRx=0;
    $maipTotalKbsTx=0;
    $currentTotalKbpsRx = 0;
    $currentTotalKbpsTx = 0;

    ## verify that aip has state
    @res = ();
    @res = cmdSendGet("entity state aip $aip");
    if(scalar @res == 1){
      print "<errorMsg>***Error: aip $aip has no state ->$res[0]</errorMsg>\n\n";
      next;
    }else{
      
      ## get aip information for trafficSummary
      @res = ();
      @res = cmdSendGet("report aip $aip");
      
      $line = getTagLine("clientBytesRx", \@res);
      if($debug){print "servopTrafficSumm: [$line]\n";}
      $totalBytes = getTagValueInLine("clientBytesRx", $line);
      checkNumber($totalBytes);
      $nbTotalBytesRx  += $totalBytes;

      $line = getTagLine("clientBytesTx", \@res);
      if($debug){print "servopTrafficSumm: [$line]\n";}
      $totalBytes = getTagValueInLine("clientBytesTx", $line);
      checkNumber($totalBytes);
      $nbTotalBytesTx += $totalBytes;

      $line = getTagLine("wormholeBytesRx", \@res);
      if($debug){print "servopTrafficSumm: [$line]\n";}
      $totalBytes = getTagValueInLine("wormholeBytesRx", $line);
      checkNumber($totalBytes);
      $nbTotalBytesRx += $totalBytes;

      $line = getTagLine("wormholeBytesTx", \@res);
      if($debug){print "servopTrafficSumm: [$line]\n";}
      $totalBytes = getTagValueInLine("wormholeBytesTx", $line);
      checkNumber($totalBytes);
      $nbTotalBytesTx += $totalBytes;
      
      ## get the currentTotalKbs
      ## get first link throughputbytesout
      ## count bytes Out
      @linkThroughput=();
      @linkThroughput = getTagLines("curThroughputBytesOut", \@res);
      foreach $line (@linkThroughput){
	$totalBytes = getTagValueInLine("curThroughputBytesOut", $line);
	checkNumber($totalBytes);
	$nbTotalBytesLinkOut += $totalBytes;
	if($debug){print "servopTrafficSumm: [$line]\n";}
      }

      ## get the currentTotalKbs
      ## get first link throughputbytesout
      ## count bytes Out
      @linkThroughput=();
      @linkThroughput = getTagLines("curThroughputBytesIn", \@res);
      foreach $line (@linkThroughput){
 	$totalBytes = getTagValueInLine("curThroughputBytesIn", $line);
 	checkNumber($totalBytes);
 	$nbTotalBytesLinkIn += $totalBytes;
 	if($debug){print "servopTrafficSumm: [$line]\n";}
      }


      ## here we get the time period of the stats
      ## to get the actual Kbps for this maip computed at the end
      ##
      $line = getTagLine("timePeriod", \@res);
      $aipTimePeriod = getTagValueInLine("timePeriod", $line);

      ##
      ## totalize the traffic for the aipp
      ## all + half on the links / 2
      $aipTotalKbsRx = $nbTotalBytesRx + $nbTotalBytesLinkIn;
      $aipTotalKbsTx = $nbTotalBytesTx + $nbTotalBytesLinkOut;

      ## check if the aip has also a maip running
      @tmp = ();
      @tmp = grep /^$aip$/, @$maipList;
      $nb = scalar @tmp;
      if( $nb == 0){
	$hasMaip = 0;
      }elsif ($nb == 1){
	if($debug){print "servopTrafficSumm: $aip has maip\n";}
	$maipTotalKbsRx=0;
	$maipTotalKbsTx=0;

	## get maip information for trafficSummary
	@res = ();
	@res = cmdSendGet("report maip $aip");
	
	$line = getTagLine("maipBytesRx", \@res);
	if($debug){print "servopTrafficSumm: [$line]\n";}
	$totalBytes = getTagValueInLine("maipBytesRx", $line);
	$totalBytes = checkNumber($totalBytes);
	$maipTotalKbsRx = $totalBytes;

	$line = getTagLine("maipBytesTx", \@res);
	if($debug){print "servopTrafficSumm: [$line]\n";}

	$totalBytes = getTagValueInLine("maipBytesTx", $line);
	$totalBytes = checkNumber($totalBytes);
	$maipTotalKbsTx = $totalBytes;
	
	## here we get the time period of the stats
	## to get the actual Kbps for this maip computed at the end
	##
	$line = getTagLine("timePeriod", \@res);
	$maipTimePeriod = getTagValueInLine("timePeriod", $line);
	
	$hasMaip = 1;
      }else {
	print "<errorMsg>**Error: $aip duplicate name in maiplist $nb</errorMsg>\n\n";
	exit;
      }
    }

    ## here need to summ all the $aip response to get the total
    ## Kbps
    if( $aipTimePeriod > 0){
      $currentTotalKbpsRx = ($aipTotalKbsRx / $aipTimePeriod);
      $currentTotalKbpsTx = ($aipTotalKbsTx / $aipTimePeriod);
    }

    if( $maipTimePeriod > 0){
	$currentTotalKbpsRx +=  ($maipTotalKbsRx / $maipTimePeriod);
	$currentTotalKbpsTx +=  ($maipTotalKbsTx / $maipTimePeriod);
    }
    
    ## maybe here need to do a time computation to get
    ## nbBytes / period of time
    ## tricky because we dont have the same time period for aip and maip

    ## printout the report
    ##
    print 
      "<trafficRep>\n",
      "<entityName>$aip</entityName>\n",
#      "<profiledKbps>N/A</profiledKbps>\n",
      "<currentKbpsIn>$currentTotalKbpsRx</currentKbpsIn>\n",
      "<currentKbpsOut>$currentTotalKbpsTx</currentKbpsOut>\n",
      "</trafficRep>\n\n";

  }
  print "</servopTrafficSumm>\n";

  return;
}

##
## summaryStatus Report
##
sub summaryStatus{
  my($aipList, $maipList) = @_;
  my $aip;
  my $aipUp = 0;
  my $aipDown = 0;
  my $aipUnknown = 0;
  my $currentTotalKbs=0;
  my $nbTotalActiveStreams=0;
  my $whormholeBytesRx=0;
  my $whormholeBytesTx=0;
  my $currConnectedClient = 0;
  my @tmpRes=();my $status;
  my $line;
  my(@res)=();
  my $aipTotalKbs=0;
  my $maipTotalKbs=0;
  my $aipTimePeriod=0;
  my $maipTimePeriod=0;

  print 
      "<networkSummary>\n";

  foreach $aip (@$aipList){
    $aipTotalKbs=0;
    $maipTotalKbs=0;

    ##________________________________________
    ##
    ## get the status of the Aip
    @res = cmdSendGet("entity state aip $aip");
    if(scalar @res == 1){
      print "<errorMsg>***Error: aip $aip has no state ->$res[0]</errorMsg>\n\n";
      next;
    }else{
#      if($debug){print join("\n", @res);}
      @tmpRes = grep /\s*Up\sflag:/, @res;
      
      if($debug){print "Get Info for aip: $aip $tmpRes[0]\n";}
      if($tmpRes[0] =~ /\s*Up\sflag:\s+(.*)\s*/){
	$status = $1;
	if($status =~ /Up/){
	  $aipUp++;
	} elsif ( $status =~ /Down/ ){
	  $aipDown++;
	} else {
	  $aipUnknown++;
	}
      }else{
	print "bad\n";
      }
    }
    
    ## get the report for the aip
    @res = cmdSendGet("report aip $aip"); #"
    
    ## get the nbTotalActiveStreams
    $line = getTagLine("nbCurrentClientConnections", \@res);
    $currConnectedClient += getTagValueInLine("nbCurrentClientConnections", $line);

    ## get the nbTotalActiveStreams
    $line = getTagLine("nbCurrentActiveStreams", \@res);
    $nbTotalActiveStreams += getTagValueInLine("nbCurrentActiveStreams", $line);
    
    ## get the wormholeBytesRx
    $line = getTagLine("wormholeBytesRx", \@res);
    $whormholeBytesRx += getTagValueInLine("wormholeBytesRx", $line);
    
    ## get the wormholeBytesTx
    $line = getTagLine("wormholeBytesTx", \@res);
    $whormholeBytesTx += getTagValueInLine("wormholeBytesTx", $line);
    
    ## get the currentTotalKbs
    ## get first link throughputbytesout
    my @linkThroughput = getTagLines("curThroughputBytesOut", \@res);
    foreach $line (@linkThroughput){
      if($debug){print "summaryStatus: $line\n";}
      $aipTotalKbs += getTagValueInLine("curThroughputBytesOut", $line);
    }
    
    $line = getTagLine("clientBytesRx",  \@res);
    if($debug){print "summaryStatus: $line\n";}
    $aipTotalKbs += getTagValueInLine("clientBytesRx", $line);
    
    $line = getTagLine("clientBytesTx",  \@res);
    if($debug){print "summaryStatus: $line\n";}
    $aipTotalKbs += getTagValueInLine("clientBytesTx", $line);
    
    $line = getTagLine("wormholeBytesRx",  \@res);
    if($debug){print "summaryStatus: $line\n";}
    $aipTotalKbs += getTagValueInLine("wormholeBytesRx", $line);
    
    $line = getTagLine("wormholeBytesTx",  \@res);
    if($debug){print "summaryStatus: $line\n";}
    $aipTotalKbs += getTagValueInLine("wormholeBytesTx", $line);
    if($debug){print "summaryStatus: Bytes passed $aipTotalKbs\n";}

    ## here we get the time period of the stats
    ## to get the actual Kbps for this aip computed at the end
    ##
    $line = getTagLine("timePeriod", \@res);
    $aipTimePeriod = getTagValueInLine("timePeriod", $line);
    if($debug){print "summaryStatus: $line\n";}
    if($debug){print "summaryStatus: aip timeperiod $aipTimePeriod\n";}

    ## here need to get the maip Kbps
    ## and the time period
    ##
    ## check if the aip has also a maip running
    @res = ();
    @res = grep /^$aip$/, @$maipList;
    my $nb = scalar @res;
    my $hasMaip = 0;
    if( $nb == 0){
      $hasMaip = 0;
    }elsif ($nb == 1){
      
      if($debug){print "servopTrafficSumm: $aip has maip\n";}
      
      ## get maip information for trafficSummary
      @res = ();
      @res = cmdSendGet("report maip $aip");
      
      $line = getTagLine("maipBytesRx", \@res);
      if($line){
	  $maipTotalKbs = getTagValueInLine("maipBytesRx", $line);
      }

      $line = getTagLine("maipBytesTx", \@res);
      if($line){
	  $maipTotalKbs += getTagValueInLine("maipBytesTx", $line);
      }

      ## here we get the time period of the stats
      ## to get the actual Kbps for this maip computed at the end
      ##
      $line = getTagLine("timePeriod", \@res);
      if($debug){print "summaryStatus: $line\n";}
      if($line){
	  $maipTimePeriod = getTagValueInLine("timePeriod", $line);
	  if($debug){print "summaryStatus: maip timeperiod $maipTimePeriod\n";}
      }

      $hasMaip = 1;
    }else {
      print "<errorMsg>**Error: $aip duplicate name in maiplist $nb</errorMsg>\n\n";
      exit;
    }

    ## here need to summ all the $aip response to get the total
    ## Kbps
    if( $aipTimePeriod > 0){
      if($debug){print "aip OK $aipTotalKbs / $aipTimePeriod", ($aipTotalKbs / $aipTimePeriod),"\n";}
      $currentTotalKbs += ($aipTotalKbs / $aipTimePeriod);      ## add Aip Kbps
    }

    ## here need to summ all the $aip response to get the total
    ## Kbps
    if( int($maipTimePeriod) > 0){
      if($debug){print "maip OK $maipTotalKbs / $maipTimePeriod", ($aipTotalKbs / $aipTimePeriod),"\n";}
      $currentTotalKbs += ($maipTotalKbs / $maipTimePeriod ); ##add Maip Kbps
    }
  }


  ## print the summary
  print 
    "<serversUp>$aipUp</serversUp>\n",
    "<serversDown>$aipDown</serversDown>\n",
    "<nymNumber>N/A</nymNumber>\n",
    "<connectedClient>$currConnectedClient</connectedClient>\n",
    "<nbTotalActiveStreams>$nbTotalActiveStreams</nbTotalActiveStreams>\n",
    "<currentTotalKbs>$currentTotalKbs</currentTotalKbs>\n",
    "<whormholeBytesRx>$whormholeBytesRx</whormholeBytesRx>\n",
    "<whormholeBytesTx>$whormholeBytesTx</whormholeBytesTx>\n",
    "</networkSummary>\n";
}

##
## for each in the list get the server operator info
##
sub report {
  my($type, $aipList) = @_;
  my $aip;
  my @tmpRes=();
  my $total = 0;
  foreach $aip (@$aipList){
    $total = 0;
    @tmpRes=();
    print "<$type>\n";
    print "<$type","Entity>$aip</$type","Entity>\n";

    ##________________________________________
    ##
    ## get the status of the Aip
    @res = ();
    @res = cmdSendGet("entity state $type $aip");
    if(scalar @res == 1){
      print "<errorMsg>***Error: aip $aip has no state ->$res[0]</errorMsg>\n";
    }else{
      @tmpRes = grep /\s*Up\sflag:/, @res;
    
      if($debug){print "Get Info for aip: $aip $tmpRes[0]\n";}
      if($tmpRes[0] =~ /\s*Up\sflag:\s+(.*)/){
	print "<status>$1</status>\n";
      }else{
	print "<status><errorMsg>N/A</errorMsg></status>\n";
      }
    
      ##________________________________________
      ##
      ## get the version uptime total traffic
      ## <uptime> Wed Dec 31 19:00:00 1969
      ## <release> 1999102116 Linux chablis 2.2.10 i686 nicolas </release>
      ## <nbTotalBytesRx> 225 </nbTotalBytesRx>
      ## <nbTotalBytesTx> 536 </nbTotalBytesTx>
      
      @res = cmdSendGet("report $type $aip");
      ## a bit of processing to get one tag per line
      ##
      my($tmp) = join("", @res);
      $tmp =~ s/></>\n</gi;
      @res = split("\n", $tmp);
      
      ##________________________________________
      ##
      ## get the version of the Aip
      if($debug){print join("\n", @res);}
      $line = getTagLine("release", \@res);
      print "$line\n";
      
      ##________________________________________
      ##
      ## get the uptime of the Aip
      if($debug){print join("\n", @res);}
      $line = getTagLine("uptime", \@res);
      print "$line\n";
      
      if($type =~ /^aip$/){
      ##________________________________________
      ##
      ## get the totalrx of  the Aip
	if($debug){print join("\n", @res);}
	$line = getTagLine("nbTotalBytesRx", \@res);
	if($debug){print "$line\n"};
	$total += getTagValueInLine("nbTotalBytesRx", $line);
	#    $line =~ /\s*<nbTotalBytesRx>\s*(.*)\s*<\/nbTotalBytesRx>/;
	#    $total += $1;
	
	##________________________________________
	##
	## get the totaltx of the Aip
	if($debug){print join("\n", @res);}
	$line = getTagLine("nbTotalBytesTx", \@res);
	if($debug){print "$line\n";}
	$total += getTagValueInLine("nbTotalBytesTx", $line);
	
      }elsif($type =~ /^maip$/){
	##________________________________________
	##
	## get the totalrx of  the Aip
	if($debug){print join("\n", @res);}
	$line = getTagLine("nbTotalBytesRx", \@res);
	if($debug){print "$line\n"};
	$total += getTagValueInLine("nbTotalBytesRx", $line);
	#    $line =~ /\s*<nbTotalBytesRx>\s*(.*)\s*<\/nbTotalBytesRx>/;
	#    $total += $1;
	
	##________________________________________
	##
	## get the totaltx of the Aip
	if($debug){print join("\n", @res);}
	$line = getTagLine("nbTotalBytesTx", \@res);
	if($debug){print "$line\n";}
	$total += getTagValueInLine("nbTotalBytesTx", $line);
      }else{
	print "<errorMsg> report in[__FILE__:__LINE__] type not found </errorMsg>\n";
      }

	#    $line =~ /\s*<nbTotalBytesTx>\s*(.*)\s*<\/nbTotalBytesTx>/;
	#    $total += $1;
      print "<totalTraffic>$total</totalTraffic>\n";
    }
    print "</$type>\n\n";
  }
}


print FDOUT "quit\n";

close(FDIN);
close(FDOUT);
1;

#   $Log: frgetcustomersupport.pl,v $
#   Revision 1.16  2000/11/24 15:03:10  marcp
#   Merge from scarecrow_dev
#
#   Revision 1.15.22.1  2000/10/11 13:19:52  phil
#
#   License header additions in preparation for the source release.
#
#   Revision 1.15  1999/12/02 22:31:35  thierry
#   	avoid warning when no maip information is present
#
#   Revision 1.14  1999/12/01 21:24:11  thierry
#   	- correction of last error message tag
#   	- print <networkSummary> opening tag before
#   	  any other output (avoid to failed on web server query)
#
#   Revision 1.13  1999/11/17 21:01:10  thierry
#
#   	- closing tags for error Messages  in Xml
#   	- count total bytes of the maip (servopTotalTraffic, report fuctions)
#
#   Revision 1.12  1999/11/08 22:26:36  thierry
#   Bug #1293
#       fix frgetcustomersupport to avoid bad xml
#           in reporting for maip
#       [Peer: Luc]
#
#   Revision 1.11  1999/10/29 23:09:06  thierry
#   	Bug #1105
#
#   	report traffic Kbps in and Kbps Out
#   	[Peer luc]
#
#   Revision 1.10  1999/10/29 04:41:43  luc
#   991028 RC0 patches merged into the V1.0 trunk
#
#   Revision 1.9.2.1  1999/10/28 18:33:54  thierry
#   Bug #1113
#       * modified to get report subroutine to report correctly for maip
#
#       - change cmd send to nidbtool to send report aip and maip
#   						     ^^^
#
#       - add code to correctly count the maip bytesRx maipBytesTx
#
#   Revision 1.9  1999/10/28 02:54:35  thierry
#   * Bug #1098
#
#     servopTotalTraffic
#     - change lastUpdateDate to lastStateDate in
#     - compute Kbps using timePeriod of the stat
#
#   * Bug #1099
#     servopReliabReport
#     - percent Downtime not present in
#     - need to add day month to numeric
#     - get and convert the referrence date
#     - get number of seconds of down time
#
#   * Bug #1100
#     servopTrafficSumm
#     - compute Kbps for the aip and maip
#
#   	(client + wormhole + (link + link out ) / 2
#
#   * Bug #1101
#      summaryStatus
#      - in summary status add maip count to
#      - compute Kbps using timePeriod of the stat
#
#
#   [Peer: luc]
#
#   Revision 1.8  1999/10/26 22:56:58  thierry
#   	(related to previous ci)
#   	* new report type added to report total traffic per entity
#   	  #bug 1048
#
#   	[peer: luc]
#
#   	*this import remove relative path for nidb tool
#
#   Revision 1.7  1999/10/26 22:29:41  freedom
#   add totaltraffic report
#
#   Revision 1.6  1999/10/26 20:25:10  thierry
#   update fields to avoid confusion with DTD
#
#   Revision 1.5  1999/10/26 15:29:20  thierry
#   finalize servop[summary/reliability] report
#
#   Revision 1.4  1999/10/26 00:25:39  thierry
#   add servopreliability and servoptrafficsumm reports
#
#   Revision 1.3  1999/10/22 20:11:35  thierry
#   remove absolute path to nidbtool
#
#   Revision 1.2  1999/10/22 19:37:05  thierry
#   compute now total Kbs
#
#   Revision 1.1  1999/10/22 19:06:02  thierry
#   customer support and network summary status
#
