
=head1

 #
 #************************************************************************
 #
 #   File:		nidbtoolConnect.pm
 #
 #   Author(s):		Thierry Baron
 #
 #   Created:		16 May 2000
 #
 #   Last Revision:	$Date: 2000/11/24 15:03:10 $
 #
 #   Description:	
 #
 #   $Revision: 1.3 $
 #
 #
 #   $Id: nidbtoolConnect.pm,v 1.3 2000/11/24 15:03:10 marcp Exp $	
 #
 #   Copyright (c) 2000 Zero-Knowledge Systems, Inc
 #   Please see the copyright notice included in this distribution 
 #   for full details.
 #
 #***********************************************************************/

=cut 
package nidbtoolConnect;

=head1 DESCRIPTION

  This package helps in establishing a bidirectional pipe 
connection to nidbtool. This can be use to establish 
a connection to the nidbtool and perform a set of operations.

This package contain three basic operation :

=item - connect

to open a conection with nidbtool

=item - disconnect

to close an open connection

=item - cmdSendGet 

send a comment and return an array containing whta was propmt by nidbtool

_______________________________________________________________

=cut


use strict;
use FileHandle;
use IPC::Open3;

my $prompt = "XoX>\n";
chomp( my $chompPrompt = $prompt );
my $DEBUG = 0;

=head2 connect

    pid = connect (fdin, fdout, cmd);

This routine create a bidirectional pipe to the cmdline program and returns
the pid of the child process. fdin and fdout should be valid filehandle 
referrences obtained using the new filehandle object (see perldoc FileHandle).
By doing the connection fdin and fdout will be attached to the cmd. We are 
changing the prompt to a new one "XoX>\n\n" to ease the read access.

Parameter List:

    FileHandleRef fdin: 

    valid filehandle ref from where to read. If nidbConnect 
    suceed, user will be able to read lines from it.


    FileHandleRef fdout: 

    valid filehandle ref where to write. If nidbConnect 
    suceed, user will be able to write commands to it

    String          cmd: 

    cmd to attach to. This could be directly nidbtool, 
    or could also be a remote nidbtool using an ssh channel

_______________________________________________________________

=cut

##
## connect to nidbtool with bidirectional pipe
##
sub connect {
    my($FDIN, $FDOUT, @cmdIn) = @_;
    my $user = `whoami`;

    my $cmd = join(" ", @cmdIn);
    my $pid = IPC::Open3::_open3( 'connect', undef, $FDOUT, $FDIN, $FDIN, "$cmd");
    if($DEBUG){
	print "FDIN : ", fileno($FDIN),"\n";
	print "FDOUT:",  fileno($FDOUT),"\n";
	print "$pid\t$FDIN\n\t$FDOUT\n$cmd\n";
    }
    $FDIN->autoflush();
    $FDOUT->autoflush();
    $cmd = "prompt $prompt";  
    chomp($user);
    print "$user\'s password:"; 
#    my @tmpRes = cmdSendGet($FDIN, $FDOUT, $cmd);    
    my $tmpRes = cmdSendGet($FDIN, $FDOUT, $cmd);         
    if(2 == $tmpRes){
      print "Error: Failed to open a ssh connection \n";
    }    
    return $pid;
}


=head2 disconnect

    disconnect (fdin, fdout);

This routine close the bidirectional pipe to the cmdline program and returns.
fdin and fdout should be valid filehandle referrences obtained using the 
new filehandle object (see perldoc FileHandle) and used in nidbConnect. 
By disconnecting quit command is send to smdline program and fdin and fdout 
are closed.

Parameter List:

    FileHandleRef fdin: valid filehandle ref from where to read.

    FileHandleRef fdout: valid filehandle ref where to write.

_______________________________________________________________

=cut

##
## close the connection
##
sub disconnect {
    my($FDIN, $FDOUT) = @_;
    print $FDOUT "quit\n";
    close($FDOUT);
    close($FDIN);
}


=head2 cmdSendGet

    res = cmdSendGet (fdin, fdout, cmd);

This routine send a command to the pipe and return the result of the 
command if any in an array. fdin and fdout should be valid filehandle 
referrences obtained using the new filehandle object (see perldoc FileHandle) 
and connected to the bidirectioanl pipe by nidbConnect. 

Parameter List:

    FileHandleRef fdin: valid filehandle ref from where to read.

    FileHandleRef fdout: valid filehandle ref where to write.

    String          cmd: what is the command to be send

    Array           res: what was prompted by nidbtool


_______________________________________________________________

=cut

##________________________________________________________________
##
## Send/Get lines
##
##
sub cmdSendGet {
  my($FDIN, $FDOUT, @cmd) = @_;
  my(@tmp)=();

  my($strToSend) = join(" ", @cmd);
  $strToSend .= "\n";
  if($cmd[0] !~ /^prompt/){
    $strToSend .= "\n";
  }
  
  my(@answer)=();
  $FDOUT->print("$strToSend");
  my $line = $FDIN->getline();
  while ($line = $FDIN->getline()){
    chomp($line);
    last if($line =~ /^$chompPrompt/);
    if($DEBUG){chomp(my $tmp= $strToSend); chomp($tmp); print "cmdSendGet [", $tmp,"] : ->$line\n";}
    push @answer, $line;
  }
  return @answer;
}

1;

