/*
 * Network Information DataBase
 *
 * Handling of Aip update packets
 *
 * $Id: aip_update_pkts.c,v 1.9 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#include "zks.h"

#include <string.h>
#include <zkDateTime.h>
#include <zkConfig.h>
#include <zkConfProto.h>
#include <zkFile.h>
#include <zkError.h>
#include <zkDebug.h>
#include <frNIDB.h>
#include "nidb_priv.h"


#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

int FrNIDBAipDataPktNew (ZkError err, FrNIDBAipDataPkt *adp)
{
  static const char *funcname = "FrNIDBAipDataPktNew";

  if (!adp)
    return 0;
  zkMALLOC (*adp, FrNIDBAipDataPkt,
	    sizeof (struct T_FrNIDBAipDataPkt));
  if (!zkMemPoolNew(err, &((*adp)->mpool)))
    return 0;
  zkMALLOC ((*adp)->aipinfo, FrNIDBAipUpdateInfo,
	    sizeof (struct T_FrNIDBAipUpdateInfo));
  if (!zkMemPoolRegister(err,(*adp)->mpool, (*adp)->aipinfo,
			 sizeof (struct T_FrNIDBAipUpdateInfo)))
    return 0;
  
  zkMALLOC ((*adp)->linkinfo, FrNIDBLinkUpdateInfo,
	    sizeof (struct T_FrNIDBLinkUpdateInfo));
  if (!zkMemPoolRegister(err,(*adp)->mpool, (*adp)->linkinfo,
			 sizeof (struct T_FrNIDBLinkUpdateInfo)))
    return 0;
  (*adp)->numlinks = 1;
  return 1;
}

int FrNIDBAipDataPktFree (ZkError err, FrNIDBAipDataPkt *adp)
{
  static const char *funcname = "FrNIDBAipDataPktFree";
  (void) funcname;
  (void) err;
  
  if (!adp || !*adp)
    return 0;

  zkMemPoolFree(err, &((*adp)->mpool));

  if(*adp)
    free(*adp);
  
  return 1;
}

#if 0
#define INI_DONE -1
#define INI_INT  0
#define INI_UINT 1
#define INI_BOOL 2
#define INI_CHAR 3
#define INI_UINT16 4

struct T_IniSect {
  int16_t  id;
  int16_t  type;
  const char *token;
  uint16_t  count;      /* 1  means there will be only one entry */
			/* 0  means there are more than one      */
};
typedef struct T_IniSect *IniSect;

struct T_Ini {
  const char *name;
  IniSect *desc;
};

typedef struct T_IniSections *IniSections;

#define T_LINK_ID       0
#define T_LINK_AIP      1
#define T_LINK_COST     2
#define T_LINK_ISUP    	3
#define T_LINK_PKTSNT	4
#define T_LINK_PKTRCV	5
#define T_LINK_DONE     -1

struct T_IniSect LinksIniProto[] = {
  {T_LINK_ID, INI_INT, "linkid", 1},
  {T_LINK_AIP, INI_INT, "aip", 1},
  {T_LINK_ISUP, INI_BOOL, "isup", 1},
  {T_LINK_PKTSNT, INI_INT, "pktssent", 1},
  {T_LINK_PKTRCV, INI_INT, "pktsrcvd", 1},
  {T_LINK_DONE, INI_DONE, "", 0}
};

static int getIniIndexEntry (struct T_IniSect *inisect, const char *token,
			     int *id)
{
  int cnt = -1;

  while (inisect && inisect->token) {
    cnt ++;
    if (strncasecmp(token, inisect->token, strlen(token)+1) == 0) {
      *id = inisect->id;
      break;
    }
    inisect++;
  }
  if (cnt < 0)
    return 0;
  return 1;
}

static int LinkStoreData(ZkError err,  FrNIDBLinkUpdateInfo lnk,
			 const char *token, void *ptr)
{
  static const char *funcname = "LinkStoreData";
  int id = -2;
  
  if (!getIniIndexEntry (&LinksIniProto[0], token, &id)) {
    return 0;
  }
  if (lnk && ptr) {
    switch (id) {
    case T_LINK_ID:
      lnk->linkid = *(int *)ptr;
      break;
    case T_LINK_COST:
      lnk->cost = *(int *)ptr; 
      break;
    case T_LINK_ISUP:
      lnk->isUp = *(int *)ptr; 
      break;
#warning fix dates      
    case T_LINK_PKTSNT:
      lnk->pktsSent = *(int *)ptr; 
      break;
    case T_LINK_PKTRCV:
      lnk->pktsRcvd = *(int *)ptr; 
      break;
    case T_LINK_DONE:
      break;
    default:
      zkDebug_va("Couldn't determine structure element %d for [%s]", id, token);
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKERROR, "Couldn't determine structure element");
      return 0;
    }
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "Null pointer");
    return 0;
  }
  return 1;
}
static int configLoadLinkEntry(ZkError err, ZkConfig conf,
			       const char *section,
			       FrNIDBLinkUpdateInfo lnks)
{
  static const char *funcname = "frNCDBLoadLinkEntry";
  
  static int True = 1;
  static int False = 0;
  IniSect proto = NULL;
  const char *data = NULL;
  int val= 0;
  void *ptr = NULL;

  proto=&LinksIniProto[0];

  while (proto && proto->type != INI_DONE) {
    if (zkConfigGet(err, conf, section, proto->token, &data)) {
      /* extract the data according to the type */
      switch (proto->type) {
      case INI_INT:
	if (sscanf (data, "%d", &val) != 1) {  /* audit: ok */
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert uint");
	  return 0;
	}       
	break;
      case INI_BOOL:
	ptr = (void *) &val;
	if (strncasecmp ("true", data, 5) == 0) {
	  (void) memcpy(ptr, &True, sizeof(int));
	} else if (strncasecmp ("false", data, 6) == 0) {
	  (void) memcpy(ptr, &False, sizeof(int));
	} else {
	  zkErrorSet(err, funcname, ZKERR, ZKMOD,
		     ZKERROR, "Couldn't convert bool");
	  return 0;
	}
	break;
      default:
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't determine conversion type");
	return 0;
      }
      /* Integrate this data into the Route table */
      if (!LinkStoreData(err, lnks, proto->token, (void *)&val)) {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Couldn't store Route data");
	return 0; 
      }
    }
    proto ++;
  }
  return 1;
}
#endif
/**
 * Parse and AIP update buffer.
 *
 * Caution: aipdata->aipinfo must be available before this function is called
 * Caution: aipdata->linkinfo must be available before this function is called
 *
  [AIP]
  AIPEntityName = dogbert
  SerialNumber = 54
  DateGenerated = 19990122T200400Z
  Hostname = dogbert
  IP = 192.168.1.30
  
  [Link]
  LinkEntityName = gehenna
  IsUp = false
  PktsSent = 4166
  PktsRcvd = 0
 */
int frNIDBLoadAIPUpdateBuffer(ZkError err, FrNIDBAipDataPkt adp,
			      char *buffer, size_t bufsize)
{
  static const char *funcname = "frNIDBLoadAIPUpdateBuffer";
  
  struct T_ZkConfProtoSection aipProtoSect[] = {
    {"AipEntityName",	zkCvtStr,    ZKCP_REQ, &adp->aipinfo->entity, NULL},
    {"SerialNumber",	zkCvtInt,    ZKCP_REQ, &adp->aipinfo->serialnumber, NULL},
    {"DateGenerated",	zkCvtZkTime, ZKCP_REQ, &adp->aipinfo->dategen, NULL},
    {"Hostname",	zkCvtStr,    ZKCP_REQ, &adp->aipinfo->hostname, NULL},
    {"IP",		zkCvtStr,    ZKCP_REQ, &adp->aipinfo->ipaddr, NULL},
    {"", NULL, ZKCP_OPT, NULL, NULL}
  };

  struct T_ZkConfProtoSection linkProtoSect[] = {
    {"LinkEntityName",	zkCvtStr,    ZKCP_REQ, &adp->linkinfo->remoteEntity, NULL},
    {"IsUp", 		zkCvtBool,   ZKCP_REQ, &adp->linkinfo->reportingAipEntity, NULL},
    {"PktsSent",	zkCvtInt,    ZKCP_REQ, &adp->linkinfo->pktsSent,NULL},
    {"PktsRcvd",	zkCvtInt,    ZKCP_REQ, &adp->linkinfo->pktsRcvd,NULL},
    {"", NULL, ZKCP_OPT, NULL, NULL}
  };

  
  struct T_ZkConfProtoS conf_proto[] = {
    {"AIP", aipProtoSect, 0,  NULL, ZKCP_SINGLE},
    {"Link",linkProtoSect, 0, NULL, ZKCP_SINGLE},
    {"", 0, 0, NULL, 0}
  };

  (void) funcname;

  if (!zkConfProtoLoadFromBuffer (err, buffer, bufsize, conf_proto,
				  adp->mpool)) {
    if (zkErrorCode(err) == ZKENOTFOUND) {
      zkDebug_va ("Config file error: [%s]", zkErrorDetail(err));
    } else {
      zkDebug_va ("ConfLoadFile failed");
    }
    return 0;
  }

  if (!frNIDBAipDataPktDump(err, adp))
    return 0;

#if 0  
  if (!zkConfigNew(err,&(adp->config))) {
    /* Bug: cleanup */
    return 0;
  }
  
  if (!zkConfigLoadFromBuffer(err, adp->config, buffer, bufsize)) {
    /* Bug: cleanup */
    return 0;
  }
  if (!zkConfigSections(err,adp->config, &numsections)) {
    /* Bug: cleanup */
    return 0;
  }
  /* Count the links */
  for (i = 0; i < numsections; i++) {
    const char *name;
    if (zkConfigSection(err,adp->config, i, &name)) {
      zkDebug_va("%s", name);
      if (strncasecmp ("link", name, 4) == 0) {
	numlinks += 1;
      }
    } else {
      return 0;
    }
  }
    
  if (numlinks) {
    if (!(adp->linkinfo = (FrNIDBLinkUpdateInfo)
	calloc (numlinks+1,
		sizeof (struct T_FrNIDBLinkUpdateInfo)))) {
      zkErrorSet(err, funcname, ZKERR, ZKMOD,
		 ZKENOMEM, "Couldn't allocate memory for link table");
      return 0;
    }
    if (!zkMemPoolRegister(err, adp->mpool, (void *) adp->linkinfo,
		      sizeof (struct T_FrNIDBLinkUpdateInfo) * (numlinks+1))) {
      return 0;
    }
    adp->numlinks = numlinks;
  } else {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKERROR, "No links found in INI file");
    return 0;
  }
  for (i = 0, j = 0; i < numsections; i++) {
    const char *name;
    if (zkConfigSection(err,adp->config,i, &name)) {
      if (!strncasecmp ("aip", name, 3)) {
	/* we've done this section */
      } else if (!strncasecmp ("link", name, 4)) {
	configLoadLinkEntry(err, adp->config, name, adp->linkinfo+j);
	j++;
      } else {
	zkErrorSet(err, funcname, ZKERR, ZKMOD,
		   ZKERROR, "Unknown section to load");
	return 0;
      }
    } else {
      return 0;
    }
  }
  
  if (!zkConfigFree(err,&(adp->config))){
    return 0; /* Bug: improper cleanup */
  }
#endif
  return 1;
#warning Dont forget to take care of the  (signature?)
  
}

int frNIDBAipDataPktLoad(ZkError err, FrNIDBAipDataPkt adp, char *filename)
{
  char *buffer;
  size_t size;

  /* First, get the file */
  if (!zkFileLoad (err, filename, &buffer, &size)) {
    return(0);
  }
  /* Parse the file */
  if (!frNIDBLoadAIPUpdateBuffer (err, adp, buffer, size)) {
    free(buffer);
    return 0;
  };

  free(buffer);
  
  return 1;
}

int nidbGetLinkUpdateIDFromEntity (FrNIDBContext nidb, FrNIDBLinkUpdateInfo lui,
				  long *linkid)
{
  char *le;
  if (!nidbComputeLinkUpdateEntity(nidb->err, lui, &le))
    return 0;
  
  if (!nidbGetIDFromEntity (nidb, le, FRENT_TYPE_LINK, linkid))
    return 0;
  free(le);

  return 1;
}

/*
 * Consolidate information from an update.
 */
int frNIDBAipDataPktConsolidate(FrNIDBContext nidb, FrNIDBAipDataPkt adp)
{
  static const char *funcname = "frNIDBAipDataPktConsolidate";
  struct T_FrAIPinfo ai_d;
  struct T_FrLinks   lnk_d;
  FrAIPinfo ai = & ai_d;
  FrLinks   lnk= &lnk_d;
  long aipid, linkid;
  
  /*
   * First we time stamp out packet
   */
  if (!zkDateTimeSetCurrent (nidb->err, &adp->aipinfo->datercvd, ZKTIME_FL_UTC))
    return 0;

  /*
   * Next we update the link information from the aip section
   */
  adp->linkinfo->aipserialnumber = adp->aipinfo->serialnumber;
  adp->linkinfo->reportingAipEntity = adp->aipinfo->entity;
  adp->linkinfo->dategen = adp->aipinfo->dategen;
  adp->linkinfo->datercvd = adp->aipinfo->datercvd;
  /*
   * Finally, we consolidate the main database with our update
   *
   * a. Mark this AIP as up 
   */
  if (!nidbGetIDFromEntity (nidb, adp->aipinfo->entity, FRENT_TYPE_AIP, &aipid))
    return 0;
  if (aipid < 0) {
    /* can't find this */
    zkErrorSet_va(nidb->err, funcname, ZKWARN, ZKMOD,
		   ZKERROR, "Can't find an entity [%s]", adp->aipinfo->entity);
    return 1;
  }
  /* bug: This should be it's own database, I think */
  if (!nidbGetAipInfoFromDB(nidb, aipid, ai))
    return 0;
  /* Mark it up */
  ai->isup = 1;  
  if (!nidbPutAipInfoToDB(nidb, ai))
    return 0;

  /* b. Mark this link as up/down */
  if (!nidbGetLinkUpdateIDFromEntity (nidb, adp->linkinfo, &linkid))
    return 0;
  if (linkid < 0) {
    /* can't find this */
    zkErrorSet_va(nidb->err, funcname, ZKWARN, ZKMOD,
		   ZKERROR, "Can't find an entity");
    return 1;
  }
  /* bug: This should also be it's own database, I think */
  if (!nidbGetLinkInfoFromDB(nidb, linkid, lnk))
    return 0;
  /* Consolidate info */
  /* Bug: More sophisticated cost computation reqd */
  if (adp->linkinfo->isUp)
    lnk->cost = 1;
  else 
    lnk->cost = FRNDB_LINK_IS_DOWN;

  if (!nidbPutLinkInfoToDB(nidb, lnk))
    return 0;
  
  return 1;
}

#ifndef NDEBUG
int frNIDBAipDataPktDump(ZkError err, FrNIDBAipDataPkt adp)
{
  char zkdts[ZKDATETIMESTR_SZ];

  
  zkDebug_va("AIP Data Dump");
  zkDebug_va("Aipid    = %d", adp->aipinfo->aipid);
  zkDebug_va("serno    = %d", adp->aipinfo->serialnumber);
  if (!zkDateTimeStore (err, &adp->aipinfo->dategen, zkdts, ZKDATETIMESTR_SZ))
    return 0;
  zkDebug_va("dategen  = %s", zkdts);
  zkDebug_va("hostname = %s", adp->aipinfo->hostname);
  zkDebug_va("ipaddr   = %s", adp->aipinfo->ipaddr);
  zkDebug_va("Numlinks = %d", adp->numlinks);

  zkDebug_va("Link");
  zkDebug_va("  ID  #%d", adp->linkinfo->linkid);
  zkDebug_va("  Up  [%d]", adp->linkinfo->isUp);
  zkDebug_va("  Out %4d",  adp->linkinfo->pktsSent);
  zkDebug_va("  In  %4d",  adp->linkinfo->pktsRcvd);

  return 1;
}
#endif

