/*
 *  $Id: frNIDB.c,v 1.13 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 *  
 *  The Network Information Database Library
 */

#include "zks.h"

#if 0
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkFile.h>
#include <zkMemPool.h>
#include <frSec.h>
#include <frNetworkDefaults.h>
#include <frNIDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

#define P_DEBUG       "Debug"
#define P_CONFIG      "Config"
#define P_LOGFILE     "LogFile"
#define P_PORT        "Port"

#define SECTION "ZKSTATD"


/**
 * Initialize a new NIDB object
 *
 * There can only be one such object
 */
static struct T_FrNIDBContext NIDB_d;
static int NIDB_ACTIVE = 0;

int frNIDBContextNew (ZkError err, FrNIDBContext *nidb_p)
{
  static const char *funcname = "frNIDBContextNew";
  FrNIDBContext nidb = &NIDB_d;

  zkErrorClear (err);
  memset(nidb,0,sizeof(struct T_FrNIDBContext));  

  if (!nidb_p) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Parameter [nidb] is NULL");
    return 0;
  }

  if (NIDB_ACTIVE) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKEBUSY,
	       "NIDB already active. Cannot reallocate a second one.");
    return 0;
  }

  nidb->ready = 0;
  nidb->err = err;
  nidb->logFP = NULL;
    
  if (!zkMemPoolNew(nidb->err, &nidb->mpool))
    return 0;

  nidb->progname = NULL;
  nidb->name = NULL;
  nidb->spooldir = NULL;
  nidb->database = NULL;
  nidb->keyfile = NULL;
  nidb->logfile = NULL;
  nidb->outfile = NULL;
  nidb->sigFile = NULL;
  nidb->confFile = NULL;

  nidb->myPrivSigKey = NULL;
  nidb->ncdb = NULL;

  nidb->aipInfoDB = NULL;
  nidb->linkInfoDB = NULL;
  nidb->genInfoDB = NULL;
  nidb->wincInfoDB = NULL;
  nidb->serverInfoDB = NULL;
  nidb->aipUpdateDB = NULL;
  nidb->linkUpdateDB = NULL;
  nidb->entityIdDB = NULL;
  nidb->idEntityDB = NULL;
  nidb->db = NULL;         /* Not used everywhere */

  /* Stat server stuff */
  nidb->stat.flags   = 0;
  nidb->stat.port    = FR_DFLT_STAT_SERVER_PORT;
  memset (&nidb->stat.server, 0, sizeof(nidb->stat.server));
  nidb->stat.sock_fd = 0;
  nidb->stat.bucket = NULL;

  *nidb_p = nidb;
  return 1;
}

int frNIDBContextInit (ZkError err, FrNIDBContext nidb, int argc, char **argv)
{
  (void) err;

  if (!zkConfigNew(nidb->err,&(nidb->config))) 
    return 0;
  
  return 1;
}

  
/**
 * Adjust parameters based on the configuration information
 */
int frNIDBContextLoad (ZkError err, FrNIDBContext nidb)
{
  static const char *funcname = "frNIDBtContextLoad";
  uint8_t bool = 0; 
  uint16_t u16 = 0;
  char * str = NULL;
  const char * sec = SECTION;

  zkErrorClear(err);
  if (!nidb) {
    zkErrorSet(err,funcname,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    return 0;
  }
  
  /* Get the port number */
  if (zkConfigGetParameter(err,nidb->config,TYPE_UINT16,sec,P_PORT,&u16)) {
    nidb->stat.port = u16;
  }

  return 1;
}

/**
 * Once we have all of the parameters we need, initialize everything
 */
int frNIDBContextVerify (ZkError err, FrNIDBContext nidb)
{
  static const char *funcname = "frNIDBContextInit";

  if (!nidb) {
    zkErrorSet(err,funcname,ZKERR,ZKMFRSTAT,ZKENULLPTR,NULL);
    return 0;
  }
    
  /* 
    if ( ! frCryptInit(nidb->err, 0)) {
      return 0;
    }

    if ( !frCryptSigNewFromFile(nidb->err, &nidb->myPrivSigKey, nidb->keyfile)) {
      return 0;
    }
  */
  /*    
    if ( ! zkAANew (nidb->err, "AipInfo", &nidb->aipInfoDB)) {
      return 0;
    }

    if ( ! zkAANew (nidb->err, "LinkInfo", &nidb->linkInfoDB)) {
      return 0;
    }

    if ( ! zkAANew (nidb->err, "GenInfo", &nidb->genInfoDB)) {
      return 0;
    }
    
    if ( ! zkAANew (nidb->err, "ServerInfo", &nidb->serverInfoDB)) {
      return 0;
    }
    
    if ( ! zkAANew (nidb->err, "WinclientInfo", &nidb->wincInfoDB)) {
      return 0;
    }
  */
  if ( ! zkAANew (nidb->err, "AipUpdates", &nidb->aipUpdateDB)) {
    return 0;
  }
  /*
    if ( ! zkAANew (nidb->err, "LinkUpdates", &nidb->linkUpdateDB)) {
      return 0;
    }

    if ( ! zkAANew (nidb->err, "EntityId", &nidb->entityIdDB)) {
      return 0;
    }

    if ( ! zkAANew (nidb->err, "IdEntity", &nidb->idEntityDB)) {
      return 0;
    }
  */
  nidb->logFP = stderr;

  if (zkDebugIsOn) {
    ZkLog log = NULL;
    zkErrorLog(nidb->err,&log);
    zkLogSetProgname(log,nidb->progname);
    zkLogSetStream(log,stderr);
    zkLogEnable(log);
  }
  
  /* Stat Stuff */
  if (nidb->stat.port != 0) { 
    nidb->stat.server.sin_addr.s_addr = htonl(INADDR_ANY);
    nidb->stat.server.sin_port = htons(nidb->stat.port);
    nidb->stat.server.sin_family = AF_INET;
  }

  /* Startup the logfile */
  if (nidb->logfile) {
    zkLogSetFile(nidb->log,nidb->logfile);
    zkLogSetProgname(nidb->log,nidb->progname);
    zkLogEnable(nidb->log);
  }

  return 1;
}


/**
 * Free the NIDB object
 */
int frNIDBContextFree (ZkError err, FrNIDBContext *nidb)
{
  static const char *funcname = "frNIDBContextFree";

  zkErrorClear (err);
  if (!nidb || !*nidb) {
    zkErrorSet(err,funcname,ZKERR,ZKMXFER,ZKENULLPTR,NULL);
    return 0;
  }

  if ( ((*nidb)->ready) && !(--(*nidb)->ready) ) {
    (*nidb)->logFP = NULL;
    
    (*nidb)->progname = NULL;
    (*nidb)->name = NULL;
    (*nidb)->spooldir = NULL;
    (*nidb)->database = NULL;
    (*nidb)->keyfile = NULL;
    (*nidb)->logfile = NULL;
    (*nidb)->sigFile = NULL;
    (*nidb)->outfile = NULL;
    (*nidb)->myPrivSigKey = NULL;
    (*nidb)->ncdb = NULL;

    (*nidb)->aipInfoDB = NULL;
    (*nidb)->linkInfoDB = NULL;
    (*nidb)->db = NULL;         /* Not used everywhere */
    /* bug: handle the closes properly */
    zkConfigFree((*nidb)->err, &(*nidb)->config);
    zkMemPoolNew((*nidb)->err, &(*nidb)->mpool);
    memset((*nidb),0,sizeof(struct T_FrNIDBContext));  
    *nidb = NULL;
  } else {
    zkErrorSet(err,funcname,ZKWARN,ZKMOD,ZKEBUSY,"Reference count is not NULL");
  }
  return 1;
}

#endif
