/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkDump.h"
#include "zkCStr.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"


static char * chomp(char *p)
{
  if (!p) return NULL;

  if (*p != '\0') {
    if (p[strlen (p)-1] == '\n') {
      p[strlen (p)-1] = '\0';
    }
  }
  return (p);
}
  
int frNidbDumpStateBuf (
  ZkError err, 
  FrDBOTypeNsdbAip state,
  void * idx,
  char ** buf)
{
  const char *p;
  char q[64], r[64], s[64];
  int iDx = -1;

  (void) err;

  if (idx)
    iDx = * (int *) idx;
  
  p = state->reference_date
    ? ctime((time_t *)&(state->reference_date)) 
    : "Not Present";
  strncpy(q, p, 64);
  chomp (q);
  
  p = state->last_update_date
    ? ctime((time_t *)&(state->last_update_date)) 
    : "Not Present";
  strncpy (r, p, 64);
  chomp (r);

  if (iDx > -1)
    snprintf (s, 64, "/%d:", (int)p);
  else
    snprintf (s, 64, ":");
  
  zkCStrSprintf (buf,
           "-----------------------------------------\n"
           "State for %u%s \n" /* entid */
           "-----------------------------------------\n"
           "%20s: %s\n" 
           "%20s: %s\n"
           "%20s: %s\n"
           "%20s: %s\n"

           "%20s: %Lu\n"
           "%20s: %u seconds\n"
           "%20s: %u seconds\n"
           "%20s: %Lu\n"
           "%20s: %Lu\n"

           "%20s: %Lu\n"
           "%20s: %Lu\n"
           "%20s: %Lu\n"
           "%20s: %Lu\n"
           "%20s: %u\n"

           "%20s: %u\n"
           "%20s: %Lu\n"
           "%20s: %Lu\n"
           "%20s: %u\n"
           "%20s: %Lu\n"

           "%20s: %u\n"
           "%20s: %s\n"
           "-----------------------------------------\n",

           state->entid, s,
           "Reference Date", q,
           "Last Update", r,
           "Availablity", state->unavailable_flag?"Unavailable":"Available",
           "Up flag", state->up_flag?"Up":"Down",

           "TTL", state->ttl,
           "Uptime", state->uptime,
           "Downtime", state->downtime,
           "Msgs Rcvd", state->msgs_rcvd,
           "Msgs Sent", state->msgs_sent,
           
           "Msgs_Rcvd_1", state->msgs_rcvd_1,
           "Msgs_Sent_1", state->msgs_sent_1,
           "Msgs_Rcvd_2", state->msgs_rcvd_2,
           "Msgs_Sent_2", state->msgs_sent_2,
           "#Updates", state->num_updates,
           
           "Entity Rating", state->entity_rating,
           "Override Entity Rating", state->OR_entity_rating,
           "Override Up Flag", state->OR_up_flag,
           "Security Level", state->security_level,
           "Spare_1", state->spare_1,

           "Connections", state->connections,
           "Release", state->release);


  return 1;
}
  
int frNidbDumpState (
  ZkError err, 
  FrDBOTypeNsdbAip state,
  ZkDumpCallback callback,
  void * cdata,
  void * userdata)
{
  char *buf = NULL;

  if (state && callback && cdata ) {
    frNidbDumpStateBuf (err, state, userdata, & buf);
    callback (err, cdata, buf, strlen(buf));
    if(buf)
      free(buf);
  } 
  return 1;
  
}
