
/*
 * NIDB core routines 
 *
 * $Id: fr_nidb.c,v 1.23 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 * 
 */

static const char * module_id_ =
"$Id: fr_nidb.c,v 1.23 2000/11/24 15:03:11 marcp Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <string.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkDebug.h>
#include <zkDump.h>
#include <zkDB.h>
#include <zkAlloc.h>
#include <frDBOTypes.h>
#include <fr_nidb.h>

#define FREEIT(x) if (x) { free (x); x = NULL; }

/** Using the EntIdentifier, get the DBO from the requested DB */
int frNidbEidFetchDS (
  ZkError err,
  ZkDB db,
  uint32_t eid_num,
  void **ds,
  size_t *dsz,
  int *found)
{
  static const char *funcname = "frNidbEidFetchDS";
  ZkDBT key = NULL;
  ZkDBT data = NULL;
  int retcode = 0;
  void *p = NULL;
  size_t psz = 0;

  (void) module_id_;
  (void) funcname;
  *found = 0;
  
  if ( !db || !ds || !dsz || !found ) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD,
	       ZKENULLPTR, "NULL Pointer passed");
    return 0;
  }

  if ( ! zkDBTNew (err, &key) || ! zkDBTNew (err, &data)) 
    goto failure;
  
  if (!zkDBSetKeyOrData(err, key, &eid_num, sizeof(eid_num)))
    goto failure;
  
  if (!zkDBFetch(err, db, key, data, found))
    goto failure;

  if (!*found) {
    retcode = 1;
    goto failure;
  }
  
  if (!zkDBGetKeyOrData(err, data, &p, &psz))
    goto failure;

  if (!zkDBOUnpack (err, (void **) ds, dsz, p, psz))
    goto failure;

  retcode = 1;
 failure:

  if (key) zkDBTFree (err, &key);
  if (data) zkDBTFree (err, &data);

  return retcode;
}    

#if 0
/**
 * Given an entity , get the entity's DBO.
 */
int frNidbGetEntityDboFromEntity (
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  void **dbo,
  size_t *dsz,
  int *found)
{
  static const char *funcname = "frNidbGetEntityDboFromEntity";
  FrDBOTypeEntityID ent_dbo = NULL;
  FrDBOTypeNtdbHost host_dbo = NULL;
  size_t hsz = 0;
  int retcode = 0;

  if (c && entity && dbo && dsz && found) {
    if ( ! frNtdbGetEntityDboFromEntity (err, c, entity, &ent_dbo, found) ) {
        zkLogWrite_va(c->log, "Failed to locate entity %s/%d in entity database",
                      entity->name, entity->type);
        retcode = 1;
    } else  if ( ! *found ) {
        zkLogWrite_va(c->log, "Failed to find entity %s/%d in entity database",
                      ent_dbo->ent.name, ent_dbo->ent.type);
        retcode = 1;
    } else
      switch (ent_dbo->ent.type) {
      case FRENT_TYPE_HOST:
        /* Now fetch host dbo for this entidentifier */
        if ( ! frNidbEidFetchDS (err, c->hostDB, ent_dbo->entid,
                                 (void **)&host_dbo, &hsz, found)) {
        } else  if ( ! *found) {
          zkErrorSet_va(err,funcname,ZKERROR,ZKMOD,ZKENULLPTR,
                        "Error: unable to find %s in the host database \n",
                        ent_dbo->ent.name); 
          retcode = 1;
          FREEIT(host_dbo);
        } else {
          *dbo = host_dbo;
          retcode = 1;
        }
        FREEIT(ent_dbo);
        break;
      default:
        zkLogWrite_va(c->log, "Unable to deal with this entity %s/%d",
                      ent_dbo->ent.name, ent_dbo->ent.type);
        retcode = 1;
      }
        
      zkDebug ("Entity:  %s (%d)\n",ent_dbo->ent.name,ent_dbo->ent.type);
        
      retcode = 1;
      FREEIT(ent_dbo);
  } else {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
  }
  return retcode;
}
#endif
#if 0
/**
 * Given an entity , get the entity ident DBO.
 */
int frNidbGetEntityDboFromEntity (
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  FrDBOTypeEntityID *ent_dbo,
  size_t *esz,
  int *found)
{
  static const char *funcname = "frNidbGetEntityDboFromEntity";
  FrDBOTypeNtdbHost host_dbo = NULL;
  size_t hsz = 0;
  int retcode = 0;

  if (!c || !entity || !ent_dbo || !esz || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
  } else {
    /*
    int nidbGetEntityIDfromDBwithEntity(FrNIDBContext nidb, char *entity,
				     short type, FrEntityId eid)
    */
    if ( ! frNtdbGetEntityDboFromEntity (err, c, entity, &ent_dbo, found) ) {
        zkLogWrite_va(c->log, "Failed to locate entity %s/%d in entity database",
                      entity->name, entity->type);
        retcode = 1;
    } else  if ( ! *found ) {
        zkLogWrite_va(c->log, "Failed to find entity %s/%d in entity database",
                      (*ent_dbo)->ent.name, (*ent_dbo)->ent.type);
        retcode = 1;
    } else
      switch ((*ent_dbo)->ent.type) {
      case FRENT_TYPE_HOST:
        /* Now fetch host dbo for this entidentifier */
        if ( ! frNidbEidFetchDS (err, c->hostDB, (*ent_dbo)->entid,
                                 (void **)&host_dbo, &hsz, found)) {
        } else  if ( ! *found) {
          zkErrorSet_va(err,funcname,ZKERROR,ZKMOD,ZKENULLPTR,
                        "Error: unable to find %s in the host database \n",
                        (*ent_dbo)->ent.name); 
          retcode = 1;
          FREEIT(host_dbo);
        } else {
#warning check this out - isna right          
          *ent_dbo = host_dbo;
          retcode = 1;
        }
        FREEIT(ent_dbo);
        break;
      default:
        zkLogWrite_va(c->log, "Unable to deal with this entity %s/%d",
                      (*ent_dbo)->ent.name, (*ent_dbo)->ent.type);
        retcode = 1;
      }
        
      zkDebug ("Entity:  %s (%d)\n",(*ent_dbo)->ent.name,(*ent_dbo)->ent.type);
        
      retcode = 1;
      FREEIT(*ent_dbo);
  }
  return retcode;
}
#endif
