/*
 * NIDB Entity core routines 
 *
 * $Id: fr_nidb_entity.c,v 1.13 2000/11/24 15:03:11 marcp Exp $
 * 
 */

static const char * module_id_ =
"$Id: fr_nidb_entity.c,v 1.13 2000/11/24 15:03:11 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include "zkIntTypes.h"
#include <string.h>
#include <time.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkAlloc.h"
#include "zkDebug.h"
#include "zkDump.h"
#include "zkDB.h"
#include "frDBOTypes.h"
#include "fr_nidb.h"

#define FREEIT(x) if (x) { free (x); x = NULL; }

/**
 * Given the entity identifier, get the entity.
 * Fill the entity identifier object as a side effect.
 * Set the found flag as appropriate.
 */
int frNidbEidFetchEDS (
  ZkError err,
  NidbContext c,
  uint32_t eid,
  FrDBOTypeEntityID * ent_dbo,
  int *found)
{
  static const char *funcname = "frNidbEidFetchEDS";
  void *p = NULL;
  size_t psz = 0;
  (void) funcname;
  (void) module_id_;

  if ( !c || !ent_dbo || !eid || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  
  *found = 0;

  /* Is the Entity already there? */
  if (!zkAAGet(err, c->entidDB, &eid, sizeof(eid),
	       &p, &psz))
    return 0;

  /* bug: fix */
  if (psz) {
    /* BUG: FORGOT TO PACK THIS */
    zkMALLOC (*ent_dbo, FrDBOTypeEntityID, sizeof (struct T_FrDBOTypeEntityID));
    if (psz != sizeof (struct T_FrDBOTypeEntityID))
      zkDebug ("Warning. Size mismatch");

    memcpy(*ent_dbo, p, psz);

    /* if (!zkDBOUnpack (err, (void **) entid, &esz, p, psz))
       return 0; */

    *found = 1;
  } else {
    zkErrorSet_va(err,funcname,ZKWARN,ZKMOD, ZKENOTFOUND,
		  "Unable to locate entity # %u", eid);
  }
  return 1;  
}    

/**
 * Given the entity, get the entity identifier.
 * Fill the entity identifier object as a side effect.
 * Set the found flag as appropriate.
 */
/* CREATE An Entity Identifier if need be */
int frNidbGetEntityIdentifier (
  ZkError err,
  NidbContext c,
  FrDBOTypeEntityID entid,
  uint32_t *eid,
  int *found)
{
  static const char *funcname = "frNidbGetEntityIdentifier";
  void *p = NULL;
  size_t psz = 0;
  FrDBOTypeEntityID t_e;
  (void) funcname;
  (void) module_id_;

  if ( !c || !entid || !eid || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  
  *eid = 0;
  *found = 0;

  /* Is the Entity already there? */
  if (!zkAAGet(err, c->entDB, &entid->ent, sizeof(FrEntIdExp),
	       &p, &psz))
    return 0;
#if 0
  if (zkDebugIsOn() && psz) {
    zkDebug("Returned entity:");
    zkDumpBuffer(err,p,psz,zkDumpToStream,stderr);
  }
#endif
  
  if (psz) {

    t_e =  (FrDBOTypeEntityID) p;
    entid->entid = t_e->entid;
    *eid = t_e->entid;
    *found = 1;
  } else {
    const char *type;
    frEntGetTypeStr(err, &entid->ent, &type);
    zkLogWrite_va(c->log,
		  "Unable to locate %s entity [%s] in entity identifier database",
		  type, entid->ent.name);
  }
  return 1;  
}    
/**
 * Given an entity, get the entity_dbo for it
 */
int frNidbGetEntDboFromEntity (
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  FrDBOTypeEntityID *ent_dbo,
  size_t *esz,
  int *found)
{
  static const char *funcname = "frNidbGetEntIdentifier";

  void *p = NULL;
  size_t psz = 0;
  FrDBOTypeEntityID t_e;
  int retcode = 0;

  (void) funcname;
  (void) module_id_;

  if ( !c || !ent_dbo || !entity || !esz || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
  } else {

    *found = 0;

    /* bug: fix */
    if (!zkAAGet(err, c->entDB, entity, sizeof(FrEntIdExp),
                 &p, &psz)) {
      const char *type;
      frEntGetTypeStr(err, entity, &type);
      zkLogWrite_va(c->log,
          "Unable to locate %s entity [%s] in entity identifier database: %s",
           entity->name, type, zkErrorDetail (err));
    } else {

#if 0      
      if (zkDebugIsOn() && psz) {
        zkDebug("Returned entity:");
        zkDumpBuffer(err,p,psz,zkDumpToStream,stderr);
      }
#endif      
      if (psz) {
        /* We have a copy of the ent_dbo */
        t_e = (FrDBOTypeEntityID) p;
        /* Need to make a copy */
        if ( ! zkDBONew (err, t_e->regID, (void **)ent_dbo, esz) ) {
          zkLogWrite_va(c->log, "Unable to create dbo: %s", zkErrorDetail (err));
        } else {
          memcpy (*ent_dbo, p, psz);
          *esz = psz;
          *found = 1;
        }
      } else { 
        const char *type;
        frEntGetTypeStr(err, entity, &type);
        zkLogWrite_va(c->log,
             "Unable to locate %s entity [%s] in entity identifier database",
	      type, entity->name);
      }
      retcode = 1;
    }
  }
  return retcode;
}


/**
 * Get a list of all entity ID's and entities in an entity typeset
 * Each bit in the mask represent the FRENT_TYPE of the entity
 * we are interested in.
 *
 * In addition, select only those who have been
 * updated since the specified time. Use 0 for all.
 *
 * Caller must free the list when done. 
 */
int frNidbGetEntitiesInTypesetUpdatedSince (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t typeset,
  FrDBOTypeEntityID *ent_ident_list,
  size_t *num_elems)
{
  static const char *funcname = "frNidbGetEntitiesInTypesetUpdatedSince";
  void *key = NULL;
  void *pdata = NULL;
  size_t ksz = 0, esz = 0, psz = 0;
  FrDBOTypeEntityID elist = NULL, elistp = NULL;
  FrDBOTypeEntityID eid = NULL;
  int cnt = 0, retcode = 0, found = 0;
  int cursize = 0;
  int slice = 64;
  time_t last_update = 0;
  time_t status_update = 0;

  if (!c || !ent_ident_list || !num_elems) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    return 0;
  }
    
  if (!zkAA_CursorOpen(err, c->entidDB))
    return 0;

  /* Create a list of entity ID's */
  elist =  (FrDBOTypeEntityID) calloc (slice, sizeof(struct T_FrDBOTypeEntityID));
  cursize = slice;
  elistp = elist;

  while ((retcode = zkAA_CursorGetNext(err, c->entidDB, &key , &ksz, &pdata, &psz,
			       &found)) && found) {
    /* bug: should check if what we got back is a valid piece of data */
    if ( (*(FrNidbEntityID)key) == 0 ) {
      /* This is the entity placeholder */
      zkDebug("Top entity Identifier is %d", *(uint32_t *)pdata); 
      continue;
    }
    /*
    if ( ! zkDBOUnpack(err, (void **) &eid, &esz, pdata, psz) ) {
       zkAA_CursorAbort(err, c->entidDB);
       goto finish;
    }
    */
    eid = (FrDBOTypeEntityID) pdata;
    if (typeset & (1 << eid->ent.type)) {
      /* Check to make sure we have enough space */
      if ( ((cnt % slice) == 0 ) && cnt == cursize ) {
        int delta = elistp - elist;
        zkDebug("New result slice at %d", cnt);
        cursize += slice;
        if ( ! zkMemRealloc (err, (void **) &elist,
                          sizeof(struct T_FrDBOTypeEntityID)*cursize)) {
	  zkAA_CursorAbort(err, c->entidDB);
          return 0;
	}
        /* Fix the pointer */
        elistp = elist + delta;
      }
      /* Next, check to see that it satisfies the "since" constraint */

      if ( ! frNidbGetEntityAnyUpdateTS ( err, c, eid->entid,
                                           &last_update, &status_update, &found) ) {
        zkErrorSet_va(err,funcname,ZKWARN,ZKMOD,ZKENOTFOUND,
                      "Error locating the last update timestamp for %u",
                      eid->entid);
        retcode = 0; 
      } else if ( ! found ) {
        zkErrorSet_va(err,funcname,ZKWARN,ZKMOD,ZKENOTFOUND,
                      "Unable to find the last update timestamp for %u; assuming 0",
                      eid->entid);
        last_update = 0;
        status_update = 0;
      }
      /* if more recent, count it in */
      /* Bug: did a quick hack job here, without thinking it all
         the way through */
      if ( last_update > since || status_update > since  ) {
        *elistp = *eid;
        /* Needs to be explicitely set
         * as we're doing a db join here
         */
        elistp->last_update = (last_update < status_update ) ? last_update : status_update;
      
        zkDebug("Found entid: %u - %s/%d ts:%s", (elistp)->entid,
                (elistp)->ent.name, (elistp)->ent.type,
                ctime(&(elistp->last_update)) );
        cnt++;
        elistp++;
      }
    }
    /* FREEIT (eid);*/
    esz = 0;
  }

  if (!retcode) {
    zkAA_CursorAbort(err, c->entidDB);
    goto finish;
  }

  if (!zkAA_CursorClose(err, c->entidDB))
    goto finish;

  *ent_ident_list = elist;
  *num_elems = cnt;
  
  return 1;
 finish:
  FREEIT(elist);
  return 0;
}

/**
 * Get a list of all entity ID's and entities in an entity typeset
 * Each bit in the mask represent the FRENT_TYPE of the entity
 * we are interested in.
 *
 * Caller must free the list when done 
 */
int frNidbGetEntitiesInTypeset (
  ZkError err,
  NidbContext c,
  uint32_t typeset,
  FrDBOTypeEntityID *ent_ident_list,
  size_t *num_elems)
{
  return ( frNidbGetEntitiesInTypesetUpdatedSince (err, c, 0, typeset,
                                                 ent_ident_list, num_elems));

}

#if 0
/**
 * Get a list of all entity ID's in an entity typeset
 * Each bit in the mask represent the FRENT_TYPE
 *
 * Caller must free the list when done 
 */
int frNidbGetEnt_IdentsInTypeset (
  ZkError err,
  NidbContext c,
  uint32_t typeset,
  FrNidbEntityID *ent_ident_list,
  size_t *num_elems)
{
  static const char *funcname = "frNidbGetEntitiesOfType";
  void *key = NULL;
  void *pdata = NULL;
  size_t ksz = 0, esz = 0, psz = 0;
  FrNidbEntityID elist = NULL, elistp = NULL;
  FrDBOTypeEntityID eid = NULL;
  int i = 0, retcode = 0, found = 0;
  int cursize = 0;
  int slice = 2; /* 128 */

  if (!c || !ent_ident_list || !num_elems) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    return 0;
  }
    
  if (!zkAA_CursorOpen(err, c->entidDB))
    return 0;

  /* Create a list of entity ID's */
  elist =  (FrNidbEntityID) calloc (slice, sizeof(FrNidbEntityID_t));
  cursize = slice;
  elistp = elist;
  
  while ((retcode = zkAA_CursorGetNext(err, c->entidDB, &key , &ksz, &pdata, &psz,
			       &found)) && found) {
    /* bug: should check if what we got back is a valid piece of data */
    if ( (*(FrNidbEntityID)key) == 0 ) {
      /* This is the entity placeholder */
      zkDebug("Top entity Identifier is %d", *(uint32_t *)pdata); 
      continue;
    } /*
    if ( ! zkDBOUnpack(err, (void **) &eid, &esz, pdata, psz) ) {
         zkAA_CursorAbort(err, c->entidDB);
         goto finish;
    }
      */
    eid = (FrDBOTypeEntityID) pdata;
    if (typeset & (1 << eid->ent.type)) {
      /* Check to make sure we have enough space */
      if ( (i % slice) == 0 ) {
        int delta = elistp - elist;
        zkDebug("New result slice at %d", i);
        cursize += slice;
        if ( ! zkMemRealloc (err, (void **) &elist,
                          sizeof(FrNidbEntityID_t)*cursize)) {
	  zkAA_CursorAbort(err, c->entidDB);
          return 0;
	}
        /* Fix the pointer */
        elistp = elist + delta;
      }
      *elistp = *(FrNidbEntityID)key;
      zkDebug("Found entid: %u", *elistp);
      i++;
      elistp++;
    }
    /* FREEIT (eid);*/
    esz = 0;
  }

  if (!retcode) {
    zkAA_CursorAbort(err, c->entidDB);
    goto finish;
  }

  if (!zkAA_CursorClose(err, c->entidDB))
    goto finish;

  *ent_ident_list = elist;
  *num_elems = i;
  
  return 1;
 finish:
  FREEIT(elist);
  return 0;
}
#endif

/**
 * Get a list of all entity ID's
 *
 * Caller must free the list when done 
 */
int frNidbGetAllEntityIDs (
  ZkError err,
  NidbContext c,
  FrNidbEntityID *ent_ident_list,
  size_t *num_elems)
{
  static const char *funcname = "frNidbGetAllEntityIDs";
  void *key = NULL;
  void *data = NULL;
  size_t ksz = 0, dsz = 0;
  FrNidbEntityID elist = NULL, elistp = NULL;
  int i = 0, retcode = 0, found = 0;

  if (!c || !ent_ident_list || !num_elems) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    return 0;
  }
    
  if (!zkAA_CursorOpen(err, c->entidDB))
    return 0;

  /* Create a list of entity ID's */
  elist = /* (FrNidbEntityID *) */ calloc (100, sizeof(FrNidbEntityID_t));

  elistp = elist;
  
    
  while ((retcode = zkAA_CursorGetNext(err, c->entidDB, (void **) &key, &ksz,
				&data, &dsz, &found)) && found) {

    *elistp = *(FrNidbEntityID)key;
    i++;
    elistp++;
  }

  if (!retcode) {
    zkAA_CursorAbort(err, c->entidDB);
    goto finish;
  }

  if (!zkAA_CursorClose(err, c->entidDB))
    goto finish;

  *ent_ident_list = elist;
  *num_elems = i;
  
  return 1;
 finish:
  if (elist) free (elist);
  return 0;
}

#if NOT_USED
/**
 * Get a list of all entity ID's matchin a certain criteria
 *
 * Caller must free the list when done 
 */
int frNidbGetEntitiesOfType (
  ZkError err,
  NidbContext c,
  FrEntIdType type,
  FrNidbEntityID *ent_ident_list,
  size_t *num_elems)
{
  static const char *funcname = "frNidbGetEntitiesOfType";
  void *key = NULL;
  size_t ksz = 0, esz = 0;
  FrNidbEntityID elist = NULL, elistp = NULL;
  FrDBOTypeEntityID eid;
  int i = 0, retcode = 0, found = 0;

  if (!c || !ent_ident_list || !num_elems) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    return 0;
  }
    
  if (!zkAA_CursorOpen(err, c->entidDB))
    return 0;

  /* Create a list of entity ID's */
  elist = /* (FrNidbEntityID) */ calloc (100, sizeof(FrNidbEntityID_t));

  elistp = elist;
  
  while ((retcode = zkAA_CursorGetNext(err, c->entidDB, key , &ksz, (void **) &eid, &esz,
			       &found)) && found) {
    /* bug: should check if what we got back is a valid piece of data */
    if (eid->ent.type == type) {
      *elistp = *(FrNidbEntityID)key;
      i++;
      elistp++;
      zkDebug("Found entid: %u", *elistp);
    }
  }

  if (!retcode) {
    zkAA_CursorAbort(err, c->entidDB);
    goto finish;
  }

  if (!zkAA_CursorClose(err, c->entidDB))
    goto finish;

  *ent_ident_list = elist;
  
  return 1;
 finish:
  if (elist) free (elist);
  return 0;
}
#endif

/**
 * Using the EntityExp, get the EntityIdent for it
 */
int frNidbFindEntityIdDboUsingEntity(
  ZkError err,
  NidbContext c,
  FrEntIdExp *entity,
  FrDBOTypeEntityID *entid,
  int *found)
{
  static const char *funcname = "frNidbFindEntityIdDboUsingEntity";
  size_t esz = 0;
  int retcode = -1;
  uint32_t eid;

  if (!c || !entid|| !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    retcode = 0;
  } else if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) entid, &esz)) {
    retcode = 0;
  } else if (!frEntCopy(err, &((*entid)->ent), entity)) {
    retcode = 0;
  } else if ( ! frNidbGetEntityIdentifier (err, c, *entid, &eid, found)) {
    retcode = 0;
  } else if ( !*found ) {
    zkLogWrite_va(c->log, "Failed to find entity %s/%d in entity database",
		  (*entid)->ent.name, (*entid)->ent.type);
    retcode = 1;
  } else {
    zkDebug ("Entity:  %s (%d)\n",(*entid)->ent.name,(*entid)->ent.type);
    retcode = 1;
  }
  
  return retcode;
}

/**
 * Using the EntityExp, get the EntityIdent for it
 */
int frNidbFindEntIdentUsingEntity(
  ZkError err,
  NidbContext c,
  const FrEntIdExp *entity,
  FrEntIdent *ent_ident,
  int *found)
{
  static const char *funcname = "frNidbFindEntIdentFromEntity";
  FrDBOTypeEntityID entid = NULL;
  int retcode = -1; 

  if (!c || !entity || !ent_ident || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    retcode = 0;
  } else if ( ! frNidbFindEntityIdDboUsingEntity(err, c, entity, &entid, found) ) {
    retcode = 0;
  } else if ( ! *found ) {
    retcode = 1;
  } else {
    *ent_ident = entid->entid;
    retcode = 1;
  }
  FREEIT(entid);
  return retcode;
}

int frNidbEntityGetEidFromTypeAndName (
  ZkError err,
  NidbContext c,
  const char *type,
  const char *name,
  FrDBOTypeEntityID *eidbo,
  int *found)
{
  static const char *funcname = "frNidbEntityGetEidFromTypeAndName";
  FrDBOTypeEntityID e;
  int retcode = 0;
  uint32_t eid = 0;
  size_t psz = 0;

  (void) module_id_;

  if ( !c || !type || !name || !eidbo || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
    return 0;
  }
  *found = 0;  
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &e, &psz))
    goto finish;

  if ( ! frEntSetFromStr2(err, &e->ent, type , name )) 
    goto finish;

  if ( ! frNidbGetEntityIdentifier (err, c, e, &eid, found)) 
    goto finish;

  if ( ! *found) {
    zkLogWrite_va(c->log,"Error: Entity [%s/%s] is not in the database",
		  type, name);
    retcode = 1 ;
    goto finish;
  }

  *eidbo = e;
  retcode = 1;
 finish:
  return retcode;
}

/**
 * From the entity type and name get the EntityIdentifier for it
 * (if it exists)
 */
int frNidbEntityGetEntIdentFromTypeAndName (
  ZkError err,
  NidbContext c,
  const char *type,
  const char *name,
  FrEntIdent *ent_ident,
  int *found)
{
  static const char *funcname = "frNidbEntityGetEntIdentFromTypeAndName";
  FrDBOTypeEntityID e;
  int retcode = 0;

  (void) module_id_;

  if ( !c || !type || !name || !ent_ident || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD, ZKENULLPTR,"Null parameter");
  } else {
    *found = 0;  
    if ( frNidbEntityGetEidFromTypeAndName (err, c, type, name, &e, found )){
      if ( *found ) {
        *ent_ident = e->entid;
      }
      retcode = 1;
      FREEIT(e);
    }
  }
  return retcode;
}
