/*
 * NIDB Status request handler
 *
 * $Id: fr_nidb_status.c,v 1.16 2000/11/24 15:03:11 marcp Exp $
 *
 */

static const char * module_id_ =
"$Id: fr_nidb_status.c,v 1.16 2000/11/24 15:03:11 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zeroknowledge.com>

 */


#include "zks.h"
#include "zkIntTypes.h"
#include "string.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkDebug.h"
#include "zkDump.h"
#include "zkDB.h"
#include "zkAlloc.h"
#include "frNidbObj.h"
#include "fr_nidb.h"

#ifdef  FREEIT
# undef FREEIT
#endif

#define FREEIT(x) if (x) { free (x); (x) = NULL; }

/**
 * Get the number of connections for a series of entity masks
 *
 * Data is being reassembled from a number of databases
 */
int frNidbGetStatusConnections (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t entity_mask,
  FrNidbObj *resp,
  size_t *rsz,
  int *found)
{
  static const char *funcname = "frNidbGetStatusConnections";
  FrDBOTypeEntityID ent_ident_list = NULL;
  size_t numents;
  FrNidbObj obj = NULL;
  size_t n, osz = 0;
  int retcode = 1;
  uint32_t status = 0, rating = 0;

  (void) since;
  (void) module_id_;
  
  if (!c || !resp || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
  } else if ( ! frNidbGetEntitiesInTypesetUpdatedSince (err, c, since, entity_mask,
                                                        &ent_ident_list, &numents)) {
    zkLogWrite_va(c->log,
                  "Error processing entity request for [STATUS]\n");
    zkLogError(c->log, err);
  } else {

    if ( ! ( frNidbObjNew (err, &obj)
             && frNidbObjSetType (err, obj, FRNIDBOBJ_STATUS_RESP)
             && frNidbObjSetStatusParams (err, &obj, numents)) ) {
      zkLogWrite_va(c->log,
                    "Error creating response for [STATUS]\n");
      zkLogError(c->log, err);
    } else {
      retcode = 1;
      *found = 1;
      for (n = 0; n < numents; n++) {
        if ( ! frNsdbGetEntIdentStatusAndRating (err, c, ent_ident_list[n].entid,
                                                 &status, &rating, found) ) {
          zkLogWrite_va(c->log, "Error getting status information for %u\n",
                        ent_ident_list[n].entid );
          zkLogError(c->log, err);
        } else if ( ! * found ) {
#warning Bug: fix status/rating for V1.1. Should be 0/0
          status = 0; 
          rating = 0;
          if ( zkDebugIsOn() ) {
            zkLogWrite_va(c->log, "No status information available for %u\n",
                          ent_ident_list[n].entid );
            zkLogError(c->log, err);
          }
        }
        if ( ! frNidbObjSetStatusRespAtIdx(err, obj, n,
                                           ent_ident_list[n].entid,
                                           status, rating) ) {
          zkLogWrite_va(c->log,
                        "Error setting response for [STATUS] at %d\n", n);
          zkLogError(c->log, err);
          found = 0;
          retcode = 0;
          break;
        }
      }
    }
    if (retcode == 0) {
      FREEIT(obj);
    } else if ( ! frNidbObjGetSizeof (err, obj, &osz) ) {
      zkLogWrite_va(c->log,
                    "Error getting size of nidb object\n");
      zkLogError(c->log, err);
      FREEIT(obj);
      osz = 0;
    } 
    *resp = obj;
    *rsz = osz;
  }
  return retcode;
}

/**
 * Get the description NidbObj for an entity
 *
 * Data is being reassembled from a number of databases
 */
int frNidbGetStatusUpdateSince (
  ZkError err,
  NidbContext c,
  time_t since,
  uint32_t entity_mask,
  uint32_t resp_vers,
  FrNidbObj *resp,
  size_t *rsz,
  int *found)
{
  static const char *funcname = "frNidbGetStatusUpdateSince";
  FrDBOTypeEntityID ent_ident_list = NULL;
  size_t numents;
  FrNidbObj obj = NULL;
  size_t n, osz = 0;
  int retcode = 1;
  uint32_t status = 0, rating = 0;

  (void) since;
  (void) module_id_;
  
  if (!c || !resp || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
  } else if ( ! frNidbGetEntitiesInTypesetUpdatedSince (err, c, since, entity_mask,
                                                        &ent_ident_list, &numents)) {
    zkLogWrite_va(c->log,
                  "Error processing entity request for [STATUS]\n");
    zkLogError(c->log, err);
  } else {

    if ( ! ( frNidbObjNewSpecifyVersion (err, &obj, resp_vers)
             && frNidbObjSetType (err, obj, FRNIDBOBJ_STATUS_RESP)
             && frNidbObjSetStatusParams (err, &obj, numents)) ) {
      zkLogWrite_va(c->log,
                    "Error creating response for [STATUS]\n");
      zkLogError(c->log, err);
    } else {
      retcode = 1;
      *found = 1;
      for (n = 0; n < numents; n++) {
        if ( ! frNsdbGetEntIdentStatusAndRating (err, c, ent_ident_list[n].entid,
                                                 &status, &rating, found) ) {
          zkLogWrite_va(c->log, "Error getting status information for %u\n",
                        ent_ident_list[n].entid );
          zkLogError(c->log, err);
        } else if ( ! * found ) {
#warning Bug: Fix for V1.1. Should be 0/0          
          status = 0; /* should be 0 */
          rating = 0; /* should be 0 */
          if ( zkDebugIsOn() ) {
            zkLogWrite_va(c->log, "No status information available for %u\n",
                          ent_ident_list[n].entid );
            zkLogError(c->log, err);
          }
        }
        if ( ! frNidbObjSetStatusRespAtIdx(err, obj, n,
                                           ent_ident_list[n].entid,
                                           status, rating) ) {
          zkLogWrite_va(c->log,
                        "Error setting response for [STATUS] at %d\n", n);
          zkLogError(c->log, err);
          found = 0;
          retcode = 0;
          break;
        }
      }
    }
    if (retcode == 0) {
      FREEIT(obj);
    } else if ( ! frNidbObjGetSizeof (err, obj, &osz) ) {
      zkLogWrite_va(c->log,
                    "Error getting size of nidb object\n");
      zkLogError(c->log, err);
      FREEIT(obj);
      osz = 0;
    } 
    *resp = obj;
    *rsz = osz;
  }
  return retcode;
}


int frNidbGetStatusForEntIdent (
  ZkError err,
  NidbContext c,
  FrEntIdent ent_ident,
  uint32_t resp_vers,
  void **resp,
  size_t *rsz,
  int *found)
{
  static const char *funcname = "frNidbGetStatusForEntIdent";
  FrNidbObj obj = NULL;
  size_t osz = 0;
  int retcode = 1;
  uint status = 0, rating = 0;
  FrDBOTypeEntityID ent_dbo = NULL;
  int entity_found;

  
  if (!c || !resp || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter");
    retcode = 0;
  } 

  else if (!frNidbEidFetchEDS
 (err, c, ent_ident, &ent_dbo, &entity_found)) {
    zkLogWrite_va(c->log,
                  "%s: Fetch entity failed for id %d\n", funcname, ent_ident);
    zkLogError(c->log, err);
    retcode = 0;
  }

  else if (!entity_found) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,
                  "Entity not found for id %d", ent_ident);
    zkLogError(c->log, err);
    retcode = 0;
  }

  else if ( ! ( frNidbObjNewSpecifyVersion (err, &obj, resp_vers)
             && frNidbObjSetEntity (err, obj, &(ent_dbo->ent))
             && frNidbObjSetType (err, obj, FRNIDBOBJ_STATUS_RESP)
             && frNidbObjSetStatusParams (err, &obj, 1)) ) {
    zkLogWrite_va(c->log, "Error creating response for [STATUS]\n");
    zkLogError(c->log, err);
    retcode = 0;

  } else if ( ! frNsdbGetEntIdentStatusAndRating (err, c, ent_ident,
                                                  &status, &rating, found) ) {
    zkLogWrite_va(c->log, "Error getting status information for %u\n", 
                  ent_ident);
    zkLogError(c->log, err);
    retcode = 0;
  } else if ( ! found) {
#warning This the place where the client gets the notification that an entity is down.
#warning Bug: Fix rating/status for V1.1. Should be 0/0
    status = 1; /* should be 0 */
    rating = 13; /* should be 0 */
    if ( zkDebugIsOn() ) {
      zkLogWrite_va(c->log, "No status information available for %u\n", ent_ident );
      zkLogError(c->log, err);
    }
  } else if ( ! frNidbObjSetStatusRespAtIdx(err, obj, 0, ent_ident, status, rating) ) {
    zkLogWrite_va(c->log, "Error setting response for [STATUS] at %d\n", 0);
    zkLogError(c->log, err);
    retcode = 0;
      
  } else if ( ! frNidbObjGetSizeof (err, obj, &osz) ) {
    zkLogWrite_va(c->log, "Error getting size of nidb object\n");
    zkLogError(c->log, err);

    /* pack it up and send it off */
  } else if ( ! zkDBOPack(err, obj, osz, resp, rsz)) {
    *found = 0;
      
  } else {
    *found = 1;
  }

  if (retcode == 0) {
    found = 0;
    FREEIT(*resp);
    *rsz = 0;
  }

  FREEIT(obj);

  return retcode;
}


