/*
 *  $Id: nidb_aipupdates.c,v 1.7 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 *  
 *  The Network Information Database
 *  NCDB <-> NIDB translation 
 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include <zkError.h>
#include <zkDB.h> 
#include <zkDebug.h>
#include <zkDump.h>
#include <zkDateTime.h>

#include <frNDB.h>
#include <frNCDB.h>
#include <frNIDB.h>
#include "convert.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


#define CFG_SZ 256
#define FRNIDB_AIPUPDATE_DBOBJ_VERS 2

/**
 * Wire format for FrAipUpdateInfo
 *
 *DB:
 * aipUpdateDB.
 *
 *Key:
 * char *     entity (name)  Bug: change this 
 *
 *Data:
 * short      version
 * short      reserved
 * long       aipid;
 * char *     entity;
 * long       serialnumber;
 * ZkDateTime dategen;
 * ZkDateTime datercvd;
 * --- Version 2
 * long       reportSerialNumber;
 * long       accumPktsSent;
 * long       accumPktsRcvd;
 * long       accumUptime;
 * ZkDateTime lastReportDate;
 * long       lastPktsSent;
 * long       lastPktsRcvd;
 * long       lastUptime;
 */

#define FRNIDB_AIPUPDATE_DBOBJ_SIZE (sizeof(short)*2+sizeof(long)*9+\
                                  ZK_NDBNAME_SIZE+ZKDATETIMEBIN_SZ * 3)

static int nidbNewAipUpdateDBObj (ZkError err, uchar **obj, size_t *offset,
				   size_t *ds)
{
  static const char *funcname = "nidbNewAipUpdateDBObj";
  size_t len = 0;

  zkMALLOC (*obj, uchar *, FRNIDB_AIPUPDATE_DBOBJ_SIZE);
  /* reserved */
  if (!zkCvtShortToBin(err, FRNIDB_AIPUPDATE_DBOBJ_VERS, *obj, &len))
    return 0;
  *offset = len;
  if (!zkCvtShortToBin(err, FRNIDB_AIPUPDATE_DBOBJ_VERS, (*obj)+(*offset), &len))
    return 0;
  *offset += len;
  *ds = FRNIDB_AIPUPDATE_DBOBJ_SIZE;
  return 1;
}


/**
 * Save an AipUpdate DS into a DB Object
 */
static int nidbAipUpdateToDBObj (FrNIDBContext nidb, FrNIDBAipUpdateInfo aui,
			       void **data, size_t *ds)
{
  static const char *funcname = "nidbAipUpdateToDBObj";

  uchar *obj = NULL;
  size_t offset = 0;
  size_t len = 0;

  if (!nidb || !nidb->ready || !aui || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!nidbNewAipUpdateDBObj (nidb->err, &obj, &offset, ds))
    return 0;

  if (!zkCvtLongToBin (nidb->err, aui->aipid, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtStrToBin (nidb->err, aui->entity, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtLongToBin (nidb->err, aui->serialnumber, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtZkDateTimeToBin (nidb->err, &aui->dategen, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtZkDateTimeToBin (nidb->err, &aui->datercvd, obj+offset, &len))
    return 0;
  offset += len;
  /*
   * Version 2
   */
  if (!zkCvtLongToBin (nidb->err, aui->reportSerialNumber, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->accumPktsSent, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->accumPktsRcvd, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->accumUptime, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtZkDateTimeToBin (nidb->err, &aui->lastReportDate, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->lastPktsSent, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->lastPktsRcvd, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, aui->lastUptime, obj+offset, &len))
    return 0;
  offset += len;

  if (offset != *ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Internal buffer overflow");
    return 0;
  }
  zkDebug_va("Internal offsets are consistent");
  *data = obj;
  return 1;
}

/**
 * Load the AipUpdate DS from a DB object
 */
static int nidbAipUpdateFromDBObj (FrNIDBContext nidb, FrNIDBAipUpdateInfo aui,
			       void *data, size_t *ds)
{
  static const char *funcname = "nidbAipUpdateFromDBObj";

  uchar *obj = NULL;
  size_t offset = 0;
  size_t len = 0;
  long l;
  short vers, s;
  ZkError err = nidb->err;


  if (!nidb || !nidb->ready || !aui || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }

  obj = (char *) data;
  
  if (!zkCvtShortFromBin(nidb->err, &vers, obj, &len))
    return 0;
  if (vers != FRNIDB_AIPUPDATE_DBOBJ_VERS) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Database information is of the wrong version");
    return 0;
  }
  aui->aipid = l;
  offset += len;

  if (!zkCvtShortFromBin(nidb->err, &s, obj, &len))
    return 0;
  aui->reserved = s;
  offset += len;
 
  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->aipid = l;
  offset += len;

  zkMALLOC (aui->entity, char *, ZK_NDBNAME_SIZE+1); /* Bug: memory leak */
  if (!zkCvtStrFromBin (nidb->err, aui->entity, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->serialnumber = l;
  offset += len;

  if (!zkCvtZkDateTimeFromBin (nidb->err, &aui->dategen, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtZkDateTimeFromBin (nidb->err, &aui->datercvd, obj+offset, &len))
    return 0;
  offset += len;

  /*
   * Version 2
   */

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->reportSerialNumber = l;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->accumPktsSent = l;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->accumPktsRcvd = l;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->accumUptime = l;
  offset += len;

  if (!zkCvtZkDateTimeFromBin (nidb->err, &aui->lastReportDate, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->lastPktsSent = l;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->lastPktsRcvd = l;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &l, obj+offset, &len))
    return 0;
  aui->lastUptime = l;
  offset += len;

  *ds = offset;

  if (*ds != FRNIDB_AIPUPDATE_DBOBJ_SIZE) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD, ZKEDATABAD,
		"Internal buffer inconsisten while extracting DB object");
    return 0;
  }
  zkDebug_va("Internal offsets are consistent");
    
  return 1;
}


/* Sequence numbers will always start at 1; 0 means none found */
int nidbPutAipUpdateInDB(FrNIDBContext nidb, FrNIDBAipUpdateInfo aui)
{
  static const char *funcname = "nidbPutAipUpdateInDB";


  void *auiDBobj;
  size_t ds;
  
  if (!nidb || !nidb->ready || !aui) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (!nidbAipUpdateToDBObj (nidb, aui, &auiDBobj, &ds))
      return 0;

  zkDumpBuffer(nidb->err,auiDBobj,ds,zkDumpToStream,stderr);

  if (!zkAAPut(nidb->err, nidb->aipUpdateDB,
	       aui->entity, strlen(aui->entity)+1,
	       auiDBobj, ds)) 
    return (0);

  free (auiDBobj);
  
  return 1;
  
}

/**
 * Generate a report for the ecommerce server
 *
 * Open the AIP update database
 * Scan all records
 * Foreach active AIP report the stats
 * Update the records appropriately
 * Close everything
 */
int frNIDBGenECommReport (FrNIDBContext nidb)
{
  static const char *funcname = "frNIDBGenECommReport";
  int retcode, numaips, found;
  ZkConfig config = NULL;
  char sec[CFG_SZ];
  char parm[CFG_SZ];
  char val[CFG_SZ];
  char zkdts[ZKDATETIMESTR_SZ];
  char *key;
  void *auiDBobj;
  size_t ks, ds;
  struct T_FrNIDBAipUpdateInfo aui_t;
  FrNIDBAipUpdateInfo aui = &aui_t;
  ZkDateTime currDate;
  char currDateStr[ZKDATETIMESTR_SZ];

  if (!nidb || !nidb->ready) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		 ZKENULLPTR, "Null context, or nidb not ready");
    return 0;
  }

  if (!nidb->outfile) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "No output file has been specified");
    return 0;
  }

  if (!zkDateTimeSetCurrent(nidb->err, &currDate, ZKTIME_FL_UTC))
    return 0;

  if (!zkDateTimeStore(nidb->err, &currDate, currDateStr, ZKDATETIMESTR_SZ))
    return 0;

  zkConfigNew(nidb->err,&config);
  
  if (!zkAA_CursorOpen(nidb->err, nidb->aipUpdateDB))
    return 0;

  numaips = 0;
  while ((retcode = zkAA_CursorGetNext(nidb->err, nidb->aipUpdateDB,
				(void **) &key, &ks, &auiDBobj, &ds, &found))
	 && found) {

    zkDumpBuffer(nidb->err,auiDBobj,ds,zkDumpToStream,stderr);

    if (!nidbAipUpdateFromDBObj (nidb, aui, auiDBobj, &ds))
      /* Bug: need to be more gracefull here */
      goto abort;
    
    /* Update a few fields */
    numaips++;
    aui->reportSerialNumber ++;

    /* Append to the report file */
    snprintf (sec, CFG_SZ, "AIP%02d", numaips);
    
    snprintf (parm, CFG_SZ, "EntityName");
    snprintf (val, CFG_SZ, "%s", aui->entity);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "ReportDate"); /* Current date */
    snprintf (val, CFG_SZ, "%s", currDateStr);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "ReportSerialNumber"); 
    snprintf (val, CFG_SZ, "%d", (int)aui->reportSerialNumber);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "LastReportDate");
    if (!zkDateTimeStore (nidb->err, &aui->lastReportDate, zkdts,
			  ZKDATETIMESTR_SZ)){goto abort;}
    snprintf (val, CFG_SZ, "%s", zkdts);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "PacketsSentSinceLastReport");
    snprintf (val, CFG_SZ, "%ld", aui->accumPktsSent);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "PacketsReceivedSinceLastReport");
    snprintf (val, CFG_SZ, "%ld", aui->accumPktsRcvd);
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "PacketsDropRateSinceLastReport");
    snprintf (val, CFG_SZ, "0.00%%");
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;

    snprintf (parm, CFG_SZ, "UptimePercentageSinceLast");
    /* snprintf (val, CFG_SZ, "%s", aui->region); */
    
    snprintf (val, CFG_SZ, "100.00%%");
    if (!zkConfigSet (nidb->err, config, sec, NULL, parm, val)) goto abort;
    
    /* Now update the database record */
    aui->lastPktsSent += aui->accumPktsSent;
    aui->lastPktsRcvd += aui->accumPktsRcvd;
    aui->accumPktsSent = 0;
    aui->accumPktsRcvd = 0;
    aui->lastReportDate = currDate;

    if (!nidbPutAipUpdateInDB(nidb, aui))
      goto abort;
    
  }
  if (!retcode)
    goto abort;

  if (zkConfigSave (nidb->err, config, nidb->outfile))
    goto abort;

  if (!zkAA_CursorClose(nidb->err, nidb->aipUpdateDB))
    return 0;
  
  return 1;

 abort:
  zkAA_CursorAbort(nidb->err, nidb->aipUpdateDB);
  return 0;
}
