/*
 *  $Id: nidb_entity.c,v 1.6 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 *  
 *  The Network Information Database
 *  Entity Management
 */

#if 0
#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include <zkError.h>
#include <zkDB.h> 
#include <zkDebug.h>
#include <zkDump.h>
#include <zkDateTime.h>
#include <zkMemPool.h>

#include <frNDB.h>
#include <frNCDB.h>
#include <frNIDB.h>

#include "convert.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


/**
 * OLD CODE - NO LONGER USED
 */

#define FRNIDB_ENTITYID_DBOBJ_VERS 1

/**
 * Wire format for FrEntityId
 *
 *  short   version
 *  char    *entity;
 *  long    id;
 *  short   type;
 */
#define FRNIDB_ENTITYID_DBOBJ_SIZE (sizeof(long)*1+sizeof(short)*2+\
                                  ZK_NDBNAME_SIZE*1)

static int nidbNewEntityIdDBObj (ZkError err, uchar **obj, size_t *offset,
				   size_t *ds)
{
  static const char *funcname = "nidbNewEntityIdDBObj";

  zkMALLOC (*obj, uchar *, FRNIDB_ENTITYID_DBOBJ_SIZE);
  if (!zkCvtShortToBin(err, FRNIDB_ENTITYID_DBOBJ_VERS, *obj, offset))
    return 0;
  *ds = FRNIDB_ENTITYID_DBOBJ_SIZE;
  return 1;
}


static int nidbEntityIdToDBObj (FrNIDBContext nidb, FrEntityId eid,
			       void **data, size_t *ds)
{
  static const char *funcname = "nidbEntityIdToDBObj";

  uchar *obj = NULL;
  size_t offset = 0;
  size_t len = 0;

  if (!nidb || !nidb->ready || !eid || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!nidbNewEntityIdDBObj (nidb->err, &obj, &offset, ds))
    return 0;

  if (!zkCvtStrToBin (nidb->err, eid->entity, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtLongToBin (nidb->err, eid->id, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtShortToBin (nidb->err, eid->etype, obj+offset, &len))
    return 0;
  offset += len;
  
  if (offset != *ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Internal buffer overflow");
    return 0;
  }
  zkDebug_va("Internal offsets are consistent");
  *data = obj;
  return 1;
}

static int nidbEntityIdFromDBObj (FrNIDBContext nidb, FrEntityId eid,
			       void *data, size_t *ds)
{
  static const char *funcname = "nidbEntityIdFromDBObj";

  uchar *obj;
  size_t offset = 0;
  size_t len;
  long x;
  short vers, s;
  ZkError err = nidb->err;
  
  obj = (uchar *) data;

  if (!nidb || !nidb->ready || !eid || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!zkCvtShortFromBin(nidb->err, &vers, obj, &offset))
    return 0;

  if (vers != FRNIDB_ENTITYID_DBOBJ_VERS) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Database information is of the wrong version");
    return 0;
  }

  zkMALLOC (eid->entity, char *, ZK_NDBNAME_SIZE+1); /* Bug: memory leak */
  if (!zkCvtStrFromBin (nidb->err, eid->entity, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  eid->id = x;
  offset += len;

  if (!zkCvtShortFromBin (nidb->err, &s, obj+offset, &len))
    return 0;
  eid->etype = s;
  offset += len;

  *ds = offset;
  
  return 1;
}

static int nidbBuildEntityIDKey (ZkError err, char *entity,
			       short type, uchar **key, size_t *ks)
{
  static const char *funcname = "nidbBuildEntityIDKey";
  size_t offset = 0;
  
  
  *ks =  sizeof(type)+strlen(entity)+1;
  zkMALLOC(*key, uchar *, *ks);

  /* Bug: need a version number for this key object */
  if (!zkCvtShortToBin(err, type, *key, &offset))
    return 0;

  if (!zkCvtCharToBin (err, entity, (*key+offset), strlen(entity)))
    return 0;
  offset += strlen(entity)+1;

  /* Warning OK */
  if (offset != *ks) {
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Internal buffer overflow");
    return 0;
  }
  return 1;
  
}

static int nidbBuildIDEntityKey (ZkError err, long id,
			       short type, uchar **key, size_t *ks)
{
  static const char *funcname = "nidbBuildIDEntityKey";
  size_t offset = 0;
  size_t len = 0;
  
  *ks =  sizeof(type)+sizeof(id);
  zkMALLOC(*key, uchar *, *ks);

  /* Bug: need a version number for this key object */
  if (!zkCvtShortToBin(err, type, *key, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin(err, id, *key, &len))
    return 0;
  offset += len;
  

  if (offset != *ks) {
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Internal buffer overflow");
    return 0;
  }
  return 1;
  
}


int nidbPutEntityIDinDB(FrNIDBContext nidb, FrEntityId eid)
{
  static const char *funcname = "nidbPutEntityIDinDB";

  void *eidDBobj, *key;
  size_t ds, ks;
  
  if (!nidb || !nidb->ready || !eid) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (! nidbEntityIdToDBObj (nidb, eid, &eidDBobj, &ds))
      return 0;
  
#ifndef NDEBUG
  zkDumpBuffer(nidb->err,eidDBobj,ds,zkDumpToStream,stderr);
#endif

  if (! nidbBuildEntityIDKey(nidb->err, eid->entity, eid->etype,
			     (uchar **)&key, &ks))
    return 0;
			   
  if (!zkAAPut(nidb->err, nidb->entityIdDB,
	       key, ks, eidDBobj, ds)) 
    return (0);

  free (key);
    
  if (! nidbBuildIDEntityKey(nidb->err, eid->id, eid->etype,
			     (uchar **)&key, &ks))
    return 0;
			   
  if (!zkAAPut(nidb->err, nidb->idEntityDB,
	       key, ks, eidDBobj, ds)) 
    return (0);

  free (key);
  free (eidDBobj);
  
  return 1;
  
}

int nidbRegisterEntityIDinDB (FrNIDBContext nidb, char *entity,
			      long id, FrEntIdType eid)
{
  struct T_FrEntityId eid_d;

  eid_d.etype = eid;
  eid_d.entity = entity;
  eid_d.id = id;
  
  if (!nidbPutEntityIDinDB(nidb, &eid_d))
    return 0;
  return 1;
}


int nidbGetEntityIDfromDBwithEntity(FrNIDBContext nidb, char *entity,
				     short type, FrEntityId eid)
{
  static const char *funcname = "nidbGetEntityIDfromDBwithEntity";

  void *eidDBobj = NULL;
  void *key;
  size_t ds, ks;
  
  if (!nidb || !nidb->ready || !eid) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (! nidbBuildEntityIDKey(nidb->err, entity, type,
			     (uchar **)&key, &ks))
    return 0;

  if (!zkAAGet(nidb->err, nidb->entityIdDB,
	       key, ks, &eidDBobj, &ds)) 
    return (0);

  if (ds == 0) {
    zkDebug_va ("No data found");
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENOTFOUND, "Requested entityID is not found");
    eid = NULL;
    return 0;
  }
  
#ifndef NDEBUG
  zkDumpBuffer(nidb->err,eidDBobj,ds,zkDumpToStream,stderr);
#endif
      
  if (!nidbEntityIdFromDBObj (nidb, eid, eidDBobj, &ds))
      return 0;
  
  return 1;
  
}

int nidbGetEntityIDFromDBwithID(FrNIDBContext nidb, long id, short type,
				     FrEntityId eid)
{
  static const char *funcname = "nidbGetEntityIDFromDBwithID";

  void *eidDBobj = NULL;
  void *key;
  size_t ds, ks;
  
  if (!nidb || !nidb->ready || !eid) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (! nidbBuildIDEntityKey(nidb->err, id, type, (uchar **)&key, &ks))
    return 0;

  if (!zkAAGet(nidb->err, nidb->idEntityDB,
	       key, ks, &eidDBobj, &ds)) 
    return (0);

  if (ds == 0) {
    zkDebug_va ("No data found");
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENOTFOUND, "Requested EIDPid is not found");
    eid->id = -1;
    return 0;
  }
  
#ifndef NDEBUG
  zkDumpBuffer(nidb->err,eidDBobj,ds,zkDumpToStream,stderr);
  
#endif
      
  if (!nidbEntityIdFromDBObj (nidb, eid, eidDBobj, &ds))
      return 0;
  
  return 1;
  
}

int nidbGetIDFromEntity(FrNIDBContext nidb, char *entity, short type, long *id)
{
  struct T_FrEntityId eid_d;

  if (!nidbGetEntityIDfromDBwithEntity(nidb, entity, type, &eid_d))
    return 0;

  *id = eid_d.id;
  return 1;
}

int nidbGetEntityFromID(FrNIDBContext nidb, long id, short type, char **entity)
{
  struct T_FrEntityId eid_d;

  if (!nidbGetEntityIDFromDBwithID(nidb, id, type, &eid_d))
    return 0;

  *entity = strdup(eid_d.entity);
  if (!zkMemPoolRegister(nidb->err, nidb->mpool, *entity, strlen(*entity)+1))
    return 0;
  return 1;
}
#endif  
