/*
 *  $Id: nidb_linkinfo.c,v 1.7 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 *  
 *  The Network Information Database
 *  NCDB <-> NIDB translation 
 */
#if 0
#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include <zkError.h>
#include <zkDB.h> 
#include <zkDebug.h>
#include <zkDump.h>
#include <zkDateTime.h>
#include <zkCStr.h>

#include <frNDB.h>
#include <frNCDB.h>
#include <frNIDB.h>

#include "convert.h"
#include "nidb_priv.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

#define FRNIDB_LINKINFO_DBOBJ_VERS 1

/**
 * Wire format for FrAIPinfo
 *
 *  short   version
 *  long    id;      
 *  long    aip1;
 *  long    aip2;
 *  long    cost;    
 */
#define FRNIDB_LINKINFO_DBOBJ_SIZE (sizeof(long)*4+sizeof(short))

static int nidbNewLinkInfoDBObj (ZkError err, uchar **obj, size_t *offset,
			      size_t *ds)
{
  static const char *funcname = "nidbNewLinkInfoDBObj";

  zkMALLOC (*obj, uchar *, FRNIDB_LINKINFO_DBOBJ_SIZE);
  if (!zkCvtShortToBin(err, FRNIDB_LINKINFO_DBOBJ_VERS, *obj, offset))
    return 0;
  *ds = FRNIDB_LINKINFO_DBOBJ_SIZE;
  return 1;
}


static int nidbLinkInfoToDBObj (FrNIDBContext nidb, FrLinks lnk,
			       void **data, size_t *ds)
{
  static const char *funcname = "nidbLinkInfoToDBObj";

  uchar *obj;
  size_t offset = 0;
  size_t len;

  if (!nidb || !nidb->ready || !lnk || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!nidbNewLinkInfoDBObj (nidb->err, &obj, &offset, ds))
    return 0;

  if (!zkCvtLongToBin (nidb->err, lnk->id, obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, lnk->aip[0], obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, lnk->aip[1], obj+offset, &len))
    return 0;
  offset += len;

  if (!zkCvtLongToBin (nidb->err, lnk->cost, obj+offset, &len))
    return 0;
  offset += len;

  *data = obj;
  return 1;
}

static int nidbLinkInfoFromDBObj (FrNIDBContext nidb, FrLinks lnk,
			       void *data, size_t *ds)
{
  static const char *funcname = "nidbLinkInfoFromDBObj";

  uchar *obj;
  size_t offset = 0;
  size_t len;

  long x;
  short vers;
  
  obj = (uchar *) data;

  if (!nidb || !nidb->ready || !lnk || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!zkCvtShortFromBin(nidb->err, &vers, obj, &offset))
    return 0;

  if (vers != FRNIDB_LINKINFO_DBOBJ_VERS) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Database information is of the wrong version");
    return 0;
  }

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  lnk->id = x;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  lnk->aip[0] = x;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  lnk->aip[1] = x;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  lnk->cost = x;
  offset += len;
  
  *ds = offset;
  
  return 1;
}

#define DELTA(x) (!x ? 0 : (x < 0 ? 1 : -1))
/* bug: Using a hash (#) as an entity seperator */
#define FS 043 
  
int nidbComputeLinkEntityName(ZkError err, char *node1, char *node2,
				       char **linkEntity)
{
  static const char *funcname = "nidbComputeLinkEntityName";
  char *ent;
  int len;
#define NIDB_PADDING 4

  len = strlen(node1) +  strlen(node2) + NIDB_PADDING;
  zkMALLOC (ent, char *, len);
  switch (DELTA(strcasecmp(node1, node2))){
  case 0:
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKERROR, "Local and remote AIPs cannot be the same");
    return 0;
  case -1:
    snprintf (ent, len, "%s#%s", node2, node1);
    break;
  case 1:
    snprintf (ent, len, "%s#%s", node1, node2);
    break;
  default:
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKERROR, "Serious internal error (DELTA)");
    return 0;
  }
  if (!zkCStrToLowerCase(ent)) {
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKERROR, "Problem canonicalizing ");
    return 0;
  }
  *linkEntity = ent;
  return 1;
}

/* Sequence numbers will always start at 1; 0 means none found */
int nidbPutLinkInfoToDB(FrNIDBContext nidb, FrLinks lnk)
{
  static const char *funcname = "nidbPutLinkInfoToDB";

  char *a, *b;
  char *entname;
  void *linkDBobj;
  size_t ds;
  
  if (!nidb || !nidb->ready || !lnk) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  zkErrorClear(nidb->err);

  /* Need to register the link */
  if (!nidbGetEntityFromID(nidb, lnk->aip[0], FRENT_TYPE_AIP, &a) ||
      !nidbGetEntityFromID(nidb, lnk->aip[1], FRENT_TYPE_AIP, &b))
    return 0;

  if (!a || !b) {
    zkErrorSet (nidb->err, funcname, ZKWARN, ZKMOD,
		ZKENOTFOUND, "Could not locate one of the aip entity names");
    return 1;
  }
  
  if (! nidbComputeLinkEntityName(nidb->err, a, b, &entname))
    return 0;

  if (zkErrorCode(nidb->err) == ZKENOTFOUND)
    return 0;
  
  if (! nidbRegisterEntityIDinDB (nidb, entname, lnk->id, FRENT_TYPE_LINK))
    return 0;

  if (! nidbLinkInfoToDBObj (nidb, lnk, &linkDBobj, &ds))
      return 0;

  zkDumpBuffer(nidb->err,linkDBobj,ds,zkDumpToStream,stderr);
  
  if (!zkAAPut(nidb->err, nidb->linkInfoDB,
	       &lnk->id, sizeof(lnk->id),
	       linkDBobj, ds)) 
    return (0);

  free(a);
  free(b);
  free(linkDBobj);
  return 1;
  
}

/* Sequence numbers will always start at 1; 0 means none found */
int nidbGetLinkInfoFromDB(FrNIDBContext nidb, long aipid, FrLinks lnk)
{
  static const char *funcname = "nidbPutLinkInfoToDB";

  uchar *linkDBobj = NULL;
  void *obj = NULL;
  size_t ds;
  
  if (!nidb || !nidb->ready || !lnk) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (!nidbIsAipIDValid (nidb, aipid)) {
    zkErrorSet (nidb->err, funcname, ZKWARN, ZKMOD,
		 ZKEDATABAD, "Aip is invalid");
    return 0;
  }
  if (!zkAAGet(nidb->err, nidb->linkInfoDB,
	       &aipid, sizeof(aipid),
	       &obj, &ds)) 
    return (0);

  linkDBobj = obj;

  if (ds == 0) {
    zkDebug_va ("No data found");
    lnk = NULL;
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENOTFOUND, "Requested LinkID is not found");
    return 0;
  }

#ifndef NDEBUG
  zkDumpBuffer(nidb->err,linkDBobj,ds,zkDumpToStream,stderr);
#endif
      
  if (!nidbLinkInfoFromDBObj (nidb, lnk, linkDBobj, &ds))
      return 0;

  return 1;
  
}

#endif
