/*
 *  $Id: nidb_serverinfo.c,v 1.5 2000/11/24 15:03:11 marcp Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 *  
 *  The Network Information Database
 *  NCDB <-> NIDB translation 
 */

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>

#include <zkError.h>
#include <zkDB.h> 
#include <zkDebug.h>
#include <zkDump.h>
#include <zkDateTime.h>

#include <frNDB.h>
#include <frNCDB.h>
#include <frNIDB.h>

#include "convert.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB

#define FRNIDB_SERVERINFO_DBOBJ_VERS 2

/**
 * Wire format for FrServerInfo
 *
 *  short   version
 *  // char    *id;
 *  char    *host;
 *  char    *ip;
 *  uint16_t port;
 *  long     timeout;
 */
#define FRNIDB_SERVERINFO_DBOBJ_SIZE (sizeof(long)*1+sizeof(short)*2+\
                                  ZK_NDBNAME_SIZE*2)

static int nidbNewServerInfoDBObj (ZkError err, uchar **obj, size_t *offset,
				   size_t *ds)
{
  static const char *funcname = "nidbNewServerInfoDBObj";

  zkMALLOC (*obj, uchar *, FRNIDB_SERVERINFO_DBOBJ_SIZE);
  if (!zkCvtShortToBin(err, FRNIDB_SERVERINFO_DBOBJ_VERS, *obj, offset))
    return 0;
  *ds = FRNIDB_SERVERINFO_DBOBJ_SIZE;
  return 1;
}


static int nidbServerInfoToDBObj (FrNIDBContext nidb, FrServerInfo srv,
			       void **data, size_t *ds)
{
  static const char *funcname = "nidbServerInfoToDBObj";

  uchar *obj = NULL;
  size_t offset = 0;
  size_t len = 0;

  if (!nidb || !nidb->ready || !srv || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!nidbNewServerInfoDBObj (nidb->err, &obj, &offset, ds))
    return 0;
  /*
  if (!zkCvtStrToBin (nidb->err, srv->id, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;
  */
  if (!zkCvtStrToBin (nidb->err, srv->host, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtStrToBin (nidb->err, srv->ip, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtShortToBin (nidb->err, srv->port, obj+offset, &len))
    return 0;
  offset += len;
  
  if (!zkCvtLongToBin (nidb->err, srv->timeout, obj+offset, &len))
    return 0;
  offset += len;

  if (offset != *ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Internal buffer overflow");
    return 0;
  }
  zkDebug_va("Internal offsets are consistent");
  *data = obj;
  return 1;
}
#if 0
static int nidbServerInfoFromDBObj (FrNIDBContext nidb, FrServerInfo srv,
			       void *data, size_t *ds)
{
  static const char *funcname = "nidbServerInfoToDBObj";

  uchar *obj;
  size_t offset = 0;
  size_t len;
  long x;
  struct in_addr sin_addr;
  short vers, s;
  char *ip;
  
  obj = (uchar *) data;

  if (!nidb || !nidb->ready || !srv || !data || !ds) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context, paramater or nidb not ready");
    return 0;
  }
  
  if (!zkCvtShortFromBin(nidb->err, &vers, obj, &offset))
    return 0;

  if (vers != FRNIDB_SERVERINFO_DBOBJ_VERS) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKEDATABAD, "Database information is of the wrong version");
    return 0;
  }

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  srv->id = x;
  offset += len;

  if (!zkCvtCharFromBin (nidb->err, srv->entity, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtCharFromBin (nidb->err, srv->name, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtInAddrFromBin (nidb->err, &sin_addr, obj+offset, &len))
    return 0;
  offset += len;

  if (!(ip = inet_ntoa(sin_addr))) {
    zkErrorSet(nidb->err, funcname, ZKERR, ZKMOD,
	       ZKEDATABAD, "couldn't convert address");
    return 0;
  }

  strncpy (srv->ip, ip, ZK_NDBNAME_SIZE);
  
  if (!zkCvtShortFromBin (nidb->err, &s, obj+offset, &len))
    return 0;
  srv->tcpport = s;
  offset += len;

  if (!zkCvtShortFromBin (nidb->err, &s, obj+offset, &len))
    return 0;
  srv->udpport = s;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  srv->isexit = x;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  srv->isup = x;
  offset += len;

  if (!zkCvtCharFromBin (nidb->err, srv->region, obj+offset, ZK_NDBNAME_SIZE))
    return 0;
  offset += ZK_NDBNAME_SIZE;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  srv->rating = x;
  offset += len;

  if (!zkCvtLongFromBin (nidb->err, &x, obj+offset, &len))
    return 0;
  srv->protocol = x;
  offset += len;
  /*
  if (!zkCvtCharFromBin (nidb->err, zkdts, obj+offset, len))
    return 0;
  offset += len;
  */

  *ds = offset;
  
  return 1;
}
#endif

/* Sequence numbers will always start at 1; 0 means none found */
int nidbPutServerInfoToDB(FrNIDBContext nidb, FrServerInfo srv)
{
  static const char *funcname = "nidbPutServerInfoToDB";

  void *srvDBobj;
  size_t ds;
  
  if (!nidb || !nidb->ready || !srv) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (! nidbServerInfoToDBObj (nidb, srv, &srvDBobj, &ds))
      return 0;

  zkDumpBuffer(nidb->err,srvDBobj,ds,zkDumpToStream,stderr);

  /*
    if (!nidbRegisterEntityIDinDB(nidb, ai->entity, ai->id, FRENT_TYPE_))
    return 0;
  */
  
  if (!zkAAPut(nidb->err, nidb->serverInfoDB,
	       &srv->entname, strlen(srv->entname)+1,
	       srvDBobj, ds)) 
    return (0);

  free (srvDBobj);
  
  return 1;
  
}

#if 0
int nidbGetServerInfoFromDB(FrNIDBContext nidb, long serverid, FrServerInfo srv)
{
  static const char *funcname = "nidbGetServerInfoFromDB";

  uchar *srvDBobj = NULL;
  void *obj = NULL;
  size_t ds;
  
  if (!nidb || !nidb->ready || !srv) {
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context/parameter or nidb not ready");
    return 0;
  }

  if (!nidbIsServerIDValid (nidb, serverid)) {
    zkErrorSet (nidb->err, funcname, ZKWARN, ZKMOD,
		 ZKEDATABAD, "Server is invalid");
    return 0;
  }

  if (!zkAAGet(nidb->err, nidb->serverInfoDB,
	       &serverid, sizeof(serverid),
	       &obj, &ds)) 
    return (0);

  srvDBobj = obj;

  if (ds == 0) {
    zkDebug_va ("No data found");
    zkErrorSet (nidb->err, funcname, ZKERR, ZKMOD,
		ZKENOTFOUND, "Requested SERVERid is not found");
    srv = NULL;
    return 0;
  }
  
#ifndef NDEBUG
  zkDumpBuffer(nidb->err,srvDBobj,ds,zkDumpToStream,stderr);
#endif
      
  if (!nidbServerInfoFromDBObj (nidb, srv, srvDBobj, &ds))
      return 0;
  
  return 1;
  
}
#endif
