/*
 *
 * $Id: nidbctx.c,v 1.18 2000/11/24 15:03:11 marcp Exp $
 *
 */

static const
char * module_id_ = "$Id: nidbctx.c,v 1.18 2000/11/24 15:03:11 marcp Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */


#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkDebug.h"
#include "zkDB.h"
#include "zkAA.h"
#include "zkParms.h"
#include "zkDBOTypes.h"
#include "frDBOTypes.h"
#include "fr_nidbctx.h"

/*
 * Bugs: I believe a finer control of which databases are opened is
 * Bugs: required.
 */

int nidbContextInit (
  ZkError err ,
  NidbContext c )
{
  static const char *funcname = "nidbContextInit";
#warning Temporary measure for mysql (need proper wrapper)
  static MYSQL m;

  (void) module_id_;
  
  if ( ! c ) {
    zkErrorSet (err, funcname, ZKERR, ZKMOD,
		ZKENULLPTR, "Null context");
    return 0;
  }
    
  c->topodb           = DEFAULT_TOPODB;
  c->statedb          = DEFAULT_STATEDB;
  c->entdb            = DEFAULT_ENTDB;
  c->entiddb          = DEFAULT_ENTIDDB;
  c->entupddb         = DEFAULT_ENTUPDDB;
  c->neibrdb          = DEFAULT_NEIBRDB;
  c->hostdb           = DEFAULT_HOSTDB;
  c->archive_sqlhost  = DEFAULT_ARCHIVE_SQLHOST;
  c->archive_sqluser  = DEFAULT_ARCHIVE_SQLUSER;
  c->archive_sqlpswd  = DEFAULT_ARCHIVE_SQLPSWD;
  c->archive_sqldb    = DEFAULT_ARCHIVE_SQLDB;
  
  c->topoDB    = NULL;
  c->stateDB   = NULL;
  c->entDB     = NULL;
  c->entidDB   = NULL;
  c->entupdDB  = NULL;
  c->neibrDB   = NULL;
  c->hostDB    = NULL;
  c->archiveDB = &m;

  return 1;
}

int nidbContextDump (
  ZkError err ,
  NidbContext c ,
  FILE * stream )
{
  static const char *funcname = "nidbContextDump";

  zkErrorClear(err);
  if (c && stream) {
    fprintf(stream,"[ %s ]\n",S_MOD_NIDB);
    if (c->topodb) fprintf(stream,"%s = %s\n", P_NIDB_TOPO, c->topodb);
    if (c->neibrdb) fprintf(stream,"%s = %s\n", P_NIDB_NEIGHBOUR, c->neibrdb);
    if (c->statedb) fprintf(stream,"%s = %s\n", P_NIDB_STATE, c->statedb);
    if (c->entdb) fprintf(stream,"%s = %s\n",  P_NIDB_ENT, c->entdb);
    if (c->entiddb) fprintf(stream,"%s = %s\n",P_NIDB_ENTID, c->entiddb);
    if (c->entupddb) fprintf(stream,"%s = %s\n",P_NIDB_ENTUPD, c->entupddb);
    if (c->hostdb) fprintf(stream,"%s = %s\n",P_NIDB_HOST, c->hostdb);
    
    if (c->archive_sqlhost) fprintf(stream,"%s = %s\n", P_NIDB_ARCHIVE_SQLHOST,
				    c->archive_sqlhost);
    if (c->archive_sqluser) fprintf(stream,"%s = %s\n", P_NIDB_ARCHIVE_SQLUSER,
				    c->archive_sqluser);
    if (c->archive_sqlpswd) fprintf(stream,"%s = %s\n", P_NIDB_ARCHIVE_SQLPSWD,
				    c->archive_sqlpswd);
    if (c->archive_sqldb) fprintf(stream,"%s = %s\n", P_NIDB_ARCHIVE_SQLDB,
				  c->archive_sqldb);
    fprintf(stream,"\n");
    return 1;
  }
  zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,NULL);
  return 0;
}

int nidbContextLoad (
  ZkError err ,
  ZkLog log ,
  ZkConfig config,
  NidbContext c )
{
  const char * func = "nidbContextLoad";
  char * str = NULL;
 
  zkErrorClear(err);
  if (c) {
    c->err = err;
    c->log = log;
    
#define _SEC S_MOD_NIDB
    
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_TOPO,&str)) {
      c->topodb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_STATE,&str)) {
      c->statedb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ENT,&str)) {
      c->entdb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_NEIGHBOUR,&str)) {
      c->neibrdb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ENTID,&str)) {
      c->entiddb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ENTUPD,&str)) {
      c->entupddb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_HOST,&str)) {
      c->hostdb = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ARCHIVE_SQLHOST,&str)) {
      c->archive_sqlhost = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ARCHIVE_SQLUSER,&str)) {
      c->archive_sqlhost = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ARCHIVE_SQLPSWD,&str)) {
      c->archive_sqlhost = str;
    }
    if (zkConfigGetParameter(err,config,TYPE_STRING,_SEC,P_NIDB_ARCHIVE_SQLDB,&str)) {
      c->archive_sqlhost = str;
    }
     return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null Parameters passed");
  return 0;
}

int nidbContextVerify (
  ZkError err ,
  NidbContext c )
{
  const char * func = "nidbContextVerify";
  int returnValue = 1;

  zkErrorClear(err);
  if (!c) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,NULL);
    returnValue = 0;
  }

  else if ( ! frDBODriverInitAll(err) ) {
    zkLogWriteContext (c->log, err, func, "Call to frDBODriverInitAll() failed.");
    returnValue = 0;
  }
  else if ( ! zkDB_Open (err, &c->stateDB, c->statedb)) {
    c->stateDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkDB_Open() for stateDB failed.");
    returnValue = 0;
  }

  else if ( ! zkDB_Open (err, &c->topoDB, c->topodb)) {
    c->topoDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkDB_Open() for topoDB failed.");
    returnValue = 0;
  }

  else if ( ! zkDB_Open (err, &c->neibrDB, c->neibrdb)) {
    c->neibrDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkDB_Open() for neibrDB failed.");
    returnValue = 0;
  }

  else if ( ! zkDB_Open (err, &c->hostDB, c->hostdb)) {
    c->hostDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkDB_Open() for hostDB failed.");
    returnValue = 0;
  }

  else if ( ! zkAANew (err, c->entdb,  &c->entDB)) {
    c->entDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkAANew() for entdb failed.");
    returnValue = 0;
  }

  else if ( ! zkAANew (err, c->entiddb,  &c->entidDB)) {
    c->entidDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkAANew() for entiddb failed.");
    returnValue = 0;
  }

  else if ( ! zkAANew (err, c->entupddb,  &c->entupdDB)) {
    c->entupdDB = NULL;
    zkLogWriteContext (c->log, err, func, "Call to zkAANew() for entupddb failed.");
    returnValue = 0;
  }
#if 0
  /* Bug: Move this into it's own wrapper */
  else if ( ! mysql_connect (c->archiveDB,
                        c->archive_sqlhost,
                        c->archive_sqluser,
                        c->archive_sqlpswd)) { 
    zkErrorSet_va(err,func,ZKERR,ZKMOD,ZKERROR,
                  "mysql_connect: %s", mysql_error(c->archiveDB));
    returnValue = 0;
  }
  else if ( mysql_select_db (c->archiveDB,
                        c->archive_sqldb)) {
    zkErrorSet_va(err,func,ZKERR,ZKMOD,ZKERROR,
                  "mysql_select: %s", mysql_error(c->archiveDB));
    returnValue = 0;
  }
#endif

  return returnValue;
}


int nidbContextCleanup (
  ZkError err ,
  NidbContext c )
{
  const char * func = "nidbContextCleanup";
  int return_code = 1;

  zkErrorClear(err);
  if (c) {

    if (c->stateDB)
      zkDBClose(err, &c->stateDB);
    
    if (c->topoDB)
      zkDBClose(err, &c->topoDB);

    if (c->neibrDB)
      zkDBClose(err, &c->neibrDB);

    if (c->hostDB)
      zkDBClose(err, &c->hostDB);

    if (c->entDB)
      zkAAFree(err, &c->entDB);

    if (c->entidDB)
      zkAAFree(err, &c->entidDB);

    if (c->entupdDB)
      zkAAFree(err, &c->entupdDB);

    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,NULL);
  return 0;
}


