/*
 * TEST NIDB AIP UPDATE SERVER
 *
 */

#include "zks.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <zkError.h>
#include <zkLog.h>
#include <zkDB.h>
#include <zkAA.h>
#include <zkDebug.h>
#include <zkBase64.h>
#include <zkDBO.h>
#include <zkDBODefs.h>
#include <zkFile.h>
#include <frSec.h>
#include <frNIDB.h>

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNIDB


static int processUpdate(ZkError err, char * buf)
{
  char ename[64];
  void *pbuf = NULL;
  void *pASbuf = NULL;
  int  status;
  FrDBOTypeEntId dboEnt = NULL;
  FrDBOTypeAipStatus aipStat = NULL;
  size_t esz = 0, psz;
  size_t asz = 0;
  

  sscanf (buf, "%63s %d\n", ename, &status); /* audit: ok */

  if (!zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) &dboEnt, &esz))
    return 0;

  if (!zkDBONew(err, ZKDBO_REG_FRAIPSTATUS, (void **) &aipStat, &asz))
    return 0;

  if (!frEntSetFromStr2(err, &dboEnt->ent, "aip", ename))
    return 0;

  if (!zkDBOPack (err, dboEnt, esz, &pbuf, &psz))
    return 0;

  aipStat->status = status;
  
  if (!zkDBOPack (err, aipStat, asz, &pASbuf, &asz))
    return 0;

  if (!zkDBODump (err, dboEnt, pbuf, psz))
    return 0;

  if (!zkDBODump (err, aipStat, pASbuf, asz))
    return 0;

  return 1;
}
   


int main(int argc, char **argv)
{
  ZkError err;
  ZkLog log;
  struct T_FrNIDBContext context;
  char buf[260];
  int running = 1;
  FILE *testfp;
  
  if (!zkErrorNew(&err)
      || !zkErrorLog (err, &log)
      || !zkLogSetStream (log, stdout)
      || !zkLogEnable (log)) {
    fprintf (stderr, "Can't start the error reporting routines/\n");
    exit(EXIT_FAILURE);
  }
  zkDebugOn();

#if 0

  if (!frNIDBContextNew (err, &context)) {
    exit(EXIT_FAILURE);
  }

  if (!doNIDBSrvArgs (argc, argv, &context)) {
    exit (EXIT_FAILURE);
  }
  
  if (!frNIDBContextInit (err, &context)) {
    exit (EXIT_FAILURE);
  }

#endif
  
  while (fgets(buf, 256, testfp) != NULL) {
    if (!processUpdate(err, buf)) 
      exit(EXIT_SUCCESS);
    sleep (10);
  }
  
  zkErrorFree(&err);
  exit(EXIT_SUCCESS);
}

