#! /bin/sh

set -vx


#
# make sure that we're log as freedom
#
if [ $USER != "freedom" ]
then
    echo "must be log as freedom"
    exit 1
fi

#
# global variable/definition
#

out_file=/usr/freedom/etc/tp.out

db_flush=~louis/bin/db_flush
nidbtool=/usr/freedom/bin/nidbtool
fr_client_init=~louis/work/zk/arch/linux-2.2-ix86/Debug/bin/fr_client_init
fr_cache_test=~louis/work/zk/arch/linux-2.2-ix86/Debug/bin/fr_cache_test

start_niqs="/usr/freedom/sbin/frniqsd --config /usr/freedom/etc/freedom-frniqsd.conf \
            --passphrase.file /usr/freedom/etc/passphrase.file --maxconnections 1"
stop_niqs="killall frniqsd"
niqs_test=/usr/freedom/bin/niqs_test


#
# first write in the output file
#
echo " " > $out_file

#
# to avoid extra error messages must set the CDV
#
$db_flush


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Entity management validation
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function createValid () {

echo "----------------------------------------------------------------------" >> $out_file
echo "Create one entity for each of the valid types."                         >> $out_file
echo "----------------------------------------------------------------------" >> $out_file


$nidbtool <<EOF

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
entity add host host2 host2.dev.zks.net 1.2.3.5 CA10:3848N/07709W SysOp 1
entity add aip aip1 host1 51101 51102 30 3600 1
entity add aip aip2 host2 51101 51102 30 3600 1
entity add link aip1 aip2 20000 48 off

entity add fmg FMG host1 51110
entity add maip maip1 host1 51112
entity add toksrv TokenServer host1 51103
entity add nymsrv NymServer host1 51104
entity add keyqry KeyQryServer host1 51109
entity add niqs NIQS host1 51107
entity add niss NISS host1 51113
entity add lode Lode host1 51121

entity add pop pop-   host1 52001
entity add nmta nmta- host1 52002
entity add imep imep- host1 52003

entity add master master-   host1 55110
entity add nidb nidb-       host1 55110
entity add nym nym-         host1 55110
entity add sersrv sersrv-   host1 55110
entity add keyupd keyupd-   host1 55110
entity add anon anon-       host1 55110
entity add nntp nntp-       host1 55110
entity add year year-       host1 55110
entity add month month      host1 55110
entity add softupd softupd- host1 55110
entity add coreaip coreaip- host1 55110

EOF


$nidbtool >> $out_file <<EOF
entity list
EOF

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function deleteValid() {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Delete one entity for each of the valid types."                         >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >/dev/null <<EOF

entity delete link aip1 aip2

entity delete aip aip1
entity delete aip aip2

entity delete fmg FMG
entity delete maip maip1
entity delete toksrv TokenServer
entity delete nymsrv NymServer
entity delete keyqry KeyQryServer
entity delete niqs NIQS
entity delete niss NISS
entity delete lode Lode

entity delete pop pop-
entity delete nmta nmta-
entity delete imep imep-

entity delete master master-
entity delete nidb nidb-
entity delete nym nym-
entity delete sersrv sersrv-
entity delete keyupd keyupd-
entity delete anon anon-
entity delete nntp nntp-
entity delete year year-
entity delete month month
entity delete softupd softupd-
entity delete coreaip coreaip-

entity delete host host1
entity delete host host2

EOF


$nidbtool >> $out_file <<EOF
entity list
EOF

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function deleteNonExist () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Delete one non-existing entity for each of the valid types."            >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

# put some valid entities in the DB
$nidbtool <<EOF

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
entity add host host2 host2.dev.zks.net 1.2.3.5 CA10:3848N/07709W SysOp 1
entity add aip aip1 host1 51101 51102 30 3600 1
entity add aip aip2 host2 51101 51102 30 3600 1
entity add link aip1 aip2 20000 48 off
entity add fmg FMG host1 51110
entity add maip maip1 host1 51112
entity add toksrv TokenServer host1 51103
entity add nymsrv NymServer host1 51104
entity add keyqry KeyQryServer host1 51109
entity add niqs NIQS host1 51107
entity add niss NISS host1 51113
entity add lode Lode host1 51121
entity add pop pop-   host1 52001
entity add nmta nmta- host1 52002
entity add imep imep- host1 52003

EOF


$nidbtool >> $out_file <<EOF

entity delete link xxx yyy
entity delete aip xxx
entity delete fmg xxx
entity delete maip xxx
entity delete toksrv xxx
entity delete nymsrv xxx
entity delete keyqry xxx
entity delete niqs xxx
entity delete niss xxx
entity delete lode xxx
entity delete pop xxx
entity delete nmta xxx
entity delete imep xxx
entity delete host xxx

entity list
EOF

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function invalidType () {

# need at least one host for next 2 tests
$nidbtool <<EOF
entity add host host1 host1.dev.zks.net 128.128.128.001 CA10:3848N/07709W SysOp 1
EOF


echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create/Delete an entity of an invalid type."                            >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >> $out_file <<EOF
entity add xxx xxx host1 52001

entity list
EOF

$nidbtool >> $out_file <<EOF
entity delete xxx xxx

entity list
EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function nonExistingHost () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create an entity on a non-existing host."                               >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >> $out_file <<EOF
entity add aip aip1 xxx 51101 51102 30 3600 1
entity add niqs NIQS xxx 51107
EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function createTwice () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create an entity twice."                                                >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

# start with an empty DB
$db_flush

$nidbtool >> $out_file <<EOF

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
entity add host host1 host1.dev.zks.net 1.2.3.5 CA10:3848N/07709W SysOp 1
entity add aip aip1 host1 51101 51102 30 3600 1
entity add aip aip1 host1 52101 52102 30 3600 1
entity add host host2 host2.dev.zks.net 1.2.3.6 CA10:3848N/07709W SysOp 1
entity add aip aip2 host2 51101 51102 30 3600 1
entity add link aip1 aip2 20000 48 off
entity add link aip1 aip2 20000 48 off
entity add link aip2 aip1 20000 48 off
entity add niqs NIQS host1 51107
entity add niqs NIQS host1 51108

entity list

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function wrongArgs () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create/Delete an entity with wrong number of arguments"                 >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

# start with a fresh DB
$db_flush

$nidbtool >> $out_file <<EOF

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp
entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W       1
entity add host host1 host1.dev.zks.net 1.2.3.4                   SysOp 1
entity add host host1 host1.dev.zks.net         CA10:3848N/07709W SysOp 1
entity add host host1                   1.2.3.4 CA10:3848N/07709W SysOp 1

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1

entity add aip aip1 host1 51101 51102 30 3600 
entity add aip aip1 host1 51101 51102 30
entity add aip aip1 host1 51101 51102
entity add aip aip1 host1 51101
entity add aip aip1 host1

entity add niqs NIQS host1
entity add niqs NIQS

entity add aip aip1 host1 51101 51102 30 3600 1

entity delete aip
entity delete     aip1

entity delete host
entity delete      host1

entity list

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function queryEntity () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Try to query one entity of each valid type (using niqs_test)."          >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$stop_niqs
sleep 2

$db_flush

# fill DB
$nidbtool > /dev/null <<EOF
entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
entity add host host2 host2.dev.zks.net 1.2.3.5 CA10:3848N/07709W SysOp 1
entity add aip aip1 host1 51101 51102 30 3600 1
entity add aip aip2 host2 51101 51102 30 3600 1
entity add link aip1 aip2 20000 48 off

entity add fmg FMG host1 51110
entity add maip maip1 host1 51112
entity add toksrv TokenServer host1 51103
entity add nymsrv NymServer host1 51104
entity add keyqry KeyQryServer host1 51109
entity add niqs NIQS host1 51107
entity add niss NISS host1 51113
entity add lode Lode host1 51121

entity add pop pop-   host1 52001
entity add nmta nmta- host1 52002
entity add imep imep- host1 52003

EOF

$start_niqs
sleep 2 # give it time to start ...

$niqs_test >> $out_file <<EOF
connect flint
kqs

query description host host1
query description aip aip1
query description fmg FMG
query description maip maip1
query description toksrv TokenServer
query description nymsrv NymServer
query description keyqry KeyQryServer
query description niqs NIQS
query description niss NISS
query description lode Lode
query description pop pop-
query description nmta nmta-
query description imep imep-

mxquery description NONE host aip fmg maip toksrv nymsrv keyqry niqs niss lode pop nmta imep

EOF

$stop_niqs

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# host creation command line validation
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function hostInvalidIP () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create a host with a malformed IP address."                             >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$db_flush

$nidbtool >> $out_file <<EOF

entity add host host1 host1.dev.zks.net 1.2.3.4     CA10:3848N/07709W SysOp 1
entity show host host1 

entity add host host2 host2.dev.zks.net 1.2.3       CA10:3848N/07709W SysOp 1
entity show host host2

entity add host host3 host3.dev.zks.net 1.2         CA10:3848N/07709W SysOp 1
entity show host host3

entity add host host4 host4.dev.zks.net 555.2.3.4   CA10:3848N/07709W SysOp 1
entity show host host4 

entity add host host5 host5.dev.zks.net abc         CA10:3848N/07709W SysOp 1
entity show host host5 

EOF
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function hostInvalidDomain () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create a host with a malformed domain name."                            >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >> $out_file <<EOF
entity add host host6 a,b,c 1.2.3.4     CA10:3848N/07709W SysOp 1
entity show host host6
EOF

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# AIP creation command line validation
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function aipInvalidPort () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create an AIP with a malformed port number."                            >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >>$out_file <<EOF
entity add aip aip2 host1 abc 51102 30 3600 1
entity show aip aip2

entity add aip aip3 host1 51101 abc 30 3600 1
entity show aip aip3

entity add aip aip4 host1 10000000000000 51102 30 3600 1
entity show aip aip4

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# CDV management
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvDefaultSet () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Start with an empty DB and create a host."                              >> $out_file
echo "CDV should be set to a default value."                                  >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

# flush DB
# can't use db_flush since it sets the CDV to 0...
rm /usr/freedom/db/nidb/*/*/*
rm /usr/freedom/db/translog/nidb/*/*/*
rm /usr/freedom/db/niqscache/*/*
rm /usr/freedom/db/translog/niqscache/*/*

$nidbtool >>$out_file <<EOF
entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1

cache get
EOF



}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvEmptyDB () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Start with an empty DB and get the CDV."                                >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

# flush DB
# can't use db_flush since it sets the CDV to 0...
rm /usr/freedom/db/nidb/*/*/*
rm /usr/freedom/db/translog/nidb/*/*/*
rm /usr/freedom/db/niqscache/*/*
rm /usr/freedom/db/translog/niqscache/*/*

$nidbtool >>$out_file <<EOF
cache get
EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvGet () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Set the CDV and get its value."                                         >> $out_file
echo "It should return the last set value."                                   >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >>$out_file <<EOF
cache set 1
cache get

cache set 1234
cache get
EOF


$nidbtool >>$out_file <<EOF
cache get

cache set 1
cache get
EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvPersistancy () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Check CDV persistancy:"                                                 >> $out_file
echo "- restart app"                                                          >> $out_file
echo "- get CDV"                                                              >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >>$out_file <<EOF

cache set 1234
cache get
EOF

$nidbtool >>$out_file <<EOF

cache get
EOF

}


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvConsistancy () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Check CDV consistancy:"                                                 >> $out_file
echo "- start 2 nidbtool"                                                     >> $out_file
echo "- set CDV in one"                                                       >> $out_file
echo "- get CDV in the other"                                                 >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

echo " *** must be done manually *** " >> $out_file

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvIncrement () { 
echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "CDV should be incremented by one on each of these operations:"          >> $out_file
echo " entity add | delete | set | update"                                    >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$db_flush

nidbtool >>$out_file <<EOF

cache get

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
cache get
entity add aip aip1 host1 51101 51102 30 3600 1
cache get

entity update host host1 servoperator xx
cache get

entity set aip aip1 unavailable
cache get

entity delete aip aip1
cache get

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvNoIncrement () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "CDV should NOT be incremented if "--cdv_noincr" is used"                >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$db_flush

$nidbtool >>$out_file <<EOF

cache get

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1 --cdv_noincr
cache get
entity add aip aip1 host1 51101 51102 30 3600 1 --cdv_noincr
cache get

entity update host host1 servoperator xx --cdv_noincr
cache get

entity set aip aip1 unavailable --cdv_noincr
cache get

entity delete aip aip1 --cdv_noincr
cache get

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvFailOp () {       
echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "CDV should not be incremented if the operation is not succesful"        >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

nidbtool >>$out_file <<EOF

cache get

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
cache get

entity add host host1 host1.dev.zks.net 1.2.3.4 CA10:3848N/07709W SysOp 1
cache get

entity delete aip xxx
cache get

entity add host host2
cache get

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvWrap () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Set the CDV to its max value (2147483647) and do an edition operation"  >> $out_file
echo "its value should now be 0"                                              >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$nidbtool >>$out_file <<EOF

cache set 2147483647
cache get

entity add aip aip1 host1 51101 51102 30 3600 1
cache get

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function cdvBroad () {
echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Test that CDV obtained from nidbtool and niqs_test is always the same." >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

#must talk with a valid niqs

$start_niqs
sleep 1

$nidbtool >>$out_file <<EOF
cache get
EOF

$niqs_test >>$out_file <<EOF
connect flint
cachever 
EOF

$stop_niqs

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Region code validation
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

function rcInvalid () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Create a host with a malformed region string:"                          >> $out_file
echo "wrong format"                                                           >> $out_file
echo "out of range lat"                                                       >> $out_file
echo "out of range long"                                                      >> $out_file
echo "invalid region Fip10 code"                                              >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

nidbtool >>$out_file <<EOF

entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10 SysOp 1
entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:3848N/7709W SysOp 1
entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:848N/07709W SysOp 1

entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:9100N/07709W SysOp 1
entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:9001N/07709W SysOp 1

entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:3848N/18100W SysOp 1
entity add host hosta hosta.dev.zks.net 1.2.3.4 CA10:3848N/18001W SysOp 1

entity add host hosta hosta.dev.zks.net 1.2.3.4 AA10:3848N/07709W SysOp 1

EOF

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Information Availability
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

function infoMailEntAvail () {

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Use niqs_test to validate that mailing entities can be queried."        >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

echo " *** done in test Try to query one entity of each valid type (using niqs_test)." >>$out_file

echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "frNetInfo API gives the information on the new mailing entities."       >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

#must talk with a valid niqs
$fr_client_init --niqs.hostname daravongl --save etc/niqs.cache

$fr_cache_test >>$out_file <<EOF
load etc/niqs.cache
list --type pop
list --type nmta
list --type imep
EOF

rm etc/niqs.cache

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
function infoEntNotInDB () {
echo " " >> $out_file
echo "----------------------------------------------------------------------" >> $out_file
echo "Query for an entity type that is not in NIDB."                          >> $out_file
echo "----------------------------------------------------------------------" >> $out_file

$start_niqs
sleep 2

niqs_test >>$out_file <<EOF

connect flint

mxquery description NONE coreaip

query description coreaip coreaip1

EOF

$stop_niqs

}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

function allEntityMgmtValid () {
    createValid
    deleteValid
    deleteNonExist
    invalidType
    nonExistingHost
    createTwice
    wrongArgs
    queryEntity
}

function allHostCreation () {
    hostInvalidIP
    hostInvalidDomain
}

function allAipCreation () {
    aipInvalidPort
}

function allCDVMgmt () {
    cdvDefaultSet
    cdvEmptyDB
    cdvGet
    cdvPersistancy
    cdvConsistancy
    cdvIncrement
    cdvNoIncrement
    cdvFailOp
    cdvWrap
    cdvBroad
}

function allRcValid () {
    rcInvalid
}

function allInfoAvail () {
    infoMailEntAvail
    infoEntNotInDB
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------


if [ -f $out_file ]
then
    rm $out_file
fi

allEntityMgmtValid

allHostCreation

allAipCreation

allCDVMgmt

allRcValid

allInfoAvail

