/*
 * $Id: fr_niqsctx.h,v 1.18 2000/11/24 15:02:25 marcp Exp $
 *
 * Luc Boulianne <luc@zks.net>
 */


#ifndef FRNIQSCTX_H_
#define FRNIQSCTX_H_

#include <zks.h>
#include <sys/types.h>
#include <stdio.h>
#include <zkConfig.h>
#include <zkIntTypes.h>
#include <zkParms.h>
#include <frEnt.h>
#include <frFilenames.h>
#include <frNetworkDefaults.h>
#include <frKeyQry.h>
#include <frSec.h>
#include <ZKIP.h>
#include <zkipEngine.h>
#include <fr_nidbctx.h>
#include "fr_niqsstat.h"
#include "frPsrCache.h"
#include "frToken.h"
#include "frTokenType.h"
#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */


#define VERSION_NIQS_STR "Zero-Knowledge Network Information Query Server (NIQS) ("__DATE__", "__TIME__")"


enum EfrPool
{
  pool5Minutes = 0,
  poolHour,
  poolDay,
  poolMax  /* Must be the last one */
} ;

typedef struct T_FrNiqsCtx {
  ZkLog log;                  /* Log object */
  ZkConfig config;            /* Configuration object */
  const char * progname;      /* Program name */
  const char * configfile;    /* Configuration file */
  const char * logfile;       /* Log file */
  const char * clientinfo;    /* client info file */
  const char * motdfile;      /* Message Of The Day file */
  const char * ttfile;        /* The token type description file */
  const char * accessctrllist;/* access contreol list file */
  const char * niqscache;     /* Name of the NIQS response cache database */

  uint16_t niqscache_ttl;     /* Time-to-Live value for responses in niqscache */

  int debug;                  /* debug or not */
  int daemon;                 /* fork or not */
  const char *pidfile;        /* PID file */
  FILE *pidFptr;              /* Pointer to locked PID file */
  uint32_t cacheDataVersion;  /* Version # to force client cache flush */
 
  void   *ttBuf;              /* The pre-sign response to TokenInfo requests */
  size_t  ttBufSz;            /* Its size */

  FrKeyQry kq;                /* Key query cache */
  FrKeyQryHost kh;            /* Key query host IP/port/descriptor */
  unsigned short kqtimeout;   /* Timeout */
  unsigned short kqretries;   /* # of Retries */

  const char *ppfile;         /* Passphrase file */
  const char *ppprompt;       /* Passphrase prompt */
  const char *sigdir;         /* Signature directory */
  const char *cryptdir;       /* Cryption directory */
  FrSecKeyChain kc;           /* Key chain */
  int sigHandle;              /* Signature key handle */
  int cryptHandle;            /* Cryption key handle */

  unsigned short tcpPort;     /* My TCP port */
  int tcpListenFd;            /* My TCP fd */
  int tcpFd;                  /* My TCP descriptor */

  int listen_queue_sz;        /* Size of the listen queue */
  unsigned short cmdPort;     /* TCP port to listen to for commands */
  int cmdOpen;                /* Command port is connected / refuse access */
  int cmdListenq;             /* listen() queue value */
  int cmdRetries;             /* Number of connection retries */
  const char *cmdWelcome;     /* Welcome message on the descriptor */
  const char *cmdRefused;     /* Welcome message on the descriptor */

  NidbContext nidb;           /* The NIDB context information */

  ZkipSession session;        /* ZKIP Session control structure */
  ZkipEngine engine;          /* Keeps track of zkip command callbacks, etc */
  uint16_t verbose;           /* ZKIP verbose logging level */
  uint32_t maxdata;           /* Maximum amount of data/zkip command 
                               * ( 0 means no limit)
                               */
  uint16_t my_children;       /* Number of currently active children */
  pid_t * child;              /* Array, stores the PIDs of the running 
                               * child server processes.
                               */
  uint16_t max_connections;   /* Max number of simultaneous connections */
  uint16_t mxconntime;        /* Max duration of client connections (seconds) */
  uint32_t processLife;       /* Maximum number of connections a server will */
                              /* process before terminating.  Processes are
                               * immortal when this value is set to 0
                               */
  uint32_t connections_handled; /* The total number of connections a child 
                                 * server has handled during its existance.  
                                 * This is used with the processLife parameter
                                 * to determine when a child should terminate
                                 */

  FrPsrCache qc;                /* The query response cache */

  int           nbCmdsForThisConnexion;
  FrNiqsStats  *stats[ poolMax ];  /* server stats for last 5 minutes, hour, day */
 
} FrNiqsCtx;


/* Signal handler */
extern void frNiqsSigHandler(int);

#define DEFAULT_LOGFILE		FREEDOM_LOGDIR"/niqsd"
#define DEFAULT_CONFIGFILE	FREEDOM_CONFDIR"/niqsd.conf"
#define DEFAULT_PIDFILE		FREEDOM_PIDDIR"/niqsd.pid"
#define DEFAULT_CLIENTINFO      FREEDOM_CONFDIR"/clientinfo.conf"
#define DEFAULT_TTFILE          FREEDOM_CONFDIR"/tokentypeid.conf"
#define DEFAULT_MOTDFILE        FREEDOM_CONFDIR"/motd.conf"
#define DEFAULT_ACCESSCTRLLIST  FREEDOM_CONFDIR"/freedom-access.conf"
#define DEFAULT_KQHOST		FR_DFLT_KEYQRY_SERVER_HOST
#define DEFAULT_KQPORT		FR_DFLT_KEYQRY_SERVER_PORT
#define DEFAULT_KQTIMEOUT	5
#define DEFAULT_KTRETRIES	10
#define DEFAULT_CACHE_DATA_VERSION 0
#define DEFAULT_NIQSCACHE       "niqs-cache"
#define DEFAULT_NIQSCACHE_TTL   120
#define DEFAULT_PROCESS_LIFE    0
  
#define DEFAULT_PPPROMT		"Please enter the server passphrase"
#define DEFAULT_PPFILE		FREEDOM_CONFDIR"/passphrase"
#define DEFAULT_CRYTPTDIR       FREEDOM_PRIVKEYDIR"/NetInfoQryServ/crypt"
#define DEFAULT_SIGDIR		FREEDOM_PRIVKEYDIR"/NetInfoQryServ/sig"

#define DEFAULT_MAXDATA         0
#define DEFAULT_VERBOSE         0
#define DEFAULT_MAX_CONNECTIONS 50
#define DEFAULT_MXCONNTIME      900

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FRNIQSCTX_H_ */
