/*
 * $Id: fr_niqsstat.h,v 1.3 2000/10/04 12:56:27 marcp Exp $
 *
 * daniel faguy <danielf@zks.net>
 */


#ifndef FRNIQSSTAT_H_
#define FRNIQSSTAT_H_

#include <zks.h>
#include <sys/types.h>
#include "zkStatistic.h"

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */


#define  NB_MAX_COUNTER  28
#define  NB_MAX_SAMPLES  2

enum EfrCounters
{
  cntTotalConnexion = 0,
  cntError,
  cntChild,
  cntAccessCtrlList,
  cntAvailable,               /* includes those that hit the cache */
  cntAvailableHit,
  cntCacheVer,                /* includes those that hit the cache */
  cntCacheVerHit,
  cntClientInfo,              /* includes those that hit the cache */
  cntClientInfoHit,
  cntList,                            /* includes the since 0 ones */
  cntListSince0,              /* includes those that hit the cache */
  cntListSince0Hit,
  cntMxQryDescription,                /* includes the since 0 ones */
  cntMxQryDescriptionSince0,  /* includes those that hit the cache */
  cntMxQryDescriptionSince0Hit,
    /* the <cntQry...> must precede its <cntQry...Hit> counterpart */
  cntQryDescription,          /* includes those that hit the cache */
  cntQryDescriptionHit,
  cntQryStatus,               /* includes those that hit the cache */
  cntQryStatusHit,
  cntQryTopology,             /* includes those that hit the cache */
  cntQryTopologyHit,
  cntStatus,                          /* includes the since 0 ones */
  cntStatusSince0,            /* includes those that hit the cache */
  cntStatusSince0Hit,
  cntTokenInfo,               /* includes those that hit the cache */
  cntTokenInfoHit,
  cntMax /* Must be the last one */
} ;


enum EfrValues
{
  valSimultaneousConnexions = 0,
  valMax /* Must be the last one */
} ;


enum EfrSamples
{
  smpCmdsPerConnexion = 0,   /* does not include: quit and invalid */
  smpTimePerConnexion,                               /* In seconds */
  smpMax /* Must be the last one */
} ;


typedef struct
{
  ZkClock                lastReport;
  ZkStatisticPool        pool ;
  ZkStatisticIntSamples  smp[ smpMax ] ;
  ZkStatisticCounter     cnt[ cntMax ] ;
  ZkStatisticIntValue    val[ valMax ] ;
} FrNiqsStats ;


extern int frNiqsStatPoolDescNew(
    ZkError err,
    ZkStatisticPoolDesc *pDesc
) ;

extern int frNiqsStatPoolDescFree(
    ZkError err,
    ZkStatisticPoolDesc *pDesc
) ;

extern int frNiqsStatsNew(
    ZkError                err,
    ZkLog                  log,
    FrNiqsStats          **pStats,
    ZkStatisticPoolDesc    desc,
    const char            *name,
    ZkBool                 bShared
) ;

extern int frNiqsStatsFree(
    ZkError                err,
    FrNiqsStats          **pStats
) ;

extern int frNiqsStatsReportAndTransfer(
    ZkError               err,
    ZkLog                 log,
    FrNiqsStats          *pStatsToReport,
    FrNiqsStats          *pStats
) ;
#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FRNIQSSTAT_H_ */
