/*
 * Process a Description Query
 *
 * $Id: niqsDesc.c,v 1.7 1999/11/01 22:05:10 filou Exp $
 *
 * Luc Boulianne <luc@zeroknowledge.com>
 */

static const char * module_id_ =
"$Id: niqsDesc.c,v 1.7 1999/11/01 22:05:10 filou Exp $";

#include "zks.h"
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <zkError.h>
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "niqscmds.h"

#define FREEIT(x) if (x) { free (x); x = NULL; }

int frNiqsProcessDesc(
  ZkError err,
  FrNiqsCtx * c,
  uint32_t cmdid,
  FrEntIdExp *entity,
  void ** signed_response,
  size_t * sresz,
  int *found)
{
   static const char *funcname = "frNiqsProcessDesc";
  FrCryptPubKey mySigKey = NULL;
  int retcode = 0;
  size_t srsz = 0, rsz = 0, prsz = 0;
  uint32_t resp_vers;
  FrNidbObj resp = NULL;
  void * presp = NULL;
  void * sigresp = NULL;
  
  (void) module_id_; /* Get rid of compile time warning */
  
  zkErrorClear(err);
  if (!c || !signed_response || !sresz) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
  } else {
    if (CMDID_MAJVER(cmdid) >= 2) {
      resp_vers = FRNIDBOBJ_CURRENT_VERSION;
    }

    else {
      resp_vers = 0;
    }

    if ( ! frNidbGetDescObjForEntity (err, c->nidb, entity, resp_vers, &resp,
                                      &rsz, found)) {
      zkLogWriteContext (c->log, err, funcname, "Error processing request for [DESC %s/%d]",
                         entity->name, entity->type);      

    } else if ( ! *found ) {
      zkLogWriteContext (c->log, err, funcname, "Error unable to locate entity: %s/%d",
                         entity->name, entity->type);      
      retcode = 1;

    } else if ( ! zkDBOPack(err, resp, rsz, & presp, & prsz) ) {
      zkLogWriteContext (c->log, err, funcname, "Error: unable to pack response: %s/%d",
                         entity->name, entity->type);      

    } else  if (! frSecKeyChainGetVersion(err, c->kc, c->sigHandle, 0, &mySigKey) ) {
      zkLogWriteContext (c->log, err, funcname, "Couldn't get current signing key from my key chain.");      

    } else if ( ! frSecSignData(err, mySigKey, presp, prsz, (void **) &sigresp, &srsz) ) {
      zkLogWriteContext (c->log, err, funcname, "Couldn't sign response.");      
      FREEIT(sigresp);
  
    } else {
      *signed_response = sigresp;
      *sresz = srsz;
      retcode = 1;
    }
    FREEIT(resp);
    FREEIT(presp);
  }

  return retcode;
  
}
