/*
 * $Id: niqsList.c,v 1.10 1999/11/01 22:05:57 filou Exp $
 */

static const char * module_id_ = "$Id: niqsList.c,v 1.10 1999/11/01 22:05:57 filou Exp $";

#include "zks.h"
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "time.h"
#include "zkError.h"
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "niqscmds.h"

/**
 * Process the list command
 */
int frNiqsProcessList(
  ZkError err,
  FrNiqsCtx * c,
  time_t since,
  uint32_t entity_mask,
  void ** signed_response,
  size_t * sresz,
  int *found)
{
  static const char *funcname = "frNiqsProcessList";
  FrCryptPubKey mySigKey = NULL;
  FrDBOTypeEntityID ent_ident_list = NULL;
  int retcode = 0;
  const char * ent_type = NULL; /* Returns an entity type token string */
  size_t srsz = 0, rsz = 0;
  size_t numents;
  size_t n = 0; /* Loop index */
  size_t base = 0;  /* Used to move within the initial response buffer */
  char * resp = NULL;
  void * sigresp = NULL;
  int bytes = 0;             /* Number of bytes written per list entry */
  
  (void) module_id_; /* Get rid of compile time warning */
  
  zkErrorClear(err);
  if (!c || !signed_response || !sresz) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if ( ! frNidbGetEntitiesInTypesetUpdatedSince (err, c->nidb, since, entity_mask,
                                                 &ent_ident_list, &numents)) {
    zkLogWriteContext (c->log, err, funcname, "Error processing request for [LIST]");      
    goto finish;
  }


  /*
   * Allocate the response buffer
   */

  rsz = numents * (FRENT_PORTABLE_SIZE + 40);
  if ((resp = (char *) malloc(rsz)) == NULL) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENOMEM,"malloc");
    goto finish;
  }
 
  /*
   * Load it with the list. DO NOT CHANGE THIS FORMAT.  this format is used
   * by the niqs library to reconstruct a ZkList at the client side
   */

  for (n = 0; n < numents; n++) {
    if ((ent_type = frEntTypeGetStr(ent_ident_list[n].ent.type)) == NULL) {
      zkErrorSet_va(err,funcname,ZKERR,ZKMNIQS,ZKEDATABAD,
                    "Unrecognized entity type %u",
                    ent_ident_list[n].ent.type);
      goto finish;
    } else if ((bytes = snprintf(&(resp[base]),rsz - base,"%d %s %s %lu\015\012",
                                 ent_ident_list[n].entid,ent_type,
                                 ent_ident_list[n].ent.name,
                                 ent_ident_list[n].last_update)) < 0) {
      zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKEDATABAD,
                 "Ran out of buffer space!");
      goto finish;
    }
    base += bytes;
  }
  rsz = base; 

  if (!frSecKeyChainGetVersion(err, c->kc, c->sigHandle, 0, &mySigKey)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't get current signing key from my key chain.");      
    return 0;
  }

  /* Sign the response */
  if (!frSecSignData(err, mySigKey, resp, rsz, (void **) &sigresp, &srsz)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't sign response.");      
    return 0;
  }

  *found = 1;
  *signed_response = sigresp;
  *sresz = srsz;
  retcode = 1;
  
 finish:
  if (resp) {
    free(resp);
  }
  if (ent_ident_list) {
    free(ent_ident_list);
  }
  return retcode;

}
