/*
 * Process a Description List command
 * 
 * $Id: niqsMxQuery.c,v 1.4 2000/11/24 15:02:25 marcp Exp $
 * 
 * Stephane Ayotte <stephanea@zeroknowledge.com>
 */


static const char * module_id_ = "$ID$";

#include "zks.h"
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <zkError.h>
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "niqscmds.h"

int frNiqsProcessMxQuery( ZkError err,
			  FrNiqsCtx * context,
			  uint32_t cmd_id,
			  const char * attribute,
			  time_t since,
			  uint16_t encoding,
			  uint32_t entity_mask,
			  void ** signed_response,
			  size_t * signed_response_length,
			  int * valid_data)
{
  static const char * funcname = "frNiqsProcessMxQuery";
  FrDBOTypeEntityID ent_ident_list = NULL;
  size_t num_entities = 0;
  int retcode = 0;
  size_t desc_list_size = 0;
  FrNidbObj desc_list = NULL;
  int found = 0;
  void *packed = NULL;
  size_t psz = 0;
  unsigned char *comp_buf = NULL;
  size_t comp_buf_sz = 0;
  void *signed_resp = NULL;
  size_t ssz = 0;
  FrCryptPubKey mySigKey = NULL;

  (void) module_id_; /* Get rid of compile time warning */
  (void) cmd_id;
  (void) attribute;
  
  zkErrorClear(err);

  if( !context || !signed_response || !signed_response_length || !valid_data) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    goto finish;
  } 

  *valid_data = 0;
  
  if( !frNidbGetEntitiesInTypesetUpdatedSince( err, 
					       context->nidb, 
					       since,
					       entity_mask,
					       &ent_ident_list,
					       &num_entities)) {
    zkLogWriteContext( context->log, err, funcname, 
		       "Error getting entity list");
    goto finish;
  }
  
  if( !frNidbGetDescObjListFromTypeEntityIDList( err, context->nidb, 
						ent_ident_list, 
						num_entities,
						FRNIDBOBJ_CURRENT_VERSION,
						&desc_list,
						&desc_list_size,
						&found)) {
    zkLogWriteContext( context->log, err, funcname,
		       "Error getting Description ObjList");
    goto finish;
  }
      
  if( !zkDBOPack( err, desc_list, desc_list_size, &packed, &psz ) ) {
    zkLogWriteContext( context->log, err, funcname,
		       "Error Packing Description List" );
    goto finish;
  }

  if( !frNiqsCompress(err, encoding, (const unsigned char *)packed, psz, (unsigned char **)&comp_buf, &comp_buf_sz) ){
    zkLogWriteContext( context->log, err, funcname,
		       "Error compressing description list");
    goto finish;
  }

  if(!frSecKeyChainGetVersion(err, context->kc, context->sigHandle, 0, 
			      &mySigKey)){
    zkLogWriteContext( context->log, err, funcname,
		       "Error get signature key");
    goto finish;
  }

  if(!frSecSignData(err,mySigKey, comp_buf, comp_buf_sz, 
		    (void**)&signed_resp, &ssz )) {
    zkLogWriteContext( context->log, err, funcname,
		       "Error signing description list");
    goto finish;
  }

  *signed_response = signed_resp;
  *signed_response_length = ssz;
  *valid_data = 1;
  retcode = 1;
  
 finish:
  if(desc_list){
    frNidbObjListFree( err, &desc_list );
  }
  if(comp_buf) {
    free(comp_buf);
  }
  if (packed) {
    free(packed);
  }
  if (ent_ident_list) {
    free(ent_ident_list);
  }
    return retcode; /* return 1 if OK */
}

