/*
 * $Id: niqsQuery.c,v 1.15 2000/08/02 14:03:27 louis Exp $
 */

static const char * module_id_ = "$Id: niqsQuery.c,v 1.15 2000/08/02 14:03:27 louis Exp $";

#include "zks.h"

#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <zkError.h>
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "niqscmds.h"


int frNiqsProcessQuery(
  ZkError err ,
  FrNiqsCtx * c,
  uint32_t cmdid,
  const char * attribute_token,
  FrEntIdExp * entity,
  void ** response,
  size_t * resz,
  int *found)
{
  static const char *funcname = "frNiqsProcessQuery";
  FrCryptPubKey mySigKey = NULL;
  FrDBOTypeEntityID entid = NULL;
  FrNiqAttr attr;
  int retcode = 0;
  size_t srsz = 0, rsz = 0;
  uint32_t resp_vers;
  void * resp = NULL;
  void * sigresp = NULL;
  
  (void) module_id_; /* Get rid of compile time warning */
  
  zkErrorClear(err);
  if (!c || !attribute_token || !entity || !response || !resz || !found) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if (!frNiqsTokenToAttribute (err , attribute_token, &attr)) {
    zkLogWriteContext (c->log, err, funcname, "Call to frNiqsTokenToAttribute failed.");      
    return 0;
  }

  if (!frNidbFindEntityIdDboUsingEntity(err, c->nidb, entity, &entid, found)) {
    zkLogWriteContext (c->log, err, funcname, "Call to frNidbFindEntityIdDboUsingEntity failed.");      
    goto finish;
  }

  if (CMDID_MAJVER(cmdid) >= 2) {
    resp_vers = FRNIDBOBJ_CURRENT_VERSION;
  }
  else {
    resp_vers = 0;
  }

  if (! *found) {
    retcode = 1;
    goto finish;
  }

  switch (attr) {
  case FRNIQ_ATTR_TOPOLOGY:
    if ( ! frNtdbGetCompleteLocalTopology(err, c->nidb, entid, (void **)&resp,
					  &rsz, found) ) {
      zkLogWriteContext (c->log, err, funcname, "Error processing topology request for %s/%d",
                         entid->ent.name, entid->ent.type);      
      goto finish;
    }
    if ( ! *found) {
      retcode = 1;
      zkLogWriteContext (c->log, err, funcname, "Unable to find %s/%d in the topology database.",
                         entid->ent.name, entid->ent.type);      
      goto finish;
    }
    break;
  case FRNIQ_ATTR_STATUS:
    if ( ! frNidbGetStatusForEntIdent(err, c->nidb, entid->entid, resp_vers, 
                                      (void **)&resp, &rsz, found) ) {
      zkLogWriteContext (c->log, err, funcname, "Error processing status request for %s/%d",
                         entid->ent.name, entid->ent.type);      
      goto finish;
    }
    if ( ! *found) {
      retcode = 1;
      zkLogWriteContext (c->log, err, funcname, "Unable to find %s/%d in the status database.",
                         entid->ent.name, entid->ent.type);      
      goto finish;
    }
    break;
  default:
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKERROR,
		  "Unknown attibute requested [%s]", attribute_token);
    return 0;
  }

  if (!frSecKeyChainGetVersion(err, c->kc, c->sigHandle, 0, &mySigKey)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't get current signing key from my key chain.");      
    return 0;
  }
  /* Sign the response */

  if (!frSecSignData(err, mySigKey, resp, rsz, (void **) &sigresp, &srsz)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't sign response.");      
    return 0;
  }

  *response = sigresp;
  *resz = srsz;
  retcode = 1;
  
 finish:
  if (resp) free (resp);
  if (entid) free (entid);
  return retcode;

}
