/*
 * Process a Status Query
 *
 * $Id: niqsStatus.c,v 1.6 1999/11/01 22:06:42 filou Exp $
 *
 * Luc Boulianne <luc@zeroknowledge.com>
 */

static const char * module_id_ =
"$Id: niqsStatus.c,v 1.6 1999/11/01 22:06:42 filou Exp $";

#include "zks.h"
#include "zkIntTypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <zkError.h>
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "frNidbObj.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "niqscmds.h"

#define FREEIT(x) if (x) { free (x); x = NULL; }

int frNiqsProcessStatus(
  ZkError err,
  FrNiqsCtx * c,
  uint32_t cmdid,
  time_t since,
  uint32_t mask,
  void ** signed_response,
  size_t * sresz,
  int *found)
{
  static const char *funcname = "frNiqsProcessStatus";
  FrCryptPubKey mySigKey = NULL;
  int retcode = 0;
  size_t srsz = 0, prsz = 0, rsz = 0;
  uint32_t resp_vers;
  FrNidbObj resp = NULL;
  void * presp = NULL;
  void * sigresp = NULL;
  
  (void) module_id_; /* Get rid of compile time warning */
   
  zkErrorClear(err);
  if (!c || !signed_response || !sresz) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameter passed");
    return 0;
  }

  if (CMDID_MAJVER(cmdid) >= 3) {
    resp_vers = FRNIDBOBJ_CURRENT_VERSION;
  }

  else {
    resp_vers = 0;
  }

  if ( ! frNidbGetStatusUpdateSince (err, c->nidb, since, mask, resp_vers, &resp, &rsz, found)) {
    zkLogWriteContext (c->log, err, funcname, "Error processing request for [STATUS]");      
      goto finish;
#if 0
      /* note: I probably don't need this */
  } else if ( ! *found ) {
    zkLogWriteContext (c->log, err, funcname, "Error unable to find status information since: %lu/%s",
                       since, "[unavailable]");      
    retcode = 1;
#endif      
  } else if ( ! zkDBOPack(err, resp, rsz, & presp, & prsz) ) {
    zkLogWriteContext (c->log, err, funcname, "unable to pack response");      
    
  } else if (!frSecKeyChainGetVersion(err, c->kc, c->sigHandle, 0, &mySigKey)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't get current signing key from my key chain.");      

  } else if (!frSecSignData(err, mySigKey, presp, prsz,
                            (void **) &sigresp, &srsz)) {
    zkLogWriteContext (c->log, err, funcname, "Couldn't sign response.");      

  } else {
    *found = 1;
    *signed_response = sigresp;
    *sresz = srsz;
    retcode = 1;
  }
 finish:

  FREEIT (resp);
  FREEIT (presp);

  return retcode;

}
