/*
 * $Id: niqscmds.c,v 1.32 2000/11/24 15:02:25 marcp Exp $
 */

static const char * module_id_ = "$Id: niqscmds.c,v 1.32 2000/11/24 15:02:25 marcp Exp $";

#include "zks.h"
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <time.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "zkError.h"
#include "ZKIP.h"
#include "zkCStr.h"
#include "frEnt.h"
#include "frDBOTypes.h"
#include "fr_niqsctx.h"
#include "frNiqs.h"
#include "fr_nidb.h"
#include "niqsd.h"
#include "frTokenType.h"
#include "niqscmds.h"
#include "zkBase64.h"
#include "zkFile.h"


#define QUERY_NAME_SZ 128   // for cache queries


/*
 * Internal function prototypes
 */

static int niqsUtilMkEntityBitmap ( ZkError , ZkLog, ZkipSession , int ,
                                    int , uint32_t * );

/*
 *
 */

static int
niqsUtilMkEntityBitmap (
  ZkError err ,
  ZkLog log,
  ZkipSession session ,
  int start_parm ,
  int parameters ,
  uint32_t * bitmap )
{
  static const char * func = "niqsUtilMkEntityBitmap";
  int ret = 1;
  unsigned char * sval = NULL;
  int parm = start_parm;
  FrEntIdType ent_type = FRENT_TYPE_UNUSED_;

  if (session && bitmap) {
    *bitmap = 0;
    while (parm < parameters) {
      if (!zkipSessionParameter(err,session,parm,
                                (const unsigned char **) &sval)) {
        zkLogWriteContext (log, err, func, "Call to zkipSessionParameter() failed");

        if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to get entity type parameter",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (log, err, func, "Call #1 to zkipSessionResponse() failed");
        }
        ret = 0;
        break;
      } else if ((ent_type = frEntTypeFromStr(sval)) == FRENT_TYPE_UNUSED_) {
        zkLogWriteContext (log, err, func, "Command contains unrecognized entity type");
        if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Command contains unrecognized entity type",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (log, err, func, "Call #2 to zkipSessionResponse() failed");
        }
        ret = 0;
        break;
      }
      (*bitmap) |= (1 << ent_type);
      parm++;
    } } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
    ret = 0;
  }

  return ret;
}


static int myPSRCacheStore_ttl(
  ZkError      err,
  FrNiqsCtx   *c,
  const void  *qry,
  size_t       qry_sz,
  void        *rsp,
  size_t       rsp_sz )
{
  FrPSRStamp  stamp ;

  stamp.type  = EPSRStampTtl ;
  stamp.u.ttl = c->niqscache_ttl ;

  return frPsrCacheStore( err, c->qc, qry, qry_sz, rsp, rsp_sz, &stamp ) ;
}


static int myPSRCacheStore_cdv(
  ZkError      err,
  FrNiqsCtx   *c,
  const void  *qry,
  size_t       qry_sz,
  void        *rsp,
  size_t       rsp_sz )
{
  FrPSRStamp  stamp ;

  stamp.type      = EPSRStampValue ;
  stamp.u.val.len = sizeof c->cacheDataVersion ;
  stamp.u.val.ptr = &c->cacheDataVersion ;

  return frPsrCacheStore( err, c->qc, qry, qry_sz, rsp, rsp_sz, &stamp ) ;
}


static int myPSRCacheLookup_ttl(
  ZkError      err,
  FrNiqsCtx   *c,
  const void  *qry,
  size_t       qry_sz,
  void       **rsp,
  size_t      *rsp_sz,
  ZkBool      *in_cache )
{
  FrPSRStamp  stamp ;

  stamp.type = EPSRStampTtl ;

  return frPsrCacheLookup( err, c->qc, qry, qry_sz, rsp, rsp_sz, in_cache, &stamp ) ;
}


static int myPSRCacheLookup_cdv(
  ZkError      err,
  FrNiqsCtx   *c,
  const void  *qry,
  size_t       qry_sz,
  void       **rsp,
  size_t      *rsp_sz,
  ZkBool      *in_cache )
{
  FrPSRStamp  stamp ;

  stamp.type      = EPSRStampValue ;
  stamp.u.val.len = sizeof c->cacheDataVersion ;
  stamp.u.val.ptr = &c->cacheDataVersion ;

  return frPsrCacheLookup( err, c->qc, qry, qry_sz, rsp, rsp_sz, in_cache, &stamp ) ;
}


/*
 *
 */

int
niqsCmdCacheDataVersion (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  int return_code = _SESSION_ERROR;
  FrNiqsCtx * c = (FrNiqsCtx *) data; /* The application context */
  static const char * func = "niqsCmdCacheDataVersion";
  int parameters = 0; /* Number of command parameters */
  FrCryptPubKey mySigKey = NULL; /* My private signature key */
  unsigned char d[4]; /* For transporting the cache data version number */
  void * sbuf = NULL; /* The client info response buffer (signed) */
  size_t ssz = 0; /* Size of the client info response buffer (signed)  */
  int in_cache = 0;
  char resp_txt[128]; /* For a more pleasant explanation string in case a
                       * human tries to read the response (this will be the
                       * string token in the response line)
                       */

  if (session && data) {
    zkStatisticCounterIncrement(c->stats[0]->cnt[cntCacheVer]);
    c->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;

    /*
     * Make sure we did not supply any parameters to the command
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (c->log, err, func,
                         "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Could not get command parameters",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
    } else if (parameters != 0) {
      zkLogWriteContext (c->log, err, func,
                         "Command does not require parameters");
      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                               "Command does not require parameters",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #2 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;

    }
    /*
     * Bug 6638 - NIDB Lookup for the CDV is done at the connection time. This
     * is done in frNiqsRunZKIPSession().
     */
    /* else if (!frNiqsGetCacheDataVersionFromNidb(err,c,&c->cacheDataVersion)) {
      zkLogWriteContext (c->log, err, func, "frNiqsGetCacheDataVersionFromNidb() failed");

      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                               "frNiqsGetCacheDataVersionFromNidb() failed",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #2 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;

    } */
    
    else if (!myPSRCacheLookup_cdv(err,c,"CACHEVER",8,&sbuf,&ssz,&in_cache)) {
      zkLogError(c->log,err);
      zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Cache lookup failure",
                          "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
      return_code = _SESSION_ERROR;

    } else if (in_cache) {
      zkipSessionResponse(err,session,0,_SERVICE_READY,
                          "version",
                          "CACHEDATAVERSION",ZKIP_ENCODING_NONE,ssz,sbuf);

      zkStatisticCounterIncrement(c->stats[0]->cnt[cntCacheVerHit]);

      if (c->debug) {
        zkLogWrite(c->log, "Cache Data Version Returned [HIT]");
      }
    } else {

      /*
       * Convert the cache data version and send it back to the client and
       * create a slightly more pleasant response string for human consumption
       */

      d[0] = (c->cacheDataVersion & 0xFF000000) / 0x1000000;
      d[1] = (c->cacheDataVersion & 0xFF0000) / 0x10000;
      d[2] = (c->cacheDataVersion & 0xFF00) / 0x100;
      d[3] = (c->cacheDataVersion & 0xFF);
      snprintf(resp_txt,128,"CacheDataVersion %lu",(unsigned long)c->cacheDataVersion);

      /*
       * Send it to the client and we're done.
       */
      if (!(frSecKeyChainGetVersion(err,c->kc,c->sigHandle,0,&mySigKey) &&
            frSecSignData(err,mySigKey,d,4,&sbuf,&ssz))) {
        zkLogWriteContext (c->log, err, func,
                           "Failed to retreive signature key or sign response");
        if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                            "Failed to sign data",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func,
                             "Call #3 to zkipSessionResponse() failed");
        }
        return_code = _SESSION_ERROR;
      } else {
        if (!zkipSessionResponse(NULL,session,0,_SERVICE_READY,
                            resp_txt,"CACHEDATAVERSION",ZKIP_ENCODING_BINARY,
                            ssz,sbuf)) {
          zkLogWriteContext (c->log, err, func,
                             "Call #4 to zkipSessionResponse() failed");
        } else if (c->debug) {
          zkLogWrite(c->log, "Cache Data Version Returned");
        }
        if (!myPSRCacheStore_cdv(err,c,"CACHEVER",8,sbuf,ssz)) {
          zkLogError(c->log,err);
        }
        if (c->debug) {
          zkLogWrite(c->log,"CACHE: Storing the cache data version");
        }
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

int
niqsCmdTokenInfo (
  ZkError err ,
  ZkipSession session  ,
  void * data  )
{
  static const char * func = "frNiqsTokenInfo";
  FrNiqsCtx * c = (FrNiqsCtx *) data; /* The application context */
  int return_code = _SESSION_ERROR;
  int parameters = 0; /* Number of command parameters */

  (void) module_id_; /* Get rid of compile time warning */
  zkErrorClear(err);
  if (session && data)  {

    zkStatisticCounterIncrement(c->stats[0]->cnt[cntTokenInfo]);
    c->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (c->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
    } else if (parameters != 0) {
      zkLogWriteContext (c->log, err, func, "Command does not require parameters");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                               "Command does not require parameters",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #2 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;

    } else if (c->ttBuf && c->ttBufSz) {
      zkipSessionResponse(err,session,0,_SERVICE_READY,
                          "Cached response",
                          "NODATA",ZKIP_ENCODING_NONE,c->ttBufSz,c->ttBuf);

      zkStatisticCounterIncrement(c->stats[0]->cnt[cntTokenInfoHit]);

      if (c->debug) {
        zkLogWrite(c->log, "Token Info Returned [HIT]");
      }
    } else {
                                           /* If we ever get here I pay the beer */
      zkLogWriteContext (c->log, err, func, "Internal Error(No PSR for TokenInfo)");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                               "Error servicing tokeninfo request",
                               "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #3 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 *
 */

int
niqsCmdClientInfo (
                   ZkError err ,
                   ZkipSession session ,
                   void * data )
{
  static const char * func = "niqsClientInfo";
  FrNiqsCtx * c = (FrNiqsCtx *) data; /* The application context */
  int return_code = _SESSION_ERROR;
  int parameters = 0; /* Number of command parameters */
  void * buf = NULL; /* The client info response buffer */
  size_t sz = 0; /* Size of the client info response buffer */
  void * sbuf = NULL; /* The client info response buffer (signed) */
  size_t ssz = 0; /* Size of the client info response buffer (signed)  */
  ZkConfig client_info = NULL;   /* Holds the client info sections */
  FrCryptPubKey mySigKey = NULL; /* My private signature key */
  char * section = NULL;      /* client info section / version code */
  char current_UTC_s[64];     /* String version of current_UTC */
  char * expire = NULL; /* The expire DateTime for the client */
  unsigned char * motd64 = NULL;  /* To hold the optional motd message */
  unsigned char * urg64 = NULL; /* To hold the optional urgency text*/
  int fail = 0;                 /* Set to 1 on failure */
  char query_name[128];         /* Name of the query for cache lookup */
  int in_cache = 0;


  (void) module_id_; /* Get rid of compile time warning */
  zkErrorClear(err);
  if (session && data)  {

    zkStatisticCounterIncrement(c->stats[0]->cnt[cntClientInfo]);
    c->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;
    motd64 = NULL;

    /*
     * First check that we have received a client version code
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (c->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
      fail = 1;
    } else if (parameters != 1) {
      zkLogWriteContext (c->log, err, func, "Expected a client version code");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Expected a client version code",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #2 to zkipSessionResponse() failed");
      }
      fail = 1;

      /*
       * We have one, so get the section variable to point to it
       */

    } else if (!zkipSessionParameter(err,session,0,
                                     (const unsigned char **) &section)) {
      zkLogWriteContext (c->log, err, func,
                         "Call to zkipSessionParameter() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                               "Unable to read client version code",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
         zkLogWriteContext (c->log, err, func,
                            "Call #3 to zkipSessionResponse() failed");
      }
      fail = 1;
      return_code = _SESSION_ERROR;

    } else if (snprintf(query_name,128,"CLIINF-%s",section) <= 0) {
      zkLogWriteContext (c->log, err, func,
                         "Failed to create cache lookup request using snprintf");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Failed to create cache lookup request",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      fail = 1;
      return_code = _SESSION_ERROR;

    } else if (!myPSRCacheLookup_ttl(err,c,query_name,strlen(query_name),&sbuf,&ssz,&in_cache)) {
      zkLogError(c->log,err);
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Lookup cache failure",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      fail = 1;
      return_code = _SESSION_ERROR;

    } else if (in_cache) {
      zkipSessionResponse(err,session,0,_SERVICE_READY,
                          section,
                          "NODATA",ZKIP_ENCODING_NONE,ssz,sbuf);

      zkStatisticCounterIncrement(c->stats[0]->cnt[cntClientInfoHit]);

      if (c->debug) {
        zkLogWrite_va(c->log,"Client Info %s Returned [HIT]",section);
      }
      /*
       * Create a configuration object to hold the client information file
       * and load it.  This way lets us update this information on the fly
       * without having to restart or signal the server whenever it changes
       */

    } else if (!(zkConfigNew(err,&client_info) &&
                 zkConfigLoad(err,client_info,c->clientinfo))) {
      zkLogWriteContext (c->log, err, func, "Failed to create/load config");
      fail = 1;
      if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                          "Client information is currently not available",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func,
                           "Call #4 to zkipSessionResponse() failed");
      }
    } else {

      /*
       * Load the MOTD file if available
       */

      if (c->motdfile) {
        char * motd = NULL;
        size_t motd_sz = 0;

        if (!zkFileLoad(err,c->motdfile,&motd,&motd_sz)) {
          zkLogWriteContext (c->log, err, func,
                             "Failed to read MOTD file \"%s\":",c->motdfile);
        }
	else
	  {
	  motd64 = (unsigned char *) malloc( motd_sz * 4 / 3 + 6 );
	  if (motd64 == NULL) {
            zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
                       "Unable to allocate memory to encode MOTD");
            zkLogWriteContext (c->log, err, func, "Call to malloc() failed");
            fail = 1;
          } else if (!zkConvertToBase64(err,motd,motd64,motd_sz)) {
            zkLogWriteContext (c->log, err, func,
                               "Call to zkConvertToBase64() failed");
            motd64 = NULL;
            fail = 1;
          }
	  if (motd) {
	    free(motd);
	    motd = NULL;
	  }
	  /* !!! HEAVY HACK ALERT !!! */
	  /* Warning: this code shouldn't be looked at by children below 13, */
	  /* pregnant women, newbies or greens. */
	  /* Look away if your heart can't take it! */

	  /* BUG ID 1553 */
	  /* zkConfig cannot handle lines over 512 characters long */
	  /* The MOTD is stored in the zkConfig buffer as: */
	  /* motd = `my text in Base64`\n ... */
	  /* The "motd = " and \n take up 8 bytes */
	  /* since the MOTD is converted in Base64, the maximum length of */
	  /* the MOTD is (512 - 8) / 4 * 3 = 378. */
	  /* Growing the max line length in zkConfig could have adverse */
	  /* effects.  It was decided to hack it by using the extra chars in */
	  /* the unused urgency line. */
	  /* A "special" separator of " motd = " will divide the urgency */
	  /* text from the extra motd field (in Base64!) */
	  /* Currently, urgency field is always forced to "1" */
	  /* In the case where1 motd is longer than 378 bytes, the motg_urg */
	  /* will look like this: "1 motd2 = 'my text over 378 bytes'\n */
	  /* The string "motd_urg = 1 motd2 = " plus the extra '\n' */
	  /* which makes it 22 bytes long */
	  /* (512 - 22) / 4 * 3 = 366. */
	  /* The maximum MOTD length will now be: 745 characters */
	  /* using the urgency will reduce the MOTD size by as much */
	  if( strlen(motd64) > 503 ) {
	    urg64 = (char *)malloc( 512 * sizeof(char) );
	    if( urg64 ) {
	      snprintf( urg64, 501, "1 motd2 = %s", &motd64[503] );
	      urg64[501] = '\0';
	      motd64[503] = '\0';
	    } else {
	      zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENOMEM,
			 "Unable to allocate memory to encode MOTD");
	    }

	  }

        }
      }
      /*
       * Set the expire-in value if we have an expiry datetime set for this
       * version of the client.  I remove an extra 5 minutes from the expiry
       * time because this mechanism is not very accurate.  I would rather it
       * expire the clients a few minutes early rather than a few late.
       */

      if (zkConfigGet(err,client_info,section,NIQS_CLIENT_INFO_EXPIRY,
                      (const char **) &expire)) {
        time_t expiry = 0;         /* When will the client expire? */
        time_t expire_in = 0;      /* Number of seconds left.... */
        ZkDateTime dt;             /* Expiry date time */
        char expire_in_s[64];     /* Number of seconds till the client expires */

        if (!zkDateTimeParse(err,&dt,expire)) {
          zkLogWriteContext (c->log, err, func, "Call to zkDateTimeParse() failed");
          fail = 1;
        } else if (!zkDateTimeMkTime(err,&dt,ZKTIME_FL_LOCAL,&expiry)) {
          zkLogWriteContext (c->log, err, func, "Call to zkDateTimeMkTime() failed");
          fail = 1;
        }
        expire_in = expiry - time(NULL);
        expire_in -= 5 * 60;
        snprintf(expire_in_s,64,"%ld",expire_in);
        if (!zkConfigSet(err,client_info,section,NULL,
                      NIQS_CLIENT_INFO_EXPIRE_IN,expire_in_s)) {
          zkLogWriteContext (c->log, err, func, "Call #1 to zkConfigSet() failed");
          fail = 1;
        }
      }

      if (fail) {
        zkLogWriteContext (c->log, err, func, "Internal error, failed to send clientinfo");
        if (!zkipSessionResponse(NULL,session,0,_COMMAND_FAILED,
                                 "Failed to initialize clientinfo data",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #5 to zkipSessionResponse() failed");
        }


      /*
       * We need to add the current UTC time to the client info structure so
       * that the Freedom client can compute its local UTC offset and synch
       * up with the time in the routing cloud
       */

      } else if (snprintf(current_UTC_s,64,"%ld",(long) time(NULL)) <= 0) {
        zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKERROR,
                      "Failed to create UTC time parameter for clientinfo");
        zkLogWriteContext (c->log, err, func, "Call to snprintf() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Failed to create UTC time parameter for clientinfo",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #6 to zkipSessionResponse() failed");
      }
      } else if (!zkConfigSet(err,client_info,section,NULL,
                              NIQS_CLIENT_INFO_CURRENT_UTC,current_UTC_s)) {
        zkLogWriteContext (c->log, err, func, "Call #2 to zkConfigSet() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Failed to add UTC time parameter to clientinfo",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #7 to zkipSessionResponse() failed");
        }

        /*
         * Add the client info MOTD and its urgency code if they were
         * specified
         */

      } else if (motd64 && !zkConfigSet(err,client_info,section,NULL,
                                        NIQS_CLIENT_INFO_MOTD,motd64)) {
        zkLogWriteContext (c->log, err, func, "Call #3 to zkConfigSet() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Failed to add the MOTD to clientinfo",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #8 to zkipSessionResponse() failed");
        }
      } else if (motd64 && urg64 && !zkConfigSet(err,client_info,section,NULL,
                                        NIQS_CLIENT_INFO_MOTD_URG,urg64)) {
        zkLogWriteContext (c->log, err, func, "Call #4 to zkConfigSet() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Failed to add the MOTD urgency code to clientinfo",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #9 to zkipSessionResponse() failed");
        }

        /*
         * Find the requested client version (section) and convert it to a
         * response buffer
        */

      } else if (!zkConfigSectionToBuffer(err,client_info,section,&buf,&sz)) {
        zkLogWriteContext (c->log, err, func,
                           "Could not convert section \"%s\" to buffer",section);
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                           "No client information available",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func,
                             "Call #10 to zkipSessionResponse() failed");
        }
       /*
        * Get the server signature key and sign the response buffer
        */

      } else if (!(frSecKeyChainGetVersion(err,c->kc,c->sigHandle,0,
                                            &mySigKey) &&
                  frSecSignData(err,mySigKey,buf,sz,&sbuf,&ssz))) {
         zkLogWriteContext (c->log, err, func,
                            "Failed to retreive signature key or sign response");
         if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                                  "Failed to sign data",
                                  "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
           zkLogWriteContext (c->log, err, func,
                              "Call #11 to zkipSessionResponse() failed");
         }
         return_code = _SESSION_ERROR;

         /*
          * Send the response buffer to the client
          */

       } else {
         if (!zkipSessionResponse(err,session,0,_SERVICE_READY, section,
                                  "NODATA",ZKIP_ENCODING_NONE,ssz,sbuf)) {
           zkLogWriteContext (c->log, err, func,
                            "Call #12 to zkipSessionResponse() failed");
         } else if (c->debug) {
           zkLogWrite_va(c->log,"Client Info %s Returned",section);
         }
         snprintf(query_name,128,"CLIINF-%s",section);
         if (!myPSRCacheStore_ttl(err,c,query_name,strlen(query_name),sbuf,ssz)) {
           zkLogError(c->log,err);
        }
        if (c->debug) {
           zkLogWrite_va(c->log,"CACHE: Storing the client info %s data",section);
        }
      }
    }

    /*
     * clean up
     */

    if (buf) {
      free(buf);
    }
    if (sbuf) {
      free(sbuf);
    }
    if (client_info) {
      zkConfigFree(NULL,&client_info);
    }
    if(urg64) {
      free(urg64);
    }
    if (motd64) {
      free(motd64);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Return a signed block of data representing all currently active entity ID's
 * This is periodically used by the client to get rid of purge entities from
 * its cache when those entities no longer exist.
 */

int
niqsCmdAvailable (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  static const char * func = "niqsCmdAvailable";
  FrNiqsCtx * c = (FrNiqsCtx *) data; /* The application context */
  int parameters = 0;              /* number of command parameters */
  FrDBOTypeEntityID eid_list;      /* List of entity ID's in DBO format */
  size_t nelem;                    /* Number of elements in eid_list */
  size_t n;                        /* Loop index for converting entity IDs */
  int ret = _SESSION_CONTINUE;     /* Function return code */
  FrCryptPubKey mySigKey = NULL;   /* My private signature key */
  uint32_t mask = 0;               /* Entity type mask */
  size_t bsz = 0;                  /* size of buf*/
  unsigned char * buf = NULL;      /* Holds buffer of entity IDs */
  void * sbuf = NULL;              /* Holds the signed version of the buffer */
  size_t ssz = 0;                  /* Size of sbuf */
  char     query_name[QUERY_NAME_SZ];
  int      qry_sz          = 0;
  int      in_cache        = 0;

  if (session && data) {
    const uint32_t  cmdid = zkipSessionCmdid(err,session);

    zkStatisticCounterIncrement(c->stats[0]->cnt[cntAvailable]);
    c->nbCmdsForThisConnexion++;

    /*
     * Get the ZKIP command line parameter count
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (c->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      ret = _SESSION_ERROR;
    } else if (parameters < 1) {
      zkLogWriteContext (c->log, err, func, "Expected entity_type1 [ entity_type2 ...]");
      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Expected entity_type1 [ entity_type2 ...]",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #2 to zkipSessionResponse() failed");
      }
    } else if (!niqsUtilMkEntityBitmap(err, c->log, session,0,parameters,&mask)) {
      zkLogWriteContext (c->log, err, func, "Call to niqsUtilMkEntityBitmap() failed");
    } else if ((qry_sz = snprintf(query_name,QUERY_NAME_SZ,"AVL/%x/%x",cmdid,mask)) <= 0) {
      zkLogWriteContext (c->log, err, func,
                         "Failed to create cache lookup request using snprintf");
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Failed to create cache lookup request",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    } else if (!myPSRCacheLookup_cdv(err,c,query_name,(size_t)qry_sz,(void **)&sbuf,&ssz,&in_cache)) {
      zkLogError(c->log,err);
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Cache lookup failure",
                          "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
    } else if (in_cache) {
      zkipSessionResponse(err, session, 0, _SERVICE_READY,
                          "Cached response", "NODATA", ZKIP_ENCODING_NONE,
                          ssz, sbuf);

      zkStatisticCounterIncrement(c->stats[0]->cnt[cntAvailableHit]);

      if (c->debug) {
        zkLogWrite_va(c->log,"Available Succeed %x/%x [HIT]",cmdid,mask);
      }
    } else if (!frNidbGetEntitiesInTypesetUpdatedSince(err,c->nidb,0,
                                                       mask,&eid_list,&nelem)) {
      zkLogWriteContext (c->log, err, func, "Call to frNidbGetEntitiesInTypesetUpdatedSince() failed");
      if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                          "Failed to get list of entity identifiers","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #3 to zkipSessionResponse() failed");
      }
      ret = _SESSION_ERROR;
    } else {

      /*
       * We've got all of the active entity ID's, convert them to a buffer
       * and send them back to the client.
       */

      bsz = nelem * 4;
      if ((buf = (unsigned char *) malloc(bsz + 1)) == NULL) {
        zkErrorSet_va(err,func,ZKERR,ZKMNIQS,ZKENOMEM,"malloc():  %s",
                      strerror(errno));
        zkLogWriteContext (c->log, err, func, "Call to malloc() failed");
        if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                            "Internal error","NODATA",
                                 ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (c->log, err, func, "Call #4 to zkipSessionResponse() failed");
        }
       ret = _SESSION_ERROR;
      } else {
        for (n = 0; n < nelem; n++) {
          buf[n * 4] = (eid_list[n].entid & 0xFF000000) / 0x1000000;
          buf[n * 4 + 1] = (eid_list[n].entid & 0xFF0000) / 0x10000;
          buf[n * 4 + 2] = (eid_list[n].entid & 0xFF00) / 0x100;
          buf[n * 4 + 3] = (eid_list[n].entid & 0xFF);
        }
        buf[nelem * 4] = 0;

        /* Sign the entity ID buffer with my private signature key */

        if (!(frSecKeyChainGetVersion(err,c->kc,c->sigHandle,0,&mySigKey) &&
              frSecSignData(err,mySigKey,buf,bsz,&sbuf,&ssz))) {
          zkLogWriteContext (c->log, err, func, "Failed to retreive signature key or sign response");
          if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
                              "Failed to sign data",
                                   "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
            zkLogWriteContext (c->log, err, func, "Call #5 to zkipSessionResponse() failed");
          }
          ret = _SESSION_ERROR;

        /*
         * Send the response buffer to the client
         */

        } else {
          if (!zkipSessionResponse(err,session,0,_SERVICE_READY,
                              "Currently available entity IDs",
                                   "NODATA",ZKIP_ENCODING_NONE,ssz,sbuf)) {
            zkLogWriteContext (c->log, err, func, "Call #6 to zkipSessionResponse() failed");
          } else if (c->debug) {
            zkLogWrite_va(c->log,"Available Succeed %x/%x",cmdid,mask);
          }

          /*
           * If the query name was succesfully generated, we have a positive int for
           * the size.  We'll therefore store it in the cache.
           */

          if (qry_sz > 0) {
            if (!myPSRCacheStore_cdv(err,c,query_name,qry_sz,sbuf,ssz)) {
              zkLogError(c->log,err);
            }

            if (c->debug) {
              zkLogWrite_va(c->log,"CACHE: Storing the query %s",query_name);
            }
          }
        }
      }
    }

    /*
     * cleanup
     */

    if (buf) {
      free(buf);
    }
    if (sbuf) {
      free(sbuf);
    }

  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
    ret = _SESSION_ERROR;
  }

  return ret;
}

/*
 *
 */

int
niqsCmdList(
            ZkError err,
            ZkipSession session ,
            void * data )
{
  static const char * func = "niqsCmdList";
  int return_code = _SESSION_ERROR;
  FrNiqsCtx * context = (FrNiqsCtx *) data; /* The application context */
  int parameters = 0;
  uint32_t mask = 0;
  time_t since = 0;
  unsigned char * sval = NULL;
  void * buf = NULL;
  size_t sz = 0;

  int   use_cache = 0;
  int   qry_sz    = 0;
  char  query_name[QUERY_NAME_SZ];

  zkErrorClear(err);
  if (session && data) {
    const uint32_t  cmdid = zkipSessionCmdid(err,session);

    zkStatisticCounterIncrement(context->stats[0]->cnt[cntList]);
    context->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;

    /*
     * Initial parameter check
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (context->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
    } else if (parameters < 2) {
      zkLogWriteContext (context->log, err, func, "Expected since entity_type1 [ entity_type2 ...]");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                               "Expected since entity_type1 [ entity_type2 ...]",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #2 to zkipSessionResponse() failed");
      }
    } else {
      int found = 0;  /* Did we get our list? */

      /*
       * Get the last update time
       */


      if (!zkipSessionParameter(err,session,0,(const unsigned char **) &sval)) {
        zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionParameter() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to get last update time",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (context->log, err, func, "Call #3 to zkipSessionResponse() failed");
        }
        goto finish;
      } else {
        long  since_off = (time_t) atoi(sval); /* Bug? This could fail if not a number */

        if ( 2 <= CMDID_MAJVER(cmdid) && since_off < 0) {
          since = time(NULL) + since_off;
        } else {
          since = (time_t) since_off;

/* There may be a problem if someone uses proto #1 with negative since value */
/* We definitely need more validation here */

          use_cache = (since == 0); /* Only cache responses to "LIST 0 entType [entType]..." requests */
        }
      }

      /*
       * Create the bitmap which is used to identify which entities to put
       * in the list.
       */
      if (!niqsUtilMkEntityBitmap(err,context->log,session,1,parameters,&mask)) {
        zkLogWriteContext (context->log, err, func,"Call to niqsUtilMkEntityBitmap() failed");
        goto finish;
      }

      if (use_cache) {
        int  in_cache = 0;

        zkStatisticCounterIncrement(context->stats[0]->cnt[cntListSince0]);

        /*
         * Do we have a response in our cache?
         */

        if ((qry_sz = snprintf(query_name,QUERY_NAME_SZ,"LST/%x/%x",cmdid,mask)) <= 0) {
          zkLogWriteContext (context->log, err, func,
                             "Failed to create cache lookup request using snprintf");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Failed to create cache lookup request",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;

        } else if (!myPSRCacheLookup_cdv(err,context,query_name,(size_t)qry_sz,(void **)&buf,&sz,&in_cache)) {
          zkLogError(context->log,err);
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Cache lookup failure",
                              "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
          goto finish;

        } else if (in_cache) {
          zkipSessionResponse(err, session, 0, _SERVICE_READY,
                              "Cached response", "ENTITYLIST", ZKIP_ENCODING_BINARY,
                              sz, buf);

          zkStatisticCounterIncrement(context->stats[0]->cnt[cntListSince0Hit]);

          if (context->debug) {
            zkLogWrite_va(context->log,"List Succeed %x/%x [HIT]",cmdid,mask);
          }
          goto finish;
        }
      }

      if (frNiqsProcessList(err,context,since,mask,&buf,&sz,&found) &&
          found) {
        if (!zkipSessionResponse(err,session,0,_SERVICE_READY,
                            "Entity list",
                                 "ENTITYLIST",ZKIP_ENCODING_BINARY,sz,buf)) {
          zkLogWriteContext (context->log, err, func, "Call #6 to zkipSessionResponse() failed");
        } else if (context->debug) {
          zkLogWrite_va(context->log,"List Succeed %x/%x",cmdid,mask);
        }

        /*
         * If the query name was succesfully generated, we have a positive int for
         * the size.  We'll therefore store it in the cache.
         */

        if (use_cache && qry_sz > 0) {
          if (!myPSRCacheStore_cdv(err,context,query_name,qry_sz,buf,sz)) {
            zkLogError(context->log,err);
          }

          if (context->debug) {
            zkLogWrite_va(context->log,"CACHE: Storing the query %s",query_name);
          }
        }
      } else {
        zkLogWriteContext (context->log, err, func, "Call to frNiqsProcessList() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Unable to generate requested list",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (context->log, err, func, "Call #7 to zkipSessionResponse() failed");
        }
      }
    finish:
      if (buf) {
        free(buf);
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

int
niqsCmdStatus(
              ZkError err,
              ZkipSession session ,
              void * data )
{
  static const char *func = "niqsCmdStatus";
  FrNiqsCtx * context = (FrNiqsCtx *) data; /* The application context */
  int parameters = 0;
  int return_code = _SESSION_ERROR;
  size_t sz = 0;
  time_t since = 0;
  uint32_t mask = 0;
  unsigned char * sval = NULL;
  void * buf = NULL;

  int   use_cache = 0;
  int   qry_sz    = 0;
  char  query_name[QUERY_NAME_SZ];

  zkErrorClear(err);
  if (session && data) {
    const uint32_t  cmdid = zkipSessionCmdid(err,session);

    zkStatisticCounterIncrement(context->stats[0]->cnt[cntStatus]);
    context->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;

    /*
     * Initial parameter check
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (context->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
    } else if (parameters < 1) {
      zkLogWriteContext (context->log, err, func, "Expected since-time [[entity_type1 entity_name1] ... ]");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                               "Expected since-time [[entity_type1 entity_name1] ... ]",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #2 to zkipSessionResponse() failed");
      }
    } else {
      int found = 0;  /* Did we get our list? */

      /*
       * Get the last status update (This is currently in the protocol but not
       * used by the server)
       */

      if (!zkipSessionParameter(err,session,0,(const unsigned char **) &sval)) {
        zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionParameter() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to get last update time",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (context->log, err, func, "Call #3 to zkipSessionResponse() failed");
        }
        goto finish;
      } else {
        long  since_off = (time_t) atoi(sval);

        if ( 2 <= CMDID_MAJVER(cmdid) && since_off < 0) {
          since = time(NULL) + since_off;
        } else {
          since = (time_t) since_off;

/* There may be a problem if someone uses proto #1 with negative since value */
/* We definitely need more validation here */

          use_cache = (since == 0); /* Only cache responses to "STATUS 0 entType [entType]..." requests */
        }
      }

      /*
       * Create the bitmap which is used to identify which entities to put
       * in the list.
       */

      if (!niqsUtilMkEntityBitmap(err,context->log,session,1,parameters,&mask)) {
        zkLogWriteContext (context->log, err, func,"Call to niqsUtilMkEntityBitmap() failed");
        goto finish;
      }

      if (use_cache) {
        int  in_cache = 0;

        zkStatisticCounterIncrement(context->stats[0]->cnt[cntStatusSince0]);

        /*
         * Do we have a response in our cache?
         */

        if ((qry_sz = snprintf(query_name,QUERY_NAME_SZ,"ST/%x/%x",cmdid,mask)) <= 0) {
          zkLogWriteContext (context->log, err, func,
                             "Failed to create cache lookup request using snprintf");
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Failed to create cache lookup request",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL);
          goto finish;

        } else if (!myPSRCacheLookup_cdv(err,context,query_name,(size_t)qry_sz,(void **)&buf,&sz,&in_cache)) {
          zkLogError(context->log,err);
          zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                              "Cache lookup failure",
                              "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
          goto finish;

        } else if (in_cache) {
          zkipSessionResponse(err, session, 0, _SERVICE_READY,
                              "Cached response", "STATUS_RESPONSE", ZKIP_ENCODING_BINARY,
                              sz, buf);

          zkStatisticCounterIncrement(context->stats[0]->cnt[cntStatusSince0Hit]);

          if (context->debug) {
            zkLogWrite_va(context->log,"Status Succeed %x/%x [HIT]",cmdid,mask);
          }
          goto finish;
        }
      }

      if (frNiqsProcessStatus(err,context,cmdid,since,
                              mask,&buf,&sz,&found) && found) {
        if (!zkipSessionResponse(err,session,0,_SERVICE_READY,
                            "Status reponse",
                                 "STATUS_RESPONSE",ZKIP_ENCODING_BINARY,sz,buf)) {
          zkLogWriteContext (context->log, err, func, "Call #6 to zkipSessionResponse() failed");
        } else if (context->debug) {
          zkLogWrite_va(context->log,"Status Succeed %x/%x",cmdid,mask);
        }

        /*
         * If the query name was succesfully generated, we have a positive int for
         * the size.  We'll therefore store it in the cache.
         */

        if (use_cache && qry_sz > 0) {
          if (!myPSRCacheStore_cdv(err,context,query_name,qry_sz,buf,sz)) {
            zkLogError(context->log,err);
          }

          if (context->debug) {
            zkLogWrite_va(context->log,"CACHE: Storing the query %s",query_name);
          }
        }
      } else {
        zkLogWriteContext (context->log, err, func, "Call to frNiqsProcessStatus() failed");
        if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                            "Unable to generate requested list",
                                 "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
          zkLogWriteContext (context->log, err, func, "Call #7 to zkipSessionResponse() failed");
        }
      }
    finish:
      if (buf) {
        free(buf);
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Process a query command.
 *
 * Possible attributes: list, desc, or topology
 * Note: topology attribute will be phased out
 */

int
niqsCmdQuery(
             ZkError err,
             ZkipSession session ,
             void * data )
{
  static const char * func = "niqsCmdQuery";
  int return_code = _SESSION_ERROR;
  int parameters = 0;            /* Number of parameters (should be 3) */
  const char * ent_type = NULL;   /* Requested entity type */
  const char * ent_name = NULL;   /* Requested entity name */
  const char * attribute = NULL;  /* Requested attribute */
  char * signed_response = NULL;  /* The response to the query */
  char * _buf = NULL;
  FrEntIdExp entity;              /* Combination of type and name 4 lookups */
  FrNiqsCtx * context = (FrNiqsCtx *) data; /* The application context */
  FrNiqAttr niq_attr;
  char query_name[QUERY_NAME_SZ]; /* Name of the query for cache lookup */
  int qry_sz = 0;                 /* Length of the query name */
  int in_cache = 0;
  int  iCnt;  /* cntQry... index in the stats->cnt array */

  size_t srsz = 0;
  int found = 0;

  zkErrorClear(err);
  if (! session || !data) {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
    goto finish;
  }

  context->nbCmdsForThisConnexion++;
  return_code = _SESSION_CONTINUE;

  /*
   * Count the command parameters
   */

  if (!zkipSessionParameters(err,session,&parameters)) {
    zkLogWriteContext (context->log, err, func, "Call to zkipSessionParameters() failed");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Could not get command parameters","NODATA",
                             ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (parameters != 3) {
    zkLogWriteContext (context->log, err, func, "Expected attribute entity_type entity_name");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Expected attribute entity_type entity_name","NODATA",
                             ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #2 to zkipSessionResponse() failed");
    }
    goto finish;
  }
  /*
   * Identify the attribute requested
   */
  if (!zkipSessionParameter(err,session,0,(const unsigned char **)&attribute)) {
    zkLogWriteContext (context->log, err, func, "Call #1 to zkipSessionParameter() failed");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Unable to identify requested attribute",
                             "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #3 to zkipSessionResponse() failed");
    }
   goto finish;
  }

  /*
   * now reconstruct the entity structure
   */

  if (!(zkipSessionParameter(err,session,1,
                             (const unsigned char **)&ent_type) &&
        zkipSessionParameter(err,session,2,
                             (const unsigned char **)&ent_name) &&
        frEntSetFromStr2(err,&entity,ent_type,ent_name))) {
    zkLogWriteContext (context->log, err, func, "Calls #2 & #3 to zkipSessionParameter() failed");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Unable to construct entity from parameters",
                             "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #4 to zkipSessionResponse() failed");
    }
    goto finish;
  }

  if (!frNiqsTokenToAttribute (err, attribute, &niq_attr)) {
    zkLogWriteContext (context->log, err, func, "Call to frNiqsTokenToAttribute() failed");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Token is not an NIQ attribute",
                             "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #5 to zkipSessionResponse() failed");
    }
    goto finish;
  }

  switch (niq_attr) {  /* Here we have a valid attribute */
    case FRNIQ_ATTR_TOPOLOGY    : iCnt = cntQryTopology    ; break;
    case FRNIQ_ATTR_DESCRIPTION : iCnt = cntQryDescription ; break ;
    case FRNIQ_ATTR_STATUS      : iCnt = cntQryStatus      ; break ;
    default:
      zkLogWriteContext (context->log, err, func,
                         "Unable to handle this attribute query");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to handle this attribute query",
                              "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #9 to zkipSessionResponse() failed");
      }
      goto finish;
  }

  zkStatisticCounterIncrement(context->stats[0]->cnt[iCnt]);

  /*
   * Do we have a response in our cache?
   */

  if ((qry_sz = snprintf(query_name,QUERY_NAME_SZ,"QRY/%s/%s/%s",
                         attribute,ent_type,ent_name)) <= 0) {
    zkLogWriteContext (context->log, err, func,
                       "Failed to create cache lookup request using snprintf");
    zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Failed to create cache lookup request",
                        "NODATA",ZKIP_ENCODING_NONE,0,NULL);
    goto finish;

  } else if (!myPSRCacheLookup_cdv(err,context,query_name,(size_t)qry_sz,(void **)&signed_response,&srsz,&in_cache)) {
    zkLogError(context->log,err);
    zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Cache lookup failure",
                        "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
    goto finish;

  } else if (in_cache) {
    zkipSessionResponse(err, session, 0, _SERVICE_READY,
                        "Cached response", "NODATA", ZKIP_ENCODING_BINARY,
                        srsz, signed_response);

           /* the cache hit counter index follows its non-cache-hit counterpart */
    zkStatisticCounterIncrement(context->stats[0]->cnt[iCnt + 1]);

    if (context->debug) {
      zkLogWrite_va(context->log,"%s of %s/%s Returned [HIT]",
                    attribute,ent_type,ent_name);
    }
    goto finish;
  }

  switch (niq_attr) {
  case FRNIQ_ATTR_TOPOLOGY:
    /* Warning: This call will be phased out */
    if ( ! frNiqsProcessQuery(err, context, zkipSessionCmdid(err,session),
                              attribute, &entity,
                              (void **) &signed_response, &srsz, &found)) {
      zkLogWriteContext (context->log, err, func, "Search for %s of %s/%s failed",
                         attribute,ent_type,ent_name);
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to get a response to the query",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #6 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
      goto finish;
    }
    break;
  case FRNIQ_ATTR_DESCRIPTION:
    if ( ! frNiqsProcessDesc(err, context, zkipSessionCmdid(err,session), &entity,
                             (void **) &signed_response, &srsz, &found)) {
      zkLogWriteContext (context->log, err, func, "Search for %s of %s/%s failed",
                         attribute,ent_type,ent_name);
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to get a response to the query",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func, "Call #7 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
      goto finish;
    }
    break;
  case FRNIQ_ATTR_STATUS:
    if ( ! frNiqsProcessQuery(err, context, zkipSessionCmdid(err,session),
                              attribute, &entity,
                              (void **) &signed_response, &srsz, &found)) {
      zkLogWriteContext (context->log, err, func, "Search for %s of %s/%s failed",
                         attribute,ent_type,ent_name);
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to get a response to the query",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (context->log, err, func,
                           "Call #8 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
      goto finish;
    }
    break;
  default:
    zkLogWriteContext (context->log, err, func,
                       "Unable to handle this attribute query");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Unable to handle this attribute query",
                            "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #9 to zkipSessionResponse() failed");
    }
    goto finish;
  }

  if ( ! found ) {
    zkLogWriteContext (context->log, err, func, "Unable to find data");
    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Unable to find data",
                             "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func,
                         "Call #10 to zkipSessionResponse() failed");
    }
    return_code = _COMMAND_DATA_NOTFOUND;
    goto finish;
  }

  if (zkCStrSprintf (&_buf, "RESPONSE to %s %s/%d", attribute,
                     entity.name, entity.type) == -1) {
    zkLogWriteContext (context->log, err, func, "Call to zkCStrSprintf() failed");

    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                             "Internal error",
                             "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func, "Call #11 to zkipSessionResponse() failed");
    }
    goto finish;
  }

  /*
   * If the query name was succesfully generated, we have a positive int for
   * the size.  We'll therefore store it in the cache.  If we failed to
   * create the query name, we'll log an error message.
   */

  if (qry_sz > 0) {
    if (!myPSRCacheStore_cdv(err,context,query_name,qry_sz,signed_response,srsz)) {
      zkLogError(context->log,err);
    }

    if (context->debug) {
      zkLogWrite_va(context->log,"CACHE: Storing the query %s",query_name);
    }
  } else {
    zkLogWrite(context->log,"CACHE: 7.1.0 query: Query name too big!!!");
  }


  if (!zkipSessionResponse(err, session, 0, _SERVICE_READY,
                      _buf, "NODATA", ZKIP_ENCODING_BINARY,
                           srsz, signed_response)) {
    zkLogWriteContext (context->log, err, func, "Call #12 to zkipSessionResponse() failed");
  } else if (context->debug) {
    zkLogWrite_va(context->log,"%s of %s/%s Returned",attribute,ent_type,ent_name);
  }

  free(_buf);

 finish:
  if (signed_response) free (signed_response);
  return return_code;
}


int
niqsCmdAccessCtrlList (
            ZkError err ,
            ZkipSession session ,
            void * data )
{
  static const char * func = "niqsCmdAccessCtrlList";
  FrNiqsCtx * c = (FrNiqsCtx *) data; /* The application context */
  int return_code = _SESSION_ERROR;
  int parameters = 0; /* Number of command parameters */
  void * buf = NULL; /* The ACL response buffer */
  size_t sz = 0; /* Size of the ACL response buffer */
  void * sbuf = NULL; /* The ACL response buffer (signed) */
  size_t ssz = 0; /* Size of the ACL response buffer (signed)  */
  ZkConfig accessctrllist = NULL;   /* Holds the ACL sections */
  FrCryptPubKey mySigKey = NULL; /* My private signature key */
  char * section = NULL;      /* ACL section */
  int fail = 0;                 /* Set to 1 on failure */


  (void) module_id_; /* Get rid of compile time warning */
  zkErrorClear(err);
  if (session && data)  {

    zkStatisticCounterIncrement(c->stats[0]->cnt[cntAccessCtrlList]);
    c->nbCmdsForThisConnexion++;

    return_code = _SESSION_CONTINUE;

    /*
     * First check that we have received a client version code
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogWriteContext (c->log, err, func, "Call to zkipSessionParameters() failed");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                               ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #1 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;
      fail = 1;
    } else if (parameters != 0) {
      zkLogWriteContext (c->log, err, func, "Command does not require parameters");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Command does not require parameters",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #2 to zkipSessionResponse() failed");
      }
      fail = 1;

    }

    /*
     * Create a configuration object to hold the ACL information file
     * and load it.  This way lets us update this information on the fly
     * without having to restart or signal the server whenever it changes
     */

    else if (!(zkConfigNew(err,&accessctrllist) &&
               zkConfigLoad(err,accessctrllist,c->accessctrllist))) {
      zkLogWriteContext (c->log, err, func, "Config creation/initialization failed");
      fail = 1;
      if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                          "Access control list information is currently not available",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #3 to zkipSessionResponse() failed");
      }
    }

#warning No checking done on access control file!!!

    /*
     * Convert the config object to a response buffer
     */

    else if (!zkConfigToBuffer(err,accessctrllist,&buf,&sz)) {
      zkLogWriteContext (c->log, err, func, "Could not convert access control list to buffer");
      if (!zkipSessionResponse(err,session,0,_COMMAND_FAILED,
                          "No client information available",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #4 to zkipSessionResponse() failed");
      }
    }

    /*
     * Get the server signature key and sign the response buffer
     */

    else if (!(frSecKeyChainGetVersion(err,c->kc,c->sigHandle,0,&mySigKey) &&
               frSecSignData(err,mySigKey,buf,sz,&sbuf,&ssz))) {
      zkLogWriteContext (c->log, err, func, "Failed to retreive signature key or sign response");
      if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Failed to sign data",
                               "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
        zkLogWriteContext (c->log, err, func, "Call #5 to zkipSessionResponse() failed");
      }
      return_code = _SESSION_ERROR;

    }

    /*
     * Send the response buffer to the client
     */

    else {
      if (!zkipSessionResponse(err,session,0,_SERVICE_READY,
                          section,
                               "NODATA",ZKIP_ENCODING_NONE,ssz,sbuf)) {
        zkLogWriteContext (c->log, err, func, "Call #6 to zkipSessionResponse() failed");
      } else if (c->debug) {
        zkLogWrite(c->log,"Access Ctrl List Succeed");
      }
    }

    /*
     * clean up
     */

    if (buf) {
      free(buf);
    }
    if (sbuf) {
      free(sbuf);
    }
    if (accessctrllist) {
      zkConfigFree(NULL,&accessctrllist);
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Process a DescList command.
 *
 */

int
niqsCmdMxQuery(
             ZkError err,
             ZkipSession session,
             void * data )
{
  static const char * func = "niqsCmdMxQuery";
  FrNiqsCtx *context = (FrNiqsCtx *)data;
  int return_code = _SESSION_ERROR;
  int parameters = 0; /* Number of command parameters */
  uint32_t entity_mask;
  const char * attribute = NULL;
  const char * since_string = NULL;
  const char * encoding_string = NULL;
  time_t since = 0;
  long since_off = 0;
  uint32_t cmd_id;
  void *signed_response = NULL;
  size_t signed_resp_size = 0;
  uint16_t encoding = ZKIP_ENCODING_UNKNOWN;
  int valid_data = 0;

  int use_cache = 0;
  int in_cache = 0;
  int qry_sz    = 0;
  char query_name[QUERY_NAME_SZ];

  zkErrorClear(err);
  if (! session || !data) {
    zkErrorSet(err,func,ZKERR,ZKMNIQS,ZKENULLPTR,NULL);
    goto finish;
  }

  zkStatisticCounterIncrement(context->stats[0]->cnt[cntMxQryDescription]);
  context->nbCmdsForThisConnexion++;

  return_code = _SESSION_CONTINUE;

  cmd_id = zkipSessionCmdid(err, session);

  /*
   * Make sure we did supply the parameters to the command
   * parameters: since_time entity_type1 [ entity_type2 ... ]
   */
  if (!zkipSessionParameters(err,session,&parameters)) {
    zkLogWriteContext (context->log, err, func,
		       "Call to zkipSessionParameters() failed");
    if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
			     "Could not get command parameters",
			     "NODATA",ZKIP_ENCODING_BINARY,0,NULL)) {
      zkLogWriteContext (context->log, err, func,
			 "Call #1 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (parameters < 4) {
    zkLogWriteContext (context->log, err, func,
	       "Expected attribute since encoding entity_type [entity_type]*");
    if (!zkipSessionResponse(NULL,session,0,_COMMAND_ERROR,
	       "Expected atribute since encoding entity_type [entity_type*]",
			     "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func,
			 "Call #2 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (!zkipSessionParameter(err, session, 0,
			    (const unsigned char **) &attribute)) {
    zkLogWriteContext( context->log, err, func,
		       "Call to get parameter #0 failed");

    if(!zkipSessionResponse(err, session, 0,_COMMAND_ERROR,
			    "Unable to get querylist atribute", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #3 to zkipSessionResponses() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (!zkipSessionParameter(err, session, 1,
			    (const unsigned char **) &since_string)) {
    zkLogWriteContext( context->log, err, func,
		       "Call to get parameter #1 failed");
    if(!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			    "Unable to get last update time", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #4 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  since_off = (time_t) atoi(since_string);
  if ( since_off < 0) {
    since = time(NULL) + since_off;
  } else {
    since = (time_t) since_off;
  }

  if (!zkipSessionParameter(err, session, 2,
			    (const unsigned char **) &encoding_string)) {
    zkLogWriteContext( context->log, err, func,
		       "Call to get parameter #2 failed");
    if(!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			    "Unable to get encoding", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #5 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (!zkipParseEncoding(err, encoding_string, &encoding ) ){
    zkLogWriteContext( context->log, err, func, "Invalid encoding");
    if(!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			    "Invalid encoding", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #6 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  if (!niqsUtilMkEntityBitmap(err,context->log,session,3,parameters,
			      &entity_mask )) {
    zkLogWriteContext( context->log, err, func,
		       "Call to niqsUtilMkEntityBitmap failed");
    if(!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			    "Invalid entity_type", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #7 to zkipSessionResponse() failed");
    }
    return_code = _SESSION_ERROR;
    goto finish;
  }

  use_cache = (since == 0); /* Only cache responses for full update */
  in_cache = 0;
  if( use_cache ) {
    zkStatisticCounterIncrement(context->stats[0]->cnt[cntMxQryDescriptionSince0]);
    /*
     * Do we have a response in our cache?
     */

    if ((qry_sz = snprintf(query_name,QUERY_NAME_SZ,"MQR/%x/%s/%s/%x",cmd_id,
			   attribute,encoding_string,entity_mask)) <= 0) {
      zkLogWriteContext (context->log, err, func,
			 "Failed to create cache lookup request using snprintf");
    } else {
      if (!myPSRCacheLookup_cdv(err,context,query_name,(size_t)qry_sz,
                                (void **)&signed_response,&signed_resp_size,&in_cache)) {
	zkLogError(context->log,err);
	in_cache = 0;
      } else {
	if (in_cache) {

          zkStatisticCounterIncrement(context->stats[0]->cnt[cntMxQryDescriptionSince0Hit]);

	  if (context->debug) {
	    zkLogWrite_va(context->log,"MxQuery Succeed %x/%s/%s/%x [HIT]",
			  cmd_id,attribute,encoding_string,entity_mask);
	  }

	  if( !zkipSessionResponse(err, session, 0, _SERVICE_READY,
				   "Cached response", "NODATA",
				   ZKIP_ENCODING_BINARY,
				   signed_resp_size, signed_response)) {
	    zkLogWriteContext( context->log, err, func,
			       "Call #8 to zkipSessionResponse() failed()");
	    return_code = _SESSION_ERROR;
	    goto finish;
	  }

	  return_code = _SESSION_CONTINUE;
	  goto finish;
	} /* if in_cache */
      } /* if frNiqdQCLookup */
    } /* if snprintf(query_name) */
  } /* if use_cache */


  /*
   * Create the Process the Query, return the response an put it in the cache
   */
  if ( !frNiqsProcessMxQuery(err, context, cmd_id, attribute,
			     since, encoding, entity_mask,
			     &signed_response, &signed_resp_size,
			     &valid_data)) {
    if(!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			    "Error processing mxquery", "NODATA",
			    ZKIP_ENCODING_NONE, 0, NULL)) {
      zkLogWriteContext( context->log, err, func,
			 "Call #9 to zkipSessionResponse() failed");

      return_code = _SESSION_ERROR;
      goto finish;
    }

    return_code = _COMMAND_ERROR;
    goto finish;
  }

  if( !valid_data ) {

    if( context->debug ) {
      zkLogWriteContext (context->log, err, func, "Unable to find data");
    }

    if (!zkipSessionResponse(err,session,0,_COMMAND_ERROR,
			     "Unable to find data",
			     "NODATA",ZKIP_ENCODING_NONE,0,NULL)) {
      zkLogWriteContext (context->log, err, func,
			 "Call #10 to zkipSessionResponse() failed");
    }

    return_code = _COMMAND_DATA_NOTFOUND;
    goto finish;
  }

  if( !zkipSessionResponse(err, session, 0, _SERVICE_READY,
			   "MxQuery Response", "NODATA",
			   ZKIP_ENCODING_BINARY,
			   signed_resp_size, signed_response ) ) {
    zkLogWriteContext( context->log, err, func,
		       "Call #11 to zkipSessionResponse() failed" );
    return_code = _SESSION_ERROR;

    /* Do not go to "finish" just yet,
     * we have a valid Response in our hands.
     * put it in the cache!
     */
  }

  /* Put the Response in cache */
  if( use_cache ) {
    if (qry_sz < 0) {
      zkLogWrite(context->log,"CACHE: 7.1.0 query: Query name too big!!!");
      goto finish;
    }

    if (!myPSRCacheStore_cdv(err,context,query_name,qry_sz,signed_response,signed_resp_size)) {
      zkLogError(context->log,err);
    }

    if (context->debug) {
      zkLogWrite_va(context->log,"CACHE: Storing the query %s",query_name);
    }
  }

 finish:
  if (signed_response) free (signed_response);
  return return_code;
}
