/****************************************************************************
	FileName	    :	NIQSPERF.h

	Initial coding	    :	Stephane Rheaume (SR)
	(Jun 26th, 2000)

	Copyright (C) Zero-Knowledge Systems Inc. 2000, All rights reserved.
 ****************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include "zks.h"
#include "zkConfig.h"
#include "zkperf.h"


/****************************************************************************
	General constants and data types
 ****************************************************************************/

#define P_CONFIG                             "config"
#define P_HOSTNAME                           "hostname"
#define	P_PORT                               "port"
#define	P_TIMEOUT                            "timeout"
#define	P_CONCURRENT                         "concurrent"
#define	P_ITERATIONS                         "iterations"
#define	P_MXQUERYENABLED                     "mxqueryenabled"
#define	P_MINLATENCY_CONNECT                 "minconnlat"
#define	P_MAXLATENCY_CONNECT                 "maxconnlat"
#define	P_MINLATENCY_KQDLIST                 "minkqdlistlat"
#define	P_MAXLATENCY_KQDLIST                 "maxkqdlistlat"
#define	P_MINLATENCY_KQDQUERY                "minkqdquerylat"
#define	P_MAXLATENCY_KQDQUERY                "maxkqdquerylat"
#define	P_MINLATENCY_CDV                     "mincdvlat"
#define	P_MAXLATENCY_CDV                     "maxcdvlat"
#define	P_MINLATENCY_CI                      "mincilat"
#define	P_MAXLATENCY_CI                      "maxcilat"
#define	P_MINLATENCY_TI                      "mintilat"
#define	P_MAXLATENCY_TI                      "maxtilat"
#define	P_MINLATENCY_MXQ                     "minmxqlat"
#define	P_MAXLATENCY_MXQ                     "maxmxqlat"
#define	P_MINLATENCY_FULLUPDLIST             "minfullupdlistlat"
#define P_MAXLATENCY_FULLUPDLIST             "maxfullupdlistlat"
#define	P_MINLATENCY_FULLUPDQUERY            "minfullupdquerylat"
#define	P_MAXLATENCY_FULLUPDQUERY            "maxfullupdquerylat"

/*
 * Default values
 */
#define CONTEXT_DFLT_APPNAME                 "NIQSPERF"
#define CONTEXT_DFLT_CONFIGFILE              "niqsperf.conf"
#define CONTEXT_DFLT_HOSTNAME                "niqs.t1.dev.zks.net"
#define CONTEXT_DFLT_PORT                    51107
#define CONTEXT_DFLT_TIMEOUT                 900
#define CONTEXT_DFLT_CONCURRENT              100
#define CONTEXT_DFLT_ITERATIONS              500
#define CONTEXT_DFLT_MXQUERYENABLED          1
#define CONTEXT_DFLT_MINLATENCY_CONNECT      0
#define CONTEXT_DFLT_MAXLATENCY_CONNECT      0
#define CONTEXT_DFLT_MINLATENCY_KQDLIST      0
#define CONTEXT_DFLT_MAXLATENCY_KQDLIST      0
#define CONTEXT_DFLT_MINLATENCY_KQDQUERY     0
#define CONTEXT_DFLT_MAXLATENCY_KQDQUERY     0
#define CONTEXT_DFLT_MINLATENCY_CDV          0
#define CONTEXT_DFLT_MAXLATENCY_CDV          0
#define CONTEXT_DFLT_MINLATENCY_CI           0
#define CONTEXT_DFLT_MAXLATENCY_CI           0
#define CONTEXT_DFLT_MINLATENCY_TI           0
#define CONTEXT_DFLT_MAXLATENCY_TI           0
#define CONTEXT_DFLT_MINLATENCY_MXQ          0
#define CONTEXT_DFLT_MAXLATENCY_MXQ          0
#define CONTEXT_DFLT_MINLATENCY_FULLUPDLIST  0
#define CONTEXT_DFLT_MAXLATENCY_FULLUPDLIST  0
#define CONTEXT_DFLT_MINLATENCY_FULLUPDQUERY 0
#define CONTEXT_DFLT_MAXLATENCY_FULLUPDQUERY 0

typedef struct ST_APPCONTEXT                    /* ==> Application context <== */
{
  ZkConfig    config;                           /* Configuration object */
  const char  *pszAppName;                      /* Application name */
  const char  *pszConfigFile;                   /* Configuration file name */
  const char  *pszHostName;                     /* NIQS server to poke */ 
  uint16_t    uPort;                            /* NIQS port number */
  uint32_t    uTimeout;                         /* ZKIP connection timeout */
  uint32_t    uConcurrent;                      /* Number of concurrent connections */
  uint32_t    uIterations;                      /* Number of iterations */
  int         bMxQueryEnabled;                  /* FLAG: Is MxQuery command enabled ? */
  uint32_t    uMinLatency_CONNECT;              /* Connection latency, in milliseconds */
  uint32_t    uMaxLatency_CONNECT;              /* KQD latencies (LIST & QUERY), in milliseconds */
  uint32_t    uMinLatency_KQDLIST;
  uint32_t    uMaxLatency_KQDLIST;
  uint32_t    uMinLatency_KQDQUERY;
  uint32_t    uMaxLatency_KQDQUERY;
  uint32_t    uMinLatency_CDV;                  /* Cache Data Version latency, in milliseconds */
  uint32_t    uMaxLatency_CDV;
  uint32_t    uMinLatency_CI;                   /* Client Info latency, in milliseconds */
  uint32_t    uMaxLatency_CI;
  uint32_t    uMinLatency_TI;                   /* Token Info latency, in milliseconds */
  uint32_t    uMaxLatency_TI;
  uint32_t    uMinLatency_MXQ;                  /* MxQuery latency, in milliseconds */
  uint32_t    uMaxLatency_MXQ;
  uint32_t    uMinLatency_FULLUPDLIST;          /* Old Full Update latencies (LIST & QUERY), in milliseconds */
  uint32_t    uMaxLatency_FULLUPDLIST;
  uint32_t    uMinLatency_FULLUPDQUERY;
  uint32_t    uMaxLatency_FULLUPDQUERY;
} APPCONTEXT, *PAPPCONTEXT, **PPAPPCONTEXT;
typedef const APPCONTEXT *PCAPPCONTEXT;

typedef struct ST_ZKPERFOBJECT                  /* ==> ZKPERF object <== */
{
  ZkPerfClient perfClient;
  PAPPCONTEXT  pAppCtx;
} ZKPERFOBJECT, *PZKPERFOBJECT, **PPZKPERFOBJECT;
typedef const ZKPERFOBJECT *PCZKPERFOBJECT;


/****************************************************************************
	Macro definitions
 ****************************************************************************/

#define UNREFERENCED_PARAMETER(param) ((void) param)

#define DELAY(uMin, uMax, nRand) (usleep(uMin + (int) (uMax * nRand / (RAND_MAX + 1.0))))

#define FREEIT(ptr) \
{ \
 if (ptr) { \
   free(ptr); \
   ptr = NULL; \
 } \
}
