#!/bin/sh 
#
# $Id$
#H#
#H#  -------------
#H#   unlock_niqs
#H#  -------------
#H#
#H# Run this script after sending SIGTERM to the niqs if some
#H# niqs processes seem to be stuck.
#H#
#H# There are two reasons why a process will not die
#H# immediately.
#H#
#H#  1. the process is serving a client. In this case it will
#H#     exit after the session is completed.
#H#  2. the process is stuck in the accept() syscall.  In this
#H#     case, this script will unlock the child gracefully, by
#H#     establishing a connection with the server, then closing
#H#     it.
#H#
#H# Notes:
#H#     - If a client is taking too long to terminate the
#H#       session, zkip will shutdown the connection with
#H#       the client.  Unfortunately, this can take up to
#H#       15 minutes (see <maxConnectionTime> in
#H#           /freedom/etc/freedom-frniqsd.conf).
#H#
#H#     - You must run this script on the same machine as the
#H#       niqs.
#H#

#set -x
#set -v

repeatConnect()
{
nb=$1

while [ 0 -lt $nb ]
do echo connect 127.0.0.1
   echo close
   nb=$[ nb - 1 ]
done
}

if [ $# -eq 1 -a "$1" = --help ]
then grep '^#H#' "$0" | colrm 1 3
     exit 0
elif [ $# -ne 0 ]
then echo this command takes no parameter
     exit 1
fi

nbLeft=`ps axww | fgrep frniqsd | fgrep -v fgrep | wc -l`

repeatConnect $nbLeft | /freedom/bin/niqs_test >& /dev/null

sleep 2

nbLeft=`ps axww | fgrep frniqsd | fgrep -v fgrep | wc -l`

if [ $nbLeft -ne 0 ]
then echo There are $nbLeft niqs processes left
else echo Succeeded
fi
