/*
 *  $Id: nsdbwrite.c,v 1.7 2000/02/28 22:01:04 paull Exp $
 *  
 *  Record data to a file.
 *  
 *  This was written to support recording of public keys and nyms, in order to
 *  allow recovery from failurs (e.g. database corruption).
 */

#include "zks.h"

#include <errno.h>
#include <stdio.h>

#include <unistd.h>
#include <arpa/inet.h>
/* #include <zkfile_rec.h>  */
#include "zkFile.h"
#include "frDBOTypes.h"
#include "zkDump.h"
#include "nsdbconv.h"
#include "nsdbread.h"
#include "zkFile.h"
#include "zkAlloc.h"

#define SIZE_OBJECT 1024
#define MAX_LENGTH_TIME 40
#define TIME_FORMAT "%Y-%m-%d %H:%M:%S"  
char * getTime(time_t  timeval);
int getHash(char * source, int type); 
static int  open_destFile(ZkError err, const char * filename, const  char * message, FILE ** handle, int tries,const  char * fn)
{     
  int i;
  const char * mode = "a";
  for(i = 0; i < tries && ! zkFileFopenLocked(err, filename, mode, handle); i++) { 
    if (zkErrorType(err) != ZKWARN && zkErrorCode(err) != EAGAIN) {   
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR,                 
                    "error %d opening `%s' with mode `%s': %s",       
                    errno, filename, mode, strerror(errno));         
      return 0;                                                     
    }                                                                        
  }                                                                         
   if (!handle) {                                                     
    zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR,                   
                  "failed to open `%s' after %d attempts",            
                  filename, tries);                                   
     return 0;                                                             
   }   else printf (message, filename);
   return 1;
                                               
}

char * getTime(time_t   timeval)
{
  struct tm * ftime;
  int len;
  time_t calctime;
  static char  vl[MAX_LENGTH_TIME];
  calctime = timeval;
  ftime = gmtime(&calctime);
  len = strftime (vl,MAX_LENGTH_TIME,TIME_FORMAT,ftime);
  return vl;  
 
}

 int  getHash(char * source, int type) 
{
  int i;
  int hashval;
  
  hashval = type*33;
  for (i=0; source[i] != 0; i++)  
      hashval = hashval*33+source[i];

   return hashval;  
}




     
int nsdbObjWrite ( ZkError err , FrExportCtx ctx,   FrDBOTypeNsdbObj obj)
{
   struct T_frDBOTypeNsdbAipObj  *  f_aip;
   struct T_frDBOTypeNsdbMaipObj *  f_maip;
   struct T_frDBOTypeNsdbLinkObj *  f_link;
   static int firstrow=0;
   FILE * fl; 
   
   /*   int firstcol=0;*/
   (void) err;
   
#define F_INTEGER  "%d;"
#define F_STRING   "%s;"
#define F_LONG     "%Ld;"       
#define F_TIME     F_STRING

   
#define F_OUTPUT(value, typestr)  fprintf(fl,typestr,value);                          
#define F_END  { firstrow++; fprintf(fl, "\n");}

   fl = (obj->type== NsdbAip)? ctx->afp:
       (obj->type== NsdbMaip)? ctx->mafp: ctx->lfp;

   obj->entid = getHash( obj-> entity.name,  obj-> entity.type );
     
   F_OUTPUT( obj-> entid,             F_INTEGER);           /* Entity identifier MUST be first */
   F_OUTPUT( obj-> entity.type,      F_INTEGER);
   F_OUTPUT( obj-> entity.name,      F_STRING);  
   F_OUTPUT( obj-> type,              F_INTEGER);
   F_OUTPUT( obj-> last_seqno,        F_INTEGER);
   F_OUTPUT( getTime(obj-> reference_date),     F_TIME);
   F_OUTPUT( getTime(obj-> launch_date) ,       F_TIME);
   F_OUTPUT( getTime(obj->last_update_date),  F_TIME);
   F_OUTPUT( getTime(obj->last_state_date) ,   F_TIME);
   
   /* commented internal structure variables. in current version they are not use */ 
/*   F_OUTPUT( obj->  up_flag ,           F_INTEGER);    */
/*    F_OUTPUT( obj->  unavailable_flag , F_INTEGER); */
/*    F_OUTPUT( obj->  num_updates  ,     F_INTEGER);    */
/*    F_OUTPUT( obj->  entity_rating,    F_INTEGER); */
/*    F_OUTPUT( obj->  security_level,   F_INTEGER);*/  
   F_OUTPUT( obj->  release ,          F_STRING);
   /* commented internal structure variables. in current version they are not use */ 
 /*   F_OUTPUT( obj->  OR_entity_rating,  F_INTEGER);    */
/*    F_OUTPUT( obj->  OR_up_flag ,      F_INTEGER);    */
/*    F_OUTPUT( getTime(obj-> uptime) ,           F_TIME);    */
/*    F_OUTPUT( getTime(obj-> downtime) ,         F_TIME);   */
/*    */  
   switch (obj->type) {

   case  NsdbAip:
     f_aip=&obj->u.aip;
     F_OUTPUT(f_aip->new_cnx  ,       F_INTEGER);
     F_OUTPUT(f_aip->current_cnx  ,   F_INTEGER);
     F_OUTPUT(f_aip->new_streams  ,   F_INTEGER);
     F_OUTPUT(f_aip->curr_streams  ,  F_INTEGER);
     F_OUTPUT(f_aip->new_ports  ,     F_INTEGER);
     F_OUTPUT(f_aip->curr_ports  ,    F_INTEGER);
     F_OUTPUT(f_aip->bytes_rcvd  ,    F_INTEGER);
     F_OUTPUT(f_aip->bytes_sent  ,    F_INTEGER);
     F_OUTPUT(f_aip->bytes_dropped ,  F_INTEGER);

     F_OUTPUT(f_aip->pkts_rcvd  ,     F_INTEGER);
     F_OUTPUT(f_aip->pkts_sent  ,     F_INTEGER);
     F_OUTPUT(f_aip->pkts_dropped  ,  F_INTEGER);

     F_OUTPUT(f_aip->wh_configured_thput, F_INTEGER);
     F_OUTPUT(f_aip->wh_bytes_rcvd,      F_LONG);
     F_OUTPUT(f_aip->wh_bytes_sent,      F_LONG);
     
     F_OUTPUT(f_aip->wh_update,      F_INTEGER);

     F_OUTPUT(f_aip->total_cnx,           F_LONG);
     F_OUTPUT(f_aip->total_streams,       F_LONG);

     F_OUTPUT(f_aip->total_bytes_rcvd,    F_LONG);
     F_OUTPUT(f_aip->total_bytes_sent,    F_LONG);
     F_OUTPUT(f_aip->total_bytes_dropped, F_LONG);

     F_OUTPUT(f_aip->total_pkts_rcvd,     F_LONG);
     F_OUTPUT(f_aip->total_pkts_sent,     F_LONG);
     F_OUTPUT(f_aip->total_pkts_dropped,  F_LONG);

     F_OUTPUT(f_aip->total_wh_bytes_rcvd, F_LONG);
     F_OUTPUT(f_aip->total_wh_bytes_sent, F_LONG);
     F_OUTPUT(f_aip->num_neighbours,       F_INTEGER);

     break;
   case  NsdbMaip:
     f_maip=&obj->u.maip;

     F_OUTPUT(f_maip->msgs_rcvd, F_INTEGER);
     F_OUTPUT(f_maip->msgs_sent, F_INTEGER);
     F_OUTPUT(f_maip->msgs_dropped,      F_INTEGER);
     F_OUTPUT(f_maip->msgs_queued,       F_INTEGER);
    
     F_OUTPUT(f_maip->bytes_rcvd, F_INTEGER);
     F_OUTPUT(f_maip->bytes_sent, F_INTEGER);
     F_OUTPUT(f_maip->bytes_dropped,      F_INTEGER);

     F_OUTPUT(f_maip->total_msgs_rcvd, F_LONG);
     F_OUTPUT(f_maip->total_msgs_sent, F_LONG);
     F_OUTPUT(f_maip->total_msgs_dropped,      F_LONG);
 
     F_OUTPUT(f_maip->total_bytes_rcvd,    F_LONG);
     F_OUTPUT(f_maip->total_bytes_sent,    F_LONG);
     F_OUTPUT(f_maip->total_bytes_dropped, F_LONG);

     break;
   case  NsdbLink:
    f_link  = &obj->u.link;
    F_OUTPUT( f_link-> linkend , F_INTEGER);  
    F_OUTPUT( f_link-> link_ents_1 , F_LONG);  
    F_OUTPUT( f_link-> link_ents_2 , F_LONG);
    
    F_OUTPUT( f_link-> link_pkts_lost_1 , F_LONG);
    F_OUTPUT( f_link-> link_pkts_lost_2 , F_LONG);
    
    F_OUTPUT( f_link-> link_pkts_dropped_1 , F_LONG);
    F_OUTPUT( f_link-> link_pkts_dropped_2 , F_LONG);
    
    F_OUTPUT( f_link-> link_pkts_rcvd_1 , F_LONG);
    F_OUTPUT( f_link-> link_pkts_rcvd_2 , F_LONG);
    F_OUTPUT( f_link-> link_pkts_sent_1 , F_LONG);
    F_OUTPUT( f_link-> link_pkts_sent_2 , F_LONG);

    F_OUTPUT( f_link-> link_bytes_dropped_1 , F_LONG);
    F_OUTPUT( f_link-> link_bytes_dropped_2 , F_LONG);

    F_OUTPUT( f_link-> link_bytes_rcvd_1 , F_LONG);
    F_OUTPUT( f_link-> link_bytes_rcvd_2 , F_LONG);

    F_OUTPUT( f_link-> link_bytes_sent_1 , F_LONG);
    F_OUTPUT( f_link-> link_bytes_sent_2 , F_LONG);
       
    F_OUTPUT( f_link-> total_bytes_rcvd , F_LONG);
    F_OUTPUT( f_link-> total_bytes_sent , F_LONG);
    F_OUTPUT( f_link-> total_bytes_dropped , F_LONG);

    F_OUTPUT( f_link-> total_pkts_rcvd , F_LONG);
    F_OUTPUT( f_link-> total_pkts_sent , F_LONG);
    F_OUTPUT( f_link-> total_pkts_dropped , F_LONG);
     
    F_OUTPUT( f_link->  total_link_pkts_rcvd_1 , F_LONG);
    F_OUTPUT( f_link->  total_link_pkts_rcvd_2 , F_LONG);
    F_OUTPUT( f_link->  total_link_pkts_sent_1 , F_LONG); 
    F_OUTPUT( f_link->  total_link_pkts_sent_2 , F_LONG); 
    F_OUTPUT( f_link->  total_link_pkts_dropped_1 , F_LONG);
    F_OUTPUT( f_link->  total_link_pkts_dropped_2 , F_LONG);
    break;
   }


   F_END;
 
 return 1;
   
  
}

/*
 * This should not be done this way. -Luc
 */
int nsdbBufferRead(ZkError err, FrExportCtx ctx)
{
  
  const char *fn = "nsdbBufferRead";
  const char *smode = "r";
  
  const int tries = 5; /* number of attempts to open the file */
  struct T_FrDBOTypeNsdbObj object;
  size_t dsz = 0, shsz = 0;
  long nBasket= 0;
  long nAips = 0, nLinks = 0, nMaips = 0,nBads = 0,nFailes=0;
  
  void *hbuf = NULL;
  struct zkfile_rec_hdr_pk hdr = { ZKCLOCK_INIT(0), 0, 0 };    
  int i, ret = 0;
  
  char hash[255];
  
  void *buffer = NULL;
  
  if (!ctx->sourcefile  ) {
    zkErrorSet(err, fn, ZKERR, ZKMFILE , ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  
  


  
  for (i = 0; i < tries && ! (ctx->sfp  = fopen(ctx-> sourcefile    , smode)) ; i++) {
    if (zkErrorType(err) != ZKWARN && zkErrorCode(err) != EAGAIN) {
      zkErrorSet_va(err, fn, ZKERR,ZKMFILE, ZKERROR,
                    "error %d opening `%s' with mode `%s': %s",
                    errno, ctx->sourcefile, smode, strerror(errno));
      goto done;
    }
  }
  
  if ( ! ctx->sfp) {
    zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR,
                  "failed to open `%s' after %d attempts",
                  ctx->sourcefile, tries);
    goto done;
  }
  printf ("Source file : %s \n", ctx->sourcefile);



 
   if (!open_destFile (err,ctx->aipfile,"Destination Aip File %s\n",&ctx->afp, tries, fn  )) goto done;
   if (!open_destFile (err,ctx->maipfile,"Destination Maip File %s\n",&ctx->mafp, tries, fn)) goto done;
   if (!open_destFile (err,ctx->linkfile,"Destination Link File %s\n",&ctx->lfp, tries, fn  )) goto done;

  
  if ( ! zkFileRecordHeaderSerialSize (err, &shsz) ) {
    goto done;
  }

  if ( ! zkMemAlloc(err, (void **) &hbuf, shsz+128) ) {
    goto done;
  }

/*
   * Now, we want to try to read every single buffer that we can.
   * Dump those that we can't decypher.
   */
  while(!feof(ctx->sfp))  {
    size_t osz = 0;
    /* Read the header */
    if ((osz = fread(hbuf, 1, shsz, ctx->sfp)) != shsz) {
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                 "failed to read in record header (read %u bytes)", osz);
      goto done;
    }

    /* Next, extract the information */
    if ( ! zkFileRecordHeaderSerialIn (err, hbuf, shsz,
                                    &hdr, &osz) ) {
      goto done;
    }
    
    if (hdr.hashsz) {
      if (fread(hash, 1, hdr.hashsz,ctx-> sfp) != hdr.hashsz) {
        zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                      "failed to read in hash of size %d", hdr.hashsz);
        goto done;
      }
    }

    if (buffer) {
      free(buffer);
      buffer = NULL;
    }

    dsz = hdr.size;

    if (!(buffer = malloc(dsz))) {
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                      "failed to malloc %d bytes", dsz);
     goto done;
    }

    if (fread(buffer, 1, dsz, ctx-> sfp) != dsz) {
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                    "failed to read in data of size %d", dsz);
      goto done;
    }
    /*
   */
    if ( ! nsdbInitObjectFromBuffer(err, ctx,&object, buffer, dsz) ) {
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                    "failed to initialized object ");
      nFailes++;
    } else if ( ! nsdbObjWrite(err, ctx, &object) ) {
      nFailes++;   
      zkErrorSet_va(err, fn, ZKERR, ZKMFILE, ZKERROR, 
                    "failed to write object to the destination file ");
    } else if (object.type == NsdbAip ) nAips++;
      else if (object.type == NsdbMaip ) nMaips++;
      else if (object.type == NsdbLink ) nLinks++;
      else nBads++;
                                                       
    nBasket++;
   if( nBasket % 17 == 3) 
    printf ("\rProceed busket N: %ld",nBasket);
  }

  ret = 1;
  
 done:
   printf ("\rNumber of baskets %ld",nBasket);
   printf ("\nNumber of Aips = %ld",nAips);
   printf ("\nNumber of Maips= %ld",nMaips);
   printf ("\nNumber of links= %ld",nLinks);
   if (nBads > 0) printf ("\nNumber of not recognized baskets = %ld",nBads);
   if (nFailes > 0) printf ("\nNumber of failed baskets = %ld",nFailes);
  
  if (ctx->afp)   {  fclose(ctx->afp); ctx->afp = NULL; }

  if (ctx->mafp)  {  fclose(ctx->mafp); ctx-> mafp = NULL; }
 
  if (ctx->lfp)   {  fclose(ctx->lfp);  ctx-> lfp = NULL; }
 
 
  if (ctx->sfp) { fclose(ctx-> sfp); ctx->sfp = NULL; }

  
  if (buffer) {
    free(buffer);
  }
  if (hbuf) {
    free(hbuf);
  }

  return ret;
}
