#!/usr/bin/perl -w
#
#************************************************************************
#
#   File: $Id: digestAuditSubs.pl,v 1.2 1999/11/29 23:04:52 alext Exp $
#
#   Last Revision:	$Date: 1999/11/29 23:04:52 $
#
#   Description: Contains routines for accessing audit files
#
#   $Revision: 1.2 $
#
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Time::Local;

my $version = '$Id: digestAuditSubs.pl,v 1.2 1999/11/29 23:04:52 alext Exp $';
my $revision= '$Revision: 1.2 $';
my $progname= 'digestAuditSubs.pl';


###############################################################################
#
# return a partial array extracted from the logfile
# between time and time + timeOffset
#
# fills lineArray ref accessed through @$linearray
#
# curpos is the offset for the seek enable faster access to the log file
#
sub getRestOfAuditFile {
    my($filein, $lineArray, $curpos, $debug) = @_;

    # cast filein to a string
    if(ref $filein) {
        $filein = $$filein;
    }

    if ($debug) {print "getRestOfLogFile: $curpos\n";}

    ## prepare variables
    @$lineArray = ();
    my ($index) = 0;
    my ($line);

    ## open the file
    my ($res) = open(FDIN, "<$filein");

    if ( seek(FDIN, $curpos, 0) && !(eof FDIN)) {
        ## get the data
        ## curpos holds the beginning of the last successfully read line

        my ( $tempcurpos ) = $curpos;

        while ( $line = readAuditFileLine()) {
            if($debug){print "$index: $line";}
            chomp($line);
            push @$lineArray, $line;
            $curpos = $tempcurpos;
            $tempcurpos = tell FDIN;
        }
    }

    ## close the file
    close(FDIN);

    # return file offset for next readings
    return $curpos;
}


###############################################################################
#
# returns index to be read to access 
# timeStamp + timeperiod into the logfile
#
# curpos enables to open the file to any part of the logfile
# this is the offset for the seek
#

sub getIndexInAuditFileFromTime {
  my($filein, $time, $timeOffset, $curpos, $debug) = @_;

  ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  ##
  ## Do time Processing
  ##
  my($timeStr)          = addOffsetToTimeString($time, $timeOffset, $debug);
  my($maxTime)          = getTimeLocalLogFromTimeStr($timeStr, $debug);

  ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  ##
  ## Do file Processing
  ##
  my($res, %params, $index, $newtime);
  if(ref $filein) {
    $filein = $$filein;
  }
  my($newTime, $timeInLine);
  my(@tmp)=();
  $index = 0;
  $res = open(FDIN, "<$filein");
  if(seek(FDIN, $curpos, 0) && !(eof FDIN)){
    $timeInLine = 0;
    while ( readAuditFileLine(\%params) ) {

      $timeInLine = getTimeLocalLogFromTimeStr(
                             $params{date}."-".$params{time}.".0", $debug);
      $index++;
      last if ($timeInLine >= $maxTime);
    }
    if ($index > 0 ) {
      if (!(eof FDIN) ) {
        $index--;
      }
    }
  } else {
    $index = -1;
  }
  close(FDIN);

  if($debug){print "getIndexInFileFromTime: <$index>\n";}
  return $index;
}

################################################################################
# readAuditFileLine reads the next line from FDIN, and decodes and tokenizes it
# as a line from an audit file.
#
#
# It returns 0 on failure, and the whole line on success.  It returns the
# tokens in an associative array.


sub readAuditFileLine {
    my ($tokens) = @_;

    my ($line);

    if (!($line = <FDIN>)) {
        return 0;
    }

    chomp($line);

    return parseAuditFileLine($line, $tokens);
}

# parseAuditFileLine parses a single audit file line into an associative array.
#
# It returns 0 on failure, and the whole line on success.  It returns the
# tokens in an associative array.


sub parseAuditFileLine {
    my ($line, $tokens) = @_;

    my (@splitLine);

    @splitLine = split(/\t/, $line);

    return 0 if (scalar(@splitLine) != 13);

    $$tokens{"date"} = $splitLine[0];
    $$tokens{"time"} = $splitLine[1];
    $$tokens{"user"} = $splitLine[2];
    $$tokens{"host"} = $splitLine[3];
    $$tokens{"session_id"} = $splitLine[4];
    $$tokens{"service"} = $splitLine[5];
    $$tokens{"request"} = decodeBase64 ($splitLine[6]);
    $$tokens{"comment"} = decodeBase64 ($splitLine[7]);
    $$tokens{"ticket_id"} = decodeBase64 ($splitLine[8]);
    $$tokens{"client_id"} = decodeBase64 ($splitLine[9]);
    $$tokens{"transaction_id"} = decodeBase64 ($splitLine[10]);
    $$tokens{"result_code"} = $splitLine[11];
    $$tokens{"result_text"} = decodeBase64 ($splitLine[12]);

    return $line;
}

################################################################################
#
# Stolen from "Programming Perl".  I have no idea at all how it works.
#
#    

sub decodeBase64 {
    ($_) = @_;
    chomp;

    tr#A-Za-z0-9+/##cd;                   # remove non-base64 chars
    tr#A-Za-z0-9+/# -_#;                  # convert to uuencoded format
    my($len) = pack("c", 32 + 0.75*length );   # compute length byte
    return unpack("u", $len . $_);         # uudecode and print
}




1;

#   $Log: digestAuditSubs.pl,v $
#   Revision 1.2  1999/11/29 23:04:52  alext
#   Add persistency support for frnscpd and frloded reports.
#
#   Revision 1.1  1999/11/29 17:14:16  alext
#   New scripts for changes in nym state.
#
