#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestFmg.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		14 Oct 1999
#
#   Last Revision:	$Date: 1999/11/05 04:50:43 $
#
#   Description:	
#
#   $Revision: 1.8 $
#
#
#   $Id: digestFmg.pl,v 1.8 1999/11/05 04:50:43 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: digestFmg.pl,v 1.8 1999/11/05 04:50:43 thierry Exp $';
my $revision= '$Revision: 1.8 $';
my $progname= 'digestFmg.pl';

#################################################################################
#
# returns version info
#
sub getDigestFmgInfo { return "$progname:$version:$revision";}

#################################################################################
#
# Log messages found in fmg.c :
#
#
# Message related:
#
# zkLogWrite(context->log,"Processing new mail message");
# zkLogWrite_va(context->log,"Message sent to %s from %s",addr,from);
# zkLogWrite_va(context->log,"Unable to send message to \"%s\" from \"%s\"",
# zkLogWrite_va(context->log,"RBL blocked message from \"%s\"",source);
#
# zkLogWrite_va(context->log,"Could not retrieve email domain for %s",nym);
# zkLogWrite_va(context->log,"Trying to send mail to %s@%s, email domain should be %s",
# zkLogWrite_va(context->log,"Missing email domain for target nym %s",
#
# Key related:
#
# zkLogWrite(context->log,"Failed to encrypt message with target nym key");
# zkLogWrite(context->log,"Failed to encrypt message with FMG key");
# zkLogWrite(context->log,"Failed to retrieve my private signature key");
# zkLogWrite_va(context->log,"Could not retrieve keys for %s",nym);
# zkLogWrite_va(context->log,"Failed to fetch key for %s (%d)",
#
# Process Related
#
# zkLogWrite(context->log,"getprotobyname() failed for \"udp\"\n");
# zkLogWrite(context->log,"Error reading from input stream");
# zkLogWrite_va(context->log,"%s:%d response was %d '%s'",
# zkLogWrite_va(context->log, "Error creating socket (%s).\n", 
# zkLogWrite_va(context->log,"Logging incoming message to \"%s\"",
#
#

#################################################################################
#
# Get received messages
#
sub getFmgRxMsgCount{
  my($partialTmp) = @_;
  my($line,$strLength);
  my(@res)=();

  # process incoming message
  @res = grep /\"Processing new/, @$partialTmp;
#  my $prevStrLength = 85;
#   foreach $line (@res){
#     chomp($line);
#     $strLength = $prevStrLength  - length($line);
#     print "$line", " " x $strLength, "\r";
#     $prevStrLength = length($line);
#   }
  return scalar @res;
}


#################################################################################
#
# Get transmitted messages
#
sub getFmgTxMsgCount {
  my($partialTmp) = @_;
  my($line,$strLength);
  my(@res)=();

  # process outgoing message
  @res = grep /\"Message sent/, @$partialTmp;
#  my $prevStrLength = 85;
#   foreach $line (@res){
#     chomp($line);
#     $strLength = $prevStrLength  - length($line);
#     print "$line", " " x $strLength, "\r";
#     $prevStrLength = length($line);
#   }
  return scalar @res;
}

#################################################################################
#
# Get error/dropped  messages
#
sub getFmgDropMsgCount {
  my($partialTmp) = @_;
  my($line,$strLength);
  my(@res)=();

  # process Unable to send message
  @res = grep /\"Unable to send/,   @$partialTmp;
#  my $prevStrLength = 85;
#   foreach $line (@res){
#     chomp($line);
#     $strLength = $prevStrLength  - length($line);
#     print "$line", " " x $strLength, "\r";
#     $prevStrLength = length($line);
#   }
  return scalar @res;
}

#################################################################################
#
# Get blocked  messages
#
sub getFmgBlockedMsgCount {
  my($partialTmp) = @_;
  my($line,$strLength);
  my(@res)=();

  # process Unable to send message
  @res = grep /\"RBL blocked message/,   @$partialTmp;
#  my $prevStrLength = 85;
#   foreach $line (@res){
#     chomp($line);
#     $strLength = $prevStrLength  - length($line);
#     print "$line", " " x $strLength, "\r";
#     $prevStrLength = length($line);
#   }
  return scalar @res;
}

#################################################################################
#
# process all the fmg logfile starting at beginning
# starts at beginning and runs to the end of the log file
# fast acces is used by keeping offset for the access to the file
# see getPartialLogFile to get an idea
#
sub processFmgLog {
  my($infile, $timeStamp, $timePeriod, $reportType, $debug) = @_;
  
  my($line,$strLength, $prevStrLength);
  my($rxMsgs, $txMsgs, $erMsgs, $bkMsgs);
  my($time, $host, $pid, $service, @msg);

  # get first line if $time == 0
  if(! $timeStamp){
    $line = `head -n 1 $$infile`;
    ($time, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
    if($debug){
      print "\t$time\n\t$host\n\t$pid\n\t$service\n\t", join(" ", @msg),"\n";
    }
  }else{
    $time = $timeStamp;
  }

  my(@partialTmp)   =();
  my($bytesOffset)  = 0;
  my($FlagComplete) = 1;
  my($quMsgs)       = 0;

  while ($FlagComplete){
    @partialTmp = ();

    ## get partial log file and keep the byte offset for fastest access
    $bytesOffset = getPartialLogFile($infile, $time, $timePeriod, 
				     \@partialTmp, $bytesOffset, $debug);
    last if($bytesOffset < 0) ; # stops 
    if($debug){print join("\n", @partialTmp),"\n";}

    $rxMsgs = getFmgRxMsgCount(\@partialTmp);  # process incoming message
    $txMsgs = getFmgTxMsgCount(\@partialTmp);  # process outgoing message
    $erMsgs = getFmgDropMsgCount(\@partialTmp);# process Unable to send message
    $bkMsgs = getFmgBlockedMsgCount(\@partialTmp);# process Blocked message
    $quMsgs = $rxMsgs - ($erMsgs + $txMsgs + $bkMsgs);   # process Message Queue

    ## get the printout done
  SWITCH: {
      if($reportType =~ /xml/) {
	print  "<fmgInfo>\n",
	       "   <timeStamp>$time</timeStamp>\n",
	       "   <tmPeriod>$timePeriod</tmPeriod>\n",
	       "   <rxMsgs>$rxMsgs</rxMsgs>\n",
	       "   <txMsgs>$txMsgs</txMsgs>\n",
	       "   <errorMsgs>$erMsgs</errorMsgs>\n",
	       "   <blockMsgs>$bkMsgs</blockMsgs>\n",
	       "   <queueMsgs>$quMsgs</queueMsgs>\n",
	       "</fmgInfo>\n",
	       "\n";
	last SWITCH;
      }
      if($reportType =~ /line/) {
	print 
	  $time,";",
	  $timePeriod,";",
	  $rxMsgs,";",
	  $txMsgs,";",
	  $erMsgs,";",
	  $bkMsgs,";",
	  $quMsgs,"\n";
	last SWITCH;
      }
      print  "Time  start       : ", $time,"\n";
      print  "Time  period      : ", $timePeriod,"\n";
      printf("Incoming Messages : %10d\n", $rxMsgs);
      printf("Outgoing Messages : %10d\n", $txMsgs);
      printf("Error    Messages : %10d\n", $erMsgs);
      printf("Blocked  Messages : %10d\n", $bkMsgs);
      printf("Messages Queue    : %10d\n", $quMsgs);
      print  "\n";
    }
    
    last if($timeStamp);

    $time = addOffsetToTimeString($time, $timePeriod, 0);
    if($debug){
      print  "Time  end         : ", $time,"\n";
      print "\n";
    }

    if($bytesOffset < 0){
      $FlagComplete = 0;
    }
  }
  return;
}


1;

#   $Log: digestFmg.pl,v $
#   Revision 1.8  1999/11/05 04:50:43  thierry
#   	typo fixe replace ,, with ,
#
#   Revision 1.7  1999/10/25 15:44:53  thierry
#   fix bug in getPartialLogFile
#
#   Revision 1.6  1999/10/24 22:15:30  thierry
#   add Serial Number Redeemed support
#
#   Revision 1.5  1999/10/19 13:35:52  thierry
#   add line report type support
#
#   Revision 1.4  1999/10/18 21:42:42  thierry
#   add --startat [timeSTamp] Support
#
#   Revision 1.3  1999/10/15 21:25:57  thierry
#   add Sngen support
#
#   Revision 1.2  1999/10/15 15:57:20  thierry
#   add version info from all files
#
#   Revision 1.1  1999/10/15 15:18:57  thierry
#   log digest tool
#
