#!/usr/bin/perl -w
#
#************************************************************************
#
#   File: $Id: digestFrnscp.pl,v 1.9 1999/12/01 06:32:53 nicolas Exp $
#
#   Last Revision:	$Date: 1999/12/01 06:32:53 $
#
#   Description:	
#
#   $Revision: 1.9 $
#
#
#   $Id: digestFrnscp.pl,v 1.9 1999/12/01 06:32:53 nicolas Exp $
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;
my $version = '$Id: digestFrnscp.pl,v 1.9 1999/12/01 06:32:53 nicolas Exp $';
my $revision= '$Revision: 1.9 $';
my $progname= 'digestFrnscp.pl';

select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# returns version info
#
sub getDigestFrnscpInfo { return "$progname:$version:$revision";}


################################################################################
#
# Generate results from a single line.  Returns 0 on failure, 1 on success.  If
# successful, the result should be output.
#

sub processOneFrnscp {
    my ($line,
        $time,
        $nym,
        $action,
        $initiator,
        $justification,
        $result,
        $debug) = @_;

    my ($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);

    $$time = $logtime;

    my $message = join(" ", @msg);
    if ($debug) {print "processOneFrnscp: $message\n";}

  SWITCH_LINE_TYPE: {

      # Nym creation or upgrade message
      if ($line =~ /(\w*) nym (\S*) using token type (\d*)/) {

          $$nym = $2;
          $$action = lc($1);
          $$justification = "Token type $3";
          $$result = "Succeeded";

          return 1;
      }

      # Nym modify or disable message
      if ($line =~ /client issued \S* (\S*) remotenym binary \d* (\S*)/) {

          $$nym = $2;
          $$action = lc($1);

          return 1;
      }
  }

    return 0;
}

#################################################################################
#
# Log messages found in frnscpd log :
#
#
#
sub processFrnscp { 
    my($infile, $persist, $reportType, $debug) = @_;

    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

# Get and validate persistency information

    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processFrnscp: Couldn't get persistency information.");
        exit (1);
    }

    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processFrnscp: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processFrnscp: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readLogFileLine();

        if ($line ne $lineCheck) {
            logError ("processFrnscp: Sanity Check on next line failed!");
            exit 1;
        }

# Reset curpos to first unprocessed line.
        $curpos = tell(FDIN);

        close(FDIN);
    }

# Process from the curpos to the end of file

    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($timeSql) ="";
    my($logMsg)="";

# get partial log file and keep the byte offset for fastest access
    $curpos = getRestOfLogFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

# grep for interesting lines and update

    @linesOfInterest = grep(/CREATED nym|modify remotenym binary|disable remotenym binary|UPGRADED nym/,
                            @partialTmp);

    foreach $logMsg (@linesOfInterest) {
        # set defaults

        my $nym = "";
        my $action = "";
        my $initiator = "Unknown";
        my $justification = "None";
        my $result = "Unknown";            

        $writeThis = processOneFrnscp($logMsg,
                                      \$time,
                                      \$nym,
                                      \$action,
                                      \$initiator,
                                      \$justification,
                                      \$result,
                                      $debug);
        if ( $writeThis ) {
#Insert sanity check output

            outputRecord((convertTimeToSQLTime($time),
                          $nym,
                          $initiator,
                          $action,
                          $justification,
                          $result));
        }
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


1;

#   $Log: digestFrnscp.pl,v $
#   Revision 1.9  1999/12/01 06:32:53  nicolas
#   Fixed the order of the output.
#
#   Revision 1.8  1999/12/01 01:35:24  filou
#   Added logError function calls
#
#   Revision 1.7  1999/12/01 00:32:30  alext
#   Adding default directories, FDOUT descriptor, persistency cleanup commands
#
#   Revision 1.6  1999/11/30 23:45:54  filou
#   Added FDLOG file descriptor to print calls for error messages, removed unused parameters
#
#   Revision 1.5  1999/11/30 21:58:23  filou
#   added new parameters
#
#   Revision 1.4  1999/11/30 03:09:43  filou
#   Changed order of sequence for persistence
#
#   Revision 1.3  1999/11/29 23:04:52  alext
#   Add persistency support for frnscpd and frloded reports.
#
#   Revision 1.2  1999/11/29 18:41:07  alext
#   Fixing date output for frnscp and lodeaudit
#
#   Revision 1.1  1999/11/29 17:14:16  alext
#   New scripts for changes in nym state.
#
