#!/usr/bin/perl -w
#
#************************************************************************
#
#   File: $Id: digestLodeAudit.pl,v 1.9 1999/12/01 06:32:40 louis Exp $
#
#   Last Revision:	$Date: 1999/12/01 06:32:40 $
#
#   Description:	
#
#   $Revision: 1.9 $
#
#
#   $Id: digestLodeAudit.pl,v 1.9 1999/12/01 06:32:40 louis Exp $
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;

my $version = '$Id: digestLodeAudit.pl,v 1.9 1999/12/01 06:32:40 louis Exp $';
my $revision= '$Revision: 1.9 $';
my $progname= 'digestLodeAudit.pl';

select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# returns version info
#
sub getDigestLodeAuditInfo { return "$progname:$version:$revision";}


################################################################################
#
# Generate results from a single line.  Returns 0 on failure, 1 on success.  If
# successful, the result should be output.
#

sub processOneLodeAudit {
    my ($line,
        $time,
        $nym,
        $action,
        $initiator,
        $justification,
        $result,
        $debug) = @_;

    my %params = ();

    if (!parseAuditFileLine($line, \%params)) {
        return 0;
    }

    # if the command failed we don't report it.


    $$time = $params{date}."-".$params{time};

    if ($params{result_text} =~ /\(([^ )]*)\) *\(([^ )]*)\) *\(([^ )]*)\) */) {
       $$action = $2;
       $$nym = $3;
    }

    $$initiator = $params{user};
    $$justification = $params{comment};
    $$result = ($params{result_code} eq "0x00000000") ? "Succeeded" : "Failed" ;

    return 1;
}

#################################################################################
#
# Log messages found in frloded.audit :
#
#
#
sub processLodeAudit { 
    my($infile, $persist, $reportType, $debug) = @_;
  
    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

# Get and validate persistency information

    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processLodeAudit: Couldn't get persistency information.");
        exit (1);
    }

    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processLodeAudit: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processLodeAudit: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readAuditFileLine();

        if ($line ne $lineCheck) {
            logError ("processLodeAudit: Sanity Check on next line failed!");
            exit 1;
        }

# Reset curpos to first unprocessed line
        $curpos = tell(FDIN);

        close(FDIN);
    }

# Process from the curpos to the end of file

    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($logMsg)="";

# get partial audit file and keep the byte offset for fastest access
    $curpos = getRestOfAuditFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

# grep for interesting lines and update

    @linesOfInterest = grep /nymdb/, @partialTmp;

    foreach $logMsg (@linesOfInterest) {

        my $nym = "";
        my $action = "";
        my $initiator = "Unknown";
        my $justification = "None";
        my $result = "Unknown";

        $writeThis = processOneLodeAudit($logMsg,
                                         \$time,
                                         \$nym,
                                         \$action,
                                         \$initiator,
                                         \$justification,
                                         \$result,
                                         $debug);
        if ( $writeThis ) {
#Insert sanity check output
                
            outputRecord((convertTimeToSQLTime($time),
                          $nym,
                          $initiator,
                          $action,
                          $justification,
                          $result));
                
        }
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


1;

#   $Log: digestLodeAudit.pl,v $
#   Revision 1.9  1999/12/01 06:32:40  louis
#   - reordered the ouput filed as per request !!!
#
#   Revision 1.8  1999/12/01 01:35:36  filou
#   Added logError function calls
#
#   Revision 1.7  1999/11/30 23:46:02  filou
#   Added FDLOG file descriptor to print calls for error messages, removed unused parameters
#
#   Revision 1.6  1999/11/30 21:58:33  filou
#   added new parameters
#
#   Revision 1.5  1999/11/30 03:09:31  filou
#   Changed order of sequence for persistence
#
#   Revision 1.4  1999/11/29 23:04:53  alext
#   Add persistency support for frnscpd and frloded reports.
#
#   Revision 1.3  1999/11/29 20:16:10  alext
#   Fixed bug in date format
#
#   Revision 1.2  1999/11/29 18:41:08  alext
#   Fixing date output for frnscp and lodeaudit
#
#   Revision 1.1  1999/11/29 17:14:16  alext
#   New scripts for changes in nym state.
#
