#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestNymMailActivity.pl
#
#   Author(s):		Philippe Desaulniers
#
#   Created:		 29 Nov 1999
#
#   Last Revision:	$Date: 1999/12/03 21:59:49 $
#
#   Description:	
#
#   $Revision: 1.5 $
#
#
#   $Id: digestNymMailActivity.pl,v 1.5 1999/12/03 21:59:49 filou Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;

my $version = '$Id: digestNymMailActivity.pl,v 1.5 1999/12/03 21:59:49 filou Exp $';
my $revision= '$Revision: 1.5 $';
my $progname= 'digestNymMailNewsInfo.pl';


select(STDERR); $| = 1;
select(STDOUT); $| = 1;


#################################################################################
#
# returns version info
#
sub getDigestNymMailActivityInfo { return "$progname:$version:$revision";}


################################################################################
#
# 1999/11/05-16:36:25.0 afs1.freedom.net 8616 FMG-maip "QUEUEING message from foobar to blatfoo@cs.mcgill.ca"
#
#  Can be nym to internet address or nym to nym
#
#   OR
# 1999/11/28-18:55:52.0 fs3 13177 fmg "Message sent to foobar from barfoo@foo.bar"
#
#  Can be internet address to nym
#
#
sub processOneNymMailActivity {
  my ($logMsg, $numLogs, $timeLogs, $froms, $tos, $debug)= @_;
	my $returnValue = 1;

  my $logtime;
	my $host;
	my $pid;
	my $service;
	my @msg;

	my $from;
	my $to;
	my $rNym;
	my $sNym;

  ($logtime, $host, $pid, $service, @msg) = splitLogLine($logMsg, $debug);
  my $message = join(" ", @msg);

	$$numLogs = 0;

  if($debug){print "processOneNymMailActivity: $message\n";}
  if($message =~ /^\"QUEUEING\s+message\s+from\s+(.*)\s+to\s+(.*)\"/){
    if($debug){print "processOneNymMailActivity: start to parse\n";}

    $from = $1;
		$to = $2;

		if ($from =~ /(.*)\@freedom.net$/) {
			$sNym = $1;
		}

		else {
			$sNym = $from;
		}

		if ($sNym eq "fmg") {
			# Ignore, this case is processed by other $message form.
		}

		else {
			$$numLogs = 1;
			$$timeLogs[0] = $logtime;	
			$$froms[0] = $sNym;	
			$$tos[0] = $to;	

			if($debug){print "processOneNymMailActivity: found one mail from ", $$from[0], " to ", $$tos[0], "\n";}
		}
  }

  elsif($message =~ /\"Message\s+sent\s+to\s+(.*)\s+from\s+(.*)\"$/){
    if($debug){print "processOneNymMailActivity: start to parse\n";}

    $to = $1; 
    $from = $2;

		# This case is from internet to nym only...
		$$numLogs = 1;
		$$timeLogs[0] = $logtime;	
		$$froms[0] = $from;	
		$$tos[0] = $to;	

		if($debug){print "processOneNymMailActivity: found one mail from ", $$from[0], " to ", $$tos[0], "\n";}
  }

	else {
		logError ("processOneNymMailActivity: received unprocessable message ", $message);
		$returnValue = 0;
	}

	return ($returnValue);
}
  
###############################################################################
#
# 1999/11/05-16:36:25.0 afs1.freedom.net 8616 FMG-maip "QUEUEING message from foobar to blatfoo@cs.mcgill.ca"
#   OR
# 1999/11/28-18:55:52.0 fs3 13177 fmg "Message sent to foobar from barfoo@foo.bar"
#
#
sub processNymMailActivity {
    my($infile, $persist, $reportType, $debug) = @_;

    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

# Get and validate persistency information

    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processNymMailActivity: Couldn't get persistency information.");
        exit (1);
    }

    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processNymMailActivity: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processNymMailActivity: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readLogFileLine();

        if ($line ne $lineCheck) {
            logError ("processNymMailActivity: Sanity Check on next line failed!");
            exit 1;
        }

# Reset curpos to first unprocessed line.
        $curpos = tell(FDIN);

        close(FDIN);
    }

# Process from the curpos to the end of file

    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($timeSql) ="";
    my($logMsg)="";

# get partial log file and keep the byte offset for fastest access
    $curpos = getRestOfLogFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

# grep for interesting lines and update

    @linesOfInterest = grep /\"QUEUEING\s+message\s+from|\"Message\s+sent\s+to\s+/,
                            @partialTmp;

    foreach $logMsg (@linesOfInterest) {
			# set defaults

			my $numLogs = 0;
			my @timeLogs = ();
			my @froms = ();
			my @tos = ();

			$writeThis = processOneNymMailActivity($logMsg,
																						 \$numLogs, 
																						 \@timeLogs, 
																						 \@froms, 
																						 \@tos, 
																						 $debug);
			if ( $writeThis ) {
#Insert sanity check output
				
				my ($entryCount);

				for ($entryCount = 0; $entryCount < $numLogs; $entryCount = $entryCount + 1) {
					outputRecord(convertTimeToSQLTime($timeLogs[$entryCount]),
											 $froms[$entryCount],
											 $tos[$entryCount]);
				}
			}
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


1;

#   $Log: digestNymMailActivity.pl,v $
#   Revision 1.5  1999/12/03 21:59:49  filou
#   Added line of interest descriptions in comments
#
#   Revision 1.4  1999/12/01 01:35:52  filou
#   Added logError function calls
#
#   Revision 1.3  1999/11/30 23:46:11  filou
#   Added FDLOG file descriptor to print calls for error messages, removed unused parameters
#
#   Revision 1.2  1999/11/30 21:58:50  filou
#   added new parameters
#
#   Revision 1.1  1999/11/30 03:14:51  filou
#   Creation, used for Nym Vitals 1.5 report
#
#
#
