#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestNymMailNewsInfo.pl
#
#   Author(s):		Philippe Desaulniers
#
#   Created:		 25 Nov 1999
#
#   Last Revision:	$Date: 1999/12/03 22:00:11 $
#
#   Description:	
#
#   $Revision: 1.6 $
#
#
#   $Id: digestNymMailNewsInfo.pl,v 1.6 1999/12/03 22:00:11 filou Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;

my $version = '$Id: digestNymMailNewsInfo.pl,v 1.6 1999/12/03 22:00:11 filou Exp $';
my $revision= '$Revision: 1.6 $';
my $progname= 'digestNymMailNewsInfo.pl';


select(STDERR); $| = 1;
select(STDOUT); $| = 1;


#################################################################################
#
# returns version info
#
sub getDigestNymMailNewsInfoInfo { return "$progname:$version:$revision";}


################################################################################
#
# 1999/11/05-16:36:25.0 afs1.freedom.net 8616 FMG-maip "QUEUEING message from foobar to blatfoo@cs.mcgill.ca"
#
#  Can be nym to internet address, fmg to nym (interpreted as internet address to nym), nym to nym or 
#   MAILER-DAEMON to nym (interpreted as discarded message).
#
#   OR
# 1999/12/01-20:14:04.0 fs3 17719 frpostnews "Posting article "test with space agent" to microsoft.public.test from space_agent@freedom.net successful"
#
#  Nym to newsgroup(s). Multiple newsgroups are possible, they will be comma separated.
#
#   OR
# 1999/11/23-19:44:52.0 fs3 6329 frpostnews "Failed to post news article, sending mail to space_agent@freedom.net"
#
#  Nym to newsgroup. Single newsgroup only, counted as discarded.
#
#   OR
# 1999/11/19-18:25:17.0 cabbage.test.freedom.net 16546 FMG-maip "Daily quota exceeded for cucumber"
#
#  Single nym. Counted as one discarded mail message. This is a problem, because if multiple recipients were listed, only one
#   occurence of this messages will be found in the log file.
#
#   OR
# 1999/11/11-22:00:58.0 cabbage.test.freedom.net 32102 FMG-maip "Mail from roger@zks.net blocked by recipient asparagus"
#
#  Can be internet address to nym, or nym to nym. Counts as one discarded mail message for recipient, and for sender if
#   sender is a nym.
#
#
sub processOneNymMailNewsInfo {
  my ($logMsg, $numLogs, $timeLogs, $nyms, $mailSentFlags, $mailReceivedFlags, $mailDiscardedFlags, $newsPostedFlags, $newsDiscardedFlags, $debug)= @_;
	my $returnValue = 1;

  my $logtime;
	my $host;
	my $pid;
	my $service;
	my @msg;

	my $from;
	my $to;
	my $rNym;
	my $sNym;

	my @ngs;
	my $ngstring;
	my $ng;

  ($logtime, $host, $pid, $service, @msg) = splitLogLine($logMsg, $debug);
  my $message = join(" ", @msg);

	$$numLogs = 0;

  if($debug){print "processOneNymMailNewsInfo: $message\n";}
  if($message =~ /^\"QUEUEING\s+message\s+from\s+(.*)\s+to\s+(.*)\"/){
    if($debug){print "processOneNymMailNewsInfo: start to parse\n";}

    $from = $1;
		$to = $2;

		if ($from =~ /(.*)\@freedom.net$/) {
			$sNym = $1;
		}

		else {
			$sNym = $from;
		}

		$rNym = $to unless ($to =~ /.*\@.*/);

		if ($sNym =~ /^MAILER-DAEMON/) {
			if($debug){print "processOneNymMailNewsInfo: found one discarded and one received mail for ", $rNym, "\n";}
			$$numLogs = 2;

			# Assume mail was discarded by mailer daemon
			$$timeLogs[0] = $logtime;	
			$$nyms[0] = $rNym;	
			$$mailSentFlags[0] = 0;	
			$$mailReceivedFlags[0] = 0;	
			$$mailDiscardedFlags[0] = 1;	
			$$newsPostedFlags[0] = 0;	
			$$newsDiscardedFlags[0] = 0;	

			# Count this as one received mail by nym
			$$timeLogs[1] = $logtime;	
			$$nyms[1] = $rNym;	
			$$mailSentFlags[1] = 0;	
			$$mailReceivedFlags[1] = 1;	
			$$mailDiscardedFlags[1] = 0;	
			$$newsPostedFlags[1] = 0;	
			$$newsDiscardedFlags[1] = 0;	
		}

		elsif ($sNym eq "fmg") {
			if($debug){print "processOneNymMailNewsInfo: found one received mail for ", $rNym, "\n";}
			$$numLogs = 1;
			$$timeLogs[0] = $logtime;	
			$$nyms[0] = $rNym;	
			$$mailSentFlags[0] = 0;	
			$$mailReceivedFlags[0] = 1;	
			$$mailDiscardedFlags[0] = 0;	
			$$newsPostedFlags[0] = 0;	
			$$newsDiscardedFlags[0] = 0;	
		}

		else {
			if (defined($rNym)) {
				if($debug){print "processOneNymMailNewsInfo: found one received mail for ", $rNym, "\n";}
				$$numLogs = 2;
				$$timeLogs[1] = $logtime;	
				$$nyms[1] = $rNym;	
				$$mailSentFlags[1] = 0;	
				$$mailReceivedFlags[1] = 1;	
				$$mailDiscardedFlags[1] = 0;	
				$$newsPostedFlags[1] = 0;	
				$$newsDiscardedFlags[1] = 0;	
			}

			else {
				$$numLogs = 1;
			}

			if($debug){print "processOneNymMailNewsInfo: found one sent mail for ", $sNym, "\n";}
			$$timeLogs[0] = $logtime;	
			$$nyms[0] = $sNym;	
			$$mailSentFlags[0] = 1;	
			$$mailReceivedFlags[0] = 0;	
			$$mailDiscardedFlags[0] = 0;	
			$$newsPostedFlags[0] = 0;	
			$$newsDiscardedFlags[0] = 0;	
		}
  }

  elsif($message =~ /\"Failed to post news article, sending mail to (.*)\@(.*)\"$/){
    if($debug){print "processOneNymMailNewsInfo: start to parse\n";}

    $from = $1;
		$sNym = $from;

		# Limitation : will only mark one failed post, even if nym was crossposting.
		if($debug){print "processOneNymMailNewsInfo: found one discarded news post for ", $sNym, "\n";}
		$$numLogs = 1;
		$$timeLogs[0] = $logtime;	
		$$nyms[0] = $sNym;	
		$$mailSentFlags[0] = 0;	
		$$mailReceivedFlags[0] = 0;	
		$$mailDiscardedFlags[0] = 0;	
		$$newsPostedFlags[0] = 0;	
		$$newsDiscardedFlags[0] = 1;	
  }

  elsif($message =~ /\"Daily quota exceeded for (.*)\"$/){
    if($debug){print "processOneNymMailNewsInfo: start to parse\n";}

    $from = $1;
		$sNym = $from;

		# Limitation : will only mark one failed mail, even if there were multiple recipients.
		if($debug){print "processOneNymMailNewsInfo: found one discarded mail for ", $sNym, "\n";}
		$$numLogs = 1;
		$$timeLogs[0] = $logtime;	
		$$nyms[0] = $sNym;	
		$$mailSentFlags[0] = 0;	
		$$mailReceivedFlags[0] = 0;	
		$$mailDiscardedFlags[0] = 1;	
		$$newsPostedFlags[0] = 0;	
		$$newsDiscardedFlags[0] = 0;	
  }

  elsif($message =~ /\"Posting article .* to (.*) from (.*) successful\"$/){
    if($debug){print "processOneNymMailNewsInfo: start to parse\n";}

		$ngstring = $1;
		$from = $2;
		$from =~ /(.*)\@.*/;
		$sNym = $1;

		@ngs = split(/,/, $ngstring);

		$$numLogs = 0;

		foreach $ng (@ngs) {
			if($debug){print "processOneNymMailNewsInfo: found one posted newsgroup for ", $sNym, "\n";}
			$$timeLogs[$$numLogs] = $logtime;	
			$$nyms[$$numLogs] = $sNym;	
			$$mailSentFlags[$$numLogs] = 0;	
			$$mailReceivedFlags[$$numLogs] = 0;	
			$$mailDiscardedFlags[$$numLogs] = 0;	
			$$newsPostedFlags[$$numLogs] = 1;	
			$$newsDiscardedFlags[$$numLogs] = 0;	
			$$numLogs = $$numLogs + 1;
		}
  }

  elsif($message =~ /\"Mail from (.*) blocked by recipient (.*)\"$/){
    if($debug){print "processOneNymMailNewsInfo: start to parse\n";}

    $from = $1;
		$to = $2;

		$rNym = $to;

		if ($from =~ /(.*)\@freedom.net$/) {
			if($debug){print "processOneNymMailNewsInfo: found one discarded mail for ", $sNym, "\n";}
			$sNym = $1;
			$$numLogs = 2;
			$$timeLogs[1] = $logtime;	
			$$nyms[1] = $sNym;	
			$$mailSentFlags[1] = 0;	
			$$mailReceivedFlags[1] = 0;	
			$$mailDiscardedFlags[1] = 1;	
			$$newsPostedFlags[1] = 0;	
			$$newsDiscardedFlags[1] = 0;	
		}

		else {
			$$numLogs = 1;
		}

		if($debug){print "processOneNymMailNewsInfo: found one discarded mail for ", $rNym, "\n";}
		$$timeLogs[0] = $logtime;	
		$$nyms[0] = $rNym;	
		$$mailSentFlags[0] = 0;	
		$$mailReceivedFlags[0] = 0;	
		$$mailDiscardedFlags[0] = 1;	
		$$newsPostedFlags[0] = 0;	
		$$newsDiscardedFlags[0] = 0;	
  }

	else {
		logError ("processOneNymMailNewsInfo: received unprocessable message ", $message);
		$returnValue = 0;
	}

	return ($returnValue);
}
  
###############################################################################
#
# 1999/11/05-16:36:25.0 afs1.freedom.net 8616 FMG-maip "QUEUEING message from foobar to blatfoo@cs.mcgill.ca"
#   OR
# 1999/12/01-20:14:04.0 fs3 17719 frpostnews "Posting article "test with space agent" to microsoft.public.test from space_agent@freedom.net successful"
#   OR
# 1999/11/23-19:44:52.0 fs3 6329 frpostnews "Failed to post news article, sending mail to space_agent@freedom.net"
#   OR
# 1999/11/19-18:25:17.0 cabbage.test.freedom.net 16546 FMG-maip "Daily quota exceeded for cucumber"
#   OR
# 1999/11/11-22:00:58.0 cabbage.test.freedom.net 32102 FMG-maip "Mail from roger@zks.net blocked by recipient asparagus"
#
#
sub processNymMailNewsInfo {
    my($infile, $persist, $reportType, $debug) = @_;

    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

# Get and validate persistency information

    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processNymMailNewsInfo: Couldn't get persistency information.");
        exit (1);
    }

    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processNymMailNewsInfo: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processNymMailNewsInfo: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readLogFileLine();

        if ($line ne $lineCheck) {
            logError ("processNymMailNewsInfo: Sanity Check on next line failed!");
            exit 1;
        }

# Reset curpos to first unprocessed line.
        $curpos = tell(FDIN);

        close(FDIN);
    }

# Process from the curpos to the end of file

    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($timeSql) ="";
    my($logMsg)="";

# get partial log file and keep the byte offset for fastest access
    $curpos = getRestOfLogFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

# grep for interesting lines and update

    @linesOfInterest = grep /\"QUEUEING\s+message\s+from|\"Posting article .* to .* from .* successful\"$|\"Failed to post news article.*\"$|\"Daily quota exceeded for.*\"$|\"Mail from .* blocked.*\"$/,
                            @partialTmp;

    foreach $logMsg (@linesOfInterest) {
			# set defaults

			my $numLogs = 0;
			my @timeLogs = ();
			my @nyms = ();
			my @mailSentFlags = ();
			my @mailReceivedFlags = ();
			my @mailDiscardedFlags = ();
			my @newsPostedFlags = ();
			my @newsDiscardedFlags = ();

			$writeThis = processOneNymMailNewsInfo($logMsg,
																						 \$numLogs, 
																						 \@timeLogs, 
																						 \@nyms, 
																						 \@mailSentFlags, 
																						 \@mailReceivedFlags, 
																						 \@mailDiscardedFlags, 
																						 \@newsPostedFlags, 
																						 \@newsDiscardedFlags, 
																						 $debug);
			if ( $writeThis ) {
#Insert sanity check output
				
				my ($entryCount);

				for ($entryCount = 0; $entryCount < $numLogs; $entryCount = $entryCount + 1) {
					outputRecord(convertTimeToSQLTime($timeLogs[$entryCount]),
											 $nyms[$entryCount],
											 $mailSentFlags[$entryCount],
											 $mailReceivedFlags[$entryCount],
											 $mailDiscardedFlags[$entryCount],
											 $newsPostedFlags[$entryCount],
											 $newsDiscardedFlags[$entryCount]);
				}
			}
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


1;

#   $Log: digestNymMailNewsInfo.pl,v $
#   Revision 1.6  1999/12/03 22:00:11  filou
#   Added line of interest descriptions in comments
#
#   Revision 1.5  1999/12/01 01:36:03  filou
#   Added logError function calls
#
#   Revision 1.4  1999/11/30 23:46:20  filou
#   Added FDLOG file descriptor to print calls for error messages, removed unused parameters
#
#   Revision 1.3  1999/11/30 21:58:44  filou
#   added new parameters
#
#   Revision 1.2  1999/11/30 03:13:25  filou
#   Added persistency, plus code cleanup
#
#   Revision 1.1  1999/11/29 21:02:34  filou
#   Created for Nym Vitals C reporting
#   # Limitation : will only mark one failed post for a single msg body, even if nym was crossposting.
#   # Limitation : on quota exceeded, will only mark one failed mail, even if there were multiple recipients.
#   Messages blocked by recipient, if recipient is a nym, count for both sender (if a nym)
#   and receiver.
#
#
