#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestNymNewsActivity.pl
#
#   Author(s):		Philippe Desaulniers
#
#   Created:		 29 Nov 1999
#
#   Last Revision:	$Date: 1999/12/03 22:00:01 $
#
#   Description:	
#
#   $Revision: 1.5 $
#
#
#   $Id: digestNymNewsActivity.pl,v 1.5 1999/12/03 22:00:01 filou Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;

my $version = '$Id: digestNymNewsActivity.pl,v 1.5 1999/12/03 22:00:01 filou Exp $';
my $revision= '$Revision: 1.5 $';
my $progname= 'digestNymMailNewsInfo.pl';


select(STDERR); $| = 1;
select(STDOUT); $| = 1;


#################################################################################
#
# returns version info
#
sub getDigestNymNewsActivityInfo { return "$progname:$version:$revision";}


################################################################################
#
# 1999/12/01-20:14:04.0 fs3 17719 frpostnews "Posting article "test with space agent" to microsoft.public.test from space_agent@freedom.net successful"
#
#  Nym to newsgroup(s). Multiple newsgroups are possible, they will be comma separated.
#
#
sub processOneNymNewsActivity {
  my ($logMsg, $numLogs, $timeLogs, $froms, $tos, $debug)= @_;
	my $returnValue = 1;

  my $logtime;
	my $host;
	my $pid;
	my $service;
	my @msg;

	my $from;
	my $sNym;

	my @ngs;
	my $ngstring;
	my $ng;

  ($logtime, $host, $pid, $service, @msg) = splitLogLine($logMsg, $debug);
  my $message = join(" ", @msg);

	$$numLogs = 0;

  if($debug){print "processOneNymNewsActivity: $message\n";}
  if($message =~ /\"Posting article .* to (.*) from (.*) successful\"$/){
    if($debug){print "processOneNymNewsActivity: start to parse\n";}

		$ngstring = $1;
		$from = $2;

		$from =~ /(.*)\@.*/;
		$sNym = $1;

		@ngs = split(/,/, $ngstring);

		$$numLogs = 0;

		foreach $ng (@ngs) {
			if($debug){print "processOneNymNewsActivity: found posted newsgroup ", $ng," for ", $sNym, "\n";}
			$$timeLogs[$$numLogs] = $logtime;	
			$$froms[$$numLogs] = $sNym;	
			$$tos[$$numLogs] = $ng;	
			$$numLogs = $$numLogs + 1;
		}
  }

	else {
		logError ("processOneNymNewsActivity: received unprocessable message ", $message);
		$returnValue = 0;
	}

	return ($returnValue);
}
  
###############################################################################
#
# 1999/12/01-20:14:04.0 fs3 17719 frpostnews "Posting article "test with space agent" to microsoft.public.test from space_agent@freedom.net successful"
#
#
sub processNymNewsActivity {
    my($infile, $persist, $reportType, $debug) = @_;

    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

# Get and validate persistency information

    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processNymNewsActivity: Couldn't get persistency information.");
        exit (1);
    }

    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processNymNewsActivity: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processNymNewsActivity: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readLogFileLine();

        if ($line ne $lineCheck) {
            logError ("processNymNewsActivity: Sanity Check on next line failed!");
            exit 1;
        }

# Reset curpos to first unprocessed line.
        $curpos = tell(FDIN);

        close(FDIN);
    }

# Process from the curpos to the end of file

    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($timeSql) ="";
    my($logMsg)="";

# get partial log file and keep the byte offset for fastest access
    $curpos = getRestOfLogFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

# grep for interesting lines and update

    @linesOfInterest = grep /\"Posting article .* to .* from .* successful\"$/,
                            @partialTmp;

    foreach $logMsg (@linesOfInterest) {
			# set defaults

			my $numLogs = 0;
			my @timeLogs = ();
			my @froms = ();
			my @tos = ();

			$writeThis = processOneNymNewsActivity($logMsg,
																						 \$numLogs, 
																						 \@timeLogs, 
																						 \@froms, 
																						 \@tos, 
																						 $debug);
			if ( $writeThis ) {
#Insert sanity check output
				
				my ($entryCount);

				for ($entryCount = 0; $entryCount < $numLogs; $entryCount = $entryCount + 1) {
					outputRecord(convertTimeToSQLTime($timeLogs[$entryCount]),
											 $froms[$entryCount],
											 $tos[$entryCount]);
				}
			}
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


1;

#   $Log: digestNymNewsActivity.pl,v $
#   Revision 1.5  1999/12/03 22:00:01  filou
#   Added line of interest descriptions in comments
#
#   Revision 1.4  1999/12/01 01:36:13  filou
#   Added logError function calls
#
#   Revision 1.3  1999/11/30 23:46:43  filou
#   Added FDLOG file descriptor to print calls for error messages, removed unused parameters
#
#   Revision 1.2  1999/11/30 21:58:56  filou
#   added new parameters
#
#   Revision 1.1  1999/11/30 03:15:02  filou
#   Creation, used for Nym Vitals 1.5 report
#
#
#
