#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestNymStates.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 3 Nov 1999
#
#   Last Revision:	$Date: 1999/11/29 17:14:17 $
#
#   Description:	
#
#   $Revision: 1.3 $
#
#
#   $Id: digestNymStates.pl,v 1.3 1999/11/29 17:14:17 alext Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: digestNymStates.pl,v 1.3 1999/11/29 17:14:17 alext Exp $';
my $revision= '$Revision: 1.3 $';
my $progname= 'digestNymStates.pl';

select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# returns version info
#
sub getDigestNymStatesInfo { return "$progname:$version:$revision";}


#################################################################################
#
# Log messages found in sngen.c :
#
#
# Message related:
#
# 1999/11/04-20:55:24.0 cabbage.test.freedom.net 8830 frnscpd_dbck "total 155, errors 0: nuked 0, expired 91, inactive 0, disabled 0, active 64"
#
#
sub processOneNymStates {
  my($line, $time, $totalNyms, $errorNyms, $nukeNyms, $expiredNyms, $inactiveNyms, $disableNyms, $activeNyms, $debug) = @_;
  my $oneMessageProcessed = 0;
  my($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);

  my $message = join(" ", @msg);
  if($debug){print "processOneNymStates: $message\n";}
  if($message =~ /^\"total\s+/){
    if($debug){print "processOneNymStates: start to parse\n";}
    $message =~ /^\"total\s+([0-9]+), errors\s+([0-9]+):\s+nuked\s+([0-9]+),\s+expired\s+([0-9]+),\s+inactive\s+([0-9]+),\s+disabled\s+([0-9]+),\s+active\s+([0-9]+)\"$/;
    ($$totalNyms, $$errorNyms, $$nukeNyms, $$expiredNyms, $$inactiveNyms, $$disableNyms, $$activeNyms)=($1,$2,$3,$4,$5,$6,$7);
    if($debug) {
      print "processOneNymStates: ($1,$2,$3,$4,$5,$6,$7)\n";
      print "processOneNymStates: ($$totalNyms, $$errorNyms, $$nukeNyms, $$expiredNyms, $$inactiveNyms, $$disableNyms, $$activeNyms)\n";
    }
    $oneMessageProcessed = 1;
  }else{
    $oneMessageProcessed = 0;
  }
  return $oneMessageProcessed;
}

#################################################################################
#
# Log messages found in sngen.c :
#
#
#
sub processNymStates { 
  my($infile, $timeStamp, $timePeriod, $reportType, $debug) = @_;
  
  my($time, $totalNyms, $errorNyms, $nukeNyms, $expiredNyms, $inactiveNyms, $disableNyms, $activeNyms)=(0,0,0,0,0,0,0);

  # get first line if $time == 0
  if(! $timeStamp){
    my $line = `head -n 1 $$infile`;
    my($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
    if($debug){
      print "\t->$logtime\n\t->$host\n\t->$pid\n\t->$service\n\t->", join(" ", @msg),"\n";
    }
    $time = $logtime;
  }else{
    $time = $timeStamp;
  }

  my(@partialTmp)   =();
  my(@linesOfInterrest)   =();
  my($bytesOffset)  = 0;
  my($FlagComplete) = 1;
  my($flagEndProcess) = 1;
  my($timeSql) ="";
  my($logMsg)="";

  while ($FlagComplete){  
    @partialTmp = ();
    @linesOfInterrest = ();

    ## get partial log file and keep the byte offset for fastest access
    $bytesOffset = getPartialLogFile($infile, $time, $timePeriod, 
				     \@partialTmp, $bytesOffset, $debug);

    @linesOfInterrest = grep /\"total\s+[0-9]+,\s+errors\s+[0-9]+:/, @partialTmp;

    last if($bytesOffset < 0) ; # stops
    if($debug){print join("<-\n", @linesOfInterrest),"\n";}
    
    foreach $logMsg (@linesOfInterrest){
      $flagEndProcess = processOneNymStates($logMsg, \$time, \$totalNyms, \$errorNyms, \$nukeNyms, \$expiredNyms, 
			  \$inactiveNyms, \$disableNyms, \$activeNyms, $debug);
      if($flagEndProcess ){
	if($errorNyms){
	  print STDERR
	    "***Errors: Problem with DATABASE ($errorNyms errors) Need assistance\n";
	}
      SWITCH: {
	  if($reportType =~ /xml/) {
	    print 
	      "<nymStatesInfo>\n",
	      "   <time>$time</time>\n",
	      "   <dbTotalNyms>$totalNyms</totalNyms>\n",
	      "   <dbErrorNyms>$errorNyms</errorNyms>\n",
	      "   <activeNyms>$activeNyms</activeNyms>\n",
	      "   <inactiveNyms>$inactiveNyms</inactiveNyms>\n",
	      "   <disabledNyms>$disableNyms</dormantNyms>\n",
	      "   <expiredNyms>$expiredNyms</expiredNyms>\n",
	      "   <nukedNyms>$nukeNyms</nukedNyms>\n",
	      "</nymStatesInfo>\n",
	      "\n";
	    last SWITCH;
	  }
	  if($reportType =~ /line/) {
	    # 1999/11/03-00:28:49.0 -> 1999-11-03 00:28:49
	    $timeSql = $time;
	    $timeSql =~ s/\.[0-9]$//;
	    $timeSql =~ s/-/ /gi;
	    $timeSql =~ s/\//-/gi;
	    print "$timeSql;$totalNyms;$errorNyms;$activeNyms;$inactiveNyms;$disableNyms;$expiredNyms;$nukeNyms\n";
	    last SWITCH;
	  }

	  print 
	    "Report Date  : $time\n",
	    "Db Total Recs: $totalNyms\n",
	    "Db Error Recs: $errorNyms\n",
	    "Nym active   : $activeNyms\n",
	    "Nym inactive : $inactiveNyms\n",
	    "Nym disabled : $disableNyms\n",
	    "Nym expired  : $expiredNyms\n",
	    "Nym nuked    : $nukeNyms\n\n";
	}
      }
      ($totalNyms, $errorNyms, $nukeNyms, $expiredNyms, $inactiveNyms, $disableNyms, $activeNyms)=(0,0,0,0,0,0);
    }

    last if($timeStamp);

    $time = addOffsetToTimeString($time, $timePeriod, 0);
    
    if($bytesOffset < 0){
      $FlagComplete = 0;
    }    
  }
}


1;

#   $Log: digestNymStates.pl,v $
#   Revision 1.3  1999/11/29 17:14:17  alext
#   New scripts for changes in nym state.
#
#   Revision 1.2  1999/11/05 07:21:10  thierry
#   Bug# 1187
#        add support to report token counts per types
#        over period of time
#        [peer: Luc]
#
#   Revision 1.1  1999/11/05 05:16:36  thierry
#   	Bug #1185
#   	Add NymStates support
#
