#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestNymSummary.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		20 Jan 2000
#
#   Last Revision:	$Date: 2000/06/20 11:41:18 $
#
#   Description:	
#
#   $Revision: 1.4 $
#
#
#   $Id: digestNymSummary.pl,v 1.4 2000/06/20 11:41:18 thierry Exp $	
#
#   Copyright (c) 2000 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Getopt::Long;
use Sys::Hostname;

my $version = '$Id: digestNymSummary.pl,v 1.4 2000/06/20 11:41:18 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'digestNymSummary.pl';

my $DEBUG=0;

## unbuffered stdin/stdout
select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# argument parsing and checking
#
my %optctl  = ();
my @helpString = 
  (
   "\n\n",
   "Usage $progname",
   "\t--debug                 : debug mode on"
  );

my $res = GetOptions(\%optctl,
+		     "version",
		     "debug") || die "Error in Getopt";


if(defined($optctl{'debug'})){
  $DEBUG = 1;
}

if(defined($optctl{'version'})){
  print getDigestSnSummaryInfo(),"\n";
  exit 0;
}

#################################################################################
#
# directories
#
#my $HOME         = "/turf/thierry/fradmin";
my $HOME         = "/usr/fradmin";

my $PL_PATH      = $HOME . "/lib";
my $DAT_PATH     = $HOME . "/var/dat";
my $PERSIST_PATH = $HOME . "/var/persist";

my $freedomDbDir = "/freedom/etc/db/nymsdb.remote.conf";
#my $freedomDbDir = $HOME . "/../db";

#################################################################################
#
# File
#
my $destFile = "nym_summary";

#################################################################################
#
# Configuration parameters for zktsa
#
if($DEBUG){print "<$freedomDbDir>\n";}
my $myPid=$$;
#my $getNymInfo = "/usr/freedom/sbin/frnscpd_dbck --maxerrors 10000 --logfile /tmp/dbcheck.$$ --databasehome $freedomDbDir > /dev/null;/usr/bin/tail -n 1 /tmp/dbcheck.$$ ";
my $getNymInfo = "/usr/freedom/sbin/frnscpd_dbck --maxerrors 10000 --logfile /tmp/dbcheck.$myPid --databasename $freedomDbDir > /dev/null;/usr/bin/tail -n 1 /tmp/dbcheck.$myPid ";

#################################################################################
#
# Error
#
sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}


#################################################################################
#
# returns version info
#
sub getDigestNymSummaryInfo { return "$progname:$version:$revision";}

#################################################################################
#
# convert a string of the form  "X days, Y hours, Z minutes, W secs"
# to the corresponding time in seconds.
sub ConvertTextTimeToSeconds {
  my $textTime = $_[0];
  my $NA = "N/A";

  my $days    = 0;
  my $hours   = 0;
  my $minutes = 0;
  my $secs    = 0;

  my $timeInSeconds;

  
  if ($textTime eq $NA) {
    return $NA;
  }  

  if ($textTime =~ /(\d+) days.*/) {
    $days = $1;
  }
  if($textTime =~ /.*(\d+) hours.*/) {
    $hours = $1;
  } 
  if($textTime =~ /.*(\d+) minutes.*/) {
    $minutes = $1;
  } 
  if($textTime =~ /.*(\d+) secs/) {
    $secs = $1;
  }
  else { 
    ## must at least contains seconds
    die "ConvertTextTimeToSeconds: invalid time string [$textTime]\n"; 
  }

  $timeInSeconds = $secs + $minutes*60 + $hours*60*60 + $days*24*60*60;

  return $timeInSeconds;
}

#################################################################################
#
# convert time in seconds since january 1, 1970, UTC, to 
# format YYYY-MM-DD hh:mm:ss  for SQL.
sub ConvertSecsToSqlDate {
  my $seconds = $_[0];
  my $sqlDate;

  my $sec;
  my $min; 
  my $hour; 
  my $mday; 
  my $mon;
  my $year;
  my $wday;
  my $yday; 
  my $isdst;

  ##
  ## localtime returns list.
  ##   
  ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($seconds);
  
  $mon += 1;
  $year += 1900;


  $sqlDate = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year, $mon, $mday, $hour, $min, $sec);

  return $sqlDate;
}

#################################################################################
#
# Get Current status from the nym database
#
# output of the command should provide the following
#
#
sub getCurrentNymSummary {

  if($DEBUG){print "$progname: getCurrentNymSummary(start)\n"};
  my %tmpHashList=();
  my ($tmpStr, $total, $errors, $nuked, $expired, $inactive, $disabled, $active);

  ##
  ## get the count from frnscpd_dbck
  ##
  my $tmpRes = `$getNymInfo`;
  chomp($tmpRes);
  if($DEBUG){print "<$tmpRes>\n";}
  $tmpRes =~ s/.*frnscpd_dbck "//;
  $tmpRes =~ s/"$//;
  if($DEBUG){print "<$tmpRes>\n";}

  $tmpRes =~ /total\s+(\d+),\s+errors\s+(\d+):\s+nuked\s+(\d+),\s+expired\s+(\d+),\s+inactive\s+(\d+),\s+disabled\s+(\d+),\s+active\s+(\d+)/;

  ##
  ## "total 16429, errors 0: nuked 6, expired 4119, inactive 0, disabled 103, active 12201"
  ## ($nuked, $total, $expired, $inactive, $disabled, $active) = ($1, $2, $3, $4, $5, $6);
  ##  x      x        x         x         x           x
  ($total, $errors, $nuked, $expired, $inactive, $disabled, $active) = ($1, $2, $3, $4, $5, $6, $7);

  if($DEBUG){
    print 
      "Nym Summary:\n",
      "\tTotal   : $total\n",
      "\tError   : $errors\n",
      "\tNuked   : $nuked\n",
      "\tExpired : $expired\n",
      "\tInactive: $inactive\n",
      "\tDisabled: $disabled\n",
      "\tActive  : $active\n",
      "\n";
    print "$progname: getCurrentNymSummary(end)\n\n"
  };
  
  return ( $total,$active,$nuked,$expired,$inactive,$disabled);
}

#################################################################################
#
#  generate the output filename, based on entity type and actual date
#
sub getDatFilename {
  if($DEBUG){print "$progname: getDatFilename(start)\n";}
  my ($dateFile, $tmp) = split(" ", ConvertSecsToSqlDate(time));
  my($host) = hostname();
  $host =~ s/(\..*)+//g;
  my $filename = $DAT_PATH . "/" . $host . "_" . $destFile . "_" . $dateFile ."_00-00-00.dat"; 
  if($DEBUG){
    print "\tFilename is : $filename\n";
    print "$progname: getDatFilename(end)\n";
  }
  return $filename;
}

#################################################################################
#
# 
#
sub Main {
  my $filename = getDatFilename();
  if (! defined($filename) ) {
    ExitOnError("Dat file not set");
  }
  
  ## open file in append mode
  open OUTFILE, ">>$filename" or die "error opening file [$filename]: $!\n";
  
  ##
  ## Get current Values from dbchecker
  ##
  my @res = getCurrentNymSummary();

  my ($date, $time) = split(" ", ConvertSecsToSqlDate(time));
  print OUTFILE "\"$date-$time\";";
  print OUTFILE "\"", join("\";\"", @res),"\"\n";

  close(OUTFILE);
}


#################################################################################
#
# Testing to be removed
#

Main();
`rm /tmp/dbcheck.$myPid `;

1;

#   $Log: digestNymSummary.pl,v $
#   Revision 1.4  2000/06/20 11:41:18  thierry
#   sync with 1.1a
#
#   Revision 1.3  2000/03/27 15:01:38  thierry
#   modify to get a valid name for .dat file
#
#   Revision 1.2  2000/03/17 19:23:12  thierry
#   Update filename to contain hostname request for MIPAS
#
#   Revision 1.1  2000/01/21 21:30:14  thierry
#
#   	* provide repmon report 5.1 as described in
#      	  Freedom Reporting Requirement
#     	  (Dormant Nym are not in because no such status in V1)
#
#   	* provide cumulative raw data for futur SQL processing
#
#   	* call frnscpd_dbck once a day to get a count on
#   	  (total/active/expired/nuked) nyms present in the
#   	  database.
#
