#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:           digestRotatedLog.pl
#
#   Author(s):		Philippe Desaulniers
#
#   Created:		1 dec 1999 
#
#   Last Revision:	$Date: 2000/01/28 18:22:42 $
#
#   Description:	
#    This script will invoke the proper scripts to support a log rotate
#    on the various servers.
#    
#    Lots of hardcoded stuff, t'is going to be hard to maintain.
#
#   $Revision: 1.6 $
#
#
#   $Id: digestRotatedLog.pl,v 1.6 2000/01/28 18:22:42 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use English;
use Getopt::Long;
use Env;
select(STDOUT); $| = 1;

my $version = '$Id: digestRotatedLog.pl,v 1.6 2000/01/28 18:22:42 thierry Exp $';
my $revision= '$Revision: 1.6 $';
my $progname= 'digestRotatedLog.pl';

sub ExitOnError {
  my (@currentTime, $currentTimeString, $outputMessage);
  
  @currentTime = gmtime(time);
  
  $currentTime[5] = $currentTime[5] + 1900;
  
  $currentTime[4] = $currentTime[4] + 1;
  if ($currentTime[4] < 10) {
    $currentTime[4] = "0".$currentTime[4];
  } 
  
  $currentTime[3] = $currentTime[3];
  if ($currentTime[3] < 10) {
    $currentTime[3] = "0".$currentTime[3];
  } 
  
  $currentTimeString = $currentTime[5]."-".$currentTime[4]."-".$currentTime[3]." ".$currentTime[2].":".$currentTime[1].":".$currentTime[0];
  
  $outputMessage = join '', "\"", @_, "\"\n";
  
  print FDLOG $currentTimeString, " ", $outputMessage;
  close FDLOG;
  exit 1;
}

##
## argument parsing and checking
##

my $res;
my %optctl  = ();

my @helpString = 
  (
   "\n\n",
   "Usage $progname",
   "\t--entity     entityname : name of entity for which parsing is performed",
   "\t--version               : get version number",
   "\t--help                  : this help",
   "\t--debug                 : debug mode on"
  );

my ($logfile) =  $ENV{'HOME'}."/var/log/digestRotatedLog.log";

# Open log file
open FDLOG, ">>$logfile" || die "Could not open log file";

## Get the Options
##
$res = GetOptions(\%optctl, 
                  "help",
                  "debug",
                  "version",
                  "entity=s"
                 ) || ExitOnError "Error in Getopt";

## checks if arguments
##
if(  scalar keys %optctl <= 0){
  ExitOnError("No options provided");
}


## print out the usage string
##
if (defined($optctl{'help'})){
  print join("\n", @helpString),"\n";
  exit 0;
}

if(defined($optctl{'version'})){
  print STDERR 
    "$progname\n\t$revision\n",
    "\t$version\n";
  exit 0;
}

my($debug) = 0;
if(defined($optctl{'debug'})){
  $debug = 1;
}

my $entityName = "";

# Procees entity
if (defined($optctl{'entity'})) {
  $entityName = $optctl{'entity'};
  if ($debug) {print STDERR "Entity is ", $entityName, "\n"};
}

else {
  ExitOnError ("Entity isn't defined");
}

my $inputFile = "";
my @reportName = ();
my $reportCount = 0;

SWITCH_ENTITY: {
  ## entity zktsa
  if($entityName eq "zktsa") {

    $inputFile = "/freedom/var/log/zktsa.1";

    $reportCount = 1;
    $reportName[0] = "snminutes";

    last SWITCH_ENTITY;
  }

  ## entity frloded
  if($entityName eq "frloded") {

    $inputFile = "/freedom/var/log/frloded.audit.1";

    $reportCount = 1;
    $reportName[0] = "lodeaudit";

    last SWITCH_ENTITY;
  }
  
  ## entity frnscpd
  if($entityName eq "frnscpd") {

    $inputFile = "/freedom/var/log/frnscpd.1";

    $reportCount = 1;
    $reportName[0] = "frnscp";

    last SWITCH_ENTITY;
  }
  
  ## entity fmg-maip
  if($entityName eq "fmg-maip") {

    $inputFile = "/freedom/var/log/fmg-maip.1";

    $reportCount = 2;
    $reportName[0] = "nymmailnews";
    $reportName[1] = "nymmailactivity";

    last SWITCH_ENTITY;
  }
  
  ## entity frpostnews
  if($entityName eq "frpostnews") {

    $inputFile = "/freedom/var/log/frpostnews.1";

    $reportCount = 2;
    $reportName[0] = "nymmailnews";
    $reportName[1] = "nymnewsactivity";

    last SWITCH_ENTITY;
  }
  
  ## entity fmg
  if($entityName eq "fmg") {

    $inputFile = "/freedom/var/log/fmg.1";

    $reportCount = 1;
    $reportName[0] = "nymmailactivity";

    last SWITCH_ENTITY;
  }
  
  ExitOnError("Entity type ", $entityName, " invalid");
}

my $commandLine = "";
my $retValue = 0;

# Verify file existence (the -r option means "exist & is resdable")
$commandLine = "test -r ".$inputFile;
if ($debug) {print STDERR "Issuing command : \"", $commandLine, "\"\n"};
$retValue = system ($commandLine);

if ($retValue == 0) { # 0 will be returned if file exists
  my $currentReport;

  for ($currentReport = 0; $currentReport < $reportCount; $currentReport += 1) {
    $commandLine = "/usr/fradmin/lib/digestLog.pl --".$reportName[$currentReport]." --entity ".$entityName." --in ".$inputFile." --resetAfterParsing";
    if ($debug) {print STDERR "Issuing command : \"", $commandLine, "\"\n"};
    $retValue = system ($commandLine);
   
    if ($retValue != 256) { # 256 will be returned on success
      ExitOnError("Command \"", $commandLine, "\" failed.");
    }
  }
}

close FDLOG;
exit 0;

1;

#   $Log: digestRotatedLog.pl,v $
#   Revision 1.6  2000/01/28 18:22:42  thierry
#   	* add support for serial-number minutes log rotation
#   	  digestLog.pl --snminutes
#
#   Revision 1.5  1999/12/23 20:20:13  louis
#   - changed the default name of the rotated file from *.0 to *.1
#
#   Revision 1.4  1999/12/06 19:11:44  filou
#   Removed current day '+1' from erro log date
#
#   Revision 1.3  1999/12/06 18:51:31  filou
#   Added file existence verification, slight code cleanup
#
#   Revision 1.2  1999/12/01 21:09:54  filou
#   Changed bad SWITCH_ENTITY: tags
#
#   Revision 1.1  1999/12/01 19:58:03  filou
#   Creation
#
#
