#!/bin/sh
#
# script used to rotate the log files
#



#
# public settings - can be changed according to installation
#
homepath=/usr/fradmin


#
# private settings - must not be changed
#
progname=digestRotatedLog
libpath=${homepath}/lib
logfile=${homepath}/var/log/${progname}.log


#
# NOTE: these must match the other cron lock file names
#
lockday=${homepath}/var/run/cron_day.lock
lockhour=${homepath}/var/run/cron_hour.lock
lockmin=${homepath}/var/run/cron_min.lock



exec >> ${logfile} 2>&1

#
# get lock on all other cron jobs
#


# try for 30 minutes
try=1800

# get the cron day lock
delay=60
while ! ${libpath}/lock.pl ${lockday}
do
  try=`expr $try - $delay`
  sleep $delay
  if [ $try -le 0 ]
  then
    echo "${progname}: couldn't get lock on ${lockday}"
    exit 1
  fi
done 

# get the cron hour lock
delay=60
while ! ${libpath}/lock.pl ${lockhour}
do
  try=`expr $try - $delay`
  sleep $delay
  if [ $try -le 0 ]
  then
    # remove other lock
    rm ${lockday}

    echo "${progname}: couldn't get lock on ${lockhour}"
    exit 1
  fi
done 

# get the cron min lock
delay=20
while ! ${libpath}/lock.pl ${lockmin}
do
  try=`expr $try - $delay`
  sleep $delay
  if [ $try -le 0 ]
  then
    # remove other lock
    rm ${lockday}
    rm ${lockhour}

    echo "${progname}: couldn't get lock on ${lockmin}"
    exit 1
  fi
done 


#
# if this point is reached then we got the three locks
#
echo "----------------------------------------"
date
    
${libpath}/digestRotatedLog.pl $*


# release locks
rm ${lockday}
rm ${lockhour}
rm ${lockmin}

exit 0

