#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestServopTrafficSumm.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		30 Nov 1999
#
#   Last Revision:	$Date: 2000/03/17 19:23:12 $
#
#   Description:	
#
#   $Revision: 1.4 $
#
#
#   $Id: digestServopTrafficSumm.pl,v 1.4 2000/03/17 19:23:12 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use FileHandle;
use Getopt::Long;
use Sys::Hostname;

my $version = '$Id: digestServopTrafficSumm.pl,v 1.4 2000/03/17 19:23:12 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'digestServopTrafficSumm.pl';

sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}

##
## autoflush stdout stderr
##
STDOUT->autoflush();
STDERR->autoflush();

##
## argument parsing and checking
##
my %optctl  = ();
my @helpString = 
    (
     "\n\n",
     "Usage $progname",
     "\t--help                  : this help",
     "\t--remote                : remote access to the data (frlodec)",
     "\t--hostname              : host for remote access"
     );

## Get the Options
##
my($res);
$res = GetOptions(\%optctl, 
                  "help",
                  "remote",
                  "hostname=s"
                 ) || die "Error in GetOpt";

## print out the usage string
##
if (defined($optctl{'help'})){
    print join("\n", @helpString),"\n";
    exit 1;
}

if(defined($optctl{'remote'}) && ! defined($optctl{'hostname'})){
   ExitOnError("***Error: You should provide an Hostname with remote option",
	       @helpString
	      );
   exit 1;
}

##
## Verify access to directories
##
##
my $contextBaseDir = "/usr/fradmin";
if(! -d "$contextBaseDir/var"){
  print STDERR "$contextBaseDir/var is not a valid directory\n";
  exit 1;
}

if(! -d "$contextBaseDir/var/log"){
  print STDERR "$contextBaseDir/var/log is not a valid directory\n";
  exit 1;
}

if(! -d "$contextBaseDir/var/dat"){
  print STDERR "$contextBaseDir/var/dat is not a valid directory\n";
  exit 1;
}

##
## Create filenames for the logging functionality
##
##
my $logFileName = $progname;
$logFileName =~ s/^\.\///gi;
$logFileName =~ s/\.pl$/\.log/gi;

my($host) = hostname();
$host =~ s/(\..*)+//g;
my $sqlFileName = $host . "_aip_traffic";
$sqlFileName =~ s/^\.\///gi;
$sqlFileName =~ s/\.pl$//gi;


my $contextVarDir  = $contextBaseDir . "/var";
my $contextLogFile = $contextBaseDir . "/var/log/$logFileName";
my $sqlLogFile = $contextBaseDir . "/var/dat/$sqlFileName";

##
## parameters setup
##
##

my $REMOTE;
my $tmp;
my $lodeClientCommand;
my $lodeClientArgs;

if(defined($optctl{'remote'})){
  # command to call
  # frlodec --hostname fs2.freedom.net --user toto --service nidb --operation servopreliability
  #
  $lodeClientCommand = "/usr/freedom/test/frlodec"; #"frlodec";
  if( ! -f $lodeClientCommand){
    $tmp = `which $lodeClientCommand`;
    chomp($tmp);
    if($tmp =~ /which:\sno\s$lodeClientCommand/){
      ExitOnError("cannot find $lodeClientCommand : \n<$tmp>\n");
    }
  }
  $lodeClientArgs = " --hostname $optctl{'hostname'} --user toto --service netinfo --operation traffic";
}else{
  # command to call
  # frgetcustomersupport.pl --servopreliability
  #
  if(-f "/freedom/sbin/frgetcustomersupport.pl"){
    $lodeClientCommand = "/freedom/sbin/frgetcustomersupport.pl";
    $lodeClientArgs    = " --servoptraffic";
  }else{
    ExitOnError("***Error cannot find : $lodeClientCommand\n");
  }
}


## convert time in seconds since january 1, 1970, UTC, to 
## format YYYY-MM-DD hh:mm:ss  for SQL.
sub ConvertSecsToSqlDate {
  my $seconds = $_[0];
  my $sqlDate;

  my $sec;
  my $min; 
  my $hour; 
  my $mday; 
  my $mon;
  my $year;
  my $wday;
  my $yday; 
  my $isdst;

  ##
  ## localtime returns list.
  ##   
  ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($seconds);
  
  $mon += 1;
  $year += 1900;
  $wday += 1;

  $sqlDate = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year, $mon, $mday, $hour, $min, $sec);

  return $sqlDate;
}


##
## error logging
##
sub logToFile {
  my @msg = @_;

  my $timeInSec = time();
  my $sqlTime = ConvertSecsToSqlDate($timeInSec);
  $sqlTime =~ s/-/\//g;
  $sqlTime =~ s/ /_/;
  my $msg = join(" ",  @msg);
  open (FDLOG,">>$contextLogFile") || die "cannot open $contextLogFile\n";
  print FDLOG sprintf("%s %s %s %s \"%s\"\n", $sqlTime, "-", $$, $progname, $msg);
  close(FDLOG);
}

##
## result logging
##
sub logToSqlFile {
  my $msg = $_[0];

  my $timeInSec = time();
  my $sqlTime = ConvertSecsToSqlDate($timeInSec);
  $sqlTime =~ s/\s+.*/_00-00-00/;

  my $file = "$sqlLogFile"."_$sqlTime.dat";
#  print "<$sqlTime: $file>\n";

  open (FDLOG,">>$file");
  print FDLOG "$msg\n";
  close(FDLOG);
}



##
## file logging
##

## Get time for the command
##
my $timeInSec = time();
my $sqlTimeForData = ConvertSecsToSqlDate($timeInSec);

## Get the data set
##
$tmp = `$lodeClientCommand  $lodeClientArgs`;
#print "$lodeClientCommand  $lodeClientArgs\n";
#print "[$tmp]\n";
if(! $tmp){
   ExitOnError("***Error: Not able to gather data from remote site\n");
}

$tmp =~ s/<servopTrafficSumm>\n//gi;
$tmp =~ s/<\/servopTrafficSumm>\n//gi;
$tmp =~ s/<trafficRep>\n//gi;
$tmp =~ s/<\/trafficRep>\n//gi;

##
## free the data set
my $index = 0;
my @tmpArray=();
my ($entity, $KbpsIn, $KbpsOut);

##
## Process data input
##
my @array = split("\n\n", $tmp);
foreach $res (@array){
  ##
  ## 
  #print "\n$index: $res\n";

  @tmpArray=();
  @tmpArray = split("\n", $res);

  ##
  ## Get the entity name
  $entity = shift @tmpArray;
  if($entity =~  /^<entityName>/i){
    $entity =~ s/<\/*entityName>//g;

    ##
    ## Get KbpsIn
    $KbpsIn = shift @tmpArray;
    $KbpsIn =~ s/<\/*currentKbpsIn>//g;

    ##
    ## Get KbpsOut
    $KbpsOut = shift @tmpArray;
    $KbpsOut =~ s/<\/*currentKbpsOut>//g;

    #print sprintf("\"%s\";\"%s\";\"%s\"\n", $entity, $KbpsIn, $KbpsOut);
    logToSqlFile(sprintf("\"%s\";\"%s\";\"%s\";\"%s\"", $sqlTimeForData, $entity, $KbpsIn, $KbpsOut));

  }else{
    logToFile($entity);
  }

  $index++;
}

1;

#   $Log: digestServopTrafficSumm.pl,v $
#   Revision 1.4  2000/03/17 19:23:12  thierry
#   Update filename to contain hostname request for MIPAS
#
#   Revision 1.3  1999/12/02 22:15:45  thierry
#   	Fix base name to be $progname not $0
#
#   Revision 1.2  1999/12/01 06:23:00  louis
#   - cahanged the hardcode path for the call to frgetcustomersupport.pl
#
#   Revision 1.1  1999/11/30 23:55:26  thierry
#
#   	add support to get an hourly report for trafficSummary
#
