#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestSnDistrib.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		24 Oct 1999
#
#   Last Revision:	$Date: 1999/11/05 04:53:14 $
#
#   Description:	
#
#   $Revision: 1.4 $
#
#
#   $Id: digestSnDistrib.pl,v 1.4 1999/11/05 04:53:14 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: digestSnDistrib.pl,v 1.4 1999/11/05 04:53:14 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'digestSnDistrib.pl';

#################################################################################
#
# returns version info
#
sub getDigestSnDistribInfo { return "$progname:$version:$revision";}

#################################################################################
#
# Get Redeemed Count
#
sub getRedeemedSN { 
  my($partialTmp) = @_;
  my(@res)=();
  # process outgoing message
  @res = grep /\"Serial number redeemed /, @$partialTmp;

  return scalar @res;
}


#################################################################################
#
# Log messages found in snap.c
#
#
# Message related:
#
# zkLogWrite_va(context.log,"User %s is generating %d serial numbers of type %d",
#
sub processSnDistribLog {
  my($infile, $timeStamp, $timePeriod, $reportType, $debug) = @_;
  my($time, $host, $pid, $service, @msg);

  ## for now process the whole file
  #$timeStamp = 0;

  #
  # get first line if time stamp = 0
  #
  my($line);
  if(! $timeStamp){
    $line = `head -n 1 $$infile`;
    ($time, $host, $pid, $service, @msg) = splitLogLine($line, , $debug);
    if($debug){
      print "\t$time\n\t$host\n\t$pid\n\t$service\n\t", join(" ", @msg),"\n";
    }
  }else{
    $time = $timeStamp;
  }

  my(@partialTmp)   =();
  my($bytesOffset)  = 0;
  my($FlagComplete) = 1;
  my($redeemedSN) = 0;
  
  my(%redeemedSn)=();
  my($snType, $type, $key, $timelog);
  my($msg);

  ##
  ## Process all the file
  ##
#   if(! $timeStamp) {		# process the whole file
#     %redeemedSn=();
#     print "***Warning:  processing the full file for now\n";
#     my($res) = open(FDIN, "<$$infile");
#     my($counter)=0;
#     while (defined($line = <FDIN> ) && !(eof FDIN)){
#       ## get the data
#       ($timelog, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
#       $msg = join(" ", @msg);
#       if($msg =~ /^\"Serial number redeemed /) {
# 	$msg =~ /^\"Serial\s+number\s+redeemed\s+.*,\s*type\s*=\s*(.*)\s*\"/;
# 	$snType = $1;
# 	if($debug){print "extract $msg: $snType\n";}

# 	if(! defined($redeemedSn{$snType})){
# 	  if($debug){print "Create counter for type: $snType\n";}
# 	  $redeemedSn{$snType} = 0;
# 	}
# 	$redeemedSn{$snType}++;
#       }
#       $counter++;
#     }
#     close(FDIN);
#     if($debug){print "$progname: process ", $counter+1," Lines\n";}

#     $timePeriod = "all log"; 
#   SWITCH: {
#       if($reportType =~ /xml/) {
# 	print  
# 	  "<SerialNumberDistrib>\n",
# 	  "   <timeStamp>$time</timeStamp>\n",
# 	  "   <tmPeriod>$timePeriod</tmPeriod>\n";
	
# 	foreach $key (keys %redeemedSn){
# 	  print "    <typeSn> $key</typeSn> <countSn>$redeemedSn{$key} </countSn>\n";
# 	}
	
# 	print
# 	  "</SerialNumberDistrib>\n",
# 	  "\n";
# 	last SWITCH;
#       }
      
#       if($reportType =~ /line/) {
# 	foreach $key (keys %redeemedSn){
# 	  print 
# 	    $time,";",
# 	    $timePeriod,";",
# 	    $key,";",
# 	    $redeemedSn{$key},
# 	    "\n";
# 	}
# 	last SWITCH;
#       }
      
#       print  "Time  start             : ", $time,"\n";
#       print  "Time  period            : ", $timePeriod,"\n";
#       foreach $key (keys %redeemedSn){
# 	printf("Redeemed SN type %-5.5s : %10d\n", $key, $redeemedSn{$key});
#       }
#       print  "\n";
#     }
#   }else{
    ##
    ## from start with a period of hh:mm:ss
    ##
    
    while ($FlagComplete){ 
      @partialTmp = ();
      %redeemedSn = ();
      
      ## get partial log file and keep the byte offset for fastest access
      $bytesOffset = getPartialLogFile($infile, $time, $timePeriod, 
				     \@partialTmp, $bytesOffset, $debug);

##      last if($bytesOffset < 0) ; # stops 
      if($debug){print join("\n", @partialTmp),"\n";}

#      $redeemedSN = getRedeemedSN(\@partialTmp);
      foreach $line (@partialTmp){
	## get the data
	($timelog, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
	$msg = join(" ", @msg);
	if($msg =~ /^\"Serial number redeemed /) {
	  $msg =~ /^\"Serial\s+number\s+redeemed\s+.*,\s*type\s*=\s*(.*)\s*\"/;
	  $snType = $1;
	  if($debug){print "extract $msg: $snType\n";}

	  if(! defined($redeemedSn{$snType})){
	    if($debug){print "Create counter for type: $snType\n";}
	    $redeemedSn{$snType} = 0;
	  }
	  $redeemedSn{$snType}++;
	}
      }

#   if(scalar %redeemedSn){
	## here prin the report
	## get the printout done
    SWITCH: {
	if($reportType =~ /xml/) {
	  print  
	    "<SerialNumberDistrib>\n",
	    "   <timeStamp>$time</timeStamp>\n",
	    "   <tmPeriod>$timePeriod</tmPeriod>\n";
	  
	  foreach $key (keys %redeemedSn){
	    print "    <typeSn> $key</typeSn> <countSn>$redeemedSn{$key} </countSn>\n";
	  }
	  
	  print
	    "</SerialNumberDistrib>\n",
	    "\n";
	  last SWITCH;
	}
	
	if($reportType =~ /line/) {
	  foreach $key (keys %redeemedSn){
	    print 
	      $time,";",
	      $timePeriod,";",
	      $key,";",
	      $redeemedSn{$key},
	      "\n";
	  }
	  last SWITCH;
	}
	print  "Time  start             : ", $time,"\n";
	print  "Time  period            : ", $timePeriod,"\n";
	foreach $key (keys %redeemedSn){
	  printf("Redeemed SN type %-5.5s : %10d\n", $key, $redeemedSn{$key});
	}
	print  "\n";
      }
#    }
      
      last if($timeStamp);

      $time = addOffsetToTimeString($time, $timePeriod, 0);
      if($debug){
	print  "Time  end         : ", $time,"\n";
	print "\n";
      }

      if($bytesOffset < 0){
	$FlagComplete = 0;
      }
    }
#  }
  return;
}


1;

#   $Log: digestSnDistrib.pl,v $
#   Revision 1.4  1999/11/05 04:53:14  thierry
#   force report with 0 fields (two lines commented)
#
#   Revision 1.3  1999/10/26 17:44:07  thierry
#   add support to report multiple type serial numbers
#
#   Revision 1.2  1999/10/25 15:44:54  thierry
#   fix bug in getPartialLogFile
#
#   Revision 1.1  1999/10/24 22:15:31  thierry
#   add Serial Number Redeemed support
#
