#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestSnMinutes.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		24 Jan 2000
#
#   Last Revision:	$Date: 2000/01/25 16:40:01 $
#
#   Description:	
#
#   $Revision: 1.1 $
#
#
#   $Id: digestSnMinutes.pl,v 1.1 2000/01/25 16:40:01 thierry Exp $	
#
#   Copyright (c) 2000 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use DB_File;

my $version = '$Id: digestSnMinutes.pl,v 1.1 2000/01/25 16:40:01 thierry Exp $';
my $revision= '$Revision: 1.1 $';
my $progname= 'digestSnMinutes.pl';

select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# returns version info
#
sub getDigestSnMinutesInfo { return "$progname:$version:$revision";}

#################################################################################
#
# convert time in seconds since january 1, 1970, UTC, to 
# format YYYY-MM-DD hh:mm:ss  for SQL.
sub ConvertSecsToSqlDate {
  my $seconds = $_[0];
  my $sqlDate;

  my $sec;
  my $min; 
  my $hour; 
  my $mday; 
  my $mon;
  my $year;
  my $wday;
  my $yday; 
  my $isdst;

  ##
  ## localtime returns list.
  ##   
  ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($seconds);
  
  $mon += 1;
  $year += 1900;


  $sqlDate = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year, $mon, $mday, $hour, $min, $sec);

  return $sqlDate;
}

################################################################################
#
# Generate results from a single line.  Returns 0 on failure, 1 on success.  If
# successful, the result should be output.
# message is 
#   "zktsa import 8000 serials numbers type 500:  
#   created by georgia@sngen.zks.net: 
#   each sn creates 5 type 30 tokens:  
#   activated on Sat Nov 27 00:00:00 1999"

sub processOneSnMinutes {
    my ($line,
        $time,
	$snCount, 
	$snType,
	$creator,
	$tokenCount, 
	$tokenType,
	$activationDate,
        $debug) = @_;

    if( $line =~ /^\"/ ){
	return 0;
    }
    my ($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);

    $$time = $logtime;

    my $message = join(" ", @msg);
    $message =~ s/\s+/ /gi;
    $message =~ s/(\d+):(\d+):(\d+)/$1-$2-$3/g;

    # Nym creation or upgrade message
    my $tmpPartial;
    if ($message =~ /zktsa import (\d+) serials /){
	if ($debug) {print "processOneSnMinutes: $message\n";}
	my @tmp = split(/:/, $message);

	## process first part of the message
	## zktsa import 8000 serials numbers type 500
	##
	$tmpPartial = shift(@tmp);
	$tmpPartial =~ s/\D+\s*/:/gi;
	$tmpPartial =~ s/^://;
	($$snCount, $$snType) = split(":", $tmpPartial);
	if($debug){print "\t=>  ($$snCount, $$snType)\n";}

	## process second part of the message
	## created by georgia@sngen.zks.net
	##
	$tmpPartial = shift(@tmp);
	$tmpPartial =~ s/^created\s+by\s+//;
	$$creator=$tmpPartial;
	if($debug){print "\t=>  ($$creator)\n";}
	
	## process third part of the message
        ## each sn creates 5 type 30 tokens:  
	$tmpPartial = shift(@tmp);
	$tmpPartial =~ s/\D+\s*/:/gi;
	$tmpPartial =~ s/^://;
	($$tokenCount, $$tokenType) = split(":", $tmpPartial);
	if($debug){print "\t =>  ($$tokenCount, $$tokenType)\n";}
	
	## process last part (activation date)
        ## activated on Sat Nov 27 00:00:00 1999
	$tmpPartial = shift(@tmp); 
	$tmpPartial =~ /activated on\s+(\D+)\s+(\D+)\s+(\d+)\s+([\d-]+)\s+(\d+)/;
	my($year, $weekDay, $Month, $monthDay, $createTime)= ($5, $1, $2, $3, $4);
	my($hours, $min, $sec)=split("-", $createTime);
	if($debug){print "($year, $weekDay, $Month, $monthDay, $createTime)\n";}
	my %MoY = ('Jan',0,'Feb',1,'Mar',2,'Apr',3,'May',4,'Jun',5,
		   'Jul',6,'Aug',7,'Sep',8,'Oct',9,'Nov',10,'Dec',11);
	$createTime=timelocal($sec,$min,$hours,$monthDay,$MoY{$Month},$year-1900);
	if($debug){print "$createTime-> ", scalar localtime($createTime)," \n";}
	$$activationDate = ConvertSecsToSqlDate($createTime);
	if($debug){print "$$activationDate\n";}
	return 1;
    }else{
	return 0;
    }
}

#################################################################################
#
# Log messages found in frnscpd log :
#
#
#
sub processSnMinutes { 
    my($infile, $persist, $reportType, $debug) = @_;

    my ($time) = 0;
    my (%persistentData);
    my ($curpos) = 0;

    $infile = $$infile;

    ## Get and validate persistency information
    ## tie the hash
    if ( !tie(%persistentData,
              'DB_File',
              $persist,
              O_RDWR | O_CREAT,
              0644,
              $DB_HASH)) {
        logError ("processZktsa: Couldn't get persistency information.");
        exit (1);
    }

    ## Open the file and go to the previous position
    ## failed otherwise
    if ( defined($persistentData{'curpos'}) ) {

        $curpos = $persistentData{'curpos'};

        my ( $lineCheck ) = $persistentData{'lineCheck'};

        if (!open(FDIN, "<$infile")) {
            logError ("processZktsa: Couldn't open file");
            exit 1;
        }

        if (!seek(FDIN, $curpos, 0)) {
            logError ("processZktsa: Couldn't seek in file"); 
            exit 1;
        }

        my $line = readLogFileLine();

        if ($line ne $lineCheck) {
            logError ("processZktsa: Sanity Check on next line failed!");
            exit 1;
        }

	# Reset curpos to first unprocessed line.
        $curpos = tell(FDIN);

        close(FDIN);
    }

    ## Now Process from the curpos to the end of file
    ## 
    my(@partialTmp)   =();
    my(@linesOfInterest)   =();
    my($bytesOffset)  = 0;
    my($FlagComplete) = 1;
    my($writeThis) = 1;
    my($timeSql) ="";
    my($logMsg)="";

    ## get partial log file and keep the byte offset for fastest access
    $curpos = getRestOfLogFile($infile, \@partialTmp, $curpos, $debug);

    if (scalar(@partialTmp) == 0) {
        if ($debug) {print "no new data\n"};
        return 1;
    }

    ## defined the values
    my ($snCount, $snType, $creator, $tokenCount, $tokenType, $activationDate);

    ## grep for interesting lines and update
    @linesOfInterest = grep(/\s+\"zktsa\s+import/, @partialTmp);
    foreach $logMsg (@linesOfInterest) {

        # set defaults
        $snCount       = -1;
	$snType        = -1;
	$creator       = "";
	$tokenCount    = -1;
	$tokenType     = -1;
	$activationDate= "";

        $writeThis = processOneSnMinutes($logMsg,
				     \$time,
				     \$snCount, 
				     \$snType,
				     \$creator,
				     \$tokenCount,
				     \$tokenType,
				     \$activationDate,
				     $debug);
        if ( $writeThis ) {
	  #Insert sanity check output
	  outputRecord((convertTimeToSQLTime($time),
			$snCount, 
			$snType,
			$creator,
			$tokenCount,
			$tokenType,
			$activationDate
		       ));
        }
    }

# update persistency information

    $persistentData{'curpos'} = $curpos;
    $persistentData{'lineCheck'} = $partialTmp[$#partialTmp];

    return 1;
}


################################################################################
#
#
#
#
#
#require $ENV{'HOME'}."/work/digestLogSubs.pl";
#require $ENV{'HOME'}."/work/digestAuditSubs.pl";
#require "./digestLogSubs.pl";
#require "./digestAuditSubs.pl";
#
#$infile, $persist, $reportType, $debug
#my $infile="/tmp/thierry/zktsa";
#processZktsa ( \$infile, "/turf/thierry/fradmin/var/persist/sn_minutes", 0, 0);
#
#
# getDigestSnMinuteInfo ();
#
# my ($line,$time, $snCount, $snType, $creator, $tokenCount, $tokenType, $activationDate);
#
# open(FDIN,"<zktsa");
# while($line=<FDIN>){
#     chomp($line);
#     if( processOneSnMinutes($line, \$time, \$snCount, \$snType, \$creator, \$tokenCount, \$tokenType, \$activationDate, 0)){
# #	if($debug){
# 	    print STDOUT "$line\n";
# 	    print STDOUT "($time, $snCount, $snType, $creator, $tokenCount, $tokenType, $activationDate)\n";
# 	    print STDOUT "\n";
# #	}
#     }
# }
# close(FDIN);
#

1;

#   $Log: digestSnMinutes.pl,v $
#   Revision 1.1  2000/01/25 16:40:01  thierry
#   	Log file parsing to get import of serial in zktsa
#   	this script is called by digestLog.pl with the
#   	appropriate parameters
#
