#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestSngen.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		15 Oct 1999
#
#   Requirement         nm-10.6.0
#
#   Last Revision:	$Date: 1999/11/05 04:56:09 $
#
#   Description:	
#
#   $Revision: 1.5 $
#
#
#   $Id: digestSngen.pl,v 1.5 1999/11/05 04:56:09 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: digestSngen.pl,v 1.5 1999/11/05 04:56:09 thierry Exp $';
my $revision= '$Revision: 1.5 $';
my $progname= 'digestSngen.pl';

#################################################################################
#
# returns version info
#
sub getDigestSngenInfo { return "$progname:$version:$revision";}


#################################################################################
#
# Log messages found in sngen.c :
#
#
# Message related:
#
# zkLogWrite_va(context.log,"User %s is generating %d serial numbers of type %d",
# zkLogWrite_va(context.log,"Each serial number generates %d type %d tokens",
# zkLogWrite_va(context.log,"These serial numbers will activate on %s",
#
#
sub processOneSnGenReport{
  my ($fdin, $timeToLoad, $creator, $validationTime, $sernumNb, $sernumType, $tokenNb, $tokenType, $debug)= @_;

  my($line, $msg);
  my($time, $host, $pid, $service, @msg);
  my($returnValue) = 1;

  $$timeToLoad     = "";
  $$creator        = "";
  $$validationTime = "";
  $$sernumNb       = "";
  $$sernumType     = "";
  $$tokenNb        = "";
  $$tokenType      = "";

  $line = <$fdin>; ## need to check here eof
  if( $line){
    ## get the data
    ($time, $host, $pid, $service, @msg) = splitLogLine($line, , $debug);
    $msg=join(" ", @msg);
    if($debug){print $msg,"\n";}
    if($msg =~ /^"User/){
      $msg =~ /"User (.*) is generating ([0-9]+) serial numbers of type ([0-9]+)/;
       $$creator    = $1;
       $$sernumNb   = $2;
       $$sernumType = $3;
       $$timeToLoad = $time;
       if($debug){print ">$creator $sernumNb $sernumType\n";}
   }
  }else{
    $returnValue = 0;
  }

  if($returnValue){
    $line = <$fdin>;
    if( $line){
      ($time, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
      $msg=join(" ", @msg);
      if($debug){print $msg,"\n";}
      if($msg =~ /^"Each serial/){
      $msg =~ /"Each serial number generates ([0-9]+) type ([0-9]+) tokens/;
	 $$tokenNb = $1 * $$sernumNb;
	 $$tokenType = $2;
	 if($debug){print ">>",$1, $2, "\n";}
       }
    }else{
      $returnValue = 0;
    }
  }

  if($returnValue){
    $line = <$fdin>;
    if( $line){
      ($time, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
      $msg=join(" ", @msg);
      if($debug){print $msg,"\n";}
      if($msg =~ /^"These seria/){
      $msg =~ /"These serial numbers will activate on (.*)/;
	 $$validationTime = $1;
	 $$validationTime =~ s/"$//; #"
	 if($debug){print ">>> $$validationTime \n";}
       }
    }else{
      $returnValue = 0;
    }
  }

  return $returnValue;
}


#################################################################################
#
# Log messages found in sngen.c :
#
#
# Message related:
#
#
sub processSngenLog {
  my($infile, $timeStamp, $timePeriod, $reportType, $granularity, $verbose, $debug) = @_;
  print "($infile, $timeStamp, $timePeriod, $reportType, $granularity, $verbose, $debug)\n";
  my($sernumNb,$sernumType,$creator,$tokenNb,$tokenType,$validationTime);
  my($time, $host, $pid, $service, @msg);
 

  #
  # get first line
  my($line);
  if(! $timeStamp){
    $line = `head -n 1 $$infile`;
    ($timeStamp, $host, $pid, $service, @msg) = splitLogLine($line, , $debug);
    if($debug){
      print "\t$time\n\t$host\n\t$pid\n\t$service\n\t", join(" ", @msg),"\n";
    }
  }
 
  ## for now process the whole file
  my $startTimeInSec = getTimeLocalLogFromTimeStr($timeStamp);

  my $endTimeInSec   = "";
  my($year, $month, $mday, $hours, $min, $sec)=getDateAndTimeFromString($timeStamp, $debug);
  $year -= 1900; # need to do this for year
  $month -= 1;   # 0 based substring
  if($granularity =~ /month/){
    if($debug){print "month granularity\n";}
    $month += 1;
  }elsif($granularity =~ /year/){
    if($debug){print "year granularity\n";}
    $year += 1;
  }else{
    my($addHour, $addMin, $addSec) = getTimeFromString($timePeriod);
    $hours += $addHour;
    $min   += $addMin;
    $sec   += $addSec;
  }
  if($debug){print "$sec, $min, $hours, $mday, $month, $year\n";}

  $endTimeInSec = timelocal($sec, $min, $hours, $mday, $month, $year);

  if($debug){
    print 
      "start date   : $timeStamp\n",
      "start in secs: $startTimeInSec\n",
      "end   date   : ", getLocalTimeLog($endTimeInSec),"\n",
      "end   in secs: $endTimeInSec\n";
  }

  my($msg);
  my %SnTotalTypeCount = ();
  my %SnPeriodTypeCount = ();
  my $currenLogTimeInSec = 0;
  ($time, $creator, $validationTime, $sernumNb, $sernumType, $tokenNb, $tokenType)=(0,0,0,0,0,0,0);

  my($res) = open(FDIN, "<$$infile");
  while (processOneSnGenReport(\*FDIN, \$time, \$creator, \$validationTime, \$sernumNb, \$sernumType, \$tokenNb, \$tokenType, $debug)){
    if($debug){print "processSngenLog: <$time $creator $validationTime $sernumNb $sernumType $tokenNb $tokenType>\n";}

    $currenLogTimeInSec = getTimeLocalLogFromTimeStr($time);
    if($currenLogTimeInSec >= $startTimeInSec && $currenLogTimeInSec < $endTimeInSec){
      if(! defined($SnPeriodTypeCount{$sernumType})){
	$SnPeriodTypeCount{$sernumType} = 0;
      }
      $SnPeriodTypeCount{$sernumType} += $sernumNb;
    }

    ## do the counting for the serial numbers per types
    if(! defined($SnTotalTypeCount{$sernumType})){
      $SnTotalTypeCount{$sernumType} = 0;
    }
    $SnTotalTypeCount{$sernumType} += $sernumNb;

    if($verbose){
    ## get the printout done
    SWITCH: {
	if($reportType =~ /xml/) {
	  print  
	    "<sngenInfo>\n",
	    "   <time>$time</time>\n",
	    "   <creator>$creator</creator>\n",
	    "   <validationTime>$validationTime</validationTime>\n",
	    "   <sernumNb>$sernumNb</sernumNbtimeStamp>\n",
	    "   <sernumType>$sernumType</sernumType>\n",
	    "   <tokenNb>$tokenNb</tokenNb>\n",
	    "   <tokenType>$tokenType</tokenType>\n",
	    "</sngenInfo>\n",
	    "\n";
	  last SWITCH;
	}
	if($reportType =~ /line/) {
	  print "$time;$creator;$validationTime;$sernumNb;$sernumType;$tokenNb;$tokenType\n";
	  last SWITCH;
	}
	
	print  "Date              : $time\n";
	print  "Created by        : $creator\n";
	print  "Activation Date   : $validationTime\n";
	print  "Serial Numbers    : $sernumNb\n";
	print  "Type              : $sernumType\n";
	print  "Tokens            : $tokenNb\n";
	print  "Token Types       : $tokenType\n";
	print  "\n";
      }
    }
    ($time, $creator, $validationTime, $sernumNb, $sernumType, $tokenNb, $tokenType)=(0,0,0,0,0,0,0);
  }

  print
    "report period   : $granularity\n",
    "Report start at : $timeStamp\n",
    "Report End   at : ",getLocalTimeLog($endTimeInSec),"\n\n";

  my ($key, $periodTimeCount) = (0,0);
  foreach $key (keys %SnTotalTypeCount){
    if(defined($SnPeriodTypeCount{$key})){
      $periodTimeCount = $SnPeriodTypeCount{$key};
    }else{
      $periodTimeCount = 0;
    }
    print 
      "\tSN Type Number  : $key\n",
      "\tPeriod Created  : $periodTimeCount\n",
      "\tTotal  Created  : $SnTotalTypeCount{$key}\n\n",
  }
  print "\n";

  close(FDIN);
}  

1;

#   $Log: digestSngen.pl,v $
#   Revision 1.5  1999/11/05 04:56:09  thierry
#
#   # bug: 1188
#
#   	Modified sngen to do a report over a period of time
#   	modified digestReport
#
#   	[Peer: Luc]
#
#   Revision 1.4  1999/10/18 21:46:40  thierry
#   add timeStamp parameter to  processSngenLog
#
#   Revision 1.3  1999/10/18 21:42:44  thierry
#   add --startat [timeSTamp] Support
#
#   Revision 1.2  1999/10/15 21:58:29  thierry
#   parse full file
#
#   Revision 1.1  1999/10/15 21:26:00  thierry
#   add Sngen support
#
