#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestZktsaCount.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 5 Nov 1999
#
#   Last Revision:	$Date: 1999/11/05 21:50:46 $
#
#   Description:	
#
#   $Revision: 1.4 $
#
#
#   $Id: digestZktsaCount.pl,v 1.4 1999/11/05 21:50:46 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: digestZktsaCount.pl,v 1.4 1999/11/05 21:50:46 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'digestZktsaCount.pl';

select(STDERR); $| = 1;
select(STDOUT); $| = 1;

#################################################################################
#
# returns version info
#
sub getDigestZktsaCountInfo { return "$progname:$version:$revision";}


#################################################################################
#
# Log messages from zktsa -count
#
#
# Message related:
#
#"SNCOUNT: Type   500:  total 5200, active 5077, inactive 100, redeemed 23"
#
sub processOneZktsaCount{
  my($line, $time, $type, $total, $active, $inactive, $redeemed, $debug) = @_;

  my $oneMessageProcessed = 0;
  my($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
  my $message = join(" ", @msg);
  
  if($debug){print "processOneZktsaCount: $message\n";}
  if($message =~ /^\"SNCOUNT:\s+/){
    if($debug){print "processOneZktsaCount: start to parse\n";}
    $message =~ /\"SNCOUNT:\s+Type\s+([0-9]+):\s+total\s+([0-9]+),\s+active\s+([0-9]+),\s+inactive\s+([0-9]+),\s+redeemed\s+([0-9]+)\"/;
    ($$type, $$total, $$active, $$inactive, $$redeemed)=($1,$2,$3,$4,$5);
    if($debug) {
      print "processOneZktsaCount: ($1,$2,$3,$4,$5)\n";
      print "processOneZktsaCount: ($$type, $$total, $$active, $$inactive, $$redeemed)\n";
    }
  }
}

#################################################################################
#
# Log messages from zktsa -count
#
#
# Message related:
#
#
sub processZktsaCount {
  my($infile, $timeStamp, $timePeriod, $reportType, $debug) = @_;

  my($time, $bytesOffset, $logMsg, $timeSql)=(0,0,0,0);

  # get first line if $time == 0
  if(! $timeStamp){
    my $line = `head -n 1 $$infile`;
    my($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
    if($debug){
      print "\t->$logtime\n\t->$host\n\t->$pid\n\t->$service\n\t->", join(" ", @msg),"\n";
    }
    $time = $logtime;
  }else{
    $time = $timeStamp;
  }

  my($type, $total, $active, $inactive, $redeemed)=(0,0,0,0,0);
  my(@partialTmp)   =();
  my(@linesOfInterrest)   =();
  my($FlagComplete) = 1;
  while ($FlagComplete){
    @partialTmp = ();
    @linesOfInterrest = ();

    ## get partial log file and keep the byte offset for fastest access
    $bytesOffset = getPartialLogFile($$infile, $time, $timePeriod, \@partialTmp, $bytesOffset, $debug);
    if($debug){print join("<-\n", @partialTmp),"\n";}

    @linesOfInterrest = grep /\"SNCOUNT:\s+Type/, @partialTmp;

    if($debug){print "Res:\n", join("->\n", @linesOfInterrest),"\n";}
    
    foreach $logMsg (@linesOfInterrest){
      processOneZktsaCount($logMsg, \$time, \$type, \$total, \$active, \$inactive, \$redeemed, $debug);
    SWITCH: {
	if($reportType =~ /xml/) {
	  print 
	    "<totalCount>\n",
	    "   <time>$time</time>\n",
	    "   <type>$type</type>\n",
	    "   <total>$total</total>\n",
	    "   <active>$active</active>\n",
	    "   <inactive>$inactive</inactive>\n",
	    "   <redeemed>$redeemed</redeemed>\n",
	    "</totalCount>\n",
	    "\n";
	  last SWITCH;
	}
	if($reportType =~ /line/) {
	  # 1999/11/03-00:28:49.0 -> 1999-11-03 00:28:49
	  $timeSql = $time;
	  $timeSql =~ s/\.[0-9]$//;
	  $timeSql =~ s/-/ /gi;
	  $timeSql =~ s/\//-/gi;
	  print "$timeSql;$type;$total;$active;$inactive;$redeemed\n";
	  last SWITCH;
	}
	
	print 
	  "Report Date  : $time\n",
	  "type         : $type\n",
	  "total        : $total\n",
	  "active       : $active\n",
	  "inactive     : $inactive\n",
	  "redeemed     : $redeemed\n",
	  "\n";
      }
      ($type, $total, $active, $inactive, $redeemed)=(0,0,0,0,0);
    }

    last if($timeStamp);

    $time = addOffsetToTimeString($time, $timePeriod, 0);
    
    last if($bytesOffset < 0) ; # stops
  }
}

1;

#   $Log: digestZktsaCount.pl,v $
#   Revision 1.4  1999/11/05 21:50:46  thierry
#   	Fix debug information
#
#   Revision 1.3  1999/11/05 20:20:45  thierry
#   	add support for version string
#   	for digestZktsaCount.pl
#
#   Revision 1.2  1999/11/05 07:31:45  thierry
#   Correct Bug Number was #1189
#   	for the changes
#
#   Revision 1.1  1999/11/05 07:21:11  thierry
#   Bug# 1187
#        add support to report token counts per types
#        over period of time
#        [peer: Luc]
#
