#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		digestZktsaHeader.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 5 Nov 1999
#
#   Last Revision:	$Date: 1999/12/01 00:53:29 $
#
#   Description:	
#
#   $Revision: 1.3 $
#
#
#   $Id: digestZktsaHeader.pl,v 1.3 1999/12/01 00:53:29 alext Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Env;
my $version = '$Id: digestZktsaHeader.pl,v 1.3 1999/12/01 00:53:29 alext Exp $';
my $revision= '$Revision: 1.3 $';
my $progname= 'digestZktsaHeader.pl';

require $ENV{'HOME'}."/lib/digestLogSubs.pl";

#################################################################################
#
# returns version info
#
sub getDigestZktsaHeaderInfo { return "$progname:$version:$revision";}

################################################################################
#
# 
#
sub processOneZktsaHeader {
  my ($logMsg, $logtime, $nbSn, $snType, $creator, $nbToken, $tokenType, $activationDate, $debug)= @_;

  my($host, $pid, $service, @msg);
  ($$logtime, $host, $pid, $service, @msg) = splitLogLine($logMsg, $debug);
  my $message = join(" ", @msg);
  if($debug){print "processOneZktsaHeader: $message\n";}
  if($message =~ /^\"zktsa\s+import\s+[0-9]+\s+serial\s+numbers\s+/){
    if($debug){print "processOneZktsaHeader: start to parse\n";}
    $message =~ /^\"zktsa\s+import\s+([0-9]+)\s+serial\s+numbers\s+type\s+([0-9]+):\s+created\s+by\s+(.*@.*):\s+each\s+sn\s+creates\s+([0-9]+)\s+type\s+([0-9]+)\s+token:\s+activated\s+on\s+(.*)\"/;
    ($$nbSn, $$snType, $$creator, $$nbToken, $$tokenType, $$activationDate)=($1,$2,$3,$4,$5,$6);
    if($debug) {
      print "processOneZktsaHeader: ($1,$2,$3,$4,$5, $6)\n";
      print "processOneZktsaHeader: ($$nbSn, $$snType, $$creator, $$nbToken, $$tokenType, $$activationDate)\n";
    }
  }
  
}



################################################################################
#
# 
#
sub processZktsaHeader {
  my($infile, $timeStamp, $timePeriod, $reportType, $debug) = @_;

  my($time);
  # get first line if $time == 0
  #
  if(! $timeStamp){
    my $line = `head -n 1 $$infile`;
    my($logtime, $host, $pid, $service, @msg) = splitLogLine($line, $debug);
    if($debug){
      print "\t->$logtime\n\t->$host\n\t->$pid\n\t->$service\n\t->", join(" ", @msg),"\n";
    }
    $time = $logtime;
  }else{
    $time = $timeStamp;
  }
  my($timeLog, $nbSn, $snType, $creator, $nbToken, $tokenType, $activationDate)=("",0,0,0,0,0,"");
  my(@partialTmp)   =();
  my(@linesOfInterrest)   =();
  my($bytesOffset)  = 0;
  my($FlagComplete) = 1;
  my($flagEndProcess) = 1;
  my($timeSql) ="";
  my($logMsg)="";

  while ($FlagComplete){  
    @partialTmp = ();
    @linesOfInterrest = ();

    ## get partial log file and keep the byte offset for fastest access
    $bytesOffset = getPartialLogFile($infile, $time, $timePeriod, 
				     \@partialTmp, $bytesOffset, $debug);
    
    @linesOfInterrest = grep /\"zktsa\s+import\s+[0-9]+\s+serial\s+numbers\s+type\s+[0-9]+:/, @partialTmp;
    
    last if($bytesOffset < 0) ; # stops
    if($debug){print join("<-\n", @linesOfInterrest),"\n";}
    
    ($timeLog, $nbSn, $snType, $creator, $nbToken, $tokenType, $activationDate)=(0,0,0,0,0,0);
    ## need to do a cumulative summary 
    foreach $logMsg (@linesOfInterrest){
      processOneZktsaHeader($logMsg, \$timeLog, \$nbSn, \$snType, \$creator, \$nbToken, \$tokenType, \$activationDate, $debug);
      ## get the printout done
    SWITCH: {
	if($reportType =~ /xml/) {
	  print
	    "<zktsaImport>\n",
	    "   <time>$timeLog</time>\n",
	    "   <creator>$creator</creator>\n",
	    "   <validationTime>$activationDate</validationTime>\n",
	    "   <sernumNb>$nbSn</sernumNbtimeStamp>\n",
	    "   <sernumType>$snType</sernumType>\n",
	    "   <tokenNb>$nbToken</tokenNb>\n",
	    "   <tokenType>$tokenType</tokenType>\n",
	    "<zktsaImport>\n",
	    "\n";
	  last SWITCH;
	}
	if($reportType =~ /line/) {
	  print "$timeLog;$creator;$activationDate;$nbSn;$snType;$nbToken;$tokenType\n";
	  last SWITCH;
	}

	print 
	  "Date              : $timeLog\n",
	  "Created by        : $creator\n",
	  "Activation Date   : $activationDate\n",
	  "Serial Numbers    : $nbSn\n",
	  "Type              : $snType\n",
	  "Tokens            : $nbToken\n",
	  "Token Types       : $tokenType\n",
	  "\n";
      }
    }
    
    last if($timeStamp);
    
    $time = addOffsetToTimeString($time, $timePeriod, 0);
    
    last if($bytesOffset < 0) ; # stops
  }
}

1;

#   $Log: digestZktsaHeader.pl,v $
#   Revision 1.3  1999/12/01 00:53:29  alext
#   making requires $HOME relative
#
#   Revision 1.2  1999/11/05 21:49:16  thierry
#   	add support to generate xml and sql output formats
#   	fix debug information
#
#   Revision 1.1  1999/11/05 20:28:49  thierry
#   	# bug: 1187
#   	add reporting capabilities for serial number reports
#   	from zktsa log files (nm 10.5.0)
#   	[Peer: Luc]
#
