#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:           lock.pl
#
#   Author(s):		Alex Taler & Philippe Desaulniers
#
#   Graciously debugged by Louis Beaumier   
#
#   Created:		30 nov 1999 (but barely!)
#
#   Last Revision:	$Date: 2000/03/14 21:12:51 $
#
#   Description:	
#    This function grabs the pid lock if it can.
#    Return value is 0 on failure, 1 on success.
#
#    This function assumes that we have /proc
#
#    Extra sanity checks on the process could be useful.
#
#   $Revision: 1.4 $
#
#
#   $Id: lock.pl,v 1.4 2000/03/14 21:12:51 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use English;
use Getopt::Long;
use Env;
select(STDOUT); $| = 1;

my $version = '$Id: lock.pl,v 1.4 2000/03/14 21:12:51 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'lock.pl';

my @args = @ARGV;

# grap the access lock, if no other process is active
# Don't forget to remove the file when you're done with it.

my $filename = $args[0];
my $ppid = $args[1];
my $pid;


if (-f $filename) {
  chomp($pid = `cat $filename`);
  
  # Return failure if the process exists
  if (-d "/proc/$pid") {
    exit 1;
  }

  # Continue and overwrite the file
}

# write to pid file

# Get the parent process ID.
$ppid = getppid() unless (defined($ppid));

open(FDOUT, ">$filename") or exit 1; # should be die "Can't open file";
print FDOUT "$ppid\n" or exit 1;     # should be die "Can't write to file";
close(FDOUT) or exit 1;              # should be die "Can't close opened file";
exit 0;
1;

#   $Log: lock.pl,v $
#   Revision 1.4  2000/03/14 21:12:51  thierry
#   fix bug #1563: checks errors on pid file open, write close
#
#   Revision 1.3  1999/12/01 16:37:46  filou
#   Corrected parent pid problem
#
#   Revision 1.2  1999/12/01 06:23:01  louis
#   - cahanged the hardcode path for the call to frgetcustomersupport.pl
#
#   Revision 1.1  1999/12/01 05:15:06  filou
#   Creation
#
#
