#! /bin/sh
#
# this script is used to make sure that the .dat files generated from nidbtool 
# request are updated every hour.
#
# this validation must be done after every call to the /usr/fradmin/cron_hour.sh 
#
# at least one file for each entity type (aip,maip,fmg-maip) must have been updated 
# if not a message is sent indicating which entity file hasn't
#

#
# basic config
#
homepath=/usr/fradmin
datpath=${homepath}/var/dat

#
# people to advise in case of problem
#
mailList="louis@zks.net,luc@zks.net,gary@zks.net"

# do we want successfull update to be flagged?: 0=no, 1=yes
repSuccess=0

timemark=${datpath}/.timemark



#
# proceed only if there is a timemark
#
if [ -f ${timemark} ]
then
    #
    # get the number of updated file for every report type
    #
    aip_update=`find ${datpath} -newer ${timemark} -ls | grep "aip_activity_" | wc -l`
    maip_update=`find ${datpath} -newer ${timemark} -ls | grep "maip_activity_" | wc -l`
    fmg_maip_update=`find ${datpath} -newer ${timemark} -ls | grep "fmg-maip_activity_" | wc -l`


    #
    # report any problem
    # 
    message="$0:  [`date`]\n\nlist of .dat file that have NOT been updated in the last hour:\n\n"
    mustSend=0
    if [ $aip_update -eq 0 ] 
    then
	message=${message}"\taip_activity_*.dat\n"
	mustSend=1
    fi
    if [ $fmg_maip_update -eq 0 ]
    then
	message=${message}"\tfmg-maip_activity_*.dat\n"
	mustSend=1
    fi
    if [ $maip_update -eq 0 ]
    then
	message=${message}"\tmaip_activity_*.dat\n"
	mustSend=1
    fi

    if [ $mustSend -eq 1 ]
    then
	printf "${message}" | /usr/bin/Mail -s "RepMon problem" ${mailList}
    fi


    #
    # report success (only if enabled)
    #
    if [ $repSuccess -eq 1 ]
    then
	message="$0:  [`date`]\n"
	mustSend=0
	if [ $aip_update -ne 0 ]
	then
	    message=${message}"aip_activity_*.dat updated\n"
	    mustSend=1
	fi
	if [ $fmg_maip_update -ne 0 ]
	then
	    message=${message}"fmg-maip_activity_*.dat updated\n"
	    mustSend=1
	fi
	if [ $maip_update -ne 0 ]
	then
	    message=${message}"maip_activity_*.dat updated\n"
	    mustSend=1
	fi
	if [ $mustSend -eq 1 ]
	then
	    printf "${message}" | /usr/bin/Mail -s "RepMon success" ${mailList}
	fi
    fi
fi

#
# create/update the timemark
#
touch ${timemark}
