
/* $Id: frnissd.h,v 1.11 2001/04/19 19:22:10 alainc Exp $ */

#ifndef FRNISSD_H_ 
#define FRNISSD_H_

#include "zks.h"

#include <sys/types.h>
#include <stdio.h>
#include <zkConfig.h>
#include <zkIntTypes.h>
#include <zkParms.h>
#include <frEnt.h>
#include <frFilenames.h>
#include <frStatUpd.h>
#include <frStatClient.h>
#include <frStatBucket.h>
#include "frDBOFuncs.h"
#include <fr_nidbctx.h>

#include "frNetInfo.h"

#include "zkExpiryTable.h"
#include "zkNameTable.h"

#ifdef __cplusplus 
extern "C" {
#endif /* __cplusplus */

  typedef
  struct T_FrStatCtx {
    ZkConfig config;            /* Configuration object */
    ZkLog log;                  /* Log object */
    const char *progname;       /* Program name */
    const char *configfile;     /* Configuration file */
    const char *logfile;        /* Log file */
#if 0
    const char *tempdatafile;   /* 
                                 * Bug: This will need to be replaced with
                                 * real functions to parse the data and save
                                 * it to a database at one point.  Temporary
                                 * file where all stat packets will be
                                 * written once they are decrypted and the
                                 * signature is checked
                                 */  
#endif
    int debug;                  /* debug or not */
    int fork;                   /* fork or not */
    const char *pidfile;        /* PID file */

    FrNetInfo ni;               /* Freedom Network Information object */

    const char *niqsHostname;   /* NIQS hostname */
    unsigned short niqsPort;    /* NIQS host port */
    unsigned short niqsTimeout; /* NIQS Timeout */

    FrKeyQry kq;                /* Key query cache */
    FrKeyQryHost kh;            /* Key query host IP/port/descriptor */
    unsigned short kqtimeout;   /* Timeout */
    unsigned short kqretries;   /* # of Retries */

    const char *ppfile;         /* Passphrase file */
    const char *ppprompt;       /* Passphrase prompt */
    FrSecKeyChain kc;           /* Key chain */
    int sigHandle;              /* Signature key handle */
    int cryptHandle;            /* Cryption key handle */
    const char *sigdir;         /* Signature directory */
    const char *cryptdir;       /* Cryption directory */
  
    unsigned short myUdpPort;   /* My UDP port */
    int myUdpFd;                /* My UDP descriptor */
    struct sockaddr_in me;      /* My IP address and port */

    unsigned short cmdPort;     /* TCP port to listen to for commands */
    int cmdOpen;                /* Command port is connected / refuse access */
    int cmdFd;                  /* Command socket descriptor */
    int cmdListenFd;            /* Command socket descriptor */
    int cmdListenq;             /* listen() queue value */
    int cmdRetries;             /* Number of connection retries */
    const char *cmdWelcome;     /* Welcome message on the descriptor */
    const char *cmdRefused;     /* Welcome message on the descriptor */

    int maxFd;


    const char * event_record_db;
    ZkAA evRecDB;

    NidbContext nidb;           /* The NIDB context information */
    FrStatBucket bucket;

    ZkExpiryTable expiryTable;  /* used for the aip status setting */
    ZkNameTable   requestTable; /* used for the aip status setting */


    FILE *fptr;
    
  } FrStatCtx;

  /* Signal handler */
  extern void frStatdSigHandler(int);

  extern int frStatdCtxEventLoop(ZkError, FrStatCtx *);

  extern int frNissDSendEntityEvent (ZkError err,
				     FrStatCtx *c,
				     uint32_t r_ipaddr,
				     uint16_t r_port,
				     int event);
static int helpOrVersionArgument(ZkError err, int argc,const char **argv);

#define DEFAULT_CMD_WELCOMEMSG  "** NISS Command Port\n"
#define DEFAULT_CMD_REFUSEDMSG  "** NISS Command port already in use ...\n"
#define DEFAULT_CMD_LISTENQ     3 
#define DEFAULT_CMD_RETRIES     3
#define DEFAULT_KQ_TIMEOUT      10
#define DEFAULT_KQ_RETRIES      5
#define DEFAULT_PASSPHRASE_FILE "/dev/fd/0"  

#define DEFAULT_LOGFILE		FREEDOM_LOGDIR"/nissd"
#define DEFAULT_CONFIGFILE	FREEDOM_CONFDIR"/freedom-nissd.conf"
#define DEFAULT_PIDFILE		FREEDOM_PIDDIR"/nissd.pid"
#define DEFAULT_KQTIMEOUT	5
#define DEFAULT_KTRETRIES	10
  
#define DEFAULT_PPPROMPT	NULL
#define DEFAULT_CRYTPTDIR       FREEDOM_PRIVKEYDIR"/NetInfoQryServ/crypt"
#define DEFAULT_SIGDIR		FREEDOM_PRIVKEYDIR"/NetInfoQryServ/sig"

#define VERSION_STR "Zeroknowledge Freedom Status Deamon v1.1.22.1"\
                    "("__DATE__", "__TIME__") \n"

#define OPTIONS "\nOPTIONS\n"\
"  Where options can be one or more of the following:\n\n"\
"  --"P_CONFIG" path \n"\
"      Location of the configuration information.\n"\
"  --"P_DEBUG"  \n"\
"      Enable debugging mode.\n" \
"  --"P_DAEMON"  \n"\
"      Forks and creates a background daemon process.\n"\
"  --"P_PIDFILE" PIDfile   \n"\
"      File where the parent process PID is written.\n"\
"  --"P_LOGFILE" LogFile  \n"\
"      Log file.\n"\
"  --"P_KEYQRY_HOSTNAME" "P_HOSTNAME"\n"\
"      Key query server hostname.\n"\
"  --"P_KEYQRY_PORT" "P_PORT"\n"\
"      Key query server port.\n"\
"  --"P_KEYQRY_RETRIES" retries \n"\
"      Number of retries to connect to the key query server.\n"\
"  --"P_KEYQRY_TIMEOUT" "P_TIMEOUT"\n"\
"      Timeout value for a key lookup.\n"\
"  --"P_PASSPHRASE_FILE" filename\n"\
"      Passphrase filename.\n"\
"  --"P_PASSPHRASE_PROMPT" prompt\n"\
"      Passphrase prompt.\n"\
"  --"P_KEYINFO_DIRECTORY_CRYPT" path \n"\
"      Path to all the server's private cryption keys.\n"\
"  --"P_KEYINFO_DIRECTORY_SIG" path \n"\
"      Path to all the server's private signature keys.\n"\
"  --"P_PORT" "P_PORT"Number \n"\
"      UCP and command (TCP) port number of the server. \n"\
"  --"P_NIQS_SERVER" niqsHostname \n"\
"      Host name of the target NIQS.\n"\
"  --"P_NIQS_PORT" portnNumber \n"\
"      Port number of the NIQS target.\n"\
"  --"P_NIQS_TIMEOUT" timeout \n"\
"      Timeout (ZKIP) of the NIQS target.\n"

#define DATABASE_OPTIONS "\nDATABASE OPTIONS\n"\
"  Additionally, there are a number of database-specific options.\n\n"\
"  --"P_NIDB_STATE" configFile \n"\
"      Configuration file for the state database.\n"\
"  --"P_NIDB_TOPO" configFile\n"\
"      Configuration file for the topology database.\n"\
"  --"P_NIDB_ENT" configFile \n"\
"      Configuration file for the entity database.\n"\
"  --"P_NIDB_ENTID" configFile \n"\
"      Configuration file for the entity identity database.\n"\
"  --"P_NIDB_NEIGHBOUR" configFile\n"\
"      Configuration file for the neighbour database.\n"\
"  --"P_NIDB_ARCHIVE" configFile\n"\
"      Configuration file for the archive database.\n"\
"  --"P_NIDB_ENTUPD" configFile \n"\
"      Configuration file for the entity last update database.\n"\
"  --"P_NIDB_HOST" configFile \n"\
"      Configuration file for the host database.\n"\
"  --"P_NIDB_ARCHIVE_SQLHOST" configFile \n"\
"      Configuration file for the host SQL database. \n"\
"  --"P_NIDB_ARCHIVE_SQLUSER" configFile \n"\
"      Configuration file for the user name SQL database.\n"\
"  --"P_NIDB_ARCHIVE_SQLPSWD" configFile \n"\
"      Configuration file for the password SQL database.\n"\
"  --"P_NIDB_ARCHIVE_SQLDB" configFile \n"\
"      Configuration file for the archive SQL database.\n"

#define MESG_HELP VERSION_STR"\nUsage:  %s --options\n"OPTIONS""DATABASE_OPTIONS"\n"

#ifdef __cplusplus 
}
#endif /* __cplusplus */

#endif /* FRNISSD_H_ */
