/*
 * The Network Information State Service
 *
 * $Id: niss.c,v 1.17 2000/03/09 16:35:52 danielf Exp $
 * Luc Boulianne <luc@zks.net>
 */

#ifdef _NISS_REPLAY_FEATURE_
#  include "nissReplay.h"
#endif

#include "zks.h"

#include <time.h>
#include <sys/types.h>
#include "zkHash.h"
#include "zkDebug.h"
#include "frStatPkt.h"
#include "frStatUpd.h"
#include "fr_entid.h"
#include "fr_nidb.h"
#include "fr_niss.h"
#include "frnissd.h"

#define NISS_MAX_NEIGHBOURS 30
#define FSU_MOST_RECENT_HISTORY 0

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMNISS


int frNissHandleUpdateBuffer(
  ZkError err ,
  FrStatCtx *c,
  FrStatBucket b ,
  void * buf,
  size_t bsz)
{
  static const char *funcname = "frNissHandleUpdateBuffer";
  int retcode = 1;

  /* frStatUpd things */
  FrStatUpd fsu = NULL;
  FrStatUpdTypes type = FRSTAT_T_UNUSED;
  uint32_t sig = 0, vers = 0;
  
  
  zkErrorClear(err);
  /*
   * We will start by extracting all usefull information from the
   * status packet
   */

  if (! c || !b || !buf) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
    /* Do this first - to decrypt and verify signature */
    retcode = 0;
  } else if ( ! frStatUpdFromBuffer(err, &fsu, buf, bsz)) {
    zkLogWriteContext (c->log, err, funcname, "Unable to process update packet");
    retcode = 0;
  } else if ( ! frStatUpdGetSigVers(err, fsu, &sig, &vers) ) {
    zkLogWriteContext (c->log, err, funcname, "Unable to extract signature/version on update packet");
    retcode = 0;
  } else if ((sig != FRSTAT_DEFAULT_SIGNATURE) || (vers != FRSTAT_DEFAULT_VERSION)) {
    zkErrorSet_va(err,funcname,ZKERR,ZKMOD,ZKEDATABAD,
                  "Invalid signature(0x%x) or version number(0x%x) on update packet",
                  sig, vers);
    retcode = 0;
  } else if  ( ! frStatUpdType(err,fsu, &type) ) {
    zkLogWriteContext (c->log, err, funcname, 
                       "Unable to extract type information from update packet");
    retcode = 0;
  } else {
    /*
     * The best way to handle these kinds of packets would be to
     * use a generic tuple of some sort. I've decided to use a single type
     * of stats format, at the expense of space. It'll be quicker to
     * handle in this manner
     */


#ifndef NDEBUG
    frStatUpdDump (err, fsu, 0, stderr);
#endif
  
    switch (type) {
    case FRSTAT_T_AIP:
      if (!frNissHandleAip (err, c, fsu)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissHandleAip failed");
        retcode = 0;
      }
      break;
    case FRSTAT_T_MAIP:
      if (!frNissHandleMaip (err, c, fsu)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissHandleMaip failed");
        retcode = 0;
      }
      break;
    case FRSTAT_T_LINK:
      if (!frNissHandleLink (err, c, fsu)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissHandleLink failed");
        retcode = 0;
      }
      break;
    case FRSTAT_T_WORMHOLE:
      if (!frNissHandleWormhole (err, c, fsu)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissHandleWormhole failed");
        retcode = 0;
      }
      break;
    default:
      zkErrorSet_va(err,funcname,ZKERROR,ZKMFRSTAT,ZKENOSYS,
                    "Unknown status update type (0x%x)", type);
      retcode = 0;
      break;
    }
  }

  if (fsu) {
    frStatUpdFree(err, &fsu);
    fsu = NULL;
  }
    
  return retcode;
}


