/*
 * The Network Information State Service
 * The AIP update
 *
 * $Id: nissAip.c,v 1.38 2000/11/24 14:59:30 marcp Exp $
 * Luc Boulianne <luc@zks.net>
 */


#ifdef _NISS_REPLAY_FEATURE_
#  include "nissReplay.h"
#endif

#include "zks.h"

#include <time.h>
#include <sys/types.h>
#ifdef ZK_SOLARIS
#include <sys/ddi.h>
#endif
#include "zkLog.h"
#include "zkHash.h"
#include "zkDebug.h"
#include "zkAlloc.h"
#include "frStatPkt.h"
#include "frStatUpd.h"
#include "frstats.h"
#include "frDBOFuncs.h"
#include "fr_entid.h"
#include "fr_nidb.h"
#include "fr_niss.h"
#include "frnissd.h"

#include "frNetworkDefaults.h"

#define FSU_MOST_RECENT_HISTORY 0


#if 0   /*  Not used  */

static int frNissGetStateUsingEntIdent (
  ZkError err,
  FrStatCtx *c,
  FrEntIdent eid,
  FrDBOTypeNsdbAip *state,
  size_t *ssz,
  int *found)
{
  static const char *funcname = "frNsdbGetStateUsingEntIdent";
  int return_code = 0;
  size_t psz = 0;

  zkErrorClear(err);
  if (! c || !state || !ssz || !found ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
  }

  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNSDBAIP, (void **) state, ssz)) {
    zkLogWriteContext(c->log, err, funcname, "error with zkDBONew");
    goto finish;
  }
  (*state)->entid = eid;
  
  if ( ! frNsdbFetch (err, c->nidb, *state, found)) {
    zkLogWriteContext(c->log, err, funcname, "Error trying to fetch entity identifier %u from state database",
                      eid);
    goto finish;
  }
  
  if ( ! *found ) {
    zkLogWrite_va(c->log, "Failed to locate entity identifier %u in state database",
                  eid);
  }
  return_code = 1;
  
 finish:
  return return_code;

}

#endif


static int frNissGetStateUsingEntity (
  ZkError err ,
  FrStatCtx *c,
  FrEntIdExp *entity,
  FrDBOTypeNsdbAip *state,
  size_t *ssz,
  FrDBOTypeEntityID *eid_dbo,
  size_t *esz,
  int *found)
{
  static const char *funcname = "frNsdbGetStateUsingEntity";
  int return_code = 0;
  uint32_t eid = 0;
  size_t psz = 0;

  zkErrorClear(err);
  if (! c || !state || !eid_dbo) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
  }

  /* Convert the entity into a dbo */
  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRENTITYID, (void **) eid_dbo, esz)){
    zkLogWriteContext(c->log, err, funcname, "error with zkDBONew");
    goto finish;
  }

  psz = 0;
  if ( ! zkDBONew(err, ZKDBO_REG_FRNSDBAIP, (void **) state, ssz)) {
    zkLogWriteContext(c->log, err, funcname, "error with zkDBONew");
    goto finish;
  }
  if (!frEntCopy(err, &((*eid_dbo)->ent), entity)) {
     zkLogWriteContext(c->log, err, funcname, "error with frEntCopy");
     goto finish;
  }
  
  /* Get the entity identifier (ie: the key) for this entity */
  if ( ! frNidbGetEntityIdentifier (err, c->nidb, *eid_dbo, &eid, found)) {
    zkLogWriteContext(c->log, err, funcname, "error with frNidbGetEntityIdentifier");
    goto finish;
  }

  if ( ! *found ) {
    zkLogWrite_va(c->log, "Failed to find entity %s/%d in entity database",
                  (*eid_dbo)->ent.name, (*eid_dbo)->ent.type);
    return_code = 1;
    goto finish ; 
  }
  zkDebug ("Originator:  %s (%d)\n",(*eid_dbo)->ent.name,(*eid_dbo)->ent.type);
  
  (*state)->entid = eid;
  
  if ( ! frNsdbFetch (err, c->nidb, *state, found)) {
    zkLogWriteContext(c->log, err, funcname,
		  "Error trying to fetch entity %s/%d state from state database",
		  (*eid_dbo)->ent.name, (*eid_dbo)->ent.type);
    goto finish;
  }
  
  if ( ! *found ) {
    zkLogWrite_va(c->log, "Failed to find entity %s/%d state in state database",
		  (*eid_dbo)->ent.name, (*eid_dbo)->ent.type);
  }
  return_code = 1;
  
 finish:
  return return_code;

}

/**
 * Adjust the AIP using this links information
 */
static int makeLinkAdjustmentsToItsAip(
  ZkError err,
  NidbContext c,
  FrEntIdExp *aip_entity,
  FrDBOTypeNsdbAip aipstate,
  time_t            reptime,
  /*  uint32_t          seqno;*/
  const FrEntIdExp *nb_entity,
  time_t            launchtime,
  time_t            last_change_time,
  time_t            last_status_time,
  UINT64_T          new_pkts_rx,
  UINT64_T          new_pkts_tx,
  UINT64_T          new_pkts_drop,
  UINT64_T          new_bytes_rx,
  UINT64_T          new_bytes_tx,
  UINT64_T          new_bytes_drop)
{
  static const char *func = "makeLinkAdjustmentsToItsAip";
  int retcode = 0;

  time_t    o_reptime = 0;
  time_t    o_launchtime = 0;
  time_t    o_last_status_time = 0;
  time_t    o_last_niqs_reload_time = 0;
  uint32_t  o_new_client_connections = 0;
  uint32_t  o_curr_connected_clients = 0;
  UINT64_T  o_client_bytes_rx = 0;
  UINT64_T  o_client_bytes_tx = 0;
  UINT64_T  o_client_bytes_dropped = 0;
  UINT64_T  o_client_pkts_rx = 0;
  UINT64_T  o_client_pkts_tx = 0;
  UINT64_T  o_client_pkts_dropped = 0;
  UINT64_T  o_total_bytes_rx = 0;
  UINT64_T  o_total_bytes_tx = 0;
  UINT64_T  o_total_bytes_dropped = 0;
  time_t    o_time_period = 0;
  char      *o_release = NULL;

  (void) reptime;
  (void) launchtime;
  (void) last_change_time;
  (void) last_status_time;
  (void) new_pkts_rx;
  (void) new_pkts_tx;
  (void) new_pkts_drop;

  if ( !aip_entity || ! aipstate || ! nb_entity) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  } else if ( ! frNsdbGetStatusForAip( err, aipstate,
                                       &o_reptime,
                                       &o_launchtime,
                                       &o_last_status_time,
                                       &o_last_niqs_reload_time,
                                       &o_new_client_connections,
                                       &o_curr_connected_clients,
                                       &o_client_bytes_rx,
                                       &o_client_bytes_tx,
                                       &o_client_bytes_dropped,
                                       &o_client_pkts_rx,
                                       &o_client_pkts_tx,
                                       &o_client_pkts_dropped,
                                       &o_total_bytes_rx,
                                       &o_total_bytes_tx,
                                       &o_total_bytes_dropped,
                                       &o_time_period,
                                       &o_release) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
    
  } else { 
    o_total_bytes_rx += new_bytes_rx;
    o_total_bytes_tx += new_bytes_tx;
    o_total_bytes_dropped += new_bytes_drop;
    
    if ( ! frNsdbSetStatusForAip( err, aipstate,
                                  o_reptime,
                                  o_launchtime,
                                  o_last_status_time,
                                  o_last_niqs_reload_time,
                                  o_new_client_connections,
                                  o_curr_connected_clients,
                                  o_client_bytes_rx,
                                  o_client_bytes_tx,
                                  o_client_bytes_dropped,
                                  o_client_pkts_rx,
                                  o_client_pkts_tx,
                                  o_client_pkts_dropped,
                                  o_total_bytes_rx,
                                  o_total_bytes_tx,
                                  o_total_bytes_dropped,
                                  o_time_period,
                                  o_release) ) {
      
      
      zkLogWriteContext(c->log, err, func, "Failure to set aip information for [%s]",
                        aip_entity->name);
    } else 
      retcode = 1;
  }
  FREEIT(o_release);
  return retcode;
}

static int makeAipAdjustments(
  ZkError err,
  NidbContext c,
  FrEntIdExp *aip_entity,
  FrDBOTypeNsdbAip aipstate,
  time_t reptime,
  uint32_t seqno,
  time_t launchtime,
  time_t last_niqs_reload_time, 
  time_t last_status_time,
  uint32_t new_client_connections,
  uint32_t curr_connected_clients,
  UINT64_T client_bytes_rx,
  UINT64_T client_bytes_tx,
  UINT64_T client_bytes_dropped,
  UINT64_T client_pkts_rx,
  UINT64_T client_pkts_tx,
  UINT64_T client_pkts_dropped,
  char *release)
{
  static const char *func = "makeAipAdjustments";
  
  time_t    o_reptime = 0;
  time_t    o_launchtime = 0;
  time_t    o_last_status_time = 0;
  time_t    o_last_niqs_reload_time = 0;
  uint32_t  o_new_client_connections = 0;
  uint32_t  o_curr_connected_clients = 0;
  UINT64_T  o_client_bytes_rx = 0;
  UINT64_T  o_client_bytes_tx = 0;
  UINT64_T  o_client_bytes_dropped = 0;
  UINT64_T  o_client_pkts_rx = 0;
  UINT64_T  o_client_pkts_tx = 0;
  UINT64_T  o_client_pkts_dropped = 0;
  UINT64_T  o_total_bytes_rx = 0;
  UINT64_T  o_total_bytes_tx = 0;
  UINT64_T  o_total_bytes_dropped = 0;
  time_t    o_time_period = 0;
  char     *o_release = NULL;
  int retcode = 0;

  (void) seqno;
  (void) last_niqs_reload_time;
    
  if ( !aip_entity || ! aipstate ) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  } else if ( ! frNsdbGetStatusForAip( err, aipstate,
                                       &o_reptime,
                                       &o_launchtime,
                                       &o_last_status_time,
                                       &o_last_niqs_reload_time,
                                       &o_new_client_connections,
                                       &o_curr_connected_clients,
                                       &o_client_bytes_rx,
                                       &o_client_bytes_tx,
                                       &o_client_bytes_dropped,
                                       &o_client_pkts_rx,
                                       &o_client_pkts_tx,
                                       &o_client_pkts_dropped,
                                       &o_total_bytes_rx,
                                       &o_total_bytes_tx,
                                       &o_total_bytes_dropped,
                                       &o_time_period,
                                       &o_release) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
  } else { 
    o_reptime = reptime;
    o_launchtime = launchtime;
    
    if(o_last_status_time) {
        o_time_period = last_status_time - o_last_status_time;
    }
    o_last_status_time = last_status_time;

    o_new_client_connections += new_client_connections;
    o_curr_connected_clients = curr_connected_clients;
    o_client_bytes_rx = client_bytes_rx;
    o_client_bytes_tx = client_bytes_tx;
    o_client_bytes_dropped = client_bytes_dropped;
    o_client_pkts_rx = client_pkts_rx;
    o_client_pkts_tx = client_pkts_tx;
    o_client_pkts_dropped = client_pkts_dropped;
    o_total_bytes_rx += client_bytes_rx;
    o_total_bytes_tx += client_bytes_tx;
    o_total_bytes_dropped += client_bytes_dropped;
    strncpy(o_release, release, FRSTATUPD_RELEASE_STRING_SZ-1);
 
    if ( ! frNsdbSetStatusForAip( err, aipstate,
                                  o_reptime,
                                  o_launchtime,
                                  o_last_status_time,
                                  o_last_niqs_reload_time,
                                  o_new_client_connections,
                                  o_curr_connected_clients,
                                  o_client_bytes_rx,
                                  o_client_bytes_tx,
                                  o_client_bytes_dropped,
                                  o_client_pkts_rx,
                                  o_client_pkts_tx,
                                  o_client_pkts_dropped,
                                  o_total_bytes_rx,
                                  o_total_bytes_tx,
                                  o_total_bytes_dropped,
                                  o_time_period,
                                  o_release) ) {
      
      
      zkLogWriteContext(c->log, err, func, "Failure to set aip information for [%s]",
                        aip_entity->name);
    } else 
      retcode = 1;
  }
  FREEIT(o_release);
  return retcode;
}



static int makeWormholeAdjustmentsToItsAip(
  ZkError          err,
  NidbContext      c,
  FrEntIdExp      *aip_entity,
  FrDBOTypeNsdbAip aipstate,
  time_t           launchtime,
  time_t           last_status_time,
  uint32_t         maxin_thrput,
  uint32_t         new_streams,
  uint32_t         current_active_streams,
  uint32_t         new_ports,
  uint32_t         curr_used_ports,
  UINT64_T         new_bytes_rx,
  UINT64_T         new_bytes_tx
)
{
  static const char *func = "makeLinkAdjustmentsToItsAip";
  int retcode = 0;

  time_t    o_reptime = 0;
  time_t    o_launchtime = 0;
  time_t    o_last_status_time = 0;
  time_t    o_last_niqs_reload_time = 0;
  uint32_t  o_new_client_connections = 0;
  uint32_t  o_curr_connected_clients = 0;
  UINT64_T  o_client_bytes_rx = 0;
  UINT64_T  o_client_bytes_tx = 0;
  UINT64_T  o_client_bytes_dropped = 0;
  UINT64_T  o_client_pkts_rx = 0;
  UINT64_T  o_client_pkts_tx = 0;
  UINT64_T  o_client_pkts_dropped = 0;
  UINT64_T  o_total_bytes_rx = 0;
  UINT64_T  o_total_bytes_tx = 0;
  UINT64_T  o_total_bytes_dropped = 0;
  time_t    o_time_period = 0;
  char      *o_release = NULL;

  (void)launchtime;
  (void)last_status_time;
  (void)maxin_thrput;
  (void)new_streams;
  (void)current_active_streams;
  (void)new_ports;
  (void)curr_used_ports;

  if ( !aip_entity || ! aipstate ) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  } else if ( ! frNsdbGetStatusForAip( err, 
                                       aipstate,
                                       &o_reptime,
                                       &o_launchtime,
                                       &o_last_status_time,
                                       &o_last_niqs_reload_time,
                                       &o_new_client_connections,
                                       &o_curr_connected_clients,
                                       &o_client_bytes_rx,
                                       &o_client_bytes_tx,
                                       &o_client_bytes_dropped,
                                       &o_client_pkts_rx,
                                       &o_client_pkts_tx,
                                       &o_client_pkts_dropped,
                                       &o_total_bytes_rx,
                                       &o_total_bytes_tx,
                                       &o_total_bytes_dropped,
                                       &o_time_period,
                                       &o_release) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
    
  } else { 
    o_total_bytes_rx += new_bytes_rx;
    o_total_bytes_tx += new_bytes_tx;
    
    if ( ! frNsdbSetStatusForAip( err, 
                                  aipstate,
                                  o_reptime,
                                  o_launchtime,
                                  o_last_status_time,
                                  o_last_niqs_reload_time,
                                  o_new_client_connections,
                                  o_curr_connected_clients,
                                  o_client_bytes_rx,
                                  o_client_bytes_tx,
                                  o_client_bytes_dropped,
                                  o_client_pkts_rx,
                                  o_client_pkts_tx,
                                  o_client_pkts_dropped,
                                  o_total_bytes_rx,
                                  o_total_bytes_tx,
                                  o_total_bytes_dropped,
                                  o_time_period,
                                  o_release) ) {
      
      
      zkLogWriteContext(c->log, err, func, "Failure to set aip information for [%s]",
                        aip_entity->name);
    } else 
      retcode = 1;
  }
  FREEIT(o_release);
  return retcode;
}


static int makeWormholeAndAipAdjustments(
  ZkError err,
  NidbContext c,
  FrEntIdExp *aip_entity,
  FrDBOTypeNsdbAip aipstate,
  time_t launchtime,
  time_t last_status_time,
  uint32_t maxin_thrput,
  uint32_t new_streams,
  uint32_t current_active_streams,
  uint32_t new_ports,
  uint32_t curr_used_ports,
  UINT64_T new_bytes_rx,
  UINT64_T new_bytes_tx
)
{
  static const char *func = "makeWormholeAndAipAdjustments";
  int                retcode = 0;
  uint32_t           o_maxin_thrput;
  uint32_t           o_new_streams;
  uint32_t           o_current_active_streams;
  uint32_t           o_new_ports;
  uint32_t           o_curr_used_ports;
  UINT64_T           o_new_bytes_rx;
  UINT64_T           o_new_bytes_tx;

  (void)launchtime;
  (void)last_status_time;
    
  if ( !aip_entity || ! aipstate ) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  } else if ( !frNsdbGetStatusForWormhole(err,
                                          aipstate,
                                          &o_maxin_thrput,
                                          &o_new_streams,
                                          &o_current_active_streams,
                                          &o_new_ports,
                                          &o_curr_used_ports, 
                                          &o_new_bytes_rx,
                                          &o_new_bytes_tx) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
    
  } else { 
    o_maxin_thrput           = maxin_thrput;
    o_new_streams           += new_streams;
    o_current_active_streams = current_active_streams;
    o_new_ports             += new_ports;
    o_curr_used_ports        = curr_used_ports;
    o_new_bytes_rx           = new_bytes_rx;
    o_new_bytes_tx           = new_bytes_tx;

    if ( ! frNsdbSetStatusForWormhole( err, 
                                       aipstate,
                                       o_maxin_thrput,
                                       o_new_streams,
                                       o_current_active_streams,
                                       o_new_ports,
                                       o_curr_used_ports, 
                                       o_new_bytes_rx,
                                       o_new_bytes_tx) ) {
      zkLogWriteContext(c->log, err, func, "Failure to set aip information for [%s]",
                        aip_entity->name);
    } else {
      retcode = 1;
    }
    if ( !retcode ) {
        zkLogWrite_va(c->log, "Failure to set link name information for [%d]",
                      aipstate->entid);
      } else if ( ! makeWormholeAdjustmentsToItsAip(err,
                                                    c, 
                                                    aip_entity,
                                                    aipstate, 
                                                    launchtime,
                                                    last_status_time,
                                                    maxin_thrput,
                                                    new_streams,
                                                    current_active_streams,
                                                    new_ports,
                                                    curr_used_ports,
                                                    new_bytes_rx,
                                                    new_bytes_tx) ) {
        zkLogWriteContext(c->log, err, func, "Failure to adjust the aip %s data with new link info",
                          aip_entity->name);
      } else {
        retcode = 1;
        /* TRANS- */
      }
  }
  return retcode;

}


static int makeLinkandAipAdjustments (
  ZkError err,
  NidbContext c,
  FrEntIdExp *aip_entity,
  FrDBOTypeNsdbAip aipstate,
  FrDBOTypeNsdbAip linkstate,
  /* time_t reptime,*/
  /*  uint32_t seqno, */
  const FrEntIdExp *nb_entity,
  time_t launchtime,  
  time_t last_change_time, 
  time_t last_status_time, 
  UINT64_T new_pkts_rx,
  UINT64_T new_pkts_tx,
  UINT64_T new_pkts_drop,
  UINT64_T new_bytes_rx,
  UINT64_T new_bytes_tx,
  UINT64_T new_bytes_drop)
{
  static const char *func = "makeLinkAndAipAdjustments";
  FrEntIdent nb_eid;
  int retcode = 0;
  int found = 0;

  time_t o_reptime = 0;
  time_t reptime = 0;
  /* uint32_t seqno, */
  FrEntIdExp *o_nb_entity = NULL;
  ZkBool o_up_flag;
  time_t o_launchtime = 0;
  time_t o_last_change_time = 0;
  time_t o_last_status_time = 0;
  UINT64_T o_new_pkts_rx = 0;
  UINT64_T o_new_pkts_tx = 0;
  UINT64_T o_new_pkts_drop = 0;
  UINT64_T o_new_bytes_rx = 0;
  UINT64_T o_new_bytes_tx = 0;
  UINT64_T o_new_bytes_drop = 0;

  (void) o_reptime;

  /*
   * Get my link
   * adjust my endpoint
   * set it back
   * get my aip
   * adjust totals
   * set it back
   */
  if ( !aip_entity || ! linkstate || ! nb_entity) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  } else if ( ! frNidbFindEntIdentUsingEntity(err, c, nb_entity, &nb_eid,
                                              &found)) {
  } else {
    int rcode = 0;
    if ( aipstate->entid < nb_eid ) {
      /* We are the low end */
      /* TRANS+ */
      rcode = frNsdbGetStatusForLink_lowEnd(err, linkstate, 
                                            (const FrEntIdExp **) &o_nb_entity, &o_up_flag, &o_launchtime,
                                            &o_last_change_time, &o_last_status_time,
                                            &o_new_pkts_rx, &o_new_pkts_tx, &o_new_pkts_drop,
                                            &o_new_bytes_rx, &o_new_bytes_tx,
                                            &o_new_bytes_drop);
    } else {
      /* We are the high end */
      rcode = frNsdbGetStatusForLink_highEnd(err, linkstate, 
                                             (const FrEntIdExp **) &o_nb_entity, &o_up_flag, &o_launchtime,
                                             &o_last_change_time, &o_last_status_time,
                                             &o_new_pkts_rx, &o_new_pkts_tx, &o_new_pkts_drop,
                                             &o_new_bytes_rx, &o_new_bytes_tx,
                                             &o_new_bytes_drop);
    }
    
    if ( ! rcode ) {
      zkLogWriteContext(c->log, err, func, "Failure to  extract link information");
      
    } else { /* FIX: Could make an entity sanity check here */
      /* Make the adjustments */
      /* o_launchtime = launchtime; */
      o_last_change_time = last_change_time; 
      o_last_status_time = last_status_time;
      o_new_pkts_rx = new_pkts_rx;
      o_new_pkts_tx = new_pkts_tx;
      o_new_pkts_drop = new_pkts_drop;
      o_new_bytes_rx = new_bytes_rx;
      o_new_bytes_tx = new_bytes_tx;
      o_new_bytes_drop = new_bytes_drop;
      
      if ( aipstate->entid < nb_eid ) {
        /* We are the low end */
        rcode = frNsdbSetStatusForLink_lowEnd(err, linkstate, 
                                              (const FrEntIdExp *) o_nb_entity, o_up_flag, o_launchtime,
                                              o_last_change_time, o_last_status_time,
                                              o_new_pkts_rx, o_new_pkts_tx, o_new_pkts_drop,
                                              o_new_bytes_rx, o_new_bytes_tx,
                                              o_new_bytes_drop);
      } else {
        /* We are the high end */
        rcode = frNsdbSetStatusForLink_highEnd(err, linkstate, 
                                              (const FrEntIdExp *) o_nb_entity, o_up_flag, o_launchtime,
                                              o_last_change_time, o_last_status_time,
                                              o_new_pkts_rx, o_new_pkts_tx, o_new_pkts_drop,
                                              o_new_bytes_rx, o_new_bytes_tx,
                                              o_new_bytes_drop);
      }

      if ( ! rcode ) {
        zkLogWriteContext(c->log, err, func, "Failure to set link name information for [%d]",
                      aipstate->entid);
      } else if ( ! makeLinkAdjustmentsToItsAip(err, c, aip_entity,
                                          aipstate, reptime, nb_entity,
                                          launchtime,
                                          last_change_time, last_status_time,
                                          new_pkts_rx, new_pkts_tx, new_pkts_drop,
                                          new_bytes_rx, new_bytes_tx,
                                          new_bytes_drop) ) {

        zkLogWriteContext(c->log, err, func, "Failure to adjust the aip %s data with new link info",
                          aip_entity->name);
      } else {
        retcode = 1;
        /* TRANS- */
      }
    }
  }
  /* TRANSx */
  return retcode;
}




/*
 * A simple rating factor, for now
 * Rating is from 0 to 255
 * Computed: 100 pts for uptime/downtime ratio
 *           100 pts for packet delivery
 *           50 points, 1 for each day up since last crash
 * Add 1 to the divisor to prevent division by 0 
 */

static int frNissAipComputeState(
  ZkError err,
  FrStatCtx *c,
  FrEntIdExp * aip_entity,
  FrDBOTypeNsdbAip oldstate,
  FrDBOTypeNsdbAip state
  )

{
  static const char *func = "frNissAipComputeState";

  /* 0 for the old , 1 for the new */
  time_t    a_reptime[2] = {0,0};
  time_t    a_launchtime[2] = {0,0};
  time_t    a_last_status_time[2] = {0,0};
  time_t    a_last_niqs_reload_time[2] = {0,0};
  uint32_t  a_new_client_connections[2] = {0,0};
  uint32_t  a_curr_connected_clients[2] = {0,0};
  UINT64_T  a_client_bytes_rx[2] = {0,0};
  UINT64_T  a_client_bytes_tx[2] = {0,0};
  UINT64_T  a_client_bytes_dropped[2] = {0,0};
  UINT64_T  a_client_pkts_rx[2] = {0,0};
  UINT64_T  a_client_pkts_tx[2] = {0,0};
  UINT64_T  a_client_pkts_dropped[2] = {0,0};
  UINT64_T  a_total_bytes_rx[2] = {0,0};
  UINT64_T  a_total_bytes_tx[2] = {0,0};
  UINT64_T  a_total_bytes_dropped[2] = {0,0};
  time_t    a_time_period[2] = {0,0};
  char     *a_release[2] = {NULL,NULL};
  int retcode = 0;

  if ( ! c || ! aip_entity || ! oldstate || ! state ) {
    zkErrorSet(err,func,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
    retcode =  0;
  } else if ( ! frNsdbGetStatusForAip( err, oldstate,
                                       &a_reptime[0],
                                       &a_launchtime[0],
                                       &a_last_status_time[0],
                                       &a_last_niqs_reload_time[0],
                                       &a_new_client_connections[0],
                                       &a_curr_connected_clients[0],
                                       &a_client_bytes_rx[0],
                                       &a_client_bytes_tx[0],
                                       &a_client_bytes_dropped[0],
                                       &a_client_pkts_rx[0],
                                       &a_client_pkts_tx[0],
                                       &a_client_pkts_dropped[0],
                                       &a_total_bytes_rx[0],
                                       &a_total_bytes_tx[0],
                                       &a_total_bytes_dropped[0],
                                       &a_time_period[0],
                                       &a_release[0]) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
  } else if ( ! frNsdbGetStatusForAip( err, state,
                                       &a_reptime[1],
                                       &a_launchtime[1],
                                       &a_last_status_time[1],
                                       &a_last_niqs_reload_time[1],
                                       &a_new_client_connections[1],
                                       &a_curr_connected_clients[1],
                                       &a_client_bytes_rx[1],
                                       &a_client_bytes_tx[1],
                                       &a_client_bytes_dropped[1],
                                       &a_client_pkts_rx[1],
                                       &a_client_pkts_tx[1],
                                       &a_client_pkts_dropped[1],
                                       &a_total_bytes_rx[1],
                                       &a_total_bytes_tx[1],
                                       &a_total_bytes_dropped[1],
                                       &a_time_period[1],
                                       &a_release[1]) ) {
    zkLogWriteContext(c->log, err, func, "Failure to extract aip information for [%s]",
                      aip_entity->name);
  } else {
    if (state->up_flag == 0) {
      /* The host was down. Compute down time */
      state->downtime += a_reptime[1] - a_reptime[0];
    } else {

      int up_ratio, deliv_ratio, reliab_idx, i;

    /* Simple uptime ratio */
      up_ratio =  (1-(state->downtime /
                      (a_reptime[1] - a_launchtime[1]+ 0.01))) * 100.0;

 
      deliv_ratio = (1 - (a_total_bytes_dropped[1] /
                          ((a_total_bytes_tx[1]+a_total_bytes_rx[1])/2 +.001)))
        * 100.0;

      reliab_idx =  ZK_MIN ((int) ((a_reptime[1] - a_launchtime[1] - state->downtime)
                                   /(60.0*60.0*24.0)), 50);

      if ( reliab_idx <= 0 ) {
        reliab_idx = 1;
      };
    
      /*
     * Use the previous rating as a dampening factor.
     */
      i = up_ratio + deliv_ratio + reliab_idx;
      zkLogWrite_va (c->log, "RATING computed to be [%d] (was %d)", i,
                     oldstate->entity_rating);
      zkLogWrite_va (c->log, "[up_ratio=%d] [deliv_ratio=%d] [reliab_idx=%d]",
                     up_ratio, deliv_ratio, reliab_idx);
    
      i = i - (i-oldstate->entity_rating)/2;
      zkLogWrite_va (c->log, "RATING adjusted to [%d] ", i);

      if (i < 0) {
        zkLogWrite_va (c->log, "RATING computation underflowed (%d)", i);
        i = 0;
      }
    
      if (i > 255) {
        zkLogWrite_va (c->log, "RATING computation overflowed (%d)", i);
        i = 255;
      }
      state->entity_rating = i;
    }
    retcode = 1;
  }

  FREEIT(a_release[0]);
  FREEIT(a_release[1]);
  return retcode;
}



/*
**------------------------------------------------------------------------------
** Public functions section
**------------------------------------------------------------------------------
*/

/**
 * Handle an update packet from an AIP
 * A truly messy piece of crap.
 */
int frNissHandleAip (
  ZkError err,
  FrStatCtx *c,
  FrStatUpd fsu
)
{
  static const char *funcname = "frNissHandleAip";
  size_t fsu_history = 0, ssz = 0, esz = 0;
  FrEntIdExp * owner_entity = NULL;
  char *release = NULL;
  size_t rsz = 0;

  FrStatsCtx fsc = NULL;
  time_t reptime = 0;
  uint32_t seqno = 0;
  time_t launchtime = 0;
  time_t last_niqs_reload_time = 0;
  time_t last_status_time = 0;
  uint32_t new_client_connections = 0;
  uint32_t curr_connected_clients = 0;
  UINT64_T client_bytes_rx = 0;
  UINT64_T client_bytes_tx = 0;
  UINT64_T client_bytes_dropped = 0;
  UINT64_T client_pkts_rx = 0;
  UINT64_T client_pkts_tx = 0;
  UINT64_T client_pkts_dropped = 0;
  
  FrDBOTypeEntityID eid_dbo = NULL;
  FrDBOTypeEntityID rem_eid_dbo = NULL;
  FrDBOTypeNsdbAip state = NULL;
  FrDBOTypeNsdbAip oldstate = NULL;
  int found = 0, replaced = 0;
  int new_entry_flag = 0; /* a brand new entry. Never seen this guy yet */
  int new_update_flag = 0; /* event we've gotten is more recentthan previously  */
  int state_changed = 0;
  int retcode = 0;
  int go_on = 0;

  /* 
   * Assumption is that the packet is valid at this point.
   */ 
  if ( ! c || ! fsu ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");

  }
  else if  (! frStatUpdEntity(err, fsu, (const FrEntIdExp **) &owner_entity) ) {
    zkLogWriteContext(c->log, err, funcname, "error with frStatUpdEntity");

  }
  else if ( ! frStatUpdGetReleaseString(err, fsu, &release, &rsz)) {
    zkLogWriteContext(c->log, err, funcname, "error with frStatUpdGetReleaseString");

  }
  else if ( ! frStatsNewWithNoSU(err, &fsc ) ||
            ! frStatsSetUpdate(err, fsc, fsu)) {
    zkLogWriteContext(c->log, err, funcname, "error with frStatsNewWithNoSU or frStatsSetUpdate");

  }
  else if ( ! frStatsReadAipMsg(err, fsc, &reptime, &seqno, &launchtime,
                                &last_niqs_reload_time, &last_status_time,
                                &new_client_connections, &curr_connected_clients,
                                &client_pkts_rx, &client_pkts_tx, &client_pkts_dropped,
                                &client_bytes_rx, &client_bytes_tx, &client_bytes_dropped) ) {

    zkLogWriteContext(c->log, err, funcname, "error with frStatsReadAipMsg");
  }
  else if (! frNissGetStateUsingEntity (err, c, owner_entity, 
                                          &state, &ssz, &eid_dbo, &esz, &found)) {
    zkLogWriteContext(c->log, err, funcname, "error with frNissGetStateUsingEntity");
  }
  else if (! zkDBODup (err, state, ssz, (void **) &oldstate) ) {
    zkLogWriteContext(c->log, err, funcname, "error with zkDBODup");
  }
  else if ( ! frNissEvRecCheckAndStore (err, c, eid_dbo->entid, FRSTAT_T_AIP,
                                          reptime, &new_entry_flag,
                                          &new_update_flag) ) {
    zkLogWriteContext(c->log, err, funcname, "error with frNissEvRecCheckAndStore");
  }
  else if ( ! frStatUpdGetHistorySize (err, fsu, &fsu_history) ) { 
    zkLogWriteContext(c->log, err, funcname, "error with frStatUpdGetHistorySize");
  }
  else if( ! found) {
    zkLogWrite_va(c->log, "Didn't find a state for the given entity %s",
                  owner_entity->name);
    retcode = 1;
    go_on = 1;
  }
  else {
    go_on = 1;
  }

  if (go_on) {
    go_on = 0;
    
    if (new_entry_flag) {
      state->reference_date = reptime;
      state->entity_rating = 0;
    }
      
    /* The event database has flagged this entry as a new one
     * It's possible that I get an out of order packet. Those should be
     * ignored, to avoid reply attacks especially
     */
    if (new_update_flag) {

      if ( state->up_flag != 1 ) {
        state_changed = 1;
        state->up_flag = 1;
      }
      if (new_entry_flag) {
        time_t t = time(NULL);
        /* Make time differential adjustments */
        if (state->uptime > (t - state->reference_date)) {
          state->reference_date = t - state->uptime;
        }
      }
      if (!makeAipAdjustments(err, c->nidb, owner_entity, state,
                              reptime, seqno, launchtime,
                              last_niqs_reload_time, last_status_time, 
                              new_client_connections, curr_connected_clients,
                              client_bytes_rx, client_bytes_tx,
                              client_bytes_dropped, client_pkts_rx,
                              client_pkts_tx, client_pkts_dropped, release)) {
            
        zkLogWriteContext(c->log, err, funcname,"Unable to make aip adjustments for [%s]",
                          owner_entity->name);
      } else {

        if ( new_entry_flag ) {
          zkLogWrite_va(c->log, "Welcome %s [aip] to the freedom network",
                        owner_entity->name);
          retcode = 1;
        }
        
        if ( ! new_entry_flag && oldstate &&
             ! frNissAipComputeState(err, c, owner_entity, oldstate, state) ) {
          zkLogWriteContext(c->log, err, funcname, "Welcome %s [aip] to the freedom network",
                            owner_entity->name);
          retcode = 0;

        } else {
          retcode = 1;
          /*
           * This will replace and record an event according to system time.
           * This is potentially a problem when 
           */
          if ( ! frNsdbReplace (err, c->nidb, state, state_changed, &replaced )) {
            retcode = 0;

          } else if ( ! replaced ) {
            zkLogWrite_va(c->log, "Problem replacing %s's state in database",
                          owner_entity->name);
          }
          else {
              zkLogWrite_va(c->log,
                            "Successfully updated aip [%s] seq.no: %u",
                            owner_entity->name, seqno);
          }
        }
      }
    } else { /* new_update_flag */
      zkLogWrite_va(c->log,
                    "Received an update packet from %s, that is older than"
                    " the last one I processed. Ignoring.",
                    owner_entity->name);
    retcode = 1;
    }
  }

  FREEIT(state);
  FREEIT(oldstate);
  FREEIT(eid_dbo);
  FREEIT(rem_eid_dbo);
  FREEIT(release);
  if (fsc) frStatsFree(err, &fsc);

  return retcode;
}



/*
**------------------------------------------------------------------------------
** Handle an update packet from a Link
**------------------------------------------------------------------------------
*/
int frNissHandleLink(
    ZkError     err,
    FrStatCtx * c,
    FrStatUpd   fsu
)
{
  int retcode = 0; 
  static const char *funcname = "frNissHandleLink";
  size_t  lsz = 0, lesz = 0, asz = 0, aesz = 0;
  FrEntIdExp * aip_entity = NULL;
  char *release = NULL;
  size_t rsz = 0;
    
  int found = 0;

  FrStatsCtx fsc = NULL;

  time_t            reptime;
  uint32_t          seqno;
  FrEntIdExp       *nb_entity = NULL;
  time_t            launchtime;
  time_t            last_change_time;
  time_t            last_status_time;
  UINT64_T          new_pkts_rx;
  UINT64_T          new_pkts_tx;
  UINT64_T          new_pkts_drop;
  UINT64_T          new_bytes_rx;
  UINT64_T          new_bytes_tx;
  UINT64_T          new_bytes_drop;
  ZkBool            link_up_flag;

  FrDBOTypeEntityID  link_dbo = NULL;
  FrDBOTypeEntityID  aip_dbo = NULL;
  FrDBOTypeEntityID  nb_dbo = NULL;
  FrDBOTypeNsdbAip   linkstate = NULL;
  FrDBOTypeNsdbAip   aipstate = NULL;
  FrDBOTypeNsdbAip   nbstate = NULL;
  FrEntIdExp *link_entity = NULL;

  int new_link_entry_flag = 0;
  int new_link_update_flag = 0;
  int new_aip_entry_flag = 0;
  int new_aip_update_flag = 0;

  int link_state_changed = 0;
  int aip_state_changed = 0;

  int replaced = 0;
  int go_on = 0;


  /* validate param */
  if ( ! c || ! fsu ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
  }
  /* get aip entity */
  else if ( ! frStatUpdEntity(err, fsu, (const FrEntIdExp **) &aip_entity) ) {
    zkLogWriteContext(c->log, err, funcname, "Missing required data in update packet");
  }
  /* get release string */
  else if ( ! frStatUpdGetReleaseString(err, fsu, &release, &rsz)) {
    zkLogWriteContext(c->log, err, funcname, "Could not find release string in update packet: bogus?");
  }
  /* create a status context */
  else if ( ! frStatsNewWithNoSU(err, &fsc) || 
            ! frStatsSetUpdate(err, fsc, fsu)) {
    zkLogWriteContext(c->log, err, funcname, "Trouble setting new status context");

  }  /* read the Link message */
  else if ( ! frStatsReadLinkMsg(err, fsc, &reptime, &seqno, &link_up_flag,
                                 (const FrEntIdExp **) &nb_entity, &launchtime,
                                 &last_change_time, &last_status_time,
                                 &new_pkts_rx, &new_pkts_tx, &new_pkts_drop, 
                                 &new_bytes_rx, &new_bytes_tx, &new_bytes_drop) ) {
    zkLogWriteContext(c->log, err, funcname, "Unable to extract Link information from packet");

  }
  else if ( ! nb_entity || ((nb_entity->type!=FRENT_TYPE_AIP)&&
                            (nb_entity->type!=FRENT_TYPE_COREAIP)
			   ) ) {
    zkLogWrite_va(c->log, "Error: Invalid neighbour entry (type = %s)",
                  nb_entity?frEntTypeGetStr(nb_entity->type):"Null entity pointer");
  }
  else if ( ! frNidbGetLinkEntityFromEntities (err, c->nidb,
                                               aip_entity, nb_entity,
                                               &link_entity) ) {
    zkLogWriteContext(c->log, err, funcname, "Problem determining the link name for [%s][%s]",
                      aip_entity->name, nb_entity->name);
  }
  else if ( ! frNissGetStateUsingEntity (err, c, link_entity, 
                                         &linkstate, &lsz,&link_dbo,&lesz,&found)) {
    zkLogWriteContext(c->log, err, funcname, "problem getting linkstate for entity");
  }

  else if ( ! frNissEvRecCheckAndStore (err, c, link_dbo->entid, FRSTAT_T_LINK,
                                        reptime, &new_link_entry_flag,
                                        &new_link_update_flag) ){
    zkLogWriteContext(c->log, err, funcname, "error checking event rec for link %d",
                  link_dbo->entid);

  }

  else if( ! found) {
    zkLogWrite_va(c->log, "Didn't find a linkstate for the given link entity");
    go_on = 1;
    /* Not really a system error, just means we have to create it */
  } else {
    go_on = 1;
  }    

  if ( go_on ) {
    go_on = 0;
    if ( ! frNissGetStateUsingEntity (err, c, aip_entity, 
                                      &aipstate, &asz, &aip_dbo, &aesz,&found)) {
      zkLogWriteContext(c->log, err, funcname, "problem getting state for aip entity");
      

    } else if ( ! frNissEvRecCheckAndStore (err, c, aip_dbo->entid, FRSTAT_T_AIP,
                                            reptime, &new_aip_entry_flag,
                                            &new_aip_update_flag) ){
      zkLogWriteContext(c->log, err, funcname, "error checking event rec for aip %d",
                        aip_dbo->entid);

    } else if( ! found) {
      zkLogWrite_va(c->log, "Didn't find a state for the given aip entity");
      /* Not really a system error, just means we have to create it */
      go_on = 1;
    } else {
      go_on = 1;
    }
  }

  if ( go_on ) {
    go_on = 0;
    /* With new link or aips, we have the allocated datastructures. We'll
       proceed to fill them up =:-) */
    
    if ( new_link_entry_flag) {
      aipstate->reference_date = reptime;
      aipstate->entity_rating = 0;
    }

    if (new_aip_entry_flag) {
      linkstate->reference_date = reptime;
      linkstate->entity_rating = 0;
    }
    
    /*
     * The event database has flagged this entry as a new one
     * It's possible that I get an out of order packet. Those should be
     * ignored, to avoid reply attacks especially
     */

    if (new_link_update_flag) {
      /* Only handle a new aip if we have to */
      if (new_aip_update_flag) {
        if ( aipstate->up_flag != 1 ) {
          aip_state_changed = 1;
          aipstate->up_flag = 1;
        }
        if (new_aip_entry_flag) {
          time_t t = time(NULL);
          /* Make time differential adjustments */
          if (aipstate->uptime > (t - aipstate->reference_date)) {
            aipstate->reference_date = t - aipstate->uptime;
          }
        }
      }

      if ( link_up_flag !=  linkstate->up_flag ) {
        link_state_changed = 1;
        linkstate->up_flag = link_up_flag;
      }
      if (new_link_entry_flag) {
        time_t t = time(NULL);
        /* Make time differential adjustments */
        if (linkstate->uptime > (t - linkstate->reference_date)) {
          linkstate->reference_date = t - linkstate->uptime;
        }
      }
          
      if (!makeLinkandAipAdjustments(err, c->nidb, aip_entity, aipstate, linkstate,
                                     (const FrEntIdExp *) nb_entity,
                                     launchtime, last_change_time,
                                     last_status_time, new_pkts_rx,
                                     new_pkts_tx, new_pkts_drop,
                                     new_bytes_rx, new_bytes_tx,
                                     new_bytes_drop)) {
        
        zkLogWriteContext(c->log, err, funcname, "Unable to make link adjustments for [%s] [%s]",
                          aip_entity->name, nb_entity->name);
      } else {
        
        if ( new_link_entry_flag ) {
          zkLogWrite_va(c->log, "Welcome %s [link] to the freedom network",
                        link_entity->name);
        }
        if ( new_aip_entry_flag ) {
          zkLogWrite_va(c->log, "Welcome %s [aip] to the freedom network",
                        aip_entity->name);
        }
        
        /*
         * This will replace and record an event according to system time.
         */
        if ( ! frNsdbReplace (err, c->nidb, linkstate, link_state_changed, &replaced )) {
          zkLogWriteContext(c->log, err, funcname, "error with frNsdbReplace");
          
        } else if ( ! replaced ) {
          zkLogWrite_va(c->log, "Problem replacing %s's link state to %s in database",
                        aip_entity->name, nb_entity->name);
          
        } else if ( ! frNsdbReplace (err, c->nidb, aipstate, aip_state_changed, &replaced )) {
          zkLogWriteContext(c->log, err, funcname, "error with frNsdbReplace");
          
        } else if ( ! replaced ) {
          zkLogWrite_va(c->log, "Problem replacing %s's state in database",
                        aip_entity->name);
        } else {
          go_on = 1;
          retcode = 1;
          zkLogWrite_va(c->log,
                        "Successfully updated link [%s<->%s] seq.no: %u",
                        aip_entity->name, nb_entity->name, seqno);
                      
        }
      }
    } else { /* new_link_update_flag */
      zkLogWrite_va(c->log,
                    "Received an update packet from %s, that is older than"
                    " the last one I processed. Ignoring.",
                    aip_entity->name);
      go_on = 1;
      retcode = 1;
    }
  }
  

  if (!retcode) {
    zkLogWrite_va(c->log,
	"Could not properly handle %s Link update", aip_entity->name);
  }

  /* memory management */
  if (link_entity) frEntFree(err, &link_entity);
  FREEIT(linkstate);
  FREEIT(link_dbo);
  FREEIT(aipstate);
  FREEIT(aip_dbo);
  FREEIT(nbstate);
  FREEIT(nb_dbo);
  FREEIT(release);
  if (fsc) frStatsFree(err, &fsc);

  return retcode;
}


/*
**------------------------------------------------------------------------------
** Handle an update packet from a Wormhole
**------------------------------------------------------------------------------
*/
int frNissHandleWormhole(
                         ZkError     err,
                         FrStatCtx * c,
                         FrStatUpd   fsu
                         )
{
  int                retcode = 0; 
  static const char *funcname = "frNissHandleWormhole";
  size_t             asz = 0;
  size_t             aesz = 0;
  FrEntIdExp        *aip_entity = NULL;
  char              *release = NULL;
  size_t             rsz = 0;

  int                found = 0;

  FrStatsCtx         fsc = NULL;

  time_t             reptime;
  uint32_t           seqno;
  time_t             launchtime;
  time_t             last_status_time;

  uint32_t           maxin_thrput;
  uint32_t           new_streams;
  uint32_t           current_active_streams;
  uint32_t           new_ports;
  uint32_t           curr_used_ports;
  UINT64_T           new_bytes_rx;
  UINT64_T           new_bytes_tx;

  FrDBOTypeEntityID  aip_dbo = NULL;
  FrDBOTypeNsdbAip   aipstate = NULL;

  int                new_aip_entry_flag = 0;
  int                new_aip_update_flag = 0;

  int                aip_state_changed = 0;
  int                replaced = 0;
  int                go_on = 0;


  /* validate param */
  if ( ! c || ! fsu ) 
    {
      zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
    }
  /* get aip entity */
  else if ( ! frStatUpdEntity(err, fsu, (const FrEntIdExp **) &aip_entity) ) 
    {
      zkLogWriteContext(c->log, err, funcname, "Missing required data in update packet");
    }
  /* get release string */
  else if ( ! frStatUpdGetReleaseString(err, fsu, &release, &rsz)) 
    {
      zkLogWriteContext(c->log, err, funcname, "Could not find release string in update packet: bogus?");
    }
  /* create a status context */
  else if ( ! frStatsNewWithNoSU(err, &fsc) || 
            ! frStatsSetUpdate(err, fsc, fsu)) 
    {
      zkLogWriteContext(c->log, err, funcname, "Trouble setting new status context");

    }  /* read the Wormhole message */
  else if ( !  frStatsReadWormholeMsg(err, 
                                     fsc, 
                                     &reptime, 
                                     &seqno, 
                                     &launchtime,       /* not used */
                                     &last_status_time, /* not used */
                                     &maxin_thrput,
                                     &new_streams,
                                     &current_active_streams,
                                     &new_ports,
                                     &curr_used_ports, 
                                     &new_bytes_rx,
                                     &new_bytes_tx) )
    {
      zkLogWriteContext(c->log, err, funcname, "Unable to extract Wormhole information from packet");

    }
  else if ( ! frNissGetStateUsingEntity (err, 
                                         c,
                                         aip_entity, 
                                         &aipstate, 
                                         &asz,
                                         &aip_dbo,
                                         &aesz,
                                         &found)) 
    {
      zkLogWriteContext(c->log, err, funcname, "problem getting wormholestate for entity");
    }
  else if ( ! frNissEvRecCheckAndStore (err, 
                                        c, 
                                        aip_dbo->entid, 
                                        FRSTAT_T_WORMHOLE,
                                        reptime, 
                                        &new_aip_entry_flag,
                                        &new_aip_update_flag) )
    {
      zkLogWriteContext(c->log, err, funcname, "error checking event rec for wormhole %d",
                    aip_dbo->entid);

    } 
  else if( ! found) 
    {
      zkLogWrite_va(c->log, "Didn't find a wormholestate for the given wormhole entity");
      go_on = 1;
      /* Not really a system error, just means we have to create it */
    }
  else 
    {
      go_on = 1;
    }    

  if ( go_on )
    {
      go_on = 0;
      /* With new wormhole or aips, we have the allocated datastructures. We'll
         proceed to fill them up =:-) */

      if (new_aip_entry_flag) 
        {
          aipstate->reference_date = reptime;
          aipstate->entity_rating = 0;
        }

      /*
       * The event database has flagged this entry as a new one
       * It's possible that I get an out of order packet. Those should be
       * ignored, to avoid reply attacks especially
       */

      if (new_aip_update_flag) 
        {
          if (!makeWormholeAndAipAdjustments(err, 
                                             c->nidb, 
                                             aip_entity, 
                                             aipstate, 
                                             launchtime,       /* not used */
                                             last_status_time, /* not used */
                                             maxin_thrput,
                                             new_streams,
                                             current_active_streams,
                                             new_ports,
                                             curr_used_ports, 
                                             new_bytes_rx,
                                             new_bytes_tx) )
            {

              zkLogWriteContext(c->log, err, funcname, "Unable to make wormhole adjustments for [%s]",
                            aip_entity->name);
            }
          else 
            {

              if ( new_aip_entry_flag ) 
                {
                  zkLogWrite_va(c->log, "Welcome %s [aip] to the freedom network",
                                aip_entity->name);
                }

              /*
               * This will replace and record an event according to system time.
               */
              else if ( ! frNsdbReplace (err, 
                                         c->nidb, 
                                         aipstate, 
                                         aip_state_changed, 
                                         &replaced )) 
                {
                  zkLogWriteContext(c->log, err, funcname, "error with frNsdbReplace");
                } 
              else if ( ! replaced ) 
                {
                  zkLogWrite_va(c->log, "Problem replacing %s's state in database",
                                aip_entity->name);
                } 
              else
                {
                  retcode = 1;
                  zkLogWrite_va(c->log,
                                "Successfully updated wormhole [%s], seq.no: %u",
                                aip_entity->name, seqno);

                }
            }
        }
      else
        { /* new_wormhole_update_flag */
          zkLogWrite_va(c->log,
                        "Received an update packet from %s, that is older than"
                        " the last one I processed. Ignoring.",
                        aip_entity->name);
          retcode = 1;
        }
    }

  /* memory management */
  FREEIT(aipstate);
  FREEIT(aip_dbo);
  FREEIT(release);
  if (fsc) frStatsFree(err, &fsc);

  return retcode;
}
