/*
 * The Network Information State Service
 * The AIP update
 *
 * $Id: nissEvRec.c,v 1.6 2000/03/09 16:35:58 danielf Exp $
 * Luc Boulianne <luc@zks.net>
 */

#ifdef _NISS_REPLAY_FEATURE_
#  include "nissReplay.h"
#endif

#include "zks.h"

#include <time.h>
#include <sys/types.h>
#include "zkHash.h"
#include "zkDebug.h"
#include "frStatPkt.h"
#include "frStatUpd.h"
#include "fr_entid.h"
#include "fr_nidb.h"
#include "fr_niss.h"
#include "frnissd.h"


struct FrNissEvRec_T {
  FrEntIdent eid;
  FrStatUpdTypes type; 
  time_t last_update;
};

typedef struct FrNissEvRec_T *FrNissEventRecord;

/**
 * Check if we've seen a particular parameter from a remote AIP
 * If we have, record this sighting
 */
int frNissEvRecCheckAndStore (
  ZkError err,
  FrStatCtx *ctx,
  FrEntIdent eid,
  FrStatUpdTypes type,
  time_t last_update,
  int *new_entry_flag,		/* This is the first time I've heard from you */
  int *new_update_flag          /* This event is more recent that what I had  */
  )
{
  static const char *funcname = "frNissEvRecCheckAndStore";
  struct FrNissEvRec_T evRecKey;
  struct FrNissEvRec_T evRecData, *ev = NULL;
  size_t ds = 0;

  if (! eid || ! new_entry_flag ) {
    zkErrorSet(err,funcname,ZKERR,ZKMOD,ZKENULLPTR,"Null parameters passed");
    return 0;
  }

  memset(&evRecKey, 0, sizeof(evRecKey));
  memset(&evRecKey, 0, sizeof(evRecData));
    
  /* This database entry uses the FrNissEvRec as the key and the data for this
   * persistent database. For the key, the last_update_time is set to 0.  The
   * data portion is the same FrNissEvRec with the last_update time set to the
   * appropriate value.
   */
  evRecKey.eid = eid;
  evRecKey.type = type;

  if ( ! zkAAGet (err, ctx->evRecDB, (void *) &evRecKey, sizeof(evRecKey),
                  (void **) &ev, &ds) ) {
    zkLogWriteContext (ctx->log, err, funcname, "Call to zkAAGet failed");
    return 0;
  }

  if ( ds == 0 ) {
    /* A new entry */
    evRecData = evRecKey;
    evRecData.last_update = last_update;
    *new_entry_flag = 1;
    *new_update_flag = 1;
  } else {
    /* A previous record exists */
    if ( ev->last_update <= last_update ) {
      /* This is an update */
      evRecData = evRecKey;
      evRecData.last_update = last_update;
      *new_update_flag = 1;
    } else {
      *new_update_flag = 0;
    }
  }
  *new_update_flag = 1; /* Hack */
  if (*new_update_flag) {
    if ( ! zkAAPut (err, ctx->evRecDB, (void *) &evRecKey, sizeof(evRecKey),
                    (void *) &evRecData, sizeof(evRecData)) ) {
      zkLogWriteContext (ctx->log, err, funcname, "Call to zkAAPut failed");
      return 0;
    }
  }
  return 1;
}
  
