/*
 * $Id: nissd.c,v 1.18 2000/11/24 14:59:30 marcp Exp $
 *
 * Kent Tse <kent@zks.net>
 * Luc Boulianne <luc@zks.net>
 */

#ifdef _NISS_REPLAY_FEATURE_
#  include "nissReplay.h"
#endif

#include "zks.h"

#include <arpa/inet.h>
#include "fr_niss.h"
#include "frnissd.h"
#include "signal.h"
#include "zkSignal.h"
#include "frStatPkt.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <frNetworkDefaults.h>
#include <zkDebug.h>
#include "nissCmd.h"
#include "fr_niss.h"
#include "zkFile.h"

#define DEBUG_STAT_CRC 0


#if DEBUG_STAT_CRC
/*
 *  bug: this code taken from crc.c in frpktfmt/src.
 */

/*
 *  CRC 0102010
 */
static unsigned short crc_table[256] = {
  0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
  0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
  0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
  0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
  0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
  0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
  0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
  0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
  0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
  0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
  0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
  0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
  0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
  0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
  0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
  0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
  0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
  0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
  0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
  0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
  0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
  0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
  0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
  0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
  0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
  0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
  0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
  0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
  0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
  0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
  0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
  0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
};


static unsigned short cksum(unsigned char *s, int n)
{
  unsigned int crc;

  for (crc = 0; n > 0; n--) {
    crc = crc_table[(crc ^ *s++) & 0xff] ^ (crc >> 8);
  }

  return crc;
}
#endif


static volatile sig_atomic_t running = 1;
static volatile sig_atomic_t chgkey = 0;
static volatile sig_atomic_t RotateLog = 0;
static volatile sig_atomic_t AllowConnections = 1;


#define max(a, b) ((a < b) ? b : a)

void frStatdSigHandler(int sig)
{
  switch (sig) {
  case SIGINT:
  case SIGTERM:
    zkDebug ("INT/TERM signal caught. Shutting down.");
    running = 0;
    break;

  case SIGUSR1:
    /* Change keys */
    zkDebug ("USR1 signal caught. Changing keys.");
    chgkey = 1;
    break;

  case SIGUSR2:
    /* Change keys */
    zkDebug ("USR1 signal caught. Changing keys.");
    AllowConnections = !AllowConnections;
    break;

  case SIGHUP:
    /* Change keys */
    zkDebug ("HUP signal caught. Need to reopen log files.");
    RotateLog = 1;
    break;

  default:
    zkDebug ("Signal %d NOT Handled.", sig);
    return;
  }
}


#ifdef _NISS_REPLAY_FEATURE_

/*
 * Faked function. Does nothing on replay except parameter validation
 * and returns a copy of the input packet. (Data comes in in clear)
 */
static int frStatdProcessPacket(ZkError err, FrStatCtx *c, char *pkt,
                                size_t psz, void **stbuf, size_t *stbsz)
{
  const char *func = "frStatdProcessPacket";
  void *dbuf = 0;

  if (!c || !pkt || !stbuf || !stbsz) {
    zkErrorSet(err, func, ZKERR, ZKMSTATD, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!zkMemAlloc(err,&dbuf,psz)) {
    return 0;
  }

  memcpy(dbuf, pkt, psz);
  
  *stbuf = dbuf; dbuf = 0; /* the caller now owns it */
  *stbsz = psz;

  return 1;
}

#else /* _NISS_REPLAY_FEATURE_ */

/*
 *  Process the wire format stat packet contained in `buf'.
 *  
 *  The entire packet is decrypted with our private key, revealing a buffer
 *  signed by the sending AIP or MAIP.  If the signature is valid the
 *  cleartext data is returned to the caller, via `stbuf'.  This cleartext
 *  buffer is the original stats packet.
 */
static int frStatdProcessPacket(ZkError err, FrStatCtx *c, char *pkt,
                                size_t psz, void **stbuf, size_t *stbsz)
{
  FrCryptCrypt encrKey;
  FrCryptPubKey sigKey;
  FrEntIdExp entity;
  FrPktType type;
  const char *func = "frStatdProcessPacket";
  int id, ret = 0, valid;
  size_t bsz, dsz, ssz;
  uint32_t ttl;
  unsigned int encrKeyVers, keyVers, version;
  void *buf = 0; /* pointer to cleartext, after signature removal */
  void *dbuf = 0; /* pointer to cleartext data portion of packet */
  void *sbuf = 0; /* pointer to cleartext, signed buffer */

  if (!c || !pkt || !stbuf || !stbsz) {
    zkErrorSet(err, func, ZKERR, ZKMSTATD, ZKENULLPTR, "NULL param(s)");
    goto done;
  }

  /*
   *  Decrypt the packet, then check the signature.
   */

  if ( ! (frSecPubKeyGetCryptKeyVersion(err, pkt, psz, &encrKeyVers) &&
          frSecKeyChainGetVersion(err, c->kc, c->cryptHandle, encrKeyVers,
                                  &encrKey) &&
          frSecPubKeyDecryptData(err, encrKey, pkt, psz, &sbuf, &ssz) &&
          frStatPktGetSigInfo(err, sbuf, ssz, &entity, &keyVers) &&
          frKeyQryFetchPubKey(err, c->kq, keyVers, FRCRYPT_KEYTYPE_SIGNING, 
                              entity, c->kh, c->kqretries, c->kqtimeout,
                              &sigKey) &&
          frSecSigVerify(err, sigKey, sbuf, ssz, &valid))) {
    zkLogWriteContext (c->log, err, func, "Could not get key / verify sig");
    goto done;
  }

  if ( ! valid) {
    char entstr[100];

    if ( ! frEntSnprint(err, entstr, sizeof entstr, &entity)) {
      zkLogWriteContext (c->log, err, func, "Call to frEntSnprint failed");
      snprintf(entstr, sizeof entstr, "???");
    }
    zkErrorSet_va(err, func, ZKERR, ZKMSTATD, ZKEINVAL,
                  "invalid signature (from %s) over stats packet", entstr);
    goto done;
  }
  
  if ( ! frSecSigRemove(err, sbuf, ssz, &buf, &bsz)) {
    zkLogWriteContext (c->log, err, func, "Call to frSecSigRemove failed");
    goto done;
  }

  if ( ! frStatPktGetUpdInfo(err, buf, bsz, &version, &type, &ttl, &id,
                             &dbuf, &dsz)) {
    zkLogWriteContext (c->log, err, func, "Call to frStatPktGetUpdInfo failed");
    goto done;
  }

  if (type != FRPKTTYPE_NIDB_UPDATE) {
    zkErrorSet_va(err, func, ZKERR, ZKMSTATD, ZKEINVAL, "Invalid type [%d]",
                  type);
    goto done;
  }

  if (c->debug) {
    fprintf(stderr, "%s: Packet signature checked\n", c->progname);
    fprintf(stderr, "--> Entity %s / Type %d\n", entity.name, entity.type);
    fprintf(stderr, "--> Version %d / TTL %d / ID %d\n", version, ttl, id);
  }
  
  *stbuf = dbuf; dbuf = 0; /* the caller now owns it */
  *stbsz = dsz;

  ret = 1;
  
 done:

  if (buf) { memset(buf, 0, bsz); free(buf); }
  if (sbuf) { memset(sbuf, 0, ssz); free(sbuf); }

  return ret;
}

#endif /* _NISS_REPLAY_FEATURE_ */

/**
 * Send to the host server (host IP and UDP port) a packet of type to request
 * an update request.  
 */
static int frStatdPollServer(ZkError err, FrStatCtx *c, 
                             struct sockaddr_in host, FrPktType type)
{
  FrCryptPubKey mySig = NULL;
  const char *func = "frStatdPollServer";
  int ret = 0;
  size_t dsz;
  void *data = NULL;

  if (!c) {
    zkErrorSet(err, func, ZKERR, ZKMSTATD, ZKENULLPTR, "NULL ctx");
    goto done;
  }

  switch (type) {
  case FRPKTTYPE_NIDB_UPD_REQ:
    if (c->debug) {
      zkLogWriteContext (c->log, err, func, "Requesting an UPDATE from %s:%d", 
              inet_ntoa(host.sin_addr), ntohs(host.sin_port));
    }
    break;

  case FRPKTTYPE_NIDB_UPD_REGTOP:
    if (c->debug) {
      zkLogWriteContext (c->log, err, func, "Requesting Net Topology Refresh from %s:%d", 
              inet_ntoa(host.sin_addr), ntohs(host.sin_port));
    }
    break;

  case FRPKTTYPE_NIDB_UPD_URGTOP:
    if (c->debug) {
      zkLogWriteContext (c->log, err, func, "Requesting URGENT Net Topology Refresh from %s:%d", 
              inet_ntoa(host.sin_addr), ntohs(host.sin_port));
    }
    break;

  default:
    zkErrorSet_va(err, func, ZKERR, ZKMSTATD, ZKEINVAL, "Invalid %d type",
                  type);
    goto done;
  }

  if (c->sigHandle < 0) {
    zkErrorSet(err, func, ZKERR, ZKMSTATD, ZKENULLPTR,
               "Missing signature keys in my key chain");
    goto done;
  }

  if (!frSecKeyChainGetVersion(err, c->kc, c->sigHandle, 0, &mySig)) {
    zkLogWriteContext (c->log, err, func, "Call to frSecKeyChainGetVersion failed"); 
    goto done;
  }

  if (!frStatPktUpdateReq(err, type, 1, 1, 1, mySig, &data, &dsz)) {
    zkLogWriteContext (c->log, err, func, "Call to frStatPktUpdateReq failed"); 
    goto done;
  }

  if (sendto(c->myUdpFd, data, dsz, 0,
             (struct sockaddr *) &host, sizeof(host)) != (int)dsz) {
    zkErrorSet_va(err, func, ZKERR, ZKMSTATD, errno,
                  "error %d from sendto() writing %lu bytes: %s",
                  errno, (unsigned long)dsz, strerror(errno));
    goto done;
  }

  ret = 1;
  
 done:
  
  if (data) { free(data); }

  return ret;
}


int frNissDSendEntityEvent(ZkError err, FrStatCtx *c, uint32_t r_ipaddr,
				 uint16_t r_port, int event)
{
  static const char *funcname = "frNissDSendEntityEvent";
  struct sockaddr_in remote;

  if (!c) {
    zkErrorSet(err, funcname, ZKERR, ZKMSTATD, ZKENULLPTR, "NULL ctx");
    return 0;
  }

  remote.sin_family = AF_INET;
  remote.sin_port = htons(r_port);
  remote.sin_addr.s_addr = r_ipaddr;

  if (!frStatdPollServer(err, c, remote, event)) {
    zkLogWriteContext (c->log, err, funcname, "Call to frStatdPollServer failed"); 
    return 0;
  }
  return 1;
}


static int frNissHandleUdpPkt(ZkError err, FrStatCtx *c)
{
  static const char *funcname = "frNissHandleUdpPkt";
  char *stbuf = NULL;
  char buf[64 * 1024]; /* Max UDP size */
  int nbytes;
  int retcode = 0;
  size_t bsz = sizeof(buf);
  size_t stbsz = 0;
  struct sockaddr_in remote;
  size_t rLen = sizeof(remote);
  
  if (!c) {
    zkErrorSet(err, funcname, ZKERR, ZKMLAIP, ZKENULLPTR, "NULL param");
    return 1;
  }

  if ((nbytes = recvfrom(c->myUdpFd, buf, bsz, 0,
			 (struct sockaddr *) &remote, &rLen)) < 0) {
    if (errno == EINTR) {
      retcode = 1;
      zkLogWriteContext (c->log, err, funcname, "recvfrom(%s) was interrupted (EINTR)", strerror(errno));
      goto cleanup;
    } else {
      zkLogWriteContext (c->log, err, funcname, "recvfrom(%s) failed", strerror(errno));
      retcode = 1 ;
      goto cleanup;
      
    }
  }
  if (nbytes > 0) {

    if (c->debug) {
      zkLogWriteContext (c->log, err, funcname, "Received %d bytes from %s:%d.\n", nbytes,
		    inet_ntoa(remote.sin_addr), ntohs(remote.sin_port));
    }

    /* Check signature on the packet */
    if (!frStatdProcessPacket(err, c, buf, nbytes, (void **) &stbuf,
			      &stbsz)) {
      zkLogWriteContext (c->log, err, funcname, "Invalid signature from %s:%hu\n",
		    inet_ntoa(remote.sin_addr), ntohs(remote.sin_port));
      retcode = 1;
      goto cleanup;
    }
#if 0 /* Bug 6121: Leaving this in case we wish to reuse this feature. */
    /* Write out the buffer that has been received in the file */
    if (c->tempdatafile) {
      if (!zkFileRecordData(err, c->tempdatafile, stbuf, stbsz,
                            NULL, NULL)) {
        zkLogWriteContext (c->log, err, funcname, "zkFileRecordData() failed");
      }
#if DEBUG_STAT_CRC
      fprintf(stderr, "*** %s: %lu bytes, crc = %hu.\n",
              funcname, (unsigned long)stbsz, cksum(stbuf, stbsz));
#endif
    }
#endif
    /* Send the packet to NIDB */
    if (!frNissHandleUpdateBuffer (err, c, c->bucket, stbuf, stbsz)) {
      zkLogWriteContext (c->log, err, funcname, "frNissHandleUpdateBuffer() failed");
      retcode = 1;
      goto cleanup;
    }
    
    /* And (for now) send it to frStatUpd stuff */
    if (!frStatBucketAddUpdateBuffer(err, c->bucket, stbuf, &stbsz)) {
      zkLogWriteContext (c->log, err, funcname, "frStatBucketAddUpdateBuffer() failed");
      retcode = 1;
      goto cleanup;
    }

  } else {
    zkLogWriteContext (c->log, err, funcname, "read no data from recvfrom()");
    retcode = 1;
    goto cleanup;
  }
  retcode = 1;
  
 cleanup:
  if (stbuf) {
    free(stbuf);
    stbuf = NULL;
  }
  return retcode;
  
}

/**
 * Service each of the descriptors that are ready to be read.  Add to the 
 * fset or remove descriptors as needed.
 */
static int frNissHandleEvents(ZkError err, FrStatCtx *c, fd_set *rset, 
                            fd_set *fset, int n)
{
  static const char *funcname = "frNissHandleEvents";
  int i;
  int fd;
  fd_set set;
  int eof = 0;
  (void) funcname;

  if (!c || !fset || !rset || n < 1) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD, ZKENULLPTR, "params no good");
    return 0;
  }

  set = *rset;
  for (i = 0; i < n; i++) {
    fd = -1;

    if ((c->cmdListenFd > 0) && FD_ISSET(c->cmdListenFd, &set)) {
      /* Set up my accept() and mark that we won't accept another connection */
      fd = c->cmdListenFd;

      if (c->cmdOpen) {
        int s = -1;
        /* Already a connection, printout the refusal an close 
           the descriptor */
        if (!frNissCmdAccept(err, c->cmdListenFd, c->cmdRefused, 
                           strlen(c->cmdRefused), &s)) {
          zkLogWriteContext (c->log, err, funcname, "Call #1 to frNissCmdAccept failed.");
          return 0;
        }
        close(s);               /* Close it */
      } else {
        if (!frNissCmdAccept(err, c->cmdListenFd, c->cmdWelcome, 
                           strlen(c->cmdWelcome), &c->cmdFd)) {
          zkLogWriteContext (c->log, err, funcname, "Call #2 to frNissCmdAccept failed.");
          return 0;
        }
        c->cmdOpen = 1;
        FD_SET(c->cmdFd, fset);
      }
      
    } else if ((c->cmdFd > 0) && FD_ISSET(c->cmdFd, &set)) {

      fd = c->cmdFd;
      if (!frNissCmdInterpreter(err, c, &eof)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissCmdInterpreter failed.");
        return 0;
      }
      
      if (eof) {
        FD_CLR(c->cmdFd, fset);
        close(c->cmdFd);
        c->cmdFd = -1;
        c->cmdOpen = 0;
      }
      
    } else if ((c->myUdpFd > 0) && FD_ISSET(c->myUdpFd, &set)) {
      fd = c->myUdpFd;

      if (!frNissHandleUdpPkt(err, c)) {
        zkLogWriteContext (c->log, err, funcname, "Call to frNissHandleUdpPkt failed.");
        return 0;
      }

    } else {
      zkLogWriteContext (c->log, err, funcname, "Unknown event error");
    }
    
    /* fd will be set to the descriptor that was just serviced from fd_set */
    if ( fd != -1 ) {
      FD_CLR(fd, &set);
    }
  }

  return 1;
}

static int computeMaxFd(ZkError err, FrStatCtx *c, int *high)
{
  static const char *funcname = "computeMaxFd";

  zkErrorClear(err);

  if (!c || !high) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD, ZKENULLPTR, "param is NULL");
    return 0;
  }
  
  *high = 0;
  if (c->cmdListenFd > 0) { *high = c->cmdListenFd; }
  if (c->myUdpFd > 0) { *high = max(*high, c->myUdpFd); }
  if (c->cmdFd > 0) { *high = max(*high, c->cmdFd); }

  return 1;
}

/* Add all descriptors of my context into the fd_set */
static int addAllCtxFds(ZkError err, FrStatCtx *c, fd_set *fset)
{
  static const char *funcname = "addAllCtxFds";

  zkErrorClear(err);

  if (!c || !fset) {
    zkErrorSet(err, funcname, ZKERR, ZKMOD, ZKENULLPTR, "param is NULL");
    return 0;
  }
  
  if (c->cmdFd > 0) FD_SET(c->cmdFd, fset);
  if (c->cmdListenFd > 0) FD_SET(c->cmdListenFd, fset);
  if (c->myUdpFd > 0) FD_SET(c->myUdpFd, fset);

  return 1;
}

/**
 * The main event loop.
 */
int frNissEventLoop(ZkError err, FrStatCtx *c)
{
  static const char *funcname = "frNissEventLoop";
  fd_set rset, orset;
  int highFd = -1;
  int nsel = -1;
  struct timeval timeout = {1, 0}; /* 1 second timeout */
    
  FD_ZERO(&rset);
  FD_ZERO(&orset);
  if (!addAllCtxFds(err, c, &orset)) {
    zkLogWriteContext (c->log, err, funcname, "Call to addAllCtxFds failed.");
    return 0;
  }

  while (running) {

    if (chgkey) {
      /* If we've been signaled, reload the keys */
      if (!frSecKeyChainReloadKeys(err, c->kc)) {
        zkLogWriteContext (c->log, err, funcname, "Error: problem reloading keys");
        return 0;
      }
      chgkey = 0;
    }

    if (RotateLog) {
      if (!zkLogReopenFile(c->log, c->logfile)) {
        zkLogWriteContext (c->log, err, funcname, "Call to zkLogReopenFile failed");
      }
      RotateLog = 0;
    }

    if (!AllowConnections) {
#warning Need to handle server idling better 
      zkLogWriteContext (c->log, err, funcname, "NOTICE: request to idle server is unimplemented.");
      AllowConnections = !AllowConnections;
    }

    rset = orset;
    if (!computeMaxFd(err, c, &highFd)) {
      zkLogWriteContext (c->log, err, funcname, "Error: Problem computing MaxFD");
      return 0;
    }
    if ((nsel = 
         zkSelectUpdateTimeout(err, highFd + 1, &rset, 0, 0, &timeout)) < 0) {
      if (errno != EINTR) {  
        zkLogWriteContext (c->log, err, funcname, "Error: zkSelectUpdateTimeout (errno=%d)",
                       errno);
        return 0;
      } else {               
        continue;
      }
    } else if (nsel == 0) {
      timeout.tv_sec = 1;
      timeout.tv_usec = 0;
    } else {
      /* Handle each of the descriptors */
      if (!frNissHandleEvents(err, c, &rset, &orset, nsel)) {
        zkLogWriteContext (c->log, err, funcname, "Error: Problem Handling event");
        return 0;
      }
    }
  }
  return 1;  
}
