#ifndef NNTP_H_
#define NNTP_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define NNTPSERVER_VERSION_STR "Zero-Knowledge NNTP server V1.0 ("__DATE__", "__TIME__")"

                          /* Hostname, version, posting ok */
#define NNTPSERVER_BANNER "201 %s %s ready %s\r\n"
#define NNTPSERVER_LIST "215 Newsgroups in form \"group high low flags\"\r\n"
#define NNTPSERVER_NEWGROUPS "231 Newsgroups in form \"group high low flags\"\r\n"
#define NNTPSERVER_GOODBYE "205 Goodbye\r\n"
#define NNTPSERVER_FAIL "500 Command not recognized\r\n"
#define NNTPSERVER_TIMEOUT "205 Connection timed out - Goodbye\r\n"
#define NNTPSERVER_ERROR "503 Server error\r\n"
#define NNTPSERVER_USAGE "501 Usage: %s\r\n"
#define NNTPSERVER_NOTIMPL "500 Command [%s] not implemented\r\n"
#define NNTPSERVER_HELP "100 Help text follows\r\n\tHELP\r\n\tLIST\r\n\tNEWGROUPS yymmdd hhmmss [\"GMT\"]\r\n\tQUIT\r\n"
#define NNTPSERVER_TOOMANYCONN "502 Too many connections at this time, try again later\r\n"

  typedef struct fr_nntp_ctx {
    int debug;
    ZkError err;
    ZkLog log;
    ZkConfig config;
    struct sockaddr_in server;  /* Server IP/port */
    const char *progname;       /* Program name */
    const char *configfile;     /* Config file */
    const char *pidfile;
    FILE *pidFptr;              /* PID fptr */
    const char *logfile;
    const char *name;
    const char *activefile;     /* NG Active pathname */
    const char *activetimes;    /* NG Active pathname */
    int maxconn;
    int maxconntime;
  } FrNNTPCtx;

  extern int frNNTPServer(ZkError, FrNNTPCtx *);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* NNTP_H_ */
