/*
 * $Id: cmdniqs.c,v 1.1 1999/04/09 18:35:19 philippe Exp $
 */

static const char * module_id_ = "$Id";

#include <zks.h>
#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <sys/param.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkShell.h>
#include <zkDB.h>
#include <zkList.h>
#include <zkFile.h>
#include <frEnt.h>
#include "nymtool.h"
#include "cmdniqs.h"

#define CMD_NIQS_LIST "list"

/*
 * 
 */

static int cmdNiqsList ( ZkError , ZkShell , NymToolContext * , char * );

/*
 *
 */

static int
cmdNiqsList (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  char * ent_type_s )
{
  static const char * func = "cmdNiqsList";
  int return_code = 0;
  FrEntIdType type = FRENT_TYPE_UNUSED_;
  ZkList list = NULL;
  FrEntIdExp entity;
  FrNetInfoDatum datum = NULL;
  int count = 0;
  
  (void) module_id_; /* Get rid of compile warning */
  zkErrorClear(err);
  if (c && shell && ent_type_s) {
    if ((type = frEntTypeFromStr(ent_type_s)) == FRENT_TYPE_UNUSED_) {
      zkShellWrite_va(err,shell,"Unrecognized entity type \"%s\".\n",
                      ent_type_s);
    } else if (!frNetInfoGetListOf(err,c->fni,&list,type)) {
      zkShellWrite_va(err,shell,"Unable to retrieve list of \"%s\" entities.\n",
                      ent_type_s);
    } else if (!zkListMoveCursor(err,list,LIST_HEAD)) {
      zkShellWrite_va(err,shell,"Internal ZkList error.\n");
    } else {
      return_code = 1;
      while (zkListGet(err,list,(void **) &datum,LIST_FORWARDS)) {
        if (!frNetInfoDatumGetEntity(err,datum,&entity)) {
          zkShellWrite_va(err,shell,"Internal data error.\n");
          return_code = 0;
          break;
        } else {
          zkShellWrite_va(err,shell,"%3d %s\n",count,entity.name);
          count++;
        }
      }
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  }
  return return_code;
}


DLLExport int
cmdNiqs (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{
  static const char * func = "cmdNiqs";
  int return_code = ZKSHELL_SESSION_CONTINUE;
  NymToolContext * context = (NymToolContext *) userdata;
  int rc = 0;

  (void) func;
  zkErrorClear(err);
  if (tokens < 2) {
    zkShellWrite_va(err,shell,"Insufficient number of tokens\n");
  } else if (strcasecmp(token[1],CMD_NIQS_LIST) == 0) {
    if (tokens == 3) {
      rc = cmdNiqsList(err,shell,context,token[2]);
    } else {
      zkShellWrite_va(err,shell,"Error:  You must specify an entity type.\n");
    }
  } else {
    zkShellWrite_va(err,shell,"Unrecognized command \"%s\".\n",token[1]);
  }
  return return_code;
}
