/*
 * $Id: commands.c,v 1.9 2000/10/04 12:56:34 marcp Exp $
 */

static const char * module_id_ = "$Id: commands.c,v 1.9 2000/10/04 12:56:34 marcp Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkShell.h>
#include <zkDB.h>
#include <zkURL.h>
#include <frSerial.h>
#include <frCapability.h>
#include <frToken.h>
#include <frTokenType.h>
#include <frTokenCache.h>
#include "nymtool.h"
#include "commands.h"


#define DEMO_SERIAL_NUMBER "demo99"
#define SNID_PREFIX        "snid"

static int cmdTokenRedeem ( ZkError , ZkShell , NymToolContext *, const char *);
static int cmdTokenList ( ZkError , ZkShell , NymToolContext * );
static int cmdTokenSave ( ZkError , ZkShell , NymToolContext * );
static int cmdTokenDelete ( ZkError , ZkShell, NymToolContext *, unsigned int);
static int cmdTokenDescr ( ZkError , ZkShell, NymToolContext *, unsigned int);
static int cmdTokenMark ( ZkError , ZkShell , NymToolContext *, 
                          unsigned int , char *);


DLLExport int
cmdShowContext (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{ 
  NymToolContext * context = (NymToolContext *) userdata;

  nymToolContextDump(err,context,stdout);

  return ZKSHELL_SESSION_CONTINUE;
}

#define CMD_TOK_REDEEM "redeem"
#define CMD_TOK_LIST   "list"
#define CMD_TOK_SAVE   "save"
#define CMD_TOK_DELETE "delete"
#define CMD_TOK_DESCRIPTION "description"
#define CMD_TOK_MARK   "mark"

static int
cmdTokenRedeem (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  const char * sn )
{
  const char * func = "cmdTokenRedeem";
  FrSerialNumber frsn = NULL;
  TokenServerResponse tsresp;
  char ** tok = NULL;
  size_t toks = 0;
  size_t n = 0;
  int return_code = 0;
  int sn_sz = 0;
  unsigned int ndx = 0;
  char * scalper_sn = NULL;

  (void) module_id_; /* Get rid of compile time warning */
  zkErrorClear(err);
  if (c && sn && shell) {

    if (strcasecmp(sn,DEMO_SERIAL_NUMBER) == 0) {
      size_t scalper_sz = 0;
      if (!c->demo_url) {
        zkShellWrite_va(err,shell,
                        "The %s option was not set in the nymtool config file.\n",
                        P_DEMO_URL);
        goto finish;
      }
      
      if (!zkURLGetBody(err,c->demo_url,&scalper_sn,&scalper_sz) ||
          !scalper_sz) {
        zkShellWrite_va(err,shell,"Unable to get serial number from \"%s\".\n",
                        c->demo_url);
        goto finish;
      }
      sn = scalper_sn;
      sn_sz = (int) scalper_sz;
    } else if (strncasecmp(sn,SNID_PREFIX,strlen(SNID_PREFIX)) == 0) {
      size_t prefixSize, scalper_sz = 0;
      char url[500];
      
      if (!c->scalperUrlPrefix) {
        zkShellWrite_va(err,shell,
                        "The %s option was not set in the nymtool config file.\n",
                        P_SCALPER_URL_PREFIX);
        goto finish;
      }

      /* Now build the URL with the given SNID */
      prefixSize = strlen(c->scalperUrlPrefix);
      memcpy(url, c->scalperUrlPrefix, prefixSize);
      memcpy(url + prefixSize, sn+4, strlen(sn) - 4 + 1);

      if (!zkURLGetBody(err,url,&scalper_sn,&scalper_sz) ||
          !scalper_sz) {
        zkShellWrite_va(err,shell,"Unable to get serial number from \"%s\".\n",
                        url);
        goto finish;
      }
      sn = scalper_sn;
      sn_sz = (int) scalper_sz;
    } else {
      sn_sz = strlen(sn);
    }

    if (!(frSerialNumberNew(err,&frsn) &&
          frSerialNumberParse(err,frsn,sn,sn_sz))) {
      zkShellWrite_va(err,shell,"Invalid serial number\n");
      goto finish;
    }

    /*
     * Query the server before using the serial number
     */

    {
      char cap[128];
      size_t capsz = 128;
      size_t token_count = 0;
      unsigned short token_type = 0;
      unsigned short sn_type = 0;
      time_t activate = 0;
      if (!frSerialNumberGetCapability(err,c->kh,c->kq,frsn,
                                       &(c->tokserv),&tsresp,cap,&capsz)) {
        zkShellWrite_va(err,shell,"Failed to get capability:  %d %s\n",
                        tsresp.code,tsresp.message);
      } else if (!frCapabilityCodeParse(err,cap,capsz,&token_count,
                                        &token_type,
                                        &sn_type,&activate)) {
        zkShellWrite_va(err,shell,"Failed to parse capability code.\n");
      } else {
        zkShellWrite_va(err,shell,"Serial number type id %d.\n"
                        "Generates %d type %d tokens which activate on %s",
                        sn_type,token_count,token_type,ctime(&activate));
      }
    }

    if (!frSerialNumberRedeem(err,frsn,&(c->tokserv),&tsresp,&tok,&toks)) {
      if (tsresp.code != 0) {
        zkShellWrite_va(err,shell,"Token server response was \"%d %s\"\n",
                        tsresp.code,tsresp.message);
      } 
      zkShellWrite_va(err,shell,"Unable to get tokens\n");
      goto finish;
    }

    if (tok) {
      for (n = 0; n < toks; n++) {
        if (tok[n]) {
          if (!frTokenCacheAddToken(err,c->tk,tok[n],&ndx)) {
            zkShellWrite_va(err,shell,"Unable to add token %d\n",n);
          } else {
            free(tok[n]);
            tok[n] = NULL;
          }
        }
      }
    } 
    return_code = 1;  

finish:
    if (tok) {
      free(tok);
    }
    if (frsn) {
      frSerialNumberFree(err,&frsn);
    }
    if (scalper_sn) {
      free(scalper_sn);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

/*
 * Set the token flags 
 */

static int
cmdTokenMark (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  unsigned int ndx ,
  char * setting )
{
  const char * func = "cmdTokenMark";
  int return_code = 0;

  if (shell && c) {
    if (strcasecmp(setting,"SUSPECT") == 0) {
      if (!frTokenCacheSetTokenFlags(err,c->tk,ndx,
                                     FL_TOKEN_CACHE_TOKEN_SUSPECT)) {
        return_code = 0;
        zkShellWrite_va(err,shell,"Unable to mark token %d as %s.\n",
                        ndx,setting);
      }
    } else if (strcasecmp(setting,"OK") == 0) {
      if (!frTokenCacheSetTokenFlags(err,c->tk,ndx,0)) {
        return_code = 0;
        zkShellWrite_va(err,shell,"Unable to mark token %d as %s.\n",
                        ndx,setting);
      }
    } else {
      zkShellWrite_va(err,shell,
                      "You can only mark a token as SUSPECT or OK.\n");
    }
    return_code = 1;
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  }
  return return_code;
}
  
static int
cmdTokenDelete (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  unsigned int ndx )
{
  const char * func = "cmdTokenDelete";
  int return_code = 0;

  zkErrorClear(err);
  if (shell && c) {
    if (!frTokenCacheDeleteToken(err,c->tk,ndx)) {
      zkShellWrite_va(err,shell,"Unable to delete token %d.\n",ndx);
    } else {
      zkShellWrite_va(err,shell,"Token %d deleted.\n",ndx);
      return_code = 1;
    } 
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
cmdTokenDescr (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ,
  unsigned int ndx )
{
  const char * func = "cmdTokenDescription";
  const FrTokenTypeDescription * descr = NULL;
  int return_code = 0;
  FrToken frToken = NULL;
  const char * token = NULL;
 
  zkErrorClear(err);
  if (shell && c) {
    if (!frTokenCacheGetToken(err,c->tk,ndx,&token)) {
      zkShellWrite_va(err,shell,"Token %d not found.\n",ndx);
    } else if (token) {
      if (frTokenNewFromBase64(err,&frToken,token,NULL)) {
        unsigned short typeid = 0;
        frTokenTypeId(err,frToken,&typeid);
        if (frTokenTypeLookup(err,c->tt,typeid,&descr)) {
          zkShellWrite_va(err,shell,
                          "Label           = %s\n"
                          "TypeID          = %d\n"
                          "Access-Level    = %d\n"
                          "Duration        = %d days\n"
                          "Reserved        = %d days\n"
                          "Flags           = %s %s %s %s\n"
                          "Email-Limit     = %d mails/day\n"
                          "Email-Domain    = %s\n"
                          "Crosspost-Limit = %d\n\n",
                          descr->label,typeid,descr->access_level,
                          descr->duration,descr->reserved,
                          (descr->flags&FL_TOKEN_CREATE)?FL_TOKEN_CREATE_STR:"",
                          (descr->flags&FL_TOKEN_UPGRADE)?FL_TOKEN_UPGRADE_STR:"",
                          (descr->flags&FL_TOKEN_DURATION_OVERIDE)?FL_TOKEN_DURATION_OVERIDE_STR:"",
                          (descr->flags&FL_TOKEN_TRIAL)?FL_TOKEN_TRIAL_STR:"",
                          descr->email_limit,
                          descr->email_domain,descr->crosspost_limit);
          return_code = 1;
        } else {
          zkShellWrite_va(err,shell,"Description not available.\n");
        }
      }
    }
    if (frToken) {
      frTokenFree(NULL,&frToken);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
} 
  

static int
cmdTokenList (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c ) 
{
  const char * func = "cmdTokenList";
  const char * token = NULL;
  const FrTokenTypeDescription * descr = NULL;
  size_t toks = 0;
  size_t n = 0;
  int return_code = 0;
  uint32_t tok_flags = 0;
  FrToken frToken = NULL;


  zkErrorClear(err);
  if (shell && c) {
    if (!frTokenCacheTokens(err,c->tk,&toks)) {
      goto finish;
    }
    for (n = 0; n < toks; n++) {
      if (!frTokenCacheGetToken(err,c->tk,(unsigned int) n,&token)) {
        zkShellWrite_va(err,shell,"%3d { error }\n",n);
      } else if (token) {
        frTokenCacheGetTokenFlags(err,c->tk,(unsigned int) n,&tok_flags);
        if (frTokenNewFromBase64(err,&frToken,token,NULL)) {
          unsigned short typeid = 0;
          frTokenTypeId(err,frToken,&typeid);
          if (!frTokenTypeLookup(err,c->tt,typeid,&descr)) {
            zkShellWrite_va(err,shell,"%3d %d ",n,typeid);
          } else {
            zkShellWrite_va(err,shell,"%3d %s ",n,descr->label);
          }
          zkShellWrite_va(err,shell,"%s\n",
                          (tok_flags&FL_TOKEN_CACHE_TOKEN_SUSPECT)?"(suspect)":"");

        }
        if (frToken) {
          frTokenFree(err,&frToken);
        }
      } else {
        zkShellWrite_va(err,shell,"%3d { unassigned }\n",n);
      }
    }

    return_code = 1;
finish:
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
cmdTokenSave (
  ZkError err ,
  ZkShell shell ,
  NymToolContext * c )
{
  const char * func = "cmdTokenSave";
  char pathname[MAXPATHLEN];
  int return_code = 0;

  zkErrorClear(err);
  if (shell && c) {
    snprintf(pathname,sizeof pathname,"%s/%s",c->root,c->tokencache);
    if (!frTokenCacheSaveToFile(err,c->tk,pathname)) {
      zkShellWrite_va(err,shell,"Unable to save token cache to \"%s\".\n",
                      pathname);
    } else {
      return_code = 1;
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}



DLLExport int
cmdToken (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * userdata )
{
  int return_code = ZKSHELL_SESSION_CONTINUE;
  NymToolContext * context = (NymToolContext *) userdata;
  int rc = 0;

  zkErrorClear(err);
  
  if (tokens < 2) {
    zkShellWrite_va(err,shell,"Insufficient number of tokens\n"); 
    goto finish;
  }
  if (strcasecmp(token[1],CMD_TOK_REDEEM) == 0) {
    if (tokens == 3) {
      rc = cmdTokenRedeem(err,shell,context,token[2]); 
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a serial number or \"demo99\".\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_LIST) == 0) {
    if (tokens == 2) {
      rc = cmdTokenList(err,shell,context);
    } else {
      zkShellWrite_va(err,shell,"Error:  incorrect number of parameters.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_SAVE) == 0) {
    if (tokens == 2) {
      rc = cmdTokenSave(err,shell,context);
    } else {
      zkShellWrite_va(err,shell,"Error:  incorrect number of parameters.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_DELETE) == 0) {
    if (tokens == 3) {
      rc = cmdTokenDelete(err,shell,context,atoi(token[2]));
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a token number index.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_MARK) == 0) {
    if (tokens == 4) {
      rc = cmdTokenMark(err,shell,context,atoi(token[2]),token[3]);
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a token number index and setting.\n");
    }
  } else if (strcasecmp(token[1],CMD_TOK_DESCRIPTION) == 0) {
    if (tokens == 3) {
      rc = cmdTokenDescr(err,shell,context,atoi(token[2]));
    } else {
      zkShellWrite_va(err,shell,
                      "You must specify a token number index.\n");
    }
  } else {
    zkShellWrite_va(err,shell,"unrecognized command \"%s\".\n",token[1]);
  }

finish:
  return return_code;
}
  




