/**
 * $Id: makerb.h,v 1.2 1999/04/06 15:52:26 kent Exp $
 *
 * Make reply blocks.  Typical order of calls:
 *
 * frMaipRbBuildCtxNew()
 * frMaipRbBuildCtxSetKeyParms()
 * frMaipRbBuildCtxSetLocalInfo()
 * frMaipRbBuildCtxMake()
 * frMaipRbBuildCtxGetRb()
 * frMaipRbBuildCtxFree()
 */
#ifndef MAKERB_H_
#define MAKERB_H_

#include "zks.h"
#include "zkError.h"
#include "zkErrdef.h"
#include "frEnt.h"
#include "frNetInfo.h"
#include "frMaip.h"
#include "frKeyQry.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  /**
   * Context to hold information needed for making reply blocks.
   */
  typedef struct T_FrMaipRbBuildCtx *FrMaipRbBuildCtx;

  /**
   * Create a new context variable for reply block building.
   */
  extern int frMaipRbBuildCtxNew(ZkError err, 
                                 FrMaipRbBuildCtx *rb);

  /**
   * Free the memory of the context variable.
   */
  extern int frMaipRbBuildCtxFree(ZkError err, 
                                  FrMaipRbBuildCtx *rb);

  /**
   * Set key query parameters in the context so that they can be used
   * when making a reply block.  The key query cache and key query host
   * are assumed to be initialized.  Later keys will appear in the key
   * query cache after the frMaipRbBuildCtxMake() function call.  None
   * of the other parameters will be modified.
   */
  extern int frMaipRbBuildCtxSetKeyParms(ZkError err,
                                         FrMaipRbBuildCtx rb, 
                                         FrKeyQry kq, 
                                         const FrKeyQryHost *kh,
                                         int kqretries,
                                         int kqtimeout);

  /**
   * Set information about the local user.  The email address and the nym
   * entity must be set.  However, the reply block name is only optional, so
   * it should be NULL if it will not be used.  None of the parameters will
   * be modified.
   */
  extern int frMaipRbBuildCtxSetLocalInfo(ZkError err, 
                                          FrMaipRbBuildCtx rb,
                                          const char *email,
                                          const FrEntIdExp *nym,
                                          const char *rbname);

  /**
   * Make the reply block from the array of maip entities.  Do *not* include
   * the FMG in the list, it will be added automatically.  Keys for the
   * entities will be fetched.
   */ 
  extern int frMaipRbBuildCtxMake(ZkError err,
                                  FrMaipRbBuildCtx rb,
                                  const FrEntIdExp *fmg,
                                  const FrEntIdExp *maips,
                                  size_t msz);

  /**
   * Extract the private reply block from the context variable.
   * After this call it is up to the caller to free this memory.
   */
  extern int frMaipRbBuildCtxGetRb(ZkError err,
                                   FrMaipRbBuildCtx rb, 
                                   FrMaipRbPriv *rbpriv);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* MAKERB_H_ */
