/*
 * $Id: nym.h,v 1.6 2001/04/19 19:21:46 alainc Exp $
 */


#ifndef NYM_H_
#define NYM_H_


#include <frEnt.h>
#include <frMaip.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define ZKMNYM 0

#define MAX_RB                   5    /* Max number of reply blocks */


typedef struct T_FrNym * FrNym;

DLLExport int frNymInit (
  ZkError err ,
  FrNym nym );

DLLExport int frNymNew (
  ZkError err ,
  FrNym * nym );

DLLExport int frNymFree (
  ZkError err ,
  FrNym * nym );

DLLExport int frNymToBuffer (
  ZkError err ,
  FrNym nym ,
  void ** b ,
  size_t * sz );

DLLExport int frNymFromBuffer (
  ZkError err ,
  FrNym * nym ,
  void * b ,
  size_t sz );

DLLExport int frNymLoadFromFile (
  ZkError err ,
  FrNym * nym ,
  const char * filename );

DLLExport int frNymSaveToFile (
  ZkError err ,
  FrNym nym ,
  const char * filename );

DLLExport int frNymRemoteNym (
  ZkError err ,
  FrNym nym ,
  FrRemoteNym * rnym );

DLLExport int frNymSetName (
  ZkError err ,
  FrNym nym ,
  const char * name );

DLLExport int frNymName (
  ZkError err ,
  FrNym nym ,
  const char ** name );

DLLExport int frNymAddBlockedAddress (
  ZkError ,
  FrNym ,
  char * ); /* NUL terminated string, an email address to block at fmg-maip */

DLLExport int frNymGetBlockedAddress (
  ZkError ,
  FrNym ,
  size_t ,  /* index of email address to return */
  char ** ); /* an email address blocked st the fmg-maip */

DLLExport int frNymGetBlockedAddressList (
  ZkError ,
  FrNym ,
  ZkList * ); /* list of email addresses blocked at the fmg-maip */

DLLExport int frNymSetPrivateReplyBlock (
  ZkError ,
  FrNym ,
  size_t ,
  void * ,
  size_t );

DLLExport int frNymSetPublicReplyBlock (
  ZkError ,
  FrNym ,
  size_t  ,
  void *  ,
  size_t );

DLLExport int frNymPublicReplyBlock (
  ZkError ,
  FrNym n,
  size_t ,
  void **  ,
  size_t * );

DLLExport int frNymPrivateReplyBlock (
  ZkError ,
  FrNym ,
  size_t ,
  void ** ,
  size_t * );

DLLExport int frNymLongname (
  ZkError ,
  FrNym ,
  const char ** );     /* Returns a pointer to the long (informational) name */

DLLExport int frNymSetLongname (
  ZkError,
  FrNym ,
  const char * );  /* Specify the nym's long (informational) name */

DLLExport int frNymSetPrivateEmailAddr (
  ZkError ,
  FrNym ,
  const char * );   /* The nym user's private email address */

DLLExport int frNymPrivateEmailAddr (
  ZkError ,
  FrNym ,
  const char ** );   /* Returns the nym user's private email address */



DLLExport int frNymSetExpiry (
  ZkError err ,
  FrNym nym ,
  ZkClock exp );

DLLExport int frNymGenerateCryptionKey (
  ZkError err ,
  FrNym nym ,
  int size );

DLLExport int frNymGenerateSignatureKey (
  ZkError err ,
  FrNym nym ,
  int size );

DLLExport int frNymGenerateKeys (
  ZkError err ,
  FrNym nym ,
  int size );

DLLExport int frNymGetSignatureKey (
  ZkError err ,
  FrNym nym ,
  FrCryptPubKey * key );

DLLExport int frNymGetCryptionKey (
  ZkError err ,
  FrNym nym ,
  FrCryptPubKey * key );

DLLExport int frNymGetSpamControls (
  ZkError err ,
  FrNym nym ,
  uint32_t * ctl );

DLLExport int frNymSetSpamControls (
  ZkError err ,
  FrNym nym ,
  uint32_t ctl );


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* NYM_H_ */

