/*
 * $Id: nymtool.c,v 1.18 2000/10/04 12:56:34 marcp Exp $
 */

static const char * module_id_ = "$Id: nymtool.c,v 1.18 2000/10/04 12:56:34 marcp Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <sys/param.h>
#include <string.h>
#include <ctype.h>
#include <zkError.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <zkLog.h>
#include <zkConfig.h>
#include <frSec.h>
#include <frEnt.h>
#include <zkShell.h>
#include <ZKIP.h>
#include <zkipClient.h>
#include <zkParms.h>
#include "zkDNS.h"
#include <frNiqs.h>
#include "nymtool.h"
#include "commands.h"
#include "cmdnym.h"
#include "cmdniqs.h"
#include "frFilenames.h"
#include "zkCStr.h"

#ifndef MAXPATHLEN
#define MAXPATHLEN 256
#endif /* !MAXPATHLEN */

/*
 * Internal function prototypes
 */

static int nymToolContextInit ( ZkError , NymToolContext * , int , char ** );
static int nymToolContextLoad ( ZkError , NymToolContext * );
static int nymToolContextSlurp ( ZkError , NymToolContext * );
static int nymToolContextVerify ( ZkError , NymToolContext * );
static int nymToolContextCleanup ( ZkError , NymToolContext * );
static int nymToolContextGetServerData ( ZkError, NymToolContext *, FrEntIdType,
                                         char * , size_t , uint16_t * );
static int nymToolContextCreateSymkey ( ZkError , NymToolContext * );



static int
nymToolContextInit (
  ZkError err ,
  NymToolContext * c ,
  int argc ,
  char ** argv )
{
  const char * func = "NymToolContextInit";
  
  (void) module_id_; /* Get rid of compile warning */
  zkErrorClear(err);
  if (c && argv && (argc != 0)) {
    memset(c,0,sizeof(NymToolContext));
 
    if ((c->progname = strrchr(argv[0],'/')) == NULL) {
      c->progname = argv[0];
    } else {
      c->progname++;
    }

    if (!(zkErrorNew(&(c->err)) &&
          zkLogNew(&(c->log)) &&
          zkConfigNew(c->err,&(c->clientinfo)) &&
          frKeyQryNew(c->err,&(c->kq)) &&
          frNetInfoNew(c->err,&(c->fni)))) {
      zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKERROR,
                 "Unable to create internal data structure");
      return 0;
    }

    if (!(zkConfigNew(err,&(c->config)) &&
          zkConfigSetCmdParms(err,c->config,argc,argv))) {
      return 0;
    }

    c->niqs_host = DEFAULT_NIQS_HOST;
    c->niqs_port = DEFAULT_NIQS_PORT;
    c->niqs_timeout = DEFAULT_NIQS_TIMEOUT;

    c->flags = DEFAULT_FLAGS;
    c->config_file = DEFAULT_CONFIG_FILE;
    if (DEFAULT_NYMSERV_HOST) {
      zkCStrlcpy(c->nymserv.nymserver,DEFAULT_NYMSERV_HOST,sizeof c->nymserv.nymserver);
    }
    c->nymserv.port = DEFAULT_NYMSERV_PORT;
    c->nymserv.timeout = DEFAULT_NYMSERV_TOUT;
    if (DEFAULT_TOKSERV_HOST) {
      zkCStrlcpy(c->tokserv.tokenserver,DEFAULT_TOKSERV_HOST,sizeof c->tokserv.tokenserver);
    }
    c->tokserv.port = DEFAULT_TOKSERV_PORT;
    c->tokserv.timeout = DEFAULT_TOKSERV_TOUT;
    c->root = DEFAULT_ROOT;
    c->prompt = DEFAULT_PROMPT;
    c->tokencache = DEFAULT_TOKENCACHE;
    c->tk = NULL;        /* Token cache */
    c->tt = NULL;        /* Token type description */
    c->demo_url = DEFAULT_DEMO_URL;
    c->scalperUrlPrefix = DEFAULT_SCALPER_URL_PREFIX;
    c->nym = NULL;
    c->kh.fd = -1;
    zkCStrlcpy(c->kq_host,DEFAULT_KQ_HOST, sizeof c->kq_host);
    c->kq_port = DEFAULT_KQ_PORT;
    c->kq_retries = DEFAULT_KQ_RETRIES;
    c->kq_timeout = DEFAULT_KQ_TIMEOUT;
    c->passphrase_prompt = DEFAULT_PASSPHRASE_PROMPT;
    c->passphrase_file = DEFAULT_PASSPHRASE_FILE;
    c->passphrase_info = DEFAULT_PASSPHRASE_INFO;
    c->symkey = NULL;
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}


DLLExport int 
nymToolContextDump (
  ZkError err ,
  NymToolContext * c ,
  FILE * stream )
{
  const char * func = "nymToolContextDump";

  zkErrorClear(err);
  if (c && stream) {
    fprintf(stream,"[ "SEC" ]\n\n");
    fprintf(stream,P_DEBUG" = %s\n",((c->flags&FL_DEBUG)?"yes":"no"));
    fprintf(stream,P_USENIQS" = %s\n",((c->flags&FL_USENIQS)?"yes":"no"));
    if (c->config_file) fprintf(stream,"%s = %s\n",P_CONFIG,c->config_file);
    if (c->root) fprintf(stream,"%s = %s\n",P_ROOT,c->root);
    if (c->demo_url) fprintf(stream,"%s = %s\n",P_DEMO_URL,c->demo_url);
    if (c->scalperUrlPrefix) fprintf(stream,"%s = %s\n",
                                     P_SCALPER_URL_PREFIX,c->scalperUrlPrefix);
    fprintf(stream,"%s = %s\n",P_TOKENCACHE,c->tokencache);

    fprintf(stream,"\n[ "S_CLIENT_NYMSRV" ]\n\n");
    fprintf(stream,P_NYMSRV_HOSTNAME" = %s\n",c->nymserv.nymserver);
    fprintf(stream,P_NYMSRV_PORT" = %d\n",c->nymserv.port);
    fprintf(stream,P_NYMSRV_TIMEOUT" = %d\n",c->nymserv.timeout);

    fprintf(stream,"\n[ "S_CLIENT_TOKEN" ]\n\n");
    fprintf(stream,P_TOKSRV_HOSTNAME" = %s\n",c->tokserv.tokenserver);
    fprintf(stream,P_TOKSRV_PORT" = %d\n",c->tokserv.port);
    fprintf(stream,P_TOKSRV_TIMEOUT" = %d\n",c->tokserv.timeout);

    fprintf(stream,"\n[ "S_CLIENT_KEYQRY" ]\n\n");
    fprintf(stream,P_KEYQRY_HOSTNAME" = %s\n",c->kq_host);
    fprintf(stream,P_KEYQRY_PORT" = %d\n",c->kq_port);
    fprintf(stream,P_KEYQRY_TIMEOUT" = %d\n",c->kq_timeout);
    fprintf(stream,P_KEYQRY_RETRIES" = %d\n",c->kq_retries);
   
    fprintf(stream,"\n");
    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}


static int
nymToolContextLoad (
  ZkError err ,
  NymToolContext * c )
{
  const char * func = "nymToolContextLoad";
  char * str = NULL;
  uint8_t bool = 0;
  uint16_t u16 = 0;
 
  zkErrorClear(err);
  if (c) {

    /*
     * First, get the configuration file and load it if it is specified 
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,P_CONFIG,&str)) {
      c->config_file = str;
      if (!zkConfigLoad(err,c->config,c->config_file)) {
        return 0;
      }
    }
 
    /*
     * Do we want to run in debug mode?
     */

    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,SEC,P_DEBUG,&bool)) {
      if (bool) {
        ZkLog log = NULL; 
        c->flags |= FL_DEBUG; 
        zkErrorLog(c->err,&log); 
        zkLogSetStream(log,stderr); 
        zkLogSetProgname(log,c->progname); 
        zkLogEnable(log);
      }
    }

    /*
     * Do we overide the FL_USENIQS flag?
     */

    if (zkConfigGetParameter(err,c->config,TYPE_BOOLEAN,SEC,P_USENIQS,&bool)) {
      if (bool) {
        c->flags |= FL_USENIQS;
      } else {
        c->flags &= ~FL_USENIQS;
      }
    }

    /* 
     * Get the file root directory 
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,P_ROOT,&str)) {
      c->root = str;
    }

    /*
     * Load the name of the token cache file
     */
    
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,P_TOKENCACHE,&str)) {
      c->tokencache = str;
    }
      
    /*
     * Get the nym server connection parameters (hostname, port, and timeout)
     */
   
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,S_CLIENT_NYMSRV,
                             P_NYMSRV_HOSTNAME,&str)) {
      strncpy(c->nymserv.nymserver,str,NSHOST_SZ);
      c->nymserv.nymserver[NSHOST_SZ] = (char) 0;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_NYMSRV,
                             P_NYMSRV_PORT,&u16)) {
      c->nymserv.port = u16;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_NYMSRV,
                             P_NYMSRV_TIMEOUT,&u16)) {
      c->nymserv.timeout = u16;
    }

    /*
     * Get the token server connection parameters (hostname, port, and timeout)
     */
  
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,S_CLIENT_TOKEN,
                             P_TOKSRV_HOSTNAME,&str)) {
      strncpy(c->tokserv.tokenserver,str,TSHOST_SZ);
      c->tokserv.tokenserver[TSHOST_SZ] = (char) 0;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_TOKEN,
                             P_TOKSRV_PORT,&u16)) {
      c->tokserv.port = u16;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_TOKEN,
                             P_TOKSRV_TIMEOUT,&u16)) {
      c->tokserv.timeout = u16;
    }

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,P_DEMO_URL,&str)) {
      c->demo_url = str;
    }

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,P_SCALPER_URL_PREFIX,&str)) {
      c->scalperUrlPrefix = str;
    }

    /*
     * Key server connection parameters
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,S_CLIENT_KEYQRY,
                             P_KEYQRY_HOSTNAME,&str)) {
      strncpy(c->kq_host,str,KQHOST_SZ - 1);
    }  
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_KEYQRY,
                             P_KEYQRY_PORT,&u16)) {
      c->kq_port = u16;
    }  
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_KEYQRY,
                             P_KEYQRY_RETRIES,&u16)) {
      c->kq_retries = u16;
    }  
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_KEYQRY,
                             P_KEYQRY_TIMEOUT,&u16)) {
      c->kq_timeout = u16;
    }  

    /*
     * Get the NIQS parameters 
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,S_CLIENT_NIQS,
                             P_NIQS_SERVER,&str)) {
      c->niqs_host = str;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_NIQS,
                             P_NIQS_PORT,&u16)) {
      c->niqs_port = u16;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_UINT16,S_CLIENT_NIQS,
                             P_NIQS_TIMEOUT,&u16)) {
      c->niqs_timeout = u16;
    }

    /*
     * Get the passphrase data
     */

    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,
                             P_PASSPHRASE_FILE,&str)) {
      c->passphrase_file = str;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,
                             P_PASSPHRASE_PROMPT,&str)) {
      c->passphrase_prompt = str;
    }
    if (zkConfigGetParameter(err,c->config,TYPE_STRING,SEC,
                             P_PASSPHRASE_INFO_FILE,&str)) {
      c->passphrase_info = str;
    }
                       

    return 1;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}

static int
nymToolContextGetServerData (
  ZkError err ,
  NymToolContext * c ,
  FrEntIdType type ,
  char * hostname ,
  size_t hsz ,
  uint16_t * port )
{

  const char * func = "nymToolContextGetServerData";
  int return_code = 0;
  FrNiqsEntity * list_ent = NULL; 
  ZkList list = NULL;
  struct sockaddr_in saddr;
  unsigned char * addr = NULL;
  FrNetInfoDatum datum = NULL;

  zkErrorClear(err);
  if (c) {
    if (!frNetInfoGetListOf(err,c->fni,&list,type)) {
      fprintf(stderr,"%s:  Unable to get entity type %s list.\n",
             c->progname,frEntTypeGetStr(type));
    } else if (!(zkListMoveCursor(err,list,LIST_HEAD) &&
                 zkListGet(err,list,(void **) &datum,LIST_FORWARDS))) {
    } else if (!frNetInfoDatumGetSockaddr(err,datum,&saddr)) {
      fprintf(stderr,"%s:  Unable to get address and port for %s.\n",
              c->progname,list_ent->entity.name);
    } else {
      addr = (unsigned char *) &(saddr.sin_addr.s_addr);
      snprintf(hostname,hsz,"%d.%d.%d.%d",
               addr[0],addr[1],addr[2],addr[3]);
      *port = ntohs(saddr.sin_port);
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  }
  return return_code;
}

/*
 * Slurp data out of the FrNetInfo object in order to complete our context
 */

static int
nymToolContextSlurp (
  ZkError err ,
  NymToolContext * c )
{
  const char * func = "nymToolContextSlurp";
  int return_code = 0;

  zkErrorClear(err);
  if (c) {
    if (!nymToolContextGetServerData(err,c,FRENT_TYPE_NYM_SRV,
                                     c->nymserv.nymserver,NSHOST_SZ,
                                     &(c->nymserv.port))) {
      fprintf(stderr,"%s:  Unable to get nym server data.\n",c->progname);
    } else if (!nymToolContextGetServerData(err,c,FRENT_TYPE_TOK_SRV,
                                            c->tokserv.tokenserver,TSHOST_SZ,
                                            &(c->tokserv.port))) {
      fprintf(stderr,"%s:  Unable to get token server data.\n",c->progname);
    } else if (!nymToolContextGetServerData(err,c,FRENT_TYPE_KEYQRY_SRV,
                                            c->kq_host,KQHOST_SZ,
                                            &(c->kq_port))) {
      fprintf(stderr,"%s:  Unable to get key query server data.\n",c->progname);
    } else {
      return_code = 1;
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  }
  return return_code;
}
  

static int
nymToolContextVerify (
  ZkError err ,
  NymToolContext * c )
{
  const char * func = "nymToolContextVerify";
  NymServerResponse nsresp;
  ZkipClient  client = NULL;
  char pathname[MAXPATHLEN];

  zkErrorClear(err);
  if (c) {
    if (c->root == NULL) {
      fprintf(stderr,"%s:  You must specify the \"%s\" parameter.\n",
              c->progname,P_ROOT);
      goto finish;
    }

    /*
     * Load the token cache from the disk.  otherwise, initialize a new
     * token cache.
     */

    if (c->tokencache == NULL) {
      fprintf(stderr,"%s:  You must specify the \"%s\" parameter.\n",
              c->progname,P_TOKENCACHE);
      goto finish;
    }
    snprintf(pathname,sizeof pathname,"%s/%s",c->root,c->tokencache);
    if (!frTokenCacheLoadFromFile(err,&(c->tk),pathname)) {
      fprintf(stderr,"%s:  Warning:  unable to load token cache \"%s\".\n",
              c->progname,pathname);
      if (!frTokenCacheNew(err,&(c->tk))) {
        fprintf(stderr,"%s:  Unable to initialize the token cache.\n",
                c->progname);
        goto finish;
      }
    }

    /*
     * If we are using the NIQS (the default setting), Use the NIQS 
     * parameters to populate the frNetInfo object.  Otherwise, use the
     * values which were read in from the configuration file and the
     * command line parameters.
     */
    if (c->flags & FL_USENIQS) {

      if( ! frNetInfoCoreInitSequence(err, c->fni, 
                                      c->niqs_host, c->niqs_port, c->niqs_timeout,
                                      0 /* no cache */
                                      , c->kq, &(c->tt))) {
          fprintf(stderr,"%s:  call to frNetInfoNymtoolInitSequence failed\n",
                  c->progname);
        goto finish;
      }

    /*
     * Then set up the key query, nym server, token server, etc addresses
     * and ports.  This is a bit of a mess because we're merging two
     * different ways of doing it and using fairly old libraries.
     */

      if (!nymToolContextSlurp(err,c)) {
        fprintf(stderr,
                "%s:  Failed to retreive information from netinfo object.\n",
                c->progname);
        goto finish;
      } 
    }

    /*
     * Initialize our key query host structure and file descriptor.  We are
     * Only supposed to get IP addresses from the frNetInfo (either as 
     * sockaddrs or dotted quad format strings) but since I still want to
     * be able to specify a hostname in the configuration file (for testing
     * purposes) we'll try to lookup the host name string when it cannot
     * be automatically converted to an address.
     */

    if ((c->kh.fd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
      fprintf(stderr,
              "%s:  Failed to create key lookup descriptor, socket: %s\n",
              c->progname,strerror(errno));
      goto finish;
    } 
    if ((c->kh.host.sin_addr.s_addr = inet_addr(c->kq_host)) == -1) {
      struct hostent * host = NULL;
      if (!zkGetHostByName(err,c->kq_host,&host)) {
        fprintf(stderr,"%s:  Unable to resolve %s to an address.\n",
                c->progname,c->kq_host);
        goto finish;
      } else {
        memcpy(&(c->kh.host.sin_addr.s_addr),host->h_addr,host->h_length);
      }
    }
    c->kh.host.sin_family = AF_INET;
    c->kh.host.sin_port = htons(c->kq_port);

#if 0
    /* The nym server still has this command, however, the source of the
     * information is now the NIQS (This is where the nym server itself
     * gets this data).  The data was retrieved in nymToolContextSlurp()
     */
    /*
     * Try to get the token type description from the nym server
     */
    
    if (!frTokenLoadTypeDescriptions(err,&(c->tt),&(c->nymserv),&nsresp)) {
      fprintf(stderr,
              "%s:  Warning:  unable to load token description from %s:%d.\n",
              c->progname,c->nymserv.nymserver,c->nymserv.port);
    }
#endif 

    /*
     * You must have specified a passphrase file OR a passphrase prompt, but 
     * not both.
     */

    if (!c->passphrase_file && !c->passphrase_prompt) {
      fprintf(stderr,"%s:  You must specify either --"P_PASSPHRASE_PROMPT
              " or --"P_PASSPHRASE_FILE".\n",c->progname);
      goto finish;
    }

    return 1;
finish:
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKERROR,"Context verification failed");
    return 0;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
}


/*
 *
 */

static int
nymToolContextCreateSymkey (
  ZkError err ,
  NymToolContext * c )
{
  const char * func = "nymToolContexxtCreateSymkey";
  int return_code = 0;

  zkErrorClear(err);
  if (c) {
    if (c->passphrase_file) {
      return_code = frSecPassSymKey(err, c->passphrase_info,
                                    c->passphrase_file, NULL,
                                    &(c->symkey),FRCRYPT_SYMALG_BLOWFISH );
    } else if (c->passphrase_prompt) {
      return_code = frSecPassSymKey(err, c->passphrase_info,
                                    "/dev/tty", c->passphrase_prompt, &(c->symkey),
                                    FRCRYPT_SYMALG_BLOWFISH ); 
    } else {
      zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKERROR,
                 "You must specify a passphrase file or prompt");
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,"Context is NULL");
  }
  return return_code;
}



static int
nymToolContextCleanup (
  ZkError err ,
  NymToolContext * c )
{
  const char * func = "nymToolContextCleanup";
  int return_code = 1;

  zkErrorClear(err);
  if (c) {
    if (c->nym) {
      frNymFree(err,&(c->nym));
    }
    if (c->config) {
      zkConfigFree(err,&(c->config));
    }
    if (c->log) {
      zkLogFree(&(c->log));
    }
    if (c->fni) {
      frNetInfoFree(err,&(c->fni));
    }
    if (c->err) {
      zkErrorFree(&(c->err));
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMNYMTOOL,ZKENULLPTR,NULL);
  return 0;
} 

/*
 *
 */

int 
main (
  int argc ,
  char ** argv )
{
  NymToolContext context;
  ZkError err = NULL;
  int exit_code = EXIT_FAILURE;
  char * progname = NULL;
  ZkShell shell = NULL;

  if ((progname = strrchr(argv[0],'/')) == NULL) {
    progname = argv[0];
  } else {
    progname++;
  }

  zkErrorNew(&err);
  frCryptInit(err,NULL);
  if (!nymToolContextInit(err,&context,argc,argv)) {
    fprintf(stderr,"%s:  Unable to initialize application context.\n",progname);
    goto finish;
  }
  if (!nymToolContextLoad(context.err,&context)) {
    fprintf(stderr,"%s:  Unable to load application context.\n",progname);
    goto finish;
  }
  if (!nymToolContextVerify(context.err,&context)) {
    fprintf(stderr,"%s:  Unable to verify application context.\n",progname);
    goto finish;
  } 
  if (!nymToolContextCreateSymkey(context.err,&context)) {
    fprintf(stderr,"%s:  Unable to create nymtool symkey from passphrase.\n",
            progname);
    goto finish;
  }


   

  if (!(zkShellNew(err,&shell) &&
        zkShellSetContext(err,shell,&context) &&
        zkShellSetPrompt(err,shell,context.prompt))) {
    fprintf(stderr,"%s:  Unable to initialize the application shell.\n",
            progname);
    goto finish;
  }
  if (!zkShellAddCommand(err,shell,cmdShowContext,"context",
                         "Display the current application context",
       "This command gives a listing of the current context in a zkConfig"
       "file format.",
       ZKSHELL_PARMS_NONE)) {
    goto finish;
  }
  if (!zkShellAddCommand(err,shell,cmdToken,"token",
                         CMD_TOKEN_USAGE,CMD_TOKEN_HELP,
                         ZKSHELL_PARMS_3 |ZKSHELL_PARMS_2 | ZKSHELL_PARMS_1)) {
    goto finish;
  }
  if (!zkShellAddCommand(err,shell,cmdNym,"nym",
                         CMD_NYM_USAGE,CMD_NYM_HELP,
                         0xFFFE)) {
    goto finish;
  }
  if (!zkShellAddCommand(err,shell,cmdNiqs,"niqs",
                         CMD_NIQS_USAGE,CMD_NIQS_HELP,
                         ZKSHELL_PARMS_2)) {
    goto finish;
  }


  if (zkShellRun(context.err,shell)) {
    exit_code = EXIT_SUCCESS;
  }

finish:
  if (exit_code == EXIT_FAILURE) {
    fprintf(stderr,"%s:  Application failure.\n",progname);
  }
 
  nymToolContextCleanup(err,&context);

  exit(exit_code);
  return 0; /* to shut up insure */
}
