/*
 * $Id: nymtool.h,v 1.15 2000/10/04 12:56:34 marcp Exp $
 */


#ifndef NYMTOOL_H_
#define NYMTOOL_H_

#include <zkConfig.h>

#include <frCrypt.h>
#include <frSec.h>
#include <frNym.h>
#include <frToken.h>
#include <frSerial.h>
#include <frTokenClient.h>
#include <frNymClient.h>
#include <frTokenCache.h>
#include "frNetInfo.h"
#include "nym.h"

/*
 * Configuration file section code
 */

#ifndef FR_NETWORK_DEFAULTS_H_
#include "frNetworkDefaults.h"
#endif /* ! FR_NETWORK_DEFAULTS_H_ */

#define SEC "NymTool"

#define ZKMNYMTOOL 0

/*
 * Configuration parameter labels
 */

#define P_TOKENCACHE         "TokenCache"
#define P_ROOT               "RootDirectory"
#define P_DEMO_URL           "DemoUrl"
#define P_SCALPER_URL_PREFIX "scalperUrlPrefix"
#define P_USENIQS            "useNiqs"

#define FL_DEBUG   0x1          /* Debug mode enabled */
#define FL_USENIQS 0x2          /* Use the NIQS to override config parms */

#define KQHOST_SZ  256          /* Hostname size... */

struct T_NymToolContext {
  uint32_t flags;               /* Application behaviour flags */
  ZkError err;                  /* Context error structure */
  ZkLog log;                    /* Context log structure */
  ZkConfig config;              /* Context configuration structure */
  int exit_code;                /* Application exit() code */
  const char * prompt;          /* Shell prompt */
  const char * progname;        /* Application program name */
  const char * config_file;     /* Name of the nymtool config file */
  const char * root;            /* Root directory where all files are kept */
  const char * tokencache;      /* The token cache file */
  const char * demo_url;        /* Demo serial number URL */
  const char * scalperUrlPrefix;/* Scalper URL prefix for retreiving SN */
  const char * niqs_host;       /* NIQS server host name */
  uint16_t niqs_timeout;        /* NIQS Server connection timeout */
  uint16_t niqs_port;           /* NIQS server port number */
  FrTokenCache tk;                /* The active token cache */
  FrTokenType tt;               /* Token type descriptions from nym server */
  FrNym nym;                    /* The current nym */
  NymServer nymserv;
  TokenServer tokserv;
  FrNetInfo fni;                /* The FrNetInfo object */
  ZkConfig clientinfo;          /* Client info block */

  FrKeyQry kq;                  /* Key query cache */
  FrKeyQryHost kh;              /* Key query host structure */
  uint16_t kq_timeout;          /* Key lookup timeout */
  uint16_t kq_retries;          /* Key lookup retries */
  char kq_host[KQHOST_SZ];      /* Key query server hostname, this is a bit of
                                 * a hack.
                                 */
  uint16_t kq_port;             /* Key query server port */
  FrEntIdExp entity_fmg;        /* The fmg entity, used to create replyblocks */
  const char * passphrase_file; /* Passphrase file */
  const char * passphrase_prompt;  /* Passphrase prompt */
  const char * passphrase_info; /* Passphrase.info file */
  FrCryptSym symkey;            /* Symmetric encryption key for nymtool files */
};

typedef struct T_NymToolContext NymToolContext;

/*
 * Default context values
 */

#define DEFAULT_FLAGS         FL_USENIQS
#define DEFAULT_CONFIG_FILE   NULL
#define DEFAULT_NYMSERV_HOST  FR_DFLT_NYM_SERVER_HOST
#define DEFAULT_NYMSERV_PORT  FR_DFLT_NYM_SERVER_PORT
#define DEFAULT_NYMSERV_TOUT  30
#define DEFAULT_TOKSERV_HOST  FR_DFLT_TOKEN_SERVER_HOST
#define DEFAULT_TOKSERV_PORT  FR_DFLT_TOKEN_SERVER_PORT
#define DEFAULT_TOKSERV_TOUT  30
#define DEFAULT_ROOT          "."
#define DEFAULT_PROMPT        "nymtool-1.0> "
#define DEFAULT_TOKENCACHE    "TokenCache"
#define DEFAULT_DEMO_URL      NULL
#define DEFAULT_SCALPER_URL_PREFIX NULL
#define DEFAULT_NIQS_HOST     FR_DFLT_NIQS_SERVER_HOST
#define DEFAULT_NIQS_PORT     FR_DFLT_NIQS_SERVER_PORT
#define DEFAULT_NIQS_TIMEOUT  30
#define DEFAULT_KQ_HOST       FR_DFLT_KEYQRY_SERVER_HOST
#define DEFAULT_KQ_PORT       FR_DFLT_KEYQRY_SERVER_PORT
#define DEFAULT_KQ_RETRIES    3
#define DEFAULT_KQ_TIMEOUT    5
#define DEFAULT_PASSPHRASE_FILE NULL
#define DEFAULT_PASSPHRASE_PROMPT "Please enter the nymtool passphrase> "
#define DEFAULT_PASSPHRASE_INFO FREEDOM_PASSINFO_FILE

/*
 *
 */

DLLExport int nymToolContextDump ( ZkError , NymToolContext * , FILE * );


#endif /* NYMTOOL_H_ */
