#! /bin/bash
# ----------------------------------------------------------------------------
#
# $Id: forward-news.sh,v 1.3 2000/11/24 14:57:33 marcp Exp $
#
# This program enforces the news cross-posting constraints for the mail system.
#
# It reads a usenet-bound message on standard in (must be seekable), and
# either prints a bounce message (if the user is trying to crosspost past
# their limit) or else just re-prints out the input. it's intended to be
# sent into the input of qmail-qmqpc, destined for the relay.
#
# ----------------------------------------------------------------------------

QMAIL_ROOT=/freedom/qmail
QMAIL_BIN=${QMAIL_ROOT}/bin
QMAIL_CONTROL=${QMAIL_ROOT}/control
ACCT=$(frname2account $(cat ${QMAIL_CONTROL}/mailroot) $(sanitize SENDER))

#
# Make sure the sender has an account
#

if [ ! -d $ACCT ]
then
    echo "sender does not have an account"
    exit 99 
fi

#
# Get the crosspost limit
#

if [ -f $ACCT/.crosspost-limit ]
then
    CROSSLIMIT=$(cat $ACCT/.crosspost-limit)
else
    CROSSLIMIT=$(cat $QMAIL_CONTROL/.crosspost-limit)
fi

#
# How many newsgroups is the message cross-posted to?
#

CROSSPOST=$(reformail -c -x 'Newsgroups:' | tr ',;' '  ' | wc -w | tr ' ' -d)

#
# Do we let it through?
#

if [ $CROSSPOST -eq 0 ]
then

    #
    # No newsgroups specified
    #

    exit 99

elif [ $CROSSPOST -gt $CROSSLIMIT ]
then

    #
    # The sender is over their crosspost limit.  Construct a bounce message
    # and send that to the relay instead of the news message.
    #

    seek 0 0
    DAEMON="MAILER-DAEMON@$HOST"
    cat <<EOF | reformail -d1 | ${QMAIL_BIN}/qmail-qmqpc 1< <(printf "F${DAEMON}\0T${SENDER}\0\0") || exit 111
From: ${DAEMON}
To: ${SENDER}
Date: $(date -R)
Subject: DELIVERY FAILURE

Hi, this is the news delivery gateway at ${HOST}.  Your news posting was
rejected for delivery because it was cross-posted to ${CROSSPOST} newsgroups
and your cross-post limit is ${CROSSLIMIT}.  Please try posting again to fewer
newsgroups.

----- Undelivered Message Follows -----
$(cat -)
EOF

    exit 99

else

    #
    # The sender is under their crosspost limit so it's ok to forward
    # the message to the relay.
    #

    seek 0 0
    ${QMAIL_BIN}/add-tagline.sh ${ACCT} \
    | reformail -c -d0 -i"From: ${SENDER}" \
    | ${QMAIL_BIN}/qmail-qmqpc 1< <(printf "F${SENDER}\0T${LOCAL}@news.${HOST}\0\0") \
    || exit 111

    exit 99

fi

