#!/usr/bin/perl -w

# $Id: post_news.pl,v 1.2 2000/10/04 12:56:36 marcp Exp $

# Script to post to a newserver. Reads the message from standard in
# and streams it line by line to the newserver. Emits an appropriate
# error code and message upon failure for use by a qmail delivery path
# (see deliver-news.sh).

# Original author: Philippe McLean 7 Sept 2000

use strict;

use lib('/freedom/qmail/lib');

use Getopt::Long;
use Carp;
use Net::NNTP;

# These are qmail specific exit values.

my $PERMANENT_FAILURE = 100;
my $TEMPORARY_FAILURE = 111;

# Print an error message to stderr and exit with an error code.

sub fail {
    my $exit_code = shift;
    my $string = shift;

    print STDERR "$string\n";
    exit($exit_code);
}

sub usage {
    fail($TEMPORARY_FAILURE,
         "usage: $0 --host <nntp_host> --port <nntp_port>\n");
}

my $nntp_host;
my $nntp_port;
my $nntp;

if (!GetOptions("host=s" => \$nntp_host, "port=i" => \$nntp_port)
    || !defined($nntp_host)
    || !defined($nntp_port))  {
    usage();
}

$nntp = Net::NNTP->new($nntp_host, Port => $nntp_port);

if (!defined($nntp)) {
    fail( $TEMPORARY_FAILURE,
          "failed to connect to news server $nntp_host:$nntp_port" );
}

if (!$nntp->post_from_filehandle(*STDIN)) {
    my $error_message = $nntp->code . ' ' . $nntp->message() ;
    fail( $PERMANENT_FAILURE, $error_message );
}
