#!/bin/sh
# ---------------------------------------------------------------------------
#
# $Id: qmail-remote.sh,v 1.5 2000/11/24 14:57:33 marcp Exp $
#
# This script implements the remote delivery pipline of the NMTA backend.
# The qmail-remote which lives in the ??/qmail/bin directory should be
# renamed to qmail-remote.orig and this script should be put in its place.
#
# ---------------------------------------------------------------------------

QMAIL_ROOT=/freedom/qmail
QMAIL_BIN=${QMAIL_ROOT}/bin
QMAIL_CONTROL=${QMAIL_ROOT}/control
CHECK_LIMIT=${QMAIL_BIN}/check-limit
CHECK_BLOCK_LIST=${QMAIL_BIN}/checkblocklist
REFORM_MAIL=${QMAIL_BIN}/reformail
HOST=${1}
SENDER=${2}
shift
shift

RECIP_REPORT=''
SUCCESS=0

for RECIPIENT
do
    ${QMAIL_BIN}/seek 0 0
    export SENDER RECIPIENT
    ACCOUNT=$(${QMAIL_BIN}/frname2account $(cat ${QMAIL_CONTROL}/mailroot) $(${QMAIL_BIN}/sanitize SENDER))
    if ${CHECK_LIMIT} ${ACCOUNT} $(cat ${QMAIL_CONTROL}/sending-limit)
    then
	if (${CHECK_BLOCK_LIST} ${QMAIL_CONTROL} -c -s -r)
	then
	    # detatch all signatures for other recipients, forward to relay
	    ${QMAIL_BIN}/add-tagline.sh ${ACCOUNT} |  \
	    ${REFORM_MAIL} -c -d0 -i"From: ${SENDER}" | \
	    sed -e "/$RECIPIENT/! s/^X-Freedom-Envelope-Sig:.*//;td;be;:d d;:e" | \
	    ${QMAIL_BIN}/qmail-qmqpc 1< <(printf "F$SENDER\0T$RECIPIENT\0\0")
	    QMQPC=$?
	    if [ $QMQPC -eq 0 ]
	    then
		# append another "recipient accepted" indicator
		RECIP_REPORT="${RECIP_REPORT}r"
		SUCCESS=$(($SUCCESS||1))
	    else
		# append a "recipient failed temporarily" indicator
		RECIP_REPORT="${RECIP_REPORT}s"
		SUCCESS=$(($SUCCESS||0))
	    fi
	else
	    # here we lie and say it was accepted, when in fact we blocked it
	    RECIP_REPORT="${RECIP_REPORT}r"
	    SUCCESS=$(($SUCCESS||1))
	fi
    else
	# here we lie and say it was accepted, when in fact you're past your
	# sending limit and we dropped the message
	RECIP_REPORT="${RECIP_REPORT}r"
	SUCCESS=$(($SUCCESS||1))
    fi
done

# qmail-rspawn wants to know how many recipients succeeded, and whether the
# message was "accepted". we tell it the message was accepted unless _all_ the
# recipients failed, and the recipients we report on a case-by-case basis
# depending on whether we succeeded in forwarding to the relay

if [ $SUCCESS -eq 1 ] 
then
    printf "$RECIP_REPORT\0KMessage(s) forwarded to relay\0"
else
    printf "$RECIP_REPORT\0ZAll forwarding attempts failed\0"
fi

exit 0

