#!/bin/bash
# ----------------------------------------------------------------------------
#
# $Id: set-blocklist.sh,v 1.3 2000/08/03 22:02:00 roger Exp $
#
# This script updates the blocklist stored within a mail users directory.  It
# is intended to be run from a .qmail file and assumes that context.
#
# Requirements:
#
#     SENDER     the e-mail address of the sending party (from msg envelope)
#
# Returns:
#
#      0         I havn't done anything, try the next .qmail rule
#
#     99         this script has matched the user 
#
#    100         I died a horrible death, don't make the next rule suffer
#                my fate!
#
#    111         I wan't able to do anything, but try again later
#
# ----------------------------------------------------------------------------

#
# The root directory for the mail system
#

QMAIL_ROOT=/freedom/qmail

#
# Some helpful paths
#

QMAIL_CONTROL=${QMAIL_ROOT}/control
QMAIL_BIN=${QMAIL_ROOT}/bin

#
# Make sure that the sender is a nym by matching the domain against our
# list of local domains.
#

for x in $(cat ${QMAIL_CONTROL}/locals)
do
    USER_NAME=${SENDER%@${x}}

    #
    # If SENDER (which is name@domain) doesn't match USER_NAME then
    # then we must have had a domain match.
    #

    if [ "${SENDER}" != "${USER_NAME}" ]
    then
	ACCOUNT=`${QMAIL_BIN}/frname2account "$(cat ${QMAIL_CONTROL}/mailroot)" "${USER_NAME}"`
	if [ -d ${ACCOUNT} ]
	then
	    BLOCKLIST=${ACCOUNT}/blocklist.cdb
	    ${QMAIL_BIN}/reformime -s 1 -e | ${QMAIL_BIN}/txt2cdbmake | ${QMAIL_BIN}/cdbmake ${BLOCKLIST} ${BLOCKLIST}$(date +".tmp.%s")
	    exit 99
	fi

	#
	# We matched the account but it didn't exist!
	#
	# BUG: (Actually, it's just a question)
	#   
	#      What do we do now?
	#

	exit 111
    fi
done

#
# If we get here then we didn't find a match for the user.  The
# sender must have been an internet user.  There is nothing else
# for the mail system to do; so, tell it that we've taken care of
# the message.
#

exit 99

