#!/bin/sh
# ----------------------------------------------------------------------------
#
# $Id: set-option.sh,v 1.2 2000/10/04 12:56:36 marcp Exp $
#
# This script updates the blocklist stored within a mail users directory.  It
# is intended to be run from a .qmail file and assumes that context.
#
# Requirements:
#
#     SENDER     the e-mail address of the sending party (from msg envelope)
#
# Returns:
#
#     99         this script has matched the user 
# ----------------------------------------------------------------------------

#
# The root directory for the mail system
#

QMAIL_ROOT=/freedom/qmail

#
# Some helpful paths
#

QMAIL_CONTROL=${QMAIL_ROOT}/control
QMAIL_BIN=${QMAIL_ROOT}/bin

#
# Make sure that the sender is a nym by matching the domain against our
# list of local domains.
#

for x in $(cat ${QMAIL_CONTROL}/locals)
do
    USER_NAME="${SENDER%@${x}}"

    #
    # If SENDER (which is name@domain) doesn't match USER_NAME then
    # then we must have had a domain match.
    #

    if [ "$SENDER" != "$USER_NAME" ]
    then
	ACCOUNT=`${QMAIL_BIN}/frname2account $(cat ${QMAIL_CONTROL}/mailroot) ${USER_NAME}`
	if [ -d ${ACCOUNT} ]
	then

	    #
	    # Is this a paid only option?
	    #

	    if [ "$1" = "paid" ]
	    then
		ACCOUNT_TYPE=${ACCOUNT}/.type

		#
		# If the account is a paid account then we are permitted
		# to change the contents of the tagline option file.
		#

		if [ ! -f $ACCOUNT_TYPE ]
		then
		    exit 99
		fi

		VALUE=`cat ${ACCOUNT_TYPE} | dd bs=1 count=4 conv=lcase 2> /dev/null`
		if [ "$VALUE" != "paid" ]
	    	then
		    exit 99
		fi
	    fi

	    #
	    # Either this is a paid user or this is not a paid only option
	    #

	    /freedom/qmail/bin/reformime -s 1 -e | (dd bs=1 count=5 conv=lcase 2> /dev/null) >| ${ACCOUNT}/.$2

	    #
	    # Ok, we've done what we can ... there's nothing left for the
	    # mail system to do.
	    #

	    exit 99
	fi

	#
	# We matched the account but it didn't exist!
	#
	# BUG: (Actually, it's just a question)
	#   
	#      What do we do now?
	#
	
	exit 99
    fi
done

#
# If we get here then we didn't find a match for the user.  The
# sender must have been an internet user.  There is nothing else
# for the mail system to do; so, tell it that we've taken care of
# the message.
#

exit 99
