#!/usr/bin/perl

####################################################
# PURPOSE: 
#      Report on the number of local and remote emails sent, 
#      optionnaly in the logging format expected by our old 
#      system.
#
# USAGE: 
#  emails-sent.pl --legacy nym|internet
#        --legacy -  generate our legacy mail system style
#                    report. Specify nym if this the qmail logs are from a nym qmail installation, internet
#                    otherwise. (It is assumed that email sent by nyms and by the internet are not handled by
#                    the same qmail instance.)
#               
# DESCRIPTION:
#   Input (stdin):
#      This script reads from it's stdin a qmail log with tai 
#      formatted timestamps (ie 9675848.4885739) and  which has 
#      been filtered by "matchup".
#
#   Output (stdout>:
#      1) a count of local and remote successfully sent emails
#      or
#      2) (w/ --legacy) A report as per our old mail system format.
#        where nym|internet is assumed to be the sending entity. 
# RETURNS:
#   0 success
#   1 command line invocation error
#   111 unspecified error.
#
# EXAMPLE:
#      emails-sent.pl --legacy internet
#      
#      This will generate a mail log in the format of our old mail
#      system where the sender is assumed to be an internet entity.
#
#      
##############################################################

use Time::Local;


sub help(){

  print "Usage: emails-sent.pl --legacy nym|internet

                 --legacy <nym|internet> : generate our legacy mail system style
                   report. Specify nym if this the qmail logs are from a nym qmail installation, internet
                    otherwise. (It is assumed that email sent by nyms and by the internet are not handled by
                    the same qmail instance.)
               
         ex:  emails-sent.pl --legacy nym
";
  exit 1;
}


my ($localCnt, $remoteCnt);
my ($sender, $legacy);



$legacy = 0;


# parse cmd lin
if($#ARGV != -1  &&  $#ARGV !=1) {
  help;
}

if($#ARGV == 1) {
  if(!($ARGV[0] eq "--legacy")) {
    help;
  } elsif($ARGV[1] ne "nym" && $ARGV[1] ne "internet") {
    help;
  }
  else {
    $sender = $ARGV[1];
    $legacy = 1;
  }
}




# read log input from stdin. should be in matchup format w/ tai timestamps
open(TT,  "cat - | simplifylog.pl |tailocal|");
while(<TT>) {
 
#  example input to parse:
#      2000-07-17 19:28:07.582369500 local
#      2000-07-19 10:02:04.8466500 remote


m/(\d*-\d*-\d* \d*:\d*:\d*)\.[0-9]* (\w*)/;


$stamp = "$1";

my ($receiver);
if($2 eq "local") {
  $receiver = "nym";
} elsif($2 eq "remote") {
  $receiver = "internet";
} else{
  print "Expected local or remote but got: $2\n";
  exit 111;
}


if( $legacy) {
  # we want to format as:
  #    "2000-07-17 19:28:07";"nym";"nym"
  #    "2000-07-19 10:02:04";"nym";"internet"

  print "\"$stamp\"\;\"$sender\"\;\"$receiver\"\n";
      
} else {
  if($receiver eq "internet") {
    $remoteCnt++;
  } elsif($receiver eq "nym") {
    $localCnt++;
  }else {
    print "Expected local or remote but got: $receiver\n";
    exit 111;
  }
}
}
if(! $legacy ) {
  print "local: $localCnt\n";
  print "remote: $remoteCnt\n";
}


exit 0;
