#!/bin/bash

###################################################
#  PURPOSE:
#    A wrapper for qmailanalog tools (and others) that simplifies the task
#    of applying any qmail log analyzer script to
#    qmail logs dated up to some time in the past.
#
#  USAGE:
#     qmail-stat -d QMAIL LOG DIR -n SECONDS  -c COMMAND -- [ARGS]
#                -d  - The name of the directory holding the qmail logs.
#                -n  - Maximum number of seconds logs can be dated.
#	         -c  - Command to apply to filtered log output.
#	         ARGS - optional arguments to be passed to the command specified by -c.
#
#  DESCRIPTION:
#    Input (stdin);
#        A plain unprocessed qmail log data with tai64 timestramp
#        as produced by the "multilog" tool. 
#
#    Output: 
#        The resulting qmail log data that go back up to N seconds in time
#        to which "matchup" has been applied to, with tai timestamps, and to which optionaly
#        the command specified by -c has been applied to with arguments ARGS.
#  EXAMPLE:
#    qmail-stat  -d /var/log/qmail -n 3600  -c recipients
#    
#     This will generate the email volume for all recipients in the last hour.
#
# RETURNS
#   1 command line invocation error
#   0 otherwise
#
#  DEPENDENCIES:
#     time-filter.pl, matchup (see qmailanalog tools)
##############################################################



help() 
{ 
   echo "qmail-stat -d QMAIL LOG DIR -n SECONDS  -c COMMAND -- [ARGS]
                -d  - The name of the directory holding the qmail logs.
                -n  - Maximum number of seconds logs can be dated.
	         -c  - Command to apply to filtered log output.
	         ARGS - optional arguments to be passed to the command specified by -c.
   ";
    
   exit 1;
}


qmailLogDir=

if test $# -lt 6; then
	help;
fi

cmd="";
secs="";

while [ "$1" != "--" ] && [ $# -ne 0 ]; do
     case $1 in
        -n)  shift; 
	     secs=$1;
	     shift;;
	
	-c) shift;
	    cmd=$1;
	    shift;;
	-d) shift;
    	    qmailLogDir=$1;
	    shift;;
        *) help;;
esac
done

if [ $1 ]; then 
  shift;
  args=$@;
fi

time-filter.pl  $qmailLogDir $secs | matchup  | $cmd $args

exit 0;