#!/usr/bin/perl

##############################################
#  
# PURPOSE:
#  Produce a single log from multiple qmail log files, whose
#  entries date back to more than a specified number of seconds 
#  in the past.
#
# USAGE:
#   qmail.pl DIRPATH N
#      DIRPATH - path to a directory holding qmail logs.
#      N  - integer number of seconds; only logs no older than
#           this amount of time will be outputed.
#
# DESCRIPTION:
#   This script takes as arguments a directory
#   holding qmail log files and an integer x number of seconds
#   and outputs to stdout only the qmail log entries  up to
#   are no more than N seconds old.
#
#
# DEPENDENCIES:
#   tai64nfrac (used to convert the supervise time loggin format)
#
# RETURNS
#  1 invocation error (cmd line).
#  0 otherwise
##########################################################



if($#ARGV != 1) {
  print "Usage: qmail.pl DIRPATH N
                 DIRPATH - path to a directory holding qmail logs.
                 N  - integer number of seconds; only logs no older than
                       this amount of time will be outputed.\n
";
  exit 1;
}

$logdir = $ARGV[0];
$delta = $ARGV[1];
$now = time();
$start = $now - $delta;


opendir LOGDIR, $logdir || die "Couldn't open $logdir";


# sort qmail log files by age. This can be done by their names
# since a qmail log file's name corresponds to the date it was
# written in tai64
@logfiles = sort grep {/^\@/} readdir LOGDIR;
push @logfiles, "current";



my @toProcess;

if($#logfiles < 1) {
   print "Error: no qmail log files found in directory: $logdir\n";
   exit 1;
}


foreach $i (@logfiles) {

    $i =~ /\@(.*)\.s/;
    $timestamp = $1;   
    open(PROGRAM, "echo $timestamp | tai64nfrac|") || die "couldn't execute prog";   
   
    while(<PROGRAM>) {
	$secs = $_;	
    }
    if( ($secs - $start) >= 0 ){
	push  @toProcess, "$logdir/$i";
    } 
    close PROGRAM;
}



if($#toProcess != -1) {
  open( TT, "cat  @toProcess | tai64nfrac|");
  while(<TT>) {
    
    /([0-9]*)/;
     if($1 - $start >= 0) {
      print $_;
    } 
  }
}


exit 0;
