# unit tests for the mailquotacheck.c module






$numTest = 7;
print  "1..$numTest\n";

my ($quota, $dir, $msgbytes, $totalbytes, $status, $quotaSrc);
my ($input_dir, $input_quota, $input_email);


sub doTest() {


   open IN, "cat $_[0]/$_[1] | mailquotacheck $_[0] $_[2] --verbose 2>&1|";

   while(<IN>) {   	
	
    if(/Quota-Value:[\s]*([\S]*)/) { $quota =$1;  }
    elsif (/dirPath:[\s]*([\S]*)/){$dir = $1;}
    elsif(/Quota-Src:[\s]*([\S]*)/){$quotaSrc=$1;}
    elsif(/Email-Size:[\s]*([\S]*)/){$msgbytes=$1;}
    elsif(/Total-Size:[\s]*([\S]*)/){$totalbytes=$1;}
    elsif(/Status:[\s]*([\S]*)/){$status=$1;}			
    elsif(/(Error:.*)/){$error =$1; last;}	
  }
}

$input_dir ="/tmp/test";
system("rm -rf $input_dir");

#test1 empty dir, empty email, command line quota is 0 bytes, under-quota
$input_email= "email";
$input_quota="0M";

system("mkdir -p $input_dir/Maildir");
system("mkdir $input_dir/Maildir/new");
system("mkdir $input_dir/Maildir/cur");
system("touch $input_dir/$input_email");

&doTest($input_dir, $input_email, $input_quota);


print "#  $quota $dir $msgbytes $totalbytes $status $quotaSrc\n";

if(($quota != 0) || $dir ne "$input_dir" || $quotaSrc ne "cmd-line" || $msgbytes !=0 || $totalbytes != 0 || $status ne "under-quota") {
     print "not ok\n";
 } else {
     print "ok 1\n";
}



#test2 empty dir, non void email, command line quota, over-quota
system("cat /bin/sh > $input_dir/$input_email"); # create junk email

$input_quota = 1000;
&doTest($input_dir, $input_email, $input_quota);

print "#  $quota $dir $msgbytes $totalbytes $status $quotaSrc\n";
if($quota != $input_quota || $dir ne "$input_dir" || $quotaSrc ne "cmd-line" || $status ne "over-quota") {
     print "not ok\n";
 } else {
     print "ok 2\n";
}



#test 3 non-emtpy dir, 0 byte email, .quota quota, under-quota
$input_quota="300000k";
system("echo $input_quota > $input_dir/.quota");
system("cp mailquotacheck.t  $input_dir/Maildir/new/delme");
system ("> $input_dir/$input_email");

&doTest($input_dir, $input_email, $input_quota);
print "#  $quota $dir $msgbytes $totalbytes $status $quotaSrc\n";
if($quota !=  300000*1024 || $dir ne "$input_dir" || $quotaSrc ne "config-file" || $msgbytes !=0 || $status ne "under-quota") {
     print "not ok\n";
 } else {
     print "ok 3\n";
}



#test 4
system("echo 2 > $input_dir/.quota");
system("cp $input_dir/.quota $input_dir/Maildir/new/delme");
system("cp $input_dir/.quota $input_dir/Maildir/cur/delme");


&doTest($input_dir, $input_email, $input_quota);
print "#  $quota $dir $msgbytes $totalbytes $status $quotaSrc\n";
if($quota != 2 || $dir ne "$input_dir" || $quotaSrc ne "config-file" || $msgbytes !=0 || $status ne "over-quota" || $totalbytes!=4) {
     print "not ok\n";
 } else {
     print "ok 4\n";
}







#test 5. Invalid quota str format.
system("echo -2 > $input_dir/.quota");
system("cp $input_dir/.quota $input_dir/Maildir/new/delme");
system("cp $input_dir/.quota $input_dir/Maildir/cur/delme");

$error="";
&doTest($input_dir, $input_email, $input_quota);
if($error =~ m/Error/ ){
     print "ok 5\n";
 } else {
     print "not ok\n";
}


#test 6 Invalid quota str format.
system("echo -56#M > $input_dir/.quota");
system("cp $input_dir/.quota $input_dir/Maildir/new/delme");
system("cp $input_dir/.quota $input_dir/Maildir/cur/delme");

$error="";
&doTest($input_dir, $input_email, $input_quota);
if($error =~ m/Error/ ){
     print "ok 6\n";
 } else {
     print "not ok\n";
}

#test 7 Invalid quota str format.
system("echo M > $input_dir/.quota");
system("cp $input_dir/.quota $input_dir/Maildir/new/delme");
system("cp $input_dir/.quota $input_dir/Maildir/cur/delme");

$error="";
&doTest($input_dir, $input_email, $input_quota);
if($error =~ m/Error/ ){
     print "ok 7\n";
 } else {
     print "not ok\n";
}



#clean up
system("rm -rf $input_dir");
