/* Copyright (c) 2000, Len Budney. See COPYING for details. */

#include "hostname.h"
#include "stralloc.h"
#include "strerr.h"
#include "tai.h"
#include "taia.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

static
void die_nomem() { strerr_die2x(111,"safecat: fatal: ","out of memory"); }
static
void die_notime() { strerr_die2x(111, "safecat: fatal: ","can't get time"); }

/* 12sept2000plm Use /dev/urandom to generate a random file
   name. Returns 1 upon success, 0 upon failure. */

#define NUMBER_OF_RANDOM_BITS 128

int generate_random_name(stralloc *tmpf)
{
  static const char *random_device = "/dev/urandom";
  static const char *hex_digits = "0123456789abcdef";

  char random_buffer[NUMBER_OF_RANDOM_BITS / 8]; 

  /* number of hex digits, plus null */
  char hex_string_buffer[(NUMBER_OF_RANDOM_BITS / 8) * 2 + 1]; 

  /* pointer into hex_string_buffer */
  char *s; 

  struct stat stat_buffer;
  int fd;
  int i;

  /* check to see if /dev/urandom exists */

  if (stat(random_device, &stat_buffer) == -1) { 
    return 0;
  }

  /* character device? */
  if (!(stat_buffer.st_mode & S_IFCHR)) { 
    return 0;
  }

  /* open it */
  if ((fd = open(random_device, O_RDONLY)) == -1) { 
    return 0;
  }

  /* read from it */
  if (read(fd, (void*)random_buffer, sizeof(random_buffer)) != sizeof(random_buffer)) { 
    return 0;
  }

  s = hex_string_buffer;
  /* convert it to hex digits */
  for (i = 0; i < sizeof(random_buffer); ++i) { 
    *s++ = hex_digits[(random_buffer[i] >> 4) & 0xf];
    *s++ = hex_digits[random_buffer[i] & 0xf];
  }
  
  *s++ = '\0'; /* terminate */

  if (!stralloc_cats(tmpf, hex_string_buffer)) die_nomem();
  if (!stralloc_0(tmpf)) die_nomem();

  return 1;
}

/* ****************************************************************** */
void mk_tempfile(stralloc *tmpf) {
  char host[256];
  char secbuf[11];
  char atto[TAIA_FMTFRAC];
  struct taia now;
  struct tai sec;
  unsigned long int pid;
  int b = 10;

  /* Attempt to create a random file name */
  if (!generate_random_name(tmpf)) {

    /* Otherwise, use the original method */

    pid = (unsigned long)getpid();

    /* Get a microsecond timestamp with which to build a filename. */
    taia_now(&now);
    taia_tai(&now,&sec);

    /* Record the second timestamp on the string. */
    secbuf[fmt_ulong(secbuf,sec.x)] = '\0';
    if (!stralloc_cats(tmpf, secbuf)) die_nomem();
    if (!stralloc_append(tmpf, ".")) die_nomem();
  
    /* Record the microsecond timestamp + PID on the string. */
    taia_fmtfrac(atto,&now);
    atto[b--] = '\0';
    for(; b>4; pid /= 10) {
      atto[b--] = (char) ((pid % 10) + 48);
    }
    if (!stralloc_cats(tmpf, atto)) die_nomem();
    if (!stralloc_append(tmpf, ".")) die_nomem();
  
    /* Copy the hostname to the buffer. */
    get_hostname(host,sizeof(host));
    if (!stralloc_cats(tmpf, host)) die_nomem();
    if (!stralloc_0(tmpf)) die_nomem();
  }
}
/* ****************************************************************** */


