#!/usr/bin/perl 

#
# $Id: run_test_suite.pl,v 1.4 2000/10/04 12:57:12 marcp Exp $

# Harness to execute test scripts and generate test results.
#
# usage: run_test_suite.pl <test_suite>
#
# <test_suite> should be a Perl script of the form

# package Suite;
#
# $scenario = {
#	Scripts => [ 
#		     'connect.t',
#		     ],
#	};
#		
#

# The hash $scenario is expected to contain a key 'Scripts', whose
# value is an array of Test::Harness style test scripts (see the Test
# and Test::Harness modules in the Perl library). . The location of
# test_suite is used to construct a search path for the scripts
# themselves, and modules for inclusion by 'use'
# directives. Directories are appended deepest first. Scripts are
# expected to reside in directories called "scripts". For example, if
# this program is invoked as

# bin/run_test_suite.pl system/suites/t10/without_client/sanity 

# Then a search path 
# [ 'system/suites/t10/without_client', 'system/suites/t10', 'system/suites', 'system', '.' ]
# is constructed.

# The script 'connect.t' will be searched for in
# 'system/suites/t10/without_client/scripts', then in
# 'system/suites/t10/scripts', then in 'system/suites/scripts', and so
# on.

# (Note: this script will execute on win32).

# Each directory in the search path will be added as a -I<directory>
# option to the invocation of the test script, so that the same test
# script can be executed differently according to the location of the
# modules that it uses.

use lib '../lib/perl';
use lib './lib/perl';

use strict;
use Getopt::Long;
use Test::Harness;

use File::Basename;

my($scripts_dir_name) = 'scripts';

# hack to work around broken WCOREDUMP...
sub Test::Harness::WCOREDUMP {
    my($ret) = shift;
    return $ret & 0200;
}


sub usage() 
{
    die("usage: $0 <password> <test_suite>\n");
}

# Grab options...
my($verbose) = 0;
my($result) = GetOptions("verbose!" => \$verbose);

$Test::Harness::verbose = $verbose;

my($test_suite) = shift;

usage() unless defined $test_suite;

die("couldn't find test suite `$test_suite': $!\n") unless -e $test_suite;

# evaluate the test suite.
unless (eval {require "$test_suite"}) {
    die "couldn't load suite `$test_suite': $@";
}

#print $Suite::scenario->{Scripts}->[0];

# construct a search path for scripts and perl modules based on the
# test suite's location; we want the tree to be searched depth first
# when the test script is invoked.

my(@search_path) = ();

my($path) = $test_suite;

do {
    $path = dirname($path);
    push (@search_path, $path);
} while ($path ne '.' && $path ne "");


# add the default lib path 
if (-d './lib/perl') {
    push (@search_path, './lib/perl');
}

if (-d '../lib/perl') {
    push (@search_path, '../lib/perl');
}

# Add include paths to switches.
foreach $path (@search_path) {
    $Test::Harness::switches .= " -I$path ";
}

# construct list of test scripts
my(@test_scripts) = ();
my($script);
my($separator) = '/';

if ($^O =~ /MS(DOS|Win32)/) {
    $separator = '\\';
}
my($script_path);

foreach $script (@{$Suite::scenario->{Scripts}}) {
    # look for the test script, in a director called scripts, using the search_path
    foreach $path (@search_path) {
	$script_path = $path . $separator . $scripts_dir_name . $separator . $script;
	if (-e $script_path) {
	    push @test_scripts, $script_path;
	}
    }
}

# emit some information about time, host location, configuration,
# exact scripts we are running, log files...

#foreach $script (@test_scripts) {
#    print "$script\n";
#}

runtests(@test_scripts);


