package Connection::Base;

# $Id: Base.pm,v 1.2 2000/10/04 12:57:15 marcp Exp $

use strict;
use Mail_Test;

use vars qw($VERSION);
$VERSION = 1.0;


###############################################################################
#
#   Connection Base Class
#
#   This class is the base class for all protocol connection classes
#
###############################################################################

#
####################
# Private Functions

sub _initialize {
  my $self = shift;
  return $self;
}

#
####################
# Constructor

sub new {
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = { @_ };

  bless($self, $class);
  $self->_initialize();

  return $self;
}

#
####################
# Destructor


#
####################
# Accessor Methods

#
# access the user to use for this connection
sub user {
  my($self) = shift;
  my($user) = shift;

  $self->{User} = $user if defined($user);
  return $self->{User};
}

#
# return the last server message received.
sub last_msg {
    my($self) = shift;

    return undef unless defined($self->{Connection});
    return $self->{Connection}->message();
}

#
# quit the current session and close the connection.
sub quit {
  my($self) = shift;
  my($return_value) = $self;

  if (defined($self->{Connection})) {
      $self->{Connection}->quit() || ($return_value = undef);
  }
  
  $self->{Connection} = undef;

  return $return_value;
}

return 1;

#
####################
# Documentation

=head1 NAME

Connection::Base - Zero Knowledge Protocol Connection Base Class

=head1 SYNOPSIS

    use Connection::Base;

    $base = new Connection::Base();

=head1 REQUIRES

    Perl5.004+

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the base class for all C<Connection::> classes.  The
C<Connection::Base> class implements the methods common to all 
classes in this hiearchy.

The Connection::Base does not sub-class any other classes.

=head1 CONSTRUCTOR

=over 4

=item new Connection::Base()

This is the constructor for the Connection::Base object.  This object
should not be instatiated directly as it only serves as a parent
class for all other C<Connection::> classes.

Example:

  $base = new Connection::Base();

=back

=head1 METHODS

=over 4

=item user ( USER )

Return the user object associated with this connection.  If
USER is supplied, then it wil be used for future connections, but will 
not affect the current news server connection.

=item last_msg ()

Return the last network message received from the remote server

=item quit()

Close the connection to remote server.

=back

=head1 CLASS VARIABLES

None Present

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
