package Domain::Base;

# $Id: Base.pm,v 1.2 2000/10/04 12:57:16 marcp Exp $

use strict;

use vars qw( $VERSION );
$VERSION = 1.0;


###############################################################################
#
#   Domain Base Class
#
#   This class is the base class for all network domain classes.
#
###############################################################################

#
####################
#

sub _initialize {
    my($self) = shift;
    return $self;
}

#
####################
# Constructor
sub new {
    my ($class) = shift;
    my ($self) = {};

    bless ($self, $class);

    $self->_initialize();
    return $self;
}

#
####################
# Destructor

#
####################
# Accessors

#
# current ZKS network domain name
sub network_domain {
    my($self) = shift;

    $self->{Network_Domain} = shift if @_;
    return $self->{Network_Domain};
}

#
# server "hello" domain; used mostly for SMTP servers
sub hello_domain {
    my($self) = shift;

    $self->{Hello_Domain} = shift if @_;
    return $self->{Hello_Domain};
}

#
# temporary alias for "nym_news_domain"
sub news_domain {
    my ($self) = shift;

    $self->{News_Domain} = shift if @_;
    return $self->{News_Domain};
}

#
# temporary alias for "hello_domain"
sub mail_domain {
    my($self) = shift;

    $self->{Mail_Domain} = shift if @_;
    return $self->{Mail_Domain};
}

return 1;

__END__

#
####################
# Documentation

=head1 NAME

Domain::Base - Zero Knowledge Network Domain Base Class

=head1 SYNOPSIS

    use Domain::Base;

    $base = new Domain::Base();

=head1 REQUIRES

    Perl5.004+
    DomainConfig

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the base class for all C<Domain::> classes.  The
C<Domain::Base> class implements the methods common to all 
classes in this hiearchy.

The Domain::Base does not sub-class any other classes.

=head1 CONSTRUCTOR

=over 4

=item new Domain::Base()

This is the constructor for the Domain::Base object.  This object
should not be instatiated directly as it only serves as a parent
class for all other C<Domain::> classes.

Example:

  $base = new Domain::Base();

=back

=head1 METHODS

=over 4

This class does not contain instances methods.

=back

=head1 CLASS VARIABLES

None Present

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
