package Domain::NymUser;

# $Id: NymUser.pm,v 1.2 2000/10/04 12:57:16 marcp Exp $

use strict;
use Domain::NetUser;

use vars qw( @ISA $VERSION );
@ISA = qw( Domain::NetUser );
$VERSION = 1.0;

################################################################################
#
#   Domain::NymUser Class
#
#   This class encapsulates network domain for User::Nym objects.
#
################################################################################

#
####################
# Private Functions

sub _initialize {
  my $self = shift;

  $self->SUPER::_initialize();
  return $self;
}

#
####################
# Constructor

sub new {
    my ($class) = shift;
    my ($self) = {};

    bless ($self, $class);

    $self->_initialize();
    return $self;
}

#
####################
# Destructor


#
####################
# Accessor Methods

return 1;

__END__

#
####################
# Documentation

=head1 NAME

Domain::NymUser - Zero Knowledge Network NYM Domain Base Class

=head1 SYNOPSIS

use Domain::NymUser;

$domain = new Domain::NymUser();

=head1 REQUIRES

    Perl5.004+
    User::Base
    Server::Base (Sub-class)
    Domain::Base (Sub-class)

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the domain information class for network users.  This class
encapsuates Zero Knowledge, network specific domain information for
Zero Knowledge network users.

The Domain::NymUser is a sub-class of Domain::Base.

=head1 CONSTRUCTOR

=over 4

=item new Domain::Base()

This is the constructor for the Domain::NymUser object. A new domain object 
will be automagically initialized from the Zero Knowledge network specific 
parameter files.

C<OPTIONS> are passed in a hash like fashion, using key and value pairs
and may be used to override the default items.  This is B<NOT> 
recommended as the netwok specific parameters associated with this
domain are guaranteed to be correct for the currently active Zero 
Knowledge network.

Possible options are:

B<Hello_Domain> - The domain name as presented to [E]SMTP servers on initial
                  contact.

B<Mail_Domain> - The domain name used for construction of Internet email
                 addresses.

B<News_Domain> - The domain name used for construction of hostnames for Internet 
                 news servers.

Example:

$domain = new Domain::NymUser( Hello_Domain => 'test.zks.net',
			       Mail_Domain => 'dev.zks.net');

=back

=head1 METHODS

=over 4

=item hello_domain ( [ DOMAIN ] )

Returns the 'hello' domain name associated with this domain object.
If DOMAIN is supplied, then it will be set by the domain object.

=item mail_domain ( [ DOMAIN ] )

Returns the mail domain name associated with this domain object.  
If DOMAIN is supplied, then it will be set by the domain object.

=item news_domain ( [ DOMAIN ] )

Returns the news domain name associated with this domain object.  
If DOMAIN is supplied, then it will be set by the domain object.

=back

=head1 CLASS VARIABLES

None Present

=head1 SEE ALSO

L<Domain::NymUser>

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
