package Factory::Base;

# $Id: Base.pm,v 1.2 2000/10/04 12:57:17 marcp Exp $

use strict;
use Mail_Test;

use vars qw($VERSION);
$VERSION = 1.0;


###############################################################################
#
#   Factory Base Class
#
#   This class is the base class for all object factory classes
#
###############################################################################

#
####################
# Private Functions

sub _initialize {
    my($self) = shift;
    return $self;
}

#
####################
# Constructor

sub new {
    my ($class) = shift;
    my ($self) = { @_ };

    bless ($self, $class);

    $self->_initialize();
    return $self;
}

#
####################
# Destructor

#
####################
# Accessors

return 1;

__END__

#
####################
# Documentation

=head1 NAME

Factory::Base - Zero Knowledge Network Factory Base Class

=head1 SYNOPSIS

    use Factory::Base;

    $base = new Factory::Base();

=head1 REQUIRES

    Perl5.004+

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the base class for all C<Factory::> classes.  The
C<Factory::Base> class implements the methods common to all 
classes in this hiearchy.

The Factory::Base does not sub-class any other classes.

=head1 CONSTRUCTOR

=over 4

=item new Factory::Base()

This is the constructor for the Factory::Base object.  This object
should not be instatiated directly as it only serves as a parent
class for all other C<Factory::> classes.

Example:

  $base = new Factory::Base();

=back

=head1 METHODS

=over 4

This class does not contain instances methods.

=back

=head1 CLASS VARIABLES

None Present

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
