package Factory::DomainFactory;

# $Id: DomainFactory.pm,v 1.2 2000/10/04 12:57:17 marcp Exp $

use strict;
use Suite_Specific;
use Mail_Test;

use vars qw( @ISA $VERSION );
@ISA = qw( Factory::Base );
$VERSION = 1.0;


###############################################################################
#
#    Domain Factory Class
#
#    This is the network domain factory class
#
###############################################################################


#
####################
# Private Functions

#
####################
# Constructor

#
####################
# Destructor

#
####################
# Accessors

sub create_domain {
    my($self) = shift;
    my($user) = shift;

    return undef unless defined($user);

    # grab the suite specific domain entries
    my($suite_domains) = $DomainConfig::domains;

    # create the appropriate domain object
    my($domain);
    if ( $user->isa("User::Nym") ) {
	$domain = new Domain::NymUser();
	$domain->mail_domain( $suite_domains->{Nym_Mail_Domain} );
	$domain->news_domain( $suite_domains->{Nym_News_Domain} );
    }

    elsif ( $user->isa("User::Net") ) {
	$domain = new Domain::NetUser();
	$domain->mail_domain( $suite_domains->{User_Mail_Domain} );
	$domain->news_domain( $suite_domains->{User_News_Domain} );
    }

    else {
	$domain = new Domain::Base();
    }

    # "HELLO" and network domains are common to all user types
    $domain->hello_domain( $suite_domains->{Hello_Domain} );
    $domain->network_domain( $suite_domains->{Network_Domain} );

    return $domain;
}


return 1;

__END__

#
####################
# Documentation

=head1 NAME

Factory::DomainFactory - Zero Knowledge Domain Factory Class

=head1 SYNOPSIS

    use Factory::DomainFactory;

    $base = new Factory::DomainFactory();

=head1 REQUIRES

    Perl5.004+
    Factory::Base
    Domain::Base (Sub-class)

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the factory class for all domain classes.  The class methods
exposed by Factory::DomainFactory create specific instances of the
various domain classes.

The Factory::DomainFactory is a sub-class of Factory::Base.

=head1 CONSTRUCTOR

=over 4

=item new Factory::DomainFactory()

This is the constructor for the Factory::DomainFactory object.

Example:

  $factory = new Factory::DomainFactory();

=back

=head1 METHODS

=over 4

=item create_domain ( USER )

Create an Domain::Base object for the specified user.  If C<USER> is not
supplied then C<undef> will be returned.

=back

=head1 CLASS VARIABLES

None Present

=head1 SEE ALSO

L<Factory::Base>, L<Domain::NymUser>, L<Domain::NetUser>

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
