package Factory::MessageFactory;

# $Id: MessageFactory.pm,v 1.2 2000/10/04 12:57:17 marcp Exp $

use strict;
use Suite_Specific;
use Mail_Test;

use vars qw( @ISA $VERSION );
@ISA = qw(Factory::Base);
$VERSION = 1.0;


###############################################################################
#
#    Message Factory Class
#
#    Create a message object for each type of supported message.
#
###############################################################################

#
####################
# Private Functions

#
####################
# Constructor

#
####################
# Destructor

#
####################
# Accessors

#
# create a new Internet mail message
sub create_MAIL_message {
  my($self) = shift;
  my($message) = new Message::MailMessage();
  return $message;
}

#
# create a new Internet news message
sub create_NEWS_message {
  my($self) = shift;
  my($message) = new Message::NewsMessage();
  return $message;
}


return 1;

__END__

#
####################
# Documentation

=head1 NAME

Factory::MessageFactory - Zero Knowledge Message Factory Class

=head1 SYNOPSIS

    use Factory::MessageFactory;

    $base = new Factory::MessageFactory();

=head1 REQUIRES

    Perl5.004+
    Factory::Base
    Message::Base (Sub-class)
    Domain::Base (Sub-class)
    Server::Base (Sub-class)
    User::Base (Subclass)

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the factory class for all Message classes.  The class methods
exposed by Factory::MessageFactory create specific instances of the
various message classes.

The Factory::MessageFactory is a sub-class of Factory::Base.

=head1 CONSTRUCTOR

=over 4

=item new Factory::MessageFactory()

This is the constructor for the Factory::MessageFactory object.

Example:

  $factory = new Factory::MessageFactory();

=back

=head1 METHODS

=over 4

=item create_MAIL_message ()

Create an Message::MailMessage object.  The returned message object is
not initialized beyond the default initialization.

=back

=head1 CLASS VARIABLES

None Present

=head1 SEE ALSO

L<Factory::Base>, L<Message::MailMessage>, L<Message::NewsMessage>

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
