package Message::NewsMessage;

# $Id: NewsMessage.pm,v 1.2 2000/10/04 12:57:22 marcp Exp $

use strict;
use MIME::Parser;
use Message::Header;
use Message::Content;

use vars qw( @ISA $VERSION );
@ISA = qw( Message::MailMessage );
$VERSION = 1.0;


################################################################################
#
#   NewsMessage Class
#
#   This class implements a network news message.
#
################################################################################

#
####################
# Private Functions

sub _initialize {
  my $self = shift;

  $self->SUPER::_initialize();

  # create a standard header/content combo
  $self->{Header} = new Message::Header unless defined($self->{Header});
  $self->{Content} = new Message::Content unless defined($self->{Content});

  return $self;
}

#
####################
# Constructor

sub new {
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = { @_ };

  bless($self, $class);
  $self->_initialize();

  return $self;
}

#
####################
# Destructor


#
####################
# Accessor Methods

sub smells_like_a_cacophonix_newsgroup_message {
    my($self) = shift;
    return $self;
}

sub tastes_like_a_cacophonix_newsgroup_message {
    my($self) = shift;
    return $self;
}

return 1;

__END__

#
####################
# Documentation

=head1 NAME

Message::NewsMessage - Zero Knowledge News Message Class

=head1 SYNOPSIS

    use Message::NewsMessage;

    $base = new Message::NewsMessage();

=head1 REQUIRES

    Perl5.004+
    Message::Base
    User::Base (Subclass)

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the mail message class.  This class encapsulates a single
mail message .  This class contains Message::Header and 
Message::Content classed to assist with the parsing and storage of
the two message parts.

The Message::NewsMessage is is a subclass of Message::Base.

=head1 CONSTRUCTOR

=over 4

=item new Message::NewsMessage()

This is the constructor for the Message::Header object. A new 
header object is created with the I<new> method and is accessed
through the resulting object.

=back

=head1 METHODS

=over 4

=item empty ()

Empty the message header & contents.  This has the effect of rendering
the message completely empty.

=item header( HEADER )

Return the message header object associated with this message.  If HEADER is
supplied, then it will be set to the new message header.

=item content( CONTENT )

Return the message content object associated with this message.  If
CONTENT is supplied, then it will be set to the new message content.

=item parse( MESSAGE_TEXT )

Parse the supplied message text and store internally.  If MESSAGE_TEXT
is not supplied then no operation will be performed and C<undef> will
be returned.

=item freedomize ()

Transform the mail message into a ZKS Fredeom network specific mail
message.

=item as_list()

Return the raw message text as a list of strings.  The returned text
complies with RFC822 header constructs and with the content in compliance
RFC2045, RFC2047, RFC2231 and RFC2184 (MIME).

=item print()

Print the raw message text as with I<as_list> but output the resulting
list of strings to the default output device (stdout).

=back

=head1 CLASS VARIABLES

None Present

=head1 SEE ALSO

L<Message::Base>, L<Message::Header>, L<Message::Content>, L<Mail::Internet>

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
