package Server::MailServer;

# $Id: MailServer.pm,v 1.2 2000/10/04 12:57:29 marcp Exp $

use strict;
use Server::Base;

use vars qw( @ISA $VERSION );
@ISA = qw( Server::Base );
$VERSION = 1.0;



################################################################################
#
#   Server::MailServer Class
#
#   This class implements basic mail server informatiom.
#
################################################################################

#
####################
# Private Functions

sub _initialize {
  my $self = shift;

  $self->SUPER::_initialize();
  return $self;
}

#
####################
# Constructor

sub new {
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = { @_ };

  bless($self, $class);
  $self->_initialize();

  return $self;
}

#
####################
# Destructor


#
####################
# Accessor Methods


return 1;

__END__

#
####################
# Documentation

=head1 NAME

Server::MailServer - Zero Knowledge Mail Server Information Class

=head1 SYNOPSIS

    use Server::MailServer;

    $server = new Server::MailServer();

=head1 REQUIRES

    Perl5.004+
    ServerConfig
    Server::Base
    User::Base (Subclass)

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This is the mail server information class.  This class encapsulates
Zero Knowledge, network specific mail server information.

The Server::MailServer is a sub-class of Server::Base.

=head1 CONSTRUCTOR

=over 4

=item new Server::MailServer()

This is the constructor for the Server::MailServer object. A new 
server object will be automagically initialized from the Zero Knowledge 
network specific parameter files.

C<OPTIONS> are passed in a hash like fashion, using key and value pairs
and may be used to override the default items.  This is B<NOT> 
recommended as the netwok specific parameters associated with this
server are guaranteed to be correct for the currently active Zero Knowledge
network.

Possible options are:

B<Name> - The fully qualified hostname.

B<Port> - The network port this server respresents.

B<Timeout> - The connection timeout.

Example:

    $server = new User::Base( Name => 'mailhost.zks.net',
			      Port => '6092',
			      Timeout => '120');

=back

=head1 METHODS

=over 4

=item name( NAME )

Return the short-form hostname for this server information object.
If NAME is supplied then it will be set internally for this server
object.

=item port( PORT )

Return the port number associated with this server object.  If
PORT is supplied then it will be set internally for this server
object.

=item timeout( NUMBER )

Return the connection timeout associated with this server object.
If NUMBER is supplied, then it will be set internally by this 
server object.

=item fqdn( NAME )

Return the fully qualified hostname for this server object.  If
NAME is supplied then it will be set internally for this server object.

=item print()

Print the contents of the server object to the default output device
(stdout).

=back

=head1 CLASS VARIABLES

None Present

=head1 SEE ALSO

L<Server::Base>

=head1 AUTHOR

Kris Smith <krissmi@zeroknowledge.com>
Philippe McLean <phillipem@zeroknowledge.com>

=head1 COPYRIGHT

Copyright 2000 Zero Knowledge Systems, Inc.  All rights reserved

=cut
