#!/usr/bin/expect -f

#Use this script to generate all the necessary mstone config files to simulate a given load.  Look at load.conf as a sample input file for this script.  The ouput of this script will be .wld files of the form <load_name>.wld, <load_name>NtoI.wld etc..., as well as a directory <load_name> full of .wld files.  All these generated .wld files must be moved over to the mstone/conf directory from where you will be running mstone. To then generate that load on a given network move , use run_load.sh (found in top level of mstone directory).
# It should be noted that this will generate a load slightly less than specified as rates increase.
# For heavy loads, addional client machines should be added by manually editing <load_name>.wld

source $env(ZK_FREEDOM_ROOT)/tools/server-bvt/inc/general.tcl
source $env(ZK_FREEDOM_ROOT)/tools/server-bvt/inc/defs.tcl
source $env(ZK_FREEDOM_ROOT)/tools/server-bvt/inc/netinfo.tcl

proc sort_weighted_data {context data_index} {

  upvar $context Context

  set Context($data_index,num) [expr [llength $Context($data_index)]/2]

  set total_weight 0

  for {set i 0} {$i<[llength $Context($data_index)]} {incr i +2} {
    set Context($data_index,[expr $i/2],data) [lindex $Context($data_index) $i]
    set Context($data_index,[expr $i/2],weight) [lindex $Context($data_index) [expr $i + 1]]
  #strip off brackets
    set Context($data_index,[expr $i/2],weight) [string trim $Context($data_index,[expr $i/2],weight) "()"]
    set total_weight [expr $total_weight + $Context($data_index,[expr $i/2],weight)]
  }

  if {$total_weight != 100} {
    puts "ERROR: wieghted data for $data_index does not add up to 100 - please check config file"
    exit
  }
}
     

# ******************** PROCESSES THAT ARE NEEDED
# - make messsage size .wld file -> take in path
# - make msg per login file -> take in path, automatically include appropriate size files. -> rate will be defined in here
# - make rcpts per msg file -> take in path, automatically include approriate msg per login files.
# - make path file (individually - this will include any generic mail protocol stuff so that you don't have to repeat it for each size 

proc init_mail_load {MailLoad} {

  upvar $MailLoad Load
  global network ports

  set dummylog [open "/dev/null/" w]

  netinfo_GetEntityInfo dummylog pop $network eInfo
  netinfo_GetEntityInfo dummylog imep $network eInfo
  netinfo_GetEntityInfo dummylog nmta $network eInfo

  close $dummylog

  set Load(NetworkPopServer) $eInfo(pop,0,ipaddr)
  set Load(NetworkPopServerPort) $ports(pop)
  set Load(ImepServer) $eInfo(imep,0,ipaddr)
  set Load(ImepPort) $ports(imep)
  set Load(NmtaServer) $eInfo(nmta,0,ipaddr)
  set Load(NmtaPort) $ports(nmta)

}

proc gen_size_config {delivery_path config_dir MailLoad} {

  upvar $MailLoad Load
  global network

  set LocalConfig [open "./$config_dir/$delivery_path.wld" w]

  if {$Load(NumExternalAccounts) == 1} {
     set ExtBasenameAppend ""
  } else {
     set ExtBasenameAppend "%ld"
  }

  for {set i 0} {$i<$Load($delivery_path.MessageSize,num)} {incr i +1} {
    set size $Load($delivery_path.MessageSize,$i,data)
    set weight $Load($delivery_path.MessageSize,$i,weight)
    puts $LocalConfig "<include ./conf/$config_dir/$delivery_path$size.wld>"
    set LocalSizeConfig($i) [open "./$config_dir/$delivery_path$size.wld" w]
    if {$delivery_path == "Pop"} {
      puts $LocalSizeConfig($i) "<POP3>"
      puts $LocalSizeConfig($i) "weight $weight"
      puts $LocalSizeConfig($i) "passwdfile passwords.$Load(NymBasename)$size.msg"
      puts $LocalSizeConfig($i) "loginFormat $Load(NymBasename)$size%ld"
      puts $LocalSizeConfig($i) "useEHLO 0"
      puts $LocalSizeConfig($i) "useAUTHLOGIN 0"
      puts $LocalSizeConfig($i) "server $Load(NetworkPopServer)"
      puts $LocalSizeConfig($i) "portNum $Load(NetworkPopServerPort)"
      puts $LocalSizeConfig($i) "numLogins $Load(NumNymAccounts)"
    } else {
      puts $LocalSizeConfig($i) "<SMTP>"
      puts $LocalSizeConfig($i) "weight $weight"
      if {$delivery_path == "NtoN" || $delivery_path == "NtoI"} {
        puts $LocalSizeConfig($i) "useEHLO 1"
        puts $LocalSizeConfig($i) "useAUTHLOGIN 1"
        puts $LocalSizeConfig($i) "loginFormat $Load(NymBasename)%ld"
        puts $LocalSizeConfig($i) "smtpMailFromFormat $Load(NymBasename)%ld@$Load(NetworkDomain)"
        puts $LocalSizeConfig($i) "server $Load(NmtaServer)"
        puts $LocalSizeConfig($i) "portNum $Load(NmtaPort)"
        puts $LocalSizeConfig($i) "numLogins $Load(NumNymAccounts)"
        if {$delivery_path == "NtoI"} {
          puts $LocalSizeConfig($i) "numAddresses $Load(NumExternalAccounts)"
          puts $LocalSizeConfig($i) "file /var/tmp/en-$size.msg"
          puts $LocalSizeConfig($i) "addressFormat $Load(ExternalAccountBasename)$ExtBasenameAppend@$Load(ExternalNetworkDomain)"
        } else {
          puts $LocalSizeConfig($i) "numAddresses $Load(NumNymAccounts)"
          puts $LocalSizeConfig($i) "file /var/tmp/mime$size.msg"
          puts $LocalSizeConfig($i) "addressFormat $Load(NymBasename)$size%ld@$Load(NetworkDomain)"
        }
      } else {
        puts $LocalSizeConfig($i) "file /var/tmp/en-$size.msg"
        puts $LocalSizeConfig($i) "useEHLO 0"
        puts $LocalSizeConfig($i) "useAUTHLOGIN 0"
        puts $LocalSizeConfig($i) "loginFormat $Load(ExternalAccountBasename)$ExtBasenameAppend"
        puts $LocalSizeConfig($i) "smtpMailFromFormat $Load(ExternalAccountBasename)$ExtBasenameAppend@$Load(ExternalNetworkDomain)"
        puts $LocalSizeConfig($i) "server $Load(ImepServer)"
        puts $LocalSizeConfig($i) "portNum $Load(ImepPort)"
        puts $LocalSizeConfig($i) "numLogins $Load(NumExternalAccounts)"
        puts $LocalSizeConfig($i) "numAddresses $Load(NumNymAccounts)"
        puts $LocalSizeConfig($i) "addressFormat $Load(NymBasename)$size%ld@$Load(NetworkDomain)"
      }
    }
    if {$Load($delivery_path.MessageRate) == 0} {
       puts $LocalSizeConfig($i) "numLoops 0"
       set loopDelay 0
     } elseif {$Load($delivery_path.MessageRate) > 1} {
# If a rate of more than a message per second is required, will use multiple clients for that path
       set loopDelay 1
     } else {
       set loopDelay [expr 1.0/$Load($delivery_path.MessageRate)]
     }
     puts $LocalSizeConfig($i) "loopDelay $loopDelay"
     if {$delivery_path == "Pop"} {
       puts $LocalSizeConfig($i) "</POP3>\n"
     } else {
       puts $LocalSizeConfig($i) "</SMTP>\n"
     }

     close $LocalSizeConfig($i)
  }

  close $LocalConfig


}

proc usage {} {

  puts "usage: ./load_setup.tcl <load config file> <network> (client_count)"
  exit
}


#************* MAIN ************

set num_clients 4

if {$argc < 2} {usage}

set config_file [lindex $argv 0]
set network [lindex $argv 1]

if {$argc > 2} {
  set num_clients [lindex $argv 2]
}

# SET DEFAULT VALUES (ports)
init_mail_load MailLoad

#PARSE THE CONFIG FILE
puts "[ParseConfig $config_file MailLoad] items returned from $config_file"

#SET THE WEIGHTED VARIABLES UP
foreach name [array names MailLoad] {
  if {[llength $MailLoad($name)] > 1} {
    sort_weighted_data MailLoad $name
  }
}

# MAKE A DIRECTORY TO HOLD ALL GENERATED MSTONE CONFIG FILES
catch {exec mkdir $MailLoad(Title)}

set path(0) NtoN
set path(1) NtoI
set path(2) ItoN
set path(3) Pop

#DumpContext MailLoad

# Generate config file for each different message path
gen_size_config NtoN $MailLoad(Title) MailLoad
gen_size_config ItoN $MailLoad(Title) MailLoad
gen_size_config NtoI $MailLoad(Title) MailLoad
gen_size_config Pop $MailLoad(Title) MailLoad

# Generate the general config file - also make one for each path
set GenConfig [open "./$MailLoad(Title).wld" w]

puts $GenConfig "<CONFIG>"
puts $GenConfig "rampTime 1s"
puts $GenConfig "time 1s"
puts $GenConfig "telemetry 0"
puts $GenConfig "RSH /usr/local/bin/ssh"
puts $GenConfig "RCP /usr/local/bin/scp -p"
puts $GenConfig "</CONFIG> \n"


puts $GenConfig "<CLIENT HOSTS=localhost>"
puts $GenConfig "</CLIENT>\n"

puts $GenConfig "<DEFAULT>"
puts $GenConfig "usePasswdFile 1"
puts $GenConfig "passwdBASE64 0"
puts $GenConfig "LoginOnly 0"
puts $GenConfig "passwdFile passwords.$MailLoad(NymBasename).msg"
puts $GenConfig "numLogins $MailLoad(NumNymAccounts)"
puts $GenConfig "firstLogin 0"
puts $GenConfig "numAddresses $MailLoad(NumNymAccounts)"
puts $GenConfig "firstAddress 0"
puts $GenConfig "NumRecips 1"
puts $GenConfig "</DEFAULT>\n"

close $GenConfig

set Config(all) [open "./$MailLoad(Title)all.wld" w]
puts $Config(all) "<includeOnce ./conf/$MailLoad(Title).wld>"

foreach name [array names path] {
  set Config($name) [open "./$MailLoad(Title)$path($name).wld" w]
  puts $Config($name) "<CONFIG>"
  if {$MailLoad($path($name).MessageRate) > 1} {
    puts $Config($name) "clientCount [expr round($MailLoad($path($name).MessageRate))]" 
  } else {
    puts $Config($name) "clientCount 1" 
  } 
  puts $Config($name) "</CONFIG>"
  puts $Config($name) "<includeOnce ./conf/$MailLoad(Title).wld>"
  puts $Config($name) "<include ./conf/$MailLoad(Title)/$path($name).wld>"
  puts $Config(all) "<include ./conf/$MailLoad(Title)/$path($name).wld>"
}

foreach name [array names Config] {
  close $Config($name)
}







exit
