# $Id: availability.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

use strict;
use Test;

#use lib('../../lib/perl');
#use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

BEGIN { plan tests => 8; }

my($banner) = "";
my($test_count) = 0;
my($result) = undef;

# Create factories.
my($connection_factory) = new Factory::ConnectionFactory();
my($server_factory) = new Factory::ServerFactory();
my($user_factory) = new Factory::NymFactory();

# create server objects
my($imep_servers) = $server_factory->create_all_IMEP_servers();
my($imepb_servers) = $server_factory->create_all_IMEP_B_servers();
my($nmta_servers) = $server_factory->create_all_NMTA_servers();
my($nmtab_servers) = $server_factory->create_all_NMTA_B_servers();
my($popz_servers) = $server_factory->create_all_POPZ_servers();
my($relay_servers) = $server_factory->create_all_RELAY_servers();
my($nntp_servers) = $server_factory->create_all_NNTP_servers();

# create connection objects
my($smtp_conn) = $connection_factory->create_SMTP_connection();
my($popx_conn) = $connection_factory->create_POPX_connection();
my($nntp_conn) = $connection_factory->create_NNTP_connection();

# create the test user account
my($test_nym) = $user_factory->create_random_valid_nym();

###############################################################################
#
#    Test 1 - check IMEP Servers
#
###############################################################################
sub availability_1 {
    my($imep_greeters_available) = 0;

    print "\n########## IMEP Mail Servers\n\n";

    foreach my $greeter (@{$imep_servers}) {
	print "Trying connection to " .
	  $greeter->name() . ":" . $greeter->port() . "\n";
	$test_nym->smtp_server( $greeter );
	$result = $smtp_conn->connect($test_nym);
	if (!defined($result) ) {
	    print "SMTP Server (IMEP) Connection Failed\n";
	    last;
	}

	$banner = $smtp_conn->banner();
	if ( !($banner =~ m/ESMTP/) ) {
	    print "Invalid greeting banner for IMEP " . $greeter->name . "\n";
	    last
	}
	$smtp_conn->quit();
	$imep_greeters_available++;
    }
    print("\n");

    if ( $imep_greeters_available ) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 2 - check IMEP-B Servers
#
###############################################################################
sub availability_2 {
    my($imep_backends_available) = 0;

    print "\n########## IMEP BACKEND Mail Server\n\n";

    foreach my $backend (@{$imepb_servers}) {
	print "Trying SMTP (IMEP-B) connection to " .
	  $backend->name() . ":" . $backend->port();
	my($socket) = IO::Socket::INET->new(PeerAddr => $backend->name(),
					    PeerPort => $backend->port(),
					    Timeout => $backend->timeout(),
					    Proto => 'tcp',
					   );
	# check for successful connection
	if (defined($socket)) {
	    print "...success.\n";
	    $socket->close();
	    $imep_backends_available++;
	}
	else {
	    print "...failure.\n";
	}
    }
    print "\n";

    if ( $imep_backends_available ) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 3 - check NMTA Servers
#
###############################################################################
sub availability_3 {
    my($nmta_greeters_available) = 0;

    print "\n########## NMTA Mail Server\n\n";

    foreach my $greeter (@{$nmta_servers}) {
	print "Trying SMTP (NMTA) connection to " .
	  $greeter->name() . ":" . $greeter->port() . "\n\n";
	$test_nym->smtp_server( $greeter );
	$result = $smtp_conn->connect($test_nym);
	if (!defined($result) ) {
	    print "SMTP Server (NMTA) Connection Failed\n";
	    last;
	}
	
	$banner = $smtp_conn->banner();
	if ( !($banner =~ m/ESMTP/) ) {
	    print "Invalid greeting banner for NMTA " . $greeter->name . "\n";
	    last
	}
	$smtp_conn->quit();
	$nmta_greeters_available++;
    }
    print("\n");

    if ( $nmta_greeters_available ) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 4 - check NMTA-B Servers
#
###############################################################################
sub availability_4 {
    my($nmta_backends_available) = 0;

    print "\n########## NMTA BACKEND Mail Server\n\n";

    foreach my $backend (@{$nmtab_servers}) {
	print "Trying SMTP (NMTA-B) connection to " .
	  $backend->name() . ":" . $backend->port();
	my($socket) = IO::Socket::INET->new(PeerAddr => $backend->name(),
					    PeerPort => $backend->port(),
					    Timeout => $backend->timeout(),
					    Proto => 'tcp',
					   );
	# check for successful connection
	if (defined($socket)) {
	    print "...success.\n";
	    $socket->close();
	    $nmta_backends_available++;
	}
	else {
	    print "...failure.\n";
	}
    }
    print "\n";

    if ( $nmta_backends_available ) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 5 - check POP Servers
#
###############################################################################
sub availability_5 {
    my($popz_greeters_available) = 0;

    print "\n########## ZKS POP3 Server\n\n";

    foreach my $greeter (@{$popz_servers}) {
 	print "Trying POP3 connection to " . 
	  $greeter->name() . ":" . $greeter->port() . "\n";
	$test_nym->popx_server( $greeter );
	$result = $popx_conn->connect($test_nym);	
	if ( !defined($result) ) {
	    print "POP Server Connection Failed\n";
	    last;
	}

	$banner = $popx_conn->banner();
	if ( !($banner =~ m/\<\d+\.\d+\@freedom\.net\>/) ) {
	    print "POP server " . $greeter->name() . " resolves to wrong DNS name\n";
	    last;
	}
	$popx_conn->quit();
	$popz_greeters_available++;
    }
    print("\n");

    if ( $popz_greeters_available ) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 6 - check RELAY QMQP Servers
#
###############################################################################
sub availability_6 {
    my($relay_greeters_available) = 0;

    print "\n########## SMTP/QMQP Relay Server\n\n";

    foreach my $greeter (@{$relay_servers}) {
	print "Trying connection to " . 
	  $greeter->name() . ":" . $greeter->port();
	my($socket) = IO::Socket::INET->new(PeerAddr => $greeter->name(),
					    PeerPort => $greeter->port(),
					    Timeout => $greeter->timeout(),
					    Proto => 'tcp',
					    );
	# check for successful connection
	if (defined($socket)) {
	    print "...success\n";
	    $socket->close();
	    $relay_greeters_available++;
	}
	else {
	    print "...failure\n";
	}
    }
    print("\n");

    if ( $relay_greeters_available == @{$relay_servers}) { return "ok"; }
    else { return "not ok"; }
}

###############################################################################
#
#    Test 7 - check NNTP Servers
#
###############################################################################
sub availability_7 {
    my($nntp_greeters_available) = 0;
    my($greeter);

    print "\n########## NNTP Servers\n\n";

    foreach $greeter (@{$nntp_servers}) {
	print "Trying NNTP connection to " .
	  $greeter->name() . ":" . $greeter->port() . "\n";
	$test_nym->nntp_server( $greeter );
	$result = $nntp_conn->connect($test_nym);	
	if ( !defined($result) ) {
	    print "NNTP Server Connection Failed\n";
	    last;
	}

	$nntp_conn->quit();
	$nntp_greeters_available++;
    }
    print("\n");

    if ( $nntp_greeters_available ) { return "ok"; }
    else { return "not ok"; }
}


###############################################################################
#
#    Test 8 - check for invalid active ports for the sever type
#
###############################################################################
sub availability_8 {
    my($test_succeeded) = 0;
    my($services) = [ $imep_servers, $nmta_servers, $popz_servers, $relay_servers ];

    print "\nServer Port Sniff\n\n";

    foreach my $curr_index ( 0 .. $#{$services} ) {
        my($curr_list) = $services->[$curr_index];

	foreach my $server_index ( 0 .. $#{$curr_list} ) {

	    # save the server name for this type list
	    my $server = $curr_list->[$server_index];
	    print "\nserver " . $server->name() . 
	      " has primary service " . $server->port() . "\n";

	    foreach my $srch_index ( 0 .. $#{$services} ) {

		# save the port number for the primary host for this type
		my $port = $services->[$srch_index][0]->port();

		# we are testing invalid services on the current server, so
		# ignore connection to correct service port for this server
		if ( $server->port() != $port ) {
		    # try to connect
		    print "\tconnection to [" . $port . "] should fail\n\t";
		    my($socket) = IO::Socket::INET->new(PeerAddr => $server->name(),
							PeerPort => $port,
							Timeout => $server->timeout(),
							Proto => 'tcp',
						       );
		    # check for successful connection
		    if (!defined($socket)) {
			$test_succeeded++;
		    }
		}
	    }
        }
    }
    print "\n>>> ";
    print "tested " . scalar(@$services) . " server types across " .
      $test_succeeded . " services ports\n\n";

    if ( $test_succeeded ) { return "ok"; }
    else { return "not ok"; }
}

#############################################################
# Execute each function in this TCO and return
# success/failure result.
#

print("\n");
ok( &availability_1, "ok");
ok( &availability_2, "ok");
ok( &availability_3, "ok");
ok( &availability_4, "ok");
ok( &availability_5, "ok");
ok( &availability_6, "ok");
ok( &availability_7, "ok");
ok( &availability_8, "ok");
print "\n";
