# $Id: blocklist.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

#
####################
#

use strict;
use Test;

use Mail_Test;
use Suite_Specific;
use Test::Error;

BEGIN { plan tests => 6 }



#                  #
# GLOBAL VARIABLES #
#                  #


#############################################
# Globally used directory names and filepaths
#############################################

my($SSHAGT) = "";  # set to either the string "agt" or the empty string depending on wether or not you need passwords for scp

my($MAILUSER) = 'root';
my($MAILHOME) = '/usr/freedom/qmail/alias';
my($BL_PROP_DIR) = $ENV{ZK_FREEDOM_ROOT}."/popmail/admin/tools";
my($REVOKE_LIST) = $ENV{ZK_FREEDOM_ROOT}."/popmail/test/system/suites/t10/revoke-list.txt";
my($NYM_TO_NET_BLOCK_LIST) = $ENV{ZK_FREEDOM_ROOT}."/popmail/test/system/suites/t10/nym-to-net-block-list.txt";
my($TMPF1) = $ENV{ZK_FREEDOM_ROOT}."/popmail/test/system/suites/t10/tmp-bl-1";
my($TMPF2) = $ENV{ZK_FREEDOM_ROOT}."/popmail/test/system/suites/t10/tmp-bl-2";
my($HOSTFILE) = undef;

#########################################
# Times to wait for messages to propagate
#########################################

my($nym2nym_propt) = 10;
my($net2nym_propt) = 60;
my($nym2net_propt) = 60;
my($config_propt)  = $nym2nym_propt * 2;


#######################
# Generic error message
#######################

my($error_msg) = "";


##############################
# Generic function result code
##############################

my($result) = undef;


#################################
# Create user factories and users
#################################

my($nym_factory) = new Factory::NymFactory();
my($usr_factory) = new Factory::UserFactory();
my($srv_factory) = new Factory::ServerFactory();

my($nym_a,$nym_b,$nym_revoked,$inet_t);
my($config_blocklist) = $nym_factory->create_config_blocklist_usr();
my($config_spam_control) = $nym_factory->create_config_spam_control_usr();


#################################################
# Create connection factory and empty connections
#################################################

my($conn_factory) = new Factory::ConnectionFactory();
my($smtp_conn) = $conn_factory->create_SMTP_connection();
my($popx_conn) = $conn_factory->create_POPX_connection();


############################################
# Create message factory and message objects
############################################

my($msg_factory) = new Factory::MessageFactory();
my($message) = $msg_factory->create_MAIL_message();



#             #
# SUBROUTINES #
#             #

#########################################################################
#
# genhostfile
#
#########################################################################

sub genhostfile {

  my($i) = 1;

  open( HOSTS , '>' . $HOSTFILE ) or
    die "couldn't open $HOSTFILE\n";
  print HOSTS "set BLOCKLIST_HOSTS [list ";

  foreach my $host (@_) {
    print HOSTS "[set HOSTS(".$i.",name) "."\"".$host->name()."\"] ";
    print HOSTS "[set HOSTS(".$i.",passwd) "."\"\"] ";
    print HOSTS "[set HOSTS(".$i.",destdir) "."\"".$MAILHOME."\"] ";
    $i++;
  }
  print HOSTS "]\n";
  close HOSTS;
}

sub sethostfile { $HOSTFILE = shift }

#########################################################################
#
# cleanup( BLOCK-LIST , BLOCK-HOSTS , NYM , CONFIG-USER )
#
# - cleanup routine, clears any generated blocklists
# - if NYM and CONFIG-USER are not specified then the BLOCK-LIST and
#   and BLOCK-HOSTS parameters will be used.  Otherwise these can
#   be passed as the empty string.
# - if NYM and CONFIG-USER are specified, this function either clears
#   the nym's personal blocklist, or disables its spam control (depending
#   on the value of CONFIG-USER (which should be one of the special
#   nyms generated for each of config-blocklist and config-spam-control
#   users).
# - if NYM and CONFIG-USER are _not_ specified, then this routine clears
#   the system-wide blocklists specified by BLOCK-LIST and propagates
#   them to BLOCK-HOSTS.
#
###########################################################################

sub cleanup {
  my($rlist) = shift;
  my($rhost) = shift;
  my($nym) = shift || undef;
  my($config) = shift || undef;
  my($msgstr) = undef;

  if ( !defined($nym) || !defined($config) ) {

    # We're cleaning up mailsystem wide blocklists
    modify_block_list( '>' , $rlist , $rhost , "" , "" );
    
  } else {

    # We're cleaning up a nym's blocklist
    if ( $config->name() eq "config-blocklist" ) {
      my $msgtxtfile = 'blocklist.txt';
      open( BLOCK , '>' . $msgtxtfile ) or
        die "couldn't open cleanup blocklist file for writing!\n";
      print BLOCK "\n";
      close BLOCK;
      $msgstr = `cat $msgtxtfile | $ENV{ZK_EXEC_DIR}/list2hashlist`;
      system "rm $msgtxtfile";
    } elsif ( $config->name() eq "config-spam-control" ) {
      $msgstr = "No\n";
    }
    send_message( $nym , $config , [ $msgstr ] );
    sleep($config_propt);
  }

}



###########################################################################
#
#  empty_POP_box( USER )
#
#  - deletes all mesages in the given USER's POP box
#  - USER must be of type User::Base.
#
###########################################################################

sub empty_POP_box {
  my($user) = shift;

  print "\n\n===> EMPTY POP BOX : ". $user->name() . " ***********\n\n";

  # Login and create POP box if non-existent;
  $result = $popx_conn->connect( $user );
  if ( !defined($result) ) {
    $error_msg = "Could not open connection for " . $user->name() .
      "\'s POP box on " . $user->popx_server()->name() . ".";
    return Test::Error::error($error_msg);
  }
  $popx_conn->empty_box();
  $popx_conn->quit();
  return "ok";
}



###########################################################################
#
#  send_message( SENDER , RECIPIENT , [MESSAGE BODY] )
#
#  - Sends a mail message from SENDER to RECIPIENT with the given body
#  - SENDER and RECIPIENT are of type User::Base.
#  - [MESSAGE BODY] is a list of strings
#
###########################################################################

sub send_message {
  my($s_user) = shift;
  my($r_user) = shift;
  my($msgtext) = shift;

  print "\n\n===> SEND MESSAGE : [ ". $s_user->name() . " ] -> [ " . $r_user->name . " ]\n\n";

  # clear the message
  $message->empty();

  # create message enveope
  $message->header()->from($s_user);
  $message->header()->to($r_user);

  # add some message text and freedomize it
  $message->content()->add_part( $message->content()->new_part($msgtext) );

  # Send the message to the recipient

  # open SMTP connection
  $result = $smtp_conn->connect( $s_user );
  if ( !defined($result) ) {
    $error_msg = "Could not open connection to " . $s_user->smtp_server()->name() . ".";
    return Test::Error::error($error_msg);
  }
  
  # Sent the test mail
  $result = $smtp_conn->send_msg( $message );
  if ( !defined($result) ) {
    $error_msg = "Could not send message through " . $s_user->smtp_server()->name() . ".";
    return Test::Error::error($error_msg);
  }
  
  # close SMTP connection
  $result = $smtp_conn->quit();
  if ( !defined($result) ) {
    $error_msg = "Could not close connection to " . $s_user->smtp_server()->name() . ".";
    return Test::Error::error($error_msg);
  }
  
  # open SMTP connection
  $result = $smtp_conn->connect( $s_user );
  if ( !defined($result) ) {
    $error_msg = "Could not open connection to " . $s_user->smtp_server()->name() . ".";
    return Test::Error::error($error_msg);
  }
  
  return "ok";
}


###########################################################################
#
#  check_mail( USER )
#
#  - Returns the number of messages in USER's POP box
#  - USER is of type User::Base
#
###########################################################################

sub check_mail {
  my($user) = shift;
  my($timeout) = shift || 30;
  my($retries) = shift || 2;
  my($num_msg) = undef;

  print "\n\n===> CHECK MAIL : ". $user->name() . " ***********\n\n";

  $popx_conn->connect( $user );
  
# Polling version of mailchecking

  my $message_list = $popx_conn->poll_msgs( $timeout , $retries );
  if ( !defined($message_list) ) {
    $num_msg = 0;
  } else {
    $num_msg = scalar( @{$message_list} );
  }

# Non-polling version of mailchecking
#
#  my($message_list) = $popx_conn->get_msgs();
#  $popx_conn->quit();
#  if ( !defined($message_list) ) {
#    $error_msg = "Could not retrieve ". $user->name() . "\'s mail on " .
#      $user->popx_server()->name() . ".";
#    return Test::Error::error($error_msg);
#  }
#  $num_msg = scalar( @{$message_list} );

  print "\n\n===> ". $user->name() . " : $num_msg MESSAGE(S) ***********\n\n";
  return ( $num_msg );
}


###########################################################################
#
#  modify_block_list( MODE , LISTFILE , HOSTFILE , SENDER , RECEIVER )
#
#  - modify the blocklist specified by LISTFILE and propagate it to the
#    hosts specified in HOSTFILE.
#  - MODE is one of '>' or '>>' depending on whether or not the list should
#    be generated anew or appended to respectively.
#  - LISTFILE is a textfile name that will be created if it doesn't exist
#  - HOSTIFLE is a file of the type specified by the prop-block-list
#    script.
#  - SENDER and RECIPIENT are strings that should specify the email address,
#    domain name, or nym name (for revocation) of the blocked party.  These
#    values  will either replace the old blocklist or be appended to it
#    depending on the value of MODE.  Either of them can be specified as the
#    empty string.
#
###########################################################################

sub modify_block_list {
  my($mode)       = shift;
  my($list_txt)   = shift;
  my($list_hosts) = shift;
  my($sender)     = shift;
  my($receiver)   = shift;

  if (($mode ne '>') && ($mode ne '>>')) {
    return Test::Error::error("Incorrect MODE specified to modify_block_list");
  }

  open( REVOKE , $mode . $list_txt ) or
    return Test::Error::error("Could not open revoke list");
  if ($sender ne "")   { print REVOKE $sender; }
  if ($receiver ne "") { print REVOKE ",",$receiver; }
  print REVOKE "\n";
  close REVOKE;

  # create blocklist creation/propagation command line
  my($bl_prop_cmd) = "expect " . $BL_PROP_DIR . "/prop-block-list";
  $bl_prop_cmd .= " " . $list_txt . " " . $list_hosts . " " . $MAILUSER . " " . $SSHAGT;

  my($rc) = 0xffff & system $bl_prop_cmd;
  if ($rc != 0) {
    return Test::Error::error("Couldn't propagate cdb file");
  }

  return "ok";
}



################################################################################
#
# The following are just tests I added to test that nym-to-nym, nym-to-inet,
# and inet-to-nym mail tests work without blocking.
#
################################################################################

sub control_1 {
  my($rc) = undef;

  my($nym_x) = $nym_factory->create_random_valid_nym();
  my($nym_y) = $nym_factory->create_random_valid_nym();

  my($num) = undef;

  # Nym to Nym
  $rc = empty_POP_box( $nym_y ); if ($rc ne "ok") { return $rc; }
  $num = check_mail( $nym_y , 10 );
  $rc = send_message( $nym_x , $nym_y , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") { return $rc; }
  $num = check_mail( $nym_y , $nym2nym_propt );
  if ($num != 1) {
    return Test::Error::error("No mail received for nym " . $nym_y->name() );
  }
  return "ok";
}

sub control_2 {
  my($rc) = undef;

  my($nym_x) = $nym_factory->create_random_valid_nym();
  my($inet_z) = $usr_factory->create_random_valid_netuser();

  my($num) = undef;

  # Nym to I-Net
  $rc = empty_POP_box( $inet_z ); if ($rc ne "ok") { return $rc; }
  $num = check_mail( $inet_z , 10 );
  $rc = send_message( $nym_x , $inet_z , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") { return $rc; }
  $num = check_mail( $inet_z , $nym2net_propt );
  if ($num != 1) {
    return Test::Error::error("No mail received for inet " . $inet_z->name() );
  }
  return "ok";
}

sub control_3 {
  my($rc) = undef;

  my($nym_y) = $nym_factory->create_random_valid_nym();
  my($inet_z) = $usr_factory->create_random_valid_netuser();

  my($num) = undef;

  # I-Net to Nym
  $rc = empty_POP_box( $nym_y ); if ($rc ne "ok") { return $rc; }
  $num = check_mail( $nym_y , 10 );
  $rc = send_message( $inet_z , $nym_y , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") { return $rc; }
  $num = check_mail( $nym_y , $net2nym_propt );
  if ($num != 1) {
    return Test::Error::error("No mail received for nym " . $nym_y->name() );
  }
  return "ok";
}

##############################################################################
#
# Block Lists - Test 1 - Revoke a NYM's access to mail system
#
##############################################################################

sub blocklist_1 {
  my($rc) = undef;

  print "\n############ NYM REVOCATION\n\n";

  $nym_revoked = $nym_factory->create_random_valid_nym();
  my($nym_arbitrary) = $nym_factory->create_random_valid_nym();

  # add the nym to the revoke list and propagate to NMTA's
  sethostfile( $TMPF1 );
  my $nmtas = $srv_factory->create_all_NMTA_servers();
  genhostfile( @{$nmtas} );
  $rc = modify_block_list( '>' , $REVOKE_LIST , $TMPF1 ,
                           $nym_revoked->name() , "" );
  if ($rc ne "ok") { return $rc; }

  # add the nym to the revoke list and propagate to POPZs
  sethostfile( $TMPF2 );
  my $popzs = $srv_factory->create_all_POPZ_servers();
  genhostfile( @{$popzs} );
  $rc = modify_block_list( '>' , $REVOKE_LIST , $TMPF2 ,
                           $nym_revoked->name() , "" );
  if ($rc ne "ok") { return $rc; }
   
  # empty the nym's POP box
  $rc = empty_POP_box( $nym_arbitrary );
  if ($rc ne "ok") {
    cleanup($REVOKE_LIST,$TMPF1);
    cleanup($REVOKE_LIST,$TMPF2);
    return $rc;
  }


  # attempt to send a message using the revoked nym
  $rc = send_message( $nym_revoked , $nym_arbitrary , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") {
    cleanup($REVOKE_LIST,$TMPF1);
    cleanup($REVOKE_LIST,$TMPF2);
    return $rc;
  }


  # make sure the receiving nym didn't receive anything
  my $num = check_mail( $nym_arbitrary , $nym2nym_propt );
  if ($num != 0) {
    $rc = Test::Error::error("Revoked nym managed to send mail : ["
                             . $nym_revoked->name() . "]->[" . $nym_arbitrary->name() );
    cleanup($REVOKE_LIST,$TMPF1);
    cleanup($REVOKE_LIST,$TMPF2);
    return $rc;
  }


  # generate an empty revocation list and propagate it
  # $rc = modify_block_list( '>' , $REVOKE_LIST , $REVOKE_HOST , "" , "" );
  # if ($rc ne "ok") { return $rc; }


  cleanup($REVOKE_LIST,$TMPF1);
  cleanup($REVOKE_LIST,$TMPF2);
  return "ok"
}



##############################################################################
#
# Block Lists - Test 2 - Block delivery from nym a to nym b
#
##############################################################################

sub blocklist_2 {
  my($rc) = undef;

  print "\n############ BLOCK NYM-A TO NYM-B\n\n";

  $nym_a = $nym_factory->create_random_valid_nym();
  $nym_b = $nym_factory->create_random_valid_nym();

  # GENERATE THE MESSAGE BODY OF THE BLOCKLIST REQUEST ($num_block addresses)

  # create a textfile version of the blocklist
  my $blocklist_txt = 'blocklist.txt';
  open( BLOCK , '>' . $blocklist_txt ) or
    die "Couldn't open blocklist.txt for writing!";
  print BLOCK $nym_a->email() . "\n";
  close BLOCK;

  # hash the textfile into a variable and remove the file
  print "\nBLOCKLIST TEXT\n\n".`cat $blocklist_txt`;
  my $hashed_blocklist = `cat $blocklist_txt | $ENV{ZK_EXEC_DIR}/list2hashlist`;
  print "\nHASHED LIST\n\n".$hashed_blocklist;
  system "rm $blocklist_txt";


  # send the blocklist request message (sleep a little longer than usual)
  $rc = send_message( $nym_b , $config_blocklist , [ $hashed_blocklist ] );
  if ($rc ne "ok") { return $rc; }
  sleep($config_propt);


  # clear nym_b's POP boxe
  $rc = empty_POP_box( $nym_b );
  if ($rc ne "ok") {
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # attempt to send a message from nym_a to nym_b
  $rc = send_message( $nym_a , $nym_b , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") {
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # check that nym_b didn't get the delivery
  my $num = check_mail( $nym_b , $nym2nym_propt );
    if ($num != 0) {
    $rc = Test::Error::error("NYM-B received forbidden message : "
                             . $nym_b->name() );
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # empty the nym's block list by sending an empty request
  # $rc = send_message( $nym_b , $config_blocklist , [ "\n" ] );
  # if ($rc ne "ok") { return $rc; }
  # sleep($config_propt);

  cleanup("","",$nym_b,$config_blocklist);
  return "ok";
}



##############################################################################
#
# Block Lists - Test 3 - Block delivery from internet t to nym b
#
##############################################################################

sub blocklist_3 {
  my($rc) = undef;

  print "\n############ BLOCK INTERNET-T TO NYM-B\n\n";

  $nym_b  = $nym_factory->create_random_valid_nym();
  $inet_t = $usr_factory->create_random_valid_netuser();


  # send the block request and wait until it's completed
  $rc = send_message( $nym_b , $config_blocklist , [ $inet_t->email() . "\n" ] );
  if ($rc ne "ok") { return $rc; }
  sleep($config_propt);


  # clear nym_b's POP boxes
  $rc = empty_POP_box( $nym_b );
  if ($rc ne "ok") {
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # attempt to send a message from inet_t to nym_b
  $rc = send_message( $inet_t , $nym_b , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") {
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # check that nym_b didn't get the delivery
  my $num = check_mail( $nym_b , $net2nym_propt );
  if ($num != 0) {
    $rc = Test::Error::error("NYM-B received forbidden message : "
                             . $nym_b->name() );
    cleanup("","",$nym_b,$config_blocklist);
    return $rc;
  }


  # empty the nym's block list by sending an empty request
  # $rc = send_message( $nym_b , $config_blocklist , [ "\n" ] );
  # if ($rc ne "ok") { return $rc; }
  # sleep ($config_propt);


  cleanup("","",$nym_b,$config_blocklist);
  return $rc;
}



##############################################################################
#
# Block Lists - Test 4 - Block delivery from nym a to internet t
#
##############################################################################

sub blocklist_4 {
  my($rc) = undef;

  print "\n############ BLOCK NYM-A TO INTERNET-T\n\n";

  $nym_a = $nym_factory->create_random_valid_nym();
  $inet_t = $usr_factory->create_random_valid_netuser();

  # add the nym2net pairing to the blocklist and propagate to NMTAB's
  sethostfile( $TMPF1 );
  my $nmtabs = $srv_factory->create_all_NMTA_B_servers();
  genhostfile( @{$nmtabs} );
  $rc = modify_block_list(  '>' , $NYM_TO_NET_BLOCK_LIST , $TMPF1 ,
                            $nym_a->email() ,  $inet_t->email() );
  if ($rc ne "ok") { return $rc; }


  # empty the net-user's POP box
  $rc = empty_POP_box( $inet_t );
  if ($rc ne "ok") {
    cleanup( $NYM_TO_NET_BLOCK_LIST , $TMPF1 );
    return $rc;
  }

  
  # attempt to send a message from nym_a to inet_t
  $rc = send_message( $nym_a , $inet_t , [ "Bubkus message text\n" ] );
  if ($rc ne "ok") {
    cleanup( $NYM_TO_NET_BLOCK_LIST , $TMPF1 );
    return $rc;
  }

  
  # check the net-user's mailbox to verify failure
  my $num = check_mail( $inet_t , $nym2net_propt );
  if ($num != 0) {
    $rc = Test::Error::error("INTERNET-T's mailbox contains forbidden message " . $inet_t->name() );
    cleanup( $NYM_TO_NET_BLOCK_LIST , $TMPF1 );
    return $rc;
  }

  
  # generate an empty blocklist and propagate it
  # $rc = modify_block_list( '>' , $NYM_TO_NET_BLOCK_LIST , $NYM_TO_NET_BLOCK_HOST , "" , "" );
  # if ($rc ne "ok") { return $rc; }

  
  cleanup( $NYM_TO_NET_BLOCK_LIST , $TMPF1 );
  return $rc;
}



##############################################################################
#
# Block Lists - Test 5 - Deliver messages to nym with multiple blocked nyms
#
##############################################################################

sub blocklist_5 {
  my($rc) = undef;
  my($x,$y) = (1,1);

  print "\n############ NYM TO NYM WITH MULTIPLE BLOCKED NYMS\n";

  $nym_a  = $nym_factory->create_random_valid_nym();
  $nym_b  = $nym_factory->create_random_valid_nym();

  foreach my $num_block ( $y , $y * 1000 , $y * 10000 ) {

    # GENERATE THE MESSAGE BODY OF THE BLOCKLIST REQUEST ($num_block addresses)

    # create a textfile version of the blocklist
    my $blocklist_txt = 'blocklist.txt';
    open( BLOCK , '>' . $blocklist_txt ) or
      die "Couldn't open blocklist.txt for writing!";
    for (my $i = 0; $i < $num_block; $i++) {
      print BLOCK "blocked_nym_$i\@tsktsk.net\n";
    }
    close BLOCK;

    # hash the textfile into a variable and remove the file
    my $hashed_blocklist = `cat $blocklist_txt | $ENV{ZK_EXEC_DIR}/list2hashlist`;
    system "rm $blocklist_txt";

    # send the blocklist request message (sleep a little longer than usual)
    $rc = send_message( $nym_b , $config_blocklist , [ $hashed_blocklist ] );
    if ($rc ne "ok") { return $rc; }
    sleep($config_propt);


    # empty the receiving nym's POP box
    $rc = empty_POP_box( $nym_b );
    if ($rc ne "ok") {
      cleanup("","",$nym_b,$config_blocklist);
      return $rc;
    }

    
    # send x mesages to the nym
    for (my $i = 0; $i < $x; $i++) {
      $rc = send_message( $nym_a , $nym_b , [ "crap crap crap crap" , "\n" ] );
      if ($rc ne "ok") {
        cleanup("","",$nym_b,$config_blocklist);
        return $rc;
      }
    }


    # check that the emails were received
    my $num = check_mail( $nym_b , $nym2nym_propt * $x );
    if ($num != $x) {
      $rc = Test::Error::error( "Incorrect number of messages in " . $nym_b->name() .
                                 "\'s mailbox : " . $num . " found instead of " . $x );
      cleanup("","",$nym_b,$config_blocklist);
      return $rc;
    }
  
  }

  cleanup("","",$nym_b,$config_blocklist);
  return "ok";
}



##############################################################################
#
# Spam control - Test 1 - send mail from [internet] spammer to nym
#
##############################################################################

sub spam_control_1 {
  my($rc) = undef;

  print "\n############# SPAMMER TO NYM\n";

  $nym_a = $nym_factory->create_random_valid_nym();

  # enable the nym's spam control
  $rc = send_message( $nym_a , $config_spam_control , [ "Yes" , "\n" ] );
  if ($rc ne "ok") { return $rc; }
  sleep($config_propt);

  print "\n( DON'T KNOW WHO SPAMMERS ARE YET )\n";


  # disable the nym's spam control
  # $rc = send_message( $nym_a , $config_spam_control , [ "No" , "\n" ] );
  # if ($rc ne "ok") { return $rc; }
  # sleep($config_propt);

  cleanup("","",$nym_a,$config_spam_control);
  return "ok";
}

##############################################################################
#
# Spam control - Test 2 - send mail from [internet] non-spammer to nym.
#
##############################################################################

sub spam_control_2 {
  my($rc) = undef;

  print "\n############# NON-SPAMMER TO NYM\n";

  $nym_a = $nym_factory->create_random_valid_nym();

  $rc = send_message( $nym_a , $config_spam_control , [ "Yes" , "\n" ] );
  if ($rc ne "ok") { return $rc; }
  sleep($config_propt);

  print "\n NOTE - WE CAN'T ADD TO THE SPAMMER LISTS\n";

  # disable the nym's spam control
  # $rc = send_message( $nym_a , $config_spam_control , [ "No" , "\n" ] );
  # if ($rc ne "ok") { return $rc; }
  # sleep($config_propt);

  cleanup("","",$nym_a,$config_spam_control);
  return "ok";
}

#############################################################
# Execute each function in this TCO and return
# success/failure result.
#

print "\n";

# control tests
print "\n\n[I] ##++++++++++++++ CONTROL TEST - CHECKING END TO END FUNCTIONALITY ++++++++++++##\n\n";
# ok (&control_1, "ok");
# ok (&control_2, "ok");
# ok (&control_3, "ok");

# blocklist tests
print "\n\n[II] ##+++++++++++++++++++++++++++ BLOCKLIST TEST PLAN +++++++++++++++++++++++++++##\n\n";
ok (&blocklist_1, "ok");    # NYM REVOCATION
ok (&blocklist_2, "ok");    # NYM TO NYM BLOCK
ok (&blocklist_3, "ok");    # INET TO NYM BLOCK
ok (&blocklist_4, "ok");    # NYM TO INET BLOCK
ok (&blocklist_5, "ok");    # NYM TO NYM MULTI-BLOCK

# spam-control tests
print "\n\n[II] ##+++++++++++++++++++++++++ SPAM-CONTROL TEST PLAN ++++++++++++++++++++++++++##\n\n";
ok (&spam_control_1, "ok"); # SPAMMER TO NYM
ok (&spam_control_2, "ok"); # NON-SPAMMER TO NYM

print "\n";
