# $Id: bounce.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

#
####################
#

use strict;
use Test;

#use lib('../../lib/perl');
#use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

use Test::Case;


BEGIN { plan tests => 4 }

#
####################
#

# create user factories and empty users
my($nym_factory) = new Factory::NymFactory();
my($usr_factory) = new Factory::UserFactory();
my($s_user, $r_user_1, $r_user_2, $r_user_3, $r_user_4);

# create connection factory
my($conn_factory) = new Factory::ConnectionFactory();

# create the connections necessary for this test
my($connections) = {};
$connections->{SMTP} = $conn_factory->create_SMTP_connection();
$connections->{POPX} = $conn_factory->create_POPX_connection();

#
####################
#


###############################################################################
#
# Test 1 - NYM to invalid NYM
#
###############################################################################

sub bounce_1 {

    print "\n########## NYM TO INVALID NYM\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $nym_factory->create_random_bogus_nym();

    # create the list of senders & recipients
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $s_users );
}

###############################################################################
#
# Test 2 - NYM to invalid Internet recipient
#
###############################################################################

sub bounce_2 {

    print "\n########## NYM TO INVALID INTERNET\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $usr_factory->create_random_bogus_netuser();

    # create the list of senders & recipients
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $s_users );
}

###############################################################################
#
# Test 3 - Internet user to invalid nym
#
###############################################################################

sub bounce_3 {

    print "\n########## INTERNET to INVALID NYM\n\n";

    # test specific parameters
    $s_user = $usr_factory->create_random_valid_netuser();
    $r_user_1 = $nym_factory->create_random_bogus_nym();

    # create the list of senders & recipients
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $s_users );
}

###############################################################################
#
# Test 4 - NYM to multiple valid and invalid nyms
#
###############################################################################

sub bounce_4 {

    print "\n########## NYM TO MULTIPLE VALID & INVALID NYMS\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $nym_factory->create_random_valid_nym();
    $r_user_2 = $nym_factory->create_random_valid_nym();
    $r_user_3 = $nym_factory->create_random_bogus_nym();
    $r_user_4 = $nym_factory->create_random_bogus_nym();

    # create the list of senders & recipients
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user_1, $r_user_2, $r_user_3, $r_user_4 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $s_users );
}

#
####################
#

print "\n";
ok(&bounce_1, 'ok');
ok(&bounce_2, 'ok');
ok(&bounce_3, 'ok');
ok(&bounce_4, 'ok');
print "\n";
