# $Id: end2end.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

#
################
#

use strict;
use Test;

#use lib('../../lib/perl');
#use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

use Test::Case;

BEGIN { plan tests => 6 }


#
###############
#

# create user factories and empty users
my($nym_factory) = new Factory::NymFactory();
my($usr_factory) = new Factory::UserFactory();
my($s_user, $n_user, $r_user_1, $r_user_2, $r_user_3, $r_user_4);

# create connection factory
my($conn_factory) = new Factory::ConnectionFactory();

# create the connections necesary for this test set
my($connections) = {};
$connections->{SMTP} = $conn_factory->create_SMTP_connection();
$connections->{POPX} = $conn_factory->create_POPX_connection();

#
################
#



#########################################################
# Test #1:  nym to nym test. do not verify message 
#           contents.
#

sub end2end_1 {

    print "\n########## NYM TO NYM\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $nym_factory->create_random_valid_nym();

    # create the list of recipients
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

###########################################################
# Test #2:  nym to single Internet user test. do not verify 
#           message contents.
#

sub end2end_2 {

    print "\n########## NYM TO SINGLE INTERNET\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $usr_factory->create_random_valid_netuser();

    # create the list of recipients
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

############################################################
# Test #3:  single Internet user to nym test. do not verify
#           message contents.
#

sub end2end_3 {

    print "\n########## INTERNET TO NYM\n\n";

    # test specific parameters
    $s_user = $usr_factory->create_random_valid_netuser();
    $r_user_1 = $nym_factory->create_random_valid_nym();

    # create the list of recipients
    my($r_users) = [ $r_user_1 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

############################################################
# Test #5:  nym to multiple nym recipients test. do not 
#           verify message contents.
#

sub end2end_4 {

    print "\n########## NYM TO MULTIPLE NYM\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $nym_factory->create_random_valid_nym();
    $r_user_2 = $nym_factory->create_random_valid_nym();
    $r_user_3 = $nym_factory->create_random_valid_nym();
    $r_user_4 = $nym_factory->create_random_valid_nym();

    # create the list of recipients
    my($r_users) = [ $r_user_1, $r_user_2, $r_user_3, $r_user_4 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

############################################################
# Test #6:  nym to multiple nym and multiple Internet 
#           recipients test. do not verify message contents.
#

sub end2end_5 {

    print "\n########## NYM TO MULTIPLE NYM & INTERNET\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user_1 = $nym_factory->create_random_valid_nym();
    $r_user_2 = $nym_factory->create_random_valid_nym();
    $r_user_3 = $usr_factory->create_random_valid_netuser();
    $r_user_4 = $usr_factory->create_random_valid_netuser();

    # create the list of recipients
    my($r_users) = [ $r_user_1, $r_user_2, $r_user_3, $r_user_4 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

############################################################
# Test #7:  internet to multiple nym and multiple Internet
#           recipients. do not verify message content.
#

sub end2end_6 {

    print "\n########## INTERNET TO MULTIPLE NYM & INTERNET\n\n";

    # test specific parameters
    $s_user = $usr_factory->create_random_valid_netuser();
    $r_user_1 = $nym_factory->create_random_valid_nym();
    $r_user_2 = $nym_factory->create_random_valid_nym();
    $r_user_3 = $usr_factory->create_random_valid_netuser();
    $r_user_3 = $usr_factory->create_random_valid_netuser();

    # create the list of recipients
    my($r_users) = [ $r_user_1, $r_user_2, $r_user_3, $r_user_4 ];

    # execute the test case
    return Test::Case::Run_TCO_Single_Mail_Message( $connections,
						     $s_user, $r_users, $r_users );
}

#############################################################
# Execute each function in this TCO and return
# success/failure result.
#

print "\n";

# single user to user tests
print "\n\n##++++++++++ SINGLE SENDER/RECIPIENT TESTS ++++++++++##\n\n";
ok (&end2end_1, "ok");
ok (&end2end_2, "ok");
ok (&end2end_3, "ok");

# single to multiple user tests
print "\n\n##++++++++++ MULTIPLE SENDER/RECIPIENT TESTS ++++++++++##\n\n";
ok (&end2end_4, "ok");
ok (&end2end_5, "ok");
ok (&end2end_6, "ok");

print "\n";
