# $Id: login.t,v 1.2 2000/10/04 12:57:34 marcp Exp $ 

# Login: 

# confirms that the IMEP, NMTA, and POP services are available
# and that we can log into each, using both valid and invalid identities.

use strict;
use Test;

use lib('../../lib/perl');
use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

my($test_count) = 0;

# Create factories.

my($server_factory) = new Factory::ServerFactory();
my($connection_factory) = new Factory::ConnectionFactory();
my($nym_factory) = new Factory::NymFactory();

# Create servers, and increment the test count.

my($nmta_servers) = $server_factory->create_all_NMTA_servers();
$test_count += (@{$nmta_servers} * 2); # once with a valid nym, once with a bogus nym.

my($popz_servers) = $server_factory->create_all_POPZ_servers();
$test_count += (@{$popz_servers} * 2); # once with a valid nym, once with a bogus nym.

my($imep_servers) = $server_factory->create_all_IMEP_servers();
$test_count += @{$imep_servers};

# Create nyms

my($valid_nym) = $nym_factory->create_random_valid_nym();
my($bogus_nym) = $nym_factory->create_random_bogus_nym();

# Create the domain

print "\n";
plan tests => $test_count;

# Run the tests.
{

    print "\n########## NMTA LOGIN\n\n";
    foreach my $nmta (@{$nmta_servers}) {
	print "#### Valid NMTA\n";
	my($smtp_valid) = $connection_factory->create_SMTP_connection($valid_nym, $nmta);
	ok( defined($smtp_valid->connect()) ); print "\n";
	
	print "#### Invalid NMTA\n";
	my($smtp_invalid) = $connection_factory->create_SMTP_connection($bogus_nym, $nmta);
	ok( !defined($smtp_invalid->connect()) ); print "\n";
    }
}

{
    print "\n########## ZKS POP3 LOGIN\n\n";
    foreach my $popz (@{$popz_servers}) {
	print "#### Valid POP Server\n";
	my($popz_valid) = $connection_factory->create_POPX_connection($valid_nym, $popz);
	ok( defined($popz_valid->connect()) ); print "\n";
	
	print "#### Invalid POP Server\n";
	my($popz_invalid) = $connection_factory->create_POPX_connection($bogus_nym, $popz);
	ok( !defined($popz_invalid->connect()) ); print "\n";
    }
}

{
    print "\n########## IMEP LOGIN\n\n";
    foreach my $imep (@{$imep_servers}) {
	print "#### Valid IMEP Server\n";
	my($relay) = $connection_factory->create_SMTP_connection($valid_nym, $imep);
	ok( defined($relay->connect()) ); print "\n";
    }
}
