# $Id: mqueue.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

#
####################
#

use strict;
use Test;

#use lib('../../lib/perl');
#use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

use Test::Case;

BEGIN { plan tests => 3 }

#
####################
#

# test case result code
my($result);

# create user factories and empty users
my($nym_factory) = new Factory::NymFactory();
my($usr_factory) = new Factory::UserFactory();
my($s_user, $r_user);

# create connection factory and empty connections
my($conn_factory) = new Factory::ConnectionFactory();
my($connections) = {};
$connections->{POPX} = $conn_factory->create_POPX_connection();
$connections->{SMTP} = $conn_factory->create_SMTP_connection();

#
####################
#


###############################################################################
#
# Test 1 - NYM to NYM; multiple messages, empty mailbox
#
###############################################################################

sub mqueue_1 {

    my($limit);

    print "\n########## NYM TO NYM - MULTIPLE MESSAGES, EMPTY MAILBOX\n\n";

    # create test users
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user = $nym_factory->create_random_valid_nym();

    # setup test user lists
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user ];

    # set the number of messages to send
    $limit = $r_user->mail_limit() - int(rand($r_user->mail_limit() - 2));
    $limit = $limit < 0 ? 0 : $limit;

    # execute the test case
    return Test::Case::Run_TCO_Multiple_Mail_Message( $connections,
						      $s_user, $r_users, $r_users,
						      $limit,
						    );
}

###############################################################################
#
# Test 2 - NYM to NYM; multiple messages, non-empty mailbox
#
###############################################################################

sub mqueue_2 {

    my($limit);

    print "\n########## NYM TO NYM  - MULTIPLE MESSAGES, NON-EMPTY MAILBOX\n\n";

    # create test users
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user = $nym_factory->create_random_valid_nym();

    # setup test user lists
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user ];

    # set the number of messages to send
    $limit = $r_user->mail_limit() - int(rand($r_user->mail_limit() - 2));
    $limit = $limit < 0 ? 0 : $limit;

    # seed the mailbox with messages
    $result =  Test::Case::Run_TCO_Multiple_Mail_Message( $connections,
							  $s_user, $r_users, undef,
							  $limit,
							);
    if ( defined($result) ) {
	return $result;
    }

    # grab a new message count for the 'real' test
    $limit = $r_user->mail_limit() - int(rand($r_user->mail_limit() - 2));
    $limit = $limit < 0 ? 0 : $limit;
	
    # send the real batch of messages
    $result =  Test::Case::Run_TCO_Multiple_Mail_Message( $connections,
							  $s_user, $r_users, $r_users,
							  $limit,
							);
    if ( defined($result) ) {
	return $result;
    }

    return "ok";
}

###############################################################################
#
# Test 3 - NYM to NYM; multiple messages, full run
#
###############################################################################

sub mqueue_3 {

    my($limit);

    print "\n########## NYM TO NYM - MULTIPLE MESSAGES [MUA SIMULATOR]\n\n";

    # create test users
    $s_user = $nym_factory->create_random_valid_nym();
    $r_user = $nym_factory->create_random_valid_nym();

    # setup test user lists
    my($s_users) = [ $s_user ];
    my($r_users) = [ $r_user ];

    # set the number of messages to send
    $limit = $r_user->mail_limit() - int(rand($r_user->mail_limit() - 2));
    $limit = $limit < 0 ? 0 : $limit;

    # seed the mail box and check messages
    $result =  Test::Case::Run_TCO_Multiple_Mail_Message( $connections,
							  $s_user, $r_users, $r_users,
							  $limit,
							);
    if ( defined($result) ) {
	return $result;
    }

    # grab a new message count for the 'real' test
    $limit = $r_user->mail_limit() - int(rand($r_user->mail_limit() - 2));
    $limit = $limit < 0 ? 0 : $limit;
	
    # send the real batch of messages
    $result =  Test::Case::Run_TCO_Multiple_Mail_Message( $connections,
							  $s_user, $r_users, $r_users,
							  $limit,
							);
    if ( defined($result) ) {
	return $result;
    }

    return "ok";
}

#
####################
#

print "\n";
ok(&mqueue_1, 'ok');
ok(&mqueue_2, 'ok');
ok(&mqueue_3, 'ok');
print "\n";


