# $Id: netnews.t,v 1.2 2000/10/04 12:57:34 marcp Exp $

#
################
#

use strict;
use Test;

#use lib('../../lib/perl');
#use lib('../suites/t10');

use Mail_Test;
use Suite_Specific;

use Test::Case;

BEGIN { plan tests => 4 }


#
###############
#

# generic error message
my($error_msg) = "";

# generic function result code
my($result) = undef;

# create user factories and empty users
my($nym_factory) = new Factory::NymFactory();
my($usr_factory) = new Factory::UserFactory();
my($s_user, $n_user, $r_user_1, $r_user_2, $r_user_3, $r_user_4);

# create connection factory and empty connections
my($conn_factory) = new Factory::ConnectionFactory();
my($connections) = {};
$connections->{SMTP} = $conn_factory->create_SMTP_connection();
$connections->{NNTP} = $conn_factory->create_NNTP_connection();

#
################
#

############################################################
# Test 1:  nym to newsgroup posting test. do not verify
#          message contents.
#

sub netnews_1 {

    print "\n########## NYM TO SINGLE NEWSGROUP\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $n_user = $nym_factory->create_random_valid_nym();

    # create te newsgroup list
    my($newsgroups) = [ "alt.test" ];

    # execute the test case
    return Test::Case::Run_TCO_Single_News_Message( $connections,
						    $s_user, $n_user,
						    $newsgroups );
}

############################################################
# Test 2:  nym to multiple newsgroups. do not verify
#          message contents.
#

sub netnews_2 {

    print "\n########## NYM TO MULTIPLE NEWSGROUPS\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $n_user = $usr_factory->create_random_valid_netuser();

    # create a list of newsgroups
    my($newsgroups) = [ "alt.test",
			"alt.test.a",
			"alt.test.binaries",
			"alt.test.bogus",
			"alt.test.cjc",
			"alt.test.cmsg",
			"alt.test.d",
			"alt.test.ds",
			"alt.test.fishing",
			"alt.test.group",
			"alt.test.test",
			"alt.test.test2",
			"alt.test.test3",
			"alt.test.z",
		      ];

    # execute the test case
    return Test::Case::Run_TCO_Single_News_Message( $connections,
						    $s_user, $n_user,
						    $newsgroups );
}

###############################################################################
#
# Test 3 - NYM to NEWSGROUP exceeding crosspost limit
#
###############################################################################

sub netnews_3 {

    print "\n########## NYM TO NEWSGROUP (EXCEEEDING CROSSPOST LIMIT)\n\n";

    $s_user = $nym_factory->create_random_valid_nym();
    $n_user = $nym_factory->create_random_valid_nym();

    # create a list of newsgroups
    my($newsgroups) = [];
    my($group_list) = [ "alt.test",
			"alt.test.a",
			"alt.test.binaries",
			"alt.test.bogus",
			"alt.test.cjc",
			"alt.test.cmsg",
			"alt.test.d",
			"alt.test.ds",
			"alt.test.fishing",
			"alt.test.group",
			"alt.test.test",
			"alt.test.test2",
			"alt.test.test3",
			"alt.test.z",
		      ];

    # simulate crossposting by created a news posting messag with repeated but mutiply
    # newgroup entries.
    my($list_index) = 0;
    while ( 1 ) {
	# reached crosspost limit
	if ( @{$newsgroups} > ($n_user->crosspost_limit() + int(rand(5))) ) {
	    last;
	}

	# add the group
	push( @{$newsgroups}, $group_list->[ $list_index ] );

	# next newsgroup or start from beginning
	if ( $list_index > $#{$group_list} ) {
	    $list_index = 0;
	}
	else {
	    $list_index++;
	}
    }

    # execute the test case
    return Test::Case::Run_TCO_Single_News_Message( $connections,
						    $s_user, $n_user,
						    $newsgroups );
}

###############################################################################
#
# Test 4 - NYM to invalid newsgroup
#
###############################################################################

sub netnews_4 {

    print "\n########## NYM TO INVALID NEWSGROUP\n\n";

    # test specific parameters
    $s_user = $nym_factory->create_random_valid_nym();
    $n_user = $nym_factory->create_random_valid_nym();

    # create te newsgroup list
    my($newsgroups) = [ "xxx.yyy.zzz" ];

    # execute the test case
    return Test::Case::Run_TCO_Single_News_Message( $connections,
						    $s_user, $n_user,
						    $newsgroups );
}

#############################################################
# Execute each function in this TCO and return
# success/failure result.
#

print "\n";

# single user to user tests
ok (&netnews_1, "ok");
ok (&netnews_2, "ok");
ok (&netnews_3, "ok");
ok (&netnews_4, "ok");

print "\n";
