#! /bin/bash
#
# this is a driver script for the esmtpc client
#
# call as driver.sh <nymname>
#
# don't include the @test.network.zks.net!
#

set -e

rm -f email*

TEST_NET=t12.dev.zks.net
POP_HOST=avocado
ESMTP_HOST=cabbage

if [ $# -ne 2 ]
then
    echo "Wrong usage"
    exit
fi

USERNAME="$1"
ENVELOPE_TO="$2"
ENVELOPE_FROM="$USERNAME@$TEST_NET"
BASE64_USERNAME="$(echo -n $USERNAME | mimencode)"
BASE64_PASSWD="$($ZK_ROOT/arch/$ZK_ARCH/Debug/bin/genpasswd $USERNAME)"
ESMTPPORT=51115
POPPORT=51114
DATE="$(date -R)"

# ----------------------------------------------------------------------------
# Construct a raw email messages
#

#cat /home/caesar/roger/big-mail.txt > email.body

cat > email.body <<EOF
Message-ID: <$(./random 13)@freedom.net>
Path: localhost
From: $USERNAME <$USERNAME@$TEST_NET> 
Newsgroups: zks.cacophonix.test
Subject: This is just a test
Date: $DATE
Organization: zks mail to news gateway
Lines: 1

This is just a test, please ignore
EOF


cat > /dev/null <<EOF
Date: $DATE
From: $USERNAME@$TEST_NET
MIME-Version: 1.0
To: $ENVELOPE_TO
Subject: gif attached
Content-Type: multipart/alternative;
 boundary="------------8B6C43E9F1868E9E272C9861"

This is a multi-part message in MIME format.
--------------8B6C43E9F1868E9E272C9861
Content-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

This is a test email sent by the esmtpc test client
it contains a bunch of bytes, none of which are of any intrest
but they're bytes nonetheless, and they need to be here
so lets all just take a moment and pause and appreciate 
those bytes for who they are.

thank you.

P.S. There is a gif file attached
--------------8B6C43E9F1868E9E272C9861
Content-Type: text/html
Content-Transfer-Encoding: 7bit

<html>
<body>
<h1>Test Message</h1>

<p>This is a test email sent by the esmtpc test client it
contains a bunch of bytes, none of which are of any intrest
but they're bytes nonetheless, and they need to be here
so lets all just take a moment and pause and appreciate 
those bytes for who they are.</p>

<p>thank you.</p>

<p>P.S. There is a .gif file attached</p>
--------------8B6C43E9F1868E9E272C9861
Content-Type: image/gif;
 name="av.gif"
Content-Transfer-Encoding: base64
Content-Disposition: attachment;
 filename="av.gif"

R0lGODlhTQAeAOMAAP///9bn/87e773O56W13pzO/4yl1nOl/2uUzkpzvTFjpTFarRA5pQAx
nP///////yH+Dk1hZGUgd2l0aCBHSU1QACwAAAAATQAeAAAE7LDISau9mIYdwM5gKI4iZwKo
F5Bs65pwl6Kra99SHM9zjf8gnY5HBAJhKSGSWDS+ABMZc0rlOUmbhDaRk1a/zSsmu9VqwGir
uEIumyvpOHRdaLvfbDnadyzc72Neekx8Lyt/iIE7UydJGzQ2doiAGQFbDQgJCI89SieFgX6T
o1wSZQ1aDaqrq1oynjsfY6S0Zqy3uLiZNLAcKhe1d7kKucXGWry9vxS1xs7PzwoJyZ4eB9cH
0Nrb3K0JHb3Y2d3k5cUKgpy/1+bt7q2CTO/z7dPqvgD0+uWXwvv/AAMKHEiwoMGDCBMqXMiw
oaoIADs=
--------------8B6C43E9F1868E9E272C9861--
EOF

cat > email.failure.body <<EOF
Date: $DATE
From: $USERNAME@$TEST_NET
To: $USERNAME@$TEST_NET
Subject: Failed delivery

The message "gif attached" sent on $DATE to $ENVELOPE_TO
could not be delivered.
EOF

# ----------------------------------------------------------------------------
# Construct the final message
# ----------------------------------------------------------------------------


DOMAIN="${ENVELOPE_TO##*@}"
if [ "$DOMAIN" = "$TEST_NET" -a "${ENVELOPE_TO%@*}" != "mail2news" ]
then

# ------------------------------------------------
# We're simulating nym to nym
#

#
# Set up the email header
#

cat > email.header <<EOF
X-Freedom-Message-Auth: testheader
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary=-

This is a MIME encapsulated message
EOF

#
# Set up the body header
#

cat > email.body.header <<EOF
---
Content-Type: application/x-Freedom-Mail-Message-Body
Content-Encoding: base64

EOF

#
# Set up the failure header
#

cat > email.failure.header <<EOF
---
Content-Type: application/x-Freedom-Mail-Failure-Message
Content-Encoding: base64

EOF

#
# Set up the footer
#
cat > email.footer <<EOF
-----
EOF

#
# Build the message
#
cat email.body | mimencode > email.body.base64

cat email.failure.body | mimencode > email.failure

cat email.header \
    email.body.header \
    email.failure \
    email.footer | mimencode > email.failure.base64

cat email.header \
    email.body.header \
    email.body.base64 \
    email.failure.header \
    email.failure.base64 \
    email.footer > email.data

else

# ------------------------------------------------
# We're simulating nym to internet
#

cat > email.header <<EOF
EOF
#X-Freedom-Envelope-Sig: $ENVELOPE_TO 1234678BLAH
#EOF

cat email.header email.body > email.data

fi

#tcpclient $POP_HOST \
#          $POPPORT \
#          ./popclient \
#          "$USERNAME" \
#          "$BASE64_PASSWD" ;
# 
#echo "established account exists"

tcpclient $ESMTP_HOST \
          $ESMTPPORT ./esmtpc \
          $ENVELOPE_TO \
          $ENVELOPE_FROM \
	  email.data \
          "$BASE64_USERNAME" \
          "$BASE64_PASSWD" ;

echo "sent mail"
