#!/usr/bin/perl -w

use strict;
use URI::Escape;
use Getopt::Long;

require fr_scriptServerLib;

my $SCRIPTDIR = "./scripts";

##########################
## The commandline args ##
##########################

if ($#ARGV ne 1)
  {
    print "Usage: m2zk.pl <mapping file> <mozilla file>\n";
    exit;
  }

my ($mapFile, $mozFile) = @ARGV;

########################################
## Parse the mapping file into a hash ##
########################################

open MAPIN, $mapFile or die ("can't open $mapFile for read!\n");

my %mapHash;

foreach my $line (<MAPIN>)
  {
    if ($line =~ /.*\t.*/)
      {
	$line =~ s/\n//g;
	$mapHash{ (split "\t", $line)[0] } = (split "\t", $line)[1];
      }
  }

close MAPIN;

######################################
## Get the Mozilla file to be added ##
######################################

open MOZIN, $mozFile or die ("can't open $mozFile for read!\n");

my @completeMozFile = <MOZIN>;
my $mozFileAsString = join "", @completeMozFile;
my @mozDomains = split "\n\n\n", $mozFileAsString;

#######################################################
## Treat each section of the mozilla file seperately ##
#######################################################

foreach my $domain (@mozDomains)
  {
    ## get the timeStamp
    my $TIMESTRING = fr_getDateString();

    ## get rid of weirdness (seemingly empty domains)
    $domain =~ s/^.*?\n\n(.*)/$1/;

    ## make a copy
    my $completeScript = $domain;

    ## get just the domainname, not the url, the prefix server or the port
    $domain =~ s/^(.*?)\/.*?\n.*/$1/s;
    $domain =~ s/(.*):\d+/$1/;
    $domain =~ s/^.*?(\w+\.\w+)$/$1/;

    ## extract the script for the domain and throw it into a hash
    $completeScript =~ s/^.*?\n(.*)/$1/s;
    my @scriptChunks = split "\n\n", $completeScript;
    my %scriptHash;
    foreach my $chunk (@scriptChunks)
      {
	$scriptHash{ (split "\n", $chunk)[0] } = (split "\n", $chunk)[1];
      }

    ## get rid of any lines that can't be mapped
    ## and create a hash of mappings to zk names (escaped)
    my %resultHash;
    foreach (keys %scriptHash)
      {
	if ( defined $mapHash{$scriptHash{$_}})
	  {
	    $resultHash{ uri_escape($_ , "^A-Za-z1-9")} = uri_escape($mapHash{$scriptHash{$_}} , "^A-Za-z1-9");
	  }
	else
	  {delete $scriptHash{$_};}
      }

    ## ignore Ecom* entries as they are already dealt with by the client
    foreach (keys %resultHash)
      {
	if ($_ =~ /^Ecom/) {delete $resultHash{$_};}
      }

    ## if a script already exists, will have to watch for repeats and conflicts
    my $scriptFileName = `/bin/ls $SCRIPTDIR/$domain* 2>&1`;
    $scriptFileName =~ s/\n//;
    if ( $scriptFileName =~ /^$SCRIPTDIR\/$domain\.txt\.\d+$/ )
      {
	## get and parse the lines from the existing file
	open FIN , "$scriptFileName" or die ("Can't open $scriptFileName for read!\n");
	my @currentLines = <FIN>;
	close FIN;
	
	## verify each candidate
	foreach my $candidate (keys %resultHash)
	  {
	    ## compare to each line
	    LOOPINFILE: foreach my $existingLine (@currentLines)
	      {
		if ($candidate eq (split "\t", $existingLine)[0])
		  {
		    ## simply omit if the line is the same
		    if ("$resultHash{$candidate}\n" ne (split "\t", $existingLine)[1])
		      {print uri_unescape("CONFLICT:$domain:$candidate\t$resultHash{$candidate}:$existingLine");}
		    delete $resultHash{$candidate};
		    ## get the hell outa this mess!
		    last LOOPINFILE;
		  }
	      }
	  }
	## move the file to it's new location
	`/bin/mv $scriptFileName $SCRIPTDIR/$domain.txt.$TIMESTRING 2>&1`;
      }

    if (scalar keys %resultHash > 0)
      {

	open FOUT, ">>$SCRIPTDIR/$domain.txt.$TIMESTRING" or die ("can't open $SCRIPTDIR/$domain.txt.$TIMESTRING for write!\n");
	
	## tack on the new, non-conflicting lines
	foreach (keys %resultHash)
	  {print FOUT "$_\t$resultHash{$_}\n";}

	close FOUT;
      }

  }

