#ifndef TKSRV_H
# define TKSRV_H

// Include the protocol libray
# include "zktksrv.h"


// Module name
# define ZKS_TKSRV_MOD                                        "TokenServer"

// Errors definition
# define ZKS_TKSRV_EXIT_ERR                                   0x0001
# define ZKS_TKSRV_ABORT_ERR                                  0x0002
# define ZKS_TKSRV_EBADF_ERR                                  0x0003
# define ZKS_TKSRV_EINTR_ERR                                  0x0004
# define ZKS_TKSRV_EPIPE_ERR                                  0x0005
# define ZKS_TKSRV_EINVAL_ERR                                 0x0006
# define ZKS_TKSRV_DBOPEN_ERR                                 0x0007
# define ZKS_TKSRV_DBCLOSE_ERR                                0x0008
# define ZKS_TKSRV_DBPUT_ERR                                  0x0009
# define ZKS_TKSRV_DBGET_ERR                                  0x000a
# define ZKS_TKSRV_DBDEL_ERR                                  0x000b
# define ZKS_TKSRV_DBSYNC_ERR                                 0x000c
# define ZKS_TKSRV_DBCURSOR_ERR                               0x000d
# define ZKS_TKSRV_DBCGET_ERR                                 0x000e
# define ZKS_TKSRV_DBCPUT_ERR                                 0x000f
# define ZKS_TKSRV_DBCDEL_ERR                                 0x0010
# define ZKS_TKSRV_DBCCLOSE_ERR                               0x0011
# define ZKS_TKSRV_OPEN_ERR                                   0x0012
# define ZKS_TKSRV_GETPROTOBYNAME_ERR                         0x0013
# define ZKS_TKSRV_SOCKET_ERR                                 0x0014
# define ZKS_TKSRV_SETSOCKOPT_ERR                             0x0015
# define ZKS_TKSRV_BIND_ERR                                   0x0016
# define ZKS_TKSRV_LISTEN_ERR                                 0x0017
# define ZKS_TKSRV_ACCEPT_ERR                                 0x0018
# define ZKS_TKSRV_GETHOSTNAME_ERR                            0x0019
# define ZKS_TKSRV_GETHOSTBYNAME_ERR                          0x001a
# define ZKS_TKSRV_SELECT_ERR                                 0x001b
# define ZKS_TKSRV_EVTIME_TYPE_ERR                            0x001c
# define ZKS_TKSRV_EVENT_TYPE_ERR                             0x001d
# define ZKS_TKSRV_UNAME_ERR                                  0x001e
# define ZKS_TKSRV_IPVERSION_ERR                              0x001f
# define ZKS_TKSRV_BUFFSZ_ERR                                 0x0020
# define ZKS_TKSRV_CALLOC_ERR                                 0x0021
# define ZKS_TKSRV_FLAGS_ERR                                  0x0022
# define ZKS_TKSRV_KEYCHAIN_ERR                               0x0023
# define ZKS_TKSRV_PROTOCOL_ERR                               0x0024
# define ZKS_TKSRV_STATE_ERR                                  0x0025
# define ZKS_TKSRV_TIMEOUT_ERR                                0x0026


// Application version
# define ZKS_TKSRV_VERSION                                    "2.0"

// Ini section
# define ZKS_TKSRV_SECTION                                    "TOKEN"

// Ini keys
# define ZKS_TKSRV_LOG_FILE_KEY                               "log.file"                  // path
# define ZKS_TKSRV_DEBUG_KEY                                  "debug"                     // bool
# define ZKS_TKSRV_VERBOSE_KEY                                "verbose"                   // bool
# define ZKS_TKSRV_DAEMON_KEY                                 "daemon"                    // bool

# define ZKS_TKSRV_ENTYTY_NAME_KEY                            "entity.name"               // string
# define ZKS_TKSRV_ENTYTY_TYPE_KEY                            "entity.type"               // string

# define ZKS_TKSRV_KEY_PRIV_CRYPT_DIR_KEY                     "key.crypt.dir"             // path
# define ZKS_TKSRV_KEY_PRIV_SIG_DIR_KEY                       "key.sig.dir"               // path
# define ZKS_TKSRV_KEY_PRIV_CRYPT_VERSION_KEY                 "key.crypt.version"         // uint16_t
# define ZKS_TKSRV_KEY_PRIV_SIG_VERSION_KEY                   "key.sig.version"           // uint16_t

# define ZKS_TKSRV_PASSPHRASE_FILE_KEY                        "passphrase.file"           // path
# define ZKS_TKSRV_PASSPHRASE_PROMPT_KEY                      "passphrase.prompt"         // string

# define ZKS_TKSRV_DAEMON_INET_COUNT_KEY                      "daemon.inet.srvcount"      // uint32_t
# define ZKS_TKSRV_DAEMON_INET_IP_KEY                         "daemon.inet.ip"            // string
# define ZKS_TKSRV_DAEMON_INET_PORT_KEY                       "daemon.inet.port"          // uint16_t
# define ZKS_TKSRV_DAEMON_INET_BACKLOG_KEY                    "daemon.inet.backlog"       // int
# define ZKS_TKSRV_DAEMON_INET_REUSEADDR_KEY                  "daemon.inet.reuseaddr"     // bool
# define ZKS_TKSRV_DAEMON_INET_ADDR_ANY_KEY                   "daemon.inet.any"           // bool

# define ZKS_TKSRV_DAEMON_LOCAL_SOCKFILE_KEY                  "daemon.local.sockfile"     // path
# define ZKS_TKSRV_DAEMON_LOCAL_BACKLOG_KEY                   "daemon.local.backlog"      // int
# define ZKS_TKSRV_DAEMON_LOCAL_REUSEADDR_KEY                 "daemon.local.reuseaddr"    // int

# define ZKS_TKSRV_INET_RCVBUF_KEY                            "inet.recv.buffsz"          // int
# define ZKS_TKSRV_INET_SNDBUF_KEY                            "inet.send.buffsz"          // int

# define ZKS_TKSRV_LOCAL_RCVBUF_KEY                           "local.recv.buffsz"         // int
# define ZKS_TKSRV_LOCAL_SNDBUF_KEY                           "local.send.buffsz"         // int

# define ZKS_TKSRV_SERVER_INET_RCV_TIMEOUT_KEY                "server.inet.recv.timeout"  // in seconds
# define ZKS_TKSRV_SERVER_INET_SND_TIMEOUT_KEY                "server.inet.send.timeout"  // in seconds
# define ZKS_TKSRV_SERVER_LOCAL_RCV_TIMEOUT_KEY               "server.local.recv.timeout" // in seconds
# define ZKS_TKSRV_SERVER_LOCAL_SND_TIMEOUT_KEY               "server.local.send.timeout" // in seconds

# define ZKS_TKSRV_SERIAL_DB_FILENAME_KEY                     "db.serial.filename"        // path


#endif /* TKSRV_H */
