#ifndef TKSRV_APP_H
# define TKSRV_APP_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif


# ifdef __cplusplus  

#  include "tksrv.h"
#  include "bdb.h"
#  include "keys.h"

#  include "zks.h" 
#  include "zkIntTypes.h"
#  include "zkscmdline.h"
#  include "zksini.h"

#  include <string> 


// /////////////////////////////////////////////////////////////////////////////
// Application context handler
//
class Application {
  Application(const Application &);
  Application & operator=(const Application &);  

 public:
  enum State { 
    ACCEPT = 0,  // Accept state or wait to be on accept mode
    HOLD,        // Hold state or wait to be on hold
    STOP};       // Stop state or wait to be on stop

 public:
  Application()
    :m_debug(false)
    ,m_verbose(false)
    ,m_passprompt("")
    ,m_passfile("")
    ,m_daemon(false)
    ,m_state(ACCEPT) 
    ,m_waitfor(false) 
    ,m_priv_keys(0)
    ,m_crypt_version(0)
    ,m_sig_version(0)
    ,m_serial_db(0) 
    {
    }

  virtual ~Application()
    {
      delete m_priv_keys;
    }

  void init_appname(int32_t argc, const char ** argv);
  void init_cmdline(int32_t argc, const char ** argv);
  void init_ini();
  void init_logfile();
  void init_debug_status();
  void init_verbose_status();
  void init_passphrase();
  void init_priv_keys();
  void init_daemon_status();

  void run();

  void startupBanner()        const;
  void shutdownBanner()       const;

  void daemonyze();

  void open_db();
  void close_db();

  // State handling
  bool  asReach(State s)      const;  // return true if we have reach the state 's'
  bool  isWaitingfor(State s) const;  // return true if we are waiting for the state 's'

  void  waitfor(State s);             // set state we are waiting for
  void  reach(State s);               // signal that we reach the state we are waiting for

  inline bool isWaitingFor()  const  { return m_waitfor; }

  // Attribute access
  inline const std::string & name()              const { return m_appname; } 
  inline const CZKS::Ini   & ini()               const { return m_ini; } 
  inline TkBdb               serial_db()         const { return m_serial_db; } 
  inline TkKeys            * priv_keys()         const { return m_priv_keys; } 
  inline uint16_t            crypt_key_version() const { return m_crypt_version; }
  inline uint16_t            sig_key_version()   const { return m_sig_version; }

  inline bool                debug()             const { return m_debug; }
  inline bool                verbose()           const { return m_verbose; }

 protected:
  std::string   m_appname;            // Name of the application.
  CZKS::Ini     m_ini;                // Configuration file handler.
  CZKS::CmdLine m_cmdline;            // Command line handler.
  std::string   m_logfile;            // Log file name.
  std::string   m_inifile;            // Configuration file name.
  bool          m_debug;              // true if running in debug mode. (enable console logging)
  bool          m_verbose;            // true if running as a daemon.   (enable extra messages)
  std::string   m_passprompt;         // Passphrase prompt
  std::string   m_passfile;           // Passphrase file name
  bool          m_daemon;             // true if running as a daemon.
  bool          m_state;              // return the current or expected state (depending if waitfor is true )
  bool          m_waitfor;            // true if the daemon wait for a given state
  TkKeys      * m_priv_keys;          // Private keys handler
  uint16_t      m_crypt_version;      // Version of the current crypto key
  uint16_t      m_sig_version;        // Version of the current signature key
  TkBdb         m_serial_db;          // Serial number database handler
};


# endif /* __cplusplus */

#endif /* TKSRV_APP_H */
