#ifndef TKSRV_DB_H
# define TKSRV_DB_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus 
extern "C" {
# endif

# include "zks.h" 
# include "zkTypes.h"
# include "zkIntTypes.h"
# include "zkserr.h" 

  typedef struct S_TkBdb      * TkBdb;
  typedef struct S_TkdbCursor * TkdbCursor;

  typedef enum {
    TKBDB_NONE       = 0x00000000,
    TKBDB_AFTER      = 0x00000001,
    TKBDB_BEFORE     = 0x00000002,
    TKBDB_CURRENT    = 0x00000004,
    TKBDB_KEYFIRST   = 0x00000008,
    TKBDB_KEYLAST    = 0x00000010,
    TKBDB_FIRST      = 0x00000020,
    TKBDB_LAST       = 0x00000040,
    TKBDB_NEXT       = 0x00000080,
    TKBDB_NEXT_DUP   = 0x00000100,
    TKBDB_PREV       = 0x00000200,
    TKBDB_SET        = 0x00000400,
    TKBDB_SET_RANGE  = 0x00000800,
    TKBDB_GET_BOTH   = 0x00001000,
    TKBDB_SET_RECNO  = 0x00002000,
    TKBDB_GET_RECNO  = 0x00004000,
  } TkBdbMode;


  ZkBool    tkBdbOpen                     (ZksErr       err,
                                           TkBdb      * dbhP,
                                           const char * pathname, 
                                           ZkBool       no_dup, 
                                           ZkBool       auto_rec_no);


  int       tkBdbClose                    (ZksErr   err, 
                                           TkBdb  * dbhP);


  int       tkBdbPut                      (ZksErr       err, 
                                           TkBdb        dbh, 
                                           void       * key, 
                                           uint32_t     ksz, 
                                           const void * data, 
                                           uint32_t     dsz);


  int       tkBdbGet                      (ZksErr       err, 
                                           TkBdb        dbh, 
                                           const void * key, 
                                           uint32_t     ksz, 
                                           void       * data, 
                                           uint32_t     dsz);



  int       tkBdbDel                      (ZksErr       err, 
                                           TkBdb        dbh, 
                                           const void * key, 
                                           uint32_t     ksz);


  int       tkBdbSync                     (ZksErr       err, 
                                           TkBdb        dbh);


  int       tkBdbCursorNew                (ZksErr       err, 
                                           TkdbCursor * dbcP,
                                           TkBdb        dbh);


  int       tkBdbCursorFree               (ZksErr       err, 
                                           TkdbCursor * dbcP);


  int       tkBdbCursorPut                (ZksErr        err, 
                                           TkdbCursor    dbc, 
                                           void        * key, 
                                           uint32_t      ksz, 
                                           const void  * data, 
                                           uint32_t      dsz,
                                           TkBdbMode     mode);


  int       tkBdbCursorGet                (ZksErr        err, 
                                           TkdbCursor    dbc, 
                                           void        * key, 
                                           uint32_t      ksz, 
                                           const void  * data, 
                                           uint32_t      dsz,
                                           TkBdbMode     mode);


  int       tkBdbCursorDel                (ZksErr       err, 
                                           TkdbCursor   dbc);

# ifdef __cplusplus 
}
# endif

#endif /* TKSRV_DB_H */
