#ifndef TKSRV_DBO_H
# define TKSRV_DBO_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus  

#  include "tksrv.h"

#  include "bdb.h"

#  include "zks.h" 
#  include "zkIntTypes.h"
#  include "zkssmartptr.h"
#  include "zksexception.h"



// /////////////////////////////////////////////////////////////////////////////
// TkDbObject description
//
class TkDbObject {
 public:
  TkDbObject() {}

  // Key/Data access
  virtual const void    * key()  const = 0;
  virtual void          * key()        = 0;
  virtual uint32_t        ksz()        = 0;

  virtual const void    * data() const = 0;
  virtual void          * data()       = 0;
  virtual uint32_t        dsz()        = 0;
};



class TkDbSerial: public TkDbObject {
  enum { 
    SERIAL_POS      = 0,
    CKSUM_POS       = SERIAL_POS   + ZKS_ZKTKSRV_SERIAL_CKSUM_LENGTH,    
    CAPACITY_POS    = CKSUM_POS    + ZKS_ZKTKSRV_SERIAL_CAPACITY_LENGTH,
    PASSWD_POS      = CAPACITY_POS + ZKS_ZKTKSRV_SERIAL_LENGTH,    
    STATUS_POS      = PASSWD_POS   + ZKS_ZKTKSRV_SERIAL_PASSWORD_LENGTH,
    END_POS         = STATUS_POS   + sizeof(char)
  };
 public:
  TkDbSerial() {}
    
  static uint32_t size()       { return END_POS; }
  void            clear()      { memset(m_buff, 0, size()+1); }

  // Key/Data access
  const void    * key()  const { return &m_buff[0]; }
  void          * key()        { return &m_buff[0]; }
  uint32_t        ksz()        { return ZKS_ZKTKSRV_SERIAL_LENGTH; }

  const void    * data() const { return &m_buff[0]; }
  void          * data()       { return &m_buff[0]; }
  uint32_t        dsz()        { return size(); }


  // Attributs access
  void            serial(const std::string & s)   { memcpy(&m_buff[SERIAL_POS], s.c_str(), ZKS_ZKTKSRV_SERIAL_LENGTH); }
  std::string     serial() const                  { return std::string(&m_buff[SERIAL_POS], ZKS_ZKTKSRV_SERIAL_LENGTH); }

  void            cksum(const std::string & s)    { memcpy(&m_buff[CKSUM_POS], s.c_str(), ZKS_ZKTKSRV_SERIAL_CKSUM_LENGTH); }
  std::string     cksum() const                   { return std::string(&m_buff[CKSUM_POS], ZKS_ZKTKSRV_SERIAL_CKSUM_LENGTH); }

  void            capacity(const std::string & s) { memcpy(&m_buff[CAPACITY_POS], s.c_str(), ZKS_ZKTKSRV_SERIAL_CAPACITY_LENGTH); }
  std::string     capacity() const                { return std::string(&m_buff[CAPACITY_POS], ZKS_ZKTKSRV_SERIAL_CAPACITY_LENGTH); }

  void            passwd(const std::string & s)   { memcpy(&m_buff[PASSWD_POS], s.c_str(), ZKS_ZKTKSRV_SERIAL_PASSWORD_LENGTH); }
  std::string     passwd() const                  { return std::string(&m_buff[PASSWD_POS], ZKS_ZKTKSRV_SERIAL_PASSWORD_LENGTH); }

  void            status(bool enable)             {  m_buff[STATUS_POS] = enable ? 1 : 0; }
  bool            status() const                  {  return m_buff[STATUS_POS]; }

 private:
  char m_buff[END_POS+1];
};




# endif /* __cplusplus */

#endif /* TKSRV_DBO_H */
