#ifndef TKSRV_EVENT_H
# define TKSRV_EVENT_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus  

#  include "zkstimeval.h"

#  include "zks.h" 
#  include "zkIntTypes.h"


class Server;


// /////////////////////////////////////////////////////////////////////////////
// Event handler
//
// - These event are used by servers to signal the main loop of which next state
//   is expected. When the state is ready the main loop callback the server.
//
class Event {
 public:
  enum Type {
    CNX,       // wait for connection
    RCV,       // wait for reception
    SND,       // wait for transmission
    TIMEOUT    // wait for a timeout
  };
 public:
  inline Event(Server * srv, Type type, CZKS::Timeval timeout);

  inline Event(const Event & ev);

  Server        * server()  const { return m_srv; }
  const Type    & type()    const { return m_type; }
  const CZKS::Timeval   timeout() const { return m_timeout; }

 private:
  Server        * m_srv;
  Type            m_type;
  CZKS::Timeval         m_timeout;
};


Event::Event(Server * srv, Type type, CZKS::Timeval timeout)
  :m_srv(srv)
  ,m_type(type)
  ,m_timeout(timeout + CZKS::Timeval(CZKS::Timeval::NOW))
{
}


Event::Event(const Event & ev)
  :m_srv(ev.m_srv)
  ,m_type(ev.m_type)
  ,m_timeout(ev.m_timeout) 
{
}


# endif /* __cplusplus */

#endif /* TKSRV_EVENT_H */
