#ifndef TKSRV_KEYS_H
# define TKSRV_KEYS_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus  

#  include "tksrv.h"

#  include "zkspath.h"
#  include "zkssmartptr.h"
#  include "zksexception.h"

#  include <frSec.h>
#  include <string>
#  include <iostream>


//
// Run Once use to initialyse the crypto library
//
class CryptoLib {
 private:
  explicit CryptoLib() {
    if( !frCryptInit(0, 0) ) {
      std::cout << "Startup failure: frCryptInit failed" << std::endl;
    }
  }
  static CryptoLib * m_instance;
};




//
// Key handler
//
class TkKeys {
  enum { SIG_KEY_VERSION   = 1 };
  enum { CRYPT_KEY_VERSION = 1 };

 private:
  TkKeys(const TkKeys & k);
  TkKeys & operator=(const TkKeys & k);

 public:
  TkKeys(const std::string & entityName,
         FrEntIdType         entityType);

  virtual ~TkKeys();

  void          load(const CZKS::Path  & priv_crypt,
                     const CZKS::Path  & priv_sig,
                     const std::string & prompt, 
                     const CZKS::Path  & passphrase_file);

  const FrCryptPubKey getCryptKey(uint32_t version) const;
  const FrCryptSig    getSigKey  (uint32_t version) const;

  const std::string & getEntityName() const { return m_entname; }
  const FrEntIdExp  & getEntity()     const { return m_entity;  }

 private:
  bool  validateKeys(const FrCryptPubKey & key) const;

 private:
  FrSecKeyChain  m_keys;        // structure to store key info
  std::string    m_entname;     // entity name
  FrEntIdExp     m_entity;      // entity
  size_t         m_sigHandle;   // handle to the signature key
  size_t         m_cryptHandle; // handle to the crypt key
};


# endif /* __cplusplus */

#endif /* TKSRV_KEYS_H */
