#ifndef TKSRV_TKREQUEST_H
# define TKSRV_TKREQUEST_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

# ifdef __cplusplus  

#  include "tksrv.h"
#  include "engine.h"
#  include "app.h"
#  include "zkstimeval.h"
#  include "event.h"
#  include "dbo.h"

#  include "zks.h" 
#  include "zkIntTypes.h"
#  include "zkIntTypes.h"

#  include <sys/time.h>
#  include <sys/types.h>
#  include <unistd.h>
#  include <string>


// /////////////////////////////////////////////////////////////////////////////
//
// SN-Token echange state machine handler
//
class TokenRequestEngine: public Engine {
  enum State { WAIT_RCV_SN = 0,   // Wait to receive Serial number
               WAIT_SND_TOKEN,    // Wait to send Token
               WAIT_SND_ERROR,    // Wait to send and error
               DONE};

 public:
  TokenRequestEngine(Server * srv);
  ~TokenRequestEngine();

  Event rcv();
  Event snd();
  Event timeout();

 protected:
  void retrieveAndLockSerial(TkDbSerial        & record, 
                             const std::string & serial, 
                             const std::string & passwd);

  void parseSerialCapacity(const std::string & capacity, 
                           uint16_t          & token_count,
                           uint16_t          & token_type,
                           time_t            & activation);

  void generateToken(char      * buffer, 
                     uint16_t    bsz,                   
                     uint16_t    token_type);

  void fillResponse(TokenResponsePacket & pkt,
                    const TkDbSerial    & record);

 private:
  State       m_state;
  ZksErrCode  m_errcode;
  TkDbSerial  m_serial_record;
  FrToken     m_token;
};


# endif /* __cplusplus */

#endif /* TKSRV_TKREQUEST_H */
