#include "Client.h"

int netsock, shimsock;
bool runthread;

Client::Client() {

  try {
	//Look at me, i'm a hack.
	//netthread, shimthread and the main thread were entering a race
	//condition over the creation of a linkmanager instance (and instances
	//of the other objects listed).  this has the unfortunately side-effect
	//of having multiple instances of each object, which pretty much 
	//undermines the whole singleton concept.  this forces the creation of
	//each object, and then everyone plays nice.  
	//also... a neater method would be to have instance be a locked method,
	//but it's static so that don't work. :(
	LinkManager * linkman = linkman->Instance();
	RouteManager * routeman = routeman->Instance();
	ConfigManager * configman = configman->Instance();
  
    int ret_val = pthread_create(&netthread, 0, NetworkListener, 0);
    if (ret_val != 0)
      throw TestModuleException(ret_val, "pthread_create()");
    ret_val = pthread_detach(netthread);
    if (ret_val != 0)
      throw TestModuleException(ret_val, "pthread_detach()");

    ret_val = pthread_create(&shimthread, 0, ShimListener, 0);
    if (ret_val != 0)
      throw TestModuleException(ret_val, "pthread_create()");
    ret_val = pthread_detach(shimthread);
    if (ret_val != 0)
      throw TestModuleException(ret_val, "pthread_detach()");

    netsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (netsock < 0)
      throw TestModuleException(errno, "socket()");

    try {
      ShimHandler * shim = shim->Instance();
      shimsock = shim->GetShimFD();
    }
    catch (TestModuleException & tme) {
      cerr << tme.what();
    }
  }
  catch (TestModuleException & tme) {
    tme.AddScope("Client ctor");
    throw;
  }
}

Client::~Client() {
  runthread = false;
  close(netsock);
}

void Client::AddRoute(LayerType layert, string * hosts, unsigned short port,
                      string nymtype) {
  try {
    
    RouteManager * routemgr = routemgr->Instance();
    LinkManager * linkmgr = linkmgr->Instance();

    RouteBuilder rb;
    FreedomCreatePacket frpkt;
    Route * newroute = rb.Create(frpkt, layert, hosts, port, nymtype);
    routemgr->AddRoute(newroute);
    struct sockaddr_in addr = newroute->GetFirstHopAddress();
    addr.sin_family = AF_INET;
    addr.sin_port = htons(51101);
    frpkt.SetAddr(addr);
    linkmgr->OutgoingPacket(frpkt);
    frpkt.SendToNetwork(netsock,addr);
  }
  catch (TestModuleException &tme) {
    RemoveRoute(port);
    tme.AddScope("AddRoute()");
    throw;
  }
}

uint16_t Client::ExtendRoute(uint16_t aci, LayerType layert, string * hosts) {

  try {
    RouteManager * routemgr = routemgr->Instance();
    LinkManager * linkmgr = linkmgr->Instance();
    
    Route * oldroute = routemgr->GetRoute(aci);
    uint16_t newaci = static_cast<uint16_t>(rand() % 65535 + 1);
    RouteBuilder rb;
    FreedomCreatePacket frpkt;
    Route * newroute = rb.Extend(*oldroute, frpkt, layert, hosts, newaci);
    routemgr->AddRoute(newroute);
    struct sockaddr_in addr = newroute->GetFirstHopAddress();
    addr.sin_family = AF_INET;
    addr.sin_port = htons(51101);
    frpkt.SetAddr(addr);
    linkmgr->OutgoingPacket(frpkt);
    frpkt.SendToNetwork(netsock,addr);
    return newaci;
  }
  catch (TestModuleException &tme) {
    RemoveRoute(aci);
    tme.AddScope("ExtendRoute()");
    throw;
  }
}

void Client::RemoveRoute(uint16_t aci) {

  RouteManager * routemgr = routemgr->Instance();
  LinkManager * linkmgr = linkmgr->Instance();

  Route* oldroute = routemgr->RemoveRoute(aci);
  if (oldroute!=0) {
    RouteBuilder rb;
    FreedomDestroyPacket frpkt;
    oldroute->BuildRouteDestroyPacket(frpkt,1,"death to route");
    struct sockaddr_in addr = oldroute->GetFirstHopAddress();
    addr.sin_family = AF_INET;
    addr.sin_port = htons(51101);
    frpkt.SetAddr(addr);
    linkmgr->OutgoingPacket(frpkt);
    frpkt.SendToNetwork(netsock,addr);
  }
}

void * Client::NetworkListener(void * noop) {
  try {
    RouteManager * routemgr = routemgr->Instance();
    LinkManager * linkmgr = linkmgr->Instance();
    ConfigManager * configmgr = configmgr->Instance();
    
    struct pollfd netpoll;

    while (1) {
      memset(&netpoll, 0, sizeof(netpoll));
      netpoll.fd = netsock;
      netpoll.events = POLLIN;
      netpoll.revents = 0;
      int ret = poll(&netpoll, 1, configmgr->Timeout() * 1000);
      if (ret == 1) {
		try {
		  FreedomPacket frpkt;
		  frpkt.ReceiveFromNetwork(netsock);
		  FreedomPacket * ldpkt = linkmgr->IncomingPacket(frpkt);
		  if (ldpkt) {
			FreedomDataPacket * datapkt = routemgr->IncomingPacket(*ldpkt);
			if (datapkt)
			  datapkt->SendToShim(shimsock);
		  }
		}
		catch (RouteCreateException &rce) {
		  cerr<<"picked up an ack from the network\n";
		  cerr.flush();
		}
		catch (RouteDestroyException &rde) {
		  cerr<<"picked up a destroy from the network\n";
		  cerr.flush();
		}
		catch (TestModuleException &tme) {
		  tme.AddScope("Client::NetworkListener()");
		  cerr<<tme.what()<<"\n";
		  cerr.flush();
		}
		catch (...) {
		  cerr<<"something bad just happened over the network\n";
		  cerr.flush();
		  break;
		}
      }
    }
    return noop;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("NetworkListener()");
    throw;
  }
}

void * Client::ShimListener(void * noop) {

  try {
    RouteManager * routemgr = routemgr->Instance();
    LinkManager * linkmgr = linkmgr->Instance();
    ConfigManager * configmgr = configmgr->Instance();

    struct pollfd shimpoll;
    while (1) {
      memset(&shimpoll, 0, sizeof(shimpoll));
      shimpoll.fd = shimsock;
      shimpoll.events = POLLIN;
      shimpoll.revents = 0;
      int ret = poll(&shimpoll, 1, configmgr->Timeout() * 1000);
      if (ret>0) {
	try {
	  FreedomDataPacket frpkt;
	  frpkt.ReceiveFromShim(shimsock);
	  routemgr->OutgoingPacket(frpkt);
	  linkmgr->OutgoingPacket(frpkt);
	  sockaddr_in addr;
	  frpkt.GetAddr(addr);
	  frpkt.SendToNetwork(netsock,addr);
	}
	catch (TestModuleException &tme) {
	  tme.AddScope("Client::ShimListener()");
	  cerr<<tme.what()<<"\n";
	}
	catch (...) {
	  cerr<<"something bad just happened processing shim data\n";
	}
      }
    }
    return noop;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("ShimListener()");
    throw;
  }
}

string Client::ToString() const{
  ostringstream returnstream;
  RouteManager * routemgr = routemgr->Instance();
  returnstream<<routemgr->ToString();
  return returnstream.str();
}
