/*  Author(s):    Alice Nodelman and Troy McLelan
 *  Description:  A test client class, performs test client functions such as
 *                creating and destroying routes.
 *  Todo:         - Fix ToString() for maximum output.
 *                - More control over singletons, which should be destroy-albe
 *                  by this object.
 *                - Must handle cases where incoming and outgoing packets
 *                  don't follow the data packet path.
 *                - Better exception handling, especially within threads.
 *  Known Bugs:   None.
 */

#ifndef ZK_AIP_TESTCLI_JJDFKSZK_DEF
#define ZK_AIP_TESTCLI_JJDFKSZK_DEF

#include "RouteManager.h"

class Client {
public:
  Client();
  ~Client();
  void AddRoute(LayerType layert, string * hosts, unsigned short port,
                string nymtype);
  uint16_t ExtendRoute(uint16_t aci, LayerType layert, string * hosts);
  void RemoveRoute(uint16_t aci);
  void TestAllRoutes();
  void TestNetwork();
  string ToString() const;
  string SystemTest(size_t numloops = 1);
private:
  Client(Client &);
  Client& operator=(const Client&);
  void operator&();

  static void * NetworkListener(void * noop);
  static void * ShimListener(void * noop);
  
  pthread_t netthread, shimthread;
};

extern int netsock, shimsock;

#endif
