#include "TestModuleSet.h"

ClientModule::ClientModule(){
  myname = "client";
  if (reg[myname] !=0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  activated = false;
  helpmessage = "This module handles route creation/extension.\n";
  helpmessage += "client help\n";
  helpmessage += "client create <numhops> <hostlist>...\n";
  helpmessage += "client auth <numhops> <hostlist>...\n";
  helpmessage += "client extend <aci> <numhops> <hostlist>...\n";
  helpmessage += "client remove <aci>\n";
  helpmessage += "client block <port> <numhops> <hostlist>...\n";
  helpmessage += "client blockauth <port> <numhops> <hostlist>...\n";
  helpmessage += "client send <aci> <iterations>\n";
  helpmessage += "client print\n";
  helpmessage += "\nBuilt in test programs:\n";
  helpmessage += "client single <iterations>\n";
  helpmessage += "client multi <numhops> <iterations>\n";
}

ClientModule::~ClientModule(){
  Deactivate();
  reg.erase(reg.find(myname));
}

void ClientModule::Activate(){
  if (!activated) {
    client = new Client();
    activated = true;
  }
}

void ClientModule::Deactivate(){
  if (activated) {
    delete client;
    activated = false;
  }
}

void ClientModule::HandleRequest(UserRequest & request) {
  try {
    Activate();
    if (request.IsCommandEqual("client help"))
      request.SetResponse(helpmessage);
    else if (request.IsCommandEqual("client shutdown")) {
      Deactivate();
      request.SetResponse(myname + " module shutdown");
    } 
    else if (request.IsCommandEqual("client test")) {
      request.SetResponse("Not supported");
    } 
    else if (request.GetNthToken(1) == "create") {
      ostringstream returnstream;
      size_t numhops = static_cast<size_t>(atoi(request.GetNthToken(2).c_str()));
      if ((numhops == 0) || (numhops > 5)) {
        request.SetResponse("Bad numhops value:  " + request.GetNthToken(2));
        return;
      }
      string hostlist[5];
      for (size_t i = 0; i < numhops; i++)
        hostlist[i] = request.GetNthToken(3 + i);
      uint16_t aci = static_cast<uint16_t>(rand()>>16);
      client->AddRoute(static_cast<LayerType>(numhops), hostlist,aci, "");
      returnstream<<"Successfully created route with aci "<<static_cast<int>(aci);
      request.SetResponse(returnstream.str());
    } 
	else if (request.GetNthToken(1) == "block") {
      ostringstream returnstream;
      uint16_t aci = static_cast<uint16_t>(atoi(request.GetNthToken(2).c_str()));
      size_t numhops = static_cast<size_t>(atoi(request.GetNthToken(3).c_str()));
      if ((numhops == 0) || (numhops > 5)) {
        request.SetResponse("Bad numhops value:  " + request.GetNthToken(2));
        return;
      }
      string hostlist[5];
      for (size_t i = 0; i < numhops; i++)
        hostlist[i] = request.GetNthToken(4 + i);
      client->AddRoute(static_cast<LayerType>(numhops), hostlist,aci, "");
      returnstream<<"Successfully created route with aci "<<static_cast<int>(aci);
      request.SetResponse(returnstream.str());

	}
    else if (request.GetNthToken(1) == "extend") {
      ostringstream returnstream;
      uint16_t aci =
	static_cast<uint16_t>(atoi(request.GetNthToken(2).c_str()));
      size_t numhops = static_cast<size_t>(atoi(request.GetNthToken(3).c_str()));
      if ((numhops == 0) || (numhops > 5)) {
        request.SetResponse("Bad numhops value:  " + request.GetNthToken(3));
        return;
      }
      string hostlist[5];
      for (size_t i = 0; i < numhops; i++)
        hostlist[i] = request.GetNthToken(4 + i);
      uint16_t newaci = client->ExtendRoute(aci,static_cast<LayerType>(numhops), hostlist);
      returnstream<<"Successfully extended route, new aci is "<<static_cast<int>(newaci);
      request.SetResponse(returnstream.str());
    }
    else if (request.GetNthToken(1) == "remove") {
      ostringstream returnstream;
      uint16_t aci =
	static_cast<uint16_t>(atoi(request.GetNthToken(2).c_str()));
      client->RemoveRoute(aci);
      returnstream<<"Removed route";
      request.SetResponse(returnstream.str());

    }

    else if (request.GetNthToken(1) == "auth") {
      ostringstream returnstream;
      size_t numhops = static_cast<size_t>(atoi(request.GetNthToken(2).c_str()));
      if ((numhops == 0) || (numhops > 5)) {
        request.SetResponse("Bad numhops value:  " + request.GetNthToken(2));
        return;
      }
      string hostlist[5];
      for (size_t i = 0; i < numhops; i++)
        hostlist[i] = request.GetNthToken(3 + i);
      uint16_t aci = static_cast<uint16_t>(rand()>>16);
      client->AddRoute(static_cast<LayerType>(numhops), hostlist,aci, "premium");
      returnstream<<"Successfully created route with aci "<<static_cast<int>(aci);
      request.SetResponse(returnstream.str());
    } 
	else if (request.GetNthToken(1) == "blockauth") {
      ostringstream returnstream;
      uint16_t aci = static_cast<uint16_t>(atoi(request.GetNthToken(2).c_str()));
      size_t numhops = static_cast<size_t>(atoi(request.GetNthToken(3).c_str()));
      if ((numhops == 0) || (numhops > 5)) {
        request.SetResponse("Bad numhops value:  " + request.GetNthToken(2));
        return;
      }
      string hostlist[5];
      for (size_t i = 0; i < numhops; i++)
        hostlist[i] = request.GetNthToken(4 + i);

      client->AddRoute(static_cast<LayerType>(numhops), hostlist,aci, "premium");
      returnstream<<"Successfully created route with aci "<<static_cast<int>(aci);
      request.SetResponse(returnstream.str());
	}
    else if (request.GetNthToken(1)=="single"){
      ostringstream returnstream;
      NetData * netobj = netobj->Instance();
      string hostlist[5];
      size_t count = 0;
      size_t iterations = atoi(request.GetNthToken(2).c_str());
      if (iterations==0) iterations++;
      size_t pass, fail, destroy, timeout;
      netobj->GetAipNames(hostlist,5,count);
      for (size_t i = 0;i<count;i++) {
		pass = fail = destroy = timeout = 0;
		string host[1];
		host[0] = hostlist[i];
		uint16_t aci =0;
		for (size_t j = 0;j<iterations;j++) {
		  try {
			aci = static_cast<uint16_t>(rand()>>16);
			client->AddRoute(LAYER1, host,aci, "");
			pass++;
		  }
		  catch (RouteDestroyException &rd) {
			destroy++;
		  }
		  catch (TimeoutException &te) {
			timeout++;
		  }
		  catch (TestModuleException &tme) {
			fail++;
		  }
		  try {
			client->RemoveRoute(aci);
		  }
		  catch (...) {
		  }
		}
		returnstream<<"Route through "<<host[0]<<":\n";
		if (pass>0)
		  returnstream<<"\t"<<pass<<" [passed]\n";
		if (destroy>0)
		  returnstream<<"\t"<<destroy<<" [DESTROYED]\n";
		if (timeout>0)
		  returnstream<<"\t"<<timeout<<" [TIMED OUT]\n";
		if (fail>0)
		  returnstream<<"\t"<<fail<<" [FAILED]\n";
      }
      request.SetResponse(returnstream.str());
	  
    }
    else if (request.GetNthToken(1)=="multi"){
      ostringstream returnstream;
      size_t numhops = atoi(request.GetNthToken(2).c_str());
      size_t iterations = atoi(request.GetNthToken(3).c_str());
      NetData * netobj = netobj->Instance();
      list<string> routelist;
      routelist = netobj->allRoutes(numhops);
      if (iterations==0) iterations++;
      size_t pass, fail, destroy, timeout;
      for (list<string>::iterator i = routelist.begin();
	   i!=routelist.end();++i) {
		pass = fail = destroy = timeout = 0;
		string route;
		route = *i;
		string route_array[numhops];
		UserRequest req;
		req.SetCommand(route);
		for (size_t i = 0;i<numhops;i++) 
		  route_array[i] = req.GetNthToken(i);
		uint16_t aci =0;
		for (size_t j = 0;j<iterations;j++) {
		  try {
			aci = static_cast<uint16_t>(rand()>>16);
			client->AddRoute(static_cast<LayerType>(numhops),
							 route_array,aci, "");
			pass++;
		  }
		  catch (RouteDestroyException &rd) {
			destroy++;
		  }
		  catch (TimeoutException &te) {
			timeout++;
		  }
		  catch (TestModuleException &tme) {
			fail++;
		  }
		  try {
			client->RemoveRoute(aci);
		  }
		  catch (...) {
		  }
		}
		returnstream<<"Route through ";
		for (size_t k = 0;k<numhops;k++){
		  returnstream<<route_array[k];
		  if (k!=(numhops-1)) returnstream<<"->";
		}
		returnstream<<": \n";
		if (pass>0)
		  returnstream<<"\t"<<pass<<" [passed]\n";
		if (destroy>0)
		  returnstream<<"\t"<<destroy<<" [DESTROYED]\n";
		if (timeout>0)
		  returnstream<<"\t"<<timeout<<" [TIMED OUT]\n";
		if (fail>0)
		  returnstream<<"\t"<<fail<<" [FAILED]\n";
      }
      request.SetResponse(returnstream.str());
    }
    else if (request.IsCommandEqual("client print")) {
      request.SetResponse(client->ToString());
    }
	
    else
      request.SetBadCommand(myname);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("ClientModule::HandleRequest()");
    request.SetResponse(tme.what());
  }
}
