#include "Nym.h"
#include "ConfigManager.h"

ConfigManager * cfgptr;
/*
 *
 */
ConfigManager::ConfigManager() {
  err = NULL;
  domain = "test.freedom.net";
  timeout = 10;
  keyversion = 1;
  basedir = "";
  network = "";
  pktversion = VERSION1;
  pphrase = "22222222";
  ppfile = "passphrase.file";
  pifile = "passphrase.info";
  outputfile = "toscreen";
  nymtype = NYMTYPE_STR_PREMIUM;

  struct ifreq ifr;
  memset(&ifr, 0, sizeof(ifr));

  // An evil hack is afoot, this code assumes an eth0 device, and will fail
  // otherwise.
    
  strncpy(ifr.ifr_name, "eth0", sizeof(ifr.ifr_name));
  int fd = socket(AF_INET, SOCK_STREAM, 0);

  if (fd < 0)
    throw TestModuleException(errno, "socket()");
  
  if (ioctl(fd, SIOCGIFADDR, &ifr) != 0)
    throw TestModuleException(errno, "ioctl()");

  if (close(fd) != 0)
    throw TestModuleException(errno, "close()");

  struct sockaddr_in addr;
  memset(&addr, 0, sizeof(addr));
  memcpy(&addr, &(ifr.ifr_addr), sizeof(addr));
  localIP = addr.sin_addr.s_addr;
    
  PARAMLIST =
    "domain\nbasedir\nppfile\npifile\ntimeout\npktversion\nlocalip\noutputfile\n";
  if (!zkErrorNew(&err))
    throw TestModuleException("zkErrorNew() failed");

  // Try to load a default config file from the user's home dir
/*
  string defltConfig;
  defltConfig += getenv( "HOME");
  defltConfig += "/.aip-module-conf";
  (void)ConfigFromFile( defltConfig);
*/
}

uint32_t ConfigManager::GetLocalIP(){
  return localIP;
}

/*
 *
 */
ConfigManager * ConfigManager::Instance() {
  if (cfgptr == 0)
    cfgptr = new ConfigManager();
  return cfgptr;
}
/*
 *
 */
string ConfigManager::ToString() {

  char tempstr[256];
  ostringstream sstrm;
  sstrm<<"Global Configuration\n";

  memset(tempstr, 0, sizeof(tempstr));
  zkErrorMessage(err, tempstr, sizeof(tempstr));
  sstrm<<"ZkError Message:  ";
  sstrm<<tempstr;
  
  sstrm << "\nBase Directory:  " << basedir;

  sstrm << "\nDomain Name:  " << domain;

  sstrm << "\nLog File Name:  " << logfile;

  sstrm << "\nPassphrase File Name:  " << ppfile;

  sstrm << "\nPassinfo File Name:  " << pifile;

  sstrm << "\nNetwork Name:  " << network;

  sstrm << "\nScalper:  " << scalper;

  sstrm << "\nPktVersion: " << static_cast<unsigned int>(pktversion);

  sstrm << "\nOutput File: "<<outputfile;

  sstrm << "\nNym Type: "<<nymtype;

  struct in_addr localaddr;
  localaddr.s_addr = localIP;
  sstrm << "\nLocal IP Address:  " << inet_ntoa(localaddr);
  
  sstrm << "\nTimeout:  " << timeout << "\nKeyVersion:  " << keyversion;

  return sstrm.str();
}
/*
 *
 */
void ConfigManager::LogMessage(string logmsg) {
  cerr << logmsg << "\n";
}
/*
 *
 */
string ConfigManager::SetValue(string valname, string value) {
  try {
    if (valname == "domain")
      return (domain = value);
    if (valname == "basedir")
      return (basedir = value);
    if (valname == "ppfile")
      return (ppfile = value);
    if (valname == "pifile")
      return (pifile = value);
    if (valname == "logfile")
      return (logfile = value);
    if (valname == "network")
      return (network = value);
    if (valname == "scalper")
      return (scalper = value);
    if (valname =="timeout") {
      ostringstream returnstream;
      timeout = atoi(value.c_str());
      returnstream<<timeout;
      return(returnstream.str());
    }
    if (valname =="localip") {
      ostringstream returnstream;
      localIP = inet_addr(value.c_str());
      returnstream << localIP;
      return(value.c_str());
    }
    if (valname =="pktversion") {
      ostringstream returnstream;
      unsigned int vers = atoi(value.c_str());
      if ((vers!=1)&&(vers!=2))
	throw TestModuleException("Bad version, must be 1 or 2");
      pktversion = static_cast<VersionType>(vers);
      returnstream<<vers;
      return (returnstream.str());
    }
    if (valname == "output") {
      /*HACK HACK HACK*/
      /*force the basedir to be properly configured before you allow
	an output file to be set*/
      BaseDir();
      return (outputfile=value);
    }
    if (valname == "nymtype") {
      if (!isValidTypeStr(value))
        throw TestModuleException("Invalid nym type");
      else return (nymtype=value);
    }
    throw TestModuleException("Bad config parameter, no value set");
  }
  catch (TestModuleException &tme) {
    tme.AddScope("ConfigManager::SetValue()");
    throw;
  }
}
/*
 *
 */
bool ConfigManager::ConfigFromFile(string filename) {

	char 			whitespace[] = " \t\v\n";
	const int		maxLineLen = 255;
	ifstream		cfgfile;
	string			sline;
	string			param, val;
	char			line[maxLineLen];
	unsigned int	tokenStart;
	unsigned int	tokenEnd;
	unsigned int	lineNum;
  
    // Open the file
	cfgfile.open(filename.c_str());

	// Does the file exist?
	if ( !cfgfile)
	{
		return false;
	}
  
	// Read the configuration file
	for( lineNum = 0;; lineNum++)
	{
		cfgfile.getline( line, maxLineLen);
		if ( cfgfile.eof())
		{
			// End of file
			break;
		}
		sline = line;

		tokenStart = sline.find_first_not_of( whitespace, 0);
		if ( tokenStart == string::npos || sline[tokenStart] == '#')
		{
			// Either a blank line or a comment
			continue;
		}

		tokenEnd = sline.find_first_of( whitespace, tokenStart);
		param = sline.substr( tokenStart, tokenEnd - tokenStart);

		tokenStart = sline.find_first_not_of( whitespace, tokenEnd);
		if ( tokenStart == string::npos)
		{
			// bad config line
    		throw TestModuleException( "Invalid config line in: " + filename);
			cfgfile.close();
			return false;
		}
		tokenEnd = sline.find_first_of( whitespace, tokenStart);
		val = sline.substr( tokenStart, tokenEnd - tokenStart);

		// Set the value
	    SetValue( param, val);
	}
	cfgfile.close();

	return true;
}

/*
 *
 */
void ConfigManager::ConfigToFile(string filename) {

  ofstream cfgfile;
  
  cfgfile.open(filename.c_str());
  
  if (cfgfile.good()) {
    cfgfile << "domain " << domain << endl;
    cfgfile << "basedir " << basedir << endl;
    cfgfile << "ppfile " << ppfile << endl;
    cfgfile << "pifile " << pifile << endl;
    cfgfile << "logfile " << logfile << endl;
    cfgfile << "timeout " << timeout << endl;
  }
  else
    throw TestModuleException("Unable to save to file: " + filename);
  cfgfile.close();
}

string ConfigManager::BaseDir() const {
  if (basedir=="")
    throw BaseDirNotSpecified("Base directory must be specified before proceeding, try 'config set basedir <dir>'");

  if (Network()=="")
    return basedir;
  else
    return basedir + "/" + Network();
}

string ConfigManager::PassphraseFile() const {
  return this->BaseDir() + "/" + ppfile;
}

string ConfigManager::PassinfoFile() const {
  return this->BaseDir() + "/" + pifile;
}

void ConfigManager::SetNetwork(string net) {
  network = net;
}

void ConfigManager::SetScalper(string scalp) {
  scalper = scalp;
}

string ConfigManager::OutputFile() {
  return outputfile;
}
string ConfigManager::NymType() {
  return nymtype;
}


/**
 * Returns a PASS/FAIL string depending on the value of val
 */

string ok(bool val)
{ 
  return ((val ? TEST_PASS_STR : TEST_FAIL_STR) + "\n");
}



/**
 * Generates a random string useable as a nymname
 */

string GenerateRandomString() {

  char legalchars[] = "-_0123456789abcdefghijklmnopqrstuvwxyz";
  const float NUM_LEGAL_CHARS = 38.0;
  const float MAX_STR_SZ = 16.0;
  const int MIN_STR_SZ = 6;

  size_t strsz;
  string randstr;
  unsigned int seed = (unsigned int) time(NULL);
  srand(seed);

  strsz = MIN_STR_SZ + (size_t)((MAX_STR_SZ-MIN_STR_SZ)*rand()/(RAND_MAX+1.0));
  for (size_t i = 0; i < strsz; ++i) {
    int index = (int) ((NUM_LEGAL_CHARS)*rand()/(RAND_MAX+1.0));
    randstr += legalchars[index];
  }
  return randstr;
}
