#ifndef CONFIG_PZEA_DEF
#define CONFIG_PZEA_DEF

#include "TestModuleException.h"
#include <iostream>
#include <sstream>
#include <fstream>

#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>

#include "frentids.h"

typedef enum {
  VERSION1 = 1,
  VERSION2 = 2
} VersionType;

namespace {

  // Maximum Nym Name Size
  size_t MAX_NYM_NAME_SZ = FRENT_NAME_SIZE;

  // Test result strings
  string TEST_PASS_STR = string("[  OK  ]");
  string TEST_FAIL_STR = string("[FAILED]");

}


string ok(bool);
string GenerateRandomString();


class ConfigManager {
public:
  ConfigManager(ConfigManager &) {throw BadOperatorException("ConfigManager copy");}
  ~ConfigManager() { zkErrorFree(&err); }
  static ConfigManager * Instance();
  string ToString();
  string Domain() const { return domain; }
  void SetDomain(string domain_in) { domain = domain_in; }
  string BaseDir() const;
  string Passphrase() const { return pphrase; }
  string PassphraseFile() const;
  string PassinfoFile() const;
  void SetPassinfoFile(string pifile_in) { pifile = pifile_in; }
  string Network() const { return network; }
  void SetNetwork(string);
  string scalperUrl() const { return scalper; }
  void SetScalper(string);
  unsigned int Timeout() const { return timeout; }
  ZkError Error() const;
  unsigned int KeyVersion() const;
  void LogMessage(string);
  bool ConfigFromFile(string);
  void ConfigToFile(string);
  string ParamList() const { return PARAMLIST; }
  string SetValue(string, string);
  string NymType();
  VersionType PktVersion() const { return pktversion; }
  void SetPktVersion(VersionType type_in)
    { pktversion = type_in; }
  string OutputFile();
  unsigned int LocalIPAddress() const { return localIP; }
 private:
  uint32_t GetLocalIP();
  ConfigManager();
  ConfigManager& operator=(const ConfigManager&) {throw BadOperatorException("ConfigManager =");}
  void operator&() {throw BadOperatorException("ConfigManager &");}
  ZkError err;
  string PARAMLIST, basedir, domain, logfile, ppfile, pifile, network, scalper,outputfile, nymtype, pphrase;
  unsigned int timeout, keyversion, localIP;
  VersionType pktversion;
  
};
extern ConfigManager * cfgptr;

class BaseDirNotSpecified : public TestModuleException {
public:
  BaseDirNotSpecified(string msg_in) {SetMessage(msg_in);}
  ~BaseDirNotSpecified() {}
private:
};



#else
//#warning ConfigManager.h redefined.
#endif /*CONFIG_PZEA_DEF*/

