#include "TestModuleSet.h"
/*
 *
 */
ConfigModule::ConfigModule() {

  myname = "config";
  if (reg[myname] != 0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  activated = false;
  helpmessage = "Config Module manages the global variables, aka the configuration.\n  Note that anything that is set here immediately impacts all other modules.\n";
  helpmessage += "config help\nconfig shutdown\nconfig print\n";
  helpmessage += "config load <filename>\n";
  helpmessage += "config save <filename>\n";
  helpmessage += "config set <parameter> <value>\n";
  helpmessage += "config paramlist:  Lists configurable parameters\n";
}
/*
 *  Calls cleanup incase it was not called yet, then explicitly deregisters
 *  the module.
 */
ConfigModule::~ConfigModule() {
  Deactivate();
  reg.erase(reg.find(myname));
}
/*
 *  Allocates the memory for the NetData object.
 */
void ConfigModule::Activate() {
  if (!activated) {
    cfghnd = cfghnd->Instance();
    activated = true;
  }
}
/*
 *
 */
void ConfigModule::Deactivate() {

  if (activated) {
    activated = false;
  }
}
/*
 *  config load <filename>
 */
void ConfigModule::HandleRequest(UserRequest & request) {

  try {
    Activate();
    if (request.IsCommandEqual("config help"))
      request.SetResponse(helpmessage);
    else if (request.IsCommandEqual("config shutdown")) {
      Deactivate();
      request.SetResponse("Currently a no-op");
    }
    else if (request.IsCommandEqual("config print")) {
      request.SetResponse(cfghnd->ToString());
    }
    else if (request.GetNthToken(1) == "load") {
      if ( cfghnd->ConfigFromFile(request.GetNthToken(2)))
     	request.SetResponse("Config file loaded");
	  else
     	request.SetResponse("Failed to load Config file: " + request.GetNthToken(2));
    }
    else if (request.GetNthToken(1) == "save") {
      cfghnd->ConfigToFile(request.GetNthToken(2));
      request.SetResponse("save file appears to work");
    }
    else if (request.GetNthToken(1) == "paramlist") {
      request.SetResponse(cfghnd->ParamList());
    }
    else if (request.GetNthToken(1) == "set") {
      request.SetResponse(cfghnd->SetValue(request.GetNthToken(2),
                                           request.GetNthToken(3)) +
                          " is the new value");
    }
    else
      request.SetBadCommand(myname);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("ConfigModule::HandleRequest()");
    request.SetResponse(tme.what());
  }
}
