#include "TestModuleSet.h"

DHModule::DHModule(){
  myname = "dh";
  if (reg[myname] !=0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  activated = false;
  helpmessage = "This module handles diffie-helman exchanges.\n";
  helpmessage += "dh help\ndh shutdown\n";
  helpmessage += "dh exchange <aip name>\n";
}

DHModule::~DHModule(){
  Deactivate();
  reg.erase(reg.find(myname));
}

void DHModule::Activate(){
  if (!activated) {
    activated = true;
  }
}

void DHModule::Deactivate(){
  if (activated) {
    activated = false;
  }
}

void DHModule::HandleRequest(UserRequest & request) {
  try {
    Activate();
    if (request.IsCommandEqual("dh help"))
      request.SetResponse(helpmessage);
    else if (request.IsCommandEqual("dh shutdown")) {
      Deactivate();
      request.SetResponse(myname + " module shutdown");
    } 
    else if (request.IsCommandEqual("dh test")) {
      DHNegotiator dh;
      ostringstream testreport;
      testreport << "Backwards compatibility test:  ";
      if (dh.CompatTest(1))
        testreport << "[PASSED]" << endl;
      else
        testreport << "[FAILED]" << endl;
      request.SetResponse(testreport.str());
    } 
    else if (request.GetNthToken(1)=="exchange") {
      struct sockaddr_in addr;
      memset(&addr,0,sizeof(addr));
      
      string aip;
      FrEntIdExp ent;
	
      aip = request.GetNthToken(2);
      NetData* netobj = netobj->Instance();
      if (!netobj->GetAIP(aip,&addr,&ent))
	throw TestModuleException("No info for aip " + aip);

      LinkManager * lm = lm->Instance();
      lm->AddLink(addr);
      lm->RemoveLink(addr);
      request.SetResponse("successful dh exchange to "+aip);
    }
    else
      request.SetBadCommand(myname);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("DHModule::HandleRequest()");
    request.SetResponse(tme.what());
  }
}
