/*  Maintainer(s):Alice Nodelman and Troy McLelan
 *  Description:  This object handles the Diffie-Hellman Negotiation between
 *                2 parties.  It can perform client or AIP side, send or
 *                receive.  Currently only supports Version 1.
 *  Todo:         - Add functionality to the ToString() call.
 *                - Make the socket calls more robust.
 *                - Add const where appropriate.
 *                - Improve exception handling.
 *                - Eliminate resource leaks on failure.
 *                - Add a system test to do bad DH Exchanges.
 *  Known Bugs:   None.
 */

#ifndef ZK_AIP_DHNEG_JJDFKSZK_DEF
#define ZK_AIP_DHNEG_JJDFKSZK_DEF

#include "KeyHandler.h"
#include <sys/poll.h>
#include <sys/wait.h>

class DHNegotiator {
public:

  DHNegotiator();
  ~DHNegotiator();

  string ToString();
  string SelfTest(size_t numloops = 1);
  string SystemTest(string & active, size_t numloops = 1);
  string PerfTest(string & active, size_t numloops = 1);
  bool CompatTest(size_t numloops = 1);
  string BadDHTest(size_t numloops = 1);

  FrCryptSym AnonDH(struct sockaddr_in aipaddr, uint8_t & initlekv);
  FrCryptSym SignedDH(struct sockaddr_in aipaddr, FrCryptSig myPrivSig,
                      uint8_t & initlekv);
  FrCryptSym AcceptDH(FrCryptSig myPrivSig, uint8_t & initlekv,
                      struct sockaddr_in remoteaddr, int listensock);
private:

  DHNegotiator(DHNegotiator &);
  DHNegotiator& operator=(const DHNegotiator&);
  void operator&();

  void SetupSocket(struct sockaddr_in);
  void SendMyHalfSecret(uint8_t, FrCryptSig myPrivSig = 0);
  void ReceiveTheirHalfSecret(uint8_t &, bool);
  FrCryptSym BuildKey();

  void SpoofAIPTest(size_t numloops = 1);
  void AuthenticateNymTest(size_t numloops = 1);
  void FakeAIPTest(size_t numloops = 1);
  void HangUpTest(size_t numloops = 1);
  void MalformedPacketTest(size_t numloops = 1);
  void ClientToServerPortTest(size_t numloops = 1);
  void ServerToClientPortTest(size_t numloops = 1);

  CryptoWrapper * crypto;
  int dh_fd;
  ZkError err;
  BIGNUM * firsthalf;
  BIGNUM * secondhalf;
  unsigned char secBuf[BITS_TO_BYTES(2048)];
};
#endif
