/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   This object handles all byte level manipulations for Freedom
 *                 Packets.  It is inspired by the sk_buffs in the Linux
 *                 kernel.
 *  Todo:          - Enforce function call dependancies.
 *                 - Integrate with PacketBuffer.
 *                 - Apply an inheritance hierarchy.
 *                 - Nicer pointer manipulation.
 *                 - Eliminate resource leaks on failure.
 *                 - Add thread support, if necessary.
 *                 - Remove pragma pack().
 *  Known Bugs:    None.
 */

#ifndef ZK_AIP_FRACKPKT_JJDFKSZK_DEF
#define ZK_AIP_FRACKPKT_JJDFKSZK_DEF

#include "FreedomDataPacket.h"

class FreedomAckPacket : public FreedomDataPacket {

 public:
  
  FreedomAckPacket() {}  
  FreedomAckPacket(FreedomPacket & frpkt) : FreedomDataPacket(frpkt) {}
  ~FreedomAckPacket() {}

  uint16_t GetExtendAci() const {

    uint16_t extend_aci;
    memcpy(&extend_aci, begin + EXTEND_ACI_OFFSET, sizeof(uint16_t));
    return ntohs(extend_aci);
  }

 private:

  // To prevent inadvertant copying.  Although we could support them, they
  // could be a hidden performance drain.
  FreedomAckPacket& operator=(const FreedomAckPacket&);
};

#endif
