/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   This object handles all byte level manipulations for Freedom
 *                 Packets.  It is inspired by the sk_buffs in the Linux
 *                 kernel.
 *  Todo:          - Enforce function call dependancies.
 *                 - Integrate with PacketBuffer.
 *                 - Apply an inheritance hierarchy.
 *                 - Nicer pointer manipulation.
 *                 - Eliminate resource leaks on failure.
 *                 - Add thread support, if necessary.
 *                 - Remove pragma pack().
 *  Known Bugs:    None.
 */

#ifndef ZK_AIP_FRACKPKT_JJDFKSZK_DEF
#define ZK_AIP_FRACKPKT_JJDFKSZK_DEF

#include "FreedomPacket.h"

/*
 *  Handles all the byte level manipulation that is performed on Freedom
 *  packets.
 */

class FreedomAlivePacket {
public:
  FreedomAlivePacket();
  ~FreedomAlivePacket() {}

  string ToString() const;
  string SelfTest(size_t numloops = 1);

 protected:

  // To support testing in classes that use this class.  Not to be used
  // normally

  void SetPayload(char * buffer, size_t buffersize);
  char * GetPayload(size_t & pktsz) const
    {pktsz = end - begin; return begin;}

  // Packet type specific functions, these should be soemwhere else.
  uint16_t GetACI() const;
  unsigned short GetPort() const;
  uint32_t GetSrcIP() const;
  void GetAddr(struct sockaddr_in & addr_out) const
    { memcpy(&addr_out, &pktaddr, sizeof(addr_out));}
  void SetAddr(struct sockaddr_in addr_in) 
  {memcpy(&pktaddr,&addr_in,sizeof(pktaddr));}

 private:

  // To prevent inadvertant copying.  Although we could support them, they
  // could be a hidden performance drain.
  
  FreedomAlivePacket(FreedomAlivePacket &);
  FreedomAlivePacket& operator=(const FreedomAlivePacket&);
  void operator&();

  struct sockaddr_in pktaddr;
};

#endif
