#include "FreedomCreatePacket.h"
  
// This structure is not currently used, it is just here for info.
  
/*
struct freedom_route_create_packet_outline {
  struct freedom_header_mapping frhdr;
  struct freedom_route_create_layer * rclyr;  // Up to 5 layers.
  struct freedom_nym_authentication nauth;    // Undefined for unauthenticated
  char * padding;                      // random filler for unused route layers
  struct freedom_trailer_mapping frtrl;
};
*/

/*
 *  Limit local variables to this file with an unnamed namespace.
 */

namespace {
  //  The structure of a layer in the freedom route creation packet.
#pragma pack(1)
  struct freedom_route_create_layer {
    char keySeed[FRCRYPT_SECRET_SIZE];// Key seed based on the aip's public key
    uint8_t keyAlg;                     // The algorithm used, blowfish.
    uint8_t cryptKeyVers;     // The version of the aip's key used for the seed
    char pad0[6];               // Padding set to a known value, for validation
    
    //  Layered symmetric encryption begins here.  This offset (within the
    //  structure) should be a mutliple of eight bytes.
    
    uint8_t bckKeyAlg;          // The algorithm used, blowfish.
    uint8_t fwdKeyAlg;          // The algorithm used, blowfish.
    uint8_t flags;              // Specifies route extension and last hop.
    char hostEnt[FRENT_PORTABLE_SIZE]; // The next AIP in the route
    uint16_t specAci;           // aci used to extend the route.
    uint32_t expTime;           // route expiry time.
    char pad1[6];               // More padding, same as above.
  };
  
  //  The nym authentication information used for authenticated route creates.
   
  struct freedom_nym_authentication {
    char serialNum[8];          // A random vector to prevent replay attacks
    uint8_t nymType;            // Entity type of the Nym
    char nymName[64];           // Name of the nym
    char nymSig[FRCRYPT_SIG_SIZE];  // Signature of the packet hash stored here
    char pad0[6];               // Padding, as in route create layer
  };

#pragma pack()

  typedef struct freedom_route_create_layer* frc_ptr;
  typedef struct freedom_nym_authentication* fna_ptr;

  const size_t RCL_STRUCT_SZ = sizeof(struct freedom_route_create_layer);
  const size_t NA_STRUCT_SZ = sizeof(struct freedom_nym_authentication);

  const size_t ROUTE_CRYPT_OFFSET = FRCRYPT_SECRET_SIZE + 8;
  const size_t NYM_HASH_SZ = RCL_STRUCT_SZ + 73;
  const size_t HASH_OFFSET = RCL_STRUCT_SZ - FRCRYPT_SECRET_SIZE;

  //  These values are used for packet padding, and they are always the same.
  const unsigned char SIXES[] = {0x06, 0x06, 0x06, 0x06, 0x06, 0x06};
};

/*
 *  Sets the proper buffer in the route create packet with the provided values.
 */

void FreedomCreatePacket::SetRCFields(LayerType layer_type, struct fr_rc_data rcdata) {

  try {
    // The first layer initializes the end pointer.

    if (layer_type == LAYER1) {
      end = begin + FR_HEADER_SZ + MAX_RC_LAYERS * RCL_STRUCT_SZ +
        NA_STRUCT_SZ + FR_TRAILER_SZ;
      if (end > tail)
        throw BadPointerException("end > tail", head, tail, begin, end);
    }

    unsigned int layer_index = static_cast<unsigned int>(layer_type) - 1;
    
    // For ease of use, create a temporary pointer to the start of the buffer.

    frc_ptr rtePtr = reinterpret_cast<frc_ptr>(begin + FR_HEADER_SZ +
                                               layer_index * RCL_STRUCT_SZ);

    if (begin + FR_HEADER_SZ + layer_index * RCL_STRUCT_SZ > end)
      throw BadPointerException("Pointing beyond end", head, tail, begin, end);

    // Copy the keyseed, this will be used to build matching symmetric keys.

    memcpy(rtePtr->keySeed, rcdata.kseed, sizeof(rtePtr->keySeed));

    // Symmetric algorithm, always Blowfish.

    rtePtr->keyAlg = FRCRYPT_SYMALG_BLOWFISH;

    // The version of the AIP's crypto key used to generate the key seed.

    rtePtr->cryptKeyVers = rcdata.kversion;

    // A predetermined value, used to verify decryption.

    memcpy(rtePtr->pad0, SIXES, sizeof(SIXES));

    // ***Layer encryption starts here***

    // Symmetric algorithm, always Blowfish.

    rtePtr->bckKeyAlg = FRCRYPT_SYMALG_BLOWFISH;
    rtePtr->fwdKeyAlg = FRCRYPT_SYMALG_BLOWFISH;

    // Flags for special functions, route extension, or last hop verification.

    rtePtr->flags = rcdata.flags;  // Should we verify this?

    // The entity of the next AIP, or on the last hop, the nym entity.

    rtePtr->hostEnt[0] = rcdata.hostEnt.type;
    strncpy(rtePtr->hostEnt + 1, rcdata.hostEnt.name,
           sizeof(rtePtr->hostEnt) - 2);

    // For route extension, the aci to use.  Otherwise, a predetermined value.

    rtePtr->specAci = htons(rcdata.extaci);

    // The route expiry time, assumed to be normalized to freedom time.

    rtePtr->expTime = htonl(rcdata.exptime);

    // A predetermined value, used to verify decryption.

    memcpy(rtePtr->pad1, SIXES, sizeof(SIXES));
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SetRCFields()");
    throw;
  }
}

/*
 *  Used to extract route create parameters from a route create packet.  Moves
 *  up the rest of the packet when it is done.  Should verify pointers.
 */

void FreedomCreatePacket::GetRCFields(struct fr_rc_data & rcdata) const {

  try {

    frc_ptr rtePtr = reinterpret_cast<frc_ptr>(begin + FR_HEADER_SZ);

    // Can this be added to the entropy pool?

    memcpy(rcdata.kseed, rtePtr->keySeed, FRCRYPT_SECRET_SIZE);

    rcdata.kversion = rtePtr->cryptKeyVers;

    if (memcmp(rtePtr->pad0, SIXES, sizeof(SIXES)))
      throw BadPacketException("Bad sixes padding in pad0", begin,
                               end - begin);

    rcdata.flags = rtePtr->flags;

    rcdata.hostEnt.type = static_cast<FrEntIdType>(rtePtr->hostEnt[0]);
    strncpy(rcdata.hostEnt.name, rtePtr->hostEnt + 1,
           sizeof(rtePtr->hostEnt) - 2);
    
    rcdata.extaci = ntohs(rtePtr->specAci);
    rcdata.exptime = ntohl(rtePtr->expTime);

    if (memcmp(rtePtr->pad1, SIXES, sizeof(SIXES)))
      throw BadPacketException("Bad sixes padding in pad1", begin,
                               end - begin);

    // For all hops move the rest of the packet up and
    // scribble random data in the remaining space.

    if (!(rcdata.flags & LAST_HOP_FLAG)) {

      // We have to use memmove() here because the buffers overlap.
      memmove(begin + FR_HEADER_SZ, begin + FR_HEADER_SZ + RCL_STRUCT_SZ,
              (MAX_RC_LAYERS - 1) * RCL_STRUCT_SZ + NA_STRUCT_SZ);

      crypto->FillBufferWithRandomData(begin + FR_HEADER_SZ +
                                       (MAX_RC_LAYERS - 1) * RCL_STRUCT_SZ +
                                       NA_STRUCT_SZ, RCL_STRUCT_SZ);
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetRCFields()");
    throw;
  }
}

void FreedomCreatePacket::GetRCDecryptData(struct fr_rc_data & rcdata) const {

  try {

    frc_ptr rtePtr = reinterpret_cast<frc_ptr>(begin + FR_HEADER_SZ);

    // Can this be added to the entropy pool?

    memcpy(rcdata.kseed, rtePtr->keySeed, FRCRYPT_SECRET_SIZE);

    rcdata.kversion = rtePtr->cryptKeyVers;

    if (memcmp(rtePtr->pad0, SIXES, sizeof(SIXES)))
      throw BadPacketException("Bad sixes padding in pad0", begin,
                               end - begin);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetRCDecryptData()");
    throw;
  }
}

/*
 *  This function inserts Nym data into the route create packet, determines
 *  the hash of the packet, and signs the hash with the Nym's signature key.
 *  WARNING:  The HASH values have not been verified and should not be trusted.
 */

void FreedomCreatePacket::SetNymFields(LayerType layer_type, FrEntIdExp nyment,
                                       FrCryptSig nymPrivSig,
                                       unsigned char* ksm, size_t ksmsz) {

  try {

    if (ksm == 0)
      throw BadParameterException("ksm pointer is NULL");

    unsigned int layer_index = static_cast<unsigned int>(layer_type);
    
    fna_ptr nymPtr = reinterpret_cast<fna_ptr>(begin + FR_HEADER_SZ +
                                               layer_index * RCL_STRUCT_SZ);

    if (begin + FR_HEADER_SZ + layer_index * RCL_STRUCT_SZ > end)
      throw BadPointerException("Bad pointers", head, tail, begin, end);

    // Set the serial number to a random vector.  This must not be repeated
    // when the route is extended, so it should always be random.

    crypto->FillBufferWithRandomData(nymPtr->serialNum,
                                     sizeof(nymPtr->serialNum));

    nymPtr->nymType = nyment.type;

    memcpy(nymPtr->nymName, nyment.name, sizeof(nymPtr->nymName));

    memcpy(nymPtr->pad0, SIXES, sizeof(SIXES));

    // Hash over the key seed, the route data and the nym data.  Then sign the
    // hash.  That should prevent forgeries.  Question:  can we just do one
    // update, since all the hash components are contiguous?

    unsigned char hash_buf[NYM_HASH_SZ];
    memcpy(hash_buf, ksm, ksmsz);
    memcpy(hash_buf + ksmsz,
           begin + FR_HEADER_SZ + (layer_index - 1) * RCL_STRUCT_SZ +
           FRCRYPT_SECRET_SIZE, NYM_HASH_SZ - ksmsz);
    unsigned char * hash = crypto->HashBuffer(hash_buf, NYM_HASH_SZ);
    crypto->SignHash(nymPrivSig, hash, 20, nymPtr->nymSig,
                     sizeof(nymPtr->nymSig));
    delete [] hash;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SetNymFields()");
    throw;
  }
}
/*
 *  Extracts the nym authentication data from the route create buffer.  Does
 *  not authenticate the nym, that is left for the caller to do.
 */

/* assumes that the nym fields are located afte the freedom header and
   one route create structure.  thus, it should be called *after* you
   have stripped the route create structures using getRCFields, then
   the pointers will be in the correct location.  An example of use is
   found in the RouteHandler selftest.*/

void FreedomCreatePacket::GetNymFields(FrEntIdExp * nyment,
                                       unsigned char * ksm,
                                       size_t ksmsz, char * hashsig,
                                       unsigned char ** hash) const {

  try {
    if ((nyment == 0) || (ksm == 0) || (hashsig == 0) || (hash == 0))
      throw BadParameterException("Pointer is NULL");

    fna_ptr nymPtr = reinterpret_cast<fna_ptr>(begin + FR_HEADER_SZ +
                                               RCL_STRUCT_SZ);

    static_cast<uint8_t>(nyment->type) = nymPtr->nymType;

    memcpy(nyment->name, nymPtr->nymName, sizeof(nymPtr->nymName));

    unsigned char hash_buf[NYM_HASH_SZ];
    memcpy(hash_buf, ksm, ksmsz);
    memcpy(hash_buf + ksmsz, begin + FR_HEADER_SZ + FRCRYPT_SECRET_SIZE,
           NYM_HASH_SZ - ksmsz);
    *hash = crypto->HashBuffer(hash_buf, NYM_HASH_SZ);

    memcpy(hashsig, nymPtr->nymSig, sizeof(nymPtr->nymSig));
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetNymFields()");
    throw;
  }
}

/*
 *  Adds all n layers of encryption to the route create packet.
 */

void FreedomCreatePacket::EncryptRCPacket(LayerType layer_type,
                                          FrCryptSym* rteCrypt,
                                          unsigned char* ivec) {

  try {

    if (rteCrypt == 0)
      throw BadParameterException("rteCrypt pointer is NULL");

    if (ivec == 0)
      throw BadParameterException("ivec pointer is NULL");

    unsigned int num_layers = static_cast<unsigned int>(layer_type);
    
    // The cryptptr points to the start of the encryption layer.  At first, it
    // should point to the end of the route create data, so we can add random
    // data, or memset it to zero.

    char * cryptptr =
      (begin + FR_HEADER_SZ + num_layers * RCL_STRUCT_SZ + NA_STRUCT_SZ);

    // Points one past the last byte to be encrypted.

    char * cryptend = end - FR_TRAILER_SZ;

    if (cryptptr > cryptend)
      throw BadPointerException("Bad pointers", head, tail, begin, end);

    // Before encrypting, we set this pad, I don't know what it does.

    fr_CryptSetPad(begin + 11, 3);

    // Move the cryptend pointer to 1 beyond the end of the current data.
    // This is a performance optimization, don't encrypt anymore than we must. 

    cryptend = cryptptr;

    // Move the cryptptr to the start of the nym authentication field, then
    // move it forward by the route layer encryption offset.

    cryptptr -= NA_STRUCT_SZ;
    cryptptr += ROUTE_CRYPT_OFFSET;

    // For each layer, move the cryptptr back the size of a route layer.
    // Since it is already offset properly, this should be the right place to
    // start encryption.  Start with the last hop's key.

    for (int i = num_layers - 1; i >= 0; i--) {
      cryptptr -= RCL_STRUCT_SZ;
      crypto->SymEncrypt(rteCrypt[i], cryptptr, cryptend - cryptptr,
                         ivec + i * FRCRYPT_IVEC_SIZE);
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("EncryptRCPacket()");
    throw;
  }
}

/*
 *  Decrypt one layer of the freedom route create packet.  We assume that the
 *  packet has been shifted forward, so we decrypting right from the front.
 */

void FreedomCreatePacket::DecryptRCPacketLayer(FrCryptSym rteCrypt,
                                               unsigned char* ivec) {
  try {
    if (ivec == 0)
      throw BadParameterException("ivec pointer is NULL");

    char * cryptptr = begin + FR_HEADER_SZ + ROUTE_CRYPT_OFFSET;

    // Okay, now here's a funny question...how do we know what to unencrypt?
    // Is the encryption stream or block based?  If it is block based, would
    // the random data later on not cause pain???  

    char * cryptend = end - FR_TRAILER_SZ;

    crypto->SymDecrypt(rteCrypt, cryptptr, cryptend - cryptptr, ivec);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("DecryptRCPacketLayer()");
    throw;
  }
}

/*
 *
 */

string TestFreedomCreatePacket::UnitTest(size_t numloops) {

  try {
    
    string testresults = "Freedom Route Create Packet Test:\n";
    srand(time(0));
    testresults += "Route Create Packet Test";
    testresults += (RCPacketTest(numloops) ? 
		    "..........[passed]\n" : 
		    "..........[FAILED]\n"); 
    testresults += "Nym Authentication Test";
    testresults += (NymAuthTest(numloops) ? 
		    "..........[passed]\n" : 
		    "..........[FAILED]\n"); 
    testresults += "Multiple Cryption Test";
    testresults += (MultipleCryptionTest(numloops) ?
		    "..........[passed]\n" : 
		    "..........[FAILED]\n"); 
    return testresults;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SelfTest()");
    throw;
  }
}

bool TestFreedomCreatePacket::RCPacketTest(size_t numloops) {

  try {
    bool testresult = true;
    FreedomCreatePacket rcpkt;
    CryptoWrapper * crypto = crypto->Instance();
    for (size_t i = 0; ((i < numloops) && (testresult)); ++i) {
      struct fr_rc_data predata[MAX_RC_LAYERS], postdata[MAX_RC_LAYERS];
      // I don't *want* to do this, but I have to because of holes in the
      // structure
      
      memset(&predata[0], 0, sizeof(struct fr_rc_data));
      memset(&postdata[0], 0, sizeof(struct fr_rc_data));
      memset(&predata[1], 0, sizeof(struct fr_rc_data));
      memset(&postdata[1], 0, sizeof(struct fr_rc_data));
      memset(&predata[2], 0, sizeof(struct fr_rc_data));
      memset(&postdata[2], 0, sizeof(struct fr_rc_data));
      memset(&predata[3], 0, sizeof(struct fr_rc_data));
      memset(&postdata[3], 0, sizeof(struct fr_rc_data));
      
      FrCryptSym rteCrypt[MAX_RC_LAYERS];
      unsigned char ivec[MAX_RC_LAYERS][FRCRYPT_IVEC_SIZE];
      
      size_t num_layers = ((rand() >> 30) % 4) + 1;
      for (size_t i = 0; i < num_layers; i++) {
        rteCrypt[i] = crypto->GenerateSymkey();
        crypto->FillBufferWithRandomData(ivec[i], FRCRYPT_IVEC_SIZE);
        
        if (i == num_layers)
          predata[i].flags = LAST_HOP_FLAG;
        else
          predata[i].flags = 0;
        
        crypto->FillBufferWithRandomData(reinterpret_cast<char*>(&(predata[i].hostEnt)), FRENT_PORTABLE_SIZE);

        predata[i].exptime = static_cast<time_t>(rand());
        predata[i].extaci = static_cast<uint16_t>(rand() >> 16);
        predata[i].kversion = static_cast<uint8_t>(rand() >> 24);
        predata[i].hostEnt.type = FRENT_TYPE_AIP;
        string name = "an_aip";
        strncpy(predata[i].hostEnt.name, name.c_str(),
                sizeof(predata[i].hostEnt.name));
        crypto->FillBufferWithRandomData(predata[i].kseed,
                                         sizeof(predata[i].kseed));
        rcpkt.SetRCFields(static_cast<LayerType>(i + 1), predata[i]);
      }
      
      struct fr_hdr_data hdr;
      hdr.version = VERSION2;
      hdr.type = FRPKTTYPE_AIP_ROUTE_CREATE;
      hdr.aci = static_cast<uint16_t>(rand() >> 16);
      rcpkt.SetHeader(hdr);
      rcpkt.EncryptRCPacket(static_cast<LayerType>(num_layers),
                            static_cast<fr_crypt_sym**>(rteCrypt),
                            reinterpret_cast<unsigned char*>(ivec));
      // Start with the first hop's key.
      for (int j = 0; j < static_cast<int>(num_layers); j++) {
        rcpkt.DecryptRCPacketLayer(rteCrypt[j], ivec[j]);
        rcpkt.GetRCFields(postdata[j]);
        crypto->SymkeyFree(rteCrypt[j]);
      }
      
      for (size_t k = 0; k < num_layers; k++) {    
        if (memcmp(&predata[k], &postdata[k],
                   sizeof(struct fr_rc_data)) != 0) {
          cerr << "layer " << k << " of the packet does not match" << endl;
          if (predata[k].hostEnt.type!=postdata[k].hostEnt.type) 
            cerr<<"hostEnt types are different\n";
          string prename = predata[k].hostEnt.name;
          string postname = postdata[k].hostEnt.name;
          if (prename!=postname) 
            cerr<<"hostEnt names are different "<<prename<<" "<<postname<<"\n";
          if (predata[k].flags != postdata[k].flags)
            cerr << "flags are different pre: " << (int)predata[k].flags
                 << " post: " << (int)postdata[k].flags << endl;
          if (predata[k].exptime != postdata[k].exptime)
            cerr << "exptimes are different" << endl;
          if (predata[k].extaci != postdata[k].extaci)
            cerr << "extacis are different" << endl;
          if (predata[k].kversion != postdata[k].kversion)
            cerr << "kversions are different" << endl;
          if (memcmp(predata[k].kseed, postdata[k].kseed,
                     sizeof(predata[k].kseed)) != 0)
            cerr << "kseeds are different" << endl;
          testresult = false;
        }
      }
    }
    return testresult;
  }
  catch (TestModuleException & tme) {
    tme.AddScope("RCPacketTest()");
    cerr << tme.what();
    return false;
  }
}

/*
 *
 */

bool TestFreedomCreatePacket::NymAuthTest(size_t numloops = 1) {

  try {
    bool testresult = true;
    FreedomCreatePacket rcpkt;
    CryptoWrapper * crypto = crypto->Instance();
    for (size_t i = 0; ((i < numloops) && (testresult)); ++i) {      
      FrEntIdExp preEntity, postEntity;
      struct fr_rc_data rcdata;
      memset(&rcdata, 0, sizeof(rcdata));
      memset(&preEntity, 0, sizeof(preEntity));
      memset(&postEntity, 0, sizeof(postEntity));
      preEntity.type = FRENT_TYPE_NYM;
      strcpy(preEntity.name, "bubba");
      FrCryptSig nymPrivSig = crypto->GeneratePubkey(preEntity,
                                                     FRCRYPT_KEYTYPE_SIGNING);

      unsigned char preksm[FRCRYPT_SECRET_SIZE];
      unsigned char postksm[FRCRYPT_SECRET_SIZE];
      unsigned char * hash;
      char hashsig[FRCRYPT_SIG_SIZE];

      crypto->FillBufferWithRandomData(preksm, FRCRYPT_SECRET_SIZE);
      memcpy(postksm, preksm, FRCRYPT_SECRET_SIZE);
      rcpkt.SetRCFields(LAYER1, rcdata);
      rcpkt.SetNymFields(LAYER1, preEntity, nymPrivSig, preksm,
                         FRCRYPT_SECRET_SIZE);
      rcpkt.GetNymFields(&postEntity, postksm, FRCRYPT_SECRET_SIZE, hashsig,
                         &hash);
      testresult &= !memcmp(&postEntity, &preEntity, sizeof(postEntity));
      testresult &= crypto->VerifyHash(nymPrivSig, hash, 20, hashsig,
                                       sizeof(hashsig));
      crypto->PubkeyFree(nymPrivSig);
      delete [] hash;
    }
    return testresult;
  }
  catch (TestModuleException & tme) {
    tme.AddScope("NymAuthTest()");
    cerr << tme.what();
    return false;
  }
}

/*
 * Similar to the RCPacket test, but this link encrypts as well
 */

bool TestFreedomCreatePacket::MultipleCryptionTest(size_t numloops) {
  try {
    bool testresult = true;
    FreedomCreatePacket rcpkt;
    CryptoWrapper * crypto = crypto->Instance();
    for (size_t i = 0; ((i < numloops) && (testresult)); ++i) {
      struct fr_rc_data predata[MAX_RC_LAYERS], postdata[MAX_RC_LAYERS];
      // I don't *want* to do this, but I have to because of holes in the
      // structure
      uint8_t lekv = 0;
      memset(&predata[0], 0, sizeof(struct fr_rc_data));
      memset(&postdata[0], 0, sizeof(struct fr_rc_data));
      memset(&predata[1], 0, sizeof(struct fr_rc_data));
      memset(&postdata[1], 0, sizeof(struct fr_rc_data));
      memset(&predata[2], 0, sizeof(struct fr_rc_data));
      memset(&postdata[2], 0, sizeof(struct fr_rc_data));
      memset(&predata[3], 0, sizeof(struct fr_rc_data));
      memset(&postdata[3], 0, sizeof(struct fr_rc_data));
      
      unsigned char ivec[MAX_RC_LAYERS][FRCRYPT_IVEC_SIZE];
      FrCryptSym rteCrypt[MAX_RC_LAYERS];
      size_t num_layers = ((rand() >> 30) % 4) + 1;
      for (size_t i = 0; i < num_layers; i++) {
        rteCrypt[i] = crypto->GenerateSymkey();
        crypto->FillBufferWithRandomData(ivec[i], FRCRYPT_IVEC_SIZE);
        
        if (i == num_layers)
          predata[i].flags = LAST_HOP_FLAG;
        else
          predata[i].flags = 0;
        crypto->FillBufferWithRandomData(reinterpret_cast<char*>(&(predata[i].hostEnt)), FRENT_PORTABLE_SIZE);
        
        predata[i].exptime = static_cast<time_t>(rand());
        predata[i].extaci = static_cast<uint16_t>(rand() >> 16);
        predata[i].kversion = static_cast<uint8_t>(rand() >> 24);
        predata[i].hostEnt.type = FRENT_TYPE_AIP;
        string name = "an_aip";
        strncpy(predata[i].hostEnt.name, name.c_str(),
                sizeof(predata[i].hostEnt.name));
        crypto->FillBufferWithRandomData(predata[i].kseed,
                                         sizeof(predata[i].kseed));
        rcpkt.SetRCFields(static_cast<LayerType>(i + 1), predata[i]);
      }
      
      struct fr_hdr_data hdr;
      hdr.version = VERSION2;
      hdr.type = FRPKTTYPE_AIP_ROUTE_CREATE;
      hdr.aci = static_cast<uint16_t>(rand() >> 16);
      rcpkt.SetHeader(hdr);
      rcpkt.EncryptRCPacket(static_cast<LayerType>(num_layers),
                      static_cast<fr_crypt_sym**>(rteCrypt),
                      reinterpret_cast<unsigned char*>(ivec));
      FrCryptSym symkeys[2];
      symkeys[0] = crypto->GenerateSymkey();
      symkeys[1] = crypto->GenerateSymkey();
      rcpkt.LinkEncrypt(symkeys[lekv], static_cast<LekvType>(lekv));
      rcpkt.LinkDecrypt(symkeys);
      // Start with the first hop's key.
      for (int j = 0; j < static_cast<int>(num_layers); j++) {
        rcpkt.DecryptRCPacketLayer(rteCrypt[j], ivec[j]);
        rcpkt.GetRCFields(postdata[j]);
        crypto->SymkeyFree(rteCrypt[j]);
      }
      crypto->SymkeyFree(symkeys[0]);
      crypto->SymkeyFree(symkeys[1]);
      
      for (size_t k = 0; k < num_layers; k++) {    
        
        if (memcmp(&predata[k], &postdata[k],
                   sizeof(struct fr_rc_data)) != 0) {
          
          cerr << "layer " << k << " of the packet does not match" << endl;
          
          if (predata[k].hostEnt.type!=postdata[k].hostEnt.type) 
            cerr<<"hostEnt types are different\n";
          
          string prename = predata[k].hostEnt.name;
          string postname = postdata[k].hostEnt.name;
          
          if (prename!=postname) 
            cerr<<"hostEnt names are different "<<prename<<" "<<postname<<"\n";
          
          if (predata[k].flags != postdata[k].flags)
            cerr << "flags are different pre: "
                 << static_cast<int>(predata[k].flags)
                 << " post: " << static_cast<int>(postdata[k].flags) << endl;
          
          if (predata[k].exptime != postdata[k].exptime)
            cerr << "exptimes are different" << endl;
          
          if (predata[k].extaci != postdata[k].extaci)
            cerr << "extacis are different" << endl;
          
          if (predata[k].kversion != postdata[k].kversion)
            cerr << "kversions are different" << endl;
          
          if (memcmp(predata[k].kseed, postdata[k].kseed,
                     sizeof(predata[k].kseed)) != 0)
            cerr << "kseeds are different" << endl;
          
          testresult = false;
        }
      }
    }
    return testresult;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("MultipleCryptionTest()");
    throw;
  }
}

bool TestFreedomCreatePacket::ErrorHandlingTest(size_t numloops) {

  ostringstream testresults;
  try {
    FrCryptSym symkeys[2];
    CryptoWrapper * crypto = crypto->Instance();
    FreedomCreatePacket rcpkt;
    symkeys[0] = crypto->GenerateSymkey();
    symkeys[1] = crypto->GenerateSymkey();
    for (size_t i = 0; i < numloops; ++i) {

      struct fr_rc_data rcdata;
      
      // Test pointer sanity checking
      char * oldbegin = rcpkt.begin;
      rcpkt.begin = rcpkt.tail;
      try {
        rcpkt.SetRCFields(LAYER1, rcdata);
      }
      catch (BadPointerException & bpe) {
        testresults << bpe.what() << endl;
      }
      rcpkt.begin = oldbegin;

      // Test pointer sanity checking
      oldbegin = rcpkt.begin;
      rcpkt.begin = rcpkt.end;
      try {
        rcpkt.SetRCFields(LAYER2, rcdata);
      }
      catch (BadPointerException & bpe) {
        testresults << bpe.what() << endl;
      }
      rcpkt.begin = oldbegin;

      // Test sixes padding checking
      try {
        rcpkt.GetRCFields(rcdata);
      }
      catch (BadPacketException & bpe) {
        testresults << bpe.what() << endl;
      }
        
      frc_ptr rtePtr = reinterpret_cast<frc_ptr>(rcpkt.begin + FR_HEADER_SZ);
      memcpy(rtePtr->pad0, SIXES, sizeof(SIXES));
      
      // Test sixes padding checking
      try {
        rcpkt.GetRCFields(rcdata);
      }
      catch (BadPacketException & bpe) {
        testresults << bpe.what() << endl;
      }
      
      // Test NULL pointer checking.
      FrEntIdExp tempent;
      memset(&tempent,0,sizeof(tempent));
      FrCryptSig tempsig =0;
      try {
        rcpkt.SetNymFields(LAYER1, tempent, tempsig, 0, 0);
      }
      catch (BadParameterException & bpe) {
        testresults << bpe.what() << endl;
      }

      // Test pointer sanity check.
      oldbegin = rcpkt.begin;
      rcpkt.begin = rcpkt.end;
      unsigned char ksm[FRCRYPT_SECRET_SIZE];
      try {
        rcpkt.SetNymFields(LAYER1, tempent, tempsig, ksm, sizeof(ksm));
      }
      catch (BadPointerException & bpe) {
        testresults << bpe.what() << endl;
      }
      rcpkt.begin = oldbegin;

      FrCryptSym rteCrypt[MAX_RC_LAYERS];
      // Test NULL pointer checking.
      try {
        rcpkt.DecryptRCPacketLayer(rteCrypt[0], 0);
      }
      catch (BadParameterException & bpe) {
        testresults << bpe.what() << endl;
      }
    }
    cerr << testresults;
    return true;
  }
  catch (TestModuleException & tme) {
    cerr << testresults.str();
    tme.AddScope("ErrorHandlingTest()");
    throw;
  }
}

