#include "FreedomDataPacket.h"
  
// This structure is not currently used, it is just here for info.
  
/*
struct freedom_data_packet_outline {
  struct freedom_header_mapping frhdr;
  char randvec[8];    // Another random vector, crypto?  Replay prevention?
  uint16_t payloadsz; // Needed to determine where padding begins.
  char * payload;     // Payload, the meat of the packet.
  char * padding;     // Padding to satisfy crypto requirements.
  char macdata[10];   // Message authentication code.  Only used on last hop.
  struct freedom_trailer_mapping frtrl;
};
*/

namespace {
  const size_t VERSION1_PAYLOAD_SZ = 297 - DATA_PACKET_OVERHEAD + 2;
};

/*
 *  Adds one layer of encryption to the data packet.  The payload is assumed
 *  to be properly padded.
 */

void FreedomDataPacket::EncryptDataPacketLayer(FrCryptSym symkey) {
  try {

    char * cryptptr = begin + FR_HEADER_SZ;
    char * cryptend = end - FR_TRAILER_SZ;

    crypto->SymEncrypt(symkey, cryptptr, cryptend - cryptptr);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("EncryptDataPacket()");
    throw;
  }
}

/*
 *  Remove one layer of encryption from the data packet.
 */

void FreedomDataPacket::DecryptDataPacketLayer(FrCryptSym symkey) {
  try {

    char * cryptptr = begin + FR_HEADER_SZ;
    char * cryptend = end - FR_TRAILER_SZ;

    crypto->SymDecrypt(symkey, cryptptr, cryptend - cryptptr);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("DecryptDataPacket()");
    throw;
  }
}

/*
 *  Inserts padding, calculates the payload size and the MAC, then inserts
 *  both in the packet.
 */

void FreedomDataPacket::SetDataFields(FrCryptMacKey mackey) {

  try {

    // Set the 8 byte random vector, needed for good crypto.
    
    crypto->FillBufferWithRandomData(begin + FR_HEADER_SZ, RANDOM_VECTOR_SZ);

    // Payload starts after the freedom header, the random vector, the
    // payload length field and 2 bytes of padding for #%$*(& backwards
    // compatibility.  It should have been loaded there by the recv function.
    
    char * payloadptr = begin + DATA_PAYLOAD_OFFSET - 2;

    uint16_t payloadpad, payloadsz;
    uint16_t truepayloadsz = end - begin;
    payloadsz = truepayloadsz + 2;
    
    // *Sigh* in order to handle v1 packets, we have to pad the payload to
    // VERSION1_PAYLOAD_SZ.  Otherwise, we have to pad for crypto alignment,
    // and offset by 2 bytes so as to align the IP header as it was done
    // before.  This padding was covered by selecting the right magic # in v1.

    ConfigManager * cfgmgr = cfgmgr->Instance();
    if (cfgmgr->PktVersion() == VERSION1) {
      if (payloadsz > VERSION1_PAYLOAD_SZ) {
        ostringstream errstrm;
        errstrm << "Payload too big:  " << payloadsz;
        throw BadPacketException(errstrm.str(), begin, end - begin);
      }
      else
        payloadpad = VERSION1_PAYLOAD_SZ - truepayloadsz - 2;
    }
    else {
      // The magic crypto alignment calculation...no logic be found here.
      payloadpad = ((((payloadsz - 4) + 7) & ~7) + 4) - payloadsz;
    }

    payloadsz += payloadpad;
    uint16_t nbosz = htons(payloadsz);
    memcpy(begin + FR_HEADER_SZ + RANDOM_VECTOR_SZ, &nbosz, sizeof(uint16_t));
    nbosz = htons(truepayloadsz);
    // The actual payload length is placed (redundantly) in the first 2 bytes
    // of the payload field.
    memcpy(payloadptr, &nbosz, sizeof(uint16_t));
    
    char * macptr = payloadptr + payloadsz;
    
    // Calculate the MAC over the actual payload, not the padding.  But don't
    // try to calculate a MAC over a null payload.
    if (payloadsz)
      crypto->ApplyMACOverBuffer(mackey, payloadptr - MAC_SZ,
                                 payloadsz + MAC_SZ, macptr, MAC_SZ);

    // The end is found after the payload, the padding, the MAC, the eights
    // and the lekv.  Since we didn't know the amount of padding before now,
    // we must set the end pointer here.
    
    end = macptr + MAC_SZ + FR_TRAILER_SZ;
    if (end > tail)
      throw BadPointerException("packet overflows buffer", head, tail,
                                begin, end);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SetDataFields()");
    throw;
  }
}

/*
 *  Used to verify the MAC at the last hop.
 */

void FreedomDataPacket::VerifyDataFields(FrCryptMacKey mackey) const {

  try {

    char * payloadptr = begin + DATA_PAYLOAD_OFFSET - 2;

    uint16_t payloadsz;
    memcpy(&payloadsz, begin + FR_HEADER_SZ + RANDOM_VECTOR_SZ,
           sizeof(uint16_t));
    payloadsz = ntohs(payloadsz);
    ConfigManager * cfgmgr = cfgmgr->Instance();
    if (cfgmgr->PktVersion() == VERSION1) {
      if (payloadsz > VERSION1_PAYLOAD_SZ) {
        ostringstream errstrm;
        errstrm << "Payload too big:  " << payloadsz;
        throw BadPacketException(errstrm.str(), begin, end - begin);
      }
    }

    if (payloadsz != 0) {
      char * macptr = payloadptr + payloadsz;

      if ((macptr + MAC_SZ + FR_TRAILER_SZ) != end)
        throw BadPacketException("payloadsz is incorrect", begin, end - begin);

      if (!crypto->VerifyMACOverBuffer(mackey, payloadptr - MAC_SZ,
                                       payloadsz + MAC_SZ, macptr, MAC_SZ))
        throw BadPacketException("MAC Invalid", begin, end - begin);
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("VerifyDataFields()");
    throw;
  }
}

/*
 *
 */

void FreedomDataPacket::ReceiveFromShim(int shimfd) {

  int retval = read(shimfd, head + DATA_PAYLOAD_OFFSET,
                    tail - head - DATA_PACKET_OVERHEAD);
  if (retval <= 0)
    throw BadPacketException(strerror(errno), head, tail - head);

  // This is WRONG...the end is actually DATA_PAYLOAD_OFFSET bytes later,
  // but that is addressed in SetDataFields().
  end = begin + retval;
  if (end > tail)
    throw BadPointerException("packet overflows buffer", head, tail,
                              begin, end);
  uint16_t ippktsz;
  memcpy(&ippktsz, begin + DATA_PAYLOAD_OFFSET + 2, 2);
  ippktsz = ntohs(ippktsz);
  if (ippktsz != static_cast<uint16_t>(retval)) {
    ostringstream errstrm;
    errstrm << "IP packet length field:  " << ippktsz
            << " doesn't match packet length:  " << end - begin;
    throw BadPacketException(errstrm.str(), head, tail - head);
  }

  memset(&pktaddr, 0, sizeof(pktaddr));

  memcpy(&(pktaddr.sin_port), begin + DestPortOffset(),
         sizeof(pktaddr.sin_port));

  memcpy(&(pktaddr.sin_addr.s_addr), begin + DestIPOffset(),
         sizeof(pktaddr.sin_addr.s_addr));
}

/*
 *
 */

void FreedomDataPacket::SendToShim(int shimfd) const {

  uint16_t ippktsz;
  memcpy(&ippktsz, begin + DATA_PAYLOAD_OFFSET + 2, 2);
  ippktsz = ntohs(ippktsz);
  if (begin + DATA_PAYLOAD_OFFSET + ippktsz > end) {
    throw BadPacketException("IP Packet points beyond end of packet", begin,
                             end - begin);
  }

  int retval = write(shimfd, begin + DATA_PAYLOAD_OFFSET, ippktsz);
  if (retval <= 0)
    throw BadPacketException(strerror(errno), head, tail - head);
}

/*
 *
 */

void FreedomDataPacket::SetPayload(char* payload, size_t payloadsz) {

  memcpy(begin + DATA_PAYLOAD_OFFSET, payload, payloadsz);

  end = begin + payloadsz;
  if (end > tail)
    throw BadPointerException("packet overflows buffer", head, tail,
                              begin, end);
}

/*
 *  Indexes into the buffer as if it were an IP packet, and extracts the
 *  port number.
 */

unsigned short FreedomDataPacket::GetPort() const {

  uint16_t destport;

  memcpy(&destport, begin + DestPortOffset(), sizeof(destport));
  return ntohs(destport);
}

uint32_t FreedomDataPacket::GetSrcIP() const {

  uint32_t srcip;

  memcpy(&srcip, begin + SrcIPOffset(), sizeof(srcip));
  return ntohl(srcip);
}

size_t FreedomDataPacket::SrcIPOffset() const {
  return DATA_PAYLOAD_OFFSET + 12;
}

size_t FreedomDataPacket::DestIPOffset() const {
  return DATA_PAYLOAD_OFFSET + 16;
}

size_t FreedomDataPacket::DestPortOffset() const {

  // Offset into the payload by sizeof an IP header (20 bytes) plus 2 bytes to
  // skip over the source port field.  In both UDP and TCP headers, the
  // destination port is found here.  Note that to be absopositively sure we
  // should do 2 more things:  Check the IP header length field to verify that
  // it is in fact 20, and check the protocol field to verify that this is
  // indeed TCP or UDP.  Oh well.

  return DATA_PAYLOAD_OFFSET + 22;
}

/*
 *
 */

string TestFreedomDataPacket::UnitTest(size_t numloops) {

  try {
    
    string testresults = "Freedom Data Packet Test:\n";
    ConfigManager * cfgmgr = cfgmgr->Instance();
    srand(time(0));
    VersionType oldversion = cfgmgr->PktVersion();
    cfgmgr->SetPktVersion(VERSION1);
    testresults += "Data Payload Test, version 1";
    testresults += (DataPayloadTest(numloops) ? 
		    "..........[passed]\n" : 
		    "..........[FAILED]\n");
    cfgmgr->SetPktVersion(VERSION2);
    testresults += "Data Payload Test, version 2";
    testresults += (DataPayloadTest(numloops) ? 
		    "..........[passed]\n" : 
		    "..........[FAILED]\n");
    cfgmgr->SetPktVersion(oldversion);
    testresults += "[WARNING] The following functions are not tested:\n";
    testresults += "\tGetPort()\n";
    testresults += "\tReceiveFromShim()\n";
    testresults += "\tSendToShim()\n";
    
    return testresults;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SelfTest()");
    throw;
  }
}

/*
 *
 */

bool TestFreedomDataPacket::DataPayloadTest(size_t numloops) {

  try {
    bool testresult = true;
    FreedomDataPacket datapkt;
    CryptoWrapper * crypto = crypto->Instance();
    for (size_t i = 0; ((i < numloops) && (testresult)); ++i) {
      FrCryptSym datakey = crypto->GenerateSymkey();
      FrCryptMacKey datamac = crypto->GenerateMackey();
      ConfigManager * cfgmgr = cfgmgr->Instance();
      size_t maxpayload = ((cfgmgr->PktVersion() == VERSION1) ?
                           VERSION1_PAYLOAD_SZ - 2 : 1399);
      size_t databufsz = (rand() >> 16) % maxpayload + 1;
      char * databuf = new char[databufsz];
      crypto->FillBufferWithRandomData(databuf, databufsz);
      crypto->AddBufferToEntropyPool(databuf, databufsz);
      datapkt.SetPayload(databuf, databufsz);
      datapkt.SetDataFields(datamac);
      datapkt.EncryptDataPacketLayer(datakey);    
      datapkt.DecryptDataPacketLayer(datakey);
      datapkt.VerifyDataFields(datamac);
      crypto->MackeyFree(datamac);
      crypto->SymkeyFree(datakey);
      if (memcmp(datapkt.begin + DATA_PAYLOAD_OFFSET, databuf, databufsz)) {
        cerr << "Data Test Failed for buffer size:  " << databufsz << endl;
        testresult = false;
      }
      delete [] databuf;
    }
    return testresult;
  }
  catch (TestModuleException & tme) {
    tme.AddScope("DataPayloadTest()");
    cerr << tme.what();
    return false;
  }
}

string TestFreedomDataPacket::ErrorHandlingTest(size_t numloops) {

  ostringstream testresults;
  CryptoWrapper * crypto = crypto->Instance();
  FreedomDataPacket datapkt;
  try {
    FrCryptSym symkeys[2];
    symkeys[0] = crypto->GenerateSymkey();
    symkeys[1] = crypto->GenerateSymkey();
    for (size_t i = 0; i < numloops; ++i) {

      // Test pointer sanity check.
      FrCryptMacKey mackey =0;
      char * oldbegin = datapkt.begin;
      datapkt.begin = datapkt.end;
      try {
        datapkt.SetDataFields(mackey);
      }
      catch (BadPointerException & bpe) {
        testresults << bpe.what() << endl;
      }
      datapkt.begin = oldbegin;
    }
    return testresults.str();
  }
  catch (TestModuleException & tme) {
    cerr << testresults.str();
    tme.AddScope("ErrorHandlingTest()");
    throw;
  }
}
