/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   This object handles all byte level manipulations for Freedom
 *                 Packets.  It is inspired by the sk_buffs in the Linux
 *                 kernel.
 *  Todo:          - Enforce function call dependancies.
 *                 - Integrate with PacketBuffer.
 *                 - Apply an inheritance hierarchy.
 *                 - Nicer pointer manipulation.
 *                 - Eliminate resource leaks on failure.
 *                 - Add thread support, if necessary.
 *                 - Remove pragma pack().
 *  Known Bugs:    None.
 */

#ifndef ZK_AIP_FRDATAPKT_JJDFKSZK_DEF
#define ZK_AIP_FRDATAPKT_JJDFKSZK_DEF

#include "FreedomPacket.h"

class TestFreedomDataPacket;

class FreedomDataPacket : public FreedomPacket {

  friend class TestFreedomDataPacket;
  
 public:
  
  FreedomDataPacket() {}
  FreedomDataPacket(FreedomPacket & incomingpkt) :
    FreedomPacket(incomingpkt) {}
  ~FreedomDataPacket() {}

  // Sets padding and computes the MAC, or vice versa for the 2nd function.
  
  void SetDataFields(FrCryptMacKey mackey);
  void VerifyDataFields(FrCryptMacKey mackey) const;
  
  void EncryptDataPacketLayer(FrCryptSym layerkey);
  void DecryptDataPacketLayer(FrCryptSym layerkey);

  void ReceiveFromShim(int socket);
  void SendToShim(int socket) const;

  unsigned short GetPort() const;
  uint32_t GetSrcIP() const;

 protected:
  
  // To support testing in classes that use this class.  Not to be used
  // normally

  void SetPayload(char * buffer, size_t buffersize);
  char * GetPayload(size_t & pktsz) const
    {pktsz = end - begin; return begin;}

 private:

  size_t SrcIPOffset() const;
  size_t DestIPOffset() const;
  size_t DestPortOffset() const;
  // To prevent inadvertant copying.  Although we could support them, they
  // could be a hidden performance drain.
  
  FreedomDataPacket& operator=(const FreedomDataPacket&);
};

class TestFreedomDataPacket {

 public:

  TestFreedomDataPacket() {}
  ~TestFreedomDataPacket() {}

  string UnitTest(size_t numloops = 1);
  string ToString() const;

  bool DataPayloadTest(size_t numloops = 1);
  bool MultipleCryptionTest(size_t numloops = 1);
  string ErrorHandlingTest(size_t numloops = 1);
};

#endif
