/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   This object represents Freedom Route Destroy Packets.
 *  Todo:          - Better exception handling/checking
 *                 - Better inheritance.
 *  Known Bugs:    None.
 */

#ifndef ZK_AIP_FRDESTROYPKT_JJDFKSZK_DEF
#define ZK_AIP_FRDESTROYPKT_JJDFKSZK_DEF

#include "FreedomDataPacket.h"

class FreedomDestroyPacket : public FreedomDataPacket {

 public:
  
  FreedomDestroyPacket() {}  
  FreedomDestroyPacket(FreedomPacket & frpkt)
    : FreedomDataPacket(frpkt) {}
  ~FreedomDestroyPacket() {}

  uint32_t GetCode() const {

    // the route destroy error code is located in the pktspecific data field

    uint32_t tempcode;
    if (Version() == VERSION1)
      return 0;
    memcpy(&tempcode, begin + 10, 4);
    return ntohl(tempcode);
  }

  string GetEntityName() const {

    //the entity that generated the route destroy is located in the
    //location where the extend aci is usually found

    char entname[64];
    if (Version() == VERSION1)
      return "";
    strncpy(entname, begin + EXTEND_ACI_OFFSET, 62);    
    return entname;
  }

  void SetDestroyFields(uint32_t ErrorCode, string BadEntity) {
    if (Version() == VERSION1)
      return;
    uint32_t tempcode = htonl(ErrorCode);
    memcpy(begin + 10, &tempcode, 4);
    strncpy(begin + EXTEND_ACI_OFFSET, BadEntity.c_str(),
            BadEntity.length() + 1);
  }
  
 private:

  // To prevent inadvertant copying.  Although we could support them, they
  // could be a hidden performance drain.
  FreedomDestroyPacket& operator=(const FreedomDestroyPacket&);
};

#endif
