#ifndef ZK_AIP_KEY_HNDLR_DEF
#define ZK_AIP_KEY_HNDLR_DEF

#include "NetData.h"
#include "frSec.h"
#include "frKeyQry.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

/*  Title:       KeyHandler
 *  Author(s):   Troy McLelan
 *  Description: Put something here.
 *  ToDo:        - stuff
 *  Known Bugs:  None
 */

/*
 *
 */
class KeyHandler {
 public:
  static KeyHandler * Instance();
  ~KeyHandler();
  KeyHandler(KeyHandler &) { throw BadOperatorException("KeyHandler copy"); }

 public:
  static std::string  keyTypeToStr(FrPubKeyType);
  static FrPubKeyType strToKeyType(std::string);

 public:
  std::string toString();
  std::string test(std::string);
  std::string system();

 public:
  // Queries to the key Server
  FrCryptPubKey keyServerQuery       (unsigned int,FrPubKeyType,FrEntIdExp);
  void          keyServerQueryStatus (unsigned int,FrPubKeyType,FrEntIdExp,
                                      ZkBool *,ZkBool *);

  // Save/Load a private key to/from a file
  std::string   SavePrivateKey (FrCryptPubKey, FrEntIdExp);
  FrCryptPubKey LoadPrivateKey (FrPubKeyType, unsigned int, FrEntIdExp);

  // Private key verification
  bool VerifyPrivateKey(FrCryptPubKey);

  // Function to build a key query (soon to be depracated
  void buildQuery(string,string,string,string,FrKeyQryQueryExp *);
  void buildEntityFromString(FrEntIdExp *,std::string,std::string);

private:
  // Create a directory to save the private key in
  void        keyMkDir   (FrEntIdExp, FrPubKeyType);
  std::string keyGetPath (FrEntIdExp, FrPubKeyType, unsigned int = 0);
  KeyHandler();

 private:  // Forbidden ops
  KeyHandler & operator=(const KeyHandler &) {
    throw BadOperatorException("KeyHandler =");
  }
  void operator&() {
    throw BadOperatorException("KeyHandler &");
  }

 private:
  ZkError err;
  FrSecKeyChain kc;
  FrKeyQry kq;
  FrKeyQryHost keyHost;

 private:  // Self and system tests
  std::string runSystemQueryTest(FrPubKeyType, FrEntIdExp, unsigned int);

  std::string runPubKeyTests (FrPubKeyType, bool *);
  bool keyFetchPubKeyTest    (FrPubKeyType, FrEntIdType, unsigned int);
  bool keyStatusPubKeyTest   (FrPubKeyType, FrEntIdType, unsigned int);

  std::string runPrivKeyTests(FrPubKeyType, std::string, bool *);
  bool keyGenPrivKeyTest     (FrPubKeyType, FrEntIdExp, FrCryptPubKey *);
  bool keySavePrivKeyTest    (FrCryptPubKey,FrEntIdExp);
  bool keyLoadPrivKeyTest    (FrPubKeyType, FrEntIdExp, FrCryptPubKey *);
  bool keyVerifyPrivKeyTest  (FrCryptPubKey);

};

extern KeyHandler * keyhndptr;
#endif

