/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   Handles link-level functionality.
 *  Todo:          - Handle incoming packets better (link decrypt failure).
 *                 - Track link key creation times.
 *                 - Track link status.
 *                 - Add support for keep alives.
 *  Known Bugs:    - IncomingPacket() will occasionally destroy a valid packet.
 */

#ifndef ZK_LINK_JJDFKSZK_DEF
#define ZK_LINK_JJDFKSZK_DEF

#include "FreedomPacket.h"
#include "DHNegotiator.h"

class Link {
public:
  Link(struct sockaddr_in);
  ~Link();
  void OutgoingPacket(FreedomPacket&);
  void IncomingPacket(FreedomPacket&);
  void RenegotiateLinkKey();
  unsigned int SecondsUntilReKey();
  unsigned int SecondsUntilKeepAlive();
  string ToString();
private:

  Link(Link &);
  Link& operator=(const Link&);
  void operator&();
  
  void AddKey(FrCryptSym, LekvType);
  FrCryptSym activekey;
  list<FrCryptSym> keylist;
  LekvType lekvt;
  struct sockaddr_in addr;
  struct timeval lastrekey, lastpacket;
};

#endif ZK_LINK_JJDFKSZK_DEF
