/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   Adds and removes links from internal mapping, handles
 *                 incoming and outgoing packets.
 *  Todo:          - Need to signal rekey and keep alive instances.
 *                 - IPPortHash() doesn't handle identical IP addresses
 *                   correctly.
 *                 - Implement reference counting.
 *                 - Implement link monitoring.
 *  Known Bugs:    None.
 */

#ifndef ZK_LINK_MGR_JJDFKSZK_DEF
#define ZK_LINK_MGR_JJDFKSZK_DEF

#include <pthread.h>

#include "Link.h"
#include "ShimHandler.h"
#include "Route.h"

class LinkManager {
public:
  LinkManager();
  ~LinkManager();
  static LinkManager * Instance();

  // For explicitly creating and destroying a link object.
  
  Link * AddLink(struct sockaddr_in);
  void RemoveLink(struct sockaddr_in);

  FreedomPacket * IncomingPacket(FreedomPacket & incomingpkt);
  void OutgoingPacket(FreedomPacket & outgoingpkt);
  string ToString();

 private:

  LinkManager(LinkManager &);
  LinkManager& operator=(const LinkManager&);
  void operator&();

  static void * MonitorLinks(void * noop);
  
  uint32_t IPPortHash(struct sockaddr_in addr)
    { return addr.sin_addr.s_addr; }
  pthread_t linkthread;
  pthread_mutex_t link_lock;

  void Lock() { pthread_mutex_lock(&link_lock); }
  void Unlock() { pthread_mutex_unlock(&link_lock); }
  map <uint32_t, Link *> linkmap;
  size_t numlinks;
};

extern LinkManager * linkman;
extern bool runthread;

class TimeoutException : public TestModuleException {
 public:
  TimeoutException(string msg_in) {SetMessage(msg_in);}
  ~TimeoutException() {}
 private:
};

class ExpiredRouteException : public TestModuleException {
public:
  ExpiredRouteException(string msg_in) {SetMessage(msg_in);}
  ~ExpiredRouteException() {}
private:
};

#endif ZK_LINK_MGR_JJDFKSZK_DEF
