#include "NetData.h"

namespace {
  const unsigned short niqsport = 51107;
};

NetData * niqsptr;
/*
 *
 */
NetData::NetData(string domain_in) {

  try {
    ni = 0;
    zkErrorNew(&err);
    tt = 0;
    FreedomDelta = 0;
    memset(&ci, 0, sizeof(ci));
    domain = "";
    scalperurl = "";
    DatumN = 0; /*iterator, of a sort...*/
    
    if (domain_in == "")
      throw NetDataNotInitialized("NetData must be initialized before proceeding, try 'net load <domain name>'");
    else
      domain = domain_in;
      LoadNetInfo(domain_in);

  }
  catch (TestModuleException & tme) {
    tme.AddScope("NetData()");
    throw;
  }
}

/*
 *
 */
NetData::~NetData() {

  if (ni) frNetInfoFree(err, &ni);
  zkErrorFree(&err);
}
/*
 *
 */
NetData * NetData::Instance(string domain_in) {
  if (niqsptr == 0)
    niqsptr = new NetData(domain_in);
  return niqsptr;
}

bool NetData::GetDatum(string name,FrNetInfoDatum* datum, FrEntIdType ent_type) {
  string tempname;
  ZkList listptr = NULL;
  size_t listsz = 0;
  FrEntIdExp ent;
  try {

    if (!frNetInfoGetListOf(err, ni, &listptr, ent_type))
      throw TestModuleException(err, "frNetInfoGetListOf()");
    
      if (!zkListMoveCursor(err, listptr, LIST_HEAD))
        throw TestModuleException(err, "zkListMoveCursor()");
      if (!zkListSize(err, listptr, &listsz))
        throw TestModuleException(err, "zkListSize()");
    
      for (size_t i = 0; i < listsz; i++) {
        if (!zkListGet(err, listptr, reinterpret_cast<void**>(datum), LIST_FORWARDS))
          throw TestModuleException(err, "zkListGet()");
	if (!frNetInfoDatumGetEntity(err, *datum, &ent))
	  throw TestModuleException(err, "frNetInfoDatumGetEntity()");
	tempname = ent.name;
	if (tempname == name) {
	  return true;
	}
      }
      return false; /*didn't find it*/
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetDatum()");
    throw;
  }
}

void NetData::ResetDatumList() {
  DatumN = 0;
}

bool NetData::GetNextDatum(FrNetInfoDatum* datum,FrEntIdType ent_type) {
  string tempname;
  ZkList listptr = NULL;
  size_t listsz = 0;
  try {


    if (!frNetInfoGetListOf(err, ni, &listptr, ent_type))
      throw TestModuleException(err, "frNetInfoGetListOf()");
    
    // if there are no entities of the given type, NULL is returned instead
    // of an empty non-NULL list, this must be checked for
    if (listptr != NULL)
      {
        if (!zkListMoveCursor(err, listptr, LIST_HEAD))
          throw TestModuleException(err, "zkListMoveCursor()");
        if (!zkListSize(err, listptr, &listsz))
          throw TestModuleException(err, "zkListSize()");
    
        if (DatumN >= listsz) return false;
    
        /*get the nth datum from the list*/
        for (size_t i = 0; i < listsz; i++) {
          if (!zkListGet(err, listptr, reinterpret_cast<void**>(datum),
                         LIST_FORWARDS))
            throw TestModuleException(err, "zkListGet()");
          if (i == DatumN) {
            DatumN++; /*increase the global DatumN number*/
            return true;
          }
        }
      }
    return false; /*error state*/
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetNextDatum()");
    throw;
  }
}


/*
 *
 */
string NetData::ToString() {

  try {
    ostringstream sstrm;
    sstrm << "Network Data"  << endl;
    sstrm << "FreedomDelta:  " << FreedomDelta << endl;
    sstrm << "scalperurl " << scalperurl << endl;

    return sstrm.str();
  }
  catch (TestModuleException &tme) {
    tme.AddScope("NetData::ToString()");
    throw;
  }
}

bool NetData::LoadTest() {
  try {
    LoadNetInfo(domain);
    FrTokenType tokentype;
    tokentype = GetTokenType();

    /*indicates that there was a problem loading the net info*/
    if (tokentype == 0)
      return false;
    
    string scalper;
    scalper = GetScalperUrl();
    if (scalper == "") 
      return false;

    return true;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("LoadTest()");
    cerr<<tme.what();
    return false;
  }
}

bool NetData::GetInfoTest() {
  try {
    struct sockaddr_in sockaddr;
    FrEntIdExp ent;
    memset(&ent,0,sizeof(ent));
    memset(&sockaddr,0,sizeof(sockaddr));

    /*try and pull out the key query server*/
    GetCoreServer(FRENT_TYPE_KEYQRY_SRV,&sockaddr,&ent);

    /*pull out a non-existent aip*/
    memset(&ent,0,sizeof(ent));
    memset(&sockaddr,0,sizeof(sockaddr));

    if (GetAIP("bad_aip_name",&sockaddr,&ent)) 
      throw TestModuleException("recognized non-existant aip");

    /*hmmmm, i'm not sure that this test is valid... i'm assuming that
      there is always a "host1" aip on a network - which could crash
      and burn*/

    memset(&ent,0,sizeof(ent));
    memset(&sockaddr,0,sizeof(sockaddr));

    if (!GetAIP("host1",&sockaddr,&ent))
      throw TestModuleException("didn't recognize known aip");


    return true;


  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetInfoTest()");
    cerr<<tme.what();
    return false;
  }
}

string NetData::SystemTest() {

  try {
    ostringstream returnstream;
    
    returnstream<<"LoadNetInfo() test: ";
    if (LoadTest()) 
      returnstream<<"\n\t\t..........[passed]\n";
    else returnstream<<"\n\t\t..........[FAILED]\n";

    returnstream<<"Querying netinfo object: ";
    if (GetInfoTest())
      returnstream<<"\n\t\t..........[passed]\n";
    else returnstream<<"\n\t\t..........[FAILED]\n";

    return returnstream.str();
  }
  catch (TestModuleException &tme) {
    tme.AddScope("NetData::SelfTest()");
    throw;
  }
}
/*
 *
 */
void NetData::LoadNetInfo(string domain_in) {

  try {
    ConfigManager * cfghnd = cfghnd->Instance();
    cfghnd->SetDomain(domain_in);

    // HACK! Getting the network name from the domain_in 
    //       parameter assuming that it is s of the form
    //       <netname>.[ freedom.net | dev.zks.net ]
    //
    // UPDATE! All scalpers have been changed to domain
    // freedom.net so this hack ain't so bad no more.

    string::size_type index = domain_in.find('.');
    if (index != string::npos) {
      cfghnd->SetNetwork( domain_in.substr(0,index) );
    } 
    else {
      cfghnd->SetNetwork(domain_in);
    }

    cfghnd->SetScalper( "http://scalper.freedom.net/" );

    if (ni && !frNetInfoFree(err, &ni))
      throw TestModuleException(err, "frNetInfoFree()");
  
    uint16_t numiter = 1;
    string clientid = "freedom-windows-1.1";

    struct sockaddr_in addr;
    memset(&addr, 0, sizeof(addr));

    if (!frNetInfoNew(err, &ni))
      throw TestModuleException(err, "frNetInfoNew()");
    string niqshost = "niqs.";
    niqshost += cfgptr->Domain();

    // frCryptInit(), located in the CryptoWrapper constructor, must be called
    // before any calls to crypto functions, including frKeyQryNew().
    
    CryptoWrapper * crypto = crypto->Instance();
    FrKeyQry kq = 0;
    if (!frKeyQryNew(err, &kq))
      throw TestModuleException(err, "frKeyQryNew()");
    if (!frNetInfoClientInitSequence(err, ni, niqshost.c_str(),
                                     niqsport, cfghnd->Timeout(), numiter, kq,
                                     clientid.c_str(), &ci, &tt))
      throw TestModuleException(err, "frNetInfoClientInitSequence");
    FreedomDelta = ci.current_UTC + time(0);
    scalperurl = ci.sn_url;
    if (!frKeyQryFree(err, &kq))
      throw TestModuleException(err, "frKeyQryFree()");
  }
  catch (TestModuleException &tme) {
    tme.AddScope("LoadNetInfo()");
    throw;
  }
}
/*
 *
 */
void NetData::GetCoreServer(FrEntIdType enttype, struct sockaddr_in* addr,
                            FrEntIdExp * entity) {

  FrNetInfoDatum datum = NULL;

  try {
    if (addr == 0)
      throw BadParameterException("addr pointer is NULL");
    if (entity == 0)
      throw BadParameterException("entity pointer is NULL");

    ResetDatumList();
    if (!GetNextDatum(&datum,enttype)) 
      throw TestModuleException("No datum avaliable");

    if (!frNetInfoDatumGetSockaddr(err, datum, addr))
      throw TestModuleException(err, "frNetInfoDatumGetSockaddr()");
    if (!frNetInfoDatumGetEntity(err, datum, entity))
      throw TestModuleException(err, "frNetInfoDatumGetEntity()");
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetCoreServer()");
    throw;
  }
}
/*
 *
 */
void NetData::GetEntityList(ZkList * olst) {

  FrEntIdExp * entptr = NULL;
  FrNetInfoDatum datum = NULL;

  try {
    if (olst == 0)
      throw BadParameterException("lst pointer is NULL");

    for (int i = static_cast<int> (FRENT_TYPE_UNUSED_ + 1); 
         i < static_cast<int> (FRENT_TYPE_MAX); ++i) {

      ResetDatumList();
      while (GetNextDatum(&datum,(FrEntIdType) i)) {
	if ( !(entptr = (FrEntIdExp *) malloc(sizeof(FrEntIdExp))) )
	  throw TestModuleException(err,
				    "Couldn't allocate entity structure");
	
	if (!frNetInfoDatumGetEntity(err, datum, entptr))
	  throw TestModuleException(err, "frNetInfoDatumGetEntity()");
	
	if (!zkListInsert(err, *olst, reinterpret_cast<void*>(entptr),
			  LIST_TAIL))
	  throw TestModuleException(err, "zkListInsert()");
	
	entptr = NULL;
      }

    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetEntityList()");

    if (entptr) free(entptr);

    throw;
  }
}
/*
 *
 */
FrTokenType NetData::GetTokenType() {

  return tt;
}
/*
 *
 */
string NetData::EntityList() {

  string entlist = frEntTypeGetStr(FRENT_TYPE_AIP);
  entlist += "\n";

  for (int i = static_cast<int>(FRENT_TYPE_FMG);
       i < static_cast<int>(FRENT_TYPE_MAX); ++i) {
    entlist += frEntTypeGetStr((FrEntIdType)i);
    entlist += "\n";
  }
  
  return entlist;
}

bool NetData::GetAIP(string name, struct sockaddr_in* addr, FrEntIdExp *ent) {

  FrNetInfoDatum datum = NULL;

  try {
    if (addr == 0)
      throw BadParameterException("addr pointer is NULL");
    if (ent == 0)
      throw BadParameterException("ent pointer is NULL");

    if (!GetDatum(name,&datum)) return false;
    
    if (!frNetInfoDatumGetEntity(err, datum, ent))
      throw TestModuleException(err, "frNetInfoDatumGetEntity()");
    if (!frNetInfoDatumGetSockaddr(err, datum, addr))
      throw TestModuleException(err, "frNetInfoDatumGetSockaddr()");

    return true;

  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetAIP()");
    throw;
  }
}

string NetData::GetAipNames() {
  string tempname = "";
  FrNetInfoDatum datum = NULL;
  FrEntIdExp ent;

  try {

    ResetDatumList();
    while (GetNextDatum(&datum)) {
      int status;
      if (!frNetInfoDatumGetStatus(err, datum, &status))
	throw TestModuleException(err, "frNetInfoDatumGetStatus()");
      /*at the moment i am ignoring all aips that are
	down... possibly they should be returned with a special
	message that they aren't currently up*/
      if (status) {
	if (!frNetInfoDatumGetEntity(err,datum,&ent))
	  throw TestModuleException(err,"frNetInfoDatumGetEntity()");
	string aipname = ent.name;
	tempname += aipname + " ";
      }
    }
    return tempname;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetAipNames()");
    throw;
  }
}
void NetData::GetAipNames(string* names,size_t max,size_t& count) {
  string tempname = "";
  FrNetInfoDatum datum = NULL;
  FrEntIdExp ent;

  try {
    count = 0;
    ResetDatumList();
    while (GetNextDatum(&datum)) {
      int status;
      if (!frNetInfoDatumGetStatus(err, datum, &status))
	throw TestModuleException(err, "frNetInfoDatumGetStatus()");
      /*at the moment i am ignoring all aips that are
	down... possibly they should be returned with a special
	message that they aren't currently up*/
      if (status) {
	if (!frNetInfoDatumGetEntity(err,datum,&ent))
	  throw TestModuleException(err,"frNetInfoDatumGetEntity()");
	string aipname = ent.name;
	if (count<max) {
	  names[count] = aipname;
	  count++;
	}
      }
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetAipNames()");
    throw;
  }
}
/*
 *
 */
string NetData::GetNetDatum(FrEntIdType ent_type) {

  FrNetInfoDatum datum = NULL;
  uint32_t id, nbid;
  FrEntIdExp entity;
  struct sockaddr_in addr;
  int status, rating;
  size_t numNb;
  ostringstream sstrm;

  sstrm << setw(4) << "id" << setw(15) << "name" << setw(15) << "address" <<
    setw(8) << "status" << setw(8) << "rating";

  if (FRENT_TYPE_AIP == ent_type)
    sstrm << setw(20) << "neighbours";

  sstrm<<"\n";

  try { 
    ResetDatumList();
    while (GetNextDatum(&datum,ent_type)) {
      if (!frNetInfoDatumGetId(err, datum, &id))
        throw TestModuleException(err, "frNetInfoDatumId()");
      if (!frNetInfoDatumGetSockaddr(err, datum, &addr))
        throw TestModuleException(err, "frNetInfoDatumGetSockaddr()");
      if (!frNetInfoDatumGetEntity(err, datum, &entity))
        throw TestModuleException(err, "frNetInfoDatumGetEntity()");
      if (!frNetInfoDatumGetStatus(err, datum, &status))
        throw TestModuleException(err, "frNetInfoDatumGetStatus()");
      if (!frNetInfoDatumGetRating(err, datum, &rating))
        throw TestModuleException(err, "frNetInfoDatumGetRating()");
      sstrm << setw(4) << id << setw(15) << entity.name << setw(15) <<
        inet_ntoa(addr.sin_addr) << setw(8) << status << setw(8) << rating;

      if (FRENT_TYPE_AIP == ent_type) {
        if (!frNetInfoDatumGetNumNb(err, datum, &numNb))
          throw TestModuleException(err, "frNetInfoDatumGetNumNb()");
        for (size_t j = 0; j < numNb; ++j) {
          if (!frNetInfoDatumGetNbX(err, datum, j, &nbid))
            throw TestModuleException(err, "frNetInfoDatumGetNbX()");
          sstrm << setw(4) << nbid;
        }
      }
      sstrm<<"\n";
    }
    return (sstrm.str());
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetNetDatum()");
    throw;
  }
  return "";
}

/* isLegal
 *		Determines if two AIPs are listed as neighbours
 */
bool NetData::isLegal(string aip1, string aip2) {

  FrNetInfoDatum datum1 = NULL,datum2 = NULL;
  uint32_t aip1id = 0,nbid;
  size_t numNb;

  try {

    if ((!GetDatum(aip1,&datum1))||(!GetDatum(aip2,&datum2))) return false;

    if (!frNetInfoDatumGetId(err, datum1, &aip1id))
      throw TestModuleException(err, "frNetInfoDatumId()");

    if (!frNetInfoDatumGetNumNb(err, datum2, &numNb))
      throw TestModuleException(err, "frNetInfoDatumGetNumNb()");
    for (size_t j = 0; j < numNb; ++j) {
      /*search the list of neighbours to find out if aip1 is
	in the neighbour list*/
      if (!frNetInfoDatumGetNbX(err, datum2, j, &nbid))
	throw TestModuleException(err,"frNetInfoDatumGetNbX()");
      if (nbid==aip1id) return true; /*yay!  we're neighbours,
				       return true*/
    }
    return false;

  }
  catch (TestModuleException &tme) {
    tme.AddScope("isLegal()");
    throw;
  }

}

/* isValidAIPName
 *
 * Determines if the name of an AIP is valid
 *
 * NOTE: this probably needs expanding to check against coreAIP names
 **      too
 */
bool NetData::isValidAIPName( string name)
{
  FrNetInfoDatum datum;
  
  try {
    if (!GetDatum(name,&datum)) return false;
    else return true;
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetEnd()");
    throw;
  }
}

/* isValidRoute
 *		Determines if a given route is legal, both in terms of the validity
 *		of the AIP names and their ability to connect to their neighbours
 */
bool NetData::isValidRoute( size_t numhosts, uint8_t version, string* hosts)
{
	// Legal number of hosts?
	if ( numhosts == 0)
		return false;

	// Validate that all the hostnames are corrected
	for( size_t i = 0; i < numhosts; i++) {
		if ( !isValidAIPName( hosts[i]))
			return false;
	}

	// For AIP 1.0 a route can only be made through valid neighbours
	//    AIP 2.0 can connect connect to any other host
	if ( version == 1) {
		// A route must connect via neighbours!
		for( size_t i = 1; i < numhosts; i++) {
			if ( !isLegal( hosts[i-1], hosts[i]))
				return false;
		}
	}

	return true;
}

void NetData::dig(size_t depth,size_t current,FrNetInfoDatum
                    datum,string path,list<string>& routes) {
  size_t numNb;
  FrEntIdExp ent;
  uint32_t nbid;
  FrNetInfoDatum otherdatum;
  string name;

  int status;
  if (!frNetInfoDatumGetStatus(err, datum, &status))
    throw TestModuleException(err, "frNetInfoDatumGetStatus()");
  /*at the moment i am ignoring all aips that are
    down... possibly they should be returned with a special
    message that they aren't currently up*/
  if (!status) return;

  if (!frNetInfoDatumGetEntity(err, datum,&ent))
    throw TestModuleException(err, "frNetInfoDatumGetEntity()");
  name = ent.name;
  path += name + " ";
  if ((current+1)==depth) {
    routes.push_back(path);
    return;
  }
  /*pull out the number of neighbours*/
  if (!frNetInfoDatumGetNumNb(err, datum, &numNb))
    throw TestModuleException(err, "frNetInfoDatumGetNumNb()");
  for (size_t j = 0; j < numNb; j++) {
    if (!frNetInfoDatumGetNbX(err, datum, j, &nbid))
      throw TestModuleException(err,"frNetInfoDatumGetNbX()");
    if (!frNetInfoSearchById(err,ni,nbid,&otherdatum)) 
      throw TestModuleException(err,"frNetInfoSearchById");
    if (!frNetInfoDatumGetEntity(err, otherdatum,&ent))
      throw TestModuleException(err, "frNetInfoDatumGetEntity()");
    dig(depth,current+1,otherdatum,path,routes);
  }
}

list<string> NetData:: allRoutes(size_t sz) {

  FrNetInfoDatum datum = NULL;
  string path = "";
  list<string> routes;

  try {

    ResetDatumList();

    while (GetNextDatum(&datum)) {
      dig(sz,0,datum,path,routes);
    }

    return routes;

  }
  catch (TestModuleException &tme) {
    tme.AddScope("allRoutes()");
    throw;
  }
}

/*
 *
 */

void NetData::GetRandomRoute(LayerType layert, string * hostlist) {

  FrNetInfoDatum datum = NULL;
  int depth = static_cast<int>(layert);
  FrEntIdExp ent;
  bool done = false;

  try {
    while (!done) {
      ResetDatumList();
      while (GetNextDatum(&datum)) {
	bool stop = static_cast<bool>(rand() %2);
	if (stop) {
	  if (!frNetInfoDatumGetEntity(err, datum,&ent))
	    throw TestModuleException(err, "frNetInfoDatumGetEntity()");
	  int status;
	  if (!frNetInfoDatumGetStatus(err, datum, &status))
	    throw TestModuleException(err, "frNetInfoDatumGetStatus()");
	  if (status) {
	    hostlist[0] = ent.name;
	    done = true;
	    break;
	  }
	}
      }
    }

    int j = 1;

    while(j<depth) {
      /*randomly choose an aip*/
      ResetDatumList();
      while (GetNextDatum(&datum)) {
	bool stop = static_cast<bool>(rand() % 2);
	if (stop) {
	  if (!frNetInfoDatumGetEntity(err, datum,&ent))
	    throw TestModuleException(err,
				      "frNetInfoDatumGetEntity()");
	  int status;
	  if (!frNetInfoDatumGetStatus(err, datum, &status))
	    throw TestModuleException(err, "frNetInfoDatumGetStatus()");
	  if (status) hostlist[j] = ent.name;
	  else hostlist[j] = "badaip";
	  break;
	}
      }
      /*the assumption here is that we will eventually find a valid
	neighbour to our random aip... i guess that it is technically
	possible that this loop could run forever if it was unlucky
	enough to either hit bad aips, or if the network only had one
	aip on it.*/
      if ((hostlist[j]!="badaip")&&isLegal(hostlist[j-1],hostlist[j])) j++;
    } /*while*/

  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetRandomRoute()");
    throw;
  }
    


}
 
