#ifndef PZEWNCVB_DEF
#define PZEWNCVB_DEF

#include "CryptoWrapper.h"
#include "frTokenType.h"
#include "frNetInfo.h"
#include <iomanip>

/*
  
  these methods are not currently being exercised in the
  selftest():
  
  string GetNetDatum(FrEntIdType);
  string EntityList();
  bool isLegal(string,string);
  string allRoutes(size_t);
  time_t GetFreedomTime() const;
*/

typedef enum {
  LAYER1 = 1,
  LAYER2 = 2,
  LAYER3 = 3,
  LAYER4 = 4,
  LAYER5 = 5
} LayerType;


class NetData {
public:
  ~NetData();
  static NetData * Instance(string domain_in = "");
  NetData(NetData &);
  string ToString();
  string SystemTest();
  void LoadNetInfo(string);
  void GetCoreServer(FrEntIdType, struct sockaddr_in *, FrEntIdExp *);
  bool GetAIP(string, struct sockaddr_in *, FrEntIdExp *);
  string GetNetDatum(FrEntIdType);
  string EntityList();
  bool isLegal(string,string);
  bool isValidAIPName(string);
  bool isValidRoute(size_t, uint8_t, string *);
  list<string> allRoutes(size_t);
  FrTokenType GetTokenType();
  void GetEntityList(ZkList *);
  string GetScalperUrl() const { return scalperurl; }
  time_t GetFreedomTime() const { return FreedomDelta; }
  string GetAipNames();
  void GetAipNames(string*,size_t,size_t&);
  void GetRandomRoute(LayerType, string *);
  NetData(string domain_in = "");
 private:
  bool LoadTest();
  bool GetInfoTest();
  /*picks out a datum based upon its name, as stored in the FrEntIdExp*/
  bool GetDatum(string,FrNetInfoDatum*,FrEntIdType ent_type =
		FRENT_TYPE_AIP);
  /*this is a very simple iterator set up:\

    first: resetdatumlist() sets DatumN to 0, or the start of the list
    second: GetNextDatum takes datum # DatumN out of the list, and
      then increments DatumN  
    third: repeat the second step until a) you have the information you
      need or b) you run out of datums
  */
  size_t DatumN;
  void ResetDatumList();
  bool GetNextDatum(FrNetInfoDatum*,FrEntIdType ent_type =
		    FRENT_TYPE_AIP);

  void dig(size_t,size_t,FrNetInfoDatum,string,list<string>&);
  NetData& operator=(const NetData&);
  void operator&();
  ZkError err;
  FrNetInfo ni;
  FrNiqsClientInfo ci;
  FrTokenType tt;
  time_t FreedomDelta;
  string scalperurl;
  string domain;
};
extern NetData * niqsptr;

/*
 *
 */
class NetDataNotInitialized : public TestModuleException {
 public:
  NetDataNotInitialized(string msg_in) {SetMessage(msg_in);}
  ~NetDataNotInitialized() {}
 private:
};

#else
//#warning NetData.h redefined.
#endif /*NETDATA_DEF*/

