#include "TestModuleSet.h"
/*
 *
 */
NetModule::NetModule() {
  myname = "net";
  if (reg[myname] != 0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  activated = false;
  helpmessage = "The Net Module interfaces with the NIQS and displays information received from the NIQS\n";
  helpmessage += "net help\nnet shutdown\nnet print\nnet system\n";
  helpmessage += "net load <domain name>:  Loads the network info for the specified network\n";
  helpmessage += "net list <entity type>:  Lists the network entities of the specified type\n";
  helpmessage += "net entitylist:  Lists all possible entity types\n";
  helpmessage += "net try <domain name>: Attempts to load the network info for te specified network\n";
  helpmessage += "net test: Tests a list of known networks\n";
}
/*
 *  Calls cleanup incase it was not called yet, then explicitly deregisters
 *  the module.
 */
NetModule::~NetModule() {
  Deactivate();
  reg.erase(reg.find(myname));
}
/*
 *  Allocates the memory for the NetData object.
 */
void NetModule::Activate() {
  if (!activated) {
    netobj = netobj->Instance();
    activated = true;
  }
}
/*
 *
 */
void NetModule::Deactivate() {
}
/*
 *  Syntax:
 *  net load <ipaddr>
 *  net list <aip>|<coreaip>|<maip>|<fmg>|<keyqry>|<toksrv>|<nymsrv>|<niqs>|
 *           <niss>|<pop>|<nntp>|<imep>
 */
void NetModule::HandleRequest(UserRequest & request) {

  try {
    if (request.GetNthToken(1) == "load") {
      netobj->Instance(request.GetNthToken(2));
      request.SetResponse("loaded netinfo for "+request.GetNthToken(2));
      Activate();
      return;
    }
    else if (request.IsCommandEqual("net help")) {
      request.SetResponse(helpmessage);
	}
    else if (request.IsCommandEqual("net test")) {
      ostringstream returnstream;
      string networks[4];
      networks[0] = "area51";
      networks[1] = "devtrans";
      networks[2] = "gamma"; 
      networks[3] = "delta";
      returnstream<<"Querying known networks\n\n";
      for (int i = 0;i<4;i++) {
		try {
		  NetData nettest(networks[i]);
		  returnstream<<networks[i]<<"\n\t [passed]\n";
		} catch (TestModuleException &tme) {
		  returnstream<<networks[i]<<"\n\t\t [FAILED]\n";
		}
      }
      request.SetResponse(returnstream.str());
    }	
	else if (request.GetNthToken(1)=="try") {
	  ostringstream returnstream;
	  returnstream<<"Querying...\n\n";
	  try {
		NetData nettest(request.GetNthToken(2));
		returnstream<<request.GetNthToken(2)<<"\t [passed]\n";
	  } catch (TestModuleException &tme) {
		returnstream<<request.GetNthToken(2)<<"\t [FAILED\n";
	  }
	  request.SetResponse(returnstream.str());
	}
	else if (!activated) Activate();
    else if (request.IsCommandEqual("net shutdown")) {
      Deactivate();
      request.SetResponse("currently a no-op");
    }
    else if (request.IsCommandEqual("net print")) {
      request.SetResponse(netobj->ToString());
    }
    else if (request.GetNthToken(1) == "list") {
      FrEntIdType ent_type = frEntTypeFromStr(request.GetNthToken(2).c_str());
      request.SetResponse(netobj->GetNetDatum(ent_type));
    }
    else if (request.GetNthToken(1) == "entitylist") {
      request.SetResponse(netobj->EntityList());
    }
    else if (request.GetNthToken(1) == "isLegal") {
      bool response;
      response =
        netobj->isLegal(request.GetNthToken(2),request.GetNthToken(3));
      if (response) request.SetResponse("neighbours");
      else request.SetResponse("not neighbours");
    }
    else if (request.GetNthToken(1)== "system") {
      request.SetResponse(netobj->SystemTest());
    }

    else
      request.SetBadCommand(myname);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("NetModule::HandleRequest()");
    request.SetResponse(tme.what());
  }
}
