#include "Nym.h"

/********************************************
 * Static functions to handle type conversions
 **/

FrRemoteNymParameter Nym::toFrRemoteNymParameter(const Nym::parm_id & id)
{
  FrRemoteNymParameter frid;
  try
    {
      switch( id ) {      
      case Version:              frid = NymVersion;              break;
      case ReplyBlock:           frid = NymReplyBlock;           break;
      case Name:                 frid = NymName;                 break;
      case LongName:             frid = NymLongName;             break;
      case EncryptionKey:        frid = NymEncryptionKey;        break;
      case SigningKey:           frid = NymSigningKey;           break;
      case Photo:                frid = NymPhoto;                break;
      case Capability:           frid = NymCapability;           break;
      case Sound:                frid = NymSound;                break;
      case Note:                 frid = NymNote;                 break;
      case Language:             frid = NymLanguage;             break;
      case Extension:            frid = NymExtension;            break;
      case Token:                frid = NymToken;                break;
      case TimeStamp:            frid = NymTimeStamp;            break;
      case LastModified:         frid = NymLastModified;         break;
      case Expiry:               frid = NymExpiry;               break;
      case AccessLevel:          frid = NymAccessLevel;          break;
      case EmailDomain:          frid = NymEmailDomain;          break;
      case Release:              frid = NymRelease;              break;
      case EmailLimit:           frid = NymEmailLimit;           break;
      case Description:          frid = NymDescription;          break;
      case BlockList:            frid = NymBlockList;            break;
      case PrivateReplyBlock:    frid = NymPrivateReplyBlock;    break;
      case Passphrase:           frid = NymPassphrase;           break;
      case ScanWord:             frid = NymScanWord;             break;
      case Friends:              frid = NymFriends;              break;
      case Rules:                frid = NymRules;                break;
      case Ports:                frid = NymPorts;                break;
      case PrivateEncryptionKey: frid = NymPrivateEncryptionKey; break;
      case PrivateSigningKey:    frid = NymPrivateSigningKey;    break;
      case TokenCache:           frid = NymTokenCache;           break;
      case PrivateEmailAddr:     frid = NymPrivateEmailAddr;     break;
      case NetworkPrefs:         frid = NymNetworkPrefs;         break;
      case PublicNote:           frid = NymPublicNote;           break;
      case PrivateNote:          frid = NymPrivateNote;          break;
      case SpamControl:          frid = NymSpamControl;          break;
      case Preferences:          frid = NymPreferences;          break;
      case SecuritySettings:     frid = NymSecuritySettings;     break;
      case CrossPostLimit:       frid = NymCrossPostLimit;       break;
      case Type:                 frid = NymType;                 break;
      case Smooshed:             frid = NymSmooshed;             break;
      case Nuked:                frid = NymNuked;                break;
      case Disabled:             frid = NymDisabled;             break;
      case Signature:            frid = NymSignature;            break;
      case Unknown:              frid = NymUnknown;              break;
      default:
        throw TestModuleException("Unsupported parm_id");
      }
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::toFrRemoteNymParameter()");
    throw;
  }
  return frid;
}



Nym::parm_id Nym::toParmId(const FrRemoteNymParameter & frid)
{
  Nym::parm_id id;
  try
    {
      switch( frid ) {      
      case NymVersion:              id = Version;              break;
      case NymReplyBlock:           id = ReplyBlock;           break;
      case NymName:                 id = Name;                 break;
      case NymLongName:             id = LongName;             break;
      case NymEncryptionKey:        id = EncryptionKey;        break;
      case NymSigningKey:           id = SigningKey;           break;
      case NymPhoto:                id = Photo;                break;
      case NymCapability:           id = Capability;           break;
      case NymSound:                id = Sound;                break;
      case NymNote:                 id = Note;                 break;
      case NymLanguage:             id = Language;             break;
      case NymExtension:            id = Extension;            break;
      case NymToken:                id = Token;                break;
      case NymTimeStamp:            id = TimeStamp;            break;
      case NymLastModified:         id = LastModified;         break;
      case NymExpiry:               id = Expiry;               break;
      case NymAccessLevel:          id = AccessLevel;          break;
      case NymEmailDomain:          id = EmailDomain;          break;
      case NymRelease:              id = Release;              break;
      case NymEmailLimit:           id = EmailLimit;           break;
      case NymDescription:          id = Description;          break;
      case NymBlockList:            id = BlockList;            break;
      case NymPrivateReplyBlock:    id = PrivateReplyBlock;    break;
      case NymPassphrase:           id = Passphrase;           break;
      case NymScanWord:             id = ScanWord;             break;
      case NymFriends:              id = Friends;              break;
      case NymRules:                id = Rules;                break;
      case NymPorts:                id = Ports;                break;
      case NymPrivateEncryptionKey: id = PrivateEncryptionKey; break;
      case NymPrivateSigningKey:    id = PrivateSigningKey;    break;
      case NymTokenCache:           id = TokenCache;           break;
      case NymPrivateEmailAddr:     id = PrivateEmailAddr;     break;
      case NymNetworkPrefs:         id = NetworkPrefs;         break;
      case NymPublicNote:           id = PublicNote;           break;
      case NymPrivateNote:          id = PrivateNote;          break;
      case NymSpamControl:          id = SpamControl;          break;
      case NymPreferences:          id = Preferences;          break;
      case NymSecuritySettings:     id = SecuritySettings;     break;
      case NymCrossPostLimit:       id = CrossPostLimit;       break;
      case NymType:                 id = Type;                 break;
      case NymSmooshed:             id = Smooshed;             break;
      case NymNuked:                id = Nuked;                break;
      case NymDisabled:             id = Disabled;             break;
      case NymSignature:            id = Signature;            break;
      case NymUnknown:              id = Unknown;              break;
      default:
        throw TestModuleException("Unsupported FrRemoteNymParameter");
      }
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::toFrRemoteNymParameter()");
    throw;
  }
  return id;
}



uint32_t Nym::parameterFlags(const Nym::parm_id & id)
{
  return frRemoteNymParameterFlags( toFrRemoteNymParameter(id) );
}



FrNymType Nym::toFrNymType(const Nym::nym_type & type)
{
  FrNymType frtype;
  try
    {
      switch( type ) {
      case TYPE_INIT:  frtype = FRNYM_TYPE_INIT;  break;
      case TYPE_ONE:   frtype = FRNYM_TYPE_ONE;   break;
      case TYPE_TWO:   frtype = FRNYM_TYPE_TWO;   break;
      case TYPE_THREE: frtype = FRNYM_TYPE_THREE; break;
      default:
        throw TestModuleException("Unsupported nym_type");
      }
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::toFrNymType()");
    throw;
  }
  return frtype;
}


Nym::nym_type Nym::toNymType(const FrNymType frtype)
{
  Nym::nym_type type;
  try
    {
      switch( frtype ) {
      case FRNYM_TYPE_INIT:  type = TYPE_INIT;  break;
      case FRNYM_TYPE_ONE:   type = TYPE_ONE;   break;
      case FRNYM_TYPE_TWO:   type = TYPE_TWO;   break;
      case FRNYM_TYPE_THREE: type = TYPE_THREE; break;
      default:
        throw TestModuleException("Unsupported FrNymType");
      }
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::toTypeId()");
    throw;
  }
  return type;
}


/*********************************************
 * Constructors & Destructors
 **/

Nym::Nym()
{
  try
    {
      if (!zkErrorNew(&err))
        throw TestModuleException("Couldn't allocate error structure!");

      if (!frRemoteNymNew(err,&m_nym))
        throw TestModuleException(err,"frRemoteNymNew()");

      m_ent.type = FRENT_TYPE_NYM;
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::Nym()");
    if (m_nym) frRemoteNymFree(err,&m_nym);
    if (err) zkErrorFree(&err);
    throw;
  }
}


Nym::Nym(const string & name)
{
  try
    {
      if (!zkErrorNew(&err))
        throw TestModuleException("Couldn't allocate error structure!");

      if (!frRemoteNymNew(err,&m_nym))
        throw TestModuleException(err,"frRemoteNymNew()");

      m_ent.type = FRENT_TYPE_NYM;
      setName( name );
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::Nym(name)");
    if (m_nym) frRemoteNymFree(err,&m_nym);
    if (err) zkErrorFree(&err);
    throw;
  }
}



Nym::Nym(const FrRemoteNym nym)
{
  try
    {
      if (!zkErrorNew(&err))
        throw TestModuleException("Couldn't allocate error structure!");

      m_nym = nym;

      m_ent.type = FRENT_TYPE_NYM;
      strncpy(m_ent.name, name().c_str(), MAX_NYM_NAME_SZ);
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::Nym(nym)");
    if (m_nym) frRemoteNymFree(err,&m_nym);
    if (err) zkErrorFree(&err);
    throw;
  }
}



Nym::~Nym()
{
  if (m_nym) frRemoteNymFree(err,&m_nym);
  if (err) zkErrorFree(&err);
}

void Nym::Clear() {

  if (m_nym) frRemoteNymFree(err,&m_nym);
  if (err) zkErrorFree(&err);
  try
    {
      if (!zkErrorNew(&err))
        throw TestModuleException("Couldn't allocate error structure!");
	  
      if (!frRemoteNymNew(err,&m_nym))
        throw TestModuleException(err,"frRemoteNymNew()");

      m_ent.type = FRENT_TYPE_NYM;
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::Clear()");
    if (m_nym) frRemoteNymFree(err,&m_nym);
    if (err) zkErrorFree(&err);
    throw;
  }
}


/*****************************************
 * Basic functions
 **/

string Nym::toString()
{
  string paramstr = string();
  const unsigned char * parambuf = NULL;
  size_t paramsz = 0;
  Nym::parm_id param = Version;
  ostringstream sstrm;

  try {

    resetParameterSearch();

    while (param < Unknown) {

      if ((parameterFlags(param) & FL_DATA_ASCII) || (param == Type))
        {
          // if we can't find the paramter, we'll just skip over it
          try {
            getParameter(param,&parambuf,&paramsz);
            paramstr = parameterToToken( param );

            sstrm << setw(PARAM_STR_MAX) << paramstr << " = ";

            // Special case for NymType Parameter
            if (param == Type) {
              sstrm << getTypeAsString();
            } else { sstrm << parambuf; }
            sstrm << "\n";
          } catch(...) {}
        }
      static_cast<int>(param) += 1;
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::toString()");
    throw;
  }
  return sstrm.str();
}



void Nym::verify()
{
  FrCryptPubKey sig = NULL;
  try
    {
      // Verify the nym's name
      verifyName();

      // Verify the private signature key
      sig = privateSigKey();
      KeyHandler * keyhnd = keyhnd->Instance();
      if (!keyhnd->VerifyPrivateKey(sig))
        throw TestModuleException("Nym's signing key failed verification");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::verify()");
    throw;
  }
}



/***************************************************
 * Parameter <-> String conversion
 **/

string Nym::parameterToToken(const Nym::parm_id & parm)
{
  const char * paramstr = NULL;
  try
    {
      if (!frRemoteNymParameterToToken(err,
                                       toFrRemoteNymParameter(parm),
                                       &paramstr))
        throw TestModuleException(err,"frRemoteNymParameterToToken()");
    }
  catch(TestModuleException &tme) {
    tme.AddScope("Nym::parameterToToken()");
    throw;
  }
  return string( paramstr );
}



Nym::parm_id Nym::tokenToParameter(const string & parmstr)
{
  FrRemoteNymParameter parm;
  try
    {
      if (!frRemoteNymTokenToParameter(err,parmstr.c_str(),&parm))
        throw TestModuleException(err,"frRemoteNymParameterToToken()");
    }
  catch(TestModuleException &tme) {
    tme.AddScope("Nym::tokenToParameter()");
    throw;
  }
  return toParmId( parm );
}



/********************************************
 * High level accessors
 **/

string Nym::name()
{
  const unsigned char * name = NULL;
  size_t namesz = 0;

  try
    {
      resetParameterSearch();
      getParameter(Name,&name,&namesz);
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::name()");
    throw;
  }
  return string( reinterpret_cast<const char *> (name) );
}



void Nym::setName(string name)
{
  const unsigned char * cname = NULL;
  try
    {
      deleteParameter(Name);
      cname = reinterpret_cast<const unsigned char *> (name.c_str());
      addParameter(Name, cname, name.length());
      strncpy(m_ent.name, name.c_str(), MAX_NYM_NAME_SZ);
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::setName()");
    throw;
  }
}



Nym::nym_type Nym::getType()
{
  FrNymType type;

  try
    {
      if (!frRemoteNymGetType(err, m_nym, &type))
        throw TestModuleException(err,"frRemoteNymGetType()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::getType()");
    throw;
  }
  return( toNymType(type) );
}



string Nym::getTypeAsString()
{
  string ret;
  try
    {
      switch( getType() ) {
      case TYPE_ONE:   ret = NYMTYPE_STR_PREMIUM;  break;
      case TYPE_TWO:   ret = NYMTYPE_STR_MAILONLY; break;
      case TYPE_THREE: ret = NYMTYPE_STR_STANDARD; break;
      default:
        throw TestModuleException("Unrecognized Nym Type");
      }
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::getTypeAsString()");
    throw;
  }
  return( ret );
}



void Nym::getPrivateKeys(FrCryptPubKey * cryption, FrCryptPubKey * signing)
{
  try
    {
      if (!frRemoteNymGetPrivateKeys(err,m_nym,cryption,signing))
        throw TestModuleException(err,"frRemoteNymGetPrivateKeys()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::getPrivateKeys()");
    throw;
  }
}



void Nym::setPrivateKeys(FrCryptPubKey cryption, FrCryptPubKey signing)
{
  FrCryptPubKey dupsig = 0;
  try
    {
      if (!frRemoteNymSetPrivateKeys(err, m_nym, cryption, signing))
        throw TestModuleException(err, "frRemoteNymSetPrivateKeys()");

      // Must set the private signature key *again* here. Stupid API.
      CryptoWrapper * crypto = crypto->Instance();
      dupsig = crypto->PubkeyDup(signing);
      if (!frRemoteNymSetPrivateSignatureKey(err, m_nym, dupsig))
        throw TestModuleException(err, "frRemoteNymSetPrivateSignatureKey()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::setPrivateKeys()");
    throw;
  }
}



FrCryptPubKey Nym::privateSigKey()
{
  FrCryptPubKey psk = NULL;
  try
    {
      if (!frRemoteNymPrivateSignatureKey(err, m_nym, &psk))
        throw TestModuleException(err, "frRemoteNymPrivateSignatureKey()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::privateSigKey()");
    throw;
  }
  return psk;
}



void Nym::setPublicKeys(FrCryptPubKey cryption, FrCryptPubKey signing)
{
  try
    {
      if (!frRemoteNymSetPubKeys(err, m_nym, cryption, signing))
        throw TestModuleException(err,"frRemoteNymSetPubKeys()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::setPublickKeys()");
    throw;
  }
}



void Nym::getPublicKeys(FrCryptCrypt * cryption, FrCryptSig * signing)
{
  try
    {
      if (!frRemoteNymGetKeys(err, m_nym, cryption, signing))
        throw TestModuleException(err,"frRemoteNymGetKeys()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::getPublicKeys()");
    throw;
  }
}



/**********************************************************
 * Low level accessors (thin wrapper around frnym library
 **/

void Nym::deleteParameter(const Nym::parm_id & parm)
{
  try
    {
      if (!frRemoteNymDeleteParameter(err,m_nym,toFrRemoteNymParameter(parm)))
        throw TestModuleException(err,"frRemoteNymDeleteParameter()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::deleteParameter()");
    throw;
  }
}



void Nym::addParameter(const Nym::parm_id & parm, 
                       const unsigned char * data,
                       size_t datasz)
{
  try
    {
      if (!frRemoteNymAddParameter(err,m_nym,toFrRemoteNymParameter(parm),
                                   data, datasz))
        throw TestModuleException(err,"frRemoteNymAddParameter()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::addParameter()");
    throw;
  }
}



void Nym::getParameter(const Nym::parm_id & parm,
                       const unsigned char ** data,
                       size_t * datasz)
{
  try
    {
      if (!frRemoteNymGetParameter(err,m_nym,toFrRemoteNymParameter(parm),
                                   data,datasz))
        throw TestModuleException(err,"frRemoteNymGetParameter()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::getParameter()");
    throw;
  }
}



uint32_t Nym::countParameter(const Nym::parm_id & parm)
{
  uint32_t count;
  try
    {
      if (!frRemoteNymCountParameter(err,m_nym,toFrRemoteNymParameter(parm),
                                     &count))
        throw TestModuleException(err,"frRemoteNymCountParameter()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::countParameter()");
    throw;
  }
  return count;
}



void Nym::resetParameterSearch()
{
  try
    {
      if (!frRemoteNymResetParameterSearch(err,m_nym))
        throw TestModuleException(err,"frRemoteNymResetParameterSearch()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::resetParameterSearch()");
    throw;
  }
}



void Nym::verifyName()
{
  try
    {
      if (!frRemoteNymVerifyName(err,m_nym))
        throw TestModuleException(err,"frRemoteNymVerifyName()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::verifyName()");
    throw;
  }
}



void Nym::verifyPublicKeys()
{
  try
    {
      if (!frRemoteNymVerifyPublicKeys(err,m_nym))
        throw TestModuleException(err,"frRemoteNymVerifyPublicKeys()");
    }
  catch (TestModuleException &tme) {
    tme.AddScope("Nym::verifyPublicKeys()");
    throw;
  }
}
