#ifndef NYMDATA_DEF
#define NYMDATA_DEF

#include "NymBuilder.h"

/*  Title:          NymManager
 *  Maintainerr(s): Sam Sanjabi and Troy McLelan.
 *  Description:    Put something here.
 *  ToDo:           - Take tokentype info from the NIQS for the token listing.
 *                  - Allow the scalper to be specified and then automatically
 *                    create tokens on demand.
 *                  - Allow the Nym to be upgraded/extended/modified.
 *                  - Cleaner support for multiple active nyms.
 *                  - Allow a single file to hold all the necessary nym info.
 *                  - Token verification, and stronger nym verification.
 *  Known Bugs:     None
 */

/*
 *  This class handles all the necessary Nym manipulation required by the test
 *  client.  It encapsulates the token handler and invokes that functionality
 *  where required.  It supports the nymtool style nyms, which have their
 *  private keys stored in special directories.
 */
class NymManager {
  map<string,unsigned short> TokenTypeID;
  map<string,unsigned short> SerialNumID;
  void initPrivateMaps();

public:
  NymManager();
  ~NymManager();

  bool isActive() { if (active) return true; return false; }

  string toString();
  string test(string); // self-test routine

  void updateNymBuilder() {
    nb->UpdateServerInfo();
  }

  void nymCreate(string, string);
  void nymExtendActive (unsigned char = 0);
  void nymDisableActive();
  void nymGetActive(FrEntIdExp &, FrCryptSig &);
  void nymKeySaveActive();
  void nymUnloadActive();
  void nymLoad(string);
  void nymLookup(string,Nym &);
  string nymDisplay();

private:
  // Forbidden operators
  NymManager(NymManager &) {
    throw BadOperatorException("NymManager copy");
  }
  NymManager& operator=(const NymManager&) {
    throw BadOperatorException("NymManager =");
  }
  void operator&() {
    throw BadOperatorException("NymManager &");
  }

  string runNymTests(string,string,bool*); // tests per nym type

  // Self-test methods
  bool nymCreateTest(string,string);
  bool nymLookupTest(string);
  bool nymExtendTest(bool);
  bool nymKeySaveTest();
  bool nymUnloadTest();
  bool nymLoadTest(string);
  bool nymDisableTest();

  ZkError err;
  Nym nym;
  NymBuilder * nb;
  bool active;

};

#endif /*NYMDATA_DEF*/
