#include "RouteBuilder.h"

namespace {
  const time_t ROUTE_LIFE = 3600;
};

Route * RouteBuilder::Create(FreedomCreatePacket & frpkt, LayerType layert,
                             string * hosts, uint16_t newaci, string nymtype) {

  try {
    
    NetData * niqshnd = niqshnd->Instance();
    ConfigManager * cfgmgr = cfgmgr->Instance();
    
    VersionType vers = cfgmgr->PktVersion();

    // This exception should be wrapped up inside the function call.
    
    if (!niqshnd->isValidRoute(layert, vers, hosts)) {
      string names = "";
      for (size_t i = 0; i < static_cast<size_t>(layert); i++) {
	names += hosts[i] + " ";
      }
      throw IllegalRouteException("Invalid route specified:  " +
				  names);
    }

    struct sockaddr_in addr;
    FrEntIdExp ent;
    memset(&addr, 0, sizeof(addr));
    niqshnd->GetAIP(hosts[0], &addr, &ent);

    struct rc_info rc_init_data;
    rc_init_data.layert = layert;
    rc_init_data.aci = newaci;
    rc_init_data.expiry = niqshnd->GetFreedomTime() + ROUTE_LIFE;
    rc_init_data.addr = addr;
	rc_init_data.version = vers;

    // Question:  Are we responsible for deleting the AIPkeys?
    for (size_t i = 0; i < static_cast<size_t>(layert); i++) {
      rc_init_data.hostlist[i] = hosts[i];
      rc_init_data.AipPubKeys[i] = GetPubKey(hosts[i]);
    }

    // This should be an auto_ptr...
    Route * rte;

    if (nymtype == "")
      rte = new Route;
    else {
      // Question:  Are we responsible for deleting the sigkey?
      FrEntIdExp tempent;
      FrCryptSig tempsig;
      GetNym(nymtype, tempent, tempsig);
      rte = new Route(tempent, tempsig);
    }    

    rte->BuildRouteCreatePacket(frpkt, rc_init_data);
    return rte;
  }
  catch (TestModuleException & tme) {
    tme.AddScope("RouteBuilder::Create()");
    throw;
  }
}

/*
 *
 */

Route * RouteBuilder::Extend(Route & rte, FreedomCreatePacket & frpkt,
                             LayerType layert, string * hosts, uint16_t aci) {
  try {
    
    NetData * niqshnd = niqshnd->Instance();
    ConfigManager * cfgmgr = cfgmgr->Instance();
    
    VersionType vers = cfgmgr->PktVersion();

    // This exception should be wrapped up inside the function call.
    
    if (!niqshnd->isValidRoute(layert, vers, hosts)) {
      string names = "";
      for (size_t i = 0; i < static_cast<size_t>(layert); i++) {
	names += hosts[i] + " ";
      }
      throw IllegalRouteException("Invalid route specified:  " +
				  names);
    }

    struct sockaddr_in addr;
    FrEntIdExp ent;
    memset(&addr, 0, sizeof(addr));
    niqshnd->GetAIP(hosts[0], &addr, &ent);

    struct rc_info rc_init_data;
    rc_init_data.layert = layert;
    rc_init_data.aci = aci;
    rc_init_data.expiry = niqshnd->GetFreedomTime() + ROUTE_LIFE;
    rc_init_data.addr = addr;

    // Question:  Are we responsible for deleting the AIPkeys?
    for (size_t i = 0; i < static_cast<size_t>(layert); i++) {
      rc_init_data.hostlist[i] = hosts[i];
      rc_init_data.AipPubKeys[i] = GetPubKey(hosts[i]);
    }

    Route * newrte = rte.BuildRouteExtendPacket(frpkt, rc_init_data);

    return newrte;
  }
  catch (TestModuleException & tme) {
    tme.AddScope("RouteBuilder::Create()");
    throw;
  }  
}

/*
 *  Fetches the public key for the given AIP.
 */

FrCryptCrypt RouteBuilder::GetPubKey(string host_name) const {

  try {
    FrEntIdExp aipent;
    struct sockaddr_in addr;
    NetData    * niqshnd = niqshnd->Instance();
    KeyHandler * keyhnd = keyhnd->Instance();
  
    niqshnd->GetAIP(host_name, &addr, &aipent);
    return keyhnd->keyServerQuery(0, FRCRYPT_KEYTYPE_CRYPTION, aipent);
  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetPubKey()");
    throw;
  }
}

/*
 *  Needs to be redone.
 */

void RouteBuilder::GetNym(string type, FrEntIdExp & nym_ent,
                          FrCryptSig & nym_sig) const {
  try {

    NymManager nymobj;
    nymobj.nymCreate( GenerateRandomString() , type );
    // Now that we have a Nym, let's just take the parts we need, and discard
    // the rest.
    nymobj.nymGetActive(nym_ent, nym_sig);

  }
  catch (TestModuleException &tme) {
    tme.AddScope("GetNym()");
    throw;
  }
}

/*
 *
 */

string RouteBuilder::ToString() const {
  return "";
}
