/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   Handles route creation, extension and destruction.
 *                 Similiar in nature to the DHNegotiator.
 *  Todo:          - Break the Create function into sub-functions.
 *                 - Add a SelfTest and the necessary functionality.
 *                 - Eliminate resource leaks on failure.
 *                 - Add thread support, if necessary.
 *  Known Bugs:   None.
 */

#ifndef ZK_AIP_RTEBLD_JJDFKSZK_DEF
#define ZK_AIP_RTEBLD_JJDFKSZK_DEF

#include "LinkManager.h"
#include "Route.h"
#include "NymManager.h"

/*
 *
 */

class RouteBuilder {
 public:
  RouteBuilder() {}
  ~RouteBuilder() {}
  Route * Create(FreedomCreatePacket & frpkt, LayerType layert, string * hosts,
                 uint16_t aci, string nymtype);
  Route * Extend(Route & rte, FreedomCreatePacket & frpkt, LayerType layert,
                 string * hosts, uint16_t aci);
  void Destroy(Route & rte);
  Route * Accept(FreedomPacket & frpkt);
  string ToString() const;
  string SystemTest(string & active, size_t numloops = 1);
 private:
  FrCryptCrypt GetPubKey(string) const;
  void GetNym(string, FrEntIdExp&, FrCryptSig&) const;
};

class IllegalRouteException : public TestModuleException {
 public:
  IllegalRouteException(string msg_in) {SetMessage(msg_in);}
  ~IllegalRouteException() {}
 private:
};

#endif
