#include "RouteManager.h"

RouteManager * routemgr;

RouteManager * RouteManager::Instance() {
  if (!routemgr)
    routemgr = new RouteManager;
  return routemgr;
}

/*
 *
 */

void RouteManager::AddRoute(Route * rte) {

  try {
    Lock();
    if (routemap[rte->GetACI()])
      throw TestModuleException("Route already registered on aci");
    routemap[rte->GetACI()] = rte;
    Unlock();
  }
  catch (TestModuleException & tme) {
    Unlock();
    tme.AddScope("AddRoute()");
    throw;
  }
}

/*
 *
 */

Route* RouteManager::RemoveRoute(uint16_t aci) {

  try {
    Lock();
    if (routemap[aci] == 0) {
      routemap.erase(aci);
      Unlock();
      return 0;
    }
    Route * oldrte = routemap[aci];
    routemap.erase(aci);
    Unlock();
    return oldrte;
  }
  catch (TestModuleException & tme) {
    Unlock();
    tme.AddScope("RemoveRoute()");
    throw;
  }  
}

Route * RouteManager::GetRoute(uint16_t aci) {

  Route * rte = NULL;
  Lock();
  if (routemap[aci] == 0)
    routemap.erase(aci);
  else
    rte = routemap[aci];
  Unlock();
  return rte;
}

/*
 *
 */

string RouteManager::ToString() {

  ostringstream returnstream;
  TestRoute trte;
  for (map<uint16_t, Route*>::iterator i = routemap.begin(); i!=routemap.end();++i)
    returnstream << trte.ToString(*(i->second));
  return returnstream.str();
}

/*
 *
 */

void RouteManager::OutgoingPacket(FreedomDataPacket & outgoingpkt) {
  try {
    Lock();
    if (routemap[outgoingpkt.ACI()] == 0) {
      routemap.erase(outgoingpkt.ACI());
      throw TestModuleException("Route not found");
    }
    try {
      routemap[outgoingpkt.ACI()]->EncryptPacket(outgoingpkt);
    }
    catch (TestModuleException &tme) {
      /*BUG? the assumption is that the route has become corrupted and
	should be destroyed*/
      RemoveRoute(outgoingpkt.ACI());
      throw;
    }
    Unlock();
  }
  catch (TestModuleException & tme) {
    Unlock();
    tme.AddScope("OutgoingPacket");
    throw;
  }
}

/*
 *
 */

FreedomDataPacket * RouteManager::IncomingPacket(FreedomPacket & incomingpkt) {
  try {
    
    if (incomingpkt.IsCreate()) {
      FreedomCreatePacket rcpkt(incomingpkt);
      Route * newrte = new Route(rcpkt, MyPrivKey);
      AddRoute(newrte);
      return NULL;
    }

    Lock();

    if (routemap[incomingpkt.ACI()] == 0) {
      routemap.erase(incomingpkt.ACI());
      throw TestModuleException("Route not found");
    }
    if (incomingpkt.IsData()) {
      FreedomDataPacket * datapkt;

      // This is a hack.  It should be removed once we have determined the
      // proper place to perform packet conversion.
      static_cast<FreedomPacket*>(datapkt) =
        routemap[incomingpkt.ACI()]->DecryptPacket(incomingpkt);
      Unlock();
      return datapkt;
    }
    Unlock();
    if (incomingpkt.IsDestroy()) {
      RemoveRoute(incomingpkt.ACI());
      return NULL;
    }
    if (incomingpkt.IsAck()) {
      routemap[incomingpkt.ACI()]->Establish();
      return NULL;
    }
    throw TestModuleException("Unknown packet");
  }
  catch (TestModuleException & tme) {
    Unlock();
    tme.AddScope("IncomingPacket");
    throw;
  }  
}
