/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   Adds, removes and updates routes via an internal mapping;
 *                 based on aci.  Also encrypts and decrypts packets.
 *  Todo:          - 
 *  Known Bugs:   Fix RemoveRoute() so it actually does something.
 */

#ifndef ZK_AIP_RTEMAN_JJDFKSZK_DEF
#define ZK_AIP_RTEMAN_JJDFKSZK_DEF

#include "RouteBuilder.h"

/*
 *
 */

class RouteManager {
 public:
  static RouteManager * Instance();
  RouteManager() { pthread_mutex_init(&route_lock, 0); }
  ~RouteManager() { pthread_mutex_destroy(&route_lock); }
  void AddRoute(Route * rte);
  Route* RemoveRoute(uint16_t aci);
  uint16_t UpdateRoute(uint16_t aci);
  Route * GetRoute(uint16_t aci);
  void OutgoingPacket(FreedomDataPacket & outgoingpkt);
  FreedomDataPacket * IncomingPacket(FreedomPacket & incomingpkt);
  string ToString();
 private:

  RouteManager(RouteManager &);
  RouteManager& operator=(const RouteManager&);
  void operator&();

  void Lock() { pthread_mutex_lock(&route_lock); }
  void Unlock() { pthread_mutex_unlock(&route_lock); }
  pthread_mutex_t route_lock;
  map<uint16_t, Route*> routemap;

  // For AIP simulation.
  FrCryptPubKey MyPrivKey;
};

extern RouteManager * routeman;

#endif
