/*  Maintainer(s): Alice Nodelman and Troy McLelan
 *  Description:   This object interfaces with the shim.  It was created to
 *                 allow a cleaner separation of that code from unrelated
 *                 routing code.
 *  Todo:          - Add other rules.
 *                 - Clear rules on first instantiation.
 *                 - Enforce function call dependancies.
 *                 - Add const where appropriate.
 *                 - Eliminate resource leaks on failure.
 *                 - Add thread support, if necessary.
 *  Known Bugs:   None.
 */

#ifndef ZK_AIP_SHIMHND_JJDFKSZK_DEF
#define ZK_AIP_SHIMHND_JJDFKSZK_DEF

#include "CryptoWrapper.h"

#include <list>

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/poll.h>
#include <zkshim.h>

/*
 *
 */

class ShimHandler {
 public:
  static ShimHandler * Instance();
  ~ShimHandler();

  int GetShimFD() { return shimfd; }
  void BlockPort(uint16_t);
  void UnblockPort(uint16_t);

  bool CheckForPort(uint16_t);

  string ToString();
  string SelfTest(size_t numloops = 1);

  // We want the ctor to be private, but gcc whines...
  ShimHandler();
 private:

  // To prevent inadvertant copying.  Although we could support them, they
  // could be a hidden performance drain.
  
  ShimHandler(ShimHandler &)
    {throw BadOperatorException("ShimHandler copy");}
  ShimHandler& operator=(const ShimHandler&)
  {throw BadOperatorException("ShimHandler =");}
  void operator&()
    {throw BadOperatorException("ShimHandler &");}

  bool TestBlockAndUnblock(size_t numloops);
  bool TestPortBlocking(size_t numloops);
  void SetRule(uint16_t, int);
  bool UsingClientShim;
  list<uint16_t> blocked_ports;
  int shimfd;
};

extern ShimHandler * shimptr;

class ShimException : public TestModuleException {
 public:
  ShimException(string msg_in) {SetMessage(msg_in);}
  ~ShimException() {}
 private:
};

class PortInuseException : public TestModuleException {
 public:
  PortInuseException(string msg_in) {SetMessage(msg_in);}
  ~PortInuseException() {}
 private:
};

#endif
