#include "TestModule.h"
map<string, TestModule*> reg;

/*
 *
 */
TestModule::TestModule() {
  activated = false;
  helpmessage = "help:\t\t\tThis message.\n";
  helpmessage += "list:\t\t\tA list of all modules\n";
  helpmessage += "exit:\t\tEnd your session \n";
  helpmessage += "<modulename> help:\tLists all the commands for that module\n";
}
/*
 *
 */
TestModule::~TestModule() {
  Deactivate();
}
/*
 *
 */
void TestModule::HandleRequest(UserRequest &request) {
  try {
    string token = "";

    Activate();
    token = request.GetModuleName();
    if (token == "help")
      request.SetResponse(helpmessage);
    else if (token == "exit") {
      Deactivate();
      throw EndSession();
    }
    else if (token == "list") {
      request.SetResponse(ListAllModules());
    }
    else {
      TestModule * tempmod = 0;
      tmmap::iterator i = reg.find(token);
      if (i==reg.end()) {
        request.SetBadCommand("UnknownModule");
      }
      else {
        tempmod = i->second;
        tempmod->HandleRequest(request);
      }
    }
  }
  catch (TestModuleException &tme) {
    request.SetResponse(tme.what());
    throw;
  }
}
/*
 *
 */
void TestModule::Activate() {
  activated = true;
}
/*
 *
 */
void TestModule::Deactivate() {

  if (activated) {
    for (tmmap::iterator i = reg.begin(); i != reg.end(); ++i)
      if (i->second!=this) (i->second)->Deactivate();
    activated = false;
  }
}
/*
 *
 */
string TestModule::MyName() const {
  return myname;
}
/*
 *  This method returns a pointer to the requested module, or a pointer to the
 *  default module if the requested module is not found.  This is done so that
 *  the subsequent request can be directed to the default module if it is
 *  global, or a bad command error can be set.
 */
TestModule * TestModule::GetModule(string modname) {

  TestModule * tempmod;
  tempmod = reg[modname];
  if (tempmod == 0)
    tempmod = this;
  return tempmod;
}
/*
 *  Iterates thru the registry and builds a string of all the module names.
 */
string TestModule::ListAllModules() {

  string modulelist;
  for (tmmap::iterator i = reg.begin(); i != reg.end(); ++i)
    modulelist += (i->first + "\n");

  string::size_type k = modulelist.find_last_of('\n');
  if (k!=string::npos) modulelist.erase(k,k);
  
  return modulelist;
}
