#ifndef _ZK_TROYBNITH_H
#define _ZK_TROYBNITH_H
/*  Program:         Test Module Base Definition
 *  Author(s):       Troy McLelan
 *  Description:     This is the base definition for all test modules.  They
 *                   must all conform to the interface specified here.
 *  ToDo:            Lots.
 *  Known Bugs:      None.
 */
#include "UserRequest.h"
/*
 *  The base class for any test module.
 */
class TestModule {
 public:
  // The constructor should do 3 things:
  // 1 - Set the name for the module.
  // 2 - Register the module in the registry, throwing an exception if there
  //     is already a module registered.
  // 3 - Flag the module as uninitialized.
  //  NICOLAS SUGGESTION:  move map insertion into this constructor.
  TestModule();
  // The destructor should unmap the module from the registry.
  virtual ~TestModule();
  // This is the main function, it should accept all legal commands, including
  // a help command and a shutdown command.  It should also handle any bad
  // commands by invoking the bad command method of the UserRequest class.  It
  // must check for initialization, and run Initialize() if it has not been
  // run already.
  virtual void HandleRequest(UserRequest &);
  // This method should only execute if the initialized flag is false.  It is
  // responsible for acquiring resoures for the test module.  It must then set
  // the initialized flag to true.
  virtual void Activate();
  // This method should only execute if the initialized flag is true.  It is
  // responsible for freeing all resources.  It must then set the initialized
  // flag to false.
  virtual void Deactivate();
  // Must return the name of the module exactly as it is in the registry.
  //  NICOLAS SUGGESTION:  Make this pure virtual.
  virtual string MyName() const;
 protected:
  //  NICOLAS SUGGESTION:  Move this functionality into another module.
  string ListAllModules();
  TestModule * GetModule(string);
  string myname;
  bool activated;
  string helpmessage;
  // There should be no copying or copy-assignment.  This will stop us from
  // doing something that we don't want to do.
  TestModule(TestModule &) {throw BadOperatorException("TestModule copy");}
  TestModule& operator=(const TestModule&) {throw BadOperatorException("TestModule =");}
  void operator&() {throw BadOperatorException("TestModule &");}
};

// A global variable, until I figure out a better way...
extern map<string, TestModule*> reg;
// A typedef to make this more manageable.
typedef map<string, TestModule*> tmmap;
#else
#warning TestModule.h redefined.
#endif

