#include "TestModuleException.h"
/*
 *  Parameterized constructor that sets the error message field.
 */
TestModuleException::TestModuleException(string errmsg_in) {
  errmsg.clear();
  errmsg = errmsg_in;
}
/*
 *  Parameterized constructor that sets the error number field.  It is assumed
 *  that this number corresponds to a standard error. 
 */

TestModuleException::TestModuleException( int error_number,string errmsg_in) {
  errmsg.clear();
  errmsg = strerror(error_number);
  AddScope(errmsg_in);
}

/*
 *  Parameterized constructor that handles ZkError structures.
 */
TestModuleException::TestModuleException(ZkError err, string funcname) {
  errmsg.clear();
  char zkerrmsg[256];
  zkErrorMessage(err, zkerrmsg, sizeof(zkerrmsg));
  errmsg = zkerrmsg;
  AddScope(funcname);
}
/*
 *  Adds scope parameter.
 */
void TestModuleException::AddScope(string callername) {
  errmsg += "\n\t";
  errmsg += callername;  
}
/*
 *  Prints the error to a string which is returned to the caller
 */
string TestModuleException::what() {
  return errmsg;
}
/*
 *  Allows deriving classes to set the error message.
 */
void TestModuleException::SetMessage(string msg_in) {
  errmsg = msg_in;
}
