#ifndef TEST_MODULE_SET_HYERNBSF_H
#define TEST_MODULE_SET_HYERNBSF_H
/*  Program:         Test Module Set.
 *  Author(s):       Troy McLelan
 *  Description:     This is the set of all available test modules, which in
 *                   turn must be defined here in order to be registered.  All
 *                   modules must inherit from the TestModule parent class.
 *  ToDo:            Lots.
 *  Known Bugs:      None.
 */
#include "TestModule.h"

#include <list>
#include "ShimHandler.h"
#include "RouteBuilder.h"
#include "LinkManager.h"
#include "NymManager.h"
#include "Client.h"
/*
 *  The token test module. Interfaces with tokenserver and scalper
 */
class TokenModule : public TestModule {
 public:
  TokenModule(); 
  ~TokenModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  TokenModule(const TokenModule &);
  TokenModule& operator=(const TokenModule&);
  void operator&();
  TokenHandler * tokhnd;
};

class TestingModule: public TestModule {
public:
  TestingModule();
  ~TestingModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
private:
  TestingModule(const TestingModule &);
  TestingModule& operator=(const TestingModule&);
  void operator&();
};

/*
 *  The network test module.  Mostly interfaces with the NIQS and KQD.
 */
class NetModule : public TestModule {
 public:
  NetModule(); 
  ~NetModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  NetModule(const NetModule &);
  NetModule& operator=(const NetModule&);
  void operator&();
  NetData * netobj;
};
/*
 *  The nym test module.  Mostly interfaces with the nym and token servers
 */
class NymModule : public TestModule {
 public:
  NymModule(); 
  ~NymModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  NymModule(const NymModule &);
  NymModule& operator=(const NymModule&);
  void operator&();
  NymManager * nymobj;
};
/*
 *  The key test module.
 */
class KeyModule : public TestModule {
 public:
  KeyModule(); 
  ~KeyModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  KeyModule(const KeyModule &);
  KeyModule& operator=(const KeyModule&);
  void operator&();
  KeyHandler * keyhnd;
};
class ClientModule : public TestModule {
 public:
  ClientModule(); 
  ~ClientModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  ClientModule(const ClientModule &);
  ClientModule& operator=(const ClientModule&);
  void operator&();
  Client * client;
};

class DHModule : public TestModule {
 public:
  DHModule(); 
  ~DHModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  DHModule(const DHModule &);
  DHModule& operator=(const DHModule&);
  void operator&();
};
/*
 *  The configuration module.
 */
class ConfigModule : public TestModule {
 public:
  ConfigModule(); 
  ~ConfigModule();
  void HandleRequest(UserRequest &);
  void Activate();
  void Deactivate();
 private:
  ConfigModule(const ConfigModule &);
  ConfigModule& operator=(const ConfigModule&);
  void operator&();
  ConfigManager * cfghnd;
};

#else
#warning TestModuleSet.h redefined.
#endif
