#include "TestModuleSet.h"
/*
 *
 */
TestingModule::TestingModule() {

  myname = "test";
  if (reg[myname] != 0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  activated = false;
  helpmessage = "test help\n";
  helpmessage += "test self <num iterations>\n";
  helpmessage += "test system <num iterations>\n";
  helpmessage += "test regress\n";
}
/*
 *  Deactivates in case it has not happened yet, then explicitly deregisters
 *  the module.
 */
TestingModule::~TestingModule() {
  Deactivate();
  reg.erase(reg.find(myname));
}

/*
 *  does nothing for now.
 */
void TestingModule::Activate() {

  activated = true;

}
/*
 *
 */
void TestingModule::Deactivate() {

  activated = false;
}
/*
 *
 */
void TestingModule::HandleRequest(UserRequest & request) {

  try {
    Activate();
    if (request.IsCommandEqual("test help"))
      request.SetResponse(helpmessage);
    else if (request.IsCommandEqual("test shutdown")) {
      Deactivate();
      request.SetResponse(myname + " module shutdown\n");
    }
    else if (request.GetNthToken(1)=="system") {
      ostringstream returnstream;
      int iterations = atoi(request.GetNthToken(2).c_str());
      returnstream << "\nRunning global SystemTest()\n";

      if (iterations == 0)
        iterations = 1;

      NetData* net = net->Instance();
      returnstream << "Running NetData SystemTest():\n";
      returnstream << net->SystemTest()<<endl;

      KeyHandler * keytest = keytest->Instance();
      returnstream << "Running KeyHandler SystemTest():\n";
      returnstream << keytest->system() << endl;

      TokenHandler tokentest;
      returnstream << "Running TokenHandler SystemTest():\n";
      returnstream << tokentest.test() << endl;

      NymManager nymtest;
      returnstream << "Running NymManager SystemTest():\n";
      returnstream << nymtest.test(GenerateRandomString()) << endl;

      request.SetResponse(returnstream.str());
    }
    else if (request.GetNthToken(1) == "self") {
      ostringstream returnstream;
      int iterations = atoi(request.GetNthToken(2).c_str());
      if (iterations == 0) 
        iterations = 1;
      returnstream << "\nGlobal Selftest Results\n";

      // i can run selftest directly on the active cryptowrapper
      // because the wrapper has no state that i can effect (as far as
      // i can tell...)
	  
      returnstream << "CryptoWrapper SelfTest" << endl;
      CryptoWrapper * crypto = crypto->Instance();
      returnstream << crypto->SelfTest(iterations) << endl;
	  
	  
      returnstream << "ShimHandler SelfTest" << endl;
      try {
        ShimHandler * shim = shim->Instance();
        returnstream << shim->SelfTest(iterations) << endl;
      }
      catch (ShimException &se) {
        returnstream <<"Failed to run ShimHandler SystemTest()\n";
        returnstream << se.what()<<"\n";
      }
      
	  
      TestFreedomPacket tfrpkt;
      returnstream << tfrpkt.UnitTest(iterations) << endl;
	  
      
      TestFreedomCreatePacket tcreatepkt;
      returnstream << tcreatepkt.UnitTest(iterations) << endl;
	  
      
      TestFreedomDataPacket tdatapkt;
      returnstream << tdatapkt.UnitTest(iterations) << endl;
	  
	  
      
      DHNegotiator dh;
      returnstream << dh.SelfTest(iterations) << endl;
	  
	  
	  /*
      TestRoute troute;
      returnstream<< troute.UnitTest(iterations) <<endl;
	  */

      request.SetResponse(returnstream.str());
    }
    else request.SetBadCommand(myname);

  }
  catch (TestModuleException &tme) {
    tme.AddScope("TestingModule::HandleRequest()");
    throw;
  }
}
