#ifndef TOKENHANDLER_DEF
#define TOKENHANDLER_DEF

#include "KeyHandler.h"
#include "frToken.h"
#include "frTokenCache.h"
#include "frTokenClient.h"
#include "zkURL.h"

namespace
{
  // Token Type ID's
  unsigned short TOKTYPE_PREMIUM         = 30;
  unsigned short TOKTYPE_MAILONLY        = 31;
  unsigned short TOKTYPE_STANDARD_CREATE = 32;
  unsigned short TOKTYPE_STANDARD_EXTEND = 33;

  unsigned short TOKTYPE_PREMIUM_ONEDAY         = 80;
  unsigned short TOKTYPE_MAILONLY_ONEDAY        = 81;
  unsigned short TOKTYPE_STANDARD_CREATE_ONEDAY = 82;
  unsigned short TOKTYPE_STANDARD_EXTEND_ONEDAY = 83;

  // Serial Number ID's
  unsigned short SNID_PREMIUM         = 500;
  unsigned short SNID_MAILONLY        = 501;
  unsigned short SNID_STANDARD_CREATE = 502;
  unsigned short SNID_STANDARD_EXTEND = 503;
  
  unsigned short SNID_PREMIUM_ONEDAY         = 900;
  unsigned short SNID_MAILONLY_ONEDAY        = 901;
  unsigned short SNID_STANDARD_CREATE_ONEDAY = 902;
  unsigned short SNID_STANDARD_EXTEND_ONEDAY = 903;
}

class TokenHandler {
public:

  TokenHandler();
  ~TokenHandler();
  TokenHandler * Instance();
  string toString();
  string test();
  void config();

  void tokenCacheSave();
  void tokenCacheLoad();
  string tokenCacheList();
  string tokenCacheGet(unsigned short);
  string snGetFromScalper(unsigned short);
  void snRedeem(string);

 private:
  TokenHandler(TokenHandler &);
  TokenHandler& operator=(const TokenHandler&);
  void operator&();

  // Error structure
  ZkError err;

  // Token cache structure
  FrTokenCache tk;

  // Token Server Structure
  TokenServer tokensvr;

  // Run tests for a given nym type
  string runTokenTests(unsigned short,unsigned short,bool*);

  // Self-test methods
  bool tokenSerialNumberScalpTest(unsigned short,string *);
  bool tokenSerialNumberRedeemTest(string);
  bool tokenTokenCacheSaveTest();
  bool tokenTokenCacheLoadTest();
  bool tokenTokenCacheGetTest(unsigned short type);

};
#else

#endif /*TOKENHANDLER_DEF*/

