#include "TestModuleSet.h"
/*
 *
 */
TokenModule::TokenModule() {

  myname = "token";
  if (reg[myname] != 0)
    throw TestModuleException("Previous registry entry for " + myname);
  reg[myname] = this;
  helpmessage =  "token help\ntoken shutdown\ntoken print\n";
  helpmessage += "token selftest\n";
  helpmessage += "token list\n";
  helpmessage += "token snredeem <SERIAL-NUMBER>\n";
  helpmessage += "token snscalp <SERIAL-NUMBER-ID>\n";
  activated = false;
}
/*
 *  Calls cleanup incase it was not called yet, then explicitly deregisters
 *  the module.
 */
TokenModule::~TokenModule() {
  Deactivate();
  reg.erase(reg.find(myname));
}
/*
 *  Allocates the memory for the NetData object.
 */
void TokenModule::Activate() {
  if (!activated) {
    tokhnd = tokhnd->Instance();
    activated = true;
  }
}
/*
 *
 */
void TokenModule::Deactivate() {
  if (activated) {
    activated = false;
  }
}
/*
 *
 */
void TokenModule::HandleRequest(UserRequest & request) {

  string shmsg = string();
  try
    {
      this->Activate();


      // Print the help message
      if (request.IsCommandEqual("token help")) {
        shmsg = helpmessage;
      }


      // Shut down the token module
      else if (request.IsCommandEqual("token shutdown")) {
        this->Deactivate();
        shmsg = myname + " module shutdown\n";
      }


      // Print the TokenHandler structure's parameters
      else if (request.IsCommandEqual("token print")) {
        shmsg = tokhnd->toString();
      }


      // Self test
      else if (request.GetNthToken(1) == "selftest") {
        shmsg = tokhnd->test();
      }
      
      // Print the current token cache to the screen
      else if (request.GetNthToken(1) == "list") {
        shmsg = tokhnd->tokenCacheList();
      }


      // Load a nym (make it the active nym, requires keys)
      else if (request.GetNthToken(1) == "snredeem") {
        tokhnd->snRedeem(request.GetNthToken(2));
        shmsg = "Serial number " 
              + request.GetNthToken(2) + " successfully redeemed";
      }


      // Unload the active nym
      else if (request.GetNthToken(1) == "snscalp") {
        shmsg = tokhnd->snGetFromScalper(atoi(request.GetNthToken(2).c_str()));
      }


      // if there is a shell-message, then the command was valid
      if ( shmsg.empty() )
        request.SetBadCommand(myname);
      else
        request.SetResponse( shmsg );

    }

  catch (TestModuleException &tme) {
    tme.AddScope("TokenModule::HandleRequest()");
    request.SetResponse(tme.what());
  }

}
