#include "UserRequest.h"
/*
 *  Constructor, sets the internal strings to empty strings.
 */
UserRequest::UserRequest() {
  response = "";
  usercommand = "";
  modname = "";
}
/*
 *  Accepts a command from the caller and copies it to the local variable.
 *  It then sets the module name variable to represent the first word.  This
 *  is one of the main parsing functions and as such should be made more
 *  robust.  For starters, the command string passed in should be converted to
 *  lower case.
 */
void UserRequest::SetCommand(string command_in) {

  usercommand.clear();
  modname.clear();
  response.clear();
  vs.clear();
  response = "";
  usercommand = "";
  modname = "";

  usercommand = command_in;

  //populate the vector of string tokens, now it should be child's play to
  //access each token.  we hope.
  istringstream iss(usercommand);
  istream_iterator<string> iss_iter(iss), end;
  copy(iss_iter, end, back_inserter(vs));

  modname = GetNthToken(0);

}
/*
 *  Returns a copy of the module name.
 */
string UserRequest::GetModuleName() const {
  return modname;
}
/*
 *  Returns a boolean value that specifies whether the command passed in
 *  matches that inside the object.  Note that this is case sensitive, so
 *  both commands should be in lower case.
 */
bool UserRequest::IsCommandEqual(string compare_str) const {

  return (usercommand == compare_str);
}
/*
 *  To be used for parsing command strings, returns the nth token (word) in
 *  the command string, where nth is a value passed in by the caller. 
 */
string UserRequest::GetNthToken(unsigned int nth) {

  if (nth >= vs.size()) return "notoken";
  else return vs[nth];

}
/*
 *  Clears the old response and sets a new one given the caller's input.
 */
void UserRequest::SetResponse(string responsestr) {

  ConfigManager * config = config->Instance();
  string outfile = config->OutputFile();
  response.clear();
  try {
    if (outfile=="toscreen") {
      response = responsestr;
    }
    else {
      response = "Output written to file "+outfile+"\n";
      ofstream out;
      outfile = config->BaseDir() +"/"+ outfile;
      out.open(outfile.c_str(),ios_base::app);
      if (out.good()) {
	time_t curtime;
	curtime = time(NULL);
	if (curtime==(time_t)-1) 
	  throw TestModuleException("Bad time value");
	string date = asctime(localtime(&curtime));
	out<<endl<<"*****"<<date<<endl;
	out<<responsestr;
      }
      else
	throw TestModuleException("Unable to save to file");
    }
  }
  catch (TestModuleException &tme) {
    tme.AddScope("SetResponse()");
    throw;
  }
}
/*
 *  Sets a standard bad command message that includes the command and the
 *  handler's name.  The name must be specified in in case it was the cause of
 *  the error.  It would be good to combine this with the SetResponse()
 *  method in case we chose to have response signalling later.
 */
void UserRequest::SetBadCommand(string handlername) {

  response.clear();
  response = handlername;
  response.append(" couldn't handle command:  ");
  response.append(usercommand);
}
/*
 *  Returns a copy of the response string to the caller.
 */
string UserRequest::GetResponse() const {

  return response;
}
