use strict;
use Support;

#
# a simple perl object to store information about host entities
#

package HostEnt;

sub new {

  my $self={};
  $self->{hostName} = "";
  $self->{fqdn} = "";
  $self->{region} = "";
  $self->{servop} = "";
  $self->{ip} = "";

  bless($self);
}


sub printEnt {
  my ($self, $fh) = @_;

  print $fh "entity add host $self->{hostName} $self->{fqdn} $self->{ip} $self->{region} $self->{servop} 1\n";
}


# a routine to collect information on hosts from fr-getaips (at least until we 
# get the information in the niqs_test program
#
# $niqsAddr - the address of the niqs to contact
# $hostMap  - reference to the map containing the preliminary host information read 
#             from niqs_test
#

sub collectHostInfo {
  my ($niqsAddr, $hostMap) = @_;

  open(frgetaipsFH,"fr_getaips --niqs.hostname $niqsAddr --long |") || die "Can't run ft_getaips - is it in your path?\n";
  
  # skip the first two lines
  my $dummy = <frgetaipsFH>;
  $dummy= <frgetaipsFH>;
  my $foundAips = 0;
  while(<frgetaipsFH>) {
    my @args = split(/\s*\t+/);
    
    $foundAips = 1;
    my ($id, $name, $status, $rate, $hostname, $address, $region,$servop) = @args;
    if ($hostname =~ /\(unresolved\)/) {
      $hostMap->{$name}->{fqdn} = $name . ".aip.freedom.net";
    } else {
      $hostMap->{$name}->{fqdn} = $hostname;
    }
    my ($fips, @rest) = split(/\s+/,$region);
    $hostMap->{$name}->{region} = $fips;
#    print "ID : $id :: $name :: $hostname :: $hostMap->{$name}->{fqdn} ::  $fips\n";
  }
  
  if (!$foundAips) {
    die("Error: Unable to get information from fr_getaips. \n" .
	"Ensure that fr_getaips is on your path and that it is the proper version\n" .
	"for the network you are using\n");
  }

  close(frgetaipsFH);


}

#
# a simple perl object to store information about aip entities
#

package AipEnt;


sub new {

  my $self = {};
  $self->{entName} = "";
  $self->{entID} = "";
  $self->{hostName} = "";
  $self->{tcpport} = "";
  $self->{udpport} = "";
  $self->{neighbours} = [];
  bless($self);
}

sub printEnt {
  my ($self,$fh) = @_;

  
  print $fh "entity add aip $self->{entName} $self->{hostName} $self->{tcpport} $self->{udpport} 30 3600 1\n";
  print $fh "entity add maip $self->{entName} $self->{hostName} 51112\n";
#  print $fh "echo neighbours are ", join(", ", @{$self->{neighbours}}), "\n";
}


# code to collect all information about the aips.
# Also collects some info about the hosts
#
# $inFH, and $outFH are expected to be read and write pipes pointing at an
# active niqs_test.  
#
# $aipEntList - a reference to a list to be filled in with references to AipEnts
# $aipByID - a reference to a list to be populated with aips based on their ID
# $hostMap - a reference to a map to be populated with host entries, keyed by hostname
# $hostIPMap - a reference to a map to be populated with host entries, keyed by ip addr
#
#

sub collectAipInfo {
  my ($inFH, $outFH, $aipEntList, $aipByID, $hostMap, $hostIPMap) = @_;

  # Get the list of aips, and then get information on each aip
  
  print $outFH "list aip\n";
  
  my $val = Support::readOutput($inFH);
  
  my @aipIDList = split(/\n/,$val);
  #remove the first and last entries (for the command and the prompt)
  shift(@aipIDList);
  pop(@aipIDList);
  foreach my $aip (@aipIDList) {
    my @aipDesc = split(/[\s\/]+/,$aip);
    my $aipEnt = new AipEnt;
#    print "Aip is : $aipDesc[0] $aipDesc[1] $aipDesc[2]\n";
    
    $aipEnt->{entID} = int($aipDesc[0]);
    $aipEnt->{entName} = $aipDesc[2];
    # HACK!!!  For the moment, assume that the host entity name is the same as the
    # aip entity name, since we don't have info on the host entity associated with the aip
    $aipEnt->{hostName} = $aipDesc[2];
    push(@$aipEntList,$aipEnt);
    
    
  }
  
  
  # do a query description for each aip, and collect all of the data we need.
  
  foreach my $aip (@$aipEntList) {
    
    print $outFH "query description aip $aip->{entName}\n";
    
    my $hostEnt = new HostEnt;
    #  HACK : really shouldn't assume that the host entity name == aip entity name
    $hostEnt->{hostName} = $aip->{entName};
    
    my $resp = Support::readOutput($inFH);
    
    my @respList = split(/\n/,$resp);
    
    # remove the first and last lines:
    pop(@respList);
    shift(@respList);
    
#    print "Response from $aip->{entName} is\n$resp";
    
    my $saveNeighbour = 0;
    foreach $resp(@respList) {
      if ($resp =~ /ipaddr:main_port:dh_port = ([\d\.]+):(\d+):(\d+)/) {
	$hostEnt->{ip} = $1;
	$aip->{tcpport} = $2;
	$aip->{udpport} = $3;
      }
      
      if ($resp =~ /servop\[(.*)\]/) {
	my $servop = $1;
	# need to convert spaces into underscores in the server op name
	$servop =~ s/ /_/g;
	$hostEnt->{servop} = $servop;
      }
      
      if ($saveNeighbour) {
	if ($resp =~ /(\d+)/) {
#	  print "Adding neighbour $1\n";
	  push(@{$aip->{neighbours}}, $1);
	}
      }
      if ($resp =~ /neighbours:.*/m) {
	$saveNeighbour = 1;
	
      }
    }
    
    # add the aip to the aipByID array, and the host to the hostmap
    
    $aipByID->[$aip->{entID}] = $aip;
    $hostMap->{$hostEnt->{hostName}} = $hostEnt;
    $hostIPMap->{$hostEnt->{ip}} = $hostEnt;
    
  }

}

#
# a simple perl object to store information about core servers
#

package CoreEnt;

sub new {

  my $self = {};
  $self->{entName} = "";
  $self->{entID} = "";
  $self->{entType} = "";
  $self->{hostName} = "";
  $self->{port} = "";

  bless($self);
}

sub printEnt {
  my ($self,$fh) = @_;

  print $fh "entity add $self->{entType} $self->{entName} $self->{hostName} $self->{port}\n";

}

# Given the name of a core server type in a form niqs_test will understand,
# (keyqry, lode, niqs, etc.) this subroutine collects info on all servers
# of that type, and creates entries for them in the coreServerMap.
#
# $inFH, and $outFH are expected to be read and write pipes pointing at an
# active niqs_test.  
# 
# $coreServerMap : a reference to the map to be populated with core servers
# $hostIPMap :  a reference to a map associating IP addresses with hostEnt refs

sub collectCoreServers {
  my ($type, $inFH, $outFH, $coreServerMap, $hostIPMap) = @_;
  
  print $outFH "list $type\n";
  
  my $resp = Support::readOutput($inFH);
  
  my @coreIDList = split(/\n/,$resp);
  #remove the first and last entries (for the command and the prompt)
  shift(@coreIDList);
  pop(@coreIDList);
  
  foreach my $core (@coreIDList) {
    my @coreDesc = split(/[\s\/]+/,$core);
    my $coreEnt = new CoreEnt;
#    print "Core Server is : $coreDesc[0] $coreDesc[1] $coreDesc[2]\n";
    
    $coreEnt->{entID} = int($coreDesc[0]);
    $coreEnt->{entName} = $coreDesc[2];
    $coreEnt->{entType} = $type;
    $coreServerMap->{$coreDesc[2]} = $coreEnt;
    # now query the precise information we need
    
    print $outFH "query description $type $coreEnt->{entName}\n";
    
    $resp = Support::readOutput($inFH);
    
    if ($resp =~ /ipaddr:port = ([\d\.]+):(\d+)/) {
      $coreEnt->{port} = $2;
      $coreEnt->{hostName} = $hostIPMap->{$1}->{hostName};
    }
  }
}

1;
