
use strict;
use Env;

require $ENV{'HOME'} . "/lib/freedomAdminLoadDbUtils.pl";


my $APPNAME          = "FILL LATER";

my $PATH             = $ENV{'HOME'} . "/var/dat/";
my $FILENAME_SUFFIX  = '_[12][0-9]{3}-[0-9]{2}-[0-9]{2}_[0-9]{2}-[0-9]{2}-[0-9]{2}\.dat';

my $FILENAME_PREEFIX = "FILL LATER";
my $FILENAME         = "FILL LATER";
my $TABLE            = "FILL LATER";

my $treatReject      = 0;

my $DEBUG            = 0;



#
# Return the global PATH variable
#
sub GetPath {
	return $PATH;
}

#
# Tracing utility
#
sub trace {
	if( $DEBUG != 0 ) {
		my $str  = join "", @_;
		printf "Enter " . $str . "\n";
	}
}


#
# Debug utility
#
sub debug {
	if( $DEBUG != 0  ) {
		my $str  = join "", @_;
		printf $str . "\n";
	}
}

#
# Debug utility for list
#
sub dumpList {
	if( $DEBUG != 0  ) {
		my @list = @_;
		my $entry;
		my $idx = 0;
		
		foreach $entry (@list) {
			debug("LIST[$idx]: " . $entry);
			$idx++;
		}
	}
}

#
# List the files matching the current pattern into the directory
#
sub ListMatchingFiles {
  trace("ListMatchingFiles");
  my $exp      = '*_' . $FILENAME_PREEFIX . '*\.dat';
  my $cmd      = "find $PATH -name \"$exp\" -print";
  my @fulllist = `$cmd`;
#  print join(" ", @fulllist),"\n";
  my $expr     = $FILENAME;
  my @list     = grep /$expr/, @fulllist;
  my $item     = "";
  my $i        = 0;
  my %sortlist;
	
  chomp @list;
	
  for( $i = 0; $i < @list; $i++ ) {
    $_ = $list[$i];

    s/^.*\/(\w+_$FILENAME_PREEFIX.*)/$1/;
    $list[$i] = $_;
  }
	
  return @list;
}


#
# Return the list of file that has to be treat.
#
sub CurFileList {
	trace("CurFileList");
	my @list;
	my @retlist;
	my $retoffset;
	my $idx     = 0;
	my $entry   = "";
	my $file    =  "";
	my $offset    = 0;
	
	#
	# Retreive the matching list of file from the directory
	#
	@list = ListMatchingFiles();
	@list = sort @list;
#print "\n\t", join("\n\t", @list), "\n";

	#
	# Retreive the already acced file from the SQL database
	#
	SendSQL("SELECT count(*) " .
					"FROM admin_load_state " .
					"WHERE filename REGEXP '_$FILENAME_PREEFIX'");	
	
	my ($count) = FetchRow();
#print "find $FILENAME_PREEFIX: ($count) files\n";

	if( $count != 0 ) {
		SendSQL("SELECT MAX(filename) " .
						"FROM admin_load_state " .
						"WHERE filename REGEXP '_$FILENAME_PREEFIX'");
		
		my ($file) = FetchRow();
#print "Current File $FILENAME_PREEFIX : $file\n";

		if( defined($file) ) {
			SendSQL("SELECT offset " .
							"FROM admin_load_state " .
							"WHERE filename = \"$file\"");
			
			($offset) = FetchRow();
			if( defined($offset) ) {
				#
				# Chack that the file still exist
				#
				if( scalar (grep /$file/, @list) == 0 ) {
					die "$APPNAME: Can't find $file, offset=$offset, << file disapear >> ";
				} else {
					#
					# Complete the list with the rest of the available files
					#
					foreach $entry (@list) {
						if( $entry ge $file ) {
							push @retlist, $entry;
						}
					}
					$retoffset = $offset;
				}
			} else {
				die "$APPNAME: SQL error: join filename/offset failed";
			}
		} else {
			die "$APPNAME: SQL error: filename not found";
		}		
	} else {
		#
		# If there is now files available than return the files 
		# available into the directory.
		#
		@retlist   = @list;
		$retoffset = 0;
	}
	
	return ($retoffset, @retlist);
}


sub UpdateAdminLoadStateTable {
	trace("UpdateAdminLoadStateTable");
	my ($offset, $filename) = @_;
	my $sqlerr;
	
	$offset   =  "\"$offset\"";
	$filename =  "\"$filename\"";
	
	SendSQL("INSERT admin_load_state(filename, offset) " .
					"VALUES($filename,$offset)");
	
	$sqlerr = SqlErrNo();
	
	if( defined($sqlerr) && ($sqlerr != 0) ) {
		SendSQL("UPDATE admin_load_state " .
						"SET offset = $offset " .
						"WHERE filename = $filename");
		
		$sqlerr = SqlErrNo();
		
		if( defined($sqlerr) && ($sqlerr != 0) ) {
			die "$APPNAME: UpdateAdminLoadStateTable failed: filename = $filename, offset = $offset, (" . SqlErrMsg() . ")";
		}
	}
}


sub GetOffsetFromLoadStateTable {
	trace("GetOffsetFromLoadStateTable");
	my ($filename) = @_;
	my $sqlerr;
	my $offset    = 0;
	my $retoffset = 0;

	$filename =  "\"$filename\"";

	#
	# Retreive the already acced file from the SQL database
	#
	SendSQL("SELECT count(*) " .
					"FROM admin_load_state " .
					"WHERE filename = $filename");
	
	my ($count) = FetchRow();
	
	if( $count != 0 ) {
			SendSQL("SELECT offset " .
							"FROM admin_load_state " .
							"WHERE filename = $filename");
			
			($offset) = FetchRow();
			if( defined($offset) ) {
				$retoffset = $offset;
			}
	}
	return $retoffset;
}


#
# Load the Database using this file as input
# [in] $offset:   Offset of the first entry on the first file of the file list
# [in] @filelist: File list 
#
sub LoadTable {
	trace("LoadTable");
	my $offset;
	my @filelist;
	my $curfile;
	my $entry;
	my $count = 0;
	my $reject = 0;	
	my $sqlerr;
	my $idx;
	
	($offset, @filelist) = @_;

	QuietModeEnable();
	
	foreach $entry (@filelist) {
		my $rejfile = "";
		my $REJECT;
		
		$count  = 0;
		$reject = 0;
		
		$_ = $entry;
		
		debug("Scanning file $entry");
		
		#
		# Create the reject filename
		#
		$_ = $entry; s/\.dat$/.rej/; $rejfile = $_;		
		
		#
		# Open the input file
		#
		open (INPUT, $PATH . $entry ) || die "$APPNAME: Can't open input file: $PATH$entry";		
		
		#
		# Check if file has not been truncated
		#
		my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($PATH . $entry);
		if( $size < $offset ) {
			seek INPUT, $offset - 1 , 0 || die "$APPNAME: Seek failed file: $PATH$entry truncated";		
		}
		
		#
		# Reach the last read position
		#
		seek INPUT, $offset, 0;		
		
		#
		# Read the next record on the file				
		#
		while ( <INPUT> ) {					# Read the next record on the file				
			if( /^[^\#]/ ) {	                         	# If the entry is not a comment
				my $line    = $_;
				my $escline;
				my $values  = "";
				my @strlist;
				
				chomp $line;
				
				#
				# Convert the line, substitute ';' by ','
				#
				$escline = $line;
				$escline =~ s/([^\\]);/$1;;/g; # Create a pattern that will be usable by split
				@strlist = split /;;/, $escline; # Split the values
				for( $idx = 0; $idx < @strlist; $idx++ ) {
					$strlist[$idx] =~ s/\\;/;/;	# Remove the escaping of '\;'
				}				
				$values  = join ',', @strlist; # Reconstruct the output string
				
				#
				# Retreive the already acced file from the SQL database
				#
				SendSQL("INSERT $TABLE VALUES($values)");
				
				$sqlerr = SqlErrNo();
				
				if( defined($sqlerr) && ($sqlerr != 0) ) { # Reject the record in case of error
					#
					# If we are not actualy repassing a reject file
					#
					if( $treatReject == 0 ) {
						#
						# Open the reject file if not already open
						#
						if( !defined($REJECT) ) {
							open (REJECT, ">>" . $PATH . $rejfile) || die "$APPNAME: Can't open output file: $PATH .$rejfile";		
						}					
						printf REJECT "# Reject: 'INSERT $TABLE VALUES($values)' errno=%d, errmsg='%s'\n" ,SqlErrNo(), SqlErrMsg();
						printf REJECT "%s\n", $line;
					} else {
						printf STDOUT "# Reject: 'INSERT $TABLE VALUES($values)' errno=%d, errmsg='%s'\n" ,SqlErrNo(), SqlErrMsg();
						printf STDOUT "%s\n", $line;
					}
					$reject++;
					
				}	else {
					$count++;
				}
				
				$offset = tell INPUT;
				debug("Offset = $offset");
				#
				# Update the database use to repassing an input file
				#
				if( $treatReject == 0 ) {
					UpdateAdminLoadStateTable($offset, $entry);
				}
			}	
		}
		# Close the input file 
		close (INPUT);
		# Close the reject file  
		if( defined($REJECT) ) {
			close (REJECT);
		}
		$offset = 0;
		printf STDOUT "Loading $entry: $count success, $reject rejected\n";
	}
	return ($offset, $curfile);
}



sub LoadFreedomAdmin {
	trace("LoadFreedomAdmin");
	my $rejectFile = "";
	my $offset    = 0;
	my @filelist;
	
	#
	# retreive input arguments
	#
	($APPNAME, $FILENAME_PREEFIX, $TABLE, $rejectFile) = @_;
#        print "($APPNAME, $FILENAME_PREEFIX, $TABLE, $rejectFile)\n";	

	#
	# Build the input filename pattern
	#
	$FILENAME     = $FILENAME_PREEFIX . $FILENAME_SUFFIX;
#        print "<$FILENAME> <$FILENAME_SUFFIX>\n";
	
	#
	# make standard output unbuffered
	#
	select(STDOUT); $| = 1;
		
	#
	# Build the input file list
	#
	if( !defined($rejectFile) || ($rejectFile eq "") ) {
		#
		# Find the current file
		#
		($offset, @filelist) = CurFileList();
		
		if( $#filelist == -1 ) {
			printf STDOUT "$APPNAME: Nothing to do\n";
			return 0;
		}
	} else {
		$treatReject = 1;
		@filelist    = ($rejectFile);
		$offset      = 0;
	}
	
	#
	# Load the Database using this file as input
	#
	LoadTable($offset, @filelist);
	
	return 0;
}

1;
