#!/usr/bin/perl -w

use strict;
use English;
use Getopt::Long;

require $ENV{'HOME'} . "/lib/freedomAdminLoadDbUtils.pl";

my $APPNAME      = $PROGRAM_NAME;
my $FROMTABLE    = "nym_activity_event";
my $TOTABLE      = "nym_activity";
my $full         = 0;
my $keep         = 0;


sub LoadTableFromEvent {
	my ($from, $to, $force, $keep) = @_;
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
	my $sqlerr;
	my $delta=0;
	my $curdate;
	

	if( $force != 0 ) {
		SendSQL("DELETE FROM $TOTABLE");	
	}

        ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time);
        $curdate = sprintf "%04d-%02d-%02d %02d:%02d:%02d", 1900 + $year, $mon+1, $mday, $hour, $min, $sec;

	SendSQL("SELECT count(*) FROM $FROMTABLE");
	$sqlerr = SqlErrNo();
        if( !defined($sqlerr) || ($sqlerr == 0) ) {
		my @row = FetchRow();
		my ($count) = @row;
		if( $count > 0 ) {
			SendSQL("SELECT UNIX_TIMESTAMP(\"$curdate\") - UNIX_TIMESTAMP(min(date)) FROM $FROMTABLE");
			$sqlerr = SqlErrNo();
			if( !defined($sqlerr) || ($sqlerr == 0) ) {
				@row = FetchRow();		
				($delta) = @row;
			}	
		}
	}
	
	($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time);
	$curdate = sprintf "%04d-%02d-%02d %02d:%02d:%02d", 1900 + $year, $mon+1, $mday, $hour, $min, $sec;

	SendSQL("INSERT $to (date, nym, delta, mailsent, mailrecv, maildisc, newspost, newsdisc)" .
					"SELECT \"$curdate\", nym, $delta, sum(mailsent), sum(mailrecv), sum(maildisc), sum(newspost), sum(newsdisc) " .
					"FROM $from " .
					"group by nym");

	$sqlerr = SqlErrNo();

	if( defined($sqlerr) && ($sqlerr != 0) ) {
		die "$APPNAME: insert failed (" . SqlErrMsg() . ")";			

	}
	if( $keep == 0 ) {
		SendSQL("DELETE FROM $FROMTABLE");	
	}
	printf STDOUT "$APPNAME: success\n";
}

#
# make standard output unbuffered
#
select(STDOUT); $| = 1;

#
# argument parsing and checking
#
my $res;
my %optctl = ();

my $helpString = 
    ("Usage $APPNAME\n" .
     "\t--help     : this help\n" .
     "\t--full     : force a full reload of the table\n" .
     "\t--keep     : keep  the nym_activity_event data, do not wipe it");

# Get the Options
$res = GetOptions(\%optctl, "help","full", "keep") || die "Error in Getopt";

##
## print out the usage string
##
if (defined($optctl{'help'})) {
    print $helpString . "\n";
    exit 0;
}

#
# retrieve the reject filename
#
if (defined($optctl{'full'})) {
	$full = 1;
}

#
# retrieve the reject filename
#
if (defined($optctl{'keep'})) {
        $keep = 1;
}


#
# Connect to the database
#
ConnectToDatabase() || die "$APPNAME: Can't connect to the database";

#
# Load the Database using this file as input
#
LoadTableFromEvent($FROMTABLE, $TOTABLE, $full, $keep);

exit 0;
