#!/bin/sh 
#
# Quick script to install a network
#
# Luc Boulianne <luc@zks.net>
#
usage () {
    echo "$0 [--install] [--start] [--stop] [--sngen] [--timesync] [--testssh]"
    exit
}
VERSION=1.3

CUR_VERS=`wget -q http://sysdev.dev.zks.net/genesis/vers.cgi -O -`

if [ x$CUR_VERS != x$VERSION ] ; then 
    echo "** ERROR: Script out of date"
    echo "** Please download the latest \`do_gamma.sh' and \`genesis-bootstrap.sh' scripts"
    exit
fi

MAIL_B_LIST="actarus.b spiderman.b superman.b ironman.b hulk.b"
#MAIL_B_LIST=""
MAIL_LIST="imep relay"
#MAIL_LIST=""
CORE_AIP="coreaip1 coreaip2 coreaip3 coreaip4 coreaip5"
#CORE_AIP=""
LIST="kqd niqs1 niqs2 fmg nym1 nym2 token"
#LIST=""
SOL="brute.beta.freedom.net"
SOL=""
SSHPROXY="root@jumppoint.beta.freedom.net"
NETWORK=beta
DOMAIN=beta.freedom.net
TOKSRV=token
SSH=ssh1
SCP=scp1
SNGenSets=1

clean=""
Install="no";
Start="no";
Status="no";
Stop="no";
TimeSync="no";
SNGen="no";
TestSSH="no";
Backup="no";

while [ $# -gt 0 ] ; do
    case "$1" in
    --config)
	shift; ZKGEN_CONFIG=$1; export  ZKGEN_DBCONFIG
	;;
    --dbconfig)
	shift; ZKGEN_DBCONFIG=$1; export ZKGEN_DBCONFIG
	;;
    --clean)
        clean=$1
	;;
    --backup)
        Backup="yes";
	;;
    --testssh)
        TestSSH="yes";
	;;
    --timesync)
        TimeSync="yes";
	;;
    --install)
        Install="yes";
	;;
    --start)
	Start="yes";
	;;
    --status)
	Status="yes";
	;;
    --sngen)
	SNGen="yes";
	;;
    --stop)
	Stop="yes";
	;;
    --help)
        usage
	;;
    *)
	echo "* Error: unknown parameter[$1]"
	usage
	;;
    esac
    shift;
done

msg () {

    echo "[***]"
    echo "[***] $*"
    echo "[***]"

}    

frame () {

    printf "+-----[%-17s]------\n" $1
    sed 's/^/| /'
    printf "+------------------------------\n\n" $1

}    


if [ $TestSSH = "yes" ] ; then 
    msg "Testing SSH"
    for name in $SOL ; do 
	printf "%-30s: " $name
	ssh1 -x root@$name "h=\`hostname\` && d=\`date\` && printf \"%-30s [%s]\n\" \$h \"\$d\" ; sleep 1"
    done
    for name in $CORE_AIP $LIST $MAIL_LIST; do 
	printf "%-30s: " $name.$DOMAIN
	$SSH -x root@$name.$DOMAIN "h=\`hostname\` && d=\`date\` && printf \"%-30s [%s]\n\" \$h \"\$d\" ; sleep 1"
    done
    for name in $CORE_AIP $MAIL_B_LIST ; do 
	printf "%-30s: " $name.$DOMAIN
	$SSH -x  $SSHPROXY \
	    "ssh -x $name.$DOMAIN \" h=\\\`hostname\\\` && d=\\\`date\\\` && printf \\\"%-30s [%s]\n\\\" \\\$h \\\"\\\$d\\\" ; sleep 1\"; sleep 1"
    done

elif [ $TimeSync = "yes" ] ; then 
    msg "Syncing time."
    for name in $SOL ; do 
    	#ssh1 -x -R37:caesar.zks.net:37 root@$name "hostname; rdate -s localhost ; sleep 1"
	/bin/true
    done
    for name in $CORE_AIP $LIST $MAIL_LIST; do 
	$SSH -x -R37:caesar.zks.net:37 root@$name.$DOMAIN "hostname; rdate -s localhost; sleep 5"
    done
    for name in $MAIL_B_LIST ; do 
	echo "Need to sleep a while for the ports to become available again ..."
	sleep 60
	$SSH -x -R37:caesar.zks.net:37 $SSHPROXY \
	    "ssh -x -R37:localhost:37 $name.$DOMAIN \"hostname; rdate -s localhost; sleep 5\"; sleep 5 "
    done

elif [ $Backup = "yes" ] ; then 
	msg "Doing Backup"
	for name in $CORE_AIP $LIST $MAIL_LIST; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x root@$name.$DOMAIN "tar -cvzpf /usr/freedom.tgz /usr/freedom; sleep 1"
	done
	for name in $MAIL_B_LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x  $SSHPROXY \
		"ssh -x $name.$DOMAIN \" tar -cvzpf /usr/freedom.tgz /usr/freedom ; sleep 1\"; sleep 1"
	done
elif [ $Install = "yes" ] ; then 
    msg "Proceeding with Installation"

    echo "* Installing freedom on [$SOL]"
#    for name in $SOL ; do 
    for name in "" ; do 
	echo "[***] $name"
	genesis-bootstrap.sh --dest $name \
	    --network $NETWORK --ssh-prog ssh1 --scp-prog scp1 --repository-url http://sysdev.dev.zks.net/genesis/ $clean
    done
    
    echo "* Installing freedom on [$LIST $MAIL_LIST $CORE_AIP]"
    for name in $LIST $CORE_AIP $MAIL_LIST; do 
	echo "[***] $name"
	genesis-bootstrap.sh --dest ${name}.$DOMAIN \
	    --network $NETWORK --ssh-prog $SSH --scp-prog $SCP --repository-url http://sysdev.dev.zks.net/genesis/ $clean
	sleep 5
    done
	
    echo "* Installing freedom on [$MAIL_B_LIST]"
    for name in $MAIL_B_LIST ; do 
	echo "[***] $name"
	genesis-bootstrap.sh --dest ${name}.$DOMAIN \
	    --network $NETWORK --ssh-prog $SSH --scp-prog $SCP --repository-url http://sysdev.dev.zks.net/genesis/ $clean \
	    --ssh-proxy $SSHPROXY --ssh-proxy-prog /usr/bin/$SSH \
	    --scp-proxy-prog /usr/bin/$SCP 
	sleep 80
    done

elif [ $SNGen = "yes" ] ; then 
    msg "NOT Generating Serial Numbers"
    echo NOT DOING genesis-sngen.sh --ssh1 --toksrv $TOKSRV.$DOMAIN --network $NETWORK \
	--url http://afs6.freedom.net/update/ --numsets $SNGenSets $clean

elif [ $Start = "yes" ] ; then 
    msg "Starting Services"
    echo "* SOLARIS SERVERS not affected by this command"
    echo "* Starting freedom on [$LIST $CORE_AIP]"
    for name in $LIST $CORE_AIP ; do 
	genesis-bootstrap.sh --dest ${name}.$DOMAIN \
	    --network $NETWORK --ssh-prog $SSH --scp-prog $SCP --repository-url http://sysdev.dev.zks.net/genesis/ $clean --start
    done
    echo "* Starting freedom on mail system"
    for name in $MAIL_LIST ; do 
	$SSH -x root@$name.$DOMAIN \
	    "printf '[%s]\n' \`hostname\` ; qmail start ; sleep 1"
    done
    for name in actarus.b spiderman.b superman.b ironman.b hulk.b ; do
	$SSH -x $SSHPROXY \
	    "ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail start ; sleep 1\"; sleep 1 "
    done

elif [ $Stop = "yes" ] ; then 
    msg "Stopping Services"
    echo "* SOLARIS SERVERS not affected by this command"
    echo "* Stopping freedom on [$LIST $CORE_AIP]"
    for name in $LIST $CORE_AIP; do 
	genesis-bootstrap.sh --dest ${name}.$DOMAIN \
	    --network $NETWORK --ssh-prog $SSH --scp-prog $SCP --repository-url http://sysdev.dev.zks.net/genesis/ $clean --stop
    done

    echo "* Stopping freedom on mail system"
    for name in $MAIL_LIST ; do 
	$SSH -x root@$name.$DOMAIN \
	    "printf '[%s]\n' \`hostname\` ; qmail stop ; sleep 1"
    done
    for name in actarus.b spiderman.b superman.b ironman.b hulk.b ; do
	$SSH -x $SSHPROXY \
	    "ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail stop ; sleep 1\"; sleep 1 "
    done

elif [ $Status = "yes" ] ; then 
    msg "Getting process status"
    for name in $SOL ; do 
	ssh1 -x root@$name "printf '[%s]\n' `hostname` ; ps -aef | grep $NETWORK | grep -v grep ; sleep 1" | frame $name 2>&1
    done
    for name in $LIST $CORE_AIP ; do 
	$SSH -x root@$name.$DOMAIN "printf '[%s]\n' \`hostname\` ; ps agxfw | grep freedom | egrep -v '(grep|su|agxf)'  ; sleep 1" | frame $name 2>&1
    done
    for name in $MAIL_LIST ; do 
	$SSH -x root@$name.$DOMAIN \
	    "printf '[%s]\n' \`hostname\` ; /usr/bin/qmail stat ; sleep 1" | frame $name 2>&1
    done
    for name in $MAIL_B_LIST ; do
	$SSH -x $SSHPROXY \
	    "ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail stat ; sleep 1\"; sleep 1 " | frame $name 2>&1
    done

fi


