#!/bin/sh 
#
# Support routines for do_network.sh
#
# $Id: do_funcs.sh,v 1.8 2001/04/23 17:33:11 alainc Exp $
#
# Luc Boulianne <luc@zks.net>
#

PATH=${PATH}:/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin:/usr/ucb
export PATH

VERSION=1.3
CUR_VERS=`wget -q http://sysdev.dev.zks.net/genesis/vers.cgi -O -`


# ssh defaults
SSH=${SSH:=ssh}
SCP=${SCP:=scp}
SOL_SSH=${SOL_SSH:=ssh1}
SOL_SCP=${SOL_SCP:=scp1}



if [ x$CUR_VERS != x$VERSION ] ; then 
    echo "** ERROR: $0 script out of date"
    echo "** Please download the latest \`do_network.sh' and \`genesis-bootstrap.sh' scripts"
    exit
fi


usage () {
    echo "$0 [--install] [--fix \\\"sh-fragment\\\"] [--start] [--stop] [--sngen] [--timesync] [--testssh] [--debug]"
    cleanup
    exit
}


pname=`basename $0`;

trap 'cleanup; exit 2' 1 2 3 13 15


cleanup () {
    /bin/rm -f .$pname
}


msg () {
    echo "[***]"
    echo "[***] $*"
    echo "[***]"
}
    

frame () {
    printf "+-----[%-17s]------\n" $1
    sed 's/^/| /'
    printf "+------------------------------\n\n" $1
}    


do_gen () {

    clean=""
    debug=""
    ssh_debug=""
    genesis_debug=""
    Install="no"
    Start="no"
    Status="no"
    Stop="no"
    TimeSync="no"
    SNGen="no"
    SNGenSets=${SNGenSets:-10}
    TestSSH="no"
    Fix="no"
    Backup="no"

    if [ -e .$pname ] ; then
	echo -n "Upgrade already in progress by: "
	cat .$pname
	exit 2
    fi

    echo "$LOGNAME on" `date` > .$pname
    
    while [ $# -gt 0 -a "$Fix" = "no" ] ; do
        case "$1" in
        --config)
    	shift; ZKGEN_CONFIG=$1; export  ZKGEN_DBCONFIG
    	;;
        --dbconfig)
    	shift; ZKGEN_DBCONFIG=$1; export ZKGEN_DBCONFIG
    	;;
        --clean)
            clean=$1
    	;;
	--debug)
	    debug=1
	    ssh_debug="-v"
	    genesis_debug="--debug"
	;;
        --testssh)
            TestSSH="yes";
    	;;
	--backup)
	    Backup="yes";
	;;
        --timesync)
            TimeSync="yes";
    	;;
        --install)
            Install="yes";
    	;;
        --start)
    	Start="yes";
    	;;
        --status)
    	Status="yes";
    	;;
        --fix)
    	shift; Fix="$*";
    	;;
        --sngen)
    	SNGen="yes";
    	;;
        --stop)
    	Stop="yes";
    	;;
        --help)
            usage
    	;;
        *)
    	echo "* Error: unknown parameter[$1]"
    	usage
    	;;
        esac
        shift;
    done

    
    if [ $TestSSH = "yes" ] ; then 

	msg "Testing SSH keys [SSH=$SSH SOL_SSH=$SOL_SSH]"

        msg "Testing SSH on Freedom DB backend"
        for name in $SOL ; do
	    printf "%-30s: " $name
	    $SOL_SSH -x $ssh_debug root@$name "h=\`hostname\` && d=\`date\` && printf \"%-30s [%s]\n\" \$h \"\$d\" ; sleep 1"
        done

	msg "Testing SSH on Freedom Mail system"
        for name in $MAIL_B_LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x $ssh_debug $SSHPROXY \
		"ssh $ssh_debug -x $name.$DOMAIN \" h=\\\`hostname\\\` && d=\\\`date\\\` && printf \\\"%-30s [%s]\n\\\" \\\$h \\\"\\\$d\\\" ; sleep 1\"; sleep 1"
        done
        for name in $MAIL_LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x $ssh_debug root@$name.$DOMAIN "h=\`hostname\` && d=\`date\` && printf \"%-30s [%s]\n\" \$h \"\$d\" ; sleep 1"
        done

	msg "Testing SSH on Freedom servers"
        for name in $CORE_AIP $LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x $ssh_debug root@$name.$DOMAIN "h=\`hostname\` && d=\`date\` && printf \"%-30s [%s]\n\" \$h \"\$d\" ; sleep 1"
        done

    
    elif [ $TimeSync = "yes" ] ; then 

	msg "Synchronizing time"

	msg "Skipping Freedom DB backend"
        #msg "Syncing time on Freedom DB backend"
        #for name in $SOL ; do 
	    #$SOL_SSH -x -R37:caesar.zks.net:37 root@$name "hostname; \
		#rdate -s localhost ; sleep 1"
	    #/bin/true
        #done

        msg "Syncing time on Freedom Mail system"
        for name in $MAIL_B_LIST ; do 
	    echo "sleep a while for the ports to become available again ..."
	    sleep 60
	    $SSH -x -R37:caesar.zks.net:37 $SSHPROXY \
		"ssh -x -R37:localhost:37 $name.$DOMAIN \"hostname; rdate -s localhost; sleep 5\"; sleep 5 "
        done
        for name in $MAIL_LIST ; do 
	    $SSH -x -R37:caesar.zks.net:37 root@$name.$DOMAIN "hostname; rdate -s localhost; sleep 5"
	done

        msg "Syncing time on Freedom servers"
        for name in $CORE_AIP $LIST ; do 
	    $SSH -x -R37:caesar.zks.net:37 root@$name.$DOMAIN "hostname; rdate -s localhost; sleep 5"
	done


    elif [ "$Fix" != "no" ] ; then 

	msg "Proceeding to Apply supplied fix"

	msg "Skipping Freedom DB backend"
        #msg "Applying fix [$Fix] to Freedom DB backend"
        #for name in $SOL ; do 
	    #$SOL_SSH -x -R37:caesar.zks.net:37 root@$name "hostname; \
		#rdate -s localhost ; sleep 1"
	    #/bin/true
        #done

	msg "Applying fix [$Fix] to Freedom Mail system"
        for name in $MAIL_B_LIST ; do 
	    $SSH -x -R37:caesar.zks.net:37 $SSHPROXY \
		"ssh  $name.$DOMAIN -x \"$Fix\"; sleep 5 "
	    echo "sleep a while for the ports to become available again ..."
	    sleep 60
        done
        for name in $MAIL_LIST ; do 
	    $SSH -x root@$name.$DOMAIN "$Fix; sleep 3"
        done

	msg "Applying fix [$Fix] to Freedom servers"
        for name in $CORE_AIP $LIST ; do 
	    $SSH -x root@$name.$DOMAIN "$Fix; sleep 3"
        done

    
    elif [ $Install = "yes" ] ; then 

        msg "Proceeding with Installation"
    
        msg "Installing Freedom DB backend"
        for name in $SOL ; do 
	    echo "[***] $name"
	    genesis-bootstrap.sh --dest $name --network $NETWORK \
		--ssh-prog $SOL_SSH --scp-prog $SOL_SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		$clean $genesis_debug
        done

        msg "Installing Freedom Mail system"
        for name in $MAIL_B_LIST ; do 
	    echo "[***] $name"
	    genesis-bootstrap.sh --dest ${name}.$DOMAIN --network $NETWORK \
		--ssh-prog $SSH --scp-prog $SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		--ssh-proxy $SSHPROXY --ssh-proxy-prog /usr/local/bin/$SSH \
		--scp-proxy-prog /usr/local/bin/$SCP \
		$clean $genesis_debug
	    sleep 80
        done
        for name in $MAIL_LIST ; do 
	    echo "[***] $name"
	    genesis-bootstrap.sh --dest ${name}.$DOMAIN --network $NETWORK \
		--ssh-prog $SSH --scp-prog $SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		$clean $genesis_debug
	    sleep 5
        done

        msg "Installing Freedom servers"
        for name in $LIST $CORE_AIP ; do 
	    echo "[***] $name"
	    genesis-bootstrap.sh --dest ${name}.$DOMAIN --network $NETWORK \
		--ssh-prog $SSH --scp-prog $SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		$clean $genesis_debug
	    sleep 5
        done
    

    elif [ $SNGen = "yes" ] ; then 

        msg "Generating Serial Numbers"

	if [ x$SSH = xssh1 ] ; then
		echo Using ssh1
		snssh=--ssh1
	else
		snssh=""
	fi
        genesis-sngen.sh --toksrv $TOKSRV.$DOMAIN --network $NETWORK \
	    --url http://afs6.freedom.net/update/ $snssh \
	    --numsets $SNGenSets \
	    $clean

    
    elif [ $Start = "yes" ] ; then 

        msg "Starting Services"

        msg "Skipping Freedom DB backend"

        msg "Starting Freedom servers"
        for name in $LIST $CORE_AIP ; do 
	    genesis-bootstrap.sh --dest ${name}.$DOMAIN --network $NETWORK \
		--ssh-prog $SSH --scp-prog $SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		$clean $genesis_debug --start
        done

        msg "Starting Freedom Mail system"
        for name in $MAIL_B_LIST ; do
	    $SSH -x $SSHPROXY \
		"ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail start ; sleep 1\"; sleep 1 "
        done
        for name in $MAIL_LIST ; do 
	    $SSH -x root@$name.$DOMAIN \
		"printf '[%s]\n' \`hostname\` ; qmail start ; sleep 1"
        done

    
    elif [ $Stop = "yes" ] ; then 

        msg "Stopping Services"

        msg "Skipping Freedom DB backend"

        msg "Stopping Freedom Mail system"
        for name in $MAIL_LIST ; do 
	    $SSH -x root@$name.$DOMAIN \
		"printf '[%s]\n' \`hostname\` ; qmail stop ; sleep 1"
        done
        for name in $MAIL_B_LIST ; do
    	$SSH -x $SSHPROXY \
    	    "ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail stop ; sleep 1\"; sleep 1 "
        done

        msg "Stopping Freedom servers"
        for name in $LIST $CORE_AIP ; do 
	    genesis-bootstrap.sh --dest ${name}.$DOMAIN --network $NETWORK \
		--ssh-prog $SSH --scp-prog $SCP \
		--repository-url http://sysdev.dev.zks.net/genesis-mig/ \
		$clean $genesis_debug --stop
        done

    
    elif [ $Status = "yes" ] ; then 

        msg "Fetching Status of Services"

	msg "Status of Freedom DB backend"
        for name in $SOL ; do 
	    $SOL_SSH -x root@$name "printf '[%s]\n' `hostname` ; ps -aef | grep $NETWORK | grep -v grep ; sleep 1" \
		| frame $name 2>&1
        done

	msg "Status of Freedom servers"
        for name in $LIST $CORE_AIP ; do 
	    $SSH -x root@$name.$DOMAIN "printf '[%s]\n' \`hostname\` ; ps agxfw | grep freedom | egrep -v '(grep|su|agxf)'  ; sleep 1" | frame $name 2>&1
        done

	msg "Status of Freedom Mail system"
        for name in $MAIL_LIST ; do 
	    $SSH -x root@$name.$DOMAIN \
		"printf '[%s]\n' \`hostname\` ; /usr/bin/qmail stat ; sleep 1" | frame $name 2>&1
        done
        for name in $MAIL_B_LIST ; do
	    $SSH -x $SSHPROXY \
		"ssh -x $name.$DOMAIN \"printf '[%s]\n' \\\`hostname\\\` ; qmail stat ; sleep 1\"; sleep 1 " | frame $name 2>&1
        done

    
    elif [ $Backup = "yes" ] ; then 

	msg "Doing Backup"

	msg "Backup of Freedom DB backend"
	for name in $SOL ; do 
	    printf "%-30s: " $name
	    $SOL_SSH -x root@$name "PATH=\$PATH:/usr/local/bin && export PATH && /usr/local/bin/gtar -cvzpf /home/gamma.tgz /home/gamma; sleep 1"
	done

	msg "Backup of Freedom Mail system"
	for name in $MAIL_B_LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x  $SSHPROXY \
		"ssh -x $name.$DOMAIN \" tar -cvzpf /usr/freedom.tgz /usr/freedom ; sleep 1\"; sleep 1"
	done
	for name in $MAIL_LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x root@$name.$DOMAIN "tar -cvzpf /usr/freedom.tgz /usr/freedom; sleep 1"
	done

	msg "Backup of Freedom servers"
	for name in $CORE_AIP $LIST ; do 
	    printf "%-30s: " $name.$DOMAIN
	    $SSH -x root@$name.$DOMAIN "tar -cvzpf /usr/freedom.tgz /usr/freedom; sleep 1"
	done


    fi

    cleanup
}
