#!/usr/bin/perl
#
# Luc Boulianne <luc@zks.net>
# $Id: post-sn.pl,v 1.4 2001/04/23 17:33:27 alainc Exp $
#

use strict;
use Getopt::Long;

sub escape {
  my $toencode = shift;
  return undef unless defined($toencode);
  $toencode=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
  return $toencode;
}

sub load_file {
  my ($name, $buf) = @_;

  if (open (_F, "$name")) {
    @{$buf} = <_F>;
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name] for reading: $!");
    0;
  }
}

sub save_file {
  my ($name, $buf) = @_;

  if (open (_F, ">$name")) {
    print _F join ("", @{$buf});
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name] for writing: $!");
    0;
  }
}

sub save_buf_in_file {
  my ($name, $buf) = @_;

  if (open (_F, ">$name")) {
    print _F $buf;
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name] for writing: $!");
    0;
  }
}

sub usage {
  print 
    "post-sn.pl - Post Serial Numbers\n",
    "usage: post-sn.pl --network name --url URL [--clean]\n";
  exit;
}

my (%opt);    
my($res) = GetOptions(\%opt, "help",
		      "network=s",
		      "url=s",
		      "clean",
		      "keep",
		      "version") || die "Error in Getopt";

## print out the usage string
##
if (defined($opt{'help'})){
  usage;
}

if (! defined($opt{'network'})){
  usage;
}

if (! defined($opt{'url'})){
  usage;
}

my $data  = "nw=" . escape ($opt{'network'}) . "&" ;

if ( defined($opt{'clean'})){
  $data .= "clear=yes&";
}


my (@data) = <STDIN>;

$data .= "data=" . escape (join ("", @data));

my $name = "/tmp/post-data-" . time();

save_buf_in_file ($name, $data);

my(@res);

if ($opt{'network'} ne "beta") {
  printf "Uploading $name to $opt{url}\n";
  `/usr/bin/lynx -dump -post_data $opt{'url'} < $name > "${name}-res" 2>&1`;
  load_file ("${name}-res", \@res);
  if ($?) {
    print "Problem with posting serial numbers:\n\t", 
    join ("\t", @res);
  } else {
    print "\t", join ("\t", @res);
  }
  unlink ($name);
  unlink (${name}-res);
} else {
  printf "Beta Network: NOT Uploading [$name]\n";
}

