#!/bin/sh
#
# Luc Boulianne <luc@zks.net>
# Genesis frontend
# $Id: genesis-bootstrap.sh,v 1.10 2001/04/24 16:30:58 francisk Exp $
#

PATH=${PATH}:/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin:/usr/ucb
export PATH


prog=`basename $0`
args=""
remote=0
start=""
http_proxy=localhost:3128; export http_proxy

ZK_DEBUG=${ZK_DEBUG:=0}
SSH_DEBUG=${SSH_DEBUG:=}
GENESIS_DEBUG=${GENESIS_DEBUG:=}

ZKGEN_ROOT=/tmp/Genesis;
ZKGEN_CONFIG=${ZKGEN_CONFIG:=genesis-remote.conf}
ZKGEN_DBCONFIG=${ZKGEN_DBCONFIG:=dbs-remote.conf}
ZKGEN_CLEAN=${ZK_CLEAN:=}
export ZKGEN_ROOT
export ZKGEN_CONFIG
export ZKGEN_DBCONFIG
export ZKGEN_CLEAN

ZK_SSH=ssh2
ZK_SCP=scp2
ZK_SSH=ssh
ZK_SCP=scp


# Bloody lazy to do it this way. But it'll do for now.
# Going to have to add smarts to the FrontEnd to deal with
# these properly.
#SolURL="http://devnet.zks.net/%7Ebuild/FNOPS-RLS/Eng13" 
#solURL="http://builds.zks.net/builds1/db2/20000928/Server/Solaris/Release/"
#pkdbURL="http://fishbowl.dev.zks.net/build/pubkeydb"
#


usage () {
    echo "$0: [--help]"
    echo "    --dest remote-host --network networkname"
    echo "    --repository-url repository-url"
    echo "    [--ssh1] [--ssh-proxy [user@]ssh-proxy --web-proxy hostname]"
    echo "    [ --start | --stop ] [--clean]"
    echo ""
    echo "    install freedom on a remote machine for network 'networkname'"
    echo "    authorization file and pub keys must be valid for root"
}



load_freedom () {

    echo ""
    echo "-----------------------------"
    echo "Genesis - Installing Freedom."
    echo "-----------------------------"
    echo ""


    echo ""
    echo " - syncing genesis repository"
    URL=$ZK_REPURL
    if [ ! -d $ZKGEN_ROOT ] ; then
	    mkdir $ZKGEN_ROOT
    fi
    cd $ZKGEN_ROOT
    if /usr/bin/wget --proxy=on -a /tmp/genesis-wget.log \
	--mirror --level 1 -nd -nH --no-parent --reject gif \
	$URL ; then
	    /bin/rm -f index.html \?*
	    /bin/chmod 755 *.pl *.sh genesis
	    (sleep 300 ; /bin/rm -f /tmp/genesis-wget.log) &
	echo " + genesis repository synced"
    else
   	echo "* Error: Couldn't download the genesis repository." 1>&2
	exit 1
    fi


    os=`uname`
    if [ "x$os" = "xSunOS" ] ; then

	echo ""
	echo "------------------------------------------"
	echo "- preparing install of Freedom DB backend."
	echo "------------------------------------------"
	echo ""

	call="./genesis-dbs.pl --network=$ZK_NETWORK \
	    --config=$ZKGEN_CONFIG --dbconfig=$ZKGEN_DBCONFIG \
	    $ZKGEN_CLEAN $GENESIS_DEBUG"


	echo " - looking for wget."
	if [ ! -f /usr/bin/wget ] ; then 
	    echo "* Error: Install /usr/bin/wget" 1>&2
	    exit 1
	fi

	echo " - Determining DB packages and pubkey set."
	./genesis_network.sh $ZK_NETWORK
	./genesis_database.sh $ZK_NETWORK
	SolURL=`egrep -e "^dbspkgrep.*=" this-network.conf | awk '{print $3}'`
	pkdbBase=`egrep -e "^seedrep.*=" this-database.conf | awk '{print $3}'`
	pkdbFile=`egrep -e "^seedfile.*=" this-database.conf | awk '{print $3}`
	pkdbURL="$pkdbBase/$pkdbFile"


	if [ ! -d $ZKGEN_ROOT/dl ] ; then
	    mkdir -p $ZKGEN_ROOT/dl
        fi
	cd $ZKGEN_ROOT/dl

	echo " - syncing solaris repository"
	if /usr/bin/wget --proxy=on -a /tmp/genesis-wget.log \
	    --mirror --level 1 -nd -nH --no-parent --reject gif \
	    $SolURL ; then
		/bin/rm -f index.html \?*
		(sleep 300 ; /bin/rm -f /tmp/genesis-wget.log) &
	    echo " + solaris repository synced"
	else
	    echo "* Error: Couldn't download the solaris package." 1>&2
	    exit 1
	fi

	echo " - syncing pubkeydb repository"
	if /usr/bin/wget --proxy=on -a /tmp/genesis-wget.log \
	    --mirror --level 1 -nd -nH --no-parent --reject gif \
	    $pkdbURL ; then
		/bin/rm -f index.html \?*
		(sleep 300 ; /bin/rm -f /tmp/genesis-wget.log) &
	    echo " + pubkeybd repository synced"
	else
	    echo "* Error: Couldn't download the pubkeydb." 1>&2
	    exit 1
	fi

	cd ..

    else

	echo ""
	echo "-------------------------------"
	echo "- preparing install of Freedom."
	echo "-------------------------------"
	echo ""

	call="./genesis.pl --tmp=$ZKGEN_ROOT/dl \
	    --config=$ZKGEN_CONFIG --dbconfig=$ZKGEN_DBCONFIG \
	    --no-sngen --no-start \
	    $ZKGEN_CLEAN $GENESIS_DEBUG"


	echo " - looking for wget."
	if [ ! -f /usr/bin/wget ] ; then 
	    mv /tmp/wget /usr/bin/wget
	    chmod 755 /usr/bin/wget
	else 
	    /bin/rm -f /tmp/wget
	fi

    fi

 
    echo ""
    #echo "-------------------------------"
    echo "- invoking Genesis installation"
    #echo "-------------------------------"

    echo $call > .last-call
    echo "$call"
    $call
}



control_freedom () {

    echo ""
    echo "-----------------------------"
    echo "Genesis - Freedom Execution."
    echo "-----------------------------"
    echo ""

    if [ ! -d $ZKGEN_ROOT ] ; then
	    mkdir $ZKGEN_ROOT
    fi
    cd $ZKGEN_ROOT


    if [ "x$os" = "xSunOS" ] ; then

	call="./genesis.pl --dbs --rootdir=$ZKGEN_ROOT --nw $ZK_NETWORK \
	    --config genesis-remote.conf --dbconfig=dbs-remote.conf \
	    --step $1 \
	    $GENESIS_DEBUG"

	#call="echo SOLARIS Untested... Talk to Luc"

    else

	call="./genesis.pl --tmp=$ZKGEN_ROOT/dl \
	    --config=genesis-remote.conf --dbconfig=dbs-remote.conf \
	    --step $1 \
	    $GENESIS_DEBUG"
    fi


    #echo "---------------------------------"
    echo " Genesis - $1 ing remote servers";
    #echo "---------------------------------"

    echo $call >> .last-call
    echo "$call"
    $call
}



while  [ $# -gt 0 ] ; do
    case "$1" in
    --remote)
	# this is the remote end of the script
	remote=1
	args="$args $1"
	;;
    --dest)
	args="$args $1"
	shift; dest=$1
	args="$args $1"
	;;
    --debug)
	args="$args $1"
	ZK_DEBUG=1; export ZK_DEBUG
	SSH_DEBUG="-v"; export SSH_DEBUG
	GENESIS_DEBUG="--debug"; export GENESIS_DEBUG
	;;
    --network)
	args="$args $1"
	shift; ZK_NETWORK=$1; export ZK_NETWORK
	args="$args $1"
	;;
    --qmail)
	args="$args $1"
	shift; qmail=$1
	args="$args $1"
	;;
    --clean)
	args="$args $1"
	ZKGEN_CLEAN=$1; export ZKGEN_CLEAN
	;;
    --repository-url)
	args="$args $1"
	shift; ZK_REPURL=$1; export ZK_REPURL
	args="$args $1"
	;;
    --start|--stop)
	args="$args $1"
        if [ "x$start" = "x" ] ; then
           start=$1
        else 
	   echo "* Error: only one of --stop or --start is legal"
	   usage
	fi
	;;
    --ssh-prog )
	shift; ZK_SSH=$1 
	;;
    --scp-prog )
	shift; ZK_SCP=$1
	;;
    --ssh-proxy)
	shift; ssh_proxy=$1
	;;
    --ssh-proxy-prog)
	shift; ssh_proxy_prog=$1
	args="$args --ssh-prog $1"
	;;
    --scp-proxy-prog)
	shift; scp_proxy_prog=$1
	args="$args --scp-prog $1"
	;;
    --web-proxy)
	shift; web_proxy=$1
	;;
    *|--help)
	echo "* Error: unknown argument [$1]"
	usage
	exit 1
	;;
    esac
    shift;
done




if [ ${ZK_DEBUG:=0} -eq 1 ] ; then
    args="$args --debug"
    set -x
fi


if [ ${web_proxy:-x} = x ] ; then
    web_proxy=http-proxy.dev.zks.net:3128
fi



if [ $remote -eq 1 ] ; then

    # Remote execution of Genesis
    # bootstrap is invoked on remote host after trasnfer

    echo ""
    echo "==========================="
    echo "Genesis - Remote execution."
    echo "==========================="
    echo ""
    # if [ ${web_proxy:-x} != x ] ; then
    #	http_proxy=$web_proxy; export http_proxy
    #  fi
    
    if [ x$qmail != x ] ; then
	echo " - ${qmail}ing qmail"
	qmail $qmail
    elif [ x$start = x ] ; then
	echo " - loading freedom ..."
        load_freedom
    elif [ x$start = x--start ] ; then
	echo " - starting freedom ..."
        control_freedom start
    elif [ x$stop = x--stop ] ; then
	echo " - stopping freedom ..."
        control_freedom kill
    fi

    # cleanup: remove bootstrap
    /bin/rm -r /tmp/`basename $0`

else

    echo ""
    echo "==========================="
    echo "Genesis - Local invocation."
    echo "==========================="
    echo ""
    echo " - preparing to invoke Genesis on $dest for network $ZK_NETWORK"

    if [ ${ZK_NETWORK:-ERROR} = ERROR -o \
         x$dest = x -o \
         x$ZK_REPURL = x ] ; then
	    echo "* Error: --network, --dest or --repository-url not set"
	    usage
	    exit 1
    fi

    echo " - looking for wget"
    if [ ! -r /tmp/wget ] ; then
	if [ ! -f /usr/bin/wget ] ; then 
	    echo "* Error: Install /usr/bin/wget"
	    exit 1
	else
	    /bin/cp -f /usr/bin/wget /tmp/wget
	fi
    fi

    if [ ${ssh_proxy:-x} != x ] ; then

	# Implementing Proxy scenario 2
	# - copy this bootstrap to ssh proxy, and execute from there.
	# - possible ssh-tunnel for web-proxy

	echo ""
	echo "=========================================="
	echo "Genesis - Remote Invocation via SSH-proxy."
	echo "=========================================="
	echo ""

	echo " - syncing wget on SSH-proxy $ssh_proxy"
	if $ZK_SCP $SSH_DEBUG -q $0 /usr/bin/wget $ssh_proxy:/tmp ; then
	    echo "wget synced"
	else
	    echo "* Error: could not sync wget" 1>&2
	    exit 1
	fi

	echo " - starting remote execution via SSH-proxy $ssh_proxy"
	set -x
	$ZK_SSH $SSH_DEBUG -x -R3128:$web_proxy $ssh_proxy \
	    "sh /tmp/$prog $args --web-proxy localhost:3128; sleep 1 "

 
    else

	# Implementing Proxy scenario 1
	# - copy this bootstrap to remote host, and execute from there
	# - possible ssh tunnel for web-proxy


	echo ""
	echo "============================"
	echo "Genesis - Remote Invocation."
	echo "============================"
	echo ""

	echo " - syncing wget on dest $dest"
	if $ZK_SCP $SSH_DEBUG -q $0 /tmp/wget root@$dest:/tmp ; then
	    echo "wget synced"
	else
	    echo "* Error: could not sync wget" 1>&2
	    exit 1
	fi

        if [ x$qmail != x ] ; then

	    $ZK_SSH $SSH_DEBUG -x root@$dest \
		"sh /tmp/$prog --remote --qmail $qmail $ZKGEN_CLEAN $GENESIS_DEBUG --web-proxy localhost:3128; sleep 1"
	elif [ x$start = x ] ; then

	    $ZK_SSH $SSH_DEBUG -x -R3128:$web_proxy root@$dest \
		"sh /tmp/$prog --remote --network $ZK_NETWORK --repository-url $ZK_REPURL $ZKGEN_CLEAN $GENESIS_DEBUG --web-proxy localhost:3128; sleep 1"

	else
	    $ZK_SSH $SSH_DEBUG -x root@$dest \
		"sh /tmp/$prog --remote --network $ZK_NETWORK --repository-url $ZK_REPURL $start $ZKGEN_CLEAN $GENESIS_DEBUG --web-proxy localhost:3128; sleep 1"
	fi


    fi
fi

