#!/usr/bin/perl -w
#
# Genesis Freedom Network Builder for the database servers
#
# $Id: genesis-dbs.pl,v 1.18 2001/04/24 16:28:47 francisk Exp $
# Luc Boulianne <luc@zks.net>
# Zero-Knowledge Systems Inc.
#

use IniConf;
use strict;
use Getopt::Long;
use Expect;

my (%Net, %Network, %HostEntity);
my ($MySection);
my ($MyNetworkSection);
my ($MyDomain);
my (@DBs, %DBs);


my ($CONFIG) = "genesis.conf";
my ($DBCONFIG) = "dbs.conf";
my ($PWD) = "/bin/pwd";
my ($cfg, $dbcfg, $group, $sect, $param);

my ($NOSW_INST) = 0;
my ($DO_CLEAN) = 0;
$ENV{GEN_DEBUG} = 0;
my ($tdir) = "/usr/genesis";

##
## Options related part
##
my %optctl  = ();

$| = 1;

my ($ProgName) = $0;

$ProgName =~ s?^.*/??;

sub title {
	print <<'EOH';
===============================================================
Genesis - Freedom Network Creator (Database Server)
Luc Boulianne <luc@zeroknowledge.com>
$Id: genesis-dbs.pl,v 1.18 2001/04/24 16:28:47 francisk Exp $

EOH
	print "[$ProgName]\n\n";

}

sub usage {
	title;
	print <<'EOH2';
Usage: genesis-dbs.pl

        --help
        --network [name] 
        --step [start|kill]
        --config filename
        --dbconfig filename
        --clean
	--debug 

	* Start by configuring the network by editing the
	  genesis.conf file
	* Run this script (typically found in /home/caesar/genesis)
	  on the local machine 
	* The servers should start be started up for you
        * Do not forget to checkin your changes

===============================================================
EOH2
}

my($res) = GetOptions(\%optctl,
                  "help",
		  "debug",
                  "config=s",
                  "dbconfig=s",
		  "network=s",
		  "step=s",
                  "clean",
                  "version") || die "Error in Getopt";

title;
## print out the usage string
##
if (defined($optctl{'help'})){
  usage;
  exit 1;
}

if (! defined($optctl{'network'})){
  usage;
  exit 1;
}

if (defined($optctl{'config'})){
  $CONFIG=$optctl{'config'};
}
if (defined($optctl{'dbconfig'})){
  $DBCONFIG=$optctl{'dbconfig'};
}

## set debug mode on
##
if (defined($optctl{'debug'}) || defined($ENV{'DEBUG'})){
    $ENV{GEN_DEBUG} = 1;
    print "Debug mode On\n";
}

if ( $#ARGV >= 0 ) {
	usage;
	exit 2;
}

sub tail {
  my ($file) = @_;
  print "--------------------------------------------------------------------\n";
  advise ($file . "\n");
  system ("/usr/bin/tail", "$file");
  print "--------------------------------------------------------------------\n";
}


my ($dbugnl) = 1 ; # Last message ended with a newline

sub debug {
  my ($msg) = @_;

  if (!$ENV{GEN_DEBUG}) {return};

  if ($dbugnl) {
    print STDERR "[debug] ";
  } 
  print STDERR $msg;
  if ($msg =~ m/\n$/) {
    $dbugnl = 1;
  } else {
    $dbugnl = 0;
  }
}

my ($advisenl) = 1 ; # Last message ended with a newline
sub advise {
  my ($msg) = @_;
  if ($advisenl) {
    print "# ";
  } 
  print $msg;
  if ($msg =~ m/\n$/) {
    $advisenl = 1;
  } else {
    $advisenl = 0;
  }
}

my ($msgnl) = 1 ; # Last message ended with a newline
sub msg {
  my ($msg) = @_;
  if ($msgnl) {
    print "\n[===]\n[===] ";
  } 
  print $msg,"[===]\n";
  if ($msg =~ m/\n$/) {
    $msgnl = 1;
  } else {
    $msgnl = 0;
  }
}

sub wget {
  my ($url,$dest) = @_;
  my ($d);
  if ($ENV{"GEN_DEBUG"}) {
    $d = "-nv";
  } else {
    $d = "-nv";
  }
  return 1 if ($NOSW_INST);
  advise ("Downloading [$url]\n");
  system("wget $d -m -l1 -nd $url 1>&2");
  if ($?) {die "Can't retrieve RPM from $url"};
}

sub chown_by_name {
  my ($user, @list) = @_;
  chown ((getpwnam($user))[2,3], @list);
}

sub rm_rpm {
  my ($rpm) = @_;
  return 1 if ($NOSW_INST);
  system ("rpm -e $rpm");
  #if ($?) {die "Can't remove RPM $rpm"};
}

sub inst_rpm {
  my ($rpm) = @_;
  return 1 if ($NOSW_INST);
  advise ("Installing RPM $rpm. One moment... \n");
  system ("rpm --nodeps -i $rpm");
  if ($?) {die "Can't install RPM $rpm"};
}

sub inst_solpkg {
  my ($pkg, $root) = @_;
  # my ($rdr, $wtr, $err) = (FileHandle->new, FileHandle->new, '');

  return 1 if ($NOSW_INST);
  advise ("Installing Solaris DB pkg $pkg at $root. One moment... \n");

  if ( -f $pkg ) {
    my ($pos, $error, $string, $before, $after);
    my ($pkgadd);

    ($pkgadd = Expect->spawn("pkgadd -d $pkg -R $root")) || die ("Can't start pkg installer for $pkg");
    if ( !defined ($pkgadd) ) {
	die ("Couldn't spawn [pkgadd -d $pkg -R $root]");
    }
    advise (" - pkgadd spawned\n");
    while ( (($pos, $error, $string, $before, $after) 
		= $pkgadd->expect(30, 
		   "all packages). (default: all) [?,??,q]:",
		   "Do you want this directory created now [y,n,?,q]",
		   "Do you want to continue with the installation of <ZKSfrdbs> [y,n,?]",
		   "Installation of <ZKSfrdbs> was successful.") )
	   && defined ($pos)) {
	if ( defined ($error)) {
	   die ("Unexpected Error: $error");
	}
	if ($pos == 1) {
	    print "[1: $string]\n";
	    print $pkgadd "\n";
        } elsif ($pos == 2 || $pos == 3) {
	    print "[2/3: $string]\n";
	    print $pkgadd "y\n";
	} elsif ($pos == 4) {
	    print "[4: $string]\n";
	    last;
	} else {
	    die "Got something I didn't expect at [$pos]:".$pkgadd->exp_error()."\n";
        }
    }
    if ( ! defined ($pos)) {
	warn "Got something I didn't expect:".$pkgadd->exp_error()."\n";
    }
    $pkgadd->soft_close();
    advise (" - pkgadd completed\n");
  } else {
    die ("No pkg found at $pkg");
  }
}

sub inst_pkg {
  my ($pkg, $root) = @_;
  return 1 if ($NOSW_INST);
  advise ("Installing Solaris pkg $pkg at $root. One moment... \n");
  if ( -f $pkg ) {
    system ("pkgadd -d $pkg -R $root");
    if ($?) {die "Can't install pkg $pkg"};
  } else {
    die ("Not pkg found at $pkg");
  }
    
}

sub rm_pkg {
  my ($pkg, $root) = @_;
  return 1 if ($NOSW_INST);
  advise ("Removing Solaris pkg $pkg at $root. One moment... \n");
  if ( -f $pkg ) {
    $pkg =~ s/.pkg//;
    system ("pkgrm $pkg -R $root");
    if ($?) {die "Can't install pkg $pkg"};
  } else {
    die "Not pkg found at $pkg";
  }
    
}

sub killall {
  my ($list, $sig) = @_;
  system ("kill $sig $list > /dev/null 2>&1");
}

sub find_ip {
  my ($n) = @_;
  my ($name);
  $name = $n;
  $name = `host $name | grep address`;
  if ($?) {
    warn "Can't find IP address of $n"; 
    undef;
  } else {
    $name =~ m/([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/;
    $1;
  }
}

sub load_file {
  my ($name, $buf) = @_;

  if (open (_F, "$name")) {
    @{$buf} = <_F>;
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name]: $!");
    0;
  }
}

sub save_file {
  my ($name, $buf) = @_;

  if (open (_F, ">$name")) {
    print _F join ("", @{$buf});
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name] for writing: $!");
    0;
  }
}

sub sed {
  my ($buf, $expr) = @_;

  eval "\$buf =~ $expr";
  $buf;
}

sub install_file {
  my ($path) = @_;
  if ( ! -f $path ) {
    warn "source path to $path does not exist"; 
    undef;
  } else {
    `tar cf - $path | (cd / ; tar xpf -)`;
    if ($?) {
      warn "Can't copy path $path"; 
      undef;
    }
    1;
  }
}

sub system_freedom {
  my(@cmd) = @_;
  my($cmd);
  
  $cmd = join (" ", @cmd);
  `echo $cmd | su freedom 1>&2`;
}

# ======================================================================
my ($login) = `id | tr ' ' '\012' | grep uid | sed -e 's/^.*(\\(.*\\))/\\1/'`;
chomp($login);
if ($login ne "root") {
  print STDERR "User $login does not have sufficient priviledges to\n" .
    "install this network. Become root.\n";
  exit 2;
}

msg ("Loading configuration files\n");

$cfg = IniConf->new( -file => $CONFIG, -nocase => 1, -default => "Default" );

if ( ! defined ($cfg) ) {
  print STDERR "Can't initialize configuration from file $CONFIG";
  print STDERR join ('\n', @IniConf::errors);
}
   
if ( ! defined ($cfg->ReadConfig) ) {
  print STDERR "Can't read configuration file $CONFIG";
  print STDERR join ('\n', @IniConf::errors);
}

$dbcfg = IniConf->new( -file => $DBCONFIG, -nocase => 1, -default => "Default" );
if ( ! defined ($cfg) ) {
  print STDERR "Can't initialize configuration from file $DBCONFIG";
  print STDERR join ('\n', @IniConf::errors);
}
   
if ( ! defined ($dbcfg->ReadConfig) ) {
  print STDERR "Can't read configuration file $DBCONFIG";
  print STDERR join ('\n', @IniConf::errors);
}

foreach $sect ($cfg->NonGroupSections) {
  debug ("Section = $sect\n");
}

foreach $group ($cfg->Groups) {
  debug ("Group = $group\n");
}

my ($section, $hostentity, $hname );
foreach $section ($cfg->Sections) {
  if ($section =~ m/^network\s+(.+)/) {
    foreach $hostentity ($cfg->GroupMembers($1)) {
      $hname = $cfg->val($1 . " $hostentity", "hostname");
      if ( ! defined($Net{$1}) ){
	$Net{$1} = [];
      }
      push (@{$Net{$1}}, $hname);
      $Network{$hname} = $1;
      $HostEntity{$hname} = $hostentity;
      debug ("Processed host $hname for network $1\n");
    }
  } 
}

# 1. Gather some config information
# 2. Determine where you belong
# 3. Downloading the RPMs

my ($Hostname, $FQN);
my ($Pwd) = `$PWD`;

chomp ($Pwd);
chop($Hostname = `hostname`);

# HACK!!!
if ($Hostname !~ m/dev.zks.net/) {
  $FQN = $Hostname . ".dev.zks.net";
} else {
  $FQN = $Hostname;
}  
advise ("FQN is $FQN\n");

sub freespace {
  my ($part) = @_;
  my ($x);
  $part =~ s/\/genesis$//;
  $x = `df -k $part | tail -1`;
  debug ($x);
  $x=(split(/\s+/,$x))[3];
  debug ("$x\n");
  $x;
}

sub KillServers {
  my ($nw) = @_;
  my (@myps);
  my (@piddeathrow);
  
  advise ("Killing previous servers for network [$nw]\n");
  
  @myps = `/bin/ps -aef | egrep $nw | egrep -v egrep | egrep -v $ProgName | egrep 'zkdbsrvd' `;
  if (scalar (@myps) > 0) {
    @piddeathrow = map {/\s*(\d+)/} @myps;
    print join("\n",@myps);
    killall ("@piddeathrow", -15);
  } else {
    advise ("No processes to kill\n");
    return 1;
  }
  
  # If processes don't react to SIGTERM then this is a problem to be analyzed: don't do a SIGKILL blindly.
   advise ("Waiting 15 seconds for db process to die.\n");
  if ( sleep (15) < 13 ) {
    advise ("WARNING: sleep() is misbehaving.\n");
  }
  
  @myps = `/bin/ps -aef | egrep $nw | egrep -v egrep | egrep -v $ProgName | egrep 'zkdbsrvd' `;
  if (scalar (@myps) > 0) {
    advise ("ERROR: Not all processes have died.\n");
    print STDERR join("\n",@myps);
    die ("Aborting...");
  }
}

sub InstallSolarisPkg {
  my($network) = @_;
  my($netsect, $x, $y);

  $netsect = "network " . $network;

  if ($NOSW_INST) {
    advise ("NOTE: NO Software Installation Option in effect. NO RPM update will occur\n");
  }

  msg ("Stopping previous servers\n");
  KillServers ($network);

  msg ("Uninstalling older pkg\n");
  if ($DO_CLEAN) {
    msg ("Removing all trace of previous installation\n");
    if (defined ($cfg->val($netsect, "dbspkgroot")) ) {
      # BUG: pkgrm should probably be used here
      advise("running [/bin/rm -rf " . $cfg->val($netsect, "dbspkgroot") . "/opt/freedom/]\n");
      system("/bin/rm", "-rf", $cfg->val($netsect, "dbspkgroot") . "/opt/freedom/");
    } else {
      advise ("ERROR: dbspkgroot not defined");
    }
  }
  msg ("Installing Solaris Package for network $network\n");
  if ( defined ($cfg->val($netsect, "dbspkgfile")) &&
       defined ($cfg->val($netsect, "dbspkgroot")) ) {
    if ( ! -d $cfg->val($netsect, "dbspkgroot") ) {
	advise ("Creating " . $cfg->val($netsect, "dbspkgroot") . "\n");
 	mkdir $cfg->val($netsect, "dbspkgroot"), 0755;
    }
    if ( ! -d $cfg->val($netsect, "dbspkgroot") . "/etc" ) {
	advise ("Creating " . $cfg->val($netsect, "dbspkgroot") . "/etc\n");
	mkdir $cfg->val($netsect, "dbspkgroot") . "/etc", 0755;
	symlink "/etc/passwd", $cfg->val($netsect, "dbspkgroot") . "/etc/passwd";
	symlink "/etc/group", $cfg->val($netsect, "dbspkgroot") . "/etc/group";
    }
    if ( defined ($cfg->val($netsect, "dbspkgldir")) ) {
      inst_solpkg($cfg->val($netsect, "dbspkgldir") . "/" . $cfg->val($netsect, "dbspkgfile"), 
		  $cfg->val($netsect, "dbspkgroot"));
    } else {
      inst_solpkg($cfg->val($netsect, "dbspkgfile"), 
		  $cfg->val($netsect, "dbspkgroot"));
    }
    $x = $cfg->val($netsect, "dbspkgroot") . "/opt/freedom";
    $y = "/freedom-" . $network;
    `rm $y`;
    `echo ln -s $x $y 1>&2`;
    `ln -s $x $y 1>&2`;
    # To handle a dummy account for user 'freedom'
    if ( ! -d "/usr/freedom" ) {
 	mkdir "/usr/freedom", 0755;
        chown_by_name ("freedom", "/usr/freedom");
    }
    if ( ! -l "/freedom" ) {
	symlink "/usr/freedom", "/freedom";
    }
    return (1);
  } else {
    advise ("No Database information in [$netsect]\n");
    return (0);
  }
}

sub InstallLinuxRPM {
  my($network) = @_;
  my ($rpms) = "rpms pre-v1.1";

  if ($NOSW_INST) {
    advise ("NOTE: NO Software Installation Option in effect. NO RPM update will occur\n");
  }
  msg ("Downloading RPMs\n");

  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "base"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "knt"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "fmg"));
  
  # 4. Backup/wipe the previous installation
  
  msg ("Stopping previous servers\n");
  
  killall ("killall fraipd frmaipd", "-15");
  sleep(1);
  killall ("killall fraipd frmaipd", "-9");
  
  msg ("Uninstalling older RPMs\n");

  rm_rpm ("freedom-dbs");
  rm_rpm ("freedom");
  if ($DO_CLEAN) {
    msg ("Removing all trace of previous installation\n");
    system("find /usr/freedom ! -type d -print | xargs rm -f 1>&2");
  }
  
  # 5. Install the new rpms
  
  msg ("Installing " . $cfg->val($MyNetworkSection, "release") . " RPMs\n");
  
  $ENV{ZK_AGREE}=1;
  inst_rpm($tdir . "/". $cfg->val($rpms, "base"));
  killall("fr_probe_net", "-15");
  inst_rpm($tdir . "/". $cfg->val($rpms, "dbs"));

}

sub InstallPkg {
  my($network) = @_;
  if (defined $dbcfg->val($network . " general", "os") &&
      ($dbcfg->val($network . " general", "os") eq "linux")) {
      InstallLinuxRPM($network);
  } else {
      InstallSolarisPkg($network);
  }
}

my (%DBMapping) = (
		   "pubkeydb"		=> "PubkeyDB",
		   "pubkeydb-kqd"	=> "PubkeyDB_kqd",
		   "pubkeydb-nuked"	=> "PubkeyDB_Nuked",
		   "nymsdb"	 	=> "NymsDatabase",
		   "nymsdb-nuked"	=> "NymsDatabase_Nuked",
		   "spenttokendb" 	=> "SpentTokenDB",
		   "nymcache" 		=> "NymCache",
		   "mailstatdb" 	=> "MailStatDB",
		   "nymblockdb" 	=> "NymBlockDB"
);

my (%tmp);
sub config_db {
  my ($dbname, $root, $port, $authkey) = @_;
  my ($env, $val);

  if (defined ($tmp{$dbname})) {
    advise ("$dbname already done\n");
    return 1;
  }    
  $tmp{$dbname} = 1;


  $ENV{"zkfr_root"} = $root;

  $env = "zkfr_" . $DBMapping{$dbname} . "_Server";
  $val = $FQN .":". $port;
  $ENV{$env} = $val;
  debug ("$env = $val\n");

  $env = "zkfr_" . $DBMapping{$dbname} . "_ServerPort";
  $val = $port;
  $ENV{$env} = $val;
  debug ("$env = $val\n");

  $env = "zkfr_" . $DBMapping{$dbname} . "_AuthKey";
  $val = $authkey;
  $ENV{$env} = $val;
  debug ("$env = $val\n");

  if ( ! -f $root . "/etc/db/$dbname.local.conf.tmpl" ) {
    advise ("ERROR: unable to locate the db config file [$root/etc/db/$dbname.local.conf.tmpl]\n");
  } else {
    debug("$Pwd/genesis-confsub-dbs.sh $root/etc/db/$dbname.local.conf.tmpl $root/etc/db/$dbname.local.conf\n"); 
    `$Pwd/genesis-confsub-dbs.sh $root/etc/db/$dbname.local.conf.tmpl $root/etc/db/$dbname.local.conf 1>&2` ; 
    if (`egrep "\@..*\@" $root/etc/db/$dbname.local.conf`) {
      advise ("WARNING: incomplete substitution for $dbname in \n\t[$root/etc/db/$dbname.local.conf\n");
      0;
    } else {
      1;
    }
  }
}
my (%Start);

sub AdjustDBSetup {
  my($network) = @_;
  my($root) = "/freedom-" . $network;
  my($netsect, $x, $dbname);
  my($ndbutil_config) = $root . "/etc/freedom-ndbutil.conf";
  
  msg ("Adjusting database configurations\n");
  foreach $dbname (@DBs) {
    debug ("Testing remote db [$dbname]: \n");
    if ($dbname ne "general" && $dbcfg->val($network . " " . $dbname, "isremote") eq "yes") {
      advise ("Configuring for remote db [$dbname]: \n");
      $Start{$dbname} = $dbname; # So that we know what to start later on
      $x = $network . " " . $dbname;
      debug ("$x\n");
      if (defined ($dbcfg->val($x, "port")) &&
	  defined ($dbcfg->val($x, "authkey"))) {
	if (config_db ($dbname, $root, $dbcfg->val($x, "port"),
		       $dbcfg->val($x, "authkey"))) {
	  debug ("--- $dbname OK\n");
	} else {
	  debug ("--- $dbname ERROR\n");
	}
      } else {
	advise ("Missing port or authkey in section [$x]\n");
      }
    }
  }

  advise ("Configuring $ndbutil_config: \n");
  $ENV{"zkfr_root"} = $root;
  debug ("zkfr_root = $root\n");
  if ( ! -f $ndbutil_config . ".tmpl" ) {
    advise ("ERROR: unable to locate the config file [$ndbutil_config.tmpl]\n");
  } else {
    debug("$Pwd/genesis-confsub-dbs.sh $ndbutil_config.tmpl $ndbutil_config\n"); 
    `$Pwd/genesis-confsub-dbs.sh $ndbutil_config.tmpl $ndbutil_config 1>&2`; 
    if (`egrep "\@..*\@" $ndbutil_config`) {
      advise ("WARNING: incomplete substitution in \n\t[$ndbutil_config\n");
    } else {
      advise ("Completed substitution in \n\t[$ndbutil_config]\n");
    }
  }

  `$Pwd/fixroot.sh $root/etc/rc/fr-dbsrv.rc $root`;
  if ($?) {
    advise ("Adjusted [$root/etc/rc/fr-dbsrv.rc]\n");
  } else {
    advise ("Problem adjusting [$root/etc/rc/fr-dbsrv.rc]\n");
  }
  `cp $Pwd/access.conf $root/etc/db/access.conf 1>&2`;
  if ($?) {
    advise ("Installed [$root/etc/db/access.conf]\n");
  } else {
    advise ("Problem setting [$root/etc/db/access.conf]\n");
  }
}

sub SeedDB {
  my ($network) = @_;
  my (%tmpx);
  my ($root) = "/freedom-" . $network;
  my ($seeddir) = "";
  my ($netsect, $seedfile, $seed, $dbname, $ans);
  my ($seedprog);

  # try explicit db_load_2.7.7 first
  if ( -f "/usr/local/bin/db_load_2.7.7") {
    $seedprog = "/usr/local/bin/db_load_2.7.7";
  } else {
    $seedprog = "/usr/local/bin/db_load";
  }

  msg ("Seeding Databases: " . join (", ", @DBs) . "\n");
  foreach $dbname (@DBs) {
    
    debug ("Currently at: $dbname\n");
    
    if (defined ($tmpx{$dbname})) {
      return 1;
    }    
    $tmpx{$dbname} = 1;
    
    $seeddir = $dbcfg->val($network . " " . $dbname, "seeddir");
    if (defined ($seeddir)) {
      $seed = "$seeddir/";
    }
    $seedfile = $dbcfg->val($network . " " . $dbname, "seedfile");
    if (defined ($seedfile)) {
      $seed .= $seedfile;
    }
    if (defined ($seed) && -f "$seed" && $DO_CLEAN) {
      advise ("Loading the seedfile [$seed] database for [$dbname]\n");
      debug ("$seedprog $root/db/$dbname/$dbname/$dbname < $seed 1>&2");
      `$seedprog $root/db/$dbname/$dbname/$dbname < $seed 1>&2`;
      if ($?) {
	advise ("Load OK\n");
      } else {
	advise ("Return code ($?) non-zero. Probably OK\n");
      }
    } elsif (defined ($seed) && ! -f $seed) {
      advise ("Unable to locate seedfile: $seed\n");
    }
  }
}

sub CleanUp {
  my($network) = @_;
  my($root) = "/freedom-" . $network . "/";

  advise ("Adjusting permissions\n") ;
  `chown -R freedom $root/* 1>&2` ;
  advise ("Removing crontab\n") ;
  `crontab -r freedom`; # -u option not required on Solaris, required on Linux.
}

sub StartServers {
  my($network) = @_;
  my($root) = "/freedom-" . $network;
  my(@start, $prog);

  msg ("Starting Database Servers\n");

  @start = `ls $root/etc/rc/S*`;

  `find $root/db \\( -name __\\* -o -name log\\* \\) -print | xargs -t rm 1>&2`;
  foreach $prog (sort @start) {
    my ($srv);
    chomp ($prog);
    $srv = $prog;
    $prog =~ s/^.+S...\.//;
    advise ("Starting [$prog] $srv: \n");
    system_freedom ("$srv", "start");  
    if ($?) {
      sleep (3);
      advise ("Problems starting [$srv]\n");
      advise ("For more info: $root/var/log/zkdbsrv-$prog.log\n");
      tail ("$root/var/log/zkdbsrv-$prog.log");
#     die "Problem starting service"; 
    }
    sleep(3);
    advise ("for more info: tail $root/var/log/zkdbsrv-$prog.log\n");
    tail ("$root/var/log/zkdbsrv-$prog.log");
  }
}

my($net);

if (defined ($optctl{'network'})) {
  $net = $optctl{'network'};
  if (defined($optctl{'clean'})) {
    $DO_CLEAN = 1;
  }
  if (defined ($optctl{'step'})) {
    if ($optctl{'step'} eq "start") {
      StartServers ($net);
    } elsif ($optctl{'step'} eq "kill") {
      KillServers ($net);
    }
  } else{
    if (InstallPkg ($net)) {
      @DBs = $dbcfg->GroupMembers($net);
      # Bug - the [general] member is NOT a database.
      debug ("Databases: ". join (", ", @DBs) . "\n");
      AdjustDBSetup ($net);
      SeedDB ($net);
      CleanUp ($net);
      StartServers($net);
    }
  }
}else{
  advise ("Error: the --network must be specified.\n");
}

msg ("Solaris Installation complete.\n");

