#!/usr/bin/perl -w
#
# Genesis Freedom Network Builder
#
# $Id: genesis.pl,v 1.145 2001/05/17 16:49:46 thierry Exp $
# Luc Boulianne <luc@zks.net>
# Zero-Knowledge Systems Inc.
#

use IniConf;
use strict;
use Getopt::Long;
use Carp;
use Socket;

my $Version = '$Revision: 1.145 $'; #' Help emacs

$Version =~ s/\$//g;
$Version =~ s/Revision: //;

#
# %Net: $Net{netname} = @(list of hostnames in this net)
#
my (%Net);

#
# %Network: $Network{Hostname} = netname
#
my (%Network);

#
# %HostEntity: $Hostentity{Hostname} = hostentity-name
#
my (%HostEntity);

#
# MySectionName = [$Network{$Hostname} . " " . $HostEntity{$Hostname}]
 #
my ($MySection);

#
# $MyNetworkSection) = "network " . $Network{$Hostname};
#
my ($MyNetworkSection);

#
# My DomainName
#
my ($MyDomain);

#
# @DBs = $dbcfg->GroupMembers($Network{$Hostname});
#
my (@DBs);

#
#
# Scan all databases in our network and see which servers our host needs to
# use (ie: all of the databases we need to connect to)
#
# $DBs{$dbname} .= join (" ", @found);
#
my(%DBs);

#
#
# Scan all databases in our network and see which servers our host needs to
# use (ie: all of the databases we need to connect to)
#
# $IPAddr{$hostname} = ip-address-of-hostname;
# $IPNATAddr{$hostname} = nat-ip-address-of-hostname;
#
my (%IPAddr);
my (%IPNATAddr);

#
# Required databases for my services
#
my (%DBrequired);

my ($CONFIG) = "genesis.conf";
my ($DBCONFIG) = "dbs.conf";
my ($MachineMOTD) = "/etc/motd";
my ($PWD) = "/bin/pwd";
my ($rpms, $release,  $coreaip_rpms);
my ($cfg, $dbcfg, $group, $sect, $param);

my ($NOSW_INST) = 0;
my ($NO_WGET) = 0;
my ($DO_CLEAN) = 0;

$ENV{GEN_DEBUG} = 0;
my ($tdir) = "/usr/genesis";
my ($RootDir) = "/usr/freedom";     #Start to thing in terms of relative root
$ENV{ZK_FREEDOM_ROOTDIR} = $RootDir;
my $REPLICATOR_EXISTS=0;

my (%KeyPrivFileMapping, %MACKeyFileMapping, %Servers);

%KeyPrivFileMapping = (
		   "frmonth" => "FrMonth",
		   "aip"    => "AIP",
		   "maip"   => "MAIP",
		   "nymsrv" => "NymServer",
		   "toksrv" => "TokenServer",
		   "keyupd" => "KeyUpdServer",
		   "keyqry" => "KeyQryServer",
		   "niqs"   => "NIQSD",
		   "niss"   => "NISSD",
		   "nidb"   => "NIDB",
		   "fmg"    => "FMG",
		   "fmgmaip"  => "FMG"
		  );

%MACKeyFileMapping = (
                   "nymsrv" => "privkey/NymServer/mailmackey",
                   "imepb"  => "qmail/control/mailmackey",
                   "nmta"   => "qmail/control/mailmackey",
                   "nmtab"  => "qmail/control/mailmackey",
                   "pop"    => "qmail/control/mailmackey",
                  );

%Servers = (
	    "aip"    => "fraipd",
	    "fmg"    => "fmg",
	    "fmgmaip"=> "fmg-maip",
	    "keyqry" => "kqd",
	    "keyupd" => "frkscpd",
	    "maip"   => "frmaipd",
	    "niqs"   => "frniqsd",
	    "niss"   => "frnissd",
	    "nntp"   => "nntpserver",
	    "nymsrv" => "frnscpd",
	    "post"   => "frpostnews",
	    "replic" => "frreplicd",
	    "toksrv" => "frsnapd",
	    "lode"   => "frloded",
            "imep"   => "xxx-imep",
            "nmta"   => "xxx-nmta",
            "pop"    => "xxx-pop"
	   );

my (%DBMapping) = (
		   "pubkeydb"		=> "PubkeyDB",
		   "pubkeydb-kqd"	=> "PubkeyDB_kqd",
		   "pubkeydb-nuked"	=> "PubkeyDB_Nuked",
		   "nymsdb"	 	=> "NymsDatabase",
		   "nymsdb-nuked"	=> "NymsDatabase_Nuked",
		   "spenttokendb" 	=> "SpentTokenDB",
		   "nymcache" 		=> "NymCache",
		   "mailstatdb" 	=> "MailStatDB",
		   "nymblockdb" 	=> "NymBlockDB"
);

##
## Options related part
##
my %optctl  = ();

$| = 1;

my ($pwd) = `$PWD`;

sub title {
	print <<"EOH";
===============================================================
Genesis v$Version - Freedom Network Generator
EOH
	print <<'EOH';
Luc Boulianne <luc@zeroknowledge.com>
$Id: genesis.pl,v 1.145 2001/05/17 16:49:46 thierry Exp $

EOH
}

sub usage {
	title;
	print <<'EOH2';
Usage: genesis.pl

	--help                  : this help
        --backup                : make a backup of the current system w/o log files
	--check                 : check the setup
        --clean                 : do a clean install, wipe all files
	--config {filename}     : an alternate configuration file
	--dbconfig {filename}   : an alternate database configuration file
	--db_flush              : erase nidb and niqscache files
	--dump                  : dump the host assignment
	--debug                 : enter in debug mode
	--kill                  : kill all servers
        --step [rpm|conf|keys|nidb|db|clean|qmail_conf|mailmackey] 
        --step [kill|start|test|serials|extras|sendmail] 
                                : Do only one of these steps 
	--show {network}        : summarize the configurations
        --restart [fraipd|frmaipd] 
                                : Restart this service
        --tmp [tmpdir]          : select another temp directory

	--download-only {network} : download the software for this network
	--no-download           : don't download the latest software
	--no-wget               : do not do any wgets
	--no-sngen              : don't generate serial numbers
	--no-start              : don't start the servers
	--version               : get version number

	* Start by configuring the network by editing the
	  genesis.conf file
	* Run this script (typically found in /home/caesar/genesis)
	  on the local machine 
	* The servers should start be started up for you
        * Don't forgot to checkin your changes

===============================================================
EOH2
}
my($res) = GetOptions(\%optctl, "help",
		  "backup",
		  "check",
		  "clean",
		  "dump",
                  "config=s",
                  "dbconfig=s",
		  "debug",
		  "keep",
		  "restart=s",
		  "step=s",
		  "show:s",
		  "hostname=s",
		  "tmp=s",
		  "download-only=s",
		  "no-download",
                  "no-wget",		      
		  "no-sngen",
		  "no-start",
                  "version") || die "Error in Getopt";

## print out the usage string
##
if (defined($optctl{'help'})){
  title;
  usage;
  exit 1;
}

if (defined($optctl{'version'})){
  title;
  exit 1;
}

## set debug mode on
##
if (defined($optctl{'debug'}) || defined($ENV{'DEBUG'})){
    $ENV{GEN_DEBUG} = 1;
    print "Debug mode On\n";
}

## set debug mode on
##
if (defined($optctl{'no-download'})) {
    $NOSW_INST = 1;
}

if (defined($optctl{'no-wget'})) {
    $NO_WGET = 1;
}

if (defined($optctl{'clean'})) {
    $DO_CLEAN = 1;
}

if (defined($optctl{'tmp'})) {
    $tdir = $optctl{'tmp'} . "/genesis";
}

if (defined($optctl{'config'})) {
    $CONFIG = $optctl{'config'};
}
if (defined($optctl{'dbconfig'})) {
    $DBCONFIG = $optctl{'dbconfig'};
}

if ( $#ARGV >= 0 ) {
	usage;
	exit 2;
}

title;

sub tail {
  my ($file) = @_;
  print "--------------------------------------------------------------------\n";
  advise ($file . "\n");
  system ("/usr/bin/tail", "$file");
  print "--------------------------------------------------------------------\n";
}


my ($dbugnl) = 1 ; # Last message ended with a newline
sub debug {
  my ($msg) = @_;

  if (!$ENV{GEN_DEBUG}) {return};

  if ($dbugnl) {
    print STDERR "[debug] ";
  } 
  print STDERR $msg;
  if ($msg =~ m/\n$/) {
    $dbugnl = 1;
  } else {
    $dbugnl = 0;
  }
}

my ($advisenl) = 1 ; # Last message ended with a newline
sub advise {
  my ($msg) = @_;
  if ($advisenl) {
    print "# ";
  } 
  print $msg;
  if ($msg =~ m/\n$/) {
    $advisenl = 1;
  } else {
    $advisenl = 0;
  }
}

my ($msgnl) = 1 ; # Last message ended with a newline
sub msg {
  my ($msg) = @_;
  if ($msgnl) {
    print "\n[===]\n[===] ";
  } 
  print $msg,"[===]\n";
  if ($msg =~ m/\n$/) {
    $msgnl = 1;
  } else {
    $msgnl = 0;
  }
}

sub wget {
  my ($url) = @_;
  my ($d);
  if ($ENV{"GEN_DEBUG"}) {
    $d = "-v";
  } else {
    $d = "-nv -q";
  }
  return 1 if ($NOSW_INST);
  if ($NO_WGET) {
      advise ("skipping download [wget $d -m -l1 -nd $url]\n");
      if ($url =~ /\/([^\/]+)$/) {
	  my $file = $1;
	  if (!(-f $file)) {
	      warn("WARNING, file `$file' not in " . `/bin/pwd`);
	  }	      
      }
  } else {
      advise ("Downloading [wget $d -m -l1 -nd $url]\n");
      system("wget $d -m -l1 -nd $url 1>&2");
      if ($?) {die "Can't retrieve RPM from $url"};
  }

  return (1);
}

sub rm_rpm {
  my ($rpm) = @_;
  return 1 if ($NOSW_INST);
  system ("rpm -e $rpm");
  #if ($?) {die "Can't remove RPM $rpm"};
}

sub killall {
  my ($list, $sig) = @_;
  system ("killall $sig $list > /dev/null 2>&1");
}

sub inst_rpm {
  my ($rpm) = @_;
  return 1 if ($NOSW_INST);
  advise ("Installing RPM $rpm. One moment... \n");
  system ("rpm -U --nodeps --force $rpm");
  if ($?) {die "Can't install RPM $rpm"};
}

sub find_ip {
  my ($host) = @_;
  my $name;
  #first try host
  $name = `host $host | grep address`;
  if ($?) {
    warn "Can't find IP address of $host $name";
    undef;
  } else {
    $name =~ m/([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/;
    $1;
  }
}

sub get_host_coords{
  my ($entity_name, $network_name) = @_;
  my @coords_list= qw(4536N/07326W 3848N/07709W 3344N/08423W 2925N/09829W 3721N/12158W 4007N/08814W 4045N/07338W 3545N/13930E 5146N/00021W 3723N/12157W 4723N/00855E 3934N/07845W 3355S/15110E 4530N/07336W 2218N/11410E);
  if (defined $cfg->val("$network_name $entity_name","latlong"))
    {return $cfg->val("$network_name $entity_name","latlong");}
  else
    {return @coords_list[ int(rand($#coords_list))];}
}

sub load_file {
  my ($name, $buf) = @_;

  if (open (_F, "$name")) {
    @{$buf} = <_F>;
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name]: $!");
    0;
  }
}

sub save_file {
  my ($name, $buf) = @_;

  if (open (_F, ">$name")) {
    print _F join ("", @{$buf});
    close (_F);
    1;
  } else {
    warn ("Can't open file [$name] for writing: $!");
    0;
  }
}

sub sed {
  my ($buf, $expr) = @_;

  eval "\$buf =~ $expr";
  $buf;
}

sub untar_file {
  my ($path) = @_;
  if ( ! -f $path ) {
    warn "source path to tar file '$path' does not exist"; 
    undef;
  } else {

    `(cd $tdir; tar -zxpPvf $path) 1>&2`;
    if ($?) {
      advise ("error untarring file [$path]\n");
      undef;
    } else {
       1;
    }
  }
}

sub install_file {
  my ($path) = @_;
  if ( ! -f $path ) {
    warn "source path to $path does not exist"; 
    undef;
  } else {
    `tar cf - $path | (cd / ; tar xpf -)`;
    if ($?) {
      warn "Can't copy path $path"; 
      undef;
    }
    1;
  }
}

sub system_freedom {
  my(@cmd) = @_;
  my($cmd);
  
  $cmd = join (" ", @cmd);
  `echo $cmd | su - freedom 1>&2`;
}



# ======================================================================

msg ("Loading configuration files\n");

$cfg = IniConf->new( -file => $CONFIG, -nocase => 1, -default => "Default" );
if ( ! defined ($cfg) ) {
  print STDERR "Can't initialize configuration from file $CONFIG";
  print STDERR join ('\n', @IniConf::errors);
}
   
if ( ! defined ($cfg->ReadConfig) ) {
  print STDERR "Can't read configuration file $CONFIG";
  print STDERR join ('\n', @IniConf::errors);
}

$dbcfg = IniConf->new( -file => $DBCONFIG, -nocase => 1, -default => "Default" );
if ( ! defined ($dbcfg) ) {
  print STDERR "Can't initialize configuration from file $DBCONFIG";
  print STDERR join ('\n', @IniConf::errors);
}
   
if ( ! defined ($dbcfg->ReadConfig) ) {
  print STDERR "Can't read configuration file $DBCONFIG";
  print STDERR join ('\n', @IniConf::errors);
}

foreach $sect ($cfg->NonGroupSections) {
  debug ("Section = $sect\n");
}

foreach $group ($cfg->Groups) {
  debug ("Group = $group\n");
}


my ($section, $hostentity, $hname );
foreach $section ($cfg->Sections) {
  my (%x);
  if ($section =~ m/^network\s+(.+)/) {
    foreach $hostentity ($cfg->GroupMembers($1)) {
      $hname = $cfg->val($1 . " $hostentity", "hostname");
      if ( ! defined($Net{$1}) ){
	$Net{$1} = [];
      }
      if ( ! defined($x{$hname}) ){
	$x{$hname} = 1;
	push (@{$Net{$1}}, $hname);
	$Network{$hname} = $1;
	$HostEntity{$hname} = $hostentity;
	debug ("Processed host $hname for network $1\n");
      }
    }
  } 
}



# 1. Gather some config information
# 2. Determine where you belong
# 3. Downloading the RPMs

my ($Hostname);
if (defined($optctl{'hostname'})) {
  $Hostname = $optctl{'hostname'};
}
else {
  chop($Hostname = `hostname`);
  #$Hostname = lc $Hostname;
}


my $must_be_root = 1;

if (defined($optctl{'download-only'})) {
  # HACK HACK HACK
  # 13nov2000 plm
  # I simply wish to download all the required rpms, but to an arbitrary
  # host which is not on any test network. Since the rest of genesis
  # needs to have $Hostname set, I will pick the first host on the
  # network, arbitrarily.
    my $desired_network = $optctl{'download-only'};
    foreach my $hostname (keys(%Network)) {
	if ($Network{$hostname} eq $desired_network) {
	    $Hostname = $hostname;
	    msg("arbitrarily pretending to be $Hostname on $desired_network\n");
	    $must_be_root = 0;
	    last;
	}
    }
    if ($must_be_root) {
        advise ("**warning: arbitrary host $Hostname ,
                not on network $desired_network \n");
    }
} else {


  my @test_networks = (keys (%Net));
  my $test_network;
  #my @test_domains;
  my $test_domain;
  my $short_hostname;
  my $long_hostname = `hostname --long`;	# fully-qualified
  chop $long_hostname;
  $long_hostname = lc $long_hostname;
  my $test_hostname = $Hostname;
  my $found = 0;
 
  print "looking for Hostname: $Hostname\n" if ($dbugnl);

  foreach $test_network (keys %Net) {
    if (defined ($cfg->val("network " . "$test_network", 'domain'))) {
      $test_domain = $cfg->val("network " . "$test_network", 'domain');
      #push (@test_domains, $test_domain);
      #print "test_domain = $test_domain\n";

      #
      # try to guess network by scanning for hostname in genesis conf
      #
      
      # first if short hostname, look for <hostname>.<test_domain>
      if ($Hostname =~ /^[^\.]+$/) {
	$short_hostname = $Hostname;
	$test_hostname = $Hostname . "." . $test_domain;
        #print "test_hostname: $test_hostname\n";
        if (defined ($Network{$test_hostname})) {
	  $Hostname = $test_hostname;
	  print "test_hostname: $test_hostname\n";
	  $found = 1;
	  last;
        }
      }
      # next try long hostname as FQDN
      if (defined ($Network{$test_hostname})) {
	$Hostname = $test_hostname;
	print "test_hostname: $test_hostname\n";
	$found = 1;
	last;
      }
      # else try hosts's canonical FQDN (fully qualified domain name)
      elsif (defined ($Network{$long_hostname})) {
	$Hostname = $long_hostname;
	print "long_hostname: $long_hostname\n";
	$found = 1;
        last;
      }
    }

  #  push (@test_domains, $test_domain);
  }
  
  #foreach $dom (@test_domains) {
   #  advise ("** Looking for " . $Hostname . $dom . "\n");
    #if ( defined($Network{$Hostname . $dom}) ) {
     # $Hostname = $Hostname . $dom;
      #$found = 1;
      #last;
    #}
  #}
  if ( $found == 0 ) {
    die "No entries found for $Hostname (in [" , join (", ", @test_networks), "])";
  }
}


# Verify that all the 'required_serv' services are defined for our network.

my $release_to_check = $cfg->val("network $Network{$Hostname}", "release");

if (defined $cfg->val( "rpms ".$release_to_check, "required_serv")){
  
  my %required_services;

  #make a 'checklist' hash to verify all the services

  foreach my $service ( split ":" , $cfg->val( "rpms $release_to_check", "required_serv")){
    $required_services{$service}="";
  }

  foreach my $host ($cfg->GroupMembers($Network{$Hostname})){
    foreach my $p ($cfg->Parameters("$Network{$Hostname} $host")){
      $required_services{$p}="present";
      # we get some junk, but that's ok
    }
  }

  my @missing_services;
  foreach my $val_to_check (keys %required_services){
    if ($required_services{$val_to_check} eq "")
      {push @missing_services, $val_to_check;}
  }

  if (defined @missing_services){
    die("Can't install because network $Network{$Hostname} is missing the following required services:" . (join "\n", @missing_services) ."\n")
  }
}
else{
  die("**ERROR** required_serv for $release_to_check is not defined.\n");
}

# Check if we have a replicator defined for this network or not
foreach my $host ($cfg->GroupMembers($Network{$Hostname})) {
  foreach my $p ($cfg->Parameters("$Network{$Hostname} $host")) {
    if ($p eq "replic") {
      $REPLICATOR_EXISTS = 1;
    } 
  }
}



sub ShowConfig {
  my ($net, $host, @hosts, $entity, $p) ;
  
  foreach $net (sort (keys (%Net))) {
    printf "\nNetwork [%s]\n", $net;
    foreach $host (sort (@{$Net{$net}})) {
      printf "  %-20s [%s]:  ", $host, $HostEntity{$host};
      foreach $p ($cfg->Parameters("$net $HostEntity{$host}")) {
	if (defined $Servers{$p}) {
	  printf "%s ", $p;
	}
      }
      printf "\n";
    }
  }

#  if ( defined($cfg->val($MySection, "niqs")) ) {
#    $IS_NIQS=1;
#    advise ("Host in an NIQS for this network.\n");
#    advise ("Start this server second.\n");
#  }
#
#    for ($i = 1; $i <= $#hostname_host; $i++) {
#      if (defined ($cfg->val($Network{$Hostname} ." ". $HostEntity{$hostname_host[$i]}, "niss"))) {
#        advise ($hostname_host[$i] . "[" . $HostEntity{$hostname_host[$i]} . "] is an NISS\n");
#	my ($s);
#	$s = sprintf ("%-16s = 51113\n", $ipaddr_host[$i]); # For now the port number is fixed
#	push @buf, $s;
#      }
#    }


  exit;
#  foreach $hostentity ($cfg->GroupMembers($Network{$Hostname})) {
#    my ($x);
#    $x = $cfg->val($Network{$Hostname} ." ". $hostentity, "hostname");
#    if (!defined ($tmp{$x})) {
#      $tmp{$x} = $x;
#      $hostname_host[$i] = $x;
#      $ipaddr_host[$i] = find_ip ($hostname_host[$i]);
#      if (!defined ($ipaddr_host[$i]) ) {
#	die "No IP address found for hostname [". $hostname_host[$i] 
#	  . "]. Fix in DNS.";
#      }
#      advise ($hostname_host[$i] ."[". $ipaddr_host[$i] ."] is part of this network\n");
#      $i++;
#    }
#  }
}


if (defined($optctl{'show'})){
  ShowConfig ($optctl{'show'});
  exit 0;
}

my($whost);
my (@hostname_host, @ipaddr_host, %tmp);
my $IS_KEY_QUERY=0;
my $IS_NIQS = 0;
my $IS_MASTER_NISS=0;
my $NOT_FREE=0;
my ($aip_ent);
my ($wh_ip);
my ($keyfile, $keyrep);
my (@MyEntities) = [];
my ($dbname);

my ($protocol)='1.0';



sub Setup {
  advise ("$Hostname is part of network $Network{$Hostname} as $HostEntity{$Hostname}\n");
  
  $MySection = $Network{$Hostname} . " " . $HostEntity{$Hostname};
  
  $MyNetworkSection = "network " . $Network{$Hostname};
  
  $MyDomain = $cfg->val($MyNetworkSection, "domain");
  
  if (!defined ($MyDomain) ) {
    die "No domainname found for the network $Network{$Hostname}";
  }



  # 
  # scan DB entities
  #
  foreach $sect ($dbcfg->NonGroupSections) {
    debug ("DB Section = $sect\n");
  }
  
  foreach $group ($dbcfg->Groups) {
    debug ("DB Group = $group\n");
  }

  @DBs = $dbcfg->GroupMembers($Network{$Hostname});
  debug ("Databases: ". join (", ", @DBs) . "\n");


  #
  # check host-entities IP adresses
  # 
  my ($i) = 1;
  
  foreach $hostentity ($cfg->GroupMembers($Network{$Hostname})) {
    my ($x,$n);
    $x = $cfg->val($Network{$Hostname} ." ". $hostentity, "hostname");
    $n = $cfg->val($Network{$Hostname} ." ". $hostentity, "nat-name");
    if (!defined ($tmp{$x})) {
      $tmp{$x} = $x;
      $hostname_host[$i] = $x;
      $ipaddr_host[$i] = find_ip ($hostname_host[$i]);
      if (!defined ($ipaddr_host[$i]) ) {
	die "No IP address found for hostname [". $hostname_host[$i] 
	  . "]. Fix in DNS.";
      }
      if ( defined ($n) ) {
	my ($ip);
	advise ("We have a NAT address [$x == $n]\n");
	$ip = find_ip ($n);
	if (!defined ($ip) ) {
	  die "No IP address found for NAT name [$n]. Fix in DNS.";
	}
	$IPNATAddr{$x} = $ip;
      }
      $IPAddr{$x} = $ipaddr_host[$i];
      advise ($hostname_host[$i] ."[". $ipaddr_host[$i] ."] is part of this network\n");
      $i++;
    }
  }



  #
  # some general network section checks
  #

  if (! defined ($cfg->val($MyNetworkSection, "keyfile")) ) {
    die "No Key filename defined in configuration file";
  }
  if (! defined ($cfg->val($MyNetworkSection, "keyrep")) ) {
    die "No Key URL repository defined in configuration file";
  }
  $keyfile = $cfg->val($MyNetworkSection, "keyfile");
  $keyrep = $cfg->val($MyNetworkSection, "keyrep");



  #
  # specific host-entity section checks
  #
  foreach $param ($cfg->Parameters($MySection)) {
    if (defined $KeyPrivFileMapping{$param}) {
      push (@MyEntities, $cfg->val($MySection, $param));
    }
  }

  if( defined($cfg->val($MySection,"lode"))){
    push (@MyEntities, $cfg->val($MySection,"lode"));
  }

  if ( defined($cfg->val($MySection, "keyqry")) ) {
    $IS_KEY_QUERY=1;
    advise ("Host in an Key Query Server for this network.\n");
    advise ("Start this server first.\n");
  }
  if ( defined($cfg->val($MySection, "niqs")) ) {
    $IS_NIQS=1;
    advise ("Host in an NIQS for this network.\n");
    advise ("Start this server second.\n");
  }
  if ( defined($cfg->val($MySection, "niss")) &&
       $cfg->val($MySection, "niss") eq "NISS" ) {
    $IS_MASTER_NISS=1;
    advise ("Host in the Master NISS for this network.\n");
  }

  if ( defined($cfg->val($MySection, "notfree")) && 
       ($cfg->val($MySection, "notfree")) eq "yes" ) {
    $NOT_FREE=1;
    advise ("Freedom standard installation has been disabled.\n");
    advise ("***INSTALLING QMAIL ONLY***\n");
  }


  
  if (defined($optctl{'check'})) {
    advise ("Checkout complete\n\n");
    exit 0;
  }


  #
  # RPM release checks
  #

  if ( ! defined ($cfg->val($MyNetworkSection, "release")) ) {
    die "No release defined in 'network' section of configuration file";
  } elsif ( defined ($cfg->val($MySection, "release-overide")) ) {
    msg ("Overiding Network 'release' value with Host 'release-overide' value\n");
    $release = $cfg->val($MySection, "release-overide");
  } else {
    $release = $cfg->val($MyNetworkSection, "release");
  }
  $rpms = "rpms " . $release;

  if (! defined ($cfg->val($rpms, "repository")) ) {
    die "No RPM repository found for $rpms in configuration file";
  }
  if (!defined ($cfg->val($rpms, "base")) ) {
    die "No base RPM for $rpms defined in configuration file";
  }
  if (!defined ($cfg->val($rpms, "knt")) ) {
    die "No knt RPM for $rpms defined in configuration file";
  }
  if (!defined ($cfg->val($rpms, "fmg")) ) {
    die "No fmg RPM for $rpms defined in configuration file";
  }


  #
  # AIPs
  #

  # AIP 2.0: (defines protocol=2.0)
  if ( defined ($cfg->val($MyNetworkSection, "protocol")) )
  {
    $protocol = $cfg->val($MyNetworkSection, "protocol");
    if ("$protocol" == "2.0") {
      msg ("using  AIP protocol 2.0");
      if (!defined ($cfg->val($rpms, "kaip")) ) {
        die "No kaip RPM for $rpms defined in configuration file";
      }
      if (!defined ($cfg->val($rpms, "uaip")) ) {
	die "No uaip RPM for $rpms defined in configuration file";
      }
    }
  }
  # AIP 1.0: (coreaip-release overides)
  elsif ( defined ($cfg->val($MyNetworkSection, "coreaip-release")) )
  {    
    $coreaip_rpms = "rpms " . $cfg->val($MyNetworkSection, "coreaip-release");
    if (! defined ($cfg->val($coreaip_rpms, "repository")) ) {
      die "No coreaip repository found for $coreaip_rpms in configuration file";
    }
    if (!defined ($cfg->val($coreaip_rpms, "coreaip")) ) {
      die "No coreaip RPM for $coreaip_rpms defined in configuration file";
    }
  }
  else
  {
#    die "No coreaip-release defined in 'network' section of configuration file";
    msg ("***Warning: No coreaip-release defined in 'network' section.\n[===]    Defaulting coreaip RPMs to the 'network' release parameter.\n"); 
  }



  #
  # additional AIP configuration checks
  #

  if ( (defined $cfg->val($MySection, "aip")) ||
       (defined $cfg->val($MySection, "coreaip")) )
  {
    $whost = $cfg->val($MySection, "worm-hostname");
    if (!defined ($whost) ) {
      die "No wormhole hostname found for $Hostname";
    }

    if ($whost =~ /([A-Za-z0-9-_\.]+)\.$/) {
      advise ("Wormhole hostname is $whost\n");
      $wh_ip = find_ip($1);
    } else {
      advise ("Wormhole hostname is $whost.$MyDomain\n");
      $wh_ip = find_ip($whost . "." . $MyDomain);
    }
    if (!defined ($wh_ip) ) {
      die "No IP address found for wormhole hostname ["
        .  $whost . "." . $MyDomain . "]. Fix DNS.";
    }
    advise ("Wormhole IP address is $wh_ip\n");
  
    $aip_ent = $cfg->val($MySection, "aip");
    if (!defined ($aip_ent) ) {
      warn "No aip entity name found for $Hostname";
    }
  
    advise ("Aip entity name is $aip_ent\n");
  }
  else
  {
    # HACK for running fr_confsubst on non-aip core servers
    $aip_ent=$HostEntity{$Hostname};
    $wh_ip='127.0.0.2';
  }



  #
  # core-server specific checks:
  #
  # Scan all databases in our network and see 
  # which servers our host we need to use
  #

  debug ("entities = " . join (" ", @MyEntities) . "\n");
  foreach $dbname (@DBs) {
    my ($list,@list,%mark,@found);
    $list = $dbcfg->val($Network{$Hostname} . " " . $dbname, "servers");
    if (! defined ($list)  ) {
      if ( $dbname ne "general" ) { 
	warn "No servers listed for database $dbname";
      }
    } else {
      @list = split(/ /,$list);
      # debug ("dbname   = " . join (", ", @list) . "\n");
      grep ($mark{$_}++, @list); # Add these to the AA as a side-effect
      # debug ("marked   = " . join (", ", keys (%mark)) . "\n");
      @found = grep ($mark{$_},@MyEntities);
      if ( @found != 0 ) {
	grep ($DBrequired{$_}++, @found); # Add these to my list
	$DBs{$dbname} .= join (" ", @found);
	# debug ("found = " . join (" ", @found)  . "\n" );
	debug ("dbname = " . $dbname . "; DBs(dbname) = " . join (" ", @found)  . "\n" );
      }
    }
  }
  
  debug ("DBs required for  = " . join (", ", keys (%DBrequired)) . "\n");
  debug ("DBs to activate   = " . join (", ", keys (%DBs)) . "\n");
  return 1;
}



# Skip the logic to figure what RPMS we need

sub freespace {
  my ($part) = @_;
  my ($x);
  $part =~ s/\/genesis$//;
  $x = `df -k $part | tail -1`;
  if ($?) {
     warn "Problem with df"; 
     return (-1);
  }
  debug ($x);
  $x=(split(/\s+/,$x))[3];
  debug ("$x\n");
  $x;
}


chomp ($pwd);

`mkdir -p $tdir`;
`chmod 0777 $tdir`;
if ( freespace($tdir) < 50000) {
  if (freespace($tdir) == -1) {
     advise ("ERROR: woth $tdir\n");
     exit (2);
  } else {
     advise ("****** WARNING: there may not be enought space in $tdir. \nUse the --tmp option\n\n");
  }
}

chdir ($tdir) || die "Can't cd to $tdir: $!";


sub InstallRPMs {

  if ($NOSW_INST) {
    advise ("NOTE: NO Software Installation Option in effect. NO RPM update will occur\n");
  }
  msg ("Downloading " . $release . " RPMs from\n[===] " . $cfg->val($rpms, "repository") . "\n");

  # base rpms
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "base"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "knt"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "fmg"));

  # AIP 2.0 hack:
  if ( "$protocol" == "2.0" ) {
    wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "kaip"));
    wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "uaip"));
  }

  if (defined ($cfg->val($rpms, "snmp")) ) {
    wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "snmp"));
  }
  
  # 4. Backup/wipe the previous installation
  
  KillServers();
  
  msg ("Uninstalling older RPMs\n");
  
  rm_rpm ("freedom-fmg");
  rm_rpm ("freedom-knt");
  rm_rpm ("freedom-pki");
  rm_rpm("freedom-snmp"); ## thiery add this to avoid bad uninstall on delta
  rm_rpm ("freedom-coreaip");

  # AIP 2.0 hack
  rm_rpm ("freedom-uaip");
  rm_rpm ("freedom-kaip");

  rm_rpm ("freedom");


  if ($DO_CLEAN) {
    msg ("Removing all trace of previous installation\n");
    # FIX FIX FIX
    #   removing everything in /usr/freedom is just plain wrong!
    #   things that should be cleaned-up include:
    #     local databases, var logs and semaphores, private keys, key vault
    #
    #system("find /freedom  ! -type d -print | xargs rm -f 1>&2");
    system("find /usr/freedom/db  ! -type d -print | xargs rm -f 1>&2");
    system("find /usr/freedom/var ! -type d -print | xargs rm -f 1>&2");
    system("find /usr/freedom/privkey  ! -type d -print | xargs rm -f 1>&2");
    system("find /usr/freedom/vault  ! -type d -print | xargs rm -f 1>&2");
    #system("find /usr/freedom/etc  ! -type d -print | xargs rm -f 1>&2");
  }


  
  # 5. Install the new rpms
  
  msg ("Installing " . $release . " RPMs\n");
  $ENV{ZK_AGREE}=1;

  # install base rpms
  inst_rpm($tdir . "/". $cfg->val($rpms, "base"));
  killall("fr_probe_net", "-15");
  inst_rpm($tdir . "/". $cfg->val($rpms, "knt"));
  inst_rpm($tdir . "/". $cfg->val($rpms, "fmg"));



  # additional RPMs:
  
  # AIP 2.0 hack:
  # all the code here is shitty
  # will be better after rpm refactoring
  if ( "$protocol" == "2.0" ) {
    # kaip installation
    inst_rpm($tdir . "/". $cfg->val($rpms, "kaip"));
    # cleaning 
    `cd /lib/modules/2.2.16-3aip/misc; rm -f kaip.o blowfish-openssl-asm.o hmac.o`;
    # copying modules to lib modules new modutil dont like symlinks
    # not pointing on same executable names
    `/etc/rc.d/init.d/aipconf stop`;
    `cp /usr/freedom/drv/kaip_redhat-62_2.2.16-3aip.o /lib/modules/2.2.16-3aip/misc/kaip.o`;
    `cp /usr/freedom/drv/blowfish-openssl-asm_redhat-62_2.2.16-3aip.o /lib/modules/2.2.16-3aip/misc/blowfish-openssl-asm.o`;
    `cp /usr/freedom/drv/hmac_redhat-62_2.2.16-3aip.o /lib/modules/2.2.16-3aip/misc/hmac.o`;
    `depmod -a`;
    `/etc/rc.d/init.d/aipconf start`;

    # uaip install
    #    inst_rpm($tdir . "/". $cfg->val($rpms, "uaip"));
    # comment previous line to fix problem with cfengine
    my $localuaiprpm=$cfg->val($rpms, "uaip");
    `rpm -Uvh --nodeps --force $localuaiprpm > /tmp/uaip_install`;
    if ($?) {die "Can't install RPM uaip rpm"};
  }
  
  # SNMP
  ## should be added - thierry
  if (defined ($cfg->val($rpms, "snmp")) ) {
    inst_rpm($tdir . "/". $cfg->val($rpms, "snmp")); 
  }

}



sub AdjReplicConfig {
  my ($niqsfile, $i);
  my (@buf);

  $niqsfile = "/freedom/etc/freedom-nidb.conf";
  msg ("Adjusting Replicator Configuration\n");

  if (load_file ($niqsfile, \@buf)) {
    pop @buf;
    push @buf, "; Entries generated by Genesis for network ". $Network{$Hostname} ."\n";
    for ($i = 1; $i <= $#hostname_host; $i++) {
      if (defined ($cfg->val($Network{$Hostname} ." ". $HostEntity{$hostname_host[$i]}, "niss"))) {
        advise ($hostname_host[$i] . "[" . $HostEntity{$hostname_host[$i]} . "] is an NISS\n");
	my ($s);
	$s = sprintf ("%-16s = 51123\n", $ipaddr_host[$i]); # For now the port number is fixed
	push @buf, $s;
      }
    }
    save_file ($niqsfile, \@buf);
  } 
}


sub AdjSendmailConfig {
  my (@buf);
  my ($template); 
  my $sm_daemon = "yes";
  if ( defined($cfg->val("info", "remote")) && 
	 ($cfg->val("info", "remote") eq "yes")) {
    if ( defined($cfg->val($MySection, "fmg")) ) {
      msg ("*** Adjusting Sendmail Configuration for external FMG\n"); 
      $template="$pwd/fmg-external.cf.tmpl";
    } else {
      msg ("*** Adjusting Sendmail Configuration for external MAIP\n"); 
      $template="$pwd/maip-external.cf.tmpl";
      $sm_daemon = "no";
    }
  } else {
    $template="$pwd/fmg-genesis.cf.tmpl";
  }
  if ( $Hostname !~ /celery/ ) {
     msg ("Adjusting Sendmail Configuration\n"); 
  } else {
     msg ("*** NOT Adjusting Sendmail Configuration on celery\n"); 
     return 1;
  }
  if (wget ($cfg->val("sendmail", "linux"))) {
    system ("$pwd/genesis_sendmail.sh", $template, $MyDomain, "$tdir/" .
     substr($cfg->val("sendmail", "linux"),
             rindex($cfg->val("sendmail", "linux"),"/")), $RootDir,
             $sm_daemon);
    if ($?) {
      advise "Sendmail configuration problem\n";
    } else {
      advise "Done\n";
      0;
    }
  } else {
    advise "Error retrieving sendmail binary\n";
    1;
  }
}


sub db_utils {
  if ( (defined $cfg->val($MyNetworkSection, "dbutils")) 
       && ($cfg->val($MyNetworkSection, "dbutils") eq "yes") ) {
    if (defined $cfg->val("dbutils", "linux")) {
      if (wget ($cfg->val("dbutils", "linux"))) {
         untar_file ("$tdir/" .
                 substr($cfg->val("dbutils", "linux"),
                 rindex($cfg->val("dbutils", "linux"),"/")));
        if ($?) {
          advise "Trouble installing db_utils\n";
        } else {
          advise "Done\n";
          0;
        }
      } else {
        advise "Error retrieving db_utils binary\n";
        1;
      }
    }
    1;
  }
}


sub Extras {
  db_utils;
}

     
sub HackConfigs {
  my ($tokenfile, $clientfile, $nidbfile, $bogusmaster, $postnewsfile, $keyupdserverfile, $kscpdfile);
  my ($aipfile, $accessfile, $accessfile_backup, $allowTcpPortRange, $allowUdpPortRange);
  my ($network_entity, @whunauth_permit_list, $whunauth_permit, $ed);
  my ($throughput_incoming);
  my (@buf, @buf2);

  $keyupdserverfile = "/freedom/etc/freedom-keyupdserver.conf";
  $kscpdfile = "/freedom/etc/freedom-frkscpd.conf";
  $clientfile = "/freedom/etc/clientinfo.conf";
  $tokenfile = "/freedom/etc/tokentypeid.conf";
  $nidbfile = "/freedom/etc/freedom-nidb.conf";
  $aipfile = "/freedom/etc/freedom.conf";
  $postnewsfile = "/freedom/etc/freedom-frpostnews.conf";
  $accessfile = "/freedom/etc/freedom-access.conf";
  $accessfile_backup = "/freedom/etc/freedom-access.conf.closedports";
  $allowTcpPortRange = "allowTcpPortRange = 1025-65534";
  $allowUdpPortRange = "allowUdpPortRange = 1025-65534";
  $throughput_incoming = 625000;
  $bogusmaster = "bogus.master";

  msg ("Apply hacks to some config files\n");

  # client config
  if (load_file ($clientfile, \@buf)) {
    advise ("Adjusting clientinfo.conf\n");
    advise ("    Changing domain to $MyDomain\n");
    @buf2 = map { sed ($_, "s/^domain = .*freedom.net/domain = $MyDomain/") } @buf;
    save_file ($clientfile, \@buf2);
  }

  # token config
  if (load_file ($tokenfile, \@buf)) {
    advise ("Adjusting tokentypeid.conf\n");
    advise ("    Changing domain to $MyDomain\n");
    @buf2 = map { sed ($_, "s/^EmailDomain = .*freedom.net/EmailDomain = $MyDomain/") } @buf;
    save_file ($tokenfile, \@buf2);
  }

  # key config
  if ( -f $keyupdserverfile ){
    if( ! -l $kscpdfile ){
      advise ("    doing symlink $kscpdfile to $keyupdserverfile");
      `ln -s $keyupdserverfile  $kscpdfile`;
    }
  }

  # net info
  if ( load_file ($nidbfile, \@buf)) {
    advise ("Adjusting freedom-nidb.conf\n");
    if ( $IS_MASTER_NISS ) {
      advise ("    Changing NISS MasterServer to $Hostname\n");
      @buf2 = map { sed ($_, "s/^MasterServer = flint.dev.zks.net/MasterServer = $Hostname/") } @buf;
    } else {
      advise ("    Changing NISS MasterServer to $bogusmaster\n");
      @buf2 = map { sed ($_, "s/^MasterServer = flint.dev.zks.net/MasterServer = $bogusmaster/") } @buf;
    }
    save_file ($nidbfile, \@buf2);
  }

  # access control
  if ( ! -e $accessfile_backup ) {
    `cp $accessfile $accessfile_backup`;
  }
  if (load_file ($accessfile, \@buf)) {
    advise ("Adjusting freedom-access.conf\n");
    advise ("    Opening TCP Port Range $allowTcpPortRange\n");
    advise ("    Opening UDP Port Range $allowUdpPortRange\n");
    @buf2 = map { sed($_, "s/^allowTcpPortRange = 51103-51112/$allowTcpPortRange/")} @buf;
    @buf2 = map { sed($_, "s/^allowUdpPort = 51109/$allowUdpPortRange/")} @buf2;
    save_file ($accessfile, \@buf2);
  }

  # aip config
  if (load_file ($aipfile, \@buf)) {
    advise ("    Increasing wormhole throughputIncoming to $throughput_incoming\n");
    @buf2 = map { sed($_, "s/^throughputIncoming.*\$/throughputIncoming = $throughput_incoming/")} @buf;
    save_file ($aipfile, \@buf2);
  }

  # coreaip config
 if ( defined($cfg->val($MySection, "coreaip")) ) {
    advise ("Adjusting freedom.conf for coreaip\n");
    if (!defined ($cfg->val($MyNetworkSection, "whunauth-permit"))) {
      die "No whunauth-permit defined in 'network' section of configuration file";
    }
    # Clear aipfile of previously defined wormholeUnauthPermit
    if (load_file ($aipfile, \@buf)) {
      advise ("\tRemoving previously defined wormholeUnauthPermit in freedom.conf\n");
      @buf2 = map { sed($_, "s/^wormholeUnauthPermit.*\$//")} @buf;
      save_file ($aipfile, \@buf2);
    } else {
      advise ("\tCould not load file $aipfile: $!\n");	
    }
    # Append the list of wormholeUnauthPermit paramters to [wormhole] section of freedom.conf
    @whunauth_permit_list = split ",", $cfg->val($MyNetworkSection, "whunauth-permit");
    # AIP 2.0 Hack:
    # if core-aip, we want to run '/freedom/rc/aipcore' configurator
    # to specify coreaip mask, routes and ipchains rules.
    #
    if ( "$protocol" == "2.0" )
    {
      my (@whunauth_permits, $whunauth_permits);
      # use whunauth-permit param and convert 'a.b.c.d' netmask into '/bits'
      # note: multiple comma-separated netmasks are supported
      for $whunauth_permit (@whunauth_permit_list)
      {
	if ( $whunauth_permit =~ /([0-9]+\.[0-9]*\.*[0-9]*\.*[0-9]*):(.+)/ )
	{
	  my ($net, $netmask) = ($1, $2);
	  # if netmask is dotted-decimal, convert to '/bits' subnet mask
	  # we use log to fetch num of significant bits in netmask:
	  #
	  # log     Returns the natural logarithm (base e) of EXPR.
	  #         If EXPR is omitted, returns log of $_.  To get the
	  #         log of another base, use basic algebra: The base-N
	  #         log of a number is is equal to the natural log of
	  #         that number divided by the natural log of N
	  #	  

	  my $netbits;

	  print " => inetaddr netmask = $net : $netmask\n";
	  foreach my $bytes (split ('\.', $netmask))
	  {
  	    my $mask = log($bytes + 1)/log(2);
  	    $mask = $mask - ($mask % 1);
  	    if ( $mask =~ /([0-9]+)\.[0-9]+/ )
  	    {
    	      $mask = $1 + 1;
  	    }
  	    $netbits += $mask;
	  }

	  print " => significant-bits netmask = $net / $netbits\n";
	  push (@whunauth_permits, "$net/$netbits");
	}
	elsif ($whunauth_permit =~ /([0-9]+\.[0-9]*\.*[0-9]*\.*[0-9]*)\/([0-9]+)/ )
	{
	  my ($net, $netbits) = ($1, $2);
	  print "significant-bits netmask = $net / $netbits\n";
	  push (@whunauth_permits, "$net/$netbits");
	}
	else
	{
	  die "error: unparseable whunauth-permit netmask: $whunauth_permit";
	}
      } # for
      $whunauth_permits = join (",", @whunauth_permits);
      # now run aipconf and aipcore to configure mask.
      system ("/usr/freedom/rc/aipconf stop");
      system ("/usr/freedom/rc/aipcore true $whunauth_permits");
      system ("/usr/freedom/rc/aipconf start");
    }
    # AIP 1.0 : configure unauth permit list
    else
    {
      for $whunauth_permit (@whunauth_permit_list) {
	$whunauth_permit = join " ", split ":", $whunauth_permit;
	$ed = "1\n/^wormholeAddress/\na\nwormholeUnauthPermit = $whunauth_permit\n.\nw\nq\n";
	advise ("\tSetting: wormholeUnauthPermit = $whunauth_permit\n");
	`ed $aipfile << ${ed}`;
      }
    }
  }

  # news config
  if ( defined($cfg->val($MySection, "nntp-hostname")) ) {
    my $nntphost=$cfg->val($MySection, "nntp-hostname");
    advise ("Adjusting $postnewsfile\n");
    $ed = "1\n/^nntp.hostname/\ns/=.*\$/= $nntphost/\nw\nq\n";
   `ed $aipfile << ${ed}`;
    if ( defined ($cfg->val($MyNetworkSection, "nntp-port")) ) {
      my $nntpport=$cfg->val($MySection, "nntp-hostname");
      advise ("Adjusting $postnewsfile for nntp.port\n");
      $ed = "1\n/^nntp.port/\ns/=.*\$/= $nntpport/\nw\nq\n";
      `ed $aipfile << ${ed}`;
    }
  }

}



sub AdjConfigFiles {
  # 6. Configure

  msg ("Adjusting configuration files\n");

  # 6.1 Remove all useless links in /freedom/etc/rc
  # 
  # This is a hacky way to do this, but I don't have the time to 
  # parse the original freedom.spec file for now
  # 
  # Deprecated - F.K.
  #
  #`/bin/rm /usr/freedom/etc/rc/*` ;
  #foreach $param ($cfg->Parameters($MySection)) {
  #  advise ("Checking service $param\n");
  #  `$pwd/genesis-rc-links.sh $param` ;
  #}
  #
  # I feel we shouldn't have to tinker with this at all:
  # the rpm installation should take care of this.
  # besides, we may have want some non-freedom stuff in there (ie snmpd)
  #
  # AIP 2.0 and qmail hack:
  #
  # However, should this prove to be a necessity,
  # I have modified 'genesis-rc-links.sh' to support
  # AIP 2.0 via 'uaip' param and qmail queues with 'qmail param',
  # in this case, uncomment the following code
  #
  `/bin/rm /usr/freedom/etc/rc/*` ;
  foreach $param ($cfg->Parameters($MySection))
  {    
    # freedom core servers
    if ($param =~ /keyqry|keyupd|niqs|niss|nymsrv|toksrv/ )
    {
      advise ("Checking service $param\n");
      `$pwd/genesis-rc-links.sh $param`;
    }
  
    # freedom support servers
    elsif ($param =~ /nntp|post|replic|lode/ )
    {
      advise ("Checking service $param\n");    
      `$pwd/genesis-rc-links.sh $param`;
    }
  
    # freedom node (aip)
    elsif ($param =~ /aip/ )
    {
      # AIP 2.0 Hack
      if ( "$protocol" == "2.0") {
  	$param = "uaip";
      }
      advise ("Checking service $param\n");
      `$pwd/genesis-rc-links.sh $param`;
    }
  
    # mail 1.0 -deprecated
    elsif ($param =~ /fmg|fmgmaip|maip/ )
    {
      advise ("Checking service $param\n");
      `$pwd/genesis-rc-links.sh $param`;
    }
  
    # scarecrow mail
    elsif ($param =~ /relay|imep|imepb|nmta|nmtab|pop/ )
    {
      $param = "qmail";
      advise ("Checking service $param\n");
      `$pwd/genesis-rc-links.sh $param`;
    }
  
    # unsupported for now
    elsif ($param =~ /snmp/ )
    {
      advise ("Service $param is unsupported\n");
    }
  
    # default
    #else
    #{
    #
    #}
  
   }

  
  # 6.2 Run fr_confsubst
  #
  $ENV{ZK_OLDNETWORK}="freedom.net";
  $ENV{ZK_NETWORK}=$MyDomain;


  # AIP 2.0 hack:
  if ( "$protocol" == "2.0" )
  {
    # AIP 2.0 has new fr_confsubst: takes <entityName> <niqs.hostname>
    msg ("$pwd/genesis-confsub.sh  $aip_ent niqs.$MyDomain\n");
    `$pwd/genesis-confsub.sh  $aip_ent niqs.$MyDomain` ;
  }
  else
  {
    msg ("$pwd/genesis-confsub.sh  $aip_ent $wh_ip\n");
    `$pwd/genesis-confsub.sh  $aip_ent $wh_ip` ;
  }


  # 6.3 Set the passphrase file
  #
  if (! defined ($cfg->val("info", "passphrase-file")) ) {
    die "No passphrase file defined in configuration file";
  }
  open (PP, ">" . $cfg->val("info", "passphrase-file")) || die "Can't open passphrase file " .
    $cfg->val("info", "passphrase-file") . ": $!";
  print PP $cfg->val($MySection, "passphrase");
  close (PP);

  
  # 6.4 Handle the specific config files
  if ( defined($cfg->val($MySection, "replic")) ) {
    AdjReplicConfig;
  }

  # Fix up conf files for domains and NISS Master
  HackConfigs;

  # Fix Sendmail, if needed
  if ( defined($cfg->val($MySection, "fmg")) &&
       defined($cfg->val($MySection, "fix-sendmail")) && 
       ($cfg->val($MySection, "fix-sendmail") eq "yes") ) {
   AdjSendmailConfig;
  } else {
    msg ("No sendmail to adjust in $MySection\n");
  }

}


sub hashpjw {
    # hashpjw from the dragon book
    my $string = shift;
    my $h = 0;
    my $i = 0;
    for($i = 0; $i < length($string); ++$i) {
	$h = ($h << 4) + ord(substr($string, $i, 1));
	my $g = $h & 0xf0000000;
	if ($g) {
	    $h = $h ^ ($g >> 24);
	    $h = $h ^ $g;
	}
    }
    return $h;
}


sub install_mackey ($) {
  my ($mackey_file) = @_;

  msg ("Installing the mail MAC key\n");
  if (open (PP, ">$mackey_file")) {
    print PP sprintf("%x %s", hashpjw($Network{$Hostname}), "Mail MAC key shared secret for $Network{$Hostname}");
    close (PP);
    advise "Mail MAC key set to '" . `cat $mackey_file` . "'\n\n";
  } else {
    advise "Can't open Mail MAC key file (" . $mackey_file . "): $!";
  }
}


sub install_keys {
  my ($source, $ent, $name) = @_;
  my ($path, $dest, $keyname);
  
  my %FMGKeys = (
		"fmg"		=> "FMG",
		"fmgmaip"	=> "FMG"
		);
  my %NISSKeys = (
		"niss"		=> "NISS",
		);

  if (defined($FMGKeys{$ent})) {
    $keyname=$FMGKeys{$ent};
    # Some minor hacks to handle the special keys
    # Note: copies of all private FMG keys are in fmg:FMG
    $ent = "fmg";
    $name =~ s/FMGMAIP/FMG/;
    # Yep, another hack to get the FMG keys where we expect
    # them. Remove this hack when we dynamically generate
    # the keys during installation.
    `cp $source/fmg:FMG1/* $source/fmg:FMG 1>&2`;
  } elsif (defined($NISSKeys{$ent})) {
    $keyname = $NISSKeys{$ent};
    $name = $NISSKeys{$ent};
  } else {
    $keyname=$name;
  }
  $path = $source ."/". $ent .":". $keyname ."/".  $ent .":". $name . ".sig.priv";
  if ( ! -f $path ) {
    warn "source path to file $path does not exist"; 
    undef;
  } else {
    $dest = "/usr/freedom/privkey/" . $KeyPrivFileMapping{$ent} . "/sig" ;
    `mkdir -p $dest`;
    debug ("installing $path to $dest/1\n");
    `cp $path $dest/1 1>&2`;
  }

  $path = $source ."/". $ent .":". $keyname ."/".  $ent .":". $name . ".enc.priv";
  if ( ! -f $path ) {
    warn "source path to file $path does not exist"; 
    undef;
  } else {
    $dest = "/usr/freedom/privkey/" . $KeyPrivFileMapping{$ent} . "/crypt" ;
    `mkdir -p $dest`;
    debug ("installing $path to $dest/1\n");
    `cp $path $dest/1 1>&2`;
  }

  # Special case to install the monthly key onto
  # both keyqry and keyupd machines. Not sure
  # if both need it...
  if ( $ent eq "keyqry" || $ent eq "keyupd" ) {
    $keyname=$KeyPrivFileMapping{"frmonth"};
    $path = $tdir ."/usr/freedom/privkey/". $keyname ."/sig/1";
    if ( ! -f $path ) {
      warn "source path to file $path does not exist"; 
      undef;
    } else {
      $dest = "/usr/freedom/privkey/" . $keyname . "/sig" ;
      `mkdir -p $dest`;
      debug ("installing $path to $dest (special key for KQD only)\n");
      `cp $path $dest/1 1>&2`;
    }
  }

  if ( $ent eq "nymsrv" && defined $MACKeyFileMapping{"nymsrv"} ) {
    install_mackey("/usr/freedom/" . $MACKeyFileMapping{"nymsrv"});
  }


  1;
}



sub InstallKeys {
  msg ("Installing keys\n");
  wget ( $keyrep . "/" . $keyfile);
  if ( ! $NOSW_INST) {
    `(cd $tdir; tar -zpxvf $keyfile)`;
    if ($?) {
      advise ("error untarring keyfile [$keyfile]\n");
      exit (2);
    }
  };
  install_file ("usr/freedom/etc/passphrase.info");
  install_file ("usr/freedom/var/run/fradmin/.sysinit");
  foreach $param ($cfg->Parameters($MySection)) {
    advise ("Checking service $param\n");
    if (defined $KeyPrivFileMapping{$param}) {
      advise (" - installing keys for $param\n");
      install_keys("$tdir/usr/freedom/vault", $param, $cfg->val($MySection, $param));
    }
  }
  advise ("Adjusting permissions\n") ;
  `chown -R freedom /freedom/privkey` ;
}


sub seed_add_serv ($$$) {
  my ($serv, $ent, $host) = @_;
  my %info = (
	      "aip"    => "51101 51102 30 3600 1",
	      "maip"   => "51112",
	      "nymsrv" => "51104",
	      "toksrv" => "51103",
	      "keyupd" => "51108",
	      "keyqry" => "51109",
	      "niqs"   => "51107",
	      "replic" => "51113", # Really, where ths NISS appears to be
	      "niss"   => "51123", 
	      "fmgmaip"=> "51110", # Really, where the FMG is 
	      "lode"   => "51121",
              "imep"   => "25",
              "nmta"   => "51115",
              "pop"    => "51114"
	     );
	      
  if (defined ($info{$serv})) {
    my $s = $serv;
    if ($s eq "fmgmaip") { # HACK de HACK HACK
      $s = "fmg";
      $ent =~ s/FMGMAIP/FMG/;
    }
    # HACK1: If the replic service exists and the entity is an niss,
    #        don't create an NISS entity.
    # HACK2: If the replic service exists, create the NISS entity
    #        where the replicator is defined.
    #
    # HACK3: If no replicator exists and it isn't a required
    #               service, then create the NISS entity where the
    #               niss is defined using port 51113.
    if ($s eq "niss" && $REPLICATOR_EXISTS) { # HACK1
     return ;
    }
    if ($s eq "replic") { # HACK2
      $s = "niss";
      $ent = "NISS";
    }
    if ($s eq "niss") { # HACK3
      $serv = "replic"; # Use port 51113 for the NISS
    }
    return "entity add $s $ent $host " . $info{$serv} ;
  } else {
    undef
  }
}


sub seed_add_links (@) {
  my (@aips) = @_;
  my ($link, $buf);
  my %info = (
	      "0"    => "",
	      "1"    => "",
	      "2"    => "1:2",
	      "3"    => "1:2 2:3 3:1",
	      "4"    => "1:2 2:3 3:1 2:4 3:4 ",
	      "5"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5",
	      "6"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5 1:6 4:6",
	      "7"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5 1:6 4:6 5:7 6:7",
	      "8"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5 1:6 4:6 5:7 6:7 2:8 6:8",
	      "9"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5 1:6 4:6 5:7 6:7 2:8 6:8 3:9 8:9",
	      "10"    => "1:2 2:3 3:1 2:4 3:4 1:5 4:5 1:6 4:6 5:7 6:7 2:8 6:8 3:9 8:9 4:10 9:10",
	      "11"   =>"2:1 3:1 3:2 5:3 5:4 6:2 6:4 7:5 7:6 8:1 8:4 9:1 9:5 9:8 10:5 10:6 10:9 11:7 11:9",
	      "12"   =>"2:1 3:2 5:1 5:2 5:3 6:4 7:1 7:3 7:6 8:3 8:6 9:4 9:5 10:1 10:3 10:8 11:2 11:4 11:6 12:4 12:7 12:9",
	      "13"   =>"3:2 5:1 5:4 6:3 6:5 7:1 7:3 8:3 8:6 9:1 9:4 9:7 10:1 10:7 11:2 11:4 11:8 12:2 12:6 12:10 13:3 13:4 13:7",
	      "14"   =>"2:1 3:1 5:2 5:4 6:2 6:4 7:1 7:4 8:2 8:3 9:1 9:5 10:4 10:9 11:3 11:5 12:2 12:7 12:8 13:3 13:6 13:7 14:8 14:9",
	      "15"   =>"2:1 5:2 5:3 6:1 6:3 6:5 7:2 7:3 7:6 8:4 8:6 9:3 10:1 10:4 10:8 11:5 12:7 12:9 12:11 13:2 13:10 14:2 14:7 14:13 15:4 15:9 15:13",
	      "16"   =>"3:2 4:1 4:2 5:1 5:3 6:1 6:2 7:1 8:7 9:5 9:7 10:6 11:9 12:4 12:7 12:8 13:2 13:10 13:11 14:3 14:6 14:11 15:2 15:8 16:9 16:10",
	      "17"   =>"3:2 4:3 5:3 8:2 8:6 8:7 9:3 9:4 10:2 10:5 11:1 11:6 11:7 12:2 13:1 13:2 13:3 14:5 14:9 15:1 15:4 15:12 15:14 16:6 16:10 16:12 17:8 17:9 17:15",
	      "18"   =>"3:2 5:2 6:4 6:5 7:5 7:6 8:3 8:6 9:3 10:2 11:1 11:4 12:1 12:5 12:11 13:3 13:8 14:2 14:11 14:13 15:2 15:8 15:10 16:3 16:4 16:10 17:6 17:7 17:8 17:13 18:4 18:9 18:11",
	      "19"   =>"3:1 4:2 5:3 6:1 6:5 7:3 7:5 8:2 8:6 9:2 10:1 10:7 11:5 11:8 12:6 12:11 13:1 13:3 14:4 14:7 15:12 15:14 16:10 16:15 17:6 17:11 17:13 18:10 18:15 19:1 19:9 19:16",
	      "20"   =>"3:2 4:2 4:3 5:1 5:2 6:5 7:1 7:2 10:2 11:7 11:9 12:6 12:9 13:3 13:8 13:9 14:8 14:11 14:13 15:1 15:5 15:8 16:7 16:11 16:12 17:3 17:6 17:9 17:11 18:4 18:7 18:10 19:6 19:14 19:16 20:1 20:10"
	     );
	      
  if (defined ($info{$#aips})) {
    $buf = "echo ".($#aips + 1)." node configuration\n";
    foreach $link (split (/ /, $info{$#aips+1})) {
      my ($a, $b) = split (/:/, $link);
      $buf .= "entity add link $aips[$a-1] $aips[$b-1] 5000 2 off\n";
#      $buf .= "entity add link $aips[$a-1] $aips[$b-1] 20000 48 off\n";
    }
    return $buf;
  } else {
    advise ("Don't know how to configure a " . $#aips  . " node network\n");
    return undef;
  }
}


sub SetMOTD ($$){
  my ($netname,$motd) = @_;
  my $owner = $cfg->val("network $netname", "owner");
  my $lease = $cfg->val("network $netname", "lease");
  my $purpose = $cfg->val("network $netname", "purpose");
  my $hybridfile = "";

  # Set the Hybrid file
  if ( defined ($cfg->val($MySection, "hybridfile-overide")) &&
       defined ($cfg->val($MySection, "hybridrep-overide")) ) {
    $hybridfile = $cfg->val($MySection, "hybridfile-overide");
  } elsif ( defined ($cfg->val($MyNetworkSection, "hybridfile")) ) {
    $hybridfile = $cfg->val($MyNetworkSection, "hybridfile");
  } else {
    $hybridfile = "No hybrid file used";
  }

  msg ("Create and loading initial motd\n");
  open (MOTD, ">$motd") || die "Can't create [$motd]: $!";
  print MOTD "Welcome to [$netname] ($MyDomain)\n";
  print MOTD "  owner: $owner\n";
  print MOTD "  purpose: $purpose\n";
  print MOTD "  until: $lease\n"; 
  print MOTD "  network release: $release\n";
  print MOTD "  hybrid file: $hybridfile\n";
  print MOTD "  genesis installed: " . `date +"%Y-%m-%d %H:%M"`;
  print MOTD "  genesis version: $Version\n";
  close (MOTD);
}


sub LoadNidbData ($) {
  my ($netname) = @_;
  my ($seed, $h, $p, $ent, $sect, $buf, $links, $ip);
  my (@aips);
  my ($cacheDataVersion) = `date +%Y%m%d%H`;

  msg ("Create and loading initial NIDB seed data\n");
  $seed = "/usr/freedom/etc/nidb-seed.dat";
  if ( ! -f $seed || -f $seed) {
    open (SEED, ">$seed") || die "Can't create seed data file [$seed]: $!";
    print SEED "echo ----------------------------------------\n";
    print SEED "echo Automatically generated NIDB Seed file\n";
    print SEED "echo ----------------------------------------\n\n";
    $buf = "";
    # Set the cacheDataVersion: Bug #5110, 5206
    $buf .= "cache set $cacheDataVersion\n";
    $buf .= "echo Errors with 'unrecognised command: cache' can be ignored\n";

    # Scan each host entity and add the host entry for it
    foreach $h (sort (@{$Net{$netname}})) {
      debug ("$h is in network $netname\n");
      my $latlong = get_host_coords($HostEntity{$h}, $netname);
      if ( defined ($IPNATAddr{$h}) ) {
	advise ("Setting an NAT'fd address for [$h/$IPAddr{$h} => $IPNATAddr{$h}]\n");
	$ip = $IPNATAddr{$h};
      } else {
	$ip = $IPAddr{$h};
      }
      $buf .= "entity add host $HostEntity{$h} $h " 
	. "$ip CA10:$latlong Zero-Knowledge_Systems_[100] 1\n";
	# . "$ip CA10 Zero-Knowledge_Systems_[100] 1\n";
    }
    # Now add the service for each host
    foreach $h (sort (@{$Net{$netname}})) {
      $sect = $netname . " " . $HostEntity{$h};
      foreach $p ($cfg->Parameters($sect)) {
	if (defined $Servers{$p}) {
	  $ent = seed_add_serv ($p, $cfg->val($sect, $p), $HostEntity{$h});
	  if (defined ($ent)) {
	    $buf .= $ent . "\n";
	    if ($p eq "aip") {
	      push (@aips, $cfg->val($sect, $p));
	    }
	  }
	}
      }
    }
    $links = seed_add_links (@aips);
    if (defined ($links)) {
      $buf .= $links;
    } else {
      advise ("ERROR: No links\n");
    }
    debug ($buf);
    print SEED $buf;
   close (SEED);
  } else {
    advise ("Not replacing NIDB seed file\n");
  }
  advise ("Loading NIDB seed file\n");
  `cat $seed | su -c /freedom/bin/nidbtool - freedom 1>&2`;
  SetMOTD ($netname,  "/usr/freedom/etc/motd.conf");
}


sub config_remotedb {
  my ($dbname, $host, $port, $authkey) = @_;
  my ($env, $val);

  if (!($host =~ /\.$/)) {
    $host .= "." . $MyDomain;
  } else {
    $host =~ s/\.$//;
  }
  $env = "zkfr_" . $DBMapping{$dbname} . "_Server";
  $val = $host .":". $port;
  $ENV{$env} = $val;
  debug ("$ENV{$env} = $val\n");

  $env = "zkfr_" . $DBMapping{$dbname} . "_ServerPort";
  $val = $port;
  $ENV{$env} = $val;
  debug ("$ENV{$env} = $val\n");

  $env = "zkfr_" . $DBMapping{$dbname} . "_AuthKey";
  $val = $authkey;
  $ENV{$env} = $val;
  debug ("$ENV{$env} = $val\n");

  if ( ! -f "/usr/freedom/etc/db/$dbname.remote.conf.tmpl" ) {
    advise ("ERROR: unable to locate the db config file [/usr/freedom/etc/db/$dbname.remote.conf.tmpl]\n");
  } else {
    `$pwd/genesis-confsub-dbs.sh /usr/freedom/etc/db/$dbname.remote.conf.tmpl /usr/freedom/etc/db/$dbname.remote.conf` ; 
    if (`egrep "\@..*\@" /usr/freedom/etc/db/$dbname.remote.conf`) {
      advise ("WARNING: incomplete substitution for $dbname in \n\t[/usr/freedom/etc/db/$dbname.remote.conf\n");
      0;
    } else {
      1;
    }
  }
}


sub AdjRemoteDatabase {
  msg ("Adjusting remote database configurations\n");
  foreach $dbname (keys (%DBs)) {
    advise ("Testing remote db [$dbname]: \n");
    if ($dbcfg->val($Network{$Hostname} . " " . $dbname, "isremote") eq "yes") {
      advise ("Configuring for remote db [$dbname]: \n");
      if (config_remotedb ($dbname, 
			   $dbcfg->val($Network{$Hostname} . " " . $dbname, "host"),
			   $dbcfg->val($Network{$Hostname} . " " . $dbname, "port"),
			   $dbcfg->val($Network{$Hostname} . " " . $dbname, "authkey"))) {
	debug ("--- $dbname OK\n");
      } else {
	debug ("--- $dbname ERROR\n");
      }
    }
  }
}

    
sub DoHacks {
  msg ("Applying Hacks\n");
  my (@buf, @buf2);

  if ( defined($cfg->val($MySection, "keyqry")) ) {
    advise ("Fixing KeyQueryServer to use local db... ");
    if (load_file ("/usr/freedom/etc/freedom-keyqryserver.conf", \@buf)) {
      advise ("[adjusting config] ");
      @buf2 = map { sed ($_, "s/pubkeydb.remote.conf/pubkeydb.local.conf/") } @buf;
      save_file ("/usr/freedom/etc/freedom-keyqryserver.conf", \@buf2);
      `mkdir -p /freedom/db/pubkeydb/pubkeydb`;
      `mkdir -p /freedom/db/translog/pubkeydb/pubkeydb`;
      `ln -s /freedom/etc/db/pubkeydb.local.conf /freedom/etc/db/pubkey`;
      advise ("[copying db] ");
      `cp $tdir/usr/freedom/db/pubkey/pubkey /freedom/db/pubkeydb/pubkeydb/pubkeydb 1>&2`;
      advise ("done\n");
    }
  }
}


sub CleanUp {
  if (!defined($optctl{'keep'})) {
    msg ("Cleaning up temp files\n");
    chdir ($pwd);
  } else {
    msg ("Not cleaning up temp files\n");
  }
  
  advise ("Adjusting permissions\n") ;
  `chown -R freedom /freedom/etc /freedom/db /freedom/privkey /freedom/var` ;
  advise ("Removing crontab\n") ;
  `crontab -r -u freedom`;
}


#
# And now, start the servers
# This will be tricky
#


sub BackupFreedom {
  my ($tag);
  $tag=`date +%Y%m%d-%H%M`;
  chomp ($tag);
  my $backup = "/usr/freedom-" . $tag . ".tgz";
  msg ("Backing up previous installation to $backup\n");
  system ("/bin/rm", "-f", "/usr/freedom/var/log/*") ;
  system ("gtar", "zcf", $backup, "/usr/freedom");
}



sub KillServers {

  advise ("Killing previous servers\n");

  killall ("fraipd frmaipd frniqsd frnissd frsnapd frkscpd frpostnews " .
	   "fmg fmg-maip kqd fresh nntpserver frkscpd frnscpd frwhd fridentd frloded frreplicd", "-15");
  sleep(1);
  killall ("fraipd frmaipd frniqsd frnissd frsnapd frkscpd frpostnews " .
	   "fmg fmg-maip kqd fresh nntpserver frkscpd frnscpd frwhd fridentd frloded frreplicd", "-9");

  #AIP 2.0 hack
  if  ( -f "/usr/freedom/rc/aipconf" ) {
    killall ("ident2 zkinfod fruaipd uaipd kaipd", "-15");
    sleep(1);
    killall ("ident2 zkinfod fruaipd uaipd kaipd", "-9");
    system ("/usr/freedom/rc/aipconf stop");
  }
}


sub StartSingleServer {
  my ($srv) = @_;
  my ($prog);
  
  $prog = "/freedom/sbin/fr_srvctl";
  msg ("[Re]Starting Freedom Server [$srv]\n");
  advise ("Stopping $srv: $prog $srv stop\n");
  system_freedom ("$prog", "$srv", "stop");  
  advise ("Starting $srv: $prog start $srv  --passphrase.file " . $cfg->val("info", "passphrase-file") . "\n");
  system_freedom ("$prog", "$srv", "start", "--passphrase.file", $cfg->val("info", "passphrase-file"));  
  sleep(3);
  advise ("for more info: tail /freedom/var/log/$srv\n");
  tail ("/freedom/var/log/$srv");
  `chown -R freedom /freedom/etc /freedom/db /freedom/privkey /freedom/var 1>&2` ;
}


sub InstallHybridServers {
  my ($hybridfile, $hybridrep);

  if ( defined ($cfg->val($MyNetworkSection, "hybridfile")) &&
       defined ($cfg->val($MyNetworkSection, "hybridrep")) ) {

    if ( defined ($cfg->val($MySection, "hybridfile-overide")) &&
         defined ($cfg->val($MySection, "hybridrep-overide")) ) {
      $hybridrep = $cfg->val($MySection, "hybridrep-overide");
      $hybridfile = $cfg->val($MySection, "hybridfile-overide");
    } else {
      $hybridrep = $cfg->val($MyNetworkSection, "hybridrep");
      $hybridfile = $cfg->val($MyNetworkSection, "hybridfile");
    }

    msg ("Installing the $hybridfile Hybrid Tarball\n");
    wget ( $hybridrep . "/" . $hybridfile);
    if ( ! $NOSW_INST) {
      `(cd $tdir; tar -zxpPvf $hybridfile)`;
      if ($?) {
	advise ("error untarring hybrid servers [$hybridfile]\n");
	exit (2);
      }
    }
    if ( -f "/tmp/fr_hybrid_install.sh" ) {
      if (system ("/bin/sh", "/tmp/fr_hybrid_install.sh", $tdir, $hybridfile)) {
	advise ("ERROR: with hybrid installation\n");
	exit (2);
      } else {
	`/bin/rm /tmp/fr_hybrid_install.sh`;
      }	
    } else {
      advise ("ERROR: Attempting to install an old hybrid file.\n");
      exit (2);
#      `killall fraipd frmaipd frwhd 1>&2`;
#      install_file ("usr/freedom/sbin/fraipd");
#      install_file ("usr/freedom/sbin/frmaipd");
#      install_file ("usr/freedom/sbin/fr_whlaunch");
#      `chown root /usr/freedom/sbin/fr_whlaunch 1>&2`;
#      `chgrp freedom /usr/freedom/sbin/fr_whlaunch 1>&2`;
#      `chmod 4550 /usr/freedom/sbin/fr_whlaunch 1>&2`;
#      install_file ("usr/freedom/sbin/frwhd");
#      install_file ("usr/freedom/sbin/fr_getaips_1.0h");
#      #    StartSingleServer ("fraipd");
#      #    StartSingleServer ("frmaipd");
    }
  }

  # Coreaip-specific stuff
  if ( defined($cfg->val($MySection, "coreaip")) ) {
    # AIP 1.0: has separate core-aip package
    if ( "$protocol" == "1.0" )
    {
      wget ($cfg->val($coreaip_rpms, "repository") . "/" 
	    . $cfg->val($coreaip_rpms, "coreaip"));
      msg ("Installing Coreaip RPM\n");
      inst_rpm($tdir . "/". $cfg->val($coreaip_rpms, "coreaip"));
    }
  }

}

sub StartServers {

  my (@start, $prog);
  msg ("Starting Freedom Servers\n");

#  KillServers;
  @start = `ls /freedom/etc/rc/S*`;

#  `find /freedom/db \\( -name __\\* -o -name log\\* \\) -print | xargs -t rm 1>&2`;
  foreach $prog (sort @start) {
    my ($srv);
    chomp ($prog);
    $srv = $prog;
    $srv =~ s/^.+S.+\.//;
    if ($prog =~ m/kqd/) {
      # Different arguments for this one
      advise ("Starting $srv: $prog \n");
      system_freedom ("$prog");  
    } else {
      advise ("Starting $srv: $prog --passphrase.file " . $cfg->val("info", "passphrase-file") . "\n");
      system_freedom ("$prog", "--passphrase.file", $cfg->val("info", "passphrase-file"));  
    }
    if ($?) {
      sleep (3);
      advise ("Problems starting [$srv]\n");
      advise ("For more info: tail /freedom/var/log/$srv\n");
      tail ("/freedom/var/log/$srv");
      die "Problem starting service"; 
    }
    sleep(3);
    advise ("for more info: tail /freedom/var/log/$srv\n");
    tail ("/freedom/var/log/$srv");
  }

  #`chown -R freedom /freedom/etc /freedom/db /freedom/privkey /freedom/var 1>&2` ;
}

sub GenSerNums {
  if ( defined($cfg->val($MySection, "toksrv")) ) {
    msg ("Serial Numbers\n");
    advise ("Host in the token server for this network.\n");
    system_freedom("$pwd/genesis-sngen.sh", "/freedom", $pwd, 
		   $Network{$Hostname},
		  $cfg->val($MyNetworkSection, "numserials"));
  }
}

sub DumpHostAssignments {
    my ($nm);
    msg ("Dumping Hosts Assignements\n");
    foreach $nm (keys (%Network)) {
      printf ("Host: %s is part of network %s\n", $nm, $Network{$nm});
    }
}


sub SanityTests {
  msg ("Performing simple sanity tests\n");
  advise ("Testing KeyQueryServer [looking for host1's signing key]...\n");
  if (system_freedom ("/freedom/sbin/tkq", "host1." . $MyDomain, "51109", "aip", "host1", "s", "1")) {
    #advise ("NOT FOUND\n");
  } else {
    #advise ("Found\n");
  }
  if (system_freedom ("/freedom/sbin/tkq", "host1." . $MyDomain, "51109", "aip", "host199", "s", "1")) {
    #advise ("Found\n");
  } else {
    #advise ("NOT FOUND\n");
  }
  if ( defined($cfg->val($MySection, "keyqry")) ) {
    if ( `cd /freedom/etc/db && /freedom/sbin/tpkdb | grep "^Owner" | head 1>&2` ) {
      #advise ("Found\n");
    } else {
      #advise ("NOT FOUND\n");
    }
  }
  advise ("Testing NIQS [running fr_getaips]...\n");
  if (system_freedom ("/freedom/sbin/fr_getaips", "--niqs.hostname", "niqs." . $MyDomain, "--long", "--nodns")) {
    #advise ("NOT FOUND\n");
  } else {
    #advise ("Found\n");
  }
}

Setup;

if (defined($optctl{'restart'})) {
  if ($optctl{'restart'} eq "aip") {
    StartSingleServer("fraipd");
 } elsif ($optctl{'restart'} eq "maip") {
    StartSingleServer("frmaipd");
  } else {
    advise ("Don't know how to restart service $optctl{'restart'}\n");
  }
  exit;
}

if (defined($optctl{'backup'})) {
  BackupFreedom;
}

sub SetTag {
  my ($genesis) = @_;
  my $tag=`date +%Y%m%d-%H%M`;
  my (@buf);
  chomp ($tag);
  push @buf, "[Genesis]\n";
  push @buf, sprintf ("#\n# This file has been automatically generated by Genesis\n#\n", $genesis);
  push @buf, sprintf ("Version   = %s\n", $genesis);
  push @buf, sprintf ("Installed = %s\n\n", $tag);
  push @buf, "[Freedom]\n";
  push @buf, sprintf ("Release   = %s\n", $release);
  save_file ("/usr/freedom/genesis.vers", \@buf);
}

sub DbFlush {
  `/bin/rm -f /freedom/db/nidb/*/*/*` ;  
  `/bin/rm -f /freedom/db/translog/nidb/*/*/*` ;

  `/bin/rm -f /freedom/db/niqscache/*/*` ;
  `/bin/rm -f /freedom/db/translog/niqscache/*/*` ;
}

sub DownloadAllSoftware {
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "base"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "knt"));
  wget ($cfg->val($rpms, "repository") . "/" . $cfg->val($rpms, "fmg"));
  wget ($cfg->val("sendmail", "linux"));
  if (defined $cfg->val("dbutils", "linux")) {
      wget ($cfg->val("dbutils", "linux"));
  }

  if ( defined ($cfg->val($MyNetworkSection, "hybridfile")) &&
       defined ($cfg->val($MyNetworkSection, "hybridrep")) ) {
      my ($hybridfile, $hybridrep);      
    if ( defined ($cfg->val($MyNetworkSection, "hybridfile-overide")) &&
         defined ($cfg->val($MyNetworkSection, "hybridrep-overide")) ) {
      $hybridrep = $cfg->val($MyNetworkSection, "hybridrep-overide");
      $hybridfile = $cfg->val($MyNetworkSection, "hybridfile-overide");
    } else {
      $hybridrep = $cfg->val($MyNetworkSection, "hybridrep");
      $hybridfile = $cfg->val($MyNetworkSection, "hybridfile");
    }

      wget ( $hybridrep . "/" . $hybridfile);
  }

  wget ( $keyrep . "/" . $keyfile);

  if ( defined($cfg->val($MyNetworkSection, "coreaip-release")) ) {
      wget ($cfg->val($coreaip_rpms, "repository") . "/" . $cfg->val($coreaip_rpms, "coreaip"));
  }

  GetQmailRPMs();

}

my ($commonqmail_rpms, $qmail_rpms);
sub GetQmailRPMs {

  if ($NOSW_INST) {
    advise ("NOTE: NO Software Installation Option in effect. NO RPM update will occur\n");
  }

  msg ("Checking Qmail configuration parameters\n");
  # Checking for and setting rpm locations
  if (! defined ($cfg->val($MyNetworkSection, "commonqmail-release")) ) {
    die "No commonqmail-release parameter found in Network section of config file";
  } else {
    $commonqmail_rpms = "rpms " . $cfg->val($MyNetworkSection, "commonqmail-release");
    if (! defined ($cfg->val($commonqmail_rpms, "repository")) ) {
      die "No commonqmail-rpm repository found for $rpms in configuration file";
    }
  }
  if ( defined ($cfg->val($MySection, "qmail-overide")) ) {
    $release = $cfg->val($MySection, "qmail-overide");
    $qmail_rpms = "rpms " . $release;
  } elsif ( defined ($cfg->val($MySection, "release-overide")) ) {
    $release = $cfg->val($MySection, "release-overide");
    $qmail_rpms = "rpms " . $release;
  } elsif ( defined ($cfg->val($MyNetworkSection, "release"))) {
    $release = $cfg->val($MyNetworkSection, "release");
    $qmail_rpms = "rpms " . $release;
  } else {
    die "No qmail nor rpm release defined in configuration file";
  }
  # Checking all rpm files are defined in genesis.conf
  if (!defined ($cfg->val($commonqmail_rpms, "ucspi-tcp")) ) {
    die "No ucspi-tcp file in '$commonqmail_rpms' common-Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($commonqmail_rpms, "daemontools")) ) {
    die "No daemontools file for '$commonqmail_rpms' common-Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail")) ) {
    die "No qmail file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-IMEP")) ) {
    die "No qmail-IMEP file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-IMEPB")) ) {
    die "No qmail-IMEPB file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-NMTA")) ) {
    die "No qmail-NMTA file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-NMTAB")) ) {
    die "No qmail-NMTAB file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-POP")) ) {
    die "No qmail-POP file for '$qmail_rpms' Qmail repository defined in configuration file";
  }
  if (!defined ($cfg->val($qmail_rpms, "qmail-RELAY")) ) {
    die "No qmail-RELAY file for '$qmail_rpms' Qmail repository defined in configuration file";
  }

  # Downloading common-qmail RPMs
  msg ("Downloading common-Qmail (" . $release . " release) RPMs from\n[===] " . $cfg->val($commonqmail_rpms, "repository") . "\n");
  wget ($cfg->val($commonqmail_rpms, "repository") . "/" . $cfg->val($commonqmail_rpms, "ucspi-tcp"));
  wget ($cfg->val($commonqmail_rpms, "repository") . "/" . $cfg->val($commonqmail_rpms, "daemontools"));
  # Downloading (freedom) qmail RPMs
  msg ("Downloading Qmail (" . $release . " release) RPMs from\n[===] " . $cfg->val($qmail_rpms, "repository") . "\n");
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-IMEP"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-IMEPB"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-NMTA"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-NMTAB"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-POP"));
  wget ($cfg->val($qmail_rpms, "repository") . "/" . $cfg->val($qmail_rpms, "qmail-RELAY"));
}

sub InstallCommonQmailRPMs {
  my ($entity);
  my %server2rpm = (
                   "imep"           => "qmail-IMEP",
                   "imepb"          => "qmail-IMEPB",
                   "nmta"           => "qmail-NMTA",
                   "nmtab"          => "qmail-NMTAB",
                   "pop"            => "qmail-POP",
                   "relay"          => "qmail-RELAY"
                   );
  # Uninstall the previous installation
  msg ("Stopping qmail servers\n");
  msg ("Uninstalling older Qmail RPMs\n");
  rm_rpm ("--nodeps freedom-qmail-RELAY");
  rm_rpm ("--nodeps freedom-qmail-POP");
  rm_rpm ("--nodeps freedom-qmail-NMTAB");
  rm_rpm ("--nodeps freedom-qmail-NMTA");
  rm_rpm ("--nodeps freedom-qmail-IMEPB");
  rm_rpm ("--nodeps freedom-qmail-IMEP");
  rm_rpm ("--nodeps freedom-qmail");
  rm_rpm ("--nodeps daemontools");
  rm_rpm ("--nodeps ucspi-tcp");

  # Check if we are wiping everything
  if ($DO_CLEAN) {
    msg ("Removing all trace of previous Qmail installation\n");
    
    # FIX FIX FIX
    #   removing everything in /usr/freedom is just plain wrong!
    #
    # system("find /freedom ! -type d -print | xargs rm -f 1>&2");
    system("find /usr/freedom/qmail ! -type d -print | xargs rm -f 1>&2");
  }
  
  # Install the common Qmail RPMs
  msg ("Installing common Qmail RPMs\n");
  inst_rpm($tdir . "/". $cfg->val($commonqmail_rpms, "ucspi-tcp"));
  inst_rpm($tdir . "/". $cfg->val($commonqmail_rpms, "daemontools"));
  inst_rpm($tdir . "/". $cfg->val($qmail_rpms, "qmail"));
}

sub InstallSpecialQmailRPM ($) {
  my ($ent) = @_;
  my %server2rpm = (
                   "imep"           => "qmail-IMEP",
                   "imepb"          => "qmail-IMEPB",
                   "nmta"           => "qmail-NMTA",
                   "nmtab"          => "qmail-NMTAB",
                   "pop"            => "qmail-POP",
                   "relay"          => "qmail-RELAY"
                   );

  # Install the machine specific Qmail RPMs
  if (defined $server2rpm{$ent}) {
    msg ("Installing machine specific Qmail RPM for $ent\n");
    inst_rpm($tdir . "/". $cfg->val($qmail_rpms, $server2rpm{$ent}));
  }
}

sub write_file {
    my $file = shift;
    my $string = shift;

    open(FILE, ">$file") || croak("couldn't open $file: $!\n");
    
    print FILE $string;
    close FILE;
}

sub InstallMailMacKey {
    foreach $param ($cfg->Parameters($MySection)) {
	if (defined($MACKeyFileMapping{$param})) {
	    install_mackey("/usr/freedom/" . $MACKeyFileMapping{$param});	    
	}
    }
    return 1;
}
  
sub GlueQmail {
  my ($h, $p, $sect, $ip_list, $news_locals);

  my ($qmqpservers) = "/freedom/qmail/control/qmqpservers";
  my ($qmqpclients) = "/freedom/qmail/control/qmqpclients";
  my ($rcpthosts) = "/freedom/qmail/control/rcpthosts";
  my ($nntp_host) = "/freedom/qmail/control/nntp.host";
  my ($nntp_port) = "/freedom/qmail/control/nntp.port";

  my ($kqd_host) = "/freedom/qmail/control/kqd.host";
  my ($kqd_port) = "/freedom/qmail/control/kqd.port";

  my ($locals) = "/freedom/qmail/control/locals";

  my ($welcome_domain) = "/freedom/qmail/control/welcome-domain";

  my ($relay_mail_from) = "/freedom/qmail/control/relaymailfrom";

  my ($qmail_domains); 

  my %qmail_qp_clients = (

  # Each key is a client of all entities in the value list (each entity
  # must appear in the key's qmqp_servers file, so that the key will
  # know the servers it can talk to for qmqp.)

			  'imep' => ['imepb',], # e.g., each imep is a client for each imepb.
			  'nmta' => ['nmtab',],
			  'nmtab' => ['relay',],
			  'imepb' => ['relay',],
			  'pop'=> ['imepb',],
			   );

  my %qmail_qp_servers = ( 
  
 
  # Each key is a server for all entities in the value list (each entity
  # must appear in the key's qmqp_clients file, so that the key will know to
  # accept qmqp connections from the entity.)

			   'imepb' => ['imep', 'pop'], # e.g., the imepb is a server for all imeps and pops
			   'nmtab' => ['nmta',],
			   'relay' => ['nmtab', 'imepb',],
			   );

  # This hash of lists is populated with names of actual hosts.

  my %QmailServerLists = (
			  'imep' => [],
			  'imepb' => [],
			  'nmta' => [],
			  'nmtab' => [],
			  'pop' => [],
			  'relay' => []
			  );


  # Given a reference to %QmailServerLists and a list of mail services
  # (i.e. 'imep', 'nmtab', etc.)  returns a string of newline
  # separated ip addresses of all hosts in all services in
  # %QmailServerLists.

  sub generate_qmqp_ip_set {
      my $servers = shift;
      my $services = shift;
      
      my $set = {};

      # Generate set of all hosts for all services.
      foreach my $service (@{$services}) {
	  foreach my $server (@{$servers->{$service}}) {
	      $set->{$server} = 1; 
	  }
      }

      my $string = "";

      foreach my $host (sort(keys(%{$set}))) {
	  $string .= find_ip($host) . "\n";
      }
      return $string;
  }  

  msg "Wiring Qmail servers together on local machine\n";
  print "Hostname = $Hostname\n";

  # Check we have list of qmail domains
  if (! defined($cfg->val($MyNetworkSection,"qmail-domains")) ) {
    die "No qmail-domains defined in 'network' section of configuration file"; 
  } else {
    ($qmail_domains = $cfg->val($MyNetworkSection,"qmail-domains")) =~ s/:/\n/g;
  }

  # Build the lists of Qmail servers
  foreach $h (sort (@{$Net{$Network{$Hostname}}})) {
    $sect = $Network{$Hostname} . " " . $HostEntity{$h};
    foreach $p ($cfg->Parameters($sect)) {
      if (exists $QmailServerLists{$p}) {
        push (@{$QmailServerLists{$p}}, $cfg->val($sect, "hostname"));
      }
    }
  }

  # Write qmail client/server files and domain lists.

  foreach $p ($cfg->Parameters($MySection)) {
    # Wire me to my back-end
    if (defined $qmail_qp_clients{$p}) {
      $ip_list = generate_qmqp_ip_set(\%QmailServerLists, $qmail_qp_clients{$p});
      print "Setting qmqpservers file for $p on $Hostname to contain these ips: \n[\n$ip_list]\n"; 
      write_file($qmqpservers, $ip_list);
    }
    # Wire me to my front-end
    if (defined $qmail_qp_servers{$p}) {
      $ip_list = generate_qmqp_ip_set(\%QmailServerLists, $qmail_qp_servers{$p});
      print "Setting qmqpclients file for $p on $Hostname to contain these ips: \n[\n$ip_list]\n"; 
      write_file($qmqpclients, $ip_list);
    }
    # Set up domains for imep
    if ($p eq 'imep' || $p eq 'imepb' || $p eq 'nmta' || $p eq 'nmtab') {
      print "Setting rcpthosts file for $p on $Hostname to contain these domains:\n";
      print "[\n$qmail_domains]\n";
      write_file($rcpthosts, $qmail_domains);
    }
    
    # Write welcome-domain for pop
    if ($p eq 'pop') {
      if (defined($cfg->val($MyNetworkSection, "domain"))) {
        my $welcome_domain_string = $cfg->val($MyNetworkSection, "domain");
	print "Writing welcome-domain `$welcome_domain_string' for pop host $p\n";
	write_file($welcome_domain, $welcome_domain_string);
      }
    }

    # Set up kqd information imepb
    if ($p eq 'imepb') {
      if ( defined($cfg->val($MyNetworkSection, "kqd-hostname")) ) {
        my $kqd_host_value=$cfg->val($MyNetworkSection, "kqd-hostname");
        advise ("Creating $kqd_host\n");
	write_file($kqd_host, $kqd_host_value);
        if ( defined ($cfg->val($MyNetworkSection, "kqd-port")) ) {
          my $kqd_port_value=$cfg->val($MyNetworkSection, "kqd-port");
          advise ("Creating $kqd_port\n");
	  write_file($kqd_port, $kqd_port_value);
        }
      }
    }

    # Set up domains for imepb and nmtab
    if ($p eq 'nmta' || $p eq 'nmtab' || $p eq 'imepb') {
      print "Setting locals file for $p on $Hostname to contain these domains:\n";
      print "[\n$qmail_domains]\n";
      write_file($locals, $qmail_domains);
    }

    # Write relaymailfrom file for nmta
    if ($p eq 'nmta') {
      if (defined($cfg->val($MyNetworkSection, "qmail-domains"))) {
	  my $relay_mail_from_string = '';
	  foreach my $domain (split ":", $cfg->val($MyNetworkSection,"qmail-domains")) {
	      $relay_mail_from_string .= '@' . $domain . "\n";
	  } 
	  print "Setting relaymailfrom file for $p on $Hostname to contain this string:\n";
	  print "[\n$relay_mail_from_string]\n";
	  write_file($relay_mail_from, $relay_mail_from_string);
      }
    }

    # Set up locals, nntp.port and nntp.host for relay
    if ($p eq 'relay') {
      # Create nntp.hostname and nntp.port
      if ( defined($cfg->val($MySection, "nntp-hostname")) ) {
        my $nntphost=$cfg->val($MySection, "nntp-hostname");
        advise ("Creating $nntp_host\n");
	write_file($nntp_host, $nntphost);
        if ( defined ($cfg->val($MySection, "nntp-port")) ) {
          my $nntpport=$cfg->val($MySection, "nntp-port");
          advise ("Creating $nntp_port\n");
	  write_file($nntp_port, $nntpport);
        }
      }
      # Create locals
      advise ("Creating $locals\n");
      $news_locals = '';
      foreach my $domain (split ":", $cfg->val($MyNetworkSection,"qmail-domains")) {
        $news_locals .= "news." . $domain . "\n";
      } 
      write_file($locals, $news_locals);
    }
    # Bug 7408 - Set up aliases for imepb
    # Set up domains for imep
    if ($p eq 'imep' || $p eq 'imepb' || $p eq 'nmta' || $p eq 'nmtab') {
      print "Initializing imep aliases on $Hostname to contain these domains:\n";
      `cp /freedom/qmail/control/locals /freedom/qmail/control/defaulthost 2>&1 `;
      `/freedom/qmail/bin/compile-aliases /freedom/qmail/delivery/alias-list 2>&1`;
    }
    
  }
}


sub InstallQmail {
  my ($qmail_entity);

  # Does everything related to Qmail installation except setting up the NIDB
  # and some stuff required from the setup() subroutine
  GetQmailRPMs;
  InstallCommonQmailRPMs;
  foreach $qmail_entity ($cfg->Parameters($MySection)) {
    InstallSpecialQmailRPM($qmail_entity);
    if (defined($MACKeyFileMapping{$qmail_entity})) {
#      install_mackey("/usr/freedom/" . $MACKeyFileMapping{$qmail_entity});
      install_mackey("/freedom/" . $MACKeyFileMapping{$qmail_entity});
    }
  }
  GlueQmail;
}



if (defined($optctl{'dump'})) {
  DumpHostAssignments;
} else {
  my ($login) = `whoami`;
  chomp($login);
  if ($must_be_root && $login ne "root") {
     print STDERR "User $login does not have sufficient privileges to\n" .
    "install this network. Become root.\n";
    exit 2;
  }
  if (defined($optctl{'step'})) {
    if ($optctl{'step'} eq "rpm") {
      InstallRPMs;
      InstallHybridServers;
    } elsif ($optctl{'step'} eq "conf") {
      AdjConfigFiles;
    } elsif ($optctl{'step'} eq "keys") {
      InstallKeys;
    } elsif ($optctl{'step'} eq "nidb") {
      if ($IS_NIQS) {
        LoadNidbData ($Network{$Hostname});
      }
    } elsif ($optctl{'step'} eq "db") {
      AdjRemoteDatabase;
    } elsif ($optctl{'step'} eq "clean") {
      CleanUp;
    } elsif ($optctl{'step'} eq "start") {
      StartServers;
    } elsif ($optctl{'step'} eq "kill") {
      KillServers;
    } elsif ($optctl{'step'} eq "serials") {
      GenSerNums;
    } elsif ($optctl{'step'} eq "extras") {
      Extras;
    } elsif ($optctl{'step'} eq "sendmail") {
      AdjSendmailConfig;
    } elsif ($optctl{'step'} eq "test") {
      SanityTests;
    } elsif ($optctl{'step'} eq "db_flush") {
      DbFlush;
    } elsif ($optctl{'step'} eq "qmail_conf") {
      GlueQmail;
    } elsif ($optctl{'step'} eq "mailmackey") {
      InstallMailMacKey;
    } else {
      msg ("Unknown step [$optctl{'step'}]\n");
    }
  } elsif (defined($optctl{'download-only'})) {
      DownloadAllSoftware;
      exit(0);
  } else {
    InstallRPMs;
    InstallHybridServers;
    if ( $NOT_FREE ||
	    ( defined($cfg->val($MySection, "relay")) ) ||
	    ( defined($cfg->val($MySection, "imep")) ) ||
	    ( defined($cfg->val($MySection, "imepb")) ) ||
	    ( defined($cfg->val($MySection, "nmta")) ) ||
	    ( defined($cfg->val($MySection, "nmtab")) ) ||
	    ( defined($cfg->val($MySection, "pop")) )
        )
    {
      InstallQmail;
    }
    AdjConfigFiles;
    InstallKeys;
    if ($IS_NIQS) {
      LoadNidbData ($Network{$Hostname});
    }
    AdjRemoteDatabase;
    #DoHacks;
    if ( ! defined ($optctl{'no-sngen'}) ) {
       GenSerNums;
    } else {
       advise ("Not Generating Serial numbers");
    }
    CleanUp;
    SetTag($Version);
    if ( ! defined ($optctl{'no-start'}) ) {
      # StartServers;
      SanityTests;
    } else {
      advise ("Not Starting Servers");
    }
  }
  # Set the machine's MOTD if a fresh install or
  # append a note if this is a step
  if (defined($optctl{'step'})) {
    msg ("Appending to the machine's MOTD ($MachineMOTD).\n");
    open (MOTD, ">>$MachineMOTD") || die "Can't create [$MachineMOTD]: $!";
    print MOTD "\n";
    print MOTD "*** Genesis step $optctl{'step'} was run on:\n";
    print MOTD "      " . `date +"%Y-%m-%d %H:%M"`;
    print MOTD "\n";
    close (MOTD);
  } else {
    msg ("Setting the machine's MOTD ($MachineMOTD).\n");
    SetMOTD ($Network{$Hostname},  $MachineMOTD);
  }
    
  msg ("Installation complete.\n");
  msg ("Your NIQS should be: niqs.$MyDomain\n");

}

