#!/bin/sh -eu
#
# $Id: genesis_sendmail.sh,v 1.6 2000/05/21 22:04:05 luc Exp $
#
# fr_confsubst - Perform configuration substitutions
#
# This script is called by fradmin with various arguments.
#
# Copyright (c) 1998-1999, Zero-Knowledge Systems Inc. All Rights Reserved.
#

PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/ucb:$PATH
export PATH


tmp=/tmp/gs-$$

cleanup () {
    /bin/rm -f $tmp
    if [ $# -gt 0 ] ; then
	exit 1;
    else
	exit 0;
    fi
}

sed_ip () {
    sed $* > $tmp;
    shift $[$#-1]

    cp $tmp $1
}

trap 'cleanup 1' 1 2 3 15;

aliases=0;

if [ $# -eq 5 ]; then
	inputfile=$1
	domainname=$2
	sendmail=$3
	fr_root=$4
	sm_daemon=$5
else
	echo 1>&2 "Usage: $0 <inputfile> <domainname>"
	exit 1
fi

freedom_domain="${domainname}";

echo INPUT $inputfile

echo freedom domain $freedom_domain $2

for name in /etc/sendmail.cf /etc/sendmail.cL ; do
    if [ -f $name -a ! -f $name.orig ]; then
	mv $name $name.orig
    fi
done

if [ ! -f /etc/sendmail.ct ] ; then 
    touch /etc/sendmail.ct
fi

echo "# Adjusting sendmail.cf file";
sed < ${inputfile} > /etc/sendmail.cf \
	-e "s;@domain_name@;${freedom_domain};g" 

cat - <<EOF > $fr_root/sbin/frspoolnews.sh
#!/bin/sh
#
# Automatically generated by Genesis
#
$fr_root/sbin/frspoolnews --config /freedom/etc/freedom-fmg.conf --post no

EOF
chmod 755 $fr_root/sbin/frspoolnews.sh

if grep --quiet "/sbin/frspoolnews.sh" /etc/aliases ; then 
    :
else 
    sed_ip 's/^mail2news:/#mail2news/' /etc/aliases;
    echo "# Adding aliase for mail2news";
    printf '# --[Genesis]--\n#\n# mail2news entry added by genesis $Revision: 1.6 $\n#\n#\n' >> /etc/aliases
    echo 'mail2news:	"|/freedom/sbin/frspoolnews.sh"'\
	    >> /etc/aliases
    printf '#\n# --[end]--\n' >> /etc/aliases
    aliases=1;
fi

cat - <<EOF > $fr_root/sbin/fmgfilter.sh
#!/bin/sh
#
# Automatically generated by Genesis 
#
$fr_root/sbin/fmgfilter --fmgport 51111

EOF
chmod 755 $fr_root/sbin/fmgfilter.sh

if grep --quiet "/sbin/fmgfilter.sh" /etc/aliases ; then 
    :
else 
    echo "# Adding aliase for fmg's filter";
    sed_ip 's/^fmg:/#fmg/' /etc/aliases;
    printf '# --[Genesis]--\n#\n# fmg entry added by genesis $Revision: 1.6 $\n#\n#\n' >> /etc/aliases
    echo 'fmg:		"|/freedom/sbin/fmgfilter.sh"' >> /etc/aliases
	
    printf '#\n# --[end]--\n' >> /etc/aliases
    aliases=1;
fi

do_cL () {

    cat - >> /etc/sendmail.cL <<EOE
MAILER-DAEMON
abuse
abusetest
accounts
adm
admin
billing
bin
daemon
decode
dumper
fmg
freedom
freedomsupport
ftp
games
gopher
halt
hostmaster
info
ingres
lp
mail
mail2news
majordomo
manager
news
nobody
operator
policy
postmaster
root
security
server_keys
server_support
shutdown
support
sync
sysadmin
system
toor
uucp
webmaster
freedomsupport
requests
server-tickets
store
__genesis_marker
EOE

}

if [ -f /etc/sendmail.cL ] ; then 
    if grep --quiet __genesis_marker /etc/sendmail.cL ; then 
	:
    else 
        echo "# Adjusting local delivery list";
	do_cL;
    fi
else
    do_cL;
fi

/etc/rc.d/init.d/sendmail stop

#
# Finally fix sendmail binary
# 
A=`strings /usr/sbin/sendmail | grep 8 | tail -1 `;
a=`echo $A | sed 's/\.//g'`;
B=`strings $sendmail | grep 8 | tail -1 `;
b=`echo $B | sed 's/\.//g'`;
if [ "$a" -lt "$b" ] ; then
    echo "# Replacing sendmail ($A) binary with a more recent version ($B)"
    mv /usr/sbin/sendmail /usr/sbin/sendmail.old
    cp $sendmail /usr/sbin/sendmail
    chown root.root /usr/sbin/sendmail
    chmod 6555 /usr/sbin/sendmail
    aliases=1;
else
    echo "# sendmail is version $A"
fi

echo "# Rebuilding mail aliases"
if /usr/sbin/sendmail -bi ; then
  :
else
  echo "# Problem with new sendmail aliases file\n";
  cleanup 1;
fi

if [ "x$sm_daemon" = "xyes" ] ; then
    echo "# Enabling and starting sendmail daemon"
    /sbin/chkconfig --level 345 sendmail on
    /etc/rc.d/init.d/sendmail start
else 
    echo "# Disabling sendmail daemon"
    /sbin/chkconfig --level 345 sendmail off
fi

cleanup ;
