#!/usr/bin/perl -w

# $Id: list_networks.pl,v 1.1 2000/10/10 15:34:31 philippem Exp $

use IniConf;
use strict;
use Getopt::Long;
use Data::Dumper;

my $options = {};
my $configuration_file = 'genesis.conf';
my $cfg;

sub usage {
  die("usage: $0 [--config <configuration_file>]\n");
}

if (!GetOptions($options, 
		"config=s",
		)) {
  usage();
}

if (defined($options->{'config'})) {
  $configuration_file = $options->{'config'};
}

$cfg = IniConf->new( -file => $configuration_file, -nocase => 1, -default => "Default" );
if (!defined ($cfg) ) {
  print STDERR "Can't initialize configuration from file $configuration_file";
  print STDERR join ('\n', @IniConf::errors);
}
   
package Network_Set;

sub new 
{
  my($class) = shift;
  my($self) = { };
  bless($self, $class);

  return $self;
}

sub add_network
{
  my($self) = shift;
  my($network) = shift;
  $self->{$network->get('network-name')} = $network;
  return $self;
}

sub get_networks
{
  my($self) = shift;
  return [ sort(values(%{$self})) ];
}
  
package Set_Get;
use vars qw($AUTOLOAD);

sub AUTOLOAD
{
  my $self = shift;
  my $attr = $AUTOLOAD;
  $attr =~ s/.*:://;
  return unless $attr =~ /[^A-Z]/;
  if (!defined($self->{$attr})) {
    carp("attempt to get undefined value `$attr'");
    return undef;
  } 
  return $self->{$attr};
}

sub set
{
  my($self) = shift;
  my($key) = shift;
  my($value) = shift;
  $self->{$key} = $value;
  return $self;
}

sub get
{
  my($self) = shift;
  my($key) = shift;
  return $self->{$key};
}

package Network;
use vars qw(@ISA);
@ISA= qw(Set_Get);

sub new 
{
  my($class) = shift;
  my($self) = { @_ };
  bless($self, $class);

  $self->{'_host_entities'} = [];
  return $self;
}

sub add_host_entity
{
  my($self) = shift;
  my($host_entity) = shift;
  print "adding host " . $host_entity->hostname() . " to network " . $self->{'network-name'} . "\n";
  
  push(@{$self->{'_host_entities'}}, $host_entity);
  return $self;
}

sub get_host_entities
{
  my($self) = shift;
  return $self->{'_host_entities'};
}

package Host;
use vars qw(@ISA);
@ISA= qw(Set_Get);

sub new
{
  my($class) = shift;
  my($self) = { @_ };
  bless($self, $class);
  return $self;
}

package main;

my $networks = new Network_Set;

foreach my $section ($cfg->Sections()) {
  if ($section =~ m/^network\s+(.+)/) {
    my $network_name = $1;
    my $network = new Network();
    foreach my $parameter ($cfg->Parameters($section)) {
      $network->set($parameter, $cfg->val($section, $parameter));
    }
    foreach my $host_entity ($cfg->GroupMembers($network_name)) {
      my $host = new Host();
      my $sub_section = "$network_name $host_entity";
      foreach my $parameter ($cfg->Parameters($sub_section)) {
	$host->set($parameter, $cfg->val($sub_section, $parameter));
      }
      $network->add_host_entity($host);
    }
    $networks->add_network($network);
  }
}

foreach my $network (@{$networks->get_networks()}) {
  print $network->get('network-name');
  print "\n";
  foreach my $host_entity (@{$network->get_host_entities()}) {
    print $host_entity->get('host-entity');
    print $host_entity->hostname();
    print "\n";
  }
}
print Dumper($networks);
    
  
