#include <stdio.h>
#include "load.h"

#define NOBOUNDARY(b) { \
                        b->title = NULL; \
                        b->connections = -1.0; \
                        b->requests = -1.0; \
                        b->data = -1.0; \
                        b->traffic = None; \
                      }

/*
 *
 */

int
loadDump (
  FILE * stream ,
  Load * l )
{
  int ret = 1;

  if (!stream || !l) {

  } else {
    if (l->title) {
      fprintf(stream,"\n%s\n",l->title);
    }

    if (l->connections >= 0.0) {
      fprintf(stream,"  Connections: %f\n",l->connections);
    } 

    if (l->requests >= 0.0) {
      fprintf(stream,"  Requests: %f\n",l->requests);
    }

    if (l->data >= 0.0) {
      fprintf(stream,"  Data: %f\n",l->data);
    }
  }

  return ret;
}

int
loadDump2 (
  FILE * stream ,
  FreedomParameters * p ,
  Load * l )
{
  int ret = 1;
  double duration = 0.0;

  if (!stream || !p) {
    ret = 0;
  } else if (l == NULL) {
    fprintf(stream,"\n\n%14s  %10s %10s %10s %10s %10s %10s %10s\n",
            "subscribers",
            "client",
            "aip",
            "reqs/sec",
            "cnx/sec",
            "simul_cnx",
            "bytes/sec",
            "cnx_dur");
  } else {
    duration = l->data / l->connections / p->nym_min_throughput;
    fprintf(stream,"%14d %10d %10d %10.3f %10.3f %10.3f %10.3f %10.3f\n",
            (unsigned int)(p->freedom_active_clients * p->freedom_active_nym_per_subscriber),
            (unsigned int)p->freedom_active_clients,
            (unsigned int)p->aips,
            l->requests / 3600.0,
            l->connections / 3600.0 ,
            l->connections / 3600.0 * duration ,
            l->data / 3600.0,
            duration);
  }

  return ret;
}

/*
 *
 */

int
loadAdd (
  Load * l_total,
  Load * l )
{
  int ret = 0;

  if (!l_total || !l) {
    ret = 0;
  } else {
    l_total->requests += l->requests;
    l_total->data += l->data;
    l_total->connections += l->connections;
  }

  return ret;
}

/*
 *
 */

int 
load_WH_to_Internet (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_WH_to_Internet"; 

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = InternetTraffic;
    
  }
  return ret;
}

/*
 *
 */

int 
load_WH_to_NISS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_WH_to_NISS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
#warning The wormhole stat update frequency is hard coded to 12 / hour
    l->title = func;
    l->traffic = ClearTraffic;
    l->requests = p->aips * 12;
    l->connections = l->requests;
    l->data = l->requests * (p->wormhole_stat_report_size + p->signature_size);
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_TS_to_AIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;

  const char * func = "load_TS_to_AIP";
  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = CloudTraffic;

  }
  return ret;
}

/*
 *
 */

int 
load_AIP_to_KQD (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_AIP_to_KQD";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;

    /*
     * connects is the same as the number of requests since this is UDP
     */
    /* BUGBUG .. is it force_start instead ? */
    l->connections = p->avg_nym_route_create * p->freedom_active_clients +
                     (p->avg_aip_force_update * p->aips * 10);
    l->requests = p->avg_nym_route_create * p->freedom_active_clients +
                     (p->avg_aip_force_update * p->aips * 10);
    l->data = l->requests * (p->pub_key_request_size + p->pub_key_size + p->signature_size);
  
    b->connections = -1;
    b->requests = -1;
    b->data = -1;

  }
  return ret;
}

/*
 *
 */

int 
load_AIP_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_AIP_to_NIQS";
  int requests_per_connect = 0;
  int request_size = 0;

  if (!p || !l || !b) {
    ret = 0;
  } else {
#warning Hard coded aip NIQS update frequency, 2 (2 updates/aip/hour)

    /* The +1 is for the access control list */

    requests_per_connect = (p->avg_aip_neighbours + 1.0 + p->core_servers);

    request_size = (((p->avg_niqs_response_size * 
                      (p->avg_aip_neighbours + p->core_servers)) +
                     p->avg_niqs_accessctrl_size)) + 
      (requests_per_connect * p->signature_size);

    l->title = func;
    l->traffic = ClearTraffic;
    l->connections = (2.0 * (1.0 + p->avg_aip_force_update)) * p->aips;
    l->requests = requests_per_connect * l->connections;
    l->data = request_size * l->connections;

    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_AIP_to_NISS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_AIP_to_NISS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;
#warning hard coded NISS stat update frequency for an AIP (1/hour)
    
    l->requests = p->aips * (1.0 + p->avg_aip_force_stats) * 
                  (1.0 * (1.0 + p->avg_aip_neighbours));
    l->connections = l->requests;
    l->data = (p->aip_stat_report_size + p->signature_size) * l->requests;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_NISS_to_AIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_NISS_to_AIP";
 
  if (!p || !b || !l) {
    ret = 0;
  } else {
#warning "double check the AIP <-> NISS equations.  I'm sure I didnt do this right"
    l->title = func;
    l->traffic = ClearTraffic;
    l->requests = p->aips * (1.0 + p->avg_aip_force_stats);
    l->connections = l->requests;
    l->data = l->requests * (p->aip_stat_request_size + p->signature_size);
    NOBOUNDARY(b);
  }

  return ret;
}

/*
 *
 */

int 
load_MAIP_to_NISS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_MAIP_to_NISS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;
#warning MAIP niss update frequency is set to 1 /hour
    l->requests = 1.0 * p->maips;
    l->connections = l->requests;
    l->data = l->requests * (p->maip_stat_report_size + p->signature_size);
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_MAIP_to_MAIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_MAIP_to_MAIP";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;
    l->connections = p->avg_mail_rcvd_per_nym * p->freedom_subscribers * p->avg_replyblocks_per_nym; 
    /* 
     * There are four specific requests associated with each maip->maip
     * mail transfer.
     */
    l->requests = l->connections * 4;
    l->data = l->connections * p->avg_mail_message_size;

    b->title = NULL;
    b->connections = (3600.0 / p->maip_spool_interval) * p->maip_msgs_per_interval;
    b->requests = b->connections * 4.0 + 1;
    b->data = -1.0; /* No limit imposed here */
    b->traffic = ClearTraffic;
  }
  return ret;
}

/*
 *
 */

int 
load_MAIP_to_KQD (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_MAIP_to_KQD";

  if (!p || !l || !b) {
    ret = 0;
  } else {

    /*
     * NOTE: This is not quite correct.  The maip does lookup the NISS public
     *       key on startup, but since the maip is started only once every
     *       few weeks, it is easier to leave out this inconsequential 
     *       traffic.
     */

    l->title = func;
    l->traffic = ClearTraffic;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int
load_frnetinfo_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_frnetinfo_to_NIQS";
  double all_entities = 0.0;
  double some_entities = 0.0;
  double complete_update = 0.0;
  double partial_update = 0.0;
  double complete_update_size = 0.0;
  double partial_update_size = 0.0;

  if (!p || !l || !b) {
    ret = 0;
  } else {

    all_entities = p->core_servers + p->aips + p->maips;
    some_entities = all_entities * p->percentage_entity_change;

    complete_update = all_entities
                      + 1.0 /* clientinfo */
                      + 1.0 /* Available command */
                      + 1.0 /* list */
                      + 1.0; /* status */
    partial_update =  some_entities
                      + 1.0 /* clientinfo */
                      + 1.0 /* Available command */
                      + 1.0 /* list */
                      + 1.0; /* status */

#warning "The record size for the list and status NIQS entries are bad guesses" 
    complete_update_size = all_entities * p->avg_niqs_response_size
    /* clientinfo */       + (1.0 * p->avg_niqs_response_size)
    /* available */        + (all_entities * 4.0)
    /* list */             + (all_entities * 50.0)
    /* status */           + (all_entities * 20.0)
    /* the sigs */         + complete_update * p->signature_size;

    partial_update_size = some_entities * p->avg_niqs_response_size
    /* clientinfo */       + (1.0 * p->avg_niqs_response_size)
    /* available */        + (some_entities * 4.0)
    /* list */             + (some_entities * 50.0)
    /* status */           + (some_entities * 20.0)
    /* the sigs */         + partial_update * p->signature_size;

    l->title = func; l->traffic = None;
    l->requests = (complete_update * p->percentage_full_niqs_update) + 
                  (partial_update * (1.0 - p->percentage_full_niqs_update));
    l->connections = 1.0;
    l->data = (complete_update_size * p->percentage_full_niqs_update) + 
              (partial_update_size * (1.0 - p->percentage_full_niqs_update));
    NOBOUNDARY(b);
  }

  return ret;
}

/*
 *
 */

int 
load_MAIP_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_MAIP_to_NIQS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    ret = load_frnetinfo_to_NIQS(p,l,b);
#warning Hard coded MAIP NIQS update frequency to 2/hour
    l->title = func;
    l->traffic = ClearTraffic;
    l->connections *= (p->maips * 2.0);
    l->requests *= (p->maips * 2.0);
    l->data *= (p->maips * 2.0);
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_MAIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_MAIP";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;

    b->title = NULL;
    b->connections = (3600.0 / p->maip_spool_interval) * p->maip_msgs_per_interval;
    b->requests = b->connections * 4 + 1;
    b->data = -1.0; /* No limit imposed here */
    b->traffic = ClearTraffic;
  }
  return ret;
}

/*
 *
 */

int 
load_CORE_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CORE_to_NIQS";
  double core_servers = 0.0;

  if (!p || !l || !b) {
    ret = 0;
  } else {
    ret = load_frnetinfo_to_NIQS(p,l,b);
    l->title = func;
    l->traffic = ClearTraffic;
    core_servers = p->nym_servers + p->token_servers;
#warning CORE Server NIQS update frequency hardwired to 2.0 per hour
    l->connections *= (core_servers * 2.0);
    l->requests *= (core_servers * 2.0);
    l->data *= (core_servers * 2.0);
    NOBOUNDARY(b);

  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_NIQS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    ret = load_frnetinfo_to_NIQS(p,l,b);
    l->title = func;
    l->traffic = ClearTraffic;
#warning FMG MAIP NIQS update frequency hardwired to 2.0 per hour
    l->connections *= (p->fmgs * 2.0);
    l->requests *= (p->fmgs * 2.0);
    l->data *= (p->fmgs * 2.0);
    NOBOUNDARY(b);

  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_NISS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_NISS";

  if (!p || !l || !b) {
    ret = 0;
  } else {

    /*
     * This is the same value as for MAIP to NISS (since it is almost the
     * same system)
     */

    ret = load_MAIP_to_NISS(p,l,b);
    l->title = func;
    l->traffic = ClearTraffic;
  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_KQD (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_KQD";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;
    l->requests = (p->avg_mail_sent_per_client * p->freedom_active_clients)
                  + (p->avg_mail_rcvd_per_nym * p->freedom_subscribers);
    l->connections = l->requests;
    l->data = l->requests * (p->signature_size + p->pub_key_size + p->pub_key_request_size);
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_MAIP_to_Internet (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_MAIP_to_Internet";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = InternetTraffic;

  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_Internet (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_Internet";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = InternetTraffic;

  }
  return ret;
}

/*
 *
 */

int 
load_Internet_to_FMG (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_Internet_to_FMG";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = InternetTraffic;
    
#warning "Internet to FMG load uses number of subscribers, not nyms"
    l->connections = p->avg_mail_rcvd_per_nym * p->freedom_subscribers;
    l->requests = l->connections * 4;
    l->data = l->connections * p->avg_mail_message_size;
    NOBOUNDARY(b);

  }
  return ret;
}

/*
 *
 */

int 
load_CLIENT_to_AIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CLIENT_to_AIP";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = CloudTraffic;

  }
  return ret;
}

/*
 *
 */

int 
load_CLIENT_to_KQD (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CLIENT_to_KQD";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = CloudTraffic;
#warning hard coded client NIQS update frequency
    l->requests = p->freedom_active_clients * 
                  (((p->core_servers + p->aips + p->maips) * 
                         p->percentage_entity_change * 1.0)
                   + p->avg_mail_sent_per_client + p->avg_news_sent_per_client);
    l->connections = l->requests;
    l->data = (p->pub_key_size + p->signature_size + p->pub_key_request_size)
               * l->requests;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_CLIENT_to_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CLIENT_to_NIQS";

  if (!p || !l || !b) {
    ret = 0;
  } else {
#warning FREEDOM client NIQS update frequency hardwired to 1/hour
    ret = load_frnetinfo_to_NIQS(p,l,b);
    l->title = func;
    l->traffic = CloudTraffic;
    l->connections *= (p->freedom_active_clients * 1.0 );
    l->requests *= (p->freedom_active_clients * 1.0 );
    l->data *= (p->freedom_active_clients * 1.0 );
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_FMG_to_NYMSRV (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMG_to_NYMSRV";

  if (!p || !l || !b) {
    ret = 0;
  } else {

    /*
     * Another worst case here, I'll assume that all mail messages sent
     * to a nym actually come from the internet.  This assumption will only
     * affect the FMG->NYMSRV numbers.
     */
 
    l->title = func;
    l->traffic = ClearTraffic;
    l->connections = p->avg_mail_rcvd_per_nym * p->freedom_subscribers;
    l->requests = l->connections;
    l->data = l->connections * p->avg_nym_size;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP_to_NYMSRV (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_FMGMAIP_to_NYMSRV";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = ClearTraffic;
    l->connections= p->avg_mail_rcvd_per_nym * p->freedom_subscribers;
    l->requests = l->connections;
    l->data = l->connections * p->avg_nym_size;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int
load_CLIENT_to_FMGMAIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CLIENT_to_FMGMAIP";
  
  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = CloudTraffic;
    l->connections = (p->avg_mail_sent_per_client + p->avg_news_sent_per_client) * p->freedom_active_clients;
    l->requests = l->connections * 4;
    l->data = l->connections * p->avg_mail_message_size;
    NOBOUNDARY(b);
  }

  return ret;
}


/*
 *
 */

int 
load_CLIENT_to_NYMSRV (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_CLIENT_to_NYMSRV";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = CloudTraffic;
#warning "All nym creation and modification parameters are hardcoded"
    l->connections = p->freedom_active_clients * 0.01 * 2.0 +  /* Create */
                     p->freedom_active_clients * 0.01 +  /* modify */
                     p->freedom_active_clients * 0.02; /* rbactivates */
    l->requests = l->connections;   
    l->data = (p->freedom_active_clients * 0.01 * 2.0 +  /* Create */
              p->freedom_active_clients * 0.01) * p->avg_nym_size + /* modify */
              p->freedom_active_clients * 0.02 * 250; /* rbactivates */
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_NYMSRV_to_FMGMAIP (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_NYMSRV_to_FMGMAIP";

  if (!p || !l || !b) {
    ret = 0;
  } else {

    /*
     * This happens at least once for each reply block added to the system. 
     * This value is based on the client's nym create and modify parameters
     * Currently, modify commands are only issued to add or delete reply
     * blocks.  We'll assume a worst case here that people only add new
     * reply blocks. 
     */

#warning "All nym creation and modification parameters are hardcoded"
    l->title = func;
    l->traffic = ClearTraffic;
    l->connections = p->avg_replyblocks_per_nym * (p->freedom_active_clients * 0.01) + /* Initial set of reply blocks */
                     p->freedom_active_clients * 0.01; /* nym modifies */
    /*
     * Note that since the rb activation token is sent as a mail message, I'll
     * use the average mail message size here.  When we look at the system, it
     * is not possible for us to tell these apart from regular messages so the
     * average mail message size that we see includes these tokens as well.
     */

    l->requests = l->connections * 4;
    l->data = l->connections * p->avg_mail_message_size;
    NOBOUNDARY(b);
  }
  return ret;
}

/*
 *
 */

int 
load_frpostnews_to_Internet (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  int ret = 1;
  const char * func = "load_frpostnews_to_Internet";

  if (!p || !l || !b) {
    ret = 0;
  } else {
    l->title = func;
    l->traffic = InternetTraffic;

  }
  return ret;
}


/*
 * This is the complete load on the nym server, from both the
 * clients and the mail system
 */

int 
load_NYMSRV  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_NYMSRV";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;


    load_FMGMAIP_to_NYMSRV(p,&load,b);
    loadAdd(l,&load);
    load_FMG_to_NYMSRV(p,&load,b);
    loadAdd(l,&load);
    load_CLIENT_to_NYMSRV(p,&load,b);
    loadAdd(l,&load);
  }
  
  return ret;
}

/*
 * This is the load on the nym server only from the client.
 */

int
load_clientNYMSRV (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_NYMSRV";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;


    load_CLIENT_to_NYMSRV(p,&load,b);
    loadAdd(l,&load);
  }
  
  return ret;
}

  

/*
 *
 */

int 
load_avgMAIP  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_avgMAIP";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    load_MAIP_to_MAIP(p,&load,b);
    loadAdd(l,&load);
    load_FMGMAIP_to_MAIP(p,&load,b);
    loadAdd(l,&load);

    l->requests /= p->maips;
    l->connections /= p->maips;
    l->data /= p->maips;
  }
  
  return ret;
}

/*
 *
 */

int 
load_MAIP  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_MAIP";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    load_MAIP_to_MAIP(p,&load,b);
    loadAdd(l,&load);
    load_FMGMAIP_to_MAIP(p,&load,b);
    loadAdd(l,&load);
  }
  
  return ret;
}

/*
 *
 */

int 
load_FMGMAIP  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_FMGMAIP";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    /* hmm */
    load_Internet_to_FMG(p,&load,b);
    loadAdd(l,&load);
    load_NYMSRV_to_FMGMAIP(p,&load,b);
    loadAdd(l,&load);
    load_CLIENT_to_FMGMAIP(p,&load,b);
    loadAdd(l,&load);
  }
  
  return ret;
}

/*
 *
 */

int 
load_NISS  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_NISS";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    load_WH_to_NISS(p,&load,b);
    loadAdd(l,&load);
    load_AIP_to_NISS(p,&load,b);
    loadAdd(l,&load);
    load_MAIP_to_NISS(p,&load,b);
    loadAdd(l,&load);
    load_FMGMAIP_to_NISS(p,&load,b);
    loadAdd(l,&load);
  }
  
  return ret;
}

/*
 *
 */

int 
load_KQD  (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_KQD";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    load_AIP_to_KQD(p,&load,b);
    loadAdd(l,&load);
    load_MAIP_to_KQD(p,&load,b);
    loadAdd(l,&load);
    load_FMGMAIP_to_KQD(p,&load,b);
    loadAdd(l,&load);
    load_CLIENT_to_KQD(p,&load,b);
    loadAdd(l,&load);

  }
  
  return ret;
}

/*
 *
 */

int
load_NIQS (
  FreedomParameters * p ,
  Load * l ,
  Load * b )
{
  Load load;
  int ret = 1;
  const char * func = "load_NIQS";

  if (!p || !b || !l) {

  } else {
    l->title = func;
    l->requests = 0.0;
    l->connections = 0.0;
    l->data = 0.0;

    load_CLIENT_to_NIQS(p,&load,b);
    loadAdd(l,&load);
    load_AIP_to_NIQS(p,&load,b);
    loadAdd(l,&load);
    load_MAIP_to_NIQS(p,&load,b);
    loadAdd(l,&load);
    load_FMGMAIP_to_NIQS(p,&load,b);
    loadAdd(l,&load);
    load_CORE_to_NIQS(p,&load,b);
    loadAdd(l,&load);
  }

  return ret;
}  
