#include <stdio.h>
#include <stdlib.h>
#include "parameters.h"
#include "load.h"


/* This is for an avg of 400 aips in a mixture of 1.54Mb/sec and 2 Mb/sec
   topology for handling 250000 .. i.e. 31.25 sim. connections / sec */
#define ACTIVE_NYM_PER_AIPS ((250000.0/60.0/400.0*3.0)/3.0)

int NIQSServer(double from , double to ,  double step);
int KeyQueryServer ( double from , double to , double step);
int NymServer ( double from , double to , double step);
int FmgMaip ( double from , double to , double step);
int NISSServer ( double from , double to , double step);
int MaipServer ( double from , double to , double step);


int NIQSServer(
  double from ,
  double to ,
  double step)
{
  const char * testname = "NIQS";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;

  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.aips = 0.0;
  p.nym_servers = 1.0;
  p.maips = 0.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */

  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);
  freedomParametersDump(stdout,&p);
  loadDump2(stdout,&p,NULL);

  for (loop = from; 
       loop < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;

    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_CLIENT_to_NIQS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_AIP_to_NIQS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_MAIP_to_NIQS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_FMGMAIP_to_NIQS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_CORE_to_NIQS(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}



/*
 *
 */

int
KeyQueryServer (
  double from ,
  double to ,
  double step)
{
  const char * testname = "KQD";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;

  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.aips = 0.0;
  p.nym_servers = 1.0;
  p.maips = 0.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */

  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);

  freedomParametersDump(stdout,&p);
  loadDump2(stdout,&p,NULL);

  for (loop  = from; 
       loop  < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;
    p.freedom_subscribers = p.freedom_active_clients * p.freedom_active_nym_per_subscriber;
    
    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_AIP_to_KQD(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_MAIP_to_KQD(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_FMGMAIP_to_KQD(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_CLIENT_to_KQD(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}

/*
 *
 */

int
NISSServer (
  double from ,
  double to ,
  double step)
{
  const char * testname = "NISS";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;

  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.nym_servers = 1.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */
  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);

  freedomParametersDump(stdout,&p);

  loadDump2(stdout,&p,NULL);

  for (loop = from; 
       loop < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;

    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_WH_to_NISS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_AIP_to_NISS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_MAIP_to_NISS(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_FMGMAIP_to_NISS(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}
  
/*
 *
 */

int
FmgMaip (
  double from ,
  double to ,
  double step)
{
  const char * testname = "FMG_MAIP";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;


  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.aips = 50.0;
  p.nym_servers = 1.0;
  p.maips = 50.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */
  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);

  freedomParametersDump(stdout,&p);
  loadDump2(stdout,&p,NULL);

  for (loop = from; 
       loop < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;

    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_NYMSRV_to_FMGMAIP(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_CLIENT_to_FMGMAIP(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}
  
/*
 *
 */

int
MaipServer (
  double from ,
  double to ,
  double step)
{
  const char * testname = "MAIP";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;


  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.aips = 50.0;
  p.nym_servers = 1.0;
  p.maips = 50.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */
  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);

  freedomParametersDump(stdout,&p);
  loadDump2(stdout,&p,NULL);

  for (loop = from; 
       loop < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;
    p.freedom_subscribers = p.freedom_active_clients * p.freedom_active_nym_per_subscriber;

    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    p.freedom_active_clients = p.freedom_subscribers / p.freedom_active_nym_per_subscriber;

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_MAIP_to_MAIP(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_FMGMAIP_to_MAIP(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Adjust the totals for the average MAIP
     */

    load_total.requests /= p.maips;
    load_total.connections /= p.maips;
    load_total.data /= p.maips;

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}

/*
 *
 */

int
NymServer (
  double from ,
  double to ,
  double step)
{
  const char * testname = "NymServer";
  FreedomParameters p;
  Load load;
  Load load_total;
  Load bound;
  unsigned int loop;

  freedomParametersInit(&p);
  freedomParametersSetConstants(&p);

  /*
   * Some initial assumptions about the shape of the network
   */

  p.aips = 50.0;
  p.nym_servers = 1.0;
  p.maips = 50.0;
  p.niqs_servers = 1.0;
  p.key_query_servers = 1.0;
  p.niss_servers = 1.0;
  p.token_servers = 1.0;
  p.fmgs = 1.0;

  freedomParametersCompute(&p);

  /*
   * Display our starting parameters
   */
  fprintf(stdout,"\n===============================================================================\n");
  fprintf(stdout,"%s Projection\n\n", testname);

  freedomParametersDump(stdout,&p);
  loadDump2(stdout,&p,NULL);

  for (loop = from; 
       loop < to; 
       loop += step) {

    p.maips = (double)loop;
    p.aips  = (double)loop;
    p.freedom_active_clients = p.aips * (double)ACTIVE_NYM_PER_AIPS;
    p.freedom_subscribers = p.freedom_active_clients * p.freedom_active_nym_per_subscriber;

    /*
     * Whenever you change one of the Freedom parameters, you must 
     * call freedomParametersCompute() to update the dependant variables
     */

    freedomParametersCompute(&p);

    /*
     * Initialize a load structure to keep a running total of the various
     * components we wish to compute.
     */

    load_total.requests = 0.0;
    load_total.connections = 0.0;
    load_total.data = 0.0;

    /*
     * Compute the contribution of every separate system to the NIQS load
     */

    load_FMGMAIP_to_NYMSRV(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_FMG_to_NYMSRV(&p,&load,&bound);
    loadAdd(&load_total,&load);
    load_CLIENT_to_NYMSRV(&p,&load,&bound);
    loadAdd(&load_total,&load);

    /*
     * Print the results
     */

    loadDump2(stdout,&p,&load_total);
  }

  return 1;
}
  
/*
 *
 */

int
main (
  int argc ,
  char ** argv )
{
  double min_aip   = 25;
  double max_aip   = 3000;
  double increment = 25;

  (void)argc;
  (void)argv;

  NIQSServer(min_aip, max_aip, increment);
  KeyQueryServer(min_aip, max_aip, increment);
  NymServer(min_aip, max_aip, increment);
  FmgMaip(min_aip, max_aip, increment);
  NISSServer(min_aip, max_aip, increment);
  //  MaipServer(min_aip, max_aip, increment);

  exit(EXIT_SUCCESS);
}
