#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "parameters.h"


/*
 *
 */

int
freedomParametersInit (
  FreedomParameters * p )
{
  int ret = 1;

  if (!p) {
    ret = 0;
  } else {
    memset(p,0,sizeof(FreedomParameters));
  }

  return ret;
}

/*
 *
 */

int
freedomParametersCompute (
  FreedomParameters * p )
{
  int ret = 1;

  if (!p) {
    ret = 0;
  } else {

    p->freedom_nym_min_throughput = 
      ( p->nym_min_throughput / p->freedom_packet_payload_size )
      * (p->freedom_packet_header_size + p->freedom_packet_size);

    p->freedom_nym_min_packet_throughput = 
      p->freedom_nym_min_throughput / 
      (p->freedom_packet_header_size + p->freedom_packet_size);

    p->ts_throughput = p->ts_packet_throughput * p->freedom_packet_size;

    p->core_servers = p->nym_servers + p->niqs_servers + p->key_query_servers
                      + p->niss_servers + p->token_servers + p->fmgs;
    
#warning "The average nym size hard codes the reply block size"
    p->avg_nym_size = p->avg_replyblocks_per_nym * 1210 +
                      p->pub_key_size * 2 + p->signature_size
                      + 500; /* Overhead */
    p->freedom_subscribers = p->freedom_active_nym_per_subscriber *
                             p->freedom_active_clients;
  }

  return ret;
}

/*
 *
 */

int 
freedomParametersSetConstants (
  FreedomParameters * p )
{ 
  int ret = 1;
  
  if (!p) {
    ret = 0;
  } else {
    p->freedom_active_nym_per_subscriber = FREEDOM_ACTIVE_NYM_PER_SUBSCRIBER;
    p->freedom_packet_payload_size = FREEDOM_PACKET_PAYLOAD_SIZE;
    p->freedom_packet_size = FREEDOM_PACKET_SIZE;
    p->freedom_packet_header_size = FREEDOM_PACKET_HEADER_SIZE;
    p->freedom_route_create_packet_size = FREEDOM_ROUTE_CREATE_PACKET_SIZE;
    p->freedom_route_create_packet_header_size = FREEDOM_ROUTE_CREATE_PACKET_HEADER_SIZE;
    p->aip_stat_request_size = AIP_STAT_REQUEST_SIZE;
    p->aip_stat_report_size = AIP_STAT_REPORT_SIZE;
    p->wormhole_stat_report_size = WORMHOLE_STAT_REPORT_SIZE;
    p->maip_stat_report_size = MAIP_STAT_REPORT_SIZE;
    p->ts_packet_throughput = TS_PACKET_THROUGHPUT;
    p->ip_header_size = IP_HEADER_SIZE;
    p->avg_aip_neighbours = AVG_AIP_NEIGHBOURS;
    p->avg_aip_restart = AVG_AIP_RESTART;
    p->avg_aip_force_update = AVG_AIP_FORCE_UPDATE;
    p->avg_aip_force_stats = AVG_AIP_FORCE_STATS;
    p->avg_nym_route_create = AVG_NYM_ROUTE_CREATES;
    p->pub_key_size = PUB_KEY_SIZE; 
    p->signature_size = SIGNATURE_SIZE;
    p->avg_mail_sent_per_client = AVG_MAIL_SENT_PER_CLIENT;
    p->avg_news_sent_per_client = AVG_NEWS_SENT_PER_CLIENT;
    p->avg_mail_rcvd_per_nym = AVG_MAIL_RCVD_PER_NYM;
    p->percentage_mail_to_nym = PERCENTAGE_MAIL_TO_NYM;
    p->avg_replyblocks_per_nym = AVG_REPLYBLOCKS_PER_NYM;
    p->avg_mail_message_size = AVG_MAIL_MESSAGE_SIZE;
    p->avg_niqs_response_size = AVG_NIQS_RESPONSE_SIZE;
    p->avg_niqs_accessctrl_size = AVG_NIQS_ACCESSCTRL_SIZE;
    p->percentage_entity_change = PERCENTAGE_ENTITY_CHANGE;
    p->maip_spool_interval = MAIP_SPOOL_INTERVAL;
    p->maip_msgs_per_interval = MAIP_MSGS_PER_INTERVAL;
    p->percentage_full_niqs_update = PERCENTAGE_FULL_NIQS_UPDATE;
    p->pub_key_request_size = PUB_KEY_REQUEST_SIZE;
    p->avg_mail_latency = AVG_MAIL_LATENCY;
    p->nym_min_throughput = NYM_MIN_THROUGHPUT;
  }

  return ret;
}

/*
 *
 */

int
freedomParametersDump (
  FILE * stream ,
  FreedomParameters * p )
{
  int ret = 1;

  if (!p || !stream) {
    ret = 0;
  } else {
    fprintf(stream, "freedom_active_nym_per_subscriber = %f\n"
                    "freedom_subscribers = %f\n"
                    "freedom_active_clients = %f\n"
                    "aips = %f\n"
                    "maips = %f\n"
                    "nym_servers = %f\n"
                    "niqs_servers = %f\n"
                    "key_query_servers = %f\n"
                    "niss_servers = %f\n"
                    "token_servers = %f\n"
                    "fmgs = %f\n"
                    "core_servers = %f\n"
                    "route_distribution_compensation = %f\n"
                    "freedom_network_throughput = %f\n"
                    "nym_min_throughput = %f\n"
                    "one_hop_route = %f\n"
                    "two_hop_route = %f\n"
                    "three_hop_route = %f\n"
                    "wormhole_input_max_throughput = %f\n"
                    "freedom_packet_payload_size = %f\n"
                    "freedom_packet_size = %f\n"
                    "freedom_packet_header_size = %f\n"
                    "freedom_route_create_packet_size = %f\n"
                    "freedom_route_create_packet_header_size = %f\n"
                    "aip_stat_request_size = %f\n"
                    "aip_stat_report_size = %f\n"
                    "wormhole_stat_report_size = %f\n"
                    "maip_stat_report_size = %f\n"
                    "ts_packet_throughput = %f\n"
                    "ip_header_size = %f\n"
                    "avg_aip_neighbours = %f\n"
                    "avg_aip_restart = %f\n"
                    "avg_aip_force_update = %f\n"
                    "avg_aip_force_stats = %f\n"
                    "avg_nym_route_create = %f\n"
                    "pub_key_size = %f\n"
                    "pub_key_request_size = %f\n"
                    "signature_size = %f\n"
                    "avg_mail_sent_per_client = %f\n"
                    "avg_news_sent_per_client = %f\n"
                    "avg_mail_rcvd_per_nym = %f\n"
                    "percentage_mail_to_nym = %f\n"
                    "avg_nym_size = %f\n"
                    "avg_replyblocks_per_nym = %f\n"
                    "avg_mail_message_size = %f\n"
                    "avg_mail_latency = %f (seconds)\n"
                    "avg_niqs_response_size = %f\n"
                    "avg_niqs_accessctrl_size = %f\n"
                    "percentage_entity_change  = %f\n"
                    "maip_spool_interval = %f\n"
                    "maip_msgs_per_interval = %f\n"
                    "freedom_nym_min_throughput = %f\n"
                    "freedom_nym_min_packet_throughput = %f\n"
                    "ts_throughput = %f\n"
                    "percentage_full_niqs_update = %f\n"
                    "effective_active_nym = %f\n",

                    p->freedom_active_nym_per_subscriber,
                    p->freedom_subscribers,
                    p->freedom_active_clients,
                    p->aips,
                    p->maips,
                    p->nym_servers,
                    p->niqs_servers,
                    p->key_query_servers,
                    p->niss_servers,
                    p->token_servers,
                    p->fmgs,
                    p->core_servers,
                    p->route_distribution_compensation,
                    p->freedom_network_throughput,
                    p->nym_min_throughput,
                    p->one_hop_route,
                    p->two_hop_route,
                    p->three_hop_route,
                    p->wormhole_input_max_throughput,
                    p->freedom_packet_payload_size,
                    p->freedom_packet_size,
                    p->freedom_packet_header_size,
                    p->freedom_route_create_packet_size,
                    p->freedom_route_create_packet_header_size,
                    p->aip_stat_request_size,
                    p->aip_stat_report_size,
                    p->wormhole_stat_report_size,
                    p->maip_stat_report_size,
                    p->ts_packet_throughput,
                    p->ip_header_size,
                    p->avg_aip_neighbours,
                    p->avg_aip_restart,
                    p->avg_aip_force_update,
                    p->avg_aip_force_stats,
                    p->avg_nym_route_create,
                    p->pub_key_size,
                    p->pub_key_request_size,
                    p->signature_size,
                    p->avg_mail_sent_per_client,
                    p->avg_news_sent_per_client,
                    p->avg_mail_rcvd_per_nym,
                    p->percentage_mail_to_nym,
                    p->avg_nym_size,
                    p->avg_replyblocks_per_nym,
                    p->avg_mail_message_size,
                    p->avg_mail_latency,
                    p->avg_niqs_response_size,
                    p->avg_niqs_accessctrl_size,
                    p->percentage_entity_change ,
                    p->maip_spool_interval,
                    p->maip_msgs_per_interval,
                    p->freedom_nym_min_throughput,
                    p->freedom_nym_min_packet_throughput,
                    p->ts_throughput,
                    p->percentage_full_niqs_update,
                    p->effective_active_nyms);
  }

  return ret;
}
