/*
 *
 */

#ifndef PARAMETERS_H_
#define PARAMETERS_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


struct FreedomParameters_T {

  /* 
   * Entities on the network
   */

  double freedom_active_nym_per_subscriber; /* Number of active nym per subscriber */
  double freedom_subscribers;        /* Total number of freedom users */
  double freedom_active_clients;     /* Number of active clients on the network */
  double aips;               /* Number of AIPs in the network */
  double maips;              /* Number of MAIPs in the network */
  double nym_servers;        /* Number of nym server nodes on the network */
  double niqs_servers;       /* Number of NIQS nodes on the network */
  double key_query_servers;  /* Number of key query server nodes on the network */
  double niss_servers;       /* Number of NISS server nodes on the network */
  double token_servers;      /* Number of token server nodes on the network */
  double fmgs;               /* Number of FMGs (fmg + fmg-maip) on the network */

  /*
   * Constants
   */

  
  double route_distribution_compensation;            /* percentage */
    /* 
     * This is the compensation percentage to convert the freedom_active_nyms 
     * into the effective_active_nyms.
     */ 
  
  double freedom_network_throughput;                    /* kB/s */ 
    /*
     * Effective throughput of the Freedom network.  It is the summation of 
     * all links that compose the network.  The assumption is that the network
     * is balanced.
     */


  double one_hop_route;                              /* percentage */
  double two_hop_route;                              /* percentage */  
  double three_hop_route;                           /* percentage */
  
  double wormhole_input_max_throughput;                 /* kB/s */
    /* 
     * Maximum number of kilobytes per second that a wormhole will accept. 
     * value can be retrieved from the freedom.conf file.  This model is
     * assuming that all wormholes have the same setting (this does not need
     * to be the case on the real system however)
     */


  /*
   * Real constants.  These can be set automatically by calling
   * freedomParameterSetConstants().  These values are wired into the 
   * Freedom network as it currently stands.
   */

  double nym_min_throughput;                            /* bytes/s */
    /* 
     * The minimum acceptable effective throughput for a user connection.  It
     * represents the real data only, it does not include packet header
     * overhead, etc.  This value must be adjusted to represent the effective
     * bandwidth use.  This value is based on a 56kbps modem.
     */

  double freedom_packet_payload_size;
    /*
     * The number of bytes of useful data which can be transported in a freedom
     * packet.  This does not count encryption overhead nor the original 
     * packet's IP/TCP/UDP headers.  In setting this value I'm assuming the
     * worst case that the Freedom packet is always transporting TCP data
     * which has the largest header size (min 20 bytes + options).  I could
     * also compute a weighted average size for TCP/UDP traffic and thus 
     * increase slightly the value of this parameter.
     */ 

  double freedom_packet_size;   
    /*
     * Size of a data packet on the freedom network, not counting its
     * IP and UDP headers (28 bytes).
     */

  double freedom_packet_header_size;
    /*
     * Size of the freedom packet header.  We need to know this since this
     * falls under the bandwidth we pay for.  This should always be 28 since
     * the Freedom packet is sent over IP/UDP (20 bytes for IP, 8 for UDP)
     */

  double freedom_route_create_packet_size;              /* Bytes */
    /*
     * This is the size of the route create packet without counting its
     * transport head (freedom_route_create_packet_header_size)
     */
  
  double freedom_route_create_packet_header_size;
    /*
     * This is the transport header for the route create packet.  Currently
     * 28 bytes, since it is just IP/UDP.
     */
  
  double aip_stat_request_size;                       /* Bytes */
    /*
     * This is the size of the packet sent by the NISS to an aip when
     * it wants a stat update (aip_stat_report_size) packet sent back.
     */

  double aip_stat_report_size;                       /* bytes */
    /*
     * Size of an AIP stats report packet to the NISS.
     */

  double wormhole_stat_report_size;
    /*
     * Size of the wormhole stat report packet to the NISS.
     */

  double maip_stat_report_size;
    /*
     * Size of the MAIP stat report packet to the NISS.
     */  

  double ts_packet_throughput;                       /* Packets/sec */ 
    /*
     * This is the throughput of the traffic shapper expressed 
     * in packets per second
     */

  double ip_header_size;                      /* bytes */
    /*
     * Size of an Internet IP header
     */

  double avg_aip_neighbours;                       /* int */
    /*
     * The average number of neighbours per AIP 
     */

  double avg_aip_restart;                      /* Percentage/hour */
    /*
     * The percentage of AIPs which are restarted every hour.
     */
  
  double avg_aip_force_update;              /* Percentage / hour */
    /*
     * Percentage of AIPs which receive a force update packet from
     * the NISS every hour.  This causes the AIP to make a new
     * set of queries to the NIQS.
     */
  
  double avg_aip_force_stats;               /* Percentage / hour */
    /*
     * Percentage of the AIPs which receive a force stats packet from
     * the NISS.  This forces the AIP to send back a stat packet.
     */

  double avg_nym_route_create;                   /* New routes/ hour */
    /*
     * This is the number of route create requests which are generated
     * per nym per hour.
     */

  double pub_key_size;                                  /* bytes */
    /*
     * This is the size of a public key.
     */

  double signature_size;                                   /* Bytes */
    /*
     * This is the size of a signature block
     */
  
  double avg_mail_sent_per_client;                      /* Mails/hour */  

  double avg_news_sent_per_client;                       /* posts/hour */

  double avg_mail_rcvd_per_nym;                      /* Mails/hour */
    /*
     * This is the number of mail messages received per nym in the system
     * per hour.  Note that a nym does not have to be active in order to
     * receive mail.
     */

  double percentage_mail_to_nym;                     /* percentage */
    /* 
     * Of all the mail sent from Freedom clients, this is the percentage
     * which is sent to nyms.
     */


  double avg_replyblocks_per_nym;                   /* rb / nym */
   
  double avg_mail_message_size;                   /* bytes */

  double avg_mail_latency;                             /* seconds */
    /*
     * The average number of seconds which a message will spend inside
     * a particular MAIP's outgoing queue before being sent out.
     */

  double avg_niqs_response_size;                          /* bytes */
    /*
     * Averge size of an NIQS request response 
     */

  double pub_key_request_size;                   /* Bytes */
    /*
     * Size of a public key request sent to the key query server
     */

  double avg_niqs_accessctrl_size;            /* Bytes */
    /*
     * Size of the access control list response from the NIQS.  This is only
     * used by the AIPs.
     */

  double percentage_entity_change;        /* percentage / hour */
   /*
    * Percentage of network entities which change over one hour 
    */

  double maip_spool_interval;                     /* seconds */
    /*
     * The number of seconds between bursts where a maip sends out 
     * messages. This is a configurable parameter in freedom.conf
     */
 
  double maip_msgs_per_interval;                 /* messages */
   /*
    * Number of mails which a maip will send out at each interval.  This
    * is a configurable parameter in freedom.conf
    */

  double percentage_full_niqs_update;             /* percentage */
    /* 
     * Percentage of all NIQS updates from frnetinfo which are
     * actuall full updates as opposed to partial ones
     */


  

  /*
   * Dependent values.  These parameters are dependendant on others.
   */

  double freedom_nym_min_throughput;                    /* bytes/s */
    /*
     * The actual amount of bandwith required to provide the nym_min_throughput
     */
  
  double freedom_nym_min_packet_throughput;            /* Packets/s */
    /* 
     * This is the freedom_nym_min_throughput measured in packets per second.
     */

  double ts_throughput;                                 /* bytes/sec */
    /* 
     * This is the effective throughput of the traffic shapper in bytes per
     * second.  This parameter is determined by ts_packet_throughput
     */

  double avg_nym_size;                             /* bytes */
    /*
     * This is the average size in bytes of a nym record in the nym server.
     * This depends on the average number of reply blocks and the sizes of
     * the public keys.
     */

  double effective_active_nyms;

  double core_servers;
    /*
     * Number of core servers in the system
     */
};

typedef struct FreedomParameters_T FreedomParameters;

/*
 * Constant values
 * Most of these values are currently guesstimates but can be pulled out
 * of the source code and the configuration files.
 */

#define IP_HEADER_SIZE                          20.0
#define UDP_HEADER_SIZE                          8.0
#define FREEDOM_PACKET_PAYLOAD_SIZE            250.0
#define FREEDOM_PACKET_SIZE                    297.0
#define FREEDOM_PACKET_HEADER_SIZE             (IP_HEADER_SIZE + UDP_HEADER_SIZE) 
#define FREEDOM_ROUTE_CREATE_PACKET_SIZE      1251.0
#define FREEDOM_ROUTE_CREATE_PACKET_HEADER_SIZE 28.0
#define AIP_STAT_REQUEST_SIZE                 ((double)FREEDOM_PACKET_SIZE)
#define AIP_STAT_REPORT_SIZE                   221.0 /* Bytes */
#define WORMHOLE_STAT_REPORT_SIZE              209.0 /* Bytes */
#define MAIP_STAT_REPORT_SIZE                  207.0 /* Bytes */
#define TS_PACKET_THROUGHPUT                   400.0 /* Packets/sec */
#define AVG_AIP_NEIGHBOURS                       3.0 /* Currently on our network 2000-01-14 */
                                                /* 1 / week / aip  == 0.006 */
#define AVG_AIP_RESTART                         0.006   /* Guess */ /*NOTUSED*/
#define AVG_AIP_FORCE_UPDATE                    0.006   /* Guess */
#define AVG_NYM_ROUTE_CREATES   60.0/(60.*0.50 + 30.0*0.50) /* Guess */ /* ok */
#define AVG_AIP_FORCE_STATS                     0.006   /* Guess */ 
#define PUB_KEY_SIZE                           488.0    /* Guess */ /* ok */
#define SIGNATURE_SIZE                         136.0
#define AVG_MAIL_SENT_PER_CLIENT                 0.417  /* 10 email sent per user per day  (in msgs/hour)*/
#define AVG_NEWS_SENT_PER_CLIENT                 0.208  /* 5 posts per user per day (in msgs/hour)*/
#define AVG_MAIL_RCVD_PER_NYM                    0.625  /* 15 mails received per nym per day (in msgs/hour)*/
#define PERCENTAGE_MAIL_TO_NYM                   0.5    
#define AVG_REPLYBLOCKS_PER_NYM                  3.0
#define AVG_MAIL_MESSAGE_SIZE                20000.0    /* Bytes */
#define AVG_NIQS_RESPONSE_SIZE                 210.0    /* Bytes */
#define AVG_NIQS_ACCESSCTRL_SIZE               430.0    /* Guess */
#define PERCENTAGE_ENTITY_CHANGE                 0.1    /* Guess */
#define MAIP_SPOOL_INTERVAL                     30.0    /* Config */
#define MAIP_MSGS_PER_INTERVAL                   5.0    /* Config */
#define PERCENTAGE_FULL_NIQS_UPDATE              0.05    /* guess */
#define PUB_KEY_REQUEST_SIZE                     100 /* bytes */
#define AVG_MAIL_LATENCY                         600 /* seconds , guess */
#define NYM_MIN_THROUGHPUT                      4000 /* Bytes  per second */
#define FREEDOM_ACTIVE_NYM_PER_SUBSCRIBER         60 /* guess */

/*
 * Parameter relationships
 *
 * active_nyms >= active_clients

 * one_hop_route + two_hop_route + three_hop_route = 1.0
 */


/*
 * Functions
 */


int freedomParametersSetConstants ( FreedomParameters * p );
int freedomParametersDump ( FILE * f , FreedomParameters * p );
int freedomParametersCompute ( FreedomParameters * p );
int freedomParametersInit ( FreedomParameters * p );




#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#endif /* ! PARAMETERS_H_ */

