#!/usr/bin/perl -w

# $Id: DNS.pm,v 1.2 2001/04/19 19:17:06 alainc Exp $


package DNS;

#use strict;

Logger::informational("using module DNS:");
#Morpheus::prompt_user();



# options
local $debug;



# patterns & constants
my $dns_cache_regexp = '^\W*[0-9]{8}[0-9]{2,}\W*;.*';
my $dns_niqs_regexp = '^\W*niqs[0-9]\W+IN\W+CNAME\W+';
my $dns_keyqry_regexp = '^\W*keyqrys[0-9]\W+IN\W+CNAME\W+';



# context & environment
#defined($context) || Logger::error("context not defined");
#my $network = $context->get_network();  # {'zeta'}
#my $network_name = $network->get_name();  # 'zeta'
#my $network_domain = $network->get_domain();  # 'zeta.freedom.net'

#my $host = $context->get_host();  # {'host1'}
#my $host_entity = $host->get_name();  # 'host1'
#my $host_name = $host->get_parameters()->{'hostname'};  # 'test001.syslab.freedom.net'

#my $dns = $network->get_parameters()->{'dns'};  # 'ns.syslab.freedom.net'
#my $dns_path = $network->get_parameters()->{'dns_path'};  # '/var/named/pri/'



# global variables
#my $dns_file = "$dns_path" . "$network_domain";


#my @dns_contents = ();
#my $dns_cache = '0000000000';
#my $dns_last_cache = '0000000000';



# $bool nslookup ($host, \$host_name, \$host_ip, \$host_name)
sub nslookup ($ $)
{
  my $host = shift;
  my $server = shift;
  my $server_ip;
  #my ($server, $server_ip);
  my ($host_name, $host_ip, $aliases) = ('', '', '', '');

  my (@lines) =  [];

  @lines = (`nslookup $host $server`);
  die ("nslookup failed: $host: $!") unless (@lines);

  print "\nnslookup:\n@lines\n" if ($debug);
  foreach my $line (@lines)
  {
    #chop;
    if ($line =~ /^\W*Server:\W*(\w+.*)/i)
    {
      $server = $1;
      print "lookup server: $server\n"  if ($debug);
    }
    elsif ($line =~ /^\W*Name:\W*(\w+.*)/i)
    {
      $host_name = $1;
      print "host_name: $host_name\n"  if ($debug);
    }
    elsif ($line =~ /^\W*Aliases:\W*(\w+.*)/i)
    {
      $aliases = join (" ", split (/[ \t]+/, $1));
      print "aliases: $aliases\n"  if ($debug);
    }
    elsif ($line =~ /^\W*Address:\W*(\w+.*)/i)
    {
      # if Name token defined, this is host ipaddress,
      if ($host_name)
      {
	$host_ip = $1;
	print "host_ip: $host_ip\n"  if ($debug);
      }
      # elsif only Server token defined, this is lookup server ip address     
      elsif ($server)
      {
	$server_ip = $1;
	print "server_ip: $server_ip\n"  if ($debug);
      }
    }
  }

#  if (! $host_name)
#  {
#    return;
#  }
#  elsif (! $host_ip)
#  {
#    return;
#  }
#  elsif (! $aliases)
#  {
#    return ($host_name, $host_ip, '');
#  }
#  else
#  {
#    return ($host_name, $host_ip, $aliases);
#  }
  return ($host_name, $host_ip, $aliases);
}


# dns_load ($dns_file)
sub dns_load ($)
{
  my $dns_file = shift;
  @dns_contents = ();

  open(DNS_FILE, "<$dns_file") || die("couldn't open dns_file for reading: $dns_file: $!");
  while (<DNS_FILE>)
  {
    push (@dns_contents, $_);    
  }
  close(DNS_FILE);
}


# dns_save ($dns_file)
sub dns_save ($)
{
  my $dns_file = shift;

  open(DNS_FILE, ">$dns_file") || die("couldn't open dns_file for writing: $dns_file: $!");
  foreach $_ (@dns_contents)
  {
    printf (DNS_FILE $_);
  }
  close(DNS_FILE);
}


# dns_backup ($dns_file)
sub dns_backup ($)
{
  my $dns_file = shift;

  &dns_save ($dns_file . ".bak_morpheus_");
}


# dns_restore ($dns_file)
sub dns_restore ($)
{
  my $dns_file = shift;

  &dns_load ($dns_file . ".bak_morpheus_");
}


# dns_print ()
sub dns_print ()
{

  foreach $_ (@dns_contents)
  {
    print "$_";
  }
}



# $dns_new_cache dns_increment_cache ($dns_last_cache)
sub dns_increment_cache ()
{
  # if last cache specified
  # my $dns_last_cache = shift;

  my ($date, $month, $day, $year, @now);
  my ($dns_cache, $dns_date, $dns_serial);
  my ($i, $loop) = (0, 1);

  @now = localtime(time);
  $month = $now[4];
  $month++;
  $year = $now[5];
  $year += 1900;
  $day = $now[3];
  $day++;

  $date = "$year" . "$month" . "$day";
  print "today: $date\n"  if ($debug);

  print "dns last cache: $dns_last_cache\n"  if ($debug);

  for ($i=0; $loop && $dns_contents[$i]; $i++)
  {

    # if found cache serial version
    if ( $dns_contents[$i] =~ /$dns_cache_regexp/ )
    {
      print "this cache ver: $dns_contents[$i]\n"  if ($debug);

      $dns_contents[$i] =~ /([0-9]{10,})/; 
      ($dns_cache) = ($1);

      # if restoring from backup: last cache exists
      if ($dns_last_cache > $dns_cache)
      {
	print "last cache: $dns_last_cache\n"  if ($debug);
	$dns_cache = $dns_last_cache;
      }
      # otherwise record last cache for backup
      else
      {
	print "this cache: $dns_cache\n"  if ($debug);
	$dns_last_cache = $dns_cache;
      }

      $dns_cache =~ /([0-9]{8})([0-9]{2,})/;
      ($dns_date, $dns_serial) = ($1, $2);

      # if cache date is today, increment serial
      if ( $dns_date == $date )
      {
	$_ = ++$dns_serial;
	$dns_serial = sprintf ("%02d", $_);
	print "new serial : $dns_serial\n"  if ($debug);
      }
      # else set date to today and start at serial '01'
      else
      {
	$dns_date = $date;
	$dns_serial = '01';
      }


      # now update dns_cache in dns_contents aray      
      $dns_cache = "$dns_date$dns_serial";      
      $dns_contents[$i] =~  s/[0-9]{10,}/$dns_cache/;
      print "new cache ver : $dns_contents[$i]\n"  if ($debug);

      # break loop
      $loop = 0;
    }
  }

  $dns_last_cache = $dns_cache;
  return ($dns_cache);
}


sub dns_get_niqs ()
{
  my $dns_niqs = $context->get_network()->get_entities('niqs');
  my @dns_niqs = @{$dns_niqs};
  foreach my $niqs (@dns_niqs)
  {
    print "$niqs\n";
    my %niqs = %{$niqs};
    foreach my $attr (keys %niqs) {
      print "$attr $niqs{$attr}\n";
      print "Name:  ${niqs}->getName()\n";
    }
  }
  #my %dns_niqs = %{$dns_niqs};
}

sub dns_get_keyqry ()
{
  my $dns_keyqry = $context->get_network()->get_entities('keyqry');
  my @dns_keyqry = @{$dns_keyqry};
  foreach my $keyqry (@dns_keyqry)
  {
    print "$keyqry\n";
    my %keyqry = %{$keyqry};
    foreach my $attr (keys %keyqry) {
      print "$attr $keyqry{$attr}\n";
    }
  }
  #my %dns_keyqry = %{$dns_keyqry};
}


# dns_remove_niqs_names ()
sub dns_remove_niqs_names
{

  my (@tmp_dns) = ();
  my ($i);

  for ($i=0; $dns_contents[$i]; $i++) {

    # if found niqs[n] CNAME
    if ( $dns_contents[$i] =~ /$dns_niqs_regexp/ )
    {
      print "old niqs: $dns_contents[$i]\n";
    }
    else
    {
      push (@tmp_dns, $dns_contents[$i]);
    }
  }

  (@dns_contents) = (@tmp_dns);
}


# dns_remove_keyqry_names ()
sub dns_remove_keyqry_names {

  my (@tmp_dns) = ();
  my ($i);

  for ($i=0; $dns_contents[$i]; $i++)
  {
    # if found niqs[n] CNAME
    if ( $dns_contents[$i] =~ /$dns_keyqry_regexp/ )
    {
      print "old keyqry : $dns_contents[$i]\n";
    }
    else
    {
      push (@tmp_dns, $dns_contents[$i]);
    }
  }

  (@dns_contents) = (@tmp_dns);
}


# dns_add_niqs_names (\%dns_niqs)
sub dns_add_niqs_names
{

  my %dns_niqs = ( 'niqs0', [ 'test002.freedom.net', '207.107.115.132' ],
		   'niqs1', [ 'test031.syslab.freedom.net', '207.107.115.42' ]
		 );

  my ($niqs_name);


  my $host = "niqs.beta";
  my ($name, $address, $aliases) = ('', '', '', []);
  ($name, $address, $aliases) = &nslookup($host);
  if (! $name)
  {
    die ("could not do nslookup for $host: $!");
  }
  print "NSLOOKUP: $name, $address, $aliases\n";


  # if multiple niqs, then set-up round-robin dns via dns aliases to host ip's
  if ( (keys (%dns_niqs)) > 1)
  {
    # format is:
    # 
    #   niqs       IN  A       207.107.115.132             ; _morpheus_
    #   niqs       IN  A       207.107.115.42              ; _morpheus_
    #   ...
    #   niqs0      IN  CNAME   test002.freedom.net.        ; _morpheus_
    #   niqs1      IN  CNAME   test031.syslab.freedom.net. ; _morpheus_

    push (@dns_contents, "\n");
    foreach $niqs_name (keys (%dns_niqs))
    {
      $_ = sprintf ( "%s \t\t%s \t%s \t%s \t; _morpheus_\n",
		     'niqs', 'IN', 'A', $dns_niqs{$niqs_name}[1]
		    );
      
      print ("new niqs: $_\n");
      push (@dns_contents, $_);
    }
    foreach $niqs_name (keys (%dns_niqs))
    {
      $_ = sprintf ( "%s \t\t%s \t%s \t%s. \t; _morpheus_\n",
		     $niqs_name, 'IN', 'CNAME', $dns_niqs{$niqs_name}[0]
		    );
      
      print ("new niqs: $_\n");
      push (@dns_contents, $_);
    }
  }
  # else if single niqs, then set-up a single CNAME to canonical hostname
  elsif ( (keys (%dns_niqs)) == 1 )
  {
    # format is:
    #
    #   niqs        IN  A       207.107.115.32             ; _morpheus_
    #   niqs0       IN  CNAME   test002.freedom.net.       ; _morpheus_

    push (@dns_contents, "\n");

    $niqs_name = (keys (%dns_niqs))[0];
    $_ = sprintf ( "%s \t\t%s \t%s \t%s \t; _morpheus_\n",
		   'niqs', 'IN', 'A', $dns_niqs{$niqs_name}[0]
		 );
    print ("new niqs: $_\n");
    push (@dns_contents, $_);

    $_ = sprintf ( "%s \t\t%s \t%s \t%s. \t; _morpheus_\n",
		   $niqs_name, 'IN', 'CNAME', $dns_niqs{$niqs_name}[1]
		 );  
    print ("new niqs: $_\n");
    push (@dns_contents, $_);
  }
  # else log error and die
  else
  {
    die("no dns_niqs were defined: $dns_file: $!");
  }

}


# dns_add_keyqry_names (\%dns_keyqry)
sub dns_add_keyqry_names {

  my %dns_keyqry = ( 'keyqrys0', [ 'test010.freedom.net', '207.107.115.162' ]
		);
  my ($keyqry_name);


  # canonical 'keyqry' is always first in list
  if ( (keys (%dns_keyqry)) > 0)
  {
    # format is:
    # 
    #   keyqry         IN  A       207.107.115.162            ; _morpheus_
    #   keyqry0        IN  CNAME   test010.freedom.net.       ; _morpheus_
    #   ...

    push (@dns_contents, "\n");

    $keyqry_name = (keys (%dns_keyqry))[0];
    $_ = sprintf ( "%s \t\t%s \t%s \t%s \t; _morpheus_\n",
		   'keyqry', 'IN', 'A', $dns_keyqry{$keyqry_name}[1]
		 );
      
    print ("new keyqry : $_\n");
    push (@dns_contents, $_);   

    foreach $keyqry_name (keys (%dns_keyqry)) {
      $_ = sprintf ( "%s \t\t%s \t%s \t%s. \t; _morpheus_\n",
		     $keyqry_name, 'IN', 'CNAME', $dns_keyqry{$keyqry_name}[0]
		    );
      
      print ("new keyqry : $_\n");
      push (@dns_contents, $_);
    }  
  }
  # else log error and die
  else
  {
    die("no dns_niqs were defined: $dns_file: $!");
  }

}


# dns_change ()
sub dns_change
{
  &dns_load($dns_file);
  &dns_backup($dns_file);
  &dns_increment_cache();

  &dns_remove_niqs_names();
  &dns_remove_keyqry_names();
  &dns_add_niqs_names();
  &dns_add_keyqry_names();

  &dns_print();
  #&dns_save ($dns_file . ".changed");
  &dns_save ($dns_file);
}


# dns_revert ()
sub dns_revert
{
  &dns_restore($dns_file);
  &dns_increment_cache();

  &dns_print();
  #&dns_save ($dns_file . ".reverted");
  &dns_save ($dns_file);
}



# main
# {
  #&dns_change ($dns_file);

  #&dns_revert ($dns_file);

# }
# main

1;
