#!/usr/bin/perl -w
#UpdateSince.cgi
#
#This script is served to callers by a web server daemon, hence the cgi.
#It is run by the freedom client when it starts up.  It is used to serve 
#two types of data to the freedom client:
#script information for form filling and add blocking information
#
# freedom client: via scriptserver.freedom.net:51140 http protocol:w

# command line: 
#	UpdateSince.cgi "zipIt=true&lastScriptUpdate=0&lastAdUpdate=0"
# browser line: 
#	scriptserver.freedom.net:port/UpdateSince.cgi?zipIt=true&lastScriptUpdate=0&lastAdUpdate=0"
#
# when executed it recieves 3 parameters:
# zipIt  {<any value = true>, false}
#		       [default:true, send clear text or bzip2'd]
# lastScriptUpdate     [default:return no scripts,
#			date of last script rec'd by client or 0]
# lastAdUpdate         [default:return no add strings, 
#			number of last add blocker line rec'd by client or 0]
# signIt               [default:false, sign the answer]
#
# nonce                [a ramdom 8 bytes data buffer (based 64)]
# 

use strict;
use CGI;
use FileHandle;

require fr_scriptServerLib;

# LOCKING CODE START  
# if tar file exists we're either in the middle of or
# want to update the scripts dir &/or Adblocker
# sleep until the update is done
#
my $transfn = "./scp-scripts.tgz";
my $LOK_SHARE = 1;
my $LOK_UNDO  = 8; 
my $lokfile = "./.lock";

my $dir = "/usr/freedom/privkey/ScriptServer/sig";
my $key = "1";
my $passdir = "/usr/freedom/etc";
my $sigCmd = "|/usr/freedom/scriptsign/fr_signfile --passInfo $passdir/passphrase-scriptserver.info --inputFile - --sigFile - --passfile $passdir/pass --signingKeyFile $dir/$key --symAlg blowfish";

my $ct = 0;
while ( -e $transfn ) { 
    sleep(5); 
    $ct++;
    die "UpdateSince.cgi, wait for lock too long" unless ($ct <= 15);
    }

# no update on going so shared lock on the lock file.
# if lockfile doesn't exist create it
if (! -e $lokfile) {
        # unable to check for error dies if it works or not
 	# the open will catch it if it's not created
 	system ( "/bin/touch $lokfile" ); 
   }

die "unable to open $lokfile" unless open LOKFILE, $lokfile;
die "unable to sh-lock $lokfile" unless flock LOKFILE, $LOK_SHARE;

my $DEBUG = 0;
if ($DEBUG eq 1)
  {
    open DEBUGOUT , ">>/tmp/UpdateSince.log";
    DEBUGOUT->autoflush(1);
	print DEBUGOUT "\n--UpdateSince Starting--\n";
  }

my $q = new CGI;

my $lastScriptUpdate = $q->param('lastScriptUpdate');
my $lastAdUpdate = $q->param('lastAdUpdate');
my $zipIt = $q->param('zipIt');
my $signIt = $q->param('signIt');
my $nonce = $q->param('nonce');

my $currentScriptDate = fr_getDateString();
my $lastDate = 0;

if ($DEBUG eq 1) {
	print DEBUGOUT "Got param lastScriptUpdate = $lastScriptUpdate\n";
	print DEBUGOUT "Got param lastAdUpdate = $lastAdUpdate\n";
	print DEBUGOUT "Current Update: $currentScriptDate\n"; 
	}

################################################################ 
#  Check if the nonce is specified along the signed request   #
################################################################
#if ((defined $signIt) and ($signIt ne "false")) {
#  if (! defined $nonce ) {
#    die("Missing nonce parameter.");
#  } 
#}

###############################################
## Now get the list of files and their dates ##
###############################################

my @scriptListArray = split "\n", `/bin/ls ./scripts/;`;
my %scriptListHash;

if ($DEBUG eq 1) { print DEBUGOUT "Script List: \n". join "\n", @scriptListArray; }

if (defined($lastScriptUpdate)) {
$lastDate = $lastScriptUpdate;
foreach (@scriptListArray)
  {
    my ($dateString, $scriptName) = ($_, $_);
    $dateString =~ s/^.*\.txt\.(\d+)$/$1/;
    $scriptName =~ s/^(.*)\.txt\.\d+$/$1/;
    $scriptListHash{$scriptName}=$dateString;
    if ( $lastDate < $dateString ) {
      $lastDate = $dateString;
    }
  }
$currentScriptDate = $lastDate;

if ($DEBUG eq 1)
  {
    print DEBUGOUT "\nScript Hash List:\n";
    foreach (keys %scriptListHash)
      {
	print DEBUGOUT "$_ : $scriptListHash{$_}\n";
      }
  }
}

####################################################
## Now get the list of ad rules and their indices ##
####################################################

open ADBLOCKER, "./AdBlocker.txt";

my %adBlockerHash;

if (defined($lastAdUpdate)) {
while (<ADBLOCKER>)
  {
    my ($indexString, $ruleString) = ($_, $_);
    $indexString =~ s/^(\d+)\t.*\n$/$1/;
    $ruleString =~ s/^\d+\t(.*)\n$/$1/;
    $adBlockerHash{$indexString}=$ruleString;
  }

close ADBLOCKER;

if ($DEBUG eq 1)
  {
    print DEBUGOUT "\nAd Hash List:\n";
    foreach (keys %adBlockerHash)
      {
	print DEBUGOUT "$_ : $adBlockerHash{$_}\n";
      }
  }
}

############################################
## Now time to construct the return value ##
############################################

my $clientRespString = "";
my $scriptsString ="";
my $addString ="";
my $currentAdUpdate;

if (defined $lastScriptUpdate)
  {
    
    $scriptsString .= "---ScriptUpdate---\n\n";
    foreach (keys %scriptListHash)
      {
      next if ($scriptListHash{$_} =~ /^CVS/);
	if ($scriptListHash{$_} >= $lastScriptUpdate)
	  {
	    $scriptsString .=  $_ . "\n\n";
	    open SCRIPTFILE, "./scripts/$_.txt.$scriptListHash{$_}";
	    while (<SCRIPTFILE>)
	      {
		$scriptsString .=  $_;
	      }
	    $scriptsString .= "\n";
	    close SCRIPTFILE;
	  }
      }

    $scriptsString .= "---EndScriptUpdate---\n\n";
  }

sub numerically { $a <=> $b;}
if (defined $lastAdUpdate)
  {

    $addString .= "---AdBlockUpdate---\n";
    
    foreach (sort numerically keys %adBlockerHash)
      {
	if ($_ > $lastAdUpdate)
	  {
	    $addString .= "$adBlockerHash{$_}\n";
	  }
	$currentAdUpdate = $_;
      }

    $addString .= "---EndAdBlockerUpdate---\n";

  }

if ($DEBUG eq 1) {print DEBUGOUT "\nReturn String:\n$clientRespString";}

####################
## Print the page ##
####################

if (defined  $clientRespString)
  {
    # disable buffering in stdout
    $| = 1;
       
    # set up client response data
    if (defined  $nonce and defined $signIt and $signIt == "true" )
	  {$clientRespString = "nonce=$nonce\n".$clientRespString;}
    if (defined  $lastScriptUpdate)
	  {$clientRespString = "thisScriptUpdate=$currentScriptDate\n".$clientRespString;}
    if (defined  $lastAdUpdate)
	  {$clientRespString = 
		"thisAdUpdate=$currentAdUpdate\n".$clientRespString;}

    $clientRespString = $clientRespString.$scriptsString.$addString;

    # default => zip'd zipIt==false => not zip'd
    if ((defined $zipIt) and ($zipIt eq "false"))
      {
	if ((defined $signIt) and ($signIt ne "false"))
	  {
	    print "Content-Type: text/compressed\n\n";
	    open(SIG, $sigCmd ) || die("Couldn't open compress: $!\n");	    
	    print $clientRespString;
	    print "\n---Signature---\n";
	    print SIG $clientRespString;
	    close(SIG);
	    print "\n---EndSignature---\n";

	  }
	else
	  {
	    print "Content-Type: text/plain\n\n";
	    print $clientRespString;
	  }

      }
    else
      {
	print "Content-Type: text/compressed\n\n";

	# default => not signed signIt==true => sign
	if ((defined $signIt) and ($signIt ne "false"))
	  {
	    open(BZIP, "|/usr/bin/bzip2 --compress --stdout" ) || die("Couldn't open compress: $!\n");	    
	    $clientRespString = $clientRespString;
	    print BZIP $clientRespString;
	    close(BZIP);
	    print "\n---Signature---\n";
	    open(SIG, "|/usr/bin/bzip2 --compress --stdout ".$sigCmd ) || die("Couldn't open compress: $!\n");	    
	    print SIG $clientRespString;
            close(SIG);
	    print "\n---EndSignature---\n";
	  }
	else
	  {
	    open(BZIP, "|/usr/bin/bzip2 --compress --stdout" ) || die("Couldn't open compress: $!\n");	    
	    print BZIP $clientRespString;
	    close(BZIP);
	  }


      }


  }

if ($DEBUG eq 1) {print DEBUGOUT "\n--UpdateSince Done--\n";}

flock LOKFILE, $LOK_UNDO;
close LOKFILE;
