#
# scriptserver.spec 
# 
# rpm spec file for scriptserver install & removal
# the scriptserver is the back end for the client
# another component scriptupdate is for csis to update the data
#

# nsa ~ production
%define prefix /usr/freedom
# me ~ test

%define name freedom-scriptserver
%define version v2.0
%define release B2
%define zerotag "#zeroknowledge-"
%define etcserv  "/etc/services"
%define httpdconf "/etc/httpd/conf/httpd.conf"

Summary: Zero-Knowledge Freedom ScriptServer(tm)
Name: %{name}
Version: %{version}
Release: %{release}
Prefix: %{prefix}
Copyright: commercial
Group: Networking/Daemons
Source: http://www.zeroknowledge.com/
NoSource: 0
#BuildRoot: $ZK_FREEDOM_PKG_ROOT
Vendor: Zero-Knowledge Systems Inc.
Packager: Zero-Knowledge Systems Inc. <server_support@admin.freedom.net>
URL: http://www.zeroknowledge.com/
AutoReqProv: no
Requires: redhat-release = 6.2, perl >= 5, bzip2 >= 0.9.5d


%description
For ZeroKnowledge Systems Inc. Internal Use only.
This rpm wll install or remove the ScriptServer application components
from a ZeroKnowledge internet connected server.  
It is not intended for external release.   There is intended to be only
one ScriptServer to which the Freedom(tm) client will connect.  The 
ScriptServer delivers both add blocking and form filling information
to the client.  The client contacts the Scriptserver every time it starts
up with a date so that the ScriptServer delivers a delta of the information 
using the date.


%changelog

* Mon Nov 20 2000 Zero-Knowledge Systems Inc. <server_support@admin.freedom.net>

- created.


%package ss
Summary: Freedom ScriptServer
Group: BackEnd
AutoReqProv: no
#Requires: %{name} = %{version}
%description ss
Freedom Internal ScriptServer
Do not distribute outside of Zero-Knowledge!

%pre
# ensure that we are running under the proper kernel version
unamesr="`uname -sr`"
case "${unamesr}" in
"Linux 2.2."*)
	;;
*)
	cat 1>&2 <<EOT

Your operating-system kernel (${unamesr}) is not supported
by this release of Freedom(tm).

Please install the Linux 2.2.x kernel
before using this package.

EOT
	exit 1
	;;
esac

#get ip to modify httpd.conf file & ensure we have a dns entry.

cat 1>&2 <<ETT

You are installing the scriptserver application, it relies on
apache web server.  The scriptserver is an apache virtual host
listening on port 51140.  To use this scriptserver a DNS
entry for scriptserver.freedom.net must be available.
Please enter the ip to which scriptserver.freedom.net is mapped. 
format 999.999.999.999

ETT

echo -n "Please enter the IP for the virtual server: " 

read ip </dev/tty

echo ""
echo -n "if $ip is valid then enter y otherwise the install will abort: "

read ans </dev/tty

if [ ${ans} = "y" ] ; then
   echo "Installation proceeding, using IP $ip for apache."
   echo ""
   else 
   echo "Installation aborted."
   exit 1
fi

echo ${ip} > /var/tmp/rpm.ip

# add the freedom group if it doesn't exist
if grep "^freedom:" /etc/group >/dev/null; then
	true
else
	if [ -x /usr/sbin/groupadd ]; then
		/usr/sbin/groupadd -r freedom
	else
		if [ ! -r /etc/group.rpmsave ]; then
			cp -p /etc/group /etc/group.rpmsave
		fi
		echo "freedom:x:300:" >>/etc/group
	fi
fi
# add the freedom user if it doesn't exist
if grep "^freedom:" /etc/passwd >/dev/null; then
	true
else
	if [ -x /usr/sbin/useradd ]; then
		/usr/sbin/useradd -r -c "Zero-Knowledge Freedom(tm) Server" -d /freedom -g freedom -M -n freedom
	else
		if [ ! -r /etc/passwd.rpmsave ]; then
			cp -p /etc/passwd /etc/passwd.rpmsave
		fi
		echo "freedom:*:300:300:Zero-Knowledge Freedom(tm) Server:/freedom:" >>/etc/passwd
	fi
fi

cd /

exit 0

##############################################################################
#
# Post all installations
#
##############################################################################

%post

#
# Add freedom port names to services file
#
# should have an %{etcserv} file
if [ ! -f %{etcserv} ]; then
	echo "%{etcserv} file... doesn't exist... no update"
else 	
#       if a pair of tags exists remove them & inbetween
#       odd number of tags means tampering
#        returns 0 if we have a clean file to update
	cnt=`grep -c %zerotag %{etcserv}`;
	if [ $cnt -eq 0 ]; then
#	   backup the sucker
           if [ ! -r %{etcserv}.rpmsave ]; then
	 	cp -p %{etcserv} %{etcserv}.rpmsave
	   fi
#          tack tag, freedom ports, tag on the end
	   echo "%{zerotag}%{name} %{version} %{release}" >> %{etcserv}
	   cat %{prefix}/etc/services >> %{etcserv}
	   echo "%{zerotag}%{name} %{version} %{release}" >> %{etcserv}
        fi
fi 


#
# Add freedom virtual server to %{httpdconf}
#


# should have an %{httpdconf} file
if [ ! -f %{httpdconf} ]; then
	echo "%{httpdconf} file... doesn't exist... UPDATE MANUALLY WITH"
	echo "\n /usr/freedom/scriptserver/scriptserver.conf"
else 	
#       if a pair of tags exists remove them & inbetween
#       odd number of tags means tampering
#        returns 0 if we have a clean file to update
	cnt=`grep -c %zerotag %{httpdconf}`;
	if [ $cnt -eq 0 ]; then
#	   backup the sucker
           if [ ! -r %{httpdconf}.rpmsave ]; then
	 	cp -p %{httpdconf} %{httpdconf}.rpmsave
	   fi
#          tack tag, freedom ports, tag on the end
	   echo "%{zerotag}%{name} %{version} %{release}" >> %{httpdconf}
	   cat %{prefix}/scriptserver/scriptserver.conf >> %{httpdconf}
	   echo "%{zerotag}%{name} %{version} %{release}" >> %{httpdconf}
#	   change the zinsertipz to a valid ip
           ipnum=`cat /var/tmp/rpm.ip`;
 	ed -s %{httpdconf} << ENDED >/dev/null
/zinsertipz
s/zinsertipz/${ipnum}/
/
s/zinsertipz/${ipnum}/
w
q
ENDED
        fi
fi 

chmod u+s %{prefix}/scriptsign/fr_signfile
 
echo "Looking for any files that should be examined for differences..."
find %{prefix} -name "*.rpmsave" -print
find /etc -name "*.rpmsave" -print
chown freedom /freedom/etc/*
echo "Done."

cat <<EOT

The ZeroKnowledge ScriptServer is now installed.
Reloading web server conf files: 
`/etc/rc.d/init.d/httpd reload`

EOT

exit 0

##############################################################################
#
# Pre uninstall all installations
#
##############################################################################

%preun

# End of pre uninstall all installations

%postun
#
# remove stuff we inserted 
#
# %{etcserv} first
# check for framing tags, if we don't have two then assume tampering
# or never changed
cnt=`grep -c %zerotag %{etcserv}`;
if [ $cnt -ne 2 ]; then 
   echo "No freedom ports removed: %{etcserv} has been tampered with, or we never added the ports."
else 
   ed -s %{etcserv}  << EOED >/dev/null;
/#zeroknowledge-/
ka
/
kb
'a,'bd
w
q
EOED
   echo "%{etcserv} file has been updated" 
fi

# %{httpdconf} next
# check for framing tags, if we don't have two then assume tampering
# or never changed
cnt=`grep -c %zerotag %{httpdconf}`;
if [ $cnt -ne 2 ]; then 
   echo "No freedom mods removed: %{httpdconf} has been tampered with, or we never added the ports."
else 
   ed -s %{httpdconf}  << EOED >/dev/null;
/#zeroknowledge-/
ka
/
kb
'a,'bd
w
q
EOED
   echo "%{httpdconf} file has been updated" 
fi

cat <<EOT

The ZeroKnowledge ScriptServer has been uninstalled.
Reloading web server conf files: 
`/etc/rc.d/init.d/httpd reload`

EOT


exit 0

# install  files

%files
%dir %attr(775, root, freedom) %{prefix}
%dir %attr(775, nobody, nobody) %{prefix}/scriptserver

%defattr(644, nobody, nobody) 
%{prefix}/scriptserver/.htaccess
%{prefix}/scriptserver/scriptserver.conf
%{prefix}/scriptserver/AdBlocker.txt
%{prefix}/scriptserver/ECMLMappings.txt
%{prefix}/scriptserver/Mozilla2ZkMappings.txt
%{prefix}/scriptserver/URLFieldSchema.tbl.txt

%defattr(755, nobody, nobody)
%{prefix}/scriptserver/UpdateSince.cgi
%{prefix}/scriptserver/fr_scriptServerLib.pm
%{prefix}/scriptserver/m2zk.pl
%{prefix}/scriptserver/scp-update.pl
%dir %{prefix}/scriptserver/scripts

%dir %{prefix}/scriptsign

%attr(755, freedom, freedom) %{prefix}/scriptsign/fr_signfile

#%defattr(700, freedom, freedom)

#%dir %{prefix}/privkey
#%dir %{prefix}/privkey/ScriptServer
#%dir %{prefix}/privkey/ScriptServer/sig

%defattr(600, freedom, freedom)

%{prefix}/privkey/ScriptServer/sig/1

%dir %attr(775, freedom, freedom) %{prefix}/etc
%{prefix}/etc/pass
%{prefix}/etc/services
%{prefix}/etc/passphrase-scriptserver.info
