#! /bin/sh -f

#$Id: core_BVT.sh,v 1.4 2001/04/19 19:17:54 alainc Exp $

#You need the following binaries compiled in your tree (pointed to by $ZK_FRTOOLBOX_ROOT):
#nymtool, laip, tkq, niqs_test, fr_listroutes, frpasschg, frlodec, fr_pokesrv 

#if cloud is specified (as cloud) then all server connections will be through unauthenticated routes 
#otherwise only the laip tests will be through the cloud

#The following test are performed:
#laip test: creates all routes and dowload a web page through that route (authenticated)
#         : DNS queries through all routes (unauthenticated)
#         : create new nyms  verifies that it can make authenticated routes
#nym server test: creates, upgrades, modifies, disables, deletes, nukes , restores nyms and does relevant checks on all of these operations (concerning lookups, key validity etc...
#kqd test: queries for all keys for all entities as well as bogus keys (for tkq error handling)
#the niqs is no longer tested explicitly, but all other tests depend on successful retrieval of netinfo stuff

if [ $# -lt 3 ] ; then
    echo "Usage: $0 <network> <nym_name> <routes | nyms | mail | keys | all> (cloud | coreaip)"
    exit;
fi

if [ "X$ZK_FRTOOLBOX_ROOT" = "X" ] ; then
    echo '$ZK_FRTOOLBOX_ROOT has not been set. Set this variable to the frtoolbox home dir (default = /usr/frtoolbox)'
    exit;
fi

network=$1
nym_name=$2
test=$3
cloud=$4

root_dir="/turf/$USER"
nympoplog="./nympop.log"

num_nyms=10


if [ "$test" = "routes" -o "$test" = "mail" -o "$test" = "all" -o "$cloud" = "cloud" -o "$cloud" = "coreaip" ] ; then
  laip --config $root_dir/.freedom/$network/$network.conf 
fi


#IT IS SAFER TO NOT AUTOMATICALLY RUN CONFIG FILE GENERATION EVERYTIME YOU RUN THE BVT.
#TO SET UP YOUR CONFIGURATION FILES, RUN netgenconfig.sh first (use /turf/$USER as your rootdir) 
#generate config file for network and make appropriate directories
#./netgenconfig.sh $network $network_domain $root_dir

if [ "$test" = "routes" -o "$test" = "all" ] ; then
  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/laip.exp $network laip_$nym_name  
fi

# I have commented out the mailpasswd checking, because checkpasswd (in frmailauth lib) looks for the mailmackey in /freedom/qmail/control.  I didn't want to automatically generate these directories and place the mailmackey there in case your machine is running a mail server on a test network.
# If you want to run the mailpassword checking, create a /freedom/qmail/control directory, containing dummy files mailowner, mailroot and tmpmailroot.  You will also need the correct mailmackey for each network you are running the bvt on, in the form mailmackey.$network.  Check the setup on colm.dev.zks.net for a sample setup.
 
if [ "$test" = "nyms" -o "$test" = "all" ] ; then
  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/nympop.exp $nym_name $num_nyms $network $nympoplog $cloud 
#  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/check_mailpass.exp $nym_name $num_nyms $nympoplog $network
#  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/check_mailpass.exp 501_$nym_name $num_nyms $nympoplog $network
fi



if [ "$test" = "keys" -o "$test" = "all" ] ; then
  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/key_queries.exp $network $cloud
fi

if [ "$test" = "mail" -o "$test" = "all" ] ; then
  $ZK_FRTOOLBOX_ROOT/tools/server-bvt/bin/mail.exp --network $network --nym_name $nym_name --user_email
fi


#kill laip process if still running
kill `cat /tmp/laip.pid`

#write summary
echo "BVT TEST SUMMARY:" > bvt_summary.txt	
echo "----------------" >> bvt_summary.txt	
if [ "$test" = "routes" -o "$test" = "all" ] ; then
        cat laip_summary.log >> bvt_summary.txt
 	echo "ERROR MESSAGES FROM LAIP TEST:" >> bvt_summary.txt
 	echo "-----------------------------" >> bvt_summary.txt
	grep "FAILED" routes.dump >> bvt_summary.txt
	grep "ERROR" routes.dump >> bvt_summary.txt
	grep "ERROR" laip_test.log >> bvt_summary.txt
fi
if [ "$test" = "nyms" -o "$test" = "all" ] ; then
        cat nym_summary.log >> bvt_summary.txt
 	echo "ERROR MESSAGES FROM NYM SERVER TEST:" >> bvt_summary.txt
	grep "FAILURE" $nympoplog >> bvt_summary.txt
	grep "ERROR" $nympoplog >> bvt_summary.txt
fi
if [ "$test" = "keys" -o "$test" = "all" ] ; then
 	echo "ERROR MESSAGES FROM KEY TEST:" >> bvt_summary.txt
	grep "invalid" key_queries.log >> bvt_summary.txt
fi
if [ "$test" = "mail" -o "$test" = "all" ] ; then
 	echo "ERROR MESSAGES FROM MAIL TEST:" >> bvt_summary.txt
	grep -i "Error" mail_test.log >> bvt_summary.txt
fi
