# freedomMail.pm
#************************************************************************
#
#   File:		freedomMail.pm
#
#   Author(s):		Louis Jacob
#
#   Created:		March 19th  2001
#
#   Last Revision:	$Date: 2001/04/19 19:17:54 $
#
#   $Revision: 1.2 $
#
#
#   $Id: freedomMail.pm,v 1.2 2001/04/19 19:17:54 alainc Exp $	
#
#   Copyright (c) 2001 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/
package freedomMail;

use strict;
use FileHandle;
use Net::Telnet;
use MIME::Base64;


#----------
# Global Variables
#----------

my $nmta="";
my $imep="";
my $nbMsg;
my $pop="";

my $line="";
my @lines; 
my $tmp;
my $debug = 1;



#--------
# pop_login
#
# logs into specified pop server and waits for password authentication.
#
# $_ pop_login ($nym, $password, $host, $port)
#--------
sub pop_login ($ $ $ $)
{
  my ($nym, $passwd, $host, $port) = (@_);

  ## Create the pop object
  if($debug){
    print "Creating POP connection\n";}

  $pop = new Net::Telnet(Telnetmode => 0);
  $pop->open(Host => $host,
	     Port => $port)|| return;

  if($debug){
    $line = $pop->getline();
    print "$line\n";}


  $line = $pop->print("USER $nym");

  if($debug){
    $line = $pop->getline();
    print "$line\n";}

  $line = $pop->print("PASS $passwd");
  sleep 1;

  if($debug){
    $line = $pop->getline();
    print "$line\n";}
}


#---------
# pop_stat
#
# retrieves stat info from pop server.
# (assumes user is already logged in)
#
# $pending pop_stat (@_)
#---------
sub pop_stat ()
{
  $line = $pop->print("STAT");

  $line = $pop->getline;
  @lines = split(" ",$line);

  if($debug){
    print "$lines[1] messages in POP box\n";}

  return $lines[1];
}


#--------
# pop_quit
#
# ends pop session
# (assumes already logged in to pop server)
#
# $_ pop_quit ($@_)
#--------
sub pop_quit ()
{
  $line = $pop->print("QUIT");

  if($debug){
    $line = $pop->getline();
    print "$line\n";}
}


#--------
# pop_retrieve
#
# reads a given message in a pop box 
# (must already be logged in)
#
# @lines pop_retrieve ($msg_index)
#--------
sub pop_retrieve ($)
{
  my ($msg_index) = (@_);

  $line = $pop->print("RETR $msg_index");

  if($debug){
    @lines = $pop->getlines;
    print "@lines\n";}

  return (@lines);
}


#--------
# pop_delete
#
# deletes a given message in a pop box
# (must already be logged in)
#
# $_ pop_delete ($msg_index) 
#--------
sub pop_delete ($)
{
  my ($msg_index) = (@_);

  $line = $pop->print("DELE $msg_index");

  if($debug){
    @lines = $pop->getlines;
    print "@lines\n";}
}


#--------
# nmta_login

# login to nmta and authenticate oneself
# returns 'failure' on failure
#
# $_ nmta_login ($nym, $password, $host, $port)
#--------
sub nmta_login ($ $ $ $)
{
  my ($nym, $passwd, $host, $port) = (@_);

  ## Create the nmta object
  if($debug){
    print "Creating NMTA connection\n";}

  $nmta = new Net::Telnet(Telnetmode => 0);
  $nmta->open(Host => $host,
	      Port => $port)|| return;

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

  $line = $nmta->print("AUTH LOGIN");

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

  $line = MIME::Base64::encode($nym);
  chomp $line;
  $tmp = $nmta->print($line);

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

  $line = $nmta->print($passwd);
  sleep 1;

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

}

  
#--------
# nmta_send
#
# sends message from nym to recpt
# (assumes already logged in to nmta)
#
# $_ nmta_send ($sender, $recipient, $message)
#--------
sub nmta_send ($ $ $)
{
  my ($sender, $recipient, $message) = (@_);

  $line = $nmta->print("MAIL FROM:<$sender>");

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}  

  $line = $nmta->print("RCPT TO:<$recipient>");

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

  $line = $nmta->print("DATA");

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}

  $line = $nmta->print($message);
  $line = $nmta->print('.');  

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}
}


#--------
# nmta_quit
#
# ends nmta session
# (assumes already logged in to nmta)
#
# $_ nmta_quit (@_)
#--------
sub nmta_quit ()
{
  $line = $nmta->print("QUIT");

  if($debug){
    $line = $nmta->getline();
    print "$line\n";}
}


#--------
# imep_login

# logs in to imep
#
# $_ imep_login ($host, $port)
#--------
sub imep_login ($ $)
{
  my ($host, $port) = (@_);

  ## Create the imep object
  if($debug){
    print "Creating IMEP connection\n";}

  $imep = new Net::Telnet(Telnetmode => 0);
  $imep->open(Host => $host,
	      Port => $port)|| return;

  if($debug){
    $line = $imep->getline();
    print "$line\n";}

  $line = $imep->print("HELO");

  if($debug){
    $line = $imep->getline();
    print "$line\n";}
}


#--------
# imep_send
#
# sends message from sender to recpt
# (assumes already logged into imep)
#
# $_ imep_send ($sender, $recipient, $message)
#--------
sub imep_send ($ $ $)
{
  my ($sender, $recipient, $message) = (@_);

  $line = $imep->print("MAIL FROM:<$sender>");

  if($debug){
    $line = $imep->getline();
    print "$line\n";}

  $line = $imep->print("RCPT TO:<$recipient>");  

  if($debug){
    $line = $imep->getline();
    print "$line\n";}

  $line = $imep->print("DATA");  

  if($debug){
    $line = $imep->getline();
    print "$line\n";}

  $line = $imep->print($message);
  $line = $imep->print('.');  

  if($debug){
    $line = $imep->getline();
    print "$line\n";}
}


#--------
# imep_quit
#
# ends imep session
# (assumes already logged in to imep)
#
# $_ imep_quit (@_)
#--------
sub imep_quit ()
{
  $line = $imep->print("QUIT");

  if($debug){
    $line = $imep->getline();
    print "$line\n";}
}



1;
