#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		generateTraffic.pl
#
#   Author(s):		Louis Jacob
#
#   Created:		Feb 19th 2001
#
#   Last Revision:	$Date: 2001/04/19 19:17:54 $
#
#   Description:	script to generate traffic on an aip.  Used to test
#                       performance.
#
#   Usage :             generateTraffic.pl --network   <network name>
#                                          --config    <config file for network>
#                                          --nym       <existing nym to be used by the laip>
#                                          --filepath  <valid URL that will be passed to wget>
#
#
#
#   $Revision: 1.2 $
#
#
#   $Id: generateTraffic.pl,v 1.2 2001/04/19 19:17:54 alainc Exp $	
#
#   Copyright (c) 2001 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/


use strict;
use Getopt::Long;
use FileHandle;
use Net::Telnet();



##
## Global Variables
##
my $version = '$Id: generateTraffic.pl,v 1.2 2001/04/19 19:17:54 alainc Exp $';
my $revision= '$Revision: 1.2 $';
my $progname= 'generateTraffic.pl';

##
## Static Variables
##

my $tel="";
my $line="";
my $list ="la";
my $starttime;
my $entityid;
my @entitylist;
my $nym;
my $filepath;
my $configpath;
my $network;


##
## autoflush mode
##
STDERR->autoflush(1);
STDOUT->autoflush(1);

##
## exit on error
##
sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}

##
## checkArguments
##
sub checkArguments {
  my $res;
  my %optctl  = ();
  my @helpString =
    (
     "\n\n",
     "Usage $progname --network <NETWORK> --config <CONFIGFILE> --nym <NYMNAME> --filepath <URL> \n\n",
     "Optional: \n",
     "\t--version       : version information",
     "\n"
    );
  
  $res = GetOptions(\%optctl,
                    "network=s",
                    "config=s",
                    "version",
                    "nym=s",
                    "filepath=s"
                   ) || do {
                         print  STDOUT
                           join("\n", @helpString),
                           "\n";
                         exit 1;
                         };

  if (defined($optctl{'network'})){
    $network = $optctl{'network'};
    $network = "niqs." . $network;
  }
  else{
    print STDOUT join("\n", @helpString);
    exit 1;
  }

  if (defined($optctl{'config'})){
    $configpath = $optctl{'config'};
  }
  else{
    print STDOUT join("\n", @helpString);
    exit 1;
  }

  if (defined($optctl{'nym'})){
    $nym = $optctl{'nym'};
  }
  else{
    print STDOUT join("\n", @helpString);
    exit 1;
  }

  if (defined($optctl{'filepath'})){
    $filepath = $optctl{'filepath'};
  }
  else{
    print STDOUT join("\n", @helpString);
    exit 1;
  }

  if (defined($optctl{'version'})){
    print  STDOUT "$progname: $version Rev $revision\n";
    exit 1;
  }


}


## 
## startLaip
##
sub startLaip {
  my $tmp;
  ## check if laip is already running
  $tmp = `ps aux | grep laip | grep -v grep`;
  if ($tmp){
    print "\n";
    print "Error : The Laip is currently running, you must close it before running generateTraffic\n";
    print "\n";
    exit 1;
  }
  ## Start Laip
  open(laipFH,"|laip --config $configpath");
  ##sleep for 1 second to let the laip finish initializing
  sleep 3;

}

## 
## openConnection
##
sub openConnection {
    ## Create the telnet object
    $tel = new Net::Telnet();

    ## Open The connection
    $tel->open(Port => 51116);
    $line = $tel->getline;
    print "$line\n";

    ## Open Logfile
    open(logFH, "> generateTraffic.log");
}


## 
## getEntityList
##
sub getEntityList {
    
  ##check if fr_getaips is in the path

  ## using fr_getaips and awk get the entity ID (and do not list core servers)
    @entitylist = ` fr_getaips --niqs.hostname $network --long --nodns | grep -v core | grep -v DOWN | awk '{ print \$1 }' `;
    
    shift @entitylist;
    ##print "ready";

}

## 
## startTraffic
##
sub startTraffic {
  
  ##Loop for each entity

  foreach $line (@entitylist){
    $starttime = time();
    ##Create Route
    
    ##set current entityid
    $entityid = $line;

    ##call to create the route
    &createRoute;

    ##Generate Traffic    
    print logFH "Generating traffic on entity : $entityid\n";
    &getStuff;

  }

  ##close telnet connection and close aip
  $line = $tel->print("exit");
}




##
## createRoute
##
sub createRoute {
    ## Create the route

    #destroy previous routes
    $line = $tel->print("da");
      ## $line = $tel->getline;
      ## print "$line\n";

    #clean up rules
    $line = $tel->print("nr");
      ## $line = $tel->getline;
      ## print "$line\n";

    ##set nym as specified by command line parameters
    $line = $tel->print("sn $nym");
      ## $line = $tel->getline;
      ## print "$line\n";

    ##block http port
    $line = $tel->print("bp 80");
      ## $line = $tel->getline;
      ## print "$line\n";

    ##make a 1 hop route
    $line = $tel->print("sh 1");
      ## $line = $tel->getline;
      ## print "$line\n";

    ##use entityid in route
    $line = $tel->print("uh $entityid");
      ## $line = $tel->getline;
      ## print "$line\n";

    ##create route
    $line = $tel->print("cr");
      ## $line = $tel->getline;
      ## print "$line\n"; 
      ## $line = $tel->getline;
      ## print "$line\n"; 
       
}

##
## getStuff
##
sub getStuff {
  my $tmp;
  while(1){
  ##call wget on a file
  system("wget -O temp -o log.traffic $filepath") == 0 or die "wget failed"; 
  ##delete file after download to prevent disk clustering
  system("rm -f temp");
  ##Statistics
  $tmp = `more log.traffic | grep saved`;
  sleep 2;
  print logFH "$tmp\n";
   
  ##last iteration if sufficient time has elapsed
  last if(time() - $starttime >= 300);
  }
}


&checkArguments;
&startLaip;
&openConnection;
&getEntityList;
&startTraffic;



