
if [ $# -lt 2 ] ; then
    echo "Usage: $0 <network> <network_domain (i.e. gamma.freedom.net)> [rootdir]"
    echo ""
    echo "Generates configuration for test network usage"
    echo ""
    echo "Parameters: - <network>: test network for which config is to be created"
    echo "            - <network.domain>: network domain (usually dev.zks.net for internal and freedom.net for external)"
    echo "            - [rootdir]: directory where config files are to be placed"
    echo "                         default is \$HOME"
    echo "            - Any additional parameters are ignored"
    echo ""
    echo "Output: - creates [rootdir]/.freedom/<network> directory"
    echo "        - creates [rootdir]/.freedom/<network>/keys directory"
    echo "        - sets up a passphrase.info file in <network> directory.  The"
    echo "          passphrase is set to 11111111"
    echo "        - generates <network>.conf configuration file in <network>"
    echo "          directory"
    echo "        - generates <network>_i.conf configuration file for each"
    echo "          nymserver in the network"
    exit;
fi

if [ "X$ZK_FRTOOLBOX_ROOT" = "X" ] ; then
    echo '$ZK_FRTOOLBOX_ROOT has not been set. Set his variable to the frtoolbox home dir (default = /usr/frtoolbox)'
    exit;
fi

# parse arguments
network=$1
network_domain=$2
homeDir=$HOME
if [ $# -ge 3 ] ; then
    homeDir=$3
fi

# global constants
pphrs="11111111"

# location of support files, should change as per user
scriptDir=../lib

if [ ! -d $homeDir/.freedom ] ; then
        mkdir -p $homeDir/.freedom
fi
if [ ! -d $homeDir/.freedom/$network ] ; then
        mkdir -p $homeDir/.freedom/$network
fi
if [ ! -d $homeDir/.freedom/$network/keys ] ; then
        mkdir -p $homeDir/.freedom/$network/keys
fi

if [ ! -d $homeDir/.freedom/$network/keys/laip_$network ] ; then
   #setup the new passphrase.info file
   if [ ! -f $homeDir/.freedom/$network/passphrase.info ] ; then
      $scriptDir/fr_passchg.exp $homeDir/.freedom/$network/passphrase.info $pphrs
   fi
fi

# generate config file for network
$scriptDir/__netgen_configure_.exp $network $network_domain $homeDir

