#!/usr/bin/perl -w

#************************************************************************
#
#   File:          nymcert.pl
#   Author:        Francis Korning francisk@zeroknowledge.com
#   Created:	   2001-03-27
#   Copyright:     Copyright (c) 2001 Zero-Knowledge Systems, Inc
#
#   $Id: nymcert.pl,v 1.2 2001/04/19 19:17:54 alainc Exp $
#   Revision:      $Revision: 1.2 $
#   Revised:	   $Date: 2001/04/19 19:17:54 $
#
#   Description:   fetch/store a nym's base64 mailcert.
#                  prints cert to stdout.
#
#                  Requires a working nymtool and a configured nymtool
#                  test network profile generated via netgenconfig.sh.
#                  Also requires valid nyms on that test network
#                  and saved nym keys.
#
#   Usage:         $progname <network> <nymtool config> <nym name>
#
#   Details:
#
#     expects config:
#       ${HOME}/.freedom/${NETWORK}/${NETWORK}.conf
#
#     expects nyms in:
#       ${HOME}/.freedom/${NETWORK}/keys/
#
#     stores/fetches mailcerts to/from:
#       ${HOME}/.freedom/${NETWORK}/keys/${NYM}/mailcert/1
#
#***********************************************************************/


use strict;


# globals

my ($debug) = 0;
my ($home) = `echo "\${HOME}"`;  chomp ($home);


my ($query);
my ($query_template) = "nymcert_query.tmpl";
my ($mailcert_file) = "mailcert64.out";


my ($network, $config, $nym);
my ($cert);



#------
# usage
#------
sub usage ()
{
  my ($progname) = $ARGV[0];
  printf STDERR
    "\n\nUsage: $progname <network> <nymtool config> <nym name> \n\n";
}



#--------
# nym_cert
#
# fetch/store a nym's base64 mailcert.
#
# $mailcert nym_cert ($network, $config, $nym)
#--------
sub nym_cert ($ $ $)
{
  my ($network, $config, $nym) = (@_);
    
  my ($nymcert_dir) = "$home/.freedom/$network/keys/$nym/mailcert/";
  my ($nymcert_file) = $nymcert_dir . "1";

  my ($cert);

  print STDERR "looking for cert $nymcert_file\n" if ($debug);
  # fetch cert if present
  if ( -d $nymcert_dir && -r $nymcert_file )
  {
    $cert = `cat $nymcert_file`;
    chomp $cert;
    print STDERR "found cert\n" if ($debug);
    return ($cert);
  }

  # else invoke nymtool nymcert_query and store new mailcert
  else
  {
    print STDERR "invoking nymtool to fetch cert\n" if ($debug);
    $query = `cat $query_template | sed -e "s/<nym>/$nym/g" | nymtool --config $config`;

    print STDERR "query results:\n\n$query\n\n" if ($debug);
    if ( $query =~ /\s+Entity:\s+type:nym\s+name:([A-Za-z0-9\._-]+)/ )
    {
      print STDERR "fetched cert\n" if ($debug);
      if ( $nym =~ /$1/ )
      {
	print STDERR "storing cert\n" if ($debug);
	if ( ! -d $nymcert_dir)
	{
	  `mkdir -p $nymcert_dir`;
	  `chmod 700 $nymcert_dir`;
	}
	`cp -fp $mailcert_file $nymcert_file`;
	$cert = `cat $nymcert_file`;
	chomp $cert;
	print STDERR "stored cert\n" if ($debug);
	return ($cert);
      }
    }

  }

  # failed!
  return;
}



# main

  if (@ARGV != 3)
  {
    &usage();
    exit 1;
  }


  ($network, $config, $nym) = (@ARGV);
  $cert = &nym_cert ($network, $config,$nym);


  if (defined ($cert))
  { 
    print "$cert\n";
  }
  else
  {
    print STDERR "cert not found\n";
    exit 1;
  }

# main
