# A File that includes some standard ZKS definitions used in various scripts

#$Id: defs.tcl,v 1.4 2001/04/19 19:18:00 alainc Exp $

set TESTLIB_RT $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt
source $TESTLIB_RT/inc/netconf.tcl 

# Boolean Values
# ==============

set FALSE 0
set TRUE 1

# Key Bindings
# ============

set keys(CTRL_C) \003

# Regular Expressions
# ===================

set regexp(unixfname) "^(/?(\[^/]+/)*)(\[^/]+)$"

# Port Definitions
# ================

set ports(anon)    51100
set ports(nymsrv)  51104
set ports(toksrv)  51103
set ports(niqs)    51107
set ports(keyqry)  51109
set ports(fmgmaip) 51110
set ports(maip)    51112
set ports(cmd)     51116
set ports(pop)     51114
set ports(nmta)    51115
set ports(imep)    25

# Welcome Messages
# ================
set welcome_message(pop) "OK "
set welcome_message(imep) "ESMTP"
set welcome_message(nmta) "ESMTP"

# Quit messages
# =============
set quit_message(pop) "QUIT\r"
set quit_message(imep) "QUIT\r"
set quit_message(nmta) "QUIT\r"

# Zkip Command Ids
# ================

set zkipcmd(zkip)    0 ;# basic zkip commands (list,quit,...)
set zkipcmd(nymsrv)  1 ;# nym server access
set zkipcmd(amtp)    2 ;# anonymous mail transfer
set zkipcmd(tokauth) 3 ;# token authentication (unused)
set zkipcmd(keyqry)  4 ;# key query server access
set zkipcmd(toksrv)  5 ;# token server access
set zkipcmd(data)    6 ;# token server access
set zkipcmd(niqs)    7 ;# niqs access (network topology)
set zkipcmd(lode)    9 ;# support admin (lode server)

# Command Line Prompts (from various tools)
# =========================================

set cmdline(zknsa)     "ZKNSA> "
set cmdline(zktsa)     "ZKTSAI1.0> "
set cmdline(nymtool)   "nymtool-1.0> "
set cmdline(niqs_test) "NiqsClient-2.2> "
set cmdline(dbregress) "dbregress-1.1a> "

# Entity Types
# ============

set entity_type(0) aip
set entity_type(1) maip
set entity_type(2) niss
set entity_type(3) niqs
set entity_type(4) keyqry
set entity_type(5) nymsrv
set entity_type(6) toksrv
set entity_type(7) fmg
set entity_type(8) pop
set entity_type(9) imep
set entity_type(10) nmta
set entity_type(11) coreaip

# Key Types
# =========

set keyInfo(type,num) 2
set keyInfo(type,0) c
set keyInfo(type,1) s
set keyInfo(version,num) 2
set keyInfo(version,0) 0
set keyInfo(version,1) 1

# Lode Server
# ===========
set frlodec(user)       default
set frlodec(passphrase) titi
set frlodec(reason)     testing

# Mail Password
# =============
set mailpasswdfile "mailcert64.out" 
set testpasswdfile "/tmp/testmailpass"

# Token/SN Types
# ===========
# Type 500 - mail and cloud access (standard paid nym) - can create and extend
set tokenInfo(500,snid) 500
set tokenInfo(500,typeid) 30
set tokenInfo(500,ntok) 5
set tokenInfo(500,nymtool_msg) "(30|Standard Freedom Token \r\n)"
set tokenInfo(500,mail) "yes"
set tokenInfo(500,cloud) "yes"
# Type 900 - mail and cloud access (standard paid nym) - can create and extend - 1 Day token
set tokenInfo(900,snid) 900
set tokenInfo(900,typeid) 80
set tokenInfo(900,ntok) 5
set tokenInfo(900,nymtool_msg) "(80|1 Day Freedom Token \r\n)"
set tokenInfo(900,mail) "yes"
set tokenInfo(900,cloud) "yes"
# Type 501 - mail access, no cloud - can create and extend 
set tokenInfo(501,snid) 501
set tokenInfo(501,typeid) 31
set tokenInfo(501,ntok) 5
set tokenInfo(501,nymtool_msg) "(31|Standard Freedom Token Type 2|Mail Standard Freedom Token Type 2)"
set tokenInfo(501,mail) "yes"
set tokenInfo(501,cloud) "no"
# Type 901 - mail access, no cloud - can create and extend - 1 day token
set tokenInfo(901,snid) 901
set tokenInfo(901,typeid) 81
set tokenInfo(901,ntok) 5
set tokenInfo(901,nymtool_msg) "(81|1 Day Freedom Token Type 2)"
set tokenInfo(901,mail) "yes"
set tokenInfo(901,cloud) "no"
# Type 502 - no mail access, no cloud - only for nym creation (of nyms w/ no functionality)
set tokenInfo(502,snid) 502
set tokenInfo(502,typeid) 32
set tokenInfo(502,ntok) 3
set tokenInfo(502,nymtool_msg) "(32|Standard Freedom Token Type 3 creation|FreeCreate Standard Freedom Token Type 3 create)"
set tokenInfo(502,mail) "no"
set tokenInfo(502,cloud) "no"
# Type 902 - no mail access, no cloud - only for nym creation (of nyms w/ no functionality) - 1 day token
set tokenInfo(902,snid) 902
set tokenInfo(902,typeid) 82
set tokenInfo(902,ntok) 3
set tokenInfo(902,nymtool_msg) "(82|1 Day Freedom Token Type 3 creation|1 Day Standard Freedom Token Type 3 create)"
set tokenInfo(902,mail) "no"
set tokenInfo(902,cloud) "no"
# Type 503 - no mail access, no cloud - only for nym extension (of nyms w/ no functionality)
set tokenInfo(503,snid) 503
set tokenInfo(503,typeid) 33
set tokenInfo(503,ntok) 5
set tokenInfo(503,nymtool_msg) "(33|Standard Freedom Token Type 3 extension|FreeExtend Standard Freedom Token Type 3 extension)"
set tokenInfo(503,mail) "no"
set tokenInfo(503,cloud) "no"
# Type 903 - no mail access, no cloud - only for nym extension (of nyms w/ no functionality) - 1 day variety
set tokenInfo(903,snid) 903
set tokenInfo(903,typeid) 83
set tokenInfo(903,ntok) 5
set tokenInfo(903,nymtool_msg) "(83|1 Day Freedom Token Type 3 extension)"
set tokenInfo(903,mail) "no"
set tokenInfo(903,cloud) "no"

# Serial Number Types
# ===================
set SNType(0) 500
set SNType(1) 501
set SNType(2) 502
set SNType(3) 503
set SNType(4) 900
set SNType(5) 901
set SNType(6) 902
set SNType(7) 903

#Combinations for token checking
#===============================
set num_combos 13

set combo(0,create) 900
set combo(0,add) 900
set combo(0,result) success
set combo(1,create) 900
set combo(1,add) 901
set combo(1,result) failure
set combo(2,create) 900
set combo(2,add) 902
set combo(2,result) failure
set combo(3,create) 900
set combo(3,add) 903
set combo(3,result) failure
set combo(4,create) 901
set combo(4,add) 900
set combo(4,result) failure
set combo(5,create) 901
set combo(5,add) 901
set combo(5,result) success
set combo(6,create) 901
set combo(6,add) 902
set combo(6,result) failure
set combo(7,create) 901
set combo(7,add) 903
set combo(7,result) failure
set combo(8,create) 902
set combo(8,add) 900
set combo(8,result) failure
set combo(9,create) 902
set combo(9,add) 901
set combo(9,result) failure
set combo(10,create) 902
set combo(10,add) 902
set combo(10,result) failure
set combo(11,create) 902
set combo(11,add) 903
set combo(11,result) success
set combo(12,create) 903
set combo(12,add) 903
set combo(12,result) failure


# Nymtool/Laip passphrase
# =======================
set passphrase 11111111
set passphrase_prompt "Passphrase"

# MAIL STUFF
# ===========
set mailmackey_dir "/usr/freedom/qmail/control"

# GENESIS STUFF
# =============
set genesis_config_path "/home/caesar/genesis/genesis.conf"
