#$Id: laip.tcl,v 1.4 2001/04/19 19:18:00 alainc Exp $

source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/defs.tcl
source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/general.tcl


#----------------------
# connect_laip_cmd_port - telnets localhost 51114 (laip command port) and returns one of {timeout,connected,in_use,unable}
#----------------------
proc connect_laip_cmd_port {logFile test_spawn_id} {

  upvar $logFile logf
  upvar $test_spawn_id sp_id 
  global ports

  spawn telnet localhost $ports(cmd)
  set sp_id(telnet_local) $spawn_id
  expect {
       timeout          {puts $logf "Timeout trying to telnet localhost."
                         set status "timeout"}
       "Local Aip Command Port"  {puts $logf "Connected to localhost $ports(cmd)"
			 set status "connected"}
       "Command port already in use"  {puts $logf "Command port on localhost already in use"
                         set status "in_use"}
       "Unable to connect to remote host"  {puts $logf "Unable to connect to localhost"
                         set status "unable"}
  }

  if {$status == "connected"} {
     send "db\r"
     set timeout 1; expect "WAIT WAIT"; set timeout 10
  }
  return $status
}

#------------------
# quit_laip - destroys routes, clears rules and exits
#------------------
proc quit_laip {} {

  send "db\r"
  set timeout 1; expect "WAIT WAIT"; set timeout 10
  send "nr\r"
  expect "Attempt to clear rules succeeded."
  send "da\r"
  expect "ROUTE DESTROY sent on all valid ACIs."
  send "quit\r"
}
  
  
#----------------
# unblock_port - unblocks port $port_num on route defined by ACI $ACI - returns 0 if timeout, 1 otherwise
#----------------
proc unblock_port {port_num ACI logFile} {

  upvar $logFile logf

   set return_code 1

   send "up $port_num\r"
     expect {       
      timeout        {puts $logf "Timeout trying to block port $port_num"
                       set return_code 0} 
      -re "Shim will unblock port .$port_num. on route "   {puts $logf "Port $port_num unblocked for ACI $ACI"}
      -re "Port .$port_num. is not blocked"
     }
   return $return_code
}

#----------------
# block_port - blocks port $port_num on route defined by ACI $ACI - returns 1 if successful or if port already blocked on route specified by $ACI
#----------------
proc block_port {port_num ACI logFile} {

  upvar $logFile logf

  set return_code 1

   send "bp $port_num\r"
     expect {
       timeout        {puts $logf "Timeout trying to block port $port_num --ERROR"
                        set return_code 0}
       -re "Shim will block port .$port_num. on route .$ACI"   {puts $logf "Port $port_num blocked for ACI $ACI"}
       -re "Port .$port_num. already "   {puts $logf "Port $port_num already blocked for ACI $ACI"}
     }
   return $return_code
}

#-----------------
# get_aip_id_from_ip - returns the entity id of the aip at a given ip address (for being able to create routes w/ last hop a core server based on ip address of server)
#-----------------
proc get_aip_id_from_ip  {ip entityInfo}   {

    upvar $entityInfo eInfo

    set aip_id 0

    for {set i 0}  {$i<$eInfo(aip,num)}  {incr i +1}   {
        if {$eInfo(aip,$i,ipaddr) == $ip}   {
            set aip_id $eInfo(aip,$i,id)
            break
        }
    }
    return $aip_id
}

#-----------------
# get_aip_id_from_name - returns the entity id of the aip with name 'name'
#-----------------
proc get_aip_id_from_name  {name entityInfo}   {

    upvar $entityInfo eInfo

    set aip_id 0

    for {set i 0}  {$i<$eInfo(aip,num)}  {incr i +1}   {
        if {$eInfo(aip,$i,name) == $name}   {
            set aip_id $eInfo(aip,$i,id)
            break
        }
    }
    return $aip_id
}

#-----------------
# get_aip_ip_from_id - returns the ip address of aip specified by id  (for being able to check if given aip has core server on it)
#-----------------
proc get_aip_ip_from_id {id entityInfo} {

  upvar $entityInfo eInfo

     for {set i 0} {$i<$eInfo(aip,num)} {incr i +1}  {
        if {$eInfo(aip,$i,id) == $id}  {
            set aip_ip $eInfo(aip,$i,ipaddr)
            break
        }
     }
     return $aip_ip
}


#-----------------
# get_aip_name_from_id - returns the aip entity name based on its id
#-----------------
proc get_aip_name_from_id  {id entityInfo}   {
  upvar $entityInfo eInfo

    for {set i 0} {$i<$eInfo(aip,num)} {incr i +1}  {
        if {$eInfo(aip,$i,id) == $id}  {
            set aip_name $eInfo(aip,$i,name)
            break
        }
    }
    return $aip_name

}

#--------------
# check_if_coreaip - returns 'yes' if given aip is a coreaip (based on if it is called host5 or host6 for now)
#-------------

proc check_if_coreaip {id entityInfo logfile} {

   upvar $entityInfo eInfo
   upvar $logfile logf
   global spawn_id

   set aip_name [get_aip_name_from_id $id eInfo] 
   puts $logf "Aip name for id $id is $aip_name"
   if {[regexp "core-zks-\[0-9]*" $aip_name ignore]} {
       set retmsg "yes"
       puts $logf "Aip with id # $id is a coreaip"
   } elseif {("$aip_name" == "host5") || ("$aip_name" == "host6")} {
       set retmsg "yes"
       puts $logf "Aip with id # $id is a coreaip"
   } else {
       set retmsg "no"
       puts $logf "Aip with id # $id is not a coreaip"
   }
   
   return $retmsg
}


#-----------------
# create_route_w_last_hop - creates a route with the last hop being the aip on core server specified by type and index (for creating unauthenticated routes to a given core server) - returns ACI of route created
#-----------------
proc create_route_w_last_hop {type index logFile entityInfo} {

  upvar $entityInfo eInfo 
  global ports
  upvar $logFile logf

    set ACI -1
    if {$eInfo($type,num) > $index}  {

       clear_rules logf

# make sure route is unauthenticated
#       unauth_route logf

# get id num of aip that is on same machine as ip address specified
       set id_index "$type$index"
       set_exit_hop [get_aip_id_from_ip $eInfo($type,$index,ipaddr) eInfo] logf

       set ACI [create_route logf]
       if {$ACI != "DESTROYED"} {
         unblock_packets $eInfo($type,$index,ipaddr) $ports($type) logf
         block_packets $ACI $eInfo($type,$index,ipaddr) $ports($type) logf
       } else {
         puts $logf "ERROR - Could not create a route to $type #$index" 
       }
    } else {
        puts $logf "ERROR - Could not create route with last hop as $type #$index.  There are only $eInfo($type,num) $type"
    }
    return $ACI

}

#-----------------
# create_specific_coreaip_route - creates specified route and blocks shitload of ports
#-----------------
proc create_specific_coreaip_route {route num_hops logFile spid} {

  global ports
  upvar $logFile logf

  set ACI [create_single_route $route $num_hops 0 logf $spid]
  
if {$ACI != "DESTROYED"} {
  set server(0) nymsrv
  set server(1) toksrv
  set server(2) keyqry
  set server(3) niqs
  set server(4) nmta
  set server(5) pop

    foreach name [array names server] {
       if { ![block_port $ports($server($name)) $ACI logf]} {
           puts $logf "ERROR - Could not add shim rule for  $ports($server($name))"
       } else {
           puts $logf "Shim rule for $ports($server($name)) added"
       }
    }
 }
  return $ACI

}

#----------------
# create_unauth_routes - creates unautheticated routes to all nym servers, kqds, token servers anqs niqs
#----------------

proc create_unauth_routes {entityInfo logfile} {

	upvar $entityInfo eInfo
	upvar $logfile logf

	for {set i 0} {$i<$eInfo(nymsrv,num)} {incr i +1} {
		create_route_w_last_hop nymsrv $i logf eInfo
	}
	for {set i 0} {$i<$eInfo(keyqry,num)} {incr i +1} {
		create_route_w_last_hop keyqry $i logf eInfo
	}
	for {set i 0} {$i<$eInfo(niqs,num)} {incr i +1} {
		create_route_w_last_hop niqs $i logf eInfo
	}
	for {set i 0} {$i<$eInfo(toksrv,num)} {incr i +1} {
		create_route_w_last_hop toksrv $i logf eInfo
	}
}
 

#----------------
# create_coreaip_routes - creates unauthenticated route through coreaip to all  nym servers, kqds, token servers and niqs  
#----------------

proc create_coreaip_routes {entityInfo logfile} {

	upvar $entityInfo eInfo
	upvar $logfile logf
        global ports
 
     #list of servers for which you want to route traffic through coreaip 
        set server(0) nymsrv
        set server(1) toksrv
        set server(2) keyqry
        set server(3) niqs
        set server(4) nmta
        set server(5) pop

# create a route ending in a coreaip

        set coreaip_num 0
        clear_rules logf
        if {$eInfo(coreaip,num) <= $coreaip_num} {
          puts $logf "ERROR - There are only $eInfo(coreaip,num) coreaips in network -- cannot make route through coreaip $coreaip_num"
        } else {
          if {[set_exit_hop $eInfo(coreaip,$coreaip_num,id) logf]} {		
            set ACI [create_route logf]
            if {$ACI == "DESTROYED"} {
                puts $logf "ERROR - Could not create route ending at coreaip $eInfo(coreaip,$coreaip_num,name)"
            } else {
                puts $logf "Route created ending at coreaip $eInfo(coreaip,$coreaip_num,name)"
        # now block packets going to any of the servers mentioned above 
                foreach name [array names server] {
                   if { ![block_port $ports($server($name)) $ACI logf]} {
                       puts $logf "ERROR - Could add shim rule for $eInfo(coreaip,$coreaip_num,name) $ports($server($name))"
                   } else {
                       puts $logf "Shim rule for $eInfo(coreaip,$coreaip_num,name) $ports($server($name)) added"
		   }
                }
            }
          }
        }
}
 

#-------------------
# clear_rules - clear all the rules associated with creating a route
#-------------------
proc clear_rules  {logFile}  {

  upvar $logFile logf

    set return_code 0
    send "nr\r"
    expect {
           timeout  {puts $logf "Timeout during 'new route' -- ERROR"
                        set return_code 0}
           "Attempt to clear rules succeeded."  {puts $logf "Route cleared"
						set return_code 1}
    }
    return $return_code
}

#--------------------
# set_exit_hop - set exit hop to aip with specified id - returns 1 if successful, 0 otherwise
#--------------------
proc set_exit_hop {id logFile}  {
# specify the last hop
  upvar $logFile logf

    send "se $id\r"

    expect {
           timeout  {puts $logf "Timeout during 'set exit hop' -- ERROR"
                        set return_code 0}
           -re "Set exit hop .$id. succeeded."  {puts $logf "exit hop set at $id"
						 set return_code 1}
           -re "Set exit hop .$id. failed."  {puts $logf "exit hop set at $id FAILED -- ERROR"
                        set return_code 0}
    }
    return $return_code
}


#---------------------
# set_num_hops - set the number of hops in given route - returns 1 if successful, 0 otherwise
#---------------------
proc set_num_hops {num_hops logFile} {

  upvar $logFile logf

  send "sh $num_hops\r"
  expect {
           timeout  {puts $logf "Timeout during 'set num hop' -- ERROR"
                        set return_code 0}
           -re "Number of hops .$num_hops. set."  {
              puts $logf "Number of hops set at $num_hops"
			set return_code 1
           }
  }
  return $return_code
}
  
#--------------------
# set_next_hop - set next hop in route - returns 1 if successful, 0 otherwise
#--------------------
proc set_next_hop {id logFile} {

  upvar $logFile logf
  
  send "sp $id\r"
       expect {
           timeout  {puts $logf "Timeout during 'set next hop' -- ERROR"
                        set return_code 0}
           -re "Set next hop .$id. succeeded."  {puts $logf "next hop set at $id"
						 set return_code 1}
           -re "Set next hop .$id. failed."  {puts $logf "next hop set at $id FAILED -- ERROR"
                        set return_code 0}
       }
  return $return_code
}
 

#--------------------
# unauth_route - unauthenticates route (needs to be called before you create route)
#--------------------
proc unauth_route {logFile} {
# unauthenticate the route
  upvar $logFile logf

        send "sn\r"
        expect {
               timeout {puts $logf "Could not unauthenticate route -- ERROR"
			set return_code 0}
               "Current Nym set to \"Anonymous User (unauthenticated routes only"  {puts $logf "Route has been set to unauthenticated."
			set return_code 1}
        }
    return $return_code
}

#---------------------
# create_route - create a route, based on any previously specified rules.  returns the ACI of route if successful, DESTROYED if unsuccessful.
#---------------------
proc create_route  {logFile}   {

   upvar $logFile logf

   send "cr\r"
   expect {
           timeout  {puts $logf "Timeout during route creation. - ERROR"
                                 set ACI DESTROYED}
           -re "ROUTE CREATE acked on ACI (\[0-9]*)"  {puts $logf "Route CREATED on ACI $expect_out(1,string)"
						       set ACI $expect_out(1,string)}

           -re "ROUTE DESTROY sent on ACI (\[0-9]*)"  {puts $logf "Route NOT CREATED"
                                                       set ACI DESTROYED    }
           "No routes available for the set or rules"     {puts $logf "Route could not be created"
                                                           set ACI DESTROYED}
   }
   return $ACI
}


#----------------------
# destroy_route - destroys route specified by ACI $ACI - returns 1 if DESTROY sent, 0 otherwise
#----------------------
proc destroy_route {ACI logFile}  {

   upvar $logFile logf

        send "dr $ACI\r"
           expect {
                   timeout {puts $logf "Timeout trying to destroy route with ACI $ACI -- ERROR"
                             set return_code 0}
                   "ROUTE DESTROY sent on ACI $ACI" {puts $logf "ROUTE DESTROY sent for ACI $ACI"
			     set return_code 1}
                  }
	return $return_code
}

#----------------------
# destroy_coreaip_route - destroys route specified by ACI $ACI - returns 1 if DESTROY sent, 0 otherwise
#----------------------
proc destroy_coreaip_route {ACI logFile}  {

   upvar $logFile logf
   global ports

     #list of servers for which you want to route traffic through coreaip
        set server(0) nymsrv
        set server(1) toksrv
        set server(2) keyqry
        set server(3) niqs
        set server(4) nmta
        set server(5) pop

foreach name [array names server] {
     unblock_port $ports($server($name)) $ACI logf
}

        send "dr $ACI\r"
           expect {
                   timeout {puts $logf "Timeout trying to destroy route with ACI $ACI -- ERROR"
                             set return_code 0}
                   "ROUTE DESTROY sent on ACI $ACI" {puts $logf "ROUTE DESTROY sent for ACI $ACI"
			     set return_code 1}
                  }
	return $return_code
}

#-----------------------
# block_packets - adds shim rule to block packets on a given port of a given hop (specified by ip addr) - returns 1 if successful, 0 otherwise
#-----------------------
proc block_packets {aci ip port logFile} {

   upvar $logFile logf

    send "sb $aci $ip $port\r"
    expect {
         -re "Shim rule add $ip:$port to route .$aci. succeeded." {puts $logf "Shim rule add $ip:$port to route $aci succeeded." 
			set return_code 1}
         -re "Shim rule add $ip:$port to route .$aci. failed." {puts $logf "Shim rule add $ip:$port to route $aci FAILED -- ERROR"
                        set return_code 0}
    }
    return $return_code

}

#-----------------------
# unblock_packets - deletes shim rule on a given port of a given hop (specified by ip addr) - returns 1 if successful, 0 otherwise
#-----------------------
proc unblock_packets {ip port logFile} {

   upvar $logFile logf

    send "su $ip $port\r"
    expect {
        "Shim rule delete $ip:$port succeeded." {puts $logf "Shim rule delete $ip:$port succeeded."
			set return_code 1 }
        "Shim rule delete $ip:$port failed." {puts $logf "Shim rule delete $ip:$port FAILED."
                        set return_code 0}
    }
    return $return_code
}

#-----------------------
# set_nym - sets nym for a given route (need to set nym before creating route) - returns 1 if successful, 0 otherwise
#-----------------------
proc set_nym {nym_name logFile} {
# specify a nym (used for a given route)
   upvar $logFile logf
   
   set return_code 0
   send "sn $nym_name\r"
   expect {
       timeout        {puts $logf "Timeout trying to set Nym"
		       set return_code 0}
       -re "Current Nym set to .$nym_name."   {puts $logf "Nym set to $nym_name"
		       set return_code 1}
       "Requested nym $nym_name not found" {puts $logf "Could not set current nym to $nym_name - nym not found"
		       set return_code 0}
   }
   return $return_code
}

#---------------------
# extract_hops - given a route in the form '<num_hops> <aip_id1> ... <aip_idn> will return an array of hops (hop(0) is aip_id1 hop(1) is aip_id2...)
#---------------------

proc extract_hops {single_route num_hops hop_array logfile} {

   upvar $logfile logf
   upvar $single_route route
   upvar $hop_array hop

	
   set return_code 1

#  not most efficient way but easiest for me for now
# extract all the hop ID info

   if {$num_hops == 1}  {
      scan $route "%d %d" dummy hop(0)
   } elseif {$num_hops == 2} {
      scan $route "%d %d %d" dummy hop(0) hop(1)
   } elseif {$num_hops == 3} {
      scan $route "%d %d %d %d" dummy hop(0) hop(1) hop(2)
   } elseif {$num_hops == 4} {
      scan $route "%d %d %d %d %d" dummy hop(0) hop(1) hop(2) hop(3)
   } elseif {$num_hops == 5} {
      scan $route "%d %d %d %d %d %d" dummy hop(0) hop(1) hop(2) hop(3) hop(4)
   } else {
      puts $logf "Invalid number of hops on route $route_num or non-existant route"
      set return_code 0
   }

   return $return_code
}



#----------------------
# get_routes - using fr_listroutes, returns an array (route) of all 'num_hops' hop routes.  
# The route array is in the following format:
# route(num_routes) - number of routes in the array
# route($i) - route $i in form : <num_hops> <id_hop1> <id_hop2> ... <id_hopn>
# route($i,num_hops) - number of hops in route $i
# route($i,id) - same as route($i) but without the <num_hops> at the beginning
# route($i,names) - lists aip names instead of ids
# If route already contains a list of routes, the new routes will be appended as new entries in the array
# If you want to start with a fresh array, you can initialize an array by using 'unset <array>'
#----------------------
proc get_routes {num_hops route_array entityInfo logfile} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)
     
   if {$num_hops == 1} { 
	get_one_hop_routes route eInfo logf $start_route 
   } elseif {$num_hops > 5} {
	puts $logf "Invalid number of hops: $num_hops"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
        expect {
            -re "(\[0-9| ]*)end"   {set route($route(num_routes)) $expect_out(1,string)
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {puts $logf "There are $route(num_routes) routes"
                           set not_finished 0}
        }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         scan $route($i) "%d " route($i,num_hops)

         if [extract_hops route($i) $route($i,num_hops) hop logf] {
            for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
               append route($i,ids) "$hop($j) "
               append route($i,names) [get_aip_name_from_id $hop($j) eInfo]
               append route($i,names) " "
#                puts $logf  "hop($i) is $hop($i)"
            }
         }
       }
   }

}

proc get_routes_ending_in {last_hop num_hops route_array entityInfo logfile} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)

   puts $logf "Last hop coreaip id is $last_hop"

   if {$num_hops == 1} { 
	set route($start_route) "1 $last_hop" 
	set route($start_route,num_hops) 1 
	set route($start_route,ids) $last_hop 
	set route($start_route,names) [get_aip_name_from_id $last_hop eInfo] 
        set route(num_routes) [expr $route(num_routes) + 1]
   } elseif {$num_hops > 5} {
	puts $logf "Invalid number of hops: $num_hops"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
        expect {
            -re "(\[0-9| ]*) $last_hop end"   {set route($route(num_routes)) "$expect_out(1,string) $last_hop"
				    puts $logf "Adding route $route($route(num_routes))"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {puts $logf "There are $route(num_routes) $num_hops hop routes"
                           set not_finished 0}
        }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         scan $route($i) "%d " route($i,num_hops)

         if [extract_hops route($i) $route($i,num_hops) hop logf] {
            for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
               append route($i,ids) "$hop($j) "
               append route($i,names) [get_aip_name_from_id $hop($j) eInfo]
               append route($i,names) " "
#                puts $logf  "hop($i) is $hop($i)"
            }
         }
       }
   }

}

proc get_routes_with {hop num_hops hop_index route_array entityInfo logfile} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)

   if {$num_hops == 1} { 
	set route($start_route) "1 $hop" 
	set route($start_route,num_hops) 1 
	set route($start_route,ids) $hop 
	set route($start_route,names) [get_aip_name_from_id $hop eInfo] 
        set route(num_routes) [expr $route(num_routes) + 1]
   } elseif {$num_hops > 3} {
	puts $logf "Invalid number of hops: $num_hops"
   } elseif {($hop_index != "all") && ($hop_index > $num_hops)} {
        puts $logf "Cannot have hop_index ($hop_index) greater than num_hops ($num_hops)"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
       if {$hop_index == 1} {
        expect {
            -re "(\[0-9] $hop \[0-9| ]*) end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {$hop_index == 2} {
        expect {
            -re "(\[0-9] \[0-9]+ $hop \[0-9| ]*)end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {$hop_index == 3} {
        expect {
            -re "(\[0-9| ]* $hop) end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {"$hop_index" == "all"} {
        expect {
            -re "(\[0-9| ]* $hop \[0-9| ]*)end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         puts "route #$i is $route($i)"
         scan $route($i) "%d " route($i,num_hops)

         if [extract_hops route($i) $route($i,num_hops) temp_hop logf] {
            for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
               append route($i,ids) "$temp_hop($j) "
               append route($i,names) [get_aip_name_from_id $temp_hop($j) eInfo]
               append route($i,names) " "
#                puts $logf  "temp_hop($i) is $temp_hop($i)"
            }
         }
       }
   }

}

proc get_one_hop_routes {route_array entityInfo logfile start_route} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo

	set last_route [expr $start_route + $eInfo(aip,num)] 
 	for {set i $start_route} {$i<$last_route} {incr i +1} {
	     set route($i) "1 $eInfo(aip,$i,id)"
	     set route($i,num_hops) 1 
	     set route($i,ids) "$eInfo(aip,$i,id) "
	     set route($i,names) "$eInfo(aip,$i,name) "
	     set route(num_routes) [expr $route(num_routes) + 1]
	}
}
	
proc get_one_hop_coreaip_routes {route_array entityInfo logfile start_route} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo

	set last_route [expr $start_route + $eInfo(coreaip,num)] 
 	for {set i $start_route} {$i<$last_route} {incr i +1} {
	     set route($i) "1 $eInfo(coreaip,$i,id)"
	     set route($i,num_hops) 1 
	     set route($i,ids) "$eInfo(coreaip,$i,id) "
	     set route($i,names) "$eInfo(coreaip,$i,name) "
	     set route(num_routes) [expr $route(num_routes) + 1]
	}
}
	

#-------------
# list_routes - lists all routes in given route array (for debugging)
#-------------

proc list_routes {route_array logf} {

   upvar $route_array route
   upvar $logf log

	for {set i 0} {$i<$route(num_routes)} {incr i +1} {
		puts "Route $i: $route($i,names)"
		puts $log "Route $i: $route($i,names)"
	}
}

	
#-------------
# check_route_array - checks 'num_routes' routes that are in route array 'route' by downloading webpage through that route and comparing 
# downloaded page to template that was downloaded in the clear.
#------------- 

proc check_route_array {route_array num_routes nym_name entityInfo logfile web_address laip_spawn_id} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo

	set template "/tmp/web_template.txt"

	if {$num_routes == "all" || $num_routes>$route(num_routes)} {
		set num_routes $route(num_routes)
	}

	if [get_webpage $web_address $template logf] {

	   for {set route_num 0} {$route_num<$num_routes} {incr route_num +1} {
	     puts -nonewline "Checking route $route($route_num,names): "
	     puts $logf "\nChecking route $route($route_num,names): "
# should put some logging here with return value
	     set route($route_num,route_check) [check_single_route $route($route_num) $route($route_num,num_hops) $nym_name logf $web_address $template $laip_spawn_id]
	   }
        }

}

#-------------
# check_DNS_array - checks DNS queries for 'num_routes' routes  
#------------- 

proc check_DNS_array {route_array num_routes entityInfo logfile web_address laip_spawn_id} {

   upvar $logfile logf
   upvar $route_array route
   upvar $entityInfo eInfo


	if {$num_routes == "all"} {
		set num_routes $route(num_routes)
	}

        for {set route_num 0} {$route_num<$num_routes} {incr route_num +1} {
		puts -nonewline "Checking DNS for route $route($route_num,names): "
		puts $logf "\nChecking DNS for route $route($route_num,names): "
# should put some logging here with return value
		set route($route_num,DNS_check) [check_single_DNS $route($route_num) $route($route_num,num_hops) logf $web_address $laip_spawn_id eInfo]
        }

}

#-------------
# dump_route_results - dumps results of check_route_array and check_DNS_array to file
#-------------

proc dump_route_results {route_array dumpfile} {

   upvar $route_array route
   upvar $dumpfile dump
	
      for {set i 0} {$i<$route(num_routes)} {incr i +1} {
	if {[info exists route($i,route_check)]} {
		puts $dump "Route check for $route($i,names): $route($i,route_check)" 
        } else {		
		puts $dump "Route check for $route($i,names): NOT CHECKED" 
        }
	if {[info exists route($i,DNS_check)]} {
		puts $dump "DNS check for $route($i,names): $route($i,DNS_check)" 
        } else {		
		puts $dump "DNS check for $route($i,names): NOT CHECKED" 
        }
	puts $dump ""
      }

}


#-------------
# create_single_route - create a route based on a route specified as command line
#-------------

proc create_single_route {single_route num_hops nym_name logfile laip_spawn_id} {

   upvar $logfile logf
   global spawn_id

	  set ACI DESTROYED

	  set return_code [extract_hops single_route $num_hops hop logf]

	  set spawn_id $laip_spawn_id
          # send 'new route' command to laip
          if {$return_code == 1} {
            set return_code [clear_rules logf]
          }
          # set the number of hops
          if {$return_code == 1} {
            set return_code [set_num_hops $num_hops logf]
          }

          # specify the hops in the route
          for {set i 0} {$i<$num_hops} {incr i +1} {
            if {$return_code == 1} {
               set return_code [set_next_hop $hop($i) logf]
            }
          }

          # spec the nym to be used
          if {$return_code == 1} {
            if {$nym_name == 0}  {
               set return_code [unauth_route logf]
            } else {
               set return_code [set_nym $nym_name logf]
            }
          }

          if {$return_code == 1} {
            set ACI [create_route logf]
            if {$ACI == "DESTROYED"}  {
               set return_code 0
            }
          } else {
            set ACI "DESTROYED"
          }

	  return $ACI
}
	  
	  
#--------------
# check_single_route - checks one route by dowloading webpage and comparing it to a template
#--------------

proc check_single_route {single_route num_hops nym_name logfile web_address template laip_spawn_id} {

   upvar $logfile logf
   global spawn_id

   set webcheck "/tmp/web_check.txt"

          #download through cloud
	  set ACI [create_single_route $single_route $num_hops $nym_name logf $laip_spawn_id]
          if {$ACI != "DESTROYED"} {
             # block port 80 on newly created route
             if {[block_port 80 $ACI logf]} { 
               if [get_webpage $web_address $webcheck logf] {
                 set check_status [check_webfile logf $template $webcheck]
                 if {$check_status != "same"} {
                       puts "Route check FAILED : $check_status"
		       set return_status FAILED
                 } else {
                       puts "Route check PASSED"
		       set return_status PASSED		
                 }
               } else {
	           puts "Could not retrieve webpage $web_address" 
		   set return_status FAILED
	       }
	     } else {
		   puts "Could not block port for route"
		   set return_status FAILED
	     }
	  } else {
	     puts "Route check FAILED:  Could not create route"
	     set return_status FAILED
	  }


          #unblock port 80 and destroy route
          if {$ACI != "DESTROYED"} {
            unblock_port 80 $ACI logf
            destroy_route $ACI logf
          }
	  
	  return $return_status
}

#--------------
# check_single_DNS - checks DNS query for given route
#--------------

proc check_single_DNS {single_route num_hops logfile web_address laip_spawn_id entityInfo} {

   upvar $logfile logf
   upvar $entityInfo eInfo
   global spawn_id

          #create route
          set ACI [create_single_route $single_route $num_hops 0 logf $laip_spawn_id]
          if {$ACI != "DESTROYED"} {
	     extract_hops single_route $num_hops hop logf 
             set last_hop_index [expr $num_hops - 1]
	     set last_hop_ip [get_aip_ip_from_id $hop($last_hop_index) eInfo]
	     block_packets $ACI $last_hop_ip 53 logf 
             set DNS_info [DNS_query $web_address logf] 
             puts $logf "DNS check returned $DNS_info"
	     if {$DNS_info == "empty"}   {
                 puts "Returned host info for route is empty.  DNS query failed -- ERROR"
		 set return_status FAILED
             } else  {
	         puts "Returned host info for route is valid -- $DNS_info" 
		 set return_status PASSED
	     }
	     unblock_packets $last_hop_ip 53 logf 
	     destroy_route $ACI logf
	  } else {
	     puts "Could not create route"
	     set return_status FAILED
          }

	  return $return_status
}


#------------
# get_webpage - downloads a webpage from http://$address (using wget) and saves it to file $webfile
#------------
proc get_webpage {address webfile logfile} {

   upvar $logfile logf

	set return_code 0

	spawn /bin/sh -c "wget -T 30 -O $webfile http://$address"

        expect {
            timeout        {puts $logf "Timeout trying to connect to $address -- ERROR"
                             set return_code 0}
            "connected!"   {puts $logf "Connected to $address"
			     set return_code 1}
            "Host not found"  {puts $logf "Cannot connect to $address -- ERROR"
                             set return_code 0}
            }
        if {$return_code == 1}  {
          expect {
		"$webfile' saved"   {puts $logf "Web page saved to $webfile"}
		timeout 	    {puts $logf "Timeout trying to save $address"
			             set return_code 0}
	  }
        }
	wait 
	close

        spawn /bin/sh -c "\rm wget-log*"
	wait -nowait
	close

	return $return_code
}

#-----------
# check_webfile - checks that 'template' and 'check' are identical files
#-----------

proc check_webfile {logfile template check}  {
 
   upvar $logfile logf

   spawn diff -s -q $template $check
   expect {
          timeout {puts $logf "Timeout during diff of web_files -- ERROR"
                        set status "timeout"}
          "Files $template and $check differ"  {puts $logf "Downloaded web page is different -- ERROR"
                        set status "different"}
          "Files $template and $check are identical"  {puts $logf "Downloaded web page is same -- GOOD"
			set status "same"}
   }
   close 
   wait -nowait

   return $status
}

#------------
# DNS query - spawns 'host $web_address' and returns either the ip address or 'empty' if host info was not returned
#------------

proc DNS_query {web_address logfile} {

    upvar $logfile file

    spawn host $web_address
    expect  {
            timeout {puts $file "Timeout trying to get ip address of $web_address"
                     set DNS_result empty}
            -re "$web_address has address (\[0-9]*\.\[0-9]*\.\[0-9]*\.\[0-9]*)"   {
                               puts $file "DNS query for $web_address successful - returned $expect_out(1,string)"
                               set DNS_result $expect_out(1,string)}  
            }

    wait -nowait
    close

    return $DNS_result
}

#-------------
# TestCoreAips - given a server type, one route for each aip (as exit hop) will be created and it will be ensured that only routes ending in coreaips can access the server
#-------------

proc TestCoreAips {server_type nym_name entityInfo test_spawn_id logfile} {

   upvar $entityInfo eInfo
   upvar $logfile logf
   global ports spawn_id welcome_message quit_message

   set connected 0

      for {set server_index 0} {$server_index<$eInfo($server_type,num)} {incr server_index +1} {
	for {set i 0} {$i<$eInfo(aip,num)} {incr i +1} {
          set spawn_id $test_spawn_id
          set could_create 0
	  if {[clear_rules logf] == 1} {
	    if {[set_exit_hop $eInfo(aip,$i,id) logf] == 1} {
              if {[set_nym $nym_name logf] == 1} {
                set ACI [create_route logf]
                if {$ACI != "DESTROYED"} {
                   puts $logf "Route with $eInfo(aip,$i,name) as last hop created"
                   set could_create 1
                   unblock_port $ports($server_type) $ACI logf
                   block_port $ports($server_type) $ACI logf
                   puts $logf "Trying to connect to $server_type $eInfo($server_type,$server_index,name)"
                  # HERE WE NEED TO TELNET TO THE RIGHT PORT AND EXPECT THE RIGHT COMMAND 
		   spawn /bin/sh -c "telnet $eInfo($server_type,$server_index,ipaddr) $ports($server_type)"
                   expect {
                        timeout {set connected "timeout"}
                        "$welcome_message($server_type)" {set connected "yes"
       						   TimeWait 2
						   send "$quit_message($server_type)"
 					#	   wait
					#	   close}
                        "telnet: Unable to connect to remote host" {set connected "unable"}
                   }
                   if {[check_if_coreaip $eInfo(aip,$i,id) eInfo logf] == "yes"} {
                      puts $logf "Aip $eInfo(aip,$i,name) is a core aip:"
                      if {$connected == "yes"} {
                        puts $logf "SUCCESS - SUCCESSFULLY CONNECTED TO $server_type $eInfo($server_type,$server_index,name) through coreaip" 
                      } else {
                        puts $logf "ERROR - COULD NOT CONNECT TO $server_type $eInfo($server_type,$server_index,name) through coreaip" 
                      }
                   } else {
                      puts $logf "Aip $eInfo(aip,$i,name) is not a core aip:"
                      if {$connected == "yes"} {
                        puts $logf "ERROR - WAS ABLE TO CONNECT TO $server_type $eInfo($server_type,$server_index,name) through normal aip" 
                      } else {
                        puts $logf "SUCCESS - COULD NOT CONNECT TO $server_type $eInfo($server_type,$server_index,name) through normal aip" 
                      }
                   }
	         } 
               }
             }
           }
           if {$could_create == 0} {
             puts $logf "Could not create route with aip $eInfo(aip,$i,name) as last hop and nym set to $nym_name"
           }
         }
      }
}
            
