#---- POP PROCEDURES ------#

#--------
# pop_login - logs into specified pop server and waits for password authentication.  
#--------
proc pop_login {pop_server port nymname password spid} {

  upvar $spid pop_spawn 
  global spawn_id

  spawn /bin/sh -c "telnet $pop_server $port"
  set pop_spawn $spawn_id
  expect {
    "OK" {}
    "No route to host" {return failure}
  }
  send "USER $nymname\r"
  expect {
    "OK" {}
     timeout {return failure}
  }
  send "PASS $password\r"
  expect {
    "OK" {}
     timeout {return failure}
  }

  return success
}


#---------
# pop_stat - retrieves stat info from pop server.  Assumes user is already logged in
#---------

proc pop_stat {pop_stats spid} {
  
  upvar $pop_stats stats
  global spawn_id

  set spawn_id $spid

  send "STAT\r"
  expect {
 	timeout {return failure}
        -re "OK (\[0-9]+) (\[0-9]+)\r\n" {set stats(num_message) $expect_out(1,string)
                                          set stats(pop_box_size) $expect_out(2,string)}
  }
 
  return success
}

#--------
# pop_quit - ends pop session (assumes already logged in to pop server)
#--------
proc pop_quit {spid} {

  global spawn_id

  set spawn_id $spid

  send "QUIT\r"
  expect {
     "OK" {}
      timeout {return "failure"} 
  }
  wait 
  close
 
  return success
}
      
#--------
# pop_retrieve - reads a given message in a pop box (must already be logged in)
#--------

proc pop_retrieve {spid msg_index message} {

  upvar $message msg
  global spawn_id

  set spawn_id $spid

  send "RETR $msg_index\r"
  expect "RETR $msg_index"
  expect {
    timeout {return "failure"}
    "+OK " {}
  }
  expect {
    timeout {return failure}
    -re "(.*)\r\n\.\r\n" {set msg $expect_out(1,string)}
  }

  return success
}


#--------
# pop_delete - deletes a given message in a pop box (must already be logged in)
#--------

proc pop_delete {spid msg_index} {

  global spawn_id

  set spawn_id $spid

  send "DELE $msg_index\r"
  expect "DELE $msg_index"
  expect {
    timeout {return "failure"}
    "+OK " {}
  }

  return success
}
 
#*********** NMTA PROCEDURES *******

#--------
# proc convert_to_base64 - converts string to bse 64
#--------

proc convert_to_base64 {string} {

  spawn /bin/sh -c "echo -n $string | mimencode"
  expect {
      -re "(.*)\r\n" {return $expect_out(1,string)}
      timeout {return failure} 
  }
  wait
  close
}


#---------
# proc nmta_login - login to nmta and authenticate oneself - returns 'failure' on failure
#---------

proc nmta_login {nmta port nym passwd spid} {

  upvar $spid nmta_spid
  global spawn_id

  spawn /bin/sh -c "telnet $nmta $port"
  set nmta_spid $spawn_id

  expect {
    "ESMTP" {}
    "No route to host" {return "No route to host"}
    timeout {return failure}
  }
  send "EHLO\r"
  expect {
    "8BITMIME" {}
    timeout {return failure}
  }
  send "AUTH LOGIN\r"
  expect {
    -re "334(.*)\r\n" {}
     timeout {return failure}
  }
  send "[convert_to_base64 $nym]\r"
  expect {
    -re "334(.*)\r\n" {}
     timeout {return failure}
  }
  send "$passwd\r"
  expect {
    "go ahead" {}
    timeout {return failure}
    "auth failure" {return failure}
  }
      
  return success 
}
  
  
#------------
# nmta_send - sends message from nym to recpt - assumes alredy logged in to nmta
#------------
  
proc nmta_send {nym_address recpt message spid} {

  global spawn_id

  set spawn_id $spid

  send "MAIL FROM:<$nym_address>\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }
  send "RCPT TO:<$recpt>\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }
  send "DATA\r"
  expect {
    "go ahead" {}
    timeout {return failure}
  } 
  send "$message\r.\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }

  return success
}
  
#--------
# nmta_quit - ends nmta session (assumes already logged in to nmta)
#--------
proc nmta_quit {spid} {

  global spawn_id

  set spawn_id $spid

  send "QUIT\r"
  expect {
     "Connection closed by foreign host" {}
      timeout {return "failure"} 
  }
  wait 
  close
 
  return success
}



# IMEP PROCEDURES
#----------------
#--------
# imep_login - logs in to imep
#--------

proc imep_login {imep port spid} {

  upvar $spid imep_spid
  global spawn_id

  spawn /bin/sh -c "telnet $imep $port"
  set imep_spid $spawn_id

  expect {
    "ESMTP" {}
    "No route to host" {return "No route to host"}
    timeout {return failure}
  }

  send "HELO\r"
  expect {
    -re "250(.*)\r\n" {}
    timeout {return failure}
  }
    
  return success
}

#------
# imep_send - sends message from sender to recpt - assumes already logged into imep
#------

proc imep_send {sender recpt message spid} {
 
  global spawn_id

  set spawn_id $spid

  send "MAIL FROM:<$sender>\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }
  send "RCPT TO:<$recpt>\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }
  send "DATA\r"
  expect {
    "go ahead" {}
    timeout {return failure}
  } 
  send "$message\r.\r"
  expect {
    "250 ok" {}
    timeout {return failure}
  }

  return success
}

#-----
# imep_quit - ends imep session (assumes already logged in to imep)
#-----

proc imep_quit {spid} {

  global spawn_id
 
  set spawn_id $spid

  send "QUIT\r"
  expect {
    "Connection closed by foreign host" {}
    timeout {return "failure"} 
  }
  wait 
  close
 
  return success
}


#************* USEFUL FUNCTIONS NEEDED

#------------
# WaitForMail - will check account at pop_server until pop account has total_messages - will return message #total_messages in message - Will check every time_internal and give up after max_time
#------------

proc WaitForMail {time_interval max_time total_messages pop_server port account passwd message} {

  upvar $message msg

  set arrived 0
  set total_time 0
  while {!$arrived} {
    puts "Waiting $time_interval..."
    exec sleep 5
    set total_time [expr $total_time + $time_interval]
    puts "Checking if mail arrived for $account..."
    if  {[pop_login $pop_server $port $account $passwd pop_spid] != "success"} {
      puts "Error logging into pop server $pop_server"
      return 1
    }
    if {[pop_stat stats $pop_spid] != "success"} {
      puts "Could not retrieve pop stats for $account at $pop_server"
      return 1
    }
    if {$stats(num_message) < $total_messages} {
      puts "Mail has not arrived for $account - Only has $stats(num_message) messages"
      if {$total_time >= $max_time} {
        puts "Message #$total_messages did not arrive for $account after $max_time ... giving up"
        pop_quit $pop_spid
        return 1
      }
    } else {
      puts "Retrieving mail message $total_messages for $account"
      if {[pop_retrieve $pop_spid $total_messages msg] != "success"} {
        puts "Could not retrieve mail message #$total_messages for $account"
        pop_quit $pop_spid
        return 1
      } 
      if {[pop_delete $pop_spid $total_messages] != "success"} {
        puts "Could not delete mail message #$total_messages for $account"
        pop_quit $pop_spid
        return 1
      }
      set arrived 1
    }
    pop_quit $pop_spid
  }
  return 0
}



