# $Id: nympop.tcl,v 1.3 2001/04/19 19:18:00 alainc Exp $

# ---------
# GetTokens
# ---------
# go get  serial number(s) from website (as specified in nymtool config
# file) and trade for at least `nymcount` tokens

proc nympop_GetTokens {log nymcount SNtype} {
    global entityInfo cmdline ntspawnid
    upvar $log logf

    puts $logf "**** GET TOKENS **** --------------------------------------------"
    puts $logf "     nympop_GetTokens {log $nymcount}"
    puts $logf "     Get tokens from scalper"

    set numsn [expr ((($nymcount)/$entityInfo(nymsrv,num) - 1)/5 + 1) * \
	            $entityInfo(nymsrv,num)]

    for {set i 0} {$i < $numsn} {incr i +1} {

	set index [expr $i % $entityInfo(nymsrv,num)]
	set ntsid $ntspawnid(nymtool,$index)

	# redeem a serial nymber
	set msg [toksrv_RedeemSN $ntsid $SNtype]
	if ![string compare $msg "success"] {
	    puts $logf "     --- SUCCESS --- 5 tokens type $SNtype- Redemption successful"
	} else {
            puts $logf "ERROR  --- Token Redemption unsuccessful -type $SNtype-- ABORT"
            exit
        }
    }
}

# ------------------
# CreateNyms
# ------------------
# Create nymcount nyms. Make sure that the nyms you are creating do not already
# exist or the script will signal failure.
#
#     Nyms created: [ $basename0, $basename1, ... , $basename[$nymcount-1] ]

proc nympop_CreateNyms {log basename nymcount SNtype} {
    global cmdline entityInfo ntspawnid network netconf tokenInfo
    upvar $log logf

    puts $logf "**** CREATE NYMS **** -------------------------------------------"
    puts $logf "     nympop_CreateNyms {log $basename $nymcount}"
    puts $logf "     Creating $nymcount nyms:"

    for {set i 0} {$i < $nymcount} {incr i +1} {

        set index [expr $i % $entityInfo(nymsrv,num)]
        set ntsid   $ntspawnid(nymtool,$index)
        set nymname $basename$i

	# Get a new tokens if required (every 5 nyms)
#	if ![expr $i % $tokenInfo($SNtype,ntok)] { nympop_GetTokens logf 5 $SNtype}

	if ![expr $i % ($entityInfo(nymsrv,num)*$tokenInfo($SNtype,ntok))] {  
          for {set srv 0} {$srv<$entityInfo(nymsrv,num)} {incr srv +1} {
            toksrv_RedeemSN $ntspawnid(nymtool,$srv) $SNtype
          }
        }

	# make sure nym not there -- shouldn't be
	nympop_DoNotThereLookup logf $nymname $ntsid

	# create the nym
	set msg [nymsrv_CreateNym $nymname $ntsid $SNtype]
	if ![string compare $msg "timeout"] {
	    puts $logf "     --- TIMEOUT --- $nymname : SNtype \"$SNtype\" - nym creation failed"
	} elseif ![string compare $msg "success"] {
	    puts $logf "     --- SUCCESS --- $nymname : SNtype \"$SNtype\" - nym created"
	} elseif ![string compare $msg "failure"] {
	    puts $logf "     --- FAILURE --- $nymname : SNtype \"$SNtype\" - nym creation failed"
	} elseif ![string compare $msg "repeat"] {
	    puts $logf "     --- FAILURE --- $nymname : SNtype \"$SNtype\" - nym  exists --- ABORT"
	    exit
	}

	if {[nymsrv_KeysaveNym $ntsid $netconf($network,keydir)] != "success"} {
	    puts $logf "     could not save keys for $nymname "
	}

	# should be there now
	nympop_DoLookup logf $nymname $ntsid
    }
}

# ----------
# ModifyNyms
# ----------
# modify nyms both in replyblocks and email address  -- actual functionality of
# these reply blocks and email addresses is not explicitly checked (by sending
# mail -- will need client) -- error is flagged if there is a failure to create
# the correct number of reply_blocks specified for a nym during a lookup is
# checked (in nympop_DoLookup function)

proc nympop_ModifyNyms {log basename nymcount email rboffset} {
    global cmdline ntspawnid entityInfo
    upvar $log logf

    puts $logf "**** MODIFY NYMS **** -------------------------------------------"
    puts $logf "     nympop_ModifyNyms {log $basename $nymcount ... }"
    puts $logf "     Adding RB's and email to $nymcount nyms:"

    for {set i 0} {$i < $nymcount} {incr i +1} {

	set index   [expr $i % $entityInfo(nymsrv,num)]
	set nymname $basename$i
	set ntsid   $ntspawnid(nymtool,$index)

	# lookup and load the appropriate nym
	nympop_DoLookup logf $nymname $ntsid
	if [catch {nymsrv_LoadNym $nymname $ntsid} errmsg] {
	    if ![string compare $errmsg "timeout"] {
		puts $logf "LOAD --- TIMEOUT --- $nymname - load failed"
	    } elseif ![string compare $errmsg "failure"] {
		puts $logf "LOAD --- FAILURE --- $nymname - load failed"
	    }
	}

	# shift reply block order for each nym and add reply blocks
	set ioffset1 [expr $i + $rboffset]
	set ioffset2 [expr $i + $rboffset + 1]
	set ioffset3 [expr $i + $rboffset + 2]
	set maipi1 [expr $ioffset1 % $entityInfo(maip,num)]
	set maipi2 [expr $ioffset2 % $entityInfo(maip,num)]
	set maipi3 [expr $ioffset3 % $entityInfo(maip,num)]

	# Add a reply block for this nym
	set msg [nymsrv_NymAddReplyBlock $ntsid $entityInfo(maip,$maipi1,name)\
		                                $entityInfo(maip,$maipi2,name)\
						$entityInfo(maip,$maipi3,name)]
	if ![string compare $msg "success"] {
	    puts $logf "     --- $nymname - reply block added"
	} elseif ![string compare $msg "failure"] {
	    puts $logf "     --- $nymname - reply block not added"
	}

	#  add private email for nym
	set msg [nymsrv_NymAddPrivateEmail $ntsid $email]
	if ![string compare $msg "success"] {
	    puts $logf "     --- SUCCESS --- $nymname - private email added"
	}

	# make sure nym is still there...
	nympop_DoLookup logf $nymname $ntsid
    }
}


# ---------
# AddTokens
# ---------
# add existing tokens to nyms (how do we verify that this has happened - nym
# server does not seem to provide this info in a lookup)

proc nympop_AddTokens {log basename nymcount SNtype}  {
    global cmdline ntspawnid entityInfo
    upvar $log logf

    puts $logf "**** EXTEND NYMS **** -------------------------------------------"
    puts $logf "     nympop_AddTokens {log $basename $nymcount }"
    puts $logf "     Adding tokens to existing nyms:"

    for {set i 0} {$i < $nymcount} {incr i +1} {
	
        
        set index [expr $i % $entityInfo(nymsrv,num)]
        set ntsid   $ntspawnid(nymtool,$index)
        
        set nymname $basename$i

	# Lookup and load the nyms
	nympop_DoLookup logf $nymname $ntsid
	if [catch {nymsrv_LoadNym $nymname $ntsid} errmsg] {
	    if ![string compare $errmsg "timeout"] {
		puts $logf "LOAD --- TIMEOUT --- $nymname - load failed"
	    } elseif ![string compare $errmsg "failure"] {
		puts $logf "LOAD --- FAILURE --- $nymname - load failed"
	    }
	}

	# Add a token to the active nym
	set msg [nymsrv_NymAddToken $ntsid $SNtype]
	if ![string compare $msg "timeout"] {
	    puts $logf "     --- TIMEOUT --- $nymname - nym not extended"
	} elseif ![string compare $msg "success"] {
	    puts $logf "     --- CHECK --- $nymname - frnscpd log for success"
	}
    }
}

# ---------
# CheckKeys
# ---------
# use tkq to check nyms' keys validity.  The keys should be valid if the nyms are
# valid and invalid/not found if the nym is disabled and not found if nym keys have been deleted.

proc nympop_CheckKeys {log basename nymcount start_index}   {
    global entityInfo ports env
    upvar $log logf

    puts $logf "**** VERIFY KEYS **** -------------------------------------------"
    puts $logf "     nympop_CheckKeys {log $basename $nymcount $start_index}"
    puts $logf "     Check keys for nyms:"

    for {set i $start_index} {$i < [expr $nymcount + $start_index]} {incr i +1} {

	set nymname $basename$i

	# Check version 1 cryption key
	keyqry_CheckKey nym $nymname 0 msg entityInfo
        foreach name [array names msg]  {
   	  if ![string compare $msg($name) "timeout"] {
	    puts $logf "     --- TIMEOUT --- $nymname - key query ($name) failed"
	  } elseif  ![string compare $msg($name) "valid"] {
	    puts $logf "     --- VALID --- $nymname - key ($name) valid"
	  } elseif  ![string compare $msg($name) "invalid"] {
	    puts $logf "     --- INVALID --- $nymname -  key ($name) invalid"
	  } elseif  ![string compare $msg($name) "notfound"] {
	    puts $logf "     --- NOT FOUND --- $nymname - key ($name) not found"
	  }
        }
    }
}

# -----------
# DisableNyms
# -----------
# disabling 'nymcount' nyms from the beginning of the created nyms list

proc nympop_DisableNyms {log basename nymcount} {
    global cmdline entityInfo ntspawnid
    upvar $log logf

    puts $logf "**** DISABLE NYMS **** ------------------------------------------"
    puts $logf "     nympop_DisableNyms {log $basename $nymcount }"
    puts $logf "     Disable nyms:"

    for {set i 0} {$i < $nymcount} {incr i +1} {

	set index [expr $i % $entityInfo(nymsrv,num)]
	set nymname $basename$i
	set ntsid   $ntspawnid(nymtool,$index)

	# Lookup and load the nym
	nympop_DoLookup logf $nymname $ntsid
	if [catch {nymsrv_LoadNym $nymname $ntsid} errmsg] {
	    if ![string compare $errmsg "timeout"] {
		puts $logf "LOAD --- TIMEOUT --- $nymname - load failed"
	    } elseif ![string compare $errmsg "failure"] {
		puts $logf "LOAD --- FAILURE --- $nymname - load failed"
	    }
	}

	# attempt to disable the nym and log appropriate message
	set msg [nymsrv_DisableNym $nymname $ntsid]
	if ![string compare $msg "timeout"] {
	    puts $logf "     --- TIMEOUT --- $basename$i - nym disable failed"
	} elseif ![string compare $msg "success"] {
	    puts $logf "     --- SUCCESS --- $basename$i - nym disabled"
	} elseif ![string compare $msg "failure"] {
	    puts $logf "     --- FAILURE --- $basename$i - nym disable failed"
	} elseif ![string compare $msg "inactive"] {
	    puts $logf "     --- INACTIVE -- disable failed (no active nym)"
	}

	# make sure the nym can no longer be looked up
	nympop_DoNotThereLookup logf $nymname $ntsid
    }
}

# --------
# DoLookup
# --------
# attempt to lookup a nym, error if lookup not successful

proc nympop_DoLookup {log nymname ntspawnid} {
    upvar $log logf

    set msg [nymsrv_LookupNym $nymname $ntspawnid]
    if ![string compare $msg "timeout"] {
	puts $logf "LOOKUP - TIMEOUT --- $nymname - lookup failed"
    } elseif ![string compare $msg "failure"] {
	puts $logf "LOOKUP - FAILURE --- $nymname - not there (should be)"
    } elseif ![string compare $msg "success"] {
	puts $logf "LOOKUP - SUCCESS --- $nymname - nym found"
    }
}

# ----------------
# DoNotThereLookup
# ----------------
# attempt to lookup a nym, error if lookup successful

proc nympop_DoNotThereLookup {log nymname ntspawnid} {
    upvar $log logf

    set msg [nymsrv_LookupNym $nymname $ntspawnid]
    if ![string compare $msg "timeout"] {
	puts $logf "LOOKUP - TIMEOUT --- $nymname - lookup failed"
    } elseif ![string compare $msg "success"] {
	puts $logf "LOOKUP - FAILURE --- $nymname - still there (shouldn't be)"
    } elseif ![string compare $msg "failure"] {
	puts $logf "LOOKUP - SUCCESS --- $nymname not found (shouldn't be there)"
    }
}

# --------------
# RemoveNymFiles
# --------------

proc numpop_RemoveNymFiles {log basename nymcount} {
    upvar $log logf
    global env network netconf

    # save current process if there is one
    if [info exists spawn_id] {	set oldspawnid spawn_id }

    for {set i 0} {$i < $nymcount} {incr i +1} {
	set nymname "$basename$i"
	spawn /bin/sh -c "\\rm $netconf($network,homedir)/$nymname.nym"
	wait
	close
	spawn /bin/sh -c "\\rm -r $netconf($network,keydir)/$nymname"
	wait
	close
    }

    # restore the old process if there was one
    if [info exists oldspawnid] { set spawn_id $oldspawnid }
}

# ----------
# DeleteNyms
# ----------

proc numpop_DeleteNyms {log basename nymcount starti deletednym} {
    upvar $log logf $deletednym dnym
    global entityInfo ntspawnid

    for {set i $starti} {$i < [expr $nymcount + $starti]} {incr i +1} {
	puts $logf "     --- DELETE --- $basename$i - from nymsdb"
	nymsrv_DeleteNym $basename$i entityInfo
	keyqry_DeleteKey $basename$i entityInfo
	set dnym([expr $i - $starti]) $basename$i
	# make sure the nym can no longer be looked up
	nympop_DoNotThereLookup logf $basename$i $ntspawnid(nymtool)
	keyqry_CheckKey nym $basename$i 0 key_status entityInfo
	foreach name [array names key_status]  {
           if {$key_status($name) != "notfound"} {
                puts $logf "ERROR -- Key ($name) for deleted nym key $basename$i returned: $key_status($name)" 
	   }
	}	
    }
}

# --------
# NukeNyms
# --------

proc nympop_NukeNyms {log basename nymcount starti nukednym} {
    upvar $log logf $nukednym nnym
    global entityInfo ntspawnid

    for {set i $starti} {$i < [expr $nymcount + $starti]} {incr i +1} {
	puts $logf "     --- NUKE --- $basename$i - from nymsdb"
	nymsrv_NukeNym $basename$i entityInfo
	keyqry_DeleteKey $basename$i entityInfo
	set nnym([expr $i - $starti]) $basename$i
	# make sure the nym can no longer be looked up
	nympop_DoNotThereLookup logf $basename$i $ntspawnid(nymtool)
	foreach name [array names key_status]  {
           if {$key_status($name) != "notfound"} {
                puts $logf "ERROR -- Key ($name) for deleted nym key $basename$i returned: $key_status($name)" 
	   }
	}	
    }
}

# -----------
# RestoreNyms
# -----------

proc nympop_RestoreNyms {log ndeleted dNyms nnuked nNyms}  {
    upvar $log logf
    upvar $dNyms dNym $nNyms nNym
    global entityInfo ntspawnid

    for {set i 0} {$i < $ndeleted} {incr i +1} {
	puts $logf "     --- RESTORE --- $dNym($i) - from deleted"
	nymsrv_RestoreNym $dNym($i) entityInfo
	keyqry_RestoreKey $dNym($i) entityInfo
	nympop_DoLookup logf $dNym($i) $ntspawnid(nymtool)
    }

    for {set i 0} {$i < $nnuked} {incr i +1} {
	puts $logf "     --- RESTORE --- $nNym($i) - from nuked"
	nymsrv_RestoreNym $nNym($i) entityInfo
	keyqry_RestoreKey $nNym($i) entityInfo
	nympop_DoLookup logf $nNym($i) $ntspawnid(nymtool)
	foreach name [array names key_status]  {
           if {$key_status($name) != "valid"} {
                puts $logf "ERROR -- Key ($name) for restored nym key $basename$i returned: $key_status($name)" 
	   }
	}	
    }
}

# ----------
# CheckMailPasswd 
# ----------

proc nympop_CheckMailPasswd {log basename nymcount SNtype} {

   upvar $log logf
   global ntspawnid
   global testpasswdfile
   global tokenInfo
 
   for {set i 0} {$i < $nymcount} {incr i +1} {
  
     set nymname $basename$i

     if {$tokenInfo($SNtype,mail) == "yes"} {
	if {[nymsrv_GetMailCert $ntspawnid(nymtool) $nymname 0 mailcert] != "success"} {
           puts $logf "ERROR: could not retrieve mailcert for nym $nymname"
        } else {
           if {[nymsrv_CheckMailPasswd $nymname $mailcert(passwd)] != "success"} {
	     puts $logf "ERROR: mail password for nym $nymname is invalid"   
           } else {
#	     puts $logf "SUCCESS: mail password for nym $nymname is valid"   
	     puts $logf "Please run check_mail.exp to check validity of dumped mail password"
           }  
        }
      } else {
        if {[nymsrv_GetMailCert $ntspawnid(nymtool) $nymname 0 mailcert] == "success"} { 	
           puts $logf "ERROR: Nym created with SN type $SNtype should not have valid mailcert"
        } else {
           puts $logf "SUCCESS: Nym created with SN type $SNtype does not have valid mailcert"
        }
      }
   }
}
   
# ----------
# CheckTokenTypes
# ----------
#ensure that nyms can only be created/upgraded with appropriate token types

proc nympop_CheckTokenTypes {log basename ntspawnid} {

    upvar $log logf
    upvar $ntspawnid ntsid
    global num_combos combo

    set spid $ntsid(nymtool)

    # redeem sufficient SNs before checking so don't end up with obscene number of tokens

    set SNs_required 2 
    set SNtypes_required(0) 900
    set SNtypes_required(1) 901
    set SNtypes_required(2) 902
    set SNtypes_required(3) 903

    for {set i 0} {$i<$SNs_required} {incr i +1} {
      foreach name [array names SNtypes_required] {
        if {[toksrv_RedeemSN $spid $SNtypes_required($name)] != "success"} {
           puts $logf "ERROR - could not redeem serial number of type $SNtypes_required($name)"
        }
      }
    }


    for {set i 0} {$i <  $num_combos} {incr i +1} {
        puts $logf "\nChecking adding $combo($i,add) token to nym created w/ $combo($i,create) token:"
	set retmsg [check_token_combo tokencheck_$basename$i $combo($i,create) $combo($i,add) ntsid logf]
	if {$retmsg != $combo($i,result)} {
           puts $logf "ERROR: creating nym with $combo($i,create) token and extending with $combo($i,add) token - expecting $combo($i,result) - got $retmsg"
        } else {
           puts $logf "SUCCESS: expecting $combo($i,result) - got $retmsg"
        }
    }
}

# ------------
# check_token_combo - check a given combination of token types used to create/upgrade
# ------------
proc check_token_combo {nymname create_type add_type ntspawnid log} {

    upvar $log logf
    upvar $ntspawnid ntsid
    
    puts $logf "Creating nym $nymname with token type $create_type"
    set spid $ntsid(nymtool)
    if {[nymsrv_CreateNym $nymname $spid $create_type] != "success"} {
        puts $logf "Could not create nym $nymname with token type $create_type"
        set retmsg "failure"
    } else {
        # wait one second so nym file can be saved - needs to be loaded by process seperate from the one that created the nym
        TimeWait 1
        set spid $ntsid(nymtool)
        if {[nymsrv_LoadNym $nymname $spid] != "success"} {     
            puts $logf "ERROR - Could not load nym $nymname"
            set retmsg "could not load nym" 
        } else {
            puts $logf "Extending nym $nymname with token type $add_type"
            if {[nymsrv_NymAddToken $spid $add_type] != "success"} {
                puts $logf "Could not extend nym $nymname"
                set retmsg "failure"
            } else {
                set retmsg "success"
            }
        }
     }
    

    return $retmsg
}
 
    
