# Some basic tcl procedures

#$Id: stdproc.tcl,v 1.3 2001/04/19 19:18:01 alainc Exp $

source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/defs.tcl
source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/error.tcl

######################################################################
#
# streq - String Equality
# -----------------------
# return 1 if both given strings are equal, 0 otherwise
#
# PARMS + str1 : a string
#       + str2 : another string
#
######################################################################
proc streq {str1 str2} {
   return [expr ![string compare $str1 $str2]]
}

######################################################################
#
# lc - Line Count
# ---------------
# return a line count of a file
#
# PARMS + file : filename of desired file to be counted
#
#######################################################################
proc lc {file} {
   spawn wc $file
   expect {
     -re "(\[0-9]+).*(\[0-9]+).*(\[0-9]+).*$file"
     {
       return $expect_out(1,string)
     }
   }
}

#######################################################################
#
# std_ParseArgument
# -----------------
# Find the given argument in the command line and set its value
# appropriately depending if 'type' is "bool" or "val"
#
# PARMS + log  : logfile
#       + parm : outputted parameter
#       + type : type of parameter this is (must be ["val"|"bool"])
#       + argc : argument count
#       + argv : argument command line
#
#######################################################################
proc std_ParseArgument {log parm type argc argv} {
   upvar $log logf
   global ERR

   set func "std_ParseArgument"
   set found 0
   set opt "--$parm"

   for {set i 0} {$i < $argc} {incr i +1} {
        if [streq [lindex $argv $i] $opt] {
             set found 1
             if [streq $type "val"] {
	        incr i +1
                if {$i < $argc} {
	           return [lindex $argv $i]
                } else {
                   err logf $func "$parm : Parameter value missing"
		   return $ERR(FATAL)
                }
             } elseif [streq $type "bool"] {
                return 1
             } else {
                err logf $func "$parm : Invalid parameter type received"
             }
        }
   }
   if {!$found} {
      return $ERR(NOT_FOUND)
   }
}

#######################################################################
#
# ******** NOTE: This function is essentially considered depracated
# ********       for the pure and simple reason that it doesn't bloody
# ********       work right.  Iterate through a list of parameters
# ********       and use the std_ParseArgument list manually
#
# std_ParseArgumentsList
# ----------------------
# Given two lists of argument lists, parse them appropriately and set
# variables equivalent to their names with their values.
#
# PARMS + vlist : list of arguments for which values will be specified
#                 on the command line as follows...
#
#                         --vlist_elm_n vlist_elm_n_value
#
#                 for each element in vlist.  Those not specified on the
#                 command line will be set to the defaults specified in
#                 dlist.
#
#       + blist : list of arguments which are boolean and will be set to
#                 true if specified on the command line as follows...
#
#                         --blist_elm_n
#
#                 Variables not specified on the command line will be set
#                 to FALSE by default
#
#       + dlist : list of default values for arguments in vlist
#
##########################################################################
proc std_ParseArgumentsList {log vlist blist dlist argc argv} {
  upvar $log logf
  global ERR

  set func "std_ParseArgumentsList"

  # Check for inconsitencies
  if {[llength $vlist] != [llength $dlist]} {
     err logf $func "Inconsistent mapping of default values to variables"
     return $ERR(FATAL)
  }
  
  # Set value parameter defaults
  for {set i 0} {$i < [llength $vlist]} {incr i +1} {
      set [lindex $vlist $i] [lindex $dlist $i]
  }

  # Set boolean paramter defaults (to FALSE)
  for {set i 0} {$i < [llength $blist]} {incr i +1} {
      set [lindex $vlist $i] 0
  }

  # Parse vlist and set values for parameters
  foreach parm $vlist {
        set opt "--$parm"
        for {set i 0} {$i < $argc} {incr i +1} {
            if [streq [lindex $argv $i] $opt] {
	         incr i +1
                 if {$i < $argc} {
                      set $parm [lindex $argv $i]
                      break;
                 } else {
                      err $func "$parm: Parameter value missing.\n"
		      return $ERR(FATAL)
                 }
            }
       }
  }

  # Parse blist and set values for parameters
  foreach parm $blist {
       set opt "--$parm"
       for {set i 0} {$i < $argc} {incr i +1} {
            if [streq [lindex $argv $i] $opt] {
	         set $parm $TRUE
            }
       }
  }
}
