#$Id: toksrv.tcl,v 1.4 2001/04/19 19:18:01 alainc Exp $

source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/defs.tcl

#############################################################################
#                                                                           #
#  FUNCTIONS THAT WORK THROUGH NYMTOOL TO WORK (require a nymtool spawn id) #
#                                                                           #
#############################################################################

# --------
# RedeemSN
# --------
# Attempt to redeem a serial nymber using nymtool
#
# Assumptions: - 'ntspawnid' is a valid nymtool spawn id
#
# Return Value: - "timeout" if operation times out
#               - "success" for successful lookup

proc toksrv_RedeemSN {ntspawnid SNtype} {
    global cmdline spawn_id tokenInfo

    # make nymtool the current process
    set spawn_id $ntspawnid

    send "token redeem snid$SNtype\r"
    expect {
	timeout	                   { set retmsg "timeout" }
	"Serial number type id $SNtype" { set retmsg "success" }
        "Unable" 		   { set retmsg "failure" }
        "Invalid serial number"    { set retmsg "failure" }
    }
    expect $cmdline(nymtool)


    return $retmsg
}

# ---------
# get_token_index
# ---------
# Find the token index in the nymtool tokencache corresponding to the required serial number type

proc get_token_index {ntspawnid SNtype} {

    global cmdline spawn_id tokenInfo

    # save current process if there is one
    if [info exists spawn_id] {	set oldspawnid spawn_id }

    set spawn_id $ntspawnid

    send "token list\r"
    expect {
	timeout	                   { set retmsg "failure" }
	-re "(\[0-9]*) ($tokenInfo($SNtype,label)|$tokenInfo($SNtype,typeid))" { set retmsg $expect_out(1,string) }
        $cmdline(nymtool)          { set retmsg "failure" }
    }
     
    if {$retmsg != "failure"} {expect $cmdline(nymtool)}

    # restore the old process if there was one
    if [info exists oldspawnid] { set spawn_id $oldspawnid }

    return $retmsg
}    

# ---------
# GetTokenLabels
#-----------
# Download token labels from niqs 

proc GetTokenLabels {TokenInfo logf network} {

  upvar $TokenInfo Info
  upvar $logf log
  global spawn_id cmdline

  netinfo_niqs_test_connect log $network niqs_spid

  set spawn_id $niqs_spid

  send "kqs\r"
  expect $cmdline(niqs_test) 
  send "tokeninfo\r"
  expect {
	-re "Type ID number\[\t ]*:  (\[0-9]*)\r\n" {
             puts $log "Type ID is $expect_out(1,string)"
             set snid [GetSNIDfromTokenID $expect_out(1,string)]
             puts $log "SNID for this type id is $snid"
             expect {
                  -re "Token label\[\t ]*:  (\[a-z|A-Z|0-9|+-| ]*)\r\n"
                      {set Info($snid,label) $expect_out(1,string)
                       puts $log "SNID $snid: token label is $Info($snid,label)"}
                  timeout {puts $log "Error - Could not get token label for SNID $snid"}
             }
             exp_continue
        }
        $cmdline(niqs_test)
  }
  send "quit\r"
}

# ---------
# GetSNIDfromTokenID
#-----------
# Given a token ID type, will return the corresponding SN type that generated it

proc GetSNIDfromTokenID {tokenID} {

  global tokenInfo SNType

  foreach name [array names SNType] {
    if {$tokenInfo($SNType($name),typeid) == "$tokenID"} {
      return $SNType($name)
    }
  }

  return 0
}


# ----------
# SaveTokens
# ----------
# Will save tokens to tokencache
# Tokens from cache only get loaded (and do so automatically if TokenCache exists) when nymtool starting
# This should only be used when trying to spend the same token twice.
# This procedure should be called before Launching other nymtool processes

proc SaveTokens {SNtype logf entityInfo} {

  upvar $logf log
  upvar $entityInfo eInfo 
  global passphrase passphrase_prompt netconf network spawn_id

# first remove the old tokencache
  RemoveTokenCache $network

  if {[Spawn_nymtool_process $netconf($network,homedir)/$network.conf dummy_spid dummy_sp_index $passphrase $passphrase_prompt] != "success"} {
    puts $log "ERROR - Could not spawn nymtool process for saving tokens"
    return failure
  }

  if {[toksrv_RedeemSN $dummy_spid(dummy_sp_index) $SNtype] != "success"} {
    puts "Dummy Spawn id is $dummy_spid(dummy_sp_index)"
    puts "Spawn id is $spawn_id"
    puts $log "ERROR - Could not redeem SN of type $SNtype when trying to save tokens"
    nymtool_quit $dummy_spid(dummy_sp_index)
    return failure
  }

  set spawn_id $dummy_spid(dummy_sp_index)
  send "token save\r"
  expect {
    timeout {set return_value timeout}
    "Unable to save token cache" {set return_value failure}
    "Token cache saved to" {set return_value success}
  }

  nymtool_quit $dummy_spid(dummy_sp_index)
  return $return_value
}
  
#---------
# RemoveTokenCache
#---------

proc RemoveTokenCache {network} {

  global netconf

  spawn /bin/sh -c "\\rm $netconf($network,homedir)/TokenCache"
  wait
  close
}

#---------
# WgetSN - redeems serial number directly from scalper (w/o using nymtool and returns in var SN
#---------

proc WgetSN {scalperURL SN error} {
 
  
  upvar $error err 
  upvar $SN serialnum
    
  set SNfile "/tmp/serialnum"

  spawn /bin/sh -c "wget -O $SNfile \"$scalperURL\""
  expect {
    "Host not found" {SetError err "Could not connect to scalper $scalper_demourl"
                      return 1}
    "Not found" {SetError err "Could not retrieve token from scalper $scalper_demourl"
                 return 1}
    "saved" {}
  }

  if {[gets [open $SNfile r] line] != -1} {
    set serialnum $line
  } else {
    SetError err "Could not read Serial Number from file $SNfile"
    return 1
  }

  return 0
}
    
  
