#$Id: zkip.tcl,v 1.2 2001/04/19 19:18:01 alainc Exp $

source $env(ZK_FRTOOLBOX_ROOT)/tools/server-bvt/inc/defs.tcl

#############################################################################
#                                                                           #
#  ZKIP FUNCTIONS  #
#                                                                           #
#############################################################################


# --------
# zkipTelnet - telnets to host and port and ensures proper connection
# --------

proc zkipTelnet {error host port spid} {

  upvar $error err
  upvar $spid telnet_spid
  global spawn_id

  spawn /bin/sh -c "telnet $host $port"
  set telnet_spid $spawn_id
  expect {
    timeout {SetError err "Timeout trying to telnet to $host port $port"
             return 1}
    "Connection refused" {SetError err "Connection to $host port $port refused"
                          return 1}
    "0 0.1.0 220 NODATA BINARY 0" {set telnet_spid $spawn_id}
  }

  return 0
}

# ------
# zkipQuit - quits telnet session a la zkip
# ------

proc zkipQuit {error spid} {

  upvar $error err
  global spawn_id

  set spawn_id $spid
  send "0 0.1.0 quit nodata binary 0\r"
  expect {
    timeout {SetError err "Timeout trying to quit telnet session"
             return 1}
    "Connection closed by foreign host" {}
  }

  close
  return 0
} 
  
# --------
# zkipSNCapability
# --------
# Checks capability of serial number using zkip command

proc zkipSNCapability {host SN capability error} {

    upvar $error err
    upvar $capability capab
    global ports spawn_id 

    if {[zkipTelnet err $host $ports(toksrv) telnet_spid]} {
      SetError err ""
    } else {
      set spawn_id $telnet_spid
      send "1 5.1.0 capability SERIALNUMBER BASE64 0 $SN\r"
      expect {
	timeout	                   {SetError err "Timeout getting SN capability for serial number $SN"
                                    return 1}
	"1 5.1.0 501 NODATA BASE64 0 \"Serial number is not in active database\"" {SetError err "Timeout getting SN capability for serial number $SN" 
                                                                                  return 1}
        "1 5.1.0 220 CAPABILITYCODE BASE64 208 \"Capability code\"" {}
      }
    }

    if {[zkipQuit err $telnet_spid]} {
      SetError err ""
      return 1
    } else {
      return 0
    }
}
