#$Id: ns-general.tcl,v 1.2 2001/04/19 19:18:15 alainc Exp $

# --------
# DumpSpawnids - dumps a list of active spawnids (for debugging)
# --------

proc DumpSpawnids {spawnid_array} {

    upvar $spawnid_array spid

    foreach name [array names spid] {
      puts "$name : $spid($name)"
    }
}

# ----------
# TimeWait - hack script that waits for specified number of seconds
# ----------

proc TimeWait {num_secs} {

  global timeout

  set timeout_save $timeout
  set timeout $num_secs
  expect "EXPOSWINWORLDSERIES"
  set timeout $timeout_save

}

# ----------
# ParseConfig - parses a config file and places all values read from config file in a specified array.  
# 		Items in config file must be of form A = B (i.e. NymServer = nymsrv.freedom.net)
# ----------

proc ParseConfig {config_file ConfigContext} {

  upvar $ConfigContext context

  if [catch {open $filename} msg] {
    error $msg
  }

  set i 0

  while 1 {
    if {[gets $ConfigFile line] == -1} break
      if {[regexp "^(.*) = (.*)$" $line ignore param value]} {
        if {[string index $line 0] != "#"} {
          set context($param) $value
          incr i
        }
      }
  }

  close $ConfigFile

  return 0

}
    

# --------
# LoadArgs
#---------
# Load values from command line into the ctx() structure
#
#      ctxfields : list of fields needed in the ctx structure
#      argc      : number of command line arguments
#      argv      : list of command line arguments
  

proc LoadArgs {ctxfields argc argv} {


    foreach fieldnm $ctxfields {
        set opt "--$fieldnm"
        for {set j 0} {$j < $argc} {incr j} {
            if [streq [lindex $argv $j] "--help"] {
                uplevel 1 set ctx(help) 1
                return 0
            } elseif [streq [lindex $argv $j] $opt] {
                incr j
                if {$j < $argc} {
                    set val [lindex $argv $j]
                    foreach f $ctxfields {
                        if [streq $val "--$f"] {
                            error "$fieldnm : Parameter value missing\n"
                        }
                    }
                    uplevel 1 set ctx($fieldnm) $val
                    break;
                } else {
                    error "$fieldnm : Paramter value missing\n"
                }
            }
        }
    }
    return 0
}


#-----------
# UnsetBlankArgs - This is a bit of a hack - Will unset any arguments that are existing but blank
#-----------

proc UnsetBlankArgs {ctx} {

  upvar $ctx context

  foreach field [array names context] {
    if {$context($field) == ""} {
      unset context($field)
    }
  }

  return 0
}

#------------
# CheckArgSet - Check that needed arguments are set within a context - exit if not
#------------

proc CheckArgSet {ctx arglist} {

  upvar $ctx context

  foreach arg $arglist {
    if {![info exists context($arg)]} {
      error "$arg not defined"
    }
  } 

  return 0
}

#----------
# LoadFlags
#----------
# Load flags from command line into the ctx() structure - 1 for set, 0 for not set

proc LoadFlags {ctxflags argc argv} {


    foreach fieldnm $ctxflags {
        set opt "--$fieldnm"
        for {set j 0} {$j < $argc} {incr j} {
            if [streq [lindex $argv $j] "--help"] {
                uplevel 1 set ctx(help) 1
                return 1
            } elseif [streq [lindex $argv $j] $opt] {
                uplevel 1 set ctx($fieldnm) 1
                break;
            }
        }
    }
    return 0
}


# --------
# DumpContext - dumps all values in specified array
# --------

proc DumpContext {context} {

   upvar $context Context

   foreach name [array names Context] {
     puts "$name = $Context($name)"
   }

   return 0
}


# ---------
# CheckArrayFor - checks to make sure a given value is in an array
# ---------

proc CheckArrayFor {Array String} {

  upvar $Array A

  foreach name [array names A] {
    if {"$String" == "$A($name)"} {
      return 1
    }
  }

  return 0
}

# String equality function
proc streq {str1 str2} {
    return [expr ![string compare $str1 $str2]]
}


#GetTimeMXXSXX - returns time in minutes and seconds - for coarse timing < 1 hour
proc GetTimeInSecs {} {

  set time [timestamp -format "M%MS%S"]
  scan $time "M%dS%d" minutes seconds
  set total_time [expr (60 * $minutes) + $seconds]
  return $total_time
}

proc TimeDiffMMSS {start end} {

  if {$end<$start} {
    set end [expr $end + (24*3600)]  
  }

  return [expr $end - $start]
}

proc SetError {error msg} {

  upvar $error err

  set err "[info level -1] : $msg"
}

#--------
# putsFile - writes to stdout and file
#--------

proc putsFile {file msg} {

  upvar $file logfile

  puts $logfile "$msg"
  puts "$msg"
}


#---------
# Vputs - puts with verbosity control
#---------

proc Vputs {level msg} {

  global verbosity

  if {[info exists verbosity] == 0} {
    set verbosity 1
  }

  if {$level <= $verbosity} {
    puts "$msg"
  }

  return 0
}
  
