#$Id: ns-laip.tcl,v 1.2 2001/04/19 19:18:16 alainc Exp $

source $env(NETSANITY_ROOT)/inc/ns-defs.tcl
source $env(NETSANITY_ROOT)/inc/ns-general.tcl


#----------------------
# connect_laip_cmd_port - telnets localhost laip command port and returns spawn id of telnet process
#----------------------
proc connect_laip_cmd_port {} {

  global ports spawn_id

  spawn telnet localhost $ports(cmd)
  set spid $spawn_id
  expect {
       timeout          {error "Timeout trying to connect to laip command port ($ports(cmd))"}
       "Local Aip Command Port"  {}
       "Command port already in use"  {error "Laip Command port on localhost already in use"}
       "Unable to connect to remote host"  {error "Unable to connect to laip command port $ports(cmd)"}
  }

#  send "db\r"
  set timeout 1; expect "WAIT WAIT"; set timeout 10

  return $spid
}

#------------------
# quit_laip - destroys routes, clears rules and exits
#------------------
proc quit_laip {} {

  send "nr\r"
  expect "Attempt to clear rules succeeded."
  send "da\r"
  expect "ROUTE DESTROY sent on all valid ACIs."
  send "quit\r"
  wait -nowait
}
  
  
#----------------
# unblock_port - unblocks port $port_num on route defined by ACI $ACI
#----------------
proc unblock_port {port_num ACI} {

   send "up $port_num\r"
     expect {       
      timeout {error "Timeout trying to unblock port $port_num"}
      -re "Shim will unblock port .$port_num. on route "   {}
      -re "Port .$port_num. is not blocked" {}
     }
   return 0
}

#----------------
# unblock_all_ports - unblocks all core server ports on ACI 0
#----------------
proc unblock_all_ports {} {

   set ports [list 51103 51104 51107 51109 51114 51115]
   foreach port $ports {
     send "up $port\r"
       expect {       
        timeout {error "Timeout trying to unblock port $port"}
        -re "Shim will unblock port .$port. on route "   {}
        -re "Port .$port. is not blocked" {}
       }
   }
   return 0
}

#----------------
# block_port - blocks port $port_num on route defined by ACI $ACI - 
#----------------
proc block_port {port_num ACI} {

   send "bp $port_num\r"
     expect {
       timeout        {error "Timeout trying to block port $port_num"}
       -re "Shim will block port .$port_num. on route .$ACI"   {}
       -re "Port .$port_num. already "   {}
     }
   return 0
}

#-----------------
# get_aip_id_from_ip - returns the entity id of the aip at a given ip address (for being able to create routes w/ last hop a core server based on ip address of server)
#-----------------
proc get_aip_id_from_ip  {ip entityInfo}   {

    upvar $entityInfo eInfo

    set aip_id 0

    for {set i 0}  {$i<$eInfo(aip,num)}  {incr i +1}   {
        if {$eInfo(aip,$i,ipaddr) == $ip}   {
            set aip_id $eInfo(aip,$i,id)
            break
        }
    }
    return $aip_id
}

#-----------------
# get_aip_id_from_name - returns the entity id of the aip with name 'name'
#-----------------
proc get_aip_id_from_name  {name entityInfo}   {

    upvar $entityInfo eInfo

    set aip_id 0

    for {set i 0}  {$i<$eInfo(aip,num)}  {incr i +1}   {
        if {$eInfo(aip,$i,name) == $name}   {
            set aip_id $eInfo(aip,$i,id)
            break
        }
    }
    return $aip_id
}

#-----------------
# get_aip_num_from_name - returns the index of aip with name 'name' in the entityInfo array
#-----------------
proc get_aip_id_from_name  {name entityInfo}   {

    upvar $entityInfo eInfo


    for {set i 0}  {$i<$eInfo(aip,num)}  {incr i +1}   {
        if {$eInfo(aip,$i,name) == $name}   {
            set aip_num $i
            break
        }
    }
    return $aip_num
}

#-----------------
# get_aip_ip_from_id - returns the ip address of aip specified by id  (for being able to check if given aip has core server on it)
#-----------------
proc get_aip_ip_from_id {id entityInfo} {

  upvar $entityInfo eInfo

     for {set i 0} {$i<$eInfo(aip,num)} {incr i +1}  {
        if {$eInfo(aip,$i,id) == $id}  {
            set aip_ip $eInfo(aip,$i,ipaddr)
            break
        }
     }
     return $aip_ip
}


#-----------------
# get_aip_name_from_id - returns the aip entity name based on its id
#-----------------
proc get_aip_name_from_id  {id entityInfo}   {
  upvar $entityInfo eInfo

    for {set i 0} {$i<$eInfo(aip,num)} {incr i +1}  {
        if {$eInfo(aip,$i,id) == $id}  {
            set aip_name $eInfo(aip,$i,name)
            break
        }
    }
    return $aip_name

}

#--------------
# check_if_coreaip - returns 'yes' if given aip is a coreaip (based on if it is called host5 or host6 for now)
#-------------

proc check_if_coreaip {id entityInfo} {

   upvar $entityInfo eInfo
   global spawn_id

   set aip_name [get_aip_name_from_id $id eInfo] 
   if {[regexp "core-zks-\[0-9]*" $aip_name ignore]} {
       return yes
   } elseif {($aip_name == "host5") || ($aip_name == "host6")} {
       return yes
   } else {
       return no
   }
   
}


#-----------------
# create_specific_coreaip_route - creates specified route and blocks shitload of ports
#-----------------
proc create_specific_coreaip_route {route num_hops spid} {

  global ports

  if [catch {create_single_route $route $num_hops 0 $spid} ACI] {
    error "$ACI"  
  }
  
  if {$ACI != "DESTROYED"} {
    set server(0) nymsrv
    set server(1) toksrv
    set server(2) keyqry
    set server(3) niqs
    set server(4) nmta
    set server(5) pop

    foreach name [array names server] {
       if [catch {block_port $ports($server($name)) $ACI} ret] {
           error "$ret"
       } 
    }
 }

 return $ACI

}


#----------------
# create_coreaip_routes - creates unauthenticated route through coreaip to all  nym servers, kqds, token servers and niqs  
#----------------

proc create_coreaip_routes {entityInfo coreaip} {

	upvar $entityInfo eInfo
        global ports
 
     #list of servers for which you want to route traffic through coreaip 
        set server(0) nymsrv
        set server(1) toksrv
        set server(2) keyqry
        set server(3) niqs
        set server(4) nmta
        set server(5) pop

# create a route ending in a coreaip

        set coreaip_num [get_aip_num_from_name $coreaip]
        if [catch {clear_rules} ret] {
          error "$ret"
        }
        if {$eInfo(coreaip,num) <= $coreaip_num} {
          error "ERROR - There are only $eInfo(coreaip,num) coreaips in network -- cannot make route through coreaip $coreaip_num"
        } else {
          if [catch {set_exit_hop $eInfo(coreaip,$coreaip_num,id)} ret] {		
            error "$ret"
          }
          if [catch {create_route} ACI] {
            error "$ACI"
          }
          if {$ACI == "DESTROYED"} {
                error "Could not create route ending at coreaip $eInfo(coreaip,$coreaip_num,name)"
          } else {
        # now block packets going to any of the servers mentioned above 
                foreach name [array names server] {
                   if [catch {block_port $ports($server($name)) $ACI} ret] {
                     error "$ret"
                   }
                }
          }
        }
        return 0
}
 

#-------------------
# clear_rules - clear all the rules associated with creating a route
#-------------------
proc clear_rules  {}  {


    send "nr\r"
    expect {
           timeout  {error "Timeout trying to clear rules (nr)"}
           "Attempt to clear rules succeeded."  {}
    }
    return 0
}

#--------------------
# set_exit_hop - set exit hop to aip with specified id - 
#--------------------
proc set_exit_hop {id}  {
# specify the last hop

    send "se $id\r"

    expect {
           timeout  {error "Timeout trying to set exit hop to id $id"}
           -re "Set exit hop .$id. succeeded."  {}
           -re "Set exit hop .$id. failed."  {error "Could  not set exit hop set to id $id"}
    }
    return 0
}


#---------------------
# set_num_hops - set the number of hops in given route 
#---------------------
proc set_num_hops {num_hops} {

  send "sh $num_hops\r"
  expect {
           timeout  {error "Timeout trying to set number of hops"}
           -re "Number of hops .$num_hops. set."  {}
  }
  return 0
}
  
#--------------------
# set_next_hop - set next hop in route 
#--------------------
proc set_next_hop {id} {

  send "sp $id\r"
       expect {
           timeout  {error "Timeout trying to set next hop"}
           -re "Set next hop .$id. succeeded."  {}
           -re "Set next hop .$id. failed."  {error "Could not set next hop to id $id"}
       }
  return 0
}
 

#--------------------
# unauth_route - unauthenticates route (needs to be called before you create route)
#--------------------
proc unauth_route {} {
# unauthenticate the route

        Vputs 1 "Setting route to unauthenticated"
        send "sn\r"
        expect {
               timeout {error "Could not unauthenticate route"}
               "Current Nym set to \"Anonymous User (unauthenticated routes only"  {}
        }
    return 0
}

#---------------------
# create_route - create a route, based on any previously specified rules.  returns the ACI of route if successful
#---------------------
proc create_route  {}   {

   send "cr\r"
   expect {
           timeout  {error "Timeout during route creation"}
           -re "ROUTE CREATE acked on ACI (\[0-9]*)"  {set ACI $expect_out(1,string)}

           -re "ROUTE DESTROY sent on ACI (\[0-9]*)"  {error "Could not create route"}
           "No routes available for the set or rules"     {error "Route could not be created"}
   }
   return $ACI
}


#----------------------
# destroy_route - destroys route specified by ACI $ACI 
#----------------------
proc destroy_route {ACI}  {


        send "dr $ACI\r"
           expect {
                   timeout {error "Timeout trying to destroy route with ACI $ACI"}
                   "ROUTE DESTROY sent on ACI $ACI" {}
                  }
	return 0
}

#----------------------
# destroy_coreaip_route - destroys route specified by ACI $ACI 
#----------------------
proc destroy_coreaip_route {ACI}  {

   global ports

     #list of servers for which you want to route traffic through coreaip
        set server(0) nymsrv
        set server(1) toksrv
        set server(2) keyqry
        set server(3) niqs
        set server(4) nmta
        set server(5) pop

   foreach name [array names server] {
     if [catch {unblock_port $ports($server($name)) $ACI} ret] {
       error "$ret"
     }
   }
   if [catch {destroy_route $ACI} ret] {
     error "$ret"
   }

   return 0
}

#-----------------------
# block_packets - adds shim rule to block packets on a given port of a given hop (specified by ip addr)
#-----------------------
proc block_packets {aci ip port} {

    send "sb $aci $ip $port\r"
    expect {
         -re "Shim rule add $ip:$port to route .$aci. succeeded." {puts "Shim rule add $ip:$port to route $aci succeeded."} 
         -re "Shim rule add $ip:$port to route .$aci. failed." {error "Shim rule add $ip:$port to route $aci FAILED"}
    }
    return 0

}

#-----------------------
# unblock_packets - deletes shim rule on a given port of a given hop (specified by ip addr) 
#-----------------------
proc unblock_packets {ip port} {

    send "su $ip $port\r"
    expect {
        "Shim rule delete $ip:$port succeeded." {}
        "Shim rule delete $ip:$port failed." {error "Shim rule delete $ip:$port FAILED."}
    }
    return 0
}

#-----------------------
# set_nym - sets nym for a given route (need to set nym before creating route) - returns 1 if successful, 0 otherwise
#-----------------------
proc set_nym {nym_name} {
# specify a nym (used for a given route)
   
   Vputs 1 "Setting nym for route to $nym_name"
   send "sn $nym_name\r"
   expect {
       timeout        {error "Timeout trying to set nymto $nym_name"}
       -re "Current Nym set to .$nym_name."   {}
       "Requested nym $nym_name not found" {error "Could not find saved keys for nym $nym_name"}
   }
   return 0
}

#---------------------
# extract_hops - given a route in the form '<num_hops> <aip_id1> ... <aip_idn> will return an array of hops (hop(0) is aip_id1 hop(1) is aip_id2...)
#---------------------

proc extract_hops {single_route num_hops hop_array} {

   upvar $single_route route
   upvar $hop_array hop

	
#  not most efficient way but easiest for me for now
# extract all the hop ID info

   if {$num_hops == 1}  {
      scan $route "%d %d" dummy hop(0)
   } elseif {$num_hops == 2} {
      scan $route "%d %d %d" dummy hop(0) hop(1)
   } elseif {$num_hops == 3} {
      scan $route "%d %d %d %d" dummy hop(0) hop(1) hop(2)
   } elseif {$num_hops == 4} {
      scan $route "%d %d %d %d %d" dummy hop(0) hop(1) hop(2) hop(3)
   } elseif {$num_hops == 5} {
      scan $route "%d %d %d %d %d %d" dummy hop(0) hop(1) hop(2) hop(3) hop(4)
   } else {
      error "Invalid number of hops on route $route_num or non-existant route"
   }
 

   return 0
}



#----------------------
# get_routes - using fr_listroutes, returns an array (route) of all 'num_hops' hop routes.  
# The route array is in the following format:
# route(num_routes) - number of routes in the array
# route($i) - route $i in form : <num_hops> <id_hop1> <id_hop2> ... <id_hopn>
# route($i,num_hops) - number of hops in route $i
# route($i,id) - same as route($i) but without the <num_hops> at the beginning
# route($i,names) - lists aip names instead of ids
# If route already contains a list of routes, the new routes will be appended as new entries in the array
# If you want to start with a fresh array, you can initialize an array by using 'unset <array>'
#----------------------
proc get_routes {num_hops route_array entityInfo} {

   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)
     
   if {$num_hops == 1} { 
     if [catch {get_one_hop_routes route eInfo $start_route} ret] {
       error "$ret"
     }
   } elseif {$num_hops > 5} {
	error "Invalid number of hops: $num_hops"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
        expect {
            -re "(\[0-9| ]*)end"   {set route($route(num_routes)) $expect_out(1,string)
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         scan $route($i) "%d " route($i,num_hops)

         if [catch {extract_hops route($i) $route($i,num_hops) hop} ret] {
           error "$ret"
         }
         for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
               append route($i,ids) "$hop($j) "
               append route($i,names) [get_aip_name_from_id $hop($j) eInfo]
               append route($i,names) " "
         }
       }
   }

   return 0

}

proc get_routes_ending_in {last_hop num_hops route_array entityInfo} {

   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)

   if {$num_hops == 1} { 
	set route($start_route) "1 $last_hop" 
	set route($start_route,num_hops) 1 
	set route($start_route,ids) $last_hop 
	set route($start_route,names) [get_aip_name_from_id $last_hop eInfo] 
        set route(num_routes) [expr $route(num_routes) + 1]
   } elseif {$num_hops > 5} {
	error "Invalid number of hops: $num_hops"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
        expect {
            -re "(\[0-9| ]*) $last_hop end"   {set route($route(num_routes)) "$expect_out(1,string) $last_hop"
                                               set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         scan $route($i) "%d " route($i,num_hops)

         if [catch {extract_hops route($i) $route($i,num_hops) hop} ret] {
           error "$ret"
         }
         for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
             append route($i,ids) "$hop($j) "
             append route($i,names) [get_aip_name_from_id $hop($j) eInfo]
             append route($i,names) " "
         }
       }
   }
 
   return 0

}

proc get_routes_with {hop num_hops hop_index route_array entityInfo} {

   upvar $route_array route
   upvar $entityInfo eInfo
   
   set not_finished 1
   if {[info exists route(num_routes)] == 0} {
	set route(num_routes) 0
   }
   set start_route $route(num_routes)

   if {$num_hops == 1} { 
	set route($start_route) "1 $hop" 
	set route($start_route,num_hops) 1 
	set route($start_route,ids) $hop 
	set route($start_route,names) [get_aip_name_from_id $hop eInfo] 
        set route(num_routes) [expr $route(num_routes) + 1]
   } elseif {$num_hops > 3} {
	error "Invalid number of hops: $num_hops"
   } elseif {($hop_index != "all") && ($hop_index > $num_hops)} {
        error "Cannot have hop_index ($hop_index) greater than num_hops ($num_hops)"
   } else {
      spawn /bin/sh -c "fr_listroutes --NHops $num_hops --niqs.hostname $eInfo(niqs,0,ipaddr)"
      while {$not_finished == 1} {
       if {$hop_index == 1} {
        expect {
            -re "(\[0-9] $hop \[0-9| ]*) end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {$hop_index == 2} {
        expect {
            -re "(\[0-9] \[0-9]+ $hop \[0-9| ]*)end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {$hop_index == 3} {
        expect {
            -re "(\[0-9| ]* $hop) end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       } elseif {"$hop_index" == "all"} {
        expect {
            -re "(\[0-9| ]* $hop \[0-9| ]*)end"   {set route($route(num_routes)) "$expect_out(1,string)"
                                    set route(num_routes) [expr $route(num_routes) + 1] }
            eof           {set not_finished 0}
        }
       }
      }

      for {set i $start_route} {$i<$route(num_routes)} {incr i +1}  {
# extract the number of hops in the route
         scan $route($i) "%d " route($i,num_hops)

         if [catch {extract_hops route($i) $route($i,num_hops) temp_hop} ret] {
           error "$ret"
         }
         for {set j 0} {$j<$route($i,num_hops)} {incr j +1} {
            append route($i,ids) "$temp_hop($j) "
            append route($i,names) [get_aip_name_from_id $temp_hop($j) eInfo]
            append route($i,names) " "
         }
       }
   }
   return 0

}

proc get_one_hop_routes {route_array entityInfo start_route} {

   upvar $route_array route
   upvar $entityInfo eInfo

	set last_route [expr $start_route + $eInfo(aip,num)] 
 	for {set i $start_route} {$i<$last_route} {incr i +1} {
	     set route($i) "1 $eInfo(aip,$i,id)"
	     set route($i,num_hops) 1 
	     set route($i,ids) "$eInfo(aip,$i,id) "
	     set route($i,names) "$eInfo(aip,$i,name) "
	     set route(num_routes) [expr $route(num_routes) + 1]
	}
    return 0
}
	
proc get_one_hop_coreaip_routes {route_array entityInfo start_route} {

   upvar $route_array route
   upvar $entityInfo eInfo

	set last_route [expr $start_route + $eInfo(coreaip,num)] 
 	for {set i $start_route} {$i<$last_route} {incr i +1} {
	     set route($i) "1 $eInfo(coreaip,$i,id)"
	     set route($i,num_hops) 1 
	     set route($i,ids) "$eInfo(coreaip,$i,id) "
	     set route($i,names) "$eInfo(coreaip,$i,name) "
	     set route(num_routes) [expr $route(num_routes) + 1]
	}
}
	

#-------------
# list_routes - lists all routes in given route array (for debugging)
#-------------

proc list_routes {route_array} {

   upvar $route_array route

	for {set i 0} {$i<$route(num_routes)} {incr i +1} {
		puts "Route $i: $route($i,names)"
	}
}

	
#-------------
# check_route_array - checks 'num_routes' routes that are in route array 'route' by downloading webpage through that route and comparing 
# downloaded page to template that was downloaded in the clear.
#------------- 

proc check_route_array {route_array num_routes nym_name entityInfo web_address laip_spawn_id} {

   upvar $route_array route
   upvar $entityInfo eInfo

	set template "/tmp/web_template.txt"

	if {$num_routes == "all" || $num_routes>$route(num_routes)} {
		set num_routes $route(num_routes)
	}

	if [catch {get_webpage $web_address $template} ret] {
          error "$ret"
        }

	for {set route_num 0} {$route_num<$num_routes} {incr route_num +1} {
	  puts "Checking route $route($route_num,names) "
	  if [catch {check_single_route $route($route_num) $route($route_num,num_hops) $nym_name $web_address $template $laip_spawn_id} route($route_num,route_check)] {
            error "$route($route_num,route_check)"
	  }
        }

  return 0

}

#-------------
# check_DNS_array - checks DNS queries for 'num_routes' routes  
#------------- 

proc check_DNS_array {route_array num_routes entityInfo web_address spid} {

   upvar $route_array route
   upvar $entityInfo eInfo


	if {$num_routes == "all"} {
		set num_routes $route(num_routes)
	}

        for {set route_num 0} {$route_num<$num_routes} {incr route_num +1} {
		puts -nonewline "Checking DNS for route $route($route_num,names): "
		if [catch {check_single_DNS $route($route_num) $route($route_num,num_hops) $web_address $sid eInfo} route($route_num,DNS_check)] {
                  error "$route($route_num,DNS_check)"
               }
               puts "$route($route,DNS_check)"
        }

  return 0

}

#-------------
# dump_route_results - dumps results of check_route_array and check_DNS_array to file
#-------------

proc dump_route_results {route_array dumpfile} {

   upvar $route_array route
   upvar $dumpfile dump
	
      for {set i 0} {$i<$route(num_routes)} {incr i +1} {
	if {[info exists route($i,route_check)]} {
		puts $dump "Route check for $route($i,names): $route($i,route_check)" 
        } else {		
		puts $dump "Route check for $route($i,names): NOT CHECKED" 
        }
	if {[info exists route($i,DNS_check)]} {
		puts $dump "DNS check for $route($i,names): $route($i,DNS_check)" 
        } else {		
		puts $dump "DNS check for $route($i,names): NOT CHECKED" 
        }
	puts $dump ""
      }

}


#-------------
# create_single_route - create a route based on a route specified as command line
#-------------

proc create_single_route {single_route num_hops nym_name spid} {

   global spawn_id

	  set ACI DESTROYED
	
          if [catch {extract_hops single_route $num_hops hop} ret] {
            error "$ret"
          }

	  set spawn_id $spid
          # send 'new route' command to laip
          if [catch {clear_rules} ret] {
            error "$ret"
          }
          # set the number of hops
          if [catch {set_num_hops $num_hops} ret] {
            error "$ret"
          }

          # specify the hops in the route
          for {set i 0} {$i<$num_hops} {incr i +1} {
            if [catch {set_next_hop $hop($i)} ret] {
              error "$ret"
            }
          }

          # spec the nym to be used
          if {$nym_name == 0}  {
            if [catch {unauth_route} ret] {
              error "$ret"
            }
          } else {
            if [catch {set_nym $nym_name} ret] {
              error "$ret"
            }
          }

          Vputs 1 "Creating route $single_route" 
          if [catch {create_route} ACI] {
            error "$ACI"
          }

	  return $ACI
}
	  
	  
#--------------
# check_single_route - checks one route by dowloading webpage and comparing it to a template
#--------------

proc check_single_route {single_route num_hops nym_name web_address template spid} {

   global spawn_id

   set webcheck "/tmp/web_check.txt"

          #download through cloud
	  if [catch {create_single_route $single_route $num_hops $nym_name $spid} ACI] {
            error "$ACI"
          }
          # block port 80 on newly created route
          if [catch {block_port 80 $ACI} ret] { 
            error "$ret"
          }
          if [catch {get_webpage $web_address $webcheck} ret] {
            error "$ret"
          }
          if [catch {check_webfile $template $webcheck} check_status] {
            error "$check_status"
          }
          if {$check_status != "same"} {
            puts "Route check FAILED : $check_status"
	    set return_status FAILED
          } else {
            puts "Route check PASSED"
	    set return_status PASSED		
	  }


          #unblock port 80 and destroy route
          if {$ACI != "DESTROYED"} {
            unblock_port 80 $ACI 
            destroy_route $ACI 
          }
	  
	  return $return_status
}

#--------------
# check_single_DNS - checks DNS query for given route
#--------------

proc check_single_DNS {single_route num_hops web_address spid entityInfo} {

   upvar $entityInfo eInfo
   global spawn_id

          #create route
          if [catch {create_single_route $single_route $num_hops 0 $spid} ACI] {
            error "$ACI"
          }
          if {$ACI != "DESTROYED"} {
	     extract_hops single_route $num_hops hop  
             set last_hop_index [expr $num_hops - 1]
	     set last_hop_ip [get_aip_ip_from_id $hop($last_hop_index) eInfo]
	     if [catch {block_packets $ACI $last_hop_ip 53} ret] { 
               error "$ret" 
             }
             if [catch {DNS_query $web_address} DNS_info] {
               error "$DNS_info"
             }
	     if {$DNS_info == "empty"}   {
		 set return_status FAILED
             } else  {
		 set return_status PASSED
	     }
	     unblock_packets $last_hop_ip 53  
	     destroy_route $ACI 
	  } else {
	     set return_status FAILED
          }

	  return $return_status
}


#------------
# get_webpage - downloads a webpage from http://$address (using wget) and saves it to file $webfile
#------------
proc get_webpage {address webfile} {


	set return_code 0

	spawn /bin/sh -c "wget -T 30 -O $webfile http://$address"

        expect {
            timeout        {error "Timeout trying to connect to $address -- ERROR"}
            "connected!"   {}
            "Host not found"  {error "Cannot connect to $address"}
            }
        expect {
	    "$webfile saved"   {}
	    timeout 	        {error "Timeout trying to save $address"}
        }
#	wait 
#	close

        spawn /bin/sh -c "\rm wget-log*"
	wait -nowait
	close

	return 0
}

#-----------
# check_webfile - checks that 'template' and 'check' are identical files
#-----------

proc check_webfile {template check}  {
 
   spawn diff -s -q $template $check
   expect {
          timeout {error "Timeout during diff of web_files"}
          "Files $template and $check differ"  {set status "different"}
          "Files $template and $check are identical"  {set status "same"}
   }
   close 
   wait -nowait

   return $status
}

#------------
# DNS query - spawns 'host $web_address' and returns either the ip address or 'empty' if host info was not returned
#------------

proc DNS_query {web_address} {

    spawn host $web_address
    expect  {
            timeout {error "Timeout trying to get ip address of $web_address"}
            -re "$web_address has address (\[0-9]*\.\[0-9]*\.\[0-9]*\.\[0-9]*)"   {
                               set DNS_result $expect_out(1,string)}  
            }

    wait -nowait
    close

    return $DNS_result
}

            
