#$Id: ns-mail.tcl,v 1.2 2001/04/19 19:18:16 alainc Exp $

#---- POP PROCEDURES ------#

#--------
# pop_login - logs into specified pop server and waits for password authentication.  
#--------
proc pop_login {pop_server port nymname password} {

  global spawn_id

  spawn /bin/sh -c "telnet $pop_server $port"
  set pop_spawn $spawn_id
  expect {
    "OK" {}
    "No route to host" {error "Could not connect to $pop_server $port - No route to host"}
  }
  send "USER $nymname\r"
  expect {
    "OK" {}
     timeout {error "Error logging into $pop_server $port as USER $nymname"}
  }
  send "PASS $password\r"
  expect {
    "OK" {}
     timeout {error "Error sending password for $nymname at $pop_server $port}
  }

  return $pop_spawn
}


#---------
# pop_stat - retrieves stat info from pop server.  Assumes user is already logged in
#---------

proc pop_stat {pop_stats spid} {
  
  upvar $pop_stats stats
  global spawn_id

  set spawn_id $spid

  send "STAT\r"
  expect {
 	timeout {error "Could not get pop box stats"}
        -re "OK (\[0-9]+) (\[0-9]+)\r\n" {set stats(messages) $expect_out(1,string)
                                          set stats(pop_box_size) $expect_out(2,string)}
  }
 
  return 0
}

#--------
# pop_quit - ends pop session (assumes already logged in to pop server)
#--------
proc pop_quit {spid} {

  global spawn_id

  set spawn_id $spid

  send "QUIT\r"
  expect {
     "OK" {}
      timeout {error "Could not log out of pop server"} 
  }
  wait 
  close
 
  return 0
}
      
#--------
# pop_retrieve - reads a given message in a pop box (must already be logged in)
#--------

proc pop_retrieve {spid msg_index message} {

  upvar $message msg
  global spawn_id

  set spawn_id $spid

  send "RETR $msg_index\r"
  expect "RETR $msg_index"
  expect {
    timeout {error "Could not retrieve msg $msg_index from pop server"}
    "+OK " {}
  }
  expect {
    timeout {error "Could not retrieve msg $msg_index from pop server"}
    -re "(.*)\r\n\.\r\n" {set msg $expect_out(1,string)}
  }

  return 0
}


#--------
# pop_delete - deletes a given message in a pop box (must already be logged in)
#--------

proc pop_delete {spid msg_index} {

  global spawn_id

  set spawn_id $spid

  send "DELE $msg_index\r"
  expect "DELE $msg_index"
  expect {
    timeout {error "Could not delete message $msg_index at pop server"}
    "+OK " {}
  }

  return 0
}
 
#*********** NMTA PROCEDURES *******

#--------
# proc convert_to_base64 - converts string to bse 64
#--------

proc convert_to_base64 {string} {

  spawn /bin/sh -c "echo -n $string | mimencode"
  expect {
      -re "(.*)\r\n" {return $expect_out(1,string)}
      timeout {error "Could not convert string $string to base64"} 
  }
  wait
  close
}


#---------
# proc nmta_login - login to nmta and authenticate oneself - returns 'failure' on failure
#---------

proc nmta_login {nmta port nym passwd} {

  global spawn_id

  spawn /bin/sh -c "telnet $nmta $port"
  set nmta_spid $spawn_id

  expect {
    "ESMTP" {}
    "No route to host" {error "Could not connect to nmta $nmta $port - No route to host"}
    timeout {error "Timeout trying to connect to nmta $nmta $port"}
  }
  send "EHLO\r"
  expect {
    "8BITMIME" {}
    timeout {error "Error logging into nmta $nmta $port with nym $nym (EHLO)"}
  }
  send "AUTH LOGIN\r"
  expect {
    -re "334(.*)\r\n" {}
    timeout {error "Error logging into nmta $nmta $port with nym $nym (AUTH LOGIN)"}
  }
  send "[convert_to_base64 $nym]\r"
  expect {
    -re "334(.*)\r\n" {}
    timeout {error "Error logging into nmta $nmta $port with nym $nym"}
  }
  send "$passwd\r"
  expect {
    "go ahead" {}
    timeout {error "Error logging into nmta $nmta $port with nym $nym"}
    "auth failure" {error "Error logging into nmta $nmta $port with nym $nym - authentication failure"}
  }
      
  return $nmta_spid
}
  
  
#------------
# nmta_send - sends message from nym to recpt - assumes alredy logged in to nmta
#------------
  
proc nmta_send {nym_address recpt message spid} {

  global spawn_id

  set spawn_id $spid

  send "MAIL FROM:<$nym_address>\r"
  expect {
    "250 ok" {}
    timeout {error "Error sending mail from $nym_address to $recpt (MAIL FROM)"}
  }
  send "RCPT TO:<$recpt>\r"
  expect {
    "250 ok" {}
    timeout {error "Error sending mail from $nym_address to $recpt (RCPT TO)"}
  }
  send "DATA\r"
  expect {
    "go ahead" {}
    timeout {error "Error sending mail from $nym_address to $recpt (DATA)"}
  } 
  send "$message\r.\r"
  expect {
    "250 ok" {}
    "451 connection to mail server rejected" {error "Connection to mail server rejected"}
    timeout {error "Error sending mail from $nym_address to $recpt"}
  }

  return 0
}
  
#--------
# nmta_quit - ends nmta session (assumes already logged in to nmta)
#--------
proc nmta_quit {spid} {

  global spawn_id

  set spawn_id $spid

  send "QUIT\r"
  expect {
     "Connection closed by foreign host" {}
      timeout {error "Could not log out of nmta"} 
  }
  wait 
  close
 
  return 0
}

#--------
# imep_login - logs in to imep
#--------

proc imep_login {imep port} {

  global spawn_id

  spawn /bin/sh -c "telnet $imep $port"
  set imep_spid $spawn_id

  expect {
    "ESMTP" {}
    "No route to host" {error "Could not connect to imep $imep $port - No route to host"}
    timeout {error "Timeout trying to connect to imep $imep $port"}
  }

  send "HELO\r"
  expect {
    -re "250(.*)\r\n" {}
    timeout {error "Error logging into imep $imep $port (HELO)}
  }
    
  return $imep_spid
}

#------
# imep_send - sends message from sender to recpt - assumes already logged into imep
#------

proc imep_send {sender recpt message spid} {
 
  global spawn_id

  set spawn_id $spid

  send "MAIL FROM:<$sender>\r"
  expect {
    "250 ok" {}
    timeout {error "Error sending mail from $sender to $recpt (MAIL FROM)"}
  }
  send "RCPT TO:<$recpt>\r"
  expect {
    "250 ok" {}
    timeout {error "Error sending mail from $sender to $recpt (RCPT TO)"}
  }
  send "DATA\r"
  expect {
    "go ahead" {}
    timeout {error "Error sending mail from $sender to $recpt (DATA)"}
  } 
  send "$message\r.\r"
  expect {
    "250 ok" {}
    timeout {error "Error sending mail from $sender to $recpt"}
  }

  return 0
}

#-----
# imep_quit - ends imep session (assumes already logged in to imep)
#-----

proc imep_quit {spid} {

  global spawn_id
 
  set spawn_id $spid

  send "QUIT\r"
  expect {
    "Connection closed by foreign host" {}
    timeout {error "Could not log out of imep"} 
  }
  wait 
  close
 
  return 0
}


#************* USEFUL FUNCTIONS NEEDED

#------------
# WaitForMail - will check account at pop_server until pop account has total_messages - will return message #total_messages in message - Will check every time_internal and give up after max_time
#------------

proc WaitForMail {time_interval max_time total_messages pop_server port account passwd message} {

  upvar $message msg

  set arrived 0
  set total_time 0
  while {!$arrived} {
    puts "Waiting $time_interval seconds..."
    exec sleep 5
    set total_time [expr $total_time + $time_interval]
    puts "Checking if mail arrived for $account..."
    if  [catch {pop_login $pop_server $port $account $passwd} pop_spid] {
      error "$pop_spid"
    }
    if [catch {pop_stat stats $pop_spid} ret] {
      error "$ret"
    }
    if {$stats(messages) < $total_messages} {
      puts "Mail has not arrived for $account - Only has $stats(messages) messages"
      if {$total_time >= $max_time} {
        if [catch {pop_quit $pop_spid} ret] {
          error "$ret"
        }
        error "Message #$total_messages did not arrive for $account after $max_time seconds... giving up"
      }
    } else {
      puts "Retrieving mail message $total_messages for $account"
      if [catch {pop_retrieve $pop_spid $total_messages msg} ret] {
        if [catch {pop_quit $pop_spid} ret2] {
          error "$ret2"
        } 
        error "$ret"
      } 
      if [catch {pop_delete $pop_spid $total_messages} ret] {
        if [catch {pop_quit $pop_spid} ret2] {
          error "$ret2"
        } 
        error "$ret"
      }
      set arrived 1
      return 0
    }
  }
}



