#$Id: ns-nymsrv.tcl,v 1.2 2001/04/19 19:18:16 alainc Exp $

source $env(NETSANITY_ROOT)/inc/ns-defs.tcl
source $env(NETSANITY_ROOT)/inc/ns-netinfo.tcl

#############################################################################
#                                                                           #
#  FUNCTIONS THAT WORK THROUGH NYMTOOL TO WORK (require a nymtool spawn id) #
#                                                                           #
#############################################################################

# ---------
# LookupNym
# ---------
# Attempt to lookup a nym using nymtool
#
# Assumptions: - 'spid' is a valid nymtool spawn id
#

proc nymsrv_LookupNym {nymname spid} {

    global cmdline

    # make nymtool the current process
    set spawn_id $spid

    Vputs 1 "Looking up nym $nymname"
    send "nym lookup $nymname\r"
    expect {
        timeout	                        { error "[info level 0]: Timeout looking up nym $nymname"}
        "Unable to lookup \"$nymname\""	{ error "[info level 0]: Unable to look up nym $nymname}
	"Name          = $nymname"      {}
    }
    expect $cmdline(nymtool)

    return 0
}

# ---------
# CreateNym
# ---------
# Attempt to create a nym using nymtool
#
# Assumptions: - 'spid' is a valid nymtool spawn id
#

proc nymsrv_CreateNym {nymname spid SNtype} {

    global cmdline

    # make nymtool the current process
    set spawn_id $spid
    
    Vputs 1 "Creating nym $nymname with tokens from SN type $SNtype"
    if [catch {get_token_index $spid $SNtype} token_index] {
       error "[info level 0]: $token_index"
    }
     
    send "nym create $nymname $token_index\r"
    expect {
        timeout	                              { error "[info level 0]: Timeout creating nym $nymname"}
	"Remote nym $nymname created"         {}
	"Unable to"                           { error "[info level 0]: Unable to create nym $nymname"}
	"You already have a nym by that name" { error "[info level 0]: Nym $nymname already exists" }
    } 

    expect $cmdline(nymtool)
    return 0
}

# -------
# LoadNym
# -------
# load a nym -- flag an error into log if nym is not loaded
#
# Assumptions: - spid is a valid nymtool spawnid
#

proc nymsrv_LoadNym {nymname spid}  {

    global cmdline

    # make nymtool the current process
    set spawn_id $spid

    Vputs 2 "Loading nym $nymname"
    send "nym load $nymname\r"
    expect {
        timeout	                   { error "[info level 0]: Timeout loading nym $nymname" }
	"Name          = $nymname" {}
	"Unable to load nym"       { error "[info level 0]: Unable to load nym $nymname"}
    }
    expect $cmdline(nymtool)

    return 0
}

# ----------
# DisableNym
# ----------
# disable a nym
#
# Assumptions: - spid is a valid nymtool spawnid
#              - nym is already loaded in that nymtool
#

proc nymsrv_DisableNym {nymname spid} {

    global cmdline

    # make nymtool the current process
    set spawn_id $spid

    Vputs 1 "Disabling nym $nymname"
    send "nym disable\r"
    expect {
	timeout                              { error "[info level 0]: Timeout disabling nym $nymname" }
	"Failed to"                          { error "[info level 0]: Failed to disable nym $nymname" }
	"No nym is currently active"         { error "[info level 0]: Could not disable nym $nymname - no nym active" }
	"nym $nymname successfully disabled" {}
    }
    expect $cmdline(nymtool)

    return 0
}

# -----------
# NymAddToken
# -----------
# extend a nym by adding a token to it
#
# Assumptions: - spid is a valid nymtool spawnid
#              - nym is already loaded in that nymtool
#              - there is at least one token available to be spent

proc nymsrv_NymAddToken {nymname spid SNtype} {
    
    global cmdline

    # make nymtool the current process
    set spawn_id $spid

    Vputs 1 "Adding token from SN type $SNtype"

    if [catch {nymsrv_LoadNym $nymname $spid} ret] {
       error "[info level 0]: $ret"
    } 

    if [catch {get_token_index $spid $SNtype} token_index] {
      error "[info level 0]: $token_index"
    }
 
    send "nym addtoken $token_index\r"
    expect { 
	timeout           { error "[info level 0]: Timeout adding token to existing nym" }
        "Unable"	  { error "[info level 0]: Unable to extend nym" }
	"Token added to nym" {}
    }

    expect $cmdline(nymtool)

    return 0
}

#################################################################################
#                                                                               #
#  FUNCTIONS THAT WORK THROUGH FRLODEC (require netinfo array)                  #
#                                                                               #
#################################################################################

# ---------
# DeleteNym
# ---------
# erase all trace of nym with frlodec

proc nymsrv_DeleteNym {nymname entityInfo} {

    upvar $entityInfo info
    global ports frlodec network env

    Vputs 1 "Deleting nym $nymname at lode server $info(nymsrv,0,ipaddr)"
    spawn /bin/sh -c "frlodec --hostname $info(nymsrv,0,ipaddr) --user $frlodec(user) --passphrase $frlodec(passphrase) --reason $frlodec(reason) --service nymdb --operation \"delete $nymname\""
    wait 
    close

}

# -------
# NukeNym
# -------
# erase all trace of nym with frlodec

proc nymsrv_NukeNym {nymname entityInfo} {
    upvar $entityInfo info
    global ports frlodec network env

    Vputs 1 "Nuking nym $nymname at lode server $info(nymsrv,0,ipaddr)"
    spawn /bin/sh -c "frlodec --hostname $info(nymsrv,0,ipaddr) --user $frlodec(user) --passphrase $frlodec(passphrase) --reason $frlodec(reason) --service nymdb --operation \"nuke $nymname\""
    wait 
    close

}

# ----------
# RestoreNym
# ----------
# restore a nym with frlodec

proc nymsrv_RestoreNym {nymname entityInfo} {

    upvar $entityInfo info
    global ports frlodec network env

    Vputs 1 "Restoring nym $nymname at lode server $info(nymsrv,0,ipaddr)"
    spawn /bin/sh -c "frlodec --hostname $info(nymsrv,0,ipaddr) --user $frlodec(user) --passphrase $frlodec(passphrase) --reason $frlodec(reason) --service nymdb --operation \"restore $nymname\""
    wait 
    close

}

# ----------
# KeysaveNym
# ----------
# saves keys for a nym (so that it can be used with laip)

proc nymsrv_KeysaveNym {spid keydir} { 

    global cmdline

    # make nymtool the current process
    set spawn_id $spid

    Vputs 2 "Saving nym keys to $keydir"
    send "nym keysave $keydir\r"
    expect { 
	timeout           { error "[info level 0]: Timeout trying to save nym keys to $keydir" }
	"Unable to save"  { error "[info level 0]: Unable to save nym keys to $keydir" }
	"Saved nym" {}
        "No nym is currently active" { error "[info level 0]: Could not save nym keys - no nym active" }
    }
    expect $cmdline(nymtool)

    return 0

}

# ---------
# GetMailCert
# ---------
# checks that a valid mailcert was created by nym server and returns the mail password (in base 64) as mailpasswd

proc nymsrv_GetMailCert {spid nym_name certversion mailcert} {

    global cmdline mailpasswdfile
    upvar $mailcert mcert

    # make nymtool the current process
    set spawn_id $spid
    
    Vputs 1 "Retrieving mailcert for nym $nym_name"
    if [catch {nymsrv_LoadNym $nym_name $spid} ret] {
       error "[info level 0]: $ret"
    } 
    send "nym mailcert $certversion\r"
    expect {
           timeout		{error "[info level 0]: Timeout looking for mail certificate for nym $nym_name" }
           "Failed to request Mail Certificate"	{ error "[info level 0]: Failed to get mailcert for nym $nym_name" }
	   "Mail Cert" { 
		expect {
			timeout {error "[info level 0]: Timeout looking for nym name in mailcert for nym $nym_name"}
   			"name:$nym_name" {}
 		}
		expect {
			timeout {error "[info level 0]: Timeout looking for expiration time in mailcert for nym $nym_name"}
   			-re "Expiration: (\[0-9]+) seconds" {set mcert(expiration) $expect_out(1,string)}
                }
		expect {
			timeout {error "[info level 0]: Timeout looking for nym type in mailcert"}
   			-re "Nym type: (\[0-9])" {set mcert(nym_type) $expect_out(1,string)}
                }
		expect {
			timeout {error "[info level 0]: Timeout looking for mail limit in mailcert"}
   			-re "Mail Limit: (\[0-9]+)\r\n" {set mcert(mail_limit) $expect_out(1,string)}
                }
		expect {
			timeout {error "[info level 0]: Timeout looking for mail volume limit in mailcert"}
   			-re "Mail Volume Limit: (\[0-9]+)\r\n" {set mcert(mail_vol_limit) $expect_out(1,string)}
                }
		expect {
			timeout {error "[info level 0]: Timeout looking for mailbox size limit in mailcert"}
   			-re "Mailbox Size Limit: (\[0-9]+)\r\n" {set mcert(mailbox_size_limit) $expect_out(1,string)}
                }
		expect {
			timeout {error "[info level 0]: Timeout looking for cross-post limit in mailcert"}
   			-re "Cross-post Limit: (\[0-9]+)\r\n" {set mcert(cross_post_limit) $expect_out(1,string)}
                }
       }
       expect $cmdline(nymtool)
    }

# extract password from base64 dump from nymtool
      set timeout 2; expect "SADFSAKDPOFSKADF"; set timeout 10
      set passwdfile [open "./$mailpasswdfile" r]
      if {[gets $passwdfile mcert(passwd)] == -1} {
  	error "[info level 0]: Could not extract mail password from file $mailpasswdfile"
      } 

    return 0
}

proc Spawn_nymtool_process {config_file passphrase passphrase_prompt} {

    global netconf spawn_id cmdline
	  

    Vputs 2 "Spawning nymtool process using $config_file"
        if {[file exists $config_file]} {
	  spawn /bin/sh -c "nymtool --config $config_file"
          expect {
            "Unable to connect to NIQS niqs"  {error "[info level 0]: Could not spawn nymtool - Can't connect to NIQS"}
            "Failed to get network information from niqs" {error "[info level 0]: Could not spawn nymtool - Could not retrieve netinfo from NIQS"}
            "$passphrase_prompt" {}
          }
          send "$passphrase\r"
          expect "$cmdline(nymtool)" {}
        } else {
          error "[info level 0]: Could not spawn nymtool: $config_file does not exist"
        }
     return $spawn_id
}
        

proc nymtool_quit {spid} {

  global spawn_id

  Vputs 2 "Quitting nymtool"

  set spawn_id $spid
  
  send "quit\r"
#  wait

}

	
