#$Id: ns-toksrv.tcl,v 1.2 2001/04/19 19:18:16 alainc Exp $

source $env(NETSANITY_ROOT)/inc/ns-defs.tcl

#############################################################################
#                                                                           #
#  FUNCTIONS THAT WORK THROUGH NYMTOOL TO WORK (require a nymtool spawn id) #
#                                                                           #
#############################################################################

# --------
# RedeemSN
# --------
# Attempt to redeem a serial nymber using nymtool
#
# Assumptions: - 'spid' is a valid nymtool spawn id
#

proc toksrv_RedeemSN {spid SNtype} {

    global cmdline spawn_id tokenInfo

    # make nymtool the current process
    set spawn_id $spid

    Vputs 1 "Redeeming serial number of type $SNtype"
    send "token redeem snid$SNtype\r"
    expect {
	timeout	                   { error "Timeout redeeming SN type $SNtype"}
	"Serial number type id $SNtype" {}
        "Unable" 		   { error "Unable to redeem SN type $SNtype"}
        "Invalid serial number"    { error "Invalid serial number for type $SNtype"}
    }
    expect $cmdline(nymtool)


    return 0
}

# ---------
# get_token_index
# ---------
# Find the token index in the nymtool tokencache corresponding to the required serial number type

proc get_token_index {spid SNtype} {

    global cmdline spawn_id tokenInfo

    set spawn_id $spid

    Vputs 2 "Getting token index for SN type $SNtype"
    send "token list\r"
    expect {
	timeout	                   { error "Timeout trying to find token of from SN type $SNtype" }
	-re "(\[0-9]*) ($tokenInfo($SNtype,label)|$tokenInfo($SNtype,typeid))" {}
        $cmdline(nymtool)          { error "Could not find token for SN type $SNtype"}
    }
     
    expect $cmdline(nymtool)

    return 0
}    

# ---------
# GetTokenLabels
#-----------
# Download token labels from niqs 

proc GetTokenLabels {TokenInfo network} {

  upvar $TokenInfo Info
  global spawn_id cmdline

  Vputs 1 "Getting tokeninfo from niqs" 
  if {[catch {netinfo_niqs_test_connect $network} niqs_spid]} {
    error $niqs_spid
  }

  set spawn_id $niqs_spid

  send "kqs\r"
  expect "$cmdline(niqs_test)" {}
  send "tokeninfo\r"
  expect {
	-re "Type ID number\[\t ]*:  (\[0-9]*)\r\n" {
             Vputs 2 "Type ID is $expect_out(1,string)"
             set snid [GetSNIDfromTokenID $expect_out(1,string)]
             if {$snid != "INVALID"} {
               Vputs 2 "SNID for this type id is $snid"
               expect {
                  -re "Token label\[\t ]*:  (\[a-z|A-Z|0-9|+-| ]*)\r\n"
                      {set Info($snid,label) $expect_out(1,string)
                       Vputs 2 "SNID $snid: token label is $Info($snid,label)"}
                  timeout {error "Could not get token label for SNID $snid"}
               }
             }
             exp_continue
        }
        $cmdline(niqs_test)
  }
  send "quit\r"
  return 0
}

# ---------
# GetSNIDfromTokenID
#-----------
# Given a token ID type, will return the corresponding SN type that generated it

proc GetSNIDfromTokenID {tokenID} {

  global tokenInfo SNType

  set Type INVALID
  foreach name [array names SNType] {
    if {$tokenInfo($SNType($name),typeid) == "$tokenID"} {
      set Type $SNType($name)
    }
  }

  if {"$Type" == "INVALID"} {
    Vputs 2 "Could not find SN type corresponding to token type $tokenID"
  } 

  return $Type
}


# ----------
# SaveTokens
# ----------
# Will save tokens to tokencache
# Tokens from cache only get loaded (and do so automatically if TokenCache exists) when nymtool starting
# This should only be used when trying to spend the same token twice.
# This procedure should be called before Launching other nymtool processes

proc SaveTokens {SNtype entityInfo} {

  upvar $entityInfo eInfo 
  global passphrase passphrase_prompt netconf network spawn_id

# first remove the old tokencache
  if {[catch {RemoveTokenCache $network} ret]} {
    error "$ret"
  }

  if {[catch {Spawn_nymtool_process $netconf($network,homedir)/$network.conf $passphrase $passphrase_prompt} spid]} {
    error "$spid"
  }

  if {[catch {toksrv_RedeemSN $spid $SNtype} ret]} {
    error "$ret"
  }

  set spawn_id $spid
  send "token save\r"
  expect {
    timeout {error "Timeout trying to save tokens"}
    "Unable to save token cache" {error "Unable to save token cache"}
    "Token cache saved to" {}
  }

  if {[catch {nymtool_quit $spid} ret]} {
    error "$ret"
  }
  return 0
}
  
#---------
# RemoveTokenCache
#---------

proc RemoveTokenCache {network} {

  global netconf

  spawn /bin/sh -c "\\rm $netconf($network,homedir)/TokenCache"
  wait
  close

  return 0
}

#---------
# WgetSN - redeems serial number directly from scalper (w/o using nymtool and returns in var SN
#---------

proc WgetSN {scalperURL} {
 
    
  set SNfile "/tmp/serialnum"

  Vputs 1 "Getting SN via wget from scalper $scalperURL"
  spawn /bin/sh -c "wget -O $SNfile \"$scalperURL\""
  expect {
    "Host not found" {error "Could not connect to scalper $scalper_demourl"}
    "Not found" {error "Could not retrieve token from scalper $scalper_demourl"}
    "saved" {}
  }

  if {[gets [open $SNfile r] line] != -1} {
    return $line
  } else {
    error "Could not read Serial Number from file $SNfile"
  }

}
    
  
