#$Id: ns-zkip.tcl,v 1.2 2001/04/19 19:18:16 alainc Exp $

source $env(NETSANITY_ROOT)inc/ns-defs.tcl

#############################################################################
#                                                                           #
#  ZKIP FUNCTIONS  #
#                                                                           #
#############################################################################


# --------
# zkipTelnet - telnets to host and port and ensures proper connection - returns spawn id of telnet process
# --------

proc zkipTelnet {host port} {

  global spawn_id

  spawn /bin/sh -c "telnet $host $port"
  set telnet_spid $spawn_id
  expect {
    timeout {error "Timeout trying to telnet to $host port $port"}
    "Connection refused" {error "Connection to $host port $port refused"}
    "0 0.1.0 220 NODATA BINARY 0" {return $telnet_spid}
  }

}

# ------
# zkipQuit - quits telnet session a la zkip
# ------

proc zkipQuit {spid} {

  global spawn_id

  set spawn_id $spid
  send "0 0.1.0 quit nodata binary 0\r"
  expect {
    timeout {error "Timeout trying to quit telnet session"}
    "Connection closed by foreign host" {}
  }

  close
  return 0
} 
  
# --------
# zkipSNCapability
# --------
# Checks capability of serial number using zkip command

proc zkipSNCapability {host SN capability} {

    upvar $capability capab
    global ports spawn_id 

    if {[catch {zkipTelnet err $host $ports(toksrv)} telnet_spid] {
      error "$telnet_spid"
    } else {
      set spawn_id $telnet_spid
      send "1 5.1.0 capability SERIALNUMBER BASE64 0 $SN\r"
      expect {
	timeout	                   {error "Timeout getting SN capability for serial number $SN"}
	"1 5.1.0 501 NODATA BASE64 0 \"Serial number is not in active database\"" {error "Timeout getting SN capability for serial number $SN"}
        "1 5.1.0 220 CAPABILITYCODE BASE64 208 \"Capability code\"" {}
      }
    }

    if {[catch {zkipQuit err $telnet_spid} ret} {
      error "$ret"
    } else {
      return 0
    }
}
