#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		fr_aipnetworktool.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 2 Sep 1999
#
#   Last Revision:	$Date: 1999/10/08 19:11:31 $
#
#   Description:	
#
#   $Revision: 1.9 $
#
#
#   $Id: fr_aipnetworktool.pl,v 1.9 1999/10/08 19:11:31 thierry Exp $	
#
#   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Getopt::Long;

my $version  = '$Id: fr_aipnetworktool.pl,v 1.9 1999/10/08 19:11:31 thierry Exp $';
my $revision = '$Revision: 1.9 $';
my $progname = 'fr_aipnetworktool.pl';

sub ExitOnError {
    print STDERR 
	"$progname\n\t$revision\n",
	"\t$version\n",
	join("\n", @_),"\n";
    exit 0;
}

my($TOPOTOOLDIR);
$TOPOTOOLDIR  = $ENV{'TOPOTOOLDIR'};
if(defined($TOPOTOOLDIR)){
    if($TOPOTOOLDIR !~ /\/$/){
        $TOPOTOOLDIR  = $TOPOTOOLDIR  . "/";
    }
}else{
    print STDERR "TOPOTOOLDIR environment variable not defined\n";
    exit 0;
}

push @INC, "$TOPOTOOLDIR/bin";


##
## argument parsing and checking
##

my $res;
my %optctl  = ();

my @helpString = 
    (
     "\n\n",
     "Usage $progname",
     "\t--help                  : this help",
     "\t--forceGetAips          : use fr_getAips to get the Freedom AIPS info",
     "\t--forceGeTopology       : use fr_show_network_topo.pl to get the topology info",
     "\t--aipNodeInfo   aipname : print  aipname node info",
     "\t--aipNodeCheck  aipname : check  aipname node info",
     "\t--aipNodeInsert aipname : insert aipname node in the network",
#     "\t--aipNodeDelete aipname : delete aipname node from the network",
     "\t--graphFileIn  filename : graph file input",
     "\t--graphFileOut filename : graph file output",
     "\t--dotFile      filename : create dot file",
     "\t--version               : get version number"
     );


## Get the Options
##
$res = GetOptions(\%optctl, "help",
		  "forceGetAips",
		  "forceGetTopology",
		  "aipNodeInfo=s",
		  "aipNodeCheck=s",
		  "aipNodeInsert=s",
#		  "aipNodeDelete=s",
		  "graphFileIn=s",
		  "graphFileOut=s",
		  "dotFile=s",
		  "version"
		  ) || die "Error in Getopt";

## checks if arguments
##
if(  scalar keys %optctl <= 0){
    ExitOnError("***No options provided",  
		@helpString
		);
}

## print out the usage string
##
if (defined($optctl{'help'})){
    print join("\n", @helpString),"\n";
    exit 1;
}


####################################################################################
#
# Environment Parameter
# Debug Information activation
#
my($DEBUG);
$DEBUG  = $ENV{'DEBUG'};
if($DEBUG){
    print "Debug On\n";
}

my($DEBUG_CREATE_DOT);
$DEBUG_CREATE_DOT  = $ENV{'DEBUG_CREATE_DOT'};
if($DEBUG_CREATE_DOT && $DEBUG){
    print "Debug Dot On\n";
}

my($TOPOLOGY_GRAPH_DIR);
$TOPOLOGY_GRAPH_DIR  = $ENV{'TOPOLOGY_GRAPH_DIR'};
if(defined($TOPOLOGY_GRAPH_DIR)){
    if($DEBUG){print "TOPOLOGY_GRAPH_DIR: $TOPOLOGY_GRAPH_DIR\n";}
    $TOPOLOGY_GRAPH_DIR  = $TOPOLOGY_GRAPH_DIR  . "/";
}else{
    print STDERR "TOPOLOGY_GRAPH_DIR environment variable is missing\n";
    print STDERR "\tdefaulting to /tmp/topology\n";
    $TOPOLOGY_GRAPH_DIR  = "/tmp/topology/";
}
if(! -d "$TOPOLOGY_GRAPH_DIR"){
    mkdir $TOPOLOGY_GRAPH_DIR, 0755;
}


####################################################################################
## include subroutines
##
require 'fr_subroutines.pl';

####################################################################################
## VARIABLES
## 
##
checkFindExecutable("gv");
checkFindExecutable("neato");
checkFindExecutable("fr_servop_dbquery.pl");
checkFindExecutable("fr_summarize_probe.pl");
checkFindExecutable("fr_getFips10.pl");

##
## AipsTopology
## gets the Freedom network topology if graphFileIn not provided
##
##
if(! defined($optctl{'graphFileIn'}) ||  defined($optctl{'forceGetTopology'})){
    print STDOUT "forcing topology query\n";
    my($GETAIPSTOPOLOGY);
    checkFindExecutable("fr_show_network_topo.pl");
    if(! -x "$TOPOTOOLDIR/bin/fr_listroutes"){
	print STDERR "fr_listroutes is missing\n";
    }else{
#	$GETAIPSTOPOLOGY = "fr_show_network_topo.pl" . " --niqs-hostname akira --raw-graph ";
	$GETAIPSTOPOLOGY = "fr_show_network_topo.pl" . " --raw-graph ";
    }
    my($AIPSTOPOLOGYFILE) =  $TOPOLOGY_GRAPH_DIR  . "aipsTopology";
    if(! -f "$AIPSTOPOLOGYFILE"){
	print STDERR "$AIPSTOPOLOGYFILE file is missing\n";
	print STDERR "\tadding one\n";
	`$GETAIPSTOPOLOGY > $AIPSTOPOLOGYFILE`;
    }
    $optctl{'graphFileIn'} = $AIPSTOPOLOGYFILE;
}

##
## AipsInfoFile
##
my($GETAIPS)      =  "$TOPOTOOLDIR/bin/fr_getaips"; # --long --nodns 
if(! -x $GETAIPS){
    print STDERR "fr_getaips is missing\n";
}else{
    $GETAIPS = $GETAIPS . " --long --nodns "
}
my($AIPSINFOFILE) =  $TOPOLOGY_GRAPH_DIR  . "aipsInfo";
if(! -f "$AIPSINFOFILE"){
    print STDERR "$AIPSINFOFILE file is missing\n";
    print STDERR "\tadding one\n";
    `$GETAIPS > $AIPSINFOFILE`;
}


##
## AipsDigest
##
my($AIPSDIGEST) =  $TOPOLOGY_GRAPH_DIR  . "aipsDigest";
if(! -f "$AIPSDIGEST"){
    print STDERR "$AIPSDIGEST file is missing\n";
    print STDERR "\tgenerating one\n";
    `fr_summarize_probe.pl --nosortbyaip $TOPOTOOLDIR/var/ZksTopology > $AIPSDIGEST`;
}

##
## Log file
##
my($PROGLOG)      =  $TOPOLOGY_GRAPH_DIR  . "$progname.log";
$PROGLOG          =~ s/\.pl\./\./;
unlink "$PROGLOG";
open (FDLOG,">$PROGLOG");

if(defined($optctl{'graphFileIn'})){
    my $AIPTOPOLOGY =  $optctl{'graphFileIn'};
}
#my($AIPTOPOLOGY)  =  $TOPOLOGY_GRAPH_DIR  . "freedom.dot";



my(@aipList) = ();
my(%aipNameToRecord)=(); 
my(%aipIpToRecord)=(); 



## checks if graph input file name argument exists
## when using aipNodeInsert and aipNodedelete and aipNodeInfo
##
if(!  defined($optctl{'graphFileIn'}) && 
   ( defined($optctl{'aipNodeInfo'}) || defined($optctl{'aipNodeCheck'}) || 
     defined($optctl{'aipNodeInsert'}) || defined($optctl{'aipNodeDelete'})  )){
    ExitOnError("***No input graph file name",  
		@helpString
		);
}


## checks if graph input file argument exists when needed 
##
if(defined($optctl{'graphFileIn'})){
    if(! -f "$optctl{'graphFileIn'}"){
	ExitOnError("***Input graph file not found",
		    @helpString
		    );
    }
}

## checks if graph output file name argument exists
## when using aipNodeInsert and aipNodedelete
##
if(!  defined($optctl{'graphFileOut'}) && ! defined($optctl{'aipNodeInfo'})
   && ( defined($optctl{'aipNodeInsert'}) || defined($optctl{'aipNodeDelete'}))){
    ExitOnError("***No output graph file name",  
		@helpString
		);
}

##
## Force GetAIp to get the file
##
if(defined($optctl{'version'})){
    print STDERR 
        "$progname\n\t$revision\n",
        "\t$version\n";
    exit 1;
}

##
## Force GetAIp to get the file
##
if(defined($optctl{'forceGetAips'})){
    print FDLOG "Forcing fr_getAips\n";
    unlink "$AIPSINFOFILE";
    `$GETAIPS > $AIPSINFOFILE`;
}

##
## Collect the Aip Info
##
if(defined($optctl{'graphFileIn'})){
    getAipInfo(\@aipList, 
	   \%aipNameToRecord, 
	   \%aipIpToRecord, 
	   $GETAIPS, 
	   $AIPSINFOFILE, 
	   $optctl{'graphFileIn'}, 
	   $AIPSDIGEST);
}else{
    ExitOnError("***No graphFileIn",  
		@helpString
		);    
}


## 
##
if (defined($optctl{'aipNodeCheck'})){ 
    print STDOUT "\nRequired Check Info Node : $optctl{'aipNodeCheck'}\n";
    checkInfoForAip($optctl{'aipNodeCheck'}, 
		    \%aipNameToRecord, $AIPSDIGEST);
}

## 
##
if (defined($optctl{'aipNodeInfo'})){ 
    print STDOUT "\nRequired Info Node : $optctl{'aipNodeInfo'}\n";
    printOutAipInfo($optctl{'aipNodeInfo'}, 
		    \%aipNameToRecord, 
		    \%aipIpToRecord);
}

## 
##
if (defined($optctl{'graphFileOut'}) && ! defined($optctl{'aipNodeInsert'})){
    print STDOUT "\nCreating graph files: $optctl{'graphFileOut'}\n";
    createGraphFile(\@aipList, \%aipNameToRecord, $optctl{'graphFileOut'});
}

## 
##
if (defined($optctl{'aipNodeInsert'})){
#    printOutAipInfo($optctl{'aipNodeInsert'}, \%aipNameToRecord, \%aipIpToRecord);
    my $whereToInsert = insertNodeWithProbeInfo($optctl{'aipNodeInsert'}, 
						\@aipList,
						\%aipNameToRecord, 
						\%aipIpToRecord, 
						$AIPSDIGEST);
    if(defined($whereToInsert)){
	
	print STDOUT "Node $optctl{'aipNodeInsert'} has been connected to $whereToInsert\n";
	if($DEBUG){
	    printOutAipInfo($optctl{'aipNodeInsert'}, \%aipNameToRecord, \%aipIpToRecord);
	    printOutAipInfo($whereToInsert, \%aipNameToRecord, \%aipIpToRecord);
	}

        print STDERR "Creating graph file   : $optctl{'graphFileOut'}\n";
        createGraphFile(\@aipList, \%aipNameToRecord, $optctl{'graphFileOut'});
        
        print STDERR "Creating dot file file: $optctl{'dotFile'}\n";
        createGraphFile(\@aipList, \%aipNameToRecord, 
                        $optctl{'dotFile'}, $optctl{'dotFile'});

	## update AIPSINFO for the momment
	##
	updateAipsInfoFile($aipNameToRecord{$optctl{'aipNodeInsert'}}, $AIPSINFOFILE);
    }else{
	exit 0;
    }
}


## 
##
if (defined($optctl{'dotFile'})){ 
    if($DEBUG){print STDOUT "Creating dot files\n";}
    createGraphFile(\@aipList, \%aipNameToRecord, $optctl{'dotFile'}, $optctl{'dotFile'});
    my($PSFILE) = $TOPOLOGY_GRAPH_DIR . "network.ps";
    if($DEBUG){print STDERR "Creating $PSFILE from  $optctl{'dotFile'}\n";}
    $res = `neato  -Gstart=5 $optctl{'dotFile'} -Tps -o $PSFILE`;
    if($res){
	print STDERR $res,"\n";
	exit 0;
    }

    ## here if gv already started ask to refresh sending SIGHUP
    my($res) = `ps | grep -e\"[0-9] gv $PSFILE\"`;#"
    if(defined($res)){
	chomp($res);
	$res =~ s/^\s+//;
	my(@tmpArray) = split(/\s+/, $res);
	my($pid) = shift @tmpArray;
	if($DEBUG){print "<$res>\n";}
	if($res){
	    if($DEBUG){print STDOUT "Visualization already started sending refresh signal to gv: PID($pid)\n";}
	    `kill -1 $pid`;
	}else{
	    print STDOUT "Something went wrong\n";
	}
    }else{
	print STDOUT "Spawning Visualization tool\n";
	$res = system("gv $PSFILE &");
    }
}


## 
##
if (defined($optctl{'aipNodeDelete'})){
    print FDLOG "Deleting NODE $optctl{'aipNodeDelete'}\n";
}

close(FDLOG);

1;
#   $Log: fr_aipnetworktool.pl,v $
#   Revision 1.9  1999/10/08 19:11:31  thierry
#   add TOPOTOOLDIR environment for execution
#
#   Revision 1.8  1999/09/14 21:31:36  thierry
#   remove exit after aipNodeCheck and aipNodeInfo
#
#   Revision 1.7  1999/09/14 17:23:55  thierry
#   Fips10 support
#
#   Revision 1.6  1999/09/13 17:38:21  thierry
#   add the use of fr_show_network_topo.pl to get the topology file
#
#   Revision 1.5  1999/09/13 12:57:47  luc
#   Fixed typo
#
#   Revision 1.4  1999/09/10 19:33:23  thierry
#   clean debug mode
#
#   Revision 1.3  1999/09/10 17:09:31  thierry
#   sync both to avoid aipsBandwidth file use
#
#   Revision 1.2  1999/09/10 15:19:33  thierry
#   add a checkFind executable subroutine; summary to checkInfoForAip
#
#   Revision 1.1  1999/09/09 21:51:18  thierry
#   Tool to add AIP to freedom network
#
