#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		fr_getFips10.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		13 Sep 1999
#
#   Last Revision:	$Date: 1999/10/08 19:11:31 $
#
#   Description:	
#
#   $Revision: 1.3 $
#
#
#   $Id: fr_getFips10.pl,v 1.3 1999/10/08 19:11:31 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Getopt::Long;

my $version = '$Id: fr_getFips10.pl,v 1.3 1999/10/08 19:11:31 thierry Exp $';
my $revision= '$Revision: 1.3 $';
my $progname= 'fr_getFips10.pl';

sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}

my($TOPOTOOLDIR);
$TOPOTOOLDIR  = $ENV{'TOPOTOOLDIR'};
if(defined($TOPOTOOLDIR)){
    if($TOPOTOOLDIR !~ /\/$/){
        $TOPOTOOLDIR  = $TOPOTOOLDIR  . "/";
    }
}else{
    print STDERR "TOPOTOOLDIR environment variable not defined\n";
    exit 0;
}

################################################################################
#
##
## argument parsing and checking
##
#
my $res;
my %optctl  = ();
$res = GetOptions(\%optctl, "help",
                  "country=s",
		  "state=s",
		  "quiet"
		  ) || die "Error in Getopt";
my @helpString = 
    (
     "\n",
     "This program convert between country and state name to FIPS10 4 digits value.",
     "Options need to be quoted in names containing spaces.",
     "This program generates automatically \"nameToFips.pl\" file in \$TOPOLOGY_GRAPH_DIR",
     "which contains 2 subroutines to get the hashes enabling the fips10 mapping\n",
     "Usage $progname",
     "\t--help         this help",
     "\t--country:    \"country name\"",
     "\t--state  :    \"state/province/territory name\"",
     "\t--quiet  :    silent mode return FIPS10 or nothing",
     "\n"
     );


##
if (defined($optctl{'help'}) || ! defined($optctl{'country'}) || ! defined($optctl{'state'}) ){
    print join("\n", @helpString),"\n";
    exit 1;
}else{
    $optctl{'country'} = lc $optctl{'country'};
    $optctl{'state'} = lc $optctl{'state'};
}


################################################################################
#
# Environment Parameter
# Debug Information activation
#
my($DEBUG);
$DEBUG  = $ENV{'DEBUG'};
if($DEBUG){
    print "Debug On\n";
}

my($TOPOLOGY_GRAPH_DIR);
$TOPOLOGY_GRAPH_DIR  = $ENV{'TOPOLOGY_GRAPH_DIR'};
if(defined($TOPOLOGY_GRAPH_DIR)){
    if($DEBUG){print "TOPOLOGY_GRAPH_DIR: $TOPOLOGY_GRAPH_DIR\n";}
    $TOPOLOGY_GRAPH_DIR  = $TOPOLOGY_GRAPH_DIR  . "/";
}else{
    print STDERR "TOPOLOGY_GRAPH_DIR environment variable is missing\n";
    print STDERR "\tdefaulting to /tmp/topology\n";
    $TOPOLOGY_GRAPH_DIR  = "/tmp/topology/";
}
if(! -d "$TOPOLOGY_GRAPH_DIR"){
    mkdir $TOPOLOGY_GRAPH_DIR, 0755;
}

################################################################################
####
## VARIABLES
## 
##
push @INC, $TOPOLOGY_GRAPH_DIR;
my($NAMETOFIPSFILE)= $TOPOLOGY_GRAPH_DIR  . "nameToFips.pl";
my(%nameToFips)=();
my(%countryToId)=();

################################################################################
#
#
#
#
if(! -f "$NAMETOFIPSFILE"){
    my($fipsFile) = $TOPOTOOLDIR . "var/fips.h";
    my(%countryToFips2)=();

    my(@entry) = `cat $fipsFile`;
    if($DEBUG){print "entry: ", scalar @entry;}

################################################################################
#
# Generate the file if not already there
# avoid to reparse everything for each AIP
#
    my(@countryList) = grep /^\s+\{\"([A-Z]{2})\",\s*\"(.*)\"/, @entry; # "
    my($country, $state, $countryId, $countryName);
    my($fips,$whichState, $alternate);

    foreach $country (@countryList){
	chomp($country);
	if($country =~ /^\s+\{\"([A-Z]{2})\",\s*\"(.*)\"/){ # "
	    $countryId  =    $1;
	    $countryName= lc($2);
	    $countryToId{$countryName} = $1;
	}else{
	    if($DEBUG){print STDERR "Bad $country\n";}
	}
	
	my(@stateList) = grep /^\s+\{\"$1[0-9]{2}\"/,  @entry; # "
	if(scalar @stateList){
	    foreach $state (@stateList){
		chomp($state);
		$state =~ s/\s*\{\"//; # "
		$state =~ s/\"\},//; # "
		($fips, $whichState, $alternate) = split(/\"\s*,\s*\"/, $state);# "
		    if(defined( $alternate)){
			if($DEBUG){print "$fips, $whichState $alternate\n";}
		    }else{
			if($DEBUG){print "$fips, $whichState\n";}
		    }
		$whichState = lc $whichState;
		$nameToFips{"$countryName $whichState"} = $fips;
	    }
	}
    }

    ## write the countryTopId Hash
    ##
    my($keyInd);
    open(FDNAMETOFIPS, ">$NAMETOFIPSFILE");
    print FDNAMETOFIPS 
	"sub getCountryToIdHash {\n",
	"  my(%countryToId) = (\n";
    foreach $keyInd (keys %countryToId){
	print FDNAMETOFIPS "\t\"$keyInd\"=>\"$countryToId{$keyInd}\",\n";
    }
    print FDNAMETOFIPS 
	"\t);\n",
	"  return %countryToId;\n",
	"}\n1;\n";

    ## write the nameToFips Hash
    ##
    print FDNAMETOFIPS 
	"sub getNameToFipsHash {\n",
	"  my(%nameToFips) = (\n";
    foreach $keyInd (keys %nameToFips){
	print FDNAMETOFIPS "\t\"$keyInd\"=>\"$nameToFips{$keyInd}\",\n";
    }
    print FDNAMETOFIPS 
	"\t);\n",
	"  return %nameToFips;\n",
	"}\n1;\n";
    close(FDNAMETOFIPS);
}else{
    ## get the hashes from files
    ##
    require 'nameToFips.pl';
    %countryToId = getCountryToIdHash();
    %nameToFips  = getNameToFipsHash();
}


if( defined( $countryToId{$optctl{'country'}} ) ){
    if($DEBUG){print "$progname: $optctl{'country'} is a valid country\n";}
}else{
    if(! defined($optctl{'quiet'})){print STDERR "***$progname: $optctl{'country'} is not a valid country\n";}
}

if( defined( $nameToFips{"$optctl{'country'} $optctl{'state'}"})){
    if($DEBUG){print STDERR "$progname: $optctl{'state'} is defined\n";}
    print $nameToFips{"$optctl{'country'} $optctl{'state'}"},"\n";
}else{
    if(! defined($optctl{'quiet'})){print STDERR "***$progname: $optctl{'state'} is not a valid state in $optctl{'country'} country\n";}
}

1;
#   $Log: fr_getFips10.pl,v $
#   Revision 1.3  1999/10/08 19:11:31  thierry
#   add TOPOTOOLDIR environment for execution
#
#   Revision 1.2  1999/09/14 17:24:26  thierry
#   frFips10 quiet mode added
#
#   Revision 1.1  1999/09/14 15:42:19  thierry
#   	Script to get country, state conversion to Fips10 4 digits number
#
