#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		fr_maximizeBandwidthUse.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		17 Sep 1999
#
#   Last Revision:	$Date: 1999/10/08 19:11:32 $
#
#   Description:	
#
#   $Revision: 1.4 $
#
#
#   $Id: fr_maximizeBandwidthUse.pl,v 1.4 1999/10/08 19:11:32 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
my $version = '$Id: fr_maximizeBandwidthUse.pl,v 1.4 1999/10/08 19:11:32 thierry Exp $';
my $revision= '$Revision: 1.4 $';
my $progname= 'fr_maximizeBandwidthUse.pl';

my $DEBUG = 0;

require 'fr_subroutines.pl';

##
##
##
my(@aipList) = ();
my(%aipNameToRecord)=(); 
my(%aipIpToRecord)=(); 

my $GETAIPS       = "/home/luc/test/bin/fr_getaips";
my $AIPSINFOFILE  = "/tmp/topo-thierry/aipsInfo";
my $GRAPHIN       = "DataSample/TestNetwork";
my $AIPSDIGEST    = "/tmp/topo-thierry/aipsDigest";

my($PROGLOG)      =  "$progname.log";
$PROGLOG          =~ s/\.pl\./\./;
open (FDLOG,">$PROGLOG");


getAipInfo(\@aipList, 
           \%aipNameToRecord, 
           \%aipIpToRecord, 
           $GETAIPS, 
           $AIPSINFOFILE, 
           $GRAPHIN, 
           $AIPSDIGEST);

##
## Compute Remain and Current Caps
##
my $aip;
my(@connected_node_list) = ();
foreach $aip (@aipList){
    if( defined @{$aip->{CONECTIONS}}){
	$aip->{USED_CAP} = getTotalBandWidthForRec($aip);
	$aip->{FREE_CAP} = $aip->{BANDWIDTH} - $aip->{USED_CAP};
	if($aip->{FREE_CAP} < .25){
	    $aip->{FREE_CAP} = 0;
	}else{
	    $aip->{FREE_CAP} = int( $aip->{FREE_CAP} / .25) * .25;
	}
	push @connected_node_list, $aip;
    }
}


## Nominal Value from Curent Network Topology
##
printf "i/   Total Free Capacity of the Freedom Network: %10.2f\n", 
    getTotalFreeCap (getNodeWithFreeCap(@connected_node_list));

## Iterate to maximize link usage in term of Bandwidth
##
increaseLinkSpeed ( getNodeWithFreeCap(@connected_node_list));
printf "ii/  Total Free Capacity of the Freedom Network: %10.2f\n", 
    getTotalFreeCap (getNodeWithFreeCap(@connected_node_list));

## Make New Connection if there is still capacity avalaible
##
makeNewConnections ( getNodeWithFreeCap(@connected_node_list));
printf "iii/ Total Free Capacity of the Freedom Network: %10.2f\n", 
    getTotalFreeCap (getNodeWithFreeCap(@connected_node_list));

##
##
##
createGraphFile(\@connected_node_list, \%aipNameToRecord, "tmp.dot", "tmp.dot");
`neato  -Gstart=5 tmp.dot -Tps -o /tmp/topo-thierry/network.ps`;

## For each Aip verify that node information was updated
##
foreach $aip (@connected_node_list){
    printOutRecord($aip);
}

close(FDLOG);
exit 1;

################################################################################
##
## Sub routine Chunk
##

##
## get the list of connected AIPS
##
sub getAipRecList {
    my $neib;
    my @recList = ();
    foreach $neib (@_){
	push @recList, $aipNameToRecord{$neib};
    }
    return @recList;
}


##
## Get and Order an input list based
## on remaining capacity
##
sub remain_cap {
    $a->{FREE_CAP} <=> $b->{FREE_CAP}
}

sub getNodeWithFreeCap {
    my @freeCapList = ();
    my @list = sort remain_cap @_;
    foreach $aip (@list){
	if($aip->{FREE_CAP}){
	    push @freeCapList, $aip;
	}
    }
    return @freeCapList;
}


##
## Get Total Free Capacity from List
##
sub getTotalFreeCap {
    my @list = sort remain_cap @_;
    my $totalFreeCap = 0;
    foreach $aip (@list){
	if($aip->{FREE_CAP}){
	    if($DEBUG){print "getTotalFreeCap: adding $aip->{NAME} <$aip->{FREE_CAP}>\n";}
	    $totalFreeCap += $aip->{FREE_CAP};
	}
    }
    return $totalFreeCap;

}

##
## Compute sr, usec from Bandwidth
##
sub getSrUsecFromBandWidth{
    my($bandwidth) = @_;
    my($sr,$usec,$res);
    if($bandwidth > 2.25){
	$usec = 5000;
	$sr = int( ($bandwidth / .5) + .5);
    }else{
	$usec = 10000;
	$sr = int( ($bandwidth / .25) + .5);
    }
    $res = "$sr:$usec";
    if($res =~ /[0-9]{1,2}\:10000/ || $res =~ /[0-9]{1,2}\:5000/){
	return $res;
    }else{
	print  "getSrUsecFromBandWidth: $bandwidth <$res>\n";
	exit 0;
    }
}


##
## Make New Connection between two nodes
## update Connection CONECTIONS list, BANDWIDTH_HASH
## in record for both nodes
## 
sub makeNewConnectionBetweenTwoNodes {
    my($node1, $node2) = @_;
    my($speed) = .25;

    my $test1 = \@{$node1->{CONECTIONS}};
    push @{$test1}, $node2->{NAME};
    my $test =  \%{$node1->{BANDWIDTH_HASH}};
    ${$test}{$node2->{NAME}} = 	getSrUsecFromBandWidth($speed);

    $test1 = \@{$node2->{CONECTIONS}};
    push @{$test1}, $node1->{NAME};
    $test =  \%{$node2->{BANDWIDTH_HASH}};
    ${$test}{$node1->{NAME}} = getSrUsecFromBandWidth($speed);
}

##
## Increase Link speed between two nodes
## update BANDWIDTH_HASH to the new Used Capacity
## in record for both nodes
## 
sub increaseLinkSpeedBetweenTwoNodes {
    my($node1, $node2) = @_;

    my($speed) = .25;
    my($res, $sr, $usec, $bandwidth);

    ## need her to access the BANDWIDTH_HASH
    my $test =  \%{$node1->{BANDWIDTH_HASH}};
    my($entry)     =  ${$test}{$node2->{NAME}};
    if($DEBUG){print "increaseLinkSpeedBetweenTwoNodes: current Rate : $node1->{NAME} $entry\n";}
    if($entry =~ /^([0-9]+):([0-9]+)$/){
	$sr        = $1;
	$usec      = $2;
	$bandwidth = getBandWidthFromSrUsec($sr, $usec);
	if($DEBUG){print "increaseLinkSpeedBetweenTwoNodes: $bandwidth [$sr, $usec]\n";}
	$bandwidth += $speed;
	$res = getSrUsecFromBandWidth($bandwidth);
	${$test}{$node2->{NAME}} = $res;
    }else{
        print "increaseLinkSpeedBetweenTwoNodes: Bad entry <$entry> in BANDWIDTH_HASH for $node1->{NAME} $node2->{NAME}\n"; 
        printOutRecord($node1);
	exit 0;
    }

    ## need her to access the BANDWIDTH_HASH
    $test =  \%{$node2->{BANDWIDTH_HASH}};
    $entry     =  ${$test}{$node1->{NAME}};
    if($DEBUG){print "increaseLinkSpeedBetweenTwoNodes: current Rate : $node2->{NAME} $entry\n";}
    if($entry =~ /^([0-9]+):([0-9]+)$/){
	$sr        = $1;
	$usec      = $2;
	$bandwidth = getBandWidthFromSrUsec($sr, $usec);
	if($DEBUG){print "increaseLinkSpeedBetweenTwoNodes: $bandwidth [$sr, $usec]\n";}
	$bandwidth += $speed;
	$res = getSrUsecFromBandWidth($bandwidth);
	${$test}{$node1->{NAME}} = $res;
    }else{
	print "increaseLinkSpeedBetweenTwoNodes: Bad entry <$entry> in BANDWIDTH_HASH for $node2->{NAME} $node2->{NAME}\n";
        printOutRecord($node2);
	exit 0;
    }
}


##
## 
## Iterate increase Speed until a stable configuration is found
## this sub routine iterates trying to increase link speed 
## for all the already connected nodes of Freedom Network
## until a stable configuration is obtained.
## 
sub increaseLinkSpeed{
    my @list = @_;
    my $modifFlag = 1;
    while($modifFlag){
	$modifFlag = 0;
	foreach $aip (@list){
	    if($aip->{FREE_CAP}){
		if($DEBUG){print "selected node: $aip->{NAME} <$aip->{FREE_CAP}>>\n";}
		my @ordered_neighbour_list =  sort remain_cap getAipRecList(@{$aip->{CONECTIONS}});
		my $neighbour;
		while($aip->{FREE_CAP} && scalar @ordered_neighbour_list){
		    $neighbour = pop @ordered_neighbour_list;
		    if($neighbour->{FREE_CAP} >= $aip->{FREE_CAP}){
			$neighbour->{FREE_CAP} -= .25;
			$aip->{FREE_CAP}       -= .25;
			increaseLinkSpeedBetweenTwoNodes ($aip, $neighbour);
			if($DEBUG){
			    print "easy: add capacity between $aip->{NAME} and $neighbour->{NAME}\n";
			    print "                           $aip->{FREE_CAP} $neighbour->{FREE_CAP}\n";
			}
			$modifFlag = 1;
		    }
		}
	    }
	}
    }
}

##
## 
## Iterate new connections until a stable configuration is found
## this sub routine iterates trying to make new connections in Freedom Network
## for all the nodes with a free capacity until a stable configuration is obtained.
## 
sub makeNewConnections{
    my @list = @_;
    my $modifFlag = 1;
    
    while($modifFlag){
	$modifFlag = 0;
	foreach $aip (@list){
	    if($aip->{FREE_CAP}){
		if($DEBUG){print "selected node: $aip->{NAME} <$aip->{FREE_CAP}>>\n";}
		my @ordered_neighbour_list =  sort remain_cap @list;
		my $neighbour;
		while($aip->{FREE_CAP} && scalar @ordered_neighbour_list){
		    $neighbour = pop @ordered_neighbour_list;
		    if($neighbour->{NAME} =~ /^$aip->{NAME}$/){
			next;
		    }
		    if($neighbour->{FREE_CAP} >= $aip->{FREE_CAP} && $aip->{FREE_CAP} > 0 && \$aip !~ $neighbour){
			$neighbour->{FREE_CAP} -= .25;
			$aip->{FREE_CAP}       -= .25;
			if($DEBUG){
			    print "easy: add capacity between $aip->{NAME} and $neighbour->{NAME}\n";
			    print "                           $aip->{FREE_CAP} $neighbour->{FREE_CAP}\n";
			}
			makeNewConnectionBetweenTwoNodes($neighbour, $aip);
			$modifFlag = 1;
		    }
		}
	    }
	}
    }
}

1;

#   $Log: fr_maximizeBandwidthUse.pl,v $
#   Revision 1.4  1999/10/08 19:11:32  thierry
#   add TOPOTOOLDIR environment for execution
#
#   Revision 1.3  1999/09/21 19:24:44  thierry
#   	* bandwidth roundig
#   	* change computation in getSrUsecFromBandwidth
#   	* better testing in makeNewConnections
#
#   Revision 1.2  1999/09/20 14:58:23  thierry
#   Add comments to subroutines
#
#   Revision 1.1  1999/09/17 21:30:25  thierry
#   tool to maximize Freedom bandwidth usage
#

