#!/usr/bin/perl -w
#
#   File:		$File:$
#   Author(s):		Luc Boulianne
#   Created:		05 May 1999
#   Last Revision:	$Date: 1999/10/08 19:11:32 $
#   Description:	
# 	Script to query the server operator database for information
#       Internal use tool
#
#   $Id: fr_servop_dbquery.pl,v 1.7 1999/10/08 19:11:32 thierry Exp $	
#
#   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
use strict;

my($TOPOTOOLDIR);
$TOPOTOOLDIR  = $ENV{'TOPOTOOLDIR'};
if(defined($TOPOTOOLDIR)){
    if($TOPOTOOLDIR !~ /\/$/){
	$TOPOTOOLDIR  = $TOPOTOOLDIR  . "/";
    }
}else{
    print STDERR "TOPOTOOLDIR environment variable not defined\n";
    exit 0;
}

my ($HOMEDIR)= $TOPOTOOLDIR;
my ($SOA)=$HOMEDIR . "var/isp-affiliates.dat";	# All server ops
my ($SO)=$HOMEDIR . "var/server-operators.dat";	# Active server ops
my ($LONG);
my ($ENTITY, $USER_NAME, $use_entity);
my ($rec, @SOAfields, @SOfields, %SOAfields, %SOfields, $i);
my (@info, @SOAinfo, $frec, $pat);



sub usage {
  printf "usage: $ARGV[-1] [--help] [--long|--l|-l] [--wide|--w|-w]\n" .
         "                 [(--username|--u|-u) username|[pattern|]]\n" . 
         "                 [(--entity|--e|-e) hostname|[pattern|]]\n";
  printf "       scans the isp affiliates for a match and prints \n";
  printf "       the records for these.\n";
  printf "       the records for these.\n";
  printf "       --long     prints out in long format, with field names\n";
  printf "       --wide     prints matches on one line\n";
  printf "       --username tries to match the username (sometimes entity) of the site\n";
  printf "       --entity   tries to match with the hostname of the site\n";
  exit 0;
}

unshift(@ARGV, '.') if $#ARGV < $[;

$LONG = 0;
while ($_ = $ARGV[0], /^-/) {
  shift;
  last if /^--$/;
  /^--h(.*)/ && usage;
  /^(-|--)l(ong|)/ && ($LONG = 1); 
  /^(-|--)w(ide|)/ && ($LONG = 2); 
  /^(-|--)e(ntity|)/ && ($ENTITY=1) && ($LONG = 1) ; 
  /^(-|--)u(sername|)/ && ($USER_NAME=1) && ($LONG = 1) ; 
}
  
if ($#ARGV != $[) {
  usage();
}

$pat = shift;
$pat =~ s!\?!\\?!g;

open(SOA, $SOA) || die "Can't open $SOA: $!";
open(SO, $SO) || die "Can't open $SO: $!";

# Do some preliminary setup
$rec=<SOA>;
chomp($rec);
$rec =~ s/^'//;
$rec =~ s/'$//;
@SOAfields=split(/','/, $rec);

for ($i = 0; $i <= $#SOAfields; $i++) {
  $SOAfields{$SOAfields[$i]} = $i;
}

$rec=<SO>;
chomp($rec);
$rec =~ s/^'//;
$rec =~ s/'$//;
@SOfields=split(/','/, $rec);

for ($i = 0; $i <= $#SOfields; $i++) {
	$SOfields{$SOfields[$i]} = $i;
}

# Now scan the SOA list for a match 
while(<SOA>) {
    s/^'//;
    s/'$//;
    chomp();
    @info=split(/','/);
    if ($#info > 0) {
	$SOAinfo[$info[$SOAfields{"PartnerID"}]]= $_; 
	if ( defined ($ENTITY) ) {
	    if ($info[$SOAfields{"HostName"}] =~ /^$pat$/io) {
		display (@info);
	    }
	} elsif ( defined ($USER_NAME) ) {
	    if ($info[$SOAfields{"Username"}] =~ /^$pat$/io) {
		display (@info);
	    }
	} elsif ($_ =~ /$pat/io) {
	    display (@info);
	}
    }
    undef @info;
}

my($first);

sub display {
  my (@info, $frec, $i);
  @info = @_;

  if ($LONG == 0) {
    $^ = "HEADER";
    $~ = "ENTRY";
    write;
  } elsif ($LONG == 1) {
    $frec="";
    print "===========================================\n";
    for ($i = 0; $i <= $#info; $i++) {
      $frec = $frec .  sprintf ("%-20s   = %s\n", 
				$SOAfields[$i], $info[$i]);
    }
    print $frec;
    print "-" x 80, "\n";    
  } else {
    $frec="";
    if ( ! defined($first) ) {
      $first = 1;
      for ($i = 0; $i <= $#info; $i++) {
	$frec = $frec .  sprintf ("%s  ", $SOAfields[$i]);
      }
      print $frec, "\n";
      print "-" x length($frec), "\n";
      $frec = "";
    }
    for ($i = 0; $i <= $#info; $i++) {
      $frec = $frec .  sprintf ("%s  ", $info[$i]);
    }
    print $frec, "\n";
    
  }
};



format HEADER =
-------------------------------------------------------------------------------
ID    Hostname     Company              AIPip           State Country Bandwdth
----- ------------ -------------------- --------------- ----- ------- ---------
.

format ENTRY =
@>>>> @<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<< @<<<< @<<<<<< @<<<<<<<<
$info[$SOAfields{"PartnerID"}],$info[$SOAfields{"HostName"}],$info[$SOAfields{"Company"}],$info[$SOAfields{"IP1"}],$info[$SOAfields{"State"}],$info[$SOAfields{"Country"}],$info[$SOAfields{"BandwidthFreedom"}]
.

1;

#
# $Log: fr_servop_dbquery.pl,v $
# Revision 1.7  1999/10/08 19:11:32  thierry
# add TOPOTOOLDIR environment for execution
#
# Revision 1.6  1999/09/12 02:28:16  luc
# Corrected missing regex for general matching
#
# Revision 1.5  1999/09/10 16:20:12  thierry
# fix parsing and match loop
#
# Revision 1.4  1999/09/10 16:02:22  luc
# * Added header comments
# * Changed -e option to -u
# * Added support for searching from what usually used as the entity
# name; the hostname
#
# Revision 1.3  1999/09/09 18:14:49  thierry
# fix parsing for entity search
#
