#!/usr/bin/perl -w
#
#   Author(s):		Luc Boulianne
#   Created:		11 Sept, 1999
#   Last Revision:	$Date: 1999/11/27 05:22:51 $
#   Description:	
# 	Script that combines the output of fr_getaips and fr_listroutes
#       to produce a list of all links
#
#   $Id: fr_show_network_topo.pl,v 1.5 1999/11/27 05:22:51 luc Exp $	
#
#   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
use strict;
use Getopt::Long;

my($TOPOTOOLDIR);
$TOPOTOOLDIR  = $ENV{'TOPOTOOLDIR'};
if(defined($TOPOTOOLDIR)){
    if($TOPOTOOLDIR !~ /\/$/){
        $TOPOTOOLDIR  = $TOPOTOOLDIR  . "/";
    }
}else{
    print STDERR "TOPOTOOLDIR environment variable not defined\n";
    exit 0;
}

my ($FR_GETAIPS)="$TOPOTOOLDIR/bin/fr_getaips";
my ($FR_LISTROUTES)="$TOPOTOOLDIR/bin/fr_listroutes";
my ($FONTSIZE)=16;
my ($prog) = "fr_show_network_topo.pl";

my (%Aip); # entity-id  -> entity-name
my (%Status); # entity-id -> up/down
my (%Link); # entity-id:entity-id -> up/down

sub usage {
  printf "usage:  $prog [--help] [--niqs.hostname niqs]\n"
       . "                 [--niqs.port niqsport] [--raw-graph]\n"
       . "\n"
       . "Returns all valid interaip links.\n"
       . "Default is dot format, \n"
       . "--raw-graph option get the graph without dot formatting cmds\n"
       . "\n";
  exit 0;
}

sub start_dot {
  print <<"EOP";
graph G {
	nodesep=20.;
	//size="7.5,10";
	size="10,16";
	ratio=compress;
	concentrate=true;
	node [ fontname=Arial, fontsize=$FONTSIZE];
EOP
}

sub start_dot_edges {
  printf "\n\tedge [ fontname=Arial, fontsize=$FONTSIZE];\n";
}

sub close_dot {

  my($datetime) = scalar localtime;

  print <<"EOP2";
	fontsize=$FONTSIZE;
	fontname=Arial;
	label = "\\n\\nFreedom Network: $datetime\\n";
}
EOP2
}


#
# Handle command line options
#
my %optctl  = ();

# Defaults
$optctl{"niqs-hostname"} = "niqs.freedom.net";
$optctl{"niqs-port"} = 51107;

if (GetOptions(\%optctl, 
	       "help",
	       "niqs-hostname=s",
	       "niqs-port=i",
	       "raw-graph"
	       ) == 0) {
  usage;
}

if (defined ($optctl{"help"}) ) {
  usage;
}

$FR_GETAIPS .=  " --niqs.hostname " . $optctl{"niqs-hostname"} .
      " --niqs.port ". $optctl{"niqs-port"} . " --long --nodns";

#printf (STDERR $FR_GETAIPS . "\n" );

$FR_LISTROUTES .= " --niqs.hostname " . $optctl{"niqs-hostname"}.
      " --niqs.port " . $optctl{"niqs-port"} . " --NHops 2 --ignoredead";

#printf (STDERR $FR_LISTROUTES . "\n" );

if(! defined($optctl{"raw-graph"})){
    start_dot();
}

open (GETAIPS, $FR_GETAIPS . "|") || die "Can't invoke $FR_GETAIPS: $!";

#
# Start by establishing the entity-id to entity mapping 
#
while (<GETAIPS>) {
  my(@line);
  chomp();
  next if (/^$/);
  next if (/^[ \t]*ID/);
  $_ =~ s/^/ /;
  @line = split (/[ \t]+/, $_);

  if(! defined($optctl{"raw-graph"})){ # dot doesn't like -
    $line[2] =~ s/-/_/g;
  }

  $Aip{$line[1]} = $line[2];
  if ($line[3] eq "UP") {
    $Status{$line[1]} = 1;
  } else {
    $Status{$line[1]} = 0
  }
  if(! defined($optctl{"raw-graph"})){
      printf "\t%-16s [color=%s,fontcolor=%s,style=filled];\n", $Aip{$line[1]},
         ($Status{$line[1]}?"green":"red"),
	 ($Status{$line[1]}?"black":"white");
  }
}

close (GETAIPS);

open (LISTRTS, $FR_LISTROUTES  . "|") || die "Can't invoke $FR_GETAIPS: $!";

while (<LISTRTS>) {
  my (@line, $aip1, $aip2);
  chomp ();
  next if ( ! /^2 /);
  @line = split();
  if ($line[1] > $line[2]) {
    $Link{"$line[2]:$line[1]"} = 1;
  } else {
    $Link{"$line[1]:$line[2]"} = 1;
  }
}

if(! defined($optctl{"raw-graph"})){ 
    start_dot_edges();
}

my ($link);
foreach $link (sort (keys (%Link))) {
  my($aip1,$aip2) = split (/:/, $link);
  my($style);
  if ($Status{$aip1} && $Status{$aip2}) {
    $style = "filled";
  } else {
    $style = "dashed";
  }
  if (defined ($Aip{$aip1}) ) {
    $aip1 = $Aip{$aip1};
  }
  if (defined ($Aip{$aip2}) ) {
    $aip2 = $Aip{$aip2};
  }

  printf "\t%16s -- %-16s", $aip1, $aip2;
  if(! defined($optctl{"raw-graph"})){
      printf " [len=2.5, color=red, style=%s]", $style;
  }
  printf "\n";
}

if(! defined($optctl{"raw-graph"})){
    close_dot();
}

1;
#
# $Log: fr_show_network_topo.pl,v $
# Revision 1.5  1999/11/27 05:22:51  luc
# Bug #1407: CONFIG at V1: Migrate domain name from rc.freedom.net to freedom.net
# * Changed script and config files
#
# Revision 1.4  1999/11/18 04:34:48  luc
# Bug #1382 - Merge RC-0 changes onto the V1.0 main trunk
# Reviewers: Etienne
#
# * Bug # 921 - Migrating beta.freedom.net to rc.freedom.net
#
# Revision 1.3.2.1  1999/11/07 17:12:31  luc
# Bug 921
# * changing occurances of beta.freedom.net to rc.freedom.net
#
# Revision 1.3  1999/10/08 19:11:32  thierry
# add TOPOTOOLDIR environment for execution
#
# Revision 1.2  1999/09/13 17:39:21  thierry
# add raw-graph option to get flat graph file
#
# Revision 1.1  1999/09/13 12:57:21  luc
# Preliminary tool to built a .dot file for
#  displaying the network
#
#

