#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		fr_subroutines.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 1 Sep 1999
#
#   Last Revision:	$Date: 1999/10/08 19:11:32 $
#
#   Description:	
#
#   $Revision: 1.15 $
#
#
#   $Id: fr_subroutines.pl,v 1.15 1999/10/08 19:11:32 thierry Exp $
#	
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use Env;

##
## Usefull subroutines probably need to bee somewhere else
##
##
sub dumpToStderr {
    print STDERR join(": ", @_);
}

sub dumpToStdout {
    print STDOUT join(": ", @_);
}

sub dumpToFdlog {
    print main::FDLOG join(":", @_);
}

##
## Checks executables
##
sub checkFindExecutable {
    my($whichExec) = @_;
    $res = `which $whichExec`;
    if($res =~ /no $whichExec in/){
	print STDERR "***Error: $whichExec is missing\n";
	exit 0;
    }
}


##
## Check Consistency
##
sub checkInfoForAip {
    my($aipName, $nameToRec,$probefile) = @_;
    my($rec);
    my($alreadyInFlag, $recInServopFlag, $probeInFlag);

    ##
    ## checks record in curent topology
    if(defined ($$nameToRec{$aipName})){
	if($DEBUG){printOutRecord($$nameToRec{$aipName});}
	print sprintf("   %-16s: network topology list     OK\n", $aipName);
	$alreadyInFlag = 1;
    }else{
	print sprintf("***%-16s: network topology list     NO\n", $aipName);
	$alreadyInFlag = 0;
    }	

    ##
    ## checks record in aipsDatabase using fr_servop
    $rec = getRecordFromServopDbQuery($aipName);
    if(defined ($rec)){
	print sprintf("   %-16s: servop database           OK\n", $aipName);
	$recInServopFlag = 1;
	if($rec->{REGION} =~ /\[[A-Z]{2}[0-9]{2}\]/){
	    print sprintf("   %-16s: FIPS from servopdb db     OK\n", $aipName);
	}else{
	    print sprintf("***%-16s: FIPS from servopdb db     NO\n", $aipName);
	    $recInServopFlag = 0;
	}
    }else{
	print sprintf("***%-16s: servop database           NO\n", $aipName);
	$recInServopFlag = 0;
    }	


    ##
    ## checks for Probe information
    my(@res) = `grep -e\"^$aipName \" $probefile`;#"
#    print join("<", @res),"\n";
    if($#res < 0){
	print sprintf("***%-16s: probeinformation          NO\n", $aipName);
	$probeInFlag = 0;
    }else{
	print sprintf("   %-16s: probe information         OK\n", $aipName);
	$probeInFlag = 1;
    }

    ##
    ## report summary
    ##
    print "\nSummary: "; 
    if($alreadyInFlag){
	print "AIP $aipName is already in the current Freedom Network\n";
    }else{
	if($probeInFlag && $recInServopFlag ){
	    print "AIP $aipName is ready to be inserted\n";
	}else{
	    print "***AIP $aipName is NOT ready to be inserted -> ";
	    if(! $recInServopFlag){
		print " (no record in servop DB)";
	    }
	    if(! $probeInFlag){
		print "(no probe information)";
	    }
	    print "\n";
	}
    }
    print "=======\n\n";
}

##
## Update the aipInfoFile
## will be deleted
##
sub updateAipsInfoFile {
    my($aipRec, $aipinfofile) = @_;
    $res = `grep -e\" $aipRec->{NAME} \" $aipinfofile`;#"
  
    if(length($res)){
	print main::FDLOG "updateAipsInfoFile: $aipRec->{NAME} has already an entry in $aipinfofile\n";
	return 0;
    }else{
	print "<$res>\n";
	print "updateAipsInfoFile: adding new entry in $aipinfofile (debug)\n";
	$stringToAdd = sprintf("%5s  %-11s  %6s  %5s  %16s  %16s  %32s  %16s",
			       $aipRec->{ID}, $aipRec->{NAME}, 
			       $aipRec->{STATUS}, $aipRec->{RATE},
			       $aipRec->{HOSTNAME}, $aipRec->{IP}, 
			       $aipRec->{REGION},  $aipRec->{COMPANY});
	$cmd = "echo \"$stringToAdd\" >>  $aipinfofile";
	if($DEBUG){print "updateAipsInfoFile: <$cmd>\n";}
	`$cmd`;
	return 1;
    }
    return 0;
}

##
##
##
sub printOutRecord {
    my($rec) = @_;
 
    ##
    ## Printout the info
    ##
    print STDOUT sprintf("\t%-16s: %-10s\n", "ID", $rec->{ID});
    print STDOUT sprintf("\t%-16s: %-10s\n", "NAME", $rec->{NAME});
    print STDOUT sprintf("\t%-16s: %-10s\n", "STATUS", $rec->{STATUS});
    print STDOUT sprintf("\t%-16s: %-10s\n", "RATE", $rec->{RATE});
    print STDOUT sprintf("\t%-16s: %-10s\n", "HOSTNAME", $rec->{HOSTNAME});
    print STDOUT sprintf("\t%-16s: %-10s\n", "IP", $rec->{IP});
    print STDOUT sprintf("\t%-16s: %-10s\n", "REGION", $rec->{REGION});
    print STDOUT sprintf("\t%-16s: %-10s\n", "COMPANY", $rec->{COMPANY});
    print STDOUT sprintf("\t%-16s: %-10s\n", "BANDWIDTH", $rec->{BANDWIDTH});

    if( defined (@{$rec->{OTHER}})){
	print STDOUT sprintf("\t%-16s: %-10s\n", "OTHER", $rec->{OTHER});
    }

    if( defined (@{$rec->{CONECTIONS}})){
	print STDOUT sprintf("\t%-12s %3d: %s\n",    
			     "CONECTIONS", scalar ( @{ $rec->{CONECTIONS}}),
			     join(" ", @{ $rec->{CONECTIONS}}));
    }else{
	print STDOUT sprintf("\t%-16s: nil\n", "CONECTIONS");
    }

    if(defined ( @{ $rec->{UPCONECTIONS}})){
	print STDOUT sprintf("\t%-12s %3d: %s\n",    
			     "UPCONECTIONS", scalar ( @{ $rec->{UPCONECTIONS}}),
			     join(" ", @{ $rec->{UPCONECTIONS}}));
    }else{
	print STDOUT sprintf("\t%-16s: nil%s\n", "UPCONECTIONS");
    }
    
    if( defined (%{$rec->{BANDWIDTH_HASH}})){
	print STDOUT sprintf("\t%-16s: %s\n", "TOTALBANDWIDTH", getTotalBandWidthForRec($rec));
    }

    print STDOUT "\n";
}

##
## Print out aipInformation
##
sub printOutAipInfo {
    my($aipName, $nameToRec, $ipToRec) = @_;

    my $rec = $$nameToRec{$aipName};

    ## verify if the node has a record
    ## should return if not because not enough information
    ## to do the insert
    if(! $rec){
	print STDERR sprintf("%16s: %-16s\n", $aipName, 
			     "has no info on freedom network (not connected)");
	return;
    }

    printOutRecord($rec);
}

## 
## File input
## Probe Digest
##
sub insertNodeWithProbeInfo {
    my($aipName, $aipList, $nameToRec, $ipToRec, $probefile) = @_;
    my($line);
    my($name, $dest, $time, $hops);
    my($rec);

    ## verify if the node has a record
    ## should return if not because not enough information
    ## to do the insert
    if(! $$nameToRec{$aipName}){
	if(! insertRecordFromServopDbQuery($aipName, $aipList, $nameToRec, $ipToRec)){
	    dumpToStderr("insertNodeWithProbeInfo", 
			 sprintf("%16s: %-16s\n", 
				 $aipName, 
				 "has no Record (Needed info not available)"));
	    return;
	}else{
	    
	}
    }
    

    ## verify if the node has a ProbeInformation
    ## should return if not because not enough information
    ## to do the insert
    my(@res) = `grep -e\"^$aipName \" $probefile`;#"
    if($#res < 0){
	dumpToStderr("insertNodeWithProbeInfo", 
		     sprintf("%16s %-16s\n", 
			     $aipName, "has no Probe information"));
	return;
    }

    ## verify if the node is UP
    ## should return if not because not enough information
    ## to do the insert
    if( $$nameToRec{$aipName}->{STATUS} eq "DOWN" ){
	dumpToStdout("insertNodeWithProbeInfo", 
		     " $aipName is down but try to put it anyway\n");
    }
    

    ## verify if the node is not already connected
    ## should return if not because not enough information
    ## to do the insert
#    print "<<<<<<>>>>>>>>>>>>\n";
#    print join("|",@$nameToRec{$aipName}->{UPCONECTIONS});
#    printOutAipInfo($aipName, $nameToRec, $IpToRec);
#    print "<<<<<<>>>>>>>>>>>>\n";
    if( defined ( @$nameToRec{$aipName}->{UPCONECTIONS}) ){
#	print "@$nameToRec{$aipName}->{UPCONECTIONS}\n";
	dumpToStderr("insertNodeWithProbeInfo", 
		     sprintf("%16s: %-16s\n", $aipName, "already connected"));
	return;
    }

    $DEBUG = 1;
    my(%hashCoeffs)=();
    foreach $line (@res){
	($name, $dest, $time, $hops)=split(/\s+/, $line);
	if($dest =~ /([0-9]+\.)+/){
	    ## checks for a valid ip record
	    if($DEBUG){print "\t$dest -> IP ->";}
	    if($$ipToRec{$dest}){
		if($DEBUG){print  $$ipToRec{$dest}->{NAME},"\n";}
		$dest = $$ipToRec{$dest}->{NAME};
	    }else{
		print main::FDLOG sprintf("insertNodeWithProbeInfo: %16s: %-16s\n", "Unkwown Ip", $dest);
		$dest = "unknown";
	    }
	}else{
	    ## checks for a valid name record
	    if($$nameToRec{$dest}){
		if($DEBUG){print  $$nameToRec{$dest}->{NAME},"\n";}
		$dest = $$nameToRec{$dest}->{NAME};
	    }else{
		print main::FDLOG sprintf("insertNodeWithProbeInfo: %16s: %-16s\n", "Unkwown entity",  $dest);
		$dest = "unknown";
	    }
	}

	##
	## got a metric value to get the nearest node to connect
	## consider conections all
	my($bandwidthCoeff, $coeff);
	if($dest !~ /unknown/){
	    $rec = $$nameToRec{$dest};

	    if( $#{$rec->{CONECTIONS}} == -1) {
		print main::FDLOG sprintf("insertNodeWithProbeInfo: %16s: %-16s\n", 
					  $rec->{NAME},  "not connected to the network");
		next;
	    }
	    
	    
	    if($rec->{STATUS} eq "DOWN") {
		print main::FDLOG sprintf("insertNodeWithProbeInfo: %16s: %-16s\n", 
					  $rec->{NAME},  "host down");
#		next;
	    }
	       

	    $bandwidthCoeff = (70 -  $rec->{BANDWIDTH} / .5 * 20);
	    $coeff = $time + $hops + $bandwidthCoeff + ($#{ $rec->{CONECTIONS}}+1) * 10;
	    $hashCoeffs{$coeff}=$rec;

	    if($DEBUG) {print main::FDLOG sprintf("%8s %16s %8s %8s %8s %8s %8s\n", 
				      $name, $dest, $time, $hops, 
				      $bandwidthCoeff, ($#{ $rec->{CONECTIONS}}+1) * 10, $coeff);}
	}
    }

    my(@keysCoeffs) = sort numerically (keys(%hashCoeffs));
    my($val, $nodeNameToConnect);
    
    if($DEBUG){
	foreach $val (@keysCoeffs) {
	    print sprintf("%8.2d %-16s %s\n", $val, $hashCoeffs{$val}->{NAME}, $hashCoeffs{$val}->{IP});
	}
    }
    $DEBUG = 0;

    $val = shift @keysCoeffs;
    $nodeNameToConnect = $hashCoeffs{$val}->{NAME};

    $rec = $$nameToRec{$aipName};
    push @{$rec->{CONECTIONS}}, $nodeNameToConnect;
    push @{$rec->{UPCONECTIONS}}, $nodeNameToConnect;
    ${$rec->{JUSTINSERTED}} = 1;

    $rec = $$nameToRec{$nodeNameToConnect};
    push @{$rec->{CONECTIONS}}, $aipName;
    push @{$rec->{UPCONECTIONS}}, $aipName;

    return $nodeNameToConnect;
}

sub numerically {$a <=> $b;}

#
# subroutine to get the associated bandwidth
# for an aip
#
# getBandWidth($aipname)
# take input from servop DB
# 
sub getBandwidth {
    my($aipName) = @_;
    my($bandwidth);
    my($rec) = getRecordFromServopDbQuery($aipName);
    if(defined ($rec)){
	$bandwidth = $rec->{BANDWIDTH};
    }else{
	print main::FDLOG "getBandwidth: defaulting bandwidth to 1 for $aipName\n";
	$bandwidth = 1.5;
    }
    if($DEBUG){print "getBandwidth: $aipName->$bandwidth\n";}
    return $bandwidth;
}



# 
# subroutine to parse connection and setup bandwidth
# from the raw graph file
#  aip1 -- aip2 [sr=3, usec= 5000]
#  subroutine returns aip1 aip2 sr pk
#
sub parseLinkEntry {
    my($entry) = @_;
    if($entry =~ /\s*([\w-_]+)\s*--\s*([\w-_]+)\s*\[\s*sr=([0-9]+)\s*,\s*usec=([0-9]+)\s*\]/ ){
	if($DEBUG) {print "parseLinkEntry: Good Link $1 $2 $3 $4\n";}
	return ($1,$2,$3,$4);
    }else{
	if($entry =~ /\s*([\w-_]+)\s*--\s*([\w-_]+)\s*/){
	    print main::FDLOG "parseLinkEntry: Good Link $1 $2 But no bandwidth Info\n";
	    return ($1,$2);
	}else{
	    print main::FDLOG "parseLinkEntry: Bad link <$entry>\n";
	    return;
	}
    }
}

# 
# subroutine to convert sr, usec to bandwidth
#  subroutine returns bandwidth
#
sub getBandWidthFromSrUsec {
    my($sr, $usec) = @_;
    my($coeff) = 2500;
    $bandwidth = $sr * $coeff / $usec;
    if($DEBUG){print "getBandWidthFromSrUsec: ($sr, $usec) => Bandwidth: $bandwidth Mbs\n";}
    return $bandwidth;
}

# 
# subroutine to convert sr, usec to bandwidth
#  subroutine returns bandwidth
#
sub getTotalBandWidthForRec {
    my($record) = @_;
    my(%neibHash) = %{$record->{BANDWIDTH_HASH}};
#    if($DEBUG){print "getTotalBandWidthForRec: $record->{NAME} %{$record->{BANDWIDTH_HASH}}", 
#	       join("|", keys %neibHash), "\n";}

    my($totalBandwidth) = 0;

    if(scalar keys %neibHash){
	foreach $neighbour ( keys  %neibHash){
	    $entry = $neibHash{$neighbour};
	    if($entry =~ /^([0-9]+):([0-9]+)$/){
		$sr  = $1;
		$usec= $2;
		if($sr && $usec){
		    $totalBandwidth +=  getBandWidthFromSrUsec($sr, $usec);
		}else{
		    $totalBandwidth +=  0;
		}
	    }else{
		print main::FDLOG "getTotalBandWidthForRec: $record->{NAME} has problem in BANDWIDTH_HASH\n";
		return 0;
	    }
	}
    }else{
	print main::FDLOG "getTotalBandWidthForRec: $record->{NAME} has no  BANDWIDTH_HASH\n";
	return 0;
    }
    if($DEBUG){print "getTotalBandWidthForRec: $record->{NAME} is $totalBandwidth\n";}
    return $totalBandwidth;
}

# 
# subroutine to get connected nodes to an aip
#
# nodes need to be specified with the following parameters:
# name (entity name) [ip mapping, bandwidth]
#
#
sub getAipConnections {
    my($aipName, $topologyfile) = @_;
    my(@tmp) = ();
    my(%neighbourHash)=();
    my(@neighbour)=();
    my($connect);
    @tmp=();
    my($ind)=0;
    @tmp = `grep -e\"$aipName\" $topologyfile`; # "
    foreach $connect (@tmp){
	chomp $connect;
	($aip1, $aip2, $sr, $usec) = parseLinkEntry($connect);

	if($aip1 !~ /^$aipName$/ && $aip2 !~ /^$aipName$/){
	    if($DEBUG){print "getAipConnections: drop entry  $aipName !~ [$aip1, $aip2]\n";}
	    next;
	}
	if($aip1 =~ /$aipName/){
	    $connect = $aip2;
	}else{
	    $connect = $aip1;
	}
	if($sr && $usec){
	    $neighbourHash{$connect} = "$sr:$usec";
	}else{
	    $neighbourHash{$connect} = "0:0";
	}
#	push @neighbour, $connect;
	$ind = $ind+1;
    }

    ##
    ## display keys
    ##
    if($DEBUG){
	$nb = scalar keys %neighbourHash;
	if( $nb){
	    print "getAipConnections $nb: ";
	    foreach $connect (keys %neighbourHash){
		print "$connect <$neighbourHash{$connect}> |";
	    }
	    print "\n";
	}
    }

    return %neighbourHash;
#    return @neighbour;
}


# 
# subroutine to get the graph
#
#
sub createConnectionGraph {
    my($aipL, $aipN, $dotFlag)=@_;
    my($node);
    my($neib,$neibRec);
    my($tmp);
    my(@res)=();
    my($bandwidth,$color);
    my(@upNodes) = ();
    my(@downNodes) = ();
    my(@shape) = ();
    my(@resultArray) = ();

    foreach $node (@$aipL){
	if($DEBUG) {print "createConnectionGraph: working on $node->{NAME}\n";}
	if($#{$node->{CONECTIONS}} > -1){
	    ## we push the node as up or down depending on the status
	    ##
	    $tmp = $node->{NAME};
#	    $tmp =~ s/\-//gi;
#	    print STDERR "x $node->{NAME}:",$node->{STATUS},"\n";
	    if($node->{STATUS} eq "UP"){
		if( defined($node->{JUSTINSERTED})){
		    push @shape, "\t$tmp [color=red,style=filled];";
		}else{
		    push @shape, "\t$tmp [color=green,style=filled];";
		}
	    }else{
		if( defined($node->{JUSTINSERTED})){
		    push @shape, "\t$tmp [shape=polygon,sides=4,color=red,style=filled];";
		}else{
		    push @shape, "\t$tmp [shape=polygon,sides=4,color=grey,style=filled];";
		}
	    }
	    if($DEBUG){print $node->{NAME},"->",join("|", @ { $node->{CONECTIONS}}),"\n";}
	    if($DEBUG){
		print sprintf("%10s %10s :", $node->{NAME}, $node->{BANDWIDTH});
		print join(" ", @{ $node->{CONECTIONS}}),"\n";
	    }
	    foreach $neib ( @{ $node->{CONECTIONS}}){
		$neibRec =  $$aipN{$neib};

		## checks if connection already there
		## need to revert
		@res = grep /\s$neib\s+--\s+$node->{NAME}\s/, @upNodes;
		if($#res == -1){
		    @res = grep /\s$neib\s+--\s+$node->{NAME}\s/, @downNodes;
		}
		if($#res == -1){
		    if( $node->{BANDWIDTH} < $neibRec->{BANDWIDTH}){
			$bandwidth = $node->{BANDWIDTH};
		    }else{
			$bandwidth = $neibRec->{BANDWIDTH};
		    }
		    
                    ## create the Node String
		    ##test
		    $tmp = sprintf("%12s -- %-12s", $node->{NAME} , $neib);
#		    $tmp =~ s/\[bandwidth=[0-9.]+\]//;
#		    $tmp =~ s/([a-z0-9]+)\-/$1/gi;
		    if($bandwidth >= 1){
			$color = "red";
		    }else{
			$color = "blue";
		    }
		    if(defined($dotFlag)){
			## little trick to decrease length betweeen
			## nodes starting with same 3 first letters
			if ((substr $node->{NAME}, 0, 3) eq (substr $neib, 0, 3)){
			    $len = "len=1.5,";
			}else{
			    $len = "len=2.5,";
			}
#			$link = sprintf("\t$tmp [w=%f, color=%s];", $bandwidth * 10, $color);
                       $link = sprintf("\t$tmp [$len color=%s]", $color);
		    }else{
			$link = $tmp;
		    }
#		    print "$link\n";
		    if( ($node->{STATUS} eq "UP") && ($neibRec->{STATUS} eq "UP")){
			push @upNodes, $link;
		    }else{
			push @downNodes, $link;
		    }
		}else{
		    if($DEBUG){print "connection already exists: $neib -- $node->{NAME}\n";}
		}
	    }
	}else{
	    if($DEBUG){print "createConnectionGraph: $node->{NAME} not connected\n";}
	    if(defined($dotFlag)){
		print  main::FDLOG "createConnectionGraph $dotFlag: $node->{NAME} not connected\n";
	    }else{
		print  main::FDLOG "createConnectionGraph: $node->{NAME} not connected\n";
	    }
	}
    } 


    if($#shape > -1){
	if(defined($dotFlag)){
	    if($DEBUG){print join("\n", @shape),"\n";}
	    push @resultArray, @shape;
	}
    }

    if($#upNodes > -1){
	if(defined($dotFlag)){
	    push @resultArray, "\tedge [style=filled];";
	}else{
	    push @resultArray, "\nup conections :";
	}
	if($DEBUG){print join("\n", @upNodes),"\n";}
	
	push @resultArray, @upNodes;
    }

    if($#downNodes > -1){
	if(defined($dotFlag)){
	    push @resultArray, "\tedge [style=dashed];";
	}else{
	    push @resultArray, "\ndown conections:";
	}

	if($DEBUG){print join("\n", @downNodes),"\n";}
	push @resultArray, @downNodes;
    }

    return @resultArray;
}


#
# File Input 
# Host IP Mappings resolution
# needed for the inter-AIP pinging
#
# here we can take the digest and extract the 
# host ip mapping then process the rejected entries and remove them
# from the actual 
# probably not needed for here if we have the Current topology
# which should contain this actual information
#
sub getAipInfo {
    my($aipL, $aipN, $aipI, $getaips, $aipsinfofile, $aiptopology, $probeinfofile) = @_;
    my($line, $aipId, $aipName, $aipStatus, $aipRate, $aipHostname, $aipIp, $aipRegion, $aipState, $aipCountry, $aipCompany, $aipOther);
    my(@tmp);
    my($thisAip);

    ## verify if aip info file exists
    ##
    if(! -f "$aipsinfofile"){
	print main::FDLOG "$aipsinfofile not found, regenerate it\n";
	`$getaips > $aipsinfofile`;
    }else{
	print main::FDLOG "$aipsinfofile found, use it\n";
    }

    ## verify if aip info file exists
    ##
    if(! -f "$probeinfofile"){
#	print main::FDLOG "$probeinfofile not found, regenerate it\n";
#	`$getaips > $aipsinfofile`;
	print "cannot open $probeinfofile";
	exit 0;
    }else{
	print main::FDLOG "$probeinfofile found, use it\n";
    }

    ## Parse the file to get the info
    ##
    if($DEBUG ){print "\ngetAipInfo: Start Parse $aipsinfofile\n";}
    open (fdIn, "<$aipsinfofile");
    while(<fdIn>){
	$line = $_;
	chomp($line);
	$line =~ s/^\s+//gi;
	$line =~ s/\s+-\s+/ /gi;
	$line =~ s/\t+/  /gi;
	if($line =~ /^[0-9]+\s/){
	    ($aipId, $aipName, $aipStatus, $aipRate, $aipHostname, $aipIp, $aipRegion, $aipComp, @tmp) = split(/\s{2,}/, $line);
	    $aipOther = join(":", @tmp);
	    $thisAip={
		ID       => $aipId,
		NAME     => $aipName,
		STATUS   => $aipStatus,
		RATE     => $aipRate,
		HOSTNAME => $aipHostname,
		IP       => $aipIp,
		REGION   => $aipRegion,
		COMPANY  => $aipComp,
		OTHER    => $aipOther
		};

	    if($DEBUG ){print "getAipInfo: _______ $aipName ______\n\t$aipId\n\t$aipName\n\t$aipStatus\n\t$aipRate\n\t$aipHostname\n\t$aipIp\n\t$aipRegion\n\t$aipComp\n";}

	    # Get associated bandwidth with this AIP
	    $thisAip->{BANDWIDTH} = getBandwidth($thisAip->{NAME}, $aipbandwidth);
	    if($DEBUG ){print "\t$thisAip->{BANDWIDTH}\n";}

	    # Get associated connected AIPS
	    %neighbourHash = getAipConnections($thisAip->{NAME}, $aiptopology);
	    
	    if(scalar keys %neighbourHash ){
		$thisAip->{CONECTIONS} = [ keys %neighbourHash ];
		if($#{ $thisAip->{CONECTIONS}} > -1){
		    if($DEBUG ){print "\t", join("|", @ { $thisAip->{CONECTIONS}} ),"\n";}
		}else{
		    if($DEBUG ){print "\t$thisAip->{NAME} has no connections\n";}
		}

		$thisAip->{BANDWIDTH_HASH} = { %neighbourHash };

		##
		## Should be for DEBUG mode
		##
		if($DEBUG){
		    %newHash = %{$thisAip->{BANDWIDTH_HASH}};
		    if(scalar keys %newHash){
			print "getAipInfo: ";
			foreach $tutu (keys %{$thisAip->{BANDWIDTH_HASH}}){
			    print "$tutu <$newHash{$tutu}> | ";
			}
			print "\n";
		    }else{
			print "Bad Info for $thisAip->{NAME}\n";
			exit 0;
		    }
		}
	    }else{
		print main::FDLOG "getAipInfo: $thisAip->{NAME} this aip is not on current graph file\n";
	    }

	    ##
	    ## store what has to be stored
	    push @$aipL , $thisAip;
	    $name = $thisAip->{NAME};
	    $ip   = $thisAip->{IP};
	    $$aipN{$name}= $thisAip;
	    $$aipI{$ip}= $thisAip;
	    if($DEBUG){print "getAipInfo:___________________\n\n";}
	}
    }
    close fdIn;
    if($DEBUG ){print "getAipInfo: End Parse $aipsinfofile\n";}


    ## Iterates through the aip list
    ## to get Up connections
#    $DEBUG = 1;
    my($aipRec);
    my($aipNeib);
    my($aipNeibRec);
    my(@probe);
    my($probeString);

    if($DEBUG ){print "\ngetAipInfo: Start find Up connections\n";}
    foreach $aipRec (@$aipL){
	# verifies that probe info exists
	@probe = `grep -e^$aipRec->{NAME} $probeinfofile`;
#	print "grep -e^$aipRec->{NAME} $probeinfofile\n","<<$#probe>><@probe>\n";
	if($#probe < 0){
	    $probeString = "<No probe info in $probeinfofile>";
	}else{
	    $probeString = "";
	}
	
	# Get up connected AIPS
	if($#{$aipRec->{CONECTIONS}} > -1){
	    if($DEBUG ){print "$aipRec->{NAME} : ", $#{$aipRec->{CONECTIONS}}+1, " connection(s) status is ", $aipRec->{STATUS}, "\n";}

	    ## iterates through the neighbours
	    foreach $aipNeib (@ { $aipRec->{CONECTIONS}}){
		$aipNeibRec = $$aipN{$aipNeib};
		if($DEBUG){print $aipNeibRec->{NAME},"-> ", $aipNeibRec->{STATUS} ,"->";}
		if( $aipRec->{STATUS} eq "UP" && $aipNeibRec->{STATUS} eq "UP"){
		    push @{ $aipRec->{UPCONECTIONS} }, $aipNeib;
		}
	    }

	    ## reports non connected nodes
	    if( $#{$aipRec->{UPCONECTIONS}} == -1){
 		print main::FDLOG sprintf("getAipInfo: %s %s\n", $aipRec->{NAME}, "connected but Down");
		if( length($probeString)){
		    print main::FDLOG sprintf("getAipInfo: %s %s\n", $aipRec->{NAME}, "$probeString");
		}
	    }else{
		if( length($probeString)){
		    print main::FDLOG sprintf("getAipInfo: %s %s\n", $aipRec->{NAME}, $probeString);
		}
	    }

	    if($DEBUG){
		print "\n";
		print "Link ", $#{$aipRec->{CONECTIONS}}+1,": ", join("->", @{ $aipRec->{CONECTIONS} }),"\n";
		if($#{$aipRec->{UPCONECTIONS}} > -1){
		    print "Up   ", $#{$aipRec->{UPCONECTIONS}}+1, ": ", join("->", @{ $aipRec->{UPCONECTIONS} }),"\n";
		}else{
		    print "Up   0:\n";
		}
		print "\n_______\n";
	    }
	}else{
	    print main::FDLOG sprintf("getAipInfo: %s %s\n", $aipRec->{NAME}, "not connected $probeString");
	    if( length($probeString)){
		print main::FDLOG sprintf("getAipInfo: %s %s\n", $aipRec->{NAME}, "$probeString");
	    }
	}
    }
    if($DEBUG ){print "getAipInfo: End find Up connections\n";}
#    $DEBUG = 0;
}

##
## Createnew record from the main database calling 
## luc's program: fr_servop_dbquery.pl --entity
## this program returns a new record to be added in the record list
##
sub insertRecordFromServopDbQuery{
    my($aipName, $aipL, $aipN, $aipI, $queryProg)= @_;

    ##
    ## Get the record from servop DB
    ##
    my($thisAip) = getRecordFromServopDbQuery($aipName);
    if(! defined ($thisAip)){
	return 0;
    }   

    ##
    ## insert the record in the aipList
    ##
    push @$aipL, $thisAip;

    ##
    ## update nameTorecord hash
    ##
    $$aipN{$aipName}= $thisAip;

    ##
    ## update ipToRecord hash
    ##
    my($ip) = $thisAip->{IP};
    $$aipI{$ip}= $thisAip;
    
    ##
    ## Printout the info
    ## this is also a verification test
    ##
    if($DEBUG){printOutAipInfo($aipName, $aipN, $aipI);}

    return 1;
}


sub isNumber {
    my($nb) = @_;
    if($nb =~ /^\d+$/){
	return 1;
    }else{
	return 0;
    }
}

sub getRecordFromServopDbQuery{
    my($aipName)= @_;
    my(%hashRec) = ();
    my($flagUseWhat);

    if(! defined($queryProg)){
	$queryProg = "fr_servop_dbquery.pl";
    }
    
    ##
    ## Checks the result of the query
    ##
    my(@res) = `$queryProg --entity $aipName`;
    if(! scalar @res){
	 @res = `$queryProg --username $aipName`;
	 if(! scalar @res){
	     return;
	 }else{
	    $flagUseWhat = "Username"; 
	 }
     }else{
	    $flagUseWhat = "HostName"; 
     }

    shift(@res); ## get rid of first line
    pop(@res);   ## get rid of last  line

    ##
    ## create a Hash from the record
    ##
    foreach $entry (@res){
	chomp($entry);
	$entry =~ s/[\t\s]+=[\t\s]+/\|/;
	($mykey, $myval) = split(/\|/, $entry);
#	print "($mykey, $myval)\n";
	$hashRec{$mykey} = $myval;
    }

    ##
    ## Get the bandwidth
    ## 
    $res = $hashRec{'BandwidthFreedom'};
    if($DEBUG){print "getRecordFromServopDbQuery: Bandwidth [$res]->";}
    if( defined($res)){
	if($res =~ /T3/) {
	    $res = 10.5;
	}
	elsif($res =~ /T1/) {
	    $res = 1.5;
	}
	elsif($res =~ /\s*([0-9]+)\s*k/gi) {
#	    if($DEBUG) { print "getRecordFromServopDbQuery: [$res] $1\n";}
	    $res = $1;
	    $res =~ s/\s*k.*//gi;
	    if(! isNumber($res)){
		print "getRecordFromServopDbQuery: $res is not a number\n";
		print STDERR sprintf("\n%10s: %10s [%10s]\n", 
			  "getRecordFromServopDbQuery: $aipName", 
				 "Unknown Bandwidth", $res);
		return;
		
	    } 
	    $res = $res / 1000;
	}
	elsif($res =~ /^([0-9]+)\s+.*/){
	    if($DEBUG) {print "scalar [",scalar $res,"]\n";}
#	    print STDERR "assume First digits of $res are $1 Mbs\n";
	    $res = $1;
	    if(! isNumber($res)){
		print "getRecordFromServopDbQuery: $res is not a number\n";
		print STDERR sprintf("\n%10s: %10s [%10s]\n", 
				     "getRecordFromServopDbQuery: $aipName", 
				     "Unknown Bandwidth", $res);
		return;
	    } 
	}
	elsif($res =~ /\sM/gi) {
	    $res =~ s/\sM.*//gi;
	}else{
	    $res = "$res??";
	    if($DEBUG){print "\n";}
	    print STDERR sprintf("\n%10s: %10s [%10s]\n", 
			  "getRecordFromServopDbQuery: $aipName", 
				 "Unknown Bandwidth", $res);
	    return;
	}
	$hashRec{'BandwidthFreedom'} = $res;
    }
    if($DEBUG){print "[$res]\n";}

    ##
    ##Here we can check for FIPS10 support
    ## 
    $fips10 = 'UNKNOWN';
    if(defined( $hashRec{'State'}) && $hashRec{'State'}){
	$res = `fr_getFips10.pl --quiet --country \"$hashRec{'Country'}\" --state \"$hashRec{'State'}\"`; # "
	chomp($res);
	if( $res =~ /[A-Z]{2}[0-9]{2}/){
	    if($DEBUG){
		print STDERR 
		    "getRecordFromServopDbQuery: $hashRec{'Country'} $hashRec{'State'} ->[$res] Fips 10 Ok\n";
	    }
	    $fips10 = $res;
	}else{
	    print main::FDLOG
		"getRecordFromServopDbQuery: *** Fips10 NOT Ok for ", $hashRec{"$flagUseWhat"}, "\n",
		"getRecordFromServopDbQuery: *** country: <$hashRec{'Country'}>\n",
		"getRecordFromServopDbQuery: *** state  : <", uc $hashRec{'State'},">\n",
		"\n";
	}	
    }else{
	if(defined( $hashRec{'City'} ) && $hashRec{'City'}){
	    $res = `fr_getFips10.pl --quiet --country \"$hashRec{'Country'}\" --state \"$hashRec{'City'}\"`; # "
	    chomp($res);
	    if( $res =~ /[A-Z]{2}[0-9]{2}/){
		if($DEBUG){
		    print STDERR 
			"getRecordFromServopDbQuery: $hashRec{'Country'} $hashRec{'City'} ->[$res] Fips 10 Ok\n";
		}
		$fips10 = $res;
	    }else{
		print main::FDLOG
		    "getRecordFromServopDbQuery: ", $hashRec{"$flagUseWhat"} ," has no FIPS10\n";
	    }
	}else{
	    print main::FDLOG
		"getRecordFromServopDbQuery: ", $hashRec{"$flagUseWhat"} ," has no FIPS10\n";
	}
    }

    ##
    ## create a Hash from the record
    ##
    $thisAip={
 	ID       => $hashRec{'PartnerID'},
 	NAME     => $hashRec{"$flagUseWhat"},
 	STATUS   => "DOWN",
 	RATE     => '0',
 	HOSTNAME => $hashRec{'HostName'},
 	IP       => $hashRec{'IP1'},
 	REGION   => sprintf("[%s], %s %s, %s", 
			    $fips10, $hashRec{'State'}, $hashRec{'City'}, $hashRec{'Country'}),
 	COMPANY  => $hashRec{'Company'},
 	BANDWIDTH=> $hashRec{'BandwidthFreedom'}
 	};

    if($DEBUG){
	print "getRecordFromServopDbQuery:>>>>>>>>>>>>\n"; 
	printOutRecord($thisAip);
	print "<<<<<<<<<<<<<<<<\n";
    }

    return $thisAip;
}


##
##
##
##
sub createGraphFile {
    my($aipL, $aipN, $graphFile, $dotFlag) = @_;

    open(FDOUT, ">$graphFile");

    ## checks to Create the DotFile
    ##
    if(defined($dotFlag)){
	print FDOUT "graph G {\n",
	"nodesep=20.;\n",
	"//edge [len=1.5];\n",
#	"size=\"10,17\";\n",
	"size=\"7.5,10\";\n",
	"ratio=compress;\n",
	"concentrate=true;\n",
	"\tnode [ fontname=Arial, fontsize=18]\;\n";
    }

    my @graph =  createConnectionGraph($aipL, $aipN, $dotFlag);

    if(defined($dotFlag)){
	foreach $f (@graph){
	    $f =~ s/([a-z0-9]+)\-/$1_/gi;
	    print FDOUT $f,"\n";
	}
    }else{
	print FDOUT join("\n", @graph),"\n";
    }

    my($date) = scalar localtime;
#    $date =~ s/[0-9]+:[0-9]+:[0-9]+//;
    if(defined($dotFlag)){
	print FDOUT
	    "\tlabel = \"\\n\\nFreedom Network: $date\\n\"\;\n",
	    "\tfontsize=18;\n",
	    "\tfontname=Arial;\n",
	    "}\n";
    }
    close(FDOUT)
}

1;

#   $Log: fr_subroutines.pl,v $
#   Revision 1.15  1999/10/08 19:11:32  thierry
#   add TOPOTOOLDIR environment for execution
#
#   Revision 1.14  1999/09/15 17:37:02  thierry
#   createConnectionGraph logs when creating a dot file
#
#   Revision 1.13  1999/09/15 17:16:19  thierry
#   added support to use the actual real bandwidth of freedom network from nidb
#
#   Revision 1.12  1999/09/14 21:30:36  thierry
#   	* Fix parsing in getConnection
#   	* dump fips errors in log file
#
#   Revision 1.11  1999/09/14 17:24:25  thierry
#   frFips10 quiet mode added
#
#   Revision 1.10  1999/09/14 15:56:58  thierry
#   Fips10 in internal aipRecord
#
#   Revision 1.9  1999/09/14 15:43:45  thierry
#   Fips10 support added
#
#   Revision 1.8  1999/09/13 17:51:18  thierry
#   * better parsing of bandwidth in getRecordFromServopDbQuery
#   add printOutAipInfo subroutine
#   add some debug information
#   replace - by _ in generated dot files
#
#   Revision 1.7  1999/09/13 12:59:06  luc
#   * Corrected fonts
#   * Enabled display of date
#   * Change dotted to dashed for generation of GIF files.
#
#   Revision 1.6  1999/09/10 23:06:39  thierry
#   fix problem in insertRecordFromServopDbQuery
#
#   Revision 1.5  1999/09/10 19:33:24  thierry
#   clean debug mode
#
#   Revision 1.4  1999/09/10 17:09:46  thierry
#   sync both to avoid aipsBandwidth file use
#
#   Revision 1.3  1999/09/10 17:02:06  thierry
#   add better logs, getBandwidth use now servop DB
#
#   Revision 1.2  1999/09/10 15:19:32  thierry
#   add a checkFind executable subroutine; summary to checkInfoForAip
#
#   Revision 1.1  1999/09/09 21:51:18  thierry
#   Tool to add AIP to freedom network
#
