#!/usr/bin/perl -w
#
# This script attempts to analyse the results of the freedom network
# probe results.
#
# Bug: Need to make this a proper Perl5 program
#
# Luc Boulianne <luc@zks.net>
# $Id: fr_summarize_probe.pl,v 1.2 1999/09/09 17:05:48 thierry Exp $
#
use strict;
use FileHandle;

######################################################################
##
## Argument 
##
use Getopt::Long;


## Get the Options
##
my %optctl  = ();
GetOptions(\%optctl, "help",
	   "nosortbyaip");
STDERR->autoflush();
STDOUT->autoflush();

######################################################################
## subroutines
##
sub gather_host_info {
  my ($line) = $_;
  my ($dummy, $entity, $ip, $hostname);
  
  while (<>) {
    last if (/^\[ / || /^\/freedom\/sbin\/fr_getaips: Failure to complete query/);
    chomp;
    if (/^Entity/) {
       ($dummy, $entity) = split(/[ ]+=[ ]+/);
    }
    if (/^Ipaddr/) {
       ($dummy, $ip) = split(/[ ]+=[ ]+/);
     }
    if (/^Hostname/) {
       ($dummy, $hostname) = split(/[ ]+=[ ]+/);
    }
#    print "gather_host_info: $_\n";
  }

#  print "***gather_host_info: <$entity> <$hostname> <$ip>\n";
  return ($entity, $hostname, $ip);
}
sub getaverage {
  my ($m1, $m2, $m3) = @_;
  my ($samples, $ave) = 3;

  if ($m1 == 0 && $m2 == 0 && $m3 == 0) {
    return -1;
  }

  $ave = ($m1 + $m2 + $m3) / $samples;

  return $ave;
}

sub gather_trace_info {
  my ($line) = @_;
  my ($dummy, $src, $dest, $last, $tracing, $ave, $problem, @trace, $i);

  if (! (($src, $dest) = ($line =~ /\s+(\S+):(\S+)\s*/)) ) {
    die "Error parsing file. See Luc"
  }

  $tracing = 0;
  while (<>) {
    chomp;
    if (/^ 1 /) {
      $tracing = 1;
    }
    last if (/^$/ && $tracing);
    $last = $_;
  }
  if ($tracing) {
    # the last line contains the final hop of the traceroute
    # /([0-9]+)\s+\S+\s+([0-9.*]+)\s+.*([0-9.*]+)\s+.*([0-9.*]+)/)) ) {
    # 3  209.3.188.189  5.669 ms !H *  4.839 ms !H
    #18  209.5.124.70  110.841 ms  108.004 ms *
    # 6  146.188.161.154  2.179 ms 146.188.161.182  3.979 ms 146.188.161.154  2.029 ms
    # Hop # ([0-9]+)\s+
    # HopIP  ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|)\s*
    # HopDur ([0-9]+.[0-9]+\s+ms|\*)\s*
    # HopErr (![HNPSFX]|!<[0-9]+>\s*|)
    #
    if ( ! ( @trace = ($last =~
		       /([0-9]+)\s+([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|)\s*([0-9]+.[0-9]+\s+ms|\*)\s*(![HNPSFX]|!<[0-9]+>\s*|)([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|)\s*([0-9]+.[0-9]+\s+ms|\*)\s*(![HNPSFX]|!<[0-9]+>\s*|)([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|)\s*([0-9]+.[0-9]+\s+ms|\*)\s*(![HNPSFX]|!<[0-9]+>\s*|)/ ))) {
      warn "Error traceroute line. See Luc. [$last]";
      return ($src,$dest, 0, -1, "Error parsing output");
    }
    # print join(":",@trace), "\n";
    # remove the ms string, and convert * to 0 
    foreach $i (2,5,8) {
      if (($trace[$i] =~ /([0-9]+.[0-9]+)\s+ms/ )) {
	$trace[$i] = $1;
      } else {
	$trace[$i] = 0;
      }
    }
    $problem = "";
    foreach $i (3,6,9) {
      if (! ($trace[$i] =~ /^$/ )) {
	$problem = $trace[$i];
	last;
      }
    }
    foreach $i (4,7) {
      if (! ($trace[$i] =~ /^$/ )) {
	if ($problem =~ /^$/) {
	  $problem = "route flap";
	} else {
	  $problem = ", route flap";
	}
	last;
      }
    }
    # print join(":",@trace), "\n";
    $ave = getaverage ($trace[2],$trace[5],$trace[8]);
  }
  return ($src, $dest, $trace[0], $ave, $problem);
}

######################################################################
##
## main part
##
my ($entity, $hostname, $ip);
my (%entity, %ip, %revip);
my ($src, $dest, $hops, $ave, $prob);
my (%site, %dest, %hops, %ave);
my (@rejects);

printf ("\n[ Host IP Mappings ]\n\n");
while(<>) {
  if (/^\[ Freedom Locator/ || /^\[ Freedom Network Probe Report \]/ ) {
    ($entity, $hostname, $ip) = gather_host_info ($_) ;
    $entity{$hostname} = $entity;
    $ip{$hostname} = $ip;
    $revip{$ip} = $hostname;
    printf ("[ %-16s %-16s %-16s]\n", $entity, $hostname, $ip);
   }else{
#       print "main REJ: $_";
   }

#  next;
  if (/^\[ Traceroute/) {
    ($src, $dest, $hops, $ave, $prob) = gather_trace_info ($_) ;
    if (($hops != 30) && ($ave != -1) && (length($prob) == 0)) {
      $site{$src} = $src;
      $dest{$dest} = $dest;
      $hops{$src.":".$dest} = $hops;
      $ave{$src.":".$dest} = $ave;
#      printf ("%-16s %-16s %3d %-7.2f %s\n", $src,$dest,$hops,$ave, $prob);
    } else {
      push(@rejects, join(":", $src,$dest,$hops,$ave, $prob));
    }
  }
}

my ($rej, @r);
printf ("\n[ Rejected Entries ]\n\n");
while ( $rej = shift (@rejects) ) {
  ($src, $dest, $hops, $ave, $prob) = split(":", $rej);
  if ($revip{$dest}) {
    $dest = $entity{$revip{$dest}};
  }
#  print "<$src> <$dest> <$hops> <$ave> <$prob>\n";
  printf ("rejected: %-30s %-16s %3d %-7.2f %s\n", $entity{$src}, $dest, $hops,	$ave, $prob);
}

#
# Now we need to figure who's close to whom
# Bug: Doing an insert sort. This is too slow, but acceptable for now
# Bug: Will need to change this into a proper Heap
#
my ($i, @SortedHops, @SortedPaths, @SortedAves, $site);
foreach $site (keys %hops) {
  for ( $i = 0; $i <= $#SortedHops; $i ++) {
#    if ($hops{$site} <= $SortedHops[$i]) {
    if ($ave{$site} <= $SortedAves[$i]) {
      splice (@SortedHops, $i, 0, $hops{$site});
      splice (@SortedPaths, $i, 0, $site);
      splice (@SortedAves, $i, 0, $ave{$site});
#      print join(" ", @SortedHops), "\n";
#      print join(" ", @SortedPaths), "\n";
      last;
    } 
  }
  if ($i > $#SortedHops) {
    push (@SortedAves, $ave{$site});
    push (@SortedHops, $hops{$site});
    push (@SortedPaths, $site);
#    print join(" ", @SortedHops), " [$i $#SortedHops]\n";
#    print join(" ", @SortedPaths), "\n";
  }
}

printf ("\n[ Sorted Neighbours ]\n\n");
printf ("%-20s %-20s %8s %4s\n", "Source", "Dest", "mSecs", "Hops");
printf ("-------------------------------------------------------\n");

my ($buf, %byent);

for ( $i = 0; $i <= $#SortedHops; $i ++) {
  ($src, $dest) = split (":",$SortedPaths[$i]);
  if ($revip{$dest}) {
    $dest = $entity{$revip{$dest}};
  }
  if ($SortedHops[$i] > 1) {
    $buf = sprintf ("%-20s %-20s %-8.2f %4d\n", $entity{$src}, $dest, $SortedAves[$i], $SortedHops[$i]);
    $byent{$entity{$src}} .= $buf;
    printf $buf;
  }
}

if(! defined($optctl{'nosortbyaip'})){
    printf ("\n[ Sorted Links by AIP ]\n\n");
    foreach $src (sort (keys %byent)) {
	printf ("\n%-20s %-20s %8s %4s\n", "Source", "Dest", "mSecs", "Hops");
	printf ("-------------------------------------------------------\n");
	printf  $byent{$src};
    }
}

1;
#  $Log: fr_summarize_probe.pl,v $
#  Revision 1.2  1999/09/09 17:05:48  thierry
#  nosortbyaip option added, fix to stop gather_host_info to parse on fr_getaips failure
#
