#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		fr_topotoolinstall.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		 8 Oct 1999
#
#   Last Revision:	$Date: 1999/10/08 19:08:30 $
#
#   Description:	
#
#   $Revision: 1.1 $
#
#
#   $Id: fr_topotoolinstall.pl,v 1.1 1999/10/08 19:08:30 thierry Exp $	
#
#   Copyright (c) 1999 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Getopt::Long;
use File::Path;
use File::Copy;

my $version = '$Id: fr_topotoolinstall.pl,v 1.1 1999/10/08 19:08:30 thierry Exp $';
my $revision= '$Revision: 1.1 $';
my $progname= 'fr_topotoolinstall.pl';
select(STDOUT); $| = 1;

my($TOPOTOOLDIR);
$TOPOTOOLDIR  = $ENV{'TOPOTOOLDIR'};
if(defined($TOPOTOOLDIR)){
    if($TOPOTOOLDIR =~ /\/$/){
        chop($TOPOTOOLDIR);
    }
}else{
    print STDERR 
      "TOPOTOOLDIR environment variable not defined\n",
      "Need to define where you want to do the install\n";
    exit 0;
}

##
## Options related part
##
my %optctl  = ();
my @helpString = 
    (
     "\n\n",
     "Usage $progname",
     "\t--help                  : this help",
     "\t--debug                 : enter in debug mode",
     "\t--version               : get version number"
     );

my($res) = GetOptions(\%optctl, "help",
		  "debug",
                  "version") || die "Error in Getopt";

## print out the usage string
##
if (defined($optctl{'help'})){
    print join("\n", @helpString),"\n";
    exit 1;
}

## set debug mode on
##
my($DEBUG) = 0;
if (defined($optctl{'debug'}) || defined($ENV{'DEBUG'})){
    $DEBUG = 1;
    print "Debug mode On\n";
}

## subroutine ideally should be in a package
## (ideally should check size of the file and ask
## to replace only if size is different or modification time )
##
sub copyFilesToDir{
  my($filesToCopy, $debug, $path) = @_;
  my($rtnCode) = 1;
  my($strLength);
  $filesToCopy = [$filesToCopy] unless ref $filesToCopy;
  if( $debug ){print join(" ", @$filesToCopy), "\n";}
  my($file);
  print "Copying Files to $path\n";
  foreach $file (@$filesToCopy){
    $strLength = 79 - length($file);
    print "Copying $file", " " x $strLength, "\r";
    $res = copy($file,$path);
    if(!$res){
      print "\n***Error: cannot copy $file\n";
      $rtnCode = 0;
    }
  }
  print "\rDone", " " x 75, "\n";
  return $rtnCode;
}

## check if directory exists
## and create it if not
##
my(@dirToCreate) = (
 		    $TOPOTOOLDIR,
 		    $TOPOTOOLDIR."/archives",
 		    $TOPOTOOLDIR."/bin",
 		    $TOPOTOOLDIR."/doc",
 		    $TOPOTOOLDIR."/var"
 		   );

if ( ! -d "$TOPOTOOLDIR" ){
  print "creating topotooldir: $TOPOTOOLDIR\n";
  mkpath ( \@dirToCreate, 1, 0755); #create dir and printout what is done
}

## copy needed files in bin
## 
my($file);
my(@filesToCopyInBin)=(
		  "fr_aipnetworktool.Readme",
		  "fr_aipnetworktool.pl",
		  "fr_getFips10.pl",
		  "fr_servop_dbquery.pl",
		  "fr_show_network_topo.pl",
		  "fr_subroutines.pl",
		  "fr_summarize_probe.pl",
		  "/home/luc/test/bin/fr_getaips",
		  "/home/luc/test/bin/fr_listroutes"
		);

copyFilesToDir(\@filesToCopyInBin, $DEBUG, "$TOPOTOOLDIR/bin");

## copy needed files in var
## 
my(@filesToCopyInVar)=(
		  "/home/thierry/nsmail/Zks.sbd/ZksTopology",
		  "/home/luc/work/zk/freedom/libs/frnidb/src/fips.h",
		  "/home/luc/netops/isp-affiliates.dat",
		  "/home/luc/netops/server-operators.dat"
		);
copyFilesToDir(\@filesToCopyInVar, $DEBUG, "$TOPOTOOLDIR/var");

## copy needed files in doc
## 
my(@filesToCopyInDoc)=(
		"fr_servop_dbquery.pl.8",
		"fr_summarize_probe.pl.8",
		"fr_aipnetworktool.Readme",
                "nidbtool_command.sample",
		"README"
		);
copyFilesToDir(\@filesToCopyInDoc, $DEBUG, "$TOPOTOOLDIR/doc");

## copy needed files in archives
## 
my(@filesToCopyInArchives)=(
		"archives/gv1.4.tgz",
		"archives/neatoFonts.tgz"
		);
copyFilesToDir(\@filesToCopyInArchives, $DEBUG, "$TOPOTOOLDIR/archives");


1;

#   $Log: fr_topotoolinstall.pl,v $
#   Revision 1.1  1999/10/08 19:08:30  thierry
#   create exportable topotool installation
#
