#!/usr/bin/perl -w
#
#************************************************************************
#
#   File:		getEntityMapping.pl
#
#   Author(s):		Thierry Baron
#
#   Created:		15 Feb 2000
#
#   Last Revision:	$Date: 2000/03/21 19:50:58 $
#
#   Description:	
#
#   $Revision: 1.1 $
#
#
#   $Id: getEntityMapping.pl,v 1.1 2000/03/21 19:50:58 thierry Exp $	
#
#   Copyright (c) 2000 Zero-Knowledge Systems, Inc
#   Please see the copyright notice included in this distribution 
#   for full details.
#
#***********************************************************************/

use strict;
use Getopt::Long;
use FileHandle;

##
## Global Variables
##
my $version = '$Id: getEntityMapping.pl,v 1.1 2000/03/21 19:50:58 thierry Exp $';
my $revision= '$Revision: 1.1 $';
my $progname= 'getEntityMapping.pl';
my $DEBUG = 0;
my $DEFAULT_TOPOLOGY_REPORT = "/turf/thierry/test/tmp/TopoPerAip";
my $DEFAULT_NIQS_HOSTNAME   = "niqs.freedom.net";
my $DEFAULT_NIQS_PORT       = "51107";

##
## autoflush mode
##
STDERR->autoflush(1);
STDOUT->autoflush(1);


##
## exit on error
##
sub ExitOnError {
    print STDERR 
	"***Error $progname $revision\n",
	"***$version\n",
	join("\n", @_),"\n";
    exit 0;
}


##
## Get the mapping probe_net files entry table
##
##         entity -> ip
##         ip     -> entity 
##
sub getProbeEntityMapping {
  my($getAipsTable, $mapping, $mappingType) = @_;
  my ($entity);
  my @tmp;
  my($entityName, $ipAddr);
  foreach $entity (@$getAipsTable) {
    chomp($entity);
    if( $entity =~ /^(.*):RealAddr\s+=\s+([0-9\.]+)/){
      ($entityName, $ipAddr) = ($1,$2);
    }else{
      print STDERR "ERROR   in getProbeEntityMapping: $entity not a valid ip addresse\n";
      next;
    }

    if($DEBUG){print "$entity: ($entityName, $ipAddr)\n";}

    if ($mappingType =~ /entityip/i){
      if(! defined ($$mapping{$entityName})){
	if($DEBUG){print "$entityName $ipAddr\n";}
	$$mapping{$entityName}=$ipAddr;
      }else{
	print STDERR "Error Multiple mapping\n";
	print STDERR "$entityName $$mapping{$entityName}\n";
	print STDERR "$entityName $ipAddr\n";
      }
    }elsif($mappingType =~ /ipentity/i){
      if(! defined ($$mapping{$ipAddr})){
	if($DEBUG){print "$ipAddr $entityName\n";}
	$$mapping{$ipAddr}=$entityName;
      }else{
	print STDERR "Error Multiple mapping\n";
	print STDERR "$ipAddr $$mapping{$ipAddr}\n";
	print STDERR "$ipAddr $entityName\n";
      }
    }else{
      print "***Error: not valid mapping type ipEntity or entityIp ($mappingType)\n$entity\n";
    }
  }
  return;
}

##
## Get the mapping from fr_getaips
##
##         entity -> ip
##         ip     -> entity 
##
sub getAipsEntityMapping {
  my($getAipsTable, $mapping, $mappingType) = @_;
  my ($entity);
  my @tmp;

  foreach $entity (@$getAipsTable) {
    chomp($entity);
    $entity =~ s/^\s+//gi;
    $entity =~ s/\s+/ /gi;
    $entity =~ s/\s+^//gi;
    @tmp = split(/\s+/, $entity);

    if ($mappingType =~ /entityip/i){
      if(! defined ($$mapping{$tmp[1]})){
	if($DEBUG){print "$tmp[1] $tmp[5]\n";}
	$$mapping{$tmp[1]}=$tmp[5];
      }else{
	print STDERR "Error Multiple mapping\n";
	print STDERR "$tmp[1] $$mapping{$tmp[1]}\n";
	print STDERR "$tmp[1] $tmp[5]\n";
      }
    }elsif($mappingType =~ /ipentity/i){
      if(! defined ($$mapping{$tmp[5]})){
	if($DEBUG){print "$tmp[5] $tmp[1]\n";}
	$$mapping{$tmp[5]}=$tmp[1];
      }else{
	print STDERR "Error Multiple mapping\n";
	print STDERR "$tmp[5] $$mapping{$tmp[5]}\n";
	print STDERR "$tmp[5] $tmp[1]\n";
      }
    }else{
      print "***Error: not valid mapping type ipEntity or entityIp ($mappingType)\n$entity\n";
    }
  }
  return;
}

##
## Get the aips info
##
##
sub getAipsInfo {
  my($getAipsTable, $niqsHostname, $niqsPort) = @_;
  @$getAipsTable = `fr_getaips --long --nodns --niqs.hostname $niqsHostname --niqs.port $niqsPort`;
  shift(@$getAipsTable);
  shift(@$getAipsTable);
  return;
}

##
## Get the aips info from probe
##
##
sub getProbeInfo {
  my($getAipsProbeTable, $probeDirectory) = @_;
  @$getAipsProbeTable = `cd $probeDirectory; grep RealAddr *`;
  return;
}

##
## Get the mapping directly
##
##
sub getAipsMapping {
  my($mappingHash, $niqsHostname, $niqsPort, $whichMapping) = @_;

  if( ($whichMapping !~ /entityIp/gi) && ($whichMapping !~ /ipEntity/gi)){
    ExitOnError("getMapping", "Bad mapping option");
  }

  my @aipsInfoTable =();
  getAipsInfo(\@aipsInfoTable, $niqsHostname, $niqsPort);
  getAipsEntityMapping(\@aipsInfoTable, $mappingHash, $whichMapping);

  return 1;
}

sub getProbeMapping {
  my($mappingHash, $directoryName, $whichMapping) = @_;

  my @aipsProbeInfoTable =();
  getProbeInfo(\@aipsProbeInfoTable, $directoryName);
  getProbeEntityMapping(\@aipsProbeInfoTable, $mappingHash, $whichMapping);
  return 1;
}

##
## Get the mapping directly
##
##
sub diffProbeGetAipsMappings {
  my($niqsHostname, $niqsPort, $directoryName) = @_;

  my %getAipsHash=();
  my %probeHash  =();

  getAipsMapping(\%getAipsHash, $niqsHostname, $niqsPort, "entityip");
  getProbeMapping(\%probeHash, $directoryName, "entityip");

  if(defined scalar %getAipsHash){
    my $key;
    foreach $key (keys %getAipsHash){

      ## test validity of data
      ##
      if (defined($probeHash{$key})){
	if( $getAipsHash{$key} !~ $probeHash{$key}){
	  print STDERR "ERROR   in compareProbegetGetaipsMapping: entity $key ips differs fr_getaips=$getAipsHash{$key} probe=$probeHash{$key}\n";
	}

      }else{
	print STDERR sprintf("WARNING in compareProbegetGetaipsMapping: entity %-10s (%-15s) only in getaips\n", $key, $getAipsHash{$key});
      }
    }
  }

  if(defined scalar %probeHash){
    my $key;
    foreach $key (keys %probeHash){

      ## test validity of data
      ##
      if (defined($getAipsHash{$key})){
#	if( $getAipsHash{$key} !~ $probeHash{$key}){
#	  print STDERR "ERROR   in compareProbegetGetaipsMapping: entity $key ips differs fr_getaips=$getAipsHash{$key} probe=$probeHash{$key}\n";
#	}
      }else{
	print STDERR sprintf("WARNING in compareProbegetGetaipsMapping: entity %-10s (%-15s) only in probefiles\n", $key, $probeHash{$key});
      }
    }
  }
  return 1;
}


##
## if program is not run as sub programm do the job
## and print out the information
##
if($0 =~ /$progname/){

  ##
  ## option 
  ##
  my %optctl  = ();
  if (GetOptions(\%optctl, 
		 "compare",
		 "getaips",
		 "niqs-hostname=s",
		 "niqs-port=s",
		 "probe",
		 "probedir=s",
		 "mapping=s"
		) == 0) {
    print "Option not supported\n";
    exit 1;
  }


  my $niqsHostname;
  if(defined($optctl{"niqs-hostname"})){
    $niqsHostname = $optctl{"niqs-hostname"};
  }else{
    $niqsHostname = $DEFAULT_NIQS_HOSTNAME;
  }
  my $niqsPort;
  if(defined($optctl{"niqs-port"})){
    $niqsPort = $optctl{"niqs-port"};
  }else{
    $niqsPort = $DEFAULT_NIQS_PORT;
  }

  my $probedir;
  if(defined($optctl{"probedir"})){
    $probedir = $optctl{"probedir"};
  }else{
    $probedir = $DEFAULT_TOPOLOGY_REPORT;
  }

  if(defined($optctl{"compare"})){
    print "($niqsHostname, $probedir)\n";
    diffProbeGetAipsMappings($niqsHostname, $niqsPort, $probedir);
    exit 1;
  }

  if(!defined($optctl{"mapping"})){
    ExitOnError("Need to define one of the --mapping option: entityIp or ipEntity");
  }else{
    if( ($optctl{"mapping"} !~ /entityIp/gi) && ($optctl{"mapping"} !~ /ipEntity/gi)){
      ExitOnError("Bad mapping option");
    }
  }

  if(! defined($optctl{"getaips"}) && ! defined($optctl{"probe"})){
    ExitOnError(
		"Need to define one of the input option: ",
		"--probe   [--probedir directory]",
		"or",
		"--getaips [--niqs-hostname niqs.network.net] [--niqs-port 51107]"
	       );
  }


  if(defined($optctl{"getaips"})){
    ##
    ## Get the mapping from getaips
    ##
    my %entityMapping=();
    getAipsMapping(\%entityMapping, $niqsHostname, $niqsPort, $optctl{"mapping"});
  
    ##
    ## Print the result
    ##
    if(defined scalar %entityMapping){
      my $key;
      foreach $key (sort keys %entityMapping){
	printf("%-16s %-16s\n", $key, $entityMapping{$key});
      }
    }
  }


  if(defined($optctl{"probe"})){
    ##
    ## Get the mapping from probe files
    ##
    my %entityProbeMapping=();
    getProbeMapping(\%entityProbeMapping, $probedir, $optctl{"mapping"});
    
    ##
    ## Print the result
    ##
    if(defined scalar %entityProbeMapping){
      my $key;
      foreach $key (sort keys %entityProbeMapping){
	printf("%-16s %-16s\n", $key, $entityProbeMapping{$key});
      }
    }
  }
}

1;

#   $Log: getEntityMapping.pl,v $
#   Revision 1.1  2000/03/21 19:50:58  thierry
#   add tools for topology: get number of link for an aip and get entity mapping
#
