#ifndef ACIMAP_H_
#define ACIMAP_H_

#ifndef _ZKHASH_H
#include <zkHash.h>
#endif /* _ZKHASH_H */

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#ifndef LIBPKFMT_H_
#include <frPktFmt.h>
#endif /* LIBPKFMT_H_ */

#ifndef _PORTMAP_H_
#include "portmap.h"
#endif /* _PORTMAP_H_ */

/*
 * The src_addr is used to pass four bytes of information back to the 
 * Freedom client via the IP header source address field.
 */

struct T_FrAciPortKey {
  uint16_t port;
  FrAci aci;
  uint32_t src_addr; 
};

typedef struct T_FrAciPortKey FrAciPortKey;

struct T_FrAciMap {
  uint16_t baseport;        /* Number of first available port */
  ZkHash acimap;            /* Holds the aci/port to port maps */
  int maps;                 /* The number of possible maps */
  FrAciPortKey * reverse;   /* Array to loop up ACI/port from dest port */
  uint16_t * portnum;       /* portnumber array */
  uint16_t * freelist;
  uint16_t   udpTimeout;    /* Timeout (in second) to consider a port has dead */ 
  uint16_t   tcpTimeout;    /* Timeout (in second) to consider a port has dead */ 
  WhAciPortList portmap;    /* List of port map per ACI */
  uint16_t max_port_per_aci;/* Maximum number of port allowed per ACI */
  int freehead;
  int freetail; 
  int freeports;
  int active_acis;          /* Number of active ACIs */
  void * wormhole;          /* Associate Wormhole */
  ZkLog log;
};

typedef struct T_FrAciMap FrAciMap;

int frAciMapCleanup (
  ZkError ,
  FrAciMap * );

int frAciMapInit (
  ZkError ,
  ZkLog ,
  FrAciMap * ,
  uint16_t ,
  int ,
  uint16_t, 
  uint16_t,
  uint16_t,
  void *);


int frAciMapGetSourcePort (
  ZkError ,
  FrAciMap * ,
  uint16_t ,
  FrAci ,
  uint32_t ,    /* The contents of the source address field of the IP header */
  uint16_t ,
  uint16_t * ,
  int * );   /* Returns a 1 if this is a new map */


int frAciMapGetUsedPortCount (
  ZkError ,
  FrAciMap * ,
  uint16_t * );


int frAciMapGetDestPortAndAci (
  ZkError ,
  FrAciMap * ,
  uint16_t ,
  uint16_t * ,
  FrAci * ,
  uint32_t * );  /* Four bytes of information which are sent back to the Freedom 
                * client via the IP header's destination address field.
                */

int frAciMapGetBaseport(
  ZkError    err,        /* Return the error if one occured */
  FrAciMap  *acimap,     /* The current aci mapping structure */
  uint16_t  *baseportP   /* Return the baseport of the mapping */
);

int frAciMapAddAci (
  ZkError ,              /* Return the error if one occured */
  FrAciMap * ,           /* The current aci mapping structure */
  FrAci );               /* The new aci number which is being added */


int frAciMapDeleteAci (
  ZkError ,
  FrAciMap * ,
  FrAci );


int frAciMapActiveAcis (
  ZkError ,              /* Return the error if one occured */
  FrAciMap * ,           /* The aci map structure to analize */
  int * );               /* Return the number of currently active ACIs */



#endif /* ACIMAP_H_ */

