#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#define _GNU_SOURCE
#include <string.h>
#include <errno.h>
#if defined(__sparc__)
#include <stropts.h>
#endif /* __sparc__ */
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#if defined(__linux__)
#include <linux/if_ether.h>
#endif /* __linux__ */
#include <sys/un.h>
#include "headers.h"
#include "worm.h"

#define	__ARP_DATAGRAM_SZ       4096

#define PRO_ALEN 4
#define	__ARPREQ_SZ (sizeof(struct ethhdr) + \
                     sizeof(struct arphdr) + \
                     sizeof(struct arp_req))

struct arp_req {
  unsigned char sender_hrd[ETH_ALEN];
  unsigned char sender_pro[PRO_ALEN];
  unsigned char target_hrd[ETH_ALEN];
  unsigned char target_pro[PRO_ALEN];
};

/*
 * These functions are no longer used.
 */

#if 0
static void dumpEthernetHeader ( FILE * ,struct ethhdr * );
#endif
void dumpARPHeader ( FILE * , struct arphdr * );
void dumpARPRequest ( FILE * , struct arp_req * );

static void convertToReply ( struct ethhdr * , char * );

/*
 * This function is not used.
 */

#if 0
static void
dumpEthernetHeader (
  FILE * stream ,
  struct ethhdr * ethh )
{
  int i = 0;  /* Loop index */
  
  fprintf(stream,"Destination Ethernet Address:  %02X",ethh->h_dest[0]);
  for (i = 1; i < ETH_ALEN ; i++) {
    fprintf(stream,":%02X",ethh->h_dest[i]);
  }
  fprintf(stream,"\nSource Ethernet Address:  %02X",ethh->h_source[0]);
  for (i = 1; i < ETH_ALEN ; i++) {
    fprintf(stream,":%02X",ethh->h_source[i]);
  }
  fprintf(stream,"\nPacket type ID field:  0x%X\n",ethh->h_proto);
  return;
}
#endif

/*
 * This function is not used.
 */

#if 1
void
dumpARPHeader (
  FILE * stream ,
  struct arphdr * arph )
{
  fprintf(stream,"Hardware Address Format 0x%X\n",ntohs(arph->ar_hrd));
  fprintf(stream,"Protocol Address Format 0x%X\n",ntohs(arph->ar_pro));
  fprintf(stream,"Length of hardware address %d\n",arph->ar_hln);
  fprintf(stream,"Length of protocol address %d\n",arph->ar_pln);
  fprintf(stream,"ARP opcode command 0x%X\n",ntohs(arph->ar_op));
  return;
}
#endif

/*
 * This function is not used.
 */

#if 1
void
dumpARPRequest (
  FILE * stream ,
  struct arp_req * arpreq )
{
  int i = 0; 

  fprintf(stream,"Sender hardware address:  %02X",arpreq->sender_hrd[0]);
  for (i = 1; i < ETH_ALEN; i++) {
    fprintf(stream,":%02X",arpreq->sender_hrd[i]);
  }
  fprintf(stream,"\nSender protocol address: %d",arpreq->sender_pro[0]);
  for (i = 1; i < PRO_ALEN; i++) {
    fprintf(stream,".%d",arpreq->sender_pro[i]);
  }
  fprintf(stream,"\nTarget hardware address:  %02X",arpreq->target_hrd[0]);
  for (i = 1; i < ETH_ALEN; i++) {
    fprintf(stream,":%02X",arpreq->target_hrd[i]);
  }
  fprintf(stream,"\nTarget protocol address: %d",arpreq->target_pro[0]);
  for (i = 1; i < PRO_ALEN; i++) {
    fprintf(stream,".%d",arpreq->target_pro[i]);
  }
  fprintf(stream,"\n");
  return;
}
#endif

/**
 *  This function was developed for Linux.
 */

static void
convertToReply (
  struct ethhdr * ethh ,
  char * addr )
{
  const char * func = "convertToReply";
  struct arphdr * arph = NULL; /* ARP command structure */
  struct arp_req * arpreq = NULL; /* ARP structure for request addresses */ 
  char tmp = 0; /* Used to swap the sender and target IP addresses */
  int i = 0; /* Loop index */

  (void)func;

  arph = (struct arphdr *) &(ethh[1]);
  arpreq = (struct arp_req *) &(arph[1]);

  /*
   * Ethernet source address now becomes the destination address
   * Source address is the one we have just specified.
   */

  for (i = 0; i < ETH_ALEN; i++) {
    ethh->h_dest[i] = ethh->h_source[i];
    ethh->h_source[i] = addr[i];
  }
  
  /*
   * ARP sender addresses now become the target IP and the ethernet address
   * we have specified.  The target address become what was in the sender
   * fields of the arp request.
   */

  for (i = 0; i < ETH_ALEN; i++) {
    arpreq->target_hrd[i] = arpreq->sender_hrd[i];
    arpreq->sender_hrd[i] = addr[i];
  }
  for (i = 0; i < PRO_ALEN; i++) {
    tmp = arpreq->sender_pro[i];
    arpreq->sender_pro[i] = arpreq->target_pro[i];
    arpreq->target_pro[i] = tmp;
  }

  /*
   * Now set the ARP command code to be a reply
   */

  arph->ar_op = htons(2);

  return;
}

/**
 *  The wrapper for the Solaris ARP request is a bit different because I 
 *  already have the packet in this case.  All I need to handle it is the
 *  pointer to the beginning of the ethernet frame.
 *
 */

#if defined(__sparc__)
int
zkWormholeHandleARPRequestSolaris (
  ZkError err ,
  ZkWormhole * wormhole ,
  unsigned char * data ,
  int datalen )
{
  const char * func = "zkWormholeHandleARPRequestSolaris";
#define ARP_OFFSET 14
#define ARPREQ_OFFSET (ARP_OFFSET + 8) 
#define TARGET_HA_OFFSET (ARPREQ_OFFSET + 10)
#define SENDER_HA_OFFSET (ARPREQ_OFFSET + 0)
#define TARGET_PA_OFFSET (ARPREQ_OFFSET + 16)
#define SENDER_PA_OFFSET (ARPREQ_OFFSET + 6)
#define OPERATION_OFFSET (ARP_OFFSET + 6)
  unsigned char * target = NULL;
  unsigned char * waddr = NULL;
  struct strbuf response;
  int i = 0; /* Loop index */
  

  zkErrorClear(err);
  waddr = (unsigned char *) &(wormhole->wormhole_addr.sa_data[2]);
  target = &(data[TARGET_PA_OFFSET]);
  if (memcmp(waddr,target,4) == 0) {
    
    /*
     * First, build an ARP response.  Since most of the data I need is in
     * the data stream passed to this function, and it is not used by 
     * anything else afterwards, I just recycle the buffer to answer the
     * request.
     */

    for (i = 0; i < 6; i++) {
      data[i] = data[i + 6];
      data[i + 6] = wormhole->hardware_addr.sa_data[i];
      data[TARGET_HA_OFFSET + i] = data[SENDER_HA_OFFSET + i];
      data[SENDER_HA_OFFSET + i] = wormhole->hardware_addr.sa_data[i];
    }
    for (i = 0; i < 4; i++) {
      data[TARGET_PA_OFFSET + i] = data[SENDER_PA_OFFSET + i];
      data[SENDER_PA_OFFSET + i] = waddr[i];
    }
    data[OPERATION_OFFSET] = 0;
    data[OPERATION_OFFSET + 1] = 2;

    /*
     * Now send the ARP response to the network layer.
     */
   
    response.maxlen = 70;
    response.buf = data;
    response.len = 70;

    if (putmsg(wormhole->read_sock,NULL,&response,0) < 0) {
      zkErrorSet(err,"zkWormholeHandleARPRequestSolaris",ZKERR,ZKMWORMHOLE,
                 ZKESOCKET,"putmsg failed");
      return 0;
    }
 
  }    
  return 1;
}
#endif /* __sparc__ */
  
int
zkWormholeHandleARPRequest(
  ZkError err ,
  ZkWormhole * wormhole )
{
  const char * func = "zkWormholeHandleARPRequest";
  struct ethhdr * ethh = NULL;
  struct arphdr * arph = NULL;
  struct arp_req * arpreq = NULL;
  unsigned char datagram[__ARP_DATAGRAM_SZ];
  struct sockaddr_in remote;
  struct sockaddr_in * waddr = (struct sockaddr_in *) 
                               &(wormhole->wormhole_addr);
  int rlen;
  int dlen = 0;
  unsigned long  target ;
  int bytes = 0;  /* Number of bytes written for the ARP reply */
  struct sockaddr ni;
  int ret = 1;

  (void)func;

  zkErrorClear(err);

  rlen = sizeof(struct sockaddr_in);
  dlen = recvfrom(wormhole->arp_sock,(char *)datagram,__ARP_DATAGRAM_SZ,0,
                  (struct sockaddr *)&remote,&rlen);
  if( dlen == -1 ) {
    zkErrorSystemSet(err, "zkWormholeHandleARPRequest", ZKERR, ZKMWORMHOLE,
                     "recvfrom failed");    
    ret = 0;
  } else {
    ethh = (struct ethhdr *) &(datagram[0]);
    arph = (struct arphdr *) &(ethh[1]);
    arpreq = (struct arp_req *) &(arph[1]);
    target = *((unsigned long *) &(arpreq->target_pro[0]));
    if (target == waddr->sin_addr.s_addr) {
      memset(&ni,0,sizeof(struct sockaddr));
      ni.sa_family = AF_INET;
      strcpy(ni.sa_data,wormhole->interface); /* bug: audit - nuls? */
      convertToReply(ethh,wormhole->hardware_addr.sa_data);
      bytes = sendto(wormhole->arp_sock,(char *)ethh,__ARPREQ_SZ,
                     0,&ni,sizeof(ni));
      if (bytes < 0) {
        zkErrorSystemSet(err, "zkWormholeHandleARPRequest", ZKERR, ZKMWORMHOLE,
                         "sendto failed");
        ret = 0;
      }
    }
  }
  return ret;
}
  

