#ifdef USE_DLPI

#include "zks.h"

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stropts.h>
#include <sys/socket.h>
#include <sys/dlpi.h>
#include <zkError.h>
#include "dlpi.h"

#define __ADDRBUF_SZ          32     /* Size of the SAP address buffer */

/**
 *  Verify an expected acknowledgement from a Solaris STREAM.  This function 
 *  is based on a tutorial example by Neal Nuckolls, "How to use DLPI" and
 *  on the pcap library.
 * 
 *  @param err      Returns the error if any occured
 *  @param fd       File Descriptor from which to read the acknowledgement
 *  @param buffer   Buffer space to hold the response
 *  @param bufsz    Size of the buffer
 *  @author         Philippe Boucher
 *  @version        1.0
 *  @return         A non-zero value on success, a 0 on failure.
 */

int
zkDLPIVerifyAck (
  ZkError err ,
  int fd ,
  char * buffer ,
  size_t bufsz )
{
  const char * func = "zkDLPIVerifyAck";
  union DL_primitives * dlp;
  struct strbuf ctl;
  int flags = 0;

  zkErrorClear(err);
  ctl.maxlen = bufsz;
  ctl.len = 0;
  ctl.buf = buffer;
  if (getmsg(fd,&ctl,NULL,&flags) < 0) {
    zkErrorSystemSet(err,"zkDLPIVerifyAck",ZKERR,ZKMWORMHOLE,"getmsg failed");
    return 0;
  }
  dlp = (union DL_primitives *) ctl.buf;
  if (dlp->dl_primitive == DL_ERROR_ACK) {
    zkErrorSet(err,"zkDLPIVerifyAck",ZKERR,ZKMWORMHOLE,ZKERROR,
               "Acknowledgement error");
    return 0;
  }
  if (ctl.len < sizeof(dl_ok_ack_t)) {
    zkErrorSet(err,"zkDLPIVerifyAck",ZKERR,ZKMWORMHOLE,ZKERROR,
               "response too short");
    return 0;
  }
  if (flags != RS_HIPRI) {
    zkErrorSet(err,"zkDLPIVerifyAck",ZKERR,ZKMWORMHOLE,ZKERROR,
               "DL_OK_ACK not M_PCPROTO");
    return 0;
  }
  return 1;
}
  

int
zkDLPIAddAddress (
  ZkError err ,
  int fd ,
  struct sockaddr * phys ,
  struct sockaddr * sap )
{
  const char * func = "zkDLPIAddAddress";
  unsigned char addrbuf[__ADDRBUF_SZ];
  int i = 0;
  
  zkErrorClear(err);


  return 1;
}

#endif /* USE_DLPI */
