/*
 * $Id: icmp.c,v 1.4 2001/04/19 19:14:21 alainc Exp $
 */

static const char * module_id_ = "$Id: icmp.c,v 1.4 2001/04/19 19:14:21 alainc Exp $";

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <zkError.h>
#include <zkPacket.h>
#include <netinet/in.h>
#include "icmp.h"
#include "zkPackBuf.h"

/*
 * Take the ICMP echo request packet that the wormhole received and turn it
 * around. 
 */

int
icmpTransformToEchoReply (
  ZkError err ,
  void /* ZkIPHeader */ * iph )
{
  static const char * func = "icmpTransformToEchoReply";
  void /* ZkICMPHeader */ * icmph = NULL;
  int return_code = 0;
  uint32_t addr_t = 0;

  (void) module_id_; /* Get rid of warning */
  zkErrorClear(err);

  if (iph) {
    /* Strip the options if any are present and get pointer to the ICMP 
     * header 
     */
    zkIPHeaderStripOptions(iph);
    icmph = (void /* ZkICMPHeader */ *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SIZEOF);
    if ((zkIPHeaderGetProtocol(iph) == ZK_IPPROTO_ICMP) && 
        (zkICMPHeaderGetType(icmph) == 8) && (zkICMPHeaderGetCode(icmph) == 0)) {
    

      /* Swap the addresses */
      addr_t = zkIPHeaderGetSrcIpAddressNBO(iph);
      zkIPHeaderSetSrcIpAddressNBO(iph, zkIPHeaderGetDstIpAddressNBO(iph));
      zkIPHeaderSetDstIpAddressNBO(iph, addr_t);
      zkIPHeaderSetHeaderChkSumNBO(iph, 0);

      /* Now make the ICMP packet an echo reply */
      zkICMPHeaderSetType(icmph, 0);
      zkICMPHeaderSetChecksum(icmph, 0);

      zkICMPHeaderChecksum(icmph,zkIPHeaderGetTotalLength(iph) - zkIPHeaderGetIHL(iph)*4);

      zkIPHeaderChecksum(iph);

      return_code = 1;  

    } else {
      unsigned char * addr = (unsigned char *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SRCIPADDRESS);
      zkErrorSet_va(err,func,ZKERR,ZKMWORMHOLE,ZKEDATABAD,
                   "ICMP packet from %d.%d.%d.%d is not an echo request "
                   "(type = %d, code = %d)",
                   addr[0],addr[1],addr[2],addr[3],
                   zkICMPHeaderGetType(icmph),zkICMPHeaderGetCode(icmph));
    }
  } else {
    zkErrorSet(err,func,ZKERR,ZKMWORMHOLE,ZKENULLPTR,NULL);
  }
  return return_code;
}

  
