
/* 
 * Mapping of a ACI with the list of Port it's owning
 *
 * Nicolas Roy <nicolas@zeroknowledge.com>
 * $Id: portmap.h,v 1.1 1999/09/29 20:48:46 nicolas Exp $
 */
#ifndef _PORTMAP_H_
# define _PORTMAP_H_

# include "zks.h"
# include "zkIntTypes.h"
# include "zkTypes.h"
# include "zkError.h"


#define TCP_STATE_NONE                 (0x0000U)
#define TCP_STATE_URG                  (0x0001U)
#define TCP_STATE_ACK                  (0x0002U)
#define TCP_STATE_PSH                  (0x0004U)
#define TCP_STATE_RST                  (0x0008U)
#define TCP_STATE_SYN                  (0x0010U)
#define TCP_STATE_FIN                  (0x0020U)



typedef struct S_WhAciPortList * WhAciPortList;

extern int whAciPortListNew
(
 ZkError        err,
 WhAciPortList *listP,
 uint32_t       max_aci,
 uint16_t       max_port_per_aci
 );

extern int whAciPortListFree
(
 ZkError        err,
 WhAciPortList *listP
 );

extern int whAciPortListPortAdd
(
 ZkError        err,
 WhAciPortList  list,
 uint32_t       aci,
 uint16_t       port,
 uint16_t       ipproto
 );

extern int whAciPortListPortRem
(
 ZkError        err,
 WhAciPortList  list,
 uint32_t       aci,
 uint16_t       port
 );

extern int whAciPortListPortRemAll
(
 ZkError        err,
 WhAciPortList  list,
 uint32_t       aci
 );

extern int whAciPortListPortCount
(
 ZkError        err,
 WhAciPortList  list,
 uint32_t       aci,
 uint16_t      *countP
 );

extern int whAciPortListPortMarkIo
(
 ZkError        err,
 WhAciPortList  list,
 uint32_t       aci,
 uint16_t       port,
 uint16_t       lastTcpState,
 uint32_t       ack
 );

extern int whAciPortListPortAvailableCount
(
 ZkError        err,                   /* [in]  ZkError */
 WhAciPortList  list,                  /* [in]  WhAciPortList */
 uint32_t       aci,                   /* [in]  ACI to scan for */
 uint16_t       udpTimeout,            /* [in]  time (in second) to mark a timeout (UDP connection) */
 uint16_t       tcpTimeout,            /* [in]  time (in second) to mark a timeout (TCP connection) */
 uint16_t      *countP
 );

extern int whAciPortListPortInfoGet
(
 ZkError         err,
 WhAciPortList   list,
 uint32_t        aci,
 uint16_t        port,
 uint16_t       *ipprotoP,
 uint16_t       *lastTcpStateP,
 uint32_t       *ackP          
 );

extern int whAciPortListPortAvailable
(
 ZkError        err,                   /* [in]  ZkError */
 WhAciPortList  list,                  /* [in]  WhAciPortList */
 uint32_t       aci,                   /* [in]  ACI to scan for */
 uint16_t       udpTimeout,            /* [in]  time (in second) to mark a timeout (UDP connection) */
 uint16_t       tcpTimeout,            /* [in]  time (in second) to mark a timeout (TCP connection) */
 ZkBool        *foundP,                /* [out] return ZK_TRUE if a port is available */
 ZkBool        *lastP,                 /* [out] return ZK_TRUE if a port is the last port available */
 uint16_t      *portP                  /* [out] return port number */
 );

#endif /* _PORTMAP_H_ */


