#ifndef REDIRECT_H_
#define REDIRECT_H_

#ifndef _ZKHASH_H
#include <zkHash.h>
#endif /* _ZKHASH_H */

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#define MAX_REDIRECTS 32

struct T_FrAddrPortKey {
  uint32_t addr;
  uint16_t port; /* Source port */
  uint16_t dest; /* Destination port */
};

typedef struct T_FrAddrPortKey FrAddrPortKey;

struct T_RedirectMap {
  FrAddrPortKey original; /* Original address and ports */
  FrAddrPortKey new;  /* new address and ports */
  uint32_t ack; /* last sequence number, used to kill stream */
  uint32_t seq; /* Last acknumber, user to kill stream */
  time_t birth; /* Time when map was born */
};

typedef struct T_RedirectMap RedirectMap;

struct T_FrRedirectMap {
  uint32_t hostaddr;                       /* Localhost IP address */
  uint16_t baseport;                       /* Base redirection port */
  int maps;                                /* Maximum number of maps */
  int tail;                                /* Next map insert position */
  RedirectMap * map;                       /* Map array */
  int * ndx;                               /* Hold port index data for zkhash */
  ZkHash mapht;                            /* active source addr + port */
  int redirects;                           /* Total number of redirections */
  int minlife;                             /* Minimum redirect duration */
  uint16_t original_port[MAX_REDIRECTS];
  uint16_t new_port[MAX_REDIRECTS];
  ZkLog log;
};

typedef struct T_FrRedirectMap FrRedirectMap;

int frRedirectMapCleanup (
  ZkError err ,
  FrRedirectMap *rmap );

int redirectKeyFree (
  ZkHash h ,
  void * key ,
  void * userdata );

int frRedirectMapInit (
  ZkError err ,
  ZkLog log,
  FrRedirectMap * rmap ,
  uint16_t baseport ,
  int maps ,
  int minlife ,
  uint32_t hostaddr );

int frRedirectMapAddEntry (
  ZkError err ,
  FrRedirectMap * rmap ,
  uint16_t original_port ,
  uint16_t new_port );


#endif /* ACIMAP_H_ */

