#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkPacket.h>
#include "worm.h"


/**
 *  Send a TCP reset to a remote host to kill an active connection.
 *  This is a special purpose function used to terminate connections when
 *  a redirect map is recycled for use with a new machine or when a connection
 *  is refused. 1 is automatically added to the acknowledgement number
 */

int
zkWormholeResetTcpConnection (
  ZkError err ,
  ZkWormhole * wormhole ,
  uint32_t target ,
  uint16_t srcport ,
  uint16_t dstport ,
  uint32_t ack)
{
  const char * func = "zkWormholeResetTcpConnection";
#define PACKET_SZ 40
  unsigned char packet[PACKET_SZ];
  struct sockaddr_in * waddr = NULL;


  zkErrorClear(err);
  waddr = (struct sockaddr_in *) &(wormhole->wormhole_addr);
  target = htonl(target);  /* put the address in network byte order */
  ack = htonl(ack + 1); /* Put the ack in network byte order */
  packet[0] = 0x45; /* Version and header length */
  packet[1] = 0; /* Type of service */
  packet[2] = PACKET_SZ / 0x100; /* First byte of total packet length */
  packet[3] = PACKET_SZ % 0x100; /* Second byte of total packet length */
  packet[4] = 0xfe; /* First half of identification */
  packet[5] = 0xed; /* Second half of identification */
  packet[6] = 0; /* Fragmentation flags and first half of frag offset */
  packet[7] = 0; /* Second half of frag offset */
  packet[8] = 255; /* Time to live */
  packet[9] = ZK_IPPROTO_TCP; /* TCP Protocol type */
  packet[10] = 0; /*First half of checksum */
  packet[11] = 0; /* Second half of checksum */
  memcpy((void *)&(packet[12]),(void *)&(waddr->sin_addr.s_addr),4);
  memcpy((void *)&(packet[16]),(void *)&target,4);
  packet[20] = srcport / 0x100; /* First half of source port */
  packet[21] = srcport % 0x100; /* Second half of source port */
  packet[22] = dstport / 0x100; /* First half of destination port */
  packet[23] = dstport % 0x100; /* Second hald of destination port */
  packet[24] = 0;
  packet[25] = 0;
  packet[26] = 0;
  packet[27] = 0;   /* Sequence number */
  memcpy((void *)&(packet[28]),(void *) &ack,4); /* Acknowledgement */
  packet[32] = 0x50; /* Header size + 4 reserved bits set to 0 */
  packet[33] = 0x34; /* URG, ACK,RST bits set, reserved bits set to 0 */
  packet[34] = 0;
  packet[35] = 0; /* Window size is set to zero octets */
  packet[36] = 0;
  packet[37] = 0; /* Header checksum is set to 0 */ 
  packet[38] = 0;
  packet[39] = 0; /* Urgent pointer is set to 0 */

  if (!zkWormholeWritePacket(err,wormhole,(void /* ZkIPHeader */ *) packet,PACKET_SZ)) {
    zkLogWriteContext(wormhole->log, err, func, "zkWormholeWritePacket failed");
    return 0;
  }

  return 1;
}






