/* $Id: worm_stats.c,v 1.11 2001/04/19 19:14:21 alainc Exp $ */

#include "zks.h"

#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>
#include "frStatClient.h"
#include "frStatUpd.h"
#include "worm_stats.h"
#include "portmap.h"
#include "worm.h"

/*
 * Send a Stats Update message to the AIP client.
 */
int WormStatUpdate(ZkError err, ZkWormhole * ctx, ZkBool force)
{
  const char * func = "WormStatUpdate";
  uint16_t       errcode = 0;
  const char    *errmesg = NULL; 
  struct timeval currentTime;
  uint32_t       seconds;
  uint16_t       curr_used_ports = 0;
  uint32_t       maxin_thrput = 0;
  uint32_t       current_active_streams = 0;
  char          *msg = NULL;
  size_t         msgSize = 0;


  if( gettimeofday(&currentTime, 0) == -1 ) 
    {
      errcode = errno;
      errmesg = "gettimeofday failed";
      zkErrorSystemSet(err, func, ZKERR, ZKMWORMHOLE, errmesg);
    }
  else
    {
      seconds = currentTime.tv_sec;

      /*
       * Force a stat update or wait until next period 
       */
      if( force || ((seconds - ctx->stats.lastTryStatsTime) > ctx->stats.statsTimeDelta) ) 
        {
          if( !errcode )
            {
              zkLogWrite_va(ctx->log, "[ STATS ]"
                            " received (bytes=%Ld), send (bytes=%Ld)", 
                            ctx->stats.bytesRcvd,
                            ctx->stats.bytesSent);
                
              if( !frAciMapGetUsedPortCount(err, &ctx->fracimap, &curr_used_ports) ) 
                {
                  errcode = zkErrorCodeFromErr(err);
                  errmesg = "frAciMapGetUsedPortCount failed";
                }
              else if( !frAciMapActiveAcis(err, &ctx->fracimap, &current_active_streams) ) 
                {
                  errcode = zkErrorCodeFromErr(err);
                  errmesg = "frAciMapActiveAcis failed";
                }
              else 
                {
                  maxin_thrput = (uint32_t)ctx->bytes_per_sec;

                  /* Send stats to the AIP client */
                  if( !frStatsUpdSetType(err, ctx->stats.statsCtx,
                                         FRSTAT_T_WORMHOLE))
                    {
                       errcode = zkErrorCodeFromErr(err);
                       errmesg = "frStatsUpdSetType failed";
                    }
                  else if( !frStatsFillWormholeMsg(err,
                                             ctx->stats.statsCtx,
                                             ctx->stats.sequence,
                                             ctx->stats.uptime,
                                             ctx->stats.lastStatsTime,
                                             maxin_thrput,
                                             ctx->stats.newAcis,
                                             current_active_streams,
                                             ctx->stats.newPorts,
                                             (uint32_t)curr_used_ports, 
                                             ctx->stats.bytesRcvd,
                                             ctx->stats.bytesSent) ) 
                    {
                      errcode = zkErrorCodeFromErr(err);
                      errmesg = "frStatsFillWormholeMsg failed";
                    }
                  else if( !frStatsBuildMsg(err, ctx->stats.statsCtx, (void **)&msg, &msgSize) )
                    {
                      errcode = zkErrorCodeFromErr(err);
                      errmesg = "frStatsBuildMsg failed";
                    }
                  else if( !zkWormholeSendStatus(err, ctx, msg, msgSize) )
                    {
                      errcode = zkErrorCodeFromErr(err);
                      errmesg = "zkWormholeSendStatus failed";
                    }
                  else
                    {
                      ctx->stats.sequence     += 1;
                      ctx->stats.lastStatsTime = currentTime.tv_sec;
                      ctx->stats.newAcis       = 0;
                      ctx->stats.newPorts      = 0;
                      ctx->stats.bytesRcvd     = 0;
                      ctx->stats.bytesSent     = 0;
                    }
                }
            }
          ctx->stats.lastTryStatsTime = currentTime.tv_sec;          
        }
    }

  if (msg)
  {
    free(msg);
  }

  if( errcode ) 
    {
      zkLogWriteContext(ctx->log, err, func, errmesg);
    }
  return !errcode;
}
