#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#if defined(ZK_SOLARIS)
#include <stropts.h>
#include <sys/sockio.h>
#elif defined(ZK_LINUX)
#include <linux/sockios.h>
#endif /*  */
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include "arpfnc.h"


#define DEV_ARP "/dev/arp"

extern struct ether_addr * ether_aton (
  char * );


/*
 * Set an entry in the system's ARP cache
 */

#if defined(ZK_SOLARIS)
int
arpSet (
  struct sockaddr * paddr,   /* Protocol (IP) address */
  struct sockaddr * haddr,   /* Hardware (ethernet) address */
  int flags )              /* ARP flags */
{
  struct arpreq arpreq;
  struct strioctl strbuf;
  int p = 0;

  /*
   * Gain access to the ARP cache
   */

  if ((p = open(DEV_ARP,O_RDWR)) < 0) {
    perror(DEV_ARP);
    return -1;
  }

  /*
   * Build up the ARP request for setting an entry in the cache
   */

  arpreq.arp_pa = *paddr;
  arpreq.arp_ha = *haddr;
  arpreq.arp_flags = flags;

  strbuf.ic_cmd = SIOCSARP;
  strbuf.ic_timout = 0;
  strbuf.ic_dp = (caddr_t) &arpreq;
  strbuf.ic_len = sizeof(struct arpreq);

  if (ioctl(p,I_STR,&strbuf) < 0) {
    perror("SIOCSARP");
    close(p);
    return -1;
  }

  close(p);
  return 0;
}

#elif defined(ZK_LINUX)

int
arpSet (
  struct sockaddr * paddr,   /* Protocol (IP) address */
  struct sockaddr * haddr,   /* Hardware (ethernet) address */
  int flags )              /* ARP flags */
{
  int p = 0;
  struct arpreq arpreq;

  if ((p = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
    perror(DEV_ARP);
    return -1;
  }

  arpreq.arp_pa = *paddr;
  arpreq.arp_ha = *haddr;
  arpreq.arp_flags = flags;

  if (ioctl(p,SIOCSARP,&arpreq) < 0) {
    perror("SIOCSARP");
    close(p);
    return -1;
  }

  close(p);
  return 0;
}


#endif /* ZK_LINUX */

