#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include "openssl/sha.h"

int ComputeHashForFile ( char * filename , unsigned char * );

#define BUFFER_SZ 1024

int
ComputeHashForFile (
  char * filename ,        /* Filename to check */
  unsigned char * hash )   /* Returns the hash value for the file (20 bytes) */
{
  int fd = 0;
  unsigned char buffer[BUFFER_SZ];
  int bytes = 0;
  SHA_CTX sha;

  fd = open(filename,O_RDONLY);
  if (fd < 0) {
    return 0;
  }
  SHA1_Init(&sha);
  while ((bytes = read(fd,buffer,BUFFER_SZ)) > 0) {
    SHA1_Update(&sha,buffer,bytes);
  }
  close(fd);
  if (bytes < 0) {
    return 0;
  }
  SHA1_Final(hash,&sha);
  return 1;
}

int
main (
  int argc ,
  char ** argv )
{
  char * progname = NULL;     /* Cleaned up program name */
  unsigned char hash[20];     /* Hash value for the specified file */
  int i = 0;                  /* Loop index */
  struct stat sbuf;           /* For getting the size of the file */

  if ((progname = strrchr(argv[0],'/')) != NULL) {
    progname++;
  } else {
    progname = argv[0];
  }

  if (argc != 2) {
    fprintf(stderr,"usage:  %s filename\n\n",progname);
    exit(EXIT_FAILURE);
  }

  if (stat(argv[1],&sbuf) != 0) {
    fprintf(stderr,"%s: Unable to stat file \"%s\"\n",progname,argv[1]);
    exit(EXIT_FAILURE);
  } 
  if (ComputeHashForFile(argv[1],hash) == 0) {
    fprintf(stderr,"%s: Unable to compute hash for \"%s\"\n\n",
            progname,argv[1]);
    exit(EXIT_FAILURE);
  }
  printf("\n\n/* This block of code is automatically generated by %s.c */\n\n",
         progname);
  printf("/* Code was generated on "__DATE__" at "__TIME__" */\n");
  printf("\n\nconst unsigned long cklen = %d;\n",(int) sbuf.st_size);
  printf("const unsigned char cksum[20] = {\n     ");
  for (i = 0; i < 10; i++) {
    printf("0x%02X, ",hash[i]);
  }
  printf("\n     ");
  for (i = 10; i < 19; i++) {
    printf("0x%02X, ",hash[i]);
  }
  printf("0x%02X\n};\n\n\n",hash[19]);
  exit(EXIT_SUCCESS);
  return 0; /* to shut up insure */
}
