#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#if defined(ZK_SOLARIS)
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <sys/sockio.h>
#include <stropts.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#if 0
#include <netinet/arp.h>
#endif


#define DEV_ARP "/dev/arp"

extern struct ether_addr * ether_aton (
  char * );

static int SetARP (
  struct sockaddr,         /* Protocol (IP) address */
  struct sockaddr,         /* Hardware (ethernet) address */
  int );                   /* ARP flags */


static int
SetARP (
  struct sockaddr paddr,   /* Protocol (IP) address */
  struct sockaddr haddr,   /* Hardware (ethernet) address */
  int flags )              /* ARP flags */
{
  struct arpreq arpreq;
  struct strioctl strbuf;
  int p = 0;
  int result = 0;

  /*
   * Gain access to the ARP cache
   */

  if ((p = open(DEV_ARP,O_RDWR)) < 0) {
    perror(DEV_ARP);
    result = -1;
    goto finish;
  }

  /*
   * Build up the ARP request for setting an entry in the cache
   */

  arpreq.arp_pa = paddr;
  arpreq.arp_ha = haddr;
  arpreq.arp_flags = flags;

  strbuf.ic_cmd = SIOCSARP;
  strbuf.ic_timout = 0;
  strbuf.ic_dp = (caddr_t) &arpreq;
  strbuf.ic_len = sizeof(struct arpreq);

  if (ioctl(p,I_STR,&strbuf) < 0) {
    perror("SIOCSARP");
    result = -1;
    goto finish;
  }

finish:
  if (p >= 0) {
    close(p);
  }
  return result;
}

int
main (
  int argc ,
  char ** argv )
{
  int exit_code = EXIT_SUCCESS;
  struct sockaddr paddr;
  struct sockaddr haddr;
  struct sockaddr_in * sin = NULL;
  struct ether_addr * ethaddr = NULL;

  if (argc != 3) {
    fprintf(stderr,"Usage:  %s protocol_address hardware_address\n\n",argv[0]);
    goto finish;
  }

  /*
   * Get the protocol address into a structure.
   */

  sin = (struct sockaddr_in *) &paddr;
  sin->sin_family = AF_INET;
  if ((sin->sin_addr.s_addr = inet_addr(argv[1])) == -1) {
    fprintf(stderr,"%s:  Incorrect protocol address %s\n",argv[0],argv[1]);
    exit_code = EXIT_FAILURE;
    goto finish;
  }
  
  /*
   * Get the hardware (ethernet) address into a structure 
   */
    
  haddr.sa_family = AF_UNSPEC;
  if ((ethaddr = ether_aton(argv[2])) == NULL) {
    fprintf(stderr,"%s: Incorrect hardware address %s\n",argv[0],argv[2]);
    exit_code = EXIT_FAILURE;
    goto finish;
  } 
  memcpy((char *)&(haddr.sa_data),ethaddr,6);
 
  /*
   * Update the arp cache (the flags indicate the entry is completed and
   * permanent).  We do not want the address published however (if we did
   * we would have to set the ATF_PUBL flag as well)
   */

  if (SetARP(paddr,haddr,ATF_PERM|ATF_COM) < 0) {
    fprintf(stderr,"%s:  SetARP failed\n",argv[0]);
    exit_code = EXIT_FAILURE;
    goto finish;
  }

finish:
  exit(exit_code);
}
#else
#warning __FILE__ not supported on this platform
#endif
