/* Unit test fr zkWormholeSendStatus() */
/* It is a standalone executable */

/* worm.c MUST have been compiled with SEND_STATUS_UNIT_TEST defined */
/* However, when compiling with that definition, it won't link because */
/* of missing reference to unitTestSendStatusCallback() */

#include <zkError.h>
#include <worm.h>

#define MSG_SIZE  64

FrWormholeCommand * WormCmd = NULL;

int main (
  int argc ,
  char ** argv )

{
  ZkError     err = NULL;
  int         rc = 1;
  ZkWormhole *ctx = NULL;
  char        msg[MSG_SIZE];
  char       *buf = NULL;
  size_t      msgSize = MSG_SIZE;
  size_t      i;


  /* Shut up compiler */
  (void)argc;
  (void)argv;

  for (i = 0; i < msgSize; i++)
  {
    msg[i] = (char)((0x80 + i) % 0x100);
  }

  zkErrorNew(&err);

  /* Init so not NULL, but it won't be used by zkWormholeSendStatus */
  ctx = (ZkWormhole *)1;

  if ( zkWormholeSendStatus(err, NULL, msg, msgSize) )
  {
    fprintf(stderr, "zkWormholeSendStatus doesn't handle erroneous ctx param properly\n");
    rc = 0;
  }
  else if ( zkWormholeSendStatus(err, ctx, NULL, msgSize) )
  {
    fprintf(stderr, "zkWormholeSendStatus doesn't handle erroneous msg param properly\n");
    rc = 0;
  }
  else if ( zkWormholeSendStatus(err, ctx, msg, 0) )
  {
    fprintf(stderr, "zkWormholeSendStatus doesn't handle erroneous msgSize param properly\n");
    rc = 0;
  }
  else if ( !zkWormholeSendStatus(err, ctx, msg, msgSize) )
  {
    fprintf(stderr, "zkWormholeSendStatus failed\n");
    rc = 0;
  }
  else if ( WormCmd == NULL)
  {
    fprintf(stderr, "zkWormholeSendStatus sent a NULL message\n");
    rc = 0;
  }
  else if ( WormCmd->length != (msgSize + sizeof(FrWormholeCommand)) )
  {
    fprintf(stderr, "zkWormholeSendStatus sent a message of wrong size\n");
    rc = 0;
  }
  else
  {
    buf = ((char *)WormCmd) + sizeof(FrWormholeCommand);

    for (i = 0; i < msgSize; i++)
    {
        if ( buf[i] != (char)((i + 0x80) % 0x100) )
        {
          fprintf(stderr, "Message sent has bad content\n");
          break;
        }
    }
  }

  if (WormCmd)
  {
    free(WormCmd);
  }
  
  if (rc)
  {
    fprintf(stderr, "\nUnit Test sendStat passed\n");
  }

  return 0;
}


/* Callback used to catch the packet sent by zkWormholeSendStatus */
int unitTestSendStatusCallback(FrWormholeCommand *wormcmd)
{
  int rc = 0;

  if (!wormcmd)
  {
    fprintf(stderr, "Sending a NULL buffer\n");
  }
  else
  {
    if ( !(WormCmd = calloc(1, wormcmd->length)) )
    {
      fprintf(stderr, "Problem allocating memory\n");
    }
    else
    {
      memcpy(WormCmd, wormcmd, wormcmd->length);
      rc = 1;
    } 
  }

  return rc;
}
