#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkshimlib.h>

#define BUFFER_SZ 2046

int DataDump ( FILE * , unsigned char  * , size_t );


int
DataDump (
  FILE * stream ,
  unsigned char  * buffer ,
  size_t buffer_len )
{
#define LINE_SZ 128
  char line[LINE_SZ+ 1];       /* A line of output */
  int i = 0;                   /* Loop index */
  int j = 0;                   /* Loop index */
  const char * digit = "0123456789ABCDEF";

  for (i = 0; i < (int) buffer_len; i += 16) {
    memset(line,32,LINE_SZ);
    snprintf(line,sizeof line,"%05d:  ",i);
    for (j = 0; j < 16; j++) {
      if ((i + j) >= (int) buffer_len) {
        break;
      }
      line[8 + j * 3] = digit[buffer[j + i] / 16]; 
      line[9 + j * 3] = digit[buffer[j + i] % 16];
      if (isprint(buffer[j + i])) {
        line[56 + j] = buffer[j + i];
      } else {
        line[56 + j] = '.';
      }
    }
    line[LINE_SZ] = 0;
    fprintf(stream,"%s\n",line);
    fflush(stream);
  }
  return 0;
#undef LINE_SZ
}

int
main ( 
  int argc ,
  char ** argv )
{
  ZkShim shim = NULL;       /* Structure used to access the shim */
  ZkError err = NULL;       /* Error structure passed to ZK functions */
  ZkLog log = NULL;         /* Log associated with the error structure */
  int fd = 0;               /* File descriptor to read/write data to shim */
  int bytes = 0;            /* Number of bytes read/writen */
  char buffer[BUFFER_SZ];   /* input / output buffer */
  int mode = 0;             /* the mode */
  
  if (argc != 2) {
    fprintf(stderr,"usage:  %s mode\n\n",argv[0]);
    exit(EXIT_FAILURE);
  } 

  mode = atoi(argv[1]);
  
  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if (!zkShimOpen(err,shim, "/dev/zkshim")) {
    printf("zkShimOpen:  failed\n");
  }
  if (!zkShimRuleSetProto(err,shim,IPPROTO_IP)) {
    printf("zkShimRuleSetProto:  failed\n");
  }
  if (mode == 1) {
    fprintf(stdout,"Listening to packets with source port 23\n");
    if (!zkShimRuleSetSrcPort(err,shim,23)) {
      printf("zkShimRuleSetSrcPort:  failed\n");
    }
  } else {
    fprintf(stdout,"Listening to packets with destination port 23\n");
    if (!zkShimRuleSetDestPort(err,shim,23)) {
      printf("zkShimRuleSetSrcPort:  failed\n");
    }
  }
  if (!zkShimRuleApply(err,shim)) {
    printf("zkShimRuleApply:  failed\n");
  }
  if (!zkShimGetFD(err,shim,&fd)) {
    printf("zkShimGetFD:  failed\n");
  }

  for (;;) {
    bytes = read(fd,buffer,BUFFER_SZ);
    fprintf(stderr,"\nRead %d bytes from the shim\n",bytes);
    DataDump(stdout,(unsigned char *)buffer,bytes);
  }

  if (shim) {
    zkShimClose(err,shim);
    zkShimFree(err, &shim);
  } 
  if (err) {
    zkErrorFree(&err);
  }
  exit(EXIT_FAILURE);
}
