#include <stdio.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>

#define LPBK_IP_ADDR              0x7F000001
#define DEFAULT_ZKDB_SERVER_PORT  51130


char *prog;

static void usage(int ok)
{
  fprintf(stderr, "Usage: %s packetSize <IP address> <port #>\n", prog);
  fprintf(stderr, "    default IP address is local machine\n");
  fprintf(stderr, "    default port number is 51130\n");
  exit(ok ? 1 : 0);
}


int main(int ac, char **av)
{
  int                rc = 0;
  int                sockFd = 0;
  int                nbBytes = 0;
  struct in_addr     ipAddr;
  unsigned short     port = 0;
  struct sockaddr_in sockAddr;
  char *             buff = NULL;
  int                i = 0;

  prog = av[0];

  /* Process params */
  if (ac < 2) {
    usage(0);
  }
  
  nbBytes = atoi(av[1]);

  if (ac >= 3) {
    if ( !inet_aton(av[2], &ipAddr) ) {
      fprintf(stdout, "Bad hostname or IP address\n");
      return 0;
    }
  }
  else {
    ipAddr.s_addr = htonl(LPBK_IP_ADDR); /* Default is loopback IP address */
  }
  fprintf(stdout, "ipAddr = 0x%x\n", ipAddr.s_addr);

  if (ac >= 4) {
    port = (unsigned short)htons(atoi(av[3]));
  }
  else {
    port = (unsigned short)htons(DEFAULT_ZKDB_SERVER_PORT); /* Default */
  }
  fprintf(stdout, "port = %x\n", port);

  while (1) {
    if ((sockFd = socket (AF_INET, SOCK_DGRAM, 0)) <= 0) {
      fprintf(stdout, "Couldn't open socket\n");
      return 0;
    }
    else {
      bzero (&sockAddr, sizeof(sockAddr));
      sockAddr.sin_family = AF_INET;
      sockAddr.sin_addr.s_addr = ipAddr.s_addr;
      sockAddr.sin_port = (unsigned short)port;
    }

    /* Try connecting to zkdb server */
    if (connect(sockFd, (struct sockaddr *)&sockAddr, sizeof(sockAddr))) {
      fprintf(stdout, "Couldn't connect to zkDB server\n");
      fprintf(stdout, "errno = %d\n", errno);
      return(0);
    }

    /* Send the requested amount of garbage */
    if (!(buff = (char *)malloc(nbBytes))) {
      fprintf(stdout, "Couldn't allocate buffer\n");
    }

    for (i = 0; i < nbBytes; i++) {
      buff[i] = (i+128) % 256;
    }

    while (1) {
      if ((rc = send(sockFd, buff, nbBytes, 0)) == -1) {
        break;
      }
    }

    free (buff);
    close (sockFd);
  }

  return (1);
}
