/*
 *  $Id: zkidentd.c,v 1.18 2000/10/04 12:57:52 marcp Exp $
 */

static const char * module_id_ = "$Id: zkidentd.c,v 1.18 2000/10/04 12:57:52 marcp Exp $";

#warning "Don't use this code! It has security bugs and will be deleted before the next release (bug #4195)."

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <pwd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <time.h>
#include <sys/time.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkFile.h>
#include <zkLog.h>
#include <zkParms.h>
#include "zkidentd.h"
#include "zkCStr.h"

#define ZKMSERVER 0 /* FIX ME */

#define DEFAULT_FLAGS FLAG_SUPPRESS
#define DEFAULT_ALTERNATE "FreedomUser"
#define DEFAULT_PORT 113  /* Default fingerd port */
#define DEFAULT_USER NULL /* Default user which will run the zkidentd */
#define DEFAULT_LOGFILE NULL /* Default logging file */
#define DEFAULT_PIDFILE NULL /* Default PID file */
#define DEFAULT_CONFIG_FILE NULL; /* Dedault configuration file */
#define DEFAULT_MAXCONN 60 /* Maximum connection time in seconds */
#define DEFAULT_COMMAND "/tmp/fridentd.cmd" /* Default unix domain socket for commands */

#define ERRMSG_SZ 128
#define HOSTNAME_SZ 64

#define LINE_SZ 128 /* Maximum size of an ident query */

#define FLAG_VERBOSE 0x1
#define FLAG_REDIRECTION 0x2
#define FLAG_LISTALL 0x04






/*
 * Stuff to implement the identd server database
 */

#define ERRMESG_1 "0 , 0 : ERROR : UNKNOWN-ERROR\015\012"


#define MESG_SZ  256 
#define BUFFER_SZ 256 /* Max size of a ident server command */


int zkIdentDBInit ( ZkError , ZkIdentDB * , int , int );
int zkIdentDBAdd ( ZkError,ZkIdentDB,unsigned short,unsigned short,uint32_t ,char * );
int zkIdentDBDelete ( ZkError , ZkIdentDB , const char * );
int zkIdentDBDeleteByACI ( ZkError , ZkIdentDB , uint32_t );

int zkIdentDBLookup (ZkError,ZkIdentDB,unsigned short,unsigned short,const char **);
int zkIdentDBFree( ZkError , ZkIdentDB * );
int zkIdentDBHandleRequest ( ZkError , uint32_t ,
                             ZkLog , ZkIdentDB , char * , int , int );
int zkIdentUsage ( char * );


/*
 * Initialize the Ident server internal database structure
 */

int
zkIdentDBInit (
  ZkError err ,
  ZkIdentDB * idb ,
  int baseport ,
  int connections )
{
  zkErrorClear(err);
  if (idb == NULL) {
    return 0;
  } 
  *idb = (ZkIdentDB) malloc(sizeof(struct T_ZkIdentDB));
  if ((*idb) == NULL) {
    return 0;
  }
  (*idb)->connections = connections;
  (*idb)->baseport = baseport;
  (*idb)->rec = (ZkIdentRecord *)malloc(sizeof(struct T_ZkIdentRecord) * connections);
  if ((*idb)->rec == NULL) {
    return 0;
  }
  memset((char *) (*idb)->rec,0,sizeof(struct T_ZkIdentRecord) * connections);
  return 1;
}
  
/*
 * Add a (serverport, clientport, nymname) to the database.
 */

int
zkIdentDBAdd (
  ZkError err ,
  ZkIdentDB idb ,
  unsigned short server ,
  unsigned short client ,
  uint32_t aci ,
  char * nym )
{
  zkErrorClear(err);
  if (idb) {
    if ((server < idb->baseport) || (server >= (idb->baseport + idb->connections))) {
      /*
       * Invalid port
       */
      zkErrorSet(err,"zkIdentDBAdd",ZKERR,ZKMSERVER,ZKERROR,"Invalid port");
      return 0;
    }
    idb->rec[server - idb->baseport].client_port = client;
    strncpy(idb->rec[server - idb->baseport].nym,nym,NYM_SZ);
    idb->rec[server - idb->baseport].nym[NYM_SZ] = (char) 0;
    idb->rec[server - idb->baseport].aci = aci;
    return 1;
  }
  zkErrorSet(err,"zkIdentDBAdd",ZKERR,ZKMSERVER,ZKENULLPTR,NULL);
  return 0;
}

/*
 * Do a lookup to identify the nym from a given port
 */  

int
zkIdentDBLookup (
  ZkError err ,
  ZkIdentDB idb ,
  unsigned short server ,
  unsigned short client ,
  const char ** nym )
{
  zkErrorClear(err);
  if (idb && nym) {
    *nym = NULL;
    if ((server < idb->baseport) || 
        (server >= (idb->baseport + idb->connections)) ||
        (client < 1) || (client > 65535)) { 

      /*
       * Invalid ports
       */
      return 0;
    }
    if (idb->rec[server - idb->baseport].client_port == client) {
      *nym = (const char *) idb->rec[server - idb->baseport].nym;
      return 1;
    }
    return 0;
  }
  return 0;
}

/*
 * Delete all ports assigned to a given nym
 */

int
zkIdentDBDelete (
  ZkError err ,
  ZkIdentDB idb ,
  const char * nym )
{
  int i = 0; /* Loop index */

  zkErrorClear(err);
  if (idb && nym) {
    for (i = 0; i < idb->connections; i++) {
      if (idb->rec[i].nym[0] == nym[0]) {
        if (strcmp(idb->rec[i].nym,nym) == 0) {
          idb->rec[i].client_port = 0;
        }
      }
    }
    return 1;
  }
  return 0;
}

int
zkIdentDBDeleteByACI (
  ZkError err ,
  ZkIdentDB idb ,
  uint32_t aci )
{
  int i = 0; /* Loop index */

  zkErrorClear(err);
  if (idb) {
    for (i = 0; i < idb->connections; i++) {
      if (idb->rec[i].aci == aci) {
        idb->rec[i].client_port = 0;
      }
    }
    return 1;
  }  
  return 0;
}


/*
 * Destroy the identd internal database
 */

int
zkIdentDBFree( 
  ZkError err ,
  ZkIdentDB * idb )
{
  zkErrorClear(err);
  if (idb) {
    if (*idb) {
      if ((*idb)->rec) {
        free((*idb)->rec);
      }
      free(*idb);
      *idb = NULL;
      return 1;
    }
  } 
  return 0;
}


int
zkIdentDBHandleRequest (
  ZkError err ,
  uint32_t flags ,
  ZkLog log ,
  ZkIdentDB idb ,
  char * alternate ,
  int in ,
  int out )
{
  char line[LINE_SZ + 1];
  char mesg[MESG_SZ + 1];
  char * port1_s = NULL;
  char * port2_s = NULL;
  int bytes = 0;
  int port1 = 0;
  int port2 = 0;
  int offset = 0;
  int i = 0;
  const char * nym = NULL;

  zkErrorClear(err);

  /*
   * Read and NUL terminate the query line
   */

  for (offset = 0; offset < LINE_SZ; offset++) {
    bytes = read(in,&(line[offset]),1); 
    if (line[offset] == (char) 0x0D) {
      line[offset] = (char) 0;
    } else if (line[offset] == (char) 0x0A) {
      break;
    }
  }
  line[offset] = (char) 0;
       
  /*
   * Get the two port number tokens
   */
  
  port1_s = &(line[0]);
  port2_s = NULL;
  for (i = 0; i < offset; i++) {
    if (line[i] == ',') {
      line[i] = (char) 0;
      port2_s = &(line[i + 1]);
      break;
    } else if ((!isdigit(line[i])) && (!isspace(line[i]))) {
    
      /*
       * Parse error, the ident query contained a character which was not 
       * a digit or blank space.
       */

      zkErrorSet(err,"zkIdentDBHandleRequest",ZKERR,ZKMSERVER,ZKEPARSE,
                 "Request contained invalid characters");
      write(out,ERRMESG_1,strlen(ERRMESG_1));
      return 0;
    }
  } 
 
  if (port2_s == NULL) {

    /*
     * Error, we did not find the second port number (there was no comma)
     */

    zkErrorSet(err,"zkIdentDBHandleRequest",ZKERR,ZKMSERVER,ZKEPARSE,
               "Request is missing client port");
    write(out,ERRMESG_1,strlen(ERRMESG_1));
    return 0; 
  }

  /*
   * Convert the portnumbers to integers
   */

  port1 = atoi(port1_s);
  port2 = atoi(port2_s);

  if ((port1 < 1) || (port1 > 65535) || (port2 < 1) || (port2 > 65535)) {
    zkErrorSet(err,"zkIdentDBHandleRequest",ZKERR,ZKMSERVER,ZKEPARSE,
               "Request specified an invalid port");
    snprintf(mesg,sizeof mesg,"%d , %d : ERROR : INVALID-PORT\015\012",port1,port2);
    write(out,mesg,strlen(mesg));
    return 0;
  }
 
  if (!(flags & FLAG_SUPPRESS) &&
      zkIdentDBLookup(err,idb,port1,port2,&nym)) {
    snprintf(mesg,sizeof mesg,"%d , %d : USERID : OTHER : %s\015\012",
            port1,port2,nym);
  } else {
    zkErrorSet(err,"zkIdentDBHandleRequest",ZKERR,ZKMSERVER,ZKEPARSE,
               "Failed lookup");
    if (alternate[0] != (char) 0) {
      snprintf(mesg,sizeof mesg,"%d , %d : USERID : OTHER : %s\015\012",port1,port2, alternate); 
    } else {
      snprintf(mesg,sizeof mesg,"%d , %d : ERROR : NO-USER\015\012",port1,port2);
    }
  }
  write(out,mesg,strlen(mesg));
  zkLogWrite(log,mesg);
  return 1;
}

/*
 * Ident server update command format
 * byte 1,2:  Total length in bytes
 * byte 3:  command:  1 = ADD
 *                    2 = DEL
 * for ADD command
 * byte 4,5: server port
 * byte 6,7: client port (both in network order)
 * bytes 8-> end: nym name
 * 
 * for DEL command
 * bytes 4-> end? nym name
 */
 
int
zkIdentDBHandleCommand (
  ZkError err ,
  ZkIdentDB idb ,
  int fd ) 
{
  int size = 0;
  uint32_t aci = 0;
  unsigned short server = 0;
  unsigned short client = 0;
  char nym[NYM_SZ + 1];
  unsigned char buffer[BUFFER_SZ];
  int bytes = 0; /* Bytes read for the command */


  zkErrorClear(err);
  memset(buffer,0,BUFFER_SZ);
  bytes = recv(fd,buffer,BUFFER_SZ,0);
  if (bytes < 2) {
    return 0;
  }
  size = buffer[0] * 0x100 + buffer[1];
  if (size != bytes) {
    return 0;
  }
  switch (buffer[2]) {
  case 1:
    if (size < 12) {
      break;  
    }
    server = buffer[3] * 0x100 + buffer[4]; 
    client = buffer[5] * 0x100 + buffer[6];
    aci = buffer[7] * 0x1000000 + buffer[8] * 0x10000 + 
          buffer[9] * 0x100 + buffer[10];
    if (size > NYM_SZ) {
      size = NYM_SZ;
    }
    memcpy(nym,&(buffer[11]),size - 11);
    nym[size - 11] = (char) 0;
    return zkIdentDBAdd(err,idb,server,client,aci,nym);
  case 2:
    if (size < 4) {
      break;
    }
    memcpy(nym,&(buffer[3]),NYM_SZ);
    nym[NYM_SZ] = (char) 0;
    return zkIdentDBDelete(err,idb,nym);
  case 3:
    if (size != 7) {
      break;
    }
    aci = buffer[3] * 0x1000000 + buffer[4] * 0x10000 + 
          buffer[5] * 0x100 + buffer[6];
    return zkIdentDBDeleteByACI(err,idb,aci);
  }
  zkErrorSet(err,"zkIdentDBHandleCommand",ZKERR,ZKMSERVER,ZKERROR,
             "binary command format error");
  return 0;
} 

int
zkIdentdMain (
  struct sockaddr_in remote ,
  int argc ,
  char ** argv ,
  void * contextptr )
{
  ZkIdentdContext * context = NULL;
  struct hostent * host = NULL;
  char errmsg[ERRMSG_SZ];

  context = (ZkIdentdContext *) contextptr;

  (void) argc;
  (void) argv;
  host = gethostbyaddr((const char *)&(remote.sin_addr.s_addr),4,AF_INET);
  if (host != NULL) {
    strcpy(errmsg,"Connection from "); /* bug: audit -- snprintf */
    strncat(errmsg,host->h_name,HOSTNAME_SZ);
  } else {
    unsigned char * ptr = NULL;
    ptr = (unsigned char *) &(remote.sin_addr.s_addr);
    snprintf(errmsg,sizeof errmsg,"Connection from %u.%u.%u.%u",ptr[0],ptr[1],ptr[2],ptr[3]);
  }
  zkLogWrite(context->log,errmsg);

  if (!zkIdentDBHandleRequest(context->err,context->flags,context->log,
                              context->idb,context->alternate,0,1)) {
    zkLogError(context->log,context->err);
  }
  return 1;
}

int
zkIdentUsage (
  char * progname )
{
  fprintf(stderr,"%s\n\nUsage:  %s\n"
"     --portspec \"baseport count\"      Specify baseport and portcount\n"
"     [ --"P_CONFIG" config_file ]         Configuration file\n"
"     [ --"P_USER" username ]              Process username (default is nobody)\n"
"     [ --"P_LOGFILE" logfile ]            The server logfile\n"
"     [ --"P_PORT" port ]                  The server port (default is 113)\n"
"     [ --"P_MAX_CONNECTION_TIME" timeout ]  Max duration of connection (seconds)\n"
"     [ --"P_RESPONSE" name ]              Alternate name instead of NOUSER\n"
"     [ --"P_SUPPRESS" ]                   Always answer with NOUSER (or alternate)\n"
"     [ --"P_DONTFORK" ]                   Dont fork a subchild\n"
"     [ --"P_PIDFILE" pidfile ]            Holds the pid\n"
"     [ --"P_HELP" ]                       This message\n"
"     [ --"P_COMMAND" command_file ]       identd command socket\n"
"     [ --"P_VERSION" ]                    Server version information\n"
"     Options surrounded by square brackets are optional.\n\n",
          VERSION_STR,
          progname);
  exit(EXIT_FAILURE);
  return 0; /* to shut up insure */
} 
  
int 
main (
  int argc ,
  char ** argv )
{
  char * progname = NULL;                /* Cleaned up name of the program */
  char errmsg[ERRMSG_SZ] = "";           /* Error message generated by opts */
  int opt = 1;                           /* Loops through cmdline opts */
  struct passwd * pwent = NULL;          /* User entry for settin user id */
  pid_t childpid = 0;                    /* process id of the finger server */
  char * logfile = DEFAULT_LOGFILE;      /* Log file */
  ZkIdentdContext context;               /* Finger server context */
  char * user = DEFAULT_USER;            /* Default user name */
  uid_t uid = 0;                         /* Id for user name */
  unsigned short port = DEFAULT_PORT;    /* finger port */
  int dontfork = 0;                      /* Dont return immediatly */
  struct sockaddr_in server;             /* Server address structure */
  int maxconn = DEFAULT_MAXCONN;         /* Maximum connection time */
  struct sockaddr_un cmdaddr;            /* Command socket address */
  char * pval = NULL;                    /* For getting values from config */
  uint8_t bool = 0;                      /* For getting booleans from config */

  (void) module_id_;
  /*
   * Set up the context 
   */
  
  memset(&context,0,sizeof(ZkIdentdContext));
  context.flags = DEFAULT_FLAGS;
  context.command = DEFAULT_COMMAND;      /* Command socket file */
  context.pidfile = DEFAULT_PIDFILE;      /* Default PID file */
  context.config_file = DEFAULT_CONFIG_FILE; /* Default configuration file */
  context.logfile = NULL;
  context.pid_fptr = NULL; 
  zkCStrlcpy(context.alternate,DEFAULT_ALTERNATE,sizeof context.alternate);
  zkErrorNew(&context.err);
  zkConfigNew(context.err,&context.config);
 
  /*
   * Cleanup the program name
   */

  if ((progname = strrchr(argv[0],'/')) == NULL) {
    progname = argv[0];
  } else {
    progname++;
  }

  /*
   * Load the configuration file if we specified one 
   */

  opt = 1;
  while (opt < argc) {
    if (strcasecmp(argv[opt],"--"P_CONFIG) == 0) {
      context.config_file = argv[opt];
      break;
    } else if (strcasecmp(argv[opt],"--"P_HELP) == 0) {
      zkIdentUsage(progname);
    } else if (strcasecmp(argv[opt],"--"P_VERSION) == 0) {
      zkCStrlcpy(errmsg,VERSION_STR,sizeof errmsg);
      goto failure;
    }
    opt++;
  }

  if (context.config_file) {
    if (!zkConfigLoad(context.err,context.config,context.config_file)) {
      snprintf(errmsg,ERRMSG_SZ,"Failed to load configuration file \"%s\": %s",
               context.config_file,strerror(errno));
      goto failure;
    }
  }
  zkConfigSetCmdParms(context.err,context.config,argc,(const char **) argv);

  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_USER,&pval)) {
    user = pval;
  }

  /* Read the portspec */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_PORTSPEC,&pval)) {
    char * c_ptr = NULL;
    context.baseport = atoi(pval);
    c_ptr = strpbrk(pval," \t\n\r");
    if (c_ptr) {
      context.connections = atoi(c_ptr);
    } else {
      snprintf(errmsg,ERRMSG_SZ,
               "--%s must have two integer value sperated by whitespace",
               P_PORTSPEC);
      goto failure;
    }
  }

  /* Alternate response string instead of NOUSER */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_RESPONSE,&pval)) {
    strncpy(context.alternate,pval,ALTERNATE_SZ);
    context.alternate[ALTERNATE_SZ] = (char) 0;
  }

  /* Get the logfile */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_LOGFILE,&pval)) {
    logfile = pval;
  }

  /* Get the PID file name */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_PIDFILE,&pval)) {
    context.pidfile = pval;
  }

  /* Get the command filename (to get commands from the wormhole) */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_COMMAND,&pval)) {
    context.command = pval;
  }

  /* Get the server port number */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_PORT,&pval)) {
    port = atoi(pval);
  }

  /* Get the maximum connection time */
  if (zkConfigGetParameter(context.err,context.config,TYPE_STRING,
                           S_SERVER_IDENT,P_MAX_CONNECTION_TIME,&pval)) {
    maxconn = atoi(pval);
  }

  /* Should we fork? */
  if (zkConfigGetParameter(context.err,context.config,TYPE_BOOLEAN,
                           S_SERVER_IDENT,P_DONTFORK,&bool)) {
    if (bool) {
      dontfork = 1;
    }
  } 

  /* Suppress the nym name? */

  if (zkConfigGetParameter(context.err,context.config,TYPE_BOOLEAN,
                           S_SERVER_IDENT,P_SUPPRESS,&bool)) {
    if (bool) {
      context.flags |= FLAG_SUPPRESS; 
    } else {
      context.flags &= ~FLAG_SUPPRESS;
    }
  }
  
  /* Sanity checks */

  /*
   * Did we specify the portspec?
   */
  
  if ((context.baseport == 0) || (context.connections == 0))  {
    zkCStrlcpy(errmsg,"You must specify baseport and portcount with --"P_PORTSPEC,sizeof errmsg);
    goto failure;
  }

  /*
   * Set the userid of the process
   */

  if (user != NULL) {
    pwent = getpwnam(user);
    if (pwent) {
      uid = pwent->pw_uid;
      if (uid == 0) {
        zkCStrlcpy(errmsg,"You cannot run this program as root",sizeof errmsg);
        goto failure;
      } 
  
      /*
       * The user id is set by the server function since it must be able to 
       * grab a priviliged port
       */

    } else {
      strcpy(errmsg,"Unknown user name "); /* bug: audit -- snprintf */
      strncat(errmsg,user,32);
      goto failure;
    }
  } else {
    uid = getuid();
  }

  /*
   * Build the server sockaddr structure
   */

  server.sin_family = AF_INET;
  server.sin_port = htons(port);
  server.sin_addr.s_addr = htonl(INADDR_ANY);

  /*
   * Build the command socket address structure
   */

  cmdaddr.sun_family = AF_UNIX;
  strncpy(cmdaddr.sun_path,context.command,64);

  /*
   * Turn on the log
   */

  if (logfile) {
    zkLogNew(&(context.log));
    zkLogSetFile(context.log,logfile);
    zkLogSetProgname(context.log,progname);
    zkLogEnable(context.log); 
    context.logfile = logfile;
  } else {
    context.log = NULL;
  }
 

  /*
   * do we run as a backgrounded server?
   */

  if (!dontfork) {
    if ((childpid = fork()) != 0) {
      exit(EXIT_SUCCESS);
    }
  }

  /* Save the PID file */
  if (!zkFileWritePid(context.err,context.pidfile,&(context.pid_fptr))) {
    zkLogError(context.log,context.err);
    fprintf(stderr,"%s:  Unable to access pidfile %s: %s\n",
            progname,context.pidfile?context.pidfile:"(NULL)",strerror(errno));
    goto failure;
  }        

  /*
   * Create the command socket.
   */

  unlink(context.command);
  if ((context.cmd = socket(AF_UNIX,SOCK_DGRAM,0)) < 0) {
    snprintf(errmsg,sizeof errmsg,"socket: %s",strerror(errno));
    goto failure;
  } 
  if (bind(context.cmd,(struct sockaddr *) &cmdaddr,sizeof(cmdaddr)) != 0) {
    snprintf(errmsg,sizeof errmsg,"bind: %s",strerror(errno));
    goto failure;
  } 

  if (!zkIdentDBInit(context.err,&context.idb,context.baseport,
                     context.connections)) {
    zkCStrlcpy(errmsg,"Unable to initiate internal database",sizeof errmsg);
    goto failure;    
  }

  if (!RunAsStreamServer(context.err,context.log,&server,uid,zkIdentdMain,
                         argc,argv,&context,maxconn)) {
    zkLogError(context.log,context.err);
    zkErrorMessage(context.err,errmsg,ERRMSG_SZ);
    zkLogWrite(context.log,"Server terminated");
    goto failure;
  }
  zkLogWrite(context.log,"Server terminated");
 
  unlink(context.pidfile);
  exit(EXIT_SUCCESS);

failure:

  /*
   * Things didnt work out, explain why and exit.
   */

  fprintf(stderr,"%s: %s\n",progname,errmsg);
  fprintf(stderr,"%s: Use --"P_HELP" to get a list of available options.\n\n",
          progname);
  exit(EXIT_FAILURE);
  return 0; /* to shut up insure */
}
