/*
 * $Id: zkidentd.h,v 1.6 1999/09/13 01:08:57 kent Exp $
 */

#ifndef ZKIDENTD_H_
#define ZKIDENTD_H_

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */

#define ZKIDENT_SERVER  /* Enables upgrades to server.c */

#define VERSION_STR "Zero-Knowledge Systems IDENT server, version 1.0 ("__DATE__", "__TIME__")"

typedef int (*ZkServerMain)( struct sockaddr_in , int , char ** , void *);

#define NYM_SZ   32
#define ALTERNATE_SZ 32

struct T_ZkIdentRecord {
  unsigned short client_port;   /* 0 == unassigned */
  char nym[NYM_SZ + 1];         /* Name of pseudonym currently using port */
  uint32_t aci;                 /* The aci to which this connection belongs */
};

typedef struct T_ZkIdentRecord ZkIdentRecord;

struct T_ZkIdentDB {
  int connections;     /* # of simult. connections which can be supported */
  int baseport;        /* The base port for the wormhole portmap */
  ZkIdentRecord * rec; /* The array of ports and nym names */
};

typedef struct T_ZkIdentDB * ZkIdentDB;

/*
 * Flags
 */

#define FLAG_SUPPRESS  0x1    /* Always answer with NOUSER response */

struct T_ZkIdentdContext {
  uint32_t flags;             /* Behaviour flags */
  ZkLog log;
  const char *logfile;
  ZkError err;
  ZkConfig config;
  const char * config_file;  /* Stores the zkidentd config file */
  char alternate[ALTERNATE_SZ + 1]; /* alternate instead of NOUSER response */
  int sock; /* The listening socket used by the server */
  const char * command; /* Points to the command unix domain socket file */
  char * pidfile; /* Points to the file which holds the server pid */
  int cmd; /* Socket which will receive update commands */
  char * nymserver; /* Name of the nymserver */
  uint16_t nsport; /* Port where the nymserver listens */
  ZkIdentDB idb; 
  int baseport ;
  int connections;
  FILE * pid_fptr;    /* PID file pointer */
};

typedef struct T_ZkIdentdContext ZkIdentdContext;

int zkIdentDBHandleCommand ( 
  ZkError , 
  ZkIdentDB , 
  int );

int zkIdentdMain (
  struct sockaddr_in ,             /* Remote client address */
  int ,                            /* Command line argc */
  char ** ,                        /* Command line argv */
  void * );                        /* Key server context structure pointer */


int RunAsStreamServer (
  ZkError ,                        /* Returns the error if one occured */
  ZkLog ,                          /* Main server log object */
  struct sockaddr_in * ,           /* Server address */
  uid_t ,                          /* UID under which to run the server */
  ZkServerMain ,                   /* Connection callback */
  int ,                            /* Command line argument count (argc) */
  char ** ,                        /* Command line argument array (argv) */
  void * ,                         /* Callback context structure */
  int );                           /* Maximum connection time in seconds */

#endif /* ZKIDENTD_H_ */
