#include "zks.h"

#include "zkAlloc.h"
#include "zkDBODefs.h"
#include "../../libs/frnidb/include/frNidbObj.h"
#include "../../libs/frdbo/include/fr_dbo_nidb_obj.h"

#include "frKeyQry.h"
#include "frNetworkDefaults.h"
#include "frDBOFuncs.h"
#include "frNiqs.h"
#include "frSec.h"
#include "zkError.h"
#include "zkList.h"
#include "zkipClient.h"
#include "frNetInfo.h"
#include "zkCStr.h"
#include "testzkinfod.h"
#include "sim.h"

/* Prototypes */
int aipNetInfoGetType(ZkError err, ZkLog log, FrNetInfo ni, FrEntIdType type,
                             char **hname, unsigned short *port);


/*
 * aipNetInfoGetType 
 *
 * moved Mar. 2000, Frank Foster
 *
 */

int aipNetInfoGetType(ZkError err, ZkLog log, FrNetInfo ni, FrEntIdType type,
                             char **hname, unsigned short *port)
{
  const char *func = "aipNetInfoGetType";
  ZkList list = NULL;
  FrNetInfoDatum data = NULL;
  char *p = NULL;
  struct sockaddr_in addr;

  if (!ni || !hname || !port) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "NULL param(s)");
    return 0;
  }

  if (!frNetInfoGetListOf(err, ni, &list, type)) {
    zkLogWriteContext(log, err, func, "frNetInfoGetListOf failed");
    return 0;
  }

  if (!list) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENULLPTR, "Server type not found");
    return 0;
  }

  if (!zkListMoveCursor(err, list, LIST_HEAD)) {
    zkLogWriteContext(log, err, func, "zkListMoveCursor failed");
    return 0;
  }
  
  if (!zkListGet(err, list, (void *) &data, LIST_STAY)) {
    zkLogWriteContext(log, err, func, "zkListGet failed");
    return 0;
  }
  
  if (!frNetInfoDatumGetSockaddr(err, data, &addr)) {
    zkLogWriteContext(log, err, func, "frNetInfoDatumGetSockaddr failed");
    return 0;
  }

  *port = ntohs(addr.sin_port);
  
  if ((p = inet_ntoa(addr.sin_addr)) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKERROR, 
               "Can't convert sockaddr to dotted quad");
    return 0;
  }

  /* Make space for the hostname */
  if ((*hname = malloc(sizeof(char) * (strlen(p) + 1))) == NULL) {
    zkErrorSet(err, func, ZKERR, ZKMMAIP, ZKENOMEM, "malloc() failed");
    return 0;
  }
  zkCStrlcpy(*hname, p, strlen(p) + 1);

  return 1;
}    


void checkNetInfo(
  ZkError err,
  FrNetInfo ni,
  char** kqHostname, unsigned short* kqPort)
{
  FrEntIdExp Entity;        /* NISS entity */

  if (!aipNetInfoGetEntity(err, NULL, ni, FRENT_TYPE_AIP, &Entity)) {
    printf(" checknetinfo bad \n");
  }

  printf(" found entity %s \n",Entity.name);


  if (!aipNetInfoGetType(err, NULL, ni, FRENT_TYPE_KEYQRY_SRV,
                         kqHostname, kqPort)) {
    printf("No key query server found in Freedom network, quiting");
  }

  printf(" kqhost %s kqport %d \n",*kqHostname, *kqPort);

  
}

