/*
 * $Id: zkip.c,v 1.6 2000/10/04 12:57:53 marcp Exp $
 */

static const char * module_id_ = "$Id: zkip.c,v 1.6 2000/10/04 12:57:53 marcp Exp $";

#include "zks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <zkIntTypes.h>
#include <zkLog.h>
#include <zkError.h>
#include <zkMemBuf.h>
#include <frCrypt.h>
#include "zkkeyd.h"
#include "zkp.h"



/*
 * _SESSION_CONTINUE is returned when everything went ok
 * _SESSION_END when the session MUST be terminated
 * _SESSION_ERROR when an error occured.
 */


int
zkKeyCommandStatus (
  ZkError err,
  ZkipSession session ,
  void * data )
{
#define MESG_SZ 256
  ZkMemBuf status;   /* Holds the status message */
  int parameters = 0; /* Number of parameters in the command */
  uint32_t uptime = 0; /* Number of seconds server has been up */
  double load = 0.0;  /* Server load (connections / second) */
  int return_code = _SESSION_ERROR;
  char mesg[MESG_SZ] = ""; /* For adding messages to the status */
  int mesgsz = 0; /* Length of the message string */
  unsigned char * statstr = NULL; /* Status message string */
  size_t statstrsz = 0; /* Length of the status message string */


  (void) module_id_; /* Get rid of compile warning */
  data = NULL; /* gets rid of the compile time warning */

  if (!zkipSessionParameters(err,session,&parameters)) {
    zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                        "Could not get command parameters",
                        "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
    goto finish;
  }
  if (parameters > 0) {
    zkipSessionResponse(err,session,0,_COMMAND_ERROR,"No parameters expected",
                        "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
    goto finish;
  }
  if (!zkMemBufNew(err,&status)) {
    goto finish;
  }

  /*
   * Add the server name to the status 
   */

  mesgsz = snprintf(mesg,MESG_SZ,"name: %s\015\012",zkKeyContext->myEnt.name);
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);

  /*
   * Add the protocol type
   */

  mesgsz = snprintf(mesg,MESG_SZ,"protocol: %d\015\012",zkKeyContext->protocol);
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);

  /*
   * Add the startup date and time of the server
   */

  mesgsz = snprintf(mesg,MESG_SZ,"startup: %s\015\012",zkKeyContext->status.dtstart);
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);

  /* 
   * Add the number of seconds the server has been up 
   */

  uptime = time(NULL) - zkKeyContext->status.starttime;
  mesgsz = snprintf(mesg,MESG_SZ,"uptime (seconds): %d\015\012",uptime); 
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);
   
  /*
   * Add the total number of connections
   */

  mesgsz = snprintf(mesg,MESG_SZ,"connections: %d\015\012",
                   zkKeyContext->status.connections);
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);

  /*
   * Add the server load
   */

  if (uptime > 0) {
    load = (double) zkKeyContext->status.connections / (double) uptime;
  }
  mesgsz = snprintf(mesg,MESG_SZ,"load (connections / second): %f\015\012",load);
  zkMemBufAppend(err,status,mesg,(size_t) mesgsz);

  /* 
   *
   */

  if (zkMemBufGetData(err,status,(void **)&statstr,&statstrsz)) {
    zkipSessionResponse(err,session,0,_SERVICE_READY,"Server status",
                        "TEXT",ZKIP_ENCODING_8BIT,statstrsz,statstr);
    return_code = _SESSION_CONTINUE;
  } else {
    zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Unable to display status",
                        "NODATA",ZKIP_ENCODING_BINARY,0,NULL);
  }  

finish:
  if (status) {
    zkMemBufFree(NULL,&status);
  }
  return return_code;
#undef MESG_SZ
}
