/* 
 * $Id: zkkeyd.h,v 1.33 2000/10/04 12:57:53 marcp Exp $
 */

#ifndef ZKKEYD_H_
#define ZKKEYD_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#include <frCrypt.h>

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */

#ifndef _ZKIP_H
#include <ZKIP.h>
#endif /* _ZKIP_H */

#ifndef _ZKIPENGINE_H
#include <zkipEngine.h>
#endif /* _ZKIPENGINE_H */

#ifndef FRKEY_H_
#include <frKeyUpdate.h>
#endif /* FRKEY_H_ */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifndef ZKDB_H
#include <zkDB.h>
#endif /* ZKDB_H */

#ifndef FRTOKENTYPE_H_
#include <frTokenType.h>
#endif /* FRTOKENTYPE_H_ */

#ifndef FRSEC_H_
#include <frSec.h>
#endif /* FRSEC_H_ */

#ifndef FRKEYQRY_H_
#include <frKeyQry.h>
#endif /* FRKEYQRY_H */

#ifndef FRSTATUPD_H_
#include <frStatUpd.h>
#endif /* FRSTATUPD_H_ */

#ifndef FRSTATCLIENT_H_
#include <frStatClient.h>
#endif

#ifndef FRAMTPCLIENT_H_
#include <frAmtpClient.h>
#endif

#include "frEnt.h"
#include "frNetInfo.h"
#include "niqs_qc.h"

/*
 * The version string
 */

#define VERSION_ZKIP_STR "Zero-Knowledge Systems ZKIP Server V1.0 ("__DATE__")"

/*
 * Currently available protocols
 */

#define PROTOCOL_UNKNOWN 0
#define PROTOCOL_ZKIP   1
#define PROTOCOL_KSCP   2
#define PROTOCOL_NSCP   3
#define PROTOCOL_SNAP   7

#define PROTOCOL_ZKIP_TOK "ZKIP"
#define PROTOCOL_KSCP_TOK "KSCP"
#define PROTOCOL_NSCP_TOK "NSCP"
#define PROTOCOL_SNAP_TOK "SNAP"

/*
 * Default values
 */

#define DEFAULT_CONFIG NULL               /* Default config file */
#define DEFAULT_LOGFILE NULL              /* Default logfile */
#define DEFAULT_PIDFILE NULL              /* Default PID file */
#define DEFAULT_PORT 0                 /* Default key server port */
#define DEFAULT_MXCONNTIME 300            /* Maximum connection time */
#define DEFAULT_VERBOSE 0                 /* Default verbose is set to off */
#define DEFAULT_MAXCONN 50                /* Default max simult connects */
#define DEFAULT_DATABASE NULL             /* Default database */
#define DEFAULT_TOKENDB NULL              /* Default spent token database */
#define DEFAULT_KEYSERVERTIMEOUT 15       /* Timeout for talking to K.S. */
#define DEFAULT_MAXDATA 0
#define DEFAULT_RECORD_FILE NULL          /* Default record file */

#define DEFAULT_KEYQRYHOST    NULL          /* Key server hostname */
#define DEFAULT_KEYQRYPORT    0             /* Port number for key server */
#define DEFAULT_KEYQRYTOUT    3             /* Key lookup timeout */
#define DEFAULT_KEYQRYRETR    5             /* Key lookup timeout */

#define DEFAULT_QCNAME        NULL          /* Default response cache name */
#define DEFAULT_QC_TTL        300           /* Default cache time to live */
#define DEFAULT_QC_CLEAN      3600          /* Default cache clean interval */

/*
 * Default NetInfo update frequence
 */
#define DEFAULT_NI_FREQUENCY  1800

/*
 * Default NIQS connection parameters
 */

#define DEFAULT_NIQS_HOST     NULL
#define DEFAULT_NIQS_PORT     51107
#define DEFAULT_NIQS_TIMEOUT  30

/*
 * Default statistics parameters
 */

#define DEFAULT_STAT_HISTORY        1   /* Only the most recent datum */
#define DEFAULT_STAT_FREQUENCY      300   /* Every 5 minutes */

/*
 * Default Mail Certificate parameters
 */
#define DEFAULT_MAILCERT_LIFETIME        2*365 /* 2 years */
#define DEFAULT_MAILCERT_VOLUME_LIMIT    1024  /* 1 MB */
#define DEFAULT_MAILCERT_MAILBOX_LIMIT   3072  /* 3 MB */
#define DEFAULT_MAILCERT_CROSSPOST_LIMIT 5     /* 5 simultaneous news group */

/*
 * Data related to Mail Certificate generation 
 */
struct T_ZkMailCertContext {
  const char *macKeyFile;       /* File containing the MAC Key */
  char *macKeyData;             /* MAC key raw data */
  size_t macKeyDataSize;        /* Size of the MAC key raw data */
  FrCryptMacKey macKey;         /* MAC key required for Mail Certificate authentication */
  uint16_t volLimit;            /* Mail volume limit */
  uint16_t mailboxLimit;        /* Mailbox size limit */
  uint16_t crossPostLimit;      /* News cross-post limit */
  uint16_t lifetime;            /* Mail certficate lifetime (in days) */
};

#define ZKCTX_DTSTART_SZ 32
struct T_ZkKeyContextStatus {
  time_t starttime;        /* time the server was launched */
  uint32_t connections;    /* Total number of connections so far */
  char dtstart[ZKCTX_DTSTART_SZ]; /* UTC Start timestamp */
};

/*
 * Server behaviour flags
 */

#define FLAG_REFUSEACCESS 0x1
#define FLAG_DEBUG        0x2
#define FLAG_DONTFORK     0x4   /* Dont fork a child on start up */
#define FLAG_RUNFORINETD  0x8   /* Run for inetd, I/O goes to std in/out */

/*
 * Key server context structure. The function which spawns off servers will
 * give each thread/process a copy of this.
 */

struct T_ZkKeyContext {
  uint32_t flags;          /* Behaviour flags */
  ZkError err;
  ZkLog log;
  ZkConfig config;
  ZkipEngine engine;
  ZkDB zkdb;               /* The database structure */
  const char * database;   /* The database name to use */
  const char * tokendb;    /* Name of the spent token database for nym server */
  const char * keydbname;  /* Name of key database for nym server */
  int protocol;            /* The ZKIP protocol type */
  int connections;         /* Number of active connections */
  ZkipSession session;     /* session structure for a specific session */
  struct T_ZkKeyContextStatus status; /* Server status */
  FrTokenType tt;          /* Description for token type IDs */
  FrEntIdExp myEnt;        /* Server entity */
  const char * logfile;    /* Server logfile */
  const char * progname;   /* Application program name */
  const char * pidfile;    /* Pointer to the pid file */
  const char * pubencrkey; /* Public Encryption keyfile */
  const char * prvencrkey; /* Private Encryption keyfile */
  const char * pubsigkey;  /* Public signature keyfile */
  const char * prvsigkey;  /* Private signature keyfile */
  const char * recordFile;  /* (optional) record file for update records */
  unsigned int verbose;    /* Verbose logging level (see zkipSession.h) */
  int maxdata;             /* Maximum amount of data allowed on command */
  int maxconn;             /* Maximum simultenous connections */
  uint32_t mxconntime;     /* Maximum number of seconds a connection 
                            * can stay up 
                            */
  struct sockaddr_in server;    /* Server address structure */

  struct T_ZkMailCertContext mailcert; /* Mail Certificate context */

  /* Key query stuff */
  FrKeyQryHost kh;              /* KeyQry Host addr and fd */
  const char *khhost;
  unsigned short khport;
  unsigned short kqtimeout;     /* Timeout for fetching a key */
  int kqretries;                /* Timeout for fetching a key */
  FrKeyQry kq;                  /* Public key cache */

  /* Private key stuff */
  const char *cryptDir;         /* Filename of the crypt key file */
  const char *sigDir;           /* Filename of the sig key file */
  FrSecKeyChain keys;           /* Key chain */
  size_t cryptHandle;           /* Handles for the key chain */
  size_t sigHandle;
  const char *prompt;           /* Prompt for the system passphrase */
  int fd;                       /* Pass phrase file descriptor */
  const char * passfile;        /* passphrase phile */

  FrCryptSig prvsig;            /* Private signature key */
  
  /* Stat collection */
  size_t stat_history;     /* Stat history */
/* Bug: Better inegration of frNetInfo with currently existing way of
 *      storing and accessing server names is required
 */
#define HOST_SZ 256  /* Hack, but need to do this to integrate the frNetInfo
                      * way of getting the stat server name with the old way
                      * of storing and accessing it.
                      */
  char stat_server[HOST_SZ]; /* Stat server */
  unsigned short stat_port; /* Stat port */
  unsigned int stat_frequency; /* Stat frequency */
  FrStatUpd fsu;           /* For stat updates */
  FrStatClient fsc;        /* For sending stat updates */

  FrNetInfo fni;           /* The network information database */
  FrNetInfoChildCtx niCtx;      /* NIQS updater process */
  pid_t niPid;                  /* PID */
  uint32_t ni_frequency;        /* Frequency for updating NetInfo */

  const char *shmtemplate;      /* Shared memory template filename */
  const char *mutextemplate;    /* Mutex template filename */

  /*
   *  bug: we do a fetch on this once, at start-up.  In the future, when key
   *  rotation is in place, we'll have to do it `on the fly'.
   */
  FrCryptCrypt nissPubEncr; /* NISS public encryption key */

  /*
   * Access parameters for the NIQS
   */

  const char * niqs_host;  /* The NIQS hostname */
  uint16_t niqs_port;      /* The NIQS port number */
  uint16_t niqs_timeout;   /* The NIQS connection timeout */
  
  FILE *fptr;                   /* File pointer to the PID file */

  const char * qcname;     /* name of signed response cache */
  uint16_t ttl;            /* Time to live */
  uint16_t qcclean;        /* Interval to clean the response cache file */
};

extern struct T_ZkKeyContext * zkKeyContext;

/*
 * Connection callback
 */

typedef int (*ZkServerMain)( struct sockaddr_in , int , char ** , void *);


int RunAsStreamServer (
  ZkError ,                        /* Returns the error if one occured */
  ZkLog ,                          /* Main server log object */
  struct sockaddr_in * ,           /* Server address */
  ZkServerMain ,                   /* Connection callback */
  int ,                            /* Command line argument count (argc) */
  char ** ,                        /* Command line argument array (argv) */
  void * );                        /* Callback context structure */

int zkKeyMain (
  struct sockaddr_in ,             /* Remote client address */
  int ,                            /* Command line argc */
  char ** ,                        /* Command line argv */
  void * );                        /* Key server context structure pointer */

extern int zkKeyContextGetServerSockaddr ( ZkError , struct T_ZkKeyContext *, 
                                           FrEntIdType , struct sockaddr *,
                                           FrEntIdExp * );

extern int
zkKeyContextGetServerData (
  ZkError err ,
  struct T_ZkKeyContext * c ,
  FrEntIdType type ,
  char * hostname ,
  size_t hsz ,
  uint16_t * port,
  FrEntIdExp * );

#endif /* ZKKEYD_H_ */
