/*
 * $Id: zknsa.h,v 1.1 1999/11/03 01:50:41 philippe Exp $
 */

#ifndef ZKNSA_H_
#define ZKNSA_H_


#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */

#ifndef ZKDB_H
#include <zkDB.h>
#endif /* ZKDB_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define VERSION_STR "Zero-Knowledge Nym Server Administrator V1.0 ("\
                    __DATE__", "__TIME__")"

#define FLAG_DEBUG 0x1

struct T_ZkNSAContext {
  uint32_t flags; 
  ZkError err;
  ZkLog log;
  ZkConfig config;                   /* Holds the configuration structure */
  ZkDB zkdb;                         /* Nym database */
  FILE * input;                      /* Command input stream */
  FILE * output;                     /* Command output stream */
  const char * cmdprompt;            /* Command prompt */
  const char * progname;             /* Name of the application */
  const char * logfile;              /* Name of the logfile */
  const char * configfile;           /* Name of the configuration file */
  const char * pidfile;              /* Name of the pid file */
  const char * database;             /* Nym server database name */
};

#define DEFAULT_CMDPROMPT "ZKNSA>"  /* Default command prompt */
#define DEFAULT_LOGFILE     NULL     /* Default log file */
#define DEFAULT_PIDFILE     NULL     /* Default pid file */
#define DEFAULT_CONFIGFILE  "/etc/zknsa.rc"    /* Default configuration file */
#define DEFAULT_DATABASE    NULL     /* Default nym server database */
#define DEFAULT_INPUT      stdin     /* Default command input stream */
#define DEFAULT_OUTPUT    stdout     /* Default command output stream */

typedef struct T_ZkNSAContext ZkNSAContext;

/*
 * Command line options
 */

#define OPT_VERSION        "--version"
#define OPT_CONFIG         "--config"
#define OPT_LOGFILE        "--logfile"
#define OPT_PIDFILE        "--pidfile"
#define OPT_DATABASE       "--database"
#define OPT_COMMAND        "--command"
#define OPT_HELP           "--help"
#define OPT_DEBUG          "--debug"

/*
 * Configuration file section
 */

#define SECTION "ZKNSA"

/*
 * Configuration file parameters
 */

#define PARM_LOGFILE       "LogFile"
#define PARM_PIDFILE       "PidFile"
#define PARM_DATABASE      "DataBase"
#define PARM_PROMPT        "Prompt"

/*
 * Function prototypes 
 */

int zknsaProcessCommand (
  ZkNSAContext * ,                 /* Application context */
  char * );                        /* Command buffer */

int zknsaCommandLoop (
  ZkNSAContext *);                 /* Application context */


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* ZKNSA_H_ */
