/*******************************************************************************
; Public definitions for the ZKSHIM driver
; Written by Marc Boucher <marc@mbsi.ca>
*******************************************************************************/

/******************************************************************************
 * Extensions for Linux client by Oliver Lavery <oliver@zks.net>
 ******************************************************************************/

#ifndef _ZKSHIM_H
#define _ZKSHIM_H

#ifdef ZK_SOLARIS
#include <sys/ioccom.h>
#endif

#ifdef ZK_LINUX
#include <linux/types.h>
#include <linux/ioctl.h>

#if defined(__KERNEL_STRICT_NAMES) && !defined(_ZKINTTYPES_H)
typedef         __u8            uint8_t;
typedef         __u16           uint16_t;
typedef         __u32           uint32_t;
#endif

#define ZKSHIM_MAJOR 226
#endif

#ifdef  __cplusplus
extern "C" {
#endif

#define ZKSHIM_NAME		"zkshim"
#define ZKSHIM_DEVNAME		"/dev/"ZKSHIM_NAME


#ifdef CLIENT_SHIM

/* Client shim is a minor version upgrade for now. */
#define ZKSHIM_VERSION		"0.8"
#define ZKSHIM_PATCHLEV		"1"

/* Shim Action constants
 *	pretty obvious
 */
#  define ZSA_ALLOW 0

/*	this should implement a firewall style deny (no ICMP) */
#  define ZSA_DENY 1

/*	
 *	Redirect. In the case of a filtered packet it should go to userland. In the
 * 	case of a connect() call it should be redirected to a local proxy service 
 */
#  define ZSA_REDIRECT 2

/*	Redirect with IP transmission.
 *	This is used by freedom to get original dst IPs for connect() calls.
 *	should be equivilent to ZSA_REDIRECT for filtered packets.
 */
#  define ZSA_REDIRECT_TXIP 3

#  if 0
/*	Prompt user process for action on connections which match this rule.
 *	this is achieved by sending a signal to the process which owns this rule
 */
#  	define ZSA_PROMPT 4
#  endif /* 0 */

/* Rule Type constants */

/*	'Socket' rule -- one that is applied to local connect() calls
 */
#  define ZRU_SOCKET 0

/*	'Filter' rule -- one that is applied to packets being transfered to userspace
 */
#  define ZRU_FILTER 1

#else

#define ZKSHIM_VERSION		"0.7"
#define ZKSHIM_PATCHLEV		"2"

#endif /* CLIENT_SHIM */


typedef struct ZkShimRule {
	uint8_t		zru_ip_tos;
	uint8_t		zru_ip_p;
	uint32_t	zru_ip_src, zru_ip_src_mask;
	uint32_t	zru_ip_dst, zru_ip_dst_mask;
	uint16_t	zru_sport;
	uint16_t	zru_dport;
	uid_t		zru_uid;
	pid_t		zru_pid;
	pid_t		zru_sid;
#ifdef CLIENT_SHIM
	/* Client extensions. These are really only useful for zru_type == ZRU_SOCKET */
	uint8_t		zru_type;
	uint8_t		zru_action;
	uint32_t	zru_ip_arg;
	uint16_t	zru_port_arg;
#endif /* CLIENT_SHIM */
} ZkShimRule;

#define ZKSHIM_IOCTL_ID	'Z'

#define ZKIOC_ADD_RULE _IOW (ZKSHIM_IOCTL_ID, 0x01, ZkShimRule)
#define ZKIOC_DEL_RULE _IOW (ZKSHIM_IOCTL_ID, 0x02, ZkShimRule)
#ifdef CLIENT_SHIM
#  define ZKIOC_CLEAR_RULES _IO (ZKSHIM_IOCTL_ID, 0x03)
#  if 0
#    define ZKIOC_SIGPROMPT_ENABLE _IOW (ZKSHIM_IOCTL_ID, 0x04, int)
#    define ZKIOC_SIGPROMPT_DISABLE _IO (ZKSHIM_IOCTL_ID, 0x05)
#    define ZKIOC_SIGPROMPT_REPLY _IOW (ZKSHIM_IOCTL_ID, 0x06, int)
#  endif /* 0 */
#endif

#ifdef  __cplusplus
}
#endif

#endif /* _ZKSHIM_H */
