/*******************************************************************************
; zkshim driver
; Written by Marc Boucher <marc@mbsi.ca>
*******************************************************************************/

#include "../zkshim_priv.h"

proc_t *
zkShimFindVnodeProc(vnode_t *vnode)
{
	proc_t *p;
	user_t *uu;
	struct uf_entry *uf;

	for(p = practive; p != NULL; p = p->p_next) {
		int i;

		uu = PTOU(p);

		for(i=0, uf = uu->u_flist; i < uu->u_nofiles; i++, uf++) {
			if(!uf->uf_ofile)
				continue;

			if((uf->uf_ofile->f_count <= 0) || uf->uf_ofile->f_vnode != vnode)
				continue;

			ZKDBG(15, "p=0x%p (%s) uf slot %d: 0x%p %d %d\n", p, PTOU(p)->u_psargs, i, uf->uf_ofile, (int)uf->uf_pofile, (int)uf->uf_refcnt);
			return p;
		}
	}

	return NULL;
}

static boolean_t
zkShimFindModuleState(queue_t *wrq, ipc_t *ips, char *name, char *ts, int tsl)
{
	int got_ip = 0;
	int got_tp = 0;

	ZKDBG(55, "wrq=0x%p name=%s tsl=%d\n", wrq, name, tsl);

	for(; wrq; wrq = wrq->q_next) {
		if(!wrq->q_qinfo || !wrq->q_qinfo->qi_minfo || !wrq->q_qinfo->qi_minfo->mi_idname)
			continue;
		if(!got_ip && !strcmp(wrq->q_qinfo->qi_minfo->mi_idname, "ip")) {
			ZKDBG(55, "mi_idname=ip found, wrq=0x%p\n", wrq);
			bcopy(wrq->q_ptr, ips, sizeof(*ips));
			got_ip = 1;
		} else if(!got_tp && !strcmp(wrq->q_qinfo->qi_minfo->mi_idname, name)) {
			ZKDBG(55, "mi_idname=%s found, wrq=0x%p\n", name, wrq);
			bcopy(wrq->q_ptr, ts, tsl);
			got_tp = 1;
		} else
			ZKDBG(55, "mi_idname=%s\n", wrq->q_qinfo->qi_minfo->mi_idname);
	}

	return got_ip && got_tp;
}

struct sonode *
zkShimFindLocalSocket(uint8_t ip_proto, uint32_t ip_laddr, uint16_t ip_lport)
{
	int i;
	kmem_cache_t *kc;
	kmem_slab_t *ks;

	kc = sock_cache;
	if(!kc) {
		return NULL;
	}
	
	ZKDBG(35, "sock_cache=0x%p cache_bufsize=%d cache_hash_table=0x%p cache_chunksize=%d\n", kc, kc->cache_bufsize, kc->cache_hash_table, kc->cache_chunksize);

	ASSERT(kc->cache_bufsize == sizeof(struct sonode));

	ks = kc->cache_nullslab.slab_next;
	while(ks && (ks != &kc->cache_nullslab)) {
		ZKDBG(35, "ks=0x%p slab_base=0x%p slab_refcnt=%d slab_chunks=%d\n", ks, ks->slab_base, ks->slab_refcnt, ks->slab_chunks);
		for(i= 0/* ks->slab_chunks - ks->slab_refcnt*/; i<ks->slab_chunks; i++) {
			struct sonode *so;
			int stype;
			char *mname;
			void *ts;
			int tsl;
			ipc_t ips;
			struct tcp_s tcps;
			struct udp_s {
				uint udp_state;     /* TPI state */
				unsigned char d1[2];
				uint16_t udp_port;  /* port bound to this stream */
				uint32_t udp_src;   /* source address of this stream */
			} udps;
			uint16_t lport;
			uint32_t laddr;

			if(ip_proto == IPPROTO_TCP) {
				stype = SOCK_STREAM;
				mname = "tcp";
				ts = &tcps;
				tsl = sizeof(tcps);
			} else if(ip_proto == IPPROTO_UDP) {
				stype = SOCK_DGRAM;
				mname = "udp";
				ts = &udps;
				tsl = sizeof(udps);
			} else {
				stype = 0;
				mname = "ip";
				ts = NULL;
				tsl = 0;
			}

			so = (struct sonode *)((unsigned char *)ks->slab_base + i * kc->cache_chunksize);
			if((so->so_vnode.v_count > 0) && so->so_vnode.v_stream && (so->so_count > 0) && (so->so_family == AF_INET) && (!stype || (stype == so->so_type))) {
				if (!zkShimFindModuleState(so->so_vnode.v_stream->sd_wrq, &ips, mname, ts, tsl))
					continue;

				laddr = ips.ipc_laddr;
				lport = ips.ipc_lport;

				if((laddr == INADDR_ANY) && !lport) {
					if(ip_proto == IPPROTO_TCP) {
						if(tcps.tcp_hdr_len) {
							lport = *((uint16_t *)&tcps.tcp_tcph->th_lport[0]);
						} else {
							lport = tcps.tcp_lport;
						}
						laddr = *((uint32_t *)&tcps.tcp_iph.iph_src[0]);
					} else if(ip_proto == IPPROTO_UDP) {
						lport = udps.udp_port;
						laddr = udps.udp_src;
					}
				}

				ZKDBG(55, "laddr=0x%x lport=%d\n", laddr, lport);

				if((laddr == ip_laddr) && (lport == ip_lport)) {
					ZKDBG(35, "FOUND so=0x%p v_type=%d so_laddr_len=%d 0x%x:%d\n", so, so->so_vnode.v_type, so->so_laddr_len, ((struct sockaddr_in *)so->so_laddr_sa)->sin_addr.s_addr, ((struct sockaddr_in *)so->so_laddr_sa)->sin_port);
					return so;
				}
			}
		}
		ks = ks->slab_next;
	}

	ZKDBG(35, "returning NULL\n");
	return NULL;

}
