/*******************************************************************************
; zkshim driver, misc utility functions
; Written by Marc Boucher <marc@mbsi.ca>
*******************************************************************************/

#include "zkshim_priv.h"

#ifdef ZKSHIM_DEBUG
int zkshim_debug = ZKSHIM_DEBUG;
#ifdef ZK_LINUX
MODULE_PARM(zkshim_debug, "i");
EXPORT_SYMBOL(zkshim_debug);
#endif

#if defined(ZK_SOLARIS) || !defined(__GNUC__)
void
zkDebugOut_va(int level, char *fmt, __va_list ap)
{
	if(level >= 2) {
		char buf[100];

#warning "Please explain this code"
		strcpy(buf, "!"); /* prevent output to console */
		strcat(buf, fmt);
		vcmn_err(CE_CONT, buf, ap);
	} else
		vcmn_err(CE_CONT, fmt, ap);
}
#endif /* ZK_SOLARIS || !__GNUC__ */

#ifdef ZK_SOLARIS
void
zkDebugOut(int level, char *fmt, ...)
{
	__va_list ap;

	va_start(ap, fmt);
	zkDebugOut_va(level, fmt, ap);
	va_end(ap);
}

char *
zkDebugOutQueue(struct queue *q, char *buf)
{
	if(q) {
		char *idname = NULL;

		if(q->q_qinfo && q->q_qinfo->qi_minfo)
			idname = q->q_qinfo->qi_minfo->mi_idname;

		sprintf(buf, "0x%p/%s/0x%p", q, idname ? idname : "", q->q_ptr);
	} else {
		sprintf(buf, "NULL");
	}

	return buf;
}

char *
zkDebugOutMblk(mblk_t *mp, char *buf)
{
	strcpy(buf, "(");
	while (mp) {
		if(mp->b_datap) {
			switch(mp->b_datap->db_type) {
			case M_IOCTL:
				sprintf(buf+strlen(buf), "0x%p[%s len=%d cmd=%s]", mp, zkDebugMsgType(mp->b_datap->db_type), msgblen(mp), zkDebugIoctlType(((struct iocblk *)mp->b_rptr)->ioc_cmd));
				break;
#if 0 /* XXX print DLPI primtype ? */
			case M_PROTO:
			case M_PCPROTO:
				break;
#endif
			default:
				sprintf(buf+strlen(buf), "0x%p[%s len=%d]", mp, zkDebugMsgType(mp->b_datap->db_type), msgblen(mp));
				break;
			}
		} else
			sprintf(buf+strlen(buf), "0x%p[b_datap=NULL]", mp);

		if((mp = mp->b_cont)) {
			strcat(buf, ", ");
		}
	}
	strcat(buf, ")");

	return buf;
}

const char *
zkDebugMsgType(unsigned char type)
{
	static char nbuf[60]; /* not reentrant */

#define caseretstr(x) case x: return #x
	switch(type) {
	caseretstr(M_DATA);
	caseretstr(M_PROTO);
	caseretstr(M_BREAK);
	caseretstr(M_PASSFP);
	caseretstr(M_EVENT);
	caseretstr(M_SIG);
	caseretstr(M_DELAY);
	caseretstr(M_CTL);
	caseretstr(M_IOCTL);
	caseretstr(M_SETOPTS);
	caseretstr(M_RSE);
	caseretstr(M_IOCACK);
	caseretstr(M_IOCNAK);
	caseretstr(M_PCPROTO);
	caseretstr(M_PCSIG);
	caseretstr(M_READ);
	caseretstr(M_FLUSH);
	caseretstr(M_STOP);
	caseretstr(M_START);
	caseretstr(M_HANGUP);
	caseretstr(M_ERROR);
	caseretstr(M_COPYIN);
	caseretstr(M_COPYOUT);
	caseretstr(M_IOCDATA);
	caseretstr(M_PCRSE);
	caseretstr(M_STOPI);
	caseretstr(M_STARTI);
	caseretstr(M_PCEVENT);
	caseretstr(M_UNHANGUP);
    default:
    snprintf(nbuf, sizeof nbuf, "<unknown type 0x%x>", (int)type);
		return (const char *)nbuf;
	}
}
#endif /* ZK_SOLARIS */

const char *
zkDebugIoctlType(int type)
{
	static char nbuf[60]; /* not reentrant */

#define caseretstr(x) case x: return #x
	switch(type) {
	caseretstr(ZKIOC_ADD_RULE);
	caseretstr(ZKIOC_DEL_RULE);
#ifdef ZK_SOLARIS
	caseretstr(SIOCSHIWAT);
	caseretstr(SIOCGHIWAT);
	caseretstr(SIOCSLOWAT);
	caseretstr(SIOCGLOWAT);
	caseretstr(SIOCATMARK);
	caseretstr(SIOCSPGRP);
	caseretstr(SIOCGPGRP);
	caseretstr(SIOCADDRT);
	caseretstr(SIOCDELRT);
	caseretstr(SIOCGETVIFCNT);
	caseretstr(SIOCGETSGCNT);
	caseretstr(SIOCSIFADDR);
	caseretstr(SIOCGIFADDR);
	caseretstr(SIOCSIFDSTADDR);
	caseretstr(SIOCGIFDSTADDR);
	caseretstr(SIOCSIFFLAGS);
	caseretstr(SIOCGIFFLAGS);
	caseretstr(SIOCSIFMEM);
	caseretstr(SIOCGIFMEM);
	caseretstr(SIOCGIFCONF);
	caseretstr(SIOCSIFMTU);
	caseretstr(SIOCGIFMTU);
	caseretstr(SIOCGIFBRDADDR);
	caseretstr(SIOCSIFBRDADDR);
	caseretstr(SIOCGIFNETMASK);
	caseretstr(SIOCSIFNETMASK);
	caseretstr(SIOCGIFMETRIC);
	caseretstr(SIOCSIFMETRIC);
	caseretstr(SIOCSARP);
	caseretstr(SIOCGARP);
	caseretstr(SIOCDARP);
	caseretstr(SIOCUPPER);
	caseretstr(SIOCLOWER);
	caseretstr(SIOCSETSYNC);
	caseretstr(SIOCGETSYNC);
	caseretstr(SIOCSSDSTATS);
	caseretstr(SIOCSSESTATS);
	caseretstr(SIOCSPROMISC);
	caseretstr(SIOCADDMULTI);
	caseretstr(SIOCDELMULTI);
	caseretstr(SIOCGETNAME);
	caseretstr(SIOCGETPEER);
	caseretstr(SIOCXPROTO);
	caseretstr(SIOCIFDETACH);
	caseretstr(SIOCGENPSTATS);
	caseretstr(SIOCX25XMT);
	caseretstr(SIOCX25RCV);
	caseretstr(SIOCX25TBL);
	caseretstr(SIOCSLGETREQ);
	caseretstr(SIOCSLSTAT);
	caseretstr(SIOCSIFNAME);
	caseretstr(SIOCGENADDR);
	caseretstr(SIOCGIFNUM);
	caseretstr(SIOCGIFMUXID);
	caseretstr(SIOCSIFMUXID);
	caseretstr(SIOCGIFINDEX);
	caseretstr(SIOCSIFINDEX);
#endif /* ZK_SOLARIS */
    default:
		sprintf(nbuf, sizeof nbuf, "<unknown type 0x%u>", type);
		return (const char *)nbuf;
	}
}

#endif /* ZKSHIM_DEBUG */
