/*
 * $Id: directory.c,v 1.3 2000/10/04 12:57:56 marcp Exp $
 */

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <zkList.h>
#include <frCrypt.h>
#include <frSec.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "zkxfer.h"
#include "directory.h"
#include <zkMemBuf.h>

int
xferCommandCat (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  const char * func = "xferCommandCat";
  ZkMemBuf membuf = NULL;
  int return_code = _SESSION_CONTINUE;
  int parameters = 0;
  ZKXFERContext * context = (ZKXFERContext *) data;
  DIR * dir = NULL;
  struct dirent * dirent = NULL;
  char * buffer = NULL;
  int add_crlf = 0; /* For cleaner looking output */
  size_t sz = 0;
  char pathname[MAXPATHLEN + 1];
  struct stat sbuf;  /* For getting description of files in directory */

  zkErrorClear(err);
  if (session && data) {

    /*
     * Make sure we got no parameters
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogError(context->log,err);
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 0) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "No parameters expected","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    zkMemBufNew(err,&membuf);

    /*
     * Open the server outbox directory
     */

    if (context->outbox) {
      if ((dir = opendir(context->outbox)) == NULL) {
        zkLogWrite_va(context->log,"Unable to open outbox \"%s\"",
                      context->outbox);
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Unable to open server outbox","NODATA",
                            ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      }
  
      /*
       * Spit out the directory contents
       */
  
      while ((dirent = readdir(dir)) != NULL) {
        if (dirent->d_name) {
          snprintf(pathname,sizeof pathname,"%s/%s",context->outbox,dirent->d_name);
          if (stat(pathname,&sbuf) != 0) {
            continue;
          }
          if (!(sbuf.st_mode & S_IFREG)) {
            continue;
          }
          if (add_crlf) {
            if (!zkMemBufAppend(err,membuf,"\015\012",2)) {
              zkLogError(context->log,err);
            } 
          } else {
            add_crlf = 1;
          }
          if (!zkMemBufAppend(err,membuf,dirent->d_name,
                              strlen(dirent->d_name))) {
            zkLogError(context->log,err);
          }
        }
      }
      closedir(dir);
      if (!zkMemBufGetData(err,membuf,(void **)&buffer,&sz)) {
        zkLogError(context->log,err);
      }
    } else { /* No outbox directory */
      buffer = NULL;
      sz = 0;
    }
  
    zkipSessionResponse(err,session,0,_SERVICE_READY,"Files available",
                        "DIRECTORY",ZKIP_ENCODING_BINARY,sz,buffer);

finish:
    zkMemBufFree(NULL,&membuf);
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMXFER,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}
