/*
 * $Id: download.c,v 1.3 2000/10/04 12:57:56 marcp Exp $
 */

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <zkList.h>
#include <frCrypt.h>
#include <frSec.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <zkFile.h>
#include "zkxfer.h"
#include "download.h"

int
xferCommandDownload (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
  const char * func = "xferCommandCat";
  int return_code = _SESSION_CONTINUE;
  int parameters = 0;
  ZKXFERContext * context = (ZKXFERContext *) data;
  DIR * dir = NULL;
  struct dirent * dirent = NULL;
  char * buffer = NULL;
  int add_crlf = 0; /* For cleaner looking output */
  size_t sz = 0;
  char pathname[MAXPATHLEN + 1];
  char * filename = NULL;
  struct stat sbuf;  /* For getting description of files in directory */

  zkErrorClear(err);
  if (session && data) {

    /*
     * Get the filename to download
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkLogError(context->log,err);
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 1) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "No parameters expected","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }
    if (!zkipSessionParameter(err,session,0,&filename)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to retrieve filename",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    /*
     * Do we have an outbox?
     */

    if (context->outbox == NULL) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Sorry, this server does not have an outbox",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL); 
      goto finish;
    }

    /*
     * Check the filename to make sure it wont wander about the filesystem
     */
  
    if (strchr(filename,'/') != NULL) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Specified filename is invalid",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    
    /*
     * Load the file
     * Bug: check the combined lengths of these strings to avoid overflow
     */
    
    snprintf(pathname,sizeof pathname,"%s/%s",context->outbox,filename);
    if (!zkFileLoad(err,pathname,&buffer,&sz)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to load requested file",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }
  
    zkipSessionResponse(err,session,0,_SERVICE_READY,filename,
                        "FILE",ZKIP_ENCODING_BINARY,sz,buffer);

finish:
    if (buffer) {
      free(buffer);
    }
    return return_code;
  }
  zkErrorSet(err,func,ZKERR,ZKMXFER,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}
