/*
 * $Id: upload.c,v 1.4 2000/10/04 12:57:56 marcp Exp $
 */

#include "zks.h"

#include <zkIntTypes.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif /* !WIN32 */
#include <string.h>
#include <zkList.h>
#include <frCrypt.h>
#include <frSec.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "zkxfer.h"
#include "upload.h"

/*
 * Bug:
 *    Right now, (let's hear it for expedience) the client upload command will
 *    create a new connection for each file.  It would be better to send 
 *    multiple files per connection (The server part already supports this)
 */

#warning "Allow the upload command to send multiple files over a single connection"

int
xferCommandUpload (
  ZkError err ,
  ZkipSession session ,
  void * data )
{
#define PATH_SZ 256
  int return_code = _SESSION_CONTINUE;
  char pathname[PATH_SZ + 1];
  char * filename = NULL;
  char * filedata = NULL;
  size_t filedata_sz = 0;
  int parameters = 0;
  int fd = 0;
  int n = 0;
  struct stat sbuf;
  ZKXFERContext * context = (ZKXFERContext *) data;

  zkErrorClear(err);
  if (session) {

    /* 
     * Get the filename
     */

    if (!zkipSessionParameters(err,session,&parameters)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Could not get command parameters","NODATA",
                          ZKIP_ENCODING_NONE,0,NULL);
      return_code = _SESSION_ERROR;
      goto finish;
    } else if (parameters != 1) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "You must specify a filename",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 
    if (!zkipSessionParameter(err,session,0,&filename)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to retrieve filename",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /*
     * The filename cannot contain slashes...
     */

    while (filename[n]) {
      if (filename[n] == '/') {
        zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                            "Illegal filename",
                            "NODATA", ZKIP_ENCODING_NONE,0,NULL);
        goto finish;
      } else {
        n++;
      }
    }
    
    /*
     * Build up a proper full pathname
     */

    if ((strlen(filename) + strlen(zkXFERContext->spooldir) + 2) >= PATH_SZ) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Combined destination path and filename too large",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 
    snprintf(pathname,sizeof pathname,"%s/%s",zkXFERContext->spooldir,filename);
    pathname[PATH_SZ] = (char) 0;

    /* 
     * If the file is there, dont overwrite it unless the OVERWRITE flag
     * has been set.
     */

    if ((stat(pathname,&sbuf) == 0) && 
        (!(zkXFERContext->flags & FLAG_OVERWRITE))) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "File exists",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    }

    /*
     * Open the file
     */
    
    if ((fd = open(pathname,O_WRONLY|O_CREAT|O_TRUNC)) < 0) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,
                          "Unable to create file",
                          "NODATA", ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 
    if (context) {
      chmod(pathname,context->permissions);
    }

    /*
     * Read the file data
     */
 
    if (!zkipSessionReadData(err,session,&filedata,&filedata_sz)) {
      zkipSessionResponse(err,session,0,_COMMAND_ERROR,"Incorrect data block",
                          "NODATA",ZKIP_ENCODING_NONE,0,NULL);
      goto finish;
    } 

    /* 
     * Write the filedata to disk 
     */

    write(fd,filedata,filedata_sz);

    zkipSessionResponse(err,session,0,_SERVICE_READY,
                        "File received",
                        "NODATA",ZKIP_ENCODING_8BIT,0,NULL);
    return_code =  _SESSION_CONTINUE;
 
finish:
    if (fd > 0) {
      close(fd);
    }
    free(filedata);
    return return_code;
  }
  zkErrorSet(err,"xferCommandUpload",ZKERR,ZKMXFER,ZKENULLPTR,NULL);
  return _SESSION_ERROR;
}

