/*
 *  $Id: zkxfer.h,v 1.6 1999/02/19 20:07:02 bill Exp $
 */

#ifndef ZKXFER_H_
#define ZKXFER_H_

#ifndef _ZKERROR_H
#include <zkError.h>
#endif /* _ZKERROR_H */

#ifndef _ZKLOG_H
#include <zkLog.h>
#endif /* _ZKLOG_H */

#ifndef _ZKCONFIG_H
#include <zkConfig.h>
#endif /* _ZKCONFIG_H */

#ifndef _ZKIP_H
#include <ZKIP.h>
#endif /* _ZKIP_H */

#ifndef _ZKIPENGINE_H
#include <zkipEngine.h>
#endif /* _ZKIPENGINE_H */

#include <frCrypt.h>


#ifndef ZKLIST_H_
#include <zkList.h>
#endif /* ZKLIST_H_ */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <frFilenames.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define VERSION_XFER_STR "Zero-Knowledge XFER server V1.0 ("__DATE__", "__TIME__")"

/*
 * Command line parameters (OPT_...) and configuration file paramerers (PARM_)
 */

#define OPT_DEBUG       "--debug"         /* Turn on debugging mode */
#define OPT_CONFIG      "--config"
#define OPT_HELP        "--help"
#define OPT_PORT        "--port"
#define OPT_VERSION     "--version"
#define OPT_PIDFILE     "--pidfile"
#define OPT_NAME        "--name"
#define OPT_MAXDATA     "--maxdata"
#define OPT_VERBOSE     "--verbose"
#define OPT_MAXCONNTIME "--maxconnectiontime"
#define OPT_MAXCONN     "--maxconnections"
#define OPT_DONTFORK    "--dontfork"
#define OPT_LOGFILE     "--logfile"
#define OPT_TIMEOUT     "--timeout"
#define OPT_SPOOLDIR    "--inbox"
#define OPT_OUTBOX      "--outbox"
#define OPT_SIGFILE     "--sigfile"
#define OPT_CRYPTFILE   "--cryptfile"
#define OPT_COMMAND     "--command"   /* Client mode only */
#define OPT_RUNASSERVER "--servermode"
#define OPT_OVERWRITE   "--overwrite"
#define OPT_REPEAT      "--repeat"   /* Loop on this interval (for client) */
#define OPT_SERVER      "--server"
#define OPT_LFILENAME   "--local"    /* Local filename (for client)  */
#define OPT_RFILENAME   "--remote"   /* Remote filename (for client) */
#define OPT_PERMISSIONS "--permissions" /* File permission (for server ) */
#define OPT_DELETE      "--delete"   /* Remote files as the are sent (client) */

#ifndef WIN32
#warning "not updated to new config file conventions"
#endif

#define PARM_PORT        "Port"
#define PARM_PIDFILE     "PidFile"
#define PARM_NAME        "Name"
#define PARM_MAXDATA     "MaxData"
#define PARM_VERBOSE     "Verbose"
#define PARM_MAXCONNTIME "MaxConnectionTime"
#define PARM_MAXCONN     "MaxConnections"
#define PARM_DONTFORK    "DontFork"
#define PARM_LOGFILE     "Logfile"
#define PARM_TIMEOUT     "Timeout"           /* Client mode only */
#define PARM_SPOOLDIR    "InBox"
#define PARM_COMMAND     "Command"   /* Client mode only */
#define PARM_OUTBOX      "OutBox"
#define PARM_REPEAT      "--repeat"   /* Loop on this interval (for client) */
#define PARM_SIGFILE     "SigFile"
#define PARM_CRYPTFILE   "CryptFile"
#define PARM_OVERWRITE   "Overwrite"
#define PARM_RUNASSERVER "ServerMode"
#define PARM_PERMISSIONS "Permissions"  /* File permissions (server) */
#define PARM_SERVER      "Server"
#define PARM_LFILENAME   "Local"           /* Local filename for client */
#define PARM_RFILENAME   "Remote"          /* Remote filename for client */
#define PARM_DELETE      "Delete"   /* Remove files as they are sent */
#define PARM_ENTITY      "Allow"


struct T_ZKXFERContextStatus {
  time_t starttime;       /* Time the server was launcher */
  uint32_t connections;   /* Total number of connections so far */
  uint32_t active;        /* Current number of active connections */
  char dtstart[32];       /* UTC Start timestamp */
};

typedef struct T_ZKXFERContextStatus ZKXFERContextStatus;

#define CMD_UNKNOWN           0
#define CMD_NONE              1
#define CMD_DOWNLOAD          2
#define CMD_UPLOAD            3
#define CMD_DIRECTORY         4
 

/*
 * Some context default values
 */

#define DEFAULT_CONFIGFILE    NULL          /* Default configuration file */
#define DEFAULT_PROGNAME      "zkXFER"
#define DEFAULT_PIDFILE       NULL          /* Stores a PID on disk */
#define DEFAULT_NAME          NULL          /* Optional name for server */
#define DEFAULT_LOGFILE       NULL          /* Default logfile */
#define DEFAULT_MAXDATA       0             /* Maximum data / command 0 = off */
#define DEFAULT_VERBOSE       0             /* Default verbose logging mode */
#define DEFAULT_CMDID         CMD_NONE      /* Default command ID */
#define DEFAULT_MXCONN        50            /* Max number of simult. connects */
#define DEFAULT_MXCONNTIME    900           /* Maximum connection duration */
#define DEFAULT_TIMEOUT       30            /* dflt conn. timeout (client)*/
#define DEFAULT_SPOOLDIR      NULL          /* Default spool directory */
#define DEFAULT_OUTBOX        NULL          /* Default outbox directory */
#define DEFAULT_CRYPTFILE     FREEDOM_PRIVKEYDIR"/aip.%s.c.1024.1"
#define DEFAULT_SIGFILE       FREEDOM_PRIVKEYDIR"aip.%s.s.1024.1"
#define DEFAULT_PORT          9090          /* XFER port (server and client) */
#define DEFAULT_RFILENAME     NULL          /* dflt target filename (client) */
#define DEFAULT_REPEAT        0             /* Disabled when set to 0 */
#define DEFAULT_LFILENAME     NULL          /* Local filname (for client) */
#define DEFAULT_ALTTEXTFILE   NULL
#define DEFAULT_MTAOPT_LIST   NULL
#define DEFAULT_SERVER        NULL
#define DEFAULT_PERMISSIONS   0640          /*Default perm -rw-r----- (both)*/

/*
 * Context behaviour flags (store in the flags variable)
 */

#define FLAG_DEBUG        0x1 /* Debug mode on */
#define FLAG_REFUSEACCESS 0x2 /* Refuse client connections */
#define FLAG_DONTFORK     0x4 /* Dont immediatly fork */
#define FLAG_RUNASSERVER  0x8 /* Run as server instead of client */
#define FLAG_OVERWRITE    0x10 /* Allow files to be overwritten */
#define FLAG_RUNASCLIENT  0x20 /* Run the program as a zkxfer client */
#define FLAG_DELETE       0x40 /* Delete files as they are sent */

struct T_ZKXFERContext {
  uint32_t flags;             /* Application behaviour flags */
  int exit_code;              /* Application exit code on termination */
  ZkError err;                /* Common error structure for application */
  ZkLog log;                  /* Application log file */
  ZkConfig config;            /* Application configuration */
  ZkipSession session;        /* ZkipSession */
  ZkipEngine engine;
  ZKXFERContextStatus status; /* Server status description */
  struct sockaddr_in server;  /* The server address structure */
  const char * progname;      /* Application program name (argv[0]) */
  const char * name;          /* A maybe unique name for the server */
  const char * configfile;    /* Name of the configuration file */
  const char * pidfile;       /* Name of the PID file */
  const char * logfile;       /* Name of the logfile */
  const char * spooldir;      /* Spool directory */
  const char * outbox;        /* Outbox directory for downloads and dirs */
  const char * rfilename;     /* target remote filename (for client) */
  const char * lfilename;     /* local filename (for client) */
  const char * rserver;       /* remote XFER server (when used as a client)*/
  const char * command;       /* The command name (when used as a client) */
  unsigned short cmdid;       /* Command ID (client, see list above) */
  unsigned short port;        /* XFER port (client and server) */
  unsigned short timeout;     /* XFER server timeout (when used as client) */
  char * data;                /* Data to send over */
  size_t dsz;                 /* Number of bytes to send */
  int maxdata;                /* Maximum amount of data per ZKIP command */
  unsigned int verbose;       /* Verbose logging level (see ZKIP) */
  uint32_t mxconn;            /* Maximum number of simultaneous connections */
  unsigned short repeat;      /* Client repeat interval */
  int mxconntime;             /* Maximum duration of a connection */
  unsigned int permissions;   /* Default file create permissions (server) */
  const char *cryptFile;      /* Filename of the crypt key file */
  const char *sigFile;        /* Filename of the sig key file */
  FrCryptCrypt myCryptKey;    /* My private crypto key */
  FrCryptSig mySigKey;        /* My private crypto key */
  size_t aclsz;               /* Number of entries in the ACL */
  char ** acl;                /* Access Control List */
};

typedef struct T_ZKXFERContext ZKXFERContext;

/*
 * Ok, its a global variable (*cringe*) but it is needed for the signal 
 * handlers so they have some clue as to what is going on 
 */

extern ZKXFERContext * zkXFERContext;


/*
 * Connection callback
 */

typedef int (*ZkServerMain)( struct sockaddr_in , void *);
typedef int (*ZkQueueLoop)(void *);

int RunAsStreamServer (
  ZkError ,                        /* Returns the error if one occured */
  ZkLog ,                          /* Main server log object */
  struct sockaddr_in * ,           /* Server address */
  ZkServerMain ,                   /* Connection callback */
  void * );                        /* Callback context structure */

int zkXFERMain (
  struct sockaddr_in ,             /* Remote client address */
  void * );                        /* Key server context structure pointer */

/*
 * Some other prototypes....
 */

int zkXFERUsage (
  ZKXFERContext * ,          /* The context */
  FILE *  );   /* The output stream */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKXFER_H_ */
