/*
 *  $Id: zkIntTypes.h,v 1.13 2001/04/23 18:49:26 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  zkIntTypes.h
 */ 

#ifndef _ZKINTTYPES_H
#define _ZKINTTYPES_H

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif



#if defined(ZK_KERNEL)
  
  #define HAVE_INTTYPES		0
  #define HAVE_OPERATORS	0
  
#elif defined(ZK_SOLARIS)
  
  #define HAVE_INTTYPES		1
  #define HAVE_OPERATORS	0
  
#elif defined(macintosh)
  
  #define HAVE_INTTYPES		1
  #define HAVE_OPERATORS	1
  
#elif defined(__linux)

  /*
   *  RedHat 5.2 doesn't have inttypes.h, but 6.1 does.  Under Linux we'll
   *  assume its existence depends on the version of the GNU C compiler.
   *  
   *  bug: what's a better way to do this?
   */
  
  #ifndef __GNUC__
  #error "expecting Linux to use GNU C"
  #endif

  #if __GNUC__ >= 2 && __GNUC_MINOR__ >= 91
  #define HAVE_INTTYPES 1
  #else
  #define HAVE_INTTYPES 0
  #endif
  
  
  #define HAVE_OPERATORS	0
  
#else
  
  #define HAVE_INTTYPES		0
  
  #define HAVE_OPERATORS	0
  
#endif


#if 0

  /*
   *  Typedefs similar to the following appear in the new C standard, but
   *  currently don't exist on all systems.  Define the ones that are missing.
   *  
   *  This is sample code only; don't `#if 1' it.
   */
  
  typedef signed char int8_t;
  typedef unsigned char uint8_t;

  typedef short int int16_t;
  typedef unsigned short uint16_t;

  typedef int int32_t;
  typedef unsigned int uint32_t;

  typedef long long int int64_t;
  typedef unsigned long long uint64_t;
  
  typedef signed char int_least8_t;
  typedef unsigned char uint_least8_t;
  
  typedef short int int_least16_t;
  typedef unsigned short int uint_least16_t;
  
  typedef int int_least32_t;
  typedef unsigned int uint_least32_t;

  typedef long long int_least64_t;
  typedef unsigned long long uint_least64_t;

  typedef int64_t intmax_t;
  typedef uint64_t uintmax_t;

  /* for some reason, both uint64_t and UINT64_T are used in the code */
  typedef uint64_t UINT64_T;
  
#endif


#if HAVE_INTTYPES
  /*
   *  This header file is required by the latest version of the C standard.
   */

	#include <inttypes.h>

#elif defined(ZK_KERNEL)

	#include "zkKrtIntTypes.h"

#elif defined(WIN32)
  

	/*
	*  Define unsigned types of specific sizes.  The #if's are used to ensure
	*  the code won't compile if the sizes are not what we're expecting.
	*/
	#include <limits.h>

	# ifdef __cplusplus
		extern "C" {
	# endif

	typedef char int8_t;
	#if CHAR_MAX != 127
	#error "the size of this type is not what we expect!"
	#endif
	#if CHAR_BIT != 8
	#error "we expect chars to be eight bits!"
	#endif

	typedef unsigned char  uint8_t;
	#if UCHAR_MAX != 0xff
	#error "the size of this type is not what we expect!"
	#endif

	typedef short int16_t;
	#if SHRT_MAX != 0x7fff
	#error "the size of this type is not what we expect!"
	#endif

	typedef unsigned short uint16_t; 
	#if USHRT_MAX != 0xffff
	#error "the size of this type is not what we expect!"
	#endif

	typedef int int32_t;
	#if INT_MAX != 0x7fffffff
	#error "the size of this type is not what we expect!"
	#endif

	typedef unsigned int uint32_t;
	#if UINT_MAX != 0xffffffff
	#error "the size of this type is not what we expect!"
	#endif

	/* The following type is guaranteed to be signed and 64 bits wide. */
	typedef __int64 int64_t;


	/* The following type is guaranteed to be unsigned and 64 bits wide. */
	typedef unsigned __int64 uint64_t;

	typedef int8_t int_least8_t;
	typedef uint8_t uint_least8_t;

	typedef int16_t int_least16_t;
	typedef uint16_t uint_least16_t;

	typedef int32_t int_least32_t;
	typedef uint32_t uint_least32_t;

	typedef int64_t int_least64_t;
	typedef uint64_t uint_least64_t;

	typedef int64_t intmax_t;
	typedef uint64_t uintmax_t;

	#define INT8_MAX        _I8_MAX
	#define INT16_MAX       _I16_MAX
	#define INT32_MAX       _I32_MAX
	#define UINT8_MAX       _UI8_MAX
	#define UINT16_MAX      _UI16_MAX
	#define UINT32_MAX      _UI32_MAX
	#define INT64_MAX       _I64_MAX
	#define UINT64_MAX      _UI64_MAX
	#define INTMAX_MAX      _I64_MAX
	#define UINTMAX_MAX     _I64_MAX
	#define INT_LEAST8_MAX  INT8_MAX
	#define INT_LEAST16_MAX INT16_MAX
	#define INT_LEAST32_MAX INT32_MAX
	#define INT_LEAST64_MAX INT64_MAX
	#define UINT_LEAST8_MAX UINT8_MAX
	#define UINT_LEAST16_MAX UINT16_MAX
	#define UINT_LEAST32_MAX UINT32_MAX
	#define UINT_LEAST64_MAX UINT64_MAX
	#define INTPTR_MAX
	#define UINTPTR_MAX
	#define INT8_MIN        _I8_MIN
	#define INT16_MIN       _I16_MIN
	#define INT32_MIN       _I32_MIN
	#define INT64_MIN       _I64_MIN
	#define INTMAX_MIN      INT64_MIN
	#define INT_LEAST8_MIN  INT8_MIN
	#define INT_LEAST16_MIN INT16_MIN
	#define INT_LEAST32_MIN INT32_MIN
	#define INT_LEAST64_MIN INT64_MIN

	#define LONG_LONG_FTM	"%I64d"
	#define LONG_LONG		__int64

	# ifdef __cplusplus
		}
	# endif


#elif defined(__linux)

  
  /*
   *
   *
   *                                Linux
   *
   *
   */

# include <sys/types.h>

  #ifdef __cplusplus
    extern "C" {
  #endif /* __cplusplus */


  /*
   *  In the new C standard, the following preprocessor definitions appear in
   *  inttypes.h.
   */
  
  #define INT8_MAX        (127)
  #define INT16_MAX       (32767)
  #define INT32_MAX       (2147483647)
  #define UINT8_MAX       (255U)
  #define UINT16_MAX      (65535U)
  #define UINT32_MAX      (4294967295U)

  #ifdef __GNUC__
  #define INT64_MAX       (9223372036854775807)
  #define UINT64_MAX      (18446744073709551615U)
  #endif

  #ifdef INT64_MAX
  #define INTMAX_MAX      INT64_MAX
  #else
  #define INTMAX_MAX      INT32_MAX
  #endif

  #ifdef UINT64_MAX
  #define UINTMAX_MAX     UINT64_MAX
  #else
  #define UINTMAX_MAX     UINT32_MAX
  #endif

  #define INT_LEAST8_MAX  INT8_MAX
  #define INT_LEAST16_MAX INT16_MAX
  #define INT_LEAST32_MAX INT32_MAX
  #ifdef INT64_MAX
  #define INT_LEAST64_MAX INT64_MAX
  #endif

  /*
   * The following four defines are currently missing from the standard and
   * that is assumed to be an oversight.
   */
  #define UINT_LEAST8_MAX UINT8_MAX
  #define UINT_LEAST16_MAX UINT16_MAX
  #define UINT_LEAST32_MAX UINT32_MAX
  #ifdef UINT64_MAX
  #define UINT_LEAST64_MAX UINT64_MAX
  #endif

  /*
   * The following 2 macros are provided for testing whether the types
   * intptr_t and uintptr_t (integers large enough to hold a void *) are
   * defined in this header. They are needed in case the architecture can't
   * represent a pointer in any standard integral type.
   */
  #define INTPTR_MAX
  #define UINTPTR_MAX

  #define INT8_MIN        (-128)
  #define INT16_MIN       (-32767-1)
  #define INT32_MIN       (-2147483647-1)
  #ifdef __GNUC__
  #define INT64_MIN       (-9223372036854775807-1)
  #endif

  #ifdef INT64_MIN
  #define INTMAX_MIN      INT64_MIN
  #else
  #define INTMAX_MIN      INT32_MIN
  #endif

  #define INT_LEAST8_MIN  INT8_MIN
  #define INT_LEAST16_MIN INT16_MIN
  #define INT_LEAST32_MIN INT32_MIN
  #ifdef INT64_MIN
  #define INT_LEAST64_MIN INT64_MIN
  #endif


  typedef unsigned char uint8_t;

  typedef unsigned short uint16_t;

  typedef unsigned int uint32_t;

  typedef unsigned long long uint64_t;
  
  typedef signed char int_least8_t;
  typedef unsigned char uint_least8_t;
  
  typedef short int int_least16_t;
  typedef unsigned short int uint_least16_t;
  
  typedef int int_least32_t;
  typedef unsigned int uint_least32_t;

  typedef long long int_least64_t;
  typedef unsigned long long uint_least64_t;

  typedef int64_t intmax_t;
  typedef uint64_t uintmax_t;


  #ifdef __cplusplus
    }
  #endif /* __cplusplus */

#elif defined(__APPLE__)

	/* Mac OS X -- gcc */
	
#	include <sys/types.h>
#	include <limits.h>
    
#	define LLONG_MIN			LONG_LONG_MIN
#	define LLONG_MAX			LONG_LONG_MAX
#	define ULLONG_MAX			ULONG_LONG_MAX

	typedef u_int8_t			uint8_t;
	typedef u_int16_t			uint16_t;
	typedef u_int32_t			uint32_t;
#	if defined (__GNU_LIBRARY__) ? defined (__USE_GNU) : !defined (__STRICT_ANSI__)
	typedef u_int64_t			uint64_t;
#	endif
	typedef int8_t				int_least8_t;
	typedef uint8_t				uint_least8_t;
	typedef int16_t				int_least16_t;
	typedef uint16_t			uint_least16_t;
	typedef int32_t				int_least32_t;
	typedef uint32_t			uint_least32_t;
#	if defined (__GNU_LIBRARY__) ? defined (__USE_GNU) : !defined (__STRICT_ANSI__)
	typedef int64_t				int_least64_t;
	typedef uint64_t			uint_least64_t;
#	endif
#	if defined (__GNU_LIBRARY__) ? defined (__USE_GNU) : !defined (__STRICT_ANSI__)
	typedef int64_t				intmax_t;
	typedef uint64_t			uintmax_t;
#	else
	typedef int32_t				intmax_t;
	typedef uint32_t			uintmax_t;
#	endif
#	define INT8_MAX				SCHAR_MAX
#	define INT16_MAX			SHRT_MAX
#	define INT32_MAX			LONG_MAX
#	define UINT8_MAX			UCHAR_MAX
#	define UINT16_MAX			USHRT_MAX
#	define UINT32_MAX			ULONG_MAX
#	define INT64_MAX			LLONG_MAX
#	define UINT64_MAX			ULLONG_MAX
#	define INTMAX_MAX			LLONG_MAX
#	define UINTMAX_MAX			ULLONG_MAX
#	define INT_LEAST8_MAX		SCHAR_MAX
#	define INT_LEAST16_MAX		SHRT_MAX
#	define INT_LEAST32_MAX		LONG_MAX
#	define INT_LEAST64_MAX		LLONG_MAX
#	define UINT_LEAST8_MAX		UCHAR_MAX
#	define UINT_LEAST16_MAX		USHRT_MAX
#	define UINT_LEAST32_MAX		ULONG_MAX
#	define UINT_LEAST64_MAX		ULLONG_MAX
#	define INTPTR_MIN			LONG_MIN
#	define INTPTR_MAX			LONG_MAX
#	define UINTPTR_MAX			ULONG_MAX
#	define INT8_MIN				SCHAR_MIN
#	define INT16_MIN			SHRT_MIN
#	define INT32_MIN			LONG_MIN
#	define INT64_MIN			LLONG_MIN
#	define INTMAX_MIN			LLONG_MIN
#	define INT_LEAST8_MIN		SCHAR_MIN
#	define INT_LEAST16_MIN		SHRT_MIN
#	define INT_LEAST32_MIN		LONG_MIN
#	define INT_LEAST64_MIN		LLONG_MIN

#else

	#error "unknown platform!"
	
#endif	/* __linux, WIN32, etc. */


#ifdef __cplusplus
	extern "C" {
#endif

/* for some reason, both uint64_t and UINT64_T are used in the code */
typedef uint64_t UINT64_T;

#ifdef __cplusplus
	}
#endif


#if !HAVE_OPERATORS

#define not				!
#define and				&&
#define or				||
#define bitand			&
#define bitor			|

#endif


#endif /* _ZKINTTYPES_H */
