/*
 * zkrev.h by Nicolas Roy
 *
 * The purpose of this file is to create a persistant symbol into
 * the object file that will represent the RCS revision ID of the file
 *
 * Here is an example of how to use it.
 *
 * i.e:  At the top of my source file
 * 
 * #define   REV_ID "@(#) $Id: zkrev.h,v 1.2 2001/04/19 17:34:58 alainc Exp $"
 * #include <zkrcs.h>
 *
 * Then use the command ident to dump the revision id:
 *
 * i.e: ident mybin
 *
 */
#ifndef ZKREVISION_ID_H
# define ZKREVISION_ID_H

# if defined( REV_ID )
	
/* define the persistant symbol for each platform */
#  if defined( WIN32 )
#   pragma optimize("", off)  /* turn off optimization */
#   pragma warning( push )
#   pragma warning( disable : 4101 ) /* remove unuse warninig */
static const char * __revision_id = REV_ID;
#   pragma warning( pop )     /* recover warning state */
#   pragma optimize("", on)	  /* recover original optimization state */
	
#  elif defined( ZK_LINUX )
static const __attribute((unused)) char *__revision_id = REV_ID;
	
#  else
static const char *__revision_id = REV_ID;
#  endif

#  undef REV_ID
# endif /* REV_ID */

#endif /* ZKREVISION_ID_H */

