/* $Id: zks.h,v 1.72 2001/06/06 19:11:23 thierry Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKS_H_
#define ZKS_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#include <stddef.h>
#include <stdarg.h>

/*======================= COMMON DEFINES ========================*/

#ifdef ZK_DMALLOC
  #include "/usr/include/dmalloc.h"
#endif

/*
 *  These macros support the Zero-Knowledge coding guidelines.  They are used
 *  to indicate how function parameters are used.
 */

#define ZK_IN
#define ZK_INOUT
#define ZK_OUT

typedef enum ConsSeverity { 
  CONS_VERY_IMPORTANT	=0, /* Perhaps will provoke a beep, message box or other strong indication */
  CONS_IMPORTANT	=1, /* Appears on the main UI */
  CONS_NORMAL		=2, /* Appears in the console only */
  CONS_UI_NOT_CONSOLE   =3, /* A transient message which appears on status
                               bar but does not pollute the console 
                               used to update or build a message on the screen */
  CONS_VERBOSE		=4, /* Once we have a VERBOSE preference, these message
                               will only appears when it is set */
  CONS_DEBUG		=5  /* Appears only in internal version */
} ConsSeverity;


#if defined(WIN32)
/*======================= WIN 32 Specific =======================*/
#ifdef __cplusplus

#ifndef _CPPUNWIND
#error("Error, you need to enable exception handling for Windows!")
#endif

#ifndef _CPPRTTI
#error("Error, you need to enable RTTI for Windows!")
#endif

#endif /* __cplusplus */

#define ZK_LITTLE_ENDIAN	1
#define ZK_CLIENT			1		/* we are building a client */

#include <winsock2.h>
#include <windows.h>
#include <process.h>


/* These macros make using 
 #pragma message(Reminder "whatever")
 fun to use under MSVC
*/
#define Stringize( L )		#L
#define MakeString( M, L )	M(L)
#define $LINE	MakeString(Stringize, __LINE__)
#define Reminder __FILE__	"("$LINE") : Reminder: "

#ifndef __cplusplus /* This is a keyword in C++, please abstain from using it */
	typedef BOOL bool;
#endif

#ifdef _WINDLL
#define DLLExport __declspec(dllexport)
#else
#define DLLExport __declspec(dllimport)
#endif

#define DLLImport __declspec(dllimport)



#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Alot of the Unix DLLs print to stderr. This function can be used inside those DLLs, which aren't using MFC,
 * to trace to the debug window.
 */ 
DLLExport void ClientTrace(const char*, const char*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#elif defined(macintosh)
/*======================= Macintosh Specific ===============================*/

#define ZK_BIG_ENDIAN	1
#define ZK_CLIENT		1		/* we are building a client */

#ifndef DLLExport
# define DLLExport	__declspec(export)
#endif

#ifndef DLLImport
# define DLLImport __declspec(import)
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

DLLExport void	ClientTrace(const char*, const char*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#define U_LONG_LONG_FMT	"%llu"
#define LONG_LONG_FMT	"%lld"
/* The following is to support an old typo. */
#define LONG_LONG_FTM LONG_LONG_FMT
#define LONG_LONG		long long

#elif defined(__APPLE__)
/*======================= Mac OS X Specific ===============================*/

#define ZK_BIG_ENDIAN	1
#define ZK_CLIENT		1		/* we are building a client */

#define DLLExport
#define DLLImport

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

extern void	ClientTrace(const char*, const char*);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#define U_LONG_LONG_FMT	"%llu"
#define LONG_LONG_FMT	"%lld"
/* The following is to support an old typo. */
#define LONG_LONG_FTM LONG_LONG_FMT
#define LONG_LONG		long long

#else /* WIN32 */
/*======================= Unix Specific ===============================*/
#define DLLExport
#define DLLImport
#define ClientTrace(a,b)
#define __TIMESTAMP__ __TIME__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#define U_LONG_LONG_FMT	"%Lu"
#define LONG_LONG_FMT	"%Ld"
/* The following is to support an old typo. */
#define LONG_LONG_FTM LONG_LONG_FMT
#define LONG_LONG		long long

#endif /* ! WIN32 */

/*======================= WIN 32 And Unix Stuff =======================*/

#define FR_WINDOWSCLIENT_VERSION "0.5"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
