#ifndef _czkEventHandler

#define _czkEventHandler

#include "czkEventHandlerData.h"
#include "czkSocket.h"

// Virtual class to handle I/O events
//
// this class should NOT be instantiated. the user should create
// his own descendant class from this one, and provide 
// concrete implementations of the handleXXXX methods.
//
// Methods:
//
//      handleTimeout(userData) - called when a timeout event occurs
//      NOTE that this method must return an int32_t value:
//      0 - do not reschedule  
//      1 - automatically reschedule  
//
//      handleInput( sock, userData) - called when the socket has 
//                                       pending input
//
//      handleOutput( sock, userData) - called when the socket has 
//                                        is writeable
//
//      handleException( sock, userData) - called when the socket has 
//                                           has an error condition
//
//      handleSignal( signum, userData) - called when a registered signal  
//                                         has been raised
//
// Notes:
//
// designed for use in conjunction with the czkCoreLoop class
//
// see czkCoreLoop.c for more info
//
class czkEventHandler {

 private:
 
 public:

   czkEventHandler() {};
   virtual ~czkEventHandler() {};
  
   virtual int32_t  handleTimeout(czkEventHandlerData* userData) = 0;
                     
   virtual void     handleInput(czkSocket* sock, 
                                czkEventHandlerData* userData) = 0;
                     
   virtual void     handleOutput(czkSocket* sock, 
                                 czkEventHandlerData* userData) = 0;
                   
   virtual void     handleException(czkSocket* sock, 
                                    czkEventHandlerData* userData) = 0;
                   
   virtual void     handleSignal(int signum, 
                                 czkEventHandlerData* userData) 
                      { (void)signum; (void)userData; assert(0); };
                    
};

#endif



