#ifndef _czkFile

#define _czkFile

#include "czknspr.h"

// Wrapper class for NSPR's file i/o routines
//
class czkFile {

 private:

   ZKPRFileDesc* fd;

 public:

   // constructor
   czkFile();
  
   // destructor
   virtual ~czkFile();

   // delete an existing file
   int32_t deleteFile(char*);  // full path name of the file

   // rename an existing file
   int32_t renameFile(char*,   // full path name of the file to rename
                      char*);  // full path name of the new file

   // return file info from an opened file
   int32_t getFileInfo(ZKPRFileInfo*); //fileinfo returned - see PR_GetFileInfo

   // return file info for specified file
   int32_t getFileInfo(ZKPRFileInfo*, // fileinfo returned - see PR_GetFileInfo
                       char*);        // full path name of file

   // return file info for file associated with specified file descriptor
   int32_t getFileInfo(ZKPRFileInfo*, // fileinfo returned - see PR_GetFileInfo
                       ZKPRFileDesc*);// file descriptor

   // return true if file can be accessed in specified manner
   int32_t getFileAccess( char* name,      // full path name of file
                          ZKPRAccessHow);  // access manner - see PR_Access

   // open a file for I/O
   int32_t open(const char *,  // full path name of file
                int32_t,       // access flags - see PR_Open
                int32_t);      // access mode  - see PR_Open

   // close the file opened for I/O
   int32_t close();
 
   // read n bytes from the open file - returns number of bytes read
   int32_t read( char* buf, int32_t n);

   // write n bytes to the open file - returns number of bytes written
   int32_t write( char* buf, int32_t n);

   // returns number of bytes available for reading
   int32_t getAvailable();

   // flushes the buffers of the opened file
   int32_t sync();
};

#endif


