#ifndef _czkIOEvent
 
#define _czkIOEvent

#include <sys/time.h>
#include "czkSocket.h"

// Class to encapsulate I/O events for czkCoreLoop
//
// each object contains a socket, eventhandler, data & mask
//
// Methods:
//
// czkIOEvent( socketin, ehin, datain, maskin) - constructor
//
// getSocket() - returns the socket
// setSocket(socketin) - sets the socket
//
// getData() - returns the user data
// setData(datain) - sets the user data
//
// getMask() - returns the interest mask
// setMask(maskin) - sets the interest mask
//
// getEventHandler() - returns the event handler
// setEventHandler(ehin) - sets the event handler
//
// Notes:
//
// an IOEvent object is instantiated whenever a socket is registered
// with czkCoreLoop
//
// see czkCoreLoop.c for more info
//
class czkIOEvent {

 private:

   int32_t mask;              // interest mask
   czkEventHandler* eh;       // pointer to eventhandler object
   czkSocket* socket;         // pointer to socket object
   czkEventHandlerData* data; // pointer to client-specific data

 public:

   int32_t nb;

   // default constructor
   czkIOEvent() {}; 

   // constructor specifying all parameters
   czkIOEvent( czkSocket* socketin, czkEventHandler* ehin, 
               czkEventHandlerData* datain, int32_t maskin) 
      { socket = socketin; eh = ehin; data = datain; mask = maskin; }

   // destructor
   virtual ~czkIOEvent() {};
 
   // return pointer to socket object
   czkSocket* getSocket() { return socket; };

   // set socket object
   void setSocket(czkSocket* socketin) { socket = socketin; };

   // return user-specific data
   czkEventHandlerData* getData() { return data; };

   // set user-specific data 
   void setData(czkEventHandlerData* datain) { data = datain; }; 

   // set interest mask
   void setMask( int32_t maskin) { mask = maskin; }

   // return interest mask
   int32_t getMask() { return mask; }

   // set eventhandler object
   void setEventHandler( czkEventHandler* ehin) { eh = ehin; }

   // return eventhandler object
   czkEventHandler* getEventHandler() { return eh; }
};


#endif


