#ifndef _czkLock

#define _czkLock

#include <assert.h>
#include "czknspr.h"

// Wrapper class for NSPR's mutual exclusion lock
//    (to perform synchronization between threads of the same process)
//
// Methods:
//
//      open    - creates the mutex context
//
//      close   - deletes the mutex context
//
//      acquire - acquires the lock
//
//      release - release the lock
//
//      getLockid - return handle to lock (ZKPRLock*) 
//

class czkLock {

 private:

   ZKPRLock* lockid;  // lock context

 public:

   // constructor
   czkLock();

   // destructor
   virtual ~czkLock();

   // open (initialize) the lock context
   int32_t open();

   // acquire the lock
   void acquire();

   // release the lock
   int32_t release();        // return value is status code

   // close (free) the lock context
   void close();

   // return the lock context to inquiring minds.
   ZKPRLock* getLockid();  // return value is lock handle (ZKPRLock*)

};

#endif


