#ifndef _czkNetAddr

#define _czkNetAddr

#include <stdlib.h>

#include "czknspr.h"

// Wrapper class for NSPR's netwrok address translation routines
//
class czkNetAddr {

 private:

   char* hostentrybuf;  //temp storage for getHostbyName & getHostbyAddr
   ZKPRHostEnt hostent;   //temp storage for getHostbyName & getHostbyAddr
 
 public:

   // constructor
   czkNetAddr();

   // destructor
   virtual ~czkNetAddr();

   // return initialized ZKPRNetAddr structure - see PR_InitializeNetAddr 
   int32_t initNetAddr(int32_t val, uint16_t port, ZKPRNetAddr *addr);

   // convert a qot-quad address string into ZKPRNetAddr format
   int32_t getNetAddr(const char*, ZKPRNetAddr*);

   // get HostEnt info from a host name
   int32_t getHostbyName(char*, int32_t);

   // het HostEnt info from a net address
   int32_t getHostbyAddr(ZKPRNetAddr*, int32_t);
 
   // return 'n'th net addr from a previous call to 
   //     getHostbyName or getHostbyAddr 
   // see PR_EnumerateHostEnt
   int32_t enumerateHostEnt( int32_t n, uint16_t port, ZKPRNetAddr*);

   // free memory allocated for HostEnt
   void freeHostEnt();

   // convert from network to host byte order (short)
   static uint16_t NtoHs( uint16_t);

   // convert from network to host byte order (long)
   static uint32_t NtoHl( uint32_t);

   // convert from host to network byte order (short)
   static uint16_t HtoNs( uint16_t);

   // convert from host to network byte order (long)
   static uint32_t HtoNl( uint32_t);
};

#endif


