#ifndef _czkPoll

#define _czkPoll

#include <sys/time.h>
#include "czknspr.h"
#include "czkEventHandler.h"
#include "czkIOEvent.h"

// Class to manage polling of a set of socket & mask pairs
//
// contains a PollDesc array (for the poll call)
// and an associated IOEvent array to track the socket and eventhandler pairs.
//
// Methods:
//
// addFD( ioevent) - adds a file descriptor and mask to the PollDesc array
//                   and addes the ioevent to the ioev array
//
// removeFD( socket) - removes the associated entries from the PollDesc and
//                     ioev arrays
//
// poll(timeout)  - performs the poll  
//
// getEventFD( socket, mask, ehout, clientData) 
//                - returns the next socket, mask, eventhandler, and data
//                  set that has an event pending (as determined by the last
//                  poll call) - designed to be called repeatedly
// 

#define MAXPOLLFD 1000

class czkPoll {

 private:
 
   ZKPRPollDesc* pd;  // poll descriptor array

   czkIOEvent** ioev; // IOEvent array (index will always match pd array)
 
   int32_t npd;         // no of descriptors in pd array

   int32_t curpd;       // current index in pd array (for getEventFD)

 public:

   //constructor
   czkPoll();
   
   //destructor
   virtual ~czkPoll();

   // add an FD and IOEvent to the arrays
   int32_t addFD(czkIOEvent* ioevent);

   // remove an FD and IOEvent (that sock belongs to) from the arrays
   int32_t removeFD(czkSocket* sock);

   // do the poll - returns no. of events to process
   int32_t poll(ZKPRIntervalTime timeout);

   // returns the next socket with an event
   int32_t getEventFD(czkSocket** fd, int32_t* mask, czkEventHandler **eh,
                      czkEventHandlerData** clientData);
};

#endif



