#ifndef _czkScopeLock

#define _czkScopeLock

#include "czkLock.h"

// Class to enable "safe" handling of a czkLock object.
// It will automatically release an acquired lock if the object is deleted
// (e.g. goes out of scope)
//

class czkScopeLock {

 private:

   czkLock* lock;  // pointer to the lock object 
   bool     acquired;

 public:

   // constructor
   czkScopeLock(czkLock* lockin) 
   { 
     lock = lockin;
     acquired = false; 
   };

   // destructor
   virtual ~czkScopeLock() 
   { 
      if (acquired) {
         lock->release(); 
      }
   };

  //copy constructor (disabled)
  czkScopeLock( const czkScopeLock&) { assert(0); };

  //assignment operator (disabled)
  czkScopeLock& operator=( const czkScopeLock&) { assert(0); };

   // acquire the lock
   void acquire() 
   { 
      assert(!acquired); lock->acquire(); acquired = true; 
   } ;

   // release the lock
   void release() 
   { 
      assert(acquired); acquired = false; lock->release(); 
   };      

};

#endif


