#ifndef _czkSocket

#define _czkSocket

#include "nspr.h"
#include "prpriv.h"

#include <assert.h> 
#include <stdlib.h>
#include <string.h>

#include "czknspr.h"

// Wrapper class for NSPR's socket handling
//
// Methods:
//
//      openTCPSocket()   - creates a TCP socket
//
//      openUDPSocket()   - creates a UDP socket
//
//      setHandle( fd)    - sets the object's file descriptor
//
//      getHandle()       - returns the object's file descriptor
//
//      setOption( opt)   - sets a socket option
//
//      bind( addr)       - binds the socket to an address
//
//      listen( backlog)  - listens on the socket for incoming requests
//
//      connect( address, timeout) - connects to the specified address
//
//      accept( newsocket, addrfrom, timeout) - accepts a remote connection
//
//      send( buf, nb, flags, timeout) - send data on a connected socket
//
//      sendto( buf, nb, flags, addr, timeout) - send data to the 
//                                               specified address
//
//      recv( buf, nb, flags, timeout) - receive data on a connected socket
//
//      recvfrom( buf, nb, flags, addr, timeout) - receive data on an 
//                                                 unconnected socket
//
//      close()          - close and free the socket
//
class czkSocket {

private:
  ZKPRFileDesc *sock;  // the file descriptor

public:

  // constructor
  czkSocket() { };

  // destructor
  virtual ~czkSocket() { };
 
  // open a TCP socket
  int32_t openTCPSocket(); 

  // open a UDP socket
  int32_t openUDPSocket(); 

  // set the file descriptor
  void setHandle( ZKPRFileDesc* fd) { assert(fd); sock = fd; }
 
  // return the file descriptor 
  ZKPRFileDesc* getHandle() { assert( sock); return sock; }
  
  // an evil method to return the native file descriptor
  int32_t getNativeFD() { assert(sock); return PR_FileDesc2NativeHandle(sock); }

  // get a socket option
  int32_t getOption(ZKPRSocketOptionData* opt); 

  // set a socket option
  int32_t setOption(ZKPRSocketOptionData* opt); 

  // bind the socket to the specified address
  int32_t bind(ZKPRNetAddr addr);

  // set the socket to listen for incoming connections
  int32_t listen(int32_t backlog);

  // connect to a remote socket
  int32_t connect( ZKPRNetAddr, ZKPRIntervalTime);

  // return the address of the socket
  int32_t getSockName( ZKPRNetAddr* addr);

  //return the address of the connected peer host
  int32_t getPeerName( ZKPRNetAddr* addr);

  // accept an incoming connection
  int32_t accept( czkSocket** newSocket, ZKPRNetAddr* addrfrom, 
                  ZKPRIntervalTime timeout);

  // send data on a connected socket
  int32_t send( char* buf, int32_t buflen, int32_t flags, 
                ZKPRIntervalTime timeout);

  // recv data on a connected socket
  int32_t recv( char* buf, int32_t buflen, int32_t flags, 
                ZKPRIntervalTime timeout);

  // send data on an unconnected socket 
  int32_t sendto( char* buf, int32_t buflen, int32_t flags, 
                  const ZKPRNetAddr* addr, ZKPRIntervalTime timeout);

  // recv data on an unconnected socket
  int32_t recvfrom( char* buf, int32_t buflen, int32_t flags,  
                    ZKPRNetAddr* addr, ZKPRIntervalTime timeout);

  // close the socket
  int32_t close(); 
};

#endif


