#ifndef _czkTimer

#define _czkTimer

#include <sys/time.h>

#include "czknspr.h"
#include "czkEventHandler.h"

class czkTimerProxy;

class czkTimer {

  friend class czkCoreLoop;
  friend class czkTimerProxy;

 private:
 
   czkTimerProxy* pTimerProxy;  // pointer to the associated TimerProxy object

   ZKPRIntervalTime start;      // start time of timer
   ZKPRIntervalTime delta;      // delta time for when it times out
   ZKPRIntervalTime finish;     // end time of timer
   czkEventHandler* eh;         // associated event handler
   czkEventHandlerData* data;   // data passed to event handler method
   bool cancelled;

 public:

   // constructor
   czkTimer(ZKPRIntervalTime deltain, czkEventHandler*, czkEventHandlerData*);

   // destructor
   virtual ~czkTimer();

   // cancel the timer
   void cancel();

   // return if the timer was cancelled
   bool isCancelled();

   // overload <
   bool operator<(const czkTimer&);

   // return true if timer has timed out
   bool isTimedOut();

   // call eventhandler method
   int32_t  doevent();

   // reset start time to now
   void reset();
  
   // set the eventhandler
   void setEventHandler( czkEventHandler* ehin) { eh = ehin; }

   // return the eventhandler
   czkEventHandler* getEventHandler() { return eh; }
 
   // get finish time
   ZKPRIntervalTime getFinish(); 

   // get time remaining
   ZKPRIntervalTime getTimeRemaining() 
   { 
     ZKPRIntervalTime now = PR_IntervalNow();

     return (ZKPRIntervalTime)(finish - now);
   }

};

#endif


