//
// the czkTimerProxy class is an interface class to enable safe 
// management of timers as they are registered and delete by czkCoreLoop
//
// when a timer is scheduled, a czkTimerProxy object is created, and
// its pointer is passed back to the caller.   
//
// access to this czkTimerProxy object allows the caller a means to 
// cancel the timer (via method "cancelTimer")
//
// the caller must delete the czkTimerProxy object that is returned
// by czkCoreLoop::scheduleTimer.
//

#ifndef _czkTimerProxy

#define _czkTimerProxy

#include <sys/time.h>

#include "czknspr.h"
#include "czkEventHandlerData.h"
#include "czkTimer.h"

class czkTimerProxy {

 friend class czkCoreLoop;

 private:
 
   czkTimer* pTimer;   // pointer to the associated timer object

 public:

   // constructor
   czkTimerProxy(czkTimer* timer) { pTimer = timer; };

   // destructor
   virtual ~czkTimerProxy() 
   {
     // if the associated timer object still exists, 
     // NULL out its pointer to this object (cause it's going away)
     if (pTimer) {
       pTimer->pTimerProxy = NULL;
     }
   };

   // cancel Timer (only if it still exists)
   czkEventHandlerData* cancelTimer() 
   { 
     czkEventHandlerData* ret = NULL;
     if (pTimer) {
       pTimer->cancel();
       ret = pTimer->data;
     }
     return ret;
   };

};

#endif


