/* 
   set of definitions mapping zk to PR data types & structures
*/

#include "nspr.h"
#include "prpriv.h"
#include "prio.h"
#include "prcvar.h"
#include "prerror.h"
#include "prlock.h"
#include "prnetdb.h"
#include "plgetopt.h"  
#include "prsystem.h"
#include "prenv.h"
#include "prthread.h"

#include "zks.h"
#include "zkDefs.h"
#include "zkIntTypes.h"
#include "zkIntLimits.h"
#include "zkTypes.h"
#include "zkserr.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkslogagent.h"

typedef PRIntervalTime     ZKPRIntervalTime;
typedef PRCondVar          ZKPRCondVar;
typedef PRStatus           ZKPRStatus;
typedef PRDir              ZKPRDir;
typedef PRDirEntry         ZKPRDirEntry;
typedef PRDirFlags         ZKPRDirFlags;
typedef PRFileDesc         ZKPRFileDesc;
typedef PRFileInfo         ZKPRFileInfo;
typedef PRAccessHow        ZKPRAccessHow;
typedef PRLock             ZKPRLock;
typedef PRHostEnt          ZKPRHostEnt;
typedef PRNetAddrValue     ZKPRNetAddrValue;
typedef PRNetAddr          ZKPRNetAddr;
typedef PRPollDesc         ZKPRPollDesc;
typedef PRSocketOptionData ZKPRSocketOptionData;
typedef PRThread           ZKPRThread;
typedef PRThreadScope      ZKPRThreadScope;
typedef PRThreadState      ZKPRThreadState;
typedef PRThreadPriority   ZKPRThreadPriority;

#define ZKPR_CALLBACK            PR_CALLBACK       
 
#define ZKPR_POLL_EXCEPT         PR_POLL_EXCEPT
#define ZKPR_POLL_ERR            PR_POLL_ERR
#define ZKPR_POLL_NVAL           PR_POLL_NVAL
#define ZKPR_POLL_READ           PR_POLL_READ
#define ZKPR_POLL_WRITE          PR_POLL_WRITE

#define ZKPR_INTERVAL_NO_TIMEOUT PR_INTERVAL_NO_TIMEOUT
#define ZKPR_INTERVAL_NO_WAIT    PR_INTERVAL_NO_WAIT

#define ZKPR_TRUE                PR_TRUE
#define ZKPR_FALSE               PR_FALSE

#define ZKPR_FAILURE             PR_FAILURE
#define ZKPR_SUCCESS             PR_SUCCESS

#define ZKPR_SI_HOSTNAME         PR_SI_HOSTNAME
#define ZKPR_SI_SYSNAME          PR_SI_SYSNAME
#define ZKPR_SI_RELEASE          PR_SI_RELEASE
#define ZKPR_SI_ARCHITECTURE     PR_SI_ARCHITECTURE

#define ZKPR_USER_THREAD         PR_USER_THREAD 
#define ZKPR_LOCAL_THREAD        PR_LOCAL_THREAD 
#define ZKPR_GLOBAL_THREAD       PR_GLOBAL_THREAD
#define ZKPR_JOINABLE_THREAD     PR_JOINABLE_THREAD 
#define ZKPR_UNJOINABLE_THREAD   PR_UNJOINABLE_THREAD

#define ZKPR_AF_INET             PR_AF_INET
#define ZKPR_IpAddrNull          PR_IpAddrNull
#define ZKPR_IpAddrAny           PR_IpAddrAny
#define ZKPR_IpAddrLoopback      PR_IpAddrLoopback
#define ZKPR_NETDB_BUF_SIZE      PR_NETDB_BUF_SIZE

#define ZKPR_SockOpt_Nonblocking     PR_SockOpt_Nonblocking
#define ZKPR_SockOpt_Linger          PR_SockOpt_Linger
#define ZKPR_SockOpt_Reuseaddr       PR_SockOpt_Reuseaddr
#define ZKPR_SockOpt_Keepalive       PR_SockOpt_Keepalive
#define ZKPR_SockOpt_RecvBufferSize  PR_SockOpt_RecvBufferSize
#define ZKPR_SockOpt_SendBufferSize  PR_SockOpt_SendBufferSize
#define ZKPR_SockOpt_IpTimeToLive    PR_SockOpt_IpTimeToLive
#define ZKPR_SockOpt_IpTypeOfService PR_SockOpt_IpTypeOfService
#define ZKPR_SockOpt_AddMember       PR_SockOpt_AddMember
#define ZKPR_SockOpt_DropMember      PR_SockOpt_DropMember
#define ZKPR_SockOpt_McastInterface  PR_SockOpt_McastInterface
#define ZKPR_SockOpt_McastTimeToLive PR_SockOpt_McastTimeToLive
#define ZKPR_SockOpt_McastLoopback   PR_SockOpt_McastLoopback
#define ZKPR_SockOpt_NoDelay         PR_SockOpt_NoDelay
#define ZKPR_SockOpt_MaxSegment      PR_SockOpt_MaxSegment

#define ZKPR_ACCESS_EXISTS           PR_ACCESS_EXISTS
#define ZKPR_ACCESS_WRITE_OK         PR_ACCESS_WRITE_OK 
#define ZKPR_ACCESS_READ_OK          PR_ACCESS_READ_OK 

#define ZKPR_RDONLY                  PR_RDONLY
#define ZKPR_WRONLY                  PR_WRONLY
#define ZKPR_RDWR                    PR_RDWR
#define ZKPR_CREATE_FILE             PR_CREATE_FILE
#define ZKPR_APPEND                  PR_APPEND
#define ZKPR_TRUNCATE                PR_TRUNCATE

#define ZKPR_SKIP_NONE               PR_SKIP_NONE
#define ZKPR_SKIP_DOT                PR_SKIP_DOT
#define ZKPR_SKIP_DOT_DOT            PR_SKIP_DOT_DOT
#define ZKPR_SKIP_BOTH               PR_SKIP_BOTH
#define ZKPR_SKIP_HIDDEN             PR_SKIP_HIDDEN

