#include "czkCondition.h"

// constructor
czkCondition::czkCondition()
{
  condid = NULL;
}

// destructor
czkCondition::~czkCondition()
{
}

// open (initialize CondVar) - "lock" argument must already be open
int32_t czkCondition::open(czkLock& condlock)
{
  int32_t status;

  condid = PR_NewCondVar( condlock.getLockid());
  if (condid == NULL) {
    status = ZKPR_ErrorConditionOpen;
  }
  else {
    status = ZKPR_SUCCESS;
  }
  return status;
}

// close (destroy) CondVar
void czkCondition::close()
{
  assert( condid);
  PR_DestroyCondVar( condid);
}

// wait for condition to be signalled (with timeout)
int32_t czkCondition::wait(ZKPRIntervalTime timeout)
{
  assert( condid);
  PRStatus status = PR_WaitCondVar( condid, timeout);
  return PRtoZKStatus(status);
}

// wait for condition to be signalled (no timeout)
int32_t czkCondition::wait()
{
  assert( condid);
  PRStatus status = PR_WaitCondVar( condid, ZKPR_INTERVAL_NO_TIMEOUT);
  return PRtoZKStatus(status);
}

// signal condition to another thread
int32_t czkCondition::signal()
{
  assert( condid);
  PRStatus status = PR_NotifyCondVar( condid);
  return PRtoZKStatus(status);
}

// broadcast signal to all waiting threads
int32_t czkCondition::broadcast()
{
  assert( condid);
  PRStatus status = PR_NotifyAllCondVar( condid);
  return PRtoZKStatus(status);
}

// return handle to CondVar
ZKPRCondVar* czkCondition::getCondid()
{
  assert( condid);
  return condid;
}


