#include "czkNetAddr.h"

// constructor
czkNetAddr::czkNetAddr()
{
  hostentrybuf = NULL;
}

//destructor 
czkNetAddr::~czkNetAddr()
{
}

//initialize a ZKPRNetAddr record
int32_t czkNetAddr::initNetAddr(int32_t val, uint16_t port, ZKPRNetAddr *addr)
{
  ZKPRStatus status = PR_InitializeNetAddr( (PRNetAddrValue)val, port, addr);
  return PRtoZKStatus(status);
} 
 
// convert a dot-quad address string to ZKPRNetAddr format
int32_t czkNetAddr::getNetAddr(const char *string, ZKPRNetAddr* netaddr)
{
  ZKPRStatus status = PR_StringToNetAddr( string, netaddr);
  return PRtoZKStatus(status);
}

// get hostent from a host name
int32_t czkNetAddr::getHostbyName(char* name, int32_t bufsize)
{
  if (hostentrybuf) {
    delete hostentrybuf;
  }
  hostentrybuf = new char[bufsize];
  ZKPRStatus status = PR_GetHostByName( name, hostentrybuf, bufsize, &hostent);
  return PRtoZKStatus(status);
}

// get hostent from a host address
int32_t czkNetAddr::getHostbyAddr(ZKPRNetAddr* netaddr, int32_t bufsize)
{
  if (hostentrybuf) {
    delete hostentrybuf;
  }
  hostentrybuf = new char[bufsize];
  ZKPRStatus status = PR_GetHostByAddr( netaddr, hostentrybuf, bufsize, 
                                      &hostent);
  return PRtoZKStatus(status);
}

// get the 'ind'th address from the previously generated hostent
// (assumes previous call to getHostbyName or getHostbyAddr
int32_t czkNetAddr::enumerateHostEnt( int32_t ind,  
                                  uint16_t port, ZKPRNetAddr* netaddr)
{
  return PR_EnumerateHostEnt( ind, &hostent, port, netaddr);
}

// free memory allocated for HostEnt
void czkNetAddr::freeHostEnt()
{
  if (hostentrybuf) {
    delete hostentrybuf;
  }
}

// convert from network to host byte order (short)
uint16_t czkNetAddr::NtoHs( uint16_t in)
{
  return PR_ntohs( in);
}

// convert from network to host byte order (long)
uint32_t czkNetAddr::NtoHl( uint32_t in)
{
  return PR_ntohl( in);
}

// convert from host to network byte order (short)
uint16_t czkNetAddr::HtoNs( uint16_t in)
{
  return PR_htons( in);
}

// convert from host to network byte order (long)
uint32_t czkNetAddr::HtoNl( uint32_t in)
{
  return PR_htonl( in);
}


