#include "czkPoll.h"

// constructor - initialize and allocate the arrays
czkPoll::czkPoll()
{
  npd = 0;
  curpd = 0;

  pd = NULL;
  ioev = NULL;

  pd = new ZKPRPollDesc[MAXPOLLFD];
  ioev = new czkIOEvent*[MAXPOLLFD];
}

// destructor - free the IOEvents and arrays
czkPoll::~czkPoll()
{
  for (int32_t i = 0; i < npd; i++)
  {
     delete ioev[i];
  }
  delete pd;
  delete ioev;
}

// add an FD and IOEvent to the arrays
int32_t czkPoll::addFD(czkIOEvent* event) 
{
  int32_t rc;

  if (npd < MAXPOLLFD) {
    pd[npd].fd = event->getSocket()->getHandle();
    pd[npd].in_flags = event->getMask();
    ioev[npd] = event;
    npd++;
    rc = ZKPR_SUCCESS;
  } 
  else {
    rc = ZKPR_ErrorAddFD;
  }
  return rc;
}

// remove an FD and IOEvent from the arrays (that belong to sock)
int32_t czkPoll::removeFD(czkSocket* sock)
{
  int32_t rc;
  int32_t i;
  int32_t removed = 0;

  for (i = 0; i < npd; i++)
  {
    if (pd[i].fd == sock->getHandle()) {
      removed = 1;
      delete ioev[i];
      break;
    }
  }
  if (removed) {
    if (i < npd-1) {
      pd[i] = pd[npd-1];
      ioev[i] = ioev[npd-1];
    }
    npd--;
    rc = ZKPR_SUCCESS;
  }
  else {
    rc = ZKPR_ErrorRemoveFD;
  }
  return rc;
}

// do the poll
int32_t czkPoll::poll(ZKPRIntervalTime timeout)
{
   curpd = 0;
   int32_t n = PR_Poll( pd, npd, timeout);
   return n;
}
  
// return the next event info
int32_t czkPoll::getEventFD( czkSocket** sock, int32_t* mask, 
                             czkEventHandler** ehout, 
                             czkEventHandlerData** clientData)
{
  int32_t rc = ZKPR_ErrorGetEventFD;

  int32_t i;
  for (i = curpd; i < npd; i++)
  {
    if (pd[i].out_flags) {
      *sock = ioev[i]->getSocket();
      *mask = pd[i].out_flags;
      *ehout = ioev[i]->getEventHandler();
      *clientData = ioev[i]->getData();
      curpd = i+1;
      rc = ZKPR_SUCCESS;
      break;
    }
  }
  return rc;
}     
 

