#include <czkSocket.h>

// open a TCP socket
int32_t czkSocket::openTCPSocket() 
{
  int32_t rc;
 
  sock = PR_NewTCPSocket();
  if (sock == NULL) {
    rc = ZKPR_ErrorTCPSocketOpen;
  } 
  else {
    rc = ZKPR_SUCCESS;
  }
  return rc;

}; 

// open a UDP socket
int32_t czkSocket::openUDPSocket() 
{
  int32_t rc;
    
  sock = PR_NewUDPSocket();
  if (sock == NULL) {
    rc = ZKPR_ErrorUDPSocketOpen;
  } 
  else {
    rc = ZKPR_SUCCESS;
  }
  return rc;
}; 

// get a socket option
int32_t czkSocket::getOption(ZKPRSocketOptionData* opt) 
{
  assert(sock);
  assert(opt);

  ZKPRStatus status = PR_GetSocketOption( sock, opt);
  return PRtoZKStatus(status);
}

// set a socket option
int32_t czkSocket::setOption(ZKPRSocketOptionData* opt) 
{
  assert(sock);
  assert(opt);

  ZKPRStatus status = PR_SetSocketOption( sock, opt);
  return PRtoZKStatus(status);
}

// accept a new connection; returns a new socket
int32_t czkSocket::accept( czkSocket** newSocket, 
                           ZKPRNetAddr* addrfrom, 
                           ZKPRIntervalTime timeout)
{
  assert(sock);
  assert(newSocket);
  assert(addrfrom);
 
  ZKPRFileDesc *fd2 = PR_Accept( sock, addrfrom, timeout);
  if (fd2) { 
    *newSocket = new czkSocket;
    (*newSocket)->setHandle( fd2);
  }
  return fd2 ? ZKPR_SUCCESS: ZKPR_ErrorSocketAccept;
}    

// binds the socket to the specified address
int32_t czkSocket::bind(ZKPRNetAddr addr)
{
  assert(sock);
  ZKPRStatus status = PR_Bind(sock, &addr);
  return PRtoZKStatus(status);
}

// sets the socket to listen for incoming connection requests
int32_t czkSocket::listen(int32_t backlog)
{
  assert(sock);
  ZKPRStatus status = PR_Listen(sock, backlog);
  return PRtoZKStatus(status);
}

// connect the socket to a remote address
int32_t czkSocket::connect( ZKPRNetAddr addr, 
                            ZKPRIntervalTime timeout)
{
  assert(sock);
  ZKPRStatus status = PR_Connect( sock, &addr, timeout);
  return PRtoZKStatus(status);
}

// return the address of the socket
int32_t czkSocket::getSockName( ZKPRNetAddr* addr)
{
  assert(sock);

  ZKPRStatus status = PR_GetSockName( sock, addr);
  return PRtoZKStatus(status);
}


//return the address of the connected peer host
int32_t czkSocket::getPeerName( ZKPRNetAddr* addr)
{
  assert(sock);

  ZKPRStatus status = PR_GetPeerName( sock, addr);
  return PRtoZKStatus(status);
}

// send data on the socket
int32_t czkSocket::send( char* buf, int32_t buflen, int32_t flags, 
                         ZKPRIntervalTime timeout)
{ 
  assert(sock);
  int32_t n = PR_Send( sock, buf, buflen, flags, timeout); 
  return PRtoZKStatus(n);
}

// recv data on the socket
int32_t czkSocket::recv( char* buf, int32_t buflen, int32_t flags, 
                         ZKPRIntervalTime timeout)
{ 
  assert(sock);
  int32_t n = PR_Recv( sock, buf, buflen, flags, timeout); 
  return PRtoZKStatus(n);
}

// send data to the specified address
int32_t czkSocket::sendto( char* buf, int32_t buflen, int32_t flags, 
                       const ZKPRNetAddr* addr, ZKPRIntervalTime timeout)
{ 
  assert(sock);
  int32_t n = PR_SendTo( sock, buf, buflen, flags, addr, timeout); 
  return PRtoZKStatus(n);
}

// recv data on the socket and return the originating address
int32_t czkSocket::recvfrom( char* buf, int32_t buflen, int32_t flags,  
                             ZKPRNetAddr* addr, ZKPRIntervalTime timeout)
{ 
  assert(sock);
  int32_t n = PR_RecvFrom( sock, buf, buflen, flags, addr, timeout); 
  return PRtoZKStatus(n);
}

// close the socket
int32_t czkSocket::close() 
{ 
  assert(sock);
  ZKPRStatus status = PR_Close(sock);
  return PRtoZKStatus(status); 
};






