#include "czkTimer.h"

// constructor
czkTimer::czkTimer( ZKPRIntervalTime deltain, 
                    czkEventHandler* ehin, czkEventHandlerData* datain)
{
  assert(ehin);

  start = PR_IntervalNow();
  delta = deltain;
  eh = ehin;
  data = datain;

  finish = (ZKPRIntervalTime)(start + delta);

  cancelled = false;

  pTimerProxy = NULL;

}

czkTimer::~czkTimer()
{
}

// cancel the timer
void czkTimer::cancel()
{
  cancelled = true;
}

// return if the timer was cancelled
bool czkTimer::isCancelled()
{
  return cancelled;
}

// reset a timer (set start time to now; recalculate finish time)
void czkTimer::reset()
{
   start = PR_IntervalNow();
   finish = (ZKPRIntervalTime)(start + delta);
}

// less than operator used by priority queue to determine priority
// the smaller the time remaining, the higher the priority
bool czkTimer::operator<( const czkTimer& Timerc)
{
  return ( finish < Timerc.finish);
}

// returns true if timer has completed
bool czkTimer::isTimedOut()
{

   ZKPRIntervalTime now = PR_IntervalNow();
   ZKPRIntervalTime d = (ZKPRIntervalTime)(now - start);

   if (d >= delta) {
     return true;
   }

   return false;
}
  
// call the handle_timeout method of the event handler
int32_t czkTimer::doevent()
{
  return eh->handleTimeout(data);
}     

ZKPRIntervalTime czkTimer::getFinish() 
{ 
  return finish; 
}




