//
// unit test program for nspr-class library
//
//  contains the following tests
// 
//  TCPSOCKET - tests communication between server & client 
//              uses czkSockAcceptor, czkSockConnector, czkCoreLoop
//              requires two processes 
//              process 1: run "test TCPSOCKET SERVER"
//              process 2: run "test TCPSOCKET CLIENT"
//
//  TIMER     - tests czkTimer and czkCoreLoop functionality
//              run "test TIMER" 
//
//  THREAD    - tests czkThread functionality
//              run "test THREAD" 
//
//  LOCK      - tests czkLock functionality
//              run "test LOCK" 
//
//  CONDITION - tests czkCondition functionality
//              run "test CONDITION" 
//
//  FILE      - tests czkFile functionality
//              run "test FILE" 
//
//  DIRECTORY - tests czkDirectory functionality
//              run "test DIRECTORY" 
//
//  NETADDR   - tests czkNetAddr functionality
//              run "test NETADDR" 
//
//  SYSTEMINFO- tests czkSystemInfo functionality
//              run "test SYSTEMINFO" 
     
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <zks.h>
//#include "zkserr.h"
//#include "zksmodule.h"
//#include "zkslog.h"
//#include "zkslogagent.h"
    
#include <iostream>
#include <ext/hash_map>
   
#include "czkLock.h" 
#include "czkCondition.h" 
#include "czkThread.h"
#include "czkFile.h"
#include "czkDirectory.h"
#include "czkNetAddr.h"
#include "czkSystemInfo.h" 
#include "czkSockAcceptor.h"
#include "czkSockConnector.h"
#include "czkSockDgram.h"
#include "czkSockCoDgram.h"
#include "czkCoreLoop.h"
#include "czkIntervalTime.h"

static unsigned int sleepret;

class Condition_Test {

 public: 
    czkLock  *lock;
    czkCondition     *condition;
    int32_t state;
    Condition_Test   *next;
 
    Condition_Test(czkLock* lockin) 
    { lock = lockin; state = 0; condition = new czkCondition();
      condition->open( *lockin); 
      next=NULL;}
    ~Condition_Test() { condition->close(); delete condition; }
}; 
  
class Ready_Test {

 public: 
    czkLock  lock;
    czkCondition     *condition;
    bool state;
 
    Ready_Test() 
    { 
      lock.open(); state = false; condition = new czkCondition(); 
      condition->open( lock);
    }
    bool waitForReadySignal(ZKPRIntervalTime timeout)
    {
      lock.acquire();
      cout << " waiting " << timeout << endl;
      if (!state) condition->wait( timeout);
      cout << " aftwer wait " << time(NULL) << endl;
      lock.release();
      return state;
    }
    void startup()
    {
      for (int32_t i = 0; i < 5; i++)
      {
        cout << " sleeping " << endl;
        sleepret = sleep(1);
      }
      lock.acquire();
      state = true;
      condition->broadcast();
      lock.release();
    } 
    ~Ready_Test() { condition->close(); delete condition; lock.close(); }
};
  
class myCoreLoop: public czkEventHandlerData {

public:
  czkCoreLoop* cl;
};

class Timer_Test: public czkEventHandlerData {

public:
  ZKPRIntervalTime start;
  ZKPRIntervalTime duration;
  int32_t repeat;
  int32_t count;
  int32_t retryconnect;
  czkSocket* socket;
  czkCoreLoop* cl;
  czkTimerProxy* timerHandle;
};

class myczkEventHandler: public czkEventHandler {
  
 private:
  int32_t acceptedsock;
  
   
 public: 
   myczkEventHandler() 
      { acceptedsock = 0; cout << " construct myeh " << endl; };
   ~myczkEventHandler() { cout << " destruct myeh " << endl; };
   void handleInput( czkSocket*, czkEventHandlerData*);
   void handleOutput( czkSocket*, czkEventHandlerData*);
   void handleException( czkSocket*, czkEventHandlerData*);
   int32_t handleTimeout(czkEventHandlerData* user_data);
   void handleSignal(int32_t signum, czkEventHandlerData* user_data);
};
 
void myczkEventHandler::handleSignal(int32_t signum, 
                                     czkEventHandlerData* user_data)
{
  cout << " in handleSignal signum " << signum << " data " << user_data << endl;
}

int32_t myczkEventHandler::handleTimeout(czkEventHandlerData* user_data)
{
  Timer_Test* t = (Timer_Test*) user_data;

  if (t->retryconnect) {

    czkNetAddr netaddr;
    ZKPRNetAddr addr;
    memset(&addr, 0 , sizeof(addr));
    addr.inet.family = ZKPR_AF_INET;
    netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  
   
    czkIntervalTime interval;
    ZKPRIntervalTime timeout = interval.getSeconds(0);

    czkSockConnector* zkfd = (czkSockConnector*) t->socket;      
    zkfd->open();

    ZKPRSocketOptionData opt;
    opt.option = ZKPR_SockOpt_Reuseaddr;
    opt.value.reuse_addr = ZKPR_TRUE;
    zkfd->setOption( &opt);
    opt.option = ZKPR_SockOpt_Nonblocking;
    opt.value.non_blocking = 1;
    zkfd->setOption( &opt);

    zkfd->connect( addr, timeout);
    
    myCoreLoop* mcl = new myCoreLoop;
    mcl->cl = t->cl;
    t->cl->registerHandler( this, mcl, (czkSocket*)zkfd, 
                            ZKPR_POLL_WRITE | ZKPR_POLL_EXCEPT);
    return 0;
  }

  if (t->timerHandle) {
    cout << " trying to cancel timer " << endl;
    t->cl->cancelTimer(t->timerHandle);
  }
  cout << " timeout " << time(NULL) << endl;
  czkIntervalTime interval;
  if ((interval.getIntervalNow()-t->start - t->duration) > 
       interval.getMilliseconds(100)) 
  {
    cout << " error in timer? " << interval.getIntervalNow()-t->start << 
            " duration " << t->duration << endl;
    // exit(1);
  }
  t->count++;
  if (t->count > 10) {
    t->repeat = 0;
    t->cl->abortEventLoop();
  }
  t->start = interval.getIntervalNow();
  return t->repeat;
} 

void myczkEventHandler::handleException(czkSocket* zkfd, 
                                        czkEventHandlerData* client_data)
{ 
  czkIntervalTime interval;
  ZKPRIntervalTime timeout;
  static int32_t n = 0;

  PRErrorCode status = PR_GetError();
  
  cout << " got exception " << status << " n " << n << endl;
  n++;   

  myCoreLoop* mcl = (myCoreLoop*) client_data;
  czkCoreLoop *cl = mcl->cl;
  cl->removeHandler( zkfd);
  zkfd->close();

  if (acceptedsock) return;
  
    timeout = interval.getSeconds(10);

    Timer_Test* t1 = new Timer_Test();
    t1->start = interval.getIntervalNow();
    t1->duration = timeout;
    t1->repeat = 0; 
    t1->count = 0;
    t1->retryconnect = 1;
    t1->cl = cl;
    t1->socket= zkfd;
    cl->scheduleTimer( this, t1, timeout);

  //exit(1);
}

void myczkEventHandler::handleInput(czkSocket* zkfd, 
                                    czkEventHandlerData* client_data)
{
  char buf[100];
  int32_t n;   
  static int32_t c= 0;
  static int32_t connected = 0;

  myCoreLoop* mcl = (myCoreLoop*) client_data;
  czkCoreLoop *cl = mcl->cl;

  czkIntervalTime interval;
  ZKPRIntervalTime timeout; 
  czkSocket* stream;

  int32_t mydata;

  ZKPRNetAddr addrfrom;

  //cout << " in handle input " << endl;

  timeout = interval.getSeconds(0);

  if (!acceptedsock) {
    if (zkfd->accept( &stream, &addrfrom, timeout) == 0) {
      cout << " got an accept " << endl;
      myczkEventHandler* eh = new myczkEventHandler();
      cout << " calling register handle " << endl;
      eh->acceptedsock = 1;
   
      cl->registerHandler( eh, mcl, stream, 
                           ZKPR_POLL_READ | ZKPR_POLL_EXCEPT);
      connected = 1;
    } 
    else {
      cout << " error " << PR_GetError() << endl;
    }
  }
  else if (c >= 0) { 
    n = zkfd->recv( buf, 100, 0, timeout); 
   
    if (n > 0) {
      cout << " recv n " << n << endl;
      //printf(" buf %s \n", buf);
      //cl->removeHandler( &zkfd);
      //n = zkfd.close();
      //cout << " after close " << n << endl;
      //if (n != PR_SUCCESS) cout << " error " << PR_GetError() << endl;
      c++;
    } 
    else {
      cout << " n  " << n << " error " << PR_GetError() << endl;
    }
  }  
}

void myczkEventHandler::handleOutput(czkSocket* zkfd, 
                                     czkEventHandlerData* data)
{
  char buf[100];  
  
  czkIntervalTime interval;

  myCoreLoop* mcl = (myCoreLoop*) data;
  czkCoreLoop *cl = mcl->cl;

  //if (n > 100) return 0;
  //cout << " i am connected " << n << endl;

  zkfd->send( buf, 100, 0, interval.getSeconds(0)); 
  //static int n = 0;
  //czkCoreLoop *cl = (czkCoreLoop*) data;
  //if (n == 100) {
  //  cl->removeHandler( &zkfd);
  //  zkfd.close();
  //} 
  //n++;
  //cout << " output done " << endl;
}

class myudpEventHandler: public czkEventHandler {
  
 private:
  int32_t acceptedsock;
  
   
 public: 
   myudpEventHandler() 
      { acceptedsock = 0; cout << " construct myeh " << endl; };
   ~myudpEventHandler() { cout << " destruct myeh " << endl; };
   void handleInput( czkSocket*, czkEventHandlerData*);
   void handleOutput( czkSocket*, czkEventHandlerData*);
   void handleException( czkSocket*, czkEventHandlerData *);
   int32_t handleTimeout(czkEventHandlerData *) {};
};
 
void myudpEventHandler::handleException(czkSocket* zkfd, 
                                        czkEventHandlerData* client_data)
{ 
  czkIntervalTime interval;
  ZKPRIntervalTime timeout;
  static int32_t n = 0;

  PRErrorCode status = PR_GetError();
  
  cout << " got exception " << status << " n " << n << endl;
  n++;   

  myCoreLoop *mcl = (myCoreLoop*) client_data;
  czkCoreLoop *cl = mcl->cl;
  cl->removeHandler( zkfd);
  zkfd->close();

}

void myudpEventHandler::handleInput(czkSocket* zkfd, 
                                    czkEventHandlerData* client_data)
{
  char buf[100];
  int32_t n;   
  static int32_t c= 0;
  static int32_t connected = 0;

  myCoreLoop *mcl = (myCoreLoop*) client_data;
  czkCoreLoop *cl = mcl->cl;
 
  czkIntervalTime interval;
  ZKPRIntervalTime timeout; 
  czkSocket* stream;

  int32_t mydata;

  ZKPRNetAddr addrfrom;

  //cout << " in handle input " << endl;

  timeout = interval.getSeconds(0);

  ZKPRNetAddr addr;

    n = zkfd->recvfrom( buf, 100, 0, &addr, timeout); 
   
    if (n > 0) {
      cout << " recv n " << n << endl;
      n = zkfd->sendto( buf, 100, 0, &addr, timeout); 

      //printf(" buf %s \n", buf);
      //cl->removeHandler( &zkfd);
      //n = zkfd.close();
      //cout << " after close " << n << endl;
      //if (n != PR_SUCCESS) cout << " error " << PR_GetError() << endl;
      c++;
    } 
    else {
      cout << " n  " << n << " error " << PR_GetError() << endl;
    }
}

void myudpEventHandler::handleOutput(czkSocket* zkfd, 
                                     czkEventHandlerData* data)
{
  char buf[100];  
  
  czkIntervalTime interval;

  myCoreLoop *mcl = (myCoreLoop*) data;
  czkCoreLoop *cl = mcl->cl;
  
  //if (n > 100) return 0;
  //cout << " i am connected " << n << endl;

  //static int n = 0;
  //czkCoreLoop *cl = (czkCoreLoop*) data;
  //if (n == 100) {
  //  cl->removeHandler( &zkfd);
  //  zkfd.close();
  //} 
  //n++;
  //cout << " output done " << endl;
}

void ZKPR_CALLBACK eventloop(void *client_data)
{
  czkCoreLoop *cl = (czkCoreLoop*) client_data;

  int32_t mydata = 0;

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  

  cout << " create accept " << endl;
  czkSockAcceptor* me = new czkSockAcceptor();
  me->open(); 
  me->bind( addr);
  me->listen( 1);

  myczkEventHandler* eh = new myczkEventHandler();
  cout << " calling register handle " << endl;
  myCoreLoop* mcl = new myCoreLoop;
  mcl->cl = cl;
  cl->registerHandler( eh, mcl, (czkSocket*)me, 
                       ZKPR_POLL_READ  | ZKPR_POLL_EXCEPT);

}  
    
void ZKPR_CALLBACK eventloop2(void *client_data)
{
  czkCoreLoop *cl = (czkCoreLoop*) client_data;

  int32_t mydata = 0;

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  

  cout << " create accept " << endl;
  czkSockDgram* me = new czkSockDgram();
  me->open(); 
  me->bind( addr);

  myudpEventHandler* eh = new myudpEventHandler();
  cout << " calling register handle " << endl;
  myCoreLoop* mcl = new myCoreLoop;
  mcl->cl = cl;
  cl->registerHandler( eh, mcl, (czkSocket*)me, 
                       ZKPR_POLL_READ  | ZKPR_POLL_EXCEPT);

}  
    
void ZKPR_CALLBACK client(void *client_data)
{
  ZKPRIntervalTime timeout;
  czkIntervalTime interval;

#define MAXCONNECT 1
 
  czkSockConnector stream[MAXCONNECT];
  char buf[100];

  czkCoreLoop *cl = (czkCoreLoop*) client_data;

  timeout = interval.getSeconds(0);

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  
       
  for (int32_t i = 0; i < MAXCONNECT; i++)
  {     
    stream[i].open();

    ZKPRSocketOptionData opt;
    opt.option = ZKPR_SockOpt_Reuseaddr;
    opt.value.reuse_addr = ZKPR_TRUE;
    stream[i].setOption( &opt);
    opt.option = ZKPR_SockOpt_Nonblocking;
    opt.value.non_blocking = 1;
    stream[i].setOption( &opt);

    stream[i].connect( addr, timeout);
    //if (stream[i].connect( addr, timeout) < 0) {
    //  cout << " connect error " << PR_GetError() << endl;
    //  exit(1);
    //}
    myczkEventHandler* eh = new myczkEventHandler();
    cout << " calling register handle " << endl;
    myCoreLoop* mcl = new myCoreLoop;
    mcl->cl = cl;
    cl->registerHandler( eh, mcl, (czkSocket*)&stream[i], 
                         ZKPR_POLL_WRITE | ZKPR_POLL_EXCEPT);
  } 
   
  sleepret = sleep(100); 
} 
 
void ZKPR_CALLBACK client2(void *client_data)
{
  ZKPRIntervalTime timeout;
  czkIntervalTime interval;

#define MAXCONNECT 1
 
  char buf[100];

  czkCoreLoop *cl = (czkCoreLoop*) client_data;

  timeout = interval.getSeconds(0);

  czkNetAddr netaddr;
  ZKPRNetAddr addr;
  memset(&addr, 0 , sizeof(addr));
  addr.inet.family = ZKPR_AF_INET;
  netaddr.initNetAddr( ZKPR_IpAddrLoopback, 6666, &addr);  
       
  czkSockCoDgram sock;

    sock.open();

    ZKPRSocketOptionData opt;
    opt.option = ZKPR_SockOpt_Reuseaddr;
    opt.value.reuse_addr = ZKPR_TRUE;
    sock.setOption( &opt);
    opt.option = ZKPR_SockOpt_Nonblocking;
    opt.value.non_blocking = 1;
    sock.setOption( &opt);

    sock.connect( addr, timeout);
    //if (stream[i].connect( addr, timeout) < 0) {
    //  cout << " connect error " << PR_GetError() << endl;
    //  exit(1);
    //}

    myudpEventHandler* eh = new myudpEventHandler();
    cout << " calling register handle " << endl;
    myCoreLoop* mcl = new myCoreLoop;
    mcl->cl = cl;
    cl->registerHandler( eh, mcl, (czkSocket*)&sock, 
                         ZKPR_POLL_READ | ZKPR_POLL_EXCEPT);
   
  sock.send( buf, 100, 0, interval.getSeconds(0)); 
   
  sleepret = sleep(100); 
} 
 
void thread1(void* client_data)
{
  for (int32_t i = 0; i < 1000; i++)
    cout << " thread1 i " << i << endl; 
}

void thread2( void* client_data)
{
  czkLock* m;

  m = (czkLock*) client_data;

  cout << " in thread2 attempting to acquire lock " << endl;

  m->acquire();

  cout << " in thread2 acquired lock " << time(NULL) << endl;

  sleepret = sleep(5);

  if (m->release() != ZKPR_SUCCESS) {
    cout << " error " << PR_GetError() << endl;
    exit(1);
  }

}

void thread3( void* client_data)
{

  Condition_Test* ct;

  ct = (Condition_Test*) client_data;

  ct->lock->acquire();
  int32_t temp = ct->state; 
  while (ct->state == temp)
  {
    cout << " waiting for signal " << endl;    
    ct->condition->wait(); 
    cout << " got signal " << endl;    
  } 
  cout << " got state " << ct->state << endl;
  if (ct->next) {
    ct->next->state++;
    ct->next->condition->signal();
  }
  if (ct->lock->release() != ZKPR_SUCCESS) {
    cout << " error " << PR_GetError() << endl;
    exit(1);
  }
} 
  
void thread4( void* client_data)
{

  czkCoreLoop* cl;
  ZKPRIntervalTime d;
  czkIntervalTime interval;
  myczkEventHandler eh;

  cl = (czkCoreLoop*) client_data;

  for (int32_t i = 0; i < 10; i++)   
  { 
    d = interval.getSeconds(3);
    Timer_Test* t1 = new Timer_Test();
    t1->start = interval.getIntervalNow();
    t1->duration = d;
    t1->repeat = 0;
    t1->count = 0;
    t1->cl = cl;
    t1->retryconnect = 0;
    t1->timerHandle = NULL;
    cl->scheduleTimer( &eh, t1, d);
    sleepret = sleep(5);
  } 
} 
  
void thread5( void* client_data)
{
  Ready_Test* test = (Ready_Test*) client_data;
  test->startup();
  sleepret = sleep(100);
}

typedef struct hashkey HashKey;
struct hashkey {
     int32_t type;
     char name[10];
};

typedef struct hashdata HashData;
struct hashdata {
     int32_t d1;
     char d2[10];
};

class myclass {

     friend ostream& operator << (ostream& os, myclass& s);

     public:
     
     HashData data;
     HashKey  key;

     myclass() {};
     HashData& getData() { return data; };
     void setData( int32_t d1in, char* d2in) 
     {
       data.d1 = d1in;
       strcpy( data.d2, d2in);
     }

     HashKey& getkey() { return key; };
     void setkey(int32_t typein, char* namein) 
      { 
        key.type = typein; 
        strcpy(key.name, "         ");
        strcpy(key.name, namein); };
};

ostream& operator<<(ostream& os, myclass& s)
{
  return os << s.key.type << " " << s.key.name << " " << 
               s.data.d1 << "  " << s.data.d2;
}

struct eqstr
{
  bool operator()(const HashKey s1, const HashKey s2) const
  {
    return (s1.type == s2.type) && (strcmp(s1.name, s2.name) == 0);
  }
};

template <class T> struct myhash : unary_function<T,size_t> {
 public:

  size_t operator()(const T& key) const;
}; 

template <class T> size_t myhash<T>::operator() (const T& key) const
{
    size_t res = 0;
    size_t len = sizeof(T);
    //size_t len = sizeof(key.name);

    const char* p = reinterpret_cast<const char *> (&key);
    
    while (len--) res = (res<<1)^*p++;
    //cout << " key " << key.type << " " << key.name << " " << res << endl; 
    return res;
 
};

typedef hash_map<const HashKey, myclass*, myhash<HashKey>, eqstr> hash_map_type;

hash_map_type::iterator findmonth( HashKey key, hash_map_type months)
{
   hash_map_type::iterator i = months.find(key);
   //if (i == months.end())
   //return NULL;
   // else
     return i;
}

class SigHandler {

private:  
  czkCoreLoop* coreloop;

public:

  void setCoreLoop(czkCoreLoop* cl) { coreloop = cl; };

  void addSignal( int32_t signum) { coreloop->addSignal( signum); }
};
 
SigHandler GlobalSigHandler;

void sighandler( int signum)
{
  GlobalSigHandler.addSignal( signum);
}

class mylock {

protected:
  czkLock lock;

public:
  mylock() { lock.open(); }

  virtual ~mylock() { lock.close(); }

  void acquire() { lock.acquire(); }

  void release() { lock.release(); }
};

class mymylock : public mylock {

public:

};

int main(int32_t argc, char **argv)
{ 
  czkCoreLoop cl;
  myczkEventHandler eh;
  myCoreLoop mcl;
  ZKPRIntervalTime d;
  czkIntervalTime interval;

  czkThread client_thread;
  czkLock  lock;
  
  bool doready      =  (strcmp(argv[1],"READY") == 0);
  bool dotcpsocket  =  (strcmp(argv[1],"TCPSOCKET") == 0);
  bool doudpsocket  =  (strcmp(argv[1],"UDPSOCKET") == 0);
  bool dotimer      =  (strcmp(argv[1],"TIMER") == 0);
  bool dothread     =  (strcmp(argv[1],"THREAD") == 0);
  bool dolock      =  (strcmp(argv[1],"LOCK") == 0);
  bool docondition  =  (strcmp(argv[1],"CONDITION") == 0);
  bool dofile       =  (strcmp(argv[1],"FILE") == 0);
  bool dodirectory  =  (strcmp(argv[1],"DIRECTORY") == 0);
  bool donetaddr    =  (strcmp(argv[1],"NETADDR") == 0);
  bool dosysteminfo =  (strcmp(argv[1],"SYSTEMINFO") == 0);
  bool dohashmap    =  (strcmp(argv[1],"HASHMAP") == 0);
  bool domylock     =  (strcmp(argv[1],"MYLOCK") == 0);
 
  if (domylock) {

    mymylock Mylock;

    Mylock.acquire();
    Mylock.release();

  }

  if (dotcpsocket) { 
    
    if (strcmp(argv[2],"SERVER") == 0) {
      client_thread.spawn( eventloop, (void*)&cl, (ZKPRThreadScope)0, 
                           (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);
    } 
    else {
      client_thread.spawn( client, (void*)&cl, (ZKPRThreadScope)0, 
                           (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);
    }
    cl.runEventLoop();

    cout << " PASS " << endl; 
  }

  if (doudpsocket) { 
    
    if (strcmp(argv[2],"SERVER") == 0) {
      client_thread.spawn( eventloop2, (void*)&cl, (ZKPRThreadScope)0, 
                           (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);
    } 
    else {
      client_thread.spawn( client2, (void*)&cl, (ZKPRThreadScope)0, 
                           (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);
    }
    cl.runEventLoop();

    cout << " PASS " << endl; 
  }

  if (dotimer) {

    cout << " start " << time(NULL) << endl;
   
    d = interval.getSeconds(5);
    cout << " D " << d << endl;
    Timer_Test* t1 = new Timer_Test();
    t1->start = interval.getIntervalNow();
    t1->duration = d;
    t1->repeat = 0;
    t1->count = 0;
    t1->cl = &cl;
    t1->retryconnect = 0;
    t1->timerHandle = NULL;
    czkTimerProxy* timer1 = cl.scheduleTimer( &eh, t1, d);
    
    d = interval.getSeconds(4);
    Timer_Test* t2 = new Timer_Test();
    t2->start = interval.getIntervalNow();
    t2->duration = d;
    t2->repeat = 1;
    t2->count = 0;
    t2->cl = &cl;
    t2->retryconnect = 0;
    t2->timerHandle = timer1;
    //t2->timerHandle = NULL;
    czkTimerProxy* timer2 = cl.scheduleTimer( &eh, t2, d);

    delete timer2;
    //cout << " trying to cancel timer " << endl;
    //cl.cancelTimer(timer1); 

    //PRThread* h = client_thread.spawn( thread4, &cl, (PRThreadScope)0, 
    //                                   (PRThreadState)0, (PRThreadPriority)0, 0);
    cl.runEventLoop();
 
    cout << " PASS " << endl;
  }

  if (dothread) {

    ZKPRThread* h = client_thread.spawn( thread1, NULL, (ZKPRThreadScope)0, 
                                       (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);
    if (client_thread.join(h) != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1); 
    }
    cout << " PASS " << endl;
  }    

  if (dolock) {

    lock.open();

    cout << " acquiring lock " << time(NULL) << endl;
    lock.acquire();
    ZKPRThread* h = client_thread.spawn( thread2, &lock, (ZKPRThreadScope)0, 
                                       (ZKPRThreadState)0, (ZKPRThreadPriority)0, 0);  
    sleepret = sleep(5);
    if (lock.release() != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1);
    }
    sleepret = sleep(1);
    lock.acquire();
    cout << " in main acquired lock " << time(NULL) << endl;
    sleepret = sleep(5);
    if (lock.release() != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1); 
    }
    if (client_thread.join(h) != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1);
    } 

    lock.close();

    cout << " PASS " << endl;
  }

  if (doready) {
    Ready_Test* test = new Ready_Test();

    cout << " Start " << time(NULL) << endl;
    
    ZKPRThread* h1 = client_thread.spawn( thread5, (void*)test, 
                                        (ZKPRThreadScope)0, 
                                        (ZKPRThreadState)0, 
                                        (ZKPRThreadPriority)0, 0);
    //sleepret = sleep(1);

    d = interval.getSeconds(1);
    bool restest = test->waitForReadySignal(d);
    cout << " timeout " << restest << " time " << time(NULL) << endl;

    d = interval.getSeconds(10);
    test->waitForReadySignal(d);

    cout << " end " << time(NULL) << endl;

    delete test;
  }
    

  if (docondition) {
    czkLock* lock = new czkLock(); 
    lock->open();
    Condition_Test* ct = new Condition_Test(lock);
    Condition_Test* ct1 = new Condition_Test(lock);
    Condition_Test* ct2 = new Condition_Test(lock);

    ct->next = ct1;
    ct1->next = ct2;

    ZKPRThread* h1 = client_thread.spawn( thread3, ct1, 
                                        (ZKPRThreadScope)0, 
                                        (ZKPRThreadState)0, 
                                        (ZKPRThreadPriority)0, 0);
    ZKPRThread* h2 = client_thread.spawn( thread3, ct2, 
                                        (ZKPRThreadScope)0, 
                                        (ZKPRThreadState)0, 
                                        (ZKPRThreadPriority)0, 0);
    sleepret = sleep(4);
    ct->lock->acquire();
    ct->next->state++;
    ct->next->condition->signal();
    cout << " sent signal " << endl;
    if (ct->lock->release() != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1);
    }
    if (client_thread.join(h1) != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1);
    }
    if (client_thread.join(h2) != ZKPR_SUCCESS) {
      cout << " error " << PR_GetError() << endl;
      exit(1);
    }
    lock->close();
    cout << " PASS " << endl;
  }

  if (dofile) {
  
    czkFile file;
    char buf[6];
    char filename[128];
    char filename2[128];
    ZKPRFileInfo fileinfo;

    strcpy( filename, "/turf/frankf/zk_test.dat");
    strcpy( filename2, "/turf/frankf/zk_test2.dat");

    int32_t flags;
    int32_t mode;

    if (file.getFileAccess( filename, ZKPR_ACCESS_EXISTS) == ZKPR_SUCCESS) 
    {
      cout << " file exists " << endl;     
      flags = ZKPR_RDWR;
    }
    else {
      cout << " new file " << endl;
      flags = ZKPR_CREATE_FILE | ZKPR_WRONLY;
      mode = 0666;
    } 

    if (file.open(filename, flags, mode) != ZKPR_SUCCESS) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }
    if (file.write("HELLO", 6) == ZKPR_FAILURE) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }
    if (file.close() != ZKPR_SUCCESS) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }

    if (file.open(filename, ZKPR_RDONLY, 0) != ZKPR_SUCCESS) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }
    if (file.getFileInfo(&fileinfo) != ZKPR_SUCCESS) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }

    cout << " file type " << fileinfo.type << endl;
    cout << " file size " << fileinfo.size << endl;
    cout << " file creation time " << fileinfo.creationTime << endl;
    cout << " file modify time " << fileinfo.modifyTime << endl;

    cout << " available " << file.getAvailable() << endl;

    if (file.read(buf, 6) == ZKPR_FAILURE) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }
    if (strcmp(buf, "HELLO") != 0) {
      cout << " string don't match " << endl;
      exit(1);
    }
    if (file.close() != ZKPR_SUCCESS) {
      cout << " close Error " << PR_GetError() << endl;
      exit(1);
    }

    if (file.renameFile( filename, filename2) != ZKPR_SUCCESS) {
      cout << " rename Error " << PR_GetError() << endl;
      exit(1);
    }
    if (file.getFileInfo(&fileinfo, filename2) != ZKPR_SUCCESS) {
      cout << " getfiuleinfo Error " << PR_GetError() << endl;
      exit(1);
    }

    cout << " file type " << fileinfo.type << endl;
    cout << " file size " << fileinfo.size << endl;
    cout << " file creation time " << fileinfo.creationTime << endl;
    cout << " file modify time " << fileinfo.modifyTime << endl;

    if (file.deleteFile( filename2) != ZKPR_SUCCESS) {
      cout << " Error " << PR_GetError() << endl;
      exit(1);
    }
     
    if (file.getFileAccess( filename2, ZKPR_ACCESS_EXISTS) == ZKPR_SUCCESS)
    {
      cout << " file not deleted " << endl;
      exit(1);
    }
  
    cout << " PASS " << endl;
  }
    
  if (dodirectory) {

    czkDirectory dir;
    czkFile file;
    ZKPRDirEntry* direntry;
    ZKPRFileInfo fileinfo;

    if (dir.open("/turf/frankf") == ZKPR_FAILURE) {
      cout << " dir open error " << PR_GetError() << endl;
      exit(1);
    }

    if (dir.read( &direntry, ZKPR_SKIP_NONE) == ZKPR_FAILURE) {
      cout << " dir read error " << PR_GetError() << endl;
      exit(1);
    }
    while (direntry)
    {
      cout << " entry " << direntry->name << endl;
      if (dir.read( &direntry, ZKPR_SKIP_NONE)  == ZKPR_FAILURE) {
        cout << " dir read error " << PR_GetError() << endl;
        exit(1);
      }
    }
    if (dir.close() == ZKPR_FAILURE) {
      cout << " dir close error " << PR_GetError() << endl;
      exit(1);
    }

    if (dir.mkDir("/turf/frankf/testdir",0666) == ZKPR_FAILURE) {
      cout << " mkdir error " << PR_GetError() << endl;
      exit(1);
    } 

    if (file.getFileInfo(&fileinfo, "/turf/frankf/testdir") == ZKPR_FAILURE) {
      cout << " dir getfileinfo error " << PR_GetError() << endl;
      exit(1);
    }

    cout << " file type " << fileinfo.type << endl;
    cout << " file size " << fileinfo.size << endl;
    cout << " file creation time " << fileinfo.creationTime << endl;
    cout << " file modify time " << fileinfo.modifyTime << endl;

    if (dir.rmDir("/turf/frankf/testdir") == ZKPR_FAILURE) {
      cout << " rmdir error " << PR_GetError() << endl;
      exit(1);
    }

  }
     
  if (donetaddr) {

    czkNetAddr na;  
    ZKPRNetAddr  netaddr;
    ZKPRNetAddr  netaddr2;
    int32_t n; 

    if (na.getNetAddr( "10.16.128.115", &netaddr) == ZKPR_FAILURE) {
      cout << " getNetaddr error " << PR_GetError() << endl;
      exit(1);
    }
    cout << " ip " << netaddr.inet.ip << endl;
    
    if (na.getHostbyAddr( &netaddr, ZKPR_NETDB_BUF_SIZE)  == ZKPR_FAILURE) {
      cout << " dir open error " << PR_GetError() << endl;
      exit(1);
    }
    //na.getHostbyName( "frankf", ZKPR_NETDB_BUF_SIZE);
    n = 0;
    while ((n = na.enumerateHostEnt( n, 0, &netaddr2)))
    { 
      cout << " ip2 " << netaddr2.inet.ip << endl;
    }

  }
 
  if (dosysteminfo) {

    czkSystemInfo sysinfo;
    char buf[128];

    if (sysinfo.getHostname( buf, sizeof(buf)) == ZKPR_FAILURE) {
      cout << " gethostname error " << PR_GetError() << endl;
      exit(1);
    } 
    printf( " hostname %s \n", buf);
    
    if (sysinfo.getSysname( buf, sizeof(buf)) == ZKPR_FAILURE) {
      cout << " getSysname error " << PR_GetError() << endl;
      exit(1);
    }
    printf( " sysname %s \n", buf);
    
    if (sysinfo.getRelease( buf, sizeof(buf)) == ZKPR_FAILURE) {
      cout << " getRelease error " << PR_GetError() << endl;
      exit(1);
    }
    printf( " release %s \n", buf);
    
    if (sysinfo.getArchitecture( buf, sizeof(buf)) == ZKPR_FAILURE) {
      cout << " getArchitecture error " << PR_GetError() << endl;
      exit(1);
    }
    printf( " architecture %s \n", buf);
    
    cout << " pagesize " << sysinfo.getPageSize() << endl;
    cout << " pageshift " << sysinfo.getPageShift() << endl;
    printf( " env %s \n", sysinfo.getEnvVariable("HOST"));
    
  }

  if (dohashmap) {

   myclass* x;

   hash_map_type months;

   x = new myclass();
   x->setData(1,"A");
   x->setkey(1,"january");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(1,"february");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(1,"march");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(1,"april");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(1,"may");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(1,"june");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(2,"july");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(2,"B");
   x->setkey(2,"august");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(3,"C");
   x->setkey(2,"september");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(3,"C");
   x->setkey(2,"october");
   months[x->getkey()] = x;
  
   x = new myclass();
   x->setData(3,"C");
   x->setkey(2,"november");
   months[x->getkey()] = x;

   x = new myclass();
   x->setData(3,"C");
   x->setkey(2,"december");
   months[x->getkey()] = x;
  
   x = new myclass();
   x->setData(4,"D");
   x->setkey(258,"december");
   months[x->getkey()] = x;
  
   for (hash_map_type::iterator i = months.begin(); i != months.end(); i++)
   {
     cout << *(i->second) << endl;
   }
   
   cout << " ** " << endl;
   x = new myclass();
   x->setkey(2,"september");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(1,"february");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(2,"december");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(258,"december");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(2,"july");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(1,"april");
   cout << "" << *months[x->getkey()] << endl;
   x->setkey(2,"august");
   cout << "" << *months[x->getkey()] << endl;

   x->setkey(2,"september");
   hash_map_type::iterator y = findmonth(x->getkey(), months);
   if (y == months.end())
     cout << " not found " << endl;
   else
     cout << " found " << *(y->second) << endl;

   y->second->setData(99,"Z");
   //months[y->getkey()] = *y;
   hash_map_type::iterator z = findmonth(x->getkey(), months);
   if (z == months.end())
     cout << " not found " << endl;
   else
     cout << " found " << *(z->second) << endl;

   cout << " ** " << endl;
   for (hash_map_type::iterator i = months.begin(); i != months.end(); i++)
   {
     cout << *(i->second) << endl;
   }
   

  }

}

