/*
 *	zkArpaInet.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkArpaInet.h,v 1.2 2001/04/19 17:26:48 alainc Exp $
 *	
 *	Wrapper for <arpa/inet.h>
 *	
 *	This file defines at least the following symbols:
 *		
 *	Functions:
 *		uint32_t	htonl(uint32_t hostlong);
 *		uint16_t	htons(uint16_t hostshort);
 *		uint32_t	ntohl(uint32_t netlong);
 *		uint16_t	ntohs(uint16_t netshort);
 *		in_addr_t	zkInet_addr(const char *cp);
 *		char*		zkInet_ntoa(struct in_addr in);
 *		// not in Unix 98:
 *		int			zkInet_aton(const char* cp, struct in_addr* inp);
 *	
 */

#ifndef ZKARPAINET_H_
#define ZKARPAINET_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if defined(WIN32)

	/* Win32 */
	
#	include "zkNetinetIn.h"

#elif defined(macintosh)
	
	/* Mac OS */
	
#	include "MacSockLib.h"
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
#	include <sys/types.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>

#	include "zkNetinetIn.h"
	
#else
	
	/* UNIX */
	
#	include <arpa/inet.h>
#	if defined(ZK_LINUX)
#		include "zkNetinetIn.h"		/* in_addr_t, in_port_t */
#	endif

#endif

#if !defined(ZK_KERNEL)

#ifdef __cplusplus
	extern "C" {
#endif


/* Wrapper Function Declarations */

DLLExport in_addr_t	zkInet_addr(const char *cp);
DLLExport char*		zkInet_ntoa(struct in_addr in);
DLLExport int		zkInet_aton(const char* cp, struct in_addr* inp);


#ifdef __cplusplus
	}
#endif

#endif	/* !defined(ZK_KERNEL) */

#endif	/* ZKARPAINET_H_ */
