/*
 *	zkAtomic.h
 *	
 *	Copyright (c) 2001 Zero-Knowledge Systems Inc.  All rights reserved.
 *	
 *	$Id: zkAtomic.h,v 1.2 2001/04/19 17:26:48 alainc Exp $
 */

#ifndef ZKATOMIC_H_
#define	ZKATOMIC_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#include "zkIntTypes.h"

#if defined(ZK_KERNEL)
#	include "zkKrtAtomic.h"
#elif defined(ZK_LINUX)
#	include <pthread.h>
#elif defined(WIN32)
#	include <windows.h>
#endif

#ifdef __cplusplus
	extern "C" {
#endif


#if defined(_MSC_VER)
#	pragma pack(push, 4)
#elif defined(__MWERKS__)
#	pragma options align=power
#endif

typedef struct {
#if defined(ZK_KERNEL)
	ZkKrtAtomicInt	value;
#elif defined(macintosh)
	int32_t			value;
#elif defined(WIN32)
	LONG			value;
#elif defined(ZK_LINUX)
	pthread_mutex_t	mutex;
	int32_t			value;
#else
#	error we need a definition of ZkAtomicInt for this platform ...
#endif
} ZkAtomicInt;

#if defined(_MSC_VER)
#	pragma pack(pop)
#elif defined(__MWERKS__)
#	pragma options align=reset
#endif


void	zkAtomicNew(ZkAtomicInt* atomicInt, int32_t initialValue);
void	zkAtomicFree(ZkAtomicInt* atomicInt);
void	zkAtomicSet(ZkAtomicInt* atomicInt, int32_t newValue);
int32_t	zkAtomicIncrement(ZkAtomicInt* atomicInt);
int32_t	zkAtomicDecrement(ZkAtomicInt* atomicInt);
int32_t	zkAtomicExchange(ZkAtomicInt* atomicInt, int32_t newValue);


#ifdef __cplusplus
	}
#endif

#endif /* ZKATOMIC_H_ */
