/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKDATETIME_H_
#define ZKDATETIME_H_

#ifndef _ZKINTTYPES_H
#include "zkIntTypes.h"
#endif /* _ZKINTTYPES_H */


#ifndef _ZKERROR_H
#include "zkError.h"
#endif /* _ZKERROR_H */

#include "zkTime.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * String lengths for zkDateStore, zkTimeStore, zkDateTimeStore
 */

#define ZKDATESTR_SZ     10  
#define ZKTIMESTR_SZ     10
#define ZKDATETIMESTR_SZ 22

/*
 * Warning:  for the moment, ZkDateTimes will ONLY work in utc
 */

struct T_ZkDate {
  uint16_t year;  /* year is 0 to 9999 */
  uint8_t month;  /* Month is 1 to 12 */
  uint8_t day;  /* Day is 1 to 31 */
  uint8_t flags;
};

/*
 * ZkDate flags 
 */

#define ZKDATE_FL_GREGORIAN 0x1 /* DateTime is in gregorian calendar (dflt) */

typedef struct T_ZkDate ZkDate;

struct T_ZkTime {
  uint8_t hour;  /* 0 to 23 */
  uint8_t minute; /* 0 to 59 */
  uint8_t second; /* 0 to 60 (yes 60, for leap seconds) */
  short offset;          /* Offset from UTC (in minutes) if local time */
  uint8_t flags;
};

typedef struct T_ZkTime ZkTime;

/* 
 * ZkTime flags
 */

#define ZKTIME_FL_UTC     0x1  /* Date time is in UTC format */
#define ZKTIME_FL_LOCAL   0x2  /* Date time is in local time */

/*
 * Temporal units, used in the zkDateTimeAdd() function
 */

#define ZKDATETIME_SECONDS     1
#define ZKDATETIME_MINUTES     2
#define ZKDATETIME_HOURS       3
#define ZKDATETIME_DAYS        4
#define ZKDATETIME_MONTHS      5
#define ZKDATETIME_YEARS       6

/* 
 *
 */

struct T_ZkDateTime {
  ZkDate date;
  ZkTime time;
};

typedef struct T_ZkDateTime ZkDateTime;

/*
 *
 */

#define zkIsLeapYear(_year) ( ((_year)%4 == 0) && ( ((_year)%100 != 0) || ((_year)%400 == 0)) )

/*
 * Function prototypes
 */

DLLExport int zkDateTimeCompare (
  ZkError ,
  ZkDateTime * ,       /* a */
  ZkDateTime * ,       /* b */
  int * );             /* -1 if a < b, 0 if a = b, 1 if a > b */

DLLExport int zkDateCompare (
  ZkError ,
  ZkDate * ,
  ZkDate * ,
  int * );

DLLExport int zkDateTimeAdd (
  ZkError ,            /* Returns the error */
  ZkDateTime * ,       /* ZkDateTime structure to modify */
  unsigned int ,       /* Temporal offset (must be positive) */
  unsigned short );    /* Unit of the offset ZKDATETIME_DAYS, etc */


DLLExport int zkTimeCompare (
  ZkError ,
  ZkTime * ,
  ZkTime * ,
  int * );

DLLExport int zkDateParse (
  ZkError ,
  ZkDate * ,
  const char * );

DLLExport int zkDateValidate (
  ZkError ,
  ZkDate * );

DLLExport int zkTimeValidate (
  ZkError ,
  ZkTime * );

DLLExport int zkTimeParse (
  ZkError ,
  ZkTime * ,
  const char * );

DLLExport int zkDateTimeParse (
  ZkError ,
  ZkDateTime * ,
  const char * );

DLLExport int zkDateTimeSetCurrent (
  ZkError ,
  ZkDateTime * ,
  uint8_t );         /* ZKTIME_FL_..... */

DLLExport int zkDateTimeMkTime (
  ZkError ,
  ZkDateTime * ,
  uint8_t ,         /* ZKTIME_FL_LOCAL is supported only */
  time_t * );       /* Localtime in seconds */

DLLExport int zkDateSetCurrent (
  ZkError ,
  ZkDate * ,
  uint8_t );         /* ZKTIME_FL_..... */

DLLExport int zkTimeSetCurrent (
  ZkError ,
  ZkTime * ,
  uint8_t );         /* ZKTIME_FL_..... */

DLLExport int zkDateTimeStore (
  ZkError ,
  ZkDateTime * ,
  char * ,
  size_t );

DLLExport int zkDateStore (
  ZkError ,
  ZkDate * ,
  char * ,
  size_t );

DLLExport int zkTimeStore (
  ZkError ,
  ZkTime * ,
  char * ,
  size_t );




#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKDATETIME_H_ */
