/*
 *  zkDebug.h
 *  $Id: zkDebug.h,v 1.8 2001/04/19 17:26:48 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 *  Luc Boulianne <luc@zks.net>

 */

#ifndef _ZKDEBUG_H
#define	_ZKDEBUG_H
#include "zkStdio.h"
#include <stdarg.h>
#include "zks.h"
#include "zkDefs.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define zkDebug_va zkDebug

  DLLExport extern int zkDebugOn(void);
  DLLExport extern int zkDebugOff(void);
  DLLExport extern int zkDebugToggle(void);
#if !defined(WIN32) && !defined(macintosh) && !defined(ZK_KERNEL)
  DLLExport extern int zkDebugAddStream(FILE * aStream);
#endif

  
  DLLExport extern int zkDebug (
	const char * ,                /* Message format string to send to log */
	... ) ZK_PRINTF_LIKE(1, 2);   /* Optional arguments */
  
#ifndef NDEBUG
  DLLExport extern int  zkDebugIsOn(void);
  DLLExport extern void zkDebugStr(const char* message);
  DLLExport extern void zkDebugPrint(const char* message, ...);
  DLLExport extern void zkDebugVPrint(const char* message, va_list args);
  DLLExport extern void zkDebugBreak(const char* message, ...);
  DLLExport extern void zkDebugVBreak(const char* message, va_list args);
  DLLExport extern void zkDebugAssertFailed(const char* c, const char* f, int l);
# define zkDebugAssert(c) ((c) ? ((void) 0) : zkDebugAssertFailed(#c,__FILE__, __LINE__))
#else
# define zkDebugIsOn()			0
# define zkDebugStr(m)			((void) 0)
# define zkDebugPrint			while(0)
# define zkDebugVPrint(m, a)	((void) 0)
# define zkDebugBreak			while(0)
# define zkDebugVBreak(m, a)	((void) 0)
# define zkDebugAssert(c)		((void) 0)
#endif

  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKLOG_H */

