/*
 *	zkErrno.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkErrno.h,v 1.2 2001/04/19 17:26:48 alainc Exp $
 *	
 *	Wrapper for <errno.h>
 *	
 *	This file defines at least the following symbols:
 *
 *	Macros:
 *		E2BIG			Argument list too long. 
 *		EACCES			Permission denied. 
 *		EADDRINUSE		Address in use. 
 *		EADDRNOTAVAIL	Address not available. 
 *		EAGAIN			Resource unavailable, try again (may be the same value as EWOULDBLOCK). 
 *		EALREADY		Connection already in progress. 
 *		EBADF			Bad file descriptor. 
 *		EBADMSG			Bad message. 
 *		EBUSY			Device or resource busy. 
 *		ECANCELED		Operation canceled. 
 *		ECHILD			No child processes. 
 *		ECONNABORTED	Connection aborted. 
 *		ECONNREFUSED	Connection refused. 
 *		ECONNRESET		Connection reset. 
 *		EDEADLK			Resource deadlock would occur. 
 *		EDESTADDRREQ	Destination address required. 
 *		EDOM			Mathematics argument out of domain of function. 
 *		EEXIST			File exists. 
 *		EFAULT			Bad address. 
 *		EFBIG			File too large. 
 *		EHOSTDOWN		Host is down. 
 *		EHOSTUNREACH	Host is unreachable. 
 *		EINPROGRESS		Operation in progress. 
 *		EINTR			Interrupted function. 
 *		EINVAL			Invalid argument. 
 *		EIO				I/O error. 
 *		EISDIR			Is a directory. 
 *		EISCONN			Socket is connected. 
 *		EMFILE			Too many open files. 
 *		EMSGSIZE		Message too large. 
 *		EMLINK			Too many links. 
 *		ENAMETOOLONG	Filename too long. 
 *		ENETDOWN		Network is down. 
 *		ENETUNREACH		Network unreachable. 
 *		ENFILE			Too many files open in system. 
 *		ENOBUFS			No buffer space available. 
 *		ENODEV			No such device. 
 *		ENODATA			No message is available on the STREAM head read queue. 
 *		ENOENT			No such file or directory. 
 *		ENOEXEC			Executable file format error. 
 *		ENOLCK			No locks available. 
 *		ENOMEM			Not enough space. 
 *		ENOMSG			No message of the desired type. 
 *		ENOPROTOOPT		Protocol not available. 
 *		ENOSPC			No space left on device. 
 *		ENOSR			No STREAM resources. 
 *		ENOSTR			Not a STREAM. 
 *		ENOSYS			Function not supported. 
 *		ENOTCONN		The socket is not connected. 
 *		ENOTDIR			Not a directory. 
 *		ENOTEMPTY		Directory not empty. 
 *		ENOTSOCK		Not a socket. 
 *		ENOTTY			Inappropriate I/O control operation. 
 *		ENXIO			No such device or address. 
 *		EOPNOTSUPP		Operation not supported on socket. 
 *		EPERM			Operation not permitted. 
 *		EPIPE			Broken pipe. 
 *		EPROTO			Protocol error. 
 *		EPROTONOSUPPORT	Protocol not supported. 
 *		EPROTOTYPE		Socket type not supported. 
 *		ERANGE			Result too large. 
 *		EROFS			Read-only file system. 
 *		ESHUTDOWN		Can't send after socket shutdown. 
 *		ESOCKTNOSUPPORT	Socket type not supported. 
 *		ESPIPE			Invalid seek. 
 *		ESRCH			No such process. 
 *		ETIME			Stream ioctl() timeout. 
 *		ETIMEDOUT		Connection timed out. 
 *		ETOOMANYREFS	Too many references: cannot splice. 
 *		EWOULDBLOCK		Operation would block (may be the same value as [EAGAIN]).
 *		EXDEV			Cross-device link. 
 *
 *	Variables:
 *		int	errno;
 *	
 *
 *	UNIX 98 symbols (we'll have to see which of the values we want to support, if any):
 *		EAFNOSUPPORT	Address family not supported. 
 *		EDQUOT			Reserved. 
 *		EIDRM			Identifier removed. 
 *		EILSEQ			Illegal byte sequence. 
 *		ELOOP			Too many levels of symbolic links. 
 *		EMULTIHOP		Reserved. 
 *		ENOLINK			Reserved. 
 *		ENOTSUP			Not supported. 
 *		EOVERFLOW		Value too large to be stored in data type. 
 *		ESTALE			Reserved. 
 *		ETXTBSY			Text file busy. 
 *	
 */

#ifndef ZKERRNO_H_
#define ZKERRNO_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#if !defined(ZK_KERNEL)
	/* standard headers */
#	if defined(__MWERKS__) && defined(macintosh)
		/* Compensate for broken errno implementation in CW Pro 6 beta */
#		include <ansi_parms.h>
#		if __MSL__ < 0x6000
#			include <errno.h>
#		elif __MSL__ > 0x6007
#			error unknown version of the MSL -- check out its errno support
#		endif
#	else
#		include <errno.h>
#	endif
#else
#	include "zkKrtErrno.h"
#endif

#ifdef __cplusplus
	extern "C" {
#endif


#if defined(__MWERKS__) && defined(macintosh)
	
#	if !defined(ZK_KERNEL) && !defined(__OPENTRANSPORT__)
	
		/* The values below are set up to be the same as the one defined 
		in OpenTransport & MPW */
	
#		define E2BIG			7
#		define EACCES			13
#		define EADDRINUSE		48
#		define EADDRNOTAVAIL	49
#		define EAGAIN			11
#		define EALREADY			37
#		define EBADF			9
#		define EBADMSG			73
#		define EBUSY			16
#		define ECANCELED		74	/* OT calls this ECANCEL */
#		define ECHILD			10
#		define ECONNABORTED		53
#		define ECONNREFUSED		61
#		define ECONNRESET		54
		/* EDEADLK conflicts with another errno value for CodeWarrior */
#		define EDESTADDRREQ		39
		/* EDOM is defined by CodeWarrior */
#		define EEXIST			17
#		define EFAULT			14
#		define EFBIG			27
#		define EHOSTDOWN		64
#		define EHOSTUNREACH		65
#		define EINPROGRESS		77
#		define EINTR			4
#		define EINVAL			22
#		define EIO				5
#		define EISCONN			56
#		define EISDIR			21
#		define EMFILE			24
#		define EMLINK			31
#		define EMSGSIZE			40
		/* ENAMETOOLONG not defined by CodeWarrior -- NEED VALUE */
#		define ENETDOWN			50
#		define ENETRESET		52
#		define ENETUNREACH		51
#		define ENFILE			23
#		define ENOBUFS			55
#		define ENODATA			76
#		define ENODEV			19
#		define ENOENT			2
#		define ENOEXEC			8
		/* ENOLCK not defined by CodeWarrior -- NEED VALUE */
#		define ENOMEM			12
#		define ENOMSG			79
#		define ENOPROTOOPT		42
#		define ENORSRC			3		/* non-standard -- but defined by OT */
#		define ENOSPC			81
#		define ENOSR			72
#		define ENOSTR			75
#		define ENOSYS			ENORSRC	/* fake it */
#		define ENOTCONN			57
#		define ENOTDIR			20
		/* ENOTEMPTY not defined by CodeWarrior -- NEED VALUE */
#		define ENOTSOCK			38
#		define ENOTTY			25
#		define ENXIO			6
#		define EOPNOTSUPP		45
#		define EPERM			1
#		define EPIPE			32
#		define EPROTO			70
#		define EPROTONOSUPPORT	43
#		define EPROTOTYPE		41
		/* ERANGE is defined by CodeWarrior */
#		define EROFS			30
#		define ESHUTDOWN		58
#		define ESOCKTNOSUPPORT	44
#		define ESPIPE			29
#		define ESRCH			78
#		define ETIME			71
#		define ETOOMANYREFS		59
#		define ETIMEDOUT		60
		/* EWOULDBLOCK conflicts with another errno value for CodeWarrior */
#		define EXDEV			18

#		if __MSL__ >= 0x6000
#			define ENOERR		0
#			define EDOM			33
#			define ERANGE		34
#			define EFPOS		35
#			define ESIGPARM		36
#		endif

#	endif	/* !defined(ZK_KERNEL) && !defined(__OPENTRANSPORT__) */

#	if __MSL__ >= 0x6000
		_MSL_IMP_EXP_C	__tls extern int errno;
		/* Getting even more sneaky ... prevent MSL's errno.h from being included */
#		define _ERRNO_H
#		define __cerrno__
#	endif

#endif	/* defined(__MWERKS__) && defined(macintosh) */


#ifdef __cplusplus
	}
#endif

#endif	/* ZKERRNO_H_ */
