#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
/*
 * Mutex manipulation
 *
 * $Id: zkMutex.h,v 1.3 2001/04/19 17:26:48 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Daniel Faguy <danielf@zeroknowledge.com>

 */
#ifndef ZK_MUTEX_H_
#define ZK_MUTEX_H_

#include "zks.h"
#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkError.h"
#include "zkipc.h"


/*
** name maximum length
*/
# define  ZKMTX_MAXNAME_LENGTH      ZKIPC_MAXNAME_LENGTH

#ifdef __cplusplus

class RMutex
{
protected:
  RMutex( ) { } ;

public:
  virtual ~RMutex( ) { } ;

  virtual bool take( )                  = 0 ;
  virtual bool tryTake( bool *pbGotIt ) = 0 ;
  virtual bool give( )                  = 0 ;
} ;


// The following class does not provide priority inheritance

#include "zksem.h"

class RInterProcessMutex : public RMutex
{
  int    m_count ;
  ZkSem  m_sem ;

public:
  RInterProcessMutex( const char  *pName = NULL ) ;
  virtual ~RInterProcessMutex( ) ;

  virtual bool  take( ) ;
  virtual bool  tryTake( bool *pbGotIt ) ;
  virtual bool  give( ) ;

  static bool cleanup( const char *pName ) ;
} ;

extern "C"
{

#endif // ifdef __cplusplus

/*
** ZKMTX handle
*/
typedef  struct S_zkMtx  *ZkMtx ; // Generic mutex handle


/*
** Exportable C calls
*/

DLLExport ZkBool  zkInterProcessMutexNew( ZkMtx       *pMtx,
                                          const char  *name ) ;

DLLExport ZkBool  zkInterProcessMutexFree( ZkMtx  *pMtx ) ;

DLLExport ZkBool  zkMtxTake( ZkMtx  mtx ) ;

DLLExport ZkBool  zkMtxTryTake ( ZkMtx    mtx,
                                 ZkBool  *pSuccess ) ;

DLLExport ZkBool  zkMtxGive( ZkMtx  mtx ) ;

DLLExport ZkBool  zkInterProcessMutexCleanup ( const char  *pName ) ;


#ifdef __cplusplus

} ;

#endif // ifdef __cplusplus
#endif // ifndef ZK_MUTEX_H_
#endif // if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
