/*
 *	zkNetdb.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkNetdb.h,v 1.2 2001/04/19 17:26:48 alainc Exp $
 *	
 *	Wrapper for <netdb.h>
 *	
 *	This file defines at least the following symbols:
 *	
 *	Macros:
 *		HOST_NOT_FOUND		Error value for zkGethostbyname and zkGethostbyaddr.
 *		NO_DATA				Error value for zkGethostbyname and zkGethostbyaddr.
 *		NO_RECOVERY			Error value for zkGethostbyname and zkGethostbyaddr.
 *		TRY_AGAIN			Error value for zkGethostbyname and zkGethostbyaddr.
 *		
 *	Types:
 *		
 *		struct hostent, containing at least:
 *			char*			h_name
 *			char**			h_aliases
 *			int				h_addrtype
 *			int				h_length
 *			char**			h_addr_list
 *		
 *		struct protoent, containing at least:
 *			char*			p_name
 *			char**			p_aliases
 *			int				p_proto
 *	
 *		struct servent, containing at least:
 *			char*			s_name
 *			char**			s_aliases
 *			int				s_port
 *			char*			s_proto
 *
 *	Functions:
 *		struct hostent*		zkGethostbyaddr(const void *addr, size_t len, int type);
 *		struct hostent*		zkGethostbyname(const char *name);
 *		struct protoent*	zkGetprotobyname(const char *name);
 *		struct protoent*	zkGetprotobynumber(int proto);
 *		struct servent*		zkGetservbyname(const char *name, const char *proto);
 *		struct servent*		zkGetservbyport(int port, const char *proto);
 *		// not in Unix 98:
 *		int					zkHErrno(void);		// wrapper for h_errno
 */

#ifndef ZKNETDB_H_
#define ZKNETDB_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if defined(WIN32)

	/* Win32 */
	
/* #	include <winsock2.h>	NEEDS IMPLEMENTATION */

#elif defined(macintosh)
	
	/* Mac OS */
	
#	include "MacSockLib.h"

#elif !defined(macintosh)
	
	/* UNIX */
	
#	include <netdb.h>

#endif

#if !defined(ZK_KERNEL)

#ifdef __cplusplus
	extern "C" {
#endif


/* Wrapper Function Declarations */

DLLExport struct hostent*	zkGethostbyaddr(const void *addr, size_t len, int type);
DLLExport struct hostent*	zkGethostbyname(const char *name);
DLLExport struct protoent*	zkGetprotobyname(const char *name);
DLLExport struct protoent*	zkGetprotobynumber(int proto);
DLLExport struct servent*	zkGetservbyname(const char *name, const char *proto);
DLLExport struct servent*	zkGetservbyport(int port, const char *proto);
DLLExport int				zkHErrno(void);


#ifdef __cplusplus
	}
#endif

#endif	/* !defined(ZK_KERNEL) */

#endif	/* ZKNETDB_H_ */
