/*
 *	zkNetinetIn.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkNetinetIn.h,v 1.2 2001/04/19 17:26:48 alainc Exp $
 *	
 *	Wrapper for <netinet/in.h>
 *	
 *	This file defines at least the following symbols:
 *	
 *	Macros:
 *		IPPROTO_IP			Dummy for IP. 
 *		IPPROTO_ICMP		Control message protocol. 
 *		IPPROTO_TCP			TCP. 
 *		IPPROTO_UDP			User datagram protocol. 
 *		
 *		INADDR_ANY			Local host address. 
 *		INADDR_BROADCAST	Broadcast address. 
 *		
 *	Types:
 *		in_port_t			An unsigned integral type of exactly 16 bits. 
 *		in_addr_t			An unsigned integral type of exactly 32 bits. 
 *		
 *		struct in_addr, containing at least:
 *			in_addr_t		s_addr
 *	
 *		struct sockaddr_in, containing at least:
 *			sa_family_t		sin_family
 *			in_port_t		sin_port
 *			struct in_addr	sin_addr
 *			unsigned char	sin_zero[8]
 *	
 */

#ifndef ZKNETINETIN_H_
#define ZKNETINETIN_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#if defined(WIN32)

	/* Win32 */
	
#	include <winsock2.h>

#elif defined(macintosh) && !defined(ZK_KERNEL)
	
	/* Mac OS -- user space */
	
#	include "MacSockLib.h"
	
#elif defined(macintosh) && defined(ZK_KERNEL)
	
	/* Mac OS -- kernel space */
	
#	include "zkKrtOTKernel.h"
#	include "MacSockLib.h"
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
#	include <sys/types.h>
#	include <netinet/in.h>
	
#else
	
	/* UNIX */
	
#	include <netinet/in.h>

#endif

#ifdef __cplusplus
	extern "C" {
#endif


#if defined(WIN32)
	
	/* u_long is the type of in_addr.s_addr */
	typedef u_long		in_addr_t;

	/* u_short is the type of sockaddr_in.sin_port */
	typedef u_short		in_port_t;

#elif defined(ZK_LINUX) || defined(__APPLE__)

	/* unsigned int is the type of in_addr.s_addr */
	typedef unsigned int		in_addr_t;

	/* unsigned short int is the type of sockaddr_in.sin_port */
	typedef unsigned short int	in_port_t;
	
#endif


#ifdef __cplusplus
	}
#endif

#endif	/* ZKNETINETIN_H_ */
