/*
 * $Id: zkParms.h,v 1.95 2001/04/23 17:20:23 alainc Exp $
 */

#ifndef ZKPARMS_H_ 
#define ZKPARMS_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Standard section names
 */

#define S_CLIENT_KEYQRY      "KeyQueryClient"
#define S_CLIENT_KEYUPD      "KeyUpdateClient"
#define S_CLIENT_MAIP        "MAIPClient"
#define S_CLIENT_NYMSRV      "NymClient"
#define S_CLIENT_SERIALNUM   "SerialNumberClient"
#define S_CLIENT_TOKEN       "TokenClient"
#define S_CLIENT_AMTP        "AmtpClient"
#define S_CLIENT_NNTP        "NNTPClient"
#define S_CLIENT_LODE        "LodeClient"

/* For the stat update packets which are generated by some of the servers */
#define S_CLIENT_STAT        "StatClient"
#define S_SERVER_STAT        "StatServer"

#define S_SERVER_FMG         "FMG"
#define S_SERVER_FMGMAIP     "FMG-MAIP"
#define S_SERVER_IDENT       "IdentServer"
#define S_SERVER_KEYQRY      "KeyQueryServer"
#define S_SERVER_KEYUPD      "KeyUpdateServer"
#define S_SERVER_MAIP        "MAIP"
#define S_SERVER_NNTP        "NNTPServer"
#define S_SERVER_NYM         "NymServer"
#define S_SERVER_RNDDATA     "RandomDataServer"
#define S_SERVER_SERIALNUM   "SerialNumberServer"
#define S_SERVER_TOKEN       "TokenServer"
#define S_SERVER_QCD         "QueryCacheServer"
#define S_SERVER_LAIP        "LocalAipDaemon"
#define S_SERVER_PNEWS       "PostNews"
#define S_SERVER_LODE        "Lode"
#define S_DATABASE           "Database"
#define S_COMMAND_SERVER     "CommandServer"
#define S_DISPATCHER         "Dispatcher"

/* For NIDB servers */
#define S_SERVER_NISS        "NISS" 
/* For the niqs queries which are generated by some of the servers */
#define S_CLIENT_NIQS        "NiqsClient"
#define S_SERVER_NIQS        "NiqsServer"
#define S_DB_SYNC            "NdbRepl"

#define S_DAEMON_KEYCHANGE   "KeyChangeDaemon" 

/* For various tools */ 
#define S_TOOL_NYMTOOL     "NymTool" 
#define S_TOOL_NIDBTOOL    "NIDBTool" 

/* for the UDP packet replicator */
#define S_SERVER_REPLIC       "ReplicServer"


/* This server type does nothing but is used as a test 
 * configuration for zkkeyd 
 */
#define S_SERVER_ZKIP        "ZKIP"


#define S_MOD_WORMHOLE       "Wormhole"
#define S_MOD_HOSTINFO       "LocalHostInfo"
#define S_MOD_AIP            "Aip"
#define S_MOD_NIDB           "NIDB"
#define S_MOD_FRADMIN        "FreedomAdmin"

/*
 * Standard base parameter names
 */

#define P_REFUSE_CONNECTIONS_SIGNAL    "RefuseConnectionsSignal"

/* ROUTE CREATE ACK timeout value (secs) */
#define P_ACKWAITTIME                  "ackWaitTime"

/* NNTPServer newsgroups serving files: active newsgroups */
#define P_ACTIVE_FILE                  "activeFile" 

/* NNTPServer newsgroups serving files: newsgroup creation time/date */
#define P_ACTIVE_TIME                  "activeTimes" 

/* Name of an audit file, used by the zkAudit library */
#define P_AUDITFILE                    "auditFile"
  
/* Name of an authentication file, used by the zkAuth library */
#define P_AUTHFILE                     "authFile"

/* Remaps a name to a real email address at the FMG filter daemon.  used to 
 * trap things like postmaster or abuse. 
 */ 
#define P_ALIAS                        "alias" 

/* The name of the alternate text file which is sent through the maips.  This
 * text is displayed by a MIME compliant mailer if the message did not pass 
 * through the Freedom client POP proxy 
 */
#define P_ALTERNATE_TEXT               "alternateText" 
#define P_TAGLINE_FILE_TEXT            "tagLineTextFile" 
#define P_TAGLINE_FILE_HTML            "tagLineHtmlFile" 

#define P_AUDIO_DEVICE                 "audioDevice"
#define P_AUDIO_INPUT                  "audioInput"
#define P_AUTO_CHANGE_KEY              "autoChangeKey" 

/*
 * To specify a serial number activation date
 */

#define P_ACTIVATE                     "activate"

/* This is the base wormhole port
 */
#define P_BASE_PORT                    "basePort" 

/* Identify outgoing (destination) ports to block in the wormhole.  */
#define P_BLOCK_PORT                   "blockPort"

#define P_SUPPRESS_ARP                 "suppressArp"

/* This is set to YES if you wish the wormhole to clean up its ACI maps when
 * a new AIP client connects to it
 */
#define P_CLEAN_ON_CONNECT             "cleanOnConnect" 

/* The name of the configuration file
 */
#define P_CONFIG                       "config"

/* Behaviour of a program to POST or to SPOOL mail2news */
#define P_POST                         "post" 

#define P_DAEMON                       "daemon" 
#define P_DATABASE_NAME                "databaseName" 
#define P_DB_REOPEN_TIME               "dbReopenTime" 
#define P_DB_REOPEN_WAIT               "dbReopenWait" 

/* Set to yes if you wish to enable debug mode  */
#define P_DEBUG                        "debug" 
/* Don't fork children */
#define P_NOCHILD                      "nochildren"
/* Use numeric IP addresses instead of resolvinf the hostname (dispatcher) */
#define P_NUMERIC                      "numeric"

/* IPC specific parameters 
 */
#define P_IPC                          "ipc"
#define P_IPC_DIRECTORY                P_IPC ".directory"
#define P_IPC_SHM                      P_IPC ".shm"
#define P_IPC_SHM_SIZE                 P_IPC_SHM ".size"

/* Maximum number of packet received per iteration */
#define P_PG_PKT_PER_ITERATION         "pgPktPerIteration"


/* Don't fork children */
#define P_NODNS                        "noDNS"

/* Indicate that you don't want signatures */
#define P_NOSIGS                        "noSigs" 

/* Controls local connection only on the query cache daemon */
#define P_LOCAL                        "local"
/* Controls paranoid mode in the query cache daemon */
#define P_PARANOID                     "paranoid"
/* Control the number of concurrent clients which frqcd can spawn */
#define P_CLIENTS                      "clients"
/* Upper bound on the cached Time to Live in the query cache server */
#define P_CACHE_TTL                    "cacheTTL"
#define P_CACHE_TTL_FAIL               "cacheTTLFail"

/*
 * The data version code of the NIQS database.  If the niqs data is ever
 * mangled beyond repair, we increase the value of this parameter in the
 * freedom-frniqsd.conf file by 1, thus forcing all clients to flush their
 * cache and reload the new data.
 */

#define P_CACHE_DATA_VERSION           "cacheDataVersion"

#define P_DEVICE                       "device" 

/* The end wormhole output port
 */
#define P_END_PORT                     "endPort" 

/* Entity name to be used for the local server */
#define P_ENTITY_NAME                  "entityName" 
#define P_BANNER_FILE                  "bannerFile"

/* Boolean value to daemonize oneself */
#define P_FORK                         "fork" 
#define P_DONTFORK                     "dontFork"
#define P_HACKS                        "hacks" 

/* The number of datagrams which the wormhole fragmentation cache can
 * manage concurrently
 */
#define P_FRAG_CACHE_SIZE              "fragCacheSize"

#define P_MAX_MAIL_RECIPIENTS          "maxMailRecipients"

/* For the various client sections, the hostname of the target server */
#define P_HOSTNAME                     "hostName" 

/* Run the server for inetd, ie, attach input and output to the stdin and
 * stdout streams.
 */
#define P_INETD                        "inetd" 

/* 
 * These go into your server's own configuration section
 */

#define P_KEYINFO   "keyInfo"
#define P_KEYINFO_DIRECTORY_PFX    P_KEYINFO ".directory"
#define P_KEYINFO_LIFETIME_PFX     P_KEYINFO ".lifetime"
#define P_KEYINFO_WINDOW_PFX       P_KEYINFO ".window"
#define P_KEYINFO_FORCE            P_KEYINFO ".force" 
#define P_KEYINFO_AUTH_KEY_DIR     P_KEYINFO ".authKeyDir" 
#define P_KEYINFO_DIRECTORY_CRYPT  P_KEYINFO_DIRECTORY_PFX ".crypt" 
#define P_KEYINFO_DIRECTORY_SIG    P_KEYINFO_DIRECTORY_PFX ".sig" 
#define P_KEYINFO_LIFETIME_CRYPT   P_KEYINFO_LIFETIME_PFX ".crypt" 
#define P_KEYINFO_SIGNAL           P_KEYINFO ".signal" 
#define P_KEYINFO_WINDOW_CRYPT     P_KEYINFO_WINDOW_PFX ".crypt" 

/*
 * These go into the S_CLIENT_KEYQRY section of your config file
 */

#define P_KEYQRY          "keyQuery"
#define P_KEYQRY_HOSTNAME P_KEYQRY ".hostname"  /* Key query server host */
#define P_KEYQRY_RETRIES  P_KEYQRY ".retries"   /* Key query retries */
#define P_KEYQRY_PORT     P_KEYQRY ".port"      /* Key query server port */
#define P_KEYQRY_TIMEOUT  P_KEYQRY ".timeout"   /* Key quert server timeout */

/*
 * For NNTP client parameters.
 */
#define P_NNTP            "nntp"
#define P_NNTP_HOSTNAME   P_NNTP ".hostname"     /* NNTP server hostname */
#define P_NNTP_RETRIES    P_NNTP ".retries"      /* NNTP server retries */
#define P_NNTP_PORT       P_NNTP ".port"         /* NNTP server port */
#define P_NNTP_TIMEOUT    P_NNTP ".timeout"      /* NNTP server timeout */
#define P_NNTP_ORG        P_NNTP ".organization" /* NNTP Organization: */
#define P_NNTP_XHEADER    P_NNTP ".xheader"   /* NNTP X-ZKS-Freedom-Comment: */
#define P_NNTP_PATH       P_NNTP ".path"         /* NNTP Path: */
#define P_NNTP_ADVERT     P_NNTP ".advertFile"   /* NNTP file blurb */
#define P_NNTP_CROSSPOST_LIMIT P_NNTP ".crossPostLimit" /* Cross-Post Limit */

/*
 * Parameters for some of the admin tools
 */
 
#define P_PARM        "parameter"           /* Request a parameter name */
#define P_PROGRAM     "program"             /* Specify a program name */

/*
 * Parameters for key update clients. These go in the S_CLIENT_KEYUPD section
 */

#define P_KEYUPD           "keyupd"
#define P_KEYUPD_HOSTNAME  P_KEYUPD ".hostname" /* Key update server host */
#define P_KEYUPD_PORT      P_KEYUPD ".port"     /* Key update server port */
#define P_KEYUPD_TIMEOUT   P_KEYUPD ".timeout"  /* Key update srvr timeout */

/*
 *  These go into the S_CLIENT_MAIP section of your config file.  Mainly for
 *  use by the FMG.
 */

#define P_MAIP          "maip"
#define P_MAIP_HOSTNAME P_MAIP ".hostname"  /* Maip host */
#define P_MAIP_PORT     P_MAIP ".port"      /* Maip port */
#define P_MAIP_TIMEOUT  P_MAIP ".timeout"   /* Maip timeout */

/*
 * This is used by the database verification programs to limit the number
 * of errors reported before aborting.
 */

#define P_MAXERRORS         "maxErrors"

/*
 * Parameters used by the NIDB
 */

#define P_NIDB                  "nidb"
#define P_NIDB_TOPO             P_NIDB  ".topologyDB"
#define P_NIDB_STATE            P_NIDB  ".stateDB"
#define P_NIDB_NEIGHBOUR        P_NIDB  ".neighbourDB"
#define P_NIDB_ARCHIVE          P_NIDB  ".archiveDB"
#define P_NIDB_ENT              P_NIDB  ".entityDB"
#define P_NIDB_ENTID            P_NIDB  ".entityIdentityDB"
#define P_NIDB_ENTUPD           P_NIDB  ".entityUpdateDB"
#define P_NIDB_HOST             P_NIDB  ".hostDB"  
#define P_NIDB_ARCHIVE_SQLHOST  P_NIDB  ".archiveSqlHost"
#define P_NIDB_ARCHIVE_SQLUSER  P_NIDB  ".archiveSqlUserame"
#define P_NIDB_ARCHIVE_SQLPSWD  P_NIDB  ".archiveSqlPassword"
#define P_NIDB_ARCHIVE_SQLDB    P_NIDB  ".archiveSqlDB"

/*
 *  These go into the S_CLIENT_NYMSRV section of your config file.
 */

#define P_NYMSRV          "nymsrv"
#define P_NYMSRV_HOSTNAME P_NYMSRV ".hostname"  /* Key query server host */
#define P_NYMSRV_PORT     P_NYMSRV ".port"      /* Key query server port */
#define P_NYMSRV_TIMEOUT  P_NYMSRV ".timeout"   /* Key quert server timeout */

/* Tells the AIP where the wormhole launcher program is.  Must be a FULL
 * path name.
 */
#define P_LAUNCHER_PATH                "launcherPath" 

#define P_LISTEN_QUEUE_LENGTH          "listenQueueLength" 
#define P_LOCAL_ADDRESS                "localAddress" 

/* Unix SHIM device name (usually /dev/zkshim) */
#define P_LOCAL_DEVICE                 "localDevice" 

#define P_DEBUGFILE                    "debugFile"

/* Specify the log file.  This should be a full pathname
 */
#define P_LOGFILE                      "logFile" 

/* The maximum of time that a ZKIP server will remain connected to a client
 */
#define P_MAX_CONNECTION_TIME          "maxConnectionTime" 

/* Maximum number of simultaneous connections on a ZKIP server
 */
#define P_MAX_CONNECTIONS              "maxConnections" 

/* The maximum amount of data which can be transfered for a given ZKIP command.
 * This is a server side switch.  A server should ignore this value if it is
 * set to 0.
 */
#define P_MAX_DATA                     "maxData" 

/* The maximum IP packet size that the wormhole can accept (and the AIP as well)
 */
#define P_MAX_PACKET_SIZE              "maxPacketSize" 

/* Parameters to set the AIP internal table sizes.
 */
#define P_MAX_ROUTES                   "maxRoutes" 
#define P_MAX_NEIGHBOURS               "maxNeighbours" 

/*
 * The servers which record data (Nym server, Key update server) can specify
 * this file name to keep a recoverable file which contains all creates and
 * updates.
 */

#define P_RECORD_FILE                   "recordFile"

/*
 * Note on wormhole parameter naming:
 * Incoming refers to the data stream going from the internet to the wormhole
 * Outgoing refers to the data stream goinh from the wormhole to the internet
 */
/* The maximum segment size of TCP packets going in and out of the wormhole
 * The wormhole will automatically reset the MSS of a TCP SYN packet to this
 * value whenever one is encountered.
 */
#define P_MAX_SEGMENT_SIZE_INCOMING    "MSSIncoming" 
#define P_MAX_SEGMENT_SIZE_OUTGOING    "MSSOutgoing" 

/*
 * Not currently used, for specifying TCP window field max size in the
 * wormhole for traffic going in both directions.
 */
#define P_MAX_WINDOW_SIZE_INCOMING     "MaxWindowSizeIncoming"
#define P_MAX_WINDOW_SIZE_OUTGOING     "MaxWindowSizeOutgoing"

/*
 * For message of the day files.  Currently only used by the NIQS
 */

#define P_MOTDFILE "motdFile"

/*
 * For access control list files.  Currently only used by the NIQS
 */
#define P_ACLFILE "aclFile"

/*
 * ACL configuration parameters (and section)
 */
#define S_ACL "ACL%d"
#define P_BLOCK_TCP_RANGE "blockTcpPortRange"
#define P_BLOCK_UDP_RANGE "blockUdpPortRange"
#define P_ALLOW_TCP_RANGE "allowTcpPortRange"
#define P_ALLOW_UDP_RANGE "allowUdpPortRange"
#define P_BLOCK_TCP "blockTcpPort"
#define P_BLOCK_UDP "blockUdpPort"
#define P_ALLOW_TCP "allowTcpPort"
#define P_ALLOW_UDP "allowUdpPort"

/*
 * For pre-forking servers, the maximum number of connections the process
 * should handle berfore terminating
 */

#define P_PROCESS_LIFE    "processLife"

/*
 *  Number of bytes per second that a wormhole should accept from the 
 *  internet.
 */

#define P_THROUGHPUT_INCOMING          "throughputIncoming"

/* When set to yes, the wormhole will bounce unsolicited traffic back to 
 * the sender.  This is mostly a debugging mode
 */
#define P_MIRROR_MODE                  "mirrorMode" 

/*
 * Network device that the wormhole will listen to or send on for packets
 */
#define P_NETWORK_DEVICE               "networkDevice" 

/* The file which contains a message sent back by the FMG Filter when a mail
 * comes in for a non-existant or disabled nym
 */
#define P_NOT_FOUND_MSG                "notFoundMsg"
#define P_ALIAS_MSG                    "aliasMsg"
#define P_SENDER_MAIL_ADDR             "senderMailAddress"


#define P_NUM_COPIES                   "numCopies"
#define P_NUM_RESPONSES                "numResponses" 
#define P_PIDFILE                      "pidFile"

/* 
 * Specifying the passphrase parameters
 */
#define P_PASSPHRASE_FILE              "passPhrase.file"
#define P_PASSPHRASE_PROMPT            "passPhrase.prompt"
#define P_PASSPHRASE_FD                "passPhrase.fd"
#define P_PASSPHRASE_INFO_FILE         "passPhrase.info"

/* Specify a port number for a client or server
 */
#define P_PORT                         "port"

#define P_MAXNBPORT                    "maxNbPort"

#define P_PROTOCOL                     "protocol"

#define P_RANDFILE                     "randFile"

/*
 * Enable queueing mode (this is for the dispatcher)
 */

#define P_QUEUE                        "queue"

/*
 *  The configuration file in which frkeychgd will look for key to update.  By
 *  default this is the standard configuration file.
 */
#define P_PROXY_CONFIG_FILE            "proxyConfigFile"
#define P_ONCE                         "once"

/* The real IP address of a wormhole's or AIP's host computer
 */
#define P_REAL_ADDRESS                 "realAddress"

/* The base port for the wormhole to redirect traffic to the local host for
 * processing 
 */
#define P_REDIRECT_BASE_PORT           "redirectBasePort"

/* The number of seconds in which a local host redirection map will live in
 * the wormhole.  Generally this should be around 5 minutes
 */
#define P_REDIRECT_DURATION            "redirectDuration"

/* The end (highest) wormhole redirection port
 */
#define P_REDIRECT_END_PORT            "redirectEndPort"


#define P_RETRY_SLEEP                  "retrySleep"

#define P_ROOT                         "RootDirectory"

/* Tells a maip to run as an FMG MAIP.  This will make it refuse reply block
 * targets and instead require nym names or internet email addresses.
 */
#define P_RUN_AS_FMG                   "runAsFMG" 

#define P_RUNDIR                       "runDir"

/* Full path of the local sendmail binary
 */
#define P_MTA_PATH                     "mtaPath"
#define P_MTA_OPTIONS                  "mtaOptions"
#define P_SERVER_NAME                  "serverName"

/* Identd server pathname (full path required) */
#define P_SERVER_PATH                  "serverPath"

/* Single user mode flag, used by the lode server to only allow one user at
 * a time when working with the various types of database.  This is a boolean
 * value */
#define P_SINGLEUSER                   "singleUser"

/* A port redirection map so that the wormhole can send traffic to known
 * ports (eg identd 113) to a server on a local machine
 */
#define P_SERVICE_REDIRECT             "serviceRedirect"
#define P_SOCK_RETRIES                 "sockRetries"

#define P_FRAGMENTATION_SIZE           "fragmentationSize"

#define P_RECIPIENT_COUNT_DB           "recipientCountDatabase"

/*
 * Database which holds the email and domain address to which a nym cannot
 * send
 */

#define P_NYM_BLOCK_DB                 "nymBlockDatabase"

/*
 *  The maximum number of processes the MAIP will spawn, to pass on messages
 *  in the spool directory.
 */
#define P_MAX_SPOOL_PROCS              "maxSpoolProcs"

/*
 *  The amount of time the spooler proccess sleep between examinations of the
 *  spool directory.
 */
#define P_SPOOL_SLEEP_TIME             "spoolSleepTime"
  
/*
 *  The maximum number of messages processed on each queue run.
 */
#define P_SPOOL_MXMSGPLOOP             "spoolMaxMsgPerLoop"

/* A MAIP spool directory for storing message waiting to be forwarded
 */
#define P_SPOOL_DIR                    "spoolDir"

/* Maximum lifetime of a mail message on the spool
 */
#define P_SPOOL_MAIL_LIFETIME          "spoolMailLifeTime"

/*
 * Statistics parameters for the S_CLIENT_STAT section
 * The StatClient section will be renamed by March 19th, 1999
 */
#define P_STAT           "stat"
#define P_STAT_HISTORY   P_STAT ".history"   /* How many old entries to keep */
#define P_STAT_SERVER    P_STAT ".hostname"  /* The target stat server host */
#define P_STAT_PORT      P_STAT ".port"      /* The target stat server port */
#define P_STAT_FREQUENCY P_STAT ".frequency" /* Seconds between updates */
#define P_STAT_UPDPORT   P_STAT ".myport"    /* Stat client update request
                                                listening port */

/*
 * In the process of renaming
 * All renaming should be done by March 19th, 1999
 */
#define P_NISS           "stat"
/* #define P_NISS           "nissd" */
#define P_NISS_HISTORY   P_NISS ".history"   /* How many old entries to keep */
#define P_NISS_SERVER    P_NISS ".hostname"  /* The target stat server host */
#define P_NISS_CMDPORT   P_NISS ".port"      /* The target stat server port */
#define P_NISS_FREQUENCY P_NISS ".frequency" /* Seconds between updates */
#define P_NISS_UPDPORT   P_NISS ".myport"    /* Stat client update request
                                                listening port */

/*
 * Niqs client parameters in S_CLIENT_NIQS
 */
#define P_NIQS           "niqs"
#define P_NIQS_SERVER    P_NIQS ".hostname"  /* Target NIQS server hostname */
#define P_NIQS_PORT      P_NIQS ".port"      /* Target NIQS server port */
#define P_NIQS_TIMEOUT   P_NIQS ".timeout"   /* ZKIP timeout value */
#define P_NIQS_FREQUENCY P_NIQS ".frequency" /* Frequency of updates */
#define P_NIQS_FULLUPDATE P_NIQS ".fullUpdate" /* Do a completely new update
                                                * every X seconds. (0 disables
                                                * this in the frNetInfo 
                                                */
#define P_NIQS_SHARED_MEM_SIZE P_NIQS ".sharedMemSize"

/* Templates for the filenames for shared memory and mutexes */
#define P_NIQS_SHMTEMPLATE P_NIQS ".shmtemplate"
#define P_NIQS_MUTEXTEMPLATE P_NIQS ".mutextemplate"

#define P_NIQS_DUMP_NETINFO_ON_UPDATE P_NIQS ".dumpNetInfoOnUpdate"  /* boolean specifying if
                                                            a dump of the NetInfo should be
                                                            done everytime it is updated */

/*
 * This is defined for the server.  It indicates the name of the database
 * where it should keep its cache of signed responses.  The TTL indicates
 * how long (in seconds) an entry should live.  Always keep this number low
 * (maximum a few minutes)
 */

#define P_NIQS_RESPONSE_CACHE "responseCache"

#define P_NIQS_RESPONSE_CACHE_TTL                 "responseCacheTTL"
#define P_NIQS_RESPONSE_LIST_SINCE_0_CACHE_TTL    P_NIQS_RESPONSE_CACHE_TTL ".ListSince0"
#define P_NIQS_RESPONSE_STATUS_SINCE_0_CACHE_TTL  P_NIQS_RESPONSE_CACHE_TTL ".StatusSince0"
#define P_NIQS_RESPONSE_AVAILABLE_CACHE_TTL       P_NIQS_RESPONSE_CACHE_TTL ".Available"

#define P_NIQS_RESPONSE_CACHE_CLEAN "responseCacheClean"

/*
 * Token server client parameters
 */

#define P_TOKSRV           "token"
#define P_TOKSRV_HOSTNAME  P_TOKSRV ".hostname" /* Token server hostname */
#define P_TOKSRV_TIMEOUT   P_TOKSRV ".timeout"/* Token server connect timeout */
#define P_TOKSRV_PORT      P_TOKSRV ".port"    /* Token server port */

/*
 * Traffic Shaper specific parameters 
 */
#define P_TRAFFICSHAPER                "ts"
#define P_TRAFFICSHAPER_LOGFILE        P_TRAFFICSHAPER "." P_LOGFILE

/* When set to yes, the wormhole will strip any UP options that it sees in
 * an IP datagram (to remove source routing etc...)
 */
#define P_STRIP_IP_HEADER              "stripIpHeader"

/* When set to yes, the wormhole will remove an TCP option in a TCP datagram
 */
#define P_STRIP_TCP_HEADER             "stripTcpHeader"

#define P_TCP_PORT                     "tcpPort"


/* Some (not all) Databases */

/* hack so databases lock the same to each zkkeyd instantiation. */
#define P_DATABASE "database"

/* Spent token database (used by nym server)  */
#define P_DATABASE_SPENTTOKEN   P_DATABASE ".spentToken"

/* key database (used by nym server) */
#define P_DATABASE_KEYS         P_DATABASE ".keydb"

/* name of partner database holding backups of disabled entries */
#define P_DATABASE_DISABLED     P_DATABASE ".nuked"

/* The name of the file which stores the token descriptions
 */
#define P_TOKEN_TYPE_ID                "tokenTypeID"
#define P_TS_HISTORY_FILE              "tsHistoryFile"
#define P_UDP_PORT                     "udpPort"
#define P_UNIX_DEVICE                  "unixDevice"

/* Set to yes if you wish the wormhole to run an ident server
 */
#define P_USE_IDENT      "useIdent"
#define P_PORTSPEC       "portSpec"    /* zkident port specification */
#define P_USER           "user"        /* User name to run zkidentd */
#define P_RESPONSE       "response"    /* Alternate response to NOUSER */
#define P_VERSION        "version"     /* Print out the version string */
#define P_COMMAND        "command"     /* zkidentd command name pipe file */
#define P_HELP           "help"        /* Display the help information */
#define P_SUPPRESS       "suppress"    /* Suppress the nym name */

#define P_CMDPORT        "cmdPort" /* Dispatcher command port */
#define P_ADDRESS        "address" /* An internet address (for dispatcher) */

/* The verbose value (a value between 0 and 100) which is given to a ZKIP
 * server to describe the level of logging
 */
#define P_VERBOSE                      "verbose"

/* The wormhole's IP address (this is NOT the machine IP address)
 */
#define P_WORMHOLE_ADDRESS             "wormholeAddress"
/* The wormhole config pathname which is used by the AIP to launch the wormhole
 */
#define P_WORMHOLE_CONFIG_FILE         "wormholeConfigFile"

#define P_WORMHOLE_CONNECT_TIME        "wormholeConnectTime"

/* The location of the wormhole binary (this must be a full pathname)
 */
#define P_WORMHOLE_PATH                "wormholePath"

/*
 *  Template file used by zkshownym.
 */
#define P_TEMPLATE_FILE                "templateFile"
#define P_NOT_FOUND_FILE               "notFoundFile"

/*
 *  File containing serial numbers (for the scalper).
 */

#define P_SERIAL_NUM_LOAD_FILE         "serialNumLoadFile"
#define P_COUNT                        "count"

/*
 *  Various lock files.
 */

#define P_LOCKFILE "lockFile"
#define P_LOCKFILE_CRYPT P_LOCKFILE ".crypt"

/*
 * Parameters for the simple access client for Lode server
 */
#define P_PASSPHRASE   "passphrase"
#define P_SERVICE      "service"
#define P_OPERATION    "operation"
#define P_TIMEOUT      "timeout"
#define P_TCP_TIMEOUT  "tcpTimeout"
#define P_UDP_TIMEOUT  "udpTimeout"
#define P_REASON       "reason"

#define P_TYPE         "type"

/*
 * TCP flag for the dispatcher
 */

#define P_TCP_NO_DELAY  "tcpNoDelay"

/*
 * Mail Certificate parameters (used by Nym Server).
 */
#define P_MAILCERT                 "mailcert"
#define P_MAILCERT_KEY_FILE        P_MAILCERT ".keyFile"
#define P_MAILCERT_LIFETIME        P_MAILCERT ".lifetime"
#define P_MAILCERT_VOLUME_LIMIT    P_MAILCERT ".volLimit"
#define P_MAILCERT_MAILBOX_LIMIT   P_MAILCERT ".mailboxLimit"
#define P_MAILCERT_CROSSPOST_LIMIT P_MAILCERT ".crossPostLimit"

/*
 * Parameter taht specifies to generate XML output
 */
#define P_XML          "xml"

/* Bug: This is a temporary parameter to get a filename */
#define P_TEMPDATAFILE "tempdatafile"

#define P_AIP_LOGFILE         "AipStat"
#define P_MAIP_LOGFILE        "MaipStat"
#define P_LINK_LOGFILE        "LinkStat"  
#define P_FMG_LOGFILE         "FMGStat"  
#define P_ENTITY_LOGFILE      "EntityStat"
#define P_WHORMHOLE_LOGFILE   "WhormHoleStat"


#define P_DO_LOGNAMES         "ColumnNameOutput"
#define P_COLUMN_DELIMITER    "ColumnDelimiter"
#define P_ROWDELIMITER        "RowDelimiter"
 
/* parameters used for the replications */
#define P_REPLIC "replication"
#define P_REPLIC_FREQUENCY P_REPLIC ".time" 
  /* database options parameters */
#define  P_RECOVERY             "recovery"
#define  P_CHECKPOINT           "checkpoint"
#define  P_MIN_CHECKPOINT_TIME  P_CHECKPOINT ".mintime"
#define  P_MIN_CHECKPOINT_SIZE  P_CHECKPOINT ".minsize"  
  
#define  P_CACHESIZE            "cachesize"
#define  P_PAGESIZE             "pagesize"
#define  P_LOGFILE_SIZE         "logfilesize" 
#define  P_HASHTABLE_SIZE       "hashtablesize"

#define  P_FLASH_LOG            "logflash"
#define  P_LOGFLASH_TIME        "logflashtime"  
#define  P_TRANSACTIONS_MODE    "transactionmode"  
#define  P_LOCK_MODE            "lockmode"
#define  P_KEYLOCK              "keylock"
#define  P_LOGDIR               "TransactLogDir"
#define  P_DATABASES            "Databases"
#define  P_HOME                 "HomeDir"     
#define  P_CHECKDEADLOCK        "CheckDeadlock"

#define  P_FATALRECOVERY      P_RECOVERY ".Fatal"   
#define   P_CHECKLOCK            "EarlyLockRecognition"
/*
** parameter used to specify the name of the master server
** (when more than one instance is running
*/
#define P_MASTER_HOSTNAME "MasterServer"


/*
 * this parameters are use particulary to get a aips (see:fr_getaips.c)
 */
#define P_LONG "long"
#define P_LOAD "load"
#define P_SAVE "save"
#define P_MAIP "maip"
#define P_CONSOLE "console"
#define P_NOUPDATE "noupdate"
#define P_AIPUPDATE "aipupdate"
#define P_FORCE  "force"

 /*
 * Parameter names as command line option strings passed to str(case)cmp()
 */

/*
 * Special command line arguments that shouldn't be used in the config file
 */
#define O_CONFIG_FILE           "--config"
#define O_HELP                  "--help"
#define O_VERSION               "--version"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKPARMS_H_ */
