/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKSIGNAL_H_
#define ZKSIGNAL_H_

#include "zks.h"

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

#include <signal.h>


/*
 *  Wrapper around POSIX sigset_t.
 */
typedef sigset_t ZkSignalSet;


/*
 *  Function type of signal handler.
 */
typedef void ZkSigFunc(int);


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

  extern DLLExport ZkSigFunc *zkSignal(int sig, ZkSigFunc *fn);
  extern DLLExport ZkSigFunc *zkSignalIntr(int sig, ZkSigFunc *fn);

  extern DLLExport int zkSignalSetAdd(ZkSignalSet *set, int sig);
  extern DLLExport int zkSignalSetInit(ZkSignalSet *set, int sig, ...);
  extern DLLExport int zkSignalProtectBegin(ZkSignalSet block, ZkSignalSet *restore);
  extern DLLExport int zkSignalProtectEnd(ZkSignalSet restore);

  extern DLLExport int zkSignalNumFromName(const char *name);
  extern DLLExport const char *zkSignalNameFromNum(int signo);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_LINUX || ZK_SOLARIS */

#endif /* ZKSIGNAL_H_ */
