/*
 *	zkStdio.h		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkStdio.h,v 1.3 2001/04/19 17:26:48 alainc Exp $
 *	
 *	Wrapper for <stdio.h>
 *	
 *	In addition to the symbols defined in <stdio.h> by the C standard, this 
 *	file defines at least the following symbols:
 *
 *	Macros:
 *		EOF
 *		NULL
 *		FOPEN_MAX
 *		SEEK_SET
 *		SEEK_CUR
 *		SEEK_END
 *		L_tmpnam
 *		BUFSIZ
 *		_IONBF
 *		_IOLBF
 *		_IOFBF
 *
 *	Types:
 *		FILE
 *		fpos_t
 *	
 *	Functions:
 *		int			zkFclose(FILE* fp);
 *		int			zkFflush(FILE* fp);
 *		int			zkFgetc(FILE* fp);
 *		int			zkFgetpos(FILE* fp, fpos_t* pos);
 *		char*		zkFgets(char* s, int n, FILE* fp);
 *		int			zkFileno(FILE* fp);
 *		int			zkFisatty(FILE* fp);
 *		FILE*		zkFopen(const char* name, const char* mode);
 *		int			zkFprintf(FILE* fp, const char* format, ...);
 *		int			zkFputc(int c, FILE* fp);
 *		int			zkFputs(const char* s, FILE* fp);
 *		size_t		zkFread(void* buff, size_t memb_size, size_t num_memb, FILE* fp);
 *		int			zkFseek(FILE* fp, long offset, int whence);
 *		int			zkFsetpos(FILE* fp, const fpos_t* pos);
 *		long		zkFtell(FILE* fp);
 *		size_t		zkFwrite(const void* buff, size_t memb_size, size_t num_memb, FILE* fp);
 *		int			zkPrintf(const char* format, ...);
 *		int			zkRemove(const char *path);
 *		int			zkRename(const char *oldpath, const char *newpath);
 *		void		zkRewind(FILE* fp);
 *		int			zkSnprintf(char* s, size_t n, const char* format, ...);
 *		int			zkSprintf(char* s, const char* format, ...);
 *		int			zkVfprintf(FILE* fp, const char* format, va_list args);
 *		int			zkVprintf(const char* format, va_list args);
 *		int			zkVsnprintf(char* s, size_t n, const char* format, va_list args);
 *		int			zkVsprintf(char* s, const char* format, va_list args);
 */

#ifndef ZKSTDIO_H_
#define ZKSTDIO_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if !defined(ZK_KERNEL)
	/* standard headers */
#	include <stdarg.h>
#	include <stdio.h>
#else
#	include "zkKrtStdio.h"
#endif

#ifdef __cplusplus 
	extern "C" {
#endif


/* Wrapper Function Declarations */

#if !defined(ZK_KERNEL)

DLLExport int		zkFclose(FILE* fp);
DLLExport int		zkFflush(FILE* fp);
DLLExport int		zkFgetc(FILE* fp);
DLLExport int		zkFgetpos(FILE* fp, fpos_t* pos);
DLLExport char*		zkFgets(char* s, int n, FILE* fp);
DLLExport int		zkFileno(FILE* fp);
DLLExport int		zkFisatty(FILE* fp);
DLLExport FILE*		zkFopen(const char* name, const char* mode);
DLLExport int		zkFprintf(FILE* fp, const char* format, ...);
DLLExport int		zkFputc(int c, FILE* fp);
DLLExport int		zkFputs(const char* s, FILE* fp);
DLLExport size_t	zkFread(void* buff, size_t memb_size, size_t num_memb, FILE* fp);
DLLExport int		zkFseek(FILE* fp, long offset, int whence);
DLLExport int		zkFsetpos(FILE* fp, const fpos_t* pos);
DLLExport long		zkFtell(FILE* fp);
DLLExport size_t	zkFwrite(const void* buff, size_t memb_size, size_t num_memb, FILE* fp);
DLLExport int		zkPrintf(const char* format, ...);
DLLExport int		zkRemove(const char *path);
DLLExport int		zkRename(const char *oldpath, const char *newpath);
DLLExport void		zkRewind(FILE* fp);
DLLExport int		zkSprintf(char* s, const char* format, ...);
DLLExport int		zkVfprintf(FILE* fp, const char* format, va_list args);
DLLExport int		zkVprintf(const char* format, va_list args);
DLLExport int		zkVsprintf(char* s, const char* format, va_list args);

#endif	/* !defined(ZK_KERNEL) */

DLLExport int		zkSnprintf(char* s, size_t n, const char* format, ...);
DLLExport int		zkVsnprintf(char* s, size_t n, const char* format, va_list args);


#ifdef __cplusplus 
	}
#endif

#endif /* ZKSTDIO_H_ */
