/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *	
 *	$Id: zkString.h,v 1.9 2001/04/23 18:12:49 alainc Exp $
 *	
 *	In addition to containing the prototypes of the zkString* functions, 
 *	this header is now also a wrapper for <string.h>.  Why?  Because that 
 *	way the header follows the same naming convention as the other 
 *	wrapper headers, for example <netdb.h> becomes "zkNetdb.h".
 *	
 *	This wrapper portion of this file defines at least the following symbols:
 *		
 *	Functions:
 *		int			strcasecmp(const char* s1, const char* s2);
 *		int			strncasecmp(const char* s1, const char* s2, size_t n);
 *		char*		strdup(const char* s);
 *		char*		strndup(const char* s, size_t n);
 *		int			zkStrcoll(const char *s1, const char *s2);
 *		const char*	zkStrerror(int errnum);
 *		const char*	zkStrerror_r(int errnum, char* s, size_t n);
 *		size_t		zkStrxfrm(char *s1, const char *s2, size_t n);
 */

#ifndef ZKSTRING_H_
#define ZKSTRING_H_

#if !defined(ZK_KERNEL)
#	if defined(ZK_LINUX)
#		ifndef _GNU_SOURCE
#			define _GNU_SOURCE			/* strndup */
#		endif
#		define _XOPEN_SOURCE_EXTENDED 1	/* strdup, strcasecmp, strncasecmp  */
#	elif defined(ZK_SOLARIS)
#		define _XOPEN_SOURCE			/* strdup, strcasecmp, strncasecmp  */
#		define _XOPEN_SOURCE_EXTENDED	/* strdup, strcasecmp, strncasecmp  */
#	endif
#endif	/* !defined(ZK_KERNEL) */

#include <string.h>

#ifndef ZKS_H_
#	include "zks.h"
#endif

#ifndef _ZKERROR_H
#	include "zkError.h"
#endif


/* ------------------ ZkString stuff ------------------ */

#ifdef __cplusplus 
	extern "C" {
#endif

typedef struct T_ZkString * ZkString;

DLLExport int zkStringNew (
  ZkError ,
  ZkString * );

DLLExport int zkStringFree (
  ZkError ,
  ZkString * );

int zkStringSet (
  ZkError ,
  ZkString s ,
  const unsigned char * );

DLLExport int zkStringAdd (
  ZkError ,
  ZkString s ,
  const unsigned char * );

#define zkStringAddStr(x,y,z) zkStringAdd(x,y,(const unsigned char *)z)

DLLExport  
int zkStringLength (
  ZkError ,
  ZkString ,
  unsigned int * );

DLLExport int zkStringData (
  ZkError ,
  ZkString ,
  const unsigned char ** );

DLLExport const unsigned char *zkStringGetData (
  ZkError ,
  ZkString );


#ifdef __cplusplus 
	}
#endif

/* ------------------ <string.h> wrapper stuff ------------------ */

#if !defined(ZK_KERNEL)

#	ifdef __cplusplus 
		extern "C" {
#	endif


#	if defined(WIN32)
	
	/* Win32 */
	
	DLLExport int		strcasecmp(const char* s1, const char* s2);
	DLLExport int		strncasecmp(const char* s1, const char* s2, size_t n);
	DLLExport char*		strndup(const char* s, size_t n);
	
#	elif defined(macintosh)
	
	/* Mac OS */

	DLLExport int		strcasecmp(const char* s1, const char* s2);
	DLLExport int		strncasecmp(const char* s1, const char* s2, size_t n);
	DLLExport char*		strdup(const char* s);
	DLLExport char*		strndup(const char* s, size_t n);
	
#	elif !defined(ZK_LINUX)
	
	/* Everything else except Linux */
	
	DLLExport char*		strndup(const char* s, size_t n);

#	endif	/* platform */

#	ifdef __cplusplus 
		}
#	endif

#else	/* !defined(ZK_KERNEL) */

#	include "zkKrtString.h"

#	ifdef __cplusplus 
		extern "C" {
#	endif

	DLLExport char*		strndup(const char* s, size_t n);

#	ifdef __cplusplus 
		}
#	endif
	
#endif	/* defined(ZK_KERNEL) */


#ifdef __cplusplus 
	extern "C" {
#endif

/* Wrapper Function Declarations */

DLLExport int			zkStrcoll(const char *s1, const char *s2);
#if !defined(ZK_KERNEL)
DLLExport const char*	zkStrerror(int errnum);
#endif	/* defined(ZK_KERNEL) */
DLLExport const char*	zkStrerror_r(int errnum, char* s, size_t n);
DLLExport size_t		zkStrxfrm(char *s1, const char *s2, size_t n);


/* Platform-specific declarations */

#if defined(macintosh)
	
	/* Mac OS */
	
	int					MacErr2Errno(long macErr);
	
#endif	/* defined(macintosh) */


#ifdef __cplusplus 
	}
#endif

#endif /* ZKSTRING_H_ */
