/*
 *	zkSysSelect.h		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysSelect.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <sys/select.h>
 *	
 *	This file defines at least the following symbols:
 *	
 *	Macros:
 *		FD_SETSIZE	Maximum number of file descriptors in an fd_set structure. 
 *		
 *	Types:
 *		
 *		type struct fd_set, containing at least:
 *			long	fds_bits[]
 *
 *	Functions:
 *		int			zkSelect(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 *		void		FD_CLR(fd, fdsetp)
 *		int			FD_ISSET(fd, fdsetp)
 *		void		FD_SET(fd, fdsetp)
 *		void		FD_ZERO(fdsetp)
 *		int			ZK_FD_MAXFD1(fd)
 *		int			ZK_FD_MAXFDS(fd_set *, fd_set *, fd_set *)
 *	
 */

#ifndef ZKSYSSELECT_H_
#define ZKSYSSELECT_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if !defined(ZK_KERNEL)

#if defined(WIN32)

	/* Win32 */
	
/* #	include <winsock2.h>	NEEDS IMPLEMENTATION */

#elif defined(macintosh)
	
	/* Mac OS */
	
#	include "zkSysTime.h"
#	include "MacSockLib.h"

#	undef FD_ZERO
#	undef FD_SET
#	undef FD_CLR
#	undef FD_ISSET
	
#	define FD_ZERO(set)			zkFdZero((set), FD_SETSIZE)	/* clear all entries in set */
#	define FD_SET(skt, set)		zkFdSet((skt), (set))		/* turn on the entry for fd in set */
#	define FD_CLR(skt, set)		zkFdClr((skt), (set))		/* turn off the entry for fd in set */
#	define FD_ISSET(skt, set)	zkFdIsSet((skt), (set))		/* is the entry for fd on in set ? */
	
	/* implementation specifics */
	DLLExport void	zkFdZero(fd_set* set, uint16_t maxentries);
	DLLExport void	zkFdSet(SOCKET skt, fd_set* set);
	DLLExport void	zkFdClr(SOCKET skt, fd_set* set);
	DLLExport int	zkFdIsSet(SOCKET skt, const fd_set* set);
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
#	include <unistd.h>

#else
	
	/* UNIX */
	
#	include <sys/select.h>

#endif

#ifdef __cplusplus
	extern "C" {
#endif


/* Wrapper Function Declarations */

DLLExport int	zkSelect(int nfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, struct timeval* timeout);
DLLExport int	ZK_FD_MAXFDS(fd_set* readfds, fd_set* writefds, fd_set* exceptfds);

#define ZK_FD_MAXFD1(fd)	((fd) + 1)


#ifdef __cplusplus
	}
#endif

#endif	/* !defined(ZK_KERNEL) */

#endif	/* ZKSYSSELECT_H_ */
