/*
 *	zkSysSocket.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysSocket.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <sys/socket.h>
 *	
 *	This file defines the following symbols:
 *	
 *	Macros:
 *		AF_UNSPEC		Unspecified
 *		AF_INET			Internet domain sockets
 *		
 *		SOCK_DGRAM		Datagram socket
 *		SOCK_STREAM		Byte-stream socket
 *		
 *		SHUT_RD			Disables further receive operations.
 *		SHUT_WR			Disables further send operations.
 *		SHUT_RDWR		Disables further send and receive operations.
 *		
 *		SOL_SOCKET		Options to be accessed at socket level, not protocol level.
 *		SO_DEBUG		Debugging information is being recorded.
 *		SO_KEEPALIVE	Connections are kept alive with periodic messages.
 *		SO_LINGER		Socket lingers on close.
 *		SO_REUSEADDR	Reuse of local addresses is supported.
 *
 *	Types:
 *		socklen_t		An unsigned opaque integral type of length of at least 32 bits.
 *		sa_family_t		An unsigned integral type. 
 *		
 *		struct sockaddr, containing at least:
 *			sa_family_t	sa_family
 *			char		sa_data[]
 *		
 *		struct linger, containing at least:
 *			int			l_onoff
 *			int			l_linger
 *		
 *	Functions:
 *		int		zkAccept(int sock, struct sockaddr *addr, socklen_t *len);
 *		int		zkBind(int sock, const struct sockaddr *addr, socklen_t len);
 *		int		zkConnect(int sock, const struct sockaddr *addr, socklen_t len);
 *		int		zkGetpeername(int sock, struct sockaddr *addr, socklen_t *len);
 *		int		zkGetsockname(int sock, struct sockaddr *addr, socklen_t *len);
 *		int		zkGetsockopt(int sock, int level, int opt, void *val, socklen_t *len);
 *		int		zkListen(int sock, int backlog);
 *		ssize_t	zkRecv(int sock, void *buff, size_t len, int flags);
 *		ssize_t	zkRecvfrom(int sock, void *buff, size_t len, int flags, struct sockaddr *addr, socklen_t *len);
 *		ssize_t	zkSend(int sock, const void *buff, size_t len, int flags);
 *		ssize_t	zkSendto(int sock, const void *buff, size_t len, int flags, const struct sockaddr *addr, socklen_t len);
 *		int		zkSetsockopt(int sock, int level, int opt, const void *val, socklen_t len);
 *		int		zkShutdown(int sock, int how);
 *		int		zkSocket(int domain, int type, int protocol);
 *		// not in Unix 98:
 *		int		zkSockClose(int sock);
 *		ssize_t	zkSockRead(int sock, void *buff, size_t len);
 *		ssize_t	zkSockWrite(int sock, const void *buff, size_t len);
 *	
 */

#ifndef ZKSYSSOCKET_H_
#define ZKSYSSOCKET_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if defined(WIN32)

	/* Win32 */
	
#	include <winsock2.h>
#	include "zkSysTypes.h"

#elif defined(macintosh)
	
	/* Mac OS */
	
#	include "MacSockLib.h"
#	include "zkSysTypes.h"
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
#	include <sys/types.h>		/* ssize_t */
#	include <sys/socket.h>

#else
	
	/* UNIX */
	
#	include <sys/types.h>		/* ssize_t */
#	include <sys/socket.h>

#endif

#ifdef __cplusplus
	extern "C" {
#endif


#if defined(WIN32)
	
	/* Win32 */
	
	typedef INT			socklen_t;
	typedef u_short		sa_family_t;
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
	typedef int			socklen_t;
	typedef u_short		sa_family_t;
	
#endif


/* Wrapper Function Declarations */

#if !defined(ZK_KERNEL)

DLLExport int		zkAccept(int sock, struct sockaddr *addr, socklen_t *len);
DLLExport int		zkBind(int sock, const struct sockaddr *addr, socklen_t len);
DLLExport int		zkConnect(int sock, const struct sockaddr *addr, socklen_t len);
DLLExport int		zkGetpeername(int sock, struct sockaddr *addr, socklen_t *len);
DLLExport int		zkGetsockname(int sock, struct sockaddr *addr, socklen_t *len);
DLLExport int		zkGetsockopt(int sock, int level, int opt, void *val, socklen_t *len);
DLLExport int		zkListen(int sock, int backlog);
DLLExport ssize_t	zkRecv(int sock, void *buff, size_t len, int flags);
DLLExport ssize_t	zkRecvfrom(int sock, void *buff, size_t len, int flags, struct sockaddr *addr, socklen_t *lenp);
DLLExport ssize_t	zkSend(int sock, const void *buff, size_t len, int flags);
DLLExport ssize_t	zkSendto(int sock, const void *buff, size_t len, int flags, const struct sockaddr *addr, socklen_t lenp);
DLLExport int		zkSetsockopt(int sock, int level, int opt, const void *val, socklen_t len);
DLLExport int		zkShutdown(int sock, int how);
DLLExport int		zkSocket(int domain, int type, int protocol);
DLLExport int		zkSockClose(int sock);
DLLExport ssize_t	zkSockRead(int sock, void *buff, size_t len);
DLLExport ssize_t	zkSockWrite(int sock, const void *buff, size_t len);

#endif	/* !defined(ZK_KERNEL) */


#ifdef __cplusplus
	}
#endif

#endif	/* ZKSYSSOCKET_H_ */
