/*
 *	zkSysTime.h		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysTime.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <sys/time.h>
 *	
 *	This file defines at least the following symbols:
 *		
 *	Types:
 *		
 *		tv_sec_t		Integral type expressing a number of 
 *						elapsed seconds since 00:00 Coordinated 
 *						Universal Time (UTC), January 1, 1970.
 *		
 *		struct timeval, containing at least:
 *			tv_sec_t	tv_sec
 *			suseconds_t	tv_usec
 *		
 *	Functions:
 *		int			zkGettimeofday(struct timeval* tv, void* junk);
 *	
 */

#ifndef ZKSYSTIME_H_
#define ZKSYSTIME_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if !defined(ZK_KERNEL)

#	if defined(WIN32)

		/* Win32 */
	
#		include <winsock2.h>

#	elif defined(macintosh)
	
		/* Mac OS */
	
#		include "zkTime.h"
#		include "zkIntTypes.h"
#		include "zkSysTypes.h"
	
#	elif defined(__APPLE__)
	
		/* Mac OS X */
	
#		include <sys/time.h>
	
#	else
	
		/* UNIX */
	
#		include <sys/time.h>

#	endif

#	ifdef __cplusplus
		extern "C" {
#	endif

#	if defined(macintosh)
	
		/* Mac OS */
		
		typedef unsigned long	tv_sec_t;
		
		struct timeval {
			tv_sec_t	tv_sec;
			suseconds_t	tv_usec;
		};
		
		extern void	zkMacTimeGetUTC(uint64_t* secs, uint16_t* fract);
		extern void	zkMacTimeUTCToLocal(uint64_t* secs, uint16_t* fract);
		extern void	zkMacTimeLocalToUTC(uint64_t* secs, uint16_t* fract);
		extern void	zkMacTimeUTCToUNIX(uint64_t* secs, uint16_t* fract);
		extern void	zkMacTimeUNIXToUTC(uint64_t* secs, uint16_t* fract);
		extern void	zkMacTimeUNIXToTimeval(uint64_t secs, uint16_t fract, 
							struct timeval* tv);
		extern void	zkMacTimeTimevalToUNIX(const struct timeval* tv, 
							uint64_t* secs, uint16_t* fract);
		extern long	zkMacTimeGetUTCDelta(int forceReadLoc);
		extern void	zkMacTimeFlushUTCCache(void);
	
#	elif defined(__APPLE__)

		typedef int32_t	tv_sec_t;

#	elif defined(ZK_LINUX)

		typedef time_t	tv_sec_t;

#	elif defined(ZK_SOLARIS)

		typedef time_t	tv_sec_t;

#	elif defined(WIN32)

		typedef long	tv_sec_t;

#	else

#		 error need definition for tv_sec_t

#	endif

#	ifdef __cplusplus
		}
#	endif

#else	/* !defined(ZK_KERNEL) */

#	include "zkKrtSysTime.h"

#endif	/* defined(ZK_KERNEL) */

#ifdef __cplusplus
	extern "C" {
#endif


/* Wrapper Function Declarations */

DLLExport int	zkGettimeofday(struct timeval* tv, void* junk);


#ifdef __cplusplus
	}
#endif

#endif	/* ZKSYSTIME_H_ */
