/*
 *	zkSysTypes.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysTypes.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <sys/types.h>
 *	
 *	This file defines at least the following symbols:
 *	
 *	Types:
 *		blkcnt_t	Used for file block counts.
 *		blksize_t	Used for block sizes.
 *		dev_t		Arithmetic type used for device numbers. 
 *		gid_t		Arithmetic type used for group IDs. 
 *		ino_t		Arithmetic type used for file serial numbers. 
 *		mode_t		Arithmetic type used for file attributes. 
 *		nlink_t		Arithmetic type used for link counts. 
 *		off_t		Signed Arithmetic type used for file sizes. 
 *		pid_t		Signed Arithmetic type used for process and process group IDs. 
 *		ssize_t		Arithmetic type used for a count of bytes or an error indication. 
 *		suseconds_t	A signed arithmetic type used for time in microseconds. 
 *		uid_t		Arithmetic type used for user IDs. 
 *		useconds_t	Integral type used for time in microseconds. 
 *
 */

#ifndef ZKSYSTYPES_H_
#define ZKSYSTYPES_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if defined(WIN32)

	/* Win32 */
	
#	if !defined(ZK_KERNEL)
#		include <sys\types.h>
#	else
#		error check if we need to include a header here ...
#	endif
	
#elif defined(macintosh)
	
	/* Mac OS */
	
#elif defined(__APPLE__)
	
	/* Mac OS X */
	
#	include <sys/types.h>

#else
	
	/* Linux / UNIX */
	
#	if !defined(ZK_KERNEL)
#		include <sys/types.h>
#	else
#		error check if we need to include a header here ...
#	endif

#endif

#ifdef __cplusplus
	extern "C" {
#endif


#if defined(WIN32)

	/* Win32 */
	
	typedef unsigned long	blkcnt_t;
	typedef unsigned long	blksize_t;
	typedef _dev_t			dev_t;
	typedef short			gid_t;
	typedef _ino_t			ino_t;
	typedef unsigned short	mode_t;
	typedef short			nlink_t;
	typedef _off_t			off_t;
	typedef int				pid_t;
	typedef int				ssize_t;
	typedef long			suseconds_t;
	typedef short			uid_t;
	typedef unsigned long	useconds_t;

#elif defined(macintosh)
	
	/* Mac OS */
	
	/* OpenTransportProtocol.h defines dev_t, gid_t, & uid_t 
	(as well as others when OTKERNEL is defined).  If you need 
	to #include <OpenTransportProtocol.h>, you should do so 
	*before* including this file. */
	
#	if defined(ZK_KERNEL)
#		include "zkKrtOTKernel.h"
#	endif

	typedef unsigned long	blkcnt_t;
	typedef unsigned long	blksize_t;
	typedef long			ino_t;
	typedef unsigned short	mode_t;
	typedef short			nlink_t;
	typedef long			off_t;
	typedef long long		pid_t;
	typedef long			ssize_t;
	typedef long			suseconds_t;
	typedef unsigned long	useconds_t;
#	if !defined(ZK_KERNEL) && !defined(__OPENTRANSPORTPROTOCOL__)
	typedef long			dev_t;
	typedef long			gid_t;
	typedef long			uid_t;
#	endif

#elif defined(__APPLE__)
	
	/* Mac OS X */
	
	typedef int64_t			blkcnt_t;
	typedef u_int32_t		blksize_t;
	typedef int32_t			suseconds_t;
	typedef int32_t			useconds_t;

#endif


#ifdef __cplusplus
	}
#endif

#endif	/* ZKSYSTYPES_H_ */
