/*
 *	zkTime.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkTime.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <time.h>
 *	
 *	This file defines at least the following symbols:
 *
 *	Macros:
 *		CLOCKS_PER_SEC
 *
 *	Types:
 *		clock_t
 *		time_t
 *		struct tm
 *	
 *	Functions:
 *		clock_t		clock(void);
 *		double		difftime(time_t t1, time_t t0);
 *		time_t		mktime(struct tm *tptr);
 *		size_t		strftime(char *s, size_t n, const char *format, const struct tm *tptr);
 *		time_t		time(time_t *tod);
 *		char*		zkAsctime_r(const struct tm *tptr, char *s);
 *		char*		zkCtime_r(const time_t *tod, char *s);
 *		struct tm*	zkGmtime_r(const time_t *tod, struct tm *tptr);
 *		struct tm*	zkLocaltime_r(const time_t *tod, struct tm *tptr);
 *	
 */

#ifndef ZKTIME_H_
#define ZKTIME_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if !defined(ZK_KERNEL)
#	include <time.h>
#else
#	include "zkKrtTime.h"
#endif

#ifdef __cplusplus 
	extern "C" {
#endif


/* Wrapper Function Declarations */

DLLExport char*			zkAsctime_r(const struct tm *tptr, char *s);
DLLExport char*			zkCtime_r(const time_t *tod, char *s);
DLLExport struct tm*	zkGmtime_r(const time_t *tod, struct tm *tptr);
DLLExport struct tm*	zkLocaltime_r(const time_t *tod, struct tm *tptr);


#ifdef __cplusplus 
	}
#endif

#endif	/* ZKTIME_H_ */
