/*
 *	zkUnistd.h	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkUnistd.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 *	
 *	Wrapper for <unistd.h>
 *	
 *	This file defines at least the following symbols:
 *	
 *	Functions:
 *		int			zkChdir(const char *path);
 *		int			zkClose(int fd);
 *		int			zkFtruncate(int fd, off_t length);
 *		int			zkGethostname(char* name, size_t len);
 *		pid_t		zkGetpid(void);
 *		int			zkIsatty(int fd);
 *		off_t		zkLseek(int fd, off_t offset, int whence);
 *		ssize_t		zkRead(int fd, void* buff, size_t n);
 *		unsigned int zkSleep(unsigned int seconds);
 *		int			zkTruncate(const char* path, off_t length);
 *		ssize_t		zkWrite(int fd, const void* buff, size_t n);
 */

#ifndef ZKUNISTD_H_
#define ZKUNISTD_H_

#if defined(__MWERKS__) || defined(_MSC_VER)
#	pragma once
#endif

#ifndef ZKS_H_
#	include "zks.h"
#endif

#if !defined(ZK_KERNEL)

#if defined(WIN32)

	/* Win32 */
	
#	include "zkSysTypes.h"

#elif defined(macintosh)
	
	/* Mac OS */
	
#	include "zkSysTypes.h"
	
#else
	
	/* UNIX */
	
#	if defined(ZK_LINUX)
#		define __USE_XOPEN	1			/* gethostname */
#	elif defined(ZK_SOLARIS)
#		define _XOPEN_SOURCE			/* gethostname */
#		define _XOPEN_SOURCE_EXTENDED	/* gethostname */
#	endif
#	include <unistd.h>
#	if defined(ZK_LINUX)
#		undef __USE_XOPEN
#	elif defined(ZK_SOLARIS)
#		undef _XOPEN_SOURCE
#		undef _XOPEN_SOURCE_EXTENDED
#	endif
#	include "zkSysTypes.h"

#endif

#ifdef __cplusplus
	extern "C" {
#endif

	
#if defined(macintosh)
	
	/* Mac OS */
	
	extern short	GetCurDirSpec(struct FSSpec* spec);
	extern short	SetCurDirSpec(const struct FSSpec* spec);

#endif

/* Wrapper Function Declarations */

DLLExport int		zkChdir(const char *path);
DLLExport int		zkClose(int fd);
DLLExport int		zkFtruncate(int fd, off_t length);
DLLExport int		zkGethostname(char* name, size_t len);
DLLExport pid_t		zkGetpid(void);
DLLExport int		zkIsatty(int fd);
DLLExport off_t		zkLseek(int fd, off_t offset, int whence);
DLLExport ssize_t	zkRead(int fd, void* buff, size_t n);
DLLExport unsigned int zkSleep(unsigned int seconds);
DLLExport int		zkTruncate(const char* path, off_t length);
DLLExport ssize_t	zkWrite(int fd, const void* buff, size_t n);


#ifdef __cplusplus
	}
#endif

#endif	/* !defined(ZK_KERNEL) */

#endif	/* ZKUNISTD_H_ */
