/*
 * $Id: zkclientconsole.h,v 1.3 2001/04/19 17:26:49 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKCLIENTCONSOLE_H_
#define ZKCLIENTCONSOLE_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#if !defined(ZK_KERNEL)

/*
 * Keep these in order of decreasing importance
 */

typedef enum {
  ClientConsoleMessageUrgent ,
  ClientConsoleMessageError ,
  ClientConsoleMessageWarning ,
  ClientConsoleMessage ,
  ClientConsoleMessageOther
} ZkClientConsoleMessage;


/*
 * Implement callbacks which match this prototype in order to get information
 * from some of the libzk/freedom objects displayed in the client console
 * Some of the libzk and freedom objects will have a function like
 * zkFooSetClientConsoleCallback(ZkError,foo,ZkClientConsoleCallback,extra)
 * which will allow you to set both the callback and the optional extra
 * data pointer.
 */

typedef int (* ZkClientConsoleCallback) ( 
  ZkError ,
  const char * ,             /* Prefix */
  ZkClientConsoleMessage ,   /* Message type */ 
  ConsSeverity ,      /* Importance of the message */
  const char * ,             /* String token indicating action */ 
  const char * ,             /* String containing short message */
  const char * ,             /* (optional) String containing long message */
  void * );                  /* Extra data pointer */

/*
 * A sample callback used for debugging, it writes to stderr on unix 
 */

DLLExport int zkClientConsoleFprintf (
  ZK_INOUT ZkError err ,
  ZK_IN    const char * prefix ,
  ZK_IN    ZkClientConsoleMessage  msg,
  ZK_IN    ConsSeverity lvl ,
  ZK_IN    const char * action ,
  ZK_IN    const char * msg_short ,
  ZK_IN    const char * msg_long ,
  ZK_IN    void * extra );           /* This is a pointer to an output stream */

#endif	/* !defined(ZK_KERNEL) */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKCLIENTCONSOLE_H_ */

