/*
 * Shared Memory manipulation
 *
 * $Id: zkipc.h,v 1.3 2001/04/19 17:26:49 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */
#ifndef ZK_IPC_H_
# define ZK_IPC_H_

#include "zks.h"
#include "zkIntTypes.h"
#include "zkError.h" 

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
** pathname maximum length
*/
#define ZKIPC_MAXNAME_LENGTH                               (1024U)

/*
** read/write permission mode
*/
#define ZKIPC_NONE_PMODE                                   ((uint32_t) 0x00000000UL)
#define ZKIPC_USER_READ_PMODE                              ((uint32_t) 0x00000001UL)
#define ZKIPC_USER_WRITE_PMODE                             ((uint32_t) 0x00000002UL)
#define ZKIPC_GROUP_READ_PMODE                             ((uint32_t) 0x00000004UL)
#define ZKIPC_GROUP_WRITE_PMODE                            ((uint32_t) 0x00000008UL)
#define ZKIPC_OTHER_READ_PMODE                             ((uint32_t) 0x00000010UL)
#define ZKIPC_OTHER_WRITE_PMODE                            ((uint32_t) 0x00000020UL)
#define ZKIPC_FULL_PMODE                                   ( ZKIPC_USER_READ_PMODE  | ZKIPC_USER_WRITE_PMODE   | \
                                                             ZKIPC_GROUP_READ_PMODE | ZKIPC_GROUP_WRITE_PMODE | \
                                                             ZKIPC_OTHER_READ_PMODE | ZKIPC_OTHER_WRITE_PMODE )

/*
** Other flags
*/
#define ZKIPC_PRIVATE_KEY                                   ((uint32_t) 0x80000000UL)


/*
** Exportable calls
*/
DLLExport int zkPathNameCreate
(
 ZkError     err,                      /* [in]   pointer on a ZkError */
 const char *pathname,                 /* [in]   unique string to identify the semaphore */
 uint32_t    pmode                     /* [in]   read/write permission mode */
);

DLLExport int zkPathNameRemove
(
 ZkError    err,                       /* [in]   pointer on a ZkError */
 const char *pathname                  /* [in]   unique string to identify the semaphore */
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_LINUX || ZK_SOLARIS */

#endif /* ZK_IPC_H_ */
