/*
 * $Id: zkserr.h,v 1.2 2001/04/19 17:26:49 alainc Exp $
 */
#ifndef ZKS_ERR_H
# define ZKS_ERR_H

#include <stdarg.h>

# include "zks.h"
# include "zkIntTypes.h"
# include "zkIntLimits.h"
# include "zkTypes.h"
# include "zkDefs.h"
# include "zkError.h"
# include "zkSysTime.h"
# include "zksmodule.h"

/*
** ZksErr 
*/
typedef struct S_ZksErr * ZksErr; 


# include "zkspid.h"


/*
** Error hadling and ModuleId section 
*/
# define ZKS_ERR_MOD                        "ZksErr"


/*
** Return code constants
*/
# define ZKS_FAILURE                        ZK_FALSE
# define ZKS_SUCCESS                        ZK_TRUE

/*
** ZksErr handling Limits
*/
# define ZKS_ERR_FUNCNAME_MAX               (63)
# define ZKS_ERR_MESG_MAX                   (127)
# define ZKS_ERR_STACK_MAX                  (64)


/*
** NONE Values
*/
# define ZKS_ERR_CODE_NONE                  ((uint32_t)0x00000000)
# define ZKS_ERR_MOD_ERR_NONE               ((uint16_t)0x0000)
# define ZKS_ERR_CATEGORY_NONE              ((uint16_t)0x0000)
# define ZKS_ERR_SYSERRCODE_NONE            ((uint16_t)0x0000)


/*
** Flags
*/
# define ZKS_ERR_NONE_FLAG                  ((uint16_t)0x0000)
# define ZKS_ERR_LOWMEMORY_FLAG             ((uint16_t)0x0001)
# define ZKS_ERR_TRANSLATED_FLAG            ((uint16_t)0x0002)
# define ZKS_ERR_SETTED_FLAG                ((uint16_t)0x0004)
# define ZKS_ERR_RESETTED_FLAG              ((uint16_t)0x0008)
# define ZKS_ERR_STACK_OVERFLOW_FLAG        ((uint16_t)0x0010)


/*
** ZksErrCode macros.
*/
# define ZKS_ERRCODE_SET(modId, modErr)     ( (((uint32_t)modId) << 16) | ((uint32_t)modErr & 0x0000ffffUL) )
# define ZKS_ERRCODE_ID(errcode)            ( (uint16_t) (((uint32_t)errcode & 0xffff0000UL) >> 16))
# define ZKS_ERRCODE_ERR(errcode)           ( (uint16_t)  ((uint32_t)errcode & 0x0000ffffUL) )


/*
** Function name declaration. 
*/
# define DECL_FUNC(name)                    static const char * func = name


/*
** ZksErr Code use to describe an ZksErr.
*/
typedef uint32_t                          ZksErrCode;




# ifdef __cplusplus
extern "C" {
# endif


  /* =========================================================================  
  ** System specific call.
  */

  ZkBool zksErrGetLastSysError              (uint32_t       * syserrP);

  ZkBool zksErrGetSysErrorMessage           (uint32_t         syserrcode, 
                                             char           * mesg, 
                                             uint32_t         mesgsz);



  /* =========================================================================  
  ** ZKS_ERRCODE API
  */

  ZksErrCode    zksErrCodeBuild             (const char     * modName, 
                                             uint16_t         modErr);



  /* =========================================================================
  ** ZksErr API
  */

  ZkBool       zksErrNew                     (ZksErr         * errP);

  void         zksErrFree                    (ZksErr         * errP);

  void         zksErrClear                   (ZksErr           err);

  ZkBool       zksErrSet                     (ZksErr           err, 
                                              const char     * modName, 
                                              uint16_t         modErr,
                                              const char     * func, 
                                              const char     * mesg, 
                                              ...);

  ZkBool       zksErrPush                    (ZksErr           err, 
                                              const char     * func, 
                                              const char     * mesg, 
                                              ...);

#if !defined(ZK_KERNEL)

  ZkBool       zksErrInfo                    (const ZksErr     err,
                                              struct timeval * errtimeP,
                                              T_ZksPid       * syspidP,
                                              ZksErrCode     * errcodeP,
                                              uint32_t       * syserrcodeP,
                                              uint16_t       * flagsP,
                                              char           * func,
                                              uint32_t         funcsz,
                                              char           * mesg,
                                              uint32_t         mesgsz);

#endif /*  !defined(ZK_KERNEL) */

  ZkBool       zksErrTranslate               (ZksErr           err,
                                              const char     * CriteriaModule,
                                              uint16_t         CriteriaCode,
                                              const char     * toModule,
                                              uint16_t         toErrcode,
                                              const char     * toFunc, 
                                              const char     * toMesg,
                                              ...);

  ZksErr       zksErrTmp                     (const ZksErr     err);

  ZksErrCode   zksErrCode                    (const ZksErr     err);

  const char * zksErrMessage                 (const ZksErr     err);

  uint16_t     zksErrFlags                   (const ZksErr     err);


  /* =========================================================================
  ** Extented API use to support category
  */

  ZkBool       zksErrCodeSet                 (ZksErr           err, 
                                              uint32_t         errcode,
                                              const char     * func, 
                                              const char     * mesg,
                                              ...);

  ZkBool       zksErrSetEx                   (ZksErr           err, 
                                              uint16_t         category,
                                              const char     * modName, 
                                              uint16_t         modErr, 
                                              const char     * func, 
                                              const char     * mesg, 
                                              ...);

  void         zksErrCategorySet             (const ZksErr     err, 
                                              uint16_t         category);

#if !defined(ZK_KERNEL)

  ZkBool       zksErrInfoEx                  (const ZksErr     err,
                                              uint16_t       * categoryP,
                                              struct timeval * errtimeP,
                                              T_ZksPid       * syspidP, 
                                              ZksErrCode     * errcodeP,
                                              uint32_t       * syserrcodeP,
                                              uint16_t       * flagsP,
                                              char           * func,
                                              uint32_t         funcsz,
                                              char           * mesg,
                                              uint32_t         mesgsz);

#endif /*  !defined(ZK_KERNEL) */

  ZkBool       zksErrTranslateEx             (ZksErr           err, 
                                              const char     * CriteriaModule,
                                              uint16_t         CriteriaCode,
                                              uint16_t         toCategory,
                                              const char     * toModule,
                                              uint16_t         toErrcode,
                                              const char     * toFunc, 
                                              const char     * toMesg,
                                              ...);


  ZksErr        zksErr                       (const ZkError err);

  ZkError       zkError                      (const ZksErr  err);


  /* =========================================================================
  ** ZksErr Message Stack API
  */
  ZkBool       zksErrMsgEmpty                (ZksErr           err);


  ZkBool       zksErrMsgPopHead              (ZksErr           err,
                                              struct timeval * errTimeP,
                                              char           * func,
                                              uint32_t         funcsz,
                                              char           * mesg,
                                              uint32_t         mesgsz);


  ZkBool       zksErrMsgPushTail             (ZksErr           err,
                                              struct timeval   errTime,
                                              const char     * func,
                                              const char     * mesg,
                                              ...);

  /* =========================================================================
  ** Utility calls use to allowe coexistance with old error handling skim
  ** and C++ exception handling.
  */

  ZkBool       zksErrCtxSet_valist           (ZksErr           err,
                                              ZksErrCode       errcode,
                                              const char     * func, 
                                              const char     * mesg, 
                                              va_list          vargs);

  ZkBool       zksErrCtxInit                 (ZksErr           err);

  ZkBool       zksErrCtxClone                (ZksErr           err, 
                                              const ZksErr     from);

# ifdef __cplusplus
}
# endif

#endif /* ZKS_ERR_H */
