/* **************************************************************************
** $Id: zksocket.h,v 1.15 2001/04/19 17:26:49 alainc Exp $
** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for a socket wrapper
** object and its associated API.
**
** **************************************************************************
*/

# ifndef zksocket_h_INCLUDED
# define zksocket_h_INCLUDED

# include "zkError.h"
# include "zksockettypes.h"
# include "zkList.h"
# include "zkSysTime.h"

#if !defined(ZK_KERNEL)

typedef struct T_ZkSocket *     ZkSocket;

typedef enum T_ZkSocketOption   ZkSocketOption;

# if defined (WIN32)
# include <winsock2.h>
# endif

# ifdef __cplusplus
extern "C" {
# endif


DLLExport int
parse_inet_addr(
    ZK_INOUT ZkError               error,
    ZK_IN    const char *          path,
    ZK_OUT   struct T_ZkInetAddr *   addr );

DLLExport
int zkSocketNew(
    ZK_INOUT ZkError                error,
    ZK_IN    int                    family,  /* AF_INET, AF_LOCAL */
    ZK_IN    int                    type,    /* SOCK_STREAM, SOCK_DGRAM */
    ZK_IN    int                    protocol, /* IPPROTO_xxx */
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketNewFromDescriptor(
    ZK_INOUT ZkError                error,
    ZK_IN    int                    family,  /* AF_INET, AF_LOCAL */
    ZK_IN    ZkSocketDescriptor     desriptor,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketFree(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkSocket *             out_socket );

DLLExport
int zkSocketGetRemoteAddress(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_OUT   const char * *         out_path );

DLLExport
int zkSocketAddressCompare(
    ZK_INOUT ZkError                error,
    ZK_IN    uint32_t               flags,
    ZK_IN    const char *           path1,
    ZK_IN    const char *           path2,
    ZK_OUT   ZkBool *               is_same );

DLLExport
int zkSocketSetOption(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    ZkSocketOption         option,
    ZK_IN    uint32_t               value );

DLLExport
int zkSocketBind(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    const char *           path );

DLLExport
int zkSocketListen(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s );

DLLExport
int zkSocketAccept(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               in_socket,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketAcceptWithTimeoutUpdate(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               in_socket,
    ZK_INOUT uint32_t *             io_timeout_in_secs,
    ZK_OUT   ZkSocket *             out_socket );

DLLExport
int zkSocketConnect(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_IN    const char *           path );

DLLExport
int zkSocketConnectAddr(
	ZK_INOUT	ZkError					error,
	ZK_IN		ZkSocket				s,
	ZK_IN		const ZkSocketAddress	address);

DLLExport
int zkSocketWaitReadableOrClosed(
	ZK_INOUT ZkError				error,
	ZK_IN	 ZkList 				list_of_sockets,
	ZK_IN    struct timeval			*timeout,
	ZK_OUT   int					*whichSocket,
    ZK_IN    int                    signal_fd);

DLLExport
int zkSocketRead(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT void *                 target_buffer,
    ZK_IN    uint32_t               max_bytes_to_read,
    ZK_OUT   uint32_t *             out_bytes_read );

DLLExport
int zkSocketReadAll(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT void *                 target_buffer,
    ZK_IN    uint32_t               bytes_to_read );

DLLExport
int zkSocketReadFrom(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT void *                 target_buffer,
    ZK_IN    uint32_t               max_bytes_to_read,
    ZK_OUT   uint32_t *             out_bytes_read,
    ZK_OUT   ZkSocketAddress *      out_src_address );

DLLExport
int zkSocketWrite(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT const void *           source_buffer,
    ZK_IN    uint32_t               max_bytes_to_write,
    ZK_OUT   uint32_t *             out_bytes_written );

DLLExport
int zkSocketWriteAll(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT const void *           source_buffer,
    ZK_IN    uint32_t               bytes_to_write );

DLLExport
int zkSocketWriteTo(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_INOUT const void *           source_buffer,
    ZK_IN    uint32_t               max_bytes_to_write,
    ZK_OUT   uint32_t *             out_bytes_written,
	ZK_IN    const ZkSocketAddress* dst_address);

DLLExport
int zkSocketGetFD(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkSocket               s,
    ZK_OUT   ZkSocketDescriptor*    fd);

DLLExport
int zkSocketParseAddr(
	ZK_INOUT ZkError                error,
    ZK_IN    const char *           path,
    ZK_IN    int *                  out_family,  /* AF_INET, AF_LOCAL */
	ZK_IN    ZkSocketAddress *      out_address);

# ifdef __cplusplus
}
# endif

#endif	/* !defined(ZK_KERNEL) */

# endif /* zksocket_h_INCLUDED */
