/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKSOCKET_TYPES_H__
#define ZKSOCKET_TYPES_H__

#include "zkNetinetIn.h"

#if defined(WIN32)
#include <winsock2.h>
#ifndef PATH_MAX
#define PATH_MAX 512
#endif
#define ZKSOCKET_PATH_MAX	PATH_MAX
#define ZKSOCKET_USE_LOCAL	0

#elif defined(macintosh)
#define ZKSOCKET_PATH_MAX	32
#define ZKSOCKET_USE_LOCAL	0

#elif defined(ZK_SOLARIS)
#define SOL_TCP INET_TCP
#include <sys/xti_inet.h>
#include <limits.h>
#include <sys/un.h>
#define ZKSOCKET_PATH_MAX	PATH_MAX
#define ZKSOCKET_USE_LOCAL	1

#else	/* Linux */
#include <netinet/tcp.h>
#include <sys/un.h>
#include <sys/param.h>
#define ZKSOCKET_PATH_MAX	PATH_MAX
#define ZKSOCKET_USE_LOCAL	1

#endif /* platform */

# if defined (WIN32)
typedef SOCKET                  ZkSocketDescriptor;
# else
typedef int                     ZkSocketDescriptor;
#ifndef INVALID_SOCKET
#define INVALID_SOCKET -1
#endif
# endif

#ifndef ZK_LINUX
#  define MSG_NOSIGNAL			0
#endif

enum T_ZkSocketOption {
    ZKSOCKET_OPTION_BlockSize = 1,
    ZKSOCKET_OPTION_Timeout,
    ZKSOCKET_OPTION_TimeoutAccept,
    ZKSOCKET_OPTION_TimeoutConnect,
    ZKSOCKET_OPTION_TimeoutRead,
    ZKSOCKET_OPTION_TimeoutWrite,
    ZKSOCKET_OPTION_QueueDepth,
    ZKSOCKET_OPTION_ReuseAddress,
    ZKSOCKET_OPTION_NoTransmitDelay,
    ZKSOCKET_OPTION_SendTimeout, /* Performance improvement by avoiding kernel transition overhead of the select call.*/
    ZKSOCKET_OPTION_RecvTimeout  /* Parm Values are in milliseconds when used in zkSocketSetOption.*/
								 /* Use ZKSOCKET_TIMEOUT_INFINITE for infinite timeout.*/
};

enum {
    ZKSOCKET_TIMEOUT_INFINITE = 0xffffffff,
    ZKSOCKET_CMP_INETADDR     = 1,
    ZKSOCKET_CMP_INETPORT     = 2
};


typedef struct T_ZkInetAddr {
    struct sockaddr_in          sa;       
    uint32_t                    mask;
} T_ZkInetAddr;



enum {
    ZKSOCKET_DFLT_BLOCK_SIZE      = 32768, /* read & write block size        */
    ZKSOCKET_DFLT_TIMEOUT_ACCEPT  = 45,    /* time allocated for accept      */
    ZKSOCKET_DFLT_TIMEOUT_CONNECT = 45,    /* read allocated for connect     */
    ZKSOCKET_DFLT_TIMEOUT_READ    = 45,    /* read time allocated per block  */
    ZKSOCKET_DFLT_TIMEOUT_WRITE   = 45,    /* write time allocated per block */
    ZKSOCKET_DFLT_QUEUE_DEPTH     = 5      /* connection queue depth         */
};

typedef union T_ZkSocketAddress ZkSocketAddress;

union T_ZkSocketAddress {
    struct sockaddr             m_gnrl;     /* general address                */
    struct T_ZkInetAddr         m_inet;     /* Internet domain socket address */  
#if ZKSOCKET_USE_LOCAL
    struct sockaddr_un          m_unix;     /* Unix domain socket address     */
#endif
};


#define CLOSE_OFF 0
#define CLOSE_ON  1
/* added by Frank Foster to not close link for Unix Domain sockets */
#define CLOSE_ON_KEEP_LINK 2

struct T_ZkSocket {
    ZkSocketDescriptor          descriptor;
    int                         close_descriptor_on_free;

    int                         family;
    ZkSocketAddress             local_addr;
    ZkSocketAddress             remote_addr;
    char                        remote_path[ZKSOCKET_PATH_MAX];

    /*
     * Options
     */
    uint32_t                    block_size;
    uint32_t                    timeout_accept;
    uint32_t                    timeout_connect;
    uint32_t                    timeout_read;
    uint32_t                    timeout_write;
    uint32_t                    queue_depth;
};



#endif	/* ZKSOCKET_TYPES_H__ */
