/*
 * Virtual Heap Handler
 *
 * $Id: zkvheap.h,v 1.6 2001/04/19 17:26:49 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */
#ifndef ZK_VHEAP_H_
#define ZK_VHEAP_H_


#include "zks.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zksem.h"

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
** ZkVHeap handle
*/
typedef struct S_ZkVHeap * ZkVHeap;



/*
** Creation flags
*/
#define ZKVHEAP_NONE_CREATION_FLAGS                        0x00  /* No flags */
#define ZKVHEAP_WIPE_FREE_SPACE_CREATION_FLAGS             0x01  /* Wipe free space */




/*
** Exportable prototypes
*/
DLLExport int zkVHeapNew
(
  ZkError     err,           /* [in]     pointer on a ZkError */
  ZkVHeap    *vheapP,        /* [in/out] pointer on a ZkVHeap */
  void       *memP,          /* [in]     pointer on a memory block to be partition */
  size_t      msz,           /* [in]     size (in bytes) of the memory block */
  ZkSem       mutex,         /* [in]     mutex use to synchronyze critical section (optional) */
  uint32_t    flags          /* [in]     optional flags */
);


int zkVHeapOpen
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap *vheapP,        /* [in/out] pointer on a ZkVHeap */
  void    *memP,          /* [in]     pointer on a memory block already partition */
  ZkSem    mutex          /* [in]     mutex use to synchronyze critical section (optional) */
);


DLLExport int zkVHeapFree 
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap *vheapP         /* [in/out] pointer on a ZkVHeap */
);


DLLExport int zkVHeapMemoryAlloc
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap  vheap,         /* [in]     ZkVHeap */
  void   **addrP,         /* [in/out] pointer on the allocated address space */
  uint32_t size           /* [in]     size (in bytes) to be allocate */
);


DLLExport int zkVHeapMemoryRealloc
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap  vheap,         /* [in]     ZkVHeap */
  void   **addrP,         /* [in/out] pointer on the allocated address space (may change) */
  uint32_t size           /* [in]     new size (in bytes) */
);


DLLExport int zkVHeapMemoryFree
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap  vheap,         /* [in]     ZkVHeap */
  void   **addrP          /* [in]     pointer on the allocated address space */
);


DLLExport int zkVHeapMemoryAvailable
(
  ZkError   err,           /* [in/out] pointer on a ZkError */
  ZkVHeap   vheap,         /* [in]     ZkVHeap */
  uint32_t *sizeP          /* [in]     size (in bytes) available */
);


DLLExport int zkVHeapSetHook
(
  ZkError  err,           /* [in]     pointer on a ZkError */
  ZkVHeap  vheap,         /* [in]     pointer on a ZkVHeap */
  char    *hook           /* [in]     pointer the extra data (may be NULL) */
);


DLLExport int zkVHeapGetHook
(
  ZkError  err,           /* [in] pointer on a ZkError */
  ZkVHeap  vheap,         /* [in] pointer on a ZkVHeap */
  char   **hookP          /* [out]  pointer the extra data */
);

DLLExport int zkVHeapCheckIntegrity
(
  ZkError  err,           /* [in] pointer on a ZkError */
  ZkVHeap  vheap,         /* [in] pointer on a ZkVHeap */
  ZkBool   quiet          
);


#ifdef __cplusplus
}
#endif /* __cplusplus */
    
#endif /* ZK_LINUX || ZK_SOLARIS */

#endif /* ZK_VHEAP_H_ */
