/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/**************************************************************************

   File:                zkxml.h

   Author(s):           Thierry Baron

   Created:             29 Sep 1999

   Last Revision:       $Date: 2001/04/19 17:26:49 $

   Description: 

   $Revision: 1.9 $
	
   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
   Please see the copyright notice included in this distribution 
   for full details.

 **************************************************************************/

#ifndef ZKXML_H
# define ZKXML_H

/**********************
     Include Files
 **********************/
#include "zks.h"
#include "zkError.h"
#include "zkIntTypes.h"
#include "zkxmlvalidtags.h"

#if !defined(ZK_KERNEL)

/**********************
     Public Defines
 **********************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/**********************
      Public Types
 **********************/

typedef struct T_zkXml* ZkXml;

typedef enum ZkXmlValidTags_en ZkXmlValidTags;

/********************
   Public Variables 
 ********************/

enum ZkXmlFlagSub {
    zkXmlFreeFlag,
    zkXmlPcdata,
    zkXmlSubPtr
};

DLLExport int
zkXmlNew(
    ZkError err,            /* Returns the error if any occured */
    ZkXml * xml
);

DLLExport int
zkXmlFree(
    ZkError err,            /* Returns the error if any occured */
    ZkXml * xml
);

DLLExport int
zkXmlPrintToBuf(
    ZkError  err,            /* Returns the error if any occured */
    ZkXml    xml,
    char **  buf,
    size_t * bufSize
);

DLLExport int
zkXmlAppendDataElement(
    ZkError        err,            /* Returns the error if any occured */
    ZkXml          xmlParent,
    ZkXmlValidTags tag,
    char *         value
);

  DLLExport int
zkXmlAppendDataLongElement(
    ZkError        err,            /* Returns the error if any occured */
    ZkXml          xmlParent,
    ZkXmlValidTags tag,
    int64_t        value
);

DLLExport int
zkXmlAppendSectionElement(
    ZkError err,            /* Returns the error if any occured */
    ZkXml   xmlParent,
    ZkXml   xmlChild
);

DLLExport int
zkXmlSetSectionTag(
    ZkError        err,    /* Returns the error if any occured */
    ZkXml          xml,
    ZkXmlValidTags tag
);

DLLExport int
zkXmlGetSectionTag(
    ZkError          err,           /* Returns the error if any occured */
    ZkXml            xml,
    ZkXmlValidTags * tag
);

/*
  DLLExport int
  zkXmlSetValue(
  ZkError err, 
  ZkXml   xml,
  void *value,
  size_t size
  );


  DLLExport extern int
  zkXmlSetTagAndValue(
  ZkError err,
  ZkXml   xml,
  enum ZkXmlValidTags  tag,
  char *value
  );
*/

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	/* !defined(ZK_KERNEL) */

#endif  /* ZKXML_H */

/***************************************************************************************

   $Log: zkxml.h,v $
   Revision 1.9  2001/04/19 17:26:49  alainc
   Merge from INT_FREEZE_MAC2_1_INT_20010417 to main trunk

   Revision 1.8.54.2  2001/01/25 03:18:36  paull
   Bringing in changes between tags BASE_SCARECROW_BETA2_0 and INT_SCARECROW_DEV_20010124_MAC2_1_INT to the mac2_1_int branch.

   Revision 1.8.54.1  2000/11/29 23:51:41  paull
   Bringing in changes from Freedom-Mac-Port branch.

   Revision 1.8.10.3  2000/05/29 16:45:30  paull
   Made code ZK_KERNEL-aware.  Functions that aren't suitable for a kernel environment (eg, fopen()) are omitted from the library when ZK_KERNEL is defined.

   Revision 1.8.10.2  2000/01/05 18:11:10  paull
   header wasn't self-contained

   Revision 1.8.10.1  2000/01/05 18:08:13  paull
   changed 'long long' argument to portable type int64_t.

   Revision 1.8  1999/10/06 20:57:34  thierry
   updated version of valid tags and minor change to zkXmlSetDataLongElement

   Revision 1.7  1999/10/06 15:47:31  thierry
   	* add zkXmlAppendDataLongElement
   	* fix unitialized pointer
   	* fix uninitialized value

   Revision 1.6  1999/10/05 14:13:12  louis
   get in sync...

   Revision 1.5  1999/10/05 13:51:56  thierry
   fixed exported func declares

   Revision 1.4  1999/10/03 04:38:08  build
   * Fixed broken build - Luc

   Revision 1.3  1999/10/01 16:23:28  thierry
   	* error checking on tagString
   	* error checkink on new element/data insert
   	* remove unused functions
   	* access to set/get xml element tag

   Revision 1.2  1999/09/30 21:58:02  thierry
   working version

   Revision 1.1  1999/09/30 03:08:18  louis
   addition of zkxml stuf, for xml output generation


*/
