/*
 *	zkArpaInet.c	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkArpaInet.c,v 1.2 2001/04/19 17:26:50 alainc Exp $
 *	
 *	Implementation of wrapper functions for <arpa/inet.h>
 *	
 */

#include "zkArpaInet.h"


#if !defined(ZK_KERNEL)

/* zkInet_addr -- wrapper for inet_addr */

#if defined(macintosh)

	/* Mac version -- inet_addr is implemented in terms of inet_aton */

DLLExport in_addr_t	zkInet_addr(const char *cp)
{
	return (MacSockInetAddr(cp));
}

#else

DLLExport in_addr_t	zkInet_addr(const char *cp)
{
	return (inet_addr(cp));
}

#endif


/* zkInet_ntoa -- wrapper for inet_ntoa */

#if defined(macintosh)

DLLExport char*	zkInet_ntoa(struct in_addr in)
{
	static char	sNToABuff[INET_ADDRSTRLEN];
	
	MacSockInetNToA(in, sNToABuff);
	
	return (sNToABuff);
}

#else

DLLExport char*	zkInet_ntoa(struct in_addr in)
{
	return (inet_ntoa(in));
}

#endif


/* zkInet_aton -- wrapper for inet_aton */

#if defined(WIN32)

	/* Win32 version.  This file belongs with the zklib.Dll project 
	and it is exported from that DLL only */
	
DLLExport int	zkInet_aton(const char* cp, struct in_addr* inp)
{
#if 0
	/* ### WE NEED TO INTEGRATE THIS WITH MFC WHEN WE MERGE THE FREEDOM-MAC-PORT 
	BRANCH BACK ONTO THE MAIN TRUNK. ### */
	_ASSERT(cp);
	_ASSERT( !IsBadWritePtr((LPVOID)inp, sizeof(*inp)) );
#endif

	inp->s_addr = inet_addr(cp);

	return(inp->s_addr != INADDR_NONE);
}

#elif defined(macintosh)

DLLExport int	zkInet_aton(const char* cp, struct in_addr* inp)
{
	return (MacSockInetAToN(cp, inp));
}

#else

DLLExport int	zkInet_aton(const char* cp, struct in_addr* inp)
{
	return (inet_aton(cp, inp));
}

#endif

#endif	/* !defined(ZK_KERNEL) */
