/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkCtype.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkDump.h"

#if !defined(ZK_KERNEL)

DLLExport int
zkDumpToStream (
  ZkError err ,
  void * cdata ,
  const char * data ,
  size_t size )
{
  zkErrorClear(err);
  if (cdata && data) {
    zkFwrite(data,1,size,(FILE *)cdata);
    zkFflush((FILE *)cdata);
  }
  return 1;
}

#endif	/* !defined(ZK_KERNEL) */

/**
 *  
 *
 */

DLLExport int
zkDumpBuffer (
  ZkError err, 
  const unsigned char * buffer ,
  size_t buffer_len ,
  ZkDumpCallback callback,
  void * cdata )
{
#define LINE_SZ 80         /* Characters in a line */
#define STEP_SZ 16         /* Number of octets per line */

  const char hexdigit[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
                              '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
  char line[LINE_SZ + 1];
  int n = 0;
  int k = 0;
  int end = 0;

  line[LINE_SZ] = (char) 0;
  if (buffer && callback && cdata) {
    for (n = 0; n < (int) buffer_len; n += STEP_SZ) {
      end = ((buffer_len - n) < STEP_SZ)?(buffer_len - n):STEP_SZ; 
      memset(line,(int) ' ',LINE_SZ);
      zkSnprintf(line, sizeof line, "%06d  ", n);
      for (k = 0; k < end; k++) {
        line[7 + k * 3] = (char) hexdigit[buffer[n + k] / 16];
        line[8 + k * 3] = (char) hexdigit[buffer[n + k] % 16];
        line[9 + k * 3] = ' ';
        if (isprint(buffer[n + k])) {
          line[STEP_SZ * 3 + 7 + k] = buffer[n + k];
        } else {
          line[STEP_SZ * 3 + 7 + k] = '.';
        }
      }
      line[STEP_SZ * 4 + 8] = '\n';
      line[STEP_SZ * 4 + 9] = '\0';
      callback(err,cdata,line,STEP_SZ * 4 + 9);
    }
    return 1;
  }
  zkErrorSet(err,"zkDumpBuffer",ZKERR,ZKMDUMP,ZKENULLPTR,NULL);
  return 0;
#undef LINE_SZ
}
