/**
 * @name zkLib
 *
 * An assortment of library calls that exists in one flavour of Unix
 * but not in another. 
 *
 * $Id: zkLib.c,v 1.6 2001/04/19 17:26:51 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * @author Luc Boulianne <luc@zks.net>

 *
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkLib.h"

#ifdef ZKMOD
# undef ZKMOD
#endif
#define ZKMOD ZKMLIB


const char *zkLibGetBuildRelease(ZkError err)
{
  (void) err;

#ifdef FREEDOM_DEVELOPMENT_BUILD
# define VERSION_STR "FREEDOM DEVELOPMENT BUILD ("__DATE__", "__TIME__")"
#elif defined(FREEDOM_PRODUCTION_BUILD)
# define VERSION_STR "FREEDOM PRODUCTION BUILD ("__DATE__", "__TIME__")"
#else
# define VERSION_STR "FREEDOM BUILD ("__DATE__", "__TIME__")"
#endif

  return (VERSION_STR);
}


#if defined(macintosh)

#	include <CodeFragments.h>
#	if !defined(ZK_KERNEL)
#		include "DCon.h"
#	endif

extern pascal OSErr	__initialize(const CFragInitBlock *initBlock);
extern pascal void	__terminate(void);


/* globals */
static FSSpec	gZksLibSpec;


/* CFM init & term procs suitable for applications such as testbeds */

pascal short	ZksAppInit(const struct CFragSystem7InitBlock *initBlock)
{
	gZksLibSpec	= *initBlock->fragLocator.u.onDisk.fileSpec;
	
#ifndef NDEBUG
	{
		char	name[sizeof(Str63)];
		size_t	len;
		
		len	= StrLength(gZksLibSpec.name);
		memcpy(name, gZksLibSpec.name+1, len);
		name[len] = '\0';
		
#	if !defined(ZK_KERNEL)
		dopen(name);
#	endif
	}
#endif
	
	return (noErr);
}

pascal void	ZksAppTerm(void)
{
}


/* CFM init & term procs suitable for shared libraries */

pascal short	ZksLibInit(const struct CFragSystem7InitBlock *initBlock)
{
	OSErr	err;
	
	err = __initialize(initBlock);
	
	if (err == noErr)
	{
		err = ZksAppInit(initBlock);
		
		if (err != noErr)
			__terminate();
	}
	
	return (err);
}

pascal void	ZksLibTerm(void)
{
	__terminate();
	ZksAppTerm();
}


/* returns the shared library's FSSpec */
const struct FSSpec*	ZksLibGetSpec(void)
{
	return (&gZksLibSpec);
}

#endif
