/*
 *	zkNetdb.c	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkNetdb.c,v 1.2 2001/04/19 17:26:51 alainc Exp $
 *	
 *	Implementation of wrapper functions for <netdb.h>
 */

#include "zkNetdb.h"


#if !defined(ZK_KERNEL)

/* globals */
#if defined(macintosh)
	
#	include <MacErrors.h>
	
	/* OSStatus-to-netdb error conversion */
	static int	StatusToHErrno(long error);
	
	/* globals */
	static long				sHErrno;
	static struct hostent	sHostent;
	static struct protoent	sProtoent;
	static struct servent	sServent;
	
#endif


/* zkGethostbyaddr -- wrapper for gethostbyaddr */

#if defined(macintosh)

DLLExport struct hostent*	zkGethostbyaddr(const void *addr, size_t len, int type)
{
	struct hostent*	entPtr;
	long			error;
	
	error = MacSockGetHostByAddr(addr, len, type, &sHostent);
	
	if (error)
	{
		entPtr	= NULL;
		sHErrno = StatusToHErrno(error);
	}
	else
	{
		entPtr	= &sHostent;
	}
	
	return (entPtr);
}

#else

DLLExport struct hostent*	zkGethostbyaddr(const void *addr, size_t len, int type)
{
	return (gethostbyaddr(addr, len, type));
}

#endif


/* zkGethostbyname -- wrapper for gethostbyname */

#if defined(macintosh)

DLLExport struct hostent*	zkGethostbyname(const char *name)
{
	struct hostent*	entPtr;
	long			error;
	
	error = MacSockGetHostByName(name, &sHostent);
	
	if (error)
	{
		entPtr	= NULL;
		sHErrno = StatusToHErrno(error);
	}
	else
	{
		entPtr	= &sHostent;
	}
	
	return (entPtr);
}

#else

DLLExport struct hostent*	zkGethostbyname(const char *name)
{
	return (gethostbyname(name));
}

#endif


/* zkGetprotobyname -- wrapper for getprotobyname */

#if defined(macintosh)

DLLExport struct protoent*	zkGetprotobyname(const char *protoname)
{
	struct protoent*	entPtr	= NULL;
	
	if (MacSockGetProtoByName(protoname, &sProtoent) == 0)
	{
		entPtr = &sProtoent;
	}
	
	return (entPtr);
}

#else

DLLExport struct protoent*	zkGetprotobyname(const char *name)
{
	return (getprotobyname(name));
}

#endif


/* zkGetprotobynumber -- wrapper for getprotobynumber */

#if defined(macintosh)

DLLExport struct protoent*	zkGetprotobynumber(int proto)
{
	struct protoent*	entPtr	= NULL;
	
	if (MacSockGetProtoByNumber(proto, &sProtoent) == 0)
	{
		entPtr = &sProtoent;
	}
	
	return (entPtr);
}

#else

DLLExport struct protoent*	zkGetprotobynumber(int proto)
{
	return (getprotobynumber(proto));
}

#endif


/* zkGetservbyname -- wrapper for getservbyname */

#if defined(macintosh)

DLLExport struct servent*	zkGetservbyname(const char *name, const char *proto)
{
	struct servent*	entPtr	= NULL;
	
	if (MacSockGetServByName(name, proto, &sServent) == 0)
	{
		entPtr = &sServent;
	}
	
	return (entPtr);
}

#else

DLLExport struct servent*	zkGetservbyname(const char *name, const char *proto)
{
	return (getservbyname(name, proto));
}

#endif


/* zkGetservbyport -- wrapper for getservbyport */

#if defined(macintosh)

DLLExport struct servent*	zkGetservbyport(int port, const char *proto)
{
	struct servent*	entPtr	= NULL;
	
	if (MacSockGetServByPort(port, proto, &sServent) == 0)
	{
		entPtr = &sServent;
	}
	
	return (entPtr);
}

#else

DLLExport struct servent*	zkGetservbyport(int port, const char *proto)
{
	return (getservbyport(port, proto));
}

#endif


/* zkHErrno -- wrapper for h_errno */

#if defined(macintosh)

DLLExport int	zkHErrno(void)
{
	return (sHErrno);
}

#else

DLLExport int	zkHErrno(void)
{
	return (h_errno);
}

#endif


#if defined(macintosh)

static int	StatusToHErrno(long error)
{
	switch (error)
	{
	case kOTBadNameErr:
		error = HOST_NOT_FOUND;
		break;
		
	case kOTNoDataErr:
		error = NO_DATA;
		break;
		
	default:
		error = NO_RECOVERY;
		break;
	}
	
	return (error);
}
	
#endif

#endif	/* !defined(ZK_KERNEL) */
