/*
 *	zkSysSelect.c		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysSelect.c,v 1.2 2001/04/19 17:26:51 alainc Exp $
 *	
 *	Implementation of wrapper functions for <sys/select.h>
 */

#include "zkErrno.h"
#include "zkString.h"
#include "zkSysSelect.h"
#include "zkSysTime.h"


#if !defined(ZK_KERNEL)

/* zkSelect -- wrapper for select */

#if defined(macintosh)

DLLExport int	zkSelect(int nfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, struct timeval* timeout)
{
#pragma unused(nfds)
	
	SOCKET_TIMEOUT	sockTimeout;
	int				numSel;
	long			error;
	
	if (timeout == NULL)
	{
		sockTimeout = SOCKET_WAITFOREVER;
	}
	else if ((timeout->tv_sec != 0) || (timeout->tv_usec != 0))
	{
		sockTimeout	= timeout->tv_sec;
		sockTimeout	*= 1000;
		sockTimeout	+= timeout->tv_usec / 1000;
	}
	else
	{
		sockTimeout = SOCKET_NOWAIT;
	}
	
	error = MacSockSelect(nfds, readfds, writefds, exceptfds, sockTimeout, &numSel);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		numSel = -1;
	}
	
	return (numSel);
}

#else

DLLExport int	zkSelect(int nfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, struct timeval* timeout)
{
	return (select(nfds, readfds, writefds, exceptfds, timeout));
}

#endif


/* ZK_FD_MAXFDS -- calculate 'nfds' argument to zkSelect() */

#if defined(macintosh) || defined(WIN32)

DLLExport int	ZK_FD_MAXFDS(fd_set* readfds, fd_set* writefds, fd_set* exceptfds)
{
	/* silence compiler */
	(void) readfds;
	(void) writefds;
	(void) exceptfds;
	
	return (0);
}

#else

DLLExport int	ZK_FD_MAXFDS(fd_set* readfds, fd_set* writefds, fd_set* exceptfds)
{
	fd_set	fakeReadfds, fakeWritefds, fakeExceptfds;
	int		i;
	
	if (readfds == NULL)
	{
		FD_ZERO(&fakeReadfds);
		readfds = &fakeReadfds;
	}
	
	if (writefds == NULL)
	{
		FD_ZERO(&fakeWritefds);
		writefds = &fakeWritefds;
	}
	
	if (exceptfds == NULL)
	{
		FD_ZERO(&fakeExceptfds);
		exceptfds = &fakeExceptfds;
	}
	
	for (i = FD_SETSIZE - 1; i >= 0; i--)
	{
		if (FD_ISSET(i, readfds) || FD_ISSET(i, writefds) || FD_ISSET(i, exceptfds))
		{
			return (i + 1);
		}
	}
	
	return (0);
}

#endif


/* FD_ZERO, FD_SET, FD_CLR, FD_ISSET -- socket set manipulation */

#if defined(macintosh)

DLLExport void	zkFdZero(fd_set* set, uint16_t maxentries)
{
	MacSockFdZero(set, maxentries);
}

DLLExport void	zkFdSet(SOCKET skt, fd_set* set)
{
	MacSockFdSet(skt, set);
}

DLLExport void	zkFdClr(SOCKET skt, fd_set* set)
{
	MacSockFdClr(skt, set);
}

DLLExport int	zkFdIsSet(SOCKET skt, const fd_set* set)
{
	return (MacSockFdIsSet(skt, set));
}

#endif

#endif	/* !defined(ZK_KERNEL) */
