/*
 *	zkSysSocket.c	(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkSysSocket.c,v 1.2 2001/04/19 17:26:52 alainc Exp $
 *	
 *	Implementation of wrapper functions for <sys/socket.h>
 */

#if defined(ZK_LINUX) || defined(ZK_SOLARIS) || defined(__APPLE__)
#	include <unistd.h>
#endif
#include "zkErrno.h"
#include "zkString.h"
#include "zkSysSocket.h"


#if !defined(ZK_KERNEL)

/* zkAccept -- wrapper for accept */

#if defined(macintosh)

DLLExport int	zkAccept(int sock, struct sockaddr *addr, socklen_t *len)
{
	SOCKET	newSock;
	long	error;
	
	error = MacSockAccept(sock, addr, len, &newSock, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		newSock = INVALID_SOCKET;
	}
	
	return (newSock);
}

#else

DLLExport int	zkAccept(int sock, struct sockaddr *addr, socklen_t *len)
{
	return (accept(sock, addr, len));
}

#endif


/* zkBind -- wrapper for bind */

#if defined(macintosh)

DLLExport int	zkBind(int sock, const struct sockaddr *addr, socklen_t len)
{
	long	error;
	
	error = MacSockBind(sock, addr, len);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkBind(int sock, const struct sockaddr *addr, socklen_t len)
{
	return (bind(sock, addr, len));
}

#endif


/* zkConnect -- wrapper for connect */

#if defined(macintosh)

DLLExport int	zkConnect(int sock, const struct sockaddr *addr, socklen_t len)
{
	long	error;
	
	error = MacSockConnect(sock, addr, len, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkConnect(int sock, const struct sockaddr *addr, socklen_t len)
{
	return (connect(sock, addr, len));
}

#endif


/* zkGetpeername -- wrapper for getpeername */

#if defined(macintosh)

DLLExport int	zkGetpeername(int sock, struct sockaddr *addr, socklen_t *len)
{
	long	error;
	
	error = MacSockGetPeerName(sock, addr, len);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkGetpeername(int sock, struct sockaddr *addr, socklen_t *len)
{
	return (getpeername(sock, addr, len));
}

#endif


/* zkGetsockname -- wrapper for getsockname */

#if defined(macintosh)

DLLExport int	zkGetsockname(int sock, struct sockaddr *addr, socklen_t *len)
{
	long	error;
	
	error = MacSockGetSockName(sock, addr, len);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkGetsockname(int sock, struct sockaddr *addr, socklen_t *len)
{
	return (getsockname(sock, addr, len));
}

#endif


/* zkGetsockopt -- wrapper for getsockopt */

#if defined(macintosh)

DLLExport int	zkGetsockopt(int sock, int level, int opt, void *val, socklen_t *len)
{
	long	error;
	
	error = MacSockGetSockOpt(sock, level, opt, val, len);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkGetsockopt(int sock, int level, int opt, void *val, socklen_t *len)
{
	return (getsockopt(sock, level, opt, val, len));
}

#endif


/* zkListen -- wrapper for listen */

#if defined(macintosh)

DLLExport int	zkListen(int sock, int backlog)
{
	long	error;
	
	error = MacSockListen(sock, backlog);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkListen(int sock, int backlog)
{
	return (listen(sock, backlog));
}

#endif


/* zkRecv -- wrapper for recv */

#if defined(macintosh)

DLLExport ssize_t	zkRecv(int sock, void *buff, size_t len, int flags)
{
	long	error;
	size_t	nread;
	ssize_t	result;
	
	error = MacSockRecv(sock, buff, len, flags, &nread, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nread;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkRecv(int sock, void *buff, size_t len, int flags)
{
	return (recv(sock, buff, len, flags));
}

#endif


/* zkRecvfrom -- wrapper for recvfrom */

#if defined(macintosh)

DLLExport ssize_t	zkRecvfrom(int sock, void *buff, size_t len, int flags, struct sockaddr *addr, socklen_t *lenp)
{
	long	error;
	size_t	nread;
	ssize_t	result;
	
	error = MacSockRecvFrom(sock, buff, len, flags, addr, lenp, &nread, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nread;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkRecvfrom(int sock, void *buff, size_t len, int flags, struct sockaddr *addr, socklen_t *lenp)
{
	return (recvfrom(sock, buff, len, flags, addr, lenp));
}

#endif


/* zkSend -- wrapper for send */

#if defined(macintosh)

DLLExport ssize_t	zkSend(int sock, const void *buff, size_t len, int flags)
{
	long	error;
	size_t	nwritten;
	ssize_t	result;
	
	error = MacSockSend(sock, buff, len, flags, &nwritten, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nwritten;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkSend(int sock, const void *buff, size_t len, int flags)
{
	return (send(sock, buff, len, flags));
}

#endif


/* zkSendto -- wrapper for sendto */

#if defined(macintosh)

DLLExport ssize_t	zkSendto(int sock, const void *buff, size_t len, int flags, const struct sockaddr *addr, socklen_t lenp)
{
	long	error;
	size_t	nwritten;
	ssize_t	result;
	
	error = MacSockSendTo(sock, buff, len, flags, addr, lenp, &nwritten, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nwritten;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkSendto(int sock, const void *buff, size_t len, int flags, const struct sockaddr *addr, socklen_t lenp)
{
	return (sendto(sock, buff, len, flags, addr, lenp));
}

#endif


/* zkSetsockopt -- wrapper for setsockopt */

#if defined(macintosh)

DLLExport int	zkSetsockopt(int sock, int level, int opt, const void *val, socklen_t len)
{
	long	error;
	
	error = MacSockSetSockOpt(sock, level, opt, val, len);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkSetsockopt(int sock, int level, int opt, const void *val, socklen_t len)
{
	return (setsockopt(sock, level, opt, val, len));
}

#endif


/* zkShutdown -- wrapper for shutdown */

#if defined(macintosh)

DLLExport int	zkShutdown(int sock, int how)
{
	long	error;
	
	error = MacSockShutdown(sock, how);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkShutdown(int sock, int how)
{
	return (shutdown(sock, how));
}

#endif


/* zkSocket -- wrapper for socket */

#if defined(macintosh)

DLLExport int	zkSocket(int domain, int type, int protocol)
{
	SOCKET	sock;
	long	error;
	
	error = MacSockSocket(domain, type, protocol, &sock);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		sock = INVALID_SOCKET;
	}
	
	return (sock);
}

#else

DLLExport int	zkSocket(int domain, int type, int protocol)
{
	return (socket(domain, type, protocol));
}

#endif


/* zkSockClose -- wrapper for close */

#if defined(WIN32)

DLLExport int	zkSockClose(int sock)
{
    shutdown(sock, SD_BOTH);
	return (closesocket(sock));
}

#elif defined(macintosh)

DLLExport int	zkSockClose(int sock)
{
	long	error;
	
	error = MacSockClose(sock);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		error = -1;
	}
	
	return (error);
}

#else

DLLExport int	zkSockClose(int sock)
{
	return (close(sock));
}

#endif


/* zkSockRead -- wrapper for read */

#if defined(WIN32)

DLLExport ssize_t	zkSockRead(int sock, void *buff, size_t len)
{
	int	nbytes;

	nbytes = recv(sock, buff, len, 0);

	if (nbytes == SOCKET_ERROR)
	{
		errno = WSAGetLastError();
		nbytes = -1;
	}

	return (nbytes);
}

#elif defined(macintosh)

DLLExport ssize_t	zkSockRead(int sock, void *buff, size_t len)
{
	long	error;
	size_t	nread;
	ssize_t	result;
	
	error = MacSockRecv(sock, buff, len, 0, &nread, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nread;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkSockRead(int sock, void *buff, size_t len)
{
	return (read(sock, buff, len));
}

#endif


/* zkSockWrite -- wrapper for write */

#if defined(WIN32)

DLLExport ssize_t	zkSockWrite(int sock, const void *buff, size_t len)
{
	int	nbytes;

	nbytes = send(sock, buff, len, 0);
	
	if (nbytes == SOCKET_ERROR)
	{
		errno = WSAGetLastError();
		nbytes = -1;
	}
	
	return (nbytes);
}

#elif defined(macintosh)

DLLExport ssize_t	zkSockWrite(int sock, const void *buff, size_t len)
{
	long	error;
	size_t	nwritten;
	ssize_t	result;
	
	error = MacSockSend(sock, buff, len, 0, &nwritten, SOCKET_WAITFOREVER);
	
	if (error)
	{
		errno = MacErr2Errno(error);
		result = -1;
	}
	else
	{
		result = nwritten;
	}
	
	return (result);
}

#else

DLLExport ssize_t	zkSockWrite(int sock, const void *buff, size_t len)
{
	return (write(sock, buff, len));
}

#endif

#endif	/* !defined(ZK_KERNEL) */
