/*
 *	zkTime.c		(c) 2000 Zero-Knowledge Systems, Inc.
 *	
 *	$Id: zkTime.c,v 1.2 2001/04/19 17:26:52 alainc Exp $
 *	
 *	Implementation of wrapper functions for <time.h>
 *	
 *	Some functions defined in this source file (those that end in "_r" 
 *	are meant to be reentrant versions of functions in the Standard C 
 *	library.  If ZK_KERNEL is defined, these functions call into our 
 *	Kernel RunTime (krt) library.  If ZK_KERNEL is *not* defined, they 
 *	instead call into the normal (Standard C library) versions.  This 
 *	is actually safe because:
 *		
 *		1. On Win32, the C runtime library is thread-safe.
 *		2. On Linux, gcc's C runtime library is also thread-safe (at 
 *		   least, that's what Oliver says).
 *		3. On traditional Mac OS, we don't have preemptive multitasking, 
 *		   so this is not a problem.
 *		   
 */

#include "zkString.h"
#include "zkTime.h"



/* zkAsctime_r -- wrapper for asctime_r */

#if defined(ZK_KERNEL)

DLLExport char*	zkAsctime_r(const struct tm *tptr, char *s)
{
	return (asctime_r(tptr, s));
}

#else

DLLExport char*	zkAsctime_r(const struct tm *tptr, char *s)
{
	return (strcpy(s, asctime(tptr)));
}

#endif


/* zkCtime_r -- wrapper for ctime_r */

#if defined(ZK_KERNEL)

DLLExport char*	zkCtime_r(const time_t *tod, char *s)
{
	return (ctime_r(tod, s));
}

#else

DLLExport char*	zkCtime_r(const time_t *tod, char *s)
{
	return (strcpy(s, ctime(tod)));
}

#endif


/* zkGmtime_r -- wrapper for gmtime_r */

#if defined(ZK_KERNEL)

DLLExport struct tm*	zkGmtime_r(const time_t *tod, struct tm *tptr)
{
	return (gmtime_r(tod, tptr));
}

#else

DLLExport struct tm*	zkGmtime_r(const time_t *tod, struct tm *tptr)
{
	struct tm*	result	= gmtime(tod);
	
	if (result != NULL)
	{
		*tptr = *result;
		result = tptr;
	}
	
	return (result);
}

#endif


/* zkLocaltime_r -- wrapper for localtime_r */

#if defined(ZK_KERNEL)

DLLExport struct tm*	zkLocaltime_r(const time_t *tod, struct tm *tptr)
{
	return (localtime_r(tod, tptr));
}

#else

DLLExport struct tm*	zkLocaltime_r(const time_t *tod, struct tm *tptr)
{
	struct tm*	result	= localtime(tod);
	
	if (result != NULL)
	{
		*tptr = *result;
		result = tptr;
	}
	
	return (result);
}

#endif
