/* 
 * $Id: zkserr_p.h,v 1.2 2001/04/19 17:26:52 alainc Exp $
 */
#ifndef ZKS_ERR_P_H
# define ZKS_ERR_P_H

# include "zks.h"
# include "zkIntTypes.h"
# include "zkIntLimits.h"
# include "zkTypes.h"
# include "zkDefs.h"
# include "zkSysTime.h"
# include "zkUnistd.h"
# include "zkserr.h"
# include "zkspid.h"


# ifdef __cplusplus
extern "C" {
# endif

  /* ==================================================================================
   *
   *  New ZkErr context declaration
   *
   *  ================================================================================*/

  /*
  ** ZksErr Message context used by the list of ZksErr Message.
  */
  typedef struct S_ZksErrMsgCtx {
    struct timeval       errTime;                  /* ZksErr launched time */
    char                 func[ZKS_ERR_FUNCNAME_MAX+1];
    char                 mesg[ZKS_ERR_MESG_MAX+1]; /* C style string message */
  } * ZksErrMsgCtx;


  /*
  ** ZksErr Context
  */
  struct S_ZksErr {
    uint16_t              category;                /* User Define Category */
    ZksErrCode            code;                    /* ZksErr Code */
#if !defined(ZK_KERNEL)
    T_ZksPid              sysPid;                  /* System Id of the Process that did launched the ZksErr */
#endif
    uint32_t              sysErrCode;              /* System ZksErr Code value at ZksErr launched time */
    uint16_t              flags;                   /* Context specific flags */
    struct S_ZksErrMsgCtx detail;                  /* Original message context  */
    uint32_t              stacksz;                 /* length of the message list */
    ZksErrMsgCtx          stack[ZKS_ERR_STACK_MAX];/* ZksErr message context stack */
    struct S_ZksErr     * tmpErr;                  /* Temporary error context */
    void                * hook;                    /* User data hook pointer */
  };


  /* ==================================================================================
   *
   *  Old ZkError context declaration
   *
   *  ================================================================================*/


# define	__FNC_SZ      ZKS_ERR_FUNCNAME_MAX       /* Length of function name */
# define __DETAIL_SZ    ZKS_ERR_MESG_MAX           /* Length of the detail string */

  /*
  ** ZkError Context
  */
  struct T_ZkError {
    int             flags;                         /* Internal flag field */
    unsigned short  msgtype;                       /* Message type */
    void          * log;                           /* Internal Debugging Log object */
    struct S_ZksErr context;                       /* Inbeded ZksErr context */
  };


# ifdef __cplusplus
}
# endif

#endif /* ZKS_ERR_P_H */
