/*
 * Socket fork handling
 *
 * $Id: zksfork.c,v 1.4 2001/04/19 17:26:52 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */
#include "zks.h"
#include "zkIntTypes.h"

#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkErrno.h"
#include "zkUnistd.h"
#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include <sys/un.h>

#include "zksfork.h"

#ifndef AF_LOCAL
#define AF_LOCAL AF_UNIX
#endif /* ! AF_LOCAL */


#ifndef PF_LOCAL
#define PF_LOCAL PF_UNIX
#endif /* ! AF_LOCAL */

/*  
**  Create a child.  Both parent and child receive a UNIX domain socket
**  descriptor for communicating with each other.
*/
int zkSFork(int *sock)
{
  int fd[2];
  int pid;

  if (socketpair(AF_LOCAL, SOCK_STREAM, 0, fd) == -1) {
    return -1;
  }

  switch (pid = fork()) 
    {
    case -1:
      zkClose(fd[0]);
      zkClose(fd[1]);
      return -1;

    case 0:                       /* child */
      *sock = fd[0];
      zkClose(fd[1]);
      return 0;

    default:                      /* parent */
      *sock = fd[1];
      zkClose(fd[0]);
      return pid;
    }
}



/*  
**  Create a child.  Both parent and child receive a UNIX domain socket
**  descriptor for communicating with each other.
*/
int zkFileSFork(FILE ** readFdP, FILE ** writeFdP)
{
  int    error   = 0;
  int    fd      = -1;
  int    pid     = -1; 
  FILE * readFd  = NULL;
  FILE * writeFd = NULL;

  switch( pid = zkSFork(&fd) ) 
    {
    case -1:
      return -1;

    default:                      /* parent and child */
      if( readFdP ) 
        {
          if( (readFd = fdopen(fd, "r")) == NULL )
            {
              error = 1;
            }
        }
      if( writeFdP ) 
        {
          if( (writeFd = fdopen(fd, "w")) == NULL )
            {
              error = 1;
            }
        }
    }  
  if( error )
    {
      if( readFd )  zkFclose(readFd);
      if( writeFd ) zkFclose(writeFd);
    }
  else
    {
      if( readFdP )  *readFdP  = readFd;
      if( writeFdP ) *writeFdP = writeFd;
    }
  return ( error ) ? -1 : pid;
}


#endif /* ZK_LINUX || ZK_SOLARIS */
