/* 
 * $Id: zkspid.c,v 1.2 2001/04/19 17:26:52 alainc Exp $
 */
#include "zks.h"
#include "zkIntTypes.h"

#if !defined(ZK_KERNEL)

#include "zkStdlib.h"
#include "zkStdio.h"
#include "zkTime.h"
#include "zkUnistd.h"

#if defined( ZK_LINUX )
# include <pthread.h>
#elif defined( macintosh )
# include <Math64.h>
# include <Processes.h>
# include <Threads.h>
#endif

#include "zkserr.h"
#include "zkspid.h"


/*
 *
 */
static ZkBool pidInit(T_ZksPid * pid)
{
#if defined( WIN32 )
  pid->pId = (ZksProcessId) GetCurrentProcessId();
  pid->tId = (ZksThreadId)  GetCurrentThreadId();

#elif defined( ZK_LINUX )
  pid->pId = (ZksProcessId) getpid();
  pid->tId = (ZksThreadId)  pthread_self();

#elif defined( macintosh )
  
  ProcessSerialNumber	psn;
  UnsignedWide			uw;
  ThreadID				tid;
  
  GetCurrentThread(&tid);
  GetCurrentProcess(&psn);
  
  uw.hi	= psn.highLongOfPSN;
  uw.lo	= psn.lowLongOfPSN;
  
  pid->pId = UnsignedWideToUInt64(uw);
  pid->tId = tid;
  
#endif
  return ZKS_SUCCESS;
}




/*
 *
 */
ZkBool zksPidInit(T_ZksPid * pid)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !pid ) { 
    success = ZKS_FAILURE;

  } else {
    pidInit(pid);
  }
  return success;  
}




/*
 *
 */
ZkBool zksPidEqual(const T_ZksPid * pidA, 
                   const T_ZksPid * pidB, 
                   ZkBool         * equalP)
{
  ZkBool  success = ZKS_SUCCESS;
  ZkBool  equal   = ZK_FALSE;

  if( !pidA || !pidB  ) { 
    success = ZKS_FAILURE;

  } else {
    equal = ( (pidA->pId == pidB->pId) && (pidA->tId == pidB->tId) ) ? ZK_TRUE: ZK_FALSE;
  }

  if( equalP ) { *equalP = equal; }

  return success;
}




/*
 *
 */
ZkBool zksPidAssign(T_ZksPid       * dstPid, 
                    const T_ZksPid * srcPid)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !dstPid || !srcPid ) { 
    success = ZKS_FAILURE;

  } else {
    *dstPid = *srcPid;
  }
  return success;  
}





/*
 *
 */
ZkBool zksPidAsString(const T_ZksPid * pid, 
                      char           * buffer, 
                      uint32_t         buffersz)
{
  ZkBool  success = ZKS_SUCCESS;

  if( !pid ) { 
    success = ZKS_FAILURE;

  } else if( buffer && buffersz ){
    zkSnprintf(buffer, buffersz, LONG_LONG_FTM ":" LONG_LONG_FTM, pid->pId, pid->tId);
  }
  return success;  
}

#endif /*  !defined(ZK_KERNEL) */
