/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkFile.h"
#include "zkShell.h"

#define VERSION_STR "zkConfig test program ("__DATE__", "__TIME__")"

#define DUMP 1

int
main (
  int argc ,
  char ** argv )
{
  char * configfile = NULL;
  char * command = NULL;
  int opt = 1;
  const char * progname = NULL;
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkConfig config = NULL;
  int fail = 0;
  int cmdid = 0;
  int i = 0;
  size_t sections  = 0;
  const char * secname = NULL;
  const char * name = NULL;
  const char * value = NULL;
  char * buffer = NULL;
  size_t bsz = 0;
  
  progname = zkShellGetProgramName(&argc, &argv);

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  zkConfigNew(err,&config);

  while (opt < argc) {
    if (strcmp(argv[opt],"--config") == 0) {
      if (++opt < argc) {
        configfile = argv[opt];
      } else {
        zkFprintf(stderr,"%s: Missing configuration filename\n",progname);
        fail = 1;
      }
    } else if (strcmp(argv[opt],"--command") == 0) {
      if (++opt < argc) {
        command = argv[opt];
      } else {
        zkFprintf(stderr,"%s: You must specify a command\n",progname);
        fail = 0;
      }
    } else {
      zkFprintf(stderr,"%s: Unrecognized option \"%s\"\n",progname,argv[opt]);
      fail = 1;
    }
    if (fail) {
      break;
    }
    opt++;
  }
  if (fail) {
    exit(EXIT_FAILURE);
  }
  
  if (command) {
    if (strcmp(command,"dump") == 0) {
      cmdid = DUMP;
    } else {
      zkFprintf(stderr,"%s: Unrecognized command \"%s\"\n",progname,command);
      fail = 1;
    }
  } else {
    zkFprintf(stderr,"%s: You must specify a command with --command\n",progname);
    fail = 1;
  }
  if (configfile) {
#if 1
    if (!zkFileLoad(err,configfile,&buffer,&bsz)) {
      zkFprintf(stderr,"Unable to load file\n");
      fail = 1;
    } else if (!zkConfigLoadFromBuffer(err,config,buffer,bsz)) {
      zkFprintf(stderr,"Unable to load from buffer\n");
      fail = 1;
    }
#else
    if (!zkConfigLoad(err,config,configfile)) {
      zkFprintf(stderr,"%s: Could not load %s\n",progname,configfile);
      fail = 1;
    }
#endif
  } else {
    zkFprintf(stderr,"%s: You must specify a configuration file with --config\n",
            progname);
    fail = 1;
  }
  if (fail) {
    exit(EXIT_FAILURE);
  }


  switch(cmdid) {
  case DUMP:
    zkConfigSections(err,config,&sections);
    for (i = 0; i < (int) sections; i++) {
      zkConfigSection(err,config,(size_t)i,&secname);
      while (zkConfigGetPair(err,config,secname,&name,&value)) {
        if (secname) {
          zkPrintf("[%s] %s = %s\n",secname,name,value);
        } else { 
          zkPrintf("[  ] %s = %s\n",name,value);
        }
        secname = NULL;
      }
    }
    break;
    
  }

  return (EXIT_SUCCESS);
}


