/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkStdio.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkConfig.h"
#include "zkShell.h"


int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkConfig config = NULL;
  size_t sections = 0;
  const char * name = NULL;
  size_t i = 0; /* Loop index */
  const char* progname;

  progname = zkShellGetProgramName(&argc, &argv);
  
  if (argc != 3) {
    zkFprintf(stderr,"usage:  %s source dest\n\n",progname);
    exit(0);
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkConfigNew(err,&config);
 
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if (!zkConfigLoad(err,config,argv[1])) {
    zkLogError(log,err);
  }
  if (!zkConfigSections(err,config,&sections)) {
    zkLogError(log,err);
  }
  for (i = 0; i < sections; i++) {
    if (!zkConfigSection(err,config,i,&name)) {
      zkLogError(log,err);
    } else {
      zkPrintf("%s\n",name);
    }
  }
  if (!zkConfigSave(err,config,argv[2])) {
    zkLogError(log,err);
  }

  zkConfigFree(err,&config);
  zkLogFree(&log);
  zkErrorFree(&err);

  return (EXIT_SUCCESS);
}
  
