/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkFcntl.h"
#include "zkUnistd.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkDump.h"
#include "zkFile.h"
#include "zkShell.h"


int
main (
  int argc ,
  char ** argv )
{
  char * buffer = NULL;
  size_t bufferlen = 0;
  ZkLog log = NULL;
  ZkError err = NULL;
  const char* progname;
  
  progname = zkShellGetProgramName(&argc, &argv);

  if (argc != 3) {
    zkFprintf(stderr,"Usage:  %s in-filename out-filenamemake \n\n",progname);
    exit(EXIT_FAILURE);
  }

  if (! zkErrorNew(&err) ||
      ! zkErrorLog(err,&log) ||
      ! zkLogSetStream(log,stdout) ||
      ! zkLogEnable(log) ) {
    zkFprintf(stderr,"Can't get error system to start\n");
  }
  
  if (!zkFileLoad(err,argv[1],&buffer,&bufferlen)) {
    zkFprintf(stderr,"zkLoadFile:  failed\n");
  }
  
  if (!zkDumpBuffer(err,(unsigned char *) buffer,bufferlen,zkDumpToStream,stdout)) {
    zkFprintf(stderr,"zkDumpBuffer:  failed\n");
  }
  
  if (!zkFileSave(err,argv[2], buffer, bufferlen, O_CREAT|O_TRUNC, 0600 )) {
    zkFprintf(stderr,"zkFileSave:  failed\n");
  }
  
  if (buffer) {
    zkFree(buffer);
  }
  zkErrorFree(&err); 
  
  return (EXIT_SUCCESS);
}


