/*
 *  $Id: getline.c,v 1.3 2001/04/19 17:26:54 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test zkFileGetLineFp().
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkFile.h"
#include "zkShell.h"


const char *prog;


int main(int ac, char **av)
{
  ZkError err;
  char *line = 0;
  size_t lsz;
  unsigned short code;
  
  prog = zkShellGetProgramName(&ac, &av);
  
  if (ac != 1) {
    zkFprintf(stderr, "Usage: %s\n", prog);
    zkFprintf(stderr, "  This test program will copy stdin to stdout.\n");
    zkFprintf(stderr, "  Compare the output for equality with the input.\n");
    zkFprintf(stderr, "  The comparison will fail if the input file does\n");
    zkFprintf(stderr, "  not end with a newline: this program will add one.\n");
    exit(EXIT_FAILURE);
  }

  assert(zkErrorNew(&err));

  while (zkFileGetLineFp(err, stdin, &line, &lsz)) {
    assert(line);
    puts(line);
  }

  if (line) { zkFree(line); }

  switch ((code = zkErrorCode(err))) {
  case ZKEEMPTY:
    zkFprintf(stderr, "%s: got EOF on input.\n", prog);
    break;

  default:
    zkFprintf(stderr, "%s: failed with %hu.\n", prog, code);
    exit(EXIT_FAILURE);
  }

  return (EXIT_SUCCESS);
}
