/*
 * Test program to try out the graph library
 *
 * $Id: graph-test.c,v 1.10 2001/04/19 17:26:54 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Luc Boulianne <luc@zks.net>

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include <assert.h>

#include "zkError.h"
#include "zkLog.h"
#include "zkIntTypes.h"
#include "zkGraph.h"
#include <assert.h>

int main (int argc ,char ** argv)
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkGraph Graph;
  uint32_t *pi;
  int yes = 0;
  uint32_t *path;
  size_t psz, i;

  (void) argc;
  (void) argv;

  assert(zkErrorNew(&err));
  assert(zkErrorLog(err,&log));
  assert(zkLogSetStream(log,stderr));
  assert(zkLogEnable(log));

  assert(zkGraphNew(err, &Graph));

  assert(zkGraphAddNode(err, Graph, 10));
  assert(zkGraphAddNode(err, Graph, 12));
  assert(zkGraphAddNode(err, Graph, 1));
  assert(zkGraphAddNode(err, Graph, 22));
  assert(zkGraphAddNode(err, Graph, 23));
  assert(zkGraphAddNode(err, Graph, 24));
  assert(zkGraphAddNode(err, Graph, 32));
  assert(zkGraphAddNode(err, Graph, 8));
  assert(zkGraphAddNode(err, Graph, 100));
  assert(zkGraphAddNode(err, Graph, 10210));
  assert(zkGraphAddNode(err, Graph, 2));
  assert(zkGraphAddNode(err, Graph, 99));

  assert(zkGraphEdgeAdd(err, Graph, 10, 12, 1));
  assert(zkGraphEdgeAdd(err, Graph, 10, 1, 2));
  assert(zkGraphEdgeAdd(err, Graph, 1, 10, 12));
  assert(zkGraphEdgeAdd(err, Graph, 100, 99, 23));
  assert(zkGraphEdgeAdd(err, Graph, 100, 2, 0));
  assert(zkGraphEdgeAdd(err, Graph, 8, 32, 5));
  assert(zkGraphEdgeAdd(err, Graph, 23, 32, 12));
  assert(zkGraphEdgeAdd(err, Graph, 22, 2, 8));
  assert(zkGraphEdgeAdd(err, Graph, 23, 10210, 81));
  assert(zkGraphEdgeAdd(err, Graph, 100, 8, 2));
  assert(zkGraphEdgeAdd(err, Graph, 1, 8, 2));

  assert(zkGraphDump(err, Graph));

  assert(zkGraphBFS(err, Graph, 100, &pi));

  assert(zkGraphPathExists(err, Graph, 100, 12, pi, &yes));
  assert(yes);
  assert(zkGraphConstructPath(err, Graph, 100, 12, pi, &path, &psz));
  
  for (i = 0; i < psz; i++) {
    zkFprintf(stderr, "-> %d ", path[i]);
  }
  zkFprintf(stderr, "\n");
  if (path) {
    zkFree(path);
    path = NULL;
  }

  assert(zkGraphPathExists(err, Graph, 100, 24, pi, &yes));
  assert(!yes);
  assert(!zkGraphConstructPath(err, Graph, 100, 24, pi, &path, &psz));
  
  assert(zkGraphDump(err, Graph));

  assert(zkGraphFree(err, &Graph));
  assert(zkErrorFree(&err));

  if (pi) {
    zkFree(pi);
    pi = NULL;
  }
  return 1;
}
