/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkList.h"

#define NAME_SZ 32
#define BUFFER_SZ 32

struct node {
  char name[NAME_SZ + 1];
  int id;
};

struct node * mknode ( int );

struct node * mknode (
  int id )
{
  struct node * node = NULL;

  node = (struct node *) zkMalloc(sizeof(struct node));
  if (node == NULL) {
    return NULL;
  }
  zkSnprintf(node->name,sizeof node->name,"NODE%d",id);
  node->id = id;
  return node;
}

int 
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkList list = NULL;
  struct node * node = NULL;
  int i = 0; /* loop index */
  size_t size = 0;
  ZkListNode cursor = NULL;

  (void) argc;
  (void) argv;
  
  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

#if 1
  if (!zkListNew(err,&list)) {
#else
  if (!zkListNew(err,NULL)) {
#endif
    zkPrintf("zkListNew:  failed\n");
  }

#if 0
  if (!zkListSetDataFreeFunction(err,list,NULL,NULL)) {
    zkPrintf("zkListSetDataFreeFunction:  failed\n");
  }

#endif

  /*
   * Add a bunch of nodes into the list
   */

  for (i = 0; i < 100; i += 10) {
    node = mknode(i);
    zkPrintf("ADDING NODE %p name %s id %d\n",(void *)node,node->name,node->id);
#if 0
    if (!zkListInsert(err,list,node,LIST_HEAD)) {
#else
    if (!zkListInsert(err,list,node,LIST_TAIL)) {
#endif
      zkPrintf("zkListInsert:  failed\n");
    }
    if (i == 50) {
      zkListCursor(err,list,&cursor);
    }
  }

  /*
   * Move the pointer to the head of the list
   */
 
  if (!zkListMoveCursor(err,list,LIST_HEAD)) {
    zkPrintf("zkListMoveCursor:  failed\n");
  }
  
  /*
   * Dump the contents of the list
   */

  zkListSize(err,list,&size);
  zkPrintf("Current list size is %d\n",size);
  while (zkListGet(err,list,(void **) &node,LIST_FORWARDS)) {
    zkPrintf("NODE %p name %s id %d\n",(void *)node,node->name,node->id);
  }

  zkListMoveCursor(err,list,LIST_HEAD);
  zkListDelete(err,list,LIST_FORWARDS);
  zkListMoveCursor(err,list,LIST_FORWARDS);
  zkListMoveCursor(err,list,LIST_FORWARDS);
  zkListDelete(err,list,LIST_FORWARDS);
  zkListDelete(err,list,LIST_FORWARDS);
 
  zkListMoveCursor(err,list,LIST_TAIL);
  zkListDelete(err,list,LIST_FORWARDS);

  node = mknode(777);
  zkListInsert(err,list,node,LIST_TAIL);

  zkListMoveCursor(err,list,LIST_HEAD);

  zkPrintf("------------------\n");
  zkListSize(err,list,&size);
  zkPrintf("Current list size is %d\n",size);
  while (zkListGet(err,list,(void **) &node,LIST_FORWARDS)) {
    zkPrintf("NODE %p name %s id %d\n",(void *)node,node->name,node->id);
  }


  /*
   * Dump the list backwards
   */
  zkListMoveCursor(err,list,LIST_TAIL);
  while (zkListGet(err,list,(void **) &node,LIST_BACKWARDS)) {
    zkPrintf("NODE %p name %s id %d\n",(void *)node,node->name,node->id);
  }

  /*
   * Remove the thing we logged a cursor for...
   */

  zkPrintf("Deleting a specific item at known cursor position\n");
  zkListSetCursor(err,list,cursor);
  zkListDelete(err,list,LIST_FORWARDS);
  zkListMoveCursor(err,list,LIST_HEAD);
  while (zkListGet(err,list,(void **) &node,LIST_FORWARDS)) {
    zkPrintf("NODE %p name %s id %d\n",(void *)node,node->name,node->id);
  }
  
  /*
   * Destroy the list
   */

  if (!zkListFree(err,&list)) {
    zkPrintf("zkListFree:  failed\n");
  }
 
  node = mknode(999);
  zkPrintf("Single node test\n");
  zkListNew(err,&list);
  zkListDelete(err,list,LIST_FORWARDS);
  zkListInsert(err,list,node,LIST_HEAD);
  zkPrintf("------------dump\n");
  zkListMoveCursor(err,list,LIST_HEAD);
  while (zkListGet(err,list,(void **) &node,LIST_FORWARDS)) {
    zkPrintf("NODE %p name %s id %d\n",(void *)node,node->name,node->id);
  }
  zkListMoveCursor(err,list,LIST_TAIL);
  zkListDelete(err,list,LIST_FORWARDS);
  zkListFree(err,&list);

  zkPrintf("Test: destroying an empty list\n");
  zkListNew(err,&list);
  zkListFree(err,&list);
  
  return (EXIT_SUCCESS);
}
