/*
 *  $Id: nbo.c,v 1.2 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Testing network byte ordering
 */

#include <assert.h>
#include "zkStdlib.h"
#include "zkStdio.h"
#include "zkString.h"
#include "zks.h"
#include "zkIntTypes.h"
#include "zkDump.h"
#include "zknbo.h"

#if defined(__MWERKS__)
#	define TEST_FMT_U64	PRIx64
#elif defined(WIN32)
#	define TEST_FMT_U64	"I64x"
#else
#	define TEST_FMT_U64	"Lx"
#endif


static void uint64_test(UINT64_T x)
{
  UINT64_T a = 0;
  char buf[8] = {0,0,0,0,0,0,0,0};

  zkPrintf("NBO 64 test ----------\n");
  zkPrintf("Original:  [0x%" TEST_FMT_U64 "]\n", x, x);
  zkNboFromHostUint64 (buf, x);
  zkPrintf("NBO     : ");
  zkDumpBuffer(NULL, (unsigned char*) buf, sizeof(buf), zkDumpToStream, stdout);  
  a = zkNboToHostUint64 (buf);
  zkPrintf("HostBO  :  [0x%" TEST_FMT_U64 "]\n", a, a);
  
}

static void uint32_test(uint32_t x)
{
  uint32_t a = 0;
  char buf[4] = {0,0,0,0};

  zkPrintf("NBO 32 test ----------\n");
  zkPrintf("Original: %lu [0x%lx]\n", (unsigned long) x, (unsigned long) x);
  zkNboFromHostUint32 (buf, x);
  zkPrintf("NBO     : ");
  zkDumpBuffer(NULL, (unsigned char*) buf, sizeof(buf), zkDumpToStream, stdout);  
  a = zkNboToHostUint32 (buf);
  zkPrintf("HostBO  : %lu [0x%lx]\n", (unsigned long) a, (unsigned long) a);
  
}

static void uint16_test(uint16_t x)
{
  uint16_t a = 0;
  char buf[2] = {0,0};

  zkPrintf("NBO 16 test ----------\n");
  zkPrintf("Original: %hu [0x%hx]\n", (unsigned short) x, (unsigned short) x);
  zkNboFromHostUint16 (buf, x);
  zkPrintf("NBO     : ");
  zkDumpBuffer(NULL, (unsigned char*) buf, sizeof(buf), zkDumpToStream, stdout);  
  a = zkNboToHostUint16 (buf);
  zkPrintf("HostBO  : %hu [0x%hx]\n", (unsigned short) a, (unsigned short) a);
  
}

int main(int argc, char **argv)
{
  (void) argc;
  (void) argv;

  uint64_test(0x0000000010101010);
  uint64_test(0xffffffff00000000);
  uint64_test(0x00000000f0f0f0f0);
  uint64_test(0x1010101000000000);
  uint64_test(0x00000000ffffffff);
  uint64_test(0xffffffffffffffff);
  uint64_test(0xf0f0f0f000000000);

  uint32_test(0x00001010UL);
  uint32_test(0xffff0000UL);
  uint32_test(0x0000f0f0UL);
  uint32_test(0x10100000UL);
  uint32_test(0x0000ffffUL);
  uint32_test(0xffffffffUL);
  uint32_test(0xf0f00000UL);

  uint16_test(0x0010U);
  uint16_test(0xff00U);
  uint16_test(0x00f0U);
  uint16_test(0x1000U);
  uint16_test(0x00ffU);
  uint16_test(0xffffU);
  uint16_test(0xf000U);

  return (EXIT_SUCCESS);
}
