/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkUnistd.h"
#include "zkErrno.h"
#include <sys/wait.h>
#include "zkSysSocket.h"
#include "zkPFork.h"
#include "zksfork.h"

#define BUFF_SZ ((1024*1024)*2)

int MSG_SZ  = 0;



static int child_thread(int fd);
static int parent_thread(int fd);

static int child_thread(int fd) {
  const char * func = "child_thread";
  fd_set rset;
  static char buff[BUFF_SZ] = { 0 };
  
  zkSleep(3);
  zkFprintf(stderr, "%s: Enter\n", func);

  zkFprintf(stderr, "%s: fd=%d\n", func, fd);

  zkFprintf(stderr, "%s: WAIT\n", func);

  FD_ZERO(&rset);
  FD_SET(fd, &rset);

  switch( zkSelect(fd+1, &rset, NULL, NULL, NULL) ) {

  case -1:
    zkFprintf(stderr, "%s: select(%d) failed\n", func, errno);
    break;

  default:
    if( FD_ISSET(fd, &rset) ) {
      if( recv(fd, buff, MSG_SZ, 0) != -1 )
          {
            zkFprintf(stderr, "%s: recv success\n", func);
            zkFprintf(stderr, "%s: [%d]\n", func, strlen(buff));        
          } else {
            zkFprintf(stderr, "%s: recv(%d) failed\n", func, errno);
          }      
    }
  }
  zkFprintf(stderr, "%s: Exit\n", func);
  return 1;
}



static int parent_thread(int fd) {
  const char * func = "parent_thread";
  fd_set wset;
  static char buff[BUFF_SZ] = { 0 };

  memset(buff, 'z', MSG_SZ);

  zkFprintf(stderr, "%s: Enter\n", func);

  zkFprintf(stderr, "%s: fd=%d\n", func, fd);

  zkFprintf(stderr, "%s: WAIT\n", func);

  FD_ZERO(&wset);
  FD_SET(fd, &wset);

  switch( zkSelect(fd+1, NULL, &wset, NULL, NULL) ) {

  case -1:
    zkFprintf(stderr, "%s: select(%d) failed\n", func, errno);
    break;

  default:
    if( FD_ISSET(fd, &wset) ) {
      if( send(fd, buff, MSG_SZ, 0) != -1 ) 
          {
            zkFprintf(stderr, "%s: send success\n", func);
          } else {
            zkFprintf(stderr, "%s: send(%d) failed\n", func, errno);
          }
    }
  } 
  zkFprintf(stderr, "%s: Exit\n", func);
  return 1;
}



void main(int argc, char * argv[]) {
  int fd  = -1;
  int opt = 400;
  int optsz = sizeof(int);

  if( argc != 3) {
    zkFprintf(stderr, "usage: %s <mesg size> <buffer size>\n", argv[0]);
    exit(1);
  }

  MSG_SZ = atoi(argv[1]);
  opt = atoi(argv[2]);

  switch( zkSFork(&fd) ) {
  case -1:
    zkFprintf(stderr, "main: zkSFork failed\n");
    break;
  case 0: /* Child */    
    if( setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void *)&opt, sizeof(int)) == -1 ) {
      zkFprintf(stderr, "main: setsockopt(parent) failed\n");
    } else if( getsockopt(fd, SOL_SOCKET, SO_RCVBUF, (void *)&opt, &optsz) == -1 ) {
      zkFprintf(stderr, "main: getsockopt(child) failed\n");
    } else {
      zkFprintf(stderr, "main: child buff = %d\n", opt);
      child_thread(fd);
    }
    break;
  default: /* Parent */
    if( setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void *)&opt, sizeof(int)) == -1 ) {
      zkFprintf(stderr, "main: setsockopt(parent) failed\n");
    } else if( getsockopt(fd, SOL_SOCKET, SO_SNDBUF, (void *)&opt, &optsz) == -1 ) {
      zkFprintf(stderr, "main: getsockopt(parent) failed\n");
    } else {
      zkFprintf(stderr, "main: parent buff = %d\n", opt);
      parent_thread(fd);
      close(fd);
      zkFprintf(stderr, "main: parent start waiting\n");
      waitpid(0, NULL, 0);
      zkFprintf(stderr, "main: parent stop waiting\n");
    }
  }
}
