/*
** **************************************************************************
** $Id: sharedobj-test.c,v 1.3 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

**
** Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
** **************************************************************************
*/

# include "zks.h"
# include "zkSharedObject.h"
# include "zkShell.h"
# include "zkStdio.h"

typedef double (*TrigFunc)(double);

static void DumpError(const char *, ZkError);

#ifdef WIN32
#define SHARED_OBJECT "msvcrt.dll"
#define FUNC_NAME     "cos"
#elif macintosh
#define SHARED_OBJECT ":sharedobj-target.bundle"
#define FUNC_NAME     "my_cos"
#else
#define SHARED_OBJECT "/lib/libm.so.6"
#define FUNC_NAME     "cos"
#endif

int main(int argc, char **argv)
{
    ZkError        err    = NULL;
    ZkSharedObject object = NULL;
    TrigFunc       func   = NULL;
    const char *   progname;

    progname = zkShellGetProgramName(&argc, &argv);

    if (!zkErrorNew(&err))
    {
        zkFprintf( stderr,
                 "%s:%s\n",
                 progname,
                 "couldn't construct error object" );

    }
    else if ( zkSharedObjectLoad(err, SHARED_OBJECT, &object)
              && zkSharedObjectFindSymbol(err, object, FUNC_NAME, (void**)&func) )
    {
        zkPrintf("cos(2.0) = %f\n", (*func)(2.0));
    }
    else
    {
        DumpError(progname, err);
    }
    
    if (object && !zkSharedObjectUnload(err, &object))
    {
        DumpError(progname, err);
    }

    if (err)
    {
        zkErrorFree(&err);
    }

    return (0);
}


static void
DumpError(const char * context, ZkError err)
{
    char message[1024];
    
    zkErrorMessage(err, message, sizeof(message)-1);
    
    zkFprintf(stderr, "%s: %s\n", context, message);
}
