/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkShell.h"

#define CHECK(_foo) if (!(_foo)) { zkFprintf(stderr,#_foo": failed.\n"); exit_code = EXIT_FAILURE; }

int zkShellCommandFoobar ( ZkError err , ZkShell shell , char ** token , size_t tokens , void * context );

int
zkShellCommandFoobar (
  ZkError err ,
  ZkShell shell ,
  char ** token ,
  size_t tokens ,
  void * context )
{ 
  size_t i = 0;
  
  /* silence compiler */
  (void) context;
  
  zkErrorClear(err);
  for (i = 0; i < tokens; i++) {
    zkShellWrite_va(err,shell,"TOKEN %d is \"%s\"\n",i,token[i]);
  }
  return ZKSHELL_SESSION_CONTINUE;
}


int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkShell shell = NULL;
  int exit_code = EXIT_SUCCESS;
  const char * progname;

  progname = zkShellGetProgramName(&argc, &argv);

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  CHECK(zkShellNew(err,&shell));
  CHECK(zkShellSetPrompt(err,shell,progname));
  CHECK(zkShellAddCommand(err,shell,zkShellCommandFoobar,"foobar","foobar","this is a stupid little command which takes 0 to 7 parameters and prints them out",0xFF));
  CHECK(zkShellRun(err,shell));
  CHECK(zkShellFree(err,&shell));

  return (exit_code);
}
