/* **************************************************************************
** $Id: socket-addrcmp.c,v 1.2 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file defines a test application for exercising the ZkSocket object.
**
** **************************************************************************
*/

# include "zks.h"
# include "zksocket.h"
# include "zkStdio.h"
# include "zkShell.h"
# include "zkString.h"

static const char * path1 = NULL;
static const char * path2 = NULL;
static uint32_t     flags = 0;

static void show_usage(const char * program)
{
    zkFprintf(stderr,
            "Usage\t%s path1 path2 [-a] [-p]\n\n"
            "\tpath1 : the first socket address\n"
            "\tpath2 : the second socket address\n"
            "\t-a    : for internet sockets make sure IP addresses match\n"
            "\t-p    : for internet sockets make sure port numbers match\n",
            program);
}

static int validate_params(int argc, char * argv[])
{
    int success = 0;

    if ((argc >= 3) && (argc <= 5))
    {
        int i;

        path1   = argv[1];
        path2   = argv[2];
        success = 1;

        for (i = 3; success && (i < argc); ++i)
        {
            if ((argv[i][0] != '-')
                || (strlen(argv[i]) != 2))
            {
                success = 0;
            }
            else switch (argv[i][1])
            {
            case 'A':
            case 'a':
                flags |= ZKSOCKET_CMP_INETADDR;
                break;

            case 'P':
            case 'p':
                flags |= ZKSOCKET_CMP_INETPORT;
                break;

            default:
                success = 0;
                break;
            }             
        }
    }

    return (success);
}


int main(int argc, char * argv[])
{
    ZkError error = NULL;
    const char* progname;
    
  #ifdef WIN32
    WSADATA	stWSAData;
    WORD wVersionRequired = MAKEWORD(2, 0);
    WSAStartup(wVersionRequired, &stWSAData);
  #endif

    progname = zkShellGetProgramName(&argc, &argv);

    if (!validate_params(argc,argv))
    {
        show_usage(progname);
    }
    else if (zkErrorNew(&error))
    {
        ZkBool is_same = 0;

        if (!zkSocketAddressCompare(error, flags, path1, path2, &is_same))
        {
            char message[256];
            zkErrorMessage(error,message,sizeof(message));
            zkFprintf(stderr,"%s\n", message);
        }
        else
        {
            zkFprintf( stdout,
                     "%s: addresses %smatch\n",
                     progname,
                     (is_same ? "" : "don't ") );
        }

        zkErrorFree(&error);
    }

  #ifdef WIN32
    WSACleanup();
  #endif

    return (0);
}
