/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Test the zkMemBuf routines.
 */

#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkTime.h"
#include "zkMemBuf.h"


int main(void)
{
  ZkMemBuf b;
  ZkError err;
  char d[100];
  size_t cnt[1000], i, psz, tot = 0;
  void *p;

  srand(time(0));
  
  /* New. */

  assert(zkErrorNew(&err));
  assert(zkMemBufNew(err, &b));

  /* Check appending. */

  for (i = 0; i < sizeof cnt / sizeof cnt[0]; i++) {
    cnt[i] = rand() % sizeof d;

    memset(d, 'a' + (i % 26), cnt[i]); tot += cnt[i];
    assert(zkMemBufAppend(err, b, d, cnt[i]));
  }

  assert(zkMemBufGetData(err, b, &p, &psz));
  assert(psz == tot);

  /* Verify the data in the buffer. */

  tot = 0;
  for (i = 0; i < sizeof cnt / sizeof cnt[0]; i++) {
    memset(d, 'a' + (i % 26), cnt[i]);
    assert(memcmp((char *)p + tot, d, cnt[i]) == 0);
    tot += cnt[i];
  }
  
#if !defined(macintosh)
  /* Check writing to a file descriptor. */

  {
    FILE *tfp;
    char *copy, *orig;
    int empty = 0;

    assert(copy = zkMalloc(psz + 1));
    assert(orig = zkMalloc(psz + 1));

    memset(copy, 0, psz + 1);
    memcpy(orig, p, psz + 1); orig[psz] = '\0';

    assert(tfp = popen("./twrt /tmp/tbuf.out", "wb")); /* audit: ok */
    (void)setbuf(tfp, 0);
    for (i = 0; i <= psz; i++) {
      assert(zkMemBufWriteToFd(err, b, zkFileno(tfp), &empty));
      if (empty) break;
    }
    assert(pclose(tfp) != -1);
    assert(i < psz); /* should have written at least 1 byte each time */

    zkPrintf("Wrote %lu bytes in %lu calls to zkMemBufWriteToFd().\n",
           (unsigned long)psz, (unsigned long)i + 1);

    assert(tfp = zkFopen("/tmp/tbuf.out", "rb"));
    assert(zkFread(copy, 1, psz + 1, tfp) == psz);
    (void)zkFclose(tfp);

    assert(memcmp(copy, orig, psz + 1) == 0);

    zkFree(copy); zkFree(orig);
  }
#endif

  assert(zkMemBufFree(err, &b));
  assert(b == 0);
  assert(zkErrorFree(&err));
  assert(err == 0);
  
  return (EXIT_SUCCESS);
}
