/*
 *  $Id: tclock.c,v 1.4 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test ZkClock routines.
 */


#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkClock.h"
#include "zkShell.h"
#include "zkString.h"


int main(int ac, char **av)
{

  ZkClock   clk = zkClockMaxPos();
  time_t    tim;
  struct tm tm;
  char      buff1[64], buff2[64];
  const char* progname;
  
  progname = zkShellGetProgramName(&ac, &av);

  
  assert(zkClockToSeconds(clk) > 0);
  assert(zkClockToSeconds(clk) + 1 < 0);
  

  zkFprintf(stderr, "%s: clk is " LONG_LONG_FTM ".\n",
          progname, (LONG_LONG) zkClockToSeconds(clk));
  
  /* test conversion from time_t */
  
  tm.tm_year = 80;
  tm.tm_mon  = 2;
  tm.tm_mday = 15;
  tm.tm_hour = 8;
  tm.tm_min  = 30;
  tm.tm_sec  = 45;
  tim = mktime(&tm);
  clk = zkClockFromTime(tim);
  zkClockStrftime(buff1, sizeof(buff1), "%c", clk);
  strftime(buff2, sizeof(buff2), "%c", localtime(&tim));
  
  assert(tim == zkClockToTime(clk));
  assert(strcmp(buff1, buff2) == 0);
  
  /* test conversion to time_t */
  
  clk = zkClockNow();
  tim = zkClockToTime(clk);
  zkClockStrftime(buff1, sizeof(buff1), "%c", clk);
  strftime(buff2, sizeof(buff2), "%c", localtime(&tim));
  
  assert(clk.rep == zkClockFromTime(tim).rep);
  assert(strcmp(buff1, buff2) == 0);
  


  return (EXIT_SUCCESS);
}
