/*
 *  $Id: testzkLock.c,v 1.2 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Program to test file locking.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "zks.h"
#include "zkError.h"
#include "zkLock.h"



#if !defined(ZK_CLIENT)

static const char *prog;


static char *tail(char *path)
{
  char *p = strrchr(path, '/');
  return p ? p+1 : path;
}


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--lockmode {x|s}]"
          " [--sleep <secs>]"
          " <lock file path>"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}

#endif	/* !defined(ZK_CLIENT) */


int main(int ac, char **av)
{
#if !defined(ZK_CLIENT)

    char *  sleepStr    = 0;
    char *  lockPath    = 0;
    char *  lockmode    = 0;
    int     granted     = 0;
    int     ok          = 0;
    int     sleepSecs   = -1;
    int     bWait       = 0;
    ZkLock  lock        = NULL;
    ZkError err         = NULL;


    prog = tail(av[0]);

    while (av++, --ac) {
      if (strcasecmp(av[0], "-h") == 0 ||
          strcasecmp(av[0], "-help") == 0 ||
          strcasecmp(av[0], "--help") == 0) {
        usage(1);
      } else if (strcasecmp(av[0], "--lockmode") == 0) {
        if (av++, --ac < 1) { usage(0); }
        lockmode = av[0];
      } else if (strcasecmp(av[0], "--sleep") == 0) {
        if (av++, --ac < 1) { usage(0); }
        sleepStr = av[0];
      } else {
        break;
      }
    }

    if (ac != 1) {
      usage(0);
    }

    if (sleepStr &&
        (sleepSecs = atoi(sleepStr)) == 0 &&
        sleepStr[0] != '0') {
      usage(0);
    }

    lockPath = av[0];

    if (!zkErrorNew(&err) ) {
        return EXIT_FAILURE;
    }
    
    /* Create a new lock handle */
    if (!zkLockNew(err,&lock,lockPath,NULL)) {
        printf("Error [%s] in %s.\n",zkErrorFunction (err), zkErrorDetail (err));
        return EXIT_FAILURE;
    }
    
    /* Request lock */    
    if (lockmode[0]=='x'){
        if (!zkLockRequestExclNoWait(err,lock,&granted)) {
            printf("Error [%s] in %s.\n",zkErrorFunction (err), zkErrorDetail (err));
            goto done;
        } else if (!granted) {
            printf("Exclusive lock not granted\n");
        } else {
            bWait = ZK_TRUE;
        }
    } else if (lockmode[0]=='s'){
        if (!zkLockRequestSharNoWait(err,lock,&granted)) {
            printf("Error [%s] in %s.\n",zkErrorFunction (err), zkErrorDetail (err));
            goto done;
        } else if (!granted) {
            printf("Shared lock not granted\n");
        } else {
            bWait = ZK_TRUE;
        }
    } else {
        printf("Wrong lock type.\n");
    }
    
    /* wait if granted */


    if (bWait) {
        if (sleepSecs > 0) {
            printf("Waiting %d seconds...\n",sleepSecs);
            sleep(sleepSecs);
        } else {
            printf("Waiting indefinitely...\n");
            pause();
        }
    }
    
    
    
    
    ok = 1;

    printf("...Done.\n");
  
 done: 
    /* release lock and exit */
    
    if (!zkLockRelease(err,lock)) {
        printf("Error [%s] in %s.\n",zkErrorFunction (err), zkErrorDetail (err));
    } else if (!zkLockDelete(err,&lock)) {
        printf("Error [%s] in %s.\n",zkErrorFunction (err), zkErrorDetail (err));
    }

    exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);

#else	/* !defined(ZK_CLIENT) */

	/* silence compiler */
	
	(void) ac;
	(void) av;
	
	return (EXIT_SUCCESS);
	
#endif	/* defined(ZK_CLIENT) */
}
