/* $Id: tgraph.c,v 1.5 2001/04/19 17:26:56 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include <assert.h>

#include "zkError.h"
#include "zkLog.h"
#include "zkIntTypes.h"
#include "zkGraph.h"
#include <assert.h>

int main (int argc ,char ** argv)
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkGraph Graph;
  uint32_t *pi = NULL;
  uint32_t *path = NULL;
  size_t psz, i;

  (void) argc;
  (void) argv;
  assert(zkErrorNew(&err));
  assert(zkErrorLog(err,&log));
  assert(zkLogSetStream(log,stderr));
  assert(zkLogEnable(log));

  assert(zkGraphNew(err, &Graph));

  assert(zkGraphAddNode(err, Graph, 1));
  assert(zkGraphAddNode(err, Graph, 2));
  assert(zkGraphAddNode(err, Graph, 3));

  assert(zkGraphEdgeAdd(err, Graph, 1, 2, 1));
  assert(zkGraphEdgeAdd(err, Graph, 1, 3, 2));
  assert(zkGraphEdgeAdd(err, Graph, 2, 1, 12));
  assert(zkGraphEdgeAdd(err, Graph, 2, 3, 23));
  assert(zkGraphEdgeAdd(err, Graph, 3, 1, 0));
  assert(zkGraphEdgeAdd(err, Graph, 3, 2, 5));

  assert(zkGraphDump(err, Graph));

  assert(zkGraphEnumerateAllPaths(err, Graph, 1, 3, &path, &psz));

  for (i = 0; i < psz; i++) {
    if (path[i] != 0) {
      zkFprintf(stderr, "-> %d", path[i]);
    } else {
      zkFprintf(stderr, "\n");
    }
  }
  
  assert(zkGraphDump(err, Graph));

  if (path) {
    zkFree(path);
    path = NULL;
  }

  zkFprintf(stderr, "Enumerating all 2 node paths\n");
  assert(zkGraphEnumerateXPaths(err, Graph, 1, 3, 2, &path, &psz));

  for (i = 0; i < psz; i++) {
    if (path[i] != 0) {
      zkFprintf(stderr, "-> %d", path[i]);
    } else {
      zkFprintf(stderr, "\n");
    }
  }
  
  assert(zkGraphDump(err, Graph));

  assert(zkGraphDeleteNode(err, Graph, 1));
  assert(zkGraphAddNode(err, Graph, 1));
  assert(zkGraphEdgeAdd(err, Graph, 1, 2, 1));
  assert(zkGraphDump(err, Graph));

  assert(zkGraphFree(err, &Graph));
  assert(zkErrorFree(&err));

  if (pi) {
    zkFree(pi);
    pi = NULL;
  }

  if (path) {
    zkFree(path);
    path = NULL;
  }

  return 1;
}
