/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkErrno.h"
#include "zkStdlib.h"
#include <assert.h>
#include "zkTime.h"
#include "zkUnistd.h"
#include "zkSysTypes.h"
#include "zkSysStat.h"
#include "zkFcntl.h"


#include "zkError.h"
#include "zkLog.h"

#include "zktimer.h"


#if !defined(ZK_CLIENT)

static void SpendTime() 
{
#if 1
  zkSleep(2);
#else  
  uint32_t loop;

  for( loop=0; loop<10000; loop++ ) {
    if( !(loop%1) ) zkPrintf("line %d ", loop);
  }
  zkPrintf("\n");
#endif
}


static int Run(ZkError err, ZkBool pause)
{
  ZkTimer timer;
  int     error = 0;
  double  realTime;
  double  userTime;
  double  sysTime; 


  if( !zkTimerNew(err, &timer) )
    {
      error = 1;
    }
  else
    {
      if( !zkTimerStart(err, timer) )
        {
          error = 1;
        }
      else
        {
          int i;

          for( i=0; i<5; i++ ) 
            {
              if( pause )
                {
                  if( !zkTimerPause(err, timer) )
                    {
                      error = 1;
                    }
                  else 
                    {
                      SpendTime();

                      if( !zkTimerResume(err, timer) )
                        {
                          error = 1;
                        }
                    }
                }
              else
                {
                  SpendTime();
                }
            }         
        }
      if( !error )
        {
          if( !zkTimerStop(err, timer) )
            {
              error = 1;
            }
        }
      if( !error )
        {
          if( !zkTimerProcessTimesGet(err, timer, &realTime, &userTime, &sysTime) )
            {
              error = 1;
            }          
          else
            {
              zkPrintf("\n[LIFETIME] real=%f, user=%f, sys=%f \n", realTime, userTime, sysTime);
            }          
        }
      if( !error )
        {
          if( !zkTimerPauseTimesGet(err, timer, &realTime, &userTime, &sysTime) )
            {
              error = 1;
            }          
          else
            {
              zkPrintf("[PAUSETIME] real=%f, user=%f, sys=%f \n", realTime, userTime, sysTime);
            }          
        }
    }
  return !error;
}

#endif	/* !defined(ZK_CLIENT) */


int main()
{                             
#if !defined(ZK_CLIENT)
  static const char *fn = "main";
  ZkError err = NULL;
  ZkLog   log; 

  zkErrorNew(&err);

  if(  !( zkErrorLog(err, &log) &&
          zkLogSetProgname(log, "TIMER test") &&
          zkLogSetStream(log,stdout) &&
          zkLogEnable(log) ) )
    {
      zkPrintf("main: can't set log object\n");
    }
  else 
    {
      if( !Run(err, ZK_FALSE) )
        {
          zkLogError(log, err);
        }
      else if( !Run(err, ZK_TRUE) )
        {
          zkLogError(log, err);
        }
    }
  zkErrorFree(&err);
#endif	/* !defined(ZK_CLIENT) */
  return 0;
}

