/*
 *  $Id: tlock.c,v 1.3 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Program to test file locking.
 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkUnistd.h"
#include "zkError.h"
#include "zkFile.h"
#include "zkLog.h"
#include "zkShell.h"


static const char *prog;


static void usage(int ok)
{
  zkFprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--logfile <path>]"
          " [--mode <fopen-mode>]"
          " [--sleep <secs>]"
          " <lock-file>"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  FILE *lfp = 0;
  ZkError err = 0;
  ZkLog log;
  char *sleepStr = 0;
  const char *lockFile = 0;
  const char *logFile = "/dev/tty";
  const char *mode = "w";
  int ok = 0;
  int sleepSecs = -1;

  prog = zkShellGetProgramName(&ac, &av);

  while (av++, --ac) {
    if (strcasecmp(av[0], "-h") == 0 ||
        strcasecmp(av[0], "-help") == 0 ||
        strcasecmp(av[0], "--help") == 0) {
      usage(1);
    } else if (strcasecmp(av[0], "--logfile") == 0) {
      if (av++, --ac < 1) { usage(0); }
      logFile = av[0];
    } else if (strcasecmp(av[0], "--mode") == 0) {
      if (av++, --ac < 1) { usage(0); }
      mode = av[0];
    } else if (strcasecmp(av[0], "--sleep") == 0) {
      if (av++, --ac < 1) { usage(0); }
      sleepStr = av[0];
    } else {
      break;
    }
  }

  if (ac != 1) {
    usage(0);
  }

  if (sleepStr &&
      (sleepSecs = atoi(sleepStr)) == 0 &&
      sleepStr[0] != '0') {
    usage(0);
  }

  lockFile = av[0];

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logFile) &&
          zkLogEnable(log))) {
    zkFprintf(stderr, "%s: can't set log object -- exiting.\n", prog);
    goto done;
  }

  if ( ! zkFileFopenLocked(err, lockFile, mode, &lfp)) {
    goto done;
  }

  if (sleepSecs > 0) {
    zkSleep(sleepSecs);
  } else {
#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
    pause();
#endif /* ZK_LINUX || ZK_SOLARIS */
  }

  ok = 1;
  
 done:

  if (lfp) { zkFclose(lfp); }
  if (err) { zkErrorFree(&err); }
  
  return (ok ? EXIT_SUCCESS : EXIT_FAILURE);
}
