/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

/*
 *  Test zkSelectUpdateTimeout().
 */

#include "zkSysTime.h"
#include "zkSysTypes.h"
#include <assert.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkError.h"
#include "zkNet.h"


int main(int ac, char **av)
{
  ZkError err;
  fd_set rd;
  int fd = zkFileno(stdin);
  struct timeval to;
  
#ifdef _WIN32
  WSADATA stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  WSAStartup(wVersionRequired, &stWSAData);
#endif

  /* silence compiler */
  (void) ac;
  (void) av;

  assert(zkErrorNew(&err));

  to.tv_sec = 60; to.tv_usec = 0;
  
  zkPrintf("Initial timeout set to %lu seconds.\n", (unsigned long)to.tv_sec);

  zkPrintf("Repeatedly hit return.\n");

  FD_ZERO(&rd);
  FD_SET(fd, &rd);
  while (zkSelectUpdateTimeout(err, ZK_FD_MAXFD1(fd), &rd, 0, 0, &to)) {
    zkPrintf("Time remaining: %lu, %lu\n",
           (unsigned long)to.tv_sec, (unsigned long)to.tv_usec);
    if (getchar() == EOF) { break; }
    FD_SET(fd, &rd);
  }
  
  assert(zkErrorFree(&err));
  
#ifdef _WIN32
  WSACleanup();
#endif

  return (EXIT_SUCCESS);
}
