/*
 *  $Id: tsigprot.c,v 1.2 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test blocking.
 */


#include <assert.h>
#include <signal.h>
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkSignal.h"


#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )

static volatile sig_atomic_t shup = 0;
static volatile sig_atomic_t sint = 0;


static void sig(int signo)
{
  switch (signo) {
  case SIGHUP: shup = 1; break;
  case SIGINT: sint = 1; break;

  case SIGALRM:
  default:
    assert(!"unexpected signal");
  }
}

#endif /* ZK_LINUX || ZK_SOLARIS */


int main(int ac, char **av)
{
#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
  /*
   *  Send a signal while in a `critical region', making sure it's caught only
   *  after we've exited the region.
   */

  ZkSignalSet block, restore;

  /* silence compiler */
  (void) ac;
  (void) av;
  
  zkSignalIntr(SIGALRM, sig);
  zkSignalIntr(SIGHUP, sig);
  zkSignalIntr(SIGINT, sig);

  assert(zkSignalSetInit(&block, SIGALRM, 0));
  assert(zkSignalSetAdd(&block, SIGINT));

  assert(shup == 0);
  assert(sint == 0);

  assert(zkSignalProtectBegin(block, &restore));
  raise(SIGHUP);
  assert(shup == 1);
  raise(SIGINT);
  assert(sint == 0);
  assert(zkSignalProtectEnd(restore));

  assert(sint == 1);
  
#else /* defined( ZK_LINUX ) || defined( ZK_SOLARIS ) */

  /* silence compiler */
  (void) ac;
  (void) av;
  
#endif /* !(defined( ZK_LINUX ) || defined( ZK_SOLARIS )) */

  return (EXIT_SUCCESS);
}
