/*
 *  $Id: ttempfile.c,v 1.3 2001/04/19 17:26:56 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test zkFileTempFp().
 */

#ifdef NDEBUG
#undef NDEBUG
#endif

#include "zks.h"

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "zkError.h"
#include "zkFile.h"
#include "zkLog.h"


static char *prog;


static char *tail(char *path)
{
  char *p = strrchr(path, '/'); /* bug: Windows */
  return p ? p+1 : path;
}


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " [-h|-help|--help]"
          " [--logFile <path>]"
          "\n", prog);
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


static void test0(ZkError err)
{
  FILE *fp;
  char file0[] = "/tmp/ttempfile.XXXXXX";
  char file1[] = "/tmp/foo";
  const char *fn = "test0";

  umask(0);

  /* Valid argument. */

  assert(zkFileTempFp(err, file0, &fp));
  assert(fclose(fp) != EOF);

  /* Ensure that the umask didn't change. */

  assert(umask(0) == 0);

  /* Invalid argument. */

  assert( ! zkFileTempFp(err, file1, &fp));

  /* Ensure that the umask didn't change. */

  assert(umask(0) == 0);

  printf("%s ok\n", fn);
}


static void test1(ZkError err)
{
  FILE *fp;
  char buf[100];
  char file0[] = "/tmp/ttempfile.XXXXXX";
  const char *fn = "test1";
  const char *s = "foo\n";
  struct stat sbuf;

  umask(0);

  assert(zkFileTempFp(err, file0, &fp));

  /* Check that we can read and write. */

  assert(fputs(s, fp) != EOF);
  assert(fflush(fp) != EOF);
  rewind(fp);
  assert(fgets(buf, sizeof buf, fp));
  assert(strcmp(buf, s) == 0);

  /* Check that the file has suitable permissions. */

  assert(stat(file0, &sbuf) != -1);
  assert((sbuf.st_mode & 0077) == 0);

  assert(fclose(fp) != EOF);

  printf("%s ok\n", fn);
}


int main(int ac, char **av)
{
  ZkError err;
  ZkLog log;
  const char *logFile = "/dev/tty";
  int ok = 0;

  prog = tail(av[0]);

  while (av++, --ac) {
    if (strcasecmp("-h",     av[0]) == 0 ||
        strcasecmp("-help",  av[0]) == 0 ||
        strcasecmp("--help", av[0]) == 0) {
      usage(1);
    } else if (strcasecmp("--logFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      logFile = av[0];
    } else {
      fprintf(stderr, "%s: invalid option `%s'.\n", prog, av[0]);
      usage(0);
    }
  }
  
  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logFile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: error initializing logging -- exiting.\n", prog);
    goto done;
  }

  test0(err);
  test1(err);

  ok = 1;

 done:
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
  return ok ? EXIT_SUCCESS : EXIT_FAILURE; /* keep insure happy */
}
