/************************************************************************

   File:		xmltest.c

   Author(s):		Louis Beaumier (originally frlodetest.c)
                        Modified Thierry to testxml.c Thierry Baron

   Created:		30 Sep 1999

   Last Revision:	$Date: 2001/04/19 17:26:57 $

   Description:	

   $Revision: 1.11 $

   Copyright (c) 1999 by Zero-Knowledge Systems, Inc
   Please see the copyright notice included in this distribution 
   for full details.

 ***********************************************************************/

#define XMLTEST_C

/**********************
     Include Files
 **********************/
#include "zks.h"

#include "zkUnistd.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"

#include "zkIntTypes.h"
#include "zkTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkShell.h"
#include "zkMemBuf.h"
#include "zkxml.h"

/**********************
    Private Defines
 **********************/

#define VERSION_STR "xml test program ("__DATE__", "__TIME__")"
#define MAX_CMD_STRING 1024
#define TEST_PROMPT "xml>"

/**********************
     Private Types
 **********************/
/*
** structure that holds data specific to the tested system
*/
typedef struct {
    ZkError    err;
    ZkXml      xml;
    ZkXml      subXml;
    char *     rtnBuf;
} SpecData;

/**********************
    Private Variables
 **********************/

/**********************
    Private Functions
 **********************/
static int 
cmdNewXmlObj(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;

    (void)shell;
    (void)token;
    (void)tokens;

    rtnCode = zkXmlNew( err, &(sd->xml));

    return rtnCode;
}

static int 
cmdNewSubXmlObj(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;

    (void)shell;
    (void)token;
    (void)tokens;

    rtnCode = zkXmlNew( err, &(sd->subXml));

    return rtnCode;
}

static int 
cmdFreeXmlObj(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;

    (void)shell;
    (void)token;
    (void)tokens;

    rtnCode = zkXmlFree( err, &(sd->xml));

    return rtnCode;
}

static int 
cmdPrintXml(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    char *lpchBuf = NULL;
    size_t lpchBufSize = 0;
    
    SpecData *sd = (SpecData*) userdata;
    /* allocate a buffer and fill it with  printable xml */
    rtnCode = zkXmlPrintToBuf( err, sd->xml, &lpchBuf, &lpchBufSize);

    if(lpchBuf) {
        zkPrintf("%s", lpchBuf);
        /* here need to free the buffer */
        zkFree(lpchBuf);
    }

    (void)shell; 
    (void)token; 
    (void)tokens; 

    return rtnCode;
}

static int 
cmdAppendData(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;

    (void)shell; 
    (void)token; 
    (void)tokens; 

    rtnCode = zkXmlAppendDataElement(err, sd->xml,
				     (ZkXmlValidTags)atoi(token[1]),
				     token[2]);
    return rtnCode;
}

static int 
cmdNew2Sub(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode = 1;
    SpecData *sd = (SpecData*) userdata;

    (void)err; 
    (void)shell; 
    (void)token; 
    (void)tokens;

    sd->subXml = sd->xml;
    sd->xml = NULL;

    return rtnCode;
}

static int 
cmdAppendSection(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;

    (void)shell; 
    (void)token;
    (void)tokens;
    
    rtnCode = zkXmlAppendSectionElement(err, 
					sd->xml,
					sd->subXml);
    return rtnCode;
}
static int 
cmdSetTag(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    SpecData *sd = (SpecData*) userdata;
    
    (void)shell; 
    (void)tokens;
    
    rtnCode = zkXmlSetSectionTag(err, 
				 sd->xml,
				 (ZkXmlValidTags)atoi(token[1]));
    return rtnCode;
}

static int 
cmdAllTagsUse(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode = 1;
    int iInd;
    char strTestString[256];
    
    SpecData *sd = (SpecData*) userdata;
    for(iInd = 0; iInd <= (int) entity && rtnCode; iInd++){
      zkSnprintf(strTestString,sizeof strTestString,"my test %5d", iInd);
      rtnCode = zkXmlAppendDataElement(err, sd->xml,
				       (ZkXmlValidTags) iInd,
				       strTestString);
      if(!rtnCode){
        zkPrintf("something bad for tag: %d\n", iInd);
      }
    }
    
    (void)shell; 
    (void)token; 
    (void)tokens; 
    return rtnCode;
}

static int 
cmdInsertXmlInRoot(
    ZkError err ,
    ZkShell shell ,
    char ** token ,
    size_t tokens ,
    void * userdata
)
{
    int rtnCode;
    ZkXml         newXml = NULL;
    
    
    SpecData *sd = (SpecData*) userdata;

    /* create New Xml Element */
    rtnCode = zkXmlNew( err, &newXml); /* newly created xml with NoTag */
    rtnCode = zkXmlSetSectionTag(err, newXml, TokenService); /* set the Tag */
    
    /* insert existing xml elemnt in the newly created one */
    rtnCode = zkXmlAppendSectionElement(err, newXml, sd->xml);
    
    /* store the pointer to the root document */
    sd->xml = newXml;
    
    (void)shell; 
    (void)token; 
    (void)tokens; 
    return rtnCode;
}


static int
defineShell(
    ZkError    err,
    ZkShell *  shell,
    SpecData * specData
)
{
    int rtnCode = 1;

    if (!(zkShellNew(err, shell) &&
	  zkShellSetContext(err, (*shell), specData) &&
	  zkShellSetPrompt(err, (*shell), TEST_PROMPT))) {
	zkErrorSet(err, "defineShell", ZKERR, ZKMLODE, ZKERROR,
		   "Unable to initialize the application shell");
	rtnCode = 0;
    }
    else if (!zkShellAddCommand(err, (*shell), cmdNewXmlObj,     "new",
				"new",
				"create xml object and set its tag",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdNewSubXmlObj,     "newsub",
				"newsub",
				"create sub xml object and set its tag",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdFreeXmlObj,     "free",
				"free",
				"free xml object",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdPrintXml,         "print",
				"print",
				"print xml content",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdAppendData,         "data",
				"data <tag no> <text data>",
				"append an element as data",
				ZKSHELL_PARMS_2
				) ||
	     !zkShellAddCommand(err, (*shell), cmdAppendSection,      "section",
				"section",
				"append an element as a section\n"
				"NOTE: a sub xml object must be created prior to this call",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdSetTag,         "settag",
				"settag <sectiontag>",
				"set section tag",
				ZKSHELL_PARMS_1
				) ||
	     !zkShellAddCommand(err, (*shell), cmdNew2Sub,         "new2sub",
				"new2sub",
				"move xml to subXml",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdAllTagsUse,     "alltags",
				"alltags",
				"use all xml tags",
				ZKSHELL_PARMS_NONE
				) ||
	     !zkShellAddCommand(err, (*shell), cmdInsertXmlInRoot,  "insertroot",
				"insertroot",
				"insert current elemnt in new root xml",
				ZKSHELL_PARMS_NONE
				)
	     ){
      /* failed to add command */
      rtnCode = 0;
    }
    return rtnCode;
}

/**********************
     Main

 **********************/
int
main(
    int     argc,
    char ** argv
)
{
    ZkError err = NULL;
    ZkLog log = NULL;

    const char * progname;
    /*    const char * logFile = NULL; */

    SpecData specData;

    ZkShell shell = NULL;
    
    progname = zkShellGetProgramName(&argc, &argv);

    /*
    ** preliminary settings
    */
    zkErrorNew(&err);
    zkErrorLog(err,&log);
    zkLogSetStream(log,stderr);
    zkLogEnable(log);

    /*
    ** fill the specific data structure
    */
    specData.err    = err;
    specData.xml    = NULL;
    specData.subXml = NULL;

    /*
    ** define the shell commands
    */
    if(!defineShell(err, &shell, &specData)) {
        /* problem with the shell definition */
    }
    else if (zkShellRun(specData.err, shell)) {
        /* everything is ok */
    }

    zkShellFree(specData.err, &shell);
    zkErrorFree(&err);
    
    /*
    ** Free used Data
    */
    if(specData.xml){
      zkXmlFree( err, &specData.xml);
    }
    
    return 1;
}



/***************************************************************************************

   $Log: xmltest.c,v $
   Revision 1.11  2001/04/19 17:26:57  alainc
   Merge from INT_FREEZE_MAC2_1_INT_20010417 to main trunk

   Revision 1.10.2.2  2001/01/25 03:14:52  paull
   Bringing in changes between tags BASE_SCARECROW_BETA2_0 and INT_SCARECROW_DEV_20010124_MAC2_1_INT to the mac2_1_int branch.

   Revision 1.8.32.3  2000/12/14 17:54:30  jean
   Merge from aip2 to scarecrow_dev

   Revision 1.8.32.2  2000/10/11 13:20:05  phil

   License header additions in preparation for the source release.

   Revision 1.8.32.1  2000/08/15 16:02:55  bill
   Security code audit: changed sprintf to snprintf.

   Revision 1.8.26.2  2000/09/29 20:21:46  jean
   Attempt to correct post-scarecrow merge inconsistencies.

   Merged latest SCARECROW_RSD sources (for libzk only) into the
   post-merge aip2 branch, and resolved conflicts.

   This allows the rest of the aip2 tree to build.

   Note that the AIP2_SC-RSD_POSTEMERGE_000929 tag describes libzk
   before these changes were committed (in case they need to be
   reverted).

   Revision 1.8.26.1  2000/09/29 13:29:04  build
   Merged from scarecrow_dev 0907

   Revision 1.8.32.1  2000/08/15 16:02:55  bill
   Security code audit: changed sprintf to snprintf.

   Revision 1.8  1999/10/07 21:14:02  thierry
   fix memory leak

   Revision 1.7  1999/10/06 15:47:48  thierry
   	* add zkXmlAppendDataLongElement
   	* fix unitialized pointer
   	* fix uninitialized value

   Revision 1.6  1999/10/05 15:22:18  louis
   -small modif to xmltest
   -correction of the print to buffer function

   Revision 1.5  1999/10/05 14:09:40  louis
   modification to the xml test program

   Revision 1.4  1999/10/05 13:54:26  thierry
   modify calls to libzkxml

   Revision 1.3  1999/10/01 22:18:06  thierry
   add libzk to makefile and modify xmltest

   Revision 1.2  1999/10/01 16:23:51  thierry
   	* error checking on tagString
   	* error checkink on new element/data insert
   	* remove unused functions
   	* access to set/get xml element tag

   Revision 1.1  1999/09/30 21:58:51  thierry
   test program for zkxml


*/
