/*
** **************************************************************************
** $Id: zkAuditEntry.c,v 1.5 2001/04/23 17:18:42 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

**
** Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
** **************************************************************************
*/

# include <zks.h>
# include <zkAlloc.h>
# include <zkAudit.h>
# include "zkAuditHelper.h"


/* --------------------------------------------------------------------------
 * zkAuditEntryFree
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryFree(
    ZkError          err,
    ZkAuditEntry *   audit_entry_ptr )
{
    static const char * func = "zkAuditEntryFree";


    zkErrorClear(err);

    if (!(audit_entry_ptr && *audit_entry_ptr))
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    zkMemFree(NULL, (void**)&((*audit_entry_ptr)->user));
    zkMemFree(NULL, (void**)&((*audit_entry_ptr)->host));
    zkMemFree(NULL, (void**)&((*audit_entry_ptr)->service));
    zkMemFree(NULL, (void**)&((*audit_entry_ptr)->request));

    if ((*audit_entry_ptr)->comment)
    {
        zkMemFree(NULL, (void**)&((*audit_entry_ptr)->comment));
    }

    if ((*audit_entry_ptr)->ticket_id)
    {
        zkMemFree(NULL, (void**)&((*audit_entry_ptr)->ticket_id));
    }

    if ((*audit_entry_ptr)->client_id)
    {
        zkMemFree(NULL, (void**)&((*audit_entry_ptr)->client_id));
    }

    if ((*audit_entry_ptr)->transaction_id)
    {
        zkMemFree(NULL, (void**)&((*audit_entry_ptr)->transaction_id));
    }

    if ((*audit_entry_ptr)->result_text)
    {
        zkMemFree(NULL, (void**)&((*audit_entry_ptr)->result_text));
    }

    zkMemFree(NULL, (void**)audit_entry_ptr);

    return (1);
}



/* --------------------------------------------------------------------------
 * zkAuditEntryGetSessionId
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetSessionId(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    uint32_t*           session_id_ptr )
{
    static const char * func = "zkAuditEntryGetSessionId";


    zkErrorClear(err);

    if ( !audit_entry || !session_id_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *session_id_ptr = audit_entry->session_id;

    return (1);
}



/* --------------------------------------------------------------------------
 * zkAuditEntryGetHost
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetHost(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    const char **       host_ptr )
{
    static const char * func = "zkAuditEntryGetHost";


    zkErrorClear(err);

    if ( !audit_entry || !host_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *host_ptr = audit_entry->host;

    return (1);
}


/* --------------------------------------------------------------------------
 * zkAuditEntryGetUser
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetUser(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    const char **       user_ptr )
{
    static const char * func = "zkAuditEntryGetUser";


    zkErrorClear(err);

    if ( !audit_entry || !user_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *user_ptr = audit_entry->user;

    return (1);
}



/* --------------------------------------------------------------------------
 * zkAuditEntryGetTicketId
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetTicketId(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    const char **       ticket_id_ptr )
{
    static const char * func = "zkAuditEntryGetTicketId";


    zkErrorClear(err);

    if ( !audit_entry || !ticket_id_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *ticket_id_ptr = audit_entry->ticket_id;

    return (1);
}



/* --------------------------------------------------------------------------
 * zkAuditEntryGetClientId
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetClientId(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    const char **       client_id_ptr )
{
    static const char * func = "zkAuditEntryGetClientId";


    zkErrorClear(err);

    if ( !audit_entry || !client_id_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *client_id_ptr = audit_entry->client_id;

    return (1);
}



/* --------------------------------------------------------------------------
 * zkAuditEntryGetTransactionId
 * --------------------------------------------------------------------------
 *
 *
 */
DLLExport int
zkAuditEntryGetTransactionId(
    ZkError             err,
    ZkAuditEntry        audit_entry,
    const char **       transaction_id_ptr )
{
    static const char * func = "zkAuditEntryGetTransactionId";


    zkErrorClear(err);

    if ( !audit_entry || !transaction_id_ptr )
    {
        zkErrorSet(err,func,ZKERR,ZKMAUDIT,ZKENULLPTR,
                   "input params are null");
        return (0);
    }

    *transaction_id_ptr = audit_entry->transaction_id;

    return (1);
}
