/*
 *  $Id: tchgpass.c,v 1.2 2001/04/19 17:26:26 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  Test generateSaltString(), via its public test interface.
 */

#include "zks.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "zkError.h"
#include "zkLog.h"
#include "zkauth.h"


static char *prog;


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " [-h | -help | --help]"
          " [--logfile <path>]"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  char salt[2];
  const char *logFile = "/dev/tty";
  const char *plotFile = "/tmp/tchgpass.out";
  int i;
  int plotCount = 2000;
  int ok = 0;
  FILE *fp;
  ZkError err;
  ZkLog log;

  prog = av[0];

  while (av++, --ac) {
    if (strcasecmp("-h",     av[0]) == 0 ||
        strcasecmp("-help",  av[0]) == 0 ||
        strcasecmp("--help", av[0]) == 0) {
      usage(1);
    } else if (strcasecmp("--logfile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      logFile = av[0];
    } else {
      fprintf(stderr, "%s: invalid option `%s' -- exiting.\n",
              prog, av[0]);
      goto done;
    }
  }

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logFile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: can't set log object.", prog);
    goto done;
  }
  
  if ( ! (fp = fopen(plotFile, "w"))) {
    zkLogWrite_va(log, "error %d opening `%s': %s\n",
                  errno, plotFile, strerror(errno));
    goto done;
  }

  for (i = 0; i < plotCount; i++) {
    if ( ! zkAuth_generateSaltString(err, salt)) {
      zkLogError(log, err);
      goto done;
    }
    if (fprintf(fp, "%u %u\n", (unsigned)salt[0], (unsigned)salt[1]) < 0) {
      zkLogWrite_va(log, "error %d printing to `%s': %s\n",
                    errno, plotFile, strerror(errno));
      goto done;
    }
  }

  if (fclose(fp) == EOF) {
    zkLogWrite_va(log, "error %d closing `%s': %s\n",
                  errno, plotFile, strerror(errno));
    goto done;
  }

  ok = 1;

 done:
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
  return ok ? EXIT_SUCCESS : EXIT_FAILURE; /* keep Insure happy */
}
