/* **************************************************************************
** $Id: zkcp.h,v 1.7 2001/04/19 17:04:08 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# ifndef ZKCP_INCLUDED
# define ZKCP_INCLUDED

# include "zkDefs.h"
# include "zkError.h"

# ifdef __cplusplus
extern "C" {
# endif


/* ------------------------------------------------------------------------
 * Constants
 *
 */

enum {
    /*
     * Pre-defined Message IDs
     */

    ZKCP_MESSAGE_ID_ProtocolErr = (0x00),

    /*
     * When defining application specific type identifiers you can use
     * an enumeration where the first identifier has the following
     * value.
     */
    ZKCP_MESSAGE_ID__custom__ = (0x01)
};

enum {
    /*
     * Pre-defined Parameter IDs
     */

    ZKCP_PARAM_ID_MAC = (0x00)
};


/* ------------------------------------------------------------------------
 * Type Definitions
 *
 */

typedef struct T_ZkcpContext          * ZkcpContext;
typedef struct T_ZkcpConnection       * ZkcpConnection;
typedef struct T_ZkcpCommand          * ZkcpCommand;
typedef struct T_ZkcpValue            * ZkcpValue;
typedef struct T_ZkcpValueConstructor * ZkcpValueConstructor;

typedef uint8_t                         ZkcpCommandId;
typedef uint8_t                         ZkcpParameterId;

typedef int (*ZkcpCallback_Read)(
    ZK_INOUT ZkError                error,
    ZK_IN    void *                 io_context,
    ZK_INOUT void *                 raw_buffer,
    ZK_IN    uint32_t               size );

typedef int (*ZkcpCallback_Write)(
    ZK_INOUT ZkError                error,
    ZK_IN    void *                 io_context,
    ZK_INOUT const void *           raw_buffer,
    ZK_IN    uint32_t               size );


/* ------------------------------------------------------------------------
 * These functions manipulate the context object
 *
 */

int zkcpContext_Create(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext *                          context );

int zkcpContext_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext *                          context );

int zkcpContext_AddType(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    ZkcpValueConstructor                   constructor );

int zkcpContext_SetKey(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    const void *                           key,
    ZK_IN    size_t                                 length );

int zkcpContext_SetMaxCommandSize(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    uint32_t                               max_size );

int zkcpContext_GetMaxCommandSize(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    uint32_t *                             max_size );

int
zkcpContext_SetIOBufferSize(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    uint32_t                               size );

int
zkcpContext_GetIOBufferSize(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    uint32_t *                             size );

int
zkcpContext_NewConnection(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpContext                            context,
    ZK_IN    void *                                 io_context,
    ZK_IN    ZkcpCallback_Read                      read_func,
    ZK_IN    ZkcpCallback_Write                     write_func,
    ZK_OUT   ZkcpConnection *                       out_connection );

/* ------------------------------------------------------------------------
 * These functions manipulate the connection object
 *
 */

int zkcpConnection_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_OUT   ZkcpConnection *                       connection );

int zkcpConnection_SetKey(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpConnection                         connection,
    ZK_IN    const void *                           key,
    ZK_IN    size_t                                 length );

int zkcpConnection_ReadCommand(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpConnection                         connection,
    ZK_OUT   ZkcpCommand*                           out_command );

int zkcpConnection_WriteCommand(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpConnection                         connection,
    ZK_IN    ZkcpCommand                            command );


/* ------------------------------------------------------------------------
 * These functions manipulate commands
 *
 */

int zkcpCommand_Create(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommandId                          command_id,
    ZK_OUT   ZkcpCommand *                          command );

int zkcpCommand_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_OUT   ZkcpCommand *                          command );

int zkcpCommand_GetId(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommand                            command,
    ZK_OUT   ZkcpCommandId *                        command_id );

int zkcpCommand_AddParam(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommand                            command,
    ZK_IN    ZkcpParameterId                        param_id,
    ZK_IN    ZkcpValue                              value );

int zkcpCommand_GetParamById(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommand                            command,
    ZK_IN    ZkcpParameterId                        param_id,
    ZK_OUT   ZkcpValue *                            value );

int zkcpCommand_GetParamByIndex(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommand                            command,
    ZK_IN    uint32_t                               param_index,
    ZK_OUT   ZkcpParameterId *                      param_id,
    ZK_OUT   ZkcpValue *                            value );

int zkcpCommand_GetParamCount(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpCommand                            command,
    ZK_IN    uint32_t *                             count );


/* ------------------------------------------------------------------------
 * Destructors for auxilliary objects
 *
 */

int zkcpValueConstructor_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor *                 constructor );

int zkcpValue_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_OUT   ZkcpValue *                            value );


# ifdef __cplusplus
}
# endif

# endif /*  ZKCP_INCLUDED */
