/* **************************************************************************
** $Id: zkcpValue_Binary.h,v 1.5 2001/04/19 17:04:08 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# ifndef zkcpValue_Binary_h_INCLUDED
# define zkcpValue_Binary_h_INCLUDED

# include "zkcp.h"

# ifdef __cplusplus
extern "C" {
# endif

/*
 * These function are the ones that most users will want to use.  They
 * allow you to work only with commands and user space data values.
 * 
 */

int zkcpValueConstructor_Binary_Create(
    ZK_INOUT ZkError                    error,
    ZK_OUT   ZkcpValueConstructor *     out_constructor );

int zkcpCommand_AddParamAsBinary(
    ZK_INOUT ZkError                    err,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_IN    const void *               buffer,
    ZK_IN    uint32_t                   size );

int zkcpCommand_GetParamAsBinary(
    ZK_INOUT ZkError                    err,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_OUT   const void * *             out_buffer,
    ZK_OUT   uint32_t *                 out_size );

/*
 * These functions are for people slightly more adventurous who want
 * to actually play with values directly (why?).  Remember that if
 * you create a value object and don't add it to a command that you're
 * responsible for destroying it with ZkcpValue_Destroy.  However, if
 * you HAVE added it to a command object then the command obect will
 * take care of destroying the value for you.
 *
 * The zkcpValue_???_Create function creates a ZkcpValue object
 * representing the application level object.
 *
 * The zkcpValue_As??? function perform a type checked "cast" on the
 * zkcpValue object, converting into the appropriate application
 * level object.
 */

int zkcpValue_Binary_Create(
    ZK_INOUT ZkError                    err,
    ZK_IN    const void *               in_blob,
    ZK_IN    size_t                     in_size,
    ZK_OUT   ZkcpValue *                out_value );

int zkcpValue_AsBinary(
    ZK_INOUT ZkError                    err,
    ZK_IN    ZkcpValue                  value,
    ZK_OUT   const void * *             out_blob,
    ZK_OUT   size_t *                   out_size );

# ifdef __cplusplus
}
# endif

# endif /* zkcpValue_Binary_h_INCLUDED */
