/* **************************************************************************
** $Id: zkcp_impl.h,v 1.3 2001/04/19 17:04:08 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for parties wishing to
** implement extensions to the Zero-Knowledge Command Protocol (ZKCP).
**
** **************************************************************************
*/

# ifndef zkcp_impl_h_INCLUDED
# define zkcp_impl_h_INCLUDED

# include "zkcp.h"

# ifdef __cplusplus
extern "C" {
# endif


enum {
    /*
     * Pre-defined Type IDs.
     */

    ZKCP_TYPE_ID_MAC       = (0x00),

    /*
     * The following definitions denote types which, for convenience,
     * have been included with this library implementation.
     */

    ZKCP_TYPE_ID_U32       = (0x01),
    ZKCP_TYPE_ID_String    = (0x02),
    ZKCP_TYPE_ID_Boolean   = (0x03),
    ZKCP_TYPE_ID_Binary    = (0x04),

    /*
     * When defining application specific type identifiers you can use
     * an enumeration where the first identifier has the following
     * value.
     */
    ZKCP_TYPE_ID__custom__ = (0x0F)
};



/* ------------------------------------------------------------------------
 * Type definitions for the ValueConstructor interface
 *
 */

typedef uint8_t                                     ZkcpTypeId;
typedef const struct T_ZkcpValueConstructor_vtbl *  ZkcpValueConstructor_vtbl;

struct T_ZkcpValueConstructor_vtbl {
    int (*Destroy)        ( ZkError                 error,
                            ZkcpValueConstructor *  constructor );

    int (*GetTypeId)      ( ZkError                 error,
                            ZkcpValueConstructor    constructor,
                            ZkcpTypeId *            out_type_id );

    int (*FromBuffer)     ( ZkError                 error,
                            ZkcpValueConstructor    constructor,
                            const void *            in_buffer_ptr,
                            uint32_t                in_buffer_size,
                            ZkcpValue *             out_new_value );
};

struct T_ZkcpValueConstructor {
    ZkcpValueConstructor_vtbl   vtbl;
};


/* ------------------------------------------------------------------------
 * Type definitions for the Value interface
 *
 */

typedef const struct T_ZkcpValue_vtbl * ZkcpValue_vtbl;

struct T_ZkcpValue_vtbl {
    int (*Destroy)        ( ZkError                 error,
                            ZkcpValue *             io_value );

    int (*GetTypeId)      ( ZkError                 error,
                            ZkcpValue               in_value,
                            ZkcpTypeId *            out_type_id );

    int (*GetSize)        ( ZkError                 error,
                            ZkcpValue               in_value,
                            uint32_t *              out_size );

    int (*WriteToBuffer)  ( ZkError                 error,
                            ZkcpValue               in_value,
                            void *                  io_buffer,
                            uint32_t                in_max_size,
                            uint32_t *              out_bytes_written );
};

struct T_ZkcpValue {
    ZkcpValue_vtbl   vtbl;
};

/* ------------------------------------------------------------------------
 * The following helper functions call the approprate "virtual"
 * function through the ZkcpValueConstrucor vtbl.  This means you
 * don't have to do all the pointer stuff yourself.
 *
 */

int zkcpValueConstructor_GetTypeId(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor                   constructor,
    ZK_OUT   ZkcpTypeId *                           type_id );

int zkcpValueConstructor_FromBuffer(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor                   constructor,
    ZK_IN    const void *                           in_buffer,
    ZK_IN    uint32_t                               in_buffer_size,
    ZK_OUT   ZkcpValue *                            out_new_value );


/* ------------------------------------------------------------------------
 * The following helper functions call the approprate "virtual"
 * function through the ZkcpValue vtbl.  This means you don't have
 * to do all the pointer stuff yourself.
 *
 */

int zkcpValue_GetTypeId (
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValue                              value,
    ZK_OUT   ZkcpTypeId *                           out_type_id );

int zkcpValue_GetSize(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValue                              value,
    ZK_OUT   uint32_t *                             out_size );

int zkcpValue_WriteToBuffer(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValue                              value,
    ZK_OUT   void *                                 buffer_ptr,
    ZK_IN    size_t                                 max_size,
    ZK_OUT   size_t *                               bytes_written );

# ifdef __cplusplus
}
# endif

# endif /*  zkcp_impl_h_INCLUDED */
