/* **************************************************************************
** $Id: zkcpValue.c,v 1.5 2001/04/19 17:04:11 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include "zkcp_impl.h"

static const char * const
_module_id = "$Id: zkcpValue.c,v 1.5 2001/04/19 17:04:11 alainc Exp $";


/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the Destroy member function through the vtable.
 *
 */
int
zkcpValueConstructor_Destroy(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor *                 constructor )
{
    const char * const func   = "zkcpValueConstructor_FromStream";
    int                result =  0;

    (void)_module_id; /* compiler complains about unreferenced variable */

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (*constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' doesn't point to anything");
    }
    else if ((*constructor)->vtbl->Destroy == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = (*constructor)->vtbl->Destroy(error,constructor);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the GetTypeId member function through the vtable.
 *
 */
int
zkcpValueConstructor_GetTypeId(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor                   constructor,
    ZK_OUT   ZkcpTypeId *                           out_type_id )
{
    const char * const func   = "zkcpValueConstructor_GetTypeId";
    int                result = 0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (out_type_id == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else if (constructor->vtbl->GetTypeId == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = constructor->vtbl->GetTypeId(error,constructor,out_type_id);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the FromBuffer member function through the vtable.
 *
 */
int
zkcpValueConstructor_FromBuffer(
    ZK_INOUT ZkError                                error,
    ZK_IN    ZkcpValueConstructor                   constructor,
    ZK_IN    const void *                           in_buffer,
    ZK_IN    size_t                                 in_buffer_size,
    ZK_OUT   ZkcpValue *                            out_new_value )
{
    const char * const func   = "zkcpValueConstructor_FromBuffer";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if ((in_buffer == NULL) && (in_buffer_size != 0))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'in_buffer' is NULL but 'in_buffer_size' isn't zero");
    }
    else if (out_new_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_new_value' is NULL");
    }
    else if (constructor->vtbl->FromBuffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = constructor->vtbl->FromBuffer( error,
                                                constructor,
                                                in_buffer,
                                                in_buffer_size,
                                                out_new_value );
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the Destroy member function through the vtable.
 *
 */
int
zkcpValue_Destroy(
    ZK_INOUT ZkError       error,
    ZK_OUT   ZkcpValue *   value )
{
    const char * const func   = "zkcpValue_Destroy";
    int                result = 0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (*value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' doesn't point to anything");
    }
    else if ((*value)->vtbl->Destroy == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = (*value)->vtbl->Destroy(error,value);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the GetTypeId member function through the vtable.
 *
 */
int
zkcpValue_GetTypeId (
    ZK_INOUT ZkError       error,
    ZK_IN    ZkcpValue     value,
    ZK_OUT   ZkcpTypeId *  out_type_id )
{
    const char * const func   = "zkcpValue_GetTypeId";
    int                result = 0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_type_id == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else if (value->vtbl->GetTypeId == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = value->vtbl->GetTypeId(error,value,out_type_id);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the GetSize member function through the vtable.
 *
 */
int
zkcpValue_GetSize(
    ZK_INOUT ZkError       error,
    ZK_IN    ZkcpValue     value,
    ZK_OUT   uint32_t *    out_size )
{
    const char * const func   = "zkcpValue_GetSize";
    int                result = 0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_size == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_size' is NULL");
    }
    else if (value->vtbl->GetSize == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = value->vtbl->GetSize(error,value,out_size);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 * This helper function wraps up the pointer manipulation needed to
 * call the WriteToBuffer member function through the vtable.
 *
 */
int
zkcpValue_WriteToBuffer(
    ZK_INOUT ZkError            error,
    ZK_IN    ZkcpValue          value,
    ZK_INOUT void *             buffer,
    ZK_IN    size_t             max_size,
    ZK_OUT   size_t *           bytes_written )
{
    const char * const func   = "zkcpValue_WriteToBuffer";
    int                result = 0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'buffer' is NULL");
    }
    else if (bytes_written == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'bytes_written' is NULL");
    }
    else if (value->vtbl->WriteToBuffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKESTUB,
                   "member function not implemented");
    }
    else
    {
        result = value->vtbl->WriteToBuffer(error,
                                            value,
                                            buffer,
                                            max_size,
                                            bytes_written);
    }

    return (result);
}
