/* **************************************************************************
** $Id: zkcpValue_Boolean.c,v 1.4 2001/04/19 17:04:11 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include "zkcp_impl.h"
# include "zkcpValue_Boolean.h"
# include "zkAlloc.h"

static const char * const _module_id = "$Id: zkcpValue_Boolean.c,v 1.4 2001/04/19 17:04:11 alainc Exp $";


/* ------------------------------------------------------------------------
 * Static and Member function prototypes
 */

static int value_type_is_Boolean(
    ZK_IN    ZkcpValue             value);

static int constructor_type_is_Boolean(
    ZK_IN    ZkcpValueConstructor   constructor);


static int zkcpValue_Boolean_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValue *            value );

static int zkcpValue_Boolean_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValue_Boolean_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   uint32_t *             out_size );

static int zkcpValue_Boolean_WriteToBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_INOUT void *                 buffer,
    ZK_IN    size_t                 max_size,
    ZK_OUT   size_t *               bytes_written );

static int zkcpValueConstructor_Boolean_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor );

static int zkcpValueConstructor_Boolean_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValueConstructor_Boolean_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    uint32_t               in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value );

/* ------------------------------------------------------------------------
 * Type infomation and function table
 */

typedef struct T_ZkcpValue_Boolean
{
    ZkcpValue_vtbl                  vtbl;
    ZkBool                          val;
} *
/* ---> */
/* ---> */ ZkcpValue_Boolean;
/* ---> */


typedef ZkcpValueConstructor ZkcpValueConstructor_Boolean;

static const struct
{
    ZkcpTypeId                                  type_id;
    const struct T_ZkcpValueConstructor_vtbl    constructor_vtbl;
    const struct T_ZkcpValue_vtbl               value_vtbl;
}
/* ---> */
/* ---> */ zkcpValue_Boolean_TypeInfo =
/* ---> */
{
    ZKCP_TYPE_ID_Boolean,
    {   zkcpValueConstructor_Boolean_Destroy,
        zkcpValueConstructor_Boolean_GetTypeId,
        zkcpValueConstructor_Boolean_FromBuffer },
    {   zkcpValue_Boolean_Destroy,
        zkcpValue_Boolean_GetTypeId,
        zkcpValue_Boolean_GetSize,
        zkcpValue_Boolean_WriteToBuffer },
};



/* ------------------------------------------------------------------------
 *
 */
static int
value_type_is_Boolean( ZkcpValue value )
{
    (void)_module_id; /* compiler complains about unreferenced variable */

    return (value->vtbl == &zkcpValue_Boolean_TypeInfo.value_vtbl);
}


/* ------------------------------------------------------------------------
 *
 */
static int
constructor_type_is_Boolean( ZkcpValueConstructor constructor )
{
    return (constructor->vtbl == &zkcpValue_Boolean_TypeInfo.constructor_vtbl);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_AddParamAsBoolean(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_IN    ZkBool                     true_false )
{
    const char * const func   = "zkcpCommand_AddParamAsBoolean";
    ZkcpValue          value  = NULL;
    int                result =  0;

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if (zkcpValue_Boolean_Create(error,true_false,&value)
             && zkcpCommand_AddParam(error,command,param_id,value))
    {
        value  = NULL;
        result = 1;
    }

    if (value != NULL)
    {
        zkcpValue_Destroy(NULL,&value);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_GetParamAsBoolean(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_OUT   ZkBool *                   out_true_false )
{
    const char * const func   = "zkcpCommand_GetParamAsBoolean";
    ZkcpValue          value  = NULL;
    int                result =  0;

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if (out_true_false == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_true_false' is NULL");
    }
    else if (zkcpCommand_GetParamById(error,command,param_id,&value)
             && zkcpValue_AsBoolean(error,value,out_true_false))
    {
        result = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_Boolean_Create(
    ZK_INOUT ZkError                 error,
    ZK_IN    ZkBool                  true_false,
    ZK_OUT   ZkcpValue *             out_value )
{
    const char * const func   = "zkcpValue_Boolean_Create";
    int                result =  0;

    if (out_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_value' is NULL");
    }
    else
    {
        ZkcpValue_Boolean new_value = NULL;

        (*out_value) = NULL;

        if (zkMemAlloc(error,(void**)&new_value,sizeof(*new_value)))
        {
            new_value->vtbl = &zkcpValue_Boolean_TypeInfo.value_vtbl;
            new_value->val  = (true_false != 0);

            (*out_value)    = (ZkcpValue) new_value;

            result          = 1;
        }
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_AsBoolean(
    ZK_INOUT ZkError           error,
    ZK_IN    ZkcpValue         value,
    ZK_OUT   ZkBool *          out_true_false )
{
    const char * const func   = "zkcpValue_AsBoolean";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_true_false == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_true_false' is NULL");
    }
    else if (!value_type_is_Boolean(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch");
    }
    else
    {
        ZkcpValue_Boolean p = (ZkcpValue_Boolean) value;

        (*out_true_false) = (p->val != 0);
        result = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int zkcpValue_Boolean_Destroy(
     ZK_INOUT ZkError       error,
     ZK_INOUT ZkcpValue *   value )
{
    const char * const func   = "zkcpValue_Boolean_Destroy";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (*value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' doesn't point to anything");
    }
    else if (!value_type_is_Boolean(*value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        result = zkMemFree(NULL,(void**)value);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Boolean_GetTypeId(
    ZkError       error,
    ZkcpValue     value,
    ZkcpTypeId *  out_type_id )
{
    const char * const func   = "zkcpValue_Boolean_GetTypeId";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_type_id == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else
    {
        if (!value_type_is_Boolean(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            (*out_type_id) = zkcpValue_Boolean_TypeInfo.type_id;
            result         = 1;
        }
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Boolean_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_OUT   uint32_t *             out_size )
{
    const char * const func   = "zkcpValue_Boolean_GetSize";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_size == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_size' is NULL");
    }
    else
    {
        (*out_size) = 0;

        if (!value_type_is_Boolean(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            (*out_size) = sizeof(uint8_t);
            result      = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_Boolean_WriteToBuffer(
    ZK_INOUT ZkError            error,
    ZK_IN    ZkcpValue          value,
    ZK_INOUT void *             buffer,
    ZK_IN    size_t             max_size,
    ZK_OUT   size_t *           bytes_written )
{
    const char * const func   = "zkcpValue_Boolean_WriteToBuffer";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'buffer' is NULL");
    }
    else if (bytes_written == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'bytes_written' is NULL");
    }
    else if (!value_type_is_Boolean(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else if (max_size < 1)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "insufficient buffer size");
    }
    else
    {
        uint8_t *          array = (uint8_t *) buffer;
        ZkcpValue_Boolean  data  = (ZkcpValue_Boolean) value;

        array[0] = data->val;

        (*bytes_written) = 1;
        result           = 1;
    }

    return (result);
}




/* ------------------------------------------------------------------------
 *
 */
int
zkcpValueConstructor_Boolean_Create(
    ZK_INOUT ZkError                error,
    ZK_OUT   ZkcpValueConstructor * out_constructor )
{
    const char * const func   = "zkcpValueConstructor_Boolean_Create";
    int                result =  0;

    if (out_constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_constructor' is NULL");
    }
    else
    {
        ZkcpValueConstructor_Boolean new_constructor = NULL;

        (*out_constructor) = NULL;

        if (zkMemAlloc(error,(void**)&new_constructor,sizeof(*new_constructor)))
        {
            new_constructor->vtbl = &zkcpValue_Boolean_TypeInfo.constructor_vtbl;
            (*out_constructor)    = (ZkcpValueConstructor) new_constructor;
            result                = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Boolean_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor )
{
    const char * const func   = "zkcpValueConstructor_Boolean_Destroy";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (*constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' doesn't point to anything");
    }
    else if (!constructor_type_is_Boolean(*constructor))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        result = zkMemFree(error,(void**)constructor);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Boolean_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id )
{
    const char * const func   = "zkcpValueConstructor_Boolean_GetTypeId";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (!constructor_type_is_Boolean(constructor))
    {
      zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                 "type mismatch (is vtbl set up properly?)");
    }
    else
    {
      (*out_type_id) = zkcpValue_Boolean_TypeInfo.type_id;
      result         = 1;
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_Boolean_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    size_t                 in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value )
{
    const char * const func   = "zkcpValue_Boolean_FromBuffer";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (in_buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'in_buffer' is NULL");
    }
    else if (out_new_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_new_value' is NULL");
    }
    else if (in_buffer_size < 1)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "insufficient data (buffer is too small)");
    }
    else
    {
        result = zkcpValue_Boolean_Create(error,
                                          *((const uint8_t *)in_buffer),
                                          out_new_value);
    }

    return (result);
}
