/* **************************************************************************
** $Id: zkcpValue_U32.c,v 1.5 2001/04/19 17:04:11 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include "zkcp_impl.h"
# include "zkcpValue_U32.h"
# include "zkAlloc.h"
# include "zknbo.h"

static const char * const
_module_id = "$Id: zkcpValue_U32.c,v 1.5 2001/04/19 17:04:11 alainc Exp $";


/* ------------------------------------------------------------------------
 * Static and Member function prototypes
 */

static int value_type_is_U32(
    ZK_IN    ZkcpValue             value);

static int constructor_type_is_U32(
    ZK_IN    ZkcpValueConstructor   constructor);

static int zkcpValue_U32_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValue *            value );

static int zkcpValue_U32_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValue_U32_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              in_value,
    ZK_OUT   uint32_t *             out_size );

static int zkcpValue_U32_WriteToBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_INOUT void *                 buffer,
    ZK_IN    size_t                 max_size,
    ZK_OUT   size_t *               bytes_written );

static int zkcpValueConstructor_U32_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor );

static int zkcpValueConstructor_U32_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id );

static int zkcpValueConstructor_U32_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    uint32_t               in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value );

/* ------------------------------------------------------------------------
 * Type infomation and function table
 */

typedef struct T_ZkcpValue_U32
{
    ZkcpValue_vtbl                  vtbl;
    uint32_t                        val;
} *
/* ---> */
/* ---> */ ZkcpValue_U32;
/* ---> */


typedef ZkcpValueConstructor ZkcpValueConstructor_U32;

static const struct
{
    ZkcpTypeId                                  type_id;
    const struct T_ZkcpValueConstructor_vtbl    constructor_vtbl;
    const struct T_ZkcpValue_vtbl               value_vtbl;
}
/* ---> */
/* ---> */ zkcpValue_U32_TypeInfo =
/* ---> */
{
    ZKCP_TYPE_ID_U32,
    {   zkcpValueConstructor_U32_Destroy,
        zkcpValueConstructor_U32_GetTypeId,
        zkcpValueConstructor_U32_FromBuffer },
    {   zkcpValue_U32_Destroy,
        zkcpValue_U32_GetTypeId,
        zkcpValue_U32_GetSize,
        zkcpValue_U32_WriteToBuffer },
};



/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_AddParamAsU32(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_IN    uint32_t                   number )
{
    const char * const func   = "zkcpCommand_AddParamAsU32";
    ZkcpValue          value  = NULL;
    int                result =  0;

    (void)_module_id; /* compiler complains about unreferenced variable */

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if (zkcpValue_U32_Create(error,number,&value)
             && zkcpCommand_AddParam(error,command,param_id,value))
    {
        value  = NULL;
        result = 1;
    }

    if (value != NULL)
    {
        zkcpValue_Destroy(NULL,&value);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
int
zkcpCommand_GetParamAsU32(
    ZK_INOUT ZkError                    error,
    ZK_INOUT ZkcpCommand                command,
    ZK_IN    ZkcpParameterId            param_id,
    ZK_OUT   uint32_t *                 out_number )
{
    const char * const func   = "zkcpCommand_GetParamAsU32";
    ZkcpValue          value  = NULL;
    int                result =  0;

    if (command == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'command' is NULL");
    }
    else if (out_number == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_number' is NULL");
    }
    else if (zkcpCommand_GetParamById(error,command,param_id,&value)
             && zkcpValue_AsU32(error,value,out_number))
    {
        result = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_U32_Create(
    ZK_INOUT ZkError                 error,
    ZK_IN    uint32_t                number,
    ZK_OUT   ZkcpValue *             out_value )
{
    const char * const func   = "zkcpValue_U32_Create";
    int                result =  0;

    if (out_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_value' is NULL");
    }
    else
    {
        ZkcpValue_U32 new_value = NULL;

        (*out_value) = NULL;

        if (zkMemAlloc(error,(void**)&new_value,sizeof(*new_value)))
        {
            new_value->vtbl = &zkcpValue_U32_TypeInfo.value_vtbl;
            new_value->val  = number;

            (*out_value)    = (ZkcpValue) new_value;

            result          = 1;
        }
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
int
zkcpValue_AsU32(
    ZK_INOUT ZkError           error,
    ZK_IN    ZkcpValue         value,
    ZK_OUT   uint32_t *        out_number )
{
    const char * const func   = "zkcpValue_AsU32";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_number == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_number' it NULL");
    }
    else if (!value_type_is_U32(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch");
    }
    else
    {
        ZkcpValue_U32 p = (ZkcpValue_U32) value;

        (*out_number) = p->val;
        result = 1;
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
value_type_is_U32( ZkcpValue value )
{
    return (value->vtbl == &zkcpValue_U32_TypeInfo.value_vtbl);
}


/* ------------------------------------------------------------------------
 *
 */
static int
constructor_type_is_U32( ZkcpValueConstructor constructor )
{
    return (constructor->vtbl == &zkcpValue_U32_TypeInfo.constructor_vtbl);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_U32_Destroy(
     ZK_INOUT ZkError       error,
     ZK_INOUT ZkcpValue *   value )
{
    const char * const func   = "zkcpValue_U32_Destroy";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (*value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' doesn't point to anything");
    }
    else if (!value_type_is_U32(*value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        result = zkMemFree(NULL,(void**)value);
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_U32_GetTypeId(
    ZkError       error,
    ZkcpValue     value,
    ZkcpTypeId *  out_type_id )
{
    const char * const func   = "zkcpValue_U32_GetTypeId";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_type_id == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_type_id' is NULL");
    }
    else
    {
        if (!value_type_is_U32(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            (*out_type_id) = zkcpValue_U32_TypeInfo.type_id;
            result         = 1;
        }
    }

    return (result);
}



/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_U32_GetSize(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValue              value,
    ZK_OUT   uint32_t *             out_size )
{
    const char * const func   = "zkcpValue_U32_GetSize";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (out_size == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_size' is NULL");
    }
    else
    {
        (*out_size) = 0;

        if (!value_type_is_U32(value))
        {
            zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                       "type mismatch (is vtbl set up properly?)");
        }
        else
        {
            (*out_size) = sizeof(uint32_t);
            result      = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValue_U32_WriteToBuffer(
    ZK_INOUT ZkError            error,
    ZK_IN    ZkcpValue          value,
    ZK_INOUT void *             buffer,
    ZK_IN    size_t             max_size,
    ZK_OUT   size_t *           bytes_written )
{
    const char * const func   = "zkcpValue_U32_WriteToBuffer";
    int                result =  0;

    if (value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'value' is NULL");
    }
    else if (buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'buffer' is NULL");
    }
    else if (bytes_written == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'bytes_written' is NULL");
    }
    else if (!value_type_is_U32(value))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else if (max_size < 4)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "insufficient buffer size");
    }
    else
    {
        uint8_t *      array = (uint8_t *)     buffer;
        ZkcpValue_U32  data  = (ZkcpValue_U32) value;

        zkNboFromHostUint32(array,data->val);

        (*bytes_written) = 4;
        result           = 1;
    }

    return (result);
}




/* ------------------------------------------------------------------------
 *
 */
int
zkcpValueConstructor_U32_Create(
    ZK_INOUT ZkError                error,
    ZK_OUT   ZkcpValueConstructor * out_constructor )
{
    const char * const func   = "zkcpValueConstructor_U32_Create";
    int                result =  0;

    if (out_constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_constructor' is NULL");
    }
    else
    {
        ZkcpValueConstructor_U32 new_constructor = NULL;

        (*out_constructor) = NULL;

        if (zkMemAlloc(error,(void**)&new_constructor,sizeof(*new_constructor)))
        {
            new_constructor->vtbl = &zkcpValue_U32_TypeInfo.constructor_vtbl;
            (*out_constructor)    = (ZkcpValueConstructor) new_constructor;
            result                = 1;
        }
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_U32_Destroy(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpValueConstructor * constructor )
{
    const char * const func   = "zkcpValueConstructor_U32_Destroy";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (*constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' doesn't point to anything");
    }
    else if (!constructor_type_is_U32(*constructor))
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "type mismatch (is vtbl set up properly?)");
    }
    else
    {
        result = zkMemFree(NULL,(void**)constructor);
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_U32_GetTypeId(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_OUT   ZkcpTypeId *           out_type_id )
{
    const char * const func   = "zkcpValueConstructor_U32_GetTypeId";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (!constructor_type_is_U32(constructor))
    {
      zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                 "type mismatch (is vtbl set up properly?)");
    }
    else
    {
      (*out_type_id) = zkcpValue_U32_TypeInfo.type_id;
      result         = 1;
    }

    return (result);
}


/* ------------------------------------------------------------------------
 *
 */
static int
zkcpValueConstructor_U32_FromBuffer(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpValueConstructor   constructor,
    ZK_IN    const void *           in_buffer,
    ZK_IN    size_t                 in_buffer_size,
    ZK_OUT   ZkcpValue *            out_new_value )
{
    const char * const func   = "zkcpValue_U32_FromBuffer";
    int                result =  0;

    if (constructor == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'constructor' is NULL");
    }
    else if (in_buffer == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'in_buffer' is NULL");
    }
    else if (out_new_value == NULL)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKENULLPTR,
                   "'out_new_value' is NULL");
    }
    else if (in_buffer_size < 4)
    {
        zkErrorSet(error,func,ZKERR,ZKMZKCP,ZKERROR,
                   "isufficient data (buffer is too small)");
    }
    else 
    {
        result = zkcpValue_U32_Create(error,
                                      zkNboToHostUint32(in_buffer),
                                      out_new_value);
    }

    return (result);
}
