/* **************************************************************************
** $Id: zkcp-test.c,v 1.6 2001/04/19 17:04:11 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains definitions and declarations for the Zero-Knowledge
** Command Protocol.
**
** **************************************************************************
*/

# include "zkcp.h"
# include "zkcpValue_U32.h"
# include "zkcpValue_String.h"
# include "zkcpValue_Binary.h"
# include "zkcpValue_Boolean.h"
# include <unistd.h>
# include <stdlib.h>
# include <stdio.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <ctype.h>
# include <time.h>

# define ARRAY_DIM(x)  (sizeof(x)/sizeof((x)[0]))

typedef struct {
    int fd;
} IO_Context;


const char * text_table[] = {
    "This is the first string",
    "This is the second string",
    "This is the third string",
    "Ok, so now what?",
    "Hmmm ... very interesting.",
    ""
};

const uint32_t blob1[] = {
    0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555,
    0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555,
    0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555,
    0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555, 0x55555555
};

const uint32_t blob2[] = {
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
    0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666, 0x66666666,
};

const char blob3[] = "blob full of characters";

struct {
    const void *   data;
    const uint32_t size;
} blob_table[] = {
    { blob1, sizeof(blob1) },
    { blob2, sizeof(blob2) },
    { blob3, sizeof(blob3) },
    { NULL, 0 },
    { blob1, 0 },
};



static int zkcpCallback_Read(
    ZK_INOUT ZkError                error,
    ZK_IN    void *                 raw_context,
    ZK_INOUT void *                 raw_buffer,
    ZK_IN    uint32_t               size )
{
    const char * const func            = "zkcpCallback_Read";
    IO_Context *       io_context      = (IO_Context*) raw_context;
    uint8_t *          buffer          = (uint8_t *) raw_buffer;
    uint32_t           bytes_remaining = size;
    ssize_t            bytes_read      = 0;
    int                result          = 0;
    
    while ( (bytes_remaining > 0)
            && ((bytes_read = read( io_context->fd,
                                    buffer + size - bytes_remaining,
                                    bytes_remaining)) > 0) )
    {
        bytes_remaining -= bytes_read;
    }
    
    if (bytes_remaining == 0)
    {
        result = 1;
    }
    else
    {
      zkErrorSet(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                 "failed to read data");
        
    }
    return (result);
}


static int zkcpCallback_Write(
    ZK_INOUT ZkError                error,
    ZK_IN    void *                 raw_context,
    ZK_INOUT const void *           raw_buffer,
    ZK_IN    uint32_t               size )
{
    const char * const func            = "zkcpCallback_Write";
    IO_Context *       io_context      = (IO_Context*) raw_context;
    const uint8_t *    buffer          = (const uint8_t *) raw_buffer;
    uint32_t           bytes_remaining = size;
    ssize_t            bytes_written   = 0;
    int                result          = 0;
    
    while ( (bytes_remaining > 0)
            && ((bytes_written = write( io_context->fd,
                                        buffer + size - bytes_remaining,
                                        bytes_remaining)) > 0) )
    {
        bytes_remaining -= bytes_written;
    }
    
    if (bytes_remaining == 0)
    {
        result = 1;
    }
    else
    {
      zkErrorSet(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                 "failed to write data");        
    }
    return (result);
}


static int add_random_parameter(
    ZK_INOUT ZkError                error,
    ZK_INOUT ZkcpCommand            command,
    ZK_IN    ZkcpParameterId        param_id )
{
    int result = 0;
    int type = rand() % 4;

    switch (type)
    {
    case 0:
        result = zkcpCommand_AddParamAsU32(
            error,
            command,
            param_id,
            rand());
        break;
    case 1:
        if (1)
        {
            int text_index = rand() % ARRAY_DIM(text_table);
            result = zkcpCommand_AddParamAsString(
                error,
                command,
                param_id,
                text_table[ text_index ] );
        }
        break;
    case 2:
        if (1)
        {
            int blob_index = rand() % ARRAY_DIM(blob_table);
            result = zkcpCommand_AddParamAsBinary(
                error,
                command,
                param_id,
                blob_table[blob_index].data,
                blob_table[blob_index].size);
        }
        break;
    case 3:
        result = zkcpCommand_AddParamAsBoolean(
            error,
            command,
            param_id,
            rand() % 2);
        break;
    }

    return (result);
}


static int dump_value(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpParameterId        id,
    ZK_IN    ZkcpValue              value )
{
    const char * const func = "dump_value";
    const char * data       = NULL;
    uint32_t     number     = 0;
    ZkBool       true_false = ZK_FALSE;
    int          result     = 1;

    if (zkcpValue_AsU32(error, value, &number))
    {
        fprintf(stdout,"Param %2d  : (U32) 0x%08x\n",id, number);
    }
    else if (zkcpValue_AsString(error,value, &data))
    {
        fprintf(stdout,"Param %2d  : (String) \"%s\"\n",id, data);
    }
    else if (zkcpValue_AsBinary(error,value, (const void**)&data, &number))
    {
        uint32_t counter = 0;
        fprintf(stdout,"Param %2d  : (Binary) Size = %u",id,number);
        for (counter = 0; counter < number; ++counter)
        {
            if ((counter % 10) == 0)
            {
                fprintf(stdout,"\n            ");
            }
            fprintf(stdout," %02x", data[counter]);
        }
        fprintf(stdout,"\n");
    }
    else if (zkcpValue_AsBoolean(error, value, &true_false))
    {
        fprintf(stdout,"Param %2d  : (Boolean) %s\n",id,
                (true_false ? "true" : "false"));
    }
    else
    {
        result = 0;
        zkErrorSet(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                   "unrecognized data type");
    }

    return (result);
}


static int dump_command(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpCommand            command )
{
    ZkcpCommandId   command_id = 0;
    uint32_t        num_params = 0;
    uint32_t        counter    = 0;
    int             result     = 0;

    if (!zkcpCommand_GetId(error,command,&command_id))
    {
        goto done;
    }

    fprintf(stdout,"Command ID : 0x%02x\n", command_id);

    if (!zkcpCommand_GetParamCount(error,command,&num_params))
    {
        goto done;
    }

    for (counter = 0; counter < num_params; ++counter)
    {
        ZkcpValue        value  = NULL;
        ZkcpParameterId  id     = 0;

        if (!zkcpCommand_GetParamByIndex(error,command,counter,&id,&value))
        {
            goto done;
        }

        if (!dump_value(error, id, value))
        {
            goto done;
        }
    }

    fprintf(stdout,"done.\n");

    result = 1;

 done:

    return (result);
}

static int create_command(
    ZK_INOUT ZkError                error,
    ZK_OUT   ZkcpCommand *          out_command )
{
    ZkcpCommand       command    = NULL;
    ZkcpValue         value      = NULL;
    ZkcpCommandId     cmd_id     = 0; 
    ZkcpParameterId   param_id   = 0;
    int               num_params = 0;
    int               result     = 0;

    srand(time(NULL));

    cmd_id = (ZkcpCommandId)((rand() & 254)+1);

    if (!zkcpCommand_Create(error, cmd_id, &command))
    {
        goto done;
    }

    num_params = rand() % 100;

    for (param_id = 1; param_id <= num_params; ++param_id)
    {
        if (!add_random_parameter(error,command,param_id))
        {
            goto done;
        }
    }

    if (!dump_command(error,command))
    {
        goto done;
    }

    (*out_command) = command;
    command        = NULL;
    result         = 1;

done:

    if (value)
    {
        zkcpValue_Destroy(NULL,&value);
    }

    if (command)
    {
        zkcpCommand_Destroy(NULL,&command);
    }

    return (result);
}


static int write_command(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpContext            zkcp_context,
    ZK_IN    const char * const     filename )
{
    const char * const func       = "write_command";
    ZkcpConnection     connection = NULL;
    ZkcpCommand        command    = NULL;
    int                result     = 0;
    IO_Context         io_context;

    memset(&io_context,0,sizeof(io_context));

    io_context.fd = creat(filename, S_IRUSR | S_IWUSR);

    if (io_context.fd < 0)
    {
        zkErrorSet_va(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                      "failed to open file (%s)",filename);
        goto done;
    }

    if (!zkcpContext_NewConnection( error,
                                    zkcp_context,
                                    &io_context,
                                    zkcpCallback_Read,
                                    zkcpCallback_Write,
                                    &connection ))
    {
        goto done;
    }

    if (!create_command(error,&command))
    {
        goto done;
    }
    
    if (!zkcpConnection_WriteCommand( error,
                                      connection,
                                      command ))
    {
        goto done;
    }
    
    result = 1;
    
 done:
    
    if (command)
    {
        zkcpCommand_Destroy(NULL,&command);
    }
    
    if (connection)
    {
        zkcpConnection_Destroy(NULL, &connection);
    }
    
    if (io_context.fd != -1)
    {
        close(io_context.fd);
    }
    
    return (result);
}


static int read_command(
    ZK_INOUT ZkError                error,
    ZK_IN    ZkcpContext            zkcp_context,
    ZK_IN    const char * const     filename )
{
    const char * const func       = "read_command";
    ZkcpConnection     connection = NULL;
    ZkcpCommand        command    = NULL;
    int                result     = 0;
    IO_Context         io_context;

    memset(&io_context,0,sizeof(io_context));

    io_context.fd = open(filename,O_RDONLY, 0);

    if (io_context.fd < 0)
    {
        zkErrorSet_va(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                      "failed to open file (%s)",filename);
        goto done;
    }

    if (!zkcpContext_NewConnection( error,
                                    zkcp_context,
                                    &io_context,
                                    zkcpCallback_Read,
                                    zkcpCallback_Write,
                                    &connection ))
    {
        goto done;
    }

    if (!zkcpConnection_ReadCommand( error,
                                     connection,
                                     &command ))
    {
        goto done;
    }
    
    if (!dump_command(error,command))
    {
        goto done;
    }
    
    result = 1;
    
 done:
    
    if (command)
    {
        zkcpCommand_Destroy(NULL,&command);
    }
    
    if (connection)
    {
        zkcpConnection_Destroy(NULL, &connection);
    }
    
    if (io_context.fd != -1)
    {
        close(io_context.fd);
    }
    
    return (result);    
}


static int create_context(
    ZK_INOUT ZkError                error,
    ZK_OUT   ZkcpContext *          out_context )
{
    const char * const   func                = "create_context";
    ZkcpContext          context             = NULL;
    ZkcpValueConstructor constructor_U32     = NULL;
    ZkcpValueConstructor constructor_String  = NULL;
    ZkcpValueConstructor constructor_Binary  = NULL;
    ZkcpValueConstructor constructor_Boolean = NULL;
    int                  result              = 0;
    char                 passphrase[128];

    fprintf(stdout,"Enter the session key: ");
    if (fgets(passphrase,sizeof(passphrase)-1,stdin) == NULL)
    {
        zkErrorSet(error,func,ZKWARN,ZKMZKCP,ZKERROR,
                   "failed to read session key");
        goto done;
    }
    else
    {
        int last = strlen(passphrase) - 1;

        while ((last > 0)
               && ((passphrase[last] == '\r')
                   || (passphrase[last] == '\n')))
        {
            passphrase[last--] = '\0';
        }
    }
    

    if (!zkcpValueConstructor_U32_Create(error,&constructor_U32)
        || !zkcpValueConstructor_String_Create(error,&constructor_String)
        || !zkcpValueConstructor_Binary_Create(error,&constructor_Binary)
        || !zkcpValueConstructor_Boolean_Create(error,&constructor_Boolean))
    {
        goto done;
    }

    if (!zkcpContext_Create(error,&context))
    {
        goto done;
    }

    if (!zkcpContext_AddType(error,context,constructor_U32)
        || !zkcpContext_AddType(error,context,constructor_String)
        || !zkcpContext_AddType(error,context,constructor_Binary)
        || !zkcpContext_AddType(error,context,constructor_Boolean))
    {
        goto done;
    }

    if (!zkcpContext_SetKey(error,context,passphrase,strlen(passphrase)))
    {
        goto done;
    }

    (*out_context)      = context;
    context             = NULL;
    constructor_U32     = NULL;
    constructor_String  = NULL;
    constructor_Binary  = NULL;
    constructor_Boolean = NULL;
    result              = 1;

    srand(time(NULL));

done:

    if (constructor_U32)
    {
        zkcpValueConstructor_Destroy(NULL,&constructor_U32);
    }

    if (constructor_String)
    {
        zkcpValueConstructor_Destroy(NULL,&constructor_String);
    }

    if (constructor_Binary)
    {
        zkcpValueConstructor_Destroy(NULL,&constructor_Binary);
    }

    if (constructor_Boolean)
    {
        zkcpValueConstructor_Destroy(NULL,&constructor_Boolean);
    }

    if (context)
    {
        zkcpContext_Destroy(NULL,&context);
    }

    return (result);
}


int main(int argc, const char * argv[])
{
    ZkError            error     = NULL;
    ZkcpContext        context   = NULL;
    int                result    = 0;

    if ( (argc < 3)
         || (argc > 4)
         || (argv[1][0] != '-')
         || (strlen(argv[1]) != 2)
         || ( ('r' !=  tolower(argv[1][1]))
              && ('w' != tolower(argv[1][1])) ) )
    {
        fprintf(stderr,"%s: Usage\n\t%s -[r|w] filename [n]\n\n", argv[0], argv[0]);
        exit(1);
    }

    if (!zkErrorNew(&error))
    {
        fprintf(stderr,"%s: Failed to create error object.\n", argv[0]);
        exit(1);
    }

    if (!create_context(error,&context))
    {
        goto done;
    }

    switch (argv[1][1])
    {
    case 'r':
    case 'R':
        result = 1;

        if (argc == 4)
        {
            result = zkcpContext_SetMaxCommandSize( error,
                                                    context,
                                                    atoi(argv[3]) );
        }

        if (result)
        {
            result = read_command(error,context,argv[2]);
        }
        break;

    case 'w':
    case 'W':
        result = write_command(error,context,argv[2]);
        break;

    default:
        fprintf(stderr, "%s: logic error\n",argv[0]);
    }

 done:

    if (!result)
    {
        char message[512];
        memset(message,0,sizeof(message));
        zkErrorMessage(error,message,sizeof(message));
        fprintf(stderr,"%s: %s\n",argv[0],message);
    }

    return !result;
}
