/* **************************************************************************
** $Id: zkCmdSrvCore.h,v 1.16 2001/04/19 17:26:06 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

** Copyright (C) 2000 Zero-Knowledge Systems Inc.  All rights reserved.
** --------------------------------------------------------------------------
**
** This file contains the definitions and declarations of the core functions
** of the ZK Command Server.
**
** **************************************************************************
*/

#ifndef ZKCMDSRVCORE_H_
#define ZKCMDSRVCORE_H_

#include <unistd.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkConfig.h>
#include <zkIntTypes.h>
#include <zksocket.h>
#include <zkCmdSrv.h>
#include <zkParms.h>

#ifdef ZKMOD
#undef ZKMOD
#endif

/* To allow child forks to return something more than just EXIT_FAILURE or EXIT_SUCCESS,
   command server will accept also other return codes. This codes will require the server to do something also in error handling */
   
#define ZK_EXIT_RESTART  71          /* restart child process */
                                 



#define ZKMOD ZKMCMDSRV
#define VERSION_COMMAND_SERVER "Zero-Knowledge Command Server ("__DATE__", "__TIME__")"

struct T_zkCmdSrv {
  ZkError             err;             /* Error object */
  ZkLog               log;             /* Log object */
  ZkCmdSrvConfig_s    userConfig;      /* User configurable options */
  uint8_t             debug;           /* debug or not */
  uint8_t             dontFork;        /* fork or not */
  uint8_t             tcpDelay;        /* Indicates if TCP_NODELAY socket option should be used */
  uint16_t            verbosityLevel;  /* Specify to type of information to log */
  FILE*               pidFptr;         /* Pointer to locked PID file */
  ZkSocket            socket;          /* Server socket for incoming connections */
  uint16_t            processCount;    /* Number of running processes. */
  uint16_t            maxConn;         /* Max number of simultaneous connections */
  uint32_t            processLife;     /* Max number of connections a process can handle */
  uint16_t            idleConnTimer;   /* Max duration of client connections (seconds) */
  uint16_t            inQueueLength;   /* Maximum number of incoming pending connections */
  uint16_t            maxPacketSize;   /* Maximum size of packets transmitted */
  char **             ipTable;         /* Table of allowed IP addresses */
  uint16_t            ipTableSize;     /* Number of addresses in the IP table */
  const char*         passphrase;      /* Passphrase for HMAC encryption */
  ZkCmdSrvCallback*   callbackTable;   /* Table for callback functions */
  ZkCmdSrvInitSession initSession;     /* Session (connection) initialzer */
  ZkCmdSrvTermSession termSession;     /* Session (connection) terminator */
  ZkCmdSrvInitProcess initProcess;     /* Callback function invoked when process starts */
  ZkCmdSrvTermProcess termProcess;     /* Callback function invoked when process terminates */
  ZkCmdSrvSignal      signalCallback;  /* Callback function for SIGURG signal */
};

/* New command line or config file option */
#define P_ACCESS_FILE     "accessFile"
#define P_AUTHENTICATION  "authKey"
#define P_MAX_IDLE_TIME   "maxIdleTime"
#define P_ADDRESS_PATH    "addressPath"
#define P_TCP_DELAY       "tcpDelay"
#define P_DUMP_OPTIONS    "dumpOptions"

/* Default values for some server configurable fields */
#define DEFAULT_CMDSRV_MAX_IDLE_CONN_TIME 30    /* Maximum idle connection time */
#define DEFAULT_CMDSRV_MAX_CONNECTIONS    50    /* Default max simult connects */
#define DEFAULT_CMDSRV_PROCESS_LIFE       50000 /* Default process life */
#define DEFAULT_CMDSRV_INQUEUE_LENGTH     32    /* Maximum incoming pending connections */
#define DEFAULT_CMDSRV_MAX_PACKET_SIZE    16    /* Maximum packet size (in kB) */

/* Command Server core functions */
extern int zkCmdSrv_RunDaemon(ZK_OUT ZkError err, 
			      ZK_IN  ZkCmdSrv cmdsrv);

#endif /* ZK_CMDSRVCORE_H_ */
