/*
 * $Id: zkipClient.h,v 1.11 2001/04/19 16:24:50 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKIPCLIENT_H_
#define ZKIPCLIENT_H_

#include "zks.h"                                                                                                                          

#include "zkIntTypes.h"   

#ifndef _ZKERROR_H
#include "zkError.h"
#endif /* _ZKERROR_H */

#include "zkSysTypes.h"
#include "zkSysSocket.h"
#include "zkNetinetIn.h"

#include "zkIntTypes.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define DATATYPE_SZ   32     /* Length of the datatype (token 4) */
#define MESSAGE_SZ   256     /* Maximum length of the message string */

struct ZkipCommandResponse {
  uint32_t transid;               /* The transaction identifier */
  uint32_t cmdid;                 /* The command identifier (see ZKIP.h) */
  uint32_t response;              /* The 3 digit response code (see ZKIP.h) */
  char datatype[DATATYPE_SZ + 1]; /* The datatype tag */
  uint16_t encoding;              /* The datatype encoding (see ZKIP.h) */
  uint32_t datalen;               /* The number of bytes of extra data */
  char message[MESSAGE_SZ + 1];   /* The response message */
};

#define MAX_PARMS 8
#define COMMAND_SZ 32
#define PARAMETER_SZ 64

struct ZkipCommandRequest {
  uint32_t transid;               /* Client assigned transaction id */
  uint32_t cmdid;                 /* Command identifier (see ZKIP.H) */
  char command[COMMAND_SZ + 1];   /* The command label */
  char datatype[DATATYPE_SZ + 1]; /* The data label (can be "NODATA") */
  uint16_t encoding;              /* The encoding type (see ZKIP.h) */
  uint32_t datalen;               /* Length of data (set to 0 if nodata) */
  int parameters;                 /* Number of command parameters */
  char parameter[MAX_PARMS][PARAMETER_SZ + 1]; /* array of parameter values */
};

typedef struct T_ZkipClient * ZkipClient;

DLLExport int zkipClientNew (
  ZkError ,
  ZkipClient * );

DLLExport int zkipClientFree (
  ZkError ,
  ZkipClient * );

DLLExport int zkipClientFd (
  ZkError ,
  ZkipClient ,
  int * );

DLLExport int zkipClientMakeNonBlocking (
  ZkError ,
  ZkipClient );

DLLExport int zkipClientMakeBlocking (
  ZkError ,
  ZkipClient ,              
  int );                     /* Timeout in seconds */


DLLExport int zkipClientConnectToHost (
  ZkError ,
  ZkipClient ,
  const char * ,              /* hostname of remote server */
  uint16_t );                 /* TCP port of remote server */

DLLExport int zkipClientConnectToAddress (
  ZkError ,
  ZkipClient ,
  struct sockaddr_in );       /* internet socket address of remote server */

DLLExport int zkipClientDisconnect (
  ZkError ,
  ZkipClient ); 


DLLExport int zkipClientVerifyResponse (
  ZkError ,
  ZkipClient );

DLLExport int zkipClientVerifyData (
  ZkError ,
  ZkipClient );

DLLExport int zkipClientIsReady (
  ZkError ,
  ZkipClient ,
  struct ZkipCommandResponse ** ,
  char ** );

DLLExport int zkipClientAddData (
  ZkError ,
  ZkipClient ,
  const char * ,
  size_t ,
  size_t * );

#if 0
DLLExport int zkipClientMakeCommand (
  ZkError ,
  ZkipClient ,
  struct ZkipCommandRequest * ,
  char * ,
  size_t * );
#endif

DLLExport int zkipClientWrite (
  ZkError ,
  ZkipClient ,
  const char * ,   /* Buffer to write */
  int );           /* Bytes of extra data */


DLLExport int zkipClientReadResponse (
  ZkError ,
  ZkipClient , 
  struct ZkipCommandResponse ** ,
  char ** );


DLLExport int zkipClientIsConnected( 
  ZkError err, 
  ZkipClient );





#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKIPCLIENT_H_ */
