/*
 * $Id: zkipSession.h,v 1.11 2001/04/19 16:24:50 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKIPSESSION_H
#define _ZKIPSESSION_H

#ifndef _ZKLOG_H
#include "zkLog.h"
#endif /* _ZKLOG_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef _T_ZKIPSESSION
#define _T_ZKIPSESSION
typedef struct T_ZkipSession * ZkipSession;
#endif /* _T_ZKIPSESSION */

/*
 * These defines control the logging level for a session, use them with the
 * zkipSessionSetVerbose() function.
 */

#define VERBOSE_OFF                  0  /* No verbose logging (default) */
#define VERBOSE_SESSION_LOGGING      5  /* Log session connect/close */
#define VERBOSE_CMDERR_LOGGING      10  /* Badly formed command line */
#define VERBOSE_COMMAND_LOGGING     15  /* Log the command and cmdid */
#define VERBOSE_DATA_LOGGING        20  /* Log # of data bytes expected */
#define VERBOSE_ANALDATA_LOGGING    80  /* Log all of the extra data */

int zkipSessionNew (
  ZkError ,
  ZkipSession * );

DLLExport int zkipSessionSetMaxData (
  ZkError ,                 /* Return the error if one occured */
  ZkipSession ,             /* Zkip session to modify */
  uint32_t );               /* Max bytes of data /command (set to 0 for off) */

DLLExport int
zkipSessionSetMaxConnTime (
  ZkError ,
  ZkipSession ,
  uint32_t );

int zkipSessionFree (
  ZkError ,
  ZkipSession * );

int zkipSessionSet (
  ZkError ,
  ZkipSession ,
  ZkipEngine ,
  ZkLog log ,
  int ,
  int ,
  void * );

int zkipSessionRun (
  ZkError ,
  ZkipSession );

int zkipSessionReadLine (
  ZkError ,
  ZkipSession );

int zkipSessionTokenizeLine (
  ZkError ,
  ZkipSession );

int zkipSessionParseLine (
  ZkError ,
  ZkipSession );

DLLExport int zkipSessionSetHostname (
  ZkError ,            /* Return the error if any occured */
  ZkipSession ,        /* Session to modify */
  char * );             /* Remote host name to which session is attached */

DLLExport int zkipSessionSetTimeout (
  ZkError ,            /* Return the error if one occured */
  ZkipSession ,        /* The session whose timeout you wish to change */
  int ,                /* Time out seconds */
  int );               /* Time out microseconds */

DLLExport int zkipSessionHeartbeatOn (
  ZkError ,
  ZkipSession );

DLLExport int zkipSessionHeartbeatOff (
  ZkError ,
  ZkipSession );


int zkipSessionResponse (
  ZkError ,
  ZkipSession ,
  uint32_t ,           /* Transaction ID.  0 will use whatever is in the session obj */
  uint16_t ,
  const char * ,      /* The response message string (in UTF8) */
  const char * ,      /* Datatype string token, TEXT, NYM, etc... */
  uint16_t ,                               /* Data encoding (ZKIP_ENCODING..) */
  uint32_t ,   /* Number of bytes in data buffer */
  const unsigned char * ); /* Data buffer to send back (optional) */

int zkipEngineSetCmdid (
  ZkError ,
  ZkipEngine ,
  uint32_t );

uint32_t zkipEngineCmdid (
  ZkError ,
  ZkipEngine );

DLLExport int zkipSessionSetCmdid (
  ZkError ,
  ZkipSession ,
  uint32_t );

DLLExport uint32_t zkipSessionCmdid (
  ZkError ,
  ZkipSession );

ZkipEngine zkipSessionEngine (
  ZkError ,
  ZkipSession );

int zkipSessionGetDataSize(
  ZkError,
  ZkipSession,
  size_t * );

int zkipSessionReadData (
  ZkError ,
  ZkipSession ,
  unsigned char ** ,
  unsigned int * );

DLLExport int zkipSessionSetVerbose (
  ZkError ,                           /* Return the error if any occured */
  ZkipSession ,                       /* Session to modify */
  unsigned int );                     /* Versbose level (0 is off) */


int zkipSessionParameters (
  ZkError ,                           /* Used to return the error */
  ZkipSession ,                       /* Session to analyze */
  int * );                            /* Returns number of parameters */

int zkipSessionParameter (
  ZkError ,                           /* Used to return the error */
  ZkipSession ,                       /* Session to analyze */
  int ,                               /* Parameter number (0, 1, 2 ... ) */
  const unsigned char ** );           /* Returns a pointer to parm value */


DLLExport int zkipSessionTerminateOnError (
  ZkError ,                           /* Returns the error if any occured */
  ZkipSession );                      /* Session to set to terminate on error */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKIPSESSION_H */
