/*
 * $Id: zkip.c,v 1.13 2001/04/19 16:24:51 alainc Exp $
 */

static const char * module_id_ = "$Id: zkip.c,v 1.13 2001/04/19 16:24:51 alainc Exp $";

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkIntTypes.h"
#include "zkString.h"
#include "ZKIP.h"
#include "zkipCommand.h"

int
ZKIP_CALLBACK(zkipCallbackPulse)
{
  int parameters = 0; /* Number of parameters passed */
  const unsigned char * tok = NULL;
  int seconds = 0;
  int microseconds = 0;

  (void) module_id_;
  data = NULL; /* This variable is not used by this function */
  zkErrorClear(err);
  zkipSessionParameters(err,session,&parameters);
  if (parameters != 2) {
    zkipSessionResponse(err, session, 0, _COMMAND_ERROR,
                        "You must specify seconds and microseconds", "NODATA",
                        ZKIP_ENCODING_NONE, 0, NULL);
    return _SESSION_ERROR;
  }
  zkipSessionParameter(err,session,0,&tok);
  seconds = atoi((const char *) tok);
  zkipSessionParameter(err,session,1,&tok);
  microseconds = atoi((const char *) tok);
  if ((seconds == 0) && (microseconds == 0)) {
    zkipSessionHeartbeatOff(err,session);
    zkipSessionResponse(err, session, 0, _SERVICE_READY,
                        "Heartbeat is OFF", "NODATA",
                        ZKIP_ENCODING_NONE, 0, NULL);
  } else {
    zkipSessionSetTimeout(err,session,seconds,microseconds);
    zkipSessionHeartbeatOn(err,session);
    zkipSessionResponse(err, session, 0, _SERVICE_READY,
                        "Heartbeat is ON", "NODATA",
                        ZKIP_ENCODING_NONE, 0, NULL);
  }  
  return _SESSION_CONTINUE;
}

int
ZKIP_CALLBACK(zkipCallbackEcho)
{
  unsigned char * datablock = NULL;
  unsigned int datalen = 0;

  data = NULL; /* this variable is not used by this function */

  zkErrorClear(err);
  if (!zkipSessionReadData(err,session,&datablock,&datalen)) {
    zkipSessionResponse(err, session, 0, _COMMAND_BADDATA,
                        "Incorrect data block", "NODATA",
                        ZKIP_ENCODING_NONE, 0, NULL);
    return _SESSION_ERROR;
  } else { 
    zkipSessionResponse(err, session, 0, _SERVICE_READY,
                        "Echoing data block", "BINARY",
                        ZKIP_ENCODING_8BIT, datalen, datablock);
    return _SESSION_CONTINUE;
  }
}

int 
ZKIP_CALLBACK(zkipCallbackQuit)
{
  data = NULL; /* this variable is not used by this function */

  zkErrorClear(err);

  zkipSessionResponse(err, session, 0, _SERVICE_CLOSING, 
                      "Server closing connection",
                      "NODATA", ZKIP_ENCODING_NONE, 0, NULL);
  return _SESSION_END;
}

int
ZKIP_CALLBACK(zkipCallbackList)
{
  ZkipCommand command = NULL;
  ZkipEngine engine = NULL;
  ZkString s = NULL;
  uint32_t cmdid;
  const char * token;
  const unsigned char * str = NULL;
  unsigned char buffer[256];
  unsigned int datalen = 0;
  
  data = NULL; /* this variable is not used by this function */

  zkErrorClear(err);
  if (!zkStringNew(err,&s)) {
    goto error;
  }
  engine = zkipSessionEngine(err,session);
  command = zkipEngineCommand(err,engine);
  while (command != NULL) {
    cmdid = zkipCommandCmdid(err,command);
    token = zkipCommandToken(err,command);
    if (token != NULL) {
      zkSnprintf((char *)buffer,
               sizeof buffer,
               "%d.%d.%d %s",
               (cmdid & 0xFF0000) / 0x10000,
               (cmdid & 0x00FF00) / 0x100,
               (cmdid & 0xFF),
               token);
    }
    zkStringAdd(err,s,buffer);
    command = zkipCommandNext(err,command);
    if (command != NULL) {
      zkStringAdd(err,s,(const unsigned char *) "\015\012");
    }
  }
  zkStringLength(err,s,&datalen);
  zkStringData(err,s,&str);
  if (!zkipSessionResponse(err, session, 0, _SERVICE_READY, "Command list", "TEXT",
                          ZKIP_ENCODING_BASE64, datalen,
                          (const unsigned char *) str)) {
    goto error;
  }
 
  zkStringFree(err,&s);
  
  return _SESSION_CONTINUE;

error:
  if (s) {
    zkStringFree(err,&s);
  }
  return _SESSION_ERROR;
}
