/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkSysTypes.h"
#include "zkSysStat.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "ZKIP.h"
#include "zkipClient.h"
#include "zkShell.h"

#ifdef _WIN32
#include <windows.h>
#endif

DLLExport int LoadTextFile ( ZkError , char * , char ** , int * );

DLLExport int
LoadTextFile (
  ZkError err,
  char * filename ,
  char ** buffer ,
  int * size )
{
  FILE * f = NULL;
  struct stat sbuf;

  zkErrorClear(err);
  if ((filename == NULL) || (buffer == NULL) || (size == NULL)) {
    zkErrorSet(err,"LoadTextFile",ZKERR,0,ZKENULLPTR,NULL);
    return 0;
  }
  if (zkStat(filename,&sbuf) != 0) {
    zkErrorSet(err,"LoadTextFile",ZKERR,0,ZKEFILEOPEN,NULL);
    return 0;
  }
  *buffer = (char *) zkMalloc(sbuf.st_size + 1);
  if (*buffer == NULL) {
    zkErrorSet(err,"LoadTextFile",ZKERR,0,ZKENOMEM,NULL);
    return 0;
  }
  f = zkFopen(filename,"rb");
/* #warning clean this up */
  if (f == NULL) {
    zkErrorSet(err,"LoadTextFile",ZKERR,0,ZKEFILEOPEN,filename);
    return 0;
  }
  *size = zkFread(*buffer,1,sbuf.st_size,f);
  zkFclose(f);
  (*buffer)[sbuf.st_size] = (char) 0;
  return 1;
}


int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkipClient * client = NULL;
  int i = 0; /* Loop index */
  int buffer_size = 0; /* Number of bytes for the echo test  */
  int connects = 0;
  char * buffer = "ECHOTEST";
  
#ifdef _WIN32
  WSADATA stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  WSAStartup(wVersionRequired, &stWSAData);
#endif

  zkShellGetProgramName(&argc, &argv);

  if (argc != 4) {
    zkFprintf(stderr,"Usage:  %s host port connects\n\n",argv[0]);
    exit(EXIT_FAILURE);
  }
  zkPrintf("connects...\n");

  connects = atoi(argv[3]);
  client = (ZkipClient *) zkMalloc(sizeof(ZkipClient) * connects);
  if (client == NULL) {
    perror("malloc");
    exit(EXIT_FAILURE);
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);


  for (i = 0; i < connects; i++) {
    zkFprintf(stderr,"Connecting session %d\n",i);
    if (!zkipClientNew(err,&(client[i]))) {
      zkFprintf(stderr,"zkipClientNew:  failed\n");
    }

    if (!zkipClientConnectToHost(err,client[i],argv[1],(uint16_t)atoi(argv[2]))) {
      zkFprintf(stderr,"zkipClientConnectToHost:  failed\n");
    }
  /*
    EchoTest do not exist
    
    if (!zkipClientEchoTest(err,client[i],buffer,strlen(buffer))) {
      zkFprintf(stderr,"zkipClientEchoTest:  failed\n");
    }
    */
  }

  for (i = 0; i < connects; i++) {
    /*
    echo test does not exist
    zkFprintf(stderr,"Disconnecting session %d\n",i); 
    if (!zkipClientEchoTest(err,client[i],buffer,strlen(buffer))) {
      zkFprintf(stderr,"zkipClientEchoTest:  failed\n");
    }
    */

    /*
     * Terminate the connection with the server
     */
  
    if (!zkipClientDisconnect(err,client[i])) {
      zkFprintf(stderr,"zkipClientDisconnect:  failed\n");
    }

    if (!zkipClientFree(err,&(client[i]))) {
      zkFprintf(stderr,"zkipClientFree:  failed\n");
    }
  }
   
#ifdef _WIN32
  WSACleanup();
#endif

  zkPrintf(".\n");
   
  exit(1);
  return 1;
}
