/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "ZKIP.h"
#include "zkipClient.h"

#define TEST_STR "FOOBAR.\n"

int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkipClient client = NULL;
  ZkipClientCommand command;
  int i = 0; /* Loop index */

  if (argc != 3) {
    zkFprintf(stderr,"Usage:  %s host port\n\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  zkipClientNew(err,&client);
  zkipClientConnectToHost(err,client,argv[1],atoi(argv[2]));
  zkipClientDump(err,client,stdout);

  /*
   *  Create a list command and send it.  The dump will display the results
   */

  memset(&command,0,sizeof(ZkipClientCommand));
  command.cmdid = CMDID(0,1,0);
  strcpy(command.command,"list"); /* audit: ok */
  strcpy(command.datatype,"nodata"); /* audit: ok */
  command.encoding = ZKIP_ENCODING_BINARY;
  command.data = NULL;
  command.datasize = 0;
  command.parameters = 1;
  strcpy(command.parameter[0],"This is parameter 1"); /* audit: ok */
  if (!zkipClientWriteCommand(err,client,command)) {
    zkPrintf("zkipClientWriteCommand:  failed\n");
  }
  zkipClientDump(err,client,stdout);

  if (!zkipClientWriteCommand(err,client,command)) {
    zkPrintf("zkipClientWriteCommand:  failed\n");
  }
  zkipClientDump(err,client,stdout);

  if (!zkipClientWriteCommand(err,client,command)) {
    zkPrintf("zkipClientWriteCommand:  failed\n");
  }
  zkipClientDump(err,client,stdout);

  if (!zkipClientWriteCommand(err,client,command)) {
    zkPrintf("zkipClientWriteCommand:  failed\n");
  }
  zkipClientDump(err,client,stdout);

  for (i = 0 ; i < 1000000; i++) {
    zkPrintf("echo test %d\n",i);
    if (!zkipClientEchoTest(err,client,TEST_STR,strlen(TEST_STR))) {
      zkPrintf("zkipClientEchoTest:  failed\n");
    }
/*
    zkipClientDump(err,client,stdout);
 */
  }

  /*
   * Terminate the connection with the server
   */

  if (!zkipClientDisconnect(err,client)) {
    zkPrintf("zkipClientDisconnect:  failed\n");
  }
  zkipClientDump(err,client,stdout);

  exit(1);
}
