/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkSysTypes.h"
#include "zkSysStat.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "ZKIP.h"
#include "zkipClient.h"
#include "zkShell.h"

#ifdef _WIN32
#include <windows.h>
#endif

int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkipClient client = NULL;
  int i = 0; /* Loop index */
  char * buffer = "ECHOTEST";
  int connects = 0;

#ifdef _WIN32
  WSADATA stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  WSAStartup(wVersionRequired, &stWSAData);
#endif

  zkShellGetProgramName(&argc, &argv);

  if (argc != 4) {
    zkFprintf(stderr,"Usage:  %s host port connects\n\n",argv[0]);
    exit(EXIT_FAILURE);
  }
  
  zkPrintf("wait...\n");

  connects = atoi(argv[3]);

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  for ( i = 0 ; i < connects ;i++) {
    zkPrintf("Connections %d\n",i);
    if (!zkipClientNew(err,&client)) {
      zkFprintf(stderr,"zkipClientNew:  failed\n");
    }

    if (!zkipClientConnectToHost(err,client,argv[1],(uint16_t)atoi(argv[2]))) {
      zkFprintf(stderr,"zkipClientConnectToHost:  failed\n");
    }
  /*
  while (1) {
    if (!zkipClientEchoTest(err,client,buffer,strlen(buffer))) {
      zkFprintf(stderr,"zkipClientEchoTest:  failed\n");
      break;
    }
    zkSleep(3);
  }
  */
 

    /*
     * Terminate the connection with the server
     */
  
    if (!zkipClientDisconnect(err,client)) {
      zkFprintf(stderr,"zkipClientDisconnect:  failed\n");
    }

    if (!zkipClientFree(err,&(client))) {
      zkFprintf(stderr,"zkipClientFree:  failed\n");
    }
  }
     
#ifdef _WIN32
  WSACleanup();
#endif

  zkPrintf(".\n");

  exit(1);
  return 1;
}
