/*
 * $Id: zkEmail.h,v 1.21 2001/04/19 17:03:33 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef ZKEMAIL_H_
#define ZKEMAIL_H_

#ifndef _ZKERROR_H
#include "zkError.h"
#endif /* _ZKERROR_H */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifndef ZKEMAIL_T
#define ZKEMAIL_T
typedef struct T_ZkEmail * ZkEmail;
#endif /* ZKEMAIL_T */

DLLExport int zkEmailNew (
  ZkError ,
  ZkEmail * );

DLLExport int zkEmailFree (
  ZkError ,
  ZkEmail * );

DLLExport int zkEmailAddText (
  ZkError ,
  ZkEmail ,
  const unsigned char * ,    /* Text buffer to add to the email object */
  int );                     /* Number of characters in the text buffer */


/*
 * This function allocates a buffer to store the freedom message.  you must
 * be sure to free it when it is no longer needed.  Once you have this buffer
 * you do not need to keep the email object, unless you have requested a 
 * pointer for the message's MIME header.  The function will automatically
 * convert the message from base 64 if required.
 */

DLLExport int zkEmailGetFreedomEncryptedMessage (
  ZkError ,                  /* Return the error if one occured */
  ZkEmail ,                  /* Email object to search */
  ZkEmailHeader * ,          /* Returns pointer to the header (if not NULL) */
  void **  ,                 /* Allocates a buffer which contains the message */
  size_t * );                /* Number of bytes in the message */

DLLExport int zkEmailGetFreedomPubkey (
  ZkError ,                  /* Return the error if one occured */
  ZkEmail ,                  /* Email object to search */
  ZkEmailHeader * ,          /* Returns pointer to the header (if not NULL) */
  void **  ,                 /* Allocates a buffer which contains the pubkey */
  size_t * );                /* Number of bytes in the message */

DLLExport int zkEmailGetFreedomFingerprint (
  ZkError ,                  /* Return the error if one occured */
  ZkEmail ,                  /* Email object to search */
  ZkEmailHeader * ,          /* Returns pointer to the header (if not NULL) */
  void **  ,                 /* Allocates a buffer which contains signature */
  size_t * );                /* Number of bytes in the message */

DLLExport int zkEmailGetFreedomRbActivationToken (
  ZkError ,                  /* Return the error if one occured */
  ZkEmail ,                  /* Email object to search */
  ZkEmailHeader * ,          /* Returns pointer to the header (if not NULL) */
  void ** ,                  /* Allocated a buffer which contains token */
  size_t * );                /* Number of bytes in the token */

DLLExport int zkEmailAddLineOfText (
  ZkError ,
  ZkEmail ,                    /* Email object to which to add the line */
  unsigned char * ,            /* A NUL terminated line of text */
  int );

DLLExport int zkEmailHeader (
  ZkError ,
  ZkEmail ,
  ZkEmailHeader * );

DLLExport int zkEmailBody (
  ZkError ,
  ZkEmail ,
  ZkEmailBody * );

DLLExport int zkEmailStripHeader (
  ZkError ,
  ZkEmail ,
  uint32_t );   /* Strip flags, see the zkMIME.h file for the list */

DLLExport int zkEmailGetMessageData (
  ZkError ,
  ZkEmail ,
  unsigned char **  ,
  size_t * );


DLLExport int zkEmailDump (
  ZkError ,
  ZkEmail  ,
  ZkEmailDumpCallback ,
  void * );

DLLExport int zkEmailStatus (
  ZkError ,
  ZkEmail ,
  int * );

DLLExport int zkEmailIsMIME (
  ZkError ,
  ZkEmail );

DLLExport int zkEmailGetMIMEPart (
  ZkError ,                   /* Return the error if any occured */
  ZkEmail ,                   /* The ZkEmail from which to pull out the part */
  ZkEmailBodyPart * );        /* Returns the first MIME part */

DLLExport int zkEmailParseAsMIME (
  ZkError ,
  ZkEmail );

DLLExport int zkEmailParse (
  ZkError ,
  ZkEmail );

DLLExport int zkEmailIsPartialMessage (
  ZkError ,                     /* Return the error if one occured */
  ZkEmail );                    /* The email structure to analyze */


DLLExport int zkEmailDup (
  ZkError ,
  ZkEmail ,
  ZkEmail * );

DLLExport int zkEmailConvertToMIME (
  ZkError ,
  ZkEmail );

DLLExport int zkEmailAddHtmlBlipvert (
  ZkError ,
  ZkEmail ,
  int ,   /* Set to non-zero for the blipvert to be the first MIME part */
  const unsigned char * ,                 
  int );

DLLExport int zkEmailAddTextBlipvert (
  ZkError ,
  ZkEmail ,
  int ,   /* Set to non-zero for the blipvert to be the first MIME part */
  const unsigned char * ,                 
  int );

DLLExport int zkEmailAddBlipvert (
  ZkError ,
  ZkEmail ,
  const char * ,                    /* MIME type string (eg text/html) */
  int ,   /* Set to non-zero for the blipvert to be the first MIME part */
  const unsigned char * ,           /* html text to add as a blipvert */
  int );                            /* Number of characters in the html text */


/*
 * Return a buffer which contains a text representation of the email.
 * You must zkFree() the buffer yourself when you no longer need it.
 */

DLLExport int zkEmailBuffer (
  ZkError ,                        
  ZkEmail  ,
  char **  ,
  size_t * );


/*
 *  This is a callback for the zkEmail...Dump() functions.  Do not call it
 *  directly, it is only to be used as a parameter.
 */

DLLExport int zkEmailWrite (
  ZkError ,
  void * ,
  const unsigned char * ,
  int );

DLLExport int zkEmailScanForSMTPTermination (
  ZkError ,
  ZkEmail );


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* ZKEMAIL_H_ */
