/*
 * $Id: zkEmailUtil.h,v 1.9 2001/04/19 17:03:33 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#ifndef ZKEMAIL_UTIL_H_
#define ZKEMAIL_UTIL_H_

#ifndef ZKLIST_H_
#include "zkList.h"
#endif /* ZKLIST_H_ */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Some behaviour flags for the zkEmailUtilSplitRecipientsByDomain()
 * function.
 */

#define FLAG_IGNORE_CASE 0x1  /* Ignores case when doing the domain compare */

/*
 * Email addresses are stored in the list as NUL terminated strings.  They
 * have been cleaned.  You can use the value string returned for the
 * To: and CC: and BCC: fields in an email header.
 */

DLLExport int zkEmailUtilAddToRecipientList (
  ZkError ,                   /* Return the error if any occured */
  ZkList ,                    /* The list to which email addresses are added */
  const char * );             /* The string which contains the addresses */

DLLExport int zkEmailUtilSplitRecipientsByDomain (
  ZkError ,                    /* Return the error if one occured */
  ZkList,                      /* Source list of recipients */
  ZkList * ,                   /* Returns a new list containing spec. addrs */
  const char * ,               /* NUL terminated domain str to look for */
  uint32_t );                  /* Behaviour flags (0 or FLAG_IGNORE_CASE) */

DLLExport int zkEmailUtilAddFieldToRecipientList (
  ZkError ,                   /* Return the error if one occured */
  ZkEmailHeader  ,            /* Email header from which to get data */
  enum ZkMIMEFieldId ,        /* Email field to look for */
  ZkList );                   /* List to hold recipients */


DLLExport int zkEmailUtilCleanAddress (
  ZkError ,                 /* Returns the error if any occured */
  char * );                 /* Email address to clean */

DLLExport int zkEmailUtilGenerateBoundaryString (
  ZkError ,                 /* Return the error if any occured */
  char * ,                  /* Pointer to the preallocated boundary buffer */
  int );                    /* size of the buffer */


DLLExport int zkEmailUtilGetMIMEPart (
  ZkError ,
  ZkEmail ,
  enum ZkMIMEContentType ,
  const char * ,
  ZkEmailBodyPart * );


DLLExport int
zkEmailUtilGetMIMEPartEx (
  ZkError ,
  ZkEmail ,                  /* the email from which to get the part */
  enum ZkMIMEContentType ,   /* the type of part you want to find */
  const char * [] , /* an array of subtype strings to find */
  int ,                      /* number of string in the preceding array */
  ZkEmailBodyPart * ,        /* output : matching body part */
  const char** );   /* output : matching subtype */


DLLExport int
zkEmailUtilAppendBlipvert (
  ZkError ,
  ZkEmail ,
  const unsigned char * ,   /* plain text to add as a blipvert */
  int ,                     /* Number of characters in the plain text */
  const unsigned char * ,   /* html text to add as a blipvert */
  int );                    /* Number of characters in the html text */



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZKEMAIL_UTIL_H_ */
