/*
 * $Id: zkMIME.h,v 1.24 2001/04/19 17:03:33 alainc Exp $
 */


#ifndef ZKMIME_H_
#define ZKMIME_H_

#include "zks.h"

#ifndef _ZKINTTYPES_H
#include "zkIntTypes.h"
#endif /* _ZKINTTYPES_H */

#ifndef _ZKERROR_H
#include "zkError.h"
#endif /* _ZKERROR_H */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define MIME_MESSAGE_PREAMBLE "\015\012\015\012Zero-Knowledge MIME Encapsulated Message\015\012\015\012"


typedef int (* ZkEmailDumpCallback)( ZkError , void * , const unsigned char * , int );

enum ZkMIMEContentType {
  MIMEContentTypeText,
  MIMEContentTypeImage,
  MIMEContentTypeAudio,
  MIMEContentTypeVideo,
  MIMEContentTypeApplication,
  MIMEContentTypeMultipart,
  MIMEContentTypeMessage
};

enum ZkMIMEFieldId {
  UnknownField,                /* Unknown field name */
  Extension,              /* Extension field name */
  X_ZKS_Freedom_Comment,            /* For adding comments via Freedom */
  X_ZKS_Freedom_MessageID,
  Return_Path,
  Received,
  Reply_To,
  Sender,
  From,
  Resent_From,
  Resent_Sender,
  Resent_Date,
  To,
  Resent_To,
  cc,
  Resent_cc,
  bcc,
  Resent_bcc,
  Message_ID,
  Resent_Message_ID,
  In_Reply_To,
  References,
  Keywords,
  Subject,
  Comments,
  Encrypted,
  MIME_Version,                     /* MIME Version field */
  Content_Type,                     /* Content type of a MIME message */
  Content_Transfer_Encoding,        /* Encoding type of a MIME message */
  Content_Disposition,
  Content_Base,
  Content_Conversion, 
  Content_Description, 
  Content_ID, 
  Content_Language, 
  Content_Length, 
  Content_Location,
  Content_MD5,
  Content_Return,
  Content_SGML_Entity,
  Autoforwarded,  
  Content_Identifier,  
  Conversion,  
  Conversion_With_Loss,  
  Deferred_Delivery,  
  Delivery_Date,  
  Discarded_X400_IPMS_Extensions,  
  Discarded_X400_MTS_Extensions,  
  DL_Expansion_History,  
  Expiry_Date,  
  Importance,  
  Incomplete_Copy,  
  Language,  
  Latest_Delivery_Time,  
  Newsgroups ,
  Path ,
  Followup_To,
  Control ,
  Distribution ,
  Organization ,
  Summary ,
  Approved ,
  Lines ,
  XRef ,
  Message_Type,  
  Obsoletes,  
  Original_Encoded_Information_Types,  
  Originator_Return_Address,  
  Priority,  
  Reply_By,  
  Requested_Delivery_Method,  
  Sensitivity,  
  X400_Content_Type,  
  X400_MTS_Identifier,  
  X400_Originator,  
  X400_Received,  
  X400_Recipients,
  Date,                       /* HTTP 1.0 Field */
  Pragma,                     /* HTTP 1.0 Field */
  Authorization,              /* HTTP 1.0 Field */
  If_Modified_Since,          /* HTTP 1.0 Field */
  If_Unmodified_Since,        /* HTTP 1.1 Field */
  Referer,                    /* HTTP 1.0 Field */
  User_Agent,                 /* HTTP 1.0 Field */
  Location,                   /* HTTP 1.0 Field */
  Server,                     /* HTTP 1.0 Field */
  WWW_Authenticate,           /* HTTP 1.0 Field */
  Allow,                      /* HTTP 1.0 Field */
  Content_Encoding,           /* HTTP 1.0 Field */
  Expires,                    /* HTTP 1.0 Field */
  Last_Modified,              /* HTTP 1.0 Field */
  Accept,                     /* HTTP 1.0 Field */
  Accept_Charset,             /* HTTP 1.0 Field */
  Accept_Encoding,            /* HTTP 1.0 Field */
  Accept_Language,            /* HTTP 1.0 Field */
  Link,                       /* HTTP 1.0 Field */
  Retry_After,                /* HTTP 1.0 Field */
  Title,                      /* HTTP 1.0 Field */
  URI,                        /* HTTP 1.0 Field */
  Host,                       /* HTTP 1.1 Field */
  Cookie,                     /* RFC2109 Cookie field */
  Set_Cookie,                 /* RFC2109 Cookie field */
  If_Match,                   /* HTTP 1.1 Field */
  If_None_Match,              /* HTTP 1.1 Field */

  /* E-mail notification fields */

  Disposition,                         /* RFC2298 Field */
  Disposition_Notification_To,         /* RFC2298 Field */
  Disposition_Notification_Options,    /* RFC2298 Field */
  Reporting_UA,                        /* RFC2298 Field */
  MDN_Gateway,                         /* RFC2298 Field */
  Original_Message_ID,                 /* RFC2298 Field */
  Original_Recipient,                  /* RFC2298 Field */
  Final_Recipient,                     /* RFC2298 Field */

  /* better add these at the end otherwise dire consequences might ensue */

  X_ZKS_Freedom_Header_Sig,
  X_ZKS_Freedom_Auth_From,
  X_ZKS_Freedom_Auth_To,
  X_ZKS_Freedom_Auth_Cc, 
  X_ZKS_Freedom_Auth_Newsgroups,
  X_ZKS_Freedom_Auth_Date
};

#define ZKEMAILBODYPART_T
#define ZKEMAIL_T
#define ZKEMAILBODY_T
#define ZKEMAILHEADER_T
#define ZKEMAILFIELD_T
#define ZKHTTPFIELD_T
#define ZKHTTPHEADER_T
typedef struct T_ZkEmailBodyPart * ZkEmailBodyPart;
typedef struct T_ZkEmail * ZkEmail;
typedef struct T_ZkEmailField * ZkEmailField;
typedef struct T_ZkEmailHeader * ZkEmailHeader;
typedef struct T_ZkEmailBody * ZkEmailBody;
typedef struct T_ZkHTTPHeader * ZkHTTPHeader;
typedef struct T_ZkEmailField * ZkHTTPField;

/*
 * Possible codes returned by zkEmailStatus() and zkHTTPHeaderStatus()
 */

#define PARSE_HEADER 1         /* Currently updating / parsing message header */
#define PARSE_BODY   2         /* Currently updating / parsing message body */
#define PARSE_ERROR  3         /* Encountered a parse error */
#define PARSE_COMPLETE 4       /* A complete email message has been received */
#define PARSE_HEADER_COMPLETE 5 /* A complete http header has been received,
                                 * Now scanning the body of the http transaction
                                 */
#define PARSE_NOHEADER 98      /* No header type specified (HTTP headers) */
#define PARSE_UNKNOWN 99       /* Unrecognized header (HTTP headers) */

/*
 * Control flags for the zkEmailStripHeader(), zkEmailHeaderStripField() and
 * zkEmailFieldStrip() functions.
 */

#define DONT_STRIP_UNKNOWN 0x1  /* Leave unknown fields in the header */
#define DONT_STRIP_NEWSGROUPS 0x2 /* Dont remove the Newsgroups field (this 
                                   * is for dealing with usenet message which 
                                   * are sent over the mail2news gateway */

/*
 * Some string values for various things
 */

/*
 * This is the content type of a MIME part which contains a freedom encrypted
 * message
 */

#define CT_FREEDOM_ENCRYPTED_MESSAGE "application/x-zks-Freedom-Encrypted-Mesg"
#define CT_FREEDOM_PUBKEY "application/x-zks-Freedom-Pubkey-1-0"
#define CT_FREEDOM_FINGERPRINT "application/x-zks-Freedom-Fingerprint-1-0"
#define CT_FREEDOM_RBACTIVATION "application/x-zks-Freedom-ReplyBlock-Activation-Token"

DLLExport char * 
generateBoundaryString (
  char * boundary ,
  int size );


#ifdef __cplusplus
}
#endif /* __cplusplus */


#ifndef ZKEMAILFIELD_H_
#include "zkEmailField.h"
#endif /* ZKEMAILFIELD_H_ */

#ifndef ZKEMAILHEADER_H_
#include "zkEmailHeader.h"
#endif /* ZKEMAILHEADER_H_ */

#ifndef ZKEMAILBODY_H_
#include "zkEmailBody.h"
#endif /* ZKEMAILBODY_H_ */

#ifndef ZKEMAIL_H_ 
#include "zkEmail.h"
#endif /* ZKEMAIL_H_ */

#ifndef ZKEMAILBODYPART_H_
#include "zkEmailBodyPart.h"
#endif /* ZKEMAILBODYPART_H_ */

#ifndef ZKHTTPFIELD_H_
#include "zkHTTPField.h"
#endif /* ZKHTTPFIELD_H_ */

#ifndef ZKHTTPHEADER_H_
#include "zkHTTPHeader.h"
#endif /* ZKHTTPHEADER_H_ */


#endif /* ZKMIME_H_ */
