/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkMIME.h"
#include "zkChunk.h"

int
main (
  int argc ,
  char ** argv )
{
#define TEST_SZ 30000
#define CHUNK_SZ 700
  int i = 0;
  int ret = 0;
  char buffer1[TEST_SZ];
  char * buffer2 = NULL;
  size_t buffer2sz = 0;
  int offset = 0;
  ZkError err  = NULL;
  ZkChunk chunk = NULL;
  ZkLog log = NULL;
  
  /* silence compiler */
  (void) argc;
  (void) argv;
  
  for (i = 0; i < TEST_SZ; i++) {
    buffer1[i] = rand()%26 + 'A';
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);
  zkChunkNew(err,&chunk);
  
  while (offset < TEST_SZ) {
    if ((TEST_SZ - offset)  < CHUNK_SZ) {
      ret = zkChunkAppend(err,chunk,&(buffer1[offset]),TEST_SZ - offset);
    } else {
      ret = zkChunkAppend(err,chunk,&(buffer1[offset]),CHUNK_SZ);
    }
    if (ret == 0) {
      zkPrintf("zkChunkAppend: failed\n");
    }
    offset += CHUNK_SZ;
  }
  zkPrintf("OFFSET iS %d\n",offset);
  if (!zkChunkGetData(err,chunk,&buffer2,&buffer2sz)) {
    zkPrintf("zkChunkGetData:  failed\n");
  }
  if (buffer2sz != TEST_SZ) {
    zkPrintf("Sizes do not match (source %d returned %d)\n",TEST_SZ,buffer2sz);
  }
  if (memcmp(buffer1,buffer2,buffer2sz) != 0) {
    zkPrintf("Memory contents do not match\n");
  }
  zkChunkFree(err,&chunk);

  return (EXIT_SUCCESS);
}
     
  

