/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkSysTypes.h"
#include "zkFcntl.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkMIME.h"
#include "zkHTTPHeader.h"
#include "zkShell.h"

#define LINE_SZ 32000

int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  const char * progname = NULL;
  char line[LINE_SZ + 1];
  size_t size = 0;
  FILE * in = stdin;
  ZkHTTPHeader header = NULL;
  int exit_code = EXIT_FAILURE;
  int status = 0;
  int opt = 1;
  int f = 0;
  char * filename = NULL;

  progname = zkShellGetProgramName(&argc, &argv);

  /*
   * Enable auto debugging 
   */

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetProgname(log,progname);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if(!zkHTTPHeaderNew(err,&header)) {
    exit(exit_code);
  }
  if(!zkHTTPHeaderSetType(err,header,REQUEST_HEADER)) {
    exit(exit_code);
  }

  while (opt < argc) {
    if (strcmp(argv[opt],"--keep") == 0) {
      if(!zkHTTPHeaderKeepBodyData(err,header)) {
        exit(exit_code);
      }
    } else if (strcmp(argv[opt],"--dontkeep") == 0) {
      /* Do diddly */
    } else if (strcmp(argv[opt],"--filename") == 0) {
      filename = argv[++opt];
    } else if (strcmp(argv[opt],"--help") == 0) {
       zkFprintf(stderr,"usage: %s --keep | --dontkeep [ --help ]\n\n",argv[0]);
    } else {
      zkFprintf(stderr,"%s:  unrecognized option %s\n",argv[0],argv[opt]);
    }
    opt++;
  }

  f = zkOpen(filename,O_RDONLY);
  size = zkRead(f,line,LINE_SZ);
  zkClose(f);
  

    if (!zkHTTPHeaderAddText(err,header,(unsigned char *) line,size)) {
      exit(exit_code);
    }
    if (!zkHTTPHeaderStatus(err,header,&status)) {
      exit(exit_code);
    }
    if (status == PARSE_COMPLETE) {
      zkPrintf("************** encountered parse complete\n");
    } else if (status == PARSE_HEADER_COMPLETE) {
      zkPrintf("************** encountered parse header complete\n");
    } else {
      zkPrintf("************** status is %d!!!\n",status);
    }

  if (!zkHTTPHeaderParse(err,header)) {
    zkPrintf("zkHTTPHEaderParse:  failed\n");
  }
  if (status == PARSE_COMPLETE) {
    zkPrintf("OUT OF THE LOOP, status is parse complete\n");
  } else if (status == PARSE_HEADER_COMPLETE) {
    zkPrintf("OUT OF THE LOOP, status is parse header complete\n");
  } else {
    zkPrintf("OUT OF THE LOOP, status is %d!!!\n",status);
  }
  if (!zkHTTPHeaderForceReferer(err,header,"WWW.FREEDOM.NET")) {
    zkPrintf("zkHTTPHeaderForceReferer:  failed\n");
  }
  if (!zkHTTPHeaderDump(err,header,zkEmailWrite,stdout)) {
    exit(exit_code);
  } 
  
  if (!zkHTTPHeaderFree(err,&header)) {
    exit(exit_code);
  }

  exit_code = EXIT_SUCCESS;

  return (exit_code);
}
 

  

