/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkList.h"
#include "zkMIME.h"
#include "zkEmailUtil.h"
#include "zkShell.h"

#define LINE_SZ 256

int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkEmail email = NULL;
  ZkEmailHeader header = NULL;
  unsigned char line[LINE_SZ + 1]; 
  size_t bytes = 0;
  FILE * f = stdin;
  ZkList recipient = NULL;
  ZkList nymlist = NULL;
  char * addr = NULL;
  const char* progname;
  
  progname = zkShellGetProgramName(&argc, &argv);
  
  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  zkEmailNew(err,&email);
  zkEmailHeader(err,email,&header);

  zkListNew(err,&recipient);

  while (!feof(f)) {
    bytes = zkFread(line,1,LINE_SZ,f);
    line[bytes] = (char) 0;
    if (!zkEmailAddText(err,email,line,bytes)) {
      zkFprintf(stderr,"zkEmailAddText:  failed\n");
    }
  }
  if (!zkEmailParse(err,email)) {
    zkFprintf(stderr,"zkEmailParse:  failed\n");
  }
 
  if (!zkEmailUtilAddFieldToRecipientList(err,header,To,recipient)) {
    zkFprintf(stderr,"zkEmailUtilAddFieldToRecipientList:  failed\n");
  }

  if (!zkEmailUtilAddFieldToRecipientList(err,header,cc,recipient)) {
    zkFprintf(stderr,"zkEmailUtilAddFieldToRecipientList:  failed\n");
  }
  
  if (!zkEmailUtilSplitRecipientsByDomain(err,recipient,&nymlist,argv[1],
                                         FLAG_IGNORE_CASE)) {
    zkFprintf(stderr,"zkEmailUtilSplitRecipientsByDomain:  failed\n");
  }

  zkListMoveCursor(err,recipient,LIST_HEAD);
  while (zkListGet(err,recipient,(void **)&addr,LIST_FORWARDS)) {
    zkPrintf("RECIPIENT:  %s\n",addr);
  }
  zkListMoveCursor(err,nymlist,LIST_HEAD);
  while (zkListGet(err,nymlist,(void **)&addr,LIST_FORWARDS)) {
    zkPrintf("NYM:  %s\n",addr);
  }

  return (EXIT_SUCCESS);
}

