/*
 * $Id: part.c,v 1.2 2001/04/19 17:03:36 alainc Exp $
 */

static const char * module_id_ = "$Id: part.c,v 1.2 2001/04/19 17:03:36 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"
#include "zkIntTypes.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkString.h"
#include "zkError.h"
#include "zkFile.h"
#include "zkLog.h"
#include "zkMIME.h"
#include "zkEmailUtil.h"
#include "zkShell.h"


int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkEmail email = NULL;
  ZkEmailBodyPart bp = NULL;
  ZkEmailBody body = NULL;
  ZkLog log = NULL;
  char * buf = NULL;
  int i = 0;  /* Loop index */
  size_t sz = 0;
  const char* progname;
  
  progname = zkShellGetProgramName(&argc, &argv);

  (void) module_id_;

  if ((argc != 2) && (argc != 3))  {
    zkFprintf(stderr,"usage:  %s filename [ subtype ]\n\n",argv[0]);
    exit(0);
  }

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogSetProgname(log,argv[0]);
  zkLogEnable(log);
  zkEmailNew(err,&email);
  
  if (!zkFileLoad(err,argv[1],&buf,&sz)) {
    exit(0);
  }
  for (i = 0; i < (int) sz; i += 1024) {
    if (!zkEmailAddText(err,email,(unsigned char *) &(buf[i]),((sz - i)<1024)?(sz-i):1024)) {
      exit(0);
    }
  }
  if (!zkEmailParse(err,email)) {
    exit(0);
  }
  if (!zkEmailConvertToMIME(err,email)) {
    exit(0);
  }
  zkFree(buf);
  
  bp = NULL;
  while (zkEmailUtilGetMIMEPart(err,email,MIMEContentTypeText,(argc == 3) ? argv[2]: NULL,&bp)) {
    zkEmailBodyPartBody(err,bp,&body); 
    zkEmailBodyGetData(err,body,(unsigned char **) &buf,&sz);
    /*
     *   The buffer returned points into the ZkEmail object.  DO NOT FREE IT
     *   (or modify it for that matter)
     */
    if (buf) {
      zkFprintf(stderr,"-----------------------------------\n%s\n",buf);
    }
  }   
  
  return (EXIT_FAILURE);
}
