/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"
#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkFile.h"
#include "zkMIME.h"
#include "zkShell.h"

int 
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkEmail mail = NULL;
  char * buf = NULL;
  size_t bsz = 0;

  zkShellGetProgramName(&argc, &argv);

  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);
  zkEmailNew(err,&mail);

  if (argc != 2) {
    zkFprintf(stderr,
            "%s: You must specify exactly one file name as a parameter\n",
            argv[0]);
  } else if (!zkFileLoad(err,argv[1],&buf,&bsz)) {
    zkFprintf(stderr,"%s: zkFileLoad failed\n");
  } else if (!zkEmailAddText(err,mail,(const unsigned char *)buf,bsz)) {
    zkFprintf(stderr,"%s: zkEmailAddText failed\n");
  } else if (!zkEmailParse(err,mail)) {
    zkFprintf(stderr,"%s: zkEmailParse failed\n");
  } else if (!zkEmailDump(err,mail,zkEmailWrite,stdout)) {
    zkFprintf(stderr,"%s: zkEmailDump failed\n");
  } else if (!zkEmailFree(err,&mail)) {
    zkFprintf(stderr,"%s: zkEmailFree failed\n");
  }  

  return(0);
}
