/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkString.h"
#include "zkSysTypes.h"
#include "zkSysStat.h"
#include "zkIntTypes.h"
#include "zkMIME.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkShell.h"

#define __LINE_SZ  1024
#define _BUFSZ 1024


#define ADVERT "<center><img src=\"http://www.zks.net/logo.gif\"></center>\
This Mail is brought to you by <a href=\"http://www.zks.net\">Zero-Knowledge\
 Systems</a><br>"

int foo(void* ptr);
int zkWriteEmail( ZkError err, void * cdata, const unsigned char * data, int dlen );

/*
 * This is a simple callback which will dump the email object to an output
 * stream such as stdout or stderr.
 */

int
foo (
  void * ptr )
{
  /* silence compiler */
  (void) ptr;
  
  return 0;
}

int
zkWriteEmail (
  ZkError err ,
  void * cdata ,
  const unsigned char * data ,
  int dlen )
{
  zkErrorClear(err);
  if (cdata && data) {
    zkFwrite(data,dlen,1,(FILE *)cdata);
    zkFflush((FILE *)cdata);
  }
  return 1;
}

int
main (
  int argc ,
  char ** argv )
{
  int i = 0;
  FILE * f = NULL;
  ZkEmail email = NULL;
  ZkEmailField field = NULL;
  ZkEmailBodyPart part = NULL;
  ZkEmailHeader part_header = NULL;
  ZkEmailBody part_body = NULL;
  ZkEmailHeader header = NULL;
  ZkEmailBody body = NULL;
  ZkError err = NULL;
  ZkLog log = NULL;
  ZkEmail email_copy = NULL;
  struct stat sbuf;
  unsigned char * buffer = NULL;
  int len = 0;
  ZkEmailField ctype = NULL;
  char * bound = NULL;
  int status = 0;
  const char * progname;
  
  progname = zkShellGetProgramName(&argc, &argv);
  
  zkErrorNew(&err);
  zkLogNew(&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);
  zkLogSetProgname(log,argv[0]);
  zkEmailNew(err,&email);

#if 1
  zkEmailScanForSMTPTermination(err,email);
#endif

/*
 * Read the email file into a buffer 
 */

  if(argc == 1)
  {
	zkPrintf("invalid number of arguments\n");
	return(0);
  }
  zkStat(argv[1],&sbuf);
  buffer = (unsigned char *)zkMalloc(sbuf.st_size + _BUFSZ);
  f = zkFopen(argv[1],"r");
  if (f == NULL) {
    perror(argv[1]);
    goto finished;
  }
  zkFread(buffer,sbuf.st_size,1,f);
  buffer[sbuf.st_size] = (char) 0;
  zkFclose(f);

/*
 * As a test, insert the text 5 characters at a time into the email object
 */

  len = strlen((char *) buffer);
  for (i = 0; i < len; i+=_BUFSZ) {
    if (!zkEmailAddText(err,email,&(buffer[i]), ((len -i)<_BUFSZ?(len-i):_BUFSZ))){
      zkLogError(log,err);
    }
    zkEmailStatus(err,email,&status);
    if (status == PARSE_COMPLETE) {
      zkPrintf("DONE DOOD\n");
    }
  }
  
  { unsigned char * data;
    size_t data_sz = 0;
    if (!zkEmailGetMessageData(err,email,&data,&data_sz)) {
      zkPrintf("unable to get the data\n");
    } else {
      zkFprintf(stdout,"-----------------------\n%s\n---------------------\n",data);
    }
  }

  if (!zkEmailParse(err,email)) {
    zkLogError(log,err);
  }
  zkEmailHeader(err,email,&header);
/*
  zkEmailHeaderFormatForInews(err,header);
*/
  if (!zkEmailHeaderDeleteFieldById(err,header,To)) {
    zkLogError(log,err);
  }
#if 0
  zkEmailStripHeader(err,email,DONT_STRIP_NEWSGROUPS);
#endif
  if (!zkEmailDump(err,email,zkWriteEmail,stdout)) {
    zkLogError(log,err);
  }


  {
    ZkEmailHeader h2 = NULL;
    zkEmailHeaderNew(err,&h2);
    zkEmailHeaderAddFieldData(err,h2,(unsigned char *) "MIME-Version",(unsigned char *) "1.0");
    zkEmailHeaderAddFieldData(err,h2,(unsigned char *) "X-Foobar",(unsigned char *) "This is a test");
    zkEmailHeaderMerge(err,header,h2);
    zkEmailHeaderDump(err,header,zkWriteEmail,stdout);
    zkEmailHeaderFree(err,&h2);
    zkEmailHeaderDump(err,header,zkWriteEmail,stdout);
  }

#if 0
  if (!zkEmailDup(err,email,&email_copy)) {
    zkLogError(log,err);
  }
  if (!zkEmailFree(err,&email)) {
    zkLogError(log,err);
  }
  email = email_copy;
#endif
/*
 * Now print out the mail using the dump function zkWriteEmail() (above)
 */

#if 0
  if (!zkEmailBodyParse(err,body)) {
    zkLogError(log,err);
  }
#endif
#if 0
  if (zkEmailIsMIME(err,email)) {
    zkPrintf("Mail is in MIME format\n");
  } else {
    zkPrintf("Mail is RFC822\n");
  }
  zkLogError(log,err);
#endif

  zkEmailHeader(err,email,&header);
#if 0
  if (zkEmailHeaderGetField(err,header,Content_Type,&ctype)) {
    if (zkEmailFieldGetParameter(err,ctype,"boundary",&bound)) {
      zkPrintf("MIME BOUNDARY IS <%s>\n",bound);
    }
  }
  zkLogError(log,err);
#endif

#if 1
  if (!zkEmailConvertToMIME(err,email)) {
    zkLogError(log,err);
  }
  if (!zkEmailHeaderAddFieldData(err,header,(unsigned char *) "X-Freedom-remailer",
     (unsigned char *) "Converted to MIME format by the ZKMIME library version 1.0")) {
    zkLogError(log,err);
  }

  
  /*
   * Create a little mime part and add it to the message 
   */

zkFprintf(stderr,"******************************************\n");
  if (!zkEmailAddTextBlipvert(err,email,1,(unsigned char *) ADVERT,strlen(ADVERT))) {
    return 0;
  }
#endif
#if 0
  if (!zkEmailBodyPartNew(err,&part)) {
    zkLogError(log,err);
  }
  if (!zkEmailBodyPartHeader(err,part,&part_header)) {
    zkLogError(log,err);
  }
  if (!zkEmailBodyPartBody(err,part,&part_body)) {
    zkLogError(log,err);
  }
  if (!zkEmailFieldNew(err,&field)) {
    zkLogError(log,err);
  }
  zkEmailFieldSetId(err,field,Content_Type);
  zkEmailFieldSetValue(err,field,"text/html");
  zkEmailBodyAddText(err,part_body,ADVERT,strlen(ADVERT));
  zkEmailBodyParse(err,part_body);
  zkEmailHeaderAddField(err,part_header,field);
  zkEmailBody(err,email,&body);
  zkEmailBodyAddPart(err,body,part);
#endif


#if 0
  if (!zkEmailParse(err,email)) {
    zkLogError(log,err);
  }
#endif
#if 1
  if (!zkEmailDump(err,email,zkWriteEmail,stdout)) {
    zkLogError(log,err);
  }

#endif
 
  
/*
 * Cleanup 
 */

finished:
  zkEmailFree(err,&email);
  zkErrorFree(&err);
  zkLogFree(&log);
  
  return (EXIT_SUCCESS);
}
  
  


