/* $Id: urltest.c,v 1.3 2001/04/19 17:03:36 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkNetinetIn.h"
#include <assert.h>
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkList.h"
#include "zkDebug.h"
#include "zkURL.h"
#include "zkShell.h"

extern void usage (const char *);

void usage (const char * progname )
{
  zkDebug_va("%s --url url", progname);
  exit (0);
}

int main(int argc, char **argv)
{

  ZkError err;
  ZkLog	log;

  const char * progname = NULL;
  char * url_s = NULL; /* The url to resolve */
  char * str;
  uint32_t len;
  
  int opt = 1;

#ifdef WIN32
  WSADATA	stWSAData;
  WORD wVersionRequired = MAKEWORD(2, 0);
  assert(0 == WSAStartup(wVersionRequired, &stWSAData));
#endif

  zkErrorNew(&err);
  zkErrorLog (err, &log);
  zkLogSetStream (log, stdout);
  zkLogEnable (log);

#ifndef WIN32
  zkLogWrite (log, "zkURLGet");
#endif

  progname = zkShellGetProgramName(&argc, &argv);

  while (opt < argc) {
    if (strcmp(argv[opt],"--url") == 0) {
      if (++opt < argc) {
        url_s = argv[opt]; 
      } else {
        zkDebug_va("You must specify a target URL");
	return 0;
      }

    } else {
      zkDebug_va("Unrecognized option: [%s]", argv[opt]);
      return 0;
    }
    opt++;
  }

  if (!url_s) usage(progname);
  
  assert (zkURLGetBody (err, url_s, &str, &len));
  zkDebug_va ("Length of message: %d", len);
  zkDebug_va ("Message:\n%s\n", str);
  zkFree(str);

  assert(zkErrorFree(&err));
#ifdef WIN32
  WSACleanup();
#endif

  return 0;
}
     


