/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkUnistd.h"
#include "zkIntTypes.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkMIME.h"
#include "zkShell.h"

#define TEST2

int
main (
  int argc ,
  char ** argv )
{
  ZkError err = NULL;
  ZkEmail email = NULL;
  ZkLog log = NULL;
  int status  = 0;
  const char* progname;
  
  progname = zkShellGetProgramName(&argc, &argv);
  
  zkErrorNew(&err);
  zkErrorLog(err,&log);
  zkLogSetStream(log,stderr);
  zkLogEnable(log);

  if (!zkEmailNew(err,&email)) {
    zkFprintf(stderr,"zkEmailNew: failed\n");
  }
  if (!zkEmailScanForSMTPTermination(err,email)) {
    zkFprintf(stderr,"zkEmailScanForSMTPTermination:  failed\n");
  }
#if defined(TEST1)
  if (!zkEmailAddText(err,email, (unsigned char *) "X",1)) {
#elif defined(TEST2)
  if (!zkEmailAddText(err,email, (unsigned char *) "\015\012.\015\012",5)) {
#elif defined(TEST3)
  zkEmailAddText(err,email, (unsigned char *) "foobar\015\012",8);
  if (!zkEmailAddText(err,email,".\015\012",3)) {
#elif defined(TEST4)
  zkEmailAddText(err,email, (unsigned char *) "\015\015\012",8);
  if (!zkEmailAddText(err,email, (unsigned char *) ".\015\012",3)) {
#endif /* TESTS */
    zkFprintf(stderr,"zkEmailAddText:  failed\n");
  }
  if (!zkEmailStatus(err,email,&status)) {
    zkFprintf(stderr,"zkEmailStatus:  failed\n");
  }
  zkFprintf(stderr,"Email status is ");
  switch (status) {
  case PARSE_HEADER:
    zkFprintf(stderr,"header"); break;
  case PARSE_BODY: 
    zkFprintf(stderr,"body"); break;
  case PARSE_ERROR:
    zkFprintf(stderr,"error"); break;
  case PARSE_COMPLETE:
    zkFprintf(stderr,"complete"); break;
  default:
    zkFprintf(stderr,"unknown %d",status); break;
  }
  if (!zkEmailFree(err,&email)) {
    zkFprintf(stderr,"zkEmailFree:  failed\n");
  }
  return 0;
}
