/*
 * $Id: zkPacket.h,v 1.28 2001/04/19 15:54:10 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKPACKET_H
#define	_ZKPACKET_H

#ifndef _ZKINTTYPES_H
#include "zkIntTypes.h"
#endif /* _ZKINTTYPES_H */

#include "zks.h"
#include "zkStdio.h"
#include "zkPackBuf.h"

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)
/*   #include	<DCon.h>		*/
void dopen(const char *log);
void dprintf(const char *format,...);
void vdprintf(const char *format,va_list arg);
void dprintmem(const void *data,size_t len);

void dfprintf(const char *log,const char *format,...);
void vdfprintf(const char *log,const char *format,va_list arg);
void dfprintmem(const char *log,const void *data,size_t len);
#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * These are the protocol ID numbers for the Protocol field of an IP header.
 */

#define ZK_IPPROTO_IP       0   /* Dummy protocol for TCP.  */
#define ZK_IPPROTO_ICMP     1   /* Internet Control Message Protocol.  */
#define ZK_IPPROTO_IGMP     2   /* Internet Group Management Protocol. */
#define ZK_IPPROTO_IPIP     4   /* IPIP tunnels (older KA9Q tunnels use 94).  */
#define ZK_IPPROTO_TCP      6   /* Transmission Control Protocol.  */
#define ZK_IPPROTO_EGP      8   /* Exterior Gateway Protocol.  */
#define ZK_IPPROTO_PUP     12   /* PUP protocol.  */
#define ZK_IPPROTO_UDP     17   /* User Datagram Protocol.  */
#define ZK_IPPROTO_IDP     22   /* XNS IDP protocol.  */
#define ZK_IPPROTO_IPV6    41   /* IPv6-in-IPv4 tunnelling.  */
#define ZK_IPPROTO_ICMPV6  58   /* ICMPv6.  */
#define ZK_IPPROTO_RAW    255   /* Raw IP packets.  */

/* ICMP message types */

#define ZK_ICMP_ECHOREPLY		0		/* Echo Reply */
#define ZK_ICMP_UNREACH			3		/* Destination Unreachable, (see message codes) below */
#define ZK_ICMP_SOURCEQUENCH	4		/* Source Quench */
#define ZK_ICMP_REDIRECT		5		/* Redirect (change route) */
#define ZK_ICMP_ECHO            8		/* Echo Request */
#define ZK_ICMP_ROUTERADVERT	9		/* Router advertisement */		
#define ZK_ICMP_TIMXCEED		11		/* Time Exceeded (see messages below) */
#define ZK_ICMP_PARAMPROB       12		/* Parameter Problem */
#define	ZK_ICMP_MAXTYPE         18		/* Maximum */


/*
 * ICMP message codes for unreachable destinations [Type == 3].  These are passed as values
 * for the code parameter of the zkICMPCreateDestinationUnreachable() 
 * function.
 */

#define ZK_ICMP_NETWORK_UNREACHABLE                     0	/* Network Unreachable */
#define ZK_ICMP_HOST_UNREACHABLE                        1	/* Host Unreachable */
#define ZK_ICMP_PROTOCOL_UNREACHABLE                    2	/* Protocol Unreachable */
#define ZK_ICMP_PORT_UNREACHABLE                        3	/* Port Unreachable */
#define ZK_ICMP_FRAGMENTATION_NEEDED                    4	/* Fragmentation Needed/DF set */
#define ZK_ICMP_SOURCE_ROUTE_FAILED                     5	/* Source Route failed */
#define ZK_ICMP_DESTINATION_NETWORK_UNKNOWN             6	/* Destination Network Unknown */
#define ZK_ICMP_DESTINATION_HOST_UNKNOWN                7	/* Destination Host Unknown */
#define ZK_ICMP_SOURCE_HOST_ISOLATED                    8	/* Source Host Isolated */
#define ZK_ICMP_NETWORK_COMMUNICATION_PROHIBITED        9	/* Communication w/destination net administrativly prohibited */
#define ZK_ICMP_HOST_COMMUNICATION_PROHIBITED			10	/* Communication w/destination host administrativly prohibited */
#define ZK_ICMP_NETWORK_UNREACHABLE_FOR_TOS				11	/* Network Unreachable for Type-of-Service */
#define ZK_ICMP_HOST_UNREACHABLE_FOR_TOS				12	/* Host Unreachable for Type-of-Service */
#define ZK_ICMP_UNREACH_PKTFILT							13	/* Packet filtered */
#define ZK_ICMP_UNREACH_PRECVIO							14	/* Precedence violation */
#define ZK_ICMP_UNREACH_PRECCUT							15	/* Precedence cut off */
#define ZK_ICMP_UNREACH_MAX								15	/* Maximum */


/*
 * ICMP message codes for time exceeded packets [Type == 11].  These are passed as values
 * for the code parameter of the zkICMPCreateTimeExceeded() function.
 */

#define ZK_ICMP_TTL_COUNT_EXCEEDED                      0	/* TTL count exceeded	in transit */
#define ZK_ICMP_FRAGMENT_REASSEMBLY_TIME_EXCEEDED       1	/* Fragment Reassembly time exceeded */


/*
 * Defines for TCP Options
 */
#define ZK_TCPOPT_EOL			0
#define ZK_TCPOPT_NOP			1
#define ZK_TCPOPT_MSS			2
#define ZK_TCPOPT_WINSCALEFACT	3
#define ZK_TCPOPT_TIMESTAMP		8

#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
typedef struct ZkIPHeader ZkIPHeader;
typedef struct ZkICMPHeader ZkICMPHeader;
typedef struct ZkTCPHeader ZkTCPHeader;
typedef struct ZkUDPHeader ZkUDPHeader;
#endif

/* 20 bytes for the IP header */
#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
struct ZkIPHeader {
#ifdef ZK_LITTLE_ENDIAN
  uint8_t ihl:4;
  uint8_t version:4;
#elif ZK_BIG_ENDIAN
  uint8_t version:4;
  uint8_t ihl:4;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif
  uint8_t TypeOfSvr;
  uint16_t TotalLength;
  uint16_t Identification;
  uint16_t Flags_FrameOffs;
  uint8_t  TimeToLive;
  uint8_t  Protocol;
  uint16_t HeaderChkSum;
  uint32_t SrcIpAddress;
  uint32_t DstIpAddress;
};
#pragma pack()
#endif

enum {
	ZKIPHEADER_IHLVERSION		= 0,
	ZKIPHEADER_TYPEOFSVR		= ZKIPHEADER_IHLVERSION      + sizeof(uint8_t),
	ZKIPHEADER_TOTALLENGTH		= ZKIPHEADER_TYPEOFSVR       + sizeof(uint8_t),
	ZKIPHEADER_IDENTIFICATION	= ZKIPHEADER_TOTALLENGTH     + sizeof(uint16_t),
	ZKIPHEADER_FLAGS_FRAMEOFFS	= ZKIPHEADER_IDENTIFICATION  + sizeof(uint16_t),
	ZKIPHEADER_TIMETOLIVE		= ZKIPHEADER_FLAGS_FRAMEOFFS + sizeof(uint16_t),
	ZKIPHEADER_PROTOCOL			= ZKIPHEADER_TIMETOLIVE      + sizeof(uint8_t),
	ZKIPHEADER_HEADERCHKSUM		= ZKIPHEADER_PROTOCOL        + sizeof(uint8_t),
	ZKIPHEADER_SRCIPADDRESS		= ZKIPHEADER_HEADERCHKSUM    + sizeof(uint16_t),
	ZKIPHEADER_DSTIPADDRESS		= ZKIPHEADER_SRCIPADDRESS    + sizeof(uint32_t),
	ZKIPHEADER_SIZEOF			= ZKIPHEADER_DSTIPADDRESS    + sizeof(uint32_t),
	
	/* bitfield access */
	
	ZKIPHEADER_VERSION_BIT		= 0,
	ZKIPHEADER_VERSION_WIDTH	= 4,
	ZKIPHEADER_IHL_BIT			= ZKIPHEADER_VERSION_BIT + ZKIPHEADER_VERSION_WIDTH,
	ZKIPHEADER_IHL_WIDTH		= 4,
	
	ZKIPHEADER_FLAGS_BIT		= 0,
	ZKIPHEADER_FLAGS_WIDTH		= 3,
	ZKIPHEADER_FRAMEOFFS_BIT	= ZKIPHEADER_FLAGS_BIT + ZKIPHEADER_FLAGS_WIDTH,
	ZKIPHEADER_FRAMEOFFS_WIDTH	= 13
};

/* 20 bytes for the TCP header unless options are present */
#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
struct ZkTCPHeader {
  char /* ZkIPHeader */ IP[ZKIPHEADER_SIZEOF];
  uint16_t SrcPort;
  uint16_t DstPort;
  uint32_t SequenceNumber;
  uint32_t AckNumber;
#ifdef ZK_LITTLE_ENDIAN
  uint16_t res1:4;
  uint16_t doff:4;
  uint16_t fin:1;
  uint16_t syn:1;
  uint16_t rst:1;
  uint16_t psh:1;
  uint16_t ack:1;
  uint16_t urg:1;
  uint16_t res2:2;
#elif ZK_BIG_ENDIAN
  uint16_t doff:4;
  uint16_t res1:4;
  uint16_t res2:2;
  uint16_t urg:1;
  uint16_t ack:1;
  uint16_t psh:1;
  uint16_t rst:1;
  uint16_t syn:1;
  uint16_t fin:1;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif 
  uint16_t Window;
  uint16_t TcpCheckSum;
  uint16_t UrgentPointer;
};
#pragma pack()
#endif

enum {
	ZKTCPHEADER_IP				= 0,
	ZKTCPHEADER_SRCPORT			= ZKTCPHEADER_IP             + ZKIPHEADER_SIZEOF,
	ZKTCPHEADER_DSTPORT			= ZKTCPHEADER_SRCPORT        + sizeof(uint16_t),
	ZKTCPHEADER_SEQUENCENUMBER	= ZKTCPHEADER_DSTPORT        + sizeof(uint16_t),
	ZKTCPHEADER_ACKNUMBER		= ZKTCPHEADER_SEQUENCENUMBER + sizeof(uint32_t),
	ZKTCPHEADER_DOFF_RES		= ZKTCPHEADER_ACKNUMBER      + sizeof(uint32_t),
	ZKTCPHEADER_FLAGS			= ZKTCPHEADER_DOFF_RES       + sizeof(uint8_t),
	ZKTCPHEADER_WINDOW			= ZKTCPHEADER_FLAGS          + sizeof(uint8_t),
	ZKTCPHEADER_TCPCHECKSUM		= ZKTCPHEADER_WINDOW         + sizeof(uint16_t),
	ZKTCPHEADER_URGENTPOINTER	= ZKTCPHEADER_TCPCHECKSUM    + sizeof(uint16_t),
	ZKTCPHEADER_SIZEOF			= ZKTCPHEADER_URGENTPOINTER  + sizeof(uint16_t),
	
	/* bitfield access */
	
	ZKTCPHEADER_DOFF_BIT	= 0,
	ZKTCPHEADER_DOFF_WIDTH	= 4,
	ZKTCPHEADER_RES_BIT		= ZKTCPHEADER_DOFF_BIT + ZKTCPHEADER_DOFF_WIDTH,
	ZKTCPHEADER_RES_WIDTH	= 6,
	
	ZKTCPHEADER_URG_BIT		= 2,
	ZKTCPHEADER_ACK_BIT		= ZKTCPHEADER_URG_BIT + 1,
	ZKTCPHEADER_PSH_BIT		= ZKTCPHEADER_ACK_BIT + 1,
	ZKTCPHEADER_RST_BIT		= ZKTCPHEADER_PSH_BIT + 1,
	ZKTCPHEADER_SYN_BIT		= ZKTCPHEADER_RST_BIT + 1,
	ZKTCPHEADER_FIN_BIT		= ZKTCPHEADER_SYN_BIT + 1
};

/* 8 bytes for the UDP header */
#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
struct ZkUDPHeader {
  char /* ZkIPHeader */ IP[ZKIPHEADER_SIZEOF];
  uint16_t SrcPort;
  uint16_t DstPort;
  uint16_t UdpLength;
  uint16_t UdpCheckSum;
};
#pragma pack()
#endif

enum {
	ZKUDPHEADER_IP			= 0,
	ZKUDPHEADER_SRCPORT		= ZKUDPHEADER_IP          + ZKIPHEADER_SIZEOF,
	ZKUDPHEADER_DSTPORT		= ZKUDPHEADER_SRCPORT     + sizeof(uint16_t),
	ZKUDPHEADER_UDPLENGTH	= ZKUDPHEADER_DSTPORT     + sizeof(uint16_t),
	ZKUDPHEADER_UDPCHECKSUM	= ZKUDPHEADER_UDPLENGTH   + sizeof(uint16_t),
	ZKUDPHEADER_SIZEOF		= ZKUDPHEADER_UDPCHECKSUM + sizeof(uint16_t)
};

#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
struct ZkICMPHeader {
  uint8_t Type;
  uint8_t code;
  uint16_t CheckSum;  
  union
	  {
	  struct ih_id_seq {
		uint16_t	icd_id;
		uint16_t	icd_seq;
	  } ih_id_seq;
	  struct mtu {       /* RFC1191, section 4 -- MTU discovery */
		uint16_t mtu_void;
		uint16_t NextHop_MTU;
	  } mtu;
  } icmp_payload;

#define icmp_NextHop_MTU		icmp_payload.mtu.NextHop_MTU
#define icmp_Identifier			icmp_payload.ih_id_seq.icd_id
#define icmp_SequenceNumber		icmp_payload.ih_id_seq.icd_seq
};
#pragma pack()
#endif

enum {
	ZKICMPHEADER_TYPE			= 0,
	ZKICMPHEADER_CODE			= ZKICMPHEADER_TYPE     + sizeof(uint8_t),
	ZKICMPHEADER_CHECKSUM		= ZKICMPHEADER_CODE     + sizeof(uint8_t),
	
	ZKICMPHEADER_IH_ID			= ZKICMPHEADER_CHECKSUM + sizeof(uint16_t),
	ZKICMPHEADER_IH_SEQ			= ZKICMPHEADER_IH_ID	+ sizeof(uint16_t),
	
	ZKICMPHEADER_MTU_VOID		= ZKICMPHEADER_CHECKSUM + sizeof(uint16_t),
	ZKICMPHEADER_MTU_NEXTHOP	= ZKICMPHEADER_MTU_VOID + sizeof(uint16_t),
	
	ZKICMPHEADER_SIZEOF			= ZKICMPHEADER_IH_SEQ   + sizeof(uint16_t),
	
	/* backward compatibility */
	ZKICMPHEADER_IDENTIFIER		= ZKICMPHEADER_IH_ID,
	ZKICMPHEADER_SEQUENCENUMBER	= ZKICMPHEADER_IH_SEQ
};

#ifndef WIN32
#define ZKBUFFER_LEN	4096
#else
#define ZKBUFFER_LEN	1500
#endif

#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
struct ZkBuffer{
	union{
		char /* ZkTCPHeader */	Tcp[ZKTCPHEADER_SIZEOF];	/* use ZkBuffer.Tcp.IP.Protocol to determine which union member is valid */
		char /* ZkUDPHeader */	Udp[ZKUDPHEADER_SIZEOF];
	} pkt;
	unsigned char Data[ZKBUFFER_LEN];
};
#pragma pack()
#endif

enum {
	ZKBUFFER_TCP		= 0,
	ZKBUFFER_UDP		= 0,
	ZKBUFFER_DATA		= ZKBUFFER_TCP  + ZKTCPHEADER_SIZEOF,
	ZKBUFFER_SIZEOF		= ZKBUFFER_DATA + ZKBUFFER_LEN
};

/* There's bit too much code in the client that relies on the existence of 
ZkBuffer, so I've reinstated it, but with a declaration that minimises the risk 
of alignment problems -- paull */

struct ZkBuffer {
	char x[ZKBUFFER_SIZEOF];
};

#define ZKDIXHEADER_DSTADDRESS_SIZE	6
#define ZKDIXHEADER_SRCADDRESS_SIZE	6
#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
typedef struct ZkDIXHeader {
  uint8_t	DstAddress[ZKDIXHEADER_DSTADDRESS_SIZE];
  uint8_t	SrcAddress[ZKDIXHEADER_SRCADDRESS_SIZE];
  uint16_t	Type;
} ZkDIXHeader;
#pragma pack()
#endif

enum {
	ZKDIXHEADER_DSTADDRESS	= 0,
	ZKDIXHEADER_SRCADDRESS	= ZKDIXHEADER_DSTADDRESS + ZKDIXHEADER_DSTADDRESS_SIZE,
	ZKDIXHEADER_TYPE		= ZKDIXHEADER_SRCADDRESS + ZKDIXHEADER_SRCADDRESS_SIZE,
	ZKDIXHEADER_SIZEOF		= ZKDIXHEADER_TYPE       + sizeof(uint16_t)
};

#if 0	/* NOTE: Replacing packed struct with calls to zkPackBuf */
#pragma pack(1)
typedef struct ZkNetFrame {
	ZkDIXHeader	DIX;	
	union{
		char /* ZkTCPHeader */	TCP[ZKTCPHEADER_SIZEOF];	/* use ZkNetFrame.TCP.IP.Protocol to determine which union member is valid */
		char /* ZkUDPHeader */	UDP[ZKUDPHEADER_SIZEOF];	/* this has the ZkIPHeader structure within it */
		char /* ZkICMPHeader */	ICMP[ZKICMPHEADER_SIZEOF];  /* this does not have the ZkIPHeader structure within it */
	} frm;
	unsigned char Data[ZKBUFFER_LEN];
} ZkNetFrame;
#pragma pack()
#endif

enum {
	ZKNETFRAME_DIX			= 0,
	ZKNETFRAME_FRAME		= ZKNETFRAME_DIX   + ZKDIXHEADER_SIZEOF,
	ZKNETFRAME_FRAME_TCP	= ZKNETFRAME_FRAME,
	ZKNETFRAME_FRAME_UDP	= ZKNETFRAME_FRAME,
	ZKNETFRAME_FRAME_ICMP	= ZKNETFRAME_FRAME,
	ZKNETFRAME_DATA			= ZKNETFRAME_FRAME + ZKTCPHEADER_SIZEOF,
	ZKNETFRAME_SIZEOF		= ZKNETFRAME_DATA  + ZKBUFFER_LEN
};

/* There's bit too much code in the client that relies on the existence of 
ZkBuffer, so I've reinstated it, but with a declaration that minimises the risk 
of alignment problems -- paull */

struct ZkNetFrame {
	char x[ZKNETFRAME_SIZEOF];
};


DLLExport uint16_t zkTCPHeaderChecksum (
  void * );

DLLExport uint16_t zkUDPHeaderChecksum (
  void * ,
  size_t );

/*
 * IP Header manipulation functions
 */

DLLExport uint16_t zkIPHeaderChecksum (
  void * );

#if !defined(ZK_KERNEL)

DLLExport int zkIPHeaderDump (
  void * ,                        /* IP Header to dump */
  FILE * );                             /* destination file (eg stdout) */

#endif	/* !defined(ZK_KERNEL) */

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)

DLLExport int zkIPHeaderDump (
  void * );                             /* destination file (eg stdout) */

#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

DLLExport int zkIPHeaderSourceAddress (
  void * ,
  uint32_t * );                          /* Returns source IP address */

DLLExport int zkIPHeaderDestinationAddress (
  void * ,
  uint32_t * );                          /* Returns destination IP address */

DLLExport int zkIPHeaderSetSourceAddress (
  void * ,
  uint32_t );                            /* source IP address */

DLLExport int zkIPHeaderSetDestinationAddress (
  void * ,
  uint32_t );                            /* destination IP address */

DLLExport int zkIPHeaderStripOptions (
  void * );                       /* IP header to strip */

/*
 * ICMP header functions 
 */

DLLExport uint16_t zkICMPHeaderChecksum (
  void * ,            /* Pointer to the ICMP header */
  size_t );                   /* Number of bytes over which to compute cksum */

DLLExport int zkICMPCreateDestinationUnreachable (
  void * ,              /* Original (offending) IP packet */
  uint8_t ,                   /* Reason code (see list above) */
  unsigned char * ,           /* Buffer to hold the icmp packet */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateFragmentationNeeded (
  void * ,              /* Original (offending) IP packet */
  unsigned char * ,           /* Buffer to hold ICMP response (128 bytes) */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateSourceQuench (
  void * ,              /* Original (offending) IP packet */
  unsigned char * ,           /* Buffer to hold ICMP response (128 bytes) */
  size_t * );                 /* original size of buffer, returns new size */

DLLExport int zkICMPCreateTimeExceeded (
  void * ,              /* Original (offending) IP packet */
  uint8_t ,                   /* Reason code (see list above) */
  unsigned char * ,           /* Buffer to hold the icmp packet */
  size_t * );                 /* original size of buffer, returns new size */


/*
 * TCP Header manipulation functions
 */

#if !defined(ZK_KERNEL)

DLLExport int zkTCPHeaderDump (
  void * ,                       /* TCP packet to dump */
  FILE * );                             /* FILE to dump it to (eg stdout) */

#endif	/* !defined(ZK_KERNEL) */

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)

DLLExport int zkTCPHeaderDump (
  void * );                             /* FILE to dump it to (eg stdout) */

#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

DLLExport int zkTCPHeaderAcknowledgement (
  void * ,                       /* TCP header */
  uint32_t * );                         /* Returns the acknowledgement number */

DLLExport int zkTCPHeaderSequence (
  void * ,                       /* TCP header */
  uint32_t * );                         /* Return the sequence number */

DLLExport uint16_t zkTCPHeaderSourcePort (
  void * ,                       /* TCP Header */
  uint16_t * );                         /* Returns the TCP source port */

DLLExport uint16_t zkTCPHeaderDestinationPort (
  void * ,                       /* TCP Header */
  uint16_t * );                         /* Returns the TCP destination port */

DLLExport int zkTCPHeaderSetSourcePort (
  void * ,                       /* TCP Header */
  uint16_t );                           /* TCP source port */

DLLExport int zkTCPHeaderSetDestinationPort (
  void * ,                       /* TCP Header */
  uint16_t );                           /* TCP destination port */

DLLExport int zkTCPHeaderGetMSS (
  void * ,                       /* TCP header */
  uint16_t * );                         /* Returns the Maximum Segment Size */


DLLExport int zkTCPHeaderSetMSS (
  void * ,                       /* TCP Header */
  uint16_t );                           /* Maximum Segment Size */

DLLExport int zkTCPHeaderStripOptions (
  void * );                      /* TCP Header */

DLLExport int zkTCPHeaderModifyMSS (
  void * ,                       /* TCP Header */
  const uint16_t );


/*
 * UDP header manipulation functions
 */

#if !defined(ZK_KERNEL)

DLLExport int zkUDPHeaderDump (
  void * ,
  FILE * );

#endif	/* !defined(ZK_KERNEL) */

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)

DLLExport int zkUDPHeaderDump (
  void * );

#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

DLLExport uint16_t zkUDPHeaderSourcePort (
  void * ,                       /* UDP Header */
  uint16_t * );                         /* Returns the UDP source port */

DLLExport uint16_t zkUDPHeaderDestinationPort (
  void * ,                       /* UDP Header */
  uint16_t * );                         /* Returns the UDP destination port */

DLLExport int zkUDPHeaderSetSourcePort (
  void * ,                       /* UDP Header */
  uint16_t );                           /* UDP source port */

DLLExport int zkUDPHeaderSetDestinationPort (
  void * ,                       /* UDP Header */
  uint16_t );                           /* UDP destination port */


/*
 *	IP header field manipulation
 */

#define zkIPHeaderGetIHL(buf)					zkPackBufGetFieldNbo((buf),  ZKIPHEADER_IHLVERSION, ZKIPHEADER_IHL_BIT, ZKIPHEADER_IHL_WIDTH)
#define zkIPHeaderGetVersion(buf)				zkPackBufGetFieldNbo((buf),  ZKIPHEADER_IHLVERSION, ZKIPHEADER_VERSION_BIT, ZKIPHEADER_VERSION_WIDTH)
#define zkIPHeaderGetTypeOfSvr(buf)				zkPackBufGetUint8((buf),     ZKIPHEADER_TYPEOFSVR)
#define zkIPHeaderGetTotalLength(buf)			zkPackBufGetUint16Nbo((buf), ZKIPHEADER_TOTALLENGTH)
#define zkIPHeaderGetIdentification(buf)		zkPackBufGetUint16Nbo((buf), ZKIPHEADER_IDENTIFICATION)
#define zkIPHeaderGetFlags_FrameOffs(buf)		zkPackBufGetUint16Nbo((buf), ZKIPHEADER_FLAGS_FRAMEOFFS)
#define zkIPHeaderGetFlags_FrameOffsNBO(buf)	zkPackBufGetUint16((buf),    ZKIPHEADER_FLAGS_FRAMEOFFS)
#define zkIPHeaderGetTimeToLive(buf)			zkPackBufGetUint8((buf),     ZKIPHEADER_TIMETOLIVE)
#define zkIPHeaderGetProtocol(buf)				zkPackBufGetUint8((buf),     ZKIPHEADER_PROTOCOL)
#define zkIPHeaderGetHeaderChkSum(buf)			zkPackBufGetUint16Nbo((buf), ZKIPHEADER_HEADERCHKSUM)
#define zkIPHeaderGetHeaderChkSumNBO(buf)		zkPackBufGetUint16((buf),    ZKIPHEADER_HEADERCHKSUM)
#define zkIPHeaderGetSrcIpAddress(buf)			zkPackBufGetUint32Nbo((buf), ZKIPHEADER_SRCIPADDRESS)
#define zkIPHeaderGetSrcIpAddressNBO(buf)		zkPackBufGetUint32((buf),    ZKIPHEADER_SRCIPADDRESS)
#define zkIPHeaderGetDstIpAddress(buf)			zkPackBufGetUint32Nbo((buf), ZKIPHEADER_DSTIPADDRESS)
#define zkIPHeaderGetDstIpAddressNBO(buf)		zkPackBufGetUint32((buf),    ZKIPHEADER_DSTIPADDRESS)

#define zkIPHeaderSetIHL(buf, val)				zkPackBufSetFieldNbo((buf),  ZKIPHEADER_IHLVERSION, ZKIPHEADER_IHL_BIT, ZKIPHEADER_IHL_WIDTH, (val))
#define zkIPHeaderSetVersion(buf, val)			zkPackBufSetFieldNbo((buf),  ZKIPHEADER_IHLVERSION, ZKIPHEADER_VERSION_BIT, ZKIPHEADER_VERSION_WIDTH, (val))
#define zkIPHeaderSetTypeOfSvr(buf, val)		zkPackBufSetUint8((buf),     ZKIPHEADER_TYPEOFSVR, (val))
#define zkIPHeaderSetTotalLength(buf, val)		zkPackBufSetUint16Nbo((buf), ZKIPHEADER_TOTALLENGTH, (val))
#define zkIPHeaderSetIdentification(buf, val)	zkPackBufSetUint16Nbo((buf), ZKIPHEADER_IDENTIFICATION, (val))
#define zkIPHeaderSetFlags_FrameOffs(buf, val)	zkPackBufSetUint16Nbo((buf), ZKIPHEADER_FLAGS_FRAMEOFFS, (val))
#define zkIPHeaderSetTimeToLive(buf, val)		zkPackBufSetUint8((buf),     ZKIPHEADER_TIMETOLIVE, (val))
#define zkIPHeaderSetProtocol(buf, val)			zkPackBufSetUint8((buf),     ZKIPHEADER_PROTOCOL, (val))
#define zkIPHeaderSetHeaderChkSum(buf, val)		zkPackBufSetUint16Nbo((buf), ZKIPHEADER_HEADERCHKSUM, (val))
#define zkIPHeaderSetHeaderChkSumNBO(buf, val)	zkPackBufSetUint16((buf),    ZKIPHEADER_HEADERCHKSUM, (val))
#define zkIPHeaderSetSrcIpAddress(buf, val)		zkPackBufSetUint32Nbo((buf), ZKIPHEADER_SRCIPADDRESS, (val))
#define zkIPHeaderSetSrcIpAddressNBO(buf, val)	zkPackBufSetUint32((buf),    ZKIPHEADER_SRCIPADDRESS, (val))
#define zkIPHeaderSetDstIpAddress(buf, val)		zkPackBufSetUint32Nbo((buf), ZKIPHEADER_DSTIPADDRESS, (val))
#define zkIPHeaderSetDstIpAddressNBO(buf, val)	zkPackBufSetUint32((buf),    ZKIPHEADER_DSTIPADDRESS, (val))


/*
 *	TCP header field manipulation
 */

#define zkTCPHeaderGetSrcPort(buf)				zkPackBufGetUint16Nbo((buf), ZKTCPHEADER_SRCPORT)
#define zkTCPHeaderGetSrcPortNBO(buf)			zkPackBufGetUint16((buf),    ZKTCPHEADER_SRCPORT)
#define zkTCPHeaderGetDstPort(buf)				zkPackBufGetUint16Nbo((buf), ZKTCPHEADER_DSTPORT)
#define zkTCPHeaderGetDstPortNBO(buf)			zkPackBufGetUint16((buf),    ZKTCPHEADER_DSTPORT)
#define zkTCPHeaderGetSequenceNumber(buf)		zkPackBufGetUint32Nbo((buf), ZKTCPHEADER_SEQUENCENUMBER)
#define zkTCPHeaderGetAckNumber(buf)			zkPackBufGetUint32Nbo((buf), ZKTCPHEADER_ACKNUMBER)
#define zkTCPHeaderGetDoff(buf)					zkPackBufGetFieldNbo((buf),  ZKTCPHEADER_DOFF_RES, ZKTCPHEADER_DOFF_BIT, ZKTCPHEADER_DOFF_WIDTH)
#define zkTCPHeaderGetRes(buf)					zkPackBufGetFieldNbo((buf),  ZKTCPHEADER_DOFF_RES, ZKTCPHEADER_RES_BIT, ZKTCPHEADER_RES_WIDTH)
#define zkTCPHeaderGetUrg(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_URG_BIT)
#define zkTCPHeaderGetAck(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_ACK_BIT)
#define zkTCPHeaderGetPsh(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_PSH_BIT)
#define zkTCPHeaderGetRst(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_RST_BIT)
#define zkTCPHeaderGetSyn(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_SYN_BIT)
#define zkTCPHeaderGetFin(buf)					zkPackBufGetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_FIN_BIT)
#define zkTCPHeaderGetWindow(buf)				zkPackBufGetUint16Nbo((buf), ZKTCPHEADER_WINDOW)
#define zkTCPHeaderGetTcpCheckSum(buf)			zkPackBufGetUint16Nbo((buf), ZKTCPHEADER_TCPCHECKSUM)
#define zkTCPHeaderGetTcpCheckSumNBO(buf)		zkPackBufGetUint16((buf),    ZKTCPHEADER_TCPCHECKSUM)
#define zkTCPHeaderGetUrgentPointer(buf)		zkPackBufGetUint16Nbo((buf), ZKTCPHEADER_URGENTPOINTER)

#define zkTCPHeaderSetSrcPort(buf, val)			zkPackBufSetUint16Nbo((buf), ZKTCPHEADER_SRCPORT, (val))
#define zkTCPHeaderSetSrcPortNBO(buf, val)		zkPackBufSetUint16((buf),    ZKTCPHEADER_SRCPORT, (val))
#define zkTCPHeaderSetDstPort(buf, val)			zkPackBufSetUint16Nbo((buf), ZKTCPHEADER_DSTPORT, (val))
#define zkTCPHeaderSetDstPortNBO(buf, val)		zkPackBufSetUint16((buf),    ZKTCPHEADER_DSTPORT, (val))
#define zkTCPHeaderSetSequenceNumber(buf, val)	zkPackBufSetUint32Nbo((buf), ZKTCPHEADER_SEQUENCENUMBER, (val))
#define zkTCPHeaderSetAckNumber(buf, val)		zkPackBufSetUint32Nbo((buf), ZKTCPHEADER_ACKNUMBER, (val))
#define zkTCPHeaderSetDoff(buf, val)			zkPackBufSetFieldNbo((buf),  ZKTCPHEADER_DOFF_RES, ZKTCPHEADER_DOFF_BIT, ZKTCPHEADER_DOFF_WIDTH, (val))
#define zkTCPHeaderSetRes(buf, val)				zkPackBufSetFieldNbo((buf),  ZKTCPHEADER_DOFF_RES, ZKTCPHEADER_RES_BIT, ZKTCPHEADER_RES_WIDTH, (val))
#define zkTCPHeaderSetUrg(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_URG_BIT, (val))
#define zkTCPHeaderSetAck(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_ACK_BIT, (val))
#define zkTCPHeaderSetPsh(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_PSH_BIT, (val))
#define zkTCPHeaderSetRst(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_RST_BIT, (val))
#define zkTCPHeaderSetSyn(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_SYN_BIT, (val))
#define zkTCPHeaderSetFin(buf, val)				zkPackBufSetBit((buf),       ZKTCPHEADER_FLAGS, ZKTCPHEADER_FIN_BIT, (val))
#define zkTCPHeaderSetWindow(buf, val)			zkPackBufSetUint16Nbo((buf), ZKTCPHEADER_WINDOW, (val))
#define zkTCPHeaderSetTcpCheckSum(buf, val)		zkPackBufSetUint16Nbo((buf), ZKTCPHEADER_TCPCHECKSUM, (val))
#define zkTCPHeaderSetTcpCheckSumNBO(buf, val)	zkPackBufSetUint16((buf),    ZKTCPHEADER_TCPCHECKSUM, (val))
#define zkTCPHeaderSetUrgentPointer(buf, val)	zkPackBufSetUint16Nbo((buf), ZKTCPHEADER_URGENTPOINTER, (val))


/*
 *	UDP header field manipulation
 */

#define zkUDPHeaderGetSrcPort(buf)				zkPackBufGetUint16Nbo((buf), ZKUDPHEADER_SRCPORT)
#define zkUDPHeaderGetSrcPortNBO(buf)			zkPackBufGetUint16((buf),    ZKUDPHEADER_SRCPORT)
#define zkUDPHeaderGetDstPort(buf)				zkPackBufGetUint16Nbo((buf), ZKUDPHEADER_DSTPORT)
#define zkUDPHeaderGetDstPortNBO(buf)			zkPackBufGetUint16((buf),    ZKUDPHEADER_DSTPORT)
#define zkUDPHeaderGetUdpLength(buf)			zkPackBufGetUint16Nbo((buf), ZKUDPHEADER_UDPLENGTH)
#define zkUDPHeaderGetUdpCheckSum(buf)			zkPackBufGetUint16Nbo((buf), ZKUDPHEADER_UDPCHECKSUM)
#define zkUDPHeaderGetUdpCheckSumNBO(buf)		zkPackBufGetUint16((buf),    ZKUDPHEADER_UDPCHECKSUM)

#define zkUDPHeaderSetSrcPort(buf, val)			zkPackBufSetUint16Nbo((buf), ZKUDPHEADER_SRCPORT, (val))
#define zkUDPHeaderSetSrcPortNBO(buf, val)		zkPackBufSetUint16((buf),    ZKUDPHEADER_SRCPORT, (val))
#define zkUDPHeaderSetDstPort(buf, val)			zkPackBufSetUint16Nbo((buf), ZKUDPHEADER_DSTPORT, (val))
#define zkUDPHeaderSetDstPortNBO(buf, val)		zkPackBufSetUint16((buf),    ZKUDPHEADER_DSTPORT, (val))
#define zkUDPHeaderSetUdpLength(buf, val)		zkPackBufSetUint16Nbo((buf), ZKUDPHEADER_UDPLENGTH, (val))
#define zkUDPHeaderSetUdpCheckSum(buf, val)		zkPackBufSetUint16Nbo((buf), ZKUDPHEADER_UDPCHECKSUM, (val))
#define zkUDPHeaderSetUdpCheckSumNBO(buf, val)	zkPackBufSetUint16((buf),    ZKUDPHEADER_UDPCHECKSUM, (val))

/*
 *	ICMP header field manipulation
 */

#define zkICMPHeaderGetType(buf)					zkPackBufGetUint8((buf),     ZKICMPHEADER_TYPE)
#define zkICMPHeaderGetCode(buf)					zkPackBufGetUint8((buf),     ZKICMPHEADER_CODE)
#define zkICMPHeaderGetChecksum(buf)				zkPackBufGetUint16Nbo((buf), ZKICMPHEADER_CHECKSUM)
#define zkICMPHeaderGetIdentifier(buf)				zkPackBufGetUint16Nbo((buf), ZKICMPHEADER_IH_ID)
#define zkICMPHeaderGetIdentifierNBO(buf)			zkPackBufGetUint16((buf),    ZKICMPHEADER_IH_ID)
#define zkICMPHeaderGetSequenceNumber(buf)			zkPackBufGetUint16Nbo((buf), ZKICMPHEADER_IH_SEQ)
#define zkICMPHeaderGetSequenceNumberNBO(buf)		zkPackBufGetUint16((buf),    ZKICMPHEADER_IH_SEQ)
#define zkICMPHeaderGetNextHop_MTU(buf)				zkPackBufGetUint16Nbo((buf), ZKICMPHEADER_MTU_NEXTHOP)
#define zkICMPHeaderGetNextHop_MTUNBO(buf)			zkPackBufGetUint16((buf),    ZKICMPHEADER_MTU_NEXTHOP)

#define zkICMPHeaderSetType(buf, val)				zkPackBufSetUint8((buf),     ZKICMPHEADER_TYPE, (val))
#define zkICMPHeaderSetCode(buf, val)				zkPackBufSetUint8((buf),     ZKICMPHEADER_CODE, (val))
#define zkICMPHeaderSetChecksum(buf, val)			zkPackBufSetUint16Nbo((buf), ZKICMPHEADER_CHECKSUM, (val))
#define zkICMPHeaderSetIdentifier(buf, val)			zkPackBufSetUint16Nbo((buf), ZKICMPHEADER_IH_ID, (val))
#define zkICMPHeaderSetIdentifierNBO(buf, val)		zkPackBufSetUint16((buf),    ZKICMPHEADER_IH_ID, (val))
#define zkICMPHeaderSetSequenceNumber(buf, val)		zkPackBufSetUint16Nbo((buf), ZKICMPHEADER_IH_SEQ, (val))
#define zkICMPHeaderSetSequenceNumberNBO(buf, val)	zkPackBufSetUint16((buf),    ZKICMPHEADER_IH_SEQ, (val))
#define zkICMPHeaderSetNextHop_MTU(buf, val)		zkPackBufSetUint16Nbo((buf), ZKICMPHEADER_MTU_NEXTHOP, (val))

/*
 *	DIX header field manipulation
 */

#define zkDIXHeaderGetDstAddressAddr(buf)		ZKPACKBUF_CONST_ADDR_OF((buf), ZKDIXHEADER_DSTADDRESS)
#define zkDIXHeaderGetSrcAddressAddr(buf)		ZKPACKBUF_CONST_ADDR_OF((buf), ZKDIXHEADER_SRCADDRESS)
#define zkDIXHeaderGetType(buf)					zkPackBufGetUint16Nbo((buf),   ZKDIXHEADER_TYPE)
#define zkDIXHeaderGetTypeNBO(buf)				zkPackBufGetUint16((buf),      ZKDIXHEADER_TYPE)


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKPACKET_H */
