/*
 * $Id: cksum.c,v 1.15 2001/04/19 15:54:11 alainc Exp $
 */

static const char * module_id_ = "$Id: cksum.c,v 1.15 2001/04/19 15:54:11 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"


#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkNetinetIn.h" /* htons(), ntohs(), ... */
#include "zkArpaInet.h"
#include "zkPacket.h"


#define __IPPACKET_SZ          20
#define	__UDPPACKET_SZ          8
#define __TCPPACKET_SZ         20



/**
 *  Compute a ZkTCPIPHeader checksum
 *
 *
 */

DLLExport uint16_t
zkTCPHeaderChecksum (
  void * tcph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  void * iph = NULL;
  size_t i = 0;
  size_t datalen = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!tcph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = ZKPACKBUF_ADDR_OF(tcph, ZKUDPHEADER_IP);
  zkPackBufSetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM, 0);

  datalen = zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4);
  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SRCIPADDRESS);
  cksum = data[0] + data[1];
  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_DSTIPADDRESS);
  cksum += data[0] + data[1];
  cksum += htons(6) + 
           htons((uint16_t) (zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4)));
  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)tcph)[datalen + (zkIPHeaderGetIHL(iph) * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SIZEOF);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  } 

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  zkPackBufSetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM, (uint16_t) ~cksum);

  zkIPHeaderChecksum(iph);

  return zkPackBufGetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM);
}

/**
 *  Compute a ZkUDPIPHeader checksum.
 *
 */

DLLExport uint16_t
zkUDPHeaderChecksum (
  void * udph ,
  size_t datalen )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  void * iph = NULL;
  size_t i = 0;

  if (!udph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = ZKPACKBUF_ADDR_OF(udph, ZKUDPHEADER_IP);
  zkPackBufSetUint16(udph, ZKUDPHEADER_UDPCHECKSUM, 0);

  datalen = zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4);
  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SRCIPADDRESS);
  cksum = data[0] + data[1];
  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_DSTIPADDRESS);
  cksum += data[0] + data[1];
  cksum += htons(ZK_IPPROTO_UDP) + 
           htons((uint16_t) (zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4)));

  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)udph)[datalen + (zkIPHeaderGetIHL(iph) * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) ZKPACKBUF_ADDR_OF(iph, ZKIPHEADER_SIZEOF);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  zkPackBufSetUint16(udph, ZKUDPHEADER_UDPCHECKSUM, (uint16_t) ~cksum);

  zkIPHeaderChecksum(iph);

  return zkPackBufGetUint16(udph, ZKUDPHEADER_UDPCHECKSUM);
}


