/*
 * $Id: zkIP.c,v 1.12 2001/04/19 15:54:12 alainc Exp $
 */

static const char * module_id_ = "$Id: zkIP.c,v 1.12 2001/04/19 15:54:12 alainc Exp $";

#include "zks.h"


#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkArpaInet.h"
#include "zkNetinetIn.h" /* htons(), ntohs(), ... */
#include "zkPacket.h"


#define __IPPACKET_SZ          20
#define	__UDPPACKET_SZ          8
#define __TCPPACKET_SZ         20

/**
 *  Compute a ZkIPHeader checksum.
 *
 *
 */

uint16_t
zkIPHeaderChecksum (
  void * iph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  size_t datalen = 0;
  size_t i = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!iph) {
    return 0;
  }
  zkPackBufSetUint16(iph, ZKIPHEADER_HEADERCHKSUM, 0);
  data = (unsigned short *) iph;
  datalen = zkIPHeaderGetIHL(iph) * 2;
  cksum = 0;
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }
  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  zkPackBufSetUint16(iph, ZKIPHEADER_HEADERCHKSUM, (uint16_t) ~cksum);

  return zkPackBufGetUint16(iph, ZKIPHEADER_HEADERCHKSUM);
}

#if !defined(ZK_KERNEL)

static char *
zkIPAddrToStr (
  uint32_t addr )
{
  unsigned char * ptr = NULL;
  static char buffer[32];

  ptr = (unsigned char *) &addr;
  zkSnprintf(buffer,sizeof buffer,"%d.%d.%d.%d",ptr[0],ptr[1],ptr[2],ptr[3]);
  return buffer;
}
  
int
zkIPHeaderDump (
  void * iph ,
  FILE * stream )
{

  zkFprintf(stream,"IP Packet Header pointer %p\n", (void *)iph);
  if (iph == NULL) {
    return 0;
  }
  zkFprintf(stream,"  IP Packet Length        %d bytes\n",zkIPHeaderGetIHL(iph) * 4);
  zkFprintf(stream,"  Version                 %d\n",zkIPHeaderGetVersion(iph));
  zkFprintf(stream,"  Type Of Service         %d\n",zkIPHeaderGetTypeOfSvr(iph));
  zkFprintf(stream,"  Total Length            %d\n",zkIPHeaderGetTotalLength(iph));
  zkFprintf(stream,"  Identification          %d\n",zkIPHeaderGetIdentification(iph));
  zkFprintf(stream,"  Fragmentation Flags     0x%X\n",
          (zkIPHeaderGetFlags_FrameOffs(iph)&0xE000)>>13);
  zkFprintf(stream,"  Fragmentation Offset    %d\n",
          zkIPHeaderGetFlags_FrameOffs(iph)&0x1FFF);
  zkFprintf(stream,"  Time To Live            %d\n",zkIPHeaderGetTimeToLive(iph));
  zkFprintf(stream,"  Protocol                %d\n",zkIPHeaderGetProtocol(iph));
  zkFprintf(stream,"  Header Checksum         %d\n",zkPackBufGetUint16(iph, ZKIPHEADER_HEADERCHKSUM));
  zkFprintf(stream,"  Source IP Address       %s\n",
          zkIPAddrToStr(zkPackBufGetUint32(iph, ZKIPHEADER_SRCIPADDRESS)));
  zkFprintf(stream,"  Destination IP Address  %s\n",
          zkIPAddrToStr(zkPackBufGetUint32(iph, ZKIPHEADER_DSTIPADDRESS)));

  return 1;
}

#endif	/* !defined(ZK_KERNEL) */

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)

int zkIPHeaderDump ( void * iph )
{
	uint32_t	addr;
	uint8_t*	p = (uint8_t*)&addr;

  dprintf("IP Packet Header pointer 0x%x\n", (void *)iph);
  if (iph == NULL) {
    return 0;
  }
  dprintf("  IP Packet Length        %d bytes\n",zkIPHeaderGetIHL(iph) * 4);
  dprintf("  Version                 %d\n",zkIPHeaderGetVersion(iph));
  dprintf("  Type Of Service         %d\n",zkIPHeaderGetTypeOfSvr(iph));
  dprintf("  Total Length            %d\n",zkIPHeaderGetTotalLength(iph));
  dprintf("  Identification          %d\n",zkIPHeaderGetIdentification(iph));
  dprintf("  Fragmentation Flags     0x%X\n",
          (zkIPHeaderGetFlags_FrameOffs(iph)&0xE000)>>13);
  dprintf("  Fragmentation Offset    %d\n",
          zkIPHeaderGetFlags_FrameOffs(iph)&0x1FFF);
  dprintf("  Time To Live            %d\n",zkIPHeaderGetTimeToLive(iph));
  dprintf("  Protocol                %d\n",zkIPHeaderGetProtocol(iph));
  dprintf("  Header Checksum         %d\n",zkPackBufGetUint16(iph, ZKIPHEADER_HEADERCHKSUM));
  addr = zkPackBufGetUint32(iph, ZKIPHEADER_SRCIPADDRESS);
  dprintf("  Source IP Address       %d.%d.%d.%d\n", p[0], p[1], p[2], p[3]);
  addr = zkPackBufGetUint32(iph, ZKIPHEADER_DSTIPADDRESS);
  dprintf("  Destination IP Address  %d.%d.%d.%d\n", p[0], p[1], p[2], p[3]);

  return 1;
}
#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

int
zkIPHeaderSourceAddress (

  void * iph ,
  uint32_t * addr )

{
  *addr = zkPackBufGetUint32(iph, ZKIPHEADER_SRCIPADDRESS);
  return 1;
}

int
zkIPHeaderDestinationAddress (

  void * iph ,
  uint32_t * addr )

{
  *addr = zkPackBufGetUint32(iph, ZKIPHEADER_DSTIPADDRESS);
  return 1;
}

int
zkIPHeaderSetSourceAddress (

  void * iph ,
  uint32_t addr )

{
  zkPackBufSetUint32(iph, ZKIPHEADER_SRCIPADDRESS, addr);
  return 1;
}
 
DLLExport int
zkIPHeaderSetDestinationAddress (

  void * iph ,
  uint32_t addr )

{
  zkPackBufSetUint32(iph, ZKIPHEADER_DSTIPADDRESS, addr);
  return 1;
}

/**
 *  Strip out all of the IP packet options.  This is fairly draconian but
 *  what we're doing either makes the IP options useless (in the case of
 *  source routing) or dangerous (in the case of route record).
 *  This function assumes that the data immediatly follows the IP header in
 *  memory.  You must recompute the IP checksum after striping the options 
 *  (this function will not recompute it for you).
 *  Note:  In order to avoid breaking things, I am copying the data portion
 *         of the packet overtop the IP options.  It may be more efficient to
 *         go the otherway, ie copy the header data forward to overwrite the
 *         options, however, this then makes the IP header pointer move.
 *  Note:  We may want to keep one option, StreamID, if it is preserved
 *         both ways, we might be able to use it to keep track of ACI <->
 *         ICMP message going through the wormhole.
 *
 *
 *  @param iph  The IP packet header pointer
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int
zkIPHeaderStripOptions (
  void * iph )
{
  int i = 0;
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (zkIPHeaderGetIHL(iph) == 5) {
    return 1;
  } else if (zkIPHeaderGetIHL(iph) < 5) {
    return 0;        /* Cant have a proper IP packet with length  < 5 */
  }
  cat = &(((char *)iph)[20]);
  mouse = &(((char *)iph)[zkIPHeaderGetIHL(iph) * 4]);
  len = (int) zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4);
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++; 
  }
  zkIPHeaderSetTotalLength(iph, (short) (zkIPHeaderGetTotalLength(iph) - (zkIPHeaderGetIHL(iph) * 4 - 20)));
  zkIPHeaderSetIHL(iph, 5);
  zkPackBufSetUint16(iph, ZKIPHEADER_HEADERCHKSUM, 0);
  return 1;
}
