/*
 * $Id: zkTCP.c,v 1.11 2001/04/19 15:54:12 alainc Exp $
 */

static const char * module_id_ = "$Id: zkTCP.c,v 1.11 2001/04/19 15:54:12 alainc Exp $";
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include "zks.h"

#include "zkStdio.h"
#include "zkStdlib.h"
#include "zkArpaInet.h"
#include "zkNetinetIn.h" /* htons(), ntohs(), ... */
#include "zkPacket.h"


#if !defined(ZK_KERNEL)

DLLExport int
zkTCPHeaderDump (
  void * tcph ,
  FILE * stream )
{
  char yn[2] = { ' ', 'X' };

  zkFprintf(stream,"TCP Packet Header Pointer %p\n",(void *)tcph);
  if (tcph == NULL) {
    return 0;
  }
  zkFprintf(stream,"  Source Port             %d\n",zkTCPHeaderGetSrcPort(tcph));
  zkFprintf(stream,"  Destination Port        %d\n",zkTCPHeaderGetDstPort(tcph));
  zkFprintf(stream,"  Sequence Number         %lu\n",(unsigned long)zkTCPHeaderGetSequenceNumber(tcph));
  zkFprintf(stream,"  Acknowledgement number  %lu\n",(unsigned long)zkTCPHeaderGetAckNumber(tcph));
  zkFprintf(stream,"  Reserved                %d\n",zkTCPHeaderGetRes(tcph));
  zkFprintf(stream,"  Header Length           %d\n",zkTCPHeaderGetDoff(tcph) * 4);
  zkFprintf(stream,"  FIN flag                %c\n",yn[zkTCPHeaderGetFin(tcph)]);
  zkFprintf(stream,"  SYN flag                %c\n",yn[zkTCPHeaderGetSyn(tcph)]);
  zkFprintf(stream,"  RST flag                %c\n",yn[zkTCPHeaderGetRst(tcph)]);
  zkFprintf(stream,"  PSH flag                %c\n",yn[zkTCPHeaderGetPsh(tcph)]);
  zkFprintf(stream,"  ACK flag                %c\n",yn[zkTCPHeaderGetAck(tcph)]);
  zkFprintf(stream,"  URG flag                %c\n",yn[zkTCPHeaderGetUrg(tcph)]);
  zkFprintf(stream,"  Window                  %d\n",zkTCPHeaderGetWindow(tcph));
  zkFprintf(stream,"  Header Checksum         %d\n",zkPackBufGetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM));
  zkFprintf(stream,"  Urgent Pointer          %d\n",zkTCPHeaderGetUrgentPointer(tcph));
  return 1;
}

#endif	/* !defined(ZK_KERNEL) */

#if defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG)

DLLExport int
zkTCPHeaderDump ( void * tcph )
{
  char yn[2] = { ' ', 'X' };

  dprintf("TCP Packet Header Pointer %p\n",(void *)tcph);
  if (tcph == NULL) {
    return 0;
  }
  dprintf("  Source Port             %d\n",zkTCPHeaderGetSrcPort(tcph));
  dprintf("  Destination Port        %d\n",zkTCPHeaderGetDstPort(tcph));
  dprintf("  Sequence Number         %lu\n",(unsigned long)zkTCPHeaderGetSequenceNumber(tcph));
  dprintf("  Acknowledgement number  %lu\n",(unsigned long)zkTCPHeaderGetAckNumber(tcph));
  dprintf("  Reserved                %d\n",zkTCPHeaderGetRes(tcph));
  dprintf("  Header Length           %d\n",zkTCPHeaderGetDoff(tcph) * 4);
  dprintf("  FIN flag                %c\n",yn[zkTCPHeaderGetFin(tcph)]);
  dprintf("  SYN flag                %c\n",yn[zkTCPHeaderGetSyn(tcph)]);
  dprintf("  RST flag                %c\n",yn[zkTCPHeaderGetRst(tcph)]);
  dprintf("  PSH flag                %c\n",yn[zkTCPHeaderGetPsh(tcph)]);
  dprintf("  ACK flag                %c\n",yn[zkTCPHeaderGetAck(tcph)]);
  dprintf("  URG flag                %c\n",yn[zkTCPHeaderGetUrg(tcph)]);
  dprintf("  Window                  %d\n",zkTCPHeaderGetWindow(tcph));
  dprintf("  Header Checksum         %d\n",zkPackBufGetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM));
  dprintf("  Urgent Pointer          %d\n",zkTCPHeaderGetUrgentPointer(tcph));
  return 1;
}
#endif	/* defined(ZK_KERNEL_MAC_OS8) && defined(_DEBUG) */

DLLExport int
zkTCPHeaderAcknowledgement (
  void * tcph ,
  uint32_t * ack )
{
  (void) module_id_; /* Get rid of compile warning */
  if (tcph && ack) {
    *ack = zkTCPHeaderGetAckNumber(tcph);
    return 1;
  } 
  return 0;
} 

DLLExport int
zkTCPHeaderSequence (
  void * tcph ,
  uint32_t * seq )
{
  if (tcph && seq) {
    *seq = zkTCPHeaderGetSequenceNumber(tcph);
    return 1;
  }
  return 0;
}



uint16_t
zkTCPHeaderSourcePort (
  void * tcph ,
  uint16_t * port ) 
{
  *port = zkTCPHeaderGetSrcPort(tcph);
  return *port ;
}

uint16_t
zkTCPHeaderDestinationPort (
  void * tcph ,
  uint16_t * port )
{
  *port = zkTCPHeaderGetDstPort(tcph);
  return *port;
}

int
zkTCPHeaderSetSourcePort (
  void * tcph ,
  uint16_t port )
{  
  zkTCPHeaderSetSrcPort(tcph, port);
  return 1;
}

int
zkTCPHeaderSetDestinationPort (
  void * tcph ,
  uint16_t port )
{
  zkTCPHeaderSetDstPort(tcph, port);
  return 1;
}

/**
 *  Set the maximum segment size option on the TCP packet to the specified
 *  value.  This function will move the packet data payload forward 4 bytes
 *  in order to accomodate this TCP option.  The header lengths will be 
 *  updated accordingly but the header checksums will not.  You must be
 *  sure to recompute them yourself.
 * 
 *  @param tcph  A pointer to a TCP header structure
 *  @param mss   The new Maximum Segment Size option
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

DLLExport int 
zkTCPHeaderSetMSS (
  void * tcph ,
  uint16_t mss )
{
  int i = 0; /* Loop index for moving packet payload */
  int len = 0;  /* Number of bytes to move */
  int hlen = 0;  /* Length of combined headers */
  char * cat = NULL;
  char * mouse = NULL;
  unsigned char *mss_ptr = NULL;
  
  /*
   * Move the payload forward by 4 bytes
   */

  cat = &(((char *)tcph)[zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) - 1]);
  mouse = &(((char *)tcph)[zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) - 5]);
  hlen = (zkIPHeaderGetIHL(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) + zkTCPHeaderGetDoff(tcph)) * 4;
  len = zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) - hlen;
  for (i = 0; i < len; i++) {
    *cat-- = *mouse--;
  }
  
  /*
   * Append the MSS to the end of the option list
   */

  mss_ptr = (unsigned char *) &(((char *)tcph)[hlen]);
  mss_ptr[0] = 0x02; /* Set the MSS option code, 2 */
  mss_ptr[1] = 0x04; /* Set the MSS length, 4 */
  mss_ptr[2] = (unsigned char) (mss / 0x100);
  mss_ptr[3] = (unsigned char) (mss % 0x100);
#if 0
  mss_ptr[0] = 0x0402;   /* Set the MSS option code, kind 2, length 4 */
  mss_ptr[1] = htons(mss);  /* Set the new MSS value */
#endif

  /*
   * Update header lengths
   */

  zkTCPHeaderSetDoff(tcph, zkTCPHeaderGetDoff(tcph) + 1);
  zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP), (short) (zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) + 4));
  zkPackBufSetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM, 0);
  zkPackBufSetUint16(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP), ZKIPHEADER_HEADERCHKSUM, 0);
  return 1;
}

/**
 *  This function will remove all of the TCP options from the TCP header.  It
 *  assumes that the accompanying packet data follows immediatly after in 
 *  memory and that the tcp header also has a proper IP header immediatly
 *  before.
 *
 *  @param tcph The TCP header to strip of options
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure.
 */

DLLExport int
zkTCPHeaderStripOptions (
  void * tcph )
{
  int i = 0; /* Loop index */
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (zkTCPHeaderGetDoff(tcph) == 5) {
    return 1;
  } else if (zkTCPHeaderGetDoff(tcph) < 5) {
    return 0;
  }
  cat = &(((char *)tcph)[zkIPHeaderGetIHL(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) * 4 + 20]);
  mouse = &(((char *)tcph)[(zkIPHeaderGetIHL(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) + zkTCPHeaderGetDoff(tcph)) * 4]);
  len = zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) - (zkIPHeaderGetIHL(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP)) + zkTCPHeaderGetDoff(tcph)) * 4;
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++;
  }
  zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP), (short) (zkIPHeaderGetTotalLength(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP))
                               - (zkTCPHeaderGetDoff(tcph) * 4 - 20) ));
  zkTCPHeaderSetDoff(tcph, 5);
  zkPackBufSetUint16(tcph, ZKTCPHEADER_TCPCHECKSUM, 0);
  zkPackBufSetUint16(ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_IP), ZKIPHEADER_HEADERCHKSUM, 0);
  return 1;
}


/*
 *  Modify the existing MSS value.  If the MSS option cannot
 *  be found, the function fails and returns a 0.
 *
 *  @param out_tcph The TCP header structure to analyze.  It is assumed that the
 *              packet data follows immediatly after in memory.
 *  @param in_mss  New MSS value in host byte order HBO
 *  @author Mario Contestabile
 *  @version 1.0
 *  @return A non-zero value if the MSS option was found, 0 otherwise.
*/
DLLExport int
zkTCPHeaderModifyMSS (
  void * out_tcph,                       /* TCP Header */
  const uint16_t in_mss )
{
  unsigned char * ptr = NULL;
  int len = 0;
  int i = 0; /* Loop index for scanning option data */

  if (zkTCPHeaderGetDoff(out_tcph) <= 5) {
    return 0;      /* No options means no MSS specified */
  }
  len = zkTCPHeaderGetDoff(out_tcph) * 4 - 20;  /* Number of bytes devoted to options */
  ptr = (unsigned char *) ZKPACKBUF_ADDR_OF(out_tcph, ZKTCPHEADER_SIZEOF);
  for (i = 0; i < len; i++,ptr++) {
    switch (*ptr) {
    case ZK_TCPOPT_EOL:  /* End of option list, terminate... */
      i = len;
      break;
    case ZK_TCPOPT_NOP: /* Nop, skip it */
      break;
    case ZK_TCPOPT_MSS: /* This is the MSS option */
      i++;
      if (i >= len) {
        break;  /* Do we have enough data left in the packet for an MSS ? */
      } 
      ptr++;
      if (*ptr != 4) {   /* Is the length correct */
        i = len;
        break;
      }
      ptr++;

      /* There are two bytes where to store the mss value */
      ptr[0] = (unsigned char) (in_mss / 0x100);
      ptr[1] = (unsigned char) (in_mss % 0x100);
      return 1;
      break;
    default:
      i++; /* Skip the length byte */
      ptr++;
      break;
    }
  }
  return 0;
}

/**
 *  Analyze a TCP packet and return the MSS option.  If this option cannot
 *  be found, the function fails and returns a 0.
 *
 *  @param tcph The TCP header structure to analyze.  It is assumed that the
 *              packet data follows immediatly after in memory.
 *  @param mss  Returns the MSS (Maximum Segment Size for the connection)
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value if the MSS option was found, 0 otherwise.
 */

DLLExport int
zkTCPHeaderGetMSS (
  void * tcph ,
  uint16_t * mss )
{
  unsigned char * ptr = NULL;
  int len = 0;
  int i = 0; /* Loop index for scanning option data */

  if (zkTCPHeaderGetDoff(tcph) <= 5) {
    return 0;      /* No options means no MSS specified */
  }
  len = zkTCPHeaderGetDoff(tcph) * 4 - 20;  /* Number of bytes devoted to options */
  ptr = (unsigned char *) ZKPACKBUF_ADDR_OF(tcph, ZKTCPHEADER_SIZEOF);
  for (i = 0; i < len; i++,ptr++) {
    switch (*ptr) {
    case 0:  /* End of option list, terminate... */
      i = len;
      break;
    case 1: /* Nop, skip it */
      break;
    case 2: /* This is the MSS option */
      i++;
      if (i >= len) {
        break;  /* Do we have enough data left in the packet for an MSS ? */
      } 
      ptr++;
      if (*ptr != 4) {   /* Is the length correct */
        i = len;
        break;
      }
      ptr++;
#if 0
      *mss = (uint16_t) ntohs(*((uint16_t *)ptr));  /* Get the MSS */
#else
      *mss = (uint16_t) ptr[0] * 0x100 + ptr[1];
#endif
      return 1;
      break;
    default:
      i++; /* Skip the length byte */
      ptr++;
      break;
    }
  }
  return 0;
}  

