/*	tgetset.c
 *	
 *  Tests the get/set functions for the "new-style" zkPackBuf structures
 *  by comparing results with the "old-style" structures
 *
 */
static const char * module_id_ = "$Id: manipfct.c,v 1.2 2001/04/19 15:54:16 alainc Exp $";

#include "zkPacket.h"
#include "zkArpaInet.h"
#include "zkString.h"
#include "zkIntTypes.h"
#include "zkStdlib.h"

#if defined(__MWERKS__)
#	pragma ANSI_strict off
#endif

/* original IP header struct */
typedef struct ZkIPHeader ZkIPHeader;
#pragma pack(1)
struct ZkIPHeader {
#ifdef ZK_LITTLE_ENDIAN
  uint8_t ihl:4;
  uint8_t version:4;
#elif ZK_BIG_ENDIAN
  uint8_t version:4;
  uint8_t ihl:4;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif
  uint8_t TypeOfSvr;
  uint16_t TotalLength;
  uint16_t Identification;
  uint16_t Flags_FrameOffs;
  uint8_t  TimeToLive;
  uint8_t  Protocol;
  uint16_t HeaderChkSum;
  uint32_t SrcIpAddress;
  uint32_t DstIpAddress;
};
#pragma pack()

/* original TCP header struct */
typedef struct ZkTCPHeader ZkTCPHeader;
#pragma pack(1)
struct ZkTCPHeader {
  ZkIPHeader  IP;
  uint16_t SrcPort;
  uint16_t DstPort;
  uint32_t SequenceNumber;
  uint32_t AckNumber;
#ifdef ZK_LITTLE_ENDIAN
  uint16_t res1:4;
  uint16_t doff:4;
  uint16_t fin:1;
  uint16_t syn:1;
  uint16_t rst:1;
  uint16_t psh:1;
  uint16_t ack:1;
  uint16_t urg:1;
  uint16_t res2:2;
#elif ZK_BIG_ENDIAN
  uint16_t doff:4;
  uint16_t res1:4;
  uint16_t res2:2;
  uint16_t urg:1;
  uint16_t ack:1;
  uint16_t psh:1;
  uint16_t rst:1;
  uint16_t syn:1;
  uint16_t fin:1;
#else
#error "You must specify -DZK_LITTLE_ENDIAN or -DZK_BIG_ENDIAN (set ZK_BYTEORDER using the zk_endian script)"
#endif 
  uint16_t Window;
  uint16_t TcpCheckSum;
  uint16_t UrgentPointer;
};
#pragma pack()

/* original UDP header struct */
#pragma pack(1)
typedef struct ZkUDPHeader ZkUDPHeader;
struct ZkUDPHeader {
  ZkIPHeader IP;
  uint16_t SrcPort;
  uint16_t DstPort;
  uint16_t UdpLength;
  uint16_t UdpCheckSum;
};
#pragma pack()

/* original ICMP header struct */
typedef struct ZkICMPHeader ZkICMPHeader;
#pragma pack(1)
struct ZkICMPHeader {
  uint8_t Type;
  uint8_t code;
  uint16_t CheckSum;  
  uint16_t Identifier;
  uint16_t SequenceNumber; 
};
#pragma pack()

/* common data size to match new style buffers with the following structs...*/
#define DATA_SIZE 16

/* IP header struct with data*/
typedef struct ZkIPHeaderAndData ZkIPHeaderAndData;
#pragma pack(1)
struct ZkIPHeaderAndData {
	ZkIPHeader header;
	char data[DATA_SIZE];
};
#pragma pack()

/* TCP header struct with data*/
typedef struct ZkTCPHeaderAndData ZkTCPHeaderAndData;
#pragma pack(1)
struct ZkTCPHeaderAndData {
	ZkTCPHeader header;
	char data[DATA_SIZE];
};
#pragma pack()

/* UDP header struct with data*/
typedef struct ZkUDPHeaderAndData ZkUDPHeaderAndData;
#pragma pack(1)
struct ZkUDPHeaderAndData {
	ZkUDPHeader header;
	char data[DATA_SIZE];
};
#pragma pack()

/* IP header struct with options*/
typedef struct ZkIPHeaderAndOptions ZkIPHeaderAndOptions;
#pragma pack(1)
struct ZkIPHeaderAndOptions {
	ZkIPHeader header;
	char options[40];
	char data[DATA_SIZE];
};
#pragma pack()

/* TCP header struct with options*/
typedef struct ZkTCPHeaderAndOptions ZkTCPHeaderAndOptions;
#pragma pack(1)
struct ZkTCPHeaderAndOptions {
	ZkTCPHeader header;
	char options[40];
	char data[DATA_SIZE];
};
#pragma pack()

#if defined(__MWERKS__)
#	pragma ANSI_strict reset
#endif

/* old functions signatures */

uint16_t oldICMPHeaderChecksum ( ZkICMPHeader * icmph , size_t datalen );
int oldICMPCreateFragmentationNeeded ( ZkIPHeader * iph ,  unsigned char * mesg ,  size_t * size );
int oldICMPCreateTimeExceeded (  ZkIPHeader * iph ,  uint8_t code ,  unsigned char * mesg ,  size_t * size );
int oldICMPCreateDestinationUnreachable ( ZkIPHeader * iph ,  uint8_t code ,  unsigned char * mesg ,  size_t * size );
int oldICMPCreateSourceQuench ( ZkIPHeader * iph , unsigned char * mesg ,  size_t * size );

static char *oldIPAddrToStr ( uint32_t addr );
int oldIPHeaderDump ( ZkIPHeader * iph , FILE * stream );
int oldIPHeaderSourceAddress ( ZkIPHeader * iph , uint32_t * addr );
int oldIPHeaderDestinationAddress ( ZkIPHeader * iph , uint32_t * addr );
int oldIPHeaderSetSourceAddress ( ZkIPHeader * iph , uint32_t addr );
int oldIPHeaderSetDestinationAddress ( ZkIPHeader * iph , uint32_t addr );
int oldIPHeaderStripOptions (  ZkIPHeader * iph );
uint16_t oldIPHeaderChecksum ( ZkIPHeader * iph );

int oldTCPHeaderDump ( ZkTCPHeader * tcph , FILE * stream );
int oldTCPHeaderAcknowledgement ( ZkTCPHeader * tcph , uint32_t * ack );
int oldTCPHeaderSequence ( ZkTCPHeader * tcph , uint32_t * seq );
uint16_t oldTCPHeaderSourcePort ( ZkTCPHeader * tcph , uint16_t * port );
uint16_t oldTCPHeaderDestinationPort ( ZkTCPHeader * tcph , uint16_t * port );
int oldTCPHeaderSetSourcePort ( ZkTCPHeader * tcph , uint16_t port );
int oldTCPHeaderSetDestinationPort ( ZkTCPHeader * tcph , uint16_t port );
int oldTCPHeaderSetMSS ( ZkTCPHeader * tcph , uint16_t mss );
int oldTCPHeaderStripOptions ( ZkTCPHeader * tcph );
int oldTCPHeaderGetMSS ( ZkTCPHeader * tcph , uint16_t * mss );
uint16_t oldTCPHeaderChecksum ( ZkTCPHeader * tcph );


int oldUDPHeaderDump ( ZkUDPHeader * udph , FILE * stream );
uint16_t oldUDPHeaderSourcePort ( ZkUDPHeader * udph , uint16_t * port );
uint16_t oldUDPHeaderDestinationPort ( ZkUDPHeader * udph , uint16_t * port );
int oldUDPHeaderSetSourcePort ( ZkUDPHeader * udph , uint16_t port );
int oldUDPHeaderSetDestinationPort ( ZkUDPHeader * udph , uint16_t port );
uint16_t oldUDPHeaderChecksum ( ZkUDPHeader * udph , size_t datalen );

/* Original ICMP functions */

/**
 *  Compute the checksum of an ICMP datagram.  The checksum will be written 
 *  inside the packet itself
 * 
 *  @param icmph The ICMP header pointer for which to return the check sum.
 *  @param datalen The number of bytes for which to compute the checksum
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return The header checksum
 */

uint16_t oldICMPHeaderChecksum (
  ZkICMPHeader * icmph ,
  size_t datalen ) /* Number of bytes */
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  size_t i = 0;

  if (!icmph) {
    return 0;
  }
  icmph->CheckSum = 0;
  data = (unsigned short *) icmph;
  datalen /= 2;
  cksum = 0;
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }
  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  icmph->CheckSum = (uint16_t) ~cksum;

  return icmph->CheckSum;
}

/*
 * The following ICMP support functions are hacks to do some network magic
 *
 */

/**
 * This function creates an ICMP fragmentation needed message which is can
 * be sent back to a remote host.
 *
 * @param iph The offending IP packet you wish to get fragmented
 * @param mesg Buffer to hold the message
 * @param size Holds the original size of buffer, returns bytes used
 * @author Philippe Boucher
 * @version 1.0
 * @return A non-zero value on success, a 0 on failure
 */

int oldICMPCreateFragmentationNeeded (
  ZkIPHeader * iph ,
  unsigned char * mesg ,
  size_t * size )
{
  return oldICMPCreateDestinationUnreachable(iph,
                                            ZK_ICMP_FRAGMENTATION_NEEDED,
                                            mesg,size);
}

/**
 * Function to create a generic time exceeded message
 *
 */

int oldICMPCreateTimeExceeded (
  ZkIPHeader * iph ,
  uint8_t code ,
  unsigned char * mesg ,
  size_t * size )
{
  uint16_t totlen = 0;
  ZkIPHeader * header = NULL;

  header = (ZkIPHeader *) mesg;

  /*
   * ip header (20) + icmp header (8) + returned iph (ihl * 4) + 8 bytes
   */

  totlen = iph->ihl * 4 + 36;  
  if (*size < (size_t) totlen) {
    return 0;
  }
  *size = (size_t) totlen;

  /*
   * Make the return IP datagram
   */

  mesg[0] = 0x45; /* Version and header length */
  mesg[1] = 0; /* Type of service */
  mesg[2] = totlen / 0x100; /* First byte of total packet length */
  mesg[3] = totlen % 0x100; /* Second byte of total packet length */
  mesg[4] = 0xfe; /* First half of identification */
  mesg[5] = 0xed; /* Second half of identification */
  mesg[6] = 0; /* Fragmentation flags and first half of frag offset */
  mesg[7] = 0; /* Second half of frag offset */
  mesg[8] = 255; /* Time to live */
  mesg[9] = ZK_IPPROTO_ICMP; /* IP Protocol type */
  mesg[10] = 0; /*First half of checksum */
  mesg[11] = 0; /* Second half of checksum */
  memcpy((void *)&(mesg[12]),(void *)&(iph->DstIpAddress),4);
  memcpy((void *)&(mesg[16]),(void *)&(iph->SrcIpAddress),4);
  
  /*
   * ICMP Header  
   */

  mesg[20] = 11; /* Type = Time exceeded */
  mesg[21] = code; /* Code (see zkPacket.h for list) */
  mesg[22] = 0; /* First half of ICMP checksum */
  mesg[23] = 0; /* Second half of ICMP checksum */
  mesg[24] = 0; /* unused, must be 0 */
  mesg[25] = 0; /* unused, must be 0 */
  mesg[26] = 0; /* unused, must be 0 */
  mesg[27] = 0; /* unused, must be 0 */
 
  /*
   * IP Header + first 64 bits of the datagram
   */

  memcpy((void *) &(mesg[28]),(void *) iph,iph->ihl * 4 + 8); 

  /*
   * Compute the ICMP header checksum
   */

  oldICMPHeaderChecksum((ZkICMPHeader *)&(mesg[20]),iph->ihl * 4 + 16);

  /*
   * Compute the IP header checksum 
   */

  oldIPHeaderChecksum(header);

  /*
   * All done.  the buffer can be dropped onto the network (with a raw socket)
   */

  return 1;
}

/**
 * Function to generate generic Destination Unreachable ICMP messages.
 * 
 *  @param iph The offending IP header
 *  @param code The message code (see the list in zkPacket.h)
 *  @param mesg Buffer to hold the ICMP message
 *  @param size The size of the buffer, returns the number of bytes used
 *  @version 1.0
 *  @author Philippe Boucher
 *  @return A non-zero value on success, a 0 on failure
 */

int oldICMPCreateDestinationUnreachable (
  ZkIPHeader * iph ,
  uint8_t code ,
  unsigned char * mesg ,
  size_t * size )
{
  uint16_t totlen = 0;
  ZkIPHeader * header = NULL;

  header = (ZkIPHeader *) mesg;

  /*
   * ip header (20) + icmp header (8) + returned iph (ihl * 4) + 8 bytes
   */

  totlen = iph->ihl * 4 + 36;  
  if (*size < (size_t) totlen) {
    return 0;
  }
  *size = (size_t) totlen;

  /*
   * Make the return IP datagram
   */

  mesg[0] = 0x45; /* Version and header length */
  mesg[1] = 0; /* Type of service */
  mesg[2] = totlen / 0x100; /* First byte of total packet length */
  mesg[3] = totlen % 0x100; /* Second byte of total packet length */
  mesg[4] = 0xfe; /* First half of identification */
  mesg[5] = 0xed; /* Second half of identification */
  mesg[6] = 0; /* Fragmentation flags and first half of frag offset */
  mesg[7] = 0; /* Second half of frag offset */
  mesg[8] = 255; /* Time to live */
  mesg[9] = ZK_IPPROTO_ICMP; /* IP Protocol type */
  mesg[10] = 0; /*First half of checksum */
  mesg[11] = 0; /* Second half of checksum */
  memcpy((void *)&(mesg[12]),(void *)&(iph->DstIpAddress),4);
  memcpy((void *)&(mesg[16]),(void *)&(iph->SrcIpAddress),4);
  
  /*
   * ICMP Header  
   */

  mesg[20] = 3; /* Type = destination unreachable */
  mesg[21] = code; /* Code (see zkPacket.h for list) */
  mesg[22] = 0; /* First half of ICMP checksum */
  mesg[23] = 0; /* Second half of ICMP checksum */
  mesg[24] = 0; /* unused, must be 0 */
  mesg[25] = 0; /* unused, must be 0 */
  mesg[26] = 0; /* unused, must be 0 */
  mesg[27] = 0; /* unused, must be 0 */
 
  /*
   * IP Header + first 64 bits of the datagram
   */

  memcpy((void *) &(mesg[28]),(void *) iph,iph->ihl * 4 + 8); 

  /*
   * Compute the ICMP header checksum
   */

  oldICMPHeaderChecksum((ZkICMPHeader *)&(mesg[20]),iph->ihl * 4 + 16);

  /*
   * Compute the IP header checksum 
   */

  oldIPHeaderChecksum(header);

  /*
   * All done.  the buffer can be dropped onto the network (with a raw socket)
   */

  return 1;
}

/**
 *  Create a source quench to stop the remote host from sending the 
 *  offending packet.
 *
 * @param iph The offending IP packet you wish to get fragmented
 * @param mesg Buffer to hold the message
 * @param size Holds the original size of buffer, returns bytes used
 * @author Philippe Boucher
 * @version 1.0
 * @return A non-zero value on success, a 0 on failure
 */

int oldICMPCreateSourceQuench (
  ZkIPHeader * iph ,
  unsigned char * mesg ,
  size_t * size )
{
  uint16_t totlen = 0;
  ZkIPHeader * header = NULL;

  header = (ZkIPHeader *) mesg;

  /*
   * ip header (20) + icmp header (8) + returned iph (ihl * 4) + 8 bytes
   */

  totlen = iph->ihl * 4 + 36;  
  if (*size < (size_t) totlen) {
    return 0;
  }
  *size = (size_t) totlen;

  /*
   * Make the return IP datagram
   */

  mesg[0] = 0x45; /* Version and header length */
  mesg[1] = 0; /* Type of service */
  mesg[2] = totlen / 0x100; /* First byte of total packet length */
  mesg[3] = totlen % 0x100; /* Second byte of total packet length */
  mesg[4] = 0xfe; /* First half of identification */
  mesg[5] = 0xed; /* Second half of identification */
  mesg[6] = 0; /* Fragmentation flags and first half of frag offset */
  mesg[7] = 0; /* Second half of frag offset */
  mesg[8] = 255; /* Time to live */
  mesg[9] = ZK_IPPROTO_ICMP; /* IP Protocol type */
  mesg[10] = 0; /*First half of checksum */
  mesg[11] = 0; /* Second half of checksum */
  memcpy((void *)&(mesg[12]),(void *)&(iph->DstIpAddress),4);
  memcpy((void *)&(mesg[16]),(void *)&(iph->SrcIpAddress),4);
  
  /*
   * ICMP Header  
   */

  mesg[20] = 4; /* Type = Source quench */
  mesg[21] = 0; /* Code  */
  mesg[22] = 0; /* First half of ICMP checksum */
  mesg[23] = 0; /* Second half of ICMP checksum */
  mesg[24] = 0; /* unused, must be 0 */
  mesg[25] = 0; /* unused, must be 0 */
  mesg[26] = 0; /* unused, must be 0 */
  mesg[27] = 0; /* unused, must be 0 */
 
  /*
   * IP Header + first 64 bits of the datagram
   */

  memcpy((void *) &(mesg[28]),(void *) iph,iph->ihl * 4 + 8); 

  /*
   * Compute the ICMP header checksum
   */

  oldICMPHeaderChecksum((ZkICMPHeader *)&(mesg[20]),iph->ihl * 4 + 16);

  /*
   * Compute the IP header checksum 
   */

  oldIPHeaderChecksum(header);

  /*
   * All done.  the buffer can be dropped onto the network (with a raw socket)
   */

  return 1;
}


/* Original IP functions */

/**
 *  Compute a ZkIPHeader checksum.
 *
 *
 */

uint16_t oldIPHeaderChecksum (
  ZkIPHeader * iph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  size_t datalen = 0;
  size_t i = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!iph) {
    return 0;
  }
  iph->HeaderChkSum = 0;
  data = (unsigned short *) iph;
  datalen = iph->ihl * 2;
  cksum = 0;
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }
  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  iph->HeaderChkSum = (uint16_t) ~cksum;

  return iph->HeaderChkSum;
}

static char *oldIPAddrToStr (
  uint32_t addr )
{
  unsigned char * ptr = NULL;
  static char buffer[32];

  ptr = (unsigned char *) &addr;
  zkSprintf(buffer,"%d.%d.%d.%d",ptr[0],ptr[1],ptr[2],ptr[3]);
  return buffer;
}
  
int oldIPHeaderDump (
  ZkIPHeader * iph ,
  FILE * stream )
{

  zkFprintf(stream,"IP Packet Header pointer %p\n", (void *)iph);
  if (iph == NULL) {
    return 0;
  }
  zkFprintf(stream,"  IP Packet Length        %d bytes\n",iph->ihl * 4);
  zkFprintf(stream,"  Version                 %d\n",iph->version);
  zkFprintf(stream,"  Type Of Service         %d\n",iph->TypeOfSvr);
  zkFprintf(stream,"  Total Length            %d\n",ntohs(iph->TotalLength));
  zkFprintf(stream,"  Identification          %d\n",ntohs(iph->Identification));
  zkFprintf(stream,"  Fragmentation Flags     0x%X\n",
          (ntohs(iph->Flags_FrameOffs)&0xE000)>>13);
  zkFprintf(stream,"  Fragmentation Offset    %d\n",
          ntohs(iph->Flags_FrameOffs)&0x1FFF);
  zkFprintf(stream,"  Time To Live            %d\n",iph->TimeToLive);
  zkFprintf(stream,"  Protocol                %d\n",iph->Protocol);
  zkFprintf(stream,"  Header Checksum         %d\n",iph->HeaderChkSum);
  zkFprintf(stream,"  Source IP Address       %s\n",
          oldIPAddrToStr(iph->SrcIpAddress));
  zkFprintf(stream,"  Destination IP Address  %s\n",
          oldIPAddrToStr(iph->DstIpAddress));

  return 1;
}

int oldIPHeaderSourceAddress (
  ZkIPHeader * iph ,
  uint32_t * addr )
{
  *addr = iph->SrcIpAddress;
  return 1;
}

int oldIPHeaderDestinationAddress (
  ZkIPHeader * iph ,
  uint32_t * addr )
{
  *addr = iph->DstIpAddress;
  return 1;
}

int oldIPHeaderSetSourceAddress (
  ZkIPHeader * iph ,
  uint32_t addr )
{
  iph->SrcIpAddress = addr;
  return 1;
}
 
int oldIPHeaderSetDestinationAddress (
  ZkIPHeader * iph ,
  uint32_t addr )
{
  iph->DstIpAddress = addr;
  return 1;
}

/**
 *  Strip out all of the IP packet options.  This is fairly draconian but
 *  what we're doing either makes the IP options useless (in the case of
 *  source routing) or dangerous (in the case of route record).
 *  This function assumes that the data immediatly follows the IP header in
 *  memory.  You must recompute the IP checksum after striping the options 
 *  (this function will not recompute it for you).
 *  Note:  In order to avoid breaking things, I am copying the data portion
 *         of the packet overtop the IP options.  It may be more efficient to
 *         go the otherway, ie copy the header data forward to overwrite the
 *         options, however, this then makes the IP header pointer move.
 *  Note:  We may want to keep one option, StreamID, if it is preserved
 *         both ways, we might be able to use it to keep track of ACI <->
 *         ICMP message going through the wormhole.
 *
 *
 *  @param iph  The IP packet header pointer
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

int oldIPHeaderStripOptions (
  ZkIPHeader * iph )
{
  int i = 0;
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (iph->ihl == 5) {
    return 1;
  } else if (iph->ihl < 5) {
    return 0;        /* Cant have a proper IP packet with length  < 5 */
  }
  cat = &(((char *)iph)[20]);
  mouse = &(((char *)iph)[iph->ihl * 4]);
  len = (int) ntohs(iph->TotalLength) - (iph->ihl * 4);
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++; 
  }
  iph->TotalLength = htons((short) (ntohs(iph->TotalLength) - (iph->ihl * 4 - 20)));
  iph->ihl = 5;
  iph->HeaderChkSum = 0;
  return 1;
}


/* original TCP functions*/

int oldTCPHeaderDump (
  ZkTCPHeader * tcph ,
  FILE * stream )
{
  char yn[2] = { ' ', 'X' };

  (void) module_id_; /* Get rid of compile warning */
  zkFprintf(stream,"TCP Packet Header Pointer %p\n",(void *)tcph);
  if (tcph == NULL) {
    return 0;
  }
  zkFprintf(stream,"  Source Port             %d\n",ntohs(tcph->SrcPort));
  zkFprintf(stream,"  Destination Port        %d\n",ntohs(tcph->DstPort));
  zkFprintf(stream,"  Sequence Number         %lu\n",(unsigned long)ntohl(tcph->SequenceNumber));
  zkFprintf(stream,"  Acknowledgement number  %lu\n",(unsigned long)ntohl(tcph->AckNumber));
  zkFprintf(stream,"  Reserved1               %d\n",tcph->res1);
  zkFprintf(stream,"  Header Length           %d\n",tcph->doff * 4);
  zkFprintf(stream,"  FIN flag                %c\n",yn[tcph->fin]);
  zkFprintf(stream,"  SYN flag                %c\n",yn[tcph->syn]);
  zkFprintf(stream,"  RST flag                %c\n",yn[tcph->rst]);
  zkFprintf(stream,"  PSH flag                %c\n",yn[tcph->psh]);
  zkFprintf(stream,"  ACK flag                %c\n",yn[tcph->ack]);
  zkFprintf(stream,"  URG flag                %c\n",yn[tcph->urg]);
  zkFprintf(stream,"  Reserved2               %d\n",tcph->res2);
  zkFprintf(stream,"  Window                  %d\n",htons(tcph->Window));
  zkFprintf(stream,"  Header Checksum         %d\n",tcph->TcpCheckSum);
  zkFprintf(stream,"  Urgent Pointer          %d\n",ntohs(tcph->UrgentPointer));
  return 1;
}

int oldTCPHeaderAcknowledgement (
  ZkTCPHeader * tcph ,
  uint32_t * ack )
{
  if (tcph && ack) {
    *ack = ntohl(tcph->AckNumber);
    return 1;
  } 
  return 0;
} 

int oldTCPHeaderSequence (
  ZkTCPHeader * tcph ,
  uint32_t * seq )
{
  if (tcph && seq) {
    *seq = ntohl(tcph->SequenceNumber);
    return 1;
  }
  return 0;
}

uint16_t oldTCPHeaderSourcePort (
  ZkTCPHeader * tcph ,
  uint16_t * port ) 
{
  *port = ntohs(tcph->SrcPort);
  return *port ;
}

uint16_t oldTCPHeaderDestinationPort (
  ZkTCPHeader * tcph ,
  uint16_t * port )
{
  *port = ntohs(tcph->DstPort);
  return *port;
}

int oldTCPHeaderSetSourcePort (
  ZkTCPHeader * tcph ,
  uint16_t port )
{  
  tcph->SrcPort = htons(port);
  return 1;
}

int oldTCPHeaderSetDestinationPort (
  ZkTCPHeader * tcph ,
  uint16_t port )
{
  tcph->DstPort = htons(port);
  return 1;
}

/**
 *  Set the maximum segment size option on the TCP packet to the specified
 *  value.  This function will move the packet data payload forward 4 bytes
 *  in order to accomodate this TCP option.  The header lengths will be 
 *  updated accordingly but the header checksums will not.  You must be
 *  sure to recompute them yourself.
 * 
 *  @param tcph  A pointer to a TCP header structure
 *  @param mss   The new Maximum Segment Size option
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure
 */

int oldTCPHeaderSetMSS (
  ZkTCPHeader * tcph ,
  uint16_t mss )
{
  int i = 0; /* Loop index for moving packet payload */
  int len = 0;  /* Number of bytes to move */
  int hlen = 0;  /* Length of combined headers */
  char * cat = NULL;
  char * mouse = NULL;
  unsigned char *mss_ptr = NULL;
  
  /*
   * Move the payload forward by 4 bytes
   */

  cat = &(((char *)tcph)[ntohs(tcph->IP.TotalLength) - 1]);
  mouse = &(((char *)tcph)[ntohs(tcph->IP.TotalLength) - 5]);
  hlen = (tcph->IP.ihl + tcph->doff) * 4;
  len = ntohs(tcph->IP.TotalLength) - hlen;
  for (i = 0; i < len; i++) {
    *cat-- = *mouse--;
  }
  
  /*
   * Append the MSS to the end of the option list
   */

  mss_ptr = (unsigned char *) &(((char *)tcph)[hlen]);
  mss_ptr[0] = 0x02; /* Set the MSS option code, 2 */
  mss_ptr[1] = 0x04; /* Set the MSS length, 4 */
  mss_ptr[2] = (unsigned char) (mss / 0x100);
  mss_ptr[3] = (unsigned char) (mss % 0x100);
#if 0
  mss_ptr[0] = 0x0402;   /* Set the MSS option code, kind 2, length 4 */
  mss_ptr[1] = htons(mss);  /* Set the new MSS value */
#endif

  /*
   * Update header lengths
   */

  tcph->doff += 1;
  tcph->IP.TotalLength = htons((short) (ntohs(tcph->IP.TotalLength) + 4));
  tcph->TcpCheckSum = 0;
  tcph->IP.HeaderChkSum = 0;
  return 1;
}

/**
 *  This function will remove all of the TCP options from the TCP header.  It
 *  assumes that the accompanying packet data follows immediatly after in 
 *  memory and that the tcp header also has a proper IP header immediatly
 *  before.
 *
 *  @param tcph The TCP header to strip of options
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value on success, a 0 on failure.
 */

int oldTCPHeaderStripOptions (
  ZkTCPHeader * tcph )
{
  int i = 0; /* Loop index */
  int len = 0;
  char * cat = NULL;
  char * mouse = NULL;

  if (tcph->doff == 5) {
    return 1;
  } else if (tcph->doff < 5) {
    return 0;
  }
  cat = &(((char *)tcph)[tcph->IP.ihl * 4 + 20]);
  mouse = &(((char *)tcph)[(tcph->IP.ihl + tcph->doff) * 4]);
  len = ntohs(tcph->IP.TotalLength) - (tcph->IP.ihl + tcph->doff) * 4;
  for (i = 0; i < len; i++) {
    *cat++ = *mouse++;
  }
  tcph->IP.TotalLength = htons((short) (ntohs(tcph->IP.TotalLength)
                               - (tcph->doff * 4 - 20) ));
  tcph->doff = 5;
  tcph->TcpCheckSum = 0;
  tcph->IP.HeaderChkSum = 0;
  return 1;
}

/**
 *  Analyze a TCP packet and return the MSS option.  If this option cannot
 *  be found, the function fails and returns a 0.
 *
 *  @param tcph The TCP header structure to analyze.  It is assumed that the
 *              packet data follows immediatly after in memory.
 *  @param mss  Returns the MSS (Maximum Segment Size for the connection)
 *  @author Philippe Boucher
 *  @version 1.0
 *  @return A non-zero value if the MSS option was found, 0 otherwise.
 */

int oldTCPHeaderGetMSS (
  ZkTCPHeader * tcph ,
  uint16_t * mss )
{
  unsigned char * ptr = NULL;
  int len = 0;
  int i = 0; /* Loop index for scanning option data */

  if (tcph->doff <= 5) {
    return 0;      /* No options means no MSS specified */
  }
  len = tcph->doff * 4 - 20;  /* Number of bytes devoted to options */
  ptr = (unsigned char *) &(tcph[1]);
  for (i = 0; i < len; i++,ptr++) {
    switch (*ptr) {
    case 0:  /* End of option list, terminate... */
      i = len;
      break;
    case 1: /* Nop, skip it */
      break;
    case 2: /* This is the MSS option */
      i++;
      if (i >= len) {
        break;  /* Do we have enough data left in the packet for an MSS ? */
      } 
      ptr++;
      if (*ptr != 4) {   /* Is the length correct */
        i = len;
        break;
      }
      ptr++;
#if 0
      *mss = (uint16_t) ntohs(*((uint16_t *)ptr));  /* Get the MSS */
#else
      *mss = (uint16_t) ptr[0] * 0x100 + ptr[1];
#endif
      return 1;
      break;
    default:
      i++; /* Skip the length byte */
      ptr++;
      break;
    }
  }
  return 0;
}  

uint16_t oldTCPHeaderChecksum (
  ZkTCPHeader * tcph )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  ZkIPHeader * iph = NULL;
  size_t i = 0;
  size_t datalen = 0;

  (void) module_id_; /* Get rid of compile warning */
  if (!tcph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = &(tcph->IP);
  tcph->TcpCheckSum = 0;

  datalen = ntohs(iph->TotalLength) - (iph->ihl * 4);
  data = (unsigned short *) &(iph->SrcIpAddress);
  cksum = data[0] + data[1];
  data = (unsigned short *) &(iph->DstIpAddress);
  cksum += data[0] + data[1];
  cksum += htons(6) + 
           htons((uint16_t) (ntohs(iph->TotalLength) - (iph->ihl * 4)));
  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)tcph)[datalen + (iph->ihl * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) &(iph[1]);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  } 

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  tcph->TcpCheckSum = (uint16_t) ~cksum;

  zkIPHeaderChecksum(iph);

  return tcph->TcpCheckSum;
}

/* Original UDP functions*/

int oldUDPHeaderDump (
  ZkUDPHeader * udph ,
  FILE * stream )
{
  (void) module_id_; /* Get rid of compile time warning */
  zkFprintf(stream,"UDP Packet Header Pointer %p\n", (void *)udph);
  if (udph == NULL) {
    return 0;
  }
  zkFprintf(stream,"  Source port             %d\n",ntohs(udph->SrcPort));
  zkFprintf(stream,"  Destination Port        %d\n",ntohs(udph->DstPort));
  zkFprintf(stream,"  Length                  %d\n",ntohs(udph->UdpLength));
  zkFprintf(stream,"  Header Checksum         %d\n",ntohs(udph->UdpCheckSum));
  return 1;
}

uint16_t oldUDPHeaderSourcePort (
  ZkUDPHeader * udph ,
  uint16_t * port ) 
{
  *port = ntohs(udph->SrcPort);
  return *port ;
}

uint16_t oldUDPHeaderDestinationPort (
  ZkUDPHeader * udph ,
  uint16_t * port )
{
  *port = ntohs(udph->DstPort);
  return *port;
}

int oldUDPHeaderSetSourcePort (
  ZkUDPHeader * udph ,
  uint16_t port )
{  
  udph->SrcPort = htons(port);
  return 1;
}

int oldUDPHeaderSetDestinationPort (
  ZkUDPHeader * udph ,
  uint16_t port )
{
  udph->DstPort = htons(port);
  return 1;
}

/**
 *  Compute a ZkUDPIPHeader checksum.
 *
 */

uint16_t oldUDPHeaderChecksum (
  ZkUDPHeader * udph ,
  size_t datalen )
{
  unsigned long cksum = 0;
  unsigned short * data = NULL;
  ZkIPHeader * iph = NULL;
  size_t i = 0;

  if (!udph) {
    return 0;
  }

  /*
   * First we compute a checksum on the TCP pseudo header (this consists of
   * the source and destination IP addresses, the lengths listed in the IP
   * packet and the IP protocol code (6).
   */

  iph = &(udph->IP);
  udph->UdpCheckSum = 0;

  datalen = ntohs(iph->TotalLength) - (iph->ihl * 4);
  data = (unsigned short *) &(iph->SrcIpAddress);
  cksum = data[0] + data[1];
  data = (unsigned short *) &(iph->DstIpAddress);
  cksum += data[0] + data[1];
  cksum += htons(ZK_IPPROTO_UDP) + 
           htons((uint16_t) (ntohs(iph->TotalLength) - (iph->ihl * 4)));

  /*
   * Possibly pad one extra byte at the end of the data so that we can
   * properly convert the data to an array of unsigned shorts.
   */

  if (datalen % 2) {
    ((char *)udph)[datalen + (iph->ihl * 4)] = (char) 0;
    datalen += 1;
  }
  datalen >>= 1;

  /*
   * Compute the checksum
   */

  data = (unsigned short *) &(iph[1]);
  for (i = 0; i < datalen; i++) {
    cksum += *data++;
  }

  /*
   * Handle the overflow
   */

  cksum = (cksum >> 16) + (cksum & 0xFFFF);
  cksum += (cksum >> 16);
  udph->UdpCheckSum = (uint16_t) ~cksum;

  zkIPHeaderChecksum(iph);

  return udph->UdpCheckSum;
}


static void		test_icmp(void);
static void		test_ip(void);
static void		test_tcp(void);
static void		test_udp(void);

static void		testICMPCreateDestinationUnreachable(void);
static void		testICMPHeaderChecksum(void);
static void		testICMPCreateFragmentationNeeded(void);
static void		testICMPCreateTimeExceeded(void);
static void		testICMPCreateSourceQuench(void);

static void		testIPHeaderDump(void);
static void		testIPHeaderSourceAddress(void);
static void		testIPHeaderDestinationAddress(void);
static void		testIPHeaderSetSourceAddress(void);
static void		testIPHeaderSetDestinationAddress(void);
static void		testIPHeaderStripOptions(void);
static void		testIPHeaderChecksum(void);

static void		testTCPHeaderDump(void);
static void		testTCPHeaderAcknowledgement(void);
static void		testTCPHeaderSequence(void);
static void		testTCPHeaderSourcePort(void);
static void		testTCPHeaderDestinationPort(void);
static void		testTCPHeaderSetSourcePort(void);
static void		testTCPHeaderSetDestinationPort(void);
static void		testTCPHeaderGetMSS(void);
static void		testTCPHeaderSetMSS(void);
static void		testTCPHeaderStripOptions(void);
static void		testTCPHeaderChecksum(void);

static void		testUDPHeaderDump(void);
static void		testUDPHeaderSourcePort(void);
static void		testUDPHeaderDestinationPort(void);
static void		testUDPHeaderSetSourcePort(void);
static void		testUDPHeaderSetDestinationPort(void);
static void		testUDPHeaderChecksum(void);

static void		fill_icmp(ZkICMPHeader* strc, void* buff);
static void		fill_ip(ZkIPHeader* strc, void* buff);
static void		fill_tcp(ZkTCPHeader* strc, void* buff);
static void		fill_udp(ZkUDPHeader* strc, void* buff);

static void fill_ipOptions( void *oldStyle, void *newStyle);
static void fill_tcpOptions(void *oldStyle, void *newStyle);

static void fill_ipData( void *oldStyle, void *newStyle);

#if 0	/* currently unused */
static int		cmp_icmp(ZkICMPHeader* strc, void* buff); /*compare old-style struct and */ 
#endif
static int		cmp_ip(ZkIPHeader* strc, void* buff);     /*new-style buffer field to field*/
static int		cmp_tcp(ZkTCPHeader* strc, void* buff);   /*returns 1 if equal*/
#if 0	/* currently unused */
static int		cmp_udp(ZkUDPHeader* strc, void* buff);
#endif

static uint8_t rand8(void);
static uint16_t rand16(void);
static uint32_t rand32(void);
#if 0	/* currently unused */
static uint64_t rand64(void);
#endif


int main(void)
{
	#	define NUM_ITERS	5
	int i=0;

	srand(clock());

	testIPHeaderDump();
	testTCPHeaderDump();
	testUDPHeaderDump();
		
	for(i=0; i<NUM_ITERS ;i++)
	{
	
		test_icmp();
		test_ip();
		test_tcp();
		test_udp();

		testICMPCreateDestinationUnreachable();
		testICMPHeaderChecksum();
		testICMPCreateFragmentationNeeded();
		testICMPCreateTimeExceeded();
		testICMPCreateSourceQuench();

		testIPHeaderSourceAddress();
		testIPHeaderDestinationAddress();
		testIPHeaderSetSourceAddress();
		testIPHeaderSetDestinationAddress();
		testIPHeaderStripOptions();
		testIPHeaderChecksum();

		testTCPHeaderAcknowledgement();
		testTCPHeaderSequence();
		testTCPHeaderSourcePort();
		testTCPHeaderDestinationPort();
		testTCPHeaderSetSourcePort();
		testTCPHeaderSetDestinationPort();
		testTCPHeaderGetMSS();
		testTCPHeaderSetMSS(); 
		testTCPHeaderStripOptions();
		testTCPHeaderChecksum(); 

		testUDPHeaderSourcePort();
		testUDPHeaderDestinationPort();
		testUDPHeaderSetSourcePort();
		testUDPHeaderSetDestinationPort();
		testUDPHeaderChecksum(); 
	}

	return (EXIT_SUCCESS);
}

static void		test_icmp(void)
{
	#	define NUM_ICMP_ITERS	100

	/* new-style buffer declaration */
	unsigned char newStyle[ZKICMPHEADER_SIZEOF];
	/* old-style struct declaration */
	ZkICMPHeader oldStyle;
	int i;
	uint8_t			type, code;
	uint16_t		checksum, identifier, sequencenumber;

	for(i=0; i<NUM_ICMP_ITERS; i++)
	{
		/* zero-fill both structures */
		memset(newStyle, 0, sizeof(newStyle));
		memset(&oldStyle, 0, sizeof(oldStyle));

		type = rand8();
		code = rand8();
		checksum = rand16();
		identifier = rand16();
		sequencenumber = rand16();

		/* test the set & get functions */
		zkICMPHeaderSetType(newStyle, type);
		zkICMPHeaderSetCode(newStyle, code);
		zkICMPHeaderSetChecksum(newStyle, checksum);
		zkICMPHeaderSetIdentifier(newStyle, identifier);
		zkICMPHeaderSetSequenceNumber(newStyle, sequencenumber);
		oldStyle.Type			= type;
		oldStyle.code			= code;
		oldStyle.CheckSum		= htons(checksum);
		oldStyle.Identifier		= htons(identifier);
		oldStyle.SequenceNumber	= htons(sequencenumber);
	
		if (memcmp(&oldStyle, newStyle, sizeof(oldStyle)))
		{
			zkPrintf("### ICMP Failure: zkICMPHeaderSetXXX !\n");
		}

		if((zkICMPHeaderGetType(newStyle) != oldStyle.Type) ||
		   (zkICMPHeaderGetCode(newStyle) != oldStyle.code) ||
	  	   (zkICMPHeaderGetChecksum(newStyle) != ntohs(oldStyle.CheckSum)) ||
		   (zkICMPHeaderGetIdentifier(newStyle) != ntohs(oldStyle.Identifier)) ||
		   (zkICMPHeaderGetSequenceNumber(newStyle) != ntohs(oldStyle.SequenceNumber)))
		{
			zkPrintf("### ICMP Failure: zkICMPHeaderGetXXX !\n");
		}
	}
	return;
}

static void		test_ip(void)
{
	#	define NUM_IP_ITERS	100

	/* new-style buffer declaration */
	unsigned char newStyle[ZKIPHEADER_SIZEOF];
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	int i;
	uint8_t		ihl, version, typeofsvr, timetolive, protocol;
	uint16_t	totallength, identification, flags_frameoffs, headerchecksum;
	uint32_t	srcipaddress, dstipaddress;

	for(i=0; i<NUM_IP_ITERS; i++)
	{
		/* zero-fill both structures */
		memset(newStyle, 0, sizeof(newStyle));
		memset(&oldStyle, 0, sizeof(oldStyle));

		ihl = rand8();
		version = rand8();
		typeofsvr = rand8();
		timetolive = rand8();
		protocol = rand8();
		totallength = rand16();
		identification = rand16();
		flags_frameoffs = rand16();
		headerchecksum = rand16();
		srcipaddress = rand32();
		dstipaddress = rand32();

		/* test the set & get functions */
		zkIPHeaderSetIHL(newStyle, ihl);
		zkIPHeaderSetVersion(newStyle, version);
		zkIPHeaderSetTypeOfSvr(newStyle, typeofsvr);
		zkIPHeaderSetTimeToLive(newStyle, timetolive);
		zkIPHeaderSetProtocol(newStyle, protocol);
		zkIPHeaderSetTotalLength(newStyle, totallength);
		zkIPHeaderSetIdentification(newStyle, identification);
		zkIPHeaderSetFlags_FrameOffs(newStyle, flags_frameoffs);
		zkIPHeaderSetHeaderChkSum(newStyle, headerchecksum);
		zkIPHeaderSetSrcIpAddress(newStyle, srcipaddress);
		zkIPHeaderSetDstIpAddress(newStyle, dstipaddress);

		oldStyle.ihl			= ihl;
		oldStyle.version		= version;
		oldStyle.TypeOfSvr		= typeofsvr;
		oldStyle.TimeToLive		= timetolive;
		oldStyle.Protocol		= protocol;
		oldStyle.TotalLength	= htons(totallength);
		oldStyle.Identification = htons(identification);
		oldStyle.Flags_FrameOffs= htons(flags_frameoffs);
		oldStyle.HeaderChkSum	= htons(headerchecksum);
		oldStyle.SrcIpAddress	= htonl(srcipaddress);
		oldStyle.DstIpAddress	= htonl(dstipaddress);
	
		if (memcmp(&oldStyle, newStyle, sizeof(oldStyle)))
		{
			zkPrintf("### IP Failure: zkIPHeaderSetXXX !\n");
		}

		if((zkIPHeaderGetIHL(newStyle) != oldStyle.ihl) ||
		(zkIPHeaderGetVersion(newStyle) != oldStyle.version) ||
		(zkIPHeaderGetTypeOfSvr(newStyle) != oldStyle.TypeOfSvr) ||
		(zkIPHeaderGetTimeToLive(newStyle) != oldStyle.TimeToLive) ||
		(zkIPHeaderGetProtocol(newStyle) != oldStyle.Protocol) ||
		(zkIPHeaderGetTotalLength(newStyle) != ntohs(oldStyle.TotalLength)) ||
		(zkIPHeaderGetIdentification(newStyle) != ntohs(oldStyle.Identification)) ||
		(zkIPHeaderGetFlags_FrameOffs(newStyle) != ntohs(oldStyle.Flags_FrameOffs)) ||
		(zkIPHeaderGetHeaderChkSum(newStyle) != ntohs(oldStyle.HeaderChkSum)) ||
		(zkIPHeaderGetSrcIpAddress(newStyle) != ntohl(oldStyle.SrcIpAddress)) ||
		(zkIPHeaderGetDstIpAddress(newStyle) != ntohl(oldStyle.DstIpAddress)))
		{
			zkPrintf("### IP Failure: zkIPHeaderGetXXX !\n");
		}
	}
	return;
}

static void		test_tcp(void)
{
	#	define NUM_TCP_ITERS	100

	/* new-style buffer declaration */
	unsigned char newStyle[ZKTCPHEADER_SIZEOF];
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	int i;
	uint8_t		doff, res, urg, ack, psh, rst, syn, fin;
	uint16_t	srcport, dstport, window, tcpchecksum, urgentpointer;
	uint32_t	sequencenumber, acknumber;


	for(i=0; i<NUM_TCP_ITERS; i++)
	{
		/* zero-fill both structures */
		memset(newStyle, 0, sizeof(newStyle));
		memset(&oldStyle, 0, sizeof(oldStyle));

		doff = rand8() & 0x0F;
		res = rand8() & 0x3F;
		urg = rand8() & 0x01;
		ack = rand8() & 0x01;
		psh = rand8() & 0x01;
		rst = rand8() & 0x01;
		syn = rand8() & 0x01;
		fin = rand8() & 0x01;
		srcport = rand16();
		dstport = rand16();
		window = rand16();
		tcpchecksum = rand16();
		urgentpointer = rand16();
		sequencenumber = rand32();
		acknumber = rand32();

		/* test the set & get functions */
		zkTCPHeaderSetDoff(newStyle, doff);
		zkTCPHeaderSetRes(newStyle, res);
		zkTCPHeaderSetUrg(newStyle, urg);
		zkTCPHeaderSetAck(newStyle, ack);
		zkTCPHeaderSetPsh(newStyle, psh);
		zkTCPHeaderSetRst(newStyle, rst);
		zkTCPHeaderSetSyn(newStyle, syn);
		zkTCPHeaderSetFin(newStyle, fin);
		zkTCPHeaderSetSrcPort(newStyle, srcport);
		zkTCPHeaderSetDstPort(newStyle, dstport);
		zkTCPHeaderSetWindow(newStyle, window);
		zkTCPHeaderSetTcpCheckSum(newStyle, tcpchecksum);
		zkTCPHeaderSetUrgentPointer(newStyle, urgentpointer);
		zkTCPHeaderSetSequenceNumber(newStyle, sequencenumber);
		zkTCPHeaderSetAckNumber(newStyle, acknumber);
	
		oldStyle.doff	= doff;
		oldStyle.res1	= (res >> 2);  /*res1 is a 4 bits bitfield*/
		oldStyle.res2	= res; /*res2 is a 2-bits bitfield*/
		oldStyle.urg	= urg;
		oldStyle.ack	= ack;
		oldStyle.psh	= psh;
		oldStyle.rst	= rst;
		oldStyle.syn	= syn;
		oldStyle.fin	= fin;
		oldStyle.SrcPort = htons(srcport);
		oldStyle.DstPort = htons(dstport);
		oldStyle.Window = htons(window);
		oldStyle.TcpCheckSum = htons(tcpchecksum);
		oldStyle.UrgentPointer = htons(urgentpointer);
		oldStyle.SequenceNumber = htonl(sequencenumber);
		oldStyle.AckNumber = htonl(acknumber);

		if (memcmp(&oldStyle, newStyle, sizeof(oldStyle)))
		{
			zkPrintf("### TCP Failure: zkTCPHeaderSetXXX !\n");
		}

		if((zkTCPHeaderGetDoff(newStyle) != oldStyle.doff) ||
		(zkTCPHeaderGetRes(newStyle) != (uint32_t)((oldStyle.res1<<2)|(oldStyle.res2))) ||  
		(zkTCPHeaderGetUrg(newStyle) != oldStyle.urg) ||
		(zkTCPHeaderGetAck(newStyle) != oldStyle.ack) ||
		(zkTCPHeaderGetPsh(newStyle) != oldStyle.psh) ||
		(zkTCPHeaderGetRst(newStyle) != oldStyle.rst) ||
		(zkTCPHeaderGetSyn(newStyle) != oldStyle.syn) ||
		(zkTCPHeaderGetFin(newStyle) != oldStyle.fin) ||
		(zkTCPHeaderGetSrcPort(newStyle) != ntohs(oldStyle.SrcPort)) ||
		(zkTCPHeaderGetDstPort(newStyle) != ntohs(oldStyle.DstPort)) ||
		(zkTCPHeaderGetWindow(newStyle) != ntohs(oldStyle.Window)) ||
		(zkTCPHeaderGetTcpCheckSum(newStyle) != ntohs(oldStyle.TcpCheckSum)) ||
		(zkTCPHeaderGetUrgentPointer(newStyle) != ntohs(oldStyle.UrgentPointer)) ||
		(zkTCPHeaderGetSequenceNumber(newStyle) != ntohl(oldStyle.SequenceNumber)) ||
		(zkTCPHeaderGetAckNumber(newStyle) != ntohl(oldStyle.AckNumber))) 
		{
			zkPrintf("### TCP Failure: zkTCPHeaderGetXXX !\n");
		}
	}
	return;
}

static void		test_udp(void)
{
	#	define NUM_UDP_ITERS	100

	/* new-style buffer declaration */
	unsigned char newStyle[ZKUDPHEADER_SIZEOF];
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	int i;
	uint16_t	srcport, dstport, udplength, udpchecksum;

	for(i=0; i<NUM_UDP_ITERS; i++)
	{
		/* zero-fill both structures */
		memset(newStyle, 0, sizeof(newStyle));
		memset(&oldStyle, 0, sizeof(oldStyle));

		srcport = rand16();
		dstport = rand16();
		udplength = rand16();
		udpchecksum = rand16();

		/* test the set & get functions */
		zkUDPHeaderSetSrcPort(newStyle, srcport);
		zkUDPHeaderSetDstPort(newStyle, dstport);
		zkUDPHeaderSetUdpLength(newStyle, udplength);
		zkUDPHeaderSetUdpCheckSum(newStyle, udpchecksum);

		oldStyle.SrcPort	= htons(srcport);
		oldStyle.DstPort	= htons(dstport);
		oldStyle.UdpLength	= htons(udplength);
		oldStyle.UdpCheckSum= htons(udpchecksum);

		if (memcmp(&oldStyle, newStyle, sizeof(oldStyle)))
		{
			zkPrintf("### UDP Failure: zkUDPHeaderSetXXX !\n");
		}

		if((zkUDPHeaderGetSrcPort(newStyle) != ntohs(oldStyle.SrcPort)) ||
		(zkUDPHeaderGetDstPort(newStyle) != ntohs(oldStyle.DstPort)) ||
		(zkUDPHeaderGetUdpLength(newStyle) != ntohs(oldStyle.UdpLength)) ||
		(zkUDPHeaderGetUdpCheckSum(newStyle) != ntohs(oldStyle.UdpCheckSum)))
		{
			zkPrintf("### UDP Failure: zkUDPHeaderGetXXX !\n");
		}
	}
	return;
}

static void	testICMPCreateDestinationUnreachable(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF + DATA_SIZE];
	int newRet;
	unsigned char *newMesg;
	size_t newSize;
	
	/* old-style struct declaration */
	ZkIPHeaderAndData oldStyle;
	int oldRet;
	unsigned char *oldMesg;
	size_t oldSize;

	/* common parameters */
	uint8_t code = rand8() & 0x0F;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.header, newStyle);
	
	/* set ihl with a valid value => 5 */
	zkIPHeaderSetIHL(newStyle, 5);
	oldStyle.header.ihl = 5;
	zkIPHeaderSetTotalLength(newStyle, 20);
	oldStyle.header.TotalLength = htons(20);

	/* fill ip data with the same values */
	fill_ipData(&oldStyle.data, &newStyle[zkIPHeaderGetIHL(newStyle)*4]);

	newMesg = malloc(zkIPHeaderGetIHL(newStyle)*4 + 36);
	newSize = oldStyle.header.ihl*4 + 36;
	oldMesg = malloc(oldStyle.header.ihl*4 + 36);
	oldSize = oldStyle.header.ihl*4 + 36;

	if(oldSize != newSize)
		zkPrintf("### ICMP Failure: zkICMPCreateDestinationUnreachable() !\n");

	/* test zkICMPCreateDestinationUnreachable */
	newRet = zkICMPCreateDestinationUnreachable ( newStyle , code , newMesg , &newSize); 
	oldRet = oldICMPCreateDestinationUnreachable( &oldStyle.header, code , oldMesg , &oldSize);

	/* compare mesg */
	if(oldSize != newSize || memcmp(oldMesg, newMesg, oldSize))
		zkPrintf("### ICMP Failure: zkICMPCreateDestinationUnreachable() !\n");

	free(newMesg);
	free(oldMesg);
	return;
}

static void		testICMPHeaderChecksum(void)
{
	/* new-style buffer */
	char newStyle[ZKICMPHEADER_SIZEOF];
	uint16_t newRet;
	
	/* old-style struct declaration */
	ZkICMPHeader oldStyle;
	uint16_t oldRet;

	/* fill the structs with the same values...*/
	fill_icmp(&oldStyle, newStyle);

	/* test zkICMPHeaderChecksum */
	newRet = zkICMPHeaderChecksum ( newStyle , ZKICMPHEADER_SIZEOF); 
	oldRet = oldICMPHeaderChecksum( &oldStyle, ZKICMPHEADER_SIZEOF);

	/*compare the retured values, it's the checksum*/
	if(newRet != oldRet)
		zkPrintf("### Failure: zkICMPHeaderChecksum() !\n");
	return;
}

static void		testICMPCreateFragmentationNeeded(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF + DATA_SIZE];
	int newRet;
	unsigned char *newMesg;
	size_t newSize;
	
	/* old-style struct declaration */
	ZkIPHeaderAndData oldStyle;
	int oldRet;
	unsigned char *oldMesg;
	size_t oldSize;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.header, newStyle);

	/* set ihl with a valid value => 5 */
	zkIPHeaderSetIHL(newStyle, 5);
	oldStyle.header.ihl = 5;
	zkIPHeaderSetTotalLength(newStyle, 20);
	oldStyle.header.TotalLength = htons(20);

	/* fill ip data with the same values */
	fill_ipData(&oldStyle.data, &newStyle[zkIPHeaderGetIHL(newStyle)*4]);

	newMesg = malloc(zkIPHeaderGetIHL(newStyle)*4 + 36);
	newSize = oldStyle.header.ihl*4 + 36;
	oldMesg = malloc(oldStyle.header.ihl*4 + 36);
	oldSize = oldStyle.header.ihl*4 + 36;


	if(oldSize != newSize)
		zkPrintf("### ICMP Failure: zkICMPCreateFragmentationNeeded() !\n");

	/* test zkICMPCreateFragmentationNeeded */
	newRet = zkICMPCreateFragmentationNeeded ( newStyle , newMesg, &newSize); 
	oldRet = oldICMPCreateFragmentationNeeded( &oldStyle.header, oldMesg, &oldSize);

	/* compare mesg */
	if(oldSize != newSize || memcmp(oldMesg, newMesg, oldSize))
		zkPrintf("### ICMP Failure: zkICMPCreateFragmentationNeeded() !\n");
	return;
}

static void		testICMPCreateTimeExceeded(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF + DATA_SIZE];
	int newRet;
	unsigned char *newMesg;
	size_t newSize;
	
	/* old-style struct declaration */
	ZkIPHeaderAndData oldStyle;
	int oldRet;
	unsigned char *oldMesg;
	size_t oldSize;

	/* common parameters */
	uint8_t code = rand8() & 0x01;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.header, newStyle);

	/* set ihl with a valid value => 5 */
	zkIPHeaderSetIHL(newStyle, 5);
	oldStyle.header.ihl = 5;
	zkIPHeaderSetTotalLength(newStyle, 20);
	oldStyle.header.TotalLength = htons(20);

	/* fill ip data with the same values */
	fill_ipData(&oldStyle.data, &newStyle[zkIPHeaderGetIHL(newStyle)*4]);
	
	newMesg = malloc(zkIPHeaderGetIHL(newStyle)*4 + 36);
	newSize = oldStyle.header.ihl*4 + 36;
	oldMesg = malloc(oldStyle.header.ihl*4 + 36);
	oldSize = oldStyle.header.ihl*4 + 36;

	if(oldSize != newSize)
		zkPrintf("### ICMP Failure: zkICMPCreateTimeExceeded() !\n");

	/* test zkICMPCreateTimeExceeded */
	newRet = zkICMPCreateTimeExceeded ( newStyle , code, newMesg, &newSize); 
	oldRet = oldICMPCreateTimeExceeded( &oldStyle.header, code, oldMesg, &oldSize);

	/* compare mesg */
	if(oldSize != newSize || memcmp(oldMesg, newMesg, oldSize))
		zkPrintf("### ICMP Failure: zkICMPCreateTimeExceeded() !\n");
	return;



}

static void		testICMPCreateSourceQuench(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF + DATA_SIZE];
	int newRet;
	unsigned char *newMesg;
	size_t newSize;
	
	/* old-style struct declaration */
	ZkIPHeaderAndData oldStyle;
	int oldRet;
	unsigned char *oldMesg;
	size_t oldSize;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.header, newStyle);

	/* set ihl with a valid value => 5 */
	zkIPHeaderSetIHL(newStyle, 5);
	oldStyle.header.ihl = 5;
	zkIPHeaderSetTotalLength(newStyle, 20);
	oldStyle.header.TotalLength = htons(20);

	/* fill ip data with the same values */
	fill_ipData(&oldStyle.data, &newStyle[zkIPHeaderGetIHL(newStyle)*4]);

	newMesg = malloc(zkIPHeaderGetIHL(newStyle)*4 + 36);
	newSize = oldStyle.header.ihl*4 + 36;
	oldMesg = malloc(oldStyle.header.ihl*4 + 36);
	oldSize = oldStyle.header.ihl*4 + 36;

	if(oldSize != newSize)
		zkPrintf("### ICMP Failure: zkICMPCreateSourceQuench() !\n");

	/* test zkICMPCreateSourceQuench */
	newRet = zkICMPCreateSourceQuench ( newStyle , newMesg, &newSize); 
	oldRet = oldICMPCreateSourceQuench( &oldStyle.header, oldMesg, &oldSize);

	/* compare mesg */
	if(oldSize != newSize || memcmp(oldMesg, newMesg, oldSize))
		zkPrintf("### ICMP Failure: zkICMPCreateSourceQuench() !\n");
	return;
}

static void		testIPHeaderDump(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/* test zkIPHeaderDump */
	zkPrintf("testing IPHeaderDump()...\n");
	zkPrintf("Buffer (new-style) function:\n");
	newRet = zkIPHeaderDump(newStyle, stdout);
	zkPrintf("Struct (old-style) function:\n");
	oldRet = oldIPHeaderDump(&oldStyle, stdout);
}

static void		testIPHeaderSourceAddress(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	uint32_t newAddr = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	uint32_t oldAddr = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/* test zkIPHeaderSourceAddress */
	newRet = zkIPHeaderSourceAddress(newStyle, &newAddr);
	oldRet = oldIPHeaderSourceAddress(&oldStyle, &oldAddr); 

	/* compare oldAddr and newAddr */
	if(newAddr != oldAddr)
		zkPrintf("### Failure: zkIPHeaderSourceAddress() !\n");
	return;
}

static void		testIPHeaderDestinationAddress(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	uint32_t newAddr = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	uint32_t oldAddr = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/* test zkIPHeaderDestinationAddress */
	newRet = zkIPHeaderDestinationAddress(newStyle, &newAddr);
	oldRet = oldIPHeaderDestinationAddress(&oldStyle, &oldAddr);

	/* compare oldAddr and newAddr */
	if(newAddr != oldAddr)
		zkPrintf("### Failure: zkIPHeaderDestinationAddress() !\n");
	return;
}

static void		testIPHeaderSetSourceAddress(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint32_t addr = rand32();

	/* return values for the test*/
	uint32_t oldRetAddr = 0;
	uint32_t newRetAddr = 0;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/* test zkIPHeaderSetSourceAddress */
	newRet = zkIPHeaderSetSourceAddress(newStyle, addr);
	oldRet = oldIPHeaderSetSourceAddress(&oldStyle, addr);

	/* to test, use function IPHeaderSourceAddress */
	zkIPHeaderSourceAddress(newStyle, &newRetAddr);
	oldIPHeaderSourceAddress(&oldStyle, &oldRetAddr);
	if(newRetAddr != oldRetAddr)
		zkPrintf("### Failure: zkIPHeaderSetSourceAddress() !\n");
	return;
}

static void		testIPHeaderSetDestinationAddress(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint32_t addr = 0;

	/* return values for the test*/
	uint32_t oldRetAddr = 0;
	uint32_t newRetAddr = 0;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/* test zkIPHeaderSetDestinationAddress */
	newRet = zkIPHeaderSetDestinationAddress(newStyle, addr);
	oldRet = oldIPHeaderSetDestinationAddress(&oldStyle, addr);

	/* to test, use function IPHeaderDestinationAddress */
	zkIPHeaderDestinationAddress(newStyle, &newRetAddr);
	oldIPHeaderDestinationAddress(&oldStyle, &oldRetAddr);
	if(newRetAddr != oldRetAddr)
		zkPrintf("### Failure: zkIPHeaderSetDestinationAddress() !\n");
	return;
}

static void		testIPHeaderStripOptions(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF + 40 + DATA_SIZE]; /* we need options in the ip header */
	int newRet;
	
	/* old-style struct declaration */
	ZkIPHeaderAndOptions oldStyle;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.header, newStyle);

	/* set some options */
	/* possible options are: */
	/* Record Route code=7 Len=39 ptr=4, 8, 12... */
	/* Timestamp code=0x44 Len=40 ptr=5, 9, 13... OF=0 FL=1, 2, 3*/
	/* Source Routing code=0x83, 0x89 Len=39 ptr=4, 8, 12...+ fill ip fields*/ 
	fill_ipOptions(&oldStyle.options, newStyle);

	/* set length fields */
	zkIPHeaderSetIHL(newStyle, 6); /*we need a header with at least some options!!!*/
	oldStyle.header.ihl = 6;
	zkIPHeaderSetTotalLength(newStyle, 24);
	oldStyle.header.TotalLength = htons(24);
	
	/* test zkIPHeaderStripOptions */
	newRet = zkIPHeaderStripOptions(newStyle);
	oldRet = oldIPHeaderStripOptions(&oldStyle.header);

	/* compare both IP headers field-to-field seems like the only way... */
	if(!cmp_ip(&oldStyle.header, newStyle))
		zkPrintf("### Failure: zkIPHeaderStripOptions() !\n");
	return;
}

static void		testIPHeaderChecksum(void)
{
	/* new-style buffer */
	char newStyle[ZKIPHEADER_SIZEOF];
	uint16_t newRet;
	
	/* old-style struct declaration */
	ZkIPHeader oldStyle;
	uint16_t oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle, newStyle);

	/*set ihl and TotalLength fields*/
	zkIPHeaderSetIHL(newStyle, 5);
	oldStyle.ihl = 5;
	zkIPHeaderSetTotalLength(newStyle, 20);
	oldStyle.TotalLength = htons(20);

	/* test zkIPHeaderChecksum */
	newRet = zkIPHeaderChecksum(newStyle);
	oldRet = oldIPHeaderChecksum(&oldStyle);

	/* compare newRet and OldRet, they're the checksums...*/
	if(newRet != oldRet)
		zkPrintf("### Failure: zkIPHeaderChecksum() !\n");
	return;
}

static void		testTCPHeaderDump(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderDump */
	zkPrintf("testing TCPHeaderDump()...\n");
	zkPrintf("Buffer (new-style) function:\n");
	newRet = zkTCPHeaderDump(newStyle, stdout);
	zkPrintf("Struct (old-style) function:\n");
	oldRet = oldTCPHeaderDump(&oldStyle, stdout);
}

static void		testTCPHeaderAcknowledgement(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint32_t newAck = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint32_t oldAck = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderAcknowledgement */
	newRet = zkTCPHeaderAcknowledgement(newStyle, &newAck);
	oldRet = oldTCPHeaderAcknowledgement(&oldStyle, &oldAck);

	/* compare newAck and OldAck + ret==1 => success*/
	if(newAck != oldAck || newRet == 0 || oldRet == 0)
		zkPrintf("### Failure: zkTCPHeaderAcknowledgement() !\n");
	return;
}

static void		testTCPHeaderSequence(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint32_t newSeq = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint32_t oldSeq = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderSequence */
	newRet = zkTCPHeaderSequence(newStyle, &newSeq);
	oldRet = oldTCPHeaderSequence(&oldStyle, &oldSeq);

	/* compare newSeq and OldSeq + ret==1 => success*/
	if(newSeq != oldSeq || newRet == 0 || oldRet == 0)
		zkPrintf("### Failure: zkTCPHeaderSequence() !\n");
	return;
}

static void		testTCPHeaderSourcePort(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint16_t newPort = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint16_t oldPort = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderSourcePort */
	newRet = zkTCPHeaderSourcePort(newStyle, &newPort);
	oldRet = oldTCPHeaderSourcePort(&oldStyle, &oldPort);

	/* compare either newPort and oldPort or newRet and oldRet */
	if(newPort != oldPort || newRet != oldRet)
		zkPrintf("### Failure: zkTCPHeaderSourcePort() !\n");
	return;
}

static void		testTCPHeaderDestinationPort(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint16_t newPort = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint16_t oldPort = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderDestinationPort */
	newRet = zkTCPHeaderDestinationPort(newStyle, &newPort);
	oldRet = oldTCPHeaderDestinationPort(&oldStyle, &oldPort);

	/* compare either newPort and oldPort or newRet and oldRet */
	if(newPort != oldPort || newRet != oldRet)
		zkPrintf("### Failure: zkTCPHeaderSourcePort() !\n");
	return;
}

static void		testTCPHeaderSetSourcePort(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint16_t port = rand16();

	/* return values for the test*/
	uint16_t oldRetPort = 0;
	uint16_t newRetPort = 0;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderSetSourcePort */
	newRet = zkTCPHeaderSetSourcePort(newStyle, port);
	oldRet = oldTCPHeaderSetSourcePort(&oldStyle, port);

	/* test with TCPHeaderSourcePort functions, ret always = 1 */
	zkTCPHeaderSourcePort(newStyle, &newRetPort);
	oldTCPHeaderSourcePort(&oldStyle, &oldRetPort);
	if(newRetPort != oldRetPort)
		zkPrintf("### Failure: zkTCPHeaderSetSourcePort() !\n");
	return;
}

static void		testTCPHeaderSetDestinationPort(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint16_t port = rand16();

	/* return values for the test*/
	uint16_t oldRetPort = 0;
	uint16_t newRetPort = 0;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderSetDestinationPort */
	newRet = zkTCPHeaderSetDestinationPort(newStyle, port);
	oldRet = oldTCPHeaderSetDestinationPort(&oldStyle, port);

	/* test with TCPHeaderDestinationPort functions, ret always = 1 */
	zkTCPHeaderDestinationPort(newStyle, &newRetPort);
	oldTCPHeaderDestinationPort(&oldStyle, &oldRetPort);
	if(newRetPort != oldRetPort)
		zkPrintf("### Failure: zkTCPHeaderSetDestinationPort() !\n");
	return;

}

static void		testTCPHeaderGetMSS(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint16_t newMSS = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint16_t oldMSS = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle, newStyle);

	/* test zkTCPHeaderGetMSS */
	newRet = zkTCPHeaderGetMSS(newStyle, &newMSS);
	oldRet = oldTCPHeaderGetMSS(&oldStyle, &oldMSS);

	/* compare newMSS and oldMSS */
	if(newMSS != oldMSS)
		zkPrintf("### Failure: zkTCPHeaderGetMSS() !\n");
	return;
}

static void		testTCPHeaderSetMSS(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF + DATA_SIZE]; 
	int newRet;

	/* old-style struct declaration */
	ZkTCPHeaderAndData oldStyle;
	int oldRet;

	/* common parameters */
	uint16_t mss = rand16();

	/* return values for the test*/
	uint16_t oldRetMSS = 0;
	uint16_t newRetMSS = 0;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle.header, newStyle);
	fill_ip(&oldStyle.header.IP, ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP));

	/* set length fields in bot TCP and IP Headers */
	zkTCPHeaderSetDoff(newStyle, 5);
	oldStyle.header.doff = 5;
	zkIPHeaderSetIHL(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 5);
	oldStyle.header.IP.ihl = 5;
	zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 40);
	oldStyle.header.IP.TotalLength = htons(40);

	/* test zkTCPHeaderSetMSS */
	newRet = zkTCPHeaderSetMSS(newStyle, mss);
	oldRet = oldTCPHeaderSetMSS(&oldStyle.header, mss);
	
	/* test with TCPHeaderGetMSS function, ret always = 1 */
	zkTCPHeaderGetMSS(newStyle, &newRetMSS);
	oldTCPHeaderGetMSS(&oldStyle.header, &oldRetMSS);
	if(newRetMSS != oldRetMSS)
		zkPrintf("### Failure: zkTCPHeaderSetMSS() !\n");
	return;
}

static void		testTCPHeaderStripOptions(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF + 40 + DATA_SIZE];
	int newRet;
	
	/* old-style struct declaration */
	ZkTCPHeaderAndOptions oldStyle;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_tcp(&oldStyle.header, newStyle);

	/* set some options */
	/* possible options are: */
	/* end of options kind=0x00 */
	/* nop kind=0x01 */
	/* mss kind=0x02, len=4, mss(2 bytes) */ 
	/* window scale kind=3, len=3, shiftcount(1 byte) */
	/* timestamp kind=8 len=10 timestampvalue(4 bytes) timestampecho(4 bytes)*/
	fill_tcpOptions(&oldStyle.options, newStyle);

	/* set length fields in bot TCP and IP Headers */
	zkTCPHeaderSetDoff(newStyle, 6); /* we need at least some options */
	oldStyle.header.doff = 6;
	zkIPHeaderSetIHL(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 5);
	oldStyle.header.IP.ihl = 5;
	zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 44);
	oldStyle.header.IP.TotalLength = htons(44);

	/* test zkTCPHeaderStripOptions */
	newRet = zkTCPHeaderStripOptions(newStyle);
	oldRet = oldTCPHeaderStripOptions(&oldStyle.header);

	/* compare field to field ??? */
	if(!cmp_tcp(&oldStyle.header, newStyle))
		zkPrintf("### Failure: zkTCPHeaderStripOptions() !\n");
	return;
}

static void		testTCPHeaderChecksum(void)
{
	/* new-style buffer */
	char newStyle[ZKTCPHEADER_SIZEOF];
	uint16_t newRet;
	
	/* old-style struct declaration */
	ZkTCPHeader oldStyle;
	uint16_t oldRet;

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.IP, newStyle);
	fill_tcp(&oldStyle, newStyle);

	/* set length fields in bot TCP and IP Headers */
	zkTCPHeaderSetDoff(newStyle, 5);
	oldStyle.doff = 5;
	zkIPHeaderSetIHL(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 5);
	oldStyle.IP.ihl = 5;
	zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 40);
	oldStyle.IP.TotalLength = htons(40);

	/* test zkTCPHeaderChecksum */
	newRet = zkTCPHeaderChecksum(newStyle);
	oldRet = oldTCPHeaderChecksum(&oldStyle);

	/* compare newRet and OldRet*/
	if(newRet != oldRet)
		zkPrintf("### Failure: zkTCPHeaderChecksum() !\n");
	return;
}


static void		testUDPHeaderDump(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_udp(&oldStyle, newStyle);

	/* test zkTCPHeaderDump */
	zkPrintf("testing UDPHeaderDump()...\n");
	zkPrintf("Buffer (new-style) function:\n");
	newRet = zkUDPHeaderDump(newStyle, stdout);
	zkPrintf("Struct (old-style) function:\n");
	oldRet = oldUDPHeaderDump(&oldStyle, stdout);
}

static void		testUDPHeaderSourcePort(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	uint16_t newPort = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	uint16_t oldPort = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_udp(&oldStyle, newStyle);

	/* test zkTCPHeaderSourcePort */
	newRet = zkUDPHeaderSourcePort(newStyle, &newPort);
	oldRet = oldUDPHeaderSourcePort(&oldStyle, &oldPort);

	/* compare either newPort and oldPort or newRet and oldRet */
	if(oldPort != newPort)
		zkPrintf("### Failure: zkUDPHeaderSourcePort() !\n");
	return;
}

static void		testUDPHeaderDestinationPort(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	uint16_t newPort = 0;
	int newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	uint16_t oldPort = 0;
	int oldRet;

	/* fill the structs with the same values...*/
	fill_udp(&oldStyle, newStyle);

	/* test zkUDPHeaderDestinationPort */
	newRet = zkUDPHeaderDestinationPort(newStyle, &newPort);
	oldRet = oldUDPHeaderDestinationPort(&oldStyle, &oldPort);

	/* compare either newPort and oldPort or newRet and oldRet */
	if(oldPort != newPort)
		zkPrintf("### Failure: zkUDPHeaderDestinationPort() !\n");
	return;
}

static void		testUDPHeaderSetSourcePort(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint16_t port = rand16();

	/* return values for the test*/
	uint16_t oldRetPort = 0;
	uint16_t newRetPort = 0;

	/* fill the structs with the same values...*/
	fill_udp(&oldStyle, newStyle);

	/* test zkUDPHeaderSetSourcePort */
	newRet = zkUDPHeaderSetSourcePort(newStyle, port);
	oldRet = oldUDPHeaderSetSourcePort(&oldStyle, port);

	/* test with UDPHeaderSourcePort functions, ret always = 1 */
	zkUDPHeaderSourcePort(newStyle, &newRetPort);
	oldUDPHeaderSourcePort(&oldStyle, &oldRetPort);
	if(newRetPort != oldRetPort)
		zkPrintf("### Failure: zkUDPHeaderSetSourcePort() !\n");
	return;
}

static void		testUDPHeaderSetDestinationPort(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	int newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	int oldRet;

	/* common parameters */
	uint16_t port = rand16();

	/* return values for the test*/
	uint16_t oldRetPort = 0;
	uint16_t newRetPort = 0;

	/* fill the structs with the same values...*/
	fill_udp(&oldStyle, newStyle);

	/* test zkUDPHeaderSetDestinationPort */
	newRet = zkUDPHeaderSetDestinationPort(newStyle, port);
	oldRet = oldUDPHeaderSetDestinationPort(&oldStyle, port);

	/* test with UDPHeaderDestinationPort functions, ret always = 1 */
	zkUDPHeaderDestinationPort(newStyle, &newRetPort);
	oldUDPHeaderDestinationPort(&oldStyle, &oldRetPort);
	if(newRetPort != oldRetPort)
		zkPrintf("### Failure: zkUDPHeaderSetDestinationPort() !\n");
	return;
}

static void		testUDPHeaderChecksum(void)
{
	/* new-style buffer */
	char newStyle[ZKUDPHEADER_SIZEOF];
	uint16_t newRet;
	
	/* old-style struct declaration */
	ZkUDPHeader oldStyle;
	uint16_t oldRet;

	/* common parameters */
	unsigned int datalen = 0; /* TEMPORARY */

	/* fill the structs with the same values...*/
	fill_ip(&oldStyle.IP, newStyle);
	fill_udp(&oldStyle, newStyle);

	/*fill some fields...*/
	zkUDPHeaderSetUdpLength(newStyle, 8);
	oldStyle.UdpLength = htons(8);
	zkIPHeaderSetIHL(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 5);
	oldStyle.IP.ihl = 5;
	zkIPHeaderSetTotalLength(ZKPACKBUF_ADDR_OF(newStyle, ZKTCPHEADER_IP), 28);
	oldStyle.IP.TotalLength = htons(28);

	/* test zkUDPHeaderChecksum */
	newRet = zkUDPHeaderChecksum(newStyle, datalen);
	oldRet = oldUDPHeaderChecksum(&oldStyle, datalen);

	/* compare newRet and OldRet*/
	if(newRet != oldRet)
		zkPrintf("### Failure: zkUDPHeaderChecksum() !\n");
	return;
}

static void		fill_icmp(ZkICMPHeader* strc, void* buff)
{
	uint8_t type = rand8();
	uint8_t code = rand8();
	uint16_t checksum = rand16();
	uint16_t identifier = rand16();
	uint16_t sequencenumber = rand16();

	if(buff)
	{
		zkICMPHeaderSetType(buff, type);
		zkICMPHeaderSetCode(buff, code);
		zkICMPHeaderSetChecksum(buff, checksum);
		zkICMPHeaderSetIdentifier(buff, identifier);
		zkICMPHeaderSetSequenceNumber(buff, sequencenumber);
	}

	if(strc)
	{
		strc->Type			= type;
		strc->code			= code;
		strc->CheckSum		= htons(checksum);
		strc->Identifier		= htons(identifier);
		strc->SequenceNumber	= htons(sequencenumber);
	}
}


static void		fill_ip(ZkIPHeader* strc, void* buff)
{
	uint8_t ihl				= rand8() & 0x0F;
	uint8_t version			= rand8() & 0x0F;
	uint8_t typeofsvr		= rand8();
	uint8_t timetolive		= rand8();
	uint8_t protocol		= rand8();
	uint16_t totallength	= rand16();
	uint16_t identification	= rand16();
	uint16_t flags_frameoffs= rand16();
	uint16_t headerchecksum	= rand16();
	uint32_t srcipaddress	= rand32();
	uint32_t dstipaddress	= rand32();

	if (buff)
	{
		zkIPHeaderSetIHL(buff, ihl);
		zkIPHeaderSetVersion(buff, version);
		zkIPHeaderSetTypeOfSvr(buff, typeofsvr);
		zkIPHeaderSetTotalLength(buff, totallength);
		zkIPHeaderSetIdentification(buff, identification);
		zkIPHeaderSetFlags_FrameOffs(buff, flags_frameoffs);
		zkIPHeaderSetTimeToLive(buff, timetolive);
		zkIPHeaderSetProtocol(buff, protocol);
		zkIPHeaderSetHeaderChkSum(buff, headerchecksum);
		zkIPHeaderSetSrcIpAddress(buff, srcipaddress);
		zkIPHeaderSetDstIpAddress(buff, dstipaddress);
	}
	if(strc)
	{
		strc->ihl				= ihl;
		strc->version			= version;
		strc->TypeOfSvr			= typeofsvr;
		strc->TotalLength		= htons(totallength);
		strc->Identification	= htons(identification);
		strc->Flags_FrameOffs	= htons(flags_frameoffs);
		strc->TimeToLive		= timetolive;
		strc->Protocol			= protocol;
		strc->HeaderChkSum		= htons(headerchecksum);
		strc->SrcIpAddress		= htonl(srcipaddress);
		strc->DstIpAddress		= htonl(dstipaddress);
	}
}


static void		fill_tcp(ZkTCPHeader* strc, void* buff)
{
	uint16_t srcport		= rand16();
	uint16_t dstport		= rand16();
	uint32_t sequencenumber	= rand32();
	uint32_t acknumber		= rand32();
	uint8_t doff			= rand8() & 0x0F;
	uint8_t res				= rand8() & 0x3F;
	uint8_t urg				= rand8() & 0x01;
	uint8_t ack				= rand8() & 0x01;
	uint8_t psh				= rand8() & 0x01;
	uint8_t rst				= rand8() & 0x01;
	uint8_t syn				= rand8() & 0x01;
	uint8_t fin				= rand8() & 0x01;
	uint16_t window			= rand16();
	uint16_t tcpchecksum	= rand16();
	uint16_t urgentpointer	= rand16();

	if (buff)
	{
		zkTCPHeaderSetSrcPort(buff, srcport);
		zkTCPHeaderSetDstPort(buff, dstport);
		zkTCPHeaderSetSequenceNumber(buff, sequencenumber);
		zkTCPHeaderSetAckNumber(buff, acknumber);
		zkTCPHeaderSetDoff(buff, doff);
		zkTCPHeaderSetRes(buff, res);
		zkTCPHeaderSetUrg(buff, urg);
		zkTCPHeaderSetAck(buff, ack);
		zkTCPHeaderSetPsh(buff, psh);
		zkTCPHeaderSetRst(buff, rst);
		zkTCPHeaderSetSyn(buff, syn);
		zkTCPHeaderSetFin(buff, fin);
		zkTCPHeaderSetWindow(buff, window);
		zkTCPHeaderSetTcpCheckSum(buff, tcpchecksum);
		zkTCPHeaderSetUrgentPointer(buff, urgentpointer);
	}
	if(strc)
	{
		strc->SrcPort		= htons(srcport);
		strc->DstPort		= htons(dstport);
		strc->SequenceNumber= htonl(sequencenumber);
		strc->AckNumber		= htonl(acknumber);
		strc->doff			= doff;
		strc->res1			= (res >> 2);
		strc->res2			= res;
		strc->urg			= urg;
		strc->ack			= ack;
		strc->psh			= psh;
		strc->rst			= rst;
		strc->syn			= syn;
		strc->fin			= fin;
		strc->Window		= htons(window);
		strc->TcpCheckSum	= htons(tcpchecksum);
		strc->UrgentPointer	= htons(urgentpointer);
	}
}


static void		fill_udp(ZkUDPHeader* strc, void* buff)
{
	uint16_t srcport		= rand16();
	uint16_t dstport		= rand16();
	uint16_t udplength		= rand16();
	uint16_t udpchecksum	= rand16();

	if (buff)
	{
		zkUDPHeaderSetSrcPort(buff, srcport);
		zkUDPHeaderSetDstPort(buff, dstport);
		zkUDPHeaderSetUdpLength(buff, udplength);
		zkUDPHeaderSetUdpCheckSum(buff, udpchecksum);
	}
	if(strc)
	{
		strc->SrcPort		= htons(srcport);
		strc->DstPort		= htons(dstport);
		strc->UdpLength		= htons(udplength);
		strc->UdpCheckSum	= htons(udpchecksum);
	}
}

static void fill_ipOptions( void *oldStyle, void *newStyle)
{
	/* possible options are: */
	/* Record Route code=0x07 Len=39 ptr=4, 8, 12... */
	/* Timestamp code=0x44 Len=40 ptr=5, 9, 13... OF=0 FL=1, 2, 3*/
	/* Source Routing code=0x83, 0x89 Len=39 ptr=4, 8, 12...+ fill ip fields*/ 
	uint8_t code;
	uint8_t len; /*depends on code*/
	uint8_t ptr; /*depends on code*/
	uint8_t offl; /*only when code==0x44*/
	uint32_t ipAddr[9] = {0,0,0,0,0,0,0,0,0};/*depends on len*/

	int curIndex = 0;

	code = rand8() & 0x03;
	switch(code)
	{
	case(0):
		code = 0x07;
		len = 39;
		ptr = 4;
		offl = 0xFF;
		break;
	case(1):
		code = 0x44;
		len = 40;
		ptr = 5;
		offl = rand8() & 0x13;
		if((offl&0x03) == 0x03)
		{
			ipAddr[0] = rand32();
			ipAddr[2] = rand32();
			ipAddr[4] = rand32();
			ipAddr[6] = rand32();
		}
		break;
	case(2):
		code = 0x83;
		len = 39;
		ptr = 4;
		offl = 0xFF;
		ipAddr[0] = rand32();
		ipAddr[1] = rand32();
		ipAddr[2] = rand32();
		ipAddr[3] = rand32();
		ipAddr[4] = rand32();
		ipAddr[5] = rand32();
		ipAddr[6] = rand32();
		ipAddr[7] = rand32();
		ipAddr[8] = rand32();
		break;
	case(3):
		code = 0x89;
		len = 39;
		ptr = 4;
		offl = 0xFF;
		ipAddr[0] = rand32();
		ipAddr[1] = rand32();
		ipAddr[2] = rand32();
		ipAddr[3] = rand32();
		ipAddr[4] = rand32();
		ipAddr[5] = rand32();
		ipAddr[6] = rand32();
		ipAddr[7] = rand32();
		ipAddr[8] = rand32();
		break;
	}

	if(oldStyle)
	{
		curIndex = 0;
		zkPackBufSetUint8(oldStyle, curIndex++, code); 
		zkPackBufSetUint8(oldStyle, curIndex++, len);  
		zkPackBufSetUint8(oldStyle, curIndex++, ptr);  
		if(offl != 0xFF)
		{
			zkPackBufSetUint8(oldStyle, curIndex++, offl);  
		}
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[0]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[1]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[2]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[3]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[4]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[5]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[6]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[7]); curIndex = curIndex+4;
		zkPackBufSetUint32(oldStyle, curIndex, ipAddr[8]); curIndex = curIndex+4;
	}
	if(newStyle)
	{
		curIndex = ZKIPHEADER_SIZEOF;
		zkPackBufSetUint8(newStyle, curIndex++, code);
		zkPackBufSetUint8(newStyle, curIndex++, len);  
		zkPackBufSetUint8(newStyle, curIndex++, ptr);  
		if(offl != 0xFF)
		{
			zkPackBufSetUint8(newStyle, curIndex++, offl);  
		}
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[0]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[1]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[2]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[3]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[4]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[5]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[6]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[7]); curIndex = curIndex+4;
		zkPackBufSetUint32(newStyle, curIndex, ipAddr[8]); curIndex = curIndex+4;
	}
}

static void fill_tcpOptions(void *oldStyle, void *newStyle)
{
	/* possible options are: */
	/* end of options kind=0x00 */
	/* nop kind=0x01 */
	/* mss kind=0x02, len=4, mss(2 bytes) */ 
	/* window scale kind=3, len=3, shiftcount(1 byte) */
	/* timestamp kind=8 len=10 timestampvalue(4 bytes) timestampecho(4 bytes)*/
	uint8_t kind;
	uint8_t len;/*when kind==2,3,8*/
	uint16_t mss;/*when kind==2*/
	uint8_t shiftcount;/*when kind==3*/
	uint32_t tsValue;/*when kind==8*/
	uint32_t tsEcho; /*when kind==8*/
	int nbOptions = rand8() & 0x04; /* max 4 options*/
	int oldIndex = 0;
	int newIndex = 0;
	int i;

	oldIndex = 0;	
	newIndex = ZKTCPHEADER_SIZEOF;
	for(i=0; i<nbOptions; i++)
	{
		kind = rand8() & 0x04;
		if(kind==0)
			kind = 8;

		switch(kind)
		{
		case(0):
			break;
		case(1):
			break;
		case(2):
			len=4;
			mss = rand16();
			break;
		case(3):
			len=3;
			shiftcount = rand8();
			break;
		case(8):
			len = 10;
			tsValue = rand32();
			tsEcho = rand32();
			break;
		}

		if(oldStyle)
		{
			zkPackBufSetUint8(oldStyle, oldIndex++, kind); 
			if(kind == 2 || kind == 3 || kind == 8)
			{
				zkPackBufSetUint8(oldStyle, oldIndex++, len);  
			}
			if(kind == 2)
			{
				zkPackBufSetUint16(oldStyle, oldIndex, mss); oldIndex = oldIndex+2;
			}
			if(kind == 3)
			{
				zkPackBufSetUint8(oldStyle, oldIndex++, shiftcount);  
			}
			if(kind == 8)
			{
				zkPackBufSetUint32(oldStyle, oldIndex, tsValue); oldIndex = oldIndex+4; 
				zkPackBufSetUint32(oldStyle, oldIndex, tsEcho); oldIndex = oldIndex+4;  
			}
		}
		if(newStyle)
		{
			zkPackBufSetUint8(newStyle, newIndex++, kind); 
			if(kind == 2 || kind == 3 || kind == 8)
			{
				zkPackBufSetUint8(newStyle, newIndex++, len);  
			}
			if(kind == 2)
			{
				zkPackBufSetUint16(newStyle, newIndex, mss); newIndex = newIndex+2;
			}
			if(kind == 3)
			{
				zkPackBufSetUint8(newStyle, newIndex++, shiftcount);  
			}
			if(kind == 8)
			{
				zkPackBufSetUint32(newStyle, newIndex, tsValue); newIndex = newIndex+4; 
				zkPackBufSetUint32(newStyle, newIndex, tsEcho); newIndex = newIndex+4;  
			}	
		}
	}
	/*add the end-of-options*/
	if(oldStyle)
	{
		zkPackBufSetUint8(oldStyle, oldIndex++, 0x00);  
	}
	if(newStyle)
	{
		zkPackBufSetUint8(newStyle, newIndex++, 0x00);  
	}
}

static void fill_ipData( void *oldStyle, void *newStyle)
{
	int i;
	uint8_t val;
	for(i=0; i<DATA_SIZE; i++)
	{
		val = rand8();
		if(oldStyle)
		{
			zkPackBufSetUint8(oldStyle, i, val);  
		}
		if(newStyle)
		{
			zkPackBufSetUint8(newStyle, i, val);  
		}
	}
}

#if 0	/* currently unused */

static int		cmp_icmp(ZkICMPHeader* strc, void* buff)
{
	if((zkICMPHeaderGetType(buff) != strc->Type) ||
	   (zkICMPHeaderGetCode(buff) != strc->code) ||
	   (zkICMPHeaderGetChecksum(buff) != ntohs(strc->CheckSum)) ||
	   (zkICMPHeaderGetIdentifier(buff) != ntohs(strc->Identifier)) ||
	   (zkICMPHeaderGetSequenceNumber(buff) != ntohs(strc->SequenceNumber)))
		return 0;
	return 1;
}

#endif

static int		cmp_ip(ZkIPHeader* strc, void* buff)
{
	if((zkIPHeaderGetIHL(buff) != strc->ihl) ||
	   (zkIPHeaderGetVersion(buff) != strc->version) ||
	   (zkIPHeaderGetTypeOfSvr(buff) != strc->TypeOfSvr) ||
	   (zkIPHeaderGetTotalLength(buff) != ntohs(strc->TotalLength)) ||
	   (zkIPHeaderGetIdentification(buff) != ntohs(strc->Identification)) ||
	   (zkIPHeaderGetFlags_FrameOffs(buff) != ntohs(strc->Flags_FrameOffs)) ||
	   (zkIPHeaderGetTimeToLive(buff) != strc->TimeToLive) ||
	   (zkIPHeaderGetProtocol(buff) != strc->Protocol) ||
	   (zkIPHeaderGetHeaderChkSum(buff) != ntohs(strc->HeaderChkSum)) ||
	   (zkIPHeaderGetSrcIpAddress(buff) != ntohl(strc->SrcIpAddress)) ||
	   (zkIPHeaderGetDstIpAddress(buff) != ntohl(strc->DstIpAddress)))
		return 0;
	return 1;
}

static int		cmp_tcp(ZkTCPHeader* strc, void* buff)
{
	if((zkTCPHeaderGetSrcPort(buff) != ntohs(strc->SrcPort)) ||
	   (zkTCPHeaderGetDstPort(buff) != ntohs(strc->DstPort)) ||
	   (zkTCPHeaderGetSequenceNumber(buff) != ntohl(strc->SequenceNumber)) ||
	   (zkTCPHeaderGetAckNumber(buff) != ntohl(strc->AckNumber)) ||
	   (zkTCPHeaderGetDoff(buff) != strc->doff) ||
	   (zkTCPHeaderGetRes(buff) != (uint32_t)((strc->res1<<2)|(strc->res2))) ||
	   (zkTCPHeaderGetUrg(buff) != strc->urg) ||
	   (zkTCPHeaderGetAck(buff) != strc->ack) ||
	   (zkTCPHeaderGetPsh(buff) != strc->psh) ||
	   (zkTCPHeaderGetRst(buff) != strc->rst) ||
	   (zkTCPHeaderGetSyn(buff) != strc->syn) ||
	   (zkTCPHeaderGetFin(buff) != strc->fin) ||
	   (zkTCPHeaderGetWindow(buff) != ntohs(strc->Window)) ||
	   (zkTCPHeaderGetTcpCheckSum(buff) != ntohs(strc->TcpCheckSum)) ||
	   (zkTCPHeaderGetUrgentPointer(buff) != ntohs(strc->UrgentPointer)))
		return 0;
	return 1;
}

#if 0	/* currently unused */

static int		cmp_udp(ZkUDPHeader* strc, void* buff)
{
	if((zkUDPHeaderGetSrcPort(buff) != ntohs(strc->SrcPort)) ||
		(zkUDPHeaderGetDstPort(buff) != ntohs(strc->DstPort)) ||
		(zkUDPHeaderGetUdpLength(buff) != ntohs(strc->UdpLength)) ||
		(zkUDPHeaderGetUdpCheckSum(buff) != ntohs(strc->UdpCheckSum)))
		return 0;
	return 1;
}

#endif

static uint8_t rand8(void)
{
	return (rand());
}

static uint16_t rand16(void)
{
	return ((((uint16_t) rand8()) << 8) | 
	         ((uint16_t) rand8()));
}

static uint32_t rand32(void)
{
	return ((((uint32_t) rand16()) << 16) | 
	         ((uint32_t) rand16()));
}

#if 0	/* currently unused */

static uint64_t rand64(void)
{
	return ((((uint64_t) rand32()) << 32) | 
	         ((uint64_t) rand32()));
}

#endif
