/*
 * $Id: zkperf.h,v 1.7 2001/04/19 16:49:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#ifndef _ZKPERF_H
#define _ZKPERF_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <sys/time.h>
#include "zkallocator.h"
  
typedef struct T_ZkPerfClient *ZkPerfClient;
typedef struct T_ZkPerfTimer  *ZkPerfTimer;  


struct S_ZkPerfClientContext {
  ZkLog log;
  void *obj;
}; 

typedef struct S_ZkPerfClientContext *ZkPerfClientContext;

struct S_ZkPerfClientTime {
  int valid;
  double system;
  double user;
  double real;
};

struct S_ZkPerfClientChild {
  int valid;
  struct timeval startTime;
  struct timeval endTime;
  struct timeval elapsedTime;
};

struct S_ZkPerfClientReport {
  int num;
  struct S_ZkPerfClientTime *entries;
  struct S_ZkPerfClientChild *ff;
  double sysAvg;
  double userAvg;
  double realAvg;
};

typedef struct S_ZkPerfClientReport *ZkPerfClientReport;
  
  /* The structure ZkPerfFunction defines the possible call-back behavior of
     the client code. At each iteration, a function is selected by some
     probability ``prob''. If available, the prefunc() and postfunc() are
     called before and after the main function fun() is executed. For each
     such set of functions there is a symbolic timer that will be get run.
  */
  
typedef struct {
  const char *name;
  int (*prefunc)(ZkError err, ZkPerfClientContext ctx);
  int (*func)(ZkError err, ZkPerfClientContext ctx);
  int (*postfunc)(ZkError err, ZkPerfClientContext ctx);
  const char *preTimer;
  const char *Timer;
  const char *postTimer;
  double prob;

} ZkPerfFunction;

  /* Default Symbolic Timer */

#define ZKPERF_CLIENT_DEFAULT_TIMER "main"
  
  /* prototypes */

  /* zkperfClient */
  
DLLExport int zkperfClientNew( ZkError err, ZkPerfClient *c );
DLLExport int zkperfClientFree( ZkError err, ZkPerfClient *c );
DLLExport int zkperfClientSetServer( ZkError err, ZkPerfClient c,
                                     const char *server, const char *service );
DLLExport int zkperfClientSetMonitor( ZkError err, ZkPerfClient c,
                                      const char *host, const char *port );
DLLExport int zkperfClientSetObject( ZkError err, ZkPerfClient c, void *obj );
DLLExport int zkperfClientSetLog( ZkError err, ZkPerfClient c, ZkLog log );  
DLLExport int zkperfClientSetRunParams( ZkError err, ZkPerfClient c,
                                         int concurrent, int iterations );
DLLExport int zkperfClientSetFunctions( ZkError err, ZkPerfClient c,
                                        int (*pre)(ZkError err, ZkPerfClientContext ctx),
                                        int (*post)(ZkError err, ZkPerfClientContext ctx),
                                        ZkPerfFunction *funcs, size_t num );
DLLExport int zkperfClientSetSynchSig( ZkError err, ZkPerfClient c, int sig );
DLLExport int zkperfClientStart( ZkError err, ZkPerfClient c );
DLLExport int zkperfClientReport( ZkError err, ZkPerfClient c,const char *name,
                                  ZkPerfClientReport report);
DLLExport int zkperfClientReportToMonitor(  ZkError err, ZkPerfClient c );
DLLExport int zkperfClientReportTask( ZkError err, ZkPerfClient c, FILE *fp, int header);
DLLExport int zkperfClientReportRun( ZkError err, ZkPerfClient c, FILE *fp, int header);  
DLLExport int zkperfClientReportConcurrent( ZkError err, ZkPerfClient c,
                                            FILE *fp, int header);
DLLExport int zkperfClientReportAvg( ZkError err, ZkPerfClient c, FILE *fp,
                                     int header);
  
DLLExport int zkperfClientUseThreads( ZkError err, ZkPerfClient c );

DLLExport int zkperfClientTimerAdd (ZkError err, ZkPerfClient c, const char **names);
DLLExport int zkperfClientTimerStart (ZkError err, ZkPerfClient c, const char *name);
DLLExport int zkperfClientTimerStop (ZkError err, ZkPerfClient c, const char *name);
DLLExport int zkperfClientTimerResume (ZkError err, ZkPerfClient c, const char *name);
DLLExport int zkperfClientTimerPause (ZkError err, ZkPerfClient c, const char *name);
  
  /* zkperfTimer */

DLLExport int zkperfTimerNew (ZkError err, ZkPerfTimer *t, ZkAllocator a);
DLLExport int zkperfTimerFree (ZkError err, ZkPerfTimer *t);
DLLExport int zkperfTimerAdd (ZkError err, ZkPerfTimer t, const char *name, int dupl);
DLLExport int zkperfTimerStart (ZkError err, ZkPerfTimer t, const char *name);
DLLExport int zkperfTimerStop (ZkError err, ZkPerfTimer t, const char *name);
DLLExport int zkperfTimerResume (ZkError err, ZkPerfTimer t, const char *name);
DLLExport int zkperfTimerPause (ZkError err, ZkPerfTimer t, const char *name);
DLLExport int zkperfTimerProcessTimesGet(ZkError err, ZkPerfTimer t,
                                         const char *name,
                                         double *rv, double *uv, double *sv);

  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ZKPERF_H */
