#if defined( ZK_LINUX ) || defined( ZK_SOLARIS )
/*
 * Pthread compliant library
 *
 * $Id: zkpthread.h,v 1.4 2001/04/19 16:49:21 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * Nicolas Roy <nicolas@zeroknowledge.com>

 */

#include <zks.h>
#include <zkIntTypes.h>
#include <zkError.h>
#include <zkLog.h>
#include <zkAlloc.h>

#include <stdlib.h>
#include <stdarg.h>
#include <pthread.h>
#include <signal.h>





typedef struct ZkPThread_S                 *ZkPThread;
typedef pthread_t                           ZkPThreadId;

typedef void                               *ZkPThreadData;
typedef int                                (*ZkPThreadProc) (ZkError, ZkLog, ZkPThreadData);



#define ZK_PTHREAD_NONE_FLAG                (0x00000000)
#define ZK_PTHREAD_NO_ERROR_OBJECT_FLAG     (0x00000001)




/* ********************************************************************
 * Exportable functions
 * *******************************************************************/


int zkPThreadNew(ZkError       err,
                 ZkPThread    *hdlP, 
                 ZkPThreadProc proc, 
                 ZkPThreadData data, 
                 ZkLog         log, 
                 uint32_t      flags);

int zkPThreadFree(ZkError    err, 
                  ZkPThread *hdlP);

int zkPThreadSetProc(ZkError err, 
                     ZkPThread hdl, 
                     ZkPThreadProc proc, 
                     ZkPThreadData data);

int zkPThreadGetProc(ZkError err, 
                     ZkPThread hdl, 
                     ZkPThreadProc * procP, 
                     ZkPThreadData * dataP);

int zkPThreadSetLog(ZkError err, 
                    ZkPThread hdl, 
                    ZkLog log);

int zkPThreadGetLog(ZkError err, 
                    ZkPThread hdl, 
                    ZkLog *logP);

int zkPThreadSetFlags(ZkError   err, 
                      ZkPThread hdl, 
                      uint32_t  flags);

int zkPThreadGetFlags(ZkError    err, 
                      ZkPThread  hdl, 
                      uint32_t  *flagsP);

int zkPThreadRun(ZkError   err,
                 ZkPThread hdl);

int zkPThreadDetach(ZkError err, 
                    ZkPThread hdl);

int zkPThreadJoin(ZkError err, 
                  ZkPThread hdl, 
                  void **retcodeP);

int zkPThreadKill(ZkError err, 
                  ZkPThread hdl, 
                  int sig);

int zkPThreadDestroy(ZkError err, 
                     ZkPThread hdl);


/* ********************************************************************
** Running functions, to be use by the thread it self.
*/

ZkPThreadId zkPThreadId(void);

int zkPThreadExit(ZkError err, 
                  void * retval);

int zkPThreadYield(ZkError err);

int zkPThreadSigmask(ZkError         err, 
                     int             how, 
                     const sigset_t *newmask, 
                     sigset_t       *oldmask);

#endif /* ZK_LINUX || ZK_SOLARIS */
