/*
 * $Id: client.h,v 1.7 2001/04/19 16:49:22 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *
 *
 */

#ifndef _CLIENT_H
#define _CLIENT_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <sys/timeb.h>
#include <pthread.h>

#include "zksocket.h"
#include "zkvheap.h"
#include "zkshm.h"
  
struct T_PerfClientInfo {
  int state;
  ZkPerfTimer timer;
  struct timeval startTime;
  struct timeval endTime;
  struct timeval elapsedTime;  
}; 

typedef struct T_PerfClientInfo *PerfClientInfo;  

  
#define ZKPERF_CLIENT_STATE_READY     0
#define ZKPERF_CLIENT_STATE_RUNNING   1
#define ZKPERF_CLIENT_STATE_FINISHED  2
  

#define FORK   0
#define THREAD 1
  
#define DEFAULT_SYNCH_SIGNAL           SIGPWR
#define ZKPERF_SHMNAME_LENGTH          ZKSHM_MAXNAME_LENGTH
#define ZKPERF_SHAREDMEM_NAME_MASK        "zkperf.shm.%d"

struct T_ZkPerfClient {
  char comment;
  char *server;
  char *service;
  void *obj;
  int concurrent;
  int iterations;
  int type;
  pthread_t *thread;
  int *pid;
  
  /* Monitor information */
  int monitor;
  char *monitorHost;
  char *monitorPort;
  ZkSocket monitorSocket;
  
  /* Callback functions */
  int (*spawn)(ZkError err, ZkPerfClient c);
  int (*synch)(ZkError err, ZkPerfClient c, int ischild);
  
  /* Application functions */
  ZkPerfFunction *funcs;
  int (*pre)(ZkError err, ZkPerfClientContext ctx);
  int (*post)(ZkError err, ZkPerfClientContext cts);
  int numfuncs;
  int sig;
  
  /* Log */
  ZkLog log;
  
  /* Timers */
  const char **timerList;
  
  /* Shared memory */
  ZkShm shm;
  char shmname[ZKPERF_SHMNAME_LENGTH+2];
  void *addr;
  ZkVHeap vheap;
  
  /* Shared info */
  PerfClientInfo info;
  size_t infosize;
  int which;
  
  /* Memory Allocator */
  ZkAllocator allocator;
};


int zkperfRegisterTimers( ZkError err, ZkPerfClient c  );
int zkperfClientChild(  ZkError err, ZkPerfClient c, int childnum );
int zkperfClientUseProcess( ZkError err, ZkPerfClient c );
int zkperfClient_buildTimerList( ZkError err, ZkPerfClient c, int all, 
                                 const char ***list, int *listnum );
int zkperfClient_MonitorWrite( ZkError err, ZkPerfClient c);
int zkperfClientDup( ZkError err, ZkPerfClient c, ZkPerfClient *copy);
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _client_H */
