
/* $Id: monitor.c,v 1.6 2001/04/19 16:49:22 alainc Exp $ */
/*
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zks.h"


#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <sys/socket.h>

#include "zkError.h"
#include "zkLog.h"
#include "zktimer.h"
#include "zkallocator.h"

#include "zkperf.h"
#include "timer.h"
#include "client.h"
#include "monitor.h"

static char *extractDouble(char *buff, double *val)
{

  memcpy(val, buff, sizeof(double));
  buff += sizeof(double);

  return buff;
}

static char *extractLong(char *buff, long *val)
{
  char *tmp = buff;
  long len;
  
  memcpy(&len, tmp, sizeof(int));
  *val = ntohl(len);
  tmp += sizeof(int);
  
  return tmp;
}

static char *extract(char *buff, int *val, char **str)
{
  char *tmp = buff;
  int len;
  
  if ( str == NULL ) {
    memcpy(&len, tmp, sizeof(int));
    *val = ntohl(len);
    tmp += sizeof(int);
  } else {
    *str = tmp;
    tmp += strlen(*str);
    tmp++;
  }

  return tmp;
}

static char *appendDouble(char *buff, size_t *bsize, double val)
{
  if ( buff ) {
    if ( sizeof(val) < *bsize ) {
      memcpy(buff, &val, sizeof(val));
      buff += sizeof(val);
      *bsize -= sizeof(val);
    } else {
      return NULL;
    }
  }

  return buff;
}

static char *append(char *buff, size_t *bsize, int val, const char *str)
{
  char *tmp = buff;
  size_t len;

  if ( buff ) {
    if ( str == NULL ) {
      len = htonl(val);
      if ( sizeof(int) < *bsize ) {
        memcpy(tmp, &len, sizeof(int));
        tmp += sizeof(int);
        *bsize -= sizeof(int);
      } else {
        return NULL;
      }
    } else {
      len = strlen(str);
      if ( len < *bsize ) {
        strncpy(tmp, str, len);
        tmp += len;
        *bsize -= len;
        *tmp = '\0';
        tmp++;
      } else {
        return NULL;
      }
    }
  }

  return tmp;
}


static int computeSize( ZkError err, ZkPerfClient c, const char **list,
                        int listnum,  size_t *size)
{
  int i;
  int ret = 0;
  size_t len = 0;
  
  (void)err;

  len += sizeof(c->concurrent);
  len += sizeof(c->iterations);  
  len += sizeof(listnum);
  for ( i = 0; i < listnum; i++ ) {
    len += strlen(list[i])+1;
  }
  for ( i = 0; i < c->concurrent; i++ ) {
    if ( c->info[i].state == ZKPERF_CLIENT_STATE_FINISHED ) {
      len += sizeof(struct timeval) * 2;
      len += (listnum * sizeof(double) * 3 );
    } else {
      /*      fprintf(stderr,"\n\n\nHmm %d\n\n\n",c->info[i].state); */
    }              
  }

  *size = len;
  ret = 1;

  return ret;
}

static int serialize( ZkError err, ZkPerfClient c, char *buff, size_t bsize,
                      const char **list, int listnum )
{
  int i,j;
  int ret = 0;
  char *tmp;
  double rv, uv, sv;
  size_t size;
  
  (void)err;

  size = bsize;
  tmp = buff;

  tmp = append(tmp, &size, c->concurrent, NULL);
  tmp = append(tmp, &size, c->iterations, NULL);
  
  tmp = append(tmp, &size, listnum, NULL);
  for ( i = 0; i < listnum; i++ ) {
    tmp = append(tmp, &size, 0, list[i]);
  }
  
  for ( i = 0; i < c->concurrent; i++ ) {
    if ( c->info[i].state == ZKPERF_CLIENT_STATE_FINISHED ) {
    
      tmp = append(tmp, &size, c->info[i].startTime.tv_sec, NULL);
      tmp = append(tmp, &size, c->info[i].startTime.tv_usec, NULL);  
      tmp = append(tmp, &size, c->info[i].endTime.tv_sec, NULL);
      tmp = append(tmp, &size, c->info[i].endTime.tv_usec, NULL);
    
      for ( j = 0; j < listnum; j++ ) {
        if ( ! zkperfTimerProcessTimesGet(err, c->info[i].timer, list[j],
                                          &rv, &uv, &sv) ) {
          goto done;
        }
        tmp = appendDouble(tmp, &size, rv);
        tmp = appendDouble(tmp, &size, uv);
        tmp = appendDouble(tmp, &size, sv);
        //        fprintf(stderr,"--> %.4f %.4f %.4f\n",rv,uv,sv);
      }
    }
  }

  ret = 1;
  
 done:
  
  return ret;
}



static int deserialize( ZkError err, char *buff, size_t sbuff,
                        T_MonitorSlave *slave)
{
  int i,j,ret = 0;
  char *tmp = NULL;
  char **list = NULL;
  int listnum;

  (void)err;
  fprintf(stderr, "size of buffer %d",sbuff);
  
  tmp = buff;

  tmp = extract(tmp, &slave->concurrents, NULL);
  if ( (slave->info = (T_MonitorInfo*)malloc(sizeof(T_MonitorInfo)*slave->concurrents)) == NULL ) {
    goto done;
  }
  tmp = extract(tmp, &slave->iterations, NULL);
  tmp = extract(tmp, &listnum, NULL);
  if ( (list = (char**)malloc(listnum*sizeof(char*))) == NULL ) {
    goto done;
  }
  for ( i = 0; i < listnum; i++ ) {
    tmp = extract(tmp, 0, &list[i]);
  }

  for ( i = 0; i < slave->concurrents; i++ ) {
    slave->info[i].num = listnum;
    tmp = extractLong(tmp, &(slave->info[i].startTime.tv_sec));
    tmp = extractLong(tmp, &(slave->info[i].startTime.tv_usec));  
    tmp = extractLong(tmp, &slave->info[i].endTime.tv_sec);
    tmp = extractLong(tmp, &slave->info[i].endTime.tv_usec);

    if ( (slave->info[i].timer = (T_MonitorTimer*)malloc(sizeof(T_MonitorTimer)* listnum)) == NULL ) {
      goto done;
    }

    for ( j = 0; j < listnum; j++ ) {
      slave->info[i].timer[j].name = strdup(list[j]);
      tmp = extractDouble(tmp, &(slave->info[i].timer[j].r));
      tmp = extractDouble(tmp, &(slave->info[i].timer[j].u));
      tmp = extractDouble(tmp, &(slave->info[i].timer[j].s));
      fprintf(stderr,"==> %.4f\n",slave->info[i].timer[j].r);
    }
  }

  ret = 1;
  
 done:
  if ( list ) {
    free(list);
  }
  
  return ret;
}

int zkperfClient_MonitorWrite( ZkError err, ZkPerfClient c)
{
  int ret = 0;
  size_t len = 0;
  size_t out;
  char *tmp, *buff = NULL;
  const char **list;
  int listnum;
  int bsize;

  if ( ! zkperfClient_buildTimerList(err,c,1,&list, &listnum)) {
    goto done;
  }

  /* Compute size */
  if ( ! computeSize(err, c, list, listnum, &len) ) {
    goto done;
  }
  if ( (buff = (char*)malloc(sizeof(char)*len)) == NULL ) {
    goto done;
  }
  
  bsize = sizeof(char)*len;
  tmp = append(buff, &bsize, len, NULL);
  
  if ( ! zkSocketWrite(err, c->monitorSocket, buff, sizeof(int), &out) ) {
    goto done;
  }
  if ( out != sizeof(int) ) {
    goto done;
  }
  
  /* Serialize data */

  if ( ! serialize(err, c, buff, len, list, listnum ) ) {
    goto done;
  }
  
  if ( ! zkSocketWrite(err, c->monitorSocket, buff, len, &out) ) {
    goto done;
  }

  if ( out != len ) {
    goto done;
  }
  
  ret = 1;
  
 done:
  
  if ( buff ) {
    free(buff);
  }
  if ( list != NULL ) {
    free(list); /* Don't free individual list members */
  }
  
  return ret;
}


int zkperfClient_MonitorRead( ZkError err, T_MonitorSlave *ms )
{
  int ret = 0;
  int len,tmp;
  int sofar, res;
  char *buff = NULL;

  if ( !ms ) {
    return 0;
  }
  
  /* Read size */

  read(ms->fd, &tmp, 4);
  len = ntohl(tmp);

  /* Alloc buffer */
  if ( (buff = (char*)malloc(len*sizeof(char))) == NULL ) {
    goto done;
  }
  
  /* Read data */

  sofar = 0;
  while (sofar < len) {
    if ( ( res = read(ms->fd, buff+sofar, len-sofar)) <= 0 ) {
      goto done;
    }
    sofar += res;
  }

  if ( ! deserialize( err, buff, len, ms ) ) {
    goto done;
  }
  
  ret = 1;
  
 done:
  if ( buff ) {
    free(buff);
  }
  
  return ret;
}



int zkperfClient_MonitorSynch( ZkError err, ZkPerfClient c )
{
  char buff[100];
  int num;
    
  if ( ! zkSocketNew(err, AF_INET,
                     SOCK_STREAM, IPPROTO_IP, &(c->monitorSocket)) ) {
    return 0;
  }
  if ( !zkSocketSetOption(err, c->monitorSocket,
                          ZKSOCKET_OPTION_Timeout, ZKSOCKET_TIMEOUT_INFINITE) ) {
    return 0;
  }
  
  if ( ! zkSocketConnect( err, c->monitorSocket, c->monitorHost ) ) {
    return 0;
  }
  num = 0;
  if ( ! zkSocketRead( err, c->monitorSocket, buff, 100, &num) ) {
    return 0;
  }
  
  return 1;
}
