/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */


#include <stdio.h>
#include <unistd.h>
#include <assert.h>

#include "zkError.h"
#include "zkLog.h"
#include "zkperf.h"

typedef struct {
  ZkPerfClient c;
} Obj;

static int tfunc1( ZkError err, ZkPerfClientContext ctx)
{
  Obj *o = ctx->obj;
  
  /* remove some warnings */
  (void)err;
  zkperfClientTimerResume(err, o->c, "foo1");
  sleep(1);
  zkperfClientTimerPause(err, o->c, "foo1");
  sleep(1);

  return 1;
}

static int tfunc2( ZkError err, ZkPerfClientContext ctx )
{
  /* remove some warnings */
  (void)err;
  (void)ctx;
  
  sleep(1);
  
  return 1;
}

int main(void)
{
  int j,ret = 0;
  int useMonitor = 0;
  ZkError err;
  ZkLog log;
  ZkPerfClient c;
  ZkPerfFunction tfuncs[] = {
    {"func1", NULL, tfunc1, NULL, NULL, "main1", NULL, 0.4},
    {"func2", NULL, tfunc2, NULL, NULL, "main2", NULL, 0.6},    
  };
  const char *timers[] = { "foo1", NULL };
  Obj o;
  struct S_ZkPerfClientReport report;


  if( !zkLogNew(&log) ||
      !zkLogSetFile(log, "tclient.log") ||
      !zkLogSetProgname(log, "tclient") ||
      !zkLogEnable(log) ) {
    fprintf(stderr, "Log creation failure\n");
    return 0;
  }


  assert(zkErrorNew(&err));

  if ( ! zkperfClientNew(err, &c) ) {
    goto done;
  }

  if ( ! zkperfClientSetLog(err, c, log) ) {
    goto done;
  }
       
  o.c = c;
  if ( ! zkperfClientSetObject(err, c, &o) ) {
    goto done;
  }

  useMonitor = 0;
  if ( useMonitor ) {
    if ( ! zkperfClientSetMonitor(err, c, "localhost", "52000") ) {
      goto done;
    }
  }
  /*  
  if ( ! zkperfClientUseThreads( err, c ) ) {
    goto done;
  }
  */
  if ( ! zkperfClientSetFunctions( err, c, NULL, NULL, tfuncs,2 ) ) {
    goto done;
  }

  if ( ! zkperfClientTimerAdd( err, c, timers) ) {
    goto done;
  }

  for ( j = 0; j < 1; j++ ) {
    if ( ! zkperfClientSetRunParams( err, c, 2, /*(j+1)*5*/ 1 ) ) {
      goto done;
    }
    if ( ! zkperfClientStart(err, c) ) {
      zkLogWriteContext(log, err, "tclient", "zkperfClientStart failed");
      goto done;
    }

    if ( useMonitor ) {
      if ( ! zkperfClientReportToMonitor(err, c) ) {
        goto done;
      }
    }

    if ( ! zkperfClientReportAvg(err, c, stdout, (j==0)) ) {
      goto done;
    }

    if ( ! zkperfClientReportConcurrent(err, c, stdout, j==0) ) {
      goto done;
    }
            
    if ( ! zkperfClientReportTask(err, c, stdout, j==0) ) {
      goto done;
    }
  }

  if ( ! zkperfClientReportAvg(err, c, stdout, 1) ) {
    goto done;
  }

  if ( ! zkperfClientReport(err, c, "foo1", &report ) ) {
    goto done;
  }
  
  fprintf(stderr, "foo1: \n");
  fprintf(stderr, "Avg  real: %.4f user: %.4f sys: %.4f\n",report.realAvg,
          report.userAvg, report.sysAvg);

  ret = 1;
  
 done:
  
  if ( c && ! zkperfClientFree(err, &c) ) {
    return 0;
  }

  return ret;
}
