/*
** **************************************************************************
** $Id: zkPluginImpl.h,v 1.3 2001/04/19 15:44:41 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

**
** Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
** **************************************************************************
*/

# ifndef ZKPLUGINIMPL_H_
# define ZKPLUGINIMPL_H_

# ifndef ZKERROR_H_
# include <zkError.h>
# endif

# ifdef __cplusplus
extern "C" {
# endif


# define ZKPLUGIN_APIVERSION         ZKPLUGIN_APIVERSION_1

/*
 * This entry point must be implemented and exported by any module
 * wishing to allow itself to be used by the plugin API.
 *
 * This entry point must only be called once by each client of the plugin.
 * However, a plugin loaded into memory can expect that it may be called
 * several times by different processes.  This means that you are safe if you
 * don't store anything in static variables.  If you do, watch out, or switch
 * to using a context in the private data of the plugin_info.  
 */

typedef struct t_ZkPluginInfo * ZkPluginInfo;

DLLExport int
zkPluginGetInfo(
    ZkError                    err,
    uint32_t                   desired_version,
    ZkPluginInfo               plugin_info);




/* --------------------------------------------------------------------------
 * Implementation Details
 * --------------------------------------------------------------------------
 *
 * Ignore this stuff unless you're implementing a plugin.  Users of the
 * plugin API ... make use of this information at your own peril!  In
 * general assume that this information is subject to change without
 * notice.
 *
 */


struct t_ZkPluginInfo {
    const char *               name;           /* identifier of plugin */
    uint32_t                   version;        /* version id of plugin */
    uint32_t                   api_version;    /* api version used by plugin */
    const void *               vtable;         /* plugin's function table */
    void *                     private_data;   /* plugin specific data */
    void *                     object_handle;  /* opaque handle to DLL */
    const char *               conf_file_name; /* name of conf file  */
};


/* --------------------------------------------------------------------------
 * Version 1 Structures
 */

# define ZKPLUGIN_APIVERSION_1       (1)

typedef int (*ZkPluginDestructor_1)(
    ZkError                    err,
    struct t_ZkPluginInfo *    plugin_info );
    
typedef int (*ZkPluginDataFreeFunc_1)(
    ZkError                    err,
    void *                     plugin_data,
    void **                    data_to_free );

typedef int (*ZkPluginCommandHandler_1)(
    ZkError                    err,
    void *                     plugin_data,
    const char *               params[],
    size_t                     numparams,
    const void *               in_data,
    size_t                     in_data_size,
    void **                    out_data,
    size_t *                   out_data_size );

typedef struct t_ZkPluginVersion_1 {
    ZkPluginDestructor_1       destructor;
    ZkPluginDataFreeFunc_1     free_func;
    ZkPluginCommandHandler_1   handler_func;
} * ZkPlugin_1;

# ifdef __cplusplus
}
# endif

# endif /* ZKPLUGINIMPL_H_ */
