/*
** **************************************************************************
** $Id: plugin-test.c,v 1.2 2001/04/19 15:44:29 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

**
** Copyright (C) 1999 Zero-Knowledge Systems Inc.  All Rights Reserved
** **************************************************************************
*/

# include <zks.h>
# include <zkAlloc.h>
# include <stdio.h>
# include "zkPlugin.h"


static const char * PLUGIN_FILE = "./plugin.so";

void DumpError(const char *, ZkError);
void DoPluginStuff(ZkError, const char*[], int);

void
DumpError(const char * context, ZkError err)
{
    char message[1024];
    
    zkErrorMessage(err, message, sizeof(message)-1);
    
    fprintf(stderr, "%s: %s\n", context, message);
}

void
DoPluginStuff(ZkError err, const char * argv[], int argc)
{
    static const char * context        = "DoPluginStuff";
    char                in_data[]      = "This is the data to be sent";
    const char **       new_argv       = NULL;
    ZkPluginInfo        plugin         = NULL;
    uint32_t            plugin_version = 0;
    uint32_t            api_version    = 0;
    char *              out_data       = NULL;
    size_t              out_data_size  = 0;
    int                 result         = 0;
    int                 i;

    if ( zkMemAlloc(err, (void**)&new_argv, sizeof(char*) * argc-1)
         && zkPluginLoad(err, PLUGIN_FILE, &plugin)
         && zkPluginGetVersion(err, plugin, &plugin_version)
         && zkPluginGetApiVersion(err, plugin, &api_version) )
    {
        for (i = 1 ; i < argc; ++i)
        {
            new_argv[i-1] = argv[i];
        }

        printf( "Plugin Name    : %s\n"   \
                "Plugin Version : %d\n"   \
                "API Version    : %d\n\n" \
                "Call Arguments :\n",
                new_argv[0], plugin_version, api_version );

        for (i = 0; i < (argc-1); ++i)
        {
            printf("\t%s\n", new_argv[i]);
        }

        if ( zkPluginCommand(err, plugin, new_argv, argc-1,
                             in_data, sizeof(in_data),
                             (void**)&out_data, &out_data_size) )
        {

            printf( "\nThe function call was successful.  " \
                    "Here are the results:\n\nout_data_size: %d"
                    "\n------\n%s", out_data_size, out_data );
            result = 1;
        }
    }

    if ( !result )
    {
        DumpError(context, err);
    }

    if ( out_data && !zkPluginMemFree(err, plugin, (void**)&out_data) )
    {
        DumpError(context, err);
    }

    if ( plugin && !zkPluginUnload(err, &plugin) )
    {
        DumpError(context, err);
    }

    if ( new_argv && !zkMemFree(err, (void**)&new_argv) )
    {
        DumpError(context, err);
    }
}


int
main(int argc, const char *argv[])
{
    ZkError        err    = NULL;

    if (zkErrorNew(&err))
    {
        DoPluginStuff(err, argv, argc);
        zkErrorFree(&err);
    }
    else
    {
        fprintf(stderr,"%s:%s\n",argv[0],"couldn't construct error object");
        
    }

    return (0);
}

