/*
 *  $Id: zkrand.h,v 1.5 2001/04/19 16:48:43 alainc Exp $
 */

/* This library should now be used to generate random numbers.

   The old frRand functions are still available, but they should not be
   used in new code any more.

   libzkrand must be initialized before it an be used. If you also use
   frcrypt, the frCryptInit() function call will take care of this.
   If you don't use frcrypt, you can call zkRand_Init() instead.

   The second argument to the Init functions (the file name for the
   entropy file) is only used in the Windows client. On Linux it can be
   NULL.

   To clean up, call frCryptFinal() or zkRand_Final() respectively.

   zkRandGetData(), zkRandGetUInt32InRange() etc. work exactly in the same
   way as their frRand counterparts. */

#ifndef ZK_RAND_H_
#define ZK_RAND_H_

#include "openssl/crypto.h"
#ifndef HAVE_YARROW
# define HAVE_YARROW
#endif
#if OPENSSL_VERSION_NUMBER < 0x0090581fL
# undef HAVE_YARROW
#endif

#ifdef linux
# undef HAVE_YARROW
# define USE_DEVRANDOM
#endif

/*
 *
 *
 *                           Randomness routines.
 *
 *
 */

#include "zks.h"
#include "zkError.h"
#include "zkStateMachine.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY)
  extern DLLExport int zkRandEntropyAdd(ZkError err, void *data, size_t dsz);
  extern DLLExport int zkRandGetData(ZkError err, void *data, size_t dsz);
#endif	/* !defined(ZK_KERNEL_MAC_OS8_ASYNC_ONLY) */
#if !defined(ZK_KERNEL_MAC_OS8)
  extern DLLExport int zkRandGetUInt32InRange(ZkError err, uint32_t min, uint32_t max, uint32_t *rn);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */
  extern DLLExport void zkRand_Debug(void);
#if !defined(ZK_KERNEL_MAC_OS8)
  extern DLLExport int zkRandEntropyAddFromSource(ZkError err, int source_id, void *data, size_t dsz, unsigned int bits);
  /* *enough is set to 1 if the PRNG has enough entropy */
  extern DLLExport int zkRandEntropyGetCount(ZkError err, int *enough, int *num_sources, unsigned *source_id, size_t *entropy_bits, size_t *entropy_max);
  extern DLLExport int zkRandEntropyAddSource(ZkError err, unsigned *source);
#endif	/* !defined(ZK_KERNEL_MAC_OS8) */
  
  /* Kernel-space versions of some routines. */
  extern DLLExport int zkRandEntropyAddAsync(ZkStateMachine sm, void *data, size_t dsz);
  extern DLLExport int zkRandGetDataAsync(ZkStateMachine sm, void *data, size_t dsz);

  /*
   *  Internal Functions
   */
  
  extern DLLExport int zkRand_Init(ZkError err, const char *entropyfile);
  extern DLLExport int zkRand_Final(ZkError err);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* ZK_RAND_H_ */
