/*
 *  $Id: trnplot.c,v 1.2 2001/04/19 16:48:45 alainc Exp $
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 *  
 *  A simple visual test of numbers produced by zkRandGetUInt32InRange().
 *  
 *  To view the output, run gnuplot with a command like:
 *  
 *    plot "/tmp/trnplot.out" with dots
 */

#include "zks.h"

#include "zkErrno.h"
#include <stdio.h>
#include <stdlib.h>
#include "zkString.h"
#include "zkError.h"
#include "zkLog.h"
#include "zkrand.h"
#include "zkShell.h"

static char *prog;


static void usage(int ok)
{
  fprintf(stderr,
          "Usage: %s"
          " [-h | -help | --help]"
          " [--logFile <path>]"
          " [--max <n>]"
          " [--min <n>]"
          " [--plotFile <path>]"
          "\n", prog);
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
}


int main(int ac, char **av)
{
  const char *logFile = "output.txt";
  const char *plotFile = "trnplot.out";
  unsigned long i;
  uint32_t max = UINT32_MAX, min = 0;
  int ok = 0;
  unsigned long plotCount = 2000;
  FILE *fp;
  ZkError err;
  ZkLog log;

  zkShellGetProgramName(&ac, &av);

  prog = av[0];

  while (av++, --ac) {
    if (strcasecmp("-h",     av[0]) == 0 ||
        strcasecmp("-help",  av[0]) == 0 ||
        strcasecmp("--help", av[0]) == 0) {
      usage(1);
    } else if (strcasecmp("--logFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      logFile = av[0];
    } else if (strcasecmp("--max", av[0]) == 0) {
      unsigned long x;
      if (av++, --ac < 1) { usage(0); }
      if (sscanf(av[0], "%lu", &x) != 1) {
        fprintf(stderr, "%s: invalid unsigned integer `%s' -- exiting.\n",
                prog, av[0]);
        usage(0);
      }
      max = x;
    } else if (strcasecmp("--min", av[0]) == 0) {
      unsigned long x;
      if (av++, --ac < 1) { usage(0); }
      if (sscanf(av[0], "%lu", &x) != 1) {
        fprintf(stderr, "%s: invalid unsigned integer `%s' -- exiting.\n",
                prog, av[0]);
        usage(0);
      }
      min = x;
    } else if (strcasecmp("--plotCount", av[0]) == 0) {
      unsigned long x;
      if (av++, --ac < 1) { usage(0); }
      if (sscanf(av[0], "%lu", &x) != 1) {
        fprintf(stderr, "%s: invalid unsigned integer `%s' -- exiting.\n",
                prog, av[0]);
        usage(0);
      }
      plotCount = x;
    } else if (strcasecmp("--plotFile", av[0]) == 0) {
      if (av++, --ac < 1) { usage(0); }
      plotFile = av[0];
    } else {
      fprintf(stderr, "%s: invalid option `%s' -- exiting.\n",
              prog, av[0]);
      goto done;
    }
  }

  if ( ! (zkErrorNew(&err) &&
          zkErrorLog(err, &log) &&
          zkLogSetProgname(log, prog) &&
          zkLogSetFile(log, logFile) &&
          zkLogEnable(log))) {
    fprintf(stderr, "%s: can't set log object.", prog);
    goto done;
  }
  
  if ( ! (fp = fopen(plotFile, "w"))) {
    zkLogWrite_va(log, "error %d opening `%s': %s\n",
                  errno, plotFile, strerror(errno));
    goto done;
  }

  for (i = 0; i < plotCount; i++) {
    uint32_t a, b;
    
    /*seed it*/
	zkRand_Init(err, NULL);
	zkRand_Debug();

    if ( ! zkRandGetUInt32InRange(err, min, max, &a)) { goto done; }
    if ( ! zkRandGetUInt32InRange(err, min, max, &b)) { goto done; }
    
    if (fprintf(fp, "%lu %lu\n", (unsigned long)a, (unsigned long)b) < 0) {
      zkLogWrite_va(log, "error %d printing to `%s': %s\n",
                    errno, plotFile, strerror(errno));
      goto done;
    }
  }

  if (fclose(fp) == EOF) {
    zkLogWrite_va(log, "error %d closing `%s': %s\n",
                  errno, plotFile, strerror(errno));
    goto done;
  }

  ok = 1;

 done:
  
  exit(ok ? EXIT_SUCCESS : EXIT_FAILURE);
  return ok ? EXIT_SUCCESS : EXIT_FAILURE; /* keep Insure happy */
}
