/* 
 * $Id: zksexception.h,v 1.2 2001/04/19 14:44:17 alainc Exp $
 */
#ifndef ZKS_EXCEPTION_H
# define ZKS_EXCEPTION_H

/*
** Error hadling and ModuleId section 
*/
# define ZKS_EXCEPTION_MOD                        "CZKS::Exception"

# define ZKS_EXCEPTION_UNKNOWN_EXCEPTION_ERR      ((uint16_t) 0x0001)
# define ZKS_EXCEPTION_STD_EXCEPTION_ERR          ((uint16_t) 0x0002)
# define ZKS_EXCEPTION_STD_RUNTIME_ERROR_ERR      ((uint16_t) 0x0003)
# define ZKS_EXCEPTION_STD_LOGIC_ERROR_ERR        ((uint16_t) 0x0004)
# define ZKS_EXCEPTION_STD_DOMAIN_ERROR_ERR       ((uint16_t) 0x0005)
# define ZKS_EXCEPTION_STD_INVALID_ARGUMENT_ERR   ((uint16_t) 0x0006)
# define ZKS_EXCEPTION_STD_LENGTH_ERROR_ERR       ((uint16_t) 0x0007)
# define ZKS_EXCEPTION_STD_OUT_OF_RANGE_ERR       ((uint16_t) 0x0008)
# define ZKS_EXCEPTION_STD_RANGE_ERROR_ERR        ((uint16_t) 0x0009)
# define ZKS_EXCEPTION_STD_OVERFLOW_ERROR_ERR     ((uint16_t) 0x000A)
# define ZKS_EXCEPTION_STD_UNDERFLOW_ERROR_ERR    ((uint16_t) 0x000B)
# define ZKS_EXCEPTION_STD_BAD_EXCEPTION_ERR      ((uint16_t) 0x000C)
# define ZKS_EXCEPTION_STD_BAD_ALLOC_ERR          ((uint16_t) 0x000D)
# define ZKS_EXCEPTION_STD_BAD_TYPEID_ERR         ((uint16_t) 0x000E)
# define ZKS_EXCEPTION_STD_BAD_CAST_ERR           ((uint16_t) 0x000F)



# ifdef __cplusplus

#  include <exception>
#  include <string>
#  include <stdio.h>

#  include <zks.h>
#  include <zkIntTypes.h>
#  include <zkIntLimits.h>
#  include <zkTypes.h>
#  include <zkDefs.h>

#  include "zkserrdef.h"
#  include "zkserr.h"

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif


namespace CZKS {


  // =============================================================================
  // Error Handling Exception Handler
  //

  class Exception: public std::exception {
  private:
    //
    // Imbeded ZkError handler
    //
    class Error {
    public:
      Error();
      Error(const Error & err);      
      ~Error();

      Error(const char * module,
            uint16_t     errcode,
            const char * func,
            const char * mesg);
    
      Error &                 operator=(const Error & err);

      inline ZkError          zkError()       { return &m_zkerror; }
      inline const ZkError    zkError() const { return const_cast<const ZkError>(&m_zkerror); }

      inline ZksErr           zksErr()        { return ::zksErr(zkError()); }
      inline const ZksErr     zksErr() const  { return ::zksErr(zkError()); }
      
    private:
      struct T_ZkError        m_zkerror;
    };

  public:
    void                      Throw() const;

    static void               Throw(const char * module,
                                    uint16_t     errcode,
                                    const char * func, 
                                    const char * mesg);

    static void               Rethrow(const char * func, 
                                      const char * mesg);


  public:
    Exception();

    const char *              what() const;
   
    ZksErrCode                Code() const;

    const char *              Message() const;

    Exception &               Push(const char * func, 
                                   const char * mesg);

    bool                      Translate(const char     * CriteriaModule,
                                        uint16_t         CriteriaCode,
                                        const char     * toModule,
                                        uint16_t         toErrcode,
                                        const char     * toFunc, 
                                        const char     * toMesg);

    bool                      Cancel(const char     * CriteriaModule,
                                     uint16_t         CriteriaCode);


    void                      Clear();

    inline ZksErr             zksErr()        { return m_error.zksErr(); }
    inline const ZksErr       zksErr() const  { return m_error.zksErr(); }

    inline ZkError            zkError()       { return m_error.zkError(); }
    inline const ZkError      zkError() const { return m_error.zkError(); }

    bool                      isErrCode(const char * module, uint16_t code) const;
    bool                      isFromModule(const char * module) const;

    bool                      isUnknown() const;
    bool                      isStd(bool fatal) const;

  public:
    virtual ~Exception() {}

    friend  void              LogException(const Exception & ex);

  private:
    Exception(const char * module,
              uint16_t     errcode,
              const char * func, 
              const char * mesg);

  private:
    Error                     m_error;
  };



  // =============================================================================
  // Exported logging function
  //

  void LogException           (const Exception & ex);


  // =============================================================================
  // Exported Exception handling function
  //
  inline void Throw           (const Exception & ex);

  inline void Throw           (const char      * module, 
                               uint16_t          errcode,
                               const char      * func,
                               const char      * mesg,
                               ...) ZK_PRINTF_LIKE(4, 5);


  inline void Throw           (const std::string module, 
                               uint16_t          errcode,
                               const std::string func,
                               const std::string mesg);

  inline void Throw           (const char      * module, 
                               uint16_t          errcode);

  inline void Throw           (const std::string module, 
                               uint16_t          errcode);


  inline ZkBool ThrowIf       (ZkBool            test, 
                               const char      * module,                          
                               uint16_t          errcode,
                               const char      * func,
                               const char      * mesg,
                               ...) ZK_PRINTF_LIKE(5, 6);


  inline ZkBool ThrowIf       (ZkBool            test, 
                               const std::string module,                          
                               uint16_t          errcode,
                               const std::string func,
                               const std::string mesg);

  inline ZkBool ThrowIf       (ZkBool            test,
                               const Exception & ex);

  inline void Rethrow         (const char      * func,
                               const char      * mesg,
                               ...) ZK_PRINTF_LIKE(2, 3);

  inline void Rethrow         (const std::string func,
                               const std::string mesg);



  // =============================================================================
  // Utility functions use to invoke appropriate exception method
  //


  //
  // Append a message to an already setted exception
  //
  inline void Push(Exception  & ex,
                   const char * func,
                   const char * format,
                   ...)
    {
      try {
        char      buffer[ZKS_ERR_MESG_MAX+1] = "";
        ::va_list args;

        if( format ) {
          va_start(args, format);            
          ::vsnprintf(buffer, ZKS_ERR_MESG_MAX, format, args);         
          va_end(args);     
        }

        ex.Push(func, buffer);
      }
      catch( ... ) {
      }
    }


  //
  // std::string version
  //
  inline void Push(Exception        & ex,
                   const std::string  func,
                   const std::string  mesg)
    {
      try {
        ex.Push(func.c_str(), mesg.c_str());
      }
      catch( ... ) {
      }
    }



  //
  // Throw an Already set Exception
  //
  inline void Throw(const Exception & ex)
    {
      try {
        ex.Throw();
      }
      catch( ... ) {
        throw;
      }
    }



  //
  // Throw an Exception if 'test' is true
  //
  // (follow an non-OO style to make programmers happy
  // without having to use evil elipses method into the
  // class it self)
  //
  inline void Throw(const char * module, 
                    uint16_t     errcode,
                    const char * func,
                    const char * format,
                    ...)
    {
      try {
        char      buffer[ZKS_ERR_MESG_MAX+1] = "";
        ::va_list args;

        if( format ) {
          va_start(args, format);            
          ::vsnprintf(buffer, ZKS_ERR_MESG_MAX, format, args);         
          va_end(args);     
        }

        Exception::Throw(module, errcode, func, buffer);
      }
      catch( ... ) {
        throw;
      }
    }

  inline void Throw(const char * module, 
                    uint16_t     errcode)
    {
      try {
        Exception::Throw(module, errcode, "", "");
      }
      catch( ... ) {
        throw;
      }
    }
 
 

  //
  // std::string version
  //
  inline void Throw(const std::string module, 
                    uint16_t          errcode,
                    const std::string func,
                    const std::string mesg)
    {
      try {
        Exception::Throw(module.c_str(), errcode, func.c_str(), mesg.c_str());
      }
      catch( ... ) {
        throw;
      }
    }

  inline void Throw(const std::string module, 
                    uint16_t          errcode)
    {
      try {
        Exception::Throw(module.c_str(), errcode, "", "");
      }
      catch( ... ) {
        throw;
      }
    }




  //
  // Test and throw function use to light the code
  // 
  // -Throw a CZKS::Exception if 'test' is true
  //
  // (follow an non-OO style to make programmers happy
  // without having to use evil elipses method into the
  // class it self)
  //
  inline ZkBool ThrowIf(ZkBool       test, 
                        const char * module,                          
                        uint16_t     errcode,
                        const char * func,
                        const char * format,
                        ...)
    {
      try {
        char      buffer[ZKS_ERR_MESG_MAX+1] = "";
        ::va_list args;

        if( format ) {
          va_start(args, format);            
          ::vsnprintf(buffer, ZKS_ERR_MESG_MAX, format, args);         
          va_end(args); 
        }    

        if( test ) {
          Exception::Throw(module, errcode, func, buffer);
        }
      }
      catch( ... ) {
        throw;
      }
      return ZK_TRUE;
    }

  //
  // std::string version
  //
  inline ZkBool ThrowIf(ZkBool            test, 
                        const std::string module,                          
                        uint16_t          errcode,
                        const std::string func,
                        const std::string mesg)
    {
      if( test ) {
        Throw(module.c_str(), errcode, func.c_str(), mesg.c_str());
      }
      return ZK_TRUE;
    }


  //
  // Throw a CZKS::Exception if 'test' is true
  ///
  inline ZkBool ThrowIf(ZkBool            test,
                        const Exception & ex)
    {      
      try {
        if( test ) ex.Throw();
      }
      catch( ... ) {
        throw;
      }
      return ZK_TRUE;
    }





  //
  // Append a message to the current exception than rethrow it.
  // Also catch standard exception and convert them into Exception.
  //
  // i.e.: try {
  //           .... operation A ...
  //       }
  //       catch( ... ) {
  //           CZKS::Rethrow("my function", "operation A failed");
  //       }
  //
  // (follow an non-OO style to make programmers happy
  // without having to use evil elipses method into the
  // class it self)
  //
  inline void Rethrow(const char * func,
                      const char * format,
                      ...)
    {
      try {
        char      buffer[ZKS_ERR_MESG_MAX+1] = "";
        ::va_list args;

        if( format ) {
          va_start(args, format);            
          ::vsnprintf(buffer, ZKS_ERR_MESG_MAX, format, args);         
          va_end(args); 
        }
    
        Exception::Rethrow(func, buffer);
      }
      catch( ... ) {
        throw;
      }
    }


  //
  // std::string version
  //
  inline void Rethrow(const std::string func,
                      const std::string mesg)
    {
      try {
        Exception::Rethrow(func.c_str(), mesg.c_str());
      }
      catch( ... ) {
        throw;
      }
    }

} // namespace CZKS

# endif /* __cplusplus */
#endif /* ZKS_EXCEPTION_H */
