/*
 * $Id: zkslog.h,v 1.2 2001/04/19 14:44:17 alainc Exp $
 */
#ifndef ZKS_LOG_H
# define ZKS_LOG_H

# include <zks.h>
# include <zkIntTypes.h>
# include <zkIntLimits.h>
# include <zkTypes.h>
# include <zkDefs.h>

# include "zksmodule.h"
# include "zkslogmng.h"


/*
 * Error hadling and ModuleId section
 */
# define ZKS_LOG_MOD                         "ZksLog"


/*
 * Log mask bitmask
 */
# define ZKS_LOG_NONE_MASK                  ((ZksLogMask)0x00000000) /* log nothing */
# define ZKS_LOG_TRACE_MASK                 ((ZksLogMask)0x00000001) /* log user trace */
# define ZKS_LOG_MESG_MASK                  ((ZksLogMask)0x00000002) /* log user messages */
# define ZKS_LOG_ARGS_MASK                  ((ZksLogMask)0x00000004) /* log program argument */
# define ZKS_LOG_CONFIG_MASK                ((ZksLogMask)0x00000008) /* log configuration setting */ 
# define ZKS_LOG_PARAM_MASK                 ((ZksLogMask)0x00000010) /* log parameters */ 
# define ZKS_LOG_LOCAL_MASK                 ((ZksLogMask)0x00000020) /* log local variable */ 
# define ZKS_LOG_GLOBAL_MASK                ((ZksLogMask)0x00000040) /* log global variable */ 
# define ZKS_LOG_THIS_MASK                  ((ZksLogMask)0x00000080) /* log 'this' pointer (c++) */ 
# define ZKS_LOG_STEP_MASK                  ((ZksLogMask)0x00000100) /* log 'step' tracing messages */ 
# define ZKS_LOG_DATA_MASK                  ((ZksLogMask)0x00000200) /* log transaction data */
# define ZKS_LOG_REJECT_MASK                ((ZksLogMask)0x00000400) /* log transaction reject */
# define ZKS_LOG_ANY_MASK                   ((ZksLogMask)0xffffffff) /* log everything */


/*
 * Tags definition
 */
# define ZKS_LOG_ERROR_DEFAULT_MASK         (ZKS_LOG_ANY_MASK)
# define ZKS_LOG_MESG_DEFAULT_MASK          (ZKS_LOG_ANY_MASK)
# define ZKS_LOG_DEBUG_DEFAULT_MASK         (ZKS_LOG_ANY_MASK)


/*
 * Logging Type definition
 */
#define ZKS_LOG_TYPE_ERROR                  (LOG_ERROR)
#define ZKS_LOG_TYPE_MESSAGE                (LOG_MESG)
#define ZKS_LOG_TYPE_DEBUG                  (LOG_DEBUG)


/*
 * Macro use to NOP Trace/Debug logging into production code
 */
# if !defined(FREEDOM_PRODUCTION_BUILD) && defined(FREEDOM_DEVELOPMENT_BUILD)
#  define ZKS_LOGDEBUG_0(mask, module, mesg)                  zksLogDebug(mask, module, mesg)
#  define ZKS_LOGDEBUG_1(mask, module, mesg, a1)              zksLogDebug(mask, module, mesg, a1)
#  define ZKS_LOGDEBUG_2(mask, module, mesg, a1, a2)          zksLogDebug(mask, module, mesg, a1, a2)
#  define ZKS_LOGDEBUG_3(mask, module, mesg, a1, a2, a3)      zksLogDebug(mask, module, mesg, a1, a2, a3)
#  define ZKS_LOGDEBUG_4(mask, module, mesg, a1, a2, a3, a4)  zksLogDebug(mask, module, mesg, a1, a2, a3, a4)
#  define ZKS_LOGDEBUG_5(mask, module, mesg, a1, a2, a3, a4)  zksLogDebug(mask, module, mesg, a1, a2, a3, a4, a5);
#  define ZKS_LOGDEBUG_6(mask, module, mesg, a1, a2, a3, a4)  zksLogDebug(mask, module, mesg, a1, a2, a3, a4, a5, a6);
#  define ZKS_LOGDEBUG_7(mask, module, mesg, a1, a2, a3, a4)  zksLogDebug(mask, module, mesg, a1, a2, a3, a4, a5, a6, a7);
#  define ZKS_LOGDEBUG_8(mask, module, mesg, a1, a2, a3, a4)  zksLogDebug(mask, module, mesg, a1, a2, a3, a4, a5, a6, a7, a8);

# else
#  define ZKS_LOGDEBUG_0(mask, module, mesg)
#  define ZKS_LOGDEBUG_1(mask, module, mesg, a1)
#  define ZKS_LOGDEBUG_2(mask, module, mesg, a1, a2)
#  define ZKS_LOGDEBUG_3(mask, module, mesg, a1, a2, a3)
#  define ZKS_LOGDEBUG_4(mask, module, mesg, a1, a2, a3, a4)
#  define ZKS_LOGDEBUG_5(mask, module, mesg, a1, a2, a3, a4, a5);
#  define ZKS_LOGDEBUG_6(mask, module, mesg, a1, a2, a3, a4, a5, a6);
#  define ZKS_LOGDEBUG_7(mask, module, mesg, a1, a2, a3, a4, a5, a6, a7);
#  define ZKS_LOGDEBUG_8(mask, module, mesg, a1, a2, a3, a4, a5, a6, a7, a8);
# endif


/*
 * Macro use to NOP Trace/Debug logging into production code
 */
# if !defined(FREEDOM_PRODUCTION_BUILD) && defined(FREEDOM_DEVELOPMENT_BUILD)
#  if defined __STDC__ && __STDC__
#   define TEXT(x)    #x
#  else
#   define TEXT(x)    "x"
#  endif 

#   define ZKS_ASSERT(module, expr) ((void) ((expr) ? 0 :	\
                                    (zksLogMesg(module, "file %s, line %d: %s failed", \
                                    __FILE__, __LINE__, TEXT(expr)), abort(), 0)))
# else
#   define ZKS_ASSERT(module, expr) ((void) 0)
# endif



# ifdef __cplusplus
extern "C" {
# endif


  /* =========================================================================
  ** zksLog API
  */


  ZkBool zksLogInit                          (const char     * appname);

  ZkBool zksLogInitEx                        (const char     * appname,
                                              const char     * hostname);

  ZkBool zksLogShutdown                      (void);

  ZkBool zksLogError                         (ZksErr           err);

  ZkBool zksLogMesg                          (const char     * module,
                                              const char     * mesg,
                                              ...);

  ZkBool zksLogDebug                         (ZksLogMask       mask,
                                              const char     * module,
                                              const char     * mesg,
                                              ...);

  ZkBool zksLogDefaultAdd                    (ZksLogAgent      agent);

  ZkBool zksLogDefaultClear                  (void);

  ZkBool zksLogDefaultDebugMaskGet           (ZksLogMask     * maskP);

  ZkBool zksLogDefaultDebugMaskSet           (ZksLogMask       mask);

  ZkBool zksLogSpecificAdd                   (const char     * module,
                                              ZksLogAgent      agent);

  ZkBool zksLogSpecificClear                 (const char     * module);

  ZkBool zksLogSpecificDebugMaskGet          (const char     * module, 
                                              ZksLogMask     * maskP);
    
  ZkBool zksLogSpecificDebugMaskSet          (const char     * module,
                                              ZksLogMask       mask);


  /* =========================================================================
  ** zksLog terminal handling API
  */

  ZkBool zksLogTermInit                      (void);


  ZkBool zksLogTermEnable                    (void);

  ZkBool zksLogTermDisable                   (void);

  ZkBool zksLogTermSetState                  (ZkBool           state);

  ZkBool zksLogTermGetState                  (ZkBool         * stateP);

  ZkBool zksLogTermSet                       (const ZksLogAgent term);

  ZkBool zksLogTermGet                       (ZksLogAgent    * termP);


  /* =========================================================================
  ** zksLog advance API
  */

  ZkBool zksLogLock                          (void);

  ZkBool zksLogUnlock                        (void);

  ZkBool zksLogWriteMesg                     (ZksLogType       type,
                                              ZksLogMask       mask,
                                              const char     * module,
                                              struct timeval   logtime,
                                              const T_ZksPid * pid,
                                              const char     * mesg);

  ZkBool zksLogWriteEntry                    (ZksLogType       type,
                                              ZksLogMask       mask,
                                              void           * userdata,
                                              ZkBool        (* getEntryFnc) (void *, ZksLogEntry *),
                                              ZkBool        (* emptyFnc)    (void *, ZkBool *));

# ifdef __cplusplus
}
# endif

#endif /* ZKS_LOG_H */

