/* 
 * $Id: zkspath.h,v 1.2 2001/04/19 14:44:17 alainc Exp $
 */
#ifndef ZKS_PATH_H
# define ZKS_PATH_H

/*
 * C++ declarations
 */
# ifdef __cplusplus
 
# ifdef WIN32
#  pragma warning( disable: 4786 )
# endif

# if defined( WIN32 )
#  define OS_PATH_SEPARATOR              '\\'

# elif defined( ZK_LINUX )
#  define OS_PATH_SEPARATOR              '/'
# endif

# include <string>
# include <deque>

namespace CZKS {

  class Path {
  public:
    typedef std::deque<std::string> split_list;

  public:
    //
    // Constructor
    //
    Path(const std::string & pathname,
         char                sep = OS_PATH_SEPARATOR);

    Path(char sep = OS_PATH_SEPARATOR);

    Path(const Path & p);

    //
    // Assignment operator
    //
    Path & operator=(const Path & p);
    Path & operator=(const std::string & pathname);

    //
    // Arithmetic operator
    //
    Path & operator+=(const Path & p);
    Path   operator+ (const Path & p) const;
    Path & operator+=(const std::string & s);
    Path   operator+ (const std::string & s) const;

    //
    // Logical opertator
    //
    bool   operator==(const Path & p) const;
    bool   operator==(const std::string & s) const;

    //
    // direct members access
    //
    char                separator() const { return m_separator; }
    const std::string & pathname() const  { return m_pathname; }

    //
    // tools methods
    //
    std::string         basename() const;
    std::string         dirname()  const;

    bool                relative() const { return !absolute(); }
    bool                absolute() const;

    bool                empty()   const  { return m_pathname.empty(); }
    void                clear()          { m_pathname.clear(); }

    Path              & reduce();

    split_list          split() const;

  private:
    std::string m_pathname;
    char        m_separator;
  };

} // namespace CZKS


# endif /* __cplusplus */

#endif /* ZKS_PATH_H */
