/* 
 * $Id: zkstimeval.h,v 1.2 2001/04/23 13:44:30 alainc Exp $
 */
#ifndef ZKS_TIMEVAL_H
# define ZKS_TIMEVAL_H

# ifdef WIN32
#  pragma warning(disable: 4786)
# endif

/*
 * C++ declarations
 */
# ifdef __cplusplus  

#  include "zks.h" 
#  include "zkIntTypes.h"

#  include <sys/time.h>
#  include <sys/types.h>
#  include <unistd.h>


namespace CZKS {

  class Timeval {
  public:
    enum Type {
      NUL,      // Nul time
      INFINITE, // Infinite time
      NOW       // Current time
    };

  public:
    Timeval(Type type = INFINITE);
    Timeval(const Timeval & tv);
    Timeval(uint32_t sec, uint32_t usec); 

    Timeval & operator= (const Timeval & t);

    Timeval & set(Type type);
    Timeval & set(const Timeval & tv);
    Timeval & set(uint32_t sec, uint32_t usec);

    const struct timeval * tvP() const { return (m_infinite ? 0 : &m_tm); }
    struct       timeval * tvP()       { return (m_infinite ? 0 : &m_tm); }

    Timeval & operator-=(const Timeval & t);
    Timeval & operator+=(const Timeval & t);

    Timeval   operator- (const Timeval & t) const;
    Timeval   operator+ (const Timeval & t) const;

    bool      operator< (const Timeval & t) const;
    bool      operator> (const Timeval & t) const;
    bool      operator<=(const Timeval & t) const;
    bool      operator>=(const Timeval & t) const;
    bool      operator==(const Timeval & t) const;

    bool      isNul()      const;
    bool      isInfinite() const;
    bool      isFinite()   const;

  private:
    bool            m_infinite;
    struct timeval  m_tm;
  };

} // namespace CZKS

# endif /* __cplusplus */

#endif /* ZKS_TIMEVAL_H */
