/* 
 * $Id: zkslogagent.c,v 1.2 2001/04/19 14:44:21 alainc Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
# include <io.h>
#endif 

#include <zkTypes.h>
#include "zkslogagent.h"
#include "zkserr.h"
#include "zkslogmng.h"


/*
 * Function prototypes.
 */

/*  static ZkBool  logAgentXXXInit     (void * userdata) */
/*  static ZkBool  logAgentXXXShutdown (void * userdata) */
/*  static ZkBool  logAgentXXXPreLog   (void * userdata, ZksLogType type) */
/*  static ZkBool  logAgentXXXLog      (void * userdata, ZksLogType type, const char * mesg, uint32_t mesgsz) */
/*  static ZkBool  logAgentXXXPostLog  (void * userdata, ZksLogType type) */



  /* =========================================================================  
  ** Term zksLogAgent API
  */

/*
 *
 */
static ZkBool  logAgentTermLog(void       * userdata, 
                               ZksLogType   type, 
                               const char * mesg, 
                               uint32_t     mesgsz)
{
  ZkBool success = ZKS_SUCCESS;  
  FILE  *output  = NULL;

  (void) userdata;
  (void) mesgsz;

  switch( type )
    {
    case LOG_ERROR: output = stderr; break;
    case LOG_MESG:  
    case LOG_DEBUG: output = stdout; break;
    default:        success = ZKS_FAILURE;
    }

  if( success ) {
    fwrite(mesg, mesgsz, 1, output);
    fwrite("\n", sizeof("\n"), 1, output);
    fflush(output);
  }
  return success;
}


/*
 *
 */
ZkBool zksLogAgentTermNew(ZksErr        err,
                          ZksLogAgent * agentP)
{
  DECL_FUNC("zksLogAgentTermNew");
  ZkBool success = ZKS_SUCCESS;

  if( !zksLogAgentNew(agentP, 
                      NULL, 
                      NULL, 
                      NULL, 
                      NULL, 
                      NULL, 
                      logAgentTermLog,
                      NULL) ) {
    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR,
                        func, "zksLogAgentNew failed");
  }
  return success;  
}



/*
 *
 */
ZkBool zksLogAgentTermFree(ZksErr        err,
                           ZksLogAgent * agentP)
{
  DECL_FUNC("zksLogAgentTermFree");
  ZkBool success = ZKS_SUCCESS;

  if( !zksLogAgentFree(agentP) ) {
    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR,
                        func, "zksLogAgentFree failed");
  }
  return success;
}




/*
 *
 */
ZkBool zksLogAgentNulNew(ZksErr        err,
                         ZksLogAgent * agentP)
{
  DECL_FUNC("zksLogAgentNulNew");
  ZkBool success = ZKS_SUCCESS;

  if( !zksLogAgentNew(agentP, 
                      NULL, 
                      NULL, 
                      NULL, 
                      NULL, 
                      NULL, 
                      NULL,
                      NULL) ) {
    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR,
                        func, "zksLogAgentNew failed");
  }
  return success;  
}



/*
 *
 */
ZkBool zksLogAgentNulFree(ZksErr        err,
                          ZksLogAgent * agentP)
{
  DECL_FUNC("zksLogAgentNulFree");
  ZkBool success = ZKS_SUCCESS;

  if( !zksLogAgentFree(agentP) ) {
    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR,
                        func, "zksLogAgentFree failed");
  }
  return success;
}



/* =========================================================================  
** File zksLogAgent API
*/

typedef struct S_LogAgentFileCtx {
  char              * filename;
  FILE              * file;
  ZksLogAgentFilePerm perm;
} * LogAgentFileCtx;


/*
 *
 */
static ZkBool logAgentFileInit(void * userdata)
{
  LogAgentFileCtx ctx       = (LogAgentFileCtx)userdata;
  ZkBool          success   = ZKS_SUCCESS;  
  mode_t          mask      = 0;
  mode_t          old_mask  = 0;

  if( ctx->file ) {
    success = ZKS_FAILURE;

  } else if( ctx->perm ) {
#if defined( ZK_LINUX )
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_USER_READ)   ? S_IRUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_USER_WRITE)  ? S_IWUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_GROUP_READ)  ? S_IRGRP : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_GROUP_WRITE) ? S_IWGRP : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_OTHER_READ)  ? S_IROTH : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_OTHER_WRITE) ? S_IWOTH : 0;
#elif defined( WINDOWS )
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_USER_READ)   ? S_IRUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_USER_WRITE)  ? S_IWUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_GROUP_READ)  ? S_IRUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_GROUP_WRITE) ? S_IWUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_OTHER_READ)  ? S_IRUSR : 0;
    mask |= (ctx->perm & ZKS_LOGAGENT_FILEPERM_OTHER_WRITE) ? S_IWUSR : 0;
#endif

    /* Set permission mask */
    old_mask = umask(~mask);

    /* Create/Open the file  */
    if( (ctx->file = fopen(ctx->filename, "a")) == NULL ) {
      success = ZKS_FAILURE;
    }

    /* Reset the user mask to it's previous value */
    umask(old_mask);
  }

  return success;
}



/*
 *
 */
static ZkBool logAgentFileShutdown(void * userdataP)
{
  ZkBool success = ZKS_SUCCESS;  

  if( userdataP ) { 
    LogAgentFileCtx ctx = (LogAgentFileCtx)userdataP;

    if( ctx && ctx->file ) { 
      fclose(ctx->file); 
      ctx->file = NULL;      
    }
  }
  return success;
}


/*
 *
 */
static ZkBool logAgentFileReset(void * userdataP)
{
  ZkBool          success   = ZKS_SUCCESS;  
  LogAgentFileCtx ctx       = (LogAgentFileCtx)userdataP;

  if( ctx->file ) {
    success = ZKS_FAILURE;

  } else {
    if( (success = logAgentFileShutdown(userdataP)) ) { /* Close the original file */
      success = logAgentFileInit(userdataP);
    }
  }
  return success;
}



/*
 *
 */
static ZkBool logAgentFileLog(void       * userdata, 
                              ZksLogType   type, 
                              const char * mesg, 
                              uint32_t     mesgsz)
{
  LogAgentFileCtx ctx = (LogAgentFileCtx)userdata;

  (void)type;
  (void)mesgsz;

  if( !ctx->file ) { return ZKS_FAILURE; }

  fprintf(ctx->file, mesg);
  fprintf(ctx->file, "\n");

  fflush(ctx->file);

  return ZKS_SUCCESS;
}




/*
 *
 */
ZkBool zksLogAgentFileNew(ZksErr              err,
                          ZksLogAgent       * agentP, 
                          const char        * filename,
                          ZksLogAgentFilePerm perm)
{
  DECL_FUNC("zksLogAgentFileNew");
  ZkBool          success    = ZKS_SUCCESS;
  LogAgentFileCtx ctx       = NULL;
  char          * tfilename = NULL;

  if( !agentP || !filename ) {

    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_BADPTR_ERR, 
                        func, "invalid input parameter(null)");

  } else if( (ctx = (LogAgentFileCtx) calloc(1, sizeof(struct S_LogAgentFileCtx) )) == NULL ) {

    success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_ALLOC_ERR, 
                        func, "calloc failed");

  } else if( (tfilename = strdup(filename)) == NULL ) {

    success = zksErrSet(err, ZKS_LOGAGENT_MOD,  ZKS_LOGAGENT_ALLOC_ERR,
                        func, "strdup failed");

  } else {
    ctx->filename = tfilename;
    ctx->file     = NULL;
    ctx->perm     = perm;

    if( !zksLogAgentNew(agentP,                              
                        (void *)ctx,
                        logAgentFileInit,
                        logAgentFileShutdown,
                        logAgentFileReset,
                        NULL,
                        logAgentFileLog,
                        NULL) ) {

      success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR, 
                          func, "zksLogAgentNew failed");
    }
  }

  if( !success ) {
    if( tfilename ) free(tfilename);
    if( ctx )       free(ctx);    
  }

  return success;
}




/*
 *
 */
ZkBool zksLogAgentFileFree(ZksErr        err,
                           ZksLogAgent * agentP)
{
  DECL_FUNC("zksLogAgentFileFree");
  ZkBool success = ZKS_SUCCESS;

  if( agentP ) {
    LogAgentFileCtx ctx = NULL;

    if( !zksLogAgentUserDataGet(*agentP, (void **) &ctx) ) {  

      success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR, func, "zksLogAgentUserDataGet failed");

    } else if( !zksLogAgentFree(agentP) ) {

      success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR, func, "zksLogAgentFree failed");

    } else if( ctx ) {
      if( ctx->filename ) { 
        free(ctx->filename); 
        ctx->filename = NULL;
      }      
      free(ctx);    
    }
  }
  return success;
}



ZkBool zksLogAgentFileReset(ZksErr              err,
                            ZksLogAgent         agent)
{
  DECL_FUNC("zksLogAgentFileReset");
  ZkBool success = ZKS_SUCCESS;

  if( agent ) {
    if( !zksLogAgentReset(agent) ) {
      success = zksErrSet(err, ZKS_LOGAGENT_MOD, ZKS_LOGAGENT_LOGMNG_ERR, func, "zksLogAgentReset failed");
    }
  }
  return success;
}
