#define REV_ID "@(#) $Id: zkstimeval.cpp,v 1.2 2001/04/23 13:44:40 alainc Exp $"
#include "zkrev.h"

#include "zkstimeval.h"

#include "zks.h" 
#include "zkIntTypes.h"

#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


namespace CZKS {

  //
  // Infinite time ctor
  //
  Timeval::Timeval(Type type)
  {
    set(type);
  }


  //
  // Finite time ctor
  //
  Timeval::Timeval(uint32_t sec, uint32_t usec)
  {
    set(sec, usec);
  }

  //
  // Copy ctor
  //
  Timeval::Timeval(const Timeval & tv)
  {
    (*this) = tv;    
  }

  //
  // Copy operator
  //
  Timeval & Timeval::operator=(const Timeval & tv)
  {
    if( this != &tv ) {
      m_infinite = tv.m_infinite;
      m_tm       = tv.m_tm;
    }
    return *this;
  }


  //
  //
  //
  Timeval & Timeval::set(Type type)
  {
    switch( type )
      {
      case NUL:       
        m_infinite = false; 
        timerclear(&m_tm);
        break;

      case INFINITE:  
        m_infinite = true; 
        timerclear(&m_tm);
        break;

      case NOW:       
        m_infinite = false; 
        ::gettimeofday(&m_tm, 0);  
        break;
      }    
    return *this;
  }


  //
  //
  //
  Timeval & Timeval::set(uint32_t sec, uint32_t usec)
  {
    m_infinite   = false;
    m_tm.tv_sec  = sec;
    m_tm.tv_usec = usec;
    return *this;
  }

  //
  //
  //
  Timeval & Timeval::set(const Timeval & tv)
  {
    (*this) = tv;
    return *this;
  }


  //
  //
  //
  Timeval & Timeval::operator-=(const Timeval & t)
  {
    if( tvP() && t.tvP() ) {
      timersub(&Timeval(*this).m_tm, &t.m_tm, &m_tm);

    } else if( tvP() && !t.tvP() ) {
      timerclear(&m_tm);
    }
    return *this;
  }


  //
  //
  //
  Timeval Timeval::operator-(const Timeval & t) const
  {
    return (Timeval(*this) -= t);
  }


  //
  //
  //
  Timeval & Timeval::operator+=(const Timeval & t)
  {
    if( tvP() && t.tvP() ) {
      timeradd(&Timeval(*this).m_tm, &t.m_tm, &m_tm);

    } else if( tvP() && !t.tvP() ) {
      m_infinite = true;
    }
    return *this;
  }


  //
  //
  //
  Timeval Timeval::operator+(const Timeval & t) const
  {
    return (Timeval(*this) += t);
  }


  //
  //
  //
  bool Timeval::operator<(const Timeval & t) const
  {
    if( tvP() && t.tvP() ) {
      return timercmp(&m_tm, &t.m_tm, <);

    } else if( tvP() && !t.tvP() ) {
      return true;
    } 
    return false;
  }


  //
  //
  //
  bool Timeval::operator>(const Timeval & t) const
  {
    if( tvP() && t.tvP() ) {
      return timercmp(&m_tm, &t.m_tm, >);

    } else if( tvP() && !t.tvP() ) {
      return false;
    } 
    return true;
  }


  //
  //
  //
  bool Timeval::operator<=(const Timeval & t) const
  {
    if( tvP() && t.tvP() ) {
      return timercmp(&m_tm, &t.m_tm, <=);

    } else if( tvP() && !t.tvP() ) {
      return true;
    } 
    return false;
  }


  //
  //
  //
  bool Timeval::operator>=(const Timeval & t) const
  {
    if( tvP() && t.tvP() ) {
      return timercmp(&m_tm, &t.m_tm, >=);

    } else if( tvP() && !t.tvP() ) {
      return false;
    } 
    return true;
  }


  //
  //
  //
  bool Timeval::operator==(const Timeval & t) const
  {
    if( tvP() && t.tvP() ) {
      return timercmp(&m_tm, &t.m_tm, ==);

    } else if( !tvP() && !t.tvP() ) {
      return true;
    } 
    return false;
  }


  //
  //
  //
  bool Timeval::isNul() const 
  { 
    return (!m_infinite && !timerisset(&m_tm)) ? true : false;
  }


  //
  //
  //
  bool Timeval::isInfinite() const
  {
    return (m_infinite) ? true : false;
  }


  //
  //
  //
  bool Timeval::isFinite() const
  {
    return (!m_infinite && timerisset(&m_tm)) ? true : false;
  }


} // namespace CZKS
