#include "zksexception.h"
#include <stdexcept>
#include <iostream>
#include <errno.h>
#include "zkslog.h"
#include "zkspath.h"



void CallD(int test)
{
  try{
    switch( test ) {
    case 0: CZKS::Throw("mod 1", 1, "CallD", "failure 0");
    case 1: throw std::out_of_range("tata");    
    }
  } 
  catch( ... ) {
    CZKS::Rethrow("CallD", "failure 1");
  }
}

void CallC(int test)
{
  try{
    CallD(test);
  } 
  catch( ... ) {
    CZKS::Rethrow("CallC", "failure 2");
  }
}

void CallB(int test)
{
  try{
    CallC(test);
  } 
  catch( ... ) {
    CZKS::Rethrow("CallB", "failure 3");
  } 
}

void CallA(int test)
{
  try{
    CallB(test);
  } 
  catch( ... ) {
    CZKS::Rethrow("CallA", "failure 4");
  }
}


static void zkErrorTest(CZKS::Exception & e)
{
  ZkError zerr = e.zkError();

  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test zkErrorTest 1\n\n");

  ::zkErrorSet(zerr, "zkLogTest", ZKERR, ZKS_MODULE_ID_BASE, 2, "test the encapsulation");
  
  CZKS::LogException(e);  


  errno = 0;

  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test zkErrorTest 2\n\n");

  e.Clear();
  ::zkErrorSystemSet(zerr, "zkLogTest", ZKERR, ZKS_MODULE_ID_BASE, "test the syserror");
  
  CZKS::LogException(e);  

  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test zkErrorTest 2\n\n");

  errno = 26;

  e.Clear();
  ::zkErrorSystemSet(zerr, "zkLogTest", ZKERR, ZKS_MODULE_ID_BASE, "test throw with errno set");

  try {
    CZKS::Throw(e);
  }
  catch( CZKS::Exception & ex ) {
    CZKS::LogException(ex);
  }  
}



int main(int, char * argv[])
{
  std::cout << "==============================================\n";
  std::cout << "Test exception stacking\n\n";


  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
    return 1;
  }

  try {
    std::cout << "==============================================\n";
    std::cout << "Test exception stacking #1\n\n";
    CallA(1);
    std::cout << "Test failed\n\n";
  }
  catch( CZKS::Exception ex ) {
    CZKS::LogException(ex);
    std::cout << "Test ok\n\n";
  }

  try {
    std::cout << "==============================================\n";
    std::cout << "Test exception stacking #0\n\n";
    CallA(0);
    std::cout << "Test failed\n\n";
  }
  catch( CZKS::Exception ex ) {
    CZKS::LogException(ex);
    std::cout << "Test ok\n\n";
  }

  try {
    std::cout << "==============================================\n";
    std::cout << "Test exception stacking #2\n\n";
    CallA(2);
    std::cout << "Test ok\n\n";
  }
  catch( CZKS::Exception ex ) {
    CZKS::LogException(ex);
    std::cout << "Test failed\n\n";
  }

  std::cout.flush();
  
  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test zkErrorTest\n\n");

  CZKS::Exception aEx;

  zkErrorTest(aEx);

  if( !::zksLogShutdown() ) {
    return 1;
  }

  std::cout << "==============================================\n";
  std::cout << "Test completed succefully\n\n";

  std::cout.flush();

  return 0;
}
