/* 
 * $Id: log3.c,v 1.2 2001/04/19 14:44:26 alainc Exp $
 */
#include <stdio.h>


#include <zks.h>
#include "zkserr.h"
#include "zksmodule.h"
#include "zkslog.h"
#include "zkserr.h"
#include "zkslogagent.h"
#include "zkspath.h"

#include <zkThread.h>
#include <string.h>
#include <unistd.h>


static void * thread_call(void * udata)
{
  ZksErr err = NULL;
  int      idx = (int)udata;
  int      i;

  if( !zksErrNew(&err) ) {
    return (void *)0;

  } else {
    for( i=0; i<100; i++ ) {

      if( (idx == 25) && (i == 80) ) {
        zksLogMesg(ZKS_GENERIC_TEST_MOD, "LOCK");
        zksLogLock();
        usleep(5000000);
        zksLogUnlock();
        zksLogMesg(ZKS_GENERIC_TEST_MOD, "UNLOCK");
      }
      zksLogMesg(ZKS_GENERIC_TEST_MOD, "Here is the message %d", idx);
    }
    zksErrFree(&err);
  }
  return (void *) 1;
}


static ZkBool run(ZksErr err)
{
#define MAXI 100
  int      i;
  ZkThread th[MAXI];
  int maxi = MAXI;
  
  for( i=0; i<maxi; i++ ) {
    if( !zkThreadCreate(zkError(err),
                        &th[i],
                        thread_call,
                        (void *)i) ) {
      zksErrSet(err,ZKS_GENERIC_TEST_MOD, 0x0001, "run", "zkThreadCreate failed");
    } 
  }
  for( i=0; i<maxi; i++ ) {
    void *ret;

    if( !zkThreadJoin(zkError(err),
                      &th[i],
                      &ret) ) {
      zksErrSet(err,ZKS_GENERIC_TEST_MOD, 0x0002, "run", "zkThreadJoin failed");
    } 
  }
  return zksErrCode(err) ? ZK_FALSE : ZK_TRUE;
}

 
int main(int argc, const char * argv[])
{                             
  ZksErr       err       = NULL;
  ZksLogAgent  fileAgent = NULL;

  (void)argc;

  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test logging\n\n");
 
  if( !zksLogInit(argv[0]) ||
      !zksLogTermInit() ) {
    return 1;

  } else  if( !zksErrNew(&err) ) {
    return 1;

#if 1
  } else if( !zksLogTermDisable() ) {
    return 1;
#endif

  } else if( !zksLogAgentFileNew(err, 
                                 &fileAgent, 
                                 "./test.log",
                                 ZKS_LOGAGENT_FILEPERM_USER_WRITE |
                                 ZKS_LOGAGENT_FILEPERM_USER_READ) ) {
    return 1;

  } else if( !zksLogDefaultAdd(fileAgent) ) {
    return 1;

  } else {
    run(err);
  }
  
  zksLogError(err);
  zksErrFree(&err);

  if( !zksLogAgentFileFree(err, &fileAgent) ) {
    return 1;
  }

  if( !zksLogShutdown() ) {
    return 1;
  }

  fprintf(stdout, "==============================================\n");
  fprintf(stdout, "Test completed succefully\n\n");
  return 0;
}     
