#ifdef WIN32 
# pragma warning(disable: 4786)
#endif

#include <fstream>
#include <iostream>

#include <zkslog.h>
#include "zkscmdline.h"
#include "zkspath.h"


int main(int, char ** argv)
{
  CZKS::CmdLine cmdline(argv[0], "Program use to unit test the command line handler");

  if( !::zksLogInit(CZKS::Path(argv[0]).basename().c_str()) ||
      !::zksLogTermInit() ) {
    return 1;
  }

  try {
    try {
      CZKS::CmdArg help('h',
                        "help",
                        CZKS::CmdArg::NONE,
                        0,
                        0,
                        "print usage");

      CZKS::CmdArg unused("unused",
                          CZKS::CmdArg::NONE,
                          2,
                          4,
                          "unused arg");

      CZKS::CmdArg file('f',
                        "filename",
                        CZKS::CmdArg::REQUIRED_PARM,
                        1,
                        1,
                        "requested file name");

      CZKS::CmdArg hid("hiden_parm",
                        CZKS::CmdArg::HIDDEN_PARM, 
                        0,
                        1,
                        "test hiden param");

      CZKS::CmdArg test(std::string("test"),
                        CZKS::CmdArg::REQUIRED_PARM,
                        1, 
                        3,
                        "testing arguments");

      CZKS::CmdArg trailling(CZKS::CmdArg::NONE,
                             0, 
                             3,
                             "trailling arguments");

      const char * targv[] = { {""}, 
                               {"--help"}, 
                               {"--filename"}, 
                               {"--test=111111"}, 
                               {"4"}, 
                               {"5"}, 
                               {"--filename"}, 
                               {"./output.txt"},
                               {"--hiden_parm"},
                               {"h1"},
                               {"--"},
                               {"a"},
                               {"b"},
                               {"c"},  
                               {"d"} };

      int targc   = sizeof(targv) / sizeof(targv[0]);

      cmdline += help;
      cmdline += file;
      cmdline += hid;
      cmdline += test;
      cmdline += unused;
      cmdline += trailling;

      std::cout << "================================================================" << std::endl;
      std::cout << "Test usage" << std::endl << std::endl;

      std::cout << cmdline.usage() << std::endl;

      std::cout << "================================================================" << std::endl;
      std::cout << "Test help" << std::endl << std::endl;

      std::cout << cmdline.help() << std::endl;

      std::cout << "================================================================" << std::endl;
      std::cout << "Test parse" << std::endl << std::endl;

      cmdline.parse(targc, targv).validate();

      std::cout << "================================================================" << std::endl;
      std::cout << "Test parse arglist" << std::endl << std::endl;

      std::cout << cmdline.arglist() << std::endl;

      std::cout << "================================================================" << std::endl;
      std::cout << "Test parse dump " << std::endl << std::endl;

      if( cmdline.isSet("help") ) {
        std::cout << "help is set, ";

        std::string val;

        std::cout << "args=";
        for( uint32_t i=0; i<cmdline.countOf("help"); i++ ) {
          std::cout << cmdline.valueOf(std::string("help"), val, i) << " ";
        }      
        std::cout << std::endl << std::endl;
      } else {
        std::cout << "help is not set" << std::endl;
      }


      if( cmdline.isSet("test") ) {
        std::cout << "test is set, ";

        int val;

        std::cout << "args=";
        for( uint32_t i=0; i<cmdline.countOf("test"); i++ ) {
          std::cout << cmdline.valueOf("test", val, i) << " ";
        }      
        std::cout << std::endl << std::endl;
      } else {
        std::cout << "test is not set" << std::endl;
      }
    

      if( cmdline.isSet('f') ) {
        std::cout << "file  is set, ";

        std::string val;

        std::cout << "args=";
        for( uint32_t i=0; i<cmdline.countOf('f'); i++ ) {
          std::cout << cmdline.valueOf('f', val, i) << " ";
        }      
        std::cout << std::endl << std::endl;
      } else {
        std::cout << "file is not set" << std::endl;
      }

      if( cmdline.isSet("dummy") ) {
        std::cout << "dummy  is set, ";

        std::string val;

        std::cout << "args=";
        for( uint32_t i=0; i<cmdline.countOf("dummy"); i++ ) {
          std::cout << cmdline.valueOf("dummy", val, i) << " ";
        }      
        std::cout << std::endl << std::endl;
      } else {
        std::cout << "dummy is not set" << std::endl;
      }

      if( cmdline.isSetElipse() ) {
        std::string val;

        std::cout << "trailling args=";
        for( uint32_t i=0; i<cmdline.countOfElipse(); i++ ) {
          std::cout << cmdline.valueOfElipse(val, i) << " ";
        }      
        std::cout << std::endl << std::endl;
      } else {
        std::cout << "no trailling arguments set" << std::endl;
      }

    } 
    catch( ... ) {
      CZKS::Rethrow("main", "cmdline test failure");
    }
  }
  catch( CZKS::Exception & e ) {
    //
    // Check for Argument line parsing error
    //
    if( e.isErrCode(ZKS_CMDLINE_MOD, ZKS_CMDLINE_VALIDATE_ERR)  ) {
      std::cout << e.Message() << std::endl;
      std::cout << cmdline.usage() << std::endl;
      return 1;
    } 
    CZKS::LogException(e);
  }

  ::zksLogShutdown();

	return 0;
}

