/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkThread.h"
#include "zkStdio.h"
#include "zkUnistd.h"

#define MSG_BUF_LEN 1024

/*
 * Incredibly simple event testing code.
 */

typedef struct __tag_TestStruct 
{
	int threadno;
	ZkEvent event;
} TestStruct;

static int g_threads = 0;
static ZkEvent g_event = NULL;

static void * ZK_THREAD_CALLBACK event_thread(void *context)
{
	char msg_buf[MSG_BUF_LEN];
	ZkEvent event = (ZkEvent) ((TestStruct *)context)->event;
	int threadno = ((TestStruct *)context)->threadno;
	ZkError err;
	zkErrorNew(&err);
	if (!event) return NULL;
	if (!zkEventWait(err, event, ZK_INFINITE)) {
		  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
		  puts (msg_buf);
	}
	printf("Thread %d running free\n", threadno);
	return NULL;
}

static unsigned int spawn_thread(ZkError err)
{
	TestStruct *context;
	ZkThread thread;
	context = (TestStruct *)malloc(sizeof(TestStruct));
	context->threadno = g_threads++;
	context->event = g_event;
	printf("Spawning thread %d\n", context->threadno);
	return zkThreadCreate(err, &thread, &event_thread, context);
}

int main(void)
{
	unsigned char in;
	ZkError err;
	char msg_buf[MSG_BUF_LEN];
	
	zkErrorNew(&err);
	if (!zkEventNew(err, &g_event, ZK_FALSE)) goto die;
	
	puts("type h for help.");
	while ((in = getchar()) != EOF) {
		switch(in) {
		case 'q':
			puts("done.");
			return 0;
		case 's':
			if (!zkEventSet(err, g_event)) goto die;
			puts("event set");
			break;
		case 'p':
			if (!zkEventPulse(err, g_event)) goto die;
                        puts("event pulsed");
			break;
		case 'r':
			if (!zkEventReset(err, g_event)) goto die;
                        puts("event reset");
			break;
		case 'a':
			if (!zkEventFree(err, &g_event)) goto die;
			if (!zkEventNew(err, &g_event, ZK_FALSE)) goto die;
			puts("Event type is now AUTOMATIC");
			break;
		case 'm':
			if (!zkEventFree(err, &g_event)) goto die;
			if (!zkEventNew(err, &g_event, ZK_TRUE)) goto die;
			puts("Event type is now MANUAL");
			break;
		case 't':
			if (!spawn_thread(err)) goto die;
			break;
		case 'h':
			puts("Simple interactive event tester");
			puts("*******************************");
			puts("If you don't know what this does, ask <oliver@zks.net>");
			puts("");
			puts("Keys:");
			puts("\tq: quit");
			puts("\ts: SetEvent");
			puts("\tp: PulseEvent");
			puts("\tr: ResetEvent");
			puts("\ta: Create new automatic event (no threads can be waiting)");
			puts("\tm: Create new manual event (no threads can be waiting)");
			puts("\tt: Spawn new waiting thread");
		}
		zkThreadYield(err);
	}
die:
  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
  puts (msg_buf);
  return -1;	
}
