/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkThread.h"
#include "zkStdio.h"
#include "zkUnistd.h"

#define MSG_BUF_LEN 1024
int count = 0;

static void ZK_ITIMER_CALLBACK
second_count (void *p, unsigned int i1, unsigned int i2, unsigned long time)
{
  /* silence compiler */
  (void) p;
  (void) i1;
  (void) i2;
  (void) time;
  
  printf ("Second\n");
  count = 0;
}

static void ZK_ITIMER_CALLBACK
decisecond_count (void *p, unsigned int i1, unsigned int i2,
		  unsigned long time)
{
  /* silence compiler */
  (void) p;
  (void) i1;
  (void) i2;
  (void) time;
  
  printf ("DeciSecond %d\n", count++);
}

int
main (void)
{
  char msg_buf[MSG_BUF_LEN];
  unsigned int sleeptime = 10;
  ZkError err;
  ZkITimer timer1 = NULL;
  ZkITimer timer2 = NULL;

  zkErrorNew (&err);
  if (!zkITimerInit (err))
    goto error;
  else if (!zkITimerNew (err, &timer1, &second_count, 1000))
    goto error;
  else if (!zkITimerNew (err, &timer2, &decisecond_count, 100))
    goto error;

#if	defined(ZK_LINUX) || defined(macintosh)
  while ((sleeptime = zkSleep (sleeptime)) != 0)
  	;
#elif defined(WIN32)
  while (1) {
	  /* this just bites. Need a message loop to process timers on win32.
	   * might be an argument to use thread based timers like Linux does */
	MSG msg = { 0, 0, 0, 0 };
	if ( GetMessage(&msg, 0, 0, 0) )
	DispatchMessage( &msg );
  }
#else
#	error "This code only compiles on Win32, Linux or Mac OS, sorry"
#endif

  if (!zkITimerFree (err, &timer1))
    goto error;
  else if (!zkITimerFree (err, &timer2))
    goto error;
  else if (!zkITimerShutdown (err))
    goto error;

  return 0;
error:
  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
  puts (msg_buf);
  return -1;
}
