/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkThread.h"
#include "zkStdio.h"
#include "zkUnistd.h"

#define MSG_BUF_LEN 1024

/*
 * Incredibly simple semaphore testing code.
 */

typedef struct __tag_TestStruct 
{
	int threadno;
	ZkSemaphore semaphore;
} TestStruct;

static int g_threads = 0;
static ZkSemaphore g_semaphore = NULL;

static void *ZK_THREAD_CALLBACK  semaphore_thread(void *context)
{
	char msg_buf[MSG_BUF_LEN];
	ZkSemaphore semaphore = (ZkSemaphore) ((TestStruct *)context)->semaphore;
	int threadno = ((TestStruct *)context)->threadno;
	ZkError err;
	zkErrorNew(&err);
	if (!semaphore) return NULL;
	if (!zkSemaphoreWait(err, semaphore)) {
		  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
		  puts (msg_buf);
	}
	printf("Thread %d running free\n", threadno);
	return NULL;
}

static unsigned int spawn_thread(ZkError err)
{
	TestStruct *context;
	ZkThread thread;
	context = (TestStruct *)malloc(sizeof(TestStruct));
	context->threadno = g_threads++;
	context->semaphore = g_semaphore;
	printf("Spawning thread %d\n", context->threadno);
	return zkThreadCreate(err, &thread, &semaphore_thread, context);
}

int main(void)
{
	unsigned char in;
	ZkError err;
	char msg_buf[MSG_BUF_LEN];
	
	zkErrorNew(&err);
	if (!zkSemaphoreNew(err, &g_semaphore, 0)) goto die;
	
	puts("type h for help.");
	while ((in = getchar()) != EOF) {
		switch(in) {
		case 'q':
			puts("done.");
			return 0;
		case 'p':
			if (!zkSemaphorePost(err, g_semaphore)) goto die;
			break;
		case 't':
			if (!spawn_thread(err)) goto die;
			break;
		case 'h':
			puts("Simple interactive semaphore tester");
			puts("*******************************");
			puts("If you don't know what this does, ask <oliver@zks.net>");
			puts("");
			puts("Keys:");
			puts("\tq: quit");
			puts("\tp: PostSemaphore");
			puts("\tt: Spawn new waiting thread");
		}
		zkThreadYield(err);
	}
die:
  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
  puts (msg_buf);
  return -1;	
}
