/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Freedom code.
 * 
 * The Initial Developer of the Original Code is Zero-Knowledge Systems,
 * Inc.  Portions created by Zero-Knowledge are Copyright (C) 2000
 * Zero-Knowledge Systems, Inc.  All Rights Reserved.
 * 
 * Contributor(s):

 */

#include "zkThread.h"
#include "zkStdio.h"
#include "zkUnistd.h"

#define MSG_BUF_LEN 1024

/*
 * Incredibly simple thread API testing code.
 */


ZkThread thread;

static void * ZK_THREAD_CALLBACK test_thread(void *context)
{
	char msg_buf[MSG_BUF_LEN];
	ZkThread self;
	ZkError err;
	void *p;
	
	/* silence compiler */
	(void) context;
	
	zkErrorNew(&err);
	
	if (!zkThreadSelf(err, &self)) goto error;
	printf ("Thread : zkThreadSelf self at %p, real self is at %p\n", self, thread);
	if (!zkThreadGetThreadNP(err, self, &p)) goto error;

#if defined(ZK_LINUX)
	printf ("Thread : NP thread object at %p = %d\n", p, *(int*)p);
#elif defined(WIN32)
	printf ("Thread : NP thread object at %p = %p\n", p, *(HANDLE*)p);
#elif defined(macintosh)
	printf ("Thread : NP thread object at %p = %lu\n", p, *(unsigned long*)p);
#else
#	error "This code only compiles properly on Win32, Linux or Mac OS, sorry."
#endif

	printf ("Thread : Equality? %d\n", zkThreadEqual(err, self, thread));
	printf ("Thread : yielding\n");
	if (!zkThreadYield(err)) goto error;
	
	return (void *)0xbadbeef;
error:
  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
  puts (msg_buf);
  return NULL;		
}


int main(void)
{
	char msg_buf[MSG_BUF_LEN];
	void *retval;
	ZkError err;
	zkErrorNew(&err);
	printf("Spawning thread...\n");
	if (!zkThreadCreate(err, &thread, &test_thread, NULL)) goto die;
	if (!zkThreadJoin(err, &thread, &retval)) goto die;
	printf("Thread terminated. Retval was %p\n", retval);
    printf ("Spawning Detached Thread...\n");
	if (!zkThreadCreateDetached(err, &test_thread, NULL)) goto die;
	zkSleep(2);
    printf ("Done...\n");
	return 0;
die:
  zkErrorMessage (err, msg_buf, MSG_BUF_LEN);
  puts (msg_buf);
  return -1;	
}
