/*
 * $id$ 
 * Win32 implementation of (semi) portable interval timer API
 * -- Oliver Lavery <oliver@zks.net>
 */
#include <zkThread.h>
#include <zkAlloc.h>

struct __tag_ZkITimer
{
	UINT timer;
	void (ZK_ITIMER_CALLBACK *callback) (void *, unsigned int, unsigned int, unsigned long);
};


/*********************************
 * Initialize Interval Timer thread (nothing for win32)
 */
DLLExport int
zkITimerInit (ZkError err	/* [in/out]   ZkError */
  )
{
  zkErrorClear (err);
  return ZK_TRUE;
}


/*********************************
 * Kill interval timer thread (nothing for win32)
 */
DLLExport int
zkITimerShutdown (ZkError err	/* [in/out]   ZkError */
  )
{
  zkErrorClear (err);
  return ZK_TRUE;
}


/*********************************
 * Create a new interval timer
 */
DLLExport int
zkITimerNew (ZkError err,	/* [in/out]   ZkError */
	     ZkITimer * pTimer,	/* [in/out] indirect pointer to the timer */
	     void (ZK_ITIMER_CALLBACK *callback) (void *, unsigned int, unsigned int, unsigned long),	/*[in] timer callback fn (for win32 ths MUST be declared __stdcall)*/
	     unsigned long millis	/* [in] interval for the new timer */
  )
{
  const char *fn = "zkITimerNew";
  struct __tag_ZkITimer *l_pTimer = NULL;
  int ret = ZK_FALSE;
  //struct timeval now;

  zkErrorClear (err);

  /*
   * Check input values
   */
  if (!pTimer)
    {
      zkErrorSet_va (err, fn, ZKERR, ZKMSEM, ZKENULLPTR, "pTimer is null");	/* XXX not MSEM */
      goto die;
    }
  /* 
   * Allocate system resources 
   */
  else if (!zkMemAlloc
	   (err, (void **) &l_pTimer,
	    (size_t) sizeof (struct __tag_ZkITimer)))
    {
      goto die;
	} 
	else if (!(l_pTimer->timer = SetTimer(NULL, 1, millis, callback)))
	{
      zkErrorSet_va (err, fn, ZKERR, ZKMSEM, ZKEINVAL, "Unable to create new timer");	/* XXX not MSEM */
      goto die;
	}

  /* Good to go. Return the timer to the client and keep truckin' */
  *pTimer = l_pTimer;
  ret = ZK_TRUE;
die:
  return ret;
}


/*********************************
 * Release an interval timer
 */
DLLExport int
zkITimerFree (ZkError err,	/* [in/out]   ZkError */
	      ZkITimer * pTimer	/* [in/out] indirect pointer to the timer */
  )
{
  const char *fn = "zkITimerFree";
  int ret = ZK_FALSE;

  zkErrorClear (err);

  /*
   * Check input values
   */
  if (!pTimer || !*pTimer)
    {
      zkErrorSet_va (err, fn, ZKERR, ZKMSEM, ZKENULLPTR, "pTimer is null");	/* XXX not MSEM */
      goto die;
    }
	if (!KillTimer(NULL, (*pTimer)->timer))
	{
      zkErrorSet_va (err, fn, ZKERR, ZKMSEM, ZKEINVAL, "Unable to kill timer");	/* XXX not MSEM */
      goto die;
	}
  /* 
   * Free system resources 
   */
  if (!zkMemFree (err, (void **) pTimer))
    {
      goto die;
    }

  /* Good to go. Null out the client's ptr */
  *pTimer = NULL;
  ret = ZK_TRUE;
die:
  return ret;
}
