/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <string.h>
#include <sys/wait.h>

#include <glib.h>
#include <openssl/bn.h>
#include <openssl/crypto.h>

#include "autoconf.h"

#include "man_packet.h"
#include "topo.h"

/*
 * I love global variables.  I do.
 */
unsigned char *global_ptr = NULL;
size_t global_len;
extern void fatal(int);

int topo_setup(void)
{
    char *remote, *local;
    struct stat st;
    int fd;
    int status;

    if ((remote = getenv("NYMIP_TOPOLOGY_REMOTE")) == NULL) {
        printf("need to set NYMIP_TOPOLOGY_REMOTE variable\n");
        return 1;
    }

    if ((local = getenv("NYMIP_TOPOLOGY_LOCAL")) == NULL) {
        printf("need to set NYMIP_TOPOLOGY_LOCAL variable\n");
        return 1;
    }

    switch(fork()) {
        case 0:
            execl("/usr/bin/rsync", "/usr/bin/rsync",
                  "-v",
                  remote,
                  local,
                  NULL);
            break;
        case -1:
            perror("fork");
            fatal(1);
        default:
            wait(&status);
    }

    fd = open(local, O_RDONLY);

    if (fd < 0) {
        perror("open");
        return 1;
    }

    if (fstat(fd, &st)) {
        perror("fstat()");
        return 1;
    }

    global_ptr = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);

    if (global_ptr == MAP_FAILED){
        perror("mmap()");
        return 1;
    }

    global_len = st.st_size;

    return 0;
}


void topo_show(int argc, char **argv, struct management_packet *mp)
{
    struct nlsahdr *lh;
    int offset = 0, left_to_parse, lsa_count, payload_offset;

    if (global_ptr == NULL)
        if(topo_setup())
            return;

    /* Walk the file once to figure out how many headers there are */
    left_to_parse = global_len;
    lsa_count = 0;
    while (left_to_parse >= sizeof(*lh) && offset < global_len) {
        lh = (struct nlsahdr *)(global_ptr + offset);
        offset += sizeof(*lh);
        left_to_parse -= sizeof(*lh);
        left_to_parse -= ntohs(lh->payload_len);
        lsa_count++;
    }

    if (left_to_parse != 0)
        fprintf(stderr, "Warning: left_to_parse != 0: bogus topology file?\n");

    offset = 0;
    payload_offset = lsa_count * sizeof(*lh);
    while (lsa_count--) {
        unsigned char *p = global_ptr + payload_offset;
        int i;
        BIGNUM *seq;
        guint32 *addr;
        char *seqtext;
        struct in_addr incredible;

        lh = (struct nlsahdr *)(global_ptr + offset);
        offset += sizeof(*lh);

        seq = BN_bin2bn(lh->seq, 16, NULL);
        seqtext = BN_bn2hex(seq);

        incredible.s_addr = lh->addr;
        printf("[%s] 0x%s\n", inet_ntoa(incredible), seqtext);
        BN_free(seq);
        OPENSSL_free(seqtext);

        addr = (guint32 *)p;
        for (i = 0; i < lh->numneighbors ; i++) {
            incredible.s_addr = *addr;
            printf("\t%s\n", inet_ntoa(incredible));
            p += sizeof(*addr);
            addr = (guint32 *)p;
        }
        payload_offset += ntohs(lh->payload_len);
    }

    fatal(0); /* XXX duh */
}
