#ifndef __AUTH_H__
#define __AUTH_H__

#include "cf.h"
#include "op.h"
#include "munge.h"

enum {
    AUTH_NONE = 0,
    AUTH_GPG = 1,
    AUTH_NULL = 2,
    AUTH_SSH = 3,
    AUTH_AGP = 4
};

typedef enum {
    AUTH_STATUS_UNKNOWN = 0,
    AUTH_STATUS_PAID = 1,
    AUTH_STATUS_FREE = 2
} auth_cert_status;

typedef enum {
    AUTH_TYPE_UNKNOWN = 0,
    AUTH_TYPE_CLIENT = 0,
    AUTH_TYPE_NODE = 1
} auth_cert_type;

struct auth_instance {
    auth_cert_status status;
    auth_cert_type ctype;
    struct auth_funcs *af;
};

struct auth_funcs {
    int (*sign)(struct auth_instance *ainst, struct our_packet *op,
                unsigned char *data, int len);
    int (*verify)(struct auth_instance *ainst, unsigned char *data, int len,
                  unsigned char *sig, int siglen);
    void (*destroy)(struct auth_funcs *af);
    void *auth_data;
    unsigned int data_len;
    guint16 __id;
};

void auth_append(struct our_packet *op);
void auth_append_id(struct our_packet *op, guint16 id);
void auth_inst_free(struct auth_instance *ainst);
struct auth_instance * auth_pick(unsigned char *data, guint16 len);
struct auth_instance * auth_inst_from_id(guint16 id);
struct auth_funcs * auth_is_registered(guint16 id);
int auth_register(struct auth_funcs *sch, guint16 id);
void auth_unregister(guint16 id);
void auth_destroy(void);
int auth_init(struct our_config *cf);

#endif
