/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <glib.h>

#include "cf.h"
#include "log.h"

/*
 * my first yacc .com
 *
 * I can't beleive I'm writing this..
 */

/*
 * These describe both the type of data in 
 * the config item and what to do with the
 * data.. 
 */
enum {
    T_STRING,
    T_STRINGLIST,
    T_IPV4,
    T_INT,
    T_NETSHORT,
    T_IPPORTSIN,
    T_IPPORTLIST,
    T_IPNETWORK,
};

#define DECLARE_ITEM(NAME, TYPE) \
    {#NAME, T_##TYPE, offsetof(struct our_config, NAME)}

static struct cf_item {
    char *token;
    int type;
    size_t offset;
} cf_items[] =  {
    DECLARE_ITEM(debug_level, INT),
    DECLARE_ITEM(man_socket_path, STRING),
    DECLARE_ITEM(neighbour, STRINGLIST),
    DECLARE_ITEM(lsa_duration, INT),
    DECLARE_ITEM(lsa_expiration, INT),
    DECLARE_ITEM(lsa_retransmit_throttle, INT),
    DECLARE_ITEM(file_refresh_duration, INT),
    DECLARE_ITEM(lsa_retry_duration, INT),
    DECLARE_ITEM(lsa_retry_maximum, INT),
    DECLARE_ITEM(topology_file, STRING),
    DECLARE_ITEM(topology_file_tmp, STRING),
    DECLARE_ITEM(local_addr, IPPORTSIN),
    DECLARE_ITEM(lsa_sequence_file, STRING),
    DECLARE_ITEM(lsa_auth_type, INT),
    DECLARE_ITEM(log_file, STRING),
    DECLARE_ITEM(log_fd, STRING),
    DECLARE_ITEM(ip_pool, IPNETWORK),
    DECLARE_ITEM(module_dir, STRING),
    DECLARE_ITEM(ssh_identity, STRING),
    DECLARE_ITEM(neg_rekey_duration, INT),
    DECLARE_ITEM(nymip_port, INT),
    DECLARE_ITEM(gpg_certificate, STRING),
    DECLARE_ITEM(agp_ca_public_key, STRING),
    DECLARE_ITEM(agp_public_key, STRING),
    DECLARE_ITEM(agp_private_key, STRING),
    DECLARE_ITEM(link_idle_probe_delay, INT),
    DECLARE_ITEM(link_idle_probe_interval, INT),
    DECLARE_ITEM(link_idle_timeout, INT),
    DECLARE_ITEM(link_rekey_interval, INT),
    DECLARE_ITEM(retransmit_interval, INT),
    {0,}
};

#define DECLARE_IT(TYPE, VAR, CF, OFFSET)   \
    TYPE * VAR = (TYPE *)((void *)CF + OFFSET)

static int grab_config_item(struct our_config *oc, 
        unsigned char *token, unsigned char *args)
{
    struct cf_item *cf = cf_items;

    for(; cf->token != NULL; cf++) {
        if (strcmp(cf->token, token))
            continue;

        switch(cf->type) {
            case T_IPPORTSIN: {
                char *sep = index(args, ':');
                DECLARE_IT(struct sockaddr_in, addr, oc, cf->offset);
                        
                if( sep == NULL )
                    break;
                *sep = '\0';
                addr->sin_family = AF_INET;
                addr->sin_addr.s_addr = inet_addr(args);
                addr->sin_port = htons(atol(sep + 1));
            } break;
                                        
            case T_IPPORTLIST: {
                char *sep = index(args, ':');
                DECLARE_IT(GList *, list, oc, cf->offset);
                struct sockaddr_in *sin = calloc(1,sizeof(struct sockaddr_in));

                if( sep == NULL || sin == NULL)
                    break;
                *sep = '\0';
                sin->sin_family = AF_INET;
                sin->sin_addr.s_addr = inet_addr(args);
                sin->sin_port = htons(atol(sep + 1));
                *list = g_list_insert(*list, sin, 0);
            } break;

            case T_IPNETWORK: {
                int boogers;
                char *sep = index(args, '/');
                DECLARE_IT(struct ip_pool, addr, oc, cf->offset);

                if( sep == NULL )
                    break;
                *sep = '\0';
                addr->start.s_addr = inet_addr(args);
                boogers = atol(sep + 1);
                if (boogers < 8 || boogers > 32) {
                    LOG("Invalid netmask\n");
                    return 1;
                }
                addr->size = 1 << (32 - boogers);
                if (ntohl(addr->start.s_addr) & (addr->size - 1)) {
                    LOG("Host part includes network part\n");
                    return 1;
                }
            } break;

            case T_IPV4: {
                DECLARE_IT(unsigned long int, addr, oc, cf->offset);
                *addr = inet_addr(args);
            } break;
                         
            case T_NETSHORT: {
                DECLARE_IT(guint16, var, oc, cf->offset);
                *var  = htons(atol(args));
            } break;

            case T_INT: {
                DECLARE_IT(int, var, oc, cf->offset);
                *var  = atol(args);
            } break;

            case T_STRING: {
                DECLARE_IT(char *, str, oc, cf->offset);
                *str  = strdup(args);
            } break;

            case T_STRINGLIST: {
                DECLARE_IT(GList *, list, oc, cf->offset);
                *list = g_list_insert(*list, strdup(args), 0);
            } break;
        }
        return 0;
    }

    return 1;
}

unsigned char * skip_whitespace(char *buf)
{
    if(!buf)
        return NULL;

    for( ; *buf ; buf++) {
        if(! isspace(*buf))
            return buf;
    }

    return NULL;
}

void kill_whitespace(char *start, char *end)
{
    for(; start < end; start++)
        if(isspace(*start))
            *start = '\0';
}

int parse_config(struct our_config *cf, char *filename)
{
    char buf[4096];
    FILE *frig;
    int line = 0;
    char *line_copy = NULL;

    frig = fopen(filename, "r");
    if(!frig) {
        LOG("couldn't open %s\n",filename);
        return -1;
    }

    while(fgets(buf, 4095, frig) != NULL) {
        char *token, *args,  *eq;

        line++;
        if(line_copy) {
            free(line_copy);
            line_copy = NULL;
        }
        line_copy = strdup(buf);
        line_copy[strlen(line_copy) - 1] = '\0';

        eq = token = args = NULL;

        if( (token = skip_whitespace(buf)) == NULL)
            continue;

        switch(token[0]) {
            case '#': case ';':
                continue;
            default:break;
        }

        if( (eq = index(token + 1, '=')) == NULL)
            goto parse_error;

        kill_whitespace(token, eq);
        *eq = '\0';

        if( (args = skip_whitespace(eq + 1)) == NULL)
            goto parse_error;

        /* kill newline, bad hack */
        kill_whitespace(args, args + strlen(args));

        if( grab_config_item(cf, token, args)) 
            goto parse_error;
    }

    if(line_copy)
        free(line_copy);
    fclose(frig);
    return 0;

parse_error:
    if(line_copy) {
        LOG("error at line %d: ``%s''\n", line, line_copy);
        free(line_copy);
    } else
        LOG("error at line %d\n", line);
    fclose(frig);
    return -1;
}

struct our_config *cf = NULL;

int load_config(struct our_config *l_cf, char *filename)
{
    if (filename == NULL)
        return -1;

    cf = l_cf;

    cf->local_addr.sin_addr.s_addr = INADDR_ANY;
    cf->local_addr.sin_port = -1;
    cf->nymip_port = 31337; /* default */

    if (parse_config(cf, filename)) {
        LOG("error parsing config file.\n");
        return -1;
    }

    if (cf->man_socket_path == NULL) {
        LOG("need a management socket path\n");
        return -1;
    }

    if (cf->retransmit_interval <= 0) {
        LOG("retransmit_interval must be greater than zero\n");
        return -1;
    }

    return 0;
}
