#ifndef __CIPHDIG_H__
#define __CIPHDIG_H__

#include <glib.h>

/*
 * its currently important that both of these
 * start at 1.. dorky?
 */
enum {
    C_BF    = 1,
    C_NULL  = 2,
};

enum {
    D_MD5   = 1,
    D_SHA1  = 2,
};

#define MAX_DIGEST_BYTES 16

struct cipher_instance {
    guint16 id;

    struct cipher_instance * (*new)(unsigned char *data, size_t len);
    size_t (*encipher)(struct cipher_instance *ciph, 
            unsigned char *in, unsigned char *out, 
            size_t len);
    size_t (*decipher)(struct cipher_instance *ciph, 
            unsigned char *in, unsigned char *out, 
            size_t len);
    void (*destroy)(struct cipher_instance *ciph);

    void *private_data;
};

struct digest_instance {
    guint16 id;

    struct digest_instance * (*new)(unsigned char *data, size_t len);
    void (*reset)(struct digest_instance *dig);
    void (*hash)(struct digest_instance *dig,
            unsigned char *in, size_t len);
    void (*get_output)(struct digest_instance  *dig,
            unsigned char *out, size_t len);
    void (*destroy)(struct digest_instance *dig);

    void *private_data;
};

int ciphdig_init(void);

int cipher_register(struct cipher_instance *ciph);
struct cipher_instance *cipher_new(guint16 id,
       unsigned char *data, size_t len);
void cipher_destroy(struct cipher_instance *dig);
int cipher_id_exists(guint16 id);

int digest_register(struct digest_instance *ciph);
struct digest_instance *digest_new(guint16 id,
       unsigned char *data, size_t len);
void digest_destroy(struct digest_instance *dig);
int digest_id_exists(guint16 id);

size_t ciph_walk(GList *ciphers, 
        unsigned char *in_arg, unsigned char *out_arg,
        size_t len, int forward, int encipher);

#define ciph_walk_backward_encipher(CIP, IN, OUT, LEN) \
                        ciph_walk(CIP, IN, OUT, LEN, 0, 1)
#define ciph_walk_forward_decipher(CIP, IN, OUT, LEN) \
                        ciph_walk(CIP, IN, OUT, LEN, 1, 0)

#endif
