#ifndef __CIRCS_H__
#define __CIRCS_H__

#include <glib.h>

#include "neg.h"
#include "util.h"

/*
 * we're calling a circuit the path of
 * hops a packet takes while it is
 * in the cloud.   its really just a series
 * of tunneling/rewriting rules.
 *
 * this structure records a one-way forwarding
 * decision we'll make when we get a packet from
 * a certain source.  
 *
 * 'remote_id' is how we recognize the imcoming 
 * packet.  it is ignored for packets entering
 * the cloud.
 *
 * 'local_id' is what we mark the circuit id
 * as it leaves the machine.  
 *
 * you need two of these structs to make a 
 * full duplex routing hop.
 */
enum {
    /* 
     * I'm not really fond of the concept of
     * circuit state.  if there are different
     * types of circs they should be stored
     * in different places.  but our circ
     * shared secret buildup suggests that
     * we forward packets over a pending circ,
     * so we have partial circs in the fast path..
     *
     * this might want to be cleaned up.
     */
    CIRC_ESTABLISHED,
    CIRC_PENDING
};

/*
 * lovely.  the ciphers need to know which way
 * traffic is going so they can encipher or
 * decipher.. man.. 
 */
enum {
    CIRC_ENCIPHER,
    CIRC_DECIPHER
};

struct circuit {
    guint32 local_id, remote_id;
    guint32 seq;
    struct seqctl seqctl;
    struct link *in_link, *out_link;
    GList *payload_ciphers;
    int (*exit)(struct circuit *circ, struct our_packet *op);
    int (*configure)(struct circuit *circ);
    void (*grab_string)(struct circuit *circ, char *str);
    void (*shutdown)(struct circuit *circ);
    void *tap_data;
    struct circuit *partner; /* two circs make up a bidirectional hop */ 
    unsigned int    state:1,
                    cipher_mode:1,
                    responder:1;  
    guint32 ip_addr, netmask;
};

int circ_neg_begin(struct sock_endpoint *sock, guint32 *hops, int num_hops);
extern void circ_neg_got_packet(struct link *link, struct our_packet *op, 
        guint8 link_nh, guint link_circuit_id, guint32 opt_circuit_id);
void circ_destroy(struct circuit *circ, struct link *no_message);

#endif /*__CIRCS_H__*/
