#ifndef NYMIP_KEYING_H
#define NYMIP_KEYING_H

#include <sys/uio.h>

#include "ciphdig.h"
#include "scheme.h"

enum {
    K_REKEY_REQ = 0,
    K_REKEY_ACK,
    K_REKEY_PROBE,
};

#define    K_LEFT      0 
#define    K_RIGHT     1 
#define    K_PRIMARY   0 
#define    K_SECONDARY 1
#define    K_TX        0
#define    K_RX        1

#define    KEYING_INIT 1
#define    KEYING_RESP 2

struct keying {
    /*
     * [old] are only valid while we're rekeying..
     */
    /* [left|right][current|old][tx|rx] */
    struct cipher_instance *ciphs[2][2][2];

    int rekeying;
    int dir_toggle;
    int cipher_id;
    struct digest_instance *dig;
    guint req_retrans_id;

    void (*send_event)(struct keying *ks, int event);
    void *private_data;

    unsigned char current_secret[MAX_DIGEST_BYTES];
    unsigned char session_secret[MAX_DIGEST_BYTES];
};

int key_init_session(struct keying *ks, struct scheme_instance *sinst,
        struct iovec *iov, int iov_len);
int key_derive(struct keying *ks, int level);
void key_destroy(struct keying *ks);
void key_got_rekey_event(struct keying *ks, int event);
void key_got_packet(struct keying *ks, int level);
int key_initiate_rekey(struct keying *ks);

#endif
