#ifndef NFWDD_LINKS_H
#define NFWDD_LINKS_H

#include <sys/types.h>
#include <sys/socket.h>
#include <glib.h>

#include "util.h"
#include "cf.h"
#include "ciphdig.h"
#include "circs.h"
#include "op.h"
#include "neg.h"
#include "keying.h"
#include "cf.h"

enum {
    NYMIP_E2E = 0,
    NYMIP_LSA,
    NYMIP_LSAACK,
    NYMIP_CNEG,
    NYMIP_CREP,
    NYMIP_LNKOPT,
    NYMIP_CTO1,
    NYMIP_CC,
    NYMIP_CONTROL,
    NYMIP_ECHO_REQUEST,
    NYMIP_ECHO_REPLY,
    NYMIP_REKEY_REQ,
    NYMIP_REKEY_ACK,
};


/*
 * A link represents a logical connection
 * between two nodes.  
 *
 * local_id represents the local endpoint
 * of the link.  we put it in the headers
 * of outgoing packets to tell the remote
 * receiver what link the incoming packet
 * came from.
 *
 * remote_id represents the remote endpoint.
 * We use it to identify a link based on
 * the headers of received packets.  
 */
struct link {
    struct sockaddr_in  remote_sin;
    struct sock_endpoint *sock;
    guint32             seq;
    struct seqctl       seqctl;
    GHashTable          *circ_hash_remote, *circ_hash_local;
    GSList              *circ_list;  /* ugh. */
    struct keying       keying;
    int                 probe_id;
    int                 rekey_timeout_id;
    time_t              last_activity;
};

extern struct link *link_new(void);
int links_init(struct sock_endpoint *sock);
void links_shutdown(struct sock_endpoint *sock); 
extern void link_neg_got_packet(struct sock_endpoint *sock, 
        struct our_packet *op, struct sockaddr_in *from);
extern struct link * find_cloud_link_by_ip(struct sock_endpoint *sock, guint32 *addr);
int link_neg_begin(struct sock_endpoint *sock, struct sockaddr_in *sin);
int link_destroy(struct link *link);

#endif /*__LINKS_H__*/
