#ifndef __LOG_H__

#include <openssl/bn.h>

#include "cf.h"

#define DEBUG /* configurable! */

extern void log_bootstrap(void);
extern int log_init(struct our_config *cf);
extern void log_printf(unsigned int level, char *format, ... );
extern void log_bignum(unsigned int level, BIGNUM *bn, char *format, ...);
extern void hexdump(unsigned int level, char * func, char *string, void *ptr,
                    int len);

enum {
    D_LOG    = 0,
    D_TOPO   = 1 << 1,
    D_NEG    = 1 << 2,
    D_AUTH   = 1 << 3,
    D_SCHEME = 1 << 4,
    D_TUNNEL = 1 << 5,
    D_ROUTE  = 1 << 6,
    D_KEY    = 1 << 7,
    D_ERROR  = 1 << 31,
};

/*
 * These are the functions you really want to call.
 *
 * internally, LOG turns into a level of -1 which will
 * always be printed.
 */

#ifdef DEBUG
#  define DPRINTF(level, args...) log_printf(level, __FUNCTION__ "(): "args)
#  define HEXDUMP(level, args...) log_printf(level, __FUNCTION__ "(): ", args)
#else
#  define DPRINTF(args...) do {} while (0)
#  define HEXDUMP(args...) do {} while (0)
#endif

#define LOG(args...) log_printf(D_LOG, __FUNCTION__ "(): "args)
#define LOG_BN(bn, args...) log_bignum(D_LOG, bn, __FUNCTION__ "(): " args)


/* Aid in pretty-printing IP addresses */
#define U32QUAD(U32) \
        ((unsigned char *)&(U32))[0], \
        ((unsigned char *)&(U32))[1], \
        ((unsigned char *)&(U32))[2], \
        ((unsigned char *)&(U32))[3]

#define SOCKQUAD(SOCKADDR) U32QUAD((SOCKADDR).sin_addr.s_addr)

#endif
