/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>

#include <glib.h>
#include <gmodule.h>

#include "module.h"
#include "log.h"

/*
 * a shared way of loading modules in a directory.
 *
 * we insert the module container into the hash, which
 * the caller can then walk/lookup based on the id.
 * they also cast funcs to their func ptr struct.  symbol
 * should be defined in a header to be a string that
 * has versioning info that covers the contents of the
 * func ptr struct.
 */

int module_dir_walk(struct our_config *cf, char *symbol)
{
    DIR *d;
    struct dirent *e;
    int numok = 0;
    char *dir = cf->module_dir;

    if ((d = opendir(dir)) == NULL) {
        LOG("Unable to open directory `%s'\n", dir);
        return -1;
    }

    LOG("trying to call `%s' in modules found in dir `%s'\n",
            symbol, dir);
                                    
    while ((e = readdir(d)) != NULL) {
        char *p;
        char *filename;
        GModule *handle;
        int (*symbol_ptr)(struct our_config *cf);

        if ((p = strrchr(e->d_name, '.')) == NULL)
            continue;

        if(strcmp(p, ".so"))
            continue;

        filename = g_strconcat(dir, "/", e->d_name, NULL);

        /*
         * We're leaking these handles on purpose.  We're counting
         * on the process teardown to clean up after them.  Thats
         * pretty lame, but..
         */

        if( (handle = g_module_open(filename, 0)) == NULL) {
            LOG("couldn't open module `%s' (%s)\n",
                    filename, g_module_error());
            g_free(filename);
            continue;
        }

        if (!g_module_symbol(handle, symbol, (gpointer *) &symbol_ptr)) {

            LOG("module file `%s' does not contain symbol `%s'\n",
                    filename, symbol);
            g_free(filename);
            g_module_close(handle);
            continue;
        }

        LOG("calling %s() in %s\n", symbol, filename);

        if (symbol_ptr(cf)) {
            LOG("error calling module `%s' init\n",filename);
        } else {
            numok++;
        }

        g_free(filename);
    }

    closedir(d);

    return numok;
}
