/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>

#include <gmodule.h>

#include "module.h"
#include "auth.h"
#include "log.h"
#include "util.h"

#define AUTH_STR "for great justice launch all zig"
#define AUTH_LEN 32

/*
 * The NULL auth is just static string checking
 */
static int a_sign(struct auth_instance *ainst, struct our_packet *op,
                  unsigned char *data, int len)
{
    unsigned char *p;

    p = op_push_tail(op, AUTH_LEN);
    memcpy(p, AUTH_STR, AUTH_LEN);

    DPRINTF(D_AUTH, "sending NULL auth message\n");

    return AUTH_LEN;
}

static int a_verify(struct auth_instance *ainst, unsigned char *data, int len,
                    unsigned char *sig, int siglen)
{
    if (siglen != AUTH_LEN)
        return 0; /* obviously invalid */

    if (memcmp(sig, AUTH_STR, AUTH_LEN) == 0) {
        ainst->status = AUTH_STATUS_UNKNOWN;
        ainst->ctype = AUTH_TYPE_UNKNOWN;

        return 1; /* valid */
    } else
        return 0; /* invalid */
}

static void
a_destroy(struct auth_funcs *af)
{
    auth_unregister(AUTH_NULL);
}

MODULE_INIT(struct our_config *cf)
{
    struct auth_funcs *funcs;

    LOG("a_null starting up: howdy!\n");

    if ((funcs = calloc(1, sizeof(*funcs))) == NULL) {
        return -ENOMEM;
    }

    funcs->sign = a_sign;
    funcs->verify = a_verify;
    funcs->destroy = a_destroy;
    funcs->auth_data = NULL;

    return auth_register(funcs, AUTH_NULL);
}
