/* -*- mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Null cipher scheme for multi-hop routes and debugging
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ciphdig.h"
#include "log.h"
#include "module.h"

struct cipher_instance null_ciph;

static struct cipher_instance * null_new(unsigned char *data, size_t len)
{
    struct cipher_instance *ciph;

    if ((ciph = calloc(sizeof(struct cipher_instance), 1)) == NULL)
        return NULL;

    memcpy(ciph, &null_ciph, sizeof(struct cipher_instance));

    return ciph;
}

static size_t null_encipher(struct cipher_instance *ciph,
                            unsigned char *in, unsigned char *out, size_t len)
{
    return len;
}

static size_t null_decipher(struct cipher_instance *ciph,
                            unsigned char *in, unsigned char *out, size_t len)
{
    return len;
}

static void null_destroy(struct cipher_instance *ciph)
{
    return;
}

struct cipher_instance null_ciph = {
    new:        null_new,
    encipher:   null_encipher,
    decipher:   null_decipher,
    destroy:    null_destroy,
    id:         C_NULL,
};


MODULE_INIT(struct our_config *cf)
{
    if (cipher_register(&null_ciph)) {
        LOG("couldn't register null cipher id %d\n", C_NULL);
        return -1;
    }

    LOG("registered null cipher id %d\n", C_NULL);

    return 0;
}

