/* -*- mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * An OpenSSL blowfish cipher scheme
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ciphdig.h"
#include "log.h"
#include "module.h"
#include <openssl/blowfish.h>

struct bf_local {
    BF_KEY key;
    struct cipher_instance  ciph;
};

struct cipher_instance bf_ciph;

static struct cipher_instance * bf_new(unsigned char *data, size_t len)
{
    struct bf_local *bfl;

    if ((bfl = calloc(sizeof(struct bf_local), 1)) == NULL)
        return NULL;

    memcpy(&bfl->ciph, &bf_ciph, sizeof(struct cipher_instance));
    bfl->ciph.private_data = bfl;

    BF_set_key(&bfl->key, len, data);

    return &bfl->ciph;
}

static size_t bf_encipher(struct cipher_instance *ciph,
        unsigned char *in, unsigned char *out, size_t len)
{
    struct bf_local *bfl = ciph->private_data;
    unsigned char ivec[8]={0,};
    int num = 0;

    BF_cfb64_encrypt( in, out, len, &bfl->key, ivec, &num, BF_DECRYPT);
            
    return len;
}
static size_t bf_decipher(struct cipher_instance *ciph,
        unsigned char *in, unsigned char *out, size_t len)
{
    struct bf_local *bfl = ciph->private_data;
    unsigned char ivec[8]={0,};
    int num = 0;

    BF_cfb64_encrypt( in, out, len, &bfl->key, ivec, &num, BF_ENCRYPT);
            
    return len;
}
static void bf_destroy(struct cipher_instance *ciph)
{
    struct bf_local *bfl = ciph->private_data;
    free(bfl);
}

struct cipher_instance bf_ciph = {
    new:        bf_new,
    encipher:   bf_encipher,
    decipher:   bf_decipher,
    destroy:    bf_destroy,
    id:         C_BF,
};


MODULE_INIT(struct our_config *cf)
{
    if (cipher_register(&bf_ciph)) {
        LOG("couldn't register openssl blowfish cipher id %d\n", 
                C_BF);
        return -1;
    }

    LOG("registered blowfish cipher id %d\n", C_BF);

    return 0;
}

