/* -*- mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * MD5 digest scheme
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <openssl/md5.h>

#include "ciphdig.h"
#include "log.h"
#include "module.h"
#include "util.h"

struct md5_local {
    struct digest_instance dig;
    MD5_CTX c; 
};
    
struct digest_instance md5_dig;


void md5_reset(struct digest_instance *dig)
{
    struct md5_local *sl = dig->private_data;

    MD5_Init(&sl->c);
}

static struct digest_instance * md5_new( unsigned char *data,
        size_t len)
{
    struct md5_local *sl = calloc(1, sizeof(struct md5_local));

    if (MD5_DIGEST_LENGTH < MAX_DIGEST_BYTES) {
        LOG("MD5_DIGEST_LENGTH (%d) is less than MAX_DIGEST_BYTES (%d)\n",
                MD5_DIGEST_LENGTH, MAX_DIGEST_BYTES);
        return NULL;
    }

    if (sl == NULL)
        return NULL;

    memcpy(&sl->dig, &md5_dig, sizeof(struct digest_instance));
    sl->dig.private_data = sl;

    md5_reset(&sl->dig);

    return &sl->dig;
}

static void md5_hash(struct digest_instance *dig,
                    unsigned char *in, size_t len)
{
    struct md5_local *sl = dig->private_data;

    MD5_Update(&sl->c, in, len);

}

static void md5_get_output(struct digest_instance  *dig,
            unsigned char *out, size_t len)
{
    struct md5_local *sl = dig->private_data;

    unsigned char md[MD5_DIGEST_LENGTH];

    MD5_Final(md, &sl->c);

    memcpy(out, md, len);
}

static void md5_destroy(struct digest_instance *dig)
{
    struct md5_local *sl = dig->private_data;

    free(sl);
}

struct digest_instance md5_dig = {

new:        md5_new,
reset:      md5_reset,
hash:       md5_hash,
get_output: md5_get_output,
destroy:    md5_destroy,
id:         D_MD5,

};

MODULE_INIT(struct our_config *cf)
{
    if (digest_register(&md5_dig)) {
        LOG("couldn't register MD51 digest id %d\n", 
                D_MD5);
        return -1;
    }

    LOG("registered MD51 digest id %d\n", D_MD5);

    return 0;
}

