/* -*- mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * SHA1 digest scheme
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <openssl/sha.h>

#include "ciphdig.h"
#include "log.h"
#include "module.h"
#include "util.h"

struct sha1_local {
    struct digest_instance dig;
    SHA_CTX c; 
};
    
struct digest_instance sha1_dig;


void sha1_reset(struct digest_instance *dig)
{
    struct sha1_local *sl = dig->private_data;

    SHA1_Init(&sl->c);
}

static struct digest_instance * sha1_new( unsigned char *data,
        size_t len)
{
    struct sha1_local *sl = calloc(1, sizeof(struct sha1_local));

    if (SHA_DIGEST_LENGTH < MAX_DIGEST_BYTES) {
        LOG("SHA_DIGEST_LENGTH (%d) is less than MAX_DIGEST_BYTES (%d)\n",
                SHA_DIGEST_LENGTH, MAX_DIGEST_BYTES);
        return NULL;
    }

    if (sl == NULL)
        return NULL;

    memcpy(&sl->dig, &sha1_dig, sizeof(struct digest_instance));
    sl->dig.private_data = sl;

    sha1_reset(&sl->dig);

    return &sl->dig;
}

static void sha1_hash(struct digest_instance *dig,
                    unsigned char *in, size_t len)
{
    struct sha1_local *sl = dig->private_data;

    SHA1_Update(&sl->c, in, len);

}

static void sha1_get_output(struct digest_instance  *dig,
            unsigned char *out, size_t len)
{
    struct sha1_local *sl = dig->private_data;

    unsigned char md[SHA_DIGEST_LENGTH];

    SHA1_Final(md, &sl->c);

    memcpy(out, md, len);
}

static void sha1_destroy(struct digest_instance *dig)
{
    struct sha1_local *sl = dig->private_data;

    free(sl);
}

struct digest_instance sha1_dig = {
    new:        sha1_new,
    reset:      sha1_reset,
    hash:       sha1_hash,
    get_output: sha1_get_output,
    destroy:    sha1_destroy,
    id:         D_SHA1,
};

MODULE_INIT(struct our_config *cf)
{
    if (digest_register(&sha1_dig)) {
        LOG("couldn't register SHA1 digest id %d\n", 
                D_SHA1);
        return -1;
    }

    LOG("registered SHA1 digest id %d\n", D_SHA1);

    return 0;
}

