/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <gmodule.h>

#include "module.h"
#include "scheme.h"
#include "log.h"
#include "util.h"

#include <openssl/dh.h>
#include <openssl/bn.h>
#include <openssl/err.h>

static unsigned char P_str[] = 
    "FFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD1"
    "29024E088A67CC74020BBEA63B139B22514A08798E3404DD"
    "EF9519B3CD3A431B302B0A6DF25F14374FE1356D6D51C245"
    "E485B576625E7EC6F44C42E9A63A3620FFFFFFFFFFFFFFFF";

static unsigned char G_str[] =  "2";

struct dh_local {
    DH *dh;
    struct scheme_funcs funcs;
};

#define GX_LEN 96

void dh_append(struct scheme_funcs *sch, struct our_packet *op)
{
    unsigned char *data;
    struct dh_local *dhl = sch->scheme_data;

    if(BN_num_bytes(dhl->dh->pub_key) != GX_LEN) {
        LOG("finding a pub_key with len %d makes me sad\n",
                BN_num_bytes(dhl->dh->pub_key));
        return;
    }

    data = op_push_tail(op, GX_LEN);

    BN_bn2bin(dhl->dh->pub_key, data);

    LOG_BN(dhl->dh->pub_key, "sending dh g^x mod p: ");
}

struct scheme_instance * dh_parse(struct scheme_funcs *sch, 
        unsigned char *data, unsigned int len)
{
    struct dh_local *dhl = sch->scheme_data;
    BIGNUM pub_key;
    struct scheme_instance *sinst;

    if(len < GX_LEN) {
        LOG("parsing a packet with with len %d makes me sad\n", len);
        return NULL;
    }

    if ((sinst = scheme_inst_alloc(DH_size(dhl->dh))) == NULL) {
        LOG("couldn't allocate scheme instance\n");
        return NULL;
    }

    BN_init(&pub_key);
    BN_bin2bn(data, GX_LEN, &pub_key);
    LOG_BN(&pub_key, "read other sides dh g^x mod p: ");
    DH_compute_key(sinst->secret, &pub_key, dhl->dh);

    HEXDUMP(D_SCHEME, "resulting dh key", sinst->secret, sinst->secret_len);

    return sinst;
}

MODULE_INIT(struct our_config *cf)
{
    struct dh_local *dhl;

    if( (dhl = calloc(1, sizeof(*dhl))) == NULL) 
        return -ENOMEM;

    if( (dhl->dh = DH_new()) == NULL) {
        free(dhl);
        return -ENOMEM;
    }

    BN_hex2bn(&dhl->dh->p, P_str);
    BN_hex2bn(&dhl->dh->g, G_str);

    LOG_BN(dhl->dh->p, "initializes ->p: ");
    LOG_BN(dhl->dh->g, "initializes ->g: ");

    if (DH_generate_key(dhl->dh) < 0)
	LOG("DH_generate_key pooched: %ld\n", ERR_get_error());

    dhl->funcs.append = dh_append;
    dhl->funcs.parse = dh_parse;
    dhl->funcs.scheme_data = dhl;

    return scheme_register(&dhl->funcs, SCHEME_DH);

}
