/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "munge.h"

/*
 * some simple mungers 
 */
static void *null_init(unsigned char *data, size_t len)
{
    return (void *)1;
}
static size_t null_func(unsigned char *in, unsigned char *out, size_t len, 
                    void *state)
{
    memcpy(out, in, len);
    return len;
}
static void null_destroy(void *state)
{
    return ;
}
struct munger m_null = {
    init:       null_init,
    in:         null_func,
    out:        null_func,
    destroy:    null_destroy
};

static void *xor_init(unsigned char *data, size_t len)
{
    return (void *) (data ? ((unsigned long)data[0]) : 0x42);
}
static size_t xor_func(unsigned char *in, unsigned char *out, size_t len, 
                    void *state)
{
    size_t i;
    for(i = 0; i < len; i++)
        out[i] = in[i] ^ (unsigned char)((unsigned long)state);

    return len;
}
static void xor_destroy(void *state)
{
    return ;
}
struct munger m_xor = {
    init:       xor_init,
    in:         xor_func,
    out:        xor_func,
    destroy:    xor_destroy
};

/*
 * munger management :)
 */

extern struct munger m_gzip;

static struct mungers {
    int type; 
    struct munger *m;
} mungers[] = {
    {M_NULL, &m_null},
    {M_XOR, &m_xor},
    {M_GZIP, &m_gzip},

    {0,}
};

struct munger * munger_new(int munger_type, void *data, size_t len)
{
    struct munger *new;

    if(munger_type < M_MIN || munger_type >= M_MAX)
        return NULL;

    if( (new = calloc(1, sizeof(struct munger))) == NULL)
        return NULL;

    memcpy(new, mungers[munger_type].m, sizeof(struct munger));

    new->state = new->init(data, len);

    if(new->state == NULL) {
        free(new);
        return NULL;
    }

    return new;
}

int munger_destroy(struct munger *m)
{
    m->destroy(m->state);

    free(m);

    return 0;
}
