#ifndef __NEG_H__
#define __NEG_H__

#include "auth.h"
#include "cf.h"
#include "op.h"
#include "scheme.h"

#define NEG_ID_BYTES 16

#define SECRET_BYTES 64

/*
 * next header values in the negotiation headers.. these starting at
 * 128 makes it easier for packet dissectors to guess that the packets
 * are negotiation packets rather than encrypted garbage.
 */
/*
 * neg packet types
 */
enum {
    NEG_LISTAUTH = 128, 
    NEG_LISTSCHEME, 
    NEG_LISTOPT, 
    NEG_PICKOPT,
};

/*
 * neg payload types 
 */
typedef enum {
    PL_LISTAUTH = 0,
    PL_PICKAUTH,
    PL_LISTSCHEME,
    PL_PICKSCHEME,
    PL_LISTOPT,
    PL_PICKOPT,
} neg_payload_type;

/*
 * option types
 */
enum {
    OPT_IPV4ADDR = 0,
    OPT_NEXTHOP,
};

struct nneghdr {
    unsigned char   initiator_id[NEG_ID_BYTES];
    unsigned char   responder_id[NEG_ID_BYTES];
    guint8          type;
    guint8          __reserved;
    guint16         flags;
} __attribute__ ((packed));

struct neg_instance {
    /*
     * initialized by the caller before _begin()
     */
    void (*send_packet)(struct neg_instance  *neg);
    int (*append_options)(struct neg_instance *neg, struct our_packet *op);
    void (*auth_append)(struct neg_instance *neg);
    int (*parse_options)(struct neg_instance *neg, 
            unsigned char *data, unsigned int len);
    void (*destroy)(struct neg_instance *neg);
    unsigned int tx_attempts;
    void *private_data;
    /*
     * initialized by us before being passed to
     * the callbacks via neg_instance pointers.
     * the callbacks can assume ownership of these
     * by zeroing the pointers before the 
     * callbacks return, that will stop us
     * from trying to clean them up.
     */
    struct scheme_instance *sinst;
    struct auth_instance *ainst;
    struct our_packet *op;
    /*
     * internal to neg.c
     */
    gint timeout_id;
};

int neg_init(struct our_config *cf);
int neg_begin(struct neg_instance *neg);
void neg_cleanup(struct neg_instance *neg);

struct auth_instance * generate_listsch( struct our_packet *op,
                                         unsigned char *data, int data_len);
void generate_listopt(struct our_packet *in_op);
int generate_pickopt(struct neg_instance *ne, struct our_packet *op,
                     unsigned char *data, unsigned int data_len);
struct neg_instance * process_pickopt(struct our_packet *op);

#endif
