#ifndef __ROUTE_H__
#define __ROUTE_H__

#include <glib.h>

#include "circs.h"

#define CTO1_MAC_BYTES 10

#define NYMIP_OVERHEAD (20 + 20 /* ip+udp header */ + \
        2*sizeof(struct nlnkhdr) + sizeof(struct ne2ehdr))

/*
 * client to one hop options
 */
enum {
    CTO1_NXTHOP = 0
};

enum {
    E2E_IP
};

enum {
    CTL_LINK_DESTROY,
    CTL_TUNNEL_DESTROY
};

struct nctlhdr {
    guint16 cmd, __reserved;
};

struct nlnkhdr {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    guint32     nh:8,
                seq:24;
#elif __BYTE_ORDER == __BIG_ENDIAN
    guint32     seq:24,
                nh:8;
#endif
    guint32 circuit_id;
} __attribute__ ((packed));

/*
 * end to end header, follows the optional
 * mac after the link header
 */
struct ne2ehdr {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    guint32     nh:8,
                seq:24;
#elif __BYTE_ORDER == __BIG_ENDIAN
    guint32     seq:24,
                nh:8;
#endif
    /* data */
} __attribute__ ((packed));


extern void init_seq(struct seqctl *seqctl);
extern void route_incoming(struct sock_endpoint *sock, struct our_packet *op, 
        struct sockaddr_in *from);
extern void route_enter(struct circuit *circ, struct our_packet *op);
extern struct sock_endpoint * new_sock_endpoint(struct our_config *cf, 
        struct sockaddr_in *sin);
void sock_endpoint_destroy(struct sock_endpoint *sock);
extern void route_link_send(struct link *link, struct our_packet *op, 
        guint8 nh, guint32 circid);
extern void route_link_send_u32(struct link *link, struct our_packet *op, 
        guint8 nh, guint32 circid, guint32 junk);
extern void circ_send_cto1(struct circuit *circ, struct our_packet *op, 
        guint8 nh);
void route_send_control(struct link *link, guint16 cmd, 
        guint32 circuit_id);

#endif
