/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * 
 * The Chainsaw anonymous IP network daemon
 *
 * Copyright (c) 2001 Zero-Knowledge Systems, Inc.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <sys/uio.h>

#include <glib.h>

#include "route.h"
#include "util.h"
#include "links.h"
#include "op.h"
#include "log.h"

#ifndef MSG_ERRQUEUE
#  define MSG_ERRQUEUE 0x2000
#endif

static gboolean socket_prepare(gpointer source_data,
    GTimeVal *current_time, gint *timeout, gpointer user_data)
{
    *timeout = -1;
    return FALSE;
}

static char throw_away[8192];

static gboolean socket_check(gpointer source_data,
    GTimeVal *current_time, gpointer user_data)
{
    GPollFD *pfd = source_data;

    /*
     * XXX hack to clear pending udp errors
     * under linux
     */

    if(pfd->revents & G_IO_ERR) {
        struct msghdr msg;
        struct iovec iov;
        int len;

        msg.msg_name = NULL;
        msg.msg_namelen = 0;
        msg.msg_iov = &iov;
        msg.msg_iovlen = 1;
        iov.iov_base = throw_away;
        iov.iov_len = 8192;
        msg.msg_control = throw_away;
        msg.msg_controllen = 8192;

        while( ( len = recvmsg(pfd->fd, &msg, MSG_ERRQUEUE)) > 0 ) {
            HEXDUMP(D_ERROR, "Received udp error",iov.iov_base, len);
        }
    }

    return (pfd->revents & G_IO_IN);
}

static unsigned char oh_crap[MAX_PACKET_LEN];

/*
 * dunno what the return code means here..
 */
static gboolean socket_dispatch (gpointer source_data, 
    GTimeVal *current_time, gpointer user_data)
{
    GPollFD *pfd = source_data;
    struct sock_endpoint *sock = user_data;
    struct our_packet *op;
    int len;
    struct sockaddr_in from;
    socklen_t fromlen = sizeof(struct sockaddr_in);

    if( (op = op_create()) == NULL) {
        LOG("op_create failed\n");
        read(pfd->fd, oh_crap, MAX_PACKET_LEN);
        return TRUE;
    }

    if( (len = recvfrom(pfd->fd, op->head, MAX_PACKET_LEN, 0, &from, &fromlen)) < 0) 
        goto error;

    op_push_tail(op, len);

    route_incoming(sock, op, &from);

    return TRUE;

error:
    op_free(op);
    return TRUE;
}

static void socket_destroy(gpointer data)
{
    GPollFD *pfd = data;
    g_main_remove_poll(pfd);
    free(pfd);
}

GSourceFuncs socket_source_funcs = {
    socket_prepare,
    socket_check,
    socket_dispatch,
    socket_destroy,
};

int socketin_init(struct sock_endpoint *sock)
{
    GPollFD *pfd;
    if( (pfd = calloc(1, sizeof(*pfd))) == NULL) {
        LOG("no mem for gpollfd\n");
        return 1;
    }

    pfd->fd = sock->fd;
    pfd->events = G_IO_IN | G_IO_HUP | G_IO_ERR;

    g_main_add_poll(pfd, G_PRIORITY_DEFAULT);

    g_source_add(G_PRIORITY_DEFAULT, FALSE,
            &socket_source_funcs,
            pfd, 
            sock, 
            NULL);

    return 0;
}
