#ifndef __UTIL_H__
#define __UTIL_H__

#include <sys/types.h>
#include <sys/socket.h>
#include <glib.h>
#include <openssl/bn.h>

#include "op.h"
#include "socketin.h"

/* Timing macros */
#define rdtsc(tim) __asm__ __volatile__("rdtsc" : "=a" ((tim)->low), "=d" ((tim)->high))
#define START_TIMER() { rdtsc(&start); }
#define REPORT(MESSAGE) { rdtsc(&end) ; print_diff(MESSAGE, &start, &end); }
struct tim {
    unsigned long low, high;
};

extern void print_diff(char *message, struct tim *start, struct tim *end);

#define ARRAY_SIZE(x) ( sizeof(x) / sizeof(x[0]) )

/*
 * these really should be somewhere else, but our header files
 * really don't like including each other :/
 */
#define SEQ_BF_TYPE unsigned long
#define SEQ_BF_WIDTH (sizeof(SEQ_BF_TYPE) * 4)
struct seqctl {
    guint32 left,right;
    SEQ_BF_TYPE bf;
};

#define htonl24(X) (htonl(X) >> 8)
#define ntohl24(X) (htonl(X << 8))

extern int set_nonblock(int fd);
extern int create_bound_udp(struct sockaddr_in *sin);
extern void send_op_payload(int fd, struct sockaddr_in *sin, struct our_packet *op);
extern gboolean u32_equal(gconstpointer a, gconstpointer b);
extern guint u32_hash(gconstpointer a);
extern gboolean u16_equal(gconstpointer a, gconstpointer b);
extern guint u16_hash(gconstpointer a);
extern gint memcmp_16(gconstpointer a, gconstpointer b);

extern guint sockaddr_in_hash(gconstpointer key);
extern gboolean sockaddr_in_equal(gconstpointer a, gconstpointer b);
extern gint sockaddr_in_compare(gconstpointer a, gconstpointer b);

extern int lame_global_socket;
extern int get_dev_addr(struct sockaddr_in *sin, char *dev);

extern guint32 ip_allocate(struct sock_endpoint *sock);
extern void ip_deallocate(struct sock_endpoint *sock, guint32 ip);

extern int bignum_to_array(BIGNUM *bn, guint8 *buffer, int buffer_len);

#endif
