// =============================================================================
//	ADBSource.h								 (c)2000 Zero-Knowledge Systems Inc.
// =============================================================================
//	

#ifndef ADBSource_H_
#define ADBSource_H_

#if defined(__MWERKS__)
#	pragma once
#endif

// system headers
#include <DeskBus.h>

// project headers
#include "EntropySource.h"
#include "YarrowUtils.h"


// forward declarations
struct	ADBInitHook;
struct	ADBServiceData;
struct	ADBServiceHook;


class ADBSource : public EntropySource
{
protected:
	
	// constants
	enum				{ kMaxADBAddrs = 16 };
	
	// accessing the sole instance
	static ADBSource&	Get();
	
	// constructors / destructor
						ADBSource(
							const Byte* const	inSourceName,
							const char* const	inLogName);
	virtual				~ADBSource();
	
	// life cycle
	virtual OSStatus	Install(
							SInt16			inResFile);
	virtual OSStatus	Uninstall();
	virtual OSStatus	Open(
							UInt16			inSource, 
							DriverRefNum	inDriverRefNum);
	virtual OSStatus	Close();
	virtual OSStatus	Activate(
							Boolean			inFlagIt = true);
	virtual OSStatus	Deactivate();
	virtual void		Sleep();
	virtual void		Wake();
	
	virtual void		InstallServiceHooks();
	void				UninstallServiceHooks(
							Boolean				inReinit);
	virtual Boolean		AcceptDevice(
							ADBAddress			inAddr, 
							const ADBDataBlock&	inData);
	
	virtual void		HandleData(
							ADBAddress			inAddr, 
							StringPtr			inData, 
							UInt8				inCmd);
	
	// member variables
	ADBServiceData*		mServiceData[kMaxADBAddrs];
	
private:
	
	void				HandleReinit(
							Boolean				inPostProcess);
	
	// callbacks
	static pascal void	ADBInitProc(
							SInt8						callOrder REG_D0_ARG);
	static pascal void	ADBServiceProc(
							Ptr							buffer, 
							TempADBServiceRoutineUPP	completionProc, 
							Ptr							refCon, 
							long						command);
#if TARGET_CPU_68K && !TARGET_RT_MAC_CFM
	static pascal void	ADBServiceProcAsm();
#endif
	
	// member variables
	ADBInitHook*		mInitHook;
	ADBServiceHook*		mServiceHook;
	
	// static member variables
	static ADBSource*			sSource;
	static ADBInitUPP			sADBInitUPP;
	static ADBServiceRoutineUPP	sADBServiceUPP;
#if TARGET_RT_MAC_CFM
	static RoutineDescriptor	sADBInitRD;
	static RoutineDescriptor	sADBServiceRD;
#endif
};


//	Inline member function definitions

inline ADBSource&
ADBSource::Get()
{
	return (*sSource);
}


#endif	/* ADBSource_H_ */
