// =============================================================================
//	ADBUtils.cp								 (c)2000 Zero-Knowledge Systems Inc.
// =============================================================================
//	

// system headers
#include <Memory.h>

// compiler headers
#include <A4Stuff.h>

// library headers
#include "AtmOps.h"
#include "MoreOSUtils.h"

// project headers
#include "ADBUtils.h"
#include "YarrowUtils.h"


#pragma options align=mac68k

struct ADBInitHook {
	UInt16		bra;
	UInt16		fakeProc;
	SInt32		ourA4;
	ADBInitUPP	ourProc;
	ADBInitUPP	prevProc;
	UInt16		instrs[1];
};

typedef struct ADBServiceHook {
	UInt16		bra;
	UInt16		fakeProc;
	SInt32		refcount;
	UInt16		instrs[1];
} ADBServiceHook;

struct ADBServiceData {
	SInt32					ourA4;
	ADBServiceRoutineUPP	ourProc;
	SInt32					ourData;
	ADBServiceRoutineUPP	prevProc;
	Ptr						prevData;
};

#pragma options align=reset

// prototypes of local functions
extern "C" {
	void	ADBReinitHookProc(void);
	void	ADBReinitHookProc_End(void);
	void	ADBServiceHookProc(void);
	void	ADBServiceHookProc_End(void);
}


// -----------------------------------------------------------------------------
//	 LMGetADBInitPtr
// -----------------------------------------------------------------------------
//

inline volatile ADBInitUPP*	LMGetADBInitPtr(void)
{
	return (((volatile ADBInitUPP *) 0x06B8));
}


// -----------------------------------------------------------------------------
//	 InstallADBInitHook
// -----------------------------------------------------------------------------
//

OSStatus	InstallADBInitHook(
	SInt32			inGlobals, 
	ADBInitHook**	outHook)
{
	Size			hookSize;
	ADBInitHook*	hook;
	OSStatus		err;
	
	hookSize = (char *) ADBReinitHookProc_End - (char *) ADBReinitHookProc;
	
	hook = (ADBInitHook *) NewPtrSys(hookSize);
	
	if (hook != NULL)
	{
		BlockMoveData(ADBReinitHookProc, hook, hookSize);
		
		hook->ourA4		= inGlobals;
		hook->ourProc	= (ADBInitUPP) &hook->fakeProc;
		
		MakeData68KExecutable(hook, (ByteCount) hookSize);
		
		volatile ADBInitUPP*	uppPtr	= LMGetADBInitPtr();
		
		do
		{
			hook->prevProc = *uppPtr;
			
		} while (!AtmCompareAndSwap((UInt32) hook->prevProc, 
									(UInt32) hook, 
									(UInt32 *) uppPtr));
		
		*outHook	= hook;
		err			= noErr;
	}
	else
	{
		err = memFullErr;
	}
	
	return (err);
}


// -----------------------------------------------------------------------------
//	 ActivateADBInitHook
// -----------------------------------------------------------------------------
//	

void	ActivateADBInitHook(
	ADBInitHook*	inHook, 
	ADBInitUPP		inUPP)
{
	ADBInitUPP	oldUPP;
	
	do
	{
		oldUPP = inHook->ourProc;
		
	} while (!AtmCompareAndSwap((UInt32) oldUPP, 
								(UInt32) inUPP, 
								(UInt32 *) &inHook->ourProc));
}


// -----------------------------------------------------------------------------
//	 DeactivateADBInitHook
// -----------------------------------------------------------------------------
//	

void	DeactivateADBInitHook(
	ADBInitHook*	inHook, 
	ADBInitUPP*		outUPP)
{
	ADBInitUPP	oldUPP;
	
	do
	{
		oldUPP = inHook->ourProc;
		
	} while (!AtmCompareAndSwap((UInt32) oldUPP, 
								(UInt32) &inHook->fakeProc, 
								(UInt32 *) &inHook->ourProc));
	
	if (outUPP != NULL)
		*outUPP = oldUPP;
}


// -----------------------------------------------------------------------------
//	 InstallADBServiceHook
// -----------------------------------------------------------------------------
//	

OSStatus	InstallADBServiceHook(
	ADBAddress				inAddr, 
	const ADBDataBlock*		inData, 
	SInt32					inGlobals, 
	ADBServiceHook**		ioHook, 
	ADBServiceData**		outData)
{
	ADBServiceHook*	hook	= *ioHook;
	OSStatus		err		= noErr;
	
	if (hook == NULL)
	{
		Size	hookSize;
		
		hookSize = (char *) ADBServiceHookProc_End - (char *) ADBServiceHookProc;
		
		hook = (ADBServiceHook *) NewPtrSys(hookSize);
		
		if (hook != NULL)
		{
			BlockMoveData(ADBServiceHookProc, hook, hookSize);
			
			MakeData68KExecutable(hook, (ByteCount) hookSize);
			
			*ioHook = hook;
		}
		else
		{
			err = memFullErr;
		}
	}
	
	if (err == noErr)
	{
		ADBServiceData*	data;
		
		data = (ADBServiceData *) NewPtrSys(sizeof(*data));
		
		if (data != NULL)
		{
			ADBSetInfoBlock	info;
			
			data->ourA4		= inGlobals;
			data->ourProc	= (ADBServiceRoutineUPP) &hook->fakeProc;
			data->ourData	= NULL;
			data->prevProc	= inData->dbServiceRtPtr;
			data->prevData	= inData->dbDataAreaAddr;
			
			info.siService		= (ADBServiceRoutineUPP) hook;
			info.siDataAreaAddr	= (Ptr) data;
			
			err = SetADBInfo(&info, inAddr);
			
			if (err == noErr)
			{
				AtmInc(&hook->refcount);
				
				*outData = data;
			}
		}
		else
		{
			err = memFullErr;
		}
		
		if (err != noErr)
		{
			if (hook->refcount == 0)
			{
				DisposePtr((Ptr) hook);
				*ioHook = NULL;
			}
		}
	}
	
	return (err);
}


// -----------------------------------------------------------------------------
//	 UninstallADBServiceHook
// -----------------------------------------------------------------------------
//	

void	UninstallADBServiceHook(
	ADBAddress				inAddr, 
	ADBServiceHook**		ioHook, 
	ADBServiceData*			inData,
	Boolean					/* inReinit */, 
	ADBServiceRoutineUPP*	outUPP)
{
	ADBServiceHook*	hook		= *ioHook;
	Boolean			canDelete	= false;
	ADBDataBlock	adbData;
	OSStatus		err;
	
	DeactivateADBServiceHook(hook, inData, outUPP);
	
	err = GetADBInfo(&adbData, inAddr);
	
	if ((err == noErr) && 
		(adbData.dbServiceRtPtr == (ADBServiceRoutineUPP) hook) && 
		(adbData.dbDataAreaAddr == (Ptr) inData))
	{
		ADBSetInfoBlock	info;
		
		info.siService		= inData->prevProc;
		info.siDataAreaAddr	= inData->prevData;
		
		err = SetADBInfo(&info, inAddr);
		
		if (err == noErr)
			canDelete = true;
	}
	
	if (canDelete)
	{
		DisposePtr((Ptr) inData);
		
		if (AtmDec(&hook->refcount) == 0)
		{
			DisposePtr((Ptr) hook);
			*ioHook = NULL;
		}
	}
}


// -----------------------------------------------------------------------------
//	 ActivateADBServiceHook
// -----------------------------------------------------------------------------
//	

void	ActivateADBServiceHook(
	ADBServiceHook*			/* inHook */, 
	ADBServiceData*			inData, 
	ADBServiceRoutineUPP	inUPP, 
	SInt32					inArg)
{
	UInt16	savedInterruptMask	= SetInterruptMask(7);
	
	inData->ourProc	= inUPP;
	inData->ourData	= inArg;
	
	SetInterruptMask(savedInterruptMask);
}


// -----------------------------------------------------------------------------
//	 DeactivateADBServiceHook
// -----------------------------------------------------------------------------
//	

void	DeactivateADBServiceHook(
	ADBServiceHook*			inHook, 
	ADBServiceData*			inData, 
	ADBServiceRoutineUPP*	outUPP)
{
	UInt16	savedInterruptMask	= SetInterruptMask(7);
	
	if (outUPP != NULL)
		*outUPP = inData->ourProc;
	
	inData->ourProc	= (ADBServiceRoutineUPP) &inHook->fakeProc;
	inData->ourData	= NULL;
	
	SetInterruptMask(savedInterruptMask);
}


// -----------------------------------------------------------------------------
//	 SendADBCommand
// -----------------------------------------------------------------------------
//	Sends a command to an ADB device.  For Listen commands, buff contains 
//	the data to transmist.  For Talk commands, it contains on return the 
//	received data.	

#if TARGET_CPU_68K && !TARGET_RT_MAC_CFM

//	68K register usage:
//		A0.L	- buffer
//		A2.L	- refCon
//		D0.B	- command

asm static pascal void	SendADBCommandProc(void)
{
	move.b	#1,(a2)
	rts
}

#else	// TARGET_CPU_68K && !TARGET_RT_MAC_CFM

static void	SendADBCommandProc(Ptr /* buffer */, Ptr refCon, long /* command */)
{
	*refCon = true;
}

#endif	// !(TARGET_CPU_68K && !TARGET_RT_MAC_CFM)

#if TARGET_RT_MAC_CFM
static RoutineDescriptor	sSendADBCommandRD	= BUILD_ROUTINE_DESCRIPTOR(uppADBCompletionProcInfo, SendADBCommandProc);
static ADBCompletionUPP		sSendADBCommandUPP	= &sSendADBCommandRD;
#else
static ADBCompletionUPP		sSendADBCommandUPP	= SendADBCommandProc;
#endif

OSStatus	SendADBCommand(
	StringPtr				buff, 
	SInt16					cmd)
{
	volatile Boolean	done	= false;
	int					retries	= 3;
	OSStatus			err;
	
	do
	{
		err = ADBOp((Ptr) &done, sSendADBCommandUPP, (Ptr) buff, cmd);
		
		if (err == noErr)
		{
			while (!done)
			{
///				DelayMinimum();
			}
		}
		else if (retries > 0)
		{
///			DelayMinimum();
		}
		
	} while ((err != noErr) && (retries-- > 0));
	
	return (err);
}


#pragma macsbug off

asm void	ADBReinitHookProc(void)
{
	bra.s	@start
	
@fakeProc:
	bra.s	@empty
	
@ourA4:
	dc.l	0
	
@ourProc:
	dc.l	0
	
@prevProc:
	dc.l	0

@start:
	movem.l	d0/a4,-(sp)
	movea.l	@ourA4,a4
	movea.l	@ourProc,a0
	jsr		(a0)
	movem.l	(sp)+,d0/a4
	movea.l	@prevProc,a0
	jmp		(a0)
	
@empty:
	rts
	
	entry	ADBReinitHookProc_End
}

asm void	ADBServiceHookProc(void)
{
	bra.s	@start
	
@fakeProc:
	bra.s	@empty
	
@count:
	dc.l	0
	
@start:
	movem.l	d0-d2/a0/a2/a4,-(sp)
	movea.l	(a2)+,a4
	movea.l	(a2)+,a1
	movea.l	(a2)+,a2
	jsr		(a1)
	movem.l	(sp)+,d0-d2/a0/a2/a4
	lea		12(a2),a2
	movea.l	(a2)+,a1
	movea.l	(a2)+,a2
	jmp		(a1)
	
@empty:
	rts
	
	entry	ADBServiceHookProc_End
}
