// ============================================================================
//	CYarrowToolApp.cp	2000 Zero-Knowledge Systems, Inc. All rights reserved.
// ============================================================================

// standard headers
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

// system headers
#include <Appearance.h>
#include <Dialogs.h>
#include <Folders.h>
#include <Gestalt.h>
#include <LowMem.h>

// PowerPlant headers
#include <LGrowZone.h>
#include <LIconPane.h>
#include <LWindow.h>
#include <PP_KeyCodes.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <UDesktop.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UModalDialogs.h>
#include <UEnvironment.h>
#include <UReanimator.h>

// library headers
#include "TradDriverLoaderLib.h"

// project headers
#include "CYarrowToolApp.h"
#include "DriverLoader.h"
#include "YarrowDriverDefs.h"
#include "YarrowDriverVers.h"
#include "YarrowGestalt.h"
#include "yarrow.h"
#include "YarrowInstallClient.h"

using namespace std;


// Constant declarations
const ResIDT	PPob_YarrowToolWindow		= 1000;
const PaneIDT	kDriverLoadedTextPaneID		= 1003;
const PaneIDT	kLoadDriverButtonPaneID		= 1004;
const PaneIDT	kDriverOpenedTextPaneID		= 1005;
const PaneIDT	kOpenDriverButtonPaneID		= 1006;
const PaneIDT	kNumUnseededLabelPaneID		= 1007;
const PaneIDT	kNumUnseededTextPaneID		= 1008;
const PaneIDT	kClosestSourceLabelPaneID	= 1009;
const PaneIDT	kClosestSourceTextPaneID	= 1010;
const PaneIDT	kEntropyBitsLabelPaneID		= 1011;
const PaneIDT	kEntropyBitsTextPaneID		= 1012;
const PaneIDT	kEntropyMaxLabelPaneID		= 1013;
const PaneIDT	kEntropyMaxTextPaneID		= 1014;
const PaneIDT	kPRNGSeededTextPaneID		= 1016;

const ResIDT	PPob_AlertWindow			= 1100;
const PaneIDT	kAlertOKButtonPaneID		= 1101;
const PaneIDT	kAlertErrorTextPaneID		= 1102;
const PaneIDT	kAlertReasonTextPaneID		= 1103;
const PaneIDT	kAlertIconPaneID			= 1104;

const ResIDT	kToolWindowStrsID			= 1000;
enum {
	kDriverClosedIndex = 1,
	kDriverOpenIndex,
	kOpenBtnTextIndex,
	kCloseBtnTextIndex,
	kPRNGNotSeededIndex,
	kPRNGSeededIndex,
	kShowToolWindowIndex,
	kHideToolWindowIndex,
	
	kInstallIndex,
	kUninstallIndex,
	kWhyIndex,
	kCantInstallTextIndex,
	kNotInstalledTextIndex,
	kBadVersionTextIndex,
	kOldVersionTextIndex,
	kNewVersionTextIndex,
	kInstalledTextIndex
};

const ResIDT	kErrorStrsID			= 1001;
enum {
	kInitNotLoadedErrorMsgIndex = 1,
	kInitNotLoadedErrorReasonIndex,
	kCantLoadDriverErrorMsgIndex,
	kCantLoadDriverErrorReasonIndex,
	kCantUnloadDriverErrorMsgIndex,
	kCantUnloadDriverErrorReasonIndex,
	kNoDriverStatusErrorMsgIndex,
	kNoDriverStatusErrorReasonIndex,
	kCantOpenDriverErrorMsgIndex,
	kCantOpenDriverErrorReasonIndex,
	kCantCloseDriverErrorMsgIndex,
	kCantCloseDriverErrorReasonIndex
};

const CommandT	kFirstTraceCmd	= 1000;
const CommandT	kLastTraceCmd	= 1015;
const CommandT	kTraceAllOnCmd	= 1020;
const CommandT	kTraceAllOffCmd	= 1021;
const CommandT	kToolWindowCmd	= 2000;

const UInt32	kRefreshTicks	= 30;	/* 0.5 seconds */


// ===========================================================================
//	 main
// ===========================================================================

int main()
{							
		// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

		// Initialize Memory Manager. Parameter is the number of
		// master pointer blocks to allocate
	InitializeHeap(3);
	
		// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox();
	
		// Install a GrowZone to catch low-memory situations	
	new LGrowZone(20000);
	
		// Create the application object and run
	CYarrowToolApp	theApp;
	theApp.Run();
	
	return 0;
}


// ---------------------------------------------------------------------------
//	 CYarrowToolApp								[public]
// ---------------------------------------------------------------------------
//	Application object constructor

CYarrowToolApp::CYarrowToolApp()
	: mWindow(NULL), /* mInitWasLoaded(false), mDriverIsLoaded(false), */ 
	  mDriverIsOpen(false), mLastTicks(0)
{
		// Register ourselves with the Appearance Manager
	if (UEnvironment::HasFeature(env_HasAppearance)) {
		::RegisterAppearanceClient();
	}

	RegisterClasses();
}


// ---------------------------------------------------------------------------
//	 ~CYarrowToolApp								[public, virtual]
// ---------------------------------------------------------------------------
//	Application object destructor

CYarrowToolApp::~CYarrowToolApp()
{
}


// ---------------------------------------------------------------------------
//	 StartUp										[protected, virtual]
// ---------------------------------------------------------------------------
//	Perform an action in response to the Open Application AppleEvent.

void
CYarrowToolApp::StartUp()
{
#if 0
	mInitWasLoaded	= IsExtensionLoaded();
#endif
	
	mWindow = LWindow::CreateWindow(PPob_YarrowToolWindow, this);
	ThrowIfNil_(mWindow);
	
	UReanimator::LinkListenerToControls(this, mWindow, PPob_YarrowToolWindow);
	
#if 0
	
	if (mInitWasLoaded)
#endif
	{
		FillInFields();
	}
#if 0
	else
	{
///		SetTextBtnState(kDriverLoadedTextPaneID, kDriverNotLoadedIndex, false, false);
///		SetTextBtnState(kDriverOpenedTextPaneID, kDriverClosedIndex, false, false);
		
///		for (PaneIDT paneID = kNumUnseededLabelPaneID; 
			 paneID <= kEntropyMaxTextPaneID; 
			 paneID++)
		{
			EnablePaneByID(mWindow, paneID, false);
		}
	}
#endif
	
	mWindow->Show();
}


// ---------------------------------------------------------------------------
//	 DoReopenApp												   [protected]
// ---------------------------------------------------------------------------
//	Respond to Reopen Application AppleEvent
//
//	The system sends the reopen application AppleEvent when the user
//	resumes an application that has no open windows.

void
CYarrowToolApp::DoReopenApp()
{
	if (mWindow != NULL)
	{
		mWindow->Show();
		mWindow->Select();
	}
}


// ---------------------------------------------------------------------------
//	 SetPaneText								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::SetPaneText(PaneIDT paneID, short textIndex)
{
	Str255	str;
	
	GetIndString(str, kToolWindowStrsID, textIndex);
	mWindow->SetDescriptorForPaneID(paneID, str);
}


// ---------------------------------------------------------------------------
//	 SetTextState								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::SetTextState(PaneIDT paneID, short textIndex, Boolean state)
{
	short	stateOffset	= state ? 1 : 0;
	Str255	str;
	
	GetIndString(str, kToolWindowStrsID, textIndex + stateOffset);
	mWindow->SetDescriptorForPaneID(paneID, str);
}


// ---------------------------------------------------------------------------
//	 SetTextBtnState								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::SetTextBtnState(PaneIDT paneID, short textIndex, Boolean state, Boolean enableBtn)
{
	short	stateOffset	= state ? 1 : 0;
	LPane*	btnPane		= mWindow->FindPaneByID(paneID+1);
	Str255	str;
	
	SetTextState(paneID, textIndex, state);
	
	GetIndString(str, kToolWindowStrsID, textIndex + 2 + stateOffset);
	mWindow->SetDescriptorForPaneID(paneID+1, str);
	EnablePaneByID(mWindow, paneID, enableBtn);
	EnablePaneByID(mWindow, paneID+1, enableBtn);
}


// ---------------------------------------------------------------------------
//	 FillInFields								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::FillInFields()
{
	short		textIndex, titleIndex;
	Boolean		titleEnable;
	
	titleIndex	= kUninstallIndex;
	titleEnable	= true;
	
	mInstallState = IsDriverInstalled();
	
	switch (mInstallState)
	{
	case kYarrowCantInstall:
	case kYarrowBadVersion:
		textIndex	= kCantInstallTextIndex;
		titleIndex	= kWhyIndex;
		break;
		
	case kYarrowNotInstalled:
		textIndex	= kNotInstalledTextIndex;
		titleIndex	= kInstallIndex;
		break;
		
	case kYarrowOldVersion:
		textIndex	= kOldVersionTextIndex;
		break;
		
	case kYarrowNewVersion:
		textIndex	= kNewVersionTextIndex;
		break;
		
	case kYarrowInstalled:
		textIndex	= kInstalledTextIndex;
		break;
	}
	
	SetPaneText(kDriverLoadedTextPaneID, textIndex);
	SetPaneText(kLoadDriverButtonPaneID, titleIndex);
	EnablePaneByID(mWindow, kLoadDriverButtonPaneID, titleEnable);
	
	mDriverIsOpen = false;
	
	if ((mInstallState == kYarrowBadVersion) || 
		(mInstallState == kYarrowOldVersion) || 
		(mInstallState == kYarrowNewVersion) || 
		(mInstallState == kYarrowInstalled))
	{
		UnitNumber		unit;
		DriverFlags		flags;
		DriverOpenCount	count;
		
		if (YarrowFindNamedDriver(kYarrowDriverName, mDriverRefNum, unit, flags, count) && 
			(count > 0))
		{
			mDriverIsOpen = true;
			
			Str63	driverName;
			
			YarrowGetDriverInformation(mDriverRefNum, unit, flags, count, driverName);
		}
		
		if (mDriverIsOpen)
		{
			textIndex	= kDriverOpenIndex;
			titleIndex	= kCloseBtnTextIndex;
		}
		else
		{
			textIndex	= kDriverClosedIndex;
			titleIndex	= kOpenBtnTextIndex;
		}
		
		titleEnable = true;
		SetTextBtnState(kDriverOpenedTextPaneID, kDriverClosedIndex, 
						mDriverIsOpen, true);
	}
	else
	{
		textIndex	= kDriverClosedIndex;
		titleIndex	= kOpenBtnTextIndex;
		titleEnable = false;
	}
	
	SetPaneText(kDriverOpenedTextPaneID, textIndex);
	SetPaneText(kOpenDriverButtonPaneID, titleIndex);
	EnablePaneByID(mWindow, kOpenDriverButtonPaneID, titleEnable);
	
	for (PaneIDT paneID = kNumUnseededLabelPaneID; 
		 paneID <= kEntropyMaxTextPaneID; 
		 paneID++)
	{
		EnablePaneByID(mWindow, paneID, mDriverIsOpen);
	}
	
	UInt16	debugFlags	= 0;
	
	if (mDriverIsOpen)
	{
		Boolean	seeded;
		
		mLastTicks = TickCount();
		seeded = RefreshDriverState();
		
		if (!IsRepeating() && !seeded)
		{
			StartRepeating();
		}
		else if (IsRepeating() && seeded)
		{
			StopRepeating();
		}
		
		GetDebugFlags(debugFlags);
	}
	else
	{
		if (IsRepeating())
			StopRepeating();
		
		for (PaneIDT paneID = kNumUnseededTextPaneID; 
			 paneID <= kEntropyMaxTextPaneID; 
			 paneID += 2)
		{
			mWindow->SetDescriptorForPaneID(paneID, "\p-");
		}
	}
	
	if (mDebugFlags != debugFlags)
	{
		mDebugFlags = debugFlags;
		SetUpdateCommandStatus(true);
	}
}


// ---------------------------------------------------------------------------
//	 RefreshDriverState								[protected]
// ---------------------------------------------------------------------------
//	

Boolean
CYarrowToolApp::RefreshDriverState()
{
	Boolean		seeded;
	int			num_sources;
	unsigned	source_id;
	size_t		entropy_bits, entropy_max;
	OSErr		err;
	
	err = DriverStatus(seeded, num_sources, source_id, entropy_bits, 
					   entropy_max);
	
	if (err == noErr)
	{
		SetTextState(kPRNGSeededTextPaneID, kPRNGNotSeededIndex, seeded);
		
		mWindow->SetValueForPaneID(kNumUnseededTextPaneID, num_sources);
		mWindow->SetValueForPaneID(kClosestSourceTextPaneID, source_id);
		mWindow->SetValueForPaneID(kEntropyBitsTextPaneID, entropy_bits);
		mWindow->SetValueForPaneID(kEntropyMaxTextPaneID, entropy_max);
		
		for (PaneIDT paneID = kNumUnseededLabelPaneID; 
			 paneID <= kEntropyMaxTextPaneID; 
			 paneID++)
		{
			EnablePaneByID(mWindow, paneID, !seeded);
		}
	}
	else
	{
		seeded = false;
		ShowErrorBox(kAlertStopAlert, kNoDriverStatusErrorMsgIndex, 
					 kNoDriverStatusErrorReasonIndex, err);
	}
	
	return (seeded);
}


// ---------------------------------------------------------------------------
//	 ObeyCommand													  [public]
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CYarrowToolApp::ObeyCommand(
	CommandT	inCommand,
	void*		ioParam)
{
	Boolean		cmdHandled = true;
	
	switch (inCommand)
	{
	case kToolWindowCmd:
		if (mWindow->IsVisible())
		{
			mWindow->Hide();
		}
		else
		{
			mWindow->Show();
			mWindow->Select();
		}
		break;
		
	case kTraceAllOnCmd:
		SetDebugFlags(0xFFFF);
		FillInFields();
		break;
		
	case kTraceAllOffCmd:
		SetDebugFlags(0);
		FillInFields();
		break;
		
	default:
		if ((inCommand >= kFirstTraceCmd) && (inCommand <= kLastTraceCmd))
		{
			UInt16	cmdMask		= (1 << (inCommand - kFirstTraceCmd));
			UInt16	debugFlags	= mDebugFlags;
			
			if (debugFlags & cmdMask)
				debugFlags &= ~cmdMask;
			else
				debugFlags |= cmdMask;
			
			SetDebugFlags(debugFlags);
			FillInFields();
		}
		else
		{
			cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
		}
		break;
	}
	
	return cmdHandled;
}


// ---------------------------------------------------------------------------
//	 FindCommandStatus												  [public]
// ---------------------------------------------------------------------------
//	Pass back whether a Command is enabled and/or marked (in a Menu)

void
CYarrowToolApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean&	outEnabled,
	Boolean&	outUsesMark,
	UInt16&		outMark,
	Str255		outName)
{
	switch (inCommand)
	{
	case kToolWindowCmd:
		outEnabled = (mWindow != NULL);
		if (outEnabled)
		{
			GetIndString(outName, kToolWindowStrsID, 
						 (mWindow->IsVisible() ? kHideToolWindowIndex 
						 					   : kShowToolWindowIndex));
		}
		break;
		
	case kTraceAllOnCmd:
	case kTraceAllOffCmd:
		outEnabled = mDriverIsOpen;
		break;
		
	default:
		if ((inCommand >= kFirstTraceCmd) && (inCommand <= kLastTraceCmd))
		{
			UInt16		cmdMask	= (1 << (inCommand - kFirstTraceCmd));
			
			outEnabled	= mDriverIsOpen;
			outUsesMark	= true;
			outMark		= ((mDebugFlags & cmdMask) ? char_Radical : 0);
		}
		else
		{
			LApplication::FindCommandStatus(inCommand, outEnabled,
											outUsesMark, outMark, outName);
		}
		break;
	}
}


// ---------------------------------------------------------------------------
//	 ListenToMessage								[public]
// ---------------------------------------------------------------------------
//	

#if TARGET_CPU_PPC
static void	usb_test();
#endif

void
CYarrowToolApp::ListenToMessage(
	MessageT		inMessage,
	void*			/* ioParam */)
{
	switch (inMessage)
	{
///	case kExtensionButtonPaneID:
///		ShowErrorBox(kAlertNoteAlert, kInitNotLoadedErrorMsgIndex, 
///					 kInitNotLoadedErrorReasonIndex);
///		break;
		
	case kLoadDriverButtonPaneID:
		switch (mInstallState)
		{
		case kYarrowCantInstall:
		case kYarrowBadVersion:
			ShowErrorBox(kAlertNoteAlert, kInitNotLoadedErrorMsgIndex, 
						 kInitNotLoadedErrorReasonIndex);
			break;
			
		case kYarrowNotInstalled:
			InstallDriver();
			FillInFields();
			break;
			
		case kYarrowOldVersion:
		case kYarrowNewVersion:
		case kYarrowInstalled:
			UninstallDriver();
			FillInFields();
			break;
		}
		break;
		
	case kOpenDriverButtonPaneID:
		if (mDriverIsOpen)
			CloseYarrowDriver();
		else
			OpenYarrowDriver();
		FillInFields();
		break;
	}
}


// ---------------------------------------------------------------------------
//	 SpendTime								[public]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::SpendTime(
	const EventRecord&	inMacEvent)
{
	if (inMacEvent.when >= mLastTicks + GetCaretTime())
	{
		Boolean	seeded;
		
		mLastTicks = inMacEvent.when;
		seeded = RefreshDriverState();
		
		if (IsRepeating() && seeded)
		{
			StopRepeating();
		}
	}
}


// ---------------------------------------------------------------------------
//	 AllowSubRemoval
// ---------------------------------------------------------------------------

Boolean
CYarrowToolApp::AllowSubRemoval(
	LCommander*	inSub)
{
	if ((mWindow != NULL) && (mWindow == inSub))
	{
		mWindow->Hide();
		return false;
	}
	else
	{
		return true;
	}
}


// ---------------------------------------------------------------------------
//	 IsDriverLoaded								[public]
// ---------------------------------------------------------------------------
//	

Boolean
CYarrowToolApp::IsDriverLoaded() const
{
	DriverRefNum	refNum;
	UnitNumber		unit;
	DriverFlags		flags;
	DriverOpenCount	count;
	
	return (FindYarrowDriver(refNum, unit, flags, count));
}


// ---------------------------------------------------------------------------
//	 IsDriverOpen								[public]
// ---------------------------------------------------------------------------
//	

Boolean
CYarrowToolApp::IsDriverOpen(DriverRefNum& outRefNum) const
{
	UnitNumber		unit;
	DriverFlags		flags;
	DriverOpenCount	count;
	
	return (FindYarrowDriver(outRefNum, unit, flags, count) && (count > 0));
}


// ---------------------------------------------------------------------------
//	 FindYarrowDriver								[protected]
// ---------------------------------------------------------------------------
//	

Boolean
CYarrowToolApp::FindYarrowDriver(
	DriverRefNum&		refNum, 
	UnitNumber&			unit, 
	DriverFlags&		flags, 
	DriverOpenCount&	count)
{
	return (YarrowFindNamedDriver("\p.Yarrow", refNum, unit, flags, count));
}


// ---------------------------------------------------------------------------
//	 IsDriverInstalled								[protected]
// ---------------------------------------------------------------------------
//	

UInt16
CYarrowToolApp::IsDriverInstalled()
{
	UInt16		result	= kYarrowCantInstall;
	FSSpec		spec;
	OSStatus	err;
	
	err = YarrowLocate(&spec);
	
	if (err == noErr)
	{
		result = YarrowIsInstalled(&spec);
	}
	
	return (result);
}


// ---------------------------------------------------------------------------
//	 InstallDriver								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::InstallDriver()
{
	FSSpec		spec;
	OSStatus	err;
	
	err = YarrowLocate(&spec);
	
	if (err == noErr)
	{
		err = YarrowInstall(&spec, NULL);
	}
	
	if (err != noErr)
	{
		ShowErrorBox(kAlertStopAlert, kCantLoadDriverErrorMsgIndex, 
					 kCantLoadDriverErrorReasonIndex, err);
	}
}


// ---------------------------------------------------------------------------
//	 UninstallDriver								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::UninstallDriver()
{
	FSSpec		spec;
	OSStatus	err;
	
	err = YarrowLocate(&spec);
	
	if (err == noErr)
	{
		err = YarrowUninstall(&spec);
	}
	
	if (err != noErr)
	{
		ShowErrorBox(kAlertStopAlert, kCantUnloadDriverErrorMsgIndex, 
					 kCantUnloadDriverErrorReasonIndex, err);
	}
}


// ---------------------------------------------------------------------------
//	 OpenYarrowDriver								[protected]
// ---------------------------------------------------------------------------
//	

/* Some extra dCtlFlags bits, as documented in:
 * <http://developer.apple.com/qa/dv/dv27.html>
 */
#define dNativeDriverMask	0x0008

void
CYarrowToolApp::OpenYarrowDriver()
{
	short	refNum;
	OSErr	err;
	
	err = OpenDriver(kYarrowDriverName, &refNum);
	
	if (err == noErr)
	{
		DCtlHandle	dCtl;
		
		dCtl = GetDCtlEntry(refNum);
		
		if (((*dCtl)->dCtlFlags & dNativeDriverMask) == 0)
		{
			CntrlParam		pb;
			
			pb.ioCompletion	= NULL;
			pb.ioVRefNum	= 0;
			pb.ioCRefNum	= refNum;
			pb.csCode		= kYarrowAddRef;
			
			err = PBControlSync((ParmBlkPtr) &pb);
		}
		
		if (err != noErr)
		{
			CloseDriver(refNum);
		}
	}
	
	if (err != noErr)
	{
		ShowErrorBox(kAlertStopAlert, kCantOpenDriverErrorMsgIndex, 
					 kCantOpenDriverErrorReasonIndex, err);
	}
}


// ---------------------------------------------------------------------------
//	 CloseYarrowDriver								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::CloseYarrowDriver()
{
	CntrlParam	pb;
	DCtlHandle	dCtl;
	OSErr		err;
	
	dCtl = GetDCtlEntry(mDriverRefNum);
	
	if (((*dCtl)->dCtlFlags & dNativeDriverMask) == 0)
	{
		pb.ioCompletion	= NULL;
		pb.ioVRefNum	= 0;
		pb.ioCRefNum	= mDriverRefNum;
		pb.csCode		= kYarrowRmvRef;
		
		err = PBControlSync((ParmBlkPtr) &pb);
	}
	else
	{
		pb.csParam[0]	= 0;
		err				= noErr;
	}
	
	if ((err == noErr) && (pb.csParam[0] == 0))
	{
		err = CloseDriver(mDriverRefNum);
	}
	
	if (err != noErr)
	{
		ShowErrorBox(kAlertStopAlert, kCantCloseDriverErrorMsgIndex, 
					 kCantCloseDriverErrorReasonIndex, err);
	}
}


// ---------------------------------------------------------------------------
//	 DriverStatus								[protected]
// ---------------------------------------------------------------------------
//	

OSErr
CYarrowToolApp::DriverStatus(
	Boolean&			seeded,
	int&				num_sources, 
	unsigned&			source_id, 
	size_t&				entropy_bits, 
	size_t&				entropy_max)
{
	CntrlParam			pb;
	YarrowCoreParams*	params	= (YarrowCoreParams *) pb.csParam;
	OSErr				err;
	
	params->version	= kYarrowCoreStatsVersionCurrent;
	pb.ioCompletion	= NULL;
	pb.ioVRefNum	= 0;
	pb.ioCRefNum	= mDriverRefNum;
	pb.csCode		= kYarrowCoreStats;
	
	err = PBStatusSync((ParmBlkPtr) &pb);
	
	if ((err == noErr) || (err == eofErr))
	{
		seeded			= (err == noErr);
		num_sources		= params->numSources;
		source_id		= params->closestSource;
		entropy_bits	= params->closestSourceBits;
		entropy_max		= params->closestSourceMax;
		err				= noErr;
	}
	
	return (err);
}


// ---------------------------------------------------------------------------
//	 GetDebugFlags								[protected]
// ---------------------------------------------------------------------------
//	

OSErr
CYarrowToolApp::GetDebugFlags(
	UInt16&			out_flags)
{
	CntrlParam	pb;
	OSErr		err;
	
	pb.ioCompletion	= NULL;
	pb.ioVRefNum	= 0;
	pb.ioCRefNum	= mDriverRefNum;
	pb.csCode		= kYarrowGetDbgFlags;
	
	err = PBStatusSync((ParmBlkPtr) &pb);
	
	if (err == noErr)
	{
		out_flags = pb.csParam[0];
	}

	return (err);
}


// ---------------------------------------------------------------------------
//	 SetDebugFlags								[protected]
// ---------------------------------------------------------------------------
//	

OSErr
CYarrowToolApp::SetDebugFlags(
	UInt16				in_flags)
{
	CntrlParam	pb;
	OSErr		err;
	
	pb.ioCompletion	= NULL;
	pb.ioVRefNum	= 0;
	pb.ioCRefNum	= mDriverRefNum;
	pb.csCode		= kYarrowSetDbgFlags;
	pb.csParam[0]	= in_flags;
	
	err = PBControlSync((ParmBlkPtr) &pb);

	return (err);
}


// ---------------------------------------------------------------------------
//	 YarrowResultToStr								[protected]
// ---------------------------------------------------------------------------
//	

const char*
CYarrowToolApp::YarrowResultToStr(int result)
{
	const char*	str;
	
	switch (result)
	{
	case YARROW_OK:					str = "All is well";							break;
	case YARROW_NOT_INIT:			str = "Yarrow library not initialised";			break;
	case YARROW_ALREADY_INIT:		str = "Yarrow library already initialised";		break;
	case YARROW_NO_DRIVER:			str = "Yarrow driver doesn't exist";			break;
	case YARROW_CANT_OPEN:			str = "Can't open Yarrow driver";				break;
	case YARROW_BAD_SOURCE:			str = "Invalid source ID";						break;
	case YARROW_TOO_MANY_SOURCES:	str = "Can't create any more source IDs";		break;
	case YARROW_BAD_ARG:			str = "Invalid argument";						break;
	case YARROW_ACCESS:				str = "Insufficient privileges";				break;
	case YARROW_NOMEM:				str = "Out of memory";							break;
	case YARROW_NORSRC:				str = "Insufficient resources";					break;
	case YARROW_GENERIC_ERROR:		str = "Catch-all error";						break;
	case YARROW_NOT_SEEDED:			str = "Not enough entropy to generate output";	break;
	default:						str = "Unknown error";							break;
	}
	
	return (str);
}


// ---------------------------------------------------------------------------
//	 MacErrToYarrowResult								[protected]
// ---------------------------------------------------------------------------
//	

int
CYarrowToolApp::MacErrToYarrowResult(OSErr err)
{
	int	result;
	
	switch (err)
	{
	case noErr:			result = YARROW_OK;					break;
	case fnfErr:		result = YARROW_NO_DRIVER;			break;
	case paramErr:		result = YARROW_BAD_ARG;			break;
	case openErr:		result = YARROW_CANT_OPEN;			break;
	case rfNumErr:		result = YARROW_BAD_SOURCE;			break;
	case tmfoErr:		result = YARROW_TOO_MANY_SOURCES;	break;
	case memFullErr:	result = YARROW_NOMEM;				break;
	case resNotFound:	result = YARROW_NORSRC;				break;
	case eofErr:		result = YARROW_NOT_SEEDED;			break;
	default:			result = YARROW_GENERIC_ERROR;		break;
	}
	
	return (result);
}


// ---------------------------------------------------------------------------
//	 EnablePaneByID								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::EnablePaneByID(
	LView*				inView,
	PaneIDT				inPaneID,
	Boolean				inEnable)
{
	LPane*	thePane	= inView->FindPaneByID(inPaneID);
	
	if (inEnable)
	{
		thePane->Enable();
	}
	else
	{
		thePane->Disable();
	}
}


// ---------------------------------------------------------------------------
//	 ShowErrorBox								[protected]
// ---------------------------------------------------------------------------
//	

void
CYarrowToolApp::ShowErrorBox(
	AlertType		alertType,
	int				textID, 
	int				reasonID, 
	...)
{
	va_list	args;
	Str255	errorFormat, reasonText;
	char	errorText[sizeof(Str255)];
	
	GetIndString(errorFormat, kErrorStrsID, textID);
	GetIndString(reasonText, kErrorStrsID, reasonID);
	
	va_start(args, reasonID);
	vsnprintf(errorText, sizeof(errorText), p2cstr(errorFormat), args);
	va_end(args);
	
	StDialogHandler	dialog(PPob_AlertWindow, this);
	LWindow*		window	= dialog.GetDialog();
	LIconPane*		icon	= dynamic_cast<LIconPane*>(window->FindPaneByID(kAlertIconPaneID));
	MessageT		msg;
	ResIDT			iconID;
	
	switch (alertType)
	{
	case kAlertStopAlert:		iconID = 1000;	break;
	case kAlertNoteAlert:		iconID = 1001;	break;
	case kAlertCautionAlert:	iconID = 1002;	break;
	default:					iconID = 0;		break;
	}
	
	window->SetDescriptorForPaneID(kAlertErrorTextPaneID, c2pstr(errorText));
	window->SetDescriptorForPaneID(kAlertReasonTextPaneID, reasonText);
	icon->SetIconID(iconID);
	window->Show();
	
	do {
		msg = dialog.DoDialog();
	} while (msg != kAlertOKButtonPaneID);
}
