// ============================================================================
//	CYarrowToolApp.h	2000 Zero-Knowledge Systems, Inc. All rights reserved.
// ============================================================================

#ifndef CYarrowToolApp_H_
#define CYarrowToolApp_H_
#pragma once

#include <stddef.h>
#include <Devices.h>
#include <LApplication.h>
#include <LListener.h>
#include <LPeriodical.h>


class CYarrowToolApp : public LApplication, public LListener, public LPeriodical {

public:
	
					CYarrowToolApp();
	virtual			~CYarrowToolApp();
					
	virtual Boolean	ObeyCommand(
						CommandT			inCommand,
						void*				ioParam);
	virtual void	FindCommandStatus(
						CommandT			inCommand,
						Boolean&			outEnabled,
						Boolean&			outUsesMark,
						UInt16&				outMark,
						Str255				outName);
	virtual void	ListenToMessage(
						MessageT			inMessage,
						void*				ioParam);
	virtual	void	SpendTime(
						const EventRecord&	inMacEvent);
	virtual Boolean	AllowSubRemoval(
						LCommander*			inSub);
	
	Boolean			IsDriverLoaded() const;
	Boolean			IsDriverOpen(
						DriverRefNum&		outRefNum) const;
	
protected:
	virtual void	StartUp();
	virtual void	DoReopenApp();
	void			RegisterClasses();
	void			SetPaneText(
						PaneIDT				paneID, 
						short				textIndex);
	void			SetTextState(
						PaneIDT				paneID, 
						short				textIndex, 
						Boolean				state);
	void			SetTextBtnState(
						PaneIDT				paneID, 
						short				textIndex, 
						Boolean				state, 
						Boolean				enableBtn);
	void			FillInFields();
	Boolean			RefreshDriverState();
	UInt16			IsDriverInstalled();
	void			InstallDriver();
	void			UninstallDriver();
	void			OpenYarrowDriver();
	void			CloseYarrowDriver();
	OSErr			DriverStatus(
						Boolean&			seeded,
						int&				num_sources, 
						unsigned&			source_id, 
						size_t&				entropy_bits, 
						size_t&				entropy_max);
	OSErr			GetDebugFlags(
						UInt16&				out_flags);
	OSErr			SetDebugFlags(
						UInt16				in_flags);
	const char*		YarrowResultToStr(int result);
	int				MacErrToYarrowResult(OSErr err);
	static void		EnablePaneByID(
						LView*				inView,
						PaneIDT				inPaneID,
						Boolean				inEnable);
	
	static Boolean	FindYarrowDriver(
						DriverRefNum&		refNum, 
						UnitNumber&			unit, 
						DriverFlags&		flags, 
						DriverOpenCount&	count);
						
	void			ShowErrorBox(
						AlertType			alertType,
						int					textID, 
						int					reasonID, 
						...);
							
private:
	
	// member variables
	LWindow*		mWindow;
	UInt16			mInstallState;
	Boolean			mDriverIsOpen;
	DriverRefNum	mDriverRefNum;
	UInt32			mLastTicks;
	UInt16			mDebugFlags;
};

#endif // CYarrowToolApp_H_
