// =============================================================================
//	DriverLoader.h							 (c)2000 Zero-Knowledge Systems Inc.
// =============================================================================
//	
//	The functions contained herein are similar in spirit to those in 
//	DriverLoaderLib, but work with "fat" drivers, i.e., drivers that are 
//	packaged in both 68K ('DRVR') and PowerPC ('ndrv') flavours.
//	
//	If running under PowerPC *and* DriverLoaderLib is available, then the 
//	functions attempt to call DriverLoaderLib to perform their work on a 
//	PowerPC driver, if any.  If that fails, or if running under 68K or if 
//	DriverLoaderLib isn't available, then functions in TradDriverLoaderLib 
//	are called to perform their work on a 68K driver.
//
//	Here is a matrix describing the behaviour of the library, as a function 
//	of the target CPU, native CPU, and presence of DriverLoaderLib.
//
//		Target CPU	Native CPU	D.L.L.	Behaviour
//		----------	----------	------	----------
//		68K			68K			n/a		Load 'DRVR'
//		68K			PowerPC		Yes		Load 'DRVR'
//		68K			PowerPC		No		Load 'DRVR'
//		PowerPC		PowerPC		Yes		Load 'ndrv'
//		PowerPC		PowerPC		No		Load 'DRVR'
//	
//	Note that 68K code running on a native PowerPC CPU will always load the 
//	'DRVR', regardless of the presence of DriverLoaderLib.  For this reason, 
//	code calling this library should generally be "fat", so that the presence 
//	DriverLoaderLib can be detected when running on a native PowerPC CPU.
//	
//	In order to work properly in cases where DriverLoaderLib isn't available 
//	on PowerPC (i.e., pre-PCI Power Macintosh models), you need to weak-link 
//	to DriverLoaderLib when building an executable containing this library.

#ifndef DriverLoader_H_
#define DriverLoader_H_

#if defined(__MWERKS__)
#	pragma once
#endif

// system headers
#include <Devices.h>

#pragma options align=power

#ifdef __cplusplus
	extern "C" {
#endif


Boolean		YarrowFindNamedDriver(
				ConstStringPtr		inName,
				DriverRefNum&		outRefNum,
				UnitNumber&			outUnit, 
				DriverFlags&		outFlags, 
				DriverOpenCount&	outOpenCount);
UnitNumber	YarrowHighestUnitNumber(void);
OSStatus	YarrowLookupDrivers(
				UnitNumber			inBeginningUnit, 
				UnitNumber			inEndingUnit,
				Boolean				inEmptyUnits, 
				ItemCount&			outRefNumCount, 
				DriverRefNum*		outRefNums);
OSStatus	YarrowGetDriverInformation(
				DriverRefNum		inRefNum, 
				UnitNumber&			outUnit, 
				DriverFlags&		outFlags, 
				DriverOpenCount&	outOpenCount, 
				StringPtr			outName);
OSStatus	YarrowInstallDriver(
				const FSSpec&		inDriverSpec, 
				SInt16				inDriverResFile, 
				SInt16				inDriverRsrcID, 
				DriverRefNum&		outRefNum);
OSStatus	YarrowRemoveDriver(
				DriverRefNum		inRefNum, 
				DriverFlags			inFlags);


#ifdef __cplusplus
	}
#endif

#pragma options align=reset

#endif	// DriverLoader_H_
