// =============================================================================
//	EntropySourceClient.h					 (c)2000 Zero-Knowledge Systems Inc.
// =============================================================================
//
//	Contains functions that are used to communicate between an entropy source 
//	and its various clients.  At present, those clients include the Yarrow 
//	driver itself, and the driver installation functions.

#ifndef EntropySourceClient_H_
#define EntropySourceClient_H_

#if defined(__MWERKS__)
#	pragma once
#endif

// system headers
#include <CodeFragments.h>
#include <OSUtils.h>

// project headers
#include "EntropySourceIntf.h"

#pragma options align=power

#ifdef __cplusplus
	extern "C" {
#endif


// Instances of this struct are chained together in a linked list whose head 
// is embedded within  at the YarrowGestalt structure.  There is one struct 
// per entropy source.

typedef struct EntropySourceQElem {
	QElemPtr				link;		// The next entropy source.
    YarrowEntropySourceUPP	upp;		// The entropy source's entry point.
    UInt16					source;		// The entropy source's source id.
    Boolean					cfrag;		// Code fragment or code resource ?
    union {
    	CFragConnectionID	id;			// Conn ID for CFM-based code.
    	Handle				h;			// Handle for resource-based code.
    } u;
} EntropySourceQElem;


// High-level APIs
OSStatus	ESC_GetGestalt(
				struct YarrowGestalt**	outGestalt);
void		ESC_GetPrefsName(
				struct YarrowGestalt*	inGestalt, 
				StringPtr				outPrefsName, 
				UInt32					inMaxPrefsSize);
UInt16		ESC_CountSources(
				struct YarrowGestalt*	inGestalt);
EntropySourceQElem*
			ESC_GetFirstSource(
				struct YarrowGestalt*	inGestalt);
EntropySourceQElem*
			ESC_GetNextSource(
				struct YarrowGestalt*	inGestalt, 
				EntropySourceQElem*		inElem);
UInt16		ESC_GetSourceID(
				struct YarrowGestalt*	inGestalt, 
				EntropySourceQElem*		inElem);
OSStatus	ESC_Add(
				struct YarrowGestalt*	inGestalt,
				YarrowEntropySourceUPP	inUPP, 
				CFragConnectionID		inConnID, 
				Handle					inHandle, 
				SInt16					inResFile);
OSStatus	ESC_Remove(
				struct YarrowGestalt*	inGestalt,
				EntropySourceQElem*		inElem);
void		ESC_RemoveAll(void);
void		ESC_CloseAll(void);
Boolean		ESC_CanSleepAll(void);
void		ESC_SleepAll(void);
void		ESC_WakeAll(void);
void		ESC_UnsleepAll(void);
void		ESC_FlushAll(void);
void		ESC_SetDebugFlagsAll(
				UInt16					inFlags);

// Low-level APIs
OSStatus	ESC_Init(
				YarrowEntropySourceUPP	inUPP);
void		ESC_Term(
				YarrowEntropySourceUPP	inUPP);
OSStatus	ESC_Preflight(
				YarrowEntropySourceUPP	inUPP);
OSStatus	ESC_Install(
				EntropySourceQElem*		inElem, 
				SInt16					inResFile);
OSStatus	ESC_Uninstall(
				EntropySourceQElem*		inElem);
OSStatus	ESC_Open(
				EntropySourceQElem*		inElem, 
				DriverRefNum			inRefNum,
				UInt32					inSource);
OSStatus	ESC_Close(
				EntropySourceQElem*		inElem);
Boolean		ESC_CanSleep(
				EntropySourceQElem*		inElem);
void		ESC_Sleep(
				EntropySourceQElem*		inElem);
void		ESC_Wake(
				EntropySourceQElem*		inElem);
void		ESC_Unsleep(
				EntropySourceQElem*		inElem);
OSStatus	ESC_GetStats(
				EntropySourceQElem*		inElem, 
				struct YarrowSrcStats*	outStats);
void		ESC_Flush(
				EntropySourceQElem*		inElem);
void		ESC_SetDebugFlags(
				EntropySourceQElem*		inElem, 
				UInt16					inFlags);


#ifdef __cplusplus
	}
#endif

#pragma options align=reset

#endif	/* EntropySourceClient_H_ */
