/* =============================================================================
 *	EntropySourceIntf.h						 (c)2000 Zero-Knowledge Systems Inc.
 * =============================================================================
 *	
 *	Contains APIs that are used to communicate between an entropy source 
 *	and its various clients.  At present, those clients include the Yarrow 
 *	driver itself, and the driver installation functions.
 */

#ifndef EntropySourceIntf_H_
#define EntropySourceIntf_H_

#if defined(__MWERKS__)
#	pragma once
#endif

/* system headers */
#include <Devices.h>

#pragma options align=power

#ifdef __cplusplus
	extern "C" {
#endif


typedef union YarrowEntropySourceArgs	YarrowEntropySourceArgs;

/* Entropy Source entry point. */

#define kCFragEntropySourceEntryPt	"\pYarrowEntropySourceEntryPointUPP"

typedef CALLBACK_API(OSStatus, YarrowEntropySourceProcPtr)(SInt32 selector, YarrowEntropySourceArgs* args);
typedef STACK_UPP_TYPE(YarrowEntropySourceProcPtr) YarrowEntropySourceUPP;

enum {
	uppYarrowEntropySourceProcInfo = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(OSStatus)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(SInt32)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(YarrowEntropySourceArgs *)))
};

#define NewYarrowEntropySourceUPP(proc)		\
			(YarrowEntropySourceUPP) NewRoutineDescriptor((ProcPtr)(proc), uppYarrowEntropySourceProcInfo, GetCurrentArchitecture())
#define DisposeYarrowEntropySourceUPP(upp)	\
			DisposeRoutineDescriptor(upp)
#define InvokeYarrowEntropySourceUPP(selector, args, upp)	\
			CALL_TWO_PARAMETER_UPP((upp), uppYarrowEntropySourceProcInfo, (SInt32) (selector), (args))


/* Entropy Source requests. */

enum {
	kYarrowEntropySourceInit = 1,
	kYarrowEntropySourceTerm,
	kYarrowEntropySourcePreflight,
	kYarrowEntropySourceInstall,
	kYarrowEntropySourceUninstall,
	kYarrowEntropySourceOpen,
	kYarrowEntropySourceClose,
	kYarrowEntropySourceCanSleep,
	kYarrowEntropySourceSleep,
	kYarrowEntropySourceWake,
	kYarrowEntropySourceUnsleep,
	kYarrowEntropySourceGetStats,
	kYarrowEntropySourceFlush,
	kYarrowEntropySourceSetDbgFlags
};

/* Entropy source arguments. */

union YarrowEntropySourceArgs {
	/* kYarrowEntropySourceInit			*/
	/*	- no args						*/
	/* kYarrowEntropySourceTerm			*/
	/*	- no args						*/
	/* kYarrowEntropySourcePreflight	*/
	/*	- no args						*/
	/* kYarrowEntropySourceInstall		*/
	struct {
		SInt16					resfile;		/* => */
	} install;
	/* kYarrowEntropySourceUninstall	*/
	/*	- no args						*/
	/* kYarrowEntropySourceOpen			*/
	struct {
		DriverRefNum			refnum;			/* => */
		UInt16					source;			/* => */
	} open;
	/* kYarrowEntropySourceClose		*/
	/*	- no args						*/
	/* kYarrowEntropySourceCanSleep		*/
	struct {
		Boolean					cansleep;		/* <= */
	} cansleep;
	/* kYarrowEntropySourceSleep		*/
	/*	- no args						*/
	/* kYarrowEntropySourceWake			*/
	/*	- no args						*/
	/* kYarrowEntropySourceUnsleep		*/
	/*	- no args						*/
	/* kYarrowEntropySourceGetStats		*/
	struct {
		struct YarrowSrcStats*	stats;			/* => */
	} stats;
	/* kYarrowEntropySourceFlush		*/
	/*	- no args						*/
	/* kYarrowEntropySourceSetDbgFlags	*/
	struct {
		UInt16					flags;			/* => */
	} setdbgflags;
};


#ifdef __cplusplus
	}
#endif

#pragma options align=reset

#endif	/* EntropySourceIntf_H_ */
