/* ===========================================================================
 * FragUtils.h				 (c) 1999 Paul Lalonde enrg.  All rights reserved.
 * ===========================================================================
 *	
 *	Redistribution and use in source and binary forms, with or without 
 *	modification, are permitted provided that redistributions of source code 
 *	retain the above copyright notice and the following disclaimer.
 *	
 *	THIS SOFTWARE IS PROVIDED BY PAUL LALONDE ENRG. ``AS IS'' AND ANY EXPRESS 
 *	OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 *	DISCLAIMED.  IN NO EVENT SHALL PAUL LALONDE ENRG. BE LIABLE FOR ANY 
 *	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 *	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 *	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 *	ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _H_FragUtils
#define _H_FragUtils

/* system headers */
#ifndef __ALIASES__
#	include <Aliases.h>
#endif
#ifndef __CODEFRAGMENTS__
#	include <CodeFragments.h>
#endif

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif


#define kPowerPCArchitecture	(((ISAType) kPowerPCISA) | ((RTAType) kPowerPCRTA))
#define kCFM68KArchitecture		(((ISAType) kM68kISA)    | ((RTAType) kCFM68kRTA))
#define kClassic68KArchitecture	(((ISAType) kM68kISA)    | ((RTAType) kOld68kRTA))


typedef CFragResourceMember	CFragRsrcMbr;

typedef void	(*CfrgIterProcPtr)(
	UInt16				inIndex, 
	const CFragRsrcMbr*	inEntry, 
	Boolean*			outQuitFlag, 
	void*				ioArg);

typedef void	(*FragIterProcPtr)(
	const FSSpec*		inSpec, 
	const FInfo*		inInfo, 
	Boolean				inIsAppl, 
	Boolean*			outQuitFlag, 
	void*				ioArg);


/* --- Iterating over the import library search path --- */

OSErr	YarrowIterateFragPath(
			const FSSpec*		inRootFragSpec, 
			Boolean				inExplicitLoad, 
			FragIterProcPtr		inProc, 
			void*				ioArg);

OSErr	YarrowFindFragment(
			const FSSpec*		inRootFragSpec, 
			Boolean				inExplicitLoad, 
			FragIterProcPtr		inFileProc, 
			void*				ioFileArg, 
			CfrgIterProcPtr		inCfrgProc, 
			void*				ioCfrgArg, 
			FSSpec*				outSpec, 
			UInt16*				outIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outName);

OSErr	YarrowFindFragmentSimple(
			const FSSpec*		inRootFragSpec, 
			Boolean				inExplicitLoad, 
			ConstStr63Param		inFragName, 
			CFragArchitecture	inArchitecture, 
			FSSpec*				outSpec, 
			UInt16*				outIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outName);


/* Returns the 'cfrg' resource residing in 'inResFile'.  If 
   'inResFile' is kResFileNotOpened (-1), the resource is 
   looked for in the file pointed to by 'inSpec'. */
OSErr	YarrowGetCfrgResource(
			SInt16				inResFile, 
			const FSSpec*		inSpec, 
			Handle*				outCfrgH);

OSErr	YarrowIterateCfrgEntries(
			Handle				inCfrgH, 
			CfrgIterProcPtr		inProc, 
			void*				ioArg);

OSErr	YarrowFindCfrgEntry(
			Handle				inCfrgH, 
			CfrgIterProcPtr		inProc, 
			void*				ioArg, 
			UInt16*				outIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outName);

/* Return the number of entries in a 'cfrg' resource */
OSErr	YarrowCountCfrgEntries(Handle inCfrg, UInt16* outCount);

/* Return information about the inIndex'th entry in a 'cfrg' resource.  
   Any of the output parameters may be NULL. */
OSErr	YarrowGetNthCfrgEntry(
			Handle				inCfrg, 
			UInt16				inIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outFragName);

/* Return information about the entry having a given name and architecture 
   in a 'cfrg' resource.  Any of the output parameters may be NULL. */
OSErr	YarrowFindNamedCfrgEntry(
			Handle				inCfrg, 
			ConstStr63Param		inFragName, 
			CFragArchitecture	inArchitecture, 
			UInt16*				outIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength);

/* Return information about the entry having a given usage and architecture 
   in a 'cfrg' resource.  Any of the output parameters may be NULL. */
OSErr	YarrowFindCfrgEntryByUsage(
			Handle				inCfrg, 
			CFragUsage			inUsage, 
			CFragArchitecture	inArchitecture, 
			UInt16*				outIndex, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outFragName);

/* Constructs an alias to a code fragment.  Takes the same arguments as 
   MakeAlias, plus the fragment name and architecture.  Note:  the 
   fragment must actually exist inside the target file. */
OSErr	YarrowMakeCfrgAlias(
			const FSSpec*		inFromFile, 
			const FSSpec*		inTargetFile, 
			ConstStr63Param		inFragName, 
			CFragArchitecture	inArchitecture, 
			AliasHandle*		outAlias);

/* Resolves an alias to a code fragment.  Conceptually, it's similar to 
   a call to ResolveAlias followed by a call to GetNamedShlbEntry.  Any 
   of the output parameters may be NULL. */
OSErr	YarrowResolveCfrgAlias(
			const FSSpec*		inFromFile, 
			AliasHandle			inAlias, 
			Boolean				inMountRemoteVols, 
			Boolean				inResolveAliasChains, 
			CFragArchitecture	inArchitecture, 
			FSSpec*				outTargetFile, 
			Boolean*			outChanged, 
			CFragArchitecture*	outArchitecture, 
			UInt8*				outUpdateLevel, 
			CFragVersionNumber*	outCurrentVersion, 
			CFragVersionNumber*	outOldDefVersion, 
			CFragUsage1Union*	outUsage1, 
			CFragUsage2Union*	outUsage2, 
			CFragUsage*			outUsage, 
			CFragLocatorKind*	outLocator, 
			UInt32*				outOffset, 
			UInt32*				outLength, 
			Str63				outFragName);

/* Are the given fragment names the same ? */
Boolean	YarrowEqualFragName(
			ConstStr63Param		inFragName1, 
			ConstStr63Param		inFragName2);

/* Loads a code fragment residing in a resource.  The resource is of 
   type 'rsrcType' and ID 'rsrcID', and resides in 'resFile'.  The 
   fragment will be loaded into the 'inZone' heap zone.  If 'searchChain' 
   is true, the function will call GetResource to retrieve the code 
   resource, else Get1Resource will be used.  The resource is loaded as 
   high or as low as possible in 'inZone', depending on 'loadHi'. If 
   'detach' is true, the resource will be detached from its resource 
   map.
   
   The function returns the resource handle in 'rsrcHPtr', the CFM 
   connection ID in 'connID', the main entry point in 'mainAddr', and 
   any CFM error message in 'errMsg'. */
OSErr	YarrowLoadRsrcFragment(
			SInt16				resFile, 
			THz					inZone, 
			ResType				rsrcType, 
			SInt16				rsrcID, 
			Boolean				searchChain, 
			Boolean				loadHi, 
			Boolean				detach, 
			Handle*				rsrcHPtr, 
			CFragConnectionID*	connID, 
			Ptr*				mainAddr, 
			Str255				errMsg);

/* Unloads a code fragment previously loaded with LoadRsrcFragment.  
   'inZone' is the heap zone in which the resource was loaded, 'rsrcH' 
   is the resource handle, and 'connID' is the CFM connection ID. */
OSErr	YarrowUnloadRsrcFragment(
			THz					inZone, 
			Handle				rsrcH, 
			CFragConnectionID	connID);

/* Returns the native (i.e., CFM) instruction set architecture of the 
   computer we're running on.  This is the value of interest to the 
   Mixed Mode Manager. */
ISAType	YarrowGetNativeISA(void);

/* Returns the native code fragment architecture of the computer we're 
   running on.  This is the value of interest to the Code Fragment Manager. */
CFragArchitecture	YarrowGetNativeArchitecture(void);

/* Is the Mixed Mode Manager available ? */
Boolean	YarrowHasMixedModeMgr(void);

/* Is the Code Fragment Manager available ? */
Boolean	YarrowHasCodeFragmentMgr(void);


#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif

#endif	/* _H_FragUtils*/
