// ===========================================================================
// RsrcUtils.cp				 (c) 1999 Paul Lalonde enrg.  All rights reserved.
// ===========================================================================
//	
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that redistributions of source code 
//	retain the above copyright notice and the following disclaimer.
//	
//	THIS SOFTWARE IS PROVIDED BY PAUL LALONDE ENRG. ``AS IS'' AND ANY EXPRESS 
//	OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
//	DISCLAIMED.  IN NO EVENT SHALL PAUL LALONDE ENRG. BE LIABLE FOR ANY 
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
//	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
//	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//	ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.

// standard headers
#include <string.h>

// system headers
#include <Errors.h>
#include <LowMem.h>

// project headers
#include "RsrcUtils.h"


// prototypes of local functions.
static OSErr	CopyResource(
	SInt16				srcResFile, 
	Handle				srcHand, 
	SInt16				dstResFile, 
	ResType				dstRsrcType, 
	SInt16				dstRsrcID);
static OSErr	ReplaceResource(
	SInt16				srcResFile, 
	Handle				srcHand, 
	SInt16				dstResFile, 
	Handle				dstHand);


// ---------------------------------------------------------------------------
//	 YarrowLoadCodeResource
// ---------------------------------------------------------------------------
//	

OSErr	YarrowLoadCodeResource(
	SInt16	resFile, 
	THz		inZone, 
	ResType	rsrcType, 
	SInt16	rsrcID, 
	Boolean	searchChain, 
	Boolean	loadHi, 
	Boolean	detach, 
	Handle*	rsrcHPtr)
{
	THz		savedZone		= GetZone();
	SInt16	savedResFile	= CurResFile();
	Boolean	savedResLoad	= LMGetResLoad();
	OSErr	err;
	Handle	rsrcH;
	
	*rsrcHPtr = NULL;
	
	if (inZone != NULL)
		SetZone(inZone);
	
	if (resFile != kResFileNotOpened)
		UseResFile(resFile);
	
	SetResLoad(false);
	if (searchChain)
		rsrcH = GetResource(rsrcType, rsrcID);
	else
		rsrcH = Get1Resource(rsrcType, rsrcID);
	err = ResError();
	SetResLoad(true);
	
	if (rsrcH != NULL)
	{
		if (!loadHi)
		{
			long	rsrcSize;
			
			rsrcSize = GetMaxResourceSize(rsrcH);
			
			ReserveMem(rsrcSize);
		}
		
		LoadResource(rsrcH);
		err = ResError();
		
		if (err == noErr)
		{
			HNoPurge(rsrcH);
			err = MemError();
		}
		
		if ((err == noErr) && loadHi)
		{
			MoveHHi(rsrcH);
			err = MemError();
		}
		
		if (err == noErr)
		{
			HLock(rsrcH);
			err = MemError();
		}
		
		if ((err == noErr) && detach)
		{
			DetachResource(rsrcH);
			err = ResError();
		}
		
		if (err == noErr)
		{
			*rsrcHPtr = rsrcH;
		}
		else
		{
			ReleaseResource(rsrcH);
		}
	}
	else if (err == noErr)
	{
		err = resNotFound;
	}
	
	SetResLoad(savedResLoad);
	UseResFile(savedResFile);
	SetZone(savedZone);
	
	return (err);
}


// ---------------------------------------------------------------------------
//	 YarrowLoadIndCodeResource
// ---------------------------------------------------------------------------
//	

OSErr	YarrowLoadIndCodeResource(
	SInt16	resFile, 
	THz		inZone, 
	ResType	rsrcType, 
	SInt16	rsrcIndex, 
	Boolean	searchChain, 
	Boolean	loadHi, 
	Boolean	detach, 
	Handle*	rsrcHPtr)
{
	THz		savedZone		= GetZone();
	SInt16	savedResFile	= CurResFile();
	Boolean	savedResLoad	= LMGetResLoad();
	OSErr	err;
	Handle	rsrcH;
	
	*rsrcHPtr = NULL;
	
	if (inZone != NULL)
		SetZone(inZone);
	
	if (resFile != kResFileNotOpened)
		UseResFile(resFile);
	
	SetResLoad(false);
	if (searchChain)
		rsrcH = GetIndResource(rsrcType, rsrcIndex);
	else
		rsrcH = Get1IndResource(rsrcType, rsrcIndex);
	err = ResError();
	SetResLoad(true);
	
	if (rsrcH != NULL)
	{
		if (!loadHi)
		{
			long	rsrcSize;
			
			rsrcSize = GetMaxResourceSize(rsrcH);
			
			ReserveMem(rsrcSize);
		}
		
		LoadResource(rsrcH);
		err = ResError();
		
		if (err == noErr)
		{
			HNoPurge(rsrcH);
			err = MemError();
		}
		
		if ((err == noErr) && loadHi)
		{
			MoveHHi(rsrcH);
			err = MemError();
		}
		
		if (err == noErr)
		{
			HLock(rsrcH);
			err = MemError();
		}
		
		if ((err == noErr) && detach)
		{
			DetachResource(rsrcH);
			err = ResError();
		}
		
		if (err == noErr)
		{
			*rsrcHPtr = rsrcH;
		}
		else
		{
			ReleaseResource(rsrcH);
		}
	}
	else if (err == noErr)
	{
		err = resNotFound;
	}
	
	SetResLoad(savedResLoad);
	UseResFile(savedResFile);
	SetZone(savedZone);
	
	return (err);
}


// ---------------------------------------------------------------------------
//	 YarrowUnloadCodeResource
// ---------------------------------------------------------------------------
//	

OSErr	YarrowUnloadCodeResource(
	THz		inZone, 
	Handle	rsrcH)
{
	THz		savedZone	= GetZone();
	SInt8	hState;
	OSErr	err;
	
	SetZone(inZone);
	
	hState = HGetState(rsrcH);
	
	if (hState & 0x20)
	{
		ReleaseResource(rsrcH);
		err = ResError();
	}
	else
	{
		DisposeHandle(rsrcH);
		err = MemError();
	}
	
	SetZone(savedZone);
	
	return (err);
}


// ---------------------------------------------------------------------------
//	 YarrowGetResourceName
// ---------------------------------------------------------------------------
//	

OSErr	YarrowGetResourceName(
	SInt16		resFile, 
	ResType		rsrcType, 
	SInt16		rsrcID, 
	Boolean		searchChain, 
	StringPtr	rsrcName)
{
	SInt16	savedResFile	= CurResFile();
	Boolean	savedResLoad	= LMGetResLoad();
	OSErr	err;
	Handle	rsrcH;
	
	if (resFile != kResFileNotOpened)
		UseResFile(resFile);
	
	SetResLoad(false);
	if (searchChain)
		rsrcH = GetResource(rsrcType, rsrcID);
	else
		rsrcH = Get1Resource(rsrcType, rsrcID);
	err = ResError();
	SetResLoad(true);
	
	if (rsrcH != NULL)
	{
		SInt16	junkID;
		ResType	junkType;
		
		GetResInfo(rsrcH, &junkID, &junkType, rsrcName);
		err = ResError();
	}
	else if (err == noErr)
	{
		err = resNotFound;
	}
	
	SetResLoad(savedResLoad);
	UseResFile(savedResFile);
	
	return (err);
}


OSErr	YarrowGetResourceFromFile(
	const FSSpec*		inFile, 
	ResType				rsrcType, 
	SInt16				rsrcID, 
	Handle*				rsrcHPtr)
{
	SInt16	savedResFile	= CurResFile();
	Boolean	savedResLoad	= LMGetResLoad();
	OSErr	err				= noErr;
	SInt16	resFile;
	
	SetResLoad(false);
	resFile = FSpOpenResFile(inFile, fsRdPerm);
	if (resFile == kResFileNotOpened)
		err = ResError();
	SetResLoad(true);
	
	if (err == noErr)
	{
		Handle	rsrcH;
		
		rsrcH = Get1Resource(rsrcType, rsrcID);
		
		if (rsrcH != NULL)
		{
			HNoPurge(rsrcH);
			DetachResource(rsrcH);
			
			*rsrcHPtr = rsrcH;
		}
		else
		{
			err = ResError();
			if (err == noErr)
				err = resNotFound;
		}
		
		CloseResFile(resFile);
	}
	
	SetResLoad(savedResLoad);
	UseResFile(savedResFile);
	
	return (err);
}


OSErr	YarrowCopyResource(
	SInt16				srcResFile, 
	ResType				srcRsrcType, 
	SInt16				srcRsrcID, 
	SInt16				dstResFile, 
	ResType				dstRsrcType, 
	SInt16				dstRsrcID, 
	Boolean				replace)
{
	SInt16	savedResFile	= CurResFile();
	Boolean	savedResLoad	= LMGetResLoad();
	OSErr	err				= noErr;
	Handle	srcHand, dstHand;
	
	SetResLoad(false);
	
	/* Retrieve the source resource. */
	
	UseResFile(srcResFile);
	srcHand = Get1Resource(srcRsrcType, srcRsrcID);
	if (srcHand == NULL)
	{
		err = ResError();
		if (err == noErr)
			err = resNotFound;
	}
	
	if (err == noErr)
	{
		/* Determine if the destination resource already exists. */
		
		UseResFile(dstResFile);
		dstHand = Get1Resource(dstRsrcType, dstRsrcID);
		
		if (dstHand != NULL)
		{
			/* The destination resource already exists:  replace it 
			if requested to do so. */
			
			if (replace)
			{
				err = ReplaceResource(srcResFile, srcHand, 
									  dstResFile, dstHand);
			}
		}
		else
		{
			/* The destination resource doesn't exist:  add it. */
			
			err = CopyResource(srcResFile, srcHand, 
							   dstResFile, dstRsrcType, dstRsrcID);
		}
	}
	
	SetResLoad(savedResLoad);
	UseResFile(savedResFile);
	
	return (err);
}


static OSErr	CopyResource(
	SInt16				srcResFile, 
	Handle				srcHand, 
	SInt16				dstResFile, 
	ResType				dstRsrcType, 
	SInt16				dstRsrcID)
{
	SInt16	savedResFile	= CurResFile();
	OSErr	err;
	
	UseResFile(srcResFile);
	LoadResource(srcHand);
	err = ResError();
	
	if (err == noErr)
	{
		SInt8	srcState;
		Handle	dstHand;
		
		/* Prevent the source resource from unloading out from 
		underneath us while we're trying to copy it. */
		
		srcState = HGetState(srcHand);
		HNoPurge(srcHand);
		
		dstHand = srcHand;
		err = HandToHand(&dstHand);
		
		if (err == noErr)
		{
			UseResFile(dstResFile);
			AddResource(dstHand, dstRsrcType, dstRsrcID, "\p");
			err = ResError();
			
			if (err != noErr)
			{
				DisposeHandle(dstHand);
				dstHand = NULL;
			}
		}
		
		HSetState(srcHand, srcState);
	}
	
	UseResFile(savedResFile);
	
	return (err);
}


static OSErr	ReplaceResource(
	SInt16				srcResFile, 
	Handle				srcHand, 
	SInt16				dstResFile, 
	Handle				dstHand)
{
	SInt16	savedResFile	= CurResFile();
	OSErr	err;
	
	UseResFile(srcResFile);
	LoadResource(srcHand);
	err = ResError();
	
	if (err == noErr)
	{
		SInt8	srcState;
		
		/* Prevent the source resource from unloading out from 
		underneath us while we're trying to copy it. */
		
		srcState = HGetState(srcHand);
		HNoPurge(srcHand);
		
		/* The caller wants us to replace the resource.  However, 
		we only do so if the resources are different.  Or else, 
		what's the point?. */
		
		UseResFile(dstResFile);
		LoadResource(dstHand);
		err = ResError();
		
		if (err == noErr)
		{
			HNoPurge(dstHand);
			
			Size	srcSize	= GetHandleSize(srcHand);
			Size	dstSize	= GetHandleSize(dstHand);
			
			if ((srcSize != dstSize) || 
				(memcmp(*srcHand, *dstHand, srcSize) != 0))
			{
				/* The resources are different. */
				
				HLock(srcHand);
				
				err = PtrToXHand(*srcHand, dstHand, srcSize);
				
				ChangedResource(dstHand);
				err = ResError();
			}
		}
		
		HSetState(srcHand, srcState);
	}
	
	UseResFile(savedResFile);
	
	return (err);
}
