/* ===========================================================================
 * RsrcUtils.h				 (c) 1999 Paul Lalonde enrg.  All rights reserved.
 * ===========================================================================
 *	
 *	Redistribution and use in source and binary forms, with or without 
 *	modification, are permitted provided that redistributions of source code 
 *	retain the above copyright notice and the following disclaimer.
 *	
 *	THIS SOFTWARE IS PROVIDED BY PAUL LALONDE ENRG. ``AS IS'' AND ANY EXPRESS 
 *	OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 *	DISCLAIMED.  IN NO EVENT SHALL PAUL LALONDE ENRG. BE LIABLE FOR ANY 
 *	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 *	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 *	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 *	ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _H_RsrcUtils
#define _H_RsrcUtils

#ifndef __RESOURCES__
#	include <Resources.h>
#endif

#if PRAGMA_ONCE
#	pragma once
#endif

#ifdef __cplusplus
	extern "C" {
#endif

#if PRAGMA_STRUCT_ALIGN
#	pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
#	pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
#	pragma pack(2)
#endif


/* Loads a code resource.  The resource is of type 'rsrcType' and ID 
   'rsrcID', and resides in 'resFile'.  The resource will be loaded into 
   the 'inZone' heap zone.  If 'searchChain' is true, the function will 
   call GetResource to retrieve the resource, else Get1Resource will be 
   used.  The resource is loaded as high or as low as possible in 'inZone', 
   depending on 'loadHi'.  If 'detach' is true, the resource will be 
   detached from its resource map.
   
   The function returns the resource handle in 'rsrcHPtr'. */
OSErr	YarrowLoadCodeResource(
			SInt16				resFile, 
			THz					inZone, 
			ResType				rsrcType, 
			SInt16				rsrcID, 
			Boolean				searchChain, 
			Boolean				loadHi, 
			Boolean				detach, 
			Handle*				rsrcHPtr);

/* Loads a code resource.  The resource is of type 'rsrcType' and index 
   'rsrcIndex', and resides in 'resFile'.  The resource will be loaded 
   into the 'inZone' heap zone.  If 'searchChain' is true, the function 
   will call GetResource to retrieve the resource, else Get1Resource will 
   be used.  The resource is loaded as high or as low as possible in 
   'inZone', depending on 'loadHi'.  If 'detach' is true, the resource 
   will be detached from its resource map.
   
   The function returns the resource handle in 'rsrcHPtr'. */
OSErr	YarrowLoadIndCodeResource(
			SInt16				resFile, 
			THz					inZone, 
			ResType				rsrcType, 
			SInt16				rsrcIndex, 
			Boolean				searchChain, 
			Boolean				loadHi, 
			Boolean				detach, 
			Handle*				rsrcHPtr);

/* Unloads a code resource previously loaded with LoadCodeResource.  
   'inZone' is the heap zone in which the resource was loaded, and 'rsrcH' 
   is the resource handle. */
OSErr	YarrowUnloadCodeResource(
			THz					inZone, 
			Handle				rsrcH);

/* Returns the name of a resource identified by 'resFile', 'rsrcType', 
   and 'rsrcID'. */
OSErr	YarrowGetResourceName(
			SInt16				resFile, 
			ResType				rsrcType, 
			SInt16				rsrcID, 
			Boolean				searchChain, 
			StringPtr			rsrcName);

/* Retrieves a resource from the given file.  The returned handle is 
   detached and made non-purgeable. */
OSErr	YarrowGetResourceFromFile(
			const FSSpec*		inFile, 
			ResType				rsrcType, 
			SInt16				rsrcID, 
			Handle*				rsrcHPtr);

OSErr	YarrowCopyResource(
			SInt16				srcResFile, 
			ResType				srcRsrcType, 
			SInt16				srcRsrcID, 
			SInt16				dstResFile, 
			ResType				dstRsrcType, 
			SInt16				dstRsrcID, 
			Boolean				replace);


#if PRAGMA_STRUCT_ALIGN
#	pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
#	pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
#	pragma pack()
#endif

#ifdef PRAGMA_IMPORT_OFF
#	pragma import off
#elif PRAGMA_IMPORT
#	pragma import reset
#endif

#ifdef __cplusplus
	}
#endif

#endif	/* _H_RsrcUtils */
