/* =============================================================================
 *	YarrowDriverCore.h						 (c)2000 Zero-Knowledge Systems Inc.
 * =============================================================================
 *	
 *	Core driver functions.  These functions are the main bridges between 
 *	the driver interface (whether 'DRVR' or 'ndrv' style) and the Yarrow 
 *	core.
 */

#ifndef YarrowDriverCore_H_
#define YarrowDriverCore_H_

#if defined(__MWERKS__)
#	pragma once
#endif

/* system headers */
#include <Devices.h>

#pragma options align=power

#ifdef __cplusplus
	extern "C" {
#endif


/* This first group of functions correspond to requests that are sent to 
 * the DoDriverIO entry point of a native ('ndrv') driver.
 */

OSStatus	HandleInitialise(
				const char*		logname);
OSStatus	HandleFinalise(void);
OSStatus	HandleOpen(
				short			driverRefNum,
				ParmBlkPtr		pb);
OSStatus	HandleClose(
				ParmBlkPtr		pb);
OSStatus	HandleRead(
				DCtlPtr			opaqueDCtl, 
				AddressSpaceID	space, 
				IOCommandID		id, 
				ParmBlkPtr		pb, 
				IOCommandKind	kind);
OSStatus	HandleWrite(
				DCtlPtr			opaqueDCtl, 
				AddressSpaceID	space, 
				IOCommandID		id, 
				ParmBlkPtr		pb, 
				IOCommandKind	kind);
OSStatus	HandleControl(
				DCtlPtr			opaqueDCtl, 
				AddressSpaceID	space, 
				IOCommandID		id, 
				ParmBlkPtr		pb, 
				IOCommandKind	kind);
OSStatus	HandleStatus(
				DCtlPtr			opaqueDCtl, 
				AddressSpaceID	space, 
				IOCommandID		id, 
				ParmBlkPtr		pb, 
				IOCommandKind	kind);

/* The following group of functions read and write the yarrow state to 
 * and from an internal cache.
 */

int			PerformStateRead(
				struct Yarrow_STATE*		state);
void		PerformStateWrite(
				const struct Yarrow_STATE*	state);
				

#ifdef __cplusplus
	}
#endif

#pragma options align=reset

#endif	/* YarrowDriverCore_H_ */
