/* =============================================================================
 *	YarrowDriverDefs.h						 (c)2000 Zero-Knowledge Systems Inc.
 * =============================================================================
 *	
 *	Describes how to call the Yarrow driver.  The information presented here 
 *	might be used to build a user-space access library for the driver.
 */

#ifndef YarrowDriverDefs_H_
#define YarrowDriverDefs_H_

/* system headers */
#include <MacTypes.h>

#pragma options align=mac68k

#ifdef __cplusplus
	extern "C" {
#endif


/* -----------------------------------------------------------------------------
 *	Open
 *	
 *	OpenDriver is called in the normal way.
 */


/* -----------------------------------------------------------------------------
 *	Close
 *	
 *	PBClose is called in the normal way.  However, care must be taken if the 
 *	68K driver is used, because 68K device drivers don't have their open calls 
 *	reference-counted.  The application may choose not to close the driver in 
 *	this case, in order to prevent the driver from disappearing from underneath 
 *	other applications.
 */


/* -----------------------------------------------------------------------------
 *	Read
 *	
 *	PBRead is called in the normal way, except that the ioPosMode field must be 
 *	set to 0 (fsAtMark).  The ioPosOffset field must be set to zero as well.
 *	
 *	Note that immediate read requests aren't supported.
 */


/* -----------------------------------------------------------------------------
 *	Write
 *	
 *	PBWrite is called in the normal way, except that the ioPosMode field is 
 *	interpreted as the source doing the writing, as returned from the 
 *	kYarrowNewSource Control call.  Also, the ioPosOffset field must contain 
 *	the number of bits of entropy in the buffer being written.
 *	
 *	Note that immediate write requests aren't supported.
 */


/* -----------------------------------------------------------------------------
 *	Control
 *	
 *	PBControl is called in the normal way.  The possible control codes, and the 
 *	values passed in the parameter block, are listed below.
 *	
 *	Note that immediate control requests aren't supported.
 */


/* -----------------------------------------------------------------------------
 *	Status
 *	
 *	PBStatus is called in the normal way.  The possible status codes, and the 
 *	values passed and returned from the parameter block, are listed below.
 *	
 *	Note that immediate status requests (apart from DriverGestalt) aren't 
 *	supported.
 */


/* -----------------------------------------------------------------------------
 *	KillIO
 *	
 *	PBKillIO is not supported.
 */


/* -----------------------------------------------------------------------------
 *	PBControl codes
 */

#define kYarrowReseed		258		/* => csParam[0]:  pool to reseed		*/
#define kYarrowGate			259		/* -									*/

#define kYarrowStretch		260		/* => csParam[0] to csParam[10]:		*/
									/*		YarrowStretchArgs				*/
typedef struct YarrowStretchArgs {
	const void*	srcaddr;
	Size		srcsize;
	void*		dstaddr;
	Size		dstsize;
	UInt16		reserved[3];
} YarrowStretchArgs;

/* these two only apply to the 68K driver */
#define kYarrowAddRef		261		/* -									*/
#define kYarrowRmvRef		262		/* <= csParam[0]:  close flag			*/

#define kYarrowFixedOutput	263		/* => csParam[0]:  copy byte to output?	*/
									/* => csParam[1]:  byte to copy			*/
#define kYarrowSetDbgFlags	264		/* => csParam[0]:  flags				*/

/* -----------------------------------------------------------------------------
 *	PBStatus codes
 */

#define kYarrowDriverStats	256		/* csParam[0] to csParam[10]:			*/
									/*		YarrowDriverParams				*/
typedef struct YarrowDriverParams {
	UInt16	version;				/* <=> version of this struct			*/
	UInt16	numBuiltInSources;		/* <= # of built-in sources				*/
	UInt16	userSource;				/* <= source id for client apps			*/
	UInt16	reserved[8];
} YarrowDriverParams;

#define kYarrowDriverStatsVersion1			1
#define kYarrowDriverStatsVersionCurrent	kYarrowDriverStatsVersion1


#define kYarrowSourceStats	257		/* csParam[0] to csParam[10]:			*/
									/*		YarrowSourceParams				*/

typedef struct YarrowSrcStats	YarrowSrcStats;

typedef struct YarrowSourceParams {
	UInt16			source;			/* => source (1..numBuiltInSources)		*/
	YarrowSrcStats*	stats;			/* => stats for given source			*/
	UInt16			reserved[8];
} YarrowSourceParams;

#define kYarrowSourceStatsVersion1			1
#define kYarrowSourceStatsVersionCurrent	kYarrowSourceStatsVersion1

struct YarrowSrcStats {
	UInt16	version;				/* version of this struct				*/
	SInt32	numDevices;				/* # of devices on this source			*/
	SInt32	packetsReceived;		/* packets received						*/
	SInt32	bytesReceived;			/* bytes received						*/
	SInt32	packetsOverflowed;		/* packets dropped b/o buffer overflow	*/
	SInt32	bytesOverflowed;		/* bytes dropped b/o buffer overflow	*/
	SInt32	packetsNoBuffers;		/* packets dropped b/o too few buffers	*/
	SInt32	bytesNoBuffers;			/* bytes dropped b/o too few buffers	*/
	SInt32	bytesWritten;			/* bytes written (to PBWrite)			*/
	SInt32	buffersWritten;			/* buffers written (to PBWrite)			*/
	UInt64	totalWriteTime;			/* time taken by PBWrite calls (secs)	*/
	UInt64	minWriteTime;			/* min time taken by PBWrite (secs)	*/
	UInt64	maxWriteTime;			/* max time taken by PBWrite (secs)	*/
	Str31	description;
};

#define kYarrowCoreStats	258		/* csParam[0] to csParam[10]:			*/
									/*		YarrowDriverParams				*/
typedef struct YarrowCoreParams {
	UInt16	version;				/* <=> version of this struct			*/
	SInt16	numSources;				/* <= # of non-seeded sources			*/
	SInt16	closestSource;			/* <= source "nearest" to being seeded	*/
	UInt32	closestSourceBits;		/* <= bits of entropy for closestSource	*/
	UInt32	closestSourceMax;		/* <= entropy thresh for closestSource	*/
	UInt16	reserved[4];
} YarrowCoreParams;

#define kYarrowCoreStatsVersion1		1
#define kYarrowCoreStatsVersionCurrent	kYarrowCoreStatsVersion1

#define kYarrowGetDbgFlags	259		/* <= csParam[0]:  flags				*/


#ifdef __cplusplus
	}
#endif

#pragma options align=reset

#endif	/* YarrowDriverDefs_H_ */
