/* =============================================================================
 *	YarrowINIT.c							 (c)2000 Zero-Knowledge Systems Inc.
 * =============================================================================
 *	
 *	This is the source for the INIT component of the Mac OS implementation of 
 *	Yarrow.  All it does is load the installer code resource, and call it 
 *	to install the driver and entropy sources.
 */

/* system headers */
#include <LowMem.h>
#include <Resources.h>

/* compiler headers */
#include <A4Stuff.h>

/* library headers */
#include "ShowInitIcon.h"

/* project headers */
#include "YarrowInstallClient.h"


// constants
#define kGoodIconId		128
#define kBadIconId		129


// prototypes of local functions
static OSStatus	Preflight();


/* -----------------------------------------------------------------------------
 *	 main
 * -----------------------------------------------------------------------------
 */

pascal void	main(void)
{
	OSStatus	err;
	
	EnterCodeResource();
	
#ifndef NDEBUG
///	Debugger();
#endif
	
	err = Preflight();
	
	if (err == noErr)
	{
		err = YarrowInstallFromResFile(CurResFile(), NULL);
	}
	
	ShowInitIcon(err ? kBadIconId : kGoodIconId, true);
	
	ExitCodeResource();
}


/* -----------------------------------------------------------------------------
 *	 Preflight
 * -----------------------------------------------------------------------------
 *	Check that the machine we're running on can actually give us the 
 *	functionality  we require.  Note that we only check for the features 
 *	required to load and call the installer code resource.  The installer 
 *	contains its own Preflight() function that determines if the driver can 
 *	actually be installed.
 */

static OSStatus	Preflight()
{
	extern pascal OSErr	SYSENVIRONS(short versionRequested, SysEnvRec* theWorld);
	
	SysEnvRec	environment;
	OSErr		err;
	
	// We need the 128K ROM (for 1-deep resource calls & HGetState/HSetState).
	if (LMGetROM85() < 0)
		return (notEnoughHardwareErr);
	
	// We need HFS.
	if (LMGetFSFCBLen() < 0)
		return (notEnoughHardwareErr);
	
	// Call SysEnvirons via glue, *not* via trap
	err = SYSENVIRONS(curSysEnvVers, &environment);
	if (err != noErr)
		return (err);
	
	// We need at least a 68020, because the installer code resource is 
	// compiled for that CPU.  Note that the 68K emulator under PowerPC 
	// will pass this test.
	if (environment.processor < env68020)
		return (notEnoughHardwareErr);
	
	return (noErr);
}

