// =============================================================================
//	YarrowInstallClient.cp					 (c)2000 Zero-Knowledge Systems Inc.
// =============================================================================
//	
//	Defines the structure that is returned by Yarrow's Gestalt selector.  This 
//	structure is used both by the INIT and by the driver to talk to the entropy 
//	hooks.
//

// system headers
#include <Resources.h>

// library headers
#include "RsrcUtils.h"
#include "MoreFilesExtras.h"

// project headers
#include "YarrowGestalt.h"
#include "YarrowInstall.h"
#include "YarrowInstallClient.h"
#include "YarrowUtils.h"


#define	kYarrowFileName			"\pYarrow"
#define	kYarrowFileNameDebug	"\pYarrow Debug"


// prototypes of local functions
static OSStatus	CallYarrowInstaller(
					SInt32					selector, 
					YarrowInstallerArgs*	args);


pascal OSStatus	YarrowLocate(FSSpec* outDriverSpec)
{
	OSStatus	err;
	
	// First, look for release version by name
	
	err = YarrowFindFileInExtensions(kYarrowFileName, kYarrowSig, 'INIT', 
							   false, outDriverSpec);
	
	if (err != noErr)
	{
		// Next, look for debug version by name.  If we don't find 
		// it, scan the Extensions folder for a file with the matching 
		// type & creator.
		
		err = YarrowFindFileInExtensions(kYarrowFileNameDebug, kYarrowSig, 
								   'INIT', true, outDriverSpec);
	}
	
	return (err);
}


pascal OSStatus	YarrowInstall(const FSSpec* inDriverSpec, void* reserved)
{
	short		savedResFile	= CurResFile();
	short		resFile;
	OSStatus	err;
	
	resFile = FSpOpenResFile(inDriverSpec, fsRdPerm);
	
	if (resFile != kResFileNotOpened)
	{
		YarrowInstallerArgs	args;
		
		args.install.resfile	= resFile;
		args.install.spec		= *inDriverSpec;
		args.install.reserved	= reserved;
		
		err = CallYarrowInstaller(kYarrowInstallerInstall, &args);
		
		CloseResFile(resFile);
	}
	else
	{
		err = ResError();
	}
	
	UseResFile(savedResFile);
	
	return (err);
}


pascal OSStatus	YarrowInstallFromResFile(SInt16 inDriverRefNum, void* reserved)
{
	short		savedResFile	= CurResFile();
	FSSpec		spec;
	OSStatus	err;
	
	UseResFile(inDriverRefNum);
	
	err = FSpGetFileLocation(inDriverRefNum, &spec);
	
	if (err == noErr)
	{
		YarrowInstallerArgs	args;
		
		args.install.resfile	= inDriverRefNum;
		args.install.spec		= spec;
		args.install.reserved	= reserved;
		
		err = CallYarrowInstaller(kYarrowInstallerInstall, &args);
	}
	
	UseResFile(savedResFile);
	
	return (err);
}


pascal OSStatus	YarrowUninstall(const FSSpec* inDriverSpec)
{
	short		savedResFile	= CurResFile();
	short		resFile;
	OSStatus	err;
	
	resFile = FSpOpenResFile(inDriverSpec, fsRdPerm);
	
	if (resFile != kResFileNotOpened)
	{
		YarrowInstallerArgs	args;
		
		err = CallYarrowInstaller(kYarrowInstallerUninstall, &args);
		
		CloseResFile(resFile);
	}
	else
	{
		err = ResError();
	}
	
	UseResFile(savedResFile);
	
	return (err);
}


pascal UInt16	YarrowIsInstalled(const FSSpec* inDriverSpec)
{
	short		savedResFile	= CurResFile();
	UInt16		result			= kYarrowCantInstall;
	short		resFile;
	
	resFile = FSpOpenResFile(inDriverSpec, fsRdPerm);
	
	if (resFile != kResFileNotOpened)
	{
		YarrowInstallerArgs	args;
		OSStatus			err;
		
		err = CallYarrowInstaller(kYarrowInstallerIsInstalled, &args);
		
		if (err == noErr)
		{
			result = args.isinstalled.result;
		}
		
		CloseResFile(resFile);
	}
	
	UseResFile(savedResFile);
	
	return (result);
}


static OSStatus	CallYarrowInstaller(
	SInt32					selector, 
	YarrowInstallerArgs*	args)
{
	Handle		rsrcHand;
	OSStatus	err;
	
	err = YarrowLoadCodeResource(CurResFile(), GetZone(), 'Inst', 1000, false, 
						   true, false, &rsrcHand);
	
	if (err == noErr)
	{
		YarrowInstallerUPP	upp	= (YarrowInstallerUPP) *rsrcHand;
		
		err = InvokeYarrowInstallerUPP(selector, args, upp);
		
		YarrowUnloadCodeResource(GetZone(), rsrcHand);
	}
	
	return (err);
}
