/* =============================================================================
 *	YarrowLibc.c							 (c)2000 Zero-Knowledge Systems Inc.
 * =============================================================================
 *	
 *	Because we don't want to link with a C library (it typically contains 
 *	functions that would be dangerous to call from a driver), we need to 
 *	implement a few functions from it ourselves.  This is mainly because we 
 *	link with with third-party libraries that depend on these functions.
 */

/* standard headers */
#include <stdlib.h>
#include <string.h>

/* system headers */
#include <MacMemory.h>
#include <PLStringFuncs.h>


#pragma mark memcmp

int	(memcmp)(const void *s1, const void *s2, size_t n)
{
	if (n != 0) {
		register const unsigned char *p1 = (const unsigned char *)s1;
		register const unsigned char *p2 = (const unsigned char *)s2;

		do {
			if (*p1++ != *p2++)
				return (*--p1 - *--p2);
		} while (--n != 0);
	}
	return (0);
}

#pragma mark memcpy

void* (memcpy)(register void * dst, register const void * src, register size_t n)
{
	BlockMoveData(src, dst, (Size) n);
	return (dst);
}

#pragma mark memset

void*	(memset)(void *dst, int c, size_t n)
{
	unsigned char*	cp	= (unsigned char *) dst;
	
	while (n-- > 0)
		*cp++ = (unsigned char) c;
	
	return (dst);
}

#pragma mark labs

long	(labs)(long n)
{
	if (n < 0)
		n = -n;
	
	return(n);
}

#pragma mark strncpy

char*	(strncpy)(char *dst, const char *src, register size_t n)
{
	if (n != 0)
	{
		register char *d = dst;
		register const char *s = src;

		do
		{
			if ((*d++ = *s++) == 0)
			{
				/* NUL pad the remaining n-1 bytes */
				while (--n != 0)
					*d++ = 0;
				break;
			}
		} while (--n != 0);
	}
	return (dst);
}

/* PLStringFuncs functions originally in PLStringFuncs.c, which had the 
following header: */

/*********************************************************************
File		:	PLStringFuncs.c	-- C-style manipulations of Pascal strings
Author		:	Matthias Neeracher, reimplementing Apple's interface
Language	:	Metrowerks C
*********************************************************************/

pascal StringPtr PLstrncpy(StringPtr str1, ConstStr255Param str2, short num)
{
	if (num < *str2) {
		memcpy(str1+1, str2+1, *str1 = (UInt8) num);
	} else {
		memcpy(str1, str2, (UInt8) (*str2+1));
		if (*str2 < num)
			memset(str1+*str2+1, 0, (UInt8) (num-*str2));
	}
	return str1;
}
