/* drivertest.c -- ad hoc tests for Yarrow Driver */

/* IMPORTANT:	This is not a self-running test app.  It's meant 
 *				to be used under the control of a debugger.
 */

#define YARROW_USE_DRIVER	1

#include <stdio.h>
#include <stdlib.h>

#include <Events.h>

#if 0
#	include <USB.h>
#endif

#include "DCon.h"
#include "yarrow.h"


static void	HIDInterruptProc(UInt32 refcon, void *theData);


void	main(void)
{
	int	err;
	
#if 0
	USBDeviceRef	devref	= kNoDeviceRef;
	USBInterfaceRef	intref	= kNoDeviceRef;
	THz				savedZone	= GetZone();
	
	SetZone(SystemZone());
	
///	err = USBGetVersion();
	
	do
	{
		USBDeviceDescriptor			descr;
		USBInterfaceDescriptor		intdescr;
		CFragConnectionID			connID	= 0;
		USBHIDRev2DispatchTable*	dispatchTable;
		USBBusRef					busRef;
		
		err = USBGetInterfaceDescriptor(&intref, &intdescr, sizeof(intdescr));
		
		err = USBGetNextDeviceByClass(&devref, &connID, 
									  kUSBAnyClass, 
									  kUSBAnySubClass,
									  kUSBAnyProtocol);
		
		if (err == noErr)
		{
			err = USBGetDeviceDescriptor(&devref, &descr, sizeof(descr));
		}
		
		if (err == noErr)
		{
			err = USBDeviceRefToBusRef(&devref, &busRef);
		}
		
		if (err == noErr)
		{
			const char*	str;
			
			printf("Device %lx:\n", (long) devref);
			printf("\tlength:          %u\n", descr.length);
			printf("\tdescType:        %u\n", descr.descType);
			
			descr.usbRel = USB_CONSTANT16(descr.usbRel);
			printf("\tusbRel:          0x%04.4x\n", descr.usbRel);
			
			switch (descr.deviceClass)
			{
			case kUSBCompositeClass:		str = "Composite";		break;
			case kUSBAudioClass:			str = "Audio";			break;
			case kUSBCommClass:				str = "Comm";			break;
			case kUSBHIDClass:				str = "HID";			break;
			case kUSBDisplayClass:			str = "Display";		break;
			case kUSBPrintingClass:			str = "Printing";		break;
			case kUSBMassStorageClass:		str = "MassStorage";	break;
			case kUSBHubClass:				str = "Hub";			break;
			case kUSBDataClass:				str = "Data";			break;
			case kUSBVendorSpecificClass:	str = "VendorSpecific";	break;
			default:						str = "<unknown>";		break;
			}
			printf("\tdeviceClass:     %u (%s)\n", descr.deviceClass, str);
			
			switch (descr.deviceSubClass)
			{
			case kUSBCompositeSubClass:			str = "Composite";		break;
			case kUSBHubSubClass:				str = "Hub";			break;
///				case kUSBPrinterSubclass:			str = "Printer";		break;
			case kUSBVendorSpecificSubClass:	str = "VendorSpecific";	break;
			default:							str = "<unknown>";		break;
			}
			printf("\tdeviceSubClass:  %u (%s)\n", descr.deviceSubClass,  str);
			
			
			switch (descr.protocol)
			{
			case kUSBNoInterfaceProtocol:		str = "None";			break;
			case kUSBKeyboardInterfaceProtocol:	str = "Keyboard";		break;
			case kUSBMouseInterfaceProtocol:	str = "Mouse";			break;
			case kUSBVendorSpecificProtocol:	str = "VendorSpecific";	break;
			default:							str = "<unknown>";		break;
			}
			printf("\tprotocol:        %u (%s)\n", descr.protocol,  str);
			
			printf("\tmaxPacketSize:   %u\n", descr.maxPacketSize);
			
			descr.vendor = USB_CONSTANT16(descr.vendor);
			switch (descr.vendor)
			{
			case kVendorID_AppleComputer:	str = "Apple";		break;
			default:						str = "unknown";	break;
			}
			printf("\tvendor:          0x%04.4x (%s)\n", descr.vendor,  str);
			
			descr.product = USB_CONSTANT16(descr.product);
			printf("\tproduct:         0x%04.4x\n", descr.product);

			descr.devRel = USB_CONSTANT16(descr.devRel);
			printf("\tdevRel:          0x%04.4x\n", descr.devRel);
			printf("\tmanuIdx:         %u\n", descr.manuIdx);
			printf("\tprodIdx:         %u\n", descr.prodIdx);
			printf("\tserialIdx:       %u\n", descr.serialIdx);
			printf("\tnumConf:         %u\n", descr.numConf);
			
			printf("\tbusRef:          %u\n", busRef);
		}
		
		if (err == noErr)
		{
			CFragSymbolClass	symClass;
			
			err = FindSymbol(connID, "\pTheHIDModuleDispatchTable", 
							 (Ptr *) &dispatchTable, &symClass);
			
			if (err == noErr)
			{
				static const UInt32	kSelectors[] = {
					kHIDGetLEDStateByBits,
					kHIDGetLEDStateByIDNumber,
					kHIDGetDeviceCountryCode,
					kHIDGetDeviceUnitsPerInch,
					kHIDGetInterruptHandler,
					kHIDGetCurrentKeys,
					kHIDGetInterruptRefcon,
					kHIDGetVendorID,
					kHIDGetProductID
				};
				
				static const int	kNumSelectors	= sizeof(kSelectors) / sizeof(kSelectors[0]);
				UInt32				values[kNumSelectors];
				OSStatus			errors[kNumSelectors];
				int					i;
				
				for (i = 0; i < kNumSelectors; i++)
				{
					errors[i] = (*dispatchTable->pUSBHIDGetDeviceInfo)(kSelectors[i], 
																	   &values[i]);
					
					if (errors[i] == noErr)
					{
						printf("\tdev info:        %u : %08.8lx\n", kSelectors[i], values[i]);
					}
				}
				
				if ((errors[kHIDGetInterruptHandler-kHIDGetLEDStateByBits] == noErr) && 
					(errors[kHIDGetInterruptRefcon-kHIDGetLEDStateByBits] == noErr))
				{
					unsigned long	junk;
					
					err = (*dispatchTable->pUSBHIDInstallInterrupt)(HIDInterruptProc, 
																	(UInt32) values);
					
					Delay(60, &junk);
					
					err = (*dispatchTable->pUSBHIDInstallInterrupt)(
							(HIDInterruptProcPtr) values[kHIDGetInterruptHandler-kHIDGetLEDStateByBits], 
							values[kHIDGetInterruptRefcon-kHIDGetLEDStateByBits]);
				}
			}
			
			err = noErr; 
		}
		
	} while (err == noErr);
	
	SetZone(savedZone);
	
#endif
	
#if 1
	err = Yarrow_Open();
	
	if (err == YARROW_OK)
	{
		char			buff[200];
		unsigned short	numBuiltInSources, source;
		int				numSources;
		unsigned		sourceId;
		size_t			entropyBits, entropyMax;
		
		while (!Button() && (err == YARROW_OK))
		{
			unsigned long	junk;
			
			err = Yarrow_Read(buff, sizeof(buff));
			
			Delay(6, &junk);
			
			if (err == YARROW_OK)
				err = Yarrow_Write(buff, sizeof(buff), 0);
			
			Delay(4, &junk);
		}
		
		err = Yarrow_Status(&numSources, &sourceId, &entropyBits, &entropyMax);
		
		printf("Seeded:                       %s\n", (err == YARROW_OK) ? "yes" : "no");
		printf("# Non-seeded sources:         %d\n", numSources);
		printf("Closest source:               %u\n", sourceId);
		printf("Closest source entropy bits:  %lu\n", (unsigned long) entropyBits);
		printf("Closest source entropy max:   %lu\n", (unsigned long) entropyMax);
		
		err = Yarrow_Driver_Stats(&numBuiltInSources, NULL);
		
		for (source = 1; source <= numBuiltInSources; source++)
		{
			long	numDevices, packetsReceived, bytesReceived;
			long	packetsOverflowed, bytesOverflowed, packetsNoBuffers;
			long	bytesNoBuffers, bytesWritten, buffersWritten;
			double	avgPacketSize, totalWriteTime, minWriteTime;
			double	maxWriteTime, avgWriteTime;
			char	description[32];
			
			printf("Source %hu:  ", source);
			
			err = Yarrow_Source_Stats(source, &numDevices, &packetsReceived, 
									&bytesReceived, &packetsOverflowed, &bytesOverflowed, 
									&packetsNoBuffers, &bytesNoBuffers, &bytesWritten, 
									&buffersWritten, &avgPacketSize, &totalWriteTime, 
									&minWriteTime, &maxWriteTime, &avgWriteTime, 
									description);
			
			if (err == 0)
			{
				printf("%s\n", description);
				printf("\tnumDevices:         %5ld\n", numDevices);
				printf("\tpacketsReceived:    %5ld\n", packetsReceived);
				printf("\tbytesReceived:      %5ld\n", bytesReceived);
				printf("\tpacketsOverflowed:  %5ld\n", packetsOverflowed);
				printf("\tbytesOverflowed:    %5ld\n", bytesOverflowed);
				printf("\tpacketsNoBuffers:   %5ld\n", packetsNoBuffers);
				printf("\tbytesNoBuffers:     %5ld\n", bytesNoBuffers);
				printf("\tbytesWritten:       %5ld\n", bytesWritten);
				printf("\tbuffersWritten:     %5ld\n", buffersWritten);
				printf("\tavgPacketSize:      %9.3f\n", avgPacketSize);
				printf("\ttotalWriteTime:     %9.3f\n", totalWriteTime);
				printf("\tminWriteTime:       %9.3f\n", minWriteTime);
				printf("\tmaxWriteTime:       %9.3f\n", maxWriteTime);
				printf("\tavgWriteTime:       %9.3f\n", avgWriteTime);
			}
			else
			{
				printf("unknown\n");
			}
			
			printf("\n");
		}
		
		err = Yarrow_Close();
		
	}

#endif
}


#if 0

static void	HIDInterruptProc(UInt32 refcon, void *theData)
{
	UInt32*			values	= (UInt32 *) refcon;
	USBMouseData*	mouse	= (USBMouseData *) theData;
	
	dprintf("m: x = %hd, y = %hd, b = %hx\n", mouse->XDelta, mouse->YDelta, mouse->buttons);
	
	if ((HIDInterruptProcPtr) values[kHIDGetInterruptHandler-kHIDGetLEDStateByBits] != NULL)
	{
		(* (HIDInterruptProcPtr) values[kHIDGetInterruptHandler-kHIDGetLEDStateByBits])(
				values[kHIDGetInterruptRefcon-kHIDGetLEDStateByBits], theData);
	}
}

#endif
