#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "yarrow.h"
#ifndef WIN32
#include "endian.h"
#endif
#include "timer.h"

#if defined(__MWERKS__) && defined(macintosh)
#  include <console.h>
#endif

#if YARROW_MWERKS_PROFILER
#	include "Profiler.h"
#elif YARROW_INSTRUMENTATION_SDK
#	include "Instrumentation.h"
#endif

void hex_print( FILE* f, const char* var, void* data, size_t size );
void dump_yarrow_state( FILE* f, Yarrow_CTX* y );

static void printstatus(Yarrow_CTX *yarrow){
  int num, status;
  unsigned source;
  size_t ent,max;

  status = Yarrow_Status(yarrow, &num, &source, &ent, &max);
  if (status != YARROW_OK)
    printf("Yarrow_Status = %d num=%d source=%d ent=%d max=%d\n", status,num,source,ent,max);
}

int main( int argc, char* argv[] )
{
  Yarrow_CTX yarrow;
  unsigned mouse, keyboard, user;
  int i;
  byte mouse_sample[ 4 ];
  byte keyboard_sample[ 2 ];
  byte user_sample[ 20 ];
  byte rndm[ 30 ];
  byte junk[ 1024 ];
  int ok = 1;

#if defined(__MWERKS__) && defined(macintosh)
  argc = ccommand(&argv);
#else
  /* silence compiler */
  i = argc;
  i = (int)argv;
#endif
  
#if YARROW_MWERKS_PROFILER
  if (ProfilerInit(collectDetailed, bestTimeBase, 100, 100) != noErr)
  {
    fprintf( stderr, "failed to initialize profiler\n" );
    exit( EXIT_FAILURE );
  }
#endif
  
  memset( mouse_sample,    1, sizeof( mouse_sample ) );
  memset( keyboard_sample, 2, sizeof( keyboard_sample ) );
  memset( user_sample,     3, sizeof( user_sample ) );

  Yarrow_Init( &yarrow, NULL );
  printstatus( &yarrow );

  /*  Yarrow_Set_Param_Pg( &yarrow, 10 );  */

#if defined( YARROW_DEBUG )
  dump_yarrow_state( stdout, &yarrow );
#endif

  if (Yarrow_New_Source( &yarrow, &mouse ) != YARROW_OK)
  {
    fprintf( stderr, "failed to allocate yarrow source\n" );
    exit( EXIT_FAILURE );
  }
  if (Yarrow_New_Source( &yarrow, &keyboard ) != YARROW_OK)
  {
    fprintf( stderr, "failed to allocate yarrow source\n" );
    exit( EXIT_FAILURE );
  }
  if (Yarrow_New_Source( &yarrow, &user ) != YARROW_OK)
  {
    fprintf( stderr, "failed to allocate yarrow source\n" );
    exit( EXIT_FAILURE );
  }
  
  Yarrow_Input( &yarrow, mouse, mouse_sample, sizeof( mouse_sample ), 1 );
  printstatus( &yarrow );

  Yarrow_Input( &yarrow, keyboard, keyboard_sample, 
		sizeof( keyboard_sample ), 1 );
  printstatus( &yarrow );
  
  Yarrow_Input( &yarrow, user, user_sample, sizeof( user_sample ), 1 );
  printstatus( &yarrow );

  Yarrow_Output( &yarrow, rndm, sizeof( rndm ) );

#if defined( YARROW_DEBUG )
  dump_yarrow_state( stdout, &yarrow );
#endif

  hex_print( stdout, "random", rndm, sizeof( rndm ) );
  
  /* induce fast reseed */

  for ( i = 0; i < 100; i++ )
  {
    Yarrow_Input( &yarrow, user, user_sample, sizeof( user_sample ),
		  sizeof( user_sample ) );
    printstatus( &yarrow );
  }

  timer_start();
  for ( i = 0; i < 1024; i++ )
  {
    if (Yarrow_Output( &yarrow, junk, sizeof( junk ) ) != 0)
      ok = 0;
  }
  timer_stop();
  if (ok)
    printf( "output = %f Mb/sec\n", 1.0 / (timer_time()/1000000.0) );
  else
    printf( "not seeded\n");
  
  memset( junk, 0, sizeof( junk ) );

  timer_start();
  for ( i = 0; i < 1024; i++ )
  {
    /* odd input to a different source so there are some slow reseeds */

    if ( i % 16 == 0 )
    {
      Yarrow_Input( &yarrow, mouse, junk, sizeof( junk ), sizeof( junk ) );
    }
    else
    {
      Yarrow_Input( &yarrow, user, junk, sizeof( junk ), sizeof( junk ) );
    }
    printstatus( &yarrow );

  }
  timer_stop();
  printf( "input = %f Mb/sec\n", 1.0 / (timer_time()/1000000.0) );

#if YARROW_MWERKS_PROFILER
  ProfilerDump(YARROW_MWERKS_PROFILER_FILE);
#endif

  if (Yarrow_Output( &yarrow, rndm, sizeof( rndm ) ) == YARROW_OK)
    printf("Yarrow_Output ok\n");
  else
    printf("Yarrow_Output failed\n");

  return (EXIT_SUCCESS);
}

void hex_print( FILE* f, const char* var, void* data, size_t size )
{
  const char* conv = "0123456789abcdef";
  size_t i;
  char* p = (char*) data;
  char c, d;

  fprintf( f, var );
  fprintf( f, " = " );
  for ( i = 0; i < size; i++ )
  {
    c = conv[ (p[ i ] >> 4) & 0xf ];
    d = conv[ p[ i ] & 0xf ];
    fprintf( f, "%c%c", c, d );
  }
  fprintf( f, "\n" );
}

void dump_yarrow_state( FILE* f, Yarrow_CTX* y )
{
  fprintf( f, "===Yarrow State===\n" );
  hex_print( f, "C", y->C, sizeof( y->C ) );
  hex_print( stdout, "K", y->k, sizeof( y->k ) );
}
